#ifndef Poisson1_h_IS_INCLUDED  // prevent multiple inclusions of 
#define Poisson1_h_IS_INCLUDED  // this header file

#include <FEM.h>           // FEM algorithms, FieldFE, GridFE etc
#include <DegFreeFE.h>     // mapping: nodal values -> unknowns in linear sys.
#include <LinEqAdmFE.h>    // linear systems: storage and solution
#include <SaveSimRes.h>    // storage tool for later visualization

class Poisson1 : public FEM
{
public:
  // general data:
  Handle(GridFE)     grid;   // finite element grid
  Handle(DegFreeFE)  dof;    // trivial mapping here: nodal values -> unknowns
  Handle(FieldFE)    u;      // finite element field, the primary unknown
  Vec(real)          linsol; // solution of linear system
  Handle(LinEqAdmFE) lineq;  // linear system, storage and solution
  Handle(FieldsFE)   flux;   // -k*grad(u)
  Handle(FieldFE)    error;  // the error field (analytical - numerical sol.)
  bool               dump; 

  real L1_error, L2_error, Linf_error;  // various norms of the error
  Handle(SaveSimRes) database; // store computed fields on file
  Ptv(real)          A;        // parameters in the right hand side
  Handle(FieldFunc)  uanal;    // functor for analytical solution
  bool               standard_calcelmat; 

  virtual void fillEssBC ();            // set boundary conditions
  virtual void integrands  // evaluate weak form in the FEM equations
    (ElmMatVec& elmat, const FiniteElement& fe);
  virtual void calcElmMatVec (int e, ElmMatVec& elmat, FiniteElement& fe);

  Poisson1 ();
 ~Poisson1 () {}

  virtual void adm    (MenuSystem& menu);
  virtual void define (MenuSystem& menu, int level = MAIN);
  virtual void scan (); 
  virtual void solveProblem ();  // main driver routine
  virtual void resultReport ();  // write error norms to the screen

  // source term in the PDE:
  virtual real f (const FiniteElement& fe);
  // coefficient in the diffusion term (also required by FEM::makeFlux):
  virtual real k (const FiniteElement& fe, real t = DUMMY);
  // essential boundary conditions:
  virtual real g (const Ptv(real)& x);
};


void SyFicalcElmMatVec2D (int e, ElmMatVec& elmat, FiniteElement& fe);
void SyFicalcElmMatVec3D (int e, ElmMatVec& elmat, FiniteElement& fe);
#endif

