
#include <Poisson1.h>
#include <Mat_real.h>
void SyFicalcElmMatVec3D (int element, ElmMatVec& elmat, FiniteElement& fe){

  Mat(double) G(4,3); 
  fe.grid().getElmCoor(G,element); 


  double x0 = G(1,1); double y0 = G(1,2); double z0 = G(1,3);  
  double x1 = G(2,1); double y1 = G(2,2); double z1 = G(2,3); 
  double x2 = G(3,1); double y2 = G(3,2); double z2 = G(3,3);  
  double x3 = G(4,1); double y3 = G(4,2); double z3 = G(4,3);  


  double a   = x1 - x0;
  double d   = y1 - y0;
  double g   = z1 - z0;

  double b   = x2 - x0;
  double e   = y2 - y0;
  double h   = z2 - z0;

  double c   = x3 - x0;
  double f   = y3 - y0;
  double k   = z3 - z0;


  double D = a*(e*k-f*h) - b*(d*k-f*g) + c*(d*h-g*e);   

  double JJ00 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(k*e-h*f);
  double JJ01 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(g*f-k*d);
  double JJ02 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(h*d-g*e);
  double JJ10 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(h*c-k*b);
  double JJ11 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(k*a-c*g);
  double JJ12 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(-h*a+b*g);
  double JJ20 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(b*f-c*e);
  double JJ21 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(-f*a+c*d);
  double JJ22 = 1.0/(k*a*e+h*c*d-c*g*e-k*b*d+b*g*f-h*f*a)*(a*e-b*d);


  static double A[4][4]; 


  A[0][0]=JJ20*JJ22/3.0+JJ00*JJ02/3.0+JJ12*JJ11/3.0+JJ20*JJ21/3.0+(JJ02*JJ02)/6.0+(JJ11*JJ11)/6.0+(JJ01*JJ01)/6.0+JJ00*JJ01/3.0+(JJ20*JJ20)/6.0+(JJ10*JJ10)/6.0+JJ22*JJ21/3.0+(JJ22*JJ22)/6.0+JJ11*JJ10/3.0+(JJ12*JJ12)/6.0+(JJ21*JJ21)/6.0+JJ12*JJ10/3.0+(JJ00*JJ00)/6.0+JJ02*JJ01/3.0;
  A[0][1]=-JJ20*JJ22/6.0-JJ00*JJ02/6.0-JJ20*JJ21/6.0-JJ00*JJ01/6.0-(JJ20*JJ20)/6.0-(JJ10*JJ10)/6.0-JJ11*JJ10/6.0-JJ12*JJ10/6.0-(JJ00*JJ00)/6.0;
  A[0][2]=-JJ12*JJ11/6.0-JJ20*JJ21/6.0-(JJ11*JJ11)/6.0-(JJ01*JJ01)/6.0-JJ00*JJ01/6.0-JJ22*JJ21/6.0-JJ11*JJ10/6.0-(JJ21*JJ21)/6.0-JJ02*JJ01/6.0;
  A[0][3]=-JJ20*JJ22/6.0-JJ00*JJ02/6.0-JJ12*JJ11/6.0-(JJ02*JJ02)/6.0-JJ22*JJ21/6.0-(JJ22*JJ22)/6.0-(JJ12*JJ12)/6.0-JJ12*JJ10/6.0-JJ02*JJ01/6.0;
  A[1][0]=-JJ20*JJ22/6.0-JJ00*JJ02/6.0-JJ20*JJ21/6.0-JJ00*JJ01/6.0-(JJ20*JJ20)/6.0-(JJ10*JJ10)/6.0-JJ11*JJ10/6.0-JJ12*JJ10/6.0-(JJ00*JJ00)/6.0;
  A[1][1]=(JJ20*JJ20)/6.0+(JJ10*JJ10)/6.0+(JJ00*JJ00)/6.0;
  A[1][2]=JJ20*JJ21/6.0+JJ00*JJ01/6.0+JJ11*JJ10/6.0;
  A[1][3]=JJ20*JJ22/6.0+JJ00*JJ02/6.0+JJ12*JJ10/6.0;
  A[2][0]=-JJ12*JJ11/6.0-JJ20*JJ21/6.0-(JJ11*JJ11)/6.0-(JJ01*JJ01)/6.0-JJ00*JJ01/6.0-JJ22*JJ21/6.0-JJ11*JJ10/6.0-(JJ21*JJ21)/6.0-JJ02*JJ01/6.0;
  A[2][1]=JJ20*JJ21/6.0+JJ00*JJ01/6.0+JJ11*JJ10/6.0;
  A[2][2]=(JJ11*JJ11)/6.0+(JJ01*JJ01)/6.0+(JJ21*JJ21)/6.0;
  A[2][3]=JJ12*JJ11/6.0+JJ22*JJ21/6.0+JJ02*JJ01/6.0;
  A[3][0]=-JJ20*JJ22/6.0-JJ00*JJ02/6.0-JJ12*JJ11/6.0-(JJ02*JJ02)/6.0-JJ22*JJ21/6.0-(JJ22*JJ22)/6.0-(JJ12*JJ12)/6.0-JJ12*JJ10/6.0-JJ02*JJ01/6.0;
  A[3][1]=JJ20*JJ22/6.0+JJ00*JJ02/6.0+JJ12*JJ10/6.0;
  A[3][2]=JJ12*JJ11/6.0+JJ22*JJ21/6.0+JJ02*JJ01/6.0;
  A[3][3]=(JJ02*JJ02)/6.0+(JJ22*JJ22)/6.0+(JJ12*JJ12)/6.0;


  elmat.A(1,1) = A[0][0]*D;
  elmat.A(1,2) = A[0][1]*D;
  elmat.A(1,3) = A[0][2]*D;
  elmat.A(1,4) = A[0][3]*D;
  elmat.A(2,1) = A[1][0]*D;
  elmat.A(2,2) = A[1][1]*D;
  elmat.A(2,3) = A[1][2]*D;
  elmat.A(2,4) = A[1][3]*D;
  elmat.A(3,1) = A[2][0]*D;
  elmat.A(3,2) = A[2][1]*D;
  elmat.A(3,3) = A[2][2]*D;
  elmat.A(3,4) = A[2][3]*D;
  elmat.A(4,1) = A[3][0]*D;
  elmat.A(4,2) = A[3][1]*D;
  elmat.A(4,3) = A[3][2]*D;
  elmat.A(4,4) = A[3][3]*D;

}
