from swiginac import *
from SyFi import *

x = cvar.x; y = cvar.y;  z = cvar.z; 

def Jinv_string(G,Jinv): 
  nsd = G.cols()
  Ginv   = G.inverse().transpose()
  J_str = ""
  for i in range(0,Jinv.nops()):
    J_str += "  double " +  Jinv.op(i).printc() +  " = " + Ginv.op(i).printc() + ";\n"
  J_str += "\n\n" 
  return J_str




def geometry_string_3D():
  s = """
  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

"""
  return s

def geometry_string_2D(): 
  s = """
  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
"""
  return s 



def create_header():  
  s = """
#include <math.h>
#include <utility>
#include <map>
#include <ptv.h>
#include <DofT.h>
#include <ScalarFunction.h>

using namespace std; 

"""
  return s 



def create_rhs_function_string_3D(funcname, dof_string, rhs_string): 
  nsd = 2 
  geometry_string = geometry_string_3D()
  s = """
void %s (map<int,double>& rhs, 
         DofT<ptv,ptv_is_less>& dof, 
         ScalarFunction& f, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d], double pp3[%d]){


%s
%s
%s
}
""" % ( funcname, nsd, nsd, nsd, nsd, geometry_string, dof_string, rhs_string) 
  return s
 

def create_rhs_function_string_2D(funcname, dof_string, rhs_string): 
  nsd = 2 
  geometry_string = geometry_string_2D()
  s = """

void %s (map<int,double>& rhs, 
         DofT<ptv,ptv_is_less>& dof, 
         ScalarFunction& f, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d]){

%s 
%s
%s
}
""" % ( funcname, nsd, nsd, nsd, geometry_string, dof_string, rhs_string) 
  return s
 

def create_rhs_string(fe,quad,G,p0): 
  nsd = G.cols() 
  rhs_string = ""
  rhs_string += "  double value[%d];\n"%len(quad)
  rhs_string += "  double x,y,z,integral;\n\n"
  rhs_string += "  ptv p(%d);\n"%nsd 
  for i in range(0,fe.nbf()): 
    qcounter = -1 
    for q in quad: 
      qcounter += 1
      Ni = fe.N(i)
      Ni = Ni.subs((x == q[0])) 
      Ni = Ni.subs((y == q[1])) 
      if nsd == 3: 
        Ni = Ni.subs((z == q[2])) 
      psim = matrix(1,nsd) 
      for d in range(0, nsd):  
        psim[0,d] = q[d] 

      xi = G.mul(psim.transpose()).add(p0)  
#      print Ni
      for j in range(0,xi.nops()):
          rhs_string += "  p[%d] = "%j + xi.op(j).printc() + ";\n"

      rhs_string += "  value[%d] = f(p,element)*%s;\n" %(qcounter,Ni.printc())  

    qcounter = -1 
    rhs_string += "\n"
    rhs_string += "  integral = "
    for q in quad: 
      qcounter += 1
      rhs_string += " + %e*(value[%d])"%(q[nsd],qcounter)  
    rhs_string += ";\n" 
    
    rhs_string += "  rhs[iidof[%d]] += integral*D; \n\n\n"%(i-1) 

  return rhs_string
        
#      print xi[0].printc() 
      

  

def create_vector_dof_string(fe,G,p0,prefix="i",dofprefix=""):
  nsd = p0.nops()
  D_str =""
  D_str += "  int    i%sdof[%d];\n"%(prefix,fe.nbf()) 
  for i in range(0,fe.nbf()): 
    dofi = fe.dof(i) 
    dofi0 = dofi[0]
    dofi1 = dofi[1]
    dofm = matrix(1,nsd,dofi0)
    gdofm = G.mul(dofm.transpose()).add(p0)
    D_str += "  double %sdof%d[%d];\n"%(prefix,i,nsd+1) 
    for d in range(0,nsd): 
      D_str +=  "  %sdof%d[%d]=%s;"%(prefix,i,d,gdofm.op(d).printc())
    D_str += "\n  %sdof%d[%d] = %s;" %  (prefix,i,nsd,dofi1.printc())  
    D_str += "\n  ptv %spdof%d(%d,%sdof%d);\n"%(prefix,i,nsd+1,prefix,i)  
    D_str += "  i%sdof[%d] = %sdof.insert_dof(element,%d,%spdof%d);\n\n"%(prefix,i,dofprefix,i,prefix,i)
    
  return D_str  



def create_dof_string_read(fe,G,p0,prefix="i", dofprefix=""):
  nsd = p0.nops()
  D_str =""
  D_str += "  int    i%sdof[%d];\n"%(prefix,fe.nbf()) 
  for i in range(0,fe.nbf()): 
    dofi = fe.dof(i) 
    dofm = matrix(1,nsd,dofi)
    gdofm = G.mul(dofm.transpose()).add(p0)
    D_str += "  double dof%d[%d];\n"%(i,nsd) 
    for d in range(0,nsd): 
      D_str +=  "  dof%d[%d]=%s;"%(i,d,gdofm.op(d).printc())
    D_str += "\n  ptv pdof%d(%d,dof%d);\n"%(i,nsd,i)  
    D_str += "  i%sdof[%d] = %sdof.glob_dof(pdof%d);\n\n"%(prefix,i,dofprefix,i)
  return D_str  




def create_dof_string(fe,G,p0,prefix="i", dofprefix=""):
  nsd = p0.nops()
  D_str =""
  D_str += "  int    i%sdof[%d];\n"%(prefix,fe.nbf()) 
  for i in range(0,fe.nbf()): 
    dofi = fe.dof(i) 
    dofm = matrix(1,nsd,dofi)
    gdofm = G.mul(dofm.transpose()).add(p0)

    D_str += "  double dof%d[%d];\n"%(i,nsd) 
    for d in range(0,nsd): 
      D_str +=  "  dof%d[%d]=%s;"%(i,d,gdofm.op(d).printc())
    D_str += "\n  ptv pdof%d(%d,dof%d);\n"%(i,nsd,i)  
    D_str += "  i%sdof[%d] = %sdof.insert_dof(element,%d,pdof%d);\n\n"%(prefix,i,dofprefix,i,i)
  return D_str  


def create_A_string_mass_with_c(fe): 
  A_str = "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  domain = fe.getPolygon()
  c      = symbol("c") 
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 
      integrand = c*fe.N(i)*fe.N(j)
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str


   
def create_A_string_mass(fe): 
  A_str = "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  domain = fe.getPolygon()
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 
      integrand = fe.N(i)*fe.N(j)  
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str


def create_A_string_conv(fe,G): 
  nsd = G.cols() 
  Jinv  =  symbolic_matrix(nsd,nsd,"Jinv")  
  J_str = Jinv_string(G,Jinv)
  A_str = J_str


  v0  = symbol("v0"); v1  = symbol("v1"); v2 = symbol("v2") 


#  delta   = symbol("delta") 
#  A_str += "  double h = pow(%s,%e);\n" % (G.determinant(),1.0/nsd)  
#  if nsd == 2: 
#    A_str += "  double v_norm = sqrt(v0*v0 + v1*v1);\n" 
#  elif nsd == 3: 
#    A_str += "  double v_norm = sqrt(v0*v0 + v1*v1 + v2*v2);\n" 
#  A_str += "  double delta = h/(2*v_norm);\n\n" 

  A_str += "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  domain = fe.getPolygon()
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 

      gradNj    = (Jinv*grad(fe.N(j))).evalm()
      if nsd == 2: 
        VxgradNj = v0*gradNj[0,0] + v1*gradNj[1,0] 
      elif nsd == 3: 
        VxgradNj = v0*gradNj[0,0] + v1*gradNj[1,0] + v2*gradNj[2,0] 


#      gradNi    = (Jinv*grad(fe.N(i))).evalm()
#      if nsd == 2: 
#        VxgradNi = v0*gradNi[0,0] + v1*gradNi[1,0] 
#      elif nsd == 3: 
#        VxgradNi = v0*gradNi[0,0] + v1*gradNi[1,0] + v2*gradNi[2,0] 
#      Wi = fe.N(i) + delta*VxgradNi  
#      integrand = VxgradNj*Wi  


      integrand = VxgradNj*fe.N(i) 
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str

def create_v_string(fe): 
  V_str = "" 
  for i in range(1,fe.nbf()+1): 
    V_str += "  double w%d = vel[iidof[%d]];\n" % (i-1,i-1)    
  return V_str 

def create_A_string_vector_conv(fe,G): 

  nsd = G.cols() 
  Jinv  =  symbolic_matrix(nsd,nsd,"Jinv")  
  J_str = Jinv_string(G,Jinv)

  A_str = J_str 
  A_str += "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  w = symbolic_matrix(fe.nbf(),1,"w") 
  ww = 0 
  for k in range(0,fe.nbf()): 
    ww = ww + w[k,0]*fe.N(k) 
  ww = ww.evalm()
  domain = fe.getPolygon()

  integrand = numeric(0) 

  for i in range(0,fe.nbf()): 
    Ni = fe.N(i) 

    for j in range(0,fe.nbf()): 
      Nj = fe.N(j)

      VxgradNj = matrix(nsd,1)  

      if nsd == 2: 

        gradNj0    = (Jinv*grad(fe.N(j).op(0))).evalm()
        gradNj1    = (Jinv*grad(fe.N(j).op(1))).evalm()

        VxgradNj[0,0] = ww[0,0]*gradNj0.op(0) + ww[1,0]*gradNj0.op(1) 
        VxgradNj[1,0] = ww[0,0]*gradNj1.op(0) + ww[1,0]*gradNj1.op(1)

        integrand = VxgradNj.op(0)*Ni.op(0) + VxgradNj.op(1)*Ni.op(1) 

      elif nsd == 3: 

        gradNj0    = (Jinv*grad(fe.N(j).op(0))).evalm()
        gradNj1    = (Jinv*grad(fe.N(j).op(1))).evalm()
        gradNj2    = (Jinv*grad(fe.N(j).op(2))).evalm()

        VxgradNj[0,0] = ww[0,0]*gradNj0.op(0) + ww[1,0]*gradNj0.op(1) + ww[2,0]*gradNj0.op(2) 
        VxgradNj[1,0] = ww[0,0]*gradNj1.op(0) + ww[1,0]*gradNj1.op(1) + ww[2,0]*gradNj1.op(2) 
        VxgradNj[2,0] = ww[0,0]*gradNj2.op(0) + ww[1,0]*gradNj2.op(1) + ww[2,0]*gradNj2.op(2) 

        integrand = VxgradNj.op(0)*Ni.op(0) + VxgradNj.op(1)*Ni.op(1) + VxgradNj.op(2)*Ni.op(2) 


      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str





def create_A_string_vector_mass(fe): 
  A_str = "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  domain = fe.getPolygon()
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 
      integrand = inner(fe.N(i), fe.N(j))
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str

def create_A_string_stiffness_with_M(fe,G,M): 
  nsd = G.cols()
  domain = fe.getPolygon()

  Jinv  =  symbolic_matrix(nsd,nsd,"Jinv")  
  J_str = Jinv_string(G,Jinv)

  A_str = J_str
  A_str += "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 
      gradNi    = (Jinv*grad(fe.N(i))).evalm()
      gradNj    = (Jinv*grad(fe.N(j))).evalm()
      MxgradNi = (M*gradNi).evalm()
      integrand = inner(MxgradNi,gradNj)
      Aij = domain.integrate(integrand)
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str 
 


def create_A_string_stiffness(fe,G): 
  nsd = G.cols()
  domain = fe.getPolygon()

  Jinv  =  symbolic_matrix(nsd,nsd,"Jinv")  
  J_str = Jinv_string(G,Jinv)

  A_str = J_str
  A_str += "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 
      gradNi    = (Jinv*grad(fe.N(i))).evalm()
      gradNj    = (Jinv*grad(fe.N(j))).evalm()
      integrand = inner(gradNi,gradNj)
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str 
   

def create_A_string_vector_stiffness(fe,G): 
  nsd = G.cols()
  domain = fe.getPolygon()

  Jinv  =  symbolic_matrix(nsd,nsd,"Jinv")  
  J_str =  Jinv_string(G,Jinv); 

  A_str = J_str
  A_str += "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 
      gradNi    = (Jinv*grad(fe.N(i))).evalm()
      gradNj    = (Jinv*grad(fe.N(j))).evalm()
      integrand = inner(gradNi, gradNj) 
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str 
   

def create_A_string_vector_stiffness_with_M(fe,G,M): 
  nsd = G.cols()
  domain = fe.getPolygon()

  Jinv  =  symbolic_matrix(nsd,nsd,"Jinv")  
  J_str =  Jinv_string(G,Jinv); 

  A_str = J_str
  A_str += "  double A[%d][%d];\n"%(fe.nbf(), fe.nbf())
  for i in range(0,fe.nbf()): 
    for j in range(0,fe.nbf()): 
      gradNi    = (Jinv*grad(fe.N(i))).evalm()
      gradNj    = (Jinv*grad(fe.N(j))).evalm()
      MxgradNi = (M*gradNi).evalm()
      integrand = inner(MxgradNi, gradNj) 
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int nbf = %d;" % fe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str 
   





def create_A_string_div(vfe,pfe,G): 
  domain = vfe.getPolygon()
  nsd = G.cols()
  xj = []
  if nsd == 2: xj = [x,y]  
  if nsd == 3: xj = [x,y,z]  


  Jinv  =  symbolic_matrix(nsd,nsd,"Jinv")  
  J_str =  Jinv_string(G,Jinv) 

  A_str =  J_str
  A_str += "  double A[%d][%d];\n"%(pfe.nbf(), vfe.nbf())

  for i in range(0,pfe.nbf()): 
    for j in range(0,vfe.nbf()): 

      divNj = numeric(0) 
      Nj = vfe.N(j)
      for d1 in range(0,nsd): 
        for d2 in range(0,nsd): 
          divNj += diff(Nj.op(d1),xj[d2])*Jinv[d1,d2]   
          
      integrand = divNj*pfe.N(i)  
      Aij = domain.integrate(integrand) 
      A_str += "  A[%d][%d]=(%s)*D;\n"%(i,j,Aij.evalf().printc())

  A_str += "  int vnbf = %d;" % vfe.nbf() 
  A_str += "  int pnbf = %d;" % pfe.nbf() 
  A_str += """  

  pair<int,int> index; 
  for (int i=0; i< pnbf; i++) {
    index.first = ipdof[i]; 
    for (int j=0; j< vnbf; j++) {
      index.second = ivdof[j]; 
      matrix[index] += A[i][j];   
    }
  }
"""
  return A_str


  
 

def create_function_string_2D(funcname, dof_string, elmat_string): 
  nsd = 2
  geometry_string = geometry_string_2D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d]){
  
  
  
  
  //variables
%s
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s

def create_function_string_2D_with_c(funcname, dof_string, elmat_string): 
  nsd = 2
  geometry_string = geometry_string_2D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d], double c){
  
  
  
  
  //variables
%s
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s




def create_function_string_2D_with_v(funcname, dof_string, elmat_string): 
  nsd = 2
  geometry_string = geometry_string_2D()
  s = """
 
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d],
         double v0, double v1){
  
   //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s
 
def create_function_string_2D_vector_with_v(funcname, dof_string, elmat_string): 
  nsd = 2
  geometry_string = geometry_string_2D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d],
         double* vel, int size){
  
 
  //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s

def create_function_string_3D_vector_with_v(funcname, dof_string, elmat_string): 
  nsd = 3
  geometry_string = geometry_string_3D()
  s = """
 
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d], double pp3[%d],
         double* vel, int size){
  
  
 
  //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s




  
def create_function_string_2D_with_2dofs(funcname, dof_string, elmat_string): 
  nsd = 2
  geometry_string = geometry_string_2D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& idof, 
         DofT<ptv,ptv_is_less>& jdof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d]){
  
   //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s
 

def create_function_string_2D_with_M(funcname, dof_string, elmat_string): 
  nsd = 2
  geometry_string = geometry_string_2D()
  s = """
 
using namespace std; 
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double M[%d][%d], double pp0[%d], double pp1[%d], double pp2[%d]){
  
  double M00 = M[0][0]; double M10 = M[1][0]; 
  double M01 = M[0][1]; double M11 = M[1][1]; 
 
  
   //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s
  

def create_function_string_3D(funcname, dof_string, elmat_string): 
  nsd = 3 
  geometry_string = geometry_string_3D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d], double pp3[%d]){
  
   //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s

def create_function_string_3D_with_c(funcname, dof_string, elmat_string): 
  nsd = 3 
  geometry_string = geometry_string_3D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d], double pp3[%d], double c){
  
   //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s



def create_function_string_3D_with_v(funcname, dof_string, elmat_string): 
  nsd = 3 
  geometry_string = geometry_string_3D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d], double pp3[%d],
         double v0, double v1, double v2){
  
  
 
  //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s



def create_function_string_3D_with_2dofs(funcname, dof_string, elmat_string): 
  nsd = 3 
  geometry_string = geometry_string_3D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& idof, 
         DofT<ptv,ptv_is_less>& jdof, 
         int element, double pp0[%d], double pp1[%d], double pp2[%d], double pp3[%d]){
  
  
   //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s

 
def create_function_string_3D_with_M(funcname, dof_string, elmat_string): 
  nsd = 3 
  geometry_string = geometry_string_3D()
  s = """
  
void %s (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double M[%d][%d], double pp0[%d], double pp1[%d], double pp2[%d], double pp3[%d]){
  
  double M00 = M[0][0]; double M10 = M[1][0]; double M20 = M[2][0]; 
  double M01 = M[0][1]; double M11 = M[1][1]; double M21 = M[2][1]; 
  double M02 = M[0][2]; double M12 = M[1][2]; double M22 = M[2][2]; 
 
  
  
  //variables
%s 
%s 
%s
}
"""% (funcname, nsd,nsd,nsd,nsd,nsd,nsd,geometry_string, dof_string,elmat_string) 
  return s
 

