
#include <math.h>
#include <utility>
#include <map>
#include <ptv.h>
#include <DofT.h>
#include <ScalarFunction.h>

using namespace std; 



/* ------------- order 2 1 ---------------*/


  
void matrix_factory_div_2D_order2_1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& idof, 
         DofT<ptv,ptv_is_less>& jdof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    ivdof[12];
  double vdof0[3];
  vdof0[0]=x0;  vdof0[1]=y0;
  vdof0[2] = 0.0;
  ptv vpdof0(3,vdof0);
  ivdof[0] = idof.insert_dof(element,0,vpdof0);

  double vdof1[3];
  vdof1[0]=x0;  vdof1[1]=y0;
  vdof1[2] = 1.0;
  ptv vpdof1(3,vdof1);
  ivdof[1] = idof.insert_dof(element,1,vpdof1);

  double vdof2[3];
  vdof2[0]=G00/2.0+x0;  vdof2[1]=G10/2.0+y0;
  vdof2[2] = 0.0;
  ptv vpdof2(3,vdof2);
  ivdof[2] = idof.insert_dof(element,2,vpdof2);

  double vdof3[3];
  vdof3[0]=G00/2.0+x0;  vdof3[1]=G10/2.0+y0;
  vdof3[2] = 1.0;
  ptv vpdof3(3,vdof3);
  ivdof[3] = idof.insert_dof(element,3,vpdof3);

  double vdof4[3];
  vdof4[0]=G00+x0;  vdof4[1]=G10+y0;
  vdof4[2] = 0.0;
  ptv vpdof4(3,vdof4);
  ivdof[4] = idof.insert_dof(element,4,vpdof4);

  double vdof5[3];
  vdof5[0]=G00+x0;  vdof5[1]=G10+y0;
  vdof5[2] = 1.0;
  ptv vpdof5(3,vdof5);
  ivdof[5] = idof.insert_dof(element,5,vpdof5);

  double vdof6[3];
  vdof6[0]=G01/2.0+x0;  vdof6[1]=y0+G11/2.0;
  vdof6[2] = 0.0;
  ptv vpdof6(3,vdof6);
  ivdof[6] = idof.insert_dof(element,6,vpdof6);

  double vdof7[3];
  vdof7[0]=G01/2.0+x0;  vdof7[1]=y0+G11/2.0;
  vdof7[2] = 1.0;
  ptv vpdof7(3,vdof7);
  ivdof[7] = idof.insert_dof(element,7,vpdof7);

  double vdof8[3];
  vdof8[0]=G01/2.0+G00/2.0+x0;  vdof8[1]=G10/2.0+y0+G11/2.0;
  vdof8[2] = 0.0;
  ptv vpdof8(3,vdof8);
  ivdof[8] = idof.insert_dof(element,8,vpdof8);

  double vdof9[3];
  vdof9[0]=G01/2.0+G00/2.0+x0;  vdof9[1]=G10/2.0+y0+G11/2.0;
  vdof9[2] = 1.0;
  ptv vpdof9(3,vdof9);
  ivdof[9] = idof.insert_dof(element,9,vpdof9);

  double vdof10[3];
  vdof10[0]=G01+x0;  vdof10[1]=y0+G11;
  vdof10[2] = 0.0;
  ptv vpdof10(3,vdof10);
  ivdof[10] = idof.insert_dof(element,10,vpdof10);

  double vdof11[3];
  vdof11[0]=G01+x0;  vdof11[1]=y0+G11;
  vdof11[2] = 1.0;
  ptv vpdof11(3,vdof11);
  ivdof[11] = idof.insert_dof(element,11,vpdof11);

  int    ipdof[3];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  ipdof[0] = jdof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00+x0;  dof1[1]=G10+y0;
  ptv pdof1(2,dof1);
  ipdof[1] = jdof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G01+x0;  dof2[1]=y0+G11;
  ptv pdof2(2,dof2);
  ipdof[2] = jdof.insert_dof(element,2,pdof2);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[3][12];
  A[0][0]=(-1.6666666666666666e-01*Jinv01-1.6666666666666666e-01*Jinv00)*D;
  A[0][1]=(-1.6666666666666666e-01*Jinv11-1.6666666666666666e-01*Jinv10)*D;
  A[0][2]=(-1.6666666666666666e-01*Jinv01+1.6666666666666666e-01*Jinv00)*D;
  A[0][3]=(-1.6666666666666666e-01*Jinv11+1.6666666666666666e-01*Jinv10)*D;
  A[0][4]=(0.0)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(1.6666666666666666e-01*Jinv01-1.6666666666666666e-01*Jinv00)*D;
  A[0][7]=(1.6666666666666666e-01*Jinv11-1.6666666666666666e-01*Jinv10)*D;
  A[0][8]=(1.6666666666666666e-01*Jinv01+1.6666666666666666e-01*Jinv00)*D;
  A[0][9]=(1.6666666666666666e-01*Jinv11+1.6666666666666666e-01*Jinv10)*D;
  A[0][10]=(0.0)*D;
  A[0][11]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(0.0)*D;
  A[1][2]=(-3.3333333333333331e-01*Jinv01-1.6666666666666666e-01*Jinv00)*D;
  A[1][3]=(-3.3333333333333331e-01*Jinv11-1.6666666666666666e-01*Jinv10)*D;
  A[1][4]=(1.6666666666666666e-01*Jinv00)*D;
  A[1][5]=(1.6666666666666666e-01*Jinv10)*D;
  A[1][6]=(-1.6666666666666666e-01*Jinv00)*D;
  A[1][7]=(-1.6666666666666666e-01*Jinv10)*D;
  A[1][8]=(3.3333333333333331e-01*Jinv01+1.6666666666666666e-01*Jinv00)*D;
  A[1][9]=(3.3333333333333331e-01*Jinv11+1.6666666666666666e-01*Jinv10)*D;
  A[1][10]=(0.0)*D;
  A[1][11]=(0.0)*D;
  A[2][0]=(0.0)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(-1.6666666666666666e-01*Jinv01)*D;
  A[2][3]=(-1.6666666666666666e-01*Jinv11)*D;
  A[2][4]=(0.0)*D;
  A[2][5]=(0.0)*D;
  A[2][6]=(-1.6666666666666666e-01*Jinv01-3.3333333333333331e-01*Jinv00)*D;
  A[2][7]=(-1.6666666666666666e-01*Jinv11-3.3333333333333331e-01*Jinv10)*D;
  A[2][8]=(1.6666666666666666e-01*Jinv01+3.3333333333333331e-01*Jinv00)*D;
  A[2][9]=(1.6666666666666666e-01*Jinv11+3.3333333333333331e-01*Jinv10)*D;
  A[2][10]=(1.6666666666666666e-01*Jinv01)*D;
  A[2][11]=(1.6666666666666666e-01*Jinv11)*D;
  int vnbf = 12;  int pnbf = 3;  

  pair<int,int> index; 
  for (int i=0; i< pnbf; i++) {
    index.first = ipdof[i]; 
    for (int j=0; j< vnbf; j++) {
      index.second = ivdof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 1 ---------------*/


  
void matrix_factory_div_2D_order3_1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& idof, 
         DofT<ptv,ptv_is_less>& jdof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    ivdof[20];
  double vdof0[3];
  vdof0[0]=x0;  vdof0[1]=y0;
  vdof0[2] = 0.0;
  ptv vpdof0(3,vdof0);
  ivdof[0] = idof.insert_dof(element,0,vpdof0);

  double vdof1[3];
  vdof1[0]=x0;  vdof1[1]=y0;
  vdof1[2] = 1.0;
  ptv vpdof1(3,vdof1);
  ivdof[1] = idof.insert_dof(element,1,vpdof1);

  double vdof2[3];
  vdof2[0]=G00/3.0+x0;  vdof2[1]=G10/3.0+y0;
  vdof2[2] = 0.0;
  ptv vpdof2(3,vdof2);
  ivdof[2] = idof.insert_dof(element,2,vpdof2);

  double vdof3[3];
  vdof3[0]=G00/3.0+x0;  vdof3[1]=G10/3.0+y0;
  vdof3[2] = 1.0;
  ptv vpdof3(3,vdof3);
  ivdof[3] = idof.insert_dof(element,3,vpdof3);

  double vdof4[3];
  vdof4[0]=(2.0/3.0)*G00+x0;  vdof4[1]=(2.0/3.0)*G10+y0;
  vdof4[2] = 0.0;
  ptv vpdof4(3,vdof4);
  ivdof[4] = idof.insert_dof(element,4,vpdof4);

  double vdof5[3];
  vdof5[0]=(2.0/3.0)*G00+x0;  vdof5[1]=(2.0/3.0)*G10+y0;
  vdof5[2] = 1.0;
  ptv vpdof5(3,vdof5);
  ivdof[5] = idof.insert_dof(element,5,vpdof5);

  double vdof6[3];
  vdof6[0]=G00+x0;  vdof6[1]=G10+y0;
  vdof6[2] = 0.0;
  ptv vpdof6(3,vdof6);
  ivdof[6] = idof.insert_dof(element,6,vpdof6);

  double vdof7[3];
  vdof7[0]=G00+x0;  vdof7[1]=G10+y0;
  vdof7[2] = 1.0;
  ptv vpdof7(3,vdof7);
  ivdof[7] = idof.insert_dof(element,7,vpdof7);

  double vdof8[3];
  vdof8[0]=G01/3.0+x0;  vdof8[1]=y0+G11/3.0;
  vdof8[2] = 0.0;
  ptv vpdof8(3,vdof8);
  ivdof[8] = idof.insert_dof(element,8,vpdof8);

  double vdof9[3];
  vdof9[0]=G01/3.0+x0;  vdof9[1]=y0+G11/3.0;
  vdof9[2] = 1.0;
  ptv vpdof9(3,vdof9);
  ivdof[9] = idof.insert_dof(element,9,vpdof9);

  double vdof10[3];
  vdof10[0]=G01/3.0+G00/3.0+x0;  vdof10[1]=G10/3.0+y0+G11/3.0;
  vdof10[2] = 0.0;
  ptv vpdof10(3,vdof10);
  ivdof[10] = idof.insert_dof(element,10,vpdof10);

  double vdof11[3];
  vdof11[0]=G01/3.0+G00/3.0+x0;  vdof11[1]=G10/3.0+y0+G11/3.0;
  vdof11[2] = 1.0;
  ptv vpdof11(3,vdof11);
  ivdof[11] = idof.insert_dof(element,11,vpdof11);

  double vdof12[3];
  vdof12[0]=G01/3.0+(2.0/3.0)*G00+x0;  vdof12[1]=(2.0/3.0)*G10+y0+G11/3.0;
  vdof12[2] = 0.0;
  ptv vpdof12(3,vdof12);
  ivdof[12] = idof.insert_dof(element,12,vpdof12);

  double vdof13[3];
  vdof13[0]=G01/3.0+(2.0/3.0)*G00+x0;  vdof13[1]=(2.0/3.0)*G10+y0+G11/3.0;
  vdof13[2] = 1.0;
  ptv vpdof13(3,vdof13);
  ivdof[13] = idof.insert_dof(element,13,vpdof13);

  double vdof14[3];
  vdof14[0]=(2.0/3.0)*G01+x0;  vdof14[1]=y0+(2.0/3.0)*G11;
  vdof14[2] = 0.0;
  ptv vpdof14(3,vdof14);
  ivdof[14] = idof.insert_dof(element,14,vpdof14);

  double vdof15[3];
  vdof15[0]=(2.0/3.0)*G01+x0;  vdof15[1]=y0+(2.0/3.0)*G11;
  vdof15[2] = 1.0;
  ptv vpdof15(3,vdof15);
  ivdof[15] = idof.insert_dof(element,15,vpdof15);

  double vdof16[3];
  vdof16[0]=(2.0/3.0)*G01+G00/3.0+x0;  vdof16[1]=G10/3.0+y0+(2.0/3.0)*G11;
  vdof16[2] = 0.0;
  ptv vpdof16(3,vdof16);
  ivdof[16] = idof.insert_dof(element,16,vpdof16);

  double vdof17[3];
  vdof17[0]=(2.0/3.0)*G01+G00/3.0+x0;  vdof17[1]=G10/3.0+y0+(2.0/3.0)*G11;
  vdof17[2] = 1.0;
  ptv vpdof17(3,vdof17);
  ivdof[17] = idof.insert_dof(element,17,vpdof17);

  double vdof18[3];
  vdof18[0]=G01+x0;  vdof18[1]=y0+G11;
  vdof18[2] = 0.0;
  ptv vpdof18(3,vdof18);
  ivdof[18] = idof.insert_dof(element,18,vpdof18);

  double vdof19[3];
  vdof19[0]=G01+x0;  vdof19[1]=y0+G11;
  vdof19[2] = 1.0;
  ptv vpdof19(3,vdof19);
  ivdof[19] = idof.insert_dof(element,19,vpdof19);

  int    ipdof[3];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  ipdof[0] = jdof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00+x0;  dof1[1]=G10+y0;
  ptv pdof1(2,dof1);
  ipdof[1] = jdof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G01+x0;  dof2[1]=y0+G11;
  ptv pdof2(2,dof2);
  ipdof[2] = jdof.insert_dof(element,2,pdof2);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[3][20];
  A[0][0]=(-9.1666666666666660e-02*Jinv00-9.1666666666666660e-02*Jinv01)*D;
  A[0][1]=(-9.1666666666666660e-02*Jinv11-9.1666666666666660e-02*Jinv10)*D;
  A[0][2]=(3.7499999999999999e-02*Jinv00-2.6250000000000001e-01*Jinv01)*D;
  A[0][3]=(-2.6250000000000001e-01*Jinv11+3.7499999999999999e-02*Jinv10)*D;
  A[0][4]=(3.7499999999999999e-02*Jinv00-3.7499999999999999e-02*Jinv01)*D;
  A[0][5]=(-3.7499999999999999e-02*Jinv11+3.7499999999999999e-02*Jinv10)*D;
  A[0][6]=(1.6666666666666666e-02*Jinv00)*D;
  A[0][7]=(1.6666666666666666e-02*Jinv10)*D;
  A[0][8]=(-2.6250000000000001e-01*Jinv00+3.7499999999999999e-02*Jinv01)*D;
  A[0][9]=(3.7499999999999999e-02*Jinv11-2.6250000000000001e-01*Jinv10)*D;
  A[0][10]=(2.2500000000000001e-01*Jinv00+2.2500000000000001e-01*Jinv01)*D;
  A[0][11]=(2.2500000000000001e-01*Jinv11+2.2500000000000001e-01*Jinv10)*D;
  A[0][12]=(3.7499999999999999e-02*Jinv00+3.7499999999999999e-02*Jinv01)*D;
  A[0][13]=(3.7499999999999999e-02*Jinv11+3.7499999999999999e-02*Jinv10)*D;
  A[0][14]=(-3.7499999999999999e-02*Jinv00+3.7499999999999999e-02*Jinv01)*D;
  A[0][15]=(3.7499999999999999e-02*Jinv11-3.7499999999999999e-02*Jinv10)*D;
  A[0][16]=(3.7499999999999999e-02*Jinv00+3.7499999999999999e-02*Jinv01)*D;
  A[0][17]=(3.7499999999999999e-02*Jinv11+3.7499999999999999e-02*Jinv10)*D;
  A[0][18]=(1.6666666666666666e-02*Jinv01)*D;
  A[0][19]=(1.6666666666666666e-02*Jinv11)*D;
  A[1][0]=(-1.6666666666666666e-02*Jinv00-1.6666666666666666e-02*Jinv01)*D;
  A[1][1]=(-1.6666666666666666e-02*Jinv11-1.6666666666666666e-02*Jinv10)*D;
  A[1][2]=(-3.7499999999999999e-02*Jinv00-7.4999999999999997e-02*Jinv01)*D;
  A[1][3]=(-7.4999999999999997e-02*Jinv11-3.7499999999999999e-02*Jinv10)*D;
  A[1][4]=(-3.7499999999999999e-02*Jinv00-2.9999999999999999e-01*Jinv01)*D;
  A[1][5]=(-2.9999999999999999e-01*Jinv11-3.7499999999999999e-02*Jinv10)*D;
  A[1][6]=(9.1666666666666660e-02*Jinv00)*D;
  A[1][7]=(9.1666666666666660e-02*Jinv10)*D;
  A[1][8]=(-3.7499999999999999e-02*Jinv00)*D;
  A[1][9]=(-3.7499999999999999e-02*Jinv10)*D;
  A[1][10]=(-2.2500000000000001e-01*Jinv00)*D;
  A[1][11]=(-2.2500000000000001e-01*Jinv10)*D;
  A[1][12]=(2.6250000000000001e-01*Jinv00+2.9999999999999999e-01*Jinv01)*D;
  A[1][13]=(2.9999999999999999e-01*Jinv11+2.6250000000000001e-01*Jinv10)*D;
  A[1][14]=(-3.7499999999999999e-02*Jinv00)*D;
  A[1][15]=(-3.7499999999999999e-02*Jinv10)*D;
  A[1][16]=(3.7499999999999999e-02*Jinv00+7.4999999999999997e-02*Jinv01)*D;
  A[1][17]=(7.4999999999999997e-02*Jinv11+3.7499999999999999e-02*Jinv10)*D;
  A[1][18]=(1.6666666666666666e-02*Jinv01)*D;
  A[1][19]=(1.6666666666666666e-02*Jinv11)*D;
  A[2][0]=(-1.6666666666666666e-02*Jinv00-1.6666666666666666e-02*Jinv01)*D;
  A[2][1]=(-1.6666666666666666e-02*Jinv11-1.6666666666666666e-02*Jinv10)*D;
  A[2][2]=(-3.7499999999999999e-02*Jinv01)*D;
  A[2][3]=(-3.7499999999999999e-02*Jinv11)*D;
  A[2][4]=(-3.7499999999999999e-02*Jinv01)*D;
  A[2][5]=(-3.7499999999999999e-02*Jinv11)*D;
  A[2][6]=(1.6666666666666666e-02*Jinv00)*D;
  A[2][7]=(1.6666666666666666e-02*Jinv10)*D;
  A[2][8]=(-7.4999999999999997e-02*Jinv00-3.7499999999999999e-02*Jinv01)*D;
  A[2][9]=(-3.7499999999999999e-02*Jinv11-7.4999999999999997e-02*Jinv10)*D;
  A[2][10]=(-2.2500000000000001e-01*Jinv01)*D;
  A[2][11]=(-2.2500000000000001e-01*Jinv11)*D;
  A[2][12]=(7.4999999999999997e-02*Jinv00+3.7499999999999999e-02*Jinv01)*D;
  A[2][13]=(3.7499999999999999e-02*Jinv11+7.4999999999999997e-02*Jinv10)*D;
  A[2][14]=(-2.9999999999999999e-01*Jinv00-3.7499999999999999e-02*Jinv01)*D;
  A[2][15]=(-3.7499999999999999e-02*Jinv11-2.9999999999999999e-01*Jinv10)*D;
  A[2][16]=(2.9999999999999999e-01*Jinv00+2.6250000000000001e-01*Jinv01)*D;
  A[2][17]=(2.6250000000000001e-01*Jinv11+2.9999999999999999e-01*Jinv10)*D;
  A[2][18]=(9.1666666666666660e-02*Jinv01)*D;
  A[2][19]=(9.1666666666666660e-02*Jinv11)*D;
  int vnbf = 20;  int pnbf = 3;  

  pair<int,int> index; 
  for (int i=0; i< pnbf; i++) {
    index.first = ipdof[i]; 
    for (int j=0; j< vnbf; j++) {
      index.second = ivdof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 1 0 ---------------*/


  
void matrix_factory_div_2D_order1_0 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& idof, 
         DofT<ptv,ptv_is_less>& jdof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    ivdof[6];
  double vdof0[3];
  vdof0[0]=x0;  vdof0[1]=y0;
  vdof0[2] = 0.0;
  ptv vpdof0(3,vdof0);
  ivdof[0] = idof.insert_dof(element,0,vpdof0);

  double vdof1[3];
  vdof1[0]=x0;  vdof1[1]=y0;
  vdof1[2] = 1.0;
  ptv vpdof1(3,vdof1);
  ivdof[1] = idof.insert_dof(element,1,vpdof1);

  double vdof2[3];
  vdof2[0]=G00+x0;  vdof2[1]=G10+y0;
  vdof2[2] = 0.0;
  ptv vpdof2(3,vdof2);
  ivdof[2] = idof.insert_dof(element,2,vpdof2);

  double vdof3[3];
  vdof3[0]=G00+x0;  vdof3[1]=G10+y0;
  vdof3[2] = 1.0;
  ptv vpdof3(3,vdof3);
  ivdof[3] = idof.insert_dof(element,3,vpdof3);

  double vdof4[3];
  vdof4[0]=G01+x0;  vdof4[1]=y0+G11;
  vdof4[2] = 0.0;
  ptv vpdof4(3,vdof4);
  ivdof[4] = idof.insert_dof(element,4,vpdof4);

  double vdof5[3];
  vdof5[0]=G01+x0;  vdof5[1]=y0+G11;
  vdof5[2] = 1.0;
  ptv vpdof5(3,vdof5);
  ivdof[5] = idof.insert_dof(element,5,vpdof5);

  int    ipdof[1];
  double dof0[2];
  dof0[0]=G01/3.0+G00/3.0+x0;  dof0[1]=G10/3.0+y0+G11/3.0;
  ptv pdof0(2,dof0);
  ipdof[0] = jdof.insert_dof(element,0,pdof0);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[1][6];
  A[0][0]=(-5.0000000000000000e-01*Jinv01-5.0000000000000000e-01*Jinv00)*D;
  A[0][1]=(-5.0000000000000000e-01*Jinv11-5.0000000000000000e-01*Jinv10)*D;
  A[0][2]=(5.0000000000000000e-01*Jinv00)*D;
  A[0][3]=(5.0000000000000000e-01*Jinv10)*D;
  A[0][4]=(5.0000000000000000e-01*Jinv01)*D;
  A[0][5]=(5.0000000000000000e-01*Jinv11)*D;
  int vnbf = 6;  int pnbf = 1;  

  pair<int,int> index; 
  for (int i=0; i< pnbf; i++) {
    index.first = ipdof[i]; 
    for (int j=0; j< vnbf; j++) {
      index.second = ivdof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 0 ---------------*/


  
void matrix_factory_div_2D_order2_0 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& idof, 
         DofT<ptv,ptv_is_less>& jdof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    ivdof[12];
  double vdof0[3];
  vdof0[0]=x0;  vdof0[1]=y0;
  vdof0[2] = 0.0;
  ptv vpdof0(3,vdof0);
  ivdof[0] = idof.insert_dof(element,0,vpdof0);

  double vdof1[3];
  vdof1[0]=x0;  vdof1[1]=y0;
  vdof1[2] = 1.0;
  ptv vpdof1(3,vdof1);
  ivdof[1] = idof.insert_dof(element,1,vpdof1);

  double vdof2[3];
  vdof2[0]=G00/2.0+x0;  vdof2[1]=G10/2.0+y0;
  vdof2[2] = 0.0;
  ptv vpdof2(3,vdof2);
  ivdof[2] = idof.insert_dof(element,2,vpdof2);

  double vdof3[3];
  vdof3[0]=G00/2.0+x0;  vdof3[1]=G10/2.0+y0;
  vdof3[2] = 1.0;
  ptv vpdof3(3,vdof3);
  ivdof[3] = idof.insert_dof(element,3,vpdof3);

  double vdof4[3];
  vdof4[0]=G00+x0;  vdof4[1]=G10+y0;
  vdof4[2] = 0.0;
  ptv vpdof4(3,vdof4);
  ivdof[4] = idof.insert_dof(element,4,vpdof4);

  double vdof5[3];
  vdof5[0]=G00+x0;  vdof5[1]=G10+y0;
  vdof5[2] = 1.0;
  ptv vpdof5(3,vdof5);
  ivdof[5] = idof.insert_dof(element,5,vpdof5);

  double vdof6[3];
  vdof6[0]=G01/2.0+x0;  vdof6[1]=y0+G11/2.0;
  vdof6[2] = 0.0;
  ptv vpdof6(3,vdof6);
  ivdof[6] = idof.insert_dof(element,6,vpdof6);

  double vdof7[3];
  vdof7[0]=G01/2.0+x0;  vdof7[1]=y0+G11/2.0;
  vdof7[2] = 1.0;
  ptv vpdof7(3,vdof7);
  ivdof[7] = idof.insert_dof(element,7,vpdof7);

  double vdof8[3];
  vdof8[0]=G01/2.0+G00/2.0+x0;  vdof8[1]=G10/2.0+y0+G11/2.0;
  vdof8[2] = 0.0;
  ptv vpdof8(3,vdof8);
  ivdof[8] = idof.insert_dof(element,8,vpdof8);

  double vdof9[3];
  vdof9[0]=G01/2.0+G00/2.0+x0;  vdof9[1]=G10/2.0+y0+G11/2.0;
  vdof9[2] = 1.0;
  ptv vpdof9(3,vdof9);
  ivdof[9] = idof.insert_dof(element,9,vpdof9);

  double vdof10[3];
  vdof10[0]=G01+x0;  vdof10[1]=y0+G11;
  vdof10[2] = 0.0;
  ptv vpdof10(3,vdof10);
  ivdof[10] = idof.insert_dof(element,10,vpdof10);

  double vdof11[3];
  vdof11[0]=G01+x0;  vdof11[1]=y0+G11;
  vdof11[2] = 1.0;
  ptv vpdof11(3,vdof11);
  ivdof[11] = idof.insert_dof(element,11,vpdof11);

  int    ipdof[1];
  double dof0[2];
  dof0[0]=G01/3.0+G00/3.0+x0;  dof0[1]=G10/3.0+y0+G11/3.0;
  ptv pdof0(2,dof0);
  ipdof[0] = jdof.insert_dof(element,0,pdof0);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[1][12];
  A[0][0]=(-1.6666666666666666e-01*Jinv01-1.6666666666666666e-01*Jinv00)*D;
  A[0][1]=(-1.6666666666666666e-01*Jinv11-1.6666666666666666e-01*Jinv10)*D;
  A[0][2]=(-6.6666666666666663e-01*Jinv01)*D;
  A[0][3]=(-6.6666666666666663e-01*Jinv11)*D;
  A[0][4]=(1.6666666666666666e-01*Jinv00)*D;
  A[0][5]=(1.6666666666666666e-01*Jinv10)*D;
  A[0][6]=(-6.6666666666666663e-01*Jinv00)*D;
  A[0][7]=(-6.6666666666666663e-01*Jinv10)*D;
  A[0][8]=(6.6666666666666663e-01*Jinv01+6.6666666666666663e-01*Jinv00)*D;
  A[0][9]=(6.6666666666666663e-01*Jinv11+6.6666666666666663e-01*Jinv10)*D;
  A[0][10]=(1.6666666666666666e-01*Jinv01)*D;
  A[0][11]=(1.6666666666666666e-01*Jinv11)*D;
  int vnbf = 12;  int pnbf = 1;  

  pair<int,int> index; 
  for (int i=0; i< pnbf; i++) {
    index.first = ipdof[i]; 
    for (int j=0; j< vnbf; j++) {
      index.second = ivdof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}
