
#include <math.h>
#include <utility>
#include <map>
#include <ptv.h>
#include <DofT.h>
#include <ScalarFunction.h>

using namespace std; 



/* ------------- order 1 ---------------*/


  
void matrix_factory_mass_2D_order1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[3];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00+x0;  dof1[1]=G10+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G01+x0;  dof2[1]=y0+G11;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

 
  double A[3][3];
  A[0][0]=(8.3333333333333329e-02)*D;
  A[0][1]=(4.1666666666666664e-02)*D;
  A[0][2]=(4.1666666666666664e-02)*D;
  A[1][0]=(4.1666666666666664e-02)*D;
  A[1][1]=(8.3333333333333329e-02)*D;
  A[1][2]=(4.1666666666666664e-02)*D;
  A[2][0]=(4.1666666666666664e-02)*D;
  A[2][1]=(4.1666666666666664e-02)*D;
  A[2][2]=(8.3333333333333329e-02)*D;
  int nbf = 3;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 ---------------*/


  
void matrix_factory_mass_2D_order2 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[6];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/2.0+x0;  dof1[1]=G10/2.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G00+x0;  dof2[1]=G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=G01/2.0+x0;  dof3[1]=y0+G11/2.0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G01/2.0+G00/2.0+x0;  dof4[1]=G10/2.0+y0+G11/2.0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01+x0;  dof5[1]=y0+G11;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

 
  double A[6][6];
  A[0][0]=(1.6666666666666666e-02)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(-2.7777777777777779e-03)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(-1.1111111111111112e-02)*D;
  A[0][5]=(-2.7777777777777779e-03)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(8.8888888888888892e-02)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(4.4444444444444446e-02)*D;
  A[1][4]=(4.4444444444444446e-02)*D;
  A[1][5]=(-1.1111111111111112e-02)*D;
  A[2][0]=(-2.7777777777777779e-03)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(1.6666666666666666e-02)*D;
  A[2][3]=(-1.1111111111111112e-02)*D;
  A[2][4]=(0.0)*D;
  A[2][5]=(-2.7777777777777779e-03)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(4.4444444444444446e-02)*D;
  A[3][2]=(-1.1111111111111112e-02)*D;
  A[3][3]=(8.8888888888888892e-02)*D;
  A[3][4]=(4.4444444444444446e-02)*D;
  A[3][5]=(0.0)*D;
  A[4][0]=(-1.1111111111111112e-02)*D;
  A[4][1]=(4.4444444444444446e-02)*D;
  A[4][2]=(0.0)*D;
  A[4][3]=(4.4444444444444446e-02)*D;
  A[4][4]=(8.8888888888888892e-02)*D;
  A[4][5]=(0.0)*D;
  A[5][0]=(-2.7777777777777779e-03)*D;
  A[5][1]=(-1.1111111111111112e-02)*D;
  A[5][2]=(-2.7777777777777779e-03)*D;
  A[5][3]=(0.0)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(1.6666666666666666e-02)*D;
  int nbf = 6;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 ---------------*/


  
void matrix_factory_mass_2D_order3 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[10];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/3.0+x0;  dof1[1]=G10/3.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=(2.0/3.0)*G00+x0;  dof2[1]=(2.0/3.0)*G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=G00+x0;  dof3[1]=G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G01/3.0+x0;  dof4[1]=y0+G11/3.0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01/3.0+G00/3.0+x0;  dof5[1]=G10/3.0+y0+G11/3.0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/3.0+(2.0/3.0)*G00+x0;  dof6[1]=(2.0/3.0)*G10+y0+G11/3.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=(2.0/3.0)*G01+x0;  dof7[1]=y0+(2.0/3.0)*G11;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=(2.0/3.0)*G01+G00/3.0+x0;  dof8[1]=G10/3.0+y0+(2.0/3.0)*G11;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01+x0;  dof9[1]=y0+G11;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

 
  double A[10][10];
  A[0][0]=(5.6547619047619046e-03)*D;
  A[0][1]=(1.3392857142857143e-03)*D;
  A[0][2]=(0.0)*D;
  A[0][3]=(8.1845238095238097e-04)*D;
  A[0][4]=(1.3392857142857143e-03)*D;
  A[0][5]=(2.6785714285714286e-03)*D;
  A[0][6]=(2.0089285714285712e-03)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(2.0089285714285712e-03)*D;
  A[0][9]=(8.1845238095238097e-04)*D;
  A[1][0]=(1.3392857142857143e-03)*D;
  A[1][1]=(4.0178571428571432e-02)*D;
  A[1][2]=(-1.4062500000000000e-02)*D;
  A[1][3]=(0.0)*D;
  A[1][4]=(2.0089285714285716e-02)*D;
  A[1][5]=(1.2053571428571429e-02)*D;
  A[1][6]=(-1.0044642857142858e-02)*D;
  A[1][7]=(-1.0044642857142858e-02)*D;
  A[1][8]=(-4.0178571428571425e-03)*D;
  A[1][9]=(2.0089285714285712e-03)*D;
  A[2][0]=(0.0)*D;
  A[2][1]=(-1.4062500000000000e-02)*D;
  A[2][2]=(4.0178571428571432e-02)*D;
  A[2][3]=(1.3392857142857143e-03)*D;
  A[2][4]=(-1.0044642857142858e-02)*D;
  A[2][5]=(1.2053571428571429e-02)*D;
  A[2][6]=(2.0089285714285716e-02)*D;
  A[2][7]=(-4.0178571428571425e-03)*D;
  A[2][8]=(-1.0044642857142858e-02)*D;
  A[2][9]=(2.0089285714285712e-03)*D;
  A[3][0]=(8.1845238095238097e-04)*D;
  A[3][1]=(0.0)*D;
  A[3][2]=(1.3392857142857143e-03)*D;
  A[3][3]=(5.6547619047619046e-03)*D;
  A[3][4]=(2.0089285714285712e-03)*D;
  A[3][5]=(2.6785714285714286e-03)*D;
  A[3][6]=(1.3392857142857143e-03)*D;
  A[3][7]=(2.0089285714285712e-03)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(8.1845238095238097e-04)*D;
  A[4][0]=(1.3392857142857143e-03)*D;
  A[4][1]=(2.0089285714285716e-02)*D;
  A[4][2]=(-1.0044642857142858e-02)*D;
  A[4][3]=(2.0089285714285712e-03)*D;
  A[4][4]=(4.0178571428571432e-02)*D;
  A[4][5]=(1.2053571428571429e-02)*D;
  A[4][6]=(-4.0178571428571425e-03)*D;
  A[4][7]=(-1.4062500000000000e-02)*D;
  A[4][8]=(-1.0044642857142858e-02)*D;
  A[4][9]=(0.0)*D;
  A[5][0]=(2.6785714285714286e-03)*D;
  A[5][1]=(1.2053571428571429e-02)*D;
  A[5][2]=(1.2053571428571429e-02)*D;
  A[5][3]=(2.6785714285714286e-03)*D;
  A[5][4]=(1.2053571428571429e-02)*D;
  A[5][5]=(1.4464285714285716e-01)*D;
  A[5][6]=(1.2053571428571429e-02)*D;
  A[5][7]=(1.2053571428571429e-02)*D;
  A[5][8]=(1.2053571428571429e-02)*D;
  A[5][9]=(2.6785714285714286e-03)*D;
  A[6][0]=(2.0089285714285712e-03)*D;
  A[6][1]=(-1.0044642857142858e-02)*D;
  A[6][2]=(2.0089285714285716e-02)*D;
  A[6][3]=(1.3392857142857143e-03)*D;
  A[6][4]=(-4.0178571428571425e-03)*D;
  A[6][5]=(1.2053571428571429e-02)*D;
  A[6][6]=(4.0178571428571432e-02)*D;
  A[6][7]=(-1.0044642857142858e-02)*D;
  A[6][8]=(-1.4062500000000000e-02)*D;
  A[6][9]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(-1.0044642857142858e-02)*D;
  A[7][2]=(-4.0178571428571425e-03)*D;
  A[7][3]=(2.0089285714285712e-03)*D;
  A[7][4]=(-1.4062500000000000e-02)*D;
  A[7][5]=(1.2053571428571429e-02)*D;
  A[7][6]=(-1.0044642857142858e-02)*D;
  A[7][7]=(4.0178571428571432e-02)*D;
  A[7][8]=(2.0089285714285716e-02)*D;
  A[7][9]=(1.3392857142857143e-03)*D;
  A[8][0]=(2.0089285714285712e-03)*D;
  A[8][1]=(-4.0178571428571425e-03)*D;
  A[8][2]=(-1.0044642857142858e-02)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(-1.0044642857142858e-02)*D;
  A[8][5]=(1.2053571428571429e-02)*D;
  A[8][6]=(-1.4062500000000000e-02)*D;
  A[8][7]=(2.0089285714285716e-02)*D;
  A[8][8]=(4.0178571428571432e-02)*D;
  A[8][9]=(1.3392857142857143e-03)*D;
  A[9][0]=(8.1845238095238097e-04)*D;
  A[9][1]=(2.0089285714285712e-03)*D;
  A[9][2]=(2.0089285714285712e-03)*D;
  A[9][3]=(8.1845238095238097e-04)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(2.6785714285714286e-03)*D;
  A[9][6]=(0.0)*D;
  A[9][7]=(1.3392857142857143e-03)*D;
  A[9][8]=(1.3392857142857143e-03)*D;
  A[9][9]=(5.6547619047619046e-03)*D;
  int nbf = 10;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 4 ---------------*/


  
void matrix_factory_mass_2D_order4 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[15];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/4.0+x0;  dof1[1]=G10/4.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G00/2.0+x0;  dof2[1]=G10/2.0+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=(3.0/4.0)*G00+x0;  dof3[1]=(3.0/4.0)*G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G00+x0;  dof4[1]=G10+y0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01/4.0+x0;  dof5[1]=y0+G11/4.0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/4.0+G00/4.0+x0;  dof6[1]=G10/4.0+y0+G11/4.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=G01/4.0+G00/2.0+x0;  dof7[1]=G10/2.0+y0+G11/4.0;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=G01/4.0+(3.0/4.0)*G00+x0;  dof8[1]=(3.0/4.0)*G10+y0+G11/4.0;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01/2.0+x0;  dof9[1]=y0+G11/2.0;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[2];
  dof10[0]=G01/2.0+G00/4.0+x0;  dof10[1]=G10/4.0+y0+G11/2.0;
  ptv pdof10(2,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[2];
  dof11[0]=G01/2.0+G00/2.0+x0;  dof11[1]=G10/2.0+y0+G11/2.0;
  ptv pdof11(2,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[2];
  dof12[0]=(3.0/4.0)*G01+x0;  dof12[1]=y0+(3.0/4.0)*G11;
  ptv pdof12(2,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[2];
  dof13[0]=(3.0/4.0)*G01+G00/4.0+x0;  dof13[1]=G10/4.0+y0+(3.0/4.0)*G11;
  ptv pdof13(2,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[2];
  dof14[0]=G01+x0;  dof14[1]=y0+G11;
  ptv pdof14(2,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

 
  double A[15][15];
  A[0][0]=(2.5573192239858908e-03)*D;
  A[0][1]=(1.4109347442680777e-03)*D;
  A[0][2]=(-7.0546737213403885e-04)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(-2.3809523809523810e-04)*D;
  A[0][5]=(1.4109347442680777e-03)*D;
  A[0][6]=(1.4109347442680777e-03)*D;
  A[0][7]=(-1.4109347442680777e-03)*D;
  A[0][8]=(-9.8765432098765434e-04)*D;
  A[0][9]=(-7.0546737213403885e-04)*D;
  A[0][10]=(-1.4109347442680777e-03)*D;
  A[0][11]=(-1.0582010582010582e-04)*D;
  A[0][12]=(0.0)*D;
  A[0][13]=(-9.8765432098765434e-04)*D;
  A[0][14]=(-2.3809523809523810e-04)*D;
  A[1][0]=(1.4109347442680777e-03)*D;
  A[1][1]=(2.2574955908289243e-02)*D;
  A[1][2]=(-1.1287477954144622e-02)*D;
  A[1][3]=(6.7724867724867728e-03)*D;
  A[1][4]=(0.0)*D;
  A[1][5]=(1.1287477954144622e-02)*D;
  A[1][6]=(1.1287477954144622e-02)*D;
  A[1][7]=(2.2574955908289240e-03)*D;
  A[1][8]=(4.5149911816578479e-03)*D;
  A[1][9]=(-8.4656084656084662e-03)*D;
  A[1][10]=(-2.2574955908289240e-03)*D;
  A[1][11]=(5.6437389770723099e-04)*D;
  A[1][12]=(4.5149911816578479e-03)*D;
  A[1][13]=(2.2574955908289240e-03)*D;
  A[1][14]=(-9.8765432098765434e-04)*D;
  A[2][0]=(-7.0546737213403885e-04)*D;
  A[2][1]=(-1.1287477954144622e-02)*D;
  A[2][2]=(2.7936507936507936e-02)*D;
  A[2][3]=(-1.1287477954144622e-02)*D;
  A[2][4]=(-7.0546737213403885e-04)*D;
  A[2][5]=(-8.4656084656084662e-03)*D;
  A[2][6]=(3.3862433862433864e-03)*D;
  A[2][7]=(3.3862433862433864e-03)*D;
  A[2][8]=(-8.4656084656084662e-03)*D;
  A[2][9]=(4.2328042328042330e-04)*D;
  A[2][10]=(-6.7724867724867728e-03)*D;
  A[2][11]=(4.2328042328042330e-04)*D;
  A[2][12]=(5.6437389770723099e-04)*D;
  A[2][13]=(5.6437389770723099e-04)*D;
  A[2][14]=(-1.0582010582010582e-04)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(6.7724867724867728e-03)*D;
  A[3][2]=(-1.1287477954144622e-02)*D;
  A[3][3]=(2.2574955908289243e-02)*D;
  A[3][4]=(1.4109347442680777e-03)*D;
  A[3][5]=(4.5149911816578479e-03)*D;
  A[3][6]=(2.2574955908289240e-03)*D;
  A[3][7]=(1.1287477954144622e-02)*D;
  A[3][8]=(1.1287477954144622e-02)*D;
  A[3][9]=(5.6437389770723099e-04)*D;
  A[3][10]=(-2.2574955908289240e-03)*D;
  A[3][11]=(-8.4656084656084662e-03)*D;
  A[3][12]=(2.2574955908289240e-03)*D;
  A[3][13]=(4.5149911816578479e-03)*D;
  A[3][14]=(-9.8765432098765434e-04)*D;
  A[4][0]=(-2.3809523809523810e-04)*D;
  A[4][1]=(0.0)*D;
  A[4][2]=(-7.0546737213403885e-04)*D;
  A[4][3]=(1.4109347442680777e-03)*D;
  A[4][4]=(2.5573192239858908e-03)*D;
  A[4][5]=(-9.8765432098765434e-04)*D;
  A[4][6]=(-1.4109347442680777e-03)*D;
  A[4][7]=(1.4109347442680777e-03)*D;
  A[4][8]=(1.4109347442680777e-03)*D;
  A[4][9]=(-1.0582010582010582e-04)*D;
  A[4][10]=(-1.4109347442680777e-03)*D;
  A[4][11]=(-7.0546737213403885e-04)*D;
  A[4][12]=(-9.8765432098765434e-04)*D;
  A[4][13]=(0.0)*D;
  A[4][14]=(-2.3809523809523810e-04)*D;
  A[5][0]=(1.4109347442680777e-03)*D;
  A[5][1]=(1.1287477954144622e-02)*D;
  A[5][2]=(-8.4656084656084662e-03)*D;
  A[5][3]=(4.5149911816578479e-03)*D;
  A[5][4]=(-9.8765432098765434e-04)*D;
  A[5][5]=(2.2574955908289243e-02)*D;
  A[5][6]=(1.1287477954144622e-02)*D;
  A[5][7]=(-2.2574955908289240e-03)*D;
  A[5][8]=(2.2574955908289240e-03)*D;
  A[5][9]=(-1.1287477954144622e-02)*D;
  A[5][10]=(2.2574955908289240e-03)*D;
  A[5][11]=(5.6437389770723099e-04)*D;
  A[5][12]=(6.7724867724867728e-03)*D;
  A[5][13]=(4.5149911816578479e-03)*D;
  A[5][14]=(0.0)*D;
  A[6][0]=(1.4109347442680777e-03)*D;
  A[6][1]=(1.1287477954144622e-02)*D;
  A[6][2]=(3.3862433862433864e-03)*D;
  A[6][3]=(2.2574955908289240e-03)*D;
  A[6][4]=(-1.4109347442680777e-03)*D;
  A[6][5]=(1.1287477954144622e-02)*D;
  A[6][6]=(9.4814814814814810e-02)*D;
  A[6][7]=(-1.3544973544973546e-02)*D;
  A[6][8]=(-2.2574955908289240e-03)*D;
  A[6][9]=(3.3862433862433864e-03)*D;
  A[6][10]=(-1.3544973544973546e-02)*D;
  A[6][11]=(-6.7724867724867728e-03)*D;
  A[6][12]=(2.2574955908289240e-03)*D;
  A[6][13]=(-2.2574955908289240e-03)*D;
  A[6][14]=(-1.4109347442680777e-03)*D;
  A[7][0]=(-1.4109347442680777e-03)*D;
  A[7][1]=(2.2574955908289240e-03)*D;
  A[7][2]=(3.3862433862433864e-03)*D;
  A[7][3]=(1.1287477954144622e-02)*D;
  A[7][4]=(1.4109347442680777e-03)*D;
  A[7][5]=(-2.2574955908289240e-03)*D;
  A[7][6]=(-1.3544973544973546e-02)*D;
  A[7][7]=(9.4814814814814810e-02)*D;
  A[7][8]=(1.1287477954144622e-02)*D;
  A[7][9]=(-6.7724867724867728e-03)*D;
  A[7][10]=(-1.3544973544973546e-02)*D;
  A[7][11]=(3.3862433862433864e-03)*D;
  A[7][12]=(-2.2574955908289240e-03)*D;
  A[7][13]=(2.2574955908289240e-03)*D;
  A[7][14]=(-1.4109347442680777e-03)*D;
  A[8][0]=(-9.8765432098765434e-04)*D;
  A[8][1]=(4.5149911816578479e-03)*D;
  A[8][2]=(-8.4656084656084662e-03)*D;
  A[8][3]=(1.1287477954144622e-02)*D;
  A[8][4]=(1.4109347442680777e-03)*D;
  A[8][5]=(2.2574955908289240e-03)*D;
  A[8][6]=(-2.2574955908289240e-03)*D;
  A[8][7]=(1.1287477954144622e-02)*D;
  A[8][8]=(2.2574955908289243e-02)*D;
  A[8][9]=(5.6437389770723099e-04)*D;
  A[8][10]=(2.2574955908289240e-03)*D;
  A[8][11]=(-1.1287477954144622e-02)*D;
  A[8][12]=(4.5149911816578479e-03)*D;
  A[8][13]=(6.7724867724867728e-03)*D;
  A[8][14]=(0.0)*D;
  A[9][0]=(-7.0546737213403885e-04)*D;
  A[9][1]=(-8.4656084656084662e-03)*D;
  A[9][2]=(4.2328042328042330e-04)*D;
  A[9][3]=(5.6437389770723099e-04)*D;
  A[9][4]=(-1.0582010582010582e-04)*D;
  A[9][5]=(-1.1287477954144622e-02)*D;
  A[9][6]=(3.3862433862433864e-03)*D;
  A[9][7]=(-6.7724867724867728e-03)*D;
  A[9][8]=(5.6437389770723099e-04)*D;
  A[9][9]=(2.7936507936507936e-02)*D;
  A[9][10]=(3.3862433862433864e-03)*D;
  A[9][11]=(4.2328042328042330e-04)*D;
  A[9][12]=(-1.1287477954144622e-02)*D;
  A[9][13]=(-8.4656084656084662e-03)*D;
  A[9][14]=(-7.0546737213403885e-04)*D;
  A[10][0]=(-1.4109347442680777e-03)*D;
  A[10][1]=(-2.2574955908289240e-03)*D;
  A[10][2]=(-6.7724867724867728e-03)*D;
  A[10][3]=(-2.2574955908289240e-03)*D;
  A[10][4]=(-1.4109347442680777e-03)*D;
  A[10][5]=(2.2574955908289240e-03)*D;
  A[10][6]=(-1.3544973544973546e-02)*D;
  A[10][7]=(-1.3544973544973546e-02)*D;
  A[10][8]=(2.2574955908289240e-03)*D;
  A[10][9]=(3.3862433862433864e-03)*D;
  A[10][10]=(9.4814814814814810e-02)*D;
  A[10][11]=(3.3862433862433864e-03)*D;
  A[10][12]=(1.1287477954144622e-02)*D;
  A[10][13]=(1.1287477954144622e-02)*D;
  A[10][14]=(1.4109347442680777e-03)*D;
  A[11][0]=(-1.0582010582010582e-04)*D;
  A[11][1]=(5.6437389770723099e-04)*D;
  A[11][2]=(4.2328042328042330e-04)*D;
  A[11][3]=(-8.4656084656084662e-03)*D;
  A[11][4]=(-7.0546737213403885e-04)*D;
  A[11][5]=(5.6437389770723099e-04)*D;
  A[11][6]=(-6.7724867724867728e-03)*D;
  A[11][7]=(3.3862433862433864e-03)*D;
  A[11][8]=(-1.1287477954144622e-02)*D;
  A[11][9]=(4.2328042328042330e-04)*D;
  A[11][10]=(3.3862433862433864e-03)*D;
  A[11][11]=(2.7936507936507936e-02)*D;
  A[11][12]=(-8.4656084656084662e-03)*D;
  A[11][13]=(-1.1287477954144622e-02)*D;
  A[11][14]=(-7.0546737213403885e-04)*D;
  A[12][0]=(0.0)*D;
  A[12][1]=(4.5149911816578479e-03)*D;
  A[12][2]=(5.6437389770723099e-04)*D;
  A[12][3]=(2.2574955908289240e-03)*D;
  A[12][4]=(-9.8765432098765434e-04)*D;
  A[12][5]=(6.7724867724867728e-03)*D;
  A[12][6]=(2.2574955908289240e-03)*D;
  A[12][7]=(-2.2574955908289240e-03)*D;
  A[12][8]=(4.5149911816578479e-03)*D;
  A[12][9]=(-1.1287477954144622e-02)*D;
  A[12][10]=(1.1287477954144622e-02)*D;
  A[12][11]=(-8.4656084656084662e-03)*D;
  A[12][12]=(2.2574955908289243e-02)*D;
  A[12][13]=(1.1287477954144622e-02)*D;
  A[12][14]=(1.4109347442680777e-03)*D;
  A[13][0]=(-9.8765432098765434e-04)*D;
  A[13][1]=(2.2574955908289240e-03)*D;
  A[13][2]=(5.6437389770723099e-04)*D;
  A[13][3]=(4.5149911816578479e-03)*D;
  A[13][4]=(0.0)*D;
  A[13][5]=(4.5149911816578479e-03)*D;
  A[13][6]=(-2.2574955908289240e-03)*D;
  A[13][7]=(2.2574955908289240e-03)*D;
  A[13][8]=(6.7724867724867728e-03)*D;
  A[13][9]=(-8.4656084656084662e-03)*D;
  A[13][10]=(1.1287477954144622e-02)*D;
  A[13][11]=(-1.1287477954144622e-02)*D;
  A[13][12]=(1.1287477954144622e-02)*D;
  A[13][13]=(2.2574955908289243e-02)*D;
  A[13][14]=(1.4109347442680777e-03)*D;
  A[14][0]=(-2.3809523809523810e-04)*D;
  A[14][1]=(-9.8765432098765434e-04)*D;
  A[14][2]=(-1.0582010582010582e-04)*D;
  A[14][3]=(-9.8765432098765434e-04)*D;
  A[14][4]=(-2.3809523809523810e-04)*D;
  A[14][5]=(0.0)*D;
  A[14][6]=(-1.4109347442680777e-03)*D;
  A[14][7]=(-1.4109347442680777e-03)*D;
  A[14][8]=(0.0)*D;
  A[14][9]=(-7.0546737213403885e-04)*D;
  A[14][10]=(1.4109347442680777e-03)*D;
  A[14][11]=(-7.0546737213403885e-04)*D;
  A[14][12]=(1.4109347442680777e-03)*D;
  A[14][13]=(1.4109347442680777e-03)*D;
  A[14][14]=(2.5573192239858908e-03)*D;
  int nbf = 15;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 5 ---------------*/


  
void matrix_factory_mass_2D_order5 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[21];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/5.0+x0;  dof1[1]=G10/5.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=(2.0/5.0)*G00+x0;  dof2[1]=(2.0/5.0)*G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=(3.0/5.0)*G00+x0;  dof3[1]=(3.0/5.0)*G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=(4.0/5.0)*G00+x0;  dof4[1]=(4.0/5.0)*G10+y0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G00+x0;  dof5[1]=G10+y0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/5.0+x0;  dof6[1]=y0+G11/5.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=G01/5.0+G00/5.0+x0;  dof7[1]=G10/5.0+y0+G11/5.0;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=G01/5.0+(2.0/5.0)*G00+x0;  dof8[1]=(2.0/5.0)*G10+y0+G11/5.0;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01/5.0+(3.0/5.0)*G00+x0;  dof9[1]=(3.0/5.0)*G10+y0+G11/5.0;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[2];
  dof10[0]=G01/5.0+(4.0/5.0)*G00+x0;  dof10[1]=(4.0/5.0)*G10+y0+G11/5.0;
  ptv pdof10(2,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[2];
  dof11[0]=(2.0/5.0)*G01+x0;  dof11[1]=y0+(2.0/5.0)*G11;
  ptv pdof11(2,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[2];
  dof12[0]=(2.0/5.0)*G01+G00/5.0+x0;  dof12[1]=G10/5.0+y0+(2.0/5.0)*G11;
  ptv pdof12(2,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[2];
  dof13[0]=(2.0/5.0)*G01+(2.0/5.0)*G00+x0;  dof13[1]=(2.0/5.0)*G10+y0+(2.0/5.0)*G11;
  ptv pdof13(2,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[2];
  dof14[0]=(2.0/5.0)*G01+(3.0/5.0)*G00+x0;  dof14[1]=(3.0/5.0)*G10+y0+(2.0/5.0)*G11;
  ptv pdof14(2,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

  double dof15[2];
  dof15[0]=(3.0/5.0)*G01+x0;  dof15[1]=y0+(3.0/5.0)*G11;
  ptv pdof15(2,dof15);
  iidof[15] = dof.insert_dof(element,15,pdof15);

  double dof16[2];
  dof16[0]=(3.0/5.0)*G01+G00/5.0+x0;  dof16[1]=G10/5.0+y0+(3.0/5.0)*G11;
  ptv pdof16(2,dof16);
  iidof[16] = dof.insert_dof(element,16,pdof16);

  double dof17[2];
  dof17[0]=(3.0/5.0)*G01+(2.0/5.0)*G00+x0;  dof17[1]=(2.0/5.0)*G10+y0+(3.0/5.0)*G11;
  ptv pdof17(2,dof17);
  iidof[17] = dof.insert_dof(element,17,pdof17);

  double dof18[2];
  dof18[0]=(4.0/5.0)*G01+x0;  dof18[1]=y0+(4.0/5.0)*G11;
  ptv pdof18(2,dof18);
  iidof[18] = dof.insert_dof(element,18,pdof18);

  double dof19[2];
  dof19[0]=(4.0/5.0)*G01+G00/5.0+x0;  dof19[1]=G10/5.0+y0+(4.0/5.0)*G11;
  ptv pdof19(2,dof19);
  iidof[19] = dof.insert_dof(element,19,pdof19);

  double dof20[2];
  dof20[0]=G01+x0;  dof20[1]=y0+G11;
  ptv pdof20(2,dof20);
  iidof[20] = dof.insert_dof(element,20,pdof20);

 
  double A[21][21];
  A[0][0]=(1.3894525613275613e-03)*D;
  A[0][1]=(1.1813634860509861e-03)*D;
  A[0][2]=(-9.5824314574314575e-04)*D;
  A[0][3]=(4.6268112674362677e-04)*D;
  A[0][4]=(-2.2964432686654910e-05)*D;
  A[0][5]=(1.2387745677676232e-04)*D;
  A[0][6]=(1.1813634860509861e-03)*D;
  A[0][7]=(8.9248136123136121e-04)*D;
  A[0][8]=(-9.7468359187109191e-04)*D;
  A[0][9]=(8.7943338811394365e-04)*D;
  A[0][10]=(3.6364701078242745e-04)*D;
  A[0][11]=(-9.5824314574314575e-04)*D;
  A[0][12]=(-9.7468359187109191e-04)*D;
  A[0][13]=(6.4978906124739453e-04)*D;
  A[0][14]=(2.0720181310459088e-04)*D;
  A[0][15]=(4.6268112674362677e-04)*D;
  A[0][16]=(8.7943338811394365e-04)*D;
  A[0][17]=(2.0720181310459088e-04)*D;
  A[0][18]=(-2.2964432686654910e-05)*D;
  A[0][19]=(3.6364701078242745e-04)*D;
  A[0][20]=(1.2387745677676232e-04)*D;
  A[1][0]=(1.1813634860509861e-03)*D;
  A[1][1]=(1.4705065703329592e-02)*D;
  A[1][2]=(-9.5837362547431994e-03)*D;
  A[1][3]=(7.7048281258350702e-03)*D;
  A[1][4]=(-3.9698458209742934e-03)*D;
  A[1][5]=(-2.2964432686654910e-05)*D;
  A[1][6]=(7.3525328516647961e-03)*D;
  A[1][7]=(1.0242658897172786e-02)*D;
  A[1][8]=(9.7859798380631720e-05)*D;
  A[1][9]=(-2.8053142202447757e-03)*D;
  A[1][10]=(-2.4823768855886910e-03)*D;
  A[1][11]=(-7.3525328516647961e-03)*D;
  A[1][12]=(-3.8165321368446368e-03)*D;
  A[1][13]=(-1.3700371773288440e-03)*D;
  A[1][14]=(-4.5667905910961469e-05)*D;
  A[1][15]=(5.7606801313398537e-03)*D;
  A[1][16]=(7.8287838704505376e-04)*D;
  A[1][17]=(-1.4483250160333494e-03)*D;
  A[1][18]=(-2.4823768855886910e-03)*D;
  A[1][19]=(-4.1101115319865323e-04)*D;
  A[1][20]=(3.6364701078242745e-04)*D;
  A[2][0]=(-9.5824314574314575e-04)*D;
  A[2][1]=(-9.5837362547431994e-03)*D;
  A[2][2]=(2.2077170514670515e-02)*D;
  A[2][3]=(-1.3478556230292342e-02)*D;
  A[2][4]=(7.7048281258350702e-03)*D;
  A[2][5]=(4.6268112674362677e-04)*D;
  A[2][6]=(-7.3525328516647961e-03)*D;
  A[2][7]=(-1.4352770429159319e-03)*D;
  A[2][8]=(5.2844291125541129e-03)*D;
  A[2][9]=(5.0887095157928488e-03)*D;
  A[2][10]=(5.7606801313398537e-03)*D;
  A[2][11]=(2.9488419245363691e-03)*D;
  A[2][12]=(-5.4801487093153762e-03)*D;
  A[2][13]=(-5.8715879028379029e-04)*D;
  A[2][14]=(-1.3569892042114265e-03)*D;
  A[2][15]=(-1.3569892042114265e-03)*D;
  A[2][16]=(3.1315135481802150e-03)*D;
  A[2][17]=(2.8183621933621935e-03)*D;
  A[2][18]=(-4.5667905910961469e-05)*D;
  A[2][19]=(-1.4483250160333494e-03)*D;
  A[2][20]=(2.0720181310459088e-04)*D;
  A[3][0]=(4.6268112674362677e-04)*D;
  A[3][1]=(7.7048281258350702e-03)*D;
  A[3][2]=(-1.3478556230292342e-02)*D;
  A[3][3]=(2.2077170514670515e-02)*D;
  A[3][4]=(-9.5837362547431994e-03)*D;
  A[3][5]=(-9.5824314574314575e-04)*D;
  A[3][6]=(5.7606801313398537e-03)*D;
  A[3][7]=(5.0887095157928488e-03)*D;
  A[3][8]=(5.2844291125541129e-03)*D;
  A[3][9]=(-1.4352770429159319e-03)*D;
  A[3][10]=(-7.3525328516647961e-03)*D;
  A[3][11]=(-1.3569892042114265e-03)*D;
  A[3][12]=(-5.8715879028379029e-04)*D;
  A[3][13]=(-5.4801487093153762e-03)*D;
  A[3][14]=(2.9488419245363691e-03)*D;
  A[3][15]=(2.8183621933621935e-03)*D;
  A[3][16]=(3.1315135481802150e-03)*D;
  A[3][17]=(-1.3569892042114265e-03)*D;
  A[3][18]=(-1.4483250160333494e-03)*D;
  A[3][19]=(-4.5667905910961469e-05)*D;
  A[3][20]=(2.0720181310459088e-04)*D;
  A[4][0]=(-2.2964432686654910e-05)*D;
  A[4][1]=(-3.9698458209742934e-03)*D;
  A[4][2]=(7.7048281258350702e-03)*D;
  A[4][3]=(-9.5837362547431994e-03)*D;
  A[4][4]=(1.4705065703329592e-02)*D;
  A[4][5]=(1.1813634860509861e-03)*D;
  A[4][6]=(-2.4823768855886910e-03)*D;
  A[4][7]=(-2.8053142202447757e-03)*D;
  A[4][8]=(9.7859798380631720e-05)*D;
  A[4][9]=(1.0242658897172786e-02)*D;
  A[4][10]=(7.3525328516647961e-03)*D;
  A[4][11]=(-4.5667905910961469e-05)*D;
  A[4][12]=(-1.3700371773288440e-03)*D;
  A[4][13]=(-3.8165321368446368e-03)*D;
  A[4][14]=(-7.3525328516647961e-03)*D;
  A[4][15]=(-1.4483250160333494e-03)*D;
  A[4][16]=(7.8287838704505376e-04)*D;
  A[4][17]=(5.7606801313398537e-03)*D;
  A[4][18]=(-4.1101115319865323e-04)*D;
  A[4][19]=(-2.4823768855886910e-03)*D;
  A[4][20]=(3.6364701078242745e-04)*D;
  A[5][0]=(1.2387745677676232e-04)*D;
  A[5][1]=(-2.2964432686654910e-05)*D;
  A[5][2]=(4.6268112674362677e-04)*D;
  A[5][3]=(-9.5824314574314575e-04)*D;
  A[5][4]=(1.1813634860509861e-03)*D;
  A[5][5]=(1.3894525613275613e-03)*D;
  A[5][6]=(3.6364701078242745e-04)*D;
  A[5][7]=(8.7943338811394365e-04)*D;
  A[5][8]=(-9.7468359187109191e-04)*D;
  A[5][9]=(8.9248136123136121e-04)*D;
  A[5][10]=(1.1813634860509861e-03)*D;
  A[5][11]=(2.0720181310459088e-04)*D;
  A[5][12]=(6.4978906124739453e-04)*D;
  A[5][13]=(-9.7468359187109191e-04)*D;
  A[5][14]=(-9.5824314574314575e-04)*D;
  A[5][15]=(2.0720181310459088e-04)*D;
  A[5][16]=(8.7943338811394365e-04)*D;
  A[5][17]=(4.6268112674362677e-04)*D;
  A[5][18]=(3.6364701078242745e-04)*D;
  A[5][19]=(-2.2964432686654910e-05)*D;
  A[5][20]=(1.2387745677676232e-04)*D;
  A[6][0]=(1.1813634860509861e-03)*D;
  A[6][1]=(7.3525328516647961e-03)*D;
  A[6][2]=(-7.3525328516647961e-03)*D;
  A[6][3]=(5.7606801313398537e-03)*D;
  A[6][4]=(-2.4823768855886910e-03)*D;
  A[6][5]=(3.6364701078242745e-04)*D;
  A[6][6]=(1.4705065703329592e-02)*D;
  A[6][7]=(1.0242658897172786e-02)*D;
  A[6][8]=(-3.8165321368446368e-03)*D;
  A[6][9]=(7.8287838704505376e-04)*D;
  A[6][10]=(-4.1101115319865323e-04)*D;
  A[6][11]=(-9.5837362547431994e-03)*D;
  A[6][12]=(9.7859798380631720e-05)*D;
  A[6][13]=(-1.3700371773288440e-03)*D;
  A[6][14]=(-1.4483250160333494e-03)*D;
  A[6][15]=(7.7048281258350702e-03)*D;
  A[6][16]=(-2.8053142202447757e-03)*D;
  A[6][17]=(-4.5667905910961469e-05)*D;
  A[6][18]=(-3.9698458209742934e-03)*D;
  A[6][19]=(-2.4823768855886910e-03)*D;
  A[6][20]=(-2.2964432686654910e-05)*D;
  A[7][0]=(8.9248136123136121e-04)*D;
  A[7][1]=(1.0242658897172786e-02)*D;
  A[7][2]=(-1.4352770429159319e-03)*D;
  A[7][3]=(5.0887095157928488e-03)*D;
  A[7][4]=(-2.8053142202447757e-03)*D;
  A[7][5]=(8.7943338811394365e-04)*D;
  A[7][6]=(1.0242658897172786e-02)*D;
  A[7][7]=(7.0459054834054832e-02)*D;
  A[7][8]=(-1.5657567740901074e-02)*D;
  A[7][9]=(1.1743175805675805e-02)*D;
  A[7][10]=(7.8287838704505376e-04)*D;
  A[7][11]=(-1.4352770429159319e-03)*D;
  A[7][12]=(-1.5657567740901074e-02)*D;
  A[7][13]=(3.9143919352252685e-03)*D;
  A[7][14]=(3.1315135481802150e-03)*D;
  A[7][15]=(5.0887095157928488e-03)*D;
  A[7][16]=(1.1743175805675805e-02)*D;
  A[7][17]=(3.1315135481802150e-03)*D;
  A[7][18]=(-2.8053142202447757e-03)*D;
  A[7][19]=(7.8287838704505376e-04)*D;
  A[7][20]=(8.7943338811394365e-04)*D;
  A[8][0]=(-9.7468359187109191e-04)*D;
  A[8][1]=(9.7859798380631720e-05)*D;
  A[8][2]=(5.2844291125541129e-03)*D;
  A[8][3]=(5.2844291125541129e-03)*D;
  A[8][4]=(9.7859798380631720e-05)*D;
  A[8][5]=(-9.7468359187109191e-04)*D;
  A[8][6]=(-3.8165321368446368e-03)*D;
  A[8][7]=(-1.5657567740901074e-02)*D;
  A[8][8]=(7.6330642736892731e-02)*D;
  A[8][9]=(-1.5657567740901074e-02)*D;
  A[8][10]=(-3.8165321368446368e-03)*D;
  A[8][11]=(-5.4801487093153762e-03)*D;
  A[8][12]=(-1.1743175805675805e-02)*D;
  A[8][13]=(-1.1743175805675805e-02)*D;
  A[8][14]=(-5.4801487093153762e-03)*D;
  A[8][15]=(-5.8715879028379029e-04)*D;
  A[8][16]=(3.9143919352252685e-03)*D;
  A[8][17]=(-5.8715879028379029e-04)*D;
  A[8][18]=(-1.3700371773288440e-03)*D;
  A[8][19]=(-1.3700371773288440e-03)*D;
  A[8][20]=(6.4978906124739453e-04)*D;
  A[9][0]=(8.7943338811394365e-04)*D;
  A[9][1]=(-2.8053142202447757e-03)*D;
  A[9][2]=(5.0887095157928488e-03)*D;
  A[9][3]=(-1.4352770429159319e-03)*D;
  A[9][4]=(1.0242658897172786e-02)*D;
  A[9][5]=(8.9248136123136121e-04)*D;
  A[9][6]=(7.8287838704505376e-04)*D;
  A[9][7]=(1.1743175805675805e-02)*D;
  A[9][8]=(-1.5657567740901074e-02)*D;
  A[9][9]=(7.0459054834054832e-02)*D;
  A[9][10]=(1.0242658897172786e-02)*D;
  A[9][11]=(3.1315135481802150e-03)*D;
  A[9][12]=(3.9143919352252685e-03)*D;
  A[9][13]=(-1.5657567740901074e-02)*D;
  A[9][14]=(-1.4352770429159319e-03)*D;
  A[9][15]=(3.1315135481802150e-03)*D;
  A[9][16]=(1.1743175805675805e-02)*D;
  A[9][17]=(5.0887095157928488e-03)*D;
  A[9][18]=(7.8287838704505376e-04)*D;
  A[9][19]=(-2.8053142202447757e-03)*D;
  A[9][20]=(8.7943338811394365e-04)*D;
  A[10][0]=(3.6364701078242745e-04)*D;
  A[10][1]=(-2.4823768855886910e-03)*D;
  A[10][2]=(5.7606801313398537e-03)*D;
  A[10][3]=(-7.3525328516647961e-03)*D;
  A[10][4]=(7.3525328516647961e-03)*D;
  A[10][5]=(1.1813634860509861e-03)*D;
  A[10][6]=(-4.1101115319865323e-04)*D;
  A[10][7]=(7.8287838704505376e-04)*D;
  A[10][8]=(-3.8165321368446368e-03)*D;
  A[10][9]=(1.0242658897172786e-02)*D;
  A[10][10]=(1.4705065703329592e-02)*D;
  A[10][11]=(-1.4483250160333494e-03)*D;
  A[10][12]=(-1.3700371773288440e-03)*D;
  A[10][13]=(9.7859798380631720e-05)*D;
  A[10][14]=(-9.5837362547431994e-03)*D;
  A[10][15]=(-4.5667905910961469e-05)*D;
  A[10][16]=(-2.8053142202447757e-03)*D;
  A[10][17]=(7.7048281258350702e-03)*D;
  A[10][18]=(-2.4823768855886910e-03)*D;
  A[10][19]=(-3.9698458209742934e-03)*D;
  A[10][20]=(-2.2964432686654910e-05)*D;
  A[11][0]=(-9.5824314574314575e-04)*D;
  A[11][1]=(-7.3525328516647961e-03)*D;
  A[11][2]=(2.9488419245363691e-03)*D;
  A[11][3]=(-1.3569892042114265e-03)*D;
  A[11][4]=(-4.5667905910961469e-05)*D;
  A[11][5]=(2.0720181310459088e-04)*D;
  A[11][6]=(-9.5837362547431994e-03)*D;
  A[11][7]=(-1.4352770429159319e-03)*D;
  A[11][8]=(-5.4801487093153762e-03)*D;
  A[11][9]=(3.1315135481802150e-03)*D;
  A[11][10]=(-1.4483250160333494e-03)*D;
  A[11][11]=(2.2077170514670515e-02)*D;
  A[11][12]=(5.2844291125541129e-03)*D;
  A[11][13]=(-5.8715879028379029e-04)*D;
  A[11][14]=(2.8183621933621935e-03)*D;
  A[11][15]=(-1.3478556230292342e-02)*D;
  A[11][16]=(5.0887095157928488e-03)*D;
  A[11][17]=(-1.3569892042114265e-03)*D;
  A[11][18]=(7.7048281258350702e-03)*D;
  A[11][19]=(5.7606801313398537e-03)*D;
  A[11][20]=(4.6268112674362677e-04)*D;
  A[12][0]=(-9.7468359187109191e-04)*D;
  A[12][1]=(-3.8165321368446368e-03)*D;
  A[12][2]=(-5.4801487093153762e-03)*D;
  A[12][3]=(-5.8715879028379029e-04)*D;
  A[12][4]=(-1.3700371773288440e-03)*D;
  A[12][5]=(6.4978906124739453e-04)*D;
  A[12][6]=(9.7859798380631720e-05)*D;
  A[12][7]=(-1.5657567740901074e-02)*D;
  A[12][8]=(-1.1743175805675805e-02)*D;
  A[12][9]=(3.9143919352252685e-03)*D;
  A[12][10]=(-1.3700371773288440e-03)*D;
  A[12][11]=(5.2844291125541129e-03)*D;
  A[12][12]=(7.6330642736892731e-02)*D;
  A[12][13]=(-1.1743175805675805e-02)*D;
  A[12][14]=(-5.8715879028379029e-04)*D;
  A[12][15]=(5.2844291125541129e-03)*D;
  A[12][16]=(-1.5657567740901074e-02)*D;
  A[12][17]=(-5.4801487093153762e-03)*D;
  A[12][18]=(9.7859798380631720e-05)*D;
  A[12][19]=(-3.8165321368446368e-03)*D;
  A[12][20]=(-9.7468359187109191e-04)*D;
  A[13][0]=(6.4978906124739453e-04)*D;
  A[13][1]=(-1.3700371773288440e-03)*D;
  A[13][2]=(-5.8715879028379029e-04)*D;
  A[13][3]=(-5.4801487093153762e-03)*D;
  A[13][4]=(-3.8165321368446368e-03)*D;
  A[13][5]=(-9.7468359187109191e-04)*D;
  A[13][6]=(-1.3700371773288440e-03)*D;
  A[13][7]=(3.9143919352252685e-03)*D;
  A[13][8]=(-1.1743175805675805e-02)*D;
  A[13][9]=(-1.5657567740901074e-02)*D;
  A[13][10]=(9.7859798380631720e-05)*D;
  A[13][11]=(-5.8715879028379029e-04)*D;
  A[13][12]=(-1.1743175805675805e-02)*D;
  A[13][13]=(7.6330642736892731e-02)*D;
  A[13][14]=(5.2844291125541129e-03)*D;
  A[13][15]=(-5.4801487093153762e-03)*D;
  A[13][16]=(-1.5657567740901074e-02)*D;
  A[13][17]=(5.2844291125541129e-03)*D;
  A[13][18]=(-3.8165321368446368e-03)*D;
  A[13][19]=(9.7859798380631720e-05)*D;
  A[13][20]=(-9.7468359187109191e-04)*D;
  A[14][0]=(2.0720181310459088e-04)*D;
  A[14][1]=(-4.5667905910961469e-05)*D;
  A[14][2]=(-1.3569892042114265e-03)*D;
  A[14][3]=(2.9488419245363691e-03)*D;
  A[14][4]=(-7.3525328516647961e-03)*D;
  A[14][5]=(-9.5824314574314575e-04)*D;
  A[14][6]=(-1.4483250160333494e-03)*D;
  A[14][7]=(3.1315135481802150e-03)*D;
  A[14][8]=(-5.4801487093153762e-03)*D;
  A[14][9]=(-1.4352770429159319e-03)*D;
  A[14][10]=(-9.5837362547431994e-03)*D;
  A[14][11]=(2.8183621933621935e-03)*D;
  A[14][12]=(-5.8715879028379029e-04)*D;
  A[14][13]=(5.2844291125541129e-03)*D;
  A[14][14]=(2.2077170514670515e-02)*D;
  A[14][15]=(-1.3569892042114265e-03)*D;
  A[14][16]=(5.0887095157928488e-03)*D;
  A[14][17]=(-1.3478556230292342e-02)*D;
  A[14][18]=(5.7606801313398537e-03)*D;
  A[14][19]=(7.7048281258350702e-03)*D;
  A[14][20]=(4.6268112674362677e-04)*D;
  A[15][0]=(4.6268112674362677e-04)*D;
  A[15][1]=(5.7606801313398537e-03)*D;
  A[15][2]=(-1.3569892042114265e-03)*D;
  A[15][3]=(2.8183621933621935e-03)*D;
  A[15][4]=(-1.4483250160333494e-03)*D;
  A[15][5]=(2.0720181310459088e-04)*D;
  A[15][6]=(7.7048281258350702e-03)*D;
  A[15][7]=(5.0887095157928488e-03)*D;
  A[15][8]=(-5.8715879028379029e-04)*D;
  A[15][9]=(3.1315135481802150e-03)*D;
  A[15][10]=(-4.5667905910961469e-05)*D;
  A[15][11]=(-1.3478556230292342e-02)*D;
  A[15][12]=(5.2844291125541129e-03)*D;
  A[15][13]=(-5.4801487093153762e-03)*D;
  A[15][14]=(-1.3569892042114265e-03)*D;
  A[15][15]=(2.2077170514670515e-02)*D;
  A[15][16]=(-1.4352770429159319e-03)*D;
  A[15][17]=(2.9488419245363691e-03)*D;
  A[15][18]=(-9.5837362547431994e-03)*D;
  A[15][19]=(-7.3525328516647961e-03)*D;
  A[15][20]=(-9.5824314574314575e-04)*D;
  A[16][0]=(8.7943338811394365e-04)*D;
  A[16][1]=(7.8287838704505376e-04)*D;
  A[16][2]=(3.1315135481802150e-03)*D;
  A[16][3]=(3.1315135481802150e-03)*D;
  A[16][4]=(7.8287838704505376e-04)*D;
  A[16][5]=(8.7943338811394365e-04)*D;
  A[16][6]=(-2.8053142202447757e-03)*D;
  A[16][7]=(1.1743175805675805e-02)*D;
  A[16][8]=(3.9143919352252685e-03)*D;
  A[16][9]=(1.1743175805675805e-02)*D;
  A[16][10]=(-2.8053142202447757e-03)*D;
  A[16][11]=(5.0887095157928488e-03)*D;
  A[16][12]=(-1.5657567740901074e-02)*D;
  A[16][13]=(-1.5657567740901074e-02)*D;
  A[16][14]=(5.0887095157928488e-03)*D;
  A[16][15]=(-1.4352770429159319e-03)*D;
  A[16][16]=(7.0459054834054832e-02)*D;
  A[16][17]=(-1.4352770429159319e-03)*D;
  A[16][18]=(1.0242658897172786e-02)*D;
  A[16][19]=(1.0242658897172786e-02)*D;
  A[16][20]=(8.9248136123136121e-04)*D;
  A[17][0]=(2.0720181310459088e-04)*D;
  A[17][1]=(-1.4483250160333494e-03)*D;
  A[17][2]=(2.8183621933621935e-03)*D;
  A[17][3]=(-1.3569892042114265e-03)*D;
  A[17][4]=(5.7606801313398537e-03)*D;
  A[17][5]=(4.6268112674362677e-04)*D;
  A[17][6]=(-4.5667905910961469e-05)*D;
  A[17][7]=(3.1315135481802150e-03)*D;
  A[17][8]=(-5.8715879028379029e-04)*D;
  A[17][9]=(5.0887095157928488e-03)*D;
  A[17][10]=(7.7048281258350702e-03)*D;
  A[17][11]=(-1.3569892042114265e-03)*D;
  A[17][12]=(-5.4801487093153762e-03)*D;
  A[17][13]=(5.2844291125541129e-03)*D;
  A[17][14]=(-1.3478556230292342e-02)*D;
  A[17][15]=(2.9488419245363691e-03)*D;
  A[17][16]=(-1.4352770429159319e-03)*D;
  A[17][17]=(2.2077170514670515e-02)*D;
  A[17][18]=(-7.3525328516647961e-03)*D;
  A[17][19]=(-9.5837362547431994e-03)*D;
  A[17][20]=(-9.5824314574314575e-04)*D;
  A[18][0]=(-2.2964432686654910e-05)*D;
  A[18][1]=(-2.4823768855886910e-03)*D;
  A[18][2]=(-4.5667905910961469e-05)*D;
  A[18][3]=(-1.4483250160333494e-03)*D;
  A[18][4]=(-4.1101115319865323e-04)*D;
  A[18][5]=(3.6364701078242745e-04)*D;
  A[18][6]=(-3.9698458209742934e-03)*D;
  A[18][7]=(-2.8053142202447757e-03)*D;
  A[18][8]=(-1.3700371773288440e-03)*D;
  A[18][9]=(7.8287838704505376e-04)*D;
  A[18][10]=(-2.4823768855886910e-03)*D;
  A[18][11]=(7.7048281258350702e-03)*D;
  A[18][12]=(9.7859798380631720e-05)*D;
  A[18][13]=(-3.8165321368446368e-03)*D;
  A[18][14]=(5.7606801313398537e-03)*D;
  A[18][15]=(-9.5837362547431994e-03)*D;
  A[18][16]=(1.0242658897172786e-02)*D;
  A[18][17]=(-7.3525328516647961e-03)*D;
  A[18][18]=(1.4705065703329592e-02)*D;
  A[18][19]=(7.3525328516647961e-03)*D;
  A[18][20]=(1.1813634860509861e-03)*D;
  A[19][0]=(3.6364701078242745e-04)*D;
  A[19][1]=(-4.1101115319865323e-04)*D;
  A[19][2]=(-1.4483250160333494e-03)*D;
  A[19][3]=(-4.5667905910961469e-05)*D;
  A[19][4]=(-2.4823768855886910e-03)*D;
  A[19][5]=(-2.2964432686654910e-05)*D;
  A[19][6]=(-2.4823768855886910e-03)*D;
  A[19][7]=(7.8287838704505376e-04)*D;
  A[19][8]=(-1.3700371773288440e-03)*D;
  A[19][9]=(-2.8053142202447757e-03)*D;
  A[19][10]=(-3.9698458209742934e-03)*D;
  A[19][11]=(5.7606801313398537e-03)*D;
  A[19][12]=(-3.8165321368446368e-03)*D;
  A[19][13]=(9.7859798380631720e-05)*D;
  A[19][14]=(7.7048281258350702e-03)*D;
  A[19][15]=(-7.3525328516647961e-03)*D;
  A[19][16]=(1.0242658897172786e-02)*D;
  A[19][17]=(-9.5837362547431994e-03)*D;
  A[19][18]=(7.3525328516647961e-03)*D;
  A[19][19]=(1.4705065703329592e-02)*D;
  A[19][20]=(1.1813634860509861e-03)*D;
  A[20][0]=(1.2387745677676232e-04)*D;
  A[20][1]=(3.6364701078242745e-04)*D;
  A[20][2]=(2.0720181310459088e-04)*D;
  A[20][3]=(2.0720181310459088e-04)*D;
  A[20][4]=(3.6364701078242745e-04)*D;
  A[20][5]=(1.2387745677676232e-04)*D;
  A[20][6]=(-2.2964432686654910e-05)*D;
  A[20][7]=(8.7943338811394365e-04)*D;
  A[20][8]=(6.4978906124739453e-04)*D;
  A[20][9]=(8.7943338811394365e-04)*D;
  A[20][10]=(-2.2964432686654910e-05)*D;
  A[20][11]=(4.6268112674362677e-04)*D;
  A[20][12]=(-9.7468359187109191e-04)*D;
  A[20][13]=(-9.7468359187109191e-04)*D;
  A[20][14]=(4.6268112674362677e-04)*D;
  A[20][15]=(-9.5824314574314575e-04)*D;
  A[20][16]=(8.9248136123136121e-04)*D;
  A[20][17]=(-9.5824314574314575e-04)*D;
  A[20][18]=(1.1813634860509861e-03)*D;
  A[20][19]=(1.1813634860509861e-03)*D;
  A[20][20]=(1.3894525613275613e-03)*D;
  int nbf = 21;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 1 ---------------*/


  
void matrix_factory_mass_2D_with_c_order1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2], double c){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[3];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00+x0;  dof1[1]=G10+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G01+x0;  dof2[1]=y0+G11;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

 
  double A[3][3];
  A[0][0]=(8.3333333333333329e-02*c)*D;
  A[0][1]=(4.1666666666666664e-02*c)*D;
  A[0][2]=(4.1666666666666664e-02*c)*D;
  A[1][0]=(4.1666666666666664e-02*c)*D;
  A[1][1]=(8.3333333333333329e-02*c)*D;
  A[1][2]=(4.1666666666666664e-02*c)*D;
  A[2][0]=(4.1666666666666664e-02*c)*D;
  A[2][1]=(4.1666666666666664e-02*c)*D;
  A[2][2]=(8.3333333333333329e-02*c)*D;
  int nbf = 3;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 ---------------*/


  
void matrix_factory_mass_2D_with_c_order2 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2], double c){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[6];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/2.0+x0;  dof1[1]=G10/2.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G00+x0;  dof2[1]=G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=G01/2.0+x0;  dof3[1]=y0+G11/2.0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G01/2.0+G00/2.0+x0;  dof4[1]=G10/2.0+y0+G11/2.0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01+x0;  dof5[1]=y0+G11;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

 
  double A[6][6];
  A[0][0]=(1.6666666666666666e-02*c)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(-2.7777777777777779e-03*c)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(-1.1111111111111112e-02*c)*D;
  A[0][5]=(-2.7777777777777779e-03*c)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(8.8888888888888892e-02*c)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(4.4444444444444446e-02*c)*D;
  A[1][4]=(4.4444444444444446e-02*c)*D;
  A[1][5]=(-1.1111111111111112e-02*c)*D;
  A[2][0]=(-2.7777777777777779e-03*c)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(1.6666666666666666e-02*c)*D;
  A[2][3]=(-1.1111111111111112e-02*c)*D;
  A[2][4]=(0.0)*D;
  A[2][5]=(-2.7777777777777779e-03*c)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(4.4444444444444446e-02*c)*D;
  A[3][2]=(-1.1111111111111112e-02*c)*D;
  A[3][3]=(8.8888888888888892e-02*c)*D;
  A[3][4]=(4.4444444444444446e-02*c)*D;
  A[3][5]=(0.0)*D;
  A[4][0]=(-1.1111111111111112e-02*c)*D;
  A[4][1]=(4.4444444444444446e-02*c)*D;
  A[4][2]=(0.0)*D;
  A[4][3]=(4.4444444444444446e-02*c)*D;
  A[4][4]=(8.8888888888888892e-02*c)*D;
  A[4][5]=(0.0)*D;
  A[5][0]=(-2.7777777777777779e-03*c)*D;
  A[5][1]=(-1.1111111111111112e-02*c)*D;
  A[5][2]=(-2.7777777777777779e-03*c)*D;
  A[5][3]=(0.0)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(1.6666666666666666e-02*c)*D;
  int nbf = 6;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 ---------------*/


  
void matrix_factory_mass_2D_with_c_order3 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2], double c){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[10];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/3.0+x0;  dof1[1]=G10/3.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=(2.0/3.0)*G00+x0;  dof2[1]=(2.0/3.0)*G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=G00+x0;  dof3[1]=G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G01/3.0+x0;  dof4[1]=y0+G11/3.0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01/3.0+G00/3.0+x0;  dof5[1]=G10/3.0+y0+G11/3.0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/3.0+(2.0/3.0)*G00+x0;  dof6[1]=(2.0/3.0)*G10+y0+G11/3.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=(2.0/3.0)*G01+x0;  dof7[1]=y0+(2.0/3.0)*G11;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=(2.0/3.0)*G01+G00/3.0+x0;  dof8[1]=G10/3.0+y0+(2.0/3.0)*G11;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01+x0;  dof9[1]=y0+G11;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

 
  double A[10][10];
  A[0][0]=(5.6547619047619046e-03*c)*D;
  A[0][1]=(1.3392857142857143e-03*c)*D;
  A[0][2]=(0.0)*D;
  A[0][3]=(8.1845238095238097e-04*c)*D;
  A[0][4]=(1.3392857142857143e-03*c)*D;
  A[0][5]=(2.6785714285714286e-03*c)*D;
  A[0][6]=(2.0089285714285712e-03*c)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(2.0089285714285712e-03*c)*D;
  A[0][9]=(8.1845238095238097e-04*c)*D;
  A[1][0]=(1.3392857142857143e-03*c)*D;
  A[1][1]=(4.0178571428571432e-02*c)*D;
  A[1][2]=(-1.4062500000000000e-02*c)*D;
  A[1][3]=(0.0)*D;
  A[1][4]=(2.0089285714285716e-02*c)*D;
  A[1][5]=(1.2053571428571429e-02*c)*D;
  A[1][6]=(-1.0044642857142858e-02*c)*D;
  A[1][7]=(-1.0044642857142858e-02*c)*D;
  A[1][8]=(-4.0178571428571425e-03*c)*D;
  A[1][9]=(2.0089285714285712e-03*c)*D;
  A[2][0]=(0.0)*D;
  A[2][1]=(-1.4062500000000000e-02*c)*D;
  A[2][2]=(4.0178571428571432e-02*c)*D;
  A[2][3]=(1.3392857142857143e-03*c)*D;
  A[2][4]=(-1.0044642857142858e-02*c)*D;
  A[2][5]=(1.2053571428571429e-02*c)*D;
  A[2][6]=(2.0089285714285716e-02*c)*D;
  A[2][7]=(-4.0178571428571425e-03*c)*D;
  A[2][8]=(-1.0044642857142858e-02*c)*D;
  A[2][9]=(2.0089285714285712e-03*c)*D;
  A[3][0]=(8.1845238095238097e-04*c)*D;
  A[3][1]=(0.0)*D;
  A[3][2]=(1.3392857142857143e-03*c)*D;
  A[3][3]=(5.6547619047619046e-03*c)*D;
  A[3][4]=(2.0089285714285712e-03*c)*D;
  A[3][5]=(2.6785714285714286e-03*c)*D;
  A[3][6]=(1.3392857142857143e-03*c)*D;
  A[3][7]=(2.0089285714285712e-03*c)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(8.1845238095238097e-04*c)*D;
  A[4][0]=(1.3392857142857143e-03*c)*D;
  A[4][1]=(2.0089285714285716e-02*c)*D;
  A[4][2]=(-1.0044642857142858e-02*c)*D;
  A[4][3]=(2.0089285714285712e-03*c)*D;
  A[4][4]=(4.0178571428571432e-02*c)*D;
  A[4][5]=(1.2053571428571429e-02*c)*D;
  A[4][6]=(-4.0178571428571425e-03*c)*D;
  A[4][7]=(-1.4062500000000000e-02*c)*D;
  A[4][8]=(-1.0044642857142858e-02*c)*D;
  A[4][9]=(0.0)*D;
  A[5][0]=(2.6785714285714286e-03*c)*D;
  A[5][1]=(1.2053571428571429e-02*c)*D;
  A[5][2]=(1.2053571428571429e-02*c)*D;
  A[5][3]=(2.6785714285714286e-03*c)*D;
  A[5][4]=(1.2053571428571429e-02*c)*D;
  A[5][5]=(1.4464285714285716e-01*c)*D;
  A[5][6]=(1.2053571428571429e-02*c)*D;
  A[5][7]=(1.2053571428571429e-02*c)*D;
  A[5][8]=(1.2053571428571429e-02*c)*D;
  A[5][9]=(2.6785714285714286e-03*c)*D;
  A[6][0]=(2.0089285714285712e-03*c)*D;
  A[6][1]=(-1.0044642857142858e-02*c)*D;
  A[6][2]=(2.0089285714285716e-02*c)*D;
  A[6][3]=(1.3392857142857143e-03*c)*D;
  A[6][4]=(-4.0178571428571425e-03*c)*D;
  A[6][5]=(1.2053571428571429e-02*c)*D;
  A[6][6]=(4.0178571428571432e-02*c)*D;
  A[6][7]=(-1.0044642857142858e-02*c)*D;
  A[6][8]=(-1.4062500000000000e-02*c)*D;
  A[6][9]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(-1.0044642857142858e-02*c)*D;
  A[7][2]=(-4.0178571428571425e-03*c)*D;
  A[7][3]=(2.0089285714285712e-03*c)*D;
  A[7][4]=(-1.4062500000000000e-02*c)*D;
  A[7][5]=(1.2053571428571429e-02*c)*D;
  A[7][6]=(-1.0044642857142858e-02*c)*D;
  A[7][7]=(4.0178571428571432e-02*c)*D;
  A[7][8]=(2.0089285714285716e-02*c)*D;
  A[7][9]=(1.3392857142857143e-03*c)*D;
  A[8][0]=(2.0089285714285712e-03*c)*D;
  A[8][1]=(-4.0178571428571425e-03*c)*D;
  A[8][2]=(-1.0044642857142858e-02*c)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(-1.0044642857142858e-02*c)*D;
  A[8][5]=(1.2053571428571429e-02*c)*D;
  A[8][6]=(-1.4062500000000000e-02*c)*D;
  A[8][7]=(2.0089285714285716e-02*c)*D;
  A[8][8]=(4.0178571428571432e-02*c)*D;
  A[8][9]=(1.3392857142857143e-03*c)*D;
  A[9][0]=(8.1845238095238097e-04*c)*D;
  A[9][1]=(2.0089285714285712e-03*c)*D;
  A[9][2]=(2.0089285714285712e-03*c)*D;
  A[9][3]=(8.1845238095238097e-04*c)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(2.6785714285714286e-03*c)*D;
  A[9][6]=(0.0)*D;
  A[9][7]=(1.3392857142857143e-03*c)*D;
  A[9][8]=(1.3392857142857143e-03*c)*D;
  A[9][9]=(5.6547619047619046e-03*c)*D;
  int nbf = 10;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 4 ---------------*/


  
void matrix_factory_mass_2D_with_c_order4 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2], double c){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[15];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/4.0+x0;  dof1[1]=G10/4.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G00/2.0+x0;  dof2[1]=G10/2.0+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=(3.0/4.0)*G00+x0;  dof3[1]=(3.0/4.0)*G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G00+x0;  dof4[1]=G10+y0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01/4.0+x0;  dof5[1]=y0+G11/4.0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/4.0+G00/4.0+x0;  dof6[1]=G10/4.0+y0+G11/4.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=G01/4.0+G00/2.0+x0;  dof7[1]=G10/2.0+y0+G11/4.0;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=G01/4.0+(3.0/4.0)*G00+x0;  dof8[1]=(3.0/4.0)*G10+y0+G11/4.0;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01/2.0+x0;  dof9[1]=y0+G11/2.0;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[2];
  dof10[0]=G01/2.0+G00/4.0+x0;  dof10[1]=G10/4.0+y0+G11/2.0;
  ptv pdof10(2,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[2];
  dof11[0]=G01/2.0+G00/2.0+x0;  dof11[1]=G10/2.0+y0+G11/2.0;
  ptv pdof11(2,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[2];
  dof12[0]=(3.0/4.0)*G01+x0;  dof12[1]=y0+(3.0/4.0)*G11;
  ptv pdof12(2,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[2];
  dof13[0]=(3.0/4.0)*G01+G00/4.0+x0;  dof13[1]=G10/4.0+y0+(3.0/4.0)*G11;
  ptv pdof13(2,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[2];
  dof14[0]=G01+x0;  dof14[1]=y0+G11;
  ptv pdof14(2,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

 
  double A[15][15];
  A[0][0]=(2.5573192239858908e-03*c)*D;
  A[0][1]=(1.4109347442680777e-03*c)*D;
  A[0][2]=(-7.0546737213403885e-04*c)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(-2.3809523809523810e-04*c)*D;
  A[0][5]=(1.4109347442680777e-03*c)*D;
  A[0][6]=(1.4109347442680777e-03*c)*D;
  A[0][7]=(-1.4109347442680777e-03*c)*D;
  A[0][8]=(-9.8765432098765434e-04*c)*D;
  A[0][9]=(-7.0546737213403885e-04*c)*D;
  A[0][10]=(-1.4109347442680777e-03*c)*D;
  A[0][11]=(-1.0582010582010582e-04*c)*D;
  A[0][12]=(0.0)*D;
  A[0][13]=(-9.8765432098765434e-04*c)*D;
  A[0][14]=(-2.3809523809523810e-04*c)*D;
  A[1][0]=(1.4109347442680777e-03*c)*D;
  A[1][1]=(2.2574955908289243e-02*c)*D;
  A[1][2]=(-1.1287477954144622e-02*c)*D;
  A[1][3]=(6.7724867724867728e-03*c)*D;
  A[1][4]=(0.0)*D;
  A[1][5]=(1.1287477954144622e-02*c)*D;
  A[1][6]=(1.1287477954144622e-02*c)*D;
  A[1][7]=(2.2574955908289240e-03*c)*D;
  A[1][8]=(4.5149911816578479e-03*c)*D;
  A[1][9]=(-8.4656084656084662e-03*c)*D;
  A[1][10]=(-2.2574955908289240e-03*c)*D;
  A[1][11]=(5.6437389770723099e-04*c)*D;
  A[1][12]=(4.5149911816578479e-03*c)*D;
  A[1][13]=(2.2574955908289240e-03*c)*D;
  A[1][14]=(-9.8765432098765434e-04*c)*D;
  A[2][0]=(-7.0546737213403885e-04*c)*D;
  A[2][1]=(-1.1287477954144622e-02*c)*D;
  A[2][2]=(2.7936507936507936e-02*c)*D;
  A[2][3]=(-1.1287477954144622e-02*c)*D;
  A[2][4]=(-7.0546737213403885e-04*c)*D;
  A[2][5]=(-8.4656084656084662e-03*c)*D;
  A[2][6]=(3.3862433862433864e-03*c)*D;
  A[2][7]=(3.3862433862433864e-03*c)*D;
  A[2][8]=(-8.4656084656084662e-03*c)*D;
  A[2][9]=(4.2328042328042330e-04*c)*D;
  A[2][10]=(-6.7724867724867728e-03*c)*D;
  A[2][11]=(4.2328042328042330e-04*c)*D;
  A[2][12]=(5.6437389770723099e-04*c)*D;
  A[2][13]=(5.6437389770723099e-04*c)*D;
  A[2][14]=(-1.0582010582010582e-04*c)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(6.7724867724867728e-03*c)*D;
  A[3][2]=(-1.1287477954144622e-02*c)*D;
  A[3][3]=(2.2574955908289243e-02*c)*D;
  A[3][4]=(1.4109347442680777e-03*c)*D;
  A[3][5]=(4.5149911816578479e-03*c)*D;
  A[3][6]=(2.2574955908289240e-03*c)*D;
  A[3][7]=(1.1287477954144622e-02*c)*D;
  A[3][8]=(1.1287477954144622e-02*c)*D;
  A[3][9]=(5.6437389770723099e-04*c)*D;
  A[3][10]=(-2.2574955908289240e-03*c)*D;
  A[3][11]=(-8.4656084656084662e-03*c)*D;
  A[3][12]=(2.2574955908289240e-03*c)*D;
  A[3][13]=(4.5149911816578479e-03*c)*D;
  A[3][14]=(-9.8765432098765434e-04*c)*D;
  A[4][0]=(-2.3809523809523810e-04*c)*D;
  A[4][1]=(0.0)*D;
  A[4][2]=(-7.0546737213403885e-04*c)*D;
  A[4][3]=(1.4109347442680777e-03*c)*D;
  A[4][4]=(2.5573192239858908e-03*c)*D;
  A[4][5]=(-9.8765432098765434e-04*c)*D;
  A[4][6]=(-1.4109347442680777e-03*c)*D;
  A[4][7]=(1.4109347442680777e-03*c)*D;
  A[4][8]=(1.4109347442680777e-03*c)*D;
  A[4][9]=(-1.0582010582010582e-04*c)*D;
  A[4][10]=(-1.4109347442680777e-03*c)*D;
  A[4][11]=(-7.0546737213403885e-04*c)*D;
  A[4][12]=(-9.8765432098765434e-04*c)*D;
  A[4][13]=(0.0)*D;
  A[4][14]=(-2.3809523809523810e-04*c)*D;
  A[5][0]=(1.4109347442680777e-03*c)*D;
  A[5][1]=(1.1287477954144622e-02*c)*D;
  A[5][2]=(-8.4656084656084662e-03*c)*D;
  A[5][3]=(4.5149911816578479e-03*c)*D;
  A[5][4]=(-9.8765432098765434e-04*c)*D;
  A[5][5]=(2.2574955908289243e-02*c)*D;
  A[5][6]=(1.1287477954144622e-02*c)*D;
  A[5][7]=(-2.2574955908289240e-03*c)*D;
  A[5][8]=(2.2574955908289240e-03*c)*D;
  A[5][9]=(-1.1287477954144622e-02*c)*D;
  A[5][10]=(2.2574955908289240e-03*c)*D;
  A[5][11]=(5.6437389770723099e-04*c)*D;
  A[5][12]=(6.7724867724867728e-03*c)*D;
  A[5][13]=(4.5149911816578479e-03*c)*D;
  A[5][14]=(0.0)*D;
  A[6][0]=(1.4109347442680777e-03*c)*D;
  A[6][1]=(1.1287477954144622e-02*c)*D;
  A[6][2]=(3.3862433862433864e-03*c)*D;
  A[6][3]=(2.2574955908289240e-03*c)*D;
  A[6][4]=(-1.4109347442680777e-03*c)*D;
  A[6][5]=(1.1287477954144622e-02*c)*D;
  A[6][6]=(9.4814814814814810e-02*c)*D;
  A[6][7]=(-1.3544973544973546e-02*c)*D;
  A[6][8]=(-2.2574955908289240e-03*c)*D;
  A[6][9]=(3.3862433862433864e-03*c)*D;
  A[6][10]=(-1.3544973544973546e-02*c)*D;
  A[6][11]=(-6.7724867724867728e-03*c)*D;
  A[6][12]=(2.2574955908289240e-03*c)*D;
  A[6][13]=(-2.2574955908289240e-03*c)*D;
  A[6][14]=(-1.4109347442680777e-03*c)*D;
  A[7][0]=(-1.4109347442680777e-03*c)*D;
  A[7][1]=(2.2574955908289240e-03*c)*D;
  A[7][2]=(3.3862433862433864e-03*c)*D;
  A[7][3]=(1.1287477954144622e-02*c)*D;
  A[7][4]=(1.4109347442680777e-03*c)*D;
  A[7][5]=(-2.2574955908289240e-03*c)*D;
  A[7][6]=(-1.3544973544973546e-02*c)*D;
  A[7][7]=(9.4814814814814810e-02*c)*D;
  A[7][8]=(1.1287477954144622e-02*c)*D;
  A[7][9]=(-6.7724867724867728e-03*c)*D;
  A[7][10]=(-1.3544973544973546e-02*c)*D;
  A[7][11]=(3.3862433862433864e-03*c)*D;
  A[7][12]=(-2.2574955908289240e-03*c)*D;
  A[7][13]=(2.2574955908289240e-03*c)*D;
  A[7][14]=(-1.4109347442680777e-03*c)*D;
  A[8][0]=(-9.8765432098765434e-04*c)*D;
  A[8][1]=(4.5149911816578479e-03*c)*D;
  A[8][2]=(-8.4656084656084662e-03*c)*D;
  A[8][3]=(1.1287477954144622e-02*c)*D;
  A[8][4]=(1.4109347442680777e-03*c)*D;
  A[8][5]=(2.2574955908289240e-03*c)*D;
  A[8][6]=(-2.2574955908289240e-03*c)*D;
  A[8][7]=(1.1287477954144622e-02*c)*D;
  A[8][8]=(2.2574955908289243e-02*c)*D;
  A[8][9]=(5.6437389770723099e-04*c)*D;
  A[8][10]=(2.2574955908289240e-03*c)*D;
  A[8][11]=(-1.1287477954144622e-02*c)*D;
  A[8][12]=(4.5149911816578479e-03*c)*D;
  A[8][13]=(6.7724867724867728e-03*c)*D;
  A[8][14]=(0.0)*D;
  A[9][0]=(-7.0546737213403885e-04*c)*D;
  A[9][1]=(-8.4656084656084662e-03*c)*D;
  A[9][2]=(4.2328042328042330e-04*c)*D;
  A[9][3]=(5.6437389770723099e-04*c)*D;
  A[9][4]=(-1.0582010582010582e-04*c)*D;
  A[9][5]=(-1.1287477954144622e-02*c)*D;
  A[9][6]=(3.3862433862433864e-03*c)*D;
  A[9][7]=(-6.7724867724867728e-03*c)*D;
  A[9][8]=(5.6437389770723099e-04*c)*D;
  A[9][9]=(2.7936507936507936e-02*c)*D;
  A[9][10]=(3.3862433862433864e-03*c)*D;
  A[9][11]=(4.2328042328042330e-04*c)*D;
  A[9][12]=(-1.1287477954144622e-02*c)*D;
  A[9][13]=(-8.4656084656084662e-03*c)*D;
  A[9][14]=(-7.0546737213403885e-04*c)*D;
  A[10][0]=(-1.4109347442680777e-03*c)*D;
  A[10][1]=(-2.2574955908289240e-03*c)*D;
  A[10][2]=(-6.7724867724867728e-03*c)*D;
  A[10][3]=(-2.2574955908289240e-03*c)*D;
  A[10][4]=(-1.4109347442680777e-03*c)*D;
  A[10][5]=(2.2574955908289240e-03*c)*D;
  A[10][6]=(-1.3544973544973546e-02*c)*D;
  A[10][7]=(-1.3544973544973546e-02*c)*D;
  A[10][8]=(2.2574955908289240e-03*c)*D;
  A[10][9]=(3.3862433862433864e-03*c)*D;
  A[10][10]=(9.4814814814814810e-02*c)*D;
  A[10][11]=(3.3862433862433864e-03*c)*D;
  A[10][12]=(1.1287477954144622e-02*c)*D;
  A[10][13]=(1.1287477954144622e-02*c)*D;
  A[10][14]=(1.4109347442680777e-03*c)*D;
  A[11][0]=(-1.0582010582010582e-04*c)*D;
  A[11][1]=(5.6437389770723099e-04*c)*D;
  A[11][2]=(4.2328042328042330e-04*c)*D;
  A[11][3]=(-8.4656084656084662e-03*c)*D;
  A[11][4]=(-7.0546737213403885e-04*c)*D;
  A[11][5]=(5.6437389770723099e-04*c)*D;
  A[11][6]=(-6.7724867724867728e-03*c)*D;
  A[11][7]=(3.3862433862433864e-03*c)*D;
  A[11][8]=(-1.1287477954144622e-02*c)*D;
  A[11][9]=(4.2328042328042330e-04*c)*D;
  A[11][10]=(3.3862433862433864e-03*c)*D;
  A[11][11]=(2.7936507936507936e-02*c)*D;
  A[11][12]=(-8.4656084656084662e-03*c)*D;
  A[11][13]=(-1.1287477954144622e-02*c)*D;
  A[11][14]=(-7.0546737213403885e-04*c)*D;
  A[12][0]=(0.0)*D;
  A[12][1]=(4.5149911816578479e-03*c)*D;
  A[12][2]=(5.6437389770723099e-04*c)*D;
  A[12][3]=(2.2574955908289240e-03*c)*D;
  A[12][4]=(-9.8765432098765434e-04*c)*D;
  A[12][5]=(6.7724867724867728e-03*c)*D;
  A[12][6]=(2.2574955908289240e-03*c)*D;
  A[12][7]=(-2.2574955908289240e-03*c)*D;
  A[12][8]=(4.5149911816578479e-03*c)*D;
  A[12][9]=(-1.1287477954144622e-02*c)*D;
  A[12][10]=(1.1287477954144622e-02*c)*D;
  A[12][11]=(-8.4656084656084662e-03*c)*D;
  A[12][12]=(2.2574955908289243e-02*c)*D;
  A[12][13]=(1.1287477954144622e-02*c)*D;
  A[12][14]=(1.4109347442680777e-03*c)*D;
  A[13][0]=(-9.8765432098765434e-04*c)*D;
  A[13][1]=(2.2574955908289240e-03*c)*D;
  A[13][2]=(5.6437389770723099e-04*c)*D;
  A[13][3]=(4.5149911816578479e-03*c)*D;
  A[13][4]=(0.0)*D;
  A[13][5]=(4.5149911816578479e-03*c)*D;
  A[13][6]=(-2.2574955908289240e-03*c)*D;
  A[13][7]=(2.2574955908289240e-03*c)*D;
  A[13][8]=(6.7724867724867728e-03*c)*D;
  A[13][9]=(-8.4656084656084662e-03*c)*D;
  A[13][10]=(1.1287477954144622e-02*c)*D;
  A[13][11]=(-1.1287477954144622e-02*c)*D;
  A[13][12]=(1.1287477954144622e-02*c)*D;
  A[13][13]=(2.2574955908289243e-02*c)*D;
  A[13][14]=(1.4109347442680777e-03*c)*D;
  A[14][0]=(-2.3809523809523810e-04*c)*D;
  A[14][1]=(-9.8765432098765434e-04*c)*D;
  A[14][2]=(-1.0582010582010582e-04*c)*D;
  A[14][3]=(-9.8765432098765434e-04*c)*D;
  A[14][4]=(-2.3809523809523810e-04*c)*D;
  A[14][5]=(0.0)*D;
  A[14][6]=(-1.4109347442680777e-03*c)*D;
  A[14][7]=(-1.4109347442680777e-03*c)*D;
  A[14][8]=(0.0)*D;
  A[14][9]=(-7.0546737213403885e-04*c)*D;
  A[14][10]=(1.4109347442680777e-03*c)*D;
  A[14][11]=(-7.0546737213403885e-04*c)*D;
  A[14][12]=(1.4109347442680777e-03*c)*D;
  A[14][13]=(1.4109347442680777e-03*c)*D;
  A[14][14]=(2.5573192239858908e-03*c)*D;
  int nbf = 15;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 5 ---------------*/


  
void matrix_factory_mass_2D_with_c_order5 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2], double c){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[21];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/5.0+x0;  dof1[1]=G10/5.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=(2.0/5.0)*G00+x0;  dof2[1]=(2.0/5.0)*G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=(3.0/5.0)*G00+x0;  dof3[1]=(3.0/5.0)*G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=(4.0/5.0)*G00+x0;  dof4[1]=(4.0/5.0)*G10+y0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G00+x0;  dof5[1]=G10+y0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/5.0+x0;  dof6[1]=y0+G11/5.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=G01/5.0+G00/5.0+x0;  dof7[1]=G10/5.0+y0+G11/5.0;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=G01/5.0+(2.0/5.0)*G00+x0;  dof8[1]=(2.0/5.0)*G10+y0+G11/5.0;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01/5.0+(3.0/5.0)*G00+x0;  dof9[1]=(3.0/5.0)*G10+y0+G11/5.0;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[2];
  dof10[0]=G01/5.0+(4.0/5.0)*G00+x0;  dof10[1]=(4.0/5.0)*G10+y0+G11/5.0;
  ptv pdof10(2,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[2];
  dof11[0]=(2.0/5.0)*G01+x0;  dof11[1]=y0+(2.0/5.0)*G11;
  ptv pdof11(2,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[2];
  dof12[0]=(2.0/5.0)*G01+G00/5.0+x0;  dof12[1]=G10/5.0+y0+(2.0/5.0)*G11;
  ptv pdof12(2,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[2];
  dof13[0]=(2.0/5.0)*G01+(2.0/5.0)*G00+x0;  dof13[1]=(2.0/5.0)*G10+y0+(2.0/5.0)*G11;
  ptv pdof13(2,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[2];
  dof14[0]=(2.0/5.0)*G01+(3.0/5.0)*G00+x0;  dof14[1]=(3.0/5.0)*G10+y0+(2.0/5.0)*G11;
  ptv pdof14(2,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

  double dof15[2];
  dof15[0]=(3.0/5.0)*G01+x0;  dof15[1]=y0+(3.0/5.0)*G11;
  ptv pdof15(2,dof15);
  iidof[15] = dof.insert_dof(element,15,pdof15);

  double dof16[2];
  dof16[0]=(3.0/5.0)*G01+G00/5.0+x0;  dof16[1]=G10/5.0+y0+(3.0/5.0)*G11;
  ptv pdof16(2,dof16);
  iidof[16] = dof.insert_dof(element,16,pdof16);

  double dof17[2];
  dof17[0]=(3.0/5.0)*G01+(2.0/5.0)*G00+x0;  dof17[1]=(2.0/5.0)*G10+y0+(3.0/5.0)*G11;
  ptv pdof17(2,dof17);
  iidof[17] = dof.insert_dof(element,17,pdof17);

  double dof18[2];
  dof18[0]=(4.0/5.0)*G01+x0;  dof18[1]=y0+(4.0/5.0)*G11;
  ptv pdof18(2,dof18);
  iidof[18] = dof.insert_dof(element,18,pdof18);

  double dof19[2];
  dof19[0]=(4.0/5.0)*G01+G00/5.0+x0;  dof19[1]=G10/5.0+y0+(4.0/5.0)*G11;
  ptv pdof19(2,dof19);
  iidof[19] = dof.insert_dof(element,19,pdof19);

  double dof20[2];
  dof20[0]=G01+x0;  dof20[1]=y0+G11;
  ptv pdof20(2,dof20);
  iidof[20] = dof.insert_dof(element,20,pdof20);

 
  double A[21][21];
  A[0][0]=(1.3894525613275613e-03*c)*D;
  A[0][1]=(1.1813634860509861e-03*c)*D;
  A[0][2]=(-9.5824314574314575e-04*c)*D;
  A[0][3]=(4.6268112674362677e-04*c)*D;
  A[0][4]=(-2.2964432686654910e-05*c)*D;
  A[0][5]=(1.2387745677676232e-04*c)*D;
  A[0][6]=(1.1813634860509861e-03*c)*D;
  A[0][7]=(8.9248136123136121e-04*c)*D;
  A[0][8]=(-9.7468359187109191e-04*c)*D;
  A[0][9]=(8.7943338811394365e-04*c)*D;
  A[0][10]=(3.6364701078242745e-04*c)*D;
  A[0][11]=(-9.5824314574314575e-04*c)*D;
  A[0][12]=(-9.7468359187109191e-04*c)*D;
  A[0][13]=(6.4978906124739453e-04*c)*D;
  A[0][14]=(2.0720181310459088e-04*c)*D;
  A[0][15]=(4.6268112674362677e-04*c)*D;
  A[0][16]=(8.7943338811394365e-04*c)*D;
  A[0][17]=(2.0720181310459088e-04*c)*D;
  A[0][18]=(-2.2964432686654910e-05*c)*D;
  A[0][19]=(3.6364701078242745e-04*c)*D;
  A[0][20]=(1.2387745677676232e-04*c)*D;
  A[1][0]=(1.1813634860509861e-03*c)*D;
  A[1][1]=(1.4705065703329592e-02*c)*D;
  A[1][2]=(-9.5837362547431994e-03*c)*D;
  A[1][3]=(7.7048281258350702e-03*c)*D;
  A[1][4]=(-3.9698458209742934e-03*c)*D;
  A[1][5]=(-2.2964432686654910e-05*c)*D;
  A[1][6]=(7.3525328516647961e-03*c)*D;
  A[1][7]=(1.0242658897172786e-02*c)*D;
  A[1][8]=(9.7859798380631720e-05*c)*D;
  A[1][9]=(-2.8053142202447757e-03*c)*D;
  A[1][10]=(-2.4823768855886910e-03*c)*D;
  A[1][11]=(-7.3525328516647961e-03*c)*D;
  A[1][12]=(-3.8165321368446368e-03*c)*D;
  A[1][13]=(-1.3700371773288440e-03*c)*D;
  A[1][14]=(-4.5667905910961469e-05*c)*D;
  A[1][15]=(5.7606801313398537e-03*c)*D;
  A[1][16]=(7.8287838704505376e-04*c)*D;
  A[1][17]=(-1.4483250160333494e-03*c)*D;
  A[1][18]=(-2.4823768855886910e-03*c)*D;
  A[1][19]=(-4.1101115319865323e-04*c)*D;
  A[1][20]=(3.6364701078242745e-04*c)*D;
  A[2][0]=(-9.5824314574314575e-04*c)*D;
  A[2][1]=(-9.5837362547431994e-03*c)*D;
  A[2][2]=(2.2077170514670515e-02*c)*D;
  A[2][3]=(-1.3478556230292342e-02*c)*D;
  A[2][4]=(7.7048281258350702e-03*c)*D;
  A[2][5]=(4.6268112674362677e-04*c)*D;
  A[2][6]=(-7.3525328516647961e-03*c)*D;
  A[2][7]=(-1.4352770429159319e-03*c)*D;
  A[2][8]=(5.2844291125541129e-03*c)*D;
  A[2][9]=(5.0887095157928488e-03*c)*D;
  A[2][10]=(5.7606801313398537e-03*c)*D;
  A[2][11]=(2.9488419245363691e-03*c)*D;
  A[2][12]=(-5.4801487093153762e-03*c)*D;
  A[2][13]=(-5.8715879028379029e-04*c)*D;
  A[2][14]=(-1.3569892042114265e-03*c)*D;
  A[2][15]=(-1.3569892042114265e-03*c)*D;
  A[2][16]=(3.1315135481802150e-03*c)*D;
  A[2][17]=(2.8183621933621935e-03*c)*D;
  A[2][18]=(-4.5667905910961469e-05*c)*D;
  A[2][19]=(-1.4483250160333494e-03*c)*D;
  A[2][20]=(2.0720181310459088e-04*c)*D;
  A[3][0]=(4.6268112674362677e-04*c)*D;
  A[3][1]=(7.7048281258350702e-03*c)*D;
  A[3][2]=(-1.3478556230292342e-02*c)*D;
  A[3][3]=(2.2077170514670515e-02*c)*D;
  A[3][4]=(-9.5837362547431994e-03*c)*D;
  A[3][5]=(-9.5824314574314575e-04*c)*D;
  A[3][6]=(5.7606801313398537e-03*c)*D;
  A[3][7]=(5.0887095157928488e-03*c)*D;
  A[3][8]=(5.2844291125541129e-03*c)*D;
  A[3][9]=(-1.4352770429159319e-03*c)*D;
  A[3][10]=(-7.3525328516647961e-03*c)*D;
  A[3][11]=(-1.3569892042114265e-03*c)*D;
  A[3][12]=(-5.8715879028379029e-04*c)*D;
  A[3][13]=(-5.4801487093153762e-03*c)*D;
  A[3][14]=(2.9488419245363691e-03*c)*D;
  A[3][15]=(2.8183621933621935e-03*c)*D;
  A[3][16]=(3.1315135481802150e-03*c)*D;
  A[3][17]=(-1.3569892042114265e-03*c)*D;
  A[3][18]=(-1.4483250160333494e-03*c)*D;
  A[3][19]=(-4.5667905910961469e-05*c)*D;
  A[3][20]=(2.0720181310459088e-04*c)*D;
  A[4][0]=(-2.2964432686654910e-05*c)*D;
  A[4][1]=(-3.9698458209742934e-03*c)*D;
  A[4][2]=(7.7048281258350702e-03*c)*D;
  A[4][3]=(-9.5837362547431994e-03*c)*D;
  A[4][4]=(1.4705065703329592e-02*c)*D;
  A[4][5]=(1.1813634860509861e-03*c)*D;
  A[4][6]=(-2.4823768855886910e-03*c)*D;
  A[4][7]=(-2.8053142202447757e-03*c)*D;
  A[4][8]=(9.7859798380631720e-05*c)*D;
  A[4][9]=(1.0242658897172786e-02*c)*D;
  A[4][10]=(7.3525328516647961e-03*c)*D;
  A[4][11]=(-4.5667905910961469e-05*c)*D;
  A[4][12]=(-1.3700371773288440e-03*c)*D;
  A[4][13]=(-3.8165321368446368e-03*c)*D;
  A[4][14]=(-7.3525328516647961e-03*c)*D;
  A[4][15]=(-1.4483250160333494e-03*c)*D;
  A[4][16]=(7.8287838704505376e-04*c)*D;
  A[4][17]=(5.7606801313398537e-03*c)*D;
  A[4][18]=(-4.1101115319865323e-04*c)*D;
  A[4][19]=(-2.4823768855886910e-03*c)*D;
  A[4][20]=(3.6364701078242745e-04*c)*D;
  A[5][0]=(1.2387745677676232e-04*c)*D;
  A[5][1]=(-2.2964432686654910e-05*c)*D;
  A[5][2]=(4.6268112674362677e-04*c)*D;
  A[5][3]=(-9.5824314574314575e-04*c)*D;
  A[5][4]=(1.1813634860509861e-03*c)*D;
  A[5][5]=(1.3894525613275613e-03*c)*D;
  A[5][6]=(3.6364701078242745e-04*c)*D;
  A[5][7]=(8.7943338811394365e-04*c)*D;
  A[5][8]=(-9.7468359187109191e-04*c)*D;
  A[5][9]=(8.9248136123136121e-04*c)*D;
  A[5][10]=(1.1813634860509861e-03*c)*D;
  A[5][11]=(2.0720181310459088e-04*c)*D;
  A[5][12]=(6.4978906124739453e-04*c)*D;
  A[5][13]=(-9.7468359187109191e-04*c)*D;
  A[5][14]=(-9.5824314574314575e-04*c)*D;
  A[5][15]=(2.0720181310459088e-04*c)*D;
  A[5][16]=(8.7943338811394365e-04*c)*D;
  A[5][17]=(4.6268112674362677e-04*c)*D;
  A[5][18]=(3.6364701078242745e-04*c)*D;
  A[5][19]=(-2.2964432686654910e-05*c)*D;
  A[5][20]=(1.2387745677676232e-04*c)*D;
  A[6][0]=(1.1813634860509861e-03*c)*D;
  A[6][1]=(7.3525328516647961e-03*c)*D;
  A[6][2]=(-7.3525328516647961e-03*c)*D;
  A[6][3]=(5.7606801313398537e-03*c)*D;
  A[6][4]=(-2.4823768855886910e-03*c)*D;
  A[6][5]=(3.6364701078242745e-04*c)*D;
  A[6][6]=(1.4705065703329592e-02*c)*D;
  A[6][7]=(1.0242658897172786e-02*c)*D;
  A[6][8]=(-3.8165321368446368e-03*c)*D;
  A[6][9]=(7.8287838704505376e-04*c)*D;
  A[6][10]=(-4.1101115319865323e-04*c)*D;
  A[6][11]=(-9.5837362547431994e-03*c)*D;
  A[6][12]=(9.7859798380631720e-05*c)*D;
  A[6][13]=(-1.3700371773288440e-03*c)*D;
  A[6][14]=(-1.4483250160333494e-03*c)*D;
  A[6][15]=(7.7048281258350702e-03*c)*D;
  A[6][16]=(-2.8053142202447757e-03*c)*D;
  A[6][17]=(-4.5667905910961469e-05*c)*D;
  A[6][18]=(-3.9698458209742934e-03*c)*D;
  A[6][19]=(-2.4823768855886910e-03*c)*D;
  A[6][20]=(-2.2964432686654910e-05*c)*D;
  A[7][0]=(8.9248136123136121e-04*c)*D;
  A[7][1]=(1.0242658897172786e-02*c)*D;
  A[7][2]=(-1.4352770429159319e-03*c)*D;
  A[7][3]=(5.0887095157928488e-03*c)*D;
  A[7][4]=(-2.8053142202447757e-03*c)*D;
  A[7][5]=(8.7943338811394365e-04*c)*D;
  A[7][6]=(1.0242658897172786e-02*c)*D;
  A[7][7]=(7.0459054834054832e-02*c)*D;
  A[7][8]=(-1.5657567740901074e-02*c)*D;
  A[7][9]=(1.1743175805675805e-02*c)*D;
  A[7][10]=(7.8287838704505376e-04*c)*D;
  A[7][11]=(-1.4352770429159319e-03*c)*D;
  A[7][12]=(-1.5657567740901074e-02*c)*D;
  A[7][13]=(3.9143919352252685e-03*c)*D;
  A[7][14]=(3.1315135481802150e-03*c)*D;
  A[7][15]=(5.0887095157928488e-03*c)*D;
  A[7][16]=(1.1743175805675805e-02*c)*D;
  A[7][17]=(3.1315135481802150e-03*c)*D;
  A[7][18]=(-2.8053142202447757e-03*c)*D;
  A[7][19]=(7.8287838704505376e-04*c)*D;
  A[7][20]=(8.7943338811394365e-04*c)*D;
  A[8][0]=(-9.7468359187109191e-04*c)*D;
  A[8][1]=(9.7859798380631720e-05*c)*D;
  A[8][2]=(5.2844291125541129e-03*c)*D;
  A[8][3]=(5.2844291125541129e-03*c)*D;
  A[8][4]=(9.7859798380631720e-05*c)*D;
  A[8][5]=(-9.7468359187109191e-04*c)*D;
  A[8][6]=(-3.8165321368446368e-03*c)*D;
  A[8][7]=(-1.5657567740901074e-02*c)*D;
  A[8][8]=(7.6330642736892731e-02*c)*D;
  A[8][9]=(-1.5657567740901074e-02*c)*D;
  A[8][10]=(-3.8165321368446368e-03*c)*D;
  A[8][11]=(-5.4801487093153762e-03*c)*D;
  A[8][12]=(-1.1743175805675805e-02*c)*D;
  A[8][13]=(-1.1743175805675805e-02*c)*D;
  A[8][14]=(-5.4801487093153762e-03*c)*D;
  A[8][15]=(-5.8715879028379029e-04*c)*D;
  A[8][16]=(3.9143919352252685e-03*c)*D;
  A[8][17]=(-5.8715879028379029e-04*c)*D;
  A[8][18]=(-1.3700371773288440e-03*c)*D;
  A[8][19]=(-1.3700371773288440e-03*c)*D;
  A[8][20]=(6.4978906124739453e-04*c)*D;
  A[9][0]=(8.7943338811394365e-04*c)*D;
  A[9][1]=(-2.8053142202447757e-03*c)*D;
  A[9][2]=(5.0887095157928488e-03*c)*D;
  A[9][3]=(-1.4352770429159319e-03*c)*D;
  A[9][4]=(1.0242658897172786e-02*c)*D;
  A[9][5]=(8.9248136123136121e-04*c)*D;
  A[9][6]=(7.8287838704505376e-04*c)*D;
  A[9][7]=(1.1743175805675805e-02*c)*D;
  A[9][8]=(-1.5657567740901074e-02*c)*D;
  A[9][9]=(7.0459054834054832e-02*c)*D;
  A[9][10]=(1.0242658897172786e-02*c)*D;
  A[9][11]=(3.1315135481802150e-03*c)*D;
  A[9][12]=(3.9143919352252685e-03*c)*D;
  A[9][13]=(-1.5657567740901074e-02*c)*D;
  A[9][14]=(-1.4352770429159319e-03*c)*D;
  A[9][15]=(3.1315135481802150e-03*c)*D;
  A[9][16]=(1.1743175805675805e-02*c)*D;
  A[9][17]=(5.0887095157928488e-03*c)*D;
  A[9][18]=(7.8287838704505376e-04*c)*D;
  A[9][19]=(-2.8053142202447757e-03*c)*D;
  A[9][20]=(8.7943338811394365e-04*c)*D;
  A[10][0]=(3.6364701078242745e-04*c)*D;
  A[10][1]=(-2.4823768855886910e-03*c)*D;
  A[10][2]=(5.7606801313398537e-03*c)*D;
  A[10][3]=(-7.3525328516647961e-03*c)*D;
  A[10][4]=(7.3525328516647961e-03*c)*D;
  A[10][5]=(1.1813634860509861e-03*c)*D;
  A[10][6]=(-4.1101115319865323e-04*c)*D;
  A[10][7]=(7.8287838704505376e-04*c)*D;
  A[10][8]=(-3.8165321368446368e-03*c)*D;
  A[10][9]=(1.0242658897172786e-02*c)*D;
  A[10][10]=(1.4705065703329592e-02*c)*D;
  A[10][11]=(-1.4483250160333494e-03*c)*D;
  A[10][12]=(-1.3700371773288440e-03*c)*D;
  A[10][13]=(9.7859798380631720e-05*c)*D;
  A[10][14]=(-9.5837362547431994e-03*c)*D;
  A[10][15]=(-4.5667905910961469e-05*c)*D;
  A[10][16]=(-2.8053142202447757e-03*c)*D;
  A[10][17]=(7.7048281258350702e-03*c)*D;
  A[10][18]=(-2.4823768855886910e-03*c)*D;
  A[10][19]=(-3.9698458209742934e-03*c)*D;
  A[10][20]=(-2.2964432686654910e-05*c)*D;
  A[11][0]=(-9.5824314574314575e-04*c)*D;
  A[11][1]=(-7.3525328516647961e-03*c)*D;
  A[11][2]=(2.9488419245363691e-03*c)*D;
  A[11][3]=(-1.3569892042114265e-03*c)*D;
  A[11][4]=(-4.5667905910961469e-05*c)*D;
  A[11][5]=(2.0720181310459088e-04*c)*D;
  A[11][6]=(-9.5837362547431994e-03*c)*D;
  A[11][7]=(-1.4352770429159319e-03*c)*D;
  A[11][8]=(-5.4801487093153762e-03*c)*D;
  A[11][9]=(3.1315135481802150e-03*c)*D;
  A[11][10]=(-1.4483250160333494e-03*c)*D;
  A[11][11]=(2.2077170514670515e-02*c)*D;
  A[11][12]=(5.2844291125541129e-03*c)*D;
  A[11][13]=(-5.8715879028379029e-04*c)*D;
  A[11][14]=(2.8183621933621935e-03*c)*D;
  A[11][15]=(-1.3478556230292342e-02*c)*D;
  A[11][16]=(5.0887095157928488e-03*c)*D;
  A[11][17]=(-1.3569892042114265e-03*c)*D;
  A[11][18]=(7.7048281258350702e-03*c)*D;
  A[11][19]=(5.7606801313398537e-03*c)*D;
  A[11][20]=(4.6268112674362677e-04*c)*D;
  A[12][0]=(-9.7468359187109191e-04*c)*D;
  A[12][1]=(-3.8165321368446368e-03*c)*D;
  A[12][2]=(-5.4801487093153762e-03*c)*D;
  A[12][3]=(-5.8715879028379029e-04*c)*D;
  A[12][4]=(-1.3700371773288440e-03*c)*D;
  A[12][5]=(6.4978906124739453e-04*c)*D;
  A[12][6]=(9.7859798380631720e-05*c)*D;
  A[12][7]=(-1.5657567740901074e-02*c)*D;
  A[12][8]=(-1.1743175805675805e-02*c)*D;
  A[12][9]=(3.9143919352252685e-03*c)*D;
  A[12][10]=(-1.3700371773288440e-03*c)*D;
  A[12][11]=(5.2844291125541129e-03*c)*D;
  A[12][12]=(7.6330642736892731e-02*c)*D;
  A[12][13]=(-1.1743175805675805e-02*c)*D;
  A[12][14]=(-5.8715879028379029e-04*c)*D;
  A[12][15]=(5.2844291125541129e-03*c)*D;
  A[12][16]=(-1.5657567740901074e-02*c)*D;
  A[12][17]=(-5.4801487093153762e-03*c)*D;
  A[12][18]=(9.7859798380631720e-05*c)*D;
  A[12][19]=(-3.8165321368446368e-03*c)*D;
  A[12][20]=(-9.7468359187109191e-04*c)*D;
  A[13][0]=(6.4978906124739453e-04*c)*D;
  A[13][1]=(-1.3700371773288440e-03*c)*D;
  A[13][2]=(-5.8715879028379029e-04*c)*D;
  A[13][3]=(-5.4801487093153762e-03*c)*D;
  A[13][4]=(-3.8165321368446368e-03*c)*D;
  A[13][5]=(-9.7468359187109191e-04*c)*D;
  A[13][6]=(-1.3700371773288440e-03*c)*D;
  A[13][7]=(3.9143919352252685e-03*c)*D;
  A[13][8]=(-1.1743175805675805e-02*c)*D;
  A[13][9]=(-1.5657567740901074e-02*c)*D;
  A[13][10]=(9.7859798380631720e-05*c)*D;
  A[13][11]=(-5.8715879028379029e-04*c)*D;
  A[13][12]=(-1.1743175805675805e-02*c)*D;
  A[13][13]=(7.6330642736892731e-02*c)*D;
  A[13][14]=(5.2844291125541129e-03*c)*D;
  A[13][15]=(-5.4801487093153762e-03*c)*D;
  A[13][16]=(-1.5657567740901074e-02*c)*D;
  A[13][17]=(5.2844291125541129e-03*c)*D;
  A[13][18]=(-3.8165321368446368e-03*c)*D;
  A[13][19]=(9.7859798380631720e-05*c)*D;
  A[13][20]=(-9.7468359187109191e-04*c)*D;
  A[14][0]=(2.0720181310459088e-04*c)*D;
  A[14][1]=(-4.5667905910961469e-05*c)*D;
  A[14][2]=(-1.3569892042114265e-03*c)*D;
  A[14][3]=(2.9488419245363691e-03*c)*D;
  A[14][4]=(-7.3525328516647961e-03*c)*D;
  A[14][5]=(-9.5824314574314575e-04*c)*D;
  A[14][6]=(-1.4483250160333494e-03*c)*D;
  A[14][7]=(3.1315135481802150e-03*c)*D;
  A[14][8]=(-5.4801487093153762e-03*c)*D;
  A[14][9]=(-1.4352770429159319e-03*c)*D;
  A[14][10]=(-9.5837362547431994e-03*c)*D;
  A[14][11]=(2.8183621933621935e-03*c)*D;
  A[14][12]=(-5.8715879028379029e-04*c)*D;
  A[14][13]=(5.2844291125541129e-03*c)*D;
  A[14][14]=(2.2077170514670515e-02*c)*D;
  A[14][15]=(-1.3569892042114265e-03*c)*D;
  A[14][16]=(5.0887095157928488e-03*c)*D;
  A[14][17]=(-1.3478556230292342e-02*c)*D;
  A[14][18]=(5.7606801313398537e-03*c)*D;
  A[14][19]=(7.7048281258350702e-03*c)*D;
  A[14][20]=(4.6268112674362677e-04*c)*D;
  A[15][0]=(4.6268112674362677e-04*c)*D;
  A[15][1]=(5.7606801313398537e-03*c)*D;
  A[15][2]=(-1.3569892042114265e-03*c)*D;
  A[15][3]=(2.8183621933621935e-03*c)*D;
  A[15][4]=(-1.4483250160333494e-03*c)*D;
  A[15][5]=(2.0720181310459088e-04*c)*D;
  A[15][6]=(7.7048281258350702e-03*c)*D;
  A[15][7]=(5.0887095157928488e-03*c)*D;
  A[15][8]=(-5.8715879028379029e-04*c)*D;
  A[15][9]=(3.1315135481802150e-03*c)*D;
  A[15][10]=(-4.5667905910961469e-05*c)*D;
  A[15][11]=(-1.3478556230292342e-02*c)*D;
  A[15][12]=(5.2844291125541129e-03*c)*D;
  A[15][13]=(-5.4801487093153762e-03*c)*D;
  A[15][14]=(-1.3569892042114265e-03*c)*D;
  A[15][15]=(2.2077170514670515e-02*c)*D;
  A[15][16]=(-1.4352770429159319e-03*c)*D;
  A[15][17]=(2.9488419245363691e-03*c)*D;
  A[15][18]=(-9.5837362547431994e-03*c)*D;
  A[15][19]=(-7.3525328516647961e-03*c)*D;
  A[15][20]=(-9.5824314574314575e-04*c)*D;
  A[16][0]=(8.7943338811394365e-04*c)*D;
  A[16][1]=(7.8287838704505376e-04*c)*D;
  A[16][2]=(3.1315135481802150e-03*c)*D;
  A[16][3]=(3.1315135481802150e-03*c)*D;
  A[16][4]=(7.8287838704505376e-04*c)*D;
  A[16][5]=(8.7943338811394365e-04*c)*D;
  A[16][6]=(-2.8053142202447757e-03*c)*D;
  A[16][7]=(1.1743175805675805e-02*c)*D;
  A[16][8]=(3.9143919352252685e-03*c)*D;
  A[16][9]=(1.1743175805675805e-02*c)*D;
  A[16][10]=(-2.8053142202447757e-03*c)*D;
  A[16][11]=(5.0887095157928488e-03*c)*D;
  A[16][12]=(-1.5657567740901074e-02*c)*D;
  A[16][13]=(-1.5657567740901074e-02*c)*D;
  A[16][14]=(5.0887095157928488e-03*c)*D;
  A[16][15]=(-1.4352770429159319e-03*c)*D;
  A[16][16]=(7.0459054834054832e-02*c)*D;
  A[16][17]=(-1.4352770429159319e-03*c)*D;
  A[16][18]=(1.0242658897172786e-02*c)*D;
  A[16][19]=(1.0242658897172786e-02*c)*D;
  A[16][20]=(8.9248136123136121e-04*c)*D;
  A[17][0]=(2.0720181310459088e-04*c)*D;
  A[17][1]=(-1.4483250160333494e-03*c)*D;
  A[17][2]=(2.8183621933621935e-03*c)*D;
  A[17][3]=(-1.3569892042114265e-03*c)*D;
  A[17][4]=(5.7606801313398537e-03*c)*D;
  A[17][5]=(4.6268112674362677e-04*c)*D;
  A[17][6]=(-4.5667905910961469e-05*c)*D;
  A[17][7]=(3.1315135481802150e-03*c)*D;
  A[17][8]=(-5.8715879028379029e-04*c)*D;
  A[17][9]=(5.0887095157928488e-03*c)*D;
  A[17][10]=(7.7048281258350702e-03*c)*D;
  A[17][11]=(-1.3569892042114265e-03*c)*D;
  A[17][12]=(-5.4801487093153762e-03*c)*D;
  A[17][13]=(5.2844291125541129e-03*c)*D;
  A[17][14]=(-1.3478556230292342e-02*c)*D;
  A[17][15]=(2.9488419245363691e-03*c)*D;
  A[17][16]=(-1.4352770429159319e-03*c)*D;
  A[17][17]=(2.2077170514670515e-02*c)*D;
  A[17][18]=(-7.3525328516647961e-03*c)*D;
  A[17][19]=(-9.5837362547431994e-03*c)*D;
  A[17][20]=(-9.5824314574314575e-04*c)*D;
  A[18][0]=(-2.2964432686654910e-05*c)*D;
  A[18][1]=(-2.4823768855886910e-03*c)*D;
  A[18][2]=(-4.5667905910961469e-05*c)*D;
  A[18][3]=(-1.4483250160333494e-03*c)*D;
  A[18][4]=(-4.1101115319865323e-04*c)*D;
  A[18][5]=(3.6364701078242745e-04*c)*D;
  A[18][6]=(-3.9698458209742934e-03*c)*D;
  A[18][7]=(-2.8053142202447757e-03*c)*D;
  A[18][8]=(-1.3700371773288440e-03*c)*D;
  A[18][9]=(7.8287838704505376e-04*c)*D;
  A[18][10]=(-2.4823768855886910e-03*c)*D;
  A[18][11]=(7.7048281258350702e-03*c)*D;
  A[18][12]=(9.7859798380631720e-05*c)*D;
  A[18][13]=(-3.8165321368446368e-03*c)*D;
  A[18][14]=(5.7606801313398537e-03*c)*D;
  A[18][15]=(-9.5837362547431994e-03*c)*D;
  A[18][16]=(1.0242658897172786e-02*c)*D;
  A[18][17]=(-7.3525328516647961e-03*c)*D;
  A[18][18]=(1.4705065703329592e-02*c)*D;
  A[18][19]=(7.3525328516647961e-03*c)*D;
  A[18][20]=(1.1813634860509861e-03*c)*D;
  A[19][0]=(3.6364701078242745e-04*c)*D;
  A[19][1]=(-4.1101115319865323e-04*c)*D;
  A[19][2]=(-1.4483250160333494e-03*c)*D;
  A[19][3]=(-4.5667905910961469e-05*c)*D;
  A[19][4]=(-2.4823768855886910e-03*c)*D;
  A[19][5]=(-2.2964432686654910e-05*c)*D;
  A[19][6]=(-2.4823768855886910e-03*c)*D;
  A[19][7]=(7.8287838704505376e-04*c)*D;
  A[19][8]=(-1.3700371773288440e-03*c)*D;
  A[19][9]=(-2.8053142202447757e-03*c)*D;
  A[19][10]=(-3.9698458209742934e-03*c)*D;
  A[19][11]=(5.7606801313398537e-03*c)*D;
  A[19][12]=(-3.8165321368446368e-03*c)*D;
  A[19][13]=(9.7859798380631720e-05*c)*D;
  A[19][14]=(7.7048281258350702e-03*c)*D;
  A[19][15]=(-7.3525328516647961e-03*c)*D;
  A[19][16]=(1.0242658897172786e-02*c)*D;
  A[19][17]=(-9.5837362547431994e-03*c)*D;
  A[19][18]=(7.3525328516647961e-03*c)*D;
  A[19][19]=(1.4705065703329592e-02*c)*D;
  A[19][20]=(1.1813634860509861e-03*c)*D;
  A[20][0]=(1.2387745677676232e-04*c)*D;
  A[20][1]=(3.6364701078242745e-04*c)*D;
  A[20][2]=(2.0720181310459088e-04*c)*D;
  A[20][3]=(2.0720181310459088e-04*c)*D;
  A[20][4]=(3.6364701078242745e-04*c)*D;
  A[20][5]=(1.2387745677676232e-04*c)*D;
  A[20][6]=(-2.2964432686654910e-05*c)*D;
  A[20][7]=(8.7943338811394365e-04*c)*D;
  A[20][8]=(6.4978906124739453e-04*c)*D;
  A[20][9]=(8.7943338811394365e-04*c)*D;
  A[20][10]=(-2.2964432686654910e-05*c)*D;
  A[20][11]=(4.6268112674362677e-04*c)*D;
  A[20][12]=(-9.7468359187109191e-04*c)*D;
  A[20][13]=(-9.7468359187109191e-04*c)*D;
  A[20][14]=(4.6268112674362677e-04*c)*D;
  A[20][15]=(-9.5824314574314575e-04*c)*D;
  A[20][16]=(8.9248136123136121e-04*c)*D;
  A[20][17]=(-9.5824314574314575e-04*c)*D;
  A[20][18]=(1.1813634860509861e-03*c)*D;
  A[20][19]=(1.1813634860509861e-03*c)*D;
  A[20][20]=(1.3894525613275613e-03*c)*D;
  int nbf = 21;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}
