
#include <math.h>
#include <utility>
#include <map>
#include <ptv.h>
#include <DofT.h>
#include <ScalarFunction.h>

using namespace std; 



/* ------------- order 1 ---------------*/


  
void matrix_factory_mass_3D_order1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3]){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[4];
  double dof0[3];
  dof0[0]=x0;  dof0[1]=y0;  dof0[2]=z0;
  ptv pdof0(3,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[3];
  dof1[0]=G00+x0;  dof1[1]=y0+G10;  dof1[2]=z0+G20;
  ptv pdof1(3,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[3];
  dof2[0]=G01+x0;  dof2[1]=y0+G11;  dof2[2]=G21+z0;
  ptv pdof2(3,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[3];
  dof3[0]=G02+x0;  dof3[1]=G12+y0;  dof3[2]=G22+z0;
  ptv pdof3(3,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

 
  double A[4][4];
  A[0][0]=(1.6666666666666666e-02)*D;
  A[0][1]=(8.3333333333333332e-03)*D;
  A[0][2]=(8.3333333333333332e-03)*D;
  A[0][3]=(8.3333333333333332e-03)*D;
  A[1][0]=(8.3333333333333332e-03)*D;
  A[1][1]=(1.6666666666666666e-02)*D;
  A[1][2]=(8.3333333333333332e-03)*D;
  A[1][3]=(8.3333333333333332e-03)*D;
  A[2][0]=(8.3333333333333332e-03)*D;
  A[2][1]=(8.3333333333333332e-03)*D;
  A[2][2]=(1.6666666666666666e-02)*D;
  A[2][3]=(8.3333333333333332e-03)*D;
  A[3][0]=(8.3333333333333332e-03)*D;
  A[3][1]=(8.3333333333333332e-03)*D;
  A[3][2]=(8.3333333333333332e-03)*D;
  A[3][3]=(1.6666666666666666e-02)*D;
  int nbf = 4;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 ---------------*/


  
void matrix_factory_mass_3D_order2 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3]){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[10];
  double dof0[3];
  dof0[0]=x0;  dof0[1]=y0;  dof0[2]=z0;
  ptv pdof0(3,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[3];
  dof1[0]=G00/2.0+x0;  dof1[1]=y0+G10/2.0;  dof1[2]=z0+G20/2.0;
  ptv pdof1(3,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[3];
  dof2[0]=G00+x0;  dof2[1]=y0+G10;  dof2[2]=z0+G20;
  ptv pdof2(3,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[3];
  dof3[0]=G01/2.0+x0;  dof3[1]=y0+G11/2.0;  dof3[2]=G21/2.0+z0;
  ptv pdof3(3,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[3];
  dof4[0]=G01/2.0+G00/2.0+x0;  dof4[1]=y0+G11/2.0+G10/2.0;  dof4[2]=G21/2.0+z0+G20/2.0;
  ptv pdof4(3,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[3];
  dof5[0]=G01+x0;  dof5[1]=y0+G11;  dof5[2]=G21+z0;
  ptv pdof5(3,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[3];
  dof6[0]=G02/2.0+x0;  dof6[1]=G12/2.0+y0;  dof6[2]=G22/2.0+z0;
  ptv pdof6(3,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[3];
  dof7[0]=G02/2.0+G00/2.0+x0;  dof7[1]=G12/2.0+y0+G10/2.0;  dof7[2]=G22/2.0+z0+G20/2.0;
  ptv pdof7(3,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[3];
  dof8[0]=G02/2.0+G01/2.0+x0;  dof8[1]=G12/2.0+y0+G11/2.0;  dof8[2]=G22/2.0+G21/2.0+z0;
  ptv pdof8(3,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[3];
  dof9[0]=G02+x0;  dof9[1]=G12+y0;  dof9[2]=G22+z0;
  ptv pdof9(3,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

 
  double A[10][10];
  A[0][0]=(2.3809523809523812e-03)*D;
  A[0][1]=(-1.5873015873015873e-03)*D;
  A[0][2]=(3.9682539682539683e-04)*D;
  A[0][3]=(-1.5873015873015873e-03)*D;
  A[0][4]=(-2.3809523809523812e-03)*D;
  A[0][5]=(3.9682539682539683e-04)*D;
  A[0][6]=(-1.5873015873015873e-03)*D;
  A[0][7]=(-2.3809523809523812e-03)*D;
  A[0][8]=(-2.3809523809523812e-03)*D;
  A[0][9]=(3.9682539682539683e-04)*D;
  A[1][0]=(-1.5873015873015873e-03)*D;
  A[1][1]=(1.2698412698412698e-02)*D;
  A[1][2]=(-1.5873015873015873e-03)*D;
  A[1][3]=(6.3492063492063492e-03)*D;
  A[1][4]=(6.3492063492063492e-03)*D;
  A[1][5]=(-2.3809523809523812e-03)*D;
  A[1][6]=(6.3492063492063492e-03)*D;
  A[1][7]=(6.3492063492063492e-03)*D;
  A[1][8]=(3.1746031746031746e-03)*D;
  A[1][9]=(-2.3809523809523812e-03)*D;
  A[2][0]=(3.9682539682539683e-04)*D;
  A[2][1]=(-1.5873015873015873e-03)*D;
  A[2][2]=(2.3809523809523812e-03)*D;
  A[2][3]=(-2.3809523809523812e-03)*D;
  A[2][4]=(-1.5873015873015873e-03)*D;
  A[2][5]=(3.9682539682539683e-04)*D;
  A[2][6]=(-2.3809523809523812e-03)*D;
  A[2][7]=(-1.5873015873015873e-03)*D;
  A[2][8]=(-2.3809523809523812e-03)*D;
  A[2][9]=(3.9682539682539683e-04)*D;
  A[3][0]=(-1.5873015873015873e-03)*D;
  A[3][1]=(6.3492063492063492e-03)*D;
  A[3][2]=(-2.3809523809523812e-03)*D;
  A[3][3]=(1.2698412698412698e-02)*D;
  A[3][4]=(6.3492063492063492e-03)*D;
  A[3][5]=(-1.5873015873015873e-03)*D;
  A[3][6]=(6.3492063492063492e-03)*D;
  A[3][7]=(3.1746031746031746e-03)*D;
  A[3][8]=(6.3492063492063492e-03)*D;
  A[3][9]=(-2.3809523809523812e-03)*D;
  A[4][0]=(-2.3809523809523812e-03)*D;
  A[4][1]=(6.3492063492063492e-03)*D;
  A[4][2]=(-1.5873015873015873e-03)*D;
  A[4][3]=(6.3492063492063492e-03)*D;
  A[4][4]=(1.2698412698412698e-02)*D;
  A[4][5]=(-1.5873015873015873e-03)*D;
  A[4][6]=(3.1746031746031746e-03)*D;
  A[4][7]=(6.3492063492063492e-03)*D;
  A[4][8]=(6.3492063492063492e-03)*D;
  A[4][9]=(-2.3809523809523812e-03)*D;
  A[5][0]=(3.9682539682539683e-04)*D;
  A[5][1]=(-2.3809523809523812e-03)*D;
  A[5][2]=(3.9682539682539683e-04)*D;
  A[5][3]=(-1.5873015873015873e-03)*D;
  A[5][4]=(-1.5873015873015873e-03)*D;
  A[5][5]=(2.3809523809523812e-03)*D;
  A[5][6]=(-2.3809523809523812e-03)*D;
  A[5][7]=(-2.3809523809523812e-03)*D;
  A[5][8]=(-1.5873015873015873e-03)*D;
  A[5][9]=(3.9682539682539683e-04)*D;
  A[6][0]=(-1.5873015873015873e-03)*D;
  A[6][1]=(6.3492063492063492e-03)*D;
  A[6][2]=(-2.3809523809523812e-03)*D;
  A[6][3]=(6.3492063492063492e-03)*D;
  A[6][4]=(3.1746031746031746e-03)*D;
  A[6][5]=(-2.3809523809523812e-03)*D;
  A[6][6]=(1.2698412698412698e-02)*D;
  A[6][7]=(6.3492063492063492e-03)*D;
  A[6][8]=(6.3492063492063492e-03)*D;
  A[6][9]=(-1.5873015873015873e-03)*D;
  A[7][0]=(-2.3809523809523812e-03)*D;
  A[7][1]=(6.3492063492063492e-03)*D;
  A[7][2]=(-1.5873015873015873e-03)*D;
  A[7][3]=(3.1746031746031746e-03)*D;
  A[7][4]=(6.3492063492063492e-03)*D;
  A[7][5]=(-2.3809523809523812e-03)*D;
  A[7][6]=(6.3492063492063492e-03)*D;
  A[7][7]=(1.2698412698412698e-02)*D;
  A[7][8]=(6.3492063492063492e-03)*D;
  A[7][9]=(-1.5873015873015873e-03)*D;
  A[8][0]=(-2.3809523809523812e-03)*D;
  A[8][1]=(3.1746031746031746e-03)*D;
  A[8][2]=(-2.3809523809523812e-03)*D;
  A[8][3]=(6.3492063492063492e-03)*D;
  A[8][4]=(6.3492063492063492e-03)*D;
  A[8][5]=(-1.5873015873015873e-03)*D;
  A[8][6]=(6.3492063492063492e-03)*D;
  A[8][7]=(6.3492063492063492e-03)*D;
  A[8][8]=(1.2698412698412698e-02)*D;
  A[8][9]=(-1.5873015873015873e-03)*D;
  A[9][0]=(3.9682539682539683e-04)*D;
  A[9][1]=(-2.3809523809523812e-03)*D;
  A[9][2]=(3.9682539682539683e-04)*D;
  A[9][3]=(-2.3809523809523812e-03)*D;
  A[9][4]=(-2.3809523809523812e-03)*D;
  A[9][5]=(3.9682539682539683e-04)*D;
  A[9][6]=(-1.5873015873015873e-03)*D;
  A[9][7]=(-1.5873015873015873e-03)*D;
  A[9][8]=(-1.5873015873015873e-03)*D;
  A[9][9]=(2.3809523809523812e-03)*D;
  int nbf = 10;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 ---------------*/


  
void matrix_factory_mass_3D_order3 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3]){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[20];
  double dof0[3];
  dof0[0]=x0;  dof0[1]=y0;  dof0[2]=z0;
  ptv pdof0(3,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[3];
  dof1[0]=G00/3.0+x0;  dof1[1]=y0+G10/3.0;  dof1[2]=z0+G20/3.0;
  ptv pdof1(3,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[3];
  dof2[0]=(2.0/3.0)*G00+x0;  dof2[1]=y0+(2.0/3.0)*G10;  dof2[2]=z0+(2.0/3.0)*G20;
  ptv pdof2(3,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[3];
  dof3[0]=G00+x0;  dof3[1]=y0+G10;  dof3[2]=z0+G20;
  ptv pdof3(3,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[3];
  dof4[0]=G01/3.0+x0;  dof4[1]=y0+G11/3.0;  dof4[2]=G21/3.0+z0;
  ptv pdof4(3,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[3];
  dof5[0]=G01/3.0+G00/3.0+x0;  dof5[1]=y0+G11/3.0+G10/3.0;  dof5[2]=G21/3.0+z0+G20/3.0;
  ptv pdof5(3,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[3];
  dof6[0]=G01/3.0+(2.0/3.0)*G00+x0;  dof6[1]=y0+G11/3.0+(2.0/3.0)*G10;  dof6[2]=G21/3.0+z0+(2.0/3.0)*G20;
  ptv pdof6(3,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[3];
  dof7[0]=(2.0/3.0)*G01+x0;  dof7[1]=y0+(2.0/3.0)*G11;  dof7[2]=(2.0/3.0)*G21+z0;
  ptv pdof7(3,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[3];
  dof8[0]=(2.0/3.0)*G01+G00/3.0+x0;  dof8[1]=y0+(2.0/3.0)*G11+G10/3.0;  dof8[2]=(2.0/3.0)*G21+z0+G20/3.0;
  ptv pdof8(3,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[3];
  dof9[0]=G01+x0;  dof9[1]=y0+G11;  dof9[2]=G21+z0;
  ptv pdof9(3,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[3];
  dof10[0]=G02/3.0+x0;  dof10[1]=G12/3.0+y0;  dof10[2]=G22/3.0+z0;
  ptv pdof10(3,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[3];
  dof11[0]=G02/3.0+G00/3.0+x0;  dof11[1]=G12/3.0+y0+G10/3.0;  dof11[2]=G22/3.0+z0+G20/3.0;
  ptv pdof11(3,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[3];
  dof12[0]=G02/3.0+(2.0/3.0)*G00+x0;  dof12[1]=G12/3.0+y0+(2.0/3.0)*G10;  dof12[2]=G22/3.0+z0+(2.0/3.0)*G20;
  ptv pdof12(3,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[3];
  dof13[0]=G02/3.0+G01/3.0+x0;  dof13[1]=G12/3.0+y0+G11/3.0;  dof13[2]=G22/3.0+G21/3.0+z0;
  ptv pdof13(3,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[3];
  dof14[0]=G02/3.0+G01/3.0+G00/3.0+x0;  dof14[1]=G12/3.0+y0+G11/3.0+G10/3.0;  dof14[2]=G22/3.0+G21/3.0+z0+G20/3.0;
  ptv pdof14(3,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

  double dof15[3];
  dof15[0]=G02/3.0+(2.0/3.0)*G01+x0;  dof15[1]=G12/3.0+y0+(2.0/3.0)*G11;  dof15[2]=G22/3.0+(2.0/3.0)*G21+z0;
  ptv pdof15(3,dof15);
  iidof[15] = dof.insert_dof(element,15,pdof15);

  double dof16[3];
  dof16[0]=(2.0/3.0)*G02+x0;  dof16[1]=(2.0/3.0)*G12+y0;  dof16[2]=(2.0/3.0)*G22+z0;
  ptv pdof16(3,dof16);
  iidof[16] = dof.insert_dof(element,16,pdof16);

  double dof17[3];
  dof17[0]=(2.0/3.0)*G02+G00/3.0+x0;  dof17[1]=(2.0/3.0)*G12+y0+G10/3.0;  dof17[2]=(2.0/3.0)*G22+z0+G20/3.0;
  ptv pdof17(3,dof17);
  iidof[17] = dof.insert_dof(element,17,pdof17);

  double dof18[3];
  dof18[0]=(2.0/3.0)*G02+G01/3.0+x0;  dof18[1]=(2.0/3.0)*G12+y0+G11/3.0;  dof18[2]=(2.0/3.0)*G22+G21/3.0+z0;
  ptv pdof18(3,dof18);
  iidof[18] = dof.insert_dof(element,18,pdof18);

  double dof19[3];
  dof19[0]=G02+x0;  dof19[1]=G12+y0;  dof19[2]=G22+z0;
  ptv pdof19(3,dof19);
  iidof[19] = dof.insert_dof(element,19,pdof19);

 
  double A[20][20];
  A[0][0]=(5.9523809523809529e-04)*D;
  A[0][1]=(-4.4642857142857141e-04)*D;
  A[0][2]=(2.2321428571428571e-04)*D;
  A[0][3]=(7.4404761904761911e-05)*D;
  A[0][4]=(-4.4642857142857141e-04)*D;
  A[0][5]=(6.6964285714285715e-04)*D;
  A[0][6]=(1.1160714285714285e-04)*D;
  A[0][7]=(2.2321428571428571e-04)*D;
  A[0][8]=(1.1160714285714285e-04)*D;
  A[0][9]=(7.4404761904761911e-05)*D;
  A[0][10]=(-4.4642857142857141e-04)*D;
  A[0][11]=(6.6964285714285715e-04)*D;
  A[0][12]=(1.1160714285714285e-04)*D;
  A[0][13]=(6.6964285714285715e-04)*D;
  A[0][14]=(1.3392857142857143e-03)*D;
  A[0][15]=(1.1160714285714285e-04)*D;
  A[0][16]=(2.2321428571428571e-04)*D;
  A[0][17]=(1.1160714285714285e-04)*D;
  A[0][18]=(1.1160714285714285e-04)*D;
  A[0][19]=(7.4404761904761911e-05)*D;
  A[1][0]=(-4.4642857142857141e-04)*D;
  A[1][1]=(4.0178571428571425e-03)*D;
  A[1][2]=(-2.0089285714285712e-03)*D;
  A[1][3]=(2.2321428571428571e-04)*D;
  A[1][4]=(2.0089285714285712e-03)*D;
  A[1][5]=(0.0)*D;
  A[1][6]=(-1.0044642857142856e-03)*D;
  A[1][7]=(-1.0044642857142856e-03)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(1.1160714285714285e-04)*D;
  A[1][10]=(2.0089285714285712e-03)*D;
  A[1][11]=(0.0)*D;
  A[1][12]=(-1.0044642857142856e-03)*D;
  A[1][13]=(0.0)*D;
  A[1][14]=(-2.0089285714285712e-03)*D;
  A[1][15]=(0.0)*D;
  A[1][16]=(-1.0044642857142856e-03)*D;
  A[1][17]=(0.0)*D;
  A[1][18]=(0.0)*D;
  A[1][19]=(1.1160714285714285e-04)*D;
  A[2][0]=(2.2321428571428571e-04)*D;
  A[2][1]=(-2.0089285714285712e-03)*D;
  A[2][2]=(4.0178571428571425e-03)*D;
  A[2][3]=(-4.4642857142857141e-04)*D;
  A[2][4]=(-1.0044642857142856e-03)*D;
  A[2][5]=(0.0)*D;
  A[2][6]=(2.0089285714285712e-03)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(-1.0044642857142856e-03)*D;
  A[2][9]=(1.1160714285714285e-04)*D;
  A[2][10]=(-1.0044642857142856e-03)*D;
  A[2][11]=(0.0)*D;
  A[2][12]=(2.0089285714285712e-03)*D;
  A[2][13]=(-2.0089285714285712e-03)*D;
  A[2][14]=(0.0)*D;
  A[2][15]=(0.0)*D;
  A[2][16]=(0.0)*D;
  A[2][17]=(-1.0044642857142856e-03)*D;
  A[2][18]=(0.0)*D;
  A[2][19]=(1.1160714285714285e-04)*D;
  A[3][0]=(7.4404761904761911e-05)*D;
  A[3][1]=(2.2321428571428571e-04)*D;
  A[3][2]=(-4.4642857142857141e-04)*D;
  A[3][3]=(5.9523809523809529e-04)*D;
  A[3][4]=(1.1160714285714285e-04)*D;
  A[3][5]=(6.6964285714285715e-04)*D;
  A[3][6]=(-4.4642857142857141e-04)*D;
  A[3][7]=(1.1160714285714285e-04)*D;
  A[3][8]=(2.2321428571428571e-04)*D;
  A[3][9]=(7.4404761904761911e-05)*D;
  A[3][10]=(1.1160714285714285e-04)*D;
  A[3][11]=(6.6964285714285715e-04)*D;
  A[3][12]=(-4.4642857142857141e-04)*D;
  A[3][13]=(1.3392857142857143e-03)*D;
  A[3][14]=(6.6964285714285715e-04)*D;
  A[3][15]=(1.1160714285714285e-04)*D;
  A[3][16]=(1.1160714285714285e-04)*D;
  A[3][17]=(2.2321428571428571e-04)*D;
  A[3][18]=(1.1160714285714285e-04)*D;
  A[3][19]=(7.4404761904761911e-05)*D;
  A[4][0]=(-4.4642857142857141e-04)*D;
  A[4][1]=(2.0089285714285712e-03)*D;
  A[4][2]=(-1.0044642857142856e-03)*D;
  A[4][3]=(1.1160714285714285e-04)*D;
  A[4][4]=(4.0178571428571425e-03)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(0.0)*D;
  A[4][7]=(-2.0089285714285712e-03)*D;
  A[4][8]=(-1.0044642857142856e-03)*D;
  A[4][9]=(2.2321428571428571e-04)*D;
  A[4][10]=(2.0089285714285712e-03)*D;
  A[4][11]=(0.0)*D;
  A[4][12]=(0.0)*D;
  A[4][13]=(0.0)*D;
  A[4][14]=(-2.0089285714285712e-03)*D;
  A[4][15]=(-1.0044642857142856e-03)*D;
  A[4][16]=(-1.0044642857142856e-03)*D;
  A[4][17]=(0.0)*D;
  A[4][18]=(0.0)*D;
  A[4][19]=(1.1160714285714285e-04)*D;
  A[5][0]=(6.6964285714285715e-04)*D;
  A[5][1]=(0.0)*D;
  A[5][2]=(0.0)*D;
  A[5][3]=(6.6964285714285715e-04)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(1.6071428571428570e-02)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(0.0)*D;
  A[5][8]=(0.0)*D;
  A[5][9]=(6.6964285714285715e-04)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(8.0357142857142849e-03)*D;
  A[5][12]=(0.0)*D;
  A[5][13]=(8.0357142857142849e-03)*D;
  A[5][14]=(8.0357142857142849e-03)*D;
  A[5][15]=(0.0)*D;
  A[5][16]=(-2.0089285714285712e-03)*D;
  A[5][17]=(-2.0089285714285712e-03)*D;
  A[5][18]=(-2.0089285714285712e-03)*D;
  A[5][19]=(1.3392857142857143e-03)*D;
  A[6][0]=(1.1160714285714285e-04)*D;
  A[6][1]=(-1.0044642857142856e-03)*D;
  A[6][2]=(2.0089285714285712e-03)*D;
  A[6][3]=(-4.4642857142857141e-04)*D;
  A[6][4]=(0.0)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(4.0178571428571425e-03)*D;
  A[6][7]=(-1.0044642857142856e-03)*D;
  A[6][8]=(-2.0089285714285712e-03)*D;
  A[6][9]=(2.2321428571428571e-04)*D;
  A[6][10]=(0.0)*D;
  A[6][11]=(0.0)*D;
  A[6][12]=(2.0089285714285712e-03)*D;
  A[6][13]=(-2.0089285714285712e-03)*D;
  A[6][14]=(0.0)*D;
  A[6][15]=(-1.0044642857142856e-03)*D;
  A[6][16]=(0.0)*D;
  A[6][17]=(-1.0044642857142856e-03)*D;
  A[6][18]=(0.0)*D;
  A[6][19]=(1.1160714285714285e-04)*D;
  A[7][0]=(2.2321428571428571e-04)*D;
  A[7][1]=(-1.0044642857142856e-03)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(1.1160714285714285e-04)*D;
  A[7][4]=(-2.0089285714285712e-03)*D;
  A[7][5]=(0.0)*D;
  A[7][6]=(-1.0044642857142856e-03)*D;
  A[7][7]=(4.0178571428571425e-03)*D;
  A[7][8]=(2.0089285714285712e-03)*D;
  A[7][9]=(-4.4642857142857141e-04)*D;
  A[7][10]=(-1.0044642857142856e-03)*D;
  A[7][11]=(-2.0089285714285712e-03)*D;
  A[7][12]=(0.0)*D;
  A[7][13]=(0.0)*D;
  A[7][14]=(0.0)*D;
  A[7][15]=(2.0089285714285712e-03)*D;
  A[7][16]=(0.0)*D;
  A[7][17]=(0.0)*D;
  A[7][18]=(-1.0044642857142856e-03)*D;
  A[7][19]=(1.1160714285714285e-04)*D;
  A[8][0]=(1.1160714285714285e-04)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(-1.0044642857142856e-03)*D;
  A[8][3]=(2.2321428571428571e-04)*D;
  A[8][4]=(-1.0044642857142856e-03)*D;
  A[8][5]=(0.0)*D;
  A[8][6]=(-2.0089285714285712e-03)*D;
  A[8][7]=(2.0089285714285712e-03)*D;
  A[8][8]=(4.0178571428571425e-03)*D;
  A[8][9]=(-4.4642857142857141e-04)*D;
  A[8][10]=(0.0)*D;
  A[8][11]=(-2.0089285714285712e-03)*D;
  A[8][12]=(-1.0044642857142856e-03)*D;
  A[8][13]=(0.0)*D;
  A[8][14]=(0.0)*D;
  A[8][15]=(2.0089285714285712e-03)*D;
  A[8][16]=(0.0)*D;
  A[8][17]=(0.0)*D;
  A[8][18]=(-1.0044642857142856e-03)*D;
  A[8][19]=(1.1160714285714285e-04)*D;
  A[9][0]=(7.4404761904761911e-05)*D;
  A[9][1]=(1.1160714285714285e-04)*D;
  A[9][2]=(1.1160714285714285e-04)*D;
  A[9][3]=(7.4404761904761911e-05)*D;
  A[9][4]=(2.2321428571428571e-04)*D;
  A[9][5]=(6.6964285714285715e-04)*D;
  A[9][6]=(2.2321428571428571e-04)*D;
  A[9][7]=(-4.4642857142857141e-04)*D;
  A[9][8]=(-4.4642857142857141e-04)*D;
  A[9][9]=(5.9523809523809529e-04)*D;
  A[9][10]=(1.1160714285714285e-04)*D;
  A[9][11]=(1.3392857142857143e-03)*D;
  A[9][12]=(1.1160714285714285e-04)*D;
  A[9][13]=(6.6964285714285715e-04)*D;
  A[9][14]=(6.6964285714285715e-04)*D;
  A[9][15]=(-4.4642857142857141e-04)*D;
  A[9][16]=(1.1160714285714285e-04)*D;
  A[9][17]=(1.1160714285714285e-04)*D;
  A[9][18]=(2.2321428571428571e-04)*D;
  A[9][19]=(7.4404761904761911e-05)*D;
  A[10][0]=(-4.4642857142857141e-04)*D;
  A[10][1]=(2.0089285714285712e-03)*D;
  A[10][2]=(-1.0044642857142856e-03)*D;
  A[10][3]=(1.1160714285714285e-04)*D;
  A[10][4]=(2.0089285714285712e-03)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(0.0)*D;
  A[10][7]=(-1.0044642857142856e-03)*D;
  A[10][8]=(0.0)*D;
  A[10][9]=(1.1160714285714285e-04)*D;
  A[10][10]=(4.0178571428571425e-03)*D;
  A[10][11]=(0.0)*D;
  A[10][12]=(0.0)*D;
  A[10][13]=(0.0)*D;
  A[10][14]=(-2.0089285714285712e-03)*D;
  A[10][15]=(0.0)*D;
  A[10][16]=(-2.0089285714285712e-03)*D;
  A[10][17]=(-1.0044642857142856e-03)*D;
  A[10][18]=(-1.0044642857142856e-03)*D;
  A[10][19]=(2.2321428571428571e-04)*D;
  A[11][0]=(6.6964285714285715e-04)*D;
  A[11][1]=(0.0)*D;
  A[11][2]=(0.0)*D;
  A[11][3]=(6.6964285714285715e-04)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(8.0357142857142849e-03)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(-2.0089285714285712e-03)*D;
  A[11][8]=(-2.0089285714285712e-03)*D;
  A[11][9]=(1.3392857142857143e-03)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(1.6071428571428570e-02)*D;
  A[11][12]=(0.0)*D;
  A[11][13]=(8.0357142857142849e-03)*D;
  A[11][14]=(8.0357142857142849e-03)*D;
  A[11][15]=(-2.0089285714285712e-03)*D;
  A[11][16]=(0.0)*D;
  A[11][17]=(0.0)*D;
  A[11][18]=(0.0)*D;
  A[11][19]=(6.6964285714285715e-04)*D;
  A[12][0]=(1.1160714285714285e-04)*D;
  A[12][1]=(-1.0044642857142856e-03)*D;
  A[12][2]=(2.0089285714285712e-03)*D;
  A[12][3]=(-4.4642857142857141e-04)*D;
  A[12][4]=(0.0)*D;
  A[12][5]=(0.0)*D;
  A[12][6]=(2.0089285714285712e-03)*D;
  A[12][7]=(0.0)*D;
  A[12][8]=(-1.0044642857142856e-03)*D;
  A[12][9]=(1.1160714285714285e-04)*D;
  A[12][10]=(0.0)*D;
  A[12][11]=(0.0)*D;
  A[12][12]=(4.0178571428571425e-03)*D;
  A[12][13]=(-2.0089285714285712e-03)*D;
  A[12][14]=(0.0)*D;
  A[12][15]=(0.0)*D;
  A[12][16]=(-1.0044642857142856e-03)*D;
  A[12][17]=(-2.0089285714285712e-03)*D;
  A[12][18]=(-1.0044642857142856e-03)*D;
  A[12][19]=(2.2321428571428571e-04)*D;
  A[13][0]=(6.6964285714285715e-04)*D;
  A[13][1]=(0.0)*D;
  A[13][2]=(-2.0089285714285712e-03)*D;
  A[13][3]=(1.3392857142857143e-03)*D;
  A[13][4]=(0.0)*D;
  A[13][5]=(8.0357142857142849e-03)*D;
  A[13][6]=(-2.0089285714285712e-03)*D;
  A[13][7]=(0.0)*D;
  A[13][8]=(0.0)*D;
  A[13][9]=(6.6964285714285715e-04)*D;
  A[13][10]=(0.0)*D;
  A[13][11]=(8.0357142857142849e-03)*D;
  A[13][12]=(-2.0089285714285712e-03)*D;
  A[13][13]=(1.6071428571428570e-02)*D;
  A[13][14]=(8.0357142857142849e-03)*D;
  A[13][15]=(0.0)*D;
  A[13][16]=(0.0)*D;
  A[13][17]=(0.0)*D;
  A[13][18]=(0.0)*D;
  A[13][19]=(6.6964285714285715e-04)*D;
  A[14][0]=(1.3392857142857143e-03)*D;
  A[14][1]=(-2.0089285714285712e-03)*D;
  A[14][2]=(0.0)*D;
  A[14][3]=(6.6964285714285715e-04)*D;
  A[14][4]=(-2.0089285714285712e-03)*D;
  A[14][5]=(8.0357142857142849e-03)*D;
  A[14][6]=(0.0)*D;
  A[14][7]=(0.0)*D;
  A[14][8]=(0.0)*D;
  A[14][9]=(6.6964285714285715e-04)*D;
  A[14][10]=(-2.0089285714285712e-03)*D;
  A[14][11]=(8.0357142857142849e-03)*D;
  A[14][12]=(0.0)*D;
  A[14][13]=(8.0357142857142849e-03)*D;
  A[14][14]=(1.6071428571428570e-02)*D;
  A[14][15]=(0.0)*D;
  A[14][16]=(0.0)*D;
  A[14][17]=(0.0)*D;
  A[14][18]=(0.0)*D;
  A[14][19]=(6.6964285714285715e-04)*D;
  A[15][0]=(1.1160714285714285e-04)*D;
  A[15][1]=(0.0)*D;
  A[15][2]=(0.0)*D;
  A[15][3]=(1.1160714285714285e-04)*D;
  A[15][4]=(-1.0044642857142856e-03)*D;
  A[15][5]=(0.0)*D;
  A[15][6]=(-1.0044642857142856e-03)*D;
  A[15][7]=(2.0089285714285712e-03)*D;
  A[15][8]=(2.0089285714285712e-03)*D;
  A[15][9]=(-4.4642857142857141e-04)*D;
  A[15][10]=(0.0)*D;
  A[15][11]=(-2.0089285714285712e-03)*D;
  A[15][12]=(0.0)*D;
  A[15][13]=(0.0)*D;
  A[15][14]=(0.0)*D;
  A[15][15]=(4.0178571428571425e-03)*D;
  A[15][16]=(-1.0044642857142856e-03)*D;
  A[15][17]=(-1.0044642857142856e-03)*D;
  A[15][18]=(-2.0089285714285712e-03)*D;
  A[15][19]=(2.2321428571428571e-04)*D;
  A[16][0]=(2.2321428571428571e-04)*D;
  A[16][1]=(-1.0044642857142856e-03)*D;
  A[16][2]=(0.0)*D;
  A[16][3]=(1.1160714285714285e-04)*D;
  A[16][4]=(-1.0044642857142856e-03)*D;
  A[16][5]=(-2.0089285714285712e-03)*D;
  A[16][6]=(0.0)*D;
  A[16][7]=(0.0)*D;
  A[16][8]=(0.0)*D;
  A[16][9]=(1.1160714285714285e-04)*D;
  A[16][10]=(-2.0089285714285712e-03)*D;
  A[16][11]=(0.0)*D;
  A[16][12]=(-1.0044642857142856e-03)*D;
  A[16][13]=(0.0)*D;
  A[16][14]=(0.0)*D;
  A[16][15]=(-1.0044642857142856e-03)*D;
  A[16][16]=(4.0178571428571425e-03)*D;
  A[16][17]=(2.0089285714285712e-03)*D;
  A[16][18]=(2.0089285714285712e-03)*D;
  A[16][19]=(-4.4642857142857141e-04)*D;
  A[17][0]=(1.1160714285714285e-04)*D;
  A[17][1]=(0.0)*D;
  A[17][2]=(-1.0044642857142856e-03)*D;
  A[17][3]=(2.2321428571428571e-04)*D;
  A[17][4]=(0.0)*D;
  A[17][5]=(-2.0089285714285712e-03)*D;
  A[17][6]=(-1.0044642857142856e-03)*D;
  A[17][7]=(0.0)*D;
  A[17][8]=(0.0)*D;
  A[17][9]=(1.1160714285714285e-04)*D;
  A[17][10]=(-1.0044642857142856e-03)*D;
  A[17][11]=(0.0)*D;
  A[17][12]=(-2.0089285714285712e-03)*D;
  A[17][13]=(0.0)*D;
  A[17][14]=(0.0)*D;
  A[17][15]=(-1.0044642857142856e-03)*D;
  A[17][16]=(2.0089285714285712e-03)*D;
  A[17][17]=(4.0178571428571425e-03)*D;
  A[17][18]=(2.0089285714285712e-03)*D;
  A[17][19]=(-4.4642857142857141e-04)*D;
  A[18][0]=(1.1160714285714285e-04)*D;
  A[18][1]=(0.0)*D;
  A[18][2]=(0.0)*D;
  A[18][3]=(1.1160714285714285e-04)*D;
  A[18][4]=(0.0)*D;
  A[18][5]=(-2.0089285714285712e-03)*D;
  A[18][6]=(0.0)*D;
  A[18][7]=(-1.0044642857142856e-03)*D;
  A[18][8]=(-1.0044642857142856e-03)*D;
  A[18][9]=(2.2321428571428571e-04)*D;
  A[18][10]=(-1.0044642857142856e-03)*D;
  A[18][11]=(0.0)*D;
  A[18][12]=(-1.0044642857142856e-03)*D;
  A[18][13]=(0.0)*D;
  A[18][14]=(0.0)*D;
  A[18][15]=(-2.0089285714285712e-03)*D;
  A[18][16]=(2.0089285714285712e-03)*D;
  A[18][17]=(2.0089285714285712e-03)*D;
  A[18][18]=(4.0178571428571425e-03)*D;
  A[18][19]=(-4.4642857142857141e-04)*D;
  A[19][0]=(7.4404761904761911e-05)*D;
  A[19][1]=(1.1160714285714285e-04)*D;
  A[19][2]=(1.1160714285714285e-04)*D;
  A[19][3]=(7.4404761904761911e-05)*D;
  A[19][4]=(1.1160714285714285e-04)*D;
  A[19][5]=(1.3392857142857143e-03)*D;
  A[19][6]=(1.1160714285714285e-04)*D;
  A[19][7]=(1.1160714285714285e-04)*D;
  A[19][8]=(1.1160714285714285e-04)*D;
  A[19][9]=(7.4404761904761911e-05)*D;
  A[19][10]=(2.2321428571428571e-04)*D;
  A[19][11]=(6.6964285714285715e-04)*D;
  A[19][12]=(2.2321428571428571e-04)*D;
  A[19][13]=(6.6964285714285715e-04)*D;
  A[19][14]=(6.6964285714285715e-04)*D;
  A[19][15]=(2.2321428571428571e-04)*D;
  A[19][16]=(-4.4642857142857141e-04)*D;
  A[19][17]=(-4.4642857142857141e-04)*D;
  A[19][18]=(-4.4642857142857141e-04)*D;
  A[19][19]=(5.9523809523809529e-04)*D;
  int nbf = 20;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 1 ---------------*/


  
void matrix_factory_mass_3D_with_c_order1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3], double c){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[4];
  double dof0[3];
  dof0[0]=x0;  dof0[1]=y0;  dof0[2]=z0;
  ptv pdof0(3,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[3];
  dof1[0]=G00+x0;  dof1[1]=y0+G10;  dof1[2]=z0+G20;
  ptv pdof1(3,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[3];
  dof2[0]=G01+x0;  dof2[1]=y0+G11;  dof2[2]=G21+z0;
  ptv pdof2(3,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[3];
  dof3[0]=G02+x0;  dof3[1]=G12+y0;  dof3[2]=G22+z0;
  ptv pdof3(3,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

 
  double A[4][4];
  A[0][0]=(1.6666666666666666e-02*c)*D;
  A[0][1]=(8.3333333333333332e-03*c)*D;
  A[0][2]=(8.3333333333333332e-03*c)*D;
  A[0][3]=(8.3333333333333332e-03*c)*D;
  A[1][0]=(8.3333333333333332e-03*c)*D;
  A[1][1]=(1.6666666666666666e-02*c)*D;
  A[1][2]=(8.3333333333333332e-03*c)*D;
  A[1][3]=(8.3333333333333332e-03*c)*D;
  A[2][0]=(8.3333333333333332e-03*c)*D;
  A[2][1]=(8.3333333333333332e-03*c)*D;
  A[2][2]=(1.6666666666666666e-02*c)*D;
  A[2][3]=(8.3333333333333332e-03*c)*D;
  A[3][0]=(8.3333333333333332e-03*c)*D;
  A[3][1]=(8.3333333333333332e-03*c)*D;
  A[3][2]=(8.3333333333333332e-03*c)*D;
  A[3][3]=(1.6666666666666666e-02*c)*D;
  int nbf = 4;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 ---------------*/


  
void matrix_factory_mass_3D_with_c_order2 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3], double c){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[10];
  double dof0[3];
  dof0[0]=x0;  dof0[1]=y0;  dof0[2]=z0;
  ptv pdof0(3,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[3];
  dof1[0]=G00/2.0+x0;  dof1[1]=y0+G10/2.0;  dof1[2]=z0+G20/2.0;
  ptv pdof1(3,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[3];
  dof2[0]=G00+x0;  dof2[1]=y0+G10;  dof2[2]=z0+G20;
  ptv pdof2(3,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[3];
  dof3[0]=G01/2.0+x0;  dof3[1]=y0+G11/2.0;  dof3[2]=G21/2.0+z0;
  ptv pdof3(3,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[3];
  dof4[0]=G01/2.0+G00/2.0+x0;  dof4[1]=y0+G11/2.0+G10/2.0;  dof4[2]=G21/2.0+z0+G20/2.0;
  ptv pdof4(3,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[3];
  dof5[0]=G01+x0;  dof5[1]=y0+G11;  dof5[2]=G21+z0;
  ptv pdof5(3,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[3];
  dof6[0]=G02/2.0+x0;  dof6[1]=G12/2.0+y0;  dof6[2]=G22/2.0+z0;
  ptv pdof6(3,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[3];
  dof7[0]=G02/2.0+G00/2.0+x0;  dof7[1]=G12/2.0+y0+G10/2.0;  dof7[2]=G22/2.0+z0+G20/2.0;
  ptv pdof7(3,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[3];
  dof8[0]=G02/2.0+G01/2.0+x0;  dof8[1]=G12/2.0+y0+G11/2.0;  dof8[2]=G22/2.0+G21/2.0+z0;
  ptv pdof8(3,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[3];
  dof9[0]=G02+x0;  dof9[1]=G12+y0;  dof9[2]=G22+z0;
  ptv pdof9(3,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

 
  double A[10][10];
  A[0][0]=(2.3809523809523812e-03*c)*D;
  A[0][1]=(-1.5873015873015873e-03*c)*D;
  A[0][2]=(3.9682539682539683e-04*c)*D;
  A[0][3]=(-1.5873015873015873e-03*c)*D;
  A[0][4]=(-2.3809523809523812e-03*c)*D;
  A[0][5]=(3.9682539682539683e-04*c)*D;
  A[0][6]=(-1.5873015873015873e-03*c)*D;
  A[0][7]=(-2.3809523809523812e-03*c)*D;
  A[0][8]=(-2.3809523809523812e-03*c)*D;
  A[0][9]=(3.9682539682539683e-04*c)*D;
  A[1][0]=(-1.5873015873015873e-03*c)*D;
  A[1][1]=(1.2698412698412698e-02*c)*D;
  A[1][2]=(-1.5873015873015873e-03*c)*D;
  A[1][3]=(6.3492063492063492e-03*c)*D;
  A[1][4]=(6.3492063492063492e-03*c)*D;
  A[1][5]=(-2.3809523809523812e-03*c)*D;
  A[1][6]=(6.3492063492063492e-03*c)*D;
  A[1][7]=(6.3492063492063492e-03*c)*D;
  A[1][8]=(3.1746031746031746e-03*c)*D;
  A[1][9]=(-2.3809523809523812e-03*c)*D;
  A[2][0]=(3.9682539682539683e-04*c)*D;
  A[2][1]=(-1.5873015873015873e-03*c)*D;
  A[2][2]=(2.3809523809523812e-03*c)*D;
  A[2][3]=(-2.3809523809523812e-03*c)*D;
  A[2][4]=(-1.5873015873015873e-03*c)*D;
  A[2][5]=(3.9682539682539683e-04*c)*D;
  A[2][6]=(-2.3809523809523812e-03*c)*D;
  A[2][7]=(-1.5873015873015873e-03*c)*D;
  A[2][8]=(-2.3809523809523812e-03*c)*D;
  A[2][9]=(3.9682539682539683e-04*c)*D;
  A[3][0]=(-1.5873015873015873e-03*c)*D;
  A[3][1]=(6.3492063492063492e-03*c)*D;
  A[3][2]=(-2.3809523809523812e-03*c)*D;
  A[3][3]=(1.2698412698412698e-02*c)*D;
  A[3][4]=(6.3492063492063492e-03*c)*D;
  A[3][5]=(-1.5873015873015873e-03*c)*D;
  A[3][6]=(6.3492063492063492e-03*c)*D;
  A[3][7]=(3.1746031746031746e-03*c)*D;
  A[3][8]=(6.3492063492063492e-03*c)*D;
  A[3][9]=(-2.3809523809523812e-03*c)*D;
  A[4][0]=(-2.3809523809523812e-03*c)*D;
  A[4][1]=(6.3492063492063492e-03*c)*D;
  A[4][2]=(-1.5873015873015873e-03*c)*D;
  A[4][3]=(6.3492063492063492e-03*c)*D;
  A[4][4]=(1.2698412698412698e-02*c)*D;
  A[4][5]=(-1.5873015873015873e-03*c)*D;
  A[4][6]=(3.1746031746031746e-03*c)*D;
  A[4][7]=(6.3492063492063492e-03*c)*D;
  A[4][8]=(6.3492063492063492e-03*c)*D;
  A[4][9]=(-2.3809523809523812e-03*c)*D;
  A[5][0]=(3.9682539682539683e-04*c)*D;
  A[5][1]=(-2.3809523809523812e-03*c)*D;
  A[5][2]=(3.9682539682539683e-04*c)*D;
  A[5][3]=(-1.5873015873015873e-03*c)*D;
  A[5][4]=(-1.5873015873015873e-03*c)*D;
  A[5][5]=(2.3809523809523812e-03*c)*D;
  A[5][6]=(-2.3809523809523812e-03*c)*D;
  A[5][7]=(-2.3809523809523812e-03*c)*D;
  A[5][8]=(-1.5873015873015873e-03*c)*D;
  A[5][9]=(3.9682539682539683e-04*c)*D;
  A[6][0]=(-1.5873015873015873e-03*c)*D;
  A[6][1]=(6.3492063492063492e-03*c)*D;
  A[6][2]=(-2.3809523809523812e-03*c)*D;
  A[6][3]=(6.3492063492063492e-03*c)*D;
  A[6][4]=(3.1746031746031746e-03*c)*D;
  A[6][5]=(-2.3809523809523812e-03*c)*D;
  A[6][6]=(1.2698412698412698e-02*c)*D;
  A[6][7]=(6.3492063492063492e-03*c)*D;
  A[6][8]=(6.3492063492063492e-03*c)*D;
  A[6][9]=(-1.5873015873015873e-03*c)*D;
  A[7][0]=(-2.3809523809523812e-03*c)*D;
  A[7][1]=(6.3492063492063492e-03*c)*D;
  A[7][2]=(-1.5873015873015873e-03*c)*D;
  A[7][3]=(3.1746031746031746e-03*c)*D;
  A[7][4]=(6.3492063492063492e-03*c)*D;
  A[7][5]=(-2.3809523809523812e-03*c)*D;
  A[7][6]=(6.3492063492063492e-03*c)*D;
  A[7][7]=(1.2698412698412698e-02*c)*D;
  A[7][8]=(6.3492063492063492e-03*c)*D;
  A[7][9]=(-1.5873015873015873e-03*c)*D;
  A[8][0]=(-2.3809523809523812e-03*c)*D;
  A[8][1]=(3.1746031746031746e-03*c)*D;
  A[8][2]=(-2.3809523809523812e-03*c)*D;
  A[8][3]=(6.3492063492063492e-03*c)*D;
  A[8][4]=(6.3492063492063492e-03*c)*D;
  A[8][5]=(-1.5873015873015873e-03*c)*D;
  A[8][6]=(6.3492063492063492e-03*c)*D;
  A[8][7]=(6.3492063492063492e-03*c)*D;
  A[8][8]=(1.2698412698412698e-02*c)*D;
  A[8][9]=(-1.5873015873015873e-03*c)*D;
  A[9][0]=(3.9682539682539683e-04*c)*D;
  A[9][1]=(-2.3809523809523812e-03*c)*D;
  A[9][2]=(3.9682539682539683e-04*c)*D;
  A[9][3]=(-2.3809523809523812e-03*c)*D;
  A[9][4]=(-2.3809523809523812e-03*c)*D;
  A[9][5]=(3.9682539682539683e-04*c)*D;
  A[9][6]=(-1.5873015873015873e-03*c)*D;
  A[9][7]=(-1.5873015873015873e-03*c)*D;
  A[9][8]=(-1.5873015873015873e-03*c)*D;
  A[9][9]=(2.3809523809523812e-03*c)*D;
  int nbf = 10;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 ---------------*/


  
void matrix_factory_mass_3D_with_c_order3 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3], double c){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[20];
  double dof0[3];
  dof0[0]=x0;  dof0[1]=y0;  dof0[2]=z0;
  ptv pdof0(3,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[3];
  dof1[0]=G00/3.0+x0;  dof1[1]=y0+G10/3.0;  dof1[2]=z0+G20/3.0;
  ptv pdof1(3,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[3];
  dof2[0]=(2.0/3.0)*G00+x0;  dof2[1]=y0+(2.0/3.0)*G10;  dof2[2]=z0+(2.0/3.0)*G20;
  ptv pdof2(3,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[3];
  dof3[0]=G00+x0;  dof3[1]=y0+G10;  dof3[2]=z0+G20;
  ptv pdof3(3,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[3];
  dof4[0]=G01/3.0+x0;  dof4[1]=y0+G11/3.0;  dof4[2]=G21/3.0+z0;
  ptv pdof4(3,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[3];
  dof5[0]=G01/3.0+G00/3.0+x0;  dof5[1]=y0+G11/3.0+G10/3.0;  dof5[2]=G21/3.0+z0+G20/3.0;
  ptv pdof5(3,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[3];
  dof6[0]=G01/3.0+(2.0/3.0)*G00+x0;  dof6[1]=y0+G11/3.0+(2.0/3.0)*G10;  dof6[2]=G21/3.0+z0+(2.0/3.0)*G20;
  ptv pdof6(3,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[3];
  dof7[0]=(2.0/3.0)*G01+x0;  dof7[1]=y0+(2.0/3.0)*G11;  dof7[2]=(2.0/3.0)*G21+z0;
  ptv pdof7(3,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[3];
  dof8[0]=(2.0/3.0)*G01+G00/3.0+x0;  dof8[1]=y0+(2.0/3.0)*G11+G10/3.0;  dof8[2]=(2.0/3.0)*G21+z0+G20/3.0;
  ptv pdof8(3,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[3];
  dof9[0]=G01+x0;  dof9[1]=y0+G11;  dof9[2]=G21+z0;
  ptv pdof9(3,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[3];
  dof10[0]=G02/3.0+x0;  dof10[1]=G12/3.0+y0;  dof10[2]=G22/3.0+z0;
  ptv pdof10(3,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[3];
  dof11[0]=G02/3.0+G00/3.0+x0;  dof11[1]=G12/3.0+y0+G10/3.0;  dof11[2]=G22/3.0+z0+G20/3.0;
  ptv pdof11(3,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[3];
  dof12[0]=G02/3.0+(2.0/3.0)*G00+x0;  dof12[1]=G12/3.0+y0+(2.0/3.0)*G10;  dof12[2]=G22/3.0+z0+(2.0/3.0)*G20;
  ptv pdof12(3,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[3];
  dof13[0]=G02/3.0+G01/3.0+x0;  dof13[1]=G12/3.0+y0+G11/3.0;  dof13[2]=G22/3.0+G21/3.0+z0;
  ptv pdof13(3,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[3];
  dof14[0]=G02/3.0+G01/3.0+G00/3.0+x0;  dof14[1]=G12/3.0+y0+G11/3.0+G10/3.0;  dof14[2]=G22/3.0+G21/3.0+z0+G20/3.0;
  ptv pdof14(3,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

  double dof15[3];
  dof15[0]=G02/3.0+(2.0/3.0)*G01+x0;  dof15[1]=G12/3.0+y0+(2.0/3.0)*G11;  dof15[2]=G22/3.0+(2.0/3.0)*G21+z0;
  ptv pdof15(3,dof15);
  iidof[15] = dof.insert_dof(element,15,pdof15);

  double dof16[3];
  dof16[0]=(2.0/3.0)*G02+x0;  dof16[1]=(2.0/3.0)*G12+y0;  dof16[2]=(2.0/3.0)*G22+z0;
  ptv pdof16(3,dof16);
  iidof[16] = dof.insert_dof(element,16,pdof16);

  double dof17[3];
  dof17[0]=(2.0/3.0)*G02+G00/3.0+x0;  dof17[1]=(2.0/3.0)*G12+y0+G10/3.0;  dof17[2]=(2.0/3.0)*G22+z0+G20/3.0;
  ptv pdof17(3,dof17);
  iidof[17] = dof.insert_dof(element,17,pdof17);

  double dof18[3];
  dof18[0]=(2.0/3.0)*G02+G01/3.0+x0;  dof18[1]=(2.0/3.0)*G12+y0+G11/3.0;  dof18[2]=(2.0/3.0)*G22+G21/3.0+z0;
  ptv pdof18(3,dof18);
  iidof[18] = dof.insert_dof(element,18,pdof18);

  double dof19[3];
  dof19[0]=G02+x0;  dof19[1]=G12+y0;  dof19[2]=G22+z0;
  ptv pdof19(3,dof19);
  iidof[19] = dof.insert_dof(element,19,pdof19);

 
  double A[20][20];
  A[0][0]=(5.9523809523809529e-04*c)*D;
  A[0][1]=(-4.4642857142857141e-04*c)*D;
  A[0][2]=(2.2321428571428571e-04*c)*D;
  A[0][3]=(7.4404761904761911e-05*c)*D;
  A[0][4]=(-4.4642857142857141e-04*c)*D;
  A[0][5]=(6.6964285714285715e-04*c)*D;
  A[0][6]=(1.1160714285714285e-04*c)*D;
  A[0][7]=(2.2321428571428571e-04*c)*D;
  A[0][8]=(1.1160714285714285e-04*c)*D;
  A[0][9]=(7.4404761904761911e-05*c)*D;
  A[0][10]=(-4.4642857142857141e-04*c)*D;
  A[0][11]=(6.6964285714285715e-04*c)*D;
  A[0][12]=(1.1160714285714285e-04*c)*D;
  A[0][13]=(6.6964285714285715e-04*c)*D;
  A[0][14]=(1.3392857142857143e-03*c)*D;
  A[0][15]=(1.1160714285714285e-04*c)*D;
  A[0][16]=(2.2321428571428571e-04*c)*D;
  A[0][17]=(1.1160714285714285e-04*c)*D;
  A[0][18]=(1.1160714285714285e-04*c)*D;
  A[0][19]=(7.4404761904761911e-05*c)*D;
  A[1][0]=(-4.4642857142857141e-04*c)*D;
  A[1][1]=(4.0178571428571425e-03*c)*D;
  A[1][2]=(-2.0089285714285712e-03*c)*D;
  A[1][3]=(2.2321428571428571e-04*c)*D;
  A[1][4]=(2.0089285714285712e-03*c)*D;
  A[1][5]=(0.0)*D;
  A[1][6]=(-1.0044642857142856e-03*c)*D;
  A[1][7]=(-1.0044642857142856e-03*c)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(1.1160714285714285e-04*c)*D;
  A[1][10]=(2.0089285714285712e-03*c)*D;
  A[1][11]=(0.0)*D;
  A[1][12]=(-1.0044642857142856e-03*c)*D;
  A[1][13]=(0.0)*D;
  A[1][14]=(-2.0089285714285712e-03*c)*D;
  A[1][15]=(0.0)*D;
  A[1][16]=(-1.0044642857142856e-03*c)*D;
  A[1][17]=(0.0)*D;
  A[1][18]=(0.0)*D;
  A[1][19]=(1.1160714285714285e-04*c)*D;
  A[2][0]=(2.2321428571428571e-04*c)*D;
  A[2][1]=(-2.0089285714285712e-03*c)*D;
  A[2][2]=(4.0178571428571425e-03*c)*D;
  A[2][3]=(-4.4642857142857141e-04*c)*D;
  A[2][4]=(-1.0044642857142856e-03*c)*D;
  A[2][5]=(0.0)*D;
  A[2][6]=(2.0089285714285712e-03*c)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(-1.0044642857142856e-03*c)*D;
  A[2][9]=(1.1160714285714285e-04*c)*D;
  A[2][10]=(-1.0044642857142856e-03*c)*D;
  A[2][11]=(0.0)*D;
  A[2][12]=(2.0089285714285712e-03*c)*D;
  A[2][13]=(-2.0089285714285712e-03*c)*D;
  A[2][14]=(0.0)*D;
  A[2][15]=(0.0)*D;
  A[2][16]=(0.0)*D;
  A[2][17]=(-1.0044642857142856e-03*c)*D;
  A[2][18]=(0.0)*D;
  A[2][19]=(1.1160714285714285e-04*c)*D;
  A[3][0]=(7.4404761904761911e-05*c)*D;
  A[3][1]=(2.2321428571428571e-04*c)*D;
  A[3][2]=(-4.4642857142857141e-04*c)*D;
  A[3][3]=(5.9523809523809529e-04*c)*D;
  A[3][4]=(1.1160714285714285e-04*c)*D;
  A[3][5]=(6.6964285714285715e-04*c)*D;
  A[3][6]=(-4.4642857142857141e-04*c)*D;
  A[3][7]=(1.1160714285714285e-04*c)*D;
  A[3][8]=(2.2321428571428571e-04*c)*D;
  A[3][9]=(7.4404761904761911e-05*c)*D;
  A[3][10]=(1.1160714285714285e-04*c)*D;
  A[3][11]=(6.6964285714285715e-04*c)*D;
  A[3][12]=(-4.4642857142857141e-04*c)*D;
  A[3][13]=(1.3392857142857143e-03*c)*D;
  A[3][14]=(6.6964285714285715e-04*c)*D;
  A[3][15]=(1.1160714285714285e-04*c)*D;
  A[3][16]=(1.1160714285714285e-04*c)*D;
  A[3][17]=(2.2321428571428571e-04*c)*D;
  A[3][18]=(1.1160714285714285e-04*c)*D;
  A[3][19]=(7.4404761904761911e-05*c)*D;
  A[4][0]=(-4.4642857142857141e-04*c)*D;
  A[4][1]=(2.0089285714285712e-03*c)*D;
  A[4][2]=(-1.0044642857142856e-03*c)*D;
  A[4][3]=(1.1160714285714285e-04*c)*D;
  A[4][4]=(4.0178571428571425e-03*c)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(0.0)*D;
  A[4][7]=(-2.0089285714285712e-03*c)*D;
  A[4][8]=(-1.0044642857142856e-03*c)*D;
  A[4][9]=(2.2321428571428571e-04*c)*D;
  A[4][10]=(2.0089285714285712e-03*c)*D;
  A[4][11]=(0.0)*D;
  A[4][12]=(0.0)*D;
  A[4][13]=(0.0)*D;
  A[4][14]=(-2.0089285714285712e-03*c)*D;
  A[4][15]=(-1.0044642857142856e-03*c)*D;
  A[4][16]=(-1.0044642857142856e-03*c)*D;
  A[4][17]=(0.0)*D;
  A[4][18]=(0.0)*D;
  A[4][19]=(1.1160714285714285e-04*c)*D;
  A[5][0]=(6.6964285714285715e-04*c)*D;
  A[5][1]=(0.0)*D;
  A[5][2]=(0.0)*D;
  A[5][3]=(6.6964285714285715e-04*c)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(1.6071428571428570e-02*c)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(0.0)*D;
  A[5][8]=(0.0)*D;
  A[5][9]=(6.6964285714285715e-04*c)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(8.0357142857142849e-03*c)*D;
  A[5][12]=(0.0)*D;
  A[5][13]=(8.0357142857142849e-03*c)*D;
  A[5][14]=(8.0357142857142849e-03*c)*D;
  A[5][15]=(0.0)*D;
  A[5][16]=(-2.0089285714285712e-03*c)*D;
  A[5][17]=(-2.0089285714285712e-03*c)*D;
  A[5][18]=(-2.0089285714285712e-03*c)*D;
  A[5][19]=(1.3392857142857143e-03*c)*D;
  A[6][0]=(1.1160714285714285e-04*c)*D;
  A[6][1]=(-1.0044642857142856e-03*c)*D;
  A[6][2]=(2.0089285714285712e-03*c)*D;
  A[6][3]=(-4.4642857142857141e-04*c)*D;
  A[6][4]=(0.0)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(4.0178571428571425e-03*c)*D;
  A[6][7]=(-1.0044642857142856e-03*c)*D;
  A[6][8]=(-2.0089285714285712e-03*c)*D;
  A[6][9]=(2.2321428571428571e-04*c)*D;
  A[6][10]=(0.0)*D;
  A[6][11]=(0.0)*D;
  A[6][12]=(2.0089285714285712e-03*c)*D;
  A[6][13]=(-2.0089285714285712e-03*c)*D;
  A[6][14]=(0.0)*D;
  A[6][15]=(-1.0044642857142856e-03*c)*D;
  A[6][16]=(0.0)*D;
  A[6][17]=(-1.0044642857142856e-03*c)*D;
  A[6][18]=(0.0)*D;
  A[6][19]=(1.1160714285714285e-04*c)*D;
  A[7][0]=(2.2321428571428571e-04*c)*D;
  A[7][1]=(-1.0044642857142856e-03*c)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(1.1160714285714285e-04*c)*D;
  A[7][4]=(-2.0089285714285712e-03*c)*D;
  A[7][5]=(0.0)*D;
  A[7][6]=(-1.0044642857142856e-03*c)*D;
  A[7][7]=(4.0178571428571425e-03*c)*D;
  A[7][8]=(2.0089285714285712e-03*c)*D;
  A[7][9]=(-4.4642857142857141e-04*c)*D;
  A[7][10]=(-1.0044642857142856e-03*c)*D;
  A[7][11]=(-2.0089285714285712e-03*c)*D;
  A[7][12]=(0.0)*D;
  A[7][13]=(0.0)*D;
  A[7][14]=(0.0)*D;
  A[7][15]=(2.0089285714285712e-03*c)*D;
  A[7][16]=(0.0)*D;
  A[7][17]=(0.0)*D;
  A[7][18]=(-1.0044642857142856e-03*c)*D;
  A[7][19]=(1.1160714285714285e-04*c)*D;
  A[8][0]=(1.1160714285714285e-04*c)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(-1.0044642857142856e-03*c)*D;
  A[8][3]=(2.2321428571428571e-04*c)*D;
  A[8][4]=(-1.0044642857142856e-03*c)*D;
  A[8][5]=(0.0)*D;
  A[8][6]=(-2.0089285714285712e-03*c)*D;
  A[8][7]=(2.0089285714285712e-03*c)*D;
  A[8][8]=(4.0178571428571425e-03*c)*D;
  A[8][9]=(-4.4642857142857141e-04*c)*D;
  A[8][10]=(0.0)*D;
  A[8][11]=(-2.0089285714285712e-03*c)*D;
  A[8][12]=(-1.0044642857142856e-03*c)*D;
  A[8][13]=(0.0)*D;
  A[8][14]=(0.0)*D;
  A[8][15]=(2.0089285714285712e-03*c)*D;
  A[8][16]=(0.0)*D;
  A[8][17]=(0.0)*D;
  A[8][18]=(-1.0044642857142856e-03*c)*D;
  A[8][19]=(1.1160714285714285e-04*c)*D;
  A[9][0]=(7.4404761904761911e-05*c)*D;
  A[9][1]=(1.1160714285714285e-04*c)*D;
  A[9][2]=(1.1160714285714285e-04*c)*D;
  A[9][3]=(7.4404761904761911e-05*c)*D;
  A[9][4]=(2.2321428571428571e-04*c)*D;
  A[9][5]=(6.6964285714285715e-04*c)*D;
  A[9][6]=(2.2321428571428571e-04*c)*D;
  A[9][7]=(-4.4642857142857141e-04*c)*D;
  A[9][8]=(-4.4642857142857141e-04*c)*D;
  A[9][9]=(5.9523809523809529e-04*c)*D;
  A[9][10]=(1.1160714285714285e-04*c)*D;
  A[9][11]=(1.3392857142857143e-03*c)*D;
  A[9][12]=(1.1160714285714285e-04*c)*D;
  A[9][13]=(6.6964285714285715e-04*c)*D;
  A[9][14]=(6.6964285714285715e-04*c)*D;
  A[9][15]=(-4.4642857142857141e-04*c)*D;
  A[9][16]=(1.1160714285714285e-04*c)*D;
  A[9][17]=(1.1160714285714285e-04*c)*D;
  A[9][18]=(2.2321428571428571e-04*c)*D;
  A[9][19]=(7.4404761904761911e-05*c)*D;
  A[10][0]=(-4.4642857142857141e-04*c)*D;
  A[10][1]=(2.0089285714285712e-03*c)*D;
  A[10][2]=(-1.0044642857142856e-03*c)*D;
  A[10][3]=(1.1160714285714285e-04*c)*D;
  A[10][4]=(2.0089285714285712e-03*c)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(0.0)*D;
  A[10][7]=(-1.0044642857142856e-03*c)*D;
  A[10][8]=(0.0)*D;
  A[10][9]=(1.1160714285714285e-04*c)*D;
  A[10][10]=(4.0178571428571425e-03*c)*D;
  A[10][11]=(0.0)*D;
  A[10][12]=(0.0)*D;
  A[10][13]=(0.0)*D;
  A[10][14]=(-2.0089285714285712e-03*c)*D;
  A[10][15]=(0.0)*D;
  A[10][16]=(-2.0089285714285712e-03*c)*D;
  A[10][17]=(-1.0044642857142856e-03*c)*D;
  A[10][18]=(-1.0044642857142856e-03*c)*D;
  A[10][19]=(2.2321428571428571e-04*c)*D;
  A[11][0]=(6.6964285714285715e-04*c)*D;
  A[11][1]=(0.0)*D;
  A[11][2]=(0.0)*D;
  A[11][3]=(6.6964285714285715e-04*c)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(8.0357142857142849e-03*c)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(-2.0089285714285712e-03*c)*D;
  A[11][8]=(-2.0089285714285712e-03*c)*D;
  A[11][9]=(1.3392857142857143e-03*c)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(1.6071428571428570e-02*c)*D;
  A[11][12]=(0.0)*D;
  A[11][13]=(8.0357142857142849e-03*c)*D;
  A[11][14]=(8.0357142857142849e-03*c)*D;
  A[11][15]=(-2.0089285714285712e-03*c)*D;
  A[11][16]=(0.0)*D;
  A[11][17]=(0.0)*D;
  A[11][18]=(0.0)*D;
  A[11][19]=(6.6964285714285715e-04*c)*D;
  A[12][0]=(1.1160714285714285e-04*c)*D;
  A[12][1]=(-1.0044642857142856e-03*c)*D;
  A[12][2]=(2.0089285714285712e-03*c)*D;
  A[12][3]=(-4.4642857142857141e-04*c)*D;
  A[12][4]=(0.0)*D;
  A[12][5]=(0.0)*D;
  A[12][6]=(2.0089285714285712e-03*c)*D;
  A[12][7]=(0.0)*D;
  A[12][8]=(-1.0044642857142856e-03*c)*D;
  A[12][9]=(1.1160714285714285e-04*c)*D;
  A[12][10]=(0.0)*D;
  A[12][11]=(0.0)*D;
  A[12][12]=(4.0178571428571425e-03*c)*D;
  A[12][13]=(-2.0089285714285712e-03*c)*D;
  A[12][14]=(0.0)*D;
  A[12][15]=(0.0)*D;
  A[12][16]=(-1.0044642857142856e-03*c)*D;
  A[12][17]=(-2.0089285714285712e-03*c)*D;
  A[12][18]=(-1.0044642857142856e-03*c)*D;
  A[12][19]=(2.2321428571428571e-04*c)*D;
  A[13][0]=(6.6964285714285715e-04*c)*D;
  A[13][1]=(0.0)*D;
  A[13][2]=(-2.0089285714285712e-03*c)*D;
  A[13][3]=(1.3392857142857143e-03*c)*D;
  A[13][4]=(0.0)*D;
  A[13][5]=(8.0357142857142849e-03*c)*D;
  A[13][6]=(-2.0089285714285712e-03*c)*D;
  A[13][7]=(0.0)*D;
  A[13][8]=(0.0)*D;
  A[13][9]=(6.6964285714285715e-04*c)*D;
  A[13][10]=(0.0)*D;
  A[13][11]=(8.0357142857142849e-03*c)*D;
  A[13][12]=(-2.0089285714285712e-03*c)*D;
  A[13][13]=(1.6071428571428570e-02*c)*D;
  A[13][14]=(8.0357142857142849e-03*c)*D;
  A[13][15]=(0.0)*D;
  A[13][16]=(0.0)*D;
  A[13][17]=(0.0)*D;
  A[13][18]=(0.0)*D;
  A[13][19]=(6.6964285714285715e-04*c)*D;
  A[14][0]=(1.3392857142857143e-03*c)*D;
  A[14][1]=(-2.0089285714285712e-03*c)*D;
  A[14][2]=(0.0)*D;
  A[14][3]=(6.6964285714285715e-04*c)*D;
  A[14][4]=(-2.0089285714285712e-03*c)*D;
  A[14][5]=(8.0357142857142849e-03*c)*D;
  A[14][6]=(0.0)*D;
  A[14][7]=(0.0)*D;
  A[14][8]=(0.0)*D;
  A[14][9]=(6.6964285714285715e-04*c)*D;
  A[14][10]=(-2.0089285714285712e-03*c)*D;
  A[14][11]=(8.0357142857142849e-03*c)*D;
  A[14][12]=(0.0)*D;
  A[14][13]=(8.0357142857142849e-03*c)*D;
  A[14][14]=(1.6071428571428570e-02*c)*D;
  A[14][15]=(0.0)*D;
  A[14][16]=(0.0)*D;
  A[14][17]=(0.0)*D;
  A[14][18]=(0.0)*D;
  A[14][19]=(6.6964285714285715e-04*c)*D;
  A[15][0]=(1.1160714285714285e-04*c)*D;
  A[15][1]=(0.0)*D;
  A[15][2]=(0.0)*D;
  A[15][3]=(1.1160714285714285e-04*c)*D;
  A[15][4]=(-1.0044642857142856e-03*c)*D;
  A[15][5]=(0.0)*D;
  A[15][6]=(-1.0044642857142856e-03*c)*D;
  A[15][7]=(2.0089285714285712e-03*c)*D;
  A[15][8]=(2.0089285714285712e-03*c)*D;
  A[15][9]=(-4.4642857142857141e-04*c)*D;
  A[15][10]=(0.0)*D;
  A[15][11]=(-2.0089285714285712e-03*c)*D;
  A[15][12]=(0.0)*D;
  A[15][13]=(0.0)*D;
  A[15][14]=(0.0)*D;
  A[15][15]=(4.0178571428571425e-03*c)*D;
  A[15][16]=(-1.0044642857142856e-03*c)*D;
  A[15][17]=(-1.0044642857142856e-03*c)*D;
  A[15][18]=(-2.0089285714285712e-03*c)*D;
  A[15][19]=(2.2321428571428571e-04*c)*D;
  A[16][0]=(2.2321428571428571e-04*c)*D;
  A[16][1]=(-1.0044642857142856e-03*c)*D;
  A[16][2]=(0.0)*D;
  A[16][3]=(1.1160714285714285e-04*c)*D;
  A[16][4]=(-1.0044642857142856e-03*c)*D;
  A[16][5]=(-2.0089285714285712e-03*c)*D;
  A[16][6]=(0.0)*D;
  A[16][7]=(0.0)*D;
  A[16][8]=(0.0)*D;
  A[16][9]=(1.1160714285714285e-04*c)*D;
  A[16][10]=(-2.0089285714285712e-03*c)*D;
  A[16][11]=(0.0)*D;
  A[16][12]=(-1.0044642857142856e-03*c)*D;
  A[16][13]=(0.0)*D;
  A[16][14]=(0.0)*D;
  A[16][15]=(-1.0044642857142856e-03*c)*D;
  A[16][16]=(4.0178571428571425e-03*c)*D;
  A[16][17]=(2.0089285714285712e-03*c)*D;
  A[16][18]=(2.0089285714285712e-03*c)*D;
  A[16][19]=(-4.4642857142857141e-04*c)*D;
  A[17][0]=(1.1160714285714285e-04*c)*D;
  A[17][1]=(0.0)*D;
  A[17][2]=(-1.0044642857142856e-03*c)*D;
  A[17][3]=(2.2321428571428571e-04*c)*D;
  A[17][4]=(0.0)*D;
  A[17][5]=(-2.0089285714285712e-03*c)*D;
  A[17][6]=(-1.0044642857142856e-03*c)*D;
  A[17][7]=(0.0)*D;
  A[17][8]=(0.0)*D;
  A[17][9]=(1.1160714285714285e-04*c)*D;
  A[17][10]=(-1.0044642857142856e-03*c)*D;
  A[17][11]=(0.0)*D;
  A[17][12]=(-2.0089285714285712e-03*c)*D;
  A[17][13]=(0.0)*D;
  A[17][14]=(0.0)*D;
  A[17][15]=(-1.0044642857142856e-03*c)*D;
  A[17][16]=(2.0089285714285712e-03*c)*D;
  A[17][17]=(4.0178571428571425e-03*c)*D;
  A[17][18]=(2.0089285714285712e-03*c)*D;
  A[17][19]=(-4.4642857142857141e-04*c)*D;
  A[18][0]=(1.1160714285714285e-04*c)*D;
  A[18][1]=(0.0)*D;
  A[18][2]=(0.0)*D;
  A[18][3]=(1.1160714285714285e-04*c)*D;
  A[18][4]=(0.0)*D;
  A[18][5]=(-2.0089285714285712e-03*c)*D;
  A[18][6]=(0.0)*D;
  A[18][7]=(-1.0044642857142856e-03*c)*D;
  A[18][8]=(-1.0044642857142856e-03*c)*D;
  A[18][9]=(2.2321428571428571e-04*c)*D;
  A[18][10]=(-1.0044642857142856e-03*c)*D;
  A[18][11]=(0.0)*D;
  A[18][12]=(-1.0044642857142856e-03*c)*D;
  A[18][13]=(0.0)*D;
  A[18][14]=(0.0)*D;
  A[18][15]=(-2.0089285714285712e-03*c)*D;
  A[18][16]=(2.0089285714285712e-03*c)*D;
  A[18][17]=(2.0089285714285712e-03*c)*D;
  A[18][18]=(4.0178571428571425e-03*c)*D;
  A[18][19]=(-4.4642857142857141e-04*c)*D;
  A[19][0]=(7.4404761904761911e-05*c)*D;
  A[19][1]=(1.1160714285714285e-04*c)*D;
  A[19][2]=(1.1160714285714285e-04*c)*D;
  A[19][3]=(7.4404761904761911e-05*c)*D;
  A[19][4]=(1.1160714285714285e-04*c)*D;
  A[19][5]=(1.3392857142857143e-03*c)*D;
  A[19][6]=(1.1160714285714285e-04*c)*D;
  A[19][7]=(1.1160714285714285e-04*c)*D;
  A[19][8]=(1.1160714285714285e-04*c)*D;
  A[19][9]=(7.4404761904761911e-05*c)*D;
  A[19][10]=(2.2321428571428571e-04*c)*D;
  A[19][11]=(6.6964285714285715e-04*c)*D;
  A[19][12]=(2.2321428571428571e-04*c)*D;
  A[19][13]=(6.6964285714285715e-04*c)*D;
  A[19][14]=(6.6964285714285715e-04*c)*D;
  A[19][15]=(2.2321428571428571e-04*c)*D;
  A[19][16]=(-4.4642857142857141e-04*c)*D;
  A[19][17]=(-4.4642857142857141e-04*c)*D;
  A[19][18]=(-4.4642857142857141e-04*c)*D;
  A[19][19]=(5.9523809523809529e-04*c)*D;
  int nbf = 20;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}
