%module (directors="1") MatrixFactory_highorder 
%feature("director");



%{

#define HAVE_ML_EPETRA

#include <dolfin.h>
#include <MatrixFactory_highorder.h>
#include <compute_conductivity.h>
#include <Numeric/arrayobject.h>
#include <iostream>
#include <DofT.h>
#include <ptv.h>





PyObject* vec2numpy(map<int,double>& vec) {

  int size = vec.size(); 
  PyArrayObject* array =  (PyArrayObject*) 
	  PyArray_FromDims(1, &size, PyArray_DOUBLE); 

  int counter; 
  double value; 
  for (map<int,double>::iterator i = vec.begin(); i != vec.end(); i++) {
    counter = (*i).first; 
    value = (*i).second; 
    *(double*)(array->data + array->strides[0]*(counter)) = value; 
  }
  return PyArray_Return(array);
}


PyObject* mesh2dof(Dof_ptv& dof, Mesh& mesh) {
  int size = mesh.numVertices(); 

  PyArrayObject* array =  (PyArrayObject*) 
	  PyArray_FromDims(1, &size, PyArray_INT); 

  if (mesh.numSpaceDim() == 2) {
    for (CellIterator cell(mesh); !cell.end(); ++cell) {

      	// Obtain vertices from dolfin mesh
        Vertex& v0 = (*cell).vertex(0);
        Vertex& v1 = (*cell).vertex(1);
        Vertex& v2 = (*cell).vertex(2);

        // Obtain  dofs in the grid 
        int  ig0 = (*cell).vertex(0).id();
        int  ig1 = (*cell).vertex(1).id();
        int  ig2 = (*cell).vertex(2).id();

      
        // Create double arrays with the data from the vertices
        double p0[2]; p0[0] = v0.coord().x;  p0[1] = v0.coord().y; 
        double p1[2]; p1[0] = v1.coord().x;  p1[1] = v1.coord().y; 
        double p2[2]; p2[0] = v2.coord().x;  p2[1] = v2.coord().y;

	ptv ptv0(2,p0);  
	ptv ptv1(2,p1);  
	ptv ptv2(2,p2);  

	int id0 = dof.glob_dof(ptv0); 
	int id1 = dof.glob_dof(ptv1); 
	int id2 = dof.glob_dof(ptv2); 


	*(int*)(array->data + array->strides[0]*(id0)) = ig0;  
	*(int*)(array->data + array->strides[0]*(id1)) = ig1;  
	*(int*)(array->data + array->strides[0]*(id2)) = ig2;  
    }
  }
  else if (mesh.numSpaceDim() == 3) {
    for (CellIterator cell(mesh); !cell.end(); ++cell) {
      // Obtain vertices from dolfin mesh
      Vertex& v0 = (*cell).vertex(0);
      Vertex& v1 = (*cell).vertex(1);
      Vertex& v2 = (*cell).vertex(2);
      Vertex& v3 = (*cell).vertex(3);

      // Obtain corresponding dofs in global matrix
      int  ig0 = (*cell).vertex(0).id();
      int  ig1 = (*cell).vertex(1).id();
      int  ig2 = (*cell).vertex(2).id();
      int  ig3 = (*cell).vertex(3).id();


      // Create double arrays with the data from the vertices
      double p0[3]; p0[0] = v0.coord().x;  p0[1] = v0.coord().y; p0[2] = v0.coord().z; 
      double p1[3]; p1[0] = v1.coord().x;  p1[1] = v1.coord().y; p1[2] = v1.coord().z; 
      double p2[3]; p2[0] = v2.coord().x;  p2[1] = v2.coord().y; p2[2] = v2.coord().z;
      double p3[3]; p3[0] = v3.coord().x;  p3[1] = v3.coord().y; p3[2] = v3.coord().z; 

      int id0 = dof.glob_dof(ptv(3,p0)); 
      int id1 = dof.glob_dof(ptv(3,p1)); 
      int id2 = dof.glob_dof(ptv(3,p2)); 
      int id3 = dof.glob_dof(ptv(3,p3)); 

      *(int*)(array->data + array->strides[0]*(id0)) = ig0;  
      *(int*)(array->data + array->strides[0]*(id1)) = ig1;  
      *(int*)(array->data + array->strides[0]*(id2)) = ig2;  
      *(int*)(array->data + array->strides[0]*(id3)) = ig3;  
    }
  }
  return PyArray_Return(array);
}

PyObject* dof2mesh(Dof_ptv& dof, Mesh& mesh) {
  int size = mesh.numVertices(); 

  PyArrayObject* array =  (PyArrayObject*) 
	  PyArray_FromDims(1, &size, PyArray_INT); 

  if (mesh.numSpaceDim() == 2) {
    for (CellIterator cell(mesh); !cell.end(); ++cell) {

      	// Obtain vertices from dolfin mesh
        Vertex& v0 = (*cell).vertex(0);
        Vertex& v1 = (*cell).vertex(1);
        Vertex& v2 = (*cell).vertex(2);

        // Obtain  dofs in the grid 
        int  ig0 = (*cell).vertex(0).id();
        int  ig1 = (*cell).vertex(1).id();
        int  ig2 = (*cell).vertex(2).id();

      
        // Create double arrays with the data from the vertices
        double p0[2]; p0[0] = v0.coord().x;  p0[1] = v0.coord().y; 
        double p1[2]; p1[0] = v1.coord().x;  p1[1] = v1.coord().y; 
        double p2[2]; p2[0] = v2.coord().x;  p2[1] = v2.coord().y;

	ptv ptv0(2,p0);  
	ptv ptv1(2,p1);  
	ptv ptv2(2,p2);  

	int id0 = dof.glob_dof(ptv0); 
	int id1 = dof.glob_dof(ptv1); 
	int id2 = dof.glob_dof(ptv2); 

	*(int*)(array->data + array->strides[0]*(ig0)) = id0;  
	*(int*)(array->data + array->strides[0]*(ig1)) = id1;  
	*(int*)(array->data + array->strides[0]*(ig2)) = id2;  
    }
  }
  else if (mesh.numSpaceDim() == 3) {
    for (CellIterator cell(mesh); !cell.end(); ++cell) {
      // Obtain vertices from dolfin mesh
      Vertex& v0 = (*cell).vertex(0);
      Vertex& v1 = (*cell).vertex(1);
      Vertex& v2 = (*cell).vertex(2);
      Vertex& v3 = (*cell).vertex(3);

      // Obtain corresponding dofs in global matrix
      int  ig0 = (*cell).vertex(0).id();
      int  ig1 = (*cell).vertex(1).id();
      int  ig2 = (*cell).vertex(2).id();
      int  ig3 = (*cell).vertex(3).id();


      // Create double arrays with the data from the vertices
      double p0[3]; p0[0] = v0.coord().x;  p0[1] = v0.coord().y; p0[2] = v0.coord().z; 
      double p1[3]; p1[0] = v1.coord().x;  p1[1] = v1.coord().y; p1[2] = v1.coord().z; 
      double p2[3]; p2[0] = v2.coord().x;  p2[1] = v2.coord().y; p2[2] = v2.coord().z;
      double p3[3]; p3[0] = v3.coord().x;  p3[1] = v3.coord().y; p3[2] = v3.coord().z; 

      int id0 = dof.glob_dof(ptv(3,p0)); 
      int id1 = dof.glob_dof(ptv(3,p1)); 
      int id2 = dof.glob_dof(ptv(3,p2)); 
      int id3 = dof.glob_dof(ptv(3,p3)); 

      *(int*)(array->data + array->strides[0]*(ig0)) = id0;  
      *(int*)(array->data + array->strides[0]*(ig1)) = id1;  
      *(int*)(array->data + array->strides[0]*(ig2)) = id2;  
      *(int*)(array->data + array->strides[0]*(ig3)) = id3;  
    }
  }
  return PyArray_Return(array);
}

PyObject* dof2mesh(Dof_ptv& dof, Mesh& mesh, int d) {
  int size = mesh.numVertices(); 

  PyArrayObject* array =  (PyArrayObject*) 
	  PyArray_FromDims(1, &size, PyArray_INT); 

  if (mesh.numSpaceDim() == 2) {
    for (CellIterator cell(mesh); !cell.end(); ++cell) {

      	// Obtain vertices from dolfin mesh
        Vertex& v0 = (*cell).vertex(0);
        Vertex& v1 = (*cell).vertex(1);
        Vertex& v2 = (*cell).vertex(2);

        // Obtain  dofs in the grid 
        int  ig0 = (*cell).vertex(0).id();
        int  ig1 = (*cell).vertex(1).id();
        int  ig2 = (*cell).vertex(2).id();

      
        // Create double arrays with the data from the vertices
        double p0[3]; p0[0] = v0.coord().x;  p0[1] = v0.coord().y; p0[2] = d; 
        double p1[3]; p1[0] = v1.coord().x;  p1[1] = v1.coord().y; p1[2] = d; 
        double p2[3]; p2[0] = v2.coord().x;  p2[1] = v2.coord().y; p2[2] = d; 

	ptv ptv0(3,p0);  
	ptv ptv1(3,p1);  
	ptv ptv2(3,p2);  

	int id0 = dof.glob_dof(ptv0); 
	int id1 = dof.glob_dof(ptv1); 
	int id2 = dof.glob_dof(ptv2); 

	*(int*)(array->data + array->strides[0]*(ig0)) = id0;  
	*(int*)(array->data + array->strides[0]*(ig1)) = id1;  
	*(int*)(array->data + array->strides[0]*(ig2)) = id2;  
    }
  }
  else if (mesh.numSpaceDim() == 3) {
    for (CellIterator cell(mesh); !cell.end(); ++cell) {
      // Obtain vertices from dolfin mesh
      Vertex& v0 = (*cell).vertex(0);
      Vertex& v1 = (*cell).vertex(1);
      Vertex& v2 = (*cell).vertex(2);
      Vertex& v3 = (*cell).vertex(3);

      // Obtain corresponding dofs in global matrix
      int  ig0 = (*cell).vertex(0).id();
      int  ig1 = (*cell).vertex(1).id();
      int  ig2 = (*cell).vertex(2).id();
      int  ig3 = (*cell).vertex(3).id();


      // Create double arrays with the data from the vertices
      double p0[4]; p0[0] = v0.coord().x;  p0[1] = v0.coord().y; p0[2] = v0.coord().z; p0[3] = d; 
      double p1[4]; p1[0] = v1.coord().x;  p1[1] = v1.coord().y; p1[2] = v1.coord().z; p1[3] = d;  
      double p2[4]; p2[0] = v2.coord().x;  p2[1] = v2.coord().y; p2[2] = v2.coord().z; p2[3] = d; 
      double p3[4]; p3[0] = v3.coord().x;  p3[1] = v3.coord().y; p3[2] = v3.coord().z; p3[3] = d;  

      int id0 = dof.glob_dof(ptv(4,p0)); 
      int id1 = dof.glob_dof(ptv(4,p1)); 
      int id2 = dof.glob_dof(ptv(4,p2)); 
      int id3 = dof.glob_dof(ptv(4,p3)); 

      *(int*)(array->data + array->strides[0]*(ig0)) = id0;  
      *(int*)(array->data + array->strides[0]*(ig1)) = id1;  
      *(int*)(array->data + array->strides[0]*(ig2)) = id2;  
      *(int*)(array->data + array->strides[0]*(ig3)) = id3;  
    }
  }
  return PyArray_Return(array);
}





PyObject* x_coor(Dof_ptv& dof) {
   //FIXME check size of ptv
  int size = dof.size(); 
  	  PyArrayObject* array =  (PyArrayObject*) 
	  PyArray_FromDims(1, &size, PyArray_DOUBLE); 
  for (int i=0; i< size; i++) {
   	ptv p = dof.glob_dof(i); 
    *(double*)(array->data + array->strides[0]*i) = p[0];
  }
  return PyArray_Return(array);
}

PyObject* y_coor(Dof_ptv& dof) {
   //FIXME check size of ptv
  int size = dof.size(); 
  PyArrayObject* array =  (PyArrayObject*) 
	  PyArray_FromDims(1, &size, PyArray_DOUBLE); 
  for (int i=0; i< size; i++) {
    ptv p = dof.glob_dof(i); 
    *(double*)(array->data + array->strides[0]*i) = p[1];
  }
  return PyArray_Return(array);
}

PyObject* z_coor(Dof_ptv& dof) {
   //FIXME check size of ptv
  int size = dof.size(); 
  PyArrayObject* array =  (PyArrayObject*) 
	  PyArray_FromDims(1, &size, PyArray_DOUBLE); 
  for (int i=0; i< size; i++) {
    ptv p = dof.glob_dof(i); 
    *(double*)(array->data + array->strides[0]*i) = p[2];
  }
  return PyArray_Return(array);
}

PyObject* boundary(Dof_ptv& dof, Mesh& mesh){

  Boundary boundary(mesh); 

  map<int,int> boundary_dof_map; 

  int nsd = mesh.numSpaceDim(); 
  if ( nsd == 2) {
 
    // run over all edges in the boundary 
    for (EdgeIterator edge(boundary); !edge.end(); ++edge) {

      //run over all points on the edge
      for (int k=0; k<= 1; k++) {

	// fetsch the point 
        Point& pk = edge->coord(0); 

	// create ptv from point 
        double xk = pk.x; double yk = pk.y; 
        double ppk[2]; ppk[0] = xk; ppk[1] = yk; // ppk[2] = 1.0;   
        ptv ptvk(2,ppk);   

        // fetch the global dof associated with ptv 
        int glob_dof = dof.glob_dof(ptvk); 

	// find all elements associated with the global dof 
        vector<pair<int,int> > dof_vec; 
        dof_vec = dof.glob2loc(glob_dof); 
        vector<pair<int,int> >::const_iterator iter; 

	// run over all elements associated with the global dof  
        for (iter = dof_vec.begin(); iter != dof_vec.end() ; iter++) {  
	  int e = (*iter).first;
	  int j=0; 
	  int gj = 0; 
	  // run over all local dofs in the element and check if the dof is on the boundary  
	  while ( gj >= 0 ) {
            j++; 
            gj = dof.glob_dof(e,j); 
	    if ( gj >= 0 ) {
	      ptv gptv = dof.glob_dof(gj);  
	      Point gpoint(gptv[0], gptv[1]); 
	      if ( edge->contains(gpoint))  { 
		boundary_dof_map[gj] += 1; 
		if ( boundary_dof_map[gj] == 1 ) {
		}
	      }
	    }
	  }
	}
      }
    }



  } else {
    std::cout <<"Sorry not implemented yet.. "<<std::endl; 
    // 3D 
    /*
    for (FaceIterator face(boundary); !face.end(); ++face) {
      for (int i=1; i<= face->numEdges(); i++) {
        Edge& edge = face->edge(i); 
      }
    }
    */ 
  }


  

  int size = boundary_dof_map.size(); 
  PyArrayObject* array =  (PyArrayObject*) PyArray_FromDims(1, &size, PyArray_INT); 
  map<int,int>::const_iterator bc_iter; 
  int counter =0; 
  for (bc_iter = boundary_dof_map.begin(); bc_iter != boundary_dof_map.end(); bc_iter++) {  
    *(int*)(array->data + array->strides[0]*(counter)) = (*bc_iter).first; 
    counter++; 
  }
  return PyArray_Return(array);
}

PyObject* boundary(Dof_ptv& dof, Mesh& mesh, int d){

  Boundary boundary(mesh); 

  map<int,int> boundary_dof_map; 

  int nsd = mesh.numSpaceDim(); 
  if ( nsd == 2) {
 
    // run over all edges in the boundary 
    for (EdgeIterator edge(boundary); !edge.end(); ++edge) {

      //run over all points on the edge
      for (int k=0; k<= 1; k++) {

	// fetsch the point 
        Point& pk = edge->coord(0); 

	// create ptv from point 
        double xk = pk.x; double yk = pk.y; 
        double ppk[3]; ppk[0] = xk; ppk[1] = yk;  ppk[2] = double(d);   
        ptv ptvk(3,ppk);   

        // fetch the global dof associated with ptv 
        int glob_dof = dof.glob_dof(ptvk); 

	// find all elements associated with the global dof 
        vector<pair<int,int> > dof_vec; 
        dof_vec = dof.glob2loc(glob_dof); 
        vector<pair<int,int> >::const_iterator iter; 

	// run over all elements associated with the global dof  
        for (iter = dof_vec.begin(); iter != dof_vec.end() ; iter++) {  
	  int e = (*iter).first;
	  int j=0; 
	  int gj = 0; 
	  // run over all local dofs in the element and check if the dof is on the boundary  
	  double eps = 10e-6; 
	  while ( gj >= 0 ) {
            j++; 
            gj = dof.glob_dof(e,j); 
	    if ( gj >= 0 ) {
	      ptv gptv = dof.glob_dof(gj);  
	      if ( eq(gptv[2],d,eps) ) {   
  	        Point gpoint(gptv[0], gptv[1]); 
	        if ( edge->contains(gpoint))  { 
	    	  boundary_dof_map[gj] += 1; 
		  if ( boundary_dof_map[gj] == 1 ) {
//  		  std::cout <<"gj "<<gj<<" gptv "<<gptv<<std::endl; 
		  }
	        }
	      }
	    }

	  }
	}
      }
    }


    // 3D 
    /*
    for (FaceIterator face(boundary); !face.end(); ++face) {
      for (int i=1; i<= face->numEdges(); i++) {
        Edge& edge = face->edge(i); 
      }
    }
    */ 
  }

  

  int size = boundary_dof_map.size(); 
  PyArrayObject* array =  (PyArrayObject*) PyArray_FromDims(1, &size, PyArray_INT); 
  map<int,int>::const_iterator bc_iter; 
  int counter =0; 
  for (bc_iter = boundary_dof_map.begin(); bc_iter != boundary_dof_map.end(); bc_iter++) {  
    *(int*)(array->data + array->strides[0]*(counter)) = (*bc_iter).first; 
    counter++; 
  }
  return PyArray_Return(array);
}



%}

%init%{
import_array();
%}

%feature("autodoc", "1");
%include "typemaps.i"

namespace dolfin {
    class Mesh;
};


using namespace dolfin;
using namespace std;



PyObject* dof2mesh(Dof_ptv& dof, Mesh& mesh); 

PyObject* dof2mesh(Dof_ptv& dof, Mesh& mesh, int d); 

PyObject* x_coor(Dof_ptv& dof); 

PyObject* y_coor(Dof_ptv& dof); 

PyObject* z_coor(Dof_ptv& dof); 

PyObject* boundary(Dof_ptv& dof, Mesh& mesh);
PyObject* boundary(Dof_ptv& dof, Mesh& mesh, int d);

PyObject* vec2numpy(map<int,double>& vec); 


void printDofs(Dof_ptv& dof); 


%define HAVE_ML_EPETRA 


%include typemaps.i
%include "../ScalarFunction.h" 
%include "../VectorFunction.h" 
%include "../MatrixFactory_highorder.h"
%include "../compute_conductivity.h"

%enddef HAVE_ML_EPETRA 

class ptv {
public: 
  ptv(int size_);
  virtual ~ptv(); 

  const int size() const; 
};

%extend ptv {
  void __setitem__(int i, double v) {
    (*self)[i] = v; 
  }
  
  double __getitem__(int i) {
    return (*self)[i]; 
  }
}






// vim:ft=cpp:
