#!/usr/bin/env python 


from MatrixFactory_highorder import  *
from Mesh import *
from BiCGStab import * 
from MatSparse import * 
from Numeric import *
from TensorFunc import *
from math import *
from Misc import *
from MeshList import *
from plotter import *

def f(x,y): 
  return 0.0  

def exact(x,y,mu): 
  u0 = (1-exp(-2.0/mu)) 
  u = x*(1.0-exp((y-1)/mu))/u0 
  return u 


class Vel(VectorFunction): 
  def __init__(self):
    VectorFunction.__init__(self)

  def __call__(self,p,v): 
    v[0] = 0 
    v[1] = 1



def run(l,p,mu):  
  mesh = Mesh("mesh2D.xml.gz") 
  if ( l > 0 ) : mesh.refineUniformly(l) 
  matrix_factory = MatrixFactory(mesh)
  matrix_factory.order1 = p 

  id = array([1.0,1.0])
  matrix_factory.tf = ConstTensorFunc(id)
  A  = matrix_factory.computeStiffnessMatrix()
  
  vel = Vel()
  C = matrix_factory.computeConvectionMatrix(vel)
#  C.dumpToOctave("C", "C.m") 
  A = mu*A + C 
  

  # compute mass matrix 
  M = matrix_factory.computeMassMatrix()

  # get coordinates 
  x = x_coor(matrix_factory.idof)
  y = y_coor(matrix_factory.idof)

  # create right hand side 
  b = zeros (A.n,  typecode='d') 
  for i in range(0, len(b)):   
    b[i] = f(x[i], y[i]) 

  b = M*b
  
  # enforce bc  

  boundary_ind = boundary(matrix_factory.idof, mesh)  
  (A,BC,C) = dirichlet_boundarycondition(A, boundary_ind)  

  u_bc = zeros(len(b), typecode='d') 
  for i in boundary_ind: 
    u_bc[i] = exact(x[i], y[i], mu)  

  b = BC*b - C*u_bc    
  u = u_bc.copy() 

  u = zeros(len(b), typecode='d')  
  B = FastMatPrec(A) 
  (u,iter) = precondBiCGStab(B, A, u, b, 10E-8, True) 

#  (u,iter) = BiCGStab(A, u, b, 10E-8, True) 
  


  u_exact =  zeros (A.n,  typecode='d') 
  for i in range(0, len(b)):   
    u_exact[i] = exact(x[i], y[i], mu) 


  e = zeros (A.n,  typecode='d') 
  for i in range(0, len(b)):   
    e[i] = pow((u_exact[i]- u[i]),2.0)  
  l2error = sqrt(sum(e))  


  print "n=%5d, p=%1d, l2 error =%e "%(len(u),p,l2error/len(e)) 
  print "min(u) ", min(u), " max(u) ", max(u) 
 


  meshlist = MeshList(mesh) 
  u_interpolated = zeros(len(dof2mesh(matrix_factory.idof,mesh)), typecode='d')
  u_interpolated[:]=Numeric.take(u,dof2mesh(matrix_factory.idof,mesh))
  
  p1 = plotter(meshlist, u_interpolated, min(u_interpolated) , max(u_interpolated))

  p1.update()


  


#run(3,1,1.0/200)  
#run(4,3,1.0/2)  
#raw_input()


for l in range(0,4): 
  for p in range(1,4): 
    run(l,p,1.0/2) 




