#!/usr/bin/env python 

from PyTrilinos import Epetra
from PyTrilinos import EpetraExt
from PyTrilinos import AztecOO  
from PyTrilinos import Amesos


from MatrixFactory_highorder import  *
from Mesh import *
from BiCGStab import * 
from MatSparse import * 
from Numeric import *
from TensorFunc import *
from math import *
from Misc import *
from MeshList import *
from plotter import *

def exact(x,y,mu): 
  return cos(pi*x)*cos(pi*y)  


def f(x,y,mu): 
  ff = (1+mu*2*pi*pi)*cos(pi*x)*cos(pi*y) - pi*cos(pi*x)*sin(pi*y)   
  return ff 


class Vel(VectorFunction): 
  def __init__(self):
    VectorFunction.__init__(self)

  def __call__(self,p,v): 
    v[0] = 0 
    v[1] = 1



def run(l,p,mu):  
  mesh = Mesh("mesh2D.xml.gz") 
  if ( l > 0 ) : mesh.refineUniformly(l) 
  Comm = Epetra.PyComm()
  matrix_factory = EpetraMatrixFactory(mesh,Comm)


  matrix_factory.order1 = p 

  M = matrix_factory.computeMassMatrix()
  id = array([mu,mu])
  matrix_factory.tf = ConstTensorFunc(id)
  matrix_factory.computeStiffnessMatrix(True)
  
  vel = Vel()
  A = matrix_factory.computeConvectionMatrix(vel,True)
 
  b = Epetra.Vector(matrix_factory.Map) 
  mb = Epetra.Vector(matrix_factory.Map) 
  u = Epetra.Vector(matrix_factory.Map) 
 
  # get coordinates 
  x = x_coor(matrix_factory.idof)
  y = y_coor(matrix_factory.idof)

  # create right hand side 
  for i in range(0, len(b)):   
    b[i] = f(x[i], y[i], mu) 
  M.Multiply(False,b,mb)

  Problem = Epetra.LinearProblem(A, u, mb)
  
 
  AmesosList = {
    "PrintStatus": True,
    "PrintTiming": True
  }
  Solver = Amesos.Lapack(Problem)
  Solver.SetParameters(AmesosList)
  Solver.SymbolicFactorization()
  Solver.NumericFactorization()
  ierr = Solver.Solve()
  print "Solver.Solve() return code = ", ierr
  del Solver




  u_exact = Epetra.Vector(matrix_factory.Map) 
  for i in range(0, len(u_exact)):   
    u_exact[i] = exact(x[i], y[i], mu) 


  e = Epetra.Vector(matrix_factory.Map) 
  for i in range(0, len(e)):   
    e[i] = pow((u_exact[i]- u[i]),2.0)  
  l2error = sqrt(sum(e))  


  print "n=%5d, p=%1d, l2 error =%e "%(len(u),p,l2error/len(e)) 
  print "min(u) ", min(u), " max(u) ", max(u) 

   # plot the solution 
  meshlist = MeshList(mesh) 
  u_interpolated = zeros(len(dof2mesh(matrix_factory.idof,mesh)), typecode='d')
  u_interpolated[:]=Numeric.take(u,dof2mesh(matrix_factory.idof,mesh))
  p1 = plotter(meshlist, u_interpolated, min(u_interpolated) , max(u_interpolated))
  p1.update()





  


#run(3,1,1.0/200)  
#run(4,3,1.0/2)  
#raw_input()


print "checking with mu=1/2" 
for l in range(0,4): 
  for p in range(1,4): 
    run(l,p,1.0/2) 

print "checking with mu=1/200" 
for l in range(0,4): 
  for p in range(1,4): 
    run(l,p,1.0/200) 






