#!/usr/bin/env python 

from MatrixFactory_highorder import  *
from Mesh import *
from MinRes import *
from MatSparse import * 
from BlockMatrix import *
from Numeric import *
from TensorFunc import *
from math import *
from Misc import *
from MeshList import *
from plotter import *
import sys


def fx(x,y,t): 
  #         du/dt                              conv (u)                             Lap(u)                        grad(p)  
  fx =  -pi*cos(pi*y)*sin(pi*t) - pi*cos(pi*x)*sin(pi*y)*cos(pi*t)*cos(pi*t)  +  pi*pi*cos(pi*y)*cos(pi*t)  + pi*sin(pi*x)*cos(pi*t) 
#  fx = x + 1  
  return fx  

def fy(x,y,t): 
  #      du/dt                                 conv(u)                              Lap(u)                        grad(p)  
  fy = -pi*cos(pi*x)*sin(pi*t) -  pi*cos(pi*y)*sin(pi*x)*cos(pi*t)*cos(pi*t)  +  pi*pi*cos(pi*x)*cos(pi*t)   
#  fy = y 
  return fy 

def ux_exact(x,y,t): 
  return cos(pi*y)*cos(pi*t)   
#  return y  

def uy_exact(x,y,t): 
  return cos(pi*x)*cos(pi*t)   
#  return x  

def p_exact(x,y,t): 
  return cos(pi*x)*cos(pi*t)  
#  return  -x + 0.5   


class NavierStokesSolver: 

  def create_matrices(self,mesh_string,dt,mu,l):
  
    # create mesh 
    self.mesh = Mesh(mesh_string) 
    if ( l > 0 ) : self.mesh.refineUniformly(l) 
    self.matrix_factory = MatrixFactory(self.mesh)

    print "h = ", pow(2,-(l+1))  
  
    # create matrices 
    self.matrix_factory.order1 = 2 
    self.matrix_factory.order2 = 1 
    tensorfunc  = ConstTensorFunc(array([1.0,1.0]))
    A  = self.matrix_factory.computeVectorStiffnessMatrix()
    M  = self.matrix_factory.computeVectorMassMatrix()
  #  printDofs(matrix_factory.idof, True) 
    self.dof2mesh_v0 = dof2mesh(self.matrix_factory.idof,self.mesh,0)
    self.dof2mesh_v1 = dof2mesh(self.matrix_factory.idof,self.mesh,1)
  
    B  = self.matrix_factory.computeDivMatrix()
    

    # fetch coordinates associated with degrees of freedom  
    self.x_vgrid = x_coor(self.matrix_factory.idof) 
    self.y_vgrid = y_coor(self.matrix_factory.idof) 
  
    Bt = self.matrix_factory.computeDivMatrix()
    Bt.transposed = True
    C = genZeroBlock(Bt.n)
  
    # put matrices in the block matrix 
    AA = BlockMatrix(2,2)
    AA[0,0] = (1.0/dt)*M + mu*A
    AA[1,0] = B
    AA[0,1] = Bt
    AA[1,1] = C
  
    # fetch boundary indicators 
    boundary_ind_x = boundary(self.matrix_factory.idof, self.mesh, 0)
    boundary_ind_y = boundary(self.matrix_factory.idof, self.mesh, 1)
    boundary_ind   = concatenate((boundary_ind_x, boundary_ind_y)) 
  
    # enforce boundary conditions  
    (self.AA, self.BC, self.CC) = dirichlet_boundarycondition(AA, [boundary_ind,[]])
   
    # dump matrices in matlab format 
    # AA[0,0].dumpToOctave("A", "A.m") 
    # AA[1,0].dumpToOctave("B", "B.m") 
    # AA[1,1].dumpToOctave("C", "C.m") 
    
    self.M = M 
  
  
  def create_preconditioner(self):
  
    # create preconditioners 
    self.matrix_factory.order1 = 1 
    M  = self.matrix_factory.computeMassMatrix()
    K = FastMatPrec(self.AA[0,0])
    L = FastMatPrec(M)
  
    # put preconditioners in block preconditioner
    self.prec = DiagBlockMatrix(2)
    self.prec[0,0] = K
    self.prec[1,1] = L

    self.x_pgrid = x_coor(self.matrix_factory.idof) 
    self.y_pgrid = y_coor(self.matrix_factory.idof) 
    self.dof2mesh_p = dof2mesh(self.matrix_factory.idof, self.mesh)  
 
  
  def init(self,n,m): 
  # create the solution vector 
    self.xx = BlockVector(2) 
    self.xx[0] = zeros(n, typecode='d') 
    self.xx[1] = zeros(m, typecode='d') 

  # create the previous solution vector 
    self.xx_prev = BlockVector(2) 
    self.xx_prev[0] = zeros(n, typecode='d') 
    self.xx_prev[1] = zeros(m, typecode='d') 
 
  
  def create_righthandside(self,t,dt,xx_prev): 
  
    # create right hand side 
    b0 = zeros(self.AA[0,0].n, typecode='d')       

   
    self.matrix_factory.order1 = 2 
    C = self.matrix_factory.computeVectorConvectionMatrix(xx_prev[0]) 
    v = C*xx_prev[0] 

    print "v ", v 

    

    print "time ", t 
 
    for i in range (0,self.AA[0,0].n):  
    # need to fetch degrees of freedom associated with x field or y field 
      if i%2 == 0:  
        b0[i] = fx(self.x_vgrid[i], self.y_vgrid[i], t)   
      elif i%2 == 1: 
        b0[i] = fy(self.x_vgrid[i], self.y_vgrid[i], t)   
  
    b0 = self.M*(b0 + (1.0/dt)*xx_prev[0])  - v  

  
    b1 = zeros(self.AA[1,1].n, typecode='d')   
    bb = BlockVector(2)
  
    bb[0] = b0
    bb[1] = b1
  
    # fetch boundary indicators 
    boundary_ind_x = boundary(self.matrix_factory.idof, self.mesh, 0)
    boundary_ind_y = boundary(self.matrix_factory.idof, self.mesh, 1)
    boundary_ind   = concatenate((boundary_ind_x, boundary_ind_y)) 

  
    # fill boundary conditions in u_bc 
    u_bc = zeros(self.AA[0,0].n, typecode='d')
    for i in boundary_ind_x: 
      u_bc[i] = ux_exact(self.x_vgrid[i], self.y_vgrid[i], t) 
    for i in boundary_ind_y: 
      u_bc[i] = uy_exact(self.x_vgrid[i], self.y_vgrid[i], t) 
    # dummy variable 
  
    p_bc = zeros(self.AA[1,1].n, typecode='d') 
  
  
    # create block vector for boundary conditions 
    b_bc = BlockVector(2) 
    b_bc[0] = u_bc
    b_bc[1] = p_bc


    bb = self.BC*bb - self.CC*b_bc
  
    self.bb = bb




  def set_ic(self, xx): 

    for i in range (0,self.AA[0,0].n):  
    # need to fetch degrees of freedom associated with x field or y field 
      if i%2 == 0:  
        xx[0][i] = ux_exact(self.x_vgrid[i], self.y_vgrid[i], 0)   
      elif i%2 == 1: 
        xx[0][i] = uy_exact(self.x_vgrid[i], self.y_vgrid[i], 0)   

    
 
  
  def plot(self,t): 
    xx = self.xx
    dof2mesh_v0 = self.dof2mesh_v0
    dof2mesh_v1 = self.dof2mesh_v1
    dof2mesh_p = self.dof2mesh_p
    mesh = self.mesh

    v0_mesh = take(xx[0], dof2mesh_v0)
    p  = plotter(MeshList(mesh), v0_mesh, min(v0_mesh), max(v0_mesh))
    p.update()
#    p.interactive()
    raw_input()

    v1_mesh = take(xx[0], dof2mesh_v1)
    p2 = plotter(MeshList(mesh), v1_mesh, min(v1_mesh), max(v1_mesh))
    p2.update()
#    p2.interactive()
    raw_input()

    p_mesh = take(xx[1], dof2mesh_p)
    p3 = plotter(MeshList(mesh), p_mesh, min(p_mesh), max(p_mesh))
    p3.update()
#    p3.interactive()
    raw_input()

#    e_p = zeros(self.AA[1,1].n,typecode='d')
#    xp = self.x_pgrid
#    yp = self.y_pgrid
#    for i in range(0, self.AA[1,1].n ):  
#      e_p[i] = pow(p_exact(xp[i],yp[i],t) - xx[1][i],2)   
#
#    ep_mesh = take(e_p, dof2mesh_p)
#    p4 = plotter(MeshList(mesh), ep_mesh, min(ep_mesh), max(ep_mesh))
#    p4.update()
#    p4.interactive()
#    raw_input()





  def compute_error(self,t):  

    xv = self.x_vgrid
    yv = self.y_vgrid
    xp = self.x_pgrid
    yp = self.y_pgrid
    xx = self.xx 

    e_v = zeros(self.AA[0,0].n,typecode='d')
    for i in range(0, self.AA[0,0].n):  
      if i%2 == 0:  
        e_v[i] = pow(ux_exact(xv[i],yv[i],t) - xx[0][i],2)   
      elif i%2 == 1: 
        e_v[i] = pow(uy_exact(xv[i],yv[i],t) - xx[0][i],2)   

    e_p = zeros(self.AA[1,1].n,typecode='d')
    for i in range(0, self.AA[1,1].n ):  
      e_p[i] = pow(p_exact(xp[i],yp[i],t) - xx[1][i],2)   


   
    e1 = sqrt(sum(e_v)) 
    print "l2 error of v ", e1
    e2 = sqrt(sum(e_p)) 
    print "l2 error of p ", e2



  
  def solve_system(self,mesh_string,T,n,l): 

    mu = 1
    dt = T/n 

    self.create_matrices(mesh_string,dt,mu,l)

    self.create_preconditioner() 

    self.init(self.AA[0,0].n,self.AA[1,1].n)

    self.set_ic(self.xx)
    
    print "dt ", dt 
    for k in range(1,n):    
      self.xx_prev = self.xx.copy()
      tn = k*T/n 
      print "tn ", tn 

      self.create_righthandside(tn,dt,self.xx_prev)
      (xx,iter) = precondMinRes(self.prec,self.AA, self.xx, self.bb, 1.0e-8, False, 50)
      xx[1] = xx[1] - sum(xx[1])/len(xx[1])


    print "tn final", tn 
    self.plot(tn)
    self.compute_error(tn)
  


if __name__ == '__main__':
  navier_stokes_solver = NavierStokesSolver()
  navier_stokes_solver.solve_system("mesh2D.xml.gz",0.01, 8, 1)
  navier_stokes_solver.solve_system("mesh2D.xml.gz",0.01, 8, 2)
