#!/usr/bin/env python 


from MatrixFactory_highorder import  *
from Mesh import *
from ConjGrad import * 
from BiCGStab import *
from MatSparse import * 
from Numeric import *
from TensorFunc import *
from math import *
from Misc import *
from MeshList import *
from plotter import *

# right hand side 
def f(x,y): 
  return 2*pi*pi*cos(pi*x)*cos(pi*y)  

# exact solution 
def exact(x,y): 
  return cos(pi*x)*cos(pi*y)  





def run(l,p):  
  
  #create mesh 
  mesh = Mesh("mesh2D.xml.gz") 
  if ( l > 0 ) : mesh.refineUniformly(l) 
  matrix_factory = MatrixFactory(mesh)

  #create stiffness matrix 
  matrix_factory.order1 = p 
  A = matrix_factory.computeStiffnessMatrix()

  # fetch coordinates
  x = x_coor(matrix_factory.idof)
  y = y_coor(matrix_factory.idof)
 
  # create right hand side  
  b = zeros(A.n,  typecode='d') 
  for i in range(0, len(b)):   
    b[i] = f(x[i], y[i]) 
 
  M = matrix_factory.computeMassMatrix()
  b = M*b
  
  # fill bc 
  boundary_ind = boundary(matrix_factory.idof, mesh)  
  (A,BC,C) = dirichlet_boundarycondition(A, boundary_ind)  
#  A.dumpToOctave("A", "A%d_%d.m"%(l,p)) 

  u_bc = zeros(len(b), typecode='d') 
  for i in boundary_ind: 
    u_bc[i] = exact(x[i], y[i])  
  b = BC*b - C*u_bc    

  # create solution vector 
  u = u_bc.copy() 

  # create preconditioner
  B = FastMatPrec(A) 

  # solve the problem 
  u = precondconjgrad(B, A, u, b, 10E-6, True) 

  # compute exact solution 
  u_exact =  zeros(A.n,  typecode='d') 
  for i in range(0, len(b)):   
    u_exact[i] = exact(x[i], y[i]) 

  # compute error 
  e = zeros(A.n,  typecode='d') 
  for i in range(0, len(b)):   
    e[i] = pow((u_exact[i]- u[i]),2.0)  
  l2error = sqrt(sum(e))  

  iter = 1
  print "n=%5d, p=%1d, iter=%d l2 error =%e "%(len(u),p,iter,l2error/len(e)) 
 

  # plot the solution 
  meshlist = MeshList(mesh) 
  u_interpolated = zeros(len(dof2mesh(matrix_factory.idof,mesh)), typecode='d')
  u_interpolated[:]=Numeric.take(u,dof2mesh(matrix_factory.idof,mesh))
  p1 = plotter(meshlist, u_interpolated, min(u_interpolated) , max(u_interpolated))
  p1.update()



if __name__ == '__main__': 

  for l in range(0,4): 
    for p in range(1,5): 
      run(l,p) 
  



