#!/usr/bin/env python 

from PyTrilinos import Epetra
from PyTrilinos import EpetraExt
from PyTrilinos import AztecOO  
from PyTrilinos import Amesos

from MatrixFactory_highorder import  *
from Mesh import *
from MatSparse import * 
from Numeric import *
from math import *
from Misc import *
from MeshList import *
from plotter import *
import sys

# right hand side 
def f(x,y): 
  return (1.0+2*pi*pi)*cos(pi*x)*cos(pi*y)  

# exact solution 
def exact(x,y): 
  return cos(pi*x)*cos(pi*y)  





def run(l,p):  
  
  #create mesh 
  mesh = Mesh("mesh2D.xml.gz") 
  if ( l > 0 ) : mesh.refineUniformly(l) 
  Comm = Epetra.PyComm()
  matrix_factory = EpetraMatrixFactory(mesh,Comm)

  #create stiffness matrix 
  matrix_factory.order1 = p 


  M = matrix_factory.computeMassMatrix()
  A = matrix_factory.computeStiffnessMatrix(True)
  dump_to_matlab(matrix_factory.mapfac.matrix, "A", "A.m") 

  b = Epetra.Vector(matrix_factory.Map) 
  mb = Epetra.Vector(matrix_factory.Map) 
  u = Epetra.Vector(matrix_factory.Map) 
  

  # fetch coordinates
  x = x_coor(matrix_factory.idof)
  y = y_coor(matrix_factory.idof)
 
  # create right hand side  
  for i in range(0, len(b)):   
    b[i] = f(x[i], y[i]) 

  M.Multiply(False,b,mb)

  Problem = Epetra.LinearProblem(A, u, mb)
#  Solver = AztecOO.AztecOO(Problem)
#  Solver.SetAztecOption(AztecOO.AZ_solver, AztecOO.AZ_cg)
#  Solver.SetAztecOption(AztecOO.AZ_precond, AztecOO.AZ_dom_decomp)
#  Solver.SetAztecOption(AztecOO.AZ_subdomain_solve, AztecOO.AZ_icc)
#  Solver.SetAztecOption(AztecOO.AZ_output, 16)
#  Solver.Iterate(1550, 1e-5)


  AmesosList = {
    "PrintStatus": True,
    "PrintTiming": True
  }
  Solver = Amesos.Lapack(Problem)
  Solver.SetParameters(AmesosList)
  Solver.SymbolicFactorization()
  Solver.NumericFactorization()
  ierr = Solver.Solve()
  print "Solver.Solve() return code = ", ierr
  del Solver



  # compute exact solution 
  u_exact = Epetra.Vector(matrix_factory.Map) 
  for i in range(0, len(u_exact)):   
    u_exact[i] = exact(x[i], y[i]) 

  # compute error 
  e = Epetra.Vector(matrix_factory.Map) 
  for i in range(0, len(e)):   
    e[i] = pow((u_exact[i]- u[i]),2.0)  
  l2error = sqrt(sum(e))  

  print "n=%5d, p=%1d,  l2 error =%e "%(len(u),p,l2error/len(e)) 
 

  # plot the solution 
  meshlist = MeshList(mesh) 
  u_interpolated = zeros(len(dof2mesh(matrix_factory.idof,mesh)), typecode='d')
  u_interpolated[:]=Numeric.take(u,dof2mesh(matrix_factory.idof,mesh))
  p1 = plotter(meshlist, u_interpolated, min(u_interpolated) , max(u_interpolated))
  p1.update()




  
 

if __name__ == '__main__': 
  for l in range(0,4): 
    for p in range(1,5): 
      run(l,p) 
 
  



