#!/usr/bin/env python 


"""
husk oppdatere Dof 
"""


from MatrixFactory_highorder import  *
from Mesh import *
from MinRes import *
from MatSparse import * 
from BlockMatrix import *
from Numeric import *
from TensorFunc import *
from math import *
from Misc import *
from MeshList import *
from plotter import *
import sys

def fx(x,y): 
  return pi*pi*cos(pi*y)  +  pi*sin(pi*x)  

def fy(x,y): 
  return pi*pi*cos(pi*x)   

def ux_exact(x,y): 
  return cos(pi*y)   

def uy_exact(x,y): 
  return cos(pi*x)   

def p_exact(x,y): 
  return cos(pi*x) 




def run_stokes_simulator(l): 
  # create mesh 
  mesh = Mesh("mesh2D.xml.gz") 
  if ( l > 0 ) : mesh.refineUniformly(l) 
  matrix_factory = MatrixFactory(mesh)

  # create matrices 
  matrix_factory.order1 = 2 
  matrix_factory.order2 = 1 
  A  = matrix_factory.computeVectorStiffnessMatrix()
#  printDofs(matrix_factory.idof, True) 
  dof2mesh_v0 = dof2mesh(matrix_factory.idof,mesh,0)
  dof2mesh_v1 = dof2mesh(matrix_factory.idof,mesh,1)

  B  = matrix_factory.computeDivMatrix()
#  printDofs(matrix_factory.jdof) 
  dof2mesh_p  = dof2mesh(matrix_factory.jdof,mesh)

  Bt = matrix_factory.computeDivMatrix()
  Bt.transposed = True
  C = genZeroBlock(Bt.n)

  # put matrices in the block matrix 
  AA = BlockMatrix(2,2)
  AA[0,0] = A
  AA[1,0] = B
  AA[0,1] = Bt
  AA[1,1] = C


  # fetch coordinates associated with degrees of freedom  
  x = x_coor(matrix_factory.idof) 
  y = y_coor(matrix_factory.idof) 


  # create right hand side 
  b0 = zeros(A.n, typecode='d')   
  for i in range (0,A.n):  
  # need to fetch degrees of freedom associated with x field or y field 
    if i%2 == 0:  
      b0[i] = fx(x[i], y[i])   
    elif i%2 == 1: 
      b0[i] = fy(x[i], y[i])   


  M  = matrix_factory.computeVectorMassMatrix()
  b0 = M*b0  

  b1 = zeros(C.n, typecode='d')   
  bb = BlockVector(2)
  bb[0] = b0
  bb[1] = b1

  # fetch boundary indicators 
  boundary_ind_x = boundary(matrix_factory.idof, mesh, 0)
  boundary_ind_y = boundary(matrix_factory.idof, mesh, 1)
  boundary_ind   = concatenate((boundary_ind_x, boundary_ind_y)) 

  # fill boundary conditions in u_bc 
  u_bc = zeros(A.n, typecode='d')
  for i in boundary_ind_x: 
    u_bc[i] = ux_exact(x[i], y[i]) 
  for i in boundary_ind_y: 
    u_bc[i] = uy_exact(x[i], y[i]) 


  # dummy variable 
  p_bc = zeros(C.n, typecode='d') 

  # create block vector for boundary conditions 
  b_bc = BlockVector(2) 
  b_bc[0] = u_bc
  b_bc[1] = p_bc


  # enforce boundary conditions  
  (AA, BC, CC) = dirichlet_boundarycondition(AA, [boundary_ind,[]])
  bb = BC*bb - CC*b_bc
 
  # dump matrices in matlab format 
#  AA[0,0].dumpToOctave("A", "A.m") 
#  AA[1,0].dumpToOctave("B", "B.m") 
#  AA[1,1].dumpToOctave("C", "C.m") 

  # create preconditioners 
  matrix_factory.order1 = 1 
  M  = matrix_factory.computeMassMatrix()
  K = FastMatPrec(AA[0,0])
  L = FastMatPrec(M)

  # coordinates for the p mesh
  x_p = x_coor(matrix_factory.idof) 
  y_p = y_coor(matrix_factory.idof) 

#  A.dumpToOctave("K", "K.m") 
#  M.dumpToOctave("L", "L.m") 


  # put preconditioners in block preconditioner
  prec = DiagBlockMatrix(2)
  prec[0,0] = K
  prec[1,1] = L

  # create the solution vector 
  xx = BlockVector(2) 
  xx[0] = zeros(A.n, typecode='d') 
  xx[1] = zeros(M.n, typecode='d') 

  # solve the system 
  (xx,iter) = precondMinRes(prec,AA, xx, bb, 1.0e-9, False, 50)
  xx[1] = xx[1] - sum(xx[1])/len(xx[1])



#  p  = plotter(MeshList(mesh), take(xx[0], dof2mesh_v0), min(xx[0]), max(xx[0]))
#  raw_input()

#  p2 = plotter(MeshList(mesh), take(xx[0], dof2mesh_v1), min(xx[0]), max(xx[0]))
#  raw_input()

#  p3 = plotter(MeshList(mesh), Numeric.take(xx[1], dof2mesh_p), min(xx[1]), max(xx[1]))
#  raw_input()


  e_v = zeros(A.n,typecode='d')
  for i in range(0, A.n):  
    if i%2 == 0:  
      e_v[i] = pow(ux_exact(x[i],y[i]) - xx[0][i],2)   
    elif i%2 == 1: 
      e_v[i] = pow(uy_exact(x[i],y[i]) - xx[0][i],2)   

  e_p = zeros(C.n,typecode='d')
  for i in range(0, C.n):  
    e_p[i] = pow(p_exact(x_p[i],y_p[i]) - xx[1][i],2)   

   
  e1 = sqrt(sum(e_v)) 
  print "l2 error of v ", e1
  e2 = sqrt(sum(e_p)) 
  print "l2 error of p ", e2



if __name__ == '__main__': 

  for i in range(0,5): 
    run_stokes_simulator(i) 


