#include <MatrixFactory_highorder.h>
#include <ElementMatrixCalculator.h>


using namespace std;
using namespace dolfin;

void check_div(double p0[2], double p1[2], double p2[2], int vorder, int porder, bool print_dofs = false) {

  map<pair<int,int>,double>  matrix; 
  Dof_ptv idof(true,true); 
  Dof_ptv jdof(true,true); 

  int element = 1; 

  if ( vorder == 2 && porder == 1) 
    matrix_factory_div_2D_order2_1(matrix,idof,jdof,element,p0,p1,p2); 
  else if ( vorder == 2 && porder == 0) 
    matrix_factory_div_2D_order2_1(matrix,idof,jdof,element,p0,p1,p2); 
  else if ( vorder == 3 && porder == 1) 
    matrix_factory_div_2D_order3_1(matrix,idof,jdof,element,p0,p1,p2); 

  if ( print_dofs ) {
    std::cout <<"v_dofs "<<std::endl; 
    printDofs(idof); 
    std::cout <<"p_dofs "<<std::endl; 
    printDofs(jdof); 
  }

  dump_to_matlab(matrix, "B_dolfin", "B_dolfin.m"); 


}

void check_mass(double p0[2], double p1[2], double p2[2], int order, bool print_dofs = false) {

  map<pair<int,int>,double>  matrix; 
  Dof_ptv idof(true,true); 
  Dof_ptv jdof(true,true); 

  int element = 1; 

  switch(order) {
    case 1 :  matrix_factory_mass_2D_order1(matrix,idof,element,p0,p1,p2); break; 
    case 2 :  matrix_factory_mass_2D_order2(matrix,idof,element,p0,p1,p2); break; 
    case 3 :  matrix_factory_mass_2D_order3(matrix,idof,element,p0,p1,p2); break; 
    case 4 :  matrix_factory_mass_2D_order4(matrix,idof,element,p0,p1,p2); break; 
    case 5 :  matrix_factory_mass_2D_order5(matrix,idof,element,p0,p1,p2); break; 
  }

  if ( print_dofs ) {
    std::cout <<"v_dofs "<<std::endl; 
    printDofs(idof); 
    std::cout <<"p_dofs "<<std::endl; 
    printDofs(jdof); 
  }

  dump_to_matlab(matrix, "M_dolfin", "M_dolfin.m"); 

}

void check_stiffness(double p0[2], double p1[2], double p2[2], int order, bool print_dofs = false) {

  map<pair<int,int>,double>  matrix; 
  Dof_ptv idof(true,true); 
  Dof_ptv jdof(true,true); 

  int element = 1; 


  switch(order) {
    case 1 :  matrix_factory_stiffness_2D_order1(matrix,idof,element,p0,p1,p2); break; 
    case 2 :  matrix_factory_stiffness_2D_order2(matrix,idof,element,p0,p1,p2); break; 
    case 3 :  matrix_factory_stiffness_2D_order3(matrix,idof,element,p0,p1,p2); break; 
    case 4 :  matrix_factory_stiffness_2D_order4(matrix,idof,element,p0,p1,p2); break; 
    case 5 :  matrix_factory_stiffness_2D_order5(matrix,idof,element,p0,p1,p2); break; 
  }

  if ( print_dofs ) {
    std::cout <<"v_dofs "<<std::endl; 
    printDofs(idof); 
    std::cout <<"p_dofs "<<std::endl; 
    printDofs(jdof); 
  }

  dump_to_matlab(matrix, "A_dolfin", "A_dolfin.m"); 

}

void check_conv(double p0[2], double p1[2], double p2[2], double vel0, double vel1, int order, bool print_dofs = false) {

  map<pair<int,int>,double>  matrix; 
  Dof_ptv idof(true,true); 
  Dof_ptv jdof(true,true); 


  int element = 1; 

  switch(order) {
    case 1 :  matrix_factory_conv_2D_order1(matrix,idof,element,p0,p1,p2,vel0,vel1); break; 
    case 2 :  matrix_factory_conv_2D_order2(matrix,idof,element,p0,p1,p2,vel0,vel1); break; 
    case 3 :  matrix_factory_conv_2D_order3(matrix,idof,element,p0,p1,p2,vel0,vel1); break; 
    case 4 :  matrix_factory_conv_2D_order4(matrix,idof,element,p0,p1,p2,vel0,vel1); break; 
    case 5 :  matrix_factory_conv_2D_order5(matrix,idof,element,p0,p1,p2,vel0,vel1); break; 
  }

  if ( print_dofs ) {
    std::cout <<"v_dofs "<<std::endl; 
    printDofs(idof); 
    std::cout <<"p_dofs "<<std::endl; 
    printDofs(jdof); 
  }

  dump_to_matlab(matrix, "C_dolfin", "C_dolfin.m"); 


}






int main() {

  double p0[2]; p0[0] = 0.2; p0[1] = 0.2;    
  double p1[2]; p1[0] = 0.4; p1[1] = 0.2;    
  double p2[2]; p2[0] = 0.1; p2[1] = 0.3;    

  /*
  double p0[2]; p0[0] = 0.0; p0[1] = 0.0;    
  double p1[2]; p1[0] = 1.0; p1[1] = 0.0;    
  double p2[2]; p2[0] = 0.0; p2[1] = 10.0;    
  */


  check_div(p0,p1,p2,2,1); 
  check_mass(p0,p1,p2,2,true); 
  check_stiffness(p0,p1,p2,2,true); 
  double v0 = 1.0; double v1 = v1; 
  check_conv(p0,p1,p2,v0,v1,2,true); 
}

