#include <SyFi.h>
#include <iostream>
#include <utility>
#include <map>
#include <vector>
#include <iterator>
#include <fstream.h>
#include <stdlib.h>
#include <string.h>




using namespace std;
using namespace GiNaC; 
using namespace SyFi; 


void dump_to_matlab(std::map<pair<int,int>,ex> A, const char *varname, const char *s)
{
  ofstream outfile(s, ios::out);
  outfile << "%The matrix\n"; 
  std::map<std::pair<int,int>,ex>::const_iterator Ait;
  for(Ait=A.begin(); Ait!=A.end(); Ait++){
    outfile << varname << "(" << (*Ait).first.first+1  << ","
            << (*Ait).first.second+1  << ")=" << (*Ait).second << ";\n"; 
  }
}

void check_vector_conv(lst p0, lst p1, lst p2, int order, bool print_dofs = false) { 
  map<pair<int,int>, ex> A; 

  Triangle triangle(p0,p1,p2); 
  VectorLagrangeFE fe; 
  fe.set_size(2); 
  fe.set(order); 
  fe.set(triangle); 
  fe.compute_basis_functions();

  ex v = numeric(0);  
  for (int i=0; i<fe.nbf(); i++) { 
    v = (v + fe.N(i)*(i+1)).evalm(); 
  }


  pair<int,int> index; 
  for (int i=0; i< fe.nbf(); i++) {
    index.first = i; 
    ex Ni = fe.N(i); 
    for (int j=0; j< fe.nbf(); j++) {
      index.second = j;
      ex vgradNj0  = v.op(0)*diff(fe.N(j).op(0), x) + v.op(1)*diff(fe.N(j).op(0), y);    
      ex vgradNj1  = v.op(0)*diff(fe.N(j).op(1), x) + v.op(1)*diff(fe.N(j).op(1), y);    
      ex integrand = vgradNj0*Ni.op(0) + vgradNj1*Ni.op(1);   
      ex Aij       = triangle.integrate(integrand); 
      A[index] = Aij;                           // add to global matrix  
    }
  }

  if ( print_dofs) {
    for (int j=0; j< fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "CV_syfi", "CV_syfi.m"); 

}


void check_div(lst p0, lst p1, lst p2, int vorder, int porder,  bool print_dofs = false) { 

  map<pair<int,int>, ex> A; 

  Triangle triangle(p0,p1,p2); 
  VectorLagrangeFE v_fe; 
  v_fe.set_size(2); 
  v_fe.set(vorder); 
  v_fe.set(triangle); 
  v_fe.compute_basis_functions();

  LagrangeFE p_fe; 
  p_fe.set(porder); 
//  P0 p_fe; 
  p_fe.set(triangle); 
  p_fe.compute_basis_functions();


  pair<int,int> index; 
  for (int i=0; i< p_fe.nbf(); i++) {
    index.first = i; 
    for (int j=0; j< v_fe.nbf(); j++) {
      index.second= j;
      ex divV= p_fe.N(i)*div(v_fe.N(j));  
      ex Aij = triangle.integrate(divV);    
      A[index] = Aij;                           // add to global matrix  
    }
  }

  if ( print_dofs) {
    for (int j=0; j< v_fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<v_fe.dof(j)<<endl; 
    }
    for (int j=0; j< p_fe.nbf(); j++) {
      cout <<"p_dof("<<j<<")="<<p_fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "B_syfi", "B_syfi.m"); 

}

void check_div_with_Gs(lst p0, lst p1, lst p2, int vorder, int porder,  bool print_dofs = false) { 

  map<pair<int,int>, ex> A; 

  ex Ginv = symbolic_matrix(2,2,"Jinv"); 
  matrix Ginvm = ex_to<matrix>(Ginv);
  ex D = symbol("D");  

  ReferenceTriangle triangle; 
  VectorLagrangeFE v_fe; 
  v_fe.set_size(2); 
  v_fe.set(vorder); 
  v_fe.set(triangle); 
  v_fe.compute_basis_functions();

  LagrangeFE p_fe; 
  p_fe.set(porder); 
//  P0 p_fe; 
  p_fe.set(triangle); 
  p_fe.compute_basis_functions();

  pair<int,int> index; 
  lst xj = lst(x,y); 
  for (int i=0; i< p_fe.nbf(); i++) {
    index.first = i; 
    for (int j=0; j< v_fe.nbf(); j++) {
      index.second= j;
      ex divNj = numeric(0); 
      ex Nj = v_fe.N(j); 
      for (int d1=0; d1<nsd; d1++) { 
        for (int d2=0; d2<nsd; d2++) { 
          divNj += diff(Nj.op(d1),ex_to<symbol>(xj.op(d2)))*Ginvm(d1,d2); 
	}
      }
		  
      ex divV= p_fe.N(i)*divNj*D; 
      ex Aij = triangle.integrate(divV);    
      A[index] = Aij;                           // add to global matrix  
    }
  }

  if ( print_dofs) {
    for (int j=0; j< v_fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<v_fe.dof(j)<<endl; 
    }
    for (int j=0; j< p_fe.nbf(); j++) {
      cout <<"p_dof("<<j<<")="<<p_fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "BGs_syfi", "BGs_syfi.m"); 

}


void check_div_with_G(lst p0, lst p1, lst p2, int vorder, int porder,  bool print_dofs = false) { 

  map<pair<int,int>, ex> A; 

  matrix G(2,2); 
  G(0,0) = p1.op(0) - p0.op(0); 
  G(1,0) = p1.op(1) - p0.op(1); 
  G(0,1) = p2.op(0) - p0.op(0); 
  G(1,1) = p2.op(1) - p0.op(1); 
  cout <<G <<endl; 

  matrix Ginv = G.inverse().transpose(); 
  ex D = G.determinant(); 
  cout <<"Ginv "<<Ginv<<endl; 
 

  ReferenceTriangle triangle; 
  VectorLagrangeFE v_fe; 
  v_fe.set_size(2); 
  v_fe.set(vorder); 
  v_fe.set(triangle); 
  v_fe.compute_basis_functions();

  LagrangeFE p_fe; 
  p_fe.set(porder); 
//  P0 p_fe; 
  p_fe.set(triangle); 
  p_fe.compute_basis_functions();

  pair<int,int> index; 
  lst xj = lst(x,y); 
  for (int i=0; i< p_fe.nbf(); i++) {
    index.first = i; 
    for (int j=0; j< v_fe.nbf(); j++) {
      index.second= j;
      ex divNj = numeric(0); 
      ex Nj = v_fe.N(j); 
      for (int d1=0; d1<nsd; d1++) { 
        for (int d2=0; d2<nsd; d2++) { 
          divNj += diff(Nj.op(d1),ex_to<symbol>(xj.op(d2)))*Ginv(d1,d2); 
	}
      }
		  
      ex divV= p_fe.N(i)*divNj*D; 
      ex Aij = triangle.integrate(divV);    
      A[index] = Aij;                           // add to global matrix  
    }
  }

  if ( print_dofs) {
    for (int j=0; j< v_fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<v_fe.dof(j)<<endl; 
    }
    for (int j=0; j< p_fe.nbf(); j++) {
      cout <<"p_dof("<<j<<")="<<p_fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "BG_syfi", "BG_syfi.m"); 

}

void check_mass(lst p0, lst p1, lst p2, int order,  bool print_dofs = false) { 

  map<pair<int,int>, ex> A; 


  Triangle triangle(p0,p1,p2); 
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(triangle); 
  fe.compute_basis_functions();


  pair<int,int> index; 
  for (int i=0; i< fe.nbf(); i++) {
    index.first = i; 
    for (int j=0; j< fe.nbf(); j++) {
      index.second= j;
      ex mass = fe.N(i)*fe.N(j);  
      ex Aij = triangle.integrate(mass);    
      A[index] = Aij;                           // add to global matrix  
    }
  }

  if ( print_dofs) {
    for (int j=0; j< fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "M_syfi", "M_syfi.m"); 

}

void check_stiffness(lst p0, lst p1, lst p2, int order, bool print_dofs = false) { 

  map<pair<int,int>, ex> A; 


  Triangle triangle(p0,p1,p2); 
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(triangle); 
  fe.compute_basis_functions();


  pair<int,int> index; 
  for (int i=0; i< fe.nbf(); i++) {
    index.first = i; 
    for (int j=0; j< fe.nbf(); j++) {
      index.second= j;
      ex stiffness = inner(grad(fe.N(i)),grad(fe.N(j)));  
      ex Aij = triangle.integrate(stiffness);    
      A[index] = Aij;                           // add to global matrix  
    }
  }

  if ( print_dofs) {
    for (int j=0; j< fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "A_syfi", "A_syfi.m"); 

}

void check_conv(lst p0, lst p1, lst p2, lst v,  int order, bool print_dofs = false) { 

  map<pair<int,int>, ex> A; 


  Triangle triangle(p0,p1,p2); 
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(triangle); 
  fe.compute_basis_functions();


  pair<int,int> index; 
  for (int i=0; i< fe.nbf(); i++) {
    index.first = i; 
    for (int j=0; j< fe.nbf(); j++) {
      index.second= j;
      ex gradNj   = grad(fe.N(j));   
      ex VxgradNj = v.op(0)*gradNj.op(0) + v.op(1)*gradNj.op(1); 
      ex conv = VxgradNj*fe.N(i);  
      ex Aij = triangle.integrate(conv);    
      A[index] = Aij;                           
    }
  }

  if ( print_dofs) {
    for (int j=0; j< fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "C_syfi", "C_syfi.m"); 

}

void check_conv_with_G(lst p0, lst p1, lst p2, lst v,  int order, bool print_dofs = false) { 

  map<pair<int,int>, ex> A; 

  matrix G(2,2); 
  G(0,0) = p1.op(0) - p0.op(0); 
  G(1,0) = p1.op(1) - p0.op(1); 
  G(0,1) = p2.op(0) - p0.op(0); 
  G(1,1) = p2.op(1) - p0.op(1); 
  cout <<G <<endl; 

  matrix Ginv = G.inverse().transpose(); 
  ex D = G.determinant(); 
  cout <<"Ginv "<<Ginv<<endl; 
  
  ReferenceTriangle triangle;
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(triangle); 
  fe.compute_basis_functions();

  pair<int,int> index; 
  for (int i=0; i< fe.nbf(); i++) {
    index.first = i; 
    for (int j=0; j< fe.nbf(); j++) {
      index.second= j;
      ex gradNj   = grad(fe.N(j));   
         gradNj   = Ginv.mul(ex_to<matrix>(gradNj)).evalm();  
      ex VxgradNj = v.op(0)*gradNj.op(0) + v.op(1)*gradNj.op(1); 
      ex conv = VxgradNj*fe.N(i);  
      ex Aij = triangle.integrate(conv)*D;    
      A[index] = Aij;                           
    }
  }

  if ( print_dofs) {
    for (int j=0; j< fe.nbf(); j++) {
      cout <<"v_dof("<<j<<")="<<fe.dof(j)<<endl; 
    }
  }
  
  dump_to_matlab(A, "CG_syfi", "CG_syfi.m"); 

}





int main() {

  lst p0(0.2, 0.2); 
  lst p1(0.4, 0.2); 
  lst p2(0.1, 0.3); 

  /*
  lst p0(0.0, 0.0); 
  lst p1(1.0, 0.0); 
  lst p2(0.0, 10.0); 
  */

  check_vector_conv(p0,p1,p2,2); 
  check_div(p0,p1,p2, 2,1); 
  check_div_with_G(p0,p1,p2, 2,1); 
  check_div_with_Gs(p0,p1,p2, 2,1); 
  check_mass(p0,p1,p2,2,true); 
  check_stiffness(p0,p1,p2,2,true); 
  lst v = lst(1,0); 
  check_conv(p0,p1,p2,v,2,true); 
  check_conv_with_G(p0,p1,p2,v,2,true); 
}


