#!/usr/bin/python 

from swiginac import *
from SyFi import *

funcname = "matrix_factory_stiffness_2D"
nsd = 2


p0 = [0,0]; p1 = [1,0]; p2 = [0,1]
triangle = Triangle(toex(p0), toex(p1), toex(p2))

a = symbol("a"); b = symbol("b"); c = symbol("c"); d = symbol("d")
J = matrix(2,2,[a,c,b,d])
D = J.determinant()
Jinv = J.inverse()


M = symbolic_matrix(2,2,"M")


fe = LagrangeFE()
fe.set(triangle) 
fe.set(1)
fe.compute_basis_functions()
A_str =""
for i in range(1,fe.nbf()+1): 
  for j in range(1,fe.nbf()+1): 
    gradNi    = (Jinv*(grad(fe.N(i)).eval())).evalm()
    gradNj    = (Jinv*(grad(fe.N(j)).eval())).evalm()
    MxgradNi = (M*gradNi).evalm()
    integrand = inner(toex(MxgradNi), toex(gradNj))  
    Aij = triangle.integrate(integrand) 
    A_str += "  A[%d][%d]=%s*D;\n"%(i-1,j-1,Aij.eval().printc())



s = """
#include <math.h>
void %s (double A[%d][%d], double M[%d][%d],  double p0[%d], double p1[%d], double p2[%d]){



  double x0 = p0[0]; double y0 = p0[1]; 
  double x1 = p1[0]; double y1 = p1[1]; 
  double x2 = p2[0]; double y2 = p2[1]; 

  double M00 = M[0][0]; 
  double M10 = M[1][0]; 
  double M01 = M[0][1]; 
  double M11 = M[1][1]; 

  double a = x1 - x0; 
  double b = x2 - x0;  
  double c = y1 - y0; 
  double d = y2 - y0; 

  //variables

  double D = fabs(a*d - b*c);  


%s
}
"""% (funcname, fe.nbf(),fe.nbf(),nsd,nsd,nsd,nsd,nsd,A_str) 



print "writing C code to the file %s.pre"%funcname
cfile = open("%s.pre"%funcname,'w')
cfile.write(s)



