#!/usr/bin/python 

from swiginac import *
from SyFi import *
import sys


funcname = "matrix_factory_stiffness_3D"
cvar.nsd = nsd = 3

p0 = [0,0,0]; p1 = [1,0,0]; p2 = [0,1,0]; p3 = [0,0,1] 

tetrahedon = Tetrahedron(toex(p0), toex(p1), toex(p2), toex(p3))

a = symbol("a"); b = symbol("b"); c = symbol("c"); d = symbol("d") 
e = symbol("e"); f = symbol("f"); g = symbol("g"); h = symbol("h"); k = symbol("k") 

J = matrix(3,3, [a,d,g,b,e,h,c,f,k]) 
Jinv = J.inverse()

print Jinv 


M = symbolic_matrix(3,3,"M")

fe = LagrangeFE()
fe.set(tetrahedon) 
fe.set(1)
fe.compute_basis_functions()
A_str =""
for i in range(1,fe.nbf()+1): 
  for j in range(1,fe.nbf()+1): 
    gradNi    = (Jinv*(grad(fe.N(i)).eval())).evalm()
    gradNj    = (Jinv*(grad(fe.N(j)).eval())).evalm()
    MxgradNi  = (M*gradNi).evalm()
    integrand = inner(toex(MxgradNi), toex(gradNj))  
    Aij = tetrahedon.integrate(integrand) 
    A_str += "  A[%d][%d]=%s*D;\n"%(i-1,j-1,Aij.eval().printc())


s = """
#include <math.h>
void %s (double A[%d][%d], double M[%d][%d], double p0[%d], double p1[%d], double p2[%d], double p3[%d]){



  double x0 = p0[0]; double y0 = p0[1]; double z0 = p0[2];  
  double x1 = p1[0]; double y1 = p1[1]; double z1 = p1[2];  
  double x2 = p2[0]; double y2 = p2[1]; double z2 = p2[2];  
  double x3 = p3[0]; double y3 = p3[1]; double z3 = p3[2];  

  double M00 = M[0][0]; double M10 = M[1][0]; double M20 = M[2][0];  
  double M01 = M[0][1]; double M11 = M[1][1]; double M21 = M[2][1];  
  double M02 = M[0][2]; double M12 = M[1][2]; double M22 = M[2][2];  



  double a   = x1 - x0;
  double d   = y1 - y0;
  double g   = z1 - z0;

  double b   = x2 - x0;
  double e   = y2 - y0;
  double h   = z2 - z0;

  double c   = x3 - x0;
  double f   = y3 - y0;
  double k   = z3 - z0;

  //variables 

  double D = a*(e*k-f*h) - b*(d*k-f*g) + c*(d*h-g*e);   


%s
}
"""% (funcname,fe.nbf(),fe.nbf(),nsd,nsd,nsd,nsd,nsd,nsd,A_str) 



print "writing C code to the file %s.pre" %funcname
cfile = open("%s.pre"%funcname,'w')
cfile.write(s)



