#!/usr/bin/env python 


from MatrixFactory import  *
from Mesh import *
from ConjGrad import * 
from MatSparse import * 
from Numeric import *
from TensorFunc import *
from math import *
from Misc import *
from MeshList import *
from plotter import *

# right hand side 
def f(x,y): 
  return 2*pi*pi*cos(pi*x)*cos(pi*y)  

# exact solution 
def exact(x,y): 
  return cos(pi*x)*cos(pi*y)  


def run(l):  
  
  #create mesh 
  mesh = Mesh("mesh2D.xml.gz") 
  if ( l > 0 ) : mesh.refineUniformly(l) 
  matrix_factory = MatrixFactory(mesh)

  #create stiffness matrix 
  id = array([1.0,1.0])
  matrix_factory.tf = ConstTensorFunc(id)
  A = matrix_factory.computeStiffnessMatrix()

 
  # create right hand side  
  nno = mesh.numVertices()
  b = arange(nno,  typecode='d') 
  i = 0
  for coord in mesh.allCoords():
    b[i] = f(coord[0], coord[1]) 
    i += 1
 
  M = matrix_factory.computeMassMatrix()
  b = M*b
  b = b - sum(b)/len(b) 
  
  # create preconditioner
  B = FastMatPrec(A) 
 

  # create solution vector 
  u = zeros(nno,  typecode='d') 

  # solve the problem 
  u = precondconjgrad(B, A, u, b, 10E-8, True, 30) 
  u = u - sum(u)/len(u) 

  # compute exact solution 
  u_exact =  zeros(nno,  typecode='d') 
  i = 0
  for coord in mesh.allCoords():
    u_exact[i] = exact(coord[0], coord[1]) 
    i += 1

  # compute error 
  e = arange(A.n,  typecode='d') 
  for i in range(0, len(u)):   
    e[i] = pow((u_exact[i]- u[i]),2.0)  
  l2error = sqrt(sum(e))  

  print "n=%5d, l2 error =%e "%(len(u),l2error/len(e)) 
 

  # plot the solution 
  meshlist = MeshList(mesh) 
  p1 = plotter(meshlist, u, min(u) , max(u))
  p1.update()
#  raw_input()



if __name__ == '__main__': 

  for l in range(0,7): 
    run(l) 
  



