#include <MatrixFactory1D.h>
#include <ElementComputations.h>
#include <Dof_double.h>
#include <iostream>
#include <fstream>
#include <utility>
#include <map>
#include <vector>
#include <iterator>




using namespace std;

MapMatrixFactory:: MapMatrixFactory(vector<double>* mesh_) {
    mesh = mesh_;
    matrix = new map<pair<int,int>,double>();
    dof = new Dof_double(true,true); 
    order = 1; 
}

void MapMatrixFactory::computeMassMatrix(bool append_to_matrix) {
    dof->clear(); 
    matrix->clear();
    double x0, x1; 
    int element; 
    for (int i=0; i < mesh->size()-1; i++) { 
//        cout <<"i "<<i<<endl; 

        x0 = (*mesh)[i]; 
        x1 = (*mesh)[i+1]; 

//        cout <<"x0 "<<x0<<endl; 
//        cout <<"x1 "<<x1<<endl; 

        element = i; 

	    switch(order) {
	      case 1 :  mass_matrix_1D_1(*matrix,*dof,x0,x1,element); break; 
	      case 2 :  mass_matrix_1D_2(*matrix,*dof,x0,x1,element); break; 
	      case 3 :  mass_matrix_1D_3(*matrix,*dof,x0,x1,element); break; 
	      case 4 :  mass_matrix_1D_4(*matrix,*dof,x0,x1,element); break; 
	      case 5 :  mass_matrix_1D_5(*matrix,*dof,x0,x1,element); break; 
	    }

    }
    
}

void MapMatrixFactory::computeStiffnessMatrix(bool append_to_matrix) {
    dof->clear(); 
    matrix->clear();
    double x0, x1; 
    int element; 
    for (int i=0; i < mesh->size()-1; i++) { 

        x0 = (*mesh)[i]; 
        x1 = (*mesh)[i+1]; 

        element = i; 

	    switch(order) {
	      case 1 :  stiffness_matrix_1D_1(*matrix,*dof,x0,x1,element); break; 
	      case 2 :  stiffness_matrix_1D_2(*matrix,*dof,x0,x1,element); break; 
	      case 3 :  stiffness_matrix_1D_3(*matrix,*dof,x0,x1,element); break; 
	      case 4 :  stiffness_matrix_1D_4(*matrix,*dof,x0,x1,element); break; 
	      case 5 :  stiffness_matrix_1D_5(*matrix,*dof,x0,x1,element); break; 
	    }

    }
    
}


void MapMatrixFactory::computeConvMatrix(bool append_to_matrix) {
    dof->clear(); 
    matrix->clear();
    double x0, x1; 
    int element; 
    for (int i=0; i < mesh->size()-1; i++) { 

        x0 = (*mesh)[i]; 
        x1 = (*mesh)[i+1]; 

        element = i; 

	    switch(order) {
	      case 1 :  conv_matrix_1D_1(*matrix,*dof,x0,x1,element); break; 
	      case 2 :  conv_matrix_1D_2(*matrix,*dof,x0,x1,element); break; 
	      case 3 :  conv_matrix_1D_3(*matrix,*dof,x0,x1,element); break; 
	      case 4 :  conv_matrix_1D_4(*matrix,*dof,x0,x1,element); break; 
	      case 5 :  conv_matrix_1D_5(*matrix,*dof,x0,x1,element); break; 
	    }

    }
    
}
void dump_to_matlab(std::map<pair<int,int>,double>& A, const char *varname, const char *s)
{
  ofstream outfile(s, ios::out);
  outfile << "%The matrix\n"; 
  std::map<std::pair<int,int>,double>::const_iterator Ait;
  double eps = 1.0e-10; 

  for(Ait=A.begin(); Ait!=A.end(); Ait++){
    outfile << varname << "(" << (*Ait).first.first+1  << ","
            << (*Ait).first.second+1  << ")=" << (*Ait).second << ";\n"; 
  }
}

FastMatSparseMatrixFactory:: FastMatSparseMatrixFactory(vector<double>* mesh_) {
  mapfac = new MapMatrixFactory(mesh_); 
  order = 1; 
}

FastMatSparse* FastMatSparseMatrixFactory:: computeMassMatrix(bool append_to_matrix) {
  mapfac->order = order; 
  mapfac->computeMassMatrix(append_to_matrix); 
  dof = mapfac->dof; 
  return new FastMatSparse(*(mapfac->matrix)); 
}

FastMatSparse* FastMatSparseMatrixFactory:: computeStiffnessMatrix(bool append_to_matrix) {
  mapfac->order = order; 
  mapfac->computeStiffnessMatrix(append_to_matrix); 
  dof = mapfac->dof; 
  return new FastMatSparse(*(mapfac->matrix)); 
}

FastMatSparse* FastMatSparseMatrixFactory:: computeConvMatrix(bool append_to_matrix) {
  mapfac->order = order; 
  mapfac->computeConvMatrix(append_to_matrix); 
  dof = mapfac->dof; 
  return new FastMatSparse(*(mapfac->matrix)); 
}



EpetraMatrixFactory:: EpetraMatrixFactory(vector<double>* mesh_, Epetra_Comm* Comm_) {
  mapfac = new MapMatrixFactory(mesh_); 
  Comm = Comm_; 
  Map = new Epetra_Map(1, 0, *Comm); 
  order = 1; 

}

Epetra_CrsMatrix* EpetraMatrixFactory:: computeMassMatrix(bool append_to_matrix) {
  mapfac->order = order; 
  mapfac->computeMassMatrix(append_to_matrix); 
  dof = mapfac->dof; 
  delete Map; 
  Map = new Epetra_Map(dof->size(), 0, *Comm); 
  return map2epetra(*(mapfac->matrix),*Map);  
}

Epetra_CrsMatrix* EpetraMatrixFactory:: computeStiffnessMatrix(bool append_to_matrix) {
  mapfac->order = order; 
  mapfac->computeStiffnessMatrix(append_to_matrix); 
  dof = mapfac->dof; 
  delete Map; 
  Map = new Epetra_Map(dof->size(), 0, *Comm); 
  return map2epetra(*(mapfac->matrix),*Map);  
}

Epetra_CrsMatrix* EpetraMatrixFactory:: computeConvMatrix(bool append_to_matrix) {
  mapfac->order = order; 
  mapfac->computeConvMatrix(append_to_matrix); 
  dof = mapfac->dof; 
  delete Map; 
  Map = new Epetra_Map(dof->size(), 0, *Comm); 
  return map2epetra(*(mapfac->matrix),*Map);  
}

Epetra_CrsMatrix* map2epetra(map<pair<int,int>,double>& A_from, Epetra_Map& emap) { 

  std::map<std::pair<int,int>,double>::const_iterator Ait;

  Epetra_CrsMatrix* A_to = new Epetra_FECrsMatrix(Copy, emap,1); 


//  dump_to_matlab(A_from, "A", "A.m"); 

  double value; 
  vector<int> j_at_row; 
  vector<double> value_at_row; 
  Ait=A_from.begin();
  int curr_row= (*Ait).first.first;  
  int i, j; 
  int size; 

  while ( Ait != A_from.end() ) { 

    i = (*Ait).first.first;
    j = (*Ait).first.second;
    value = (*Ait).second; 

    if ( i == curr_row) { 

      j_at_row.push_back(j); 
      value_at_row.push_back(value); 

    } else { // new row! 


      // 1. Fill Epetra_CrsMatrix.
      if ( j_at_row.size() != value_at_row.size() ) { 
         //FIXME shold this be an exception ?? 
         std::cout << "This is most likely a bug in the function map2epetra" <<std::endl; 
      }

      size = j_at_row.size(); 

      // create the temporal pointer arrays Indices and Values 
      int* Indices =  new int[size]; 
      double* Values = new double[size];

      // fill Indices and Values 
      for (int d=0; d < size; d++) { 
        Indices[d] = j_at_row[d]; 
        Values[d]  = value_at_row[d];  
      }

      // insert previous row into Epetra_CrsMatrix
      A_to->InsertGlobalValues (curr_row, size, Values, Indices);

      
      // 2. Clear the structures i_at_row, j_at_row and value_at_row.
      j_at_row.clear(); 
      value_at_row.clear(); 

      // delete temporal pointer arrays 
      delete [] Indices; 
      delete [] Values;

      // 3. Insert the new values.
      j_at_row.push_back(j); 
      value_at_row.push_back(value); 

      curr_row = i; 
    }
    Ait++; 
  }

  // Finally, fill the last values 
  size = j_at_row.size(); 

  // create the temporal pointer arrays Indices and Values 
  int* Indices =  new int[size]; 
  double* Values = new double[size];

  // fill Indices and Values 
  for (int d=0; d < size; d++) { 
    Indices[d] = j_at_row[d]; 
    Values[d]  = value_at_row[d];  
  }

  // insert into Epetra_CrsMatrix
  A_to->InsertGlobalValues (curr_row, size, Values, Indices);

      
  // 2. Clear the structures i_at_row, j_at_row and value_at_row.
  j_at_row.clear(); 
  value_at_row.clear(); 

  // delete temporal pointer arrays 
  delete [] Indices; 
  delete [] Values;

  // 3. Insert the new values.
  j_at_row.push_back(j); 
  value_at_row.push_back(value); 

  A_to->FillComplete(); 

  return A_to; 
  
}

void printDofs(Dof_double& dof, bool glob2loc ){ 
  for (int i=0; i < dof.size(); i++) {
    std::cout <<"dof "<<i<<" "<<dof.glob_dof(i)<<std::endl; 
    if ( glob2loc) {
      vector<pair<int,int> > dof_vec; 
      dof_vec = dof.glob2loc(i); 
      vector<pair<int,int> >::const_iterator iter; 
      for (iter = dof_vec.begin(); iter != dof_vec.end() ; iter++) {  
    	int e = (*iter).first;
    	int j = (*iter).second; 
    	std::cout <<"local e "<<e<<" local j "<<j<<" global i "<<dof.glob_dof(e,j)<<std::endl; 
      }
    }
  }
}




