#ifndef MATRIX1D_IS_INCLUDED
#define MATRIX1D_IS_INCLUDED

#include <Dof_double.h>

#include <FastMatSparse.h>

#include <ml_config.h>
#ifdef HAVE_ML_EPETRA 
#ifdef HAVE_MPI
#include "mpi.h"
#include "Epetra_MpiComm.h"
#else
#include "Epetra_SerialComm.h"
#endif
#include <Epetra_Map.h>
#include <Epetra_FECrsMatrix.h>
#include <Epetra_ConfigDefs.h>
#endif // HAVE_ML_EPETRA 


using namespace std;


class MapMatrixFactory 
{
    public:
    vector<double> *mesh;
	Dof_double* dof; 
    map<pair<int,int>,double> *matrix;
	int order; 

    MapMatrixFactory(vector<double>* mesh_);

    void computeMassMatrix(bool append_to_matrix=false);
    void computeStiffnessMatrix(bool append_to_matrix=false);
    void computeConvMatrix(bool append_to_matrix=false);
    int size() { return dof->size(); }

};

class FastMatSparseMatrixFactory 
{
    public:
    vector<double> *mesh;
	Dof_double* dof; 
	int order; 
    MapMatrixFactory* mapfac;  

    FastMatSparseMatrixFactory(vector<double>*_mesh);

    FastMatSparse* computeMassMatrix(bool append_to_matrix=false);
    FastMatSparse* computeStiffnessMatrix(bool append_to_matrix=false);
    FastMatSparse* computeConvMatrix(bool append_to_matrix=false);
    int size() { return dof->size(); }

};



class EpetraMatrixFactory 
{
    public:
	Epetra_Comm       *Comm; 
	Epetra_Map        *Map; 
    vector<double> *mesh;
	Dof_double* dof; 
	int order; 
    MapMatrixFactory* mapfac;  

    EpetraMatrixFactory(vector<double>*_mesh, Epetra_Comm* Comm_);

    Epetra_CrsMatrix* computeMassMatrix(bool append_to_matrix=false);
    Epetra_CrsMatrix* computeStiffnessMatrix(bool append_to_matrix=false);
    Epetra_CrsMatrix* computeConvMatrix(bool append_to_matrix=false);
    int size() { return dof->size(); }

};


Epetra_CrsMatrix* map2epetra(map<pair<int,int>,double>& A_from, Epetra_Map& emap);  

void printDofs(Dof_double& dof, bool glob2loc=false); 

void dump_to_matlab(map<pair<int,int>,double>& A, const char *varname, const char *s);


#endif 
