#include <SyFi.h>

using namespace GiNaC; 

void compute_poisson_element_matrix(Polygon& domain, int order) {
  std::map<std::pair<int,int>, ex> A;
  std::pair<int,int> index;
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(domain); 
  fe.compute_basis_functions(); 
  ex ujs = symbolic_matrix(1,fe.nbf(), "u");  
  ex u;  
  for (int k=1; k<= fe.nbf(); k++)  {
      u += ujs.op(k-1)*fe.N(k);
  }



  for (int i=1; i<= fe.nbf() ; i++) {
    index.first = i; 
    ex Fi = inner(grad(u), grad(fe.N(i)));   
    for (int j=1; j<= fe.nbf() ; j++) {
      index.second = j; 
      symbol uj = ex_to<symbol>(ujs.op(j-1)); 
      ex nabla = Fi.diff(uj,1);  
      ex Aij = domain.integrate(nabla);   
      A[index] = Aij;  
    }
  }

  map<std::pair<int,int>,ex>::iterator iter; 
  for (iter = A.begin(); iter != A.end() ; iter++) {
      ex intf = (*iter).second; 
      cout <<"A["<<(*iter).first.first<<","<<(*iter).first.second<<"]="
	   <<(*iter).second<<endl; 
  }
}

void compute_nlconvdiff_element_matrix(Polygon& domain, int order) {
  std::map<std::pair<int,int>, ex> A;
  std::pair<int,int> index;
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(domain); 
  fe.compute_basis_functions(); 
  ex ujs = symbolic_matrix(1,fe.nbf(), "u");  
  ex u;  
  for (int k=1; k<= fe.nbf(); k++)  {
      u += ujs.op(k-1)*fe.N(k);
      k++;
  }


  for (int i=1; i<= fe.nbf() ; i++) {
    index.first = i; 
    ex Fi_diffusion  = inner(grad(u), grad(fe.N(i)));   
    ex Fi_convection = u*u.diff(x,1)*fe.N(i);
    ex Fi = Fi_diffusion + Fi_convection; 
    for (int j=1; j<= fe.nbf() ; j++) {
      index.second = j; 
      symbol uj = ex_to<symbol>(ujs.op(j-1)); 
      ex nabla = Fi.diff(uj,1);  
      ex Aij = domain.integrate(nabla);   
      A[index] = Aij;  
    }
  }

  map<std::pair<int,int>,ex>::iterator iter; 
  for (iter = A.begin(); iter != A.end() ; iter++) {
      ex intf = (*iter).second; 
      cout <<"A["<<(*iter).first.first<<","<<(*iter).first.second<<"]="
	   <<(*iter).second<<endl; 
  }
}




int main() {
  cout <<"2D General Numeric ----------------------------"<<endl; 
//  Triangle T2(lst(0,0,0), lst(1,0,0), lst(0,1,0), "t"); 
  Triangle T2(lst(0,0), lst(1,0), lst(0,1), "t"); 

  try { 
    compute_poisson_element_matrix(T2, 2); 
//    compute_nlconvdiff_element_matrix(T2, 2); 
  } catch (exception &p) {
        cerr << p.what() << endl;
        return 1;
  }


}

