#include <LagrangeFE.h>

using namespace GiNaC; 

void compute_VectorLagrange_element(Polygon& p, int order, int size) {
  std::map<std::pair<int,int>, ex> A;
  std::map<std::pair<int,int>, double> Aval;
  std::pair<int,int> index;
  VectorLagrangeFE fe; 
  fe.set(order); 
  fe.set(p); 
  fe.set_size(size); 
  fe.compute_basis_functions(); 
  for (int i=1; i<= fe.nbf() ; i++) {
    cout <<"fe.N("<<i<<")="<<fe.N(i)<<endl; 
    cout <<"fe.dof("<<i<<")="<<fe.dof(i)<<endl; 
  }

}


void compute_Lagrange_element(Polygon& p, int order) { 
  std::map<std::pair<int,int>, ex> A;
  std::map<std::pair<int,int>, double> Aval;
  std::pair<int,int> index;
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(p); 
  fe.compute_basis_functions(); 
  for (int i=1; i<= fe.nbf() ; i++) {
    cout <<"fe.N("<<i<<")="<<fe.N(i)<<endl; 
    cout <<"fe.dof("<<i<<")="<<fe.dof(i)<<endl; 

    /*
    // check that the dofs are correct:  
    for (int j=1; j<= fe.nbf() ; j++) {     
      ex dofj = fe.dof(j);                  
      lst subst = lst_equals(dofj, lst(x,y,z));  
      ex Ni = fe.N(i); 
      ex r = Ni.subs(subst); 

      if ( r != 1 && i == j) {
	cout <<"r "<<r<<endl; 
      } else if ( r != 0 && i != j ) { 
	cout <<"r "<<r<<endl; 
      }
    }
    */
  }
}



void check_lagrange() {
  
  symbol x0("x0"), x1("x1"), x2("x2"), x3("x3"); 
  symbol y0("y0"), y1("y1"), y2("y2"), y3("y3"); 
  symbol z0("z0"), z1("z1"), z2("z2"), z3("z3"); 

  cout <<"\n1D ----------------------------  "<<endl; 
  cout <<"Second order Lagrange element      "<<endl; 
  ReferenceLine line("l"); 
  compute_Lagrange_element(line, 2); 

  cout <<"\n1D ----------------------------  "<<endl; 
  cout <<"Second order Lagrange element      "<<endl; 
  Line line2(lst(0,0), lst(1,0),"l"); 
  compute_Lagrange_element(line2, 2); 



  cout <<"\n2D ----------------------------  "<<endl; 
  cout <<"Second order Lagrange element      "<<endl; 
  ReferenceTriangle triangle("t"); 
  compute_Lagrange_element(triangle, 2); 

  cout <<"\n2D ----------------------------  "<<endl; 
  cout <<"Second order Lagrange element on a general triangle      "<<endl; 
  Triangle T(lst(x0,y0,z0), lst(x1,y1,z1), lst(x2,y2,z2), "t"); 
  compute_Lagrange_element(T, 2); 


  cout <<"\n3D ----------------------------  "<<endl; 
  cout <<"Second order Lagrange element      "<<endl; 
  ReferenceTetrahedron tetrahedron("t"); 
  compute_Lagrange_element(tetrahedron, 2); 

  cout <<"\n2D ----------------------------         "<<endl; 
  cout <<"Second order vector Lagrange element      "<<endl; 
  cout <<"with 4 components                         "<<endl; 
  compute_VectorLagrange_element(triangle, 2, 4); 
  
}


int main() {
  check_lagrange(); 
}



