#include <LagrangeFE.h>


using namespace GiNaC; 


// could have several versions of this example 
// 1. Std example using the reference element
// 2. Example using globale element
// 3. Simple code generation, just printing out a element matrix
// 4. Code generation towards a backend FFC
// 5. With Diffpack
// 6. With ML 
//


void compute_element_matrix(Polygon& domain, int order) { 
  std::map<std::pair<int,int>, ex> A;
  std::map<std::pair<int,int>, double> Aval;
  std::pair<int,int> index;
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(domain); 
  fe.compute_basis_functions(); 
  for (int i=1; i<= fe.nbf() ; i++) {
    index.first = i; 
    for (int j=1; j<= fe.nbf() ; j++) {
      index.second = j; 
      ex nabla = inner(grad(fe.N(i)), grad(fe.N(j))); 
      ex Aij = domain.integrate(nabla, SUBS_PERFORMED);   
      A[index] = Aij;  
    }
  }

  map<std::pair<int,int>,ex>::iterator iter; 
  for (iter = A.begin(); iter != A.end() ; iter++) {
      ex intf = (*iter).second; 
      if (!is_a<lst>(intf)) {
        cout <<"A["<<(*iter).first.first<<","<<(*iter).first.second<<"]="<<(*iter).second<<endl; 
      }
      else {
        ex intff = intf.op(0); 
        ex  intfff = intff; 
        for (int i=1; i<intf.nops(); i++) {
          intfff = intfff.subs(intf.op(i)); 
        }
        cout <<"A["<<(*iter).first.first<<","<<(*iter).first.second<<"]="<<intfff<<endl; 

      }
  }
}



void poisson_reference_element() {

  symbol x0("x0"), x1("x1"), x2("x2"), x3("x3"); 
  symbol y0("y0"), y1("y1"), y2("y2"), y3("y3"); 
  symbol z0("z0"), z1("z1"), z2("z2"), z3("z3"); 

  cout <<"1D Reference ----------------------------"<<endl; 
  ReferenceLine line("l"); 
  compute_element_matrix(line, 2); 


  cout <<"2D Reference ----------------------------"<<endl; 
  ReferenceTriangle T1("t"); 
  compute_element_matrix(T1, 2); 

  cout <<"2D General Numeric ----------------------------"<<endl; 
  Triangle T2(lst(0,0), lst(1,0), lst(0,1), "t"); 
  compute_element_matrix(T2, 2); 

  cout <<"2D General Symbolic ----------------------------"<<endl; 
  Triangle T3(lst(x0,y0,z0), lst(x1,y1,z1), lst(x2,y2,z2), "t"); 
  compute_element_matrix(T3, 2); 


  cout <<"3D Reference ----------------------------"<<endl; 
  ReferenceTetrahedron T4("t"); 
  compute_element_matrix(T4, 2); 

  cout <<"3D General Numeric ----------------------------"<<endl; 
  Tetrahedron T5(lst(0,0,0), lst(1,0,0), lst(0,1,0), lst(0,0,1), "t"); 
  compute_element_matrix(T5, 2); 
 
  /*
  cout <<"3D General Symbolic ----------------------------"<<endl; 
  Tetrahedron T6(lst(x0,y0,z0), lst(x1,y1,z1), lst(x2,y2,z2), lst(x3,y3,z3), "t"); 
  compute_element_matrix(T6, 2); 
  */
 
  
}


int main() {
  poisson_reference_element(); 
}



