#include <SimpleMesh.h>
#include <iostream>
#include <fstream>


using namespace GiNaC; 

void compute_element_matrix(FE& fe, int element, Dof& dof, 
	           	    std::map<std::pair<int,int>, double>& Aval, 
			    std::map<int,double>& bval ) 
{ 
  std::pair<int,int> index;
  std::pair<int,int> global_index;
  Polygon& domain = fe.getPolygon(); 
  // insert dofs 
  for (int i=1; i<= fe.nbf() ; i++) {
    dof.insert_dof(element, i, fe.dof(i)); 
  }

  ex f = 1; 

  for (int i=1; i<= fe.nbf() ; i++) {
    global_index.first = dof.glob_dof(element,i);  
    index.first = i; 
    for (int j=1; j<= fe.nbf() ; j++) {
      global_index.second = dof.glob_dof(element,j);  
      index.second = j; 
      ex nabla = inner(grad(fe.N(i)), grad(fe.N(j))); 
      ex Aij = domain.integrate(nabla);   
      Aval[global_index] += ex_to<numeric>(Aij).to_double();
    }
    ex bi = domain.integrate(fe.N(i)*f); 
    bval[global_index.first] = ex_to<numeric>(bi).to_double(); 
  }
}

void dump_to_matlab(std::map<std::pair<int,int>,double> A,
                        const char *varname, const char *s){
  ofstream outfile(s, ios::out);
  outfile << "%The matrix\n" << endl;
  std::map<std::pair<int,int>,double>::iterator Ait;
  for(Ait=A.begin(); Ait!=A.end(); Ait++){
    outfile << varname << "(" << (*Ait).first.first << ","
            << (*Ait).first.second << ")=" << (*Ait).second << ";" << endl;
  }
}

void dump_to_matlab(std::map<int,double> b,
                        const char *varname, const char *s){
  ofstream outfile(s, ios::out);
  outfile << "%The rhs\n" << endl;
  std::map<int,double>::iterator bit;
  for(bit=b.begin(); bit!=b.end(); bit++){
    outfile << varname << "(" << (*bit).first
	    << ")=" << (*bit).second << ";" << endl;
  }
}



int main () {
  lst x0 = lst(0,0); 
  lst xn = lst(1,1); 
  int nx = 501; 
  int ny = 501; 

  int order =1; 
  Dof dof; 
  LagrangeFE fe; 
  fe.set(order); 


  std::map<std::pair<int,int>, double> Aval; 
  std::map<int,double>                 bval; 

  Square square(x0,xn,nx,ny); 
  Triangle triangle; 
  cout <<"no of triangles "<<square.no_of_triangles() <<endl; 
  for (int i=1; i<= square.no_of_triangles(); i++) {
    cout <<"Triangle "<<i<<endl; 
    triangle = square.triangle(i); 
    fe.set(triangle); 
    fe.compute_basis_functions(); 
    compute_element_matrix(fe,i,dof,Aval,bval); 
  }

  int ig; 
  int jg; 
  pair<int,int> index; 
  pair<int,int> index2; 
  pair<int,int> index3; 
  for (int i=1; i<= square.no_of_boundary_nodes(); i++) {
    ex ptv = square.boundary_node(i); 
    ig = dof.glob_dof(ptv);  
    index.first = ig; index.second = ig;  
    index2.first = ig+1; index2.second = ig+1;  
    Aval[index] = 1.0;  
    bval[ig] = 1.0; 
    std::map<std::pair<int,int>,double>::iterator Ait;
    Ait=Aval.find(index); 
    Ait++; 
    for(; Ait!=Aval.find(index2); Ait++) { 
       (*Ait).second = 0;  
       jg = (*Ait).first.second;
       index3.first=jg;  
       index3.second=ig ; //ig = (*Ait).first.first;  
       bval[jg] -= Aval[index3]*bval[ig];   
       Aval[index3] = 0;      
    }
  }

  dump_to_matlab(Aval, "A", "A.m"); 
  dump_to_matlab(bval, "b", "b.m"); 
}
