#include "symb_transf.h"
#include <sstream>
#include <iostream>

using namespace GiNaC; 

/////////////////////////////////////////////////////////////////////////////

symb_transf  &symb_transf::operator =( const symb_transf & other )
{
  if( this != & other ) {
    copy( other );
  }

  return *this;
}

/////////////////////////////////////////////////////////////////////////////

void symb_transf::copy( const symb_transf & st )
{
  exzer = st.exzer;
  symb_list = st.symb_list;
  strg_list = st.strg_list;

  str_to_symb = st.str_to_symb;

  repl_list = st.repl_list;
}
  
/////////////////////////////////////////////////////////////////////////////

void symb_transf::create_lists( const ex & expre )
{
  // first we expand the expression to be treated
  ex expre_exp = expre.expand();

  // for add-terms, we just call this function again
  if( is_a<add>(expre_exp) )
  {  
    int add_terms = expre_exp.nops();

    // we do this for every single term
    for( int add_cnt = 0; add_cnt < add_terms; add_cnt++ ) 
       create_lists(  expre_exp.op( add_cnt ) );
    
  }
  else if(  is_a<mul>( expre_exp ) ) 
  {       
    int mul_terms = expre_exp.nops(); // the number of terms in the mul - expression
 
    for( int che = 0; che < mul_terms; che++ )
       create_lists( expre_exp.op(che) );     
  }
  else if( is_a<symbol>( expre_exp ) ) {

        bool list_already_has_symb = false;
        bool list_already_has_strg = false;

        for( lst::const_iterator lt  = symb_list.begin();
                                 lt != symb_list.end();
                                 lt ++ ) 
          if( *lt == expre_exp ) 
            list_already_has_symb = true;

        if( ! list_already_has_symb )
            symb_list.append( expre_exp );
         
        for( list<string>::const_iterator ls  = strg_list.begin();
                                          ls != strg_list.end();
                 	                  ls ++ ) {

          if( *ls  == get_string( expre_exp ) )
            list_already_has_strg = true;
	}

        if( !list_already_has_strg ) {

            strg_list.push_back( get_string( expre_exp ) );
	}

  }
}




/////////////////////////////////////////////////////////////////////////////

string symb_transf::get_string( const ex & symb )
{
  std::ostringstream s;
  s << symb;

  return s.str();
}  


/////////////////////////////////////////////////////////////////////////////

void symb_transf::create_map()
{

  for ( list<string>::const_iterator str  = strg_list.begin();
                                     str != strg_list.end(); 
                                     str++ ) {

    lst thissymb;

    for( lst::const_iterator ls  = symb_list.begin();
                             ls != symb_list.end();
                 	     ls ++ ) {

      bool this_symb_is_already_in = false;

      if( get_string( *ls ) == *str ) {
         
        if( !this_symb_is_already_in ) {
 
	  thissymb.append( *ls );
          this_symb_is_already_in = true;
	}
      }

    }

    str_to_symb[*str] = thissymb;
  }
}

/////////////////////////////////////////////////////////////////////////////

void symb_transf::create_replace_rules()
{
 for ( map<string,lst>::const_iterator msl  = str_to_symb.begin();
                                       msl != str_to_symb.end(); 
                                       msl++ ) {

   ex rem_sym = *msl->second.begin();

   for( lst::const_iterator cli  = symb_list.begin();
   	                    cli != symb_list.end();
     	                    cli++ ) {

             if( *cli != rem_sym ) 
	       repl_list.append( *cli == rem_sym );
	 }
 }
}


/////////////////////////////////////////////////////////////////////////////

ex symb_transf::replaced()
{
  create_lists( exzer );

  create_map();

  create_replace_rules();

  ex to_ret = exzer;
  
  for( lst::const_iterator rl  = repl_list.begin();
                           rl != repl_list.end();
                           rl ++ ) 
      to_ret = to_ret.subs( *rl );


  return to_ret;
}

/////////////////////////////////////////////////////////////////////////////

