#include <ArnoldFalkWintherWeakSym.h>
#include <Nedelec2Hdiv.h>
#include <DiscontinuousLagrangeFE.h>

//------------ sigma element 

int ArnoldFalkWintherWeakSymSigma:: nbf() {
  return Ns.size();  
}



void ArnoldFalkWintherWeakSymSigma:: compute_basis_functions() {

  if ( order < 1 ) {
    cout <<"Arnold-Falk-Winther elements must be of order 1 or higher."<<endl; 
    return; 
  }


  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }

  Nedelec2Hdiv fe; 
  fe.set(order); 
  fe.set(*p); 
  fe.compute_basis_functions(); 

  for (int d=0; d<3; d++) {
    for (int i=0; i<fe.nbf(); i++) {
      GiNaC::matrix Nmat = GiNaC::matrix(3,3); 
      Nmat(d,0) = fe.N(i).op(0); 
      Nmat(d,1) = fe.N(i).op(1); 
      Nmat(d,2) = fe.N(i).op(2); 
      Ns.insert(Ns.end(), Nmat); 
      dofs.insert(dofs.end(),GiNaC::lst(fe.dof(i), d)); 
    }
  }
}


void ArnoldFalkWintherWeakSymSigma:: set(Polygon& p_) {
  StandardFE::set(p_); 
}


void ArnoldFalkWintherWeakSymSigma:: set(int order_) { 
  StandardFE::set(order_); 
}


GiNaC::ex ArnoldFalkWintherWeakSymSigma:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex ArnoldFalkWintherWeakSymSigma::N(int i) {  
  return StandardFE::N(i); 
}


//------------ u element 

int ArnoldFalkWintherWeakSymU:: nbf() {
  return Ns.size();  
}


void ArnoldFalkWintherWeakSymU:: compute_basis_functions() {

  if ( order < 1 ) {
    cout <<"Arnold-Falk-Winther elements must be of order 1 or higher."<<endl; 
    return; 
  }


  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }

  
  VectorDiscontinuousLagrangeFE fe;  
  fe.set(order); 
  fe.set_size(3); 
  fe.set(*p); 
  fe.compute_basis_functions(); 

  for (int i=0; i<fe.nbf(); i++) {
      GiNaC::lst Ni = GiNaC::lst(fe.N(i).op(0), fe.N(i).op(1), fe.N(i).op(2)); 
      GiNaC::ex Nmat = GiNaC::matrix(3,1,Ni); 
      Ns.insert(Ns.end(), Nmat); 
  }
}


void ArnoldFalkWintherWeakSymU:: set(Polygon& p_) {
  StandardFE::set(p_); 
}


void ArnoldFalkWintherWeakSymU:: set(int order_) { 
  StandardFE::set(order_); 
}


GiNaC::ex ArnoldFalkWintherWeakSymU:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex ArnoldFalkWintherWeakSymU::N(int i) {  
  return StandardFE::N(i); 
}



//------------ p element 

int ArnoldFalkWintherWeakSymP:: nbf() {
  return Ns.size();  
}




void ArnoldFalkWintherWeakSymP:: compute_basis_functions() {

  if ( order < 1 ) {
    cout <<"Arnold-Falk-Winther elements must be of order 1 or higher."<<endl; 
    return; 
  }


  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }

  
  VectorDiscontinuousLagrangeFE fe;  
  fe.set(order); 
  fe.set_size(3); 
  fe.set(*p); 
  fe.compute_basis_functions(); 

  for (int d=0; d<3; d++) {
    for (int i=0; i<fe.nbf(); i++) {
      GiNaC::matrix Nmat = GiNaC::matrix(3,3); 
      Nmat(1,2) = -fe.N(i).op(0); 
      Nmat(2,1) =  fe.N(i).op(0); 

      Nmat(0,2) =  fe.N(i).op(1); 
      Nmat(2,0) = -fe.N(i).op(1); 

      Nmat(0,1) = -fe.N(i).op(2); 
      Nmat(1,0) =  fe.N(i).op(2); 

      Ns.insert(Ns.end(), Nmat); 
    }
  }
}




void ArnoldFalkWintherWeakSymP:: set(Polygon& p_) {
  StandardFE::set(p_); 
}


void ArnoldFalkWintherWeakSymP:: set(int order_) { 
  StandardFE::set(order_); 
}


GiNaC::ex ArnoldFalkWintherWeakSymP:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex ArnoldFalkWintherWeakSymP::N(int i) {  
  return StandardFE::N(i); 
}





