#include "DiscontinuousLagrangeFE.h"
#include "ElementComputations.h"

DiscontinuousLagrangeFE:: DiscontinuousLagrangeFE() : LagrangeFE() {
  element = 0; 
}

int DiscontinuousLagrangeFE:: nbf() {
  return StandardFE::nbf();       
}

void DiscontinuousLagrangeFE:: compute_basis_functions() { 

  if ( order < 1 ) {
    cout <<"Discontinuous Lagrangian elements must be of order 1 or higher."<<endl; 
    return; 
  }


  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }
 

  LagrangeFE:: compute_basis_functions(); 
  for (int i=0; i< dofs.size(); i++) { 
    dofs[i] = GiNaC::lst(dofs[i], element); 
  }
}



void DiscontinuousLagrangeFE:: set(Polygon& p_) {
  StandardFE::set(p_); 
}

void DiscontinuousLagrangeFE:: set_element_number(int element_) {
  element = element_; 
}

void DiscontinuousLagrangeFE:: set(int order_) { 
  StandardFE::set(order_); 
}


GiNaC::ex DiscontinuousLagrangeFE:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex DiscontinuousLagrangeFE::N(int i) {  
  return StandardFE::N(i); 
}

// ------------VectorDiscontinuousLagrangeFE --- 

VectorDiscontinuousLagrangeFE:: VectorDiscontinuousLagrangeFE() : 
	VectorLagrangeFE() {
  element = 0; 
}


int VectorDiscontinuousLagrangeFE:: nbf() {
  return VectorLagrangeFE::nbf();       
}

void VectorDiscontinuousLagrangeFE:: compute_basis_functions(){ 

  VectorLagrangeFE:: compute_basis_functions(); 

  for (int i=0; i< dofs.size(); i++) { 
    dofs[i] = GiNaC::lst(dofs[i], element); 
  }

}


void VectorDiscontinuousLagrangeFE:: set_size(int size_) {
  VectorLagrangeFE::set_size(size_);  
}


void VectorDiscontinuousLagrangeFE:: set(Polygon& p_) {
  StandardFE::set(p_); 
}


void VectorDiscontinuousLagrangeFE:: set(int order_) { 
  StandardFE::set(order_); 
}


GiNaC::ex VectorDiscontinuousLagrangeFE:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex VectorDiscontinuousLagrangeFE::N(int i) {  
  return VectorLagrangeFE::N(i); 
}

void VectorDiscontinuousLagrangeFE:: set_element_number(int element_) {
  element = element_; 
}





