#include "HermiteFE.h"

int HermiteFE:: nbf() {
  return StandardFE::nbf();       
}

void HermiteFE:: compute_basis_functions() {

  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }
 

  if (p->str() == "ReferenceTriangle" || p->str() == "Triangle") {
    GiNaC::ex polynom_space = pol(3, 2, "a"); 
    GiNaC::ex polynom = polynom_space.op(0); 
    GiNaC::lst variables = GiNaC::ex_to<GiNaC::lst>(polynom_space.op(1));
    GiNaC::lst equations; 

    for (int i=0; i<= 2; i++) { 
       GiNaC::ex v = p->vertex(i);  
       GiNaC::ex dofv = polynom.subs(GiNaC::lst(x == v.op(0), y == v.op(1)));  
       GiNaC::ex dofvdx = diff(polynom,x).subs(GiNaC::lst(x == v.op(0), y == v.op(1)));  
       GiNaC::ex dofvdy = diff(polynom,y).subs(GiNaC::lst(x == v.op(0), y == v.op(1)));  

       equations.append( dofv   == GiNaC::numeric(0));  
       equations.append( dofvdx == GiNaC::numeric(0));  
       equations.append( dofvdy == GiNaC::numeric(0));  

       dofs.insert(dofs.end(), GiNaC::lst(v.op(0), v.op(1), 0)); 
       dofs.insert(dofs.end(), GiNaC::lst(v.op(0), v.op(1), 1)); 
       dofs.insert(dofs.end(), GiNaC::lst(v.op(0), v.op(1), 2)); 
    }
    GiNaC::ex midpoint = GiNaC::lst((p->vertex(0).op(0) + p->vertex(1).op(0) + p->vertex(2).op(0))/3, 
                                    (p->vertex(0).op(1) + p->vertex(1).op(1) + p->vertex(2).op(1))/3); 
    GiNaC::ex dofm = polynom.subs(GiNaC::lst(x == midpoint.op(0), y == midpoint.op(1)));
    dofs.insert(dofs.end(), midpoint ); 
    equations.append( dofm == GiNaC::numeric(0));

    GiNaC::matrix b; GiNaC::matrix A; 
    matrix_from_equations(equations, variables, A, b); 

    int ncols = A.cols();  
    GiNaC::matrix vars_sq(ncols, ncols);

    // matrix of symbols 
    for (unsigned r=0; r<ncols; ++r)
      for (unsigned c=0; c<ncols; ++c)
        vars_sq(r, c) = GiNaC::symbol();

    GiNaC::matrix id(ncols, ncols);

    // identity 
    const GiNaC::ex _ex1(1);
    for (unsigned i=0; i<ncols; ++i)
      id(i, i) = _ex1;


    // invert the matrix 
    GiNaC::matrix m_inv(ncols, ncols);
    m_inv = A.solve(vars_sq, id, GiNaC::solve_algo::gauss);

    for (int i=0; i<dofs.size(); i++) {
      b.let_op(i) = GiNaC::numeric(1); 
      GiNaC::ex xx = m_inv.mul(GiNaC::ex_to<GiNaC::matrix>(b)); 
  
      GiNaC::lst subs;  
      for (int ii=0; ii<xx.nops(); ii++) { 
        subs.append(variables.op(ii) == xx.op(ii)); 
      }
      GiNaC::ex Nj= polynom.subs(subs);   
      Ns.insert(Ns.end(), Nj); 
      b.let_op(i) = GiNaC::numeric(0); 
    }
  } else if (p->str() == "ReferenceTetrahedron" || p->str() == "Tetrahedron") {
    GiNaC::ex polynom_space = pol(3, 3, "a"); 
    GiNaC::ex polynom = polynom_space.op(0); 
    GiNaC::lst variables = GiNaC::ex_to<GiNaC::lst>(polynom_space.op(1));
    GiNaC::lst equations; 

    cout <<"polynom "<<polynom<<endl; 

    for (int i=0; i<= 3; i++) { 
       GiNaC::ex v = p->vertex(i);  
       GiNaC::ex dofv   = polynom.subs(GiNaC::lst(x == v.op(0), y == v.op(1), z == v.op(2) ));  
       GiNaC::ex dofvdx = diff(polynom,x).subs(GiNaC::lst(x == v.op(0), y == v.op(1), z == v.op(2) ));  
       GiNaC::ex dofvdy = diff(polynom,y).subs(GiNaC::lst(x == v.op(0), y == v.op(1), z == v.op(2) ));  
       GiNaC::ex dofvdz = diff(polynom,z).subs(GiNaC::lst(x == v.op(0), y == v.op(1), z == v.op(2) ));  

       equations.append( dofv   == GiNaC::numeric(0));  
       equations.append( dofvdx == GiNaC::numeric(0));  
       equations.append( dofvdy == GiNaC::numeric(0));  
       equations.append( dofvdz == GiNaC::numeric(0));  

       dofs.insert(dofs.end(), GiNaC::lst(v.op(0), v.op(1), 0)); 
       dofs.insert(dofs.end(), GiNaC::lst(v.op(0), v.op(1), 1)); 
       dofs.insert(dofs.end(), GiNaC::lst(v.op(0), v.op(1), 2)); 
       dofs.insert(dofs.end(), GiNaC::lst(v.op(0), v.op(1), 3)); 

    }
    GiNaC::ex midpoint1 = GiNaC::lst(
		    (p->vertex(0).op(0)*2 + p->vertex(1).op(0) + p->vertex(2).op(0) + p->vertex(3).op(0))/5, 
                    (p->vertex(0).op(1)*2 + p->vertex(1).op(1) + p->vertex(2).op(1) + p->vertex(3).op(1))/5, 
                    (p->vertex(0).op(2)*2 + p->vertex(1).op(2) + p->vertex(2).op(2) + p->vertex(3).op(2))/5); 

    GiNaC::ex midpoint2 = GiNaC::lst(
		    (p->vertex(0).op(0) + p->vertex(1).op(0)*2 + p->vertex(2).op(0) + p->vertex(3).op(0))/5, 
                    (p->vertex(0).op(1) + p->vertex(1).op(1)*2 + p->vertex(2).op(1) + p->vertex(3).op(1))/5, 
                    (p->vertex(0).op(2) + p->vertex(1).op(2)*2 + p->vertex(2).op(2) + p->vertex(3).op(2))/5); 

    GiNaC::ex midpoint3 = GiNaC::lst(
		    (p->vertex(0).op(0) + p->vertex(1).op(0) + p->vertex(2).op(0)*2 + p->vertex(3).op(0))/5, 
                    (p->vertex(0).op(1) + p->vertex(1).op(1) + p->vertex(2).op(1)*2 + p->vertex(3).op(1))/5, 
                    (p->vertex(0).op(2) + p->vertex(1).op(2) + p->vertex(2).op(2)*2 + p->vertex(3).op(2))/5); 

    GiNaC::ex midpoint4 = GiNaC::lst(
		    (p->vertex(0).op(0) + p->vertex(1).op(0) + p->vertex(2).op(0) + p->vertex(3).op(0)*2)/5, 
                    (p->vertex(0).op(1) + p->vertex(1).op(1) + p->vertex(2).op(1) + p->vertex(3).op(1)*2)/5, 
                    (p->vertex(0).op(2) + p->vertex(1).op(2) + p->vertex(2).op(2) + p->vertex(3).op(2)*2)/5); 


    GiNaC::ex dofm1 = polynom.subs(GiNaC::lst(x == midpoint1.op(0), y == midpoint1.op(1), z == midpoint1.op(2)));
    GiNaC::ex dofm2 = polynom.subs(GiNaC::lst(x == midpoint2.op(0), y == midpoint2.op(1), z == midpoint2.op(2)));
    GiNaC::ex dofm3 = polynom.subs(GiNaC::lst(x == midpoint3.op(0), y == midpoint3.op(1), z == midpoint3.op(2)));
    GiNaC::ex dofm4 = polynom.subs(GiNaC::lst(x == midpoint4.op(0), y == midpoint4.op(1), z == midpoint4.op(2)));

    dofs.insert(dofs.end(), midpoint1 ); 
    dofs.insert(dofs.end(), midpoint2 ); 
    dofs.insert(dofs.end(), midpoint3 ); 
    dofs.insert(dofs.end(), midpoint4 ); 

    equations.append( dofm1 == GiNaC::numeric(0));
    equations.append( dofm2 == GiNaC::numeric(0));
    equations.append( dofm3 == GiNaC::numeric(0));
    equations.append( dofm4 == GiNaC::numeric(0));

    GiNaC::matrix b; GiNaC::matrix A; 
    matrix_from_equations(equations, variables, A, b); 
    cout <<"A "<<A<<endl; 

    int ncols = A.cols();  
    int nrows = A.rows();  
    GiNaC::matrix vars_sq(ncols, ncols);

    cout <<"ncols "<<ncols<<endl; 
    cout <<"nrows "<<nrows<<endl; 

    // matrix of symbols 
    for (unsigned r=0; r<ncols; ++r)
      for (unsigned c=0; c<ncols; ++c)
        vars_sq(r, c) = GiNaC::symbol();

    GiNaC::matrix id(ncols, ncols);

    // identity 
    const GiNaC::ex _ex1(1);
    for (unsigned i=0; i<ncols; ++i)
      id(i, i) = _ex1;


    // invert the matrix 
    GiNaC::matrix m_inv(ncols, ncols);
    m_inv = A.solve(vars_sq, id, GiNaC::solve_algo::gauss);

    for (int i=0; i<dofs.size(); i++) {
      b.let_op(i) = GiNaC::numeric(1); 
      GiNaC::ex xx = m_inv.mul(GiNaC::ex_to<GiNaC::matrix>(b)); 
  
      GiNaC::lst subs;  
      for (int ii=0; ii<xx.nops(); ii++) { 
        subs.append(variables.op(ii) == xx.op(ii)); 
      }
      GiNaC::ex Nj= polynom.subs(subs);   
      Ns.insert(Ns.end(), Nj); 
      b.let_op(i) = GiNaC::numeric(0); 
    }
  }
}

void HermiteFE:: set(Polygon& p_) {
  StandardFE::set(p_); 
}


GiNaC::ex HermiteFE:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex HermiteFE::N(int i) {  
  return StandardFE::N(i); 
}

