#include "LagrangeFE.h"
#include "ElementComputations.h"

int LagrangeFE:: nbf() {
  return StandardFE::nbf();       
}

void LagrangeFE:: compute_basis_functions() {

  // NOTE: in the below code dof(i) is not used to 
  // determine the basis functions 

  if ( order < 1 ) {
    cout <<"Lagrangian elements must be of order 1 or higher."<<endl; 
    return; 
  }

  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }
   

  if (p->str() == "ReferenceLine") { 
    // Look at the case with the Triangle for a documented code
    GiNaC::ex polynom; 
    GiNaC::lst variables; 
  
//    polynom = pol(order, 1, "a"); 
//    variables = coeffs(polynom); 
    GiNaC::ex polynom_space = bernstein(order, *p, "a"); 
//    GiNaC::ex polynom_space = pol(order, 1, "a"); 
    polynom = polynom_space.op(0); 
    variables = GiNaC::ex_to<GiNaC::lst>(polynom_space.op(1));
  
    GiNaC::ex increment = GiNaC::numeric(1,order); 
  
    GiNaC::ex Nj; 
    for (int j=1; j <= order+1; j++) {
      GiNaC::lst equations; 
      int i=0; 
      for (GiNaC::ex p=0; p<= 1 ; p += increment ) { 
        i++; 
        GiNaC::ex eq = polynom == dirac(i,j); 
        equations.append(eq.subs(x == p));  
        if (j == 1) dofs.insert(dofs.end(), p); 
      }
  
      GiNaC::ex subs = lsolve(equations, variables); 
      Nj = polynom.subs(subs);   
      Ns.insert(Ns.end(), Nj); 
    }
  

  } else if (p->str() == "ReferenceTriangle" ) { 
    // Look at the case with the Triangle for a documented code
    GiNaC::ex polynom; 
    GiNaC::lst variables; 
  
//    polynom = pol(order, 2, "b"); 
//    variables = coeffs(polynom); 
    GiNaC::ex polynom_space = bernstein(order, *p, "b"); 
//    GiNaC::ex polynom_space = pol(order, 2, "a"); 
    polynom = polynom_space.op(0); 
    variables = GiNaC::ex_to<GiNaC::lst>(polynom_space.op(1));
  
    GiNaC::ex increment = GiNaC::numeric(1,order); 
  
    GiNaC::ex Nj; 
    for (int j=1; j <= (order+1)*(order+2)/2; j++) {
      GiNaC::lst equations; 
      int i=0; 
      GiNaC::numeric one = 1; 
      for (GiNaC::ex q=0; q<= one  ; q += increment ) { 
        for (GiNaC::ex p=0; p<= one-q ; p += increment ) { 
          i++; 
          GiNaC::ex eq = polynom == dirac(i,j); 
          equations.append(eq.subs(GiNaC::lst(x == p, y == q)));  
          if ( j == 1) dofs.insert(dofs.end(), GiNaC::lst(p,q)); 
        }
      }

  
      GiNaC::ex subs = lsolve(equations, variables); 
      Nj = polynom.subs(subs);   
      Ns.insert(Ns.end(), Nj); 
    }
  } 
  else if ( p->str() == "Triangle" ){ 
    // Look HERE for the documented code
    GiNaC::ex polynom; 
    GiNaC::lst variables; 

    GiNaC::ex polynom_space = pol(order, 2, "a"); 
    // the polynomial spaces on the form: 
    // first item:     a0 + a1*x + a2*y + a3*x^2 + a4*x*y ...     the polynom
    // second item:    a0, a1, a2, ...                            the coefficents 
    // third  item     1, x, y, x^2, ..                           the basis  
    // Could also do:
    // GiNaC::ex polynom_space = bernstein(order, t, "a"); 


    polynom = polynom_space.op(0); 
    variables = GiNaC::ex_to<GiNaC::lst>(polynom_space.op(1));

    GiNaC::ex Nj; 
    Polygon& pp = *p; 
    Triangle& t = (Triangle&) pp; 
    // The bezier ordinates (in which the basis function should be either 0 or 1)
    GiNaC::lst points = bezier_ordinates(t,order); 

    // Loop over all basis functions Nj and all points. 
    // Each basis function Nj is determined by a set of linear equations: 
    //   Nj(xi) = dirac(i,j) 
    // This system of equations is then solved by lsolve
    for (int j=1; j <= points.nops(); j++) {
      GiNaC::lst equations; 
      int i=0; 
      for (int i=1; i<= points.nops() ; i++ ) { 
	// The point xi 
	GiNaC::ex point = points.op(i-1); 
        // The equation Nj(xi) = dirac(i,j)  
        GiNaC::ex eq = polynom == dirac(i,j); 
	// The equation is appended to the list of equations
        equations.append(eq.subs(GiNaC::lst(x == point.op(0) , y == point.op(1))));  
	// Creation of dof(j)
        if ( j == 1) dofs.insert(dofs.end(), GiNaC::lst(point.op(0),point.op(1))); 
      }
  
//      print(equations); 
//      print(variables); 
      // We solve the linear system 
      GiNaC::ex subs = lsolve(equations, variables); 
      // Substitute to get the Nj 
      Nj = polynom.subs(subs);   
      // Append Nj to the list of basis functions 
      Ns.insert(Ns.end(), Nj); 

      // NOTE: In this case we create a linear system and solve it for each Nj.   
      // However, the matrix is always the same, it is only the right-hand side
      // that changes. Hence, it is possible to optimize here.  

    }

  } else if ( p->str() == "ReferenceTetrahedron" ) {
    // Look at the case with the Triangle for a documented code
    GiNaC::ex polynom; 
    GiNaC::lst variables; 
  
//    polynom = pol(order, 3, "b"); 
//    GiNaC::ex polynom_space = pol(order, 3, "a"); 
    GiNaC::ex polynom_space = bernstein(order, *p, "b"); 
    polynom = polynom_space.op(0); 
    variables = GiNaC::ex_to<GiNaC::lst>(polynom_space.op(1));
 
  
    int nno =0; 
    for (int j=0; j<= order; j++) { 
      nno += (j+1)*(j+2)/2; 
    }
  
    GiNaC::ex increment = GiNaC::numeric(1,order); 
  
    GiNaC::ex Nj; 
    for (int j=1; j <= nno; j++) {
      GiNaC::lst equations; 
      int i=0; 
      for (GiNaC::ex r=0; r<= 1 ; r += increment ) { 
        for (GiNaC::ex q=0; q<= 1-r ; q += increment ) { 
          for (GiNaC::ex p=0; p<= 1-r-q ; p += increment ) { 
            i++; 
            GiNaC::ex eq = polynom == dirac(i,j); 
            equations.append(eq.subs(GiNaC::lst(x == p, y == q, z == r )));  
            if (j == 1) dofs.insert(dofs.end(), GiNaC::lst(p,q,r)); 
	  }
        }
      }
  
      GiNaC::ex subs = lsolve(equations, variables); 
      Nj = polynom.subs(subs);   
      Ns.insert(Ns.end(), Nj); 
    }
  }
  else if ( p->str() == "Tetrahedron" ){ 
    // Look at the case with the Triangle for a documented code
    GiNaC::ex polynom; 
    GiNaC::lst variables; 

  
    GiNaC::ex polynom_space = pol(order, 3, "a"); 
//    GiNaC::ex polynom_space = bernstein(order, *p, "b"); 
    polynom = polynom_space.op(0); 
    variables = GiNaC::ex_to<GiNaC::lst>(polynom_space.op(1));
 
  
    GiNaC::ex increment = GiNaC::numeric(1,order); 
  
    GiNaC::ex Nj; 
    Polygon& pp = *p; 
    Tetrahedron& t = (Tetrahedron&) pp; 
    GiNaC::lst points = bezier_ordinates(t,order); 
    for (int j=1; j <= points.nops(); j++) {
      GiNaC::lst equations; 
      int i=0; 
      for (int i=1; i<= points.nops() ; i++ ) { 
	GiNaC::ex point = points.op(i-1); 
        GiNaC::ex eq = polynom == dirac(i,j); 
        equations.append(eq.subs(GiNaC::lst(x == point.op(0) , y == point.op(1), z == point.op(2))));  
        if ( j == 1) dofs.insert(dofs.end(), GiNaC::lst(point.op(0),point.op(1),point.op(2))); 
      }
  
      GiNaC::ex subs = lsolve(equations, variables); 
      Nj = polynom.subs(subs);   
      Ns.insert(Ns.end(), Nj); 
    }
  }
}



void LagrangeFE:: set(Polygon& p_) {
  StandardFE::set(p_); 
}


void LagrangeFE:: set(int order_) { 
  StandardFE::set(order_); 
}


GiNaC::ex LagrangeFE:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex LagrangeFE::N(int i) {  
  return StandardFE::N(i); 
}

// ------------VectorLagrangeFE --- 

int VectorLagrangeFE:: nbf() {
  return StandardFE::nbf();       
}

void VectorLagrangeFE:: compute_basis_functions() {

  if ( order < 1 ) {
    cout <<"Lagrangian elements must be of order 1 or higher."<<endl; 
    return; 
  }


  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }
 
  LagrangeFE fe; 
  fe.set(order); 
  fe.set(*p); 
  fe.compute_basis_functions(); 
  GiNaC::lst zero_list; 
  for (int s=1; s<= size ; s++) {
    zero_list.append(0);  
  }

  for (int i=0; i< fe.nbf() ; i++) {
    for (int s=0; s< size ; s++) {
      GiNaC::lst Nis = zero_list;    
      Nis.let_op(s) = fe.N(i); 
      GiNaC::ex Nmat = GiNaC::matrix(size,1,Nis);
      Ns.insert(Ns.end(), Nmat);  

      GiNaC::lst dof = GiNaC::lst(fe.dof(i), s) ; 
      dofs.insert(dofs.end(), dof);  
    }
  }
}


void VectorLagrangeFE:: set_size(int size_) {
  size = size_; 
}


void VectorLagrangeFE:: set(Polygon& p_) {
  StandardFE::set(p_); 
}


void VectorLagrangeFE:: set(int order_) { 
  StandardFE::set(order_); 
}


GiNaC::ex VectorLagrangeFE:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex VectorLagrangeFE::N(int i) {  
  return StandardFE::N(i); 
}





