#include "RaviartThomas.h" 
#include <fstream>


int RaviartThomas:: nbf() {
  return Ns.size();  
}

void RaviartThomas:: compute_basis_functions() {

  if ( order < 1 ) {
    cout <<"Raviart-Thomas elements must be of order 1 or higher."<<endl; 
    return; 
  }


  if ( p == NULL ) {
    cout <<"You need to set a polygon before the basisfunctions can be computed"<<endl; 
    return; 
  }
 

  // see e.g. Brezzi and Fortin book page 116 for the definition
  
  if (p->str() == "ReferenceLine" || p->str() == "Line" ) { 

    cout <<"Can not define the Raviart-Thomas element on a line"<<endl;  

  } else if (p->str() == "ReferenceTriangle" || p->str() == "Triangle" ) { 

    Triangle& triangle = (Triangle&)(*p); 
    GiNaC::lst equations; 
    GiNaC::lst variables; 
    GiNaC::ex polynom_space1 = bernstein(order-1, triangle, "a"); 
    GiNaC::ex polynom1 = polynom_space1.op(0); 
    GiNaC::ex polynom1_vars = polynom_space1.op(1); 
    GiNaC::ex polynom1_basis = polynom_space1.op(2); 

    GiNaC::lst polynom_space2 = bernsteinv(2,order-1, triangle, "b"); 
    GiNaC::ex polynom2 = polynom_space2.op(0).op(0); 
    GiNaC::ex polynom3 = polynom_space2.op(0).op(1); 


    GiNaC::lst pspace = GiNaC::lst( polynom2 + polynom1*x, 
		      polynom3 + polynom1*y);   
    GiNaC::lst v2 = collapse(GiNaC::ex_to<GiNaC::lst>(polynom_space2.op(1))); 


    variables = collapse(GiNaC::lst(polynom_space1.op(1), v2));   

    // remove multiple dofs 
    if ( order >= 2) {
      GiNaC::ex expanded_pol = GiNaC::expand(polynom1);
      for (int c1=0; c1<= order-2;c1++) { 
        for (int c2=0; c2<= order-2;c2++) { 
          for (int c3=0; c3<= order-2;c3++) {
            if ( c1 + c2 + c3 <= order -2 ) {
              GiNaC::ex eq =  expanded_pol.coeff(x,c1).coeff(y,c2).coeff(z,c3);  
	      if ( eq != GiNaC::numeric(0) ) { 
		equations.append(eq == 0); 
	      }
	    }
	  }
	}
      }
    }

    int removed_dofs = equations.nops(); 



    GiNaC::ex bernstein_pol; 

    int counter = 0; 
    GiNaC::symbol t("t"); 
    GiNaC::ex dofi; 
    // dofs related to edges  
    for (int i=1; i<= 3; i++) {
      Line line = triangle.line(i); 
      GiNaC::lst normal_vec = normal(triangle, i); 
      bernstein_pol = bernstein(order-1, line, istr("a",i));
      GiNaC::ex basis_space = bernstein_pol.op(2); 
      GiNaC::ex pspace_n = inner(pspace, normal_vec);  

      GiNaC::ex basis; 
      for (int j=0; j< basis_space.nops(); j++) {
	counter++; 
        basis = basis_space.op(j); 
	GiNaC::ex integrand = pspace_n*basis; 
	dofi =  line.integrate(integrand);  
	dofs.insert(dofs.end(), dofi); 
	GiNaC::ex eq = dofi == GiNaC::numeric(0); 
	equations.append(eq); 

      }
    }

    // dofs related to the whole triangle
    GiNaC::lst bernstein_polv; 
    if ( order > 1) {
      counter++; 
      bernstein_polv = bernsteinv(2,order-2, triangle, "a");
      GiNaC::ex basis_space = bernstein_polv.op(2); 
      for (int i=0; i< basis_space.nops(); i++) {
        GiNaC::lst basis = GiNaC::ex_to<GiNaC::lst>(basis_space.op(i)); 
	GiNaC::ex integrand = inner(pspace, basis);  
        dofi = triangle.integrate(integrand);  
	dofs.insert(dofs.end(), dofi); 
	GiNaC::ex eq = dofi == GiNaC::numeric(0); 
	equations.append(eq); 
      }
    }
    // invert the matrix: 
    // GiNaC has a bit strange way to invert a matrix. 
    // It solves the system AA^{-1} = Id. 
    // It seems that this way is the only way to do
    // properly with the solve_algo::gauss flag.  
    //
    GiNaC::matrix b; GiNaC::matrix A; 
    matrix_from_equations(equations, variables, A, b); 

    int ncols = A.cols();  
    GiNaC::matrix vars_sq(ncols, ncols);

    // matrix of symbols 
    for (unsigned r=0; r<ncols; ++r)
      for (unsigned c=0; c<ncols; ++c)
        vars_sq(r, c) = GiNaC::symbol();

    GiNaC::matrix id(ncols, ncols);

    // identity 
    const GiNaC::ex _ex1(1);
    for (unsigned i=0; i<ncols; ++i)
      id(i, i) = _ex1;


    // invert the matrix 
    GiNaC::matrix m_inv(ncols, ncols);
    m_inv = A.solve(vars_sq, id, GiNaC::solve_algo::gauss);


    for (int i=0; i<dofs.size(); i++) {
      b.let_op(removed_dofs + i) = GiNaC::numeric(1); 
      GiNaC::ex xx = m_inv.mul(GiNaC::ex_to<GiNaC::matrix>(b)); 
  
      GiNaC::lst subs;  
      for (int ii=0; ii<xx.nops(); ii++) { 
        subs.append(variables.op(ii) == xx.op(ii)); 
      }
      GiNaC::ex Nj1 = pspace.op(0).subs(subs);   
      GiNaC::ex Nj2 = pspace.op(1).subs(subs);   
      Ns.insert(Ns.end(), GiNaC::matrix(1,2,GiNaC::lst(Nj1,Nj2))); 
      b.let_op(removed_dofs + i) = GiNaC::numeric(0); 
    }

  } else if ( p->str() == "ReferenceTetrahedron" || p->str() == "Tetrahedron" ) { 


    Tetrahedron& tetrahedron = (Tetrahedron&)(*p); 
    GiNaC::lst equations; 
    GiNaC::lst variables; 
    GiNaC::ex polynom_space1 = bernstein(order-1, tetrahedron, "a"); 
    GiNaC::ex polynom1 = polynom_space1.op(0); 
    GiNaC::ex polynom1_vars = polynom_space1.op(1); 
    GiNaC::ex polynom1_basis = polynom_space1.op(2); 


    GiNaC::lst polynom_space2 = bernsteinv(3,order-1, tetrahedron, "b"); 
    GiNaC::ex polynom2 = polynom_space2.op(0).op(0); 
    GiNaC::ex polynom3 = polynom_space2.op(0).op(1); 
    GiNaC::ex polynom4 = polynom_space2.op(0).op(2); 



    GiNaC::lst pspace = GiNaC::lst( polynom2 + polynom1*x, 
		                    polynom3 + polynom1*y,   
		                    polynom4 + polynom1*z);   

    GiNaC::lst v2 = collapse(GiNaC::ex_to<GiNaC::lst>(polynom_space2.op(1))); 


    variables = collapse(GiNaC::lst(polynom_space1.op(1), v2));   

    GiNaC::ex bernstein_pol; 


    // remove multiple dofs 
    if ( order >= 2) {
      GiNaC::ex expanded_pol = GiNaC::expand(polynom1);
      for (int c1=0; c1<= order-2;c1++) { 
        for (int c2=0; c2<= order-2;c2++) { 
          for (int c3=0; c3<= order-2;c3++) {
            if ( c1 + c2 + c3 <= order -2 ) {
              GiNaC::ex eq =  expanded_pol.coeff(x,c1).coeff(y,c2).coeff(z,c3);  
	      if ( eq != GiNaC::numeric(0) ) { 
		equations.append(eq == 0); 
	      }
	    }
	  }
	}
      }
    }

    int removed_dofs = equations.nops(); 



    int counter = 0; 
    GiNaC::symbol t("t"); 
    GiNaC::ex dofi; 
    // dofs related to edges  
    for (int i=1; i<= 4; i++) {
      Triangle triangle = tetrahedron.triangle(i); 
      GiNaC::lst normal_vec = normal(tetrahedron, i); 
      bernstein_pol = bernstein(order-1, triangle, istr("a",i));
      GiNaC::ex basis_space = bernstein_pol.op(2); 
      GiNaC::ex pspace_n = inner(pspace, normal_vec);  

      GiNaC::ex basis; 
      for (int j=0; j< basis_space.nops(); j++) {
	counter++; 
        basis = basis_space.op(j); 
	GiNaC::ex integrand = pspace_n*basis; 
	dofi =  triangle.integrate(integrand);  
	dofs.insert(dofs.end(), dofi); 
	GiNaC::ex eq = dofi == GiNaC::numeric(0); 
	equations.append(eq); 

      }
    }

    // dofs related to the whole tetrahedron 
    GiNaC::lst bernstein_polv; 
    if ( order > 1) {
      counter++; 
      bernstein_polv = bernsteinv(3,order-2, tetrahedron, "a");
      GiNaC::ex basis_space = bernstein_polv.op(2); 
      for (int i=0; i< basis_space.nops(); i++) {
        GiNaC::lst basis = GiNaC::ex_to<GiNaC::lst>(basis_space.op(i)); 
	GiNaC::ex integrand = inner(pspace, basis);  
        dofi = tetrahedron.integrate(integrand);  
	dofs.insert(dofs.end(), dofi); 
	GiNaC::ex eq = dofi == GiNaC::numeric(0); 
	equations.append(eq); 
      }
    }
    // invert the matrix: 
    // GiNaC has a bit strange way to invert a matrix. 
    // It solves the system AA^{-1} = Id. 
    // It seems that this way is the only way to do
    // properly with the solve_algo::gauss flag.  
    //
    GiNaC::matrix b; GiNaC::matrix A; 
    matrix_from_equations(equations, variables, A, b); 

    int ncols = A.cols();  
    GiNaC::matrix vars_sq(ncols, ncols);

    // matrix of symbols 
    for (unsigned r=0; r<ncols; ++r)
      for (unsigned c=0; c<ncols; ++c)
        vars_sq(r, c) = GiNaC::symbol();

    GiNaC::matrix id(ncols, ncols);

    // identity 
    const GiNaC::ex _ex1(1);
    for (unsigned i=0; i<ncols; ++i)
      id(i, i) = _ex1;


    // invert the matrix 
    GiNaC::matrix m_inv(ncols, ncols);
    m_inv = A.solve(vars_sq, id, GiNaC::solve_algo::gauss);


    for (int i=0; i<dofs.size(); i++) {
      b.let_op(removed_dofs + i) = GiNaC::numeric(1); 
      GiNaC::ex xx = m_inv.mul(GiNaC::ex_to<GiNaC::matrix>(b)); 
  
      GiNaC::lst subs;  
      for (int ii=0; ii<xx.nops(); ii++) { 
        subs.append(variables.op(ii) == xx.op(ii)); 
      }
      GiNaC::ex Nj1 = pspace.op(0).subs(subs);   
      GiNaC::ex Nj2 = pspace.op(1).subs(subs);   
      GiNaC::ex Nj3 = pspace.op(2).subs(subs);   
      Ns.insert(Ns.end(), GiNaC::matrix(1,3,GiNaC::lst(Nj1,Nj2,Nj3))); 
      b.let_op(removed_dofs + i) = GiNaC::numeric(0); 
    }
  }
}


void RaviartThomas:: set(Polygon& p_) {
  StandardFE:: set(p_); 
}


void RaviartThomas:: set(int order_) { 
  StandardFE:: set(order_); 
}


GiNaC::ex RaviartThomas:: dof(int i) {
  return StandardFE::dof(i);  
}


GiNaC::ex RaviartThomas::N(int i) {  
  return StandardFE::N(i); 
}



