#include "tools.h"

GiNaC::symbol x("x"); 
GiNaC::symbol y("y"); 
GiNaC::symbol z("z"); 
GiNaC::symbol infinity("infinity"); 
GiNaC::symbol DUMMY("DUMMY"); 

int nsd = 2; 





	

void print(GiNaC::lst& l) {
//  for (GiNaC::lst::const_iterator i = l.begin(); i != l.end(); ++i)
//    cout << *i << endl;
//
  GiNaC::lst::const_iterator i = l.begin(); 
  cout <<"GiNaC::lst("<<*i; 
  ++i; 

  for (; i != l.end() ; ++i) {
    cout << ","<< *i;
  }
  cout <<");"<<endl; 
}

void print(GiNaC::exvector& v) {
  cout <<"v=["; 
  for (int i=0; i< v.size()-1; i++) {
    cout <<v[i]<<"," <<endl; 
  }
  cout <<v[v.size()-1]<< "]"<<endl; 

}

GiNaC::lst cross(GiNaC::lst& v1, GiNaC::lst& v2) {
  GiNaC::lst ret; 
  if ( v1.nops() != v2.nops() ) {
    cout <<"incompatible vectors "<<endl; 
    cout <<"v1.nops() "<<v1.nops(); 
    cout <<"  v2.nops() "<<v2.nops()<<endl; ; 
    return GiNaC::lst(); 
  }
  ret.append(  v1.op(1)*v2.op(2) - v1.op(2)*v2.op(1)); 
  ret.append(- v1.op(0)*v2.op(2) + v1.op(2)*v2.op(0)); 
  ret.append(  v1.op(0)*v2.op(1) - v1.op(1)*v2.op(0)); 
  return ret; 
}

GiNaC::ex inner(GiNaC::ex a, GiNaC::ex b, bool transposed){
  if (GiNaC::is_a<GiNaC::matrix>(a) && GiNaC::is_a<GiNaC::matrix>(b)) {
    GiNaC::matrix ma = GiNaC::ex_to<GiNaC::matrix>(a); 
    GiNaC::matrix mb = GiNaC::ex_to<GiNaC::matrix>(b); 
    if ( !transposed ) {
      if (ma.cols() != mb.cols() || ma.rows() != mb.rows() ) { 
         cout <<"Incompatible matrices "<<endl; 
         cout <<"a.cols() "<<ma.cols()<<endl; 
         cout <<"a.rows() "<<ma.rows()<<endl; 
         cout <<"b.cols() "<<mb.cols()<<endl; 
         cout <<"b.rows() "<<mb.rows()<<endl; 
         cout <<"a="<<a<<endl; 
         cout <<"b="<<b<<endl; 
         return DUMMY; 
      }
  
      GiNaC::ex ret; 
      for (int i=0; i<ma.rows(); i++) { 
        for (int j=0; j<ma.cols(); j++) { 
          ret += ma(i,j)*mb(i,j);  
        }
      }
      return ret; 
    } 
    else { 
      if (ma.cols() != mb.rows() || ma.rows() != mb.cols() ) { 
        cout <<"Incompatible matrices "<<endl; 
        cout <<"a.cols() "<<ma.cols()<<endl; 
        cout <<"a.rows() "<<ma.rows()<<endl; 
        cout <<"b.cols() "<<mb.cols()<<endl; 
        cout <<"b.rows() "<<mb.rows()<<endl; 
        cout <<"a="<<a<<endl; 
        cout <<"b="<<b<<endl; 
         return DUMMY; 
       }

      GiNaC::ex ret; 
      for (int i=0; i<ma.rows(); i++) { 
        for (int j=0; j<ma.cols(); j++) { 
          ret += ma(i,j)*mb(j,i);  
        }
      }
      return ret; 
    }
  } else if (GiNaC::is_a<GiNaC::lst>(a) 
	  && GiNaC::is_a<GiNaC::lst>(b)) {
    return inner(GiNaC::ex_to<GiNaC::lst>(a), GiNaC::ex_to<GiNaC::lst>(b)); 
  } else {
    return a*b; 
  }
}


GiNaC::ex inner(GiNaC::lst v1, GiNaC::lst v2) {
  GiNaC::ex ret; 
  if ( v1.nops() != v2.nops() ) {
    cout <<"incompatible vectors "<<endl; 
    cout <<"v1.nops() "<<v1.nops(); 
    cout <<"  v2.nops() "<<v2.nops()<<endl; ; 
    return 0; 
  }
  int i; 
  for (i = 0; i <= v1.nops()-1 ; ++i) {
    if ( GiNaC::is_a<GiNaC::lst>(v1.op(i)) && 
         GiNaC::is_a<GiNaC::lst>(v2.op(i)) ) {   
      ret += inner(GiNaC::ex_to<GiNaC::lst>(v1.op(i)), 
		   GiNaC::ex_to<GiNaC::lst>(v2.op(i))); 
    } else {
      ret += v1.op(i)*v2.op(i);  
    }
  }
  return ret; 
}

GiNaC::lst matvec(GiNaC::matrix& M, GiNaC::lst& x) {
  GiNaC::lst ret; 
  int nr = M.rows(); 
  int nc = M.cols(); 
  for (int i = 0; i < nr; i++) {
    GiNaC::ex tmp; 
    for (int j = 0; j < nc; j++) {
       tmp = tmp +  M(i,j)*(x.op(j)); 
    }
    ret.append(tmp); 
  }
  return ret; 
}


GiNaC::ex matvec(GiNaC::ex A, GiNaC::ex x) {
  if (GiNaC::is_a<GiNaC::matrix>(A) && GiNaC::is_a<GiNaC::matrix>(x)) { 
    GiNaC::matrix AA = GiNaC::ex_to<GiNaC::matrix>(A); 
    GiNaC::matrix xx = GiNaC::ex_to<GiNaC::matrix>(x); 
    return AA.mul(xx); 
  }
  return DUMMY; 

}




GiNaC::ex inner(GiNaC::exvector& v1, GiNaC::exvector& v2){
  GiNaC::ex ret; 
  for (int i=0; i< v1.size(); i++) {
    ret += v1[i]*v2[i]; 
  }
  return ret; 
}


GiNaC::ex homogenous_pol(int order, int nsd, const string a){
   if ( nsd == 1) {
     GiNaC::symbol a0(istr(a,0));
     return GiNaC::lst(a0*pow(x,order), a0, pow(x,order)); 
   } else if ( nsd == 2 ) {
     GiNaC::ex variables = GiNaC::symbolic_matrix(1,order+1, a);
     GiNaC::lst basis;  
     GiNaC::ex ret; 
     for (int i=0; i<= order; i++) {
       basis.append(pow(x,i)*pow(y,order-i)); 
       ret += variables.op(i)*basis.op(i);  
     }
     return GiNaC::lst(ret, matrix_to_lst2(variables), basis); 
   } else if ( nsd == 3 ) {
     GiNaC::lst basis;  
     for (int i=0; i<= order; i++) {
       for (int j=0; j<= order; j++) {
         for (int k=0; k<= order; k++) {
           if ( i + j + k == order ) {
             basis.append(pow(x,i)*pow(y,j)*pow(z,k)); 
	   }
	 }
       }
     }
     GiNaC::ex variables = GiNaC::symbolic_matrix(1,basis.nops(), a);
     GiNaC::ex ret; 
     for (int i=0; i<basis.nops(); i++) { 
       ret += variables.op(i)*basis.op(i); 
     }
     return GiNaC::lst(ret, matrix_to_lst2(variables), basis); 
   } else { 
     cout <<"Homogenous polynomials only implemented in 1D, 2D and 3D"; 
     return DUMMY; 
   }

}

GiNaC::lst homogenous_polv(int no_fields, int order, int nsd, const string a){
  GiNaC::lst ret1;  // contains the polynom  
  GiNaC::lst ret2;  // contains the coefficients   
  GiNaC::lst ret3;  // constains the basis functions  
  GiNaC::lst basis_tmp; 
  for (int i=1; i<= no_fields; i++) { 
    GiNaC::lst basis; 
    std::ostringstream s; 
    s <<a<<""<<i<<"_"; 
    GiNaC::ex polspace = homogenous_pol(order, nsd, s.str()); 
    ret1.append(polspace.op(0)); 
    ret2.append(polspace.op(1)); 
    basis_tmp = GiNaC::ex_to<GiNaC::lst>(polspace.op(2)); 
    for (GiNaC::lst::const_iterator basis_iterator = basis_tmp.begin(); 
	 basis_iterator != basis_tmp.end(); ++basis_iterator) {
      GiNaC::lst tmp_lst; 
      for (int d=1; d<=no_fields; d++) tmp_lst.append(0);   
      tmp_lst.let_op(i-1) = (*basis_iterator);  
      ret3.append(tmp_lst); 
    }
  }
  return GiNaC::lst(ret1,ret2,ret3); 
}


GiNaC::ex pol(int order, int nsd, const string a) {
  GiNaC::ex ret; // ex to return   
  int dof; // degrees of freedom  
  GiNaC::ex A;    // ex holding the coefficients a_0 .. a_dof  
  GiNaC::lst basis; 


  if (nsd == 1) {
    /* 1D: 
     * P^n = a_0 + a_1*x + .... + a_n*x^n 
     * dof : n+1
     */ 
    dof = order+1; 
    A = GiNaC::symbolic_matrix(1,dof, a); 
    int o=0; 
    for (GiNaC::const_iterator i = A.begin(); i != A.end(); ++i)  {  
      ret += (*i)*pow(x,o); 
      basis.append(pow(x,o)); 
      o++; 
    }
  }
  else if ( nsd == 2) {

    /* 2D: structure of coefficients (a_i)  
     * [ a_0      a_1 x     a_3 x^2     a_6 x^3  
     * [ a_2 y    a_4 xy    a_7 x^2y  
     * [ a_5 y^2  a_8 xy^2  
     * [ a_9 y^3 
     */
    dof = (order+1)*(order+2)/2; 
    A = GiNaC::symbolic_matrix(1, dof , a); 

    size_t i=0; 
    for (int o = 0; o <= order; o++) {
      for (int d = 0; d <= o; d++) { 
	ret += A.op(i)*pow(y,d)*pow(x,o-d); 
	basis.append(pow(y,d)*pow(x,o-d));
	i++; 
      }
    }
  }
  else if (nsd = 3) {

  /* Similar structure as in 2D, but 
   * structured as a tetraheder, i.e., 
   *   a_o + a_1 x + a_2 y + a_3 z 
   * + a_4 x^2 + a_5 xy +  
   */
    dof = 0; 
    for (int j=0; j<= order; j++) { 
      dof += (j+1)*(j+2)/2; 
    }
    A = GiNaC::symbolic_matrix(1, dof , a); 


    size_t i=0; 
    for (int o = 0; o <= order; o++) {
      for (int d = 0; d <= o; d++) { 
        for (int f = 0; f <= o; f++) { 
	  if ( o-d-f >= 0) {
            ret += A.op(i)*pow(y,f)*pow(z,d)*pow(x,o-d-f);  
	    basis.append(pow(y,f)*pow(z,d)*pow(x,o-d-f));
            i++; 
	  }
	}
      }
    }
  }
  return GiNaC::lst(ret,matrix_to_lst2(A), basis); 
}

GiNaC::lst polv(int no_fields, int order, int nsd, const string a){
  GiNaC::lst ret1;  // contains the polynom  
  GiNaC::lst ret2;  // contains the coefficients   
  GiNaC::lst ret3;  // constains the basis functions  
  GiNaC::lst basis_tmp; 
  for (int i=1; i<= no_fields; i++) { 
    GiNaC::lst basis; 
    std::ostringstream s; 
    s <<a<<""<<i<<"_"; 
    GiNaC::ex polspace = pol(order, nsd, s.str()); 
    ret1.append(polspace.op(0)); 
    ret2.append(polspace.op(1)); 
    basis_tmp = GiNaC::ex_to<GiNaC::lst>(polspace.op(2)); 
    for (GiNaC::lst::const_iterator basis_iterator = basis_tmp.begin(); 
	 basis_iterator != basis_tmp.end(); ++basis_iterator) {
      GiNaC::lst tmp_lst; 
      for (int d=1; d<=no_fields; d++) tmp_lst.append(0);   
      tmp_lst.let_op(i-1) = (*basis_iterator);  
      ret3.append(tmp_lst); 
    }
  }
  return GiNaC::lst(ret1,ret2,ret3); 



/* Old Code: 
  GiNaC::lst ret; 
  for (int i=1; i<= nsd; i++) { 
    std::ostringstream s; 
    s <<a<<"^"<<i<<"_"; 
    GiNaC::ex p = pol(order, nsd, s.str()); 
    ret.append(p); 
  }
  return ret; 
  */
}


GiNaC::ex polb(int order, int nsd, const string a) {

  GiNaC::ex ret; // ex to return   
  int dof; // degrees of freedom  
  GiNaC::ex A;    // ex holding the coefficients a_0 .. a_dof  
  GiNaC::lst basis; 

  if (nsd == 1) {
    /* 1D: 
     * P^n = a_0 + a_1*x + .... + a_n*x^n 
     * dof : n+1
     */ 
    dof = order+1; 
    A = GiNaC::symbolic_matrix(1,dof, a); 
    int o=0; 
    for (GiNaC::const_iterator i = A.begin(); i != A.end(); ++i)  {  
      ret += (*i)*pow(x,o); 
      basis.append(pow(x,o)); 
      o++; 
    }
  }
  else if ( nsd == 2) {

    /* 2D: structure of coefficients (a_i)  
     * [ a_0      a_1 x     a_3 x^2     a_6 x^3  
     * [ a_2 y    a_4 xy    a_7 x^2y  
     * [ a_5 y^2  a_8 xy^2  
     * [ a_9 y^3 
     */


    dof = (order+1)*(order+1); 
    A = GiNaC::symbolic_matrix(1, dof , a); 


    size_t i=0; 
    for (int o = 0; o <= order; o++) {
      for (int d = 0; d <= order; d++) { 
	ret += A.op(i)*pow(y,d)*pow(x,o); 
	basis.append(pow(y,d)*pow(x,o));
	i++; 
      }
    }
  }
  else if (nsd = 3) {

  /* Similar structure as in 2D, but 
   * structured as a tetraheder, i.e., 
   *   a_o + a_1 x + a_2 y + a_3 z 
   * + a_4 x^2 + a_5 xy +  
   */
    dof = (order+1)*(order+1)*(order+1); 
    A = GiNaC::symbolic_matrix(1, dof , a); 


    size_t i=0; 
    for (int o = 0; o <= order; o++) {
      for (int d = 0; d <= order; d++) { 
        for (int f = 0; f <= order; f++) { 
            ret += A.op(i)*pow(y,f)*pow(z,d)*pow(x,o);  
	    basis.append(pow(y,f)*pow(z,d)*pow(x,o));
            i++; 
	}
      }
    }
  }

  return GiNaC::lst(ret,matrix_to_lst2(A), basis); 
}

GiNaC::ex div(GiNaC::ex v){
  GiNaC::ex ret = DUMMY;
  if (GiNaC::is_a<GiNaC::matrix>(v)) {
    GiNaC::matrix m = GiNaC::ex_to<GiNaC::matrix>(v); 
    if ( m.cols() == 1) { 
      if (nsd == 1) {
        ret = diff(m,x);
      } else if (nsd == 2) {
        ret = diff(m.op(0),x) + diff(m.op(1),y) ;
      } else if (nsd == 3) {
        ret = diff(m.op(0),x) + diff(m.op(1),y) + diff(m.op(2),z) ;
      }
    } else { 
      GiNaC::matrix retm = GiNaC::matrix(m.cols(),1); 
      if ( nsd != m.rows() ) {
         throw(std::invalid_argument("The number of rows must equal nsd."));
      }
      GiNaC::symbol xr; 
      GiNaC::ex tmp; 
      for (int c=0; c<m.cols(); c++){
        for (int r=0; r<m.rows(); r++){
          if (r+1 == 1) xr = x; 
          if (r+1 == 2) xr = y; 
          if (r+1 == 3) xr = z; 
          retm(c,0) += diff(m(c,r), xr); 
        }
      }
      ret = retm; 
    }
  }
  return ret;
}

GiNaC::ex div(GiNaC::lst& v) {
  nsd = v.nops();  
  GiNaC::ex ret; 
  if (nsd == 1) {
    ret = v.op(0).diff(x);
  }
  else if (nsd == 2) {
    ret = v.op(0).diff(x) + v.op(1).diff(y); 
  }
  else if (nsd == 3) {
    ret = v.op(0).diff(x) + v.op(1).diff(y) + v.op(2).diff(z); 
  }
  return ret; 
}

GiNaC::ex div(GiNaC::exvector& v) {
  GiNaC::ex ret; 
  if (nsd == 2) {
    ret = v[0].diff(x) + v[1].diff(y); 
  }
  else if (nsd == 3) {
    ret = v[0].diff(x) + v[1].diff(y) + v[2].diff(z); 
  }
  return ret; 
}

GiNaC::lst coeffs(GiNaC::lst pols) {
  GiNaC::lst cc; 
  GiNaC::lst tmp; 
  for (int i=0; i<= pols.nops()-1; i++) {
    tmp = coeffs(pols.op(i)); 
    cc = collapse(GiNaC::lst(cc, tmp)); 
  }
  return cc; 
}

GiNaC::lst coeffs(GiNaC::ex pol) {
  GiNaC::lst cc; 
  GiNaC::ex c, b; 
  for (int i=pol.ldegree(x); i<=pol.degree(x); ++i) {
    for (int j=pol.ldegree(y); j<=pol.degree(y); ++j) {
      for (int k=pol.ldegree(z); k<=pol.degree(z); ++k) {
        c = pol.coeff(x,i).coeff(y, j).coeff(z,k); 
        if ( c != 0 ) cc.append(c); 
      }
    }
  }
  return cc; 
}





GiNaC::exvector coeff(GiNaC::ex pol) {
  GiNaC::exvector cc; 
  GiNaC::ex c, b; 
  for (int i=pol.ldegree(x); i<=pol.degree(x); ++i) {
    for (int j=pol.ldegree(y); j<=pol.degree(y); ++j) {
      for (int k=pol.ldegree(z); k<=pol.degree(z); ++k) {
        c = pol.coeff(x,i).coeff(y, j).coeff(z,k); 
        if ( c != 0 ) cc.insert(cc.begin(),c); 
      }
    }
  }
  return cc; 
}

int dirac(int i, int j) {
  if (i==j) return 1; 
  else return 0; 
}

ex_ex_map pol2basisandcoeff(GiNaC::ex e) { 
  e = expand(e); 
  GiNaC::ex c; 
  GiNaC::ex b; 
  ex_ex_map map; 
  for (int i=e.ldegree(x); i<=e.degree(x); ++i) {
    for (int j=e.ldegree(y); j<=e.degree(y); ++j) {
      for (int k=e.ldegree(z); k<=e.degree(z); ++k) {
        c = e.coeff(x,i).coeff(y, j).coeff(z,k); 
        b = pow(x,i)*pow(y,j)*pow(z,k); 
        map[b] = c;  
      }
    }
  }
  return map; 
}

void print(ex_int_map map) {
  GiNaC::ex b; 
  int c; 
  ex_int_it iter; 
  iter = map.begin(); 
  cout <<"{" <<b<<":"<<c; 
  for (iter = map.begin(); iter != map.end(); iter++) {  
    b = (*iter).first; c = map[b]; 
    cout <<", "<<b<<":"<<c; 
  }
  cout <<"}"<<endl; 

}



void print(ex_ex_map map) {
  GiNaC::ex b; 
  GiNaC::ex c; 
  ex_ex_it iter; 
  cout <<"{" <<b<<":"<<c; 
  for (iter = map.begin(); iter != map.end(); iter++) {  
    b = (*iter).first; c = map[b]; 
    cout <<", "<<b<<":"<<c; 
  }
  cout <<"}"<<endl; 

}


GiNaC::lst ex2equations(GiNaC::ex rel) {
  GiNaC::ex lhs = rel.lhs();  
  GiNaC::ex rhs = rel.rhs(); 

  GiNaC::ex l; 
  GiNaC::ex r; 

  GiNaC::lst eqs; 

  for (int i=lhs.ldegree(x); i<=lhs.degree(x); ++i) {
    for (int j=lhs.ldegree(y); j<=lhs.degree(y); ++j) {
      for (int k=lhs.ldegree(z); k<=lhs.degree(z); ++k) {
        l = lhs.coeff(x,i).coeff(y, j).coeff(z,k); 
        r = rhs.coeff(x,i).coeff(y, j).coeff(z,k); 
//	if (! (l == 0 && r == 0 ) )  eqs.append(l == r); OLD VERSION 
	if ( (l != 0 && (r == 0 || r == 1) ) )  eqs.append(l == r); 
      }
    }
  }
  eqs.sort(); 
  return eqs; 
}

GiNaC::lst collapse(GiNaC::lst l) {
  GiNaC::lst lc;  
  GiNaC::lst::const_iterator iter1, iter2; 

  for (iter1 = l.begin(); iter1 != l.end(); ++iter1) {
     if (GiNaC::is_a<GiNaC::lst>(*iter1)) {
       for (iter2 = GiNaC::ex_to<GiNaC::lst>(*iter1).begin(); iter2 != GiNaC::ex_to<GiNaC::lst>(*iter1).end(); ++iter2) {
          lc.append(*iter2); 
       }
     } else {
       lc.append(*iter1); 
     }
  }
  lc.sort(); 
  lc.unique(); 
return lc; 
}


GiNaC::matrix equations2matrix(const GiNaC::ex &eqns, const GiNaC::ex &symbols) {

  GiNaC::matrix sys(eqns.nops(),symbols.nops());
  GiNaC::matrix rhs(eqns.nops(),1);
  GiNaC::matrix vars(symbols.nops(),1);
	
  for (size_t r=0; r<eqns.nops(); r++) {
    const GiNaC::ex eq = eqns.op(r).op(0)-eqns.op(r).op(1); // lhs-rhs==0
    GiNaC::ex linpart = eq;
    for (size_t c=0; c<symbols.nops(); c++) {
      const GiNaC::ex co = eq.coeff(GiNaC::ex_to<GiNaC::symbol>(symbols.op(c)),1);
      linpart -= co*symbols.op(c);
      sys(r,c) = co;
    }
    linpart = linpart.expand();
    rhs(r,0) = -linpart;
  }
  return sys; 
}

GiNaC::ex grad(GiNaC::ex f) {
  if (GiNaC::is_a<GiNaC::matrix>(f)) {
     GiNaC::matrix m = GiNaC::ex_to<GiNaC::matrix>(f); 
     GiNaC::matrix ret_m(nsd,m.rows()); 
     for (int r=0; r< m.rows(); r++) { 
       if (nsd == 1) {
//         ret_m(0,r) = diff(m.op(r),x); 
         return diff(f, x); 
       } else if ( nsd == 2) {
         ret_m(0,r) = diff(m.op(r),x); 
         ret_m(1,r) = diff(m.op(r),y);  
       } else if ( nsd == 3) {
         ret_m(0,r) = diff(m.op(r),x); 
         ret_m(1,r) = diff(m.op(r),y);  
         ret_m(2,r) = diff(m.op(r),z);  
       }
     }
     return ret_m; 
  } else {
  
    if (nsd == 1) {
//      return GiNaC::matrix(nsd,1,GiNaC::lst(diff(f,x))); 
      return diff(f,x); 
    } else if ( nsd == 2) {
      return GiNaC::matrix(nsd,1,GiNaC::lst(diff(f,x), diff(f,y)));  
    } else if ( nsd == 3) {
      return GiNaC::matrix(nsd,1,GiNaC::lst(diff(f,x), diff(f,y), diff(f,z)));  
    } else {
      throw(std::invalid_argument("nsd must be either 1, 2, or 3."));
      return GiNaC::lst(); 
    }
  }
}

GiNaC::lst lst_equals(GiNaC::ex a, GiNaC::ex b) { 
  GiNaC::lst ret; 
  if ( (GiNaC::is_a<GiNaC::lst>(a)) && (GiNaC::is_a<GiNaC::lst>(b)) /*&& (a.nops() == b.nops())*/ ) { 
    for (int i=0; i<= a.nops()-1; i++) {
      ret.append(b.op(i) == a.op(i)); 
    }
  } else if ( !(GiNaC::is_a<GiNaC::lst>(a)) && !(GiNaC::is_a<GiNaC::lst>(b))) { 
      ret.append(b == a); 
  } else if ( !(GiNaC::is_a<GiNaC::lst>(a)) && (GiNaC::is_a<GiNaC::lst>(b))) { 
      ret.append(b.op(0) == a); 
  } else {
    throw(std::invalid_argument("Make sure that the lists a and b are comparable."));
  }
  return ret; 
}

GiNaC::ex lst_to_matrix2(const GiNaC::lst& l)
{
     GiNaC::lst::const_iterator itr, itc;
 
     // Find number of rows and columns
     size_t rows = l.nops(), cols = 0;
     for (itr = l.begin(); itr != l.end(); ++itr) {
         if (!GiNaC::is_a<GiNaC::lst>(*itr))
//              throw (std::invalid_argument("lst_to_matrix: argument must be a list of lists"));
             cols = 1; 
         if (itr->nops() > cols)
             cols = itr->nops();
     }
     // Allocate and fill matrix
     GiNaC::matrix &M = *new GiNaC::matrix(rows, cols);
     M.setflag(GiNaC::status_flags::dynallocated);
 
     unsigned i;
     for (itr = l.begin(), i = 0; itr != l.end(); ++itr, ++i) {
         unsigned j;
	 if (cols == 1) {
             M(i, 0) = *itr;
	 } else {
           for (itc = GiNaC::ex_to<GiNaC::lst>(*itr).begin(), j = 0; itc != GiNaC::ex_to<GiNaC::lst>(*itr).end(); ++itc, ++j)
               M(i, j) = *itc;
	 }
     }
     return M;
}


GiNaC::lst matrix_to_lst2(const GiNaC::ex& m) {
   if (GiNaC::is_a<GiNaC::matrix>(m)) {
     GiNaC::matrix A = GiNaC::ex_to<GiNaC::matrix>(m);  
     int cols = A.cols(); 
     int rows = A.rows(); 
 
     GiNaC::lst ret; 
     if ( cols == 1 ) {
       for (int i=0; i<=A.rows()-1; i++) { 
         ret.append(A(i,0)); 
       }
     } else if ( rows == 1 ) {
       for (int i=0; i<=A.cols()-1; i++) { 
         ret.append(A(0,i)); 
       }
     } else {
       for (int i=0; i<=A.rows()-1; i++) { 
         GiNaC::lst rl; 
         for (int j=0; j<=A.cols()-1; j++) { 
           rl.append(A(i,j)); 
         }
         ret.append(rl); 
       }
     }
     return ret; 
   } else { 
     return GiNaC::lst(); 
   }
}



int find(GiNaC::ex e, GiNaC::lst list){
  for (int i=0; i< list.nops(); i++) {
    if ( e == list.op(i) ) return i; 
  }
  return -1; 
}

void visitor_subst_pow(GiNaC::ex e, ex_ex_map& map, ex_int_map& intmap, string a) {  
  static int i=0;  
  if (map.find(e) != map.end()) { 
    intmap[e] = intmap[e]+1;
    return;   
  }
  if (GiNaC::is_exactly_a<GiNaC::power>(e)) { 
    std::ostringstream s; 
    s <<a<<i++; 
    map[e] = GiNaC::symbol(s.str()); 
    intmap[e] = 0;  
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
//       cout <<"power e "<<e2<<endl; 
       visitor_subst_pow(e2,map,intmap, a); 
    }
  }
  else if (GiNaC::is_a<GiNaC::function>(e)) { 
    std::ostringstream s; 
    s <<a<<i++; 
    map[e] = GiNaC::symbol(s.str()); 
    intmap[e] = 0;  
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
//       cout <<"function e "<<e2<<endl; 
       visitor_subst_pow(e2,map,intmap, a); 
    }
  }
  else if (GiNaC::is_a<GiNaC::mul>(e)) { 
    if (e.nops() > 4 && e.nops() < 10 ) { 
      std::ostringstream s; 
      s <<a<<i++; 
      map[e] = GiNaC::symbol(s.str()); 
      intmap[e] = 0;  
    }

    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
       visitor_subst_pow(e2,map,intmap, a); 
    }
  }
  else if (GiNaC::is_a<GiNaC::add>(e)) { 
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
       visitor_subst_pow(e2,map,intmap,a); 
    }
  }


}




void check_visitor(GiNaC::ex e, GiNaC::lst& exlist) {
  if (find(e, exlist) >= 0) return;   

//  cout <<"ex e "<<e<<endl; 
  if (GiNaC::is_a<GiNaC::numeric>(e)) { 
  }
  else if (GiNaC::is_a<GiNaC::add>(e) ) {
//    cout <<"e "<<e <<endl; 
//    cout <<"e.nops() "<<e.nops() <<endl; 
    if (e.nops() > 4 && e.nops() < 10 ) exlist.append(e); 
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
//       cout <<"add e "<<e2<<endl; 
//       exlist.append(e2); 
       check_visitor(e2,exlist); 
    }
  } 
  else if (GiNaC::is_a<GiNaC::mul>(e)) { 
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
//       cout <<"mul e "<<e2<<endl; 
       exlist.append(e2); 
       check_visitor(e2,exlist); 
    }
  }
  else if (GiNaC::is_a<GiNaC::lst>(e)) { 
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
//       cout <<"GiNaC::lst e "<<e2<<endl; 
//       exlist.append(e2); 
       check_visitor(e2,exlist); 
    }
  }
  else if (GiNaC::is_exactly_a<GiNaC::power>(e)) { 
    exlist.append(e); 
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
//       cout <<"power e "<<e2<<endl; 
       check_visitor(e2,exlist); 
    }
  }
  else if (GiNaC::is_a<GiNaC::function>(e)) { 
    exlist.append(e); 
    for (int i=0; i< e.nops(); i++) { 
       GiNaC::ex e2 = e.op(i);  
//       cout <<"function e "<<e2<<endl; 
       check_visitor(e2,exlist); 
    }
  }



  else {
//       exlist.append(e); 
//    cout <<"atom e "<<e<<endl; 
  }

  exlist.sort(); 
  exlist.unique(); 
}


string istr(string a, int b) {
  std::ostringstream s; 
  s <<a<<b; 
  return s.str(); 
}

void EQUAL_OR_DIE(GiNaC::ex e, char* s) {
  if (!compare(e, string(s))) { 
    cout <<"ERROR: expression e: " <<e<<" is not equal to "<<s<<endl; 
    exit(-1); 
  }
}


bool compare(GiNaC::ex e, string s) { 
  std::ostringstream ss; 
  ss<<e; 
  string es = ss.str(); 
  if ( es == s) return true;  
  else return false; 
}

void matrix_from_equations(const GiNaC::ex &eqns, const GiNaC::ex &symbols, GiNaC::matrix& A, GiNaC::matrix& b) {
	// build matrix from equation system
  GiNaC::matrix sys(eqns.nops(),symbols.nops());
  GiNaC::matrix rhs(eqns.nops(),1);
  GiNaC::matrix vars(symbols.nops(),1);
	
  for (size_t r=0; r<eqns.nops(); r++) {
    const GiNaC::ex eq = eqns.op(r).op(0)-eqns.op(r).op(1); // lhs-rhs==0
    GiNaC::ex linpart = eq;
    for (size_t c=0; c<symbols.nops(); c++) {
      const GiNaC::ex co = eq.coeff(GiNaC::ex_to<GiNaC::symbol>(symbols.op(c)),1);
      linpart -= co*symbols.op(c);
      sys(r,c) = co;
    }
    linpart = linpart.expand();
    rhs(r,0) = -linpart;
  }
  A = sys; 
  b = rhs; 
}


void print(std::map<std::pair<int,int>, GiNaC::ex>& A) {
  map<std::pair<int,int>,GiNaC::ex>::iterator iter; 
  for (iter = A.begin(); iter != A.end() ; iter++) {
      cout <<"A["<<(*iter).first.first<<","<<(*iter).first.second<<"]="<<(*iter).second<<endl; 
  }
}


GiNaC::ex legendre1D(const GiNaC::symbol x, int n){
  GiNaC::ex P;
// Rodrigue's formula for Legendre polynomial of 1D
  // The interval [-1, 1]
   P=1/(pow(2,n)*GiNaC::factorial(n))*GiNaC::diff(GiNaC::pow((x*x-1),n),x,n);
  // -----------------
  // The interval [0,1]
//  GiNaC::ex xx = 2*x - 1; 
 // P=1/(pow(2,2*n)*GiNaC::factorial(n))*GiNaC::diff(GiNaC::pow((xx*xx-1),n),x,n);
  return P;
}


GiNaC::ex legendre(int order, int nsd, const string s){
  // The Legendre polynomials to be used in FiniteElement
  GiNaC::ex leg;
  GiNaC::ex A;
  GiNaC::lst basis; 
  int dof;

  GiNaC::ex b; 

  // 1D
  if(nsd == 1){
    dof = order+1;
    A = GiNaC::symbolic_matrix(1,dof,s);
    int o=0;
    for(GiNaC::const_iterator i = A.begin(); i!=A.end(); ++i){
      b= legendre1D(x,o);
      leg+= (*i)*b;
      basis.append(b); 
      o++;
    }
  }
  // 2D
  else if(nsd == 2){  // NB: Only for tensor products on TRIANGLES (not boxes)
    /* 2D: structure of coefficients (a_i)
     * [ a_0           a_1 P_1(x)           a_3 P_2(x)        a_6 P_3(x)
     * [ a_2 P_1(y)    a_4 P_1(x)*P_1(y)    a_7 P_2(x)*P_1(y)
     * [ a_5 P_2(y)    a_8 P_1(x)*P_2(y)
     * [ a_9 P_3(y)
     */
    dof = (order+1)*(order+2)/2;
    A = GiNaC::symbolic_matrix(1,dof,s);
    size_t i=0;
    for (int o = 0; o <= order; o++) {
      for (int d = 0; d <= o; d++) {
        b = legendre1D(y,d)*legendre1D(x,o-d);
        leg += A.op(i)*b;
	basis.append(b); 
	i++;

      }
    }
  }
  else if(nsd==3){
    dof = 0; 
    for (int j=0; j<= order; j++) { 
      dof += (j+1)*(j+2)/2; 
    }
    A = GiNaC::symbolic_matrix(1, dof , s); 


    size_t i=0; 
    for (int o = 0; o <= order; o++) {
      for (int d = 0; d <= o; d++) { 
        for (int f = 0; f <= o; f++) { 
	  if ( o-d-f >= 0) {
            b = legendre1D(y,f)*legendre1D(z,d)*legendre1D(x,o-d-f);
            leg += A.op(i)*b; 
	    basis.append(b);
            i++; 
	  }
	}
      }
    }
  }
  return GiNaC::lst(leg,matrix_to_lst2(A), basis); 
}

GiNaC::lst legendrev(int no_fields, int order, int nsd, const string a) {  
  GiNaC::lst ret1;  // contains the polynom  
  GiNaC::lst ret2;  // contains the coefficients   
  GiNaC::lst ret3;  // constains the basis functions  
  GiNaC::lst basis_tmp; 
  for (int i=1; i<= no_fields; i++) { 
    GiNaC::lst basis; 
    std::ostringstream s; 
    s <<a<<""<<i<<"_"; 
    GiNaC::ex polspace = legendre(order, nsd, s.str()); 
    ret1.append(polspace.op(0)); 
    ret2.append(polspace.op(1)); 
    basis_tmp = GiNaC::ex_to<GiNaC::lst>(polspace.op(2)); 
    for (GiNaC::lst::const_iterator basis_iterator = basis_tmp.begin(); 
	 basis_iterator != basis_tmp.end(); ++basis_iterator) {
      GiNaC::lst tmp_lst; 
      for (int d=1; d<=no_fields; d++) tmp_lst.append(0);   
      tmp_lst.let_op(i-1) = (*basis_iterator);  
      ret3.append(tmp_lst); 
    }
  }
  return GiNaC::lst(ret1,ret2,ret3); 
}










