#ifndef TOOLS_IS_INCLUDED 
#define TOOLS_IS_INCLUDED 

#include <ginac/ginac.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string>
#include <sstream>
#include <algorithm>
#include <map>
#include <stdexcept>
#include <vector>



//using namespace GiNaC; 
using namespace std; 

extern GiNaC::symbol x; 
extern GiNaC::symbol y; 
extern GiNaC::symbol z; 
extern int nsd; 
extern GiNaC::symbol infinity; 
extern GiNaC::symbol DUMMY; 

/* Vector of ex */ 
typedef vector<GiNaC::ex> ex_vector; 

/* Dictionary with ex as keys and values */ 
typedef map<GiNaC::ex,GiNaC::ex, GiNaC::ex_is_less>  ex_ex_map;
typedef map<GiNaC::ex,int, GiNaC::ex_is_less>  ex_int_map;

/* Iterator associated with the ex-ex dictionary */ 
typedef map<GiNaC::ex,GiNaC::ex>::iterator ex_ex_it;  
typedef map<GiNaC::ex,int>::iterator ex_int_it;  

/* Vector of vectors of ex */ 
typedef vector<GiNaC::exvector> ex_vector_vector;


// inner product of vectors or lst
GiNaC::ex inner(GiNaC::ex a, GiNaC::ex b, bool transposed = false); 
GiNaC::ex inner(GiNaC::exvector& v1, GiNaC::exvector& v2); 
GiNaC::ex inner(GiNaC::lst v1, GiNaC::lst v2); 
GiNaC::lst cross(GiNaC::lst& v1, GiNaC::lst& v2); 


// generates a polynom of arbitrary order on a line, a triangle, or a tetrahedron 
GiNaC::ex pol(int order, int nsd, const string a);  
// generates a vector polynom of arbitrary order on a line, a triangle or a tetrahedron 
GiNaC::lst polv(int no_fields, int order, int nsd, const string a);  
// generates a polynom of arbitrary order on a square or a box   
GiNaC::ex polb(int order, int nsd, const string a);  
// generates a vector polynom of arbitrary order on a squart or a box  
//lst polbv(int order, int nsd, const string a);  


// generates a homogenous polynom of arbitrary order on a line, a triangle, or a tetrahedron 
GiNaC::ex homogenous_pol(int order, int nsd, const string a);  
// generates a homogenous vector polynom of arbitrary order 
GiNaC::lst homogenous_polv(int no_fields, int order, int nsd, const string a);  

// generates a Legendre polynom of arbitrary order 
GiNaC::ex legendre(int order, int nsd, const string a);  
// generates a Legendre vector polynom of arbitrary order 
GiNaC::lst legendrev(int no_fields, int order, int nsd, const string a);  




// matrix vector product
GiNaC::lst matvec(GiNaC::matrix& M, GiNaC::lst& x);  
GiNaC::ex  matvec(GiNaC::ex A, GiNaC::ex x); 



// extracts the coefficents from a polynomial
GiNaC::exvector coeff(GiNaC::ex pol); 
GiNaC::lst coeffs(GiNaC::ex pol); 
GiNaC::lst coeffs(GiNaC::lst pols); 
// extract the basisfunctions and corresponding coefficents from a polynomial
ex_ex_map  pol2basisandcoeff(GiNaC::ex e); 


// the divergence 
GiNaC::ex div(GiNaC::exvector& v);  
GiNaC::ex div(GiNaC::lst& v);  
GiNaC::ex div(GiNaC::ex v);  
GiNaC::ex grad(GiNaC::ex f); 

// print functions 
#ifndef SWIG 
void print(GiNaC::exvector& v); 
void print(GiNaC::lst& l); 
void print(ex_ex_map m); 
void print(ex_int_map m); 
void print(std::map<std::pair<int,int>, GiNaC::ex>& A); 
#endif 

// dirac delta function 
int dirac(int i, int j); 


GiNaC::lst ex2equations(GiNaC::ex rel); 
GiNaC::lst collapse(GiNaC::lst l);  

GiNaC::matrix equations2matrix (const GiNaC::ex &eqns, const GiNaC::ex &symbols); 

GiNaC::lst lst_equals(GiNaC::ex a, GiNaC::ex b);  
GiNaC::ex lst_to_matrix2(const GiNaC::lst & l); 
GiNaC::lst matrix_to_lst2(const GiNaC::ex & m ); 

// FIXME bad name
int find(GiNaC::ex e, GiNaC::lst list); 
void check_visitor(GiNaC::ex e, GiNaC::lst& exlist);  
void visitor_subst_pow(GiNaC::ex e, ex_ex_map& map, ex_int_map& intmap, string a);  

string istr(string a, int b); 

bool compare(GiNaC::ex e, string s); 

void EQUAL_OR_DIE(GiNaC::ex e, char* s);  

void matrix_from_equations(const GiNaC::ex &eqns, const GiNaC::ex &symbols, GiNaC::matrix &A, GiNaC::matrix& b); 
 

#endif 




