#include <SyFi.h>
#include <fstream>

using namespace GiNaC; 
using namespace std; 

void compute_Poisson_element_matrix(
		FE& fe, 
                Dof& dof, 
		std::map<std::pair<int,int>, GiNaC::ex>& A)
{
  std::pair<int,int> index; 

   // Insert the local degrees of freedom into the global Dof
  for (int i=0; i< fe.nbf(); i++) {
    dof.insert_dof(1,i,fe.dof(i)); 
  }

  Polygon& domain = fe.getPolygon(); 

  // The term (grad u, grad v)  
  for (int i=0; i< fe.nbf(); i++) {
    index.first = dof.glob_dof(fe.dof(i));               // fetch the global dof for Ni 
    for (int j=0; j< fe.nbf(); j++) {
      index.second = dof.glob_dof(fe.dof(j));            // fetch the global dof for Nj 
      GiNaC::ex nabla = inner(grad(fe.N(i)),grad(fe.N(j))); 
      GiNaC::ex Aij = domain.integrate(nabla);   // compute the integral  
      A[index] += Aij;                           // add to global matrix 
    }
  }
}

void code_gen2D(FE& fe){ 
  cout <<csrc; 
  for (int i=0; i< fe.nbf(); i++) {
    cout <<"double N"<<i<<"(double x, double y){"<<endl; 
    cout <<"  return "<<fe.N(i)<<";"<<endl; 
    cout <<"}"<<endl; 
  }

  for (int i=0; i< fe.nbf(); i++) {
    cout <<"double dN"<<i<<"dx(double x, double y){"<<endl; 
    cout <<"  return "<<diff(fe.N(i),x)<<";"<<endl; 
    cout <<"}"<<endl; 
  }

  for (int i=0; i< fe.nbf(); i++) {
    cout <<"double dN"<<i<<"dy(double x, double y){"<<endl; 
    cout <<"  return "<<diff(fe.N(i),y)<<";"<<endl; 
    cout <<"}"<<endl; 
  }

  cout <<"double N(int i, double x, double y){"<<endl; 
  cout <<"  switch(i) {"<<endl; 
  for (int i=0; i< fe.nbf(); i++) {
    cout <<"    case "<<i<<" :  return N"<<i<<"(x,y);"<<endl;  
  }
  cout <<"  }"<<endl; 
  cout <<"}"<<endl; 

  cout <<"double dNdx(int i, double x, double y){"<<endl; 
  cout <<"  switch(i) {"<<endl; 
  for (int i=0; i< fe.nbf(); i++) {
    cout <<"    case "<<i<<" :  return dN"<<i<<"dx(x,y);"<<endl;  
  }
  cout <<"  }"<<endl; 
  cout <<"}"<<endl; 

  cout <<"double dNdy(int i, double x, double y){"<<endl; 
  cout <<"  switch(i) {"<<endl; 
  for (int i=0; i< fe.nbf(); i++) {
    cout <<"    case "<<i<<" :  return dN"<<i<<"dy(x,y);"<<endl;  
  }
  cout <<"  }"<<endl; 
  cout <<"}"<<endl; 
}

int main() {
  int order =1; 
  Triangle triangle(lst(0,0), lst(1,0), lst(0,1));   
  LagrangeFE fe; 
  fe.set(2); 
  fe.set(triangle); 
  fe.compute_basis_functions(); 
  code_gen2D(fe); 


  Dof dof; 
  std::map<std::pair<int,int>, ex> A; 
  compute_Poisson_element_matrix(fe, dof, A); 
  cout <<"C code format on output "<<endl; 
  cout <<csrc; 
  print(A); 





}
