
#include <math.h>
#include <utility>
#include <map>
#include <ptv.h>
#include <DofT.h>
#include <ScalarFunction.h>

using namespace std; 



/* ------------- order 1 ---------------*/


 
void matrix_factory_conv_2D_order1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2],
         double v0, double v1){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    iidof[3];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00+x0;  dof1[1]=G10+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G01+x0;  dof2[1]=y0+G11;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[3][3];
  A[0][0]=(-1.6666666666666666e-01*v1*Jinv11-1.6666666666666666e-01*v1*Jinv10-1.6666666666666666e-01*Jinv00*v0-1.6666666666666666e-01*Jinv01*v0)*D;
  A[0][1]=(1.6666666666666666e-01*v1*Jinv10+1.6666666666666666e-01*Jinv00*v0)*D;
  A[0][2]=(1.6666666666666666e-01*v1*Jinv11+1.6666666666666666e-01*Jinv01*v0)*D;
  A[1][0]=(-1.6666666666666666e-01*v1*Jinv11-1.6666666666666666e-01*v1*Jinv10-1.6666666666666666e-01*Jinv00*v0-1.6666666666666666e-01*Jinv01*v0)*D;
  A[1][1]=(1.6666666666666666e-01*v1*Jinv10+1.6666666666666666e-01*Jinv00*v0)*D;
  A[1][2]=(1.6666666666666666e-01*v1*Jinv11+1.6666666666666666e-01*Jinv01*v0)*D;
  A[2][0]=(-1.6666666666666666e-01*v1*Jinv11-1.6666666666666666e-01*v1*Jinv10-1.6666666666666666e-01*Jinv00*v0-1.6666666666666666e-01*Jinv01*v0)*D;
  A[2][1]=(1.6666666666666666e-01*v1*Jinv10+1.6666666666666666e-01*Jinv00*v0)*D;
  A[2][2]=(1.6666666666666666e-01*v1*Jinv11+1.6666666666666666e-01*Jinv01*v0)*D;
  int nbf = 3;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 ---------------*/


 
void matrix_factory_conv_2D_order2 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2],
         double v0, double v1){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    iidof[6];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/2.0+x0;  dof1[1]=G10/2.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G00+x0;  dof2[1]=G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=G01/2.0+x0;  dof3[1]=y0+G11/2.0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G01/2.0+G00/2.0+x0;  dof4[1]=G10/2.0+y0+G11/2.0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01+x0;  dof5[1]=y0+G11;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[6][6];
  A[0][0]=(-6.6666666666666666e-02*v0*Jinv00-6.6666666666666666e-02*v0*Jinv01-6.6666666666666666e-02*Jinv10*v1-6.6666666666666666e-02*v1*Jinv11)*D;
  A[0][1]=(1.0000000000000001e-01*v0*Jinv00+3.3333333333333333e-02*v0*Jinv01+1.0000000000000001e-01*Jinv10*v1+3.3333333333333333e-02*v1*Jinv11)*D;
  A[0][2]=(-3.3333333333333333e-02*v0*Jinv00-3.3333333333333333e-02*Jinv10*v1)*D;
  A[0][3]=(3.3333333333333333e-02*v0*Jinv00+1.0000000000000001e-01*v0*Jinv01+3.3333333333333333e-02*Jinv10*v1+1.0000000000000001e-01*v1*Jinv11)*D;
  A[0][4]=(-3.3333333333333333e-02*v0*Jinv00-3.3333333333333333e-02*v0*Jinv01-3.3333333333333333e-02*Jinv10*v1-3.3333333333333333e-02*v1*Jinv11)*D;
  A[0][5]=(-3.3333333333333333e-02*v0*Jinv01-3.3333333333333333e-02*v1*Jinv11)*D;
  A[1][0]=(-1.0000000000000001e-01*v0*Jinv00-1.0000000000000001e-01*v0*Jinv01-1.0000000000000001e-01*Jinv10*v1-1.0000000000000001e-01*v1*Jinv11)*D;
  A[1][1]=(-2.6666666666666666e-01*v0*Jinv01-2.6666666666666666e-01*v1*Jinv11)*D;
  A[1][2]=(1.0000000000000001e-01*v0*Jinv00+1.0000000000000001e-01*Jinv10*v1)*D;
  A[1][3]=(-1.3333333333333333e-01*v0*Jinv00+1.3333333333333333e-01*v0*Jinv01-1.3333333333333333e-01*Jinv10*v1+1.3333333333333333e-01*v1*Jinv11)*D;
  A[1][4]=(1.3333333333333333e-01*v0*Jinv00+2.6666666666666666e-01*v0*Jinv01+1.3333333333333333e-01*Jinv10*v1+2.6666666666666666e-01*v1*Jinv11)*D;
  A[1][5]=(-3.3333333333333333e-02*v0*Jinv01-3.3333333333333333e-02*v1*Jinv11)*D;
  A[2][0]=(3.3333333333333333e-02*v0*Jinv00+3.3333333333333333e-02*v0*Jinv01+3.3333333333333333e-02*Jinv10*v1+3.3333333333333333e-02*v1*Jinv11)*D;
  A[2][1]=(-1.0000000000000001e-01*v0*Jinv00-6.6666666666666666e-02*v0*Jinv01-1.0000000000000001e-01*Jinv10*v1-6.6666666666666666e-02*v1*Jinv11)*D;
  A[2][2]=(6.6666666666666666e-02*v0*Jinv00+6.6666666666666666e-02*Jinv10*v1)*D;
  A[2][3]=(3.3333333333333333e-02*v0*Jinv00+3.3333333333333333e-02*Jinv10*v1)*D;
  A[2][4]=(-3.3333333333333333e-02*v0*Jinv00+6.6666666666666666e-02*v0*Jinv01-3.3333333333333333e-02*Jinv10*v1+6.6666666666666666e-02*v1*Jinv11)*D;
  A[2][5]=(-3.3333333333333333e-02*v0*Jinv01-3.3333333333333333e-02*v1*Jinv11)*D;
  A[3][0]=(-1.0000000000000001e-01*v0*Jinv00-1.0000000000000001e-01*v0*Jinv01-1.0000000000000001e-01*Jinv10*v1-1.0000000000000001e-01*v1*Jinv11)*D;
  A[3][1]=(1.3333333333333333e-01*v0*Jinv00-1.3333333333333333e-01*v0*Jinv01+1.3333333333333333e-01*Jinv10*v1-1.3333333333333333e-01*v1*Jinv11)*D;
  A[3][2]=(-3.3333333333333333e-02*v0*Jinv00-3.3333333333333333e-02*Jinv10*v1)*D;
  A[3][3]=(-2.6666666666666666e-01*v0*Jinv00-2.6666666666666666e-01*Jinv10*v1)*D;
  A[3][4]=(2.6666666666666666e-01*v0*Jinv00+1.3333333333333333e-01*v0*Jinv01+2.6666666666666666e-01*Jinv10*v1+1.3333333333333333e-01*v1*Jinv11)*D;
  A[3][5]=(1.0000000000000001e-01*v0*Jinv01+1.0000000000000001e-01*v1*Jinv11)*D;
  A[4][0]=(3.3333333333333333e-02*v0*Jinv00+3.3333333333333333e-02*v0*Jinv01+3.3333333333333333e-02*Jinv10*v1+3.3333333333333333e-02*v1*Jinv11)*D;
  A[4][1]=(-1.3333333333333333e-01*v0*Jinv00-2.6666666666666666e-01*v0*Jinv01-1.3333333333333333e-01*Jinv10*v1-2.6666666666666666e-01*v1*Jinv11)*D;
  A[4][2]=(1.0000000000000001e-01*v0*Jinv00+1.0000000000000001e-01*Jinv10*v1)*D;
  A[4][3]=(-2.6666666666666666e-01*v0*Jinv00-1.3333333333333333e-01*v0*Jinv01-2.6666666666666666e-01*Jinv10*v1-1.3333333333333333e-01*v1*Jinv11)*D;
  A[4][4]=(2.6666666666666666e-01*v0*Jinv00+2.6666666666666666e-01*v0*Jinv01+2.6666666666666666e-01*Jinv10*v1+2.6666666666666666e-01*v1*Jinv11)*D;
  A[4][5]=(1.0000000000000001e-01*v0*Jinv01+1.0000000000000001e-01*v1*Jinv11)*D;
  A[5][0]=(3.3333333333333333e-02*v0*Jinv00+3.3333333333333333e-02*v0*Jinv01+3.3333333333333333e-02*Jinv10*v1+3.3333333333333333e-02*v1*Jinv11)*D;
  A[5][1]=(3.3333333333333333e-02*v0*Jinv01+3.3333333333333333e-02*v1*Jinv11)*D;
  A[5][2]=(-3.3333333333333333e-02*v0*Jinv00-3.3333333333333333e-02*Jinv10*v1)*D;
  A[5][3]=(-6.6666666666666666e-02*v0*Jinv00-1.0000000000000001e-01*v0*Jinv01-6.6666666666666666e-02*Jinv10*v1-1.0000000000000001e-01*v1*Jinv11)*D;
  A[5][4]=(6.6666666666666666e-02*v0*Jinv00-3.3333333333333333e-02*v0*Jinv01+6.6666666666666666e-02*Jinv10*v1-3.3333333333333333e-02*v1*Jinv11)*D;
  A[5][5]=(6.6666666666666666e-02*v0*Jinv01+6.6666666666666666e-02*v1*Jinv11)*D;
  int nbf = 6;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 ---------------*/


 
void matrix_factory_conv_2D_order3 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2],
         double v0, double v1){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    iidof[10];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/3.0+x0;  dof1[1]=G10/3.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=(2.0/3.0)*G00+x0;  dof2[1]=(2.0/3.0)*G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=G00+x0;  dof3[1]=G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G01/3.0+x0;  dof4[1]=y0+G11/3.0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01/3.0+G00/3.0+x0;  dof5[1]=G10/3.0+y0+G11/3.0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/3.0+(2.0/3.0)*G00+x0;  dof6[1]=(2.0/3.0)*G10+y0+G11/3.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=(2.0/3.0)*G01+x0;  dof7[1]=y0+(2.0/3.0)*G11;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=(2.0/3.0)*G01+G00/3.0+x0;  dof8[1]=G10/3.0+y0+(2.0/3.0)*G11;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01+x0;  dof9[1]=y0+G11;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[10][10];
  A[0][0]=(-3.8095238095238099e-02*v1*Jinv11-3.8095238095238099e-02*v0*Jinv00-3.8095238095238099e-02*Jinv10*v1-3.8095238095238099e-02*v0*Jinv01)*D;
  A[0][1]=(2.6785714285714286e-03*v1*Jinv11+6.1607142857142860e-02*v0*Jinv00+6.1607142857142860e-02*Jinv10*v1+2.6785714285714286e-03*v0*Jinv01)*D;
  A[0][2]=(-1.3392857142857142e-02*v1*Jinv11-3.4821428571428573e-02*v0*Jinv00-3.4821428571428573e-02*Jinv10*v1-1.3392857142857142e-02*v0*Jinv01)*D;
  A[0][3]=(1.1309523809523809e-02*v0*Jinv00+1.1309523809523809e-02*Jinv10*v1)*D;
  A[0][4]=(6.1607142857142860e-02*v1*Jinv11+2.6785714285714286e-03*v0*Jinv00+2.6785714285714286e-03*Jinv10*v1+6.1607142857142860e-02*v0*Jinv01)*D;
  A[0][5]=(-1.6071428571428570e-02*v1*Jinv11-1.6071428571428570e-02*v0*Jinv00-1.6071428571428570e-02*Jinv10*v1-1.6071428571428570e-02*v0*Jinv01)*D;
  A[0][6]=(1.3392857142857142e-02*v1*Jinv11+1.3392857142857142e-02*v0*Jinv00+1.3392857142857142e-02*Jinv10*v1+1.3392857142857142e-02*v0*Jinv01)*D;
  A[0][7]=(-3.4821428571428573e-02*v1*Jinv11-1.3392857142857142e-02*v0*Jinv00-1.3392857142857142e-02*Jinv10*v1-3.4821428571428573e-02*v0*Jinv01)*D;
  A[0][8]=(1.3392857142857142e-02*v1*Jinv11+1.3392857142857142e-02*v0*Jinv00+1.3392857142857142e-02*Jinv10*v1+1.3392857142857142e-02*v0*Jinv01)*D;
  A[0][9]=(1.1309523809523809e-02*v1*Jinv11+1.1309523809523809e-02*v0*Jinv01)*D;
  A[1][0]=(-6.1607142857142860e-02*v1*Jinv11-6.1607142857142860e-02*v0*Jinv00-6.1607142857142860e-02*Jinv10*v1-6.1607142857142860e-02*v0*Jinv01)*D;
  A[1][1]=(-1.9285714285714287e-01*v1*Jinv11-1.9285714285714287e-01*v0*Jinv01)*D;
  A[1][2]=(7.2321428571428578e-02*v1*Jinv11+9.6428571428571433e-02*v0*Jinv00+9.6428571428571433e-02*Jinv10*v1+7.2321428571428578e-02*v0*Jinv01)*D;
  A[1][3]=(-3.4821428571428573e-02*v0*Jinv00-3.4821428571428573e-02*Jinv10*v1)*D;
  A[1][4]=(9.6428571428571433e-02*v1*Jinv11-9.6428571428571433e-02*v0*Jinv00-9.6428571428571433e-02*Jinv10*v1+9.6428571428571433e-02*v0*Jinv01)*D;
  A[1][5]=(2.4107142857142858e-01*v1*Jinv11+1.4464285714285716e-01*v0*Jinv00+1.4464285714285716e-01*Jinv10*v1+2.4107142857142858e-01*v0*Jinv01)*D;
  A[1][6]=(-7.2321428571428578e-02*v1*Jinv11-4.8214285714285716e-02*v0*Jinv00-4.8214285714285716e-02*Jinv10*v1-7.2321428571428578e-02*v0*Jinv01)*D;
  A[1][7]=(-4.8214285714285716e-02*v1*Jinv11+2.4107142857142858e-02*v0*Jinv00+2.4107142857142858e-02*Jinv10*v1-4.8214285714285716e-02*v0*Jinv01)*D;
  A[1][8]=(-4.8214285714285716e-02*v1*Jinv11-2.4107142857142858e-02*v0*Jinv00-2.4107142857142858e-02*Jinv10*v1-4.8214285714285716e-02*v0*Jinv01)*D;
  A[1][9]=(1.3392857142857142e-02*v1*Jinv11+1.3392857142857142e-02*v0*Jinv01)*D;
  A[2][0]=(3.4821428571428573e-02*v1*Jinv11+3.4821428571428573e-02*v0*Jinv00+3.4821428571428573e-02*Jinv10*v1+3.4821428571428573e-02*v0*Jinv01)*D;
  A[2][1]=(-2.4107142857142858e-02*v1*Jinv11-9.6428571428571433e-02*v0*Jinv00-9.6428571428571433e-02*Jinv10*v1-2.4107142857142858e-02*v0*Jinv01)*D;
  A[2][2]=(-1.9285714285714287e-01*v1*Jinv11-1.9285714285714287e-01*v0*Jinv01)*D;
  A[2][3]=(6.1607142857142860e-02*v0*Jinv00+6.1607142857142860e-02*Jinv10*v1)*D;
  A[2][4]=(-2.4107142857142858e-02*v1*Jinv11+4.8214285714285716e-02*v0*Jinv00+4.8214285714285716e-02*Jinv10*v1-2.4107142857142858e-02*v0*Jinv01)*D;
  A[2][5]=(9.6428571428571433e-02*v1*Jinv11-1.4464285714285716e-01*v0*Jinv00-1.4464285714285716e-01*Jinv10*v1+9.6428571428571433e-02*v0*Jinv01)*D;
  A[2][6]=(1.9285714285714287e-01*v1*Jinv11+9.6428571428571433e-02*v0*Jinv00+9.6428571428571433e-02*Jinv10*v1+1.9285714285714287e-01*v0*Jinv01)*D;
  A[2][7]=(-2.4107142857142858e-02*v1*Jinv11+2.4107142857142858e-02*v0*Jinv00+2.4107142857142858e-02*Jinv10*v1-2.4107142857142858e-02*v0*Jinv01)*D;
  A[2][8]=(-7.2321428571428578e-02*v1*Jinv11-2.4107142857142858e-02*v0*Jinv00-2.4107142857142858e-02*Jinv10*v1-7.2321428571428578e-02*v0*Jinv01)*D;
  A[2][9]=(1.3392857142857142e-02*v1*Jinv11+1.3392857142857142e-02*v0*Jinv01)*D;
  A[3][0]=(-1.1309523809523809e-02*v1*Jinv11-1.1309523809523809e-02*v0*Jinv00-1.1309523809523809e-02*Jinv10*v1-1.1309523809523809e-02*v0*Jinv01)*D;
  A[3][1]=(2.1428571428571429e-02*v1*Jinv11+3.4821428571428573e-02*v0*Jinv00+3.4821428571428573e-02*Jinv10*v1+2.1428571428571429e-02*v0*Jinv01)*D;
  A[3][2]=(-5.8928571428571427e-02*v1*Jinv11-6.1607142857142860e-02*v0*Jinv00-6.1607142857142860e-02*Jinv10*v1-5.8928571428571427e-02*v0*Jinv01)*D;
  A[3][3]=(3.8095238095238099e-02*v0*Jinv00+3.8095238095238099e-02*Jinv10*v1)*D;
  A[3][4]=(-1.3392857142857142e-02*v0*Jinv00-1.3392857142857142e-02*Jinv10*v1)*D;
  A[3][5]=(1.6071428571428570e-02*v0*Jinv00+1.6071428571428570e-02*Jinv10*v1)*D;
  A[3][6]=(5.8928571428571427e-02*v1*Jinv11-2.6785714285714286e-03*v0*Jinv00-2.6785714285714286e-03*Jinv10*v1+5.8928571428571427e-02*v0*Jinv01)*D;
  A[3][7]=(-1.3392857142857142e-02*v0*Jinv00-1.3392857142857142e-02*Jinv10*v1)*D;
  A[3][8]=(-2.1428571428571429e-02*v1*Jinv11+1.3392857142857142e-02*v0*Jinv00+1.3392857142857142e-02*Jinv10*v1-2.1428571428571429e-02*v0*Jinv01)*D;
  A[3][9]=(1.1309523809523809e-02*v1*Jinv11+1.1309523809523809e-02*v0*Jinv01)*D;
  A[4][0]=(-6.1607142857142860e-02*v1*Jinv11-6.1607142857142860e-02*v0*Jinv00-6.1607142857142860e-02*Jinv10*v1-6.1607142857142860e-02*v0*Jinv01)*D;
  A[4][1]=(-9.6428571428571433e-02*v1*Jinv11+9.6428571428571433e-02*v0*Jinv00+9.6428571428571433e-02*Jinv10*v1-9.6428571428571433e-02*v0*Jinv01)*D;
  A[4][2]=(2.4107142857142858e-02*v1*Jinv11-4.8214285714285716e-02*v0*Jinv00-4.8214285714285716e-02*Jinv10*v1+2.4107142857142858e-02*v0*Jinv01)*D;
  A[4][3]=(1.3392857142857142e-02*v0*Jinv00+1.3392857142857142e-02*Jinv10*v1)*D;
  A[4][4]=(-1.9285714285714287e-01*v0*Jinv00-1.9285714285714287e-01*Jinv10*v1)*D;
  A[4][5]=(1.4464285714285716e-01*v1*Jinv11+2.4107142857142858e-01*v0*Jinv00+2.4107142857142858e-01*Jinv10*v1+1.4464285714285716e-01*v0*Jinv01)*D;
  A[4][6]=(-2.4107142857142858e-02*v1*Jinv11-4.8214285714285716e-02*v0*Jinv00-4.8214285714285716e-02*Jinv10*v1-2.4107142857142858e-02*v0*Jinv01)*D;
  A[4][7]=(9.6428571428571433e-02*v1*Jinv11+7.2321428571428578e-02*v0*Jinv00+7.2321428571428578e-02*Jinv10*v1+9.6428571428571433e-02*v0*Jinv01)*D;
  A[4][8]=(-4.8214285714285716e-02*v1*Jinv11-7.2321428571428578e-02*v0*Jinv00-7.2321428571428578e-02*Jinv10*v1-4.8214285714285716e-02*v0*Jinv01)*D;
  A[4][9]=(-3.4821428571428573e-02*v1*Jinv11-3.4821428571428573e-02*v0*Jinv01)*D;
  A[5][0]=(1.6071428571428570e-02*v1*Jinv11+1.6071428571428570e-02*v0*Jinv00+1.6071428571428570e-02*Jinv10*v1+1.6071428571428570e-02*v0*Jinv01)*D;
  A[5][1]=(-2.4107142857142858e-01*v1*Jinv11-1.4464285714285716e-01*v0*Jinv00-1.4464285714285716e-01*Jinv10*v1-2.4107142857142858e-01*v0*Jinv01)*D;
  A[5][2]=(-9.6428571428571433e-02*v1*Jinv11+1.4464285714285716e-01*v0*Jinv00+1.4464285714285716e-01*Jinv10*v1-9.6428571428571433e-02*v0*Jinv01)*D;
  A[5][3]=(-1.6071428571428570e-02*v0*Jinv00-1.6071428571428570e-02*Jinv10*v1)*D;
  A[5][4]=(-1.4464285714285716e-01*v1*Jinv11-2.4107142857142858e-01*v0*Jinv00-2.4107142857142858e-01*Jinv10*v1-1.4464285714285716e-01*v0*Jinv01)*D;
  A[5][5]=(0.0)*D;
  A[5][6]=(9.6428571428571433e-02*v1*Jinv11+2.4107142857142858e-01*v0*Jinv00+2.4107142857142858e-01*Jinv10*v1+9.6428571428571433e-02*v0*Jinv01)*D;
  A[5][7]=(1.4464285714285716e-01*v1*Jinv11-9.6428571428571433e-02*v0*Jinv00-9.6428571428571433e-02*Jinv10*v1+1.4464285714285716e-01*v0*Jinv01)*D;
  A[5][8]=(2.4107142857142858e-01*v1*Jinv11+9.6428571428571433e-02*v0*Jinv00+9.6428571428571433e-02*Jinv10*v1+2.4107142857142858e-01*v0*Jinv01)*D;
  A[5][9]=(-1.6071428571428570e-02*v1*Jinv11-1.6071428571428570e-02*v0*Jinv01)*D;
  A[6][0]=(-1.3392857142857142e-02*v1*Jinv11-1.3392857142857142e-02*v0*Jinv00-1.3392857142857142e-02*Jinv10*v1-1.3392857142857142e-02*v0*Jinv01)*D;
  A[6][1]=(7.2321428571428578e-02*v1*Jinv11+4.8214285714285716e-02*v0*Jinv00+4.8214285714285716e-02*Jinv10*v1+7.2321428571428578e-02*v0*Jinv01)*D;
  A[6][2]=(-1.9285714285714287e-01*v1*Jinv11-9.6428571428571433e-02*v0*Jinv00-9.6428571428571433e-02*Jinv10*v1-1.9285714285714287e-01*v0*Jinv01)*D;
  A[6][3]=(6.1607142857142860e-02*v0*Jinv00+6.1607142857142860e-02*Jinv10*v1)*D;
  A[6][4]=(2.4107142857142858e-02*v1*Jinv11+4.8214285714285716e-02*v0*Jinv00+4.8214285714285716e-02*Jinv10*v1+2.4107142857142858e-02*v0*Jinv01)*D;
  A[6][5]=(-9.6428571428571433e-02*v1*Jinv11-2.4107142857142858e-01*v0*Jinv00-2.4107142857142858e-01*Jinv10*v1-9.6428571428571433e-02*v0*Jinv01)*D;
  A[6][6]=(1.9285714285714287e-01*v1*Jinv11+1.9285714285714287e-01*v0*Jinv00+1.9285714285714287e-01*Jinv10*v1+1.9285714285714287e-01*v0*Jinv01)*D;
  A[6][7]=(2.4107142857142858e-02*v1*Jinv11+7.2321428571428578e-02*v0*Jinv00+7.2321428571428578e-02*Jinv10*v1+2.4107142857142858e-02*v0*Jinv01)*D;
  A[6][8]=(2.4107142857142858e-02*v1*Jinv11-7.2321428571428578e-02*v0*Jinv00-7.2321428571428578e-02*Jinv10*v1+2.4107142857142858e-02*v0*Jinv01)*D;
  A[6][9]=(-3.4821428571428573e-02*v1*Jinv11-3.4821428571428573e-02*v0*Jinv01)*D;
  A[7][0]=(3.4821428571428573e-02*v1*Jinv11+3.4821428571428573e-02*v0*Jinv00+3.4821428571428573e-02*Jinv10*v1+3.4821428571428573e-02*v0*Jinv01)*D;
  A[7][1]=(4.8214285714285716e-02*v1*Jinv11-2.4107142857142858e-02*v0*Jinv00-2.4107142857142858e-02*Jinv10*v1+4.8214285714285716e-02*v0*Jinv01)*D;
  A[7][2]=(2.4107142857142858e-02*v1*Jinv11-2.4107142857142858e-02*v0*Jinv00-2.4107142857142858e-02*Jinv10*v1+2.4107142857142858e-02*v0*Jinv01)*D;
  A[7][3]=(1.3392857142857142e-02*v0*Jinv00+1.3392857142857142e-02*Jinv10*v1)*D;
  A[7][4]=(-9.6428571428571433e-02*v1*Jinv11-2.4107142857142858e-02*v0*Jinv00-2.4107142857142858e-02*Jinv10*v1-9.6428571428571433e-02*v0*Jinv01)*D;
  A[7][5]=(-1.4464285714285716e-01*v1*Jinv11+9.6428571428571433e-02*v0*Jinv00+9.6428571428571433e-02*Jinv10*v1-1.4464285714285716e-01*v0*Jinv01)*D;
  A[7][6]=(-2.4107142857142858e-02*v1*Jinv11-7.2321428571428578e-02*v0*Jinv00-7.2321428571428578e-02*Jinv10*v1-2.4107142857142858e-02*v0*Jinv01)*D;
  A[7][7]=(-1.9285714285714287e-01*v0*Jinv00-1.9285714285714287e-01*Jinv10*v1)*D;
  A[7][8]=(9.6428571428571433e-02*v1*Jinv11+1.9285714285714287e-01*v0*Jinv00+1.9285714285714287e-01*Jinv10*v1+9.6428571428571433e-02*v0*Jinv01)*D;
  A[7][9]=(6.1607142857142860e-02*v1*Jinv11+6.1607142857142860e-02*v0*Jinv01)*D;
  A[8][0]=(-1.3392857142857142e-02*v1*Jinv11-1.3392857142857142e-02*v0*Jinv00-1.3392857142857142e-02*Jinv10*v1-1.3392857142857142e-02*v0*Jinv01)*D;
  A[8][1]=(4.8214285714285716e-02*v1*Jinv11+2.4107142857142858e-02*v0*Jinv00+2.4107142857142858e-02*Jinv10*v1+4.8214285714285716e-02*v0*Jinv01)*D;
  A[8][2]=(7.2321428571428578e-02*v1*Jinv11+2.4107142857142858e-02*v0*Jinv00+2.4107142857142858e-02*Jinv10*v1+7.2321428571428578e-02*v0*Jinv01)*D;
  A[8][3]=(-3.4821428571428573e-02*v0*Jinv00-3.4821428571428573e-02*Jinv10*v1)*D;
  A[8][4]=(4.8214285714285716e-02*v1*Jinv11+7.2321428571428578e-02*v0*Jinv00+7.2321428571428578e-02*Jinv10*v1+4.8214285714285716e-02*v0*Jinv01)*D;
  A[8][5]=(-2.4107142857142858e-01*v1*Jinv11-9.6428571428571433e-02*v0*Jinv00-9.6428571428571433e-02*Jinv10*v1-2.4107142857142858e-01*v0*Jinv01)*D;
  A[8][6]=(-7.2321428571428578e-02*v1*Jinv11+2.4107142857142858e-02*v0*Jinv00+2.4107142857142858e-02*Jinv10*v1-7.2321428571428578e-02*v0*Jinv01)*D;
  A[8][7]=(-9.6428571428571433e-02*v1*Jinv11-1.9285714285714287e-01*v0*Jinv00-1.9285714285714287e-01*Jinv10*v1-9.6428571428571433e-02*v0*Jinv01)*D;
  A[8][8]=(1.9285714285714287e-01*v1*Jinv11+1.9285714285714287e-01*v0*Jinv00+1.9285714285714287e-01*Jinv10*v1+1.9285714285714287e-01*v0*Jinv01)*D;
  A[8][9]=(6.1607142857142860e-02*v1*Jinv11+6.1607142857142860e-02*v0*Jinv01)*D;
  A[9][0]=(-1.1309523809523809e-02*v1*Jinv11-1.1309523809523809e-02*v0*Jinv00-1.1309523809523809e-02*Jinv10*v1-1.1309523809523809e-02*v0*Jinv01)*D;
  A[9][1]=(-1.3392857142857142e-02*v1*Jinv11-1.3392857142857142e-02*v0*Jinv01)*D;
  A[9][2]=(-1.3392857142857142e-02*v1*Jinv11-1.3392857142857142e-02*v0*Jinv01)*D;
  A[9][3]=(1.1309523809523809e-02*v0*Jinv00+1.1309523809523809e-02*Jinv10*v1)*D;
  A[9][4]=(3.4821428571428573e-02*v1*Jinv11+2.1428571428571429e-02*v0*Jinv00+2.1428571428571429e-02*Jinv10*v1+3.4821428571428573e-02*v0*Jinv01)*D;
  A[9][5]=(1.6071428571428570e-02*v1*Jinv11+1.6071428571428570e-02*v0*Jinv01)*D;
  A[9][6]=(1.3392857142857142e-02*v1*Jinv11-2.1428571428571429e-02*v0*Jinv00-2.1428571428571429e-02*Jinv10*v1+1.3392857142857142e-02*v0*Jinv01)*D;
  A[9][7]=(-6.1607142857142860e-02*v1*Jinv11-5.8928571428571427e-02*v0*Jinv00-5.8928571428571427e-02*Jinv10*v1-6.1607142857142860e-02*v0*Jinv01)*D;
  A[9][8]=(-2.6785714285714286e-03*v1*Jinv11+5.8928571428571427e-02*v0*Jinv00+5.8928571428571427e-02*Jinv10*v1-2.6785714285714286e-03*v0*Jinv01)*D;
  A[9][9]=(3.8095238095238099e-02*v1*Jinv11+3.8095238095238099e-02*v0*Jinv01)*D;
  int nbf = 10;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 4 ---------------*/


 
void matrix_factory_conv_2D_order4 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2],
         double v0, double v1){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    iidof[15];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/4.0+x0;  dof1[1]=G10/4.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=G00/2.0+x0;  dof2[1]=G10/2.0+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=(3.0/4.0)*G00+x0;  dof3[1]=(3.0/4.0)*G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=G00+x0;  dof4[1]=G10+y0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G01/4.0+x0;  dof5[1]=y0+G11/4.0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/4.0+G00/4.0+x0;  dof6[1]=G10/4.0+y0+G11/4.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=G01/4.0+G00/2.0+x0;  dof7[1]=G10/2.0+y0+G11/4.0;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=G01/4.0+(3.0/4.0)*G00+x0;  dof8[1]=(3.0/4.0)*G10+y0+G11/4.0;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01/2.0+x0;  dof9[1]=y0+G11/2.0;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[2];
  dof10[0]=G01/2.0+G00/4.0+x0;  dof10[1]=G10/4.0+y0+G11/2.0;
  ptv pdof10(2,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[2];
  dof11[0]=G01/2.0+G00/2.0+x0;  dof11[1]=G10/2.0+y0+G11/2.0;
  ptv pdof11(2,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[2];
  dof12[0]=(3.0/4.0)*G01+x0;  dof12[1]=y0+(3.0/4.0)*G11;
  ptv pdof12(2,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[2];
  dof13[0]=(3.0/4.0)*G01+G00/4.0+x0;  dof13[1]=G10/4.0+y0+(3.0/4.0)*G11;
  ptv pdof13(2,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[2];
  dof14[0]=G01+x0;  dof14[1]=y0+G11;
  ptv pdof14(2,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[15][15];
  A[0][0]=(-2.5749559082892417e-02*Jinv10*v1-2.5749559082892417e-02*Jinv01*v0-2.5749559082892417e-02*Jinv00*v0-2.5749559082892417e-02*Jinv11*v1)*D;
  A[0][1]=(4.5855379188712520e-02*Jinv10*v1-6.3492063492063492e-03*Jinv01*v0+4.5855379188712520e-02*Jinv00*v0-6.3492063492063492e-03*Jinv11*v1)*D;
  A[0][2]=(-3.2627865961199293e-02*Jinv10*v1-1.9400352733686067e-03*Jinv01*v0-3.2627865961199293e-02*Jinv00*v0-1.9400352733686067e-03*Jinv11*v1)*D;
  A[0][3]=(1.7636684303350969e-02*Jinv10*v1+7.7601410934744269e-03*Jinv01*v0+1.7636684303350969e-02*Jinv00*v0+7.7601410934744269e-03*Jinv11*v1)*D;
  A[0][4]=(-5.1146384479717815e-03*Jinv10*v1-5.1146384479717815e-03*Jinv00*v0)*D;
  A[0][5]=(-6.3492063492063492e-03*Jinv10*v1+4.5855379188712520e-02*Jinv01*v0-6.3492063492063492e-03*Jinv00*v0+4.5855379188712520e-02*Jinv11*v1)*D;
  A[0][6]=(7.0546737213403876e-03*Jinv10*v1+7.0546737213403876e-03*Jinv01*v0+7.0546737213403876e-03*Jinv00*v0+7.0546737213403876e-03*Jinv11*v1)*D;
  A[0][7]=(7.0546737213403876e-03*Jinv10*v1+7.0546737213403876e-03*Jinv01*v0+7.0546737213403876e-03*Jinv00*v0+7.0546737213403876e-03*Jinv11*v1)*D;
  A[0][8]=(-7.7601410934744269e-03*Jinv10*v1-7.7601410934744269e-03*Jinv01*v0-7.7601410934744269e-03*Jinv00*v0-7.7601410934744269e-03*Jinv11*v1)*D;
  A[0][9]=(-1.9400352733686067e-03*Jinv10*v1-3.2627865961199293e-02*Jinv01*v0-1.9400352733686067e-03*Jinv00*v0-3.2627865961199293e-02*Jinv11*v1)*D;
  A[0][10]=(7.0546737213403876e-03*Jinv10*v1+7.0546737213403876e-03*Jinv01*v0+7.0546737213403876e-03*Jinv00*v0+7.0546737213403876e-03*Jinv11*v1)*D;
  A[0][11]=(-5.1146384479717815e-03*Jinv10*v1-5.1146384479717815e-03*Jinv01*v0-5.1146384479717815e-03*Jinv00*v0-5.1146384479717815e-03*Jinv11*v1)*D;
  A[0][12]=(7.7601410934744269e-03*Jinv10*v1+1.7636684303350969e-02*Jinv01*v0+7.7601410934744269e-03*Jinv00*v0+1.7636684303350969e-02*Jinv11*v1)*D;
  A[0][13]=(-7.7601410934744269e-03*Jinv10*v1-7.7601410934744269e-03*Jinv01*v0-7.7601410934744269e-03*Jinv00*v0-7.7601410934744269e-03*Jinv11*v1)*D;
  A[0][14]=(-5.1146384479717815e-03*Jinv01*v0-5.1146384479717815e-03*Jinv11*v1)*D;
  A[1][0]=(-4.5855379188712520e-02*Jinv10*v1-4.5855379188712520e-02*Jinv01*v0-4.5855379188712520e-02*Jinv00*v0-4.5855379188712520e-02*Jinv11*v1)*D;
  A[1][1]=(-1.5802469135802469e-01*Jinv01*v0-1.5802469135802469e-01*Jinv11*v1)*D;
  A[1][2]=(7.3368606701940037e-02*Jinv10*v1+5.0793650793650794e-02*Jinv01*v0+7.3368606701940037e-02*Jinv00*v0+5.0793650793650794e-02*Jinv11*v1)*D;
  A[1][3]=(-4.5149911816578486e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[1][4]=(1.7636684303350969e-02*Jinv10*v1+1.7636684303350969e-02*Jinv00*v0)*D;
  A[1][5]=(-7.9012345679012344e-02*Jinv10*v1+7.9012345679012344e-02*Jinv01*v0-7.9012345679012344e-02*Jinv00*v0+7.9012345679012344e-02*Jinv11*v1)*D;
  A[1][6]=(9.0299823633156973e-02*Jinv10*v1+2.0317460317460317e-01*Jinv01*v0+9.0299823633156973e-02*Jinv00*v0+2.0317460317460317e-01*Jinv11*v1)*D;
  A[1][7]=(-4.5149911816578486e-02*Jinv10*v1-6.7724867724867729e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0-6.7724867724867729e-02*Jinv11*v1)*D;
  A[1][8]=(3.3862433862433865e-02*Jinv10*v1+4.5149911816578486e-02*Jinv01*v0+3.3862433862433865e-02*Jinv00*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[1][9]=(3.1040564373897708e-02*Jinv10*v1-5.9259259259259262e-02*Jinv01*v0+3.1040564373897708e-02*Jinv00*v0-5.9259259259259262e-02*Jinv11*v1)*D;
  A[1][10]=(-4.5149911816578486e-02*Jinv10*v1-6.7724867724867729e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0-6.7724867724867729e-02*Jinv11*v1)*D;
  A[1][11]=(1.4109347442680775e-02*Jinv10*v1+1.6931216931216932e-02*Jinv01*v0+1.4109347442680775e-02*Jinv00*v0+1.6931216931216932e-02*Jinv11*v1)*D;
  A[1][12]=(-1.1287477954144622e-02*Jinv10*v1+3.3862433862433865e-02*Jinv01*v0-1.1287477954144622e-02*Jinv00*v0+3.3862433862433865e-02*Jinv11*v1)*D;
  A[1][13]=(1.1287477954144622e-02*Jinv10*v1+2.2574955908289243e-02*Jinv01*v0+1.1287477954144622e-02*Jinv00*v0+2.2574955908289243e-02*Jinv11*v1)*D;
  A[1][14]=(-7.7601410934744269e-03*Jinv01*v0-7.7601410934744269e-03*Jinv11*v1)*D;
  A[2][0]=(3.2627865961199293e-02*Jinv10*v1+3.2627865961199293e-02*Jinv01*v0+3.2627865961199293e-02*Jinv00*v0+3.2627865961199293e-02*Jinv11*v1)*D;
  A[2][1]=(-7.3368606701940037e-02*Jinv10*v1+1.6931216931216932e-02*Jinv01*v0-7.3368606701940037e-02*Jinv00*v0+1.6931216931216932e-02*Jinv11*v1)*D;
  A[2][2]=(-1.6507936507936508e-01*Jinv01*v0-1.6507936507936508e-01*Jinv11*v1)*D;
  A[2][3]=(7.3368606701940037e-02*Jinv10*v1+9.0299823633156973e-02*Jinv01*v0+7.3368606701940037e-02*Jinv00*v0+9.0299823633156973e-02*Jinv11*v1)*D;
  A[2][4]=(-3.2627865961199293e-02*Jinv10*v1-3.2627865961199293e-02*Jinv00*v0)*D;
  A[2][5]=(5.9259259259259262e-02*Jinv10*v1-3.1040564373897708e-02*Jinv01*v0+5.9259259259259262e-02*Jinv00*v0-3.1040564373897708e-02*Jinv11*v1)*D;
  A[2][6]=(-1.3544973544973546e-01*Jinv10*v1+5.0793650793650794e-02*Jinv01*v0-1.3544973544973546e-01*Jinv00*v0+5.0793650793650794e-02*Jinv11*v1)*D;
  A[2][7]=(1.3544973544973546e-01*Jinv10*v1+1.8624338624338624e-01*Jinv01*v0+1.3544973544973546e-01*Jinv00*v0+1.8624338624338624e-01*Jinv11*v1)*D;
  A[2][8]=(-5.9259259259259262e-02*Jinv10*v1-9.0299823633156973e-02*Jinv01*v0-5.9259259259259262e-02*Jinv00*v0-9.0299823633156973e-02*Jinv11*v1)*D;
  A[2][9]=(1.0582010582010581e-02*Jinv10*v1-1.0582010582010581e-02*Jinv01*v0+1.0582010582010581e-02*Jinv00*v0-1.0582010582010581e-02*Jinv11*v1)*D;
  A[2][10]=(-8.4656084656084651e-02*Jinv01*v0-8.4656084656084651e-02*Jinv11*v1)*D;
  A[2][11]=(-1.0582010582010581e-02*Jinv10*v1-2.1164021164021163e-02*Jinv01*v0-1.0582010582010581e-02*Jinv00*v0-2.1164021164021163e-02*Jinv11*v1)*D;
  A[2][12]=(-2.8218694885361554e-03*Jinv10*v1+1.4109347442680775e-02*Jinv01*v0-2.8218694885361554e-03*Jinv00*v0+1.4109347442680775e-02*Jinv11*v1)*D;
  A[2][13]=(2.8218694885361554e-03*Jinv10*v1+1.6931216931216932e-02*Jinv01*v0+2.8218694885361554e-03*Jinv00*v0+1.6931216931216932e-02*Jinv11*v1)*D;
  A[2][14]=(-5.1146384479717815e-03*Jinv01*v0-5.1146384479717815e-03*Jinv11*v1)*D;
  A[3][0]=(-1.7636684303350969e-02*Jinv10*v1-1.7636684303350969e-02*Jinv01*v0-1.7636684303350969e-02*Jinv00*v0-1.7636684303350969e-02*Jinv11*v1)*D;
  A[3][1]=(4.5149911816578486e-02*Jinv10*v1+4.5149911816578486e-02*Jinv00*v0)*D;
  A[3][2]=(-7.3368606701940037e-02*Jinv10*v1-2.2574955908289243e-02*Jinv01*v0-7.3368606701940037e-02*Jinv00*v0-2.2574955908289243e-02*Jinv11*v1)*D;
  A[3][3]=(-1.5802469135802469e-01*Jinv01*v0-1.5802469135802469e-01*Jinv11*v1)*D;
  A[3][4]=(4.5855379188712520e-02*Jinv10*v1+4.5855379188712520e-02*Jinv00*v0)*D;
  A[3][5]=(-3.3862433862433865e-02*Jinv10*v1+1.1287477954144622e-02*Jinv01*v0-3.3862433862433865e-02*Jinv00*v0+1.1287477954144622e-02*Jinv11*v1)*D;
  A[3][6]=(4.5149911816578486e-02*Jinv10*v1-2.2574955908289243e-02*Jinv01*v0+4.5149911816578486e-02*Jinv00*v0-2.2574955908289243e-02*Jinv11*v1)*D;
  A[3][7]=(-9.0299823633156973e-02*Jinv10*v1+1.1287477954144620e-01*Jinv01*v0-9.0299823633156973e-02*Jinv00*v0+1.1287477954144620e-01*Jinv11*v1)*D;
  A[3][8]=(7.9012345679012344e-02*Jinv10*v1+1.5802469135802469e-01*Jinv01*v0+7.9012345679012344e-02*Jinv00*v0+1.5802469135802469e-01*Jinv11*v1)*D;
  A[3][9]=(-1.4109347442680775e-02*Jinv10*v1+2.8218694885361554e-03*Jinv01*v0-1.4109347442680775e-02*Jinv00*v0+2.8218694885361554e-03*Jinv11*v1)*D;
  A[3][10]=(4.5149911816578486e-02*Jinv10*v1-2.2574955908289243e-02*Jinv01*v0+4.5149911816578486e-02*Jinv00*v0-2.2574955908289243e-02*Jinv11*v1)*D;
  A[3][11]=(-3.1040564373897708e-02*Jinv10*v1-9.0299823633156973e-02*Jinv01*v0-3.1040564373897708e-02*Jinv00*v0-9.0299823633156973e-02*Jinv11*v1)*D;
  A[3][12]=(-1.1287477954144622e-02*Jinv10*v1+1.1287477954144622e-02*Jinv01*v0-1.1287477954144622e-02*Jinv00*v0+1.1287477954144622e-02*Jinv11*v1)*D;
  A[3][13]=(1.1287477954144622e-02*Jinv10*v1+4.5149911816578486e-02*Jinv01*v0+1.1287477954144622e-02*Jinv00*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[3][14]=(-7.7601410934744269e-03*Jinv01*v0-7.7601410934744269e-03*Jinv11*v1)*D;
  A[4][0]=(5.1146384479717815e-03*Jinv10*v1+5.1146384479717815e-03*Jinv01*v0+5.1146384479717815e-03*Jinv00*v0+5.1146384479717815e-03*Jinv11*v1)*D;
  A[4][1]=(-1.7636684303350969e-02*Jinv10*v1-9.8765432098765430e-03*Jinv01*v0-1.7636684303350969e-02*Jinv00*v0-9.8765432098765430e-03*Jinv11*v1)*D;
  A[4][2]=(3.2627865961199293e-02*Jinv10*v1+3.0687830687830688e-02*Jinv01*v0+3.2627865961199293e-02*Jinv00*v0+3.0687830687830688e-02*Jinv11*v1)*D;
  A[4][3]=(-4.5855379188712520e-02*Jinv10*v1-5.2204585537918874e-02*Jinv01*v0-4.5855379188712520e-02*Jinv00*v0-5.2204585537918874e-02*Jinv11*v1)*D;
  A[4][4]=(2.5749559082892417e-02*Jinv10*v1+2.5749559082892417e-02*Jinv00*v0)*D;
  A[4][5]=(7.7601410934744269e-03*Jinv10*v1+7.7601410934744269e-03*Jinv00*v0)*D;
  A[4][6]=(-7.0546737213403876e-03*Jinv10*v1-7.0546737213403876e-03*Jinv00*v0)*D;
  A[4][7]=(-7.0546737213403876e-03*Jinv10*v1-7.0546737213403876e-03*Jinv00*v0)*D;
  A[4][8]=(6.3492063492063492e-03*Jinv10*v1+5.2204585537918874e-02*Jinv01*v0+6.3492063492063492e-03*Jinv00*v0+5.2204585537918874e-02*Jinv11*v1)*D;
  A[4][9]=(5.1146384479717815e-03*Jinv10*v1+5.1146384479717815e-03*Jinv00*v0)*D;
  A[4][10]=(-7.0546737213403876e-03*Jinv10*v1-7.0546737213403876e-03*Jinv00*v0)*D;
  A[4][11]=(1.9400352733686067e-03*Jinv10*v1-3.0687830687830688e-02*Jinv01*v0+1.9400352733686067e-03*Jinv00*v0-3.0687830687830688e-02*Jinv11*v1)*D;
  A[4][12]=(7.7601410934744269e-03*Jinv10*v1+7.7601410934744269e-03*Jinv00*v0)*D;
  A[4][13]=(-7.7601410934744269e-03*Jinv10*v1+9.8765432098765430e-03*Jinv01*v0-7.7601410934744269e-03*Jinv00*v0+9.8765432098765430e-03*Jinv11*v1)*D;
  A[4][14]=(-5.1146384479717815e-03*Jinv01*v0-5.1146384479717815e-03*Jinv11*v1)*D;
  A[5][0]=(-4.5855379188712520e-02*Jinv10*v1-4.5855379188712520e-02*Jinv01*v0-4.5855379188712520e-02*Jinv00*v0-4.5855379188712520e-02*Jinv11*v1)*D;
  A[5][1]=(7.9012345679012344e-02*Jinv10*v1-7.9012345679012344e-02*Jinv01*v0+7.9012345679012344e-02*Jinv00*v0-7.9012345679012344e-02*Jinv11*v1)*D;
  A[5][2]=(-5.9259259259259262e-02*Jinv10*v1+3.1040564373897708e-02*Jinv01*v0-5.9259259259259262e-02*Jinv00*v0+3.1040564373897708e-02*Jinv11*v1)*D;
  A[5][3]=(3.3862433862433865e-02*Jinv10*v1-1.1287477954144622e-02*Jinv01*v0+3.3862433862433865e-02*Jinv00*v0-1.1287477954144622e-02*Jinv11*v1)*D;
  A[5][4]=(-7.7601410934744269e-03*Jinv10*v1-7.7601410934744269e-03*Jinv00*v0)*D;
  A[5][5]=(-1.5802469135802469e-01*Jinv10*v1-1.5802469135802469e-01*Jinv00*v0)*D;
  A[5][6]=(2.0317460317460317e-01*Jinv10*v1+9.0299823633156973e-02*Jinv01*v0+2.0317460317460317e-01*Jinv00*v0+9.0299823633156973e-02*Jinv11*v1)*D;
  A[5][7]=(-6.7724867724867729e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0-6.7724867724867729e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[5][8]=(2.2574955908289243e-02*Jinv10*v1+1.1287477954144622e-02*Jinv01*v0+2.2574955908289243e-02*Jinv00*v0+1.1287477954144622e-02*Jinv11*v1)*D;
  A[5][9]=(5.0793650793650794e-02*Jinv10*v1+7.3368606701940037e-02*Jinv01*v0+5.0793650793650794e-02*Jinv00*v0+7.3368606701940037e-02*Jinv11*v1)*D;
  A[5][10]=(-6.7724867724867729e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0-6.7724867724867729e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[5][11]=(1.6931216931216932e-02*Jinv10*v1+1.4109347442680775e-02*Jinv01*v0+1.6931216931216932e-02*Jinv00*v0+1.4109347442680775e-02*Jinv11*v1)*D;
  A[5][12]=(-4.5149911816578486e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[5][13]=(4.5149911816578486e-02*Jinv10*v1+3.3862433862433865e-02*Jinv01*v0+4.5149911816578486e-02*Jinv00*v0+3.3862433862433865e-02*Jinv11*v1)*D;
  A[5][14]=(1.7636684303350969e-02*Jinv01*v0+1.7636684303350969e-02*Jinv11*v1)*D;
  A[6][0]=(-7.0546737213403876e-03*Jinv10*v1-7.0546737213403876e-03*Jinv01*v0-7.0546737213403876e-03*Jinv00*v0-7.0546737213403876e-03*Jinv11*v1)*D;
  A[6][1]=(-9.0299823633156973e-02*Jinv10*v1-2.0317460317460317e-01*Jinv01*v0-9.0299823633156973e-02*Jinv00*v0-2.0317460317460317e-01*Jinv11*v1)*D;
  A[6][2]=(1.3544973544973546e-01*Jinv10*v1-5.0793650793650794e-02*Jinv01*v0+1.3544973544973546e-01*Jinv00*v0-5.0793650793650794e-02*Jinv11*v1)*D;
  A[6][3]=(-4.5149911816578486e-02*Jinv10*v1+2.2574955908289243e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0+2.2574955908289243e-02*Jinv11*v1)*D;
  A[6][4]=(7.0546737213403876e-03*Jinv10*v1+7.0546737213403876e-03*Jinv00*v0)*D;
  A[6][5]=(-2.0317460317460317e-01*Jinv10*v1-9.0299823633156973e-02*Jinv01*v0-2.0317460317460317e-01*Jinv00*v0-9.0299823633156973e-02*Jinv11*v1)*D;
  A[6][6]=(0.0)*D;
  A[6][7]=(2.7089947089947092e-01*Jinv10*v1+1.3544973544973546e-01*Jinv01*v0+2.7089947089947092e-01*Jinv00*v0+1.3544973544973546e-01*Jinv11*v1)*D;
  A[6][8]=(-6.7724867724867729e-02*Jinv10*v1-2.2574955908289243e-02*Jinv01*v0-6.7724867724867729e-02*Jinv00*v0-2.2574955908289243e-02*Jinv11*v1)*D;
  A[6][9]=(-5.0793650793650794e-02*Jinv10*v1+1.3544973544973546e-01*Jinv01*v0-5.0793650793650794e-02*Jinv00*v0+1.3544973544973546e-01*Jinv11*v1)*D;
  A[6][10]=(1.3544973544973546e-01*Jinv10*v1+2.7089947089947092e-01*Jinv01*v0+1.3544973544973546e-01*Jinv00*v0+2.7089947089947092e-01*Jinv11*v1)*D;
  A[6][11]=(-8.4656084656084651e-02*Jinv10*v1-8.4656084656084651e-02*Jinv01*v0-8.4656084656084651e-02*Jinv00*v0-8.4656084656084651e-02*Jinv11*v1)*D;
  A[6][12]=(2.2574955908289243e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0+2.2574955908289243e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[6][13]=(-2.2574955908289243e-02*Jinv10*v1-6.7724867724867729e-02*Jinv01*v0-2.2574955908289243e-02*Jinv00*v0-6.7724867724867729e-02*Jinv11*v1)*D;
  A[6][14]=(7.0546737213403876e-03*Jinv01*v0+7.0546737213403876e-03*Jinv11*v1)*D;
  A[7][0]=(-7.0546737213403876e-03*Jinv10*v1-7.0546737213403876e-03*Jinv01*v0-7.0546737213403876e-03*Jinv00*v0-7.0546737213403876e-03*Jinv11*v1)*D;
  A[7][1]=(4.5149911816578486e-02*Jinv10*v1+6.7724867724867729e-02*Jinv01*v0+4.5149911816578486e-02*Jinv00*v0+6.7724867724867729e-02*Jinv11*v1)*D;
  A[7][2]=(-1.3544973544973546e-01*Jinv10*v1-1.8624338624338624e-01*Jinv01*v0-1.3544973544973546e-01*Jinv00*v0-1.8624338624338624e-01*Jinv11*v1)*D;
  A[7][3]=(9.0299823633156973e-02*Jinv10*v1-1.1287477954144620e-01*Jinv01*v0+9.0299823633156973e-02*Jinv00*v0-1.1287477954144620e-01*Jinv11*v1)*D;
  A[7][4]=(7.0546737213403876e-03*Jinv10*v1+7.0546737213403876e-03*Jinv00*v0)*D;
  A[7][5]=(6.7724867724867729e-02*Jinv10*v1+4.5149911816578486e-02*Jinv01*v0+6.7724867724867729e-02*Jinv00*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[7][6]=(-2.7089947089947092e-01*Jinv10*v1-1.3544973544973546e-01*Jinv01*v0-2.7089947089947092e-01*Jinv00*v0-1.3544973544973546e-01*Jinv11*v1)*D;
  A[7][7]=(0.0)*D;
  A[7][8]=(2.0317460317460317e-01*Jinv10*v1+1.1287477954144620e-01*Jinv01*v0+2.0317460317460317e-01*Jinv00*v0+1.1287477954144620e-01*Jinv11*v1)*D;
  A[7][9]=(8.4656084656084651e-02*Jinv10*v1+8.4656084656084651e-02*Jinv00*v0)*D;
  A[7][10]=(-1.3544973544973546e-01*Jinv10*v1+1.3544973544973546e-01*Jinv01*v0-1.3544973544973546e-01*Jinv00*v0+1.3544973544973546e-01*Jinv11*v1)*D;
  A[7][11]=(5.0793650793650794e-02*Jinv10*v1+1.8624338624338624e-01*Jinv01*v0+5.0793650793650794e-02*Jinv00*v0+1.8624338624338624e-01*Jinv11*v1)*D;
  A[7][12]=(2.2574955908289243e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0+2.2574955908289243e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[7][13]=(-2.2574955908289243e-02*Jinv10*v1-6.7724867724867729e-02*Jinv01*v0-2.2574955908289243e-02*Jinv00*v0-6.7724867724867729e-02*Jinv11*v1)*D;
  A[7][14]=(7.0546737213403876e-03*Jinv01*v0+7.0546737213403876e-03*Jinv11*v1)*D;
  A[8][0]=(7.7601410934744269e-03*Jinv10*v1+7.7601410934744269e-03*Jinv01*v0+7.7601410934744269e-03*Jinv00*v0+7.7601410934744269e-03*Jinv11*v1)*D;
  A[8][1]=(-3.3862433862433865e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0-3.3862433862433865e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[8][2]=(5.9259259259259262e-02*Jinv10*v1+9.0299823633156973e-02*Jinv01*v0+5.9259259259259262e-02*Jinv00*v0+9.0299823633156973e-02*Jinv11*v1)*D;
  A[8][3]=(-7.9012345679012344e-02*Jinv10*v1-1.5802469135802469e-01*Jinv01*v0-7.9012345679012344e-02*Jinv00*v0-1.5802469135802469e-01*Jinv11*v1)*D;
  A[8][4]=(4.5855379188712520e-02*Jinv10*v1+4.5855379188712520e-02*Jinv00*v0)*D;
  A[8][5]=(-2.2574955908289243e-02*Jinv10*v1-1.1287477954144622e-02*Jinv01*v0-2.2574955908289243e-02*Jinv00*v0-1.1287477954144622e-02*Jinv11*v1)*D;
  A[8][6]=(6.7724867724867729e-02*Jinv10*v1+2.2574955908289243e-02*Jinv01*v0+6.7724867724867729e-02*Jinv00*v0+2.2574955908289243e-02*Jinv11*v1)*D;
  A[8][7]=(-2.0317460317460317e-01*Jinv10*v1-1.1287477954144620e-01*Jinv01*v0-2.0317460317460317e-01*Jinv00*v0-1.1287477954144620e-01*Jinv11*v1)*D;
  A[8][8]=(1.5802469135802469e-01*Jinv10*v1+1.5802469135802469e-01*Jinv01*v0+1.5802469135802469e-01*Jinv00*v0+1.5802469135802469e-01*Jinv11*v1)*D;
  A[8][9]=(-1.6931216931216932e-02*Jinv10*v1-2.8218694885361554e-03*Jinv01*v0-1.6931216931216932e-02*Jinv00*v0-2.8218694885361554e-03*Jinv11*v1)*D;
  A[8][10]=(6.7724867724867729e-02*Jinv10*v1+2.2574955908289243e-02*Jinv01*v0+6.7724867724867729e-02*Jinv00*v0+2.2574955908289243e-02*Jinv11*v1)*D;
  A[8][11]=(-5.0793650793650794e-02*Jinv10*v1+2.2574955908289243e-02*Jinv01*v0-5.0793650793650794e-02*Jinv00*v0+2.2574955908289243e-02*Jinv11*v1)*D;
  A[8][12]=(-4.5149911816578486e-02*Jinv10*v1-1.1287477954144622e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0-1.1287477954144622e-02*Jinv11*v1)*D;
  A[8][13]=(4.5149911816578486e-02*Jinv10*v1+4.5149911816578486e-02*Jinv00*v0)*D;
  A[8][14]=(1.7636684303350969e-02*Jinv01*v0+1.7636684303350969e-02*Jinv11*v1)*D;
  A[9][0]=(3.2627865961199293e-02*Jinv10*v1+3.2627865961199293e-02*Jinv01*v0+3.2627865961199293e-02*Jinv00*v0+3.2627865961199293e-02*Jinv11*v1)*D;
  A[9][1]=(-3.1040564373897708e-02*Jinv10*v1+5.9259259259259262e-02*Jinv01*v0-3.1040564373897708e-02*Jinv00*v0+5.9259259259259262e-02*Jinv11*v1)*D;
  A[9][2]=(-1.0582010582010581e-02*Jinv10*v1+1.0582010582010581e-02*Jinv01*v0-1.0582010582010581e-02*Jinv00*v0+1.0582010582010581e-02*Jinv11*v1)*D;
  A[9][3]=(1.4109347442680775e-02*Jinv10*v1-2.8218694885361554e-03*Jinv01*v0+1.4109347442680775e-02*Jinv00*v0-2.8218694885361554e-03*Jinv11*v1)*D;
  A[9][4]=(-5.1146384479717815e-03*Jinv10*v1-5.1146384479717815e-03*Jinv00*v0)*D;
  A[9][5]=(1.6931216931216932e-02*Jinv10*v1-7.3368606701940037e-02*Jinv01*v0+1.6931216931216932e-02*Jinv00*v0-7.3368606701940037e-02*Jinv11*v1)*D;
  A[9][6]=(5.0793650793650794e-02*Jinv10*v1-1.3544973544973546e-01*Jinv01*v0+5.0793650793650794e-02*Jinv00*v0-1.3544973544973546e-01*Jinv11*v1)*D;
  A[9][7]=(-8.4656084656084651e-02*Jinv10*v1-8.4656084656084651e-02*Jinv00*v0)*D;
  A[9][8]=(1.6931216931216932e-02*Jinv10*v1+2.8218694885361554e-03*Jinv01*v0+1.6931216931216932e-02*Jinv00*v0+2.8218694885361554e-03*Jinv11*v1)*D;
  A[9][9]=(-1.6507936507936508e-01*Jinv10*v1-1.6507936507936508e-01*Jinv00*v0)*D;
  A[9][10]=(1.8624338624338624e-01*Jinv10*v1+1.3544973544973546e-01*Jinv01*v0+1.8624338624338624e-01*Jinv00*v0+1.3544973544973546e-01*Jinv11*v1)*D;
  A[9][11]=(-2.1164021164021163e-02*Jinv10*v1-1.0582010582010581e-02*Jinv01*v0-2.1164021164021163e-02*Jinv00*v0-1.0582010582010581e-02*Jinv11*v1)*D;
  A[9][12]=(9.0299823633156973e-02*Jinv10*v1+7.3368606701940037e-02*Jinv01*v0+9.0299823633156973e-02*Jinv00*v0+7.3368606701940037e-02*Jinv11*v1)*D;
  A[9][13]=(-9.0299823633156973e-02*Jinv10*v1-5.9259259259259262e-02*Jinv01*v0-9.0299823633156973e-02*Jinv00*v0-5.9259259259259262e-02*Jinv11*v1)*D;
  A[9][14]=(-3.2627865961199293e-02*Jinv01*v0-3.2627865961199293e-02*Jinv11*v1)*D;
  A[10][0]=(-7.0546737213403876e-03*Jinv10*v1-7.0546737213403876e-03*Jinv01*v0-7.0546737213403876e-03*Jinv00*v0-7.0546737213403876e-03*Jinv11*v1)*D;
  A[10][1]=(4.5149911816578486e-02*Jinv10*v1+6.7724867724867729e-02*Jinv01*v0+4.5149911816578486e-02*Jinv00*v0+6.7724867724867729e-02*Jinv11*v1)*D;
  A[10][2]=(8.4656084656084651e-02*Jinv01*v0+8.4656084656084651e-02*Jinv11*v1)*D;
  A[10][3]=(-4.5149911816578486e-02*Jinv10*v1+2.2574955908289243e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0+2.2574955908289243e-02*Jinv11*v1)*D;
  A[10][4]=(7.0546737213403876e-03*Jinv10*v1+7.0546737213403876e-03*Jinv00*v0)*D;
  A[10][5]=(6.7724867724867729e-02*Jinv10*v1+4.5149911816578486e-02*Jinv01*v0+6.7724867724867729e-02*Jinv00*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[10][6]=(-1.3544973544973546e-01*Jinv10*v1-2.7089947089947092e-01*Jinv01*v0-1.3544973544973546e-01*Jinv00*v0-2.7089947089947092e-01*Jinv11*v1)*D;
  A[10][7]=(1.3544973544973546e-01*Jinv10*v1-1.3544973544973546e-01*Jinv01*v0+1.3544973544973546e-01*Jinv00*v0-1.3544973544973546e-01*Jinv11*v1)*D;
  A[10][8]=(-6.7724867724867729e-02*Jinv10*v1-2.2574955908289243e-02*Jinv01*v0-6.7724867724867729e-02*Jinv00*v0-2.2574955908289243e-02*Jinv11*v1)*D;
  A[10][9]=(-1.8624338624338624e-01*Jinv10*v1-1.3544973544973546e-01*Jinv01*v0-1.8624338624338624e-01*Jinv00*v0-1.3544973544973546e-01*Jinv11*v1)*D;
  A[10][10]=(0.0)*D;
  A[10][11]=(1.8624338624338624e-01*Jinv10*v1+5.0793650793650794e-02*Jinv01*v0+1.8624338624338624e-01*Jinv00*v0+5.0793650793650794e-02*Jinv11*v1)*D;
  A[10][12]=(-1.1287477954144620e-01*Jinv10*v1+9.0299823633156973e-02*Jinv01*v0-1.1287477954144620e-01*Jinv00*v0+9.0299823633156973e-02*Jinv11*v1)*D;
  A[10][13]=(1.1287477954144620e-01*Jinv10*v1+2.0317460317460317e-01*Jinv01*v0+1.1287477954144620e-01*Jinv00*v0+2.0317460317460317e-01*Jinv11*v1)*D;
  A[10][14]=(7.0546737213403876e-03*Jinv01*v0+7.0546737213403876e-03*Jinv11*v1)*D;
  A[11][0]=(5.1146384479717815e-03*Jinv10*v1+5.1146384479717815e-03*Jinv01*v0+5.1146384479717815e-03*Jinv00*v0+5.1146384479717815e-03*Jinv11*v1)*D;
  A[11][1]=(-1.4109347442680775e-02*Jinv10*v1-1.6931216931216932e-02*Jinv01*v0-1.4109347442680775e-02*Jinv00*v0-1.6931216931216932e-02*Jinv11*v1)*D;
  A[11][2]=(1.0582010582010581e-02*Jinv10*v1+2.1164021164021163e-02*Jinv01*v0+1.0582010582010581e-02*Jinv00*v0+2.1164021164021163e-02*Jinv11*v1)*D;
  A[11][3]=(3.1040564373897708e-02*Jinv10*v1+9.0299823633156973e-02*Jinv01*v0+3.1040564373897708e-02*Jinv00*v0+9.0299823633156973e-02*Jinv11*v1)*D;
  A[11][4]=(-3.2627865961199293e-02*Jinv10*v1-3.2627865961199293e-02*Jinv00*v0)*D;
  A[11][5]=(-1.6931216931216932e-02*Jinv10*v1-1.4109347442680775e-02*Jinv01*v0-1.6931216931216932e-02*Jinv00*v0-1.4109347442680775e-02*Jinv11*v1)*D;
  A[11][6]=(8.4656084656084651e-02*Jinv10*v1+8.4656084656084651e-02*Jinv01*v0+8.4656084656084651e-02*Jinv00*v0+8.4656084656084651e-02*Jinv11*v1)*D;
  A[11][7]=(-5.0793650793650794e-02*Jinv10*v1-1.8624338624338624e-01*Jinv01*v0-5.0793650793650794e-02*Jinv00*v0-1.8624338624338624e-01*Jinv11*v1)*D;
  A[11][8]=(-1.6931216931216932e-02*Jinv10*v1-9.0299823633156973e-02*Jinv01*v0-1.6931216931216932e-02*Jinv00*v0-9.0299823633156973e-02*Jinv11*v1)*D;
  A[11][9]=(2.1164021164021163e-02*Jinv10*v1+1.0582010582010581e-02*Jinv01*v0+2.1164021164021163e-02*Jinv00*v0+1.0582010582010581e-02*Jinv11*v1)*D;
  A[11][10]=(-1.8624338624338624e-01*Jinv10*v1-5.0793650793650794e-02*Jinv01*v0-1.8624338624338624e-01*Jinv00*v0-5.0793650793650794e-02*Jinv11*v1)*D;
  A[11][11]=(1.6507936507936508e-01*Jinv10*v1+1.6507936507936508e-01*Jinv01*v0+1.6507936507936508e-01*Jinv00*v0+1.6507936507936508e-01*Jinv11*v1)*D;
  A[11][12]=(9.0299823633156973e-02*Jinv10*v1+3.1040564373897708e-02*Jinv01*v0+9.0299823633156973e-02*Jinv00*v0+3.1040564373897708e-02*Jinv11*v1)*D;
  A[11][13]=(-9.0299823633156973e-02*Jinv10*v1-1.6931216931216932e-02*Jinv01*v0-9.0299823633156973e-02*Jinv00*v0-1.6931216931216932e-02*Jinv11*v1)*D;
  A[11][14]=(-3.2627865961199293e-02*Jinv01*v0-3.2627865961199293e-02*Jinv11*v1)*D;
  A[12][0]=(-1.7636684303350969e-02*Jinv10*v1-1.7636684303350969e-02*Jinv01*v0-1.7636684303350969e-02*Jinv00*v0-1.7636684303350969e-02*Jinv11*v1)*D;
  A[12][1]=(1.1287477954144622e-02*Jinv10*v1-3.3862433862433865e-02*Jinv01*v0+1.1287477954144622e-02*Jinv00*v0-3.3862433862433865e-02*Jinv11*v1)*D;
  A[12][2]=(2.8218694885361554e-03*Jinv10*v1-1.4109347442680775e-02*Jinv01*v0+2.8218694885361554e-03*Jinv00*v0-1.4109347442680775e-02*Jinv11*v1)*D;
  A[12][3]=(1.1287477954144622e-02*Jinv10*v1-1.1287477954144622e-02*Jinv01*v0+1.1287477954144622e-02*Jinv00*v0-1.1287477954144622e-02*Jinv11*v1)*D;
  A[12][4]=(-7.7601410934744269e-03*Jinv10*v1-7.7601410934744269e-03*Jinv00*v0)*D;
  A[12][5]=(4.5149911816578486e-02*Jinv01*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[12][6]=(-2.2574955908289243e-02*Jinv10*v1+4.5149911816578486e-02*Jinv01*v0-2.2574955908289243e-02*Jinv00*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[12][7]=(-2.2574955908289243e-02*Jinv10*v1+4.5149911816578486e-02*Jinv01*v0-2.2574955908289243e-02*Jinv00*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[12][8]=(4.5149911816578486e-02*Jinv10*v1+1.1287477954144622e-02*Jinv01*v0+4.5149911816578486e-02*Jinv00*v0+1.1287477954144622e-02*Jinv11*v1)*D;
  A[12][9]=(-2.2574955908289243e-02*Jinv10*v1-7.3368606701940037e-02*Jinv01*v0-2.2574955908289243e-02*Jinv00*v0-7.3368606701940037e-02*Jinv11*v1)*D;
  A[12][10]=(1.1287477954144620e-01*Jinv10*v1-9.0299823633156973e-02*Jinv01*v0+1.1287477954144620e-01*Jinv00*v0-9.0299823633156973e-02*Jinv11*v1)*D;
  A[12][11]=(-9.0299823633156973e-02*Jinv10*v1-3.1040564373897708e-02*Jinv01*v0-9.0299823633156973e-02*Jinv00*v0-3.1040564373897708e-02*Jinv11*v1)*D;
  A[12][12]=(-1.5802469135802469e-01*Jinv10*v1-1.5802469135802469e-01*Jinv00*v0)*D;
  A[12][13]=(1.5802469135802469e-01*Jinv10*v1+7.9012345679012344e-02*Jinv01*v0+1.5802469135802469e-01*Jinv00*v0+7.9012345679012344e-02*Jinv11*v1)*D;
  A[12][14]=(4.5855379188712520e-02*Jinv01*v0+4.5855379188712520e-02*Jinv11*v1)*D;
  A[13][0]=(7.7601410934744269e-03*Jinv10*v1+7.7601410934744269e-03*Jinv01*v0+7.7601410934744269e-03*Jinv00*v0+7.7601410934744269e-03*Jinv11*v1)*D;
  A[13][1]=(-1.1287477954144622e-02*Jinv10*v1-2.2574955908289243e-02*Jinv01*v0-1.1287477954144622e-02*Jinv00*v0-2.2574955908289243e-02*Jinv11*v1)*D;
  A[13][2]=(-2.8218694885361554e-03*Jinv10*v1-1.6931216931216932e-02*Jinv01*v0-2.8218694885361554e-03*Jinv00*v0-1.6931216931216932e-02*Jinv11*v1)*D;
  A[13][3]=(-1.1287477954144622e-02*Jinv10*v1-4.5149911816578486e-02*Jinv01*v0-1.1287477954144622e-02*Jinv00*v0-4.5149911816578486e-02*Jinv11*v1)*D;
  A[13][4]=(1.7636684303350969e-02*Jinv10*v1+1.7636684303350969e-02*Jinv00*v0)*D;
  A[13][5]=(-4.5149911816578486e-02*Jinv10*v1-3.3862433862433865e-02*Jinv01*v0-4.5149911816578486e-02*Jinv00*v0-3.3862433862433865e-02*Jinv11*v1)*D;
  A[13][6]=(2.2574955908289243e-02*Jinv10*v1+6.7724867724867729e-02*Jinv01*v0+2.2574955908289243e-02*Jinv00*v0+6.7724867724867729e-02*Jinv11*v1)*D;
  A[13][7]=(2.2574955908289243e-02*Jinv10*v1+6.7724867724867729e-02*Jinv01*v0+2.2574955908289243e-02*Jinv00*v0+6.7724867724867729e-02*Jinv11*v1)*D;
  A[13][8]=(4.5149911816578486e-02*Jinv01*v0+4.5149911816578486e-02*Jinv11*v1)*D;
  A[13][9]=(9.0299823633156973e-02*Jinv10*v1+5.9259259259259262e-02*Jinv01*v0+9.0299823633156973e-02*Jinv00*v0+5.9259259259259262e-02*Jinv11*v1)*D;
  A[13][10]=(-1.1287477954144620e-01*Jinv10*v1-2.0317460317460317e-01*Jinv01*v0-1.1287477954144620e-01*Jinv00*v0-2.0317460317460317e-01*Jinv11*v1)*D;
  A[13][11]=(2.2574955908289243e-02*Jinv10*v1-5.0793650793650794e-02*Jinv01*v0+2.2574955908289243e-02*Jinv00*v0-5.0793650793650794e-02*Jinv11*v1)*D;
  A[13][12]=(-1.5802469135802469e-01*Jinv10*v1-7.9012345679012344e-02*Jinv01*v0-1.5802469135802469e-01*Jinv00*v0-7.9012345679012344e-02*Jinv11*v1)*D;
  A[13][13]=(1.5802469135802469e-01*Jinv10*v1+1.5802469135802469e-01*Jinv01*v0+1.5802469135802469e-01*Jinv00*v0+1.5802469135802469e-01*Jinv11*v1)*D;
  A[13][14]=(4.5855379188712520e-02*Jinv01*v0+4.5855379188712520e-02*Jinv11*v1)*D;
  A[14][0]=(5.1146384479717815e-03*Jinv10*v1+5.1146384479717815e-03*Jinv01*v0+5.1146384479717815e-03*Jinv00*v0+5.1146384479717815e-03*Jinv11*v1)*D;
  A[14][1]=(7.7601410934744269e-03*Jinv01*v0+7.7601410934744269e-03*Jinv11*v1)*D;
  A[14][2]=(5.1146384479717815e-03*Jinv01*v0+5.1146384479717815e-03*Jinv11*v1)*D;
  A[14][3]=(7.7601410934744269e-03*Jinv01*v0+7.7601410934744269e-03*Jinv11*v1)*D;
  A[14][4]=(-5.1146384479717815e-03*Jinv10*v1-5.1146384479717815e-03*Jinv00*v0)*D;
  A[14][5]=(-9.8765432098765430e-03*Jinv10*v1-1.7636684303350969e-02*Jinv01*v0-9.8765432098765430e-03*Jinv00*v0-1.7636684303350969e-02*Jinv11*v1)*D;
  A[14][6]=(-7.0546737213403876e-03*Jinv01*v0-7.0546737213403876e-03*Jinv11*v1)*D;
  A[14][7]=(-7.0546737213403876e-03*Jinv01*v0-7.0546737213403876e-03*Jinv11*v1)*D;
  A[14][8]=(9.8765432098765430e-03*Jinv10*v1-7.7601410934744269e-03*Jinv01*v0+9.8765432098765430e-03*Jinv00*v0-7.7601410934744269e-03*Jinv11*v1)*D;
  A[14][9]=(3.0687830687830688e-02*Jinv10*v1+3.2627865961199293e-02*Jinv01*v0+3.0687830687830688e-02*Jinv00*v0+3.2627865961199293e-02*Jinv11*v1)*D;
  A[14][10]=(-7.0546737213403876e-03*Jinv01*v0-7.0546737213403876e-03*Jinv11*v1)*D;
  A[14][11]=(-3.0687830687830688e-02*Jinv10*v1+1.9400352733686067e-03*Jinv01*v0-3.0687830687830688e-02*Jinv00*v0+1.9400352733686067e-03*Jinv11*v1)*D;
  A[14][12]=(-5.2204585537918874e-02*Jinv10*v1-4.5855379188712520e-02*Jinv01*v0-5.2204585537918874e-02*Jinv00*v0-4.5855379188712520e-02*Jinv11*v1)*D;
  A[14][13]=(5.2204585537918874e-02*Jinv10*v1+6.3492063492063492e-03*Jinv01*v0+5.2204585537918874e-02*Jinv00*v0+6.3492063492063492e-03*Jinv11*v1)*D;
  A[14][14]=(2.5749559082892417e-02*Jinv01*v0+2.5749559082892417e-02*Jinv11*v1)*D;
  int nbf = 15;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 5 ---------------*/


 
void matrix_factory_conv_2D_order5 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2],
         double v0, double v1){
  
   //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  
 
  int    iidof[21];
  double dof0[2];
  dof0[0]=x0;  dof0[1]=y0;
  ptv pdof0(2,dof0);
  iidof[0] = dof.insert_dof(element,0,pdof0);

  double dof1[2];
  dof1[0]=G00/5.0+x0;  dof1[1]=G10/5.0+y0;
  ptv pdof1(2,dof1);
  iidof[1] = dof.insert_dof(element,1,pdof1);

  double dof2[2];
  dof2[0]=(2.0/5.0)*G00+x0;  dof2[1]=(2.0/5.0)*G10+y0;
  ptv pdof2(2,dof2);
  iidof[2] = dof.insert_dof(element,2,pdof2);

  double dof3[2];
  dof3[0]=(3.0/5.0)*G00+x0;  dof3[1]=(3.0/5.0)*G10+y0;
  ptv pdof3(2,dof3);
  iidof[3] = dof.insert_dof(element,3,pdof3);

  double dof4[2];
  dof4[0]=(4.0/5.0)*G00+x0;  dof4[1]=(4.0/5.0)*G10+y0;
  ptv pdof4(2,dof4);
  iidof[4] = dof.insert_dof(element,4,pdof4);

  double dof5[2];
  dof5[0]=G00+x0;  dof5[1]=G10+y0;
  ptv pdof5(2,dof5);
  iidof[5] = dof.insert_dof(element,5,pdof5);

  double dof6[2];
  dof6[0]=G01/5.0+x0;  dof6[1]=y0+G11/5.0;
  ptv pdof6(2,dof6);
  iidof[6] = dof.insert_dof(element,6,pdof6);

  double dof7[2];
  dof7[0]=G01/5.0+G00/5.0+x0;  dof7[1]=G10/5.0+y0+G11/5.0;
  ptv pdof7(2,dof7);
  iidof[7] = dof.insert_dof(element,7,pdof7);

  double dof8[2];
  dof8[0]=G01/5.0+(2.0/5.0)*G00+x0;  dof8[1]=(2.0/5.0)*G10+y0+G11/5.0;
  ptv pdof8(2,dof8);
  iidof[8] = dof.insert_dof(element,8,pdof8);

  double dof9[2];
  dof9[0]=G01/5.0+(3.0/5.0)*G00+x0;  dof9[1]=(3.0/5.0)*G10+y0+G11/5.0;
  ptv pdof9(2,dof9);
  iidof[9] = dof.insert_dof(element,9,pdof9);

  double dof10[2];
  dof10[0]=G01/5.0+(4.0/5.0)*G00+x0;  dof10[1]=(4.0/5.0)*G10+y0+G11/5.0;
  ptv pdof10(2,dof10);
  iidof[10] = dof.insert_dof(element,10,pdof10);

  double dof11[2];
  dof11[0]=(2.0/5.0)*G01+x0;  dof11[1]=y0+(2.0/5.0)*G11;
  ptv pdof11(2,dof11);
  iidof[11] = dof.insert_dof(element,11,pdof11);

  double dof12[2];
  dof12[0]=(2.0/5.0)*G01+G00/5.0+x0;  dof12[1]=G10/5.0+y0+(2.0/5.0)*G11;
  ptv pdof12(2,dof12);
  iidof[12] = dof.insert_dof(element,12,pdof12);

  double dof13[2];
  dof13[0]=(2.0/5.0)*G01+(2.0/5.0)*G00+x0;  dof13[1]=(2.0/5.0)*G10+y0+(2.0/5.0)*G11;
  ptv pdof13(2,dof13);
  iidof[13] = dof.insert_dof(element,13,pdof13);

  double dof14[2];
  dof14[0]=(2.0/5.0)*G01+(3.0/5.0)*G00+x0;  dof14[1]=(3.0/5.0)*G10+y0+(2.0/5.0)*G11;
  ptv pdof14(2,dof14);
  iidof[14] = dof.insert_dof(element,14,pdof14);

  double dof15[2];
  dof15[0]=(3.0/5.0)*G01+x0;  dof15[1]=y0+(3.0/5.0)*G11;
  ptv pdof15(2,dof15);
  iidof[15] = dof.insert_dof(element,15,pdof15);

  double dof16[2];
  dof16[0]=(3.0/5.0)*G01+G00/5.0+x0;  dof16[1]=G10/5.0+y0+(3.0/5.0)*G11;
  ptv pdof16(2,dof16);
  iidof[16] = dof.insert_dof(element,16,pdof16);

  double dof17[2];
  dof17[0]=(3.0/5.0)*G01+(2.0/5.0)*G00+x0;  dof17[1]=(2.0/5.0)*G10+y0+(3.0/5.0)*G11;
  ptv pdof17(2,dof17);
  iidof[17] = dof.insert_dof(element,17,pdof17);

  double dof18[2];
  dof18[0]=(4.0/5.0)*G01+x0;  dof18[1]=y0+(4.0/5.0)*G11;
  ptv pdof18(2,dof18);
  iidof[18] = dof.insert_dof(element,18,pdof18);

  double dof19[2];
  dof19[0]=(4.0/5.0)*G01+G00/5.0+x0;  dof19[1]=G10/5.0+y0+(4.0/5.0)*G11;
  ptv pdof19(2,dof19);
  iidof[19] = dof.insert_dof(element,19,pdof19);

  double dof20[2];
  dof20[0]=G01+x0;  dof20[1]=y0+G11;
  ptv pdof20(2,dof20);
  iidof[20] = dof.insert_dof(element,20,pdof20);

 
  double Jinv00 = 1.0/(G00*G11-G10*G01)*G11;
  double Jinv01 = -1.0*1.0/(G00*G11-G10*G01)*G10;
  double Jinv10 = -1.0*1.0/(G00*G11-G10*G01)*G01;
  double Jinv11 = 1.0/(G00*G11-G10*G01)*G00;


  double A[21][21];
  A[0][0]=(-1.9109748276414942e-02*v1*Jinv11-1.9109748276414942e-02*Jinv00*v0-1.9109748276414942e-02*v1*Jinv10-1.9109748276414942e-02*v0*Jinv01)*D;
  A[0][1]=(-9.2927664542247872e-03*v1*Jinv11+3.7257182439474103e-02*Jinv00*v0+3.7257182439474103e-02*v1*Jinv10-9.2927664542247872e-03*v0*Jinv01)*D;
  A[0][2]=(3.6377749051360164e-03*v1*Jinv11-3.3658551453690344e-02*Jinv00*v0-3.3658551453690344e-02*v1*Jinv10+3.6377749051360164e-03*v0*Jinv01)*D;
  A[0][3]=(1.8123634660092993e-03*v1*Jinv11+2.3263231271043770e-02*Jinv00*v0+2.3263231271043770e-02*v1*Jinv10+1.8123634660092993e-03*v0*Jinv01)*D;
  A[0][4]=(-4.8003493098979213e-03*v1*Jinv11-1.0531019103067715e-02*Jinv00*v0-1.0531019103067715e-02*v1*Jinv10-4.8003493098979213e-03*v0*Jinv01)*D;
  A[0][5]=(2.7789051226551225e-03*Jinv00*v0+2.7789051226551225e-03*v1*Jinv10)*D;
  A[0][6]=(3.7257182439474103e-02*v1*Jinv11-9.2927664542247872e-03*Jinv00*v0-9.2927664542247872e-03*v1*Jinv10+3.7257182439474103e-02*v0*Jinv01)*D;
  A[0][7]=(1.4394523943135054e-02*v1*Jinv11+1.4394523943135054e-02*Jinv00*v0+1.4394523943135054e-02*v1*Jinv10+1.4394523943135054e-02*v0*Jinv01)*D;
  A[0][8]=(-4.7246710658168994e-03*v1*Jinv11-4.7246710658168994e-03*Jinv00*v0-4.7246710658168994e-03*v1*Jinv10-4.7246710658168994e-03*v0*Jinv01)*D;
  A[0][9]=(-5.1774357329912885e-03*v1*Jinv11-5.1774357329912885e-03*Jinv00*v0-5.1774357329912885e-03*v1*Jinv10-5.1774357329912885e-03*v0*Jinv01)*D;
  A[0][10]=(4.8003493098979213e-03*v1*Jinv11+4.8003493098979213e-03*Jinv00*v0+4.8003493098979213e-03*v1*Jinv10+4.8003493098979213e-03*v0*Jinv01)*D;
  A[0][11]=(-3.3658551453690344e-02*v1*Jinv11+3.6377749051360164e-03*Jinv00*v0+3.6377749051360164e-03*v1*Jinv10-3.3658551453690344e-02*v0*Jinv01)*D;
  A[0][12]=(-4.7246710658168994e-03*v1*Jinv11-4.7246710658168994e-03*Jinv00*v0-4.7246710658168994e-03*v1*Jinv10-4.7246710658168994e-03*v0*Jinv01)*D;
  A[0][13]=(-2.2781761063011062e-03*v1*Jinv11-2.2781761063011062e-03*Jinv00*v0-2.2781761063011062e-03*v1*Jinv10-2.2781761063011062e-03*v0*Jinv01)*D;
  A[0][14]=(3.3650722669819892e-03*v1*Jinv11+3.3650722669819892e-03*Jinv00*v0+3.3650722669819892e-03*v1*Jinv10+3.3650722669819892e-03*v0*Jinv01)*D;
  A[0][15]=(2.3263231271043770e-02*v1*Jinv11+1.8123634660092993e-03*Jinv00*v0+1.8123634660092993e-03*v1*Jinv10+2.3263231271043770e-02*v0*Jinv01)*D;
  A[0][16]=(-5.1774357329912885e-03*v1*Jinv11-5.1774357329912885e-03*Jinv00*v0-5.1774357329912885e-03*v1*Jinv10-5.1774357329912885e-03*v0*Jinv01)*D;
  A[0][17]=(3.3650722669819892e-03*v1*Jinv11+3.3650722669819892e-03*Jinv00*v0+3.3650722669819892e-03*v1*Jinv10+3.3650722669819892e-03*v0*Jinv01)*D;
  A[0][18]=(-1.0531019103067715e-02*v1*Jinv11-4.8003493098979213e-03*Jinv00*v0-4.8003493098979213e-03*v1*Jinv10-1.0531019103067715e-02*v0*Jinv01)*D;
  A[0][19]=(4.8003493098979213e-03*v1*Jinv11+4.8003493098979213e-03*Jinv00*v0+4.8003493098979213e-03*v1*Jinv10+4.8003493098979213e-03*v0*Jinv01)*D;
  A[0][20]=(2.7789051226551225e-03*v1*Jinv11+2.7789051226551225e-03*v0*Jinv01)*D;
  A[1][0]=(-3.7257182439474103e-02*v1*Jinv11-3.7257182439474103e-02*Jinv00*v0-3.7257182439474103e-02*v1*Jinv10-3.7257182439474103e-02*v0*Jinv01)*D;
  A[1][1]=(-1.3982207992624660e-01*v1*Jinv11-1.3982207992624660e-01*v0*Jinv01)*D;
  A[1][2]=(4.9008187029020364e-02*v1*Jinv11+6.5109385855913637e-02*Jinv00*v0+6.5109385855913637e-02*v1*Jinv10+4.9008187029020364e-02*v0*Jinv01)*D;
  A[1][3]=(-3.6586516621238845e-02*v1*Jinv11-4.3612850144968199e-02*Jinv00*v0-4.3612850144968199e-02*v1*Jinv10-3.6586516621238845e-02*v0*Jinv01)*D;
  A[1][4]=(3.2835224349981297e-02*v1*Jinv11+2.6291665831596388e-02*Jinv00*v0+2.6291665831596388e-02*v1*Jinv10+3.2835224349981297e-02*v0*Jinv01)*D;
  A[1][5]=(-1.0531019103067715e-02*Jinv00*v0-1.0531019103067715e-02*v1*Jinv10)*D;
  A[1][6]=(6.9911039963123298e-02*v1*Jinv11-6.9911039963123298e-02*Jinv00*v0-6.9911039963123298e-02*v1*Jinv10+6.9911039963123298e-02*v0*Jinv01)*D;
  A[1][7]=(1.9258808321308321e-01*v1*Jinv11+6.7849460210571325e-02*Jinv00*v0+6.7849460210571325e-02*v1*Jinv10+1.9258808321308321e-01*v0*Jinv01)*D;
  A[1][8]=(-4.9321338383838384e-02*v1*Jinv11-6.3608868947410612e-03*Jinv00*v0-6.3608868947410612e-03*v1*Jinv10-4.9321338383838384e-02*v0*Jinv01)*D;
  A[1][9]=(4.7755581609748278e-02*v1*Jinv11+3.5881926072898293e-02*Jinv00*v0+3.5881926072898293e-02*v1*Jinv10+4.7755581609748278e-02*v0*Jinv01)*D;
  A[1][10]=(-3.2835224349981297e-02*v1*Jinv11-2.7459459425605258e-02*Jinv00*v0-2.7459459425605258e-02*v1*Jinv10-3.2835224349981297e-02*v0*Jinv01)*D;
  A[1][11]=(-6.9911039963123298e-02*v1*Jinv11+3.8726384212495323e-02*Jinv00*v0+3.8726384212495323e-02*v1*Jinv10-6.9911039963123298e-02*v0*Jinv01)*D;
  A[1][12]=(-8.4550865800865807e-02*v1*Jinv11-4.1590414311768478e-02*Jinv00*v0-4.1590414311768478e-02*v1*Jinv10-8.4550865800865807e-02*v0*Jinv01)*D;
  A[1][13]=(1.4287530563572230e-02*v1*Jinv11+1.1743175805675805e-02*Jinv00*v0+1.1743175805675805e-02*v1*Jinv10+1.4287530563572230e-02*v0*Jinv01)*D;
  A[1][14]=(-1.1169064988509433e-02*v1*Jinv11-8.8791457064026514e-03*Jinv00*v0-8.8791457064026514e-03*v1*Jinv10-1.1169064988509433e-02*v0*Jinv01)*D;
  A[1][15]=(5.9916292555181445e-02*v1*Jinv11-1.8978276899283844e-02*Jinv00*v0-1.8978276899283844e-02*v1*Jinv10+5.9916292555181445e-02*v0*Jinv01)*D;
  A[1][16]=(4.2536392362781249e-02*v1*Jinv11+3.0662736825931271e-02*Jinv00*v0+3.0662736825931271e-02*v1*Jinv10+4.2536392362781249e-02*v0*Jinv01)*D;
  A[1][17]=(-1.3974379208754209e-02*v1*Jinv11-1.1684459926647427e-02*Jinv00*v0-1.1684459926647427e-02*v1*Jinv10-1.3974379208754209e-02*v0*Jinv01)*D;
  A[1][18]=(-2.7459459425605258e-02*v1*Jinv11+5.3757649243760357e-03*Jinv00*v0+5.3757649243760357e-03*v1*Jinv10-2.7459459425605258e-02*v0*Jinv01)*D;
  A[1][19]=(-1.0751529848752071e-02*v1*Jinv11-5.3757649243760357e-03*Jinv00*v0-5.3757649243760357e-03*v1*Jinv10-1.0751529848752071e-02*v0*Jinv01)*D;
  A[1][20]=(4.8003493098979213e-03*v1*Jinv11+4.8003493098979213e-03*v0*Jinv01)*D;
  A[2][0]=(3.3658551453690344e-02*v1*Jinv11+3.3658551453690344e-02*Jinv00*v0+3.3658551453690344e-02*v1*Jinv10+3.3658551453690344e-02*v0*Jinv01)*D;
  A[2][1]=(4.3528038319704990e-02*v1*Jinv11-6.5109385855913637e-02*Jinv00*v0-6.5109385855913637e-02*v1*Jinv10+4.3528038319704990e-02*v0*Jinv01)*D;
  A[2][2]=(-1.5626252605419272e-01*v1*Jinv11-1.5626252605419272e-01*v0*Jinv01)*D;
  A[2][3]=(7.7100473150820373e-02*v1*Jinv11+5.1800453276147719e-02*Jinv00*v0+5.1800453276147719e-02*v1*Jinv10+7.7100473150820373e-02*v0*Jinv01)*D;
  A[2][4]=(-7.8894569454465283e-02*v1*Jinv11-4.3612850144968199e-02*Jinv00*v0-4.3612850144968199e-02*v1*Jinv10-7.8894569454465283e-02*v0*Jinv01)*D;
  A[2][5]=(2.3263231271043770e-02*Jinv00*v0+2.3263231271043770e-02*v1*Jinv10)*D;
  A[2][6]=(-3.8726384212495323e-02*v1*Jinv11+6.9911039963123298e-02*Jinv00*v0+6.9911039963123298e-02*v1*Jinv10-3.8726384212495323e-02*v0*Jinv01)*D;
  A[2][7]=(1.0960297418630752e-02*v1*Jinv11-1.3569892042114265e-01*Jinv00*v0-1.3569892042114265e-01*v1*Jinv10+1.0960297418630752e-02*v0*Jinv01)*D;
  A[2][8]=(1.6773169442440275e-01*v1*Jinv11+9.0031014510181182e-02*Jinv00*v0+9.0031014510181182e-02*v1*Jinv10+1.6773169442440275e-01*v0*Jinv01)*D;
  A[2][9]=(-8.2332710370904810e-02*v1*Jinv11-8.4159426607343268e-02*Jinv00*v0-8.4159426607343268e-02*v1*Jinv10-8.2332710370904810e-02*v0*Jinv01)*D;
  A[2][10]=(7.8894569454465283e-02*v1*Jinv11+5.9916292555181445e-02*Jinv00*v0+5.9916292555181445e-02*v1*Jinv10+7.8894569454465283e-02*v0*Jinv01)*D;
  A[2][11]=(4.8016541072096628e-03*v1*Jinv11-4.8016541072096628e-03*Jinv00*v0-4.8016541072096628e-03*v1*Jinv10+4.8016541072096628e-03*v0*Jinv01)*D;
  A[2][12]=(-8.0832193462401794e-02*v1*Jinv11+2.7400743546576879e-02*Jinv00*v0+2.7400743546576879e-02*v1*Jinv10-8.0832193462401794e-02*v0*Jinv01)*D;
  A[2][13]=(-3.7773882174923841e-02*v1*Jinv11-2.8379341530383198e-02*Jinv00*v0-2.8379341530383198e-02*v1*Jinv10-3.7773882174923841e-02*v0*Jinv01)*D;
  A[2][14]=(5.2322372200844427e-03*v1*Jinv11+5.7802520910159800e-03*Jinv00*v0+5.7802520910159800e-03*v1*Jinv10+5.2322372200844427e-03*v0*Jinv01)*D;
  A[2][15]=(5.7802520910159800e-03*v1*Jinv11+5.4801487093153760e-04*Jinv00*v0+5.4801487093153760e-04*v1*Jinv10+5.7802520910159800e-03*v0*Jinv01)*D;
  A[2][16]=(4.0318236932820266e-02*v1*Jinv11-1.3700371773288440e-02*Jinv00*v0-1.3700371773288440e-02*v1*Jinv10+4.0318236932820266e-02*v0*Jinv01)*D;
  A[2][17]=(2.6304713804713806e-02*v1*Jinv11+1.3152356902356903e-02*Jinv00*v0+1.3152356902356903e-02*v1*Jinv10+2.6304713804713806e-02*v0*Jinv01)*D;
  A[2][18]=(-8.8791457064026514e-03*v1*Jinv11+2.2899192821067820e-03*Jinv00*v0+2.2899192821067820e-03*v1*Jinv10-8.8791457064026514e-03*v0*Jinv01)*D;
  A[2][19]=(-1.3974379208754209e-02*v1*Jinv11-2.2899192821067820e-03*Jinv00*v0-2.2899192821067820e-03*v1*Jinv10-1.3974379208754209e-02*v0*Jinv01)*D;
  A[2][20]=(3.3650722669819892e-03*v1*Jinv11+3.3650722669819892e-03*v0*Jinv01)*D;
  A[3][0]=(-2.3263231271043770e-02*v1*Jinv11-2.3263231271043770e-02*Jinv00*v0-2.3263231271043770e-02*v1*Jinv10-2.3263231271043770e-02*v0*Jinv01)*D;
  A[3][1]=(-3.5281719309497091e-02*v1*Jinv11+4.3612850144968199e-02*Jinv00*v0+4.3612850144968199e-02*v1*Jinv10-3.5281719309497091e-02*v0*Jinv01)*D;
  A[3][2]=(2.5300019874672654e-02*v1*Jinv11-5.1800453276147719e-02*Jinv00*v0-5.1800453276147719e-02*v1*Jinv10+2.5300019874672654e-02*v0*Jinv01)*D;
  A[3][3]=(-1.5626252605419272e-01*v1*Jinv11-1.5626252605419272e-01*v0*Jinv01)*D;
  A[3][4]=(1.0863742417561861e-01*v1*Jinv11+6.5109385855913637e-02*Jinv00*v0+6.5109385855913637e-02*v1*Jinv10+1.0863742417561861e-01*v0*Jinv01)*D;
  A[3][5]=(-3.3658551453690344e-02*Jinv00*v0-3.3658551453690344e-02*v1*Jinv10)*D;
  A[3][6]=(1.8978276899283844e-02*v1*Jinv11-5.9916292555181445e-02*Jinv00*v0-5.9916292555181445e-02*v1*Jinv10+1.8978276899283844e-02*v0*Jinv01)*D;
  A[3][7]=(1.8267162364384586e-03*v1*Jinv11+8.4159426607343268e-02*Jinv00*v0+8.4159426607343268e-02*v1*Jinv10+1.8267162364384586e-03*v0*Jinv01)*D;
  A[3][8]=(7.7700679914221582e-02*v1*Jinv11-9.0031014510181182e-02*Jinv00*v0-9.0031014510181182e-02*v1*Jinv10+7.7700679914221582e-02*v0*Jinv01)*D;
  A[3][9]=(1.4665921783977340e-01*v1*Jinv11+1.3569892042114265e-01*Jinv00*v0+1.3569892042114265e-01*v1*Jinv10+1.4665921783977340e-01*v0*Jinv01)*D;
  A[3][10]=(-1.0863742417561861e-01*v1*Jinv11-6.9911039963123298e-02*Jinv00*v0-6.9911039963123298e-02*v1*Jinv10-1.0863742417561861e-01*v0*Jinv01)*D;
  A[3][11]=(-5.4801487093153760e-04*v1*Jinv11-5.7802520910159800e-03*Jinv00*v0-5.7802520910159800e-03*v1*Jinv10-5.4801487093153760e-04*v0*Jinv01)*D;
  A[3][12]=(-9.3945406445406446e-03*v1*Jinv11+2.8379341530383198e-02*Jinv00*v0+2.8379341530383198e-02*v1*Jinv10-9.3945406445406446e-03*v0*Jinv01)*D;
  A[3][13]=(-1.0823293700897868e-01*v1*Jinv11-2.7400743546576879e-02*Jinv00*v0-2.7400743546576879e-02*v1*Jinv10-1.0823293700897868e-01*v0*Jinv01)*D;
  A[3][14]=(9.6033082144193257e-03*v1*Jinv11+4.8016541072096628e-03*Jinv00*v0+4.8016541072096628e-03*v1*Jinv10+9.6033082144193257e-03*v0*Jinv01)*D;
  A[3][15]=(1.3152356902356903e-02*v1*Jinv11-1.3152356902356903e-02*Jinv00*v0-1.3152356902356903e-02*v1*Jinv10+1.3152356902356903e-02*v0*Jinv01)*D;
  A[3][16]=(5.4018608706108709e-02*v1*Jinv11+1.3700371773288440e-02*Jinv00*v0+1.3700371773288440e-02*v1*Jinv10+5.4018608706108709e-02*v0*Jinv01)*D;
  A[3][17]=(5.2322372200844427e-03*v1*Jinv11-5.4801487093153760e-04*Jinv00*v0-5.4801487093153760e-04*v1*Jinv10+5.2322372200844427e-03*v0*Jinv01)*D;
  A[3][18]=(-1.1684459926647427e-02*v1*Jinv11+2.2899192821067820e-03*Jinv00*v0+2.2899192821067820e-03*v1*Jinv10-1.1684459926647427e-02*v0*Jinv01)*D;
  A[3][19]=(-1.1169064988509433e-02*v1*Jinv11-2.2899192821067820e-03*Jinv00*v0-2.2899192821067820e-03*v1*Jinv10-1.1169064988509433e-02*v0*Jinv01)*D;
  A[3][20]=(3.3650722669819892e-03*v1*Jinv11+3.3650722669819892e-03*v0*Jinv01)*D;
  A[4][0]=(1.0531019103067715e-02*v1*Jinv11+1.0531019103067715e-02*Jinv00*v0+1.0531019103067715e-02*v1*Jinv10+1.0531019103067715e-02*v0*Jinv01)*D;
  A[4][1]=(6.5435585183849076e-03*v1*Jinv11-2.6291665831596388e-02*Jinv00*v0-2.6291665831596388e-02*v1*Jinv10+6.5435585183849076e-03*v0*Jinv01)*D;
  A[4][2]=(7.0263335237293568e-03*v1*Jinv11+4.3612850144968199e-02*Jinv00*v0+4.3612850144968199e-02*v1*Jinv10+7.0263335237293568e-03*v0*Jinv01)*D;
  A[4][3]=(-1.6101198826893270e-02*v1*Jinv11-6.5109385855913637e-02*Jinv00*v0-6.5109385855913637e-02*v1*Jinv10-1.6101198826893270e-02*v0*Jinv01)*D;
  A[4][4]=(-1.3982207992624660e-01*v1*Jinv11-1.3982207992624660e-01*v0*Jinv01)*D;
  A[4][5]=(3.7257182439474103e-02*Jinv00*v0+3.7257182439474103e-02*v1*Jinv10)*D;
  A[4][6]=(-5.3757649243760357e-03*v1*Jinv11+2.7459459425605258e-02*Jinv00*v0+2.7459459425605258e-02*v1*Jinv10-5.3757649243760357e-03*v0*Jinv01)*D;
  A[4][7]=(1.1873655536849981e-02*v1*Jinv11-3.5881926072898293e-02*Jinv00*v0-3.5881926072898293e-02*v1*Jinv10+1.1873655536849981e-02*v0*Jinv01)*D;
  A[4][8]=(-4.2960451489097322e-02*v1*Jinv11+6.3608868947410612e-03*Jinv00*v0+6.3608868947410612e-03*v1*Jinv10-4.2960451489097322e-02*v0*Jinv01)*D;
  A[4][9]=(1.2473862300251189e-01*v1*Jinv11-6.7849460210571325e-02*Jinv00*v0-6.7849460210571325e-02*v1*Jinv10+1.2473862300251189e-01*v0*Jinv01)*D;
  A[4][10]=(1.3982207992624660e-01*v1*Jinv11+6.9911039963123298e-02*Jinv00*v0+6.9911039963123298e-02*v1*Jinv10+1.3982207992624660e-01*v0*Jinv01)*D;
  A[4][11]=(-2.2899192821067820e-03*v1*Jinv11+8.8791457064026514e-03*Jinv00*v0+8.8791457064026514e-03*v1*Jinv10-2.2899192821067820e-03*v0*Jinv01)*D;
  A[4][12]=(2.5443547578964244e-03*v1*Jinv11-1.1743175805675805e-02*Jinv00*v0-1.1743175805675805e-02*v1*Jinv10+2.5443547578964244e-03*v0*Jinv01)*D;
  A[4][13]=(-4.2960451489097322e-02*v1*Jinv11+4.1590414311768478e-02*Jinv00*v0+4.1590414311768478e-02*v1*Jinv10-4.2960451489097322e-02*v0*Jinv01)*D;
  A[4][14]=(-1.0863742417561861e-01*v1*Jinv11-3.8726384212495323e-02*Jinv00*v0-3.8726384212495323e-02*v1*Jinv10-1.0863742417561861e-01*v0*Jinv01)*D;
  A[4][15]=(-2.2899192821067820e-03*v1*Jinv11+1.1684459926647427e-02*Jinv00*v0+1.1684459926647427e-02*v1*Jinv10-2.2899192821067820e-03*v0*Jinv01)*D;
  A[4][16]=(1.1873655536849981e-02*v1*Jinv11-3.0662736825931271e-02*Jinv00*v0-3.0662736825931271e-02*v1*Jinv10+1.1873655536849981e-02*v0*Jinv01)*D;
  A[4][17]=(7.8894569454465283e-02*v1*Jinv11+1.8978276899283844e-02*Jinv00*v0+1.8978276899283844e-02*v1*Jinv10+7.8894569454465283e-02*v0*Jinv01)*D;
  A[4][18]=(-5.3757649243760357e-03*v1*Jinv11+5.3757649243760357e-03*Jinv00*v0+5.3757649243760357e-03*v1*Jinv10-5.3757649243760357e-03*v0*Jinv01)*D;
  A[4][19]=(-3.2835224349981297e-02*v1*Jinv11-5.3757649243760357e-03*Jinv00*v0-5.3757649243760357e-03*v1*Jinv10-3.2835224349981297e-02*v0*Jinv01)*D;
  A[4][20]=(4.8003493098979213e-03*v1*Jinv11+4.8003493098979213e-03*v0*Jinv01)*D;
  A[5][0]=(-2.7789051226551225e-03*v1*Jinv11-2.7789051226551225e-03*Jinv00*v0-2.7789051226551225e-03*v1*Jinv10-2.7789051226551225e-03*v0*Jinv01)*D;
  A[5][1]=(5.7306697931697936e-03*v1*Jinv11+1.0531019103067715e-02*Jinv00*v0+1.0531019103067715e-02*v1*Jinv10+5.7306697931697936e-03*v0*Jinv01)*D;
  A[5][2]=(-2.1450867805034472e-02*v1*Jinv11-2.3263231271043770e-02*Jinv00*v0-2.3263231271043770e-02*v1*Jinv10-2.1450867805034472e-02*v0*Jinv01)*D;
  A[5][3]=(3.7296326358826362e-02*v1*Jinv11+3.3658551453690344e-02*Jinv00*v0+3.3658551453690344e-02*v1*Jinv10+3.7296326358826362e-02*v0*Jinv01)*D;
  A[5][4]=(-4.6549948893698893e-02*v1*Jinv11-3.7257182439474103e-02*Jinv00*v0-3.7257182439474103e-02*v1*Jinv10-4.6549948893698893e-02*v0*Jinv01)*D;
  A[5][5]=(1.9109748276414942e-02*Jinv00*v0+1.9109748276414942e-02*v1*Jinv10)*D;
  A[5][6]=(-4.8003493098979213e-03*Jinv00*v0-4.8003493098979213e-03*v1*Jinv10)*D;
  A[5][7]=(5.1774357329912885e-03*Jinv00*v0+5.1774357329912885e-03*v1*Jinv10)*D;
  A[5][8]=(4.7246710658168994e-03*Jinv00*v0+4.7246710658168994e-03*v1*Jinv10)*D;
  A[5][9]=(-1.4394523943135054e-02*Jinv00*v0-1.4394523943135054e-02*v1*Jinv10)*D;
  A[5][10]=(4.6549948893698893e-02*v1*Jinv11+9.2927664542247872e-03*Jinv00*v0+9.2927664542247872e-03*v1*Jinv10+4.6549948893698893e-02*v0*Jinv01)*D;
  A[5][11]=(-3.3650722669819892e-03*Jinv00*v0-3.3650722669819892e-03*v1*Jinv10)*D;
  A[5][12]=(2.2781761063011062e-03*Jinv00*v0+2.2781761063011062e-03*v1*Jinv10)*D;
  A[5][13]=(4.7246710658168994e-03*Jinv00*v0+4.7246710658168994e-03*v1*Jinv10)*D;
  A[5][14]=(-3.7296326358826362e-02*v1*Jinv11-3.6377749051360164e-03*Jinv00*v0-3.6377749051360164e-03*v1*Jinv10-3.7296326358826362e-02*v0*Jinv01)*D;
  A[5][15]=(-3.3650722669819892e-03*Jinv00*v0-3.3650722669819892e-03*v1*Jinv10)*D;
  A[5][16]=(5.1774357329912885e-03*Jinv00*v0+5.1774357329912885e-03*v1*Jinv10)*D;
  A[5][17]=(2.1450867805034472e-02*v1*Jinv11-1.8123634660092993e-03*Jinv00*v0-1.8123634660092993e-03*v1*Jinv10+2.1450867805034472e-02*v0*Jinv01)*D;
  A[5][18]=(-4.8003493098979213e-03*Jinv00*v0-4.8003493098979213e-03*v1*Jinv10)*D;
  A[5][19]=(-5.7306697931697936e-03*v1*Jinv11+4.8003493098979213e-03*Jinv00*v0+4.8003493098979213e-03*v1*Jinv10-5.7306697931697936e-03*v0*Jinv01)*D;
  A[5][20]=(2.7789051226551225e-03*v1*Jinv11+2.7789051226551225e-03*v0*Jinv01)*D;
  A[6][0]=(-3.7257182439474103e-02*v1*Jinv11-3.7257182439474103e-02*Jinv00*v0-3.7257182439474103e-02*v1*Jinv10-3.7257182439474103e-02*v0*Jinv01)*D;
  A[6][1]=(-6.9911039963123298e-02*v1*Jinv11+6.9911039963123298e-02*Jinv00*v0+6.9911039963123298e-02*v1*Jinv10-6.9911039963123298e-02*v0*Jinv01)*D;
  A[6][2]=(3.8726384212495323e-02*v1*Jinv11-6.9911039963123298e-02*Jinv00*v0-6.9911039963123298e-02*v1*Jinv10+3.8726384212495323e-02*v0*Jinv01)*D;
  A[6][3]=(-1.8978276899283844e-02*v1*Jinv11+5.9916292555181445e-02*Jinv00*v0+5.9916292555181445e-02*v1*Jinv10-1.8978276899283844e-02*v0*Jinv01)*D;
  A[6][4]=(5.3757649243760357e-03*v1*Jinv11-2.7459459425605258e-02*Jinv00*v0-2.7459459425605258e-02*v1*Jinv10+5.3757649243760357e-03*v0*Jinv01)*D;
  A[6][5]=(4.8003493098979213e-03*Jinv00*v0+4.8003493098979213e-03*v1*Jinv10)*D;
  A[6][6]=(-1.3982207992624660e-01*Jinv00*v0-1.3982207992624660e-01*v1*Jinv10)*D;
  A[6][7]=(6.7849460210571325e-02*v1*Jinv11+1.9258808321308321e-01*Jinv00*v0+1.9258808321308321e-01*v1*Jinv10+6.7849460210571325e-02*v0*Jinv01)*D;
  A[6][8]=(-4.1590414311768478e-02*v1*Jinv11-8.4550865800865807e-02*Jinv00*v0-8.4550865800865807e-02*v1*Jinv10-4.1590414311768478e-02*v0*Jinv01)*D;
  A[6][9]=(3.0662736825931271e-02*v1*Jinv11+4.2536392362781249e-02*Jinv00*v0+4.2536392362781249e-02*v1*Jinv10+3.0662736825931271e-02*v0*Jinv01)*D;
  A[6][10]=(-5.3757649243760357e-03*v1*Jinv11-1.0751529848752071e-02*Jinv00*v0-1.0751529848752071e-02*v1*Jinv10-5.3757649243760357e-03*v0*Jinv01)*D;
  A[6][11]=(6.5109385855913637e-02*v1*Jinv11+4.9008187029020364e-02*Jinv00*v0+4.9008187029020364e-02*v1*Jinv10+6.5109385855913637e-02*v0*Jinv01)*D;
  A[6][12]=(-6.3608868947410612e-03*v1*Jinv11-4.9321338383838384e-02*Jinv00*v0-4.9321338383838384e-02*v1*Jinv10-6.3608868947410612e-03*v0*Jinv01)*D;
  A[6][13]=(1.1743175805675805e-02*v1*Jinv11+1.4287530563572230e-02*Jinv00*v0+1.4287530563572230e-02*v1*Jinv10+1.1743175805675805e-02*v0*Jinv01)*D;
  A[6][14]=(-1.1684459926647427e-02*v1*Jinv11-1.3974379208754209e-02*Jinv00*v0-1.3974379208754209e-02*v1*Jinv10-1.1684459926647427e-02*v0*Jinv01)*D;
  A[6][15]=(-4.3612850144968199e-02*v1*Jinv11-3.6586516621238845e-02*Jinv00*v0-3.6586516621238845e-02*v1*Jinv10-4.3612850144968199e-02*v0*Jinv01)*D;
  A[6][16]=(3.5881926072898293e-02*v1*Jinv11+4.7755581609748278e-02*Jinv00*v0+4.7755581609748278e-02*v1*Jinv10+3.5881926072898293e-02*v0*Jinv01)*D;
  A[6][17]=(-8.8791457064026514e-03*v1*Jinv11-1.1169064988509433e-02*Jinv00*v0-1.1169064988509433e-02*v1*Jinv10-8.8791457064026514e-03*v0*Jinv01)*D;
  A[6][18]=(2.6291665831596388e-02*v1*Jinv11+3.2835224349981297e-02*Jinv00*v0+3.2835224349981297e-02*v1*Jinv10+2.6291665831596388e-02*v0*Jinv01)*D;
  A[6][19]=(-2.7459459425605258e-02*v1*Jinv11-3.2835224349981297e-02*Jinv00*v0-3.2835224349981297e-02*v1*Jinv10-2.7459459425605258e-02*v0*Jinv01)*D;
  A[6][20]=(-1.0531019103067715e-02*v1*Jinv11-1.0531019103067715e-02*v0*Jinv01)*D;
  A[7][0]=(-1.4394523943135054e-02*v1*Jinv11-1.4394523943135054e-02*Jinv00*v0-1.4394523943135054e-02*v1*Jinv10-1.4394523943135054e-02*v0*Jinv01)*D;
  A[7][1]=(-1.9258808321308321e-01*v1*Jinv11-6.7849460210571325e-02*Jinv00*v0-6.7849460210571325e-02*v1*Jinv10-1.9258808321308321e-01*v0*Jinv01)*D;
  A[7][2]=(-1.0960297418630752e-02*v1*Jinv11+1.3569892042114265e-01*Jinv00*v0+1.3569892042114265e-01*v1*Jinv10-1.0960297418630752e-02*v0*Jinv01)*D;
  A[7][3]=(-1.8267162364384586e-03*v1*Jinv11-8.4159426607343268e-02*Jinv00*v0-8.4159426607343268e-02*v1*Jinv10-1.8267162364384586e-03*v0*Jinv01)*D;
  A[7][4]=(-1.1873655536849981e-02*v1*Jinv11+3.5881926072898293e-02*Jinv00*v0+3.5881926072898293e-02*v1*Jinv10-1.1873655536849981e-02*v0*Jinv01)*D;
  A[7][5]=(-5.1774357329912885e-03*Jinv00*v0-5.1774357329912885e-03*v1*Jinv10)*D;
  A[7][6]=(-6.7849460210571325e-02*v1*Jinv11-1.9258808321308321e-01*Jinv00*v0-1.9258808321308321e-01*v1*Jinv10-6.7849460210571325e-02*v0*Jinv01)*D;
  A[7][7]=(0.0)*D;
  A[7][8]=(6.8501858866442195e-02*v1*Jinv11+2.5443547578964243e-01*Jinv00*v0+2.5443547578964243e-01*v1*Jinv10+6.8501858866442195e-02*v0*Jinv01)*D;
  A[7][9]=(-5.2191892469670251e-02*v1*Jinv11-1.0438378493934050e-01*Jinv00*v0-1.0438378493934050e-01*v1*Jinv10-5.2191892469670251e-02*v0*Jinv01)*D;
  A[7][10]=(1.1873655536849981e-02*v1*Jinv11+4.2536392362781249e-02*Jinv00*v0+4.2536392362781249e-02*v1*Jinv10+1.1873655536849981e-02*v0*Jinv01)*D;
  A[7][11]=(1.3569892042114265e-01*v1*Jinv11-1.0960297418630752e-02*Jinv00*v0-1.0960297418630752e-02*v1*Jinv10+1.3569892042114265e-01*v0*Jinv01)*D;
  A[7][12]=(2.5443547578964243e-01*v1*Jinv11+6.8501858866442195e-02*Jinv00*v0+6.8501858866442195e-02*v1*Jinv10+2.5443547578964243e-01*v0*Jinv01)*D;
  A[7][13]=(-9.7859798380631718e-02*v1*Jinv11-9.7859798380631718e-02*Jinv00*v0-9.7859798380631718e-02*v1*Jinv10-9.7859798380631718e-02*v0*Jinv01)*D;
  A[7][14]=(5.4018608706108709e-02*v1*Jinv11+4.0318236932820266e-02*Jinv00*v0+4.0318236932820266e-02*v1*Jinv10+5.4018608706108709e-02*v0*Jinv01)*D;
  A[7][15]=(-8.4159426607343268e-02*v1*Jinv11-1.8267162364384586e-03*Jinv00*v0-1.8267162364384586e-03*v1*Jinv10-8.4159426607343268e-02*v0*Jinv01)*D;
  A[7][16]=(-1.0438378493934050e-01*v1*Jinv11-5.2191892469670251e-02*Jinv00*v0-5.2191892469670251e-02*v1*Jinv10-1.0438378493934050e-01*v0*Jinv01)*D;
  A[7][17]=(4.0318236932820266e-02*v1*Jinv11+5.4018608706108709e-02*Jinv00*v0+5.4018608706108709e-02*v1*Jinv10+4.0318236932820266e-02*v0*Jinv01)*D;
  A[7][18]=(3.5881926072898293e-02*v1*Jinv11-1.1873655536849981e-02*Jinv00*v0-1.1873655536849981e-02*v1*Jinv10+3.5881926072898293e-02*v0*Jinv01)*D;
  A[7][19]=(4.2536392362781249e-02*v1*Jinv11+1.1873655536849981e-02*Jinv00*v0+1.1873655536849981e-02*v1*Jinv10+4.2536392362781249e-02*v0*Jinv01)*D;
  A[7][20]=(-5.1774357329912885e-03*v1*Jinv11-5.1774357329912885e-03*v0*Jinv01)*D;
  A[8][0]=(4.7246710658168994e-03*v1*Jinv11+4.7246710658168994e-03*Jinv00*v0+4.7246710658168994e-03*v1*Jinv10+4.7246710658168994e-03*v0*Jinv01)*D;
  A[8][1]=(4.9321338383838384e-02*v1*Jinv11+6.3608868947410612e-03*Jinv00*v0+6.3608868947410612e-03*v1*Jinv10+4.9321338383838384e-02*v0*Jinv01)*D;
  A[8][2]=(-1.6773169442440275e-01*v1*Jinv11-9.0031014510181182e-02*Jinv00*v0-9.0031014510181182e-02*v1*Jinv10-1.6773169442440275e-01*v0*Jinv01)*D;
  A[8][3]=(-7.7700679914221582e-02*v1*Jinv11+9.0031014510181182e-02*Jinv00*v0+9.0031014510181182e-02*v1*Jinv10-7.7700679914221582e-02*v0*Jinv01)*D;
  A[8][4]=(4.2960451489097322e-02*v1*Jinv11-6.3608868947410612e-03*Jinv00*v0-6.3608868947410612e-03*v1*Jinv10+4.2960451489097322e-02*v0*Jinv01)*D;
  A[8][5]=(-4.7246710658168994e-03*Jinv00*v0-4.7246710658168994e-03*v1*Jinv10)*D;
  A[8][6]=(4.1590414311768478e-02*v1*Jinv11+8.4550865800865807e-02*Jinv00*v0+8.4550865800865807e-02*v1*Jinv10+4.1590414311768478e-02*v0*Jinv01)*D;
  A[8][7]=(-6.8501858866442195e-02*v1*Jinv11-2.5443547578964243e-01*Jinv00*v0-2.5443547578964243e-01*v1*Jinv10-6.8501858866442195e-02*v0*Jinv01)*D;
  A[8][8]=(0.0)*D;
  A[8][9]=(1.8593361692320026e-01*v1*Jinv11+2.5443547578964243e-01*Jinv00*v0+2.5443547578964243e-01*v1*Jinv10+1.8593361692320026e-01*v0*Jinv01)*D;
  A[8][10]=(-4.2960451489097322e-02*v1*Jinv11-8.4550865800865807e-02*Jinv00*v0-8.4550865800865807e-02*v1*Jinv10-4.2960451489097322e-02*v0*Jinv01)*D;
  A[8][11]=(-2.7400743546576879e-02*v1*Jinv11+8.0832193462401794e-02*Jinv00*v0+8.0832193462401794e-02*v1*Jinv10-2.7400743546576879e-02*v0*Jinv01)*D;
  A[8][12]=(1.0275278829966331e-01*v1*Jinv11-1.0275278829966331e-01*Jinv00*v0-1.0275278829966331e-01*v1*Jinv10+1.0275278829966331e-01*v0*Jinv01)*D;
  A[8][13]=(2.0550557659932661e-01*v1*Jinv11+1.0275278829966331e-01*Jinv00*v0+1.0275278829966331e-01*v1*Jinv10+2.0550557659932661e-01*v0*Jinv01)*D;
  A[8][14]=(-1.0823293700897868e-01*v1*Jinv11-8.0832193462401794e-02*Jinv00*v0-8.0832193462401794e-02*v1*Jinv10-1.0823293700897868e-01*v0*Jinv01)*D;
  A[8][15]=(-2.8379341530383198e-02*v1*Jinv11+9.3945406445406446e-03*Jinv00*v0+9.3945406445406446e-03*v1*Jinv10-2.8379341530383198e-02*v0*Jinv01)*D;
  A[8][16]=(-9.7859798380631718e-02*v1*Jinv11-9.7859798380631718e-02*v0*Jinv01)*D;
  A[8][17]=(-3.7773882174923841e-02*v1*Jinv11-9.3945406445406446e-03*Jinv00*v0-9.3945406445406446e-03*v1*Jinv10-3.7773882174923841e-02*v0*Jinv01)*D;
  A[8][18]=(1.1743175805675805e-02*v1*Jinv11-2.5443547578964244e-03*Jinv00*v0-2.5443547578964244e-03*v1*Jinv10+1.1743175805675805e-02*v0*Jinv01)*D;
  A[8][19]=(1.4287530563572230e-02*v1*Jinv11+2.5443547578964244e-03*Jinv00*v0+2.5443547578964244e-03*v1*Jinv10+1.4287530563572230e-02*v0*Jinv01)*D;
  A[8][20]=(-2.2781761063011062e-03*v1*Jinv11-2.2781761063011062e-03*v0*Jinv01)*D;
  A[9][0]=(5.1774357329912885e-03*v1*Jinv11+5.1774357329912885e-03*Jinv00*v0+5.1774357329912885e-03*v1*Jinv10+5.1774357329912885e-03*v0*Jinv01)*D;
  A[9][1]=(-4.7755581609748278e-02*v1*Jinv11-3.5881926072898293e-02*Jinv00*v0-3.5881926072898293e-02*v1*Jinv10-4.7755581609748278e-02*v0*Jinv01)*D;
  A[9][2]=(8.2332710370904810e-02*v1*Jinv11+8.4159426607343268e-02*Jinv00*v0+8.4159426607343268e-02*v1*Jinv10+8.2332710370904810e-02*v0*Jinv01)*D;
  A[9][3]=(-1.4665921783977340e-01*v1*Jinv11-1.3569892042114265e-01*Jinv00*v0-1.3569892042114265e-01*v1*Jinv10-1.4665921783977340e-01*v0*Jinv01)*D;
  A[9][4]=(-1.2473862300251189e-01*v1*Jinv11+6.7849460210571325e-02*Jinv00*v0+6.7849460210571325e-02*v1*Jinv10-1.2473862300251189e-01*v0*Jinv01)*D;
  A[9][5]=(1.4394523943135054e-02*Jinv00*v0+1.4394523943135054e-02*v1*Jinv10)*D;
  A[9][6]=(-3.0662736825931271e-02*v1*Jinv11-4.2536392362781249e-02*Jinv00*v0-4.2536392362781249e-02*v1*Jinv10-3.0662736825931271e-02*v0*Jinv01)*D;
  A[9][7]=(5.2191892469670251e-02*v1*Jinv11+1.0438378493934050e-01*Jinv00*v0+1.0438378493934050e-01*v1*Jinv10+5.2191892469670251e-02*v0*Jinv01)*D;
  A[9][8]=(-1.8593361692320026e-01*v1*Jinv11-2.5443547578964243e-01*Jinv00*v0-2.5443547578964243e-01*v1*Jinv10-1.8593361692320026e-01*v0*Jinv01)*D;
  A[9][9]=(0.0)*D;
  A[9][10]=(1.2473862300251189e-01*v1*Jinv11+1.9258808321308321e-01*Jinv00*v0+1.9258808321308321e-01*v1*Jinv10+1.2473862300251189e-01*v0*Jinv01)*D;
  A[9][11]=(1.3700371773288440e-02*v1*Jinv11-4.0318236932820266e-02*Jinv00*v0-4.0318236932820266e-02*v1*Jinv10+1.3700371773288440e-02*v0*Jinv01)*D;
  A[9][12]=(9.7859798380631718e-02*Jinv00*v0+9.7859798380631718e-02*v1*Jinv10)*D;
  A[9][13]=(1.8593361692320026e-01*v1*Jinv11-6.8501858866442195e-02*Jinv00*v0-6.8501858866442195e-02*v1*Jinv10+1.8593361692320026e-01*v0*Jinv01)*D;
  A[9][14]=(1.4665921783977340e-01*v1*Jinv11+1.0960297418630752e-02*Jinv00*v0+1.0960297418630752e-02*v1*Jinv10+1.4665921783977340e-01*v0*Jinv01)*D;
  A[9][15]=(-1.3700371773288440e-02*v1*Jinv11-5.4018608706108709e-02*Jinv00*v0-5.4018608706108709e-02*v1*Jinv10-1.3700371773288440e-02*v0*Jinv01)*D;
  A[9][16]=(-5.2191892469670251e-02*v1*Jinv11+5.2191892469670251e-02*Jinv00*v0+5.2191892469670251e-02*v1*Jinv10-5.2191892469670251e-02*v0*Jinv01)*D;
  A[9][17]=(-8.2332710370904810e-02*v1*Jinv11+1.8267162364384586e-03*Jinv00*v0+1.8267162364384586e-03*v1*Jinv10-8.2332710370904810e-02*v0*Jinv01)*D;
  A[9][18]=(3.0662736825931271e-02*v1*Jinv11-1.1873655536849981e-02*Jinv00*v0-1.1873655536849981e-02*v1*Jinv10+3.0662736825931271e-02*v0*Jinv01)*D;
  A[9][19]=(4.7755581609748278e-02*v1*Jinv11+1.1873655536849981e-02*Jinv00*v0+1.1873655536849981e-02*v1*Jinv10+4.7755581609748278e-02*v0*Jinv01)*D;
  A[9][20]=(-5.1774357329912885e-03*v1*Jinv11-5.1774357329912885e-03*v0*Jinv01)*D;
  A[10][0]=(-4.8003493098979213e-03*v1*Jinv11-4.8003493098979213e-03*Jinv00*v0-4.8003493098979213e-03*v1*Jinv10-4.8003493098979213e-03*v0*Jinv01)*D;
  A[10][1]=(3.2835224349981297e-02*v1*Jinv11+2.7459459425605258e-02*Jinv00*v0+2.7459459425605258e-02*v1*Jinv10+3.2835224349981297e-02*v0*Jinv01)*D;
  A[10][2]=(-7.8894569454465283e-02*v1*Jinv11-5.9916292555181445e-02*Jinv00*v0-5.9916292555181445e-02*v1*Jinv10-7.8894569454465283e-02*v0*Jinv01)*D;
  A[10][3]=(1.0863742417561861e-01*v1*Jinv11+6.9911039963123298e-02*Jinv00*v0+6.9911039963123298e-02*v1*Jinv10+1.0863742417561861e-01*v0*Jinv01)*D;
  A[10][4]=(-1.3982207992624660e-01*v1*Jinv11-6.9911039963123298e-02*Jinv00*v0-6.9911039963123298e-02*v1*Jinv10-1.3982207992624660e-01*v0*Jinv01)*D;
  A[10][5]=(3.7257182439474103e-02*Jinv00*v0+3.7257182439474103e-02*v1*Jinv10)*D;
  A[10][6]=(5.3757649243760357e-03*v1*Jinv11+1.0751529848752071e-02*Jinv00*v0+1.0751529848752071e-02*v1*Jinv10+5.3757649243760357e-03*v0*Jinv01)*D;
  A[10][7]=(-1.1873655536849981e-02*v1*Jinv11-4.2536392362781249e-02*Jinv00*v0-4.2536392362781249e-02*v1*Jinv10-1.1873655536849981e-02*v0*Jinv01)*D;
  A[10][8]=(4.2960451489097322e-02*v1*Jinv11+8.4550865800865807e-02*Jinv00*v0+8.4550865800865807e-02*v1*Jinv10+4.2960451489097322e-02*v0*Jinv01)*D;
  A[10][9]=(-1.2473862300251189e-01*v1*Jinv11-1.9258808321308321e-01*Jinv00*v0-1.9258808321308321e-01*v1*Jinv10-1.2473862300251189e-01*v0*Jinv01)*D;
  A[10][10]=(1.3982207992624660e-01*v1*Jinv11+1.3982207992624660e-01*Jinv00*v0+1.3982207992624660e-01*v1*Jinv10+1.3982207992624660e-01*v0*Jinv01)*D;
  A[10][11]=(2.2899192821067820e-03*v1*Jinv11+1.3974379208754209e-02*Jinv00*v0+1.3974379208754209e-02*v1*Jinv10+2.2899192821067820e-03*v0*Jinv01)*D;
  A[10][12]=(-2.5443547578964244e-03*v1*Jinv11-1.4287530563572230e-02*Jinv00*v0-1.4287530563572230e-02*v1*Jinv10-2.5443547578964244e-03*v0*Jinv01)*D;
  A[10][13]=(4.2960451489097322e-02*v1*Jinv11+4.9321338383838384e-02*Jinv00*v0+4.9321338383838384e-02*v1*Jinv10+4.2960451489097322e-02*v0*Jinv01)*D;
  A[10][14]=(1.6101198826893270e-02*v1*Jinv11-4.9008187029020364e-02*Jinv00*v0-4.9008187029020364e-02*v1*Jinv10+1.6101198826893270e-02*v0*Jinv01)*D;
  A[10][15]=(2.2899192821067820e-03*v1*Jinv11+1.1169064988509433e-02*Jinv00*v0+1.1169064988509433e-02*v1*Jinv10+2.2899192821067820e-03*v0*Jinv01)*D;
  A[10][16]=(-1.1873655536849981e-02*v1*Jinv11-4.7755581609748278e-02*Jinv00*v0-4.7755581609748278e-02*v1*Jinv10-1.1873655536849981e-02*v0*Jinv01)*D;
  A[10][17]=(-7.0263335237293568e-03*v1*Jinv11+3.6586516621238845e-02*Jinv00*v0+3.6586516621238845e-02*v1*Jinv10-7.0263335237293568e-03*v0*Jinv01)*D;
  A[10][18]=(5.3757649243760357e-03*v1*Jinv11+3.2835224349981297e-02*Jinv00*v0+3.2835224349981297e-02*v1*Jinv10+5.3757649243760357e-03*v0*Jinv01)*D;
  A[10][19]=(-6.5435585183849076e-03*v1*Jinv11-3.2835224349981297e-02*Jinv00*v0-3.2835224349981297e-02*v1*Jinv10-6.5435585183849076e-03*v0*Jinv01)*D;
  A[10][20]=(-1.0531019103067715e-02*v1*Jinv11-1.0531019103067715e-02*v0*Jinv01)*D;
  A[11][0]=(3.3658551453690344e-02*v1*Jinv11+3.3658551453690344e-02*Jinv00*v0+3.3658551453690344e-02*v1*Jinv10+3.3658551453690344e-02*v0*Jinv01)*D;
  A[11][1]=(6.9911039963123298e-02*v1*Jinv11-3.8726384212495323e-02*Jinv00*v0-3.8726384212495323e-02*v1*Jinv10+6.9911039963123298e-02*v0*Jinv01)*D;
  A[11][2]=(-4.8016541072096628e-03*v1*Jinv11+4.8016541072096628e-03*Jinv00*v0+4.8016541072096628e-03*v1*Jinv10-4.8016541072096628e-03*v0*Jinv01)*D;
  A[11][3]=(5.4801487093153760e-04*v1*Jinv11+5.7802520910159800e-03*Jinv00*v0+5.7802520910159800e-03*v1*Jinv10+5.4801487093153760e-04*v0*Jinv01)*D;
  A[11][4]=(2.2899192821067820e-03*v1*Jinv11-8.8791457064026514e-03*Jinv00*v0-8.8791457064026514e-03*v1*Jinv10+2.2899192821067820e-03*v0*Jinv01)*D;
  A[11][5]=(3.3650722669819892e-03*Jinv00*v0+3.3650722669819892e-03*v1*Jinv10)*D;
  A[11][6]=(-6.5109385855913637e-02*v1*Jinv11+4.3528038319704990e-02*Jinv00*v0+4.3528038319704990e-02*v1*Jinv10-6.5109385855913637e-02*v0*Jinv01)*D;
  A[11][7]=(-1.3569892042114265e-01*v1*Jinv11+1.0960297418630752e-02*Jinv00*v0+1.0960297418630752e-02*v1*Jinv10-1.3569892042114265e-01*v0*Jinv01)*D;
  A[11][8]=(2.7400743546576879e-02*v1*Jinv11-8.0832193462401794e-02*Jinv00*v0-8.0832193462401794e-02*v1*Jinv10+2.7400743546576879e-02*v0*Jinv01)*D;
  A[11][9]=(-1.3700371773288440e-02*v1*Jinv11+4.0318236932820266e-02*Jinv00*v0+4.0318236932820266e-02*v1*Jinv10-1.3700371773288440e-02*v0*Jinv01)*D;
  A[11][10]=(-2.2899192821067820e-03*v1*Jinv11-1.3974379208754209e-02*Jinv00*v0-1.3974379208754209e-02*v1*Jinv10-2.2899192821067820e-03*v0*Jinv01)*D;
  A[11][11]=(-1.5626252605419272e-01*Jinv00*v0-1.5626252605419272e-01*v1*Jinv10)*D;
  A[11][12]=(9.0031014510181182e-02*v1*Jinv11+1.6773169442440275e-01*Jinv00*v0+1.6773169442440275e-01*v1*Jinv10+9.0031014510181182e-02*v0*Jinv01)*D;
  A[11][13]=(-2.8379341530383198e-02*v1*Jinv11-3.7773882174923841e-02*Jinv00*v0-3.7773882174923841e-02*v1*Jinv10-2.8379341530383198e-02*v0*Jinv01)*D;
  A[11][14]=(1.3152356902356903e-02*v1*Jinv11+2.6304713804713806e-02*Jinv00*v0+2.6304713804713806e-02*v1*Jinv10+1.3152356902356903e-02*v0*Jinv01)*D;
  A[11][15]=(5.1800453276147719e-02*v1*Jinv11+7.7100473150820373e-02*Jinv00*v0+7.7100473150820373e-02*v1*Jinv10+5.1800453276147719e-02*v0*Jinv01)*D;
  A[11][16]=(-8.4159426607343268e-02*v1*Jinv11-8.2332710370904810e-02*Jinv00*v0-8.2332710370904810e-02*v1*Jinv10-8.4159426607343268e-02*v0*Jinv01)*D;
  A[11][17]=(5.7802520910159800e-03*v1*Jinv11+5.2322372200844427e-03*Jinv00*v0+5.2322372200844427e-03*v1*Jinv10+5.7802520910159800e-03*v0*Jinv01)*D;
  A[11][18]=(-4.3612850144968199e-02*v1*Jinv11-7.8894569454465283e-02*Jinv00*v0-7.8894569454465283e-02*v1*Jinv10-4.3612850144968199e-02*v0*Jinv01)*D;
  A[11][19]=(5.9916292555181445e-02*v1*Jinv11+7.8894569454465283e-02*Jinv00*v0+7.8894569454465283e-02*v1*Jinv10+5.9916292555181445e-02*v0*Jinv01)*D;
  A[11][20]=(2.3263231271043770e-02*v1*Jinv11+2.3263231271043770e-02*v0*Jinv01)*D;
  A[12][0]=(4.7246710658168994e-03*v1*Jinv11+4.7246710658168994e-03*Jinv00*v0+4.7246710658168994e-03*v1*Jinv10+4.7246710658168994e-03*v0*Jinv01)*D;
  A[12][1]=(8.4550865800865807e-02*v1*Jinv11+4.1590414311768478e-02*Jinv00*v0+4.1590414311768478e-02*v1*Jinv10+8.4550865800865807e-02*v0*Jinv01)*D;
  A[12][2]=(8.0832193462401794e-02*v1*Jinv11-2.7400743546576879e-02*Jinv00*v0-2.7400743546576879e-02*v1*Jinv10+8.0832193462401794e-02*v0*Jinv01)*D;
  A[12][3]=(9.3945406445406446e-03*v1*Jinv11-2.8379341530383198e-02*Jinv00*v0-2.8379341530383198e-02*v1*Jinv10+9.3945406445406446e-03*v0*Jinv01)*D;
  A[12][4]=(-2.5443547578964244e-03*v1*Jinv11+1.1743175805675805e-02*Jinv00*v0+1.1743175805675805e-02*v1*Jinv10-2.5443547578964244e-03*v0*Jinv01)*D;
  A[12][5]=(-2.2781761063011062e-03*Jinv00*v0-2.2781761063011062e-03*v1*Jinv10)*D;
  A[12][6]=(6.3608868947410612e-03*v1*Jinv11+4.9321338383838384e-02*Jinv00*v0+4.9321338383838384e-02*v1*Jinv10+6.3608868947410612e-03*v0*Jinv01)*D;
  A[12][7]=(-2.5443547578964243e-01*v1*Jinv11-6.8501858866442195e-02*Jinv00*v0-6.8501858866442195e-02*v1*Jinv10-2.5443547578964243e-01*v0*Jinv01)*D;
  A[12][8]=(-1.0275278829966331e-01*v1*Jinv11+1.0275278829966331e-01*Jinv00*v0+1.0275278829966331e-01*v1*Jinv10-1.0275278829966331e-01*v0*Jinv01)*D;
  A[12][9]=(-9.7859798380631718e-02*Jinv00*v0-9.7859798380631718e-02*v1*Jinv10)*D;
  A[12][10]=(2.5443547578964244e-03*v1*Jinv11+1.4287530563572230e-02*Jinv00*v0+1.4287530563572230e-02*v1*Jinv10+2.5443547578964244e-03*v0*Jinv01)*D;
  A[12][11]=(-9.0031014510181182e-02*v1*Jinv11-1.6773169442440275e-01*Jinv00*v0-1.6773169442440275e-01*v1*Jinv10-9.0031014510181182e-02*v0*Jinv01)*D;
  A[12][12]=(0.0)*D;
  A[12][13]=(1.0275278829966331e-01*v1*Jinv11+2.0550557659932661e-01*Jinv00*v0+2.0550557659932661e-01*v1*Jinv10+1.0275278829966331e-01*v0*Jinv01)*D;
  A[12][14]=(-9.3945406445406446e-03*v1*Jinv11-3.7773882174923841e-02*Jinv00*v0-3.7773882174923841e-02*v1*Jinv10-9.3945406445406446e-03*v0*Jinv01)*D;
  A[12][15]=(9.0031014510181182e-02*v1*Jinv11-7.7700679914221582e-02*Jinv00*v0-7.7700679914221582e-02*v1*Jinv10+9.0031014510181182e-02*v0*Jinv01)*D;
  A[12][16]=(2.5443547578964243e-01*v1*Jinv11+1.8593361692320026e-01*Jinv00*v0+1.8593361692320026e-01*v1*Jinv10+2.5443547578964243e-01*v0*Jinv01)*D;
  A[12][17]=(-8.0832193462401794e-02*v1*Jinv11-1.0823293700897868e-01*Jinv00*v0-1.0823293700897868e-01*v1*Jinv10-8.0832193462401794e-02*v0*Jinv01)*D;
  A[12][18]=(-6.3608868947410612e-03*v1*Jinv11+4.2960451489097322e-02*Jinv00*v0+4.2960451489097322e-02*v1*Jinv10-6.3608868947410612e-03*v0*Jinv01)*D;
  A[12][19]=(-8.4550865800865807e-02*v1*Jinv11-4.2960451489097322e-02*Jinv00*v0-4.2960451489097322e-02*v1*Jinv10-8.4550865800865807e-02*v0*Jinv01)*D;
  A[12][20]=(-4.7246710658168994e-03*v1*Jinv11-4.7246710658168994e-03*v0*Jinv01)*D;
  A[13][0]=(2.2781761063011062e-03*v1*Jinv11+2.2781761063011062e-03*Jinv00*v0+2.2781761063011062e-03*v1*Jinv10+2.2781761063011062e-03*v0*Jinv01)*D;
  A[13][1]=(-1.4287530563572230e-02*v1*Jinv11-1.1743175805675805e-02*Jinv00*v0-1.1743175805675805e-02*v1*Jinv10-1.4287530563572230e-02*v0*Jinv01)*D;
  A[13][2]=(3.7773882174923841e-02*v1*Jinv11+2.8379341530383198e-02*Jinv00*v0+2.8379341530383198e-02*v1*Jinv10+3.7773882174923841e-02*v0*Jinv01)*D;
  A[13][3]=(1.0823293700897868e-01*v1*Jinv11+2.7400743546576879e-02*Jinv00*v0+2.7400743546576879e-02*v1*Jinv10+1.0823293700897868e-01*v0*Jinv01)*D;
  A[13][4]=(4.2960451489097322e-02*v1*Jinv11-4.1590414311768478e-02*Jinv00*v0-4.1590414311768478e-02*v1*Jinv10+4.2960451489097322e-02*v0*Jinv01)*D;
  A[13][5]=(-4.7246710658168994e-03*Jinv00*v0-4.7246710658168994e-03*v1*Jinv10)*D;
  A[13][6]=(-1.1743175805675805e-02*v1*Jinv11-1.4287530563572230e-02*Jinv00*v0-1.4287530563572230e-02*v1*Jinv10-1.1743175805675805e-02*v0*Jinv01)*D;
  A[13][7]=(9.7859798380631718e-02*v1*Jinv11+9.7859798380631718e-02*Jinv00*v0+9.7859798380631718e-02*v1*Jinv10+9.7859798380631718e-02*v0*Jinv01)*D;
  A[13][8]=(-2.0550557659932661e-01*v1*Jinv11-1.0275278829966331e-01*Jinv00*v0-1.0275278829966331e-01*v1*Jinv10-2.0550557659932661e-01*v0*Jinv01)*D;
  A[13][9]=(-1.8593361692320026e-01*v1*Jinv11+6.8501858866442195e-02*Jinv00*v0+6.8501858866442195e-02*v1*Jinv10-1.8593361692320026e-01*v0*Jinv01)*D;
  A[13][10]=(-4.2960451489097322e-02*v1*Jinv11-4.9321338383838384e-02*Jinv00*v0-4.9321338383838384e-02*v1*Jinv10-4.2960451489097322e-02*v0*Jinv01)*D;
  A[13][11]=(2.8379341530383198e-02*v1*Jinv11+3.7773882174923841e-02*Jinv00*v0+3.7773882174923841e-02*v1*Jinv10+2.8379341530383198e-02*v0*Jinv01)*D;
  A[13][12]=(-1.0275278829966331e-01*v1*Jinv11-2.0550557659932661e-01*Jinv00*v0-2.0550557659932661e-01*v1*Jinv10-1.0275278829966331e-01*v0*Jinv01)*D;
  A[13][13]=(0.0)*D;
  A[13][14]=(7.7700679914221582e-02*v1*Jinv11+1.6773169442440275e-01*Jinv00*v0+1.6773169442440275e-01*v1*Jinv10+7.7700679914221582e-02*v0*Jinv01)*D;
  A[13][15]=(2.7400743546576879e-02*v1*Jinv11+1.0823293700897868e-01*Jinv00*v0+1.0823293700897868e-01*v1*Jinv10+2.7400743546576879e-02*v0*Jinv01)*D;
  A[13][16]=(6.8501858866442195e-02*v1*Jinv11-1.8593361692320026e-01*Jinv00*v0-1.8593361692320026e-01*v1*Jinv10+6.8501858866442195e-02*v0*Jinv01)*D;
  A[13][17]=(1.6773169442440275e-01*v1*Jinv11+7.7700679914221582e-02*Jinv00*v0+7.7700679914221582e-02*v1*Jinv10+1.6773169442440275e-01*v0*Jinv01)*D;
  A[13][18]=(-4.1590414311768478e-02*v1*Jinv11+4.2960451489097322e-02*Jinv00*v0+4.2960451489097322e-02*v1*Jinv10-4.1590414311768478e-02*v0*Jinv01)*D;
  A[13][19]=(-4.9321338383838384e-02*v1*Jinv11-4.2960451489097322e-02*Jinv00*v0-4.2960451489097322e-02*v1*Jinv10-4.9321338383838384e-02*v0*Jinv01)*D;
  A[13][20]=(-4.7246710658168994e-03*v1*Jinv11-4.7246710658168994e-03*v0*Jinv01)*D;
  A[14][0]=(-3.3650722669819892e-03*v1*Jinv11-3.3650722669819892e-03*Jinv00*v0-3.3650722669819892e-03*v1*Jinv10-3.3650722669819892e-03*v0*Jinv01)*D;
  A[14][1]=(1.1169064988509433e-02*v1*Jinv11+8.8791457064026514e-03*Jinv00*v0+8.8791457064026514e-03*v1*Jinv10+1.1169064988509433e-02*v0*Jinv01)*D;
  A[14][2]=(-5.2322372200844427e-03*v1*Jinv11-5.7802520910159800e-03*Jinv00*v0-5.7802520910159800e-03*v1*Jinv10-5.2322372200844427e-03*v0*Jinv01)*D;
  A[14][3]=(-9.6033082144193257e-03*v1*Jinv11-4.8016541072096628e-03*Jinv00*v0-4.8016541072096628e-03*v1*Jinv10-9.6033082144193257e-03*v0*Jinv01)*D;
  A[14][4]=(1.0863742417561861e-01*v1*Jinv11+3.8726384212495323e-02*Jinv00*v0+3.8726384212495323e-02*v1*Jinv10+1.0863742417561861e-01*v0*Jinv01)*D;
  A[14][5]=(-3.3658551453690344e-02*Jinv00*v0-3.3658551453690344e-02*v1*Jinv10)*D;
  A[14][6]=(1.1684459926647427e-02*v1*Jinv11+1.3974379208754209e-02*Jinv00*v0+1.3974379208754209e-02*v1*Jinv10+1.1684459926647427e-02*v0*Jinv01)*D;
  A[14][7]=(-5.4018608706108709e-02*v1*Jinv11-4.0318236932820266e-02*Jinv00*v0-4.0318236932820266e-02*v1*Jinv10-5.4018608706108709e-02*v0*Jinv01)*D;
  A[14][8]=(1.0823293700897868e-01*v1*Jinv11+8.0832193462401794e-02*Jinv00*v0+8.0832193462401794e-02*v1*Jinv10+1.0823293700897868e-01*v0*Jinv01)*D;
  A[14][9]=(-1.4665921783977340e-01*v1*Jinv11-1.0960297418630752e-02*Jinv00*v0-1.0960297418630752e-02*v1*Jinv10-1.4665921783977340e-01*v0*Jinv01)*D;
  A[14][10]=(-1.0863742417561861e-01*v1*Jinv11-4.3528038319704990e-02*Jinv00*v0-4.3528038319704990e-02*v1*Jinv10-1.0863742417561861e-01*v0*Jinv01)*D;
  A[14][11]=(-1.3152356902356903e-02*v1*Jinv11-2.6304713804713806e-02*Jinv00*v0-2.6304713804713806e-02*v1*Jinv10-1.3152356902356903e-02*v0*Jinv01)*D;
  A[14][12]=(9.3945406445406446e-03*v1*Jinv11+3.7773882174923841e-02*Jinv00*v0+3.7773882174923841e-02*v1*Jinv10+9.3945406445406446e-03*v0*Jinv01)*D;
  A[14][13]=(-7.7700679914221582e-02*v1*Jinv11-1.6773169442440275e-01*Jinv00*v0-1.6773169442440275e-01*v1*Jinv10-7.7700679914221582e-02*v0*Jinv01)*D;
  A[14][14]=(1.5626252605419272e-01*v1*Jinv11+1.5626252605419272e-01*Jinv00*v0+1.5626252605419272e-01*v1*Jinv10+1.5626252605419272e-01*v0*Jinv01)*D;
  A[14][15]=(5.4801487093153760e-04*v1*Jinv11-5.2322372200844427e-03*Jinv00*v0-5.2322372200844427e-03*v1*Jinv10+5.4801487093153760e-04*v0*Jinv01)*D;
  A[14][16]=(-1.8267162364384586e-03*v1*Jinv11+8.2332710370904810e-02*Jinv00*v0+8.2332710370904810e-02*v1*Jinv10-1.8267162364384586e-03*v0*Jinv01)*D;
  A[14][17]=(-2.5300019874672654e-02*v1*Jinv11-7.7100473150820373e-02*Jinv00*v0-7.7100473150820373e-02*v1*Jinv10-2.5300019874672654e-02*v0*Jinv01)*D;
  A[14][18]=(-1.8978276899283844e-02*v1*Jinv11-7.8894569454465283e-02*Jinv00*v0-7.8894569454465283e-02*v1*Jinv10-1.8978276899283844e-02*v0*Jinv01)*D;
  A[14][19]=(3.5281719309497091e-02*v1*Jinv11+7.8894569454465283e-02*Jinv00*v0+7.8894569454465283e-02*v1*Jinv10+3.5281719309497091e-02*v0*Jinv01)*D;
  A[14][20]=(2.3263231271043770e-02*v1*Jinv11+2.3263231271043770e-02*v0*Jinv01)*D;
  A[15][0]=(-2.3263231271043770e-02*v1*Jinv11-2.3263231271043770e-02*Jinv00*v0-2.3263231271043770e-02*v1*Jinv10-2.3263231271043770e-02*v0*Jinv01)*D;
  A[15][1]=(-5.9916292555181445e-02*v1*Jinv11+1.8978276899283844e-02*Jinv00*v0+1.8978276899283844e-02*v1*Jinv10-5.9916292555181445e-02*v0*Jinv01)*D;
  A[15][2]=(-5.7802520910159800e-03*v1*Jinv11-5.4801487093153760e-04*Jinv00*v0-5.4801487093153760e-04*v1*Jinv10-5.7802520910159800e-03*v0*Jinv01)*D;
  A[15][3]=(-1.3152356902356903e-02*v1*Jinv11+1.3152356902356903e-02*Jinv00*v0+1.3152356902356903e-02*v1*Jinv10-1.3152356902356903e-02*v0*Jinv01)*D;
  A[15][4]=(2.2899192821067820e-03*v1*Jinv11-1.1684459926647427e-02*Jinv00*v0-1.1684459926647427e-02*v1*Jinv10+2.2899192821067820e-03*v0*Jinv01)*D;
  A[15][5]=(3.3650722669819892e-03*Jinv00*v0+3.3650722669819892e-03*v1*Jinv10)*D;
  A[15][6]=(4.3612850144968199e-02*v1*Jinv11-3.5281719309497091e-02*Jinv00*v0-3.5281719309497091e-02*v1*Jinv10+4.3612850144968199e-02*v0*Jinv01)*D;
  A[15][7]=(8.4159426607343268e-02*v1*Jinv11+1.8267162364384586e-03*Jinv00*v0+1.8267162364384586e-03*v1*Jinv10+8.4159426607343268e-02*v0*Jinv01)*D;
  A[15][8]=(2.8379341530383198e-02*v1*Jinv11-9.3945406445406446e-03*Jinv00*v0-9.3945406445406446e-03*v1*Jinv10+2.8379341530383198e-02*v0*Jinv01)*D;
  A[15][9]=(1.3700371773288440e-02*v1*Jinv11+5.4018608706108709e-02*Jinv00*v0+5.4018608706108709e-02*v1*Jinv10+1.3700371773288440e-02*v0*Jinv01)*D;
  A[15][10]=(-2.2899192821067820e-03*v1*Jinv11-1.1169064988509433e-02*Jinv00*v0-1.1169064988509433e-02*v1*Jinv10-2.2899192821067820e-03*v0*Jinv01)*D;
  A[15][11]=(-5.1800453276147719e-02*v1*Jinv11+2.5300019874672654e-02*Jinv00*v0+2.5300019874672654e-02*v1*Jinv10-5.1800453276147719e-02*v0*Jinv01)*D;
  A[15][12]=(-9.0031014510181182e-02*v1*Jinv11+7.7700679914221582e-02*Jinv00*v0+7.7700679914221582e-02*v1*Jinv10-9.0031014510181182e-02*v0*Jinv01)*D;
  A[15][13]=(-2.7400743546576879e-02*v1*Jinv11-1.0823293700897868e-01*Jinv00*v0-1.0823293700897868e-01*v1*Jinv10-2.7400743546576879e-02*v0*Jinv01)*D;
  A[15][14]=(-5.4801487093153760e-04*v1*Jinv11+5.2322372200844427e-03*Jinv00*v0+5.2322372200844427e-03*v1*Jinv10-5.4801487093153760e-04*v0*Jinv01)*D;
  A[15][15]=(-1.5626252605419272e-01*Jinv00*v0-1.5626252605419272e-01*v1*Jinv10)*D;
  A[15][16]=(1.3569892042114265e-01*v1*Jinv11+1.4665921783977340e-01*Jinv00*v0+1.4665921783977340e-01*v1*Jinv10+1.3569892042114265e-01*v0*Jinv01)*D;
  A[15][17]=(4.8016541072096628e-03*v1*Jinv11+9.6033082144193257e-03*Jinv00*v0+9.6033082144193257e-03*v1*Jinv10+4.8016541072096628e-03*v0*Jinv01)*D;
  A[15][18]=(6.5109385855913637e-02*v1*Jinv11+1.0863742417561861e-01*Jinv00*v0+1.0863742417561861e-01*v1*Jinv10+6.5109385855913637e-02*v0*Jinv01)*D;
  A[15][19]=(-6.9911039963123298e-02*v1*Jinv11-1.0863742417561861e-01*Jinv00*v0-1.0863742417561861e-01*v1*Jinv10-6.9911039963123298e-02*v0*Jinv01)*D;
  A[15][20]=(-3.3658551453690344e-02*v1*Jinv11-3.3658551453690344e-02*v0*Jinv01)*D;
  A[16][0]=(5.1774357329912885e-03*v1*Jinv11+5.1774357329912885e-03*Jinv00*v0+5.1774357329912885e-03*v1*Jinv10+5.1774357329912885e-03*v0*Jinv01)*D;
  A[16][1]=(-4.2536392362781249e-02*v1*Jinv11-3.0662736825931271e-02*Jinv00*v0-3.0662736825931271e-02*v1*Jinv10-4.2536392362781249e-02*v0*Jinv01)*D;
  A[16][2]=(-4.0318236932820266e-02*v1*Jinv11+1.3700371773288440e-02*Jinv00*v0+1.3700371773288440e-02*v1*Jinv10-4.0318236932820266e-02*v0*Jinv01)*D;
  A[16][3]=(-5.4018608706108709e-02*v1*Jinv11-1.3700371773288440e-02*Jinv00*v0-1.3700371773288440e-02*v1*Jinv10-5.4018608706108709e-02*v0*Jinv01)*D;
  A[16][4]=(-1.1873655536849981e-02*v1*Jinv11+3.0662736825931271e-02*Jinv00*v0+3.0662736825931271e-02*v1*Jinv10-1.1873655536849981e-02*v0*Jinv01)*D;
  A[16][5]=(-5.1774357329912885e-03*Jinv00*v0-5.1774357329912885e-03*v1*Jinv10)*D;
  A[16][6]=(-3.5881926072898293e-02*v1*Jinv11-4.7755581609748278e-02*Jinv00*v0-4.7755581609748278e-02*v1*Jinv10-3.5881926072898293e-02*v0*Jinv01)*D;
  A[16][7]=(1.0438378493934050e-01*v1*Jinv11+5.2191892469670251e-02*Jinv00*v0+5.2191892469670251e-02*v1*Jinv10+1.0438378493934050e-01*v0*Jinv01)*D;
  A[16][8]=(9.7859798380631718e-02*v1*Jinv11+9.7859798380631718e-02*v0*Jinv01)*D;
  A[16][9]=(5.2191892469670251e-02*v1*Jinv11-5.2191892469670251e-02*Jinv00*v0-5.2191892469670251e-02*v1*Jinv10+5.2191892469670251e-02*v0*Jinv01)*D;
  A[16][10]=(1.1873655536849981e-02*v1*Jinv11+4.7755581609748278e-02*Jinv00*v0+4.7755581609748278e-02*v1*Jinv10+1.1873655536849981e-02*v0*Jinv01)*D;
  A[16][11]=(8.4159426607343268e-02*v1*Jinv11+8.2332710370904810e-02*Jinv00*v0+8.2332710370904810e-02*v1*Jinv10+8.4159426607343268e-02*v0*Jinv01)*D;
  A[16][12]=(-2.5443547578964243e-01*v1*Jinv11-1.8593361692320026e-01*Jinv00*v0-1.8593361692320026e-01*v1*Jinv10-2.5443547578964243e-01*v0*Jinv01)*D;
  A[16][13]=(-6.8501858866442195e-02*v1*Jinv11+1.8593361692320026e-01*Jinv00*v0+1.8593361692320026e-01*v1*Jinv10-6.8501858866442195e-02*v0*Jinv01)*D;
  A[16][14]=(1.8267162364384586e-03*v1*Jinv11-8.2332710370904810e-02*Jinv00*v0-8.2332710370904810e-02*v1*Jinv10+1.8267162364384586e-03*v0*Jinv01)*D;
  A[16][15]=(-1.3569892042114265e-01*v1*Jinv11-1.4665921783977340e-01*Jinv00*v0-1.4665921783977340e-01*v1*Jinv10-1.3569892042114265e-01*v0*Jinv01)*D;
  A[16][16]=(0.0)*D;
  A[16][17]=(1.0960297418630752e-02*v1*Jinv11+1.4665921783977340e-01*Jinv00*v0+1.4665921783977340e-01*v1*Jinv10+1.0960297418630752e-02*v0*Jinv01)*D;
  A[16][18]=(6.7849460210571325e-02*v1*Jinv11-1.2473862300251189e-01*Jinv00*v0-1.2473862300251189e-01*v1*Jinv10+6.7849460210571325e-02*v0*Jinv01)*D;
  A[16][19]=(1.9258808321308321e-01*v1*Jinv11+1.2473862300251189e-01*Jinv00*v0+1.2473862300251189e-01*v1*Jinv10+1.9258808321308321e-01*v0*Jinv01)*D;
  A[16][20]=(1.4394523943135054e-02*v1*Jinv11+1.4394523943135054e-02*v0*Jinv01)*D;
  A[17][0]=(-3.3650722669819892e-03*v1*Jinv11-3.3650722669819892e-03*Jinv00*v0-3.3650722669819892e-03*v1*Jinv10-3.3650722669819892e-03*v0*Jinv01)*D;
  A[17][1]=(1.3974379208754209e-02*v1*Jinv11+1.1684459926647427e-02*Jinv00*v0+1.1684459926647427e-02*v1*Jinv10+1.3974379208754209e-02*v0*Jinv01)*D;
  A[17][2]=(-2.6304713804713806e-02*v1*Jinv11-1.3152356902356903e-02*Jinv00*v0-1.3152356902356903e-02*v1*Jinv10-2.6304713804713806e-02*v0*Jinv01)*D;
  A[17][3]=(-5.2322372200844427e-03*v1*Jinv11+5.4801487093153760e-04*Jinv00*v0+5.4801487093153760e-04*v1*Jinv10-5.2322372200844427e-03*v0*Jinv01)*D;
  A[17][4]=(-7.8894569454465283e-02*v1*Jinv11-1.8978276899283844e-02*Jinv00*v0-1.8978276899283844e-02*v1*Jinv10-7.8894569454465283e-02*v0*Jinv01)*D;
  A[17][5]=(2.3263231271043770e-02*Jinv00*v0+2.3263231271043770e-02*v1*Jinv10)*D;
  A[17][6]=(8.8791457064026514e-03*v1*Jinv11+1.1169064988509433e-02*Jinv00*v0+1.1169064988509433e-02*v1*Jinv10+8.8791457064026514e-03*v0*Jinv01)*D;
  A[17][7]=(-4.0318236932820266e-02*v1*Jinv11-5.4018608706108709e-02*Jinv00*v0-5.4018608706108709e-02*v1*Jinv10-4.0318236932820266e-02*v0*Jinv01)*D;
  A[17][8]=(3.7773882174923841e-02*v1*Jinv11+9.3945406445406446e-03*Jinv00*v0+9.3945406445406446e-03*v1*Jinv10+3.7773882174923841e-02*v0*Jinv01)*D;
  A[17][9]=(8.2332710370904810e-02*v1*Jinv11-1.8267162364384586e-03*Jinv00*v0-1.8267162364384586e-03*v1*Jinv10+8.2332710370904810e-02*v0*Jinv01)*D;
  A[17][10]=(7.8894569454465283e-02*v1*Jinv11+3.5281719309497091e-02*Jinv00*v0+3.5281719309497091e-02*v1*Jinv10+7.8894569454465283e-02*v0*Jinv01)*D;
  A[17][11]=(-5.7802520910159800e-03*v1*Jinv11-5.2322372200844427e-03*Jinv00*v0-5.2322372200844427e-03*v1*Jinv10-5.7802520910159800e-03*v0*Jinv01)*D;
  A[17][12]=(8.0832193462401794e-02*v1*Jinv11+1.0823293700897868e-01*Jinv00*v0+1.0823293700897868e-01*v1*Jinv10+8.0832193462401794e-02*v0*Jinv01)*D;
  A[17][13]=(-1.6773169442440275e-01*v1*Jinv11-7.7700679914221582e-02*Jinv00*v0-7.7700679914221582e-02*v1*Jinv10-1.6773169442440275e-01*v0*Jinv01)*D;
  A[17][14]=(-7.7100473150820373e-02*v1*Jinv11-2.5300019874672654e-02*Jinv00*v0-2.5300019874672654e-02*v1*Jinv10-7.7100473150820373e-02*v0*Jinv01)*D;
  A[17][15]=(-4.8016541072096628e-03*v1*Jinv11-9.6033082144193257e-03*Jinv00*v0-9.6033082144193257e-03*v1*Jinv10-4.8016541072096628e-03*v0*Jinv01)*D;
  A[17][16]=(-1.0960297418630752e-02*v1*Jinv11-1.4665921783977340e-01*Jinv00*v0-1.4665921783977340e-01*v1*Jinv10-1.0960297418630752e-02*v0*Jinv01)*D;
  A[17][17]=(1.5626252605419272e-01*v1*Jinv11+1.5626252605419272e-01*Jinv00*v0+1.5626252605419272e-01*v1*Jinv10+1.5626252605419272e-01*v0*Jinv01)*D;
  A[17][18]=(3.8726384212495323e-02*v1*Jinv11+1.0863742417561861e-01*Jinv00*v0+1.0863742417561861e-01*v1*Jinv10+3.8726384212495323e-02*v0*Jinv01)*D;
  A[17][19]=(-4.3528038319704990e-02*v1*Jinv11-1.0863742417561861e-01*Jinv00*v0-1.0863742417561861e-01*v1*Jinv10-4.3528038319704990e-02*v0*Jinv01)*D;
  A[17][20]=(-3.3658551453690344e-02*v1*Jinv11-3.3658551453690344e-02*v0*Jinv01)*D;
  A[18][0]=(1.0531019103067715e-02*v1*Jinv11+1.0531019103067715e-02*Jinv00*v0+1.0531019103067715e-02*v1*Jinv10+1.0531019103067715e-02*v0*Jinv01)*D;
  A[18][1]=(2.7459459425605258e-02*v1*Jinv11-5.3757649243760357e-03*Jinv00*v0-5.3757649243760357e-03*v1*Jinv10+2.7459459425605258e-02*v0*Jinv01)*D;
  A[18][2]=(8.8791457064026514e-03*v1*Jinv11-2.2899192821067820e-03*Jinv00*v0-2.2899192821067820e-03*v1*Jinv10+8.8791457064026514e-03*v0*Jinv01)*D;
  A[18][3]=(1.1684459926647427e-02*v1*Jinv11-2.2899192821067820e-03*Jinv00*v0-2.2899192821067820e-03*v1*Jinv10+1.1684459926647427e-02*v0*Jinv01)*D;
  A[18][4]=(5.3757649243760357e-03*v1*Jinv11-5.3757649243760357e-03*Jinv00*v0-5.3757649243760357e-03*v1*Jinv10+5.3757649243760357e-03*v0*Jinv01)*D;
  A[18][5]=(4.8003493098979213e-03*Jinv00*v0+4.8003493098979213e-03*v1*Jinv10)*D;
  A[18][6]=(-2.6291665831596388e-02*v1*Jinv11+6.5435585183849076e-03*Jinv00*v0+6.5435585183849076e-03*v1*Jinv10-2.6291665831596388e-02*v0*Jinv01)*D;
  A[18][7]=(-3.5881926072898293e-02*v1*Jinv11+1.1873655536849981e-02*Jinv00*v0+1.1873655536849981e-02*v1*Jinv10-3.5881926072898293e-02*v0*Jinv01)*D;
  A[18][8]=(-1.1743175805675805e-02*v1*Jinv11+2.5443547578964244e-03*Jinv00*v0+2.5443547578964244e-03*v1*Jinv10-1.1743175805675805e-02*v0*Jinv01)*D;
  A[18][9]=(-3.0662736825931271e-02*v1*Jinv11+1.1873655536849981e-02*Jinv00*v0+1.1873655536849981e-02*v1*Jinv10-3.0662736825931271e-02*v0*Jinv01)*D;
  A[18][10]=(-5.3757649243760357e-03*v1*Jinv11-3.2835224349981297e-02*Jinv00*v0-3.2835224349981297e-02*v1*Jinv10-5.3757649243760357e-03*v0*Jinv01)*D;
  A[18][11]=(4.3612850144968199e-02*v1*Jinv11+7.0263335237293568e-03*Jinv00*v0+7.0263335237293568e-03*v1*Jinv10+4.3612850144968199e-02*v0*Jinv01)*D;
  A[18][12]=(6.3608868947410612e-03*v1*Jinv11-4.2960451489097322e-02*Jinv00*v0-4.2960451489097322e-02*v1*Jinv10+6.3608868947410612e-03*v0*Jinv01)*D;
  A[18][13]=(4.1590414311768478e-02*v1*Jinv11-4.2960451489097322e-02*Jinv00*v0-4.2960451489097322e-02*v1*Jinv10+4.1590414311768478e-02*v0*Jinv01)*D;
  A[18][14]=(1.8978276899283844e-02*v1*Jinv11+7.8894569454465283e-02*Jinv00*v0+7.8894569454465283e-02*v1*Jinv10+1.8978276899283844e-02*v0*Jinv01)*D;
  A[18][15]=(-6.5109385855913637e-02*v1*Jinv11-1.6101198826893270e-02*Jinv00*v0-1.6101198826893270e-02*v1*Jinv10-6.5109385855913637e-02*v0*Jinv01)*D;
  A[18][16]=(-6.7849460210571325e-02*v1*Jinv11+1.2473862300251189e-01*Jinv00*v0+1.2473862300251189e-01*v1*Jinv10-6.7849460210571325e-02*v0*Jinv01)*D;
  A[18][17]=(-3.8726384212495323e-02*v1*Jinv11-1.0863742417561861e-01*Jinv00*v0-1.0863742417561861e-01*v1*Jinv10-3.8726384212495323e-02*v0*Jinv01)*D;
  A[18][18]=(-1.3982207992624660e-01*Jinv00*v0-1.3982207992624660e-01*v1*Jinv10)*D;
  A[18][19]=(6.9911039963123298e-02*v1*Jinv11+1.3982207992624660e-01*Jinv00*v0+1.3982207992624660e-01*v1*Jinv10+6.9911039963123298e-02*v0*Jinv01)*D;
  A[18][20]=(3.7257182439474103e-02*v1*Jinv11+3.7257182439474103e-02*v0*Jinv01)*D;
  A[19][0]=(-4.8003493098979213e-03*v1*Jinv11-4.8003493098979213e-03*Jinv00*v0-4.8003493098979213e-03*v1*Jinv10-4.8003493098979213e-03*v0*Jinv01)*D;
  A[19][1]=(1.0751529848752071e-02*v1*Jinv11+5.3757649243760357e-03*Jinv00*v0+5.3757649243760357e-03*v1*Jinv10+1.0751529848752071e-02*v0*Jinv01)*D;
  A[19][2]=(1.3974379208754209e-02*v1*Jinv11+2.2899192821067820e-03*Jinv00*v0+2.2899192821067820e-03*v1*Jinv10+1.3974379208754209e-02*v0*Jinv01)*D;
  A[19][3]=(1.1169064988509433e-02*v1*Jinv11+2.2899192821067820e-03*Jinv00*v0+2.2899192821067820e-03*v1*Jinv10+1.1169064988509433e-02*v0*Jinv01)*D;
  A[19][4]=(3.2835224349981297e-02*v1*Jinv11+5.3757649243760357e-03*Jinv00*v0+5.3757649243760357e-03*v1*Jinv10+3.2835224349981297e-02*v0*Jinv01)*D;
  A[19][5]=(-1.0531019103067715e-02*Jinv00*v0-1.0531019103067715e-02*v1*Jinv10)*D;
  A[19][6]=(2.7459459425605258e-02*v1*Jinv11+3.2835224349981297e-02*Jinv00*v0+3.2835224349981297e-02*v1*Jinv10+2.7459459425605258e-02*v0*Jinv01)*D;
  A[19][7]=(-4.2536392362781249e-02*v1*Jinv11-1.1873655536849981e-02*Jinv00*v0-1.1873655536849981e-02*v1*Jinv10-4.2536392362781249e-02*v0*Jinv01)*D;
  A[19][8]=(-1.4287530563572230e-02*v1*Jinv11-2.5443547578964244e-03*Jinv00*v0-2.5443547578964244e-03*v1*Jinv10-1.4287530563572230e-02*v0*Jinv01)*D;
  A[19][9]=(-4.7755581609748278e-02*v1*Jinv11-1.1873655536849981e-02*Jinv00*v0-1.1873655536849981e-02*v1*Jinv10-4.7755581609748278e-02*v0*Jinv01)*D;
  A[19][10]=(-3.2835224349981297e-02*v1*Jinv11-6.5435585183849076e-03*Jinv00*v0-6.5435585183849076e-03*v1*Jinv10-3.2835224349981297e-02*v0*Jinv01)*D;
  A[19][11]=(-5.9916292555181445e-02*v1*Jinv11-7.8894569454465283e-02*Jinv00*v0-7.8894569454465283e-02*v1*Jinv10-5.9916292555181445e-02*v0*Jinv01)*D;
  A[19][12]=(8.4550865800865807e-02*v1*Jinv11+4.2960451489097322e-02*Jinv00*v0+4.2960451489097322e-02*v1*Jinv10+8.4550865800865807e-02*v0*Jinv01)*D;
  A[19][13]=(4.9321338383838384e-02*v1*Jinv11+4.2960451489097322e-02*Jinv00*v0+4.2960451489097322e-02*v1*Jinv10+4.9321338383838384e-02*v0*Jinv01)*D;
  A[19][14]=(3.6586516621238845e-02*v1*Jinv11-7.0263335237293568e-03*Jinv00*v0-7.0263335237293568e-03*v1*Jinv10+3.6586516621238845e-02*v0*Jinv01)*D;
  A[19][15]=(6.9911039963123298e-02*v1*Jinv11+1.0863742417561861e-01*Jinv00*v0+1.0863742417561861e-01*v1*Jinv10+6.9911039963123298e-02*v0*Jinv01)*D;
  A[19][16]=(-1.9258808321308321e-01*v1*Jinv11-1.2473862300251189e-01*Jinv00*v0-1.2473862300251189e-01*v1*Jinv10-1.9258808321308321e-01*v0*Jinv01)*D;
  A[19][17]=(-4.9008187029020364e-02*v1*Jinv11+1.6101198826893270e-02*Jinv00*v0+1.6101198826893270e-02*v1*Jinv10-4.9008187029020364e-02*v0*Jinv01)*D;
  A[19][18]=(-6.9911039963123298e-02*v1*Jinv11-1.3982207992624660e-01*Jinv00*v0-1.3982207992624660e-01*v1*Jinv10-6.9911039963123298e-02*v0*Jinv01)*D;
  A[19][19]=(1.3982207992624660e-01*v1*Jinv11+1.3982207992624660e-01*Jinv00*v0+1.3982207992624660e-01*v1*Jinv10+1.3982207992624660e-01*v0*Jinv01)*D;
  A[19][20]=(3.7257182439474103e-02*v1*Jinv11+3.7257182439474103e-02*v0*Jinv01)*D;
  A[20][0]=(-2.7789051226551225e-03*v1*Jinv11-2.7789051226551225e-03*Jinv00*v0-2.7789051226551225e-03*v1*Jinv10-2.7789051226551225e-03*v0*Jinv01)*D;
  A[20][1]=(-4.8003493098979213e-03*v1*Jinv11-4.8003493098979213e-03*v0*Jinv01)*D;
  A[20][2]=(-3.3650722669819892e-03*v1*Jinv11-3.3650722669819892e-03*v0*Jinv01)*D;
  A[20][3]=(-3.3650722669819892e-03*v1*Jinv11-3.3650722669819892e-03*v0*Jinv01)*D;
  A[20][4]=(-4.8003493098979213e-03*v1*Jinv11-4.8003493098979213e-03*v0*Jinv01)*D;
  A[20][5]=(2.7789051226551225e-03*Jinv00*v0+2.7789051226551225e-03*v1*Jinv10)*D;
  A[20][6]=(1.0531019103067715e-02*v1*Jinv11+5.7306697931697936e-03*Jinv00*v0+5.7306697931697936e-03*v1*Jinv10+1.0531019103067715e-02*v0*Jinv01)*D;
  A[20][7]=(5.1774357329912885e-03*v1*Jinv11+5.1774357329912885e-03*v0*Jinv01)*D;
  A[20][8]=(2.2781761063011062e-03*v1*Jinv11+2.2781761063011062e-03*v0*Jinv01)*D;
  A[20][9]=(5.1774357329912885e-03*v1*Jinv11+5.1774357329912885e-03*v0*Jinv01)*D;
  A[20][10]=(4.8003493098979213e-03*v1*Jinv11-5.7306697931697936e-03*Jinv00*v0-5.7306697931697936e-03*v1*Jinv10+4.8003493098979213e-03*v0*Jinv01)*D;
  A[20][11]=(-2.3263231271043770e-02*v1*Jinv11-2.1450867805034472e-02*Jinv00*v0-2.1450867805034472e-02*v1*Jinv10-2.3263231271043770e-02*v0*Jinv01)*D;
  A[20][12]=(4.7246710658168994e-03*v1*Jinv11+4.7246710658168994e-03*v0*Jinv01)*D;
  A[20][13]=(4.7246710658168994e-03*v1*Jinv11+4.7246710658168994e-03*v0*Jinv01)*D;
  A[20][14]=(-1.8123634660092993e-03*v1*Jinv11+2.1450867805034472e-02*Jinv00*v0+2.1450867805034472e-02*v1*Jinv10-1.8123634660092993e-03*v0*Jinv01)*D;
  A[20][15]=(3.3658551453690344e-02*v1*Jinv11+3.7296326358826362e-02*Jinv00*v0+3.7296326358826362e-02*v1*Jinv10+3.3658551453690344e-02*v0*Jinv01)*D;
  A[20][16]=(-1.4394523943135054e-02*v1*Jinv11-1.4394523943135054e-02*v0*Jinv01)*D;
  A[20][17]=(-3.6377749051360164e-03*v1*Jinv11-3.7296326358826362e-02*Jinv00*v0-3.7296326358826362e-02*v1*Jinv10-3.6377749051360164e-03*v0*Jinv01)*D;
  A[20][18]=(-3.7257182439474103e-02*v1*Jinv11-4.6549948893698893e-02*Jinv00*v0-4.6549948893698893e-02*v1*Jinv10-3.7257182439474103e-02*v0*Jinv01)*D;
  A[20][19]=(9.2927664542247872e-03*v1*Jinv11+4.6549948893698893e-02*Jinv00*v0+4.6549948893698893e-02*v1*Jinv10+9.2927664542247872e-03*v0*Jinv01)*D;
  A[20][20]=(1.9109748276414942e-02*v1*Jinv11+1.9109748276414942e-02*v0*Jinv01)*D;
  int nbf = 21;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}
