
#include <math.h>
#include <utility>
#include <map>
#include <ptv.h>
#include <DofT.h>
#include <ScalarFunction.h>

using namespace std; 



/* ------------- order 1 ---------------*/


 
void matrix_factory_vector_conv_3D_order1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3],
         double* vel, int size){
  
  
 
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[12];
  double idof0[4];
  idof0[0]=x0;  idof0[1]=y0;  idof0[2]=z0;
  idof0[3] = 0.0;
  ptv ipdof0(4,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[4];
  idof1[0]=x0;  idof1[1]=y0;  idof1[2]=z0;
  idof1[3] = 1.0;
  ptv ipdof1(4,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[4];
  idof2[0]=x0;  idof2[1]=y0;  idof2[2]=z0;
  idof2[3] = 2.0;
  ptv ipdof2(4,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[4];
  idof3[0]=G00+x0;  idof3[1]=y0+G10;  idof3[2]=z0+G20;
  idof3[3] = 0.0;
  ptv ipdof3(4,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[4];
  idof4[0]=G00+x0;  idof4[1]=y0+G10;  idof4[2]=z0+G20;
  idof4[3] = 1.0;
  ptv ipdof4(4,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[4];
  idof5[0]=G00+x0;  idof5[1]=y0+G10;  idof5[2]=z0+G20;
  idof5[3] = 2.0;
  ptv ipdof5(4,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

  double idof6[4];
  idof6[0]=G01+x0;  idof6[1]=y0+G11;  idof6[2]=G21+z0;
  idof6[3] = 0.0;
  ptv ipdof6(4,idof6);
  iidof[6] = dof.insert_dof(element,6,ipdof6);

  double idof7[4];
  idof7[0]=G01+x0;  idof7[1]=y0+G11;  idof7[2]=G21+z0;
  idof7[3] = 1.0;
  ptv ipdof7(4,idof7);
  iidof[7] = dof.insert_dof(element,7,ipdof7);

  double idof8[4];
  idof8[0]=G01+x0;  idof8[1]=y0+G11;  idof8[2]=G21+z0;
  idof8[3] = 2.0;
  ptv ipdof8(4,idof8);
  iidof[8] = dof.insert_dof(element,8,ipdof8);

  double idof9[4];
  idof9[0]=G02+x0;  idof9[1]=G12+y0;  idof9[2]=G22+z0;
  idof9[3] = 0.0;
  ptv ipdof9(4,idof9);
  iidof[9] = dof.insert_dof(element,9,ipdof9);

  double idof10[4];
  idof10[0]=G02+x0;  idof10[1]=G12+y0;  idof10[2]=G22+z0;
  idof10[3] = 1.0;
  ptv ipdof10(4,idof10);
  iidof[10] = dof.insert_dof(element,10,ipdof10);

  double idof11[4];
  idof11[0]=G02+x0;  idof11[1]=G12+y0;  idof11[2]=G22+z0;
  idof11[3] = 2.0;
  ptv ipdof11(4,idof11);
  iidof[11] = dof.insert_dof(element,11,ipdof11);

  double w0 = vel[iidof[0]];
  double w1 = vel[iidof[1]];
  double w2 = vel[iidof[2]];
  double w3 = vel[iidof[3]];
  double w4 = vel[iidof[4]];
  double w5 = vel[iidof[5]];
  double w6 = vel[iidof[6]];
  double w7 = vel[iidof[7]];
  double w8 = vel[iidof[8]];
  double w9 = vel[iidof[9]];
  double w10 = vel[iidof[10]];
  double w11 = vel[iidof[11]];
 
  double Jinv00 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G22*G11-G21*G12);
  double Jinv01 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G20*G12-G22*G10);
  double Jinv02 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G21*G10-G20*G11);
  double Jinv10 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G21*G02-G22*G01);
  double Jinv11 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G22*G00-G02*G20);
  double Jinv12 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(-G21*G00+G01*G20);
  double Jinv20 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G01*G12-G02*G11);
  double Jinv21 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(-G12*G00+G02*G10);
  double Jinv22 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G00*G11-G01*G10);


  double A[12][12];
  A[0][0]=(-8.3333333333333332e-03*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-1.6666666666666666e-02*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-1.6666666666666666e-02*Jinv20*w2-1.6666666666666666e-02*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-1.6666666666666666e-02*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-1.6666666666666666e-02*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-1.6666666666666666e-02*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-1.6666666666666666e-02*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-1.6666666666666666e-02*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-1.6666666666666666e-02*Jinv21*w2)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(0.0)*D;
  A[0][3]=(8.3333333333333332e-03*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+1.6666666666666666e-02*Jinv20*w2+1.6666666666666666e-02*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+1.6666666666666666e-02*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[0][4]=(0.0)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(8.3333333333333332e-03*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+1.6666666666666666e-02*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+1.6666666666666666e-02*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+1.6666666666666666e-02*Jinv21*w2)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(0.0)*D;
  A[0][9]=(1.6666666666666666e-02*w0*Jinv02+1.6666666666666666e-02*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+1.6666666666666666e-02*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[0][10]=(0.0)*D;
  A[0][11]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(-8.3333333333333332e-03*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-1.6666666666666666e-02*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-1.6666666666666666e-02*Jinv20*w2-1.6666666666666666e-02*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-1.6666666666666666e-02*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-1.6666666666666666e-02*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-1.6666666666666666e-02*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-1.6666666666666666e-02*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-1.6666666666666666e-02*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-1.6666666666666666e-02*Jinv21*w2)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(0.0)*D;
  A[1][4]=(8.3333333333333332e-03*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+1.6666666666666666e-02*Jinv20*w2+1.6666666666666666e-02*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+1.6666666666666666e-02*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[1][5]=(0.0)*D;
  A[1][6]=(0.0)*D;
  A[1][7]=(8.3333333333333332e-03*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+1.6666666666666666e-02*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+1.6666666666666666e-02*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+1.6666666666666666e-02*Jinv21*w2)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(0.0)*D;
  A[1][10]=(1.6666666666666666e-02*w0*Jinv02+1.6666666666666666e-02*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+1.6666666666666666e-02*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[1][11]=(0.0)*D;
  A[2][0]=(0.0)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(-8.3333333333333332e-03*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-1.6666666666666666e-02*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-1.6666666666666666e-02*Jinv20*w2-1.6666666666666666e-02*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-1.6666666666666666e-02*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-1.6666666666666666e-02*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-1.6666666666666666e-02*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-1.6666666666666666e-02*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-1.6666666666666666e-02*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-1.6666666666666666e-02*Jinv21*w2)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(0.0)*D;
  A[2][5]=(8.3333333333333332e-03*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+1.6666666666666666e-02*Jinv20*w2+1.6666666666666666e-02*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+1.6666666666666666e-02*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[2][6]=(0.0)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(8.3333333333333332e-03*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+1.6666666666666666e-02*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+1.6666666666666666e-02*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+1.6666666666666666e-02*Jinv21*w2)*D;
  A[2][9]=(0.0)*D;
  A[2][10]=(0.0)*D;
  A[2][11]=(1.6666666666666666e-02*w0*Jinv02+1.6666666666666666e-02*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+1.6666666666666666e-02*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[3][0]=(-8.3333333333333332e-03*Jinv10*w10-1.6666666666666666e-02*w5*Jinv20-1.6666666666666666e-02*w3*Jinv00-1.6666666666666666e-02*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-1.6666666666666666e-02*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-1.6666666666666666e-02*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-1.6666666666666666e-02*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-1.6666666666666666e-02*Jinv22*w5-1.6666666666666666e-02*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-1.6666666666666666e-02*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[3][1]=(0.0)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(8.3333333333333332e-03*Jinv10*w10+1.6666666666666666e-02*w5*Jinv20+1.6666666666666666e-02*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+1.6666666666666666e-02*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(0.0)*D;
  A[3][6]=(1.6666666666666666e-02*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+1.6666666666666666e-02*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+1.6666666666666666e-02*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[3][7]=(0.0)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+1.6666666666666666e-02*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+1.6666666666666666e-02*Jinv02*w3+1.6666666666666666e-02*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[3][10]=(0.0)*D;
  A[3][11]=(0.0)*D;
  A[4][0]=(0.0)*D;
  A[4][1]=(-8.3333333333333332e-03*Jinv10*w10-1.6666666666666666e-02*w5*Jinv20-1.6666666666666666e-02*w3*Jinv00-1.6666666666666666e-02*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-1.6666666666666666e-02*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-1.6666666666666666e-02*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-1.6666666666666666e-02*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-1.6666666666666666e-02*Jinv22*w5-1.6666666666666666e-02*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-1.6666666666666666e-02*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[4][2]=(0.0)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(8.3333333333333332e-03*Jinv10*w10+1.6666666666666666e-02*w5*Jinv20+1.6666666666666666e-02*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+1.6666666666666666e-02*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(0.0)*D;
  A[4][7]=(1.6666666666666666e-02*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+1.6666666666666666e-02*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+1.6666666666666666e-02*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[4][8]=(0.0)*D;
  A[4][9]=(0.0)*D;
  A[4][10]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+1.6666666666666666e-02*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+1.6666666666666666e-02*Jinv02*w3+1.6666666666666666e-02*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[4][11]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(0.0)*D;
  A[5][2]=(-8.3333333333333332e-03*Jinv10*w10-1.6666666666666666e-02*w5*Jinv20-1.6666666666666666e-02*w3*Jinv00-1.6666666666666666e-02*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-1.6666666666666666e-02*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-1.6666666666666666e-02*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-1.6666666666666666e-02*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-1.6666666666666666e-02*Jinv22*w5-1.6666666666666666e-02*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-1.6666666666666666e-02*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[5][3]=(0.0)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(8.3333333333333332e-03*Jinv10*w10+1.6666666666666666e-02*w5*Jinv20+1.6666666666666666e-02*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+1.6666666666666666e-02*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(0.0)*D;
  A[5][8]=(1.6666666666666666e-02*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+1.6666666666666666e-02*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+1.6666666666666666e-02*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[5][9]=(0.0)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+1.6666666666666666e-02*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+1.6666666666666666e-02*Jinv02*w3+1.6666666666666666e-02*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[6][0]=(-8.3333333333333332e-03*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-1.6666666666666666e-02*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-1.6666666666666666e-02*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-1.6666666666666666e-02*Jinv12*w7-1.6666666666666666e-02*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-1.6666666666666666e-02*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-1.6666666666666666e-02*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-1.6666666666666666e-02*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-1.6666666666666666e-02*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-1.6666666666666666e-02*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[6][1]=(0.0)*D;
  A[6][2]=(0.0)*D;
  A[6][3]=(8.3333333333333332e-03*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+1.6666666666666666e-02*w6*Jinv00+1.6666666666666666e-02*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+1.6666666666666666e-02*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[6][4]=(0.0)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(8.3333333333333332e-03*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+1.6666666666666666e-02*w6*Jinv01+1.6666666666666666e-02*Jinv11*w7+1.6666666666666666e-02*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[6][7]=(0.0)*D;
  A[6][8]=(0.0)*D;
  A[6][9]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+1.6666666666666666e-02*Jinv12*w7+1.6666666666666666e-02*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+1.6666666666666666e-02*Jinv22*w8)*D;
  A[6][10]=(0.0)*D;
  A[6][11]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(-8.3333333333333332e-03*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-1.6666666666666666e-02*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-1.6666666666666666e-02*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-1.6666666666666666e-02*Jinv12*w7-1.6666666666666666e-02*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-1.6666666666666666e-02*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-1.6666666666666666e-02*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-1.6666666666666666e-02*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-1.6666666666666666e-02*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-1.6666666666666666e-02*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(0.0)*D;
  A[7][4]=(8.3333333333333332e-03*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+1.6666666666666666e-02*w6*Jinv00+1.6666666666666666e-02*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+1.6666666666666666e-02*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[7][5]=(0.0)*D;
  A[7][6]=(0.0)*D;
  A[7][7]=(8.3333333333333332e-03*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+1.6666666666666666e-02*w6*Jinv01+1.6666666666666666e-02*Jinv11*w7+1.6666666666666666e-02*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[7][8]=(0.0)*D;
  A[7][9]=(0.0)*D;
  A[7][10]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+1.6666666666666666e-02*Jinv12*w7+1.6666666666666666e-02*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+1.6666666666666666e-02*Jinv22*w8)*D;
  A[7][11]=(0.0)*D;
  A[8][0]=(0.0)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(-8.3333333333333332e-03*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-1.6666666666666666e-02*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-1.6666666666666666e-02*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-8.3333333333333332e-03*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-1.6666666666666666e-02*Jinv12*w7-1.6666666666666666e-02*w6*Jinv02-8.3333333333333332e-03*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-8.3333333333333332e-03*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-1.6666666666666666e-02*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-1.6666666666666666e-02*w6*Jinv01-8.3333333333333332e-03*w11*Jinv20-1.6666666666666666e-02*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-1.6666666666666666e-02*Jinv21*w8-8.3333333333333332e-03*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-8.3333333333333332e-03*Jinv02*w9-8.3333333333333332e-03*w10*Jinv12-1.6666666666666666e-02*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-8.3333333333333332e-03*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(0.0)*D;
  A[8][5]=(8.3333333333333332e-03*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+1.6666666666666666e-02*w6*Jinv00+1.6666666666666666e-02*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+1.6666666666666666e-02*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+8.3333333333333332e-03*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+8.3333333333333332e-03*w9*Jinv00)*D;
  A[8][6]=(0.0)*D;
  A[8][7]=(0.0)*D;
  A[8][8]=(8.3333333333333332e-03*Jinv01*w3+8.3333333333333332e-03*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+8.3333333333333332e-03*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+1.6666666666666666e-02*w6*Jinv01+1.6666666666666666e-02*Jinv11*w7+1.6666666666666666e-02*Jinv21*w8+8.3333333333333332e-03*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[8][9]=(0.0)*D;
  A[8][10]=(0.0)*D;
  A[8][11]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+8.3333333333333332e-03*w11*Jinv22+1.6666666666666666e-02*Jinv12*w7+1.6666666666666666e-02*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+8.3333333333333332e-03*Jinv02*w9+8.3333333333333332e-03*w10*Jinv12+1.6666666666666666e-02*Jinv22*w8)*D;
  A[9][0]=(-1.6666666666666666e-02*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-1.6666666666666666e-02*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-1.6666666666666666e-02*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-1.6666666666666666e-02*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-1.6666666666666666e-02*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-1.6666666666666666e-02*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-1.6666666666666666e-02*Jinv02*w9-1.6666666666666666e-02*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-1.6666666666666666e-02*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[9][1]=(0.0)*D;
  A[9][2]=(0.0)*D;
  A[9][3]=(1.6666666666666666e-02*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+1.6666666666666666e-02*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+1.6666666666666666e-02*w9*Jinv00)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(0.0)*D;
  A[9][6]=(8.3333333333333332e-03*Jinv01*w3+1.6666666666666666e-02*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+1.6666666666666666e-02*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+1.6666666666666666e-02*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[9][7]=(0.0)*D;
  A[9][8]=(0.0)*D;
  A[9][9]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+1.6666666666666666e-02*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+1.6666666666666666e-02*Jinv02*w9+1.6666666666666666e-02*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[9][10]=(0.0)*D;
  A[9][11]=(0.0)*D;
  A[10][0]=(0.0)*D;
  A[10][1]=(-1.6666666666666666e-02*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-1.6666666666666666e-02*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-1.6666666666666666e-02*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-1.6666666666666666e-02*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-1.6666666666666666e-02*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-1.6666666666666666e-02*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-1.6666666666666666e-02*Jinv02*w9-1.6666666666666666e-02*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-1.6666666666666666e-02*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[10][2]=(0.0)*D;
  A[10][3]=(0.0)*D;
  A[10][4]=(1.6666666666666666e-02*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+1.6666666666666666e-02*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+1.6666666666666666e-02*w9*Jinv00)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(0.0)*D;
  A[10][7]=(8.3333333333333332e-03*Jinv01*w3+1.6666666666666666e-02*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+1.6666666666666666e-02*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+1.6666666666666666e-02*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[10][8]=(0.0)*D;
  A[10][9]=(0.0)*D;
  A[10][10]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+1.6666666666666666e-02*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+1.6666666666666666e-02*Jinv02*w9+1.6666666666666666e-02*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  A[10][11]=(0.0)*D;
  A[11][0]=(0.0)*D;
  A[11][1]=(0.0)*D;
  A[11][2]=(-1.6666666666666666e-02*Jinv10*w10-8.3333333333333332e-03*w5*Jinv20-8.3333333333333332e-03*w3*Jinv00-8.3333333333333332e-03*Jinv01*w3-8.3333333333333332e-03*w6*Jinv00-8.3333333333333332e-03*w0*Jinv02-8.3333333333333332e-03*Jinv10*w7-8.3333333333333332e-03*Jinv20*w2-8.3333333333333332e-03*w1*Jinv12-1.6666666666666666e-02*w11*Jinv22-8.3333333333333332e-03*w0*Jinv00-8.3333333333333332e-03*Jinv12*w7-8.3333333333333332e-03*w6*Jinv02-1.6666666666666666e-02*Jinv01*w9-8.3333333333333332e-03*Jinv11*w1-1.6666666666666666e-02*w11*Jinv21-8.3333333333333332e-03*w4*Jinv12-8.3333333333333332e-03*Jinv20*w8-8.3333333333333332e-03*Jinv11*w4-8.3333333333333332e-03*w1*Jinv10-8.3333333333333332e-03*w6*Jinv01-1.6666666666666666e-02*w11*Jinv20-8.3333333333333332e-03*Jinv11*w7-8.3333333333333332e-03*Jinv22*w2-8.3333333333333332e-03*Jinv02*w3-8.3333333333333332e-03*Jinv21*w8-1.6666666666666666e-02*Jinv11*w10-8.3333333333333332e-03*Jinv22*w5-8.3333333333333332e-03*Jinv10*w4-1.6666666666666666e-02*Jinv02*w9-1.6666666666666666e-02*w10*Jinv12-8.3333333333333332e-03*Jinv22*w8-8.3333333333333332e-03*w0*Jinv01-8.3333333333333332e-03*w5*Jinv21-1.6666666666666666e-02*w9*Jinv00-8.3333333333333332e-03*Jinv21*w2)*D;
  A[11][3]=(0.0)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(1.6666666666666666e-02*Jinv10*w10+8.3333333333333332e-03*w5*Jinv20+8.3333333333333332e-03*w3*Jinv00+8.3333333333333332e-03*w6*Jinv00+8.3333333333333332e-03*Jinv10*w7+8.3333333333333332e-03*Jinv20*w2+8.3333333333333332e-03*w0*Jinv00+8.3333333333333332e-03*Jinv20*w8+8.3333333333333332e-03*w1*Jinv10+1.6666666666666666e-02*w11*Jinv20+8.3333333333333332e-03*Jinv10*w4+1.6666666666666666e-02*w9*Jinv00)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(0.0)*D;
  A[11][8]=(8.3333333333333332e-03*Jinv01*w3+1.6666666666666666e-02*Jinv01*w9+8.3333333333333332e-03*Jinv11*w1+1.6666666666666666e-02*w11*Jinv21+8.3333333333333332e-03*Jinv11*w4+8.3333333333333332e-03*w6*Jinv01+8.3333333333333332e-03*Jinv11*w7+8.3333333333333332e-03*Jinv21*w8+1.6666666666666666e-02*Jinv11*w10+8.3333333333333332e-03*w0*Jinv01+8.3333333333333332e-03*w5*Jinv21+8.3333333333333332e-03*Jinv21*w2)*D;
  A[11][9]=(0.0)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(8.3333333333333332e-03*w0*Jinv02+8.3333333333333332e-03*w1*Jinv12+1.6666666666666666e-02*w11*Jinv22+8.3333333333333332e-03*Jinv12*w7+8.3333333333333332e-03*w6*Jinv02+8.3333333333333332e-03*w4*Jinv12+8.3333333333333332e-03*Jinv22*w2+8.3333333333333332e-03*Jinv02*w3+8.3333333333333332e-03*Jinv22*w5+1.6666666666666666e-02*Jinv02*w9+1.6666666666666666e-02*w10*Jinv12+8.3333333333333332e-03*Jinv22*w8)*D;
  int nbf = 12;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 ---------------*/


 
void matrix_factory_vector_conv_3D_order2 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3],
         double* vel, int size){
  
  
 
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[30];
  double idof0[4];
  idof0[0]=x0;  idof0[1]=y0;  idof0[2]=z0;
  idof0[3] = 0.0;
  ptv ipdof0(4,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[4];
  idof1[0]=x0;  idof1[1]=y0;  idof1[2]=z0;
  idof1[3] = 1.0;
  ptv ipdof1(4,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[4];
  idof2[0]=x0;  idof2[1]=y0;  idof2[2]=z0;
  idof2[3] = 2.0;
  ptv ipdof2(4,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[4];
  idof3[0]=G00/2.0+x0;  idof3[1]=y0+G10/2.0;  idof3[2]=z0+G20/2.0;
  idof3[3] = 0.0;
  ptv ipdof3(4,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[4];
  idof4[0]=G00/2.0+x0;  idof4[1]=y0+G10/2.0;  idof4[2]=z0+G20/2.0;
  idof4[3] = 1.0;
  ptv ipdof4(4,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[4];
  idof5[0]=G00/2.0+x0;  idof5[1]=y0+G10/2.0;  idof5[2]=z0+G20/2.0;
  idof5[3] = 2.0;
  ptv ipdof5(4,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

  double idof6[4];
  idof6[0]=G00+x0;  idof6[1]=y0+G10;  idof6[2]=z0+G20;
  idof6[3] = 0.0;
  ptv ipdof6(4,idof6);
  iidof[6] = dof.insert_dof(element,6,ipdof6);

  double idof7[4];
  idof7[0]=G00+x0;  idof7[1]=y0+G10;  idof7[2]=z0+G20;
  idof7[3] = 1.0;
  ptv ipdof7(4,idof7);
  iidof[7] = dof.insert_dof(element,7,ipdof7);

  double idof8[4];
  idof8[0]=G00+x0;  idof8[1]=y0+G10;  idof8[2]=z0+G20;
  idof8[3] = 2.0;
  ptv ipdof8(4,idof8);
  iidof[8] = dof.insert_dof(element,8,ipdof8);

  double idof9[4];
  idof9[0]=G01/2.0+x0;  idof9[1]=y0+G11/2.0;  idof9[2]=G21/2.0+z0;
  idof9[3] = 0.0;
  ptv ipdof9(4,idof9);
  iidof[9] = dof.insert_dof(element,9,ipdof9);

  double idof10[4];
  idof10[0]=G01/2.0+x0;  idof10[1]=y0+G11/2.0;  idof10[2]=G21/2.0+z0;
  idof10[3] = 1.0;
  ptv ipdof10(4,idof10);
  iidof[10] = dof.insert_dof(element,10,ipdof10);

  double idof11[4];
  idof11[0]=G01/2.0+x0;  idof11[1]=y0+G11/2.0;  idof11[2]=G21/2.0+z0;
  idof11[3] = 2.0;
  ptv ipdof11(4,idof11);
  iidof[11] = dof.insert_dof(element,11,ipdof11);

  double idof12[4];
  idof12[0]=G01/2.0+G00/2.0+x0;  idof12[1]=y0+G11/2.0+G10/2.0;  idof12[2]=G21/2.0+z0+G20/2.0;
  idof12[3] = 0.0;
  ptv ipdof12(4,idof12);
  iidof[12] = dof.insert_dof(element,12,ipdof12);

  double idof13[4];
  idof13[0]=G01/2.0+G00/2.0+x0;  idof13[1]=y0+G11/2.0+G10/2.0;  idof13[2]=G21/2.0+z0+G20/2.0;
  idof13[3] = 1.0;
  ptv ipdof13(4,idof13);
  iidof[13] = dof.insert_dof(element,13,ipdof13);

  double idof14[4];
  idof14[0]=G01/2.0+G00/2.0+x0;  idof14[1]=y0+G11/2.0+G10/2.0;  idof14[2]=G21/2.0+z0+G20/2.0;
  idof14[3] = 2.0;
  ptv ipdof14(4,idof14);
  iidof[14] = dof.insert_dof(element,14,ipdof14);

  double idof15[4];
  idof15[0]=G01+x0;  idof15[1]=y0+G11;  idof15[2]=G21+z0;
  idof15[3] = 0.0;
  ptv ipdof15(4,idof15);
  iidof[15] = dof.insert_dof(element,15,ipdof15);

  double idof16[4];
  idof16[0]=G01+x0;  idof16[1]=y0+G11;  idof16[2]=G21+z0;
  idof16[3] = 1.0;
  ptv ipdof16(4,idof16);
  iidof[16] = dof.insert_dof(element,16,ipdof16);

  double idof17[4];
  idof17[0]=G01+x0;  idof17[1]=y0+G11;  idof17[2]=G21+z0;
  idof17[3] = 2.0;
  ptv ipdof17(4,idof17);
  iidof[17] = dof.insert_dof(element,17,ipdof17);

  double idof18[4];
  idof18[0]=G02/2.0+x0;  idof18[1]=G12/2.0+y0;  idof18[2]=G22/2.0+z0;
  idof18[3] = 0.0;
  ptv ipdof18(4,idof18);
  iidof[18] = dof.insert_dof(element,18,ipdof18);

  double idof19[4];
  idof19[0]=G02/2.0+x0;  idof19[1]=G12/2.0+y0;  idof19[2]=G22/2.0+z0;
  idof19[3] = 1.0;
  ptv ipdof19(4,idof19);
  iidof[19] = dof.insert_dof(element,19,ipdof19);

  double idof20[4];
  idof20[0]=G02/2.0+x0;  idof20[1]=G12/2.0+y0;  idof20[2]=G22/2.0+z0;
  idof20[3] = 2.0;
  ptv ipdof20(4,idof20);
  iidof[20] = dof.insert_dof(element,20,ipdof20);

  double idof21[4];
  idof21[0]=G02/2.0+G00/2.0+x0;  idof21[1]=G12/2.0+y0+G10/2.0;  idof21[2]=G22/2.0+z0+G20/2.0;
  idof21[3] = 0.0;
  ptv ipdof21(4,idof21);
  iidof[21] = dof.insert_dof(element,21,ipdof21);

  double idof22[4];
  idof22[0]=G02/2.0+G00/2.0+x0;  idof22[1]=G12/2.0+y0+G10/2.0;  idof22[2]=G22/2.0+z0+G20/2.0;
  idof22[3] = 1.0;
  ptv ipdof22(4,idof22);
  iidof[22] = dof.insert_dof(element,22,ipdof22);

  double idof23[4];
  idof23[0]=G02/2.0+G00/2.0+x0;  idof23[1]=G12/2.0+y0+G10/2.0;  idof23[2]=G22/2.0+z0+G20/2.0;
  idof23[3] = 2.0;
  ptv ipdof23(4,idof23);
  iidof[23] = dof.insert_dof(element,23,ipdof23);

  double idof24[4];
  idof24[0]=G02/2.0+G01/2.0+x0;  idof24[1]=G12/2.0+y0+G11/2.0;  idof24[2]=G22/2.0+G21/2.0+z0;
  idof24[3] = 0.0;
  ptv ipdof24(4,idof24);
  iidof[24] = dof.insert_dof(element,24,ipdof24);

  double idof25[4];
  idof25[0]=G02/2.0+G01/2.0+x0;  idof25[1]=G12/2.0+y0+G11/2.0;  idof25[2]=G22/2.0+G21/2.0+z0;
  idof25[3] = 1.0;
  ptv ipdof25(4,idof25);
  iidof[25] = dof.insert_dof(element,25,ipdof25);

  double idof26[4];
  idof26[0]=G02/2.0+G01/2.0+x0;  idof26[1]=G12/2.0+y0+G11/2.0;  idof26[2]=G22/2.0+G21/2.0+z0;
  idof26[3] = 2.0;
  ptv ipdof26(4,idof26);
  iidof[26] = dof.insert_dof(element,26,ipdof26);

  double idof27[4];
  idof27[0]=G02+x0;  idof27[1]=G12+y0;  idof27[2]=G22+z0;
  idof27[3] = 0.0;
  ptv ipdof27(4,idof27);
  iidof[27] = dof.insert_dof(element,27,ipdof27);

  double idof28[4];
  idof28[0]=G02+x0;  idof28[1]=G12+y0;  idof28[2]=G22+z0;
  idof28[3] = 1.0;
  ptv ipdof28(4,idof28);
  iidof[28] = dof.insert_dof(element,28,ipdof28);

  double idof29[4];
  idof29[0]=G02+x0;  idof29[1]=G12+y0;  idof29[2]=G22+z0;
  idof29[3] = 2.0;
  ptv ipdof29(4,idof29);
  iidof[29] = dof.insert_dof(element,29,ipdof29);

  double w0 = vel[iidof[0]];
  double w1 = vel[iidof[1]];
  double w2 = vel[iidof[2]];
  double w3 = vel[iidof[3]];
  double w4 = vel[iidof[4]];
  double w5 = vel[iidof[5]];
  double w6 = vel[iidof[6]];
  double w7 = vel[iidof[7]];
  double w8 = vel[iidof[8]];
  double w9 = vel[iidof[9]];
  double w10 = vel[iidof[10]];
  double w11 = vel[iidof[11]];
  double w12 = vel[iidof[12]];
  double w13 = vel[iidof[13]];
  double w14 = vel[iidof[14]];
  double w15 = vel[iidof[15]];
  double w16 = vel[iidof[16]];
  double w17 = vel[iidof[17]];
  double w18 = vel[iidof[18]];
  double w19 = vel[iidof[19]];
  double w20 = vel[iidof[20]];
  double w21 = vel[iidof[21]];
  double w22 = vel[iidof[22]];
  double w23 = vel[iidof[23]];
  double w24 = vel[iidof[24]];
  double w25 = vel[iidof[25]];
  double w26 = vel[iidof[26]];
  double w27 = vel[iidof[27]];
  double w28 = vel[iidof[28]];
  double w29 = vel[iidof[29]];
 
  double Jinv00 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G22*G11-G21*G12);
  double Jinv01 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G20*G12-G22*G10);
  double Jinv02 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G21*G10-G20*G11);
  double Jinv10 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G21*G02-G22*G01);
  double Jinv11 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G22*G00-G02*G20);
  double Jinv12 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(-G21*G00+G01*G20);
  double Jinv20 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G01*G12-G02*G11);
  double Jinv21 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(-G12*G00+G02*G10);
  double Jinv22 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G00*G11-G01*G10);


  double A[30][30];
  A[0][0]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-7.9365079365079365e-04*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22+3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+3.9682539682539683e-04*Jinv12*w7+3.9682539682539683e-04*Jinv00*w15-2.3809523809523812e-03*Jinv00*w0+3.9682539682539683e-04*w7*Jinv11-7.9365079365079365e-04*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12+3.9682539682539683e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+3.9682539682539683e-04*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18-7.9365079365079365e-04*w23*Jinv21-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*Jinv10*w16+3.9682539682539683e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-2.3809523809523812e-03*Jinv20*w2-7.9365079365079365e-04*w25*Jinv10+3.9682539682539683e-04*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv11*w22+3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*Jinv00*w21-7.9365079365079365e-04*w25*Jinv11-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*Jinv01*w27+3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w8*Jinv22-2.3809523809523812e-03*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11+3.9682539682539683e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12+3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3-7.9365079365079365e-04*Jinv20*w23-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-7.9365079365079365e-04*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-2.3809523809523812e-03*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv01*w21+3.9682539682539683e-04*Jinv02*w15-2.3809523809523812e-03*w1*Jinv10-7.9365079365079365e-04*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+3.9682539682539683e-04*Jinv01*w6+3.9682539682539683e-04*Jinv00*w27-7.9365079365079365e-04*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-2.3809523809523812e-03*Jinv11*w1-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w28*Jinv11+3.9682539682539683e-04*w17*Jinv22+3.9682539682539683e-04*Jinv20*w29-1.5873015873015873e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18-7.9365079365079365e-04*Jinv01*w24-2.3809523809523812e-03*w2*Jinv22+3.9682539682539683e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+3.9682539682539683e-04*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11-7.9365079365079365e-04*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10-7.9365079365079365e-04*Jinv12*w13-2.3809523809523812e-03*Jinv01*w0+3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w26*Jinv20+3.9682539682539683e-04*Jinv12*w16-7.9365079365079365e-04*Jinv20*w14+3.9682539682539683e-04*w29*Jinv22-2.3809523809523812e-03*w2*Jinv21+3.9682539682539683e-04*Jinv02*w6)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(0.0)*D;
  A[0][3]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-1.5873015873015873e-03*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11-7.9365079365079365e-04*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*w17*Jinv22-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21)*D;
  A[0][4]=(0.0)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22+3.9682539682539683e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0+7.9365079365079365e-04*w24*Jinv00+3.9682539682539683e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2+7.9365079365079365e-04*w25*Jinv10+3.9682539682539683e-04*Jinv20*w17-7.9365079365079365e-04*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-7.9365079365079365e-04*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5+3.9682539682539683e-04*Jinv00*w27+3.9682539682539683e-04*Jinv20*w29-1.5873015873015873e-03*w4*Jinv10+7.9365079365079365e-04*w26*Jinv20-7.9365079365079365e-04*Jinv20*w14)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(0.0)*D;
  A[0][9]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-1.5873015873015873e-03*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6-7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[0][10]=(0.0)*D;
  A[0][11]=(0.0)*D;
  A[0][12]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10+1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w2*Jinv21)*D;
  A[0][13]=(0.0)*D;
  A[0][14]=(0.0)*D;
  A[0][15]=(3.9682539682539683e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*w23*Jinv21-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv11*w22-7.9365079365079365e-04*w25*Jinv11+3.9682539682539683e-04*Jinv01*w27-1.5873015873015873e-03*w10*Jinv11-7.9365079365079365e-04*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*Jinv01*w21-7.9365079365079365e-04*w14*Jinv21+3.9682539682539683e-04*Jinv01*w6-7.9365079365079365e-04*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.9682539682539683e-04*w28*Jinv11-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w29*Jinv21-3.9682539682539683e-04*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[0][16]=(0.0)*D;
  A[0][17]=(0.0)*D;
  A[0][18]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[0][19]=(0.0)*D;
  A[0][20]=(0.0)*D;
  A[0][21]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+7.9365079365079365e-04*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv00*w6+1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*w26*Jinv20+7.9365079365079365e-04*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22)*D;
  A[0][22]=(0.0)*D;
  A[0][23]=(0.0)*D;
  A[0][24]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+7.9365079365079365e-04*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+7.9365079365079365e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22+7.9365079365079365e-04*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*w14*Jinv21+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[0][25]=(0.0)*D;
  A[0][26]=(0.0)*D;
  A[0][27]=(7.9365079365079365e-04*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+3.9682539682539683e-04*Jinv12*w7-1.5873015873015873e-03*w19*Jinv12-3.9682539682539683e-04*Jinv02*w27-1.5873015873015873e-03*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+7.9365079365079365e-04*w14*Jinv22-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22+3.9682539682539683e-04*Jinv02*w15-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*Jinv02*w21+7.9365079365079365e-04*Jinv12*w13+3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*Jinv02*w6)*D;
  A[0][28]=(0.0)*D;
  A[0][29]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-7.9365079365079365e-04*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22+3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+3.9682539682539683e-04*Jinv12*w7+3.9682539682539683e-04*Jinv00*w15-2.3809523809523812e-03*Jinv00*w0+3.9682539682539683e-04*w7*Jinv11-7.9365079365079365e-04*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12+3.9682539682539683e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+3.9682539682539683e-04*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18-7.9365079365079365e-04*w23*Jinv21-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*Jinv10*w16+3.9682539682539683e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-2.3809523809523812e-03*Jinv20*w2-7.9365079365079365e-04*w25*Jinv10+3.9682539682539683e-04*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv11*w22+3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*Jinv00*w21-7.9365079365079365e-04*w25*Jinv11-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*Jinv01*w27+3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w8*Jinv22-2.3809523809523812e-03*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11+3.9682539682539683e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12+3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3-7.9365079365079365e-04*Jinv20*w23-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-7.9365079365079365e-04*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-2.3809523809523812e-03*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv01*w21+3.9682539682539683e-04*Jinv02*w15-2.3809523809523812e-03*w1*Jinv10-7.9365079365079365e-04*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+3.9682539682539683e-04*Jinv01*w6+3.9682539682539683e-04*Jinv00*w27-7.9365079365079365e-04*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-2.3809523809523812e-03*Jinv11*w1-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w28*Jinv11+3.9682539682539683e-04*w17*Jinv22+3.9682539682539683e-04*Jinv20*w29-1.5873015873015873e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18-7.9365079365079365e-04*Jinv01*w24-2.3809523809523812e-03*w2*Jinv22+3.9682539682539683e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+3.9682539682539683e-04*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11-7.9365079365079365e-04*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10-7.9365079365079365e-04*Jinv12*w13-2.3809523809523812e-03*Jinv01*w0+3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w26*Jinv20+3.9682539682539683e-04*Jinv12*w16-7.9365079365079365e-04*Jinv20*w14+3.9682539682539683e-04*w29*Jinv22-2.3809523809523812e-03*w2*Jinv21+3.9682539682539683e-04*Jinv02*w6)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(0.0)*D;
  A[1][4]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-1.5873015873015873e-03*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11-7.9365079365079365e-04*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*w17*Jinv22-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21)*D;
  A[1][5]=(0.0)*D;
  A[1][6]=(0.0)*D;
  A[1][7]=(-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22+3.9682539682539683e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0+7.9365079365079365e-04*w24*Jinv00+3.9682539682539683e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2+7.9365079365079365e-04*w25*Jinv10+3.9682539682539683e-04*Jinv20*w17-7.9365079365079365e-04*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-7.9365079365079365e-04*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5+3.9682539682539683e-04*Jinv00*w27+3.9682539682539683e-04*Jinv20*w29-1.5873015873015873e-03*w4*Jinv10+7.9365079365079365e-04*w26*Jinv20-7.9365079365079365e-04*Jinv20*w14)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(0.0)*D;
  A[1][10]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-1.5873015873015873e-03*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6-7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[1][11]=(0.0)*D;
  A[1][12]=(0.0)*D;
  A[1][13]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10+1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w2*Jinv21)*D;
  A[1][14]=(0.0)*D;
  A[1][15]=(0.0)*D;
  A[1][16]=(3.9682539682539683e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*w23*Jinv21-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv11*w22-7.9365079365079365e-04*w25*Jinv11+3.9682539682539683e-04*Jinv01*w27-1.5873015873015873e-03*w10*Jinv11-7.9365079365079365e-04*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*Jinv01*w21-7.9365079365079365e-04*w14*Jinv21+3.9682539682539683e-04*Jinv01*w6-7.9365079365079365e-04*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.9682539682539683e-04*w28*Jinv11-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w29*Jinv21-3.9682539682539683e-04*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[1][17]=(0.0)*D;
  A[1][18]=(0.0)*D;
  A[1][19]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[1][20]=(0.0)*D;
  A[1][21]=(0.0)*D;
  A[1][22]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+7.9365079365079365e-04*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv00*w6+1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*w26*Jinv20+7.9365079365079365e-04*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22)*D;
  A[1][23]=(0.0)*D;
  A[1][24]=(0.0)*D;
  A[1][25]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+7.9365079365079365e-04*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+7.9365079365079365e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22+7.9365079365079365e-04*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*w14*Jinv21+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[1][26]=(0.0)*D;
  A[1][27]=(0.0)*D;
  A[1][28]=(7.9365079365079365e-04*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+3.9682539682539683e-04*Jinv12*w7-1.5873015873015873e-03*w19*Jinv12-3.9682539682539683e-04*Jinv02*w27-1.5873015873015873e-03*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+7.9365079365079365e-04*w14*Jinv22-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22+3.9682539682539683e-04*Jinv02*w15-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*Jinv02*w21+7.9365079365079365e-04*Jinv12*w13+3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*Jinv02*w6)*D;
  A[1][29]=(0.0)*D;
  A[2][0]=(0.0)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-7.9365079365079365e-04*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22+3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+3.9682539682539683e-04*Jinv12*w7+3.9682539682539683e-04*Jinv00*w15-2.3809523809523812e-03*Jinv00*w0+3.9682539682539683e-04*w7*Jinv11-7.9365079365079365e-04*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12+3.9682539682539683e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+3.9682539682539683e-04*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18-7.9365079365079365e-04*w23*Jinv21-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*Jinv10*w16+3.9682539682539683e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-2.3809523809523812e-03*Jinv20*w2-7.9365079365079365e-04*w25*Jinv10+3.9682539682539683e-04*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv11*w22+3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*Jinv00*w21-7.9365079365079365e-04*w25*Jinv11-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*Jinv01*w27+3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w8*Jinv22-2.3809523809523812e-03*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11+3.9682539682539683e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12+3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3-7.9365079365079365e-04*Jinv20*w23-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-7.9365079365079365e-04*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-2.3809523809523812e-03*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv01*w21+3.9682539682539683e-04*Jinv02*w15-2.3809523809523812e-03*w1*Jinv10-7.9365079365079365e-04*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+3.9682539682539683e-04*Jinv01*w6+3.9682539682539683e-04*Jinv00*w27-7.9365079365079365e-04*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-2.3809523809523812e-03*Jinv11*w1-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w28*Jinv11+3.9682539682539683e-04*w17*Jinv22+3.9682539682539683e-04*Jinv20*w29-1.5873015873015873e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18-7.9365079365079365e-04*Jinv01*w24-2.3809523809523812e-03*w2*Jinv22+3.9682539682539683e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+3.9682539682539683e-04*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11-7.9365079365079365e-04*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10-7.9365079365079365e-04*Jinv12*w13-2.3809523809523812e-03*Jinv01*w0+3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w26*Jinv20+3.9682539682539683e-04*Jinv12*w16-7.9365079365079365e-04*Jinv20*w14+3.9682539682539683e-04*w29*Jinv22-2.3809523809523812e-03*w2*Jinv21+3.9682539682539683e-04*Jinv02*w6)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(0.0)*D;
  A[2][5]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-1.5873015873015873e-03*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11-7.9365079365079365e-04*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*w17*Jinv22-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21)*D;
  A[2][6]=(0.0)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22+3.9682539682539683e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0+7.9365079365079365e-04*w24*Jinv00+3.9682539682539683e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2+7.9365079365079365e-04*w25*Jinv10+3.9682539682539683e-04*Jinv20*w17-7.9365079365079365e-04*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-7.9365079365079365e-04*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5+3.9682539682539683e-04*Jinv00*w27+3.9682539682539683e-04*Jinv20*w29-1.5873015873015873e-03*w4*Jinv10+7.9365079365079365e-04*w26*Jinv20-7.9365079365079365e-04*Jinv20*w14)*D;
  A[2][9]=(0.0)*D;
  A[2][10]=(0.0)*D;
  A[2][11]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-1.5873015873015873e-03*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6-7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[2][12]=(0.0)*D;
  A[2][13]=(0.0)*D;
  A[2][14]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10+1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w2*Jinv21)*D;
  A[2][15]=(0.0)*D;
  A[2][16]=(0.0)*D;
  A[2][17]=(3.9682539682539683e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*w23*Jinv21-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv11*w22-7.9365079365079365e-04*w25*Jinv11+3.9682539682539683e-04*Jinv01*w27-1.5873015873015873e-03*w10*Jinv11-7.9365079365079365e-04*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*Jinv01*w21-7.9365079365079365e-04*w14*Jinv21+3.9682539682539683e-04*Jinv01*w6-7.9365079365079365e-04*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.9682539682539683e-04*w28*Jinv11-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w29*Jinv21-3.9682539682539683e-04*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[2][18]=(0.0)*D;
  A[2][19]=(0.0)*D;
  A[2][20]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[2][21]=(0.0)*D;
  A[2][22]=(0.0)*D;
  A[2][23]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+7.9365079365079365e-04*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv00*w6+1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*w26*Jinv20+7.9365079365079365e-04*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22)*D;
  A[2][24]=(0.0)*D;
  A[2][25]=(0.0)*D;
  A[2][26]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+7.9365079365079365e-04*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+7.9365079365079365e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22+7.9365079365079365e-04*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*w14*Jinv21+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[2][27]=(0.0)*D;
  A[2][28]=(0.0)*D;
  A[2][29]=(7.9365079365079365e-04*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+3.9682539682539683e-04*Jinv12*w7-1.5873015873015873e-03*w19*Jinv12-3.9682539682539683e-04*Jinv02*w27-1.5873015873015873e-03*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+7.9365079365079365e-04*w14*Jinv22-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22+3.9682539682539683e-04*Jinv02*w15-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*Jinv02*w21+7.9365079365079365e-04*Jinv12*w13+3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*Jinv02*w6)*D;
  A[3][0]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-3.1746031746031746e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[3][1]=(0.0)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(-1.9047619047619049e-02*w5*Jinv21-1.9047619047619049e-02*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.9047619047619049e-02*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-9.5238095238095247e-03*Jinv01*w12-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2+3.1746031746031746e-03*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-9.5238095238095247e-03*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w13*Jinv10-1.9047619047619049e-02*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-9.5238095238095247e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-9.5238095238095247e-03*w14*Jinv21-9.5238095238095247e-03*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22-1.9047619047619049e-02*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-1.9047619047619049e-02*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11-9.5238095238095247e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21+3.1746031746031746e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(0.0)*D;
  A[3][6]=(1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w14)*D;
  A[3][7]=(0.0)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(1.2698412698412698e-02*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10+1.2698412698412698e-02*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10+1.2698412698412698e-02*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[3][10]=(0.0)*D;
  A[3][11]=(0.0)*D;
  A[3][12]=(1.9047619047619049e-02*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10+9.5238095238095247e-03*w23*Jinv21+9.5238095238095247e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+6.3492063492063492e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+1.9047619047619049e-02*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+9.5238095238095247e-03*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+1.9047619047619049e-02*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[3][13]=(0.0)*D;
  A[3][14]=(0.0)*D;
  A[3][15]=(-6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*Jinv01*w18+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv01*w27-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*w17*Jinv21)*D;
  A[3][16]=(0.0)*D;
  A[3][17]=(0.0)*D;
  A[3][18]=(-6.3492063492063492e-03*w5*Jinv21+1.2698412698412698e-02*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16+1.2698412698412698e-02*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-3.1746031746031746e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29+1.2698412698412698e-02*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[3][19]=(0.0)*D;
  A[3][20]=(0.0)*D;
  A[3][21]=(1.9047619047619049e-02*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+1.9047619047619049e-02*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*Jinv00*w21+9.5238095238095247e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+9.5238095238095247e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+9.5238095238095247e-03*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+1.9047619047619049e-02*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+9.5238095238095247e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10+9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[3][22]=(0.0)*D;
  A[3][23]=(0.0)*D;
  A[3][24]=(6.3492063492063492e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+6.3492063492063492e-03*Jinv02*w3+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[3][25]=(0.0)*D;
  A[3][26]=(0.0)*D;
  A[3][27]=(-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv02*w9-6.3492063492063492e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w15-3.1746031746031746e-03*w11*Jinv22+7.9365079365079365e-04*w17*Jinv22-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22)*D;
  A[3][28]=(0.0)*D;
  A[3][29]=(0.0)*D;
  A[4][0]=(0.0)*D;
  A[4][1]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-3.1746031746031746e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[4][2]=(0.0)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(-1.9047619047619049e-02*w5*Jinv21-1.9047619047619049e-02*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.9047619047619049e-02*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-9.5238095238095247e-03*Jinv01*w12-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2+3.1746031746031746e-03*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-9.5238095238095247e-03*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w13*Jinv10-1.9047619047619049e-02*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-9.5238095238095247e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-9.5238095238095247e-03*w14*Jinv21-9.5238095238095247e-03*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22-1.9047619047619049e-02*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-1.9047619047619049e-02*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11-9.5238095238095247e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21+3.1746031746031746e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(0.0)*D;
  A[4][7]=(1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w14)*D;
  A[4][8]=(0.0)*D;
  A[4][9]=(0.0)*D;
  A[4][10]=(1.2698412698412698e-02*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10+1.2698412698412698e-02*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10+1.2698412698412698e-02*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[4][11]=(0.0)*D;
  A[4][12]=(0.0)*D;
  A[4][13]=(1.9047619047619049e-02*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10+9.5238095238095247e-03*w23*Jinv21+9.5238095238095247e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+6.3492063492063492e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+1.9047619047619049e-02*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+9.5238095238095247e-03*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+1.9047619047619049e-02*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[4][14]=(0.0)*D;
  A[4][15]=(0.0)*D;
  A[4][16]=(-6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*Jinv01*w18+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv01*w27-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*w17*Jinv21)*D;
  A[4][17]=(0.0)*D;
  A[4][18]=(0.0)*D;
  A[4][19]=(-6.3492063492063492e-03*w5*Jinv21+1.2698412698412698e-02*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16+1.2698412698412698e-02*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-3.1746031746031746e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29+1.2698412698412698e-02*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[4][20]=(0.0)*D;
  A[4][21]=(0.0)*D;
  A[4][22]=(1.9047619047619049e-02*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+1.9047619047619049e-02*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*Jinv00*w21+9.5238095238095247e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+9.5238095238095247e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+9.5238095238095247e-03*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+1.9047619047619049e-02*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+9.5238095238095247e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10+9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[4][23]=(0.0)*D;
  A[4][24]=(0.0)*D;
  A[4][25]=(6.3492063492063492e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+6.3492063492063492e-03*Jinv02*w3+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[4][26]=(0.0)*D;
  A[4][27]=(0.0)*D;
  A[4][28]=(-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv02*w9-6.3492063492063492e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w15-3.1746031746031746e-03*w11*Jinv22+7.9365079365079365e-04*w17*Jinv22-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22)*D;
  A[4][29]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(0.0)*D;
  A[5][2]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-3.1746031746031746e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[5][3]=(0.0)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(-1.9047619047619049e-02*w5*Jinv21-1.9047619047619049e-02*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.9047619047619049e-02*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-9.5238095238095247e-03*Jinv01*w12-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2+3.1746031746031746e-03*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-9.5238095238095247e-03*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w13*Jinv10-1.9047619047619049e-02*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-9.5238095238095247e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-9.5238095238095247e-03*w14*Jinv21-9.5238095238095247e-03*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22-1.9047619047619049e-02*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-1.9047619047619049e-02*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11-9.5238095238095247e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21+3.1746031746031746e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(0.0)*D;
  A[5][8]=(1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w14)*D;
  A[5][9]=(0.0)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(1.2698412698412698e-02*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10+1.2698412698412698e-02*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10+1.2698412698412698e-02*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[5][12]=(0.0)*D;
  A[5][13]=(0.0)*D;
  A[5][14]=(1.9047619047619049e-02*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10+9.5238095238095247e-03*w23*Jinv21+9.5238095238095247e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+6.3492063492063492e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+1.9047619047619049e-02*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+9.5238095238095247e-03*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+1.9047619047619049e-02*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[5][15]=(0.0)*D;
  A[5][16]=(0.0)*D;
  A[5][17]=(-6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*Jinv01*w18+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv01*w27-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*w28*Jinv11+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*w17*Jinv21)*D;
  A[5][18]=(0.0)*D;
  A[5][19]=(0.0)*D;
  A[5][20]=(-6.3492063492063492e-03*w5*Jinv21+1.2698412698412698e-02*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16+1.2698412698412698e-02*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-3.1746031746031746e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29+1.2698412698412698e-02*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[5][21]=(0.0)*D;
  A[5][22]=(0.0)*D;
  A[5][23]=(1.9047619047619049e-02*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+1.9047619047619049e-02*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*Jinv00*w21+9.5238095238095247e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+9.5238095238095247e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+9.5238095238095247e-03*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+1.9047619047619049e-02*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+9.5238095238095247e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10+9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[5][24]=(0.0)*D;
  A[5][25]=(0.0)*D;
  A[5][26]=(6.3492063492063492e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+6.3492063492063492e-03*Jinv02*w3+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[5][27]=(0.0)*D;
  A[5][28]=(0.0)*D;
  A[5][29]=(-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv02*w9-6.3492063492063492e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w15-3.1746031746031746e-03*w11*Jinv22+7.9365079365079365e-04*w17*Jinv22-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22)*D;
  A[6][0]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+7.9365079365079365e-04*Jinv01*w18+7.9365079365079365e-04*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-7.9365079365079365e-04*w24*Jinv00+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w19*Jinv12-3.9682539682539683e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27+7.9365079365079365e-04*w20*Jinv21+7.9365079365079365e-04*w10*Jinv10+7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21+7.9365079365079365e-04*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2-7.9365079365079365e-04*w25*Jinv10-3.9682539682539683e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w25*Jinv11-3.9682539682539683e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w10*Jinv11-3.9682539682539683e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*Jinv01*w3-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*w26*Jinv21+7.9365079365079365e-04*w11*Jinv21-7.9365079365079365e-04*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-3.9682539682539683e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10+1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6-3.9682539682539683e-04*Jinv00*w27+7.9365079365079365e-04*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1-7.9365079365079365e-04*w26*Jinv22-3.9682539682539683e-04*w28*Jinv11-3.9682539682539683e-04*w17*Jinv22-3.9682539682539683e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+7.9365079365079365e-04*Jinv00*w18-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w2*Jinv22-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+7.9365079365079365e-04*Jinv20*w11+7.9365079365079365e-04*w19*Jinv10+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w26*Jinv20-3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[6][1]=(0.0)*D;
  A[6][2]=(0.0)*D;
  A[6][3]=(1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-4.7619047619047623e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-4.7619047619047623e-03*w7*Jinv11-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16-4.7619047619047623e-03*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11-3.1746031746031746e-03*Jinv00*w6-4.7619047619047623e-03*w8*Jinv22+1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-3.1746031746031746e-03*Jinv20*w5-4.7619047619047623e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w11*Jinv22+1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24+1.5873015873015873e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10-1.5873015873015873e-03*Jinv20*w14-4.7619047619047623e-03*Jinv02*w6)*D;
  A[6][4]=(0.0)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(2.3809523809523812e-03*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22-3.9682539682539683e-04*Jinv00*w15-3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w24*Jinv00+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w10*Jinv10-3.9682539682539683e-04*Jinv10*w16-3.9682539682539683e-04*Jinv20*w2+7.9365079365079365e-04*w25*Jinv10-3.9682539682539683e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20+1.5873015873015873e-03*Jinv00*w21+2.3809523809523812e-03*Jinv00*w6-3.9682539682539683e-04*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+2.3809523809523812e-03*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv20*w23-3.9682539682539683e-04*w1*Jinv10+1.5873015873015873e-03*Jinv20*w5-3.9682539682539683e-04*Jinv00*w27-3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv00*w18+1.5873015873015873e-03*w4*Jinv10+7.9365079365079365e-04*Jinv20*w11+7.9365079365079365e-04*w19*Jinv10+7.9365079365079365e-04*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14)*D;
  A[6][7]=(0.0)*D;
  A[6][8]=(0.0)*D;
  A[6][9]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[6][10]=(0.0)*D;
  A[6][11]=(0.0)*D;
  A[6][12]=(-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+7.9365079365079365e-04*Jinv00*w0+4.7619047619047623e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10+4.7619047619047623e-03*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5+4.7619047619047623e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14)*D;
  A[6][13]=(0.0)*D;
  A[6][14]=(0.0)*D;
  A[6][15]=(7.9365079365079365e-04*Jinv01*w18-7.9365079365079365e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*w20*Jinv21-1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21-7.9365079365079365e-04*Jinv01*w9-7.9365079365079365e-04*w25*Jinv11+3.9682539682539683e-04*Jinv01*w27-7.9365079365079365e-04*w10*Jinv11-7.9365079365079365e-04*w26*Jinv21-7.9365079365079365e-04*w11*Jinv21-1.5873015873015873e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+3.9682539682539683e-04*Jinv11*w1+3.9682539682539683e-04*w28*Jinv11-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11-3.9682539682539683e-04*Jinv01*w15+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21+3.9682539682539683e-04*w2*Jinv21)*D;
  A[6][16]=(0.0)*D;
  A[6][17]=(0.0)*D;
  A[6][18]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00-7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[6][19]=(0.0)*D;
  A[6][20]=(0.0)*D;
  A[6][21]=(-1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+4.7619047619047623e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6+4.7619047619047623e-03*w8*Jinv22-1.5873015873015873e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20-1.5873015873015873e-03*Jinv20*w14+4.7619047619047623e-03*Jinv02*w6)*D;
  A[6][22]=(0.0)*D;
  A[6][23]=(0.0)*D;
  A[6][24]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22+1.5873015873015873e-03*w8*Jinv22+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+7.9365079365079365e-04*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24+7.9365079365079365e-04*w2*Jinv22-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w29*Jinv22+7.9365079365079365e-04*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[6][25]=(0.0)*D;
  A[6][26]=(0.0)*D;
  A[6][27]=(-7.9365079365079365e-04*w20*Jinv22+7.9365079365079365e-04*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*w19*Jinv12-3.9682539682539683e-04*Jinv02*w27-7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22+3.9682539682539683e-04*Jinv02*w15+7.9365079365079365e-04*w11*Jinv22-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w17*Jinv22+3.9682539682539683e-04*w2*Jinv22-1.5873015873015873e-03*Jinv02*w21+3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[6][28]=(0.0)*D;
  A[6][29]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+7.9365079365079365e-04*Jinv01*w18+7.9365079365079365e-04*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-7.9365079365079365e-04*w24*Jinv00+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w19*Jinv12-3.9682539682539683e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27+7.9365079365079365e-04*w20*Jinv21+7.9365079365079365e-04*w10*Jinv10+7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21+7.9365079365079365e-04*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2-7.9365079365079365e-04*w25*Jinv10-3.9682539682539683e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w25*Jinv11-3.9682539682539683e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w10*Jinv11-3.9682539682539683e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*Jinv01*w3-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*w26*Jinv21+7.9365079365079365e-04*w11*Jinv21-7.9365079365079365e-04*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-3.9682539682539683e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10+1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6-3.9682539682539683e-04*Jinv00*w27+7.9365079365079365e-04*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1-7.9365079365079365e-04*w26*Jinv22-3.9682539682539683e-04*w28*Jinv11-3.9682539682539683e-04*w17*Jinv22-3.9682539682539683e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+7.9365079365079365e-04*Jinv00*w18-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w2*Jinv22-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+7.9365079365079365e-04*Jinv20*w11+7.9365079365079365e-04*w19*Jinv10+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w26*Jinv20-3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(0.0)*D;
  A[7][4]=(1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-4.7619047619047623e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-4.7619047619047623e-03*w7*Jinv11-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16-4.7619047619047623e-03*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11-3.1746031746031746e-03*Jinv00*w6-4.7619047619047623e-03*w8*Jinv22+1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-3.1746031746031746e-03*Jinv20*w5-4.7619047619047623e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w11*Jinv22+1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24+1.5873015873015873e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10-1.5873015873015873e-03*Jinv20*w14-4.7619047619047623e-03*Jinv02*w6)*D;
  A[7][5]=(0.0)*D;
  A[7][6]=(0.0)*D;
  A[7][7]=(2.3809523809523812e-03*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22-3.9682539682539683e-04*Jinv00*w15-3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w24*Jinv00+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w10*Jinv10-3.9682539682539683e-04*Jinv10*w16-3.9682539682539683e-04*Jinv20*w2+7.9365079365079365e-04*w25*Jinv10-3.9682539682539683e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20+1.5873015873015873e-03*Jinv00*w21+2.3809523809523812e-03*Jinv00*w6-3.9682539682539683e-04*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+2.3809523809523812e-03*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv20*w23-3.9682539682539683e-04*w1*Jinv10+1.5873015873015873e-03*Jinv20*w5-3.9682539682539683e-04*Jinv00*w27-3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv00*w18+1.5873015873015873e-03*w4*Jinv10+7.9365079365079365e-04*Jinv20*w11+7.9365079365079365e-04*w19*Jinv10+7.9365079365079365e-04*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14)*D;
  A[7][8]=(0.0)*D;
  A[7][9]=(0.0)*D;
  A[7][10]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[7][11]=(0.0)*D;
  A[7][12]=(0.0)*D;
  A[7][13]=(-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+7.9365079365079365e-04*Jinv00*w0+4.7619047619047623e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10+4.7619047619047623e-03*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5+4.7619047619047623e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14)*D;
  A[7][14]=(0.0)*D;
  A[7][15]=(0.0)*D;
  A[7][16]=(7.9365079365079365e-04*Jinv01*w18-7.9365079365079365e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*w20*Jinv21-1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21-7.9365079365079365e-04*Jinv01*w9-7.9365079365079365e-04*w25*Jinv11+3.9682539682539683e-04*Jinv01*w27-7.9365079365079365e-04*w10*Jinv11-7.9365079365079365e-04*w26*Jinv21-7.9365079365079365e-04*w11*Jinv21-1.5873015873015873e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+3.9682539682539683e-04*Jinv11*w1+3.9682539682539683e-04*w28*Jinv11-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11-3.9682539682539683e-04*Jinv01*w15+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21+3.9682539682539683e-04*w2*Jinv21)*D;
  A[7][17]=(0.0)*D;
  A[7][18]=(0.0)*D;
  A[7][19]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00-7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[7][20]=(0.0)*D;
  A[7][21]=(0.0)*D;
  A[7][22]=(-1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+4.7619047619047623e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6+4.7619047619047623e-03*w8*Jinv22-1.5873015873015873e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20-1.5873015873015873e-03*Jinv20*w14+4.7619047619047623e-03*Jinv02*w6)*D;
  A[7][23]=(0.0)*D;
  A[7][24]=(0.0)*D;
  A[7][25]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22+1.5873015873015873e-03*w8*Jinv22+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+7.9365079365079365e-04*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24+7.9365079365079365e-04*w2*Jinv22-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w29*Jinv22+7.9365079365079365e-04*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[7][26]=(0.0)*D;
  A[7][27]=(0.0)*D;
  A[7][28]=(-7.9365079365079365e-04*w20*Jinv22+7.9365079365079365e-04*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*w19*Jinv12-3.9682539682539683e-04*Jinv02*w27-7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22+3.9682539682539683e-04*Jinv02*w15+7.9365079365079365e-04*w11*Jinv22-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w17*Jinv22+3.9682539682539683e-04*w2*Jinv22-1.5873015873015873e-03*Jinv02*w21+3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[7][29]=(0.0)*D;
  A[8][0]=(0.0)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+7.9365079365079365e-04*Jinv01*w18+7.9365079365079365e-04*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-7.9365079365079365e-04*w24*Jinv00+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w19*Jinv12-3.9682539682539683e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27+7.9365079365079365e-04*w20*Jinv21+7.9365079365079365e-04*w10*Jinv10+7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21+7.9365079365079365e-04*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2-7.9365079365079365e-04*w25*Jinv10-3.9682539682539683e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w25*Jinv11-3.9682539682539683e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w10*Jinv11-3.9682539682539683e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*Jinv01*w3-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*w26*Jinv21+7.9365079365079365e-04*w11*Jinv21-7.9365079365079365e-04*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-3.9682539682539683e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10+1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6-3.9682539682539683e-04*Jinv00*w27+7.9365079365079365e-04*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1-7.9365079365079365e-04*w26*Jinv22-3.9682539682539683e-04*w28*Jinv11-3.9682539682539683e-04*w17*Jinv22-3.9682539682539683e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+7.9365079365079365e-04*Jinv00*w18-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w2*Jinv22-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+7.9365079365079365e-04*Jinv20*w11+7.9365079365079365e-04*w19*Jinv10+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-7.9365079365079365e-04*w26*Jinv20-3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(0.0)*D;
  A[8][5]=(1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-4.7619047619047623e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-4.7619047619047623e-03*w7*Jinv11-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16-4.7619047619047623e-03*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11-3.1746031746031746e-03*Jinv00*w6-4.7619047619047623e-03*w8*Jinv22+1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-3.1746031746031746e-03*Jinv20*w5-4.7619047619047623e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w11*Jinv22+1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24+1.5873015873015873e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10-1.5873015873015873e-03*Jinv20*w14-4.7619047619047623e-03*Jinv02*w6)*D;
  A[8][6]=(0.0)*D;
  A[8][7]=(0.0)*D;
  A[8][8]=(2.3809523809523812e-03*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22-3.9682539682539683e-04*Jinv00*w15-3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w24*Jinv00+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w10*Jinv10-3.9682539682539683e-04*Jinv10*w16-3.9682539682539683e-04*Jinv20*w2+7.9365079365079365e-04*w25*Jinv10-3.9682539682539683e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20+1.5873015873015873e-03*Jinv00*w21+2.3809523809523812e-03*Jinv00*w6-3.9682539682539683e-04*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+2.3809523809523812e-03*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv20*w23-3.9682539682539683e-04*w1*Jinv10+1.5873015873015873e-03*Jinv20*w5-3.9682539682539683e-04*Jinv00*w27-3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv00*w18+1.5873015873015873e-03*w4*Jinv10+7.9365079365079365e-04*Jinv20*w11+7.9365079365079365e-04*w19*Jinv10+7.9365079365079365e-04*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14)*D;
  A[8][9]=(0.0)*D;
  A[8][10]=(0.0)*D;
  A[8][11]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1-7.9365079365079365e-04*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[8][12]=(0.0)*D;
  A[8][13]=(0.0)*D;
  A[8][14]=(-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+7.9365079365079365e-04*Jinv00*w0+4.7619047619047623e-03*w7*Jinv11-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10+4.7619047619047623e-03*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5+4.7619047619047623e-03*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27+7.9365079365079365e-04*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14)*D;
  A[8][15]=(0.0)*D;
  A[8][16]=(0.0)*D;
  A[8][17]=(7.9365079365079365e-04*Jinv01*w18-7.9365079365079365e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*w20*Jinv21-1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21-7.9365079365079365e-04*Jinv01*w9-7.9365079365079365e-04*w25*Jinv11+3.9682539682539683e-04*Jinv01*w27-7.9365079365079365e-04*w10*Jinv11-7.9365079365079365e-04*w26*Jinv21-7.9365079365079365e-04*w11*Jinv21-1.5873015873015873e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+3.9682539682539683e-04*Jinv11*w1+3.9682539682539683e-04*w28*Jinv11-7.9365079365079365e-04*Jinv01*w24+3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11-3.9682539682539683e-04*Jinv01*w15+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21+3.9682539682539683e-04*w2*Jinv21)*D;
  A[8][18]=(0.0)*D;
  A[8][19]=(0.0)*D;
  A[8][20]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00-7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[8][21]=(0.0)*D;
  A[8][22]=(0.0)*D;
  A[8][23]=(-1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+4.7619047619047623e-03*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6+4.7619047619047623e-03*w8*Jinv22-1.5873015873015873e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20-1.5873015873015873e-03*Jinv20*w14+4.7619047619047623e-03*Jinv02*w6)*D;
  A[8][24]=(0.0)*D;
  A[8][25]=(0.0)*D;
  A[8][26]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22+1.5873015873015873e-03*w8*Jinv22+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+7.9365079365079365e-04*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24+7.9365079365079365e-04*w2*Jinv22-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w29*Jinv22+7.9365079365079365e-04*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[8][27]=(0.0)*D;
  A[8][28]=(0.0)*D;
  A[8][29]=(-7.9365079365079365e-04*w20*Jinv22+7.9365079365079365e-04*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*w19*Jinv12-3.9682539682539683e-04*Jinv02*w27-7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1-7.9365079365079365e-04*w25*Jinv12-7.9365079365079365e-04*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22+3.9682539682539683e-04*Jinv02*w15+7.9365079365079365e-04*w11*Jinv22-7.9365079365079365e-04*w26*Jinv22+3.9682539682539683e-04*w17*Jinv22+3.9682539682539683e-04*w2*Jinv22-1.5873015873015873e-03*Jinv02*w21+3.9682539682539683e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[9][0]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-6.3492063492063492e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*w11*Jinv21-6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+1.5873015873015873e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+1.5873015873015873e-03*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[9][1]=(0.0)*D;
  A[9][2]=(0.0)*D;
  A[9][3]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.2698412698412698e-02*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21+1.2698412698412698e-02*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10-1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+1.2698412698412698e-02*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(0.0)*D;
  A[9][6]=(7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8+7.9365079365079365e-04*Jinv00*w27+7.9365079365079365e-04*Jinv20*w29-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20)*D;
  A[9][7]=(0.0)*D;
  A[9][8]=(0.0)*D;
  A[9][9]=(3.1746031746031746e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.9047619047619049e-02*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0-9.5238095238095247e-03*w24*Jinv00-1.9047619047619049e-02*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21-1.9047619047619049e-02*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv01*w12+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10-6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1+3.1746031746031746e-03*w28*Jinv10-9.5238095238095247e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-9.5238095238095247e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-9.5238095238095247e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-9.5238095238095247e-03*Jinv02*w24-1.9047619047619049e-02*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-1.9047619047619049e-02*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-9.5238095238095247e-03*w26*Jinv22+3.1746031746031746e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15-1.9047619047619049e-02*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-9.5238095238095247e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[9][10]=(0.0)*D;
  A[9][11]=(0.0)*D;
  A[9][12]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+9.5238095238095247e-03*w24*Jinv00+1.9047619047619049e-02*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+1.9047619047619049e-02*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10+6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+9.5238095238095247e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+9.5238095238095247e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+3.1746031746031746e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-3.1746031746031746e-03*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+1.9047619047619049e-02*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+9.5238095238095247e-03*w26*Jinv20+9.5238095238095247e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[9][13]=(0.0)*D;
  A[9][14]=(0.0)*D;
  A[9][15]=(-7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27+6.3492063492063492e-03*w10*Jinv11+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*w2*Jinv21)*D;
  A[9][16]=(0.0)*D;
  A[9][17]=(0.0)*D;
  A[9][18]=(-3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22+1.2698412698412698e-02*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21+1.2698412698412698e-02*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+1.2698412698412698e-02*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[9][19]=(0.0)*D;
  A[9][20]=(0.0)*D;
  A[9][21]=(6.3492063492063492e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10+6.3492063492063492e-03*Jinv12*w13+6.3492063492063492e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[9][22]=(0.0)*D;
  A[9][23]=(0.0)*D;
  A[9][24]=(3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+1.9047619047619049e-02*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+9.5238095238095247e-03*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+9.5238095238095247e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv02*w24+1.9047619047619049e-02*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+1.9047619047619049e-02*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+9.5238095238095247e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+9.5238095238095247e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[9][25]=(0.0)*D;
  A[9][26]=(0.0)*D;
  A[9][27]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*w8*Jinv22-3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w10*Jinv12-6.3492063492063492e-03*w11*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[9][28]=(0.0)*D;
  A[9][29]=(0.0)*D;
  A[10][0]=(0.0)*D;
  A[10][1]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-6.3492063492063492e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*w11*Jinv21-6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+1.5873015873015873e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+1.5873015873015873e-03*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[10][2]=(0.0)*D;
  A[10][3]=(0.0)*D;
  A[10][4]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.2698412698412698e-02*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21+1.2698412698412698e-02*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10-1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+1.2698412698412698e-02*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(0.0)*D;
  A[10][7]=(7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8+7.9365079365079365e-04*Jinv00*w27+7.9365079365079365e-04*Jinv20*w29-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20)*D;
  A[10][8]=(0.0)*D;
  A[10][9]=(0.0)*D;
  A[10][10]=(3.1746031746031746e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.9047619047619049e-02*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0-9.5238095238095247e-03*w24*Jinv00-1.9047619047619049e-02*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21-1.9047619047619049e-02*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv01*w12+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10-6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1+3.1746031746031746e-03*w28*Jinv10-9.5238095238095247e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-9.5238095238095247e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-9.5238095238095247e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-9.5238095238095247e-03*Jinv02*w24-1.9047619047619049e-02*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-1.9047619047619049e-02*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-9.5238095238095247e-03*w26*Jinv22+3.1746031746031746e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15-1.9047619047619049e-02*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-9.5238095238095247e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[10][11]=(0.0)*D;
  A[10][12]=(0.0)*D;
  A[10][13]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+9.5238095238095247e-03*w24*Jinv00+1.9047619047619049e-02*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+1.9047619047619049e-02*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10+6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+9.5238095238095247e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+9.5238095238095247e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+3.1746031746031746e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-3.1746031746031746e-03*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+1.9047619047619049e-02*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+9.5238095238095247e-03*w26*Jinv20+9.5238095238095247e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[10][14]=(0.0)*D;
  A[10][15]=(0.0)*D;
  A[10][16]=(-7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27+6.3492063492063492e-03*w10*Jinv11+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*w2*Jinv21)*D;
  A[10][17]=(0.0)*D;
  A[10][18]=(0.0)*D;
  A[10][19]=(-3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22+1.2698412698412698e-02*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21+1.2698412698412698e-02*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+1.2698412698412698e-02*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[10][20]=(0.0)*D;
  A[10][21]=(0.0)*D;
  A[10][22]=(6.3492063492063492e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10+6.3492063492063492e-03*Jinv12*w13+6.3492063492063492e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[10][23]=(0.0)*D;
  A[10][24]=(0.0)*D;
  A[10][25]=(3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+1.9047619047619049e-02*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+9.5238095238095247e-03*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+9.5238095238095247e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv02*w24+1.9047619047619049e-02*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+1.9047619047619049e-02*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+9.5238095238095247e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+9.5238095238095247e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[10][26]=(0.0)*D;
  A[10][27]=(0.0)*D;
  A[10][28]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*w8*Jinv22-3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w10*Jinv12-6.3492063492063492e-03*w11*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[10][29]=(0.0)*D;
  A[11][0]=(0.0)*D;
  A[11][1]=(0.0)*D;
  A[11][2]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-6.3492063492063492e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*w11*Jinv21-6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+1.5873015873015873e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+1.5873015873015873e-03*Jinv12*w16+7.9365079365079365e-04*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[11][3]=(0.0)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.2698412698412698e-02*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21+1.2698412698412698e-02*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10-1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+1.2698412698412698e-02*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(0.0)*D;
  A[11][8]=(7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8+7.9365079365079365e-04*Jinv00*w27+7.9365079365079365e-04*Jinv20*w29-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20)*D;
  A[11][9]=(0.0)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(3.1746031746031746e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.9047619047619049e-02*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0-9.5238095238095247e-03*w24*Jinv00-1.9047619047619049e-02*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21-1.9047619047619049e-02*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv01*w12+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10-6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1+3.1746031746031746e-03*w28*Jinv10-9.5238095238095247e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-9.5238095238095247e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-9.5238095238095247e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-9.5238095238095247e-03*Jinv02*w24-1.9047619047619049e-02*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-1.9047619047619049e-02*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-9.5238095238095247e-03*w26*Jinv22+3.1746031746031746e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15-1.9047619047619049e-02*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-9.5238095238095247e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[11][12]=(0.0)*D;
  A[11][13]=(0.0)*D;
  A[11][14]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+9.5238095238095247e-03*w24*Jinv00+1.9047619047619049e-02*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+1.9047619047619049e-02*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10+6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+9.5238095238095247e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+9.5238095238095247e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+3.1746031746031746e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-3.1746031746031746e-03*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+1.9047619047619049e-02*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+9.5238095238095247e-03*w26*Jinv20+9.5238095238095247e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[11][15]=(0.0)*D;
  A[11][16]=(0.0)*D;
  A[11][17]=(-7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27+6.3492063492063492e-03*w10*Jinv11+3.1746031746031746e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*w2*Jinv21)*D;
  A[11][18]=(0.0)*D;
  A[11][19]=(0.0)*D;
  A[11][20]=(-3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22+1.2698412698412698e-02*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21+1.2698412698412698e-02*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+1.2698412698412698e-02*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[11][21]=(0.0)*D;
  A[11][22]=(0.0)*D;
  A[11][23]=(6.3492063492063492e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*w19*Jinv10+6.3492063492063492e-03*Jinv12*w13+6.3492063492063492e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[11][24]=(0.0)*D;
  A[11][25]=(0.0)*D;
  A[11][26]=(3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+1.9047619047619049e-02*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+9.5238095238095247e-03*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+9.5238095238095247e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv02*w24+1.9047619047619049e-02*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+1.9047619047619049e-02*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+9.5238095238095247e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+9.5238095238095247e-03*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[11][27]=(0.0)*D;
  A[11][28]=(0.0)*D;
  A[11][29]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*w8*Jinv22-3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w10*Jinv12-6.3492063492063492e-03*w11*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[12][0]=(6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00-7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*Jinv12*w1-7.9365079365079365e-04*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21-7.9365079365079365e-04*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+3.1746031746031746e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[12][1]=(0.0)*D;
  A[12][2]=(0.0)*D;
  A[12][3]=(-9.5238095238095247e-03*w5*Jinv21-9.5238095238095247e-03*w4*Jinv12-1.9047619047619049e-02*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-9.5238095238095247e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-1.9047619047619049e-02*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-9.5238095238095247e-03*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-1.2698412698412698e-02*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8-1.9047619047619049e-02*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-1.2698412698412698e-02*w13*Jinv10-9.5238095238095247e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-9.5238095238095247e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-1.9047619047619049e-02*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-1.9047619047619049e-02*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-9.5238095238095247e-03*Jinv02*w3-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-9.5238095238095247e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-9.5238095238095247e-03*Jinv02*w21-1.9047619047619049e-02*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv12*w16-1.2698412698412698e-02*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[12][4]=(0.0)*D;
  A[12][5]=(0.0)*D;
  A[12][6]=(1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w14)*D;
  A[12][7]=(0.0)*D;
  A[12][8]=(0.0)*D;
  A[12][9]=(-6.3492063492063492e-03*w4*Jinv12-1.9047619047619049e-02*Jinv02*w12-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-9.5238095238095247e-03*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-9.5238095238095247e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-9.5238095238095247e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.2698412698412698e-02*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w28*Jinv10-1.9047619047619049e-02*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-1.9047619047619049e-02*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-1.9047619047619049e-02*w13*Jinv10-6.3492063492063492e-03*Jinv20*w23-9.5238095238095247e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-9.5238095238095247e-03*Jinv02*w24-9.5238095238095247e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-1.2698412698412698e-02*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*Jinv00*w27-1.2698412698412698e-02*w13*Jinv11-9.5238095238095247e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-9.5238095238095247e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+3.1746031746031746e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w4*Jinv10-1.5873015873015873e-03*Jinv01*w15-9.5238095238095247e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-1.9047619047619049e-02*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-1.9047619047619049e-02*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[12][10]=(0.0)*D;
  A[12][11]=(0.0)*D;
  A[12][12]=(9.5238095238095247e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+9.5238095238095247e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+9.5238095238095247e-03*w10*Jinv10+9.5238095238095247e-03*w23*Jinv21+1.9047619047619049e-02*Jinv01*w12+6.3492063492063492e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*w20*Jinv20+9.5238095238095247e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+1.9047619047619049e-02*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+1.9047619047619049e-02*w13*Jinv10+9.5238095238095247e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+1.9047619047619049e-02*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-3.1746031746031746e-03*Jinv00*w27+1.9047619047619049e-02*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+9.5238095238095247e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+9.5238095238095247e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+9.5238095238095247e-03*w26*Jinv20+1.9047619047619049e-02*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[12][13]=(0.0)*D;
  A[12][14]=(0.0)*D;
  A[12][15]=(-1.5873015873015873e-03*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27+3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[12][16]=(0.0)*D;
  A[12][17]=(0.0)*D;
  A[12][18]=(-3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-3.1746031746031746e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w2*Jinv21)*D;
  A[12][19]=(0.0)*D;
  A[12][20]=(0.0)*D;
  A[12][21]=(9.5238095238095247e-03*w4*Jinv12+1.9047619047619049e-02*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+9.5238095238095247e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*Jinv00*w21+9.5238095238095247e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+1.9047619047619049e-02*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+9.5238095238095247e-03*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+9.5238095238095247e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+9.5238095238095247e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10+1.9047619047619049e-02*Jinv12*w13+6.3492063492063492e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[12][22]=(0.0)*D;
  A[12][23]=(0.0)*D;
  A[12][24]=(3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+1.9047619047619049e-02*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+9.5238095238095247e-03*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+6.3492063492063492e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.9047619047619049e-02*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+9.5238095238095247e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+9.5238095238095247e-03*Jinv02*w24+9.5238095238095247e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21+6.3492063492063492e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+9.5238095238095247e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+9.5238095238095247e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+1.9047619047619049e-02*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[12][25]=(0.0)*D;
  A[12][26]=(0.0)*D;
  A[12][27]=(-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv02*w9-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv12*w1-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22-6.3492063492063492e-03*Jinv12*w13+7.9365079365079365e-04*w29*Jinv22)*D;
  A[12][28]=(0.0)*D;
  A[12][29]=(0.0)*D;
  A[13][0]=(0.0)*D;
  A[13][1]=(6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00-7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*Jinv12*w1-7.9365079365079365e-04*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21-7.9365079365079365e-04*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+3.1746031746031746e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[13][2]=(0.0)*D;
  A[13][3]=(0.0)*D;
  A[13][4]=(-9.5238095238095247e-03*w5*Jinv21-9.5238095238095247e-03*w4*Jinv12-1.9047619047619049e-02*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-9.5238095238095247e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-1.9047619047619049e-02*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-9.5238095238095247e-03*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-1.2698412698412698e-02*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8-1.9047619047619049e-02*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-1.2698412698412698e-02*w13*Jinv10-9.5238095238095247e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-9.5238095238095247e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-1.9047619047619049e-02*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-1.9047619047619049e-02*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-9.5238095238095247e-03*Jinv02*w3-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-9.5238095238095247e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-9.5238095238095247e-03*Jinv02*w21-1.9047619047619049e-02*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv12*w16-1.2698412698412698e-02*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[13][5]=(0.0)*D;
  A[13][6]=(0.0)*D;
  A[13][7]=(1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w14)*D;
  A[13][8]=(0.0)*D;
  A[13][9]=(0.0)*D;
  A[13][10]=(-6.3492063492063492e-03*w4*Jinv12-1.9047619047619049e-02*Jinv02*w12-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-9.5238095238095247e-03*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-9.5238095238095247e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-9.5238095238095247e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.2698412698412698e-02*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w28*Jinv10-1.9047619047619049e-02*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-1.9047619047619049e-02*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-1.9047619047619049e-02*w13*Jinv10-6.3492063492063492e-03*Jinv20*w23-9.5238095238095247e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-9.5238095238095247e-03*Jinv02*w24-9.5238095238095247e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-1.2698412698412698e-02*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*Jinv00*w27-1.2698412698412698e-02*w13*Jinv11-9.5238095238095247e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-9.5238095238095247e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+3.1746031746031746e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w4*Jinv10-1.5873015873015873e-03*Jinv01*w15-9.5238095238095247e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-1.9047619047619049e-02*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-1.9047619047619049e-02*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[13][11]=(0.0)*D;
  A[13][12]=(0.0)*D;
  A[13][13]=(9.5238095238095247e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+9.5238095238095247e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+9.5238095238095247e-03*w10*Jinv10+9.5238095238095247e-03*w23*Jinv21+1.9047619047619049e-02*Jinv01*w12+6.3492063492063492e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*w20*Jinv20+9.5238095238095247e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+1.9047619047619049e-02*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+1.9047619047619049e-02*w13*Jinv10+9.5238095238095247e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+1.9047619047619049e-02*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-3.1746031746031746e-03*Jinv00*w27+1.9047619047619049e-02*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+9.5238095238095247e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+9.5238095238095247e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+9.5238095238095247e-03*w26*Jinv20+1.9047619047619049e-02*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[13][14]=(0.0)*D;
  A[13][15]=(0.0)*D;
  A[13][16]=(-1.5873015873015873e-03*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27+3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[13][17]=(0.0)*D;
  A[13][18]=(0.0)*D;
  A[13][19]=(-3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-3.1746031746031746e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w2*Jinv21)*D;
  A[13][20]=(0.0)*D;
  A[13][21]=(0.0)*D;
  A[13][22]=(9.5238095238095247e-03*w4*Jinv12+1.9047619047619049e-02*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+9.5238095238095247e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*Jinv00*w21+9.5238095238095247e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+1.9047619047619049e-02*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+9.5238095238095247e-03*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+9.5238095238095247e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+9.5238095238095247e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10+1.9047619047619049e-02*Jinv12*w13+6.3492063492063492e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[13][23]=(0.0)*D;
  A[13][24]=(0.0)*D;
  A[13][25]=(3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+1.9047619047619049e-02*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+9.5238095238095247e-03*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+6.3492063492063492e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.9047619047619049e-02*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+9.5238095238095247e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+9.5238095238095247e-03*Jinv02*w24+9.5238095238095247e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21+6.3492063492063492e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+9.5238095238095247e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+9.5238095238095247e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+1.9047619047619049e-02*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[13][26]=(0.0)*D;
  A[13][27]=(0.0)*D;
  A[13][28]=(-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv02*w9-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv12*w1-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22-6.3492063492063492e-03*Jinv12*w13+7.9365079365079365e-04*w29*Jinv22)*D;
  A[13][29]=(0.0)*D;
  A[14][0]=(0.0)*D;
  A[14][1]=(0.0)*D;
  A[14][2]=(6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00-7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*Jinv12*w1-7.9365079365079365e-04*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21-7.9365079365079365e-04*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+3.1746031746031746e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+3.1746031746031746e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[14][3]=(0.0)*D;
  A[14][4]=(0.0)*D;
  A[14][5]=(-9.5238095238095247e-03*w5*Jinv21-9.5238095238095247e-03*w4*Jinv12-1.9047619047619049e-02*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-9.5238095238095247e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-1.9047619047619049e-02*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16-6.3492063492063492e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-9.5238095238095247e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-9.5238095238095247e-03*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-1.2698412698412698e-02*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8-1.9047619047619049e-02*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-1.2698412698412698e-02*w13*Jinv10-9.5238095238095247e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-9.5238095238095247e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-1.9047619047619049e-02*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-1.9047619047619049e-02*w13*Jinv11-6.3492063492063492e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-9.5238095238095247e-03*Jinv02*w3-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-9.5238095238095247e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-9.5238095238095247e-03*Jinv02*w21-1.9047619047619049e-02*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv12*w16-1.2698412698412698e-02*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[14][6]=(0.0)*D;
  A[14][7]=(0.0)*D;
  A[14][8]=(1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w14)*D;
  A[14][9]=(0.0)*D;
  A[14][10]=(0.0)*D;
  A[14][11]=(-6.3492063492063492e-03*w4*Jinv12-1.9047619047619049e-02*Jinv02*w12-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-9.5238095238095247e-03*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-9.5238095238095247e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-9.5238095238095247e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.2698412698412698e-02*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w28*Jinv10-1.9047619047619049e-02*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-1.9047619047619049e-02*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-1.9047619047619049e-02*w13*Jinv10-6.3492063492063492e-03*Jinv20*w23-9.5238095238095247e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-9.5238095238095247e-03*Jinv02*w24-9.5238095238095247e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-1.2698412698412698e-02*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*Jinv00*w27-1.2698412698412698e-02*w13*Jinv11-9.5238095238095247e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-9.5238095238095247e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+3.1746031746031746e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w4*Jinv10-1.5873015873015873e-03*Jinv01*w15-9.5238095238095247e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-1.9047619047619049e-02*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-1.9047619047619049e-02*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[14][12]=(0.0)*D;
  A[14][13]=(0.0)*D;
  A[14][14]=(9.5238095238095247e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+9.5238095238095247e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+9.5238095238095247e-03*w10*Jinv10+9.5238095238095247e-03*w23*Jinv21+1.9047619047619049e-02*Jinv01*w12+6.3492063492063492e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*w20*Jinv20+9.5238095238095247e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+1.9047619047619049e-02*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w3*Jinv00+1.9047619047619049e-02*w13*Jinv10+9.5238095238095247e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+9.5238095238095247e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+1.9047619047619049e-02*w14*Jinv21+6.3492063492063492e-03*Jinv20*w5-3.1746031746031746e-03*Jinv00*w27+1.9047619047619049e-02*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv10+9.5238095238095247e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+9.5238095238095247e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+9.5238095238095247e-03*w26*Jinv20+1.9047619047619049e-02*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[14][15]=(0.0)*D;
  A[14][16]=(0.0)*D;
  A[14][17]=(-1.5873015873015873e-03*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27+3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[14][18]=(0.0)*D;
  A[14][19]=(0.0)*D;
  A[14][20]=(-3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22-6.3492063492063492e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-3.1746031746031746e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w2*Jinv21)*D;
  A[14][21]=(0.0)*D;
  A[14][22]=(0.0)*D;
  A[14][23]=(9.5238095238095247e-03*w4*Jinv12+1.9047619047619049e-02*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+9.5238095238095247e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*Jinv00*w21+9.5238095238095247e-03*w23*Jinv22-1.5873015873015873e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+1.9047619047619049e-02*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+9.5238095238095247e-03*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+6.3492063492063492e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29+9.5238095238095247e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+9.5238095238095247e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10+1.9047619047619049e-02*Jinv12*w13+6.3492063492063492e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[14][24]=(0.0)*D;
  A[14][25]=(0.0)*D;
  A[14][26]=(3.1746031746031746e-03*w5*Jinv21+6.3492063492063492e-03*w4*Jinv12+1.9047619047619049e-02*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+9.5238095238095247e-03*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+6.3492063492063492e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+6.3492063492063492e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.9047619047619049e-02*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+9.5238095238095247e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+9.5238095238095247e-03*Jinv02*w24+9.5238095238095247e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21+6.3492063492063492e-03*w14*Jinv21-1.5873015873015873e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+9.5238095238095247e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+9.5238095238095247e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv02*w3+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+1.9047619047619049e-02*Jinv12*w13-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[14][27]=(0.0)*D;
  A[14][28]=(0.0)*D;
  A[14][29]=(-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv02*w9-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv12*w1-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22-6.3492063492063492e-03*Jinv12*w13+7.9365079365079365e-04*w29*Jinv22)*D;
  A[15][0]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*w4*Jinv12+7.9365079365079365e-04*Jinv01*w18+7.9365079365079365e-04*w20*Jinv22-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-3.9682539682539683e-04*w7*Jinv11+1.5873015873015873e-03*w9*Jinv00+7.9365079365079365e-04*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+7.9365079365079365e-04*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27+7.9365079365079365e-04*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+7.9365079365079365e-04*Jinv02*w18-7.9365079365079365e-04*w23*Jinv21+7.9365079365079365e-04*Jinv10*w16-3.9682539682539683e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-7.9365079365079365e-04*Jinv11*w22-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*Jinv00*w21-7.9365079365079365e-04*w23*Jinv22-3.9682539682539683e-04*Jinv01*w27-3.9682539682539683e-04*Jinv00*w6-3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11-3.9682539682539683e-04*w28*Jinv10-3.9682539682539683e-04*Jinv20*w8+7.9365079365079365e-04*w3*Jinv00+7.9365079365079365e-04*Jinv01*w3-7.9365079365079365e-04*Jinv20*w23+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv01*w21+7.9365079365079365e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10+7.9365079365079365e-04*Jinv20*w5-3.9682539682539683e-04*Jinv01*w6-3.9682539682539683e-04*Jinv00*w27+1.5873015873015873e-03*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1-3.9682539682539683e-04*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22-3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv02*w3+7.9365079365079365e-04*Jinv00*w18+3.9682539682539683e-04*w2*Jinv22-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11-7.9365079365079365e-04*Jinv02*w21+7.9365079365079365e-04*w19*Jinv10+3.9682539682539683e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21-3.9682539682539683e-04*Jinv02*w6)*D;
  A[15][1]=(0.0)*D;
  A[15][2]=(0.0)*D;
  A[15][3]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[15][4]=(0.0)*D;
  A[15][5]=(0.0)*D;
  A[15][6]=(-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16+3.9682539682539683e-04*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-7.9365079365079365e-04*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-7.9365079365079365e-04*Jinv20*w23+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*Jinv20*w5+3.9682539682539683e-04*Jinv00*w27+3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv00*w18-7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w19*Jinv10-1.5873015873015873e-03*Jinv20*w14)*D;
  A[15][7]=(0.0)*D;
  A[15][8]=(0.0)*D;
  A[15][9]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*Jinv10*w22-4.7619047619047623e-03*Jinv00*w15+7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-1.5873015873015873e-03*w20*Jinv21+1.5873015873015873e-03*Jinv02*w18-1.5873015873015873e-03*Jinv01*w12-4.7619047619047623e-03*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-4.7619047619047623e-03*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22+7.9365079365079365e-04*Jinv01*w27-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21+1.5873015873015873e-03*Jinv12*w22-4.7619047619047623e-03*Jinv02*w15-1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+7.9365079365079365e-04*w28*Jinv11-4.7619047619047623e-03*w17*Jinv22+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10-3.1746031746031746e-03*w17*Jinv21-4.7619047619047623e-03*Jinv12*w16)*D;
  A[15][10]=(0.0)*D;
  A[15][11]=(0.0)*D;
  A[15][12]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18-1.5873015873015873e-03*Jinv10*w22+4.7619047619047623e-03*Jinv00*w15+1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+4.7619047619047623e-03*Jinv10*w16-1.5873015873015873e-03*Jinv01*w9+4.7619047619047623e-03*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+7.9365079365079365e-04*Jinv01*w27-1.5873015873015873e-03*w10*Jinv11-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5-3.1746031746031746e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*w19*Jinv10+7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+7.9365079365079365e-04*w2*Jinv21)*D;
  A[15][13]=(0.0)*D;
  A[15][14]=(0.0)*D;
  A[15][15]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*Jinv01*w18-3.9682539682539683e-04*w7*Jinv11+2.3809523809523812e-03*Jinv11*w16+7.9365079365079365e-04*w20*Jinv21+7.9365079365079365e-04*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-3.9682539682539683e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv11*w22+1.5873015873015873e-03*w25*Jinv11-3.9682539682539683e-04*Jinv01*w27+1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*Jinv01*w3+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*Jinv01*w21+1.5873015873015873e-03*w14*Jinv21-3.9682539682539683e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-3.9682539682539683e-04*Jinv11*w1-3.9682539682539683e-04*w28*Jinv11+1.5873015873015873e-03*Jinv01*w24-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+7.9365079365079365e-04*w4*Jinv11+2.3809523809523812e-03*Jinv01*w15-3.9682539682539683e-04*Jinv01*w0+2.3809523809523812e-03*w17*Jinv21-3.9682539682539683e-04*w2*Jinv21)*D;
  A[15][16]=(0.0)*D;
  A[15][17]=(0.0)*D;
  A[15][18]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16-7.9365079365079365e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-7.9365079365079365e-04*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[15][19]=(0.0)*D;
  A[15][20]=(0.0)*D;
  A[15][21]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv10*w16+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+1.5873015873015873e-03*Jinv02*w15+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22)*D;
  A[15][22]=(0.0)*D;
  A[15][23]=(0.0)*D;
  A[15][24]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+7.9365079365079365e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*w10*Jinv11-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+4.7619047619047623e-03*Jinv02*w15-1.5873015873015873e-03*w14*Jinv21+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+4.7619047619047623e-03*w17*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+4.7619047619047623e-03*Jinv12*w16+7.9365079365079365e-04*w2*Jinv21)*D;
  A[15][25]=(0.0)*D;
  A[15][26]=(0.0)*D;
  A[15][27]=(7.9365079365079365e-04*w4*Jinv12-7.9365079365079365e-04*w20*Jinv22+3.9682539682539683e-04*Jinv12*w7-7.9365079365079365e-04*w19*Jinv12+7.9365079365079365e-04*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27-7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24+3.9682539682539683e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+7.9365079365079365e-04*Jinv02*w3+3.9682539682539683e-04*w2*Jinv22-7.9365079365079365e-04*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*Jinv02*w6)*D;
  A[15][28]=(0.0)*D;
  A[15][29]=(0.0)*D;
  A[16][0]=(0.0)*D;
  A[16][1]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*w4*Jinv12+7.9365079365079365e-04*Jinv01*w18+7.9365079365079365e-04*w20*Jinv22-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-3.9682539682539683e-04*w7*Jinv11+1.5873015873015873e-03*w9*Jinv00+7.9365079365079365e-04*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+7.9365079365079365e-04*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27+7.9365079365079365e-04*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+7.9365079365079365e-04*Jinv02*w18-7.9365079365079365e-04*w23*Jinv21+7.9365079365079365e-04*Jinv10*w16-3.9682539682539683e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-7.9365079365079365e-04*Jinv11*w22-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*Jinv00*w21-7.9365079365079365e-04*w23*Jinv22-3.9682539682539683e-04*Jinv01*w27-3.9682539682539683e-04*Jinv00*w6-3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11-3.9682539682539683e-04*w28*Jinv10-3.9682539682539683e-04*Jinv20*w8+7.9365079365079365e-04*w3*Jinv00+7.9365079365079365e-04*Jinv01*w3-7.9365079365079365e-04*Jinv20*w23+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv01*w21+7.9365079365079365e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10+7.9365079365079365e-04*Jinv20*w5-3.9682539682539683e-04*Jinv01*w6-3.9682539682539683e-04*Jinv00*w27+1.5873015873015873e-03*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1-3.9682539682539683e-04*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22-3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv02*w3+7.9365079365079365e-04*Jinv00*w18+3.9682539682539683e-04*w2*Jinv22-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11-7.9365079365079365e-04*Jinv02*w21+7.9365079365079365e-04*w19*Jinv10+3.9682539682539683e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21-3.9682539682539683e-04*Jinv02*w6)*D;
  A[16][2]=(0.0)*D;
  A[16][3]=(0.0)*D;
  A[16][4]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[16][5]=(0.0)*D;
  A[16][6]=(0.0)*D;
  A[16][7]=(-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16+3.9682539682539683e-04*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-7.9365079365079365e-04*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-7.9365079365079365e-04*Jinv20*w23+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*Jinv20*w5+3.9682539682539683e-04*Jinv00*w27+3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv00*w18-7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w19*Jinv10-1.5873015873015873e-03*Jinv20*w14)*D;
  A[16][8]=(0.0)*D;
  A[16][9]=(0.0)*D;
  A[16][10]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*Jinv10*w22-4.7619047619047623e-03*Jinv00*w15+7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-1.5873015873015873e-03*w20*Jinv21+1.5873015873015873e-03*Jinv02*w18-1.5873015873015873e-03*Jinv01*w12-4.7619047619047623e-03*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-4.7619047619047623e-03*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22+7.9365079365079365e-04*Jinv01*w27-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21+1.5873015873015873e-03*Jinv12*w22-4.7619047619047623e-03*Jinv02*w15-1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+7.9365079365079365e-04*w28*Jinv11-4.7619047619047623e-03*w17*Jinv22+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10-3.1746031746031746e-03*w17*Jinv21-4.7619047619047623e-03*Jinv12*w16)*D;
  A[16][11]=(0.0)*D;
  A[16][12]=(0.0)*D;
  A[16][13]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18-1.5873015873015873e-03*Jinv10*w22+4.7619047619047623e-03*Jinv00*w15+1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+4.7619047619047623e-03*Jinv10*w16-1.5873015873015873e-03*Jinv01*w9+4.7619047619047623e-03*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+7.9365079365079365e-04*Jinv01*w27-1.5873015873015873e-03*w10*Jinv11-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5-3.1746031746031746e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*w19*Jinv10+7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+7.9365079365079365e-04*w2*Jinv21)*D;
  A[16][14]=(0.0)*D;
  A[16][15]=(0.0)*D;
  A[16][16]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*Jinv01*w18-3.9682539682539683e-04*w7*Jinv11+2.3809523809523812e-03*Jinv11*w16+7.9365079365079365e-04*w20*Jinv21+7.9365079365079365e-04*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-3.9682539682539683e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv11*w22+1.5873015873015873e-03*w25*Jinv11-3.9682539682539683e-04*Jinv01*w27+1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*Jinv01*w3+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*Jinv01*w21+1.5873015873015873e-03*w14*Jinv21-3.9682539682539683e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-3.9682539682539683e-04*Jinv11*w1-3.9682539682539683e-04*w28*Jinv11+1.5873015873015873e-03*Jinv01*w24-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+7.9365079365079365e-04*w4*Jinv11+2.3809523809523812e-03*Jinv01*w15-3.9682539682539683e-04*Jinv01*w0+2.3809523809523812e-03*w17*Jinv21-3.9682539682539683e-04*w2*Jinv21)*D;
  A[16][17]=(0.0)*D;
  A[16][18]=(0.0)*D;
  A[16][19]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16-7.9365079365079365e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-7.9365079365079365e-04*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[16][20]=(0.0)*D;
  A[16][21]=(0.0)*D;
  A[16][22]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv10*w16+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+1.5873015873015873e-03*Jinv02*w15+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22)*D;
  A[16][23]=(0.0)*D;
  A[16][24]=(0.0)*D;
  A[16][25]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+7.9365079365079365e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*w10*Jinv11-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+4.7619047619047623e-03*Jinv02*w15-1.5873015873015873e-03*w14*Jinv21+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+4.7619047619047623e-03*w17*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+4.7619047619047623e-03*Jinv12*w16+7.9365079365079365e-04*w2*Jinv21)*D;
  A[16][26]=(0.0)*D;
  A[16][27]=(0.0)*D;
  A[16][28]=(7.9365079365079365e-04*w4*Jinv12-7.9365079365079365e-04*w20*Jinv22+3.9682539682539683e-04*Jinv12*w7-7.9365079365079365e-04*w19*Jinv12+7.9365079365079365e-04*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27-7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24+3.9682539682539683e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+7.9365079365079365e-04*Jinv02*w3+3.9682539682539683e-04*w2*Jinv22-7.9365079365079365e-04*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*Jinv02*w6)*D;
  A[16][29]=(0.0)*D;
  A[17][0]=(0.0)*D;
  A[17][1]=(0.0)*D;
  A[17][2]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*w4*Jinv12+7.9365079365079365e-04*Jinv01*w18+7.9365079365079365e-04*w20*Jinv22-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-3.9682539682539683e-04*w7*Jinv11+1.5873015873015873e-03*w9*Jinv00+7.9365079365079365e-04*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+7.9365079365079365e-04*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27+7.9365079365079365e-04*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+7.9365079365079365e-04*Jinv02*w18-7.9365079365079365e-04*w23*Jinv21+7.9365079365079365e-04*Jinv10*w16-3.9682539682539683e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-7.9365079365079365e-04*Jinv11*w22-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*Jinv00*w21-7.9365079365079365e-04*w23*Jinv22-3.9682539682539683e-04*Jinv01*w27-3.9682539682539683e-04*Jinv00*w6-3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11-3.9682539682539683e-04*w28*Jinv10-3.9682539682539683e-04*Jinv20*w8+7.9365079365079365e-04*w3*Jinv00+7.9365079365079365e-04*Jinv01*w3-7.9365079365079365e-04*Jinv20*w23+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv01*w21+7.9365079365079365e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10+7.9365079365079365e-04*Jinv20*w5-3.9682539682539683e-04*Jinv01*w6-3.9682539682539683e-04*Jinv00*w27+1.5873015873015873e-03*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1-3.9682539682539683e-04*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22-3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv02*w3+7.9365079365079365e-04*Jinv00*w18+3.9682539682539683e-04*w2*Jinv22-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11-7.9365079365079365e-04*Jinv02*w21+7.9365079365079365e-04*w19*Jinv10+3.9682539682539683e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21-3.9682539682539683e-04*Jinv02*w6)*D;
  A[17][3]=(0.0)*D;
  A[17][4]=(0.0)*D;
  A[17][5]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-7.9365079365079365e-04*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[17][6]=(0.0)*D;
  A[17][7]=(0.0)*D;
  A[17][8]=(-3.9682539682539683e-04*w7*Jinv10-7.9365079365079365e-04*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16+3.9682539682539683e-04*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+7.9365079365079365e-04*w20*Jinv20-7.9365079365079365e-04*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6+3.9682539682539683e-04*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-7.9365079365079365e-04*Jinv20*w23+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*Jinv20*w5+3.9682539682539683e-04*Jinv00*w27+3.9682539682539683e-04*Jinv20*w29+7.9365079365079365e-04*Jinv00*w18-7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w19*Jinv10-1.5873015873015873e-03*Jinv20*w14)*D;
  A[17][9]=(0.0)*D;
  A[17][10]=(0.0)*D;
  A[17][11]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+1.5873015873015873e-03*Jinv10*w22-4.7619047619047623e-03*Jinv00*w15+7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-1.5873015873015873e-03*w20*Jinv21+1.5873015873015873e-03*Jinv02*w18-1.5873015873015873e-03*Jinv01*w12-4.7619047619047623e-03*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-4.7619047619047623e-03*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22+7.9365079365079365e-04*Jinv01*w27-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21+1.5873015873015873e-03*Jinv12*w22-4.7619047619047623e-03*Jinv02*w15-1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+7.9365079365079365e-04*w28*Jinv11-4.7619047619047623e-03*w17*Jinv22+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10-3.1746031746031746e-03*w17*Jinv21-4.7619047619047623e-03*Jinv12*w16)*D;
  A[17][12]=(0.0)*D;
  A[17][13]=(0.0)*D;
  A[17][14]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18-1.5873015873015873e-03*Jinv10*w22+4.7619047619047623e-03*Jinv00*w15+1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+4.7619047619047623e-03*Jinv10*w16-1.5873015873015873e-03*Jinv01*w9+4.7619047619047623e-03*Jinv20*w17-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+7.9365079365079365e-04*Jinv01*w27-1.5873015873015873e-03*w10*Jinv11-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5-3.1746031746031746e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+7.9365079365079365e-04*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*w19*Jinv10+7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+7.9365079365079365e-04*w2*Jinv21)*D;
  A[17][15]=(0.0)*D;
  A[17][16]=(0.0)*D;
  A[17][17]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*Jinv01*w18-3.9682539682539683e-04*w7*Jinv11+2.3809523809523812e-03*Jinv11*w16+7.9365079365079365e-04*w20*Jinv21+7.9365079365079365e-04*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-3.9682539682539683e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv11*w22+1.5873015873015873e-03*w25*Jinv11-3.9682539682539683e-04*Jinv01*w27+1.5873015873015873e-03*w10*Jinv11+7.9365079365079365e-04*Jinv01*w3+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+7.9365079365079365e-04*Jinv01*w21+1.5873015873015873e-03*w14*Jinv21-3.9682539682539683e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-3.9682539682539683e-04*Jinv11*w1-3.9682539682539683e-04*w28*Jinv11+1.5873015873015873e-03*Jinv01*w24-3.9682539682539683e-04*w29*Jinv21+7.9365079365079365e-04*w19*Jinv11+7.9365079365079365e-04*w4*Jinv11+2.3809523809523812e-03*Jinv01*w15-3.9682539682539683e-04*Jinv01*w0+2.3809523809523812e-03*w17*Jinv21-3.9682539682539683e-04*w2*Jinv21)*D;
  A[17][18]=(0.0)*D;
  A[17][19]=(0.0)*D;
  A[17][20]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv01*w18-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9-1.5873015873015873e-03*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21+1.5873015873015873e-03*w10*Jinv10+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16-7.9365079365079365e-04*w8*Jinv21+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+7.9365079365079365e-04*Jinv12*w28+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+1.5873015873015873e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-7.9365079365079365e-04*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[17][21]=(0.0)*D;
  A[17][22]=(0.0)*D;
  A[17][23]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv10*w16+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-3.1746031746031746e-03*Jinv20*w23-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+1.5873015873015873e-03*Jinv02*w15+7.9365079365079365e-04*w1*Jinv10-1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18+7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22)*D;
  A[17][24]=(0.0)*D;
  A[17][25]=(0.0)*D;
  A[17][26]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv01*w18-1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv11-1.5873015873015873e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-1.5873015873015873e-03*Jinv22*w5-3.1746031746031746e-03*w20*Jinv21-1.5873015873015873e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-1.5873015873015873e-03*Jinv01*w12+7.9365079365079365e-04*w8*Jinv21-1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*w23*Jinv22-1.5873015873015873e-03*w10*Jinv11-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-1.5873015873015873e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+4.7619047619047623e-03*Jinv02*w15-1.5873015873015873e-03*w14*Jinv21+7.9365079365079365e-04*Jinv01*w6-1.5873015873015873e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+4.7619047619047623e-03*w17*Jinv22-1.5873015873015873e-03*Jinv02*w3-3.1746031746031746e-03*Jinv01*w24-3.1746031746031746e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15-1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+4.7619047619047623e-03*Jinv12*w16+7.9365079365079365e-04*w2*Jinv21)*D;
  A[17][27]=(0.0)*D;
  A[17][28]=(0.0)*D;
  A[17][29]=(7.9365079365079365e-04*w4*Jinv12-7.9365079365079365e-04*w20*Jinv22+3.9682539682539683e-04*Jinv12*w7-7.9365079365079365e-04*w19*Jinv12+7.9365079365079365e-04*Jinv22*w5-3.9682539682539683e-04*Jinv02*w27-7.9365079365079365e-04*Jinv02*w18-3.9682539682539683e-04*Jinv12*w28-7.9365079365079365e-04*w23*Jinv22+3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24+3.9682539682539683e-04*Jinv02*w0-7.9365079365079365e-04*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+7.9365079365079365e-04*Jinv02*w3+3.9682539682539683e-04*w2*Jinv22-7.9365079365079365e-04*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16-3.9682539682539683e-04*w29*Jinv22+3.9682539682539683e-04*Jinv02*w6)*D;
  A[18][0]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-3.1746031746031746e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[18][1]=(0.0)*D;
  A[18][2]=(0.0)*D;
  A[18][3]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+1.2698412698412698e-02*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10-1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3+1.2698412698412698e-02*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21+1.2698412698412698e-02*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[18][4]=(0.0)*D;
  A[18][5]=(0.0)*D;
  A[18][6]=(7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv00*w15-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w10*Jinv10+7.9365079365079365e-04*Jinv10*w16-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv20*w8-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20)*D;
  A[18][7]=(0.0)*D;
  A[18][8]=(0.0)*D;
  A[18][9]=(6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12+1.2698412698412698e-02*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+1.2698412698412698e-02*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12+3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+1.2698412698412698e-02*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[18][10]=(0.0)*D;
  A[18][11]=(0.0)*D;
  A[18][12]=(6.3492063492063492e-03*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+6.3492063492063492e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20+6.3492063492063492e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27+3.1746031746031746e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+6.3492063492063492e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[18][13]=(0.0)*D;
  A[18][14]=(0.0)*D;
  A[18][15]=(-3.1746031746031746e-03*w5*Jinv21-6.3492063492063492e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv11+7.9365079365079365e-04*Jinv11*w16-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*Jinv01*w6-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*w17*Jinv21)*D;
  A[18][16]=(0.0)*D;
  A[18][17]=(0.0)*D;
  A[18][18]=(-6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12-1.9047619047619049e-02*Jinv01*w18+3.1746031746031746e-03*w7*Jinv10-9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27-1.9047619047619049e-02*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-9.5238095238095247e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-1.9047619047619049e-02*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28-9.5238095238095247e-03*Jinv00*w21-9.5238095238095247e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-6.3492063492063492e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-9.5238095238095247e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-9.5238095238095247e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-3.1746031746031746e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.9047619047619049e-02*Jinv00*w18-9.5238095238095247e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22-1.9047619047619049e-02*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-1.9047619047619049e-02*w19*Jinv10+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[18][19]=(0.0)*D;
  A[18][20]=(0.0)*D;
  A[18][21]=(6.3492063492063492e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+1.9047619047619049e-02*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*Jinv00*w21+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+9.5238095238095247e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+6.3492063492063492e-03*Jinv02*w3+1.9047619047619049e-02*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*Jinv02*w21+1.9047619047619049e-02*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+9.5238095238095247e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[18][22]=(0.0)*D;
  A[18][23]=(0.0)*D;
  A[18][24]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.9047619047619049e-02*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+1.9047619047619049e-02*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+9.5238095238095247e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+9.5238095238095247e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+3.1746031746031746e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+6.3492063492063492e-03*w25*Jinv12+9.5238095238095247e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+6.3492063492063492e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+9.5238095238095247e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22+1.9047619047619049e-02*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[18][25]=(0.0)*D;
  A[18][26]=(0.0)*D;
  A[18][27]=(6.3492063492063492e-03*w20*Jinv22-7.9365079365079365e-04*Jinv12*w7+6.3492063492063492e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[18][28]=(0.0)*D;
  A[18][29]=(0.0)*D;
  A[19][0]=(0.0)*D;
  A[19][1]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-3.1746031746031746e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[19][2]=(0.0)*D;
  A[19][3]=(0.0)*D;
  A[19][4]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+1.2698412698412698e-02*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10-1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3+1.2698412698412698e-02*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21+1.2698412698412698e-02*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[19][5]=(0.0)*D;
  A[19][6]=(0.0)*D;
  A[19][7]=(7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv00*w15-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w10*Jinv10+7.9365079365079365e-04*Jinv10*w16-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv20*w8-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20)*D;
  A[19][8]=(0.0)*D;
  A[19][9]=(0.0)*D;
  A[19][10]=(6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12+1.2698412698412698e-02*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+1.2698412698412698e-02*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12+3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+1.2698412698412698e-02*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[19][11]=(0.0)*D;
  A[19][12]=(0.0)*D;
  A[19][13]=(6.3492063492063492e-03*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+6.3492063492063492e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20+6.3492063492063492e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27+3.1746031746031746e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+6.3492063492063492e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[19][14]=(0.0)*D;
  A[19][15]=(0.0)*D;
  A[19][16]=(-3.1746031746031746e-03*w5*Jinv21-6.3492063492063492e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv11+7.9365079365079365e-04*Jinv11*w16-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*Jinv01*w6-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*w17*Jinv21)*D;
  A[19][17]=(0.0)*D;
  A[19][18]=(0.0)*D;
  A[19][19]=(-6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12-1.9047619047619049e-02*Jinv01*w18+3.1746031746031746e-03*w7*Jinv10-9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27-1.9047619047619049e-02*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-9.5238095238095247e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-1.9047619047619049e-02*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28-9.5238095238095247e-03*Jinv00*w21-9.5238095238095247e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-6.3492063492063492e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-9.5238095238095247e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-9.5238095238095247e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-3.1746031746031746e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.9047619047619049e-02*Jinv00*w18-9.5238095238095247e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22-1.9047619047619049e-02*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-1.9047619047619049e-02*w19*Jinv10+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[19][20]=(0.0)*D;
  A[19][21]=(0.0)*D;
  A[19][22]=(6.3492063492063492e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+1.9047619047619049e-02*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*Jinv00*w21+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+9.5238095238095247e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+6.3492063492063492e-03*Jinv02*w3+1.9047619047619049e-02*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*Jinv02*w21+1.9047619047619049e-02*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+9.5238095238095247e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[19][23]=(0.0)*D;
  A[19][24]=(0.0)*D;
  A[19][25]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.9047619047619049e-02*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+1.9047619047619049e-02*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+9.5238095238095247e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+9.5238095238095247e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+3.1746031746031746e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+6.3492063492063492e-03*w25*Jinv12+9.5238095238095247e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+6.3492063492063492e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+9.5238095238095247e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22+1.9047619047619049e-02*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[19][26]=(0.0)*D;
  A[19][27]=(0.0)*D;
  A[19][28]=(6.3492063492063492e-03*w20*Jinv22-7.9365079365079365e-04*Jinv12*w7+6.3492063492063492e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[19][29]=(0.0)*D;
  A[20][0]=(0.0)*D;
  A[20][1]=(0.0)*D;
  A[20][2]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+7.9365079365079365e-04*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+7.9365079365079365e-04*w7*Jinv11-3.1746031746031746e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18+7.9365079365079365e-04*Jinv10*w16+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+7.9365079365079365e-04*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10+7.9365079365079365e-04*Jinv20*w8-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5+7.9365079365079365e-04*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w11*Jinv22-1.5873015873015873e-03*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*w2*Jinv21+7.9365079365079365e-04*Jinv02*w6)*D;
  A[20][3]=(0.0)*D;
  A[20][4]=(0.0)*D;
  A[20][5]=(-6.3492063492063492e-03*w5*Jinv21-6.3492063492063492e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7-1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-6.3492063492063492e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+1.2698412698412698e-02*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10-1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-1.5873015873015873e-03*Jinv20*w29-6.3492063492063492e-03*Jinv02*w3+1.2698412698412698e-02*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21+1.2698412698412698e-02*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[20][6]=(0.0)*D;
  A[20][7]=(0.0)*D;
  A[20][8]=(7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv00*w15-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w10*Jinv10+7.9365079365079365e-04*Jinv10*w16-3.1746031746031746e-03*w25*Jinv10+7.9365079365079365e-04*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv20*w8-6.3492063492063492e-03*Jinv00*w18-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*w19*Jinv10-3.1746031746031746e-03*w26*Jinv20)*D;
  A[20][9]=(0.0)*D;
  A[20][10]=(0.0)*D;
  A[20][11]=(6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12+1.2698412698412698e-02*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-3.1746031746031746e-03*Jinv10*w22-6.3492063492063492e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11-6.3492063492063492e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+1.2698412698412698e-02*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-6.3492063492063492e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12+3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-6.3492063492063492e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6+1.5873015873015873e-03*Jinv00*w27-6.3492063492063492e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18+1.5873015873015873e-03*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+1.2698412698412698e-02*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-6.3492063492063492e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[20][12]=(0.0)*D;
  A[20][13]=(0.0)*D;
  A[20][14]=(6.3492063492063492e-03*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+6.3492063492063492e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+6.3492063492063492e-03*w10*Jinv10+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+6.3492063492063492e-03*w20*Jinv20+6.3492063492063492e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv01*w3+3.1746031746031746e-03*Jinv20*w23+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-1.5873015873015873e-03*Jinv00*w27+3.1746031746031746e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+3.1746031746031746e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+6.3492063492063492e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[20][15]=(0.0)*D;
  A[20][16]=(0.0)*D;
  A[20][17]=(-3.1746031746031746e-03*w5*Jinv21-6.3492063492063492e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv11+7.9365079365079365e-04*Jinv11*w16-6.3492063492063492e-03*w20*Jinv21-3.1746031746031746e-03*w23*Jinv21+7.9365079365079365e-04*w8*Jinv21-3.1746031746031746e-03*Jinv11*w22-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*Jinv01*w6-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*w17*Jinv21)*D;
  A[20][18]=(0.0)*D;
  A[20][19]=(0.0)*D;
  A[20][20]=(-6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12-1.9047619047619049e-02*Jinv01*w18+3.1746031746031746e-03*w7*Jinv10-9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27-1.9047619047619049e-02*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-9.5238095238095247e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-1.9047619047619049e-02*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28-9.5238095238095247e-03*Jinv00*w21-9.5238095238095247e-03*w25*Jinv11-3.1746031746031746e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-6.3492063492063492e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-9.5238095238095247e-03*Jinv20*w23-3.1746031746031746e-03*w25*Jinv12-9.5238095238095247e-03*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-3.1746031746031746e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w26*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.9047619047619049e-02*Jinv00*w18-9.5238095238095247e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22-1.9047619047619049e-02*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-1.9047619047619049e-02*w19*Jinv10+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[20][21]=(0.0)*D;
  A[20][22]=(0.0)*D;
  A[20][23]=(6.3492063492063492e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+6.3492063492063492e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+1.9047619047619049e-02*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*Jinv00*w21+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+9.5238095238095247e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+6.3492063492063492e-03*Jinv02*w3+1.9047619047619049e-02*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*Jinv02*w21+1.9047619047619049e-02*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13+9.5238095238095247e-03*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[20][24]=(0.0)*D;
  A[20][25]=(0.0)*D;
  A[20][26]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.9047619047619049e-02*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+6.3492063492063492e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+1.9047619047619049e-02*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+9.5238095238095247e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+9.5238095238095247e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+3.1746031746031746e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+6.3492063492063492e-03*w25*Jinv12+9.5238095238095247e-03*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+6.3492063492063492e-03*Jinv02*w24+6.3492063492063492e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+9.5238095238095247e-03*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11+6.3492063492063492e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22+1.9047619047619049e-02*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[20][27]=(0.0)*D;
  A[20][28]=(0.0)*D;
  A[20][29]=(6.3492063492063492e-03*w20*Jinv22-7.9365079365079365e-04*Jinv12*w7+6.3492063492063492e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24-1.5873015873015873e-03*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[21][0]=(3.1746031746031746e-03*Jinv02*w12+6.3492063492063492e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00-7.9365079365079365e-04*Jinv11*w16+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+6.3492063492063492e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20-7.9365079365079365e-04*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w2*Jinv21)*D;
  A[21][1]=(0.0)*D;
  A[21][2]=(0.0)*D;
  A[21][3]=(-9.5238095238095247e-03*w5*Jinv21-9.5238095238095247e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-1.2698412698412698e-02*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-9.5238095238095247e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*Jinv02*w18-1.9047619047619049e-02*w23*Jinv21-9.5238095238095247e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-1.9047619047619049e-02*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-1.2698412698412698e-02*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-1.9047619047619049e-02*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-9.5238095238095247e-03*Jinv01*w3-1.2698412698412698e-02*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-1.9047619047619049e-02*Jinv12*w22-1.9047619047619049e-02*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-9.5238095238095247e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-9.5238095238095247e-03*Jinv02*w3-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-9.5238095238095247e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-1.9047619047619049e-02*Jinv02*w21-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[21][4]=(0.0)*D;
  A[21][5]=(0.0)*D;
  A[21][6]=(1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+6.3492063492063492e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w14)*D;
  A[21][7]=(0.0)*D;
  A[21][8]=(0.0)*D;
  A[21][9]=(3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+1.5873015873015873e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13-6.3492063492063492e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*Jinv02*w6)*D;
  A[21][10]=(0.0)*D;
  A[21][11]=(0.0)*D;
  A[21][12]=(9.5238095238095247e-03*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.9047619047619049e-02*w23*Jinv21+9.5238095238095247e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+1.9047619047619049e-02*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+9.5238095238095247e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+1.9047619047619049e-02*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+9.5238095238095247e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+9.5238095238095247e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+6.3492063492063492e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[21][13]=(0.0)*D;
  A[21][14]=(0.0)*D;
  A[21][15]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*Jinv01*w18+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*w20*Jinv21-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv11*w22-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv01*w21+7.9365079365079365e-04*Jinv11*w1-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w2*Jinv21)*D;
  A[21][16]=(0.0)*D;
  A[21][17]=(0.0)*D;
  A[21][18]=(-6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*Jinv02*w12-9.5238095238095247e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-1.9047619047619049e-02*Jinv10*w22+1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.5873015873015873e-03*Jinv02*w27-9.5238095238095247e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-1.9047619047619049e-02*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-9.5238095238095247e-03*w20*Jinv20-1.9047619047619049e-02*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-1.9047619047619049e-02*Jinv00*w21-9.5238095238095247e-03*w25*Jinv11-1.2698412698412698e-02*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-6.3492063492063492e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-1.9047619047619049e-02*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-9.5238095238095247e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24+1.5873015873015873e-03*Jinv02*w0-1.2698412698412698e-02*Jinv12*w22-1.9047619047619049e-02*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-9.5238095238095247e-03*Jinv00*w18-9.5238095238095247e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-9.5238095238095247e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.2698412698412698e-02*Jinv02*w21-9.5238095238095247e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[21][19]=(0.0)*D;
  A[21][20]=(0.0)*D;
  A[21][21]=(9.5238095238095247e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+1.9047619047619049e-02*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+9.5238095238095247e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+9.5238095238095247e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+1.9047619047619049e-02*Jinv00*w21+1.9047619047619049e-02*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+9.5238095238095247e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+1.9047619047619049e-02*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+1.9047619047619049e-02*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22+9.5238095238095247e-03*Jinv02*w3+9.5238095238095247e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+1.9047619047619049e-02*Jinv02*w21+9.5238095238095247e-03*w19*Jinv10+9.5238095238095247e-03*Jinv12*w13+9.5238095238095247e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[21][22]=(0.0)*D;
  A[21][23]=(0.0)*D;
  A[21][24]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+9.5238095238095247e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+9.5238095238095247e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+1.9047619047619049e-02*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.9047619047619049e-02*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+6.3492063492063492e-03*w25*Jinv12+9.5238095238095247e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+1.9047619047619049e-02*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22+9.5238095238095247e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[21][25]=(0.0)*D;
  A[21][26]=(0.0)*D;
  A[21][27]=(3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22+6.3492063492063492e-03*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[21][28]=(0.0)*D;
  A[21][29]=(0.0)*D;
  A[22][0]=(0.0)*D;
  A[22][1]=(3.1746031746031746e-03*Jinv02*w12+6.3492063492063492e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00-7.9365079365079365e-04*Jinv11*w16+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+6.3492063492063492e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20-7.9365079365079365e-04*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w2*Jinv21)*D;
  A[22][2]=(0.0)*D;
  A[22][3]=(0.0)*D;
  A[22][4]=(-9.5238095238095247e-03*w5*Jinv21-9.5238095238095247e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-1.2698412698412698e-02*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-9.5238095238095247e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*Jinv02*w18-1.9047619047619049e-02*w23*Jinv21-9.5238095238095247e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-1.9047619047619049e-02*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-1.2698412698412698e-02*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-1.9047619047619049e-02*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-9.5238095238095247e-03*Jinv01*w3-1.2698412698412698e-02*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-1.9047619047619049e-02*Jinv12*w22-1.9047619047619049e-02*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-9.5238095238095247e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-9.5238095238095247e-03*Jinv02*w3-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-9.5238095238095247e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-1.9047619047619049e-02*Jinv02*w21-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[22][5]=(0.0)*D;
  A[22][6]=(0.0)*D;
  A[22][7]=(1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+6.3492063492063492e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w14)*D;
  A[22][8]=(0.0)*D;
  A[22][9]=(0.0)*D;
  A[22][10]=(3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+1.5873015873015873e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13-6.3492063492063492e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*Jinv02*w6)*D;
  A[22][11]=(0.0)*D;
  A[22][12]=(0.0)*D;
  A[22][13]=(9.5238095238095247e-03*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.9047619047619049e-02*w23*Jinv21+9.5238095238095247e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+1.9047619047619049e-02*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+9.5238095238095247e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+1.9047619047619049e-02*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+9.5238095238095247e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+9.5238095238095247e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+6.3492063492063492e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[22][14]=(0.0)*D;
  A[22][15]=(0.0)*D;
  A[22][16]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*Jinv01*w18+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*w20*Jinv21-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv11*w22-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv01*w21+7.9365079365079365e-04*Jinv11*w1-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w2*Jinv21)*D;
  A[22][17]=(0.0)*D;
  A[22][18]=(0.0)*D;
  A[22][19]=(-6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*Jinv02*w12-9.5238095238095247e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-1.9047619047619049e-02*Jinv10*w22+1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.5873015873015873e-03*Jinv02*w27-9.5238095238095247e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-1.9047619047619049e-02*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-9.5238095238095247e-03*w20*Jinv20-1.9047619047619049e-02*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-1.9047619047619049e-02*Jinv00*w21-9.5238095238095247e-03*w25*Jinv11-1.2698412698412698e-02*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-6.3492063492063492e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-1.9047619047619049e-02*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-9.5238095238095247e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24+1.5873015873015873e-03*Jinv02*w0-1.2698412698412698e-02*Jinv12*w22-1.9047619047619049e-02*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-9.5238095238095247e-03*Jinv00*w18-9.5238095238095247e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-9.5238095238095247e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.2698412698412698e-02*Jinv02*w21-9.5238095238095247e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[22][20]=(0.0)*D;
  A[22][21]=(0.0)*D;
  A[22][22]=(9.5238095238095247e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+1.9047619047619049e-02*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+9.5238095238095247e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+9.5238095238095247e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+1.9047619047619049e-02*Jinv00*w21+1.9047619047619049e-02*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+9.5238095238095247e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+1.9047619047619049e-02*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+1.9047619047619049e-02*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22+9.5238095238095247e-03*Jinv02*w3+9.5238095238095247e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+1.9047619047619049e-02*Jinv02*w21+9.5238095238095247e-03*w19*Jinv10+9.5238095238095247e-03*Jinv12*w13+9.5238095238095247e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[22][23]=(0.0)*D;
  A[22][24]=(0.0)*D;
  A[22][25]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+9.5238095238095247e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+9.5238095238095247e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+1.9047619047619049e-02*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.9047619047619049e-02*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+6.3492063492063492e-03*w25*Jinv12+9.5238095238095247e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+1.9047619047619049e-02*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22+9.5238095238095247e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[22][26]=(0.0)*D;
  A[22][27]=(0.0)*D;
  A[22][28]=(3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22+6.3492063492063492e-03*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[22][29]=(0.0)*D;
  A[23][0]=(0.0)*D;
  A[23][1]=(0.0)*D;
  A[23][2]=(3.1746031746031746e-03*Jinv02*w12+6.3492063492063492e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00-7.9365079365079365e-04*Jinv11*w16+6.3492063492063492e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10-7.9365079365079365e-04*Jinv20*w17+6.3492063492063492e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+3.1746031746031746e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+6.3492063492063492e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22+3.1746031746031746e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-7.9365079365079365e-04*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20-7.9365079365079365e-04*Jinv12*w16+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w2*Jinv21)*D;
  A[23][3]=(0.0)*D;
  A[23][4]=(0.0)*D;
  A[23][5]=(-9.5238095238095247e-03*w5*Jinv21-9.5238095238095247e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*Jinv01*w18-6.3492063492063492e-03*w20*Jinv22-1.5873015873015873e-03*w7*Jinv10-1.2698412698412698e-02*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-6.3492063492063492e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-9.5238095238095247e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-6.3492063492063492e-03*w20*Jinv21-6.3492063492063492e-03*Jinv02*w18-1.9047619047619049e-02*w23*Jinv21-9.5238095238095247e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16-3.1746031746031746e-03*Jinv01*w9+1.5873015873015873e-03*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-1.9047619047619049e-02*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-1.2698412698412698e-02*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-1.9047619047619049e-02*w23*Jinv22+3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-9.5238095238095247e-03*Jinv01*w3-1.2698412698412698e-02*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-1.9047619047619049e-02*Jinv12*w22-1.9047619047619049e-02*Jinv01*w21+3.1746031746031746e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-9.5238095238095247e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+3.1746031746031746e-03*w28*Jinv11+3.1746031746031746e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-9.5238095238095247e-03*Jinv02*w3-6.3492063492063492e-03*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+3.1746031746031746e-03*w29*Jinv21-6.3492063492063492e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-9.5238095238095247e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-1.9047619047619049e-02*Jinv02*w21-9.5238095238095247e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21)*D;
  A[23][6]=(0.0)*D;
  A[23][7]=(0.0)*D;
  A[23][8]=(1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-7.9365079365079365e-04*Jinv00*w15-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*Jinv10*w16-7.9365079365079365e-04*Jinv20*w2-7.9365079365079365e-04*Jinv20*w17+6.3492063492063492e-03*Jinv00*w21+1.5873015873015873e-03*Jinv00*w6-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10+6.3492063492063492e-03*Jinv20*w23-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w14)*D;
  A[23][9]=(0.0)*D;
  A[23][10]=(0.0)*D;
  A[23][11]=(3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+1.5873015873015873e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*Jinv00*w15+1.5873015873015873e-03*Jinv00*w0-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv10*w16+1.5873015873015873e-03*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10+1.5873015873015873e-03*Jinv20*w17-3.1746031746031746e-03*w20*Jinv20+1.5873015873015873e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-3.1746031746031746e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1+1.5873015873015873e-03*w28*Jinv10-6.3492063492063492e-03*Jinv00*w12+1.5873015873015873e-03*Jinv20*w8-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-3.1746031746031746e-03*w26*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22+1.5873015873015873e-03*Jinv02*w15+1.5873015873015873e-03*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22+1.5873015873015873e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13-6.3492063492063492e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*Jinv02*w6)*D;
  A[23][12]=(0.0)*D;
  A[23][13]=(0.0)*D;
  A[23][14]=(9.5238095238095247e-03*w5*Jinv21+6.3492063492063492e-03*Jinv01*w18-1.5873015873015873e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-1.5873015873015873e-03*Jinv00*w15-1.5873015873015873e-03*Jinv00*w0+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*Jinv11*w16+6.3492063492063492e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.9047619047619049e-02*w23*Jinv21+9.5238095238095247e-03*Jinv01*w12-1.5873015873015873e-03*Jinv10*w16+3.1746031746031746e-03*Jinv01*w9-1.5873015873015873e-03*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10-1.5873015873015873e-03*Jinv20*w17+3.1746031746031746e-03*w20*Jinv20+1.9047619047619049e-02*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-3.1746031746031746e-03*Jinv01*w27-1.5873015873015873e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-1.5873015873015873e-03*w28*Jinv10+6.3492063492063492e-03*Jinv00*w12-1.5873015873015873e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+9.5238095238095247e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+1.9047619047619049e-02*Jinv01*w21-1.5873015873015873e-03*w1*Jinv10+9.5238095238095247e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+9.5238095238095247e-03*w13*Jinv11-3.1746031746031746e-03*Jinv11*w1-3.1746031746031746e-03*w28*Jinv11-1.5873015873015873e-03*Jinv20*w29+3.1746031746031746e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-3.1746031746031746e-03*w29*Jinv21+6.3492063492063492e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+9.5238095238095247e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+3.1746031746031746e-03*w19*Jinv10-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21+6.3492063492063492e-03*w26*Jinv20+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w2*Jinv21)*D;
  A[23][15]=(0.0)*D;
  A[23][16]=(0.0)*D;
  A[23][17]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*Jinv01*w18+7.9365079365079365e-04*Jinv11*w16-3.1746031746031746e-03*w20*Jinv21-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv11*w22-3.1746031746031746e-03*Jinv01*w3-6.3492063492063492e-03*Jinv01*w21+7.9365079365079365e-04*Jinv11*w1-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+7.9365079365079365e-04*w2*Jinv21)*D;
  A[23][18]=(0.0)*D;
  A[23][19]=(0.0)*D;
  A[23][20]=(-6.3492063492063492e-03*w5*Jinv21-3.1746031746031746e-03*Jinv02*w12-9.5238095238095247e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-1.9047619047619049e-02*Jinv10*w22+1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-9.5238095238095247e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.5873015873015873e-03*Jinv02*w27-9.5238095238095247e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-1.9047619047619049e-02*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-9.5238095238095247e-03*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-9.5238095238095247e-03*w20*Jinv20-1.9047619047619049e-02*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-1.9047619047619049e-02*Jinv00*w21-9.5238095238095247e-03*w25*Jinv11-1.2698412698412698e-02*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-6.3492063492063492e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-6.3492063492063492e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-6.3492063492063492e-03*Jinv01*w3-1.9047619047619049e-02*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-9.5238095238095247e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24+1.5873015873015873e-03*Jinv02*w0-1.2698412698412698e-02*Jinv12*w22-1.9047619047619049e-02*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-9.5238095238095247e-03*Jinv00*w18-9.5238095238095247e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-9.5238095238095247e-03*w19*Jinv11-6.3492063492063492e-03*w4*Jinv10-6.3492063492063492e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.2698412698412698e-02*Jinv02*w21-9.5238095238095247e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-9.5238095238095247e-03*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[23][21]=(0.0)*D;
  A[23][22]=(0.0)*D;
  A[23][23]=(9.5238095238095247e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+6.3492063492063492e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+1.9047619047619049e-02*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+9.5238095238095247e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+6.3492063492063492e-03*w19*Jinv12+9.5238095238095247e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+3.1746031746031746e-03*w10*Jinv10+6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+9.5238095238095247e-03*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+9.5238095238095247e-03*w20*Jinv20-3.1746031746031746e-03*Jinv12*w28+1.9047619047619049e-02*Jinv00*w21+1.9047619047619049e-02*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+9.5238095238095247e-03*w14*Jinv22+6.3492063492063492e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+1.9047619047619049e-02*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+1.9047619047619049e-02*Jinv12*w22-3.1746031746031746e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-3.1746031746031746e-03*w17*Jinv22+9.5238095238095247e-03*Jinv02*w3+9.5238095238095247e-03*Jinv00*w18-3.1746031746031746e-03*w2*Jinv22+6.3492063492063492e-03*w4*Jinv10+3.1746031746031746e-03*Jinv20*w11+1.9047619047619049e-02*Jinv02*w21+9.5238095238095247e-03*w19*Jinv10+9.5238095238095247e-03*Jinv12*w13+9.5238095238095247e-03*w26*Jinv20-3.1746031746031746e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22)*D;
  A[23][24]=(0.0)*D;
  A[23][25]=(0.0)*D;
  A[23][26]=(6.3492063492063492e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+9.5238095238095247e-03*Jinv01*w18+3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+3.1746031746031746e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+9.5238095238095247e-03*w20*Jinv21+3.1746031746031746e-03*Jinv02*w18+1.9047619047619049e-02*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+1.9047619047619049e-02*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w14*Jinv22+6.3492063492063492e-03*Jinv01*w3+6.3492063492063492e-03*w25*Jinv12+9.5238095238095247e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+1.9047619047619049e-02*Jinv01*w21-1.5873015873015873e-03*Jinv02*w15+6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv01*w24-1.5873015873015873e-03*w2*Jinv22+9.5238095238095247e-03*w19*Jinv11+6.3492063492063492e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+6.3492063492063492e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-1.5873015873015873e-03*Jinv12*w16-1.5873015873015873e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-1.5873015873015873e-03*Jinv02*w6)*D;
  A[23][27]=(0.0)*D;
  A[23][28]=(0.0)*D;
  A[23][29]=(3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+6.3492063492063492e-03*w23*Jinv22-1.5873015873015873e-03*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*w25*Jinv12+3.1746031746031746e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-7.9365079365079365e-04*Jinv02*w15+3.1746031746031746e-03*w26*Jinv22-7.9365079365079365e-04*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22+6.3492063492063492e-03*Jinv02*w21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[24][0]=(3.1746031746031746e-03*Jinv02*w12-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21-7.9365079365079365e-04*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22+6.3492063492063492e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+6.3492063492063492e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[24][1]=(0.0)*D;
  A[24][2]=(0.0)*D;
  A[24][3]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-3.1746031746031746e-03*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15-6.3492063492063492e-03*w14*Jinv21+1.5873015873015873e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[24][4]=(0.0)*D;
  A[24][5]=(0.0)*D;
  A[24][6]=(7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv00*w0-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w10*Jinv10+7.9365079365079365e-04*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv20*w8+7.9365079365079365e-04*w1*Jinv10-3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*w26*Jinv20)*D;
  A[24][7]=(0.0)*D;
  A[24][8]=(0.0)*D;
  A[24][9]=(-3.1746031746031746e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-9.5238095238095247e-03*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-1.9047619047619049e-02*w24*Jinv00-9.5238095238095247e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-9.5238095238095247e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-1.9047619047619049e-02*w25*Jinv10-6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-1.2698412698412698e-02*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w28*Jinv10-9.5238095238095247e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-9.5238095238095247e-03*w13*Jinv10-6.3492063492063492e-03*Jinv20*w23-1.9047619047619049e-02*w25*Jinv12-1.2698412698412698e-02*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-1.9047619047619049e-02*Jinv02*w24-9.5238095238095247e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11-9.5238095238095247e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-1.9047619047619049e-02*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+3.1746031746031746e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-1.2698412698412698e-02*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w4*Jinv10-1.5873015873015873e-03*Jinv01*w15-9.5238095238095247e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.9047619047619049e-02*w26*Jinv20-9.5238095238095247e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[24][10]=(0.0)*D;
  A[24][11]=(0.0)*D;
  A[24][12]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+1.9047619047619049e-02*w24*Jinv00+9.5238095238095247e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+9.5238095238095247e-03*w10*Jinv10+6.3492063492063492e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+1.9047619047619049e-02*w25*Jinv10+6.3492063492063492e-03*w20*Jinv20+6.3492063492063492e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+9.5238095238095247e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+9.5238095238095247e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-3.1746031746031746e-03*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+9.5238095238095247e-03*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+1.9047619047619049e-02*w26*Jinv20+9.5238095238095247e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[24][13]=(0.0)*D;
  A[24][14]=(0.0)*D;
  A[24][15]=(-7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[24][16]=(0.0)*D;
  A[24][17]=(0.0)*D;
  A[24][18]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*Jinv02*w12-9.5238095238095247e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-9.5238095238095247e-03*Jinv10*w22+1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-1.9047619047619049e-02*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.5873015873015873e-03*Jinv02*w27-9.5238095238095247e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-1.9047619047619049e-02*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-9.5238095238095247e-03*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-9.5238095238095247e-03*Jinv00*w21-1.9047619047619049e-02*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-6.3492063492063492e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-9.5238095238095247e-03*Jinv20*w23-1.2698412698412698e-02*w25*Jinv12-1.9047619047619049e-02*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-1.2698412698412698e-02*Jinv02*w24+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*Jinv11*w1-1.2698412698412698e-02*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-9.5238095238095247e-03*Jinv00*w18-1.9047619047619049e-02*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-9.5238095238095247e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-9.5238095238095247e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-1.9047619047619049e-02*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[24][19]=(0.0)*D;
  A[24][20]=(0.0)*D;
  A[24][21]=(3.1746031746031746e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+1.9047619047619049e-02*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+1.9047619047619049e-02*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+9.5238095238095247e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*Jinv00*w21+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+6.3492063492063492e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+9.5238095238095247e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*Jinv02*w21+9.5238095238095247e-03*w19*Jinv10+6.3492063492063492e-03*Jinv12*w13+1.9047619047619049e-02*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[24][22]=(0.0)*D;
  A[24][23]=(0.0)*D;
  A[24][24]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+9.5238095238095247e-03*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+9.5238095238095247e-03*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+9.5238095238095247e-03*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+9.5238095238095247e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+9.5238095238095247e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+1.9047619047619049e-02*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+9.5238095238095247e-03*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+1.9047619047619049e-02*w25*Jinv12+1.9047619047619049e-02*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+1.9047619047619049e-02*Jinv02*w24+9.5238095238095247e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+9.5238095238095247e-03*Jinv01*w21+6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+9.5238095238095247e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+1.9047619047619049e-02*w26*Jinv22+3.1746031746031746e-03*Jinv02*w3+1.9047619047619049e-02*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22+9.5238095238095247e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+9.5238095238095247e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[24][25]=(0.0)*D;
  A[24][26]=(0.0)*D;
  A[24][27]=(3.1746031746031746e-03*w20*Jinv22-7.9365079365079365e-04*Jinv12*w7+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[24][28]=(0.0)*D;
  A[24][29]=(0.0)*D;
  A[25][0]=(0.0)*D;
  A[25][1]=(3.1746031746031746e-03*Jinv02*w12-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21-7.9365079365079365e-04*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22+6.3492063492063492e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+6.3492063492063492e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[25][2]=(0.0)*D;
  A[25][3]=(0.0)*D;
  A[25][4]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-3.1746031746031746e-03*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15-6.3492063492063492e-03*w14*Jinv21+1.5873015873015873e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[25][5]=(0.0)*D;
  A[25][6]=(0.0)*D;
  A[25][7]=(7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv00*w0-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w10*Jinv10+7.9365079365079365e-04*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv20*w8+7.9365079365079365e-04*w1*Jinv10-3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*w26*Jinv20)*D;
  A[25][8]=(0.0)*D;
  A[25][9]=(0.0)*D;
  A[25][10]=(-3.1746031746031746e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-9.5238095238095247e-03*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-1.9047619047619049e-02*w24*Jinv00-9.5238095238095247e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-9.5238095238095247e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-1.9047619047619049e-02*w25*Jinv10-6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-1.2698412698412698e-02*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w28*Jinv10-9.5238095238095247e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-9.5238095238095247e-03*w13*Jinv10-6.3492063492063492e-03*Jinv20*w23-1.9047619047619049e-02*w25*Jinv12-1.2698412698412698e-02*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-1.9047619047619049e-02*Jinv02*w24-9.5238095238095247e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11-9.5238095238095247e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-1.9047619047619049e-02*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+3.1746031746031746e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-1.2698412698412698e-02*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w4*Jinv10-1.5873015873015873e-03*Jinv01*w15-9.5238095238095247e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.9047619047619049e-02*w26*Jinv20-9.5238095238095247e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[25][11]=(0.0)*D;
  A[25][12]=(0.0)*D;
  A[25][13]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+1.9047619047619049e-02*w24*Jinv00+9.5238095238095247e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+9.5238095238095247e-03*w10*Jinv10+6.3492063492063492e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+1.9047619047619049e-02*w25*Jinv10+6.3492063492063492e-03*w20*Jinv20+6.3492063492063492e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+9.5238095238095247e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+9.5238095238095247e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-3.1746031746031746e-03*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+9.5238095238095247e-03*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+1.9047619047619049e-02*w26*Jinv20+9.5238095238095247e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[25][14]=(0.0)*D;
  A[25][15]=(0.0)*D;
  A[25][16]=(-7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[25][17]=(0.0)*D;
  A[25][18]=(0.0)*D;
  A[25][19]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*Jinv02*w12-9.5238095238095247e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-9.5238095238095247e-03*Jinv10*w22+1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-1.9047619047619049e-02*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.5873015873015873e-03*Jinv02*w27-9.5238095238095247e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-1.9047619047619049e-02*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-9.5238095238095247e-03*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-9.5238095238095247e-03*Jinv00*w21-1.9047619047619049e-02*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-6.3492063492063492e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-9.5238095238095247e-03*Jinv20*w23-1.2698412698412698e-02*w25*Jinv12-1.9047619047619049e-02*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-1.2698412698412698e-02*Jinv02*w24+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*Jinv11*w1-1.2698412698412698e-02*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-9.5238095238095247e-03*Jinv00*w18-1.9047619047619049e-02*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-9.5238095238095247e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-9.5238095238095247e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-1.9047619047619049e-02*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[25][20]=(0.0)*D;
  A[25][21]=(0.0)*D;
  A[25][22]=(3.1746031746031746e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+1.9047619047619049e-02*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+1.9047619047619049e-02*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+9.5238095238095247e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*Jinv00*w21+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+6.3492063492063492e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+9.5238095238095247e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*Jinv02*w21+9.5238095238095247e-03*w19*Jinv10+6.3492063492063492e-03*Jinv12*w13+1.9047619047619049e-02*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[25][23]=(0.0)*D;
  A[25][24]=(0.0)*D;
  A[25][25]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+9.5238095238095247e-03*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+9.5238095238095247e-03*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+9.5238095238095247e-03*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+9.5238095238095247e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+9.5238095238095247e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+1.9047619047619049e-02*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+9.5238095238095247e-03*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+1.9047619047619049e-02*w25*Jinv12+1.9047619047619049e-02*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+1.9047619047619049e-02*Jinv02*w24+9.5238095238095247e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+9.5238095238095247e-03*Jinv01*w21+6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+9.5238095238095247e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+1.9047619047619049e-02*w26*Jinv22+3.1746031746031746e-03*Jinv02*w3+1.9047619047619049e-02*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22+9.5238095238095247e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+9.5238095238095247e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[25][26]=(0.0)*D;
  A[25][27]=(0.0)*D;
  A[25][28]=(3.1746031746031746e-03*w20*Jinv22-7.9365079365079365e-04*Jinv12*w7+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[25][29]=(0.0)*D;
  A[26][0]=(0.0)*D;
  A[26][1]=(0.0)*D;
  A[26][2]=(3.1746031746031746e-03*Jinv02*w12-7.9365079365079365e-04*w7*Jinv10+3.1746031746031746e-03*Jinv10*w22-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0-7.9365079365079365e-04*w7*Jinv11+6.3492063492063492e-03*w24*Jinv00+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21-7.9365079365079365e-04*Jinv20*w2+6.3492063492063492e-03*w25*Jinv10+3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+3.1746031746031746e-03*w13*Jinv10+3.1746031746031746e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*w26*Jinv21+6.3492063492063492e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1+6.3492063492063492e-03*w26*Jinv22+6.3492063492063492e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+6.3492063492063492e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-7.9365079365079365e-04*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[26][3]=(0.0)*D;
  A[26][4]=(0.0)*D;
  A[26][5]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*w4*Jinv12-6.3492063492063492e-03*Jinv02*w12-3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22-3.1746031746031746e-03*Jinv10*w22-3.1746031746031746e-03*Jinv02*w9+1.5873015873015873e-03*Jinv12*w7+1.5873015873015873e-03*w7*Jinv11+3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-3.1746031746031746e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-6.3492063492063492e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*w20*Jinv20-6.3492063492063492e-03*Jinv11*w22+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*Jinv00*w21-6.3492063492063492e-03*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11-3.1746031746031746e-03*Jinv00*w12-6.3492063492063492e-03*w14*Jinv22-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-3.1746031746031746e-03*Jinv20*w23-6.3492063492063492e-03*w25*Jinv12-6.3492063492063492e-03*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-6.3492063492063492e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-6.3492063492063492e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15-6.3492063492063492e-03*w14*Jinv21+1.5873015873015873e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-6.3492063492063492e-03*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*w17*Jinv22-3.1746031746031746e-03*Jinv02*w3+3.1746031746031746e-03*Jinv00*w18-6.3492063492063492e-03*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv11+1.5873015873015873e-03*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21+3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21+1.5873015873015873e-03*Jinv12*w16-3.1746031746031746e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[26][6]=(0.0)*D;
  A[26][7]=(0.0)*D;
  A[26][8]=(7.9365079365079365e-04*w7*Jinv10+7.9365079365079365e-04*Jinv00*w0-6.3492063492063492e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00-3.1746031746031746e-03*w10*Jinv10+7.9365079365079365e-04*Jinv20*w2-6.3492063492063492e-03*w25*Jinv10-3.1746031746031746e-03*w20*Jinv20+7.9365079365079365e-04*Jinv00*w6+7.9365079365079365e-04*Jinv20*w8+7.9365079365079365e-04*w1*Jinv10-3.1746031746031746e-03*Jinv00*w18-3.1746031746031746e-03*Jinv20*w11-3.1746031746031746e-03*w19*Jinv10-6.3492063492063492e-03*w26*Jinv20)*D;
  A[26][9]=(0.0)*D;
  A[26][10]=(0.0)*D;
  A[26][11]=(-3.1746031746031746e-03*w4*Jinv12-9.5238095238095247e-03*Jinv02*w12-6.3492063492063492e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-6.3492063492063492e-03*Jinv10*w22-9.5238095238095247e-03*Jinv02*w9+3.1746031746031746e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w0+1.5873015873015873e-03*w7*Jinv11-1.9047619047619049e-02*w24*Jinv00-9.5238095238095247e-03*w9*Jinv00-6.3492063492063492e-03*w19*Jinv12-1.5873015873015873e-03*Jinv11*w16-3.1746031746031746e-03*Jinv22*w5+3.1746031746031746e-03*Jinv02*w27-9.5238095238095247e-03*w10*Jinv10-6.3492063492063492e-03*Jinv02*w18-3.1746031746031746e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+1.5873015873015873e-03*w8*Jinv21-3.1746031746031746e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-1.9047619047619049e-02*w25*Jinv10-6.3492063492063492e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22+3.1746031746031746e-03*Jinv12*w28-6.3492063492063492e-03*Jinv00*w21-1.2698412698412698e-02*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w8*Jinv22+3.1746031746031746e-03*Jinv12*w1-3.1746031746031746e-03*w10*Jinv11+3.1746031746031746e-03*w28*Jinv10-9.5238095238095247e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-9.5238095238095247e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-9.5238095238095247e-03*w13*Jinv10-6.3492063492063492e-03*Jinv20*w23-1.9047619047619049e-02*w25*Jinv12-1.2698412698412698e-02*w26*Jinv21-3.1746031746031746e-03*w11*Jinv21-1.9047619047619049e-02*Jinv02*w24-9.5238095238095247e-03*w10*Jinv12+3.1746031746031746e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-3.1746031746031746e-03*Jinv01*w21+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+1.5873015873015873e-03*Jinv01*w6+3.1746031746031746e-03*Jinv00*w27-6.3492063492063492e-03*w13*Jinv11-9.5238095238095247e-03*w11*Jinv22+1.5873015873015873e-03*Jinv11*w1-1.9047619047619049e-02*w26*Jinv22+1.5873015873015873e-03*w28*Jinv11+3.1746031746031746e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3-6.3492063492063492e-03*Jinv00*w18-1.2698412698412698e-02*Jinv01*w24+3.1746031746031746e-03*w2*Jinv22+1.5873015873015873e-03*w29*Jinv21-3.1746031746031746e-03*w4*Jinv10-1.5873015873015873e-03*Jinv01*w15-9.5238095238095247e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-6.3492063492063492e-03*w19*Jinv10-9.5238095238095247e-03*Jinv12*w13+1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21-1.9047619047619049e-02*w26*Jinv20-9.5238095238095247e-03*Jinv20*w14+3.1746031746031746e-03*w29*Jinv22+1.5873015873015873e-03*w2*Jinv21+3.1746031746031746e-03*Jinv02*w6)*D;
  A[26][12]=(0.0)*D;
  A[26][13]=(0.0)*D;
  A[26][14]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*Jinv01*w18-3.1746031746031746e-03*w7*Jinv10+6.3492063492063492e-03*Jinv10*w22-3.1746031746031746e-03*Jinv00*w0-1.5873015873015873e-03*w7*Jinv11+1.9047619047619049e-02*w24*Jinv00+9.5238095238095247e-03*w9*Jinv00-1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*w20*Jinv21+9.5238095238095247e-03*w10*Jinv10+6.3492063492063492e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-1.5873015873015873e-03*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9-3.1746031746031746e-03*Jinv20*w2+1.9047619047619049e-02*w25*Jinv10+6.3492063492063492e-03*w20*Jinv20+6.3492063492063492e-03*Jinv11*w22+6.3492063492063492e-03*Jinv00*w21+6.3492063492063492e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27-3.1746031746031746e-03*Jinv00*w6+3.1746031746031746e-03*w10*Jinv11-3.1746031746031746e-03*w28*Jinv10+9.5238095238095247e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+3.1746031746031746e-03*w3*Jinv00+9.5238095238095247e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+6.3492063492063492e-03*Jinv20*w23+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+6.3492063492063492e-03*Jinv01*w21-3.1746031746031746e-03*w1*Jinv10+6.3492063492063492e-03*w14*Jinv21+3.1746031746031746e-03*Jinv20*w5-1.5873015873015873e-03*Jinv01*w6-3.1746031746031746e-03*Jinv00*w27+6.3492063492063492e-03*w13*Jinv11-1.5873015873015873e-03*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11-3.1746031746031746e-03*Jinv20*w29+6.3492063492063492e-03*Jinv00*w18+6.3492063492063492e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+3.1746031746031746e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv10+3.1746031746031746e-03*w4*Jinv11-1.5873015873015873e-03*Jinv01*w15+9.5238095238095247e-03*Jinv20*w11+6.3492063492063492e-03*w19*Jinv10-1.5873015873015873e-03*Jinv01*w0-1.5873015873015873e-03*w17*Jinv21+1.9047619047619049e-02*w26*Jinv20+9.5238095238095247e-03*Jinv20*w14-1.5873015873015873e-03*w2*Jinv21)*D;
  A[26][15]=(0.0)*D;
  A[26][16]=(0.0)*D;
  A[26][17]=(-7.9365079365079365e-04*w7*Jinv11+1.5873015873015873e-03*Jinv11*w16+3.1746031746031746e-03*Jinv01*w12-7.9365079365079365e-04*w8*Jinv21+3.1746031746031746e-03*Jinv01*w9+6.3492063492063492e-03*w25*Jinv11-1.5873015873015873e-03*Jinv01*w27+3.1746031746031746e-03*w10*Jinv11+6.3492063492063492e-03*w26*Jinv21+3.1746031746031746e-03*w11*Jinv21+3.1746031746031746e-03*w14*Jinv21-7.9365079365079365e-04*Jinv01*w6+3.1746031746031746e-03*w13*Jinv11-7.9365079365079365e-04*Jinv11*w1-1.5873015873015873e-03*w28*Jinv11+6.3492063492063492e-03*Jinv01*w24-1.5873015873015873e-03*w29*Jinv21+1.5873015873015873e-03*Jinv01*w15-7.9365079365079365e-04*Jinv01*w0+1.5873015873015873e-03*w17*Jinv21-7.9365079365079365e-04*w2*Jinv21)*D;
  A[26][18]=(0.0)*D;
  A[26][19]=(0.0)*D;
  A[26][20]=(-3.1746031746031746e-03*w5*Jinv21-3.1746031746031746e-03*Jinv02*w12-9.5238095238095247e-03*Jinv01*w18-3.1746031746031746e-03*w20*Jinv22+3.1746031746031746e-03*w7*Jinv10-9.5238095238095247e-03*Jinv10*w22+1.5873015873015873e-03*Jinv12*w7+3.1746031746031746e-03*Jinv00*w15+3.1746031746031746e-03*Jinv00*w0+3.1746031746031746e-03*w7*Jinv11-1.9047619047619049e-02*w24*Jinv00-6.3492063492063492e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv11*w16-1.5873015873015873e-03*Jinv02*w27-9.5238095238095247e-03*w20*Jinv21-6.3492063492063492e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18-9.5238095238095247e-03*w23*Jinv21-6.3492063492063492e-03*Jinv01*w12+3.1746031746031746e-03*Jinv10*w16+3.1746031746031746e-03*w8*Jinv21-6.3492063492063492e-03*Jinv01*w9+3.1746031746031746e-03*Jinv20*w2-1.9047619047619049e-02*w25*Jinv10+3.1746031746031746e-03*Jinv20*w17-9.5238095238095247e-03*w20*Jinv20-9.5238095238095247e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28-9.5238095238095247e-03*Jinv00*w21-1.9047619047619049e-02*w25*Jinv11-6.3492063492063492e-03*w23*Jinv22+3.1746031746031746e-03*Jinv00*w6+1.5873015873015873e-03*w8*Jinv22+1.5873015873015873e-03*Jinv12*w1-6.3492063492063492e-03*w10*Jinv11-6.3492063492063492e-03*Jinv00*w12+3.1746031746031746e-03*Jinv20*w8-3.1746031746031746e-03*w14*Jinv22-3.1746031746031746e-03*w3*Jinv00-6.3492063492063492e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-9.5238095238095247e-03*Jinv20*w23-1.2698412698412698e-02*w25*Jinv12-1.9047619047619049e-02*w26*Jinv21-6.3492063492063492e-03*w11*Jinv21-1.2698412698412698e-02*Jinv02*w24+1.5873015873015873e-03*Jinv02*w0-6.3492063492063492e-03*Jinv12*w22-9.5238095238095247e-03*Jinv01*w21+1.5873015873015873e-03*Jinv02*w15+3.1746031746031746e-03*w1*Jinv10-6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv20*w5+3.1746031746031746e-03*Jinv01*w6-6.3492063492063492e-03*w13*Jinv11+3.1746031746031746e-03*Jinv11*w1-1.2698412698412698e-02*w26*Jinv22+1.5873015873015873e-03*w17*Jinv22-9.5238095238095247e-03*Jinv00*w18-1.9047619047619049e-02*Jinv01*w24+1.5873015873015873e-03*w2*Jinv22-9.5238095238095247e-03*w19*Jinv11-3.1746031746031746e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+3.1746031746031746e-03*Jinv01*w15-6.3492063492063492e-03*Jinv20*w11-6.3492063492063492e-03*Jinv02*w21-9.5238095238095247e-03*w19*Jinv10-3.1746031746031746e-03*Jinv12*w13+3.1746031746031746e-03*Jinv01*w0+3.1746031746031746e-03*w17*Jinv21-1.9047619047619049e-02*w26*Jinv20+1.5873015873015873e-03*Jinv12*w16-6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22+3.1746031746031746e-03*w2*Jinv21+1.5873015873015873e-03*Jinv02*w6)*D;
  A[26][21]=(0.0)*D;
  A[26][22]=(0.0)*D;
  A[26][23]=(3.1746031746031746e-03*w4*Jinv12+6.3492063492063492e-03*Jinv02*w12+3.1746031746031746e-03*w20*Jinv22-3.1746031746031746e-03*w7*Jinv10+9.5238095238095247e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-1.5873015873015873e-03*Jinv12*w7-3.1746031746031746e-03*Jinv00*w15-3.1746031746031746e-03*Jinv00*w0+1.9047619047619049e-02*w24*Jinv00+6.3492063492063492e-03*w9*Jinv00+3.1746031746031746e-03*w19*Jinv12+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+6.3492063492063492e-03*w10*Jinv10+3.1746031746031746e-03*Jinv02*w18-3.1746031746031746e-03*Jinv10*w16-3.1746031746031746e-03*Jinv20*w2+1.9047619047619049e-02*w25*Jinv10-3.1746031746031746e-03*Jinv20*w17+9.5238095238095247e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+9.5238095238095247e-03*Jinv00*w21+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*Jinv00*w6-1.5873015873015873e-03*w8*Jinv22-1.5873015873015873e-03*Jinv12*w1+6.3492063492063492e-03*Jinv00*w12-3.1746031746031746e-03*Jinv20*w8+6.3492063492063492e-03*w14*Jinv22+3.1746031746031746e-03*w3*Jinv00+6.3492063492063492e-03*w13*Jinv10+9.5238095238095247e-03*Jinv20*w23+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-1.5873015873015873e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15-3.1746031746031746e-03*w1*Jinv10+3.1746031746031746e-03*Jinv20*w5+3.1746031746031746e-03*w11*Jinv22+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3+9.5238095238095247e-03*Jinv00*w18-1.5873015873015873e-03*w2*Jinv22+3.1746031746031746e-03*w4*Jinv10+6.3492063492063492e-03*Jinv20*w11+6.3492063492063492e-03*Jinv02*w21+9.5238095238095247e-03*w19*Jinv10+6.3492063492063492e-03*Jinv12*w13+1.9047619047619049e-02*w26*Jinv20-1.5873015873015873e-03*Jinv12*w16+6.3492063492063492e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-1.5873015873015873e-03*Jinv02*w6)*D;
  A[26][24]=(0.0)*D;
  A[26][25]=(0.0)*D;
  A[26][26]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+9.5238095238095247e-03*Jinv02*w12+9.5238095238095247e-03*Jinv01*w18+6.3492063492063492e-03*w20*Jinv22+9.5238095238095247e-03*Jinv02*w9-3.1746031746031746e-03*Jinv12*w7-3.1746031746031746e-03*w7*Jinv11+6.3492063492063492e-03*w19*Jinv12-3.1746031746031746e-03*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+9.5238095238095247e-03*w20*Jinv21+6.3492063492063492e-03*Jinv02*w18+9.5238095238095247e-03*w23*Jinv21+6.3492063492063492e-03*Jinv01*w12-3.1746031746031746e-03*w8*Jinv21+6.3492063492063492e-03*Jinv01*w9+9.5238095238095247e-03*Jinv11*w22-3.1746031746031746e-03*Jinv12*w28+1.9047619047619049e-02*w25*Jinv11+6.3492063492063492e-03*w23*Jinv22-3.1746031746031746e-03*w8*Jinv22-3.1746031746031746e-03*Jinv12*w1+6.3492063492063492e-03*w10*Jinv11+9.5238095238095247e-03*w14*Jinv22+3.1746031746031746e-03*Jinv01*w3+1.9047619047619049e-02*w25*Jinv12+1.9047619047619049e-02*w26*Jinv21+6.3492063492063492e-03*w11*Jinv21+1.9047619047619049e-02*Jinv02*w24+9.5238095238095247e-03*w10*Jinv12-3.1746031746031746e-03*Jinv02*w0+6.3492063492063492e-03*Jinv12*w22+9.5238095238095247e-03*Jinv01*w21+6.3492063492063492e-03*w14*Jinv21-3.1746031746031746e-03*Jinv01*w6+6.3492063492063492e-03*w13*Jinv11+9.5238095238095247e-03*w11*Jinv22-3.1746031746031746e-03*Jinv11*w1+1.9047619047619049e-02*w26*Jinv22+3.1746031746031746e-03*Jinv02*w3+1.9047619047619049e-02*Jinv01*w24-3.1746031746031746e-03*w2*Jinv22+9.5238095238095247e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-3.1746031746031746e-03*Jinv01*w15+6.3492063492063492e-03*Jinv02*w21+9.5238095238095247e-03*Jinv12*w13-3.1746031746031746e-03*Jinv01*w0-3.1746031746031746e-03*w17*Jinv21-3.1746031746031746e-03*w29*Jinv22-3.1746031746031746e-03*w2*Jinv21-3.1746031746031746e-03*Jinv02*w6)*D;
  A[26][27]=(0.0)*D;
  A[26][28]=(0.0)*D;
  A[26][29]=(3.1746031746031746e-03*w20*Jinv22-7.9365079365079365e-04*Jinv12*w7+3.1746031746031746e-03*w19*Jinv12+1.5873015873015873e-03*Jinv02*w27+3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28+3.1746031746031746e-03*w23*Jinv22-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1+6.3492063492063492e-03*w25*Jinv12+6.3492063492063492e-03*Jinv02*w24-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22-1.5873015873015873e-03*Jinv02*w15+6.3492063492063492e-03*w26*Jinv22-1.5873015873015873e-03*w17*Jinv22-7.9365079365079365e-04*w2*Jinv22+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*Jinv12*w16+1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*Jinv02*w6)*D;
  A[27][0]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*w4*Jinv12-7.9365079365079365e-04*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-3.9682539682539683e-04*w7*Jinv10+7.9365079365079365e-04*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7-3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-3.9682539682539683e-04*w7*Jinv11+7.9365079365079365e-04*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+7.9365079365079365e-04*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18-7.9365079365079365e-04*Jinv01*w12-3.9682539682539683e-04*Jinv10*w16-3.9682539682539683e-04*w8*Jinv21+7.9365079365079365e-04*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2-3.9682539682539683e-04*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27-3.9682539682539683e-04*Jinv00*w6-3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w14*Jinv22+7.9365079365079365e-04*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10+7.9365079365079365e-04*Jinv01*w3+7.9365079365079365e-04*w11*Jinv21+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-3.9682539682539683e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*w14*Jinv21+7.9365079365079365e-04*Jinv20*w5-3.9682539682539683e-04*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*w13*Jinv11+7.9365079365079365e-04*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11-3.9682539682539683e-04*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29+7.9365079365079365e-04*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+3.9682539682539683e-04*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+7.9365079365079365e-04*Jinv20*w11+1.5873015873015873e-03*w19*Jinv10-7.9365079365079365e-04*Jinv12*w13+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-3.9682539682539683e-04*Jinv12*w16-7.9365079365079365e-04*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21-3.9682539682539683e-04*Jinv02*w6)*D;
  A[27][1]=(0.0)*D;
  A[27][2]=(0.0)*D;
  A[27][3]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11-7.9365079365079365e-04*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[27][4]=(0.0)*D;
  A[27][5]=(0.0)*D;
  A[27][6]=(-3.9682539682539683e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w10*Jinv10+3.9682539682539683e-04*Jinv10*w16+3.9682539682539683e-04*Jinv20*w2+3.9682539682539683e-04*Jinv20*w17-1.5873015873015873e-03*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29-7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*Jinv20*w11-7.9365079365079365e-04*Jinv20*w14)*D;
  A[27][7]=(0.0)*D;
  A[27][8]=(0.0)*D;
  A[27][9]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[27][10]=(0.0)*D;
  A[27][11]=(0.0)*D;
  A[27][12]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+1.5873015873015873e-03*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14+7.9365079365079365e-04*w2*Jinv21)*D;
  A[27][13]=(0.0)*D;
  A[27][14]=(0.0)*D;
  A[27][15]=(7.9365079365079365e-04*w5*Jinv21+3.9682539682539683e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*w8*Jinv21-7.9365079365079365e-04*Jinv01*w9-1.5873015873015873e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*w10*Jinv11+7.9365079365079365e-04*Jinv01*w3-1.5873015873015873e-03*w26*Jinv21-7.9365079365079365e-04*w11*Jinv21-7.9365079365079365e-04*w14*Jinv21+3.9682539682539683e-04*Jinv01*w6-7.9365079365079365e-04*w13*Jinv11+3.9682539682539683e-04*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+7.9365079365079365e-04*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21+3.9682539682539683e-04*w2*Jinv21)*D;
  A[27][16]=(0.0)*D;
  A[27][17]=(0.0)*D;
  A[27][18]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+1.5873015873015873e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+1.5873015873015873e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22-4.7619047619047623e-03*Jinv01*w27+7.9365079365079365e-04*w8*Jinv22+1.5873015873015873e-03*w10*Jinv11-4.7619047619047623e-03*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3-1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w11*Jinv21-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-1.5873015873015873e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-4.7619047619047623e-03*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22-4.7619047619047623e-03*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22-4.7619047619047623e-03*Jinv20*w29-1.5873015873015873e-03*Jinv02*w3-4.7619047619047623e-03*w29*Jinv21+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11+1.5873015873015873e-03*Jinv20*w11-1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[27][19]=(0.0)*D;
  A[27][20]=(0.0)*D;
  A[27][21]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22-1.5873015873015873e-03*Jinv02*w9-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv12*w1+4.7619047619047623e-03*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*Jinv20*w5+4.7619047619047623e-03*Jinv00*w27-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*w17*Jinv22+4.7619047619047623e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22)*D;
  A[27][22]=(0.0)*D;
  A[27][23]=(0.0)*D;
  A[27][24]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*Jinv02*w18-1.5873015873015873e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+1.5873015873015873e-03*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22+4.7619047619047623e-03*Jinv01*w27+7.9365079365079365e-04*w8*Jinv22+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-1.5873015873015873e-03*w14*Jinv21-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*w26*Jinv22+4.7619047619047623e-03*w28*Jinv11-1.5873015873015873e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22+4.7619047619047623e-03*w29*Jinv21-1.5873015873015873e-03*w4*Jinv11-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[27][25]=(0.0)*D;
  A[27][26]=(0.0)*D;
  A[27][27]=(7.9365079365079365e-04*w4*Jinv12+7.9365079365079365e-04*Jinv02*w12+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv22*w5+2.3809523809523812e-03*Jinv02*w27+1.5873015873015873e-03*Jinv02*w18+2.3809523809523812e-03*Jinv12*w28+1.5873015873015873e-03*w23*Jinv22-3.9682539682539683e-04*w8*Jinv22-3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w14*Jinv22+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12-3.9682539682539683e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-3.9682539682539683e-04*Jinv02*w15+7.9365079365079365e-04*w11*Jinv22+1.5873015873015873e-03*w26*Jinv22-3.9682539682539683e-04*w17*Jinv22+7.9365079365079365e-04*Jinv02*w3-3.9682539682539683e-04*w2*Jinv22+1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv12*w13-3.9682539682539683e-04*Jinv12*w16+2.3809523809523812e-03*w29*Jinv22-3.9682539682539683e-04*Jinv02*w6)*D;
  A[27][28]=(0.0)*D;
  A[27][29]=(0.0)*D;
  A[28][0]=(0.0)*D;
  A[28][1]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*w4*Jinv12-7.9365079365079365e-04*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-3.9682539682539683e-04*w7*Jinv10+7.9365079365079365e-04*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7-3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-3.9682539682539683e-04*w7*Jinv11+7.9365079365079365e-04*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+7.9365079365079365e-04*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18-7.9365079365079365e-04*Jinv01*w12-3.9682539682539683e-04*Jinv10*w16-3.9682539682539683e-04*w8*Jinv21+7.9365079365079365e-04*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2-3.9682539682539683e-04*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27-3.9682539682539683e-04*Jinv00*w6-3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w14*Jinv22+7.9365079365079365e-04*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10+7.9365079365079365e-04*Jinv01*w3+7.9365079365079365e-04*w11*Jinv21+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-3.9682539682539683e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*w14*Jinv21+7.9365079365079365e-04*Jinv20*w5-3.9682539682539683e-04*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*w13*Jinv11+7.9365079365079365e-04*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11-3.9682539682539683e-04*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29+7.9365079365079365e-04*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+3.9682539682539683e-04*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+7.9365079365079365e-04*Jinv20*w11+1.5873015873015873e-03*w19*Jinv10-7.9365079365079365e-04*Jinv12*w13+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-3.9682539682539683e-04*Jinv12*w16-7.9365079365079365e-04*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21-3.9682539682539683e-04*Jinv02*w6)*D;
  A[28][2]=(0.0)*D;
  A[28][3]=(0.0)*D;
  A[28][4]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11-7.9365079365079365e-04*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[28][5]=(0.0)*D;
  A[28][6]=(0.0)*D;
  A[28][7]=(-3.9682539682539683e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w10*Jinv10+3.9682539682539683e-04*Jinv10*w16+3.9682539682539683e-04*Jinv20*w2+3.9682539682539683e-04*Jinv20*w17-1.5873015873015873e-03*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29-7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*Jinv20*w11-7.9365079365079365e-04*Jinv20*w14)*D;
  A[28][8]=(0.0)*D;
  A[28][9]=(0.0)*D;
  A[28][10]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[28][11]=(0.0)*D;
  A[28][12]=(0.0)*D;
  A[28][13]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+1.5873015873015873e-03*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14+7.9365079365079365e-04*w2*Jinv21)*D;
  A[28][14]=(0.0)*D;
  A[28][15]=(0.0)*D;
  A[28][16]=(7.9365079365079365e-04*w5*Jinv21+3.9682539682539683e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*w8*Jinv21-7.9365079365079365e-04*Jinv01*w9-1.5873015873015873e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*w10*Jinv11+7.9365079365079365e-04*Jinv01*w3-1.5873015873015873e-03*w26*Jinv21-7.9365079365079365e-04*w11*Jinv21-7.9365079365079365e-04*w14*Jinv21+3.9682539682539683e-04*Jinv01*w6-7.9365079365079365e-04*w13*Jinv11+3.9682539682539683e-04*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+7.9365079365079365e-04*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21+3.9682539682539683e-04*w2*Jinv21)*D;
  A[28][17]=(0.0)*D;
  A[28][18]=(0.0)*D;
  A[28][19]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+1.5873015873015873e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+1.5873015873015873e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22-4.7619047619047623e-03*Jinv01*w27+7.9365079365079365e-04*w8*Jinv22+1.5873015873015873e-03*w10*Jinv11-4.7619047619047623e-03*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3-1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w11*Jinv21-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-1.5873015873015873e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-4.7619047619047623e-03*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22-4.7619047619047623e-03*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22-4.7619047619047623e-03*Jinv20*w29-1.5873015873015873e-03*Jinv02*w3-4.7619047619047623e-03*w29*Jinv21+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11+1.5873015873015873e-03*Jinv20*w11-1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[28][20]=(0.0)*D;
  A[28][21]=(0.0)*D;
  A[28][22]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22-1.5873015873015873e-03*Jinv02*w9-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv12*w1+4.7619047619047623e-03*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*Jinv20*w5+4.7619047619047623e-03*Jinv00*w27-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*w17*Jinv22+4.7619047619047623e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22)*D;
  A[28][23]=(0.0)*D;
  A[28][24]=(0.0)*D;
  A[28][25]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*Jinv02*w18-1.5873015873015873e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+1.5873015873015873e-03*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22+4.7619047619047623e-03*Jinv01*w27+7.9365079365079365e-04*w8*Jinv22+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-1.5873015873015873e-03*w14*Jinv21-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*w26*Jinv22+4.7619047619047623e-03*w28*Jinv11-1.5873015873015873e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22+4.7619047619047623e-03*w29*Jinv21-1.5873015873015873e-03*w4*Jinv11-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[28][26]=(0.0)*D;
  A[28][27]=(0.0)*D;
  A[28][28]=(7.9365079365079365e-04*w4*Jinv12+7.9365079365079365e-04*Jinv02*w12+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv22*w5+2.3809523809523812e-03*Jinv02*w27+1.5873015873015873e-03*Jinv02*w18+2.3809523809523812e-03*Jinv12*w28+1.5873015873015873e-03*w23*Jinv22-3.9682539682539683e-04*w8*Jinv22-3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w14*Jinv22+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12-3.9682539682539683e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-3.9682539682539683e-04*Jinv02*w15+7.9365079365079365e-04*w11*Jinv22+1.5873015873015873e-03*w26*Jinv22-3.9682539682539683e-04*w17*Jinv22+7.9365079365079365e-04*Jinv02*w3-3.9682539682539683e-04*w2*Jinv22+1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv12*w13-3.9682539682539683e-04*Jinv12*w16+2.3809523809523812e-03*w29*Jinv22-3.9682539682539683e-04*Jinv02*w6)*D;
  A[28][29]=(0.0)*D;
  A[29][0]=(0.0)*D;
  A[29][1]=(0.0)*D;
  A[29][2]=(7.9365079365079365e-04*w5*Jinv21+7.9365079365079365e-04*w4*Jinv12-7.9365079365079365e-04*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-3.9682539682539683e-04*w7*Jinv10+7.9365079365079365e-04*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7-3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0-3.9682539682539683e-04*w7*Jinv11+7.9365079365079365e-04*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-3.9682539682539683e-04*Jinv11*w16+7.9365079365079365e-04*Jinv22*w5+7.9365079365079365e-04*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21+7.9365079365079365e-04*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18-7.9365079365079365e-04*Jinv01*w12-3.9682539682539683e-04*Jinv10*w16-3.9682539682539683e-04*w8*Jinv21+7.9365079365079365e-04*Jinv01*w9+3.9682539682539683e-04*Jinv20*w2-3.9682539682539683e-04*Jinv20*w17+1.5873015873015873e-03*w20*Jinv20+7.9365079365079365e-04*Jinv12*w28+7.9365079365079365e-04*Jinv01*w27-3.9682539682539683e-04*Jinv00*w6-3.9682539682539683e-04*w8*Jinv22+3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w10*Jinv11+7.9365079365079365e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w14*Jinv22+7.9365079365079365e-04*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10+7.9365079365079365e-04*Jinv01*w3+7.9365079365079365e-04*w11*Jinv21+7.9365079365079365e-04*w10*Jinv12+3.9682539682539683e-04*Jinv02*w0-3.9682539682539683e-04*Jinv02*w15+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*w14*Jinv21+7.9365079365079365e-04*Jinv20*w5-3.9682539682539683e-04*Jinv01*w6+7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*w13*Jinv11+7.9365079365079365e-04*w11*Jinv22+3.9682539682539683e-04*Jinv11*w1+7.9365079365079365e-04*w28*Jinv11-3.9682539682539683e-04*w17*Jinv22+7.9365079365079365e-04*Jinv20*w29+7.9365079365079365e-04*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+3.9682539682539683e-04*w2*Jinv22+7.9365079365079365e-04*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+7.9365079365079365e-04*Jinv20*w11+1.5873015873015873e-03*w19*Jinv10-7.9365079365079365e-04*Jinv12*w13+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21-3.9682539682539683e-04*Jinv12*w16-7.9365079365079365e-04*Jinv20*w14+7.9365079365079365e-04*w29*Jinv22+3.9682539682539683e-04*w2*Jinv21-3.9682539682539683e-04*Jinv02*w6)*D;
  A[29][3]=(0.0)*D;
  A[29][4]=(0.0)*D;
  A[29][5]=(3.1746031746031746e-03*w5*Jinv21+3.1746031746031746e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12+1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+1.5873015873015873e-03*Jinv02*w9-7.9365079365079365e-04*Jinv00*w0-1.5873015873015873e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12-7.9365079365079365e-04*Jinv11*w16+3.1746031746031746e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27+1.5873015873015873e-03*w20*Jinv21-1.5873015873015873e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+3.1746031746031746e-03*w23*Jinv21+3.1746031746031746e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-7.9365079365079365e-04*Jinv20*w2-1.5873015873015873e-03*w20*Jinv20+3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+3.1746031746031746e-03*w23*Jinv22-1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*Jinv12*w1+1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w13*Jinv10+3.1746031746031746e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+1.5873015873015873e-03*w11*Jinv21+1.5873015873015873e-03*Jinv02*w24+1.5873015873015873e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+3.1746031746031746e-03*Jinv12*w22+3.1746031746031746e-03*Jinv01*w21-7.9365079365079365e-04*Jinv02*w15-7.9365079365079365e-04*w1*Jinv10+3.1746031746031746e-03*w14*Jinv21+3.1746031746031746e-03*w13*Jinv11+1.5873015873015873e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w26*Jinv22-1.5873015873015873e-03*w28*Jinv11-7.9365079365079365e-04*w17*Jinv22+3.1746031746031746e-03*Jinv02*w3-1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w29*Jinv21+1.5873015873015873e-03*w19*Jinv11+3.1746031746031746e-03*w4*Jinv11-7.9365079365079365e-04*Jinv01*w15-1.5873015873015873e-03*Jinv20*w11+3.1746031746031746e-03*Jinv02*w21-1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0-7.9365079365079365e-04*w17*Jinv21-7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21)*D;
  A[29][6]=(0.0)*D;
  A[29][7]=(0.0)*D;
  A[29][8]=(-3.9682539682539683e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+3.9682539682539683e-04*Jinv00*w15+3.9682539682539683e-04*Jinv00*w0+7.9365079365079365e-04*w9*Jinv00+7.9365079365079365e-04*w10*Jinv10+3.9682539682539683e-04*Jinv10*w16+3.9682539682539683e-04*Jinv20*w2+3.9682539682539683e-04*Jinv20*w17-1.5873015873015873e-03*Jinv00*w21-3.9682539682539683e-04*Jinv00*w6-7.9365079365079365e-04*w28*Jinv10-7.9365079365079365e-04*Jinv00*w12-3.9682539682539683e-04*Jinv20*w8-7.9365079365079365e-04*w3*Jinv00-7.9365079365079365e-04*w13*Jinv10-1.5873015873015873e-03*Jinv20*w23+3.9682539682539683e-04*w1*Jinv10-7.9365079365079365e-04*Jinv20*w5-7.9365079365079365e-04*Jinv00*w27-7.9365079365079365e-04*Jinv20*w29-7.9365079365079365e-04*w4*Jinv10+7.9365079365079365e-04*Jinv20*w11-7.9365079365079365e-04*Jinv20*w14)*D;
  A[29][9]=(0.0)*D;
  A[29][10]=(0.0)*D;
  A[29][11]=(-1.5873015873015873e-03*w5*Jinv21+1.5873015873015873e-03*w4*Jinv12+3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*Jinv01*w18+1.5873015873015873e-03*w20*Jinv22-7.9365079365079365e-04*w7*Jinv10+1.5873015873015873e-03*Jinv10*w22+3.1746031746031746e-03*Jinv02*w9-7.9365079365079365e-04*Jinv12*w7-7.9365079365079365e-04*Jinv00*w0+3.1746031746031746e-03*w24*Jinv00+3.1746031746031746e-03*w9*Jinv00+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv11*w16+1.5873015873015873e-03*Jinv22*w5-1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*w20*Jinv21+3.1746031746031746e-03*w10*Jinv10+1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12-7.9365079365079365e-04*Jinv20*w2+3.1746031746031746e-03*w25*Jinv10+1.5873015873015873e-03*w20*Jinv20-1.5873015873015873e-03*Jinv12*w28+1.5873015873015873e-03*Jinv00*w21+1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*w23*Jinv22-7.9365079365079365e-04*Jinv00*w6-7.9365079365079365e-04*w8*Jinv22-7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w28*Jinv10+3.1746031746031746e-03*Jinv00*w12-7.9365079365079365e-04*Jinv20*w8+3.1746031746031746e-03*w14*Jinv22+1.5873015873015873e-03*w3*Jinv00+3.1746031746031746e-03*w13*Jinv10-1.5873015873015873e-03*Jinv01*w3+1.5873015873015873e-03*Jinv20*w23+3.1746031746031746e-03*w25*Jinv12+1.5873015873015873e-03*w26*Jinv21+3.1746031746031746e-03*Jinv02*w24+3.1746031746031746e-03*w10*Jinv12-7.9365079365079365e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-7.9365079365079365e-04*w1*Jinv10+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-1.5873015873015873e-03*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11+3.1746031746031746e-03*w11*Jinv22-7.9365079365079365e-04*Jinv11*w1+3.1746031746031746e-03*w26*Jinv22-1.5873015873015873e-03*Jinv20*w29+1.5873015873015873e-03*Jinv02*w3+1.5873015873015873e-03*Jinv00*w18+1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w19*Jinv11+1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15+3.1746031746031746e-03*Jinv20*w11+1.5873015873015873e-03*Jinv02*w21+1.5873015873015873e-03*w19*Jinv10+3.1746031746031746e-03*Jinv12*w13-7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21+3.1746031746031746e-03*w26*Jinv20+3.1746031746031746e-03*Jinv20*w14-1.5873015873015873e-03*w29*Jinv22-7.9365079365079365e-04*w2*Jinv21-7.9365079365079365e-04*Jinv02*w6)*D;
  A[29][12]=(0.0)*D;
  A[29][13]=(0.0)*D;
  A[29][14]=(-3.1746031746031746e-03*w5*Jinv21-1.5873015873015873e-03*Jinv01*w18+7.9365079365079365e-04*w7*Jinv10-1.5873015873015873e-03*Jinv10*w22+7.9365079365079365e-04*Jinv00*w0-3.1746031746031746e-03*w24*Jinv00-3.1746031746031746e-03*w9*Jinv00+7.9365079365079365e-04*Jinv11*w16-1.5873015873015873e-03*w20*Jinv21-3.1746031746031746e-03*w10*Jinv10-3.1746031746031746e-03*w23*Jinv21-3.1746031746031746e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+7.9365079365079365e-04*Jinv20*w2-3.1746031746031746e-03*w25*Jinv10-1.5873015873015873e-03*w20*Jinv20-3.1746031746031746e-03*Jinv11*w22-1.5873015873015873e-03*Jinv00*w21-1.5873015873015873e-03*w25*Jinv11+1.5873015873015873e-03*Jinv01*w27+7.9365079365079365e-04*Jinv00*w6-1.5873015873015873e-03*w10*Jinv11+1.5873015873015873e-03*w28*Jinv10-3.1746031746031746e-03*Jinv00*w12+7.9365079365079365e-04*Jinv20*w8-1.5873015873015873e-03*w3*Jinv00-3.1746031746031746e-03*w13*Jinv10-3.1746031746031746e-03*Jinv01*w3-1.5873015873015873e-03*Jinv20*w23-1.5873015873015873e-03*w26*Jinv21-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv01*w21+7.9365079365079365e-04*w1*Jinv10-3.1746031746031746e-03*w14*Jinv21-1.5873015873015873e-03*Jinv20*w5+1.5873015873015873e-03*Jinv00*w27-3.1746031746031746e-03*w13*Jinv11+7.9365079365079365e-04*Jinv11*w1+1.5873015873015873e-03*w28*Jinv11+1.5873015873015873e-03*Jinv20*w29-1.5873015873015873e-03*Jinv00*w18-1.5873015873015873e-03*Jinv01*w24+1.5873015873015873e-03*w29*Jinv21-1.5873015873015873e-03*w19*Jinv11-1.5873015873015873e-03*w4*Jinv10-3.1746031746031746e-03*w4*Jinv11+7.9365079365079365e-04*Jinv01*w15-3.1746031746031746e-03*Jinv20*w11-1.5873015873015873e-03*w19*Jinv10+7.9365079365079365e-04*Jinv01*w0+7.9365079365079365e-04*w17*Jinv21-3.1746031746031746e-03*w26*Jinv20-3.1746031746031746e-03*Jinv20*w14+7.9365079365079365e-04*w2*Jinv21)*D;
  A[29][15]=(0.0)*D;
  A[29][16]=(0.0)*D;
  A[29][17]=(7.9365079365079365e-04*w5*Jinv21+3.9682539682539683e-04*w7*Jinv11-3.9682539682539683e-04*Jinv11*w16-7.9365079365079365e-04*Jinv01*w12+3.9682539682539683e-04*w8*Jinv21-7.9365079365079365e-04*Jinv01*w9-1.5873015873015873e-03*w25*Jinv11-7.9365079365079365e-04*Jinv01*w27-7.9365079365079365e-04*w10*Jinv11+7.9365079365079365e-04*Jinv01*w3-1.5873015873015873e-03*w26*Jinv21-7.9365079365079365e-04*w11*Jinv21-7.9365079365079365e-04*w14*Jinv21+3.9682539682539683e-04*Jinv01*w6-7.9365079365079365e-04*w13*Jinv11+3.9682539682539683e-04*Jinv11*w1-7.9365079365079365e-04*w28*Jinv11-1.5873015873015873e-03*Jinv01*w24-7.9365079365079365e-04*w29*Jinv21+7.9365079365079365e-04*w4*Jinv11-3.9682539682539683e-04*Jinv01*w15+3.9682539682539683e-04*Jinv01*w0-3.9682539682539683e-04*w17*Jinv21+3.9682539682539683e-04*w2*Jinv21)*D;
  A[29][18]=(0.0)*D;
  A[29][19]=(0.0)*D;
  A[29][20]=(1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*w20*Jinv22-1.5873015873015873e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7+1.5873015873015873e-03*w9*Jinv00-3.1746031746031746e-03*w19*Jinv12-1.5873015873015873e-03*Jinv22*w5-3.1746031746031746e-03*Jinv02*w27+1.5873015873015873e-03*w10*Jinv10-3.1746031746031746e-03*Jinv02*w18+1.5873015873015873e-03*Jinv01*w12+1.5873015873015873e-03*Jinv01*w9-3.1746031746031746e-03*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22-4.7619047619047623e-03*Jinv01*w27+7.9365079365079365e-04*w8*Jinv22+1.5873015873015873e-03*w10*Jinv11-4.7619047619047623e-03*w28*Jinv10+1.5873015873015873e-03*Jinv00*w12+1.5873015873015873e-03*w3*Jinv00+1.5873015873015873e-03*w13*Jinv10+1.5873015873015873e-03*Jinv01*w3-1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*w11*Jinv21-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12-1.5873015873015873e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15+1.5873015873015873e-03*w14*Jinv21+1.5873015873015873e-03*Jinv20*w5-4.7619047619047623e-03*Jinv00*w27+1.5873015873015873e-03*w13*Jinv11-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22-4.7619047619047623e-03*w28*Jinv11+7.9365079365079365e-04*w17*Jinv22-4.7619047619047623e-03*Jinv20*w29-1.5873015873015873e-03*Jinv02*w3-4.7619047619047623e-03*w29*Jinv21+1.5873015873015873e-03*w4*Jinv10+1.5873015873015873e-03*w4*Jinv11+1.5873015873015873e-03*Jinv20*w11-1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv12*w16+1.5873015873015873e-03*Jinv20*w14-3.1746031746031746e-03*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[29][21]=(0.0)*D;
  A[29][22]=(0.0)*D;
  A[29][23]=(-3.1746031746031746e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22-1.5873015873015873e-03*Jinv02*w9-1.5873015873015873e-03*w9*Jinv00-1.5873015873015873e-03*w19*Jinv12-3.1746031746031746e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*w10*Jinv10-1.5873015873015873e-03*Jinv02*w18+1.5873015873015873e-03*Jinv12*w28-3.1746031746031746e-03*w23*Jinv22+7.9365079365079365e-04*Jinv12*w1+4.7619047619047623e-03*w28*Jinv10-1.5873015873015873e-03*Jinv00*w12-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*w3*Jinv00-1.5873015873015873e-03*w13*Jinv10-1.5873015873015873e-03*w25*Jinv12-1.5873015873015873e-03*Jinv02*w24-1.5873015873015873e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-3.1746031746031746e-03*Jinv12*w22+7.9365079365079365e-04*Jinv02*w15-1.5873015873015873e-03*Jinv20*w5+4.7619047619047623e-03*Jinv00*w27-1.5873015873015873e-03*w11*Jinv22-1.5873015873015873e-03*w26*Jinv22+7.9365079365079365e-04*w17*Jinv22+4.7619047619047623e-03*Jinv20*w29-3.1746031746031746e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22-1.5873015873015873e-03*w4*Jinv10-1.5873015873015873e-03*Jinv20*w11-3.1746031746031746e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+7.9365079365079365e-04*Jinv12*w16-1.5873015873015873e-03*Jinv20*w14+1.5873015873015873e-03*w29*Jinv22)*D;
  A[29][24]=(0.0)*D;
  A[29][25]=(0.0)*D;
  A[29][26]=(-1.5873015873015873e-03*w5*Jinv21-1.5873015873015873e-03*w4*Jinv12-3.1746031746031746e-03*Jinv02*w12-1.5873015873015873e-03*w20*Jinv22-3.1746031746031746e-03*Jinv02*w9+7.9365079365079365e-04*Jinv12*w7-1.5873015873015873e-03*w19*Jinv12-1.5873015873015873e-03*Jinv22*w5+1.5873015873015873e-03*Jinv02*w27-1.5873015873015873e-03*Jinv02*w18-1.5873015873015873e-03*Jinv01*w12-1.5873015873015873e-03*Jinv01*w9+1.5873015873015873e-03*Jinv12*w28-1.5873015873015873e-03*w23*Jinv22+4.7619047619047623e-03*Jinv01*w27+7.9365079365079365e-04*w8*Jinv22+7.9365079365079365e-04*Jinv12*w1-1.5873015873015873e-03*w10*Jinv11-3.1746031746031746e-03*w14*Jinv22-1.5873015873015873e-03*Jinv01*w3-3.1746031746031746e-03*w25*Jinv12-1.5873015873015873e-03*w11*Jinv21-3.1746031746031746e-03*Jinv02*w24-3.1746031746031746e-03*w10*Jinv12+7.9365079365079365e-04*Jinv02*w0-1.5873015873015873e-03*Jinv12*w22-1.5873015873015873e-03*w14*Jinv21-1.5873015873015873e-03*w13*Jinv11-3.1746031746031746e-03*w11*Jinv22-3.1746031746031746e-03*w26*Jinv22+4.7619047619047623e-03*w28*Jinv11-1.5873015873015873e-03*Jinv02*w3+7.9365079365079365e-04*w2*Jinv22+4.7619047619047623e-03*w29*Jinv21-1.5873015873015873e-03*w4*Jinv11-1.5873015873015873e-03*Jinv02*w21-3.1746031746031746e-03*Jinv12*w13+1.5873015873015873e-03*w29*Jinv22+7.9365079365079365e-04*Jinv02*w6)*D;
  A[29][27]=(0.0)*D;
  A[29][28]=(0.0)*D;
  A[29][29]=(7.9365079365079365e-04*w4*Jinv12+7.9365079365079365e-04*Jinv02*w12+1.5873015873015873e-03*w20*Jinv22+7.9365079365079365e-04*Jinv02*w9-3.9682539682539683e-04*Jinv12*w7+1.5873015873015873e-03*w19*Jinv12+7.9365079365079365e-04*Jinv22*w5+2.3809523809523812e-03*Jinv02*w27+1.5873015873015873e-03*Jinv02*w18+2.3809523809523812e-03*Jinv12*w28+1.5873015873015873e-03*w23*Jinv22-3.9682539682539683e-04*w8*Jinv22-3.9682539682539683e-04*Jinv12*w1+7.9365079365079365e-04*w14*Jinv22+1.5873015873015873e-03*w25*Jinv12+1.5873015873015873e-03*Jinv02*w24+7.9365079365079365e-04*w10*Jinv12-3.9682539682539683e-04*Jinv02*w0+1.5873015873015873e-03*Jinv12*w22-3.9682539682539683e-04*Jinv02*w15+7.9365079365079365e-04*w11*Jinv22+1.5873015873015873e-03*w26*Jinv22-3.9682539682539683e-04*w17*Jinv22+7.9365079365079365e-04*Jinv02*w3-3.9682539682539683e-04*w2*Jinv22+1.5873015873015873e-03*Jinv02*w21+7.9365079365079365e-04*Jinv12*w13-3.9682539682539683e-04*Jinv12*w16+2.3809523809523812e-03*w29*Jinv22-3.9682539682539683e-04*Jinv02*w6)*D;
  int nbf = 30;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 ---------------*/


 
void matrix_factory_vector_conv_3D_order3 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[3], double pp1[3], double pp2[3], double pp3[3],
         double* vel, int size){
  
  
 
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; double z0 = pp0[2];  
  double x1 = pp1[0]; double y1 = pp1[1]; double z1 = pp1[2];  
  double x2 = pp2[0]; double y2 = pp2[1]; double z2 = pp2[2];  
  double x3 = pp3[0]; double y3 = pp3[1]; double z3 = pp3[2];  

  double G00 = x1 - x0; 
  double G10 = y1 - y0;  
  double G20 = z1 - z0;  

  double G01 = x2 - x0; 
  double G11 = y2 - y0; 
  double G21 = z2 - z0; 

  double G02 = x3 - x0; 
  double G12 = y3 - y0; 
  double G22 = z3 - z0; 
 
  double D =  G00*(G11*G22-G21*G12) 
            - G01*(G10*G22-G20*G12) 
            + G02*(G10*G21-G20*G11);

 

 
  int    iidof[60];
  double idof0[4];
  idof0[0]=x0;  idof0[1]=y0;  idof0[2]=z0;
  idof0[3] = 0.0;
  ptv ipdof0(4,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[4];
  idof1[0]=x0;  idof1[1]=y0;  idof1[2]=z0;
  idof1[3] = 1.0;
  ptv ipdof1(4,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[4];
  idof2[0]=x0;  idof2[1]=y0;  idof2[2]=z0;
  idof2[3] = 2.0;
  ptv ipdof2(4,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[4];
  idof3[0]=G00/3.0+x0;  idof3[1]=y0+G10/3.0;  idof3[2]=z0+G20/3.0;
  idof3[3] = 0.0;
  ptv ipdof3(4,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[4];
  idof4[0]=G00/3.0+x0;  idof4[1]=y0+G10/3.0;  idof4[2]=z0+G20/3.0;
  idof4[3] = 1.0;
  ptv ipdof4(4,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[4];
  idof5[0]=G00/3.0+x0;  idof5[1]=y0+G10/3.0;  idof5[2]=z0+G20/3.0;
  idof5[3] = 2.0;
  ptv ipdof5(4,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

  double idof6[4];
  idof6[0]=(2.0/3.0)*G00+x0;  idof6[1]=y0+(2.0/3.0)*G10;  idof6[2]=z0+(2.0/3.0)*G20;
  idof6[3] = 0.0;
  ptv ipdof6(4,idof6);
  iidof[6] = dof.insert_dof(element,6,ipdof6);

  double idof7[4];
  idof7[0]=(2.0/3.0)*G00+x0;  idof7[1]=y0+(2.0/3.0)*G10;  idof7[2]=z0+(2.0/3.0)*G20;
  idof7[3] = 1.0;
  ptv ipdof7(4,idof7);
  iidof[7] = dof.insert_dof(element,7,ipdof7);

  double idof8[4];
  idof8[0]=(2.0/3.0)*G00+x0;  idof8[1]=y0+(2.0/3.0)*G10;  idof8[2]=z0+(2.0/3.0)*G20;
  idof8[3] = 2.0;
  ptv ipdof8(4,idof8);
  iidof[8] = dof.insert_dof(element,8,ipdof8);

  double idof9[4];
  idof9[0]=G00+x0;  idof9[1]=y0+G10;  idof9[2]=z0+G20;
  idof9[3] = 0.0;
  ptv ipdof9(4,idof9);
  iidof[9] = dof.insert_dof(element,9,ipdof9);

  double idof10[4];
  idof10[0]=G00+x0;  idof10[1]=y0+G10;  idof10[2]=z0+G20;
  idof10[3] = 1.0;
  ptv ipdof10(4,idof10);
  iidof[10] = dof.insert_dof(element,10,ipdof10);

  double idof11[4];
  idof11[0]=G00+x0;  idof11[1]=y0+G10;  idof11[2]=z0+G20;
  idof11[3] = 2.0;
  ptv ipdof11(4,idof11);
  iidof[11] = dof.insert_dof(element,11,ipdof11);

  double idof12[4];
  idof12[0]=G01/3.0+x0;  idof12[1]=y0+G11/3.0;  idof12[2]=G21/3.0+z0;
  idof12[3] = 0.0;
  ptv ipdof12(4,idof12);
  iidof[12] = dof.insert_dof(element,12,ipdof12);

  double idof13[4];
  idof13[0]=G01/3.0+x0;  idof13[1]=y0+G11/3.0;  idof13[2]=G21/3.0+z0;
  idof13[3] = 1.0;
  ptv ipdof13(4,idof13);
  iidof[13] = dof.insert_dof(element,13,ipdof13);

  double idof14[4];
  idof14[0]=G01/3.0+x0;  idof14[1]=y0+G11/3.0;  idof14[2]=G21/3.0+z0;
  idof14[3] = 2.0;
  ptv ipdof14(4,idof14);
  iidof[14] = dof.insert_dof(element,14,ipdof14);

  double idof15[4];
  idof15[0]=G01/3.0+G00/3.0+x0;  idof15[1]=y0+G11/3.0+G10/3.0;  idof15[2]=G21/3.0+z0+G20/3.0;
  idof15[3] = 0.0;
  ptv ipdof15(4,idof15);
  iidof[15] = dof.insert_dof(element,15,ipdof15);

  double idof16[4];
  idof16[0]=G01/3.0+G00/3.0+x0;  idof16[1]=y0+G11/3.0+G10/3.0;  idof16[2]=G21/3.0+z0+G20/3.0;
  idof16[3] = 1.0;
  ptv ipdof16(4,idof16);
  iidof[16] = dof.insert_dof(element,16,ipdof16);

  double idof17[4];
  idof17[0]=G01/3.0+G00/3.0+x0;  idof17[1]=y0+G11/3.0+G10/3.0;  idof17[2]=G21/3.0+z0+G20/3.0;
  idof17[3] = 2.0;
  ptv ipdof17(4,idof17);
  iidof[17] = dof.insert_dof(element,17,ipdof17);

  double idof18[4];
  idof18[0]=G01/3.0+(2.0/3.0)*G00+x0;  idof18[1]=y0+G11/3.0+(2.0/3.0)*G10;  idof18[2]=G21/3.0+z0+(2.0/3.0)*G20;
  idof18[3] = 0.0;
  ptv ipdof18(4,idof18);
  iidof[18] = dof.insert_dof(element,18,ipdof18);

  double idof19[4];
  idof19[0]=G01/3.0+(2.0/3.0)*G00+x0;  idof19[1]=y0+G11/3.0+(2.0/3.0)*G10;  idof19[2]=G21/3.0+z0+(2.0/3.0)*G20;
  idof19[3] = 1.0;
  ptv ipdof19(4,idof19);
  iidof[19] = dof.insert_dof(element,19,ipdof19);

  double idof20[4];
  idof20[0]=G01/3.0+(2.0/3.0)*G00+x0;  idof20[1]=y0+G11/3.0+(2.0/3.0)*G10;  idof20[2]=G21/3.0+z0+(2.0/3.0)*G20;
  idof20[3] = 2.0;
  ptv ipdof20(4,idof20);
  iidof[20] = dof.insert_dof(element,20,ipdof20);

  double idof21[4];
  idof21[0]=(2.0/3.0)*G01+x0;  idof21[1]=y0+(2.0/3.0)*G11;  idof21[2]=(2.0/3.0)*G21+z0;
  idof21[3] = 0.0;
  ptv ipdof21(4,idof21);
  iidof[21] = dof.insert_dof(element,21,ipdof21);

  double idof22[4];
  idof22[0]=(2.0/3.0)*G01+x0;  idof22[1]=y0+(2.0/3.0)*G11;  idof22[2]=(2.0/3.0)*G21+z0;
  idof22[3] = 1.0;
  ptv ipdof22(4,idof22);
  iidof[22] = dof.insert_dof(element,22,ipdof22);

  double idof23[4];
  idof23[0]=(2.0/3.0)*G01+x0;  idof23[1]=y0+(2.0/3.0)*G11;  idof23[2]=(2.0/3.0)*G21+z0;
  idof23[3] = 2.0;
  ptv ipdof23(4,idof23);
  iidof[23] = dof.insert_dof(element,23,ipdof23);

  double idof24[4];
  idof24[0]=(2.0/3.0)*G01+G00/3.0+x0;  idof24[1]=y0+(2.0/3.0)*G11+G10/3.0;  idof24[2]=(2.0/3.0)*G21+z0+G20/3.0;
  idof24[3] = 0.0;
  ptv ipdof24(4,idof24);
  iidof[24] = dof.insert_dof(element,24,ipdof24);

  double idof25[4];
  idof25[0]=(2.0/3.0)*G01+G00/3.0+x0;  idof25[1]=y0+(2.0/3.0)*G11+G10/3.0;  idof25[2]=(2.0/3.0)*G21+z0+G20/3.0;
  idof25[3] = 1.0;
  ptv ipdof25(4,idof25);
  iidof[25] = dof.insert_dof(element,25,ipdof25);

  double idof26[4];
  idof26[0]=(2.0/3.0)*G01+G00/3.0+x0;  idof26[1]=y0+(2.0/3.0)*G11+G10/3.0;  idof26[2]=(2.0/3.0)*G21+z0+G20/3.0;
  idof26[3] = 2.0;
  ptv ipdof26(4,idof26);
  iidof[26] = dof.insert_dof(element,26,ipdof26);

  double idof27[4];
  idof27[0]=G01+x0;  idof27[1]=y0+G11;  idof27[2]=G21+z0;
  idof27[3] = 0.0;
  ptv ipdof27(4,idof27);
  iidof[27] = dof.insert_dof(element,27,ipdof27);

  double idof28[4];
  idof28[0]=G01+x0;  idof28[1]=y0+G11;  idof28[2]=G21+z0;
  idof28[3] = 1.0;
  ptv ipdof28(4,idof28);
  iidof[28] = dof.insert_dof(element,28,ipdof28);

  double idof29[4];
  idof29[0]=G01+x0;  idof29[1]=y0+G11;  idof29[2]=G21+z0;
  idof29[3] = 2.0;
  ptv ipdof29(4,idof29);
  iidof[29] = dof.insert_dof(element,29,ipdof29);

  double idof30[4];
  idof30[0]=G02/3.0+x0;  idof30[1]=G12/3.0+y0;  idof30[2]=G22/3.0+z0;
  idof30[3] = 0.0;
  ptv ipdof30(4,idof30);
  iidof[30] = dof.insert_dof(element,30,ipdof30);

  double idof31[4];
  idof31[0]=G02/3.0+x0;  idof31[1]=G12/3.0+y0;  idof31[2]=G22/3.0+z0;
  idof31[3] = 1.0;
  ptv ipdof31(4,idof31);
  iidof[31] = dof.insert_dof(element,31,ipdof31);

  double idof32[4];
  idof32[0]=G02/3.0+x0;  idof32[1]=G12/3.0+y0;  idof32[2]=G22/3.0+z0;
  idof32[3] = 2.0;
  ptv ipdof32(4,idof32);
  iidof[32] = dof.insert_dof(element,32,ipdof32);

  double idof33[4];
  idof33[0]=G02/3.0+G00/3.0+x0;  idof33[1]=G12/3.0+y0+G10/3.0;  idof33[2]=G22/3.0+z0+G20/3.0;
  idof33[3] = 0.0;
  ptv ipdof33(4,idof33);
  iidof[33] = dof.insert_dof(element,33,ipdof33);

  double idof34[4];
  idof34[0]=G02/3.0+G00/3.0+x0;  idof34[1]=G12/3.0+y0+G10/3.0;  idof34[2]=G22/3.0+z0+G20/3.0;
  idof34[3] = 1.0;
  ptv ipdof34(4,idof34);
  iidof[34] = dof.insert_dof(element,34,ipdof34);

  double idof35[4];
  idof35[0]=G02/3.0+G00/3.0+x0;  idof35[1]=G12/3.0+y0+G10/3.0;  idof35[2]=G22/3.0+z0+G20/3.0;
  idof35[3] = 2.0;
  ptv ipdof35(4,idof35);
  iidof[35] = dof.insert_dof(element,35,ipdof35);

  double idof36[4];
  idof36[0]=G02/3.0+(2.0/3.0)*G00+x0;  idof36[1]=G12/3.0+y0+(2.0/3.0)*G10;  idof36[2]=G22/3.0+z0+(2.0/3.0)*G20;
  idof36[3] = 0.0;
  ptv ipdof36(4,idof36);
  iidof[36] = dof.insert_dof(element,36,ipdof36);

  double idof37[4];
  idof37[0]=G02/3.0+(2.0/3.0)*G00+x0;  idof37[1]=G12/3.0+y0+(2.0/3.0)*G10;  idof37[2]=G22/3.0+z0+(2.0/3.0)*G20;
  idof37[3] = 1.0;
  ptv ipdof37(4,idof37);
  iidof[37] = dof.insert_dof(element,37,ipdof37);

  double idof38[4];
  idof38[0]=G02/3.0+(2.0/3.0)*G00+x0;  idof38[1]=G12/3.0+y0+(2.0/3.0)*G10;  idof38[2]=G22/3.0+z0+(2.0/3.0)*G20;
  idof38[3] = 2.0;
  ptv ipdof38(4,idof38);
  iidof[38] = dof.insert_dof(element,38,ipdof38);

  double idof39[4];
  idof39[0]=G02/3.0+G01/3.0+x0;  idof39[1]=G12/3.0+y0+G11/3.0;  idof39[2]=G22/3.0+G21/3.0+z0;
  idof39[3] = 0.0;
  ptv ipdof39(4,idof39);
  iidof[39] = dof.insert_dof(element,39,ipdof39);

  double idof40[4];
  idof40[0]=G02/3.0+G01/3.0+x0;  idof40[1]=G12/3.0+y0+G11/3.0;  idof40[2]=G22/3.0+G21/3.0+z0;
  idof40[3] = 1.0;
  ptv ipdof40(4,idof40);
  iidof[40] = dof.insert_dof(element,40,ipdof40);

  double idof41[4];
  idof41[0]=G02/3.0+G01/3.0+x0;  idof41[1]=G12/3.0+y0+G11/3.0;  idof41[2]=G22/3.0+G21/3.0+z0;
  idof41[3] = 2.0;
  ptv ipdof41(4,idof41);
  iidof[41] = dof.insert_dof(element,41,ipdof41);

  double idof42[4];
  idof42[0]=G02/3.0+G01/3.0+G00/3.0+x0;  idof42[1]=G12/3.0+y0+G11/3.0+G10/3.0;  idof42[2]=G22/3.0+G21/3.0+z0+G20/3.0;
  idof42[3] = 0.0;
  ptv ipdof42(4,idof42);
  iidof[42] = dof.insert_dof(element,42,ipdof42);

  double idof43[4];
  idof43[0]=G02/3.0+G01/3.0+G00/3.0+x0;  idof43[1]=G12/3.0+y0+G11/3.0+G10/3.0;  idof43[2]=G22/3.0+G21/3.0+z0+G20/3.0;
  idof43[3] = 1.0;
  ptv ipdof43(4,idof43);
  iidof[43] = dof.insert_dof(element,43,ipdof43);

  double idof44[4];
  idof44[0]=G02/3.0+G01/3.0+G00/3.0+x0;  idof44[1]=G12/3.0+y0+G11/3.0+G10/3.0;  idof44[2]=G22/3.0+G21/3.0+z0+G20/3.0;
  idof44[3] = 2.0;
  ptv ipdof44(4,idof44);
  iidof[44] = dof.insert_dof(element,44,ipdof44);

  double idof45[4];
  idof45[0]=G02/3.0+(2.0/3.0)*G01+x0;  idof45[1]=G12/3.0+y0+(2.0/3.0)*G11;  idof45[2]=G22/3.0+(2.0/3.0)*G21+z0;
  idof45[3] = 0.0;
  ptv ipdof45(4,idof45);
  iidof[45] = dof.insert_dof(element,45,ipdof45);

  double idof46[4];
  idof46[0]=G02/3.0+(2.0/3.0)*G01+x0;  idof46[1]=G12/3.0+y0+(2.0/3.0)*G11;  idof46[2]=G22/3.0+(2.0/3.0)*G21+z0;
  idof46[3] = 1.0;
  ptv ipdof46(4,idof46);
  iidof[46] = dof.insert_dof(element,46,ipdof46);

  double idof47[4];
  idof47[0]=G02/3.0+(2.0/3.0)*G01+x0;  idof47[1]=G12/3.0+y0+(2.0/3.0)*G11;  idof47[2]=G22/3.0+(2.0/3.0)*G21+z0;
  idof47[3] = 2.0;
  ptv ipdof47(4,idof47);
  iidof[47] = dof.insert_dof(element,47,ipdof47);

  double idof48[4];
  idof48[0]=(2.0/3.0)*G02+x0;  idof48[1]=(2.0/3.0)*G12+y0;  idof48[2]=(2.0/3.0)*G22+z0;
  idof48[3] = 0.0;
  ptv ipdof48(4,idof48);
  iidof[48] = dof.insert_dof(element,48,ipdof48);

  double idof49[4];
  idof49[0]=(2.0/3.0)*G02+x0;  idof49[1]=(2.0/3.0)*G12+y0;  idof49[2]=(2.0/3.0)*G22+z0;
  idof49[3] = 1.0;
  ptv ipdof49(4,idof49);
  iidof[49] = dof.insert_dof(element,49,ipdof49);

  double idof50[4];
  idof50[0]=(2.0/3.0)*G02+x0;  idof50[1]=(2.0/3.0)*G12+y0;  idof50[2]=(2.0/3.0)*G22+z0;
  idof50[3] = 2.0;
  ptv ipdof50(4,idof50);
  iidof[50] = dof.insert_dof(element,50,ipdof50);

  double idof51[4];
  idof51[0]=(2.0/3.0)*G02+G00/3.0+x0;  idof51[1]=(2.0/3.0)*G12+y0+G10/3.0;  idof51[2]=(2.0/3.0)*G22+z0+G20/3.0;
  idof51[3] = 0.0;
  ptv ipdof51(4,idof51);
  iidof[51] = dof.insert_dof(element,51,ipdof51);

  double idof52[4];
  idof52[0]=(2.0/3.0)*G02+G00/3.0+x0;  idof52[1]=(2.0/3.0)*G12+y0+G10/3.0;  idof52[2]=(2.0/3.0)*G22+z0+G20/3.0;
  idof52[3] = 1.0;
  ptv ipdof52(4,idof52);
  iidof[52] = dof.insert_dof(element,52,ipdof52);

  double idof53[4];
  idof53[0]=(2.0/3.0)*G02+G00/3.0+x0;  idof53[1]=(2.0/3.0)*G12+y0+G10/3.0;  idof53[2]=(2.0/3.0)*G22+z0+G20/3.0;
  idof53[3] = 2.0;
  ptv ipdof53(4,idof53);
  iidof[53] = dof.insert_dof(element,53,ipdof53);

  double idof54[4];
  idof54[0]=(2.0/3.0)*G02+G01/3.0+x0;  idof54[1]=(2.0/3.0)*G12+y0+G11/3.0;  idof54[2]=(2.0/3.0)*G22+G21/3.0+z0;
  idof54[3] = 0.0;
  ptv ipdof54(4,idof54);
  iidof[54] = dof.insert_dof(element,54,ipdof54);

  double idof55[4];
  idof55[0]=(2.0/3.0)*G02+G01/3.0+x0;  idof55[1]=(2.0/3.0)*G12+y0+G11/3.0;  idof55[2]=(2.0/3.0)*G22+G21/3.0+z0;
  idof55[3] = 1.0;
  ptv ipdof55(4,idof55);
  iidof[55] = dof.insert_dof(element,55,ipdof55);

  double idof56[4];
  idof56[0]=(2.0/3.0)*G02+G01/3.0+x0;  idof56[1]=(2.0/3.0)*G12+y0+G11/3.0;  idof56[2]=(2.0/3.0)*G22+G21/3.0+z0;
  idof56[3] = 2.0;
  ptv ipdof56(4,idof56);
  iidof[56] = dof.insert_dof(element,56,ipdof56);

  double idof57[4];
  idof57[0]=G02+x0;  idof57[1]=G12+y0;  idof57[2]=G22+z0;
  idof57[3] = 0.0;
  ptv ipdof57(4,idof57);
  iidof[57] = dof.insert_dof(element,57,ipdof57);

  double idof58[4];
  idof58[0]=G02+x0;  idof58[1]=G12+y0;  idof58[2]=G22+z0;
  idof58[3] = 1.0;
  ptv ipdof58(4,idof58);
  iidof[58] = dof.insert_dof(element,58,ipdof58);

  double idof59[4];
  idof59[0]=G02+x0;  idof59[1]=G12+y0;  idof59[2]=G22+z0;
  idof59[3] = 2.0;
  ptv ipdof59(4,idof59);
  iidof[59] = dof.insert_dof(element,59,ipdof59);

  double w0 = vel[iidof[0]];
  double w1 = vel[iidof[1]];
  double w2 = vel[iidof[2]];
  double w3 = vel[iidof[3]];
  double w4 = vel[iidof[4]];
  double w5 = vel[iidof[5]];
  double w6 = vel[iidof[6]];
  double w7 = vel[iidof[7]];
  double w8 = vel[iidof[8]];
  double w9 = vel[iidof[9]];
  double w10 = vel[iidof[10]];
  double w11 = vel[iidof[11]];
  double w12 = vel[iidof[12]];
  double w13 = vel[iidof[13]];
  double w14 = vel[iidof[14]];
  double w15 = vel[iidof[15]];
  double w16 = vel[iidof[16]];
  double w17 = vel[iidof[17]];
  double w18 = vel[iidof[18]];
  double w19 = vel[iidof[19]];
  double w20 = vel[iidof[20]];
  double w21 = vel[iidof[21]];
  double w22 = vel[iidof[22]];
  double w23 = vel[iidof[23]];
  double w24 = vel[iidof[24]];
  double w25 = vel[iidof[25]];
  double w26 = vel[iidof[26]];
  double w27 = vel[iidof[27]];
  double w28 = vel[iidof[28]];
  double w29 = vel[iidof[29]];
  double w30 = vel[iidof[30]];
  double w31 = vel[iidof[31]];
  double w32 = vel[iidof[32]];
  double w33 = vel[iidof[33]];
  double w34 = vel[iidof[34]];
  double w35 = vel[iidof[35]];
  double w36 = vel[iidof[36]];
  double w37 = vel[iidof[37]];
  double w38 = vel[iidof[38]];
  double w39 = vel[iidof[39]];
  double w40 = vel[iidof[40]];
  double w41 = vel[iidof[41]];
  double w42 = vel[iidof[42]];
  double w43 = vel[iidof[43]];
  double w44 = vel[iidof[44]];
  double w45 = vel[iidof[45]];
  double w46 = vel[iidof[46]];
  double w47 = vel[iidof[47]];
  double w48 = vel[iidof[48]];
  double w49 = vel[iidof[49]];
  double w50 = vel[iidof[50]];
  double w51 = vel[iidof[51]];
  double w52 = vel[iidof[52]];
  double w53 = vel[iidof[53]];
  double w54 = vel[iidof[54]];
  double w55 = vel[iidof[55]];
  double w56 = vel[iidof[56]];
  double w57 = vel[iidof[57]];
  double w58 = vel[iidof[58]];
  double w59 = vel[iidof[59]];
 
  double Jinv00 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G22*G11-G21*G12);
  double Jinv01 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G20*G12-G22*G10);
  double Jinv02 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G21*G10-G20*G11);
  double Jinv10 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G21*G02-G22*G01);
  double Jinv11 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G22*G00-G02*G20);
  double Jinv12 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(-G21*G00+G01*G20);
  double Jinv20 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G01*G12-G02*G11);
  double Jinv21 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(-G12*G00+G02*G10);
  double Jinv22 = 1.0/(G22*G00*G11+G21*G02*G10-G02*G20*G11-G22*G01*G10+G01*G20*G12-G21*G12*G00)*(G00*G11-G01*G10);


  double A[60][60];
  A[0][0]=(-6.4935064935064935e-04*Jinv00*w12-9.4696969696969697e-05*w28*Jinv10-1.2175324675324675e-04*w40*Jinv12-2.0292207792207792e-05*Jinv22*w53+3.2467532467532468e-04*Jinv20*w8-9.4696969696969697e-05*w57*Jinv02-6.4935064935064935e-04*w13*Jinv10-1.2175324675324675e-04*Jinv02*w39-2.4350649350649351e-04*Jinv21*w44-6.4935064935064935e-04*w3*Jinv00-6.4935064935064935e-04*w32*Jinv20-6.4935064935064935e-04*Jinv01*w3+3.2467532467532468e-04*Jinv20*w23-2.0292207792207792e-05*Jinv22*w38-2.0292207792207792e-05*Jinv00*w54-9.4696969696969697e-05*Jinv21*w59-2.0292207792207792e-05*w25*Jinv12-8.3874458874458877e-04*w0*Jinv02-8.3874458874458877e-04*Jinv21*w2-2.4350649350649351e-04*w43*Jinv10-2.4350649350649351e-04*w42*Jinv01-9.4696969696969697e-05*w11*Jinv21-1.2175324675324675e-04*w16*Jinv11-1.2175324675324675e-04*Jinv21*w17-2.0292207792207792e-05*Jinv01*w45-2.0292207792207792e-05*Jinv12*w55-9.4696969696969697e-05*w58*Jinv10-1.2175324675324675e-04*w15*Jinv02-9.4696969696969697e-05*w10*Jinv12+3.2467532467532468e-04*w48*Jinv00-2.0292207792207792e-05*Jinv02*w24-1.2175324675324675e-04*Jinv22*w35-9.4696969696969697e-05*w58*Jinv11-9.4696969696969697e-05*w57*Jinv01-2.0292207792207792e-05*Jinv21*w26-8.3874458874458877e-04*w0*Jinv01-8.3874458874458877e-04*w1*Jinv10-1.2175324675324675e-04*w16*Jinv10-2.0292207792207792e-05*Jinv22*w20-2.4350649350649351e-04*w43*Jinv11-2.0292207792207792e-05*w45*Jinv00-1.2175324675324675e-04*Jinv21*w41-2.4350649350649351e-04*w42*Jinv02-9.4696969696969697e-05*Jinv00*w27-9.4696969696969697e-05*w11*Jinv22-8.3874458874458877e-04*Jinv11*w1-6.4935064935064935e-04*w13*Jinv11-1.2175324675324675e-04*w15*Jinv01-9.4696969696969697e-05*w28*Jinv11+3.2467532467532468e-04*Jinv20*w50-9.4696969696969697e-05*Jinv20*w29-2.0292207792207792e-05*w46*Jinv10-1.2175324675324675e-04*Jinv22*w17-1.2175324675324675e-04*Jinv00*w33+3.2467532467532468e-04*w22*Jinv12-6.4935064935064935e-04*w31*Jinv10-6.4935064935064935e-04*Jinv02*w3+3.2467532467532468e-04*w21*Jinv00-6.4935064935064935e-04*w32*Jinv22-1.2175324675324675e-04*Jinv01*w39-8.3874458874458877e-04*Jinv22*w2-2.0292207792207792e-05*Jinv00*w18-2.0292207792207792e-05*w36*Jinv00-2.0292207792207792e-05*Jinv21*w38-2.0292207792207792e-05*Jinv02*w45-2.0292207792207792e-05*Jinv01*w24-9.4696969696969697e-05*Jinv22*w59-1.2175324675324675e-04*w34*Jinv11-2.0292207792207792e-05*w19*Jinv11-2.0292207792207792e-05*Jinv21*w53-2.0292207792207792e-05*w51*Jinv00-1.2175324675324675e-04*Jinv12*w34-2.4350649350649351e-04*Jinv22*w44-6.4935064935064935e-04*w4*Jinv10-6.4935064935064935e-04*w4*Jinv11-1.2175324675324675e-04*w41*Jinv20-6.4935064935064935e-04*Jinv12*w13-2.0292207792207792e-05*w19*Jinv10-1.2175324675324675e-04*w34*Jinv10-6.4935064935064935e-04*w32*Jinv21-2.0292207792207792e-05*Jinv21*w20+3.2467532467532468e-04*w6*Jinv00-2.0292207792207792e-05*w37*Jinv12-2.0292207792207792e-05*Jinv20*w56-1.2175324675324675e-04*Jinv22*w41-2.0292207792207792e-05*w26*Jinv20-9.4696969696969697e-05*w11*Jinv20-2.0292207792207792e-05*w52*Jinv12-1.2175324675324675e-04*Jinv21*w35-6.4935064935064935e-04*w31*Jinv11-2.0292207792207792e-05*Jinv22*w26-2.0292207792207792e-05*w46*Jinv11-6.4935064935064935e-04*Jinv20*w14-6.4935064935064935e-04*w5*Jinv21-2.0292207792207792e-05*Jinv02*w54-6.4935064935064935e-04*w4*Jinv12+3.2467532467532468e-04*w21*Jinv02+3.2467532467532468e-04*w49*Jinv10-9.4696969696969697e-05*Jinv21*w29-1.2175324675324675e-04*Jinv01*w33-1.2175324675324675e-04*w39*Jinv00-6.4935064935064935e-04*Jinv02*w12-2.0292207792207792e-05*Jinv01*w18-2.0292207792207792e-05*w37*Jinv11-9.4696969696969697e-05*Jinv02*w9+3.2467532467532468e-04*w6*Jinv01+3.2467532467532468e-04*w7*Jinv10-2.0292207792207792e-05*w47*Jinv21-2.0292207792207792e-05*Jinv20*w38+3.2467532467532468e-04*Jinv22*w23-2.0292207792207792e-05*w52*Jinv11-6.4935064935064935e-04*w31*Jinv12-2.0292207792207792e-05*w46*Jinv12-2.0292207792207792e-05*Jinv20*w53+3.2467532467532468e-04*Jinv22*w8+3.2467532467532468e-04*w22*Jinv10+3.2467532467532468e-04*Jinv22*w50+3.2467532467532468e-04*w22*Jinv11+3.2467532467532468e-04*Jinv12*w7+3.2467532467532468e-04*w21*Jinv01-2.0292207792207792e-05*Jinv01*w51-6.4935064935064935e-04*Jinv02*w30+3.2467532467532468e-04*Jinv12*w49-2.0292207792207792e-05*w36*Jinv01-2.0292207792207792e-05*w52*Jinv10+3.2467532467532468e-04*w7*Jinv11-9.4696969696969697e-05*w9*Jinv00-2.0292207792207792e-05*w37*Jinv10-2.0292207792207792e-05*Jinv21*w56-2.0292207792207792e-05*w24*Jinv00-9.4696969696969697e-05*Jinv02*w27+3.2467532467532468e-04*w6*Jinv02-2.0292207792207792e-05*w19*Jinv12-6.4935064935064935e-04*Jinv22*w5-2.0292207792207792e-05*w47*Jinv22+3.2467532467532468e-04*w49*Jinv11-6.4935064935064935e-04*Jinv21*w14-9.4696969696969697e-05*w10*Jinv10-2.4350649350649351e-04*Jinv20*w44-9.4696969696969697e-05*w58*Jinv12-2.0292207792207792e-05*Jinv02*w18-6.4935064935064935e-04*Jinv01*w12+3.2467532467532468e-04*Jinv21*w8-9.4696969696969697e-05*w57*Jinv00-9.4696969696969697e-05*Jinv20*w59-1.2175324675324675e-04*Jinv02*w33-8.3874458874458877e-04*w0*Jinv00-1.2175324675324675e-04*w35*Jinv20+3.2467532467532468e-04*Jinv21*w23-2.0292207792207792e-05*Jinv01*w54-2.0292207792207792e-05*w55*Jinv11-2.0292207792207792e-05*w25*Jinv10-8.3874458874458877e-04*Jinv20*w2-1.2175324675324675e-04*w15*Jinv00+3.2467532467532468e-04*w48*Jinv02-9.4696969696969697e-05*Jinv01*w9-1.2175324675324675e-04*Jinv20*w17-6.4935064935064935e-04*w30*Jinv00-1.2175324675324675e-04*w40*Jinv10-2.0292207792207792e-05*w20*Jinv20-9.4696969696969697e-05*Jinv22*w29-6.4935064935064935e-04*Jinv22*w14-1.2175324675324675e-04*w40*Jinv11-2.0292207792207792e-05*Jinv02*w36-2.4350649350649351e-04*Jinv12*w43-6.4935064935064935e-04*w5*Jinv20-6.4935064935064935e-04*Jinv01*w30-2.0292207792207792e-05*Jinv02*w51-2.0292207792207792e-05*Jinv22*w56-9.4696969696969697e-05*Jinv12*w28-2.0292207792207792e-05*w25*Jinv11-2.0292207792207792e-05*w55*Jinv10-2.4350649350649351e-04*w42*Jinv00-2.0292207792207792e-05*w47*Jinv20-1.2175324675324675e-04*w16*Jinv12+3.2467532467532468e-04*Jinv21*w50-9.4696969696969697e-05*Jinv01*w27+3.2467532467532468e-04*w48*Jinv01-8.3874458874458877e-04*Jinv12*w1-9.4696969696969697e-05*w10*Jinv11)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(0.0)*D;
  A[0][3]=(1.1414366883116883e-03*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.2784090909090910e-04*Jinv22*w53-4.0178571428571428e-04*Jinv20*w8+9.2329545454545453e-05*w57*Jinv02+1.1414366883116883e-03*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+6.7573051948051951e-04*Jinv21*w44+1.1870941558441558e-03*w3*Jinv00+1.1414366883116883e-03*w32*Jinv20+9.1314935064935068e-05*Jinv01*w3-6.6659902597402602e-04*Jinv20*w23+2.1002435064935065e-04*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+9.2329545454545453e-05*Jinv21*w59-1.2784090909090910e-04*w25*Jinv12-2.8409090909090908e-04*w0*Jinv02-2.8409090909090908e-04*Jinv21*w2+1.8262987012987014e-04*w43*Jinv10+6.7573051948051951e-04*w42*Jinv01+5.6818181818181818e-05*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17-6.3920454545454550e-05*Jinv01*w45-6.3920454545454550e-05*Jinv12*w55+1.5726461038961039e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+5.6818181818181818e-05*w10*Jinv12-6.6659902597402602e-04*w48*Jinv00-1.2784090909090910e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35+9.2329545454545453e-05*w58*Jinv11+9.2329545454545453e-05*w57*Jinv01-1.2784090909090910e-04*Jinv21*w26-2.8409090909090908e-04*w0*Jinv01+1.2987012987012987e-03*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+2.1002435064935065e-04*Jinv22*w20+6.7573051948051951e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+6.7573051948051951e-04*w42*Jinv02+1.5726461038961039e-04*Jinv00*w27+5.6818181818181818e-05*w11*Jinv22-2.8409090909090908e-04*Jinv11*w1+4.5657467532467534e-05*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01+9.2329545454545453e-05*w28*Jinv11-6.6659902597402602e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+1.1414366883116883e-03*w31*Jinv10+9.1314935064935068e-05*Jinv02*w3-6.6659902597402602e-04*w21*Jinv00+4.5657467532467534e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-2.8409090909090908e-04*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1002435064935065e-04*Jinv21*w38-6.3920454545454550e-05*Jinv02*w45-1.2784090909090910e-04*Jinv01*w24+9.2329545454545453e-05*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+2.1002435064935065e-04*w19*Jinv11-1.2784090909090910e-04*Jinv21*w53-1.1870941558441559e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+6.7573051948051951e-04*Jinv22*w44+1.1870941558441558e-03*w4*Jinv10+9.1314935064935068e-05*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20+4.5657467532467534e-05*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10+4.5657467532467534e-05*w32*Jinv21+2.1002435064935065e-04*Jinv21*w20-4.0178571428571428e-04*w6*Jinv00+2.1002435064935065e-04*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-1.1870941558441559e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20-1.2784090909090910e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+4.5657467532467534e-05*w31*Jinv11-1.2784090909090910e-04*Jinv22*w26-6.3920454545454550e-05*w46*Jinv11+1.1414366883116883e-03*Jinv20*w14+9.1314935064935068e-05*w5*Jinv21-6.3920454545454550e-05*Jinv02*w54+9.1314935064935068e-05*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02-6.6659902597402602e-04*w49*Jinv10+9.2329545454545453e-05*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00+4.5657467532467534e-05*Jinv02*w12+2.1002435064935065e-04*Jinv01*w18+2.1002435064935065e-04*w37*Jinv11+5.6818181818181818e-05*Jinv02*w9+1.4610389610389610e-04*w6*Jinv01-4.0178571428571428e-04*w7*Jinv10-6.3920454545454550e-05*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23-1.2784090909090910e-04*w52*Jinv11+4.5657467532467534e-05*w31*Jinv12-6.3920454545454550e-05*w46*Jinv12-1.1870941558441559e-04*Jinv20*w53+1.4610389610389610e-04*Jinv22*w8-6.6659902597402602e-04*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11+1.4610389610389610e-04*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01-1.2784090909090910e-04*Jinv01*w51+4.5657467532467534e-05*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49+2.1002435064935065e-04*w36*Jinv01-1.1870941558441559e-04*w52*Jinv10+1.4610389610389610e-04*w7*Jinv11+1.2175324675324675e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10-6.3920454545454550e-05*Jinv21*w56-1.1870941558441559e-04*w24*Jinv00+9.2329545454545453e-05*Jinv02*w27+1.4610389610389610e-04*w6*Jinv02+2.1002435064935065e-04*w19*Jinv12+9.1314935064935068e-05*Jinv22*w5-6.3920454545454550e-05*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+4.5657467532467534e-05*Jinv21*w14+1.2175324675324675e-04*w10*Jinv10+1.8262987012987014e-04*Jinv20*w44+9.2329545454545453e-05*w58*Jinv12+2.1002435064935065e-04*Jinv02*w18+4.5657467532467534e-05*Jinv01*w12+1.4610389610389610e-04*Jinv21*w8+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33+1.2987012987012987e-03*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23-6.3920454545454550e-05*Jinv01*w54-6.3920454545454550e-05*w55*Jinv11-1.1870941558441559e-04*w25*Jinv10+1.2987012987012987e-03*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02+5.6818181818181818e-05*Jinv01*w9+2.1915584415584417e-04*Jinv20*w17+1.1414366883116883e-03*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+9.2329545454545453e-05*Jinv22*w29+4.5657467532467534e-05*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.1002435064935065e-04*Jinv02*w36+6.7573051948051951e-04*Jinv12*w43+1.1870941558441558e-03*w5*Jinv20+4.5657467532467534e-05*Jinv01*w30-1.2784090909090910e-04*Jinv02*w51-6.3920454545454550e-05*Jinv22*w56+9.2329545454545453e-05*Jinv12*w28-1.2784090909090910e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10+1.8262987012987014e-04*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50+9.2329545454545453e-05*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.8409090909090908e-04*Jinv12*w1+5.6818181818181818e-05*w10*Jinv11)*D;
  A[0][4]=(0.0)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(-5.5702110389610384e-04*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10+3.6525974025974026e-05*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-1.2784090909090910e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02-5.5702110389610384e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-2.7394480519480518e-04*Jinv21*w44-6.2094155844155847e-04*w3*Jinv00-5.5702110389610384e-04*w32*Jinv20+4.7483766233766234e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38-2.7394480519480521e-05*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-4.0584415584415584e-05*w0*Jinv02-4.0584415584415584e-05*Jinv21*w2+2.5568181818181820e-04*w43*Jinv10-2.7394480519480518e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-1.1668019480519480e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-4.0584415584415584e-05*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-2.7394480519480518e-04*w43*Jinv11-2.7394480519480521e-05*w45*Jinv00+3.6525974025974026e-05*Jinv21*w41-2.7394480519480518e-04*w42*Jinv02-1.1668019480519480e-04*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-4.0584415584415584e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33+1.3697240259740259e-04*w22*Jinv12-5.5702110389610384e-04*w31*Jinv10+4.7483766233766234e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-4.0584415584415584e-05*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+2.5568181818181820e-04*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-2.7394480519480518e-04*Jinv22*w44-6.2094155844155847e-04*w4*Jinv10+4.7483766233766234e-04*w4*Jinv11+2.5568181818181820e-04*w41*Jinv20-9.1314935064935065e-06*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-9.1314935064935065e-06*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-1.2784090909090910e-04*w6*Jinv00-7.4878246753246752e-04*w37*Jinv12-2.7394480519480521e-05*Jinv20*w56+3.6525974025974026e-05*Jinv22*w41+2.5568181818181820e-04*w26*Jinv20-1.2175324675324675e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35-9.1314935064935065e-06*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-5.5702110389610384e-04*Jinv20*w14+4.7483766233766234e-04*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54+4.7483766233766234e-04*w4*Jinv12+1.3697240259740259e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33+2.5568181818181820e-04*w39*Jinv00-9.1314935064935065e-06*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.8530844155844158e-04*w6*Jinv01-1.2784090909090910e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+1.3697240259740259e-04*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.5568181818181820e-04*Jinv20*w53-7.8530844155844158e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+1.3697240259740259e-04*Jinv22*w50+1.3697240259740259e-04*w22*Jinv11-7.8530844155844158e-04*Jinv12*w7+1.3697240259740259e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30+1.3697240259740259e-04*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+2.5568181818181820e-04*w52*Jinv10-7.8530844155844158e-04*w7*Jinv11-1.2175324675324675e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.5568181818181820e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-7.8530844155844158e-04*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+4.7483766233766234e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.3697240259740259e-04*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.2175324675324675e-04*w10*Jinv10+2.5568181818181820e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-7.8530844155844158e-04*Jinv21*w8-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59-4.9310064935064932e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20+1.3697240259740259e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.5568181818181820e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00+1.3697240259740259e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17-5.5702110389610384e-04*w30*Jinv00+2.5568181818181820e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29-9.1314935064935065e-06*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-2.7394480519480518e-04*Jinv12*w43-6.2094155844155847e-04*w5*Jinv20-9.1314935064935065e-06*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11-2.7394480519480521e-05*w55*Jinv10+2.5568181818181820e-04*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12+1.3697240259740259e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+1.3697240259740259e-04*w48*Jinv01-4.0584415584415584e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(0.0)*D;
  A[0][9]=(6.4935064935064935e-05*Jinv00*w12+5.4112554112554113e-05*w28*Jinv10+2.0495129870129870e-04*Jinv20*w8+6.4935064935064935e-05*w13*Jinv10+8.3198051948051951e-05*w3*Jinv00+6.4935064935064935e-05*w32*Jinv20-1.4204545454545455e-05*Jinv20*w23+1.0247564935064935e-04*Jinv00*w54-1.9480519480519481e-04*w43*Jinv10+5.4112554112554113e-05*w58*Jinv10-1.4204545454545455e-05*w48*Jinv00+1.8939393939393939e-04*w1*Jinv10-4.2613636363636367e-05*w16*Jinv10+1.0247564935064935e-04*w45*Jinv00+5.4112554112554113e-05*Jinv00*w27-1.4204545454545455e-05*Jinv20*w50+5.4112554112554113e-05*Jinv20*w29+1.0247564935064935e-04*w46*Jinv10-4.2613636363636367e-05*Jinv00*w33+6.4935064935064935e-05*w31*Jinv10-1.4204545454545455e-05*w21*Jinv00+1.5726461038961039e-04*Jinv00*w18+1.5726461038961039e-04*w36*Jinv00-1.1668019480519480e-04*w51*Jinv00+8.3198051948051951e-05*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+1.5726461038961039e-04*w19*Jinv10-4.2613636363636367e-05*w34*Jinv10+2.0495129870129870e-04*w6*Jinv00+1.0247564935064935e-04*Jinv20*w56-1.1668019480519480e-04*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20+6.4935064935064935e-05*Jinv20*w14-1.4204545454545455e-05*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+2.0495129870129870e-04*w7*Jinv10+1.5726461038961039e-04*Jinv20*w38-1.1668019480519480e-04*Jinv20*w53-1.4204545454545455e-05*w22*Jinv10-1.1668019480519480e-04*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+1.5726461038961039e-04*w37*Jinv10-1.1668019480519480e-04*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-1.9480519480519481e-04*Jinv20*w44+5.4112554112554113e-05*w57*Jinv00+5.4112554112554113e-05*Jinv20*w59+1.8939393939393939e-04*w0*Jinv00-4.2613636363636367e-05*w35*Jinv20-1.1668019480519480e-04*w25*Jinv10+1.8939393939393939e-04*Jinv20*w2-4.2613636363636367e-05*w15*Jinv00-4.2613636363636367e-05*Jinv20*w17+6.4935064935064935e-05*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+1.5726461038961039e-04*w20*Jinv20+8.3198051948051951e-05*w5*Jinv20+1.0247564935064935e-04*w55*Jinv10-1.9480519480519481e-04*w42*Jinv00+1.0247564935064935e-04*w47*Jinv20)*D;
  A[0][10]=(0.0)*D;
  A[0][11]=(0.0)*D;
  A[0][12]=(9.1314935064935068e-05*Jinv00*w12+5.6818181818181818e-05*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12-6.3920454545454550e-05*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8+9.2329545454545453e-05*w57*Jinv02+9.1314935064935068e-05*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39+1.8262987012987014e-04*Jinv21*w44+4.5657467532467534e-05*w3*Jinv00+4.5657467532467534e-05*w32*Jinv20+1.1414366883116883e-03*Jinv01*w3+1.4610389610389610e-04*Jinv20*w23-6.3920454545454550e-05*Jinv22*w38-1.2784090909090910e-04*Jinv00*w54+1.5726461038961039e-04*Jinv21*w59+2.1002435064935065e-04*w25*Jinv12-2.8409090909090908e-04*w0*Jinv02+1.2987012987012987e-03*Jinv21*w2+6.7573051948051951e-04*w43*Jinv10+1.8262987012987014e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45-1.2784090909090910e-04*Jinv12*w55+9.2329545454545453e-05*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+9.2329545454545453e-05*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00+2.1002435064935065e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+1.2987012987012987e-03*w0*Jinv01-2.8409090909090908e-04*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10-1.2784090909090910e-04*Jinv22*w20+1.8262987012987014e-04*w43*Jinv11+2.1002435064935065e-04*w45*Jinv00+2.1915584415584417e-04*Jinv21*w41+6.7573051948051951e-04*w42*Jinv02+5.6818181818181818e-05*Jinv00*w27+9.2329545454545453e-05*w11*Jinv22+1.2987012987012987e-03*Jinv11*w1+1.1870941558441558e-03*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50+5.6818181818181818e-05*Jinv20*w29+2.1002435064935065e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+1.4610389610389610e-04*w22*Jinv12+4.5657467532467534e-05*w31*Jinv10+4.5657467532467534e-05*Jinv02*w3+1.4610389610389610e-04*w21*Jinv00+4.5657467532467534e-05*w32*Jinv22+2.1915584415584417e-04*Jinv01*w39-2.8409090909090908e-04*Jinv22*w2-1.2784090909090910e-04*Jinv00*w18-6.3920454545454550e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+2.1002435064935065e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24+9.2329545454545453e-05*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11-1.1870941558441559e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-6.3920454545454550e-05*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+6.7573051948051951e-04*Jinv22*w44+4.5657467532467534e-05*w4*Jinv10+1.1414366883116883e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+9.1314935064935068e-05*Jinv12*w13-1.2784090909090910e-04*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10+1.1414366883116883e-03*w32*Jinv21-1.1870941558441559e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00-6.3920454545454550e-05*w37*Jinv12-1.2784090909090910e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+2.1002435064935065e-04*w26*Jinv20+9.2329545454545453e-05*w11*Jinv20-6.3920454545454550e-05*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35+1.1414366883116883e-03*w31*Jinv11+2.1002435064935065e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11+9.1314935064935068e-05*Jinv20*w14+1.1414366883116883e-03*w5*Jinv21-1.2784090909090910e-04*Jinv02*w54+4.5657467532467534e-05*w4*Jinv12+1.4610389610389610e-04*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10+1.2175324675324675e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+9.1314935064935068e-05*Jinv02*w12-1.1870941558441559e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+9.2329545454545453e-05*Jinv02*w9-6.6659902597402602e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-6.3920454545454550e-05*Jinv20*w38+1.4610389610389610e-04*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+4.5657467532467534e-05*w31*Jinv12+2.1002435064935065e-04*w46*Jinv12-6.3920454545454550e-05*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8+1.4610389610389610e-04*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-4.0178571428571428e-04*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7-4.0178571428571428e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+4.5657467532467534e-05*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-6.3920454545454550e-05*w52*Jinv10-6.6659902597402602e-04*w7*Jinv11+9.2329545454545453e-05*w9*Jinv00-6.3920454545454550e-05*w37*Jinv10-1.1870941558441559e-04*Jinv21*w56+2.1002435064935065e-04*w24*Jinv00+5.6818181818181818e-05*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02-1.2784090909090910e-04*w19*Jinv12+4.5657467532467534e-05*Jinv22*w5+2.1002435064935065e-04*w47*Jinv22-6.6659902597402602e-04*w49*Jinv11+1.1870941558441558e-03*Jinv21*w14+9.2329545454545453e-05*w10*Jinv10+6.7573051948051951e-04*Jinv20*w44+9.2329545454545453e-05*w58*Jinv12-1.2784090909090910e-04*Jinv02*w18+1.1870941558441558e-03*Jinv01*w12-6.6659902597402602e-04*Jinv21*w8+9.2329545454545453e-05*w57*Jinv00+9.2329545454545453e-05*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33-2.8409090909090908e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-4.0178571428571428e-04*Jinv21*w23-1.1870941558441559e-04*Jinv01*w54-1.1870941558441559e-04*w55*Jinv11+2.1002435064935065e-04*w25*Jinv10-2.8409090909090908e-04*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02+1.5726461038961039e-04*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17+4.5657467532467534e-05*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-1.2784090909090910e-04*w20*Jinv20+5.6818181818181818e-05*Jinv22*w29+9.1314935064935068e-05*Jinv22*w14+2.1915584415584417e-04*w40*Jinv11-6.3920454545454550e-05*Jinv02*w36+6.7573051948051951e-04*Jinv12*w43+4.5657467532467534e-05*w5*Jinv20+1.1414366883116883e-03*Jinv01*w30-6.3920454545454550e-05*Jinv02*w51-1.2784090909090910e-04*Jinv22*w56+5.6818181818181818e-05*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11-1.2784090909090910e-04*w55*Jinv10+6.7573051948051951e-04*w42*Jinv00+2.1002435064935065e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-6.6659902597402602e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27-6.6659902597402602e-04*w48*Jinv01-2.8409090909090908e-04*Jinv12*w1+1.5726461038961039e-04*w10*Jinv11)*D;
  A[0][13]=(0.0)*D;
  A[0][14]=(0.0)*D;
  A[0][15]=(-1.0957792207792208e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10-1.1323051948051949e-03*w40*Jinv12+4.2004870129870131e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-2.2524350649350650e-04*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.0957792207792208e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+3.4699675324675324e-04*Jinv00*w54-1.9480519480519481e-04*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-4.8701298701298701e-04*w0*Jinv02+2.4350649350649351e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17-3.1047077922077924e-04*Jinv01*w45+4.2004870129870131e-04*Jinv12*w55-1.9480519480519481e-04*w58*Jinv10-2.2646103896103898e-03*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12+2.5568181818181820e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35-1.9480519480519481e-04*w58*Jinv11-1.9480519480519481e-04*w57*Jinv01-6.2094155844155847e-04*Jinv21*w26+2.4350649350649351e-04*w0*Jinv01+2.4350649350649351e-04*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-9.1314935064935063e-04*Jinv21*w41-2.7759740259740259e-03*w42*Jinv02-4.2613636363636367e-05*Jinv00*w27-3.0438311688311688e-05*w11*Jinv22+2.4350649350649351e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+2.5568181818181820e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-2.2646103896103898e-03*Jinv22*w17-9.1314935064935063e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10+9.8620129870129864e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+4.9310064935064932e-04*w32*Jinv22-9.1314935064935063e-04*Jinv01*w39-4.8701298701298701e-04*Jinv22*w2-6.2094155844155847e-04*Jinv00*w18-3.1047077922077924e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-6.2094155844155847e-04*Jinv01*w24-2.2524350649350650e-04*Jinv22*w59-6.9399350649350649e-04*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+3.4699675324675324e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-2.7759740259740259e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10-1.0957792207792208e-04*w4*Jinv11-6.9399350649350649e-04*w41*Jinv20+9.8620129870129864e-04*Jinv12*w13-6.2094155844155847e-04*w19*Jinv10-9.1314935064935063e-04*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+3.4699675324675324e-04*Jinv20*w56-1.1323051948051949e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20+4.2004870129870131e-04*w52*Jinv12-6.9399350649350649e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-3.1047077922077924e-04*w46*Jinv11-1.0957792207792208e-04*Jinv20*w14-1.0957792207792208e-04*w5*Jinv21+4.2004870129870131e-04*Jinv02*w54+9.8620129870129864e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.5568181818181820e-04*w49*Jinv10-6.9399350649350649e-04*Jinv01*w33-6.9399350649350649e-04*w39*Jinv00+9.8620129870129864e-04*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11-3.0438311688311688e-05*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-3.1047077922077924e-04*w47*Jinv21-3.1047077922077924e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23+3.4699675324675324e-04*w52*Jinv11+4.9310064935064932e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+3.4699675324675324e-04*Jinv01*w51+4.9310064935064932e-04*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-3.1047077922077924e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+9.8620129870129864e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+2.5568181818181820e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-1.6436688311688311e-03*Jinv20*w44-2.2524350649350650e-04*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.9480519480519481e-04*w57*Jinv00-1.9480519480519481e-04*Jinv20*w59-1.1323051948051949e-03*Jinv02*w33+2.4350649350649351e-04*w0*Jinv00-9.1314935064935063e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10+2.4350649350649351e-04*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00+3.6525974025974026e-05*w48*Jinv02-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-6.9399350649350649e-04*w40*Jinv10-6.2094155844155847e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+9.8620129870129864e-04*Jinv22*w14-9.1314935064935063e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36-2.7759740259740259e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30+4.2004870129870131e-04*Jinv02*w51+4.2004870129870131e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-6.2094155844155847e-04*w25*Jinv11+3.4699675324675324e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-2.2646103896103898e-03*w16*Jinv12+2.5568181818181820e-04*Jinv21*w50+2.5568181818181820e-04*w48*Jinv01-4.8701298701298701e-04*Jinv12*w1-4.2613636363636367e-05*w10*Jinv11)*D;
  A[0][16]=(0.0)*D;
  A[0][17]=(0.0)*D;
  A[0][18]=(1.8262987012987013e-05*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10+3.9265422077922079e-04*Jinv20*w8+1.8262987012987013e-05*w13*Jinv10+2.7394480519480518e-04*Jinv21*w44-4.8396915584415583e-04*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20-4.7483766233766234e-04*Jinv01*w3-3.1047077922077924e-04*Jinv20*w23-2.1915584415584417e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+4.0584415584415584e-05*Jinv21*w2+9.6793831168831166e-04*w43*Jinv10+2.7394480519480518e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-4.2918019480519480e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01+4.0584415584415584e-05*w1*Jinv10+1.0592532467532468e-03*w16*Jinv10+2.7394480519480518e-04*w43*Jinv11-4.2918019480519480e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-1.4204545454545455e-05*Jinv00*w27+4.0584415584415584e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10-3.1047077922077924e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+7.4878246753246752e-04*Jinv00*w18+3.7439123376623376e-04*w36*Jinv00+7.4878246753246752e-04*Jinv21*w38-4.2918019480519480e-04*Jinv01*w24+4.9310064935064932e-04*w34*Jinv11+7.4878246753246752e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-3.1960227272727273e-04*w51*Jinv00-4.8396915584415583e-04*w4*Jinv10-4.7483766233766234e-04*w4*Jinv11-7.3051948051948052e-05*w41*Jinv20+7.4878246753246752e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21+7.4878246753246752e-04*Jinv21*w20+3.9265422077922079e-04*w6*Jinv00-2.1915584415584417e-04*Jinv20*w56-1.5523538961038962e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20+4.9310064935064932e-04*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11+1.8262987012987013e-05*Jinv20*w14-4.7483766233766234e-04*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-7.3051948051948052e-05*w39*Jinv00+7.4878246753246752e-04*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+7.8530844155844158e-04*w6*Jinv01+3.9265422077922079e-04*w7*Jinv10-1.0957792207792208e-04*w47*Jinv21+3.7439123376623376e-04*Jinv20*w38-4.2918019480519480e-04*w52*Jinv11-3.1960227272727273e-04*Jinv20*w53-3.1047077922077924e-04*w22*Jinv10-1.3697240259740259e-04*w22*Jinv11-1.3697240259740259e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01-3.1960227272727273e-04*w52*Jinv10+7.8530844155844158e-04*w7*Jinv11-9.2329545454545453e-05*w9*Jinv00+3.7439123376623376e-04*w37*Jinv10-1.0957792207792208e-04*Jinv21*w56-1.5523538961038962e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14-9.2329545454545453e-05*w10*Jinv10+9.6793831168831166e-04*Jinv20*w44+9.1314935064935065e-06*Jinv01*w12+7.8530844155844158e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+4.0584415584415584e-05*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23-1.0957792207792208e-04*Jinv01*w54-1.0957792207792208e-04*w55*Jinv11-1.5523538961038962e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+1.0592532467532468e-03*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+1.0592532467532468e-03*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+7.4878246753246752e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-4.8396915584415583e-04*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30-4.2918019480519480e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+9.6793831168831166e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[0][19]=(0.0)*D;
  A[0][20]=(0.0)*D;
  A[0][21]=(4.7483766233766234e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+1.3697240259740259e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+4.7483766233766234e-04*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+2.5568181818181820e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00-9.1314935064935065e-06*w32*Jinv20-5.5702110389610384e-04*Jinv01*w3-7.8530844155844158e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-1.1668019480519480e-04*Jinv21*w59-7.4878246753246752e-04*w25*Jinv12-4.0584415584415584e-05*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2-2.7394480519480518e-04*w43*Jinv10+2.5568181818181820e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12+1.3697240259740259e-04*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-4.0584415584415584e-05*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+2.5568181818181820e-04*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-2.7394480519480518e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-6.2094155844155847e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.2175324675324675e-04*w28*Jinv11+1.3697240259740259e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.8530844155844158e-04*w22*Jinv12-9.1314935064935065e-06*w31*Jinv10-9.1314935064935065e-06*Jinv02*w3-7.8530844155844158e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-4.0584415584415584e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00-2.7394480519480521e-05*Jinv21*w38-7.4878246753246752e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59+2.5568181818181820e-04*w34*Jinv11+2.5568181818181820e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-2.7394480519480518e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-5.5702110389610384e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20+4.7483766233766234e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-5.5702110389610384e-04*w32*Jinv21+2.5568181818181820e-04*Jinv21*w20+1.3697240259740259e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12+2.5568181818181820e-04*Jinv21*w35-5.5702110389610384e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11+4.7483766233766234e-04*Jinv20*w14-5.5702110389610384e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54-9.1314935064935065e-06*w4*Jinv12-7.8530844155844158e-04*w21*Jinv02+1.3697240259740259e-04*w49*Jinv10-1.2175324675324675e-04*Jinv21*w29+2.5568181818181820e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00+4.7483766233766234e-04*Jinv02*w12+2.5568181818181820e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+1.3697240259740259e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38-7.8530844155844158e-04*Jinv22*w23-2.7394480519480521e-05*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.3697240259740259e-04*Jinv22*w8-7.8530844155844158e-04*w22*Jinv10+1.3697240259740259e-04*Jinv22*w50-1.2784090909090910e-04*w22*Jinv11+1.3697240259740259e-04*Jinv12*w7-1.2784090909090910e-04*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30+1.3697240259740259e-04*Jinv12*w49-2.7394480519480521e-05*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+2.5568181818181820e-04*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+1.3697240259740259e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12-9.1314935064935065e-06*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-6.2094155844155847e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-2.7394480519480518e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18-6.2094155844155847e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-4.0584415584415584e-05*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20-1.2784090909090910e-04*Jinv21*w23+2.5568181818181820e-04*Jinv01*w54+2.5568181818181820e-04*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-4.0584415584415584e-05*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+1.3697240259740259e-04*w48*Jinv02-1.1668019480519480e-04*Jinv01*w9-4.9310064935064932e-04*Jinv20*w17-9.1314935064935065e-06*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.7483766233766234e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36-2.7394480519480518e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20-5.5702110389610384e-04*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-2.7394480519480518e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-1.2175324675324675e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-4.0584415584415584e-05*Jinv12*w1-1.1668019480519480e-04*w10*Jinv11)*D;
  A[0][22]=(0.0)*D;
  A[0][23]=(0.0)*D;
  A[0][24]=(-4.7483766233766234e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.3697240259740259e-04*Jinv20*w8-4.7483766233766234e-04*w13*Jinv10+9.6793831168831166e-04*Jinv21*w44+9.1314935064935065e-06*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20+1.8262987012987013e-05*Jinv01*w3+7.8530844155844158e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+4.0584415584415584e-05*Jinv21*w2+2.7394480519480518e-04*w43*Jinv10+9.6793831168831166e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+1.0592532467532468e-03*w16*Jinv11+1.0592532467532468e-03*Jinv21*w17+3.7439123376623376e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01+7.4878246753246752e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01+4.0584415584415584e-05*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+9.6793831168831166e-04*w43*Jinv11+7.4878246753246752e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27+4.0584415584415584e-05*Jinv11*w1-4.8396915584415583e-04*w13*Jinv11+1.0592532467532468e-03*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10+7.8530844155844158e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.2918019480519480e-04*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00-4.2918019480519480e-04*Jinv21*w38+7.4878246753246752e-04*Jinv01*w24-7.3051948051948052e-05*w34*Jinv11-1.5523538961038962e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-1.0957792207792208e-04*w51*Jinv00+9.1314935064935065e-06*w4*Jinv10+1.8262987012987013e-05*w4*Jinv11+4.9310064935064932e-04*w41*Jinv20-4.2918019480519480e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21-1.5523538961038962e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+7.4878246753246752e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20-7.3051948051948052e-05*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11+3.7439123376623376e-04*w46*Jinv11-4.7483766233766234e-04*Jinv20*w14+1.8262987012987013e-05*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33+4.9310064935064932e-04*w39*Jinv00-1.5523538961038962e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-3.1047077922077924e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10+3.7439123376623376e-04*w47*Jinv21-1.0957792207792208e-04*Jinv20*w38-2.1915584415584417e-04*w52*Jinv11-1.0957792207792208e-04*Jinv20*w53+7.8530844155844158e-04*w22*Jinv10+3.9265422077922079e-04*w22*Jinv11+3.9265422077922079e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-1.0957792207792208e-04*w52*Jinv10-3.1047077922077924e-04*w7*Jinv11+1.0247564935064935e-04*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10-3.1960227272727273e-04*Jinv21*w56+7.4878246753246752e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11-4.8396915584415583e-04*Jinv21*w14+1.0247564935064935e-04*w10*Jinv10+2.7394480519480518e-04*Jinv20*w44-4.8396915584415583e-04*Jinv01*w12-3.1047077922077924e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+4.0584415584415584e-05*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.9265422077922079e-04*Jinv21*w23-3.1960227272727273e-04*Jinv01*w54-3.1960227272727273e-04*w55*Jinv11+7.4878246753246752e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-1.4204545454545455e-05*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-4.2918019480519480e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11+9.1314935064935065e-06*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30+7.4878246753246752e-04*w25*Jinv11-4.2918019480519480e-04*w55*Jinv10+2.7394480519480518e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[0][25]=(0.0)*D;
  A[0][26]=(0.0)*D;
  A[0][27]=(-1.9480519480519481e-04*Jinv21*w44+6.4935064935064935e-05*Jinv01*w3+5.4112554112554113e-05*Jinv21*w59+1.8939393939393939e-04*Jinv21*w2-1.9480519480519481e-04*w42*Jinv01+5.4112554112554113e-05*w11*Jinv21-4.2613636363636367e-05*w16*Jinv11-4.2613636363636367e-05*Jinv21*w17+1.5726461038961039e-04*Jinv01*w45+5.4112554112554113e-05*w58*Jinv11+5.4112554112554113e-05*w57*Jinv01+1.5726461038961039e-04*Jinv21*w26+1.8939393939393939e-04*w0*Jinv01-1.9480519480519481e-04*w43*Jinv11-4.2613636363636367e-05*Jinv21*w41+1.8939393939393939e-04*Jinv11*w1+8.3198051948051951e-05*w13*Jinv11-4.2613636363636367e-05*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-4.2613636363636367e-05*Jinv01*w39+1.0247564935064935e-04*Jinv21*w38+1.5726461038961039e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11-1.1668019480519480e-04*w19*Jinv11+1.0247564935064935e-04*Jinv21*w53+6.4935064935064935e-05*w4*Jinv11+6.4935064935064935e-05*w32*Jinv21-1.1668019480519480e-04*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35+6.4935064935064935e-05*w31*Jinv11+1.5726461038961039e-04*w46*Jinv11+6.4935064935064935e-05*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33-1.1668019480519480e-04*Jinv01*w18+1.0247564935064935e-04*w37*Jinv11-1.4204545454545455e-05*w6*Jinv01+1.5726461038961039e-04*w47*Jinv21+1.0247564935064935e-04*w52*Jinv11+2.0495129870129870e-04*w22*Jinv11+2.0495129870129870e-04*w21*Jinv01+1.0247564935064935e-04*Jinv01*w51+1.0247564935064935e-04*w36*Jinv01-1.4204545454545455e-05*w7*Jinv11-1.1668019480519480e-04*Jinv21*w56-1.4204545454545455e-05*w49*Jinv11+8.3198051948051951e-05*Jinv21*w14+8.3198051948051951e-05*Jinv01*w12-1.4204545454545455e-05*Jinv21*w8+2.0495129870129870e-04*Jinv21*w23-1.1668019480519480e-04*Jinv01*w54-1.1668019480519480e-04*w55*Jinv11+5.4112554112554113e-05*Jinv01*w9-4.2613636363636367e-05*w40*Jinv11+6.4935064935064935e-05*Jinv01*w30+1.5726461038961039e-04*w25*Jinv11-1.4204545454545455e-05*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27-1.4204545454545455e-05*w48*Jinv01+5.4112554112554113e-05*w10*Jinv11)*D;
  A[0][28]=(0.0)*D;
  A[0][29]=(0.0)*D;
  A[0][30]=(4.5657467532467534e-05*Jinv00*w12+9.2329545454545453e-05*w28*Jinv10+2.1915584415584417e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8+1.2175324675324675e-04*w57*Jinv02+4.5657467532467534e-05*w13*Jinv10+2.1915584415584417e-04*Jinv02*w39+6.7573051948051951e-04*Jinv21*w44+4.5657467532467534e-05*w3*Jinv00+9.1314935064935068e-05*w32*Jinv20+4.5657467532467534e-05*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23-1.1870941558441559e-04*Jinv22*w38+2.1002435064935065e-04*Jinv00*w54+5.6818181818181818e-05*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12+1.2987012987012987e-03*w0*Jinv02-2.8409090909090908e-04*Jinv21*w2+6.7573051948051951e-04*w43*Jinv10+6.7573051948051951e-04*w42*Jinv01+9.2329545454545453e-05*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.2784090909090910e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55+5.6818181818181818e-05*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12+1.4610389610389610e-04*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35+5.6818181818181818e-05*w58*Jinv11+5.6818181818181818e-05*w57*Jinv01-6.3920454545454550e-05*Jinv21*w26-2.8409090909090908e-04*w0*Jinv01-2.8409090909090908e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+6.7573051948051951e-04*w43*Jinv11-1.2784090909090910e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41+1.8262987012987014e-04*w42*Jinv02+9.2329545454545453e-05*Jinv00*w27+1.5726461038961039e-04*w11*Jinv22-2.8409090909090908e-04*Jinv11*w1+4.5657467532467534e-05*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+9.2329545454545453e-05*w28*Jinv11+1.4610389610389610e-04*Jinv20*w50+9.2329545454545453e-05*Jinv20*w29-1.2784090909090910e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33-6.6659902597402602e-04*w22*Jinv12+9.1314935064935068e-05*w31*Jinv10+1.1414366883116883e-03*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+1.1870941558441558e-03*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39+1.2987012987012987e-03*Jinv22*w2-6.3920454545454550e-05*Jinv00*w18-1.2784090909090910e-04*w36*Jinv00-1.2784090909090910e-04*Jinv21*w38-1.1870941558441559e-04*Jinv02*w45-6.3920454545454550e-05*Jinv01*w24+1.2175324675324675e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11-6.3920454545454550e-05*w19*Jinv11+2.1002435064935065e-04*Jinv21*w53+2.1002435064935065e-04*w51*Jinv00+2.1915584415584417e-04*Jinv12*w34+1.8262987012987014e-04*Jinv22*w44+4.5657467532467534e-05*w4*Jinv10+4.5657467532467534e-05*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+1.1414366883116883e-03*Jinv12*w13-6.3920454545454550e-05*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10+9.1314935064935068e-05*w32*Jinv21-6.3920454545454550e-05*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00-1.1870941558441559e-04*w37*Jinv12+2.1002435064935065e-04*Jinv20*w56+2.1915584415584417e-04*Jinv22*w41-6.3920454545454550e-05*w26*Jinv20+9.2329545454545453e-05*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+9.1314935064935068e-05*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26-1.2784090909090910e-04*w46*Jinv11+4.5657467532467534e-05*Jinv20*w14+4.5657467532467534e-05*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54+1.1414366883116883e-03*w4*Jinv12-6.6659902597402602e-04*w21*Jinv02+1.4610389610389610e-04*w49*Jinv10+9.2329545454545453e-05*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+1.1414366883116883e-03*Jinv02*w12-6.3920454545454550e-05*Jinv01*w18-1.2784090909090910e-04*w37*Jinv11+1.5726461038961039e-04*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10-1.2784090909090910e-04*w47*Jinv21-1.2784090909090910e-04*Jinv20*w38-6.6659902597402602e-04*Jinv22*w23+2.1002435064935065e-04*w52*Jinv11+1.1870941558441558e-03*w31*Jinv12-1.1870941558441559e-04*w46*Jinv12+2.1002435064935065e-04*Jinv20*w53-6.6659902597402602e-04*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10-4.0178571428571428e-04*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11-6.6659902597402602e-04*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01+2.1002435064935065e-04*Jinv01*w51+1.1870941558441558e-03*Jinv02*w30-4.0178571428571428e-04*Jinv12*w49-1.2784090909090910e-04*w36*Jinv01+2.1002435064935065e-04*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+9.2329545454545453e-05*w9*Jinv00-1.2784090909090910e-04*w37*Jinv10+2.1002435064935065e-04*Jinv21*w56-6.3920454545454550e-05*w24*Jinv00+1.5726461038961039e-04*Jinv02*w27-6.6659902597402602e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+1.1414366883116883e-03*Jinv22*w5-1.1870941558441559e-04*w47*Jinv22+1.4610389610389610e-04*w49*Jinv11+4.5657467532467534e-05*Jinv21*w14+9.2329545454545453e-05*w10*Jinv10+6.7573051948051951e-04*Jinv20*w44+1.2175324675324675e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+4.5657467532467534e-05*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8+5.6818181818181818e-05*w57*Jinv00+5.6818181818181818e-05*Jinv20*w59+2.1915584415584417e-04*Jinv02*w33-2.8409090909090908e-04*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23+2.1002435064935065e-04*Jinv01*w54+2.1002435064935065e-04*w55*Jinv11-6.3920454545454550e-05*w25*Jinv10-2.8409090909090908e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-4.0178571428571428e-04*w48*Jinv02+9.2329545454545453e-05*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17+9.1314935064935068e-05*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-6.3920454545454550e-05*w20*Jinv20+1.5726461038961039e-04*Jinv22*w29+1.1414366883116883e-03*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11-1.1870941558441559e-04*Jinv02*w36+1.8262987012987014e-04*Jinv12*w43+4.5657467532467534e-05*w5*Jinv20+9.1314935064935068e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-6.3920454545454550e-05*w25*Jinv11+2.1002435064935065e-04*w55*Jinv10+6.7573051948051951e-04*w42*Jinv00-1.2784090909090910e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12+1.4610389610389610e-04*Jinv21*w50+9.2329545454545453e-05*Jinv01*w27+1.4610389610389610e-04*w48*Jinv01+1.2987012987012987e-03*Jinv12*w1+9.2329545454545453e-05*w10*Jinv11)*D;
  A[0][31]=(0.0)*D;
  A[0][32]=(0.0)*D;
  A[0][33]=(-5.4788961038961042e-05*Jinv00*w12-1.9480519480519481e-04*w28*Jinv10-9.1314935064935063e-04*w40*Jinv12-6.2094155844155847e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-5.4788961038961042e-05*w13*Jinv10-9.1314935064935063e-04*Jinv02*w39-2.7759740259740259e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20+9.8620129870129864e-04*Jinv01*w3+2.5568181818181820e-04*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54-3.0438311688311688e-05*Jinv21*w59+3.4699675324675324e-04*w25*Jinv12+2.4350649350649351e-04*w0*Jinv02-4.8701298701298701e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-2.7759740259740259e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17+4.2004870129870131e-04*Jinv01*w45-3.1047077922077924e-04*Jinv12*w55-4.2613636363636367e-05*w58*Jinv10-6.9399350649350649e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+3.4699675324675324e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01+4.2004870129870131e-04*Jinv21*w26-4.8701298701298701e-04*w0*Jinv01+2.4350649350649351e-04*w1*Jinv10-9.1314935064935063e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20-2.7759740259740259e-03*w43*Jinv11+3.4699675324675324e-04*w45*Jinv00-1.1323051948051949e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-1.9480519480519481e-04*Jinv00*w27-4.2613636363636367e-05*w11*Jinv22-4.8701298701298701e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-2.2524350649350650e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50-1.9480519480519481e-04*Jinv20*w29+3.4699675324675324e-04*w46*Jinv10-6.9399350649350649e-04*Jinv22*w17-1.8262987012987013e-03*Jinv00*w33+2.5568181818181820e-04*w22*Jinv12-1.0957792207792208e-04*w31*Jinv10-1.0957792207792208e-04*Jinv02*w3+2.5568181818181820e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39+2.4350649350649351e-04*Jinv22*w2-3.1047077922077924e-04*Jinv00*w18-6.2094155844155847e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+4.2004870129870131e-04*Jinv01*w24-2.2646103896103898e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.8262987012987013e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10+9.8620129870129864e-04*w4*Jinv11-6.9399350649350649e-04*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13-3.1047077922077924e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+9.8620129870129864e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-9.1314935064935063e-04*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-6.2094155844155847e-04*w52*Jinv12-2.2646103896103898e-03*Jinv21*w35+9.8620129870129864e-04*w31*Jinv11+3.4699675324675324e-04*Jinv22*w26+4.2004870129870131e-04*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14+9.8620129870129864e-04*w5*Jinv21-3.1047077922077924e-04*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12+2.5568181818181820e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-2.2524350649350650e-04*Jinv21*w29-2.2646103896103898e-03*Jinv01*w33-6.9399350649350649e-04*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-4.2613636363636367e-05*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+4.2004870129870131e-04*w47*Jinv21-6.2094155844155847e-04*Jinv20*w38+2.5568181818181820e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+2.5568181818181820e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-6.2094155844155847e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00-1.9480519480519481e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14-1.6436688311688311e-03*Jinv20*w44+2.1915584415584417e-04*Jinv02*w18+4.9310064935064932e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-1.8262987012987013e-03*Jinv02*w33+2.4350649350649351e-04*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20+3.6525974025974026e-05*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10+2.4350649350649351e-04*Jinv20*w2-9.1314935064935063e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9-9.1314935064935063e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-6.9399350649350649e-04*w40*Jinv10-3.1047077922077924e-04*w20*Jinv20-1.9480519480519481e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20+9.8620129870129864e-04*Jinv01*w30-6.2094155844155847e-04*Jinv02*w51-3.1047077922077924e-04*Jinv22*w56-1.9480519480519481e-04*Jinv12*w28+4.2004870129870131e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+3.4699675324675324e-04*w47*Jinv20-6.9399350649350649e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-2.2524350649350650e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+2.4350649350649351e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[0][34]=(0.0)*D;
  A[0][35]=(0.0)*D;
  A[0][36]=(9.1314935064935065e-06*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.2918019480519480e-04*Jinv22*w53+3.9265422077922079e-04*Jinv20*w8+1.0247564935064935e-04*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-4.8396915584415583e-04*w3*Jinv00+1.8262987012987013e-05*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23+7.4878246753246752e-04*Jinv22*w38-4.2918019480519480e-04*Jinv00*w54-4.2918019480519480e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+9.6793831168831166e-04*w43*Jinv10-1.0957792207792208e-04*Jinv12*w55-1.4204545454545455e-05*w58*Jinv10+4.9310064935064932e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-3.1047077922077924e-04*w48*Jinv00-4.2918019480519480e-04*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35+4.0584415584415584e-05*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.4878246753246752e-04*Jinv22*w20-2.1915584415584417e-04*w45*Jinv00+2.7394480519480518e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-3.1047077922077924e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+4.9310064935064932e-04*Jinv22*w17+1.0592532467532468e-03*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12+1.8262987012987013e-05*w31*Jinv10-4.7483766233766234e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00+9.1314935064935065e-06*w32*Jinv22+4.0584415584415584e-05*Jinv22*w2+3.7439123376623376e-04*Jinv00*w18+7.4878246753246752e-04*w36*Jinv00-1.0957792207792208e-04*Jinv02*w45+1.0247564935064935e-04*Jinv22*w59-1.5523538961038962e-04*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+2.7394480519480518e-04*Jinv22*w44-4.8396915584415583e-04*w4*Jinv10-7.3051948051948052e-05*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13+3.7439123376623376e-04*w19*Jinv10+1.0592532467532468e-03*w34*Jinv10+3.9265422077922079e-04*w6*Jinv00+7.4878246753246752e-04*w37*Jinv12-4.2918019480519480e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-3.1960227272727273e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20-4.2918019480519480e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14-1.0957792207792208e-04*Jinv02*w54-4.7483766233766234e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02-3.1047077922077924e-04*w49*Jinv10-7.3051948051948052e-05*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.9265422077922079e-04*w7*Jinv10+7.4878246753246752e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23+9.1314935064935065e-06*w31*Jinv12-1.0957792207792208e-04*w46*Jinv12-1.5523538961038962e-04*Jinv20*w53+7.8530844155844158e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50+7.8530844155844158e-04*Jinv12*w7+9.1314935064935065e-06*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-1.5523538961038962e-04*w52*Jinv10-9.2329545454545453e-05*w9*Jinv00+7.4878246753246752e-04*w37*Jinv10-3.1960227272727273e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27+7.8530844155844158e-04*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-4.7483766233766234e-04*Jinv22*w5-1.0957792207792208e-04*w47*Jinv22-9.2329545454545453e-05*w10*Jinv10+9.6793831168831166e-04*Jinv20*w44+1.0247564935064935e-04*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33+4.0584415584415584e-05*w0*Jinv00+1.0592532467532468e-03*w35*Jinv20-3.1960227272727273e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17+1.8262987012987013e-05*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+3.7439123376623376e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14+7.4878246753246752e-04*Jinv02*w36+2.7394480519480518e-04*Jinv12*w43-4.8396915584415583e-04*w5*Jinv20-4.2918019480519480e-04*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28-4.2918019480519480e-04*w55*Jinv10+9.6793831168831166e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20+4.9310064935064932e-04*w16*Jinv12+4.0584415584415584e-05*Jinv12*w1)*D;
  A[0][37]=(0.0)*D;
  A[0][38]=(0.0)*D;
  A[0][39]=(9.8620129870129864e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10-1.8262987012987013e-03*w40*Jinv12-3.1047077922077924e-04*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8+9.8620129870129864e-04*w13*Jinv10-1.8262987012987013e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+4.9310064935064932e-04*w3*Jinv00+9.8620129870129864e-04*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-4.2613636363636367e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12+2.4350649350649351e-04*w0*Jinv02+2.4350649350649351e-04*Jinv21*w2-2.7759740259740259e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-1.9480519480519481e-04*w11*Jinv21-9.1314935064935063e-04*w16*Jinv11-9.1314935064935063e-04*Jinv21*w17-6.2094155844155847e-04*Jinv01*w45-6.2094155844155847e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10-6.9399350649350649e-04*w15*Jinv02-1.9480519480519481e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-9.1314935064935063e-04*Jinv22*w35-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01-3.1047077922077924e-04*Jinv21*w26+2.4350649350649351e-04*w0*Jinv01-4.8701298701298701e-04*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10+3.4699675324675324e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-1.8262987012987013e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-1.9480519480519481e-04*w11*Jinv22+2.4350649350649351e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-9.1314935064935063e-04*w15*Jinv01-4.9310064935064932e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-6.9399350649350649e-04*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+9.8620129870129864e-04*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.8262987012987013e-03*Jinv01*w39+2.4350649350649351e-04*Jinv22*w2+4.2004870129870131e-04*Jinv00*w18+4.2004870129870131e-04*w36*Jinv00+3.4699675324675324e-04*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45-3.1047077922077924e-04*Jinv01*w24-6.9399350649350649e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00-9.1314935064935063e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+4.9310064935064932e-04*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11-2.2646103896103898e-03*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13+4.2004870129870131e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-1.8262987012987013e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20-2.2524350649350650e-04*w11*Jinv20-3.1047077922077924e-04*w52*Jinv12-6.9399350649350649e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26-6.2094155844155847e-04*w46*Jinv11+9.8620129870129864e-04*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21-6.2094155844155847e-04*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-6.9399350649350649e-04*Jinv01*w33-2.2646103896103898e-03*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+3.4699675324675324e-04*w37*Jinv11-1.9480519480519481e-04*Jinv02*w9+2.5568181818181820e-04*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10-6.2094155844155847e-04*w47*Jinv21+4.2004870129870131e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+2.5568181818181820e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+2.5568181818181820e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.4699675324675324e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+2.5568181818181820e-04*w7*Jinv11-2.2524350649350650e-04*w9*Jinv00+4.2004870129870131e-04*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-4.2613636363636367e-05*Jinv02*w27+2.5568181818181820e-04*w6*Jinv02+3.4699675324675324e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-2.2524350649350650e-04*w10*Jinv10-2.7759740259740259e-03*Jinv20*w44+3.4699675324675324e-04*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+2.5568181818181820e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59-9.1314935064935063e-04*Jinv02*w33-4.8701298701298701e-04*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-4.8701298701298701e-04*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.9480519480519481e-04*Jinv01*w9-1.1323051948051949e-03*Jinv20*w17+9.8620129870129864e-04*w30*Jinv00-2.2646103896103898e-03*w40*Jinv10+4.2004870129870131e-04*w20*Jinv20-4.2613636363636367e-05*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-1.8262987012987013e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+4.9310064935064932e-04*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30-3.1047077922077924e-04*Jinv02*w51-6.2094155844155847e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-3.1047077922077924e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-2.7759740259740259e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-6.9399350649350649e-04*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.6436688311688311e-04*w48*Jinv01+2.4350649350649351e-04*Jinv12*w1-1.9480519480519481e-04*w10*Jinv11)*D;
  A[0][40]=(0.0)*D;
  A[0][41]=(0.0)*D;
  A[0][42]=(-9.8620129870129864e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12-4.2004870129870131e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+2.2524350649350650e-04*w57*Jinv02-9.8620129870129864e-04*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+2.7759740259740259e-03*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00-9.8620129870129864e-04*w32*Jinv20-9.8620129870129864e-04*Jinv01*w3+4.9310064935064932e-04*Jinv20*w23+2.7394480519480518e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+3.0438311688311688e-05*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+4.8701298701298701e-04*w0*Jinv02+4.8701298701298701e-04*Jinv21*w2+2.7759740259740259e-03*w43*Jinv10+2.7759740259740259e-03*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17-4.2004870129870131e-04*Jinv01*w45-4.2004870129870131e-04*Jinv12*w55+3.0438311688311688e-05*w58*Jinv10+2.2646103896103898e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12+4.9310064935064932e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-4.2004870129870131e-04*Jinv21*w26+4.8701298701298701e-04*w0*Jinv01+4.8701298701298701e-04*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+2.7759740259740259e-03*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+2.7759740259740259e-03*w42*Jinv02+3.0438311688311688e-05*Jinv00*w27+3.0438311688311688e-05*w11*Jinv22+4.8701298701298701e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+2.2524350649350650e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10+2.2646103896103898e-03*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33+4.9310064935064932e-04*w22*Jinv12-9.8620129870129864e-04*w31*Jinv10-9.8620129870129864e-04*Jinv02*w3+4.9310064935064932e-04*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+4.8701298701298701e-04*Jinv22*w2-4.2004870129870131e-04*Jinv00*w18-4.2004870129870131e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+2.7394480519480518e-04*Jinv02*w45-4.2004870129870131e-04*Jinv01*w24+2.2524350649350650e-04*Jinv22*w59+2.2646103896103898e-03*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+2.7759740259740259e-03*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-9.8620129870129864e-04*w4*Jinv11+2.2646103896103898e-03*w41*Jinv20-9.8620129870129864e-04*Jinv12*w13-4.2004870129870131e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-9.8620129870129864e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+2.7394480519480518e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20+2.2524350649350650e-04*w11*Jinv20-4.2004870129870131e-04*w52*Jinv12+2.2646103896103898e-03*Jinv21*w35-9.8620129870129864e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26-4.2004870129870131e-04*w46*Jinv11-9.8620129870129864e-04*Jinv20*w14-9.8620129870129864e-04*w5*Jinv21-4.2004870129870131e-04*Jinv02*w54-9.8620129870129864e-04*w4*Jinv12+4.9310064935064932e-04*w21*Jinv02+4.9310064935064932e-04*w49*Jinv10+2.2524350649350650e-04*Jinv21*w29+2.2646103896103898e-03*Jinv01*w33+2.2646103896103898e-03*w39*Jinv00-9.8620129870129864e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+3.0438311688311688e-05*Jinv02*w9+4.9310064935064932e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10-4.2004870129870131e-04*w47*Jinv21-4.2004870129870131e-04*Jinv20*w38+4.9310064935064932e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+2.7394480519480518e-04*Jinv20*w53+4.9310064935064932e-04*Jinv22*w8+4.9310064935064932e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+4.9310064935064932e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+4.9310064935064932e-04*w7*Jinv11+2.2524350649350650e-04*w9*Jinv00-4.2004870129870131e-04*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+2.7394480519480518e-04*w24*Jinv00+3.0438311688311688e-05*Jinv02*w27+4.9310064935064932e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-9.8620129870129864e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+4.9310064935064932e-04*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14+2.2524350649350650e-04*w10*Jinv10+2.7759740259740259e-03*Jinv20*w44+2.2524350649350650e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+4.9310064935064932e-04*Jinv21*w8+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+4.8701298701298701e-04*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+2.7394480519480518e-04*w25*Jinv10+4.8701298701298701e-04*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+3.0438311688311688e-05*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17-9.8620129870129864e-04*w30*Jinv00+2.2646103896103898e-03*w40*Jinv10-4.2004870129870131e-04*w20*Jinv20+3.0438311688311688e-05*Jinv22*w29-9.8620129870129864e-04*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36+2.7759740259740259e-03*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-9.8620129870129864e-04*Jinv01*w30-4.2004870129870131e-04*Jinv02*w51-4.2004870129870131e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28-4.2004870129870131e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+2.7759740259740259e-03*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+2.2646103896103898e-03*w16*Jinv12+4.9310064935064932e-04*Jinv21*w50+2.2524350649350650e-04*Jinv01*w27+4.9310064935064932e-04*w48*Jinv01+4.8701298701298701e-04*Jinv12*w1+3.0438311688311688e-05*w10*Jinv11)*D;
  A[0][43]=(0.0)*D;
  A[0][44]=(0.0)*D;
  A[0][45]=(4.9310064935064932e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+1.0247564935064935e-04*w57*Jinv02+4.9310064935064932e-04*Jinv02*w39+9.6793831168831166e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3-1.0957792207792208e-04*Jinv22*w38-1.4204545454545455e-05*Jinv21*w59+7.4878246753246752e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+4.0584415584415584e-05*Jinv21*w2+9.6793831168831166e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+7.4878246753246752e-04*Jinv01*w45-4.2918019480519480e-04*Jinv12*w55+4.9310064935064932e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+3.7439123376623376e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01-4.2918019480519480e-04*Jinv22*w20+9.6793831168831166e-04*w43*Jinv11+1.0592532467532468e-03*Jinv21*w41+2.7394480519480518e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+4.0584415584415584e-05*Jinv11*w1-4.8396915584415583e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+4.9310064935064932e-04*Jinv22*w17+7.8530844155844158e-04*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3+9.1314935064935065e-06*w32*Jinv22+1.0592532467532468e-03*Jinv01*w39+4.0584415584415584e-05*Jinv22*w2-2.1915584415584417e-04*Jinv21*w38+7.4878246753246752e-04*Jinv02*w45+3.7439123376623376e-04*Jinv01*w24+1.0247564935064935e-04*Jinv22*w59-7.3051948051948052e-05*w34*Jinv11-3.1960227272727273e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+2.7394480519480518e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11-4.7483766233766234e-04*Jinv12*w13+1.8262987012987013e-05*w32*Jinv21-3.1960227272727273e-04*Jinv21*w20-1.0957792207792208e-04*w37*Jinv12+4.9310064935064932e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12-7.3051948051948052e-05*Jinv21*w35+1.8262987012987013e-05*w31*Jinv11+7.4878246753246752e-04*Jinv22*w26+7.4878246753246752e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21-4.2918019480519480e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12+7.8530844155844158e-04*w21*Jinv02-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33-4.7483766233766234e-04*Jinv02*w12-3.1960227272727273e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01+7.4878246753246752e-04*w47*Jinv21+7.8530844155844158e-04*Jinv22*w23-4.2918019480519480e-04*w52*Jinv11+9.1314935064935065e-06*w31*Jinv12+7.4878246753246752e-04*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8-1.3697240259740259e-04*Jinv22*w50+3.9265422077922079e-04*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+3.9265422077922079e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+9.1314935064935065e-06*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11-1.5523538961038962e-04*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5+7.4878246753246752e-04*w47*Jinv22-3.1047077922077924e-04*w49*Jinv11-4.8396915584415583e-04*Jinv21*w14+1.0247564935064935e-04*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18-4.8396915584415583e-04*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.9265422077922079e-04*Jinv21*w23-1.5523538961038962e-04*Jinv01*w54-1.5523538961038962e-04*w55*Jinv11-1.3697240259740259e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.7483766233766234e-04*Jinv22*w14+1.0592532467532468e-03*w40*Jinv11-1.0957792207792208e-04*Jinv02*w36+2.7394480519480518e-04*Jinv12*w43+1.8262987012987013e-05*Jinv01*w30-1.0957792207792208e-04*Jinv02*w51-4.2918019480519480e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.7439123376623376e-04*w25*Jinv11+4.9310064935064932e-04*w16*Jinv12-3.1047077922077924e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-3.1047077922077924e-04*w48*Jinv01+4.0584415584415584e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[0][46]=(0.0)*D;
  A[0][47]=(0.0)*D;
  A[0][48]=(-9.1314935064935065e-06*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+1.3697240259740259e-04*Jinv20*w8-1.2175324675324675e-04*w57*Jinv02-9.1314935064935065e-06*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39-2.7394480519480518e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00+4.7483766233766234e-04*w32*Jinv20-9.1314935064935065e-06*Jinv01*w3+1.3697240259740259e-04*Jinv20*w23+2.5568181818181820e-04*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-2.7394480519480521e-05*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-4.0584415584415584e-05*Jinv21*w2-2.7394480519480518e-04*w43*Jinv10-2.7394480519480518e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10+2.5568181818181820e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12-7.8530844155844158e-04*w48*Jinv00-2.7394480519480521e-05*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-4.0584415584415584e-05*w0*Jinv01-4.0584415584415584e-05*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-2.7394480519480521e-05*Jinv22*w20-2.7394480519480518e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41+2.5568181818181820e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-1.1668019480519480e-04*w11*Jinv22-4.0584415584415584e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-7.8530844155844158e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10+2.5568181818181820e-04*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+4.7483766233766234e-04*w31*Jinv10-5.5702110389610384e-04*Jinv02*w3+1.3697240259740259e-04*w21*Jinv00-6.2094155844155847e-04*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38+2.5568181818181820e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-1.2175324675324675e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34+2.5568181818181820e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-9.1314935064935065e-06*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20-5.5702110389610384e-04*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+4.7483766233766234e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+1.3697240259740259e-04*w6*Jinv00+2.5568181818181820e-04*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+4.7483766233766234e-04*w31*Jinv11-2.7394480519480521e-05*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11-9.1314935064935065e-06*Jinv20*w14-9.1314935064935065e-06*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54-5.5702110389610384e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-7.8530844155844158e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00-5.5702110389610384e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11-1.1668019480519480e-04*Jinv02*w9+1.3697240259740259e-04*w6*Jinv01+1.3697240259740259e-04*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11-6.2094155844155847e-04*w31*Jinv12+2.5568181818181820e-04*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+1.3697240259740259e-04*w22*Jinv10-1.2784090909090910e-04*Jinv22*w50+1.3697240259740259e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7+1.3697240259740259e-04*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51-6.2094155844155847e-04*Jinv02*w30-1.2784090909090910e-04*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+1.3697240259740259e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.1668019480519480e-04*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12-5.5702110389610384e-04*Jinv22*w5+2.5568181818181820e-04*w47*Jinv22-7.8530844155844158e-04*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-2.7394480519480518e-04*Jinv20*w44-1.2175324675324675e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12+1.3697240259740259e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-4.0584415584415584e-05*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+1.3697240259740259e-04*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-4.0584415584415584e-05*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-1.2784090909090910e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.7483766233766234e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20-1.1668019480519480e-04*Jinv22*w29-5.5702110389610384e-04*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+2.5568181818181820e-04*Jinv02*w36+2.5568181818181820e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20+4.7483766233766234e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-2.7394480519480518e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20+2.5568181818181820e-04*w16*Jinv12-7.8530844155844158e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-7.8530844155844158e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[0][49]=(0.0)*D;
  A[0][50]=(0.0)*D;
  A[0][51]=(9.1314935064935065e-06*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+7.4878246753246752e-04*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8-9.2329545454545453e-05*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39+9.1314935064935065e-06*w3*Jinv00-4.7483766233766234e-04*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23-1.5523538961038962e-04*Jinv22*w38+7.4878246753246752e-04*Jinv00*w54-2.1915584415584417e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+2.7394480519480518e-04*w43*Jinv10+3.7439123376623376e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10-7.3051948051948052e-05*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+7.8530844155844158e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+1.0592532467532468e-03*Jinv22*w35+4.0584415584415584e-05*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-4.2918019480519480e-04*Jinv22*w20-4.2918019480519480e-04*w45*Jinv00+9.6793831168831166e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27-1.4204545454545455e-05*w11*Jinv22+7.8530844155844158e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10-7.3051948051948052e-05*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-4.7483766233766234e-04*w31*Jinv10+1.8262987012987013e-05*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-4.8396915584415583e-04*w32*Jinv22+4.0584415584415584e-05*Jinv22*w2-1.0957792207792208e-04*Jinv00*w18-4.2918019480519480e-04*w36*Jinv00-3.1960227272727273e-04*Jinv02*w45-9.2329545454545453e-05*Jinv22*w59+7.4878246753246752e-04*w51*Jinv00+1.0592532467532468e-03*Jinv12*w34+9.6793831168831166e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv10+4.9310064935064932e-04*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13-1.0957792207792208e-04*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10-1.3697240259740259e-04*w6*Jinv00-1.5523538961038962e-04*w37*Jinv12+7.4878246753246752e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41-1.0957792207792208e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20+7.4878246753246752e-04*w52*Jinv12-2.1915584415584417e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14+3.7439123376623376e-04*Jinv02*w54+1.8262987012987013e-05*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+7.8530844155844158e-04*w49*Jinv10+4.9310064935064932e-04*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-1.3697240259740259e-04*w7*Jinv10-4.2918019480519480e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-4.8396915584415583e-04*w31*Jinv12-3.1960227272727273e-04*w46*Jinv12+7.4878246753246752e-04*Jinv20*w53-3.1047077922077924e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+3.9265422077922079e-04*Jinv22*w50-3.1047077922077924e-04*Jinv12*w7-4.8396915584415583e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49+7.4878246753246752e-04*w52*Jinv10+1.0247564935064935e-04*w9*Jinv00-4.2918019480519480e-04*w37*Jinv10-1.0957792207792208e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27-3.1047077922077924e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+1.8262987012987013e-05*Jinv22*w5-3.1960227272727273e-04*w47*Jinv22+1.0247564935064935e-04*w10*Jinv10+2.7394480519480518e-04*Jinv20*w44-9.2329545454545453e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+1.0592532467532468e-03*Jinv02*w33+4.0584415584415584e-05*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.0957792207792208e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.9265422077922079e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.7483766233766234e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14-1.5523538961038962e-04*Jinv02*w36+9.6793831168831166e-04*Jinv12*w43+9.1314935064935065e-06*w5*Jinv20+7.4878246753246752e-04*Jinv02*w51+3.7439123376623376e-04*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+7.4878246753246752e-04*w55*Jinv10+2.7394480519480518e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20-7.3051948051948052e-05*w16*Jinv12+4.0584415584415584e-05*Jinv12*w1)*D;
  A[0][52]=(0.0)*D;
  A[0][53]=(0.0)*D;
  A[0][54]=(1.0592532467532468e-03*w40*Jinv12+3.7439123376623376e-04*Jinv22*w53-9.2329545454545453e-05*w57*Jinv02+1.0592532467532468e-03*Jinv02*w39+2.7394480519480518e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3-3.1960227272727273e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-4.2918019480519480e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+4.0584415584415584e-05*Jinv21*w2+2.7394480519480518e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.2918019480519480e-04*Jinv01*w45+7.4878246753246752e-04*Jinv12*w55-7.3051948051948052e-05*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01-2.1915584415584417e-04*Jinv22*w20+2.7394480519480518e-04*w43*Jinv11+4.9310064935064932e-04*Jinv21*w41+9.6793831168831166e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+4.0584415584415584e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-7.3051948051948052e-05*Jinv22*w17-3.1047077922077924e-04*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3-4.8396915584415583e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39+4.0584415584415584e-05*Jinv22*w2-4.2918019480519480e-04*Jinv21*w38-1.5523538961038962e-04*Jinv02*w45-1.0957792207792208e-04*Jinv01*w24-9.2329545454545453e-05*Jinv22*w59+4.9310064935064932e-04*w34*Jinv11-1.0957792207792208e-04*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+9.6793831168831166e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11+1.8262987012987013e-05*Jinv12*w13-4.7483766233766234e-04*w32*Jinv21-1.0957792207792208e-04*Jinv21*w20-3.1960227272727273e-04*w37*Jinv12+1.0592532467532468e-03*Jinv22*w41+3.7439123376623376e-04*w52*Jinv12+4.9310064935064932e-04*Jinv21*w35-4.7483766233766234e-04*w31*Jinv11-4.2918019480519480e-04*Jinv22*w26-4.2918019480519480e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21+7.4878246753246752e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12-3.1047077922077924e-04*w21*Jinv02+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33+1.8262987012987013e-05*Jinv02*w12-1.0957792207792208e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01-4.2918019480519480e-04*w47*Jinv21-3.1047077922077924e-04*Jinv22*w23+7.4878246753246752e-04*w52*Jinv11-4.8396915584415583e-04*w31*Jinv12-1.5523538961038962e-04*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8+3.9265422077922079e-04*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-4.8396915584415583e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49-4.2918019480519480e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11+7.4878246753246752e-04*Jinv21*w56-1.4204545454545455e-05*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5-1.5523538961038962e-04*w47*Jinv22+7.8530844155844158e-04*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14-9.2329545454545453e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+9.1314935064935065e-06*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33-1.3697240259740259e-04*Jinv21*w23+7.4878246753246752e-04*Jinv01*w54+7.4878246753246752e-04*w55*Jinv11+3.9265422077922079e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9-1.4204545454545455e-05*Jinv22*w29+1.8262987012987013e-05*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-3.1960227272727273e-04*Jinv02*w36+9.6793831168831166e-04*Jinv12*w43-4.7483766233766234e-04*Jinv01*w30+3.7439123376623376e-04*Jinv02*w51+7.4878246753246752e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28-1.0957792207792208e-04*w25*Jinv11-7.3051948051948052e-05*w16*Jinv12+7.8530844155844158e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+7.8530844155844158e-04*w48*Jinv01+4.0584415584415584e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[0][55]=(0.0)*D;
  A[0][56]=(0.0)*D;
  A[0][57]=(-4.2613636363636367e-05*w40*Jinv12+1.5726461038961039e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-4.2613636363636367e-05*Jinv02*w39-1.1668019480519480e-04*Jinv22*w38+1.0247564935064935e-04*w25*Jinv12+1.8939393939393939e-04*w0*Jinv02+1.5726461038961039e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+5.4112554112554113e-05*w10*Jinv12+1.0247564935064935e-04*Jinv02*w24-4.2613636363636367e-05*Jinv22*w35+1.0247564935064935e-04*Jinv22*w20-1.9480519480519481e-04*w42*Jinv02+5.4112554112554113e-05*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17-1.4204545454545455e-05*w22*Jinv12+6.4935064935064935e-05*Jinv02*w3+8.3198051948051951e-05*w32*Jinv22+1.8939393939393939e-04*Jinv22*w2-1.1668019480519480e-04*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-4.2613636363636367e-05*Jinv12*w34-1.9480519480519481e-04*Jinv22*w44+6.4935064935064935e-05*Jinv12*w13-1.1668019480519480e-04*w37*Jinv12-4.2613636363636367e-05*Jinv22*w41+1.5726461038961039e-04*w52*Jinv12+1.0247564935064935e-04*Jinv22*w26+1.5726461038961039e-04*Jinv02*w54+6.4935064935064935e-05*w4*Jinv12-1.4204545454545455e-05*w21*Jinv02+6.4935064935064935e-05*Jinv02*w12+5.4112554112554113e-05*Jinv02*w9-1.4204545454545455e-05*Jinv22*w23+8.3198051948051951e-05*w31*Jinv12-1.1668019480519480e-04*w46*Jinv12-1.4204545454545455e-05*Jinv22*w8+2.0495129870129870e-04*Jinv22*w50-1.4204545454545455e-05*Jinv12*w7+8.3198051948051951e-05*Jinv02*w30+2.0495129870129870e-04*Jinv12*w49+5.4112554112554113e-05*Jinv02*w27-1.4204545454545455e-05*w6*Jinv02+1.0247564935064935e-04*w19*Jinv12+6.4935064935064935e-05*Jinv22*w5-1.1668019480519480e-04*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12+1.0247564935064935e-04*Jinv02*w18-4.2613636363636367e-05*Jinv02*w33+2.0495129870129870e-04*w48*Jinv02+5.4112554112554113e-05*Jinv22*w29+6.4935064935064935e-05*Jinv22*w14-1.1668019480519480e-04*Jinv02*w36-1.9480519480519481e-04*Jinv12*w43+1.5726461038961039e-04*Jinv02*w51+1.5726461038961039e-04*Jinv22*w56+5.4112554112554113e-05*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+1.8939393939393939e-04*Jinv12*w1)*D;
  A[0][58]=(0.0)*D;
  A[0][59]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(-6.4935064935064935e-04*Jinv00*w12-9.4696969696969697e-05*w28*Jinv10-1.2175324675324675e-04*w40*Jinv12-2.0292207792207792e-05*Jinv22*w53+3.2467532467532468e-04*Jinv20*w8-9.4696969696969697e-05*w57*Jinv02-6.4935064935064935e-04*w13*Jinv10-1.2175324675324675e-04*Jinv02*w39-2.4350649350649351e-04*Jinv21*w44-6.4935064935064935e-04*w3*Jinv00-6.4935064935064935e-04*w32*Jinv20-6.4935064935064935e-04*Jinv01*w3+3.2467532467532468e-04*Jinv20*w23-2.0292207792207792e-05*Jinv22*w38-2.0292207792207792e-05*Jinv00*w54-9.4696969696969697e-05*Jinv21*w59-2.0292207792207792e-05*w25*Jinv12-8.3874458874458877e-04*w0*Jinv02-8.3874458874458877e-04*Jinv21*w2-2.4350649350649351e-04*w43*Jinv10-2.4350649350649351e-04*w42*Jinv01-9.4696969696969697e-05*w11*Jinv21-1.2175324675324675e-04*w16*Jinv11-1.2175324675324675e-04*Jinv21*w17-2.0292207792207792e-05*Jinv01*w45-2.0292207792207792e-05*Jinv12*w55-9.4696969696969697e-05*w58*Jinv10-1.2175324675324675e-04*w15*Jinv02-9.4696969696969697e-05*w10*Jinv12+3.2467532467532468e-04*w48*Jinv00-2.0292207792207792e-05*Jinv02*w24-1.2175324675324675e-04*Jinv22*w35-9.4696969696969697e-05*w58*Jinv11-9.4696969696969697e-05*w57*Jinv01-2.0292207792207792e-05*Jinv21*w26-8.3874458874458877e-04*w0*Jinv01-8.3874458874458877e-04*w1*Jinv10-1.2175324675324675e-04*w16*Jinv10-2.0292207792207792e-05*Jinv22*w20-2.4350649350649351e-04*w43*Jinv11-2.0292207792207792e-05*w45*Jinv00-1.2175324675324675e-04*Jinv21*w41-2.4350649350649351e-04*w42*Jinv02-9.4696969696969697e-05*Jinv00*w27-9.4696969696969697e-05*w11*Jinv22-8.3874458874458877e-04*Jinv11*w1-6.4935064935064935e-04*w13*Jinv11-1.2175324675324675e-04*w15*Jinv01-9.4696969696969697e-05*w28*Jinv11+3.2467532467532468e-04*Jinv20*w50-9.4696969696969697e-05*Jinv20*w29-2.0292207792207792e-05*w46*Jinv10-1.2175324675324675e-04*Jinv22*w17-1.2175324675324675e-04*Jinv00*w33+3.2467532467532468e-04*w22*Jinv12-6.4935064935064935e-04*w31*Jinv10-6.4935064935064935e-04*Jinv02*w3+3.2467532467532468e-04*w21*Jinv00-6.4935064935064935e-04*w32*Jinv22-1.2175324675324675e-04*Jinv01*w39-8.3874458874458877e-04*Jinv22*w2-2.0292207792207792e-05*Jinv00*w18-2.0292207792207792e-05*w36*Jinv00-2.0292207792207792e-05*Jinv21*w38-2.0292207792207792e-05*Jinv02*w45-2.0292207792207792e-05*Jinv01*w24-9.4696969696969697e-05*Jinv22*w59-1.2175324675324675e-04*w34*Jinv11-2.0292207792207792e-05*w19*Jinv11-2.0292207792207792e-05*Jinv21*w53-2.0292207792207792e-05*w51*Jinv00-1.2175324675324675e-04*Jinv12*w34-2.4350649350649351e-04*Jinv22*w44-6.4935064935064935e-04*w4*Jinv10-6.4935064935064935e-04*w4*Jinv11-1.2175324675324675e-04*w41*Jinv20-6.4935064935064935e-04*Jinv12*w13-2.0292207792207792e-05*w19*Jinv10-1.2175324675324675e-04*w34*Jinv10-6.4935064935064935e-04*w32*Jinv21-2.0292207792207792e-05*Jinv21*w20+3.2467532467532468e-04*w6*Jinv00-2.0292207792207792e-05*w37*Jinv12-2.0292207792207792e-05*Jinv20*w56-1.2175324675324675e-04*Jinv22*w41-2.0292207792207792e-05*w26*Jinv20-9.4696969696969697e-05*w11*Jinv20-2.0292207792207792e-05*w52*Jinv12-1.2175324675324675e-04*Jinv21*w35-6.4935064935064935e-04*w31*Jinv11-2.0292207792207792e-05*Jinv22*w26-2.0292207792207792e-05*w46*Jinv11-6.4935064935064935e-04*Jinv20*w14-6.4935064935064935e-04*w5*Jinv21-2.0292207792207792e-05*Jinv02*w54-6.4935064935064935e-04*w4*Jinv12+3.2467532467532468e-04*w21*Jinv02+3.2467532467532468e-04*w49*Jinv10-9.4696969696969697e-05*Jinv21*w29-1.2175324675324675e-04*Jinv01*w33-1.2175324675324675e-04*w39*Jinv00-6.4935064935064935e-04*Jinv02*w12-2.0292207792207792e-05*Jinv01*w18-2.0292207792207792e-05*w37*Jinv11-9.4696969696969697e-05*Jinv02*w9+3.2467532467532468e-04*w6*Jinv01+3.2467532467532468e-04*w7*Jinv10-2.0292207792207792e-05*w47*Jinv21-2.0292207792207792e-05*Jinv20*w38+3.2467532467532468e-04*Jinv22*w23-2.0292207792207792e-05*w52*Jinv11-6.4935064935064935e-04*w31*Jinv12-2.0292207792207792e-05*w46*Jinv12-2.0292207792207792e-05*Jinv20*w53+3.2467532467532468e-04*Jinv22*w8+3.2467532467532468e-04*w22*Jinv10+3.2467532467532468e-04*Jinv22*w50+3.2467532467532468e-04*w22*Jinv11+3.2467532467532468e-04*Jinv12*w7+3.2467532467532468e-04*w21*Jinv01-2.0292207792207792e-05*Jinv01*w51-6.4935064935064935e-04*Jinv02*w30+3.2467532467532468e-04*Jinv12*w49-2.0292207792207792e-05*w36*Jinv01-2.0292207792207792e-05*w52*Jinv10+3.2467532467532468e-04*w7*Jinv11-9.4696969696969697e-05*w9*Jinv00-2.0292207792207792e-05*w37*Jinv10-2.0292207792207792e-05*Jinv21*w56-2.0292207792207792e-05*w24*Jinv00-9.4696969696969697e-05*Jinv02*w27+3.2467532467532468e-04*w6*Jinv02-2.0292207792207792e-05*w19*Jinv12-6.4935064935064935e-04*Jinv22*w5-2.0292207792207792e-05*w47*Jinv22+3.2467532467532468e-04*w49*Jinv11-6.4935064935064935e-04*Jinv21*w14-9.4696969696969697e-05*w10*Jinv10-2.4350649350649351e-04*Jinv20*w44-9.4696969696969697e-05*w58*Jinv12-2.0292207792207792e-05*Jinv02*w18-6.4935064935064935e-04*Jinv01*w12+3.2467532467532468e-04*Jinv21*w8-9.4696969696969697e-05*w57*Jinv00-9.4696969696969697e-05*Jinv20*w59-1.2175324675324675e-04*Jinv02*w33-8.3874458874458877e-04*w0*Jinv00-1.2175324675324675e-04*w35*Jinv20+3.2467532467532468e-04*Jinv21*w23-2.0292207792207792e-05*Jinv01*w54-2.0292207792207792e-05*w55*Jinv11-2.0292207792207792e-05*w25*Jinv10-8.3874458874458877e-04*Jinv20*w2-1.2175324675324675e-04*w15*Jinv00+3.2467532467532468e-04*w48*Jinv02-9.4696969696969697e-05*Jinv01*w9-1.2175324675324675e-04*Jinv20*w17-6.4935064935064935e-04*w30*Jinv00-1.2175324675324675e-04*w40*Jinv10-2.0292207792207792e-05*w20*Jinv20-9.4696969696969697e-05*Jinv22*w29-6.4935064935064935e-04*Jinv22*w14-1.2175324675324675e-04*w40*Jinv11-2.0292207792207792e-05*Jinv02*w36-2.4350649350649351e-04*Jinv12*w43-6.4935064935064935e-04*w5*Jinv20-6.4935064935064935e-04*Jinv01*w30-2.0292207792207792e-05*Jinv02*w51-2.0292207792207792e-05*Jinv22*w56-9.4696969696969697e-05*Jinv12*w28-2.0292207792207792e-05*w25*Jinv11-2.0292207792207792e-05*w55*Jinv10-2.4350649350649351e-04*w42*Jinv00-2.0292207792207792e-05*w47*Jinv20-1.2175324675324675e-04*w16*Jinv12+3.2467532467532468e-04*Jinv21*w50-9.4696969696969697e-05*Jinv01*w27+3.2467532467532468e-04*w48*Jinv01-8.3874458874458877e-04*Jinv12*w1-9.4696969696969697e-05*w10*Jinv11)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(0.0)*D;
  A[1][4]=(1.1414366883116883e-03*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.2784090909090910e-04*Jinv22*w53-4.0178571428571428e-04*Jinv20*w8+9.2329545454545453e-05*w57*Jinv02+1.1414366883116883e-03*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+6.7573051948051951e-04*Jinv21*w44+1.1870941558441558e-03*w3*Jinv00+1.1414366883116883e-03*w32*Jinv20+9.1314935064935068e-05*Jinv01*w3-6.6659902597402602e-04*Jinv20*w23+2.1002435064935065e-04*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+9.2329545454545453e-05*Jinv21*w59-1.2784090909090910e-04*w25*Jinv12-2.8409090909090908e-04*w0*Jinv02-2.8409090909090908e-04*Jinv21*w2+1.8262987012987014e-04*w43*Jinv10+6.7573051948051951e-04*w42*Jinv01+5.6818181818181818e-05*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17-6.3920454545454550e-05*Jinv01*w45-6.3920454545454550e-05*Jinv12*w55+1.5726461038961039e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+5.6818181818181818e-05*w10*Jinv12-6.6659902597402602e-04*w48*Jinv00-1.2784090909090910e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35+9.2329545454545453e-05*w58*Jinv11+9.2329545454545453e-05*w57*Jinv01-1.2784090909090910e-04*Jinv21*w26-2.8409090909090908e-04*w0*Jinv01+1.2987012987012987e-03*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+2.1002435064935065e-04*Jinv22*w20+6.7573051948051951e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+6.7573051948051951e-04*w42*Jinv02+1.5726461038961039e-04*Jinv00*w27+5.6818181818181818e-05*w11*Jinv22-2.8409090909090908e-04*Jinv11*w1+4.5657467532467534e-05*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01+9.2329545454545453e-05*w28*Jinv11-6.6659902597402602e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+1.1414366883116883e-03*w31*Jinv10+9.1314935064935068e-05*Jinv02*w3-6.6659902597402602e-04*w21*Jinv00+4.5657467532467534e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-2.8409090909090908e-04*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1002435064935065e-04*Jinv21*w38-6.3920454545454550e-05*Jinv02*w45-1.2784090909090910e-04*Jinv01*w24+9.2329545454545453e-05*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+2.1002435064935065e-04*w19*Jinv11-1.2784090909090910e-04*Jinv21*w53-1.1870941558441559e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+6.7573051948051951e-04*Jinv22*w44+1.1870941558441558e-03*w4*Jinv10+9.1314935064935068e-05*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20+4.5657467532467534e-05*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10+4.5657467532467534e-05*w32*Jinv21+2.1002435064935065e-04*Jinv21*w20-4.0178571428571428e-04*w6*Jinv00+2.1002435064935065e-04*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-1.1870941558441559e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20-1.2784090909090910e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+4.5657467532467534e-05*w31*Jinv11-1.2784090909090910e-04*Jinv22*w26-6.3920454545454550e-05*w46*Jinv11+1.1414366883116883e-03*Jinv20*w14+9.1314935064935068e-05*w5*Jinv21-6.3920454545454550e-05*Jinv02*w54+9.1314935064935068e-05*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02-6.6659902597402602e-04*w49*Jinv10+9.2329545454545453e-05*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00+4.5657467532467534e-05*Jinv02*w12+2.1002435064935065e-04*Jinv01*w18+2.1002435064935065e-04*w37*Jinv11+5.6818181818181818e-05*Jinv02*w9+1.4610389610389610e-04*w6*Jinv01-4.0178571428571428e-04*w7*Jinv10-6.3920454545454550e-05*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23-1.2784090909090910e-04*w52*Jinv11+4.5657467532467534e-05*w31*Jinv12-6.3920454545454550e-05*w46*Jinv12-1.1870941558441559e-04*Jinv20*w53+1.4610389610389610e-04*Jinv22*w8-6.6659902597402602e-04*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11+1.4610389610389610e-04*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01-1.2784090909090910e-04*Jinv01*w51+4.5657467532467534e-05*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49+2.1002435064935065e-04*w36*Jinv01-1.1870941558441559e-04*w52*Jinv10+1.4610389610389610e-04*w7*Jinv11+1.2175324675324675e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10-6.3920454545454550e-05*Jinv21*w56-1.1870941558441559e-04*w24*Jinv00+9.2329545454545453e-05*Jinv02*w27+1.4610389610389610e-04*w6*Jinv02+2.1002435064935065e-04*w19*Jinv12+9.1314935064935068e-05*Jinv22*w5-6.3920454545454550e-05*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+4.5657467532467534e-05*Jinv21*w14+1.2175324675324675e-04*w10*Jinv10+1.8262987012987014e-04*Jinv20*w44+9.2329545454545453e-05*w58*Jinv12+2.1002435064935065e-04*Jinv02*w18+4.5657467532467534e-05*Jinv01*w12+1.4610389610389610e-04*Jinv21*w8+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33+1.2987012987012987e-03*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23-6.3920454545454550e-05*Jinv01*w54-6.3920454545454550e-05*w55*Jinv11-1.1870941558441559e-04*w25*Jinv10+1.2987012987012987e-03*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02+5.6818181818181818e-05*Jinv01*w9+2.1915584415584417e-04*Jinv20*w17+1.1414366883116883e-03*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+9.2329545454545453e-05*Jinv22*w29+4.5657467532467534e-05*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.1002435064935065e-04*Jinv02*w36+6.7573051948051951e-04*Jinv12*w43+1.1870941558441558e-03*w5*Jinv20+4.5657467532467534e-05*Jinv01*w30-1.2784090909090910e-04*Jinv02*w51-6.3920454545454550e-05*Jinv22*w56+9.2329545454545453e-05*Jinv12*w28-1.2784090909090910e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10+1.8262987012987014e-04*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50+9.2329545454545453e-05*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.8409090909090908e-04*Jinv12*w1+5.6818181818181818e-05*w10*Jinv11)*D;
  A[1][5]=(0.0)*D;
  A[1][6]=(0.0)*D;
  A[1][7]=(-5.5702110389610384e-04*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10+3.6525974025974026e-05*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-1.2784090909090910e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02-5.5702110389610384e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-2.7394480519480518e-04*Jinv21*w44-6.2094155844155847e-04*w3*Jinv00-5.5702110389610384e-04*w32*Jinv20+4.7483766233766234e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38-2.7394480519480521e-05*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-4.0584415584415584e-05*w0*Jinv02-4.0584415584415584e-05*Jinv21*w2+2.5568181818181820e-04*w43*Jinv10-2.7394480519480518e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-1.1668019480519480e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-4.0584415584415584e-05*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-2.7394480519480518e-04*w43*Jinv11-2.7394480519480521e-05*w45*Jinv00+3.6525974025974026e-05*Jinv21*w41-2.7394480519480518e-04*w42*Jinv02-1.1668019480519480e-04*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-4.0584415584415584e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33+1.3697240259740259e-04*w22*Jinv12-5.5702110389610384e-04*w31*Jinv10+4.7483766233766234e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-4.0584415584415584e-05*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+2.5568181818181820e-04*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-2.7394480519480518e-04*Jinv22*w44-6.2094155844155847e-04*w4*Jinv10+4.7483766233766234e-04*w4*Jinv11+2.5568181818181820e-04*w41*Jinv20-9.1314935064935065e-06*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-9.1314935064935065e-06*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-1.2784090909090910e-04*w6*Jinv00-7.4878246753246752e-04*w37*Jinv12-2.7394480519480521e-05*Jinv20*w56+3.6525974025974026e-05*Jinv22*w41+2.5568181818181820e-04*w26*Jinv20-1.2175324675324675e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35-9.1314935064935065e-06*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-5.5702110389610384e-04*Jinv20*w14+4.7483766233766234e-04*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54+4.7483766233766234e-04*w4*Jinv12+1.3697240259740259e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33+2.5568181818181820e-04*w39*Jinv00-9.1314935064935065e-06*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.8530844155844158e-04*w6*Jinv01-1.2784090909090910e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+1.3697240259740259e-04*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.5568181818181820e-04*Jinv20*w53-7.8530844155844158e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+1.3697240259740259e-04*Jinv22*w50+1.3697240259740259e-04*w22*Jinv11-7.8530844155844158e-04*Jinv12*w7+1.3697240259740259e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30+1.3697240259740259e-04*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+2.5568181818181820e-04*w52*Jinv10-7.8530844155844158e-04*w7*Jinv11-1.2175324675324675e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.5568181818181820e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-7.8530844155844158e-04*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+4.7483766233766234e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.3697240259740259e-04*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.2175324675324675e-04*w10*Jinv10+2.5568181818181820e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-7.8530844155844158e-04*Jinv21*w8-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59-4.9310064935064932e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20+1.3697240259740259e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.5568181818181820e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00+1.3697240259740259e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17-5.5702110389610384e-04*w30*Jinv00+2.5568181818181820e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29-9.1314935064935065e-06*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-2.7394480519480518e-04*Jinv12*w43-6.2094155844155847e-04*w5*Jinv20-9.1314935064935065e-06*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11-2.7394480519480521e-05*w55*Jinv10+2.5568181818181820e-04*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12+1.3697240259740259e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+1.3697240259740259e-04*w48*Jinv01-4.0584415584415584e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(0.0)*D;
  A[1][10]=(6.4935064935064935e-05*Jinv00*w12+5.4112554112554113e-05*w28*Jinv10+2.0495129870129870e-04*Jinv20*w8+6.4935064935064935e-05*w13*Jinv10+8.3198051948051951e-05*w3*Jinv00+6.4935064935064935e-05*w32*Jinv20-1.4204545454545455e-05*Jinv20*w23+1.0247564935064935e-04*Jinv00*w54-1.9480519480519481e-04*w43*Jinv10+5.4112554112554113e-05*w58*Jinv10-1.4204545454545455e-05*w48*Jinv00+1.8939393939393939e-04*w1*Jinv10-4.2613636363636367e-05*w16*Jinv10+1.0247564935064935e-04*w45*Jinv00+5.4112554112554113e-05*Jinv00*w27-1.4204545454545455e-05*Jinv20*w50+5.4112554112554113e-05*Jinv20*w29+1.0247564935064935e-04*w46*Jinv10-4.2613636363636367e-05*Jinv00*w33+6.4935064935064935e-05*w31*Jinv10-1.4204545454545455e-05*w21*Jinv00+1.5726461038961039e-04*Jinv00*w18+1.5726461038961039e-04*w36*Jinv00-1.1668019480519480e-04*w51*Jinv00+8.3198051948051951e-05*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+1.5726461038961039e-04*w19*Jinv10-4.2613636363636367e-05*w34*Jinv10+2.0495129870129870e-04*w6*Jinv00+1.0247564935064935e-04*Jinv20*w56-1.1668019480519480e-04*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20+6.4935064935064935e-05*Jinv20*w14-1.4204545454545455e-05*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+2.0495129870129870e-04*w7*Jinv10+1.5726461038961039e-04*Jinv20*w38-1.1668019480519480e-04*Jinv20*w53-1.4204545454545455e-05*w22*Jinv10-1.1668019480519480e-04*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+1.5726461038961039e-04*w37*Jinv10-1.1668019480519480e-04*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-1.9480519480519481e-04*Jinv20*w44+5.4112554112554113e-05*w57*Jinv00+5.4112554112554113e-05*Jinv20*w59+1.8939393939393939e-04*w0*Jinv00-4.2613636363636367e-05*w35*Jinv20-1.1668019480519480e-04*w25*Jinv10+1.8939393939393939e-04*Jinv20*w2-4.2613636363636367e-05*w15*Jinv00-4.2613636363636367e-05*Jinv20*w17+6.4935064935064935e-05*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+1.5726461038961039e-04*w20*Jinv20+8.3198051948051951e-05*w5*Jinv20+1.0247564935064935e-04*w55*Jinv10-1.9480519480519481e-04*w42*Jinv00+1.0247564935064935e-04*w47*Jinv20)*D;
  A[1][11]=(0.0)*D;
  A[1][12]=(0.0)*D;
  A[1][13]=(9.1314935064935068e-05*Jinv00*w12+5.6818181818181818e-05*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12-6.3920454545454550e-05*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8+9.2329545454545453e-05*w57*Jinv02+9.1314935064935068e-05*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39+1.8262987012987014e-04*Jinv21*w44+4.5657467532467534e-05*w3*Jinv00+4.5657467532467534e-05*w32*Jinv20+1.1414366883116883e-03*Jinv01*w3+1.4610389610389610e-04*Jinv20*w23-6.3920454545454550e-05*Jinv22*w38-1.2784090909090910e-04*Jinv00*w54+1.5726461038961039e-04*Jinv21*w59+2.1002435064935065e-04*w25*Jinv12-2.8409090909090908e-04*w0*Jinv02+1.2987012987012987e-03*Jinv21*w2+6.7573051948051951e-04*w43*Jinv10+1.8262987012987014e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45-1.2784090909090910e-04*Jinv12*w55+9.2329545454545453e-05*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+9.2329545454545453e-05*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00+2.1002435064935065e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+1.2987012987012987e-03*w0*Jinv01-2.8409090909090908e-04*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10-1.2784090909090910e-04*Jinv22*w20+1.8262987012987014e-04*w43*Jinv11+2.1002435064935065e-04*w45*Jinv00+2.1915584415584417e-04*Jinv21*w41+6.7573051948051951e-04*w42*Jinv02+5.6818181818181818e-05*Jinv00*w27+9.2329545454545453e-05*w11*Jinv22+1.2987012987012987e-03*Jinv11*w1+1.1870941558441558e-03*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50+5.6818181818181818e-05*Jinv20*w29+2.1002435064935065e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+1.4610389610389610e-04*w22*Jinv12+4.5657467532467534e-05*w31*Jinv10+4.5657467532467534e-05*Jinv02*w3+1.4610389610389610e-04*w21*Jinv00+4.5657467532467534e-05*w32*Jinv22+2.1915584415584417e-04*Jinv01*w39-2.8409090909090908e-04*Jinv22*w2-1.2784090909090910e-04*Jinv00*w18-6.3920454545454550e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+2.1002435064935065e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24+9.2329545454545453e-05*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11-1.1870941558441559e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-6.3920454545454550e-05*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+6.7573051948051951e-04*Jinv22*w44+4.5657467532467534e-05*w4*Jinv10+1.1414366883116883e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+9.1314935064935068e-05*Jinv12*w13-1.2784090909090910e-04*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10+1.1414366883116883e-03*w32*Jinv21-1.1870941558441559e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00-6.3920454545454550e-05*w37*Jinv12-1.2784090909090910e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+2.1002435064935065e-04*w26*Jinv20+9.2329545454545453e-05*w11*Jinv20-6.3920454545454550e-05*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35+1.1414366883116883e-03*w31*Jinv11+2.1002435064935065e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11+9.1314935064935068e-05*Jinv20*w14+1.1414366883116883e-03*w5*Jinv21-1.2784090909090910e-04*Jinv02*w54+4.5657467532467534e-05*w4*Jinv12+1.4610389610389610e-04*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10+1.2175324675324675e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+9.1314935064935068e-05*Jinv02*w12-1.1870941558441559e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+9.2329545454545453e-05*Jinv02*w9-6.6659902597402602e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-6.3920454545454550e-05*Jinv20*w38+1.4610389610389610e-04*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+4.5657467532467534e-05*w31*Jinv12+2.1002435064935065e-04*w46*Jinv12-6.3920454545454550e-05*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8+1.4610389610389610e-04*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-4.0178571428571428e-04*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7-4.0178571428571428e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+4.5657467532467534e-05*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-6.3920454545454550e-05*w52*Jinv10-6.6659902597402602e-04*w7*Jinv11+9.2329545454545453e-05*w9*Jinv00-6.3920454545454550e-05*w37*Jinv10-1.1870941558441559e-04*Jinv21*w56+2.1002435064935065e-04*w24*Jinv00+5.6818181818181818e-05*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02-1.2784090909090910e-04*w19*Jinv12+4.5657467532467534e-05*Jinv22*w5+2.1002435064935065e-04*w47*Jinv22-6.6659902597402602e-04*w49*Jinv11+1.1870941558441558e-03*Jinv21*w14+9.2329545454545453e-05*w10*Jinv10+6.7573051948051951e-04*Jinv20*w44+9.2329545454545453e-05*w58*Jinv12-1.2784090909090910e-04*Jinv02*w18+1.1870941558441558e-03*Jinv01*w12-6.6659902597402602e-04*Jinv21*w8+9.2329545454545453e-05*w57*Jinv00+9.2329545454545453e-05*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33-2.8409090909090908e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-4.0178571428571428e-04*Jinv21*w23-1.1870941558441559e-04*Jinv01*w54-1.1870941558441559e-04*w55*Jinv11+2.1002435064935065e-04*w25*Jinv10-2.8409090909090908e-04*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02+1.5726461038961039e-04*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17+4.5657467532467534e-05*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-1.2784090909090910e-04*w20*Jinv20+5.6818181818181818e-05*Jinv22*w29+9.1314935064935068e-05*Jinv22*w14+2.1915584415584417e-04*w40*Jinv11-6.3920454545454550e-05*Jinv02*w36+6.7573051948051951e-04*Jinv12*w43+4.5657467532467534e-05*w5*Jinv20+1.1414366883116883e-03*Jinv01*w30-6.3920454545454550e-05*Jinv02*w51-1.2784090909090910e-04*Jinv22*w56+5.6818181818181818e-05*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11-1.2784090909090910e-04*w55*Jinv10+6.7573051948051951e-04*w42*Jinv00+2.1002435064935065e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-6.6659902597402602e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27-6.6659902597402602e-04*w48*Jinv01-2.8409090909090908e-04*Jinv12*w1+1.5726461038961039e-04*w10*Jinv11)*D;
  A[1][14]=(0.0)*D;
  A[1][15]=(0.0)*D;
  A[1][16]=(-1.0957792207792208e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10-1.1323051948051949e-03*w40*Jinv12+4.2004870129870131e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-2.2524350649350650e-04*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.0957792207792208e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+3.4699675324675324e-04*Jinv00*w54-1.9480519480519481e-04*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-4.8701298701298701e-04*w0*Jinv02+2.4350649350649351e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17-3.1047077922077924e-04*Jinv01*w45+4.2004870129870131e-04*Jinv12*w55-1.9480519480519481e-04*w58*Jinv10-2.2646103896103898e-03*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12+2.5568181818181820e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35-1.9480519480519481e-04*w58*Jinv11-1.9480519480519481e-04*w57*Jinv01-6.2094155844155847e-04*Jinv21*w26+2.4350649350649351e-04*w0*Jinv01+2.4350649350649351e-04*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-9.1314935064935063e-04*Jinv21*w41-2.7759740259740259e-03*w42*Jinv02-4.2613636363636367e-05*Jinv00*w27-3.0438311688311688e-05*w11*Jinv22+2.4350649350649351e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+2.5568181818181820e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-2.2646103896103898e-03*Jinv22*w17-9.1314935064935063e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10+9.8620129870129864e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+4.9310064935064932e-04*w32*Jinv22-9.1314935064935063e-04*Jinv01*w39-4.8701298701298701e-04*Jinv22*w2-6.2094155844155847e-04*Jinv00*w18-3.1047077922077924e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-6.2094155844155847e-04*Jinv01*w24-2.2524350649350650e-04*Jinv22*w59-6.9399350649350649e-04*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+3.4699675324675324e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-2.7759740259740259e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10-1.0957792207792208e-04*w4*Jinv11-6.9399350649350649e-04*w41*Jinv20+9.8620129870129864e-04*Jinv12*w13-6.2094155844155847e-04*w19*Jinv10-9.1314935064935063e-04*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+3.4699675324675324e-04*Jinv20*w56-1.1323051948051949e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20+4.2004870129870131e-04*w52*Jinv12-6.9399350649350649e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-3.1047077922077924e-04*w46*Jinv11-1.0957792207792208e-04*Jinv20*w14-1.0957792207792208e-04*w5*Jinv21+4.2004870129870131e-04*Jinv02*w54+9.8620129870129864e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.5568181818181820e-04*w49*Jinv10-6.9399350649350649e-04*Jinv01*w33-6.9399350649350649e-04*w39*Jinv00+9.8620129870129864e-04*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11-3.0438311688311688e-05*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-3.1047077922077924e-04*w47*Jinv21-3.1047077922077924e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23+3.4699675324675324e-04*w52*Jinv11+4.9310064935064932e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+3.4699675324675324e-04*Jinv01*w51+4.9310064935064932e-04*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-3.1047077922077924e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+9.8620129870129864e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+2.5568181818181820e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-1.6436688311688311e-03*Jinv20*w44-2.2524350649350650e-04*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.9480519480519481e-04*w57*Jinv00-1.9480519480519481e-04*Jinv20*w59-1.1323051948051949e-03*Jinv02*w33+2.4350649350649351e-04*w0*Jinv00-9.1314935064935063e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10+2.4350649350649351e-04*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00+3.6525974025974026e-05*w48*Jinv02-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-6.9399350649350649e-04*w40*Jinv10-6.2094155844155847e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+9.8620129870129864e-04*Jinv22*w14-9.1314935064935063e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36-2.7759740259740259e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30+4.2004870129870131e-04*Jinv02*w51+4.2004870129870131e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-6.2094155844155847e-04*w25*Jinv11+3.4699675324675324e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-2.2646103896103898e-03*w16*Jinv12+2.5568181818181820e-04*Jinv21*w50+2.5568181818181820e-04*w48*Jinv01-4.8701298701298701e-04*Jinv12*w1-4.2613636363636367e-05*w10*Jinv11)*D;
  A[1][17]=(0.0)*D;
  A[1][18]=(0.0)*D;
  A[1][19]=(1.8262987012987013e-05*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10+3.9265422077922079e-04*Jinv20*w8+1.8262987012987013e-05*w13*Jinv10+2.7394480519480518e-04*Jinv21*w44-4.8396915584415583e-04*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20-4.7483766233766234e-04*Jinv01*w3-3.1047077922077924e-04*Jinv20*w23-2.1915584415584417e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+4.0584415584415584e-05*Jinv21*w2+9.6793831168831166e-04*w43*Jinv10+2.7394480519480518e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-4.2918019480519480e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01+4.0584415584415584e-05*w1*Jinv10+1.0592532467532468e-03*w16*Jinv10+2.7394480519480518e-04*w43*Jinv11-4.2918019480519480e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-1.4204545454545455e-05*Jinv00*w27+4.0584415584415584e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10-3.1047077922077924e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+7.4878246753246752e-04*Jinv00*w18+3.7439123376623376e-04*w36*Jinv00+7.4878246753246752e-04*Jinv21*w38-4.2918019480519480e-04*Jinv01*w24+4.9310064935064932e-04*w34*Jinv11+7.4878246753246752e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-3.1960227272727273e-04*w51*Jinv00-4.8396915584415583e-04*w4*Jinv10-4.7483766233766234e-04*w4*Jinv11-7.3051948051948052e-05*w41*Jinv20+7.4878246753246752e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21+7.4878246753246752e-04*Jinv21*w20+3.9265422077922079e-04*w6*Jinv00-2.1915584415584417e-04*Jinv20*w56-1.5523538961038962e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20+4.9310064935064932e-04*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11+1.8262987012987013e-05*Jinv20*w14-4.7483766233766234e-04*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-7.3051948051948052e-05*w39*Jinv00+7.4878246753246752e-04*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+7.8530844155844158e-04*w6*Jinv01+3.9265422077922079e-04*w7*Jinv10-1.0957792207792208e-04*w47*Jinv21+3.7439123376623376e-04*Jinv20*w38-4.2918019480519480e-04*w52*Jinv11-3.1960227272727273e-04*Jinv20*w53-3.1047077922077924e-04*w22*Jinv10-1.3697240259740259e-04*w22*Jinv11-1.3697240259740259e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01-3.1960227272727273e-04*w52*Jinv10+7.8530844155844158e-04*w7*Jinv11-9.2329545454545453e-05*w9*Jinv00+3.7439123376623376e-04*w37*Jinv10-1.0957792207792208e-04*Jinv21*w56-1.5523538961038962e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14-9.2329545454545453e-05*w10*Jinv10+9.6793831168831166e-04*Jinv20*w44+9.1314935064935065e-06*Jinv01*w12+7.8530844155844158e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+4.0584415584415584e-05*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23-1.0957792207792208e-04*Jinv01*w54-1.0957792207792208e-04*w55*Jinv11-1.5523538961038962e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+1.0592532467532468e-03*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+1.0592532467532468e-03*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+7.4878246753246752e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-4.8396915584415583e-04*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30-4.2918019480519480e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+9.6793831168831166e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[1][20]=(0.0)*D;
  A[1][21]=(0.0)*D;
  A[1][22]=(4.7483766233766234e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+1.3697240259740259e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+4.7483766233766234e-04*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+2.5568181818181820e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00-9.1314935064935065e-06*w32*Jinv20-5.5702110389610384e-04*Jinv01*w3-7.8530844155844158e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-1.1668019480519480e-04*Jinv21*w59-7.4878246753246752e-04*w25*Jinv12-4.0584415584415584e-05*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2-2.7394480519480518e-04*w43*Jinv10+2.5568181818181820e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12+1.3697240259740259e-04*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-4.0584415584415584e-05*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+2.5568181818181820e-04*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-2.7394480519480518e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-6.2094155844155847e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.2175324675324675e-04*w28*Jinv11+1.3697240259740259e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.8530844155844158e-04*w22*Jinv12-9.1314935064935065e-06*w31*Jinv10-9.1314935064935065e-06*Jinv02*w3-7.8530844155844158e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-4.0584415584415584e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00-2.7394480519480521e-05*Jinv21*w38-7.4878246753246752e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59+2.5568181818181820e-04*w34*Jinv11+2.5568181818181820e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-2.7394480519480518e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-5.5702110389610384e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20+4.7483766233766234e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-5.5702110389610384e-04*w32*Jinv21+2.5568181818181820e-04*Jinv21*w20+1.3697240259740259e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12+2.5568181818181820e-04*Jinv21*w35-5.5702110389610384e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11+4.7483766233766234e-04*Jinv20*w14-5.5702110389610384e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54-9.1314935064935065e-06*w4*Jinv12-7.8530844155844158e-04*w21*Jinv02+1.3697240259740259e-04*w49*Jinv10-1.2175324675324675e-04*Jinv21*w29+2.5568181818181820e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00+4.7483766233766234e-04*Jinv02*w12+2.5568181818181820e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+1.3697240259740259e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38-7.8530844155844158e-04*Jinv22*w23-2.7394480519480521e-05*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.3697240259740259e-04*Jinv22*w8-7.8530844155844158e-04*w22*Jinv10+1.3697240259740259e-04*Jinv22*w50-1.2784090909090910e-04*w22*Jinv11+1.3697240259740259e-04*Jinv12*w7-1.2784090909090910e-04*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30+1.3697240259740259e-04*Jinv12*w49-2.7394480519480521e-05*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+2.5568181818181820e-04*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+1.3697240259740259e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12-9.1314935064935065e-06*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-6.2094155844155847e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-2.7394480519480518e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18-6.2094155844155847e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-4.0584415584415584e-05*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20-1.2784090909090910e-04*Jinv21*w23+2.5568181818181820e-04*Jinv01*w54+2.5568181818181820e-04*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-4.0584415584415584e-05*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+1.3697240259740259e-04*w48*Jinv02-1.1668019480519480e-04*Jinv01*w9-4.9310064935064932e-04*Jinv20*w17-9.1314935064935065e-06*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.7483766233766234e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36-2.7394480519480518e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20-5.5702110389610384e-04*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-2.7394480519480518e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-1.2175324675324675e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-4.0584415584415584e-05*Jinv12*w1-1.1668019480519480e-04*w10*Jinv11)*D;
  A[1][23]=(0.0)*D;
  A[1][24]=(0.0)*D;
  A[1][25]=(-4.7483766233766234e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.3697240259740259e-04*Jinv20*w8-4.7483766233766234e-04*w13*Jinv10+9.6793831168831166e-04*Jinv21*w44+9.1314935064935065e-06*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20+1.8262987012987013e-05*Jinv01*w3+7.8530844155844158e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+4.0584415584415584e-05*Jinv21*w2+2.7394480519480518e-04*w43*Jinv10+9.6793831168831166e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+1.0592532467532468e-03*w16*Jinv11+1.0592532467532468e-03*Jinv21*w17+3.7439123376623376e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01+7.4878246753246752e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01+4.0584415584415584e-05*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+9.6793831168831166e-04*w43*Jinv11+7.4878246753246752e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27+4.0584415584415584e-05*Jinv11*w1-4.8396915584415583e-04*w13*Jinv11+1.0592532467532468e-03*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10+7.8530844155844158e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.2918019480519480e-04*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00-4.2918019480519480e-04*Jinv21*w38+7.4878246753246752e-04*Jinv01*w24-7.3051948051948052e-05*w34*Jinv11-1.5523538961038962e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-1.0957792207792208e-04*w51*Jinv00+9.1314935064935065e-06*w4*Jinv10+1.8262987012987013e-05*w4*Jinv11+4.9310064935064932e-04*w41*Jinv20-4.2918019480519480e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21-1.5523538961038962e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+7.4878246753246752e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20-7.3051948051948052e-05*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11+3.7439123376623376e-04*w46*Jinv11-4.7483766233766234e-04*Jinv20*w14+1.8262987012987013e-05*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33+4.9310064935064932e-04*w39*Jinv00-1.5523538961038962e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-3.1047077922077924e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10+3.7439123376623376e-04*w47*Jinv21-1.0957792207792208e-04*Jinv20*w38-2.1915584415584417e-04*w52*Jinv11-1.0957792207792208e-04*Jinv20*w53+7.8530844155844158e-04*w22*Jinv10+3.9265422077922079e-04*w22*Jinv11+3.9265422077922079e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-1.0957792207792208e-04*w52*Jinv10-3.1047077922077924e-04*w7*Jinv11+1.0247564935064935e-04*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10-3.1960227272727273e-04*Jinv21*w56+7.4878246753246752e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11-4.8396915584415583e-04*Jinv21*w14+1.0247564935064935e-04*w10*Jinv10+2.7394480519480518e-04*Jinv20*w44-4.8396915584415583e-04*Jinv01*w12-3.1047077922077924e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+4.0584415584415584e-05*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.9265422077922079e-04*Jinv21*w23-3.1960227272727273e-04*Jinv01*w54-3.1960227272727273e-04*w55*Jinv11+7.4878246753246752e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-1.4204545454545455e-05*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-4.2918019480519480e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11+9.1314935064935065e-06*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30+7.4878246753246752e-04*w25*Jinv11-4.2918019480519480e-04*w55*Jinv10+2.7394480519480518e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[1][26]=(0.0)*D;
  A[1][27]=(0.0)*D;
  A[1][28]=(-1.9480519480519481e-04*Jinv21*w44+6.4935064935064935e-05*Jinv01*w3+5.4112554112554113e-05*Jinv21*w59+1.8939393939393939e-04*Jinv21*w2-1.9480519480519481e-04*w42*Jinv01+5.4112554112554113e-05*w11*Jinv21-4.2613636363636367e-05*w16*Jinv11-4.2613636363636367e-05*Jinv21*w17+1.5726461038961039e-04*Jinv01*w45+5.4112554112554113e-05*w58*Jinv11+5.4112554112554113e-05*w57*Jinv01+1.5726461038961039e-04*Jinv21*w26+1.8939393939393939e-04*w0*Jinv01-1.9480519480519481e-04*w43*Jinv11-4.2613636363636367e-05*Jinv21*w41+1.8939393939393939e-04*Jinv11*w1+8.3198051948051951e-05*w13*Jinv11-4.2613636363636367e-05*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-4.2613636363636367e-05*Jinv01*w39+1.0247564935064935e-04*Jinv21*w38+1.5726461038961039e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11-1.1668019480519480e-04*w19*Jinv11+1.0247564935064935e-04*Jinv21*w53+6.4935064935064935e-05*w4*Jinv11+6.4935064935064935e-05*w32*Jinv21-1.1668019480519480e-04*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35+6.4935064935064935e-05*w31*Jinv11+1.5726461038961039e-04*w46*Jinv11+6.4935064935064935e-05*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33-1.1668019480519480e-04*Jinv01*w18+1.0247564935064935e-04*w37*Jinv11-1.4204545454545455e-05*w6*Jinv01+1.5726461038961039e-04*w47*Jinv21+1.0247564935064935e-04*w52*Jinv11+2.0495129870129870e-04*w22*Jinv11+2.0495129870129870e-04*w21*Jinv01+1.0247564935064935e-04*Jinv01*w51+1.0247564935064935e-04*w36*Jinv01-1.4204545454545455e-05*w7*Jinv11-1.1668019480519480e-04*Jinv21*w56-1.4204545454545455e-05*w49*Jinv11+8.3198051948051951e-05*Jinv21*w14+8.3198051948051951e-05*Jinv01*w12-1.4204545454545455e-05*Jinv21*w8+2.0495129870129870e-04*Jinv21*w23-1.1668019480519480e-04*Jinv01*w54-1.1668019480519480e-04*w55*Jinv11+5.4112554112554113e-05*Jinv01*w9-4.2613636363636367e-05*w40*Jinv11+6.4935064935064935e-05*Jinv01*w30+1.5726461038961039e-04*w25*Jinv11-1.4204545454545455e-05*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27-1.4204545454545455e-05*w48*Jinv01+5.4112554112554113e-05*w10*Jinv11)*D;
  A[1][29]=(0.0)*D;
  A[1][30]=(0.0)*D;
  A[1][31]=(4.5657467532467534e-05*Jinv00*w12+9.2329545454545453e-05*w28*Jinv10+2.1915584415584417e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8+1.2175324675324675e-04*w57*Jinv02+4.5657467532467534e-05*w13*Jinv10+2.1915584415584417e-04*Jinv02*w39+6.7573051948051951e-04*Jinv21*w44+4.5657467532467534e-05*w3*Jinv00+9.1314935064935068e-05*w32*Jinv20+4.5657467532467534e-05*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23-1.1870941558441559e-04*Jinv22*w38+2.1002435064935065e-04*Jinv00*w54+5.6818181818181818e-05*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12+1.2987012987012987e-03*w0*Jinv02-2.8409090909090908e-04*Jinv21*w2+6.7573051948051951e-04*w43*Jinv10+6.7573051948051951e-04*w42*Jinv01+9.2329545454545453e-05*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.2784090909090910e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55+5.6818181818181818e-05*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12+1.4610389610389610e-04*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35+5.6818181818181818e-05*w58*Jinv11+5.6818181818181818e-05*w57*Jinv01-6.3920454545454550e-05*Jinv21*w26-2.8409090909090908e-04*w0*Jinv01-2.8409090909090908e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+6.7573051948051951e-04*w43*Jinv11-1.2784090909090910e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41+1.8262987012987014e-04*w42*Jinv02+9.2329545454545453e-05*Jinv00*w27+1.5726461038961039e-04*w11*Jinv22-2.8409090909090908e-04*Jinv11*w1+4.5657467532467534e-05*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+9.2329545454545453e-05*w28*Jinv11+1.4610389610389610e-04*Jinv20*w50+9.2329545454545453e-05*Jinv20*w29-1.2784090909090910e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33-6.6659902597402602e-04*w22*Jinv12+9.1314935064935068e-05*w31*Jinv10+1.1414366883116883e-03*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+1.1870941558441558e-03*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39+1.2987012987012987e-03*Jinv22*w2-6.3920454545454550e-05*Jinv00*w18-1.2784090909090910e-04*w36*Jinv00-1.2784090909090910e-04*Jinv21*w38-1.1870941558441559e-04*Jinv02*w45-6.3920454545454550e-05*Jinv01*w24+1.2175324675324675e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11-6.3920454545454550e-05*w19*Jinv11+2.1002435064935065e-04*Jinv21*w53+2.1002435064935065e-04*w51*Jinv00+2.1915584415584417e-04*Jinv12*w34+1.8262987012987014e-04*Jinv22*w44+4.5657467532467534e-05*w4*Jinv10+4.5657467532467534e-05*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+1.1414366883116883e-03*Jinv12*w13-6.3920454545454550e-05*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10+9.1314935064935068e-05*w32*Jinv21-6.3920454545454550e-05*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00-1.1870941558441559e-04*w37*Jinv12+2.1002435064935065e-04*Jinv20*w56+2.1915584415584417e-04*Jinv22*w41-6.3920454545454550e-05*w26*Jinv20+9.2329545454545453e-05*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+9.1314935064935068e-05*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26-1.2784090909090910e-04*w46*Jinv11+4.5657467532467534e-05*Jinv20*w14+4.5657467532467534e-05*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54+1.1414366883116883e-03*w4*Jinv12-6.6659902597402602e-04*w21*Jinv02+1.4610389610389610e-04*w49*Jinv10+9.2329545454545453e-05*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+1.1414366883116883e-03*Jinv02*w12-6.3920454545454550e-05*Jinv01*w18-1.2784090909090910e-04*w37*Jinv11+1.5726461038961039e-04*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10-1.2784090909090910e-04*w47*Jinv21-1.2784090909090910e-04*Jinv20*w38-6.6659902597402602e-04*Jinv22*w23+2.1002435064935065e-04*w52*Jinv11+1.1870941558441558e-03*w31*Jinv12-1.1870941558441559e-04*w46*Jinv12+2.1002435064935065e-04*Jinv20*w53-6.6659902597402602e-04*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10-4.0178571428571428e-04*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11-6.6659902597402602e-04*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01+2.1002435064935065e-04*Jinv01*w51+1.1870941558441558e-03*Jinv02*w30-4.0178571428571428e-04*Jinv12*w49-1.2784090909090910e-04*w36*Jinv01+2.1002435064935065e-04*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+9.2329545454545453e-05*w9*Jinv00-1.2784090909090910e-04*w37*Jinv10+2.1002435064935065e-04*Jinv21*w56-6.3920454545454550e-05*w24*Jinv00+1.5726461038961039e-04*Jinv02*w27-6.6659902597402602e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+1.1414366883116883e-03*Jinv22*w5-1.1870941558441559e-04*w47*Jinv22+1.4610389610389610e-04*w49*Jinv11+4.5657467532467534e-05*Jinv21*w14+9.2329545454545453e-05*w10*Jinv10+6.7573051948051951e-04*Jinv20*w44+1.2175324675324675e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+4.5657467532467534e-05*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8+5.6818181818181818e-05*w57*Jinv00+5.6818181818181818e-05*Jinv20*w59+2.1915584415584417e-04*Jinv02*w33-2.8409090909090908e-04*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23+2.1002435064935065e-04*Jinv01*w54+2.1002435064935065e-04*w55*Jinv11-6.3920454545454550e-05*w25*Jinv10-2.8409090909090908e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-4.0178571428571428e-04*w48*Jinv02+9.2329545454545453e-05*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17+9.1314935064935068e-05*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-6.3920454545454550e-05*w20*Jinv20+1.5726461038961039e-04*Jinv22*w29+1.1414366883116883e-03*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11-1.1870941558441559e-04*Jinv02*w36+1.8262987012987014e-04*Jinv12*w43+4.5657467532467534e-05*w5*Jinv20+9.1314935064935068e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-6.3920454545454550e-05*w25*Jinv11+2.1002435064935065e-04*w55*Jinv10+6.7573051948051951e-04*w42*Jinv00-1.2784090909090910e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12+1.4610389610389610e-04*Jinv21*w50+9.2329545454545453e-05*Jinv01*w27+1.4610389610389610e-04*w48*Jinv01+1.2987012987012987e-03*Jinv12*w1+9.2329545454545453e-05*w10*Jinv11)*D;
  A[1][32]=(0.0)*D;
  A[1][33]=(0.0)*D;
  A[1][34]=(-5.4788961038961042e-05*Jinv00*w12-1.9480519480519481e-04*w28*Jinv10-9.1314935064935063e-04*w40*Jinv12-6.2094155844155847e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-5.4788961038961042e-05*w13*Jinv10-9.1314935064935063e-04*Jinv02*w39-2.7759740259740259e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20+9.8620129870129864e-04*Jinv01*w3+2.5568181818181820e-04*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54-3.0438311688311688e-05*Jinv21*w59+3.4699675324675324e-04*w25*Jinv12+2.4350649350649351e-04*w0*Jinv02-4.8701298701298701e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-2.7759740259740259e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17+4.2004870129870131e-04*Jinv01*w45-3.1047077922077924e-04*Jinv12*w55-4.2613636363636367e-05*w58*Jinv10-6.9399350649350649e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+3.4699675324675324e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01+4.2004870129870131e-04*Jinv21*w26-4.8701298701298701e-04*w0*Jinv01+2.4350649350649351e-04*w1*Jinv10-9.1314935064935063e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20-2.7759740259740259e-03*w43*Jinv11+3.4699675324675324e-04*w45*Jinv00-1.1323051948051949e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-1.9480519480519481e-04*Jinv00*w27-4.2613636363636367e-05*w11*Jinv22-4.8701298701298701e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-2.2524350649350650e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50-1.9480519480519481e-04*Jinv20*w29+3.4699675324675324e-04*w46*Jinv10-6.9399350649350649e-04*Jinv22*w17-1.8262987012987013e-03*Jinv00*w33+2.5568181818181820e-04*w22*Jinv12-1.0957792207792208e-04*w31*Jinv10-1.0957792207792208e-04*Jinv02*w3+2.5568181818181820e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39+2.4350649350649351e-04*Jinv22*w2-3.1047077922077924e-04*Jinv00*w18-6.2094155844155847e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+4.2004870129870131e-04*Jinv01*w24-2.2646103896103898e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.8262987012987013e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10+9.8620129870129864e-04*w4*Jinv11-6.9399350649350649e-04*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13-3.1047077922077924e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+9.8620129870129864e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-9.1314935064935063e-04*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-6.2094155844155847e-04*w52*Jinv12-2.2646103896103898e-03*Jinv21*w35+9.8620129870129864e-04*w31*Jinv11+3.4699675324675324e-04*Jinv22*w26+4.2004870129870131e-04*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14+9.8620129870129864e-04*w5*Jinv21-3.1047077922077924e-04*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12+2.5568181818181820e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-2.2524350649350650e-04*Jinv21*w29-2.2646103896103898e-03*Jinv01*w33-6.9399350649350649e-04*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-4.2613636363636367e-05*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+4.2004870129870131e-04*w47*Jinv21-6.2094155844155847e-04*Jinv20*w38+2.5568181818181820e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+2.5568181818181820e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-6.2094155844155847e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00-1.9480519480519481e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14-1.6436688311688311e-03*Jinv20*w44+2.1915584415584417e-04*Jinv02*w18+4.9310064935064932e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-1.8262987012987013e-03*Jinv02*w33+2.4350649350649351e-04*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20+3.6525974025974026e-05*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10+2.4350649350649351e-04*Jinv20*w2-9.1314935064935063e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9-9.1314935064935063e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-6.9399350649350649e-04*w40*Jinv10-3.1047077922077924e-04*w20*Jinv20-1.9480519480519481e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20+9.8620129870129864e-04*Jinv01*w30-6.2094155844155847e-04*Jinv02*w51-3.1047077922077924e-04*Jinv22*w56-1.9480519480519481e-04*Jinv12*w28+4.2004870129870131e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+3.4699675324675324e-04*w47*Jinv20-6.9399350649350649e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-2.2524350649350650e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+2.4350649350649351e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[1][35]=(0.0)*D;
  A[1][36]=(0.0)*D;
  A[1][37]=(9.1314935064935065e-06*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.2918019480519480e-04*Jinv22*w53+3.9265422077922079e-04*Jinv20*w8+1.0247564935064935e-04*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-4.8396915584415583e-04*w3*Jinv00+1.8262987012987013e-05*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23+7.4878246753246752e-04*Jinv22*w38-4.2918019480519480e-04*Jinv00*w54-4.2918019480519480e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+9.6793831168831166e-04*w43*Jinv10-1.0957792207792208e-04*Jinv12*w55-1.4204545454545455e-05*w58*Jinv10+4.9310064935064932e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-3.1047077922077924e-04*w48*Jinv00-4.2918019480519480e-04*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35+4.0584415584415584e-05*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.4878246753246752e-04*Jinv22*w20-2.1915584415584417e-04*w45*Jinv00+2.7394480519480518e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-3.1047077922077924e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+4.9310064935064932e-04*Jinv22*w17+1.0592532467532468e-03*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12+1.8262987012987013e-05*w31*Jinv10-4.7483766233766234e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00+9.1314935064935065e-06*w32*Jinv22+4.0584415584415584e-05*Jinv22*w2+3.7439123376623376e-04*Jinv00*w18+7.4878246753246752e-04*w36*Jinv00-1.0957792207792208e-04*Jinv02*w45+1.0247564935064935e-04*Jinv22*w59-1.5523538961038962e-04*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+2.7394480519480518e-04*Jinv22*w44-4.8396915584415583e-04*w4*Jinv10-7.3051948051948052e-05*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13+3.7439123376623376e-04*w19*Jinv10+1.0592532467532468e-03*w34*Jinv10+3.9265422077922079e-04*w6*Jinv00+7.4878246753246752e-04*w37*Jinv12-4.2918019480519480e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-3.1960227272727273e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20-4.2918019480519480e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14-1.0957792207792208e-04*Jinv02*w54-4.7483766233766234e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02-3.1047077922077924e-04*w49*Jinv10-7.3051948051948052e-05*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.9265422077922079e-04*w7*Jinv10+7.4878246753246752e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23+9.1314935064935065e-06*w31*Jinv12-1.0957792207792208e-04*w46*Jinv12-1.5523538961038962e-04*Jinv20*w53+7.8530844155844158e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50+7.8530844155844158e-04*Jinv12*w7+9.1314935064935065e-06*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-1.5523538961038962e-04*w52*Jinv10-9.2329545454545453e-05*w9*Jinv00+7.4878246753246752e-04*w37*Jinv10-3.1960227272727273e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27+7.8530844155844158e-04*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-4.7483766233766234e-04*Jinv22*w5-1.0957792207792208e-04*w47*Jinv22-9.2329545454545453e-05*w10*Jinv10+9.6793831168831166e-04*Jinv20*w44+1.0247564935064935e-04*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33+4.0584415584415584e-05*w0*Jinv00+1.0592532467532468e-03*w35*Jinv20-3.1960227272727273e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17+1.8262987012987013e-05*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+3.7439123376623376e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14+7.4878246753246752e-04*Jinv02*w36+2.7394480519480518e-04*Jinv12*w43-4.8396915584415583e-04*w5*Jinv20-4.2918019480519480e-04*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28-4.2918019480519480e-04*w55*Jinv10+9.6793831168831166e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20+4.9310064935064932e-04*w16*Jinv12+4.0584415584415584e-05*Jinv12*w1)*D;
  A[1][38]=(0.0)*D;
  A[1][39]=(0.0)*D;
  A[1][40]=(9.8620129870129864e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10-1.8262987012987013e-03*w40*Jinv12-3.1047077922077924e-04*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8+9.8620129870129864e-04*w13*Jinv10-1.8262987012987013e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+4.9310064935064932e-04*w3*Jinv00+9.8620129870129864e-04*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-4.2613636363636367e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12+2.4350649350649351e-04*w0*Jinv02+2.4350649350649351e-04*Jinv21*w2-2.7759740259740259e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-1.9480519480519481e-04*w11*Jinv21-9.1314935064935063e-04*w16*Jinv11-9.1314935064935063e-04*Jinv21*w17-6.2094155844155847e-04*Jinv01*w45-6.2094155844155847e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10-6.9399350649350649e-04*w15*Jinv02-1.9480519480519481e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-9.1314935064935063e-04*Jinv22*w35-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01-3.1047077922077924e-04*Jinv21*w26+2.4350649350649351e-04*w0*Jinv01-4.8701298701298701e-04*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10+3.4699675324675324e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-1.8262987012987013e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-1.9480519480519481e-04*w11*Jinv22+2.4350649350649351e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-9.1314935064935063e-04*w15*Jinv01-4.9310064935064932e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-6.9399350649350649e-04*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+9.8620129870129864e-04*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.8262987012987013e-03*Jinv01*w39+2.4350649350649351e-04*Jinv22*w2+4.2004870129870131e-04*Jinv00*w18+4.2004870129870131e-04*w36*Jinv00+3.4699675324675324e-04*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45-3.1047077922077924e-04*Jinv01*w24-6.9399350649350649e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00-9.1314935064935063e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+4.9310064935064932e-04*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11-2.2646103896103898e-03*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13+4.2004870129870131e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-1.8262987012987013e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20-2.2524350649350650e-04*w11*Jinv20-3.1047077922077924e-04*w52*Jinv12-6.9399350649350649e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26-6.2094155844155847e-04*w46*Jinv11+9.8620129870129864e-04*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21-6.2094155844155847e-04*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-6.9399350649350649e-04*Jinv01*w33-2.2646103896103898e-03*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+3.4699675324675324e-04*w37*Jinv11-1.9480519480519481e-04*Jinv02*w9+2.5568181818181820e-04*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10-6.2094155844155847e-04*w47*Jinv21+4.2004870129870131e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+2.5568181818181820e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+2.5568181818181820e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.4699675324675324e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+2.5568181818181820e-04*w7*Jinv11-2.2524350649350650e-04*w9*Jinv00+4.2004870129870131e-04*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-4.2613636363636367e-05*Jinv02*w27+2.5568181818181820e-04*w6*Jinv02+3.4699675324675324e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-2.2524350649350650e-04*w10*Jinv10-2.7759740259740259e-03*Jinv20*w44+3.4699675324675324e-04*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+2.5568181818181820e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59-9.1314935064935063e-04*Jinv02*w33-4.8701298701298701e-04*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-4.8701298701298701e-04*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.9480519480519481e-04*Jinv01*w9-1.1323051948051949e-03*Jinv20*w17+9.8620129870129864e-04*w30*Jinv00-2.2646103896103898e-03*w40*Jinv10+4.2004870129870131e-04*w20*Jinv20-4.2613636363636367e-05*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-1.8262987012987013e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+4.9310064935064932e-04*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30-3.1047077922077924e-04*Jinv02*w51-6.2094155844155847e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-3.1047077922077924e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-2.7759740259740259e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-6.9399350649350649e-04*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.6436688311688311e-04*w48*Jinv01+2.4350649350649351e-04*Jinv12*w1-1.9480519480519481e-04*w10*Jinv11)*D;
  A[1][41]=(0.0)*D;
  A[1][42]=(0.0)*D;
  A[1][43]=(-9.8620129870129864e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12-4.2004870129870131e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+2.2524350649350650e-04*w57*Jinv02-9.8620129870129864e-04*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+2.7759740259740259e-03*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00-9.8620129870129864e-04*w32*Jinv20-9.8620129870129864e-04*Jinv01*w3+4.9310064935064932e-04*Jinv20*w23+2.7394480519480518e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+3.0438311688311688e-05*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+4.8701298701298701e-04*w0*Jinv02+4.8701298701298701e-04*Jinv21*w2+2.7759740259740259e-03*w43*Jinv10+2.7759740259740259e-03*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17-4.2004870129870131e-04*Jinv01*w45-4.2004870129870131e-04*Jinv12*w55+3.0438311688311688e-05*w58*Jinv10+2.2646103896103898e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12+4.9310064935064932e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-4.2004870129870131e-04*Jinv21*w26+4.8701298701298701e-04*w0*Jinv01+4.8701298701298701e-04*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+2.7759740259740259e-03*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+2.7759740259740259e-03*w42*Jinv02+3.0438311688311688e-05*Jinv00*w27+3.0438311688311688e-05*w11*Jinv22+4.8701298701298701e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+2.2524350649350650e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10+2.2646103896103898e-03*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33+4.9310064935064932e-04*w22*Jinv12-9.8620129870129864e-04*w31*Jinv10-9.8620129870129864e-04*Jinv02*w3+4.9310064935064932e-04*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+4.8701298701298701e-04*Jinv22*w2-4.2004870129870131e-04*Jinv00*w18-4.2004870129870131e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+2.7394480519480518e-04*Jinv02*w45-4.2004870129870131e-04*Jinv01*w24+2.2524350649350650e-04*Jinv22*w59+2.2646103896103898e-03*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+2.7759740259740259e-03*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-9.8620129870129864e-04*w4*Jinv11+2.2646103896103898e-03*w41*Jinv20-9.8620129870129864e-04*Jinv12*w13-4.2004870129870131e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-9.8620129870129864e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+2.7394480519480518e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20+2.2524350649350650e-04*w11*Jinv20-4.2004870129870131e-04*w52*Jinv12+2.2646103896103898e-03*Jinv21*w35-9.8620129870129864e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26-4.2004870129870131e-04*w46*Jinv11-9.8620129870129864e-04*Jinv20*w14-9.8620129870129864e-04*w5*Jinv21-4.2004870129870131e-04*Jinv02*w54-9.8620129870129864e-04*w4*Jinv12+4.9310064935064932e-04*w21*Jinv02+4.9310064935064932e-04*w49*Jinv10+2.2524350649350650e-04*Jinv21*w29+2.2646103896103898e-03*Jinv01*w33+2.2646103896103898e-03*w39*Jinv00-9.8620129870129864e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+3.0438311688311688e-05*Jinv02*w9+4.9310064935064932e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10-4.2004870129870131e-04*w47*Jinv21-4.2004870129870131e-04*Jinv20*w38+4.9310064935064932e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+2.7394480519480518e-04*Jinv20*w53+4.9310064935064932e-04*Jinv22*w8+4.9310064935064932e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+4.9310064935064932e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+4.9310064935064932e-04*w7*Jinv11+2.2524350649350650e-04*w9*Jinv00-4.2004870129870131e-04*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+2.7394480519480518e-04*w24*Jinv00+3.0438311688311688e-05*Jinv02*w27+4.9310064935064932e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-9.8620129870129864e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+4.9310064935064932e-04*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14+2.2524350649350650e-04*w10*Jinv10+2.7759740259740259e-03*Jinv20*w44+2.2524350649350650e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+4.9310064935064932e-04*Jinv21*w8+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+4.8701298701298701e-04*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+2.7394480519480518e-04*w25*Jinv10+4.8701298701298701e-04*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+3.0438311688311688e-05*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17-9.8620129870129864e-04*w30*Jinv00+2.2646103896103898e-03*w40*Jinv10-4.2004870129870131e-04*w20*Jinv20+3.0438311688311688e-05*Jinv22*w29-9.8620129870129864e-04*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36+2.7759740259740259e-03*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-9.8620129870129864e-04*Jinv01*w30-4.2004870129870131e-04*Jinv02*w51-4.2004870129870131e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28-4.2004870129870131e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+2.7759740259740259e-03*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+2.2646103896103898e-03*w16*Jinv12+4.9310064935064932e-04*Jinv21*w50+2.2524350649350650e-04*Jinv01*w27+4.9310064935064932e-04*w48*Jinv01+4.8701298701298701e-04*Jinv12*w1+3.0438311688311688e-05*w10*Jinv11)*D;
  A[1][44]=(0.0)*D;
  A[1][45]=(0.0)*D;
  A[1][46]=(4.9310064935064932e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+1.0247564935064935e-04*w57*Jinv02+4.9310064935064932e-04*Jinv02*w39+9.6793831168831166e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3-1.0957792207792208e-04*Jinv22*w38-1.4204545454545455e-05*Jinv21*w59+7.4878246753246752e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+4.0584415584415584e-05*Jinv21*w2+9.6793831168831166e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+7.4878246753246752e-04*Jinv01*w45-4.2918019480519480e-04*Jinv12*w55+4.9310064935064932e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+3.7439123376623376e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01-4.2918019480519480e-04*Jinv22*w20+9.6793831168831166e-04*w43*Jinv11+1.0592532467532468e-03*Jinv21*w41+2.7394480519480518e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+4.0584415584415584e-05*Jinv11*w1-4.8396915584415583e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+4.9310064935064932e-04*Jinv22*w17+7.8530844155844158e-04*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3+9.1314935064935065e-06*w32*Jinv22+1.0592532467532468e-03*Jinv01*w39+4.0584415584415584e-05*Jinv22*w2-2.1915584415584417e-04*Jinv21*w38+7.4878246753246752e-04*Jinv02*w45+3.7439123376623376e-04*Jinv01*w24+1.0247564935064935e-04*Jinv22*w59-7.3051948051948052e-05*w34*Jinv11-3.1960227272727273e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+2.7394480519480518e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11-4.7483766233766234e-04*Jinv12*w13+1.8262987012987013e-05*w32*Jinv21-3.1960227272727273e-04*Jinv21*w20-1.0957792207792208e-04*w37*Jinv12+4.9310064935064932e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12-7.3051948051948052e-05*Jinv21*w35+1.8262987012987013e-05*w31*Jinv11+7.4878246753246752e-04*Jinv22*w26+7.4878246753246752e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21-4.2918019480519480e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12+7.8530844155844158e-04*w21*Jinv02-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33-4.7483766233766234e-04*Jinv02*w12-3.1960227272727273e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01+7.4878246753246752e-04*w47*Jinv21+7.8530844155844158e-04*Jinv22*w23-4.2918019480519480e-04*w52*Jinv11+9.1314935064935065e-06*w31*Jinv12+7.4878246753246752e-04*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8-1.3697240259740259e-04*Jinv22*w50+3.9265422077922079e-04*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+3.9265422077922079e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+9.1314935064935065e-06*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11-1.5523538961038962e-04*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5+7.4878246753246752e-04*w47*Jinv22-3.1047077922077924e-04*w49*Jinv11-4.8396915584415583e-04*Jinv21*w14+1.0247564935064935e-04*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18-4.8396915584415583e-04*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.9265422077922079e-04*Jinv21*w23-1.5523538961038962e-04*Jinv01*w54-1.5523538961038962e-04*w55*Jinv11-1.3697240259740259e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.7483766233766234e-04*Jinv22*w14+1.0592532467532468e-03*w40*Jinv11-1.0957792207792208e-04*Jinv02*w36+2.7394480519480518e-04*Jinv12*w43+1.8262987012987013e-05*Jinv01*w30-1.0957792207792208e-04*Jinv02*w51-4.2918019480519480e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.7439123376623376e-04*w25*Jinv11+4.9310064935064932e-04*w16*Jinv12-3.1047077922077924e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-3.1047077922077924e-04*w48*Jinv01+4.0584415584415584e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[1][47]=(0.0)*D;
  A[1][48]=(0.0)*D;
  A[1][49]=(-9.1314935064935065e-06*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+1.3697240259740259e-04*Jinv20*w8-1.2175324675324675e-04*w57*Jinv02-9.1314935064935065e-06*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39-2.7394480519480518e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00+4.7483766233766234e-04*w32*Jinv20-9.1314935064935065e-06*Jinv01*w3+1.3697240259740259e-04*Jinv20*w23+2.5568181818181820e-04*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-2.7394480519480521e-05*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-4.0584415584415584e-05*Jinv21*w2-2.7394480519480518e-04*w43*Jinv10-2.7394480519480518e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10+2.5568181818181820e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12-7.8530844155844158e-04*w48*Jinv00-2.7394480519480521e-05*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-4.0584415584415584e-05*w0*Jinv01-4.0584415584415584e-05*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-2.7394480519480521e-05*Jinv22*w20-2.7394480519480518e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41+2.5568181818181820e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-1.1668019480519480e-04*w11*Jinv22-4.0584415584415584e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-7.8530844155844158e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10+2.5568181818181820e-04*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+4.7483766233766234e-04*w31*Jinv10-5.5702110389610384e-04*Jinv02*w3+1.3697240259740259e-04*w21*Jinv00-6.2094155844155847e-04*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38+2.5568181818181820e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-1.2175324675324675e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34+2.5568181818181820e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-9.1314935064935065e-06*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20-5.5702110389610384e-04*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+4.7483766233766234e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+1.3697240259740259e-04*w6*Jinv00+2.5568181818181820e-04*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+4.7483766233766234e-04*w31*Jinv11-2.7394480519480521e-05*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11-9.1314935064935065e-06*Jinv20*w14-9.1314935064935065e-06*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54-5.5702110389610384e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-7.8530844155844158e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00-5.5702110389610384e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11-1.1668019480519480e-04*Jinv02*w9+1.3697240259740259e-04*w6*Jinv01+1.3697240259740259e-04*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11-6.2094155844155847e-04*w31*Jinv12+2.5568181818181820e-04*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+1.3697240259740259e-04*w22*Jinv10-1.2784090909090910e-04*Jinv22*w50+1.3697240259740259e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7+1.3697240259740259e-04*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51-6.2094155844155847e-04*Jinv02*w30-1.2784090909090910e-04*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+1.3697240259740259e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.1668019480519480e-04*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12-5.5702110389610384e-04*Jinv22*w5+2.5568181818181820e-04*w47*Jinv22-7.8530844155844158e-04*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-2.7394480519480518e-04*Jinv20*w44-1.2175324675324675e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12+1.3697240259740259e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-4.0584415584415584e-05*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+1.3697240259740259e-04*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-4.0584415584415584e-05*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-1.2784090909090910e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.7483766233766234e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20-1.1668019480519480e-04*Jinv22*w29-5.5702110389610384e-04*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+2.5568181818181820e-04*Jinv02*w36+2.5568181818181820e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20+4.7483766233766234e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-2.7394480519480518e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20+2.5568181818181820e-04*w16*Jinv12-7.8530844155844158e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-7.8530844155844158e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[1][50]=(0.0)*D;
  A[1][51]=(0.0)*D;
  A[1][52]=(9.1314935064935065e-06*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+7.4878246753246752e-04*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8-9.2329545454545453e-05*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39+9.1314935064935065e-06*w3*Jinv00-4.7483766233766234e-04*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23-1.5523538961038962e-04*Jinv22*w38+7.4878246753246752e-04*Jinv00*w54-2.1915584415584417e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+2.7394480519480518e-04*w43*Jinv10+3.7439123376623376e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10-7.3051948051948052e-05*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+7.8530844155844158e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+1.0592532467532468e-03*Jinv22*w35+4.0584415584415584e-05*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-4.2918019480519480e-04*Jinv22*w20-4.2918019480519480e-04*w45*Jinv00+9.6793831168831166e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27-1.4204545454545455e-05*w11*Jinv22+7.8530844155844158e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10-7.3051948051948052e-05*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-4.7483766233766234e-04*w31*Jinv10+1.8262987012987013e-05*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-4.8396915584415583e-04*w32*Jinv22+4.0584415584415584e-05*Jinv22*w2-1.0957792207792208e-04*Jinv00*w18-4.2918019480519480e-04*w36*Jinv00-3.1960227272727273e-04*Jinv02*w45-9.2329545454545453e-05*Jinv22*w59+7.4878246753246752e-04*w51*Jinv00+1.0592532467532468e-03*Jinv12*w34+9.6793831168831166e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv10+4.9310064935064932e-04*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13-1.0957792207792208e-04*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10-1.3697240259740259e-04*w6*Jinv00-1.5523538961038962e-04*w37*Jinv12+7.4878246753246752e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41-1.0957792207792208e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20+7.4878246753246752e-04*w52*Jinv12-2.1915584415584417e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14+3.7439123376623376e-04*Jinv02*w54+1.8262987012987013e-05*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+7.8530844155844158e-04*w49*Jinv10+4.9310064935064932e-04*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-1.3697240259740259e-04*w7*Jinv10-4.2918019480519480e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-4.8396915584415583e-04*w31*Jinv12-3.1960227272727273e-04*w46*Jinv12+7.4878246753246752e-04*Jinv20*w53-3.1047077922077924e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+3.9265422077922079e-04*Jinv22*w50-3.1047077922077924e-04*Jinv12*w7-4.8396915584415583e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49+7.4878246753246752e-04*w52*Jinv10+1.0247564935064935e-04*w9*Jinv00-4.2918019480519480e-04*w37*Jinv10-1.0957792207792208e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27-3.1047077922077924e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+1.8262987012987013e-05*Jinv22*w5-3.1960227272727273e-04*w47*Jinv22+1.0247564935064935e-04*w10*Jinv10+2.7394480519480518e-04*Jinv20*w44-9.2329545454545453e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+1.0592532467532468e-03*Jinv02*w33+4.0584415584415584e-05*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.0957792207792208e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.9265422077922079e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.7483766233766234e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14-1.5523538961038962e-04*Jinv02*w36+9.6793831168831166e-04*Jinv12*w43+9.1314935064935065e-06*w5*Jinv20+7.4878246753246752e-04*Jinv02*w51+3.7439123376623376e-04*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+7.4878246753246752e-04*w55*Jinv10+2.7394480519480518e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20-7.3051948051948052e-05*w16*Jinv12+4.0584415584415584e-05*Jinv12*w1)*D;
  A[1][53]=(0.0)*D;
  A[1][54]=(0.0)*D;
  A[1][55]=(1.0592532467532468e-03*w40*Jinv12+3.7439123376623376e-04*Jinv22*w53-9.2329545454545453e-05*w57*Jinv02+1.0592532467532468e-03*Jinv02*w39+2.7394480519480518e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3-3.1960227272727273e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-4.2918019480519480e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+4.0584415584415584e-05*Jinv21*w2+2.7394480519480518e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.2918019480519480e-04*Jinv01*w45+7.4878246753246752e-04*Jinv12*w55-7.3051948051948052e-05*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01-2.1915584415584417e-04*Jinv22*w20+2.7394480519480518e-04*w43*Jinv11+4.9310064935064932e-04*Jinv21*w41+9.6793831168831166e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+4.0584415584415584e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-7.3051948051948052e-05*Jinv22*w17-3.1047077922077924e-04*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3-4.8396915584415583e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39+4.0584415584415584e-05*Jinv22*w2-4.2918019480519480e-04*Jinv21*w38-1.5523538961038962e-04*Jinv02*w45-1.0957792207792208e-04*Jinv01*w24-9.2329545454545453e-05*Jinv22*w59+4.9310064935064932e-04*w34*Jinv11-1.0957792207792208e-04*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+9.6793831168831166e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11+1.8262987012987013e-05*Jinv12*w13-4.7483766233766234e-04*w32*Jinv21-1.0957792207792208e-04*Jinv21*w20-3.1960227272727273e-04*w37*Jinv12+1.0592532467532468e-03*Jinv22*w41+3.7439123376623376e-04*w52*Jinv12+4.9310064935064932e-04*Jinv21*w35-4.7483766233766234e-04*w31*Jinv11-4.2918019480519480e-04*Jinv22*w26-4.2918019480519480e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21+7.4878246753246752e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12-3.1047077922077924e-04*w21*Jinv02+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33+1.8262987012987013e-05*Jinv02*w12-1.0957792207792208e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01-4.2918019480519480e-04*w47*Jinv21-3.1047077922077924e-04*Jinv22*w23+7.4878246753246752e-04*w52*Jinv11-4.8396915584415583e-04*w31*Jinv12-1.5523538961038962e-04*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8+3.9265422077922079e-04*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-4.8396915584415583e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49-4.2918019480519480e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11+7.4878246753246752e-04*Jinv21*w56-1.4204545454545455e-05*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5-1.5523538961038962e-04*w47*Jinv22+7.8530844155844158e-04*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14-9.2329545454545453e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+9.1314935064935065e-06*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33-1.3697240259740259e-04*Jinv21*w23+7.4878246753246752e-04*Jinv01*w54+7.4878246753246752e-04*w55*Jinv11+3.9265422077922079e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9-1.4204545454545455e-05*Jinv22*w29+1.8262987012987013e-05*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-3.1960227272727273e-04*Jinv02*w36+9.6793831168831166e-04*Jinv12*w43-4.7483766233766234e-04*Jinv01*w30+3.7439123376623376e-04*Jinv02*w51+7.4878246753246752e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28-1.0957792207792208e-04*w25*Jinv11-7.3051948051948052e-05*w16*Jinv12+7.8530844155844158e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+7.8530844155844158e-04*w48*Jinv01+4.0584415584415584e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[1][56]=(0.0)*D;
  A[1][57]=(0.0)*D;
  A[1][58]=(-4.2613636363636367e-05*w40*Jinv12+1.5726461038961039e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-4.2613636363636367e-05*Jinv02*w39-1.1668019480519480e-04*Jinv22*w38+1.0247564935064935e-04*w25*Jinv12+1.8939393939393939e-04*w0*Jinv02+1.5726461038961039e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+5.4112554112554113e-05*w10*Jinv12+1.0247564935064935e-04*Jinv02*w24-4.2613636363636367e-05*Jinv22*w35+1.0247564935064935e-04*Jinv22*w20-1.9480519480519481e-04*w42*Jinv02+5.4112554112554113e-05*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17-1.4204545454545455e-05*w22*Jinv12+6.4935064935064935e-05*Jinv02*w3+8.3198051948051951e-05*w32*Jinv22+1.8939393939393939e-04*Jinv22*w2-1.1668019480519480e-04*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-4.2613636363636367e-05*Jinv12*w34-1.9480519480519481e-04*Jinv22*w44+6.4935064935064935e-05*Jinv12*w13-1.1668019480519480e-04*w37*Jinv12-4.2613636363636367e-05*Jinv22*w41+1.5726461038961039e-04*w52*Jinv12+1.0247564935064935e-04*Jinv22*w26+1.5726461038961039e-04*Jinv02*w54+6.4935064935064935e-05*w4*Jinv12-1.4204545454545455e-05*w21*Jinv02+6.4935064935064935e-05*Jinv02*w12+5.4112554112554113e-05*Jinv02*w9-1.4204545454545455e-05*Jinv22*w23+8.3198051948051951e-05*w31*Jinv12-1.1668019480519480e-04*w46*Jinv12-1.4204545454545455e-05*Jinv22*w8+2.0495129870129870e-04*Jinv22*w50-1.4204545454545455e-05*Jinv12*w7+8.3198051948051951e-05*Jinv02*w30+2.0495129870129870e-04*Jinv12*w49+5.4112554112554113e-05*Jinv02*w27-1.4204545454545455e-05*w6*Jinv02+1.0247564935064935e-04*w19*Jinv12+6.4935064935064935e-05*Jinv22*w5-1.1668019480519480e-04*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12+1.0247564935064935e-04*Jinv02*w18-4.2613636363636367e-05*Jinv02*w33+2.0495129870129870e-04*w48*Jinv02+5.4112554112554113e-05*Jinv22*w29+6.4935064935064935e-05*Jinv22*w14-1.1668019480519480e-04*Jinv02*w36-1.9480519480519481e-04*Jinv12*w43+1.5726461038961039e-04*Jinv02*w51+1.5726461038961039e-04*Jinv22*w56+5.4112554112554113e-05*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+1.8939393939393939e-04*Jinv12*w1)*D;
  A[1][59]=(0.0)*D;
  A[2][0]=(0.0)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(-6.4935064935064935e-04*Jinv00*w12-9.4696969696969697e-05*w28*Jinv10-1.2175324675324675e-04*w40*Jinv12-2.0292207792207792e-05*Jinv22*w53+3.2467532467532468e-04*Jinv20*w8-9.4696969696969697e-05*w57*Jinv02-6.4935064935064935e-04*w13*Jinv10-1.2175324675324675e-04*Jinv02*w39-2.4350649350649351e-04*Jinv21*w44-6.4935064935064935e-04*w3*Jinv00-6.4935064935064935e-04*w32*Jinv20-6.4935064935064935e-04*Jinv01*w3+3.2467532467532468e-04*Jinv20*w23-2.0292207792207792e-05*Jinv22*w38-2.0292207792207792e-05*Jinv00*w54-9.4696969696969697e-05*Jinv21*w59-2.0292207792207792e-05*w25*Jinv12-8.3874458874458877e-04*w0*Jinv02-8.3874458874458877e-04*Jinv21*w2-2.4350649350649351e-04*w43*Jinv10-2.4350649350649351e-04*w42*Jinv01-9.4696969696969697e-05*w11*Jinv21-1.2175324675324675e-04*w16*Jinv11-1.2175324675324675e-04*Jinv21*w17-2.0292207792207792e-05*Jinv01*w45-2.0292207792207792e-05*Jinv12*w55-9.4696969696969697e-05*w58*Jinv10-1.2175324675324675e-04*w15*Jinv02-9.4696969696969697e-05*w10*Jinv12+3.2467532467532468e-04*w48*Jinv00-2.0292207792207792e-05*Jinv02*w24-1.2175324675324675e-04*Jinv22*w35-9.4696969696969697e-05*w58*Jinv11-9.4696969696969697e-05*w57*Jinv01-2.0292207792207792e-05*Jinv21*w26-8.3874458874458877e-04*w0*Jinv01-8.3874458874458877e-04*w1*Jinv10-1.2175324675324675e-04*w16*Jinv10-2.0292207792207792e-05*Jinv22*w20-2.4350649350649351e-04*w43*Jinv11-2.0292207792207792e-05*w45*Jinv00-1.2175324675324675e-04*Jinv21*w41-2.4350649350649351e-04*w42*Jinv02-9.4696969696969697e-05*Jinv00*w27-9.4696969696969697e-05*w11*Jinv22-8.3874458874458877e-04*Jinv11*w1-6.4935064935064935e-04*w13*Jinv11-1.2175324675324675e-04*w15*Jinv01-9.4696969696969697e-05*w28*Jinv11+3.2467532467532468e-04*Jinv20*w50-9.4696969696969697e-05*Jinv20*w29-2.0292207792207792e-05*w46*Jinv10-1.2175324675324675e-04*Jinv22*w17-1.2175324675324675e-04*Jinv00*w33+3.2467532467532468e-04*w22*Jinv12-6.4935064935064935e-04*w31*Jinv10-6.4935064935064935e-04*Jinv02*w3+3.2467532467532468e-04*w21*Jinv00-6.4935064935064935e-04*w32*Jinv22-1.2175324675324675e-04*Jinv01*w39-8.3874458874458877e-04*Jinv22*w2-2.0292207792207792e-05*Jinv00*w18-2.0292207792207792e-05*w36*Jinv00-2.0292207792207792e-05*Jinv21*w38-2.0292207792207792e-05*Jinv02*w45-2.0292207792207792e-05*Jinv01*w24-9.4696969696969697e-05*Jinv22*w59-1.2175324675324675e-04*w34*Jinv11-2.0292207792207792e-05*w19*Jinv11-2.0292207792207792e-05*Jinv21*w53-2.0292207792207792e-05*w51*Jinv00-1.2175324675324675e-04*Jinv12*w34-2.4350649350649351e-04*Jinv22*w44-6.4935064935064935e-04*w4*Jinv10-6.4935064935064935e-04*w4*Jinv11-1.2175324675324675e-04*w41*Jinv20-6.4935064935064935e-04*Jinv12*w13-2.0292207792207792e-05*w19*Jinv10-1.2175324675324675e-04*w34*Jinv10-6.4935064935064935e-04*w32*Jinv21-2.0292207792207792e-05*Jinv21*w20+3.2467532467532468e-04*w6*Jinv00-2.0292207792207792e-05*w37*Jinv12-2.0292207792207792e-05*Jinv20*w56-1.2175324675324675e-04*Jinv22*w41-2.0292207792207792e-05*w26*Jinv20-9.4696969696969697e-05*w11*Jinv20-2.0292207792207792e-05*w52*Jinv12-1.2175324675324675e-04*Jinv21*w35-6.4935064935064935e-04*w31*Jinv11-2.0292207792207792e-05*Jinv22*w26-2.0292207792207792e-05*w46*Jinv11-6.4935064935064935e-04*Jinv20*w14-6.4935064935064935e-04*w5*Jinv21-2.0292207792207792e-05*Jinv02*w54-6.4935064935064935e-04*w4*Jinv12+3.2467532467532468e-04*w21*Jinv02+3.2467532467532468e-04*w49*Jinv10-9.4696969696969697e-05*Jinv21*w29-1.2175324675324675e-04*Jinv01*w33-1.2175324675324675e-04*w39*Jinv00-6.4935064935064935e-04*Jinv02*w12-2.0292207792207792e-05*Jinv01*w18-2.0292207792207792e-05*w37*Jinv11-9.4696969696969697e-05*Jinv02*w9+3.2467532467532468e-04*w6*Jinv01+3.2467532467532468e-04*w7*Jinv10-2.0292207792207792e-05*w47*Jinv21-2.0292207792207792e-05*Jinv20*w38+3.2467532467532468e-04*Jinv22*w23-2.0292207792207792e-05*w52*Jinv11-6.4935064935064935e-04*w31*Jinv12-2.0292207792207792e-05*w46*Jinv12-2.0292207792207792e-05*Jinv20*w53+3.2467532467532468e-04*Jinv22*w8+3.2467532467532468e-04*w22*Jinv10+3.2467532467532468e-04*Jinv22*w50+3.2467532467532468e-04*w22*Jinv11+3.2467532467532468e-04*Jinv12*w7+3.2467532467532468e-04*w21*Jinv01-2.0292207792207792e-05*Jinv01*w51-6.4935064935064935e-04*Jinv02*w30+3.2467532467532468e-04*Jinv12*w49-2.0292207792207792e-05*w36*Jinv01-2.0292207792207792e-05*w52*Jinv10+3.2467532467532468e-04*w7*Jinv11-9.4696969696969697e-05*w9*Jinv00-2.0292207792207792e-05*w37*Jinv10-2.0292207792207792e-05*Jinv21*w56-2.0292207792207792e-05*w24*Jinv00-9.4696969696969697e-05*Jinv02*w27+3.2467532467532468e-04*w6*Jinv02-2.0292207792207792e-05*w19*Jinv12-6.4935064935064935e-04*Jinv22*w5-2.0292207792207792e-05*w47*Jinv22+3.2467532467532468e-04*w49*Jinv11-6.4935064935064935e-04*Jinv21*w14-9.4696969696969697e-05*w10*Jinv10-2.4350649350649351e-04*Jinv20*w44-9.4696969696969697e-05*w58*Jinv12-2.0292207792207792e-05*Jinv02*w18-6.4935064935064935e-04*Jinv01*w12+3.2467532467532468e-04*Jinv21*w8-9.4696969696969697e-05*w57*Jinv00-9.4696969696969697e-05*Jinv20*w59-1.2175324675324675e-04*Jinv02*w33-8.3874458874458877e-04*w0*Jinv00-1.2175324675324675e-04*w35*Jinv20+3.2467532467532468e-04*Jinv21*w23-2.0292207792207792e-05*Jinv01*w54-2.0292207792207792e-05*w55*Jinv11-2.0292207792207792e-05*w25*Jinv10-8.3874458874458877e-04*Jinv20*w2-1.2175324675324675e-04*w15*Jinv00+3.2467532467532468e-04*w48*Jinv02-9.4696969696969697e-05*Jinv01*w9-1.2175324675324675e-04*Jinv20*w17-6.4935064935064935e-04*w30*Jinv00-1.2175324675324675e-04*w40*Jinv10-2.0292207792207792e-05*w20*Jinv20-9.4696969696969697e-05*Jinv22*w29-6.4935064935064935e-04*Jinv22*w14-1.2175324675324675e-04*w40*Jinv11-2.0292207792207792e-05*Jinv02*w36-2.4350649350649351e-04*Jinv12*w43-6.4935064935064935e-04*w5*Jinv20-6.4935064935064935e-04*Jinv01*w30-2.0292207792207792e-05*Jinv02*w51-2.0292207792207792e-05*Jinv22*w56-9.4696969696969697e-05*Jinv12*w28-2.0292207792207792e-05*w25*Jinv11-2.0292207792207792e-05*w55*Jinv10-2.4350649350649351e-04*w42*Jinv00-2.0292207792207792e-05*w47*Jinv20-1.2175324675324675e-04*w16*Jinv12+3.2467532467532468e-04*Jinv21*w50-9.4696969696969697e-05*Jinv01*w27+3.2467532467532468e-04*w48*Jinv01-8.3874458874458877e-04*Jinv12*w1-9.4696969696969697e-05*w10*Jinv11)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(0.0)*D;
  A[2][5]=(1.1414366883116883e-03*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.2784090909090910e-04*Jinv22*w53-4.0178571428571428e-04*Jinv20*w8+9.2329545454545453e-05*w57*Jinv02+1.1414366883116883e-03*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+6.7573051948051951e-04*Jinv21*w44+1.1870941558441558e-03*w3*Jinv00+1.1414366883116883e-03*w32*Jinv20+9.1314935064935068e-05*Jinv01*w3-6.6659902597402602e-04*Jinv20*w23+2.1002435064935065e-04*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+9.2329545454545453e-05*Jinv21*w59-1.2784090909090910e-04*w25*Jinv12-2.8409090909090908e-04*w0*Jinv02-2.8409090909090908e-04*Jinv21*w2+1.8262987012987014e-04*w43*Jinv10+6.7573051948051951e-04*w42*Jinv01+5.6818181818181818e-05*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17-6.3920454545454550e-05*Jinv01*w45-6.3920454545454550e-05*Jinv12*w55+1.5726461038961039e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+5.6818181818181818e-05*w10*Jinv12-6.6659902597402602e-04*w48*Jinv00-1.2784090909090910e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35+9.2329545454545453e-05*w58*Jinv11+9.2329545454545453e-05*w57*Jinv01-1.2784090909090910e-04*Jinv21*w26-2.8409090909090908e-04*w0*Jinv01+1.2987012987012987e-03*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+2.1002435064935065e-04*Jinv22*w20+6.7573051948051951e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+6.7573051948051951e-04*w42*Jinv02+1.5726461038961039e-04*Jinv00*w27+5.6818181818181818e-05*w11*Jinv22-2.8409090909090908e-04*Jinv11*w1+4.5657467532467534e-05*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01+9.2329545454545453e-05*w28*Jinv11-6.6659902597402602e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+1.1414366883116883e-03*w31*Jinv10+9.1314935064935068e-05*Jinv02*w3-6.6659902597402602e-04*w21*Jinv00+4.5657467532467534e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-2.8409090909090908e-04*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1002435064935065e-04*Jinv21*w38-6.3920454545454550e-05*Jinv02*w45-1.2784090909090910e-04*Jinv01*w24+9.2329545454545453e-05*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+2.1002435064935065e-04*w19*Jinv11-1.2784090909090910e-04*Jinv21*w53-1.1870941558441559e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+6.7573051948051951e-04*Jinv22*w44+1.1870941558441558e-03*w4*Jinv10+9.1314935064935068e-05*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20+4.5657467532467534e-05*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10+4.5657467532467534e-05*w32*Jinv21+2.1002435064935065e-04*Jinv21*w20-4.0178571428571428e-04*w6*Jinv00+2.1002435064935065e-04*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-1.1870941558441559e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20-1.2784090909090910e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+4.5657467532467534e-05*w31*Jinv11-1.2784090909090910e-04*Jinv22*w26-6.3920454545454550e-05*w46*Jinv11+1.1414366883116883e-03*Jinv20*w14+9.1314935064935068e-05*w5*Jinv21-6.3920454545454550e-05*Jinv02*w54+9.1314935064935068e-05*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02-6.6659902597402602e-04*w49*Jinv10+9.2329545454545453e-05*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00+4.5657467532467534e-05*Jinv02*w12+2.1002435064935065e-04*Jinv01*w18+2.1002435064935065e-04*w37*Jinv11+5.6818181818181818e-05*Jinv02*w9+1.4610389610389610e-04*w6*Jinv01-4.0178571428571428e-04*w7*Jinv10-6.3920454545454550e-05*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23-1.2784090909090910e-04*w52*Jinv11+4.5657467532467534e-05*w31*Jinv12-6.3920454545454550e-05*w46*Jinv12-1.1870941558441559e-04*Jinv20*w53+1.4610389610389610e-04*Jinv22*w8-6.6659902597402602e-04*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11+1.4610389610389610e-04*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01-1.2784090909090910e-04*Jinv01*w51+4.5657467532467534e-05*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49+2.1002435064935065e-04*w36*Jinv01-1.1870941558441559e-04*w52*Jinv10+1.4610389610389610e-04*w7*Jinv11+1.2175324675324675e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10-6.3920454545454550e-05*Jinv21*w56-1.1870941558441559e-04*w24*Jinv00+9.2329545454545453e-05*Jinv02*w27+1.4610389610389610e-04*w6*Jinv02+2.1002435064935065e-04*w19*Jinv12+9.1314935064935068e-05*Jinv22*w5-6.3920454545454550e-05*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+4.5657467532467534e-05*Jinv21*w14+1.2175324675324675e-04*w10*Jinv10+1.8262987012987014e-04*Jinv20*w44+9.2329545454545453e-05*w58*Jinv12+2.1002435064935065e-04*Jinv02*w18+4.5657467532467534e-05*Jinv01*w12+1.4610389610389610e-04*Jinv21*w8+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33+1.2987012987012987e-03*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23-6.3920454545454550e-05*Jinv01*w54-6.3920454545454550e-05*w55*Jinv11-1.1870941558441559e-04*w25*Jinv10+1.2987012987012987e-03*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02+5.6818181818181818e-05*Jinv01*w9+2.1915584415584417e-04*Jinv20*w17+1.1414366883116883e-03*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+9.2329545454545453e-05*Jinv22*w29+4.5657467532467534e-05*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.1002435064935065e-04*Jinv02*w36+6.7573051948051951e-04*Jinv12*w43+1.1870941558441558e-03*w5*Jinv20+4.5657467532467534e-05*Jinv01*w30-1.2784090909090910e-04*Jinv02*w51-6.3920454545454550e-05*Jinv22*w56+9.2329545454545453e-05*Jinv12*w28-1.2784090909090910e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10+1.8262987012987014e-04*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50+9.2329545454545453e-05*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.8409090909090908e-04*Jinv12*w1+5.6818181818181818e-05*w10*Jinv11)*D;
  A[2][6]=(0.0)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(-5.5702110389610384e-04*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10+3.6525974025974026e-05*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-1.2784090909090910e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02-5.5702110389610384e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-2.7394480519480518e-04*Jinv21*w44-6.2094155844155847e-04*w3*Jinv00-5.5702110389610384e-04*w32*Jinv20+4.7483766233766234e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38-2.7394480519480521e-05*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-4.0584415584415584e-05*w0*Jinv02-4.0584415584415584e-05*Jinv21*w2+2.5568181818181820e-04*w43*Jinv10-2.7394480519480518e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-1.1668019480519480e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-4.0584415584415584e-05*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-2.7394480519480518e-04*w43*Jinv11-2.7394480519480521e-05*w45*Jinv00+3.6525974025974026e-05*Jinv21*w41-2.7394480519480518e-04*w42*Jinv02-1.1668019480519480e-04*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-4.0584415584415584e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33+1.3697240259740259e-04*w22*Jinv12-5.5702110389610384e-04*w31*Jinv10+4.7483766233766234e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-4.0584415584415584e-05*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+2.5568181818181820e-04*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-2.7394480519480518e-04*Jinv22*w44-6.2094155844155847e-04*w4*Jinv10+4.7483766233766234e-04*w4*Jinv11+2.5568181818181820e-04*w41*Jinv20-9.1314935064935065e-06*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-9.1314935064935065e-06*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-1.2784090909090910e-04*w6*Jinv00-7.4878246753246752e-04*w37*Jinv12-2.7394480519480521e-05*Jinv20*w56+3.6525974025974026e-05*Jinv22*w41+2.5568181818181820e-04*w26*Jinv20-1.2175324675324675e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35-9.1314935064935065e-06*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-5.5702110389610384e-04*Jinv20*w14+4.7483766233766234e-04*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54+4.7483766233766234e-04*w4*Jinv12+1.3697240259740259e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33+2.5568181818181820e-04*w39*Jinv00-9.1314935064935065e-06*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.8530844155844158e-04*w6*Jinv01-1.2784090909090910e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+1.3697240259740259e-04*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.5568181818181820e-04*Jinv20*w53-7.8530844155844158e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+1.3697240259740259e-04*Jinv22*w50+1.3697240259740259e-04*w22*Jinv11-7.8530844155844158e-04*Jinv12*w7+1.3697240259740259e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30+1.3697240259740259e-04*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+2.5568181818181820e-04*w52*Jinv10-7.8530844155844158e-04*w7*Jinv11-1.2175324675324675e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.5568181818181820e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-7.8530844155844158e-04*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+4.7483766233766234e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.3697240259740259e-04*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.2175324675324675e-04*w10*Jinv10+2.5568181818181820e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-7.8530844155844158e-04*Jinv21*w8-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59-4.9310064935064932e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20+1.3697240259740259e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.5568181818181820e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00+1.3697240259740259e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17-5.5702110389610384e-04*w30*Jinv00+2.5568181818181820e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29-9.1314935064935065e-06*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-2.7394480519480518e-04*Jinv12*w43-6.2094155844155847e-04*w5*Jinv20-9.1314935064935065e-06*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11-2.7394480519480521e-05*w55*Jinv10+2.5568181818181820e-04*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12+1.3697240259740259e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+1.3697240259740259e-04*w48*Jinv01-4.0584415584415584e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[2][9]=(0.0)*D;
  A[2][10]=(0.0)*D;
  A[2][11]=(6.4935064935064935e-05*Jinv00*w12+5.4112554112554113e-05*w28*Jinv10+2.0495129870129870e-04*Jinv20*w8+6.4935064935064935e-05*w13*Jinv10+8.3198051948051951e-05*w3*Jinv00+6.4935064935064935e-05*w32*Jinv20-1.4204545454545455e-05*Jinv20*w23+1.0247564935064935e-04*Jinv00*w54-1.9480519480519481e-04*w43*Jinv10+5.4112554112554113e-05*w58*Jinv10-1.4204545454545455e-05*w48*Jinv00+1.8939393939393939e-04*w1*Jinv10-4.2613636363636367e-05*w16*Jinv10+1.0247564935064935e-04*w45*Jinv00+5.4112554112554113e-05*Jinv00*w27-1.4204545454545455e-05*Jinv20*w50+5.4112554112554113e-05*Jinv20*w29+1.0247564935064935e-04*w46*Jinv10-4.2613636363636367e-05*Jinv00*w33+6.4935064935064935e-05*w31*Jinv10-1.4204545454545455e-05*w21*Jinv00+1.5726461038961039e-04*Jinv00*w18+1.5726461038961039e-04*w36*Jinv00-1.1668019480519480e-04*w51*Jinv00+8.3198051948051951e-05*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+1.5726461038961039e-04*w19*Jinv10-4.2613636363636367e-05*w34*Jinv10+2.0495129870129870e-04*w6*Jinv00+1.0247564935064935e-04*Jinv20*w56-1.1668019480519480e-04*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20+6.4935064935064935e-05*Jinv20*w14-1.4204545454545455e-05*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+2.0495129870129870e-04*w7*Jinv10+1.5726461038961039e-04*Jinv20*w38-1.1668019480519480e-04*Jinv20*w53-1.4204545454545455e-05*w22*Jinv10-1.1668019480519480e-04*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+1.5726461038961039e-04*w37*Jinv10-1.1668019480519480e-04*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-1.9480519480519481e-04*Jinv20*w44+5.4112554112554113e-05*w57*Jinv00+5.4112554112554113e-05*Jinv20*w59+1.8939393939393939e-04*w0*Jinv00-4.2613636363636367e-05*w35*Jinv20-1.1668019480519480e-04*w25*Jinv10+1.8939393939393939e-04*Jinv20*w2-4.2613636363636367e-05*w15*Jinv00-4.2613636363636367e-05*Jinv20*w17+6.4935064935064935e-05*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+1.5726461038961039e-04*w20*Jinv20+8.3198051948051951e-05*w5*Jinv20+1.0247564935064935e-04*w55*Jinv10-1.9480519480519481e-04*w42*Jinv00+1.0247564935064935e-04*w47*Jinv20)*D;
  A[2][12]=(0.0)*D;
  A[2][13]=(0.0)*D;
  A[2][14]=(9.1314935064935068e-05*Jinv00*w12+5.6818181818181818e-05*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12-6.3920454545454550e-05*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8+9.2329545454545453e-05*w57*Jinv02+9.1314935064935068e-05*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39+1.8262987012987014e-04*Jinv21*w44+4.5657467532467534e-05*w3*Jinv00+4.5657467532467534e-05*w32*Jinv20+1.1414366883116883e-03*Jinv01*w3+1.4610389610389610e-04*Jinv20*w23-6.3920454545454550e-05*Jinv22*w38-1.2784090909090910e-04*Jinv00*w54+1.5726461038961039e-04*Jinv21*w59+2.1002435064935065e-04*w25*Jinv12-2.8409090909090908e-04*w0*Jinv02+1.2987012987012987e-03*Jinv21*w2+6.7573051948051951e-04*w43*Jinv10+1.8262987012987014e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45-1.2784090909090910e-04*Jinv12*w55+9.2329545454545453e-05*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+9.2329545454545453e-05*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00+2.1002435064935065e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+1.2987012987012987e-03*w0*Jinv01-2.8409090909090908e-04*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10-1.2784090909090910e-04*Jinv22*w20+1.8262987012987014e-04*w43*Jinv11+2.1002435064935065e-04*w45*Jinv00+2.1915584415584417e-04*Jinv21*w41+6.7573051948051951e-04*w42*Jinv02+5.6818181818181818e-05*Jinv00*w27+9.2329545454545453e-05*w11*Jinv22+1.2987012987012987e-03*Jinv11*w1+1.1870941558441558e-03*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50+5.6818181818181818e-05*Jinv20*w29+2.1002435064935065e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+1.4610389610389610e-04*w22*Jinv12+4.5657467532467534e-05*w31*Jinv10+4.5657467532467534e-05*Jinv02*w3+1.4610389610389610e-04*w21*Jinv00+4.5657467532467534e-05*w32*Jinv22+2.1915584415584417e-04*Jinv01*w39-2.8409090909090908e-04*Jinv22*w2-1.2784090909090910e-04*Jinv00*w18-6.3920454545454550e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+2.1002435064935065e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24+9.2329545454545453e-05*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11-1.1870941558441559e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-6.3920454545454550e-05*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+6.7573051948051951e-04*Jinv22*w44+4.5657467532467534e-05*w4*Jinv10+1.1414366883116883e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+9.1314935064935068e-05*Jinv12*w13-1.2784090909090910e-04*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10+1.1414366883116883e-03*w32*Jinv21-1.1870941558441559e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00-6.3920454545454550e-05*w37*Jinv12-1.2784090909090910e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+2.1002435064935065e-04*w26*Jinv20+9.2329545454545453e-05*w11*Jinv20-6.3920454545454550e-05*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35+1.1414366883116883e-03*w31*Jinv11+2.1002435064935065e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11+9.1314935064935068e-05*Jinv20*w14+1.1414366883116883e-03*w5*Jinv21-1.2784090909090910e-04*Jinv02*w54+4.5657467532467534e-05*w4*Jinv12+1.4610389610389610e-04*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10+1.2175324675324675e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+9.1314935064935068e-05*Jinv02*w12-1.1870941558441559e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+9.2329545454545453e-05*Jinv02*w9-6.6659902597402602e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-6.3920454545454550e-05*Jinv20*w38+1.4610389610389610e-04*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+4.5657467532467534e-05*w31*Jinv12+2.1002435064935065e-04*w46*Jinv12-6.3920454545454550e-05*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8+1.4610389610389610e-04*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-4.0178571428571428e-04*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7-4.0178571428571428e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+4.5657467532467534e-05*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-6.3920454545454550e-05*w52*Jinv10-6.6659902597402602e-04*w7*Jinv11+9.2329545454545453e-05*w9*Jinv00-6.3920454545454550e-05*w37*Jinv10-1.1870941558441559e-04*Jinv21*w56+2.1002435064935065e-04*w24*Jinv00+5.6818181818181818e-05*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02-1.2784090909090910e-04*w19*Jinv12+4.5657467532467534e-05*Jinv22*w5+2.1002435064935065e-04*w47*Jinv22-6.6659902597402602e-04*w49*Jinv11+1.1870941558441558e-03*Jinv21*w14+9.2329545454545453e-05*w10*Jinv10+6.7573051948051951e-04*Jinv20*w44+9.2329545454545453e-05*w58*Jinv12-1.2784090909090910e-04*Jinv02*w18+1.1870941558441558e-03*Jinv01*w12-6.6659902597402602e-04*Jinv21*w8+9.2329545454545453e-05*w57*Jinv00+9.2329545454545453e-05*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33-2.8409090909090908e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-4.0178571428571428e-04*Jinv21*w23-1.1870941558441559e-04*Jinv01*w54-1.1870941558441559e-04*w55*Jinv11+2.1002435064935065e-04*w25*Jinv10-2.8409090909090908e-04*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02+1.5726461038961039e-04*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17+4.5657467532467534e-05*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-1.2784090909090910e-04*w20*Jinv20+5.6818181818181818e-05*Jinv22*w29+9.1314935064935068e-05*Jinv22*w14+2.1915584415584417e-04*w40*Jinv11-6.3920454545454550e-05*Jinv02*w36+6.7573051948051951e-04*Jinv12*w43+4.5657467532467534e-05*w5*Jinv20+1.1414366883116883e-03*Jinv01*w30-6.3920454545454550e-05*Jinv02*w51-1.2784090909090910e-04*Jinv22*w56+5.6818181818181818e-05*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11-1.2784090909090910e-04*w55*Jinv10+6.7573051948051951e-04*w42*Jinv00+2.1002435064935065e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-6.6659902597402602e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27-6.6659902597402602e-04*w48*Jinv01-2.8409090909090908e-04*Jinv12*w1+1.5726461038961039e-04*w10*Jinv11)*D;
  A[2][15]=(0.0)*D;
  A[2][16]=(0.0)*D;
  A[2][17]=(-1.0957792207792208e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10-1.1323051948051949e-03*w40*Jinv12+4.2004870129870131e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-2.2524350649350650e-04*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.0957792207792208e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+3.4699675324675324e-04*Jinv00*w54-1.9480519480519481e-04*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-4.8701298701298701e-04*w0*Jinv02+2.4350649350649351e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17-3.1047077922077924e-04*Jinv01*w45+4.2004870129870131e-04*Jinv12*w55-1.9480519480519481e-04*w58*Jinv10-2.2646103896103898e-03*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12+2.5568181818181820e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35-1.9480519480519481e-04*w58*Jinv11-1.9480519480519481e-04*w57*Jinv01-6.2094155844155847e-04*Jinv21*w26+2.4350649350649351e-04*w0*Jinv01+2.4350649350649351e-04*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-9.1314935064935063e-04*Jinv21*w41-2.7759740259740259e-03*w42*Jinv02-4.2613636363636367e-05*Jinv00*w27-3.0438311688311688e-05*w11*Jinv22+2.4350649350649351e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+2.5568181818181820e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-2.2646103896103898e-03*Jinv22*w17-9.1314935064935063e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10+9.8620129870129864e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+4.9310064935064932e-04*w32*Jinv22-9.1314935064935063e-04*Jinv01*w39-4.8701298701298701e-04*Jinv22*w2-6.2094155844155847e-04*Jinv00*w18-3.1047077922077924e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-6.2094155844155847e-04*Jinv01*w24-2.2524350649350650e-04*Jinv22*w59-6.9399350649350649e-04*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+3.4699675324675324e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-2.7759740259740259e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10-1.0957792207792208e-04*w4*Jinv11-6.9399350649350649e-04*w41*Jinv20+9.8620129870129864e-04*Jinv12*w13-6.2094155844155847e-04*w19*Jinv10-9.1314935064935063e-04*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+3.4699675324675324e-04*Jinv20*w56-1.1323051948051949e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20+4.2004870129870131e-04*w52*Jinv12-6.9399350649350649e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-3.1047077922077924e-04*w46*Jinv11-1.0957792207792208e-04*Jinv20*w14-1.0957792207792208e-04*w5*Jinv21+4.2004870129870131e-04*Jinv02*w54+9.8620129870129864e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.5568181818181820e-04*w49*Jinv10-6.9399350649350649e-04*Jinv01*w33-6.9399350649350649e-04*w39*Jinv00+9.8620129870129864e-04*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11-3.0438311688311688e-05*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-3.1047077922077924e-04*w47*Jinv21-3.1047077922077924e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23+3.4699675324675324e-04*w52*Jinv11+4.9310064935064932e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+3.4699675324675324e-04*Jinv01*w51+4.9310064935064932e-04*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-3.1047077922077924e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+9.8620129870129864e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+2.5568181818181820e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-1.6436688311688311e-03*Jinv20*w44-2.2524350649350650e-04*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.9480519480519481e-04*w57*Jinv00-1.9480519480519481e-04*Jinv20*w59-1.1323051948051949e-03*Jinv02*w33+2.4350649350649351e-04*w0*Jinv00-9.1314935064935063e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10+2.4350649350649351e-04*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00+3.6525974025974026e-05*w48*Jinv02-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-6.9399350649350649e-04*w40*Jinv10-6.2094155844155847e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+9.8620129870129864e-04*Jinv22*w14-9.1314935064935063e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36-2.7759740259740259e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30+4.2004870129870131e-04*Jinv02*w51+4.2004870129870131e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-6.2094155844155847e-04*w25*Jinv11+3.4699675324675324e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-2.2646103896103898e-03*w16*Jinv12+2.5568181818181820e-04*Jinv21*w50+2.5568181818181820e-04*w48*Jinv01-4.8701298701298701e-04*Jinv12*w1-4.2613636363636367e-05*w10*Jinv11)*D;
  A[2][18]=(0.0)*D;
  A[2][19]=(0.0)*D;
  A[2][20]=(1.8262987012987013e-05*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10+3.9265422077922079e-04*Jinv20*w8+1.8262987012987013e-05*w13*Jinv10+2.7394480519480518e-04*Jinv21*w44-4.8396915584415583e-04*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20-4.7483766233766234e-04*Jinv01*w3-3.1047077922077924e-04*Jinv20*w23-2.1915584415584417e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+4.0584415584415584e-05*Jinv21*w2+9.6793831168831166e-04*w43*Jinv10+2.7394480519480518e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-4.2918019480519480e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01+4.0584415584415584e-05*w1*Jinv10+1.0592532467532468e-03*w16*Jinv10+2.7394480519480518e-04*w43*Jinv11-4.2918019480519480e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-1.4204545454545455e-05*Jinv00*w27+4.0584415584415584e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10-3.1047077922077924e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+7.4878246753246752e-04*Jinv00*w18+3.7439123376623376e-04*w36*Jinv00+7.4878246753246752e-04*Jinv21*w38-4.2918019480519480e-04*Jinv01*w24+4.9310064935064932e-04*w34*Jinv11+7.4878246753246752e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-3.1960227272727273e-04*w51*Jinv00-4.8396915584415583e-04*w4*Jinv10-4.7483766233766234e-04*w4*Jinv11-7.3051948051948052e-05*w41*Jinv20+7.4878246753246752e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21+7.4878246753246752e-04*Jinv21*w20+3.9265422077922079e-04*w6*Jinv00-2.1915584415584417e-04*Jinv20*w56-1.5523538961038962e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20+4.9310064935064932e-04*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11+1.8262987012987013e-05*Jinv20*w14-4.7483766233766234e-04*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-7.3051948051948052e-05*w39*Jinv00+7.4878246753246752e-04*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+7.8530844155844158e-04*w6*Jinv01+3.9265422077922079e-04*w7*Jinv10-1.0957792207792208e-04*w47*Jinv21+3.7439123376623376e-04*Jinv20*w38-4.2918019480519480e-04*w52*Jinv11-3.1960227272727273e-04*Jinv20*w53-3.1047077922077924e-04*w22*Jinv10-1.3697240259740259e-04*w22*Jinv11-1.3697240259740259e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01-3.1960227272727273e-04*w52*Jinv10+7.8530844155844158e-04*w7*Jinv11-9.2329545454545453e-05*w9*Jinv00+3.7439123376623376e-04*w37*Jinv10-1.0957792207792208e-04*Jinv21*w56-1.5523538961038962e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14-9.2329545454545453e-05*w10*Jinv10+9.6793831168831166e-04*Jinv20*w44+9.1314935064935065e-06*Jinv01*w12+7.8530844155844158e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+4.0584415584415584e-05*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23-1.0957792207792208e-04*Jinv01*w54-1.0957792207792208e-04*w55*Jinv11-1.5523538961038962e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+1.0592532467532468e-03*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+1.0592532467532468e-03*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+7.4878246753246752e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-4.8396915584415583e-04*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30-4.2918019480519480e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+9.6793831168831166e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[2][21]=(0.0)*D;
  A[2][22]=(0.0)*D;
  A[2][23]=(4.7483766233766234e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+1.3697240259740259e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+4.7483766233766234e-04*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+2.5568181818181820e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00-9.1314935064935065e-06*w32*Jinv20-5.5702110389610384e-04*Jinv01*w3-7.8530844155844158e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-1.1668019480519480e-04*Jinv21*w59-7.4878246753246752e-04*w25*Jinv12-4.0584415584415584e-05*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2-2.7394480519480518e-04*w43*Jinv10+2.5568181818181820e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12+1.3697240259740259e-04*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-4.0584415584415584e-05*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+2.5568181818181820e-04*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-2.7394480519480518e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-6.2094155844155847e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.2175324675324675e-04*w28*Jinv11+1.3697240259740259e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.8530844155844158e-04*w22*Jinv12-9.1314935064935065e-06*w31*Jinv10-9.1314935064935065e-06*Jinv02*w3-7.8530844155844158e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-4.0584415584415584e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00-2.7394480519480521e-05*Jinv21*w38-7.4878246753246752e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59+2.5568181818181820e-04*w34*Jinv11+2.5568181818181820e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-2.7394480519480518e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-5.5702110389610384e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20+4.7483766233766234e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-5.5702110389610384e-04*w32*Jinv21+2.5568181818181820e-04*Jinv21*w20+1.3697240259740259e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12+2.5568181818181820e-04*Jinv21*w35-5.5702110389610384e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11+4.7483766233766234e-04*Jinv20*w14-5.5702110389610384e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54-9.1314935064935065e-06*w4*Jinv12-7.8530844155844158e-04*w21*Jinv02+1.3697240259740259e-04*w49*Jinv10-1.2175324675324675e-04*Jinv21*w29+2.5568181818181820e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00+4.7483766233766234e-04*Jinv02*w12+2.5568181818181820e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+1.3697240259740259e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38-7.8530844155844158e-04*Jinv22*w23-2.7394480519480521e-05*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.3697240259740259e-04*Jinv22*w8-7.8530844155844158e-04*w22*Jinv10+1.3697240259740259e-04*Jinv22*w50-1.2784090909090910e-04*w22*Jinv11+1.3697240259740259e-04*Jinv12*w7-1.2784090909090910e-04*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30+1.3697240259740259e-04*Jinv12*w49-2.7394480519480521e-05*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+2.5568181818181820e-04*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+1.3697240259740259e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12-9.1314935064935065e-06*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-6.2094155844155847e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-2.7394480519480518e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18-6.2094155844155847e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-4.0584415584415584e-05*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20-1.2784090909090910e-04*Jinv21*w23+2.5568181818181820e-04*Jinv01*w54+2.5568181818181820e-04*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-4.0584415584415584e-05*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+1.3697240259740259e-04*w48*Jinv02-1.1668019480519480e-04*Jinv01*w9-4.9310064935064932e-04*Jinv20*w17-9.1314935064935065e-06*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.7483766233766234e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36-2.7394480519480518e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20-5.5702110389610384e-04*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-2.7394480519480518e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-1.2175324675324675e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-4.0584415584415584e-05*Jinv12*w1-1.1668019480519480e-04*w10*Jinv11)*D;
  A[2][24]=(0.0)*D;
  A[2][25]=(0.0)*D;
  A[2][26]=(-4.7483766233766234e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.3697240259740259e-04*Jinv20*w8-4.7483766233766234e-04*w13*Jinv10+9.6793831168831166e-04*Jinv21*w44+9.1314935064935065e-06*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20+1.8262987012987013e-05*Jinv01*w3+7.8530844155844158e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+4.0584415584415584e-05*Jinv21*w2+2.7394480519480518e-04*w43*Jinv10+9.6793831168831166e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+1.0592532467532468e-03*w16*Jinv11+1.0592532467532468e-03*Jinv21*w17+3.7439123376623376e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01+7.4878246753246752e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01+4.0584415584415584e-05*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+9.6793831168831166e-04*w43*Jinv11+7.4878246753246752e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27+4.0584415584415584e-05*Jinv11*w1-4.8396915584415583e-04*w13*Jinv11+1.0592532467532468e-03*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10+7.8530844155844158e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.2918019480519480e-04*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00-4.2918019480519480e-04*Jinv21*w38+7.4878246753246752e-04*Jinv01*w24-7.3051948051948052e-05*w34*Jinv11-1.5523538961038962e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-1.0957792207792208e-04*w51*Jinv00+9.1314935064935065e-06*w4*Jinv10+1.8262987012987013e-05*w4*Jinv11+4.9310064935064932e-04*w41*Jinv20-4.2918019480519480e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21-1.5523538961038962e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+7.4878246753246752e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20-7.3051948051948052e-05*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11+3.7439123376623376e-04*w46*Jinv11-4.7483766233766234e-04*Jinv20*w14+1.8262987012987013e-05*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33+4.9310064935064932e-04*w39*Jinv00-1.5523538961038962e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-3.1047077922077924e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10+3.7439123376623376e-04*w47*Jinv21-1.0957792207792208e-04*Jinv20*w38-2.1915584415584417e-04*w52*Jinv11-1.0957792207792208e-04*Jinv20*w53+7.8530844155844158e-04*w22*Jinv10+3.9265422077922079e-04*w22*Jinv11+3.9265422077922079e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-1.0957792207792208e-04*w52*Jinv10-3.1047077922077924e-04*w7*Jinv11+1.0247564935064935e-04*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10-3.1960227272727273e-04*Jinv21*w56+7.4878246753246752e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11-4.8396915584415583e-04*Jinv21*w14+1.0247564935064935e-04*w10*Jinv10+2.7394480519480518e-04*Jinv20*w44-4.8396915584415583e-04*Jinv01*w12-3.1047077922077924e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+4.0584415584415584e-05*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.9265422077922079e-04*Jinv21*w23-3.1960227272727273e-04*Jinv01*w54-3.1960227272727273e-04*w55*Jinv11+7.4878246753246752e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-1.4204545454545455e-05*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-4.2918019480519480e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11+9.1314935064935065e-06*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30+7.4878246753246752e-04*w25*Jinv11-4.2918019480519480e-04*w55*Jinv10+2.7394480519480518e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[2][27]=(0.0)*D;
  A[2][28]=(0.0)*D;
  A[2][29]=(-1.9480519480519481e-04*Jinv21*w44+6.4935064935064935e-05*Jinv01*w3+5.4112554112554113e-05*Jinv21*w59+1.8939393939393939e-04*Jinv21*w2-1.9480519480519481e-04*w42*Jinv01+5.4112554112554113e-05*w11*Jinv21-4.2613636363636367e-05*w16*Jinv11-4.2613636363636367e-05*Jinv21*w17+1.5726461038961039e-04*Jinv01*w45+5.4112554112554113e-05*w58*Jinv11+5.4112554112554113e-05*w57*Jinv01+1.5726461038961039e-04*Jinv21*w26+1.8939393939393939e-04*w0*Jinv01-1.9480519480519481e-04*w43*Jinv11-4.2613636363636367e-05*Jinv21*w41+1.8939393939393939e-04*Jinv11*w1+8.3198051948051951e-05*w13*Jinv11-4.2613636363636367e-05*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-4.2613636363636367e-05*Jinv01*w39+1.0247564935064935e-04*Jinv21*w38+1.5726461038961039e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11-1.1668019480519480e-04*w19*Jinv11+1.0247564935064935e-04*Jinv21*w53+6.4935064935064935e-05*w4*Jinv11+6.4935064935064935e-05*w32*Jinv21-1.1668019480519480e-04*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35+6.4935064935064935e-05*w31*Jinv11+1.5726461038961039e-04*w46*Jinv11+6.4935064935064935e-05*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33-1.1668019480519480e-04*Jinv01*w18+1.0247564935064935e-04*w37*Jinv11-1.4204545454545455e-05*w6*Jinv01+1.5726461038961039e-04*w47*Jinv21+1.0247564935064935e-04*w52*Jinv11+2.0495129870129870e-04*w22*Jinv11+2.0495129870129870e-04*w21*Jinv01+1.0247564935064935e-04*Jinv01*w51+1.0247564935064935e-04*w36*Jinv01-1.4204545454545455e-05*w7*Jinv11-1.1668019480519480e-04*Jinv21*w56-1.4204545454545455e-05*w49*Jinv11+8.3198051948051951e-05*Jinv21*w14+8.3198051948051951e-05*Jinv01*w12-1.4204545454545455e-05*Jinv21*w8+2.0495129870129870e-04*Jinv21*w23-1.1668019480519480e-04*Jinv01*w54-1.1668019480519480e-04*w55*Jinv11+5.4112554112554113e-05*Jinv01*w9-4.2613636363636367e-05*w40*Jinv11+6.4935064935064935e-05*Jinv01*w30+1.5726461038961039e-04*w25*Jinv11-1.4204545454545455e-05*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27-1.4204545454545455e-05*w48*Jinv01+5.4112554112554113e-05*w10*Jinv11)*D;
  A[2][30]=(0.0)*D;
  A[2][31]=(0.0)*D;
  A[2][32]=(4.5657467532467534e-05*Jinv00*w12+9.2329545454545453e-05*w28*Jinv10+2.1915584415584417e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8+1.2175324675324675e-04*w57*Jinv02+4.5657467532467534e-05*w13*Jinv10+2.1915584415584417e-04*Jinv02*w39+6.7573051948051951e-04*Jinv21*w44+4.5657467532467534e-05*w3*Jinv00+9.1314935064935068e-05*w32*Jinv20+4.5657467532467534e-05*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23-1.1870941558441559e-04*Jinv22*w38+2.1002435064935065e-04*Jinv00*w54+5.6818181818181818e-05*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12+1.2987012987012987e-03*w0*Jinv02-2.8409090909090908e-04*Jinv21*w2+6.7573051948051951e-04*w43*Jinv10+6.7573051948051951e-04*w42*Jinv01+9.2329545454545453e-05*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.2784090909090910e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55+5.6818181818181818e-05*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12+1.4610389610389610e-04*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35+5.6818181818181818e-05*w58*Jinv11+5.6818181818181818e-05*w57*Jinv01-6.3920454545454550e-05*Jinv21*w26-2.8409090909090908e-04*w0*Jinv01-2.8409090909090908e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+6.7573051948051951e-04*w43*Jinv11-1.2784090909090910e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41+1.8262987012987014e-04*w42*Jinv02+9.2329545454545453e-05*Jinv00*w27+1.5726461038961039e-04*w11*Jinv22-2.8409090909090908e-04*Jinv11*w1+4.5657467532467534e-05*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+9.2329545454545453e-05*w28*Jinv11+1.4610389610389610e-04*Jinv20*w50+9.2329545454545453e-05*Jinv20*w29-1.2784090909090910e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33-6.6659902597402602e-04*w22*Jinv12+9.1314935064935068e-05*w31*Jinv10+1.1414366883116883e-03*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+1.1870941558441558e-03*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39+1.2987012987012987e-03*Jinv22*w2-6.3920454545454550e-05*Jinv00*w18-1.2784090909090910e-04*w36*Jinv00-1.2784090909090910e-04*Jinv21*w38-1.1870941558441559e-04*Jinv02*w45-6.3920454545454550e-05*Jinv01*w24+1.2175324675324675e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11-6.3920454545454550e-05*w19*Jinv11+2.1002435064935065e-04*Jinv21*w53+2.1002435064935065e-04*w51*Jinv00+2.1915584415584417e-04*Jinv12*w34+1.8262987012987014e-04*Jinv22*w44+4.5657467532467534e-05*w4*Jinv10+4.5657467532467534e-05*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+1.1414366883116883e-03*Jinv12*w13-6.3920454545454550e-05*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10+9.1314935064935068e-05*w32*Jinv21-6.3920454545454550e-05*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00-1.1870941558441559e-04*w37*Jinv12+2.1002435064935065e-04*Jinv20*w56+2.1915584415584417e-04*Jinv22*w41-6.3920454545454550e-05*w26*Jinv20+9.2329545454545453e-05*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+9.1314935064935068e-05*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26-1.2784090909090910e-04*w46*Jinv11+4.5657467532467534e-05*Jinv20*w14+4.5657467532467534e-05*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54+1.1414366883116883e-03*w4*Jinv12-6.6659902597402602e-04*w21*Jinv02+1.4610389610389610e-04*w49*Jinv10+9.2329545454545453e-05*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+1.1414366883116883e-03*Jinv02*w12-6.3920454545454550e-05*Jinv01*w18-1.2784090909090910e-04*w37*Jinv11+1.5726461038961039e-04*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10-1.2784090909090910e-04*w47*Jinv21-1.2784090909090910e-04*Jinv20*w38-6.6659902597402602e-04*Jinv22*w23+2.1002435064935065e-04*w52*Jinv11+1.1870941558441558e-03*w31*Jinv12-1.1870941558441559e-04*w46*Jinv12+2.1002435064935065e-04*Jinv20*w53-6.6659902597402602e-04*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10-4.0178571428571428e-04*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11-6.6659902597402602e-04*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01+2.1002435064935065e-04*Jinv01*w51+1.1870941558441558e-03*Jinv02*w30-4.0178571428571428e-04*Jinv12*w49-1.2784090909090910e-04*w36*Jinv01+2.1002435064935065e-04*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+9.2329545454545453e-05*w9*Jinv00-1.2784090909090910e-04*w37*Jinv10+2.1002435064935065e-04*Jinv21*w56-6.3920454545454550e-05*w24*Jinv00+1.5726461038961039e-04*Jinv02*w27-6.6659902597402602e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+1.1414366883116883e-03*Jinv22*w5-1.1870941558441559e-04*w47*Jinv22+1.4610389610389610e-04*w49*Jinv11+4.5657467532467534e-05*Jinv21*w14+9.2329545454545453e-05*w10*Jinv10+6.7573051948051951e-04*Jinv20*w44+1.2175324675324675e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+4.5657467532467534e-05*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8+5.6818181818181818e-05*w57*Jinv00+5.6818181818181818e-05*Jinv20*w59+2.1915584415584417e-04*Jinv02*w33-2.8409090909090908e-04*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23+2.1002435064935065e-04*Jinv01*w54+2.1002435064935065e-04*w55*Jinv11-6.3920454545454550e-05*w25*Jinv10-2.8409090909090908e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-4.0178571428571428e-04*w48*Jinv02+9.2329545454545453e-05*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17+9.1314935064935068e-05*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-6.3920454545454550e-05*w20*Jinv20+1.5726461038961039e-04*Jinv22*w29+1.1414366883116883e-03*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11-1.1870941558441559e-04*Jinv02*w36+1.8262987012987014e-04*Jinv12*w43+4.5657467532467534e-05*w5*Jinv20+9.1314935064935068e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-6.3920454545454550e-05*w25*Jinv11+2.1002435064935065e-04*w55*Jinv10+6.7573051948051951e-04*w42*Jinv00-1.2784090909090910e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12+1.4610389610389610e-04*Jinv21*w50+9.2329545454545453e-05*Jinv01*w27+1.4610389610389610e-04*w48*Jinv01+1.2987012987012987e-03*Jinv12*w1+9.2329545454545453e-05*w10*Jinv11)*D;
  A[2][33]=(0.0)*D;
  A[2][34]=(0.0)*D;
  A[2][35]=(-5.4788961038961042e-05*Jinv00*w12-1.9480519480519481e-04*w28*Jinv10-9.1314935064935063e-04*w40*Jinv12-6.2094155844155847e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-5.4788961038961042e-05*w13*Jinv10-9.1314935064935063e-04*Jinv02*w39-2.7759740259740259e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20+9.8620129870129864e-04*Jinv01*w3+2.5568181818181820e-04*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54-3.0438311688311688e-05*Jinv21*w59+3.4699675324675324e-04*w25*Jinv12+2.4350649350649351e-04*w0*Jinv02-4.8701298701298701e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-2.7759740259740259e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17+4.2004870129870131e-04*Jinv01*w45-3.1047077922077924e-04*Jinv12*w55-4.2613636363636367e-05*w58*Jinv10-6.9399350649350649e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+3.4699675324675324e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01+4.2004870129870131e-04*Jinv21*w26-4.8701298701298701e-04*w0*Jinv01+2.4350649350649351e-04*w1*Jinv10-9.1314935064935063e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20-2.7759740259740259e-03*w43*Jinv11+3.4699675324675324e-04*w45*Jinv00-1.1323051948051949e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-1.9480519480519481e-04*Jinv00*w27-4.2613636363636367e-05*w11*Jinv22-4.8701298701298701e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-2.2524350649350650e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50-1.9480519480519481e-04*Jinv20*w29+3.4699675324675324e-04*w46*Jinv10-6.9399350649350649e-04*Jinv22*w17-1.8262987012987013e-03*Jinv00*w33+2.5568181818181820e-04*w22*Jinv12-1.0957792207792208e-04*w31*Jinv10-1.0957792207792208e-04*Jinv02*w3+2.5568181818181820e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39+2.4350649350649351e-04*Jinv22*w2-3.1047077922077924e-04*Jinv00*w18-6.2094155844155847e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+4.2004870129870131e-04*Jinv01*w24-2.2646103896103898e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.8262987012987013e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10+9.8620129870129864e-04*w4*Jinv11-6.9399350649350649e-04*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13-3.1047077922077924e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+9.8620129870129864e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-9.1314935064935063e-04*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-6.2094155844155847e-04*w52*Jinv12-2.2646103896103898e-03*Jinv21*w35+9.8620129870129864e-04*w31*Jinv11+3.4699675324675324e-04*Jinv22*w26+4.2004870129870131e-04*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14+9.8620129870129864e-04*w5*Jinv21-3.1047077922077924e-04*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12+2.5568181818181820e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-2.2524350649350650e-04*Jinv21*w29-2.2646103896103898e-03*Jinv01*w33-6.9399350649350649e-04*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-4.2613636363636367e-05*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+4.2004870129870131e-04*w47*Jinv21-6.2094155844155847e-04*Jinv20*w38+2.5568181818181820e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+2.5568181818181820e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-6.2094155844155847e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00-1.9480519480519481e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14-1.6436688311688311e-03*Jinv20*w44+2.1915584415584417e-04*Jinv02*w18+4.9310064935064932e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-1.8262987012987013e-03*Jinv02*w33+2.4350649350649351e-04*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20+3.6525974025974026e-05*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10+2.4350649350649351e-04*Jinv20*w2-9.1314935064935063e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9-9.1314935064935063e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-6.9399350649350649e-04*w40*Jinv10-3.1047077922077924e-04*w20*Jinv20-1.9480519480519481e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20+9.8620129870129864e-04*Jinv01*w30-6.2094155844155847e-04*Jinv02*w51-3.1047077922077924e-04*Jinv22*w56-1.9480519480519481e-04*Jinv12*w28+4.2004870129870131e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+3.4699675324675324e-04*w47*Jinv20-6.9399350649350649e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-2.2524350649350650e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+2.4350649350649351e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[2][36]=(0.0)*D;
  A[2][37]=(0.0)*D;
  A[2][38]=(9.1314935064935065e-06*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.2918019480519480e-04*Jinv22*w53+3.9265422077922079e-04*Jinv20*w8+1.0247564935064935e-04*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-4.8396915584415583e-04*w3*Jinv00+1.8262987012987013e-05*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23+7.4878246753246752e-04*Jinv22*w38-4.2918019480519480e-04*Jinv00*w54-4.2918019480519480e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+9.6793831168831166e-04*w43*Jinv10-1.0957792207792208e-04*Jinv12*w55-1.4204545454545455e-05*w58*Jinv10+4.9310064935064932e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-3.1047077922077924e-04*w48*Jinv00-4.2918019480519480e-04*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35+4.0584415584415584e-05*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.4878246753246752e-04*Jinv22*w20-2.1915584415584417e-04*w45*Jinv00+2.7394480519480518e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-3.1047077922077924e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+4.9310064935064932e-04*Jinv22*w17+1.0592532467532468e-03*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12+1.8262987012987013e-05*w31*Jinv10-4.7483766233766234e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00+9.1314935064935065e-06*w32*Jinv22+4.0584415584415584e-05*Jinv22*w2+3.7439123376623376e-04*Jinv00*w18+7.4878246753246752e-04*w36*Jinv00-1.0957792207792208e-04*Jinv02*w45+1.0247564935064935e-04*Jinv22*w59-1.5523538961038962e-04*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+2.7394480519480518e-04*Jinv22*w44-4.8396915584415583e-04*w4*Jinv10-7.3051948051948052e-05*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13+3.7439123376623376e-04*w19*Jinv10+1.0592532467532468e-03*w34*Jinv10+3.9265422077922079e-04*w6*Jinv00+7.4878246753246752e-04*w37*Jinv12-4.2918019480519480e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-3.1960227272727273e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20-4.2918019480519480e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14-1.0957792207792208e-04*Jinv02*w54-4.7483766233766234e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02-3.1047077922077924e-04*w49*Jinv10-7.3051948051948052e-05*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.9265422077922079e-04*w7*Jinv10+7.4878246753246752e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23+9.1314935064935065e-06*w31*Jinv12-1.0957792207792208e-04*w46*Jinv12-1.5523538961038962e-04*Jinv20*w53+7.8530844155844158e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50+7.8530844155844158e-04*Jinv12*w7+9.1314935064935065e-06*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-1.5523538961038962e-04*w52*Jinv10-9.2329545454545453e-05*w9*Jinv00+7.4878246753246752e-04*w37*Jinv10-3.1960227272727273e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27+7.8530844155844158e-04*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-4.7483766233766234e-04*Jinv22*w5-1.0957792207792208e-04*w47*Jinv22-9.2329545454545453e-05*w10*Jinv10+9.6793831168831166e-04*Jinv20*w44+1.0247564935064935e-04*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33+4.0584415584415584e-05*w0*Jinv00+1.0592532467532468e-03*w35*Jinv20-3.1960227272727273e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17+1.8262987012987013e-05*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+3.7439123376623376e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14+7.4878246753246752e-04*Jinv02*w36+2.7394480519480518e-04*Jinv12*w43-4.8396915584415583e-04*w5*Jinv20-4.2918019480519480e-04*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28-4.2918019480519480e-04*w55*Jinv10+9.6793831168831166e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20+4.9310064935064932e-04*w16*Jinv12+4.0584415584415584e-05*Jinv12*w1)*D;
  A[2][39]=(0.0)*D;
  A[2][40]=(0.0)*D;
  A[2][41]=(9.8620129870129864e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10-1.8262987012987013e-03*w40*Jinv12-3.1047077922077924e-04*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8+9.8620129870129864e-04*w13*Jinv10-1.8262987012987013e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+4.9310064935064932e-04*w3*Jinv00+9.8620129870129864e-04*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-4.2613636363636367e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12+2.4350649350649351e-04*w0*Jinv02+2.4350649350649351e-04*Jinv21*w2-2.7759740259740259e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-1.9480519480519481e-04*w11*Jinv21-9.1314935064935063e-04*w16*Jinv11-9.1314935064935063e-04*Jinv21*w17-6.2094155844155847e-04*Jinv01*w45-6.2094155844155847e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10-6.9399350649350649e-04*w15*Jinv02-1.9480519480519481e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-9.1314935064935063e-04*Jinv22*w35-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01-3.1047077922077924e-04*Jinv21*w26+2.4350649350649351e-04*w0*Jinv01-4.8701298701298701e-04*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10+3.4699675324675324e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-1.8262987012987013e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-1.9480519480519481e-04*w11*Jinv22+2.4350649350649351e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-9.1314935064935063e-04*w15*Jinv01-4.9310064935064932e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-6.9399350649350649e-04*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+9.8620129870129864e-04*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.8262987012987013e-03*Jinv01*w39+2.4350649350649351e-04*Jinv22*w2+4.2004870129870131e-04*Jinv00*w18+4.2004870129870131e-04*w36*Jinv00+3.4699675324675324e-04*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45-3.1047077922077924e-04*Jinv01*w24-6.9399350649350649e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00-9.1314935064935063e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+4.9310064935064932e-04*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11-2.2646103896103898e-03*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13+4.2004870129870131e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-1.8262987012987013e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20-2.2524350649350650e-04*w11*Jinv20-3.1047077922077924e-04*w52*Jinv12-6.9399350649350649e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26-6.2094155844155847e-04*w46*Jinv11+9.8620129870129864e-04*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21-6.2094155844155847e-04*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-6.9399350649350649e-04*Jinv01*w33-2.2646103896103898e-03*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+3.4699675324675324e-04*w37*Jinv11-1.9480519480519481e-04*Jinv02*w9+2.5568181818181820e-04*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10-6.2094155844155847e-04*w47*Jinv21+4.2004870129870131e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+2.5568181818181820e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+2.5568181818181820e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.4699675324675324e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+2.5568181818181820e-04*w7*Jinv11-2.2524350649350650e-04*w9*Jinv00+4.2004870129870131e-04*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-4.2613636363636367e-05*Jinv02*w27+2.5568181818181820e-04*w6*Jinv02+3.4699675324675324e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-2.2524350649350650e-04*w10*Jinv10-2.7759740259740259e-03*Jinv20*w44+3.4699675324675324e-04*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+2.5568181818181820e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59-9.1314935064935063e-04*Jinv02*w33-4.8701298701298701e-04*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-4.8701298701298701e-04*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.9480519480519481e-04*Jinv01*w9-1.1323051948051949e-03*Jinv20*w17+9.8620129870129864e-04*w30*Jinv00-2.2646103896103898e-03*w40*Jinv10+4.2004870129870131e-04*w20*Jinv20-4.2613636363636367e-05*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-1.8262987012987013e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+4.9310064935064932e-04*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30-3.1047077922077924e-04*Jinv02*w51-6.2094155844155847e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-3.1047077922077924e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-2.7759740259740259e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-6.9399350649350649e-04*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.6436688311688311e-04*w48*Jinv01+2.4350649350649351e-04*Jinv12*w1-1.9480519480519481e-04*w10*Jinv11)*D;
  A[2][42]=(0.0)*D;
  A[2][43]=(0.0)*D;
  A[2][44]=(-9.8620129870129864e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12-4.2004870129870131e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+2.2524350649350650e-04*w57*Jinv02-9.8620129870129864e-04*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+2.7759740259740259e-03*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00-9.8620129870129864e-04*w32*Jinv20-9.8620129870129864e-04*Jinv01*w3+4.9310064935064932e-04*Jinv20*w23+2.7394480519480518e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+3.0438311688311688e-05*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+4.8701298701298701e-04*w0*Jinv02+4.8701298701298701e-04*Jinv21*w2+2.7759740259740259e-03*w43*Jinv10+2.7759740259740259e-03*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17-4.2004870129870131e-04*Jinv01*w45-4.2004870129870131e-04*Jinv12*w55+3.0438311688311688e-05*w58*Jinv10+2.2646103896103898e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12+4.9310064935064932e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-4.2004870129870131e-04*Jinv21*w26+4.8701298701298701e-04*w0*Jinv01+4.8701298701298701e-04*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+2.7759740259740259e-03*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+2.7759740259740259e-03*w42*Jinv02+3.0438311688311688e-05*Jinv00*w27+3.0438311688311688e-05*w11*Jinv22+4.8701298701298701e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+2.2524350649350650e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10+2.2646103896103898e-03*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33+4.9310064935064932e-04*w22*Jinv12-9.8620129870129864e-04*w31*Jinv10-9.8620129870129864e-04*Jinv02*w3+4.9310064935064932e-04*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+4.8701298701298701e-04*Jinv22*w2-4.2004870129870131e-04*Jinv00*w18-4.2004870129870131e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+2.7394480519480518e-04*Jinv02*w45-4.2004870129870131e-04*Jinv01*w24+2.2524350649350650e-04*Jinv22*w59+2.2646103896103898e-03*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+2.7759740259740259e-03*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-9.8620129870129864e-04*w4*Jinv11+2.2646103896103898e-03*w41*Jinv20-9.8620129870129864e-04*Jinv12*w13-4.2004870129870131e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-9.8620129870129864e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+2.7394480519480518e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20+2.2524350649350650e-04*w11*Jinv20-4.2004870129870131e-04*w52*Jinv12+2.2646103896103898e-03*Jinv21*w35-9.8620129870129864e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26-4.2004870129870131e-04*w46*Jinv11-9.8620129870129864e-04*Jinv20*w14-9.8620129870129864e-04*w5*Jinv21-4.2004870129870131e-04*Jinv02*w54-9.8620129870129864e-04*w4*Jinv12+4.9310064935064932e-04*w21*Jinv02+4.9310064935064932e-04*w49*Jinv10+2.2524350649350650e-04*Jinv21*w29+2.2646103896103898e-03*Jinv01*w33+2.2646103896103898e-03*w39*Jinv00-9.8620129870129864e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+3.0438311688311688e-05*Jinv02*w9+4.9310064935064932e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10-4.2004870129870131e-04*w47*Jinv21-4.2004870129870131e-04*Jinv20*w38+4.9310064935064932e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+2.7394480519480518e-04*Jinv20*w53+4.9310064935064932e-04*Jinv22*w8+4.9310064935064932e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+4.9310064935064932e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+4.9310064935064932e-04*w7*Jinv11+2.2524350649350650e-04*w9*Jinv00-4.2004870129870131e-04*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+2.7394480519480518e-04*w24*Jinv00+3.0438311688311688e-05*Jinv02*w27+4.9310064935064932e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-9.8620129870129864e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+4.9310064935064932e-04*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14+2.2524350649350650e-04*w10*Jinv10+2.7759740259740259e-03*Jinv20*w44+2.2524350649350650e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+4.9310064935064932e-04*Jinv21*w8+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+4.8701298701298701e-04*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+2.7394480519480518e-04*w25*Jinv10+4.8701298701298701e-04*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+3.0438311688311688e-05*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17-9.8620129870129864e-04*w30*Jinv00+2.2646103896103898e-03*w40*Jinv10-4.2004870129870131e-04*w20*Jinv20+3.0438311688311688e-05*Jinv22*w29-9.8620129870129864e-04*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36+2.7759740259740259e-03*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-9.8620129870129864e-04*Jinv01*w30-4.2004870129870131e-04*Jinv02*w51-4.2004870129870131e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28-4.2004870129870131e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+2.7759740259740259e-03*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+2.2646103896103898e-03*w16*Jinv12+4.9310064935064932e-04*Jinv21*w50+2.2524350649350650e-04*Jinv01*w27+4.9310064935064932e-04*w48*Jinv01+4.8701298701298701e-04*Jinv12*w1+3.0438311688311688e-05*w10*Jinv11)*D;
  A[2][45]=(0.0)*D;
  A[2][46]=(0.0)*D;
  A[2][47]=(4.9310064935064932e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+1.0247564935064935e-04*w57*Jinv02+4.9310064935064932e-04*Jinv02*w39+9.6793831168831166e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3-1.0957792207792208e-04*Jinv22*w38-1.4204545454545455e-05*Jinv21*w59+7.4878246753246752e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+4.0584415584415584e-05*Jinv21*w2+9.6793831168831166e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+7.4878246753246752e-04*Jinv01*w45-4.2918019480519480e-04*Jinv12*w55+4.9310064935064932e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+3.7439123376623376e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01-4.2918019480519480e-04*Jinv22*w20+9.6793831168831166e-04*w43*Jinv11+1.0592532467532468e-03*Jinv21*w41+2.7394480519480518e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+4.0584415584415584e-05*Jinv11*w1-4.8396915584415583e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+4.9310064935064932e-04*Jinv22*w17+7.8530844155844158e-04*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3+9.1314935064935065e-06*w32*Jinv22+1.0592532467532468e-03*Jinv01*w39+4.0584415584415584e-05*Jinv22*w2-2.1915584415584417e-04*Jinv21*w38+7.4878246753246752e-04*Jinv02*w45+3.7439123376623376e-04*Jinv01*w24+1.0247564935064935e-04*Jinv22*w59-7.3051948051948052e-05*w34*Jinv11-3.1960227272727273e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+2.7394480519480518e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11-4.7483766233766234e-04*Jinv12*w13+1.8262987012987013e-05*w32*Jinv21-3.1960227272727273e-04*Jinv21*w20-1.0957792207792208e-04*w37*Jinv12+4.9310064935064932e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12-7.3051948051948052e-05*Jinv21*w35+1.8262987012987013e-05*w31*Jinv11+7.4878246753246752e-04*Jinv22*w26+7.4878246753246752e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21-4.2918019480519480e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12+7.8530844155844158e-04*w21*Jinv02-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33-4.7483766233766234e-04*Jinv02*w12-3.1960227272727273e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01+7.4878246753246752e-04*w47*Jinv21+7.8530844155844158e-04*Jinv22*w23-4.2918019480519480e-04*w52*Jinv11+9.1314935064935065e-06*w31*Jinv12+7.4878246753246752e-04*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8-1.3697240259740259e-04*Jinv22*w50+3.9265422077922079e-04*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+3.9265422077922079e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+9.1314935064935065e-06*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11-1.5523538961038962e-04*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5+7.4878246753246752e-04*w47*Jinv22-3.1047077922077924e-04*w49*Jinv11-4.8396915584415583e-04*Jinv21*w14+1.0247564935064935e-04*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18-4.8396915584415583e-04*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.9265422077922079e-04*Jinv21*w23-1.5523538961038962e-04*Jinv01*w54-1.5523538961038962e-04*w55*Jinv11-1.3697240259740259e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.7483766233766234e-04*Jinv22*w14+1.0592532467532468e-03*w40*Jinv11-1.0957792207792208e-04*Jinv02*w36+2.7394480519480518e-04*Jinv12*w43+1.8262987012987013e-05*Jinv01*w30-1.0957792207792208e-04*Jinv02*w51-4.2918019480519480e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.7439123376623376e-04*w25*Jinv11+4.9310064935064932e-04*w16*Jinv12-3.1047077922077924e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-3.1047077922077924e-04*w48*Jinv01+4.0584415584415584e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[2][48]=(0.0)*D;
  A[2][49]=(0.0)*D;
  A[2][50]=(-9.1314935064935065e-06*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+1.3697240259740259e-04*Jinv20*w8-1.2175324675324675e-04*w57*Jinv02-9.1314935064935065e-06*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39-2.7394480519480518e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00+4.7483766233766234e-04*w32*Jinv20-9.1314935064935065e-06*Jinv01*w3+1.3697240259740259e-04*Jinv20*w23+2.5568181818181820e-04*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-2.7394480519480521e-05*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-4.0584415584415584e-05*Jinv21*w2-2.7394480519480518e-04*w43*Jinv10-2.7394480519480518e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10+2.5568181818181820e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12-7.8530844155844158e-04*w48*Jinv00-2.7394480519480521e-05*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-4.0584415584415584e-05*w0*Jinv01-4.0584415584415584e-05*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-2.7394480519480521e-05*Jinv22*w20-2.7394480519480518e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41+2.5568181818181820e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-1.1668019480519480e-04*w11*Jinv22-4.0584415584415584e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-7.8530844155844158e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10+2.5568181818181820e-04*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+4.7483766233766234e-04*w31*Jinv10-5.5702110389610384e-04*Jinv02*w3+1.3697240259740259e-04*w21*Jinv00-6.2094155844155847e-04*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38+2.5568181818181820e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-1.2175324675324675e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34+2.5568181818181820e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-9.1314935064935065e-06*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20-5.5702110389610384e-04*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+4.7483766233766234e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+1.3697240259740259e-04*w6*Jinv00+2.5568181818181820e-04*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+4.7483766233766234e-04*w31*Jinv11-2.7394480519480521e-05*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11-9.1314935064935065e-06*Jinv20*w14-9.1314935064935065e-06*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54-5.5702110389610384e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-7.8530844155844158e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00-5.5702110389610384e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11-1.1668019480519480e-04*Jinv02*w9+1.3697240259740259e-04*w6*Jinv01+1.3697240259740259e-04*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11-6.2094155844155847e-04*w31*Jinv12+2.5568181818181820e-04*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+1.3697240259740259e-04*w22*Jinv10-1.2784090909090910e-04*Jinv22*w50+1.3697240259740259e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7+1.3697240259740259e-04*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51-6.2094155844155847e-04*Jinv02*w30-1.2784090909090910e-04*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+1.3697240259740259e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.1668019480519480e-04*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12-5.5702110389610384e-04*Jinv22*w5+2.5568181818181820e-04*w47*Jinv22-7.8530844155844158e-04*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-2.7394480519480518e-04*Jinv20*w44-1.2175324675324675e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12+1.3697240259740259e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-4.0584415584415584e-05*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+1.3697240259740259e-04*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-4.0584415584415584e-05*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-1.2784090909090910e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.7483766233766234e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20-1.1668019480519480e-04*Jinv22*w29-5.5702110389610384e-04*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+2.5568181818181820e-04*Jinv02*w36+2.5568181818181820e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20+4.7483766233766234e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-2.7394480519480518e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20+2.5568181818181820e-04*w16*Jinv12-7.8530844155844158e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-7.8530844155844158e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[2][51]=(0.0)*D;
  A[2][52]=(0.0)*D;
  A[2][53]=(9.1314935064935065e-06*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+7.4878246753246752e-04*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8-9.2329545454545453e-05*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39+9.1314935064935065e-06*w3*Jinv00-4.7483766233766234e-04*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23-1.5523538961038962e-04*Jinv22*w38+7.4878246753246752e-04*Jinv00*w54-2.1915584415584417e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+2.7394480519480518e-04*w43*Jinv10+3.7439123376623376e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10-7.3051948051948052e-05*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+7.8530844155844158e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+1.0592532467532468e-03*Jinv22*w35+4.0584415584415584e-05*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-4.2918019480519480e-04*Jinv22*w20-4.2918019480519480e-04*w45*Jinv00+9.6793831168831166e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27-1.4204545454545455e-05*w11*Jinv22+7.8530844155844158e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10-7.3051948051948052e-05*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-4.7483766233766234e-04*w31*Jinv10+1.8262987012987013e-05*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-4.8396915584415583e-04*w32*Jinv22+4.0584415584415584e-05*Jinv22*w2-1.0957792207792208e-04*Jinv00*w18-4.2918019480519480e-04*w36*Jinv00-3.1960227272727273e-04*Jinv02*w45-9.2329545454545453e-05*Jinv22*w59+7.4878246753246752e-04*w51*Jinv00+1.0592532467532468e-03*Jinv12*w34+9.6793831168831166e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv10+4.9310064935064932e-04*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13-1.0957792207792208e-04*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10-1.3697240259740259e-04*w6*Jinv00-1.5523538961038962e-04*w37*Jinv12+7.4878246753246752e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41-1.0957792207792208e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20+7.4878246753246752e-04*w52*Jinv12-2.1915584415584417e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14+3.7439123376623376e-04*Jinv02*w54+1.8262987012987013e-05*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+7.8530844155844158e-04*w49*Jinv10+4.9310064935064932e-04*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-1.3697240259740259e-04*w7*Jinv10-4.2918019480519480e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-4.8396915584415583e-04*w31*Jinv12-3.1960227272727273e-04*w46*Jinv12+7.4878246753246752e-04*Jinv20*w53-3.1047077922077924e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+3.9265422077922079e-04*Jinv22*w50-3.1047077922077924e-04*Jinv12*w7-4.8396915584415583e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49+7.4878246753246752e-04*w52*Jinv10+1.0247564935064935e-04*w9*Jinv00-4.2918019480519480e-04*w37*Jinv10-1.0957792207792208e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27-3.1047077922077924e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+1.8262987012987013e-05*Jinv22*w5-3.1960227272727273e-04*w47*Jinv22+1.0247564935064935e-04*w10*Jinv10+2.7394480519480518e-04*Jinv20*w44-9.2329545454545453e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+1.0592532467532468e-03*Jinv02*w33+4.0584415584415584e-05*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.0957792207792208e-04*w25*Jinv10+4.0584415584415584e-05*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.9265422077922079e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.7483766233766234e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14-1.5523538961038962e-04*Jinv02*w36+9.6793831168831166e-04*Jinv12*w43+9.1314935064935065e-06*w5*Jinv20+7.4878246753246752e-04*Jinv02*w51+3.7439123376623376e-04*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+7.4878246753246752e-04*w55*Jinv10+2.7394480519480518e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20-7.3051948051948052e-05*w16*Jinv12+4.0584415584415584e-05*Jinv12*w1)*D;
  A[2][54]=(0.0)*D;
  A[2][55]=(0.0)*D;
  A[2][56]=(1.0592532467532468e-03*w40*Jinv12+3.7439123376623376e-04*Jinv22*w53-9.2329545454545453e-05*w57*Jinv02+1.0592532467532468e-03*Jinv02*w39+2.7394480519480518e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3-3.1960227272727273e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-4.2918019480519480e-04*w25*Jinv12+4.0584415584415584e-05*w0*Jinv02+4.0584415584415584e-05*Jinv21*w2+2.7394480519480518e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.2918019480519480e-04*Jinv01*w45+7.4878246753246752e-04*Jinv12*w55-7.3051948051948052e-05*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+4.0584415584415584e-05*w0*Jinv01-2.1915584415584417e-04*Jinv22*w20+2.7394480519480518e-04*w43*Jinv11+4.9310064935064932e-04*Jinv21*w41+9.6793831168831166e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+4.0584415584415584e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-7.3051948051948052e-05*Jinv22*w17-3.1047077922077924e-04*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3-4.8396915584415583e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39+4.0584415584415584e-05*Jinv22*w2-4.2918019480519480e-04*Jinv21*w38-1.5523538961038962e-04*Jinv02*w45-1.0957792207792208e-04*Jinv01*w24-9.2329545454545453e-05*Jinv22*w59+4.9310064935064932e-04*w34*Jinv11-1.0957792207792208e-04*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+9.6793831168831166e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11+1.8262987012987013e-05*Jinv12*w13-4.7483766233766234e-04*w32*Jinv21-1.0957792207792208e-04*Jinv21*w20-3.1960227272727273e-04*w37*Jinv12+1.0592532467532468e-03*Jinv22*w41+3.7439123376623376e-04*w52*Jinv12+4.9310064935064932e-04*Jinv21*w35-4.7483766233766234e-04*w31*Jinv11-4.2918019480519480e-04*Jinv22*w26-4.2918019480519480e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21+7.4878246753246752e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12-3.1047077922077924e-04*w21*Jinv02+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33+1.8262987012987013e-05*Jinv02*w12-1.0957792207792208e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01-4.2918019480519480e-04*w47*Jinv21-3.1047077922077924e-04*Jinv22*w23+7.4878246753246752e-04*w52*Jinv11-4.8396915584415583e-04*w31*Jinv12-1.5523538961038962e-04*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8+3.9265422077922079e-04*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-4.8396915584415583e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49-4.2918019480519480e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11+7.4878246753246752e-04*Jinv21*w56-1.4204545454545455e-05*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5-1.5523538961038962e-04*w47*Jinv22+7.8530844155844158e-04*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14-9.2329545454545453e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+9.1314935064935065e-06*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33-1.3697240259740259e-04*Jinv21*w23+7.4878246753246752e-04*Jinv01*w54+7.4878246753246752e-04*w55*Jinv11+3.9265422077922079e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9-1.4204545454545455e-05*Jinv22*w29+1.8262987012987013e-05*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-3.1960227272727273e-04*Jinv02*w36+9.6793831168831166e-04*Jinv12*w43-4.7483766233766234e-04*Jinv01*w30+3.7439123376623376e-04*Jinv02*w51+7.4878246753246752e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28-1.0957792207792208e-04*w25*Jinv11-7.3051948051948052e-05*w16*Jinv12+7.8530844155844158e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+7.8530844155844158e-04*w48*Jinv01+4.0584415584415584e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[2][57]=(0.0)*D;
  A[2][58]=(0.0)*D;
  A[2][59]=(-4.2613636363636367e-05*w40*Jinv12+1.5726461038961039e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-4.2613636363636367e-05*Jinv02*w39-1.1668019480519480e-04*Jinv22*w38+1.0247564935064935e-04*w25*Jinv12+1.8939393939393939e-04*w0*Jinv02+1.5726461038961039e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+5.4112554112554113e-05*w10*Jinv12+1.0247564935064935e-04*Jinv02*w24-4.2613636363636367e-05*Jinv22*w35+1.0247564935064935e-04*Jinv22*w20-1.9480519480519481e-04*w42*Jinv02+5.4112554112554113e-05*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17-1.4204545454545455e-05*w22*Jinv12+6.4935064935064935e-05*Jinv02*w3+8.3198051948051951e-05*w32*Jinv22+1.8939393939393939e-04*Jinv22*w2-1.1668019480519480e-04*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-4.2613636363636367e-05*Jinv12*w34-1.9480519480519481e-04*Jinv22*w44+6.4935064935064935e-05*Jinv12*w13-1.1668019480519480e-04*w37*Jinv12-4.2613636363636367e-05*Jinv22*w41+1.5726461038961039e-04*w52*Jinv12+1.0247564935064935e-04*Jinv22*w26+1.5726461038961039e-04*Jinv02*w54+6.4935064935064935e-05*w4*Jinv12-1.4204545454545455e-05*w21*Jinv02+6.4935064935064935e-05*Jinv02*w12+5.4112554112554113e-05*Jinv02*w9-1.4204545454545455e-05*Jinv22*w23+8.3198051948051951e-05*w31*Jinv12-1.1668019480519480e-04*w46*Jinv12-1.4204545454545455e-05*Jinv22*w8+2.0495129870129870e-04*Jinv22*w50-1.4204545454545455e-05*Jinv12*w7+8.3198051948051951e-05*Jinv02*w30+2.0495129870129870e-04*Jinv12*w49+5.4112554112554113e-05*Jinv02*w27-1.4204545454545455e-05*w6*Jinv02+1.0247564935064935e-04*w19*Jinv12+6.4935064935064935e-05*Jinv22*w5-1.1668019480519480e-04*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12+1.0247564935064935e-04*Jinv02*w18-4.2613636363636367e-05*Jinv02*w33+2.0495129870129870e-04*w48*Jinv02+5.4112554112554113e-05*Jinv22*w29+6.4935064935064935e-05*Jinv22*w14-1.1668019480519480e-04*Jinv02*w36-1.9480519480519481e-04*Jinv12*w43+1.5726461038961039e-04*Jinv02*w51+1.5726461038961039e-04*Jinv22*w56+5.4112554112554113e-05*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+1.8939393939393939e-04*Jinv12*w1)*D;
  A[3][0]=(-1.1870941558441558e-03*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10-3.2873376623376621e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+1.0227272727272728e-03*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02-1.1870941558441558e-03*w13*Jinv10-3.2873376623376621e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-2.3741883116883116e-03*w3*Jinv00-1.1870941558441558e-03*w32*Jinv20-2.3741883116883116e-03*Jinv01*w3+6.7573051948051951e-04*Jinv20*w23+2.6481331168831169e-04*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59+1.0957792207792208e-04*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-2.0495129870129870e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10-6.5746753246753243e-04*w15*Jinv02-2.0495129870129870e-04*w10*Jinv12+6.7573051948051951e-04*w48*Jinv00+1.0957792207792208e-04*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+2.6481331168831169e-04*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00-3.2873376623376621e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-2.0495129870129870e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-1.1870941558441558e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+6.7573051948051951e-04*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-6.5746753246753243e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+6.7573051948051951e-04*w22*Jinv12-1.1870941558441558e-03*w31*Jinv10-2.3741883116883116e-03*Jinv02*w3+6.7573051948051951e-04*w21*Jinv00-1.1870941558441558e-03*w32*Jinv22-3.2873376623376621e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+2.6481331168831169e-04*Jinv00*w18+2.6481331168831169e-04*w36*Jinv00+2.6481331168831169e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-6.5746753246753243e-04*w34*Jinv11+2.6481331168831169e-04*w19*Jinv11+1.0957792207792208e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-2.3741883116883116e-03*w4*Jinv10-2.3741883116883116e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-1.1870941558441558e-03*Jinv12*w13+2.6481331168831169e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.1870941558441558e-03*w32*Jinv21+2.6481331168831169e-04*Jinv21*w20+1.0227272727272728e-03*w6*Jinv00+2.6481331168831169e-04*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56-3.2873376623376621e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.0495129870129870e-04*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-6.5746753246753243e-04*Jinv21*w35-1.1870941558441558e-03*w31*Jinv11+1.0957792207792208e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-1.1870941558441558e-03*Jinv20*w14-2.3741883116883116e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54-2.3741883116883116e-03*w4*Jinv12+6.7573051948051951e-04*w21*Jinv02+6.7573051948051951e-04*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-6.5746753246753243e-04*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-1.1870941558441558e-03*Jinv02*w12+2.6481331168831169e-04*Jinv01*w18+2.6481331168831169e-04*w37*Jinv11-2.0495129870129870e-04*Jinv02*w9+1.0227272727272728e-03*w6*Jinv01+1.0227272727272728e-03*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21+2.6481331168831169e-04*Jinv20*w38+6.7573051948051951e-04*Jinv22*w23+1.0957792207792208e-04*w52*Jinv11-1.1870941558441558e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.0227272727272728e-03*Jinv22*w8+6.7573051948051951e-04*w22*Jinv10+6.7573051948051951e-04*Jinv22*w50+6.7573051948051951e-04*w22*Jinv11+1.0227272727272728e-03*Jinv12*w7+6.7573051948051951e-04*w21*Jinv01+1.0957792207792208e-04*Jinv01*w51-1.1870941558441558e-03*Jinv02*w30+6.7573051948051951e-04*Jinv12*w49+2.6481331168831169e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.0227272727272728e-03*w7*Jinv11-2.0495129870129870e-04*w9*Jinv00+2.6481331168831169e-04*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+1.0227272727272728e-03*w6*Jinv02+2.6481331168831169e-04*w19*Jinv12-2.3741883116883116e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+6.7573051948051951e-04*w49*Jinv11-1.1870941558441558e-03*Jinv21*w14-2.0495129870129870e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+2.6481331168831169e-04*Jinv02*w18-1.1870941558441558e-03*Jinv01*w12+1.0227272727272728e-03*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+6.7573051948051951e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+6.7573051948051951e-04*w48*Jinv02-2.0495129870129870e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.1870941558441558e-03*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+2.6481331168831169e-04*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-1.1870941558441558e-03*Jinv22*w14-3.2873376623376621e-04*w40*Jinv11+2.6481331168831169e-04*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-2.3741883116883116e-03*w5*Jinv20-1.1870941558441558e-03*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-6.5746753246753243e-04*w16*Jinv12+6.7573051948051951e-04*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+6.7573051948051951e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-2.0495129870129870e-04*w10*Jinv11)*D;
  A[3][1]=(0.0)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(1.2327516233766234e-03*Jinv00*w12-1.5888798701298701e-03*w40*Jinv12+1.0683847402597404e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+1.2327516233766234e-03*w32*Jinv20-7.3965097402597406e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-5.7528409090909093e-04*Jinv22*w38-7.4878246753246752e-04*Jinv21*w59+1.0683847402597404e-03*w25*Jinv12+9.1314935064935068e-05*w0*Jinv02+9.1314935064935068e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-6.5746753246753243e-04*w11*Jinv21-4.7666396103896100e-03*w16*Jinv11-4.7666396103896100e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-4.7666396103896100e-03*w15*Jinv02-6.5746753246753243e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+1.0683847402597404e-03*Jinv02*w24-4.7666396103896100e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+1.0683847402597404e-03*Jinv21*w26+9.1314935064935068e-05*w0*Jinv01+1.1870941558441558e-03*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10-5.7528409090909093e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-1.5888798701298701e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-6.5746753246753243e-04*w11*Jinv22+9.1314935064935068e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-4.7666396103896100e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50-4.7666396103896100e-03*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33+2.0271915584415584e-03*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10-7.3965097402597406e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+9.1314935064935068e-05*Jinv22*w2-7.3965097402597403e-04*Jinv00*w18-7.3965097402597403e-04*w36*Jinv00-5.7528409090909093e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.0683847402597404e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-4.7666396103896100e-03*w34*Jinv11-5.7528409090909093e-04*w19*Jinv11+1.0683847402597404e-03*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.7666396103896100e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44-7.3965097402597406e-03*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13-7.3965097402597403e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21-5.7528409090909093e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00-5.7528409090909093e-04*w37*Jinv12-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+1.0683847402597404e-03*w52*Jinv12-4.7666396103896100e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11+1.0683847402597404e-03*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14-7.3965097402597406e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-7.3965097402597406e-03*w4*Jinv12+2.0271915584415584e-03*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.7666396103896100e-03*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-5.7528409090909093e-04*Jinv01*w18-5.7528409090909093e-04*w37*Jinv11-6.5746753246753243e-04*Jinv02*w9+1.3149350649350649e-03*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21-7.3965097402597403e-04*Jinv20*w38+2.0271915584415584e-03*Jinv22*w23+1.0683847402597404e-03*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.3149350649350649e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+2.0271915584415584e-03*Jinv22*w50+2.0271915584415584e-03*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7+2.0271915584415584e-03*w21*Jinv01+1.0683847402597404e-03*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+2.0271915584415584e-03*Jinv12*w49-5.7528409090909093e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+1.3149350649350649e-03*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-7.3965097402597403e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02-5.7528409090909093e-04*w19*Jinv12-7.3965097402597406e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+2.0271915584415584e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-5.7528409090909093e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8-4.7666396103896100e-03*Jinv02*w33+1.1870941558441558e-03*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+2.0271915584415584e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.1870941558441558e-03*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+2.0271915584415584e-03*w48*Jinv02-6.5746753246753243e-04*Jinv01*w9-4.9310064935064932e-04*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-7.3965097402597403e-04*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11-5.7528409090909093e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43-2.4655032467532467e-03*Jinv01*w30+1.0683847402597404e-03*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+1.0683847402597404e-03*w25*Jinv11-5.4788961038961036e-04*w42*Jinv00-4.7666396103896100e-03*w16*Jinv12+2.0271915584415584e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+2.0271915584415584e-03*w48*Jinv01+9.1314935064935068e-05*Jinv12*w1-6.5746753246753243e-04*w10*Jinv11)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(0.0)*D;
  A[3][6]=(-8.2183441558441553e-05*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+2.6298701298701297e-03*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-2.7394480519480521e-05*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+4.7483766233766234e-04*w0*Jinv02+4.7483766233766234e-04*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55+2.7394480519480518e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+4.7483766233766234e-04*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10+1.4793019480519481e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-2.7394480519480521e-05*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.7483766233766234e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+1.4793019480519481e-03*Jinv00*w33-5.2049512987012989e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.7483766233766234e-04*Jinv22*w2+1.1505681818181819e-03*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-9.5880681818181818e-04*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+2.6298701298701297e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13+1.1505681818181819e-03*w19*Jinv10+1.4793019480519481e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-2.7394480519480521e-05*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-9.5880681818181818e-04*w26*Jinv20+4.0178571428571428e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12-5.2049512987012989e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.6298701298701297e-03*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+1.1505681818181819e-03*Jinv20*w38-5.2049512987012989e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-9.5880681818181818e-04*Jinv20*w53+2.6298701298701297e-03*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+2.6298701298701297e-03*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-9.5880681818181818e-04*w52*Jinv10+2.6298701298701297e-03*w7*Jinv11+4.0178571428571428e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-9.5880681818181818e-04*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.6298701298701297e-03*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-5.2049512987012989e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.0178571428571428e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12+2.6298701298701297e-03*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00+1.4793019480519481e-03*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-9.5880681818181818e-04*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2+1.4793019480519481e-03*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+1.4793019480519481e-03*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+2.6298701298701297e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11-2.7394480519480521e-05*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-5.2049512987012989e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27-5.2049512987012989e-04*w48*Jinv01+4.7483766233766234e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[3][7]=(0.0)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(3.6525974025974026e-05*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10-1.0227272727272728e-03*Jinv20*w8+3.6525974025974026e-05*w13*Jinv10-2.5568181818181820e-04*w3*Jinv00+3.6525974025974026e-05*w32*Jinv20+1.1870941558441559e-04*Jinv20*w23-2.7394480519480521e-05*Jinv00*w54+2.9220779220779221e-04*w43*Jinv10-1.1668019480519480e-04*w58*Jinv10+1.1870941558441559e-04*w48*Jinv00+8.3198051948051951e-05*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10-2.7394480519480521e-05*w45*Jinv00-1.1668019480519480e-04*Jinv00*w27+1.1870941558441559e-04*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10-3.2873376623376621e-04*Jinv00*w33+3.6525974025974026e-05*w31*Jinv10+1.1870941558441559e-04*w21*Jinv00-6.7573051948051951e-04*Jinv00*w18-6.7573051948051951e-04*w36*Jinv00+4.9310064935064932e-04*w51*Jinv00-2.5568181818181820e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-6.7573051948051951e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-1.0227272727272728e-03*w6*Jinv00-2.7394480519480521e-05*Jinv20*w56+4.9310064935064932e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20+3.6525974025974026e-05*Jinv20*w14+1.1870941558441559e-04*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-1.0227272727272728e-03*w7*Jinv10-6.7573051948051951e-04*Jinv20*w38+4.9310064935064932e-04*Jinv20*w53+1.1870941558441559e-04*w22*Jinv10+4.9310064935064932e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-6.7573051948051951e-04*w37*Jinv10+4.9310064935064932e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10+2.9220779220779221e-04*Jinv20*w44-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59+8.3198051948051951e-05*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+4.9310064935064932e-04*w25*Jinv10+8.3198051948051951e-05*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-3.2873376623376621e-04*Jinv20*w17+3.6525974025974026e-05*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-6.7573051948051951e-04*w20*Jinv20-2.5568181818181820e-04*w5*Jinv20-2.7394480519480521e-05*w55*Jinv10+2.9220779220779221e-04*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20)*D;
  A[3][10]=(0.0)*D;
  A[3][11]=(0.0)*D;
  A[3][12]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8-3.7439123376623376e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-2.4655032467532467e-03*w3*Jinv00-1.2327516233766234e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54+3.7439123376623376e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+1.1414366883116883e-03*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10+3.9265422077922079e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-3.7439123376623376e-04*w58*Jinv10-3.1777597402597403e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12+1.0135957792207792e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35+3.7439123376623376e-04*w58*Jinv11+3.7439123376623376e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+1.1414366883116883e-03*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-3.1777597402597403e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+8.2183441558441553e-05*w45*Jinv00+5.4788961038961036e-04*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+1.1414366883116883e-03*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+3.5612824675324673e-04*w28*Jinv11+1.0135957792207792e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-3.1777597402597403e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12-1.2327516233766234e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22+5.4788961038961036e-04*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.7439123376623376e-04*Jinv22*w59+2.6846590909090908e-03*w34*Jinv11-3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11-1.5888798701298701e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+2.4655032467532467e-03*w32*Jinv21+1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-3.9265422077922079e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+2.6846590909090908e-03*Jinv21*w35+2.4655032467532467e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02+1.0135957792207792e-03*w49*Jinv10+3.5612824675324673e-04*Jinv21*w29+2.6846590909090908e-03*Jinv01*w33-1.5888798701298701e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-8.2183441558441553e-05*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9-1.3971185064935066e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.2327516233766234e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10+1.0135957792207792e-03*Jinv22*w50-1.1505681818181819e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7-1.1505681818181819e-03*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+1.0135957792207792e-03*Jinv12*w49-8.2183441558441553e-05*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-1.3971185064935066e-03*w7*Jinv11-3.9265422077922079e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+1.6436688311688311e-04*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-3.9265422077922079e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44-3.7439123376623376e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-1.3971185064935066e-03*Jinv21*w8-3.7439123376623376e-04*w57*Jinv00-3.7439123376623376e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.1505681818181819e-03*Jinv21*w23+1.6436688311688311e-04*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-3.1777597402597403e-03*w15*Jinv00+1.0135957792207792e-03*w48*Jinv02+3.9265422077922079e-04*Jinv01*w9-3.1777597402597403e-03*Jinv20*w17-1.2327516233766234e-03*w30*Jinv00-1.5888798701298701e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14+5.4788961038961036e-04*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20+2.4655032467532467e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-3.1777597402597403e-03*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50+3.5612824675324673e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1+3.9265422077922079e-04*w10*Jinv11)*D;
  A[3][13]=(0.0)*D;
  A[3][14]=(0.0)*D;
  A[3][15]=(2.0819805194805196e-03*Jinv00*w12+5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-1.1505681818181819e-03*Jinv22*w53+4.2004870129870131e-04*w57*Jinv02+2.0819805194805196e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44+9.8620129870129864e-04*w3*Jinv00+1.0409902597402598e-03*w32*Jinv20+9.5332792207792200e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23+1.3149350649350649e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54+9.6793831168831166e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+9.8620129870129864e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+6.9034090909090911e-03*w16*Jinv11+6.9034090909090911e-03*Jinv21*w17-7.6704545454545450e-04*Jinv12*w55+6.9399350649350649e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-1.2601461038961039e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+9.6793831168831166e-04*w58*Jinv11+9.6793831168831166e-04*w57*Jinv01-1.0957792207792208e-04*w0*Jinv01+4.3831168831168834e-04*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-3.2873376623376621e-04*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+4.9310064935064934e-03*w42*Jinv02+5.6615259740259744e-04*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.0819805194805196e-03*w13*Jinv11+6.9034090909090911e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11-1.2601461038961039e-03*Jinv20*w50+5.6615259740259744e-04*Jinv20*w29-3.2873376623376621e-04*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+2.9586038961038961e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12+1.0409902597402598e-03*w31*Jinv10-3.2873376623376621e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+9.8620129870129864e-04*Jinv22*w2+1.8628246753246754e-03*Jinv00*w18+9.3141233766233771e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+4.2004870129870131e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-1.2053571428571428e-03*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+4.9310064935064934e-03*Jinv22*w44+9.8620129870129864e-04*w4*Jinv10+9.5332792207792200e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20-2.1915584415584414e-03*Jinv12*w13+1.8628246753246754e-03*w19*Jinv10+2.9586038961038961e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+1.3149350649350649e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-6.0267857142857139e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20-1.1505681818181819e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+2.0819805194805196e-03*Jinv20*w14+9.5332792207792200e-03*w5*Jinv21-7.6704545454545450e-04*Jinv02*w54-3.2873376623376621e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02-1.2601461038961039e-03*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00-2.1915584415584414e-03*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-2.7942370129870131e-03*w6*Jinv01+9.3141233766233771e-04*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23-1.3149350649350649e-03*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-1.2053571428571428e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50-1.8628246753246754e-03*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-1.8628246753246754e-03*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01-1.2053571428571428e-03*w52*Jinv10-2.7942370129870131e-03*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00+9.3141233766233771e-04*w37*Jinv10-8.2183441558441553e-04*Jinv21*w56-6.0267857142857139e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-3.2873376623376621e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22-2.5750811688311690e-03*w49*Jinv11+2.0819805194805196e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44+4.2004870129870131e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18+2.0819805194805196e-03*Jinv01*w12-2.7942370129870131e-03*Jinv21*w8+6.9399350649350649e-04*w57*Jinv00+6.9399350649350649e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+4.3831168831168834e-04*w0*Jinv00+2.9586038961038961e-03*w35*Jinv20-1.8628246753246754e-03*Jinv21*w23-8.2183441558441553e-04*Jinv01*w54-8.2183441558441553e-04*w55*Jinv11-6.0267857142857139e-04*w25*Jinv10+4.3831168831168834e-04*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+5.9172077922077923e-03*Jinv20*w17+1.0409902597402598e-03*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10+1.8628246753246754e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-2.1915584415584414e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+1.3149350649350649e-03*Jinv02*w36+4.9310064935064934e-03*Jinv12*w43+9.8620129870129864e-04*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30-1.1505681818181819e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28-8.2183441558441553e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-3.2873376623376621e-04*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-2.5750811688311690e-03*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27-2.5750811688311690e-03*w48*Jinv01+9.8620129870129864e-04*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[3][16]=(0.0)*D;
  A[3][17]=(0.0)*D;
  A[3][18]=(3.8352272727272725e-04*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10-1.2327516233766234e-03*Jinv20*w8+3.8352272727272725e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00+1.9176136363636362e-04*w32*Jinv20+1.1505681818181819e-03*Jinv01*w3+1.3697240259740259e-04*Jinv20*w23+4.6570616883116886e-04*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-4.7483766233766234e-04*Jinv21*w2-2.5202922077922079e-03*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.1960227272727273e-04*w58*Jinv10+2.4655032467532466e-04*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26-4.7483766233766234e-04*w0*Jinv01-4.8396915584415583e-04*w1*Jinv10-2.7394480519480520e-03*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.7349837662337662e-04*Jinv00*w27-4.7483766233766234e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+1.9176136363636362e-04*w31*Jinv10+1.3697240259740259e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-2.0271915584415584e-03*Jinv00*w18-1.0135957792207792e-03*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+1.3971185064935066e-03*Jinv01*w24-2.1367694805194807e-03*w34*Jinv11-2.3833198051948050e-03*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+8.4922889610389610e-04*w51*Jinv00+1.4793019480519481e-03*w4*Jinv10+1.1505681818181819e-03*w4*Jinv11-7.1225649350649346e-04*w41*Jinv20-2.0271915584415584e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21-2.3833198051948050e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00+4.6570616883116886e-04*Jinv20*w56+4.3831168831168834e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.1367694805194807e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11+3.8352272727272725e-04*Jinv20*w14+1.1505681818181819e-03*w5*Jinv21+2.4655032467532466e-04*w49*Jinv10-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-7.1225649350649346e-04*w39*Jinv00-2.3833198051948050e-03*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-1.0135957792207792e-03*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+8.4922889610389610e-04*Jinv20*w53+1.3697240259740259e-04*w22*Jinv10+5.2049512987012989e-04*w22*Jinv11+5.2049512987012989e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+8.4922889610389610e-04*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-1.0135957792207792e-03*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-2.5202922077922079e-03*Jinv20*w44-1.6436688311688311e-04*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-3.1960227272727273e-04*w57*Jinv00-3.1960227272727273e-04*Jinv20*w59-4.8396915584415583e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20+5.2049512987012989e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-4.8396915584415583e-04*Jinv20*w2-2.7394480519480520e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-2.7394480519480520e-03*Jinv20*w17+1.9176136363636362e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-2.0271915584415584e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+1.4793019480519481e-03*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30+1.3971185064935066e-03*w25*Jinv11+4.6570616883116886e-04*w55*Jinv10-2.5202922077922079e-03*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[3][19]=(0.0)*D;
  A[3][20]=(0.0)*D;
  A[3][21]=(1.6436688311688311e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-4.0543831168831168e-03*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54-3.1960227272727273e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-5.5702110389610384e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-8.7662337662337668e-04*w16*Jinv11-8.7662337662337668e-04*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-3.1960227272727273e-04*w58*Jinv11-3.1960227272727273e-04*w57*Jinv01+5.4788961038961036e-04*Jinv21*w26-5.5702110389610384e-04*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-5.5702110389610384e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11-8.7662337662337668e-04*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+7.1225649350649346e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+5.4788961038961036e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59-2.5750811688311690e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.6570616883116886e-04*Jinv21*w53-8.2183441558441553e-05*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10-4.0543831168831168e-03*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-2.0271915584415584e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20-8.2183441558441553e-05*w52*Jinv12-2.5750811688311690e-03*Jinv21*w35-2.0271915584415584e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-4.0543831168831168e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+7.1225649350649346e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29-2.5750811688311690e-03*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9+9.5880681818181818e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+4.6570616883116886e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-8.2183441558441553e-05*Jinv20*w53-5.2049512987012989e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+7.3965097402597403e-04*w22*Jinv11-5.2049512987012989e-04*Jinv12*w7+7.3965097402597403e-04*w21*Jinv01+4.6570616883116886e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01-8.2183441558441553e-05*w52*Jinv10+9.5880681818181818e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10-2.7394480519480521e-05*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-5.2049512987012989e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12+7.1225649350649346e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.1231737012987013e-03*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+9.5880681818181818e-04*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.3965097402597403e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.7349837662337662e-04*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20-2.0271915584415584e-03*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+5.4788961038961036e-04*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12+1.1231737012987013e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+1.1231737012987013e-03*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-1.7349837662337662e-04*w10*Jinv11)*D;
  A[3][22]=(0.0)*D;
  A[3][23]=(0.0)*D;
  A[3][24]=(-1.6436688311688311e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+5.2049512987012989e-04*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23+2.7394480519480518e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+1.8262987012987013e-05*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-1.0957792207792208e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-1.0683847402597404e-03*Jinv21*w26+1.8262987012987013e-05*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27+1.8262987012987013e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+5.2049512987012989e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+1.3971185064935066e-03*Jinv21*w38-1.0683847402597404e-03*Jinv01*w24-1.6436688311688311e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11-3.8352272727272725e-04*w41*Jinv20+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+5.2049512987012989e-04*w6*Jinv00+2.7394480519480518e-04*Jinv20*w56-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.6436688311688311e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-3.8352272727272725e-04*w39*Jinv00+8.2183441558441553e-05*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11+1.4793019480519481e-03*w6*Jinv01+5.2049512987012989e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38+2.4655032467532466e-04*w52*Jinv11+8.2183441558441553e-05*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51+1.3971185064935066e-03*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10+1.4793019480519481e-03*w7*Jinv11-1.3697240259740259e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+4.6570616883116886e-04*Jinv21*w56-7.1225649350649346e-04*w24*Jinv00+5.4788961038961036e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-1.3697240259740259e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+3.8352272727272725e-04*Jinv01*w12+1.4793019480519481e-03*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+4.6570616883116886e-04*Jinv01*w54+4.6570616883116886e-04*w55*Jinv11-7.1225649350649346e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-1.0683847402597404e-03*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+5.4788961038961036e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[3][25]=(0.0)*D;
  A[3][26]=(0.0)*D;
  A[3][27]=(3.4699675324675324e-04*Jinv21*w44+1.4975649350649350e-03*Jinv01*w3+1.0247564935064935e-04*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2+3.4699675324675324e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+3.4699675324675324e-04*w43*Jinv11+2.1915584415584417e-04*Jinv21*w41+6.4935064935064935e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+2.1915584415584417e-04*Jinv01*w39-4.2918019480519480e-04*Jinv21*w38-1.0957792207792208e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53+1.4975649350649350e-03*w4*Jinv11+7.4878246753246752e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35+7.4878246753246752e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11+1.4975649350649350e-03*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+1.1870941558441559e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-5.8441558441558442e-04*w6*Jinv01-5.4788961038961042e-05*w47*Jinv21-2.1915584415584417e-04*w52*Jinv11-2.6481331168831169e-04*w22*Jinv11-2.6481331168831169e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-5.8441558441558442e-04*w7*Jinv11-2.7394480519480521e-05*Jinv21*w56-4.2918019480519480e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+3.6525974025974026e-05*Jinv01*w12-5.8441558441558442e-04*Jinv21*w8-2.6481331168831169e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11-1.4204545454545455e-05*Jinv01*w9+2.1915584415584417e-04*w40*Jinv11+7.4878246753246752e-04*Jinv01*w30-1.0957792207792208e-04*w25*Jinv11-4.2918019480519480e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[3][28]=(0.0)*D;
  A[3][29]=(0.0)*D;
  A[3][30]=(-1.2327516233766234e-03*Jinv00*w12-3.7439123376623376e-04*w28*Jinv10+5.4788961038961036e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8+3.5612824675324673e-04*w57*Jinv02-1.2327516233766234e-03*w13*Jinv10+5.4788961038961036e-04*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3+1.0135957792207792e-03*Jinv20*w23+8.2183441558441553e-05*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+1.1414366883116883e-03*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10+2.6846590909090908e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-3.7439123376623376e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.7439123376623376e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.7439123376623376e-04*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.6846590909090908e-03*Jinv22*w17-3.1777597402597403e-03*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3+1.0135957792207792e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+1.1414366883116883e-03*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+3.5612824675324673e-04*Jinv01*w24+3.5612824675324673e-04*Jinv22*w59-3.1777597402597403e-03*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00+1.0957792207792207e-03*Jinv12*w34-2.4655032467532467e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11-1.5888798701298701e-03*w41*Jinv20+2.4655032467532467e-03*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-3.1777597402597403e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*Jinv20*w56+5.4788961038961036e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-3.9265422077922079e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-3.1777597402597403e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11-1.2327516233766234e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10-3.7439123376623376e-04*Jinv21*w29-3.1777597402597403e-03*Jinv01*w33-1.5888798701298701e-03*w39*Jinv00+2.4655032467532467e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+1.6436688311688311e-04*Jinv20*w53-1.3971185064935066e-03*Jinv22*w8+1.0135957792207792e-03*w22*Jinv10-1.1505681818181819e-03*Jinv22*w50+1.0135957792207792e-03*w22*Jinv11-1.3971185064935066e-03*Jinv12*w7+1.0135957792207792e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.1505681818181819e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+1.2327516233766234e-03*w7*Jinv11-3.9265422077922079e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00+3.7439123376623376e-04*Jinv02*w27-1.3971185064935066e-03*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5+1.2327516233766234e-03*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14-3.9265422077922079e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44+3.5612824675324673e-04*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59+1.0957792207792207e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-3.1777597402597403e-03*w35*Jinv20+1.0135957792207792e-03*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.1505681818181819e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-1.5888798701298701e-03*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20+3.7439123376623376e-04*Jinv22*w29+2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11-2.4655032467532467e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+3.7439123376623376e-04*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.6846590909090908e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.7439123376623376e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+1.1414366883116883e-03*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[3][31]=(0.0)*D;
  A[3][32]=(0.0)*D;
  A[3][33]=(1.0409902597402598e-03*Jinv00*w12+6.9399350649350649e-04*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+6.2094155844155847e-04*w57*Jinv02+1.0409902597402598e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+4.9310064935064934e-03*Jinv21*w44+9.8620129870129864e-04*w3*Jinv00+2.0819805194805196e-03*w32*Jinv20-3.2873376623376621e-03*Jinv01*w3-1.2601461038961039e-03*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-3.2873376623376621e-04*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59-1.3149350649350649e-03*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02+9.8620129870129864e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10+4.9310064935064934e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45+5.6615259740259744e-04*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+6.9034090909090911e-03*Jinv22*w35+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01-1.1505681818181819e-03*Jinv21*w26+9.8620129870129864e-04*w0*Jinv01+4.3831168831168834e-04*w1*Jinv10+2.9586038961038961e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20+4.9310064935064934e-03*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02+6.9399350649350649e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+9.8620129870129864e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+4.2004870129870131e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+6.9399350649350649e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33-2.5750811688311690e-03*w22*Jinv12+2.0819805194805196e-03*w31*Jinv10+9.5332792207792200e-03*Jinv02*w3-1.2601461038961039e-03*w21*Jinv00+2.0819805194805196e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2+9.3141233766233771e-04*Jinv00*w18+1.8628246753246754e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-8.2183441558441553e-04*Jinv02*w45-1.1505681818181819e-03*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53-6.0267857142857139e-04*w51*Jinv00+6.9034090909090911e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44+9.8620129870129864e-04*w4*Jinv10-3.2873376623376621e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+9.3141233766233771e-04*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10-2.1915584415584414e-03*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41-1.2053571428571428e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+3.9448051948051946e-03*Jinv21*w35-2.1915584415584414e-03*w31*Jinv11-1.3149350649350649e-03*Jinv22*w26-7.6704545454545450e-04*w46*Jinv11+1.0409902597402598e-03*Jinv20*w14-3.2873376623376621e-03*w5*Jinv21+9.5332792207792200e-03*w4*Jinv12-2.5750811688311690e-03*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10+4.2004870129870131e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01-7.6704545454545450e-04*w47*Jinv21+1.8628246753246754e-03*Jinv20*w38-2.5750811688311690e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11+2.0819805194805196e-03*w31*Jinv12-8.2183441558441553e-04*w46*Jinv12-6.0267857142857139e-04*Jinv20*w53-2.7942370129870131e-03*Jinv22*w8-1.2601461038961039e-03*w22*Jinv10-1.8628246753246754e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-2.7942370129870131e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51+2.0819805194805196e-03*Jinv02*w30-1.8628246753246754e-03*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01-6.0267857142857139e-04*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00+1.8628246753246754e-03*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56-1.2053571428571428e-03*w24*Jinv00+9.6793831168831166e-04*Jinv02*w27-2.7942370129870131e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12+9.5332792207792200e-03*Jinv22*w5-8.2183441558441553e-04*w47*Jinv22+7.6704545454545450e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+5.6615259740259744e-04*w57*Jinv00+5.6615259740259744e-04*Jinv20*w59+6.9034090909090911e-03*Jinv02*w33+4.3831168831168834e-04*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-1.2053571428571428e-03*w25*Jinv10+4.3831168831168834e-04*Jinv20*w2+2.9586038961038961e-03*w15*Jinv00-1.8628246753246754e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+2.9586038961038961e-03*Jinv20*w17+2.0819805194805196e-03*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10+9.3141233766233771e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43+9.8620129870129864e-04*w5*Jinv20-2.1915584415584414e-03*Jinv01*w30+9.6793831168831166e-04*Jinv12*w28-1.1505681818181819e-03*w25*Jinv11-3.2873376623376621e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50+4.2004870129870131e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[3][34]=(0.0)*D;
  A[3][35]=(0.0)*D;
  A[3][36]=(1.9176136363636362e-04*Jinv00*w12-3.1960227272727273e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-4.2918019480519480e-04*w57*Jinv02+1.9176136363636362e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+1.4793019480519481e-03*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20+2.4655032467532466e-04*Jinv20*w23-2.3833198051948050e-03*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-4.7483766233766234e-04*w0*Jinv02-2.5202922077922079e-03*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+1.3697240259740259e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-4.8396915584415583e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+4.6570616883116886e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-3.1960227272727273e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+1.3697240259740259e-04*Jinv20*w50-3.1960227272727273e-04*Jinv20*w29+4.6570616883116886e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-2.7394480519480520e-03*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10+1.1505681818181819e-03*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00-1.6436688311688311e-04*w32*Jinv22-4.7483766233766234e-04*Jinv22*w2-1.0135957792207792e-03*Jinv00*w18-2.0271915584415584e-03*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45-4.2918019480519480e-04*Jinv22*w59+4.3831168831168834e-04*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10-7.1225649350649346e-04*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13-1.0135957792207792e-03*w19*Jinv10-2.7394480519480520e-03*w34*Jinv10-1.2327516233766234e-03*w6*Jinv00-2.3833198051948050e-03*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+8.4922889610389610e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+1.9176136363636362e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54+1.1505681818181819e-03*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02+1.3697240259740259e-04*w49*Jinv10-7.1225649350649346e-04*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.2327516233766234e-03*w7*Jinv10-2.0271915584415584e-03*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23-1.6436688311688311e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10+5.2049512987012989e-04*Jinv22*w50-2.6298701298701297e-03*Jinv12*w7-1.6436688311688311e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+4.3831168831168834e-04*w52*Jinv10+1.1870941558441559e-04*w9*Jinv00-2.0271915584415584e-03*w37*Jinv10+8.4922889610389610e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.1505681818181819e-03*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+1.1870941558441559e-04*w10*Jinv10-2.5202922077922079e-03*Jinv20*w44-4.2918019480519480e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-4.8396915584415583e-04*w0*Jinv00-2.7394480519480520e-03*w35*Jinv20+8.4922889610389610e-04*w25*Jinv10-4.8396915584415583e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00+5.2049512987012989e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-1.0135957792207792e-03*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-2.3833198051948050e-03*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+1.3971185064935066e-03*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28+2.4655032467532466e-04*w55*Jinv10-2.5202922077922079e-03*w42*Jinv00+4.6570616883116886e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-4.7483766233766234e-04*Jinv12*w1)*D;
  A[3][37]=(0.0)*D;
  A[3][38]=(0.0)*D;
  A[3][39]=(-1.0957792207792207e-03*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+3.8352272727272725e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-1.0957792207792207e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.0957792207792207e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+3.2873376623376621e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+3.6525974025974026e-05*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01-5.4788961038961042e-05*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.0409902597402598e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.7532467532467534e-03*w22*Jinv12-1.0957792207792207e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00+1.0409902597402598e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+3.1047077922077924e-04*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+3.2873376623376621e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-1.0957792207792207e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-1.7532467532467534e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9-1.7532467532467534e-03*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+5.4788961038961042e-05*Jinv20*w38-1.7532467532467534e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+1.0409902597402598e-03*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53-1.7532467532467534e-03*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-9.3141233766233771e-04*Jinv22*w50-9.3141233766233771e-04*w22*Jinv11-1.7532467532467534e-03*Jinv12*w7-9.3141233766233771e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+1.0409902597402598e-03*Jinv02*w30-9.3141233766233771e-04*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.7532467532467534e-03*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-3.2873376623376621e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-1.7532467532467534e-03*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22-1.7532467532467534e-03*w49*Jinv11+1.0409902597402598e-03*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+3.2873376623376621e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18+1.0409902597402598e-03*Jinv01*w12-1.7532467532467534e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-9.3141233766233771e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-9.3141233766233771e-04*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-1.0957792207792207e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30+5.2962662337662338e-04*Jinv12*w28+5.4788961038961042e-05*w55*Jinv10+3.2873376623376621e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12-1.7532467532467534e-03*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-1.7532467532467534e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[3][40]=(0.0)*D;
  A[3][41]=(0.0)*D;
  A[3][42]=(1.0957792207792207e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-4.2004870129870131e-04*w57*Jinv02+1.0957792207792207e-03*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+1.0957792207792207e-03*w32*Jinv20+3.2873376623376621e-03*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-9.8620129870129864e-04*w0*Jinv02-9.8620129870129864e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-3.9448051948051946e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-9.8620129870129864e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-9.8620129870129864e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-4.2004870129870131e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10-3.9448051948051946e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12+1.0957792207792207e-03*w31*Jinv10+3.2873376623376621e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-9.8620129870129864e-04*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45+1.1505681818181819e-03*Jinv01*w24-4.2004870129870131e-04*Jinv22*w59-3.9448051948051946e-03*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+3.2873376623376621e-03*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20+2.1915584415584414e-03*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+2.1915584415584414e-03*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12-3.9448051948051946e-03*Jinv21*w35+2.1915584415584414e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26+7.6704545454545450e-04*w46*Jinv11+1.0957792207792207e-03*Jinv20*w14+3.2873376623376621e-03*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+3.2873376623376621e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-4.2004870129870131e-04*Jinv21*w29-3.9448051948051946e-03*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00+2.1915584415584414e-03*Jinv02*w12-1.3149350649350649e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11-3.6525974025974026e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+3.2873376623376621e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-3.6525974025974026e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-4.2004870129870131e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+1.0957792207792207e-03*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29+2.1915584415584414e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-1.3149350649350649e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+2.1915584415584414e-03*Jinv01*w30+1.1505681818181819e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28+1.1505681818181819e-03*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20-3.9448051948051946e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50-4.2004870129870131e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01-9.8620129870129864e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[3][43]=(0.0)*D;
  A[3][44]=(0.0)*D;
  A[3][45]=(-3.8352272727272725e-04*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53-1.0957792207792208e-04*w57*Jinv02-3.8352272727272725e-04*Jinv02*w39-8.7662337662337668e-04*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-1.3697240259740259e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-8.7662337662337668e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-7.6704545454545450e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+5.2049512987012989e-04*Jinv22*w20-8.7662337662337668e-04*w43*Jinv11-7.1225649350649346e-04*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1+1.9176136363636362e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.6704545454545450e-04*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-3.5612824675324673e-04*w32*Jinv22-7.1225649350649346e-04*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+5.4788961038961036e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-7.1225649350649346e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-3.8352272727272725e-04*Jinv22*w41+8.2183441558441553e-05*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01-3.5612824675324673e-04*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11+2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+5.2049512987012989e-04*w49*Jinv11+1.9176136363636362e-04*Jinv21*w14-1.0957792207792208e-04*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+1.9176136363636362e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-8.2183441558441553e-05*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43-7.1225649350649346e-04*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-7.6704545454545450e-04*w16*Jinv12+5.2049512987012989e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[3][46]=(0.0)*D;
  A[3][47]=(0.0)*D;
  A[3][48]=(3.5612824675324673e-04*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+5.4788961038961036e-04*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+7.1225649350649346e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+4.6570616883116886e-04*w25*Jinv12-5.5702110389610384e-04*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-2.5750811688311690e-03*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+4.6570616883116886e-04*Jinv02*w24-8.7662337662337668e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10-2.5750811688311690e-03*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33+1.1231737012987013e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10-4.0543831168831168e-03*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-5.5702110389610384e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38-2.7394480519480521e-05*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-8.7662337662337668e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+7.1225649350649346e-04*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20-2.0271915584415584e-03*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41-8.2183441558441553e-05*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+5.4788961038961036e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+4.6570616883116886e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+7.1225649350649346e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54-4.0543831168831168e-03*w4*Jinv12+1.1231737012987013e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00-2.0271915584415584e-03*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11-1.7349837662337662e-04*Jinv02*w9-5.2049512987012989e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+1.1231737012987013e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+9.5880681818181818e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10+7.3965097402597403e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+9.5880681818181818e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30+7.3965097402597403e-04*Jinv12*w49-5.2049512987012989e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-5.2049512987012989e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-8.2183441558441553e-05*w24*Jinv00-3.1960227272727273e-04*Jinv02*w27+9.5880681818181818e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12-4.0543831168831168e-03*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-5.2049512987012989e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-8.7662337662337668e-04*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-8.2183441558441553e-05*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.3965097402597403e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-3.1960227272727273e-04*Jinv22*w29-2.0271915584415584e-03*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+5.4788961038961036e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-3.1960227272727273e-04*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20-2.5750811688311690e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-5.5702110389610384e-04*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[3][49]=(0.0)*D;
  A[3][50]=(0.0)*D;
  A[3][51]=(-3.5612824675324673e-04*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-1.0683847402597404e-03*Jinv22*w53+5.2049512987012989e-04*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+2.4655032467532466e-04*w25*Jinv12+1.8262987012987013e-05*w0*Jinv02-5.4788961038961042e-05*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+1.3971185064935066e-03*Jinv22*w20+2.7394480519480518e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+1.8262987012987013e-05*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+4.6570616883116886e-04*Jinv02*w45+1.2784090909090910e-04*Jinv22*w59-7.1225649350649346e-04*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10-3.8352272727272725e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10+5.2049512987012989e-04*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.0683847402597404e-03*w52*Jinv12+2.4655032467532466e-04*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10-3.8352272727272725e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9+5.2049512987012989e-04*w7*Jinv10+5.2049512987012989e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+3.8352272727272725e-04*w31*Jinv12+4.6570616883116886e-04*w46*Jinv12-7.1225649350649346e-04*Jinv20*w53+1.4793019480519481e-03*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50+1.4793019480519481e-03*Jinv12*w7+3.8352272727272725e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-7.1225649350649346e-04*w52*Jinv10-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+8.2183441558441553e-05*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+1.4793019480519481e-03*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5+4.6570616883116886e-04*w47*Jinv22-1.3697240259740259e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+8.2183441558441553e-05*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+8.2183441558441553e-05*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20-1.0683847402597404e-03*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-3.5612824675324673e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12+1.8262987012987013e-05*Jinv12*w1)*D;
  A[3][52]=(0.0)*D;
  A[3][53]=(0.0)*D;
  A[3][54]=(-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+6.3920454545454550e-05*w57*Jinv02-7.1225649350649346e-04*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+2.4655032467532466e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12+5.4788961038961036e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+5.4788961038961036e-04*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11-3.8352272727272725e-04*Jinv21*w41-8.7662337662337668e-04*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-1.0957792207792208e-04*Jinv22*w17+5.2049512987012989e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3+1.9176136363636362e-04*w32*Jinv22-3.8352272727272725e-04*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+5.2049512987012989e-04*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+6.3920454545454550e-05*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-8.7662337662337668e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+2.7394480519480518e-04*w47*Jinv21+5.2049512987012989e-04*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+1.9176136363636362e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8-8.2183441558441553e-05*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+1.9176136363636362e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+5.2049512987012989e-04*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-3.5612824675324673e-04*Jinv21*w56-1.3697240259740259e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+6.3920454545454550e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-8.2183441558441553e-05*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-3.8352272727272725e-04*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36-8.7662337662337668e-04*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-1.0957792207792208e-04*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[3][55]=(0.0)*D;
  A[3][56]=(0.0)*D;
  A[3][57]=(2.1915584415584417e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+2.1915584415584417e-04*Jinv02*w39+1.1870941558441559e-04*Jinv22*w38-2.1915584415584417e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-5.4788961038961042e-05*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12-2.1915584415584417e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35-4.2918019480519480e-04*Jinv22*w20+3.4699675324675324e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12+1.4975649350649350e-03*Jinv02*w3+3.6525974025974026e-05*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2-2.7394480519480521e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+4.3831168831168834e-04*Jinv12*w34+3.4699675324675324e-04*Jinv22*w44+7.4878246753246752e-04*Jinv12*w13+1.1870941558441559e-04*w37*Jinv12+2.1915584415584417e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12-2.1915584415584417e-04*Jinv22*w26-5.4788961038961042e-05*Jinv02*w54+1.4975649350649350e-03*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02+7.4878246753246752e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-4.2918019480519480e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12-5.8441558441558442e-04*Jinv22*w8-2.6481331168831169e-04*Jinv22*w50-5.8441558441558442e-04*Jinv12*w7+3.6525974025974026e-05*Jinv02*w30-2.6481331168831169e-04*Jinv12*w49+1.0247564935064935e-04*Jinv02*w27-5.8441558441558442e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+1.4975649350649350e-03*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+4.3831168831168834e-04*Jinv02*w33-2.6481331168831169e-04*w48*Jinv02+1.0247564935064935e-04*Jinv22*w29+7.4878246753246752e-04*Jinv22*w14+1.1870941558441559e-04*Jinv02*w36+3.4699675324675324e-04*Jinv12*w43-1.0957792207792208e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[3][58]=(0.0)*D;
  A[3][59]=(0.0)*D;
  A[4][0]=(0.0)*D;
  A[4][1]=(-1.1870941558441558e-03*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10-3.2873376623376621e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+1.0227272727272728e-03*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02-1.1870941558441558e-03*w13*Jinv10-3.2873376623376621e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-2.3741883116883116e-03*w3*Jinv00-1.1870941558441558e-03*w32*Jinv20-2.3741883116883116e-03*Jinv01*w3+6.7573051948051951e-04*Jinv20*w23+2.6481331168831169e-04*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59+1.0957792207792208e-04*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-2.0495129870129870e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10-6.5746753246753243e-04*w15*Jinv02-2.0495129870129870e-04*w10*Jinv12+6.7573051948051951e-04*w48*Jinv00+1.0957792207792208e-04*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+2.6481331168831169e-04*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00-3.2873376623376621e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-2.0495129870129870e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-1.1870941558441558e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+6.7573051948051951e-04*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-6.5746753246753243e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+6.7573051948051951e-04*w22*Jinv12-1.1870941558441558e-03*w31*Jinv10-2.3741883116883116e-03*Jinv02*w3+6.7573051948051951e-04*w21*Jinv00-1.1870941558441558e-03*w32*Jinv22-3.2873376623376621e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+2.6481331168831169e-04*Jinv00*w18+2.6481331168831169e-04*w36*Jinv00+2.6481331168831169e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-6.5746753246753243e-04*w34*Jinv11+2.6481331168831169e-04*w19*Jinv11+1.0957792207792208e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-2.3741883116883116e-03*w4*Jinv10-2.3741883116883116e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-1.1870941558441558e-03*Jinv12*w13+2.6481331168831169e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.1870941558441558e-03*w32*Jinv21+2.6481331168831169e-04*Jinv21*w20+1.0227272727272728e-03*w6*Jinv00+2.6481331168831169e-04*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56-3.2873376623376621e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.0495129870129870e-04*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-6.5746753246753243e-04*Jinv21*w35-1.1870941558441558e-03*w31*Jinv11+1.0957792207792208e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-1.1870941558441558e-03*Jinv20*w14-2.3741883116883116e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54-2.3741883116883116e-03*w4*Jinv12+6.7573051948051951e-04*w21*Jinv02+6.7573051948051951e-04*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-6.5746753246753243e-04*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-1.1870941558441558e-03*Jinv02*w12+2.6481331168831169e-04*Jinv01*w18+2.6481331168831169e-04*w37*Jinv11-2.0495129870129870e-04*Jinv02*w9+1.0227272727272728e-03*w6*Jinv01+1.0227272727272728e-03*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21+2.6481331168831169e-04*Jinv20*w38+6.7573051948051951e-04*Jinv22*w23+1.0957792207792208e-04*w52*Jinv11-1.1870941558441558e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.0227272727272728e-03*Jinv22*w8+6.7573051948051951e-04*w22*Jinv10+6.7573051948051951e-04*Jinv22*w50+6.7573051948051951e-04*w22*Jinv11+1.0227272727272728e-03*Jinv12*w7+6.7573051948051951e-04*w21*Jinv01+1.0957792207792208e-04*Jinv01*w51-1.1870941558441558e-03*Jinv02*w30+6.7573051948051951e-04*Jinv12*w49+2.6481331168831169e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.0227272727272728e-03*w7*Jinv11-2.0495129870129870e-04*w9*Jinv00+2.6481331168831169e-04*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+1.0227272727272728e-03*w6*Jinv02+2.6481331168831169e-04*w19*Jinv12-2.3741883116883116e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+6.7573051948051951e-04*w49*Jinv11-1.1870941558441558e-03*Jinv21*w14-2.0495129870129870e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+2.6481331168831169e-04*Jinv02*w18-1.1870941558441558e-03*Jinv01*w12+1.0227272727272728e-03*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+6.7573051948051951e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+6.7573051948051951e-04*w48*Jinv02-2.0495129870129870e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.1870941558441558e-03*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+2.6481331168831169e-04*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-1.1870941558441558e-03*Jinv22*w14-3.2873376623376621e-04*w40*Jinv11+2.6481331168831169e-04*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-2.3741883116883116e-03*w5*Jinv20-1.1870941558441558e-03*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-6.5746753246753243e-04*w16*Jinv12+6.7573051948051951e-04*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+6.7573051948051951e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-2.0495129870129870e-04*w10*Jinv11)*D;
  A[4][2]=(0.0)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(1.2327516233766234e-03*Jinv00*w12-1.5888798701298701e-03*w40*Jinv12+1.0683847402597404e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+1.2327516233766234e-03*w32*Jinv20-7.3965097402597406e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-5.7528409090909093e-04*Jinv22*w38-7.4878246753246752e-04*Jinv21*w59+1.0683847402597404e-03*w25*Jinv12+9.1314935064935068e-05*w0*Jinv02+9.1314935064935068e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-6.5746753246753243e-04*w11*Jinv21-4.7666396103896100e-03*w16*Jinv11-4.7666396103896100e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-4.7666396103896100e-03*w15*Jinv02-6.5746753246753243e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+1.0683847402597404e-03*Jinv02*w24-4.7666396103896100e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+1.0683847402597404e-03*Jinv21*w26+9.1314935064935068e-05*w0*Jinv01+1.1870941558441558e-03*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10-5.7528409090909093e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-1.5888798701298701e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-6.5746753246753243e-04*w11*Jinv22+9.1314935064935068e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-4.7666396103896100e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50-4.7666396103896100e-03*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33+2.0271915584415584e-03*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10-7.3965097402597406e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+9.1314935064935068e-05*Jinv22*w2-7.3965097402597403e-04*Jinv00*w18-7.3965097402597403e-04*w36*Jinv00-5.7528409090909093e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.0683847402597404e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-4.7666396103896100e-03*w34*Jinv11-5.7528409090909093e-04*w19*Jinv11+1.0683847402597404e-03*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.7666396103896100e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44-7.3965097402597406e-03*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13-7.3965097402597403e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21-5.7528409090909093e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00-5.7528409090909093e-04*w37*Jinv12-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+1.0683847402597404e-03*w52*Jinv12-4.7666396103896100e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11+1.0683847402597404e-03*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14-7.3965097402597406e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-7.3965097402597406e-03*w4*Jinv12+2.0271915584415584e-03*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.7666396103896100e-03*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-5.7528409090909093e-04*Jinv01*w18-5.7528409090909093e-04*w37*Jinv11-6.5746753246753243e-04*Jinv02*w9+1.3149350649350649e-03*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21-7.3965097402597403e-04*Jinv20*w38+2.0271915584415584e-03*Jinv22*w23+1.0683847402597404e-03*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.3149350649350649e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+2.0271915584415584e-03*Jinv22*w50+2.0271915584415584e-03*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7+2.0271915584415584e-03*w21*Jinv01+1.0683847402597404e-03*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+2.0271915584415584e-03*Jinv12*w49-5.7528409090909093e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+1.3149350649350649e-03*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-7.3965097402597403e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02-5.7528409090909093e-04*w19*Jinv12-7.3965097402597406e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+2.0271915584415584e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-5.7528409090909093e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8-4.7666396103896100e-03*Jinv02*w33+1.1870941558441558e-03*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+2.0271915584415584e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.1870941558441558e-03*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+2.0271915584415584e-03*w48*Jinv02-6.5746753246753243e-04*Jinv01*w9-4.9310064935064932e-04*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-7.3965097402597403e-04*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11-5.7528409090909093e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43-2.4655032467532467e-03*Jinv01*w30+1.0683847402597404e-03*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+1.0683847402597404e-03*w25*Jinv11-5.4788961038961036e-04*w42*Jinv00-4.7666396103896100e-03*w16*Jinv12+2.0271915584415584e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+2.0271915584415584e-03*w48*Jinv01+9.1314935064935068e-05*Jinv12*w1-6.5746753246753243e-04*w10*Jinv11)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(0.0)*D;
  A[4][7]=(-8.2183441558441553e-05*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+2.6298701298701297e-03*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-2.7394480519480521e-05*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+4.7483766233766234e-04*w0*Jinv02+4.7483766233766234e-04*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55+2.7394480519480518e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+4.7483766233766234e-04*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10+1.4793019480519481e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-2.7394480519480521e-05*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.7483766233766234e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+1.4793019480519481e-03*Jinv00*w33-5.2049512987012989e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.7483766233766234e-04*Jinv22*w2+1.1505681818181819e-03*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-9.5880681818181818e-04*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+2.6298701298701297e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13+1.1505681818181819e-03*w19*Jinv10+1.4793019480519481e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-2.7394480519480521e-05*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-9.5880681818181818e-04*w26*Jinv20+4.0178571428571428e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12-5.2049512987012989e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.6298701298701297e-03*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+1.1505681818181819e-03*Jinv20*w38-5.2049512987012989e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-9.5880681818181818e-04*Jinv20*w53+2.6298701298701297e-03*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+2.6298701298701297e-03*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-9.5880681818181818e-04*w52*Jinv10+2.6298701298701297e-03*w7*Jinv11+4.0178571428571428e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-9.5880681818181818e-04*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.6298701298701297e-03*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-5.2049512987012989e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.0178571428571428e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12+2.6298701298701297e-03*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00+1.4793019480519481e-03*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-9.5880681818181818e-04*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2+1.4793019480519481e-03*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+1.4793019480519481e-03*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+2.6298701298701297e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11-2.7394480519480521e-05*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-5.2049512987012989e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27-5.2049512987012989e-04*w48*Jinv01+4.7483766233766234e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[4][8]=(0.0)*D;
  A[4][9]=(0.0)*D;
  A[4][10]=(3.6525974025974026e-05*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10-1.0227272727272728e-03*Jinv20*w8+3.6525974025974026e-05*w13*Jinv10-2.5568181818181820e-04*w3*Jinv00+3.6525974025974026e-05*w32*Jinv20+1.1870941558441559e-04*Jinv20*w23-2.7394480519480521e-05*Jinv00*w54+2.9220779220779221e-04*w43*Jinv10-1.1668019480519480e-04*w58*Jinv10+1.1870941558441559e-04*w48*Jinv00+8.3198051948051951e-05*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10-2.7394480519480521e-05*w45*Jinv00-1.1668019480519480e-04*Jinv00*w27+1.1870941558441559e-04*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10-3.2873376623376621e-04*Jinv00*w33+3.6525974025974026e-05*w31*Jinv10+1.1870941558441559e-04*w21*Jinv00-6.7573051948051951e-04*Jinv00*w18-6.7573051948051951e-04*w36*Jinv00+4.9310064935064932e-04*w51*Jinv00-2.5568181818181820e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-6.7573051948051951e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-1.0227272727272728e-03*w6*Jinv00-2.7394480519480521e-05*Jinv20*w56+4.9310064935064932e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20+3.6525974025974026e-05*Jinv20*w14+1.1870941558441559e-04*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-1.0227272727272728e-03*w7*Jinv10-6.7573051948051951e-04*Jinv20*w38+4.9310064935064932e-04*Jinv20*w53+1.1870941558441559e-04*w22*Jinv10+4.9310064935064932e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-6.7573051948051951e-04*w37*Jinv10+4.9310064935064932e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10+2.9220779220779221e-04*Jinv20*w44-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59+8.3198051948051951e-05*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+4.9310064935064932e-04*w25*Jinv10+8.3198051948051951e-05*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-3.2873376623376621e-04*Jinv20*w17+3.6525974025974026e-05*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-6.7573051948051951e-04*w20*Jinv20-2.5568181818181820e-04*w5*Jinv20-2.7394480519480521e-05*w55*Jinv10+2.9220779220779221e-04*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20)*D;
  A[4][11]=(0.0)*D;
  A[4][12]=(0.0)*D;
  A[4][13]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8-3.7439123376623376e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-2.4655032467532467e-03*w3*Jinv00-1.2327516233766234e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54+3.7439123376623376e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+1.1414366883116883e-03*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10+3.9265422077922079e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-3.7439123376623376e-04*w58*Jinv10-3.1777597402597403e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12+1.0135957792207792e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35+3.7439123376623376e-04*w58*Jinv11+3.7439123376623376e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+1.1414366883116883e-03*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-3.1777597402597403e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+8.2183441558441553e-05*w45*Jinv00+5.4788961038961036e-04*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+1.1414366883116883e-03*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+3.5612824675324673e-04*w28*Jinv11+1.0135957792207792e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-3.1777597402597403e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12-1.2327516233766234e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22+5.4788961038961036e-04*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.7439123376623376e-04*Jinv22*w59+2.6846590909090908e-03*w34*Jinv11-3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11-1.5888798701298701e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+2.4655032467532467e-03*w32*Jinv21+1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-3.9265422077922079e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+2.6846590909090908e-03*Jinv21*w35+2.4655032467532467e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02+1.0135957792207792e-03*w49*Jinv10+3.5612824675324673e-04*Jinv21*w29+2.6846590909090908e-03*Jinv01*w33-1.5888798701298701e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-8.2183441558441553e-05*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9-1.3971185064935066e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.2327516233766234e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10+1.0135957792207792e-03*Jinv22*w50-1.1505681818181819e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7-1.1505681818181819e-03*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+1.0135957792207792e-03*Jinv12*w49-8.2183441558441553e-05*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-1.3971185064935066e-03*w7*Jinv11-3.9265422077922079e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+1.6436688311688311e-04*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-3.9265422077922079e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44-3.7439123376623376e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-1.3971185064935066e-03*Jinv21*w8-3.7439123376623376e-04*w57*Jinv00-3.7439123376623376e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.1505681818181819e-03*Jinv21*w23+1.6436688311688311e-04*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-3.1777597402597403e-03*w15*Jinv00+1.0135957792207792e-03*w48*Jinv02+3.9265422077922079e-04*Jinv01*w9-3.1777597402597403e-03*Jinv20*w17-1.2327516233766234e-03*w30*Jinv00-1.5888798701298701e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14+5.4788961038961036e-04*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20+2.4655032467532467e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-3.1777597402597403e-03*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50+3.5612824675324673e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1+3.9265422077922079e-04*w10*Jinv11)*D;
  A[4][14]=(0.0)*D;
  A[4][15]=(0.0)*D;
  A[4][16]=(2.0819805194805196e-03*Jinv00*w12+5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-1.1505681818181819e-03*Jinv22*w53+4.2004870129870131e-04*w57*Jinv02+2.0819805194805196e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44+9.8620129870129864e-04*w3*Jinv00+1.0409902597402598e-03*w32*Jinv20+9.5332792207792200e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23+1.3149350649350649e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54+9.6793831168831166e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+9.8620129870129864e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+6.9034090909090911e-03*w16*Jinv11+6.9034090909090911e-03*Jinv21*w17-7.6704545454545450e-04*Jinv12*w55+6.9399350649350649e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-1.2601461038961039e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+9.6793831168831166e-04*w58*Jinv11+9.6793831168831166e-04*w57*Jinv01-1.0957792207792208e-04*w0*Jinv01+4.3831168831168834e-04*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-3.2873376623376621e-04*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+4.9310064935064934e-03*w42*Jinv02+5.6615259740259744e-04*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.0819805194805196e-03*w13*Jinv11+6.9034090909090911e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11-1.2601461038961039e-03*Jinv20*w50+5.6615259740259744e-04*Jinv20*w29-3.2873376623376621e-04*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+2.9586038961038961e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12+1.0409902597402598e-03*w31*Jinv10-3.2873376623376621e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+9.8620129870129864e-04*Jinv22*w2+1.8628246753246754e-03*Jinv00*w18+9.3141233766233771e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+4.2004870129870131e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-1.2053571428571428e-03*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+4.9310064935064934e-03*Jinv22*w44+9.8620129870129864e-04*w4*Jinv10+9.5332792207792200e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20-2.1915584415584414e-03*Jinv12*w13+1.8628246753246754e-03*w19*Jinv10+2.9586038961038961e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+1.3149350649350649e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-6.0267857142857139e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20-1.1505681818181819e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+2.0819805194805196e-03*Jinv20*w14+9.5332792207792200e-03*w5*Jinv21-7.6704545454545450e-04*Jinv02*w54-3.2873376623376621e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02-1.2601461038961039e-03*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00-2.1915584415584414e-03*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-2.7942370129870131e-03*w6*Jinv01+9.3141233766233771e-04*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23-1.3149350649350649e-03*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-1.2053571428571428e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50-1.8628246753246754e-03*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-1.8628246753246754e-03*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01-1.2053571428571428e-03*w52*Jinv10-2.7942370129870131e-03*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00+9.3141233766233771e-04*w37*Jinv10-8.2183441558441553e-04*Jinv21*w56-6.0267857142857139e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-3.2873376623376621e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22-2.5750811688311690e-03*w49*Jinv11+2.0819805194805196e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44+4.2004870129870131e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18+2.0819805194805196e-03*Jinv01*w12-2.7942370129870131e-03*Jinv21*w8+6.9399350649350649e-04*w57*Jinv00+6.9399350649350649e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+4.3831168831168834e-04*w0*Jinv00+2.9586038961038961e-03*w35*Jinv20-1.8628246753246754e-03*Jinv21*w23-8.2183441558441553e-04*Jinv01*w54-8.2183441558441553e-04*w55*Jinv11-6.0267857142857139e-04*w25*Jinv10+4.3831168831168834e-04*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+5.9172077922077923e-03*Jinv20*w17+1.0409902597402598e-03*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10+1.8628246753246754e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-2.1915584415584414e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+1.3149350649350649e-03*Jinv02*w36+4.9310064935064934e-03*Jinv12*w43+9.8620129870129864e-04*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30-1.1505681818181819e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28-8.2183441558441553e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-3.2873376623376621e-04*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-2.5750811688311690e-03*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27-2.5750811688311690e-03*w48*Jinv01+9.8620129870129864e-04*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[4][17]=(0.0)*D;
  A[4][18]=(0.0)*D;
  A[4][19]=(3.8352272727272725e-04*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10-1.2327516233766234e-03*Jinv20*w8+3.8352272727272725e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00+1.9176136363636362e-04*w32*Jinv20+1.1505681818181819e-03*Jinv01*w3+1.3697240259740259e-04*Jinv20*w23+4.6570616883116886e-04*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-4.7483766233766234e-04*Jinv21*w2-2.5202922077922079e-03*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.1960227272727273e-04*w58*Jinv10+2.4655032467532466e-04*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26-4.7483766233766234e-04*w0*Jinv01-4.8396915584415583e-04*w1*Jinv10-2.7394480519480520e-03*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.7349837662337662e-04*Jinv00*w27-4.7483766233766234e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+1.9176136363636362e-04*w31*Jinv10+1.3697240259740259e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-2.0271915584415584e-03*Jinv00*w18-1.0135957792207792e-03*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+1.3971185064935066e-03*Jinv01*w24-2.1367694805194807e-03*w34*Jinv11-2.3833198051948050e-03*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+8.4922889610389610e-04*w51*Jinv00+1.4793019480519481e-03*w4*Jinv10+1.1505681818181819e-03*w4*Jinv11-7.1225649350649346e-04*w41*Jinv20-2.0271915584415584e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21-2.3833198051948050e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00+4.6570616883116886e-04*Jinv20*w56+4.3831168831168834e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.1367694805194807e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11+3.8352272727272725e-04*Jinv20*w14+1.1505681818181819e-03*w5*Jinv21+2.4655032467532466e-04*w49*Jinv10-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-7.1225649350649346e-04*w39*Jinv00-2.3833198051948050e-03*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-1.0135957792207792e-03*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+8.4922889610389610e-04*Jinv20*w53+1.3697240259740259e-04*w22*Jinv10+5.2049512987012989e-04*w22*Jinv11+5.2049512987012989e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+8.4922889610389610e-04*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-1.0135957792207792e-03*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-2.5202922077922079e-03*Jinv20*w44-1.6436688311688311e-04*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-3.1960227272727273e-04*w57*Jinv00-3.1960227272727273e-04*Jinv20*w59-4.8396915584415583e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20+5.2049512987012989e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-4.8396915584415583e-04*Jinv20*w2-2.7394480519480520e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-2.7394480519480520e-03*Jinv20*w17+1.9176136363636362e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-2.0271915584415584e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+1.4793019480519481e-03*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30+1.3971185064935066e-03*w25*Jinv11+4.6570616883116886e-04*w55*Jinv10-2.5202922077922079e-03*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[4][20]=(0.0)*D;
  A[4][21]=(0.0)*D;
  A[4][22]=(1.6436688311688311e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-4.0543831168831168e-03*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54-3.1960227272727273e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-5.5702110389610384e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-8.7662337662337668e-04*w16*Jinv11-8.7662337662337668e-04*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-3.1960227272727273e-04*w58*Jinv11-3.1960227272727273e-04*w57*Jinv01+5.4788961038961036e-04*Jinv21*w26-5.5702110389610384e-04*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-5.5702110389610384e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11-8.7662337662337668e-04*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+7.1225649350649346e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+5.4788961038961036e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59-2.5750811688311690e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.6570616883116886e-04*Jinv21*w53-8.2183441558441553e-05*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10-4.0543831168831168e-03*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-2.0271915584415584e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20-8.2183441558441553e-05*w52*Jinv12-2.5750811688311690e-03*Jinv21*w35-2.0271915584415584e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-4.0543831168831168e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+7.1225649350649346e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29-2.5750811688311690e-03*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9+9.5880681818181818e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+4.6570616883116886e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-8.2183441558441553e-05*Jinv20*w53-5.2049512987012989e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+7.3965097402597403e-04*w22*Jinv11-5.2049512987012989e-04*Jinv12*w7+7.3965097402597403e-04*w21*Jinv01+4.6570616883116886e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01-8.2183441558441553e-05*w52*Jinv10+9.5880681818181818e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10-2.7394480519480521e-05*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-5.2049512987012989e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12+7.1225649350649346e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.1231737012987013e-03*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+9.5880681818181818e-04*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.3965097402597403e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.7349837662337662e-04*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20-2.0271915584415584e-03*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+5.4788961038961036e-04*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12+1.1231737012987013e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+1.1231737012987013e-03*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-1.7349837662337662e-04*w10*Jinv11)*D;
  A[4][23]=(0.0)*D;
  A[4][24]=(0.0)*D;
  A[4][25]=(-1.6436688311688311e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+5.2049512987012989e-04*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23+2.7394480519480518e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+1.8262987012987013e-05*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-1.0957792207792208e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-1.0683847402597404e-03*Jinv21*w26+1.8262987012987013e-05*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27+1.8262987012987013e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+5.2049512987012989e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+1.3971185064935066e-03*Jinv21*w38-1.0683847402597404e-03*Jinv01*w24-1.6436688311688311e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11-3.8352272727272725e-04*w41*Jinv20+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+5.2049512987012989e-04*w6*Jinv00+2.7394480519480518e-04*Jinv20*w56-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.6436688311688311e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-3.8352272727272725e-04*w39*Jinv00+8.2183441558441553e-05*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11+1.4793019480519481e-03*w6*Jinv01+5.2049512987012989e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38+2.4655032467532466e-04*w52*Jinv11+8.2183441558441553e-05*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51+1.3971185064935066e-03*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10+1.4793019480519481e-03*w7*Jinv11-1.3697240259740259e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+4.6570616883116886e-04*Jinv21*w56-7.1225649350649346e-04*w24*Jinv00+5.4788961038961036e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-1.3697240259740259e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+3.8352272727272725e-04*Jinv01*w12+1.4793019480519481e-03*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+4.6570616883116886e-04*Jinv01*w54+4.6570616883116886e-04*w55*Jinv11-7.1225649350649346e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-1.0683847402597404e-03*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+5.4788961038961036e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[4][26]=(0.0)*D;
  A[4][27]=(0.0)*D;
  A[4][28]=(3.4699675324675324e-04*Jinv21*w44+1.4975649350649350e-03*Jinv01*w3+1.0247564935064935e-04*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2+3.4699675324675324e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+3.4699675324675324e-04*w43*Jinv11+2.1915584415584417e-04*Jinv21*w41+6.4935064935064935e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+2.1915584415584417e-04*Jinv01*w39-4.2918019480519480e-04*Jinv21*w38-1.0957792207792208e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53+1.4975649350649350e-03*w4*Jinv11+7.4878246753246752e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35+7.4878246753246752e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11+1.4975649350649350e-03*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+1.1870941558441559e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-5.8441558441558442e-04*w6*Jinv01-5.4788961038961042e-05*w47*Jinv21-2.1915584415584417e-04*w52*Jinv11-2.6481331168831169e-04*w22*Jinv11-2.6481331168831169e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-5.8441558441558442e-04*w7*Jinv11-2.7394480519480521e-05*Jinv21*w56-4.2918019480519480e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+3.6525974025974026e-05*Jinv01*w12-5.8441558441558442e-04*Jinv21*w8-2.6481331168831169e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11-1.4204545454545455e-05*Jinv01*w9+2.1915584415584417e-04*w40*Jinv11+7.4878246753246752e-04*Jinv01*w30-1.0957792207792208e-04*w25*Jinv11-4.2918019480519480e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[4][29]=(0.0)*D;
  A[4][30]=(0.0)*D;
  A[4][31]=(-1.2327516233766234e-03*Jinv00*w12-3.7439123376623376e-04*w28*Jinv10+5.4788961038961036e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8+3.5612824675324673e-04*w57*Jinv02-1.2327516233766234e-03*w13*Jinv10+5.4788961038961036e-04*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3+1.0135957792207792e-03*Jinv20*w23+8.2183441558441553e-05*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+1.1414366883116883e-03*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10+2.6846590909090908e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-3.7439123376623376e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.7439123376623376e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.7439123376623376e-04*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.6846590909090908e-03*Jinv22*w17-3.1777597402597403e-03*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3+1.0135957792207792e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+1.1414366883116883e-03*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+3.5612824675324673e-04*Jinv01*w24+3.5612824675324673e-04*Jinv22*w59-3.1777597402597403e-03*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00+1.0957792207792207e-03*Jinv12*w34-2.4655032467532467e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11-1.5888798701298701e-03*w41*Jinv20+2.4655032467532467e-03*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-3.1777597402597403e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*Jinv20*w56+5.4788961038961036e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-3.9265422077922079e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-3.1777597402597403e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11-1.2327516233766234e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10-3.7439123376623376e-04*Jinv21*w29-3.1777597402597403e-03*Jinv01*w33-1.5888798701298701e-03*w39*Jinv00+2.4655032467532467e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+1.6436688311688311e-04*Jinv20*w53-1.3971185064935066e-03*Jinv22*w8+1.0135957792207792e-03*w22*Jinv10-1.1505681818181819e-03*Jinv22*w50+1.0135957792207792e-03*w22*Jinv11-1.3971185064935066e-03*Jinv12*w7+1.0135957792207792e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.1505681818181819e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+1.2327516233766234e-03*w7*Jinv11-3.9265422077922079e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00+3.7439123376623376e-04*Jinv02*w27-1.3971185064935066e-03*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5+1.2327516233766234e-03*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14-3.9265422077922079e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44+3.5612824675324673e-04*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59+1.0957792207792207e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-3.1777597402597403e-03*w35*Jinv20+1.0135957792207792e-03*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.1505681818181819e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-1.5888798701298701e-03*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20+3.7439123376623376e-04*Jinv22*w29+2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11-2.4655032467532467e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+3.7439123376623376e-04*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.6846590909090908e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.7439123376623376e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+1.1414366883116883e-03*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[4][32]=(0.0)*D;
  A[4][33]=(0.0)*D;
  A[4][34]=(1.0409902597402598e-03*Jinv00*w12+6.9399350649350649e-04*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+6.2094155844155847e-04*w57*Jinv02+1.0409902597402598e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+4.9310064935064934e-03*Jinv21*w44+9.8620129870129864e-04*w3*Jinv00+2.0819805194805196e-03*w32*Jinv20-3.2873376623376621e-03*Jinv01*w3-1.2601461038961039e-03*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-3.2873376623376621e-04*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59-1.3149350649350649e-03*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02+9.8620129870129864e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10+4.9310064935064934e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45+5.6615259740259744e-04*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+6.9034090909090911e-03*Jinv22*w35+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01-1.1505681818181819e-03*Jinv21*w26+9.8620129870129864e-04*w0*Jinv01+4.3831168831168834e-04*w1*Jinv10+2.9586038961038961e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20+4.9310064935064934e-03*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02+6.9399350649350649e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+9.8620129870129864e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+4.2004870129870131e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+6.9399350649350649e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33-2.5750811688311690e-03*w22*Jinv12+2.0819805194805196e-03*w31*Jinv10+9.5332792207792200e-03*Jinv02*w3-1.2601461038961039e-03*w21*Jinv00+2.0819805194805196e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2+9.3141233766233771e-04*Jinv00*w18+1.8628246753246754e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-8.2183441558441553e-04*Jinv02*w45-1.1505681818181819e-03*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53-6.0267857142857139e-04*w51*Jinv00+6.9034090909090911e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44+9.8620129870129864e-04*w4*Jinv10-3.2873376623376621e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+9.3141233766233771e-04*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10-2.1915584415584414e-03*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41-1.2053571428571428e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+3.9448051948051946e-03*Jinv21*w35-2.1915584415584414e-03*w31*Jinv11-1.3149350649350649e-03*Jinv22*w26-7.6704545454545450e-04*w46*Jinv11+1.0409902597402598e-03*Jinv20*w14-3.2873376623376621e-03*w5*Jinv21+9.5332792207792200e-03*w4*Jinv12-2.5750811688311690e-03*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10+4.2004870129870131e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01-7.6704545454545450e-04*w47*Jinv21+1.8628246753246754e-03*Jinv20*w38-2.5750811688311690e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11+2.0819805194805196e-03*w31*Jinv12-8.2183441558441553e-04*w46*Jinv12-6.0267857142857139e-04*Jinv20*w53-2.7942370129870131e-03*Jinv22*w8-1.2601461038961039e-03*w22*Jinv10-1.8628246753246754e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-2.7942370129870131e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51+2.0819805194805196e-03*Jinv02*w30-1.8628246753246754e-03*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01-6.0267857142857139e-04*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00+1.8628246753246754e-03*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56-1.2053571428571428e-03*w24*Jinv00+9.6793831168831166e-04*Jinv02*w27-2.7942370129870131e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12+9.5332792207792200e-03*Jinv22*w5-8.2183441558441553e-04*w47*Jinv22+7.6704545454545450e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+5.6615259740259744e-04*w57*Jinv00+5.6615259740259744e-04*Jinv20*w59+6.9034090909090911e-03*Jinv02*w33+4.3831168831168834e-04*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-1.2053571428571428e-03*w25*Jinv10+4.3831168831168834e-04*Jinv20*w2+2.9586038961038961e-03*w15*Jinv00-1.8628246753246754e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+2.9586038961038961e-03*Jinv20*w17+2.0819805194805196e-03*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10+9.3141233766233771e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43+9.8620129870129864e-04*w5*Jinv20-2.1915584415584414e-03*Jinv01*w30+9.6793831168831166e-04*Jinv12*w28-1.1505681818181819e-03*w25*Jinv11-3.2873376623376621e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50+4.2004870129870131e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[4][35]=(0.0)*D;
  A[4][36]=(0.0)*D;
  A[4][37]=(1.9176136363636362e-04*Jinv00*w12-3.1960227272727273e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-4.2918019480519480e-04*w57*Jinv02+1.9176136363636362e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+1.4793019480519481e-03*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20+2.4655032467532466e-04*Jinv20*w23-2.3833198051948050e-03*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-4.7483766233766234e-04*w0*Jinv02-2.5202922077922079e-03*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+1.3697240259740259e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-4.8396915584415583e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+4.6570616883116886e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-3.1960227272727273e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+1.3697240259740259e-04*Jinv20*w50-3.1960227272727273e-04*Jinv20*w29+4.6570616883116886e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-2.7394480519480520e-03*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10+1.1505681818181819e-03*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00-1.6436688311688311e-04*w32*Jinv22-4.7483766233766234e-04*Jinv22*w2-1.0135957792207792e-03*Jinv00*w18-2.0271915584415584e-03*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45-4.2918019480519480e-04*Jinv22*w59+4.3831168831168834e-04*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10-7.1225649350649346e-04*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13-1.0135957792207792e-03*w19*Jinv10-2.7394480519480520e-03*w34*Jinv10-1.2327516233766234e-03*w6*Jinv00-2.3833198051948050e-03*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+8.4922889610389610e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+1.9176136363636362e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54+1.1505681818181819e-03*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02+1.3697240259740259e-04*w49*Jinv10-7.1225649350649346e-04*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.2327516233766234e-03*w7*Jinv10-2.0271915584415584e-03*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23-1.6436688311688311e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10+5.2049512987012989e-04*Jinv22*w50-2.6298701298701297e-03*Jinv12*w7-1.6436688311688311e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+4.3831168831168834e-04*w52*Jinv10+1.1870941558441559e-04*w9*Jinv00-2.0271915584415584e-03*w37*Jinv10+8.4922889610389610e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.1505681818181819e-03*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+1.1870941558441559e-04*w10*Jinv10-2.5202922077922079e-03*Jinv20*w44-4.2918019480519480e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-4.8396915584415583e-04*w0*Jinv00-2.7394480519480520e-03*w35*Jinv20+8.4922889610389610e-04*w25*Jinv10-4.8396915584415583e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00+5.2049512987012989e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-1.0135957792207792e-03*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-2.3833198051948050e-03*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+1.3971185064935066e-03*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28+2.4655032467532466e-04*w55*Jinv10-2.5202922077922079e-03*w42*Jinv00+4.6570616883116886e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-4.7483766233766234e-04*Jinv12*w1)*D;
  A[4][38]=(0.0)*D;
  A[4][39]=(0.0)*D;
  A[4][40]=(-1.0957792207792207e-03*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+3.8352272727272725e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-1.0957792207792207e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.0957792207792207e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+3.2873376623376621e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+3.6525974025974026e-05*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01-5.4788961038961042e-05*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.0409902597402598e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.7532467532467534e-03*w22*Jinv12-1.0957792207792207e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00+1.0409902597402598e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+3.1047077922077924e-04*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+3.2873376623376621e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-1.0957792207792207e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-1.7532467532467534e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9-1.7532467532467534e-03*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+5.4788961038961042e-05*Jinv20*w38-1.7532467532467534e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+1.0409902597402598e-03*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53-1.7532467532467534e-03*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-9.3141233766233771e-04*Jinv22*w50-9.3141233766233771e-04*w22*Jinv11-1.7532467532467534e-03*Jinv12*w7-9.3141233766233771e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+1.0409902597402598e-03*Jinv02*w30-9.3141233766233771e-04*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.7532467532467534e-03*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-3.2873376623376621e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-1.7532467532467534e-03*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22-1.7532467532467534e-03*w49*Jinv11+1.0409902597402598e-03*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+3.2873376623376621e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18+1.0409902597402598e-03*Jinv01*w12-1.7532467532467534e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-9.3141233766233771e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-9.3141233766233771e-04*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-1.0957792207792207e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30+5.2962662337662338e-04*Jinv12*w28+5.4788961038961042e-05*w55*Jinv10+3.2873376623376621e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12-1.7532467532467534e-03*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-1.7532467532467534e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[4][41]=(0.0)*D;
  A[4][42]=(0.0)*D;
  A[4][43]=(1.0957792207792207e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-4.2004870129870131e-04*w57*Jinv02+1.0957792207792207e-03*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+1.0957792207792207e-03*w32*Jinv20+3.2873376623376621e-03*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-9.8620129870129864e-04*w0*Jinv02-9.8620129870129864e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-3.9448051948051946e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-9.8620129870129864e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-9.8620129870129864e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-4.2004870129870131e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10-3.9448051948051946e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12+1.0957792207792207e-03*w31*Jinv10+3.2873376623376621e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-9.8620129870129864e-04*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45+1.1505681818181819e-03*Jinv01*w24-4.2004870129870131e-04*Jinv22*w59-3.9448051948051946e-03*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+3.2873376623376621e-03*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20+2.1915584415584414e-03*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+2.1915584415584414e-03*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12-3.9448051948051946e-03*Jinv21*w35+2.1915584415584414e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26+7.6704545454545450e-04*w46*Jinv11+1.0957792207792207e-03*Jinv20*w14+3.2873376623376621e-03*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+3.2873376623376621e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-4.2004870129870131e-04*Jinv21*w29-3.9448051948051946e-03*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00+2.1915584415584414e-03*Jinv02*w12-1.3149350649350649e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11-3.6525974025974026e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+3.2873376623376621e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-3.6525974025974026e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-4.2004870129870131e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+1.0957792207792207e-03*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29+2.1915584415584414e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-1.3149350649350649e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+2.1915584415584414e-03*Jinv01*w30+1.1505681818181819e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28+1.1505681818181819e-03*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20-3.9448051948051946e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50-4.2004870129870131e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01-9.8620129870129864e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[4][44]=(0.0)*D;
  A[4][45]=(0.0)*D;
  A[4][46]=(-3.8352272727272725e-04*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53-1.0957792207792208e-04*w57*Jinv02-3.8352272727272725e-04*Jinv02*w39-8.7662337662337668e-04*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-1.3697240259740259e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-8.7662337662337668e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-7.6704545454545450e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+5.2049512987012989e-04*Jinv22*w20-8.7662337662337668e-04*w43*Jinv11-7.1225649350649346e-04*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1+1.9176136363636362e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.6704545454545450e-04*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-3.5612824675324673e-04*w32*Jinv22-7.1225649350649346e-04*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+5.4788961038961036e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-7.1225649350649346e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-3.8352272727272725e-04*Jinv22*w41+8.2183441558441553e-05*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01-3.5612824675324673e-04*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11+2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+5.2049512987012989e-04*w49*Jinv11+1.9176136363636362e-04*Jinv21*w14-1.0957792207792208e-04*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+1.9176136363636362e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-8.2183441558441553e-05*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43-7.1225649350649346e-04*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-7.6704545454545450e-04*w16*Jinv12+5.2049512987012989e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[4][47]=(0.0)*D;
  A[4][48]=(0.0)*D;
  A[4][49]=(3.5612824675324673e-04*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+5.4788961038961036e-04*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+7.1225649350649346e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+4.6570616883116886e-04*w25*Jinv12-5.5702110389610384e-04*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-2.5750811688311690e-03*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+4.6570616883116886e-04*Jinv02*w24-8.7662337662337668e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10-2.5750811688311690e-03*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33+1.1231737012987013e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10-4.0543831168831168e-03*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-5.5702110389610384e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38-2.7394480519480521e-05*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-8.7662337662337668e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+7.1225649350649346e-04*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20-2.0271915584415584e-03*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41-8.2183441558441553e-05*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+5.4788961038961036e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+4.6570616883116886e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+7.1225649350649346e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54-4.0543831168831168e-03*w4*Jinv12+1.1231737012987013e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00-2.0271915584415584e-03*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11-1.7349837662337662e-04*Jinv02*w9-5.2049512987012989e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+1.1231737012987013e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+9.5880681818181818e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10+7.3965097402597403e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+9.5880681818181818e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30+7.3965097402597403e-04*Jinv12*w49-5.2049512987012989e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-5.2049512987012989e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-8.2183441558441553e-05*w24*Jinv00-3.1960227272727273e-04*Jinv02*w27+9.5880681818181818e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12-4.0543831168831168e-03*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-5.2049512987012989e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-8.7662337662337668e-04*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-8.2183441558441553e-05*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.3965097402597403e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-3.1960227272727273e-04*Jinv22*w29-2.0271915584415584e-03*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+5.4788961038961036e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-3.1960227272727273e-04*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20-2.5750811688311690e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-5.5702110389610384e-04*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[4][50]=(0.0)*D;
  A[4][51]=(0.0)*D;
  A[4][52]=(-3.5612824675324673e-04*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-1.0683847402597404e-03*Jinv22*w53+5.2049512987012989e-04*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+2.4655032467532466e-04*w25*Jinv12+1.8262987012987013e-05*w0*Jinv02-5.4788961038961042e-05*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+1.3971185064935066e-03*Jinv22*w20+2.7394480519480518e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+1.8262987012987013e-05*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+4.6570616883116886e-04*Jinv02*w45+1.2784090909090910e-04*Jinv22*w59-7.1225649350649346e-04*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10-3.8352272727272725e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10+5.2049512987012989e-04*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.0683847402597404e-03*w52*Jinv12+2.4655032467532466e-04*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10-3.8352272727272725e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9+5.2049512987012989e-04*w7*Jinv10+5.2049512987012989e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+3.8352272727272725e-04*w31*Jinv12+4.6570616883116886e-04*w46*Jinv12-7.1225649350649346e-04*Jinv20*w53+1.4793019480519481e-03*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50+1.4793019480519481e-03*Jinv12*w7+3.8352272727272725e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-7.1225649350649346e-04*w52*Jinv10-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+8.2183441558441553e-05*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+1.4793019480519481e-03*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5+4.6570616883116886e-04*w47*Jinv22-1.3697240259740259e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+8.2183441558441553e-05*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+8.2183441558441553e-05*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20-1.0683847402597404e-03*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-3.5612824675324673e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12+1.8262987012987013e-05*Jinv12*w1)*D;
  A[4][53]=(0.0)*D;
  A[4][54]=(0.0)*D;
  A[4][55]=(-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+6.3920454545454550e-05*w57*Jinv02-7.1225649350649346e-04*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+2.4655032467532466e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12+5.4788961038961036e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+5.4788961038961036e-04*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11-3.8352272727272725e-04*Jinv21*w41-8.7662337662337668e-04*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-1.0957792207792208e-04*Jinv22*w17+5.2049512987012989e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3+1.9176136363636362e-04*w32*Jinv22-3.8352272727272725e-04*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+5.2049512987012989e-04*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+6.3920454545454550e-05*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-8.7662337662337668e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+2.7394480519480518e-04*w47*Jinv21+5.2049512987012989e-04*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+1.9176136363636362e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8-8.2183441558441553e-05*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+1.9176136363636362e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+5.2049512987012989e-04*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-3.5612824675324673e-04*Jinv21*w56-1.3697240259740259e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+6.3920454545454550e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-8.2183441558441553e-05*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-3.8352272727272725e-04*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36-8.7662337662337668e-04*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-1.0957792207792208e-04*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[4][56]=(0.0)*D;
  A[4][57]=(0.0)*D;
  A[4][58]=(2.1915584415584417e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+2.1915584415584417e-04*Jinv02*w39+1.1870941558441559e-04*Jinv22*w38-2.1915584415584417e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-5.4788961038961042e-05*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12-2.1915584415584417e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35-4.2918019480519480e-04*Jinv22*w20+3.4699675324675324e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12+1.4975649350649350e-03*Jinv02*w3+3.6525974025974026e-05*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2-2.7394480519480521e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+4.3831168831168834e-04*Jinv12*w34+3.4699675324675324e-04*Jinv22*w44+7.4878246753246752e-04*Jinv12*w13+1.1870941558441559e-04*w37*Jinv12+2.1915584415584417e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12-2.1915584415584417e-04*Jinv22*w26-5.4788961038961042e-05*Jinv02*w54+1.4975649350649350e-03*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02+7.4878246753246752e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-4.2918019480519480e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12-5.8441558441558442e-04*Jinv22*w8-2.6481331168831169e-04*Jinv22*w50-5.8441558441558442e-04*Jinv12*w7+3.6525974025974026e-05*Jinv02*w30-2.6481331168831169e-04*Jinv12*w49+1.0247564935064935e-04*Jinv02*w27-5.8441558441558442e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+1.4975649350649350e-03*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+4.3831168831168834e-04*Jinv02*w33-2.6481331168831169e-04*w48*Jinv02+1.0247564935064935e-04*Jinv22*w29+7.4878246753246752e-04*Jinv22*w14+1.1870941558441559e-04*Jinv02*w36+3.4699675324675324e-04*Jinv12*w43-1.0957792207792208e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[4][59]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(0.0)*D;
  A[5][2]=(-1.1870941558441558e-03*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10-3.2873376623376621e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+1.0227272727272728e-03*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02-1.1870941558441558e-03*w13*Jinv10-3.2873376623376621e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-2.3741883116883116e-03*w3*Jinv00-1.1870941558441558e-03*w32*Jinv20-2.3741883116883116e-03*Jinv01*w3+6.7573051948051951e-04*Jinv20*w23+2.6481331168831169e-04*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59+1.0957792207792208e-04*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-2.0495129870129870e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10-6.5746753246753243e-04*w15*Jinv02-2.0495129870129870e-04*w10*Jinv12+6.7573051948051951e-04*w48*Jinv00+1.0957792207792208e-04*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+2.6481331168831169e-04*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00-3.2873376623376621e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-2.0495129870129870e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-1.1870941558441558e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+6.7573051948051951e-04*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-6.5746753246753243e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+6.7573051948051951e-04*w22*Jinv12-1.1870941558441558e-03*w31*Jinv10-2.3741883116883116e-03*Jinv02*w3+6.7573051948051951e-04*w21*Jinv00-1.1870941558441558e-03*w32*Jinv22-3.2873376623376621e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+2.6481331168831169e-04*Jinv00*w18+2.6481331168831169e-04*w36*Jinv00+2.6481331168831169e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-6.5746753246753243e-04*w34*Jinv11+2.6481331168831169e-04*w19*Jinv11+1.0957792207792208e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-2.3741883116883116e-03*w4*Jinv10-2.3741883116883116e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-1.1870941558441558e-03*Jinv12*w13+2.6481331168831169e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.1870941558441558e-03*w32*Jinv21+2.6481331168831169e-04*Jinv21*w20+1.0227272727272728e-03*w6*Jinv00+2.6481331168831169e-04*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56-3.2873376623376621e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.0495129870129870e-04*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-6.5746753246753243e-04*Jinv21*w35-1.1870941558441558e-03*w31*Jinv11+1.0957792207792208e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-1.1870941558441558e-03*Jinv20*w14-2.3741883116883116e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54-2.3741883116883116e-03*w4*Jinv12+6.7573051948051951e-04*w21*Jinv02+6.7573051948051951e-04*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-6.5746753246753243e-04*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-1.1870941558441558e-03*Jinv02*w12+2.6481331168831169e-04*Jinv01*w18+2.6481331168831169e-04*w37*Jinv11-2.0495129870129870e-04*Jinv02*w9+1.0227272727272728e-03*w6*Jinv01+1.0227272727272728e-03*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21+2.6481331168831169e-04*Jinv20*w38+6.7573051948051951e-04*Jinv22*w23+1.0957792207792208e-04*w52*Jinv11-1.1870941558441558e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.0227272727272728e-03*Jinv22*w8+6.7573051948051951e-04*w22*Jinv10+6.7573051948051951e-04*Jinv22*w50+6.7573051948051951e-04*w22*Jinv11+1.0227272727272728e-03*Jinv12*w7+6.7573051948051951e-04*w21*Jinv01+1.0957792207792208e-04*Jinv01*w51-1.1870941558441558e-03*Jinv02*w30+6.7573051948051951e-04*Jinv12*w49+2.6481331168831169e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.0227272727272728e-03*w7*Jinv11-2.0495129870129870e-04*w9*Jinv00+2.6481331168831169e-04*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+1.0227272727272728e-03*w6*Jinv02+2.6481331168831169e-04*w19*Jinv12-2.3741883116883116e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+6.7573051948051951e-04*w49*Jinv11-1.1870941558441558e-03*Jinv21*w14-2.0495129870129870e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+2.6481331168831169e-04*Jinv02*w18-1.1870941558441558e-03*Jinv01*w12+1.0227272727272728e-03*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+6.7573051948051951e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+6.7573051948051951e-04*w48*Jinv02-2.0495129870129870e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.1870941558441558e-03*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+2.6481331168831169e-04*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-1.1870941558441558e-03*Jinv22*w14-3.2873376623376621e-04*w40*Jinv11+2.6481331168831169e-04*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-2.3741883116883116e-03*w5*Jinv20-1.1870941558441558e-03*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-6.5746753246753243e-04*w16*Jinv12+6.7573051948051951e-04*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+6.7573051948051951e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-2.0495129870129870e-04*w10*Jinv11)*D;
  A[5][3]=(0.0)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(1.2327516233766234e-03*Jinv00*w12-1.5888798701298701e-03*w40*Jinv12+1.0683847402597404e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+1.2327516233766234e-03*w32*Jinv20-7.3965097402597406e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-5.7528409090909093e-04*Jinv22*w38-7.4878246753246752e-04*Jinv21*w59+1.0683847402597404e-03*w25*Jinv12+9.1314935064935068e-05*w0*Jinv02+9.1314935064935068e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-6.5746753246753243e-04*w11*Jinv21-4.7666396103896100e-03*w16*Jinv11-4.7666396103896100e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-4.7666396103896100e-03*w15*Jinv02-6.5746753246753243e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+1.0683847402597404e-03*Jinv02*w24-4.7666396103896100e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+1.0683847402597404e-03*Jinv21*w26+9.1314935064935068e-05*w0*Jinv01+1.1870941558441558e-03*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10-5.7528409090909093e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-1.5888798701298701e-03*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-6.5746753246753243e-04*w11*Jinv22+9.1314935064935068e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-4.7666396103896100e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50-4.7666396103896100e-03*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33+2.0271915584415584e-03*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10-7.3965097402597406e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+9.1314935064935068e-05*Jinv22*w2-7.3965097402597403e-04*Jinv00*w18-7.3965097402597403e-04*w36*Jinv00-5.7528409090909093e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.0683847402597404e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-4.7666396103896100e-03*w34*Jinv11-5.7528409090909093e-04*w19*Jinv11+1.0683847402597404e-03*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.7666396103896100e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44-7.3965097402597406e-03*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13-7.3965097402597403e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21-5.7528409090909093e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00-5.7528409090909093e-04*w37*Jinv12-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+1.0683847402597404e-03*w52*Jinv12-4.7666396103896100e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11+1.0683847402597404e-03*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14-7.3965097402597406e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-7.3965097402597406e-03*w4*Jinv12+2.0271915584415584e-03*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.7666396103896100e-03*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-5.7528409090909093e-04*Jinv01*w18-5.7528409090909093e-04*w37*Jinv11-6.5746753246753243e-04*Jinv02*w9+1.3149350649350649e-03*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21-7.3965097402597403e-04*Jinv20*w38+2.0271915584415584e-03*Jinv22*w23+1.0683847402597404e-03*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.3149350649350649e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+2.0271915584415584e-03*Jinv22*w50+2.0271915584415584e-03*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7+2.0271915584415584e-03*w21*Jinv01+1.0683847402597404e-03*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+2.0271915584415584e-03*Jinv12*w49-5.7528409090909093e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+1.3149350649350649e-03*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-7.3965097402597403e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02-5.7528409090909093e-04*w19*Jinv12-7.3965097402597406e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+2.0271915584415584e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-5.7528409090909093e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8-4.7666396103896100e-03*Jinv02*w33+1.1870941558441558e-03*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+2.0271915584415584e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.1870941558441558e-03*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+2.0271915584415584e-03*w48*Jinv02-6.5746753246753243e-04*Jinv01*w9-4.9310064935064932e-04*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-7.3965097402597403e-04*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11-5.7528409090909093e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43-2.4655032467532467e-03*Jinv01*w30+1.0683847402597404e-03*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+1.0683847402597404e-03*w25*Jinv11-5.4788961038961036e-04*w42*Jinv00-4.7666396103896100e-03*w16*Jinv12+2.0271915584415584e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+2.0271915584415584e-03*w48*Jinv01+9.1314935064935068e-05*Jinv12*w1-6.5746753246753243e-04*w10*Jinv11)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(0.0)*D;
  A[5][8]=(-8.2183441558441553e-05*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+2.6298701298701297e-03*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-2.7394480519480521e-05*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+4.7483766233766234e-04*w0*Jinv02+4.7483766233766234e-04*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55+2.7394480519480518e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+4.7483766233766234e-04*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10+1.4793019480519481e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-2.7394480519480521e-05*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.7483766233766234e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+1.4793019480519481e-03*Jinv00*w33-5.2049512987012989e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.7483766233766234e-04*Jinv22*w2+1.1505681818181819e-03*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-9.5880681818181818e-04*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+2.6298701298701297e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13+1.1505681818181819e-03*w19*Jinv10+1.4793019480519481e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-2.7394480519480521e-05*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-9.5880681818181818e-04*w26*Jinv20+4.0178571428571428e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12-5.2049512987012989e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.6298701298701297e-03*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+1.1505681818181819e-03*Jinv20*w38-5.2049512987012989e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-9.5880681818181818e-04*Jinv20*w53+2.6298701298701297e-03*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+2.6298701298701297e-03*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-9.5880681818181818e-04*w52*Jinv10+2.6298701298701297e-03*w7*Jinv11+4.0178571428571428e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-9.5880681818181818e-04*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.6298701298701297e-03*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-5.2049512987012989e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.0178571428571428e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12+2.6298701298701297e-03*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00+1.4793019480519481e-03*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-9.5880681818181818e-04*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2+1.4793019480519481e-03*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+1.4793019480519481e-03*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+2.6298701298701297e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11-2.7394480519480521e-05*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-5.2049512987012989e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27-5.2049512987012989e-04*w48*Jinv01+4.7483766233766234e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[5][9]=(0.0)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(3.6525974025974026e-05*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10-1.0227272727272728e-03*Jinv20*w8+3.6525974025974026e-05*w13*Jinv10-2.5568181818181820e-04*w3*Jinv00+3.6525974025974026e-05*w32*Jinv20+1.1870941558441559e-04*Jinv20*w23-2.7394480519480521e-05*Jinv00*w54+2.9220779220779221e-04*w43*Jinv10-1.1668019480519480e-04*w58*Jinv10+1.1870941558441559e-04*w48*Jinv00+8.3198051948051951e-05*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10-2.7394480519480521e-05*w45*Jinv00-1.1668019480519480e-04*Jinv00*w27+1.1870941558441559e-04*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29-2.7394480519480521e-05*w46*Jinv10-3.2873376623376621e-04*Jinv00*w33+3.6525974025974026e-05*w31*Jinv10+1.1870941558441559e-04*w21*Jinv00-6.7573051948051951e-04*Jinv00*w18-6.7573051948051951e-04*w36*Jinv00+4.9310064935064932e-04*w51*Jinv00-2.5568181818181820e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-6.7573051948051951e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-1.0227272727272728e-03*w6*Jinv00-2.7394480519480521e-05*Jinv20*w56+4.9310064935064932e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20+3.6525974025974026e-05*Jinv20*w14+1.1870941558441559e-04*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-1.0227272727272728e-03*w7*Jinv10-6.7573051948051951e-04*Jinv20*w38+4.9310064935064932e-04*Jinv20*w53+1.1870941558441559e-04*w22*Jinv10+4.9310064935064932e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-6.7573051948051951e-04*w37*Jinv10+4.9310064935064932e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10+2.9220779220779221e-04*Jinv20*w44-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59+8.3198051948051951e-05*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+4.9310064935064932e-04*w25*Jinv10+8.3198051948051951e-05*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-3.2873376623376621e-04*Jinv20*w17+3.6525974025974026e-05*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-6.7573051948051951e-04*w20*Jinv20-2.5568181818181820e-04*w5*Jinv20-2.7394480519480521e-05*w55*Jinv10+2.9220779220779221e-04*w42*Jinv00-2.7394480519480521e-05*w47*Jinv20)*D;
  A[5][12]=(0.0)*D;
  A[5][13]=(0.0)*D;
  A[5][14]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8-3.7439123376623376e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-2.4655032467532467e-03*w3*Jinv00-1.2327516233766234e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54+3.7439123376623376e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+1.1414366883116883e-03*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10+3.9265422077922079e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-3.7439123376623376e-04*w58*Jinv10-3.1777597402597403e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12+1.0135957792207792e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35+3.7439123376623376e-04*w58*Jinv11+3.7439123376623376e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+1.1414366883116883e-03*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-3.1777597402597403e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+8.2183441558441553e-05*w45*Jinv00+5.4788961038961036e-04*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+1.1414366883116883e-03*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+3.5612824675324673e-04*w28*Jinv11+1.0135957792207792e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-3.1777597402597403e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12-1.2327516233766234e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22+5.4788961038961036e-04*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.7439123376623376e-04*Jinv22*w59+2.6846590909090908e-03*w34*Jinv11-3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11-1.5888798701298701e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+2.4655032467532467e-03*w32*Jinv21+1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-3.9265422077922079e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+2.6846590909090908e-03*Jinv21*w35+2.4655032467532467e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02+1.0135957792207792e-03*w49*Jinv10+3.5612824675324673e-04*Jinv21*w29+2.6846590909090908e-03*Jinv01*w33-1.5888798701298701e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-8.2183441558441553e-05*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9-1.3971185064935066e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.2327516233766234e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10+1.0135957792207792e-03*Jinv22*w50-1.1505681818181819e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7-1.1505681818181819e-03*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+1.0135957792207792e-03*Jinv12*w49-8.2183441558441553e-05*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-1.3971185064935066e-03*w7*Jinv11-3.9265422077922079e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+1.6436688311688311e-04*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-3.9265422077922079e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44-3.7439123376623376e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-1.3971185064935066e-03*Jinv21*w8-3.7439123376623376e-04*w57*Jinv00-3.7439123376623376e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.1505681818181819e-03*Jinv21*w23+1.6436688311688311e-04*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-3.1777597402597403e-03*w15*Jinv00+1.0135957792207792e-03*w48*Jinv02+3.9265422077922079e-04*Jinv01*w9-3.1777597402597403e-03*Jinv20*w17-1.2327516233766234e-03*w30*Jinv00-1.5888798701298701e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14+5.4788961038961036e-04*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20+2.4655032467532467e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-3.1777597402597403e-03*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50+3.5612824675324673e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1+3.9265422077922079e-04*w10*Jinv11)*D;
  A[5][15]=(0.0)*D;
  A[5][16]=(0.0)*D;
  A[5][17]=(2.0819805194805196e-03*Jinv00*w12+5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-1.1505681818181819e-03*Jinv22*w53+4.2004870129870131e-04*w57*Jinv02+2.0819805194805196e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44+9.8620129870129864e-04*w3*Jinv00+1.0409902597402598e-03*w32*Jinv20+9.5332792207792200e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23+1.3149350649350649e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54+9.6793831168831166e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+9.8620129870129864e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+6.9034090909090911e-03*w16*Jinv11+6.9034090909090911e-03*Jinv21*w17-7.6704545454545450e-04*Jinv12*w55+6.9399350649350649e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-1.2601461038961039e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+9.6793831168831166e-04*w58*Jinv11+9.6793831168831166e-04*w57*Jinv01-1.0957792207792208e-04*w0*Jinv01+4.3831168831168834e-04*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-3.2873376623376621e-04*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+4.9310064935064934e-03*w42*Jinv02+5.6615259740259744e-04*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.0819805194805196e-03*w13*Jinv11+6.9034090909090911e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11-1.2601461038961039e-03*Jinv20*w50+5.6615259740259744e-04*Jinv20*w29-3.2873376623376621e-04*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+2.9586038961038961e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12+1.0409902597402598e-03*w31*Jinv10-3.2873376623376621e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+9.8620129870129864e-04*Jinv22*w2+1.8628246753246754e-03*Jinv00*w18+9.3141233766233771e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+4.2004870129870131e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-1.2053571428571428e-03*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+4.9310064935064934e-03*Jinv22*w44+9.8620129870129864e-04*w4*Jinv10+9.5332792207792200e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20-2.1915584415584414e-03*Jinv12*w13+1.8628246753246754e-03*w19*Jinv10+2.9586038961038961e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+1.3149350649350649e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-6.0267857142857139e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20-1.1505681818181819e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+2.0819805194805196e-03*Jinv20*w14+9.5332792207792200e-03*w5*Jinv21-7.6704545454545450e-04*Jinv02*w54-3.2873376623376621e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02-1.2601461038961039e-03*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00-2.1915584415584414e-03*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-2.7942370129870131e-03*w6*Jinv01+9.3141233766233771e-04*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23-1.3149350649350649e-03*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-1.2053571428571428e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50-1.8628246753246754e-03*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-1.8628246753246754e-03*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01-1.2053571428571428e-03*w52*Jinv10-2.7942370129870131e-03*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00+9.3141233766233771e-04*w37*Jinv10-8.2183441558441553e-04*Jinv21*w56-6.0267857142857139e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-3.2873376623376621e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22-2.5750811688311690e-03*w49*Jinv11+2.0819805194805196e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44+4.2004870129870131e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18+2.0819805194805196e-03*Jinv01*w12-2.7942370129870131e-03*Jinv21*w8+6.9399350649350649e-04*w57*Jinv00+6.9399350649350649e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+4.3831168831168834e-04*w0*Jinv00+2.9586038961038961e-03*w35*Jinv20-1.8628246753246754e-03*Jinv21*w23-8.2183441558441553e-04*Jinv01*w54-8.2183441558441553e-04*w55*Jinv11-6.0267857142857139e-04*w25*Jinv10+4.3831168831168834e-04*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+5.9172077922077923e-03*Jinv20*w17+1.0409902597402598e-03*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10+1.8628246753246754e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-2.1915584415584414e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+1.3149350649350649e-03*Jinv02*w36+4.9310064935064934e-03*Jinv12*w43+9.8620129870129864e-04*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30-1.1505681818181819e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28-8.2183441558441553e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-3.2873376623376621e-04*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-2.5750811688311690e-03*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27-2.5750811688311690e-03*w48*Jinv01+9.8620129870129864e-04*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[5][18]=(0.0)*D;
  A[5][19]=(0.0)*D;
  A[5][20]=(3.8352272727272725e-04*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10-1.2327516233766234e-03*Jinv20*w8+3.8352272727272725e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00+1.9176136363636362e-04*w32*Jinv20+1.1505681818181819e-03*Jinv01*w3+1.3697240259740259e-04*Jinv20*w23+4.6570616883116886e-04*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-4.7483766233766234e-04*Jinv21*w2-2.5202922077922079e-03*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.1960227272727273e-04*w58*Jinv10+2.4655032467532466e-04*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26-4.7483766233766234e-04*w0*Jinv01-4.8396915584415583e-04*w1*Jinv10-2.7394480519480520e-03*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.7349837662337662e-04*Jinv00*w27-4.7483766233766234e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+1.9176136363636362e-04*w31*Jinv10+1.3697240259740259e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-2.0271915584415584e-03*Jinv00*w18-1.0135957792207792e-03*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+1.3971185064935066e-03*Jinv01*w24-2.1367694805194807e-03*w34*Jinv11-2.3833198051948050e-03*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+8.4922889610389610e-04*w51*Jinv00+1.4793019480519481e-03*w4*Jinv10+1.1505681818181819e-03*w4*Jinv11-7.1225649350649346e-04*w41*Jinv20-2.0271915584415584e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21-2.3833198051948050e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00+4.6570616883116886e-04*Jinv20*w56+4.3831168831168834e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.1367694805194807e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11+3.8352272727272725e-04*Jinv20*w14+1.1505681818181819e-03*w5*Jinv21+2.4655032467532466e-04*w49*Jinv10-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-7.1225649350649346e-04*w39*Jinv00-2.3833198051948050e-03*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-1.0135957792207792e-03*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+8.4922889610389610e-04*Jinv20*w53+1.3697240259740259e-04*w22*Jinv10+5.2049512987012989e-04*w22*Jinv11+5.2049512987012989e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+8.4922889610389610e-04*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-1.0135957792207792e-03*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-2.5202922077922079e-03*Jinv20*w44-1.6436688311688311e-04*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-3.1960227272727273e-04*w57*Jinv00-3.1960227272727273e-04*Jinv20*w59-4.8396915584415583e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20+5.2049512987012989e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-4.8396915584415583e-04*Jinv20*w2-2.7394480519480520e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-2.7394480519480520e-03*Jinv20*w17+1.9176136363636362e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-2.0271915584415584e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+1.4793019480519481e-03*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30+1.3971185064935066e-03*w25*Jinv11+4.6570616883116886e-04*w55*Jinv10-2.5202922077922079e-03*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[5][21]=(0.0)*D;
  A[5][22]=(0.0)*D;
  A[5][23]=(1.6436688311688311e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-4.0543831168831168e-03*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54-3.1960227272727273e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-5.5702110389610384e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-8.7662337662337668e-04*w16*Jinv11-8.7662337662337668e-04*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-3.1960227272727273e-04*w58*Jinv11-3.1960227272727273e-04*w57*Jinv01+5.4788961038961036e-04*Jinv21*w26-5.5702110389610384e-04*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-5.5702110389610384e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11-8.7662337662337668e-04*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+7.1225649350649346e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+5.4788961038961036e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59-2.5750811688311690e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.6570616883116886e-04*Jinv21*w53-8.2183441558441553e-05*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10-4.0543831168831168e-03*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-2.0271915584415584e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20-8.2183441558441553e-05*w52*Jinv12-2.5750811688311690e-03*Jinv21*w35-2.0271915584415584e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-4.0543831168831168e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+7.1225649350649346e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29-2.5750811688311690e-03*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9+9.5880681818181818e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+4.6570616883116886e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-8.2183441558441553e-05*Jinv20*w53-5.2049512987012989e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+7.3965097402597403e-04*w22*Jinv11-5.2049512987012989e-04*Jinv12*w7+7.3965097402597403e-04*w21*Jinv01+4.6570616883116886e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01-8.2183441558441553e-05*w52*Jinv10+9.5880681818181818e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10-2.7394480519480521e-05*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-5.2049512987012989e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12+7.1225649350649346e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.1231737012987013e-03*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+9.5880681818181818e-04*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.3965097402597403e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.7349837662337662e-04*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20-2.0271915584415584e-03*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+5.4788961038961036e-04*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12+1.1231737012987013e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+1.1231737012987013e-03*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-1.7349837662337662e-04*w10*Jinv11)*D;
  A[5][24]=(0.0)*D;
  A[5][25]=(0.0)*D;
  A[5][26]=(-1.6436688311688311e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+5.2049512987012989e-04*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23+2.7394480519480518e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+1.8262987012987013e-05*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-1.0957792207792208e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-1.0683847402597404e-03*Jinv21*w26+1.8262987012987013e-05*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27+1.8262987012987013e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+5.2049512987012989e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+1.3971185064935066e-03*Jinv21*w38-1.0683847402597404e-03*Jinv01*w24-1.6436688311688311e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11-3.8352272727272725e-04*w41*Jinv20+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+5.2049512987012989e-04*w6*Jinv00+2.7394480519480518e-04*Jinv20*w56-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.6436688311688311e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-3.8352272727272725e-04*w39*Jinv00+8.2183441558441553e-05*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11+1.4793019480519481e-03*w6*Jinv01+5.2049512987012989e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38+2.4655032467532466e-04*w52*Jinv11+8.2183441558441553e-05*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51+1.3971185064935066e-03*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10+1.4793019480519481e-03*w7*Jinv11-1.3697240259740259e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+4.6570616883116886e-04*Jinv21*w56-7.1225649350649346e-04*w24*Jinv00+5.4788961038961036e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-1.3697240259740259e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+3.8352272727272725e-04*Jinv01*w12+1.4793019480519481e-03*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+4.6570616883116886e-04*Jinv01*w54+4.6570616883116886e-04*w55*Jinv11-7.1225649350649346e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-1.0683847402597404e-03*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+5.4788961038961036e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[5][27]=(0.0)*D;
  A[5][28]=(0.0)*D;
  A[5][29]=(3.4699675324675324e-04*Jinv21*w44+1.4975649350649350e-03*Jinv01*w3+1.0247564935064935e-04*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2+3.4699675324675324e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+3.4699675324675324e-04*w43*Jinv11+2.1915584415584417e-04*Jinv21*w41+6.4935064935064935e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+2.1915584415584417e-04*Jinv01*w39-4.2918019480519480e-04*Jinv21*w38-1.0957792207792208e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53+1.4975649350649350e-03*w4*Jinv11+7.4878246753246752e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35+7.4878246753246752e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11+1.4975649350649350e-03*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+1.1870941558441559e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-5.8441558441558442e-04*w6*Jinv01-5.4788961038961042e-05*w47*Jinv21-2.1915584415584417e-04*w52*Jinv11-2.6481331168831169e-04*w22*Jinv11-2.6481331168831169e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-5.8441558441558442e-04*w7*Jinv11-2.7394480519480521e-05*Jinv21*w56-4.2918019480519480e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+3.6525974025974026e-05*Jinv01*w12-5.8441558441558442e-04*Jinv21*w8-2.6481331168831169e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11-1.4204545454545455e-05*Jinv01*w9+2.1915584415584417e-04*w40*Jinv11+7.4878246753246752e-04*Jinv01*w30-1.0957792207792208e-04*w25*Jinv11-4.2918019480519480e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[5][30]=(0.0)*D;
  A[5][31]=(0.0)*D;
  A[5][32]=(-1.2327516233766234e-03*Jinv00*w12-3.7439123376623376e-04*w28*Jinv10+5.4788961038961036e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8+3.5612824675324673e-04*w57*Jinv02-1.2327516233766234e-03*w13*Jinv10+5.4788961038961036e-04*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3+1.0135957792207792e-03*Jinv20*w23+8.2183441558441553e-05*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+1.1414366883116883e-03*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10+2.6846590909090908e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-3.7439123376623376e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.7439123376623376e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.7439123376623376e-04*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.6846590909090908e-03*Jinv22*w17-3.1777597402597403e-03*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3+1.0135957792207792e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+1.1414366883116883e-03*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+3.5612824675324673e-04*Jinv01*w24+3.5612824675324673e-04*Jinv22*w59-3.1777597402597403e-03*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00+1.0957792207792207e-03*Jinv12*w34-2.4655032467532467e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11-1.5888798701298701e-03*w41*Jinv20+2.4655032467532467e-03*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-3.1777597402597403e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*Jinv20*w56+5.4788961038961036e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-3.9265422077922079e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-3.1777597402597403e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11-1.2327516233766234e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10-3.7439123376623376e-04*Jinv21*w29-3.1777597402597403e-03*Jinv01*w33-1.5888798701298701e-03*w39*Jinv00+2.4655032467532467e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+1.6436688311688311e-04*Jinv20*w53-1.3971185064935066e-03*Jinv22*w8+1.0135957792207792e-03*w22*Jinv10-1.1505681818181819e-03*Jinv22*w50+1.0135957792207792e-03*w22*Jinv11-1.3971185064935066e-03*Jinv12*w7+1.0135957792207792e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.1505681818181819e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+1.2327516233766234e-03*w7*Jinv11-3.9265422077922079e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00+3.7439123376623376e-04*Jinv02*w27-1.3971185064935066e-03*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5+1.2327516233766234e-03*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14-3.9265422077922079e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44+3.5612824675324673e-04*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59+1.0957792207792207e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-3.1777597402597403e-03*w35*Jinv20+1.0135957792207792e-03*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.1505681818181819e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-1.5888798701298701e-03*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20+3.7439123376623376e-04*Jinv22*w29+2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11-2.4655032467532467e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+3.7439123376623376e-04*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.6846590909090908e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.7439123376623376e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+1.1414366883116883e-03*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[5][33]=(0.0)*D;
  A[5][34]=(0.0)*D;
  A[5][35]=(1.0409902597402598e-03*Jinv00*w12+6.9399350649350649e-04*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+6.2094155844155847e-04*w57*Jinv02+1.0409902597402598e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+4.9310064935064934e-03*Jinv21*w44+9.8620129870129864e-04*w3*Jinv00+2.0819805194805196e-03*w32*Jinv20-3.2873376623376621e-03*Jinv01*w3-1.2601461038961039e-03*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-3.2873376623376621e-04*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59-1.3149350649350649e-03*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02+9.8620129870129864e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10+4.9310064935064934e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45+5.6615259740259744e-04*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+6.9034090909090911e-03*Jinv22*w35+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01-1.1505681818181819e-03*Jinv21*w26+9.8620129870129864e-04*w0*Jinv01+4.3831168831168834e-04*w1*Jinv10+2.9586038961038961e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20+4.9310064935064934e-03*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02+6.9399350649350649e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+9.8620129870129864e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+4.2004870129870131e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+6.9399350649350649e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33-2.5750811688311690e-03*w22*Jinv12+2.0819805194805196e-03*w31*Jinv10+9.5332792207792200e-03*Jinv02*w3-1.2601461038961039e-03*w21*Jinv00+2.0819805194805196e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2+9.3141233766233771e-04*Jinv00*w18+1.8628246753246754e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-8.2183441558441553e-04*Jinv02*w45-1.1505681818181819e-03*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53-6.0267857142857139e-04*w51*Jinv00+6.9034090909090911e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44+9.8620129870129864e-04*w4*Jinv10-3.2873376623376621e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+9.3141233766233771e-04*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10-2.1915584415584414e-03*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41-1.2053571428571428e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+3.9448051948051946e-03*Jinv21*w35-2.1915584415584414e-03*w31*Jinv11-1.3149350649350649e-03*Jinv22*w26-7.6704545454545450e-04*w46*Jinv11+1.0409902597402598e-03*Jinv20*w14-3.2873376623376621e-03*w5*Jinv21+9.5332792207792200e-03*w4*Jinv12-2.5750811688311690e-03*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10+4.2004870129870131e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01-7.6704545454545450e-04*w47*Jinv21+1.8628246753246754e-03*Jinv20*w38-2.5750811688311690e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11+2.0819805194805196e-03*w31*Jinv12-8.2183441558441553e-04*w46*Jinv12-6.0267857142857139e-04*Jinv20*w53-2.7942370129870131e-03*Jinv22*w8-1.2601461038961039e-03*w22*Jinv10-1.8628246753246754e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-2.7942370129870131e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51+2.0819805194805196e-03*Jinv02*w30-1.8628246753246754e-03*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01-6.0267857142857139e-04*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00+1.8628246753246754e-03*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56-1.2053571428571428e-03*w24*Jinv00+9.6793831168831166e-04*Jinv02*w27-2.7942370129870131e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12+9.5332792207792200e-03*Jinv22*w5-8.2183441558441553e-04*w47*Jinv22+7.6704545454545450e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+5.6615259740259744e-04*w57*Jinv00+5.6615259740259744e-04*Jinv20*w59+6.9034090909090911e-03*Jinv02*w33+4.3831168831168834e-04*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-1.2053571428571428e-03*w25*Jinv10+4.3831168831168834e-04*Jinv20*w2+2.9586038961038961e-03*w15*Jinv00-1.8628246753246754e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+2.9586038961038961e-03*Jinv20*w17+2.0819805194805196e-03*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10+9.3141233766233771e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43+9.8620129870129864e-04*w5*Jinv20-2.1915584415584414e-03*Jinv01*w30+9.6793831168831166e-04*Jinv12*w28-1.1505681818181819e-03*w25*Jinv11-3.2873376623376621e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50+4.2004870129870131e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[5][36]=(0.0)*D;
  A[5][37]=(0.0)*D;
  A[5][38]=(1.9176136363636362e-04*Jinv00*w12-3.1960227272727273e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-4.2918019480519480e-04*w57*Jinv02+1.9176136363636362e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+1.4793019480519481e-03*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20+2.4655032467532466e-04*Jinv20*w23-2.3833198051948050e-03*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-4.7483766233766234e-04*w0*Jinv02-2.5202922077922079e-03*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+1.3697240259740259e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-4.8396915584415583e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+4.6570616883116886e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-3.1960227272727273e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+1.3697240259740259e-04*Jinv20*w50-3.1960227272727273e-04*Jinv20*w29+4.6570616883116886e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-2.7394480519480520e-03*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10+1.1505681818181819e-03*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00-1.6436688311688311e-04*w32*Jinv22-4.7483766233766234e-04*Jinv22*w2-1.0135957792207792e-03*Jinv00*w18-2.0271915584415584e-03*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45-4.2918019480519480e-04*Jinv22*w59+4.3831168831168834e-04*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10-7.1225649350649346e-04*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13-1.0135957792207792e-03*w19*Jinv10-2.7394480519480520e-03*w34*Jinv10-1.2327516233766234e-03*w6*Jinv00-2.3833198051948050e-03*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+8.4922889610389610e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+1.9176136363636362e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54+1.1505681818181819e-03*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02+1.3697240259740259e-04*w49*Jinv10-7.1225649350649346e-04*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.2327516233766234e-03*w7*Jinv10-2.0271915584415584e-03*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23-1.6436688311688311e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10+5.2049512987012989e-04*Jinv22*w50-2.6298701298701297e-03*Jinv12*w7-1.6436688311688311e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+4.3831168831168834e-04*w52*Jinv10+1.1870941558441559e-04*w9*Jinv00-2.0271915584415584e-03*w37*Jinv10+8.4922889610389610e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.1505681818181819e-03*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+1.1870941558441559e-04*w10*Jinv10-2.5202922077922079e-03*Jinv20*w44-4.2918019480519480e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-4.8396915584415583e-04*w0*Jinv00-2.7394480519480520e-03*w35*Jinv20+8.4922889610389610e-04*w25*Jinv10-4.8396915584415583e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00+5.2049512987012989e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-1.0135957792207792e-03*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-2.3833198051948050e-03*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+1.3971185064935066e-03*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28+2.4655032467532466e-04*w55*Jinv10-2.5202922077922079e-03*w42*Jinv00+4.6570616883116886e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-4.7483766233766234e-04*Jinv12*w1)*D;
  A[5][39]=(0.0)*D;
  A[5][40]=(0.0)*D;
  A[5][41]=(-1.0957792207792207e-03*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+3.8352272727272725e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-1.0957792207792207e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.0957792207792207e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+3.2873376623376621e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+3.6525974025974026e-05*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01-5.4788961038961042e-05*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.0409902597402598e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.7532467532467534e-03*w22*Jinv12-1.0957792207792207e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00+1.0409902597402598e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+3.1047077922077924e-04*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+3.2873376623376621e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-1.0957792207792207e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-1.7532467532467534e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9-1.7532467532467534e-03*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+5.4788961038961042e-05*Jinv20*w38-1.7532467532467534e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+1.0409902597402598e-03*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53-1.7532467532467534e-03*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-9.3141233766233771e-04*Jinv22*w50-9.3141233766233771e-04*w22*Jinv11-1.7532467532467534e-03*Jinv12*w7-9.3141233766233771e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+1.0409902597402598e-03*Jinv02*w30-9.3141233766233771e-04*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.7532467532467534e-03*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-3.2873376623376621e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-1.7532467532467534e-03*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22-1.7532467532467534e-03*w49*Jinv11+1.0409902597402598e-03*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+3.2873376623376621e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18+1.0409902597402598e-03*Jinv01*w12-1.7532467532467534e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-9.3141233766233771e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-9.3141233766233771e-04*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-1.0957792207792207e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30+5.2962662337662338e-04*Jinv12*w28+5.4788961038961042e-05*w55*Jinv10+3.2873376623376621e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12-1.7532467532467534e-03*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-1.7532467532467534e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[5][42]=(0.0)*D;
  A[5][43]=(0.0)*D;
  A[5][44]=(1.0957792207792207e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-4.2004870129870131e-04*w57*Jinv02+1.0957792207792207e-03*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+1.0957792207792207e-03*w32*Jinv20+3.2873376623376621e-03*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-9.8620129870129864e-04*w0*Jinv02-9.8620129870129864e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-3.9448051948051946e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-9.8620129870129864e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-9.8620129870129864e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-4.2004870129870131e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10-3.9448051948051946e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12+1.0957792207792207e-03*w31*Jinv10+3.2873376623376621e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-9.8620129870129864e-04*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45+1.1505681818181819e-03*Jinv01*w24-4.2004870129870131e-04*Jinv22*w59-3.9448051948051946e-03*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+3.2873376623376621e-03*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20+2.1915584415584414e-03*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+2.1915584415584414e-03*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12-3.9448051948051946e-03*Jinv21*w35+2.1915584415584414e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26+7.6704545454545450e-04*w46*Jinv11+1.0957792207792207e-03*Jinv20*w14+3.2873376623376621e-03*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+3.2873376623376621e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-4.2004870129870131e-04*Jinv21*w29-3.9448051948051946e-03*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00+2.1915584415584414e-03*Jinv02*w12-1.3149350649350649e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11-3.6525974025974026e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+3.2873376623376621e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-3.6525974025974026e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-4.2004870129870131e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+1.0957792207792207e-03*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29+2.1915584415584414e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-1.3149350649350649e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+2.1915584415584414e-03*Jinv01*w30+1.1505681818181819e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28+1.1505681818181819e-03*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20-3.9448051948051946e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50-4.2004870129870131e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01-9.8620129870129864e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[5][45]=(0.0)*D;
  A[5][46]=(0.0)*D;
  A[5][47]=(-3.8352272727272725e-04*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53-1.0957792207792208e-04*w57*Jinv02-3.8352272727272725e-04*Jinv02*w39-8.7662337662337668e-04*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-1.3697240259740259e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-8.7662337662337668e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-7.6704545454545450e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+5.2049512987012989e-04*Jinv22*w20-8.7662337662337668e-04*w43*Jinv11-7.1225649350649346e-04*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1+1.9176136363636362e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.6704545454545450e-04*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-3.5612824675324673e-04*w32*Jinv22-7.1225649350649346e-04*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+5.4788961038961036e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-7.1225649350649346e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-3.8352272727272725e-04*Jinv22*w41+8.2183441558441553e-05*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01-3.5612824675324673e-04*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11+2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+5.2049512987012989e-04*w49*Jinv11+1.9176136363636362e-04*Jinv21*w14-1.0957792207792208e-04*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+1.9176136363636362e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-8.2183441558441553e-05*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43-7.1225649350649346e-04*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-7.6704545454545450e-04*w16*Jinv12+5.2049512987012989e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[5][48]=(0.0)*D;
  A[5][49]=(0.0)*D;
  A[5][50]=(3.5612824675324673e-04*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+5.4788961038961036e-04*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+7.1225649350649346e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+4.6570616883116886e-04*w25*Jinv12-5.5702110389610384e-04*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-2.5750811688311690e-03*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+4.6570616883116886e-04*Jinv02*w24-8.7662337662337668e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10-2.5750811688311690e-03*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33+1.1231737012987013e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10-4.0543831168831168e-03*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-5.5702110389610384e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38-2.7394480519480521e-05*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-8.7662337662337668e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+7.1225649350649346e-04*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20-2.0271915584415584e-03*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41-8.2183441558441553e-05*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+5.4788961038961036e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+4.6570616883116886e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+7.1225649350649346e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54-4.0543831168831168e-03*w4*Jinv12+1.1231737012987013e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00-2.0271915584415584e-03*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11-1.7349837662337662e-04*Jinv02*w9-5.2049512987012989e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+1.1231737012987013e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+9.5880681818181818e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10+7.3965097402597403e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+9.5880681818181818e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30+7.3965097402597403e-04*Jinv12*w49-5.2049512987012989e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-5.2049512987012989e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-8.2183441558441553e-05*w24*Jinv00-3.1960227272727273e-04*Jinv02*w27+9.5880681818181818e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12-4.0543831168831168e-03*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-5.2049512987012989e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-8.7662337662337668e-04*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-8.2183441558441553e-05*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.3965097402597403e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-3.1960227272727273e-04*Jinv22*w29-2.0271915584415584e-03*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+5.4788961038961036e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-3.1960227272727273e-04*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20-2.5750811688311690e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-5.5702110389610384e-04*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[5][51]=(0.0)*D;
  A[5][52]=(0.0)*D;
  A[5][53]=(-3.5612824675324673e-04*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-1.0683847402597404e-03*Jinv22*w53+5.2049512987012989e-04*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+2.4655032467532466e-04*w25*Jinv12+1.8262987012987013e-05*w0*Jinv02-5.4788961038961042e-05*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+1.3971185064935066e-03*Jinv22*w20+2.7394480519480518e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+1.8262987012987013e-05*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+4.6570616883116886e-04*Jinv02*w45+1.2784090909090910e-04*Jinv22*w59-7.1225649350649346e-04*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10-3.8352272727272725e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10+5.2049512987012989e-04*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.0683847402597404e-03*w52*Jinv12+2.4655032467532466e-04*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10-3.8352272727272725e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9+5.2049512987012989e-04*w7*Jinv10+5.2049512987012989e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+3.8352272727272725e-04*w31*Jinv12+4.6570616883116886e-04*w46*Jinv12-7.1225649350649346e-04*Jinv20*w53+1.4793019480519481e-03*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50+1.4793019480519481e-03*Jinv12*w7+3.8352272727272725e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-7.1225649350649346e-04*w52*Jinv10-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+8.2183441558441553e-05*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+1.4793019480519481e-03*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5+4.6570616883116886e-04*w47*Jinv22-1.3697240259740259e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+8.2183441558441553e-05*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+8.2183441558441553e-05*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20-1.0683847402597404e-03*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-3.5612824675324673e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12+1.8262987012987013e-05*Jinv12*w1)*D;
  A[5][54]=(0.0)*D;
  A[5][55]=(0.0)*D;
  A[5][56]=(-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+6.3920454545454550e-05*w57*Jinv02-7.1225649350649346e-04*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+2.4655032467532466e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12+5.4788961038961036e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+5.4788961038961036e-04*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11-3.8352272727272725e-04*Jinv21*w41-8.7662337662337668e-04*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-1.0957792207792208e-04*Jinv22*w17+5.2049512987012989e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3+1.9176136363636362e-04*w32*Jinv22-3.8352272727272725e-04*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+5.2049512987012989e-04*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+6.3920454545454550e-05*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-8.7662337662337668e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+2.7394480519480518e-04*w47*Jinv21+5.2049512987012989e-04*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+1.9176136363636362e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8-8.2183441558441553e-05*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+1.9176136363636362e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+5.2049512987012989e-04*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-3.5612824675324673e-04*Jinv21*w56-1.3697240259740259e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+6.3920454545454550e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-8.2183441558441553e-05*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-3.8352272727272725e-04*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36-8.7662337662337668e-04*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-1.0957792207792208e-04*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[5][57]=(0.0)*D;
  A[5][58]=(0.0)*D;
  A[5][59]=(2.1915584415584417e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+2.1915584415584417e-04*Jinv02*w39+1.1870941558441559e-04*Jinv22*w38-2.1915584415584417e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-5.4788961038961042e-05*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12-2.1915584415584417e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35-4.2918019480519480e-04*Jinv22*w20+3.4699675324675324e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12+1.4975649350649350e-03*Jinv02*w3+3.6525974025974026e-05*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2-2.7394480519480521e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+4.3831168831168834e-04*Jinv12*w34+3.4699675324675324e-04*Jinv22*w44+7.4878246753246752e-04*Jinv12*w13+1.1870941558441559e-04*w37*Jinv12+2.1915584415584417e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12-2.1915584415584417e-04*Jinv22*w26-5.4788961038961042e-05*Jinv02*w54+1.4975649350649350e-03*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02+7.4878246753246752e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-4.2918019480519480e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12-5.8441558441558442e-04*Jinv22*w8-2.6481331168831169e-04*Jinv22*w50-5.8441558441558442e-04*Jinv12*w7+3.6525974025974026e-05*Jinv02*w30-2.6481331168831169e-04*Jinv12*w49+1.0247564935064935e-04*Jinv02*w27-5.8441558441558442e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12+1.4975649350649350e-03*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+4.3831168831168834e-04*Jinv02*w33-2.6481331168831169e-04*w48*Jinv02+1.0247564935064935e-04*Jinv22*w29+7.4878246753246752e-04*Jinv22*w14+1.1870941558441559e-04*Jinv02*w36+3.4699675324675324e-04*Jinv12*w43-1.0957792207792208e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[6][0]=(6.7573051948051951e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10-2.9220779220779221e-04*w40*Jinv12-1.1870941558441559e-04*Jinv22*w53+2.5568181818181820e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02+6.7573051948051951e-04*w13*Jinv10-2.9220779220779221e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+1.0227272727272728e-03*w3*Jinv00+6.7573051948051951e-04*w32*Jinv20+1.0227272727272728e-03*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-3.6525974025974026e-05*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59-1.1870941558441559e-04*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-8.3198051948051951e-05*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17+2.7394480519480521e-05*Jinv01*w45+2.7394480519480521e-05*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02-8.3198051948051951e-05*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-1.1870941558441559e-04*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01-1.1870941558441559e-04*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-3.6525974025974026e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00-2.9220779220779221e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27-8.3198051948051951e-05*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+6.7573051948051951e-04*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+6.7573051948051951e-04*w31*Jinv10+1.0227272727272728e-03*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+6.7573051948051951e-04*w32*Jinv22-2.9220779220779221e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2-3.6525974025974026e-05*Jinv00*w18-3.6525974025974026e-05*w36*Jinv00-3.6525974025974026e-05*Jinv21*w38+2.7394480519480521e-05*Jinv02*w45-1.1870941558441559e-04*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11-3.6525974025974026e-05*w19*Jinv11-1.1870941558441559e-04*Jinv21*w53-1.1870941558441559e-04*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+1.0227272727272728e-03*w4*Jinv10+1.0227272727272728e-03*w4*Jinv11-2.9220779220779221e-04*w41*Jinv20+6.7573051948051951e-04*Jinv12*w13-3.6525974025974026e-05*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+6.7573051948051951e-04*w32*Jinv21-3.6525974025974026e-05*Jinv21*w20+2.5568181818181820e-04*w6*Jinv00-3.6525974025974026e-05*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56-2.9220779220779221e-04*Jinv22*w41-1.1870941558441559e-04*w26*Jinv20-8.3198051948051951e-05*w11*Jinv20-1.1870941558441559e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35+6.7573051948051951e-04*w31*Jinv11-1.1870941558441559e-04*Jinv22*w26+2.7394480519480521e-05*w46*Jinv11+6.7573051948051951e-04*Jinv20*w14+1.0227272727272728e-03*w5*Jinv21+2.7394480519480521e-05*Jinv02*w54+1.0227272727272728e-03*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33-2.9220779220779221e-04*w39*Jinv00+6.7573051948051951e-04*Jinv02*w12-3.6525974025974026e-05*Jinv01*w18-3.6525974025974026e-05*w37*Jinv11-8.3198051948051951e-05*Jinv02*w9+2.5568181818181820e-04*w6*Jinv01+2.5568181818181820e-04*w7*Jinv10+2.7394480519480521e-05*w47*Jinv21-3.6525974025974026e-05*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-1.1870941558441559e-04*w52*Jinv11+6.7573051948051951e-04*w31*Jinv12+2.7394480519480521e-05*w46*Jinv12-1.1870941558441559e-04*Jinv20*w53+2.5568181818181820e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11+2.5568181818181820e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01-1.1870941558441559e-04*Jinv01*w51+6.7573051948051951e-04*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49-3.6525974025974026e-05*w36*Jinv01-1.1870941558441559e-04*w52*Jinv10+2.5568181818181820e-04*w7*Jinv11-8.3198051948051951e-05*w9*Jinv00-3.6525974025974026e-05*w37*Jinv10+2.7394480519480521e-05*Jinv21*w56-1.1870941558441559e-04*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27+2.5568181818181820e-04*w6*Jinv02-3.6525974025974026e-05*w19*Jinv12+1.0227272727272728e-03*Jinv22*w5+2.7394480519480521e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+6.7573051948051951e-04*Jinv21*w14-8.3198051948051951e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.6525974025974026e-05*Jinv02*w18+6.7573051948051951e-04*Jinv01*w12+2.5568181818181820e-04*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23+2.7394480519480521e-05*Jinv01*w54+2.7394480519480521e-05*w55*Jinv11-1.1870941558441559e-04*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02-8.3198051948051951e-05*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17+6.7573051948051951e-04*w30*Jinv00-2.9220779220779221e-04*w40*Jinv10-3.6525974025974026e-05*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+6.7573051948051951e-04*Jinv22*w14-2.9220779220779221e-04*w40*Jinv11-3.6525974025974026e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+1.0227272727272728e-03*w5*Jinv20+6.7573051948051951e-04*Jinv01*w30-1.1870941558441559e-04*Jinv02*w51+2.7394480519480521e-05*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28-1.1870941558441559e-04*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1-8.3198051948051951e-05*w10*Jinv11)*D;
  A[6][1]=(0.0)*D;
  A[6][2]=(0.0)*D;
  A[6][3]=(-1.1505681818181819e-03*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-2.6298701298701297e-03*Jinv20*w8+1.5523538961038962e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+8.2183441558441553e-04*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3+9.5880681818181818e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.5523538961038962e-04*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+1.4610389610389610e-04*w0*Jinv02+1.4610389610389610e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+8.2183441558441553e-04*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+9.5880681818181818e-04*w48*Jinv00-8.2183441558441553e-05*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35+1.5523538961038962e-04*w58*Jinv11+1.5523538961038962e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26+1.4610389610389610e-04*w0*Jinv01-4.0178571428571428e-04*w1*Jinv10-1.4793019480519481e-03*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+8.2183441558441553e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+8.2183441558441553e-04*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22+1.4610389610389610e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+1.5523538961038962e-04*w28*Jinv11+9.5880681818181818e-04*Jinv20*w50-2.7394480519480518e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17-1.4793019480519481e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12-1.1505681818181819e-03*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3+9.5880681818181818e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39+1.4610389610389610e-04*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24+1.5523538961038962e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-8.2183441558441553e-05*Jinv21*w53+4.3831168831168834e-04*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34+8.2183441558441553e-04*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-1.4793019480519481e-03*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20-2.6298701298701297e-03*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41+4.3831168831168834e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20-8.2183441558441553e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02+9.5880681818181818e-04*w49*Jinv10+1.5523538961038962e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.0957792207792207e-03*Jinv02*w9-3.7804383116883116e-03*w6*Jinv01-2.6298701298701297e-03*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-3.7804383116883116e-03*Jinv22*w8+9.5880681818181818e-04*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-3.7804383116883116e-03*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01+4.3831168831168834e-04*w52*Jinv10-3.7804383116883116e-03*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00+1.5523538961038962e-04*Jinv02*w27-3.7804383116883116e-03*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+1.5523538961038962e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-3.7804383116883116e-03*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33-4.0178571428571428e-04*w0*Jinv00-1.4793019480519481e-03*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-4.0178571428571428e-04*Jinv20*w2-1.4793019480519481e-03*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02+1.0957792207792207e-03*Jinv01*w9-1.4793019480519481e-03*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20+1.5523538961038962e-04*Jinv22*w29+1.2327516233766234e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36+8.2183441558441553e-04*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+1.5523538961038962e-04*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+1.5523538961038962e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01+1.4610389610389610e-04*Jinv12*w1+1.0957792207792207e-03*w10*Jinv11)*D;
  A[6][4]=(0.0)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(7.3965097402597403e-04*Jinv00*w12-1.0957792207792207e-03*w40*Jinv12+2.3833198051948050e-03*Jinv22*w53-7.4878246753246752e-04*w57*Jinv02+7.3965097402597403e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+7.3965097402597403e-04*w32*Jinv20+2.6298701298701297e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-3.6982548701298703e-03*Jinv22*w38-7.4878246753246752e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12-7.8530844155844158e-04*w0*Jinv02-7.8530844155844158e-04*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-4.2735389610389614e-03*w16*Jinv11-4.2735389610389614e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-4.2735389610389614e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24-4.2735389610389614e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+2.3833198051948050e-03*Jinv21*w26-7.8530844155844158e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10-3.6982548701298703e-03*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.0957792207792207e-03*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-1.0957792207792207e-03*w11*Jinv22-7.8530844155844158e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11-4.2735389610389614e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50-4.2735389610389614e-03*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12+7.3965097402597403e-04*w31*Jinv10+2.6298701298701297e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.8530844155844158e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-3.6982548701298703e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+2.3833198051948050e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11-3.6982548701298703e-03*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.2735389610389614e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+2.6298701298701297e-03*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-3.6982548701298703e-03*Jinv21*w20-3.6982548701298703e-03*w37*Jinv12-1.0957792207792207e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-1.1870941558441558e-03*w11*Jinv20+2.3833198051948050e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+7.3965097402597403e-04*Jinv20*w14+2.6298701298701297e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54+2.6298701298701297e-03*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-3.6982548701298703e-03*Jinv01*w18-3.6982548701298703e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-7.3965097402597406e-03*w6*Jinv01+3.5612824675324673e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-7.3965097402597406e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11-7.3965097402597406e-03*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49-3.6982548701298703e-03*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-7.3965097402597406e-03*w7*Jinv11-1.1870941558441558e-03*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27-7.3965097402597406e-03*w6*Jinv02-3.6982548701298703e-03*w19*Jinv12+2.6298701298701297e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-1.1870941558441558e-03*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-3.6982548701298703e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-7.3965097402597406e-03*Jinv21*w8-4.2735389610389614e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+7.3965097402597403e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-3.6982548701298703e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.3833198051948050e-03*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+2.3833198051948050e-03*w25*Jinv11-5.4788961038961036e-04*w42*Jinv00-4.2735389610389614e-03*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01-7.8530844155844158e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[6][7]=(0.0)*D;
  A[6][8]=(0.0)*D;
  A[6][9]=(-2.6481331168831169e-04*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+2.3741883116883116e-03*Jinv20*w8-2.6481331168831169e-04*w13*Jinv10-1.0227272727272728e-03*w3*Jinv00-2.6481331168831169e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23-5.4788961038961042e-05*Jinv00*w54+3.2873376623376621e-04*w43*Jinv10+1.5726461038961039e-04*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00+2.0495129870129870e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-5.4788961038961042e-05*w45*Jinv00+1.5726461038961039e-04*Jinv00*w27-1.0957792207792208e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-2.6481331168831169e-04*w31*Jinv10-1.0957792207792208e-04*w21*Jinv00+1.1870941558441558e-03*Jinv00*w18+1.1870941558441558e-03*w36*Jinv00-6.7573051948051951e-04*w51*Jinv00-1.0227272727272728e-03*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+1.1870941558441558e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+2.3741883116883116e-03*w6*Jinv00-5.4788961038961042e-05*Jinv20*w56-6.7573051948051951e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-2.6481331168831169e-04*Jinv20*w14-1.0957792207792208e-04*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+2.3741883116883116e-03*w7*Jinv10+1.1870941558441558e-03*Jinv20*w38-6.7573051948051951e-04*Jinv20*w53-1.0957792207792208e-04*w22*Jinv10-6.7573051948051951e-04*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+1.1870941558441558e-03*w37*Jinv10-6.7573051948051951e-04*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+2.0495129870129870e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-6.7573051948051951e-04*w25*Jinv10+2.0495129870129870e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+6.5746753246753243e-04*Jinv20*w17-2.6481331168831169e-04*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+1.1870941558441558e-03*w20*Jinv20-1.0227272727272728e-03*w5*Jinv20-5.4788961038961042e-05*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20)*D;
  A[6][10]=(0.0)*D;
  A[6][11]=(0.0)*D;
  A[6][12]=(2.0271915584415584e-03*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+2.5202922077922079e-03*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8+3.1960227272727273e-04*w57*Jinv02+2.0271915584415584e-03*w13*Jinv10+2.5202922077922079e-03*Jinv02*w39+3.2873376623376621e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.0135957792207792e-03*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23-1.9176136363636362e-04*Jinv22*w38-4.6570616883116886e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59-1.3697240259740259e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-6.6659902597402602e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+3.2873376623376621e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21+6.0267857142857139e-04*w16*Jinv11+6.0267857142857139e-04*Jinv21*w17+2.7394480519480521e-05*Jinv01*w45-4.6570616883116886e-04*Jinv12*w55+3.1960227272727273e-04*w58*Jinv10+2.7394480519480520e-03*w15*Jinv02+4.8396915584415583e-04*w10*Jinv12-8.4922889610389610e-04*w48*Jinv00-1.3697240259740259e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26-6.6659902597402602e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+2.7394480519480520e-03*w16*Jinv10-3.8352272727272725e-04*Jinv22*w20+3.2873376623376621e-04*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+1.2053571428571428e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+4.8396915584415583e-04*w11*Jinv22-6.6659902597402602e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+6.0267857142857139e-04*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11-8.4922889610389610e-04*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+2.7394480519480520e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12+1.0135957792207792e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.0135957792207792e-03*w32*Jinv22+1.2053571428571428e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-3.8352272727272725e-04*Jinv00*w18-1.9176136363636362e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24+3.1960227272727273e-04*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11+2.5202922077922079e-03*w41*Jinv20+2.0271915584415584e-03*Jinv12*w13-3.8352272727272725e-04*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.3697240259740260e-03*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00-1.9176136363636362e-04*w37*Jinv12-4.6570616883116886e-04*Jinv20*w56+2.5202922077922079e-03*Jinv22*w41-1.3697240259740259e-04*w26*Jinv20+4.8396915584415583e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.3697240259740260e-03*w31*Jinv11-1.3697240259740259e-04*Jinv22*w26+2.7394480519480521e-05*w46*Jinv11+2.0271915584415584e-03*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21-4.6570616883116886e-04*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02-8.4922889610389610e-04*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+2.5202922077922079e-03*w39*Jinv00+2.0271915584415584e-03*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11+4.8396915584415583e-04*Jinv02*w9-3.2873376623376621e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+2.7394480519480521e-05*w47*Jinv21-1.9176136363636362e-04*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23+2.7394480519480518e-04*w52*Jinv11+1.0135957792207792e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-1.4793019480519481e-03*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-8.4922889610389610e-04*Jinv22*w50+9.5880681818181818e-04*w22*Jinv11-1.4793019480519481e-03*Jinv12*w7+9.5880681818181818e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51+1.0135957792207792e-03*Jinv02*w30-8.4922889610389610e-04*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10-3.2873376623376621e-04*w7*Jinv11+4.8396915584415583e-04*w9*Jinv00-1.9176136363636362e-04*w37*Jinv10-1.9176136363636362e-04*Jinv21*w56-1.3697240259740259e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27-1.4793019480519481e-03*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12+1.2327516233766234e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+4.8396915584415583e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44+3.1960227272727273e-04*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12-3.2873376623376621e-04*Jinv21*w8+3.1960227272727273e-04*w57*Jinv00+3.1960227272727273e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20+9.5880681818181818e-04*Jinv21*w23-1.9176136363636362e-04*Jinv01*w54-1.9176136363636362e-04*w55*Jinv11-1.3697240259740259e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+2.7394480519480520e-03*w15*Jinv00-8.4922889610389610e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+2.7394480519480520e-03*Jinv20*w17+1.0135957792207792e-03*w30*Jinv00+2.5202922077922079e-03*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29+2.0271915584415584e-03*Jinv22*w14+1.2053571428571428e-03*w40*Jinv11-1.9176136363636362e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.3697240259740260e-03*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-4.6570616883116886e-04*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-4.6570616883116886e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+2.7394480519480520e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[6][13]=(0.0)*D;
  A[6][14]=(0.0)*D;
  A[6][15]=(-1.8628246753246754e-03*Jinv00*w12-5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-9.8620129870129864e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-1.8628246753246754e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.3141233766233771e-04*w32*Jinv20-2.7942370129870131e-03*Jinv01*w3+6.0267857142857139e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02+1.6436688311688311e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-6.9399350649350649e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+1.2053571428571428e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01-4.9310064935064932e-04*Jinv21*w26+1.6436688311688311e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20+3.2873376623376621e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.6615259740259744e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+1.6436688311688311e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+1.2053571428571428e-03*Jinv20*w50-5.6615259740259744e-04*Jinv20*w29+3.2873376623376621e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-2.9586038961038961e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12-9.3141233766233771e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3+6.0267857142857139e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-2.0819805194805196e-03*Jinv00*w18-1.0409902597402598e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45-4.9310064935064932e-04*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-1.3149350649350649e-03*Jinv21*w53+1.2601461038961039e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-2.7942370129870131e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-2.0819805194805196e-03*w19*Jinv10-2.9586038961038961e-03*w34*Jinv10-1.7532467532467534e-03*w32*Jinv21-9.8620129870129864e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20-4.3831168831168834e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-1.7532467532467534e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+3.2873376623376621e-04*w46*Jinv11-1.8628246753246754e-03*Jinv20*w14-2.7942370129870131e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02+1.2053571428571428e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12+2.1367694805194807e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+8.5470779220779228e-03*w6*Jinv01-9.8620129870129864e-04*w7*Jinv10+3.2873376623376621e-04*w47*Jinv21-1.0409902597402598e-03*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23-1.3149350649350649e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+1.2601461038961039e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8+6.0267857142857139e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+6.0267857142857139e-04*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+6.0267857142857139e-04*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+1.2601461038961039e-03*w52*Jinv10+8.5470779220779228e-03*w7*Jinv11-4.3831168831168834e-04*w9*Jinv00-1.0409902597402598e-03*w37*Jinv10+1.3697240259740260e-03*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14-4.3831168831168834e-04*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12+8.5470779220779228e-03*Jinv21*w8-6.9399350649350649e-04*w57*Jinv00-6.9399350649350649e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-2.9586038961038961e-03*w35*Jinv20+6.0267857142857139e-04*Jinv21*w23+1.3697240259740260e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17-9.3141233766233771e-04*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10-2.0819805194805196e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.7532467532467534e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28-4.9310064935064932e-04*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+3.2873376623376621e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[6][16]=(0.0)*D;
  A[6][17]=(0.0)*D;
  A[6][18]=(-1.6436688311688311e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+2.4655032467532467e-03*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10+2.1367694805194807e-03*Jinv21*w44-1.2327516233766234e-03*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20-2.6298701298701297e-03*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23-3.5612824675324673e-04*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.8530844155844158e-04*Jinv21*w2+1.5888798701298701e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+4.2735389610389614e-03*w16*Jinv11+4.2735389610389614e-03*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+3.7439123376623376e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01-2.3833198051948050e-03*Jinv21*w26+7.8530844155844158e-04*w0*Jinv01+3.9265422077922079e-04*w1*Jinv10+3.1777597402597403e-03*w16*Jinv10+2.1367694805194807e-03*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+3.9265422077922079e-04*Jinv00*w27+7.8530844155844158e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11+4.2735389610389614e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-8.2183441558441553e-05*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+2.4655032467532467e-03*Jinv00*w18+1.2327516233766234e-03*w36*Jinv00+3.6982548701298703e-03*Jinv21*w38-2.3833198051948050e-03*Jinv01*w24+4.2735389610389614e-03*w34*Jinv11+3.6982548701298703e-03*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.0135957792207792e-03*w51*Jinv00-1.2327516233766234e-03*w4*Jinv10-2.6298701298701297e-03*w4*Jinv11+1.0957792207792207e-03*w41*Jinv20+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21+3.6982548701298703e-03*Jinv21*w20+2.4655032467532467e-03*w6*Jinv00-3.5612824675324673e-04*Jinv20*w56-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20+4.2735389610389614e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-2.6298701298701297e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33+1.0957792207792207e-03*w39*Jinv00+3.6982548701298703e-03*Jinv01*w18+3.6982548701298703e-03*w37*Jinv11+7.3965097402597406e-03*w6*Jinv01+2.4655032467532467e-03*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+1.2327516233766234e-03*Jinv20*w38-2.3833198051948050e-03*w52*Jinv11-1.0135957792207792e-03*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-7.1225649350649346e-04*w22*Jinv11-7.1225649350649346e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+3.6982548701298703e-03*w36*Jinv01-1.0135957792207792e-03*w52*Jinv10+7.3965097402597406e-03*w7*Jinv11-4.5657467532467534e-05*w9*Jinv00+1.2327516233766234e-03*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56-1.2327516233766234e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14-4.5657467532467534e-05*w10*Jinv10+1.5888798701298701e-03*Jinv20*w44-1.6436688311688311e-04*Jinv01*w12+7.3965097402597406e-03*Jinv21*w8+3.7439123376623376e-04*w57*Jinv00+3.7439123376623376e-04*Jinv20*w59+3.9265422077922079e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-7.1225649350649346e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-1.2327516233766234e-03*w25*Jinv10+3.9265422077922079e-04*Jinv20*w2+3.1777597402597403e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+3.1777597402597403e-03*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-1.2327516233766234e-03*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30-2.3833198051948050e-03*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10+1.5888798701298701e-03*w42*Jinv00-8.2183441558441553e-05*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[6][19]=(0.0)*D;
  A[6][20]=(0.0)*D;
  A[6][21]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39-3.2873376623376621e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20+9.5880681818181818e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+3.5612824675324673e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10-3.2873376623376621e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.7394480519480518e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+3.5612824675324673e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20-3.2873376623376621e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.2601461038961039e-03*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+3.5612824675324673e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-1.2601461038961039e-03*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10+9.5880681818181818e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+1.1231737012987013e-03*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+1.1231737012987013e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26-5.2049512987012989e-04*Jinv20*w14+9.5880681818181818e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9-1.4245129870129869e-03*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+3.5612824675324673e-04*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-1.4245129870129869e-03*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+1.9176136363636362e-04*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12-1.4245129870129869e-03*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+1.9176136363636362e-04*Jinv01*w54+1.9176136363636362e-04*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14-1.2601461038961039e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20+1.1231737012987013e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28-2.7394480519480518e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[6][22]=(0.0)*D;
  A[6][23]=(0.0)*D;
  A[6][24]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*Jinv20*w8+5.2049512987012989e-04*w13*Jinv10-8.2183441558441553e-04*Jinv21*w44+5.2049512987012989e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+1.4793019480519481e-03*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23+2.7394480519480518e-04*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-3.1047077922077924e-04*Jinv21*w2-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-1.6436688311688311e-03*w16*Jinv11-1.6436688311688311e-03*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45-1.0957792207792208e-04*w58*Jinv10+8.2183441558441553e-05*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+5.7528409090909093e-04*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-8.2183441558441553e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-3.1047077922077924e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-1.6436688311688311e-03*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11+8.2183441558441553e-05*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39-1.6436688311688311e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+5.7528409090909093e-04*Jinv01*w24-2.6298701298701297e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00+5.2049512987012989e-04*w4*Jinv10+1.4793019480519481e-03*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-1.6436688311688311e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-7.1225649350649346e-04*w6*Jinv00+2.7394480519480518e-04*Jinv20*w56-1.6436688311688311e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20-2.6298701298701297e-03*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-8.2183441558441553e-05*w46*Jinv11+5.2049512987012989e-04*Jinv20*w14+1.4793019480519481e-03*w5*Jinv21+8.2183441558441553e-05*w49*Jinv10-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.4655032467532466e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-4.7666396103896100e-03*w6*Jinv01-7.1225649350649346e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10-4.7666396103896100e-03*w7*Jinv11+9.1314935064935065e-06*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+5.4788961038961036e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14+9.1314935064935065e-06*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+1.3697240259740259e-04*Jinv01*w12-4.7666396103896100e-03*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+5.2049512987012989e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+5.7528409090909093e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+5.4788961038961036e-04*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[6][25]=(0.0)*D;
  A[6][26]=(0.0)*D;
  A[6][27]=(2.1915584415584417e-04*Jinv21*w44-5.8441558441558442e-04*Jinv01*w3+1.0247564935064935e-04*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+2.1915584415584417e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-2.6481331168831169e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+2.1915584415584417e-04*w43*Jinv11+3.4699675324675324e-04*Jinv21*w41-1.4204545454545455e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+3.4699675324675324e-04*Jinv01*w39+7.4878246753246752e-04*Jinv21*w38-2.6481331168831169e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-5.8441558441558442e-04*w4*Jinv11-4.2918019480519480e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-5.8441558441558442e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+3.6525974025974026e-05*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+1.4975649350649350e-03*w6*Jinv01-5.4788961038961042e-05*w47*Jinv21-4.2918019480519480e-04*w52*Jinv11-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01+1.4975649350649350e-03*w7*Jinv11-2.7394480519480521e-05*Jinv21*w56-2.1915584415584417e-04*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+1.1870941558441559e-04*Jinv01*w12+1.4975649350649350e-03*Jinv21*w8-1.0957792207792208e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11+6.4935064935064935e-05*Jinv01*w9+3.4699675324675324e-04*w40*Jinv11-4.2918019480519480e-04*Jinv01*w30-2.6481331168831169e-04*w25*Jinv11-2.1915584415584417e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-2.1915584415584417e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[6][28]=(0.0)*D;
  A[6][29]=(0.0)*D;
  A[6][30]=(1.0135957792207792e-03*Jinv00*w12+3.1960227272727273e-04*w28*Jinv10+1.2053571428571428e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02+1.0135957792207792e-03*w13*Jinv10+1.2053571428571428e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+2.0271915584415584e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3-8.4922889610389610e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59+2.7394480519480518e-04*w25*Jinv12-6.6659902597402602e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+4.8396915584415583e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-4.6570616883116886e-04*Jinv01*w45+2.7394480519480521e-05*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24+6.0267857142857139e-04*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-4.6570616883116886e-04*w45*Jinv00+2.5202922077922079e-03*Jinv21*w41+3.2873376623376621e-04*w42*Jinv02+3.1960227272727273e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.0135957792207792e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+3.1960227272727273e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+3.1960227272727273e-04*Jinv20*w29-4.6570616883116886e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17+2.7394480519480520e-03*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12+2.0271915584415584e-03*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3-8.4922889610389610e-04*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+2.5202922077922079e-03*Jinv01*w39-6.6659902597402602e-04*Jinv22*w2-1.9176136363636362e-04*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-3.8352272727272725e-04*Jinv21*w38-1.9176136363636362e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+2.7394480519480520e-03*w34*Jinv11-1.9176136363636362e-04*w19*Jinv11-1.3697240259740259e-04*Jinv21*w53-1.3697240259740259e-04*w51*Jinv00+6.0267857142857139e-04*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11+2.5202922077922079e-03*w41*Jinv20-1.3697240259740260e-03*Jinv12*w13-1.9176136363636362e-04*w19*Jinv10+2.7394480519480520e-03*w34*Jinv10+2.0271915584415584e-03*w32*Jinv21-1.9176136363636362e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+1.2053571428571428e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+4.8396915584415583e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+2.7394480519480520e-03*Jinv21*w35+2.0271915584415584e-03*w31*Jinv11+2.7394480519480518e-04*Jinv22*w26-4.6570616883116886e-04*w46*Jinv11+1.0135957792207792e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+2.7394480519480521e-05*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+3.1960227272727273e-04*Jinv21*w29+2.7394480519480520e-03*Jinv01*w33+2.5202922077922079e-03*w39*Jinv00-1.3697240259740260e-03*Jinv02*w12-1.9176136363636362e-04*Jinv01*w18-3.8352272727272725e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-1.4793019480519481e-03*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10-4.6570616883116886e-04*w47*Jinv21-3.8352272727272725e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23-1.3697240259740259e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-1.9176136363636362e-04*w46*Jinv12-1.3697240259740259e-04*Jinv20*w53-3.2873376623376621e-04*Jinv22*w8-8.4922889610389610e-04*w22*Jinv10+9.5880681818181818e-04*Jinv22*w50-8.4922889610389610e-04*w22*Jinv11-3.2873376623376621e-04*Jinv12*w7-8.4922889610389610e-04*w21*Jinv01-1.3697240259740259e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+9.5880681818181818e-04*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-1.3697240259740259e-04*w52*Jinv10-1.4793019480519481e-03*w7*Jinv11+4.8396915584415583e-04*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-3.2873376623376621e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5-1.9176136363636362e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.0135957792207792e-03*Jinv21*w14+4.8396915584415583e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18+1.0135957792207792e-03*Jinv01*w12-1.4793019480519481e-03*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+6.0267857142857139e-04*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+2.7394480519480520e-03*w35*Jinv20-8.4922889610389610e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00+9.5880681818181818e-04*w48*Jinv02+4.8396915584415583e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+2.0271915584415584e-03*w30*Jinv00+2.5202922077922079e-03*w40*Jinv10-1.9176136363636362e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29-1.3697240259740260e-03*Jinv22*w14+2.5202922077922079e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20+2.0271915584415584e-03*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+2.7394480519480521e-05*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-4.6570616883116886e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+3.1960227272727273e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01-6.6659902597402602e-04*Jinv12*w1+4.8396915584415583e-04*w10*Jinv11)*D;
  A[6][31]=(0.0)*D;
  A[6][32]=(0.0)*D;
  A[6][33]=(-9.3141233766233771e-04*Jinv00*w12-6.9399350649350649e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12-4.9310064935064932e-04*Jinv22*w53-9.8620129870129864e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-9.3141233766233771e-04*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.8628246753246754e-03*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3+1.2053571428571428e-03*Jinv20*w23+3.2873376623376621e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59-1.3149350649350649e-03*w25*Jinv12+1.6436688311688311e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+3.2873376623376621e-04*Jinv12*w55-5.6615259740259744e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+6.0267857142857139e-04*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-2.9586038961038961e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-6.9399350649350649e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11+6.0267857142857139e-04*Jinv20*w50-6.9399350649350649e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-5.9172077922077923e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-1.8628246753246754e-03*w31*Jinv10-2.7942370129870131e-03*Jinv02*w3+1.2053571428571428e-03*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+1.6436688311688311e-04*Jinv22*w2-1.0409902597402598e-03*Jinv00*w18-2.0819805194805196e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+1.3149350649350649e-03*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+2.1367694805194807e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-1.7532467532467534e-03*Jinv12*w13-1.0409902597402598e-03*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-9.8620129870129864e-04*w6*Jinv00+3.2873376623376621e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+1.2601461038961039e-03*w26*Jinv20-4.3831168831168834e-04*w11*Jinv20-4.9310064935064932e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11-1.3149350649350649e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-9.3141233766233771e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+3.2873376623376621e-04*Jinv02*w54-2.7942370129870131e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+6.0267857142857139e-04*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-1.7532467532467534e-03*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01-9.8620129870129864e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-2.0819805194805196e-03*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12+1.3697240259740260e-03*Jinv20*w53+8.5470779220779228e-03*Jinv22*w8+1.2053571428571428e-03*w22*Jinv10+6.0267857142857139e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+8.5470779220779228e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+6.0267857142857139e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-4.3831168831168834e-04*w9*Jinv00-2.0819805194805196e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56+1.2601461038961039e-03*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27+8.5470779220779228e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-2.7942370129870131e-03*Jinv22*w5+7.6704545454545450e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-4.3831168831168834e-04*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8-5.6615259740259744e-04*w57*Jinv00-5.6615259740259744e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+1.2601461038961039e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-2.9586038961038961e-03*w15*Jinv00+6.0267857142857139e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-2.9586038961038961e-03*Jinv20*w17-1.8628246753246754e-03*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10-1.0409902597402598e-03*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-1.7532467532467534e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+7.6704545454545450e-04*Jinv01*w30-4.9310064935064932e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11+3.2873376623376621e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01+1.6436688311688311e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[6][34]=(0.0)*D;
  A[6][35]=(0.0)*D;
  A[6][36]=(-8.2183441558441553e-05*Jinv00*w12+3.7439123376623376e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.3833198051948050e-03*Jinv22*w53+2.4655032467532467e-03*Jinv20*w8+7.4878246753246752e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.2327516233766234e-03*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+3.6982548701298703e-03*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-2.3833198051948050e-03*w25*Jinv12+7.8530844155844158e-04*w0*Jinv02+1.5888798701298701e-03*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10+4.2735389610389614e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00-2.3833198051948050e-03*Jinv02*w24+4.2735389610389614e-03*Jinv22*w35+3.9265422077922079e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+3.6982548701298703e-03*Jinv22*w20-3.5612824675324673e-04*w45*Jinv00+2.1367694805194807e-03*w42*Jinv02+3.7439123376623376e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50+3.7439123376623376e-04*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+4.2735389610389614e-03*Jinv22*w17+3.1777597402597403e-03*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-2.6298701298701297e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-1.6436688311688311e-04*w32*Jinv22+7.8530844155844158e-04*Jinv22*w2+1.2327516233766234e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00-3.5612824675324673e-04*Jinv02*w45+7.4878246753246752e-04*Jinv22*w59-1.2327516233766234e-03*w51*Jinv00+4.2735389610389614e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-1.2327516233766234e-03*w4*Jinv10+1.0957792207792207e-03*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13+1.2327516233766234e-03*w19*Jinv10+3.1777597402597403e-03*w34*Jinv10+2.4655032467532467e-03*w6*Jinv00+3.6982548701298703e-03*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-1.0135957792207792e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20-2.3833198051948050e-03*w52*Jinv12-2.3833198051948050e-03*Jinv22*w26-8.2183441558441553e-05*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-2.6298701298701297e-03*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10+1.0957792207792207e-03*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+2.4655032467532467e-03*w7*Jinv10+2.4655032467532467e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-1.6436688311688311e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12-1.2327516233766234e-03*Jinv20*w53+7.3965097402597406e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-7.1225649350649346e-04*Jinv22*w50+7.3965097402597406e-03*Jinv12*w7-1.6436688311688311e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-1.2327516233766234e-03*w52*Jinv10-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.0135957792207792e-03*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+7.3965097402597406e-03*w6*Jinv02+3.6982548701298703e-03*w19*Jinv12-2.6298701298701297e-03*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22-4.5657467532467534e-05*w10*Jinv10+1.5888798701298701e-03*Jinv20*w44+7.4878246753246752e-04*w58*Jinv12+3.6982548701298703e-03*Jinv02*w18+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+4.2735389610389614e-03*Jinv02*w33+3.9265422077922079e-04*w0*Jinv00+3.1777597402597403e-03*w35*Jinv20-1.0135957792207792e-03*w25*Jinv10+3.9265422077922079e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-7.1225649350649346e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+1.2327516233766234e-03*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14+3.6982548701298703e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-1.2327516233766234e-03*w5*Jinv20-2.3833198051948050e-03*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28-8.2183441558441553e-05*w55*Jinv10+1.5888798701298701e-03*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+4.2735389610389614e-03*w16*Jinv12+7.8530844155844158e-04*Jinv12*w1)*D;
  A[6][37]=(0.0)*D;
  A[6][38]=(0.0)*D;
  A[6][39]=(5.4788961038961042e-05*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10-1.6436688311688311e-03*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-1.6436688311688311e-03*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-1.7532467532467534e-03*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23+5.4788961038961042e-05*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-3.8352272727272725e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10-4.9310064935064932e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+8.2183441558441553e-04*Jinv01*w45+8.2183441558441553e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20+5.4788961038961042e-05*w45*Jinv00-1.6436688311688311e-03*Jinv21*w41+3.6525974025974026e-05*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-1.2601461038961039e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10-1.7532467532467534e-03*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-1.2601461038961039e-03*w32*Jinv22-1.6436688311688311e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2-1.0957792207792207e-03*Jinv00*w18-1.0957792207792207e-03*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38+3.2873376623376621e-04*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11-3.8352272727272725e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.8352272727272725e-04*w4*Jinv10-1.7532467532467534e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20-2.5750811688311690e-03*Jinv12*w13-1.0957792207792207e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-2.5750811688311690e-03*w32*Jinv21-1.0957792207792207e-03*w6*Jinv00+5.4788961038961042e-05*Jinv20*w56-1.6436688311688311e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+3.2873376623376621e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35-2.5750811688311690e-03*w31*Jinv11-3.8352272727272725e-04*Jinv22*w26+8.2183441558441553e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14-1.7532467532467534e-03*w5*Jinv21+8.2183441558441553e-04*Jinv02*w54-1.7532467532467534e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00-2.5750811688311690e-03*Jinv02*w12+1.0957792207792207e-03*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+2.1915584415584414e-03*w6*Jinv01-1.0957792207792207e-03*w7*Jinv10+8.2183441558441553e-04*w47*Jinv21-1.0957792207792207e-03*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-3.8352272727272725e-04*w52*Jinv11-1.2601461038961039e-03*w31*Jinv12+3.8352272727272725e-04*Jinv20*w53+2.1915584415584414e-03*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+1.2053571428571428e-03*Jinv22*w50+1.2053571428571428e-03*w22*Jinv11+2.1915584415584414e-03*Jinv12*w7+1.2053571428571428e-03*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51-1.2601461038961039e-03*Jinv02*w30+1.2053571428571428e-03*Jinv12*w49+1.0957792207792207e-03*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+2.1915584415584414e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-1.0957792207792207e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27+2.1915584415584414e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-1.7532467532467534e-03*Jinv22*w5-1.0957792207792208e-04*w49*Jinv11-1.2601461038961039e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44-3.8352272727272725e-04*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18-1.2601461038961039e-03*Jinv01*w12+2.1915584415584414e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.2053571428571428e-03*Jinv21*w23+3.8352272727272725e-04*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.2053571428571428e-03*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10-1.0957792207792207e-03*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-2.5750811688311690e-03*Jinv22*w14-1.6436688311688311e-03*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-2.5750811688311690e-03*Jinv01*w30+3.2873376623376621e-04*Jinv02*w51+8.2183441558441553e-04*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+3.2873376623376621e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[6][40]=(0.0)*D;
  A[6][41]=(0.0)*D;
  A[6][42]=(-5.4788961038961042e-05*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-5.4788961038961042e-05*Jinv20*w23+2.1367694805194807e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45-5.4788961038961042e-05*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-5.4788961038961042e-05*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-3.6525974025974026e-05*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2+1.0957792207792207e-03*Jinv00*w18+1.0957792207792207e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-1.3149350649350649e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11-3.2873376623376621e-03*w41*Jinv20-7.6704545454545450e-04*Jinv12*w13+1.0957792207792207e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-5.4788961038961042e-05*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-5.4788961038961042e-05*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-5.4788961038961042e-05*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-3.2873376623376621e-03*w39*Jinv00-7.6704545454545450e-04*Jinv02*w12+2.1367694805194807e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9+4.2735389610389614e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.0957792207792207e-03*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11+4.2735389610389614e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+4.2735389610389614e-03*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+1.0957792207792207e-03*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-3.6525974025974026e-05*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-3.2873376623376621e-03*w40*Jinv10+1.0957792207792207e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+2.1367694805194807e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30-1.3149350649350649e-03*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-1.3149350649350649e-03*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[6][43]=(0.0)*D;
  A[6][44]=(0.0)*D;
  A[6][45]=(-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-1.0957792207792208e-04*w57*Jinv02-5.4788961038961042e-05*Jinv02*w39-7.1225649350649346e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3-3.5612824675324673e-04*Jinv22*w38-1.3697240259740259e-04*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-7.1225649350649346e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-7.1225649350649346e-04*w43*Jinv11-8.7662337662337668e-04*Jinv21*w41-3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.6704545454545450e-04*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-8.7662337662337668e-04*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24-1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+1.9176136363636362e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.2049512987012989e-04*Jinv12*w13+5.4788961038961036e-04*w32*Jinv21+1.9176136363636362e-04*Jinv21*w20-3.5612824675324673e-04*w37*Jinv12-5.4788961038961042e-05*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+5.2049512987012989e-04*Jinv02*w12+1.9176136363636362e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23+5.2049512987012989e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14-1.0957792207792208e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+2.4655032467532466e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11+8.2183441558441553e-05*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-8.7662337662337668e-04*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+5.4788961038961036e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11-7.6704545454545450e-04*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[6][46]=(0.0)*D;
  A[6][47]=(0.0)*D;
  A[6][48]=(-2.7394480519480518e-04*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10-1.2601461038961039e-03*w40*Jinv12+7.1225649350649346e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-1.2601461038961039e-03*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-8.2183441558441553e-05*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.7394480519480518e-04*w25*Jinv12+3.5612824675324673e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41-3.2873376623376621e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10+9.5880681818181818e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+3.5612824675324673e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+1.9176136363636362e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-1.3697240259740260e-03*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+1.1231737012987013e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.2601461038961039e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20+7.6704545454545450e-04*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11+2.7394480519480518e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+9.5880681818181818e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+1.1231737012987013e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12+1.9176136363636362e-04*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-1.4245129870129869e-03*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11-1.4245129870129869e-03*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-1.4245129870129869e-03*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+9.5880681818181818e-04*Jinv22*w5+1.9176136363636362e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-1.3697240259740260e-03*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29+1.1231737012987013e-03*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30-1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+3.5612824675324673e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[6][49]=(0.0)*D;
  A[6][50]=(0.0)*D;
  A[6][51]=(2.7394480519480518e-04*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12+5.7528409090909093e-04*Jinv22*w53-7.1225649350649346e-04*Jinv20*w8-2.1002435064935065e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+5.2049512987012989e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+8.2183441558441553e-05*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-05*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-1.6436688311688311e-03*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+2.7394480519480518e-04*w45*Jinv00-8.2183441558441553e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12+5.2049512987012989e-04*w31*Jinv10+1.4793019480519481e-03*Jinv02*w3+8.2183441558441553e-05*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22-3.1047077922077924e-04*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+2.4655032467532466e-04*Jinv02*w45-2.1002435064935065e-04*Jinv22*w59-1.6436688311688311e-04*w51*Jinv00-1.6436688311688311e-03*Jinv12*w34-8.2183441558441553e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv10-5.4788961038961042e-05*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10-7.1225649350649346e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20+5.7528409090909093e-04*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-8.2183441558441553e-05*Jinv02*w54+1.4793019480519481e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10-5.4788961038961042e-05*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-7.1225649350649346e-04*w7*Jinv10-1.6436688311688311e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+1.3697240259740259e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-4.7666396103896100e-03*Jinv22*w8+8.2183441558441553e-05*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50-4.7666396103896100e-03*Jinv12*w7+1.3697240259740259e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.6436688311688311e-04*w52*Jinv10+9.1314935064935065e-06*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-4.7666396103896100e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.4793019480519481e-03*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22+9.1314935064935065e-06*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44-2.1002435064935065e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.6436688311688311e-03*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-2.4655032467532466e-04*Jinv02*w36-8.2183441558441553e-04*Jinv12*w43+5.2049512987012989e-04*w5*Jinv20+5.7528409090909093e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28-3.5612824675324673e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-3.1047077922077924e-04*Jinv12*w1)*D;
  A[6][52]=(0.0)*D;
  A[6][53]=(0.0)*D;
  A[6][54]=(-8.7662337662337668e-04*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53+6.3920454545454550e-05*w57*Jinv02-8.7662337662337668e-04*Jinv02*w39-3.8352272727272725e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+1.9176136363636362e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+5.2049512987012989e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-1.0957792207792208e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.2049512987012989e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-7.1225649350649346e-04*Jinv22*w20-3.8352272727272725e-04*w43*Jinv11-5.4788961038961042e-05*Jinv21*w41-7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-1.0957792207792208e-04*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.4655032467532466e-04*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+6.3920454545454550e-05*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-7.1225649350649346e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.4788961038961036e-04*Jinv12*w13+5.2049512987012989e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20+1.9176136363636362e-04*w37*Jinv12-8.7662337662337668e-04*Jinv22*w41-8.2183441558441553e-05*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+5.2049512987012989e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+5.4788961038961036e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+2.7394480519480518e-04*w47*Jinv21+5.4788961038961036e-04*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+2.4655032467532466e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+2.4655032467532466e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-3.5612824675324673e-04*Jinv21*w56-1.3697240259740259e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+6.3920454545454550e-05*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+8.2183441558441553e-05*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9-1.3697240259740259e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+1.9176136363636362e-04*Jinv02*w36-7.1225649350649346e-04*Jinv12*w43+5.2049512987012989e-04*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-1.0957792207792208e-04*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[6][55]=(0.0)*D;
  A[6][56]=(0.0)*D;
  A[6][57]=(3.4699675324675324e-04*w40*Jinv12-2.6481331168831169e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+3.4699675324675324e-04*Jinv02*w39+3.6525974025974026e-05*Jinv22*w38-4.2918019480519480e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02-5.4788961038961042e-05*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+7.4878246753246752e-04*Jinv22*w20+2.1915584415584417e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-2.1915584415584417e-04*w22*Jinv12-5.8441558441558442e-04*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2-2.7394480519480521e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+4.3831168831168834e-04*Jinv12*w34+2.1915584415584417e-04*Jinv22*w44-4.2918019480519480e-04*Jinv12*w13+3.6525974025974026e-05*w37*Jinv12+3.4699675324675324e-04*Jinv22*w41-2.6481331168831169e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-5.4788961038961042e-05*Jinv02*w54-5.8441558441558442e-04*w4*Jinv12-2.1915584415584417e-04*w21*Jinv02-4.2918019480519480e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9-2.1915584415584417e-04*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12+1.4975649350649350e-03*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50+1.4975649350649350e-03*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+1.0247564935064935e-04*Jinv02*w27+1.4975649350649350e-03*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-5.8441558441558442e-04*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18+4.3831168831168834e-04*Jinv02*w33-1.0957792207792208e-04*w48*Jinv02+1.0247564935064935e-04*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+3.6525974025974026e-05*Jinv02*w36+2.1915584415584417e-04*Jinv12*w43-2.6481331168831169e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[6][58]=(0.0)*D;
  A[6][59]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(6.7573051948051951e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10-2.9220779220779221e-04*w40*Jinv12-1.1870941558441559e-04*Jinv22*w53+2.5568181818181820e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02+6.7573051948051951e-04*w13*Jinv10-2.9220779220779221e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+1.0227272727272728e-03*w3*Jinv00+6.7573051948051951e-04*w32*Jinv20+1.0227272727272728e-03*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-3.6525974025974026e-05*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59-1.1870941558441559e-04*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-8.3198051948051951e-05*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17+2.7394480519480521e-05*Jinv01*w45+2.7394480519480521e-05*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02-8.3198051948051951e-05*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-1.1870941558441559e-04*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01-1.1870941558441559e-04*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-3.6525974025974026e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00-2.9220779220779221e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27-8.3198051948051951e-05*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+6.7573051948051951e-04*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+6.7573051948051951e-04*w31*Jinv10+1.0227272727272728e-03*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+6.7573051948051951e-04*w32*Jinv22-2.9220779220779221e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2-3.6525974025974026e-05*Jinv00*w18-3.6525974025974026e-05*w36*Jinv00-3.6525974025974026e-05*Jinv21*w38+2.7394480519480521e-05*Jinv02*w45-1.1870941558441559e-04*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11-3.6525974025974026e-05*w19*Jinv11-1.1870941558441559e-04*Jinv21*w53-1.1870941558441559e-04*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+1.0227272727272728e-03*w4*Jinv10+1.0227272727272728e-03*w4*Jinv11-2.9220779220779221e-04*w41*Jinv20+6.7573051948051951e-04*Jinv12*w13-3.6525974025974026e-05*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+6.7573051948051951e-04*w32*Jinv21-3.6525974025974026e-05*Jinv21*w20+2.5568181818181820e-04*w6*Jinv00-3.6525974025974026e-05*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56-2.9220779220779221e-04*Jinv22*w41-1.1870941558441559e-04*w26*Jinv20-8.3198051948051951e-05*w11*Jinv20-1.1870941558441559e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35+6.7573051948051951e-04*w31*Jinv11-1.1870941558441559e-04*Jinv22*w26+2.7394480519480521e-05*w46*Jinv11+6.7573051948051951e-04*Jinv20*w14+1.0227272727272728e-03*w5*Jinv21+2.7394480519480521e-05*Jinv02*w54+1.0227272727272728e-03*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33-2.9220779220779221e-04*w39*Jinv00+6.7573051948051951e-04*Jinv02*w12-3.6525974025974026e-05*Jinv01*w18-3.6525974025974026e-05*w37*Jinv11-8.3198051948051951e-05*Jinv02*w9+2.5568181818181820e-04*w6*Jinv01+2.5568181818181820e-04*w7*Jinv10+2.7394480519480521e-05*w47*Jinv21-3.6525974025974026e-05*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-1.1870941558441559e-04*w52*Jinv11+6.7573051948051951e-04*w31*Jinv12+2.7394480519480521e-05*w46*Jinv12-1.1870941558441559e-04*Jinv20*w53+2.5568181818181820e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11+2.5568181818181820e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01-1.1870941558441559e-04*Jinv01*w51+6.7573051948051951e-04*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49-3.6525974025974026e-05*w36*Jinv01-1.1870941558441559e-04*w52*Jinv10+2.5568181818181820e-04*w7*Jinv11-8.3198051948051951e-05*w9*Jinv00-3.6525974025974026e-05*w37*Jinv10+2.7394480519480521e-05*Jinv21*w56-1.1870941558441559e-04*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27+2.5568181818181820e-04*w6*Jinv02-3.6525974025974026e-05*w19*Jinv12+1.0227272727272728e-03*Jinv22*w5+2.7394480519480521e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+6.7573051948051951e-04*Jinv21*w14-8.3198051948051951e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.6525974025974026e-05*Jinv02*w18+6.7573051948051951e-04*Jinv01*w12+2.5568181818181820e-04*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23+2.7394480519480521e-05*Jinv01*w54+2.7394480519480521e-05*w55*Jinv11-1.1870941558441559e-04*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02-8.3198051948051951e-05*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17+6.7573051948051951e-04*w30*Jinv00-2.9220779220779221e-04*w40*Jinv10-3.6525974025974026e-05*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+6.7573051948051951e-04*Jinv22*w14-2.9220779220779221e-04*w40*Jinv11-3.6525974025974026e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+1.0227272727272728e-03*w5*Jinv20+6.7573051948051951e-04*Jinv01*w30-1.1870941558441559e-04*Jinv02*w51+2.7394480519480521e-05*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28-1.1870941558441559e-04*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1-8.3198051948051951e-05*w10*Jinv11)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(0.0)*D;
  A[7][4]=(-1.1505681818181819e-03*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-2.6298701298701297e-03*Jinv20*w8+1.5523538961038962e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+8.2183441558441553e-04*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3+9.5880681818181818e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.5523538961038962e-04*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+1.4610389610389610e-04*w0*Jinv02+1.4610389610389610e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+8.2183441558441553e-04*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+9.5880681818181818e-04*w48*Jinv00-8.2183441558441553e-05*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35+1.5523538961038962e-04*w58*Jinv11+1.5523538961038962e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26+1.4610389610389610e-04*w0*Jinv01-4.0178571428571428e-04*w1*Jinv10-1.4793019480519481e-03*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+8.2183441558441553e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+8.2183441558441553e-04*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22+1.4610389610389610e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+1.5523538961038962e-04*w28*Jinv11+9.5880681818181818e-04*Jinv20*w50-2.7394480519480518e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17-1.4793019480519481e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12-1.1505681818181819e-03*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3+9.5880681818181818e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39+1.4610389610389610e-04*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24+1.5523538961038962e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-8.2183441558441553e-05*Jinv21*w53+4.3831168831168834e-04*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34+8.2183441558441553e-04*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-1.4793019480519481e-03*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20-2.6298701298701297e-03*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41+4.3831168831168834e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20-8.2183441558441553e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02+9.5880681818181818e-04*w49*Jinv10+1.5523538961038962e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.0957792207792207e-03*Jinv02*w9-3.7804383116883116e-03*w6*Jinv01-2.6298701298701297e-03*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-3.7804383116883116e-03*Jinv22*w8+9.5880681818181818e-04*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-3.7804383116883116e-03*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01+4.3831168831168834e-04*w52*Jinv10-3.7804383116883116e-03*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00+1.5523538961038962e-04*Jinv02*w27-3.7804383116883116e-03*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+1.5523538961038962e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-3.7804383116883116e-03*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33-4.0178571428571428e-04*w0*Jinv00-1.4793019480519481e-03*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-4.0178571428571428e-04*Jinv20*w2-1.4793019480519481e-03*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02+1.0957792207792207e-03*Jinv01*w9-1.4793019480519481e-03*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20+1.5523538961038962e-04*Jinv22*w29+1.2327516233766234e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36+8.2183441558441553e-04*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+1.5523538961038962e-04*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+1.5523538961038962e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01+1.4610389610389610e-04*Jinv12*w1+1.0957792207792207e-03*w10*Jinv11)*D;
  A[7][5]=(0.0)*D;
  A[7][6]=(0.0)*D;
  A[7][7]=(7.3965097402597403e-04*Jinv00*w12-1.0957792207792207e-03*w40*Jinv12+2.3833198051948050e-03*Jinv22*w53-7.4878246753246752e-04*w57*Jinv02+7.3965097402597403e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+7.3965097402597403e-04*w32*Jinv20+2.6298701298701297e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-3.6982548701298703e-03*Jinv22*w38-7.4878246753246752e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12-7.8530844155844158e-04*w0*Jinv02-7.8530844155844158e-04*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-4.2735389610389614e-03*w16*Jinv11-4.2735389610389614e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-4.2735389610389614e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24-4.2735389610389614e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+2.3833198051948050e-03*Jinv21*w26-7.8530844155844158e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10-3.6982548701298703e-03*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.0957792207792207e-03*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-1.0957792207792207e-03*w11*Jinv22-7.8530844155844158e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11-4.2735389610389614e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50-4.2735389610389614e-03*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12+7.3965097402597403e-04*w31*Jinv10+2.6298701298701297e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.8530844155844158e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-3.6982548701298703e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+2.3833198051948050e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11-3.6982548701298703e-03*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.2735389610389614e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+2.6298701298701297e-03*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-3.6982548701298703e-03*Jinv21*w20-3.6982548701298703e-03*w37*Jinv12-1.0957792207792207e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-1.1870941558441558e-03*w11*Jinv20+2.3833198051948050e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+7.3965097402597403e-04*Jinv20*w14+2.6298701298701297e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54+2.6298701298701297e-03*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-3.6982548701298703e-03*Jinv01*w18-3.6982548701298703e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-7.3965097402597406e-03*w6*Jinv01+3.5612824675324673e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-7.3965097402597406e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11-7.3965097402597406e-03*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49-3.6982548701298703e-03*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-7.3965097402597406e-03*w7*Jinv11-1.1870941558441558e-03*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27-7.3965097402597406e-03*w6*Jinv02-3.6982548701298703e-03*w19*Jinv12+2.6298701298701297e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-1.1870941558441558e-03*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-3.6982548701298703e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-7.3965097402597406e-03*Jinv21*w8-4.2735389610389614e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+7.3965097402597403e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-3.6982548701298703e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.3833198051948050e-03*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+2.3833198051948050e-03*w25*Jinv11-5.4788961038961036e-04*w42*Jinv00-4.2735389610389614e-03*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01-7.8530844155844158e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[7][8]=(0.0)*D;
  A[7][9]=(0.0)*D;
  A[7][10]=(-2.6481331168831169e-04*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+2.3741883116883116e-03*Jinv20*w8-2.6481331168831169e-04*w13*Jinv10-1.0227272727272728e-03*w3*Jinv00-2.6481331168831169e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23-5.4788961038961042e-05*Jinv00*w54+3.2873376623376621e-04*w43*Jinv10+1.5726461038961039e-04*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00+2.0495129870129870e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-5.4788961038961042e-05*w45*Jinv00+1.5726461038961039e-04*Jinv00*w27-1.0957792207792208e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-2.6481331168831169e-04*w31*Jinv10-1.0957792207792208e-04*w21*Jinv00+1.1870941558441558e-03*Jinv00*w18+1.1870941558441558e-03*w36*Jinv00-6.7573051948051951e-04*w51*Jinv00-1.0227272727272728e-03*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+1.1870941558441558e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+2.3741883116883116e-03*w6*Jinv00-5.4788961038961042e-05*Jinv20*w56-6.7573051948051951e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-2.6481331168831169e-04*Jinv20*w14-1.0957792207792208e-04*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+2.3741883116883116e-03*w7*Jinv10+1.1870941558441558e-03*Jinv20*w38-6.7573051948051951e-04*Jinv20*w53-1.0957792207792208e-04*w22*Jinv10-6.7573051948051951e-04*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+1.1870941558441558e-03*w37*Jinv10-6.7573051948051951e-04*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+2.0495129870129870e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-6.7573051948051951e-04*w25*Jinv10+2.0495129870129870e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+6.5746753246753243e-04*Jinv20*w17-2.6481331168831169e-04*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+1.1870941558441558e-03*w20*Jinv20-1.0227272727272728e-03*w5*Jinv20-5.4788961038961042e-05*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20)*D;
  A[7][11]=(0.0)*D;
  A[7][12]=(0.0)*D;
  A[7][13]=(2.0271915584415584e-03*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+2.5202922077922079e-03*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8+3.1960227272727273e-04*w57*Jinv02+2.0271915584415584e-03*w13*Jinv10+2.5202922077922079e-03*Jinv02*w39+3.2873376623376621e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.0135957792207792e-03*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23-1.9176136363636362e-04*Jinv22*w38-4.6570616883116886e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59-1.3697240259740259e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-6.6659902597402602e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+3.2873376623376621e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21+6.0267857142857139e-04*w16*Jinv11+6.0267857142857139e-04*Jinv21*w17+2.7394480519480521e-05*Jinv01*w45-4.6570616883116886e-04*Jinv12*w55+3.1960227272727273e-04*w58*Jinv10+2.7394480519480520e-03*w15*Jinv02+4.8396915584415583e-04*w10*Jinv12-8.4922889610389610e-04*w48*Jinv00-1.3697240259740259e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26-6.6659902597402602e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+2.7394480519480520e-03*w16*Jinv10-3.8352272727272725e-04*Jinv22*w20+3.2873376623376621e-04*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+1.2053571428571428e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+4.8396915584415583e-04*w11*Jinv22-6.6659902597402602e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+6.0267857142857139e-04*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11-8.4922889610389610e-04*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+2.7394480519480520e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12+1.0135957792207792e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.0135957792207792e-03*w32*Jinv22+1.2053571428571428e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-3.8352272727272725e-04*Jinv00*w18-1.9176136363636362e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24+3.1960227272727273e-04*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11+2.5202922077922079e-03*w41*Jinv20+2.0271915584415584e-03*Jinv12*w13-3.8352272727272725e-04*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.3697240259740260e-03*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00-1.9176136363636362e-04*w37*Jinv12-4.6570616883116886e-04*Jinv20*w56+2.5202922077922079e-03*Jinv22*w41-1.3697240259740259e-04*w26*Jinv20+4.8396915584415583e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.3697240259740260e-03*w31*Jinv11-1.3697240259740259e-04*Jinv22*w26+2.7394480519480521e-05*w46*Jinv11+2.0271915584415584e-03*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21-4.6570616883116886e-04*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02-8.4922889610389610e-04*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+2.5202922077922079e-03*w39*Jinv00+2.0271915584415584e-03*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11+4.8396915584415583e-04*Jinv02*w9-3.2873376623376621e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+2.7394480519480521e-05*w47*Jinv21-1.9176136363636362e-04*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23+2.7394480519480518e-04*w52*Jinv11+1.0135957792207792e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-1.4793019480519481e-03*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-8.4922889610389610e-04*Jinv22*w50+9.5880681818181818e-04*w22*Jinv11-1.4793019480519481e-03*Jinv12*w7+9.5880681818181818e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51+1.0135957792207792e-03*Jinv02*w30-8.4922889610389610e-04*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10-3.2873376623376621e-04*w7*Jinv11+4.8396915584415583e-04*w9*Jinv00-1.9176136363636362e-04*w37*Jinv10-1.9176136363636362e-04*Jinv21*w56-1.3697240259740259e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27-1.4793019480519481e-03*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12+1.2327516233766234e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+4.8396915584415583e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44+3.1960227272727273e-04*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12-3.2873376623376621e-04*Jinv21*w8+3.1960227272727273e-04*w57*Jinv00+3.1960227272727273e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20+9.5880681818181818e-04*Jinv21*w23-1.9176136363636362e-04*Jinv01*w54-1.9176136363636362e-04*w55*Jinv11-1.3697240259740259e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+2.7394480519480520e-03*w15*Jinv00-8.4922889610389610e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+2.7394480519480520e-03*Jinv20*w17+1.0135957792207792e-03*w30*Jinv00+2.5202922077922079e-03*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29+2.0271915584415584e-03*Jinv22*w14+1.2053571428571428e-03*w40*Jinv11-1.9176136363636362e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.3697240259740260e-03*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-4.6570616883116886e-04*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-4.6570616883116886e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+2.7394480519480520e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[7][14]=(0.0)*D;
  A[7][15]=(0.0)*D;
  A[7][16]=(-1.8628246753246754e-03*Jinv00*w12-5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-9.8620129870129864e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-1.8628246753246754e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.3141233766233771e-04*w32*Jinv20-2.7942370129870131e-03*Jinv01*w3+6.0267857142857139e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02+1.6436688311688311e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-6.9399350649350649e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+1.2053571428571428e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01-4.9310064935064932e-04*Jinv21*w26+1.6436688311688311e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20+3.2873376623376621e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.6615259740259744e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+1.6436688311688311e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+1.2053571428571428e-03*Jinv20*w50-5.6615259740259744e-04*Jinv20*w29+3.2873376623376621e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-2.9586038961038961e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12-9.3141233766233771e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3+6.0267857142857139e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-2.0819805194805196e-03*Jinv00*w18-1.0409902597402598e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45-4.9310064935064932e-04*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-1.3149350649350649e-03*Jinv21*w53+1.2601461038961039e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-2.7942370129870131e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-2.0819805194805196e-03*w19*Jinv10-2.9586038961038961e-03*w34*Jinv10-1.7532467532467534e-03*w32*Jinv21-9.8620129870129864e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20-4.3831168831168834e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-1.7532467532467534e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+3.2873376623376621e-04*w46*Jinv11-1.8628246753246754e-03*Jinv20*w14-2.7942370129870131e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02+1.2053571428571428e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12+2.1367694805194807e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+8.5470779220779228e-03*w6*Jinv01-9.8620129870129864e-04*w7*Jinv10+3.2873376623376621e-04*w47*Jinv21-1.0409902597402598e-03*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23-1.3149350649350649e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+1.2601461038961039e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8+6.0267857142857139e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+6.0267857142857139e-04*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+6.0267857142857139e-04*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+1.2601461038961039e-03*w52*Jinv10+8.5470779220779228e-03*w7*Jinv11-4.3831168831168834e-04*w9*Jinv00-1.0409902597402598e-03*w37*Jinv10+1.3697240259740260e-03*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14-4.3831168831168834e-04*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12+8.5470779220779228e-03*Jinv21*w8-6.9399350649350649e-04*w57*Jinv00-6.9399350649350649e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-2.9586038961038961e-03*w35*Jinv20+6.0267857142857139e-04*Jinv21*w23+1.3697240259740260e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17-9.3141233766233771e-04*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10-2.0819805194805196e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.7532467532467534e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28-4.9310064935064932e-04*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+3.2873376623376621e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[7][17]=(0.0)*D;
  A[7][18]=(0.0)*D;
  A[7][19]=(-1.6436688311688311e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+2.4655032467532467e-03*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10+2.1367694805194807e-03*Jinv21*w44-1.2327516233766234e-03*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20-2.6298701298701297e-03*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23-3.5612824675324673e-04*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.8530844155844158e-04*Jinv21*w2+1.5888798701298701e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+4.2735389610389614e-03*w16*Jinv11+4.2735389610389614e-03*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+3.7439123376623376e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01-2.3833198051948050e-03*Jinv21*w26+7.8530844155844158e-04*w0*Jinv01+3.9265422077922079e-04*w1*Jinv10+3.1777597402597403e-03*w16*Jinv10+2.1367694805194807e-03*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+3.9265422077922079e-04*Jinv00*w27+7.8530844155844158e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11+4.2735389610389614e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-8.2183441558441553e-05*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+2.4655032467532467e-03*Jinv00*w18+1.2327516233766234e-03*w36*Jinv00+3.6982548701298703e-03*Jinv21*w38-2.3833198051948050e-03*Jinv01*w24+4.2735389610389614e-03*w34*Jinv11+3.6982548701298703e-03*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.0135957792207792e-03*w51*Jinv00-1.2327516233766234e-03*w4*Jinv10-2.6298701298701297e-03*w4*Jinv11+1.0957792207792207e-03*w41*Jinv20+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21+3.6982548701298703e-03*Jinv21*w20+2.4655032467532467e-03*w6*Jinv00-3.5612824675324673e-04*Jinv20*w56-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20+4.2735389610389614e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-2.6298701298701297e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33+1.0957792207792207e-03*w39*Jinv00+3.6982548701298703e-03*Jinv01*w18+3.6982548701298703e-03*w37*Jinv11+7.3965097402597406e-03*w6*Jinv01+2.4655032467532467e-03*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+1.2327516233766234e-03*Jinv20*w38-2.3833198051948050e-03*w52*Jinv11-1.0135957792207792e-03*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-7.1225649350649346e-04*w22*Jinv11-7.1225649350649346e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+3.6982548701298703e-03*w36*Jinv01-1.0135957792207792e-03*w52*Jinv10+7.3965097402597406e-03*w7*Jinv11-4.5657467532467534e-05*w9*Jinv00+1.2327516233766234e-03*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56-1.2327516233766234e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14-4.5657467532467534e-05*w10*Jinv10+1.5888798701298701e-03*Jinv20*w44-1.6436688311688311e-04*Jinv01*w12+7.3965097402597406e-03*Jinv21*w8+3.7439123376623376e-04*w57*Jinv00+3.7439123376623376e-04*Jinv20*w59+3.9265422077922079e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-7.1225649350649346e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-1.2327516233766234e-03*w25*Jinv10+3.9265422077922079e-04*Jinv20*w2+3.1777597402597403e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+3.1777597402597403e-03*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-1.2327516233766234e-03*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30-2.3833198051948050e-03*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10+1.5888798701298701e-03*w42*Jinv00-8.2183441558441553e-05*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[7][20]=(0.0)*D;
  A[7][21]=(0.0)*D;
  A[7][22]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39-3.2873376623376621e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20+9.5880681818181818e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+3.5612824675324673e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10-3.2873376623376621e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.7394480519480518e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+3.5612824675324673e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20-3.2873376623376621e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.2601461038961039e-03*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+3.5612824675324673e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-1.2601461038961039e-03*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10+9.5880681818181818e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+1.1231737012987013e-03*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+1.1231737012987013e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26-5.2049512987012989e-04*Jinv20*w14+9.5880681818181818e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9-1.4245129870129869e-03*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+3.5612824675324673e-04*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-1.4245129870129869e-03*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+1.9176136363636362e-04*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12-1.4245129870129869e-03*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+1.9176136363636362e-04*Jinv01*w54+1.9176136363636362e-04*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14-1.2601461038961039e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20+1.1231737012987013e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28-2.7394480519480518e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[7][23]=(0.0)*D;
  A[7][24]=(0.0)*D;
  A[7][25]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*Jinv20*w8+5.2049512987012989e-04*w13*Jinv10-8.2183441558441553e-04*Jinv21*w44+5.2049512987012989e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+1.4793019480519481e-03*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23+2.7394480519480518e-04*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-3.1047077922077924e-04*Jinv21*w2-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-1.6436688311688311e-03*w16*Jinv11-1.6436688311688311e-03*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45-1.0957792207792208e-04*w58*Jinv10+8.2183441558441553e-05*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+5.7528409090909093e-04*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-8.2183441558441553e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-3.1047077922077924e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-1.6436688311688311e-03*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11+8.2183441558441553e-05*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39-1.6436688311688311e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+5.7528409090909093e-04*Jinv01*w24-2.6298701298701297e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00+5.2049512987012989e-04*w4*Jinv10+1.4793019480519481e-03*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-1.6436688311688311e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-7.1225649350649346e-04*w6*Jinv00+2.7394480519480518e-04*Jinv20*w56-1.6436688311688311e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20-2.6298701298701297e-03*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-8.2183441558441553e-05*w46*Jinv11+5.2049512987012989e-04*Jinv20*w14+1.4793019480519481e-03*w5*Jinv21+8.2183441558441553e-05*w49*Jinv10-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.4655032467532466e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-4.7666396103896100e-03*w6*Jinv01-7.1225649350649346e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10-4.7666396103896100e-03*w7*Jinv11+9.1314935064935065e-06*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+5.4788961038961036e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14+9.1314935064935065e-06*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+1.3697240259740259e-04*Jinv01*w12-4.7666396103896100e-03*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+5.2049512987012989e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+5.7528409090909093e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+5.4788961038961036e-04*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[7][26]=(0.0)*D;
  A[7][27]=(0.0)*D;
  A[7][28]=(2.1915584415584417e-04*Jinv21*w44-5.8441558441558442e-04*Jinv01*w3+1.0247564935064935e-04*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+2.1915584415584417e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-2.6481331168831169e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+2.1915584415584417e-04*w43*Jinv11+3.4699675324675324e-04*Jinv21*w41-1.4204545454545455e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+3.4699675324675324e-04*Jinv01*w39+7.4878246753246752e-04*Jinv21*w38-2.6481331168831169e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-5.8441558441558442e-04*w4*Jinv11-4.2918019480519480e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-5.8441558441558442e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+3.6525974025974026e-05*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+1.4975649350649350e-03*w6*Jinv01-5.4788961038961042e-05*w47*Jinv21-4.2918019480519480e-04*w52*Jinv11-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01+1.4975649350649350e-03*w7*Jinv11-2.7394480519480521e-05*Jinv21*w56-2.1915584415584417e-04*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+1.1870941558441559e-04*Jinv01*w12+1.4975649350649350e-03*Jinv21*w8-1.0957792207792208e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11+6.4935064935064935e-05*Jinv01*w9+3.4699675324675324e-04*w40*Jinv11-4.2918019480519480e-04*Jinv01*w30-2.6481331168831169e-04*w25*Jinv11-2.1915584415584417e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-2.1915584415584417e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[7][29]=(0.0)*D;
  A[7][30]=(0.0)*D;
  A[7][31]=(1.0135957792207792e-03*Jinv00*w12+3.1960227272727273e-04*w28*Jinv10+1.2053571428571428e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02+1.0135957792207792e-03*w13*Jinv10+1.2053571428571428e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+2.0271915584415584e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3-8.4922889610389610e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59+2.7394480519480518e-04*w25*Jinv12-6.6659902597402602e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+4.8396915584415583e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-4.6570616883116886e-04*Jinv01*w45+2.7394480519480521e-05*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24+6.0267857142857139e-04*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-4.6570616883116886e-04*w45*Jinv00+2.5202922077922079e-03*Jinv21*w41+3.2873376623376621e-04*w42*Jinv02+3.1960227272727273e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.0135957792207792e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+3.1960227272727273e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+3.1960227272727273e-04*Jinv20*w29-4.6570616883116886e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17+2.7394480519480520e-03*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12+2.0271915584415584e-03*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3-8.4922889610389610e-04*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+2.5202922077922079e-03*Jinv01*w39-6.6659902597402602e-04*Jinv22*w2-1.9176136363636362e-04*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-3.8352272727272725e-04*Jinv21*w38-1.9176136363636362e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+2.7394480519480520e-03*w34*Jinv11-1.9176136363636362e-04*w19*Jinv11-1.3697240259740259e-04*Jinv21*w53-1.3697240259740259e-04*w51*Jinv00+6.0267857142857139e-04*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11+2.5202922077922079e-03*w41*Jinv20-1.3697240259740260e-03*Jinv12*w13-1.9176136363636362e-04*w19*Jinv10+2.7394480519480520e-03*w34*Jinv10+2.0271915584415584e-03*w32*Jinv21-1.9176136363636362e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+1.2053571428571428e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+4.8396915584415583e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+2.7394480519480520e-03*Jinv21*w35+2.0271915584415584e-03*w31*Jinv11+2.7394480519480518e-04*Jinv22*w26-4.6570616883116886e-04*w46*Jinv11+1.0135957792207792e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+2.7394480519480521e-05*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+3.1960227272727273e-04*Jinv21*w29+2.7394480519480520e-03*Jinv01*w33+2.5202922077922079e-03*w39*Jinv00-1.3697240259740260e-03*Jinv02*w12-1.9176136363636362e-04*Jinv01*w18-3.8352272727272725e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-1.4793019480519481e-03*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10-4.6570616883116886e-04*w47*Jinv21-3.8352272727272725e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23-1.3697240259740259e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-1.9176136363636362e-04*w46*Jinv12-1.3697240259740259e-04*Jinv20*w53-3.2873376623376621e-04*Jinv22*w8-8.4922889610389610e-04*w22*Jinv10+9.5880681818181818e-04*Jinv22*w50-8.4922889610389610e-04*w22*Jinv11-3.2873376623376621e-04*Jinv12*w7-8.4922889610389610e-04*w21*Jinv01-1.3697240259740259e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+9.5880681818181818e-04*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-1.3697240259740259e-04*w52*Jinv10-1.4793019480519481e-03*w7*Jinv11+4.8396915584415583e-04*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-3.2873376623376621e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5-1.9176136363636362e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.0135957792207792e-03*Jinv21*w14+4.8396915584415583e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18+1.0135957792207792e-03*Jinv01*w12-1.4793019480519481e-03*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+6.0267857142857139e-04*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+2.7394480519480520e-03*w35*Jinv20-8.4922889610389610e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00+9.5880681818181818e-04*w48*Jinv02+4.8396915584415583e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+2.0271915584415584e-03*w30*Jinv00+2.5202922077922079e-03*w40*Jinv10-1.9176136363636362e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29-1.3697240259740260e-03*Jinv22*w14+2.5202922077922079e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20+2.0271915584415584e-03*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+2.7394480519480521e-05*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-4.6570616883116886e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+3.1960227272727273e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01-6.6659902597402602e-04*Jinv12*w1+4.8396915584415583e-04*w10*Jinv11)*D;
  A[7][32]=(0.0)*D;
  A[7][33]=(0.0)*D;
  A[7][34]=(-9.3141233766233771e-04*Jinv00*w12-6.9399350649350649e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12-4.9310064935064932e-04*Jinv22*w53-9.8620129870129864e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-9.3141233766233771e-04*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.8628246753246754e-03*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3+1.2053571428571428e-03*Jinv20*w23+3.2873376623376621e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59-1.3149350649350649e-03*w25*Jinv12+1.6436688311688311e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+3.2873376623376621e-04*Jinv12*w55-5.6615259740259744e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+6.0267857142857139e-04*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-2.9586038961038961e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-6.9399350649350649e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11+6.0267857142857139e-04*Jinv20*w50-6.9399350649350649e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-5.9172077922077923e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-1.8628246753246754e-03*w31*Jinv10-2.7942370129870131e-03*Jinv02*w3+1.2053571428571428e-03*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+1.6436688311688311e-04*Jinv22*w2-1.0409902597402598e-03*Jinv00*w18-2.0819805194805196e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+1.3149350649350649e-03*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+2.1367694805194807e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-1.7532467532467534e-03*Jinv12*w13-1.0409902597402598e-03*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-9.8620129870129864e-04*w6*Jinv00+3.2873376623376621e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+1.2601461038961039e-03*w26*Jinv20-4.3831168831168834e-04*w11*Jinv20-4.9310064935064932e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11-1.3149350649350649e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-9.3141233766233771e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+3.2873376623376621e-04*Jinv02*w54-2.7942370129870131e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+6.0267857142857139e-04*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-1.7532467532467534e-03*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01-9.8620129870129864e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-2.0819805194805196e-03*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12+1.3697240259740260e-03*Jinv20*w53+8.5470779220779228e-03*Jinv22*w8+1.2053571428571428e-03*w22*Jinv10+6.0267857142857139e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+8.5470779220779228e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+6.0267857142857139e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-4.3831168831168834e-04*w9*Jinv00-2.0819805194805196e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56+1.2601461038961039e-03*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27+8.5470779220779228e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-2.7942370129870131e-03*Jinv22*w5+7.6704545454545450e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-4.3831168831168834e-04*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8-5.6615259740259744e-04*w57*Jinv00-5.6615259740259744e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+1.2601461038961039e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-2.9586038961038961e-03*w15*Jinv00+6.0267857142857139e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-2.9586038961038961e-03*Jinv20*w17-1.8628246753246754e-03*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10-1.0409902597402598e-03*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-1.7532467532467534e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+7.6704545454545450e-04*Jinv01*w30-4.9310064935064932e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11+3.2873376623376621e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01+1.6436688311688311e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[7][35]=(0.0)*D;
  A[7][36]=(0.0)*D;
  A[7][37]=(-8.2183441558441553e-05*Jinv00*w12+3.7439123376623376e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.3833198051948050e-03*Jinv22*w53+2.4655032467532467e-03*Jinv20*w8+7.4878246753246752e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.2327516233766234e-03*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+3.6982548701298703e-03*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-2.3833198051948050e-03*w25*Jinv12+7.8530844155844158e-04*w0*Jinv02+1.5888798701298701e-03*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10+4.2735389610389614e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00-2.3833198051948050e-03*Jinv02*w24+4.2735389610389614e-03*Jinv22*w35+3.9265422077922079e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+3.6982548701298703e-03*Jinv22*w20-3.5612824675324673e-04*w45*Jinv00+2.1367694805194807e-03*w42*Jinv02+3.7439123376623376e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50+3.7439123376623376e-04*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+4.2735389610389614e-03*Jinv22*w17+3.1777597402597403e-03*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-2.6298701298701297e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-1.6436688311688311e-04*w32*Jinv22+7.8530844155844158e-04*Jinv22*w2+1.2327516233766234e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00-3.5612824675324673e-04*Jinv02*w45+7.4878246753246752e-04*Jinv22*w59-1.2327516233766234e-03*w51*Jinv00+4.2735389610389614e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-1.2327516233766234e-03*w4*Jinv10+1.0957792207792207e-03*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13+1.2327516233766234e-03*w19*Jinv10+3.1777597402597403e-03*w34*Jinv10+2.4655032467532467e-03*w6*Jinv00+3.6982548701298703e-03*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-1.0135957792207792e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20-2.3833198051948050e-03*w52*Jinv12-2.3833198051948050e-03*Jinv22*w26-8.2183441558441553e-05*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-2.6298701298701297e-03*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10+1.0957792207792207e-03*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+2.4655032467532467e-03*w7*Jinv10+2.4655032467532467e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-1.6436688311688311e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12-1.2327516233766234e-03*Jinv20*w53+7.3965097402597406e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-7.1225649350649346e-04*Jinv22*w50+7.3965097402597406e-03*Jinv12*w7-1.6436688311688311e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-1.2327516233766234e-03*w52*Jinv10-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.0135957792207792e-03*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+7.3965097402597406e-03*w6*Jinv02+3.6982548701298703e-03*w19*Jinv12-2.6298701298701297e-03*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22-4.5657467532467534e-05*w10*Jinv10+1.5888798701298701e-03*Jinv20*w44+7.4878246753246752e-04*w58*Jinv12+3.6982548701298703e-03*Jinv02*w18+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+4.2735389610389614e-03*Jinv02*w33+3.9265422077922079e-04*w0*Jinv00+3.1777597402597403e-03*w35*Jinv20-1.0135957792207792e-03*w25*Jinv10+3.9265422077922079e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-7.1225649350649346e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+1.2327516233766234e-03*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14+3.6982548701298703e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-1.2327516233766234e-03*w5*Jinv20-2.3833198051948050e-03*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28-8.2183441558441553e-05*w55*Jinv10+1.5888798701298701e-03*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+4.2735389610389614e-03*w16*Jinv12+7.8530844155844158e-04*Jinv12*w1)*D;
  A[7][38]=(0.0)*D;
  A[7][39]=(0.0)*D;
  A[7][40]=(5.4788961038961042e-05*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10-1.6436688311688311e-03*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-1.6436688311688311e-03*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-1.7532467532467534e-03*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23+5.4788961038961042e-05*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-3.8352272727272725e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10-4.9310064935064932e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+8.2183441558441553e-04*Jinv01*w45+8.2183441558441553e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20+5.4788961038961042e-05*w45*Jinv00-1.6436688311688311e-03*Jinv21*w41+3.6525974025974026e-05*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-1.2601461038961039e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10-1.7532467532467534e-03*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-1.2601461038961039e-03*w32*Jinv22-1.6436688311688311e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2-1.0957792207792207e-03*Jinv00*w18-1.0957792207792207e-03*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38+3.2873376623376621e-04*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11-3.8352272727272725e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.8352272727272725e-04*w4*Jinv10-1.7532467532467534e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20-2.5750811688311690e-03*Jinv12*w13-1.0957792207792207e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-2.5750811688311690e-03*w32*Jinv21-1.0957792207792207e-03*w6*Jinv00+5.4788961038961042e-05*Jinv20*w56-1.6436688311688311e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+3.2873376623376621e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35-2.5750811688311690e-03*w31*Jinv11-3.8352272727272725e-04*Jinv22*w26+8.2183441558441553e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14-1.7532467532467534e-03*w5*Jinv21+8.2183441558441553e-04*Jinv02*w54-1.7532467532467534e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00-2.5750811688311690e-03*Jinv02*w12+1.0957792207792207e-03*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+2.1915584415584414e-03*w6*Jinv01-1.0957792207792207e-03*w7*Jinv10+8.2183441558441553e-04*w47*Jinv21-1.0957792207792207e-03*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-3.8352272727272725e-04*w52*Jinv11-1.2601461038961039e-03*w31*Jinv12+3.8352272727272725e-04*Jinv20*w53+2.1915584415584414e-03*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+1.2053571428571428e-03*Jinv22*w50+1.2053571428571428e-03*w22*Jinv11+2.1915584415584414e-03*Jinv12*w7+1.2053571428571428e-03*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51-1.2601461038961039e-03*Jinv02*w30+1.2053571428571428e-03*Jinv12*w49+1.0957792207792207e-03*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+2.1915584415584414e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-1.0957792207792207e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27+2.1915584415584414e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-1.7532467532467534e-03*Jinv22*w5-1.0957792207792208e-04*w49*Jinv11-1.2601461038961039e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44-3.8352272727272725e-04*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18-1.2601461038961039e-03*Jinv01*w12+2.1915584415584414e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.2053571428571428e-03*Jinv21*w23+3.8352272727272725e-04*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.2053571428571428e-03*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10-1.0957792207792207e-03*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-2.5750811688311690e-03*Jinv22*w14-1.6436688311688311e-03*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-2.5750811688311690e-03*Jinv01*w30+3.2873376623376621e-04*Jinv02*w51+8.2183441558441553e-04*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+3.2873376623376621e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[7][41]=(0.0)*D;
  A[7][42]=(0.0)*D;
  A[7][43]=(-5.4788961038961042e-05*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-5.4788961038961042e-05*Jinv20*w23+2.1367694805194807e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45-5.4788961038961042e-05*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-5.4788961038961042e-05*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-3.6525974025974026e-05*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2+1.0957792207792207e-03*Jinv00*w18+1.0957792207792207e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-1.3149350649350649e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11-3.2873376623376621e-03*w41*Jinv20-7.6704545454545450e-04*Jinv12*w13+1.0957792207792207e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-5.4788961038961042e-05*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-5.4788961038961042e-05*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-5.4788961038961042e-05*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-3.2873376623376621e-03*w39*Jinv00-7.6704545454545450e-04*Jinv02*w12+2.1367694805194807e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9+4.2735389610389614e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.0957792207792207e-03*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11+4.2735389610389614e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+4.2735389610389614e-03*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+1.0957792207792207e-03*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-3.6525974025974026e-05*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-3.2873376623376621e-03*w40*Jinv10+1.0957792207792207e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+2.1367694805194807e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30-1.3149350649350649e-03*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-1.3149350649350649e-03*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[7][44]=(0.0)*D;
  A[7][45]=(0.0)*D;
  A[7][46]=(-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-1.0957792207792208e-04*w57*Jinv02-5.4788961038961042e-05*Jinv02*w39-7.1225649350649346e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3-3.5612824675324673e-04*Jinv22*w38-1.3697240259740259e-04*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-7.1225649350649346e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-7.1225649350649346e-04*w43*Jinv11-8.7662337662337668e-04*Jinv21*w41-3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.6704545454545450e-04*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-8.7662337662337668e-04*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24-1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+1.9176136363636362e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.2049512987012989e-04*Jinv12*w13+5.4788961038961036e-04*w32*Jinv21+1.9176136363636362e-04*Jinv21*w20-3.5612824675324673e-04*w37*Jinv12-5.4788961038961042e-05*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+5.2049512987012989e-04*Jinv02*w12+1.9176136363636362e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23+5.2049512987012989e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14-1.0957792207792208e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+2.4655032467532466e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11+8.2183441558441553e-05*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-8.7662337662337668e-04*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+5.4788961038961036e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11-7.6704545454545450e-04*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[7][47]=(0.0)*D;
  A[7][48]=(0.0)*D;
  A[7][49]=(-2.7394480519480518e-04*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10-1.2601461038961039e-03*w40*Jinv12+7.1225649350649346e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-1.2601461038961039e-03*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-8.2183441558441553e-05*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.7394480519480518e-04*w25*Jinv12+3.5612824675324673e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41-3.2873376623376621e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10+9.5880681818181818e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+3.5612824675324673e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+1.9176136363636362e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-1.3697240259740260e-03*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+1.1231737012987013e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.2601461038961039e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20+7.6704545454545450e-04*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11+2.7394480519480518e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+9.5880681818181818e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+1.1231737012987013e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12+1.9176136363636362e-04*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-1.4245129870129869e-03*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11-1.4245129870129869e-03*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-1.4245129870129869e-03*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+9.5880681818181818e-04*Jinv22*w5+1.9176136363636362e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-1.3697240259740260e-03*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29+1.1231737012987013e-03*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30-1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+3.5612824675324673e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[7][50]=(0.0)*D;
  A[7][51]=(0.0)*D;
  A[7][52]=(2.7394480519480518e-04*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12+5.7528409090909093e-04*Jinv22*w53-7.1225649350649346e-04*Jinv20*w8-2.1002435064935065e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+5.2049512987012989e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+8.2183441558441553e-05*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-05*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-1.6436688311688311e-03*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+2.7394480519480518e-04*w45*Jinv00-8.2183441558441553e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12+5.2049512987012989e-04*w31*Jinv10+1.4793019480519481e-03*Jinv02*w3+8.2183441558441553e-05*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22-3.1047077922077924e-04*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+2.4655032467532466e-04*Jinv02*w45-2.1002435064935065e-04*Jinv22*w59-1.6436688311688311e-04*w51*Jinv00-1.6436688311688311e-03*Jinv12*w34-8.2183441558441553e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv10-5.4788961038961042e-05*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10-7.1225649350649346e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20+5.7528409090909093e-04*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-8.2183441558441553e-05*Jinv02*w54+1.4793019480519481e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10-5.4788961038961042e-05*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-7.1225649350649346e-04*w7*Jinv10-1.6436688311688311e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+1.3697240259740259e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-4.7666396103896100e-03*Jinv22*w8+8.2183441558441553e-05*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50-4.7666396103896100e-03*Jinv12*w7+1.3697240259740259e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.6436688311688311e-04*w52*Jinv10+9.1314935064935065e-06*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-4.7666396103896100e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.4793019480519481e-03*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22+9.1314935064935065e-06*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44-2.1002435064935065e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.6436688311688311e-03*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-2.4655032467532466e-04*Jinv02*w36-8.2183441558441553e-04*Jinv12*w43+5.2049512987012989e-04*w5*Jinv20+5.7528409090909093e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28-3.5612824675324673e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-3.1047077922077924e-04*Jinv12*w1)*D;
  A[7][53]=(0.0)*D;
  A[7][54]=(0.0)*D;
  A[7][55]=(-8.7662337662337668e-04*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53+6.3920454545454550e-05*w57*Jinv02-8.7662337662337668e-04*Jinv02*w39-3.8352272727272725e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+1.9176136363636362e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+5.2049512987012989e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-1.0957792207792208e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.2049512987012989e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-7.1225649350649346e-04*Jinv22*w20-3.8352272727272725e-04*w43*Jinv11-5.4788961038961042e-05*Jinv21*w41-7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-1.0957792207792208e-04*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.4655032467532466e-04*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+6.3920454545454550e-05*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-7.1225649350649346e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.4788961038961036e-04*Jinv12*w13+5.2049512987012989e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20+1.9176136363636362e-04*w37*Jinv12-8.7662337662337668e-04*Jinv22*w41-8.2183441558441553e-05*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+5.2049512987012989e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+5.4788961038961036e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+2.7394480519480518e-04*w47*Jinv21+5.4788961038961036e-04*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+2.4655032467532466e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+2.4655032467532466e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-3.5612824675324673e-04*Jinv21*w56-1.3697240259740259e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+6.3920454545454550e-05*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+8.2183441558441553e-05*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9-1.3697240259740259e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+1.9176136363636362e-04*Jinv02*w36-7.1225649350649346e-04*Jinv12*w43+5.2049512987012989e-04*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-1.0957792207792208e-04*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[7][56]=(0.0)*D;
  A[7][57]=(0.0)*D;
  A[7][58]=(3.4699675324675324e-04*w40*Jinv12-2.6481331168831169e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+3.4699675324675324e-04*Jinv02*w39+3.6525974025974026e-05*Jinv22*w38-4.2918019480519480e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02-5.4788961038961042e-05*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+7.4878246753246752e-04*Jinv22*w20+2.1915584415584417e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-2.1915584415584417e-04*w22*Jinv12-5.8441558441558442e-04*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2-2.7394480519480521e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+4.3831168831168834e-04*Jinv12*w34+2.1915584415584417e-04*Jinv22*w44-4.2918019480519480e-04*Jinv12*w13+3.6525974025974026e-05*w37*Jinv12+3.4699675324675324e-04*Jinv22*w41-2.6481331168831169e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-5.4788961038961042e-05*Jinv02*w54-5.8441558441558442e-04*w4*Jinv12-2.1915584415584417e-04*w21*Jinv02-4.2918019480519480e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9-2.1915584415584417e-04*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12+1.4975649350649350e-03*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50+1.4975649350649350e-03*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+1.0247564935064935e-04*Jinv02*w27+1.4975649350649350e-03*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-5.8441558441558442e-04*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18+4.3831168831168834e-04*Jinv02*w33-1.0957792207792208e-04*w48*Jinv02+1.0247564935064935e-04*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+3.6525974025974026e-05*Jinv02*w36+2.1915584415584417e-04*Jinv12*w43-2.6481331168831169e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[7][59]=(0.0)*D;
  A[8][0]=(0.0)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(6.7573051948051951e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10-2.9220779220779221e-04*w40*Jinv12-1.1870941558441559e-04*Jinv22*w53+2.5568181818181820e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02+6.7573051948051951e-04*w13*Jinv10-2.9220779220779221e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+1.0227272727272728e-03*w3*Jinv00+6.7573051948051951e-04*w32*Jinv20+1.0227272727272728e-03*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-3.6525974025974026e-05*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59-1.1870941558441559e-04*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-8.3198051948051951e-05*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17+2.7394480519480521e-05*Jinv01*w45+2.7394480519480521e-05*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02-8.3198051948051951e-05*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-1.1870941558441559e-04*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01-1.1870941558441559e-04*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-3.6525974025974026e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00-2.9220779220779221e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27-8.3198051948051951e-05*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+6.7573051948051951e-04*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+6.7573051948051951e-04*w31*Jinv10+1.0227272727272728e-03*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+6.7573051948051951e-04*w32*Jinv22-2.9220779220779221e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2-3.6525974025974026e-05*Jinv00*w18-3.6525974025974026e-05*w36*Jinv00-3.6525974025974026e-05*Jinv21*w38+2.7394480519480521e-05*Jinv02*w45-1.1870941558441559e-04*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11-3.6525974025974026e-05*w19*Jinv11-1.1870941558441559e-04*Jinv21*w53-1.1870941558441559e-04*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+1.0227272727272728e-03*w4*Jinv10+1.0227272727272728e-03*w4*Jinv11-2.9220779220779221e-04*w41*Jinv20+6.7573051948051951e-04*Jinv12*w13-3.6525974025974026e-05*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+6.7573051948051951e-04*w32*Jinv21-3.6525974025974026e-05*Jinv21*w20+2.5568181818181820e-04*w6*Jinv00-3.6525974025974026e-05*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56-2.9220779220779221e-04*Jinv22*w41-1.1870941558441559e-04*w26*Jinv20-8.3198051948051951e-05*w11*Jinv20-1.1870941558441559e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35+6.7573051948051951e-04*w31*Jinv11-1.1870941558441559e-04*Jinv22*w26+2.7394480519480521e-05*w46*Jinv11+6.7573051948051951e-04*Jinv20*w14+1.0227272727272728e-03*w5*Jinv21+2.7394480519480521e-05*Jinv02*w54+1.0227272727272728e-03*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33-2.9220779220779221e-04*w39*Jinv00+6.7573051948051951e-04*Jinv02*w12-3.6525974025974026e-05*Jinv01*w18-3.6525974025974026e-05*w37*Jinv11-8.3198051948051951e-05*Jinv02*w9+2.5568181818181820e-04*w6*Jinv01+2.5568181818181820e-04*w7*Jinv10+2.7394480519480521e-05*w47*Jinv21-3.6525974025974026e-05*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-1.1870941558441559e-04*w52*Jinv11+6.7573051948051951e-04*w31*Jinv12+2.7394480519480521e-05*w46*Jinv12-1.1870941558441559e-04*Jinv20*w53+2.5568181818181820e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11+2.5568181818181820e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01-1.1870941558441559e-04*Jinv01*w51+6.7573051948051951e-04*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49-3.6525974025974026e-05*w36*Jinv01-1.1870941558441559e-04*w52*Jinv10+2.5568181818181820e-04*w7*Jinv11-8.3198051948051951e-05*w9*Jinv00-3.6525974025974026e-05*w37*Jinv10+2.7394480519480521e-05*Jinv21*w56-1.1870941558441559e-04*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27+2.5568181818181820e-04*w6*Jinv02-3.6525974025974026e-05*w19*Jinv12+1.0227272727272728e-03*Jinv22*w5+2.7394480519480521e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+6.7573051948051951e-04*Jinv21*w14-8.3198051948051951e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.6525974025974026e-05*Jinv02*w18+6.7573051948051951e-04*Jinv01*w12+2.5568181818181820e-04*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23+2.7394480519480521e-05*Jinv01*w54+2.7394480519480521e-05*w55*Jinv11-1.1870941558441559e-04*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02-8.3198051948051951e-05*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17+6.7573051948051951e-04*w30*Jinv00-2.9220779220779221e-04*w40*Jinv10-3.6525974025974026e-05*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+6.7573051948051951e-04*Jinv22*w14-2.9220779220779221e-04*w40*Jinv11-3.6525974025974026e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+1.0227272727272728e-03*w5*Jinv20+6.7573051948051951e-04*Jinv01*w30-1.1870941558441559e-04*Jinv02*w51+2.7394480519480521e-05*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28-1.1870941558441559e-04*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1-8.3198051948051951e-05*w10*Jinv11)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(0.0)*D;
  A[8][5]=(-1.1505681818181819e-03*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-2.6298701298701297e-03*Jinv20*w8+1.5523538961038962e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+8.2183441558441553e-04*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3+9.5880681818181818e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.5523538961038962e-04*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+1.4610389610389610e-04*w0*Jinv02+1.4610389610389610e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+8.2183441558441553e-04*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+9.5880681818181818e-04*w48*Jinv00-8.2183441558441553e-05*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35+1.5523538961038962e-04*w58*Jinv11+1.5523538961038962e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26+1.4610389610389610e-04*w0*Jinv01-4.0178571428571428e-04*w1*Jinv10-1.4793019480519481e-03*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+8.2183441558441553e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+8.2183441558441553e-04*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22+1.4610389610389610e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+1.5523538961038962e-04*w28*Jinv11+9.5880681818181818e-04*Jinv20*w50-2.7394480519480518e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17-1.4793019480519481e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12-1.1505681818181819e-03*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3+9.5880681818181818e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39+1.4610389610389610e-04*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24+1.5523538961038962e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-8.2183441558441553e-05*Jinv21*w53+4.3831168831168834e-04*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34+8.2183441558441553e-04*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-1.4793019480519481e-03*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20-2.6298701298701297e-03*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41+4.3831168831168834e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20-8.2183441558441553e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02+9.5880681818181818e-04*w49*Jinv10+1.5523538961038962e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.0957792207792207e-03*Jinv02*w9-3.7804383116883116e-03*w6*Jinv01-2.6298701298701297e-03*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-3.7804383116883116e-03*Jinv22*w8+9.5880681818181818e-04*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-3.7804383116883116e-03*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01+4.3831168831168834e-04*w52*Jinv10-3.7804383116883116e-03*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00+1.5523538961038962e-04*Jinv02*w27-3.7804383116883116e-03*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+1.5523538961038962e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-3.7804383116883116e-03*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33-4.0178571428571428e-04*w0*Jinv00-1.4793019480519481e-03*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-4.0178571428571428e-04*Jinv20*w2-1.4793019480519481e-03*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02+1.0957792207792207e-03*Jinv01*w9-1.4793019480519481e-03*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20+1.5523538961038962e-04*Jinv22*w29+1.2327516233766234e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36+8.2183441558441553e-04*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+1.5523538961038962e-04*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+1.5523538961038962e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01+1.4610389610389610e-04*Jinv12*w1+1.0957792207792207e-03*w10*Jinv11)*D;
  A[8][6]=(0.0)*D;
  A[8][7]=(0.0)*D;
  A[8][8]=(7.3965097402597403e-04*Jinv00*w12-1.0957792207792207e-03*w40*Jinv12+2.3833198051948050e-03*Jinv22*w53-7.4878246753246752e-04*w57*Jinv02+7.3965097402597403e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+7.3965097402597403e-04*w32*Jinv20+2.6298701298701297e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-3.6982548701298703e-03*Jinv22*w38-7.4878246753246752e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12-7.8530844155844158e-04*w0*Jinv02-7.8530844155844158e-04*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-4.2735389610389614e-03*w16*Jinv11-4.2735389610389614e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-4.2735389610389614e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24-4.2735389610389614e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+2.3833198051948050e-03*Jinv21*w26-7.8530844155844158e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10-3.6982548701298703e-03*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.0957792207792207e-03*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-1.0957792207792207e-03*w11*Jinv22-7.8530844155844158e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11-4.2735389610389614e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50-4.2735389610389614e-03*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12+7.3965097402597403e-04*w31*Jinv10+2.6298701298701297e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.8530844155844158e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-3.6982548701298703e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+2.3833198051948050e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11-3.6982548701298703e-03*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.2735389610389614e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+2.6298701298701297e-03*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-3.6982548701298703e-03*Jinv21*w20-3.6982548701298703e-03*w37*Jinv12-1.0957792207792207e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-1.1870941558441558e-03*w11*Jinv20+2.3833198051948050e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+7.3965097402597403e-04*Jinv20*w14+2.6298701298701297e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54+2.6298701298701297e-03*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-3.6982548701298703e-03*Jinv01*w18-3.6982548701298703e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-7.3965097402597406e-03*w6*Jinv01+3.5612824675324673e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-7.3965097402597406e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11-7.3965097402597406e-03*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49-3.6982548701298703e-03*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-7.3965097402597406e-03*w7*Jinv11-1.1870941558441558e-03*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27-7.3965097402597406e-03*w6*Jinv02-3.6982548701298703e-03*w19*Jinv12+2.6298701298701297e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-1.1870941558441558e-03*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-3.6982548701298703e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-7.3965097402597406e-03*Jinv21*w8-4.2735389610389614e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+7.3965097402597403e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-3.6982548701298703e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.3833198051948050e-03*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+2.3833198051948050e-03*w25*Jinv11-5.4788961038961036e-04*w42*Jinv00-4.2735389610389614e-03*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01-7.8530844155844158e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[8][9]=(0.0)*D;
  A[8][10]=(0.0)*D;
  A[8][11]=(-2.6481331168831169e-04*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+2.3741883116883116e-03*Jinv20*w8-2.6481331168831169e-04*w13*Jinv10-1.0227272727272728e-03*w3*Jinv00-2.6481331168831169e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23-5.4788961038961042e-05*Jinv00*w54+3.2873376623376621e-04*w43*Jinv10+1.5726461038961039e-04*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00+2.0495129870129870e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-5.4788961038961042e-05*w45*Jinv00+1.5726461038961039e-04*Jinv00*w27-1.0957792207792208e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-2.6481331168831169e-04*w31*Jinv10-1.0957792207792208e-04*w21*Jinv00+1.1870941558441558e-03*Jinv00*w18+1.1870941558441558e-03*w36*Jinv00-6.7573051948051951e-04*w51*Jinv00-1.0227272727272728e-03*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+1.1870941558441558e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+2.3741883116883116e-03*w6*Jinv00-5.4788961038961042e-05*Jinv20*w56-6.7573051948051951e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-2.6481331168831169e-04*Jinv20*w14-1.0957792207792208e-04*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+2.3741883116883116e-03*w7*Jinv10+1.1870941558441558e-03*Jinv20*w38-6.7573051948051951e-04*Jinv20*w53-1.0957792207792208e-04*w22*Jinv10-6.7573051948051951e-04*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+1.1870941558441558e-03*w37*Jinv10-6.7573051948051951e-04*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+2.0495129870129870e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-6.7573051948051951e-04*w25*Jinv10+2.0495129870129870e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+6.5746753246753243e-04*Jinv20*w17-2.6481331168831169e-04*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+1.1870941558441558e-03*w20*Jinv20-1.0227272727272728e-03*w5*Jinv20-5.4788961038961042e-05*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20)*D;
  A[8][12]=(0.0)*D;
  A[8][13]=(0.0)*D;
  A[8][14]=(2.0271915584415584e-03*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+2.5202922077922079e-03*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8+3.1960227272727273e-04*w57*Jinv02+2.0271915584415584e-03*w13*Jinv10+2.5202922077922079e-03*Jinv02*w39+3.2873376623376621e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.0135957792207792e-03*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23-1.9176136363636362e-04*Jinv22*w38-4.6570616883116886e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59-1.3697240259740259e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-6.6659902597402602e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+3.2873376623376621e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21+6.0267857142857139e-04*w16*Jinv11+6.0267857142857139e-04*Jinv21*w17+2.7394480519480521e-05*Jinv01*w45-4.6570616883116886e-04*Jinv12*w55+3.1960227272727273e-04*w58*Jinv10+2.7394480519480520e-03*w15*Jinv02+4.8396915584415583e-04*w10*Jinv12-8.4922889610389610e-04*w48*Jinv00-1.3697240259740259e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26-6.6659902597402602e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+2.7394480519480520e-03*w16*Jinv10-3.8352272727272725e-04*Jinv22*w20+3.2873376623376621e-04*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+1.2053571428571428e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+4.8396915584415583e-04*w11*Jinv22-6.6659902597402602e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+6.0267857142857139e-04*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11-8.4922889610389610e-04*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+2.7394480519480520e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12+1.0135957792207792e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.0135957792207792e-03*w32*Jinv22+1.2053571428571428e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-3.8352272727272725e-04*Jinv00*w18-1.9176136363636362e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24+3.1960227272727273e-04*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11+2.5202922077922079e-03*w41*Jinv20+2.0271915584415584e-03*Jinv12*w13-3.8352272727272725e-04*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.3697240259740260e-03*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00-1.9176136363636362e-04*w37*Jinv12-4.6570616883116886e-04*Jinv20*w56+2.5202922077922079e-03*Jinv22*w41-1.3697240259740259e-04*w26*Jinv20+4.8396915584415583e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.3697240259740260e-03*w31*Jinv11-1.3697240259740259e-04*Jinv22*w26+2.7394480519480521e-05*w46*Jinv11+2.0271915584415584e-03*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21-4.6570616883116886e-04*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02-8.4922889610389610e-04*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+2.5202922077922079e-03*w39*Jinv00+2.0271915584415584e-03*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11+4.8396915584415583e-04*Jinv02*w9-3.2873376623376621e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+2.7394480519480521e-05*w47*Jinv21-1.9176136363636362e-04*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23+2.7394480519480518e-04*w52*Jinv11+1.0135957792207792e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-1.4793019480519481e-03*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-8.4922889610389610e-04*Jinv22*w50+9.5880681818181818e-04*w22*Jinv11-1.4793019480519481e-03*Jinv12*w7+9.5880681818181818e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51+1.0135957792207792e-03*Jinv02*w30-8.4922889610389610e-04*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10-3.2873376623376621e-04*w7*Jinv11+4.8396915584415583e-04*w9*Jinv00-1.9176136363636362e-04*w37*Jinv10-1.9176136363636362e-04*Jinv21*w56-1.3697240259740259e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27-1.4793019480519481e-03*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12+1.2327516233766234e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+4.8396915584415583e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44+3.1960227272727273e-04*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12-3.2873376623376621e-04*Jinv21*w8+3.1960227272727273e-04*w57*Jinv00+3.1960227272727273e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20+9.5880681818181818e-04*Jinv21*w23-1.9176136363636362e-04*Jinv01*w54-1.9176136363636362e-04*w55*Jinv11-1.3697240259740259e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+2.7394480519480520e-03*w15*Jinv00-8.4922889610389610e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+2.7394480519480520e-03*Jinv20*w17+1.0135957792207792e-03*w30*Jinv00+2.5202922077922079e-03*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29+2.0271915584415584e-03*Jinv22*w14+1.2053571428571428e-03*w40*Jinv11-1.9176136363636362e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.3697240259740260e-03*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-4.6570616883116886e-04*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-4.6570616883116886e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+2.7394480519480520e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[8][15]=(0.0)*D;
  A[8][16]=(0.0)*D;
  A[8][17]=(-1.8628246753246754e-03*Jinv00*w12-5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-9.8620129870129864e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-1.8628246753246754e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.3141233766233771e-04*w32*Jinv20-2.7942370129870131e-03*Jinv01*w3+6.0267857142857139e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02+1.6436688311688311e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-6.9399350649350649e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+1.2053571428571428e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01-4.9310064935064932e-04*Jinv21*w26+1.6436688311688311e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20+3.2873376623376621e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.6615259740259744e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+1.6436688311688311e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+1.2053571428571428e-03*Jinv20*w50-5.6615259740259744e-04*Jinv20*w29+3.2873376623376621e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-2.9586038961038961e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12-9.3141233766233771e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3+6.0267857142857139e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-2.0819805194805196e-03*Jinv00*w18-1.0409902597402598e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45-4.9310064935064932e-04*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-1.3149350649350649e-03*Jinv21*w53+1.2601461038961039e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-2.7942370129870131e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-2.0819805194805196e-03*w19*Jinv10-2.9586038961038961e-03*w34*Jinv10-1.7532467532467534e-03*w32*Jinv21-9.8620129870129864e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20-4.3831168831168834e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-1.7532467532467534e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+3.2873376623376621e-04*w46*Jinv11-1.8628246753246754e-03*Jinv20*w14-2.7942370129870131e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02+1.2053571428571428e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12+2.1367694805194807e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+8.5470779220779228e-03*w6*Jinv01-9.8620129870129864e-04*w7*Jinv10+3.2873376623376621e-04*w47*Jinv21-1.0409902597402598e-03*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23-1.3149350649350649e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+1.2601461038961039e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8+6.0267857142857139e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+6.0267857142857139e-04*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+6.0267857142857139e-04*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+1.2601461038961039e-03*w52*Jinv10+8.5470779220779228e-03*w7*Jinv11-4.3831168831168834e-04*w9*Jinv00-1.0409902597402598e-03*w37*Jinv10+1.3697240259740260e-03*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14-4.3831168831168834e-04*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12+8.5470779220779228e-03*Jinv21*w8-6.9399350649350649e-04*w57*Jinv00-6.9399350649350649e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-2.9586038961038961e-03*w35*Jinv20+6.0267857142857139e-04*Jinv21*w23+1.3697240259740260e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17-9.3141233766233771e-04*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10-2.0819805194805196e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.7532467532467534e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28-4.9310064935064932e-04*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+3.2873376623376621e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[8][18]=(0.0)*D;
  A[8][19]=(0.0)*D;
  A[8][20]=(-1.6436688311688311e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+2.4655032467532467e-03*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10+2.1367694805194807e-03*Jinv21*w44-1.2327516233766234e-03*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20-2.6298701298701297e-03*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23-3.5612824675324673e-04*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.8530844155844158e-04*Jinv21*w2+1.5888798701298701e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+4.2735389610389614e-03*w16*Jinv11+4.2735389610389614e-03*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+3.7439123376623376e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01-2.3833198051948050e-03*Jinv21*w26+7.8530844155844158e-04*w0*Jinv01+3.9265422077922079e-04*w1*Jinv10+3.1777597402597403e-03*w16*Jinv10+2.1367694805194807e-03*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+3.9265422077922079e-04*Jinv00*w27+7.8530844155844158e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11+4.2735389610389614e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-8.2183441558441553e-05*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+2.4655032467532467e-03*Jinv00*w18+1.2327516233766234e-03*w36*Jinv00+3.6982548701298703e-03*Jinv21*w38-2.3833198051948050e-03*Jinv01*w24+4.2735389610389614e-03*w34*Jinv11+3.6982548701298703e-03*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.0135957792207792e-03*w51*Jinv00-1.2327516233766234e-03*w4*Jinv10-2.6298701298701297e-03*w4*Jinv11+1.0957792207792207e-03*w41*Jinv20+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21+3.6982548701298703e-03*Jinv21*w20+2.4655032467532467e-03*w6*Jinv00-3.5612824675324673e-04*Jinv20*w56-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20+4.2735389610389614e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-2.6298701298701297e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33+1.0957792207792207e-03*w39*Jinv00+3.6982548701298703e-03*Jinv01*w18+3.6982548701298703e-03*w37*Jinv11+7.3965097402597406e-03*w6*Jinv01+2.4655032467532467e-03*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+1.2327516233766234e-03*Jinv20*w38-2.3833198051948050e-03*w52*Jinv11-1.0135957792207792e-03*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-7.1225649350649346e-04*w22*Jinv11-7.1225649350649346e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+3.6982548701298703e-03*w36*Jinv01-1.0135957792207792e-03*w52*Jinv10+7.3965097402597406e-03*w7*Jinv11-4.5657467532467534e-05*w9*Jinv00+1.2327516233766234e-03*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56-1.2327516233766234e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14-4.5657467532467534e-05*w10*Jinv10+1.5888798701298701e-03*Jinv20*w44-1.6436688311688311e-04*Jinv01*w12+7.3965097402597406e-03*Jinv21*w8+3.7439123376623376e-04*w57*Jinv00+3.7439123376623376e-04*Jinv20*w59+3.9265422077922079e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-7.1225649350649346e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-1.2327516233766234e-03*w25*Jinv10+3.9265422077922079e-04*Jinv20*w2+3.1777597402597403e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+3.1777597402597403e-03*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-1.2327516233766234e-03*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30-2.3833198051948050e-03*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10+1.5888798701298701e-03*w42*Jinv00-8.2183441558441553e-05*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[8][21]=(0.0)*D;
  A[8][22]=(0.0)*D;
  A[8][23]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39-3.2873376623376621e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20+9.5880681818181818e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+3.5612824675324673e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10-3.2873376623376621e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.7394480519480518e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+3.5612824675324673e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+7.6704545454545450e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20-3.2873376623376621e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.2601461038961039e-03*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+3.5612824675324673e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-1.2601461038961039e-03*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10+9.5880681818181818e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+1.1231737012987013e-03*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+1.1231737012987013e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26-5.2049512987012989e-04*Jinv20*w14+9.5880681818181818e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9-1.4245129870129869e-03*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+3.5612824675324673e-04*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-1.4245129870129869e-03*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+1.9176136363636362e-04*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12-1.4245129870129869e-03*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+1.9176136363636362e-04*Jinv01*w54+1.9176136363636362e-04*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+7.6704545454545450e-04*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+7.6704545454545450e-04*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14-1.2601461038961039e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20+1.1231737012987013e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28-2.7394480519480518e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[8][24]=(0.0)*D;
  A[8][25]=(0.0)*D;
  A[8][26]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*Jinv20*w8+5.2049512987012989e-04*w13*Jinv10-8.2183441558441553e-04*Jinv21*w44+5.2049512987012989e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+1.4793019480519481e-03*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23+2.7394480519480518e-04*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-3.1047077922077924e-04*Jinv21*w2-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-1.6436688311688311e-03*w16*Jinv11-1.6436688311688311e-03*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45-1.0957792207792208e-04*w58*Jinv10+8.2183441558441553e-05*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+5.7528409090909093e-04*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-8.2183441558441553e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-3.1047077922077924e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-1.6436688311688311e-03*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11+8.2183441558441553e-05*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39-1.6436688311688311e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+5.7528409090909093e-04*Jinv01*w24-2.6298701298701297e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00+5.2049512987012989e-04*w4*Jinv10+1.4793019480519481e-03*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-1.6436688311688311e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-7.1225649350649346e-04*w6*Jinv00+2.7394480519480518e-04*Jinv20*w56-1.6436688311688311e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20-2.6298701298701297e-03*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-8.2183441558441553e-05*w46*Jinv11+5.2049512987012989e-04*Jinv20*w14+1.4793019480519481e-03*w5*Jinv21+8.2183441558441553e-05*w49*Jinv10-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.4655032467532466e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-4.7666396103896100e-03*w6*Jinv01-7.1225649350649346e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10-4.7666396103896100e-03*w7*Jinv11+9.1314935064935065e-06*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+5.4788961038961036e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14+9.1314935064935065e-06*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+1.3697240259740259e-04*Jinv01*w12-4.7666396103896100e-03*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+5.2049512987012989e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+5.7528409090909093e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+5.4788961038961036e-04*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[8][27]=(0.0)*D;
  A[8][28]=(0.0)*D;
  A[8][29]=(2.1915584415584417e-04*Jinv21*w44-5.8441558441558442e-04*Jinv01*w3+1.0247564935064935e-04*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+2.1915584415584417e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-2.6481331168831169e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+2.1915584415584417e-04*w43*Jinv11+3.4699675324675324e-04*Jinv21*w41-1.4204545454545455e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+3.4699675324675324e-04*Jinv01*w39+7.4878246753246752e-04*Jinv21*w38-2.6481331168831169e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-5.8441558441558442e-04*w4*Jinv11-4.2918019480519480e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-5.8441558441558442e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+3.6525974025974026e-05*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+1.4975649350649350e-03*w6*Jinv01-5.4788961038961042e-05*w47*Jinv21-4.2918019480519480e-04*w52*Jinv11-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01+1.4975649350649350e-03*w7*Jinv11-2.7394480519480521e-05*Jinv21*w56-2.1915584415584417e-04*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+1.1870941558441559e-04*Jinv01*w12+1.4975649350649350e-03*Jinv21*w8-1.0957792207792208e-04*Jinv21*w23-2.7394480519480521e-05*Jinv01*w54-2.7394480519480521e-05*w55*Jinv11+6.4935064935064935e-05*Jinv01*w9+3.4699675324675324e-04*w40*Jinv11-4.2918019480519480e-04*Jinv01*w30-2.6481331168831169e-04*w25*Jinv11-2.1915584415584417e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-2.1915584415584417e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[8][30]=(0.0)*D;
  A[8][31]=(0.0)*D;
  A[8][32]=(1.0135957792207792e-03*Jinv00*w12+3.1960227272727273e-04*w28*Jinv10+1.2053571428571428e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02+1.0135957792207792e-03*w13*Jinv10+1.2053571428571428e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+2.0271915584415584e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3-8.4922889610389610e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59+2.7394480519480518e-04*w25*Jinv12-6.6659902597402602e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+4.8396915584415583e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-4.6570616883116886e-04*Jinv01*w45+2.7394480519480521e-05*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24+6.0267857142857139e-04*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-4.6570616883116886e-04*w45*Jinv00+2.5202922077922079e-03*Jinv21*w41+3.2873376623376621e-04*w42*Jinv02+3.1960227272727273e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.0135957792207792e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+3.1960227272727273e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+3.1960227272727273e-04*Jinv20*w29-4.6570616883116886e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17+2.7394480519480520e-03*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12+2.0271915584415584e-03*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3-8.4922889610389610e-04*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+2.5202922077922079e-03*Jinv01*w39-6.6659902597402602e-04*Jinv22*w2-1.9176136363636362e-04*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-3.8352272727272725e-04*Jinv21*w38-1.9176136363636362e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+2.7394480519480520e-03*w34*Jinv11-1.9176136363636362e-04*w19*Jinv11-1.3697240259740259e-04*Jinv21*w53-1.3697240259740259e-04*w51*Jinv00+6.0267857142857139e-04*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11+2.5202922077922079e-03*w41*Jinv20-1.3697240259740260e-03*Jinv12*w13-1.9176136363636362e-04*w19*Jinv10+2.7394480519480520e-03*w34*Jinv10+2.0271915584415584e-03*w32*Jinv21-1.9176136363636362e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+1.2053571428571428e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+4.8396915584415583e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+2.7394480519480520e-03*Jinv21*w35+2.0271915584415584e-03*w31*Jinv11+2.7394480519480518e-04*Jinv22*w26-4.6570616883116886e-04*w46*Jinv11+1.0135957792207792e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+2.7394480519480521e-05*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+3.1960227272727273e-04*Jinv21*w29+2.7394480519480520e-03*Jinv01*w33+2.5202922077922079e-03*w39*Jinv00-1.3697240259740260e-03*Jinv02*w12-1.9176136363636362e-04*Jinv01*w18-3.8352272727272725e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-1.4793019480519481e-03*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10-4.6570616883116886e-04*w47*Jinv21-3.8352272727272725e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23-1.3697240259740259e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-1.9176136363636362e-04*w46*Jinv12-1.3697240259740259e-04*Jinv20*w53-3.2873376623376621e-04*Jinv22*w8-8.4922889610389610e-04*w22*Jinv10+9.5880681818181818e-04*Jinv22*w50-8.4922889610389610e-04*w22*Jinv11-3.2873376623376621e-04*Jinv12*w7-8.4922889610389610e-04*w21*Jinv01-1.3697240259740259e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+9.5880681818181818e-04*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-1.3697240259740259e-04*w52*Jinv10-1.4793019480519481e-03*w7*Jinv11+4.8396915584415583e-04*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-3.2873376623376621e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5-1.9176136363636362e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.0135957792207792e-03*Jinv21*w14+4.8396915584415583e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18+1.0135957792207792e-03*Jinv01*w12-1.4793019480519481e-03*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+6.0267857142857139e-04*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+2.7394480519480520e-03*w35*Jinv20-8.4922889610389610e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00+9.5880681818181818e-04*w48*Jinv02+4.8396915584415583e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+2.0271915584415584e-03*w30*Jinv00+2.5202922077922079e-03*w40*Jinv10-1.9176136363636362e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29-1.3697240259740260e-03*Jinv22*w14+2.5202922077922079e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20+2.0271915584415584e-03*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+2.7394480519480521e-05*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-4.6570616883116886e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+3.1960227272727273e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01-6.6659902597402602e-04*Jinv12*w1+4.8396915584415583e-04*w10*Jinv11)*D;
  A[8][33]=(0.0)*D;
  A[8][34]=(0.0)*D;
  A[8][35]=(-9.3141233766233771e-04*Jinv00*w12-6.9399350649350649e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12-4.9310064935064932e-04*Jinv22*w53-9.8620129870129864e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-9.3141233766233771e-04*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.8628246753246754e-03*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3+1.2053571428571428e-03*Jinv20*w23+3.2873376623376621e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59-1.3149350649350649e-03*w25*Jinv12+1.6436688311688311e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+3.2873376623376621e-04*Jinv12*w55-5.6615259740259744e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+6.0267857142857139e-04*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-2.9586038961038961e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-6.9399350649350649e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11+6.0267857142857139e-04*Jinv20*w50-6.9399350649350649e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-5.9172077922077923e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-1.8628246753246754e-03*w31*Jinv10-2.7942370129870131e-03*Jinv02*w3+1.2053571428571428e-03*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+1.6436688311688311e-04*Jinv22*w2-1.0409902597402598e-03*Jinv00*w18-2.0819805194805196e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+1.3149350649350649e-03*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+2.1367694805194807e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-1.7532467532467534e-03*Jinv12*w13-1.0409902597402598e-03*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-9.8620129870129864e-04*w6*Jinv00+3.2873376623376621e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+1.2601461038961039e-03*w26*Jinv20-4.3831168831168834e-04*w11*Jinv20-4.9310064935064932e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11-1.3149350649350649e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-9.3141233766233771e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+3.2873376623376621e-04*Jinv02*w54-2.7942370129870131e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+6.0267857142857139e-04*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-1.7532467532467534e-03*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01-9.8620129870129864e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-2.0819805194805196e-03*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12+1.3697240259740260e-03*Jinv20*w53+8.5470779220779228e-03*Jinv22*w8+1.2053571428571428e-03*w22*Jinv10+6.0267857142857139e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+8.5470779220779228e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+6.0267857142857139e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-4.3831168831168834e-04*w9*Jinv00-2.0819805194805196e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56+1.2601461038961039e-03*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27+8.5470779220779228e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-2.7942370129870131e-03*Jinv22*w5+7.6704545454545450e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-4.3831168831168834e-04*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8-5.6615259740259744e-04*w57*Jinv00-5.6615259740259744e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+1.2601461038961039e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-2.9586038961038961e-03*w15*Jinv00+6.0267857142857139e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-2.9586038961038961e-03*Jinv20*w17-1.8628246753246754e-03*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10-1.0409902597402598e-03*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-1.7532467532467534e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+7.6704545454545450e-04*Jinv01*w30-4.9310064935064932e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11+3.2873376623376621e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01+1.6436688311688311e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[8][36]=(0.0)*D;
  A[8][37]=(0.0)*D;
  A[8][38]=(-8.2183441558441553e-05*Jinv00*w12+3.7439123376623376e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.3833198051948050e-03*Jinv22*w53+2.4655032467532467e-03*Jinv20*w8+7.4878246753246752e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.2327516233766234e-03*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+3.6982548701298703e-03*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-2.3833198051948050e-03*w25*Jinv12+7.8530844155844158e-04*w0*Jinv02+1.5888798701298701e-03*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10+4.2735389610389614e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00-2.3833198051948050e-03*Jinv02*w24+4.2735389610389614e-03*Jinv22*w35+3.9265422077922079e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+3.6982548701298703e-03*Jinv22*w20-3.5612824675324673e-04*w45*Jinv00+2.1367694805194807e-03*w42*Jinv02+3.7439123376623376e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50+3.7439123376623376e-04*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+4.2735389610389614e-03*Jinv22*w17+3.1777597402597403e-03*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-2.6298701298701297e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-1.6436688311688311e-04*w32*Jinv22+7.8530844155844158e-04*Jinv22*w2+1.2327516233766234e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00-3.5612824675324673e-04*Jinv02*w45+7.4878246753246752e-04*Jinv22*w59-1.2327516233766234e-03*w51*Jinv00+4.2735389610389614e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-1.2327516233766234e-03*w4*Jinv10+1.0957792207792207e-03*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13+1.2327516233766234e-03*w19*Jinv10+3.1777597402597403e-03*w34*Jinv10+2.4655032467532467e-03*w6*Jinv00+3.6982548701298703e-03*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-1.0135957792207792e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20-2.3833198051948050e-03*w52*Jinv12-2.3833198051948050e-03*Jinv22*w26-8.2183441558441553e-05*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-2.6298701298701297e-03*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10+1.0957792207792207e-03*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+2.4655032467532467e-03*w7*Jinv10+2.4655032467532467e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-1.6436688311688311e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12-1.2327516233766234e-03*Jinv20*w53+7.3965097402597406e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-7.1225649350649346e-04*Jinv22*w50+7.3965097402597406e-03*Jinv12*w7-1.6436688311688311e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-1.2327516233766234e-03*w52*Jinv10-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.0135957792207792e-03*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+7.3965097402597406e-03*w6*Jinv02+3.6982548701298703e-03*w19*Jinv12-2.6298701298701297e-03*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22-4.5657467532467534e-05*w10*Jinv10+1.5888798701298701e-03*Jinv20*w44+7.4878246753246752e-04*w58*Jinv12+3.6982548701298703e-03*Jinv02*w18+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+4.2735389610389614e-03*Jinv02*w33+3.9265422077922079e-04*w0*Jinv00+3.1777597402597403e-03*w35*Jinv20-1.0135957792207792e-03*w25*Jinv10+3.9265422077922079e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-7.1225649350649346e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+1.2327516233766234e-03*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14+3.6982548701298703e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-1.2327516233766234e-03*w5*Jinv20-2.3833198051948050e-03*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28-8.2183441558441553e-05*w55*Jinv10+1.5888798701298701e-03*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+4.2735389610389614e-03*w16*Jinv12+7.8530844155844158e-04*Jinv12*w1)*D;
  A[8][39]=(0.0)*D;
  A[8][40]=(0.0)*D;
  A[8][41]=(5.4788961038961042e-05*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10-1.6436688311688311e-03*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-1.6436688311688311e-03*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-1.7532467532467534e-03*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23+5.4788961038961042e-05*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-3.8352272727272725e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10-4.9310064935064932e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+8.2183441558441553e-04*Jinv01*w45+8.2183441558441553e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20+5.4788961038961042e-05*w45*Jinv00-1.6436688311688311e-03*Jinv21*w41+3.6525974025974026e-05*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-1.2601461038961039e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10-1.7532467532467534e-03*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-1.2601461038961039e-03*w32*Jinv22-1.6436688311688311e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2-1.0957792207792207e-03*Jinv00*w18-1.0957792207792207e-03*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38+3.2873376623376621e-04*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11-3.8352272727272725e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.8352272727272725e-04*w4*Jinv10-1.7532467532467534e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20-2.5750811688311690e-03*Jinv12*w13-1.0957792207792207e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-2.5750811688311690e-03*w32*Jinv21-1.0957792207792207e-03*w6*Jinv00+5.4788961038961042e-05*Jinv20*w56-1.6436688311688311e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+3.2873376623376621e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35-2.5750811688311690e-03*w31*Jinv11-3.8352272727272725e-04*Jinv22*w26+8.2183441558441553e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14-1.7532467532467534e-03*w5*Jinv21+8.2183441558441553e-04*Jinv02*w54-1.7532467532467534e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00-2.5750811688311690e-03*Jinv02*w12+1.0957792207792207e-03*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+2.1915584415584414e-03*w6*Jinv01-1.0957792207792207e-03*w7*Jinv10+8.2183441558441553e-04*w47*Jinv21-1.0957792207792207e-03*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-3.8352272727272725e-04*w52*Jinv11-1.2601461038961039e-03*w31*Jinv12+3.8352272727272725e-04*Jinv20*w53+2.1915584415584414e-03*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+1.2053571428571428e-03*Jinv22*w50+1.2053571428571428e-03*w22*Jinv11+2.1915584415584414e-03*Jinv12*w7+1.2053571428571428e-03*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51-1.2601461038961039e-03*Jinv02*w30+1.2053571428571428e-03*Jinv12*w49+1.0957792207792207e-03*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+2.1915584415584414e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-1.0957792207792207e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27+2.1915584415584414e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-1.7532467532467534e-03*Jinv22*w5-1.0957792207792208e-04*w49*Jinv11-1.2601461038961039e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44-3.8352272727272725e-04*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18-1.2601461038961039e-03*Jinv01*w12+2.1915584415584414e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.2053571428571428e-03*Jinv21*w23+3.8352272727272725e-04*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.2053571428571428e-03*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10-1.0957792207792207e-03*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-2.5750811688311690e-03*Jinv22*w14-1.6436688311688311e-03*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-2.5750811688311690e-03*Jinv01*w30+3.2873376623376621e-04*Jinv02*w51+8.2183441558441553e-04*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+3.2873376623376621e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[8][42]=(0.0)*D;
  A[8][43]=(0.0)*D;
  A[8][44]=(-5.4788961038961042e-05*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-5.4788961038961042e-05*Jinv20*w23+2.1367694805194807e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45-5.4788961038961042e-05*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-5.4788961038961042e-05*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-3.6525974025974026e-05*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2+1.0957792207792207e-03*Jinv00*w18+1.0957792207792207e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-1.3149350649350649e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11-3.2873376623376621e-03*w41*Jinv20-7.6704545454545450e-04*Jinv12*w13+1.0957792207792207e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-5.4788961038961042e-05*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-5.4788961038961042e-05*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-5.4788961038961042e-05*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-3.2873376623376621e-03*w39*Jinv00-7.6704545454545450e-04*Jinv02*w12+2.1367694805194807e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9+4.2735389610389614e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.0957792207792207e-03*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11+4.2735389610389614e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+4.2735389610389614e-03*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+1.0957792207792207e-03*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-3.6525974025974026e-05*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-3.2873376623376621e-03*w40*Jinv10+1.0957792207792207e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+2.1367694805194807e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30-1.3149350649350649e-03*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-1.3149350649350649e-03*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[8][45]=(0.0)*D;
  A[8][46]=(0.0)*D;
  A[8][47]=(-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-1.0957792207792208e-04*w57*Jinv02-5.4788961038961042e-05*Jinv02*w39-7.1225649350649346e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3-3.5612824675324673e-04*Jinv22*w38-1.3697240259740259e-04*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-7.1225649350649346e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45+2.7394480519480518e-04*Jinv12*w55-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-7.1225649350649346e-04*w43*Jinv11-8.7662337662337668e-04*Jinv21*w41-3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.6704545454545450e-04*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-8.7662337662337668e-04*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24-1.0957792207792208e-04*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+1.9176136363636362e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.2049512987012989e-04*Jinv12*w13+5.4788961038961036e-04*w32*Jinv21+1.9176136363636362e-04*Jinv21*w20-3.5612824675324673e-04*w37*Jinv12-5.4788961038961042e-05*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-3.5612824675324673e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21+2.7394480519480518e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+5.2049512987012989e-04*Jinv02*w12+1.9176136363636362e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23+5.2049512987012989e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14-1.0957792207792208e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+2.4655032467532466e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11+8.2183441558441553e-05*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-8.7662337662337668e-04*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+5.4788961038961036e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11-7.6704545454545450e-04*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[8][48]=(0.0)*D;
  A[8][49]=(0.0)*D;
  A[8][50]=(-2.7394480519480518e-04*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10-1.2601461038961039e-03*w40*Jinv12+7.1225649350649346e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-1.2601461038961039e-03*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-8.2183441558441553e-05*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.7394480519480518e-04*w25*Jinv12+3.5612824675324673e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41-3.2873376623376621e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17+7.6704545454545450e-04*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10+9.5880681818181818e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+3.5612824675324673e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+1.9176136363636362e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-1.3697240259740260e-03*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+1.1231737012987013e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10+7.6704545454545450e-04*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.2601461038961039e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20+7.6704545454545450e-04*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11+2.7394480519480518e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+9.5880681818181818e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+1.1231737012987013e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12+1.9176136363636362e-04*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-1.4245129870129869e-03*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11-1.4245129870129869e-03*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-1.4245129870129869e-03*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+9.5880681818181818e-04*Jinv22*w5+1.9176136363636362e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-1.3697240259740260e-03*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+7.6704545454545450e-04*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29+1.1231737012987013e-03*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30-1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+3.5612824675324673e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[8][51]=(0.0)*D;
  A[8][52]=(0.0)*D;
  A[8][53]=(2.7394480519480518e-04*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12+5.7528409090909093e-04*Jinv22*w53-7.1225649350649346e-04*Jinv20*w8-2.1002435064935065e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+5.2049512987012989e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+8.2183441558441553e-05*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-05*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-1.6436688311688311e-03*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+2.7394480519480518e-04*w45*Jinv00-8.2183441558441553e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12+5.2049512987012989e-04*w31*Jinv10+1.4793019480519481e-03*Jinv02*w3+8.2183441558441553e-05*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22-3.1047077922077924e-04*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+2.4655032467532466e-04*Jinv02*w45-2.1002435064935065e-04*Jinv22*w59-1.6436688311688311e-04*w51*Jinv00-1.6436688311688311e-03*Jinv12*w34-8.2183441558441553e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv10-5.4788961038961042e-05*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10-7.1225649350649346e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20+5.7528409090909093e-04*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-8.2183441558441553e-05*Jinv02*w54+1.4793019480519481e-03*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10-5.4788961038961042e-05*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-7.1225649350649346e-04*w7*Jinv10-1.6436688311688311e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+1.3697240259740259e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-4.7666396103896100e-03*Jinv22*w8+8.2183441558441553e-05*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50-4.7666396103896100e-03*Jinv12*w7+1.3697240259740259e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.6436688311688311e-04*w52*Jinv10+9.1314935064935065e-06*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-4.7666396103896100e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.4793019480519481e-03*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22+9.1314935064935065e-06*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44-2.1002435064935065e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.6436688311688311e-03*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-2.4655032467532466e-04*Jinv02*w36-8.2183441558441553e-04*Jinv12*w43+5.2049512987012989e-04*w5*Jinv20+5.7528409090909093e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28-3.5612824675324673e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-3.1047077922077924e-04*Jinv12*w1)*D;
  A[8][54]=(0.0)*D;
  A[8][55]=(0.0)*D;
  A[8][56]=(-8.7662337662337668e-04*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53+6.3920454545454550e-05*w57*Jinv02-8.7662337662337668e-04*Jinv02*w39-3.8352272727272725e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+1.9176136363636362e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+5.2049512987012989e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-3.5612824675324673e-04*Jinv12*w55-1.0957792207792208e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.2049512987012989e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-7.1225649350649346e-04*Jinv22*w20-3.8352272727272725e-04*w43*Jinv11-5.4788961038961042e-05*Jinv21*w41-7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-1.0957792207792208e-04*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.4655032467532466e-04*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+6.3920454545454550e-05*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-7.1225649350649346e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.4788961038961036e-04*Jinv12*w13+5.2049512987012989e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20+1.9176136363636362e-04*w37*Jinv12-8.7662337662337668e-04*Jinv22*w41-8.2183441558441553e-05*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+5.2049512987012989e-04*Jinv22*w26+2.7394480519480518e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-3.5612824675324673e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+5.4788961038961036e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+2.7394480519480518e-04*w47*Jinv21+5.4788961038961036e-04*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+2.4655032467532466e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+2.4655032467532466e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-3.5612824675324673e-04*Jinv21*w56-1.3697240259740259e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+6.3920454545454550e-05*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+8.2183441558441553e-05*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9-1.3697240259740259e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+1.9176136363636362e-04*Jinv02*w36-7.1225649350649346e-04*Jinv12*w43+5.2049512987012989e-04*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-1.0957792207792208e-04*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[8][57]=(0.0)*D;
  A[8][58]=(0.0)*D;
  A[8][59]=(3.4699675324675324e-04*w40*Jinv12-2.6481331168831169e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+3.4699675324675324e-04*Jinv02*w39+3.6525974025974026e-05*Jinv22*w38-4.2918019480519480e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02-5.4788961038961042e-05*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+7.4878246753246752e-04*Jinv22*w20+2.1915584415584417e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-2.1915584415584417e-04*w22*Jinv12-5.8441558441558442e-04*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2-2.7394480519480521e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+4.3831168831168834e-04*Jinv12*w34+2.1915584415584417e-04*Jinv22*w44-4.2918019480519480e-04*Jinv12*w13+3.6525974025974026e-05*w37*Jinv12+3.4699675324675324e-04*Jinv22*w41-2.6481331168831169e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-5.4788961038961042e-05*Jinv02*w54-5.8441558441558442e-04*w4*Jinv12-2.1915584415584417e-04*w21*Jinv02-4.2918019480519480e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9-2.1915584415584417e-04*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12-2.7394480519480521e-05*w46*Jinv12+1.4975649350649350e-03*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50+1.4975649350649350e-03*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+1.0247564935064935e-04*Jinv02*w27+1.4975649350649350e-03*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-5.8441558441558442e-04*Jinv22*w5-2.7394480519480521e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18+4.3831168831168834e-04*Jinv02*w33-1.0957792207792208e-04*w48*Jinv02+1.0247564935064935e-04*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+3.6525974025974026e-05*Jinv02*w36+2.1915584415584417e-04*Jinv12*w43-2.6481331168831169e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[9][0]=(-1.5726461038961039e-04*Jinv00*w12-5.4112554112554113e-05*w28*Jinv10+1.9480519480519481e-04*w40*Jinv12+1.4204545454545455e-05*Jinv22*w53-8.3198051948051951e-05*Jinv20*w8-5.4112554112554113e-05*w57*Jinv02-1.5726461038961039e-04*w13*Jinv10+1.9480519480519481e-04*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-2.0495129870129870e-04*w3*Jinv00-1.5726461038961039e-04*w32*Jinv20-2.0495129870129870e-04*Jinv01*w3+1.1668019480519480e-04*Jinv20*w23-6.4935064935064935e-05*Jinv22*w38-1.0247564935064935e-04*Jinv00*w54-5.4112554112554113e-05*Jinv21*w59+1.4204545454545455e-05*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-1.8939393939393939e-04*w11*Jinv21+4.2613636363636367e-05*w16*Jinv11+4.2613636363636367e-05*Jinv21*w17-1.0247564935064935e-04*Jinv01*w45-1.0247564935064935e-04*Jinv12*w55-5.4112554112554113e-05*w58*Jinv10+4.2613636363636367e-05*w15*Jinv02-1.8939393939393939e-04*w10*Jinv12+1.1668019480519480e-04*w48*Jinv00+1.4204545454545455e-05*Jinv02*w24+4.2613636363636367e-05*Jinv22*w35-5.4112554112554113e-05*w58*Jinv11-5.4112554112554113e-05*w57*Jinv01+1.4204545454545455e-05*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+4.2613636363636367e-05*w16*Jinv10-6.4935064935064935e-05*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11-1.0247564935064935e-04*w45*Jinv00+1.9480519480519481e-04*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-5.4112554112554113e-05*Jinv00*w27-1.8939393939393939e-04*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-1.5726461038961039e-04*w13*Jinv11+4.2613636363636367e-05*w15*Jinv01-5.4112554112554113e-05*w28*Jinv11+1.1668019480519480e-04*Jinv20*w50-5.4112554112554113e-05*Jinv20*w29-1.0247564935064935e-04*w46*Jinv10+4.2613636363636367e-05*Jinv22*w17+4.2613636363636367e-05*Jinv00*w33+1.1668019480519480e-04*w22*Jinv12-1.5726461038961039e-04*w31*Jinv10-2.0495129870129870e-04*Jinv02*w3+1.1668019480519480e-04*w21*Jinv00-1.5726461038961039e-04*w32*Jinv22+1.9480519480519481e-04*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2-6.4935064935064935e-05*Jinv00*w18-6.4935064935064935e-05*w36*Jinv00-6.4935064935064935e-05*Jinv21*w38-1.0247564935064935e-04*Jinv02*w45+1.4204545454545455e-05*Jinv01*w24-5.4112554112554113e-05*Jinv22*w59+4.2613636363636367e-05*w34*Jinv11-6.4935064935064935e-05*w19*Jinv11+1.4204545454545455e-05*Jinv21*w53+1.4204545454545455e-05*w51*Jinv00+4.2613636363636367e-05*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-2.0495129870129870e-04*w4*Jinv10-2.0495129870129870e-04*w4*Jinv11+1.9480519480519481e-04*w41*Jinv20-1.5726461038961039e-04*Jinv12*w13-6.4935064935064935e-05*w19*Jinv10+4.2613636363636367e-05*w34*Jinv10-1.5726461038961039e-04*w32*Jinv21-6.4935064935064935e-05*Jinv21*w20-8.3198051948051951e-05*w6*Jinv00-6.4935064935064935e-05*w37*Jinv12-1.0247564935064935e-04*Jinv20*w56+1.9480519480519481e-04*Jinv22*w41+1.4204545454545455e-05*w26*Jinv20-1.8939393939393939e-04*w11*Jinv20+1.4204545454545455e-05*w52*Jinv12+4.2613636363636367e-05*Jinv21*w35-1.5726461038961039e-04*w31*Jinv11+1.4204545454545455e-05*Jinv22*w26-1.0247564935064935e-04*w46*Jinv11-1.5726461038961039e-04*Jinv20*w14-2.0495129870129870e-04*w5*Jinv21-1.0247564935064935e-04*Jinv02*w54-2.0495129870129870e-04*w4*Jinv12+1.1668019480519480e-04*w21*Jinv02+1.1668019480519480e-04*w49*Jinv10-5.4112554112554113e-05*Jinv21*w29+4.2613636363636367e-05*Jinv01*w33+1.9480519480519481e-04*w39*Jinv00-1.5726461038961039e-04*Jinv02*w12-6.4935064935064935e-05*Jinv01*w18-6.4935064935064935e-05*w37*Jinv11-1.8939393939393939e-04*Jinv02*w9-8.3198051948051951e-05*w6*Jinv01-8.3198051948051951e-05*w7*Jinv10-1.0247564935064935e-04*w47*Jinv21-6.4935064935064935e-05*Jinv20*w38+1.1668019480519480e-04*Jinv22*w23+1.4204545454545455e-05*w52*Jinv11-1.5726461038961039e-04*w31*Jinv12-1.0247564935064935e-04*w46*Jinv12+1.4204545454545455e-05*Jinv20*w53-8.3198051948051951e-05*Jinv22*w8+1.1668019480519480e-04*w22*Jinv10+1.1668019480519480e-04*Jinv22*w50+1.1668019480519480e-04*w22*Jinv11-8.3198051948051951e-05*Jinv12*w7+1.1668019480519480e-04*w21*Jinv01+1.4204545454545455e-05*Jinv01*w51-1.5726461038961039e-04*Jinv02*w30+1.1668019480519480e-04*Jinv12*w49-6.4935064935064935e-05*w36*Jinv01+1.4204545454545455e-05*w52*Jinv10-8.3198051948051951e-05*w7*Jinv11-1.8939393939393939e-04*w9*Jinv00-6.4935064935064935e-05*w37*Jinv10-1.0247564935064935e-04*Jinv21*w56+1.4204545454545455e-05*w24*Jinv00-5.4112554112554113e-05*Jinv02*w27-8.3198051948051951e-05*w6*Jinv02-6.4935064935064935e-05*w19*Jinv12-2.0495129870129870e-04*Jinv22*w5-1.0247564935064935e-04*w47*Jinv22+1.1668019480519480e-04*w49*Jinv11-1.5726461038961039e-04*Jinv21*w14-1.8939393939393939e-04*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-5.4112554112554113e-05*w58*Jinv12-6.4935064935064935e-05*Jinv02*w18-1.5726461038961039e-04*Jinv01*w12-8.3198051948051951e-05*Jinv21*w8-5.4112554112554113e-05*w57*Jinv00-5.4112554112554113e-05*Jinv20*w59+4.2613636363636367e-05*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+4.2613636363636367e-05*w35*Jinv20+1.1668019480519480e-04*Jinv21*w23-1.0247564935064935e-04*Jinv01*w54-1.0247564935064935e-04*w55*Jinv11+1.4204545454545455e-05*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+4.2613636363636367e-05*w15*Jinv00+1.1668019480519480e-04*w48*Jinv02-1.8939393939393939e-04*Jinv01*w9+4.2613636363636367e-05*Jinv20*w17-1.5726461038961039e-04*w30*Jinv00+1.9480519480519481e-04*w40*Jinv10-6.4935064935064935e-05*w20*Jinv20-5.4112554112554113e-05*Jinv22*w29-1.5726461038961039e-04*Jinv22*w14+1.9480519480519481e-04*w40*Jinv11-6.4935064935064935e-05*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-2.0495129870129870e-04*w5*Jinv20-1.5726461038961039e-04*Jinv01*w30+1.4204545454545455e-05*Jinv02*w51-1.0247564935064935e-04*Jinv22*w56-5.4112554112554113e-05*Jinv12*w28+1.4204545454545455e-05*w25*Jinv11-1.0247564935064935e-04*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00-1.0247564935064935e-04*w47*Jinv20+4.2613636363636367e-05*w16*Jinv12+1.1668019480519480e-04*Jinv21*w50-5.4112554112554113e-05*Jinv01*w27+1.1668019480519480e-04*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-1.8939393939393939e-04*w10*Jinv11)*D;
  A[9][1]=(0.0)*D;
  A[9][2]=(0.0)*D;
  A[9][3]=(3.5612824675324673e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10-5.2962662337662338e-04*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53+6.2094155844155847e-04*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+1.2784090909090910e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.5568181818181820e-04*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59-2.1915584415584417e-04*w25*Jinv12+5.6818181818181818e-05*w0*Jinv02+5.6818181818181818e-05*Jinv21*w2-2.5568181818181820e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+6.0876623376623375e-04*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+6.0876623376623375e-04*w10*Jinv12-2.5568181818181820e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26+5.6818181818181818e-05*w0*Jinv01+1.2175324675324675e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00-5.2962662337662338e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27+6.0876623376623375e-04*w11*Jinv22+5.6818181818181818e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11-2.5568181818181820e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.7349837662337662e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3-2.5568181818181820e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+5.6818181818181818e-05*Jinv22*w2+5.5702110389610384e-04*Jinv00*w18+5.5702110389610384e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+1.2784090909090910e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11-2.5568181818181820e-04*w41*Jinv20-3.9265422077922079e-04*Jinv12*w13+5.5702110389610384e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-3.9265422077922079e-04*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20+6.2094155844155847e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56-5.2962662337662338e-04*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-3.9265422077922079e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12+1.7349837662337662e-04*w21*Jinv02-2.5568181818181820e-04*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-2.5568181818181820e-04*w39*Jinv00-3.9265422077922079e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+6.0876623376623375e-04*Jinv02*w9+1.0957792207792207e-03*w6*Jinv01+6.2094155844155847e-04*w7*Jinv10+1.3697240259740259e-04*w47*Jinv21+5.5702110389610384e-04*Jinv20*w38+1.7349837662337662e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8-2.5568181818181820e-04*w22*Jinv10+1.7349837662337662e-04*Jinv22*w50+1.7349837662337662e-04*w22*Jinv11+1.0957792207792207e-03*Jinv12*w7+1.7349837662337662e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+1.7349837662337662e-04*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+6.4935064935064935e-04*w9*Jinv00+5.5702110389610384e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+1.7349837662337662e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+6.4935064935064935e-04*w10*Jinv10-2.5568181818181820e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33+1.2175324675324675e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+1.7349837662337662e-04*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10+1.2175324675324675e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+1.7349837662337662e-04*w48*Jinv02+6.0876623376623375e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00-2.5568181818181820e-04*w40*Jinv10+5.5702110389610384e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-3.9265422077922079e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+1.2784090909090910e-04*w5*Jinv20-3.9265422077922079e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28-2.1915584415584417e-04*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10-2.5568181818181820e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+1.7349837662337662e-04*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+1.7349837662337662e-04*w48*Jinv01+5.6818181818181818e-05*Jinv12*w1+6.0876623376623375e-04*w10*Jinv11)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(0.0)*D;
  A[9][6]=(-2.1915584415584417e-04*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10+4.9310064935064932e-04*w40*Jinv12+5.4788961038961036e-04*Jinv22*w53-1.1870941558441558e-03*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02-2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv02*w39+5.4788961038961036e-04*Jinv21*w44+4.0178571428571428e-04*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3+1.1870941558441559e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12-6.4935064935064935e-05*w0*Jinv02-6.4935064935064935e-05*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+5.4788961038961036e-04*w42*Jinv01-1.5827922077922079e-03*w11*Jinv21+5.4788961038961036e-04*w16*Jinv11+5.4788961038961036e-04*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-1.5827922077922079e-03*w10*Jinv12+1.1870941558441559e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+5.4788961038961036e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+5.4788961038961036e-04*Jinv21*w26-6.4935064935064935e-05*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+5.4788961038961036e-04*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+4.9310064935064932e-04*Jinv21*w41+5.4788961038961036e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-1.5827922077922079e-03*w11*Jinv22-6.4935064935064935e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+5.4788961038961036e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11+1.1870941558441559e-04*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-9.1314935064935065e-06*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+1.1870941558441559e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-6.4935064935064935e-05*Jinv22*w2-1.1414366883116883e-03*Jinv00*w18-1.1414366883116883e-03*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45+5.4788961038961036e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+6.6659902597402602e-04*w51*Jinv00+5.4788961038961036e-04*Jinv12*w34+5.4788961038961036e-04*Jinv22*w44+4.0178571428571428e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-1.8262987012987014e-04*w41*Jinv20-9.1314935064935065e-06*Jinv12*w13-1.1414366883116883e-03*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10-9.1314935064935065e-06*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20-1.1870941558441558e-03*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+4.9310064935064932e-04*Jinv22*w41+6.6659902597402602e-04*w26*Jinv20-1.2987012987012987e-03*w11*Jinv20+5.4788961038961036e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35-9.1314935064935065e-06*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-9.1314935064935065e-06*w21*Jinv02+1.1870941558441559e-04*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-1.8262987012987014e-04*w39*Jinv00-9.1314935064935065e-06*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11-1.5827922077922079e-03*Jinv02*w9-1.0957792207792207e-03*w6*Jinv01-1.1870941558441558e-03*w7*Jinv10-9.1314935064935065e-06*w47*Jinv21-1.1414366883116883e-03*Jinv20*w38-9.1314935064935065e-06*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+6.6659902597402602e-04*Jinv20*w53-1.0957792207792207e-03*Jinv22*w8+1.1870941558441559e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11-1.0957792207792207e-03*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+6.6659902597402602e-04*w52*Jinv10-1.0957792207792207e-03*w7*Jinv11-1.2987012987012987e-03*w9*Jinv00-1.1414366883116883e-03*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+6.6659902597402602e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-1.0957792207792207e-03*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22-9.1314935064935065e-06*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.2987012987012987e-03*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-1.0957792207792207e-03*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59+5.4788961038961036e-04*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+6.6659902597402602e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-1.5827922077922079e-03*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-1.8262987012987014e-04*w40*Jinv10-1.1414366883116883e-03*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29-9.1314935064935065e-06*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+5.4788961038961036e-04*Jinv12*w43+4.0178571428571428e-04*w5*Jinv20-9.1314935064935065e-06*Jinv01*w30+5.4788961038961036e-04*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12-9.1314935064935065e-06*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-9.1314935064935065e-06*w48*Jinv01-6.4935064935064935e-05*Jinv12*w1-1.5827922077922079e-03*w10*Jinv11)*D;
  A[9][7]=(0.0)*D;
  A[9][8]=(0.0)*D;
  A[9][9]=(2.0292207792207792e-05*Jinv00*w12+9.4696969696969697e-05*w28*Jinv10+6.4935064935064935e-04*Jinv20*w8+2.0292207792207792e-05*w13*Jinv10-3.2467532467532468e-04*w3*Jinv00+2.0292207792207792e-05*w32*Jinv20+2.0292207792207792e-05*Jinv20*w23+2.0292207792207792e-05*Jinv00*w54+1.2175324675324675e-04*w43*Jinv10+9.4696969696969697e-05*w58*Jinv10+2.0292207792207792e-05*w48*Jinv00+9.4696969696969697e-05*w1*Jinv10+1.2175324675324675e-04*w16*Jinv10+2.0292207792207792e-05*w45*Jinv00+9.4696969696969697e-05*Jinv00*w27+2.0292207792207792e-05*Jinv20*w50+9.4696969696969697e-05*Jinv20*w29+2.0292207792207792e-05*w46*Jinv10+1.2175324675324675e-04*Jinv00*w33+2.0292207792207792e-05*w31*Jinv10+2.0292207792207792e-05*w21*Jinv00+6.4935064935064935e-04*Jinv00*w18+6.4935064935064935e-04*w36*Jinv00-3.2467532467532468e-04*w51*Jinv00-3.2467532467532468e-04*w4*Jinv10+2.4350649350649351e-04*w41*Jinv20+6.4935064935064935e-04*w19*Jinv10+1.2175324675324675e-04*w34*Jinv10+6.4935064935064935e-04*w6*Jinv00+2.0292207792207792e-05*Jinv20*w56-3.2467532467532468e-04*w26*Jinv20+8.3874458874458877e-04*w11*Jinv20+2.0292207792207792e-05*Jinv20*w14+2.0292207792207792e-05*w49*Jinv10+2.4350649350649351e-04*w39*Jinv00+6.4935064935064935e-04*w7*Jinv10+6.4935064935064935e-04*Jinv20*w38-3.2467532467532468e-04*Jinv20*w53+2.0292207792207792e-05*w22*Jinv10-3.2467532467532468e-04*w52*Jinv10+8.3874458874458877e-04*w9*Jinv00+6.4935064935064935e-04*w37*Jinv10-3.2467532467532468e-04*w24*Jinv00+8.3874458874458877e-04*w10*Jinv10+1.2175324675324675e-04*Jinv20*w44+9.4696969696969697e-05*w57*Jinv00+9.4696969696969697e-05*Jinv20*w59+9.4696969696969697e-05*w0*Jinv00+1.2175324675324675e-04*w35*Jinv20-3.2467532467532468e-04*w25*Jinv10+9.4696969696969697e-05*Jinv20*w2+1.2175324675324675e-04*w15*Jinv00+1.2175324675324675e-04*Jinv20*w17+2.0292207792207792e-05*w30*Jinv00+2.4350649350649351e-04*w40*Jinv10+6.4935064935064935e-04*w20*Jinv20-3.2467532467532468e-04*w5*Jinv20+2.0292207792207792e-05*w55*Jinv10+1.2175324675324675e-04*w42*Jinv00+2.0292207792207792e-05*w47*Jinv20)*D;
  A[9][10]=(0.0)*D;
  A[9][11]=(0.0)*D;
  A[9][12]=(-7.4878246753246752e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-9.6793831168831166e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53+4.8396915584415583e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02-7.4878246753246752e-04*w13*Jinv10-9.6793831168831166e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-3.7439123376623376e-04*w32*Jinv20+3.9265422077922079e-04*Jinv01*w3+1.5523538961038962e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54+3.1047077922077924e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+1.5726461038961039e-04*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01-5.6615259740259744e-04*w16*Jinv11-5.6615259740259744e-04*Jinv21*w17+3.1960227272727273e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-1.0592532467532468e-03*w15*Jinv02-4.0584415584415584e-05*w10*Jinv12+3.1960227272727273e-04*w48*Jinv00+3.1047077922077924e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+1.7349837662337662e-04*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-1.0592532467532468e-03*w16*Jinv10-1.8262987012987013e-05*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-6.9399350649350649e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-4.0584415584415584e-05*w11*Jinv22+1.5726461038961039e-04*Jinv11*w1-5.6615259740259744e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+3.1960227272727273e-04*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-1.0592532467532468e-03*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.5523538961038962e-04*w22*Jinv12-3.7439123376623376e-04*w31*Jinv10-3.9265422077922079e-04*Jinv02*w3+1.5523538961038962e-04*w21*Jinv00-3.7439123376623376e-04*w32*Jinv22-6.9399350649350649e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2-1.8262987012987013e-05*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00+4.2918019480519480e-04*Jinv02*w45+1.7349837662337662e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+1.3697240259740259e-04*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10+3.9265422077922079e-04*w4*Jinv11-9.6793831168831166e-04*w41*Jinv20-7.4878246753246752e-04*Jinv12*w13-1.8262987012987013e-05*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.7439123376623376e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20+4.8396915584415583e-04*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-9.6793831168831166e-04*Jinv22*w41+3.1047077922077924e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35+3.7439123376623376e-04*w31*Jinv11+3.1047077922077924e-04*Jinv22*w26+3.1960227272727273e-04*w46*Jinv11-7.4878246753246752e-04*Jinv20*w14+3.9265422077922079e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54-3.9265422077922079e-04*w4*Jinv12+1.5523538961038962e-04*w21*Jinv02+3.1960227272727273e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-9.6793831168831166e-04*w39*Jinv00-7.4878246753246752e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01+4.8396915584415583e-04*w7*Jinv10+3.1960227272727273e-04*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38+1.5523538961038962e-04*Jinv22*w23-3.7439123376623376e-04*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53+4.8396915584415583e-04*Jinv22*w8+1.5523538961038962e-04*w22*Jinv10+3.1960227272727273e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+4.8396915584415583e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-3.7439123376623376e-04*Jinv02*w30+3.1960227272727273e-04*Jinv12*w49+1.3697240259740259e-04*w52*Jinv10+9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+3.1047077922077924e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+4.8396915584415583e-04*w6*Jinv02-1.8262987012987013e-05*w19*Jinv12-3.9265422077922079e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-4.0584415584415584e-05*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-1.8262987012987013e-05*Jinv02*w18+9.1314935064935065e-06*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+3.1047077922077924e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-1.0592532467532468e-03*w15*Jinv00+3.1960227272727273e-04*w48*Jinv02-1.0592532467532468e-03*Jinv20*w17-3.7439123376623376e-04*w30*Jinv00-9.6793831168831166e-04*w40*Jinv10-1.8262987012987013e-05*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-7.4878246753246752e-04*Jinv22*w14-6.9399350649350649e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20+3.7439123376623376e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+1.7349837662337662e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-1.0592532467532468e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1)*D;
  A[9][13]=(0.0)*D;
  A[9][14]=(0.0)*D;
  A[9][15]=(6.2094155844155847e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10-1.1323051948051949e-03*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+6.2094155844155847e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+3.1047077922077924e-04*w32*Jinv20+4.3831168831168834e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38-3.4699675324675324e-04*Jinv00*w54-6.5746753246753243e-04*w25*Jinv12-3.0438311688311688e-05*w0*Jinv02-4.2613636363636367e-05*Jinv21*w2+6.9399350649350649e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-5.2962662337662338e-04*Jinv01*w45+7.3051948051948052e-05*Jinv12*w55+1.9480519480519481e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02-7.3051948051948055e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-6.5746753246753243e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-4.3831168831168834e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01+1.8262987012987013e-03*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00-4.3831168831168834e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27-7.3051948051948055e-04*w11*Jinv22-4.2613636363636367e-05*Jinv11*w1+5.6615259740259744e-04*w13*Jinv11+4.2613636363636367e-05*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+9.1314935064935063e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+3.1047077922077924e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22-3.0438311688311688e-05*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-4.9310064935064932e-04*Jinv02*w45-4.3831168831168834e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-2.5568181818181820e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-4.3831168831168834e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.3831168831168834e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+7.3051948051948052e-05*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10+9.1314935064935063e-04*w34*Jinv10+5.2962662337662338e-04*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12-3.4699675324675324e-04*Jinv20*w56-1.1323051948051949e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-2.4350649350649351e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+5.2962662337662338e-04*w31*Jinv11-6.5746753246753243e-04*Jinv22*w26-5.2962662337662338e-04*w46*Jinv11+6.2094155844155847e-04*Jinv20*w14+4.3831168831168834e-04*w5*Jinv21+7.3051948051948052e-05*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+7.3051948051948052e-05*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18-7.3051948051948055e-04*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10-5.2962662337662338e-04*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-4.9310064935064932e-04*w46*Jinv12-2.5568181818181820e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-5.6615259740259744e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-5.6615259740259744e-04*w21*Jinv01+3.6525974025974026e-05*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49-2.5568181818181820e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11-2.4350649350649351e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+3.6525974025974026e-05*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+1.2175324675324675e-05*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5-4.9310064935064932e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+5.6615259740259744e-04*Jinv21*w14-2.4350649350649351e-04*w10*Jinv10+6.9399350649350649e-04*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18+5.6615259740259744e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+1.9480519480519481e-04*w57*Jinv00+1.9480519480519481e-04*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33+9.1314935064935063e-04*w35*Jinv20-5.6615259740259744e-04*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10+1.8262987012987013e-03*w15*Jinv00+3.6525974025974026e-05*w48*Jinv02+1.8262987012987013e-03*Jinv20*w17+3.1047077922077924e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20+1.2175324675324675e-05*Jinv22*w29+7.3051948051948052e-05*Jinv22*w14+5.4788961038961036e-04*Jinv02*w36-4.3831168831168834e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+5.2962662337662338e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51+7.3051948051948052e-05*Jinv22*w56+1.2175324675324675e-05*Jinv12*w28-4.3831168831168834e-04*w25*Jinv11-3.4699675324675324e-04*w55*Jinv10+6.9399350649350649e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-3.0438311688311688e-05*Jinv12*w1)*D;
  A[9][16]=(0.0)*D;
  A[9][17]=(0.0)*D;
  A[9][18]=(1.2784090909090910e-04*Jinv00*w12-5.6818181818181818e-05*w28*Jinv10-4.5657467532467534e-05*Jinv20*w8+1.2784090909090910e-04*w13*Jinv10-5.4788961038961036e-04*Jinv21*w44+1.1870941558441559e-04*w3*Jinv00+6.3920454545454550e-05*w32*Jinv20-5.4788961038961036e-04*Jinv01*w3-2.1002435064935065e-04*Jinv20*w23+1.2784090909090910e-04*Jinv00*w54+6.4935064935064935e-05*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.5827922077922079e-03*w11*Jinv21-5.4788961038961036e-04*w16*Jinv11-5.4788961038961036e-04*Jinv21*w17+9.1314935064935065e-06*Jinv01*w45-9.2329545454545453e-05*w58*Jinv10+6.3920454545454550e-05*w48*Jinv00+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01-9.2329545454545453e-05*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-5.4788961038961036e-04*w43*Jinv11-2.1002435064935065e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41-5.6818181818181818e-05*Jinv00*w27+6.4935064935064935e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-5.4788961038961036e-04*w15*Jinv01+6.4935064935064935e-05*w28*Jinv11+6.3920454545454550e-05*Jinv20*w50-5.6818181818181818e-05*Jinv20*w29-2.1002435064935065e-04*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+6.3920454545454550e-05*w31*Jinv10-2.1002435064935065e-04*w21*Jinv00-4.9310064935064932e-04*Jinv01*w39-9.1314935064935068e-05*Jinv00*w18-4.5657467532467534e-05*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38-5.4788961038961036e-04*Jinv01*w24-5.4788961038961036e-04*w34*Jinv11+1.0957792207792207e-03*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53+1.1870941558441559e-04*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10-5.4788961038961036e-04*w4*Jinv11-6.7573051948051951e-04*w41*Jinv20-9.1314935064935068e-05*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21+1.0957792207792207e-03*Jinv21*w20-4.5657467532467534e-05*w6*Jinv00+1.2784090909090910e-04*Jinv20*w56-1.4610389610389610e-04*w26*Jinv20+2.8409090909090908e-04*w11*Jinv20-5.4788961038961036e-04*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11+9.1314935064935065e-06*w46*Jinv11+1.2784090909090910e-04*Jinv20*w14-5.4788961038961036e-04*w5*Jinv21+6.3920454545454550e-05*w49*Jinv10+6.4935064935064935e-05*Jinv21*w29-5.4788961038961036e-04*Jinv01*w33-6.7573051948051951e-04*w39*Jinv00+1.0957792207792207e-03*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+1.0957792207792207e-03*w6*Jinv01-4.5657467532467534e-05*w7*Jinv10+9.1314935064935065e-06*w47*Jinv21-4.5657467532467534e-05*Jinv20*w38-5.4788961038961036e-04*w52*Jinv11+1.1870941558441559e-04*Jinv20*w53-2.1002435064935065e-04*w22*Jinv10+9.1314935064935065e-06*w22*Jinv11+9.1314935064935065e-06*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+1.0957792207792207e-03*w36*Jinv01+1.1870941558441559e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+2.8409090909090908e-04*w9*Jinv00-4.5657467532467534e-05*w37*Jinv10+9.1314935064935065e-06*Jinv21*w56-1.4610389610389610e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14+2.8409090909090908e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+9.1314935064935065e-06*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-9.2329545454545453e-05*w57*Jinv00-9.2329545454545453e-05*Jinv20*w59-9.2329545454545453e-05*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+9.1314935064935065e-06*Jinv21*w23+9.1314935064935065e-06*Jinv01*w54+9.1314935064935065e-06*w55*Jinv11-1.4610389610389610e-04*w25*Jinv10-9.2329545454545453e-05*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00+1.5827922077922079e-03*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17+6.3920454545454550e-05*w30*Jinv00-6.7573051948051951e-04*w40*Jinv10-9.1314935064935068e-05*w20*Jinv20-4.9310064935064932e-04*w40*Jinv11+1.1870941558441559e-04*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30-5.4788961038961036e-04*w25*Jinv11+1.2784090909090910e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-2.1002435064935065e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50+6.4935064935064935e-05*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+1.5827922077922079e-03*w10*Jinv11)*D;
  A[9][19]=(0.0)*D;
  A[9][20]=(0.0)*D;
  A[9][21]=(4.2918019480519480e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.7394480519480518e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+4.2918019480519480e-04*w13*Jinv10-2.7394480519480518e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20-1.7349837662337662e-04*Jinv01*w3-7.4878246753246752e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-7.8530844155844158e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.1668019480519480e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01+5.6615259740259744e-04*w16*Jinv11+5.6615259740259744e-04*Jinv21*w17-3.7439123376623376e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-4.0584415584415584e-05*w10*Jinv12+1.0957792207792208e-04*w48*Jinv00-7.8530844155844158e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-3.9265422077922079e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10+4.7483766233766234e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00+6.9399350649350649e-04*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-4.0584415584415584e-05*w11*Jinv22-1.1668019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11+5.6615259740259744e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+1.0957792207792208e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.4878246753246752e-04*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+1.3697240259740259e-04*Jinv02*w3-7.4878246753246752e-04*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22+6.9399350649350649e-04*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2+4.7483766233766234e-04*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00-7.4878246753246752e-04*Jinv02*w45-3.9265422077922079e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+1.3697240259740259e-04*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10-1.7349837662337662e-04*w4*Jinv11-2.7394480519480518e-04*w41*Jinv20+4.2918019480519480e-04*Jinv12*w13+4.7483766233766234e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-3.1960227272727273e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-2.7394480519480518e-04*Jinv22*w41-7.8530844155844158e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35-3.1960227272727273e-04*w31*Jinv11-7.8530844155844158e-04*Jinv22*w26-3.7439123376623376e-04*w46*Jinv11+4.2918019480519480e-04*Jinv20*w14-1.7349837662337662e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+1.3697240259740259e-04*w4*Jinv12-7.4878246753246752e-04*w21*Jinv02+1.0957792207792208e-04*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-2.7394480519480518e-04*w39*Jinv00+4.2918019480519480e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10-3.7439123376623376e-04*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38-7.4878246753246752e-04*Jinv22*w23+1.0957792207792208e-04*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53-9.1314935064935065e-06*Jinv22*w8-7.4878246753246752e-04*w22*Jinv10+1.0957792207792208e-04*Jinv22*w50-9.1314935064935065e-06*Jinv12*w7+1.0957792207792208e-04*Jinv02*w30+1.0957792207792208e-04*Jinv12*w49+1.3697240259740259e-04*w52*Jinv10+9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56-7.8530844155844158e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-9.1314935064935065e-06*w6*Jinv02+4.7483766233766234e-04*w19*Jinv12+1.3697240259740259e-04*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+4.7483766233766234e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11-7.8530844155844158e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+1.0957792207792208e-04*w48*Jinv02-4.9310064935064932e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-2.7394480519480518e-04*w40*Jinv10+4.7483766233766234e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.2918019480519480e-04*Jinv22*w14+6.9399350649350649e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20-3.1960227272727273e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.9265422077922079e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1)*D;
  A[9][22]=(0.0)*D;
  A[9][23]=(0.0)*D;
  A[9][24]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10+9.1314935064935065e-06*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+4.3831168831168834e-04*Jinv21*w44-1.3697240259740259e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20+2.1915584415584417e-04*Jinv01*w3+7.4878246753246752e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54-1.4204545454545455e-05*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+4.9310064935064932e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-6.0876623376623375e-04*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17+3.9265422077922079e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+4.3831168831168834e-04*w43*Jinv11+7.4878246753246752e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27-1.4204545454545455e-05*Jinv11*w1-1.7349837662337662e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01-5.6818181818181818e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.7483766233766234e-04*Jinv00*w18+9.1314935064935065e-06*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53-1.3697240259740259e-04*w51*Jinv00-1.3697240259740259e-04*w4*Jinv10+2.1915584415584417e-04*w4*Jinv11+2.7394480519480518e-04*w41*Jinv20-4.7483766233766234e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20+9.1314935064935065e-06*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+7.8530844155844158e-04*w26*Jinv20+4.0584415584415584e-05*w11*Jinv20+2.1915584415584417e-04*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11+3.9265422077922079e-04*w46*Jinv11-4.2918019480519480e-04*Jinv20*w14+2.1915584415584417e-04*w5*Jinv21-1.0957792207792208e-04*w49*Jinv10-5.6818181818181818e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+2.7394480519480518e-04*w39*Jinv00-1.0957792207792207e-03*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-5.4788961038961036e-04*w6*Jinv01+9.1314935064935065e-06*w7*Jinv10+3.9265422077922079e-04*w47*Jinv21+9.1314935064935065e-06*Jinv20*w38+2.1915584415584417e-04*w52*Jinv11-1.3697240259740259e-04*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+3.9265422077922079e-04*w22*Jinv11+3.9265422077922079e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51-5.4788961038961036e-04*w36*Jinv01-1.3697240259740259e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11+4.0584415584415584e-05*w9*Jinv00+9.1314935064935065e-06*w37*Jinv10-1.7349837662337662e-04*Jinv21*w56+7.8530844155844158e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11-1.7349837662337662e-04*Jinv21*w14+4.0584415584415584e-05*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-1.7349837662337662e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.9265422077922079e-04*Jinv21*w23-1.7349837662337662e-04*Jinv01*w54-1.7349837662337662e-04*w55*Jinv11+7.8530844155844158e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-6.0876623376623375e-04*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00+2.7394480519480518e-04*w40*Jinv10-4.7483766233766234e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11-1.3697240259740259e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11-4.2918019480519480e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50-5.6818181818181818e-05*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01-6.0876623376623375e-04*w10*Jinv11)*D;
  A[9][25]=(0.0)*D;
  A[9][26]=(0.0)*D;
  A[9][27]=(-4.2613636363636367e-05*Jinv21*w44-1.4204545454545455e-05*Jinv01*w3+5.4112554112554113e-05*Jinv21*w59+5.4112554112554113e-05*Jinv21*w2-4.2613636363636367e-05*w42*Jinv01+1.8939393939393939e-04*w11*Jinv21-4.2613636363636367e-05*w16*Jinv11-4.2613636363636367e-05*Jinv21*w17+1.5726461038961039e-04*Jinv01*w45+5.4112554112554113e-05*w58*Jinv11+5.4112554112554113e-05*w57*Jinv01+2.0495129870129870e-04*Jinv21*w26+5.4112554112554113e-05*w0*Jinv01-4.2613636363636367e-05*w43*Jinv11-1.9480519480519481e-04*Jinv21*w41+5.4112554112554113e-05*Jinv11*w1-1.1668019480519480e-04*w13*Jinv11-4.2613636363636367e-05*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-1.9480519480519481e-04*Jinv01*w39+6.4935064935064935e-05*Jinv21*w38+2.0495129870129870e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11+8.3198051948051951e-05*w19*Jinv11-1.4204545454545455e-05*Jinv21*w53-1.4204545454545455e-05*w4*Jinv11+1.0247564935064935e-04*w32*Jinv21+8.3198051948051951e-05*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35+1.0247564935064935e-04*w31*Jinv11+1.5726461038961039e-04*w46*Jinv11-1.4204545454545455e-05*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33+8.3198051948051951e-05*Jinv01*w18+6.4935064935064935e-05*w37*Jinv11+6.4935064935064935e-05*w6*Jinv01+1.5726461038961039e-04*w47*Jinv21-1.4204545454545455e-05*w52*Jinv11+1.5726461038961039e-04*w22*Jinv11+1.5726461038961039e-04*w21*Jinv01-1.4204545454545455e-05*Jinv01*w51+6.4935064935064935e-05*w36*Jinv01+6.4935064935064935e-05*w7*Jinv11-1.1668019480519480e-04*Jinv21*w56+1.0247564935064935e-04*w49*Jinv11-1.1668019480519480e-04*Jinv21*w14-1.1668019480519480e-04*Jinv01*w12+6.4935064935064935e-05*Jinv21*w8+1.5726461038961039e-04*Jinv21*w23-1.1668019480519480e-04*Jinv01*w54-1.1668019480519480e-04*w55*Jinv11+1.8939393939393939e-04*Jinv01*w9-1.9480519480519481e-04*w40*Jinv11+1.0247564935064935e-04*Jinv01*w30+2.0495129870129870e-04*w25*Jinv11+1.0247564935064935e-04*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27+1.0247564935064935e-04*w48*Jinv01+1.8939393939393939e-04*w10*Jinv11)*D;
  A[9][28]=(0.0)*D;
  A[9][29]=(0.0)*D;
  A[9][30]=(-3.7439123376623376e-04*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-6.9399350649350649e-04*w40*Jinv12+1.7349837662337662e-04*Jinv22*w53+4.8396915584415583e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02-3.7439123376623376e-04*w13*Jinv10-6.9399350649350649e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-7.4878246753246752e-04*w32*Jinv20-3.9265422077922079e-04*Jinv01*w3+3.1960227272727273e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+1.5726461038961039e-04*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01-4.0584415584415584e-05*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45+3.1960227272727273e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02+1.5523538961038962e-04*w48*Jinv00-5.6615259740259744e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+7.3051948051948052e-05*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-9.6793831168831166e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27+9.2329545454545453e-05*Jinv11*w1-3.7439123376623376e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+1.5523538961038962e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-1.0592532467532468e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-7.4878246753246752e-04*w31*Jinv10+3.9265422077922079e-04*Jinv02*w3+3.1960227272727273e-04*w21*Jinv00-9.6793831168831166e-04*Jinv01*w39+1.5726461038961039e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18-1.8262987012987013e-05*w36*Jinv00-1.8262987012987013e-05*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-1.0592532467532468e-03*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+3.1047077922077924e-04*Jinv21*w53+3.1047077922077924e-04*w51*Jinv00-5.6615259740259744e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10-3.9265422077922079e-04*w4*Jinv11-9.6793831168831166e-04*w41*Jinv20+3.7439123376623376e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10-1.0592532467532468e-03*w34*Jinv10-7.4878246753246752e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20+4.8396915584415583e-04*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-6.9399350649350649e-04*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.7349837662337662e-04*w52*Jinv12-1.0592532467532468e-03*Jinv21*w35-7.4878246753246752e-04*w31*Jinv11+2.1915584415584417e-04*w46*Jinv11-3.7439123376623376e-04*Jinv20*w14-3.9265422077922079e-04*w5*Jinv21+3.1960227272727273e-04*Jinv02*w54+3.9265422077922079e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+1.5523538961038962e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-1.0592532467532468e-03*Jinv01*w33-9.6793831168831166e-04*w39*Jinv00+3.7439123376623376e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-1.8262987012987013e-05*w37*Jinv11+4.8396915584415583e-04*w6*Jinv01+4.8396915584415583e-04*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-1.8262987012987013e-05*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23+3.1047077922077924e-04*w52*Jinv11+1.0957792207792208e-04*w46*Jinv12+3.1047077922077924e-04*Jinv20*w53+9.1314935064935065e-06*Jinv22*w8+3.1960227272727273e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.1960227272727273e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+3.1960227272727273e-04*w21*Jinv01+3.1047077922077924e-04*Jinv01*w51-2.7394480519480518e-04*Jinv12*w49-1.8262987012987013e-05*w36*Jinv01+3.1047077922077924e-04*w52*Jinv10+4.8396915584415583e-04*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-1.8262987012987013e-05*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+9.1314935064935065e-06*w6*Jinv02+3.9265422077922079e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.5523538961038962e-04*w49*Jinv11-3.7439123376623376e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.7439123376623376e-04*Jinv01*w12+4.8396915584415583e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-5.6615259740259744e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-1.0592532467532468e-03*w35*Jinv20+3.1960227272727273e-04*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-4.0584415584415584e-05*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-7.4878246753246752e-04*w30*Jinv00-9.6793831168831166e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20+3.7439123376623376e-04*Jinv22*w14-9.6793831168831166e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20-7.4878246753246752e-04*Jinv01*w30+1.7349837662337662e-04*Jinv02*w51+3.1960227272727273e-04*Jinv22*w56+1.3697240259740259e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12+1.5523538961038962e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+1.5523538961038962e-04*w48*Jinv01+1.5726461038961039e-04*Jinv12*w1-4.0584415584415584e-05*w10*Jinv11)*D;
  A[9][31]=(0.0)*D;
  A[9][32]=(0.0)*D;
  A[9][33]=(3.1047077922077924e-04*Jinv00*w12+1.9480519480519481e-04*w28*Jinv10-4.3831168831168834e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02+3.1047077922077924e-04*w13*Jinv10-4.3831168831168834e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+6.2094155844155847e-04*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+1.2175324675324675e-05*Jinv21*w59-4.2613636363636367e-05*w0*Jinv02-3.0438311688311688e-05*Jinv21*w2+6.9399350649350649e-04*w43*Jinv10-4.3831168831168834e-04*w42*Jinv01-7.3051948051948055e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17+7.3051948051948052e-05*Jinv01*w45-5.2962662337662338e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02+5.4788961038961042e-05*w48*Jinv00+1.2175324675324675e-05*w58*Jinv11+1.2175324675324675e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26-3.0438311688311688e-05*w0*Jinv01+9.1314935064935063e-04*w16*Jinv10-4.3831168831168834e-04*w43*Jinv11-3.4699675324675324e-04*w45*Jinv00-1.1323051948051949e-03*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.9480519480519481e-04*Jinv00*w27-3.0438311688311688e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+1.9480519480519481e-04*Jinv20*w29-3.4699675324675324e-04*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+6.2094155844155847e-04*w31*Jinv10+4.3831168831168834e-04*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00+5.6615259740259744e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39-4.2613636363636367e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38+3.6525974025974026e-05*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-6.5746753246753243e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00-2.1915584415584417e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+5.2962662337662338e-04*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10+7.3051948051948052e-05*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56-2.5568181818181820e-04*w26*Jinv20-2.4350649350649351e-04*w11*Jinv20-4.3831168831168834e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+7.3051948051948052e-05*w31*Jinv11+7.3051948051948052e-05*w46*Jinv11+3.1047077922077924e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21-5.2962662337662338e-04*Jinv02*w54+4.3831168831168834e-04*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+5.2962662337662338e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10+7.3051948051948052e-05*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23-6.5746753246753243e-04*w52*Jinv11+5.6615259740259744e-04*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-5.6615259740259744e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01-6.5746753246753243e-04*Jinv01*w51+5.6615259740259744e-04*Jinv02*w30-5.6615259740259744e-04*Jinv12*w49+1.0957792207792207e-03*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11-2.4350649350649351e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10-4.9310064935064932e-04*Jinv21*w56-2.5568181818181820e-04*w24*Jinv00-5.4788961038961036e-04*w6*Jinv02+4.3831168831168834e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14-2.4350649350649351e-04*w10*Jinv10+6.9399350649350649e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12+3.6525974025974026e-05*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+1.8262987012987013e-03*w35*Jinv20+3.6525974025974026e-05*Jinv21*w23-4.9310064935064932e-04*Jinv01*w54-4.9310064935064932e-04*w55*Jinv11-2.5568181818181820e-04*w25*Jinv10+9.1314935064935063e-04*w15*Jinv00-5.6615259740259744e-04*w48*Jinv02-7.3051948051948055e-04*Jinv01*w9+9.1314935064935063e-04*Jinv20*w17+6.2094155844155847e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+7.3051948051948052e-05*Jinv01*w30-4.3831168831168834e-04*Jinv02*w51-5.2962662337662338e-04*Jinv22*w56-2.1915584415584417e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+6.9399350649350649e-04*w42*Jinv00-3.4699675324675324e-04*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-4.2613636363636367e-05*Jinv12*w1-7.3051948051948055e-04*w10*Jinv11)*D;
  A[9][34]=(0.0)*D;
  A[9][35]=(0.0)*D;
  A[9][36]=(6.3920454545454550e-05*Jinv00*w12-9.2329545454545453e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-4.5657467532467534e-05*Jinv20*w8+6.4935064935064935e-05*w57*Jinv02+6.3920454545454550e-05*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+1.1870941558441559e-04*w3*Jinv00+1.2784090909090910e-04*w32*Jinv20+6.3920454545454550e-05*Jinv20*w23+1.0957792207792207e-03*Jinv22*w38-2.1002435064935065e-04*Jinv00*w54-5.4788961038961036e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10+9.1314935064935065e-06*Jinv12*w55-5.6818181818181818e-05*w58*Jinv10-5.4788961038961036e-04*w15*Jinv02+1.5827922077922079e-03*w10*Jinv12-2.1002435064935065e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-5.4788961038961036e-04*Jinv22*w35-9.2329545454545453e-05*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20+1.2784090909090910e-04*w45*Jinv00-5.4788961038961036e-04*w42*Jinv02-9.2329545454545453e-05*Jinv00*w27+1.5827922077922079e-03*w11*Jinv22-2.1002435064935065e-04*Jinv20*w50-9.2329545454545453e-05*Jinv20*w29+1.2784090909090910e-04*w46*Jinv10-5.4788961038961036e-04*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12+1.2784090909090910e-04*w31*Jinv10-5.4788961038961036e-04*Jinv02*w3+6.3920454545454550e-05*w21*Jinv00+9.1314935064935065e-06*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2-4.5657467532467534e-05*Jinv00*w18-9.1314935064935068e-05*w36*Jinv00+9.1314935064935065e-06*Jinv02*w45+6.4935064935064935e-05*Jinv22*w59-1.4610389610389610e-04*w51*Jinv00-5.4788961038961036e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.1870941558441559e-04*w4*Jinv10-6.7573051948051951e-04*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13-4.5657467532467534e-05*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10-4.5657467532467534e-05*w6*Jinv00+1.0957792207792207e-03*w37*Jinv12-2.1002435064935065e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41+1.1870941558441559e-04*w26*Jinv20+2.8409090909090908e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-5.4788961038961036e-04*Jinv22*w26+6.3920454545454550e-05*Jinv20*w14+9.1314935064935065e-06*Jinv02*w54-5.4788961038961036e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-2.1002435064935065e-04*w49*Jinv10-6.7573051948051951e-04*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12+1.5827922077922079e-03*Jinv02*w9-4.5657467532467534e-05*w7*Jinv10-9.1314935064935068e-05*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23+9.1314935064935065e-06*w31*Jinv12+9.1314935064935065e-06*w46*Jinv12-1.4610389610389610e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8+6.3920454545454550e-05*w22*Jinv10+9.1314935064935065e-06*Jinv22*w50+1.0957792207792207e-03*Jinv12*w7+9.1314935064935065e-06*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49-1.4610389610389610e-04*w52*Jinv10+2.8409090909090908e-04*w9*Jinv00-9.1314935064935068e-05*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+6.4935064935064935e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-5.4788961038961036e-04*Jinv22*w5+9.1314935064935065e-06*w47*Jinv22+2.8409090909090908e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+6.4935064935064935e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18-5.6818181818181818e-05*w57*Jinv00-5.6818181818181818e-05*Jinv20*w59-5.4788961038961036e-04*Jinv02*w33-9.2329545454545453e-05*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10-9.2329545454545453e-05*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00+9.1314935064935065e-06*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.2784090909090910e-04*w30*Jinv00-6.7573051948051951e-04*w40*Jinv10-4.5657467532467534e-05*w20*Jinv20+6.4935064935064935e-05*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14+1.0957792207792207e-03*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.1870941558441559e-04*w5*Jinv20-5.4788961038961036e-04*Jinv02*w51+9.1314935064935065e-06*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28-2.1002435064935065e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00+1.2784090909090910e-04*w47*Jinv20-5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[9][37]=(0.0)*D;
  A[9][38]=(0.0)*D;
  A[9][39]=(4.2004870129870131e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10-5.2962662337662338e-04*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02+4.2004870129870131e-04*w13*Jinv10-1.1323051948051949e-03*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00+4.2004870129870131e-04*w32*Jinv20+5.2962662337662338e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-1.9480519480519481e-04*w0*Jinv02-1.9480519480519481e-04*Jinv21*w2-2.2646103896103898e-03*w43*Jinv10-1.1323051948051949e-03*w42*Jinv01-9.6793831168831166e-04*Jinv01*w45-9.6793831168831166e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10+1.1323051948051949e-03*w15*Jinv02-2.7394480519480518e-04*w48*Jinv00-5.2962662337662338e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-2.2524350649350650e-04*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10-1.1323051948051949e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-1.1323051948051949e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-1.9480519480519481e-04*Jinv11*w1+6.9399350649350649e-04*w13*Jinv11+1.9480519480519481e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+1.1323051948051949e-03*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+4.2004870129870131e-04*w31*Jinv10+5.2962662337662338e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00+6.9399350649350649e-04*w32*Jinv22-1.9480519480519481e-04*Jinv22*w2+9.8620129870129864e-04*Jinv00*w18+9.8620129870129864e-04*w36*Jinv00-2.7394480519480518e-04*Jinv02*w45-5.2962662337662338e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59+1.1323051948051949e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-4.9310064935064932e-04*w51*Jinv00-1.1323051948051949e-03*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+5.2962662337662338e-04*w4*Jinv11-2.7759740259740259e-03*w41*Jinv20+9.6793831168831166e-04*Jinv12*w13+9.8620129870129864e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10+9.6793831168831166e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-4.9310064935064932e-04*w26*Jinv20-4.8701298701298701e-04*w11*Jinv20-5.2962662337662338e-04*w52*Jinv12+1.1323051948051949e-03*Jinv21*w35+9.6793831168831166e-04*w31*Jinv11-9.6793831168831166e-04*w46*Jinv11+4.2004870129870131e-04*Jinv20*w14+5.2962662337662338e-04*w5*Jinv21-9.6793831168831166e-04*Jinv02*w54+5.2962662337662338e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+1.1323051948051949e-03*Jinv01*w33-2.7759740259740259e-03*w39*Jinv00+9.6793831168831166e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18-4.9310064935064932e-04*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10-9.6793831168831166e-04*w47*Jinv21+9.8620129870129864e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+6.9399350649350649e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-4.9310064935064932e-04*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10-6.9399350649350649e-04*Jinv22*w50-6.9399350649350649e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-6.9399350649350649e-04*w21*Jinv01+6.9399350649350649e-04*Jinv02*w30-6.9399350649350649e-04*Jinv12*w49-4.9310064935064932e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-4.8701298701298701e-04*w9*Jinv00+9.8620129870129864e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-4.9310064935064932e-04*w24*Jinv00-4.9310064935064932e-04*w6*Jinv02+5.2962662337662338e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11+6.9399350649350649e-04*Jinv21*w14-4.8701298701298701e-04*w10*Jinv10-2.2646103896103898e-03*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12+6.9399350649350649e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59-2.2524350649350650e-04*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-6.9399350649350649e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-4.9310064935064932e-04*w25*Jinv10-2.2524350649350650e-04*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-6.9399350649350649e-04*w48*Jinv02-1.1323051948051949e-03*Jinv20*w17+4.2004870129870131e-04*w30*Jinv00-2.7759740259740259e-03*w40*Jinv10+9.8620129870129864e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36-1.1323051948051949e-03*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+9.6793831168831166e-04*Jinv01*w30-5.2962662337662338e-04*Jinv02*w51-9.6793831168831166e-04*Jinv22*w56-5.2962662337662338e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-2.2646103896103898e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+1.1323051948051949e-03*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-1.9480519480519481e-04*Jinv12*w1)*D;
  A[9][40]=(0.0)*D;
  A[9][41]=(0.0)*D;
  A[9][42]=(-4.2004870129870131e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8+3.0438311688311688e-05*w57*Jinv02-4.2004870129870131e-04*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00-4.2004870129870131e-04*w32*Jinv20+2.1915584415584417e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54-1.2175324675324675e-05*Jinv21*w59+6.5746753246753243e-04*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02+3.0438311688311688e-05*Jinv21*w2+2.2646103896103898e-03*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01+7.3051948051948055e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-7.3051948051948052e-05*Jinv01*w45-7.3051948051948052e-05*Jinv12*w55+3.0438311688311688e-05*w58*Jinv10+4.3831168831168834e-04*w15*Jinv02+7.3051948051948055e-04*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+6.5746753246753243e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35-1.2175324675324675e-05*w58*Jinv11-1.2175324675324675e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01+2.2524350649350650e-04*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+3.0438311688311688e-05*Jinv00*w27+7.3051948051948055e-04*w11*Jinv22+3.0438311688311688e-05*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+3.0438311688311688e-05*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10+4.3831168831168834e-04*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33+4.9310064935064932e-04*w22*Jinv12-4.2004870129870131e-04*w31*Jinv10+2.1915584415584417e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+3.0438311688311688e-05*Jinv22*w2-9.8620129870129864e-04*Jinv00*w18-9.8620129870129864e-04*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24+3.0438311688311688e-05*Jinv22*w59+4.3831168831168834e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+6.5746753246753243e-04*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+2.1915584415584417e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10+2.1915584415584417e-04*w4*Jinv11+2.7759740259740259e-03*w41*Jinv20-7.3051948051948052e-05*Jinv12*w13-9.8620129870129864e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-7.3051948051948052e-05*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+4.8701298701298701e-04*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12+4.3831168831168834e-04*Jinv21*w35-7.3051948051948052e-05*w31*Jinv11+6.5746753246753243e-04*Jinv22*w26-7.3051948051948052e-05*w46*Jinv11-4.2004870129870131e-04*Jinv20*w14+2.1915584415584417e-04*w5*Jinv21-7.3051948051948052e-05*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12+4.9310064935064932e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10+3.0438311688311688e-05*Jinv21*w29+4.3831168831168834e-04*Jinv01*w33+2.7759740259740259e-03*w39*Jinv00-7.3051948051948052e-05*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11+7.3051948051948055e-04*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-7.3051948051948052e-05*w47*Jinv21-9.8620129870129864e-04*Jinv20*w38+4.9310064935064932e-04*Jinv22*w23+6.5746753246753243e-04*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+6.5746753246753243e-04*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11+4.8701298701298701e-04*w9*Jinv00-9.8620129870129864e-04*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00-1.2175324675324675e-05*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22+4.9310064935064932e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+4.8701298701298701e-04*w10*Jinv10+2.2646103896103898e-03*Jinv20*w44+3.0438311688311688e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+2.1915584415584417e-04*Jinv02*w33+2.2524350649350650e-04*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10+2.2524350649350650e-04*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+7.3051948051948055e-04*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17-4.2004870129870131e-04*w30*Jinv00+2.7759740259740259e-03*w40*Jinv10-9.8620129870129864e-04*w20*Jinv20-1.2175324675324675e-05*Jinv22*w29-7.3051948051948052e-05*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20-7.3051948051948052e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51-7.3051948051948052e-05*Jinv22*w56-1.2175324675324675e-05*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+2.2646103896103898e-03*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+4.3831168831168834e-04*w16*Jinv12+4.9310064935064932e-04*Jinv21*w50+3.0438311688311688e-05*Jinv01*w27+4.9310064935064932e-04*w48*Jinv01+3.0438311688311688e-05*Jinv12*w1+7.3051948051948055e-04*w10*Jinv11)*D;
  A[9][43]=(0.0)*D;
  A[9][44]=(0.0)*D;
  A[9][45]=(2.7394480519480518e-04*w40*Jinv12-1.3697240259740259e-04*Jinv22*w53+1.0247564935064935e-04*w57*Jinv02+2.7394480519480518e-04*Jinv02*w39+1.0592532467532468e-03*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3+9.1314935064935065e-06*Jinv22*w38-1.4204545454545455e-05*Jinv21*w59+7.8530844155844158e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+1.0592532467532468e-03*w42*Jinv01+4.0584415584415584e-05*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+7.4878246753246752e-04*Jinv01*w45-4.2918019480519480e-04*Jinv12*w55+4.9310064935064932e-04*w15*Jinv02+4.0584415584415584e-05*w10*Jinv12+7.8530844155844158e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+3.9265422077922079e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01-4.7483766233766234e-04*Jinv22*w20+1.0592532467532468e-03*w43*Jinv11+9.6793831168831166e-04*Jinv21*w41+4.9310064935064932e-04*w42*Jinv02+4.0584415584415584e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-3.1960227272727273e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+4.9310064935064932e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-1.0957792207792208e-04*w32*Jinv22+9.6793831168831166e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2+1.8262987012987013e-05*Jinv21*w38+7.4878246753246752e-04*Jinv02*w45+3.9265422077922079e-04*Jinv01*w24+1.0247564935064935e-04*Jinv22*w59-7.3051948051948052e-05*w34*Jinv11-4.8396915584415583e-04*w19*Jinv11-3.1047077922077924e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-4.2918019480519480e-04*Jinv12*w13-2.1915584415584417e-04*w32*Jinv21-4.8396915584415583e-04*Jinv21*w20+9.1314935064935065e-06*w37*Jinv12+2.7394480519480518e-04*Jinv22*w41-1.3697240259740259e-04*w52*Jinv12-7.3051948051948052e-05*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11+7.8530844155844158e-04*Jinv22*w26+7.4878246753246752e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21-4.2918019480519480e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33-4.2918019480519480e-04*Jinv02*w12-4.8396915584415583e-04*Jinv01*w18+1.8262987012987013e-05*w37*Jinv11+4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01+7.4878246753246752e-04*w47*Jinv21+7.4878246753246752e-04*Jinv22*w23-3.1047077922077924e-04*w52*Jinv11-1.0957792207792208e-04*w31*Jinv12+7.4878246753246752e-04*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50+3.7439123376623376e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+3.7439123376623376e-04*w21*Jinv01-3.1047077922077924e-04*Jinv01*w51-1.0957792207792208e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+1.8262987012987013e-05*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11-1.5523538961038962e-04*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02-4.7483766233766234e-04*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5+7.4878246753246752e-04*w47*Jinv22-4.2918019480519480e-04*w49*Jinv11-3.1960227272727273e-04*Jinv21*w14+1.0247564935064935e-04*w58*Jinv12-4.7483766233766234e-04*Jinv02*w18-3.1960227272727273e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.7439123376623376e-04*Jinv21*w23-1.5523538961038962e-04*Jinv01*w54-1.5523538961038962e-04*w55*Jinv11-1.0957792207792208e-04*w48*Jinv02+4.0584415584415584e-05*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+9.6793831168831166e-04*w40*Jinv11+9.1314935064935065e-06*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-2.1915584415584417e-04*Jinv01*w30-1.3697240259740259e-04*Jinv02*w51-4.2918019480519480e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.9265422077922079e-04*w25*Jinv11+4.9310064935064932e-04*w16*Jinv12-4.2918019480519480e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1+4.0584415584415584e-05*w10*Jinv11)*D;
  A[9][46]=(0.0)*D;
  A[9][47]=(0.0)*D;
  A[9][48]=(1.0957792207792208e-04*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10+6.9399350649350649e-04*w40*Jinv12-3.9265422077922079e-04*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10+6.9399350649350649e-04*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+4.2918019480519480e-04*w32*Jinv20+1.3697240259740259e-04*Jinv01*w3+1.0957792207792208e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-1.1668019480519480e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-4.0584415584415584e-05*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-3.7439123376623376e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02-7.4878246753246752e-04*w48*Jinv00+5.6615259740259744e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-4.9310064935064932e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-2.7394480519480518e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-1.0247564935064935e-04*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-7.4878246753246752e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+4.2918019480519480e-04*w31*Jinv10-1.7349837662337662e-04*Jinv02*w3+1.0957792207792208e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-2.7394480519480518e-04*Jinv01*w39-1.1668019480519480e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18+4.7483766233766234e-04*w36*Jinv00+4.7483766233766234e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11-7.8530844155844158e-04*Jinv21*w53-7.8530844155844158e-04*w51*Jinv00+5.6615259740259744e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10+1.3697240259740259e-04*w4*Jinv11-2.7394480519480518e-04*w41*Jinv20-3.1960227272727273e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+4.2918019480519480e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56+6.9399350649350649e-04*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20-3.9265422077922079e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+4.2918019480519480e-04*w31*Jinv11+4.2918019480519480e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+1.3697240259740259e-04*w5*Jinv21-3.7439123376623376e-04*Jinv02*w54-1.7349837662337662e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-7.4878246753246752e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-2.7394480519480518e-04*w39*Jinv00-3.1960227272727273e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18+4.7483766233766234e-04*w37*Jinv11-9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+4.7483766233766234e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-7.8530844155844158e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12-7.8530844155844158e-04*Jinv20*w53+9.1314935064935065e-06*Jinv22*w8+1.0957792207792208e-04*w22*Jinv10+1.0957792207792208e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+1.0957792207792208e-04*w21*Jinv01-7.8530844155844158e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+4.7483766233766234e-04*w36*Jinv01-7.8530844155844158e-04*w52*Jinv10-9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00+4.7483766233766234e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+9.1314935064935065e-06*w6*Jinv02-1.7349837662337662e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22-7.4878246753246752e-04*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.0957792207792208e-04*Jinv01*w12-9.1314935064935065e-06*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+5.6615259740259744e-04*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+1.0957792207792208e-04*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-4.0584415584415584e-05*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.2918019480519480e-04*w30*Jinv00-2.7394480519480518e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20-3.1960227272727273e-04*Jinv22*w14-2.7394480519480518e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20+4.2918019480519480e-04*Jinv01*w30-3.9265422077922079e-04*Jinv02*w51-3.7439123376623376e-04*Jinv22*w56+1.3697240259740259e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12-7.4878246753246752e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-7.4878246753246752e-04*w48*Jinv01-1.1668019480519480e-04*Jinv12*w1-4.0584415584415584e-05*w10*Jinv11)*D;
  A[9][49]=(0.0)*D;
  A[9][50]=(0.0)*D;
  A[9][51]=(-1.0957792207792208e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+9.1314935064935065e-06*Jinv20*w8-5.6818181818181818e-05*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39-1.3697240259740259e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+7.4878246753246752e-04*Jinv00*w54+2.1915584415584417e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02+4.9310064935064932e-04*w43*Jinv10+3.9265422077922079e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02-6.0876623376623375e-04*w10*Jinv12+7.4878246753246752e-04*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-4.2918019480519480e-04*w45*Jinv00+4.3831168831168834e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27-6.0876623376623375e-04*w11*Jinv22+7.4878246753246752e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-4.2918019480519480e-04*w31*Jinv10+2.1915584415584417e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-1.7349837662337662e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2+9.1314935064935065e-06*Jinv00*w18-4.7483766233766234e-04*w36*Jinv00-1.7349837662337662e-04*Jinv02*w45-5.6818181818181818e-05*Jinv22*w59+7.8530844155844158e-04*w51*Jinv00+4.3831168831168834e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv10+2.7394480519480518e-04*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13+9.1314935064935065e-06*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10+9.1314935064935065e-06*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+7.4878246753246752e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41-1.3697240259740259e-04*w26*Jinv20+4.0584415584415584e-05*w11*Jinv20+6.5746753246753243e-04*w52*Jinv12+2.1915584415584417e-04*Jinv22*w26-1.0957792207792208e-04*Jinv20*w14+3.9265422077922079e-04*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+7.4878246753246752e-04*w49*Jinv10+2.7394480519480518e-04*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12-6.0876623376623375e-04*Jinv02*w9+9.1314935064935065e-06*w7*Jinv10-4.7483766233766234e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-1.7349837662337662e-04*w31*Jinv12-1.7349837662337662e-04*w46*Jinv12+7.8530844155844158e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+3.9265422077922079e-04*Jinv22*w50-5.4788961038961036e-04*Jinv12*w7-1.7349837662337662e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49+7.8530844155844158e-04*w52*Jinv10+4.0584415584415584e-05*w9*Jinv00-4.7483766233766234e-04*w37*Jinv10-1.3697240259740259e-04*w24*Jinv00-1.4204545454545455e-05*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5-1.7349837662337662e-04*w47*Jinv22+4.0584415584415584e-05*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-5.6818181818181818e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+4.3831168831168834e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.3697240259740259e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.9265422077922079e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+2.7394480519480518e-04*w40*Jinv10+9.1314935064935065e-06*w20*Jinv20-1.4204545454545455e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14-1.0957792207792207e-03*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-1.3697240259740259e-04*w5*Jinv20+6.5746753246753243e-04*Jinv02*w51+3.9265422077922079e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+7.4878246753246752e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[9][52]=(0.0)*D;
  A[9][53]=(0.0)*D;
  A[9][54]=(9.6793831168831166e-04*w40*Jinv12+3.9265422077922079e-04*Jinv22*w53-9.2329545454545453e-05*w57*Jinv02+9.6793831168831166e-04*Jinv02*w39+4.9310064935064932e-04*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3-4.8396915584415583e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-3.1047077922077924e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w42*Jinv01+4.0584415584415584e-05*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.2918019480519480e-04*Jinv01*w45+7.4878246753246752e-04*Jinv12*w55-7.3051948051948052e-05*w15*Jinv02+4.0584415584415584e-05*w10*Jinv12-3.1047077922077924e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.3697240259740259e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.8262987012987013e-05*Jinv22*w20+4.9310064935064932e-04*w43*Jinv11+2.7394480519480518e-04*Jinv21*w41+1.0592532467532468e-03*w42*Jinv02+4.0584415584415584e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-1.0957792207792208e-04*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-7.3051948051948052e-05*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-3.1960227272727273e-04*w32*Jinv22+2.7394480519480518e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2-4.7483766233766234e-04*Jinv21*w38-1.5523538961038962e-04*Jinv02*w45-1.3697240259740259e-04*Jinv01*w24-9.2329545454545453e-05*Jinv22*w59+4.9310064935064932e-04*w34*Jinv11+9.1314935064935065e-06*w19*Jinv11+7.8530844155844158e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+1.0592532467532468e-03*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-2.1915584415584417e-04*Jinv12*w13-4.2918019480519480e-04*w32*Jinv21+9.1314935064935065e-06*Jinv21*w20-4.8396915584415583e-04*w37*Jinv12+9.6793831168831166e-04*Jinv22*w41+3.9265422077922079e-04*w52*Jinv12+4.9310064935064932e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-3.1047077922077924e-04*Jinv22*w26-4.2918019480519480e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21+7.4878246753246752e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-2.1915584415584417e-04*Jinv02*w12+9.1314935064935065e-06*Jinv01*w18-4.7483766233766234e-04*w37*Jinv11+4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-4.2918019480519480e-04*w47*Jinv21-4.2918019480519480e-04*Jinv22*w23+7.8530844155844158e-04*w52*Jinv11-3.1960227272727273e-04*w31*Jinv12-1.5523538961038962e-04*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8+3.7439123376623376e-04*Jinv22*w50-1.0957792207792208e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7-1.0957792207792208e-04*w21*Jinv01+7.8530844155844158e-04*Jinv01*w51-3.1960227272727273e-04*Jinv02*w30+3.7439123376623376e-04*Jinv12*w49-4.7483766233766234e-04*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11+7.4878246753246752e-04*Jinv21*w56-1.4204545454545455e-05*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02+1.8262987012987013e-05*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5-1.5523538961038962e-04*w47*Jinv22+7.4878246753246752e-04*w49*Jinv11-1.0957792207792208e-04*Jinv21*w14-9.2329545454545453e-05*w58*Jinv12+1.8262987012987013e-05*Jinv02*w18-1.0957792207792208e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33-1.0957792207792208e-04*Jinv21*w23+7.4878246753246752e-04*Jinv01*w54+7.4878246753246752e-04*w55*Jinv11+3.7439123376623376e-04*w48*Jinv02+4.0584415584415584e-05*Jinv01*w9-1.4204545454545455e-05*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+2.7394480519480518e-04*w40*Jinv11-4.8396915584415583e-04*Jinv02*w36+1.0592532467532468e-03*Jinv12*w43-4.2918019480519480e-04*Jinv01*w30+3.9265422077922079e-04*Jinv02*w51+7.4878246753246752e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28-1.3697240259740259e-04*w25*Jinv11-7.3051948051948052e-05*w16*Jinv12+7.4878246753246752e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1+4.0584415584415584e-05*w10*Jinv11)*D;
  A[9][55]=(0.0)*D;
  A[9][56]=(0.0)*D;
  A[9][57]=(-1.9480519480519481e-04*w40*Jinv12+2.0495129870129870e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-1.9480519480519481e-04*Jinv02*w39+8.3198051948051951e-05*Jinv22*w38-1.4204545454545455e-05*w25*Jinv12+5.4112554112554113e-05*w0*Jinv02+1.5726461038961039e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+1.8939393939393939e-04*w10*Jinv12-1.4204545454545455e-05*Jinv02*w24-4.2613636363636367e-05*Jinv22*w35+6.4935064935064935e-05*Jinv22*w20-4.2613636363636367e-05*w42*Jinv02+1.8939393939393939e-04*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17+1.0247564935064935e-04*w22*Jinv12-1.4204545454545455e-05*Jinv02*w3-1.1668019480519480e-04*w32*Jinv22+5.4112554112554113e-05*Jinv22*w2-1.1668019480519480e-04*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-4.2613636363636367e-05*Jinv12*w34-4.2613636363636367e-05*Jinv22*w44+1.0247564935064935e-04*Jinv12*w13+8.3198051948051951e-05*w37*Jinv12-1.9480519480519481e-04*Jinv22*w41+2.0495129870129870e-04*w52*Jinv12-1.4204545454545455e-05*Jinv22*w26+1.5726461038961039e-04*Jinv02*w54-1.4204545454545455e-05*w4*Jinv12+1.0247564935064935e-04*w21*Jinv02+1.0247564935064935e-04*Jinv02*w12+1.8939393939393939e-04*Jinv02*w9+1.0247564935064935e-04*Jinv22*w23-1.1668019480519480e-04*w31*Jinv12-1.1668019480519480e-04*w46*Jinv12+6.4935064935064935e-05*Jinv22*w8+1.5726461038961039e-04*Jinv22*w50+6.4935064935064935e-05*Jinv12*w7-1.1668019480519480e-04*Jinv02*w30+1.5726461038961039e-04*Jinv12*w49+5.4112554112554113e-05*Jinv02*w27+6.4935064935064935e-05*w6*Jinv02+6.4935064935064935e-05*w19*Jinv12-1.4204545454545455e-05*Jinv22*w5-1.1668019480519480e-04*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12+6.4935064935064935e-05*Jinv02*w18-4.2613636363636367e-05*Jinv02*w33+1.5726461038961039e-04*w48*Jinv02+5.4112554112554113e-05*Jinv22*w29+1.0247564935064935e-04*Jinv22*w14+8.3198051948051951e-05*Jinv02*w36-4.2613636363636367e-05*Jinv12*w43+2.0495129870129870e-04*Jinv02*w51+1.5726461038961039e-04*Jinv22*w56+5.4112554112554113e-05*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+5.4112554112554113e-05*Jinv12*w1)*D;
  A[9][58]=(0.0)*D;
  A[9][59]=(0.0)*D;
  A[10][0]=(0.0)*D;
  A[10][1]=(-1.5726461038961039e-04*Jinv00*w12-5.4112554112554113e-05*w28*Jinv10+1.9480519480519481e-04*w40*Jinv12+1.4204545454545455e-05*Jinv22*w53-8.3198051948051951e-05*Jinv20*w8-5.4112554112554113e-05*w57*Jinv02-1.5726461038961039e-04*w13*Jinv10+1.9480519480519481e-04*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-2.0495129870129870e-04*w3*Jinv00-1.5726461038961039e-04*w32*Jinv20-2.0495129870129870e-04*Jinv01*w3+1.1668019480519480e-04*Jinv20*w23-6.4935064935064935e-05*Jinv22*w38-1.0247564935064935e-04*Jinv00*w54-5.4112554112554113e-05*Jinv21*w59+1.4204545454545455e-05*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-1.8939393939393939e-04*w11*Jinv21+4.2613636363636367e-05*w16*Jinv11+4.2613636363636367e-05*Jinv21*w17-1.0247564935064935e-04*Jinv01*w45-1.0247564935064935e-04*Jinv12*w55-5.4112554112554113e-05*w58*Jinv10+4.2613636363636367e-05*w15*Jinv02-1.8939393939393939e-04*w10*Jinv12+1.1668019480519480e-04*w48*Jinv00+1.4204545454545455e-05*Jinv02*w24+4.2613636363636367e-05*Jinv22*w35-5.4112554112554113e-05*w58*Jinv11-5.4112554112554113e-05*w57*Jinv01+1.4204545454545455e-05*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+4.2613636363636367e-05*w16*Jinv10-6.4935064935064935e-05*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11-1.0247564935064935e-04*w45*Jinv00+1.9480519480519481e-04*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-5.4112554112554113e-05*Jinv00*w27-1.8939393939393939e-04*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-1.5726461038961039e-04*w13*Jinv11+4.2613636363636367e-05*w15*Jinv01-5.4112554112554113e-05*w28*Jinv11+1.1668019480519480e-04*Jinv20*w50-5.4112554112554113e-05*Jinv20*w29-1.0247564935064935e-04*w46*Jinv10+4.2613636363636367e-05*Jinv22*w17+4.2613636363636367e-05*Jinv00*w33+1.1668019480519480e-04*w22*Jinv12-1.5726461038961039e-04*w31*Jinv10-2.0495129870129870e-04*Jinv02*w3+1.1668019480519480e-04*w21*Jinv00-1.5726461038961039e-04*w32*Jinv22+1.9480519480519481e-04*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2-6.4935064935064935e-05*Jinv00*w18-6.4935064935064935e-05*w36*Jinv00-6.4935064935064935e-05*Jinv21*w38-1.0247564935064935e-04*Jinv02*w45+1.4204545454545455e-05*Jinv01*w24-5.4112554112554113e-05*Jinv22*w59+4.2613636363636367e-05*w34*Jinv11-6.4935064935064935e-05*w19*Jinv11+1.4204545454545455e-05*Jinv21*w53+1.4204545454545455e-05*w51*Jinv00+4.2613636363636367e-05*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-2.0495129870129870e-04*w4*Jinv10-2.0495129870129870e-04*w4*Jinv11+1.9480519480519481e-04*w41*Jinv20-1.5726461038961039e-04*Jinv12*w13-6.4935064935064935e-05*w19*Jinv10+4.2613636363636367e-05*w34*Jinv10-1.5726461038961039e-04*w32*Jinv21-6.4935064935064935e-05*Jinv21*w20-8.3198051948051951e-05*w6*Jinv00-6.4935064935064935e-05*w37*Jinv12-1.0247564935064935e-04*Jinv20*w56+1.9480519480519481e-04*Jinv22*w41+1.4204545454545455e-05*w26*Jinv20-1.8939393939393939e-04*w11*Jinv20+1.4204545454545455e-05*w52*Jinv12+4.2613636363636367e-05*Jinv21*w35-1.5726461038961039e-04*w31*Jinv11+1.4204545454545455e-05*Jinv22*w26-1.0247564935064935e-04*w46*Jinv11-1.5726461038961039e-04*Jinv20*w14-2.0495129870129870e-04*w5*Jinv21-1.0247564935064935e-04*Jinv02*w54-2.0495129870129870e-04*w4*Jinv12+1.1668019480519480e-04*w21*Jinv02+1.1668019480519480e-04*w49*Jinv10-5.4112554112554113e-05*Jinv21*w29+4.2613636363636367e-05*Jinv01*w33+1.9480519480519481e-04*w39*Jinv00-1.5726461038961039e-04*Jinv02*w12-6.4935064935064935e-05*Jinv01*w18-6.4935064935064935e-05*w37*Jinv11-1.8939393939393939e-04*Jinv02*w9-8.3198051948051951e-05*w6*Jinv01-8.3198051948051951e-05*w7*Jinv10-1.0247564935064935e-04*w47*Jinv21-6.4935064935064935e-05*Jinv20*w38+1.1668019480519480e-04*Jinv22*w23+1.4204545454545455e-05*w52*Jinv11-1.5726461038961039e-04*w31*Jinv12-1.0247564935064935e-04*w46*Jinv12+1.4204545454545455e-05*Jinv20*w53-8.3198051948051951e-05*Jinv22*w8+1.1668019480519480e-04*w22*Jinv10+1.1668019480519480e-04*Jinv22*w50+1.1668019480519480e-04*w22*Jinv11-8.3198051948051951e-05*Jinv12*w7+1.1668019480519480e-04*w21*Jinv01+1.4204545454545455e-05*Jinv01*w51-1.5726461038961039e-04*Jinv02*w30+1.1668019480519480e-04*Jinv12*w49-6.4935064935064935e-05*w36*Jinv01+1.4204545454545455e-05*w52*Jinv10-8.3198051948051951e-05*w7*Jinv11-1.8939393939393939e-04*w9*Jinv00-6.4935064935064935e-05*w37*Jinv10-1.0247564935064935e-04*Jinv21*w56+1.4204545454545455e-05*w24*Jinv00-5.4112554112554113e-05*Jinv02*w27-8.3198051948051951e-05*w6*Jinv02-6.4935064935064935e-05*w19*Jinv12-2.0495129870129870e-04*Jinv22*w5-1.0247564935064935e-04*w47*Jinv22+1.1668019480519480e-04*w49*Jinv11-1.5726461038961039e-04*Jinv21*w14-1.8939393939393939e-04*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-5.4112554112554113e-05*w58*Jinv12-6.4935064935064935e-05*Jinv02*w18-1.5726461038961039e-04*Jinv01*w12-8.3198051948051951e-05*Jinv21*w8-5.4112554112554113e-05*w57*Jinv00-5.4112554112554113e-05*Jinv20*w59+4.2613636363636367e-05*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+4.2613636363636367e-05*w35*Jinv20+1.1668019480519480e-04*Jinv21*w23-1.0247564935064935e-04*Jinv01*w54-1.0247564935064935e-04*w55*Jinv11+1.4204545454545455e-05*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+4.2613636363636367e-05*w15*Jinv00+1.1668019480519480e-04*w48*Jinv02-1.8939393939393939e-04*Jinv01*w9+4.2613636363636367e-05*Jinv20*w17-1.5726461038961039e-04*w30*Jinv00+1.9480519480519481e-04*w40*Jinv10-6.4935064935064935e-05*w20*Jinv20-5.4112554112554113e-05*Jinv22*w29-1.5726461038961039e-04*Jinv22*w14+1.9480519480519481e-04*w40*Jinv11-6.4935064935064935e-05*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-2.0495129870129870e-04*w5*Jinv20-1.5726461038961039e-04*Jinv01*w30+1.4204545454545455e-05*Jinv02*w51-1.0247564935064935e-04*Jinv22*w56-5.4112554112554113e-05*Jinv12*w28+1.4204545454545455e-05*w25*Jinv11-1.0247564935064935e-04*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00-1.0247564935064935e-04*w47*Jinv20+4.2613636363636367e-05*w16*Jinv12+1.1668019480519480e-04*Jinv21*w50-5.4112554112554113e-05*Jinv01*w27+1.1668019480519480e-04*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-1.8939393939393939e-04*w10*Jinv11)*D;
  A[10][2]=(0.0)*D;
  A[10][3]=(0.0)*D;
  A[10][4]=(3.5612824675324673e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10-5.2962662337662338e-04*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53+6.2094155844155847e-04*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+1.2784090909090910e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.5568181818181820e-04*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59-2.1915584415584417e-04*w25*Jinv12+5.6818181818181818e-05*w0*Jinv02+5.6818181818181818e-05*Jinv21*w2-2.5568181818181820e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+6.0876623376623375e-04*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+6.0876623376623375e-04*w10*Jinv12-2.5568181818181820e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26+5.6818181818181818e-05*w0*Jinv01+1.2175324675324675e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00-5.2962662337662338e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27+6.0876623376623375e-04*w11*Jinv22+5.6818181818181818e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11-2.5568181818181820e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.7349837662337662e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3-2.5568181818181820e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+5.6818181818181818e-05*Jinv22*w2+5.5702110389610384e-04*Jinv00*w18+5.5702110389610384e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+1.2784090909090910e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11-2.5568181818181820e-04*w41*Jinv20-3.9265422077922079e-04*Jinv12*w13+5.5702110389610384e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-3.9265422077922079e-04*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20+6.2094155844155847e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56-5.2962662337662338e-04*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-3.9265422077922079e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12+1.7349837662337662e-04*w21*Jinv02-2.5568181818181820e-04*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-2.5568181818181820e-04*w39*Jinv00-3.9265422077922079e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+6.0876623376623375e-04*Jinv02*w9+1.0957792207792207e-03*w6*Jinv01+6.2094155844155847e-04*w7*Jinv10+1.3697240259740259e-04*w47*Jinv21+5.5702110389610384e-04*Jinv20*w38+1.7349837662337662e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8-2.5568181818181820e-04*w22*Jinv10+1.7349837662337662e-04*Jinv22*w50+1.7349837662337662e-04*w22*Jinv11+1.0957792207792207e-03*Jinv12*w7+1.7349837662337662e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+1.7349837662337662e-04*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+6.4935064935064935e-04*w9*Jinv00+5.5702110389610384e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+1.7349837662337662e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+6.4935064935064935e-04*w10*Jinv10-2.5568181818181820e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33+1.2175324675324675e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+1.7349837662337662e-04*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10+1.2175324675324675e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+1.7349837662337662e-04*w48*Jinv02+6.0876623376623375e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00-2.5568181818181820e-04*w40*Jinv10+5.5702110389610384e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-3.9265422077922079e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+1.2784090909090910e-04*w5*Jinv20-3.9265422077922079e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28-2.1915584415584417e-04*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10-2.5568181818181820e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+1.7349837662337662e-04*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+1.7349837662337662e-04*w48*Jinv01+5.6818181818181818e-05*Jinv12*w1+6.0876623376623375e-04*w10*Jinv11)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(0.0)*D;
  A[10][7]=(-2.1915584415584417e-04*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10+4.9310064935064932e-04*w40*Jinv12+5.4788961038961036e-04*Jinv22*w53-1.1870941558441558e-03*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02-2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv02*w39+5.4788961038961036e-04*Jinv21*w44+4.0178571428571428e-04*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3+1.1870941558441559e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12-6.4935064935064935e-05*w0*Jinv02-6.4935064935064935e-05*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+5.4788961038961036e-04*w42*Jinv01-1.5827922077922079e-03*w11*Jinv21+5.4788961038961036e-04*w16*Jinv11+5.4788961038961036e-04*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-1.5827922077922079e-03*w10*Jinv12+1.1870941558441559e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+5.4788961038961036e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+5.4788961038961036e-04*Jinv21*w26-6.4935064935064935e-05*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+5.4788961038961036e-04*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+4.9310064935064932e-04*Jinv21*w41+5.4788961038961036e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-1.5827922077922079e-03*w11*Jinv22-6.4935064935064935e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+5.4788961038961036e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11+1.1870941558441559e-04*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-9.1314935064935065e-06*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+1.1870941558441559e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-6.4935064935064935e-05*Jinv22*w2-1.1414366883116883e-03*Jinv00*w18-1.1414366883116883e-03*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45+5.4788961038961036e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+6.6659902597402602e-04*w51*Jinv00+5.4788961038961036e-04*Jinv12*w34+5.4788961038961036e-04*Jinv22*w44+4.0178571428571428e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-1.8262987012987014e-04*w41*Jinv20-9.1314935064935065e-06*Jinv12*w13-1.1414366883116883e-03*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10-9.1314935064935065e-06*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20-1.1870941558441558e-03*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+4.9310064935064932e-04*Jinv22*w41+6.6659902597402602e-04*w26*Jinv20-1.2987012987012987e-03*w11*Jinv20+5.4788961038961036e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35-9.1314935064935065e-06*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-9.1314935064935065e-06*w21*Jinv02+1.1870941558441559e-04*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-1.8262987012987014e-04*w39*Jinv00-9.1314935064935065e-06*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11-1.5827922077922079e-03*Jinv02*w9-1.0957792207792207e-03*w6*Jinv01-1.1870941558441558e-03*w7*Jinv10-9.1314935064935065e-06*w47*Jinv21-1.1414366883116883e-03*Jinv20*w38-9.1314935064935065e-06*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+6.6659902597402602e-04*Jinv20*w53-1.0957792207792207e-03*Jinv22*w8+1.1870941558441559e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11-1.0957792207792207e-03*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+6.6659902597402602e-04*w52*Jinv10-1.0957792207792207e-03*w7*Jinv11-1.2987012987012987e-03*w9*Jinv00-1.1414366883116883e-03*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+6.6659902597402602e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-1.0957792207792207e-03*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22-9.1314935064935065e-06*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.2987012987012987e-03*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-1.0957792207792207e-03*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59+5.4788961038961036e-04*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+6.6659902597402602e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-1.5827922077922079e-03*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-1.8262987012987014e-04*w40*Jinv10-1.1414366883116883e-03*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29-9.1314935064935065e-06*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+5.4788961038961036e-04*Jinv12*w43+4.0178571428571428e-04*w5*Jinv20-9.1314935064935065e-06*Jinv01*w30+5.4788961038961036e-04*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12-9.1314935064935065e-06*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-9.1314935064935065e-06*w48*Jinv01-6.4935064935064935e-05*Jinv12*w1-1.5827922077922079e-03*w10*Jinv11)*D;
  A[10][8]=(0.0)*D;
  A[10][9]=(0.0)*D;
  A[10][10]=(2.0292207792207792e-05*Jinv00*w12+9.4696969696969697e-05*w28*Jinv10+6.4935064935064935e-04*Jinv20*w8+2.0292207792207792e-05*w13*Jinv10-3.2467532467532468e-04*w3*Jinv00+2.0292207792207792e-05*w32*Jinv20+2.0292207792207792e-05*Jinv20*w23+2.0292207792207792e-05*Jinv00*w54+1.2175324675324675e-04*w43*Jinv10+9.4696969696969697e-05*w58*Jinv10+2.0292207792207792e-05*w48*Jinv00+9.4696969696969697e-05*w1*Jinv10+1.2175324675324675e-04*w16*Jinv10+2.0292207792207792e-05*w45*Jinv00+9.4696969696969697e-05*Jinv00*w27+2.0292207792207792e-05*Jinv20*w50+9.4696969696969697e-05*Jinv20*w29+2.0292207792207792e-05*w46*Jinv10+1.2175324675324675e-04*Jinv00*w33+2.0292207792207792e-05*w31*Jinv10+2.0292207792207792e-05*w21*Jinv00+6.4935064935064935e-04*Jinv00*w18+6.4935064935064935e-04*w36*Jinv00-3.2467532467532468e-04*w51*Jinv00-3.2467532467532468e-04*w4*Jinv10+2.4350649350649351e-04*w41*Jinv20+6.4935064935064935e-04*w19*Jinv10+1.2175324675324675e-04*w34*Jinv10+6.4935064935064935e-04*w6*Jinv00+2.0292207792207792e-05*Jinv20*w56-3.2467532467532468e-04*w26*Jinv20+8.3874458874458877e-04*w11*Jinv20+2.0292207792207792e-05*Jinv20*w14+2.0292207792207792e-05*w49*Jinv10+2.4350649350649351e-04*w39*Jinv00+6.4935064935064935e-04*w7*Jinv10+6.4935064935064935e-04*Jinv20*w38-3.2467532467532468e-04*Jinv20*w53+2.0292207792207792e-05*w22*Jinv10-3.2467532467532468e-04*w52*Jinv10+8.3874458874458877e-04*w9*Jinv00+6.4935064935064935e-04*w37*Jinv10-3.2467532467532468e-04*w24*Jinv00+8.3874458874458877e-04*w10*Jinv10+1.2175324675324675e-04*Jinv20*w44+9.4696969696969697e-05*w57*Jinv00+9.4696969696969697e-05*Jinv20*w59+9.4696969696969697e-05*w0*Jinv00+1.2175324675324675e-04*w35*Jinv20-3.2467532467532468e-04*w25*Jinv10+9.4696969696969697e-05*Jinv20*w2+1.2175324675324675e-04*w15*Jinv00+1.2175324675324675e-04*Jinv20*w17+2.0292207792207792e-05*w30*Jinv00+2.4350649350649351e-04*w40*Jinv10+6.4935064935064935e-04*w20*Jinv20-3.2467532467532468e-04*w5*Jinv20+2.0292207792207792e-05*w55*Jinv10+1.2175324675324675e-04*w42*Jinv00+2.0292207792207792e-05*w47*Jinv20)*D;
  A[10][11]=(0.0)*D;
  A[10][12]=(0.0)*D;
  A[10][13]=(-7.4878246753246752e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-9.6793831168831166e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53+4.8396915584415583e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02-7.4878246753246752e-04*w13*Jinv10-9.6793831168831166e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-3.7439123376623376e-04*w32*Jinv20+3.9265422077922079e-04*Jinv01*w3+1.5523538961038962e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54+3.1047077922077924e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+1.5726461038961039e-04*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01-5.6615259740259744e-04*w16*Jinv11-5.6615259740259744e-04*Jinv21*w17+3.1960227272727273e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-1.0592532467532468e-03*w15*Jinv02-4.0584415584415584e-05*w10*Jinv12+3.1960227272727273e-04*w48*Jinv00+3.1047077922077924e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+1.7349837662337662e-04*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-1.0592532467532468e-03*w16*Jinv10-1.8262987012987013e-05*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-6.9399350649350649e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-4.0584415584415584e-05*w11*Jinv22+1.5726461038961039e-04*Jinv11*w1-5.6615259740259744e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+3.1960227272727273e-04*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-1.0592532467532468e-03*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.5523538961038962e-04*w22*Jinv12-3.7439123376623376e-04*w31*Jinv10-3.9265422077922079e-04*Jinv02*w3+1.5523538961038962e-04*w21*Jinv00-3.7439123376623376e-04*w32*Jinv22-6.9399350649350649e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2-1.8262987012987013e-05*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00+4.2918019480519480e-04*Jinv02*w45+1.7349837662337662e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+1.3697240259740259e-04*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10+3.9265422077922079e-04*w4*Jinv11-9.6793831168831166e-04*w41*Jinv20-7.4878246753246752e-04*Jinv12*w13-1.8262987012987013e-05*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.7439123376623376e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20+4.8396915584415583e-04*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-9.6793831168831166e-04*Jinv22*w41+3.1047077922077924e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35+3.7439123376623376e-04*w31*Jinv11+3.1047077922077924e-04*Jinv22*w26+3.1960227272727273e-04*w46*Jinv11-7.4878246753246752e-04*Jinv20*w14+3.9265422077922079e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54-3.9265422077922079e-04*w4*Jinv12+1.5523538961038962e-04*w21*Jinv02+3.1960227272727273e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-9.6793831168831166e-04*w39*Jinv00-7.4878246753246752e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01+4.8396915584415583e-04*w7*Jinv10+3.1960227272727273e-04*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38+1.5523538961038962e-04*Jinv22*w23-3.7439123376623376e-04*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53+4.8396915584415583e-04*Jinv22*w8+1.5523538961038962e-04*w22*Jinv10+3.1960227272727273e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+4.8396915584415583e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-3.7439123376623376e-04*Jinv02*w30+3.1960227272727273e-04*Jinv12*w49+1.3697240259740259e-04*w52*Jinv10+9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+3.1047077922077924e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+4.8396915584415583e-04*w6*Jinv02-1.8262987012987013e-05*w19*Jinv12-3.9265422077922079e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-4.0584415584415584e-05*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-1.8262987012987013e-05*Jinv02*w18+9.1314935064935065e-06*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+3.1047077922077924e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-1.0592532467532468e-03*w15*Jinv00+3.1960227272727273e-04*w48*Jinv02-1.0592532467532468e-03*Jinv20*w17-3.7439123376623376e-04*w30*Jinv00-9.6793831168831166e-04*w40*Jinv10-1.8262987012987013e-05*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-7.4878246753246752e-04*Jinv22*w14-6.9399350649350649e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20+3.7439123376623376e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+1.7349837662337662e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-1.0592532467532468e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1)*D;
  A[10][14]=(0.0)*D;
  A[10][15]=(0.0)*D;
  A[10][16]=(6.2094155844155847e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10-1.1323051948051949e-03*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+6.2094155844155847e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+3.1047077922077924e-04*w32*Jinv20+4.3831168831168834e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38-3.4699675324675324e-04*Jinv00*w54-6.5746753246753243e-04*w25*Jinv12-3.0438311688311688e-05*w0*Jinv02-4.2613636363636367e-05*Jinv21*w2+6.9399350649350649e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-5.2962662337662338e-04*Jinv01*w45+7.3051948051948052e-05*Jinv12*w55+1.9480519480519481e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02-7.3051948051948055e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-6.5746753246753243e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-4.3831168831168834e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01+1.8262987012987013e-03*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00-4.3831168831168834e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27-7.3051948051948055e-04*w11*Jinv22-4.2613636363636367e-05*Jinv11*w1+5.6615259740259744e-04*w13*Jinv11+4.2613636363636367e-05*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+9.1314935064935063e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+3.1047077922077924e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22-3.0438311688311688e-05*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-4.9310064935064932e-04*Jinv02*w45-4.3831168831168834e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-2.5568181818181820e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-4.3831168831168834e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.3831168831168834e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+7.3051948051948052e-05*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10+9.1314935064935063e-04*w34*Jinv10+5.2962662337662338e-04*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12-3.4699675324675324e-04*Jinv20*w56-1.1323051948051949e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-2.4350649350649351e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+5.2962662337662338e-04*w31*Jinv11-6.5746753246753243e-04*Jinv22*w26-5.2962662337662338e-04*w46*Jinv11+6.2094155844155847e-04*Jinv20*w14+4.3831168831168834e-04*w5*Jinv21+7.3051948051948052e-05*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+7.3051948051948052e-05*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18-7.3051948051948055e-04*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10-5.2962662337662338e-04*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-4.9310064935064932e-04*w46*Jinv12-2.5568181818181820e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-5.6615259740259744e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-5.6615259740259744e-04*w21*Jinv01+3.6525974025974026e-05*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49-2.5568181818181820e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11-2.4350649350649351e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+3.6525974025974026e-05*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+1.2175324675324675e-05*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5-4.9310064935064932e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+5.6615259740259744e-04*Jinv21*w14-2.4350649350649351e-04*w10*Jinv10+6.9399350649350649e-04*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18+5.6615259740259744e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+1.9480519480519481e-04*w57*Jinv00+1.9480519480519481e-04*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33+9.1314935064935063e-04*w35*Jinv20-5.6615259740259744e-04*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10+1.8262987012987013e-03*w15*Jinv00+3.6525974025974026e-05*w48*Jinv02+1.8262987012987013e-03*Jinv20*w17+3.1047077922077924e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20+1.2175324675324675e-05*Jinv22*w29+7.3051948051948052e-05*Jinv22*w14+5.4788961038961036e-04*Jinv02*w36-4.3831168831168834e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+5.2962662337662338e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51+7.3051948051948052e-05*Jinv22*w56+1.2175324675324675e-05*Jinv12*w28-4.3831168831168834e-04*w25*Jinv11-3.4699675324675324e-04*w55*Jinv10+6.9399350649350649e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-3.0438311688311688e-05*Jinv12*w1)*D;
  A[10][17]=(0.0)*D;
  A[10][18]=(0.0)*D;
  A[10][19]=(1.2784090909090910e-04*Jinv00*w12-5.6818181818181818e-05*w28*Jinv10-4.5657467532467534e-05*Jinv20*w8+1.2784090909090910e-04*w13*Jinv10-5.4788961038961036e-04*Jinv21*w44+1.1870941558441559e-04*w3*Jinv00+6.3920454545454550e-05*w32*Jinv20-5.4788961038961036e-04*Jinv01*w3-2.1002435064935065e-04*Jinv20*w23+1.2784090909090910e-04*Jinv00*w54+6.4935064935064935e-05*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.5827922077922079e-03*w11*Jinv21-5.4788961038961036e-04*w16*Jinv11-5.4788961038961036e-04*Jinv21*w17+9.1314935064935065e-06*Jinv01*w45-9.2329545454545453e-05*w58*Jinv10+6.3920454545454550e-05*w48*Jinv00+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01-9.2329545454545453e-05*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-5.4788961038961036e-04*w43*Jinv11-2.1002435064935065e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41-5.6818181818181818e-05*Jinv00*w27+6.4935064935064935e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-5.4788961038961036e-04*w15*Jinv01+6.4935064935064935e-05*w28*Jinv11+6.3920454545454550e-05*Jinv20*w50-5.6818181818181818e-05*Jinv20*w29-2.1002435064935065e-04*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+6.3920454545454550e-05*w31*Jinv10-2.1002435064935065e-04*w21*Jinv00-4.9310064935064932e-04*Jinv01*w39-9.1314935064935068e-05*Jinv00*w18-4.5657467532467534e-05*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38-5.4788961038961036e-04*Jinv01*w24-5.4788961038961036e-04*w34*Jinv11+1.0957792207792207e-03*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53+1.1870941558441559e-04*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10-5.4788961038961036e-04*w4*Jinv11-6.7573051948051951e-04*w41*Jinv20-9.1314935064935068e-05*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21+1.0957792207792207e-03*Jinv21*w20-4.5657467532467534e-05*w6*Jinv00+1.2784090909090910e-04*Jinv20*w56-1.4610389610389610e-04*w26*Jinv20+2.8409090909090908e-04*w11*Jinv20-5.4788961038961036e-04*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11+9.1314935064935065e-06*w46*Jinv11+1.2784090909090910e-04*Jinv20*w14-5.4788961038961036e-04*w5*Jinv21+6.3920454545454550e-05*w49*Jinv10+6.4935064935064935e-05*Jinv21*w29-5.4788961038961036e-04*Jinv01*w33-6.7573051948051951e-04*w39*Jinv00+1.0957792207792207e-03*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+1.0957792207792207e-03*w6*Jinv01-4.5657467532467534e-05*w7*Jinv10+9.1314935064935065e-06*w47*Jinv21-4.5657467532467534e-05*Jinv20*w38-5.4788961038961036e-04*w52*Jinv11+1.1870941558441559e-04*Jinv20*w53-2.1002435064935065e-04*w22*Jinv10+9.1314935064935065e-06*w22*Jinv11+9.1314935064935065e-06*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+1.0957792207792207e-03*w36*Jinv01+1.1870941558441559e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+2.8409090909090908e-04*w9*Jinv00-4.5657467532467534e-05*w37*Jinv10+9.1314935064935065e-06*Jinv21*w56-1.4610389610389610e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14+2.8409090909090908e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+9.1314935064935065e-06*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-9.2329545454545453e-05*w57*Jinv00-9.2329545454545453e-05*Jinv20*w59-9.2329545454545453e-05*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+9.1314935064935065e-06*Jinv21*w23+9.1314935064935065e-06*Jinv01*w54+9.1314935064935065e-06*w55*Jinv11-1.4610389610389610e-04*w25*Jinv10-9.2329545454545453e-05*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00+1.5827922077922079e-03*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17+6.3920454545454550e-05*w30*Jinv00-6.7573051948051951e-04*w40*Jinv10-9.1314935064935068e-05*w20*Jinv20-4.9310064935064932e-04*w40*Jinv11+1.1870941558441559e-04*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30-5.4788961038961036e-04*w25*Jinv11+1.2784090909090910e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-2.1002435064935065e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50+6.4935064935064935e-05*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+1.5827922077922079e-03*w10*Jinv11)*D;
  A[10][20]=(0.0)*D;
  A[10][21]=(0.0)*D;
  A[10][22]=(4.2918019480519480e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.7394480519480518e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+4.2918019480519480e-04*w13*Jinv10-2.7394480519480518e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20-1.7349837662337662e-04*Jinv01*w3-7.4878246753246752e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-7.8530844155844158e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.1668019480519480e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01+5.6615259740259744e-04*w16*Jinv11+5.6615259740259744e-04*Jinv21*w17-3.7439123376623376e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-4.0584415584415584e-05*w10*Jinv12+1.0957792207792208e-04*w48*Jinv00-7.8530844155844158e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-3.9265422077922079e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10+4.7483766233766234e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00+6.9399350649350649e-04*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-4.0584415584415584e-05*w11*Jinv22-1.1668019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11+5.6615259740259744e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+1.0957792207792208e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.4878246753246752e-04*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+1.3697240259740259e-04*Jinv02*w3-7.4878246753246752e-04*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22+6.9399350649350649e-04*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2+4.7483766233766234e-04*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00-7.4878246753246752e-04*Jinv02*w45-3.9265422077922079e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+1.3697240259740259e-04*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10-1.7349837662337662e-04*w4*Jinv11-2.7394480519480518e-04*w41*Jinv20+4.2918019480519480e-04*Jinv12*w13+4.7483766233766234e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-3.1960227272727273e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-2.7394480519480518e-04*Jinv22*w41-7.8530844155844158e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35-3.1960227272727273e-04*w31*Jinv11-7.8530844155844158e-04*Jinv22*w26-3.7439123376623376e-04*w46*Jinv11+4.2918019480519480e-04*Jinv20*w14-1.7349837662337662e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+1.3697240259740259e-04*w4*Jinv12-7.4878246753246752e-04*w21*Jinv02+1.0957792207792208e-04*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-2.7394480519480518e-04*w39*Jinv00+4.2918019480519480e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10-3.7439123376623376e-04*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38-7.4878246753246752e-04*Jinv22*w23+1.0957792207792208e-04*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53-9.1314935064935065e-06*Jinv22*w8-7.4878246753246752e-04*w22*Jinv10+1.0957792207792208e-04*Jinv22*w50-9.1314935064935065e-06*Jinv12*w7+1.0957792207792208e-04*Jinv02*w30+1.0957792207792208e-04*Jinv12*w49+1.3697240259740259e-04*w52*Jinv10+9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56-7.8530844155844158e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-9.1314935064935065e-06*w6*Jinv02+4.7483766233766234e-04*w19*Jinv12+1.3697240259740259e-04*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+4.7483766233766234e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11-7.8530844155844158e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+1.0957792207792208e-04*w48*Jinv02-4.9310064935064932e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-2.7394480519480518e-04*w40*Jinv10+4.7483766233766234e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.2918019480519480e-04*Jinv22*w14+6.9399350649350649e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20-3.1960227272727273e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.9265422077922079e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1)*D;
  A[10][23]=(0.0)*D;
  A[10][24]=(0.0)*D;
  A[10][25]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10+9.1314935064935065e-06*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+4.3831168831168834e-04*Jinv21*w44-1.3697240259740259e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20+2.1915584415584417e-04*Jinv01*w3+7.4878246753246752e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54-1.4204545454545455e-05*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+4.9310064935064932e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-6.0876623376623375e-04*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17+3.9265422077922079e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+4.3831168831168834e-04*w43*Jinv11+7.4878246753246752e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27-1.4204545454545455e-05*Jinv11*w1-1.7349837662337662e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01-5.6818181818181818e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.7483766233766234e-04*Jinv00*w18+9.1314935064935065e-06*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53-1.3697240259740259e-04*w51*Jinv00-1.3697240259740259e-04*w4*Jinv10+2.1915584415584417e-04*w4*Jinv11+2.7394480519480518e-04*w41*Jinv20-4.7483766233766234e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20+9.1314935064935065e-06*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+7.8530844155844158e-04*w26*Jinv20+4.0584415584415584e-05*w11*Jinv20+2.1915584415584417e-04*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11+3.9265422077922079e-04*w46*Jinv11-4.2918019480519480e-04*Jinv20*w14+2.1915584415584417e-04*w5*Jinv21-1.0957792207792208e-04*w49*Jinv10-5.6818181818181818e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+2.7394480519480518e-04*w39*Jinv00-1.0957792207792207e-03*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-5.4788961038961036e-04*w6*Jinv01+9.1314935064935065e-06*w7*Jinv10+3.9265422077922079e-04*w47*Jinv21+9.1314935064935065e-06*Jinv20*w38+2.1915584415584417e-04*w52*Jinv11-1.3697240259740259e-04*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+3.9265422077922079e-04*w22*Jinv11+3.9265422077922079e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51-5.4788961038961036e-04*w36*Jinv01-1.3697240259740259e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11+4.0584415584415584e-05*w9*Jinv00+9.1314935064935065e-06*w37*Jinv10-1.7349837662337662e-04*Jinv21*w56+7.8530844155844158e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11-1.7349837662337662e-04*Jinv21*w14+4.0584415584415584e-05*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-1.7349837662337662e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.9265422077922079e-04*Jinv21*w23-1.7349837662337662e-04*Jinv01*w54-1.7349837662337662e-04*w55*Jinv11+7.8530844155844158e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-6.0876623376623375e-04*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00+2.7394480519480518e-04*w40*Jinv10-4.7483766233766234e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11-1.3697240259740259e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11-4.2918019480519480e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50-5.6818181818181818e-05*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01-6.0876623376623375e-04*w10*Jinv11)*D;
  A[10][26]=(0.0)*D;
  A[10][27]=(0.0)*D;
  A[10][28]=(-4.2613636363636367e-05*Jinv21*w44-1.4204545454545455e-05*Jinv01*w3+5.4112554112554113e-05*Jinv21*w59+5.4112554112554113e-05*Jinv21*w2-4.2613636363636367e-05*w42*Jinv01+1.8939393939393939e-04*w11*Jinv21-4.2613636363636367e-05*w16*Jinv11-4.2613636363636367e-05*Jinv21*w17+1.5726461038961039e-04*Jinv01*w45+5.4112554112554113e-05*w58*Jinv11+5.4112554112554113e-05*w57*Jinv01+2.0495129870129870e-04*Jinv21*w26+5.4112554112554113e-05*w0*Jinv01-4.2613636363636367e-05*w43*Jinv11-1.9480519480519481e-04*Jinv21*w41+5.4112554112554113e-05*Jinv11*w1-1.1668019480519480e-04*w13*Jinv11-4.2613636363636367e-05*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-1.9480519480519481e-04*Jinv01*w39+6.4935064935064935e-05*Jinv21*w38+2.0495129870129870e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11+8.3198051948051951e-05*w19*Jinv11-1.4204545454545455e-05*Jinv21*w53-1.4204545454545455e-05*w4*Jinv11+1.0247564935064935e-04*w32*Jinv21+8.3198051948051951e-05*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35+1.0247564935064935e-04*w31*Jinv11+1.5726461038961039e-04*w46*Jinv11-1.4204545454545455e-05*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33+8.3198051948051951e-05*Jinv01*w18+6.4935064935064935e-05*w37*Jinv11+6.4935064935064935e-05*w6*Jinv01+1.5726461038961039e-04*w47*Jinv21-1.4204545454545455e-05*w52*Jinv11+1.5726461038961039e-04*w22*Jinv11+1.5726461038961039e-04*w21*Jinv01-1.4204545454545455e-05*Jinv01*w51+6.4935064935064935e-05*w36*Jinv01+6.4935064935064935e-05*w7*Jinv11-1.1668019480519480e-04*Jinv21*w56+1.0247564935064935e-04*w49*Jinv11-1.1668019480519480e-04*Jinv21*w14-1.1668019480519480e-04*Jinv01*w12+6.4935064935064935e-05*Jinv21*w8+1.5726461038961039e-04*Jinv21*w23-1.1668019480519480e-04*Jinv01*w54-1.1668019480519480e-04*w55*Jinv11+1.8939393939393939e-04*Jinv01*w9-1.9480519480519481e-04*w40*Jinv11+1.0247564935064935e-04*Jinv01*w30+2.0495129870129870e-04*w25*Jinv11+1.0247564935064935e-04*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27+1.0247564935064935e-04*w48*Jinv01+1.8939393939393939e-04*w10*Jinv11)*D;
  A[10][29]=(0.0)*D;
  A[10][30]=(0.0)*D;
  A[10][31]=(-3.7439123376623376e-04*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-6.9399350649350649e-04*w40*Jinv12+1.7349837662337662e-04*Jinv22*w53+4.8396915584415583e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02-3.7439123376623376e-04*w13*Jinv10-6.9399350649350649e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-7.4878246753246752e-04*w32*Jinv20-3.9265422077922079e-04*Jinv01*w3+3.1960227272727273e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+1.5726461038961039e-04*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01-4.0584415584415584e-05*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45+3.1960227272727273e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02+1.5523538961038962e-04*w48*Jinv00-5.6615259740259744e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+7.3051948051948052e-05*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-9.6793831168831166e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27+9.2329545454545453e-05*Jinv11*w1-3.7439123376623376e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+1.5523538961038962e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-1.0592532467532468e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-7.4878246753246752e-04*w31*Jinv10+3.9265422077922079e-04*Jinv02*w3+3.1960227272727273e-04*w21*Jinv00-9.6793831168831166e-04*Jinv01*w39+1.5726461038961039e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18-1.8262987012987013e-05*w36*Jinv00-1.8262987012987013e-05*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-1.0592532467532468e-03*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+3.1047077922077924e-04*Jinv21*w53+3.1047077922077924e-04*w51*Jinv00-5.6615259740259744e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10-3.9265422077922079e-04*w4*Jinv11-9.6793831168831166e-04*w41*Jinv20+3.7439123376623376e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10-1.0592532467532468e-03*w34*Jinv10-7.4878246753246752e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20+4.8396915584415583e-04*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-6.9399350649350649e-04*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.7349837662337662e-04*w52*Jinv12-1.0592532467532468e-03*Jinv21*w35-7.4878246753246752e-04*w31*Jinv11+2.1915584415584417e-04*w46*Jinv11-3.7439123376623376e-04*Jinv20*w14-3.9265422077922079e-04*w5*Jinv21+3.1960227272727273e-04*Jinv02*w54+3.9265422077922079e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+1.5523538961038962e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-1.0592532467532468e-03*Jinv01*w33-9.6793831168831166e-04*w39*Jinv00+3.7439123376623376e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-1.8262987012987013e-05*w37*Jinv11+4.8396915584415583e-04*w6*Jinv01+4.8396915584415583e-04*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-1.8262987012987013e-05*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23+3.1047077922077924e-04*w52*Jinv11+1.0957792207792208e-04*w46*Jinv12+3.1047077922077924e-04*Jinv20*w53+9.1314935064935065e-06*Jinv22*w8+3.1960227272727273e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.1960227272727273e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+3.1960227272727273e-04*w21*Jinv01+3.1047077922077924e-04*Jinv01*w51-2.7394480519480518e-04*Jinv12*w49-1.8262987012987013e-05*w36*Jinv01+3.1047077922077924e-04*w52*Jinv10+4.8396915584415583e-04*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-1.8262987012987013e-05*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+9.1314935064935065e-06*w6*Jinv02+3.9265422077922079e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.5523538961038962e-04*w49*Jinv11-3.7439123376623376e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.7439123376623376e-04*Jinv01*w12+4.8396915584415583e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-5.6615259740259744e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-1.0592532467532468e-03*w35*Jinv20+3.1960227272727273e-04*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-4.0584415584415584e-05*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-7.4878246753246752e-04*w30*Jinv00-9.6793831168831166e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20+3.7439123376623376e-04*Jinv22*w14-9.6793831168831166e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20-7.4878246753246752e-04*Jinv01*w30+1.7349837662337662e-04*Jinv02*w51+3.1960227272727273e-04*Jinv22*w56+1.3697240259740259e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12+1.5523538961038962e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+1.5523538961038962e-04*w48*Jinv01+1.5726461038961039e-04*Jinv12*w1-4.0584415584415584e-05*w10*Jinv11)*D;
  A[10][32]=(0.0)*D;
  A[10][33]=(0.0)*D;
  A[10][34]=(3.1047077922077924e-04*Jinv00*w12+1.9480519480519481e-04*w28*Jinv10-4.3831168831168834e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02+3.1047077922077924e-04*w13*Jinv10-4.3831168831168834e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+6.2094155844155847e-04*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+1.2175324675324675e-05*Jinv21*w59-4.2613636363636367e-05*w0*Jinv02-3.0438311688311688e-05*Jinv21*w2+6.9399350649350649e-04*w43*Jinv10-4.3831168831168834e-04*w42*Jinv01-7.3051948051948055e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17+7.3051948051948052e-05*Jinv01*w45-5.2962662337662338e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02+5.4788961038961042e-05*w48*Jinv00+1.2175324675324675e-05*w58*Jinv11+1.2175324675324675e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26-3.0438311688311688e-05*w0*Jinv01+9.1314935064935063e-04*w16*Jinv10-4.3831168831168834e-04*w43*Jinv11-3.4699675324675324e-04*w45*Jinv00-1.1323051948051949e-03*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.9480519480519481e-04*Jinv00*w27-3.0438311688311688e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+1.9480519480519481e-04*Jinv20*w29-3.4699675324675324e-04*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+6.2094155844155847e-04*w31*Jinv10+4.3831168831168834e-04*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00+5.6615259740259744e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39-4.2613636363636367e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38+3.6525974025974026e-05*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-6.5746753246753243e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00-2.1915584415584417e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+5.2962662337662338e-04*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10+7.3051948051948052e-05*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56-2.5568181818181820e-04*w26*Jinv20-2.4350649350649351e-04*w11*Jinv20-4.3831168831168834e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+7.3051948051948052e-05*w31*Jinv11+7.3051948051948052e-05*w46*Jinv11+3.1047077922077924e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21-5.2962662337662338e-04*Jinv02*w54+4.3831168831168834e-04*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+5.2962662337662338e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10+7.3051948051948052e-05*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23-6.5746753246753243e-04*w52*Jinv11+5.6615259740259744e-04*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-5.6615259740259744e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01-6.5746753246753243e-04*Jinv01*w51+5.6615259740259744e-04*Jinv02*w30-5.6615259740259744e-04*Jinv12*w49+1.0957792207792207e-03*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11-2.4350649350649351e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10-4.9310064935064932e-04*Jinv21*w56-2.5568181818181820e-04*w24*Jinv00-5.4788961038961036e-04*w6*Jinv02+4.3831168831168834e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14-2.4350649350649351e-04*w10*Jinv10+6.9399350649350649e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12+3.6525974025974026e-05*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+1.8262987012987013e-03*w35*Jinv20+3.6525974025974026e-05*Jinv21*w23-4.9310064935064932e-04*Jinv01*w54-4.9310064935064932e-04*w55*Jinv11-2.5568181818181820e-04*w25*Jinv10+9.1314935064935063e-04*w15*Jinv00-5.6615259740259744e-04*w48*Jinv02-7.3051948051948055e-04*Jinv01*w9+9.1314935064935063e-04*Jinv20*w17+6.2094155844155847e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+7.3051948051948052e-05*Jinv01*w30-4.3831168831168834e-04*Jinv02*w51-5.2962662337662338e-04*Jinv22*w56-2.1915584415584417e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+6.9399350649350649e-04*w42*Jinv00-3.4699675324675324e-04*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-4.2613636363636367e-05*Jinv12*w1-7.3051948051948055e-04*w10*Jinv11)*D;
  A[10][35]=(0.0)*D;
  A[10][36]=(0.0)*D;
  A[10][37]=(6.3920454545454550e-05*Jinv00*w12-9.2329545454545453e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-4.5657467532467534e-05*Jinv20*w8+6.4935064935064935e-05*w57*Jinv02+6.3920454545454550e-05*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+1.1870941558441559e-04*w3*Jinv00+1.2784090909090910e-04*w32*Jinv20+6.3920454545454550e-05*Jinv20*w23+1.0957792207792207e-03*Jinv22*w38-2.1002435064935065e-04*Jinv00*w54-5.4788961038961036e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10+9.1314935064935065e-06*Jinv12*w55-5.6818181818181818e-05*w58*Jinv10-5.4788961038961036e-04*w15*Jinv02+1.5827922077922079e-03*w10*Jinv12-2.1002435064935065e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-5.4788961038961036e-04*Jinv22*w35-9.2329545454545453e-05*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20+1.2784090909090910e-04*w45*Jinv00-5.4788961038961036e-04*w42*Jinv02-9.2329545454545453e-05*Jinv00*w27+1.5827922077922079e-03*w11*Jinv22-2.1002435064935065e-04*Jinv20*w50-9.2329545454545453e-05*Jinv20*w29+1.2784090909090910e-04*w46*Jinv10-5.4788961038961036e-04*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12+1.2784090909090910e-04*w31*Jinv10-5.4788961038961036e-04*Jinv02*w3+6.3920454545454550e-05*w21*Jinv00+9.1314935064935065e-06*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2-4.5657467532467534e-05*Jinv00*w18-9.1314935064935068e-05*w36*Jinv00+9.1314935064935065e-06*Jinv02*w45+6.4935064935064935e-05*Jinv22*w59-1.4610389610389610e-04*w51*Jinv00-5.4788961038961036e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.1870941558441559e-04*w4*Jinv10-6.7573051948051951e-04*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13-4.5657467532467534e-05*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10-4.5657467532467534e-05*w6*Jinv00+1.0957792207792207e-03*w37*Jinv12-2.1002435064935065e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41+1.1870941558441559e-04*w26*Jinv20+2.8409090909090908e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-5.4788961038961036e-04*Jinv22*w26+6.3920454545454550e-05*Jinv20*w14+9.1314935064935065e-06*Jinv02*w54-5.4788961038961036e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-2.1002435064935065e-04*w49*Jinv10-6.7573051948051951e-04*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12+1.5827922077922079e-03*Jinv02*w9-4.5657467532467534e-05*w7*Jinv10-9.1314935064935068e-05*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23+9.1314935064935065e-06*w31*Jinv12+9.1314935064935065e-06*w46*Jinv12-1.4610389610389610e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8+6.3920454545454550e-05*w22*Jinv10+9.1314935064935065e-06*Jinv22*w50+1.0957792207792207e-03*Jinv12*w7+9.1314935064935065e-06*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49-1.4610389610389610e-04*w52*Jinv10+2.8409090909090908e-04*w9*Jinv00-9.1314935064935068e-05*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+6.4935064935064935e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-5.4788961038961036e-04*Jinv22*w5+9.1314935064935065e-06*w47*Jinv22+2.8409090909090908e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+6.4935064935064935e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18-5.6818181818181818e-05*w57*Jinv00-5.6818181818181818e-05*Jinv20*w59-5.4788961038961036e-04*Jinv02*w33-9.2329545454545453e-05*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10-9.2329545454545453e-05*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00+9.1314935064935065e-06*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.2784090909090910e-04*w30*Jinv00-6.7573051948051951e-04*w40*Jinv10-4.5657467532467534e-05*w20*Jinv20+6.4935064935064935e-05*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14+1.0957792207792207e-03*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.1870941558441559e-04*w5*Jinv20-5.4788961038961036e-04*Jinv02*w51+9.1314935064935065e-06*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28-2.1002435064935065e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00+1.2784090909090910e-04*w47*Jinv20-5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[10][38]=(0.0)*D;
  A[10][39]=(0.0)*D;
  A[10][40]=(4.2004870129870131e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10-5.2962662337662338e-04*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02+4.2004870129870131e-04*w13*Jinv10-1.1323051948051949e-03*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00+4.2004870129870131e-04*w32*Jinv20+5.2962662337662338e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-1.9480519480519481e-04*w0*Jinv02-1.9480519480519481e-04*Jinv21*w2-2.2646103896103898e-03*w43*Jinv10-1.1323051948051949e-03*w42*Jinv01-9.6793831168831166e-04*Jinv01*w45-9.6793831168831166e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10+1.1323051948051949e-03*w15*Jinv02-2.7394480519480518e-04*w48*Jinv00-5.2962662337662338e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-2.2524350649350650e-04*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10-1.1323051948051949e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-1.1323051948051949e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-1.9480519480519481e-04*Jinv11*w1+6.9399350649350649e-04*w13*Jinv11+1.9480519480519481e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+1.1323051948051949e-03*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+4.2004870129870131e-04*w31*Jinv10+5.2962662337662338e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00+6.9399350649350649e-04*w32*Jinv22-1.9480519480519481e-04*Jinv22*w2+9.8620129870129864e-04*Jinv00*w18+9.8620129870129864e-04*w36*Jinv00-2.7394480519480518e-04*Jinv02*w45-5.2962662337662338e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59+1.1323051948051949e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-4.9310064935064932e-04*w51*Jinv00-1.1323051948051949e-03*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+5.2962662337662338e-04*w4*Jinv11-2.7759740259740259e-03*w41*Jinv20+9.6793831168831166e-04*Jinv12*w13+9.8620129870129864e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10+9.6793831168831166e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-4.9310064935064932e-04*w26*Jinv20-4.8701298701298701e-04*w11*Jinv20-5.2962662337662338e-04*w52*Jinv12+1.1323051948051949e-03*Jinv21*w35+9.6793831168831166e-04*w31*Jinv11-9.6793831168831166e-04*w46*Jinv11+4.2004870129870131e-04*Jinv20*w14+5.2962662337662338e-04*w5*Jinv21-9.6793831168831166e-04*Jinv02*w54+5.2962662337662338e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+1.1323051948051949e-03*Jinv01*w33-2.7759740259740259e-03*w39*Jinv00+9.6793831168831166e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18-4.9310064935064932e-04*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10-9.6793831168831166e-04*w47*Jinv21+9.8620129870129864e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+6.9399350649350649e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-4.9310064935064932e-04*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10-6.9399350649350649e-04*Jinv22*w50-6.9399350649350649e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-6.9399350649350649e-04*w21*Jinv01+6.9399350649350649e-04*Jinv02*w30-6.9399350649350649e-04*Jinv12*w49-4.9310064935064932e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-4.8701298701298701e-04*w9*Jinv00+9.8620129870129864e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-4.9310064935064932e-04*w24*Jinv00-4.9310064935064932e-04*w6*Jinv02+5.2962662337662338e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11+6.9399350649350649e-04*Jinv21*w14-4.8701298701298701e-04*w10*Jinv10-2.2646103896103898e-03*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12+6.9399350649350649e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59-2.2524350649350650e-04*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-6.9399350649350649e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-4.9310064935064932e-04*w25*Jinv10-2.2524350649350650e-04*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-6.9399350649350649e-04*w48*Jinv02-1.1323051948051949e-03*Jinv20*w17+4.2004870129870131e-04*w30*Jinv00-2.7759740259740259e-03*w40*Jinv10+9.8620129870129864e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36-1.1323051948051949e-03*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+9.6793831168831166e-04*Jinv01*w30-5.2962662337662338e-04*Jinv02*w51-9.6793831168831166e-04*Jinv22*w56-5.2962662337662338e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-2.2646103896103898e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+1.1323051948051949e-03*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-1.9480519480519481e-04*Jinv12*w1)*D;
  A[10][41]=(0.0)*D;
  A[10][42]=(0.0)*D;
  A[10][43]=(-4.2004870129870131e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8+3.0438311688311688e-05*w57*Jinv02-4.2004870129870131e-04*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00-4.2004870129870131e-04*w32*Jinv20+2.1915584415584417e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54-1.2175324675324675e-05*Jinv21*w59+6.5746753246753243e-04*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02+3.0438311688311688e-05*Jinv21*w2+2.2646103896103898e-03*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01+7.3051948051948055e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-7.3051948051948052e-05*Jinv01*w45-7.3051948051948052e-05*Jinv12*w55+3.0438311688311688e-05*w58*Jinv10+4.3831168831168834e-04*w15*Jinv02+7.3051948051948055e-04*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+6.5746753246753243e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35-1.2175324675324675e-05*w58*Jinv11-1.2175324675324675e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01+2.2524350649350650e-04*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+3.0438311688311688e-05*Jinv00*w27+7.3051948051948055e-04*w11*Jinv22+3.0438311688311688e-05*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+3.0438311688311688e-05*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10+4.3831168831168834e-04*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33+4.9310064935064932e-04*w22*Jinv12-4.2004870129870131e-04*w31*Jinv10+2.1915584415584417e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+3.0438311688311688e-05*Jinv22*w2-9.8620129870129864e-04*Jinv00*w18-9.8620129870129864e-04*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24+3.0438311688311688e-05*Jinv22*w59+4.3831168831168834e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+6.5746753246753243e-04*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+2.1915584415584417e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10+2.1915584415584417e-04*w4*Jinv11+2.7759740259740259e-03*w41*Jinv20-7.3051948051948052e-05*Jinv12*w13-9.8620129870129864e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-7.3051948051948052e-05*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+4.8701298701298701e-04*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12+4.3831168831168834e-04*Jinv21*w35-7.3051948051948052e-05*w31*Jinv11+6.5746753246753243e-04*Jinv22*w26-7.3051948051948052e-05*w46*Jinv11-4.2004870129870131e-04*Jinv20*w14+2.1915584415584417e-04*w5*Jinv21-7.3051948051948052e-05*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12+4.9310064935064932e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10+3.0438311688311688e-05*Jinv21*w29+4.3831168831168834e-04*Jinv01*w33+2.7759740259740259e-03*w39*Jinv00-7.3051948051948052e-05*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11+7.3051948051948055e-04*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-7.3051948051948052e-05*w47*Jinv21-9.8620129870129864e-04*Jinv20*w38+4.9310064935064932e-04*Jinv22*w23+6.5746753246753243e-04*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+6.5746753246753243e-04*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11+4.8701298701298701e-04*w9*Jinv00-9.8620129870129864e-04*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00-1.2175324675324675e-05*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22+4.9310064935064932e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+4.8701298701298701e-04*w10*Jinv10+2.2646103896103898e-03*Jinv20*w44+3.0438311688311688e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+2.1915584415584417e-04*Jinv02*w33+2.2524350649350650e-04*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10+2.2524350649350650e-04*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+7.3051948051948055e-04*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17-4.2004870129870131e-04*w30*Jinv00+2.7759740259740259e-03*w40*Jinv10-9.8620129870129864e-04*w20*Jinv20-1.2175324675324675e-05*Jinv22*w29-7.3051948051948052e-05*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20-7.3051948051948052e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51-7.3051948051948052e-05*Jinv22*w56-1.2175324675324675e-05*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+2.2646103896103898e-03*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+4.3831168831168834e-04*w16*Jinv12+4.9310064935064932e-04*Jinv21*w50+3.0438311688311688e-05*Jinv01*w27+4.9310064935064932e-04*w48*Jinv01+3.0438311688311688e-05*Jinv12*w1+7.3051948051948055e-04*w10*Jinv11)*D;
  A[10][44]=(0.0)*D;
  A[10][45]=(0.0)*D;
  A[10][46]=(2.7394480519480518e-04*w40*Jinv12-1.3697240259740259e-04*Jinv22*w53+1.0247564935064935e-04*w57*Jinv02+2.7394480519480518e-04*Jinv02*w39+1.0592532467532468e-03*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3+9.1314935064935065e-06*Jinv22*w38-1.4204545454545455e-05*Jinv21*w59+7.8530844155844158e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+1.0592532467532468e-03*w42*Jinv01+4.0584415584415584e-05*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+7.4878246753246752e-04*Jinv01*w45-4.2918019480519480e-04*Jinv12*w55+4.9310064935064932e-04*w15*Jinv02+4.0584415584415584e-05*w10*Jinv12+7.8530844155844158e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+3.9265422077922079e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01-4.7483766233766234e-04*Jinv22*w20+1.0592532467532468e-03*w43*Jinv11+9.6793831168831166e-04*Jinv21*w41+4.9310064935064932e-04*w42*Jinv02+4.0584415584415584e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-3.1960227272727273e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+4.9310064935064932e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-1.0957792207792208e-04*w32*Jinv22+9.6793831168831166e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2+1.8262987012987013e-05*Jinv21*w38+7.4878246753246752e-04*Jinv02*w45+3.9265422077922079e-04*Jinv01*w24+1.0247564935064935e-04*Jinv22*w59-7.3051948051948052e-05*w34*Jinv11-4.8396915584415583e-04*w19*Jinv11-3.1047077922077924e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-4.2918019480519480e-04*Jinv12*w13-2.1915584415584417e-04*w32*Jinv21-4.8396915584415583e-04*Jinv21*w20+9.1314935064935065e-06*w37*Jinv12+2.7394480519480518e-04*Jinv22*w41-1.3697240259740259e-04*w52*Jinv12-7.3051948051948052e-05*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11+7.8530844155844158e-04*Jinv22*w26+7.4878246753246752e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21-4.2918019480519480e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33-4.2918019480519480e-04*Jinv02*w12-4.8396915584415583e-04*Jinv01*w18+1.8262987012987013e-05*w37*Jinv11+4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01+7.4878246753246752e-04*w47*Jinv21+7.4878246753246752e-04*Jinv22*w23-3.1047077922077924e-04*w52*Jinv11-1.0957792207792208e-04*w31*Jinv12+7.4878246753246752e-04*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50+3.7439123376623376e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+3.7439123376623376e-04*w21*Jinv01-3.1047077922077924e-04*Jinv01*w51-1.0957792207792208e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+1.8262987012987013e-05*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11-1.5523538961038962e-04*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02-4.7483766233766234e-04*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5+7.4878246753246752e-04*w47*Jinv22-4.2918019480519480e-04*w49*Jinv11-3.1960227272727273e-04*Jinv21*w14+1.0247564935064935e-04*w58*Jinv12-4.7483766233766234e-04*Jinv02*w18-3.1960227272727273e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.7439123376623376e-04*Jinv21*w23-1.5523538961038962e-04*Jinv01*w54-1.5523538961038962e-04*w55*Jinv11-1.0957792207792208e-04*w48*Jinv02+4.0584415584415584e-05*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+9.6793831168831166e-04*w40*Jinv11+9.1314935064935065e-06*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-2.1915584415584417e-04*Jinv01*w30-1.3697240259740259e-04*Jinv02*w51-4.2918019480519480e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.9265422077922079e-04*w25*Jinv11+4.9310064935064932e-04*w16*Jinv12-4.2918019480519480e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1+4.0584415584415584e-05*w10*Jinv11)*D;
  A[10][47]=(0.0)*D;
  A[10][48]=(0.0)*D;
  A[10][49]=(1.0957792207792208e-04*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10+6.9399350649350649e-04*w40*Jinv12-3.9265422077922079e-04*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10+6.9399350649350649e-04*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+4.2918019480519480e-04*w32*Jinv20+1.3697240259740259e-04*Jinv01*w3+1.0957792207792208e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-1.1668019480519480e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-4.0584415584415584e-05*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-3.7439123376623376e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02-7.4878246753246752e-04*w48*Jinv00+5.6615259740259744e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-4.9310064935064932e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-2.7394480519480518e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-1.0247564935064935e-04*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-7.4878246753246752e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+4.2918019480519480e-04*w31*Jinv10-1.7349837662337662e-04*Jinv02*w3+1.0957792207792208e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-2.7394480519480518e-04*Jinv01*w39-1.1668019480519480e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18+4.7483766233766234e-04*w36*Jinv00+4.7483766233766234e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11-7.8530844155844158e-04*Jinv21*w53-7.8530844155844158e-04*w51*Jinv00+5.6615259740259744e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10+1.3697240259740259e-04*w4*Jinv11-2.7394480519480518e-04*w41*Jinv20-3.1960227272727273e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+4.2918019480519480e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56+6.9399350649350649e-04*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20-3.9265422077922079e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+4.2918019480519480e-04*w31*Jinv11+4.2918019480519480e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+1.3697240259740259e-04*w5*Jinv21-3.7439123376623376e-04*Jinv02*w54-1.7349837662337662e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-7.4878246753246752e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-2.7394480519480518e-04*w39*Jinv00-3.1960227272727273e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18+4.7483766233766234e-04*w37*Jinv11-9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+4.7483766233766234e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-7.8530844155844158e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12-7.8530844155844158e-04*Jinv20*w53+9.1314935064935065e-06*Jinv22*w8+1.0957792207792208e-04*w22*Jinv10+1.0957792207792208e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+1.0957792207792208e-04*w21*Jinv01-7.8530844155844158e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+4.7483766233766234e-04*w36*Jinv01-7.8530844155844158e-04*w52*Jinv10-9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00+4.7483766233766234e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+9.1314935064935065e-06*w6*Jinv02-1.7349837662337662e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22-7.4878246753246752e-04*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.0957792207792208e-04*Jinv01*w12-9.1314935064935065e-06*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+5.6615259740259744e-04*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+1.0957792207792208e-04*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-4.0584415584415584e-05*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.2918019480519480e-04*w30*Jinv00-2.7394480519480518e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20-3.1960227272727273e-04*Jinv22*w14-2.7394480519480518e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20+4.2918019480519480e-04*Jinv01*w30-3.9265422077922079e-04*Jinv02*w51-3.7439123376623376e-04*Jinv22*w56+1.3697240259740259e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12-7.4878246753246752e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-7.4878246753246752e-04*w48*Jinv01-1.1668019480519480e-04*Jinv12*w1-4.0584415584415584e-05*w10*Jinv11)*D;
  A[10][50]=(0.0)*D;
  A[10][51]=(0.0)*D;
  A[10][52]=(-1.0957792207792208e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+9.1314935064935065e-06*Jinv20*w8-5.6818181818181818e-05*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39-1.3697240259740259e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+7.4878246753246752e-04*Jinv00*w54+2.1915584415584417e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02+4.9310064935064932e-04*w43*Jinv10+3.9265422077922079e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02-6.0876623376623375e-04*w10*Jinv12+7.4878246753246752e-04*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-4.2918019480519480e-04*w45*Jinv00+4.3831168831168834e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27-6.0876623376623375e-04*w11*Jinv22+7.4878246753246752e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-4.2918019480519480e-04*w31*Jinv10+2.1915584415584417e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-1.7349837662337662e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2+9.1314935064935065e-06*Jinv00*w18-4.7483766233766234e-04*w36*Jinv00-1.7349837662337662e-04*Jinv02*w45-5.6818181818181818e-05*Jinv22*w59+7.8530844155844158e-04*w51*Jinv00+4.3831168831168834e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv10+2.7394480519480518e-04*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13+9.1314935064935065e-06*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10+9.1314935064935065e-06*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+7.4878246753246752e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41-1.3697240259740259e-04*w26*Jinv20+4.0584415584415584e-05*w11*Jinv20+6.5746753246753243e-04*w52*Jinv12+2.1915584415584417e-04*Jinv22*w26-1.0957792207792208e-04*Jinv20*w14+3.9265422077922079e-04*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+7.4878246753246752e-04*w49*Jinv10+2.7394480519480518e-04*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12-6.0876623376623375e-04*Jinv02*w9+9.1314935064935065e-06*w7*Jinv10-4.7483766233766234e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-1.7349837662337662e-04*w31*Jinv12-1.7349837662337662e-04*w46*Jinv12+7.8530844155844158e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+3.9265422077922079e-04*Jinv22*w50-5.4788961038961036e-04*Jinv12*w7-1.7349837662337662e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49+7.8530844155844158e-04*w52*Jinv10+4.0584415584415584e-05*w9*Jinv00-4.7483766233766234e-04*w37*Jinv10-1.3697240259740259e-04*w24*Jinv00-1.4204545454545455e-05*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5-1.7349837662337662e-04*w47*Jinv22+4.0584415584415584e-05*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-5.6818181818181818e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+4.3831168831168834e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.3697240259740259e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.9265422077922079e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+2.7394480519480518e-04*w40*Jinv10+9.1314935064935065e-06*w20*Jinv20-1.4204545454545455e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14-1.0957792207792207e-03*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-1.3697240259740259e-04*w5*Jinv20+6.5746753246753243e-04*Jinv02*w51+3.9265422077922079e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+7.4878246753246752e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[10][53]=(0.0)*D;
  A[10][54]=(0.0)*D;
  A[10][55]=(9.6793831168831166e-04*w40*Jinv12+3.9265422077922079e-04*Jinv22*w53-9.2329545454545453e-05*w57*Jinv02+9.6793831168831166e-04*Jinv02*w39+4.9310064935064932e-04*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3-4.8396915584415583e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-3.1047077922077924e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w42*Jinv01+4.0584415584415584e-05*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.2918019480519480e-04*Jinv01*w45+7.4878246753246752e-04*Jinv12*w55-7.3051948051948052e-05*w15*Jinv02+4.0584415584415584e-05*w10*Jinv12-3.1047077922077924e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.3697240259740259e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.8262987012987013e-05*Jinv22*w20+4.9310064935064932e-04*w43*Jinv11+2.7394480519480518e-04*Jinv21*w41+1.0592532467532468e-03*w42*Jinv02+4.0584415584415584e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-1.0957792207792208e-04*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-7.3051948051948052e-05*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-3.1960227272727273e-04*w32*Jinv22+2.7394480519480518e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2-4.7483766233766234e-04*Jinv21*w38-1.5523538961038962e-04*Jinv02*w45-1.3697240259740259e-04*Jinv01*w24-9.2329545454545453e-05*Jinv22*w59+4.9310064935064932e-04*w34*Jinv11+9.1314935064935065e-06*w19*Jinv11+7.8530844155844158e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+1.0592532467532468e-03*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-2.1915584415584417e-04*Jinv12*w13-4.2918019480519480e-04*w32*Jinv21+9.1314935064935065e-06*Jinv21*w20-4.8396915584415583e-04*w37*Jinv12+9.6793831168831166e-04*Jinv22*w41+3.9265422077922079e-04*w52*Jinv12+4.9310064935064932e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-3.1047077922077924e-04*Jinv22*w26-4.2918019480519480e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21+7.4878246753246752e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-2.1915584415584417e-04*Jinv02*w12+9.1314935064935065e-06*Jinv01*w18-4.7483766233766234e-04*w37*Jinv11+4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-4.2918019480519480e-04*w47*Jinv21-4.2918019480519480e-04*Jinv22*w23+7.8530844155844158e-04*w52*Jinv11-3.1960227272727273e-04*w31*Jinv12-1.5523538961038962e-04*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8+3.7439123376623376e-04*Jinv22*w50-1.0957792207792208e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7-1.0957792207792208e-04*w21*Jinv01+7.8530844155844158e-04*Jinv01*w51-3.1960227272727273e-04*Jinv02*w30+3.7439123376623376e-04*Jinv12*w49-4.7483766233766234e-04*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11+7.4878246753246752e-04*Jinv21*w56-1.4204545454545455e-05*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02+1.8262987012987013e-05*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5-1.5523538961038962e-04*w47*Jinv22+7.4878246753246752e-04*w49*Jinv11-1.0957792207792208e-04*Jinv21*w14-9.2329545454545453e-05*w58*Jinv12+1.8262987012987013e-05*Jinv02*w18-1.0957792207792208e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33-1.0957792207792208e-04*Jinv21*w23+7.4878246753246752e-04*Jinv01*w54+7.4878246753246752e-04*w55*Jinv11+3.7439123376623376e-04*w48*Jinv02+4.0584415584415584e-05*Jinv01*w9-1.4204545454545455e-05*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+2.7394480519480518e-04*w40*Jinv11-4.8396915584415583e-04*Jinv02*w36+1.0592532467532468e-03*Jinv12*w43-4.2918019480519480e-04*Jinv01*w30+3.9265422077922079e-04*Jinv02*w51+7.4878246753246752e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28-1.3697240259740259e-04*w25*Jinv11-7.3051948051948052e-05*w16*Jinv12+7.4878246753246752e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1+4.0584415584415584e-05*w10*Jinv11)*D;
  A[10][56]=(0.0)*D;
  A[10][57]=(0.0)*D;
  A[10][58]=(-1.9480519480519481e-04*w40*Jinv12+2.0495129870129870e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-1.9480519480519481e-04*Jinv02*w39+8.3198051948051951e-05*Jinv22*w38-1.4204545454545455e-05*w25*Jinv12+5.4112554112554113e-05*w0*Jinv02+1.5726461038961039e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+1.8939393939393939e-04*w10*Jinv12-1.4204545454545455e-05*Jinv02*w24-4.2613636363636367e-05*Jinv22*w35+6.4935064935064935e-05*Jinv22*w20-4.2613636363636367e-05*w42*Jinv02+1.8939393939393939e-04*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17+1.0247564935064935e-04*w22*Jinv12-1.4204545454545455e-05*Jinv02*w3-1.1668019480519480e-04*w32*Jinv22+5.4112554112554113e-05*Jinv22*w2-1.1668019480519480e-04*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-4.2613636363636367e-05*Jinv12*w34-4.2613636363636367e-05*Jinv22*w44+1.0247564935064935e-04*Jinv12*w13+8.3198051948051951e-05*w37*Jinv12-1.9480519480519481e-04*Jinv22*w41+2.0495129870129870e-04*w52*Jinv12-1.4204545454545455e-05*Jinv22*w26+1.5726461038961039e-04*Jinv02*w54-1.4204545454545455e-05*w4*Jinv12+1.0247564935064935e-04*w21*Jinv02+1.0247564935064935e-04*Jinv02*w12+1.8939393939393939e-04*Jinv02*w9+1.0247564935064935e-04*Jinv22*w23-1.1668019480519480e-04*w31*Jinv12-1.1668019480519480e-04*w46*Jinv12+6.4935064935064935e-05*Jinv22*w8+1.5726461038961039e-04*Jinv22*w50+6.4935064935064935e-05*Jinv12*w7-1.1668019480519480e-04*Jinv02*w30+1.5726461038961039e-04*Jinv12*w49+5.4112554112554113e-05*Jinv02*w27+6.4935064935064935e-05*w6*Jinv02+6.4935064935064935e-05*w19*Jinv12-1.4204545454545455e-05*Jinv22*w5-1.1668019480519480e-04*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12+6.4935064935064935e-05*Jinv02*w18-4.2613636363636367e-05*Jinv02*w33+1.5726461038961039e-04*w48*Jinv02+5.4112554112554113e-05*Jinv22*w29+1.0247564935064935e-04*Jinv22*w14+8.3198051948051951e-05*Jinv02*w36-4.2613636363636367e-05*Jinv12*w43+2.0495129870129870e-04*Jinv02*w51+1.5726461038961039e-04*Jinv22*w56+5.4112554112554113e-05*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+5.4112554112554113e-05*Jinv12*w1)*D;
  A[10][59]=(0.0)*D;
  A[11][0]=(0.0)*D;
  A[11][1]=(0.0)*D;
  A[11][2]=(-1.5726461038961039e-04*Jinv00*w12-5.4112554112554113e-05*w28*Jinv10+1.9480519480519481e-04*w40*Jinv12+1.4204545454545455e-05*Jinv22*w53-8.3198051948051951e-05*Jinv20*w8-5.4112554112554113e-05*w57*Jinv02-1.5726461038961039e-04*w13*Jinv10+1.9480519480519481e-04*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-2.0495129870129870e-04*w3*Jinv00-1.5726461038961039e-04*w32*Jinv20-2.0495129870129870e-04*Jinv01*w3+1.1668019480519480e-04*Jinv20*w23-6.4935064935064935e-05*Jinv22*w38-1.0247564935064935e-04*Jinv00*w54-5.4112554112554113e-05*Jinv21*w59+1.4204545454545455e-05*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-1.8939393939393939e-04*w11*Jinv21+4.2613636363636367e-05*w16*Jinv11+4.2613636363636367e-05*Jinv21*w17-1.0247564935064935e-04*Jinv01*w45-1.0247564935064935e-04*Jinv12*w55-5.4112554112554113e-05*w58*Jinv10+4.2613636363636367e-05*w15*Jinv02-1.8939393939393939e-04*w10*Jinv12+1.1668019480519480e-04*w48*Jinv00+1.4204545454545455e-05*Jinv02*w24+4.2613636363636367e-05*Jinv22*w35-5.4112554112554113e-05*w58*Jinv11-5.4112554112554113e-05*w57*Jinv01+1.4204545454545455e-05*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+4.2613636363636367e-05*w16*Jinv10-6.4935064935064935e-05*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11-1.0247564935064935e-04*w45*Jinv00+1.9480519480519481e-04*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-5.4112554112554113e-05*Jinv00*w27-1.8939393939393939e-04*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-1.5726461038961039e-04*w13*Jinv11+4.2613636363636367e-05*w15*Jinv01-5.4112554112554113e-05*w28*Jinv11+1.1668019480519480e-04*Jinv20*w50-5.4112554112554113e-05*Jinv20*w29-1.0247564935064935e-04*w46*Jinv10+4.2613636363636367e-05*Jinv22*w17+4.2613636363636367e-05*Jinv00*w33+1.1668019480519480e-04*w22*Jinv12-1.5726461038961039e-04*w31*Jinv10-2.0495129870129870e-04*Jinv02*w3+1.1668019480519480e-04*w21*Jinv00-1.5726461038961039e-04*w32*Jinv22+1.9480519480519481e-04*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2-6.4935064935064935e-05*Jinv00*w18-6.4935064935064935e-05*w36*Jinv00-6.4935064935064935e-05*Jinv21*w38-1.0247564935064935e-04*Jinv02*w45+1.4204545454545455e-05*Jinv01*w24-5.4112554112554113e-05*Jinv22*w59+4.2613636363636367e-05*w34*Jinv11-6.4935064935064935e-05*w19*Jinv11+1.4204545454545455e-05*Jinv21*w53+1.4204545454545455e-05*w51*Jinv00+4.2613636363636367e-05*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-2.0495129870129870e-04*w4*Jinv10-2.0495129870129870e-04*w4*Jinv11+1.9480519480519481e-04*w41*Jinv20-1.5726461038961039e-04*Jinv12*w13-6.4935064935064935e-05*w19*Jinv10+4.2613636363636367e-05*w34*Jinv10-1.5726461038961039e-04*w32*Jinv21-6.4935064935064935e-05*Jinv21*w20-8.3198051948051951e-05*w6*Jinv00-6.4935064935064935e-05*w37*Jinv12-1.0247564935064935e-04*Jinv20*w56+1.9480519480519481e-04*Jinv22*w41+1.4204545454545455e-05*w26*Jinv20-1.8939393939393939e-04*w11*Jinv20+1.4204545454545455e-05*w52*Jinv12+4.2613636363636367e-05*Jinv21*w35-1.5726461038961039e-04*w31*Jinv11+1.4204545454545455e-05*Jinv22*w26-1.0247564935064935e-04*w46*Jinv11-1.5726461038961039e-04*Jinv20*w14-2.0495129870129870e-04*w5*Jinv21-1.0247564935064935e-04*Jinv02*w54-2.0495129870129870e-04*w4*Jinv12+1.1668019480519480e-04*w21*Jinv02+1.1668019480519480e-04*w49*Jinv10-5.4112554112554113e-05*Jinv21*w29+4.2613636363636367e-05*Jinv01*w33+1.9480519480519481e-04*w39*Jinv00-1.5726461038961039e-04*Jinv02*w12-6.4935064935064935e-05*Jinv01*w18-6.4935064935064935e-05*w37*Jinv11-1.8939393939393939e-04*Jinv02*w9-8.3198051948051951e-05*w6*Jinv01-8.3198051948051951e-05*w7*Jinv10-1.0247564935064935e-04*w47*Jinv21-6.4935064935064935e-05*Jinv20*w38+1.1668019480519480e-04*Jinv22*w23+1.4204545454545455e-05*w52*Jinv11-1.5726461038961039e-04*w31*Jinv12-1.0247564935064935e-04*w46*Jinv12+1.4204545454545455e-05*Jinv20*w53-8.3198051948051951e-05*Jinv22*w8+1.1668019480519480e-04*w22*Jinv10+1.1668019480519480e-04*Jinv22*w50+1.1668019480519480e-04*w22*Jinv11-8.3198051948051951e-05*Jinv12*w7+1.1668019480519480e-04*w21*Jinv01+1.4204545454545455e-05*Jinv01*w51-1.5726461038961039e-04*Jinv02*w30+1.1668019480519480e-04*Jinv12*w49-6.4935064935064935e-05*w36*Jinv01+1.4204545454545455e-05*w52*Jinv10-8.3198051948051951e-05*w7*Jinv11-1.8939393939393939e-04*w9*Jinv00-6.4935064935064935e-05*w37*Jinv10-1.0247564935064935e-04*Jinv21*w56+1.4204545454545455e-05*w24*Jinv00-5.4112554112554113e-05*Jinv02*w27-8.3198051948051951e-05*w6*Jinv02-6.4935064935064935e-05*w19*Jinv12-2.0495129870129870e-04*Jinv22*w5-1.0247564935064935e-04*w47*Jinv22+1.1668019480519480e-04*w49*Jinv11-1.5726461038961039e-04*Jinv21*w14-1.8939393939393939e-04*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-5.4112554112554113e-05*w58*Jinv12-6.4935064935064935e-05*Jinv02*w18-1.5726461038961039e-04*Jinv01*w12-8.3198051948051951e-05*Jinv21*w8-5.4112554112554113e-05*w57*Jinv00-5.4112554112554113e-05*Jinv20*w59+4.2613636363636367e-05*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+4.2613636363636367e-05*w35*Jinv20+1.1668019480519480e-04*Jinv21*w23-1.0247564935064935e-04*Jinv01*w54-1.0247564935064935e-04*w55*Jinv11+1.4204545454545455e-05*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+4.2613636363636367e-05*w15*Jinv00+1.1668019480519480e-04*w48*Jinv02-1.8939393939393939e-04*Jinv01*w9+4.2613636363636367e-05*Jinv20*w17-1.5726461038961039e-04*w30*Jinv00+1.9480519480519481e-04*w40*Jinv10-6.4935064935064935e-05*w20*Jinv20-5.4112554112554113e-05*Jinv22*w29-1.5726461038961039e-04*Jinv22*w14+1.9480519480519481e-04*w40*Jinv11-6.4935064935064935e-05*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-2.0495129870129870e-04*w5*Jinv20-1.5726461038961039e-04*Jinv01*w30+1.4204545454545455e-05*Jinv02*w51-1.0247564935064935e-04*Jinv22*w56-5.4112554112554113e-05*Jinv12*w28+1.4204545454545455e-05*w25*Jinv11-1.0247564935064935e-04*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00-1.0247564935064935e-04*w47*Jinv20+4.2613636363636367e-05*w16*Jinv12+1.1668019480519480e-04*Jinv21*w50-5.4112554112554113e-05*Jinv01*w27+1.1668019480519480e-04*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-1.8939393939393939e-04*w10*Jinv11)*D;
  A[11][3]=(0.0)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(3.5612824675324673e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10-5.2962662337662338e-04*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53+6.2094155844155847e-04*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+1.2784090909090910e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.5568181818181820e-04*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38+2.7394480519480521e-05*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59-2.1915584415584417e-04*w25*Jinv12+5.6818181818181818e-05*w0*Jinv02+5.6818181818181818e-05*Jinv21*w2-2.5568181818181820e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+6.0876623376623375e-04*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+6.0876623376623375e-04*w10*Jinv12-2.5568181818181820e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26+5.6818181818181818e-05*w0*Jinv01+1.2175324675324675e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11+2.7394480519480521e-05*w45*Jinv00-5.2962662337662338e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27+6.0876623376623375e-04*w11*Jinv22+5.6818181818181818e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11-2.5568181818181820e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29+2.7394480519480521e-05*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.7349837662337662e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3-2.5568181818181820e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+5.6818181818181818e-05*Jinv22*w2+5.5702110389610384e-04*Jinv00*w18+5.5702110389610384e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+1.2784090909090910e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11-2.5568181818181820e-04*w41*Jinv20-3.9265422077922079e-04*Jinv12*w13+5.5702110389610384e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-3.9265422077922079e-04*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20+6.2094155844155847e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12+2.7394480519480521e-05*Jinv20*w56-5.2962662337662338e-04*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-3.9265422077922079e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12+1.7349837662337662e-04*w21*Jinv02-2.5568181818181820e-04*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-2.5568181818181820e-04*w39*Jinv00-3.9265422077922079e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+6.0876623376623375e-04*Jinv02*w9+1.0957792207792207e-03*w6*Jinv01+6.2094155844155847e-04*w7*Jinv10+1.3697240259740259e-04*w47*Jinv21+5.5702110389610384e-04*Jinv20*w38+1.7349837662337662e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8-2.5568181818181820e-04*w22*Jinv10+1.7349837662337662e-04*Jinv22*w50+1.7349837662337662e-04*w22*Jinv11+1.0957792207792207e-03*Jinv12*w7+1.7349837662337662e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+1.7349837662337662e-04*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+6.4935064935064935e-04*w9*Jinv00+5.5702110389610384e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+1.7349837662337662e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+6.4935064935064935e-04*w10*Jinv10-2.5568181818181820e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33+1.2175324675324675e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+1.7349837662337662e-04*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10+1.2175324675324675e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+1.7349837662337662e-04*w48*Jinv02+6.0876623376623375e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00-2.5568181818181820e-04*w40*Jinv10+5.5702110389610384e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-3.9265422077922079e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+1.2784090909090910e-04*w5*Jinv20-3.9265422077922079e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28-2.1915584415584417e-04*w25*Jinv11+2.7394480519480521e-05*w55*Jinv10-2.5568181818181820e-04*w42*Jinv00+2.7394480519480521e-05*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+1.7349837662337662e-04*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+1.7349837662337662e-04*w48*Jinv01+5.6818181818181818e-05*Jinv12*w1+6.0876623376623375e-04*w10*Jinv11)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(0.0)*D;
  A[11][8]=(-2.1915584415584417e-04*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10+4.9310064935064932e-04*w40*Jinv12+5.4788961038961036e-04*Jinv22*w53-1.1870941558441558e-03*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02-2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv02*w39+5.4788961038961036e-04*Jinv21*w44+4.0178571428571428e-04*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3+1.1870941558441559e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12-6.4935064935064935e-05*w0*Jinv02-6.4935064935064935e-05*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+5.4788961038961036e-04*w42*Jinv01-1.5827922077922079e-03*w11*Jinv21+5.4788961038961036e-04*w16*Jinv11+5.4788961038961036e-04*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-1.5827922077922079e-03*w10*Jinv12+1.1870941558441559e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+5.4788961038961036e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+5.4788961038961036e-04*Jinv21*w26-6.4935064935064935e-05*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+5.4788961038961036e-04*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+4.9310064935064932e-04*Jinv21*w41+5.4788961038961036e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-1.5827922077922079e-03*w11*Jinv22-6.4935064935064935e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+5.4788961038961036e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11+1.1870941558441559e-04*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-9.1314935064935065e-06*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+1.1870941558441559e-04*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-6.4935064935064935e-05*Jinv22*w2-1.1414366883116883e-03*Jinv00*w18-1.1414366883116883e-03*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45+5.4788961038961036e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+6.6659902597402602e-04*w51*Jinv00+5.4788961038961036e-04*Jinv12*w34+5.4788961038961036e-04*Jinv22*w44+4.0178571428571428e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-1.8262987012987014e-04*w41*Jinv20-9.1314935064935065e-06*Jinv12*w13-1.1414366883116883e-03*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10-9.1314935064935065e-06*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20-1.1870941558441558e-03*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+4.9310064935064932e-04*Jinv22*w41+6.6659902597402602e-04*w26*Jinv20-1.2987012987012987e-03*w11*Jinv20+5.4788961038961036e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35-9.1314935064935065e-06*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-9.1314935064935065e-06*w21*Jinv02+1.1870941558441559e-04*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-1.8262987012987014e-04*w39*Jinv00-9.1314935064935065e-06*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11-1.5827922077922079e-03*Jinv02*w9-1.0957792207792207e-03*w6*Jinv01-1.1870941558441558e-03*w7*Jinv10-9.1314935064935065e-06*w47*Jinv21-1.1414366883116883e-03*Jinv20*w38-9.1314935064935065e-06*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+6.6659902597402602e-04*Jinv20*w53-1.0957792207792207e-03*Jinv22*w8+1.1870941558441559e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11-1.0957792207792207e-03*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+6.6659902597402602e-04*w52*Jinv10-1.0957792207792207e-03*w7*Jinv11-1.2987012987012987e-03*w9*Jinv00-1.1414366883116883e-03*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+6.6659902597402602e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-1.0957792207792207e-03*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22-9.1314935064935065e-06*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-1.2987012987012987e-03*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-1.0957792207792207e-03*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59+5.4788961038961036e-04*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+6.6659902597402602e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-1.5827922077922079e-03*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-1.8262987012987014e-04*w40*Jinv10-1.1414366883116883e-03*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29-9.1314935064935065e-06*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+5.4788961038961036e-04*Jinv12*w43+4.0178571428571428e-04*w5*Jinv20-9.1314935064935065e-06*Jinv01*w30+5.4788961038961036e-04*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12-9.1314935064935065e-06*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-9.1314935064935065e-06*w48*Jinv01-6.4935064935064935e-05*Jinv12*w1-1.5827922077922079e-03*w10*Jinv11)*D;
  A[11][9]=(0.0)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(2.0292207792207792e-05*Jinv00*w12+9.4696969696969697e-05*w28*Jinv10+6.4935064935064935e-04*Jinv20*w8+2.0292207792207792e-05*w13*Jinv10-3.2467532467532468e-04*w3*Jinv00+2.0292207792207792e-05*w32*Jinv20+2.0292207792207792e-05*Jinv20*w23+2.0292207792207792e-05*Jinv00*w54+1.2175324675324675e-04*w43*Jinv10+9.4696969696969697e-05*w58*Jinv10+2.0292207792207792e-05*w48*Jinv00+9.4696969696969697e-05*w1*Jinv10+1.2175324675324675e-04*w16*Jinv10+2.0292207792207792e-05*w45*Jinv00+9.4696969696969697e-05*Jinv00*w27+2.0292207792207792e-05*Jinv20*w50+9.4696969696969697e-05*Jinv20*w29+2.0292207792207792e-05*w46*Jinv10+1.2175324675324675e-04*Jinv00*w33+2.0292207792207792e-05*w31*Jinv10+2.0292207792207792e-05*w21*Jinv00+6.4935064935064935e-04*Jinv00*w18+6.4935064935064935e-04*w36*Jinv00-3.2467532467532468e-04*w51*Jinv00-3.2467532467532468e-04*w4*Jinv10+2.4350649350649351e-04*w41*Jinv20+6.4935064935064935e-04*w19*Jinv10+1.2175324675324675e-04*w34*Jinv10+6.4935064935064935e-04*w6*Jinv00+2.0292207792207792e-05*Jinv20*w56-3.2467532467532468e-04*w26*Jinv20+8.3874458874458877e-04*w11*Jinv20+2.0292207792207792e-05*Jinv20*w14+2.0292207792207792e-05*w49*Jinv10+2.4350649350649351e-04*w39*Jinv00+6.4935064935064935e-04*w7*Jinv10+6.4935064935064935e-04*Jinv20*w38-3.2467532467532468e-04*Jinv20*w53+2.0292207792207792e-05*w22*Jinv10-3.2467532467532468e-04*w52*Jinv10+8.3874458874458877e-04*w9*Jinv00+6.4935064935064935e-04*w37*Jinv10-3.2467532467532468e-04*w24*Jinv00+8.3874458874458877e-04*w10*Jinv10+1.2175324675324675e-04*Jinv20*w44+9.4696969696969697e-05*w57*Jinv00+9.4696969696969697e-05*Jinv20*w59+9.4696969696969697e-05*w0*Jinv00+1.2175324675324675e-04*w35*Jinv20-3.2467532467532468e-04*w25*Jinv10+9.4696969696969697e-05*Jinv20*w2+1.2175324675324675e-04*w15*Jinv00+1.2175324675324675e-04*Jinv20*w17+2.0292207792207792e-05*w30*Jinv00+2.4350649350649351e-04*w40*Jinv10+6.4935064935064935e-04*w20*Jinv20-3.2467532467532468e-04*w5*Jinv20+2.0292207792207792e-05*w55*Jinv10+1.2175324675324675e-04*w42*Jinv00+2.0292207792207792e-05*w47*Jinv20)*D;
  A[11][12]=(0.0)*D;
  A[11][13]=(0.0)*D;
  A[11][14]=(-7.4878246753246752e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-9.6793831168831166e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53+4.8396915584415583e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02-7.4878246753246752e-04*w13*Jinv10-9.6793831168831166e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-3.7439123376623376e-04*w32*Jinv20+3.9265422077922079e-04*Jinv01*w3+1.5523538961038962e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54+3.1047077922077924e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+1.5726461038961039e-04*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01-5.6615259740259744e-04*w16*Jinv11-5.6615259740259744e-04*Jinv21*w17+3.1960227272727273e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-1.0592532467532468e-03*w15*Jinv02-4.0584415584415584e-05*w10*Jinv12+3.1960227272727273e-04*w48*Jinv00+3.1047077922077924e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+1.7349837662337662e-04*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-1.0592532467532468e-03*w16*Jinv10-1.8262987012987013e-05*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-6.9399350649350649e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-4.0584415584415584e-05*w11*Jinv22+1.5726461038961039e-04*Jinv11*w1-5.6615259740259744e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+3.1960227272727273e-04*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-1.0592532467532468e-03*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.5523538961038962e-04*w22*Jinv12-3.7439123376623376e-04*w31*Jinv10-3.9265422077922079e-04*Jinv02*w3+1.5523538961038962e-04*w21*Jinv00-3.7439123376623376e-04*w32*Jinv22-6.9399350649350649e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2-1.8262987012987013e-05*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00+4.2918019480519480e-04*Jinv02*w45+1.7349837662337662e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+1.3697240259740259e-04*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10+3.9265422077922079e-04*w4*Jinv11-9.6793831168831166e-04*w41*Jinv20-7.4878246753246752e-04*Jinv12*w13-1.8262987012987013e-05*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.7439123376623376e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20+4.8396915584415583e-04*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-9.6793831168831166e-04*Jinv22*w41+3.1047077922077924e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35+3.7439123376623376e-04*w31*Jinv11+3.1047077922077924e-04*Jinv22*w26+3.1960227272727273e-04*w46*Jinv11-7.4878246753246752e-04*Jinv20*w14+3.9265422077922079e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54-3.9265422077922079e-04*w4*Jinv12+1.5523538961038962e-04*w21*Jinv02+3.1960227272727273e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-9.6793831168831166e-04*w39*Jinv00-7.4878246753246752e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01+4.8396915584415583e-04*w7*Jinv10+3.1960227272727273e-04*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38+1.5523538961038962e-04*Jinv22*w23-3.7439123376623376e-04*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53+4.8396915584415583e-04*Jinv22*w8+1.5523538961038962e-04*w22*Jinv10+3.1960227272727273e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+4.8396915584415583e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-3.7439123376623376e-04*Jinv02*w30+3.1960227272727273e-04*Jinv12*w49+1.3697240259740259e-04*w52*Jinv10+9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+3.1047077922077924e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+4.8396915584415583e-04*w6*Jinv02-1.8262987012987013e-05*w19*Jinv12-3.9265422077922079e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-4.0584415584415584e-05*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-1.8262987012987013e-05*Jinv02*w18+9.1314935064935065e-06*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+3.1047077922077924e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-1.0592532467532468e-03*w15*Jinv00+3.1960227272727273e-04*w48*Jinv02-1.0592532467532468e-03*Jinv20*w17-3.7439123376623376e-04*w30*Jinv00-9.6793831168831166e-04*w40*Jinv10-1.8262987012987013e-05*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-7.4878246753246752e-04*Jinv22*w14-6.9399350649350649e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20+3.7439123376623376e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+1.7349837662337662e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-1.0592532467532468e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1)*D;
  A[11][15]=(0.0)*D;
  A[11][16]=(0.0)*D;
  A[11][17]=(6.2094155844155847e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10-1.1323051948051949e-03*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+6.2094155844155847e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+3.1047077922077924e-04*w32*Jinv20+4.3831168831168834e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38-3.4699675324675324e-04*Jinv00*w54-6.5746753246753243e-04*w25*Jinv12-3.0438311688311688e-05*w0*Jinv02-4.2613636363636367e-05*Jinv21*w2+6.9399350649350649e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-5.2962662337662338e-04*Jinv01*w45+7.3051948051948052e-05*Jinv12*w55+1.9480519480519481e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02-7.3051948051948055e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-6.5746753246753243e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-4.3831168831168834e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01+1.8262987012987013e-03*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00-4.3831168831168834e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27-7.3051948051948055e-04*w11*Jinv22-4.2613636363636367e-05*Jinv11*w1+5.6615259740259744e-04*w13*Jinv11+4.2613636363636367e-05*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+9.1314935064935063e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+3.1047077922077924e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22-3.0438311688311688e-05*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-4.9310064935064932e-04*Jinv02*w45-4.3831168831168834e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-2.5568181818181820e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-4.3831168831168834e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.3831168831168834e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+7.3051948051948052e-05*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10+9.1314935064935063e-04*w34*Jinv10+5.2962662337662338e-04*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12-3.4699675324675324e-04*Jinv20*w56-1.1323051948051949e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-2.4350649350649351e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+5.2962662337662338e-04*w31*Jinv11-6.5746753246753243e-04*Jinv22*w26-5.2962662337662338e-04*w46*Jinv11+6.2094155844155847e-04*Jinv20*w14+4.3831168831168834e-04*w5*Jinv21+7.3051948051948052e-05*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+7.3051948051948052e-05*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18-7.3051948051948055e-04*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10-5.2962662337662338e-04*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-4.9310064935064932e-04*w46*Jinv12-2.5568181818181820e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-5.6615259740259744e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-5.6615259740259744e-04*w21*Jinv01+3.6525974025974026e-05*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49-2.5568181818181820e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11-2.4350649350649351e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+3.6525974025974026e-05*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+1.2175324675324675e-05*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5-4.9310064935064932e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+5.6615259740259744e-04*Jinv21*w14-2.4350649350649351e-04*w10*Jinv10+6.9399350649350649e-04*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18+5.6615259740259744e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+1.9480519480519481e-04*w57*Jinv00+1.9480519480519481e-04*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33+9.1314935064935063e-04*w35*Jinv20-5.6615259740259744e-04*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10+1.8262987012987013e-03*w15*Jinv00+3.6525974025974026e-05*w48*Jinv02+1.8262987012987013e-03*Jinv20*w17+3.1047077922077924e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20+1.2175324675324675e-05*Jinv22*w29+7.3051948051948052e-05*Jinv22*w14+5.4788961038961036e-04*Jinv02*w36-4.3831168831168834e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+5.2962662337662338e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51+7.3051948051948052e-05*Jinv22*w56+1.2175324675324675e-05*Jinv12*w28-4.3831168831168834e-04*w25*Jinv11-3.4699675324675324e-04*w55*Jinv10+6.9399350649350649e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-3.0438311688311688e-05*Jinv12*w1)*D;
  A[11][18]=(0.0)*D;
  A[11][19]=(0.0)*D;
  A[11][20]=(1.2784090909090910e-04*Jinv00*w12-5.6818181818181818e-05*w28*Jinv10-4.5657467532467534e-05*Jinv20*w8+1.2784090909090910e-04*w13*Jinv10-5.4788961038961036e-04*Jinv21*w44+1.1870941558441559e-04*w3*Jinv00+6.3920454545454550e-05*w32*Jinv20-5.4788961038961036e-04*Jinv01*w3-2.1002435064935065e-04*Jinv20*w23+1.2784090909090910e-04*Jinv00*w54+6.4935064935064935e-05*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.5827922077922079e-03*w11*Jinv21-5.4788961038961036e-04*w16*Jinv11-5.4788961038961036e-04*Jinv21*w17+9.1314935064935065e-06*Jinv01*w45-9.2329545454545453e-05*w58*Jinv10+6.3920454545454550e-05*w48*Jinv00+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01-9.2329545454545453e-05*w1*Jinv10-7.6704545454545450e-04*w16*Jinv10-5.4788961038961036e-04*w43*Jinv11-2.1002435064935065e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41-5.6818181818181818e-05*Jinv00*w27+6.4935064935064935e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-5.4788961038961036e-04*w15*Jinv01+6.4935064935064935e-05*w28*Jinv11+6.3920454545454550e-05*Jinv20*w50-5.6818181818181818e-05*Jinv20*w29-2.1002435064935065e-04*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+6.3920454545454550e-05*w31*Jinv10-2.1002435064935065e-04*w21*Jinv00-4.9310064935064932e-04*Jinv01*w39-9.1314935064935068e-05*Jinv00*w18-4.5657467532467534e-05*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38-5.4788961038961036e-04*Jinv01*w24-5.4788961038961036e-04*w34*Jinv11+1.0957792207792207e-03*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53+1.1870941558441559e-04*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10-5.4788961038961036e-04*w4*Jinv11-6.7573051948051951e-04*w41*Jinv20-9.1314935064935068e-05*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+9.1314935064935065e-06*w32*Jinv21+1.0957792207792207e-03*Jinv21*w20-4.5657467532467534e-05*w6*Jinv00+1.2784090909090910e-04*Jinv20*w56-1.4610389610389610e-04*w26*Jinv20+2.8409090909090908e-04*w11*Jinv20-5.4788961038961036e-04*Jinv21*w35+9.1314935064935065e-06*w31*Jinv11+9.1314935064935065e-06*w46*Jinv11+1.2784090909090910e-04*Jinv20*w14-5.4788961038961036e-04*w5*Jinv21+6.3920454545454550e-05*w49*Jinv10+6.4935064935064935e-05*Jinv21*w29-5.4788961038961036e-04*Jinv01*w33-6.7573051948051951e-04*w39*Jinv00+1.0957792207792207e-03*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+1.0957792207792207e-03*w6*Jinv01-4.5657467532467534e-05*w7*Jinv10+9.1314935064935065e-06*w47*Jinv21-4.5657467532467534e-05*Jinv20*w38-5.4788961038961036e-04*w52*Jinv11+1.1870941558441559e-04*Jinv20*w53-2.1002435064935065e-04*w22*Jinv10+9.1314935064935065e-06*w22*Jinv11+9.1314935064935065e-06*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+1.0957792207792207e-03*w36*Jinv01+1.1870941558441559e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+2.8409090909090908e-04*w9*Jinv00-4.5657467532467534e-05*w37*Jinv10+9.1314935064935065e-06*Jinv21*w56-1.4610389610389610e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14+2.8409090909090908e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+9.1314935064935065e-06*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-9.2329545454545453e-05*w57*Jinv00-9.2329545454545453e-05*Jinv20*w59-9.2329545454545453e-05*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+9.1314935064935065e-06*Jinv21*w23+9.1314935064935065e-06*Jinv01*w54+9.1314935064935065e-06*w55*Jinv11-1.4610389610389610e-04*w25*Jinv10-9.2329545454545453e-05*Jinv20*w2-7.6704545454545450e-04*w15*Jinv00+1.5827922077922079e-03*Jinv01*w9-7.6704545454545450e-04*Jinv20*w17+6.3920454545454550e-05*w30*Jinv00-6.7573051948051951e-04*w40*Jinv10-9.1314935064935068e-05*w20*Jinv20-4.9310064935064932e-04*w40*Jinv11+1.1870941558441559e-04*w5*Jinv20+9.1314935064935065e-06*Jinv01*w30-5.4788961038961036e-04*w25*Jinv11+1.2784090909090910e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-2.1002435064935065e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50+6.4935064935064935e-05*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+1.5827922077922079e-03*w10*Jinv11)*D;
  A[11][21]=(0.0)*D;
  A[11][22]=(0.0)*D;
  A[11][23]=(4.2918019480519480e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.7394480519480518e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+4.2918019480519480e-04*w13*Jinv10-2.7394480519480518e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20-1.7349837662337662e-04*Jinv01*w3-7.4878246753246752e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-7.8530844155844158e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.1668019480519480e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01+5.6615259740259744e-04*w16*Jinv11+5.6615259740259744e-04*Jinv21*w17-3.7439123376623376e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.9310064935064932e-04*w15*Jinv02-4.0584415584415584e-05*w10*Jinv12+1.0957792207792208e-04*w48*Jinv00-7.8530844155844158e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-3.9265422077922079e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10+4.7483766233766234e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00+6.9399350649350649e-04*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-4.0584415584415584e-05*w11*Jinv22-1.1668019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11+5.6615259740259744e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+1.0957792207792208e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.9310064935064932e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.4878246753246752e-04*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+1.3697240259740259e-04*Jinv02*w3-7.4878246753246752e-04*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22+6.9399350649350649e-04*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2+4.7483766233766234e-04*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00-7.4878246753246752e-04*Jinv02*w45-3.9265422077922079e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+1.3697240259740259e-04*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10-1.7349837662337662e-04*w4*Jinv11-2.7394480519480518e-04*w41*Jinv20+4.2918019480519480e-04*Jinv12*w13+4.7483766233766234e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-3.1960227272727273e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-2.7394480519480518e-04*Jinv22*w41-7.8530844155844158e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35-3.1960227272727273e-04*w31*Jinv11-7.8530844155844158e-04*Jinv22*w26-3.7439123376623376e-04*w46*Jinv11+4.2918019480519480e-04*Jinv20*w14-1.7349837662337662e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+1.3697240259740259e-04*w4*Jinv12-7.4878246753246752e-04*w21*Jinv02+1.0957792207792208e-04*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-2.7394480519480518e-04*w39*Jinv00+4.2918019480519480e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10-3.7439123376623376e-04*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38-7.4878246753246752e-04*Jinv22*w23+1.0957792207792208e-04*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53-9.1314935064935065e-06*Jinv22*w8-7.4878246753246752e-04*w22*Jinv10+1.0957792207792208e-04*Jinv22*w50-9.1314935064935065e-06*Jinv12*w7+1.0957792207792208e-04*Jinv02*w30+1.0957792207792208e-04*Jinv12*w49+1.3697240259740259e-04*w52*Jinv10+9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56-7.8530844155844158e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-9.1314935064935065e-06*w6*Jinv02+4.7483766233766234e-04*w19*Jinv12+1.3697240259740259e-04*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+4.7483766233766234e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11-7.8530844155844158e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00+1.0957792207792208e-04*w48*Jinv02-4.9310064935064932e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-2.7394480519480518e-04*w40*Jinv10+4.7483766233766234e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.2918019480519480e-04*Jinv22*w14+6.9399350649350649e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20-3.1960227272727273e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.9265422077922079e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.9310064935064932e-04*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1)*D;
  A[11][24]=(0.0)*D;
  A[11][25]=(0.0)*D;
  A[11][26]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10+9.1314935064935065e-06*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+4.3831168831168834e-04*Jinv21*w44-1.3697240259740259e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20+2.1915584415584417e-04*Jinv01*w3+7.4878246753246752e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54-1.4204545454545455e-05*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+4.9310064935064932e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-6.0876623376623375e-04*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17+3.9265422077922079e-04*Jinv01*w45+1.0247564935064935e-04*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+4.3831168831168834e-04*w43*Jinv11+7.4878246753246752e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27-1.4204545454545455e-05*Jinv11*w1-1.7349837662337662e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01-5.6818181818181818e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.7483766233766234e-04*Jinv00*w18+9.1314935064935065e-06*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53-1.3697240259740259e-04*w51*Jinv00-1.3697240259740259e-04*w4*Jinv10+2.1915584415584417e-04*w4*Jinv11+2.7394480519480518e-04*w41*Jinv20-4.7483766233766234e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20+9.1314935064935065e-06*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+7.8530844155844158e-04*w26*Jinv20+4.0584415584415584e-05*w11*Jinv20+2.1915584415584417e-04*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11+3.9265422077922079e-04*w46*Jinv11-4.2918019480519480e-04*Jinv20*w14+2.1915584415584417e-04*w5*Jinv21-1.0957792207792208e-04*w49*Jinv10-5.6818181818181818e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+2.7394480519480518e-04*w39*Jinv00-1.0957792207792207e-03*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-5.4788961038961036e-04*w6*Jinv01+9.1314935064935065e-06*w7*Jinv10+3.9265422077922079e-04*w47*Jinv21+9.1314935064935065e-06*Jinv20*w38+2.1915584415584417e-04*w52*Jinv11-1.3697240259740259e-04*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+3.9265422077922079e-04*w22*Jinv11+3.9265422077922079e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51-5.4788961038961036e-04*w36*Jinv01-1.3697240259740259e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11+4.0584415584415584e-05*w9*Jinv00+9.1314935064935065e-06*w37*Jinv10-1.7349837662337662e-04*Jinv21*w56+7.8530844155844158e-04*w24*Jinv00-1.3697240259740259e-04*w49*Jinv11-1.7349837662337662e-04*Jinv21*w14+4.0584415584415584e-05*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-1.7349837662337662e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.9265422077922079e-04*Jinv21*w23-1.7349837662337662e-04*Jinv01*w54-1.7349837662337662e-04*w55*Jinv11+7.8530844155844158e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-6.0876623376623375e-04*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00+2.7394480519480518e-04*w40*Jinv10-4.7483766233766234e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11-1.3697240259740259e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11-4.2918019480519480e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20-1.3697240259740259e-04*Jinv21*w50-5.6818181818181818e-05*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01-6.0876623376623375e-04*w10*Jinv11)*D;
  A[11][27]=(0.0)*D;
  A[11][28]=(0.0)*D;
  A[11][29]=(-4.2613636363636367e-05*Jinv21*w44-1.4204545454545455e-05*Jinv01*w3+5.4112554112554113e-05*Jinv21*w59+5.4112554112554113e-05*Jinv21*w2-4.2613636363636367e-05*w42*Jinv01+1.8939393939393939e-04*w11*Jinv21-4.2613636363636367e-05*w16*Jinv11-4.2613636363636367e-05*Jinv21*w17+1.5726461038961039e-04*Jinv01*w45+5.4112554112554113e-05*w58*Jinv11+5.4112554112554113e-05*w57*Jinv01+2.0495129870129870e-04*Jinv21*w26+5.4112554112554113e-05*w0*Jinv01-4.2613636363636367e-05*w43*Jinv11-1.9480519480519481e-04*Jinv21*w41+5.4112554112554113e-05*Jinv11*w1-1.1668019480519480e-04*w13*Jinv11-4.2613636363636367e-05*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-1.9480519480519481e-04*Jinv01*w39+6.4935064935064935e-05*Jinv21*w38+2.0495129870129870e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11+8.3198051948051951e-05*w19*Jinv11-1.4204545454545455e-05*Jinv21*w53-1.4204545454545455e-05*w4*Jinv11+1.0247564935064935e-04*w32*Jinv21+8.3198051948051951e-05*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35+1.0247564935064935e-04*w31*Jinv11+1.5726461038961039e-04*w46*Jinv11-1.4204545454545455e-05*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33+8.3198051948051951e-05*Jinv01*w18+6.4935064935064935e-05*w37*Jinv11+6.4935064935064935e-05*w6*Jinv01+1.5726461038961039e-04*w47*Jinv21-1.4204545454545455e-05*w52*Jinv11+1.5726461038961039e-04*w22*Jinv11+1.5726461038961039e-04*w21*Jinv01-1.4204545454545455e-05*Jinv01*w51+6.4935064935064935e-05*w36*Jinv01+6.4935064935064935e-05*w7*Jinv11-1.1668019480519480e-04*Jinv21*w56+1.0247564935064935e-04*w49*Jinv11-1.1668019480519480e-04*Jinv21*w14-1.1668019480519480e-04*Jinv01*w12+6.4935064935064935e-05*Jinv21*w8+1.5726461038961039e-04*Jinv21*w23-1.1668019480519480e-04*Jinv01*w54-1.1668019480519480e-04*w55*Jinv11+1.8939393939393939e-04*Jinv01*w9-1.9480519480519481e-04*w40*Jinv11+1.0247564935064935e-04*Jinv01*w30+2.0495129870129870e-04*w25*Jinv11+1.0247564935064935e-04*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27+1.0247564935064935e-04*w48*Jinv01+1.8939393939393939e-04*w10*Jinv11)*D;
  A[11][30]=(0.0)*D;
  A[11][31]=(0.0)*D;
  A[11][32]=(-3.7439123376623376e-04*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-6.9399350649350649e-04*w40*Jinv12+1.7349837662337662e-04*Jinv22*w53+4.8396915584415583e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02-3.7439123376623376e-04*w13*Jinv10-6.9399350649350649e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-7.4878246753246752e-04*w32*Jinv20-3.9265422077922079e-04*Jinv01*w3+3.1960227272727273e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+1.5726461038961039e-04*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01-4.0584415584415584e-05*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45+3.1960227272727273e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02+1.5523538961038962e-04*w48*Jinv00-5.6615259740259744e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+7.3051948051948052e-05*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-9.6793831168831166e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27+9.2329545454545453e-05*Jinv11*w1-3.7439123376623376e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+1.5523538961038962e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-1.0592532467532468e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-7.4878246753246752e-04*w31*Jinv10+3.9265422077922079e-04*Jinv02*w3+3.1960227272727273e-04*w21*Jinv00-9.6793831168831166e-04*Jinv01*w39+1.5726461038961039e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18-1.8262987012987013e-05*w36*Jinv00-1.8262987012987013e-05*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-1.0592532467532468e-03*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11+3.1047077922077924e-04*Jinv21*w53+3.1047077922077924e-04*w51*Jinv00-5.6615259740259744e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10-3.9265422077922079e-04*w4*Jinv11-9.6793831168831166e-04*w41*Jinv20+3.7439123376623376e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10-1.0592532467532468e-03*w34*Jinv10-7.4878246753246752e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20+4.8396915584415583e-04*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-6.9399350649350649e-04*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20+1.7349837662337662e-04*w52*Jinv12-1.0592532467532468e-03*Jinv21*w35-7.4878246753246752e-04*w31*Jinv11+2.1915584415584417e-04*w46*Jinv11-3.7439123376623376e-04*Jinv20*w14-3.9265422077922079e-04*w5*Jinv21+3.1960227272727273e-04*Jinv02*w54+3.9265422077922079e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+1.5523538961038962e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-1.0592532467532468e-03*Jinv01*w33-9.6793831168831166e-04*w39*Jinv00+3.7439123376623376e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18-1.8262987012987013e-05*w37*Jinv11+4.8396915584415583e-04*w6*Jinv01+4.8396915584415583e-04*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-1.8262987012987013e-05*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23+3.1047077922077924e-04*w52*Jinv11+1.0957792207792208e-04*w46*Jinv12+3.1047077922077924e-04*Jinv20*w53+9.1314935064935065e-06*Jinv22*w8+3.1960227272727273e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.1960227272727273e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+3.1960227272727273e-04*w21*Jinv01+3.1047077922077924e-04*Jinv01*w51-2.7394480519480518e-04*Jinv12*w49-1.8262987012987013e-05*w36*Jinv01+3.1047077922077924e-04*w52*Jinv10+4.8396915584415583e-04*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00-1.8262987012987013e-05*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+9.1314935064935065e-06*w6*Jinv02+3.9265422077922079e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.5523538961038962e-04*w49*Jinv11-3.7439123376623376e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.7439123376623376e-04*Jinv01*w12+4.8396915584415583e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-5.6615259740259744e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-1.0592532467532468e-03*w35*Jinv20+3.1960227272727273e-04*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-4.0584415584415584e-05*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-7.4878246753246752e-04*w30*Jinv00-9.6793831168831166e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20+3.7439123376623376e-04*Jinv22*w14-9.6793831168831166e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20-7.4878246753246752e-04*Jinv01*w30+1.7349837662337662e-04*Jinv02*w51+3.1960227272727273e-04*Jinv22*w56+1.3697240259740259e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12+1.5523538961038962e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+1.5523538961038962e-04*w48*Jinv01+1.5726461038961039e-04*Jinv12*w1-4.0584415584415584e-05*w10*Jinv11)*D;
  A[11][33]=(0.0)*D;
  A[11][34]=(0.0)*D;
  A[11][35]=(3.1047077922077924e-04*Jinv00*w12+1.9480519480519481e-04*w28*Jinv10-4.3831168831168834e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02+3.1047077922077924e-04*w13*Jinv10-4.3831168831168834e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+6.2094155844155847e-04*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23+5.4788961038961036e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+1.2175324675324675e-05*Jinv21*w59-4.2613636363636367e-05*w0*Jinv02-3.0438311688311688e-05*Jinv21*w2+6.9399350649350649e-04*w43*Jinv10-4.3831168831168834e-04*w42*Jinv01-7.3051948051948055e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17+7.3051948051948052e-05*Jinv01*w45-5.2962662337662338e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02+5.4788961038961042e-05*w48*Jinv00+1.2175324675324675e-05*w58*Jinv11+1.2175324675324675e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26-3.0438311688311688e-05*w0*Jinv01+9.1314935064935063e-04*w16*Jinv10-4.3831168831168834e-04*w43*Jinv11-3.4699675324675324e-04*w45*Jinv00-1.1323051948051949e-03*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.9480519480519481e-04*Jinv00*w27-3.0438311688311688e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+1.9480519480519481e-04*Jinv20*w29-3.4699675324675324e-04*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+6.2094155844155847e-04*w31*Jinv10+4.3831168831168834e-04*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00+5.6615259740259744e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39-4.2613636363636367e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38+3.6525974025974026e-05*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+5.4788961038961036e-04*w19*Jinv11-6.5746753246753243e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00-2.1915584415584417e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+5.2962662337662338e-04*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10+7.3051948051948052e-05*w32*Jinv21+5.4788961038961036e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+5.4788961038961036e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56-2.5568181818181820e-04*w26*Jinv20-2.4350649350649351e-04*w11*Jinv20-4.3831168831168834e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+7.3051948051948052e-05*w31*Jinv11+7.3051948051948052e-05*w46*Jinv11+3.1047077922077924e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21-5.2962662337662338e-04*Jinv02*w54+4.3831168831168834e-04*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+5.2962662337662338e-04*Jinv02*w12+5.4788961038961036e-04*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10+7.3051948051948052e-05*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23-6.5746753246753243e-04*w52*Jinv11+5.6615259740259744e-04*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-5.6615259740259744e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01-6.5746753246753243e-04*Jinv01*w51+5.6615259740259744e-04*Jinv02*w30-5.6615259740259744e-04*Jinv12*w49+1.0957792207792207e-03*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11-2.4350649350649351e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10-4.9310064935064932e-04*Jinv21*w56-2.5568181818181820e-04*w24*Jinv00-5.4788961038961036e-04*w6*Jinv02+4.3831168831168834e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14-2.4350649350649351e-04*w10*Jinv10+6.9399350649350649e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12+3.6525974025974026e-05*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+1.8262987012987013e-03*w35*Jinv20+3.6525974025974026e-05*Jinv21*w23-4.9310064935064932e-04*Jinv01*w54-4.9310064935064932e-04*w55*Jinv11-2.5568181818181820e-04*w25*Jinv10+9.1314935064935063e-04*w15*Jinv00-5.6615259740259744e-04*w48*Jinv02-7.3051948051948055e-04*Jinv01*w9+9.1314935064935063e-04*Jinv20*w17+6.2094155844155847e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11+5.4788961038961036e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+7.3051948051948052e-05*Jinv01*w30-4.3831168831168834e-04*Jinv02*w51-5.2962662337662338e-04*Jinv22*w56-2.1915584415584417e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+6.9399350649350649e-04*w42*Jinv00-3.4699675324675324e-04*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-4.2613636363636367e-05*Jinv12*w1-7.3051948051948055e-04*w10*Jinv11)*D;
  A[11][36]=(0.0)*D;
  A[11][37]=(0.0)*D;
  A[11][38]=(6.3920454545454550e-05*Jinv00*w12-9.2329545454545453e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-4.5657467532467534e-05*Jinv20*w8+6.4935064935064935e-05*w57*Jinv02+6.3920454545454550e-05*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+1.1870941558441559e-04*w3*Jinv00+1.2784090909090910e-04*w32*Jinv20+6.3920454545454550e-05*Jinv20*w23+1.0957792207792207e-03*Jinv22*w38-2.1002435064935065e-04*Jinv00*w54-5.4788961038961036e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10+9.1314935064935065e-06*Jinv12*w55-5.6818181818181818e-05*w58*Jinv10-5.4788961038961036e-04*w15*Jinv02+1.5827922077922079e-03*w10*Jinv12-2.1002435064935065e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-5.4788961038961036e-04*Jinv22*w35-9.2329545454545453e-05*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20+1.2784090909090910e-04*w45*Jinv00-5.4788961038961036e-04*w42*Jinv02-9.2329545454545453e-05*Jinv00*w27+1.5827922077922079e-03*w11*Jinv22-2.1002435064935065e-04*Jinv20*w50-9.2329545454545453e-05*Jinv20*w29+1.2784090909090910e-04*w46*Jinv10-5.4788961038961036e-04*Jinv22*w17-7.6704545454545450e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12+1.2784090909090910e-04*w31*Jinv10-5.4788961038961036e-04*Jinv02*w3+6.3920454545454550e-05*w21*Jinv00+9.1314935064935065e-06*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2-4.5657467532467534e-05*Jinv00*w18-9.1314935064935068e-05*w36*Jinv00+9.1314935064935065e-06*Jinv02*w45+6.4935064935064935e-05*Jinv22*w59-1.4610389610389610e-04*w51*Jinv00-5.4788961038961036e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.1870941558441559e-04*w4*Jinv10-6.7573051948051951e-04*w41*Jinv20+9.1314935064935065e-06*Jinv12*w13-4.5657467532467534e-05*w19*Jinv10-7.6704545454545450e-04*w34*Jinv10-4.5657467532467534e-05*w6*Jinv00+1.0957792207792207e-03*w37*Jinv12-2.1002435064935065e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41+1.1870941558441559e-04*w26*Jinv20+2.8409090909090908e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-5.4788961038961036e-04*Jinv22*w26+6.3920454545454550e-05*Jinv20*w14+9.1314935064935065e-06*Jinv02*w54-5.4788961038961036e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-2.1002435064935065e-04*w49*Jinv10-6.7573051948051951e-04*w39*Jinv00+9.1314935064935065e-06*Jinv02*w12+1.5827922077922079e-03*Jinv02*w9-4.5657467532467534e-05*w7*Jinv10-9.1314935064935068e-05*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23+9.1314935064935065e-06*w31*Jinv12+9.1314935064935065e-06*w46*Jinv12-1.4610389610389610e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8+6.3920454545454550e-05*w22*Jinv10+9.1314935064935065e-06*Jinv22*w50+1.0957792207792207e-03*Jinv12*w7+9.1314935064935065e-06*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49-1.4610389610389610e-04*w52*Jinv10+2.8409090909090908e-04*w9*Jinv00-9.1314935064935068e-05*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+6.4935064935064935e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-5.4788961038961036e-04*Jinv22*w5+9.1314935064935065e-06*w47*Jinv22+2.8409090909090908e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+6.4935064935064935e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18-5.6818181818181818e-05*w57*Jinv00-5.6818181818181818e-05*Jinv20*w59-5.4788961038961036e-04*Jinv02*w33-9.2329545454545453e-05*w0*Jinv00-7.6704545454545450e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10-9.2329545454545453e-05*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00+9.1314935064935065e-06*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.2784090909090910e-04*w30*Jinv00-6.7573051948051951e-04*w40*Jinv10-4.5657467532467534e-05*w20*Jinv20+6.4935064935064935e-05*Jinv22*w29+9.1314935064935065e-06*Jinv22*w14+1.0957792207792207e-03*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.1870941558441559e-04*w5*Jinv20-5.4788961038961036e-04*Jinv02*w51+9.1314935064935065e-06*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28-2.1002435064935065e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00+1.2784090909090910e-04*w47*Jinv20-5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[11][39]=(0.0)*D;
  A[11][40]=(0.0)*D;
  A[11][41]=(4.2004870129870131e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10-5.2962662337662338e-04*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02+4.2004870129870131e-04*w13*Jinv10-1.1323051948051949e-03*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00+4.2004870129870131e-04*w32*Jinv20+5.2962662337662338e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-1.9480519480519481e-04*w0*Jinv02-1.9480519480519481e-04*Jinv21*w2-2.2646103896103898e-03*w43*Jinv10-1.1323051948051949e-03*w42*Jinv01-9.6793831168831166e-04*Jinv01*w45-9.6793831168831166e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10+1.1323051948051949e-03*w15*Jinv02-2.7394480519480518e-04*w48*Jinv00-5.2962662337662338e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-2.2524350649350650e-04*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10-1.1323051948051949e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-1.1323051948051949e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-1.9480519480519481e-04*Jinv11*w1+6.9399350649350649e-04*w13*Jinv11+1.9480519480519481e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+1.1323051948051949e-03*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+4.2004870129870131e-04*w31*Jinv10+5.2962662337662338e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00+6.9399350649350649e-04*w32*Jinv22-1.9480519480519481e-04*Jinv22*w2+9.8620129870129864e-04*Jinv00*w18+9.8620129870129864e-04*w36*Jinv00-2.7394480519480518e-04*Jinv02*w45-5.2962662337662338e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59+1.1323051948051949e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-4.9310064935064932e-04*w51*Jinv00-1.1323051948051949e-03*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+5.2962662337662338e-04*w4*Jinv11-2.7759740259740259e-03*w41*Jinv20+9.6793831168831166e-04*Jinv12*w13+9.8620129870129864e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10+9.6793831168831166e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-4.9310064935064932e-04*w26*Jinv20-4.8701298701298701e-04*w11*Jinv20-5.2962662337662338e-04*w52*Jinv12+1.1323051948051949e-03*Jinv21*w35+9.6793831168831166e-04*w31*Jinv11-9.6793831168831166e-04*w46*Jinv11+4.2004870129870131e-04*Jinv20*w14+5.2962662337662338e-04*w5*Jinv21-9.6793831168831166e-04*Jinv02*w54+5.2962662337662338e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+1.1323051948051949e-03*Jinv01*w33-2.7759740259740259e-03*w39*Jinv00+9.6793831168831166e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18-4.9310064935064932e-04*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10-9.6793831168831166e-04*w47*Jinv21+9.8620129870129864e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+6.9399350649350649e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-4.9310064935064932e-04*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10-6.9399350649350649e-04*Jinv22*w50-6.9399350649350649e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-6.9399350649350649e-04*w21*Jinv01+6.9399350649350649e-04*Jinv02*w30-6.9399350649350649e-04*Jinv12*w49-4.9310064935064932e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-4.8701298701298701e-04*w9*Jinv00+9.8620129870129864e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-4.9310064935064932e-04*w24*Jinv00-4.9310064935064932e-04*w6*Jinv02+5.2962662337662338e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11+6.9399350649350649e-04*Jinv21*w14-4.8701298701298701e-04*w10*Jinv10-2.2646103896103898e-03*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12+6.9399350649350649e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59-2.2524350649350650e-04*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-6.9399350649350649e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-4.9310064935064932e-04*w25*Jinv10-2.2524350649350650e-04*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-6.9399350649350649e-04*w48*Jinv02-1.1323051948051949e-03*Jinv20*w17+4.2004870129870131e-04*w30*Jinv00-2.7759740259740259e-03*w40*Jinv10+9.8620129870129864e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36-1.1323051948051949e-03*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+9.6793831168831166e-04*Jinv01*w30-5.2962662337662338e-04*Jinv02*w51-9.6793831168831166e-04*Jinv22*w56-5.2962662337662338e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-2.2646103896103898e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+1.1323051948051949e-03*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-1.9480519480519481e-04*Jinv12*w1)*D;
  A[11][42]=(0.0)*D;
  A[11][43]=(0.0)*D;
  A[11][44]=(-4.2004870129870131e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8+3.0438311688311688e-05*w57*Jinv02-4.2004870129870131e-04*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00-4.2004870129870131e-04*w32*Jinv20+2.1915584415584417e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54-1.2175324675324675e-05*Jinv21*w59+6.5746753246753243e-04*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02+3.0438311688311688e-05*Jinv21*w2+2.2646103896103898e-03*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01+7.3051948051948055e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-7.3051948051948052e-05*Jinv01*w45-7.3051948051948052e-05*Jinv12*w55+3.0438311688311688e-05*w58*Jinv10+4.3831168831168834e-04*w15*Jinv02+7.3051948051948055e-04*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+6.5746753246753243e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35-1.2175324675324675e-05*w58*Jinv11-1.2175324675324675e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01+2.2524350649350650e-04*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+3.0438311688311688e-05*Jinv00*w27+7.3051948051948055e-04*w11*Jinv22+3.0438311688311688e-05*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+3.0438311688311688e-05*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10+4.3831168831168834e-04*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33+4.9310064935064932e-04*w22*Jinv12-4.2004870129870131e-04*w31*Jinv10+2.1915584415584417e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+3.0438311688311688e-05*Jinv22*w2-9.8620129870129864e-04*Jinv00*w18-9.8620129870129864e-04*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24+3.0438311688311688e-05*Jinv22*w59+4.3831168831168834e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+6.5746753246753243e-04*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+2.1915584415584417e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10+2.1915584415584417e-04*w4*Jinv11+2.7759740259740259e-03*w41*Jinv20-7.3051948051948052e-05*Jinv12*w13-9.8620129870129864e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-7.3051948051948052e-05*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+4.8701298701298701e-04*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12+4.3831168831168834e-04*Jinv21*w35-7.3051948051948052e-05*w31*Jinv11+6.5746753246753243e-04*Jinv22*w26-7.3051948051948052e-05*w46*Jinv11-4.2004870129870131e-04*Jinv20*w14+2.1915584415584417e-04*w5*Jinv21-7.3051948051948052e-05*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12+4.9310064935064932e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10+3.0438311688311688e-05*Jinv21*w29+4.3831168831168834e-04*Jinv01*w33+2.7759740259740259e-03*w39*Jinv00-7.3051948051948052e-05*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11+7.3051948051948055e-04*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-7.3051948051948052e-05*w47*Jinv21-9.8620129870129864e-04*Jinv20*w38+4.9310064935064932e-04*Jinv22*w23+6.5746753246753243e-04*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+6.5746753246753243e-04*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11+4.8701298701298701e-04*w9*Jinv00-9.8620129870129864e-04*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00-1.2175324675324675e-05*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22+4.9310064935064932e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+4.8701298701298701e-04*w10*Jinv10+2.2646103896103898e-03*Jinv20*w44+3.0438311688311688e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+2.1915584415584417e-04*Jinv02*w33+2.2524350649350650e-04*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10+2.2524350649350650e-04*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+7.3051948051948055e-04*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17-4.2004870129870131e-04*w30*Jinv00+2.7759740259740259e-03*w40*Jinv10-9.8620129870129864e-04*w20*Jinv20-1.2175324675324675e-05*Jinv22*w29-7.3051948051948052e-05*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20-7.3051948051948052e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51-7.3051948051948052e-05*Jinv22*w56-1.2175324675324675e-05*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+2.2646103896103898e-03*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+4.3831168831168834e-04*w16*Jinv12+4.9310064935064932e-04*Jinv21*w50+3.0438311688311688e-05*Jinv01*w27+4.9310064935064932e-04*w48*Jinv01+3.0438311688311688e-05*Jinv12*w1+7.3051948051948055e-04*w10*Jinv11)*D;
  A[11][45]=(0.0)*D;
  A[11][46]=(0.0)*D;
  A[11][47]=(2.7394480519480518e-04*w40*Jinv12-1.3697240259740259e-04*Jinv22*w53+1.0247564935064935e-04*w57*Jinv02+2.7394480519480518e-04*Jinv02*w39+1.0592532467532468e-03*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3+9.1314935064935065e-06*Jinv22*w38-1.4204545454545455e-05*Jinv21*w59+7.8530844155844158e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+1.0592532467532468e-03*w42*Jinv01+4.0584415584415584e-05*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+7.4878246753246752e-04*Jinv01*w45-4.2918019480519480e-04*Jinv12*w55+4.9310064935064932e-04*w15*Jinv02+4.0584415584415584e-05*w10*Jinv12+7.8530844155844158e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01+3.9265422077922079e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01-4.7483766233766234e-04*Jinv22*w20+1.0592532467532468e-03*w43*Jinv11+9.6793831168831166e-04*Jinv21*w41+4.9310064935064932e-04*w42*Jinv02+4.0584415584415584e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-3.1960227272727273e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+4.9310064935064932e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-1.0957792207792208e-04*w32*Jinv22+9.6793831168831166e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2+1.8262987012987013e-05*Jinv21*w38+7.4878246753246752e-04*Jinv02*w45+3.9265422077922079e-04*Jinv01*w24+1.0247564935064935e-04*Jinv22*w59-7.3051948051948052e-05*w34*Jinv11-4.8396915584415583e-04*w19*Jinv11-3.1047077922077924e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-4.2918019480519480e-04*Jinv12*w13-2.1915584415584417e-04*w32*Jinv21-4.8396915584415583e-04*Jinv21*w20+9.1314935064935065e-06*w37*Jinv12+2.7394480519480518e-04*Jinv22*w41-1.3697240259740259e-04*w52*Jinv12-7.3051948051948052e-05*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11+7.8530844155844158e-04*Jinv22*w26+7.4878246753246752e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21-4.2918019480519480e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33-4.2918019480519480e-04*Jinv02*w12-4.8396915584415583e-04*Jinv01*w18+1.8262987012987013e-05*w37*Jinv11+4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01+7.4878246753246752e-04*w47*Jinv21+7.4878246753246752e-04*Jinv22*w23-3.1047077922077924e-04*w52*Jinv11-1.0957792207792208e-04*w31*Jinv12+7.4878246753246752e-04*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50+3.7439123376623376e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+3.7439123376623376e-04*w21*Jinv01-3.1047077922077924e-04*Jinv01*w51-1.0957792207792208e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+1.8262987012987013e-05*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11-1.5523538961038962e-04*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02-4.7483766233766234e-04*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5+7.4878246753246752e-04*w47*Jinv22-4.2918019480519480e-04*w49*Jinv11-3.1960227272727273e-04*Jinv21*w14+1.0247564935064935e-04*w58*Jinv12-4.7483766233766234e-04*Jinv02*w18-3.1960227272727273e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.7439123376623376e-04*Jinv21*w23-1.5523538961038962e-04*Jinv01*w54-1.5523538961038962e-04*w55*Jinv11-1.0957792207792208e-04*w48*Jinv02+4.0584415584415584e-05*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+9.6793831168831166e-04*w40*Jinv11+9.1314935064935065e-06*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-2.1915584415584417e-04*Jinv01*w30-1.3697240259740259e-04*Jinv02*w51-4.2918019480519480e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.9265422077922079e-04*w25*Jinv11+4.9310064935064932e-04*w16*Jinv12-4.2918019480519480e-04*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1+4.0584415584415584e-05*w10*Jinv11)*D;
  A[11][48]=(0.0)*D;
  A[11][49]=(0.0)*D;
  A[11][50]=(1.0957792207792208e-04*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10+6.9399350649350649e-04*w40*Jinv12-3.9265422077922079e-04*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10+6.9399350649350649e-04*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+4.2918019480519480e-04*w32*Jinv20+1.3697240259740259e-04*Jinv01*w3+1.0957792207792208e-04*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-1.1668019480519480e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-4.0584415584415584e-05*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-3.7439123376623376e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02-7.4878246753246752e-04*w48*Jinv00+5.6615259740259744e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-4.9310064935064932e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-2.7394480519480518e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-1.0247564935064935e-04*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-7.4878246753246752e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-4.9310064935064932e-04*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+4.2918019480519480e-04*w31*Jinv10-1.7349837662337662e-04*Jinv02*w3+1.0957792207792208e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-2.7394480519480518e-04*Jinv01*w39-1.1668019480519480e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18+4.7483766233766234e-04*w36*Jinv00+4.7483766233766234e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11-7.8530844155844158e-04*Jinv21*w53-7.8530844155844158e-04*w51*Jinv00+5.6615259740259744e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10+1.3697240259740259e-04*w4*Jinv11-2.7394480519480518e-04*w41*Jinv20-3.1960227272727273e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+4.2918019480519480e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56+6.9399350649350649e-04*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-4.0584415584415584e-05*w11*Jinv20-3.9265422077922079e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+4.2918019480519480e-04*w31*Jinv11+4.2918019480519480e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+1.3697240259740259e-04*w5*Jinv21-3.7439123376623376e-04*Jinv02*w54-1.7349837662337662e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-7.4878246753246752e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-2.7394480519480518e-04*w39*Jinv00-3.1960227272727273e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18+4.7483766233766234e-04*w37*Jinv11-9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+4.7483766233766234e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-7.8530844155844158e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12-7.8530844155844158e-04*Jinv20*w53+9.1314935064935065e-06*Jinv22*w8+1.0957792207792208e-04*w22*Jinv10+1.0957792207792208e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7+1.0957792207792208e-04*w21*Jinv01-7.8530844155844158e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+4.7483766233766234e-04*w36*Jinv01-7.8530844155844158e-04*w52*Jinv10-9.1314935064935065e-06*w7*Jinv11-4.0584415584415584e-05*w9*Jinv00+4.7483766233766234e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+9.1314935064935065e-06*w6*Jinv02-1.7349837662337662e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22-7.4878246753246752e-04*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-4.0584415584415584e-05*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.0957792207792208e-04*Jinv01*w12-9.1314935064935065e-06*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+5.6615259740259744e-04*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20+1.0957792207792208e-04*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-4.0584415584415584e-05*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.2918019480519480e-04*w30*Jinv00-2.7394480519480518e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20-3.1960227272727273e-04*Jinv22*w14-2.7394480519480518e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20+4.2918019480519480e-04*Jinv01*w30-3.9265422077922079e-04*Jinv02*w51-3.7439123376623376e-04*Jinv22*w56+1.3697240259740259e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12-7.4878246753246752e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-7.4878246753246752e-04*w48*Jinv01-1.1668019480519480e-04*Jinv12*w1-4.0584415584415584e-05*w10*Jinv11)*D;
  A[11][51]=(0.0)*D;
  A[11][52]=(0.0)*D;
  A[11][53]=(-1.0957792207792208e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+9.1314935064935065e-06*Jinv20*w8-5.6818181818181818e-05*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39-1.3697240259740259e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+7.4878246753246752e-04*Jinv00*w54+2.1915584415584417e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02+4.9310064935064932e-04*w43*Jinv10+3.9265422077922079e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02-6.0876623376623375e-04*w10*Jinv12+7.4878246753246752e-04*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-4.2918019480519480e-04*w45*Jinv00+4.3831168831168834e-04*w42*Jinv02+1.0247564935064935e-04*Jinv00*w27-6.0876623376623375e-04*w11*Jinv22+7.4878246753246752e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-4.2918019480519480e-04*w31*Jinv10+2.1915584415584417e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-1.7349837662337662e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2+9.1314935064935065e-06*Jinv00*w18-4.7483766233766234e-04*w36*Jinv00-1.7349837662337662e-04*Jinv02*w45-5.6818181818181818e-05*Jinv22*w59+7.8530844155844158e-04*w51*Jinv00+4.3831168831168834e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv10+2.7394480519480518e-04*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13+9.1314935064935065e-06*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10+9.1314935064935065e-06*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+7.4878246753246752e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41-1.3697240259740259e-04*w26*Jinv20+4.0584415584415584e-05*w11*Jinv20+6.5746753246753243e-04*w52*Jinv12+2.1915584415584417e-04*Jinv22*w26-1.0957792207792208e-04*Jinv20*w14+3.9265422077922079e-04*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+7.4878246753246752e-04*w49*Jinv10+2.7394480519480518e-04*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12-6.0876623376623375e-04*Jinv02*w9+9.1314935064935065e-06*w7*Jinv10-4.7483766233766234e-04*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-1.7349837662337662e-04*w31*Jinv12-1.7349837662337662e-04*w46*Jinv12+7.8530844155844158e-04*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+3.9265422077922079e-04*Jinv22*w50-5.4788961038961036e-04*Jinv12*w7-1.7349837662337662e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49+7.8530844155844158e-04*w52*Jinv10+4.0584415584415584e-05*w9*Jinv00-4.7483766233766234e-04*w37*Jinv10-1.3697240259740259e-04*w24*Jinv00-1.4204545454545455e-05*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5-1.7349837662337662e-04*w47*Jinv22+4.0584415584415584e-05*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-5.6818181818181818e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+4.3831168831168834e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.3697240259740259e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.9265422077922079e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+2.7394480519480518e-04*w40*Jinv10+9.1314935064935065e-06*w20*Jinv20-1.4204545454545455e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14-1.0957792207792207e-03*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-1.3697240259740259e-04*w5*Jinv20+6.5746753246753243e-04*Jinv02*w51+3.9265422077922079e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+7.4878246753246752e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[11][54]=(0.0)*D;
  A[11][55]=(0.0)*D;
  A[11][56]=(9.6793831168831166e-04*w40*Jinv12+3.9265422077922079e-04*Jinv22*w53-9.2329545454545453e-05*w57*Jinv02+9.6793831168831166e-04*Jinv02*w39+4.9310064935064932e-04*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3-4.8396915584415583e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-3.1047077922077924e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w42*Jinv01+4.0584415584415584e-05*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.2918019480519480e-04*Jinv01*w45+7.4878246753246752e-04*Jinv12*w55-7.3051948051948052e-05*w15*Jinv02+4.0584415584415584e-05*w10*Jinv12-3.1047077922077924e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.3697240259740259e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.8262987012987013e-05*Jinv22*w20+4.9310064935064932e-04*w43*Jinv11+2.7394480519480518e-04*Jinv21*w41+1.0592532467532468e-03*w42*Jinv02+4.0584415584415584e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-1.0957792207792208e-04*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11-7.3051948051948052e-05*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-3.1960227272727273e-04*w32*Jinv22+2.7394480519480518e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2-4.7483766233766234e-04*Jinv21*w38-1.5523538961038962e-04*Jinv02*w45-1.3697240259740259e-04*Jinv01*w24-9.2329545454545453e-05*Jinv22*w59+4.9310064935064932e-04*w34*Jinv11+9.1314935064935065e-06*w19*Jinv11+7.8530844155844158e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+1.0592532467532468e-03*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-2.1915584415584417e-04*Jinv12*w13-4.2918019480519480e-04*w32*Jinv21+9.1314935064935065e-06*Jinv21*w20-4.8396915584415583e-04*w37*Jinv12+9.6793831168831166e-04*Jinv22*w41+3.9265422077922079e-04*w52*Jinv12+4.9310064935064932e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-3.1047077922077924e-04*Jinv22*w26-4.2918019480519480e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21+7.4878246753246752e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-2.1915584415584417e-04*Jinv02*w12+9.1314935064935065e-06*Jinv01*w18-4.7483766233766234e-04*w37*Jinv11+4.0584415584415584e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-4.2918019480519480e-04*w47*Jinv21-4.2918019480519480e-04*Jinv22*w23+7.8530844155844158e-04*w52*Jinv11-3.1960227272727273e-04*w31*Jinv12-1.5523538961038962e-04*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8+3.7439123376623376e-04*Jinv22*w50-1.0957792207792208e-04*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7-1.0957792207792208e-04*w21*Jinv01+7.8530844155844158e-04*Jinv01*w51-3.1960227272727273e-04*Jinv02*w30+3.7439123376623376e-04*Jinv12*w49-4.7483766233766234e-04*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11+7.4878246753246752e-04*Jinv21*w56-1.4204545454545455e-05*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02+1.8262987012987013e-05*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5-1.5523538961038962e-04*w47*Jinv22+7.4878246753246752e-04*w49*Jinv11-1.0957792207792208e-04*Jinv21*w14-9.2329545454545453e-05*w58*Jinv12+1.8262987012987013e-05*Jinv02*w18-1.0957792207792208e-04*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33-1.0957792207792208e-04*Jinv21*w23+7.4878246753246752e-04*Jinv01*w54+7.4878246753246752e-04*w55*Jinv11+3.7439123376623376e-04*w48*Jinv02+4.0584415584415584e-05*Jinv01*w9-1.4204545454545455e-05*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+2.7394480519480518e-04*w40*Jinv11-4.8396915584415583e-04*Jinv02*w36+1.0592532467532468e-03*Jinv12*w43-4.2918019480519480e-04*Jinv01*w30+3.9265422077922079e-04*Jinv02*w51+7.4878246753246752e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28-1.3697240259740259e-04*w25*Jinv11-7.3051948051948052e-05*w16*Jinv12+7.4878246753246752e-04*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1+4.0584415584415584e-05*w10*Jinv11)*D;
  A[11][57]=(0.0)*D;
  A[11][58]=(0.0)*D;
  A[11][59]=(-1.9480519480519481e-04*w40*Jinv12+2.0495129870129870e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-1.9480519480519481e-04*Jinv02*w39+8.3198051948051951e-05*Jinv22*w38-1.4204545454545455e-05*w25*Jinv12+5.4112554112554113e-05*w0*Jinv02+1.5726461038961039e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+1.8939393939393939e-04*w10*Jinv12-1.4204545454545455e-05*Jinv02*w24-4.2613636363636367e-05*Jinv22*w35+6.4935064935064935e-05*Jinv22*w20-4.2613636363636367e-05*w42*Jinv02+1.8939393939393939e-04*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17+1.0247564935064935e-04*w22*Jinv12-1.4204545454545455e-05*Jinv02*w3-1.1668019480519480e-04*w32*Jinv22+5.4112554112554113e-05*Jinv22*w2-1.1668019480519480e-04*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-4.2613636363636367e-05*Jinv12*w34-4.2613636363636367e-05*Jinv22*w44+1.0247564935064935e-04*Jinv12*w13+8.3198051948051951e-05*w37*Jinv12-1.9480519480519481e-04*Jinv22*w41+2.0495129870129870e-04*w52*Jinv12-1.4204545454545455e-05*Jinv22*w26+1.5726461038961039e-04*Jinv02*w54-1.4204545454545455e-05*w4*Jinv12+1.0247564935064935e-04*w21*Jinv02+1.0247564935064935e-04*Jinv02*w12+1.8939393939393939e-04*Jinv02*w9+1.0247564935064935e-04*Jinv22*w23-1.1668019480519480e-04*w31*Jinv12-1.1668019480519480e-04*w46*Jinv12+6.4935064935064935e-05*Jinv22*w8+1.5726461038961039e-04*Jinv22*w50+6.4935064935064935e-05*Jinv12*w7-1.1668019480519480e-04*Jinv02*w30+1.5726461038961039e-04*Jinv12*w49+5.4112554112554113e-05*Jinv02*w27+6.4935064935064935e-05*w6*Jinv02+6.4935064935064935e-05*w19*Jinv12-1.4204545454545455e-05*Jinv22*w5-1.1668019480519480e-04*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12+6.4935064935064935e-05*Jinv02*w18-4.2613636363636367e-05*Jinv02*w33+1.5726461038961039e-04*w48*Jinv02+5.4112554112554113e-05*Jinv22*w29+1.0247564935064935e-04*Jinv22*w14+8.3198051948051951e-05*Jinv02*w36-4.2613636363636367e-05*Jinv12*w43+2.0495129870129870e-04*Jinv02*w51+1.5726461038961039e-04*Jinv22*w56+5.4112554112554113e-05*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+5.4112554112554113e-05*Jinv12*w1)*D;
  A[12][0]=(-2.3741883116883116e-03*Jinv00*w12-2.0495129870129870e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+6.7573051948051951e-04*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02-2.3741883116883116e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-1.1870941558441558e-03*w3*Jinv00-1.1870941558441558e-03*w32*Jinv20-1.1870941558441558e-03*Jinv01*w3+1.0227272727272728e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+1.0957792207792208e-04*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59+2.6481331168831169e-04*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+2.6481331168831169e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10-6.5746753246753243e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12+6.7573051948051951e-04*w48*Jinv00+2.6481331168831169e-04*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01+2.6481331168831169e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+1.0957792207792208e-04*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+2.6481331168831169e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-2.0495129870129870e-04*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-2.3741883116883116e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-2.0495129870129870e-04*w28*Jinv11+6.7573051948051951e-04*Jinv20*w50-2.0495129870129870e-04*Jinv20*w29+2.6481331168831169e-04*w46*Jinv10-6.5746753246753243e-04*Jinv22*w17-3.2873376623376621e-04*Jinv00*w33+1.0227272727272728e-03*w22*Jinv12-1.1870941558441558e-03*w31*Jinv10-1.1870941558441558e-03*Jinv02*w3+1.0227272727272728e-03*w21*Jinv00-1.1870941558441558e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38+2.6481331168831169e-04*Jinv02*w45+2.6481331168831169e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00-3.2873376623376621e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-1.1870941558441558e-03*w4*Jinv10-1.1870941558441558e-03*w4*Jinv11-6.5746753246753243e-04*w41*Jinv20-2.3741883116883116e-03*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-1.1870941558441558e-03*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+6.7573051948051951e-04*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+1.0957792207792208e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+2.6481331168831169e-04*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-1.1870941558441558e-03*w31*Jinv11+2.6481331168831169e-04*Jinv22*w26+2.6481331168831169e-04*w46*Jinv11-2.3741883116883116e-03*Jinv20*w14-1.1870941558441558e-03*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54-1.1870941558441558e-03*w4*Jinv12+1.0227272727272728e-03*w21*Jinv02+6.7573051948051951e-04*w49*Jinv10-2.0495129870129870e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33-6.5746753246753243e-04*w39*Jinv00-2.3741883116883116e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9+6.7573051948051951e-04*w6*Jinv01+6.7573051948051951e-04*w7*Jinv10+2.6481331168831169e-04*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38+1.0227272727272728e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.1870941558441558e-03*w31*Jinv12+2.6481331168831169e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+6.7573051948051951e-04*Jinv22*w8+1.0227272727272728e-03*w22*Jinv10+6.7573051948051951e-04*Jinv22*w50+1.0227272727272728e-03*w22*Jinv11+6.7573051948051951e-04*Jinv12*w7+1.0227272727272728e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.1870941558441558e-03*Jinv02*w30+6.7573051948051951e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10+6.7573051948051951e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.6481331168831169e-04*w24*Jinv00-2.0495129870129870e-04*Jinv02*w27+6.7573051948051951e-04*w6*Jinv02+1.0957792207792208e-04*w19*Jinv12-1.1870941558441558e-03*Jinv22*w5+2.6481331168831169e-04*w47*Jinv22+6.7573051948051951e-04*w49*Jinv11-2.3741883116883116e-03*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.0957792207792208e-04*Jinv02*w18-2.3741883116883116e-03*Jinv01*w12+6.7573051948051951e-04*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59-3.2873376623376621e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+1.0227272727272728e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.6481331168831169e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+6.7573051948051951e-04*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.1870941558441558e-03*w30*Jinv00-6.5746753246753243e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20-2.0495129870129870e-04*Jinv22*w29-2.3741883116883116e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-1.1870941558441558e-03*w5*Jinv20-1.1870941558441558e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56-2.0495129870129870e-04*Jinv12*w28+2.6481331168831169e-04*w25*Jinv11+1.0957792207792208e-04*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+2.6481331168831169e-04*w47*Jinv20-6.5746753246753243e-04*w16*Jinv12+6.7573051948051951e-04*Jinv21*w50-2.0495129870129870e-04*Jinv01*w27+6.7573051948051951e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[12][1]=(0.0)*D;
  A[12][2]=(0.0)*D;
  A[12][3]=(4.9310064935064934e-03*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-1.1505681818181819e-03*Jinv20*w8-3.7439123376623376e-04*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+2.4655032467532467e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3-1.3971185064935066e-03*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54-3.7439123376623376e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-3.1777597402597403e-03*w16*Jinv11-3.1777597402597403e-03*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55+3.7439123376623376e-04*w58*Jinv10-3.1777597402597403e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35-3.7439123376623376e-04*w58*Jinv11-3.7439123376623376e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+1.1414366883116883e-03*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02+3.9265422077922079e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-3.1777597402597403e-03*w15*Jinv01-3.9265422077922079e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10-3.1777597402597403e-03*Jinv22*w17+5.4788961038961036e-04*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12+2.4655032467532467e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3-1.3971185064935066e-03*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+8.2183441558441553e-05*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24-3.7439123376623376e-04*Jinv22*w59-1.5888798701298701e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11+2.6846590909090908e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+5.4788961038961036e-04*w34*Jinv10-1.2327516233766234e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20-1.1505681818181819e-03*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12-1.5888798701298701e-03*Jinv21*w35-1.2327516233766234e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10-3.9265422077922079e-04*Jinv21*w29-1.5888798701298701e-03*Jinv01*w33+2.6846590909090908e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01-1.1505681818181819e-03*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+1.2327516233766234e-03*Jinv22*w8-1.3971185064935066e-03*w22*Jinv10+1.0135957792207792e-03*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+1.0135957792207792e-03*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+1.2327516233766234e-03*w7*Jinv11+3.5612824675324673e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-3.9265422077922079e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22+1.0135957792207792e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14+3.5612824675324673e-04*w10*Jinv10-3.7439123376623376e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8+3.7439123376623376e-04*w57*Jinv00+3.7439123376623376e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+1.1414366883116883e-03*w0*Jinv00+5.4788961038961036e-04*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+1.1414366883116883e-03*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+1.0135957792207792e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9+1.0957792207792207e-03*Jinv20*w17+2.4655032467532467e-03*w30*Jinv00+2.6846590909090908e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.2327516233766234e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10-8.2183441558441553e-05*w47*Jinv20-3.1777597402597403e-03*w16*Jinv12+1.0135957792207792e-03*Jinv21*w50-3.9265422077922079e-04*Jinv01*w27+1.0135957792207792e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[12][4]=(0.0)*D;
  A[12][5]=(0.0)*D;
  A[12][6]=(-4.0543831168831168e-03*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+7.3965097402597403e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-4.0543831168831168e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.0271915584415584e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+9.5880681818181818e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+4.6570616883116886e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59-5.2049512987012989e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-8.2183441558441553e-05*Jinv12*w55-3.1960227272727273e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+1.1231737012987013e-03*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-5.5702110389610384e-04*w1*Jinv10-8.7662337662337668e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-1.7349837662337662e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+1.1231737012987013e-03*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33-5.2049512987012989e-04*w22*Jinv12-2.0271915584415584e-03*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+9.5880681818181818e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2+5.4788961038961036e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-2.7394480519480521e-05*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-2.5750811688311690e-03*w41*Jinv20+7.1225649350649346e-04*Jinv12*w13+5.4788961038961036e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+7.3965097402597403e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+4.6570616883116886e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-4.0543831168831168e-03*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-8.2183441558441553e-05*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-5.2049512987012989e-04*w21*Jinv02+1.1231737012987013e-03*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33-2.5750811688311690e-03*w39*Jinv00+7.1225649350649346e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+7.3965097402597403e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38-5.2049512987012989e-04*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-2.7394480519480521e-05*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+9.5880681818181818e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-2.7394480519480521e-05*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10-8.2183441558441553e-05*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-3.1960227272727273e-04*w57*Jinv00-3.1960227272727273e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33-5.5702110389610384e-04*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23-8.2183441558441553e-05*Jinv01*w54-8.2183441558441553e-05*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10-5.5702110389610384e-04*Jinv20*w2-8.7662337662337668e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-8.7662337662337668e-04*Jinv20*w17-2.0271915584415584e-03*w30*Jinv00-2.5750811688311690e-03*w40*Jinv10+5.4788961038961036e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+7.1225649350649346e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+4.6570616883116886e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[12][7]=(0.0)*D;
  A[12][8]=(0.0)*D;
  A[12][9]=(1.4975649350649350e-03*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-2.6481331168831169e-04*Jinv20*w8+1.4975649350649350e-03*w13*Jinv10+3.6525974025974026e-05*w3*Jinv00+7.4878246753246752e-04*w32*Jinv20-5.8441558441558442e-04*Jinv20*w23-2.1915584415584417e-04*Jinv00*w54+3.4699675324675324e-04*w43*Jinv10+1.0247564935064935e-04*w58*Jinv10-4.2918019480519480e-04*w48*Jinv00+6.4935064935064935e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10-4.2918019480519480e-04*w45*Jinv00-1.4204545454545455e-05*Jinv00*w27-4.2918019480519480e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+2.1915584415584417e-04*Jinv00*w33+7.4878246753246752e-04*w31*Jinv10-5.8441558441558442e-04*w21*Jinv00-1.0957792207792208e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.7394480519480521e-05*w51*Jinv00+3.6525974025974026e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-1.0957792207792208e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10-2.6481331168831169e-04*w6*Jinv00-2.1915584415584417e-04*Jinv20*w56+1.1870941558441559e-04*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20+1.4975649350649350e-03*Jinv20*w14-4.2918019480519480e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-2.6481331168831169e-04*w7*Jinv10-5.4788961038961042e-05*Jinv20*w38-2.7394480519480521e-05*Jinv20*w53-5.8441558441558442e-04*w22*Jinv10-2.7394480519480521e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+3.4699675324675324e-04*Jinv20*w44+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+4.3831168831168834e-04*Jinv20*w17+7.4878246753246752e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20+3.6525974025974026e-05*w5*Jinv20-2.1915584415584417e-04*w55*Jinv10+3.4699675324675324e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20)*D;
  A[12][10]=(0.0)*D;
  A[12][11]=(0.0)*D;
  A[12][12]=(-7.3965097402597406e-03*Jinv00*w12-6.5746753246753243e-04*w28*Jinv10-4.7666396103896100e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+2.0271915584415584e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02-7.3965097402597406e-03*w13*Jinv10-4.7666396103896100e-03*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+1.0683847402597404e-03*Jinv00*w54-5.7528409090909093e-04*w25*Jinv12+9.1314935064935068e-05*w0*Jinv02+1.1870941558441558e-03*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-7.3965097402597403e-04*Jinv01*w45+1.0683847402597404e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.7666396103896100e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+2.0271915584415584e-03*w48*Jinv00-5.7528409090909093e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35-7.3965097402597403e-04*Jinv21*w26+1.1870941558441558e-03*w0*Jinv01+9.1314935064935068e-05*w1*Jinv10-4.7666396103896100e-03*w16*Jinv10+1.0683847402597404e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-5.7528409090909093e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-6.5746753246753243e-04*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22+1.1870941558441558e-03*Jinv11*w1-4.9310064935064932e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11+2.0271915584415584e-03*Jinv20*w50-6.5746753246753243e-04*Jinv20*w29-5.7528409090909093e-04*w46*Jinv10-4.7666396103896100e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.3149350649350649e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39+9.1314935064935068e-05*Jinv22*w2+1.0683847402597404e-03*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-5.7528409090909093e-04*Jinv02*w45-7.3965097402597403e-04*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+3.5612824675324673e-04*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11-4.7666396103896100e-03*w41*Jinv20-7.3965097402597406e-03*Jinv12*w13+1.0683847402597404e-03*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+2.0271915584415584e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+1.0683847402597404e-03*Jinv20*w56-4.7666396103896100e-03*Jinv22*w41-5.7528409090909093e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-5.7528409090909093e-04*Jinv22*w26-7.3965097402597403e-04*w46*Jinv11-7.3965097402597406e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+1.0683847402597404e-03*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02+2.0271915584415584e-03*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-4.7666396103896100e-03*w39*Jinv00-7.3965097402597406e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+2.0271915584415584e-03*w7*Jinv10-7.3965097402597403e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-2.4655032467532467e-03*w31*Jinv12-5.7528409090909093e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+2.0271915584415584e-03*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10+2.0271915584415584e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+2.0271915584415584e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01-2.4655032467532467e-03*Jinv02*w30+2.0271915584415584e-03*Jinv12*w49+3.5612824675324673e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-5.7528409090909093e-04*w24*Jinv00-6.5746753246753243e-04*Jinv02*w27+2.0271915584415584e-03*w6*Jinv02+1.0683847402597404e-03*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5-5.7528409090909093e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11-7.4878246753246752e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+1.0683847402597404e-03*Jinv02*w18-3.5612824675324673e-04*Jinv21*w8-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+9.1314935064935068e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-5.7528409090909093e-04*w25*Jinv10+9.1314935064935068e-05*Jinv20*w2-4.7666396103896100e-03*w15*Jinv00+2.0271915584415584e-03*w48*Jinv02-4.7666396103896100e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-4.7666396103896100e-03*w40*Jinv10+1.0683847402597404e-03*w20*Jinv20-6.5746753246753243e-04*Jinv22*w29-7.3965097402597406e-03*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+1.0683847402597404e-03*Jinv22*w56-6.5746753246753243e-04*Jinv12*w28-7.3965097402597403e-04*w25*Jinv11+1.0683847402597404e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00-5.7528409090909093e-04*w47*Jinv20-4.7666396103896100e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+9.1314935064935068e-05*Jinv12*w1)*D;
  A[12][13]=(0.0)*D;
  A[12][14]=(0.0)*D;
  A[12][15]=(9.5332792207792200e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.9724025974025973e-03*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53-1.8628246753246754e-03*Jinv20*w8+4.2004870129870131e-04*w57*Jinv02+9.5332792207792200e-03*w13*Jinv10+1.9724025974025973e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+2.0819805194805196e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20+2.0819805194805196e-03*Jinv01*w3-2.7942370129870131e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38-1.3149350649350649e-03*Jinv00*w54+6.9399350649350649e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+9.8620129870129864e-04*w0*Jinv02+4.3831168831168834e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+5.6615259740259744e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17+9.3141233766233771e-04*Jinv01*w45-1.1505681818181819e-03*Jinv12*w55+9.6793831168831166e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12-2.5750811688311690e-03*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+6.9399350649350649e-04*w58*Jinv11+6.9399350649350649e-04*w57*Jinv01+1.8628246753246754e-03*Jinv21*w26+4.3831168831168834e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10+6.9034090909090911e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+2.9586038961038961e-03*Jinv21*w41+4.9310064935064934e-03*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+7.3051948051948052e-05*w11*Jinv22+4.3831168831168834e-04*Jinv11*w1+9.8620129870129864e-04*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-2.5750811688311690e-03*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10-2.1915584415584414e-03*Jinv02*w3-2.7942370129870131e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.9586038961038961e-03*Jinv01*w39+9.8620129870129864e-04*Jinv22*w2-3.2873376623376621e-04*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45+1.8628246753246754e-03*Jinv01*w24+4.2004870129870131e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-6.0267857142857139e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53-8.2183441558441553e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+4.9310064935064934e-03*Jinv22*w44+2.0819805194805196e-03*w4*Jinv10+2.0819805194805196e-03*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-3.2873376623376621e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+1.0409902597402598e-03*w32*Jinv21-6.0267857142857139e-04*Jinv21*w20-1.8628246753246754e-03*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12-1.3149350649350649e-03*Jinv20*w56+1.9724025974025973e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20-7.6704545454545450e-04*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35+1.0409902597402598e-03*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+9.3141233766233771e-04*w46*Jinv11+9.5332792207792200e-03*Jinv20*w14+2.0819805194805196e-03*w5*Jinv21-1.1505681818181819e-03*Jinv02*w54-2.1915584415584414e-03*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-2.5750811688311690e-03*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00-3.2873376623376621e-03*Jinv02*w12-6.0267857142857139e-04*Jinv01*w18-3.2873376623376621e-04*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01-1.8628246753246754e-03*w7*Jinv10+9.3141233766233771e-04*w47*Jinv21+2.1367694805194807e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12-8.2183441558441553e-04*Jinv20*w53+7.6704545454545450e-04*Jinv22*w8-2.7942370129870131e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+7.6704545454545450e-04*Jinv12*w7-8.2183441558441553e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-3.2873376623376621e-04*w36*Jinv01-8.2183441558441553e-04*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00-1.2053571428571428e-03*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.1915584415584414e-03*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-1.2601461038961039e-03*w49*Jinv11+9.8620129870129864e-04*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+4.2004870129870131e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18+9.8620129870129864e-04*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8+9.6793831168831166e-04*w57*Jinv00+9.6793831168831166e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20-1.2053571428571428e-03*Jinv01*w54-1.2053571428571428e-03*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2+6.9034090909090911e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.6615259740259744e-04*Jinv01*w9+6.9034090909090911e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-2.1915584415584417e-04*Jinv22*w29-3.2873376623376621e-03*Jinv22*w14+2.9586038961038961e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+4.9310064935064934e-03*Jinv12*w43+2.0819805194805196e-03*w5*Jinv20+1.0409902597402598e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-1.1505681818181819e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+1.8628246753246754e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-1.2601461038961039e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-1.2601461038961039e-03*w48*Jinv01+9.8620129870129864e-04*Jinv12*w1+5.6615259740259744e-04*w10*Jinv11)*D;
  A[12][16]=(0.0)*D;
  A[12][17]=(0.0)*D;
  A[12][18]=(-2.1367694805194807e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10-5.4788961038961042e-05*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+1.4793019480519481e-03*Jinv20*w23+2.4655032467532466e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-2.1915584415584417e-04*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+1.8262987012987013e-05*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-5.4788961038961042e-05*w43*Jinv11+1.3971185064935066e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+1.4793019480519481e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-1.0683847402597404e-03*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+5.2049512987012989e-04*Jinv01*w24-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53+4.6570616883116886e-04*w51*Jinv00+3.8352272727272725e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20-1.0683847402597404e-03*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+2.4655032467532466e-04*Jinv20*w56+8.2183441558441553e-05*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20-3.8352272727272725e-04*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+5.4788961038961036e-04*w49*Jinv10-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00-7.1225649350649346e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+2.7394480519480518e-04*w52*Jinv11+4.6570616883116886e-04*Jinv20*w53+1.4793019480519481e-03*w22*Jinv10+5.2049512987012989e-04*w22*Jinv11+5.2049512987012989e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-3.5612824675324673e-04*w36*Jinv01+4.6570616883116886e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.8262987012987013e-05*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+5.2049512987012989e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10+1.8262987012987013e-05*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.0683847402597404e-03*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+5.2049512987012989e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[12][19]=(0.0)*D;
  A[12][20]=(0.0)*D;
  A[12][21]=(-1.1505681818181819e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3+2.6298701298701297e-03*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+4.7483766233766234e-04*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+2.7394480519480518e-04*w11*Jinv21+1.4793019480519481e-03*w16*Jinv11+1.4793019480519481e-03*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12-5.2049512987012989e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01+4.7483766233766234e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00+1.4793019480519481e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+2.6298701298701297e-03*w13*Jinv11+1.4793019480519481e-03*w15*Jinv01+4.0178571428571428e-04*w28*Jinv11-5.2049512987012989e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.6298701298701297e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+2.6298701298701297e-03*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+1.4793019480519481e-03*Jinv01*w39+4.7483766233766234e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480521e-05*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+1.1505681818181819e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-9.5880681818181818e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-9.5880681818181818e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12+2.6298701298701297e-03*w21*Jinv02-5.2049512987012989e-04*w49*Jinv10+4.0178571428571428e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12-9.5880681818181818e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10+1.1505681818181819e-03*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+2.6298701298701297e-03*Jinv22*w23-2.7394480519480521e-05*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-5.2049512987012989e-04*Jinv22*w8+2.6298701298701297e-03*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11-5.2049512987012989e-04*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49-2.7394480519480521e-05*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10-9.5880681818181818e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-5.2049512987012989e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+2.6298701298701297e-03*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18+2.6298701298701297e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.7483766233766234e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-9.5880681818181818e-04*Jinv01*w54-9.5880681818181818e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+4.7483766233766234e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+1.4793019480519481e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.1505681818181819e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+4.0178571428571428e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01+4.7483766233766234e-04*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[12][22]=(0.0)*D;
  A[12][23]=(0.0)*D;
  A[12][24]=(1.1505681818181819e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+5.2049512987012989e-04*Jinv20*w8+1.1505681818181819e-03*w13*Jinv10-2.5202922077922079e-03*Jinv21*w44-1.6436688311688311e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-3.1960227272727273e-04*Jinv21*w59-4.8396915584415583e-04*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10-2.5202922077922079e-03*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-2.7394480519480520e-03*w16*Jinv11-2.7394480519480520e-03*Jinv21*w17-1.0135957792207792e-03*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-3.1960227272727273e-04*w58*Jinv11-3.1960227272727273e-04*w57*Jinv01-2.0271915584415584e-03*Jinv21*w26-4.8396915584415583e-04*w0*Jinv01-4.7483766233766234e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-2.5202922077922079e-03*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-4.8396915584415583e-04*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11-2.7394480519480520e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33-1.6436688311688311e-04*w31*Jinv10-2.6298701298701297e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.3971185064935066e-03*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38-2.0271915584415584e-03*Jinv01*w24-7.1225649350649346e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+4.6570616883116886e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00-1.6436688311688311e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+1.3971185064935066e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+1.9176136363636362e-04*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+5.2049512987012989e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.3833198051948050e-03*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-7.1225649350649346e-04*Jinv21*w35+1.9176136363636362e-04*w31*Jinv11-1.0135957792207792e-03*w46*Jinv11+1.1505681818181819e-03*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+4.3831168831168834e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.3697240259740259e-04*w6*Jinv01+5.2049512987012989e-04*w7*Jinv10-1.0135957792207792e-03*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38+4.6570616883116886e-04*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-2.6298701298701297e-03*w22*Jinv10-1.2327516233766234e-03*w22*Jinv11-1.2327516233766234e-03*w21*Jinv01+4.6570616883116886e-04*Jinv01*w51+2.4655032467532466e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+1.3697240259740259e-04*w7*Jinv11-4.2918019480519480e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+8.4922889610389610e-04*Jinv21*w56-2.3833198051948050e-03*w24*Jinv00+2.4655032467532466e-04*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.2918019480519480e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+1.4793019480519481e-03*Jinv01*w12+1.3697240259740259e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-4.7483766233766234e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+8.4922889610389610e-04*Jinv01*w54+8.4922889610389610e-04*w55*Jinv11-2.3833198051948050e-03*w25*Jinv10-4.7483766233766234e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00-1.7349837662337662e-04*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11-1.6436688311688311e-04*w5*Jinv20+1.9176136363636362e-04*Jinv01*w30-2.0271915584415584e-03*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+2.4655032467532466e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-1.7349837662337662e-04*w10*Jinv11)*D;
  A[12][25]=(0.0)*D;
  A[12][26]=(0.0)*D;
  A[12][27]=(2.9220779220779221e-04*Jinv21*w44+3.6525974025974026e-05*Jinv01*w3-1.1668019480519480e-04*Jinv21*w59+8.3198051948051951e-05*Jinv21*w2+2.9220779220779221e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17-6.7573051948051951e-04*Jinv01*w45-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-6.7573051948051951e-04*Jinv21*w26+8.3198051948051951e-05*w0*Jinv01+2.9220779220779221e-04*w43*Jinv11-3.2873376623376621e-04*Jinv21*w41+8.3198051948051951e-05*Jinv11*w1-2.5568181818181820e-04*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11-3.2873376623376621e-04*Jinv01*w39-2.7394480519480521e-05*Jinv21*w38-6.7573051948051951e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53+3.6525974025974026e-05*w4*Jinv11+3.6525974025974026e-05*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35+3.6525974025974026e-05*w31*Jinv11-6.7573051948051951e-04*w46*Jinv11+3.6525974025974026e-05*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+4.9310064935064932e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11+1.1870941558441559e-04*w6*Jinv01-6.7573051948051951e-04*w47*Jinv21-2.7394480519480521e-05*w52*Jinv11-1.0227272727272728e-03*w22*Jinv11-1.0227272727272728e-03*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51-2.7394480519480521e-05*w36*Jinv01+1.1870941558441559e-04*w7*Jinv11+4.9310064935064932e-04*Jinv21*w56+1.1870941558441559e-04*w49*Jinv11-2.5568181818181820e-04*Jinv21*w14-2.5568181818181820e-04*Jinv01*w12+1.1870941558441559e-04*Jinv21*w8-1.0227272727272728e-03*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-1.1668019480519480e-04*Jinv01*w9-3.2873376623376621e-04*w40*Jinv11+3.6525974025974026e-05*Jinv01*w30-6.7573051948051951e-04*w25*Jinv11+1.1870941558441559e-04*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27+1.1870941558441559e-04*w48*Jinv01-1.1668019480519480e-04*w10*Jinv11)*D;
  A[12][28]=(0.0)*D;
  A[12][29]=(0.0)*D;
  A[12][30]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+1.0135957792207792e-03*Jinv20*w8+3.5612824675324673e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44-1.2327516233766234e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20-1.2327516233766234e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+1.6436688311688311e-04*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+1.1414366883116883e-03*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10-1.0957792207792207e-03*w42*Jinv01-3.7439123376623376e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10+2.6846590909090908e-03*w15*Jinv02+3.7439123376623376e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00-8.2183441558441553e-05*Jinv02*w24+5.4788961038961036e-04*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-3.1777597402597403e-03*Jinv21*w41-3.9265422077922079e-04*Jinv00*w27+3.7439123376623376e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.9265422077922079e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+2.6846590909090908e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33-1.3971185064935066e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10+2.4655032467532467e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-3.1777597402597403e-03*Jinv01*w39+1.1414366883116883e-03*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38+8.2183441558441553e-05*Jinv01*w24+3.5612824675324673e-04*Jinv22*w59-1.5888798701298701e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00+5.4788961038961036e-04*Jinv12*w34-1.2327516233766234e-03*w4*Jinv10-1.2327516233766234e-03*w4*Jinv11-3.1777597402597403e-03*w41*Jinv20+4.9310064935064934e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+1.0135957792207792e-03*w6*Jinv00+1.6436688311688311e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-3.7439123376623376e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-1.5888798701298701e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14-1.2327516233766234e-03*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+2.4655032467532467e-03*w4*Jinv12-1.3971185064935066e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10-3.9265422077922079e-04*Jinv21*w29-1.5888798701298701e-03*Jinv01*w33-3.1777597402597403e-03*w39*Jinv00+4.9310064935064934e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11+3.7439123376623376e-04*Jinv02*w9+1.0135957792207792e-03*w6*Jinv01+1.0135957792207792e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38-1.3971185064935066e-03*Jinv22*w23+8.2183441558441553e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10-1.1505681818181819e-03*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.1505681818181819e-03*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10+1.0135957792207792e-03*w7*Jinv11-3.7439123376623376e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00+3.9265422077922079e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+2.4655032467532467e-03*Jinv22*w5+1.2327516233766234e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14-3.7439123376623376e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44+3.5612824675324673e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.0135957792207792e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59+5.4788961038961036e-04*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.1505681818181819e-03*w48*Jinv02-3.7439123376623376e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-3.1777597402597403e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14-3.1777597402597403e-03*w40*Jinv11-1.2327516233766234e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+2.6846590909090908e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.9265422077922079e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+1.1414366883116883e-03*Jinv12*w1-3.7439123376623376e-04*w10*Jinv11)*D;
  A[12][31]=(0.0)*D;
  A[12][32]=(0.0)*D;
  A[12][33]=(3.1777597402597403e-03*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12-9.3141233766233771e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+3.2873376623376621e-03*Jinv21*w44+1.0409902597402598e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20-1.0957792207792207e-03*Jinv01*w3-1.7532467532467534e-03*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+3.2873376623376621e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.2962662337662338e-04*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12-1.7532467532467534e-03*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+3.2873376623376621e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.7532467532467534e-03*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33-1.7532467532467534e-03*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3-1.7532467532467534e-03*w21*Jinv00+1.0409902597402598e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44+1.0409902597402598e-03*w4*Jinv10-1.0957792207792207e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10-1.0957792207792207e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20-9.3141233766233771e-04*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+1.3149350649350649e-03*Jinv21*w35-1.0957792207792207e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11+3.1777597402597403e-03*Jinv20*w14-1.0957792207792207e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-1.7532467532467534e-03*w21*Jinv02-1.7532467532467534e-03*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01-9.3141233766233771e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-1.7532467532467534e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11+1.0409902597402598e-03*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53-1.7532467532467534e-03*Jinv22*w8-1.7532467532467534e-03*w22*Jinv10-9.3141233766233771e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-1.7532467532467534e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51+1.0409902597402598e-03*Jinv02*w30-9.3141233766233771e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00+5.4788961038961042e-05*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-1.7532467532467534e-03*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00-9.3141233766233771e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10+5.2962662337662338e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+1.0409902597402598e-03*w5*Jinv20-1.0957792207792207e-03*Jinv01*w30+5.2962662337662338e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[12][34]=(0.0)*D;
  A[12][35]=(0.0)*D;
  A[12][36]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8-1.0957792207792208e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+1.9176136363636362e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+5.2049512987012989e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-8.7662337662337668e-04*w43*Jinv10+8.2183441558441553e-05*Jinv12*w55-1.3697240259740259e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.2049512987012989e-04*w48*Jinv00+5.2049512987012989e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+5.4788961038961036e-04*w45*Jinv00-5.4788961038961042e-05*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+5.2049512987012989e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45-1.0957792207792208e-04*Jinv22*w59+2.4655032467532466e-04*w51*Jinv00-3.8352272727272725e-04*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+1.9176136363636362e-04*w4*Jinv10-1.0957792207792208e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-8.2183441558441553e-05*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12+5.2049512987012989e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+8.2183441558441553e-05*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02+5.2049512987012989e-04*w49*Jinv10-1.0957792207792208e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-8.2183441558441553e-05*w7*Jinv10-3.5612824675324673e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23-3.5612824675324673e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-3.5612824675324673e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+2.4655032467532466e-04*w52*Jinv10+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.4655032467532466e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+6.3920454545454550e-05*w10*Jinv10-8.7662337662337668e-04*Jinv20*w44-1.0957792207792208e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-3.8352272727272725e-04*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.4655032467532466e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-3.5612824675324673e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+1.9176136363636362e-04*w5*Jinv20+2.7394480519480518e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+5.4788961038961036e-04*w55*Jinv10-8.7662337662337668e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[12][37]=(0.0)*D;
  A[12][38]=(0.0)*D;
  A[12][39]=(-3.2873376623376621e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+6.9034090909090911e-03*w40*Jinv12+5.4788961038961042e-05*Jinv20*w8+6.2094155844155847e-04*w57*Jinv02-3.2873376623376621e-03*w13*Jinv10+6.9034090909090911e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-2.1915584415584414e-03*w32*Jinv20+1.0409902597402598e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-8.2183441558441553e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+5.6615259740259744e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02+4.3831168831168834e-04*Jinv21*w2+4.9310064935064934e-03*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+6.9399350649350649e-04*w11*Jinv21+2.9586038961038961e-03*w16*Jinv11+2.9586038961038961e-03*Jinv21*w17+1.8628246753246754e-03*Jinv01*w45+7.3051948051948052e-05*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02+9.6793831168831166e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+5.6615259740259744e-04*w58*Jinv11+5.6615259740259744e-04*w57*Jinv01+9.3141233766233771e-04*Jinv21*w26+4.3831168831168834e-04*w0*Jinv01+9.8620129870129864e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10-1.3149350649350649e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+9.6793831168831166e-04*w11*Jinv22+4.3831168831168834e-04*Jinv11*w1+9.8620129870129864e-04*w13*Jinv11+2.9586038961038961e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-2.7942370129870131e-03*w22*Jinv12-2.1915584415584414e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00+2.0819805194805196e-03*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2-1.1505681818181819e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+9.3141233766233771e-04*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-1.2053571428571428e-03*w19*Jinv11-3.2873376623376621e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+1.0409902597402598e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+9.5332792207792200e-03*Jinv12*w13-1.1505681818181819e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+2.0819805194805196e-03*w32*Jinv21-1.2053571428571428e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-8.2183441558441553e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+6.9034090909090911e-03*Jinv22*w41+1.3149350649350649e-03*w26*Jinv20+4.2004870129870131e-04*w11*Jinv20+2.3011363636363637e-03*Jinv21*w35+2.0819805194805196e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26+1.8628246753246754e-03*w46*Jinv11-3.2873376623376621e-03*Jinv20*w14+1.0409902597402598e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-2.7942370129870131e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+9.5332792207792200e-03*Jinv02*w12-1.2053571428571428e-03*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11+9.6793831168831166e-04*Jinv02*w9-1.2601461038961039e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+1.8628246753246754e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38-2.7942370129870131e-03*Jinv22*w23-3.2873376623376621e-04*w52*Jinv11+2.0819805194805196e-03*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53-2.5750811688311690e-03*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-1.8628246753246754e-03*Jinv22*w50-2.5750811688311690e-03*Jinv12*w7-3.2873376623376621e-04*Jinv01*w51+2.0819805194805196e-03*Jinv02*w30-1.8628246753246754e-03*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.2601461038961039e-03*w7*Jinv11+4.2004870129870131e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-6.0267857142857139e-04*Jinv21*w56+1.3149350649350649e-03*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27-2.5750811688311690e-03*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+9.8620129870129864e-04*Jinv21*w14+4.2004870129870131e-04*w10*Jinv10+4.9310064935064934e-03*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18+9.8620129870129864e-04*Jinv01*w12-1.2601461038961039e-03*Jinv21*w8+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+9.8620129870129864e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-6.0267857142857139e-04*Jinv01*w54-6.0267857142857139e-04*w55*Jinv11+1.3149350649350649e-03*w25*Jinv10+9.8620129870129864e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00-1.8628246753246754e-03*w48*Jinv02+6.9399350649350649e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-2.1915584415584414e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-1.1505681818181819e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29+9.5332792207792200e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11-8.2183441558441553e-04*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+2.0819805194805196e-03*Jinv01*w30+4.3831168831168834e-04*Jinv12*w28+9.3141233766233771e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+4.9310064935064934e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1+6.9399350649350649e-04*w10*Jinv11)*D;
  A[12][40]=(0.0)*D;
  A[12][41]=(0.0)*D;
  A[12][42]=(3.2873376623376621e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-4.2004870129870131e-04*w57*Jinv02+3.2873376623376621e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39-3.2873376623376621e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+2.1915584415584414e-03*w32*Jinv20+1.0957792207792207e-03*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-9.8620129870129864e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55-7.3051948051948052e-05*w58*Jinv10-3.9448051948051946e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-9.8620129870129864e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-3.2873376623376621e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-7.3051948051948052e-05*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10-3.9448051948051946e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+2.1915584415584414e-03*w31*Jinv10+2.1915584415584414e-03*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-9.8620129870129864e-04*Jinv22*w2+1.1505681818181819e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-4.2004870129870131e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+1.0957792207792207e-03*w4*Jinv11-3.9448051948051946e-03*w41*Jinv20+3.2873376623376621e-03*Jinv12*w13+1.1505681818181819e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+1.0957792207792207e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-1.3149350649350649e-03*w26*Jinv20-4.2004870129870131e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35+1.0957792207792207e-03*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-5.4788961038961042e-05*w46*Jinv11+3.2873376623376621e-03*Jinv20*w14+1.0957792207792207e-03*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54+2.1915584415584414e-03*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-3.9448051948051946e-03*w39*Jinv00+3.2873376623376621e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-4.2004870129870131e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.1915584415584414e-03*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-4.2004870129870131e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-4.2004870129870131e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-9.8620129870129864e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10-9.8620129870129864e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+2.1915584415584414e-03*w30*Jinv00-3.9448051948051946e-03*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+3.2873376623376621e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+1.0957792207792207e-03*Jinv01*w30+7.6704545454545450e-04*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20-3.9448051948051946e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-9.8620129870129864e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[12][43]=(0.0)*D;
  A[12][44]=(0.0)*D;
  A[12][45]=(-2.1367694805194807e-03*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-4.2918019480519480e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-2.5202922077922079e-03*Jinv21*w44+1.9176136363636362e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-1.7349837662337662e-04*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-4.7483766233766234e-04*w0*Jinv02-4.8396915584415583e-04*Jinv21*w2-2.5202922077922079e-03*w42*Jinv01-3.1960227272727273e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.0271915584415584e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55-2.1367694805194807e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01-1.0135957792207792e-03*Jinv21*w26-4.8396915584415583e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-2.5202922077922079e-03*w43*Jinv11-2.7394480519480520e-03*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-4.8396915584415583e-04*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-2.1367694805194807e-03*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12-1.6436688311688311e-04*Jinv02*w3-1.6436688311688311e-04*w32*Jinv22-2.7394480519480520e-03*Jinv01*w39-4.7483766233766234e-04*Jinv22*w2+4.6570616883116886e-04*Jinv21*w38-2.3833198051948050e-03*Jinv02*w45-1.0135957792207792e-03*Jinv01*w24-4.2918019480519480e-04*Jinv22*w59-7.1225649350649346e-04*w34*Jinv11+8.4922889610389610e-04*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44+1.9176136363636362e-04*w4*Jinv11+1.1505681818181819e-03*Jinv12*w13+3.8352272727272725e-04*w32*Jinv21+8.4922889610389610e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-7.1225649350649346e-04*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-2.0271915584415584e-03*w46*Jinv11+1.9176136363636362e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33+1.1505681818181819e-03*Jinv02*w12+8.4922889610389610e-04*Jinv01*w18+4.6570616883116886e-04*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01-2.0271915584415584e-03*w47*Jinv21-2.6298701298701297e-03*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11-1.6436688311688311e-04*w31*Jinv12-2.3833198051948050e-03*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8+5.2049512987012989e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51-1.6436688311688311e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+4.6570616883116886e-04*w36*Jinv01+2.4655032467532466e-04*w7*Jinv11+4.3831168831168834e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-2.3833198051948050e-03*w47*Jinv22+1.3697240259740259e-04*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.2918019480519480e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.2327516233766234e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+5.2049512987012989e-04*w48*Jinv02-3.1960227272727273e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.1505681818181819e-03*Jinv22*w14-2.7394480519480520e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43+3.8352272727272725e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.0135957792207792e-03*w25*Jinv11-2.1367694805194807e-03*w16*Jinv12+1.3697240259740259e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.3697240259740259e-04*w48*Jinv01-4.7483766233766234e-04*Jinv12*w1-3.1960227272727273e-04*w10*Jinv11)*D;
  A[12][46]=(0.0)*D;
  A[12][47]=(0.0)*D;
  A[12][48]=(7.1225649350649346e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10-8.7662337662337668e-04*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+7.1225649350649346e-04*w13*Jinv10-8.7662337662337668e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-5.2049512987012989e-04*Jinv20*w23-2.7394480519480521e-05*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12-5.5702110389610384e-04*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+5.4788961038961036e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-2.5750811688311690e-03*w15*Jinv02-3.1960227272727273e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+4.6570616883116886e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-3.1960227272727273e-04*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10-2.5750811688311690e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+9.5880681818181818e-04*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10-2.0271915584415584e-03*Jinv02*w3-5.2049512987012989e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39-5.5702110389610384e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11-8.2183441558441553e-05*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20-4.0543831168831168e-03*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-8.2183441558441553e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480521e-05*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-8.7662337662337668e-04*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11+7.1225649350649346e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21+5.4788961038961036e-04*Jinv02*w54-2.0271915584415584e-03*w4*Jinv12+9.5880681818181818e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00-4.0543831168831168e-03*Jinv02*w12-8.2183441558441553e-05*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11-3.1960227272727273e-04*Jinv02*w9-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+9.5880681818181818e-04*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.1231737012987013e-03*Jinv22*w8-5.2049512987012989e-04*w22*Jinv10+7.3965097402597403e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+1.1231737012987013e-03*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30+7.3965097402597403e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-1.7349837662337662e-04*Jinv02*w27+1.1231737012987013e-03*w6*Jinv02+4.6570616883116886e-04*w19*Jinv12-2.0271915584415584e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+4.6570616883116886e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.3965097402597403e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20-1.7349837662337662e-04*Jinv22*w29-4.0543831168831168e-03*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11-2.7394480519480521e-05*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+5.4788961038961036e-04*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20-2.5750811688311690e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-5.5702110389610384e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[12][49]=(0.0)*D;
  A[12][50]=(0.0)*D;
  A[12][51]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+6.3920454545454550e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39-3.5612824675324673e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+5.4788961038961036e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-5.4788961038961042e-05*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+5.2049512987012989e-04*w45*Jinv00-8.7662337662337668e-04*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27-1.3697240259740259e-04*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+1.9176136363636362e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv02*w45+6.3920454545454550e-05*Jinv22*w59-3.5612824675324673e-04*w51*Jinv00-7.1225649350649346e-04*Jinv12*w34-8.7662337662337668e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10-7.6704545454545450e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10-7.6704545454545450e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12-1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+2.7394480519480518e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23+1.9176136363636362e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+5.2049512987012989e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50+5.2049512987012989e-04*Jinv12*w7+1.9176136363636362e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-3.5612824675324673e-04*w52*Jinv10-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-1.0957792207792208e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+6.3920454545454550e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-7.1225649350649346e-04*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+2.4655032467532466e-04*Jinv02*w36-8.7662337662337668e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[12][52]=(0.0)*D;
  A[12][53]=(0.0)*D;
  A[12][54]=(-2.1367694805194807e-03*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+1.2784090909090910e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3+4.6570616883116886e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+1.3971185064935066e-03*w25*Jinv12+1.8262987012987013e-05*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-1.0683847402597404e-03*Jinv12*w55-1.6436688311688311e-04*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+1.3971185064935066e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+2.4655032467532466e-04*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-2.1915584415584417e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-1.6436688311688311e-04*Jinv22*w17+1.4793019480519481e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3+3.8352272727272725e-04*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39+1.8262987012987013e-05*Jinv22*w2+2.7394480519480518e-04*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-3.8352272727272725e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+4.6570616883116886e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-3.8352272727272725e-04*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+1.3971185064935066e-03*Jinv22*w26+5.2049512987012989e-04*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-1.0683847402597404e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+1.4793019480519481e-03*w21*Jinv02-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01+5.2049512987012989e-04*w47*Jinv21+1.4793019480519481e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50+5.2049512987012989e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7+5.2049512987012989e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-7.1225649350649346e-04*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14+1.2784090909090910e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+5.2049512987012989e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+4.6570616883116886e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-1.0683847402597404e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-1.6436688311688311e-04*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+1.8262987012987013e-05*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[12][55]=(0.0)*D;
  A[12][56]=(0.0)*D;
  A[12][57]=(4.3831168831168834e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39-2.7394480519480521e-05*Jinv22*w38-4.2918019480519480e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-1.0957792207792208e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35-2.1915584415584417e-04*Jinv22*w20+3.4699675324675324e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-5.8441558441558442e-04*w22*Jinv12+7.4878246753246752e-04*Jinv02*w3+3.6525974025974026e-05*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2+1.1870941558441559e-04*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+2.1915584415584417e-04*Jinv12*w34+3.4699675324675324e-04*Jinv22*w44+1.4975649350649350e-03*Jinv12*w13-2.7394480519480521e-05*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-1.0957792207792208e-04*Jinv02*w54+7.4878246753246752e-04*w4*Jinv12-5.8441558441558442e-04*w21*Jinv02+1.4975649350649350e-03*Jinv02*w12+1.0247564935064935e-04*Jinv02*w9-5.8441558441558442e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12-4.2918019480519480e-04*Jinv22*w8-2.6481331168831169e-04*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7+3.6525974025974026e-05*Jinv02*w30-2.6481331168831169e-04*Jinv12*w49-1.4204545454545455e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+7.4878246753246752e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+2.1915584415584417e-04*Jinv02*w33-2.6481331168831169e-04*w48*Jinv02-1.4204545454545455e-05*Jinv22*w29+1.4975649350649350e-03*Jinv22*w14-2.7394480519480521e-05*Jinv02*w36+3.4699675324675324e-04*Jinv12*w43-5.4788961038961042e-05*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[12][58]=(0.0)*D;
  A[12][59]=(0.0)*D;
  A[13][0]=(0.0)*D;
  A[13][1]=(-2.3741883116883116e-03*Jinv00*w12-2.0495129870129870e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+6.7573051948051951e-04*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02-2.3741883116883116e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-1.1870941558441558e-03*w3*Jinv00-1.1870941558441558e-03*w32*Jinv20-1.1870941558441558e-03*Jinv01*w3+1.0227272727272728e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+1.0957792207792208e-04*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59+2.6481331168831169e-04*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+2.6481331168831169e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10-6.5746753246753243e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12+6.7573051948051951e-04*w48*Jinv00+2.6481331168831169e-04*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01+2.6481331168831169e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+1.0957792207792208e-04*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+2.6481331168831169e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-2.0495129870129870e-04*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-2.3741883116883116e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-2.0495129870129870e-04*w28*Jinv11+6.7573051948051951e-04*Jinv20*w50-2.0495129870129870e-04*Jinv20*w29+2.6481331168831169e-04*w46*Jinv10-6.5746753246753243e-04*Jinv22*w17-3.2873376623376621e-04*Jinv00*w33+1.0227272727272728e-03*w22*Jinv12-1.1870941558441558e-03*w31*Jinv10-1.1870941558441558e-03*Jinv02*w3+1.0227272727272728e-03*w21*Jinv00-1.1870941558441558e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38+2.6481331168831169e-04*Jinv02*w45+2.6481331168831169e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00-3.2873376623376621e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-1.1870941558441558e-03*w4*Jinv10-1.1870941558441558e-03*w4*Jinv11-6.5746753246753243e-04*w41*Jinv20-2.3741883116883116e-03*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-1.1870941558441558e-03*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+6.7573051948051951e-04*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+1.0957792207792208e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+2.6481331168831169e-04*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-1.1870941558441558e-03*w31*Jinv11+2.6481331168831169e-04*Jinv22*w26+2.6481331168831169e-04*w46*Jinv11-2.3741883116883116e-03*Jinv20*w14-1.1870941558441558e-03*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54-1.1870941558441558e-03*w4*Jinv12+1.0227272727272728e-03*w21*Jinv02+6.7573051948051951e-04*w49*Jinv10-2.0495129870129870e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33-6.5746753246753243e-04*w39*Jinv00-2.3741883116883116e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9+6.7573051948051951e-04*w6*Jinv01+6.7573051948051951e-04*w7*Jinv10+2.6481331168831169e-04*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38+1.0227272727272728e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.1870941558441558e-03*w31*Jinv12+2.6481331168831169e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+6.7573051948051951e-04*Jinv22*w8+1.0227272727272728e-03*w22*Jinv10+6.7573051948051951e-04*Jinv22*w50+1.0227272727272728e-03*w22*Jinv11+6.7573051948051951e-04*Jinv12*w7+1.0227272727272728e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.1870941558441558e-03*Jinv02*w30+6.7573051948051951e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10+6.7573051948051951e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.6481331168831169e-04*w24*Jinv00-2.0495129870129870e-04*Jinv02*w27+6.7573051948051951e-04*w6*Jinv02+1.0957792207792208e-04*w19*Jinv12-1.1870941558441558e-03*Jinv22*w5+2.6481331168831169e-04*w47*Jinv22+6.7573051948051951e-04*w49*Jinv11-2.3741883116883116e-03*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.0957792207792208e-04*Jinv02*w18-2.3741883116883116e-03*Jinv01*w12+6.7573051948051951e-04*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59-3.2873376623376621e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+1.0227272727272728e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.6481331168831169e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+6.7573051948051951e-04*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.1870941558441558e-03*w30*Jinv00-6.5746753246753243e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20-2.0495129870129870e-04*Jinv22*w29-2.3741883116883116e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-1.1870941558441558e-03*w5*Jinv20-1.1870941558441558e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56-2.0495129870129870e-04*Jinv12*w28+2.6481331168831169e-04*w25*Jinv11+1.0957792207792208e-04*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+2.6481331168831169e-04*w47*Jinv20-6.5746753246753243e-04*w16*Jinv12+6.7573051948051951e-04*Jinv21*w50-2.0495129870129870e-04*Jinv01*w27+6.7573051948051951e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[13][2]=(0.0)*D;
  A[13][3]=(0.0)*D;
  A[13][4]=(4.9310064935064934e-03*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-1.1505681818181819e-03*Jinv20*w8-3.7439123376623376e-04*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+2.4655032467532467e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3-1.3971185064935066e-03*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54-3.7439123376623376e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-3.1777597402597403e-03*w16*Jinv11-3.1777597402597403e-03*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55+3.7439123376623376e-04*w58*Jinv10-3.1777597402597403e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35-3.7439123376623376e-04*w58*Jinv11-3.7439123376623376e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+1.1414366883116883e-03*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02+3.9265422077922079e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-3.1777597402597403e-03*w15*Jinv01-3.9265422077922079e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10-3.1777597402597403e-03*Jinv22*w17+5.4788961038961036e-04*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12+2.4655032467532467e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3-1.3971185064935066e-03*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+8.2183441558441553e-05*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24-3.7439123376623376e-04*Jinv22*w59-1.5888798701298701e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11+2.6846590909090908e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+5.4788961038961036e-04*w34*Jinv10-1.2327516233766234e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20-1.1505681818181819e-03*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12-1.5888798701298701e-03*Jinv21*w35-1.2327516233766234e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10-3.9265422077922079e-04*Jinv21*w29-1.5888798701298701e-03*Jinv01*w33+2.6846590909090908e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01-1.1505681818181819e-03*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+1.2327516233766234e-03*Jinv22*w8-1.3971185064935066e-03*w22*Jinv10+1.0135957792207792e-03*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+1.0135957792207792e-03*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+1.2327516233766234e-03*w7*Jinv11+3.5612824675324673e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-3.9265422077922079e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22+1.0135957792207792e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14+3.5612824675324673e-04*w10*Jinv10-3.7439123376623376e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8+3.7439123376623376e-04*w57*Jinv00+3.7439123376623376e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+1.1414366883116883e-03*w0*Jinv00+5.4788961038961036e-04*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+1.1414366883116883e-03*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+1.0135957792207792e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9+1.0957792207792207e-03*Jinv20*w17+2.4655032467532467e-03*w30*Jinv00+2.6846590909090908e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.2327516233766234e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10-8.2183441558441553e-05*w47*Jinv20-3.1777597402597403e-03*w16*Jinv12+1.0135957792207792e-03*Jinv21*w50-3.9265422077922079e-04*Jinv01*w27+1.0135957792207792e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[13][5]=(0.0)*D;
  A[13][6]=(0.0)*D;
  A[13][7]=(-4.0543831168831168e-03*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+7.3965097402597403e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-4.0543831168831168e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.0271915584415584e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+9.5880681818181818e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+4.6570616883116886e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59-5.2049512987012989e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-8.2183441558441553e-05*Jinv12*w55-3.1960227272727273e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+1.1231737012987013e-03*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-5.5702110389610384e-04*w1*Jinv10-8.7662337662337668e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-1.7349837662337662e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+1.1231737012987013e-03*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33-5.2049512987012989e-04*w22*Jinv12-2.0271915584415584e-03*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+9.5880681818181818e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2+5.4788961038961036e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-2.7394480519480521e-05*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-2.5750811688311690e-03*w41*Jinv20+7.1225649350649346e-04*Jinv12*w13+5.4788961038961036e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+7.3965097402597403e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+4.6570616883116886e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-4.0543831168831168e-03*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-8.2183441558441553e-05*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-5.2049512987012989e-04*w21*Jinv02+1.1231737012987013e-03*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33-2.5750811688311690e-03*w39*Jinv00+7.1225649350649346e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+7.3965097402597403e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38-5.2049512987012989e-04*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-2.7394480519480521e-05*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+9.5880681818181818e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-2.7394480519480521e-05*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10-8.2183441558441553e-05*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-3.1960227272727273e-04*w57*Jinv00-3.1960227272727273e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33-5.5702110389610384e-04*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23-8.2183441558441553e-05*Jinv01*w54-8.2183441558441553e-05*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10-5.5702110389610384e-04*Jinv20*w2-8.7662337662337668e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-8.7662337662337668e-04*Jinv20*w17-2.0271915584415584e-03*w30*Jinv00-2.5750811688311690e-03*w40*Jinv10+5.4788961038961036e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+7.1225649350649346e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+4.6570616883116886e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[13][8]=(0.0)*D;
  A[13][9]=(0.0)*D;
  A[13][10]=(1.4975649350649350e-03*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-2.6481331168831169e-04*Jinv20*w8+1.4975649350649350e-03*w13*Jinv10+3.6525974025974026e-05*w3*Jinv00+7.4878246753246752e-04*w32*Jinv20-5.8441558441558442e-04*Jinv20*w23-2.1915584415584417e-04*Jinv00*w54+3.4699675324675324e-04*w43*Jinv10+1.0247564935064935e-04*w58*Jinv10-4.2918019480519480e-04*w48*Jinv00+6.4935064935064935e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10-4.2918019480519480e-04*w45*Jinv00-1.4204545454545455e-05*Jinv00*w27-4.2918019480519480e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+2.1915584415584417e-04*Jinv00*w33+7.4878246753246752e-04*w31*Jinv10-5.8441558441558442e-04*w21*Jinv00-1.0957792207792208e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.7394480519480521e-05*w51*Jinv00+3.6525974025974026e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-1.0957792207792208e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10-2.6481331168831169e-04*w6*Jinv00-2.1915584415584417e-04*Jinv20*w56+1.1870941558441559e-04*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20+1.4975649350649350e-03*Jinv20*w14-4.2918019480519480e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-2.6481331168831169e-04*w7*Jinv10-5.4788961038961042e-05*Jinv20*w38-2.7394480519480521e-05*Jinv20*w53-5.8441558441558442e-04*w22*Jinv10-2.7394480519480521e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+3.4699675324675324e-04*Jinv20*w44+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+4.3831168831168834e-04*Jinv20*w17+7.4878246753246752e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20+3.6525974025974026e-05*w5*Jinv20-2.1915584415584417e-04*w55*Jinv10+3.4699675324675324e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20)*D;
  A[13][11]=(0.0)*D;
  A[13][12]=(0.0)*D;
  A[13][13]=(-7.3965097402597406e-03*Jinv00*w12-6.5746753246753243e-04*w28*Jinv10-4.7666396103896100e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+2.0271915584415584e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02-7.3965097402597406e-03*w13*Jinv10-4.7666396103896100e-03*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+1.0683847402597404e-03*Jinv00*w54-5.7528409090909093e-04*w25*Jinv12+9.1314935064935068e-05*w0*Jinv02+1.1870941558441558e-03*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-7.3965097402597403e-04*Jinv01*w45+1.0683847402597404e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.7666396103896100e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+2.0271915584415584e-03*w48*Jinv00-5.7528409090909093e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35-7.3965097402597403e-04*Jinv21*w26+1.1870941558441558e-03*w0*Jinv01+9.1314935064935068e-05*w1*Jinv10-4.7666396103896100e-03*w16*Jinv10+1.0683847402597404e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-5.7528409090909093e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-6.5746753246753243e-04*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22+1.1870941558441558e-03*Jinv11*w1-4.9310064935064932e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11+2.0271915584415584e-03*Jinv20*w50-6.5746753246753243e-04*Jinv20*w29-5.7528409090909093e-04*w46*Jinv10-4.7666396103896100e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.3149350649350649e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39+9.1314935064935068e-05*Jinv22*w2+1.0683847402597404e-03*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-5.7528409090909093e-04*Jinv02*w45-7.3965097402597403e-04*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+3.5612824675324673e-04*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11-4.7666396103896100e-03*w41*Jinv20-7.3965097402597406e-03*Jinv12*w13+1.0683847402597404e-03*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+2.0271915584415584e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+1.0683847402597404e-03*Jinv20*w56-4.7666396103896100e-03*Jinv22*w41-5.7528409090909093e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-5.7528409090909093e-04*Jinv22*w26-7.3965097402597403e-04*w46*Jinv11-7.3965097402597406e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+1.0683847402597404e-03*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02+2.0271915584415584e-03*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-4.7666396103896100e-03*w39*Jinv00-7.3965097402597406e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+2.0271915584415584e-03*w7*Jinv10-7.3965097402597403e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-2.4655032467532467e-03*w31*Jinv12-5.7528409090909093e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+2.0271915584415584e-03*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10+2.0271915584415584e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+2.0271915584415584e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01-2.4655032467532467e-03*Jinv02*w30+2.0271915584415584e-03*Jinv12*w49+3.5612824675324673e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-5.7528409090909093e-04*w24*Jinv00-6.5746753246753243e-04*Jinv02*w27+2.0271915584415584e-03*w6*Jinv02+1.0683847402597404e-03*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5-5.7528409090909093e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11-7.4878246753246752e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+1.0683847402597404e-03*Jinv02*w18-3.5612824675324673e-04*Jinv21*w8-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+9.1314935064935068e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-5.7528409090909093e-04*w25*Jinv10+9.1314935064935068e-05*Jinv20*w2-4.7666396103896100e-03*w15*Jinv00+2.0271915584415584e-03*w48*Jinv02-4.7666396103896100e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-4.7666396103896100e-03*w40*Jinv10+1.0683847402597404e-03*w20*Jinv20-6.5746753246753243e-04*Jinv22*w29-7.3965097402597406e-03*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+1.0683847402597404e-03*Jinv22*w56-6.5746753246753243e-04*Jinv12*w28-7.3965097402597403e-04*w25*Jinv11+1.0683847402597404e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00-5.7528409090909093e-04*w47*Jinv20-4.7666396103896100e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+9.1314935064935068e-05*Jinv12*w1)*D;
  A[13][14]=(0.0)*D;
  A[13][15]=(0.0)*D;
  A[13][16]=(9.5332792207792200e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.9724025974025973e-03*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53-1.8628246753246754e-03*Jinv20*w8+4.2004870129870131e-04*w57*Jinv02+9.5332792207792200e-03*w13*Jinv10+1.9724025974025973e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+2.0819805194805196e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20+2.0819805194805196e-03*Jinv01*w3-2.7942370129870131e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38-1.3149350649350649e-03*Jinv00*w54+6.9399350649350649e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+9.8620129870129864e-04*w0*Jinv02+4.3831168831168834e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+5.6615259740259744e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17+9.3141233766233771e-04*Jinv01*w45-1.1505681818181819e-03*Jinv12*w55+9.6793831168831166e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12-2.5750811688311690e-03*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+6.9399350649350649e-04*w58*Jinv11+6.9399350649350649e-04*w57*Jinv01+1.8628246753246754e-03*Jinv21*w26+4.3831168831168834e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10+6.9034090909090911e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+2.9586038961038961e-03*Jinv21*w41+4.9310064935064934e-03*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+7.3051948051948052e-05*w11*Jinv22+4.3831168831168834e-04*Jinv11*w1+9.8620129870129864e-04*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-2.5750811688311690e-03*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10-2.1915584415584414e-03*Jinv02*w3-2.7942370129870131e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.9586038961038961e-03*Jinv01*w39+9.8620129870129864e-04*Jinv22*w2-3.2873376623376621e-04*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45+1.8628246753246754e-03*Jinv01*w24+4.2004870129870131e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-6.0267857142857139e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53-8.2183441558441553e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+4.9310064935064934e-03*Jinv22*w44+2.0819805194805196e-03*w4*Jinv10+2.0819805194805196e-03*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-3.2873376623376621e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+1.0409902597402598e-03*w32*Jinv21-6.0267857142857139e-04*Jinv21*w20-1.8628246753246754e-03*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12-1.3149350649350649e-03*Jinv20*w56+1.9724025974025973e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20-7.6704545454545450e-04*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35+1.0409902597402598e-03*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+9.3141233766233771e-04*w46*Jinv11+9.5332792207792200e-03*Jinv20*w14+2.0819805194805196e-03*w5*Jinv21-1.1505681818181819e-03*Jinv02*w54-2.1915584415584414e-03*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-2.5750811688311690e-03*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00-3.2873376623376621e-03*Jinv02*w12-6.0267857142857139e-04*Jinv01*w18-3.2873376623376621e-04*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01-1.8628246753246754e-03*w7*Jinv10+9.3141233766233771e-04*w47*Jinv21+2.1367694805194807e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12-8.2183441558441553e-04*Jinv20*w53+7.6704545454545450e-04*Jinv22*w8-2.7942370129870131e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+7.6704545454545450e-04*Jinv12*w7-8.2183441558441553e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-3.2873376623376621e-04*w36*Jinv01-8.2183441558441553e-04*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00-1.2053571428571428e-03*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.1915584415584414e-03*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-1.2601461038961039e-03*w49*Jinv11+9.8620129870129864e-04*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+4.2004870129870131e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18+9.8620129870129864e-04*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8+9.6793831168831166e-04*w57*Jinv00+9.6793831168831166e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20-1.2053571428571428e-03*Jinv01*w54-1.2053571428571428e-03*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2+6.9034090909090911e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.6615259740259744e-04*Jinv01*w9+6.9034090909090911e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-2.1915584415584417e-04*Jinv22*w29-3.2873376623376621e-03*Jinv22*w14+2.9586038961038961e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+4.9310064935064934e-03*Jinv12*w43+2.0819805194805196e-03*w5*Jinv20+1.0409902597402598e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-1.1505681818181819e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+1.8628246753246754e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-1.2601461038961039e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-1.2601461038961039e-03*w48*Jinv01+9.8620129870129864e-04*Jinv12*w1+5.6615259740259744e-04*w10*Jinv11)*D;
  A[13][17]=(0.0)*D;
  A[13][18]=(0.0)*D;
  A[13][19]=(-2.1367694805194807e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10-5.4788961038961042e-05*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+1.4793019480519481e-03*Jinv20*w23+2.4655032467532466e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-2.1915584415584417e-04*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+1.8262987012987013e-05*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-5.4788961038961042e-05*w43*Jinv11+1.3971185064935066e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+1.4793019480519481e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-1.0683847402597404e-03*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+5.2049512987012989e-04*Jinv01*w24-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53+4.6570616883116886e-04*w51*Jinv00+3.8352272727272725e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20-1.0683847402597404e-03*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+2.4655032467532466e-04*Jinv20*w56+8.2183441558441553e-05*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20-3.8352272727272725e-04*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+5.4788961038961036e-04*w49*Jinv10-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00-7.1225649350649346e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+2.7394480519480518e-04*w52*Jinv11+4.6570616883116886e-04*Jinv20*w53+1.4793019480519481e-03*w22*Jinv10+5.2049512987012989e-04*w22*Jinv11+5.2049512987012989e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-3.5612824675324673e-04*w36*Jinv01+4.6570616883116886e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.8262987012987013e-05*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+5.2049512987012989e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10+1.8262987012987013e-05*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.0683847402597404e-03*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+5.2049512987012989e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[13][20]=(0.0)*D;
  A[13][21]=(0.0)*D;
  A[13][22]=(-1.1505681818181819e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3+2.6298701298701297e-03*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+4.7483766233766234e-04*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+2.7394480519480518e-04*w11*Jinv21+1.4793019480519481e-03*w16*Jinv11+1.4793019480519481e-03*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12-5.2049512987012989e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01+4.7483766233766234e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00+1.4793019480519481e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+2.6298701298701297e-03*w13*Jinv11+1.4793019480519481e-03*w15*Jinv01+4.0178571428571428e-04*w28*Jinv11-5.2049512987012989e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.6298701298701297e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+2.6298701298701297e-03*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+1.4793019480519481e-03*Jinv01*w39+4.7483766233766234e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480521e-05*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+1.1505681818181819e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-9.5880681818181818e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-9.5880681818181818e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12+2.6298701298701297e-03*w21*Jinv02-5.2049512987012989e-04*w49*Jinv10+4.0178571428571428e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12-9.5880681818181818e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10+1.1505681818181819e-03*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+2.6298701298701297e-03*Jinv22*w23-2.7394480519480521e-05*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-5.2049512987012989e-04*Jinv22*w8+2.6298701298701297e-03*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11-5.2049512987012989e-04*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49-2.7394480519480521e-05*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10-9.5880681818181818e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-5.2049512987012989e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+2.6298701298701297e-03*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18+2.6298701298701297e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.7483766233766234e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-9.5880681818181818e-04*Jinv01*w54-9.5880681818181818e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+4.7483766233766234e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+1.4793019480519481e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.1505681818181819e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+4.0178571428571428e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01+4.7483766233766234e-04*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[13][23]=(0.0)*D;
  A[13][24]=(0.0)*D;
  A[13][25]=(1.1505681818181819e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+5.2049512987012989e-04*Jinv20*w8+1.1505681818181819e-03*w13*Jinv10-2.5202922077922079e-03*Jinv21*w44-1.6436688311688311e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-3.1960227272727273e-04*Jinv21*w59-4.8396915584415583e-04*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10-2.5202922077922079e-03*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-2.7394480519480520e-03*w16*Jinv11-2.7394480519480520e-03*Jinv21*w17-1.0135957792207792e-03*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-3.1960227272727273e-04*w58*Jinv11-3.1960227272727273e-04*w57*Jinv01-2.0271915584415584e-03*Jinv21*w26-4.8396915584415583e-04*w0*Jinv01-4.7483766233766234e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-2.5202922077922079e-03*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-4.8396915584415583e-04*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11-2.7394480519480520e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33-1.6436688311688311e-04*w31*Jinv10-2.6298701298701297e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.3971185064935066e-03*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38-2.0271915584415584e-03*Jinv01*w24-7.1225649350649346e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+4.6570616883116886e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00-1.6436688311688311e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+1.3971185064935066e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+1.9176136363636362e-04*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+5.2049512987012989e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.3833198051948050e-03*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-7.1225649350649346e-04*Jinv21*w35+1.9176136363636362e-04*w31*Jinv11-1.0135957792207792e-03*w46*Jinv11+1.1505681818181819e-03*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+4.3831168831168834e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.3697240259740259e-04*w6*Jinv01+5.2049512987012989e-04*w7*Jinv10-1.0135957792207792e-03*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38+4.6570616883116886e-04*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-2.6298701298701297e-03*w22*Jinv10-1.2327516233766234e-03*w22*Jinv11-1.2327516233766234e-03*w21*Jinv01+4.6570616883116886e-04*Jinv01*w51+2.4655032467532466e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+1.3697240259740259e-04*w7*Jinv11-4.2918019480519480e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+8.4922889610389610e-04*Jinv21*w56-2.3833198051948050e-03*w24*Jinv00+2.4655032467532466e-04*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.2918019480519480e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+1.4793019480519481e-03*Jinv01*w12+1.3697240259740259e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-4.7483766233766234e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+8.4922889610389610e-04*Jinv01*w54+8.4922889610389610e-04*w55*Jinv11-2.3833198051948050e-03*w25*Jinv10-4.7483766233766234e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00-1.7349837662337662e-04*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11-1.6436688311688311e-04*w5*Jinv20+1.9176136363636362e-04*Jinv01*w30-2.0271915584415584e-03*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+2.4655032467532466e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-1.7349837662337662e-04*w10*Jinv11)*D;
  A[13][26]=(0.0)*D;
  A[13][27]=(0.0)*D;
  A[13][28]=(2.9220779220779221e-04*Jinv21*w44+3.6525974025974026e-05*Jinv01*w3-1.1668019480519480e-04*Jinv21*w59+8.3198051948051951e-05*Jinv21*w2+2.9220779220779221e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17-6.7573051948051951e-04*Jinv01*w45-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-6.7573051948051951e-04*Jinv21*w26+8.3198051948051951e-05*w0*Jinv01+2.9220779220779221e-04*w43*Jinv11-3.2873376623376621e-04*Jinv21*w41+8.3198051948051951e-05*Jinv11*w1-2.5568181818181820e-04*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11-3.2873376623376621e-04*Jinv01*w39-2.7394480519480521e-05*Jinv21*w38-6.7573051948051951e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53+3.6525974025974026e-05*w4*Jinv11+3.6525974025974026e-05*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35+3.6525974025974026e-05*w31*Jinv11-6.7573051948051951e-04*w46*Jinv11+3.6525974025974026e-05*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+4.9310064935064932e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11+1.1870941558441559e-04*w6*Jinv01-6.7573051948051951e-04*w47*Jinv21-2.7394480519480521e-05*w52*Jinv11-1.0227272727272728e-03*w22*Jinv11-1.0227272727272728e-03*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51-2.7394480519480521e-05*w36*Jinv01+1.1870941558441559e-04*w7*Jinv11+4.9310064935064932e-04*Jinv21*w56+1.1870941558441559e-04*w49*Jinv11-2.5568181818181820e-04*Jinv21*w14-2.5568181818181820e-04*Jinv01*w12+1.1870941558441559e-04*Jinv21*w8-1.0227272727272728e-03*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-1.1668019480519480e-04*Jinv01*w9-3.2873376623376621e-04*w40*Jinv11+3.6525974025974026e-05*Jinv01*w30-6.7573051948051951e-04*w25*Jinv11+1.1870941558441559e-04*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27+1.1870941558441559e-04*w48*Jinv01-1.1668019480519480e-04*w10*Jinv11)*D;
  A[13][29]=(0.0)*D;
  A[13][30]=(0.0)*D;
  A[13][31]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+1.0135957792207792e-03*Jinv20*w8+3.5612824675324673e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44-1.2327516233766234e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20-1.2327516233766234e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+1.6436688311688311e-04*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+1.1414366883116883e-03*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10-1.0957792207792207e-03*w42*Jinv01-3.7439123376623376e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10+2.6846590909090908e-03*w15*Jinv02+3.7439123376623376e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00-8.2183441558441553e-05*Jinv02*w24+5.4788961038961036e-04*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-3.1777597402597403e-03*Jinv21*w41-3.9265422077922079e-04*Jinv00*w27+3.7439123376623376e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.9265422077922079e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+2.6846590909090908e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33-1.3971185064935066e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10+2.4655032467532467e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-3.1777597402597403e-03*Jinv01*w39+1.1414366883116883e-03*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38+8.2183441558441553e-05*Jinv01*w24+3.5612824675324673e-04*Jinv22*w59-1.5888798701298701e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00+5.4788961038961036e-04*Jinv12*w34-1.2327516233766234e-03*w4*Jinv10-1.2327516233766234e-03*w4*Jinv11-3.1777597402597403e-03*w41*Jinv20+4.9310064935064934e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+1.0135957792207792e-03*w6*Jinv00+1.6436688311688311e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-3.7439123376623376e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-1.5888798701298701e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14-1.2327516233766234e-03*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+2.4655032467532467e-03*w4*Jinv12-1.3971185064935066e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10-3.9265422077922079e-04*Jinv21*w29-1.5888798701298701e-03*Jinv01*w33-3.1777597402597403e-03*w39*Jinv00+4.9310064935064934e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11+3.7439123376623376e-04*Jinv02*w9+1.0135957792207792e-03*w6*Jinv01+1.0135957792207792e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38-1.3971185064935066e-03*Jinv22*w23+8.2183441558441553e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10-1.1505681818181819e-03*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.1505681818181819e-03*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10+1.0135957792207792e-03*w7*Jinv11-3.7439123376623376e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00+3.9265422077922079e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+2.4655032467532467e-03*Jinv22*w5+1.2327516233766234e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14-3.7439123376623376e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44+3.5612824675324673e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.0135957792207792e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59+5.4788961038961036e-04*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.1505681818181819e-03*w48*Jinv02-3.7439123376623376e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-3.1777597402597403e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14-3.1777597402597403e-03*w40*Jinv11-1.2327516233766234e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+2.6846590909090908e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.9265422077922079e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+1.1414366883116883e-03*Jinv12*w1-3.7439123376623376e-04*w10*Jinv11)*D;
  A[13][32]=(0.0)*D;
  A[13][33]=(0.0)*D;
  A[13][34]=(3.1777597402597403e-03*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12-9.3141233766233771e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+3.2873376623376621e-03*Jinv21*w44+1.0409902597402598e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20-1.0957792207792207e-03*Jinv01*w3-1.7532467532467534e-03*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+3.2873376623376621e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.2962662337662338e-04*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12-1.7532467532467534e-03*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+3.2873376623376621e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.7532467532467534e-03*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33-1.7532467532467534e-03*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3-1.7532467532467534e-03*w21*Jinv00+1.0409902597402598e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44+1.0409902597402598e-03*w4*Jinv10-1.0957792207792207e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10-1.0957792207792207e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20-9.3141233766233771e-04*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+1.3149350649350649e-03*Jinv21*w35-1.0957792207792207e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11+3.1777597402597403e-03*Jinv20*w14-1.0957792207792207e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-1.7532467532467534e-03*w21*Jinv02-1.7532467532467534e-03*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01-9.3141233766233771e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-1.7532467532467534e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11+1.0409902597402598e-03*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53-1.7532467532467534e-03*Jinv22*w8-1.7532467532467534e-03*w22*Jinv10-9.3141233766233771e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-1.7532467532467534e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51+1.0409902597402598e-03*Jinv02*w30-9.3141233766233771e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00+5.4788961038961042e-05*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-1.7532467532467534e-03*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00-9.3141233766233771e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10+5.2962662337662338e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+1.0409902597402598e-03*w5*Jinv20-1.0957792207792207e-03*Jinv01*w30+5.2962662337662338e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[13][35]=(0.0)*D;
  A[13][36]=(0.0)*D;
  A[13][37]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8-1.0957792207792208e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+1.9176136363636362e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+5.2049512987012989e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-8.7662337662337668e-04*w43*Jinv10+8.2183441558441553e-05*Jinv12*w55-1.3697240259740259e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.2049512987012989e-04*w48*Jinv00+5.2049512987012989e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+5.4788961038961036e-04*w45*Jinv00-5.4788961038961042e-05*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+5.2049512987012989e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45-1.0957792207792208e-04*Jinv22*w59+2.4655032467532466e-04*w51*Jinv00-3.8352272727272725e-04*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+1.9176136363636362e-04*w4*Jinv10-1.0957792207792208e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-8.2183441558441553e-05*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12+5.2049512987012989e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+8.2183441558441553e-05*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02+5.2049512987012989e-04*w49*Jinv10-1.0957792207792208e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-8.2183441558441553e-05*w7*Jinv10-3.5612824675324673e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23-3.5612824675324673e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-3.5612824675324673e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+2.4655032467532466e-04*w52*Jinv10+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.4655032467532466e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+6.3920454545454550e-05*w10*Jinv10-8.7662337662337668e-04*Jinv20*w44-1.0957792207792208e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-3.8352272727272725e-04*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.4655032467532466e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-3.5612824675324673e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+1.9176136363636362e-04*w5*Jinv20+2.7394480519480518e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+5.4788961038961036e-04*w55*Jinv10-8.7662337662337668e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[13][38]=(0.0)*D;
  A[13][39]=(0.0)*D;
  A[13][40]=(-3.2873376623376621e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+6.9034090909090911e-03*w40*Jinv12+5.4788961038961042e-05*Jinv20*w8+6.2094155844155847e-04*w57*Jinv02-3.2873376623376621e-03*w13*Jinv10+6.9034090909090911e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-2.1915584415584414e-03*w32*Jinv20+1.0409902597402598e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-8.2183441558441553e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+5.6615259740259744e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02+4.3831168831168834e-04*Jinv21*w2+4.9310064935064934e-03*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+6.9399350649350649e-04*w11*Jinv21+2.9586038961038961e-03*w16*Jinv11+2.9586038961038961e-03*Jinv21*w17+1.8628246753246754e-03*Jinv01*w45+7.3051948051948052e-05*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02+9.6793831168831166e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+5.6615259740259744e-04*w58*Jinv11+5.6615259740259744e-04*w57*Jinv01+9.3141233766233771e-04*Jinv21*w26+4.3831168831168834e-04*w0*Jinv01+9.8620129870129864e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10-1.3149350649350649e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+9.6793831168831166e-04*w11*Jinv22+4.3831168831168834e-04*Jinv11*w1+9.8620129870129864e-04*w13*Jinv11+2.9586038961038961e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-2.7942370129870131e-03*w22*Jinv12-2.1915584415584414e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00+2.0819805194805196e-03*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2-1.1505681818181819e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+9.3141233766233771e-04*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-1.2053571428571428e-03*w19*Jinv11-3.2873376623376621e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+1.0409902597402598e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+9.5332792207792200e-03*Jinv12*w13-1.1505681818181819e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+2.0819805194805196e-03*w32*Jinv21-1.2053571428571428e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-8.2183441558441553e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+6.9034090909090911e-03*Jinv22*w41+1.3149350649350649e-03*w26*Jinv20+4.2004870129870131e-04*w11*Jinv20+2.3011363636363637e-03*Jinv21*w35+2.0819805194805196e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26+1.8628246753246754e-03*w46*Jinv11-3.2873376623376621e-03*Jinv20*w14+1.0409902597402598e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-2.7942370129870131e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+9.5332792207792200e-03*Jinv02*w12-1.2053571428571428e-03*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11+9.6793831168831166e-04*Jinv02*w9-1.2601461038961039e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+1.8628246753246754e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38-2.7942370129870131e-03*Jinv22*w23-3.2873376623376621e-04*w52*Jinv11+2.0819805194805196e-03*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53-2.5750811688311690e-03*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-1.8628246753246754e-03*Jinv22*w50-2.5750811688311690e-03*Jinv12*w7-3.2873376623376621e-04*Jinv01*w51+2.0819805194805196e-03*Jinv02*w30-1.8628246753246754e-03*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.2601461038961039e-03*w7*Jinv11+4.2004870129870131e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-6.0267857142857139e-04*Jinv21*w56+1.3149350649350649e-03*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27-2.5750811688311690e-03*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+9.8620129870129864e-04*Jinv21*w14+4.2004870129870131e-04*w10*Jinv10+4.9310064935064934e-03*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18+9.8620129870129864e-04*Jinv01*w12-1.2601461038961039e-03*Jinv21*w8+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+9.8620129870129864e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-6.0267857142857139e-04*Jinv01*w54-6.0267857142857139e-04*w55*Jinv11+1.3149350649350649e-03*w25*Jinv10+9.8620129870129864e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00-1.8628246753246754e-03*w48*Jinv02+6.9399350649350649e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-2.1915584415584414e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-1.1505681818181819e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29+9.5332792207792200e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11-8.2183441558441553e-04*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+2.0819805194805196e-03*Jinv01*w30+4.3831168831168834e-04*Jinv12*w28+9.3141233766233771e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+4.9310064935064934e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1+6.9399350649350649e-04*w10*Jinv11)*D;
  A[13][41]=(0.0)*D;
  A[13][42]=(0.0)*D;
  A[13][43]=(3.2873376623376621e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-4.2004870129870131e-04*w57*Jinv02+3.2873376623376621e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39-3.2873376623376621e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+2.1915584415584414e-03*w32*Jinv20+1.0957792207792207e-03*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-9.8620129870129864e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55-7.3051948051948052e-05*w58*Jinv10-3.9448051948051946e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-9.8620129870129864e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-3.2873376623376621e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-7.3051948051948052e-05*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10-3.9448051948051946e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+2.1915584415584414e-03*w31*Jinv10+2.1915584415584414e-03*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-9.8620129870129864e-04*Jinv22*w2+1.1505681818181819e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-4.2004870129870131e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+1.0957792207792207e-03*w4*Jinv11-3.9448051948051946e-03*w41*Jinv20+3.2873376623376621e-03*Jinv12*w13+1.1505681818181819e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+1.0957792207792207e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-1.3149350649350649e-03*w26*Jinv20-4.2004870129870131e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35+1.0957792207792207e-03*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-5.4788961038961042e-05*w46*Jinv11+3.2873376623376621e-03*Jinv20*w14+1.0957792207792207e-03*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54+2.1915584415584414e-03*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-3.9448051948051946e-03*w39*Jinv00+3.2873376623376621e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-4.2004870129870131e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.1915584415584414e-03*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-4.2004870129870131e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-4.2004870129870131e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-9.8620129870129864e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10-9.8620129870129864e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+2.1915584415584414e-03*w30*Jinv00-3.9448051948051946e-03*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+3.2873376623376621e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+1.0957792207792207e-03*Jinv01*w30+7.6704545454545450e-04*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20-3.9448051948051946e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-9.8620129870129864e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[13][44]=(0.0)*D;
  A[13][45]=(0.0)*D;
  A[13][46]=(-2.1367694805194807e-03*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-4.2918019480519480e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-2.5202922077922079e-03*Jinv21*w44+1.9176136363636362e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-1.7349837662337662e-04*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-4.7483766233766234e-04*w0*Jinv02-4.8396915584415583e-04*Jinv21*w2-2.5202922077922079e-03*w42*Jinv01-3.1960227272727273e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.0271915584415584e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55-2.1367694805194807e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01-1.0135957792207792e-03*Jinv21*w26-4.8396915584415583e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-2.5202922077922079e-03*w43*Jinv11-2.7394480519480520e-03*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-4.8396915584415583e-04*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-2.1367694805194807e-03*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12-1.6436688311688311e-04*Jinv02*w3-1.6436688311688311e-04*w32*Jinv22-2.7394480519480520e-03*Jinv01*w39-4.7483766233766234e-04*Jinv22*w2+4.6570616883116886e-04*Jinv21*w38-2.3833198051948050e-03*Jinv02*w45-1.0135957792207792e-03*Jinv01*w24-4.2918019480519480e-04*Jinv22*w59-7.1225649350649346e-04*w34*Jinv11+8.4922889610389610e-04*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44+1.9176136363636362e-04*w4*Jinv11+1.1505681818181819e-03*Jinv12*w13+3.8352272727272725e-04*w32*Jinv21+8.4922889610389610e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-7.1225649350649346e-04*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-2.0271915584415584e-03*w46*Jinv11+1.9176136363636362e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33+1.1505681818181819e-03*Jinv02*w12+8.4922889610389610e-04*Jinv01*w18+4.6570616883116886e-04*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01-2.0271915584415584e-03*w47*Jinv21-2.6298701298701297e-03*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11-1.6436688311688311e-04*w31*Jinv12-2.3833198051948050e-03*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8+5.2049512987012989e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51-1.6436688311688311e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+4.6570616883116886e-04*w36*Jinv01+2.4655032467532466e-04*w7*Jinv11+4.3831168831168834e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-2.3833198051948050e-03*w47*Jinv22+1.3697240259740259e-04*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.2918019480519480e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.2327516233766234e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+5.2049512987012989e-04*w48*Jinv02-3.1960227272727273e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.1505681818181819e-03*Jinv22*w14-2.7394480519480520e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43+3.8352272727272725e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.0135957792207792e-03*w25*Jinv11-2.1367694805194807e-03*w16*Jinv12+1.3697240259740259e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.3697240259740259e-04*w48*Jinv01-4.7483766233766234e-04*Jinv12*w1-3.1960227272727273e-04*w10*Jinv11)*D;
  A[13][47]=(0.0)*D;
  A[13][48]=(0.0)*D;
  A[13][49]=(7.1225649350649346e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10-8.7662337662337668e-04*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+7.1225649350649346e-04*w13*Jinv10-8.7662337662337668e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-5.2049512987012989e-04*Jinv20*w23-2.7394480519480521e-05*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12-5.5702110389610384e-04*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+5.4788961038961036e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-2.5750811688311690e-03*w15*Jinv02-3.1960227272727273e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+4.6570616883116886e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-3.1960227272727273e-04*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10-2.5750811688311690e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+9.5880681818181818e-04*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10-2.0271915584415584e-03*Jinv02*w3-5.2049512987012989e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39-5.5702110389610384e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11-8.2183441558441553e-05*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20-4.0543831168831168e-03*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-8.2183441558441553e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480521e-05*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-8.7662337662337668e-04*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11+7.1225649350649346e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21+5.4788961038961036e-04*Jinv02*w54-2.0271915584415584e-03*w4*Jinv12+9.5880681818181818e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00-4.0543831168831168e-03*Jinv02*w12-8.2183441558441553e-05*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11-3.1960227272727273e-04*Jinv02*w9-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+9.5880681818181818e-04*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.1231737012987013e-03*Jinv22*w8-5.2049512987012989e-04*w22*Jinv10+7.3965097402597403e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+1.1231737012987013e-03*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30+7.3965097402597403e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-1.7349837662337662e-04*Jinv02*w27+1.1231737012987013e-03*w6*Jinv02+4.6570616883116886e-04*w19*Jinv12-2.0271915584415584e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+4.6570616883116886e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.3965097402597403e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20-1.7349837662337662e-04*Jinv22*w29-4.0543831168831168e-03*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11-2.7394480519480521e-05*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+5.4788961038961036e-04*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20-2.5750811688311690e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-5.5702110389610384e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[13][50]=(0.0)*D;
  A[13][51]=(0.0)*D;
  A[13][52]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+6.3920454545454550e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39-3.5612824675324673e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+5.4788961038961036e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-5.4788961038961042e-05*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+5.2049512987012989e-04*w45*Jinv00-8.7662337662337668e-04*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27-1.3697240259740259e-04*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+1.9176136363636362e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv02*w45+6.3920454545454550e-05*Jinv22*w59-3.5612824675324673e-04*w51*Jinv00-7.1225649350649346e-04*Jinv12*w34-8.7662337662337668e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10-7.6704545454545450e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10-7.6704545454545450e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12-1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+2.7394480519480518e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23+1.9176136363636362e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+5.2049512987012989e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50+5.2049512987012989e-04*Jinv12*w7+1.9176136363636362e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-3.5612824675324673e-04*w52*Jinv10-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-1.0957792207792208e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+6.3920454545454550e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-7.1225649350649346e-04*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+2.4655032467532466e-04*Jinv02*w36-8.7662337662337668e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[13][53]=(0.0)*D;
  A[13][54]=(0.0)*D;
  A[13][55]=(-2.1367694805194807e-03*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+1.2784090909090910e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3+4.6570616883116886e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+1.3971185064935066e-03*w25*Jinv12+1.8262987012987013e-05*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-1.0683847402597404e-03*Jinv12*w55-1.6436688311688311e-04*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+1.3971185064935066e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+2.4655032467532466e-04*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-2.1915584415584417e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-1.6436688311688311e-04*Jinv22*w17+1.4793019480519481e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3+3.8352272727272725e-04*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39+1.8262987012987013e-05*Jinv22*w2+2.7394480519480518e-04*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-3.8352272727272725e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+4.6570616883116886e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-3.8352272727272725e-04*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+1.3971185064935066e-03*Jinv22*w26+5.2049512987012989e-04*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-1.0683847402597404e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+1.4793019480519481e-03*w21*Jinv02-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01+5.2049512987012989e-04*w47*Jinv21+1.4793019480519481e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50+5.2049512987012989e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7+5.2049512987012989e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-7.1225649350649346e-04*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14+1.2784090909090910e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+5.2049512987012989e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+4.6570616883116886e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-1.0683847402597404e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-1.6436688311688311e-04*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+1.8262987012987013e-05*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[13][56]=(0.0)*D;
  A[13][57]=(0.0)*D;
  A[13][58]=(4.3831168831168834e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39-2.7394480519480521e-05*Jinv22*w38-4.2918019480519480e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-1.0957792207792208e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35-2.1915584415584417e-04*Jinv22*w20+3.4699675324675324e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-5.8441558441558442e-04*w22*Jinv12+7.4878246753246752e-04*Jinv02*w3+3.6525974025974026e-05*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2+1.1870941558441559e-04*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+2.1915584415584417e-04*Jinv12*w34+3.4699675324675324e-04*Jinv22*w44+1.4975649350649350e-03*Jinv12*w13-2.7394480519480521e-05*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-1.0957792207792208e-04*Jinv02*w54+7.4878246753246752e-04*w4*Jinv12-5.8441558441558442e-04*w21*Jinv02+1.4975649350649350e-03*Jinv02*w12+1.0247564935064935e-04*Jinv02*w9-5.8441558441558442e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12-4.2918019480519480e-04*Jinv22*w8-2.6481331168831169e-04*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7+3.6525974025974026e-05*Jinv02*w30-2.6481331168831169e-04*Jinv12*w49-1.4204545454545455e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+7.4878246753246752e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+2.1915584415584417e-04*Jinv02*w33-2.6481331168831169e-04*w48*Jinv02-1.4204545454545455e-05*Jinv22*w29+1.4975649350649350e-03*Jinv22*w14-2.7394480519480521e-05*Jinv02*w36+3.4699675324675324e-04*Jinv12*w43-5.4788961038961042e-05*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[13][59]=(0.0)*D;
  A[14][0]=(0.0)*D;
  A[14][1]=(0.0)*D;
  A[14][2]=(-2.3741883116883116e-03*Jinv00*w12-2.0495129870129870e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+6.7573051948051951e-04*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02-2.3741883116883116e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-1.1870941558441558e-03*w3*Jinv00-1.1870941558441558e-03*w32*Jinv20-1.1870941558441558e-03*Jinv01*w3+1.0227272727272728e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+1.0957792207792208e-04*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59+2.6481331168831169e-04*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+2.6481331168831169e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-1.5726461038961039e-04*w58*Jinv10-6.5746753246753243e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12+6.7573051948051951e-04*w48*Jinv00+2.6481331168831169e-04*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01+2.6481331168831169e-04*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+1.0957792207792208e-04*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+2.6481331168831169e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-2.0495129870129870e-04*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-2.3741883116883116e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-2.0495129870129870e-04*w28*Jinv11+6.7573051948051951e-04*Jinv20*w50-2.0495129870129870e-04*Jinv20*w29+2.6481331168831169e-04*w46*Jinv10-6.5746753246753243e-04*Jinv22*w17-3.2873376623376621e-04*Jinv00*w33+1.0227272727272728e-03*w22*Jinv12-1.1870941558441558e-03*w31*Jinv10-1.1870941558441558e-03*Jinv02*w3+1.0227272727272728e-03*w21*Jinv00-1.1870941558441558e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38+2.6481331168831169e-04*Jinv02*w45+2.6481331168831169e-04*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00-3.2873376623376621e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-1.1870941558441558e-03*w4*Jinv10-1.1870941558441558e-03*w4*Jinv11-6.5746753246753243e-04*w41*Jinv20-2.3741883116883116e-03*Jinv12*w13+1.0957792207792208e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-1.1870941558441558e-03*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+6.7573051948051951e-04*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+1.0957792207792208e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+2.6481331168831169e-04*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-1.1870941558441558e-03*w31*Jinv11+2.6481331168831169e-04*Jinv22*w26+2.6481331168831169e-04*w46*Jinv11-2.3741883116883116e-03*Jinv20*w14-1.1870941558441558e-03*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54-1.1870941558441558e-03*w4*Jinv12+1.0227272727272728e-03*w21*Jinv02+6.7573051948051951e-04*w49*Jinv10-2.0495129870129870e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33-6.5746753246753243e-04*w39*Jinv00-2.3741883116883116e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9+6.7573051948051951e-04*w6*Jinv01+6.7573051948051951e-04*w7*Jinv10+2.6481331168831169e-04*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38+1.0227272727272728e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.1870941558441558e-03*w31*Jinv12+2.6481331168831169e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+6.7573051948051951e-04*Jinv22*w8+1.0227272727272728e-03*w22*Jinv10+6.7573051948051951e-04*Jinv22*w50+1.0227272727272728e-03*w22*Jinv11+6.7573051948051951e-04*Jinv12*w7+1.0227272727272728e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.1870941558441558e-03*Jinv02*w30+6.7573051948051951e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10+6.7573051948051951e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.6481331168831169e-04*w24*Jinv00-2.0495129870129870e-04*Jinv02*w27+6.7573051948051951e-04*w6*Jinv02+1.0957792207792208e-04*w19*Jinv12-1.1870941558441558e-03*Jinv22*w5+2.6481331168831169e-04*w47*Jinv22+6.7573051948051951e-04*w49*Jinv11-2.3741883116883116e-03*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.0957792207792208e-04*Jinv02*w18-2.3741883116883116e-03*Jinv01*w12+6.7573051948051951e-04*Jinv21*w8-1.5726461038961039e-04*w57*Jinv00-1.5726461038961039e-04*Jinv20*w59-3.2873376623376621e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+1.0227272727272728e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.6481331168831169e-04*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+6.7573051948051951e-04*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.1870941558441558e-03*w30*Jinv00-6.5746753246753243e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20-2.0495129870129870e-04*Jinv22*w29-2.3741883116883116e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-1.1870941558441558e-03*w5*Jinv20-1.1870941558441558e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56-2.0495129870129870e-04*Jinv12*w28+2.6481331168831169e-04*w25*Jinv11+1.0957792207792208e-04*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+2.6481331168831169e-04*w47*Jinv20-6.5746753246753243e-04*w16*Jinv12+6.7573051948051951e-04*Jinv21*w50-2.0495129870129870e-04*Jinv01*w27+6.7573051948051951e-04*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[14][3]=(0.0)*D;
  A[14][4]=(0.0)*D;
  A[14][5]=(4.9310064935064934e-03*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-1.1505681818181819e-03*Jinv20*w8-3.7439123376623376e-04*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+2.4655032467532467e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3-1.3971185064935066e-03*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54-3.7439123376623376e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-3.1777597402597403e-03*w16*Jinv11-3.1777597402597403e-03*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55+3.7439123376623376e-04*w58*Jinv10-3.1777597402597403e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35-3.7439123376623376e-04*w58*Jinv11-3.7439123376623376e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+1.1414366883116883e-03*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02+3.9265422077922079e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-3.1777597402597403e-03*w15*Jinv01-3.9265422077922079e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10-3.1777597402597403e-03*Jinv22*w17+5.4788961038961036e-04*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12+2.4655032467532467e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3-1.3971185064935066e-03*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+8.2183441558441553e-05*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24-3.7439123376623376e-04*Jinv22*w59-1.5888798701298701e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11+2.6846590909090908e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+5.4788961038961036e-04*w34*Jinv10-1.2327516233766234e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20-1.1505681818181819e-03*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12-1.5888798701298701e-03*Jinv21*w35-1.2327516233766234e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10-3.9265422077922079e-04*Jinv21*w29-1.5888798701298701e-03*Jinv01*w33+2.6846590909090908e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01-1.1505681818181819e-03*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+1.2327516233766234e-03*Jinv22*w8-1.3971185064935066e-03*w22*Jinv10+1.0135957792207792e-03*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+1.0135957792207792e-03*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+1.2327516233766234e-03*w7*Jinv11+3.5612824675324673e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-3.9265422077922079e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22+1.0135957792207792e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14+3.5612824675324673e-04*w10*Jinv10-3.7439123376623376e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8+3.7439123376623376e-04*w57*Jinv00+3.7439123376623376e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+1.1414366883116883e-03*w0*Jinv00+5.4788961038961036e-04*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+1.1414366883116883e-03*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+1.0135957792207792e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9+1.0957792207792207e-03*Jinv20*w17+2.4655032467532467e-03*w30*Jinv00+2.6846590909090908e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.2327516233766234e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10-8.2183441558441553e-05*w47*Jinv20-3.1777597402597403e-03*w16*Jinv12+1.0135957792207792e-03*Jinv21*w50-3.9265422077922079e-04*Jinv01*w27+1.0135957792207792e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[14][6]=(0.0)*D;
  A[14][7]=(0.0)*D;
  A[14][8]=(-4.0543831168831168e-03*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+7.3965097402597403e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-4.0543831168831168e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.0271915584415584e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+9.5880681818181818e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+4.6570616883116886e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59-5.2049512987012989e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45-8.2183441558441553e-05*Jinv12*w55-3.1960227272727273e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+1.1231737012987013e-03*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-5.5702110389610384e-04*w1*Jinv10-8.7662337662337668e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-1.7349837662337662e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+1.1231737012987013e-03*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33-5.2049512987012989e-04*w22*Jinv12-2.0271915584415584e-03*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+9.5880681818181818e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2+5.4788961038961036e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-2.7394480519480521e-05*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-2.5750811688311690e-03*w41*Jinv20+7.1225649350649346e-04*Jinv12*w13+5.4788961038961036e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+7.3965097402597403e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+4.6570616883116886e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11-4.0543831168831168e-03*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-8.2183441558441553e-05*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-5.2049512987012989e-04*w21*Jinv02+1.1231737012987013e-03*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33-2.5750811688311690e-03*w39*Jinv00+7.1225649350649346e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+7.3965097402597403e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38-5.2049512987012989e-04*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12-2.7394480519480521e-05*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+9.5880681818181818e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-2.7394480519480521e-05*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10-8.2183441558441553e-05*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-3.1960227272727273e-04*w57*Jinv00-3.1960227272727273e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33-5.5702110389610384e-04*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23-8.2183441558441553e-05*Jinv01*w54-8.2183441558441553e-05*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10-5.5702110389610384e-04*Jinv20*w2-8.7662337662337668e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-8.7662337662337668e-04*Jinv20*w17-2.0271915584415584e-03*w30*Jinv00-2.5750811688311690e-03*w40*Jinv10+5.4788961038961036e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+7.1225649350649346e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+4.6570616883116886e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[14][9]=(0.0)*D;
  A[14][10]=(0.0)*D;
  A[14][11]=(1.4975649350649350e-03*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-2.6481331168831169e-04*Jinv20*w8+1.4975649350649350e-03*w13*Jinv10+3.6525974025974026e-05*w3*Jinv00+7.4878246753246752e-04*w32*Jinv20-5.8441558441558442e-04*Jinv20*w23-2.1915584415584417e-04*Jinv00*w54+3.4699675324675324e-04*w43*Jinv10+1.0247564935064935e-04*w58*Jinv10-4.2918019480519480e-04*w48*Jinv00+6.4935064935064935e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10-4.2918019480519480e-04*w45*Jinv00-1.4204545454545455e-05*Jinv00*w27-4.2918019480519480e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+2.1915584415584417e-04*Jinv00*w33+7.4878246753246752e-04*w31*Jinv10-5.8441558441558442e-04*w21*Jinv00-1.0957792207792208e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.7394480519480521e-05*w51*Jinv00+3.6525974025974026e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-1.0957792207792208e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10-2.6481331168831169e-04*w6*Jinv00-2.1915584415584417e-04*Jinv20*w56+1.1870941558441559e-04*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20+1.4975649350649350e-03*Jinv20*w14-4.2918019480519480e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-2.6481331168831169e-04*w7*Jinv10-5.4788961038961042e-05*Jinv20*w38-2.7394480519480521e-05*Jinv20*w53-5.8441558441558442e-04*w22*Jinv10-2.7394480519480521e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+3.4699675324675324e-04*Jinv20*w44+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+4.3831168831168834e-04*Jinv20*w17+7.4878246753246752e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20+3.6525974025974026e-05*w5*Jinv20-2.1915584415584417e-04*w55*Jinv10+3.4699675324675324e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20)*D;
  A[14][12]=(0.0)*D;
  A[14][13]=(0.0)*D;
  A[14][14]=(-7.3965097402597406e-03*Jinv00*w12-6.5746753246753243e-04*w28*Jinv10-4.7666396103896100e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+2.0271915584415584e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02-7.3965097402597406e-03*w13*Jinv10-4.7666396103896100e-03*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+1.0683847402597404e-03*Jinv00*w54-5.7528409090909093e-04*w25*Jinv12+9.1314935064935068e-05*w0*Jinv02+1.1870941558441558e-03*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-7.3965097402597403e-04*Jinv01*w45+1.0683847402597404e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.7666396103896100e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+2.0271915584415584e-03*w48*Jinv00-5.7528409090909093e-04*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35-7.3965097402597403e-04*Jinv21*w26+1.1870941558441558e-03*w0*Jinv01+9.1314935064935068e-05*w1*Jinv10-4.7666396103896100e-03*w16*Jinv10+1.0683847402597404e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-5.7528409090909093e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-6.5746753246753243e-04*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22+1.1870941558441558e-03*Jinv11*w1-4.9310064935064932e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11+2.0271915584415584e-03*Jinv20*w50-6.5746753246753243e-04*Jinv20*w29-5.7528409090909093e-04*w46*Jinv10-4.7666396103896100e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.3149350649350649e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39+9.1314935064935068e-05*Jinv22*w2+1.0683847402597404e-03*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-5.7528409090909093e-04*Jinv02*w45-7.3965097402597403e-04*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+3.5612824675324673e-04*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11-4.7666396103896100e-03*w41*Jinv20-7.3965097402597406e-03*Jinv12*w13+1.0683847402597404e-03*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+2.0271915584415584e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+1.0683847402597404e-03*Jinv20*w56-4.7666396103896100e-03*Jinv22*w41-5.7528409090909093e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-5.7528409090909093e-04*Jinv22*w26-7.3965097402597403e-04*w46*Jinv11-7.3965097402597406e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+1.0683847402597404e-03*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02+2.0271915584415584e-03*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-4.7666396103896100e-03*w39*Jinv00-7.3965097402597406e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+2.0271915584415584e-03*w7*Jinv10-7.3965097402597403e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-2.4655032467532467e-03*w31*Jinv12-5.7528409090909093e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+2.0271915584415584e-03*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10+2.0271915584415584e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+2.0271915584415584e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01-2.4655032467532467e-03*Jinv02*w30+2.0271915584415584e-03*Jinv12*w49+3.5612824675324673e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-5.7528409090909093e-04*w24*Jinv00-6.5746753246753243e-04*Jinv02*w27+2.0271915584415584e-03*w6*Jinv02+1.0683847402597404e-03*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5-5.7528409090909093e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11-7.4878246753246752e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+1.0683847402597404e-03*Jinv02*w18-3.5612824675324673e-04*Jinv21*w8-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+9.1314935064935068e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-5.7528409090909093e-04*w25*Jinv10+9.1314935064935068e-05*Jinv20*w2-4.7666396103896100e-03*w15*Jinv00+2.0271915584415584e-03*w48*Jinv02-4.7666396103896100e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-4.7666396103896100e-03*w40*Jinv10+1.0683847402597404e-03*w20*Jinv20-6.5746753246753243e-04*Jinv22*w29-7.3965097402597406e-03*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+1.0683847402597404e-03*Jinv22*w56-6.5746753246753243e-04*Jinv12*w28-7.3965097402597403e-04*w25*Jinv11+1.0683847402597404e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00-5.7528409090909093e-04*w47*Jinv20-4.7666396103896100e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+9.1314935064935068e-05*Jinv12*w1)*D;
  A[14][15]=(0.0)*D;
  A[14][16]=(0.0)*D;
  A[14][17]=(9.5332792207792200e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.9724025974025973e-03*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53-1.8628246753246754e-03*Jinv20*w8+4.2004870129870131e-04*w57*Jinv02+9.5332792207792200e-03*w13*Jinv10+1.9724025974025973e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+2.0819805194805196e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20+2.0819805194805196e-03*Jinv01*w3-2.7942370129870131e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38-1.3149350649350649e-03*Jinv00*w54+6.9399350649350649e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+9.8620129870129864e-04*w0*Jinv02+4.3831168831168834e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+5.6615259740259744e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17+9.3141233766233771e-04*Jinv01*w45-1.1505681818181819e-03*Jinv12*w55+9.6793831168831166e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12-2.5750811688311690e-03*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+6.9399350649350649e-04*w58*Jinv11+6.9399350649350649e-04*w57*Jinv01+1.8628246753246754e-03*Jinv21*w26+4.3831168831168834e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10+6.9034090909090911e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+2.9586038961038961e-03*Jinv21*w41+4.9310064935064934e-03*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+7.3051948051948052e-05*w11*Jinv22+4.3831168831168834e-04*Jinv11*w1+9.8620129870129864e-04*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-2.5750811688311690e-03*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10-2.1915584415584414e-03*Jinv02*w3-2.7942370129870131e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.9586038961038961e-03*Jinv01*w39+9.8620129870129864e-04*Jinv22*w2-3.2873376623376621e-04*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45+1.8628246753246754e-03*Jinv01*w24+4.2004870129870131e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-6.0267857142857139e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53-8.2183441558441553e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+4.9310064935064934e-03*Jinv22*w44+2.0819805194805196e-03*w4*Jinv10+2.0819805194805196e-03*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-3.2873376623376621e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+1.0409902597402598e-03*w32*Jinv21-6.0267857142857139e-04*Jinv21*w20-1.8628246753246754e-03*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12-1.3149350649350649e-03*Jinv20*w56+1.9724025974025973e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20-7.6704545454545450e-04*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35+1.0409902597402598e-03*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+9.3141233766233771e-04*w46*Jinv11+9.5332792207792200e-03*Jinv20*w14+2.0819805194805196e-03*w5*Jinv21-1.1505681818181819e-03*Jinv02*w54-2.1915584415584414e-03*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-2.5750811688311690e-03*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00-3.2873376623376621e-03*Jinv02*w12-6.0267857142857139e-04*Jinv01*w18-3.2873376623376621e-04*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01-1.8628246753246754e-03*w7*Jinv10+9.3141233766233771e-04*w47*Jinv21+2.1367694805194807e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12-8.2183441558441553e-04*Jinv20*w53+7.6704545454545450e-04*Jinv22*w8-2.7942370129870131e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+7.6704545454545450e-04*Jinv12*w7-8.2183441558441553e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-3.2873376623376621e-04*w36*Jinv01-8.2183441558441553e-04*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00-1.2053571428571428e-03*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.1915584415584414e-03*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-1.2601461038961039e-03*w49*Jinv11+9.8620129870129864e-04*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+4.2004870129870131e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18+9.8620129870129864e-04*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8+9.6793831168831166e-04*w57*Jinv00+9.6793831168831166e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20-1.2053571428571428e-03*Jinv01*w54-1.2053571428571428e-03*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2+6.9034090909090911e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.6615259740259744e-04*Jinv01*w9+6.9034090909090911e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-2.1915584415584417e-04*Jinv22*w29-3.2873376623376621e-03*Jinv22*w14+2.9586038961038961e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+4.9310064935064934e-03*Jinv12*w43+2.0819805194805196e-03*w5*Jinv20+1.0409902597402598e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-1.1505681818181819e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+1.8628246753246754e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-1.2601461038961039e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-1.2601461038961039e-03*w48*Jinv01+9.8620129870129864e-04*Jinv12*w1+5.6615259740259744e-04*w10*Jinv11)*D;
  A[14][18]=(0.0)*D;
  A[14][19]=(0.0)*D;
  A[14][20]=(-2.1367694805194807e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10-5.4788961038961042e-05*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+1.4793019480519481e-03*Jinv20*w23+2.4655032467532466e-04*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45-2.1915584415584417e-04*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+1.8262987012987013e-05*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-5.4788961038961042e-05*w43*Jinv11+1.3971185064935066e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+1.4793019480519481e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-1.0683847402597404e-03*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+5.2049512987012989e-04*Jinv01*w24-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53+4.6570616883116886e-04*w51*Jinv00+3.8352272727272725e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20-1.0683847402597404e-03*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+2.4655032467532466e-04*Jinv20*w56+8.2183441558441553e-05*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20-3.8352272727272725e-04*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+5.4788961038961036e-04*w49*Jinv10-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00-7.1225649350649346e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+2.7394480519480518e-04*w52*Jinv11+4.6570616883116886e-04*Jinv20*w53+1.4793019480519481e-03*w22*Jinv10+5.2049512987012989e-04*w22*Jinv11+5.2049512987012989e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-3.5612824675324673e-04*w36*Jinv01+4.6570616883116886e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.8262987012987013e-05*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+5.2049512987012989e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10+1.8262987012987013e-05*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.0683847402597404e-03*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+5.2049512987012989e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[14][21]=(0.0)*D;
  A[14][22]=(0.0)*D;
  A[14][23]=(-1.1505681818181819e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3+2.6298701298701297e-03*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+4.7483766233766234e-04*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+2.7394480519480518e-04*w11*Jinv21+1.4793019480519481e-03*w16*Jinv11+1.4793019480519481e-03*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12-5.2049512987012989e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01+4.7483766233766234e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00+1.4793019480519481e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+2.6298701298701297e-03*w13*Jinv11+1.4793019480519481e-03*w15*Jinv01+4.0178571428571428e-04*w28*Jinv11-5.2049512987012989e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.6298701298701297e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+2.6298701298701297e-03*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+1.4793019480519481e-03*Jinv01*w39+4.7483766233766234e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480521e-05*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+1.1505681818181819e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-9.5880681818181818e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-9.5880681818181818e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12+2.6298701298701297e-03*w21*Jinv02-5.2049512987012989e-04*w49*Jinv10+4.0178571428571428e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12-9.5880681818181818e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10+1.1505681818181819e-03*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+2.6298701298701297e-03*Jinv22*w23-2.7394480519480521e-05*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-5.2049512987012989e-04*Jinv22*w8+2.6298701298701297e-03*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11-5.2049512987012989e-04*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49-2.7394480519480521e-05*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10-9.5880681818181818e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-5.2049512987012989e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+2.6298701298701297e-03*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18+2.6298701298701297e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.7483766233766234e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-9.5880681818181818e-04*Jinv01*w54-9.5880681818181818e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+4.7483766233766234e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+1.4793019480519481e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.1505681818181819e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+4.0178571428571428e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01+4.7483766233766234e-04*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[14][24]=(0.0)*D;
  A[14][25]=(0.0)*D;
  A[14][26]=(1.1505681818181819e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+5.2049512987012989e-04*Jinv20*w8+1.1505681818181819e-03*w13*Jinv10-2.5202922077922079e-03*Jinv21*w44-1.6436688311688311e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-3.1960227272727273e-04*Jinv21*w59-4.8396915584415583e-04*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10-2.5202922077922079e-03*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-2.7394480519480520e-03*w16*Jinv11-2.7394480519480520e-03*Jinv21*w17-1.0135957792207792e-03*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-3.1960227272727273e-04*w58*Jinv11-3.1960227272727273e-04*w57*Jinv01-2.0271915584415584e-03*Jinv21*w26-4.8396915584415583e-04*w0*Jinv01-4.7483766233766234e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-2.5202922077922079e-03*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-4.8396915584415583e-04*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11-2.7394480519480520e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33-1.6436688311688311e-04*w31*Jinv10-2.6298701298701297e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.3971185064935066e-03*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv21*w38-2.0271915584415584e-03*Jinv01*w24-7.1225649350649346e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+4.6570616883116886e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00-1.6436688311688311e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+1.3971185064935066e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+1.9176136363636362e-04*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+5.2049512987012989e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.3833198051948050e-03*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-7.1225649350649346e-04*Jinv21*w35+1.9176136363636362e-04*w31*Jinv11-1.0135957792207792e-03*w46*Jinv11+1.1505681818181819e-03*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+4.3831168831168834e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11+1.3697240259740259e-04*w6*Jinv01+5.2049512987012989e-04*w7*Jinv10-1.0135957792207792e-03*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38+4.6570616883116886e-04*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-2.6298701298701297e-03*w22*Jinv10-1.2327516233766234e-03*w22*Jinv11-1.2327516233766234e-03*w21*Jinv01+4.6570616883116886e-04*Jinv01*w51+2.4655032467532466e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+1.3697240259740259e-04*w7*Jinv11-4.2918019480519480e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+8.4922889610389610e-04*Jinv21*w56-2.3833198051948050e-03*w24*Jinv00+2.4655032467532466e-04*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.2918019480519480e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+1.4793019480519481e-03*Jinv01*w12+1.3697240259740259e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-4.7483766233766234e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+8.4922889610389610e-04*Jinv01*w54+8.4922889610389610e-04*w55*Jinv11-2.3833198051948050e-03*w25*Jinv10-4.7483766233766234e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00-1.7349837662337662e-04*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11-1.6436688311688311e-04*w5*Jinv20+1.9176136363636362e-04*Jinv01*w30-2.0271915584415584e-03*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+2.4655032467532466e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-1.7349837662337662e-04*w10*Jinv11)*D;
  A[14][27]=(0.0)*D;
  A[14][28]=(0.0)*D;
  A[14][29]=(2.9220779220779221e-04*Jinv21*w44+3.6525974025974026e-05*Jinv01*w3-1.1668019480519480e-04*Jinv21*w59+8.3198051948051951e-05*Jinv21*w2+2.9220779220779221e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17-6.7573051948051951e-04*Jinv01*w45-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-6.7573051948051951e-04*Jinv21*w26+8.3198051948051951e-05*w0*Jinv01+2.9220779220779221e-04*w43*Jinv11-3.2873376623376621e-04*Jinv21*w41+8.3198051948051951e-05*Jinv11*w1-2.5568181818181820e-04*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11-3.2873376623376621e-04*Jinv01*w39-2.7394480519480521e-05*Jinv21*w38-6.7573051948051951e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-2.7394480519480521e-05*Jinv21*w53+3.6525974025974026e-05*w4*Jinv11+3.6525974025974026e-05*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35+3.6525974025974026e-05*w31*Jinv11-6.7573051948051951e-04*w46*Jinv11+3.6525974025974026e-05*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+4.9310064935064932e-04*Jinv01*w18-2.7394480519480521e-05*w37*Jinv11+1.1870941558441559e-04*w6*Jinv01-6.7573051948051951e-04*w47*Jinv21-2.7394480519480521e-05*w52*Jinv11-1.0227272727272728e-03*w22*Jinv11-1.0227272727272728e-03*w21*Jinv01-2.7394480519480521e-05*Jinv01*w51-2.7394480519480521e-05*w36*Jinv01+1.1870941558441559e-04*w7*Jinv11+4.9310064935064932e-04*Jinv21*w56+1.1870941558441559e-04*w49*Jinv11-2.5568181818181820e-04*Jinv21*w14-2.5568181818181820e-04*Jinv01*w12+1.1870941558441559e-04*Jinv21*w8-1.0227272727272728e-03*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-1.1668019480519480e-04*Jinv01*w9-3.2873376623376621e-04*w40*Jinv11+3.6525974025974026e-05*Jinv01*w30-6.7573051948051951e-04*w25*Jinv11+1.1870941558441559e-04*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27+1.1870941558441559e-04*w48*Jinv01-1.1668019480519480e-04*w10*Jinv11)*D;
  A[14][30]=(0.0)*D;
  A[14][31]=(0.0)*D;
  A[14][32]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53+1.0135957792207792e-03*Jinv20*w8+3.5612824675324673e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44-1.2327516233766234e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20-1.2327516233766234e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+1.6436688311688311e-04*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+1.1414366883116883e-03*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10-1.0957792207792207e-03*w42*Jinv01-3.7439123376623376e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10+2.6846590909090908e-03*w15*Jinv02+3.7439123376623376e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00-8.2183441558441553e-05*Jinv02*w24+5.4788961038961036e-04*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-3.1777597402597403e-03*Jinv21*w41-3.9265422077922079e-04*Jinv00*w27+3.7439123376623376e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.9265422077922079e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+2.6846590909090908e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33-1.3971185064935066e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10+2.4655032467532467e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-3.1777597402597403e-03*Jinv01*w39+1.1414366883116883e-03*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38+8.2183441558441553e-05*Jinv01*w24+3.5612824675324673e-04*Jinv22*w59-1.5888798701298701e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00+5.4788961038961036e-04*Jinv12*w34-1.2327516233766234e-03*w4*Jinv10-1.2327516233766234e-03*w4*Jinv11-3.1777597402597403e-03*w41*Jinv20+4.9310064935064934e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+1.0135957792207792e-03*w6*Jinv00+1.6436688311688311e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-3.7439123376623376e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-1.5888798701298701e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14-1.2327516233766234e-03*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+2.4655032467532467e-03*w4*Jinv12-1.3971185064935066e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10-3.9265422077922079e-04*Jinv21*w29-1.5888798701298701e-03*Jinv01*w33-3.1777597402597403e-03*w39*Jinv00+4.9310064935064934e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11+3.7439123376623376e-04*Jinv02*w9+1.0135957792207792e-03*w6*Jinv01+1.0135957792207792e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38-1.3971185064935066e-03*Jinv22*w23+8.2183441558441553e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+8.2183441558441553e-05*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10-1.1505681818181819e-03*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.1505681818181819e-03*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10+1.0135957792207792e-03*w7*Jinv11-3.7439123376623376e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00+3.9265422077922079e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+2.4655032467532467e-03*Jinv22*w5+1.2327516233766234e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14-3.7439123376623376e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44+3.5612824675324673e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-2.4655032467532467e-03*Jinv01*w12+1.0135957792207792e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59+5.4788961038961036e-04*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.1505681818181819e-03*w48*Jinv02-3.7439123376623376e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-3.1777597402597403e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14-3.1777597402597403e-03*w40*Jinv11-1.2327516233766234e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+2.6846590909090908e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.9265422077922079e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+1.1414366883116883e-03*Jinv12*w1-3.7439123376623376e-04*w10*Jinv11)*D;
  A[14][33]=(0.0)*D;
  A[14][34]=(0.0)*D;
  A[14][35]=(3.1777597402597403e-03*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12-9.3141233766233771e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+3.2873376623376621e-03*Jinv21*w44+1.0409902597402598e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20-1.0957792207792207e-03*Jinv01*w3-1.7532467532467534e-03*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+3.2873376623376621e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.2962662337662338e-04*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12-1.7532467532467534e-03*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+3.2873376623376621e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.7532467532467534e-03*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33-1.7532467532467534e-03*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3-1.7532467532467534e-03*w21*Jinv00+1.0409902597402598e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44+1.0409902597402598e-03*w4*Jinv10-1.0957792207792207e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20+3.1777597402597403e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10-1.0957792207792207e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20-9.3141233766233771e-04*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+1.3149350649350649e-03*Jinv21*w35-1.0957792207792207e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11+3.1777597402597403e-03*Jinv20*w14-1.0957792207792207e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-1.7532467532467534e-03*w21*Jinv02-1.7532467532467534e-03*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00+3.1777597402597403e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01-9.3141233766233771e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-1.7532467532467534e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11+1.0409902597402598e-03*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53-1.7532467532467534e-03*Jinv22*w8-1.7532467532467534e-03*w22*Jinv10-9.3141233766233771e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-1.7532467532467534e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51+1.0409902597402598e-03*Jinv02*w30-9.3141233766233771e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00+5.4788961038961042e-05*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-1.7532467532467534e-03*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00-9.3141233766233771e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10+5.2962662337662338e-04*Jinv22*w29+3.1777597402597403e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+1.0409902597402598e-03*w5*Jinv20-1.0957792207792207e-03*Jinv01*w30+5.2962662337662338e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[14][36]=(0.0)*D;
  A[14][37]=(0.0)*D;
  A[14][38]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8-1.0957792207792208e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+1.9176136363636362e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+5.2049512987012989e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-8.7662337662337668e-04*w43*Jinv10+8.2183441558441553e-05*Jinv12*w55-1.3697240259740259e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.2049512987012989e-04*w48*Jinv00+5.2049512987012989e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+5.4788961038961036e-04*w45*Jinv00-5.4788961038961042e-05*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+5.2049512987012989e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45-1.0957792207792208e-04*Jinv22*w59+2.4655032467532466e-04*w51*Jinv00-3.8352272727272725e-04*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+1.9176136363636362e-04*w4*Jinv10-1.0957792207792208e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-8.2183441558441553e-05*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12+5.2049512987012989e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+8.2183441558441553e-05*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02+5.2049512987012989e-04*w49*Jinv10-1.0957792207792208e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-8.2183441558441553e-05*w7*Jinv10-3.5612824675324673e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23-3.5612824675324673e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-3.5612824675324673e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+2.4655032467532466e-04*w52*Jinv10+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.4655032467532466e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22+6.3920454545454550e-05*w10*Jinv10-8.7662337662337668e-04*Jinv20*w44-1.0957792207792208e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-3.8352272727272725e-04*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.4655032467532466e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-3.5612824675324673e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+1.9176136363636362e-04*w5*Jinv20+2.7394480519480518e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28+5.4788961038961036e-04*w55*Jinv10-8.7662337662337668e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[14][39]=(0.0)*D;
  A[14][40]=(0.0)*D;
  A[14][41]=(-3.2873376623376621e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+6.9034090909090911e-03*w40*Jinv12+5.4788961038961042e-05*Jinv20*w8+6.2094155844155847e-04*w57*Jinv02-3.2873376623376621e-03*w13*Jinv10+6.9034090909090911e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-2.1915584415584414e-03*w32*Jinv20+1.0409902597402598e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-8.2183441558441553e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+5.6615259740259744e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02+4.3831168831168834e-04*Jinv21*w2+4.9310064935064934e-03*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+6.9399350649350649e-04*w11*Jinv21+2.9586038961038961e-03*w16*Jinv11+2.9586038961038961e-03*Jinv21*w17+1.8628246753246754e-03*Jinv01*w45+7.3051948051948052e-05*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02+9.6793831168831166e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+5.6615259740259744e-04*w58*Jinv11+5.6615259740259744e-04*w57*Jinv01+9.3141233766233771e-04*Jinv21*w26+4.3831168831168834e-04*w0*Jinv01+9.8620129870129864e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10-1.3149350649350649e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+9.6793831168831166e-04*w11*Jinv22+4.3831168831168834e-04*Jinv11*w1+9.8620129870129864e-04*w13*Jinv11+2.9586038961038961e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-2.7942370129870131e-03*w22*Jinv12-2.1915584415584414e-03*w31*Jinv10+3.1777597402597403e-03*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00+2.0819805194805196e-03*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2-1.1505681818181819e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+9.3141233766233771e-04*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-1.2053571428571428e-03*w19*Jinv11-3.2873376623376621e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+1.0409902597402598e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+9.5332792207792200e-03*Jinv12*w13-1.1505681818181819e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+2.0819805194805196e-03*w32*Jinv21-1.2053571428571428e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-8.2183441558441553e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+6.9034090909090911e-03*Jinv22*w41+1.3149350649350649e-03*w26*Jinv20+4.2004870129870131e-04*w11*Jinv20+2.3011363636363637e-03*Jinv21*w35+2.0819805194805196e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26+1.8628246753246754e-03*w46*Jinv11-3.2873376623376621e-03*Jinv20*w14+1.0409902597402598e-03*w5*Jinv21+3.1777597402597403e-03*w4*Jinv12-2.7942370129870131e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+9.5332792207792200e-03*Jinv02*w12-1.2053571428571428e-03*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11+9.6793831168831166e-04*Jinv02*w9-1.2601461038961039e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+1.8628246753246754e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38-2.7942370129870131e-03*Jinv22*w23-3.2873376623376621e-04*w52*Jinv11+2.0819805194805196e-03*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53-2.5750811688311690e-03*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-1.8628246753246754e-03*Jinv22*w50-2.5750811688311690e-03*Jinv12*w7-3.2873376623376621e-04*Jinv01*w51+2.0819805194805196e-03*Jinv02*w30-1.8628246753246754e-03*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.2601461038961039e-03*w7*Jinv11+4.2004870129870131e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-6.0267857142857139e-04*Jinv21*w56+1.3149350649350649e-03*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27-2.5750811688311690e-03*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12+3.1777597402597403e-03*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+9.8620129870129864e-04*Jinv21*w14+4.2004870129870131e-04*w10*Jinv10+4.9310064935064934e-03*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18+9.8620129870129864e-04*Jinv01*w12-1.2601461038961039e-03*Jinv21*w8+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+9.8620129870129864e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-6.0267857142857139e-04*Jinv01*w54-6.0267857142857139e-04*w55*Jinv11+1.3149350649350649e-03*w25*Jinv10+9.8620129870129864e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00-1.8628246753246754e-03*w48*Jinv02+6.9399350649350649e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-2.1915584415584414e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-1.1505681818181819e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29+9.5332792207792200e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11-8.2183441558441553e-04*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+2.0819805194805196e-03*Jinv01*w30+4.3831168831168834e-04*Jinv12*w28+9.3141233766233771e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+4.9310064935064934e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1+6.9399350649350649e-04*w10*Jinv11)*D;
  A[14][42]=(0.0)*D;
  A[14][43]=(0.0)*D;
  A[14][44]=(3.2873376623376621e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-4.2004870129870131e-04*w57*Jinv02+3.2873376623376621e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39-3.2873376623376621e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+2.1915584415584414e-03*w32*Jinv20+1.0957792207792207e-03*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-9.8620129870129864e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55-7.3051948051948052e-05*w58*Jinv10-3.9448051948051946e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-9.8620129870129864e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-3.2873376623376621e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-7.3051948051948052e-05*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10-3.9448051948051946e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+2.1915584415584414e-03*w31*Jinv10+2.1915584415584414e-03*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-9.8620129870129864e-04*Jinv22*w2+1.1505681818181819e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-4.2004870129870131e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+1.0957792207792207e-03*w4*Jinv11-3.9448051948051946e-03*w41*Jinv20+3.2873376623376621e-03*Jinv12*w13+1.1505681818181819e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+1.0957792207792207e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-1.3149350649350649e-03*w26*Jinv20-4.2004870129870131e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35+1.0957792207792207e-03*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-5.4788961038961042e-05*w46*Jinv11+3.2873376623376621e-03*Jinv20*w14+1.0957792207792207e-03*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54+2.1915584415584414e-03*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-3.9448051948051946e-03*w39*Jinv00+3.2873376623376621e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12-5.4788961038961042e-05*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-4.2004870129870131e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.1915584415584414e-03*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-4.2004870129870131e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-4.2004870129870131e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-9.8620129870129864e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10-9.8620129870129864e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+2.1915584415584414e-03*w30*Jinv00-3.9448051948051946e-03*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+3.2873376623376621e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+1.0957792207792207e-03*Jinv01*w30+7.6704545454545450e-04*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20-3.9448051948051946e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-9.8620129870129864e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[14][45]=(0.0)*D;
  A[14][46]=(0.0)*D;
  A[14][47]=(-2.1367694805194807e-03*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-4.2918019480519480e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-2.5202922077922079e-03*Jinv21*w44+1.9176136363636362e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-1.7349837662337662e-04*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-4.7483766233766234e-04*w0*Jinv02-4.8396915584415583e-04*Jinv21*w2-2.5202922077922079e-03*w42*Jinv01-3.1960227272727273e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.0271915584415584e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55-2.1367694805194807e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01-1.0135957792207792e-03*Jinv21*w26-4.8396915584415583e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-2.5202922077922079e-03*w43*Jinv11-2.7394480519480520e-03*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-4.8396915584415583e-04*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-2.1367694805194807e-03*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12-1.6436688311688311e-04*Jinv02*w3-1.6436688311688311e-04*w32*Jinv22-2.7394480519480520e-03*Jinv01*w39-4.7483766233766234e-04*Jinv22*w2+4.6570616883116886e-04*Jinv21*w38-2.3833198051948050e-03*Jinv02*w45-1.0135957792207792e-03*Jinv01*w24-4.2918019480519480e-04*Jinv22*w59-7.1225649350649346e-04*w34*Jinv11+8.4922889610389610e-04*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44+1.9176136363636362e-04*w4*Jinv11+1.1505681818181819e-03*Jinv12*w13+3.8352272727272725e-04*w32*Jinv21+8.4922889610389610e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-7.1225649350649346e-04*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-2.0271915584415584e-03*w46*Jinv11+1.9176136363636362e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33+1.1505681818181819e-03*Jinv02*w12+8.4922889610389610e-04*Jinv01*w18+4.6570616883116886e-04*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01-2.0271915584415584e-03*w47*Jinv21-2.6298701298701297e-03*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11-1.6436688311688311e-04*w31*Jinv12-2.3833198051948050e-03*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8+5.2049512987012989e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51-1.6436688311688311e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+4.6570616883116886e-04*w36*Jinv01+2.4655032467532466e-04*w7*Jinv11+4.3831168831168834e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-2.3833198051948050e-03*w47*Jinv22+1.3697240259740259e-04*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.2918019480519480e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.2327516233766234e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+5.2049512987012989e-04*w48*Jinv02-3.1960227272727273e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.1505681818181819e-03*Jinv22*w14-2.7394480519480520e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43+3.8352272727272725e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.0135957792207792e-03*w25*Jinv11-2.1367694805194807e-03*w16*Jinv12+1.3697240259740259e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.3697240259740259e-04*w48*Jinv01-4.7483766233766234e-04*Jinv12*w1-3.1960227272727273e-04*w10*Jinv11)*D;
  A[14][48]=(0.0)*D;
  A[14][49]=(0.0)*D;
  A[14][50]=(7.1225649350649346e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10-8.7662337662337668e-04*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+7.1225649350649346e-04*w13*Jinv10-8.7662337662337668e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-5.2049512987012989e-04*Jinv20*w23-2.7394480519480521e-05*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12-5.5702110389610384e-04*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+5.4788961038961036e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-2.5750811688311690e-03*w15*Jinv02-3.1960227272727273e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+4.6570616883116886e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-3.1960227272727273e-04*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10-2.5750811688311690e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+9.5880681818181818e-04*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10-2.0271915584415584e-03*Jinv02*w3-5.2049512987012989e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39-5.5702110389610384e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11-8.2183441558441553e-05*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20-4.0543831168831168e-03*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-8.2183441558441553e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480521e-05*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-8.7662337662337668e-04*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11+7.1225649350649346e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21+5.4788961038961036e-04*Jinv02*w54-2.0271915584415584e-03*w4*Jinv12+9.5880681818181818e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00-4.0543831168831168e-03*Jinv02*w12-8.2183441558441553e-05*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11-3.1960227272727273e-04*Jinv02*w9-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38+9.5880681818181818e-04*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.1231737012987013e-03*Jinv22*w8-5.2049512987012989e-04*w22*Jinv10+7.3965097402597403e-04*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11+1.1231737012987013e-03*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30+7.3965097402597403e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-1.7349837662337662e-04*Jinv02*w27+1.1231737012987013e-03*w6*Jinv02+4.6570616883116886e-04*w19*Jinv12-2.0271915584415584e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+4.6570616883116886e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.3965097402597403e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20-1.7349837662337662e-04*Jinv22*w29-4.0543831168831168e-03*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11-2.7394480519480521e-05*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+5.4788961038961036e-04*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20-2.5750811688311690e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-5.5702110389610384e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[14][51]=(0.0)*D;
  A[14][52]=(0.0)*D;
  A[14][53]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+6.3920454545454550e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39-3.5612824675324673e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+5.4788961038961036e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-5.4788961038961042e-05*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-1.6436688311688311e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+5.2049512987012989e-04*w45*Jinv00-8.7662337662337668e-04*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27-1.3697240259740259e-04*w11*Jinv22-1.6436688311688311e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+1.9176136363636362e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+2.4655032467532466e-04*Jinv02*w45+6.3920454545454550e-05*Jinv22*w59-3.5612824675324673e-04*w51*Jinv00-7.1225649350649346e-04*Jinv12*w34-8.7662337662337668e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10-7.6704545454545450e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-1.6436688311688311e-04*w49*Jinv10-7.6704545454545450e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12-1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+2.7394480519480518e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23+1.9176136363636362e-04*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+5.2049512987012989e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50+5.2049512987012989e-04*Jinv12*w7+1.9176136363636362e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-3.5612824675324673e-04*w52*Jinv10-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22-1.0957792207792208e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+6.3920454545454550e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-7.1225649350649346e-04*Jinv02*w33+9.1314935064935065e-06*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+2.4655032467532466e-04*Jinv02*w36-8.7662337662337668e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20-3.5612824675324673e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[14][54]=(0.0)*D;
  A[14][55]=(0.0)*D;
  A[14][56]=(-2.1367694805194807e-03*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+1.2784090909090910e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-5.4788961038961042e-05*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3+4.6570616883116886e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+1.3971185064935066e-03*w25*Jinv12+1.8262987012987013e-05*w0*Jinv02+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-1.0683847402597404e-03*Jinv12*w55-1.6436688311688311e-04*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+1.3971185064935066e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+2.4655032467532466e-04*Jinv22*w20-5.4788961038961042e-05*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-2.1915584415584417e-04*w11*Jinv22+9.1314935064935065e-06*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-1.6436688311688311e-04*Jinv22*w17+1.4793019480519481e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3+3.8352272727272725e-04*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39+1.8262987012987013e-05*Jinv22*w2+2.7394480519480518e-04*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-3.8352272727272725e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+4.6570616883116886e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-3.8352272727272725e-04*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+1.3971185064935066e-03*Jinv22*w26+5.2049512987012989e-04*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-1.0683847402597404e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+1.4793019480519481e-03*w21*Jinv02-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01+5.2049512987012989e-04*w47*Jinv21+1.4793019480519481e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50+5.2049512987012989e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7+5.2049512987012989e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-7.1225649350649346e-04*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14+1.2784090909090910e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+5.2049512987012989e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+4.6570616883116886e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-1.0683847402597404e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-1.6436688311688311e-04*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+1.8262987012987013e-05*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[14][57]=(0.0)*D;
  A[14][58]=(0.0)*D;
  A[14][59]=(4.3831168831168834e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39-2.7394480519480521e-05*Jinv22*w38-4.2918019480519480e-04*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-1.0957792207792208e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12-4.2918019480519480e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35-2.1915584415584417e-04*Jinv22*w20+3.4699675324675324e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-5.8441558441558442e-04*w22*Jinv12+7.4878246753246752e-04*Jinv02*w3+3.6525974025974026e-05*w32*Jinv22+6.4935064935064935e-05*Jinv22*w2+1.1870941558441559e-04*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+2.1915584415584417e-04*Jinv12*w34+3.4699675324675324e-04*Jinv22*w44+1.4975649350649350e-03*Jinv12*w13-2.7394480519480521e-05*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-1.0957792207792208e-04*Jinv02*w54+7.4878246753246752e-04*w4*Jinv12-5.8441558441558442e-04*w21*Jinv02+1.4975649350649350e-03*Jinv02*w12+1.0247564935064935e-04*Jinv02*w9-5.8441558441558442e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12-4.2918019480519480e-04*Jinv22*w8-2.6481331168831169e-04*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7+3.6525974025974026e-05*Jinv02*w30-2.6481331168831169e-04*Jinv12*w49-1.4204545454545455e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+7.4878246753246752e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+2.1915584415584417e-04*Jinv02*w33-2.6481331168831169e-04*w48*Jinv02-1.4204545454545455e-05*Jinv22*w29+1.4975649350649350e-03*Jinv22*w14-2.7394480519480521e-05*Jinv02*w36+3.4699675324675324e-04*Jinv12*w43-5.4788961038961042e-05*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+6.4935064935064935e-05*Jinv12*w1)*D;
  A[15][0]=(-6.5746753246753243e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12-3.4699675324675324e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-6.5746753246753243e-04*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-2.1915584415584417e-04*Jinv22*w38-3.4699675324675324e-04*Jinv00*w54+1.9480519480519481e-04*Jinv21*w59-4.3831168831168834e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+3.6525974025974025e-03*w16*Jinv11+3.6525974025974025e-03*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45-3.4699675324675324e-04*Jinv12*w55+1.9480519480519481e-04*w58*Jinv10+3.6525974025974025e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12-2.9220779220779221e-04*w48*Jinv00-4.3831168831168834e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35+1.9480519480519481e-04*w58*Jinv11+1.9480519480519481e-04*w57*Jinv01-4.3831168831168834e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+3.6525974025974025e-03*w16*Jinv10-4.3831168831168834e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27+4.2613636363636367e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-6.5746753246753243e-04*w13*Jinv11+3.6525974025974025e-03*w15*Jinv01+4.2613636363636367e-05*w28*Jinv11-2.9220779220779221e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+3.6525974025974025e-03*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-4.3831168831168834e-04*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38-2.1915584415584417e-04*Jinv02*w45-4.3831168831168834e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59+1.8262987012987013e-03*w34*Jinv11-4.3831168831168834e-04*w19*Jinv11-3.4699675324675324e-04*Jinv21*w53-3.4699675324675324e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-6.5746753246753243e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11+1.8262987012987013e-03*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-4.3831168831168834e-04*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10-3.2873376623376621e-04*w32*Jinv21-4.3831168831168834e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00-2.1915584415584417e-04*w37*Jinv12-3.4699675324675324e-04*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-4.3831168831168834e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20-3.4699675324675324e-04*w52*Jinv12+1.8262987012987013e-03*Jinv21*w35-3.2873376623376621e-04*w31*Jinv11-4.3831168831168834e-04*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21-3.4699675324675324e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12+3.2873376623376621e-04*w21*Jinv02-2.9220779220779221e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+1.8262987012987013e-03*Jinv01*w33+1.8262987012987013e-03*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-4.3831168831168834e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-3.4699675324675324e-04*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12-2.1915584415584417e-04*w46*Jinv12-3.4699675324675324e-04*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10-2.9220779220779221e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01-3.4699675324675324e-04*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30-2.9220779220779221e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01-3.4699675324675324e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10-3.4699675324675324e-04*Jinv21*w56-4.3831168831168834e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02-4.3831168831168834e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5-2.1915584415584417e-04*w47*Jinv22-2.9220779220779221e-04*w49*Jinv11-6.5746753246753243e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12-4.3831168831168834e-04*Jinv02*w18-6.5746753246753243e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+1.9480519480519481e-04*w57*Jinv00+1.9480519480519481e-04*Jinv20*w59+1.8262987012987013e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+1.8262987012987013e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-3.4699675324675324e-04*Jinv01*w54-3.4699675324675324e-04*w55*Jinv11-4.3831168831168834e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+3.6525974025974025e-03*w15*Jinv00-2.9220779220779221e-04*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+3.6525974025974025e-03*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+1.8262987012987013e-03*w40*Jinv10-4.3831168831168834e-04*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11-2.1915584415584417e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-6.5746753246753243e-04*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30-3.4699675324675324e-04*Jinv02*w51-3.4699675324675324e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-4.3831168831168834e-04*w25*Jinv11-3.4699675324675324e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20+3.6525974025974025e-03*w16*Jinv12-2.9220779220779221e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-2.9220779220779221e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[15][1]=(0.0)*D;
  A[15][2]=(0.0)*D;
  A[15][3]=(1.0957792207792207e-03*Jinv00*w12-5.6615259740259744e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-9.6793831168831166e-04*w57*Jinv02+1.0957792207792207e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-4.7666396103896100e-03*Jinv01*w3+6.0267857142857139e-04*Jinv20*w23+8.2183441558441553e-04*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-9.6793831168831166e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.3806818181818182e-02*w16*Jinv11-1.3806818181818182e-02*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+8.7662337662337668e-04*Jinv12*w55-6.9399350649350649e-04*w58*Jinv10-1.3806818181818182e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+1.2053571428571428e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-6.9034090909090911e-03*Jinv22*w35-9.6793831168831166e-04*w58*Jinv11-9.6793831168831166e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10+1.6436688311688311e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+3.2873376623376621e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.6615259740259744e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-3.1777597402597403e-03*w13*Jinv11-1.3806818181818182e-02*w15*Jinv01-1.0592532467532468e-03*w28*Jinv11+1.2053571428571428e-03*Jinv20*w50-5.6615259740259744e-04*Jinv20*w29+3.2873376623376621e-04*w46*Jinv10-1.3806818181818182e-02*Jinv22*w17-5.9172077922077923e-03*Jinv00*w33+2.7394480519480520e-03*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10-4.7666396103896100e-03*Jinv02*w3+6.0267857142857139e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+8.7662337662337668e-04*Jinv00*w18+4.3831168831168834e-04*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-9.6793831168831166e-04*Jinv22*w59-6.9034090909090911e-03*w34*Jinv11+1.6436688311688311e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+1.2601461038961039e-03*w51*Jinv00-6.9034090909090911e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-4.7666396103896100e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-3.1777597402597403e-03*Jinv12*w13+8.7662337662337668e-04*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+1.6436688311688311e-03*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00+8.2183441558441553e-04*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12-6.9034090909090911e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+1.0957792207792207e-03*Jinv20*w14-4.7666396103896100e-03*w5*Jinv21+8.7662337662337668e-04*Jinv02*w54-4.7666396103896100e-03*w4*Jinv12+2.7394480519480520e-03*w21*Jinv02+1.2053571428571428e-03*w49*Jinv10-1.0592532467532468e-03*Jinv21*w29-6.9034090909090911e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-3.1777597402597403e-03*Jinv02*w12+1.6436688311688311e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+6.5746753246753243e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+4.3831168831168834e-04*Jinv20*w38+2.7394480519480520e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+1.2601461038961039e-03*Jinv20*w53+6.5746753246753243e-04*Jinv22*w8+6.0267857142857139e-04*w22*Jinv10+2.5202922077922079e-03*Jinv22*w50+2.7394480519480520e-03*w22*Jinv11+6.5746753246753243e-04*Jinv12*w7+2.7394480519480520e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+2.5202922077922079e-03*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+1.2601461038961039e-03*w52*Jinv10+6.5746753246753243e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+4.3831168831168834e-04*w37*Jinv10+8.7662337662337668e-04*Jinv21*w56+1.3697240259740260e-03*w24*Jinv00-1.0592532467532468e-03*Jinv02*w27+6.5746753246753243e-04*w6*Jinv02+1.6436688311688311e-03*w19*Jinv12-4.7666396103896100e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+2.5202922077922079e-03*w49*Jinv11-3.1777597402597403e-03*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-9.6793831168831166e-04*w58*Jinv12+1.6436688311688311e-03*Jinv02*w18-3.1777597402597403e-03*Jinv01*w12+6.5746753246753243e-04*Jinv21*w8-6.9399350649350649e-04*w57*Jinv00-6.9399350649350649e-04*Jinv20*w59-6.9034090909090911e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+2.7394480519480520e-03*Jinv21*w23+8.7662337662337668e-04*Jinv01*w54+8.7662337662337668e-04*w55*Jinv11+1.3697240259740260e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00+2.5202922077922079e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-1.1834415584415585e-02*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10+8.7662337662337668e-04*w20*Jinv20-1.0592532467532468e-03*Jinv22*w29-3.1777597402597403e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+8.2183441558441553e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+8.7662337662337668e-04*Jinv22*w56-1.0592532467532468e-03*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+3.2873376623376621e-04*w47*Jinv20-1.3806818181818182e-02*w16*Jinv12+2.5202922077922079e-03*Jinv21*w50-1.0592532467532468e-03*Jinv01*w27+2.5202922077922079e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[15][4]=(0.0)*D;
  A[15][5]=(0.0)*D;
  A[15][6]=(-8.7662337662337668e-04*Jinv00*w12+5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-8.7662337662337668e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00-4.3831168831168834e-04*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55+6.9399350649350649e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-1.2601461038961039e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-3.2873376623376621e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.6615259740259744e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+7.6704545454545450e-04*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-1.2601461038961039e-03*Jinv20*w50+5.6615259740259744e-04*Jinv20*w29-3.2873376623376621e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12-4.3831168831168834e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-1.0957792207792207e-03*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53-1.2053571428571428e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10+2.1367694805194807e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-1.0957792207792207e-03*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10+3.8352272727272725e-04*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-6.0267857142857139e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11-8.7662337662337668e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02-1.2601461038961039e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-1.2053571428571428e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+7.6704545454545450e-04*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+7.6704545454545450e-04*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-1.2053571428571428e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56-6.0267857142857139e-04*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+7.6704545454545450e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18+7.6704545454545450e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8+6.9399350649350649e-04*w57*Jinv00+6.9399350649350649e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+7.6704545454545450e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-6.0267857142857139e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-4.3831168831168834e-04*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10-1.0957792207792207e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+3.8352272727272725e-04*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11-8.2183441558441553e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-3.2873376623376621e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[15][7]=(0.0)*D;
  A[15][8]=(0.0)*D;
  A[15][9]=(4.3831168831168834e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10+6.5746753246753243e-04*Jinv20*w8+4.3831168831168834e-04*w13*Jinv10-3.2873376623376621e-04*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20+4.3831168831168834e-04*Jinv20*w23+3.4699675324675324e-04*Jinv00*w54-1.8262987012987013e-03*w43*Jinv10-1.9480519480519481e-04*w58*Jinv10+3.4699675324675324e-04*w48*Jinv00-4.2613636363636367e-05*w1*Jinv10-3.6525974025974025e-03*w16*Jinv10+2.1915584415584417e-04*w45*Jinv00-4.2613636363636367e-05*Jinv00*w27+3.4699675324675324e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-1.8262987012987013e-03*Jinv00*w33+2.1915584415584417e-04*w31*Jinv10+4.3831168831168834e-04*w21*Jinv00+6.5746753246753243e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00+2.9220779220779221e-04*w51*Jinv00-3.2873376623376621e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+6.5746753246753243e-04*w6*Jinv00+3.4699675324675324e-04*Jinv20*w56-3.2873376623376621e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+4.3831168831168834e-04*Jinv20*w14+3.4699675324675324e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+6.5746753246753243e-04*w7*Jinv10+3.2873376623376621e-04*Jinv20*w38+2.9220779220779221e-04*Jinv20*w53+4.3831168831168834e-04*w22*Jinv10+2.9220779220779221e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10-3.2873376623376621e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-1.8262987012987013e-03*Jinv20*w44-1.9480519480519481e-04*w57*Jinv00-1.9480519480519481e-04*Jinv20*w59-4.2613636363636367e-05*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20-3.2873376623376621e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-3.6525974025974025e-03*w15*Jinv00-3.6525974025974025e-03*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-3.2873376623376621e-04*w5*Jinv20+3.4699675324675324e-04*w55*Jinv10-1.8262987012987013e-03*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20)*D;
  A[15][10]=(0.0)*D;
  A[15][11]=(0.0)*D;
  A[15][12]=(-4.7666396103896100e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-6.9034090909090911e-03*w40*Jinv12+8.7662337662337668e-04*Jinv22*w53+2.7394480519480520e-03*Jinv20*w8-9.6793831168831166e-04*w57*Jinv02-4.7666396103896100e-03*w13*Jinv10-6.9034090909090911e-03*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-3.1777597402597403e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20+1.0957792207792207e-03*Jinv01*w3+6.5746753246753243e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-6.9399350649350649e-04*Jinv21*w59+1.6436688311688311e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-5.6615259740259744e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+4.3831168831168834e-04*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-9.6793831168831166e-04*w58*Jinv10-1.3806818181818182e-02*w15*Jinv02-1.0592532467532468e-03*w10*Jinv12+2.5202922077922079e-03*w48*Jinv00+1.6436688311688311e-03*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-6.9399350649350649e-04*w58*Jinv11-6.9399350649350649e-04*w57*Jinv01+8.7662337662337668e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-1.3806818181818182e-02*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.0592532467532468e-03*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+2.5202922077922079e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-1.3806818181818182e-02*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+6.5746753246753243e-04*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10-3.1777597402597403e-03*Jinv02*w3+6.5746753246753243e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.2873376623376621e-04*Jinv21*w38+8.2183441558441553e-04*Jinv02*w45+8.7662337662337668e-04*Jinv01*w24-9.6793831168831166e-04*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+8.7662337662337668e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-3.1777597402597403e-03*w4*Jinv10+1.0957792207792207e-03*w4*Jinv11-6.9034090909090911e-03*w41*Jinv20-4.7666396103896100e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+2.7394480519480520e-03*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-6.9034090909090911e-03*Jinv22*w41+1.6436688311688311e-03*w26*Jinv20-1.0592532467532468e-03*w11*Jinv20+8.7662337662337668e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11+1.6436688311688311e-03*Jinv22*w26+4.3831168831168834e-04*w46*Jinv11-4.7666396103896100e-03*Jinv20*w14+1.0957792207792207e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54-3.1777597402597403e-03*w4*Jinv12+6.5746753246753243e-04*w21*Jinv02+2.5202922077922079e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-6.9034090909090911e-03*w39*Jinv00-4.7666396103896100e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.2873376623376621e-04*w37*Jinv11-1.0592532467532468e-03*Jinv02*w9+6.0267857142857139e-04*w6*Jinv01+2.7394480519480520e-03*w7*Jinv10+4.3831168831168834e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+6.5746753246753243e-04*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+8.2183441558441553e-04*w46*Jinv12+8.7662337662337668e-04*Jinv20*w53+2.7394480519480520e-03*Jinv22*w8+6.5746753246753243e-04*w22*Jinv10+2.5202922077922079e-03*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+2.7394480519480520e-03*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+2.5202922077922079e-03*Jinv12*w49+3.2873376623376621e-04*w36*Jinv01+8.7662337662337668e-04*w52*Jinv10+6.0267857142857139e-04*w7*Jinv11-1.0592532467532468e-03*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.2601461038961039e-03*Jinv21*w56+1.6436688311688311e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+2.7394480519480520e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-3.1777597402597403e-03*Jinv22*w5+8.2183441558441553e-04*w47*Jinv22+1.2053571428571428e-03*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14-1.0592532467532468e-03*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-9.6793831168831166e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+6.0267857142857139e-04*Jinv21*w8-9.6793831168831166e-04*w57*Jinv00-9.6793831168831166e-04*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23+1.2601461038961039e-03*Jinv01*w54+1.2601461038961039e-03*w55*Jinv11+1.6436688311688311e-03*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-1.3806818181818182e-02*w15*Jinv00+2.5202922077922079e-03*w48*Jinv02-5.6615259740259744e-04*Jinv01*w9-1.3806818181818182e-02*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-6.9034090909090911e-03*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-4.7666396103896100e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-3.1777597402597403e-03*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+8.7662337662337668e-04*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+8.7662337662337668e-04*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-1.3806818181818182e-02*w16*Jinv12+1.2053571428571428e-03*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+1.2053571428571428e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-5.6615259740259744e-04*w10*Jinv11)*D;
  A[15][13]=(0.0)*D;
  A[15][14]=(0.0)*D;
  A[15][15]=(6.9034090909090911e-03*Jinv00*w12-1.1834415584415585e-02*w40*Jinv12+4.9310064935064934e-03*Jinv22*w53-5.9172077922077923e-03*Jinv20*w8-2.7759740259740259e-03*w57*Jinv02+6.9034090909090911e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-5.9172077922077923e-03*Jinv21*w44+5.9172077922077923e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+6.9034090909090911e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38-2.9586038961038961e-03*w25*Jinv12-2.2646103896103898e-03*w0*Jinv02-1.8262987012987013e-03*Jinv21*w2-5.9172077922077923e-03*w43*Jinv10-5.9172077922077923e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45+4.9310064935064934e-03*Jinv12*w55-3.5503246753246752e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12-1.6436688311688311e-03*w48*Jinv00-2.9586038961038961e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-6.9034090909090911e-03*Jinv21*w26-1.8262987012987013e-03*w0*Jinv01-1.8262987012987013e-03*w1*Jinv10-2.9586038961038961e-03*Jinv22*w20-5.9172077922077923e-03*w43*Jinv11-1.7751623376623376e-02*w42*Jinv02-4.3831168831168834e-04*w11*Jinv22-1.8262987012987013e-03*Jinv11*w1+5.9172077922077923e-03*w13*Jinv11+1.8262987012987013e-03*w28*Jinv11-1.6436688311688311e-03*Jinv20*w50-3.5503246753246752e-02*Jinv22*w17-1.9724025974025973e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+3.9448051948051946e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-2.2646103896103898e-03*Jinv22*w2-6.9034090909090911e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-6.9034090909090911e-03*Jinv01*w24-2.7759740259740259e-03*Jinv22*w59+5.9172077922077923e-03*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11+1.6436688311688311e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-1.7751623376623376e-02*Jinv22*w44+5.9172077922077923e-03*w4*Jinv10+6.9034090909090911e-03*w4*Jinv11+5.9172077922077923e-03*w41*Jinv20+3.9448051948051946e-03*Jinv12*w13-6.9034090909090911e-03*w19*Jinv10+2.3011363636363637e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-5.9172077922077923e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12-1.1834415584415585e-02*Jinv22*w41-9.8620129870129864e-04*w26*Jinv20+1.8262987012987013e-03*w11*Jinv20+4.9310064935064934e-03*w52*Jinv12+5.9172077922077923e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11-2.9586038961038961e-03*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+6.9034090909090911e-03*Jinv20*w14+6.9034090909090911e-03*w5*Jinv21+4.9310064935064934e-03*Jinv02*w54+3.9448051948051946e-03*w4*Jinv12-1.9724025974025973e-03*w21*Jinv02-1.6436688311688311e-03*w49*Jinv10+1.8262987012987013e-03*Jinv21*w29+5.9172077922077923e-03*Jinv01*w33+5.9172077922077923e-03*w39*Jinv00+3.9448051948051946e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-5.9172077922077923e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38-1.9724025974025973e-03*Jinv22*w23+1.3149350649350649e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12+1.6436688311688311e-03*Jinv20*w53-1.9724025974025973e-03*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10+3.2873376623376621e-03*Jinv22*w50-5.9172077922077923e-03*w22*Jinv11-1.9724025974025973e-03*Jinv12*w7-5.9172077922077923e-03*w21*Jinv01+1.3149350649350649e-03*Jinv02*w30+3.2873376623376621e-03*Jinv12*w49+1.6436688311688311e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+1.8262987012987013e-03*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+1.6436688311688311e-03*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27-1.9724025974025973e-03*w6*Jinv02-2.9586038961038961e-03*w19*Jinv12+3.9448051948051946e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22-1.6436688311688311e-03*w49*Jinv11+5.9172077922077923e-03*Jinv21*w14+1.8262987012987013e-03*w10*Jinv10-5.9172077922077923e-03*Jinv20*w44-2.7759740259740259e-03*w58*Jinv12-2.9586038961038961e-03*Jinv02*w18+5.9172077922077923e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8-1.1834415584415585e-02*Jinv02*w33-1.8262987012987013e-03*w0*Jinv00-5.9172077922077923e-03*Jinv21*w23+1.6436688311688311e-03*Jinv01*w54+1.6436688311688311e-03*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-1.8262987012987013e-03*Jinv20*w2+3.2873376623376621e-03*w48*Jinv02+2.3011363636363637e-03*w30*Jinv00+5.9172077922077923e-03*w40*Jinv10-6.9034090909090911e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29+3.9448051948051946e-03*Jinv22*w14-9.8620129870129864e-04*Jinv02*w36-1.7751623376623376e-02*Jinv12*w43+5.9172077922077923e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30+4.9310064935064934e-03*Jinv02*w51+4.9310064935064934e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28-6.9034090909090911e-03*w25*Jinv11-5.9172077922077923e-03*w42*Jinv00-3.5503246753246752e-02*w16*Jinv12-1.6436688311688311e-03*Jinv21*w50+1.8262987012987013e-03*Jinv01*w27-1.6436688311688311e-03*w48*Jinv01-2.2646103896103898e-03*Jinv12*w1)*D;
  A[15][16]=(0.0)*D;
  A[15][17]=(0.0)*D;
  A[15][18]=(-2.1367694805194807e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+3.1777597402597403e-03*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-2.7394480519480520e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-1.6436688311688311e-03*Jinv20*w23-1.3149350649350649e-03*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+4.9310064935064932e-04*Jinv21*w2+6.9034090909090911e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+9.6793831168831166e-04*w58*Jinv10-8.7662337662337668e-04*w48*Jinv00+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+1.0592532467532468e-03*w1*Jinv10+1.3806818181818182e-02*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+4.3831168831168834e-04*Jinv00*w27+4.9310064935064932e-04*Jinv11*w1-7.6704545454545450e-04*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-8.7662337662337668e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-03*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+4.7666396103896100e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-2.1367694805194807e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-2.5202922077922079e-03*w51*Jinv00-2.7394480519480520e-03*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+4.7666396103896100e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-3.8352272727272725e-04*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+3.1777597402597403e-03*w6*Jinv00-1.3149350649350649e-03*Jinv20*w56-6.5746753246753243e-04*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11-3.8352272727272725e-04*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-8.7662337662337668e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+4.2735389610389614e-03*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+4.2735389610389614e-03*w6*Jinv01+3.1777597402597403e-03*w7*Jinv10-3.8352272727272725e-04*w47*Jinv21+1.5888798701298701e-03*Jinv20*w38-1.3149350649350649e-03*w52*Jinv11-2.5202922077922079e-03*Jinv20*w53-1.6436688311688311e-03*w22*Jinv10-7.6704545454545450e-04*w22*Jinv11-7.6704545454545450e-04*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51+2.1367694805194807e-03*w36*Jinv01-2.5202922077922079e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-7.6704545454545450e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-6.5746753246753243e-04*w24*Jinv00-5.4788961038961042e-05*w49*Jinv11-7.6704545454545450e-04*Jinv21*w14-7.6704545454545450e-04*w10*Jinv10+6.9034090909090911e-03*Jinv20*w44-7.6704545454545450e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+9.6793831168831166e-04*w57*Jinv00+9.6793831168831166e-04*Jinv20*w59+1.0592532467532468e-03*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-7.6704545454545450e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-6.5746753246753243e-04*w25*Jinv10+1.0592532467532468e-03*Jinv20*w2+1.3806818181818182e-02*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+1.3806818181818182e-02*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+4.7666396103896100e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-2.7394480519480520e-03*w5*Jinv20-3.8352272727272725e-04*Jinv01*w30-2.1367694805194807e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+6.9034090909090911e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20-5.4788961038961042e-05*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[15][19]=(0.0)*D;
  A[15][20]=(0.0)*D;
  A[15][21]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+7.6704545454545450e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+7.6704545454545450e-04*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20-8.7662337662337668e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54+6.9399350649350649e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+5.6615259740259744e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+6.9399350649350649e-04*w58*Jinv11+6.9399350649350649e-04*w57*Jinv01-1.0957792207792207e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00-3.2873376623376621e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-1.0957792207792207e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-6.0267857142857139e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+7.6704545454545450e-04*w4*Jinv10-8.7662337662337668e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-4.3831168831168834e-04*w32*Jinv21-6.0267857142857139e-04*Jinv21*w20+7.6704545454545450e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35-4.3831168831168834e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-8.7662337662337668e-04*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12-6.0267857142857139e-04*Jinv01*w18-3.2873376623376621e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01+7.6704545454545450e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+7.6704545454545450e-04*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+7.6704545454545450e-04*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-3.2873376623376621e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.2053571428571428e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22-1.2601461038961039e-03*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+4.9310064935064932e-04*Jinv21*w23-1.2053571428571428e-03*Jinv01*w54-1.2053571428571428e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.6615259740259744e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+7.6704545454545450e-04*w5*Jinv20-4.3831168831168834e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-1.0957792207792207e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-1.2601461038961039e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-1.2601461038961039e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.6615259740259744e-04*w10*Jinv11)*D;
  A[15][22]=(0.0)*D;
  A[15][23]=(0.0)*D;
  A[15][24]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-7.6704545454545450e-04*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+6.9034090909090911e-03*Jinv21*w44-7.6704545454545450e-04*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-1.3149350649350649e-03*Jinv00*w54+9.6793831168831166e-04*Jinv21*w59+1.0592532467532468e-03*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+6.9034090909090911e-03*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.3806818181818182e-02*w16*Jinv11+1.3806818181818182e-02*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45+2.7394480519480518e-04*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00+9.6793831168831166e-04*w58*Jinv11+9.6793831168831166e-04*w57*Jinv01+4.7666396103896100e-03*Jinv21*w26+1.0592532467532468e-03*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+6.9034090909090911e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+1.0592532467532468e-03*Jinv11*w1-2.7394480519480520e-03*w13*Jinv11+1.3806818181818182e-02*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-3.8352272727272725e-04*w31*Jinv10+4.2735389610389614e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-2.1367694805194807e-03*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+4.7666396103896100e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11-6.5746753246753243e-04*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-7.6704545454545450e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-6.5746753246753243e-04*Jinv21*w20-7.6704545454545450e-04*w6*Jinv00-1.3149350649350649e-03*Jinv20*w56+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.5888798701298701e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-5.4788961038961042e-05*w49*Jinv10-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-1.6436688311688311e-03*w6*Jinv01-7.6704545454545450e-04*w7*Jinv10+1.5888798701298701e-03*w47*Jinv21-3.8352272727272725e-04*Jinv20*w38-1.3149350649350649e-03*w52*Jinv11-5.4788961038961042e-05*Jinv20*w53+4.2735389610389614e-03*w22*Jinv10+3.1777597402597403e-03*w22*Jinv11+3.1777597402597403e-03*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51-8.2183441558441553e-04*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-1.6436688311688311e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-2.5202922077922079e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-8.7662337662337668e-04*w49*Jinv11-2.7394480519480520e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-2.7394480519480520e-03*Jinv01*w12-1.6436688311688311e-03*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+3.1777597402597403e-03*Jinv21*w23-2.5202922077922079e-03*Jinv01*w54-2.5202922077922079e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00+4.3831168831168834e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-7.6704545454545450e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+4.7666396103896100e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-8.7662337662337668e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-8.7662337662337668e-04*w48*Jinv01+4.3831168831168834e-04*w10*Jinv11)*D;
  A[15][25]=(0.0)*D;
  A[15][26]=(0.0)*D;
  A[15][27]=(-1.8262987012987013e-03*Jinv21*w44+4.3831168831168834e-04*Jinv01*w3-1.9480519480519481e-04*Jinv21*w59-4.2613636363636367e-05*Jinv21*w2-1.8262987012987013e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-3.6525974025974025e-03*w16*Jinv11-3.6525974025974025e-03*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45-1.9480519480519481e-04*w58*Jinv11-1.9480519480519481e-04*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-1.8262987012987013e-03*w43*Jinv11-1.8262987012987013e-03*Jinv21*w41-4.2613636363636367e-05*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-3.6525974025974025e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.8262987012987013e-03*Jinv01*w39+2.1915584415584417e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11+3.4699675324675324e-04*Jinv21*w53+4.3831168831168834e-04*w4*Jinv11+2.1915584415584417e-04*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+3.2873376623376621e-04*w46*Jinv11+4.3831168831168834e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33-3.2873376623376621e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+4.3831168831168834e-04*w6*Jinv01+3.2873376623376621e-04*w47*Jinv21+3.4699675324675324e-04*w52*Jinv11+6.5746753246753243e-04*w22*Jinv11+6.5746753246753243e-04*w21*Jinv01+3.4699675324675324e-04*Jinv01*w51+2.1915584415584417e-04*w36*Jinv01+4.3831168831168834e-04*w7*Jinv11+2.9220779220779221e-04*Jinv21*w56+3.4699675324675324e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-3.2873376623376621e-04*Jinv01*w12+4.3831168831168834e-04*Jinv21*w8+6.5746753246753243e-04*Jinv21*w23+2.9220779220779221e-04*Jinv01*w54+2.9220779220779221e-04*w55*Jinv11-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*w40*Jinv11+2.1915584415584417e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11+3.4699675324675324e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+3.4699675324675324e-04*w48*Jinv01-4.2613636363636367e-05*w10*Jinv11)*D;
  A[15][28]=(0.0)*D;
  A[15][29]=(0.0)*D;
  A[15][30]=(-1.5888798701298701e-03*Jinv00*w12-5.2962662337662338e-04*w28*Jinv10-3.6160714285714286e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-3.6160714285714286e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3+1.3697240259740260e-03*Jinv20*w23+3.2873376623376621e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-1.6436688311688311e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+6.5746753246753243e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+1.3697240259740260e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-3.6160714285714286e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-5.2962662337662338e-04*w28*Jinv11+1.3697240259740260e-03*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10+2.6846590909090908e-03*Jinv02*w3+1.3697240259740260e-03*w21*Jinv00+5.4788961038961036e-04*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39-1.6436688311688311e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+3.2873376623376621e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-4.6022727272727274e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-3.6160714285714286e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-4.6022727272727274e-03*w41*Jinv20+2.6846590909090908e-03*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+3.2873376623376621e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-3.6160714285714286e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-5.2962662337662338e-04*w11*Jinv20+6.5746753246753243e-04*w52*Jinv12-4.6022727272727274e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+6.5746753246753243e-04*Jinv02*w54+2.6846590909090908e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02+1.3697240259740260e-03*w49*Jinv10-5.2962662337662338e-04*Jinv21*w29-4.6022727272727274e-03*Jinv01*w33-4.6022727272727274e-03*w39*Jinv00+2.6846590909090908e-03*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+1.3697240259740260e-03*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+5.4788961038961036e-04*w31*Jinv12+3.2873376623376621e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8+1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+5.4788961038961036e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10+1.3697240259740260e-03*w7*Jinv11-5.2962662337662338e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+2.6846590909090908e-03*Jinv22*w5+3.2873376623376621e-04*w47*Jinv22+1.3697240259740260e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-5.2962662337662338e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+1.3697240259740260e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-3.6160714285714286e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-4.6022727272727274e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29+2.6846590909090908e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+3.2873376623376621e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+6.5746753246753243e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12+1.3697240259740260e-03*Jinv21*w50-5.2962662337662338e-04*Jinv01*w27+1.3697240259740260e-03*w48*Jinv01-1.6436688311688311e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[15][31]=(0.0)*D;
  A[15][32]=(0.0)*D;
  A[15][33]=(2.3011363636363637e-03*Jinv00*w12+3.9448051948051946e-03*w40*Jinv12-2.9586038961038961e-03*Jinv22*w53-2.9586038961038961e-03*Jinv20*w8+1.6436688311688311e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10+3.9448051948051946e-03*Jinv02*w39-1.1834415584415585e-02*Jinv21*w44+2.9586038961038961e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+1.9724025974025973e-03*Jinv01*w3-6.5746753246753243e-04*Jinv20*w23-1.1323051948051949e-03*Jinv21*w59-1.9724025974025973e-03*w25*Jinv12-6.9399350649350649e-04*w0*Jinv02-1.1323051948051949e-03*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10-1.1834415584415585e-02*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-1.9724025974025973e-03*Jinv12*w55+2.3668831168831169e-02*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-6.5746753246753243e-04*w48*Jinv00-1.9724025974025973e-03*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35-1.1323051948051949e-03*w58*Jinv11-1.1323051948051949e-03*w57*Jinv01+1.9724025974025973e-03*Jinv21*w26-1.1323051948051949e-03*w0*Jinv01-9.1314935064935063e-04*w1*Jinv10+9.8620129870129864e-04*Jinv22*w20-1.1834415584415585e-02*w43*Jinv11-7.8896103896103891e-03*Jinv21*w41+2.1915584415584417e-04*w11*Jinv22-1.1323051948051949e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-1.1323051948051949e-03*w28*Jinv11-6.5746753246753243e-04*Jinv20*w50+2.3668831168831169e-02*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3-6.5746753246753243e-04*w21*Jinv00+2.3011363636363637e-03*w32*Jinv22-7.8896103896103891e-03*Jinv01*w39-6.9399350649350649e-04*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00-9.8620129870129864e-04*Jinv21*w38+1.9724025974025973e-03*Jinv01*w24+1.6436688311688311e-03*Jinv22*w59-1.1834415584415585e-02*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11+1.9724025974025973e-03*Jinv21*w53+6.5746753246753243e-04*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+2.9586038961038961e-03*w4*Jinv10+1.9724025974025973e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+3.2873376623376621e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+1.3149350649350649e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-2.9586038961038961e-03*w6*Jinv00+3.9448051948051946e-03*Jinv22*w41+6.5746753246753243e-04*w26*Jinv20+9.1314935064935063e-04*w11*Jinv20-2.9586038961038961e-03*w52*Jinv12-1.1834415584415585e-02*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11-1.9724025974025973e-03*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+1.9724025974025973e-03*w5*Jinv21-1.9724025974025973e-03*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02-6.5746753246753243e-04*w49*Jinv10-1.1323051948051949e-03*Jinv21*w29-1.1834415584415585e-02*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+3.2873376623376621e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-9.8620129870129864e-04*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-9.8620129870129864e-04*w6*Jinv01-2.9586038961038961e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38-2.6298701298701297e-03*Jinv22*w23+1.9724025974025973e-03*w52*Jinv11+2.3011363636363637e-03*w31*Jinv12+6.5746753246753243e-04*Jinv20*w53+1.9724025974025973e-03*Jinv22*w8-6.5746753246753243e-04*w22*Jinv10-3.6160714285714286e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11+1.9724025974025973e-03*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01+1.9724025974025973e-03*Jinv01*w51+2.3011363636363637e-03*Jinv02*w30-3.6160714285714286e-03*Jinv12*w49-9.8620129870129864e-04*w36*Jinv01+6.5746753246753243e-04*w52*Jinv10-9.8620129870129864e-04*w7*Jinv11+9.1314935064935063e-04*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+1.3149350649350649e-03*Jinv21*w56+6.5746753246753243e-04*w24*Jinv00+1.1323051948051949e-03*Jinv02*w27+1.9724025974025973e-03*w6*Jinv02+9.8620129870129864e-04*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5+1.3149350649350649e-03*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+9.1314935064935063e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+1.6436688311688311e-03*w58*Jinv12+9.8620129870129864e-04*Jinv02*w18+1.3149350649350649e-03*Jinv01*w12-9.8620129870129864e-04*Jinv21*w8+5.9172077922077923e-03*Jinv02*w33-9.1314935064935063e-04*w0*Jinv00+1.3149350649350649e-03*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11+6.5746753246753243e-04*w25*Jinv10-9.1314935064935063e-04*Jinv20*w2-3.6160714285714286e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+2.3011363636363637e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20+1.1323051948051949e-03*Jinv22*w29+3.2873376623376621e-03*Jinv22*w14-7.8896103896103891e-03*w40*Jinv11+2.9586038961038961e-03*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-2.9586038961038961e-03*Jinv02*w51-1.9724025974025973e-03*Jinv22*w56+1.1323051948051949e-03*Jinv12*w28+1.9724025974025973e-03*w25*Jinv11-3.9448051948051946e-03*w42*Jinv00+2.3668831168831169e-02*w16*Jinv12+1.3149350649350649e-03*Jinv21*w50-1.1323051948051949e-03*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01-6.9399350649350649e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[15][34]=(0.0)*D;
  A[15][35]=(0.0)*D;
  A[15][36]=(-7.1225649350649346e-04*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53+1.5888798701298701e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-1.3697240259740260e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+2.1367694805194807e-03*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+4.6022727272727274e-03*w43*Jinv10-5.4788961038961042e-05*Jinv12*w55+5.2962662337662338e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+5.2962662337662338e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20-7.1225649350649346e-04*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17+4.6022727272727274e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22+4.9310064935064932e-04*Jinv22*w2+1.5888798701298701e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00-3.8352272727272725e-04*Jinv02*w45+2.7394480519480518e-04*Jinv22*w59-1.3697240259740260e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10+6.5746753246753243e-04*w41*Jinv20-7.6704545454545450e-04*Jinv12*w13+1.5888798701298701e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10+1.5888798701298701e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-1.3697240259740260e-03*w26*Jinv20-3.8352272727272725e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-5.4788961038961042e-05*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10+6.5746753246753243e-04*w39*Jinv00-7.6704545454545450e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+1.5888798701298701e-03*w7*Jinv10+1.5888798701298701e-03*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12-1.3697240259740260e-03*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+4.2735389610389614e-03*Jinv12*w7-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.3697240259740260e-03*w52*Jinv10-3.8352272727272725e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-1.3697240259740260e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22-3.8352272727272725e-04*w10*Jinv10+4.6022727272727274e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33+5.2962662337662338e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-1.3697240259740260e-03*w25*Jinv10+5.2962662337662338e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14+2.1367694805194807e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20-1.3149350649350649e-03*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10+4.6022727272727274e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+4.9310064935064932e-04*Jinv12*w1)*D;
  A[15][37]=(0.0)*D;
  A[15][38]=(0.0)*D;
  A[15][39]=(1.9724025974025973e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12-1.9724025974025973e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+1.6436688311688311e-03*w57*Jinv02+1.9724025974025973e-03*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3-9.8620129870129864e-04*Jinv20*w23+1.9724025974025973e-03*Jinv00*w54+9.8620129870129864e-04*w25*Jinv12-6.9399350649350649e-04*w0*Jinv02-9.1314935064935063e-04*Jinv21*w2-1.1834415584415585e-02*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45-2.9586038961038961e-03*Jinv12*w55-1.1323051948051949e-03*w58*Jinv10+2.3668831168831169e-02*w15*Jinv02+1.1323051948051949e-03*w10*Jinv12+1.3149350649350649e-03*w48*Jinv00+9.8620129870129864e-04*Jinv02*w24+3.9448051948051946e-03*Jinv22*w35-2.3011363636363637e-03*Jinv21*w26-9.1314935064935063e-04*w0*Jinv01-1.1323051948051949e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10-1.9724025974025973e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11-9.8620129870129864e-04*w45*Jinv00-2.1915584415584417e-04*Jinv00*w27+1.1323051948051949e-03*w11*Jinv22-9.1314935064935063e-04*Jinv11*w1+2.9586038961038961e-03*w13*Jinv11+9.1314935064935063e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-9.8620129870129864e-04*w46*Jinv10+2.3668831168831169e-02*Jinv22*w17-7.8896103896103891e-03*Jinv00*w33+1.9724025974025973e-03*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10+3.2873376623376621e-03*Jinv02*w3-9.8620129870129864e-04*w21*Jinv00+2.3011363636363637e-03*w32*Jinv22-6.9399350649350649e-04*Jinv22*w2+1.9724025974025973e-03*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00-2.3011363636363637e-03*Jinv01*w24+1.6436688311688311e-03*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+6.5746753246753243e-04*w19*Jinv11+1.3149350649350649e-03*w51*Jinv00+3.9448051948051946e-03*Jinv12*w34+1.3149350649350649e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11-1.1834415584415585e-02*w41*Jinv20+4.9310064935064934e-03*Jinv12*w13+1.9724025974025973e-03*w19*Jinv10-7.8896103896103891e-03*w34*Jinv10+2.3011363636363637e-03*w32*Jinv21+6.5746753246753243e-04*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+1.9724025974025973e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41-9.8620129870129864e-04*w26*Jinv20-1.1323051948051949e-03*w11*Jinv20-1.9724025974025973e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11+9.8620129870129864e-04*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+1.9724025974025973e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21-2.9586038961038961e-03*Jinv02*w54+3.2873376623376621e-03*w4*Jinv12+1.9724025974025973e-03*w21*Jinv02+1.3149350649350649e-03*w49*Jinv10+9.1314935064935063e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33-1.1834415584415585e-02*w39*Jinv00+4.9310064935064934e-03*Jinv02*w12+6.5746753246753243e-04*Jinv01*w18+1.1323051948051949e-03*Jinv02*w9-6.5746753246753243e-04*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+1.9724025974025973e-03*Jinv22*w23+2.3011363636363637e-03*w31*Jinv12+1.3149350649350649e-03*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8-9.8620129870129864e-04*w22*Jinv10-3.6160714285714286e-03*Jinv22*w50-2.9586038961038961e-03*w22*Jinv11-2.6298701298701297e-03*Jinv12*w7-2.9586038961038961e-03*w21*Jinv01+2.3011363636363637e-03*Jinv02*w30-3.6160714285714286e-03*Jinv12*w49+1.3149350649350649e-03*w52*Jinv10-6.5746753246753243e-04*w7*Jinv11-1.1323051948051949e-03*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+6.5746753246753243e-04*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-1.9724025974025973e-03*w19*Jinv12+3.2873376623376621e-03*Jinv22*w5-6.5746753246753243e-04*w49*Jinv11+2.9586038961038961e-03*Jinv21*w14-1.1323051948051949e-03*w10*Jinv10-1.1834415584415585e-02*Jinv20*w44+1.6436688311688311e-03*w58*Jinv12-1.9724025974025973e-03*Jinv02*w18+2.9586038961038961e-03*Jinv01*w12-6.5746753246753243e-04*Jinv21*w8-1.1323051948051949e-03*w57*Jinv00-1.1323051948051949e-03*Jinv20*w59+3.9448051948051946e-03*Jinv02*w33-1.1323051948051949e-03*w0*Jinv00-7.8896103896103891e-03*w35*Jinv20-2.9586038961038961e-03*Jinv21*w23+6.5746753246753243e-04*Jinv01*w54+6.5746753246753243e-04*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-1.1323051948051949e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-3.6160714285714286e-03*w48*Jinv02-1.1834415584415585e-02*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-1.1834415584415585e-02*w40*Jinv10+1.9724025974025973e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14+1.3149350649350649e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30-1.9724025974025973e-03*Jinv02*w51-2.9586038961038961e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-2.3011363636363637e-03*w25*Jinv11+1.9724025974025973e-03*w55*Jinv10-1.1834415584415585e-02*w42*Jinv00-9.8620129870129864e-04*w47*Jinv20+2.3668831168831169e-02*w16*Jinv12-6.5746753246753243e-04*Jinv21*w50+9.1314935064935063e-04*Jinv01*w27-6.5746753246753243e-04*w48*Jinv01-6.9399350649350649e-04*Jinv12*w1)*D;
  A[15][40]=(0.0)*D;
  A[15][41]=(0.0)*D;
  A[15][42]=(-1.9724025974025973e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+2.7759740259740259e-03*w57*Jinv02-1.9724025974025973e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+1.1834415584415585e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-1.9724025974025973e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23+9.8620129870129864e-04*Jinv22*w38-1.9724025974025973e-03*Jinv00*w54+1.1323051948051949e-03*Jinv21*w59+2.9586038961038961e-03*w25*Jinv12+2.2646103896103898e-03*w0*Jinv02+1.1323051948051949e-03*Jinv21*w2+1.1834415584415585e-02*w43*Jinv10+1.1834415584415585e-02*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55+1.1323051948051949e-03*w58*Jinv10+3.5503246753246752e-02*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00+2.9586038961038961e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+1.1323051948051949e-03*w58*Jinv11+1.1323051948051949e-03*w57*Jinv01-1.9724025974025973e-03*Jinv21*w26+1.1323051948051949e-03*w0*Jinv01+1.1323051948051949e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10+2.9586038961038961e-03*Jinv22*w20+1.1834415584415585e-02*w43*Jinv11+9.8620129870129864e-04*w45*Jinv00+7.8896103896103891e-03*Jinv21*w41+1.7751623376623376e-02*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+1.1323051948051949e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+1.1323051948051949e-03*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+9.8620129870129864e-04*w46*Jinv10+3.5503246753246752e-02*Jinv22*w17+7.8896103896103891e-03*Jinv00*w33+1.9724025974025973e-03*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-3.9448051948051946e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+7.8896103896103891e-03*Jinv01*w39+2.2646103896103898e-03*Jinv22*w2-1.9724025974025973e-03*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00+9.8620129870129864e-04*Jinv21*w38+9.8620129870129864e-04*Jinv02*w45-1.9724025974025973e-03*Jinv01*w24+2.7759740259740259e-03*Jinv22*w59+1.1834415584415585e-02*w34*Jinv11+9.8620129870129864e-04*w19*Jinv11-1.9724025974025973e-03*Jinv21*w53-1.3149350649350649e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+1.7751623376623376e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-1.9724025974025973e-03*w4*Jinv11+1.1834415584415585e-02*w41*Jinv20-3.9448051948051946e-03*Jinv12*w13-1.9724025974025973e-03*w19*Jinv10+7.8896103896103891e-03*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21+9.8620129870129864e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+9.8620129870129864e-04*w37*Jinv12-1.9724025974025973e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+9.8620129870129864e-04*w26*Jinv20+1.1323051948051949e-03*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12+1.1834415584415585e-02*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+2.9586038961038961e-03*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-1.9724025974025973e-03*Jinv20*w14-1.9724025974025973e-03*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54-3.9448051948051946e-03*w4*Jinv12+1.9724025974025973e-03*w21*Jinv02-1.3149350649350649e-03*w49*Jinv10+1.1323051948051949e-03*Jinv21*w29+1.1834415584415585e-02*Jinv01*w33+1.1834415584415585e-02*w39*Jinv00-3.9448051948051946e-03*Jinv02*w12+9.8620129870129864e-04*Jinv01*w18+9.8620129870129864e-04*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-1.3149350649350649e-03*w47*Jinv21-1.3149350649350649e-03*Jinv20*w38+1.9724025974025973e-03*Jinv22*w23-1.9724025974025973e-03*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+9.8620129870129864e-04*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53+1.9724025974025973e-03*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10-3.2873376623376621e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+1.9724025974025973e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01-1.9724025974025973e-03*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-3.2873376623376621e-03*Jinv12*w49+9.8620129870129864e-04*w36*Jinv01-1.3149350649350649e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+1.1323051948051949e-03*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10-1.3149350649350649e-03*Jinv21*w56+9.8620129870129864e-04*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27+1.9724025974025973e-03*w6*Jinv02+2.9586038961038961e-03*w19*Jinv12-3.9448051948051946e-03*Jinv22*w5+9.8620129870129864e-04*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.1323051948051949e-03*w10*Jinv10+1.1834415584415585e-02*Jinv20*w44+2.7759740259740259e-03*w58*Jinv12+2.9586038961038961e-03*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8+1.1323051948051949e-03*w57*Jinv00+1.1323051948051949e-03*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+1.1323051948051949e-03*w0*Jinv00+7.8896103896103891e-03*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11+9.8620129870129864e-04*w25*Jinv10+1.1323051948051949e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-3.2873376623376621e-03*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+1.1834415584415585e-02*w40*Jinv10-1.9724025974025973e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29-3.9448051948051946e-03*Jinv22*w14+7.8896103896103891e-03*w40*Jinv11+9.8620129870129864e-04*Jinv02*w36+1.7751623376623376e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-1.9724025974025973e-03*w25*Jinv11-1.9724025974025973e-03*w55*Jinv10+1.1834415584415585e-02*w42*Jinv00+9.8620129870129864e-04*w47*Jinv20+3.5503246753246752e-02*w16*Jinv12-1.3149350649350649e-03*Jinv21*w50+1.1323051948051949e-03*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+2.2646103896103898e-03*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[15][43]=(0.0)*D;
  A[15][44]=(0.0)*D;
  A[15][45]=(9.8620129870129864e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.7394480519480518e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39+4.6022727272727274e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-3.8352272727272725e-04*Jinv22*w38+5.2962662337662338e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+5.2962662337662338e-04*Jinv21*w2+4.6022727272727274e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+1.9724025974025973e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+5.2962662337662338e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+4.6022727272727274e-03*w43*Jinv11+4.6022727272727274e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02+4.9310064935064932e-04*w11*Jinv22+5.2962662337662338e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+1.9724025974025973e-03*Jinv22*w17+4.2735389610389614e-03*w22*Jinv12-7.6704545454545450e-04*Jinv02*w3-3.8352272727272725e-04*w32*Jinv22+4.6022727272727274e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+1.5888798701298701e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-1.3697240259740260e-03*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-7.1225649350649346e-04*w32*Jinv21-1.3697240259740260e-03*Jinv21*w20-3.8352272727272725e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+1.5888798701298701e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-3.8352272727272725e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-1.3697240259740260e-03*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+1.5888798701298701e-03*w47*Jinv21+4.2735389610389614e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-7.6704545454545450e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+1.5888798701298701e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.5888798701298701e-03*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-1.3697240259740260e-03*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14+2.7394480519480518e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+1.5888798701298701e-03*Jinv21*w23-1.3697240259740260e-03*Jinv01*w54-1.3697240259740260e-03*w55*Jinv11-5.4788961038961042e-05*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+4.6022727272727274e-03*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-7.1225649350649346e-04*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.5888798701298701e-03*w25*Jinv11+1.9724025974025973e-03*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[15][46]=(0.0)*D;
  A[15][47]=(0.0)*D;
  A[15][48]=(5.4788961038961042e-05*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+5.4788961038961042e-05*Jinv20*w8-1.8262987012987014e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+3.2873376623376621e-03*w16*Jinv11+3.2873376623376621e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-6.5746753246753243e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-1.0957792207792207e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+3.2873376623376621e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11+3.8352272727272725e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+3.2873376623376621e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.0957792207792207e-03*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10-6.5746753246753243e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10-2.5750811688311690e-03*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00+3.8352272727272725e-04*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-1.8262987012987014e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-2.5750811688311690e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+3.8352272727272725e-04*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+1.3149350649350649e-03*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-2.5750811688311690e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-1.0957792207792207e-03*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-2.5750811688311690e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+5.4788961038961036e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30+5.4788961038961036e-04*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-2.5750811688311690e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22-1.0957792207792207e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44-1.8262987012987014e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+3.2873376623376621e-03*w15*Jinv00+5.4788961038961036e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9+3.2873376623376621e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-2.5750811688311690e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+3.8352272727272725e-04*Jinv01*w30-3.6525974025974026e-05*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20-6.5746753246753243e-03*w16*Jinv12-1.0957792207792207e-03*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27-1.0957792207792207e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[15][49]=(0.0)*D;
  A[15][50]=(0.0)*D;
  A[15][51]=(-5.4788961038961042e-05*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12+1.6436688311688311e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-6.7573051948051951e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39-5.4788961038961042e-05*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-8.2183441558441553e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-7.3051948051948052e-05*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10+1.0957792207792207e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129868e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35-3.6525974025974026e-05*w1*Jinv10-3.2873376623376621e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20-3.8352272727272725e-04*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+1.0957792207792207e-03*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10-9.8620129870129868e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-3.8352272727272725e-04*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22-7.3051948051948052e-05*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45-6.7573051948051951e-04*Jinv22*w59+1.0957792207792207e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-8.2183441558441553e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20+1.6436688311688311e-03*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26-5.4788961038961042e-05*Jinv20*w14+1.0957792207792207e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-1.3149350649350649e-03*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9-5.4788961038961042e-05*w7*Jinv10-3.8352272727272725e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+1.0957792207792207e-03*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-2.6298701298701297e-03*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49+1.0957792207792207e-03*w52*Jinv10-3.6525974025974026e-05*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-5.4788961038961042e-05*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22-3.6525974025974026e-05*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-6.7573051948051951e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-3.6525974025974026e-05*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-3.2873376623376621e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02-3.2873376623376621e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-8.2183441558441553e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+1.6436688311688311e-03*Jinv02*w51+1.0957792207792207e-03*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28+1.0957792207792207e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-3.8352272727272725e-04*w47*Jinv20-9.8620129870129868e-03*w16*Jinv12-7.3051948051948052e-05*Jinv12*w1)*D;
  A[15][52]=(0.0)*D;
  A[15][53]=(0.0)*D;
  A[15][54]=(9.8620129870129864e-04*w40*Jinv12+1.0957792207792207e-03*Jinv22*w53-6.7573051948051951e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38-4.9310064935064932e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-7.3051948051948052e-05*w0*Jinv02-3.6525974025974026e-05*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-3.2873376623376621e-03*w16*Jinv11-3.2873376623376621e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+1.6436688311688311e-03*Jinv12*w55-9.8620129870129868e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12-2.6298701298701297e-03*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-7.3051948051948052e-05*w11*Jinv22-3.6525974025974026e-05*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-3.2873376623376621e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-9.8620129870129868e-03*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-7.3051948051948052e-05*Jinv22*w2-3.8352272727272725e-04*Jinv21*w38-8.2183441558441553e-04*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-6.7573051948051951e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53+6.5746753246753243e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-5.4788961038961042e-05*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-3.8352272727272725e-04*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41+1.0957792207792207e-03*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-3.8352272727272725e-04*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+1.6436688311688311e-03*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-3.8352272727272725e-04*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-3.8352272727272725e-04*w47*Jinv21-2.6298701298701297e-03*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12-8.2183441558441553e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11+1.0957792207792207e-03*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5-8.2183441558441553e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-6.7573051948051951e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+6.5746753246753243e-04*Jinv02*w33-5.4788961038961042e-05*Jinv21*w23+1.0957792207792207e-03*Jinv01*w54+1.0957792207792207e-03*w55*Jinv11+1.0957792207792207e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*Jinv01*w30+1.0957792207792207e-03*Jinv02*w51+1.6436688311688311e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11-9.8620129870129868e-03*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01-7.3051948051948052e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[15][55]=(0.0)*D;
  A[15][56]=(0.0)*D;
  A[15][57]=(-5.1136363636363640e-04*w40*Jinv12-3.1047077922077924e-04*Jinv22*w53+2.4350649350649351e-04*w57*Jinv02-5.1136363636363640e-04*Jinv02*w39+2.1915584415584417e-04*Jinv22*w38+5.4788961038961036e-04*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02-3.1047077922077924e-04*Jinv12*w55+5.5519480519480519e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12+5.4788961038961036e-04*Jinv02*w24-5.1136363636363640e-04*Jinv22*w35+5.4788961038961036e-04*Jinv22*w20-5.1136363636363640e-04*w42*Jinv02+3.0438311688311688e-05*w11*Jinv22+5.5519480519480519e-03*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+2.1915584415584417e-04*w32*Jinv22+3.0438311688311688e-05*Jinv22*w2+2.1915584415584417e-04*Jinv02*w45+2.4350649350649351e-04*Jinv22*w59-5.1136363636363640e-04*Jinv12*w34-5.1136363636363640e-04*Jinv22*w44+5.4788961038961036e-04*Jinv12*w13+2.1915584415584417e-04*w37*Jinv12-5.1136363636363640e-04*Jinv22*w41-3.1047077922077924e-04*w52*Jinv12+5.4788961038961036e-04*Jinv22*w26-3.1047077922077924e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02+5.4788961038961036e-04*Jinv02*w12+3.0438311688311688e-05*Jinv02*w9+5.4788961038961036e-04*Jinv22*w23+2.1915584415584417e-04*w31*Jinv12+2.1915584415584417e-04*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-3.1047077922077924e-04*Jinv22*w50+5.4788961038961036e-04*Jinv12*w7+2.1915584415584417e-04*Jinv02*w30-3.1047077922077924e-04*Jinv12*w49+3.0438311688311688e-05*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+2.1915584415584417e-04*w47*Jinv22+2.4350649350649351e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-5.1136363636363640e-04*Jinv02*w33-3.1047077922077924e-04*w48*Jinv02+3.0438311688311688e-05*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14+2.1915584415584417e-04*Jinv02*w36-5.1136363636363640e-04*Jinv12*w43-3.1047077922077924e-04*Jinv02*w51-3.1047077922077924e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28+5.5519480519480519e-03*w16*Jinv12+3.0438311688311688e-05*Jinv12*w1)*D;
  A[15][58]=(0.0)*D;
  A[15][59]=(0.0)*D;
  A[16][0]=(0.0)*D;
  A[16][1]=(-6.5746753246753243e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12-3.4699675324675324e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-6.5746753246753243e-04*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-2.1915584415584417e-04*Jinv22*w38-3.4699675324675324e-04*Jinv00*w54+1.9480519480519481e-04*Jinv21*w59-4.3831168831168834e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+3.6525974025974025e-03*w16*Jinv11+3.6525974025974025e-03*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45-3.4699675324675324e-04*Jinv12*w55+1.9480519480519481e-04*w58*Jinv10+3.6525974025974025e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12-2.9220779220779221e-04*w48*Jinv00-4.3831168831168834e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35+1.9480519480519481e-04*w58*Jinv11+1.9480519480519481e-04*w57*Jinv01-4.3831168831168834e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+3.6525974025974025e-03*w16*Jinv10-4.3831168831168834e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27+4.2613636363636367e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-6.5746753246753243e-04*w13*Jinv11+3.6525974025974025e-03*w15*Jinv01+4.2613636363636367e-05*w28*Jinv11-2.9220779220779221e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+3.6525974025974025e-03*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-4.3831168831168834e-04*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38-2.1915584415584417e-04*Jinv02*w45-4.3831168831168834e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59+1.8262987012987013e-03*w34*Jinv11-4.3831168831168834e-04*w19*Jinv11-3.4699675324675324e-04*Jinv21*w53-3.4699675324675324e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-6.5746753246753243e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11+1.8262987012987013e-03*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-4.3831168831168834e-04*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10-3.2873376623376621e-04*w32*Jinv21-4.3831168831168834e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00-2.1915584415584417e-04*w37*Jinv12-3.4699675324675324e-04*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-4.3831168831168834e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20-3.4699675324675324e-04*w52*Jinv12+1.8262987012987013e-03*Jinv21*w35-3.2873376623376621e-04*w31*Jinv11-4.3831168831168834e-04*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21-3.4699675324675324e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12+3.2873376623376621e-04*w21*Jinv02-2.9220779220779221e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+1.8262987012987013e-03*Jinv01*w33+1.8262987012987013e-03*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-4.3831168831168834e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-3.4699675324675324e-04*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12-2.1915584415584417e-04*w46*Jinv12-3.4699675324675324e-04*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10-2.9220779220779221e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01-3.4699675324675324e-04*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30-2.9220779220779221e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01-3.4699675324675324e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10-3.4699675324675324e-04*Jinv21*w56-4.3831168831168834e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02-4.3831168831168834e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5-2.1915584415584417e-04*w47*Jinv22-2.9220779220779221e-04*w49*Jinv11-6.5746753246753243e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12-4.3831168831168834e-04*Jinv02*w18-6.5746753246753243e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+1.9480519480519481e-04*w57*Jinv00+1.9480519480519481e-04*Jinv20*w59+1.8262987012987013e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+1.8262987012987013e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-3.4699675324675324e-04*Jinv01*w54-3.4699675324675324e-04*w55*Jinv11-4.3831168831168834e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+3.6525974025974025e-03*w15*Jinv00-2.9220779220779221e-04*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+3.6525974025974025e-03*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+1.8262987012987013e-03*w40*Jinv10-4.3831168831168834e-04*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11-2.1915584415584417e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-6.5746753246753243e-04*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30-3.4699675324675324e-04*Jinv02*w51-3.4699675324675324e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-4.3831168831168834e-04*w25*Jinv11-3.4699675324675324e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20+3.6525974025974025e-03*w16*Jinv12-2.9220779220779221e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-2.9220779220779221e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[16][2]=(0.0)*D;
  A[16][3]=(0.0)*D;
  A[16][4]=(1.0957792207792207e-03*Jinv00*w12-5.6615259740259744e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-9.6793831168831166e-04*w57*Jinv02+1.0957792207792207e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-4.7666396103896100e-03*Jinv01*w3+6.0267857142857139e-04*Jinv20*w23+8.2183441558441553e-04*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-9.6793831168831166e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.3806818181818182e-02*w16*Jinv11-1.3806818181818182e-02*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+8.7662337662337668e-04*Jinv12*w55-6.9399350649350649e-04*w58*Jinv10-1.3806818181818182e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+1.2053571428571428e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-6.9034090909090911e-03*Jinv22*w35-9.6793831168831166e-04*w58*Jinv11-9.6793831168831166e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10+1.6436688311688311e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+3.2873376623376621e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.6615259740259744e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-3.1777597402597403e-03*w13*Jinv11-1.3806818181818182e-02*w15*Jinv01-1.0592532467532468e-03*w28*Jinv11+1.2053571428571428e-03*Jinv20*w50-5.6615259740259744e-04*Jinv20*w29+3.2873376623376621e-04*w46*Jinv10-1.3806818181818182e-02*Jinv22*w17-5.9172077922077923e-03*Jinv00*w33+2.7394480519480520e-03*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10-4.7666396103896100e-03*Jinv02*w3+6.0267857142857139e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+8.7662337662337668e-04*Jinv00*w18+4.3831168831168834e-04*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-9.6793831168831166e-04*Jinv22*w59-6.9034090909090911e-03*w34*Jinv11+1.6436688311688311e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+1.2601461038961039e-03*w51*Jinv00-6.9034090909090911e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-4.7666396103896100e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-3.1777597402597403e-03*Jinv12*w13+8.7662337662337668e-04*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+1.6436688311688311e-03*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00+8.2183441558441553e-04*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12-6.9034090909090911e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+1.0957792207792207e-03*Jinv20*w14-4.7666396103896100e-03*w5*Jinv21+8.7662337662337668e-04*Jinv02*w54-4.7666396103896100e-03*w4*Jinv12+2.7394480519480520e-03*w21*Jinv02+1.2053571428571428e-03*w49*Jinv10-1.0592532467532468e-03*Jinv21*w29-6.9034090909090911e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-3.1777597402597403e-03*Jinv02*w12+1.6436688311688311e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+6.5746753246753243e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+4.3831168831168834e-04*Jinv20*w38+2.7394480519480520e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+1.2601461038961039e-03*Jinv20*w53+6.5746753246753243e-04*Jinv22*w8+6.0267857142857139e-04*w22*Jinv10+2.5202922077922079e-03*Jinv22*w50+2.7394480519480520e-03*w22*Jinv11+6.5746753246753243e-04*Jinv12*w7+2.7394480519480520e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+2.5202922077922079e-03*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+1.2601461038961039e-03*w52*Jinv10+6.5746753246753243e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+4.3831168831168834e-04*w37*Jinv10+8.7662337662337668e-04*Jinv21*w56+1.3697240259740260e-03*w24*Jinv00-1.0592532467532468e-03*Jinv02*w27+6.5746753246753243e-04*w6*Jinv02+1.6436688311688311e-03*w19*Jinv12-4.7666396103896100e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+2.5202922077922079e-03*w49*Jinv11-3.1777597402597403e-03*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-9.6793831168831166e-04*w58*Jinv12+1.6436688311688311e-03*Jinv02*w18-3.1777597402597403e-03*Jinv01*w12+6.5746753246753243e-04*Jinv21*w8-6.9399350649350649e-04*w57*Jinv00-6.9399350649350649e-04*Jinv20*w59-6.9034090909090911e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+2.7394480519480520e-03*Jinv21*w23+8.7662337662337668e-04*Jinv01*w54+8.7662337662337668e-04*w55*Jinv11+1.3697240259740260e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00+2.5202922077922079e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-1.1834415584415585e-02*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10+8.7662337662337668e-04*w20*Jinv20-1.0592532467532468e-03*Jinv22*w29-3.1777597402597403e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+8.2183441558441553e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+8.7662337662337668e-04*Jinv22*w56-1.0592532467532468e-03*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+3.2873376623376621e-04*w47*Jinv20-1.3806818181818182e-02*w16*Jinv12+2.5202922077922079e-03*Jinv21*w50-1.0592532467532468e-03*Jinv01*w27+2.5202922077922079e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[16][5]=(0.0)*D;
  A[16][6]=(0.0)*D;
  A[16][7]=(-8.7662337662337668e-04*Jinv00*w12+5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-8.7662337662337668e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00-4.3831168831168834e-04*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55+6.9399350649350649e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-1.2601461038961039e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-3.2873376623376621e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.6615259740259744e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+7.6704545454545450e-04*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-1.2601461038961039e-03*Jinv20*w50+5.6615259740259744e-04*Jinv20*w29-3.2873376623376621e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12-4.3831168831168834e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-1.0957792207792207e-03*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53-1.2053571428571428e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10+2.1367694805194807e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-1.0957792207792207e-03*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10+3.8352272727272725e-04*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-6.0267857142857139e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11-8.7662337662337668e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02-1.2601461038961039e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-1.2053571428571428e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+7.6704545454545450e-04*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+7.6704545454545450e-04*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-1.2053571428571428e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56-6.0267857142857139e-04*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+7.6704545454545450e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18+7.6704545454545450e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8+6.9399350649350649e-04*w57*Jinv00+6.9399350649350649e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+7.6704545454545450e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-6.0267857142857139e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-4.3831168831168834e-04*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10-1.0957792207792207e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+3.8352272727272725e-04*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11-8.2183441558441553e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-3.2873376623376621e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[16][8]=(0.0)*D;
  A[16][9]=(0.0)*D;
  A[16][10]=(4.3831168831168834e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10+6.5746753246753243e-04*Jinv20*w8+4.3831168831168834e-04*w13*Jinv10-3.2873376623376621e-04*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20+4.3831168831168834e-04*Jinv20*w23+3.4699675324675324e-04*Jinv00*w54-1.8262987012987013e-03*w43*Jinv10-1.9480519480519481e-04*w58*Jinv10+3.4699675324675324e-04*w48*Jinv00-4.2613636363636367e-05*w1*Jinv10-3.6525974025974025e-03*w16*Jinv10+2.1915584415584417e-04*w45*Jinv00-4.2613636363636367e-05*Jinv00*w27+3.4699675324675324e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-1.8262987012987013e-03*Jinv00*w33+2.1915584415584417e-04*w31*Jinv10+4.3831168831168834e-04*w21*Jinv00+6.5746753246753243e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00+2.9220779220779221e-04*w51*Jinv00-3.2873376623376621e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+6.5746753246753243e-04*w6*Jinv00+3.4699675324675324e-04*Jinv20*w56-3.2873376623376621e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+4.3831168831168834e-04*Jinv20*w14+3.4699675324675324e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+6.5746753246753243e-04*w7*Jinv10+3.2873376623376621e-04*Jinv20*w38+2.9220779220779221e-04*Jinv20*w53+4.3831168831168834e-04*w22*Jinv10+2.9220779220779221e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10-3.2873376623376621e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-1.8262987012987013e-03*Jinv20*w44-1.9480519480519481e-04*w57*Jinv00-1.9480519480519481e-04*Jinv20*w59-4.2613636363636367e-05*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20-3.2873376623376621e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-3.6525974025974025e-03*w15*Jinv00-3.6525974025974025e-03*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-3.2873376623376621e-04*w5*Jinv20+3.4699675324675324e-04*w55*Jinv10-1.8262987012987013e-03*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20)*D;
  A[16][11]=(0.0)*D;
  A[16][12]=(0.0)*D;
  A[16][13]=(-4.7666396103896100e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-6.9034090909090911e-03*w40*Jinv12+8.7662337662337668e-04*Jinv22*w53+2.7394480519480520e-03*Jinv20*w8-9.6793831168831166e-04*w57*Jinv02-4.7666396103896100e-03*w13*Jinv10-6.9034090909090911e-03*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-3.1777597402597403e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20+1.0957792207792207e-03*Jinv01*w3+6.5746753246753243e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-6.9399350649350649e-04*Jinv21*w59+1.6436688311688311e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-5.6615259740259744e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+4.3831168831168834e-04*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-9.6793831168831166e-04*w58*Jinv10-1.3806818181818182e-02*w15*Jinv02-1.0592532467532468e-03*w10*Jinv12+2.5202922077922079e-03*w48*Jinv00+1.6436688311688311e-03*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-6.9399350649350649e-04*w58*Jinv11-6.9399350649350649e-04*w57*Jinv01+8.7662337662337668e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-1.3806818181818182e-02*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.0592532467532468e-03*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+2.5202922077922079e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-1.3806818181818182e-02*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+6.5746753246753243e-04*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10-3.1777597402597403e-03*Jinv02*w3+6.5746753246753243e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.2873376623376621e-04*Jinv21*w38+8.2183441558441553e-04*Jinv02*w45+8.7662337662337668e-04*Jinv01*w24-9.6793831168831166e-04*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+8.7662337662337668e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-3.1777597402597403e-03*w4*Jinv10+1.0957792207792207e-03*w4*Jinv11-6.9034090909090911e-03*w41*Jinv20-4.7666396103896100e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+2.7394480519480520e-03*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-6.9034090909090911e-03*Jinv22*w41+1.6436688311688311e-03*w26*Jinv20-1.0592532467532468e-03*w11*Jinv20+8.7662337662337668e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11+1.6436688311688311e-03*Jinv22*w26+4.3831168831168834e-04*w46*Jinv11-4.7666396103896100e-03*Jinv20*w14+1.0957792207792207e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54-3.1777597402597403e-03*w4*Jinv12+6.5746753246753243e-04*w21*Jinv02+2.5202922077922079e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-6.9034090909090911e-03*w39*Jinv00-4.7666396103896100e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.2873376623376621e-04*w37*Jinv11-1.0592532467532468e-03*Jinv02*w9+6.0267857142857139e-04*w6*Jinv01+2.7394480519480520e-03*w7*Jinv10+4.3831168831168834e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+6.5746753246753243e-04*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+8.2183441558441553e-04*w46*Jinv12+8.7662337662337668e-04*Jinv20*w53+2.7394480519480520e-03*Jinv22*w8+6.5746753246753243e-04*w22*Jinv10+2.5202922077922079e-03*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+2.7394480519480520e-03*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+2.5202922077922079e-03*Jinv12*w49+3.2873376623376621e-04*w36*Jinv01+8.7662337662337668e-04*w52*Jinv10+6.0267857142857139e-04*w7*Jinv11-1.0592532467532468e-03*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.2601461038961039e-03*Jinv21*w56+1.6436688311688311e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+2.7394480519480520e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-3.1777597402597403e-03*Jinv22*w5+8.2183441558441553e-04*w47*Jinv22+1.2053571428571428e-03*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14-1.0592532467532468e-03*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-9.6793831168831166e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+6.0267857142857139e-04*Jinv21*w8-9.6793831168831166e-04*w57*Jinv00-9.6793831168831166e-04*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23+1.2601461038961039e-03*Jinv01*w54+1.2601461038961039e-03*w55*Jinv11+1.6436688311688311e-03*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-1.3806818181818182e-02*w15*Jinv00+2.5202922077922079e-03*w48*Jinv02-5.6615259740259744e-04*Jinv01*w9-1.3806818181818182e-02*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-6.9034090909090911e-03*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-4.7666396103896100e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-3.1777597402597403e-03*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+8.7662337662337668e-04*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+8.7662337662337668e-04*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-1.3806818181818182e-02*w16*Jinv12+1.2053571428571428e-03*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+1.2053571428571428e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-5.6615259740259744e-04*w10*Jinv11)*D;
  A[16][14]=(0.0)*D;
  A[16][15]=(0.0)*D;
  A[16][16]=(6.9034090909090911e-03*Jinv00*w12-1.1834415584415585e-02*w40*Jinv12+4.9310064935064934e-03*Jinv22*w53-5.9172077922077923e-03*Jinv20*w8-2.7759740259740259e-03*w57*Jinv02+6.9034090909090911e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-5.9172077922077923e-03*Jinv21*w44+5.9172077922077923e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+6.9034090909090911e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38-2.9586038961038961e-03*w25*Jinv12-2.2646103896103898e-03*w0*Jinv02-1.8262987012987013e-03*Jinv21*w2-5.9172077922077923e-03*w43*Jinv10-5.9172077922077923e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45+4.9310064935064934e-03*Jinv12*w55-3.5503246753246752e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12-1.6436688311688311e-03*w48*Jinv00-2.9586038961038961e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-6.9034090909090911e-03*Jinv21*w26-1.8262987012987013e-03*w0*Jinv01-1.8262987012987013e-03*w1*Jinv10-2.9586038961038961e-03*Jinv22*w20-5.9172077922077923e-03*w43*Jinv11-1.7751623376623376e-02*w42*Jinv02-4.3831168831168834e-04*w11*Jinv22-1.8262987012987013e-03*Jinv11*w1+5.9172077922077923e-03*w13*Jinv11+1.8262987012987013e-03*w28*Jinv11-1.6436688311688311e-03*Jinv20*w50-3.5503246753246752e-02*Jinv22*w17-1.9724025974025973e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+3.9448051948051946e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-2.2646103896103898e-03*Jinv22*w2-6.9034090909090911e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-6.9034090909090911e-03*Jinv01*w24-2.7759740259740259e-03*Jinv22*w59+5.9172077922077923e-03*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11+1.6436688311688311e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-1.7751623376623376e-02*Jinv22*w44+5.9172077922077923e-03*w4*Jinv10+6.9034090909090911e-03*w4*Jinv11+5.9172077922077923e-03*w41*Jinv20+3.9448051948051946e-03*Jinv12*w13-6.9034090909090911e-03*w19*Jinv10+2.3011363636363637e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-5.9172077922077923e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12-1.1834415584415585e-02*Jinv22*w41-9.8620129870129864e-04*w26*Jinv20+1.8262987012987013e-03*w11*Jinv20+4.9310064935064934e-03*w52*Jinv12+5.9172077922077923e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11-2.9586038961038961e-03*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+6.9034090909090911e-03*Jinv20*w14+6.9034090909090911e-03*w5*Jinv21+4.9310064935064934e-03*Jinv02*w54+3.9448051948051946e-03*w4*Jinv12-1.9724025974025973e-03*w21*Jinv02-1.6436688311688311e-03*w49*Jinv10+1.8262987012987013e-03*Jinv21*w29+5.9172077922077923e-03*Jinv01*w33+5.9172077922077923e-03*w39*Jinv00+3.9448051948051946e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-5.9172077922077923e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38-1.9724025974025973e-03*Jinv22*w23+1.3149350649350649e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12+1.6436688311688311e-03*Jinv20*w53-1.9724025974025973e-03*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10+3.2873376623376621e-03*Jinv22*w50-5.9172077922077923e-03*w22*Jinv11-1.9724025974025973e-03*Jinv12*w7-5.9172077922077923e-03*w21*Jinv01+1.3149350649350649e-03*Jinv02*w30+3.2873376623376621e-03*Jinv12*w49+1.6436688311688311e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+1.8262987012987013e-03*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+1.6436688311688311e-03*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27-1.9724025974025973e-03*w6*Jinv02-2.9586038961038961e-03*w19*Jinv12+3.9448051948051946e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22-1.6436688311688311e-03*w49*Jinv11+5.9172077922077923e-03*Jinv21*w14+1.8262987012987013e-03*w10*Jinv10-5.9172077922077923e-03*Jinv20*w44-2.7759740259740259e-03*w58*Jinv12-2.9586038961038961e-03*Jinv02*w18+5.9172077922077923e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8-1.1834415584415585e-02*Jinv02*w33-1.8262987012987013e-03*w0*Jinv00-5.9172077922077923e-03*Jinv21*w23+1.6436688311688311e-03*Jinv01*w54+1.6436688311688311e-03*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-1.8262987012987013e-03*Jinv20*w2+3.2873376623376621e-03*w48*Jinv02+2.3011363636363637e-03*w30*Jinv00+5.9172077922077923e-03*w40*Jinv10-6.9034090909090911e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29+3.9448051948051946e-03*Jinv22*w14-9.8620129870129864e-04*Jinv02*w36-1.7751623376623376e-02*Jinv12*w43+5.9172077922077923e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30+4.9310064935064934e-03*Jinv02*w51+4.9310064935064934e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28-6.9034090909090911e-03*w25*Jinv11-5.9172077922077923e-03*w42*Jinv00-3.5503246753246752e-02*w16*Jinv12-1.6436688311688311e-03*Jinv21*w50+1.8262987012987013e-03*Jinv01*w27-1.6436688311688311e-03*w48*Jinv01-2.2646103896103898e-03*Jinv12*w1)*D;
  A[16][17]=(0.0)*D;
  A[16][18]=(0.0)*D;
  A[16][19]=(-2.1367694805194807e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+3.1777597402597403e-03*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-2.7394480519480520e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-1.6436688311688311e-03*Jinv20*w23-1.3149350649350649e-03*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+4.9310064935064932e-04*Jinv21*w2+6.9034090909090911e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+9.6793831168831166e-04*w58*Jinv10-8.7662337662337668e-04*w48*Jinv00+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+1.0592532467532468e-03*w1*Jinv10+1.3806818181818182e-02*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+4.3831168831168834e-04*Jinv00*w27+4.9310064935064932e-04*Jinv11*w1-7.6704545454545450e-04*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-8.7662337662337668e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-03*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+4.7666396103896100e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-2.1367694805194807e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-2.5202922077922079e-03*w51*Jinv00-2.7394480519480520e-03*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+4.7666396103896100e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-3.8352272727272725e-04*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+3.1777597402597403e-03*w6*Jinv00-1.3149350649350649e-03*Jinv20*w56-6.5746753246753243e-04*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11-3.8352272727272725e-04*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-8.7662337662337668e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+4.2735389610389614e-03*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+4.2735389610389614e-03*w6*Jinv01+3.1777597402597403e-03*w7*Jinv10-3.8352272727272725e-04*w47*Jinv21+1.5888798701298701e-03*Jinv20*w38-1.3149350649350649e-03*w52*Jinv11-2.5202922077922079e-03*Jinv20*w53-1.6436688311688311e-03*w22*Jinv10-7.6704545454545450e-04*w22*Jinv11-7.6704545454545450e-04*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51+2.1367694805194807e-03*w36*Jinv01-2.5202922077922079e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-7.6704545454545450e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-6.5746753246753243e-04*w24*Jinv00-5.4788961038961042e-05*w49*Jinv11-7.6704545454545450e-04*Jinv21*w14-7.6704545454545450e-04*w10*Jinv10+6.9034090909090911e-03*Jinv20*w44-7.6704545454545450e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+9.6793831168831166e-04*w57*Jinv00+9.6793831168831166e-04*Jinv20*w59+1.0592532467532468e-03*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-7.6704545454545450e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-6.5746753246753243e-04*w25*Jinv10+1.0592532467532468e-03*Jinv20*w2+1.3806818181818182e-02*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+1.3806818181818182e-02*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+4.7666396103896100e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-2.7394480519480520e-03*w5*Jinv20-3.8352272727272725e-04*Jinv01*w30-2.1367694805194807e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+6.9034090909090911e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20-5.4788961038961042e-05*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[16][20]=(0.0)*D;
  A[16][21]=(0.0)*D;
  A[16][22]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+7.6704545454545450e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+7.6704545454545450e-04*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20-8.7662337662337668e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54+6.9399350649350649e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+5.6615259740259744e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+6.9399350649350649e-04*w58*Jinv11+6.9399350649350649e-04*w57*Jinv01-1.0957792207792207e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00-3.2873376623376621e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-1.0957792207792207e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-6.0267857142857139e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+7.6704545454545450e-04*w4*Jinv10-8.7662337662337668e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-4.3831168831168834e-04*w32*Jinv21-6.0267857142857139e-04*Jinv21*w20+7.6704545454545450e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35-4.3831168831168834e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-8.7662337662337668e-04*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12-6.0267857142857139e-04*Jinv01*w18-3.2873376623376621e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01+7.6704545454545450e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+7.6704545454545450e-04*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+7.6704545454545450e-04*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-3.2873376623376621e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.2053571428571428e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22-1.2601461038961039e-03*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+4.9310064935064932e-04*Jinv21*w23-1.2053571428571428e-03*Jinv01*w54-1.2053571428571428e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.6615259740259744e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+7.6704545454545450e-04*w5*Jinv20-4.3831168831168834e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-1.0957792207792207e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-1.2601461038961039e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-1.2601461038961039e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.6615259740259744e-04*w10*Jinv11)*D;
  A[16][23]=(0.0)*D;
  A[16][24]=(0.0)*D;
  A[16][25]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-7.6704545454545450e-04*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+6.9034090909090911e-03*Jinv21*w44-7.6704545454545450e-04*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-1.3149350649350649e-03*Jinv00*w54+9.6793831168831166e-04*Jinv21*w59+1.0592532467532468e-03*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+6.9034090909090911e-03*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.3806818181818182e-02*w16*Jinv11+1.3806818181818182e-02*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45+2.7394480519480518e-04*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00+9.6793831168831166e-04*w58*Jinv11+9.6793831168831166e-04*w57*Jinv01+4.7666396103896100e-03*Jinv21*w26+1.0592532467532468e-03*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+6.9034090909090911e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+1.0592532467532468e-03*Jinv11*w1-2.7394480519480520e-03*w13*Jinv11+1.3806818181818182e-02*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-3.8352272727272725e-04*w31*Jinv10+4.2735389610389614e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-2.1367694805194807e-03*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+4.7666396103896100e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11-6.5746753246753243e-04*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-7.6704545454545450e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-6.5746753246753243e-04*Jinv21*w20-7.6704545454545450e-04*w6*Jinv00-1.3149350649350649e-03*Jinv20*w56+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.5888798701298701e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-5.4788961038961042e-05*w49*Jinv10-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-1.6436688311688311e-03*w6*Jinv01-7.6704545454545450e-04*w7*Jinv10+1.5888798701298701e-03*w47*Jinv21-3.8352272727272725e-04*Jinv20*w38-1.3149350649350649e-03*w52*Jinv11-5.4788961038961042e-05*Jinv20*w53+4.2735389610389614e-03*w22*Jinv10+3.1777597402597403e-03*w22*Jinv11+3.1777597402597403e-03*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51-8.2183441558441553e-04*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-1.6436688311688311e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-2.5202922077922079e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-8.7662337662337668e-04*w49*Jinv11-2.7394480519480520e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-2.7394480519480520e-03*Jinv01*w12-1.6436688311688311e-03*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+3.1777597402597403e-03*Jinv21*w23-2.5202922077922079e-03*Jinv01*w54-2.5202922077922079e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00+4.3831168831168834e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-7.6704545454545450e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+4.7666396103896100e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-8.7662337662337668e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-8.7662337662337668e-04*w48*Jinv01+4.3831168831168834e-04*w10*Jinv11)*D;
  A[16][26]=(0.0)*D;
  A[16][27]=(0.0)*D;
  A[16][28]=(-1.8262987012987013e-03*Jinv21*w44+4.3831168831168834e-04*Jinv01*w3-1.9480519480519481e-04*Jinv21*w59-4.2613636363636367e-05*Jinv21*w2-1.8262987012987013e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-3.6525974025974025e-03*w16*Jinv11-3.6525974025974025e-03*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45-1.9480519480519481e-04*w58*Jinv11-1.9480519480519481e-04*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-1.8262987012987013e-03*w43*Jinv11-1.8262987012987013e-03*Jinv21*w41-4.2613636363636367e-05*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-3.6525974025974025e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.8262987012987013e-03*Jinv01*w39+2.1915584415584417e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11+3.4699675324675324e-04*Jinv21*w53+4.3831168831168834e-04*w4*Jinv11+2.1915584415584417e-04*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+3.2873376623376621e-04*w46*Jinv11+4.3831168831168834e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33-3.2873376623376621e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+4.3831168831168834e-04*w6*Jinv01+3.2873376623376621e-04*w47*Jinv21+3.4699675324675324e-04*w52*Jinv11+6.5746753246753243e-04*w22*Jinv11+6.5746753246753243e-04*w21*Jinv01+3.4699675324675324e-04*Jinv01*w51+2.1915584415584417e-04*w36*Jinv01+4.3831168831168834e-04*w7*Jinv11+2.9220779220779221e-04*Jinv21*w56+3.4699675324675324e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-3.2873376623376621e-04*Jinv01*w12+4.3831168831168834e-04*Jinv21*w8+6.5746753246753243e-04*Jinv21*w23+2.9220779220779221e-04*Jinv01*w54+2.9220779220779221e-04*w55*Jinv11-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*w40*Jinv11+2.1915584415584417e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11+3.4699675324675324e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+3.4699675324675324e-04*w48*Jinv01-4.2613636363636367e-05*w10*Jinv11)*D;
  A[16][29]=(0.0)*D;
  A[16][30]=(0.0)*D;
  A[16][31]=(-1.5888798701298701e-03*Jinv00*w12-5.2962662337662338e-04*w28*Jinv10-3.6160714285714286e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-3.6160714285714286e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3+1.3697240259740260e-03*Jinv20*w23+3.2873376623376621e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-1.6436688311688311e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+6.5746753246753243e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+1.3697240259740260e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-3.6160714285714286e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-5.2962662337662338e-04*w28*Jinv11+1.3697240259740260e-03*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10+2.6846590909090908e-03*Jinv02*w3+1.3697240259740260e-03*w21*Jinv00+5.4788961038961036e-04*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39-1.6436688311688311e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+3.2873376623376621e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-4.6022727272727274e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-3.6160714285714286e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-4.6022727272727274e-03*w41*Jinv20+2.6846590909090908e-03*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+3.2873376623376621e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-3.6160714285714286e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-5.2962662337662338e-04*w11*Jinv20+6.5746753246753243e-04*w52*Jinv12-4.6022727272727274e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+6.5746753246753243e-04*Jinv02*w54+2.6846590909090908e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02+1.3697240259740260e-03*w49*Jinv10-5.2962662337662338e-04*Jinv21*w29-4.6022727272727274e-03*Jinv01*w33-4.6022727272727274e-03*w39*Jinv00+2.6846590909090908e-03*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+1.3697240259740260e-03*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+5.4788961038961036e-04*w31*Jinv12+3.2873376623376621e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8+1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+5.4788961038961036e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10+1.3697240259740260e-03*w7*Jinv11-5.2962662337662338e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+2.6846590909090908e-03*Jinv22*w5+3.2873376623376621e-04*w47*Jinv22+1.3697240259740260e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-5.2962662337662338e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+1.3697240259740260e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-3.6160714285714286e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-4.6022727272727274e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29+2.6846590909090908e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+3.2873376623376621e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+6.5746753246753243e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12+1.3697240259740260e-03*Jinv21*w50-5.2962662337662338e-04*Jinv01*w27+1.3697240259740260e-03*w48*Jinv01-1.6436688311688311e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[16][32]=(0.0)*D;
  A[16][33]=(0.0)*D;
  A[16][34]=(2.3011363636363637e-03*Jinv00*w12+3.9448051948051946e-03*w40*Jinv12-2.9586038961038961e-03*Jinv22*w53-2.9586038961038961e-03*Jinv20*w8+1.6436688311688311e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10+3.9448051948051946e-03*Jinv02*w39-1.1834415584415585e-02*Jinv21*w44+2.9586038961038961e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+1.9724025974025973e-03*Jinv01*w3-6.5746753246753243e-04*Jinv20*w23-1.1323051948051949e-03*Jinv21*w59-1.9724025974025973e-03*w25*Jinv12-6.9399350649350649e-04*w0*Jinv02-1.1323051948051949e-03*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10-1.1834415584415585e-02*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-1.9724025974025973e-03*Jinv12*w55+2.3668831168831169e-02*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-6.5746753246753243e-04*w48*Jinv00-1.9724025974025973e-03*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35-1.1323051948051949e-03*w58*Jinv11-1.1323051948051949e-03*w57*Jinv01+1.9724025974025973e-03*Jinv21*w26-1.1323051948051949e-03*w0*Jinv01-9.1314935064935063e-04*w1*Jinv10+9.8620129870129864e-04*Jinv22*w20-1.1834415584415585e-02*w43*Jinv11-7.8896103896103891e-03*Jinv21*w41+2.1915584415584417e-04*w11*Jinv22-1.1323051948051949e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-1.1323051948051949e-03*w28*Jinv11-6.5746753246753243e-04*Jinv20*w50+2.3668831168831169e-02*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3-6.5746753246753243e-04*w21*Jinv00+2.3011363636363637e-03*w32*Jinv22-7.8896103896103891e-03*Jinv01*w39-6.9399350649350649e-04*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00-9.8620129870129864e-04*Jinv21*w38+1.9724025974025973e-03*Jinv01*w24+1.6436688311688311e-03*Jinv22*w59-1.1834415584415585e-02*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11+1.9724025974025973e-03*Jinv21*w53+6.5746753246753243e-04*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+2.9586038961038961e-03*w4*Jinv10+1.9724025974025973e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+3.2873376623376621e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+1.3149350649350649e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-2.9586038961038961e-03*w6*Jinv00+3.9448051948051946e-03*Jinv22*w41+6.5746753246753243e-04*w26*Jinv20+9.1314935064935063e-04*w11*Jinv20-2.9586038961038961e-03*w52*Jinv12-1.1834415584415585e-02*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11-1.9724025974025973e-03*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+1.9724025974025973e-03*w5*Jinv21-1.9724025974025973e-03*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02-6.5746753246753243e-04*w49*Jinv10-1.1323051948051949e-03*Jinv21*w29-1.1834415584415585e-02*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+3.2873376623376621e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-9.8620129870129864e-04*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-9.8620129870129864e-04*w6*Jinv01-2.9586038961038961e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38-2.6298701298701297e-03*Jinv22*w23+1.9724025974025973e-03*w52*Jinv11+2.3011363636363637e-03*w31*Jinv12+6.5746753246753243e-04*Jinv20*w53+1.9724025974025973e-03*Jinv22*w8-6.5746753246753243e-04*w22*Jinv10-3.6160714285714286e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11+1.9724025974025973e-03*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01+1.9724025974025973e-03*Jinv01*w51+2.3011363636363637e-03*Jinv02*w30-3.6160714285714286e-03*Jinv12*w49-9.8620129870129864e-04*w36*Jinv01+6.5746753246753243e-04*w52*Jinv10-9.8620129870129864e-04*w7*Jinv11+9.1314935064935063e-04*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+1.3149350649350649e-03*Jinv21*w56+6.5746753246753243e-04*w24*Jinv00+1.1323051948051949e-03*Jinv02*w27+1.9724025974025973e-03*w6*Jinv02+9.8620129870129864e-04*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5+1.3149350649350649e-03*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+9.1314935064935063e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+1.6436688311688311e-03*w58*Jinv12+9.8620129870129864e-04*Jinv02*w18+1.3149350649350649e-03*Jinv01*w12-9.8620129870129864e-04*Jinv21*w8+5.9172077922077923e-03*Jinv02*w33-9.1314935064935063e-04*w0*Jinv00+1.3149350649350649e-03*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11+6.5746753246753243e-04*w25*Jinv10-9.1314935064935063e-04*Jinv20*w2-3.6160714285714286e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+2.3011363636363637e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20+1.1323051948051949e-03*Jinv22*w29+3.2873376623376621e-03*Jinv22*w14-7.8896103896103891e-03*w40*Jinv11+2.9586038961038961e-03*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-2.9586038961038961e-03*Jinv02*w51-1.9724025974025973e-03*Jinv22*w56+1.1323051948051949e-03*Jinv12*w28+1.9724025974025973e-03*w25*Jinv11-3.9448051948051946e-03*w42*Jinv00+2.3668831168831169e-02*w16*Jinv12+1.3149350649350649e-03*Jinv21*w50-1.1323051948051949e-03*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01-6.9399350649350649e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[16][35]=(0.0)*D;
  A[16][36]=(0.0)*D;
  A[16][37]=(-7.1225649350649346e-04*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53+1.5888798701298701e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-1.3697240259740260e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+2.1367694805194807e-03*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+4.6022727272727274e-03*w43*Jinv10-5.4788961038961042e-05*Jinv12*w55+5.2962662337662338e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+5.2962662337662338e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20-7.1225649350649346e-04*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17+4.6022727272727274e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22+4.9310064935064932e-04*Jinv22*w2+1.5888798701298701e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00-3.8352272727272725e-04*Jinv02*w45+2.7394480519480518e-04*Jinv22*w59-1.3697240259740260e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10+6.5746753246753243e-04*w41*Jinv20-7.6704545454545450e-04*Jinv12*w13+1.5888798701298701e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10+1.5888798701298701e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-1.3697240259740260e-03*w26*Jinv20-3.8352272727272725e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-5.4788961038961042e-05*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10+6.5746753246753243e-04*w39*Jinv00-7.6704545454545450e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+1.5888798701298701e-03*w7*Jinv10+1.5888798701298701e-03*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12-1.3697240259740260e-03*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+4.2735389610389614e-03*Jinv12*w7-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.3697240259740260e-03*w52*Jinv10-3.8352272727272725e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-1.3697240259740260e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22-3.8352272727272725e-04*w10*Jinv10+4.6022727272727274e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33+5.2962662337662338e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-1.3697240259740260e-03*w25*Jinv10+5.2962662337662338e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14+2.1367694805194807e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20-1.3149350649350649e-03*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10+4.6022727272727274e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+4.9310064935064932e-04*Jinv12*w1)*D;
  A[16][38]=(0.0)*D;
  A[16][39]=(0.0)*D;
  A[16][40]=(1.9724025974025973e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12-1.9724025974025973e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+1.6436688311688311e-03*w57*Jinv02+1.9724025974025973e-03*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3-9.8620129870129864e-04*Jinv20*w23+1.9724025974025973e-03*Jinv00*w54+9.8620129870129864e-04*w25*Jinv12-6.9399350649350649e-04*w0*Jinv02-9.1314935064935063e-04*Jinv21*w2-1.1834415584415585e-02*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45-2.9586038961038961e-03*Jinv12*w55-1.1323051948051949e-03*w58*Jinv10+2.3668831168831169e-02*w15*Jinv02+1.1323051948051949e-03*w10*Jinv12+1.3149350649350649e-03*w48*Jinv00+9.8620129870129864e-04*Jinv02*w24+3.9448051948051946e-03*Jinv22*w35-2.3011363636363637e-03*Jinv21*w26-9.1314935064935063e-04*w0*Jinv01-1.1323051948051949e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10-1.9724025974025973e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11-9.8620129870129864e-04*w45*Jinv00-2.1915584415584417e-04*Jinv00*w27+1.1323051948051949e-03*w11*Jinv22-9.1314935064935063e-04*Jinv11*w1+2.9586038961038961e-03*w13*Jinv11+9.1314935064935063e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-9.8620129870129864e-04*w46*Jinv10+2.3668831168831169e-02*Jinv22*w17-7.8896103896103891e-03*Jinv00*w33+1.9724025974025973e-03*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10+3.2873376623376621e-03*Jinv02*w3-9.8620129870129864e-04*w21*Jinv00+2.3011363636363637e-03*w32*Jinv22-6.9399350649350649e-04*Jinv22*w2+1.9724025974025973e-03*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00-2.3011363636363637e-03*Jinv01*w24+1.6436688311688311e-03*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+6.5746753246753243e-04*w19*Jinv11+1.3149350649350649e-03*w51*Jinv00+3.9448051948051946e-03*Jinv12*w34+1.3149350649350649e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11-1.1834415584415585e-02*w41*Jinv20+4.9310064935064934e-03*Jinv12*w13+1.9724025974025973e-03*w19*Jinv10-7.8896103896103891e-03*w34*Jinv10+2.3011363636363637e-03*w32*Jinv21+6.5746753246753243e-04*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+1.9724025974025973e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41-9.8620129870129864e-04*w26*Jinv20-1.1323051948051949e-03*w11*Jinv20-1.9724025974025973e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11+9.8620129870129864e-04*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+1.9724025974025973e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21-2.9586038961038961e-03*Jinv02*w54+3.2873376623376621e-03*w4*Jinv12+1.9724025974025973e-03*w21*Jinv02+1.3149350649350649e-03*w49*Jinv10+9.1314935064935063e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33-1.1834415584415585e-02*w39*Jinv00+4.9310064935064934e-03*Jinv02*w12+6.5746753246753243e-04*Jinv01*w18+1.1323051948051949e-03*Jinv02*w9-6.5746753246753243e-04*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+1.9724025974025973e-03*Jinv22*w23+2.3011363636363637e-03*w31*Jinv12+1.3149350649350649e-03*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8-9.8620129870129864e-04*w22*Jinv10-3.6160714285714286e-03*Jinv22*w50-2.9586038961038961e-03*w22*Jinv11-2.6298701298701297e-03*Jinv12*w7-2.9586038961038961e-03*w21*Jinv01+2.3011363636363637e-03*Jinv02*w30-3.6160714285714286e-03*Jinv12*w49+1.3149350649350649e-03*w52*Jinv10-6.5746753246753243e-04*w7*Jinv11-1.1323051948051949e-03*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+6.5746753246753243e-04*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-1.9724025974025973e-03*w19*Jinv12+3.2873376623376621e-03*Jinv22*w5-6.5746753246753243e-04*w49*Jinv11+2.9586038961038961e-03*Jinv21*w14-1.1323051948051949e-03*w10*Jinv10-1.1834415584415585e-02*Jinv20*w44+1.6436688311688311e-03*w58*Jinv12-1.9724025974025973e-03*Jinv02*w18+2.9586038961038961e-03*Jinv01*w12-6.5746753246753243e-04*Jinv21*w8-1.1323051948051949e-03*w57*Jinv00-1.1323051948051949e-03*Jinv20*w59+3.9448051948051946e-03*Jinv02*w33-1.1323051948051949e-03*w0*Jinv00-7.8896103896103891e-03*w35*Jinv20-2.9586038961038961e-03*Jinv21*w23+6.5746753246753243e-04*Jinv01*w54+6.5746753246753243e-04*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-1.1323051948051949e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-3.6160714285714286e-03*w48*Jinv02-1.1834415584415585e-02*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-1.1834415584415585e-02*w40*Jinv10+1.9724025974025973e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14+1.3149350649350649e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30-1.9724025974025973e-03*Jinv02*w51-2.9586038961038961e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-2.3011363636363637e-03*w25*Jinv11+1.9724025974025973e-03*w55*Jinv10-1.1834415584415585e-02*w42*Jinv00-9.8620129870129864e-04*w47*Jinv20+2.3668831168831169e-02*w16*Jinv12-6.5746753246753243e-04*Jinv21*w50+9.1314935064935063e-04*Jinv01*w27-6.5746753246753243e-04*w48*Jinv01-6.9399350649350649e-04*Jinv12*w1)*D;
  A[16][41]=(0.0)*D;
  A[16][42]=(0.0)*D;
  A[16][43]=(-1.9724025974025973e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+2.7759740259740259e-03*w57*Jinv02-1.9724025974025973e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+1.1834415584415585e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-1.9724025974025973e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23+9.8620129870129864e-04*Jinv22*w38-1.9724025974025973e-03*Jinv00*w54+1.1323051948051949e-03*Jinv21*w59+2.9586038961038961e-03*w25*Jinv12+2.2646103896103898e-03*w0*Jinv02+1.1323051948051949e-03*Jinv21*w2+1.1834415584415585e-02*w43*Jinv10+1.1834415584415585e-02*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55+1.1323051948051949e-03*w58*Jinv10+3.5503246753246752e-02*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00+2.9586038961038961e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+1.1323051948051949e-03*w58*Jinv11+1.1323051948051949e-03*w57*Jinv01-1.9724025974025973e-03*Jinv21*w26+1.1323051948051949e-03*w0*Jinv01+1.1323051948051949e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10+2.9586038961038961e-03*Jinv22*w20+1.1834415584415585e-02*w43*Jinv11+9.8620129870129864e-04*w45*Jinv00+7.8896103896103891e-03*Jinv21*w41+1.7751623376623376e-02*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+1.1323051948051949e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+1.1323051948051949e-03*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+9.8620129870129864e-04*w46*Jinv10+3.5503246753246752e-02*Jinv22*w17+7.8896103896103891e-03*Jinv00*w33+1.9724025974025973e-03*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-3.9448051948051946e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+7.8896103896103891e-03*Jinv01*w39+2.2646103896103898e-03*Jinv22*w2-1.9724025974025973e-03*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00+9.8620129870129864e-04*Jinv21*w38+9.8620129870129864e-04*Jinv02*w45-1.9724025974025973e-03*Jinv01*w24+2.7759740259740259e-03*Jinv22*w59+1.1834415584415585e-02*w34*Jinv11+9.8620129870129864e-04*w19*Jinv11-1.9724025974025973e-03*Jinv21*w53-1.3149350649350649e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+1.7751623376623376e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-1.9724025974025973e-03*w4*Jinv11+1.1834415584415585e-02*w41*Jinv20-3.9448051948051946e-03*Jinv12*w13-1.9724025974025973e-03*w19*Jinv10+7.8896103896103891e-03*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21+9.8620129870129864e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+9.8620129870129864e-04*w37*Jinv12-1.9724025974025973e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+9.8620129870129864e-04*w26*Jinv20+1.1323051948051949e-03*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12+1.1834415584415585e-02*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+2.9586038961038961e-03*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-1.9724025974025973e-03*Jinv20*w14-1.9724025974025973e-03*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54-3.9448051948051946e-03*w4*Jinv12+1.9724025974025973e-03*w21*Jinv02-1.3149350649350649e-03*w49*Jinv10+1.1323051948051949e-03*Jinv21*w29+1.1834415584415585e-02*Jinv01*w33+1.1834415584415585e-02*w39*Jinv00-3.9448051948051946e-03*Jinv02*w12+9.8620129870129864e-04*Jinv01*w18+9.8620129870129864e-04*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-1.3149350649350649e-03*w47*Jinv21-1.3149350649350649e-03*Jinv20*w38+1.9724025974025973e-03*Jinv22*w23-1.9724025974025973e-03*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+9.8620129870129864e-04*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53+1.9724025974025973e-03*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10-3.2873376623376621e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+1.9724025974025973e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01-1.9724025974025973e-03*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-3.2873376623376621e-03*Jinv12*w49+9.8620129870129864e-04*w36*Jinv01-1.3149350649350649e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+1.1323051948051949e-03*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10-1.3149350649350649e-03*Jinv21*w56+9.8620129870129864e-04*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27+1.9724025974025973e-03*w6*Jinv02+2.9586038961038961e-03*w19*Jinv12-3.9448051948051946e-03*Jinv22*w5+9.8620129870129864e-04*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.1323051948051949e-03*w10*Jinv10+1.1834415584415585e-02*Jinv20*w44+2.7759740259740259e-03*w58*Jinv12+2.9586038961038961e-03*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8+1.1323051948051949e-03*w57*Jinv00+1.1323051948051949e-03*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+1.1323051948051949e-03*w0*Jinv00+7.8896103896103891e-03*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11+9.8620129870129864e-04*w25*Jinv10+1.1323051948051949e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-3.2873376623376621e-03*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+1.1834415584415585e-02*w40*Jinv10-1.9724025974025973e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29-3.9448051948051946e-03*Jinv22*w14+7.8896103896103891e-03*w40*Jinv11+9.8620129870129864e-04*Jinv02*w36+1.7751623376623376e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-1.9724025974025973e-03*w25*Jinv11-1.9724025974025973e-03*w55*Jinv10+1.1834415584415585e-02*w42*Jinv00+9.8620129870129864e-04*w47*Jinv20+3.5503246753246752e-02*w16*Jinv12-1.3149350649350649e-03*Jinv21*w50+1.1323051948051949e-03*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+2.2646103896103898e-03*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[16][44]=(0.0)*D;
  A[16][45]=(0.0)*D;
  A[16][46]=(9.8620129870129864e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.7394480519480518e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39+4.6022727272727274e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-3.8352272727272725e-04*Jinv22*w38+5.2962662337662338e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+5.2962662337662338e-04*Jinv21*w2+4.6022727272727274e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+1.9724025974025973e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+5.2962662337662338e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+4.6022727272727274e-03*w43*Jinv11+4.6022727272727274e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02+4.9310064935064932e-04*w11*Jinv22+5.2962662337662338e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+1.9724025974025973e-03*Jinv22*w17+4.2735389610389614e-03*w22*Jinv12-7.6704545454545450e-04*Jinv02*w3-3.8352272727272725e-04*w32*Jinv22+4.6022727272727274e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+1.5888798701298701e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-1.3697240259740260e-03*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-7.1225649350649346e-04*w32*Jinv21-1.3697240259740260e-03*Jinv21*w20-3.8352272727272725e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+1.5888798701298701e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-3.8352272727272725e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-1.3697240259740260e-03*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+1.5888798701298701e-03*w47*Jinv21+4.2735389610389614e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-7.6704545454545450e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+1.5888798701298701e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.5888798701298701e-03*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-1.3697240259740260e-03*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14+2.7394480519480518e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+1.5888798701298701e-03*Jinv21*w23-1.3697240259740260e-03*Jinv01*w54-1.3697240259740260e-03*w55*Jinv11-5.4788961038961042e-05*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+4.6022727272727274e-03*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-7.1225649350649346e-04*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.5888798701298701e-03*w25*Jinv11+1.9724025974025973e-03*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[16][47]=(0.0)*D;
  A[16][48]=(0.0)*D;
  A[16][49]=(5.4788961038961042e-05*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+5.4788961038961042e-05*Jinv20*w8-1.8262987012987014e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+3.2873376623376621e-03*w16*Jinv11+3.2873376623376621e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-6.5746753246753243e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-1.0957792207792207e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+3.2873376623376621e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11+3.8352272727272725e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+3.2873376623376621e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.0957792207792207e-03*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10-6.5746753246753243e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10-2.5750811688311690e-03*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00+3.8352272727272725e-04*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-1.8262987012987014e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-2.5750811688311690e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+3.8352272727272725e-04*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+1.3149350649350649e-03*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-2.5750811688311690e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-1.0957792207792207e-03*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-2.5750811688311690e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+5.4788961038961036e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30+5.4788961038961036e-04*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-2.5750811688311690e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22-1.0957792207792207e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44-1.8262987012987014e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+3.2873376623376621e-03*w15*Jinv00+5.4788961038961036e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9+3.2873376623376621e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-2.5750811688311690e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+3.8352272727272725e-04*Jinv01*w30-3.6525974025974026e-05*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20-6.5746753246753243e-03*w16*Jinv12-1.0957792207792207e-03*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27-1.0957792207792207e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[16][50]=(0.0)*D;
  A[16][51]=(0.0)*D;
  A[16][52]=(-5.4788961038961042e-05*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12+1.6436688311688311e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-6.7573051948051951e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39-5.4788961038961042e-05*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-8.2183441558441553e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-7.3051948051948052e-05*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10+1.0957792207792207e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129868e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35-3.6525974025974026e-05*w1*Jinv10-3.2873376623376621e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20-3.8352272727272725e-04*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+1.0957792207792207e-03*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10-9.8620129870129868e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-3.8352272727272725e-04*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22-7.3051948051948052e-05*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45-6.7573051948051951e-04*Jinv22*w59+1.0957792207792207e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-8.2183441558441553e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20+1.6436688311688311e-03*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26-5.4788961038961042e-05*Jinv20*w14+1.0957792207792207e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-1.3149350649350649e-03*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9-5.4788961038961042e-05*w7*Jinv10-3.8352272727272725e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+1.0957792207792207e-03*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-2.6298701298701297e-03*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49+1.0957792207792207e-03*w52*Jinv10-3.6525974025974026e-05*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-5.4788961038961042e-05*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22-3.6525974025974026e-05*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-6.7573051948051951e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-3.6525974025974026e-05*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-3.2873376623376621e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02-3.2873376623376621e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-8.2183441558441553e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+1.6436688311688311e-03*Jinv02*w51+1.0957792207792207e-03*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28+1.0957792207792207e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-3.8352272727272725e-04*w47*Jinv20-9.8620129870129868e-03*w16*Jinv12-7.3051948051948052e-05*Jinv12*w1)*D;
  A[16][53]=(0.0)*D;
  A[16][54]=(0.0)*D;
  A[16][55]=(9.8620129870129864e-04*w40*Jinv12+1.0957792207792207e-03*Jinv22*w53-6.7573051948051951e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38-4.9310064935064932e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-7.3051948051948052e-05*w0*Jinv02-3.6525974025974026e-05*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-3.2873376623376621e-03*w16*Jinv11-3.2873376623376621e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+1.6436688311688311e-03*Jinv12*w55-9.8620129870129868e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12-2.6298701298701297e-03*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-7.3051948051948052e-05*w11*Jinv22-3.6525974025974026e-05*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-3.2873376623376621e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-9.8620129870129868e-03*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-7.3051948051948052e-05*Jinv22*w2-3.8352272727272725e-04*Jinv21*w38-8.2183441558441553e-04*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-6.7573051948051951e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53+6.5746753246753243e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-5.4788961038961042e-05*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-3.8352272727272725e-04*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41+1.0957792207792207e-03*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-3.8352272727272725e-04*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+1.6436688311688311e-03*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-3.8352272727272725e-04*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-3.8352272727272725e-04*w47*Jinv21-2.6298701298701297e-03*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12-8.2183441558441553e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11+1.0957792207792207e-03*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5-8.2183441558441553e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-6.7573051948051951e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+6.5746753246753243e-04*Jinv02*w33-5.4788961038961042e-05*Jinv21*w23+1.0957792207792207e-03*Jinv01*w54+1.0957792207792207e-03*w55*Jinv11+1.0957792207792207e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*Jinv01*w30+1.0957792207792207e-03*Jinv02*w51+1.6436688311688311e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11-9.8620129870129868e-03*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01-7.3051948051948052e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[16][56]=(0.0)*D;
  A[16][57]=(0.0)*D;
  A[16][58]=(-5.1136363636363640e-04*w40*Jinv12-3.1047077922077924e-04*Jinv22*w53+2.4350649350649351e-04*w57*Jinv02-5.1136363636363640e-04*Jinv02*w39+2.1915584415584417e-04*Jinv22*w38+5.4788961038961036e-04*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02-3.1047077922077924e-04*Jinv12*w55+5.5519480519480519e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12+5.4788961038961036e-04*Jinv02*w24-5.1136363636363640e-04*Jinv22*w35+5.4788961038961036e-04*Jinv22*w20-5.1136363636363640e-04*w42*Jinv02+3.0438311688311688e-05*w11*Jinv22+5.5519480519480519e-03*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+2.1915584415584417e-04*w32*Jinv22+3.0438311688311688e-05*Jinv22*w2+2.1915584415584417e-04*Jinv02*w45+2.4350649350649351e-04*Jinv22*w59-5.1136363636363640e-04*Jinv12*w34-5.1136363636363640e-04*Jinv22*w44+5.4788961038961036e-04*Jinv12*w13+2.1915584415584417e-04*w37*Jinv12-5.1136363636363640e-04*Jinv22*w41-3.1047077922077924e-04*w52*Jinv12+5.4788961038961036e-04*Jinv22*w26-3.1047077922077924e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02+5.4788961038961036e-04*Jinv02*w12+3.0438311688311688e-05*Jinv02*w9+5.4788961038961036e-04*Jinv22*w23+2.1915584415584417e-04*w31*Jinv12+2.1915584415584417e-04*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-3.1047077922077924e-04*Jinv22*w50+5.4788961038961036e-04*Jinv12*w7+2.1915584415584417e-04*Jinv02*w30-3.1047077922077924e-04*Jinv12*w49+3.0438311688311688e-05*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+2.1915584415584417e-04*w47*Jinv22+2.4350649350649351e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-5.1136363636363640e-04*Jinv02*w33-3.1047077922077924e-04*w48*Jinv02+3.0438311688311688e-05*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14+2.1915584415584417e-04*Jinv02*w36-5.1136363636363640e-04*Jinv12*w43-3.1047077922077924e-04*Jinv02*w51-3.1047077922077924e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28+5.5519480519480519e-03*w16*Jinv12+3.0438311688311688e-05*Jinv12*w1)*D;
  A[16][59]=(0.0)*D;
  A[17][0]=(0.0)*D;
  A[17][1]=(0.0)*D;
  A[17][2]=(-6.5746753246753243e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12-3.4699675324675324e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-6.5746753246753243e-04*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-2.1915584415584417e-04*Jinv22*w38-3.4699675324675324e-04*Jinv00*w54+1.9480519480519481e-04*Jinv21*w59-4.3831168831168834e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+3.6525974025974025e-03*w16*Jinv11+3.6525974025974025e-03*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45-3.4699675324675324e-04*Jinv12*w55+1.9480519480519481e-04*w58*Jinv10+3.6525974025974025e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12-2.9220779220779221e-04*w48*Jinv00-4.3831168831168834e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35+1.9480519480519481e-04*w58*Jinv11+1.9480519480519481e-04*w57*Jinv01-4.3831168831168834e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+3.6525974025974025e-03*w16*Jinv10-4.3831168831168834e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27+4.2613636363636367e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-6.5746753246753243e-04*w13*Jinv11+3.6525974025974025e-03*w15*Jinv01+4.2613636363636367e-05*w28*Jinv11-2.9220779220779221e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+3.6525974025974025e-03*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-4.3831168831168834e-04*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38-2.1915584415584417e-04*Jinv02*w45-4.3831168831168834e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59+1.8262987012987013e-03*w34*Jinv11-4.3831168831168834e-04*w19*Jinv11-3.4699675324675324e-04*Jinv21*w53-3.4699675324675324e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-6.5746753246753243e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11+1.8262987012987013e-03*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-4.3831168831168834e-04*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10-3.2873376623376621e-04*w32*Jinv21-4.3831168831168834e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00-2.1915584415584417e-04*w37*Jinv12-3.4699675324675324e-04*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-4.3831168831168834e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20-3.4699675324675324e-04*w52*Jinv12+1.8262987012987013e-03*Jinv21*w35-3.2873376623376621e-04*w31*Jinv11-4.3831168831168834e-04*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21-3.4699675324675324e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12+3.2873376623376621e-04*w21*Jinv02-2.9220779220779221e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+1.8262987012987013e-03*Jinv01*w33+1.8262987012987013e-03*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-4.3831168831168834e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-3.4699675324675324e-04*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12-2.1915584415584417e-04*w46*Jinv12-3.4699675324675324e-04*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10-2.9220779220779221e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01-3.4699675324675324e-04*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30-2.9220779220779221e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01-3.4699675324675324e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10-3.4699675324675324e-04*Jinv21*w56-4.3831168831168834e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02-4.3831168831168834e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5-2.1915584415584417e-04*w47*Jinv22-2.9220779220779221e-04*w49*Jinv11-6.5746753246753243e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12-4.3831168831168834e-04*Jinv02*w18-6.5746753246753243e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+1.9480519480519481e-04*w57*Jinv00+1.9480519480519481e-04*Jinv20*w59+1.8262987012987013e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+1.8262987012987013e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-3.4699675324675324e-04*Jinv01*w54-3.4699675324675324e-04*w55*Jinv11-4.3831168831168834e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+3.6525974025974025e-03*w15*Jinv00-2.9220779220779221e-04*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+3.6525974025974025e-03*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+1.8262987012987013e-03*w40*Jinv10-4.3831168831168834e-04*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11-2.1915584415584417e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-6.5746753246753243e-04*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30-3.4699675324675324e-04*Jinv02*w51-3.4699675324675324e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-4.3831168831168834e-04*w25*Jinv11-3.4699675324675324e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20+3.6525974025974025e-03*w16*Jinv12-2.9220779220779221e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-2.9220779220779221e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[17][3]=(0.0)*D;
  A[17][4]=(0.0)*D;
  A[17][5]=(1.0957792207792207e-03*Jinv00*w12-5.6615259740259744e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-9.6793831168831166e-04*w57*Jinv02+1.0957792207792207e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-4.7666396103896100e-03*Jinv01*w3+6.0267857142857139e-04*Jinv20*w23+8.2183441558441553e-04*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-9.6793831168831166e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.3806818181818182e-02*w16*Jinv11-1.3806818181818182e-02*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+8.7662337662337668e-04*Jinv12*w55-6.9399350649350649e-04*w58*Jinv10-1.3806818181818182e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+1.2053571428571428e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-6.9034090909090911e-03*Jinv22*w35-9.6793831168831166e-04*w58*Jinv11-9.6793831168831166e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10+1.6436688311688311e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+3.2873376623376621e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.6615259740259744e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-3.1777597402597403e-03*w13*Jinv11-1.3806818181818182e-02*w15*Jinv01-1.0592532467532468e-03*w28*Jinv11+1.2053571428571428e-03*Jinv20*w50-5.6615259740259744e-04*Jinv20*w29+3.2873376623376621e-04*w46*Jinv10-1.3806818181818182e-02*Jinv22*w17-5.9172077922077923e-03*Jinv00*w33+2.7394480519480520e-03*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10-4.7666396103896100e-03*Jinv02*w3+6.0267857142857139e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+8.7662337662337668e-04*Jinv00*w18+4.3831168831168834e-04*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-9.6793831168831166e-04*Jinv22*w59-6.9034090909090911e-03*w34*Jinv11+1.6436688311688311e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+1.2601461038961039e-03*w51*Jinv00-6.9034090909090911e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-4.7666396103896100e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-3.1777597402597403e-03*Jinv12*w13+8.7662337662337668e-04*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+1.6436688311688311e-03*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00+8.2183441558441553e-04*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12-6.9034090909090911e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+1.0957792207792207e-03*Jinv20*w14-4.7666396103896100e-03*w5*Jinv21+8.7662337662337668e-04*Jinv02*w54-4.7666396103896100e-03*w4*Jinv12+2.7394480519480520e-03*w21*Jinv02+1.2053571428571428e-03*w49*Jinv10-1.0592532467532468e-03*Jinv21*w29-6.9034090909090911e-03*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-3.1777597402597403e-03*Jinv02*w12+1.6436688311688311e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+6.5746753246753243e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+4.3831168831168834e-04*Jinv20*w38+2.7394480519480520e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+1.2601461038961039e-03*Jinv20*w53+6.5746753246753243e-04*Jinv22*w8+6.0267857142857139e-04*w22*Jinv10+2.5202922077922079e-03*Jinv22*w50+2.7394480519480520e-03*w22*Jinv11+6.5746753246753243e-04*Jinv12*w7+2.7394480519480520e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+2.5202922077922079e-03*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+1.2601461038961039e-03*w52*Jinv10+6.5746753246753243e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+4.3831168831168834e-04*w37*Jinv10+8.7662337662337668e-04*Jinv21*w56+1.3697240259740260e-03*w24*Jinv00-1.0592532467532468e-03*Jinv02*w27+6.5746753246753243e-04*w6*Jinv02+1.6436688311688311e-03*w19*Jinv12-4.7666396103896100e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+2.5202922077922079e-03*w49*Jinv11-3.1777597402597403e-03*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-9.6793831168831166e-04*w58*Jinv12+1.6436688311688311e-03*Jinv02*w18-3.1777597402597403e-03*Jinv01*w12+6.5746753246753243e-04*Jinv21*w8-6.9399350649350649e-04*w57*Jinv00-6.9399350649350649e-04*Jinv20*w59-6.9034090909090911e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+2.7394480519480520e-03*Jinv21*w23+8.7662337662337668e-04*Jinv01*w54+8.7662337662337668e-04*w55*Jinv11+1.3697240259740260e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00+2.5202922077922079e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-1.1834415584415585e-02*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10+8.7662337662337668e-04*w20*Jinv20-1.0592532467532468e-03*Jinv22*w29-3.1777597402597403e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+8.2183441558441553e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+8.7662337662337668e-04*Jinv22*w56-1.0592532467532468e-03*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+3.2873376623376621e-04*w47*Jinv20-1.3806818181818182e-02*w16*Jinv12+2.5202922077922079e-03*Jinv21*w50-1.0592532467532468e-03*Jinv01*w27+2.5202922077922079e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[17][6]=(0.0)*D;
  A[17][7]=(0.0)*D;
  A[17][8]=(-8.7662337662337668e-04*Jinv00*w12+5.6615259740259744e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-8.7662337662337668e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00-4.3831168831168834e-04*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55+6.9399350649350649e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-1.2601461038961039e-03*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-3.2873376623376621e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.6615259740259744e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+7.6704545454545450e-04*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-1.2601461038961039e-03*Jinv20*w50+5.6615259740259744e-04*Jinv20*w29-3.2873376623376621e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33+7.6704545454545450e-04*w22*Jinv12-4.3831168831168834e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-1.0957792207792207e-03*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53-1.2053571428571428e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10+2.1367694805194807e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-1.0957792207792207e-03*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10+3.8352272727272725e-04*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-6.0267857142857139e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11-8.7662337662337668e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+7.6704545454545450e-04*w21*Jinv02-1.2601461038961039e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+7.6704545454545450e-04*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-1.2053571428571428e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+7.6704545454545450e-04*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+7.6704545454545450e-04*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-1.2053571428571428e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+5.4788961038961042e-05*Jinv21*w56-6.0267857142857139e-04*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+7.6704545454545450e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18+7.6704545454545450e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8+6.9399350649350649e-04*w57*Jinv00+6.9399350649350649e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+7.6704545454545450e-04*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-6.0267857142857139e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-4.3831168831168834e-04*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10-1.0957792207792207e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+3.8352272727272725e-04*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11-8.2183441558441553e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-3.2873376623376621e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[17][9]=(0.0)*D;
  A[17][10]=(0.0)*D;
  A[17][11]=(4.3831168831168834e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10+6.5746753246753243e-04*Jinv20*w8+4.3831168831168834e-04*w13*Jinv10-3.2873376623376621e-04*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20+4.3831168831168834e-04*Jinv20*w23+3.4699675324675324e-04*Jinv00*w54-1.8262987012987013e-03*w43*Jinv10-1.9480519480519481e-04*w58*Jinv10+3.4699675324675324e-04*w48*Jinv00-4.2613636363636367e-05*w1*Jinv10-3.6525974025974025e-03*w16*Jinv10+2.1915584415584417e-04*w45*Jinv00-4.2613636363636367e-05*Jinv00*w27+3.4699675324675324e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-1.8262987012987013e-03*Jinv00*w33+2.1915584415584417e-04*w31*Jinv10+4.3831168831168834e-04*w21*Jinv00+6.5746753246753243e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00+2.9220779220779221e-04*w51*Jinv00-3.2873376623376621e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+6.5746753246753243e-04*w6*Jinv00+3.4699675324675324e-04*Jinv20*w56-3.2873376623376621e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+4.3831168831168834e-04*Jinv20*w14+3.4699675324675324e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+6.5746753246753243e-04*w7*Jinv10+3.2873376623376621e-04*Jinv20*w38+2.9220779220779221e-04*Jinv20*w53+4.3831168831168834e-04*w22*Jinv10+2.9220779220779221e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10-3.2873376623376621e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-1.8262987012987013e-03*Jinv20*w44-1.9480519480519481e-04*w57*Jinv00-1.9480519480519481e-04*Jinv20*w59-4.2613636363636367e-05*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20-3.2873376623376621e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-3.6525974025974025e-03*w15*Jinv00-3.6525974025974025e-03*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-3.2873376623376621e-04*w5*Jinv20+3.4699675324675324e-04*w55*Jinv10-1.8262987012987013e-03*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20)*D;
  A[17][12]=(0.0)*D;
  A[17][13]=(0.0)*D;
  A[17][14]=(-4.7666396103896100e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-6.9034090909090911e-03*w40*Jinv12+8.7662337662337668e-04*Jinv22*w53+2.7394480519480520e-03*Jinv20*w8-9.6793831168831166e-04*w57*Jinv02-4.7666396103896100e-03*w13*Jinv10-6.9034090909090911e-03*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-3.1777597402597403e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20+1.0957792207792207e-03*Jinv01*w3+6.5746753246753243e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-6.9399350649350649e-04*Jinv21*w59+1.6436688311688311e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-5.6615259740259744e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+4.3831168831168834e-04*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-9.6793831168831166e-04*w58*Jinv10-1.3806818181818182e-02*w15*Jinv02-1.0592532467532468e-03*w10*Jinv12+2.5202922077922079e-03*w48*Jinv00+1.6436688311688311e-03*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-6.9399350649350649e-04*w58*Jinv11-6.9399350649350649e-04*w57*Jinv01+8.7662337662337668e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-1.3806818181818182e-02*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.0592532467532468e-03*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+2.5202922077922079e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-1.3806818181818182e-02*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+6.5746753246753243e-04*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10-3.1777597402597403e-03*Jinv02*w3+6.5746753246753243e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.2873376623376621e-04*Jinv21*w38+8.2183441558441553e-04*Jinv02*w45+8.7662337662337668e-04*Jinv01*w24-9.6793831168831166e-04*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+8.7662337662337668e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-3.1777597402597403e-03*w4*Jinv10+1.0957792207792207e-03*w4*Jinv11-6.9034090909090911e-03*w41*Jinv20-4.7666396103896100e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+2.7394480519480520e-03*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-6.9034090909090911e-03*Jinv22*w41+1.6436688311688311e-03*w26*Jinv20-1.0592532467532468e-03*w11*Jinv20+8.7662337662337668e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11+1.6436688311688311e-03*Jinv22*w26+4.3831168831168834e-04*w46*Jinv11-4.7666396103896100e-03*Jinv20*w14+1.0957792207792207e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54-3.1777597402597403e-03*w4*Jinv12+6.5746753246753243e-04*w21*Jinv02+2.5202922077922079e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-6.9034090909090911e-03*w39*Jinv00-4.7666396103896100e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.2873376623376621e-04*w37*Jinv11-1.0592532467532468e-03*Jinv02*w9+6.0267857142857139e-04*w6*Jinv01+2.7394480519480520e-03*w7*Jinv10+4.3831168831168834e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+6.5746753246753243e-04*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+8.2183441558441553e-04*w46*Jinv12+8.7662337662337668e-04*Jinv20*w53+2.7394480519480520e-03*Jinv22*w8+6.5746753246753243e-04*w22*Jinv10+2.5202922077922079e-03*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+2.7394480519480520e-03*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+2.5202922077922079e-03*Jinv12*w49+3.2873376623376621e-04*w36*Jinv01+8.7662337662337668e-04*w52*Jinv10+6.0267857142857139e-04*w7*Jinv11-1.0592532467532468e-03*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.2601461038961039e-03*Jinv21*w56+1.6436688311688311e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+2.7394480519480520e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-3.1777597402597403e-03*Jinv22*w5+8.2183441558441553e-04*w47*Jinv22+1.2053571428571428e-03*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14-1.0592532467532468e-03*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-9.6793831168831166e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+6.0267857142857139e-04*Jinv21*w8-9.6793831168831166e-04*w57*Jinv00-9.6793831168831166e-04*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23+1.2601461038961039e-03*Jinv01*w54+1.2601461038961039e-03*w55*Jinv11+1.6436688311688311e-03*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-1.3806818181818182e-02*w15*Jinv00+2.5202922077922079e-03*w48*Jinv02-5.6615259740259744e-04*Jinv01*w9-1.3806818181818182e-02*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-6.9034090909090911e-03*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-4.7666396103896100e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-3.1777597402597403e-03*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+8.7662337662337668e-04*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+8.7662337662337668e-04*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-1.3806818181818182e-02*w16*Jinv12+1.2053571428571428e-03*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+1.2053571428571428e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-5.6615259740259744e-04*w10*Jinv11)*D;
  A[17][15]=(0.0)*D;
  A[17][16]=(0.0)*D;
  A[17][17]=(6.9034090909090911e-03*Jinv00*w12-1.1834415584415585e-02*w40*Jinv12+4.9310064935064934e-03*Jinv22*w53-5.9172077922077923e-03*Jinv20*w8-2.7759740259740259e-03*w57*Jinv02+6.9034090909090911e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-5.9172077922077923e-03*Jinv21*w44+5.9172077922077923e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+6.9034090909090911e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38-2.9586038961038961e-03*w25*Jinv12-2.2646103896103898e-03*w0*Jinv02-1.8262987012987013e-03*Jinv21*w2-5.9172077922077923e-03*w43*Jinv10-5.9172077922077923e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45+4.9310064935064934e-03*Jinv12*w55-3.5503246753246752e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12-1.6436688311688311e-03*w48*Jinv00-2.9586038961038961e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-6.9034090909090911e-03*Jinv21*w26-1.8262987012987013e-03*w0*Jinv01-1.8262987012987013e-03*w1*Jinv10-2.9586038961038961e-03*Jinv22*w20-5.9172077922077923e-03*w43*Jinv11-1.7751623376623376e-02*w42*Jinv02-4.3831168831168834e-04*w11*Jinv22-1.8262987012987013e-03*Jinv11*w1+5.9172077922077923e-03*w13*Jinv11+1.8262987012987013e-03*w28*Jinv11-1.6436688311688311e-03*Jinv20*w50-3.5503246753246752e-02*Jinv22*w17-1.9724025974025973e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+3.9448051948051946e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-2.2646103896103898e-03*Jinv22*w2-6.9034090909090911e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-6.9034090909090911e-03*Jinv01*w24-2.7759740259740259e-03*Jinv22*w59+5.9172077922077923e-03*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11+1.6436688311688311e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-1.7751623376623376e-02*Jinv22*w44+5.9172077922077923e-03*w4*Jinv10+6.9034090909090911e-03*w4*Jinv11+5.9172077922077923e-03*w41*Jinv20+3.9448051948051946e-03*Jinv12*w13-6.9034090909090911e-03*w19*Jinv10+2.3011363636363637e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-5.9172077922077923e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12-1.1834415584415585e-02*Jinv22*w41-9.8620129870129864e-04*w26*Jinv20+1.8262987012987013e-03*w11*Jinv20+4.9310064935064934e-03*w52*Jinv12+5.9172077922077923e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11-2.9586038961038961e-03*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+6.9034090909090911e-03*Jinv20*w14+6.9034090909090911e-03*w5*Jinv21+4.9310064935064934e-03*Jinv02*w54+3.9448051948051946e-03*w4*Jinv12-1.9724025974025973e-03*w21*Jinv02-1.6436688311688311e-03*w49*Jinv10+1.8262987012987013e-03*Jinv21*w29+5.9172077922077923e-03*Jinv01*w33+5.9172077922077923e-03*w39*Jinv00+3.9448051948051946e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-5.9172077922077923e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38-1.9724025974025973e-03*Jinv22*w23+1.3149350649350649e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12+1.6436688311688311e-03*Jinv20*w53-1.9724025974025973e-03*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10+3.2873376623376621e-03*Jinv22*w50-5.9172077922077923e-03*w22*Jinv11-1.9724025974025973e-03*Jinv12*w7-5.9172077922077923e-03*w21*Jinv01+1.3149350649350649e-03*Jinv02*w30+3.2873376623376621e-03*Jinv12*w49+1.6436688311688311e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+1.8262987012987013e-03*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+1.6436688311688311e-03*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27-1.9724025974025973e-03*w6*Jinv02-2.9586038961038961e-03*w19*Jinv12+3.9448051948051946e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22-1.6436688311688311e-03*w49*Jinv11+5.9172077922077923e-03*Jinv21*w14+1.8262987012987013e-03*w10*Jinv10-5.9172077922077923e-03*Jinv20*w44-2.7759740259740259e-03*w58*Jinv12-2.9586038961038961e-03*Jinv02*w18+5.9172077922077923e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8-1.1834415584415585e-02*Jinv02*w33-1.8262987012987013e-03*w0*Jinv00-5.9172077922077923e-03*Jinv21*w23+1.6436688311688311e-03*Jinv01*w54+1.6436688311688311e-03*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-1.8262987012987013e-03*Jinv20*w2+3.2873376623376621e-03*w48*Jinv02+2.3011363636363637e-03*w30*Jinv00+5.9172077922077923e-03*w40*Jinv10-6.9034090909090911e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29+3.9448051948051946e-03*Jinv22*w14-9.8620129870129864e-04*Jinv02*w36-1.7751623376623376e-02*Jinv12*w43+5.9172077922077923e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30+4.9310064935064934e-03*Jinv02*w51+4.9310064935064934e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28-6.9034090909090911e-03*w25*Jinv11-5.9172077922077923e-03*w42*Jinv00-3.5503246753246752e-02*w16*Jinv12-1.6436688311688311e-03*Jinv21*w50+1.8262987012987013e-03*Jinv01*w27-1.6436688311688311e-03*w48*Jinv01-2.2646103896103898e-03*Jinv12*w1)*D;
  A[17][18]=(0.0)*D;
  A[17][19]=(0.0)*D;
  A[17][20]=(-2.1367694805194807e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+3.1777597402597403e-03*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-2.7394480519480520e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-1.6436688311688311e-03*Jinv20*w23-1.3149350649350649e-03*Jinv00*w54+2.7394480519480518e-04*Jinv21*w59+4.9310064935064932e-04*Jinv21*w2+6.9034090909090911e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+9.6793831168831166e-04*w58*Jinv10-8.7662337662337668e-04*w48*Jinv00+2.7394480519480518e-04*w58*Jinv11+2.7394480519480518e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+1.0592532467532468e-03*w1*Jinv10+1.3806818181818182e-02*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+4.3831168831168834e-04*Jinv00*w27+4.9310064935064932e-04*Jinv11*w1-7.6704545454545450e-04*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-8.7662337662337668e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-03*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+4.7666396103896100e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-2.1367694805194807e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-2.5202922077922079e-03*w51*Jinv00-2.7394480519480520e-03*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+4.7666396103896100e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-3.8352272727272725e-04*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+3.1777597402597403e-03*w6*Jinv00-1.3149350649350649e-03*Jinv20*w56-6.5746753246753243e-04*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11-3.8352272727272725e-04*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-8.7662337662337668e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+4.2735389610389614e-03*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+4.2735389610389614e-03*w6*Jinv01+3.1777597402597403e-03*w7*Jinv10-3.8352272727272725e-04*w47*Jinv21+1.5888798701298701e-03*Jinv20*w38-1.3149350649350649e-03*w52*Jinv11-2.5202922077922079e-03*Jinv20*w53-1.6436688311688311e-03*w22*Jinv10-7.6704545454545450e-04*w22*Jinv11-7.6704545454545450e-04*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51+2.1367694805194807e-03*w36*Jinv01-2.5202922077922079e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-7.6704545454545450e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-6.5746753246753243e-04*w24*Jinv00-5.4788961038961042e-05*w49*Jinv11-7.6704545454545450e-04*Jinv21*w14-7.6704545454545450e-04*w10*Jinv10+6.9034090909090911e-03*Jinv20*w44-7.6704545454545450e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+9.6793831168831166e-04*w57*Jinv00+9.6793831168831166e-04*Jinv20*w59+1.0592532467532468e-03*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-7.6704545454545450e-04*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-6.5746753246753243e-04*w25*Jinv10+1.0592532467532468e-03*Jinv20*w2+1.3806818181818182e-02*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+1.3806818181818182e-02*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+4.7666396103896100e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-2.7394480519480520e-03*w5*Jinv20-3.8352272727272725e-04*Jinv01*w30-2.1367694805194807e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+6.9034090909090911e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20-5.4788961038961042e-05*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[17][21]=(0.0)*D;
  A[17][22]=(0.0)*D;
  A[17][23]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+7.6704545454545450e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+7.6704545454545450e-04*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20-8.7662337662337668e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54+6.9399350649350649e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+5.6615259740259744e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+6.9399350649350649e-04*w58*Jinv11+6.9399350649350649e-04*w57*Jinv01-1.0957792207792207e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00-3.2873376623376621e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-1.0957792207792207e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-6.0267857142857139e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+7.6704545454545450e-04*w4*Jinv10-8.7662337662337668e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-4.3831168831168834e-04*w32*Jinv21-6.0267857142857139e-04*Jinv21*w20+7.6704545454545450e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35-4.3831168831168834e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-8.7662337662337668e-04*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12-6.0267857142857139e-04*Jinv01*w18-3.2873376623376621e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01+7.6704545454545450e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+7.6704545454545450e-04*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+7.6704545454545450e-04*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-3.2873376623376621e-04*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.2053571428571428e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22-1.2601461038961039e-03*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.7394480519480518e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+4.9310064935064932e-04*Jinv21*w23-1.2053571428571428e-03*Jinv01*w54-1.2053571428571428e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.6615259740259744e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+7.6704545454545450e-04*w5*Jinv20-4.3831168831168834e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-1.0957792207792207e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-1.2601461038961039e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-1.2601461038961039e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.6615259740259744e-04*w10*Jinv11)*D;
  A[17][24]=(0.0)*D;
  A[17][25]=(0.0)*D;
  A[17][26]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-7.6704545454545450e-04*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+6.9034090909090911e-03*Jinv21*w44-7.6704545454545450e-04*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-1.3149350649350649e-03*Jinv00*w54+9.6793831168831166e-04*Jinv21*w59+1.0592532467532468e-03*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+6.9034090909090911e-03*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.3806818181818182e-02*w16*Jinv11+1.3806818181818182e-02*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45+2.7394480519480518e-04*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00+9.6793831168831166e-04*w58*Jinv11+9.6793831168831166e-04*w57*Jinv01+4.7666396103896100e-03*Jinv21*w26+1.0592532467532468e-03*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+6.9034090909090911e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+1.0592532467532468e-03*Jinv11*w1-2.7394480519480520e-03*w13*Jinv11+1.3806818181818182e-02*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-3.8352272727272725e-04*w31*Jinv10+4.2735389610389614e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-2.1367694805194807e-03*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38+4.7666396103896100e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11-6.5746753246753243e-04*w19*Jinv11-1.3149350649350649e-03*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-7.6704545454545450e-04*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-6.5746753246753243e-04*Jinv21*w20-7.6704545454545450e-04*w6*Jinv00-1.3149350649350649e-03*Jinv20*w56+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.5888798701298701e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-5.4788961038961042e-05*w49*Jinv10-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-1.6436688311688311e-03*w6*Jinv01-7.6704545454545450e-04*w7*Jinv10+1.5888798701298701e-03*w47*Jinv21-3.8352272727272725e-04*Jinv20*w38-1.3149350649350649e-03*w52*Jinv11-5.4788961038961042e-05*Jinv20*w53+4.2735389610389614e-03*w22*Jinv10+3.1777597402597403e-03*w22*Jinv11+3.1777597402597403e-03*w21*Jinv01-1.3149350649350649e-03*Jinv01*w51-8.2183441558441553e-04*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-1.6436688311688311e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-2.5202922077922079e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-8.7662337662337668e-04*w49*Jinv11-2.7394480519480520e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-2.7394480519480520e-03*Jinv01*w12-1.6436688311688311e-03*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+3.1777597402597403e-03*Jinv21*w23-2.5202922077922079e-03*Jinv01*w54-2.5202922077922079e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00+4.3831168831168834e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-7.6704545454545450e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+4.7666396103896100e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-8.7662337662337668e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-8.7662337662337668e-04*w48*Jinv01+4.3831168831168834e-04*w10*Jinv11)*D;
  A[17][27]=(0.0)*D;
  A[17][28]=(0.0)*D;
  A[17][29]=(-1.8262987012987013e-03*Jinv21*w44+4.3831168831168834e-04*Jinv01*w3-1.9480519480519481e-04*Jinv21*w59-4.2613636363636367e-05*Jinv21*w2-1.8262987012987013e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-3.6525974025974025e-03*w16*Jinv11-3.6525974025974025e-03*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45-1.9480519480519481e-04*w58*Jinv11-1.9480519480519481e-04*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-1.8262987012987013e-03*w43*Jinv11-1.8262987012987013e-03*Jinv21*w41-4.2613636363636367e-05*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-3.6525974025974025e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.8262987012987013e-03*Jinv01*w39+2.1915584415584417e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11+3.4699675324675324e-04*Jinv21*w53+4.3831168831168834e-04*w4*Jinv11+2.1915584415584417e-04*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+3.2873376623376621e-04*w46*Jinv11+4.3831168831168834e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33-3.2873376623376621e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+4.3831168831168834e-04*w6*Jinv01+3.2873376623376621e-04*w47*Jinv21+3.4699675324675324e-04*w52*Jinv11+6.5746753246753243e-04*w22*Jinv11+6.5746753246753243e-04*w21*Jinv01+3.4699675324675324e-04*Jinv01*w51+2.1915584415584417e-04*w36*Jinv01+4.3831168831168834e-04*w7*Jinv11+2.9220779220779221e-04*Jinv21*w56+3.4699675324675324e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-3.2873376623376621e-04*Jinv01*w12+4.3831168831168834e-04*Jinv21*w8+6.5746753246753243e-04*Jinv21*w23+2.9220779220779221e-04*Jinv01*w54+2.9220779220779221e-04*w55*Jinv11-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*w40*Jinv11+2.1915584415584417e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11+3.4699675324675324e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+3.4699675324675324e-04*w48*Jinv01-4.2613636363636367e-05*w10*Jinv11)*D;
  A[17][30]=(0.0)*D;
  A[17][31]=(0.0)*D;
  A[17][32]=(-1.5888798701298701e-03*Jinv00*w12-5.2962662337662338e-04*w28*Jinv10-3.6160714285714286e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-3.6160714285714286e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3+1.3697240259740260e-03*Jinv20*w23+3.2873376623376621e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-1.6436688311688311e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+6.5746753246753243e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+1.3697240259740260e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-3.6160714285714286e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-5.2962662337662338e-04*w28*Jinv11+1.3697240259740260e-03*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10+2.6846590909090908e-03*Jinv02*w3+1.3697240259740260e-03*w21*Jinv00+5.4788961038961036e-04*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39-1.6436688311688311e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+3.2873376623376621e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-4.6022727272727274e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-3.6160714285714286e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-4.6022727272727274e-03*w41*Jinv20+2.6846590909090908e-03*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+3.2873376623376621e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-3.6160714285714286e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-5.2962662337662338e-04*w11*Jinv20+6.5746753246753243e-04*w52*Jinv12-4.6022727272727274e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+6.5746753246753243e-04*Jinv02*w54+2.6846590909090908e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02+1.3697240259740260e-03*w49*Jinv10-5.2962662337662338e-04*Jinv21*w29-4.6022727272727274e-03*Jinv01*w33-4.6022727272727274e-03*w39*Jinv00+2.6846590909090908e-03*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+1.3697240259740260e-03*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+5.4788961038961036e-04*w31*Jinv12+3.2873376623376621e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8+1.3697240259740260e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+5.4788961038961036e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10+1.3697240259740260e-03*w7*Jinv11-5.2962662337662338e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+2.6846590909090908e-03*Jinv22*w5+3.2873376623376621e-04*w47*Jinv22+1.3697240259740260e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-5.2962662337662338e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+1.3697240259740260e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-3.6160714285714286e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-4.6022727272727274e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29+2.6846590909090908e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+3.2873376623376621e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+6.5746753246753243e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12+1.3697240259740260e-03*Jinv21*w50-5.2962662337662338e-04*Jinv01*w27+1.3697240259740260e-03*w48*Jinv01-1.6436688311688311e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[17][33]=(0.0)*D;
  A[17][34]=(0.0)*D;
  A[17][35]=(2.3011363636363637e-03*Jinv00*w12+3.9448051948051946e-03*w40*Jinv12-2.9586038961038961e-03*Jinv22*w53-2.9586038961038961e-03*Jinv20*w8+1.6436688311688311e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10+3.9448051948051946e-03*Jinv02*w39-1.1834415584415585e-02*Jinv21*w44+2.9586038961038961e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+1.9724025974025973e-03*Jinv01*w3-6.5746753246753243e-04*Jinv20*w23-1.1323051948051949e-03*Jinv21*w59-1.9724025974025973e-03*w25*Jinv12-6.9399350649350649e-04*w0*Jinv02-1.1323051948051949e-03*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10-1.1834415584415585e-02*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-1.9724025974025973e-03*Jinv12*w55+2.3668831168831169e-02*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-6.5746753246753243e-04*w48*Jinv00-1.9724025974025973e-03*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35-1.1323051948051949e-03*w58*Jinv11-1.1323051948051949e-03*w57*Jinv01+1.9724025974025973e-03*Jinv21*w26-1.1323051948051949e-03*w0*Jinv01-9.1314935064935063e-04*w1*Jinv10+9.8620129870129864e-04*Jinv22*w20-1.1834415584415585e-02*w43*Jinv11-7.8896103896103891e-03*Jinv21*w41+2.1915584415584417e-04*w11*Jinv22-1.1323051948051949e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-1.1323051948051949e-03*w28*Jinv11-6.5746753246753243e-04*Jinv20*w50+2.3668831168831169e-02*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3-6.5746753246753243e-04*w21*Jinv00+2.3011363636363637e-03*w32*Jinv22-7.8896103896103891e-03*Jinv01*w39-6.9399350649350649e-04*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00-9.8620129870129864e-04*Jinv21*w38+1.9724025974025973e-03*Jinv01*w24+1.6436688311688311e-03*Jinv22*w59-1.1834415584415585e-02*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11+1.9724025974025973e-03*Jinv21*w53+6.5746753246753243e-04*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+2.9586038961038961e-03*w4*Jinv10+1.9724025974025973e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+3.2873376623376621e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+1.3149350649350649e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-2.9586038961038961e-03*w6*Jinv00+3.9448051948051946e-03*Jinv22*w41+6.5746753246753243e-04*w26*Jinv20+9.1314935064935063e-04*w11*Jinv20-2.9586038961038961e-03*w52*Jinv12-1.1834415584415585e-02*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11-1.9724025974025973e-03*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+1.9724025974025973e-03*w5*Jinv21-1.9724025974025973e-03*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02-6.5746753246753243e-04*w49*Jinv10-1.1323051948051949e-03*Jinv21*w29-1.1834415584415585e-02*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+3.2873376623376621e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-9.8620129870129864e-04*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-9.8620129870129864e-04*w6*Jinv01-2.9586038961038961e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38-2.6298701298701297e-03*Jinv22*w23+1.9724025974025973e-03*w52*Jinv11+2.3011363636363637e-03*w31*Jinv12+6.5746753246753243e-04*Jinv20*w53+1.9724025974025973e-03*Jinv22*w8-6.5746753246753243e-04*w22*Jinv10-3.6160714285714286e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11+1.9724025974025973e-03*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01+1.9724025974025973e-03*Jinv01*w51+2.3011363636363637e-03*Jinv02*w30-3.6160714285714286e-03*Jinv12*w49-9.8620129870129864e-04*w36*Jinv01+6.5746753246753243e-04*w52*Jinv10-9.8620129870129864e-04*w7*Jinv11+9.1314935064935063e-04*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+1.3149350649350649e-03*Jinv21*w56+6.5746753246753243e-04*w24*Jinv00+1.1323051948051949e-03*Jinv02*w27+1.9724025974025973e-03*w6*Jinv02+9.8620129870129864e-04*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5+1.3149350649350649e-03*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+9.1314935064935063e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+1.6436688311688311e-03*w58*Jinv12+9.8620129870129864e-04*Jinv02*w18+1.3149350649350649e-03*Jinv01*w12-9.8620129870129864e-04*Jinv21*w8+5.9172077922077923e-03*Jinv02*w33-9.1314935064935063e-04*w0*Jinv00+1.3149350649350649e-03*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11+6.5746753246753243e-04*w25*Jinv10-9.1314935064935063e-04*Jinv20*w2-3.6160714285714286e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+2.3011363636363637e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20+1.1323051948051949e-03*Jinv22*w29+3.2873376623376621e-03*Jinv22*w14-7.8896103896103891e-03*w40*Jinv11+2.9586038961038961e-03*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-2.9586038961038961e-03*Jinv02*w51-1.9724025974025973e-03*Jinv22*w56+1.1323051948051949e-03*Jinv12*w28+1.9724025974025973e-03*w25*Jinv11-3.9448051948051946e-03*w42*Jinv00+2.3668831168831169e-02*w16*Jinv12+1.3149350649350649e-03*Jinv21*w50-1.1323051948051949e-03*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01-6.9399350649350649e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[17][36]=(0.0)*D;
  A[17][37]=(0.0)*D;
  A[17][38]=(-7.1225649350649346e-04*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53+1.5888798701298701e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-1.3697240259740260e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+2.1367694805194807e-03*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+4.6022727272727274e-03*w43*Jinv10-5.4788961038961042e-05*Jinv12*w55+5.2962662337662338e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+5.2962662337662338e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20-7.1225649350649346e-04*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17+4.6022727272727274e-03*Jinv00*w33-7.6704545454545450e-04*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22+4.9310064935064932e-04*Jinv22*w2+1.5888798701298701e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00-3.8352272727272725e-04*Jinv02*w45+2.7394480519480518e-04*Jinv22*w59-1.3697240259740260e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10+6.5746753246753243e-04*w41*Jinv20-7.6704545454545450e-04*Jinv12*w13+1.5888798701298701e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10+1.5888798701298701e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-1.3697240259740260e-03*w26*Jinv20-3.8352272727272725e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-5.4788961038961042e-05*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-7.6704545454545450e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10+6.5746753246753243e-04*w39*Jinv00-7.6704545454545450e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+1.5888798701298701e-03*w7*Jinv10+1.5888798701298701e-03*Jinv20*w38-7.6704545454545450e-04*Jinv22*w23-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12-1.3697240259740260e-03*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+4.2735389610389614e-03*Jinv12*w7-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.3697240259740260e-03*w52*Jinv10-3.8352272727272725e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-1.3697240259740260e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22-3.8352272727272725e-04*w10*Jinv10+4.6022727272727274e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33+5.2962662337662338e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-1.3697240259740260e-03*w25*Jinv10+5.2962662337662338e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14+2.1367694805194807e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20-1.3149350649350649e-03*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10+4.6022727272727274e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+4.9310064935064932e-04*Jinv12*w1)*D;
  A[17][39]=(0.0)*D;
  A[17][40]=(0.0)*D;
  A[17][41]=(1.9724025974025973e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12-1.9724025974025973e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+1.6436688311688311e-03*w57*Jinv02+1.9724025974025973e-03*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3-9.8620129870129864e-04*Jinv20*w23+1.9724025974025973e-03*Jinv00*w54+9.8620129870129864e-04*w25*Jinv12-6.9399350649350649e-04*w0*Jinv02-9.1314935064935063e-04*Jinv21*w2-1.1834415584415585e-02*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45-2.9586038961038961e-03*Jinv12*w55-1.1323051948051949e-03*w58*Jinv10+2.3668831168831169e-02*w15*Jinv02+1.1323051948051949e-03*w10*Jinv12+1.3149350649350649e-03*w48*Jinv00+9.8620129870129864e-04*Jinv02*w24+3.9448051948051946e-03*Jinv22*w35-2.3011363636363637e-03*Jinv21*w26-9.1314935064935063e-04*w0*Jinv01-1.1323051948051949e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10-1.9724025974025973e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11-9.8620129870129864e-04*w45*Jinv00-2.1915584415584417e-04*Jinv00*w27+1.1323051948051949e-03*w11*Jinv22-9.1314935064935063e-04*Jinv11*w1+2.9586038961038961e-03*w13*Jinv11+9.1314935064935063e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-9.8620129870129864e-04*w46*Jinv10+2.3668831168831169e-02*Jinv22*w17-7.8896103896103891e-03*Jinv00*w33+1.9724025974025973e-03*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10+3.2873376623376621e-03*Jinv02*w3-9.8620129870129864e-04*w21*Jinv00+2.3011363636363637e-03*w32*Jinv22-6.9399350649350649e-04*Jinv22*w2+1.9724025974025973e-03*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00-2.3011363636363637e-03*Jinv01*w24+1.6436688311688311e-03*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+6.5746753246753243e-04*w19*Jinv11+1.3149350649350649e-03*w51*Jinv00+3.9448051948051946e-03*Jinv12*w34+1.3149350649350649e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11-1.1834415584415585e-02*w41*Jinv20+4.9310064935064934e-03*Jinv12*w13+1.9724025974025973e-03*w19*Jinv10-7.8896103896103891e-03*w34*Jinv10+2.3011363636363637e-03*w32*Jinv21+6.5746753246753243e-04*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+1.9724025974025973e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41-9.8620129870129864e-04*w26*Jinv20-1.1323051948051949e-03*w11*Jinv20-1.9724025974025973e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11+9.8620129870129864e-04*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+1.9724025974025973e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21-2.9586038961038961e-03*Jinv02*w54+3.2873376623376621e-03*w4*Jinv12+1.9724025974025973e-03*w21*Jinv02+1.3149350649350649e-03*w49*Jinv10+9.1314935064935063e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33-1.1834415584415585e-02*w39*Jinv00+4.9310064935064934e-03*Jinv02*w12+6.5746753246753243e-04*Jinv01*w18+1.1323051948051949e-03*Jinv02*w9-6.5746753246753243e-04*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+1.9724025974025973e-03*Jinv22*w23+2.3011363636363637e-03*w31*Jinv12+1.3149350649350649e-03*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8-9.8620129870129864e-04*w22*Jinv10-3.6160714285714286e-03*Jinv22*w50-2.9586038961038961e-03*w22*Jinv11-2.6298701298701297e-03*Jinv12*w7-2.9586038961038961e-03*w21*Jinv01+2.3011363636363637e-03*Jinv02*w30-3.6160714285714286e-03*Jinv12*w49+1.3149350649350649e-03*w52*Jinv10-6.5746753246753243e-04*w7*Jinv11-1.1323051948051949e-03*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+6.5746753246753243e-04*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-1.9724025974025973e-03*w19*Jinv12+3.2873376623376621e-03*Jinv22*w5-6.5746753246753243e-04*w49*Jinv11+2.9586038961038961e-03*Jinv21*w14-1.1323051948051949e-03*w10*Jinv10-1.1834415584415585e-02*Jinv20*w44+1.6436688311688311e-03*w58*Jinv12-1.9724025974025973e-03*Jinv02*w18+2.9586038961038961e-03*Jinv01*w12-6.5746753246753243e-04*Jinv21*w8-1.1323051948051949e-03*w57*Jinv00-1.1323051948051949e-03*Jinv20*w59+3.9448051948051946e-03*Jinv02*w33-1.1323051948051949e-03*w0*Jinv00-7.8896103896103891e-03*w35*Jinv20-2.9586038961038961e-03*Jinv21*w23+6.5746753246753243e-04*Jinv01*w54+6.5746753246753243e-04*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-1.1323051948051949e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-3.6160714285714286e-03*w48*Jinv02-1.1834415584415585e-02*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-1.1834415584415585e-02*w40*Jinv10+1.9724025974025973e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14+1.3149350649350649e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30-1.9724025974025973e-03*Jinv02*w51-2.9586038961038961e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-2.3011363636363637e-03*w25*Jinv11+1.9724025974025973e-03*w55*Jinv10-1.1834415584415585e-02*w42*Jinv00-9.8620129870129864e-04*w47*Jinv20+2.3668831168831169e-02*w16*Jinv12-6.5746753246753243e-04*Jinv21*w50+9.1314935064935063e-04*Jinv01*w27-6.5746753246753243e-04*w48*Jinv01-6.9399350649350649e-04*Jinv12*w1)*D;
  A[17][42]=(0.0)*D;
  A[17][43]=(0.0)*D;
  A[17][44]=(-1.9724025974025973e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+2.7759740259740259e-03*w57*Jinv02-1.9724025974025973e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+1.1834415584415585e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-1.9724025974025973e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23+9.8620129870129864e-04*Jinv22*w38-1.9724025974025973e-03*Jinv00*w54+1.1323051948051949e-03*Jinv21*w59+2.9586038961038961e-03*w25*Jinv12+2.2646103896103898e-03*w0*Jinv02+1.1323051948051949e-03*Jinv21*w2+1.1834415584415585e-02*w43*Jinv10+1.1834415584415585e-02*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55+1.1323051948051949e-03*w58*Jinv10+3.5503246753246752e-02*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00+2.9586038961038961e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+1.1323051948051949e-03*w58*Jinv11+1.1323051948051949e-03*w57*Jinv01-1.9724025974025973e-03*Jinv21*w26+1.1323051948051949e-03*w0*Jinv01+1.1323051948051949e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10+2.9586038961038961e-03*Jinv22*w20+1.1834415584415585e-02*w43*Jinv11+9.8620129870129864e-04*w45*Jinv00+7.8896103896103891e-03*Jinv21*w41+1.7751623376623376e-02*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+1.1323051948051949e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+1.1323051948051949e-03*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+9.8620129870129864e-04*w46*Jinv10+3.5503246753246752e-02*Jinv22*w17+7.8896103896103891e-03*Jinv00*w33+1.9724025974025973e-03*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-3.9448051948051946e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+7.8896103896103891e-03*Jinv01*w39+2.2646103896103898e-03*Jinv22*w2-1.9724025974025973e-03*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00+9.8620129870129864e-04*Jinv21*w38+9.8620129870129864e-04*Jinv02*w45-1.9724025974025973e-03*Jinv01*w24+2.7759740259740259e-03*Jinv22*w59+1.1834415584415585e-02*w34*Jinv11+9.8620129870129864e-04*w19*Jinv11-1.9724025974025973e-03*Jinv21*w53-1.3149350649350649e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+1.7751623376623376e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-1.9724025974025973e-03*w4*Jinv11+1.1834415584415585e-02*w41*Jinv20-3.9448051948051946e-03*Jinv12*w13-1.9724025974025973e-03*w19*Jinv10+7.8896103896103891e-03*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21+9.8620129870129864e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+9.8620129870129864e-04*w37*Jinv12-1.9724025974025973e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+9.8620129870129864e-04*w26*Jinv20+1.1323051948051949e-03*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12+1.1834415584415585e-02*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+2.9586038961038961e-03*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-1.9724025974025973e-03*Jinv20*w14-1.9724025974025973e-03*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54-3.9448051948051946e-03*w4*Jinv12+1.9724025974025973e-03*w21*Jinv02-1.3149350649350649e-03*w49*Jinv10+1.1323051948051949e-03*Jinv21*w29+1.1834415584415585e-02*Jinv01*w33+1.1834415584415585e-02*w39*Jinv00-3.9448051948051946e-03*Jinv02*w12+9.8620129870129864e-04*Jinv01*w18+9.8620129870129864e-04*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-1.3149350649350649e-03*w47*Jinv21-1.3149350649350649e-03*Jinv20*w38+1.9724025974025973e-03*Jinv22*w23-1.9724025974025973e-03*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+9.8620129870129864e-04*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53+1.9724025974025973e-03*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10-3.2873376623376621e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+1.9724025974025973e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01-1.9724025974025973e-03*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-3.2873376623376621e-03*Jinv12*w49+9.8620129870129864e-04*w36*Jinv01-1.3149350649350649e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+1.1323051948051949e-03*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10-1.3149350649350649e-03*Jinv21*w56+9.8620129870129864e-04*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27+1.9724025974025973e-03*w6*Jinv02+2.9586038961038961e-03*w19*Jinv12-3.9448051948051946e-03*Jinv22*w5+9.8620129870129864e-04*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.1323051948051949e-03*w10*Jinv10+1.1834415584415585e-02*Jinv20*w44+2.7759740259740259e-03*w58*Jinv12+2.9586038961038961e-03*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8+1.1323051948051949e-03*w57*Jinv00+1.1323051948051949e-03*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+1.1323051948051949e-03*w0*Jinv00+7.8896103896103891e-03*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11+9.8620129870129864e-04*w25*Jinv10+1.1323051948051949e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-3.2873376623376621e-03*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+1.1834415584415585e-02*w40*Jinv10-1.9724025974025973e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29-3.9448051948051946e-03*Jinv22*w14+7.8896103896103891e-03*w40*Jinv11+9.8620129870129864e-04*Jinv02*w36+1.7751623376623376e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-1.9724025974025973e-03*w25*Jinv11-1.9724025974025973e-03*w55*Jinv10+1.1834415584415585e-02*w42*Jinv00+9.8620129870129864e-04*w47*Jinv20+3.5503246753246752e-02*w16*Jinv12-1.3149350649350649e-03*Jinv21*w50+1.1323051948051949e-03*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+2.2646103896103898e-03*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[17][45]=(0.0)*D;
  A[17][46]=(0.0)*D;
  A[17][47]=(9.8620129870129864e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.7394480519480518e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39+4.6022727272727274e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-3.8352272727272725e-04*Jinv22*w38+5.2962662337662338e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+5.2962662337662338e-04*Jinv21*w2+4.6022727272727274e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+1.9724025974025973e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+5.2962662337662338e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+4.6022727272727274e-03*w43*Jinv11+4.6022727272727274e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02+4.9310064935064932e-04*w11*Jinv22+5.2962662337662338e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+1.9724025974025973e-03*Jinv22*w17+4.2735389610389614e-03*w22*Jinv12-7.6704545454545450e-04*Jinv02*w3-3.8352272727272725e-04*w32*Jinv22+4.6022727272727274e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+1.5888798701298701e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-1.3697240259740260e-03*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-7.1225649350649346e-04*w32*Jinv21-1.3697240259740260e-03*Jinv21*w20-3.8352272727272725e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+1.5888798701298701e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-3.8352272727272725e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-1.3697240259740260e-03*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+1.5888798701298701e-03*w47*Jinv21+4.2735389610389614e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-7.6704545454545450e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+1.5888798701298701e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.5888798701298701e-03*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-1.3697240259740260e-03*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14+2.7394480519480518e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+1.5888798701298701e-03*Jinv21*w23-1.3697240259740260e-03*Jinv01*w54-1.3697240259740260e-03*w55*Jinv11-5.4788961038961042e-05*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+4.6022727272727274e-03*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-7.1225649350649346e-04*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.5888798701298701e-03*w25*Jinv11+1.9724025974025973e-03*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[17][48]=(0.0)*D;
  A[17][49]=(0.0)*D;
  A[17][50]=(5.4788961038961042e-05*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+2.3011363636363637e-03*w40*Jinv12+5.4788961038961042e-05*Jinv20*w8-1.8262987012987014e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10+2.3011363636363637e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+3.8352272727272725e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-3.2873376623376621e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+3.2873376623376621e-03*w16*Jinv11+3.2873376623376621e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-6.5746753246753243e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-1.0957792207792207e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24+2.3011363636363637e-03*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+3.2873376623376621e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11+3.8352272727272725e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+3.2873376623376621e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.0957792207792207e-03*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10-6.5746753246753243e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+3.8352272727272725e-04*w31*Jinv10-2.5750811688311690e-03*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00+3.8352272727272725e-04*Jinv21*w38-3.2873376623376621e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-1.8262987012987014e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00+2.3011363636363637e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-2.5750811688311690e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+3.8352272727272725e-04*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-3.2873376623376621e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+2.3011363636363637e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+1.3149350649350649e-03*Jinv21*w35+3.8352272727272725e-04*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-2.5750811688311690e-03*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-1.0957792207792207e-03*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-2.5750811688311690e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+5.4788961038961036e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30+5.4788961038961036e-04*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-2.5750811688311690e-03*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22-1.0957792207792207e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44-1.8262987012987014e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+2.3011363636363637e-03*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+3.2873376623376621e-03*w15*Jinv00+5.4788961038961036e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9+3.2873376623376621e-03*Jinv20*w17+3.8352272727272725e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-2.5750811688311690e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-3.2873376623376621e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+3.8352272727272725e-04*Jinv01*w30-3.6525974025974026e-05*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20-6.5746753246753243e-03*w16*Jinv12-1.0957792207792207e-03*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27-1.0957792207792207e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[17][51]=(0.0)*D;
  A[17][52]=(0.0)*D;
  A[17][53]=(-5.4788961038961042e-05*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12+1.6436688311688311e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-6.7573051948051951e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39-5.4788961038961042e-05*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-8.2183441558441553e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-7.3051948051948052e-05*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10+1.0957792207792207e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129868e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35-3.6525974025974026e-05*w1*Jinv10-3.2873376623376621e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20-3.8352272727272725e-04*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+1.0957792207792207e-03*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10-9.8620129870129868e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12-3.8352272727272725e-04*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22-7.3051948051948052e-05*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-3.8352272727272725e-04*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45-6.7573051948051951e-04*Jinv22*w59+1.0957792207792207e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-8.2183441558441553e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20+1.6436688311688311e-03*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26-5.4788961038961042e-05*Jinv20*w14+1.0957792207792207e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-1.3149350649350649e-03*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9-5.4788961038961042e-05*w7*Jinv10-3.8352272727272725e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+1.0957792207792207e-03*Jinv20*w53-2.6298701298701297e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-2.6298701298701297e-03*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49+1.0957792207792207e-03*w52*Jinv10-3.6525974025974026e-05*w9*Jinv00-3.8352272727272725e-04*w37*Jinv10-5.4788961038961042e-05*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-2.6298701298701297e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22-3.6525974025974026e-05*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-6.7573051948051951e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-3.6525974025974026e-05*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-5.4788961038961042e-05*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-3.2873376623376621e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02-3.2873376623376621e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-8.2183441558441553e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+1.6436688311688311e-03*Jinv02*w51+1.0957792207792207e-03*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28+1.0957792207792207e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00-3.8352272727272725e-04*w47*Jinv20-9.8620129870129868e-03*w16*Jinv12-7.3051948051948052e-05*Jinv12*w1)*D;
  A[17][54]=(0.0)*D;
  A[17][55]=(0.0)*D;
  A[17][56]=(9.8620129870129864e-04*w40*Jinv12+1.0957792207792207e-03*Jinv22*w53-6.7573051948051951e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38-4.9310064935064932e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-7.3051948051948052e-05*w0*Jinv02-3.6525974025974026e-05*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-3.2873376623376621e-03*w16*Jinv11-3.2873376623376621e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+1.6436688311688311e-03*Jinv12*w55-9.8620129870129868e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12-2.6298701298701297e-03*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-7.3051948051948052e-05*w11*Jinv22-3.6525974025974026e-05*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-3.2873376623376621e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-9.8620129870129868e-03*Jinv22*w17-2.6298701298701297e-03*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-7.3051948051948052e-05*Jinv22*w2-3.8352272727272725e-04*Jinv21*w38-8.2183441558441553e-04*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-6.7573051948051951e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53+6.5746753246753243e-04*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-5.4788961038961042e-05*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-3.8352272727272725e-04*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41+1.0957792207792207e-03*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-3.8352272727272725e-04*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+1.6436688311688311e-03*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12-2.6298701298701297e-03*w21*Jinv02-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-3.8352272727272725e-04*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-3.8352272727272725e-04*w47*Jinv21-2.6298701298701297e-03*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12-8.2183441558441553e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11+1.0957792207792207e-03*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5-8.2183441558441553e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-6.7573051948051951e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+6.5746753246753243e-04*Jinv02*w33-5.4788961038961042e-05*Jinv21*w23+1.0957792207792207e-03*Jinv01*w54+1.0957792207792207e-03*w55*Jinv11+1.0957792207792207e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*Jinv01*w30+1.0957792207792207e-03*Jinv02*w51+1.6436688311688311e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11-9.8620129870129868e-03*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01-7.3051948051948052e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[17][57]=(0.0)*D;
  A[17][58]=(0.0)*D;
  A[17][59]=(-5.1136363636363640e-04*w40*Jinv12-3.1047077922077924e-04*Jinv22*w53+2.4350649350649351e-04*w57*Jinv02-5.1136363636363640e-04*Jinv02*w39+2.1915584415584417e-04*Jinv22*w38+5.4788961038961036e-04*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02-3.1047077922077924e-04*Jinv12*w55+5.5519480519480519e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12+5.4788961038961036e-04*Jinv02*w24-5.1136363636363640e-04*Jinv22*w35+5.4788961038961036e-04*Jinv22*w20-5.1136363636363640e-04*w42*Jinv02+3.0438311688311688e-05*w11*Jinv22+5.5519480519480519e-03*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+2.1915584415584417e-04*w32*Jinv22+3.0438311688311688e-05*Jinv22*w2+2.1915584415584417e-04*Jinv02*w45+2.4350649350649351e-04*Jinv22*w59-5.1136363636363640e-04*Jinv12*w34-5.1136363636363640e-04*Jinv22*w44+5.4788961038961036e-04*Jinv12*w13+2.1915584415584417e-04*w37*Jinv12-5.1136363636363640e-04*Jinv22*w41-3.1047077922077924e-04*w52*Jinv12+5.4788961038961036e-04*Jinv22*w26-3.1047077922077924e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02+5.4788961038961036e-04*Jinv02*w12+3.0438311688311688e-05*Jinv02*w9+5.4788961038961036e-04*Jinv22*w23+2.1915584415584417e-04*w31*Jinv12+2.1915584415584417e-04*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-3.1047077922077924e-04*Jinv22*w50+5.4788961038961036e-04*Jinv12*w7+2.1915584415584417e-04*Jinv02*w30-3.1047077922077924e-04*Jinv12*w49+3.0438311688311688e-05*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+2.1915584415584417e-04*w47*Jinv22+2.4350649350649351e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-5.1136363636363640e-04*Jinv02*w33-3.1047077922077924e-04*w48*Jinv02+3.0438311688311688e-05*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14+2.1915584415584417e-04*Jinv02*w36-5.1136363636363640e-04*Jinv12*w43-3.1047077922077924e-04*Jinv02*w51-3.1047077922077924e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28+5.5519480519480519e-03*w16*Jinv12+3.0438311688311688e-05*Jinv12*w1)*D;
  A[18][0]=(1.0957792207792208e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-3.4699675324675324e-04*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10-3.4699675324675324e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+2.6481331168831169e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20+2.6481331168831169e-04*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59+5.8441558441558442e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00+5.8441558441558442e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+5.8441558441558442e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10-1.4975649350649350e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-3.4699675324675324e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+2.6481331168831169e-04*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-3.4699675324675324e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2-1.4975649350649350e-03*Jinv00*w18-7.4878246753246752e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38+4.2918019480519480e-04*Jinv02*w45+5.8441558441558442e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-2.1915584415584417e-04*w34*Jinv11-1.4975649350649350e-03*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+2.6481331168831169e-04*w4*Jinv10+2.6481331168831169e-04*w4*Jinv11-3.4699675324675324e-04*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13-1.4975649350649350e-03*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-1.4975649350649350e-03*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00-7.4878246753246752e-04*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-3.4699675324675324e-04*Jinv22*w41+5.8441558441558442e-04*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-2.1915584415584417e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+5.8441558441558442e-04*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+2.6481331168831169e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54+2.6481331168831169e-04*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-2.1915584415584417e-04*Jinv01*w33-3.4699675324675324e-04*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12-1.4975649350649350e-03*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21-7.4878246753246752e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+2.7394480519480521e-05*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-7.4878246753246752e-04*w37*Jinv10+2.1915584415584417e-04*Jinv21*w56+5.8441558441558442e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27-3.6525974025974026e-05*w6*Jinv02-1.4975649350649350e-03*w19*Jinv12+2.6481331168831169e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-1.4975649350649350e-03*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23+2.1915584415584417e-04*Jinv01*w54+2.1915584415584417e-04*w55*Jinv11+5.8441558441558442e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-3.4699675324675324e-04*w40*Jinv10-1.4975649350649350e-03*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14-3.4699675324675324e-04*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+2.6481331168831169e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+5.8441558441558442e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[18][1]=(0.0)*D;
  A[18][2]=(0.0)*D;
  A[18][3]=(-5.4788961038961036e-04*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-5.4788961038961036e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-7.3965097402597403e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-5.7528409090909093e-04*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-4.6570616883116886e-04*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.4793019480519481e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.5750811688311690e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+1.6436688311688311e-03*w16*Jinv11+1.6436688311688311e-03*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+3.1960227272727273e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00-1.4793019480519481e-03*Jinv02*w24+8.2183441558441553e-04*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.4793019480519481e-03*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+8.7662337662337668e-04*w16*Jinv10+4.7666396103896100e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+1.6436688311688311e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+4.3831168831168834e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.7528409090909093e-04*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2+4.0543831168831168e-03*Jinv00*w18+2.0271915584415584e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-1.4793019480519481e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+8.2183441558441553e-04*w34*Jinv11+4.7666396103896100e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-1.1231737012987013e-03*w51*Jinv00+8.2183441558441553e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-7.3965097402597403e-04*w4*Jinv10-5.7528409090909093e-04*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13+4.0543831168831168e-03*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21+4.7666396103896100e-03*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-4.6570616883116886e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-9.5880681818181818e-04*w26*Jinv20+5.5702110389610384e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+8.2183441558441553e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-1.4793019480519481e-03*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-5.4788961038961036e-04*Jinv20*w14-5.7528409090909093e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54-5.7528409090909093e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+8.2183441558441553e-04*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12+4.7666396103896100e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+2.0271915584415584e-03*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12-1.1231737012987013e-03*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-1.1231737012987013e-03*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11+5.5702110389610384e-04*w9*Jinv00+2.0271915584415584e-03*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-9.5880681818181818e-04*w24*Jinv00+3.1047077922077924e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+4.7666396103896100e-03*w19*Jinv12-5.7528409090909093e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+5.5702110389610384e-04*w10*Jinv10+2.5750811688311690e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+4.7666396103896100e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8+3.1960227272727273e-04*w57*Jinv00+3.1960227272727273e-04*Jinv20*w59+8.2183441558441553e-04*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-9.5880681818181818e-04*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+8.7662337662337668e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+8.7662337662337668e-04*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+4.0543831168831168e-03*w20*Jinv20+3.1047077922077924e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-7.3965097402597403e-04*w5*Jinv20+8.2183441558441553e-05*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+3.1047077922077924e-04*Jinv12*w28-1.4793019480519481e-03*w25*Jinv11-4.6570616883116886e-04*w55*Jinv10+2.5750811688311690e-03*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[18][4]=(0.0)*D;
  A[18][5]=(0.0)*D;
  A[18][6]=(5.4788961038961036e-04*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12+2.3833198051948050e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+1.1505681818181819e-03*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.3833198051948050e-03*Jinv01*w3-3.6982548701298703e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-7.4878246753246752e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-2.6846590909090908e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-4.2735389610389614e-03*w16*Jinv11-4.2735389610389614e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-3.7439123376623376e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12+2.6298701298701297e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+2.6298701298701297e-03*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-3.5612824675324673e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10-7.3965097402597406e-03*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+8.2183441558441553e-05*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11-4.2735389610389614e-03*w15*Jinv01-7.8530844155844158e-04*w28*Jinv11-3.9265422077922079e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-5.4788961038961036e-04*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+2.3833198051948050e-03*Jinv02*w3+3.5612824675324673e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-2.4655032467532467e-03*w36*Jinv00-3.6982548701298703e-03*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45+2.6298701298701297e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11-7.3965097402597406e-03*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+1.1505681818181819e-03*w4*Jinv10+2.3833198051948050e-03*w4*Jinv11+7.1225649350649346e-04*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10-5.4788961038961036e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-7.3965097402597406e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00-3.6982548701298703e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.3971185064935066e-03*w26*Jinv20-1.1414366883116883e-03*w11*Jinv20+2.3833198051948050e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14+2.3833198051948050e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+2.3833198051948050e-03*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-7.8530844155844158e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+7.1225649350649346e-04*Jinv02*w12-7.3965097402597406e-03*Jinv01*w18-3.6982548701298703e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-3.6982548701298703e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-2.4655032467532467e-03*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+1.3697240259740260e-03*Jinv20*w53-3.6982548701298703e-03*Jinv22*w8+3.5612824675324673e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11-3.6982548701298703e-03*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-3.6982548701298703e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10-3.6982548701298703e-03*w7*Jinv11-1.1414366883116883e-03*w9*Jinv00-2.4655032467532467e-03*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+1.3971185064935066e-03*w24*Jinv00-7.8530844155844158e-04*Jinv02*w27-3.6982548701298703e-03*w6*Jinv02-7.3965097402597406e-03*w19*Jinv12+2.3833198051948050e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-1.1414366883116883e-03*w10*Jinv10-2.6846590909090908e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-7.3965097402597406e-03*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12-3.6982548701298703e-03*Jinv21*w8-3.7439123376623376e-04*w57*Jinv00-3.7439123376623376e-04*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-3.5612824675324673e-04*w0*Jinv00-5.4788961038961036e-04*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+1.3971185064935066e-03*w25*Jinv10-3.5612824675324673e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-4.9310064935064934e-03*w20*Jinv20-7.8530844155844158e-04*Jinv22*w29+7.1225649350649346e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-3.6982548701298703e-03*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+1.1505681818181819e-03*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30+2.3833198051948050e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-7.8530844155844158e-04*Jinv12*w28+2.6298701298701297e-03*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10-2.6846590909090908e-03*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-7.8530844155844158e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-7.4878246753246752e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[18][7]=(0.0)*D;
  A[18][8]=(0.0)*D;
  A[18][9]=(-1.0957792207792208e-04*Jinv00*w12+2.0495129870129870e-04*w28*Jinv10+1.1870941558441558e-03*Jinv20*w8-1.0957792207792208e-04*w13*Jinv10-6.7573051948051951e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-2.6481331168831169e-04*Jinv20*w23-1.0957792207792208e-04*Jinv00*w54+6.5746753246753243e-04*w43*Jinv10+1.5726461038961039e-04*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00+1.5726461038961039e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-2.6481331168831169e-04*w45*Jinv00+2.0495129870129870e-04*Jinv00*w27-5.4788961038961042e-05*Jinv20*w50+2.0495129870129870e-04*Jinv20*w29-2.6481331168831169e-04*w46*Jinv10+3.2873376623376621e-04*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10-2.6481331168831169e-04*w21*Jinv00+2.3741883116883116e-03*Jinv00*w18+1.1870941558441558e-03*w36*Jinv00-6.7573051948051951e-04*w51*Jinv00-6.7573051948051951e-04*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+2.3741883116883116e-03*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+1.1870941558441558e-03*w6*Jinv00-1.0957792207792208e-04*Jinv20*w56-1.0227272727272728e-03*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-1.0957792207792208e-04*Jinv20*w14-5.4788961038961042e-05*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+1.1870941558441558e-03*w7*Jinv10+1.1870941558441558e-03*Jinv20*w38-6.7573051948051951e-04*Jinv20*w53-2.6481331168831169e-04*w22*Jinv10-6.7573051948051951e-04*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+1.1870941558441558e-03*w37*Jinv10-1.0227272727272728e-03*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+6.5746753246753243e-04*Jinv20*w44+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+1.5726461038961039e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-1.0227272727272728e-03*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+6.5746753246753243e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+2.3741883116883116e-03*w20*Jinv20-6.7573051948051951e-04*w5*Jinv20-1.0957792207792208e-04*w55*Jinv10+6.5746753246753243e-04*w42*Jinv00-2.6481331168831169e-04*w47*Jinv20)*D;
  A[18][10]=(0.0)*D;
  A[18][11]=(0.0)*D;
  A[18][12]=(1.0683847402597404e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8+2.1915584415584417e-04*w57*Jinv02+1.0683847402597404e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59-1.4793019480519481e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-1.1231737012987013e-03*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02-1.8262987012987013e-05*w10*Jinv12-4.6570616883116886e-04*w48*Jinv00-1.4793019480519481e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01-9.5880681818181818e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+1.2601461038961039e-03*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27-1.8262987012987013e-05*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01-3.5612824675324673e-04*w28*Jinv11-4.6570616883116886e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+1.2601461038961039e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-1.3971185064935066e-03*Jinv02*w45-9.5880681818181818e-04*Jinv01*w24+2.1915584415584417e-04*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11+1.4245129870129869e-03*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.3149350649350649e-03*w41*Jinv20+1.0683847402597404e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10+1.4245129870129869e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.4793019480519481e-03*w26*Jinv20-1.8262987012987013e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35-1.4793019480519481e-03*Jinv22*w26-1.1231737012987013e-03*w46*Jinv11+1.0683847402597404e-03*Jinv20*w14-2.4655032467532466e-04*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-4.6570616883116886e-04*w49*Jinv10-3.5612824675324673e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+1.0683847402597404e-03*Jinv02*w12+1.4245129870129869e-03*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-1.8262987012987013e-05*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-1.1231737012987013e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-1.3971185064935066e-03*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-4.6570616883116886e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-4.6570616883116886e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11-1.8262987012987013e-05*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56-1.4793019480519481e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-1.3971185064935066e-03*w47*Jinv22-1.9176136363636362e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.8262987012987013e-05*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44+2.1915584415584417e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.4793019480519481e-03*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-4.6570616883116886e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.0683847402597404e-03*Jinv22*w14+1.2601461038961039e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-5.4788961038961036e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-9.5880681818181818e-04*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-1.9176136363636362e-04*Jinv21*w50-3.5612824675324673e-04*Jinv01*w27-1.9176136363636362e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[18][13]=(0.0)*D;
  A[18][14]=(0.0)*D;
  A[18][15]=(-4.3831168831168834e-04*w28*Jinv10+1.9724025974025973e-03*w40*Jinv12+2.6298701298701297e-03*Jinv22*w53-2.0819805194805196e-03*Jinv20*w8-5.4788961038961036e-04*w57*Jinv02+1.9724025974025973e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+1.8628246753246754e-03*w3*Jinv00+4.9310064935064932e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+4.9310064935064934e-03*w25*Jinv12-5.4788961038961036e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.7532467532467534e-03*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.6793831168831166e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+8.2183441558441553e-04*w48*Jinv00+4.9310064935064934e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+2.7942370129870131e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10-6.9034090909090911e-03*w16*Jinv10-1.2820616883116883e-02*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41-2.9586038961038961e-03*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1-6.0267857142857139e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-1.6436688311688311e-04*w28*Jinv11+8.2183441558441553e-04*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+2.6298701298701297e-03*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39-5.4788961038961036e-04*Jinv22*w2-9.5332792207792200e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+2.7942370129870131e-03*Jinv01*w24-5.4788961038961036e-04*Jinv22*w59-8.5470779220779228e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+2.5750811688311690e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-2.9586038961038961e-03*Jinv22*w44+1.8628246753246754e-03*w4*Jinv10+4.9310064935064932e-04*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-9.5332792207792200e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-3.2873376623376621e-04*w32*Jinv21-8.5470779220779228e-03*Jinv21*w20-2.0819805194805196e-03*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56+1.9724025974025973e-03*Jinv22*w41+2.7942370129870131e-03*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.6298701298701297e-03*w52*Jinv12-3.2873376623376621e-04*w31*Jinv11+4.9310064935064934e-03*Jinv22*w26+1.7532467532467534e-03*w46*Jinv11+4.9310064935064932e-04*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54+2.6298701298701297e-03*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02+8.2183441558441553e-04*w49*Jinv10-1.6436688311688311e-04*Jinv21*w29-2.9586038961038961e-03*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-8.5470779220779228e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+1.0957792207792207e-03*Jinv02*w9-2.0819805194805196e-03*w7*Jinv10+1.7532467532467534e-03*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12+2.5750811688311690e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+2.5750811688311690e-03*w52*Jinv10+1.0957792207792208e-04*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.7942370129870131e-03*w24*Jinv00-6.5746753246753243e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-1.2820616883116883e-02*w19*Jinv12+2.6298701298701297e-03*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-6.0267857142857139e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-5.4788961038961036e-04*w58*Jinv12-1.2820616883116883e-02*Jinv02*w18-6.0267857142857139e-04*Jinv01*w12-9.6793831168831166e-04*w57*Jinv00-9.6793831168831166e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.7942370129870131e-03*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2-6.9034090909090911e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-6.9034090909090911e-03*Jinv20*w17-2.9586038961038961e-03*w40*Jinv10-9.5332792207792200e-03*w20*Jinv20-6.5746753246753243e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-2.9586038961038961e-03*Jinv12*w43+1.8628246753246754e-03*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30+2.6298701298701297e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-6.5746753246753243e-04*Jinv12*w28+2.7942370129870131e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-1.6436688311688311e-04*Jinv01*w27-5.4788961038961036e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[18][16]=(0.0)*D;
  A[18][17]=(0.0)*D;
  A[18][18]=(-1.0683847402597404e-03*Jinv00*w12+6.5746753246753243e-04*w28*Jinv10+2.4655032467532467e-03*Jinv20*w8-1.0683847402597404e-03*w13*Jinv10+4.2735389610389614e-03*Jinv21*w44-2.0271915584415584e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.3833198051948050e-03*Jinv01*w3+5.7528409090909093e-04*Jinv20*w23-1.0683847402597404e-03*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+4.7666396103896100e-03*w43*Jinv10+4.2735389610389614e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+4.2735389610389614e-03*w16*Jinv11+4.2735389610389614e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01-2.6298701298701297e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+4.7666396103896100e-03*w16*Jinv10+4.2735389610389614e-03*w43*Jinv11+5.7528409090909093e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+6.5746753246753243e-04*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+4.2735389610389614e-03*w15*Jinv01+7.8530844155844158e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+6.5746753246753243e-04*Jinv20*w29+5.7528409090909093e-04*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10+5.7528409090909093e-04*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+7.3965097402597406e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00+3.6982548701298703e-03*Jinv21*w38-2.6298701298701297e-03*Jinv01*w24+2.1367694805194807e-03*w34*Jinv11+7.3965097402597406e-03*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-2.0271915584415584e-03*w51*Jinv00-2.0271915584415584e-03*w4*Jinv10-2.3833198051948050e-03*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+7.3965097402597406e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21+7.3965097402597406e-03*Jinv21*w20+2.4655032467532467e-03*w6*Jinv00-1.0683847402597404e-03*Jinv20*w56-1.3149350649350649e-03*w26*Jinv20-9.1314935064935068e-05*w11*Jinv20+2.1367694805194807e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11-1.0683847402597404e-03*Jinv20*w14-2.3833198051948050e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+7.8530844155844158e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+7.3965097402597406e-03*Jinv01*w18+3.6982548701298703e-03*w37*Jinv11+3.6982548701298703e-03*w6*Jinv01+2.4655032467532467e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+2.4655032467532467e-03*Jinv20*w38-2.3833198051948050e-03*w52*Jinv11-2.0271915584415584e-03*Jinv20*w53+5.7528409090909093e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+3.6982548701298703e-03*w36*Jinv01-2.0271915584415584e-03*w52*Jinv10+3.6982548701298703e-03*w7*Jinv11-9.1314935064935068e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-9.1314935064935068e-05*w10*Jinv10+4.7666396103896100e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12+3.6982548701298703e-03*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+4.7666396103896100e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+4.7666396103896100e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+7.3965097402597406e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-2.0271915584415584e-03*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30-2.6298701298701297e-03*w25*Jinv11-1.0683847402597404e-03*w55*Jinv10+4.7666396103896100e-03*w42*Jinv00+5.7528409090909093e-04*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50+7.8530844155844158e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[18][19]=(0.0)*D;
  A[18][20]=(0.0)*D;
  A[18][21]=(-1.3971185064935066e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-3.8352272727272725e-04*Jinv01*w3+2.3833198051948050e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21-6.0267857142857139e-04*w16*Jinv11-6.0267857142857139e-04*Jinv21*w17+1.3697240259740260e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+4.7483766233766234e-04*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10-1.1505681818181819e-03*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-1.2053571428571428e-03*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.7483766233766234e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-9.5880681818181818e-04*w13*Jinv11-6.0267857142857139e-04*w15*Jinv01+6.6659902597402602e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.3833198051948050e-03*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+2.3833198051948050e-03*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-1.2053571428571428e-03*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2-1.1505681818181819e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+1.3971185064935066e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11+3.2873376623376621e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-3.8352272727272725e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-1.1505681818181819e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-2.7394480519480521e-05*w32*Jinv21+3.2873376623376621e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+4.7483766233766234e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-2.7394480519480521e-05*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.3697240259740260e-03*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14-3.8352272727272725e-04*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+2.3833198051948050e-03*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+6.6659902597402602e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12+3.2873376623376621e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11+4.7483766233766234e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10+1.3697240259740260e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+2.3833198051948050e-03*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+2.3833198051948050e-03*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11+4.7483766233766234e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02-1.1505681818181819e-03*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22+1.9176136363636362e-04*w49*Jinv11-9.5880681818181818e-04*Jinv21*w14+4.7483766233766234e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.1505681818181819e-03*Jinv02*w18-9.5880681818181818e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10-1.1505681818181819e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14-1.2053571428571428e-03*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-2.7394480519480521e-05*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.3971185064935066e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12+1.9176136363636362e-04*Jinv21*w50+6.6659902597402602e-04*Jinv01*w27+1.9176136363636362e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[18][22]=(0.0)*D;
  A[18][23]=(0.0)*D;
  A[18][24]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-6.5746753246753243e-04*Jinv21*w44+5.2049512987012989e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-1.5523538961038962e-04*Jinv21*w59-1.5523538961038962e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-1.5523538961038962e-04*w58*Jinv11-1.5523538961038962e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26-1.5523538961038962e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-6.5746753246753243e-04*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.5523538961038962e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-1.4610389610389610e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.1505681818181819e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38-1.3149350649350649e-03*Jinv01*w24-8.2183441558441553e-04*w34*Jinv11+3.7804383116883116e-03*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+5.2049512987012989e-04*w51*Jinv00+5.2049512987012989e-04*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20+1.1505681818181819e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21+3.7804383116883116e-03*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.6298701298701297e-03*w26*Jinv20-4.7483766233766234e-04*w11*Jinv20-8.2183441558441553e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.2327516233766234e-03*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10-1.4610389610389610e-04*Jinv21*w29-8.2183441558441553e-04*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00+3.7804383116883116e-03*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11-2.4655032467532466e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+8.2183441558441553e-05*w52*Jinv11+5.2049512987012989e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-1.2327516233766234e-03*w22*Jinv11-1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51-2.4655032467532466e-04*w36*Jinv01+5.2049512987012989e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11-4.7483766233766234e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+4.3831168831168834e-04*Jinv21*w56-2.6298701298701297e-03*w24*Jinv00+2.4655032467532466e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-4.7483766233766234e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+4.3831168831168834e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11-2.6298701298701297e-03*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00-1.0957792207792207e-03*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11+5.2049512987012989e-04*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30-1.3149350649350649e-03*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+2.4655032467532466e-04*Jinv21*w50-1.4610389610389610e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-1.0957792207792207e-03*w10*Jinv11)*D;
  A[18][25]=(0.0)*D;
  A[18][26]=(0.0)*D;
  A[18][27]=(-3.2873376623376621e-04*Jinv21*w44+1.1870941558441559e-04*Jinv01*w3-1.1668019480519480e-04*Jinv21*w59-1.1668019480519480e-04*Jinv21*w2-3.2873376623376621e-04*w42*Jinv01+8.3198051948051951e-05*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17-6.7573051948051951e-04*Jinv01*w45-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-1.0227272727272728e-03*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-3.2873376623376621e-04*w43*Jinv11+2.9220779220779221e-04*Jinv21*w41-1.1668019480519480e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11+2.9220779220779221e-04*Jinv01*w39+3.6525974025974026e-05*Jinv21*w38-1.0227272727272728e-03*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11-2.5568181818181820e-04*w19*Jinv11+1.1870941558441559e-04*Jinv21*w53+1.1870941558441559e-04*w4*Jinv11-2.7394480519480521e-05*w32*Jinv21-2.5568181818181820e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35-2.7394480519480521e-05*w31*Jinv11-6.7573051948051951e-04*w46*Jinv11+1.1870941558441559e-04*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-2.5568181818181820e-04*Jinv01*w18+3.6525974025974026e-05*w37*Jinv11+3.6525974025974026e-05*w6*Jinv01-6.7573051948051951e-04*w47*Jinv21+1.1870941558441559e-04*w52*Jinv11-6.7573051948051951e-04*w22*Jinv11-6.7573051948051951e-04*w21*Jinv01+1.1870941558441559e-04*Jinv01*w51+3.6525974025974026e-05*w36*Jinv01+3.6525974025974026e-05*w7*Jinv11+4.9310064935064932e-04*Jinv21*w56-2.7394480519480521e-05*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14+4.9310064935064932e-04*Jinv01*w12+3.6525974025974026e-05*Jinv21*w8-6.7573051948051951e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+8.3198051948051951e-05*Jinv01*w9+2.9220779220779221e-04*w40*Jinv11-2.7394480519480521e-05*Jinv01*w30-1.0227272727272728e-03*w25*Jinv11-2.7394480519480521e-05*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27-2.7394480519480521e-05*w48*Jinv01+8.3198051948051951e-05*w10*Jinv11)*D;
  A[18][28]=(0.0)*D;
  A[18][29]=(0.0)*D;
  A[18][30]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+8.2183441558441553e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-1.9176136363636362e-04*Jinv20*w8+2.7394480519480521e-05*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+8.2183441558441553e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-5.4788961038961042e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-4.6570616883116886e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02-2.4655032467532466e-04*w48*Jinv00+3.2873376623376621e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+1.0957792207792208e-04*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+8.7662337662337668e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-6.3920454545454550e-05*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+8.7662337662337668e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+2.7394480519480521e-05*Jinv22*w59+7.1225649350649346e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11+8.7662337662337668e-04*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20-1.9176136363636362e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+8.2183441558441553e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+7.1225649350649346e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11-5.4788961038961036e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21-4.6570616883116886e-04*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+7.1225649350649346e-04*Jinv01*w33+8.7662337662337668e-04*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-1.9176136363636362e-04*w6*Jinv01-1.9176136363636362e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11-2.7394480519480518e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51+2.7394480519480521e-05*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10-1.9176136363636362e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02-8.2183441558441553e-05*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.7394480519480521e-05*w58*Jinv12+3.5612824675324673e-04*Jinv01*w12-1.9176136363636362e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+8.7662337662337668e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.5612824675324673e-04*Jinv22*w14+8.7662337662337668e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-4.6570616883116886e-04*Jinv22*w56-5.2049512987012989e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[18][31]=(0.0)*D;
  A[18][32]=(0.0)*D;
  A[18][33]=(-5.2962662337662338e-04*w28*Jinv10+8.2183441558441553e-04*Jinv22*w53-1.0409902597402598e-03*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02-1.9724025974025973e-03*Jinv21*w44+9.3141233766233771e-04*w3*Jinv00+1.3149350649350649e-03*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+2.1915584415584417e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+3.2873376623376621e-04*w48*Jinv00-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-1.9724025974025973e-03*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12-8.2183441558441553e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.7532467532467534e-03*w51*Jinv00-9.8620129870129864e-04*Jinv22*w44+9.3141233766233771e-04*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-1.0409902597402598e-03*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+1.7532467532467534e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+8.2183441558441553e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+7.6704545454545450e-04*w46*Jinv11+1.3149350649350649e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-8.2183441558441553e-04*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11-2.1367694805194807e-03*w6*Jinv01-1.0409902597402598e-03*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+1.7532467532467534e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+1.7532467532467534e-03*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56+1.7532467532467534e-03*w24*Jinv00+2.1367694805194807e-03*w6*Jinv02-8.2183441558441553e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-3.1047077922077924e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11+1.7532467532467534e-03*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-7.1225649350649346e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+9.3141233766233771e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+8.2183441558441553e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+2.1367694805194807e-03*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[18][34]=(0.0)*D;
  A[18][35]=(0.0)*D;
  A[18][36]=(-3.5612824675324673e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.3833198051948050e-03*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8+7.4878246753246752e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.0135957792207792e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23+3.6982548701298703e-03*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-2.6298701298701297e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+3.1777597402597403e-03*w43*Jinv10-7.1225649350649346e-04*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10+4.2735389610389614e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+3.7439123376623376e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+7.3965097402597406e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00+4.2735389610389614e-03*w42*Jinv02+3.9265422077922079e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-8.2183441558441553e-05*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+4.2735389610389614e-03*Jinv22*w17+1.5888798701298701e-03*Jinv00*w33-1.6436688311688311e-04*w22*Jinv12-3.5612824675324673e-04*w31*Jinv10-2.3833198051948050e-03*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2+2.4655032467532467e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45+7.4878246753246752e-04*Jinv22*w59-1.2327516233766234e-03*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-1.0135957792207792e-03*w4*Jinv10+1.0957792207792207e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10+1.2327516233766234e-03*w6*Jinv00+3.6982548701298703e-03*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20-2.3833198051948050e-03*w52*Jinv12-2.6298701298701297e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-7.1225649350649346e-04*Jinv02*w54-2.3833198051948050e-03*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+1.0957792207792207e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+1.2327516233766234e-03*w7*Jinv10+2.4655032467532467e-03*Jinv20*w38-1.6436688311688311e-04*Jinv22*w23-3.5612824675324673e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12-1.2327516233766234e-03*Jinv20*w53+3.6982548701298703e-03*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+3.6982548701298703e-03*Jinv12*w7-3.5612824675324673e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.2327516233766234e-03*w52*Jinv10-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.2327516233766234e-03*w24*Jinv00+7.8530844155844158e-04*Jinv02*w27+3.6982548701298703e-03*w6*Jinv02+7.3965097402597406e-03*w19*Jinv12-2.3833198051948050e-03*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-4.5657467532467534e-05*w10*Jinv10+3.1777597402597403e-03*Jinv20*w44+7.4878246753246752e-04*w58*Jinv12+7.3965097402597406e-03*Jinv02*w18+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33+3.7439123376623376e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-1.2327516233766234e-03*w25*Jinv10+3.7439123376623376e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+7.8530844155844158e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+3.6982548701298703e-03*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-1.0135957792207792e-03*w5*Jinv20-2.3833198051948050e-03*Jinv02*w51-7.1225649350649346e-04*Jinv22*w56+7.8530844155844158e-04*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10+3.1777597402597403e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20+4.2735389610389614e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[18][37]=(0.0)*D;
  A[18][38]=(0.0)*D;
  A[18][39]=(-1.1505681818181819e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+7.1225649350649346e-04*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-3.4699675324675324e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+2.6298701298701297e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+3.4699675324675324e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2+3.9448051948051946e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+2.5750811688311690e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55+7.3051948051948052e-05*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961042e-05*w48*Jinv00+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+1.7532467532467534e-03*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+4.2004870129870131e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+2.6298701298701297e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00+1.6436688311688311e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+3.8352272727272725e-04*Jinv11*w1-1.2053571428571428e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-7.6704545454545450e-04*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00-8.2183441558441553e-04*w32*Jinv22+1.6436688311688311e-03*Jinv01*w39+3.4699675324675324e-04*Jinv22*w2-3.2873376623376621e-03*Jinv00*w18-2.1915584415584414e-03*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45+1.7532467532467534e-03*Jinv01*w24-3.4699675324675324e-04*Jinv22*w59-2.1915584415584414e-03*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00+1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-3.2873376623376621e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-8.2183441558441553e-04*w32*Jinv21-2.1915584415584414e-03*Jinv21*w20-1.0957792207792207e-03*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+2.1367694805194807e-03*w26*Jinv20+9.8620129870129864e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-8.2183441558441553e-04*w31*Jinv11+2.5750811688311690e-03*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29+4.9310064935064934e-03*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12-2.1915584415584414e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11-1.0957792207792207e-03*w7*Jinv10+2.5750811688311690e-03*w47*Jinv21-2.1915584415584414e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+3.8352272727272725e-04*w52*Jinv11-8.2183441558441553e-04*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10+8.2183441558441553e-04*Jinv22*w50+1.2601461038961039e-03*w22*Jinv11+1.0957792207792207e-03*Jinv12*w7+1.2601461038961039e-03*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51-8.2183441558441553e-04*Jinv02*w30+8.2183441558441553e-04*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10+9.8620129870129864e-04*w9*Jinv00-2.1915584415584414e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00+1.0957792207792207e-03*w6*Jinv02-7.1225649350649346e-04*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-1.2053571428571428e-03*Jinv21*w14+9.8620129870129864e-04*w10*Jinv10+3.9448051948051946e-03*Jinv20*w44-3.4699675324675324e-04*w58*Jinv12-1.2053571428571428e-03*Jinv01*w12+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59+4.2004870129870131e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.2601461038961039e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.2004870129870131e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00+8.2183441558441553e-04*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-3.2873376623376621e-03*w20*Jinv20-1.3149350649350649e-03*Jinv22*w14+1.6436688311688311e-03*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-8.2183441558441553e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+1.7532467532467534e-03*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+3.9448051948051946e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-2.5568181818181820e-04*Jinv01*w27+3.4699675324675324e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[18][40]=(0.0)*D;
  A[18][41]=(0.0)*D;
  A[18][42]=(1.1505681818181819e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12-2.6298701298701297e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+5.4788961038961036e-04*w57*Jinv02+1.1505681818181819e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-4.9310064935064934e-03*w25*Jinv12+5.4788961038961036e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-7.3051948051948052e-05*w58*Jinv10+2.9586038961038961e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-5.4788961038961042e-05*w48*Jinv00-4.9310064935064934e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01-4.2004870129870131e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10+1.2820616883116883e-02*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+2.9586038961038961e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10-2.6298701298701297e-03*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+5.4788961038961036e-04*Jinv22*w2+3.2873376623376621e-03*Jinv00*w18+2.1915584415584414e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-2.1367694805194807e-03*Jinv01*w24+5.4788961038961036e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11-4.9310064935064934e-03*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13+3.2873376623376621e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-9.8620129870129864e-04*w11*Jinv20-2.6298701298701297e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-4.9310064935064934e-03*Jinv22*w26-7.6704545454545450e-04*w46*Jinv11+1.1505681818181819e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54-2.6298701298701297e-03*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-5.4788961038961042e-05*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-4.9310064935064934e-03*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12+4.2735389610389614e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-7.6704545454545450e-04*w47*Jinv21+2.1915584415584414e-03*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12-7.6704545454545450e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11+4.2735389610389614e-03*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+4.2735389610389614e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-9.8620129870129864e-04*w9*Jinv00+2.1915584415584414e-03*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+6.5746753246753243e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+1.2820616883116883e-02*w19*Jinv12-2.6298701298701297e-03*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-9.8620129870129864e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+5.4788961038961036e-04*w58*Jinv12+1.2820616883116883e-02*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-4.2004870129870131e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.2004870129870131e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-4.9310064935064934e-03*w40*Jinv10+3.2873376623376621e-03*w20*Jinv20+6.5746753246753243e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+4.2735389610389614e-03*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-2.6298701298701297e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+6.5746753246753243e-04*Jinv12*w28-2.1367694805194807e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-3.9448051948051946e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+2.9586038961038961e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01+5.4788961038961036e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[18][43]=(0.0)*D;
  A[18][44]=(0.0)*D;
  A[18][45]=(-1.3149350649350649e-03*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-4.2918019480519480e-04*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39-2.7394480519480520e-03*Jinv21*w44+2.4655032467532466e-04*Jinv01*w3-1.6436688311688311e-04*Jinv22*w38-1.7349837662337662e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-3.1960227272727273e-04*Jinv21*w2-2.7394480519480520e-03*w42*Jinv01-4.8396915584415583e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.0271915584415584e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55-2.1367694805194807e-03*w15*Jinv02-4.7483766233766234e-04*w10*Jinv12-2.6298701298701297e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01-1.2327516233766234e-03*Jinv21*w26-3.1960227272727273e-04*w0*Jinv01+1.1505681818181819e-03*Jinv22*w20-2.7394480519480520e-03*w43*Jinv11-2.5202922077922079e-03*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-4.7483766233766234e-04*w11*Jinv22-3.1960227272727273e-04*Jinv11*w1+8.4922889610389610e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-2.1367694805194807e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-2.5202922077922079e-03*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+3.8352272727272725e-04*Jinv21*w38-2.3833198051948050e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-4.2918019480519480e-04*Jinv22*w59-7.1225649350649346e-04*w34*Jinv11+1.4793019480519481e-03*w19*Jinv11+1.3697240259740259e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.4655032467532466e-04*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+4.6570616883116886e-04*w32*Jinv21+1.4793019480519481e-03*Jinv21*w20-1.6436688311688311e-04*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41+5.2049512987012989e-04*w52*Jinv12-7.1225649350649346e-04*Jinv21*w35+4.6570616883116886e-04*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-2.0271915584415584e-03*w46*Jinv11+2.4655032467532466e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+1.4793019480519481e-03*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11-4.7483766233766234e-04*Jinv02*w9+1.9176136363636362e-04*w6*Jinv01-2.0271915584415584e-03*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23+1.3697240259740259e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-2.3833198051948050e-03*w46*Jinv12-1.6436688311688311e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-1.0135957792207792e-03*w22*Jinv11-1.6436688311688311e-04*Jinv12*w7-1.0135957792207792e-03*w21*Jinv01+1.3697240259740259e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01+1.9176136363636362e-04*w7*Jinv11+4.3831168831168834e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02+1.1505681818181819e-03*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-2.3833198051948050e-03*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+8.4922889610389610e-04*Jinv21*w14-4.2918019480519480e-04*w58*Jinv12+1.1505681818181819e-03*Jinv02*w18+8.4922889610389610e-04*Jinv01*w12+1.9176136363636362e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.0135957792207792e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02-4.8396915584415583e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-2.5202922077922079e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+4.6570616883116886e-04*Jinv01*w30+5.2049512987012989e-04*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11-2.1367694805194807e-03*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-4.8396915584415583e-04*w10*Jinv11)*D;
  A[18][46]=(0.0)*D;
  A[18][47]=(0.0)*D;
  A[18][48]=(-8.2183441558441553e-05*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10-8.2183441558441553e-04*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10-8.2183441558441553e-04*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480521e-05*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+3.5612824675324673e-04*w48*Jinv00-3.2873376623376621e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+7.6704545454545450e-04*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27+1.0957792207792208e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10+2.4655032467532466e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-2.7394480519480521e-05*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-2.7394480519480521e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-3.2873376623376621e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+4.6570616883116886e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-8.2183441558441553e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-5.2049512987012989e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+4.6570616883116886e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.7394480519480521e-05*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10-2.7394480519480518e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.7394480519480521e-05*Jinv02*w30+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+2.4655032467532466e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-8.2183441558441553e-05*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-3.2873376623376621e-04*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-9.1314935064935065e-06*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+4.6570616883116886e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-5.2049512987012989e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-2.7394480519480521e-05*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[18][49]=(0.0)*D;
  A[18][50]=(0.0)*D;
  A[18][51]=(8.2183441558441553e-05*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12+5.7528409090909093e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-2.1002435064935065e-04*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+1.4793019480519481e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+1.4793019480519481e-03*Jinv02*w24-8.2183441558441553e-04*Jinv22*w35-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-4.7666396103896100e-03*Jinv22*w20+5.2049512987012989e-04*w45*Jinv00-1.6436688311688311e-03*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-3.5612824675324673e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+2.4655032467532466e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-7.1225649350649346e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+1.3697240259740259e-04*Jinv02*w45-2.1002435064935065e-04*Jinv22*w59-1.6436688311688311e-04*w51*Jinv00-8.2183441558441553e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-5.4788961038961042e-05*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13-7.1225649350649346e-04*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+5.2049512987012989e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20+5.7528409090909093e-04*w52*Jinv12+1.4793019480519481e-03*Jinv22*w26+8.2183441558441553e-05*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-5.4788961038961042e-05*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-1.6436688311688311e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23+2.4655032467532466e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+2.4655032467532466e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-1.6436688311688311e-04*w52*Jinv10+9.1314935064935065e-06*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+5.2049512987012989e-04*w24*Jinv00-3.1047077922077924e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-4.7666396103896100e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+9.1314935064935065e-06*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44-2.1002435064935065e-04*w58*Jinv12-4.7666396103896100e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-8.2183441558441553e-04*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+5.2049512987012989e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-7.1225649350649346e-04*w20*Jinv20-3.1047077922077924e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-2.4655032467532466e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+5.7528409090909093e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56-3.1047077922077924e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[18][52]=(0.0)*D;
  A[18][53]=(0.0)*D;
  A[18][54]=(-1.3149350649350649e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53+1.2784090909090910e-04*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+3.8352272727272725e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+1.4793019480519481e-03*w25*Jinv12-2.1915584415584417e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-1.0683847402597404e-03*Jinv12*w55-1.6436688311688311e-04*w15*Jinv02+1.8262987012987013e-05*w10*Jinv12+1.4793019480519481e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-5.4788961038961042e-05*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02+1.8262987012987013e-05*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-1.6436688311688311e-04*Jinv22*w17+1.3971185064935066e-03*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+4.6570616883116886e-04*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-2.1915584415584417e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+5.2049512987012989e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+2.4655032467532466e-04*Jinv12*w13+2.7394480519480518e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20+3.8352272727272725e-04*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-3.8352272727272725e-04*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11+1.4793019480519481e-03*Jinv22*w26+5.2049512987012989e-04*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21-1.0683847402597404e-03*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+1.3971185064935066e-03*w21*Jinv02-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33+2.4655032467532466e-04*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+1.8262987012987013e-05*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+5.2049512987012989e-04*w47*Jinv21+1.3971185064935066e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+4.6570616883116886e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+4.6570616883116886e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-7.1225649350649346e-04*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.2784090909090910e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-1.0683847402597404e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28+5.2049512987012989e-04*w25*Jinv11-1.6436688311688311e-04*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-2.1915584415584417e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[18][55]=(0.0)*D;
  A[18][56]=(0.0)*D;
  A[18][57]=(3.4699675324675324e-04*w40*Jinv12-2.6481331168831169e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+3.4699675324675324e-04*Jinv02*w39+3.6525974025974026e-05*Jinv22*w38-5.8441558441558442e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-1.0957792207792208e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-5.8441558441558442e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35+1.4975649350649350e-03*Jinv22*w20+4.3831168831168834e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12-4.2918019480519480e-04*Jinv02*w3-2.7394480519480521e-05*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+1.1870941558441559e-04*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+2.1915584415584417e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-2.1915584415584417e-04*Jinv12*w13+3.6525974025974026e-05*w37*Jinv12+3.4699675324675324e-04*Jinv22*w41-2.6481331168831169e-04*w52*Jinv12-5.8441558441558442e-04*Jinv22*w26-1.0957792207792208e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02-2.1915584415584417e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9-4.2918019480519480e-04*Jinv22*w23-2.7394480519480521e-05*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12+7.4878246753246752e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-2.7394480519480521e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.4204545454545455e-05*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+1.4975649350649350e-03*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12+1.4975649350649350e-03*Jinv02*w18+2.1915584415584417e-04*Jinv02*w33-5.4788961038961042e-05*w48*Jinv02-1.4204545454545455e-05*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+3.6525974025974026e-05*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-2.6481331168831169e-04*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[18][58]=(0.0)*D;
  A[18][59]=(0.0)*D;
  A[19][0]=(0.0)*D;
  A[19][1]=(1.0957792207792208e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-3.4699675324675324e-04*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10-3.4699675324675324e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+2.6481331168831169e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20+2.6481331168831169e-04*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59+5.8441558441558442e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00+5.8441558441558442e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+5.8441558441558442e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10-1.4975649350649350e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-3.4699675324675324e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+2.6481331168831169e-04*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-3.4699675324675324e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2-1.4975649350649350e-03*Jinv00*w18-7.4878246753246752e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38+4.2918019480519480e-04*Jinv02*w45+5.8441558441558442e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-2.1915584415584417e-04*w34*Jinv11-1.4975649350649350e-03*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+2.6481331168831169e-04*w4*Jinv10+2.6481331168831169e-04*w4*Jinv11-3.4699675324675324e-04*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13-1.4975649350649350e-03*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-1.4975649350649350e-03*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00-7.4878246753246752e-04*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-3.4699675324675324e-04*Jinv22*w41+5.8441558441558442e-04*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-2.1915584415584417e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+5.8441558441558442e-04*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+2.6481331168831169e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54+2.6481331168831169e-04*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-2.1915584415584417e-04*Jinv01*w33-3.4699675324675324e-04*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12-1.4975649350649350e-03*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21-7.4878246753246752e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+2.7394480519480521e-05*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-7.4878246753246752e-04*w37*Jinv10+2.1915584415584417e-04*Jinv21*w56+5.8441558441558442e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27-3.6525974025974026e-05*w6*Jinv02-1.4975649350649350e-03*w19*Jinv12+2.6481331168831169e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-1.4975649350649350e-03*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23+2.1915584415584417e-04*Jinv01*w54+2.1915584415584417e-04*w55*Jinv11+5.8441558441558442e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-3.4699675324675324e-04*w40*Jinv10-1.4975649350649350e-03*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14-3.4699675324675324e-04*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+2.6481331168831169e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+5.8441558441558442e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[19][2]=(0.0)*D;
  A[19][3]=(0.0)*D;
  A[19][4]=(-5.4788961038961036e-04*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-5.4788961038961036e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-7.3965097402597403e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-5.7528409090909093e-04*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-4.6570616883116886e-04*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.4793019480519481e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.5750811688311690e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+1.6436688311688311e-03*w16*Jinv11+1.6436688311688311e-03*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+3.1960227272727273e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00-1.4793019480519481e-03*Jinv02*w24+8.2183441558441553e-04*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.4793019480519481e-03*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+8.7662337662337668e-04*w16*Jinv10+4.7666396103896100e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+1.6436688311688311e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+4.3831168831168834e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.7528409090909093e-04*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2+4.0543831168831168e-03*Jinv00*w18+2.0271915584415584e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-1.4793019480519481e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+8.2183441558441553e-04*w34*Jinv11+4.7666396103896100e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-1.1231737012987013e-03*w51*Jinv00+8.2183441558441553e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-7.3965097402597403e-04*w4*Jinv10-5.7528409090909093e-04*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13+4.0543831168831168e-03*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21+4.7666396103896100e-03*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-4.6570616883116886e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-9.5880681818181818e-04*w26*Jinv20+5.5702110389610384e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+8.2183441558441553e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-1.4793019480519481e-03*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-5.4788961038961036e-04*Jinv20*w14-5.7528409090909093e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54-5.7528409090909093e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+8.2183441558441553e-04*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12+4.7666396103896100e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+2.0271915584415584e-03*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12-1.1231737012987013e-03*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-1.1231737012987013e-03*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11+5.5702110389610384e-04*w9*Jinv00+2.0271915584415584e-03*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-9.5880681818181818e-04*w24*Jinv00+3.1047077922077924e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+4.7666396103896100e-03*w19*Jinv12-5.7528409090909093e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+5.5702110389610384e-04*w10*Jinv10+2.5750811688311690e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+4.7666396103896100e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8+3.1960227272727273e-04*w57*Jinv00+3.1960227272727273e-04*Jinv20*w59+8.2183441558441553e-04*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-9.5880681818181818e-04*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+8.7662337662337668e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+8.7662337662337668e-04*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+4.0543831168831168e-03*w20*Jinv20+3.1047077922077924e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-7.3965097402597403e-04*w5*Jinv20+8.2183441558441553e-05*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+3.1047077922077924e-04*Jinv12*w28-1.4793019480519481e-03*w25*Jinv11-4.6570616883116886e-04*w55*Jinv10+2.5750811688311690e-03*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[19][5]=(0.0)*D;
  A[19][6]=(0.0)*D;
  A[19][7]=(5.4788961038961036e-04*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12+2.3833198051948050e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+1.1505681818181819e-03*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.3833198051948050e-03*Jinv01*w3-3.6982548701298703e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-7.4878246753246752e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-2.6846590909090908e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-4.2735389610389614e-03*w16*Jinv11-4.2735389610389614e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-3.7439123376623376e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12+2.6298701298701297e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+2.6298701298701297e-03*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-3.5612824675324673e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10-7.3965097402597406e-03*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+8.2183441558441553e-05*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11-4.2735389610389614e-03*w15*Jinv01-7.8530844155844158e-04*w28*Jinv11-3.9265422077922079e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-5.4788961038961036e-04*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+2.3833198051948050e-03*Jinv02*w3+3.5612824675324673e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-2.4655032467532467e-03*w36*Jinv00-3.6982548701298703e-03*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45+2.6298701298701297e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11-7.3965097402597406e-03*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+1.1505681818181819e-03*w4*Jinv10+2.3833198051948050e-03*w4*Jinv11+7.1225649350649346e-04*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10-5.4788961038961036e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-7.3965097402597406e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00-3.6982548701298703e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.3971185064935066e-03*w26*Jinv20-1.1414366883116883e-03*w11*Jinv20+2.3833198051948050e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14+2.3833198051948050e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+2.3833198051948050e-03*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-7.8530844155844158e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+7.1225649350649346e-04*Jinv02*w12-7.3965097402597406e-03*Jinv01*w18-3.6982548701298703e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-3.6982548701298703e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-2.4655032467532467e-03*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+1.3697240259740260e-03*Jinv20*w53-3.6982548701298703e-03*Jinv22*w8+3.5612824675324673e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11-3.6982548701298703e-03*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-3.6982548701298703e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10-3.6982548701298703e-03*w7*Jinv11-1.1414366883116883e-03*w9*Jinv00-2.4655032467532467e-03*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+1.3971185064935066e-03*w24*Jinv00-7.8530844155844158e-04*Jinv02*w27-3.6982548701298703e-03*w6*Jinv02-7.3965097402597406e-03*w19*Jinv12+2.3833198051948050e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-1.1414366883116883e-03*w10*Jinv10-2.6846590909090908e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-7.3965097402597406e-03*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12-3.6982548701298703e-03*Jinv21*w8-3.7439123376623376e-04*w57*Jinv00-3.7439123376623376e-04*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-3.5612824675324673e-04*w0*Jinv00-5.4788961038961036e-04*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+1.3971185064935066e-03*w25*Jinv10-3.5612824675324673e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-4.9310064935064934e-03*w20*Jinv20-7.8530844155844158e-04*Jinv22*w29+7.1225649350649346e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-3.6982548701298703e-03*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+1.1505681818181819e-03*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30+2.3833198051948050e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-7.8530844155844158e-04*Jinv12*w28+2.6298701298701297e-03*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10-2.6846590909090908e-03*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-7.8530844155844158e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-7.4878246753246752e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[19][8]=(0.0)*D;
  A[19][9]=(0.0)*D;
  A[19][10]=(-1.0957792207792208e-04*Jinv00*w12+2.0495129870129870e-04*w28*Jinv10+1.1870941558441558e-03*Jinv20*w8-1.0957792207792208e-04*w13*Jinv10-6.7573051948051951e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-2.6481331168831169e-04*Jinv20*w23-1.0957792207792208e-04*Jinv00*w54+6.5746753246753243e-04*w43*Jinv10+1.5726461038961039e-04*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00+1.5726461038961039e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-2.6481331168831169e-04*w45*Jinv00+2.0495129870129870e-04*Jinv00*w27-5.4788961038961042e-05*Jinv20*w50+2.0495129870129870e-04*Jinv20*w29-2.6481331168831169e-04*w46*Jinv10+3.2873376623376621e-04*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10-2.6481331168831169e-04*w21*Jinv00+2.3741883116883116e-03*Jinv00*w18+1.1870941558441558e-03*w36*Jinv00-6.7573051948051951e-04*w51*Jinv00-6.7573051948051951e-04*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+2.3741883116883116e-03*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+1.1870941558441558e-03*w6*Jinv00-1.0957792207792208e-04*Jinv20*w56-1.0227272727272728e-03*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-1.0957792207792208e-04*Jinv20*w14-5.4788961038961042e-05*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+1.1870941558441558e-03*w7*Jinv10+1.1870941558441558e-03*Jinv20*w38-6.7573051948051951e-04*Jinv20*w53-2.6481331168831169e-04*w22*Jinv10-6.7573051948051951e-04*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+1.1870941558441558e-03*w37*Jinv10-1.0227272727272728e-03*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+6.5746753246753243e-04*Jinv20*w44+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+1.5726461038961039e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-1.0227272727272728e-03*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+6.5746753246753243e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+2.3741883116883116e-03*w20*Jinv20-6.7573051948051951e-04*w5*Jinv20-1.0957792207792208e-04*w55*Jinv10+6.5746753246753243e-04*w42*Jinv00-2.6481331168831169e-04*w47*Jinv20)*D;
  A[19][11]=(0.0)*D;
  A[19][12]=(0.0)*D;
  A[19][13]=(1.0683847402597404e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8+2.1915584415584417e-04*w57*Jinv02+1.0683847402597404e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59-1.4793019480519481e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-1.1231737012987013e-03*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02-1.8262987012987013e-05*w10*Jinv12-4.6570616883116886e-04*w48*Jinv00-1.4793019480519481e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01-9.5880681818181818e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+1.2601461038961039e-03*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27-1.8262987012987013e-05*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01-3.5612824675324673e-04*w28*Jinv11-4.6570616883116886e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+1.2601461038961039e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-1.3971185064935066e-03*Jinv02*w45-9.5880681818181818e-04*Jinv01*w24+2.1915584415584417e-04*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11+1.4245129870129869e-03*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.3149350649350649e-03*w41*Jinv20+1.0683847402597404e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10+1.4245129870129869e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.4793019480519481e-03*w26*Jinv20-1.8262987012987013e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35-1.4793019480519481e-03*Jinv22*w26-1.1231737012987013e-03*w46*Jinv11+1.0683847402597404e-03*Jinv20*w14-2.4655032467532466e-04*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-4.6570616883116886e-04*w49*Jinv10-3.5612824675324673e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+1.0683847402597404e-03*Jinv02*w12+1.4245129870129869e-03*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-1.8262987012987013e-05*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-1.1231737012987013e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-1.3971185064935066e-03*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-4.6570616883116886e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-4.6570616883116886e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11-1.8262987012987013e-05*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56-1.4793019480519481e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-1.3971185064935066e-03*w47*Jinv22-1.9176136363636362e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.8262987012987013e-05*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44+2.1915584415584417e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.4793019480519481e-03*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-4.6570616883116886e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.0683847402597404e-03*Jinv22*w14+1.2601461038961039e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-5.4788961038961036e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-9.5880681818181818e-04*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-1.9176136363636362e-04*Jinv21*w50-3.5612824675324673e-04*Jinv01*w27-1.9176136363636362e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[19][14]=(0.0)*D;
  A[19][15]=(0.0)*D;
  A[19][16]=(-4.3831168831168834e-04*w28*Jinv10+1.9724025974025973e-03*w40*Jinv12+2.6298701298701297e-03*Jinv22*w53-2.0819805194805196e-03*Jinv20*w8-5.4788961038961036e-04*w57*Jinv02+1.9724025974025973e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+1.8628246753246754e-03*w3*Jinv00+4.9310064935064932e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+4.9310064935064934e-03*w25*Jinv12-5.4788961038961036e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.7532467532467534e-03*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.6793831168831166e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+8.2183441558441553e-04*w48*Jinv00+4.9310064935064934e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+2.7942370129870131e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10-6.9034090909090911e-03*w16*Jinv10-1.2820616883116883e-02*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41-2.9586038961038961e-03*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1-6.0267857142857139e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-1.6436688311688311e-04*w28*Jinv11+8.2183441558441553e-04*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+2.6298701298701297e-03*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39-5.4788961038961036e-04*Jinv22*w2-9.5332792207792200e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+2.7942370129870131e-03*Jinv01*w24-5.4788961038961036e-04*Jinv22*w59-8.5470779220779228e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+2.5750811688311690e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-2.9586038961038961e-03*Jinv22*w44+1.8628246753246754e-03*w4*Jinv10+4.9310064935064932e-04*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-9.5332792207792200e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-3.2873376623376621e-04*w32*Jinv21-8.5470779220779228e-03*Jinv21*w20-2.0819805194805196e-03*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56+1.9724025974025973e-03*Jinv22*w41+2.7942370129870131e-03*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.6298701298701297e-03*w52*Jinv12-3.2873376623376621e-04*w31*Jinv11+4.9310064935064934e-03*Jinv22*w26+1.7532467532467534e-03*w46*Jinv11+4.9310064935064932e-04*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54+2.6298701298701297e-03*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02+8.2183441558441553e-04*w49*Jinv10-1.6436688311688311e-04*Jinv21*w29-2.9586038961038961e-03*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-8.5470779220779228e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+1.0957792207792207e-03*Jinv02*w9-2.0819805194805196e-03*w7*Jinv10+1.7532467532467534e-03*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12+2.5750811688311690e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+2.5750811688311690e-03*w52*Jinv10+1.0957792207792208e-04*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.7942370129870131e-03*w24*Jinv00-6.5746753246753243e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-1.2820616883116883e-02*w19*Jinv12+2.6298701298701297e-03*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-6.0267857142857139e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-5.4788961038961036e-04*w58*Jinv12-1.2820616883116883e-02*Jinv02*w18-6.0267857142857139e-04*Jinv01*w12-9.6793831168831166e-04*w57*Jinv00-9.6793831168831166e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.7942370129870131e-03*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2-6.9034090909090911e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-6.9034090909090911e-03*Jinv20*w17-2.9586038961038961e-03*w40*Jinv10-9.5332792207792200e-03*w20*Jinv20-6.5746753246753243e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-2.9586038961038961e-03*Jinv12*w43+1.8628246753246754e-03*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30+2.6298701298701297e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-6.5746753246753243e-04*Jinv12*w28+2.7942370129870131e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-1.6436688311688311e-04*Jinv01*w27-5.4788961038961036e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[19][17]=(0.0)*D;
  A[19][18]=(0.0)*D;
  A[19][19]=(-1.0683847402597404e-03*Jinv00*w12+6.5746753246753243e-04*w28*Jinv10+2.4655032467532467e-03*Jinv20*w8-1.0683847402597404e-03*w13*Jinv10+4.2735389610389614e-03*Jinv21*w44-2.0271915584415584e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.3833198051948050e-03*Jinv01*w3+5.7528409090909093e-04*Jinv20*w23-1.0683847402597404e-03*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+4.7666396103896100e-03*w43*Jinv10+4.2735389610389614e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+4.2735389610389614e-03*w16*Jinv11+4.2735389610389614e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01-2.6298701298701297e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+4.7666396103896100e-03*w16*Jinv10+4.2735389610389614e-03*w43*Jinv11+5.7528409090909093e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+6.5746753246753243e-04*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+4.2735389610389614e-03*w15*Jinv01+7.8530844155844158e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+6.5746753246753243e-04*Jinv20*w29+5.7528409090909093e-04*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10+5.7528409090909093e-04*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+7.3965097402597406e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00+3.6982548701298703e-03*Jinv21*w38-2.6298701298701297e-03*Jinv01*w24+2.1367694805194807e-03*w34*Jinv11+7.3965097402597406e-03*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-2.0271915584415584e-03*w51*Jinv00-2.0271915584415584e-03*w4*Jinv10-2.3833198051948050e-03*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+7.3965097402597406e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21+7.3965097402597406e-03*Jinv21*w20+2.4655032467532467e-03*w6*Jinv00-1.0683847402597404e-03*Jinv20*w56-1.3149350649350649e-03*w26*Jinv20-9.1314935064935068e-05*w11*Jinv20+2.1367694805194807e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11-1.0683847402597404e-03*Jinv20*w14-2.3833198051948050e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+7.8530844155844158e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+7.3965097402597406e-03*Jinv01*w18+3.6982548701298703e-03*w37*Jinv11+3.6982548701298703e-03*w6*Jinv01+2.4655032467532467e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+2.4655032467532467e-03*Jinv20*w38-2.3833198051948050e-03*w52*Jinv11-2.0271915584415584e-03*Jinv20*w53+5.7528409090909093e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+3.6982548701298703e-03*w36*Jinv01-2.0271915584415584e-03*w52*Jinv10+3.6982548701298703e-03*w7*Jinv11-9.1314935064935068e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-9.1314935064935068e-05*w10*Jinv10+4.7666396103896100e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12+3.6982548701298703e-03*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+4.7666396103896100e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+4.7666396103896100e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+7.3965097402597406e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-2.0271915584415584e-03*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30-2.6298701298701297e-03*w25*Jinv11-1.0683847402597404e-03*w55*Jinv10+4.7666396103896100e-03*w42*Jinv00+5.7528409090909093e-04*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50+7.8530844155844158e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[19][20]=(0.0)*D;
  A[19][21]=(0.0)*D;
  A[19][22]=(-1.3971185064935066e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-3.8352272727272725e-04*Jinv01*w3+2.3833198051948050e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21-6.0267857142857139e-04*w16*Jinv11-6.0267857142857139e-04*Jinv21*w17+1.3697240259740260e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+4.7483766233766234e-04*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10-1.1505681818181819e-03*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-1.2053571428571428e-03*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.7483766233766234e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-9.5880681818181818e-04*w13*Jinv11-6.0267857142857139e-04*w15*Jinv01+6.6659902597402602e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.3833198051948050e-03*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+2.3833198051948050e-03*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-1.2053571428571428e-03*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2-1.1505681818181819e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+1.3971185064935066e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11+3.2873376623376621e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-3.8352272727272725e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-1.1505681818181819e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-2.7394480519480521e-05*w32*Jinv21+3.2873376623376621e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+4.7483766233766234e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-2.7394480519480521e-05*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.3697240259740260e-03*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14-3.8352272727272725e-04*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+2.3833198051948050e-03*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+6.6659902597402602e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12+3.2873376623376621e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11+4.7483766233766234e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10+1.3697240259740260e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+2.3833198051948050e-03*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+2.3833198051948050e-03*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11+4.7483766233766234e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02-1.1505681818181819e-03*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22+1.9176136363636362e-04*w49*Jinv11-9.5880681818181818e-04*Jinv21*w14+4.7483766233766234e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.1505681818181819e-03*Jinv02*w18-9.5880681818181818e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10-1.1505681818181819e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14-1.2053571428571428e-03*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-2.7394480519480521e-05*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.3971185064935066e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12+1.9176136363636362e-04*Jinv21*w50+6.6659902597402602e-04*Jinv01*w27+1.9176136363636362e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[19][23]=(0.0)*D;
  A[19][24]=(0.0)*D;
  A[19][25]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-6.5746753246753243e-04*Jinv21*w44+5.2049512987012989e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-1.5523538961038962e-04*Jinv21*w59-1.5523538961038962e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-1.5523538961038962e-04*w58*Jinv11-1.5523538961038962e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26-1.5523538961038962e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-6.5746753246753243e-04*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.5523538961038962e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-1.4610389610389610e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.1505681818181819e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38-1.3149350649350649e-03*Jinv01*w24-8.2183441558441553e-04*w34*Jinv11+3.7804383116883116e-03*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+5.2049512987012989e-04*w51*Jinv00+5.2049512987012989e-04*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20+1.1505681818181819e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21+3.7804383116883116e-03*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.6298701298701297e-03*w26*Jinv20-4.7483766233766234e-04*w11*Jinv20-8.2183441558441553e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.2327516233766234e-03*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10-1.4610389610389610e-04*Jinv21*w29-8.2183441558441553e-04*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00+3.7804383116883116e-03*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11-2.4655032467532466e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+8.2183441558441553e-05*w52*Jinv11+5.2049512987012989e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-1.2327516233766234e-03*w22*Jinv11-1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51-2.4655032467532466e-04*w36*Jinv01+5.2049512987012989e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11-4.7483766233766234e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+4.3831168831168834e-04*Jinv21*w56-2.6298701298701297e-03*w24*Jinv00+2.4655032467532466e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-4.7483766233766234e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+4.3831168831168834e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11-2.6298701298701297e-03*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00-1.0957792207792207e-03*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11+5.2049512987012989e-04*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30-1.3149350649350649e-03*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+2.4655032467532466e-04*Jinv21*w50-1.4610389610389610e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-1.0957792207792207e-03*w10*Jinv11)*D;
  A[19][26]=(0.0)*D;
  A[19][27]=(0.0)*D;
  A[19][28]=(-3.2873376623376621e-04*Jinv21*w44+1.1870941558441559e-04*Jinv01*w3-1.1668019480519480e-04*Jinv21*w59-1.1668019480519480e-04*Jinv21*w2-3.2873376623376621e-04*w42*Jinv01+8.3198051948051951e-05*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17-6.7573051948051951e-04*Jinv01*w45-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-1.0227272727272728e-03*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-3.2873376623376621e-04*w43*Jinv11+2.9220779220779221e-04*Jinv21*w41-1.1668019480519480e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11+2.9220779220779221e-04*Jinv01*w39+3.6525974025974026e-05*Jinv21*w38-1.0227272727272728e-03*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11-2.5568181818181820e-04*w19*Jinv11+1.1870941558441559e-04*Jinv21*w53+1.1870941558441559e-04*w4*Jinv11-2.7394480519480521e-05*w32*Jinv21-2.5568181818181820e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35-2.7394480519480521e-05*w31*Jinv11-6.7573051948051951e-04*w46*Jinv11+1.1870941558441559e-04*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-2.5568181818181820e-04*Jinv01*w18+3.6525974025974026e-05*w37*Jinv11+3.6525974025974026e-05*w6*Jinv01-6.7573051948051951e-04*w47*Jinv21+1.1870941558441559e-04*w52*Jinv11-6.7573051948051951e-04*w22*Jinv11-6.7573051948051951e-04*w21*Jinv01+1.1870941558441559e-04*Jinv01*w51+3.6525974025974026e-05*w36*Jinv01+3.6525974025974026e-05*w7*Jinv11+4.9310064935064932e-04*Jinv21*w56-2.7394480519480521e-05*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14+4.9310064935064932e-04*Jinv01*w12+3.6525974025974026e-05*Jinv21*w8-6.7573051948051951e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+8.3198051948051951e-05*Jinv01*w9+2.9220779220779221e-04*w40*Jinv11-2.7394480519480521e-05*Jinv01*w30-1.0227272727272728e-03*w25*Jinv11-2.7394480519480521e-05*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27-2.7394480519480521e-05*w48*Jinv01+8.3198051948051951e-05*w10*Jinv11)*D;
  A[19][29]=(0.0)*D;
  A[19][30]=(0.0)*D;
  A[19][31]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+8.2183441558441553e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-1.9176136363636362e-04*Jinv20*w8+2.7394480519480521e-05*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+8.2183441558441553e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-5.4788961038961042e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-4.6570616883116886e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02-2.4655032467532466e-04*w48*Jinv00+3.2873376623376621e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+1.0957792207792208e-04*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+8.7662337662337668e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-6.3920454545454550e-05*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+8.7662337662337668e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+2.7394480519480521e-05*Jinv22*w59+7.1225649350649346e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11+8.7662337662337668e-04*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20-1.9176136363636362e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+8.2183441558441553e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+7.1225649350649346e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11-5.4788961038961036e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21-4.6570616883116886e-04*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+7.1225649350649346e-04*Jinv01*w33+8.7662337662337668e-04*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-1.9176136363636362e-04*w6*Jinv01-1.9176136363636362e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11-2.7394480519480518e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51+2.7394480519480521e-05*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10-1.9176136363636362e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02-8.2183441558441553e-05*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.7394480519480521e-05*w58*Jinv12+3.5612824675324673e-04*Jinv01*w12-1.9176136363636362e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+8.7662337662337668e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.5612824675324673e-04*Jinv22*w14+8.7662337662337668e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-4.6570616883116886e-04*Jinv22*w56-5.2049512987012989e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[19][32]=(0.0)*D;
  A[19][33]=(0.0)*D;
  A[19][34]=(-5.2962662337662338e-04*w28*Jinv10+8.2183441558441553e-04*Jinv22*w53-1.0409902597402598e-03*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02-1.9724025974025973e-03*Jinv21*w44+9.3141233766233771e-04*w3*Jinv00+1.3149350649350649e-03*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+2.1915584415584417e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+3.2873376623376621e-04*w48*Jinv00-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-1.9724025974025973e-03*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12-8.2183441558441553e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.7532467532467534e-03*w51*Jinv00-9.8620129870129864e-04*Jinv22*w44+9.3141233766233771e-04*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-1.0409902597402598e-03*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+1.7532467532467534e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+8.2183441558441553e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+7.6704545454545450e-04*w46*Jinv11+1.3149350649350649e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-8.2183441558441553e-04*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11-2.1367694805194807e-03*w6*Jinv01-1.0409902597402598e-03*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+1.7532467532467534e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+1.7532467532467534e-03*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56+1.7532467532467534e-03*w24*Jinv00+2.1367694805194807e-03*w6*Jinv02-8.2183441558441553e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-3.1047077922077924e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11+1.7532467532467534e-03*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-7.1225649350649346e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+9.3141233766233771e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+8.2183441558441553e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+2.1367694805194807e-03*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[19][35]=(0.0)*D;
  A[19][36]=(0.0)*D;
  A[19][37]=(-3.5612824675324673e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.3833198051948050e-03*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8+7.4878246753246752e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.0135957792207792e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23+3.6982548701298703e-03*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-2.6298701298701297e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+3.1777597402597403e-03*w43*Jinv10-7.1225649350649346e-04*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10+4.2735389610389614e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+3.7439123376623376e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+7.3965097402597406e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00+4.2735389610389614e-03*w42*Jinv02+3.9265422077922079e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-8.2183441558441553e-05*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+4.2735389610389614e-03*Jinv22*w17+1.5888798701298701e-03*Jinv00*w33-1.6436688311688311e-04*w22*Jinv12-3.5612824675324673e-04*w31*Jinv10-2.3833198051948050e-03*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2+2.4655032467532467e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45+7.4878246753246752e-04*Jinv22*w59-1.2327516233766234e-03*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-1.0135957792207792e-03*w4*Jinv10+1.0957792207792207e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10+1.2327516233766234e-03*w6*Jinv00+3.6982548701298703e-03*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20-2.3833198051948050e-03*w52*Jinv12-2.6298701298701297e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-7.1225649350649346e-04*Jinv02*w54-2.3833198051948050e-03*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+1.0957792207792207e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+1.2327516233766234e-03*w7*Jinv10+2.4655032467532467e-03*Jinv20*w38-1.6436688311688311e-04*Jinv22*w23-3.5612824675324673e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12-1.2327516233766234e-03*Jinv20*w53+3.6982548701298703e-03*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+3.6982548701298703e-03*Jinv12*w7-3.5612824675324673e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.2327516233766234e-03*w52*Jinv10-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.2327516233766234e-03*w24*Jinv00+7.8530844155844158e-04*Jinv02*w27+3.6982548701298703e-03*w6*Jinv02+7.3965097402597406e-03*w19*Jinv12-2.3833198051948050e-03*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-4.5657467532467534e-05*w10*Jinv10+3.1777597402597403e-03*Jinv20*w44+7.4878246753246752e-04*w58*Jinv12+7.3965097402597406e-03*Jinv02*w18+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33+3.7439123376623376e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-1.2327516233766234e-03*w25*Jinv10+3.7439123376623376e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+7.8530844155844158e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+3.6982548701298703e-03*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-1.0135957792207792e-03*w5*Jinv20-2.3833198051948050e-03*Jinv02*w51-7.1225649350649346e-04*Jinv22*w56+7.8530844155844158e-04*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10+3.1777597402597403e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20+4.2735389610389614e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[19][38]=(0.0)*D;
  A[19][39]=(0.0)*D;
  A[19][40]=(-1.1505681818181819e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+7.1225649350649346e-04*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-3.4699675324675324e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+2.6298701298701297e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+3.4699675324675324e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2+3.9448051948051946e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+2.5750811688311690e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55+7.3051948051948052e-05*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961042e-05*w48*Jinv00+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+1.7532467532467534e-03*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+4.2004870129870131e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+2.6298701298701297e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00+1.6436688311688311e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+3.8352272727272725e-04*Jinv11*w1-1.2053571428571428e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-7.6704545454545450e-04*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00-8.2183441558441553e-04*w32*Jinv22+1.6436688311688311e-03*Jinv01*w39+3.4699675324675324e-04*Jinv22*w2-3.2873376623376621e-03*Jinv00*w18-2.1915584415584414e-03*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45+1.7532467532467534e-03*Jinv01*w24-3.4699675324675324e-04*Jinv22*w59-2.1915584415584414e-03*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00+1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-3.2873376623376621e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-8.2183441558441553e-04*w32*Jinv21-2.1915584415584414e-03*Jinv21*w20-1.0957792207792207e-03*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+2.1367694805194807e-03*w26*Jinv20+9.8620129870129864e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-8.2183441558441553e-04*w31*Jinv11+2.5750811688311690e-03*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29+4.9310064935064934e-03*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12-2.1915584415584414e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11-1.0957792207792207e-03*w7*Jinv10+2.5750811688311690e-03*w47*Jinv21-2.1915584415584414e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+3.8352272727272725e-04*w52*Jinv11-8.2183441558441553e-04*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10+8.2183441558441553e-04*Jinv22*w50+1.2601461038961039e-03*w22*Jinv11+1.0957792207792207e-03*Jinv12*w7+1.2601461038961039e-03*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51-8.2183441558441553e-04*Jinv02*w30+8.2183441558441553e-04*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10+9.8620129870129864e-04*w9*Jinv00-2.1915584415584414e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00+1.0957792207792207e-03*w6*Jinv02-7.1225649350649346e-04*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-1.2053571428571428e-03*Jinv21*w14+9.8620129870129864e-04*w10*Jinv10+3.9448051948051946e-03*Jinv20*w44-3.4699675324675324e-04*w58*Jinv12-1.2053571428571428e-03*Jinv01*w12+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59+4.2004870129870131e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.2601461038961039e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.2004870129870131e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00+8.2183441558441553e-04*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-3.2873376623376621e-03*w20*Jinv20-1.3149350649350649e-03*Jinv22*w14+1.6436688311688311e-03*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-8.2183441558441553e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+1.7532467532467534e-03*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+3.9448051948051946e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-2.5568181818181820e-04*Jinv01*w27+3.4699675324675324e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[19][41]=(0.0)*D;
  A[19][42]=(0.0)*D;
  A[19][43]=(1.1505681818181819e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12-2.6298701298701297e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+5.4788961038961036e-04*w57*Jinv02+1.1505681818181819e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-4.9310064935064934e-03*w25*Jinv12+5.4788961038961036e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-7.3051948051948052e-05*w58*Jinv10+2.9586038961038961e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-5.4788961038961042e-05*w48*Jinv00-4.9310064935064934e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01-4.2004870129870131e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10+1.2820616883116883e-02*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+2.9586038961038961e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10-2.6298701298701297e-03*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+5.4788961038961036e-04*Jinv22*w2+3.2873376623376621e-03*Jinv00*w18+2.1915584415584414e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-2.1367694805194807e-03*Jinv01*w24+5.4788961038961036e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11-4.9310064935064934e-03*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13+3.2873376623376621e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-9.8620129870129864e-04*w11*Jinv20-2.6298701298701297e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-4.9310064935064934e-03*Jinv22*w26-7.6704545454545450e-04*w46*Jinv11+1.1505681818181819e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54-2.6298701298701297e-03*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-5.4788961038961042e-05*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-4.9310064935064934e-03*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12+4.2735389610389614e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-7.6704545454545450e-04*w47*Jinv21+2.1915584415584414e-03*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12-7.6704545454545450e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11+4.2735389610389614e-03*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+4.2735389610389614e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-9.8620129870129864e-04*w9*Jinv00+2.1915584415584414e-03*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+6.5746753246753243e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+1.2820616883116883e-02*w19*Jinv12-2.6298701298701297e-03*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-9.8620129870129864e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+5.4788961038961036e-04*w58*Jinv12+1.2820616883116883e-02*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-4.2004870129870131e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.2004870129870131e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-4.9310064935064934e-03*w40*Jinv10+3.2873376623376621e-03*w20*Jinv20+6.5746753246753243e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+4.2735389610389614e-03*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-2.6298701298701297e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+6.5746753246753243e-04*Jinv12*w28-2.1367694805194807e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-3.9448051948051946e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+2.9586038961038961e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01+5.4788961038961036e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[19][44]=(0.0)*D;
  A[19][45]=(0.0)*D;
  A[19][46]=(-1.3149350649350649e-03*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-4.2918019480519480e-04*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39-2.7394480519480520e-03*Jinv21*w44+2.4655032467532466e-04*Jinv01*w3-1.6436688311688311e-04*Jinv22*w38-1.7349837662337662e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-3.1960227272727273e-04*Jinv21*w2-2.7394480519480520e-03*w42*Jinv01-4.8396915584415583e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.0271915584415584e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55-2.1367694805194807e-03*w15*Jinv02-4.7483766233766234e-04*w10*Jinv12-2.6298701298701297e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01-1.2327516233766234e-03*Jinv21*w26-3.1960227272727273e-04*w0*Jinv01+1.1505681818181819e-03*Jinv22*w20-2.7394480519480520e-03*w43*Jinv11-2.5202922077922079e-03*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-4.7483766233766234e-04*w11*Jinv22-3.1960227272727273e-04*Jinv11*w1+8.4922889610389610e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-2.1367694805194807e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-2.5202922077922079e-03*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+3.8352272727272725e-04*Jinv21*w38-2.3833198051948050e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-4.2918019480519480e-04*Jinv22*w59-7.1225649350649346e-04*w34*Jinv11+1.4793019480519481e-03*w19*Jinv11+1.3697240259740259e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.4655032467532466e-04*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+4.6570616883116886e-04*w32*Jinv21+1.4793019480519481e-03*Jinv21*w20-1.6436688311688311e-04*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41+5.2049512987012989e-04*w52*Jinv12-7.1225649350649346e-04*Jinv21*w35+4.6570616883116886e-04*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-2.0271915584415584e-03*w46*Jinv11+2.4655032467532466e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+1.4793019480519481e-03*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11-4.7483766233766234e-04*Jinv02*w9+1.9176136363636362e-04*w6*Jinv01-2.0271915584415584e-03*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23+1.3697240259740259e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-2.3833198051948050e-03*w46*Jinv12-1.6436688311688311e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-1.0135957792207792e-03*w22*Jinv11-1.6436688311688311e-04*Jinv12*w7-1.0135957792207792e-03*w21*Jinv01+1.3697240259740259e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01+1.9176136363636362e-04*w7*Jinv11+4.3831168831168834e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02+1.1505681818181819e-03*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-2.3833198051948050e-03*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+8.4922889610389610e-04*Jinv21*w14-4.2918019480519480e-04*w58*Jinv12+1.1505681818181819e-03*Jinv02*w18+8.4922889610389610e-04*Jinv01*w12+1.9176136363636362e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.0135957792207792e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02-4.8396915584415583e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-2.5202922077922079e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+4.6570616883116886e-04*Jinv01*w30+5.2049512987012989e-04*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11-2.1367694805194807e-03*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-4.8396915584415583e-04*w10*Jinv11)*D;
  A[19][47]=(0.0)*D;
  A[19][48]=(0.0)*D;
  A[19][49]=(-8.2183441558441553e-05*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10-8.2183441558441553e-04*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10-8.2183441558441553e-04*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480521e-05*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+3.5612824675324673e-04*w48*Jinv00-3.2873376623376621e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+7.6704545454545450e-04*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27+1.0957792207792208e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10+2.4655032467532466e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-2.7394480519480521e-05*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-2.7394480519480521e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-3.2873376623376621e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+4.6570616883116886e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-8.2183441558441553e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-5.2049512987012989e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+4.6570616883116886e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.7394480519480521e-05*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10-2.7394480519480518e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.7394480519480521e-05*Jinv02*w30+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+2.4655032467532466e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-8.2183441558441553e-05*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-3.2873376623376621e-04*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-9.1314935064935065e-06*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+4.6570616883116886e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-5.2049512987012989e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-2.7394480519480521e-05*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[19][50]=(0.0)*D;
  A[19][51]=(0.0)*D;
  A[19][52]=(8.2183441558441553e-05*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12+5.7528409090909093e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-2.1002435064935065e-04*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+1.4793019480519481e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+1.4793019480519481e-03*Jinv02*w24-8.2183441558441553e-04*Jinv22*w35-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-4.7666396103896100e-03*Jinv22*w20+5.2049512987012989e-04*w45*Jinv00-1.6436688311688311e-03*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-3.5612824675324673e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+2.4655032467532466e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-7.1225649350649346e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+1.3697240259740259e-04*Jinv02*w45-2.1002435064935065e-04*Jinv22*w59-1.6436688311688311e-04*w51*Jinv00-8.2183441558441553e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-5.4788961038961042e-05*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13-7.1225649350649346e-04*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+5.2049512987012989e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20+5.7528409090909093e-04*w52*Jinv12+1.4793019480519481e-03*Jinv22*w26+8.2183441558441553e-05*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-5.4788961038961042e-05*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-1.6436688311688311e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23+2.4655032467532466e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+2.4655032467532466e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-1.6436688311688311e-04*w52*Jinv10+9.1314935064935065e-06*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+5.2049512987012989e-04*w24*Jinv00-3.1047077922077924e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-4.7666396103896100e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+9.1314935064935065e-06*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44-2.1002435064935065e-04*w58*Jinv12-4.7666396103896100e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-8.2183441558441553e-04*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+5.2049512987012989e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-7.1225649350649346e-04*w20*Jinv20-3.1047077922077924e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-2.4655032467532466e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+5.7528409090909093e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56-3.1047077922077924e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[19][53]=(0.0)*D;
  A[19][54]=(0.0)*D;
  A[19][55]=(-1.3149350649350649e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53+1.2784090909090910e-04*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+3.8352272727272725e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+1.4793019480519481e-03*w25*Jinv12-2.1915584415584417e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-1.0683847402597404e-03*Jinv12*w55-1.6436688311688311e-04*w15*Jinv02+1.8262987012987013e-05*w10*Jinv12+1.4793019480519481e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-5.4788961038961042e-05*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02+1.8262987012987013e-05*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-1.6436688311688311e-04*Jinv22*w17+1.3971185064935066e-03*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+4.6570616883116886e-04*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-2.1915584415584417e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+5.2049512987012989e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+2.4655032467532466e-04*Jinv12*w13+2.7394480519480518e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20+3.8352272727272725e-04*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-3.8352272727272725e-04*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11+1.4793019480519481e-03*Jinv22*w26+5.2049512987012989e-04*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21-1.0683847402597404e-03*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+1.3971185064935066e-03*w21*Jinv02-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33+2.4655032467532466e-04*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+1.8262987012987013e-05*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+5.2049512987012989e-04*w47*Jinv21+1.3971185064935066e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+4.6570616883116886e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+4.6570616883116886e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-7.1225649350649346e-04*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.2784090909090910e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-1.0683847402597404e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28+5.2049512987012989e-04*w25*Jinv11-1.6436688311688311e-04*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-2.1915584415584417e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[19][56]=(0.0)*D;
  A[19][57]=(0.0)*D;
  A[19][58]=(3.4699675324675324e-04*w40*Jinv12-2.6481331168831169e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+3.4699675324675324e-04*Jinv02*w39+3.6525974025974026e-05*Jinv22*w38-5.8441558441558442e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-1.0957792207792208e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-5.8441558441558442e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35+1.4975649350649350e-03*Jinv22*w20+4.3831168831168834e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12-4.2918019480519480e-04*Jinv02*w3-2.7394480519480521e-05*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+1.1870941558441559e-04*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+2.1915584415584417e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-2.1915584415584417e-04*Jinv12*w13+3.6525974025974026e-05*w37*Jinv12+3.4699675324675324e-04*Jinv22*w41-2.6481331168831169e-04*w52*Jinv12-5.8441558441558442e-04*Jinv22*w26-1.0957792207792208e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02-2.1915584415584417e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9-4.2918019480519480e-04*Jinv22*w23-2.7394480519480521e-05*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12+7.4878246753246752e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-2.7394480519480521e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.4204545454545455e-05*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+1.4975649350649350e-03*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12+1.4975649350649350e-03*Jinv02*w18+2.1915584415584417e-04*Jinv02*w33-5.4788961038961042e-05*w48*Jinv02-1.4204545454545455e-05*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+3.6525974025974026e-05*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-2.6481331168831169e-04*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[19][59]=(0.0)*D;
  A[20][0]=(0.0)*D;
  A[20][1]=(0.0)*D;
  A[20][2]=(1.0957792207792208e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-3.4699675324675324e-04*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10-3.4699675324675324e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+2.6481331168831169e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20+2.6481331168831169e-04*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38+2.1915584415584417e-04*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59+5.8441558441558442e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00+5.8441558441558442e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+5.8441558441558442e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10-1.4975649350649350e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-3.4699675324675324e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+2.6481331168831169e-04*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-3.4699675324675324e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2-1.4975649350649350e-03*Jinv00*w18-7.4878246753246752e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38+4.2918019480519480e-04*Jinv02*w45+5.8441558441558442e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-2.1915584415584417e-04*w34*Jinv11-1.4975649350649350e-03*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+2.6481331168831169e-04*w4*Jinv10+2.6481331168831169e-04*w4*Jinv11-3.4699675324675324e-04*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13-1.4975649350649350e-03*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-1.4975649350649350e-03*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00-7.4878246753246752e-04*w37*Jinv12+2.1915584415584417e-04*Jinv20*w56-3.4699675324675324e-04*Jinv22*w41+5.8441558441558442e-04*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-2.1915584415584417e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+5.8441558441558442e-04*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+2.6481331168831169e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54+2.6481331168831169e-04*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-2.1915584415584417e-04*Jinv01*w33-3.4699675324675324e-04*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12-1.4975649350649350e-03*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21-7.4878246753246752e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+2.7394480519480521e-05*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-7.4878246753246752e-04*w37*Jinv10+2.1915584415584417e-04*Jinv21*w56+5.8441558441558442e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27-3.6525974025974026e-05*w6*Jinv02-1.4975649350649350e-03*w19*Jinv12+2.6481331168831169e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-1.4975649350649350e-03*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23+2.1915584415584417e-04*Jinv01*w54+2.1915584415584417e-04*w55*Jinv11+5.8441558441558442e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-3.4699675324675324e-04*w40*Jinv10-1.4975649350649350e-03*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14-3.4699675324675324e-04*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+2.6481331168831169e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+5.8441558441558442e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[20][3]=(0.0)*D;
  A[20][4]=(0.0)*D;
  A[20][5]=(-5.4788961038961036e-04*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-5.4788961038961036e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-7.3965097402597403e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-5.7528409090909093e-04*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-4.6570616883116886e-04*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.4793019480519481e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.5750811688311690e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+1.6436688311688311e-03*w16*Jinv11+1.6436688311688311e-03*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+3.1960227272727273e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00-1.4793019480519481e-03*Jinv02*w24+8.2183441558441553e-04*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.4793019480519481e-03*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+8.7662337662337668e-04*w16*Jinv10+4.7666396103896100e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+1.6436688311688311e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+4.3831168831168834e-04*Jinv00*w33-1.3697240259740259e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.7528409090909093e-04*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2+4.0543831168831168e-03*Jinv00*w18+2.0271915584415584e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-1.4793019480519481e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+8.2183441558441553e-04*w34*Jinv11+4.7666396103896100e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-1.1231737012987013e-03*w51*Jinv00+8.2183441558441553e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-7.3965097402597403e-04*w4*Jinv10-5.7528409090909093e-04*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13+4.0543831168831168e-03*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21+4.7666396103896100e-03*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-4.6570616883116886e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-9.5880681818181818e-04*w26*Jinv20+5.5702110389610384e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+8.2183441558441553e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-1.4793019480519481e-03*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-5.4788961038961036e-04*Jinv20*w14-5.7528409090909093e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54-5.7528409090909093e-04*w4*Jinv12-1.3697240259740259e-04*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+8.2183441558441553e-04*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12+4.7666396103896100e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+2.0271915584415584e-03*Jinv20*w38-1.3697240259740259e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12-1.1231737012987013e-03*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-1.1231737012987013e-03*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11+5.5702110389610384e-04*w9*Jinv00+2.0271915584415584e-03*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-9.5880681818181818e-04*w24*Jinv00+3.1047077922077924e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+4.7666396103896100e-03*w19*Jinv12-5.7528409090909093e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+5.5702110389610384e-04*w10*Jinv10+2.5750811688311690e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+4.7666396103896100e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8+3.1960227272727273e-04*w57*Jinv00+3.1960227272727273e-04*Jinv20*w59+8.2183441558441553e-04*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-9.5880681818181818e-04*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+8.7662337662337668e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+8.7662337662337668e-04*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+4.0543831168831168e-03*w20*Jinv20+3.1047077922077924e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-7.3965097402597403e-04*w5*Jinv20+8.2183441558441553e-05*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+3.1047077922077924e-04*Jinv12*w28-1.4793019480519481e-03*w25*Jinv11-4.6570616883116886e-04*w55*Jinv10+2.5750811688311690e-03*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[20][6]=(0.0)*D;
  A[20][7]=(0.0)*D;
  A[20][8]=(5.4788961038961036e-04*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12+2.3833198051948050e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+1.1505681818181819e-03*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.3833198051948050e-03*Jinv01*w3-3.6982548701298703e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-7.4878246753246752e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-2.6846590909090908e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-4.2735389610389614e-03*w16*Jinv11-4.2735389610389614e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-3.7439123376623376e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12+2.6298701298701297e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-7.4878246753246752e-04*w58*Jinv11-7.4878246753246752e-04*w57*Jinv01+2.6298701298701297e-03*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-3.5612824675324673e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10-7.3965097402597406e-03*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+8.2183441558441553e-05*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11-4.2735389610389614e-03*w15*Jinv01-7.8530844155844158e-04*w28*Jinv11-3.9265422077922079e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-5.4788961038961036e-04*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+2.3833198051948050e-03*Jinv02*w3+3.5612824675324673e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-2.4655032467532467e-03*w36*Jinv00-3.6982548701298703e-03*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45+2.6298701298701297e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11-7.3965097402597406e-03*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+1.1505681818181819e-03*w4*Jinv10+2.3833198051948050e-03*w4*Jinv11+7.1225649350649346e-04*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10-5.4788961038961036e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-7.3965097402597406e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00-3.6982548701298703e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.3971185064935066e-03*w26*Jinv20-1.1414366883116883e-03*w11*Jinv20+2.3833198051948050e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14+2.3833198051948050e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+2.3833198051948050e-03*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-7.8530844155844158e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+7.1225649350649346e-04*Jinv02*w12-7.3965097402597406e-03*Jinv01*w18-3.6982548701298703e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-3.6982548701298703e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-2.4655032467532467e-03*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+1.3697240259740260e-03*Jinv20*w53-3.6982548701298703e-03*Jinv22*w8+3.5612824675324673e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11-3.6982548701298703e-03*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-3.6982548701298703e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10-3.6982548701298703e-03*w7*Jinv11-1.1414366883116883e-03*w9*Jinv00-2.4655032467532467e-03*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+1.3971185064935066e-03*w24*Jinv00-7.8530844155844158e-04*Jinv02*w27-3.6982548701298703e-03*w6*Jinv02-7.3965097402597406e-03*w19*Jinv12+2.3833198051948050e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-1.1414366883116883e-03*w10*Jinv10-2.6846590909090908e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12-7.3965097402597406e-03*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12-3.6982548701298703e-03*Jinv21*w8-3.7439123376623376e-04*w57*Jinv00-3.7439123376623376e-04*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-3.5612824675324673e-04*w0*Jinv00-5.4788961038961036e-04*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+1.3971185064935066e-03*w25*Jinv10-3.5612824675324673e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-4.9310064935064934e-03*w20*Jinv20-7.8530844155844158e-04*Jinv22*w29+7.1225649350649346e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-3.6982548701298703e-03*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+1.1505681818181819e-03*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30+2.3833198051948050e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-7.8530844155844158e-04*Jinv12*w28+2.6298701298701297e-03*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10-2.6846590909090908e-03*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-7.8530844155844158e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-7.4878246753246752e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[20][9]=(0.0)*D;
  A[20][10]=(0.0)*D;
  A[20][11]=(-1.0957792207792208e-04*Jinv00*w12+2.0495129870129870e-04*w28*Jinv10+1.1870941558441558e-03*Jinv20*w8-1.0957792207792208e-04*w13*Jinv10-6.7573051948051951e-04*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-2.6481331168831169e-04*Jinv20*w23-1.0957792207792208e-04*Jinv00*w54+6.5746753246753243e-04*w43*Jinv10+1.5726461038961039e-04*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00+1.5726461038961039e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-2.6481331168831169e-04*w45*Jinv00+2.0495129870129870e-04*Jinv00*w27-5.4788961038961042e-05*Jinv20*w50+2.0495129870129870e-04*Jinv20*w29-2.6481331168831169e-04*w46*Jinv10+3.2873376623376621e-04*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10-2.6481331168831169e-04*w21*Jinv00+2.3741883116883116e-03*Jinv00*w18+1.1870941558441558e-03*w36*Jinv00-6.7573051948051951e-04*w51*Jinv00-6.7573051948051951e-04*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+2.3741883116883116e-03*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+1.1870941558441558e-03*w6*Jinv00-1.0957792207792208e-04*Jinv20*w56-1.0227272727272728e-03*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-1.0957792207792208e-04*Jinv20*w14-5.4788961038961042e-05*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+1.1870941558441558e-03*w7*Jinv10+1.1870941558441558e-03*Jinv20*w38-6.7573051948051951e-04*Jinv20*w53-2.6481331168831169e-04*w22*Jinv10-6.7573051948051951e-04*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+1.1870941558441558e-03*w37*Jinv10-1.0227272727272728e-03*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+6.5746753246753243e-04*Jinv20*w44+1.5726461038961039e-04*w57*Jinv00+1.5726461038961039e-04*Jinv20*w59+1.5726461038961039e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-1.0227272727272728e-03*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+6.5746753246753243e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+2.3741883116883116e-03*w20*Jinv20-6.7573051948051951e-04*w5*Jinv20-1.0957792207792208e-04*w55*Jinv10+6.5746753246753243e-04*w42*Jinv00-2.6481331168831169e-04*w47*Jinv20)*D;
  A[20][12]=(0.0)*D;
  A[20][13]=(0.0)*D;
  A[20][14]=(1.0683847402597404e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8+2.1915584415584417e-04*w57*Jinv02+1.0683847402597404e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59-1.4793019480519481e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-1.1231737012987013e-03*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02-1.8262987012987013e-05*w10*Jinv12-4.6570616883116886e-04*w48*Jinv00-1.4793019480519481e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01-9.5880681818181818e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+1.2601461038961039e-03*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27-1.8262987012987013e-05*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01-3.5612824675324673e-04*w28*Jinv11-4.6570616883116886e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+1.2601461038961039e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-1.3971185064935066e-03*Jinv02*w45-9.5880681818181818e-04*Jinv01*w24+2.1915584415584417e-04*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11+1.4245129870129869e-03*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.3149350649350649e-03*w41*Jinv20+1.0683847402597404e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10+1.4245129870129869e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.4793019480519481e-03*w26*Jinv20-1.8262987012987013e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35-1.4793019480519481e-03*Jinv22*w26-1.1231737012987013e-03*w46*Jinv11+1.0683847402597404e-03*Jinv20*w14-2.4655032467532466e-04*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-4.6570616883116886e-04*w49*Jinv10-3.5612824675324673e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+1.0683847402597404e-03*Jinv02*w12+1.4245129870129869e-03*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-1.8262987012987013e-05*Jinv02*w9-5.4788961038961036e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-1.1231737012987013e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-1.3971185064935066e-03*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-4.6570616883116886e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-4.6570616883116886e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-5.4788961038961036e-04*w7*Jinv11-1.8262987012987013e-05*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56-1.4793019480519481e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-1.3971185064935066e-03*w47*Jinv22-1.9176136363636362e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.8262987012987013e-05*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44+2.1915584415584417e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-5.4788961038961036e-04*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.4793019480519481e-03*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-4.6570616883116886e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.0683847402597404e-03*Jinv22*w14+1.2601461038961039e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-5.4788961038961036e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-9.5880681818181818e-04*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-1.9176136363636362e-04*Jinv21*w50-3.5612824675324673e-04*Jinv01*w27-1.9176136363636362e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[20][15]=(0.0)*D;
  A[20][16]=(0.0)*D;
  A[20][17]=(-4.3831168831168834e-04*w28*Jinv10+1.9724025974025973e-03*w40*Jinv12+2.6298701298701297e-03*Jinv22*w53-2.0819805194805196e-03*Jinv20*w8-5.4788961038961036e-04*w57*Jinv02+1.9724025974025973e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+1.8628246753246754e-03*w3*Jinv00+4.9310064935064932e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+4.9310064935064934e-03*w25*Jinv12-5.4788961038961036e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.7532467532467534e-03*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.6793831168831166e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+8.2183441558441553e-04*w48*Jinv00+4.9310064935064934e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+2.7942370129870131e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10-6.9034090909090911e-03*w16*Jinv10-1.2820616883116883e-02*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41-2.9586038961038961e-03*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1-6.0267857142857139e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-1.6436688311688311e-04*w28*Jinv11+8.2183441558441553e-04*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+2.6298701298701297e-03*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39-5.4788961038961036e-04*Jinv22*w2-9.5332792207792200e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+2.7942370129870131e-03*Jinv01*w24-5.4788961038961036e-04*Jinv22*w59-8.5470779220779228e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+2.5750811688311690e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-2.9586038961038961e-03*Jinv22*w44+1.8628246753246754e-03*w4*Jinv10+4.9310064935064932e-04*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-9.5332792207792200e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-3.2873376623376621e-04*w32*Jinv21-8.5470779220779228e-03*Jinv21*w20-2.0819805194805196e-03*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56+1.9724025974025973e-03*Jinv22*w41+2.7942370129870131e-03*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.6298701298701297e-03*w52*Jinv12-3.2873376623376621e-04*w31*Jinv11+4.9310064935064934e-03*Jinv22*w26+1.7532467532467534e-03*w46*Jinv11+4.9310064935064932e-04*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54+2.6298701298701297e-03*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02+8.2183441558441553e-04*w49*Jinv10-1.6436688311688311e-04*Jinv21*w29-2.9586038961038961e-03*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-8.5470779220779228e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+1.0957792207792207e-03*Jinv02*w9-2.0819805194805196e-03*w7*Jinv10+1.7532467532467534e-03*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12+2.5750811688311690e-03*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+2.5750811688311690e-03*w52*Jinv10+1.0957792207792208e-04*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.7942370129870131e-03*w24*Jinv00-6.5746753246753243e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-1.2820616883116883e-02*w19*Jinv12+2.6298701298701297e-03*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-6.0267857142857139e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-5.4788961038961036e-04*w58*Jinv12-1.2820616883116883e-02*Jinv02*w18-6.0267857142857139e-04*Jinv01*w12-9.6793831168831166e-04*w57*Jinv00-9.6793831168831166e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.7942370129870131e-03*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2-6.9034090909090911e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-6.9034090909090911e-03*Jinv20*w17-2.9586038961038961e-03*w40*Jinv10-9.5332792207792200e-03*w20*Jinv20-6.5746753246753243e-04*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-2.9586038961038961e-03*Jinv12*w43+1.8628246753246754e-03*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30+2.6298701298701297e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-6.5746753246753243e-04*Jinv12*w28+2.7942370129870131e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-1.6436688311688311e-04*Jinv01*w27-5.4788961038961036e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[20][18]=(0.0)*D;
  A[20][19]=(0.0)*D;
  A[20][20]=(-1.0683847402597404e-03*Jinv00*w12+6.5746753246753243e-04*w28*Jinv10+2.4655032467532467e-03*Jinv20*w8-1.0683847402597404e-03*w13*Jinv10+4.2735389610389614e-03*Jinv21*w44-2.0271915584415584e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.3833198051948050e-03*Jinv01*w3+5.7528409090909093e-04*Jinv20*w23-1.0683847402597404e-03*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+4.7666396103896100e-03*w43*Jinv10+4.2735389610389614e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+4.2735389610389614e-03*w16*Jinv11+4.2735389610389614e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01-2.6298701298701297e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+4.7666396103896100e-03*w16*Jinv10+4.2735389610389614e-03*w43*Jinv11+5.7528409090909093e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+6.5746753246753243e-04*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+4.2735389610389614e-03*w15*Jinv01+7.8530844155844158e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+6.5746753246753243e-04*Jinv20*w29+5.7528409090909093e-04*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10+5.7528409090909093e-04*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+7.3965097402597406e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00+3.6982548701298703e-03*Jinv21*w38-2.6298701298701297e-03*Jinv01*w24+2.1367694805194807e-03*w34*Jinv11+7.3965097402597406e-03*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-2.0271915584415584e-03*w51*Jinv00-2.0271915584415584e-03*w4*Jinv10-2.3833198051948050e-03*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20+7.3965097402597406e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21+7.3965097402597406e-03*Jinv21*w20+2.4655032467532467e-03*w6*Jinv00-1.0683847402597404e-03*Jinv20*w56-1.3149350649350649e-03*w26*Jinv20-9.1314935064935068e-05*w11*Jinv20+2.1367694805194807e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11-1.0683847402597404e-03*Jinv20*w14-2.3833198051948050e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+7.8530844155844158e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00+7.3965097402597406e-03*Jinv01*w18+3.6982548701298703e-03*w37*Jinv11+3.6982548701298703e-03*w6*Jinv01+2.4655032467532467e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+2.4655032467532467e-03*Jinv20*w38-2.3833198051948050e-03*w52*Jinv11-2.0271915584415584e-03*Jinv20*w53+5.7528409090909093e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+3.6982548701298703e-03*w36*Jinv01-2.0271915584415584e-03*w52*Jinv10+3.6982548701298703e-03*w7*Jinv11-9.1314935064935068e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-9.1314935064935068e-05*w10*Jinv10+4.7666396103896100e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12+3.6982548701298703e-03*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+4.7666396103896100e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+4.7666396103896100e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+7.3965097402597406e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-2.0271915584415584e-03*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30-2.6298701298701297e-03*w25*Jinv11-1.0683847402597404e-03*w55*Jinv10+4.7666396103896100e-03*w42*Jinv00+5.7528409090909093e-04*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50+7.8530844155844158e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[20][21]=(0.0)*D;
  A[20][22]=(0.0)*D;
  A[20][23]=(-1.3971185064935066e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-3.8352272727272725e-04*Jinv01*w3+2.3833198051948050e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21-6.0267857142857139e-04*w16*Jinv11-6.0267857142857139e-04*Jinv21*w17+1.3697240259740260e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+4.7483766233766234e-04*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10-1.1505681818181819e-03*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-1.2053571428571428e-03*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.7483766233766234e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-9.5880681818181818e-04*w13*Jinv11-6.0267857142857139e-04*w15*Jinv01+6.6659902597402602e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.3833198051948050e-03*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+2.3833198051948050e-03*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-1.2053571428571428e-03*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2-1.1505681818181819e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+1.3971185064935066e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11+3.2873376623376621e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-3.8352272727272725e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-1.1505681818181819e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-2.7394480519480521e-05*w32*Jinv21+3.2873376623376621e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+4.7483766233766234e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-2.7394480519480521e-05*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.3697240259740260e-03*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14-3.8352272727272725e-04*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+2.3833198051948050e-03*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+6.6659902597402602e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12+3.2873376623376621e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11+4.7483766233766234e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10+1.3697240259740260e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+2.3833198051948050e-03*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+2.3833198051948050e-03*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11+4.7483766233766234e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02-1.1505681818181819e-03*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22+1.9176136363636362e-04*w49*Jinv11-9.5880681818181818e-04*Jinv21*w14+4.7483766233766234e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.1505681818181819e-03*Jinv02*w18-9.5880681818181818e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10-1.1505681818181819e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14-1.2053571428571428e-03*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-2.7394480519480521e-05*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.3971185064935066e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12+1.9176136363636362e-04*Jinv21*w50+6.6659902597402602e-04*Jinv01*w27+1.9176136363636362e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[20][24]=(0.0)*D;
  A[20][25]=(0.0)*D;
  A[20][26]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-6.5746753246753243e-04*Jinv21*w44+5.2049512987012989e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-1.5523538961038962e-04*Jinv21*w59-1.5523538961038962e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-1.5523538961038962e-04*w58*Jinv11-1.5523538961038962e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26-1.5523538961038962e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10-6.5746753246753243e-04*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.5523538961038962e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01-1.4610389610389610e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.1505681818181819e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38-1.3149350649350649e-03*Jinv01*w24-8.2183441558441553e-04*w34*Jinv11+3.7804383116883116e-03*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+5.2049512987012989e-04*w51*Jinv00+5.2049512987012989e-04*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20+1.1505681818181819e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21+3.7804383116883116e-03*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.6298701298701297e-03*w26*Jinv20-4.7483766233766234e-04*w11*Jinv20-8.2183441558441553e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.2327516233766234e-03*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10-1.4610389610389610e-04*Jinv21*w29-8.2183441558441553e-04*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00+3.7804383116883116e-03*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11-2.4655032467532466e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+8.2183441558441553e-05*w52*Jinv11+5.2049512987012989e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-1.2327516233766234e-03*w22*Jinv11-1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51-2.4655032467532466e-04*w36*Jinv01+5.2049512987012989e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11-4.7483766233766234e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+4.3831168831168834e-04*Jinv21*w56-2.6298701298701297e-03*w24*Jinv00+2.4655032467532466e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-4.7483766233766234e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+4.3831168831168834e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11-2.6298701298701297e-03*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00-1.0957792207792207e-03*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10+1.1505681818181819e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11+5.2049512987012989e-04*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30-1.3149350649350649e-03*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+2.4655032467532466e-04*Jinv21*w50-1.4610389610389610e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-1.0957792207792207e-03*w10*Jinv11)*D;
  A[20][27]=(0.0)*D;
  A[20][28]=(0.0)*D;
  A[20][29]=(-3.2873376623376621e-04*Jinv21*w44+1.1870941558441559e-04*Jinv01*w3-1.1668019480519480e-04*Jinv21*w59-1.1668019480519480e-04*Jinv21*w2-3.2873376623376621e-04*w42*Jinv01+8.3198051948051951e-05*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17-6.7573051948051951e-04*Jinv01*w45-1.1668019480519480e-04*w58*Jinv11-1.1668019480519480e-04*w57*Jinv01-1.0227272727272728e-03*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-3.2873376623376621e-04*w43*Jinv11+2.9220779220779221e-04*Jinv21*w41-1.1668019480519480e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11+2.9220779220779221e-04*Jinv01*w39+3.6525974025974026e-05*Jinv21*w38-1.0227272727272728e-03*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11-2.5568181818181820e-04*w19*Jinv11+1.1870941558441559e-04*Jinv21*w53+1.1870941558441559e-04*w4*Jinv11-2.7394480519480521e-05*w32*Jinv21-2.5568181818181820e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35-2.7394480519480521e-05*w31*Jinv11-6.7573051948051951e-04*w46*Jinv11+1.1870941558441559e-04*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-2.5568181818181820e-04*Jinv01*w18+3.6525974025974026e-05*w37*Jinv11+3.6525974025974026e-05*w6*Jinv01-6.7573051948051951e-04*w47*Jinv21+1.1870941558441559e-04*w52*Jinv11-6.7573051948051951e-04*w22*Jinv11-6.7573051948051951e-04*w21*Jinv01+1.1870941558441559e-04*Jinv01*w51+3.6525974025974026e-05*w36*Jinv01+3.6525974025974026e-05*w7*Jinv11+4.9310064935064932e-04*Jinv21*w56-2.7394480519480521e-05*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14+4.9310064935064932e-04*Jinv01*w12+3.6525974025974026e-05*Jinv21*w8-6.7573051948051951e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+8.3198051948051951e-05*Jinv01*w9+2.9220779220779221e-04*w40*Jinv11-2.7394480519480521e-05*Jinv01*w30-1.0227272727272728e-03*w25*Jinv11-2.7394480519480521e-05*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27-2.7394480519480521e-05*w48*Jinv01+8.3198051948051951e-05*w10*Jinv11)*D;
  A[20][30]=(0.0)*D;
  A[20][31]=(0.0)*D;
  A[20][32]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+8.2183441558441553e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-1.9176136363636362e-04*Jinv20*w8+2.7394480519480521e-05*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+8.2183441558441553e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-5.4788961038961042e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-4.6570616883116886e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02-2.4655032467532466e-04*w48*Jinv00+3.2873376623376621e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+1.0957792207792208e-04*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+8.7662337662337668e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-6.3920454545454550e-05*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+8.7662337662337668e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+2.7394480519480521e-05*Jinv22*w59+7.1225649350649346e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11+8.7662337662337668e-04*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20-1.9176136363636362e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+8.2183441558441553e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+7.1225649350649346e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11-5.4788961038961036e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21-4.6570616883116886e-04*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+7.1225649350649346e-04*Jinv01*w33+8.7662337662337668e-04*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-1.9176136363636362e-04*w6*Jinv01-1.9176136363636362e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11-2.7394480519480518e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51+2.7394480519480521e-05*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10-1.9176136363636362e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02-8.2183441558441553e-05*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.7394480519480521e-05*w58*Jinv12+3.5612824675324673e-04*Jinv01*w12-1.9176136363636362e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+8.7662337662337668e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.5612824675324673e-04*Jinv22*w14+8.7662337662337668e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-4.6570616883116886e-04*Jinv22*w56-5.2049512987012989e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[20][33]=(0.0)*D;
  A[20][34]=(0.0)*D;
  A[20][35]=(-5.2962662337662338e-04*w28*Jinv10+8.2183441558441553e-04*Jinv22*w53-1.0409902597402598e-03*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02-1.9724025974025973e-03*Jinv21*w44+9.3141233766233771e-04*w3*Jinv00+1.3149350649350649e-03*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+2.1915584415584417e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+3.2873376623376621e-04*w48*Jinv00-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-1.9724025974025973e-03*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12-8.2183441558441553e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.7532467532467534e-03*w51*Jinv00-9.8620129870129864e-04*Jinv22*w44+9.3141233766233771e-04*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-1.0409902597402598e-03*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+1.7532467532467534e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+8.2183441558441553e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+7.6704545454545450e-04*w46*Jinv11+1.3149350649350649e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-8.2183441558441553e-04*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11-2.1367694805194807e-03*w6*Jinv01-1.0409902597402598e-03*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+1.7532467532467534e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+1.7532467532467534e-03*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56+1.7532467532467534e-03*w24*Jinv00+2.1367694805194807e-03*w6*Jinv02-8.2183441558441553e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-3.1047077922077924e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11+1.7532467532467534e-03*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-7.1225649350649346e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+9.3141233766233771e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+8.2183441558441553e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+2.1367694805194807e-03*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[20][36]=(0.0)*D;
  A[20][37]=(0.0)*D;
  A[20][38]=(-3.5612824675324673e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.3833198051948050e-03*Jinv22*w53+1.2327516233766234e-03*Jinv20*w8+7.4878246753246752e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-1.0135957792207792e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23+3.6982548701298703e-03*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-2.6298701298701297e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+3.1777597402597403e-03*w43*Jinv10-7.1225649350649346e-04*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10+4.2735389610389614e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+3.7439123376623376e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+7.3965097402597406e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00+4.2735389610389614e-03*w42*Jinv02+3.9265422077922079e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22-8.2183441558441553e-05*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+4.2735389610389614e-03*Jinv22*w17+1.5888798701298701e-03*Jinv00*w33-1.6436688311688311e-04*w22*Jinv12-3.5612824675324673e-04*w31*Jinv10-2.3833198051948050e-03*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2+2.4655032467532467e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45+7.4878246753246752e-04*Jinv22*w59-1.2327516233766234e-03*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-1.0135957792207792e-03*w4*Jinv10+1.0957792207792207e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10+1.2327516233766234e-03*w6*Jinv00+3.6982548701298703e-03*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20-2.3833198051948050e-03*w52*Jinv12-2.6298701298701297e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-7.1225649350649346e-04*Jinv02*w54-2.3833198051948050e-03*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+1.0957792207792207e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+1.2327516233766234e-03*w7*Jinv10+2.4655032467532467e-03*Jinv20*w38-1.6436688311688311e-04*Jinv22*w23-3.5612824675324673e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12-1.2327516233766234e-03*Jinv20*w53+3.6982548701298703e-03*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+3.6982548701298703e-03*Jinv12*w7-3.5612824675324673e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.2327516233766234e-03*w52*Jinv10-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.2327516233766234e-03*w24*Jinv00+7.8530844155844158e-04*Jinv02*w27+3.6982548701298703e-03*w6*Jinv02+7.3965097402597406e-03*w19*Jinv12-2.3833198051948050e-03*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-4.5657467532467534e-05*w10*Jinv10+3.1777597402597403e-03*Jinv20*w44+7.4878246753246752e-04*w58*Jinv12+7.3965097402597406e-03*Jinv02*w18+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33+3.7439123376623376e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-1.2327516233766234e-03*w25*Jinv10+3.7439123376623376e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+7.8530844155844158e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+3.6982548701298703e-03*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-1.0135957792207792e-03*w5*Jinv20-2.3833198051948050e-03*Jinv02*w51-7.1225649350649346e-04*Jinv22*w56+7.8530844155844158e-04*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10+3.1777597402597403e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20+4.2735389610389614e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[20][39]=(0.0)*D;
  A[20][40]=(0.0)*D;
  A[20][41]=(-1.1505681818181819e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+7.1225649350649346e-04*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-3.4699675324675324e-04*w57*Jinv02-1.1505681818181819e-03*w13*Jinv10+2.6298701298701297e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+3.4699675324675324e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2+3.9448051948051946e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+2.5750811688311690e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55+7.3051948051948052e-05*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+5.4788961038961042e-05*w48*Jinv00+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+1.7532467532467534e-03*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+4.2004870129870131e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+2.6298701298701297e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00+1.6436688311688311e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+3.8352272727272725e-04*Jinv11*w1-1.2053571428571428e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-7.6704545454545450e-04*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00-8.2183441558441553e-04*w32*Jinv22+1.6436688311688311e-03*Jinv01*w39+3.4699675324675324e-04*Jinv22*w2-3.2873376623376621e-03*Jinv00*w18-2.1915584415584414e-03*w36*Jinv00-1.0957792207792207e-03*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45+1.7532467532467534e-03*Jinv01*w24-3.4699675324675324e-04*Jinv22*w59-2.1915584415584414e-03*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00+1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-3.2873376623376621e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-8.2183441558441553e-04*w32*Jinv21-2.1915584415584414e-03*Jinv21*w20-1.0957792207792207e-03*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+2.1367694805194807e-03*w26*Jinv20+9.8620129870129864e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-8.2183441558441553e-04*w31*Jinv11+2.5750811688311690e-03*w46*Jinv11-1.1505681818181819e-03*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29+4.9310064935064934e-03*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12-2.1915584415584414e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11-1.0957792207792207e-03*w7*Jinv10+2.5750811688311690e-03*w47*Jinv21-2.1915584415584414e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+3.8352272727272725e-04*w52*Jinv11-8.2183441558441553e-04*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10+8.2183441558441553e-04*Jinv22*w50+1.2601461038961039e-03*w22*Jinv11+1.0957792207792207e-03*Jinv12*w7+1.2601461038961039e-03*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51-8.2183441558441553e-04*Jinv02*w30+8.2183441558441553e-04*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10+9.8620129870129864e-04*w9*Jinv00-2.1915584415584414e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00+1.0957792207792207e-03*w6*Jinv02-7.1225649350649346e-04*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-1.2053571428571428e-03*Jinv21*w14+9.8620129870129864e-04*w10*Jinv10+3.9448051948051946e-03*Jinv20*w44-3.4699675324675324e-04*w58*Jinv12-1.2053571428571428e-03*Jinv01*w12+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59+4.2004870129870131e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.2601461038961039e-03*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.2004870129870131e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00+8.2183441558441553e-04*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-3.2873376623376621e-03*w20*Jinv20-1.3149350649350649e-03*Jinv22*w14+1.6436688311688311e-03*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-8.2183441558441553e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+1.7532467532467534e-03*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+3.9448051948051946e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12-2.5568181818181820e-04*Jinv01*w27+3.4699675324675324e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[20][42]=(0.0)*D;
  A[20][43]=(0.0)*D;
  A[20][44]=(1.1505681818181819e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12-2.6298701298701297e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+5.4788961038961036e-04*w57*Jinv02+1.1505681818181819e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59-4.9310064935064934e-03*w25*Jinv12+5.4788961038961036e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-7.3051948051948052e-05*w58*Jinv10+2.9586038961038961e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-5.4788961038961042e-05*w48*Jinv00-4.9310064935064934e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01-4.2004870129870131e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10+1.2820616883116883e-02*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+2.9586038961038961e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10-2.6298701298701297e-03*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+5.4788961038961036e-04*Jinv22*w2+3.2873376623376621e-03*Jinv00*w18+2.1915584415584414e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-2.1367694805194807e-03*Jinv01*w24+5.4788961038961036e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11-4.9310064935064934e-03*w41*Jinv20-1.6436688311688311e-04*Jinv12*w13+3.2873376623376621e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-9.8620129870129864e-04*w11*Jinv20-2.6298701298701297e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-4.9310064935064934e-03*Jinv22*w26-7.6704545454545450e-04*w46*Jinv11+1.1505681818181819e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54-2.6298701298701297e-03*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-5.4788961038961042e-05*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-4.9310064935064934e-03*w39*Jinv00-1.6436688311688311e-04*Jinv02*w12+4.2735389610389614e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-7.6704545454545450e-04*w47*Jinv21+2.1915584415584414e-03*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12-7.6704545454545450e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11+4.2735389610389614e-03*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+4.2735389610389614e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-9.8620129870129864e-04*w9*Jinv00+2.1915584415584414e-03*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+6.5746753246753243e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+1.2820616883116883e-02*w19*Jinv12-2.6298701298701297e-03*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-9.8620129870129864e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+5.4788961038961036e-04*w58*Jinv12+1.2820616883116883e-02*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-4.2004870129870131e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.2004870129870131e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-5.4788961038961036e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-4.9310064935064934e-03*w40*Jinv10+3.2873376623376621e-03*w20*Jinv20+6.5746753246753243e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+4.2735389610389614e-03*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-2.6298701298701297e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+6.5746753246753243e-04*Jinv12*w28-2.1367694805194807e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-3.9448051948051946e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+2.9586038961038961e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01+5.4788961038961036e-04*Jinv12*w1-5.4788961038961036e-04*w10*Jinv11)*D;
  A[20][45]=(0.0)*D;
  A[20][46]=(0.0)*D;
  A[20][47]=(-1.3149350649350649e-03*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-4.2918019480519480e-04*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39-2.7394480519480520e-03*Jinv21*w44+2.4655032467532466e-04*Jinv01*w3-1.6436688311688311e-04*Jinv22*w38-1.7349837662337662e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-3.1960227272727273e-04*Jinv21*w2-2.7394480519480520e-03*w42*Jinv01-4.8396915584415583e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-2.0271915584415584e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55-2.1367694805194807e-03*w15*Jinv02-4.7483766233766234e-04*w10*Jinv12-2.6298701298701297e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01-1.2327516233766234e-03*Jinv21*w26-3.1960227272727273e-04*w0*Jinv01+1.1505681818181819e-03*Jinv22*w20-2.7394480519480520e-03*w43*Jinv11-2.5202922077922079e-03*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-4.7483766233766234e-04*w11*Jinv22-3.1960227272727273e-04*Jinv11*w1+8.4922889610389610e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-2.1367694805194807e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-2.5202922077922079e-03*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+3.8352272727272725e-04*Jinv21*w38-2.3833198051948050e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-4.2918019480519480e-04*Jinv22*w59-7.1225649350649346e-04*w34*Jinv11+1.4793019480519481e-03*w19*Jinv11+1.3697240259740259e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.4655032467532466e-04*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+4.6570616883116886e-04*w32*Jinv21+1.4793019480519481e-03*Jinv21*w20-1.6436688311688311e-04*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41+5.2049512987012989e-04*w52*Jinv12-7.1225649350649346e-04*Jinv21*w35+4.6570616883116886e-04*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-2.0271915584415584e-03*w46*Jinv11+2.4655032467532466e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+1.4793019480519481e-03*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11-4.7483766233766234e-04*Jinv02*w9+1.9176136363636362e-04*w6*Jinv01-2.0271915584415584e-03*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23+1.3697240259740259e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-2.3833198051948050e-03*w46*Jinv12-1.6436688311688311e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-1.0135957792207792e-03*w22*Jinv11-1.6436688311688311e-04*Jinv12*w7-1.0135957792207792e-03*w21*Jinv01+1.3697240259740259e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01+1.9176136363636362e-04*w7*Jinv11+4.3831168831168834e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02+1.1505681818181819e-03*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-2.3833198051948050e-03*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+8.4922889610389610e-04*Jinv21*w14-4.2918019480519480e-04*w58*Jinv12+1.1505681818181819e-03*Jinv02*w18+8.4922889610389610e-04*Jinv01*w12+1.9176136363636362e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.0135957792207792e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02-4.8396915584415583e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-2.5202922077922079e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+4.6570616883116886e-04*Jinv01*w30+5.2049512987012989e-04*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11-2.1367694805194807e-03*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-4.8396915584415583e-04*w10*Jinv11)*D;
  A[20][48]=(0.0)*D;
  A[20][49]=(0.0)*D;
  A[20][50]=(-8.2183441558441553e-05*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10-8.2183441558441553e-04*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10-8.2183441558441553e-04*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480521e-05*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+3.5612824675324673e-04*w48*Jinv00-3.2873376623376621e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+7.6704545454545450e-04*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27+1.0957792207792208e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10+2.4655032467532466e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-2.7394480519480521e-05*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-2.7394480519480521e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-3.2873376623376621e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20+4.6570616883116886e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-8.2183441558441553e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-5.2049512987012989e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00+4.6570616883116886e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.7394480519480521e-05*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10-2.7394480519480518e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.7394480519480521e-05*Jinv02*w30+1.6436688311688311e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+2.4655032467532466e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-8.2183441558441553e-05*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-3.2873376623376621e-04*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-9.1314935064935065e-06*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+4.6570616883116886e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56-5.2049512987012989e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-2.7394480519480521e-05*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[20][51]=(0.0)*D;
  A[20][52]=(0.0)*D;
  A[20][53]=(8.2183441558441553e-05*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12+5.7528409090909093e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-2.1002435064935065e-04*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+1.4793019480519481e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00+1.4793019480519481e-03*Jinv02*w24-8.2183441558441553e-04*Jinv22*w35-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-4.7666396103896100e-03*Jinv22*w20+5.2049512987012989e-04*w45*Jinv00-1.6436688311688311e-03*w42*Jinv02-1.3697240259740259e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-3.5612824675324673e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+2.4655032467532466e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-7.1225649350649346e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+1.3697240259740259e-04*Jinv02*w45-2.1002435064935065e-04*Jinv22*w59-1.6436688311688311e-04*w51*Jinv00-8.2183441558441553e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-5.4788961038961042e-05*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13-7.1225649350649346e-04*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41+5.2049512987012989e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20+5.7528409090909093e-04*w52*Jinv12+1.4793019480519481e-03*Jinv22*w26+8.2183441558441553e-05*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12+5.2049512987012989e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-5.4788961038961042e-05*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-1.6436688311688311e-04*Jinv20*w38+5.2049512987012989e-04*Jinv22*w23+2.4655032467532466e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+2.4655032467532466e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49-1.6436688311688311e-04*w52*Jinv10+9.1314935064935065e-06*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+5.2049512987012989e-04*w24*Jinv00-3.1047077922077924e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-4.7666396103896100e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+9.1314935064935065e-06*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44-2.1002435064935065e-04*w58*Jinv12-4.7666396103896100e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-8.2183441558441553e-04*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20+5.2049512987012989e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-7.1225649350649346e-04*w20*Jinv20-3.1047077922077924e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-2.4655032467532466e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+5.7528409090909093e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56-3.1047077922077924e-04*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[20][54]=(0.0)*D;
  A[20][55]=(0.0)*D;
  A[20][56]=(-1.3149350649350649e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53+1.2784090909090910e-04*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+3.8352272727272725e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59+1.4793019480519481e-03*w25*Jinv12-2.1915584415584417e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-1.0683847402597404e-03*Jinv12*w55-1.6436688311688311e-04*w15*Jinv02+1.8262987012987013e-05*w10*Jinv12+1.4793019480519481e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-5.4788961038961042e-05*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02+1.8262987012987013e-05*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-1.6436688311688311e-04*Jinv22*w17+1.3971185064935066e-03*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+4.6570616883116886e-04*w32*Jinv22-5.4788961038961042e-05*Jinv01*w39-2.1915584415584417e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+8.2183441558441553e-05*Jinv02*w45+5.2049512987012989e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+2.4655032467532466e-04*Jinv12*w13+2.7394480519480518e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20+3.8352272727272725e-04*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-3.8352272727272725e-04*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11+1.4793019480519481e-03*Jinv22*w26+5.2049512987012989e-04*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21-1.0683847402597404e-03*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+1.3971185064935066e-03*w21*Jinv02-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33+2.4655032467532466e-04*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+1.8262987012987013e-05*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+5.2049512987012989e-04*w47*Jinv21+1.3971185064935066e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11+4.6570616883116886e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51+4.6570616883116886e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-7.1225649350649346e-04*Jinv21*w56+2.1915584415584417e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.2784090909090910e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+9.1314935064935065e-06*Jinv01*w9+2.1915584415584417e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14-5.4788961038961042e-05*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-1.0683847402597404e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28+5.2049512987012989e-04*w25*Jinv11-1.6436688311688311e-04*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-2.1915584415584417e-04*Jinv12*w1+9.1314935064935065e-06*w10*Jinv11)*D;
  A[20][57]=(0.0)*D;
  A[20][58]=(0.0)*D;
  A[20][59]=(3.4699675324675324e-04*w40*Jinv12-2.6481331168831169e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+3.4699675324675324e-04*Jinv02*w39+3.6525974025974026e-05*Jinv22*w38-5.8441558441558442e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-1.0957792207792208e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-5.8441558441558442e-04*Jinv02*w24+2.1915584415584417e-04*Jinv22*w35+1.4975649350649350e-03*Jinv22*w20+4.3831168831168834e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17-4.2918019480519480e-04*w22*Jinv12-4.2918019480519480e-04*Jinv02*w3-2.7394480519480521e-05*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+1.1870941558441559e-04*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+2.1915584415584417e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-2.1915584415584417e-04*Jinv12*w13+3.6525974025974026e-05*w37*Jinv12+3.4699675324675324e-04*Jinv22*w41-2.6481331168831169e-04*w52*Jinv12-5.8441558441558442e-04*Jinv22*w26-1.0957792207792208e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12-4.2918019480519480e-04*w21*Jinv02-2.1915584415584417e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9-4.2918019480519480e-04*Jinv22*w23-2.7394480519480521e-05*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12+7.4878246753246752e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-2.7394480519480521e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.4204545454545455e-05*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+1.4975649350649350e-03*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12+1.4975649350649350e-03*Jinv02*w18+2.1915584415584417e-04*Jinv02*w33-5.4788961038961042e-05*w48*Jinv02-1.4204545454545455e-05*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+3.6525974025974026e-05*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-2.6481331168831169e-04*Jinv02*w51-1.0957792207792208e-04*Jinv22*w56-1.4204545454545455e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[21][0]=(1.0227272727272728e-03*Jinv00*w12-8.3198051948051951e-05*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12+2.7394480519480521e-05*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02+1.0227272727272728e-03*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+6.7573051948051951e-04*w3*Jinv00+6.7573051948051951e-04*w32*Jinv20+6.7573051948051951e-04*Jinv01*w3+2.5568181818181820e-04*Jinv20*w23+2.7394480519480521e-05*Jinv22*w38-1.1870941558441559e-04*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59-3.6525974025974026e-05*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17-3.6525974025974026e-05*Jinv01*w45-1.1870941558441559e-04*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-3.6525974025974026e-05*Jinv02*w24-2.9220779220779221e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01-3.6525974025974026e-05*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-1.1870941558441559e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-3.6525974025974026e-05*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02-8.3198051948051951e-05*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+1.0227272727272728e-03*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01-8.3198051948051951e-05*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-8.3198051948051951e-05*Jinv20*w29-3.6525974025974026e-05*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17-2.9220779220779221e-04*Jinv00*w33+2.5568181818181820e-04*w22*Jinv12+6.7573051948051951e-04*w31*Jinv10+6.7573051948051951e-04*Jinv02*w3+2.5568181818181820e-04*w21*Jinv00+6.7573051948051951e-04*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2-1.1870941558441559e-04*Jinv00*w18+2.7394480519480521e-05*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38-3.6525974025974026e-05*Jinv02*w45-3.6525974025974026e-05*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-2.9220779220779221e-04*w34*Jinv11-1.1870941558441559e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53+2.7394480519480521e-05*w51*Jinv00-2.9220779220779221e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+6.7573051948051951e-04*w4*Jinv10+6.7573051948051951e-04*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+1.0227272727272728e-03*Jinv12*w13-1.1870941558441559e-04*w19*Jinv10-2.9220779220779221e-04*w34*Jinv10+6.7573051948051951e-04*w32*Jinv21-1.1870941558441559e-04*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00+2.7394480519480521e-05*w37*Jinv12-1.1870941558441559e-04*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41-3.6525974025974026e-05*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20+2.7394480519480521e-05*w52*Jinv12-2.9220779220779221e-04*Jinv21*w35+6.7573051948051951e-04*w31*Jinv11-3.6525974025974026e-05*Jinv22*w26-3.6525974025974026e-05*w46*Jinv11+1.0227272727272728e-03*Jinv20*w14+6.7573051948051951e-04*w5*Jinv21-1.1870941558441559e-04*Jinv02*w54+6.7573051948051951e-04*w4*Jinv12+2.5568181818181820e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-8.3198051948051951e-05*Jinv21*w29-2.9220779220779221e-04*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+1.0227272727272728e-03*Jinv02*w12-1.1870941558441559e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-3.6525974025974026e-05*w47*Jinv21+2.7394480519480521e-05*Jinv20*w38+2.5568181818181820e-04*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11+6.7573051948051951e-04*w31*Jinv12-3.6525974025974026e-05*w46*Jinv12+2.7394480519480521e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8+2.5568181818181820e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50+2.5568181818181820e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7+2.5568181818181820e-04*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51+6.7573051948051951e-04*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01+2.7394480519480521e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+2.7394480519480521e-05*w37*Jinv10-1.1870941558441559e-04*Jinv21*w56-3.6525974025974026e-05*w24*Jinv00-8.3198051948051951e-05*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-1.1870941558441559e-04*w19*Jinv12+6.7573051948051951e-04*Jinv22*w5-3.6525974025974026e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+1.0227272727272728e-03*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-1.1870941558441559e-04*Jinv02*w18+1.0227272727272728e-03*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59-2.9220779220779221e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00-2.9220779220779221e-04*w35*Jinv20+2.5568181818181820e-04*Jinv21*w23-1.1870941558441559e-04*Jinv01*w54-1.1870941558441559e-04*w55*Jinv11-3.6525974025974026e-05*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17+6.7573051948051951e-04*w30*Jinv00+3.2873376623376621e-04*w40*Jinv10-1.1870941558441559e-04*w20*Jinv20-8.3198051948051951e-05*Jinv22*w29+1.0227272727272728e-03*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11+2.7394480519480521e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+6.7573051948051951e-04*w5*Jinv20+6.7573051948051951e-04*Jinv01*w30+2.7394480519480521e-05*Jinv02*w51-1.1870941558441559e-04*Jinv22*w56-8.3198051948051951e-05*Jinv12*w28-3.6525974025974026e-05*w25*Jinv11-1.1870941558441559e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-3.6525974025974026e-05*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-8.3198051948051951e-05*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[21][1]=(0.0)*D;
  A[21][2]=(0.0)*D;
  A[21][3]=(-1.3971185064935066e-03*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-4.6570616883116886e-04*Jinv22*w53+9.5880681818181818e-04*Jinv20*w8+3.1960227272727273e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-1.3697240259740260e-03*w32*Jinv20+2.0271915584415584e-03*Jinv01*w3-3.2873376623376621e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38+2.7394480519480518e-04*Jinv00*w54+3.1960227272727273e-04*Jinv21*w59-3.8352272727272725e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+3.2873376623376621e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+2.7394480519480520e-03*w16*Jinv11+2.7394480519480520e-03*Jinv21*w17-1.9176136363636362e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55-1.0957792207792208e-04*w58*Jinv10+2.7394480519480520e-03*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24+2.5202922077922079e-03*Jinv22*w35+3.1960227272727273e-04*w58*Jinv11+3.1960227272727273e-04*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-6.6659902597402602e-04*w1*Jinv10+6.0267857142857139e-04*w16*Jinv10-1.3697240259740259e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+2.7394480519480520e-03*w15*Jinv01+4.8396915584415583e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+2.7394480519480520e-03*Jinv22*w17+1.2053571428571428e-03*Jinv00*w33-1.4793019480519481e-03*w22*Jinv12-1.3697240259740260e-03*w31*Jinv10+2.0271915584415584e-03*Jinv02*w3-3.2873376623376621e-04*w21*Jinv00+1.0135957792207792e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+2.7394480519480521e-05*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38-1.9176136363636362e-04*Jinv02*w45-3.8352272727272725e-04*Jinv01*w24+3.1960227272727273e-04*Jinv22*w59+2.5202922077922079e-03*w34*Jinv11-1.3697240259740259e-04*w19*Jinv11-4.6570616883116886e-04*Jinv21*w53-1.9176136363636362e-04*w51*Jinv00+2.5202922077922079e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+2.0271915584415584e-03*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10+1.2053571428571428e-03*w34*Jinv10+1.0135957792207792e-03*w32*Jinv21-1.3697240259740259e-04*Jinv21*w20+9.5880681818181818e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12+2.7394480519480518e-04*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-4.6570616883116886e-04*w52*Jinv12+2.5202922077922079e-03*Jinv21*w35+1.0135957792207792e-03*w31*Jinv11-3.8352272727272725e-04*Jinv22*w26-1.9176136363636362e-04*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14+2.0271915584415584e-03*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54+2.0271915584415584e-03*w4*Jinv12-1.4793019480519481e-03*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.8396915584415583e-04*Jinv21*w29+2.5202922077922079e-03*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12-1.3697240259740259e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01+9.5880681818181818e-04*w7*Jinv10-1.9176136363636362e-04*w47*Jinv21+2.7394480519480521e-05*Jinv20*w38-1.4793019480519481e-03*Jinv22*w23-4.6570616883116886e-04*w52*Jinv11+1.0135957792207792e-03*w31*Jinv12-1.9176136363636362e-04*w46*Jinv12-1.9176136363636362e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-3.2873376623376621e-04*w22*Jinv10-8.4922889610389610e-04*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01-4.6570616883116886e-04*Jinv01*w51+1.0135957792207792e-03*Jinv02*w30-8.4922889610389610e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-1.9176136363636362e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.7394480519480521e-05*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+4.8396915584415583e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-1.3697240259740259e-04*w19*Jinv12+2.0271915584415584e-03*Jinv22*w5-1.9176136363636362e-04*w47*Jinv22-8.4922889610389610e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+3.1960227272727273e-04*w58*Jinv12-1.3697240259740259e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+2.5202922077922079e-03*Jinv02*w33-6.6659902597402602e-04*w0*Jinv00+1.2053571428571428e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-6.6659902597402602e-04*Jinv20*w2+6.0267857142857139e-04*w15*Jinv00-8.4922889610389610e-04*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+6.0267857142857139e-04*Jinv20*w17-1.3697240259740260e-03*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+4.8396915584415583e-04*Jinv22*w29+1.2327516233766234e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+1.0135957792207792e-03*Jinv01*w30-4.6570616883116886e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+4.8396915584415583e-04*Jinv12*w28-3.8352272727272725e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+2.7394480519480520e-03*w16*Jinv12-8.4922889610389610e-04*Jinv21*w50+4.8396915584415583e-04*Jinv01*w27-8.4922889610389610e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[21][4]=(0.0)*D;
  A[21][5]=(0.0)*D;
  A[21][6]=(9.5880681818181818e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+9.5880681818181818e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00+1.1231737012987013e-03*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-1.4245129870129869e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+2.7394480519480518e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2-3.2873376623376621e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+3.5612824675324673e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17-1.2601461038961039e-03*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12+1.1231737012987013e-03*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3-1.4245129870129869e-03*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+3.5612824675324673e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53+1.9176136363636362e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13-1.2601461038961039e-03*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+2.7394480519480518e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+9.5880681818181818e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+7.1225649350649346e-04*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+1.9176136363636362e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8-1.4245129870129869e-03*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+1.9176136363636362e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-2.7394480519480518e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+3.5612824675324673e-04*w0*Jinv00-1.2601461038961039e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10+3.5612824675324673e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-1.3697240259740260e-03*Jinv20*w17+1.1231737012987013e-03*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[21][7]=(0.0)*D;
  A[21][8]=(0.0)*D;
  A[21][9]=(-5.8441558441558442e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-5.8441558441558442e-04*w13*Jinv10+1.1870941558441559e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20+1.4975649350649350e-03*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+2.1915584415584417e-04*w43*Jinv10+1.0247564935064935e-04*w58*Jinv10-2.1915584415584417e-04*w48*Jinv00-1.4204545454545455e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+7.4878246753246752e-04*w45*Jinv00+6.4935064935064935e-05*Jinv00*w27-2.1915584415584417e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10+3.4699675324675324e-04*Jinv00*w33-4.2918019480519480e-04*w31*Jinv10+1.4975649350649350e-03*w21*Jinv00-2.6481331168831169e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.7394480519480521e-05*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-2.6481331168831169e-04*w19*Jinv10+3.4699675324675324e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+3.6525974025974026e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-5.8441558441558442e-04*Jinv20*w14-2.1915584415584417e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-1.0957792207792208e-04*w7*Jinv10-5.4788961038961042e-05*Jinv20*w38-2.7394480519480521e-05*Jinv20*w53+1.4975649350649350e-03*w22*Jinv10-2.7394480519480521e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10+3.6525974025974026e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+2.1915584415584417e-04*Jinv20*w44+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+3.4699675324675324e-04*w35*Jinv20+3.6525974025974026e-05*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+4.3831168831168834e-04*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-2.6481331168831169e-04*w20*Jinv20+1.1870941558441559e-04*w5*Jinv20-4.2918019480519480e-04*w55*Jinv10+2.1915584415584417e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20)*D;
  A[21][10]=(0.0)*D;
  A[21][11]=(0.0)*D;
  A[21][12]=(1.3149350649350649e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+1.5523538961038962e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.2327516233766234e-03*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3-3.7804383116883116e-03*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12+1.4610389610389610e-04*w0*Jinv02-4.0178571428571428e-04*Jinv21*w2+8.2183441558441553e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21-1.4793019480519481e-03*w16*Jinv11-1.4793019480519481e-03*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45-8.2183441558441553e-05*Jinv12*w55+1.5523538961038962e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.5523538961038962e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26-4.0178571428571428e-04*w0*Jinv01+1.4610389610389610e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00-1.4793019480519481e-03*Jinv21*w41+8.2183441558441553e-04*w42*Jinv02+1.0957792207792207e-03*Jinv00*w27+1.5523538961038962e-04*w11*Jinv22-4.0178571428571428e-04*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11-1.4793019480519481e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-3.7804383116883116e-03*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3-3.7804383116883116e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-1.4793019480519481e-03*Jinv01*w39+1.4610389610389610e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+1.5523538961038962e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+8.2183441558441553e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.1505681818181819e-03*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20+1.5523538961038962e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21-8.2183441558441553e-05*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-3.7804383116883116e-03*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+4.3831168831168834e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.5523538961038962e-04*Jinv02*w9+9.5880681818181818e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-3.7804383116883116e-03*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-3.7804383116883116e-03*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-2.6298701298701297e-03*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-2.6298701298701297e-03*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10+9.5880681818181818e-04*w7*Jinv11+1.5523538961038962e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10+4.3831168831168834e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00+1.0957792207792207e-03*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12+1.2327516233766234e-03*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22+9.5880681818181818e-04*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.5523538961038962e-04*w10*Jinv10+8.2183441558441553e-04*Jinv20*w44+1.5523538961038962e-04*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+9.5880681818181818e-04*Jinv21*w8+1.5523538961038962e-04*w57*Jinv00+1.5523538961038962e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33+1.4610389610389610e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20-2.6298701298701297e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10+1.4610389610389610e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20+1.0957792207792207e-03*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14-1.4793019480519481e-03*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+8.2183441558441553e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-8.2183441558441553e-05*w55*Jinv10+8.2183441558441553e-04*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+9.5880681818181818e-04*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27+9.5880681818181818e-04*w48*Jinv01+1.4610389610389610e-04*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[21][13]=(0.0)*D;
  A[21][14]=(0.0)*D;
  A[21][15]=(-2.7942370129870131e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+6.0267857142857139e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-2.7942370129870131e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-1.7532467532467534e-03*w32*Jinv20-1.8628246753246754e-03*Jinv01*w3+8.5470779220779228e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38-1.3149350649350649e-03*Jinv00*w54-6.9399350649350649e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-2.3011363636363637e-03*w42*Jinv01-5.6615259740259744e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17-1.0409902597402598e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55+2.7394480519480518e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-6.9399350649350649e-04*w58*Jinv11-6.9399350649350649e-04*w57*Jinv01-2.0819805194805196e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+1.6436688311688311e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-2.9586038961038961e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1-5.9172077922077923e-03*w15*Jinv01-4.3831168831168834e-04*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12-1.7532467532467534e-03*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3+8.5470779220779228e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-2.9586038961038961e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-4.9310064935064932e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00+3.2873376623376621e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-2.0819805194805196e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10-1.8628246753246754e-03*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13-4.9310064935064932e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-9.3141233766233771e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+6.0267857142857139e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12-1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41+5.4788961038961042e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35-9.3141233766233771e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-1.0409902597402598e-03*w46*Jinv11-2.7942370129870131e-03*Jinv20*w14-1.8628246753246754e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.3831168831168834e-04*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.2873376623376621e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+6.0267857142857139e-04*w6*Jinv01+6.0267857142857139e-04*w7*Jinv10-1.0409902597402598e-03*w47*Jinv21+3.2873376623376621e-04*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+7.6704545454545450e-04*Jinv22*w8+8.5470779220779228e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50-9.8620129870129864e-04*w22*Jinv11+7.6704545454545450e-04*Jinv12*w7-9.8620129870129864e-04*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+3.2873376623376621e-04*w36*Jinv01+6.0267857142857139e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10+1.2601461038961039e-03*Jinv21*w56+5.4788961038961036e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+1.2053571428571428e-03*w49*Jinv11+5.4788961038961042e-05*w10*Jinv10-2.7394480519480518e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+6.0267857142857139e-04*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+1.6436688311688311e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20-9.8620129870129864e-04*Jinv21*w23+1.2601461038961039e-03*Jinv01*w54+1.2601461038961039e-03*w55*Jinv11+1.6436688311688311e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.6615259740259744e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.7532467532467534e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-4.9310064935064932e-04*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14-2.9586038961038961e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20-9.3141233766233771e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-2.0819805194805196e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+1.2053571428571428e-03*Jinv21*w50-4.3831168831168834e-04*Jinv01*w27+1.2053571428571428e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-5.6615259740259744e-04*w10*Jinv11)*D;
  A[21][16]=(0.0)*D;
  A[21][17]=(0.0)*D;
  A[21][18]=(1.4793019480519481e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8+1.4793019480519481e-03*w13*Jinv10-3.8352272727272725e-04*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.2049512987012989e-04*Jinv01*w3-4.7666396103896100e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-8.2183441558441553e-04*w43*Jinv10-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-1.6436688311688311e-03*w16*Jinv10-3.8352272727272725e-04*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-4.7666396103896100e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39+5.7528409090909093e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38-1.6436688311688311e-04*Jinv01*w24-5.4788961038961042e-05*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00+1.3697240259740259e-04*w4*Jinv10+5.2049512987012989e-04*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.7528409090909093e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-5.4788961038961042e-05*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11+1.4793019480519481e-03*Jinv20*w14+5.2049512987012989e-04*w5*Jinv21+5.4788961038961036e-04*w49*Jinv10+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.6436688311688311e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+2.7394480519480518e-04*w52*Jinv11+2.4655032467532466e-04*Jinv20*w53-4.7666396103896100e-03*w22*Jinv10-7.1225649350649346e-04*w22*Jinv11-7.1225649350649346e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-3.5612824675324673e-04*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11-2.1002435064935065e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+8.2183441558441553e-05*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-2.1002435064935065e-04*w10*Jinv10-8.2183441558441553e-04*Jinv20*w44+5.2049512987012989e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-3.1047077922077924e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20-7.1225649350649346e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-1.6436688311688311e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-1.6436688311688311e-03*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+5.7528409090909093e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+1.3697240259740259e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-8.2183441558441553e-04*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+8.2183441558441553e-05*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27+8.2183441558441553e-05*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[21][19]=(0.0)*D;
  A[21][20]=(0.0)*D;
  A[21][21]=(2.6298701298701297e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-4.2735389610389614e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+2.6298701298701297e-03*w13*Jinv10-4.2735389610389614e-03*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+7.3965097402597403e-04*Jinv01*w3-7.3965097402597406e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54-3.6982548701298703e-03*w25*Jinv12-7.8530844155844158e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45+2.3833198051948050e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00-3.6982548701298703e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-1.2327516233766234e-03*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-7.8530844155844158e-04*w1*Jinv10-4.2735389610389614e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-3.6982548701298703e-03*w45*Jinv00+4.9310064935064932e-04*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01-1.1870941558441558e-03*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-3.6982548701298703e-03*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-7.3965097402597406e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3-7.3965097402597406e-03*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-7.8530844155844158e-04*Jinv22*w2+2.3833198051948050e-03*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.6982548701298703e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+3.5612824675324673e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+7.3965097402597403e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+2.6298701298701297e-03*Jinv12*w13+2.3833198051948050e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+7.3965097402597403e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-4.2735389610389614e-03*Jinv22*w41-3.6982548701298703e-03*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+7.3965097402597403e-04*w31*Jinv11-3.6982548701298703e-03*Jinv22*w26-1.2327516233766234e-03*w46*Jinv11+2.6298701298701297e-03*Jinv20*w14+7.3965097402597403e-04*w5*Jinv21+2.3833198051948050e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-7.3965097402597406e-03*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-1.1870941558441558e-03*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+2.6298701298701297e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38-7.3965097402597406e-03*Jinv22*w23+1.6436688311688311e-04*w31*Jinv12-3.6982548701298703e-03*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8-7.3965097402597406e-03*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*Jinv12*w7+1.6436688311688311e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+3.5612824675324673e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-3.6982548701298703e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-3.6982548701298703e-03*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.3149350649350649e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.8530844155844158e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-3.6982548701298703e-03*w25*Jinv10-7.8530844155844158e-04*Jinv20*w2-4.2735389610389614e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-4.2735389610389614e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+2.3833198051948050e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.6298701298701297e-03*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+7.3965097402597403e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+2.3833198051948050e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-3.6982548701298703e-03*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50-1.1870941558441558e-03*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-7.8530844155844158e-04*Jinv12*w1)*D;
  A[21][22]=(0.0)*D;
  A[21][23]=(0.0)*D;
  A[21][24]=(-2.6298701298701297e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-7.1225649350649346e-04*Jinv20*w8-2.6298701298701297e-03*w13*Jinv10+1.5888798701298701e-03*Jinv21*w44-1.6436688311688311e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+7.3965097402597406e-03*Jinv20*w23-2.3833198051948050e-03*Jinv00*w54+3.7439123376623376e-04*Jinv21*w59+3.9265422077922079e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+1.5888798701298701e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+3.1777597402597403e-03*w16*Jinv11+3.1777597402597403e-03*Jinv21*w17+1.2327516233766234e-03*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+3.7439123376623376e-04*w58*Jinv11+3.7439123376623376e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.9265422077922079e-04*w0*Jinv01+7.8530844155844158e-04*w1*Jinv10+4.2735389610389614e-03*w16*Jinv10+1.5888798701298701e-03*w43*Jinv11+3.6982548701298703e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+3.9265422077922079e-04*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11+3.1777597402597403e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+3.6982548701298703e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-1.6436688311688311e-04*w31*Jinv10+7.3965097402597406e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.3833198051948050e-03*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+2.4655032467532467e-03*Jinv01*w24+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00-1.6436688311688311e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11+4.2735389610389614e-03*w41*Jinv20-2.3833198051948050e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-7.1225649350649346e-04*w6*Jinv00-2.3833198051948050e-03*Jinv20*w56+3.6982548701298703e-03*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+1.0957792207792207e-03*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+1.2327516233766234e-03*w46*Jinv11-2.6298701298701297e-03*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33+4.2735389610389614e-03*w39*Jinv00-1.2327516233766234e-03*Jinv01*w18-8.2183441558441553e-05*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-7.1225649350649346e-04*w7*Jinv10+1.2327516233766234e-03*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38-3.5612824675324673e-04*w52*Jinv11-3.5612824675324673e-04*Jinv20*w53+7.3965097402597406e-03*w22*Jinv10+2.4655032467532467e-03*w22*Jinv11+2.4655032467532467e-03*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-8.2183441558441553e-05*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+7.4878246753246752e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10-1.0135957792207792e-03*Jinv21*w56+3.6982548701298703e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+7.4878246753246752e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44-1.2327516233766234e-03*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.8530844155844158e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+2.4655032467532467e-03*Jinv21*w23-1.0135957792207792e-03*Jinv01*w54-1.0135957792207792e-03*w55*Jinv11+3.6982548701298703e-03*w25*Jinv10+7.8530844155844158e-04*Jinv20*w2+4.2735389610389614e-03*w15*Jinv00+3.9265422077922079e-04*Jinv01*w9+4.2735389610389614e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-2.3833198051948050e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-1.6436688311688311e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30+2.4655032467532467e-03*w25*Jinv11-2.3833198051948050e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+3.6982548701298703e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+3.9265422077922079e-04*w10*Jinv11)*D;
  A[21][25]=(0.0)*D;
  A[21][26]=(0.0)*D;
  A[21][27]=(3.2873376623376621e-04*Jinv21*w44-2.6481331168831169e-04*Jinv01*w3+1.5726461038961039e-04*Jinv21*w59+2.0495129870129870e-04*Jinv21*w2+3.2873376623376621e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.1870941558441558e-03*Jinv01*w45+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+1.1870941558441558e-03*Jinv21*w26+2.0495129870129870e-04*w0*Jinv01+3.2873376623376621e-04*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+2.0495129870129870e-04*Jinv11*w1-1.0227272727272728e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+6.5746753246753243e-04*Jinv01*w39-5.4788961038961042e-05*Jinv21*w38+1.1870941558441558e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-6.7573051948051951e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-2.6481331168831169e-04*w4*Jinv11-2.6481331168831169e-04*w32*Jinv21-6.7573051948051951e-04*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-2.6481331168831169e-04*w31*Jinv11+1.1870941558441558e-03*w46*Jinv11-2.6481331168831169e-04*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-6.7573051948051951e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-1.0957792207792208e-04*w6*Jinv01+1.1870941558441558e-03*w47*Jinv21-5.4788961038961042e-05*w52*Jinv11+2.3741883116883116e-03*w22*Jinv11+2.3741883116883116e-03*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-5.4788961038961042e-05*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-6.7573051948051951e-04*Jinv21*w56-1.0957792207792208e-04*w49*Jinv11-1.0227272727272728e-03*Jinv21*w14-1.0227272727272728e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+2.3741883116883116e-03*Jinv21*w23-6.7573051948051951e-04*Jinv01*w54-6.7573051948051951e-04*w55*Jinv11+1.5726461038961039e-04*Jinv01*w9+6.5746753246753243e-04*w40*Jinv11-2.6481331168831169e-04*Jinv01*w30+1.1870941558441558e-03*w25*Jinv11-1.0957792207792208e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+1.5726461038961039e-04*w10*Jinv11)*D;
  A[21][28]=(0.0)*D;
  A[21][29]=(0.0)*D;
  A[21][30]=(1.2327516233766234e-03*Jinv00*w12+4.8396915584415583e-04*w28*Jinv10+6.0267857142857139e-04*w40*Jinv12+2.7394480519480521e-05*Jinv22*w53-8.4922889610389610e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10+6.0267857142857139e-04*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44+1.0135957792207792e-03*w3*Jinv00+2.0271915584415584e-03*w32*Jinv20+1.0135957792207792e-03*Jinv01*w3-1.4793019480519481e-03*Jinv20*w23-1.9176136363636362e-04*Jinv22*w38-1.3697240259740259e-04*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12-6.6659902597402602e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+3.1960227272727273e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+3.8352272727272725e-04*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+1.2053571428571428e-03*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01-1.9176136363636362e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10+2.7394480519480518e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-3.8352272727272725e-04*w45*Jinv00+2.7394480519480520e-03*Jinv21*w41+3.2873376623376621e-04*w42*Jinv02+4.8396915584415583e-04*Jinv00*w27-1.0957792207792208e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+4.8396915584415583e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+4.8396915584415583e-04*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17+2.5202922077922079e-03*Jinv00*w33-3.2873376623376621e-04*w22*Jinv12+2.0271915584415584e-03*w31*Jinv10-1.3697240259740260e-03*Jinv02*w3-1.4793019480519481e-03*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+2.7394480519480520e-03*Jinv01*w39-6.6659902597402602e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-4.6570616883116886e-04*w36*Jinv00-4.6570616883116886e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-1.9176136363636362e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+2.5202922077922079e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.2053571428571428e-03*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44+1.0135957792207792e-03*w4*Jinv10+1.0135957792207792e-03*w4*Jinv11+2.7394480519480520e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+2.5202922077922079e-03*w34*Jinv10+2.0271915584415584e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-8.4922889610389610e-04*w6*Jinv00-1.9176136363636362e-04*w37*Jinv12-1.3697240259740259e-04*Jinv20*w56+6.0267857142857139e-04*Jinv22*w41-1.9176136363636362e-04*w26*Jinv20+3.1960227272727273e-04*w11*Jinv20+2.7394480519480521e-05*w52*Jinv12+2.5202922077922079e-03*Jinv21*w35+2.0271915584415584e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26-3.8352272727272725e-04*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14+1.0135957792207792e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54-1.3697240259740260e-03*w4*Jinv12-3.2873376623376621e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+4.8396915584415583e-04*Jinv21*w29+2.5202922077922079e-03*Jinv01*w33+2.7394480519480520e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-4.6570616883116886e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9-8.4922889610389610e-04*w6*Jinv01-8.4922889610389610e-04*w7*Jinv10-3.8352272727272725e-04*w47*Jinv21-4.6570616883116886e-04*Jinv20*w38-3.2873376623376621e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-1.4793019480519481e-03*w22*Jinv10+9.5880681818181818e-04*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+9.5880681818181818e-04*Jinv12*w49-4.6570616883116886e-04*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10-8.4922889610389610e-04*w7*Jinv11+3.1960227272727273e-04*w9*Jinv00-4.6570616883116886e-04*w37*Jinv10-1.3697240259740259e-04*Jinv21*w56-1.9176136363636362e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12-1.3697240259740260e-03*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+3.1960227272727273e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-8.4922889610389610e-04*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+1.2053571428571428e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+2.5202922077922079e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23-1.3697240259740259e-04*Jinv01*w54-1.3697240259740259e-04*w55*Jinv11-1.9176136363636362e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00+9.5880681818181818e-04*w48*Jinv02+3.1960227272727273e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+2.0271915584415584e-03*w30*Jinv00+2.7394480519480520e-03*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14+2.7394480519480520e-03*w40*Jinv11-1.9176136363636362e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.0135957792207792e-03*w5*Jinv20+2.0271915584415584e-03*Jinv01*w30+2.7394480519480521e-05*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.9176136363636362e-04*w25*Jinv11-1.3697240259740259e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-3.8352272727272725e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+4.8396915584415583e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01-6.6659902597402602e-04*Jinv12*w1+3.1960227272727273e-04*w10*Jinv11)*D;
  A[21][31]=(0.0)*D;
  A[21][32]=(0.0)*D;
  A[21][33]=(-1.7532467532467534e-03*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+8.2183441558441553e-04*Jinv22*w53+1.2053571428571428e-03*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02-1.7532467532467534e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44-1.2601461038961039e-03*w3*Jinv00-2.5750811688311690e-03*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+2.1915584415584414e-03*Jinv20*w23-3.8352272727272725e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.3149350649350649e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-1.0957792207792207e-03*Jinv01*w45+3.2873376623376621e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+1.0957792207792207e-03*Jinv02*w24-1.6436688311688311e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01-1.0957792207792207e-03*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10-3.8352272727272725e-04*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-4.9310064935064932e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-1.6436688311688311e-03*Jinv00*w33+2.1915584415584414e-03*w22*Jinv12-2.5750811688311690e-03*w31*Jinv10-2.5750811688311690e-03*Jinv02*w3+2.1915584415584414e-03*w21*Jinv00-1.2601461038961039e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+3.2873376623376621e-04*Jinv00*w18+8.2183441558441553e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-1.6436688311688311e-03*Jinv12*w34-1.2601461038961039e-03*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20-1.7532467532467534e-03*Jinv12*w13+3.2873376623376621e-04*w19*Jinv10-1.6436688311688311e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+1.2053571428571428e-03*w6*Jinv00-3.8352272727272725e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41-3.8352272727272725e-04*w11*Jinv20+8.2183441558441553e-04*w52*Jinv12+3.2873376623376621e-03*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26-1.0957792207792207e-03*w46*Jinv11-1.7532467532467534e-03*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21+3.2873376623376621e-04*Jinv02*w54-2.5750811688311690e-03*w4*Jinv12+2.1915584415584414e-03*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.7532467532467534e-03*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+1.2053571428571428e-03*w7*Jinv10-1.0957792207792207e-03*w47*Jinv21+8.2183441558441553e-04*Jinv20*w38+2.1915584415584414e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.2601461038961039e-03*w31*Jinv12-1.0957792207792208e-04*Jinv22*w8+2.1915584415584414e-03*w22*Jinv10+1.2053571428571428e-03*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-1.0957792207792207e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.2601461038961039e-03*Jinv02*w30+1.2053571428571428e-03*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+8.2183441558441553e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-4.9310064935064932e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12-2.5750811688311690e-03*Jinv22*w5+5.4788961038961042e-05*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10-3.8352272727272725e-04*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-1.6436688311688311e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-1.6436688311688311e-03*w35*Jinv20-1.0957792207792207e-03*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+2.5568181818181820e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+1.2053571428571428e-03*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-2.5750811688311690e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29-1.7532467532467534e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-1.2601461038961039e-03*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+8.2183441558441553e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28-1.0957792207792207e-03*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10+1.0957792207792207e-03*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[21][34]=(0.0)*D;
  A[21][35]=(0.0)*D;
  A[21][36]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-1.0957792207792208e-04*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+2.4655032467532466e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.2049512987012989e-04*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-7.1225649350649346e-04*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55-1.3697240259740259e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-7.1225649350649346e-04*w45*Jinv00-3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+5.4788961038961036e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17-8.7662337662337668e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10+5.2049512987012989e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv02*w45-1.0957792207792208e-04*Jinv22*w59+2.4655032467532466e-04*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+2.4655032467532466e-04*w4*Jinv10-1.0957792207792208e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10-8.7662337662337668e-04*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.2049512987012989e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.9176136363636362e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10-1.0957792207792208e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-3.5612824675324673e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49+2.4655032467532466e-04*w52*Jinv10+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+1.9176136363636362e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+6.3920454545454550e-05*w10*Jinv10-7.1225649350649346e-04*Jinv20*w44-1.0957792207792208e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-8.7662337662337668e-04*w35*Jinv20+1.9176136363636362e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-3.5612824675324673e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+2.4655032467532466e-04*w5*Jinv20+2.7394480519480518e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28+5.2049512987012989e-04*w55*Jinv10-7.1225649350649346e-04*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[21][37]=(0.0)*D;
  A[21][38]=(0.0)*D;
  A[21][39]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-9.3141233766233771e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+2.1367694805194807e-03*Jinv00*w54-5.6615259740259744e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+1.6436688311688311e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-6.9399350649350649e-04*w11*Jinv21-2.9586038961038961e-03*w16*Jinv11-2.9586038961038961e-03*Jinv21*w17-2.0819805194805196e-03*Jinv01*w45-4.9310064935064932e-04*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-5.6615259740259744e-04*w58*Jinv11-5.6615259740259744e-04*w57*Jinv01-1.0409902597402598e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10-1.3149350649350649e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41+5.4788961038961036e-04*Jinv00*w27+2.7394480519480518e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1-2.9586038961038961e-03*w15*Jinv01-4.3831168831168834e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+8.5470779220779228e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10-1.7532467532467534e-03*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39+1.6436688311688311e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38-1.0409902597402598e-03*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.2601461038961039e-03*w19*Jinv11+3.2873376623376621e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.8352272727272725e-04*w4*Jinv10-9.3141233766233771e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-2.7942370129870131e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-1.8628246753246754e-03*w32*Jinv21+1.2601461038961039e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20+3.2873376623376621e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35-1.8628246753246754e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26-2.0819805194805196e-03*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-9.3141233766233771e-04*w5*Jinv21-4.9310064935064932e-04*Jinv02*w54-1.7532467532467534e-03*w4*Jinv12+8.5470779220779228e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10-4.3831168831168834e-04*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-2.7942370129870131e-03*Jinv02*w12+1.2601461038961039e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9+1.2053571428571428e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-2.0819805194805196e-03*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38+8.5470779220779228e-03*Jinv22*w23+3.2873376623376621e-04*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12+3.8352272727272725e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+6.0267857142857139e-04*Jinv22*w50-9.8620129870129864e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-9.8620129870129864e-04*w21*Jinv01+3.2873376623376621e-04*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+6.0267857142857139e-04*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+1.2053571428571428e-03*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-1.7532467532467534e-03*Jinv22*w5+6.0267857142857139e-04*w49*Jinv11-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18+1.2053571428571428e-03*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-9.8620129870129864e-04*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+6.0267857142857139e-04*w48*Jinv02-6.9399350649350649e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-2.7942370129870131e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-1.8628246753246754e-03*Jinv01*w30+3.2873376623376621e-04*Jinv02*w51-4.9310064935064932e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.0409902597402598e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+6.0267857142857139e-04*Jinv21*w50-4.3831168831168834e-04*Jinv01*w27+6.0267857142857139e-04*w48*Jinv01+1.6436688311688311e-04*Jinv12*w1-6.9399350649350649e-04*w10*Jinv11)*D;
  A[21][40]=(0.0)*D;
  A[21][41]=(0.0)*D;
  A[21][42]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02-3.6525974025974026e-05*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+1.0957792207792207e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+1.0957792207792207e-03*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22-3.6525974025974026e-05*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+4.2735389610389614e-03*w22*Jinv12-7.6704545454545450e-04*w31*Jinv10-7.6704545454545450e-04*Jinv02*w3+4.2735389610389614e-03*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-1.3149350649350649e-03*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+1.0957792207792207e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59-3.2873376623376621e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-2.1367694805194807e-03*Jinv12*w13-1.3149350649350649e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+2.1367694805194807e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20-5.4788961038961042e-05*w52*Jinv12-3.2873376623376621e-03*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+1.0957792207792207e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-3.2873376623376621e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-2.1367694805194807e-03*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.0957792207792207e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38+4.2735389610389614e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8+4.2735389610389614e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10-5.4788961038961042e-05*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-5.4788961038961042e-05*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.0957792207792207e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-5.4788961038961042e-05*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[21][43]=(0.0)*D;
  A[21][44]=(0.0)*D;
  A[21][45]=(4.2735389610389614e-03*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+7.4878246753246752e-04*w57*Jinv02+4.2735389610389614e-03*Jinv02*w39+1.5888798701298701e-03*Jinv21*w44-8.2183441558441553e-05*Jinv01*w3-3.5612824675324673e-04*Jinv22*w38+3.9265422077922079e-04*Jinv21*w59+3.6982548701298703e-03*w25*Jinv12+7.8530844155844158e-04*w0*Jinv02+3.9265422077922079e-04*Jinv21*w2+1.5888798701298701e-03*w42*Jinv01+3.7439123376623376e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45-2.3833198051948050e-03*Jinv12*w55+4.2735389610389614e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+3.6982548701298703e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+1.2327516233766234e-03*Jinv21*w26+3.9265422077922079e-04*w0*Jinv01-2.3833198051948050e-03*Jinv22*w20+1.5888798701298701e-03*w43*Jinv11+3.1777597402597403e-03*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+3.9265422077922079e-04*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11+4.2735389610389614e-03*Jinv22*w17+7.3965097402597406e-03*w22*Jinv12-1.6436688311688311e-04*Jinv02*w3-1.6436688311688311e-04*w32*Jinv22+3.1777597402597403e-03*Jinv01*w39+7.8530844155844158e-04*Jinv22*w2-3.5612824675324673e-04*Jinv21*w38+3.6982548701298703e-03*Jinv02*w45+1.2327516233766234e-03*Jinv01*w24+7.4878246753246752e-04*Jinv22*w59+1.0957792207792207e-03*w34*Jinv11-1.0135957792207792e-03*w19*Jinv11-8.2183441558441553e-05*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-8.2183441558441553e-05*w4*Jinv11-2.6298701298701297e-03*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21-1.0135957792207792e-03*Jinv21*w20-3.5612824675324673e-04*w37*Jinv12+4.2735389610389614e-03*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12+1.0957792207792207e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+3.6982548701298703e-03*Jinv22*w26+2.4655032467532467e-03*w46*Jinv11-8.2183441558441553e-05*w5*Jinv21-2.3833198051948050e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+7.3965097402597406e-03*w21*Jinv02-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33-2.6298701298701297e-03*Jinv02*w12-1.0135957792207792e-03*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+2.4655032467532467e-03*w47*Jinv21+7.3965097402597406e-03*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11-1.6436688311688311e-04*w31*Jinv12+3.6982548701298703e-03*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-7.1225649350649346e-04*Jinv22*w50+2.4655032467532467e-03*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+2.4655032467532467e-03*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51-1.6436688311688311e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-1.2327516233766234e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+3.6982548701298703e-03*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+7.4878246753246752e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+2.4655032467532467e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11-7.1225649350649346e-04*w48*Jinv02+3.7439123376623376e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.6298701298701297e-03*Jinv22*w14+3.1777597402597403e-03*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-2.3833198051948050e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+1.2327516233766234e-03*w25*Jinv11+4.2735389610389614e-03*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+7.8530844155844158e-04*Jinv12*w1+3.7439123376623376e-04*w10*Jinv11)*D;
  A[21][46]=(0.0)*D;
  A[21][47]=(0.0)*D;
  A[21][48]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-8.2183441558441553e-05*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+1.9176136363636362e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+3.5612824675324673e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24-1.2601461038961039e-03*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+2.7394480519480518e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41-3.2873376623376621e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-1.0957792207792208e-04*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33-1.4245129870129869e-03*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10+1.1231737012987013e-03*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39+3.5612824675324673e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-1.2601461038961039e-03*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+9.5880681818181818e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+1.9176136363636362e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+5.4788961038961042e-05*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-5.2049512987012989e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+1.1231737012987013e-03*w4*Jinv12-1.4245129870129869e-03*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+9.5880681818181818e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38-1.4245129870129869e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12+1.1231737012987013e-03*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-1.2601461038961039e-03*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+9.5880681818181818e-04*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11+1.9176136363636362e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30+9.1314935064935065e-06*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+3.5612824675324673e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[21][49]=(0.0)*D;
  A[21][50]=(0.0)*D;
  A[21][51]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+6.3920454545454550e-05*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-7.1225649350649346e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-05*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24-8.7662337662337668e-04*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+5.2049512987012989e-04*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27-1.3697240259740259e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+5.2049512987012989e-04*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+2.4655032467532466e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+1.9176136363636362e-04*Jinv02*w45+6.3920454545454550e-05*Jinv22*w59-3.5612824675324673e-04*w51*Jinv00-8.7662337662337668e-04*Jinv12*w34-7.1225649350649346e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-7.6704545454545450e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+8.2183441558441553e-05*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12-7.1225649350649346e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-8.2183441558441553e-05*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10-7.6704545454545450e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-1.3697240259740259e-04*Jinv02*w9+8.2183441558441553e-05*w7*Jinv10+2.7394480519480518e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+2.4655032467532466e-04*w31*Jinv12+1.9176136363636362e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+5.4788961038961036e-04*Jinv12*w7+2.4655032467532466e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-3.5612824675324673e-04*w52*Jinv10-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10-3.5612824675324673e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.9176136363636362e-04*w47*Jinv22-1.0957792207792208e-04*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+6.3920454545454550e-05*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-8.7662337662337668e-04*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14+2.4655032467532466e-04*Jinv02*w36-7.1225649350649346e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20-3.5612824675324673e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[21][52]=(0.0)*D;
  A[21][53]=(0.0)*D;
  A[21][54]=(-1.6436688311688311e-03*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-2.1002435064935065e-04*w57*Jinv02-1.6436688311688311e-03*Jinv02*w39-3.8352272727272725e-04*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+2.4655032467532466e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+5.7528409090909093e-04*Jinv12*w55-2.6298701298701297e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-3.8352272727272725e-04*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-8.2183441558441553e-04*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11-2.6298701298701297e-03*Jinv22*w17-4.7666396103896100e-03*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+1.3697240259740259e-04*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39-3.1047077922077924e-04*Jinv22*w2+2.7394480519480518e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-2.1002435064935065e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-8.2183441558441553e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+1.4793019480519481e-03*Jinv12*w13+5.2049512987012989e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+2.4655032467532466e-04*w37*Jinv12-1.6436688311688311e-03*Jinv22*w41-8.2183441558441553e-05*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21+5.7528409090909093e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-4.7666396103896100e-03*w21*Jinv02+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+1.4793019480519481e-03*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+8.2183441558441553e-05*w6*Jinv01-1.6436688311688311e-04*w47*Jinv21-4.7666396103896100e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+1.3697240259740259e-04*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50-7.1225649350649346e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-7.1225649350649346e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+1.3697240259740259e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+8.2183441558441553e-05*w7*Jinv11-1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-2.1002435064935065e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+5.2049512987012989e-04*Jinv01*w12+8.2183441558441553e-05*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33-7.1225649350649346e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.4793019480519481e-03*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36-8.2183441558441553e-04*Jinv12*w43+5.2049512987012989e-04*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51+5.7528409090909093e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-2.6298701298701297e-03*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[21][55]=(0.0)*D;
  A[21][56]=(0.0)*D;
  A[21][57]=(4.3831168831168834e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39-2.7394480519480521e-05*Jinv22*w38+7.4878246753246752e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02-2.6481331168831169e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24+3.4699675324675324e-04*Jinv22*w35-4.2918019480519480e-04*Jinv22*w20+2.1915584415584417e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17+1.4975649350649350e-03*w22*Jinv12-4.2918019480519480e-04*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2+3.6525974025974026e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+3.4699675324675324e-04*Jinv12*w34+2.1915584415584417e-04*Jinv22*w44-5.8441558441558442e-04*Jinv12*w13-2.7394480519480521e-05*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12+7.4878246753246752e-04*Jinv22*w26-2.6481331168831169e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12+1.4975649350649350e-03*w21*Jinv02-5.8441558441558442e-04*Jinv02*w12+1.0247564935064935e-04*Jinv02*w9+1.4975649350649350e-03*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-2.1915584415584417e-04*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50-2.1915584415584417e-04*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+6.4935064935064935e-05*Jinv02*w27-2.1915584415584417e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+3.4699675324675324e-04*Jinv02*w33-1.0957792207792208e-04*w48*Jinv02+6.4935064935064935e-05*Jinv22*w29-5.8441558441558442e-04*Jinv22*w14-2.7394480519480521e-05*Jinv02*w36+2.1915584415584417e-04*Jinv12*w43-5.4788961038961042e-05*Jinv02*w51-2.6481331168831169e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[21][58]=(0.0)*D;
  A[21][59]=(0.0)*D;
  A[22][0]=(0.0)*D;
  A[22][1]=(1.0227272727272728e-03*Jinv00*w12-8.3198051948051951e-05*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12+2.7394480519480521e-05*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02+1.0227272727272728e-03*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+6.7573051948051951e-04*w3*Jinv00+6.7573051948051951e-04*w32*Jinv20+6.7573051948051951e-04*Jinv01*w3+2.5568181818181820e-04*Jinv20*w23+2.7394480519480521e-05*Jinv22*w38-1.1870941558441559e-04*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59-3.6525974025974026e-05*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17-3.6525974025974026e-05*Jinv01*w45-1.1870941558441559e-04*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-3.6525974025974026e-05*Jinv02*w24-2.9220779220779221e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01-3.6525974025974026e-05*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-1.1870941558441559e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-3.6525974025974026e-05*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02-8.3198051948051951e-05*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+1.0227272727272728e-03*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01-8.3198051948051951e-05*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-8.3198051948051951e-05*Jinv20*w29-3.6525974025974026e-05*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17-2.9220779220779221e-04*Jinv00*w33+2.5568181818181820e-04*w22*Jinv12+6.7573051948051951e-04*w31*Jinv10+6.7573051948051951e-04*Jinv02*w3+2.5568181818181820e-04*w21*Jinv00+6.7573051948051951e-04*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2-1.1870941558441559e-04*Jinv00*w18+2.7394480519480521e-05*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38-3.6525974025974026e-05*Jinv02*w45-3.6525974025974026e-05*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-2.9220779220779221e-04*w34*Jinv11-1.1870941558441559e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53+2.7394480519480521e-05*w51*Jinv00-2.9220779220779221e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+6.7573051948051951e-04*w4*Jinv10+6.7573051948051951e-04*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+1.0227272727272728e-03*Jinv12*w13-1.1870941558441559e-04*w19*Jinv10-2.9220779220779221e-04*w34*Jinv10+6.7573051948051951e-04*w32*Jinv21-1.1870941558441559e-04*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00+2.7394480519480521e-05*w37*Jinv12-1.1870941558441559e-04*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41-3.6525974025974026e-05*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20+2.7394480519480521e-05*w52*Jinv12-2.9220779220779221e-04*Jinv21*w35+6.7573051948051951e-04*w31*Jinv11-3.6525974025974026e-05*Jinv22*w26-3.6525974025974026e-05*w46*Jinv11+1.0227272727272728e-03*Jinv20*w14+6.7573051948051951e-04*w5*Jinv21-1.1870941558441559e-04*Jinv02*w54+6.7573051948051951e-04*w4*Jinv12+2.5568181818181820e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-8.3198051948051951e-05*Jinv21*w29-2.9220779220779221e-04*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+1.0227272727272728e-03*Jinv02*w12-1.1870941558441559e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-3.6525974025974026e-05*w47*Jinv21+2.7394480519480521e-05*Jinv20*w38+2.5568181818181820e-04*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11+6.7573051948051951e-04*w31*Jinv12-3.6525974025974026e-05*w46*Jinv12+2.7394480519480521e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8+2.5568181818181820e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50+2.5568181818181820e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7+2.5568181818181820e-04*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51+6.7573051948051951e-04*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01+2.7394480519480521e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+2.7394480519480521e-05*w37*Jinv10-1.1870941558441559e-04*Jinv21*w56-3.6525974025974026e-05*w24*Jinv00-8.3198051948051951e-05*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-1.1870941558441559e-04*w19*Jinv12+6.7573051948051951e-04*Jinv22*w5-3.6525974025974026e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+1.0227272727272728e-03*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-1.1870941558441559e-04*Jinv02*w18+1.0227272727272728e-03*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59-2.9220779220779221e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00-2.9220779220779221e-04*w35*Jinv20+2.5568181818181820e-04*Jinv21*w23-1.1870941558441559e-04*Jinv01*w54-1.1870941558441559e-04*w55*Jinv11-3.6525974025974026e-05*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17+6.7573051948051951e-04*w30*Jinv00+3.2873376623376621e-04*w40*Jinv10-1.1870941558441559e-04*w20*Jinv20-8.3198051948051951e-05*Jinv22*w29+1.0227272727272728e-03*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11+2.7394480519480521e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+6.7573051948051951e-04*w5*Jinv20+6.7573051948051951e-04*Jinv01*w30+2.7394480519480521e-05*Jinv02*w51-1.1870941558441559e-04*Jinv22*w56-8.3198051948051951e-05*Jinv12*w28-3.6525974025974026e-05*w25*Jinv11-1.1870941558441559e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-3.6525974025974026e-05*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-8.3198051948051951e-05*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[22][2]=(0.0)*D;
  A[22][3]=(0.0)*D;
  A[22][4]=(-1.3971185064935066e-03*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-4.6570616883116886e-04*Jinv22*w53+9.5880681818181818e-04*Jinv20*w8+3.1960227272727273e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-1.3697240259740260e-03*w32*Jinv20+2.0271915584415584e-03*Jinv01*w3-3.2873376623376621e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38+2.7394480519480518e-04*Jinv00*w54+3.1960227272727273e-04*Jinv21*w59-3.8352272727272725e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+3.2873376623376621e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+2.7394480519480520e-03*w16*Jinv11+2.7394480519480520e-03*Jinv21*w17-1.9176136363636362e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55-1.0957792207792208e-04*w58*Jinv10+2.7394480519480520e-03*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24+2.5202922077922079e-03*Jinv22*w35+3.1960227272727273e-04*w58*Jinv11+3.1960227272727273e-04*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-6.6659902597402602e-04*w1*Jinv10+6.0267857142857139e-04*w16*Jinv10-1.3697240259740259e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+2.7394480519480520e-03*w15*Jinv01+4.8396915584415583e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+2.7394480519480520e-03*Jinv22*w17+1.2053571428571428e-03*Jinv00*w33-1.4793019480519481e-03*w22*Jinv12-1.3697240259740260e-03*w31*Jinv10+2.0271915584415584e-03*Jinv02*w3-3.2873376623376621e-04*w21*Jinv00+1.0135957792207792e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+2.7394480519480521e-05*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38-1.9176136363636362e-04*Jinv02*w45-3.8352272727272725e-04*Jinv01*w24+3.1960227272727273e-04*Jinv22*w59+2.5202922077922079e-03*w34*Jinv11-1.3697240259740259e-04*w19*Jinv11-4.6570616883116886e-04*Jinv21*w53-1.9176136363636362e-04*w51*Jinv00+2.5202922077922079e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+2.0271915584415584e-03*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10+1.2053571428571428e-03*w34*Jinv10+1.0135957792207792e-03*w32*Jinv21-1.3697240259740259e-04*Jinv21*w20+9.5880681818181818e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12+2.7394480519480518e-04*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-4.6570616883116886e-04*w52*Jinv12+2.5202922077922079e-03*Jinv21*w35+1.0135957792207792e-03*w31*Jinv11-3.8352272727272725e-04*Jinv22*w26-1.9176136363636362e-04*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14+2.0271915584415584e-03*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54+2.0271915584415584e-03*w4*Jinv12-1.4793019480519481e-03*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.8396915584415583e-04*Jinv21*w29+2.5202922077922079e-03*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12-1.3697240259740259e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01+9.5880681818181818e-04*w7*Jinv10-1.9176136363636362e-04*w47*Jinv21+2.7394480519480521e-05*Jinv20*w38-1.4793019480519481e-03*Jinv22*w23-4.6570616883116886e-04*w52*Jinv11+1.0135957792207792e-03*w31*Jinv12-1.9176136363636362e-04*w46*Jinv12-1.9176136363636362e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-3.2873376623376621e-04*w22*Jinv10-8.4922889610389610e-04*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01-4.6570616883116886e-04*Jinv01*w51+1.0135957792207792e-03*Jinv02*w30-8.4922889610389610e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-1.9176136363636362e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.7394480519480521e-05*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+4.8396915584415583e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-1.3697240259740259e-04*w19*Jinv12+2.0271915584415584e-03*Jinv22*w5-1.9176136363636362e-04*w47*Jinv22-8.4922889610389610e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+3.1960227272727273e-04*w58*Jinv12-1.3697240259740259e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+2.5202922077922079e-03*Jinv02*w33-6.6659902597402602e-04*w0*Jinv00+1.2053571428571428e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-6.6659902597402602e-04*Jinv20*w2+6.0267857142857139e-04*w15*Jinv00-8.4922889610389610e-04*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+6.0267857142857139e-04*Jinv20*w17-1.3697240259740260e-03*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+4.8396915584415583e-04*Jinv22*w29+1.2327516233766234e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+1.0135957792207792e-03*Jinv01*w30-4.6570616883116886e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+4.8396915584415583e-04*Jinv12*w28-3.8352272727272725e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+2.7394480519480520e-03*w16*Jinv12-8.4922889610389610e-04*Jinv21*w50+4.8396915584415583e-04*Jinv01*w27-8.4922889610389610e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[22][5]=(0.0)*D;
  A[22][6]=(0.0)*D;
  A[22][7]=(9.5880681818181818e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+9.5880681818181818e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00+1.1231737012987013e-03*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-1.4245129870129869e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+2.7394480519480518e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2-3.2873376623376621e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+3.5612824675324673e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17-1.2601461038961039e-03*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12+1.1231737012987013e-03*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3-1.4245129870129869e-03*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+3.5612824675324673e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53+1.9176136363636362e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13-1.2601461038961039e-03*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+2.7394480519480518e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+9.5880681818181818e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+7.1225649350649346e-04*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+1.9176136363636362e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8-1.4245129870129869e-03*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+1.9176136363636362e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-2.7394480519480518e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+3.5612824675324673e-04*w0*Jinv00-1.2601461038961039e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10+3.5612824675324673e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-1.3697240259740260e-03*Jinv20*w17+1.1231737012987013e-03*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[22][8]=(0.0)*D;
  A[22][9]=(0.0)*D;
  A[22][10]=(-5.8441558441558442e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-5.8441558441558442e-04*w13*Jinv10+1.1870941558441559e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20+1.4975649350649350e-03*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+2.1915584415584417e-04*w43*Jinv10+1.0247564935064935e-04*w58*Jinv10-2.1915584415584417e-04*w48*Jinv00-1.4204545454545455e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+7.4878246753246752e-04*w45*Jinv00+6.4935064935064935e-05*Jinv00*w27-2.1915584415584417e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10+3.4699675324675324e-04*Jinv00*w33-4.2918019480519480e-04*w31*Jinv10+1.4975649350649350e-03*w21*Jinv00-2.6481331168831169e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.7394480519480521e-05*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-2.6481331168831169e-04*w19*Jinv10+3.4699675324675324e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+3.6525974025974026e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-5.8441558441558442e-04*Jinv20*w14-2.1915584415584417e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-1.0957792207792208e-04*w7*Jinv10-5.4788961038961042e-05*Jinv20*w38-2.7394480519480521e-05*Jinv20*w53+1.4975649350649350e-03*w22*Jinv10-2.7394480519480521e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10+3.6525974025974026e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+2.1915584415584417e-04*Jinv20*w44+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+3.4699675324675324e-04*w35*Jinv20+3.6525974025974026e-05*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+4.3831168831168834e-04*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-2.6481331168831169e-04*w20*Jinv20+1.1870941558441559e-04*w5*Jinv20-4.2918019480519480e-04*w55*Jinv10+2.1915584415584417e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20)*D;
  A[22][11]=(0.0)*D;
  A[22][12]=(0.0)*D;
  A[22][13]=(1.3149350649350649e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+1.5523538961038962e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.2327516233766234e-03*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3-3.7804383116883116e-03*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12+1.4610389610389610e-04*w0*Jinv02-4.0178571428571428e-04*Jinv21*w2+8.2183441558441553e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21-1.4793019480519481e-03*w16*Jinv11-1.4793019480519481e-03*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45-8.2183441558441553e-05*Jinv12*w55+1.5523538961038962e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.5523538961038962e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26-4.0178571428571428e-04*w0*Jinv01+1.4610389610389610e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00-1.4793019480519481e-03*Jinv21*w41+8.2183441558441553e-04*w42*Jinv02+1.0957792207792207e-03*Jinv00*w27+1.5523538961038962e-04*w11*Jinv22-4.0178571428571428e-04*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11-1.4793019480519481e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-3.7804383116883116e-03*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3-3.7804383116883116e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-1.4793019480519481e-03*Jinv01*w39+1.4610389610389610e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+1.5523538961038962e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+8.2183441558441553e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.1505681818181819e-03*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20+1.5523538961038962e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21-8.2183441558441553e-05*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-3.7804383116883116e-03*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+4.3831168831168834e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.5523538961038962e-04*Jinv02*w9+9.5880681818181818e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-3.7804383116883116e-03*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-3.7804383116883116e-03*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-2.6298701298701297e-03*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-2.6298701298701297e-03*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10+9.5880681818181818e-04*w7*Jinv11+1.5523538961038962e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10+4.3831168831168834e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00+1.0957792207792207e-03*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12+1.2327516233766234e-03*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22+9.5880681818181818e-04*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.5523538961038962e-04*w10*Jinv10+8.2183441558441553e-04*Jinv20*w44+1.5523538961038962e-04*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+9.5880681818181818e-04*Jinv21*w8+1.5523538961038962e-04*w57*Jinv00+1.5523538961038962e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33+1.4610389610389610e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20-2.6298701298701297e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10+1.4610389610389610e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20+1.0957792207792207e-03*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14-1.4793019480519481e-03*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+8.2183441558441553e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-8.2183441558441553e-05*w55*Jinv10+8.2183441558441553e-04*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+9.5880681818181818e-04*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27+9.5880681818181818e-04*w48*Jinv01+1.4610389610389610e-04*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[22][14]=(0.0)*D;
  A[22][15]=(0.0)*D;
  A[22][16]=(-2.7942370129870131e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+6.0267857142857139e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-2.7942370129870131e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-1.7532467532467534e-03*w32*Jinv20-1.8628246753246754e-03*Jinv01*w3+8.5470779220779228e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38-1.3149350649350649e-03*Jinv00*w54-6.9399350649350649e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-2.3011363636363637e-03*w42*Jinv01-5.6615259740259744e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17-1.0409902597402598e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55+2.7394480519480518e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-6.9399350649350649e-04*w58*Jinv11-6.9399350649350649e-04*w57*Jinv01-2.0819805194805196e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+1.6436688311688311e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-2.9586038961038961e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1-5.9172077922077923e-03*w15*Jinv01-4.3831168831168834e-04*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12-1.7532467532467534e-03*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3+8.5470779220779228e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-2.9586038961038961e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-4.9310064935064932e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00+3.2873376623376621e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-2.0819805194805196e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10-1.8628246753246754e-03*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13-4.9310064935064932e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-9.3141233766233771e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+6.0267857142857139e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12-1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41+5.4788961038961042e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35-9.3141233766233771e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-1.0409902597402598e-03*w46*Jinv11-2.7942370129870131e-03*Jinv20*w14-1.8628246753246754e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.3831168831168834e-04*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.2873376623376621e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+6.0267857142857139e-04*w6*Jinv01+6.0267857142857139e-04*w7*Jinv10-1.0409902597402598e-03*w47*Jinv21+3.2873376623376621e-04*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+7.6704545454545450e-04*Jinv22*w8+8.5470779220779228e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50-9.8620129870129864e-04*w22*Jinv11+7.6704545454545450e-04*Jinv12*w7-9.8620129870129864e-04*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+3.2873376623376621e-04*w36*Jinv01+6.0267857142857139e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10+1.2601461038961039e-03*Jinv21*w56+5.4788961038961036e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+1.2053571428571428e-03*w49*Jinv11+5.4788961038961042e-05*w10*Jinv10-2.7394480519480518e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+6.0267857142857139e-04*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+1.6436688311688311e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20-9.8620129870129864e-04*Jinv21*w23+1.2601461038961039e-03*Jinv01*w54+1.2601461038961039e-03*w55*Jinv11+1.6436688311688311e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.6615259740259744e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.7532467532467534e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-4.9310064935064932e-04*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14-2.9586038961038961e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20-9.3141233766233771e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-2.0819805194805196e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+1.2053571428571428e-03*Jinv21*w50-4.3831168831168834e-04*Jinv01*w27+1.2053571428571428e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-5.6615259740259744e-04*w10*Jinv11)*D;
  A[22][17]=(0.0)*D;
  A[22][18]=(0.0)*D;
  A[22][19]=(1.4793019480519481e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8+1.4793019480519481e-03*w13*Jinv10-3.8352272727272725e-04*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.2049512987012989e-04*Jinv01*w3-4.7666396103896100e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-8.2183441558441553e-04*w43*Jinv10-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-1.6436688311688311e-03*w16*Jinv10-3.8352272727272725e-04*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-4.7666396103896100e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39+5.7528409090909093e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38-1.6436688311688311e-04*Jinv01*w24-5.4788961038961042e-05*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00+1.3697240259740259e-04*w4*Jinv10+5.2049512987012989e-04*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.7528409090909093e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-5.4788961038961042e-05*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11+1.4793019480519481e-03*Jinv20*w14+5.2049512987012989e-04*w5*Jinv21+5.4788961038961036e-04*w49*Jinv10+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.6436688311688311e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+2.7394480519480518e-04*w52*Jinv11+2.4655032467532466e-04*Jinv20*w53-4.7666396103896100e-03*w22*Jinv10-7.1225649350649346e-04*w22*Jinv11-7.1225649350649346e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-3.5612824675324673e-04*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11-2.1002435064935065e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+8.2183441558441553e-05*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-2.1002435064935065e-04*w10*Jinv10-8.2183441558441553e-04*Jinv20*w44+5.2049512987012989e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-3.1047077922077924e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20-7.1225649350649346e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-1.6436688311688311e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-1.6436688311688311e-03*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+5.7528409090909093e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+1.3697240259740259e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-8.2183441558441553e-04*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+8.2183441558441553e-05*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27+8.2183441558441553e-05*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[22][20]=(0.0)*D;
  A[22][21]=(0.0)*D;
  A[22][22]=(2.6298701298701297e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-4.2735389610389614e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+2.6298701298701297e-03*w13*Jinv10-4.2735389610389614e-03*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+7.3965097402597403e-04*Jinv01*w3-7.3965097402597406e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54-3.6982548701298703e-03*w25*Jinv12-7.8530844155844158e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45+2.3833198051948050e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00-3.6982548701298703e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-1.2327516233766234e-03*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-7.8530844155844158e-04*w1*Jinv10-4.2735389610389614e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-3.6982548701298703e-03*w45*Jinv00+4.9310064935064932e-04*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01-1.1870941558441558e-03*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-3.6982548701298703e-03*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-7.3965097402597406e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3-7.3965097402597406e-03*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-7.8530844155844158e-04*Jinv22*w2+2.3833198051948050e-03*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.6982548701298703e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+3.5612824675324673e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+7.3965097402597403e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+2.6298701298701297e-03*Jinv12*w13+2.3833198051948050e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+7.3965097402597403e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-4.2735389610389614e-03*Jinv22*w41-3.6982548701298703e-03*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+7.3965097402597403e-04*w31*Jinv11-3.6982548701298703e-03*Jinv22*w26-1.2327516233766234e-03*w46*Jinv11+2.6298701298701297e-03*Jinv20*w14+7.3965097402597403e-04*w5*Jinv21+2.3833198051948050e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-7.3965097402597406e-03*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-1.1870941558441558e-03*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+2.6298701298701297e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38-7.3965097402597406e-03*Jinv22*w23+1.6436688311688311e-04*w31*Jinv12-3.6982548701298703e-03*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8-7.3965097402597406e-03*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*Jinv12*w7+1.6436688311688311e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+3.5612824675324673e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-3.6982548701298703e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-3.6982548701298703e-03*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.3149350649350649e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.8530844155844158e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-3.6982548701298703e-03*w25*Jinv10-7.8530844155844158e-04*Jinv20*w2-4.2735389610389614e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-4.2735389610389614e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+2.3833198051948050e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.6298701298701297e-03*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+7.3965097402597403e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+2.3833198051948050e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-3.6982548701298703e-03*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50-1.1870941558441558e-03*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-7.8530844155844158e-04*Jinv12*w1)*D;
  A[22][23]=(0.0)*D;
  A[22][24]=(0.0)*D;
  A[22][25]=(-2.6298701298701297e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-7.1225649350649346e-04*Jinv20*w8-2.6298701298701297e-03*w13*Jinv10+1.5888798701298701e-03*Jinv21*w44-1.6436688311688311e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+7.3965097402597406e-03*Jinv20*w23-2.3833198051948050e-03*Jinv00*w54+3.7439123376623376e-04*Jinv21*w59+3.9265422077922079e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+1.5888798701298701e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+3.1777597402597403e-03*w16*Jinv11+3.1777597402597403e-03*Jinv21*w17+1.2327516233766234e-03*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+3.7439123376623376e-04*w58*Jinv11+3.7439123376623376e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.9265422077922079e-04*w0*Jinv01+7.8530844155844158e-04*w1*Jinv10+4.2735389610389614e-03*w16*Jinv10+1.5888798701298701e-03*w43*Jinv11+3.6982548701298703e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+3.9265422077922079e-04*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11+3.1777597402597403e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+3.6982548701298703e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-1.6436688311688311e-04*w31*Jinv10+7.3965097402597406e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.3833198051948050e-03*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+2.4655032467532467e-03*Jinv01*w24+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00-1.6436688311688311e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11+4.2735389610389614e-03*w41*Jinv20-2.3833198051948050e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-7.1225649350649346e-04*w6*Jinv00-2.3833198051948050e-03*Jinv20*w56+3.6982548701298703e-03*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+1.0957792207792207e-03*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+1.2327516233766234e-03*w46*Jinv11-2.6298701298701297e-03*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33+4.2735389610389614e-03*w39*Jinv00-1.2327516233766234e-03*Jinv01*w18-8.2183441558441553e-05*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-7.1225649350649346e-04*w7*Jinv10+1.2327516233766234e-03*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38-3.5612824675324673e-04*w52*Jinv11-3.5612824675324673e-04*Jinv20*w53+7.3965097402597406e-03*w22*Jinv10+2.4655032467532467e-03*w22*Jinv11+2.4655032467532467e-03*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-8.2183441558441553e-05*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+7.4878246753246752e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10-1.0135957792207792e-03*Jinv21*w56+3.6982548701298703e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+7.4878246753246752e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44-1.2327516233766234e-03*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.8530844155844158e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+2.4655032467532467e-03*Jinv21*w23-1.0135957792207792e-03*Jinv01*w54-1.0135957792207792e-03*w55*Jinv11+3.6982548701298703e-03*w25*Jinv10+7.8530844155844158e-04*Jinv20*w2+4.2735389610389614e-03*w15*Jinv00+3.9265422077922079e-04*Jinv01*w9+4.2735389610389614e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-2.3833198051948050e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-1.6436688311688311e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30+2.4655032467532467e-03*w25*Jinv11-2.3833198051948050e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+3.6982548701298703e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+3.9265422077922079e-04*w10*Jinv11)*D;
  A[22][26]=(0.0)*D;
  A[22][27]=(0.0)*D;
  A[22][28]=(3.2873376623376621e-04*Jinv21*w44-2.6481331168831169e-04*Jinv01*w3+1.5726461038961039e-04*Jinv21*w59+2.0495129870129870e-04*Jinv21*w2+3.2873376623376621e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.1870941558441558e-03*Jinv01*w45+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+1.1870941558441558e-03*Jinv21*w26+2.0495129870129870e-04*w0*Jinv01+3.2873376623376621e-04*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+2.0495129870129870e-04*Jinv11*w1-1.0227272727272728e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+6.5746753246753243e-04*Jinv01*w39-5.4788961038961042e-05*Jinv21*w38+1.1870941558441558e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-6.7573051948051951e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-2.6481331168831169e-04*w4*Jinv11-2.6481331168831169e-04*w32*Jinv21-6.7573051948051951e-04*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-2.6481331168831169e-04*w31*Jinv11+1.1870941558441558e-03*w46*Jinv11-2.6481331168831169e-04*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-6.7573051948051951e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-1.0957792207792208e-04*w6*Jinv01+1.1870941558441558e-03*w47*Jinv21-5.4788961038961042e-05*w52*Jinv11+2.3741883116883116e-03*w22*Jinv11+2.3741883116883116e-03*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-5.4788961038961042e-05*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-6.7573051948051951e-04*Jinv21*w56-1.0957792207792208e-04*w49*Jinv11-1.0227272727272728e-03*Jinv21*w14-1.0227272727272728e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+2.3741883116883116e-03*Jinv21*w23-6.7573051948051951e-04*Jinv01*w54-6.7573051948051951e-04*w55*Jinv11+1.5726461038961039e-04*Jinv01*w9+6.5746753246753243e-04*w40*Jinv11-2.6481331168831169e-04*Jinv01*w30+1.1870941558441558e-03*w25*Jinv11-1.0957792207792208e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+1.5726461038961039e-04*w10*Jinv11)*D;
  A[22][29]=(0.0)*D;
  A[22][30]=(0.0)*D;
  A[22][31]=(1.2327516233766234e-03*Jinv00*w12+4.8396915584415583e-04*w28*Jinv10+6.0267857142857139e-04*w40*Jinv12+2.7394480519480521e-05*Jinv22*w53-8.4922889610389610e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10+6.0267857142857139e-04*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44+1.0135957792207792e-03*w3*Jinv00+2.0271915584415584e-03*w32*Jinv20+1.0135957792207792e-03*Jinv01*w3-1.4793019480519481e-03*Jinv20*w23-1.9176136363636362e-04*Jinv22*w38-1.3697240259740259e-04*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12-6.6659902597402602e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+3.1960227272727273e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+3.8352272727272725e-04*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+1.2053571428571428e-03*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01-1.9176136363636362e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10+2.7394480519480518e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-3.8352272727272725e-04*w45*Jinv00+2.7394480519480520e-03*Jinv21*w41+3.2873376623376621e-04*w42*Jinv02+4.8396915584415583e-04*Jinv00*w27-1.0957792207792208e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+4.8396915584415583e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+4.8396915584415583e-04*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17+2.5202922077922079e-03*Jinv00*w33-3.2873376623376621e-04*w22*Jinv12+2.0271915584415584e-03*w31*Jinv10-1.3697240259740260e-03*Jinv02*w3-1.4793019480519481e-03*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+2.7394480519480520e-03*Jinv01*w39-6.6659902597402602e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-4.6570616883116886e-04*w36*Jinv00-4.6570616883116886e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-1.9176136363636362e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+2.5202922077922079e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.2053571428571428e-03*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44+1.0135957792207792e-03*w4*Jinv10+1.0135957792207792e-03*w4*Jinv11+2.7394480519480520e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+2.5202922077922079e-03*w34*Jinv10+2.0271915584415584e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-8.4922889610389610e-04*w6*Jinv00-1.9176136363636362e-04*w37*Jinv12-1.3697240259740259e-04*Jinv20*w56+6.0267857142857139e-04*Jinv22*w41-1.9176136363636362e-04*w26*Jinv20+3.1960227272727273e-04*w11*Jinv20+2.7394480519480521e-05*w52*Jinv12+2.5202922077922079e-03*Jinv21*w35+2.0271915584415584e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26-3.8352272727272725e-04*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14+1.0135957792207792e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54-1.3697240259740260e-03*w4*Jinv12-3.2873376623376621e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+4.8396915584415583e-04*Jinv21*w29+2.5202922077922079e-03*Jinv01*w33+2.7394480519480520e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-4.6570616883116886e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9-8.4922889610389610e-04*w6*Jinv01-8.4922889610389610e-04*w7*Jinv10-3.8352272727272725e-04*w47*Jinv21-4.6570616883116886e-04*Jinv20*w38-3.2873376623376621e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-1.4793019480519481e-03*w22*Jinv10+9.5880681818181818e-04*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+9.5880681818181818e-04*Jinv12*w49-4.6570616883116886e-04*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10-8.4922889610389610e-04*w7*Jinv11+3.1960227272727273e-04*w9*Jinv00-4.6570616883116886e-04*w37*Jinv10-1.3697240259740259e-04*Jinv21*w56-1.9176136363636362e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12-1.3697240259740260e-03*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+3.1960227272727273e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-8.4922889610389610e-04*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+1.2053571428571428e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+2.5202922077922079e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23-1.3697240259740259e-04*Jinv01*w54-1.3697240259740259e-04*w55*Jinv11-1.9176136363636362e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00+9.5880681818181818e-04*w48*Jinv02+3.1960227272727273e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+2.0271915584415584e-03*w30*Jinv00+2.7394480519480520e-03*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14+2.7394480519480520e-03*w40*Jinv11-1.9176136363636362e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.0135957792207792e-03*w5*Jinv20+2.0271915584415584e-03*Jinv01*w30+2.7394480519480521e-05*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.9176136363636362e-04*w25*Jinv11-1.3697240259740259e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-3.8352272727272725e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+4.8396915584415583e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01-6.6659902597402602e-04*Jinv12*w1+3.1960227272727273e-04*w10*Jinv11)*D;
  A[22][32]=(0.0)*D;
  A[22][33]=(0.0)*D;
  A[22][34]=(-1.7532467532467534e-03*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+8.2183441558441553e-04*Jinv22*w53+1.2053571428571428e-03*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02-1.7532467532467534e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44-1.2601461038961039e-03*w3*Jinv00-2.5750811688311690e-03*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+2.1915584415584414e-03*Jinv20*w23-3.8352272727272725e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.3149350649350649e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-1.0957792207792207e-03*Jinv01*w45+3.2873376623376621e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+1.0957792207792207e-03*Jinv02*w24-1.6436688311688311e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01-1.0957792207792207e-03*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10-3.8352272727272725e-04*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-4.9310064935064932e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-1.6436688311688311e-03*Jinv00*w33+2.1915584415584414e-03*w22*Jinv12-2.5750811688311690e-03*w31*Jinv10-2.5750811688311690e-03*Jinv02*w3+2.1915584415584414e-03*w21*Jinv00-1.2601461038961039e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+3.2873376623376621e-04*Jinv00*w18+8.2183441558441553e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-1.6436688311688311e-03*Jinv12*w34-1.2601461038961039e-03*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20-1.7532467532467534e-03*Jinv12*w13+3.2873376623376621e-04*w19*Jinv10-1.6436688311688311e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+1.2053571428571428e-03*w6*Jinv00-3.8352272727272725e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41-3.8352272727272725e-04*w11*Jinv20+8.2183441558441553e-04*w52*Jinv12+3.2873376623376621e-03*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26-1.0957792207792207e-03*w46*Jinv11-1.7532467532467534e-03*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21+3.2873376623376621e-04*Jinv02*w54-2.5750811688311690e-03*w4*Jinv12+2.1915584415584414e-03*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.7532467532467534e-03*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+1.2053571428571428e-03*w7*Jinv10-1.0957792207792207e-03*w47*Jinv21+8.2183441558441553e-04*Jinv20*w38+2.1915584415584414e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.2601461038961039e-03*w31*Jinv12-1.0957792207792208e-04*Jinv22*w8+2.1915584415584414e-03*w22*Jinv10+1.2053571428571428e-03*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-1.0957792207792207e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.2601461038961039e-03*Jinv02*w30+1.2053571428571428e-03*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+8.2183441558441553e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-4.9310064935064932e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12-2.5750811688311690e-03*Jinv22*w5+5.4788961038961042e-05*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10-3.8352272727272725e-04*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-1.6436688311688311e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-1.6436688311688311e-03*w35*Jinv20-1.0957792207792207e-03*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+2.5568181818181820e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+1.2053571428571428e-03*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-2.5750811688311690e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29-1.7532467532467534e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-1.2601461038961039e-03*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+8.2183441558441553e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28-1.0957792207792207e-03*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10+1.0957792207792207e-03*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[22][35]=(0.0)*D;
  A[22][36]=(0.0)*D;
  A[22][37]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-1.0957792207792208e-04*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+2.4655032467532466e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.2049512987012989e-04*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-7.1225649350649346e-04*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55-1.3697240259740259e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-7.1225649350649346e-04*w45*Jinv00-3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+5.4788961038961036e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17-8.7662337662337668e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10+5.2049512987012989e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv02*w45-1.0957792207792208e-04*Jinv22*w59+2.4655032467532466e-04*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+2.4655032467532466e-04*w4*Jinv10-1.0957792207792208e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10-8.7662337662337668e-04*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.2049512987012989e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.9176136363636362e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10-1.0957792207792208e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-3.5612824675324673e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49+2.4655032467532466e-04*w52*Jinv10+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+1.9176136363636362e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+6.3920454545454550e-05*w10*Jinv10-7.1225649350649346e-04*Jinv20*w44-1.0957792207792208e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-8.7662337662337668e-04*w35*Jinv20+1.9176136363636362e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-3.5612824675324673e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+2.4655032467532466e-04*w5*Jinv20+2.7394480519480518e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28+5.2049512987012989e-04*w55*Jinv10-7.1225649350649346e-04*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[22][38]=(0.0)*D;
  A[22][39]=(0.0)*D;
  A[22][40]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-9.3141233766233771e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+2.1367694805194807e-03*Jinv00*w54-5.6615259740259744e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+1.6436688311688311e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-6.9399350649350649e-04*w11*Jinv21-2.9586038961038961e-03*w16*Jinv11-2.9586038961038961e-03*Jinv21*w17-2.0819805194805196e-03*Jinv01*w45-4.9310064935064932e-04*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-5.6615259740259744e-04*w58*Jinv11-5.6615259740259744e-04*w57*Jinv01-1.0409902597402598e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10-1.3149350649350649e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41+5.4788961038961036e-04*Jinv00*w27+2.7394480519480518e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1-2.9586038961038961e-03*w15*Jinv01-4.3831168831168834e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+8.5470779220779228e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10-1.7532467532467534e-03*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39+1.6436688311688311e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38-1.0409902597402598e-03*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.2601461038961039e-03*w19*Jinv11+3.2873376623376621e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.8352272727272725e-04*w4*Jinv10-9.3141233766233771e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-2.7942370129870131e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-1.8628246753246754e-03*w32*Jinv21+1.2601461038961039e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20+3.2873376623376621e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35-1.8628246753246754e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26-2.0819805194805196e-03*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-9.3141233766233771e-04*w5*Jinv21-4.9310064935064932e-04*Jinv02*w54-1.7532467532467534e-03*w4*Jinv12+8.5470779220779228e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10-4.3831168831168834e-04*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-2.7942370129870131e-03*Jinv02*w12+1.2601461038961039e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9+1.2053571428571428e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-2.0819805194805196e-03*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38+8.5470779220779228e-03*Jinv22*w23+3.2873376623376621e-04*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12+3.8352272727272725e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+6.0267857142857139e-04*Jinv22*w50-9.8620129870129864e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-9.8620129870129864e-04*w21*Jinv01+3.2873376623376621e-04*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+6.0267857142857139e-04*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+1.2053571428571428e-03*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-1.7532467532467534e-03*Jinv22*w5+6.0267857142857139e-04*w49*Jinv11-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18+1.2053571428571428e-03*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-9.8620129870129864e-04*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+6.0267857142857139e-04*w48*Jinv02-6.9399350649350649e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-2.7942370129870131e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-1.8628246753246754e-03*Jinv01*w30+3.2873376623376621e-04*Jinv02*w51-4.9310064935064932e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.0409902597402598e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+6.0267857142857139e-04*Jinv21*w50-4.3831168831168834e-04*Jinv01*w27+6.0267857142857139e-04*w48*Jinv01+1.6436688311688311e-04*Jinv12*w1-6.9399350649350649e-04*w10*Jinv11)*D;
  A[22][41]=(0.0)*D;
  A[22][42]=(0.0)*D;
  A[22][43]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02-3.6525974025974026e-05*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+1.0957792207792207e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+1.0957792207792207e-03*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22-3.6525974025974026e-05*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+4.2735389610389614e-03*w22*Jinv12-7.6704545454545450e-04*w31*Jinv10-7.6704545454545450e-04*Jinv02*w3+4.2735389610389614e-03*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-1.3149350649350649e-03*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+1.0957792207792207e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59-3.2873376623376621e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-2.1367694805194807e-03*Jinv12*w13-1.3149350649350649e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+2.1367694805194807e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20-5.4788961038961042e-05*w52*Jinv12-3.2873376623376621e-03*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+1.0957792207792207e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-3.2873376623376621e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-2.1367694805194807e-03*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.0957792207792207e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38+4.2735389610389614e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8+4.2735389610389614e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10-5.4788961038961042e-05*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-5.4788961038961042e-05*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.0957792207792207e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-5.4788961038961042e-05*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[22][44]=(0.0)*D;
  A[22][45]=(0.0)*D;
  A[22][46]=(4.2735389610389614e-03*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+7.4878246753246752e-04*w57*Jinv02+4.2735389610389614e-03*Jinv02*w39+1.5888798701298701e-03*Jinv21*w44-8.2183441558441553e-05*Jinv01*w3-3.5612824675324673e-04*Jinv22*w38+3.9265422077922079e-04*Jinv21*w59+3.6982548701298703e-03*w25*Jinv12+7.8530844155844158e-04*w0*Jinv02+3.9265422077922079e-04*Jinv21*w2+1.5888798701298701e-03*w42*Jinv01+3.7439123376623376e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45-2.3833198051948050e-03*Jinv12*w55+4.2735389610389614e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+3.6982548701298703e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+1.2327516233766234e-03*Jinv21*w26+3.9265422077922079e-04*w0*Jinv01-2.3833198051948050e-03*Jinv22*w20+1.5888798701298701e-03*w43*Jinv11+3.1777597402597403e-03*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+3.9265422077922079e-04*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11+4.2735389610389614e-03*Jinv22*w17+7.3965097402597406e-03*w22*Jinv12-1.6436688311688311e-04*Jinv02*w3-1.6436688311688311e-04*w32*Jinv22+3.1777597402597403e-03*Jinv01*w39+7.8530844155844158e-04*Jinv22*w2-3.5612824675324673e-04*Jinv21*w38+3.6982548701298703e-03*Jinv02*w45+1.2327516233766234e-03*Jinv01*w24+7.4878246753246752e-04*Jinv22*w59+1.0957792207792207e-03*w34*Jinv11-1.0135957792207792e-03*w19*Jinv11-8.2183441558441553e-05*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-8.2183441558441553e-05*w4*Jinv11-2.6298701298701297e-03*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21-1.0135957792207792e-03*Jinv21*w20-3.5612824675324673e-04*w37*Jinv12+4.2735389610389614e-03*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12+1.0957792207792207e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+3.6982548701298703e-03*Jinv22*w26+2.4655032467532467e-03*w46*Jinv11-8.2183441558441553e-05*w5*Jinv21-2.3833198051948050e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+7.3965097402597406e-03*w21*Jinv02-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33-2.6298701298701297e-03*Jinv02*w12-1.0135957792207792e-03*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+2.4655032467532467e-03*w47*Jinv21+7.3965097402597406e-03*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11-1.6436688311688311e-04*w31*Jinv12+3.6982548701298703e-03*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-7.1225649350649346e-04*Jinv22*w50+2.4655032467532467e-03*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+2.4655032467532467e-03*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51-1.6436688311688311e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-1.2327516233766234e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+3.6982548701298703e-03*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+7.4878246753246752e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+2.4655032467532467e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11-7.1225649350649346e-04*w48*Jinv02+3.7439123376623376e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.6298701298701297e-03*Jinv22*w14+3.1777597402597403e-03*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-2.3833198051948050e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+1.2327516233766234e-03*w25*Jinv11+4.2735389610389614e-03*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+7.8530844155844158e-04*Jinv12*w1+3.7439123376623376e-04*w10*Jinv11)*D;
  A[22][47]=(0.0)*D;
  A[22][48]=(0.0)*D;
  A[22][49]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-8.2183441558441553e-05*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+1.9176136363636362e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+3.5612824675324673e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24-1.2601461038961039e-03*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+2.7394480519480518e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41-3.2873376623376621e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-1.0957792207792208e-04*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33-1.4245129870129869e-03*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10+1.1231737012987013e-03*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39+3.5612824675324673e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-1.2601461038961039e-03*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+9.5880681818181818e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+1.9176136363636362e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+5.4788961038961042e-05*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-5.2049512987012989e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+1.1231737012987013e-03*w4*Jinv12-1.4245129870129869e-03*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+9.5880681818181818e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38-1.4245129870129869e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12+1.1231737012987013e-03*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-1.2601461038961039e-03*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+9.5880681818181818e-04*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11+1.9176136363636362e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30+9.1314935064935065e-06*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+3.5612824675324673e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[22][50]=(0.0)*D;
  A[22][51]=(0.0)*D;
  A[22][52]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+6.3920454545454550e-05*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-7.1225649350649346e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-05*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24-8.7662337662337668e-04*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+5.2049512987012989e-04*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27-1.3697240259740259e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+5.2049512987012989e-04*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+2.4655032467532466e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+1.9176136363636362e-04*Jinv02*w45+6.3920454545454550e-05*Jinv22*w59-3.5612824675324673e-04*w51*Jinv00-8.7662337662337668e-04*Jinv12*w34-7.1225649350649346e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-7.6704545454545450e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+8.2183441558441553e-05*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12-7.1225649350649346e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-8.2183441558441553e-05*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10-7.6704545454545450e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-1.3697240259740259e-04*Jinv02*w9+8.2183441558441553e-05*w7*Jinv10+2.7394480519480518e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+2.4655032467532466e-04*w31*Jinv12+1.9176136363636362e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+5.4788961038961036e-04*Jinv12*w7+2.4655032467532466e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-3.5612824675324673e-04*w52*Jinv10-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10-3.5612824675324673e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.9176136363636362e-04*w47*Jinv22-1.0957792207792208e-04*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+6.3920454545454550e-05*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-8.7662337662337668e-04*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14+2.4655032467532466e-04*Jinv02*w36-7.1225649350649346e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20-3.5612824675324673e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[22][53]=(0.0)*D;
  A[22][54]=(0.0)*D;
  A[22][55]=(-1.6436688311688311e-03*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-2.1002435064935065e-04*w57*Jinv02-1.6436688311688311e-03*Jinv02*w39-3.8352272727272725e-04*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+2.4655032467532466e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+5.7528409090909093e-04*Jinv12*w55-2.6298701298701297e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-3.8352272727272725e-04*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-8.2183441558441553e-04*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11-2.6298701298701297e-03*Jinv22*w17-4.7666396103896100e-03*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+1.3697240259740259e-04*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39-3.1047077922077924e-04*Jinv22*w2+2.7394480519480518e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-2.1002435064935065e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-8.2183441558441553e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+1.4793019480519481e-03*Jinv12*w13+5.2049512987012989e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+2.4655032467532466e-04*w37*Jinv12-1.6436688311688311e-03*Jinv22*w41-8.2183441558441553e-05*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21+5.7528409090909093e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-4.7666396103896100e-03*w21*Jinv02+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+1.4793019480519481e-03*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+8.2183441558441553e-05*w6*Jinv01-1.6436688311688311e-04*w47*Jinv21-4.7666396103896100e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+1.3697240259740259e-04*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50-7.1225649350649346e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-7.1225649350649346e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+1.3697240259740259e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+8.2183441558441553e-05*w7*Jinv11-1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-2.1002435064935065e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+5.2049512987012989e-04*Jinv01*w12+8.2183441558441553e-05*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33-7.1225649350649346e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.4793019480519481e-03*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36-8.2183441558441553e-04*Jinv12*w43+5.2049512987012989e-04*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51+5.7528409090909093e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-2.6298701298701297e-03*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[22][56]=(0.0)*D;
  A[22][57]=(0.0)*D;
  A[22][58]=(4.3831168831168834e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39-2.7394480519480521e-05*Jinv22*w38+7.4878246753246752e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02-2.6481331168831169e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24+3.4699675324675324e-04*Jinv22*w35-4.2918019480519480e-04*Jinv22*w20+2.1915584415584417e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17+1.4975649350649350e-03*w22*Jinv12-4.2918019480519480e-04*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2+3.6525974025974026e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+3.4699675324675324e-04*Jinv12*w34+2.1915584415584417e-04*Jinv22*w44-5.8441558441558442e-04*Jinv12*w13-2.7394480519480521e-05*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12+7.4878246753246752e-04*Jinv22*w26-2.6481331168831169e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12+1.4975649350649350e-03*w21*Jinv02-5.8441558441558442e-04*Jinv02*w12+1.0247564935064935e-04*Jinv02*w9+1.4975649350649350e-03*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-2.1915584415584417e-04*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50-2.1915584415584417e-04*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+6.4935064935064935e-05*Jinv02*w27-2.1915584415584417e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+3.4699675324675324e-04*Jinv02*w33-1.0957792207792208e-04*w48*Jinv02+6.4935064935064935e-05*Jinv22*w29-5.8441558441558442e-04*Jinv22*w14-2.7394480519480521e-05*Jinv02*w36+2.1915584415584417e-04*Jinv12*w43-5.4788961038961042e-05*Jinv02*w51-2.6481331168831169e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[22][59]=(0.0)*D;
  A[23][0]=(0.0)*D;
  A[23][1]=(0.0)*D;
  A[23][2]=(1.0227272727272728e-03*Jinv00*w12-8.3198051948051951e-05*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12+2.7394480519480521e-05*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02+1.0227272727272728e-03*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+6.7573051948051951e-04*w3*Jinv00+6.7573051948051951e-04*w32*Jinv20+6.7573051948051951e-04*Jinv01*w3+2.5568181818181820e-04*Jinv20*w23+2.7394480519480521e-05*Jinv22*w38-1.1870941558441559e-04*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59-3.6525974025974026e-05*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17-3.6525974025974026e-05*Jinv01*w45-1.1870941558441559e-04*Jinv12*w55+1.1668019480519480e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-3.6525974025974026e-05*Jinv02*w24-2.9220779220779221e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01-3.6525974025974026e-05*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-1.1870941558441559e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-3.6525974025974026e-05*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02-8.3198051948051951e-05*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+1.0227272727272728e-03*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01-8.3198051948051951e-05*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-8.3198051948051951e-05*Jinv20*w29-3.6525974025974026e-05*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17-2.9220779220779221e-04*Jinv00*w33+2.5568181818181820e-04*w22*Jinv12+6.7573051948051951e-04*w31*Jinv10+6.7573051948051951e-04*Jinv02*w3+2.5568181818181820e-04*w21*Jinv00+6.7573051948051951e-04*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2-1.1870941558441559e-04*Jinv00*w18+2.7394480519480521e-05*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38-3.6525974025974026e-05*Jinv02*w45-3.6525974025974026e-05*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-2.9220779220779221e-04*w34*Jinv11-1.1870941558441559e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53+2.7394480519480521e-05*w51*Jinv00-2.9220779220779221e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+6.7573051948051951e-04*w4*Jinv10+6.7573051948051951e-04*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+1.0227272727272728e-03*Jinv12*w13-1.1870941558441559e-04*w19*Jinv10-2.9220779220779221e-04*w34*Jinv10+6.7573051948051951e-04*w32*Jinv21-1.1870941558441559e-04*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00+2.7394480519480521e-05*w37*Jinv12-1.1870941558441559e-04*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41-3.6525974025974026e-05*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20+2.7394480519480521e-05*w52*Jinv12-2.9220779220779221e-04*Jinv21*w35+6.7573051948051951e-04*w31*Jinv11-3.6525974025974026e-05*Jinv22*w26-3.6525974025974026e-05*w46*Jinv11+1.0227272727272728e-03*Jinv20*w14+6.7573051948051951e-04*w5*Jinv21-1.1870941558441559e-04*Jinv02*w54+6.7573051948051951e-04*w4*Jinv12+2.5568181818181820e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-8.3198051948051951e-05*Jinv21*w29-2.9220779220779221e-04*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+1.0227272727272728e-03*Jinv02*w12-1.1870941558441559e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-3.6525974025974026e-05*w47*Jinv21+2.7394480519480521e-05*Jinv20*w38+2.5568181818181820e-04*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11+6.7573051948051951e-04*w31*Jinv12-3.6525974025974026e-05*w46*Jinv12+2.7394480519480521e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8+2.5568181818181820e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50+2.5568181818181820e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7+2.5568181818181820e-04*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51+6.7573051948051951e-04*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01+2.7394480519480521e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+2.7394480519480521e-05*w37*Jinv10-1.1870941558441559e-04*Jinv21*w56-3.6525974025974026e-05*w24*Jinv00-8.3198051948051951e-05*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-1.1870941558441559e-04*w19*Jinv12+6.7573051948051951e-04*Jinv22*w5-3.6525974025974026e-05*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+1.0227272727272728e-03*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-1.1870941558441559e-04*Jinv02*w18+1.0227272727272728e-03*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8+1.1668019480519480e-04*w57*Jinv00+1.1668019480519480e-04*Jinv20*w59-2.9220779220779221e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00-2.9220779220779221e-04*w35*Jinv20+2.5568181818181820e-04*Jinv21*w23-1.1870941558441559e-04*Jinv01*w54-1.1870941558441559e-04*w55*Jinv11-3.6525974025974026e-05*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17+6.7573051948051951e-04*w30*Jinv00+3.2873376623376621e-04*w40*Jinv10-1.1870941558441559e-04*w20*Jinv20-8.3198051948051951e-05*Jinv22*w29+1.0227272727272728e-03*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11+2.7394480519480521e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+6.7573051948051951e-04*w5*Jinv20+6.7573051948051951e-04*Jinv01*w30+2.7394480519480521e-05*Jinv02*w51-1.1870941558441559e-04*Jinv22*w56-8.3198051948051951e-05*Jinv12*w28-3.6525974025974026e-05*w25*Jinv11-1.1870941558441559e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-3.6525974025974026e-05*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50-8.3198051948051951e-05*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[23][3]=(0.0)*D;
  A[23][4]=(0.0)*D;
  A[23][5]=(-1.3971185064935066e-03*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-4.6570616883116886e-04*Jinv22*w53+9.5880681818181818e-04*Jinv20*w8+3.1960227272727273e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-1.3697240259740260e-03*w32*Jinv20+2.0271915584415584e-03*Jinv01*w3-3.2873376623376621e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38+2.7394480519480518e-04*Jinv00*w54+3.1960227272727273e-04*Jinv21*w59-3.8352272727272725e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+3.2873376623376621e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+2.7394480519480520e-03*w16*Jinv11+2.7394480519480520e-03*Jinv21*w17-1.9176136363636362e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55-1.0957792207792208e-04*w58*Jinv10+2.7394480519480520e-03*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24+2.5202922077922079e-03*Jinv22*w35+3.1960227272727273e-04*w58*Jinv11+3.1960227272727273e-04*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-6.6659902597402602e-04*w1*Jinv10+6.0267857142857139e-04*w16*Jinv10-1.3697240259740259e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+2.7394480519480520e-03*w15*Jinv01+4.8396915584415583e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+2.7394480519480520e-03*Jinv22*w17+1.2053571428571428e-03*Jinv00*w33-1.4793019480519481e-03*w22*Jinv12-1.3697240259740260e-03*w31*Jinv10+2.0271915584415584e-03*Jinv02*w3-3.2873376623376621e-04*w21*Jinv00+1.0135957792207792e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+2.7394480519480521e-05*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38-1.9176136363636362e-04*Jinv02*w45-3.8352272727272725e-04*Jinv01*w24+3.1960227272727273e-04*Jinv22*w59+2.5202922077922079e-03*w34*Jinv11-1.3697240259740259e-04*w19*Jinv11-4.6570616883116886e-04*Jinv21*w53-1.9176136363636362e-04*w51*Jinv00+2.5202922077922079e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+2.0271915584415584e-03*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10+1.2053571428571428e-03*w34*Jinv10+1.0135957792207792e-03*w32*Jinv21-1.3697240259740259e-04*Jinv21*w20+9.5880681818181818e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12+2.7394480519480518e-04*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-4.6570616883116886e-04*w52*Jinv12+2.5202922077922079e-03*Jinv21*w35+1.0135957792207792e-03*w31*Jinv11-3.8352272727272725e-04*Jinv22*w26-1.9176136363636362e-04*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14+2.0271915584415584e-03*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54+2.0271915584415584e-03*w4*Jinv12-1.4793019480519481e-03*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.8396915584415583e-04*Jinv21*w29+2.5202922077922079e-03*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12-1.3697240259740259e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01+9.5880681818181818e-04*w7*Jinv10-1.9176136363636362e-04*w47*Jinv21+2.7394480519480521e-05*Jinv20*w38-1.4793019480519481e-03*Jinv22*w23-4.6570616883116886e-04*w52*Jinv11+1.0135957792207792e-03*w31*Jinv12-1.9176136363636362e-04*w46*Jinv12-1.9176136363636362e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-3.2873376623376621e-04*w22*Jinv10-8.4922889610389610e-04*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01-4.6570616883116886e-04*Jinv01*w51+1.0135957792207792e-03*Jinv02*w30-8.4922889610389610e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-1.9176136363636362e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.7394480519480521e-05*w37*Jinv10-2.4655032467532466e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+4.8396915584415583e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-1.3697240259740259e-04*w19*Jinv12+2.0271915584415584e-03*Jinv22*w5-1.9176136363636362e-04*w47*Jinv22-8.4922889610389610e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+3.1960227272727273e-04*w58*Jinv12-1.3697240259740259e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+2.5202922077922079e-03*Jinv02*w33-6.6659902597402602e-04*w0*Jinv00+1.2053571428571428e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-6.6659902597402602e-04*Jinv20*w2+6.0267857142857139e-04*w15*Jinv00-8.4922889610389610e-04*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+6.0267857142857139e-04*Jinv20*w17-1.3697240259740260e-03*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+4.8396915584415583e-04*Jinv22*w29+1.2327516233766234e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+1.0135957792207792e-03*Jinv01*w30-4.6570616883116886e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+4.8396915584415583e-04*Jinv12*w28-3.8352272727272725e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+2.7394480519480520e-03*w16*Jinv12-8.4922889610389610e-04*Jinv21*w50+4.8396915584415583e-04*Jinv01*w27-8.4922889610389610e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[23][6]=(0.0)*D;
  A[23][7]=(0.0)*D;
  A[23][8]=(9.5880681818181818e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+9.5880681818181818e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00+1.1231737012987013e-03*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-1.4245129870129869e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+2.7394480519480518e-04*Jinv00*w54+1.0957792207792208e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2-3.2873376623376621e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+7.6704545454545450e-04*w16*Jinv11+7.6704545454545450e-04*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-1.0957792207792208e-04*w58*Jinv10+7.6704545454545450e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+1.0957792207792208e-04*w58*Jinv11+1.0957792207792208e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+3.5612824675324673e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+7.6704545454545450e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10+7.6704545454545450e-04*Jinv22*w17-1.2601461038961039e-03*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12+1.1231737012987013e-03*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3-1.4245129870129869e-03*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+3.5612824675324673e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-2.7394480519480518e-04*Jinv21*w53+1.9176136363636362e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13-1.2601461038961039e-03*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+2.7394480519480518e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+9.5880681818181818e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+7.1225649350649346e-04*Jinv22*w23-2.7394480519480518e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+1.9176136363636362e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8-1.4245129870129869e-03*w22*Jinv10-8.2183441558441553e-05*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01-2.7394480519480518e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+1.9176136363636362e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-2.7394480519480518e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-1.0957792207792208e-04*w57*Jinv00-1.0957792207792208e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+3.5612824675324673e-04*w0*Jinv00-1.2601461038961039e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10+3.5612824675324673e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00-8.2183441558441553e-05*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-1.3697240259740260e-03*Jinv20*w17+1.1231737012987013e-03*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+2.7394480519480518e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00-3.5612824675324673e-04*w47*Jinv20+7.6704545454545450e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[23][9]=(0.0)*D;
  A[23][10]=(0.0)*D;
  A[23][11]=(-5.8441558441558442e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-5.8441558441558442e-04*w13*Jinv10+1.1870941558441559e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20+1.4975649350649350e-03*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+2.1915584415584417e-04*w43*Jinv10+1.0247564935064935e-04*w58*Jinv10-2.1915584415584417e-04*w48*Jinv00-1.4204545454545455e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+7.4878246753246752e-04*w45*Jinv00+6.4935064935064935e-05*Jinv00*w27-2.1915584415584417e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.4878246753246752e-04*w46*Jinv10+3.4699675324675324e-04*Jinv00*w33-4.2918019480519480e-04*w31*Jinv10+1.4975649350649350e-03*w21*Jinv00-2.6481331168831169e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.7394480519480521e-05*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-2.6481331168831169e-04*w19*Jinv10+3.4699675324675324e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56+3.6525974025974026e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-5.8441558441558442e-04*Jinv20*w14-2.1915584415584417e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-1.0957792207792208e-04*w7*Jinv10-5.4788961038961042e-05*Jinv20*w38-2.7394480519480521e-05*Jinv20*w53+1.4975649350649350e-03*w22*Jinv10-2.7394480519480521e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10+3.6525974025974026e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+2.1915584415584417e-04*Jinv20*w44+1.0247564935064935e-04*w57*Jinv00+1.0247564935064935e-04*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+3.4699675324675324e-04*w35*Jinv20+3.6525974025974026e-05*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+4.3831168831168834e-04*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-2.6481331168831169e-04*w20*Jinv20+1.1870941558441559e-04*w5*Jinv20-4.2918019480519480e-04*w55*Jinv10+2.1915584415584417e-04*w42*Jinv00+7.4878246753246752e-04*w47*Jinv20)*D;
  A[23][12]=(0.0)*D;
  A[23][13]=(0.0)*D;
  A[23][14]=(1.3149350649350649e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+1.5523538961038962e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.2327516233766234e-03*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3-3.7804383116883116e-03*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-2.7394480519480518e-04*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12+1.4610389610389610e-04*w0*Jinv02-4.0178571428571428e-04*Jinv21*w2+8.2183441558441553e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21-1.4793019480519481e-03*w16*Jinv11-1.4793019480519481e-03*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45-8.2183441558441553e-05*Jinv12*w55+1.5523538961038962e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.5523538961038962e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00+2.4655032467532466e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-2.7394480519480518e-04*w58*Jinv11-2.7394480519480518e-04*w57*Jinv01+8.2183441558441553e-05*Jinv21*w26-4.0178571428571428e-04*w0*Jinv01+1.4610389610389610e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+2.4655032467532466e-04*w45*Jinv00-1.4793019480519481e-03*Jinv21*w41+8.2183441558441553e-04*w42*Jinv02+1.0957792207792207e-03*Jinv00*w27+1.5523538961038962e-04*w11*Jinv22-4.0178571428571428e-04*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11-1.4793019480519481e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-3.7804383116883116e-03*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3-3.7804383116883116e-03*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22-1.4793019480519481e-03*Jinv01*w39+1.4610389610389610e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38+2.4655032467532466e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+1.5523538961038962e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+8.2183441558441553e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.1505681818181819e-03*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20+1.5523538961038962e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21-8.2183441558441553e-05*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-3.7804383116883116e-03*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+4.3831168831168834e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.5523538961038962e-04*Jinv02*w9+9.5880681818181818e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-3.7804383116883116e-03*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12+2.4655032467532466e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-3.7804383116883116e-03*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-2.6298701298701297e-03*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-2.6298701298701297e-03*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10+9.5880681818181818e-04*w7*Jinv11+1.5523538961038962e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10+4.3831168831168834e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00+1.0957792207792207e-03*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12+1.2327516233766234e-03*Jinv22*w5+2.4655032467532466e-04*w47*Jinv22+9.5880681818181818e-04*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.5523538961038962e-04*w10*Jinv10+8.2183441558441553e-04*Jinv20*w44+1.5523538961038962e-04*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+9.5880681818181818e-04*Jinv21*w8+1.5523538961038962e-04*w57*Jinv00+1.5523538961038962e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33+1.4610389610389610e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20-2.6298701298701297e-03*Jinv21*w23+4.3831168831168834e-04*Jinv01*w54+4.3831168831168834e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10+1.4610389610389610e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20+1.0957792207792207e-03*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14-1.4793019480519481e-03*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+8.2183441558441553e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-8.2183441558441553e-05*w55*Jinv10+8.2183441558441553e-04*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+9.5880681818181818e-04*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27+9.5880681818181818e-04*w48*Jinv01+1.4610389610389610e-04*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[23][15]=(0.0)*D;
  A[23][16]=(0.0)*D;
  A[23][17]=(-2.7942370129870131e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+6.0267857142857139e-04*Jinv20*w8-2.7394480519480518e-04*w57*Jinv02-2.7942370129870131e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-1.7532467532467534e-03*w32*Jinv20-1.8628246753246754e-03*Jinv01*w3+8.5470779220779228e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38-1.3149350649350649e-03*Jinv00*w54-6.9399350649350649e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-2.3011363636363637e-03*w42*Jinv01-5.6615259740259744e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17-1.0409902597402598e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55+2.7394480519480518e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-6.9399350649350649e-04*w58*Jinv11-6.9399350649350649e-04*w57*Jinv01-2.0819805194805196e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+1.6436688311688311e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-2.9586038961038961e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1-5.9172077922077923e-03*w15*Jinv01-4.3831168831168834e-04*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12-1.7532467532467534e-03*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3+8.5470779220779228e-03*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-2.9586038961038961e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-4.9310064935064932e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00+3.2873376623376621e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-2.0819805194805196e-03*Jinv01*w24-2.7394480519480518e-04*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10-1.8628246753246754e-03*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13-4.9310064935064932e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-9.3141233766233771e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+6.0267857142857139e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12-1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41+5.4788961038961042e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35-9.3141233766233771e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-1.0409902597402598e-03*w46*Jinv11-2.7942370129870131e-03*Jinv20*w14-1.8628246753246754e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.3831168831168834e-04*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.2873376623376621e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+6.0267857142857139e-04*w6*Jinv01+6.0267857142857139e-04*w7*Jinv10-1.0409902597402598e-03*w47*Jinv21+3.2873376623376621e-04*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+7.6704545454545450e-04*Jinv22*w8+8.5470779220779228e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50-9.8620129870129864e-04*w22*Jinv11+7.6704545454545450e-04*Jinv12*w7-9.8620129870129864e-04*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+3.2873376623376621e-04*w36*Jinv01+6.0267857142857139e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10+1.2601461038961039e-03*Jinv21*w56+5.4788961038961036e-04*Jinv02*w27+7.6704545454545450e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+1.2053571428571428e-03*w49*Jinv11+5.4788961038961042e-05*w10*Jinv10-2.7394480519480518e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+6.0267857142857139e-04*Jinv21*w8+2.7394480519480518e-04*w57*Jinv00+2.7394480519480518e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+1.6436688311688311e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20-9.8620129870129864e-04*Jinv21*w23+1.2601461038961039e-03*Jinv01*w54+1.2601461038961039e-03*w55*Jinv11+1.6436688311688311e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-5.6615259740259744e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.7532467532467534e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-4.9310064935064932e-04*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14-2.9586038961038961e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20-9.3141233766233771e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-2.0819805194805196e-03*w25*Jinv11-1.3149350649350649e-03*w55*Jinv10+2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+1.2053571428571428e-03*Jinv21*w50-4.3831168831168834e-04*Jinv01*w27+1.2053571428571428e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-5.6615259740259744e-04*w10*Jinv11)*D;
  A[23][18]=(0.0)*D;
  A[23][19]=(0.0)*D;
  A[23][20]=(1.4793019480519481e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8+1.4793019480519481e-03*w13*Jinv10-3.8352272727272725e-04*Jinv21*w44+1.3697240259740259e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.2049512987012989e-04*Jinv01*w3-4.7666396103896100e-03*Jinv20*w23+1.3971185064935066e-03*Jinv00*w54-1.0957792207792208e-04*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-8.2183441558441553e-04*w43*Jinv10-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-4.2918019480519480e-04*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00-1.0957792207792208e-04*w58*Jinv11-1.0957792207792208e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-1.6436688311688311e-03*w16*Jinv10-3.8352272727272725e-04*w43*Jinv11-2.3833198051948050e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-4.7666396103896100e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39+5.7528409090909093e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38-1.6436688311688311e-04*Jinv01*w24-5.4788961038961042e-05*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+2.7394480519480518e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00+1.3697240259740259e-04*w4*Jinv10+5.2049512987012989e-04*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.7528409090909093e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*Jinv20*w56-2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-5.4788961038961042e-05*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11+1.4793019480519481e-03*Jinv20*w14+5.2049512987012989e-04*w5*Jinv21+5.4788961038961036e-04*w49*Jinv10+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.6436688311688311e-04*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+2.7394480519480518e-04*w52*Jinv11+2.4655032467532466e-04*Jinv20*w53-4.7666396103896100e-03*w22*Jinv10-7.1225649350649346e-04*w22*Jinv11-7.1225649350649346e-04*w21*Jinv01+2.7394480519480518e-04*Jinv01*w51-3.5612824675324673e-04*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11-2.1002435064935065e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+8.2183441558441553e-05*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-2.1002435064935065e-04*w10*Jinv10-8.2183441558441553e-04*Jinv20*w44+5.2049512987012989e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-4.2918019480519480e-04*w57*Jinv00-4.2918019480519480e-04*Jinv20*w59-3.1047077922077924e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20-7.1225649350649346e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-1.6436688311688311e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-1.6436688311688311e-03*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+5.7528409090909093e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+1.3697240259740259e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*w25*Jinv11+1.3971185064935066e-03*w55*Jinv10-8.2183441558441553e-04*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+8.2183441558441553e-05*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27+8.2183441558441553e-05*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[23][21]=(0.0)*D;
  A[23][22]=(0.0)*D;
  A[23][23]=(2.6298701298701297e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-4.2735389610389614e-03*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+2.6298701298701297e-03*w13*Jinv10-4.2735389610389614e-03*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20+7.3965097402597403e-04*Jinv01*w3-7.3965097402597406e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54-3.6982548701298703e-03*w25*Jinv12-7.8530844155844158e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45+2.3833198051948050e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00-3.6982548701298703e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-1.2327516233766234e-03*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-7.8530844155844158e-04*w1*Jinv10-4.2735389610389614e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-3.6982548701298703e-03*w45*Jinv00+4.9310064935064932e-04*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01-1.1870941558441558e-03*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-3.6982548701298703e-03*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-7.3965097402597406e-03*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3-7.3965097402597406e-03*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-7.8530844155844158e-04*Jinv22*w2+2.3833198051948050e-03*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.6982548701298703e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+3.5612824675324673e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+7.3965097402597403e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+2.6298701298701297e-03*Jinv12*w13+2.3833198051948050e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+7.3965097402597403e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-4.2735389610389614e-03*Jinv22*w41-3.6982548701298703e-03*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+7.3965097402597403e-04*w31*Jinv11-3.6982548701298703e-03*Jinv22*w26-1.2327516233766234e-03*w46*Jinv11+2.6298701298701297e-03*Jinv20*w14+7.3965097402597403e-04*w5*Jinv21+2.3833198051948050e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-7.3965097402597406e-03*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-1.1870941558441558e-03*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+2.6298701298701297e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38-7.3965097402597406e-03*Jinv22*w23+1.6436688311688311e-04*w31*Jinv12-3.6982548701298703e-03*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8-7.3965097402597406e-03*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*Jinv12*w7+1.6436688311688311e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+3.5612824675324673e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-3.6982548701298703e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-3.6982548701298703e-03*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.3149350649350649e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.8530844155844158e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-3.6982548701298703e-03*w25*Jinv10-7.8530844155844158e-04*Jinv20*w2-4.2735389610389614e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-4.2735389610389614e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+2.3833198051948050e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.6298701298701297e-03*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+7.3965097402597403e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+2.3833198051948050e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-3.6982548701298703e-03*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50-1.1870941558441558e-03*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-7.8530844155844158e-04*Jinv12*w1)*D;
  A[23][24]=(0.0)*D;
  A[23][25]=(0.0)*D;
  A[23][26]=(-2.6298701298701297e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-7.1225649350649346e-04*Jinv20*w8-2.6298701298701297e-03*w13*Jinv10+1.5888798701298701e-03*Jinv21*w44-1.6436688311688311e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+7.3965097402597406e-03*Jinv20*w23-2.3833198051948050e-03*Jinv00*w54+3.7439123376623376e-04*Jinv21*w59+3.9265422077922079e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+1.5888798701298701e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+3.1777597402597403e-03*w16*Jinv11+3.1777597402597403e-03*Jinv21*w17+1.2327516233766234e-03*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+3.7439123376623376e-04*w58*Jinv11+3.7439123376623376e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.9265422077922079e-04*w0*Jinv01+7.8530844155844158e-04*w1*Jinv10+4.2735389610389614e-03*w16*Jinv10+1.5888798701298701e-03*w43*Jinv11+3.6982548701298703e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+3.9265422077922079e-04*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11+3.1777597402597403e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+3.6982548701298703e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-1.6436688311688311e-04*w31*Jinv10+7.3965097402597406e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.3833198051948050e-03*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+2.4655032467532467e-03*Jinv01*w24+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00-1.6436688311688311e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11+4.2735389610389614e-03*w41*Jinv20-2.3833198051948050e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-7.1225649350649346e-04*w6*Jinv00-2.3833198051948050e-03*Jinv20*w56+3.6982548701298703e-03*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+1.0957792207792207e-03*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+1.2327516233766234e-03*w46*Jinv11-2.6298701298701297e-03*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33+4.2735389610389614e-03*w39*Jinv00-1.2327516233766234e-03*Jinv01*w18-8.2183441558441553e-05*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-7.1225649350649346e-04*w7*Jinv10+1.2327516233766234e-03*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38-3.5612824675324673e-04*w52*Jinv11-3.5612824675324673e-04*Jinv20*w53+7.3965097402597406e-03*w22*Jinv10+2.4655032467532467e-03*w22*Jinv11+2.4655032467532467e-03*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-8.2183441558441553e-05*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+7.4878246753246752e-04*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10-1.0135957792207792e-03*Jinv21*w56+3.6982548701298703e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+7.4878246753246752e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44-1.2327516233766234e-03*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.8530844155844158e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+2.4655032467532467e-03*Jinv21*w23-1.0135957792207792e-03*Jinv01*w54-1.0135957792207792e-03*w55*Jinv11+3.6982548701298703e-03*w25*Jinv10+7.8530844155844158e-04*Jinv20*w2+4.2735389610389614e-03*w15*Jinv00+3.9265422077922079e-04*Jinv01*w9+4.2735389610389614e-03*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-2.3833198051948050e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-1.6436688311688311e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30+2.4655032467532467e-03*w25*Jinv11-2.3833198051948050e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+3.6982548701298703e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+3.9265422077922079e-04*w10*Jinv11)*D;
  A[23][27]=(0.0)*D;
  A[23][28]=(0.0)*D;
  A[23][29]=(3.2873376623376621e-04*Jinv21*w44-2.6481331168831169e-04*Jinv01*w3+1.5726461038961039e-04*Jinv21*w59+2.0495129870129870e-04*Jinv21*w2+3.2873376623376621e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.1870941558441558e-03*Jinv01*w45+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+1.1870941558441558e-03*Jinv21*w26+2.0495129870129870e-04*w0*Jinv01+3.2873376623376621e-04*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+2.0495129870129870e-04*Jinv11*w1-1.0227272727272728e-03*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+6.5746753246753243e-04*Jinv01*w39-5.4788961038961042e-05*Jinv21*w38+1.1870941558441558e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-6.7573051948051951e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-2.6481331168831169e-04*w4*Jinv11-2.6481331168831169e-04*w32*Jinv21-6.7573051948051951e-04*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-2.6481331168831169e-04*w31*Jinv11+1.1870941558441558e-03*w46*Jinv11-2.6481331168831169e-04*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-6.7573051948051951e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-1.0957792207792208e-04*w6*Jinv01+1.1870941558441558e-03*w47*Jinv21-5.4788961038961042e-05*w52*Jinv11+2.3741883116883116e-03*w22*Jinv11+2.3741883116883116e-03*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-5.4788961038961042e-05*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-6.7573051948051951e-04*Jinv21*w56-1.0957792207792208e-04*w49*Jinv11-1.0227272727272728e-03*Jinv21*w14-1.0227272727272728e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+2.3741883116883116e-03*Jinv21*w23-6.7573051948051951e-04*Jinv01*w54-6.7573051948051951e-04*w55*Jinv11+1.5726461038961039e-04*Jinv01*w9+6.5746753246753243e-04*w40*Jinv11-2.6481331168831169e-04*Jinv01*w30+1.1870941558441558e-03*w25*Jinv11-1.0957792207792208e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+1.5726461038961039e-04*w10*Jinv11)*D;
  A[23][30]=(0.0)*D;
  A[23][31]=(0.0)*D;
  A[23][32]=(1.2327516233766234e-03*Jinv00*w12+4.8396915584415583e-04*w28*Jinv10+6.0267857142857139e-04*w40*Jinv12+2.7394480519480521e-05*Jinv22*w53-8.4922889610389610e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10+6.0267857142857139e-04*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44+1.0135957792207792e-03*w3*Jinv00+2.0271915584415584e-03*w32*Jinv20+1.0135957792207792e-03*Jinv01*w3-1.4793019480519481e-03*Jinv20*w23-1.9176136363636362e-04*Jinv22*w38-1.3697240259740259e-04*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12-6.6659902597402602e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+3.1960227272727273e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-3.8352272727272725e-04*Jinv01*w45+3.8352272727272725e-04*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-4.3831168831168834e-04*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+1.2053571428571428e-03*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01-1.9176136363636362e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10+2.7394480519480518e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11-3.8352272727272725e-04*w45*Jinv00+2.7394480519480520e-03*Jinv21*w41+3.2873376623376621e-04*w42*Jinv02+4.8396915584415583e-04*Jinv00*w27-1.0957792207792208e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+4.8396915584415583e-04*w28*Jinv11-4.3831168831168834e-04*Jinv20*w50+4.8396915584415583e-04*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17+2.5202922077922079e-03*Jinv00*w33-3.2873376623376621e-04*w22*Jinv12+2.0271915584415584e-03*w31*Jinv10-1.3697240259740260e-03*Jinv02*w3-1.4793019480519481e-03*w21*Jinv00-3.5612824675324673e-04*w32*Jinv22+2.7394480519480520e-03*Jinv01*w39-6.6659902597402602e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-4.6570616883116886e-04*w36*Jinv00-4.6570616883116886e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-1.9176136363636362e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+2.5202922077922079e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-2.4655032467532466e-04*w51*Jinv00+1.2053571428571428e-03*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44+1.0135957792207792e-03*w4*Jinv10+1.0135957792207792e-03*w4*Jinv11+2.7394480519480520e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+2.5202922077922079e-03*w34*Jinv10+2.0271915584415584e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-8.4922889610389610e-04*w6*Jinv00-1.9176136363636362e-04*w37*Jinv12-1.3697240259740259e-04*Jinv20*w56+6.0267857142857139e-04*Jinv22*w41-1.9176136363636362e-04*w26*Jinv20+3.1960227272727273e-04*w11*Jinv20+2.7394480519480521e-05*w52*Jinv12+2.5202922077922079e-03*Jinv21*w35+2.0271915584415584e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26-3.8352272727272725e-04*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14+1.0135957792207792e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54-1.3697240259740260e-03*w4*Jinv12-3.2873376623376621e-04*w21*Jinv02-4.3831168831168834e-04*w49*Jinv10+4.8396915584415583e-04*Jinv21*w29+2.5202922077922079e-03*Jinv01*w33+2.7394480519480520e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-4.6570616883116886e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9-8.4922889610389610e-04*w6*Jinv01-8.4922889610389610e-04*w7*Jinv10-3.8352272727272725e-04*w47*Jinv21-4.6570616883116886e-04*Jinv20*w38-3.2873376623376621e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-1.4793019480519481e-03*w22*Jinv10+9.5880681818181818e-04*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30+9.5880681818181818e-04*Jinv12*w49-4.6570616883116886e-04*w36*Jinv01-2.4655032467532466e-04*w52*Jinv10-8.4922889610389610e-04*w7*Jinv11+3.1960227272727273e-04*w9*Jinv00-4.6570616883116886e-04*w37*Jinv10-1.3697240259740259e-04*Jinv21*w56-1.9176136363636362e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12-1.3697240259740260e-03*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-4.3831168831168834e-04*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+3.1960227272727273e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-8.4922889610389610e-04*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+1.2053571428571428e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+2.5202922077922079e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23-1.3697240259740259e-04*Jinv01*w54-1.3697240259740259e-04*w55*Jinv11-1.9176136363636362e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00+9.5880681818181818e-04*w48*Jinv02+3.1960227272727273e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+2.0271915584415584e-03*w30*Jinv00+2.7394480519480520e-03*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14+2.7394480519480520e-03*w40*Jinv11-1.9176136363636362e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.0135957792207792e-03*w5*Jinv20+2.0271915584415584e-03*Jinv01*w30+2.7394480519480521e-05*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.9176136363636362e-04*w25*Jinv11-1.3697240259740259e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-3.8352272727272725e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-4.3831168831168834e-04*Jinv21*w50+4.8396915584415583e-04*Jinv01*w27-4.3831168831168834e-04*w48*Jinv01-6.6659902597402602e-04*Jinv12*w1+3.1960227272727273e-04*w10*Jinv11)*D;
  A[23][33]=(0.0)*D;
  A[23][34]=(0.0)*D;
  A[23][35]=(-1.7532467532467534e-03*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+8.2183441558441553e-04*Jinv22*w53+1.2053571428571428e-03*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02-1.7532467532467534e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44-1.2601461038961039e-03*w3*Jinv00-2.5750811688311690e-03*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+2.1915584415584414e-03*Jinv20*w23-3.8352272727272725e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.3149350649350649e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-1.0957792207792207e-03*Jinv01*w45+3.2873376623376621e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-2.6298701298701297e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+1.0957792207792207e-03*Jinv02*w24-1.6436688311688311e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01-1.0957792207792207e-03*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10-3.8352272727272725e-04*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-4.9310064935064932e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-2.6298701298701297e-03*Jinv22*w17-1.6436688311688311e-03*Jinv00*w33+2.1915584415584414e-03*w22*Jinv12-2.5750811688311690e-03*w31*Jinv10-2.5750811688311690e-03*Jinv02*w3+2.1915584415584414e-03*w21*Jinv00-1.2601461038961039e-03*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+3.2873376623376621e-04*Jinv00*w18+8.2183441558441553e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-1.6436688311688311e-03*Jinv12*w34-1.2601461038961039e-03*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20-1.7532467532467534e-03*Jinv12*w13+3.2873376623376621e-04*w19*Jinv10-1.6436688311688311e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+1.2053571428571428e-03*w6*Jinv00-3.8352272727272725e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41-3.8352272727272725e-04*w11*Jinv20+8.2183441558441553e-04*w52*Jinv12+3.2873376623376621e-03*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26-1.0957792207792207e-03*w46*Jinv11-1.7532467532467534e-03*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21+3.2873376623376621e-04*Jinv02*w54-2.5750811688311690e-03*w4*Jinv12+2.1915584415584414e-03*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.7532467532467534e-03*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+1.2053571428571428e-03*w7*Jinv10-1.0957792207792207e-03*w47*Jinv21+8.2183441558441553e-04*Jinv20*w38+2.1915584415584414e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.2601461038961039e-03*w31*Jinv12-1.0957792207792208e-04*Jinv22*w8+2.1915584415584414e-03*w22*Jinv10+1.2053571428571428e-03*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-1.0957792207792207e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.2601461038961039e-03*Jinv02*w30+1.2053571428571428e-03*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+8.2183441558441553e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-4.9310064935064932e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12-2.5750811688311690e-03*Jinv22*w5+5.4788961038961042e-05*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10-3.8352272727272725e-04*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-1.6436688311688311e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-1.6436688311688311e-03*w35*Jinv20-1.0957792207792207e-03*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+2.5568181818181820e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+1.2053571428571428e-03*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-2.5750811688311690e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29-1.7532467532467534e-03*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-1.2601461038961039e-03*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+8.2183441558441553e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28-1.0957792207792207e-03*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10+1.0957792207792207e-03*w47*Jinv20-2.6298701298701297e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[23][36]=(0.0)*D;
  A[23][37]=(0.0)*D;
  A[23][38]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-1.0957792207792208e-04*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+2.4655032467532466e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.2049512987012989e-04*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-7.1225649350649346e-04*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55-1.3697240259740259e-04*w58*Jinv10-7.6704545454545450e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-7.1225649350649346e-04*w45*Jinv00-3.8352272727272725e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+5.4788961038961036e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-7.6704545454545450e-04*Jinv22*w17-8.7662337662337668e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10+5.2049512987012989e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv02*w45-1.0957792207792208e-04*Jinv22*w59+2.4655032467532466e-04*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+2.4655032467532466e-04*w4*Jinv10-1.0957792207792208e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10-8.7662337662337668e-04*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.2049512987012989e-04*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.9176136363636362e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20+2.7394480519480518e-04*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10-1.0957792207792208e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-3.5612824675324673e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49+2.4655032467532466e-04*w52*Jinv10+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+1.9176136363636362e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+6.3920454545454550e-05*w10*Jinv10-7.1225649350649346e-04*Jinv20*w44-1.0957792207792208e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-8.7662337662337668e-04*w35*Jinv20+1.9176136363636362e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-3.5612824675324673e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+2.4655032467532466e-04*w5*Jinv20+2.7394480519480518e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28+5.2049512987012989e-04*w55*Jinv10-7.1225649350649346e-04*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20-7.6704545454545450e-04*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[23][39]=(0.0)*D;
  A[23][40]=(0.0)*D;
  A[23][41]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-9.3141233766233771e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+2.1367694805194807e-03*Jinv00*w54-5.6615259740259744e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+1.6436688311688311e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-6.9399350649350649e-04*w11*Jinv21-2.9586038961038961e-03*w16*Jinv11-2.9586038961038961e-03*Jinv21*w17-2.0819805194805196e-03*Jinv01*w45-4.9310064935064932e-04*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-5.6615259740259744e-04*w58*Jinv11-5.6615259740259744e-04*w57*Jinv01-1.0409902597402598e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10-1.3149350649350649e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41+5.4788961038961036e-04*Jinv00*w27+2.7394480519480518e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1-2.9586038961038961e-03*w15*Jinv01-4.3831168831168834e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+8.5470779220779228e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10-1.7532467532467534e-03*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39+1.6436688311688311e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38-1.0409902597402598e-03*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.2601461038961039e-03*w19*Jinv11+3.2873376623376621e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.8352272727272725e-04*w4*Jinv10-9.3141233766233771e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-2.7942370129870131e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-1.8628246753246754e-03*w32*Jinv21+1.2601461038961039e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20+3.2873376623376621e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35-1.8628246753246754e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26-2.0819805194805196e-03*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-9.3141233766233771e-04*w5*Jinv21-4.9310064935064932e-04*Jinv02*w54-1.7532467532467534e-03*w4*Jinv12+8.5470779220779228e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10-4.3831168831168834e-04*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-2.7942370129870131e-03*Jinv02*w12+1.2601461038961039e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9+1.2053571428571428e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-2.0819805194805196e-03*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38+8.5470779220779228e-03*Jinv22*w23+3.2873376623376621e-04*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12+3.8352272727272725e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+6.0267857142857139e-04*Jinv22*w50-9.8620129870129864e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-9.8620129870129864e-04*w21*Jinv01+3.2873376623376621e-04*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+6.0267857142857139e-04*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+1.2053571428571428e-03*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-1.7532467532467534e-03*Jinv22*w5+6.0267857142857139e-04*w49*Jinv11-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18+1.2053571428571428e-03*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-9.8620129870129864e-04*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+6.0267857142857139e-04*w48*Jinv02-6.9399350649350649e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-2.7942370129870131e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+3.8352272727272725e-04*w5*Jinv20-1.8628246753246754e-03*Jinv01*w30+3.2873376623376621e-04*Jinv02*w51-4.9310064935064932e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.0409902597402598e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12+6.0267857142857139e-04*Jinv21*w50-4.3831168831168834e-04*Jinv01*w27+6.0267857142857139e-04*w48*Jinv01+1.6436688311688311e-04*Jinv12*w1-6.9399350649350649e-04*w10*Jinv11)*D;
  A[23][42]=(0.0)*D;
  A[23][43]=(0.0)*D;
  A[23][44]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-2.1367694805194807e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02-3.6525974025974026e-05*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+1.0957792207792207e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+1.0957792207792207e-03*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22-3.6525974025974026e-05*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+4.2735389610389614e-03*w22*Jinv12-7.6704545454545450e-04*w31*Jinv10-7.6704545454545450e-04*Jinv02*w3+4.2735389610389614e-03*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-1.3149350649350649e-03*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+1.0957792207792207e-03*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59-3.2873376623376621e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-2.1367694805194807e-03*Jinv12*w13-1.3149350649350649e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+2.1367694805194807e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20-5.4788961038961042e-05*w52*Jinv12-3.2873376623376621e-03*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+1.0957792207792207e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-3.2873376623376621e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-2.1367694805194807e-03*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.0957792207792207e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38+4.2735389610389614e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53-7.6704545454545450e-04*Jinv22*w8+4.2735389610389614e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-7.6704545454545450e-04*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10-5.4788961038961042e-05*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-7.6704545454545450e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-5.4788961038961042e-05*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.0957792207792207e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-5.4788961038961042e-05*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[23][45]=(0.0)*D;
  A[23][46]=(0.0)*D;
  A[23][47]=(4.2735389610389614e-03*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+7.4878246753246752e-04*w57*Jinv02+4.2735389610389614e-03*Jinv02*w39+1.5888798701298701e-03*Jinv21*w44-8.2183441558441553e-05*Jinv01*w3-3.5612824675324673e-04*Jinv22*w38+3.9265422077922079e-04*Jinv21*w59+3.6982548701298703e-03*w25*Jinv12+7.8530844155844158e-04*w0*Jinv02+3.9265422077922079e-04*Jinv21*w2+1.5888798701298701e-03*w42*Jinv01+3.7439123376623376e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45-2.3833198051948050e-03*Jinv12*w55+4.2735389610389614e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+3.6982548701298703e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+1.2327516233766234e-03*Jinv21*w26+3.9265422077922079e-04*w0*Jinv01-2.3833198051948050e-03*Jinv22*w20+1.5888798701298701e-03*w43*Jinv11+3.1777597402597403e-03*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+3.9265422077922079e-04*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11+4.2735389610389614e-03*Jinv22*w17+7.3965097402597406e-03*w22*Jinv12-1.6436688311688311e-04*Jinv02*w3-1.6436688311688311e-04*w32*Jinv22+3.1777597402597403e-03*Jinv01*w39+7.8530844155844158e-04*Jinv22*w2-3.5612824675324673e-04*Jinv21*w38+3.6982548701298703e-03*Jinv02*w45+1.2327516233766234e-03*Jinv01*w24+7.4878246753246752e-04*Jinv22*w59+1.0957792207792207e-03*w34*Jinv11-1.0135957792207792e-03*w19*Jinv11-8.2183441558441553e-05*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-8.2183441558441553e-05*w4*Jinv11-2.6298701298701297e-03*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21-1.0135957792207792e-03*Jinv21*w20-3.5612824675324673e-04*w37*Jinv12+4.2735389610389614e-03*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12+1.0957792207792207e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+3.6982548701298703e-03*Jinv22*w26+2.4655032467532467e-03*w46*Jinv11-8.2183441558441553e-05*w5*Jinv21-2.3833198051948050e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+7.3965097402597406e-03*w21*Jinv02-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33-2.6298701298701297e-03*Jinv02*w12-1.0135957792207792e-03*Jinv01*w18-3.5612824675324673e-04*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+2.4655032467532467e-03*w47*Jinv21+7.3965097402597406e-03*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11-1.6436688311688311e-04*w31*Jinv12+3.6982548701298703e-03*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8-7.1225649350649346e-04*Jinv22*w50+2.4655032467532467e-03*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7+2.4655032467532467e-03*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51-1.6436688311688311e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-1.2327516233766234e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+3.6982548701298703e-03*w47*Jinv22-1.6436688311688311e-04*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+7.4878246753246752e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+2.4655032467532467e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11-7.1225649350649346e-04*w48*Jinv02+3.7439123376623376e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.6298701298701297e-03*Jinv22*w14+3.1777597402597403e-03*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-2.3833198051948050e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+1.2327516233766234e-03*w25*Jinv11+4.2735389610389614e-03*w16*Jinv12-1.6436688311688311e-04*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+7.8530844155844158e-04*Jinv12*w1+3.7439123376623376e-04*w10*Jinv11)*D;
  A[23][48]=(0.0)*D;
  A[23][49]=(0.0)*D;
  A[23][50]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-8.2183441558441553e-05*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+1.9176136363636362e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+3.5612824675324673e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24-1.2601461038961039e-03*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+2.7394480519480518e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+7.6704545454545450e-04*Jinv21*w41-3.2873376623376621e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-1.0957792207792208e-04*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33-1.4245129870129869e-03*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10+1.1231737012987013e-03*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-4.3831168831168834e-04*w32*Jinv22+7.6704545454545450e-04*Jinv01*w39+3.5612824675324673e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00-1.2601461038961039e-03*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+9.5880681818181818e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+1.9176136363636362e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+5.4788961038961042e-05*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-5.2049512987012989e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+1.1231737012987013e-03*w4*Jinv12-1.4245129870129869e-03*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+9.5880681818181818e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-2.7394480519480518e-04*Jinv20*w38-1.4245129870129869e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-4.3831168831168834e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-4.3831168831168834e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12+1.1231737012987013e-03*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-1.2601461038961039e-03*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+9.5880681818181818e-04*Jinv22*w14+7.6704545454545450e-04*w40*Jinv11+1.9176136363636362e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30+9.1314935064935065e-06*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+3.5612824675324673e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[23][51]=(0.0)*D;
  A[23][52]=(0.0)*D;
  A[23][53]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+6.3920454545454550e-05*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+2.4655032467532466e-04*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-7.1225649350649346e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-3.8352272727272725e-04*w43*Jinv10-8.2183441558441553e-05*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.0957792207792208e-04*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24-8.7662337662337668e-04*Jinv22*w35-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+5.2049512987012989e-04*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27-1.3697240259740259e-04*w11*Jinv22-7.1225649350649346e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.0957792207792208e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+5.2049512987012989e-04*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+2.4655032467532466e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+1.9176136363636362e-04*Jinv02*w45+6.3920454545454550e-05*Jinv22*w59-3.5612824675324673e-04*w51*Jinv00-8.7662337662337668e-04*Jinv12*w34-7.1225649350649346e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-7.6704545454545450e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+8.2183441558441553e-05*w6*Jinv00+2.4655032467532466e-04*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-3.5612824675324673e-04*w52*Jinv12-7.1225649350649346e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-8.2183441558441553e-05*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-7.1225649350649346e-04*w49*Jinv10-7.6704545454545450e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-1.3697240259740259e-04*Jinv02*w9+8.2183441558441553e-05*w7*Jinv10+2.7394480519480518e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+2.4655032467532466e-04*w31*Jinv12+1.9176136363636362e-04*w46*Jinv12-3.5612824675324673e-04*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+5.4788961038961036e-04*Jinv12*w7+2.4655032467532466e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-3.5612824675324673e-04*w52*Jinv10-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10-3.5612824675324673e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.9176136363636362e-04*w47*Jinv22-1.0957792207792208e-04*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+6.3920454545454550e-05*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-8.7662337662337668e-04*Jinv02*w33-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14+2.4655032467532466e-04*Jinv02*w36-7.1225649350649346e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20-3.5612824675324673e-04*Jinv02*w51-8.2183441558441553e-05*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.0957792207792208e-04*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[23][54]=(0.0)*D;
  A[23][55]=(0.0)*D;
  A[23][56]=(-1.6436688311688311e-03*w40*Jinv12-8.2183441558441553e-05*Jinv22*w53-2.1002435064935065e-04*w57*Jinv02-1.6436688311688311e-03*Jinv02*w39-3.8352272727272725e-04*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+2.4655032467532466e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+5.7528409090909093e-04*Jinv12*w55-2.6298701298701297e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-3.8352272727272725e-04*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-8.2183441558441553e-04*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-1.3697240259740259e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11-2.6298701298701297e-03*Jinv22*w17-4.7666396103896100e-03*w22*Jinv12+5.2049512987012989e-04*Jinv02*w3+1.3697240259740259e-04*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39-3.1047077922077924e-04*Jinv22*w2+2.7394480519480518e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-2.1002435064935065e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-8.2183441558441553e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+1.4793019480519481e-03*Jinv12*w13+5.2049512987012989e-04*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+2.4655032467532466e-04*w37*Jinv12-1.6436688311688311e-03*Jinv22*w41-8.2183441558441553e-05*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21+5.7528409090909093e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12-4.7666396103896100e-03*w21*Jinv02+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+1.4793019480519481e-03*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+8.2183441558441553e-05*w6*Jinv01-1.6436688311688311e-04*w47*Jinv21-4.7666396103896100e-03*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+1.3697240259740259e-04*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+5.4788961038961036e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50-7.1225649350649346e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-7.1225649350649346e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+1.3697240259740259e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+8.2183441558441553e-05*w7*Jinv11-1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-7.1225649350649346e-04*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-2.1002435064935065e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+5.2049512987012989e-04*Jinv01*w12+8.2183441558441553e-05*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33-7.1225649350649346e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.4793019480519481e-03*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+2.4655032467532466e-04*Jinv02*w36-8.2183441558441553e-04*Jinv12*w43+5.2049512987012989e-04*Jinv01*w30-8.2183441558441553e-05*Jinv02*w51+5.7528409090909093e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-2.6298701298701297e-03*w16*Jinv12-7.1225649350649346e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[23][57]=(0.0)*D;
  A[23][58]=(0.0)*D;
  A[23][59]=(4.3831168831168834e-04*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39-2.7394480519480521e-05*Jinv22*w38+7.4878246753246752e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02-2.6481331168831169e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24+3.4699675324675324e-04*Jinv22*w35-4.2918019480519480e-04*Jinv22*w20+2.1915584415584417e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17+1.4975649350649350e-03*w22*Jinv12-4.2918019480519480e-04*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-1.4204545454545455e-05*Jinv22*w2+3.6525974025974026e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+3.4699675324675324e-04*Jinv12*w34+2.1915584415584417e-04*Jinv22*w44-5.8441558441558442e-04*Jinv12*w13-2.7394480519480521e-05*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12+7.4878246753246752e-04*Jinv22*w26-2.6481331168831169e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12+1.4975649350649350e-03*w21*Jinv02-5.8441558441558442e-04*Jinv02*w12+1.0247564935064935e-04*Jinv02*w9+1.4975649350649350e-03*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-2.1915584415584417e-04*Jinv22*w8-1.0957792207792208e-04*Jinv22*w50-2.1915584415584417e-04*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49+6.4935064935064935e-05*Jinv02*w27-2.1915584415584417e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18+3.4699675324675324e-04*Jinv02*w33-1.0957792207792208e-04*w48*Jinv02+6.4935064935064935e-05*Jinv22*w29-5.8441558441558442e-04*Jinv22*w14-2.7394480519480521e-05*Jinv02*w36+2.1915584415584417e-04*Jinv12*w43-5.4788961038961042e-05*Jinv02*w51-2.6481331168831169e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12-1.4204545454545455e-05*Jinv12*w1)*D;
  A[24][0]=(2.6481331168831169e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+2.6481331168831169e-04*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59-1.4975649350649350e-03*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17-7.4878246753246752e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00-1.4975649350649350e-03*Jinv02*w24-3.4699675324675324e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01-1.4975649350649350e-03*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10+5.8441558441558442e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+2.6481331168831169e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-3.4699675324675324e-04*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+5.8441558441558442e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-7.4878246753246752e-04*Jinv02*w45-1.4975649350649350e-03*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.4699675324675324e-04*w34*Jinv11+5.8441558441558442e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+2.1915584415584417e-04*w51*Jinv00-3.4699675324675324e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-2.1915584415584417e-04*w41*Jinv20+2.6481331168831169e-04*Jinv12*w13+5.8441558441558442e-04*w19*Jinv10-3.4699675324675324e-04*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21+5.8441558441558442e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41-1.4975649350649350e-03*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12-3.4699675324675324e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11-1.4975649350649350e-03*Jinv22*w26-7.4878246753246752e-04*w46*Jinv11+2.6481331168831169e-04*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29-3.4699675324675324e-04*Jinv01*w33-2.1915584415584417e-04*w39*Jinv00+2.6481331168831169e-04*Jinv02*w12+5.8441558441558442e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10-7.4878246753246752e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+2.1915584415584417e-04*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+2.7394480519480521e-05*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01+2.1915584415584417e-04*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56-1.4975649350649350e-03*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02+5.8441558441558442e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11+2.6481331168831169e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+5.8441558441558442e-04*Jinv02*w18+2.6481331168831169e-04*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-3.4699675324675324e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-3.4699675324675324e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11-1.4975649350649350e-03*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-2.1915584415584417e-04*w40*Jinv10+5.8441558441558442e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+2.6481331168831169e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-1.4975649350649350e-03*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[24][1]=(0.0)*D;
  A[24][2]=(0.0)*D;
  A[24][3]=(-9.1314935064935065e-06*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8+2.1915584415584417e-04*w57*Jinv02+7.1225649350649346e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23-1.3971185064935066e-03*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-1.9176136363636362e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10-1.4793019480519481e-03*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01-1.8262987012987013e-05*w28*Jinv11-1.9176136363636362e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+1.2601461038961039e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+1.0683847402597404e-03*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-9.5880681818181818e-04*Jinv00*w18-1.1231737012987013e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24+2.1915584415584417e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-1.4793019480519481e-03*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+1.0683847402597404e-03*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-9.5880681818181818e-04*w19*Jinv10+1.2601461038961039e-03*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-1.4793019480519481e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00-1.3971185064935066e-03*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41+1.4245129870129869e-03*w26*Jinv20-3.5612824675324673e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+1.0683847402597404e-03*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+1.0683847402597404e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-1.9176136363636362e-04*w49*Jinv10-1.8262987012987013e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-1.4793019480519481e-03*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-1.1231737012987013e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10-4.6570616883116886e-04*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-4.6570616883116886e-04*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11-3.5612824675324673e-04*w9*Jinv00-1.1231737012987013e-03*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+1.4245129870129869e-03*w24*Jinv00-1.8262987012987013e-05*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.4793019480519481e-03*w19*Jinv12+1.0683847402597404e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-4.6570616883116886e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-3.5612824675324673e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.1915584415584417e-04*w58*Jinv12-1.4793019480519481e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.2601461038961039e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+1.4245129870129869e-03*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00-4.6570616883116886e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+3.2873376623376621e-04*w40*Jinv10-9.5880681818181818e-04*w20*Jinv20-1.8262987012987013e-05*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-1.3971185064935066e-03*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56-1.8262987012987013e-05*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-4.6570616883116886e-04*Jinv21*w50-1.8262987012987013e-05*Jinv01*w27-4.6570616883116886e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[24][4]=(0.0)*D;
  A[24][5]=(0.0)*D;
  A[24][6]=(-3.8352272727272725e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-3.8352272727272725e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-9.5880681818181818e-04*w3*Jinv00-2.7394480519480521e-05*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.1505681818181819e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+1.9176136363636362e-04*w48*Jinv00-1.1505681818181819e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.1505681818181819e-03*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-6.0267857142857139e-04*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+4.7483766233766234e-04*w28*Jinv11+1.9176136363636362e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17-1.2053571428571428e-03*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12-2.7394480519480521e-05*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2+1.3971185064935066e-03*Jinv00*w18+1.3697240259740260e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-1.1505681818181819e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+2.6298701298701297e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-9.5880681818181818e-04*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13+1.3971185064935066e-03*w19*Jinv10-1.2053571428571428e-03*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+3.2873376623376621e-04*w26*Jinv20+6.6659902597402602e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-1.1505681818181819e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-3.8352272727272725e-04*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02+1.9176136363636362e-04*w49*Jinv10+4.7483766233766234e-04*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.3833198051948050e-03*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+1.3697240259740260e-03*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.3833198051948050e-03*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+2.3833198051948050e-03*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+2.3833198051948050e-03*w7*Jinv11+6.6659902597402602e-04*w9*Jinv00+1.3697240259740260e-03*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.2873376623376621e-04*w24*Jinv00+4.7483766233766234e-04*Jinv02*w27+2.3833198051948050e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+6.6659902597402602e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12+2.3833198051948050e-03*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-1.2053571428571428e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.2873376623376621e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2-6.0267857142857139e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-6.0267857142857139e-04*Jinv20*w17-2.7394480519480521e-05*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20+4.7483766233766234e-04*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-9.5880681818181818e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56+4.7483766233766234e-04*Jinv12*w28-1.1505681818181819e-03*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50+4.7483766233766234e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[24][7]=(0.0)*D;
  A[24][8]=(0.0)*D;
  A[24][9]=(1.1870941558441559e-04*Jinv00*w12+8.3198051948051951e-05*w28*Jinv10-6.7573051948051951e-04*Jinv20*w8+1.1870941558441559e-04*w13*Jinv10+4.9310064935064932e-04*w3*Jinv00-2.7394480519480521e-05*w32*Jinv20+3.6525974025974026e-05*Jinv20*w23+1.1870941558441559e-04*Jinv00*w54-3.2873376623376621e-04*w43*Jinv10-1.1668019480519480e-04*w58*Jinv10-2.7394480519480521e-05*w48*Jinv00-1.1668019480519480e-04*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+3.6525974025974026e-05*w45*Jinv00+8.3198051948051951e-05*Jinv00*w27-2.7394480519480521e-05*Jinv20*w50+8.3198051948051951e-05*Jinv20*w29+3.6525974025974026e-05*w46*Jinv10+2.9220779220779221e-04*Jinv00*w33-2.7394480519480521e-05*w31*Jinv10+3.6525974025974026e-05*w21*Jinv00-1.0227272727272728e-03*Jinv00*w18-6.7573051948051951e-04*w36*Jinv00+4.9310064935064932e-04*w51*Jinv00+4.9310064935064932e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-1.0227272727272728e-03*w19*Jinv10+2.9220779220779221e-04*w34*Jinv10-6.7573051948051951e-04*w6*Jinv00+1.1870941558441559e-04*Jinv20*w56-2.5568181818181820e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20+1.1870941558441559e-04*Jinv20*w14-2.7394480519480521e-05*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-6.7573051948051951e-04*w7*Jinv10-6.7573051948051951e-04*Jinv20*w38+4.9310064935064932e-04*Jinv20*w53+3.6525974025974026e-05*w22*Jinv10+4.9310064935064932e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-6.7573051948051951e-04*w37*Jinv10-2.5568181818181820e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59-1.1668019480519480e-04*w0*Jinv00+2.9220779220779221e-04*w35*Jinv20-2.5568181818181820e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-3.2873376623376621e-04*Jinv20*w17-2.7394480519480521e-05*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-1.0227272727272728e-03*w20*Jinv20+4.9310064935064932e-04*w5*Jinv20+1.1870941558441559e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+3.6525974025974026e-05*w47*Jinv20)*D;
  A[24][10]=(0.0)*D;
  A[24][11]=(0.0)*D;
  A[24][12]=(-5.7528409090909093e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+8.2183441558441553e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-5.7528409090909093e-04*w13*Jinv10+8.2183441558441553e-04*Jinv02*w39+2.5750811688311690e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20-5.4788961038961036e-04*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+3.1960227272727273e-04*Jinv21*w59+4.7666396103896100e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.5750811688311690e-03*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+8.7662337662337668e-04*w16*Jinv11+8.7662337662337668e-04*Jinv21*w17+2.0271915584415584e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+3.1047077922077924e-04*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00+4.7666396103896100e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+3.1960227272727273e-04*w58*Jinv11+3.1960227272727273e-04*w57*Jinv01+4.0543831168831168e-03*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+1.6436688311688311e-03*w16*Jinv10-1.4793019480519481e-03*Jinv22*w20+2.5750811688311690e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00+4.3831168831168834e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+3.1047077922077924e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-7.3965097402597403e-04*w13*Jinv11+8.7662337662337668e-04*w15*Jinv01+5.5702110389610384e-04*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+4.3831168831168834e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2-1.4793019480519481e-03*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+4.0543831168831168e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-9.5880681818181818e-04*w19*Jinv11-4.6570616883116886e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10-5.4788961038961036e-04*w4*Jinv11+8.2183441558441553e-04*w41*Jinv20-5.7528409090909093e-04*Jinv12*w13-1.4793019480519481e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21-9.5880681818181818e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+8.2183441558441553e-04*Jinv22*w41+4.7666396103896100e-03*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+4.7666396103896100e-03*Jinv22*w26+2.0271915584415584e-03*w46*Jinv11-5.7528409090909093e-04*Jinv20*w14-5.4788961038961036e-04*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+5.5702110389610384e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+8.2183441558441553e-04*w39*Jinv00-5.7528409090909093e-04*Jinv02*w12-9.5880681818181818e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+3.1047077922077924e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10+2.0271915584415584e-03*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23-4.6570616883116886e-04*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-1.3697240259740259e-04*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-4.6570616883116886e-04*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10-1.1231737012987013e-03*Jinv21*w56+4.7666396103896100e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-1.4793019480519481e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11-7.3965097402597403e-04*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.4793019480519481e-03*Jinv02*w18-7.3965097402597403e-04*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-1.1231737012987013e-03*Jinv01*w54-1.1231737012987013e-03*w55*Jinv11+4.7666396103896100e-03*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+1.6436688311688311e-03*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+1.6436688311688311e-03*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00+8.2183441558441553e-04*w40*Jinv10-1.4793019480519481e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-5.7528409090909093e-04*Jinv22*w14+4.3831168831168834e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+4.0543831168831168e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50+5.5702110389610384e-04*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[24][13]=(0.0)*D;
  A[24][14]=(0.0)*D;
  A[24][15]=(4.9310064935064932e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-5.4788961038961036e-04*w57*Jinv02+4.9310064935064932e-04*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44-6.0267857142857139e-04*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20+2.1367694805194807e-03*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-9.6793831168831166e-04*Jinv21*w59-1.2820616883116883e-02*w25*Jinv12-5.4788961038961036e-04*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-6.9034090909090911e-03*w16*Jinv11-6.9034090909090911e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+2.6298701298701297e-03*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-6.5746753246753243e-04*w10*Jinv12-1.2820616883116883e-02*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35-9.6793831168831166e-04*w58*Jinv11-9.6793831168831166e-04*w57*Jinv01-9.5332792207792200e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+4.9310064935064934e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41-2.9586038961038961e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-6.5746753246753243e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+1.8628246753246754e-03*w13*Jinv11-6.9034090909090911e-03*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+5.4788961038961042e-05*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39-5.4788961038961036e-04*Jinv22*w2+2.7942370129870131e-03*Jinv00*w18+1.7532467532467534e-03*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-9.5332792207792200e-03*Jinv01*w24-5.4788961038961036e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.7942370129870131e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34-2.9586038961038961e-03*Jinv22*w44-6.0267857142857139e-04*w4*Jinv10+2.6298701298701297e-03*Jinv12*w13+2.7942370129870131e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+2.7942370129870131e-03*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-8.5470779220779228e-03*w26*Jinv20-1.6436688311688311e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35-1.2820616883116883e-02*Jinv22*w26-3.1777597402597403e-03*w46*Jinv11+4.9310064935064932e-04*Jinv20*w14+2.6298701298701297e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+1.0957792207792208e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33+2.6298701298701297e-03*Jinv02*w12+2.7942370129870131e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-6.5746753246753243e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+1.7532467532467534e-03*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8+5.4788961038961042e-05*Jinv22*w50-2.0819805194805196e-03*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-2.0819805194805196e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-1.6436688311688311e-04*w9*Jinv00+1.7532467532467534e-03*w37*Jinv10+2.5750811688311690e-03*Jinv21*w56-8.5470779220779228e-03*w24*Jinv00+1.0957792207792207e-03*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.9310064935064934e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22+8.2183441558441553e-04*w49*Jinv11+1.8628246753246754e-03*Jinv21*w14-1.6436688311688311e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-5.4788961038961036e-04*w58*Jinv12+4.9310064935064934e-03*Jinv02*w18+1.8628246753246754e-03*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-2.0819805194805196e-03*Jinv21*w23+2.5750811688311690e-03*Jinv01*w54+2.5750811688311690e-03*w55*Jinv11-8.5470779220779228e-03*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+2.7942370129870131e-03*w20*Jinv20+1.0957792207792207e-03*Jinv22*w29+2.6298701298701297e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+2.1367694805194807e-03*Jinv02*w36-2.9586038961038961e-03*Jinv12*w43-6.0267857142857139e-04*w5*Jinv20+1.6436688311688311e-04*Jinv02*w51+2.6298701298701297e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28-9.5332792207792200e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12+8.2183441558441553e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+8.2183441558441553e-04*w48*Jinv01-5.4788961038961036e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[24][16]=(0.0)*D;
  A[24][17]=(0.0)*D;
  A[24][18]=(8.2183441558441553e-05*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-1.2327516233766234e-03*Jinv20*w8+8.2183441558441553e-05*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+8.2183441558441553e-05*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-1.5523538961038962e-04*w58*Jinv10+2.4655032467532466e-04*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-1.5523538961038962e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.0957792207792207e-03*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-4.7483766233766234e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+2.4655032467532466e-04*w31*Jinv10-2.4655032467532466e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-1.3149350649350649e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+1.1505681818181819e-03*Jinv01*w24-1.3149350649350649e-03*w34*Jinv11-2.6298701298701297e-03*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+4.3831168831168834e-04*w51*Jinv00+4.3831168831168834e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-8.2183441558441553e-04*w41*Jinv20-1.3149350649350649e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-2.6298701298701297e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*Jinv20*w56+3.7804383116883116e-03*w26*Jinv20-1.4610389610389610e-04*w11*Jinv20-1.3149350649350649e-03*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+2.4655032467532466e-04*w49*Jinv10-4.7483766233766234e-04*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-8.2183441558441553e-04*w39*Jinv00-2.6298701298701297e-03*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+4.3831168831168834e-04*Jinv20*w53-2.4655032467532466e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+4.3831168831168834e-04*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11-1.4610389610389610e-04*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10+5.2049512987012989e-04*Jinv21*w56+3.7804383116883116e-03*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-1.4610389610389610e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+5.2049512987012989e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-1.5523538961038962e-04*w57*Jinv00-1.5523538961038962e-04*Jinv20*w59-1.5523538961038962e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+5.2049512987012989e-04*Jinv01*w54+5.2049512987012989e-04*w55*Jinv11+3.7804383116883116e-03*w25*Jinv10-1.5523538961038962e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-8.2183441558441553e-04*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+4.3831168831168834e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+1.1505681818181819e-03*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50-4.7483766233766234e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[24][19]=(0.0)*D;
  A[24][20]=(0.0)*D;
  A[24][21]=(2.3833198051948050e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-2.1367694805194807e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+2.3833198051948050e-03*w13*Jinv10-2.1367694805194807e-03*Jinv02*w39-2.6846590909090908e-03*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3-3.6982548701298703e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54-3.7439123376623376e-04*Jinv21*w59-7.3965097402597406e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-3.5612824675324673e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-2.6846590909090908e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17-2.4655032467532467e-03*Jinv01*w45+2.3833198051948050e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-7.8530844155844158e-04*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00-7.3965097402597406e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-3.7439123376623376e-04*w58*Jinv11-3.7439123376623376e-04*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26-3.5612824675324673e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-4.2735389610389614e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20-2.6846590909090908e-03*w43*Jinv11-3.6982548701298703e-03*w45*Jinv00-5.4788961038961036e-04*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.8530844155844158e-04*w11*Jinv22-3.5612824675324673e-04*Jinv11*w1+1.1505681818181819e-03*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-1.1414366883116883e-03*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-3.6982548701298703e-03*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-3.6982548701298703e-03*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+7.1225649350649346e-04*Jinv02*w3-3.6982548701298703e-03*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22-5.4788961038961036e-04*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2+2.6298701298701297e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+8.2183441558441553e-05*Jinv21*w38-3.6982548701298703e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+1.3971185064935066e-03*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+2.3833198051948050e-03*Jinv12*w13+2.6298701298701297e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21+1.3971185064935066e-03*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-2.1367694805194807e-03*Jinv22*w41-7.3965097402597406e-03*w26*Jinv20-7.8530844155844158e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+2.7394480519480518e-04*w31*Jinv11-7.3965097402597406e-03*Jinv22*w26-2.4655032467532467e-03*w46*Jinv11+2.3833198051948050e-03*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21+2.3833198051948050e-03*Jinv02*w54+7.1225649350649346e-04*w4*Jinv12-3.6982548701298703e-03*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-1.1414366883116883e-03*Jinv21*w29-2.1367694805194807e-03*w39*Jinv00+2.3833198051948050e-03*Jinv02*w12+1.3971185064935066e-03*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-7.8530844155844158e-04*Jinv02*w9+1.6436688311688311e-04*w7*Jinv10-2.4655032467532467e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38-3.6982548701298703e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-3.6982548701298703e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8-3.6982548701298703e-03*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-7.8530844155844158e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56-7.3965097402597406e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12+7.1225649350649346e-04*Jinv22*w5-3.6982548701298703e-03*w47*Jinv22+1.1505681818181819e-03*Jinv21*w14-7.8530844155844158e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18+1.1505681818181819e-03*Jinv01*w12-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11-7.3965097402597406e-03*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-4.2735389610389614e-03*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9-4.2735389610389614e-03*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+2.6298701298701297e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.3833198051948050e-03*Jinv22*w14-5.4788961038961036e-04*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+2.3833198051948050e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00-3.6982548701298703e-03*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12-1.1414366883116883e-03*Jinv01*w27-7.4878246753246752e-04*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[24][22]=(0.0)*D;
  A[24][23]=(0.0)*D;
  A[24][24]=(-2.3833198051948050e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8-2.3833198051948050e-03*w13*Jinv10+4.7666396103896100e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-1.0683847402597404e-03*Jinv01*w3+3.6982548701298703e-03*Jinv20*w23-2.3833198051948050e-03*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w43*Jinv10+4.7666396103896100e-03*w42*Jinv01+6.5746753246753243e-04*w11*Jinv21+4.7666396103896100e-03*w16*Jinv11+4.7666396103896100e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01+7.3965097402597406e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+4.2735389610389614e-03*w16*Jinv10+4.7666396103896100e-03*w43*Jinv11+3.6982548701298703e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-2.0271915584415584e-03*w13*Jinv11+4.7666396103896100e-03*w15*Jinv01-9.1314935064935068e-05*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+3.6982548701298703e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10+3.6982548701298703e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.6298701298701297e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+5.7528409090909093e-04*Jinv21*w38+7.3965097402597406e-03*Jinv01*w24+1.6436688311688311e-03*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11-1.0683847402597404e-03*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-1.0683847402597404e-03*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-2.6298701298701297e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00-2.3833198051948050e-03*Jinv20*w56+7.3965097402597406e-03*w26*Jinv20+7.8530844155844158e-04*w11*Jinv20+1.6436688311688311e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.4655032467532467e-03*w46*Jinv11-2.3833198051948050e-03*Jinv20*w14-1.0683847402597404e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10-9.1314935064935068e-05*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.3149350649350649e-03*Jinv01*w18+5.7528409090909093e-04*w37*Jinv11+5.7528409090909093e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10+2.4655032467532467e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38-1.0683847402597404e-03*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53+3.6982548701298703e-03*w22*Jinv10+2.4655032467532467e-03*w22*Jinv11+2.4655032467532467e-03*w21*Jinv01-1.0683847402597404e-03*Jinv01*w51+5.7528409090909093e-04*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+5.7528409090909093e-04*w7*Jinv11+7.8530844155844158e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-2.0271915584415584e-03*Jinv21*w56+7.3965097402597406e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-2.0271915584415584e-03*Jinv21*w14+7.8530844155844158e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-2.0271915584415584e-03*Jinv01*w12+5.7528409090909093e-04*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+2.4655032467532467e-03*Jinv21*w23-2.0271915584415584e-03*Jinv01*w54-2.0271915584415584e-03*w55*Jinv11+7.3965097402597406e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+4.2735389610389614e-03*w15*Jinv00+6.5746753246753243e-04*Jinv01*w9+4.2735389610389614e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.6298701298701297e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+7.3965097402597406e-03*w25*Jinv11-2.3833198051948050e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00+3.6982548701298703e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-9.1314935064935068e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+6.5746753246753243e-04*w10*Jinv11)*D;
  A[24][25]=(0.0)*D;
  A[24][26]=(0.0)*D;
  A[24][27]=(6.5746753246753243e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3+1.5726461038961039e-04*Jinv21*w59+1.5726461038961039e-04*Jinv21*w2+6.5746753246753243e-04*w42*Jinv01+2.0495129870129870e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.1870941558441558e-03*Jinv01*w45+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+2.3741883116883116e-03*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+6.5746753246753243e-04*w43*Jinv11+3.2873376623376621e-04*Jinv21*w41+1.5726461038961039e-04*Jinv11*w1-6.7573051948051951e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+3.2873376623376621e-04*Jinv01*w39-2.6481331168831169e-04*Jinv21*w38+2.3741883116883116e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-1.0227272727272728e-03*w19*Jinv11-1.0957792207792208e-04*Jinv21*w53-1.0957792207792208e-04*w4*Jinv11-5.4788961038961042e-05*w32*Jinv21-1.0227272727272728e-03*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11+1.1870941558441558e-03*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-1.0227272727272728e-03*Jinv01*w18-2.6481331168831169e-04*w37*Jinv11-2.6481331168831169e-04*w6*Jinv01+1.1870941558441558e-03*w47*Jinv21-1.0957792207792208e-04*w52*Jinv11+1.1870941558441558e-03*w22*Jinv11+1.1870941558441558e-03*w21*Jinv01-1.0957792207792208e-04*Jinv01*w51-2.6481331168831169e-04*w36*Jinv01-2.6481331168831169e-04*w7*Jinv11-6.7573051948051951e-04*Jinv21*w56-5.4788961038961042e-05*w49*Jinv11-6.7573051948051951e-04*Jinv21*w14-6.7573051948051951e-04*Jinv01*w12-2.6481331168831169e-04*Jinv21*w8+1.1870941558441558e-03*Jinv21*w23-6.7573051948051951e-04*Jinv01*w54-6.7573051948051951e-04*w55*Jinv11+2.0495129870129870e-04*Jinv01*w9+3.2873376623376621e-04*w40*Jinv11-5.4788961038961042e-05*Jinv01*w30+2.3741883116883116e-03*w25*Jinv11-5.4788961038961042e-05*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+2.0495129870129870e-04*w10*Jinv11)*D;
  A[24][28]=(0.0)*D;
  A[24][29]=(0.0)*D;
  A[24][30]=(8.2183441558441553e-05*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12-4.6570616883116886e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+2.7394480519480521e-05*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-1.9176136363636362e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-5.4788961038961042e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02-2.4655032467532466e-04*w48*Jinv00+8.2183441558441553e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-6.3920454545454550e-05*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+8.7662337662337668e-04*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-3.5612824675324673e-04*Jinv02*w3-1.9176136363636362e-04*w21*Jinv00+7.1225649350649346e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+2.7394480519480521e-05*Jinv22*w59+8.7662337662337668e-04*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+8.2183441558441553e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+7.1225649350649346e-04*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+8.7662337662337668e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20-4.6570616883116886e-04*w52*Jinv12+8.7662337662337668e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+7.1225649350649346e-04*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+8.7662337662337668e-04*Jinv01*w33+7.1225649350649346e-04*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-2.4655032467532466e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.5612824675324673e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8-1.9176136363636362e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-1.9176136363636362e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.9176136363636362e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+2.7394480519480521e-05*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02-3.5612824675324673e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.7394480519480521e-05*w58*Jinv12+8.2183441558441553e-05*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+8.2183441558441553e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+8.7662337662337668e-04*w35*Jinv20-1.9176136363636362e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+7.1225649350649346e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-4.6570616883116886e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+7.1225649350649346e-04*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[24][31]=(0.0)*D;
  A[24][32]=(0.0)*D;
  A[24][33]=(-3.2873376623376621e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+1.3149350649350649e-03*Jinv22*w53+1.2601461038961039e-03*Jinv20*w8-3.4699675324675324e-04*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10+3.9448051948051946e-03*Jinv21*w44-1.2053571428571428e-03*w3*Jinv00-8.2183441558441553e-04*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3+1.3149350649350649e-03*Jinv22*w38+3.8352272727272725e-04*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59+3.4699675324675324e-04*w0*Jinv02+4.2004870129870131e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+3.9448051948051946e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-2.1915584415584414e-03*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01-3.2873376623376621e-03*Jinv21*w26+4.2004870129870131e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+3.9448051948051946e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+4.2004870129870131e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+9.8620129870129864e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.6436688311688311e-03*Jinv00*w33+1.0957792207792207e-03*w22*Jinv12-8.2183441558441553e-04*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-8.2183441558441553e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+3.4699675324675324e-04*Jinv22*w2+1.7532467532467534e-03*Jinv00*w18+2.5750811688311690e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45-3.2873376623376621e-03*Jinv01*w24-3.4699675324675324e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+1.9724025974025973e-03*Jinv22*w44-1.2053571428571428e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13+1.7532467532467534e-03*w19*Jinv10+1.6436688311688311e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.2601461038961039e-03*w6*Jinv00+1.3149350649350649e-03*w37*Jinv12+3.8352272727272725e-04*Jinv20*w56-2.1915584415584414e-03*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-2.1915584415584414e-03*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+9.8620129870129864e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12+2.1367694805194807e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+1.3149350649350649e-03*w6*Jinv01+1.2601461038961039e-03*w7*Jinv10-2.1915584415584414e-03*w47*Jinv21+2.5750811688311690e-03*Jinv20*w38+1.0957792207792207e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-8.2183441558441553e-04*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8+8.2183441558441553e-04*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-1.0957792207792207e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-8.2183441558441553e-04*Jinv02*w30+8.2183441558441553e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.3149350649350649e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.5750811688311690e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56-2.1915584415584414e-03*w24*Jinv00-1.3149350649350649e-03*w6*Jinv02-1.3149350649350649e-03*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44-3.4699675324675324e-04*w58*Jinv12+5.4788961038961042e-05*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+3.8352272727272725e-04*w0*Jinv00+1.6436688311688311e-03*w35*Jinv20-1.0957792207792207e-03*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11-2.1915584415584414e-03*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+8.2183441558441553e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17-8.2183441558441553e-04*w30*Jinv00+1.7532467532467534e-03*w20*Jinv20-7.1225649350649346e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+1.3149350649350649e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.2053571428571428e-03*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-3.2873376623376621e-03*w25*Jinv11+3.8352272727272725e-04*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+9.8620129870129864e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+3.4699675324675324e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[24][34]=(0.0)*D;
  A[24][35]=(0.0)*D;
  A[24][36]=(2.4655032467532466e-04*Jinv00*w12-4.8396915584415583e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53-1.0135957792207792e-03*Jinv20*w8-4.2918019480519480e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+8.4922889610389610e-04*w3*Jinv00+4.6570616883116886e-04*w32*Jinv20+1.9176136363636362e-04*Jinv20*w23-2.3833198051948050e-03*Jinv22*w38+1.3697240259740259e-04*Jinv00*w54+1.1505681818181819e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-2.7394480519480520e-03*w43*Jinv10+5.2049512987012989e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00+1.1505681818181819e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.1960227272727273e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20+3.8352272727272725e-04*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02-4.8396915584415583e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+2.4655032467532466e-04*Jinv20*w50-4.8396915584415583e-04*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-2.5202922077922079e-03*Jinv00*w33-1.6436688311688311e-04*w22*Jinv12+4.6570616883116886e-04*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+1.9176136363636362e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-2.0271915584415584e-03*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45-4.2918019480519480e-04*Jinv22*w59+4.3831168831168834e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+8.4922889610389610e-04*w4*Jinv10-7.1225649350649346e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10-2.5202922077922079e-03*w34*Jinv10-1.0135957792207792e-03*w6*Jinv00-2.3833198051948050e-03*w37*Jinv12+1.3697240259740259e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+1.4793019480519481e-03*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+1.1505681818181819e-03*Jinv22*w26+2.4655032467532466e-04*Jinv20*w14+5.2049512987012989e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10-7.1225649350649346e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.0135957792207792e-03*w7*Jinv10-2.0271915584415584e-03*Jinv20*w38-1.6436688311688311e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+1.9176136363636362e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+4.3831168831168834e-04*w52*Jinv10+1.1870941558441559e-04*w9*Jinv00-2.0271915584415584e-03*w37*Jinv10+1.4793019480519481e-03*w24*Jinv00-4.7483766233766234e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22+1.1870941558441559e-04*w10*Jinv10-2.7394480519480520e-03*Jinv20*w44-4.2918019480519480e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-3.1960227272727273e-04*w0*Jinv00-2.5202922077922079e-03*w35*Jinv20+1.4793019480519481e-03*w25*Jinv10-3.1960227272727273e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+4.6570616883116886e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-4.7483766233766234e-04*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-2.3833198051948050e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.4922889610389610e-04*w5*Jinv20+1.3971185064935066e-03*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56-4.7483766233766234e-04*Jinv12*w28+1.3697240259740259e-04*w55*Jinv10-2.7394480519480520e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[24][37]=(0.0)*D;
  A[24][38]=(0.0)*D;
  A[24][39]=(1.3149350649350649e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+7.1225649350649346e-04*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10-3.2873376623376621e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-2.1367694805194807e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59+2.1915584415584417e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+8.2183441558441553e-04*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+3.8352272727272725e-04*w48*Jinv00-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10-3.2873376623376621e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-3.1047077922077924e-04*Jinv11*w1+9.3141233766233771e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.7532467532467534e-03*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00-9.8620129870129864e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-9.8620129870129864e-04*w41*Jinv20-8.2183441558441553e-04*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+1.7532467532467534e-03*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-3.1777597402597403e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14+8.2183441558441553e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-8.2183441558441553e-04*Jinv02*w12+1.7532467532467534e-03*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11+3.2873376623376621e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50-1.0409902597402598e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-1.0409902597402598e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10+1.7532467532467534e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00-3.8352272727272725e-04*w6*Jinv02-7.1225649350649346e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11+9.3141233766233771e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+9.3141233766233771e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59-2.7394480519480518e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-1.0409902597402598e-03*Jinv21*w23+1.7532467532467534e-03*Jinv01*w54+1.7532467532467534e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-8.2183441558441553e-04*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+7.1225649350649346e-04*Jinv02*w51+8.2183441558441553e-04*Jinv22*w56-3.1777597402597403e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[24][40]=(0.0)*D;
  A[24][41]=(0.0)*D;
  A[24][42]=(-1.3149350649350649e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8+5.4788961038961036e-04*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20+1.1505681818181819e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59+1.2820616883116883e-02*w25*Jinv12+5.4788961038961036e-04*w0*Jinv02-4.2004870129870131e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+2.1915584415584414e-03*Jinv01*w45-2.6298701298701297e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+2.9586038961038961e-03*w15*Jinv02+6.5746753246753243e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00+1.2820616883116883e-02*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+3.2873376623376621e-03*Jinv21*w26-4.2004870129870131e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-4.9310064935064934e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27+6.5746753246753243e-04*w11*Jinv22-4.2004870129870131e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-9.8620129870129864e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10+2.9586038961038961e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+4.2735389610389614e-03*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+5.4788961038961036e-04*Jinv22*w2-2.1367694805194807e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+3.2873376623376621e-03*Jinv01*w24+5.4788961038961036e-04*Jinv22*w59-4.9310064935064934e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+1.1505681818181819e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.6298701298701297e-03*Jinv12*w13-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12-4.9310064935064934e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11+1.2820616883116883e-02*Jinv22*w26+2.1915584415584414e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14+1.1505681818181819e-03*w5*Jinv21-2.6298701298701297e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-9.8620129870129864e-04*Jinv21*w29-4.9310064935064934e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-2.6298701298701297e-03*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+6.5746753246753243e-04*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+2.1915584415584414e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38+4.2735389610389614e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-7.6704545454545450e-04*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.9310064935064934e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-5.4788961038961042e-05*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+5.4788961038961036e-04*w58*Jinv12-4.9310064935064934e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29-2.6298701298701297e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+7.6704545454545450e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-2.6298701298701297e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28+3.2873376623376621e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20+2.9586038961038961e-03*w16*Jinv12-5.4788961038961042e-05*Jinv21*w50-9.8620129870129864e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+5.4788961038961036e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[24][43]=(0.0)*D;
  A[24][44]=(0.0)*D;
  A[24][45]=(2.1367694805194807e-03*w40*Jinv12-7.1225649350649346e-04*Jinv22*w53+7.4878246753246752e-04*w57*Jinv02+2.1367694805194807e-03*Jinv02*w39+3.1777597402597403e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-1.6436688311688311e-04*Jinv22*w38+3.9265422077922079e-04*Jinv21*w59+7.3965097402597406e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+3.7439123376623376e-04*Jinv21*w2+3.1777597402597403e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45-2.3833198051948050e-03*Jinv12*w55+4.2735389610389614e-03*w15*Jinv02+7.8530844155844158e-04*w10*Jinv12+7.3965097402597406e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.7439123376623376e-04*w0*Jinv01-2.6298701298701297e-03*Jinv22*w20+3.1777597402597403e-03*w43*Jinv11+1.5888798701298701e-03*Jinv21*w41+4.2735389610389614e-03*w42*Jinv02+7.8530844155844158e-04*w11*Jinv22+3.7439123376623376e-04*Jinv11*w1-1.0135957792207792e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11+4.2735389610389614e-03*Jinv22*w17+3.6982548701298703e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-3.5612824675324673e-04*w32*Jinv22+1.5888798701298701e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+3.6982548701298703e-03*Jinv02*w45+2.4655032467532467e-03*Jinv01*w24+7.4878246753246752e-04*Jinv22*w59+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.3833198051948050e-03*Jinv12*w13-3.5612824675324673e-04*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-1.6436688311688311e-04*w37*Jinv12+2.1367694805194807e-03*Jinv22*w41-7.1225649350649346e-04*w52*Jinv12+1.0957792207792207e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+7.3965097402597406e-03*Jinv22*w26+2.4655032467532467e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-2.3833198051948050e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+3.6982548701298703e-03*w21*Jinv02-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33-2.3833198051948050e-03*Jinv02*w12-1.2327516233766234e-03*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+7.8530844155844158e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+2.4655032467532467e-03*w47*Jinv21+3.6982548701298703e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12+3.6982548701298703e-03*w46*Jinv12-1.6436688311688311e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11-1.6436688311688311e-04*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-8.2183441558441553e-05*w7*Jinv11-1.2327516233766234e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+3.6982548701298703e-03*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11-1.0135957792207792e-03*Jinv21*w14+7.4878246753246752e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-1.0135957792207792e-03*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+1.2327516233766234e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+3.9265422077922079e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.3833198051948050e-03*Jinv22*w14+1.5888798701298701e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-3.5612824675324673e-04*Jinv01*w30-7.1225649350649346e-04*Jinv02*w51-2.3833198051948050e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+2.4655032467532467e-03*w25*Jinv11+4.2735389610389614e-03*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+3.9265422077922079e-04*w10*Jinv11)*D;
  A[24][46]=(0.0)*D;
  A[24][47]=(0.0)*D;
  A[24][48]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10-3.2873376623376621e-04*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-3.2873376623376621e-04*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480521e-05*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+3.5612824675324673e-04*w48*Jinv00-8.2183441558441553e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+7.6704545454545450e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10+4.6570616883116886e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-2.7394480519480521e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-2.7394480519480521e-05*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-8.2183441558441553e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-3.2873376623376621e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+1.6436688311688311e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54+4.6570616883116886e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-2.7394480519480521e-05*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+3.5612824675324673e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-2.7394480519480521e-05*Jinv02*w30-5.2049512987012989e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02+4.6570616883116886e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-2.7394480519480518e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-8.2183441558441553e-04*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+1.3697240259740259e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20+2.4655032467532466e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56+7.1225649350649346e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-2.7394480519480521e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[24][49]=(0.0)*D;
  A[24][50]=(0.0)*D;
  A[24][51]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-1.0683847402597404e-03*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+8.2183441558441553e-05*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12-2.1915584415584417e-04*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+1.4793019480519481e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-3.5612824675324673e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+2.4655032467532466e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+4.6570616883116886e-04*w32*Jinv22-2.1915584415584417e-04*Jinv22*w2+5.2049512987012989e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+3.8352272727272725e-04*Jinv02*w45+1.2784090909090910e-04*Jinv22*w59-7.1225649350649346e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-3.8352272727272725e-04*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.0683847402597404e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-3.8352272727272725e-04*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+5.2049512987012989e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+4.6570616883116886e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-7.1225649350649346e-04*Jinv20*w53+1.3971185064935066e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+1.3971185064935066e-03*Jinv12*w7+4.6570616883116886e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-7.1225649350649346e-04*w52*Jinv10-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00+1.8262987012987013e-05*Jinv02*w27+1.3971185064935066e-03*w6*Jinv02+1.4793019480519481e-03*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-1.3697240259740259e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12+1.4793019480519481e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20+1.8262987012987013e-05*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14+8.2183441558441553e-05*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-1.0683847402597404e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+1.8262987012987013e-05*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12-2.1915584415584417e-04*Jinv12*w1)*D;
  A[24][52]=(0.0)*D;
  A[24][53]=(0.0)*D;
  A[24][54]=(-8.2183441558441553e-04*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-2.1002435064935065e-04*w57*Jinv02-8.2183441558441553e-04*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+8.2183441558441553e-05*Jinv01*w3+1.3697240259740259e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59-4.7666396103896100e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+5.7528409090909093e-04*Jinv12*w55-2.6298701298701297e-03*w15*Jinv02-3.1047077922077924e-04*w10*Jinv12-4.7666396103896100e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-7.1225649350649346e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01+1.4793019480519481e-03*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-3.8352272727272725e-04*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-3.1047077922077924e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11-2.6298701298701297e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+2.4655032467532466e-04*w32*Jinv22-3.8352272727272725e-04*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+5.2049512987012989e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-7.1225649350649346e-04*Jinv01*w24-2.1002435064935065e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+5.2049512987012989e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+2.7394480519480518e-04*w32*Jinv21+5.2049512987012989e-04*Jinv21*w20+1.3697240259740259e-04*w37*Jinv12-8.2183441558441553e-04*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-4.7666396103896100e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+8.2183441558441553e-05*w5*Jinv21+5.7528409090909093e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+5.2049512987012989e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11-3.1047077922077924e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-1.6436688311688311e-04*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+2.4655032467532466e-04*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8-8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+2.4655032467532466e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+5.2049512987012989e-04*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+1.4793019480519481e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-2.1002435064935065e-04*w58*Jinv12+1.4793019480519481e-03*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-8.2183441558441553e-05*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-3.8352272727272725e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51+5.7528409090909093e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-7.1225649350649346e-04*w25*Jinv11-2.6298701298701297e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[24][55]=(0.0)*D;
  A[24][56]=(0.0)*D;
  A[24][57]=(2.1915584415584417e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+2.1915584415584417e-04*Jinv02*w39+1.1870941558441559e-04*Jinv22*w38+1.4975649350649350e-03*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-2.6481331168831169e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+1.4975649350649350e-03*Jinv02*w24+3.4699675324675324e-04*Jinv22*w35-5.8441558441558442e-04*Jinv22*w20+4.3831168831168834e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-2.1915584415584417e-04*Jinv02*w3-2.7394480519480521e-05*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.6525974025974026e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+3.4699675324675324e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-4.2918019480519480e-04*Jinv12*w13+1.1870941558441559e-04*w37*Jinv12+2.1915584415584417e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12+1.4975649350649350e-03*Jinv22*w26-2.6481331168831169e-04*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-4.2918019480519480e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9+7.4878246753246752e-04*Jinv22*w23-2.7394480519480521e-05*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-4.2918019480519480e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7-2.7394480519480521e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+6.4935064935064935e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-5.8441558441558442e-04*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-5.8441558441558442e-04*Jinv02*w18+3.4699675324675324e-04*Jinv02*w33-5.4788961038961042e-05*w48*Jinv02+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+1.1870941558441559e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-1.0957792207792208e-04*Jinv02*w51-2.6481331168831169e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[24][58]=(0.0)*D;
  A[24][59]=(0.0)*D;
  A[25][0]=(0.0)*D;
  A[25][1]=(2.6481331168831169e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+2.6481331168831169e-04*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59-1.4975649350649350e-03*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17-7.4878246753246752e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00-1.4975649350649350e-03*Jinv02*w24-3.4699675324675324e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01-1.4975649350649350e-03*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10+5.8441558441558442e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+2.6481331168831169e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-3.4699675324675324e-04*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+5.8441558441558442e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-7.4878246753246752e-04*Jinv02*w45-1.4975649350649350e-03*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.4699675324675324e-04*w34*Jinv11+5.8441558441558442e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+2.1915584415584417e-04*w51*Jinv00-3.4699675324675324e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-2.1915584415584417e-04*w41*Jinv20+2.6481331168831169e-04*Jinv12*w13+5.8441558441558442e-04*w19*Jinv10-3.4699675324675324e-04*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21+5.8441558441558442e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41-1.4975649350649350e-03*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12-3.4699675324675324e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11-1.4975649350649350e-03*Jinv22*w26-7.4878246753246752e-04*w46*Jinv11+2.6481331168831169e-04*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29-3.4699675324675324e-04*Jinv01*w33-2.1915584415584417e-04*w39*Jinv00+2.6481331168831169e-04*Jinv02*w12+5.8441558441558442e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10-7.4878246753246752e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+2.1915584415584417e-04*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+2.7394480519480521e-05*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01+2.1915584415584417e-04*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56-1.4975649350649350e-03*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02+5.8441558441558442e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11+2.6481331168831169e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+5.8441558441558442e-04*Jinv02*w18+2.6481331168831169e-04*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-3.4699675324675324e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-3.4699675324675324e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11-1.4975649350649350e-03*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-2.1915584415584417e-04*w40*Jinv10+5.8441558441558442e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+2.6481331168831169e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-1.4975649350649350e-03*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[25][2]=(0.0)*D;
  A[25][3]=(0.0)*D;
  A[25][4]=(-9.1314935064935065e-06*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8+2.1915584415584417e-04*w57*Jinv02+7.1225649350649346e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23-1.3971185064935066e-03*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-1.9176136363636362e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10-1.4793019480519481e-03*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01-1.8262987012987013e-05*w28*Jinv11-1.9176136363636362e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+1.2601461038961039e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+1.0683847402597404e-03*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-9.5880681818181818e-04*Jinv00*w18-1.1231737012987013e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24+2.1915584415584417e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-1.4793019480519481e-03*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+1.0683847402597404e-03*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-9.5880681818181818e-04*w19*Jinv10+1.2601461038961039e-03*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-1.4793019480519481e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00-1.3971185064935066e-03*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41+1.4245129870129869e-03*w26*Jinv20-3.5612824675324673e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+1.0683847402597404e-03*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+1.0683847402597404e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-1.9176136363636362e-04*w49*Jinv10-1.8262987012987013e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-1.4793019480519481e-03*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-1.1231737012987013e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10-4.6570616883116886e-04*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-4.6570616883116886e-04*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11-3.5612824675324673e-04*w9*Jinv00-1.1231737012987013e-03*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+1.4245129870129869e-03*w24*Jinv00-1.8262987012987013e-05*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.4793019480519481e-03*w19*Jinv12+1.0683847402597404e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-4.6570616883116886e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-3.5612824675324673e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.1915584415584417e-04*w58*Jinv12-1.4793019480519481e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.2601461038961039e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+1.4245129870129869e-03*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00-4.6570616883116886e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+3.2873376623376621e-04*w40*Jinv10-9.5880681818181818e-04*w20*Jinv20-1.8262987012987013e-05*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-1.3971185064935066e-03*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56-1.8262987012987013e-05*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-4.6570616883116886e-04*Jinv21*w50-1.8262987012987013e-05*Jinv01*w27-4.6570616883116886e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[25][5]=(0.0)*D;
  A[25][6]=(0.0)*D;
  A[25][7]=(-3.8352272727272725e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-3.8352272727272725e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-9.5880681818181818e-04*w3*Jinv00-2.7394480519480521e-05*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.1505681818181819e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+1.9176136363636362e-04*w48*Jinv00-1.1505681818181819e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.1505681818181819e-03*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-6.0267857142857139e-04*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+4.7483766233766234e-04*w28*Jinv11+1.9176136363636362e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17-1.2053571428571428e-03*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12-2.7394480519480521e-05*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2+1.3971185064935066e-03*Jinv00*w18+1.3697240259740260e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-1.1505681818181819e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+2.6298701298701297e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-9.5880681818181818e-04*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13+1.3971185064935066e-03*w19*Jinv10-1.2053571428571428e-03*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+3.2873376623376621e-04*w26*Jinv20+6.6659902597402602e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-1.1505681818181819e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-3.8352272727272725e-04*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02+1.9176136363636362e-04*w49*Jinv10+4.7483766233766234e-04*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.3833198051948050e-03*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+1.3697240259740260e-03*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.3833198051948050e-03*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+2.3833198051948050e-03*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+2.3833198051948050e-03*w7*Jinv11+6.6659902597402602e-04*w9*Jinv00+1.3697240259740260e-03*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.2873376623376621e-04*w24*Jinv00+4.7483766233766234e-04*Jinv02*w27+2.3833198051948050e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+6.6659902597402602e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12+2.3833198051948050e-03*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-1.2053571428571428e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.2873376623376621e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2-6.0267857142857139e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-6.0267857142857139e-04*Jinv20*w17-2.7394480519480521e-05*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20+4.7483766233766234e-04*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-9.5880681818181818e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56+4.7483766233766234e-04*Jinv12*w28-1.1505681818181819e-03*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50+4.7483766233766234e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[25][8]=(0.0)*D;
  A[25][9]=(0.0)*D;
  A[25][10]=(1.1870941558441559e-04*Jinv00*w12+8.3198051948051951e-05*w28*Jinv10-6.7573051948051951e-04*Jinv20*w8+1.1870941558441559e-04*w13*Jinv10+4.9310064935064932e-04*w3*Jinv00-2.7394480519480521e-05*w32*Jinv20+3.6525974025974026e-05*Jinv20*w23+1.1870941558441559e-04*Jinv00*w54-3.2873376623376621e-04*w43*Jinv10-1.1668019480519480e-04*w58*Jinv10-2.7394480519480521e-05*w48*Jinv00-1.1668019480519480e-04*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+3.6525974025974026e-05*w45*Jinv00+8.3198051948051951e-05*Jinv00*w27-2.7394480519480521e-05*Jinv20*w50+8.3198051948051951e-05*Jinv20*w29+3.6525974025974026e-05*w46*Jinv10+2.9220779220779221e-04*Jinv00*w33-2.7394480519480521e-05*w31*Jinv10+3.6525974025974026e-05*w21*Jinv00-1.0227272727272728e-03*Jinv00*w18-6.7573051948051951e-04*w36*Jinv00+4.9310064935064932e-04*w51*Jinv00+4.9310064935064932e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-1.0227272727272728e-03*w19*Jinv10+2.9220779220779221e-04*w34*Jinv10-6.7573051948051951e-04*w6*Jinv00+1.1870941558441559e-04*Jinv20*w56-2.5568181818181820e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20+1.1870941558441559e-04*Jinv20*w14-2.7394480519480521e-05*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-6.7573051948051951e-04*w7*Jinv10-6.7573051948051951e-04*Jinv20*w38+4.9310064935064932e-04*Jinv20*w53+3.6525974025974026e-05*w22*Jinv10+4.9310064935064932e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-6.7573051948051951e-04*w37*Jinv10-2.5568181818181820e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59-1.1668019480519480e-04*w0*Jinv00+2.9220779220779221e-04*w35*Jinv20-2.5568181818181820e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-3.2873376623376621e-04*Jinv20*w17-2.7394480519480521e-05*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-1.0227272727272728e-03*w20*Jinv20+4.9310064935064932e-04*w5*Jinv20+1.1870941558441559e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+3.6525974025974026e-05*w47*Jinv20)*D;
  A[25][11]=(0.0)*D;
  A[25][12]=(0.0)*D;
  A[25][13]=(-5.7528409090909093e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+8.2183441558441553e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-5.7528409090909093e-04*w13*Jinv10+8.2183441558441553e-04*Jinv02*w39+2.5750811688311690e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20-5.4788961038961036e-04*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+3.1960227272727273e-04*Jinv21*w59+4.7666396103896100e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.5750811688311690e-03*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+8.7662337662337668e-04*w16*Jinv11+8.7662337662337668e-04*Jinv21*w17+2.0271915584415584e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+3.1047077922077924e-04*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00+4.7666396103896100e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+3.1960227272727273e-04*w58*Jinv11+3.1960227272727273e-04*w57*Jinv01+4.0543831168831168e-03*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+1.6436688311688311e-03*w16*Jinv10-1.4793019480519481e-03*Jinv22*w20+2.5750811688311690e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00+4.3831168831168834e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+3.1047077922077924e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-7.3965097402597403e-04*w13*Jinv11+8.7662337662337668e-04*w15*Jinv01+5.5702110389610384e-04*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+4.3831168831168834e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2-1.4793019480519481e-03*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+4.0543831168831168e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-9.5880681818181818e-04*w19*Jinv11-4.6570616883116886e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10-5.4788961038961036e-04*w4*Jinv11+8.2183441558441553e-04*w41*Jinv20-5.7528409090909093e-04*Jinv12*w13-1.4793019480519481e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21-9.5880681818181818e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+8.2183441558441553e-04*Jinv22*w41+4.7666396103896100e-03*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+4.7666396103896100e-03*Jinv22*w26+2.0271915584415584e-03*w46*Jinv11-5.7528409090909093e-04*Jinv20*w14-5.4788961038961036e-04*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+5.5702110389610384e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+8.2183441558441553e-04*w39*Jinv00-5.7528409090909093e-04*Jinv02*w12-9.5880681818181818e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+3.1047077922077924e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10+2.0271915584415584e-03*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23-4.6570616883116886e-04*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-1.3697240259740259e-04*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-4.6570616883116886e-04*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10-1.1231737012987013e-03*Jinv21*w56+4.7666396103896100e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-1.4793019480519481e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11-7.3965097402597403e-04*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.4793019480519481e-03*Jinv02*w18-7.3965097402597403e-04*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-1.1231737012987013e-03*Jinv01*w54-1.1231737012987013e-03*w55*Jinv11+4.7666396103896100e-03*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+1.6436688311688311e-03*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+1.6436688311688311e-03*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00+8.2183441558441553e-04*w40*Jinv10-1.4793019480519481e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-5.7528409090909093e-04*Jinv22*w14+4.3831168831168834e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+4.0543831168831168e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50+5.5702110389610384e-04*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[25][14]=(0.0)*D;
  A[25][15]=(0.0)*D;
  A[25][16]=(4.9310064935064932e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-5.4788961038961036e-04*w57*Jinv02+4.9310064935064932e-04*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44-6.0267857142857139e-04*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20+2.1367694805194807e-03*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-9.6793831168831166e-04*Jinv21*w59-1.2820616883116883e-02*w25*Jinv12-5.4788961038961036e-04*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-6.9034090909090911e-03*w16*Jinv11-6.9034090909090911e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+2.6298701298701297e-03*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-6.5746753246753243e-04*w10*Jinv12-1.2820616883116883e-02*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35-9.6793831168831166e-04*w58*Jinv11-9.6793831168831166e-04*w57*Jinv01-9.5332792207792200e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+4.9310064935064934e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41-2.9586038961038961e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-6.5746753246753243e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+1.8628246753246754e-03*w13*Jinv11-6.9034090909090911e-03*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+5.4788961038961042e-05*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39-5.4788961038961036e-04*Jinv22*w2+2.7942370129870131e-03*Jinv00*w18+1.7532467532467534e-03*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-9.5332792207792200e-03*Jinv01*w24-5.4788961038961036e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.7942370129870131e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34-2.9586038961038961e-03*Jinv22*w44-6.0267857142857139e-04*w4*Jinv10+2.6298701298701297e-03*Jinv12*w13+2.7942370129870131e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+2.7942370129870131e-03*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-8.5470779220779228e-03*w26*Jinv20-1.6436688311688311e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35-1.2820616883116883e-02*Jinv22*w26-3.1777597402597403e-03*w46*Jinv11+4.9310064935064932e-04*Jinv20*w14+2.6298701298701297e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+1.0957792207792208e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33+2.6298701298701297e-03*Jinv02*w12+2.7942370129870131e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-6.5746753246753243e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+1.7532467532467534e-03*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8+5.4788961038961042e-05*Jinv22*w50-2.0819805194805196e-03*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-2.0819805194805196e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-1.6436688311688311e-04*w9*Jinv00+1.7532467532467534e-03*w37*Jinv10+2.5750811688311690e-03*Jinv21*w56-8.5470779220779228e-03*w24*Jinv00+1.0957792207792207e-03*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.9310064935064934e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22+8.2183441558441553e-04*w49*Jinv11+1.8628246753246754e-03*Jinv21*w14-1.6436688311688311e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-5.4788961038961036e-04*w58*Jinv12+4.9310064935064934e-03*Jinv02*w18+1.8628246753246754e-03*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-2.0819805194805196e-03*Jinv21*w23+2.5750811688311690e-03*Jinv01*w54+2.5750811688311690e-03*w55*Jinv11-8.5470779220779228e-03*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+2.7942370129870131e-03*w20*Jinv20+1.0957792207792207e-03*Jinv22*w29+2.6298701298701297e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+2.1367694805194807e-03*Jinv02*w36-2.9586038961038961e-03*Jinv12*w43-6.0267857142857139e-04*w5*Jinv20+1.6436688311688311e-04*Jinv02*w51+2.6298701298701297e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28-9.5332792207792200e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12+8.2183441558441553e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+8.2183441558441553e-04*w48*Jinv01-5.4788961038961036e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[25][17]=(0.0)*D;
  A[25][18]=(0.0)*D;
  A[25][19]=(8.2183441558441553e-05*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-1.2327516233766234e-03*Jinv20*w8+8.2183441558441553e-05*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+8.2183441558441553e-05*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-1.5523538961038962e-04*w58*Jinv10+2.4655032467532466e-04*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-1.5523538961038962e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.0957792207792207e-03*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-4.7483766233766234e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+2.4655032467532466e-04*w31*Jinv10-2.4655032467532466e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-1.3149350649350649e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+1.1505681818181819e-03*Jinv01*w24-1.3149350649350649e-03*w34*Jinv11-2.6298701298701297e-03*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+4.3831168831168834e-04*w51*Jinv00+4.3831168831168834e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-8.2183441558441553e-04*w41*Jinv20-1.3149350649350649e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-2.6298701298701297e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*Jinv20*w56+3.7804383116883116e-03*w26*Jinv20-1.4610389610389610e-04*w11*Jinv20-1.3149350649350649e-03*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+2.4655032467532466e-04*w49*Jinv10-4.7483766233766234e-04*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-8.2183441558441553e-04*w39*Jinv00-2.6298701298701297e-03*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+4.3831168831168834e-04*Jinv20*w53-2.4655032467532466e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+4.3831168831168834e-04*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11-1.4610389610389610e-04*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10+5.2049512987012989e-04*Jinv21*w56+3.7804383116883116e-03*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-1.4610389610389610e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+5.2049512987012989e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-1.5523538961038962e-04*w57*Jinv00-1.5523538961038962e-04*Jinv20*w59-1.5523538961038962e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+5.2049512987012989e-04*Jinv01*w54+5.2049512987012989e-04*w55*Jinv11+3.7804383116883116e-03*w25*Jinv10-1.5523538961038962e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-8.2183441558441553e-04*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+4.3831168831168834e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+1.1505681818181819e-03*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50-4.7483766233766234e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[25][20]=(0.0)*D;
  A[25][21]=(0.0)*D;
  A[25][22]=(2.3833198051948050e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-2.1367694805194807e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+2.3833198051948050e-03*w13*Jinv10-2.1367694805194807e-03*Jinv02*w39-2.6846590909090908e-03*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3-3.6982548701298703e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54-3.7439123376623376e-04*Jinv21*w59-7.3965097402597406e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-3.5612824675324673e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-2.6846590909090908e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17-2.4655032467532467e-03*Jinv01*w45+2.3833198051948050e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-7.8530844155844158e-04*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00-7.3965097402597406e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-3.7439123376623376e-04*w58*Jinv11-3.7439123376623376e-04*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26-3.5612824675324673e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-4.2735389610389614e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20-2.6846590909090908e-03*w43*Jinv11-3.6982548701298703e-03*w45*Jinv00-5.4788961038961036e-04*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.8530844155844158e-04*w11*Jinv22-3.5612824675324673e-04*Jinv11*w1+1.1505681818181819e-03*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-1.1414366883116883e-03*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-3.6982548701298703e-03*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-3.6982548701298703e-03*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+7.1225649350649346e-04*Jinv02*w3-3.6982548701298703e-03*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22-5.4788961038961036e-04*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2+2.6298701298701297e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+8.2183441558441553e-05*Jinv21*w38-3.6982548701298703e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+1.3971185064935066e-03*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+2.3833198051948050e-03*Jinv12*w13+2.6298701298701297e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21+1.3971185064935066e-03*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-2.1367694805194807e-03*Jinv22*w41-7.3965097402597406e-03*w26*Jinv20-7.8530844155844158e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+2.7394480519480518e-04*w31*Jinv11-7.3965097402597406e-03*Jinv22*w26-2.4655032467532467e-03*w46*Jinv11+2.3833198051948050e-03*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21+2.3833198051948050e-03*Jinv02*w54+7.1225649350649346e-04*w4*Jinv12-3.6982548701298703e-03*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-1.1414366883116883e-03*Jinv21*w29-2.1367694805194807e-03*w39*Jinv00+2.3833198051948050e-03*Jinv02*w12+1.3971185064935066e-03*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-7.8530844155844158e-04*Jinv02*w9+1.6436688311688311e-04*w7*Jinv10-2.4655032467532467e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38-3.6982548701298703e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-3.6982548701298703e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8-3.6982548701298703e-03*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-7.8530844155844158e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56-7.3965097402597406e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12+7.1225649350649346e-04*Jinv22*w5-3.6982548701298703e-03*w47*Jinv22+1.1505681818181819e-03*Jinv21*w14-7.8530844155844158e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18+1.1505681818181819e-03*Jinv01*w12-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11-7.3965097402597406e-03*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-4.2735389610389614e-03*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9-4.2735389610389614e-03*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+2.6298701298701297e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.3833198051948050e-03*Jinv22*w14-5.4788961038961036e-04*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+2.3833198051948050e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00-3.6982548701298703e-03*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12-1.1414366883116883e-03*Jinv01*w27-7.4878246753246752e-04*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[25][23]=(0.0)*D;
  A[25][24]=(0.0)*D;
  A[25][25]=(-2.3833198051948050e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8-2.3833198051948050e-03*w13*Jinv10+4.7666396103896100e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-1.0683847402597404e-03*Jinv01*w3+3.6982548701298703e-03*Jinv20*w23-2.3833198051948050e-03*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w43*Jinv10+4.7666396103896100e-03*w42*Jinv01+6.5746753246753243e-04*w11*Jinv21+4.7666396103896100e-03*w16*Jinv11+4.7666396103896100e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01+7.3965097402597406e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+4.2735389610389614e-03*w16*Jinv10+4.7666396103896100e-03*w43*Jinv11+3.6982548701298703e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-2.0271915584415584e-03*w13*Jinv11+4.7666396103896100e-03*w15*Jinv01-9.1314935064935068e-05*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+3.6982548701298703e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10+3.6982548701298703e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.6298701298701297e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+5.7528409090909093e-04*Jinv21*w38+7.3965097402597406e-03*Jinv01*w24+1.6436688311688311e-03*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11-1.0683847402597404e-03*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-1.0683847402597404e-03*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-2.6298701298701297e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00-2.3833198051948050e-03*Jinv20*w56+7.3965097402597406e-03*w26*Jinv20+7.8530844155844158e-04*w11*Jinv20+1.6436688311688311e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.4655032467532467e-03*w46*Jinv11-2.3833198051948050e-03*Jinv20*w14-1.0683847402597404e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10-9.1314935064935068e-05*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.3149350649350649e-03*Jinv01*w18+5.7528409090909093e-04*w37*Jinv11+5.7528409090909093e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10+2.4655032467532467e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38-1.0683847402597404e-03*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53+3.6982548701298703e-03*w22*Jinv10+2.4655032467532467e-03*w22*Jinv11+2.4655032467532467e-03*w21*Jinv01-1.0683847402597404e-03*Jinv01*w51+5.7528409090909093e-04*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+5.7528409090909093e-04*w7*Jinv11+7.8530844155844158e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-2.0271915584415584e-03*Jinv21*w56+7.3965097402597406e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-2.0271915584415584e-03*Jinv21*w14+7.8530844155844158e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-2.0271915584415584e-03*Jinv01*w12+5.7528409090909093e-04*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+2.4655032467532467e-03*Jinv21*w23-2.0271915584415584e-03*Jinv01*w54-2.0271915584415584e-03*w55*Jinv11+7.3965097402597406e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+4.2735389610389614e-03*w15*Jinv00+6.5746753246753243e-04*Jinv01*w9+4.2735389610389614e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.6298701298701297e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+7.3965097402597406e-03*w25*Jinv11-2.3833198051948050e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00+3.6982548701298703e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-9.1314935064935068e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+6.5746753246753243e-04*w10*Jinv11)*D;
  A[25][26]=(0.0)*D;
  A[25][27]=(0.0)*D;
  A[25][28]=(6.5746753246753243e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3+1.5726461038961039e-04*Jinv21*w59+1.5726461038961039e-04*Jinv21*w2+6.5746753246753243e-04*w42*Jinv01+2.0495129870129870e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.1870941558441558e-03*Jinv01*w45+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+2.3741883116883116e-03*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+6.5746753246753243e-04*w43*Jinv11+3.2873376623376621e-04*Jinv21*w41+1.5726461038961039e-04*Jinv11*w1-6.7573051948051951e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+3.2873376623376621e-04*Jinv01*w39-2.6481331168831169e-04*Jinv21*w38+2.3741883116883116e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-1.0227272727272728e-03*w19*Jinv11-1.0957792207792208e-04*Jinv21*w53-1.0957792207792208e-04*w4*Jinv11-5.4788961038961042e-05*w32*Jinv21-1.0227272727272728e-03*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11+1.1870941558441558e-03*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-1.0227272727272728e-03*Jinv01*w18-2.6481331168831169e-04*w37*Jinv11-2.6481331168831169e-04*w6*Jinv01+1.1870941558441558e-03*w47*Jinv21-1.0957792207792208e-04*w52*Jinv11+1.1870941558441558e-03*w22*Jinv11+1.1870941558441558e-03*w21*Jinv01-1.0957792207792208e-04*Jinv01*w51-2.6481331168831169e-04*w36*Jinv01-2.6481331168831169e-04*w7*Jinv11-6.7573051948051951e-04*Jinv21*w56-5.4788961038961042e-05*w49*Jinv11-6.7573051948051951e-04*Jinv21*w14-6.7573051948051951e-04*Jinv01*w12-2.6481331168831169e-04*Jinv21*w8+1.1870941558441558e-03*Jinv21*w23-6.7573051948051951e-04*Jinv01*w54-6.7573051948051951e-04*w55*Jinv11+2.0495129870129870e-04*Jinv01*w9+3.2873376623376621e-04*w40*Jinv11-5.4788961038961042e-05*Jinv01*w30+2.3741883116883116e-03*w25*Jinv11-5.4788961038961042e-05*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+2.0495129870129870e-04*w10*Jinv11)*D;
  A[25][29]=(0.0)*D;
  A[25][30]=(0.0)*D;
  A[25][31]=(8.2183441558441553e-05*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12-4.6570616883116886e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+2.7394480519480521e-05*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-1.9176136363636362e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-5.4788961038961042e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02-2.4655032467532466e-04*w48*Jinv00+8.2183441558441553e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-6.3920454545454550e-05*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+8.7662337662337668e-04*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-3.5612824675324673e-04*Jinv02*w3-1.9176136363636362e-04*w21*Jinv00+7.1225649350649346e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+2.7394480519480521e-05*Jinv22*w59+8.7662337662337668e-04*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+8.2183441558441553e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+7.1225649350649346e-04*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+8.7662337662337668e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20-4.6570616883116886e-04*w52*Jinv12+8.7662337662337668e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+7.1225649350649346e-04*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+8.7662337662337668e-04*Jinv01*w33+7.1225649350649346e-04*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-2.4655032467532466e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.5612824675324673e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8-1.9176136363636362e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-1.9176136363636362e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.9176136363636362e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+2.7394480519480521e-05*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02-3.5612824675324673e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.7394480519480521e-05*w58*Jinv12+8.2183441558441553e-05*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+8.2183441558441553e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+8.7662337662337668e-04*w35*Jinv20-1.9176136363636362e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+7.1225649350649346e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-4.6570616883116886e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+7.1225649350649346e-04*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[25][32]=(0.0)*D;
  A[25][33]=(0.0)*D;
  A[25][34]=(-3.2873376623376621e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+1.3149350649350649e-03*Jinv22*w53+1.2601461038961039e-03*Jinv20*w8-3.4699675324675324e-04*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10+3.9448051948051946e-03*Jinv21*w44-1.2053571428571428e-03*w3*Jinv00-8.2183441558441553e-04*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3+1.3149350649350649e-03*Jinv22*w38+3.8352272727272725e-04*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59+3.4699675324675324e-04*w0*Jinv02+4.2004870129870131e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+3.9448051948051946e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-2.1915584415584414e-03*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01-3.2873376623376621e-03*Jinv21*w26+4.2004870129870131e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+3.9448051948051946e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+4.2004870129870131e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+9.8620129870129864e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.6436688311688311e-03*Jinv00*w33+1.0957792207792207e-03*w22*Jinv12-8.2183441558441553e-04*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-8.2183441558441553e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+3.4699675324675324e-04*Jinv22*w2+1.7532467532467534e-03*Jinv00*w18+2.5750811688311690e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45-3.2873376623376621e-03*Jinv01*w24-3.4699675324675324e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+1.9724025974025973e-03*Jinv22*w44-1.2053571428571428e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13+1.7532467532467534e-03*w19*Jinv10+1.6436688311688311e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.2601461038961039e-03*w6*Jinv00+1.3149350649350649e-03*w37*Jinv12+3.8352272727272725e-04*Jinv20*w56-2.1915584415584414e-03*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-2.1915584415584414e-03*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+9.8620129870129864e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12+2.1367694805194807e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+1.3149350649350649e-03*w6*Jinv01+1.2601461038961039e-03*w7*Jinv10-2.1915584415584414e-03*w47*Jinv21+2.5750811688311690e-03*Jinv20*w38+1.0957792207792207e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-8.2183441558441553e-04*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8+8.2183441558441553e-04*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-1.0957792207792207e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-8.2183441558441553e-04*Jinv02*w30+8.2183441558441553e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.3149350649350649e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.5750811688311690e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56-2.1915584415584414e-03*w24*Jinv00-1.3149350649350649e-03*w6*Jinv02-1.3149350649350649e-03*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44-3.4699675324675324e-04*w58*Jinv12+5.4788961038961042e-05*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+3.8352272727272725e-04*w0*Jinv00+1.6436688311688311e-03*w35*Jinv20-1.0957792207792207e-03*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11-2.1915584415584414e-03*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+8.2183441558441553e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17-8.2183441558441553e-04*w30*Jinv00+1.7532467532467534e-03*w20*Jinv20-7.1225649350649346e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+1.3149350649350649e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.2053571428571428e-03*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-3.2873376623376621e-03*w25*Jinv11+3.8352272727272725e-04*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+9.8620129870129864e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+3.4699675324675324e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[25][35]=(0.0)*D;
  A[25][36]=(0.0)*D;
  A[25][37]=(2.4655032467532466e-04*Jinv00*w12-4.8396915584415583e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53-1.0135957792207792e-03*Jinv20*w8-4.2918019480519480e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+8.4922889610389610e-04*w3*Jinv00+4.6570616883116886e-04*w32*Jinv20+1.9176136363636362e-04*Jinv20*w23-2.3833198051948050e-03*Jinv22*w38+1.3697240259740259e-04*Jinv00*w54+1.1505681818181819e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-2.7394480519480520e-03*w43*Jinv10+5.2049512987012989e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00+1.1505681818181819e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.1960227272727273e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20+3.8352272727272725e-04*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02-4.8396915584415583e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+2.4655032467532466e-04*Jinv20*w50-4.8396915584415583e-04*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-2.5202922077922079e-03*Jinv00*w33-1.6436688311688311e-04*w22*Jinv12+4.6570616883116886e-04*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+1.9176136363636362e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-2.0271915584415584e-03*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45-4.2918019480519480e-04*Jinv22*w59+4.3831168831168834e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+8.4922889610389610e-04*w4*Jinv10-7.1225649350649346e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10-2.5202922077922079e-03*w34*Jinv10-1.0135957792207792e-03*w6*Jinv00-2.3833198051948050e-03*w37*Jinv12+1.3697240259740259e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+1.4793019480519481e-03*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+1.1505681818181819e-03*Jinv22*w26+2.4655032467532466e-04*Jinv20*w14+5.2049512987012989e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10-7.1225649350649346e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.0135957792207792e-03*w7*Jinv10-2.0271915584415584e-03*Jinv20*w38-1.6436688311688311e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+1.9176136363636362e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+4.3831168831168834e-04*w52*Jinv10+1.1870941558441559e-04*w9*Jinv00-2.0271915584415584e-03*w37*Jinv10+1.4793019480519481e-03*w24*Jinv00-4.7483766233766234e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22+1.1870941558441559e-04*w10*Jinv10-2.7394480519480520e-03*Jinv20*w44-4.2918019480519480e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-3.1960227272727273e-04*w0*Jinv00-2.5202922077922079e-03*w35*Jinv20+1.4793019480519481e-03*w25*Jinv10-3.1960227272727273e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+4.6570616883116886e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-4.7483766233766234e-04*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-2.3833198051948050e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.4922889610389610e-04*w5*Jinv20+1.3971185064935066e-03*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56-4.7483766233766234e-04*Jinv12*w28+1.3697240259740259e-04*w55*Jinv10-2.7394480519480520e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[25][38]=(0.0)*D;
  A[25][39]=(0.0)*D;
  A[25][40]=(1.3149350649350649e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+7.1225649350649346e-04*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10-3.2873376623376621e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-2.1367694805194807e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59+2.1915584415584417e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+8.2183441558441553e-04*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+3.8352272727272725e-04*w48*Jinv00-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10-3.2873376623376621e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-3.1047077922077924e-04*Jinv11*w1+9.3141233766233771e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.7532467532467534e-03*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00-9.8620129870129864e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-9.8620129870129864e-04*w41*Jinv20-8.2183441558441553e-04*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+1.7532467532467534e-03*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-3.1777597402597403e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14+8.2183441558441553e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-8.2183441558441553e-04*Jinv02*w12+1.7532467532467534e-03*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11+3.2873376623376621e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50-1.0409902597402598e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-1.0409902597402598e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10+1.7532467532467534e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00-3.8352272727272725e-04*w6*Jinv02-7.1225649350649346e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11+9.3141233766233771e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+9.3141233766233771e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59-2.7394480519480518e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-1.0409902597402598e-03*Jinv21*w23+1.7532467532467534e-03*Jinv01*w54+1.7532467532467534e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-8.2183441558441553e-04*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+7.1225649350649346e-04*Jinv02*w51+8.2183441558441553e-04*Jinv22*w56-3.1777597402597403e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[25][41]=(0.0)*D;
  A[25][42]=(0.0)*D;
  A[25][43]=(-1.3149350649350649e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8+5.4788961038961036e-04*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20+1.1505681818181819e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59+1.2820616883116883e-02*w25*Jinv12+5.4788961038961036e-04*w0*Jinv02-4.2004870129870131e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+2.1915584415584414e-03*Jinv01*w45-2.6298701298701297e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+2.9586038961038961e-03*w15*Jinv02+6.5746753246753243e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00+1.2820616883116883e-02*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+3.2873376623376621e-03*Jinv21*w26-4.2004870129870131e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-4.9310064935064934e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27+6.5746753246753243e-04*w11*Jinv22-4.2004870129870131e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-9.8620129870129864e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10+2.9586038961038961e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+4.2735389610389614e-03*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+5.4788961038961036e-04*Jinv22*w2-2.1367694805194807e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+3.2873376623376621e-03*Jinv01*w24+5.4788961038961036e-04*Jinv22*w59-4.9310064935064934e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+1.1505681818181819e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.6298701298701297e-03*Jinv12*w13-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12-4.9310064935064934e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11+1.2820616883116883e-02*Jinv22*w26+2.1915584415584414e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14+1.1505681818181819e-03*w5*Jinv21-2.6298701298701297e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-9.8620129870129864e-04*Jinv21*w29-4.9310064935064934e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-2.6298701298701297e-03*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+6.5746753246753243e-04*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+2.1915584415584414e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38+4.2735389610389614e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-7.6704545454545450e-04*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.9310064935064934e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-5.4788961038961042e-05*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+5.4788961038961036e-04*w58*Jinv12-4.9310064935064934e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29-2.6298701298701297e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+7.6704545454545450e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-2.6298701298701297e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28+3.2873376623376621e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20+2.9586038961038961e-03*w16*Jinv12-5.4788961038961042e-05*Jinv21*w50-9.8620129870129864e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+5.4788961038961036e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[25][44]=(0.0)*D;
  A[25][45]=(0.0)*D;
  A[25][46]=(2.1367694805194807e-03*w40*Jinv12-7.1225649350649346e-04*Jinv22*w53+7.4878246753246752e-04*w57*Jinv02+2.1367694805194807e-03*Jinv02*w39+3.1777597402597403e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-1.6436688311688311e-04*Jinv22*w38+3.9265422077922079e-04*Jinv21*w59+7.3965097402597406e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+3.7439123376623376e-04*Jinv21*w2+3.1777597402597403e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45-2.3833198051948050e-03*Jinv12*w55+4.2735389610389614e-03*w15*Jinv02+7.8530844155844158e-04*w10*Jinv12+7.3965097402597406e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.7439123376623376e-04*w0*Jinv01-2.6298701298701297e-03*Jinv22*w20+3.1777597402597403e-03*w43*Jinv11+1.5888798701298701e-03*Jinv21*w41+4.2735389610389614e-03*w42*Jinv02+7.8530844155844158e-04*w11*Jinv22+3.7439123376623376e-04*Jinv11*w1-1.0135957792207792e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11+4.2735389610389614e-03*Jinv22*w17+3.6982548701298703e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-3.5612824675324673e-04*w32*Jinv22+1.5888798701298701e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+3.6982548701298703e-03*Jinv02*w45+2.4655032467532467e-03*Jinv01*w24+7.4878246753246752e-04*Jinv22*w59+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.3833198051948050e-03*Jinv12*w13-3.5612824675324673e-04*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-1.6436688311688311e-04*w37*Jinv12+2.1367694805194807e-03*Jinv22*w41-7.1225649350649346e-04*w52*Jinv12+1.0957792207792207e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+7.3965097402597406e-03*Jinv22*w26+2.4655032467532467e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-2.3833198051948050e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+3.6982548701298703e-03*w21*Jinv02-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33-2.3833198051948050e-03*Jinv02*w12-1.2327516233766234e-03*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+7.8530844155844158e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+2.4655032467532467e-03*w47*Jinv21+3.6982548701298703e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12+3.6982548701298703e-03*w46*Jinv12-1.6436688311688311e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11-1.6436688311688311e-04*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-8.2183441558441553e-05*w7*Jinv11-1.2327516233766234e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+3.6982548701298703e-03*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11-1.0135957792207792e-03*Jinv21*w14+7.4878246753246752e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-1.0135957792207792e-03*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+1.2327516233766234e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+3.9265422077922079e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.3833198051948050e-03*Jinv22*w14+1.5888798701298701e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-3.5612824675324673e-04*Jinv01*w30-7.1225649350649346e-04*Jinv02*w51-2.3833198051948050e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+2.4655032467532467e-03*w25*Jinv11+4.2735389610389614e-03*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+3.9265422077922079e-04*w10*Jinv11)*D;
  A[25][47]=(0.0)*D;
  A[25][48]=(0.0)*D;
  A[25][49]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10-3.2873376623376621e-04*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-3.2873376623376621e-04*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480521e-05*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+3.5612824675324673e-04*w48*Jinv00-8.2183441558441553e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+7.6704545454545450e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10+4.6570616883116886e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-2.7394480519480521e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-2.7394480519480521e-05*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-8.2183441558441553e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-3.2873376623376621e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+1.6436688311688311e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54+4.6570616883116886e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-2.7394480519480521e-05*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+3.5612824675324673e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-2.7394480519480521e-05*Jinv02*w30-5.2049512987012989e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02+4.6570616883116886e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-2.7394480519480518e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-8.2183441558441553e-04*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+1.3697240259740259e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20+2.4655032467532466e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56+7.1225649350649346e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-2.7394480519480521e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[25][50]=(0.0)*D;
  A[25][51]=(0.0)*D;
  A[25][52]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-1.0683847402597404e-03*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+8.2183441558441553e-05*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12-2.1915584415584417e-04*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+1.4793019480519481e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-3.5612824675324673e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+2.4655032467532466e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+4.6570616883116886e-04*w32*Jinv22-2.1915584415584417e-04*Jinv22*w2+5.2049512987012989e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+3.8352272727272725e-04*Jinv02*w45+1.2784090909090910e-04*Jinv22*w59-7.1225649350649346e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-3.8352272727272725e-04*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.0683847402597404e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-3.8352272727272725e-04*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+5.2049512987012989e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+4.6570616883116886e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-7.1225649350649346e-04*Jinv20*w53+1.3971185064935066e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+1.3971185064935066e-03*Jinv12*w7+4.6570616883116886e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-7.1225649350649346e-04*w52*Jinv10-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00+1.8262987012987013e-05*Jinv02*w27+1.3971185064935066e-03*w6*Jinv02+1.4793019480519481e-03*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-1.3697240259740259e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12+1.4793019480519481e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20+1.8262987012987013e-05*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14+8.2183441558441553e-05*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-1.0683847402597404e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+1.8262987012987013e-05*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12-2.1915584415584417e-04*Jinv12*w1)*D;
  A[25][53]=(0.0)*D;
  A[25][54]=(0.0)*D;
  A[25][55]=(-8.2183441558441553e-04*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-2.1002435064935065e-04*w57*Jinv02-8.2183441558441553e-04*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+8.2183441558441553e-05*Jinv01*w3+1.3697240259740259e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59-4.7666396103896100e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+5.7528409090909093e-04*Jinv12*w55-2.6298701298701297e-03*w15*Jinv02-3.1047077922077924e-04*w10*Jinv12-4.7666396103896100e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-7.1225649350649346e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01+1.4793019480519481e-03*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-3.8352272727272725e-04*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-3.1047077922077924e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11-2.6298701298701297e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+2.4655032467532466e-04*w32*Jinv22-3.8352272727272725e-04*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+5.2049512987012989e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-7.1225649350649346e-04*Jinv01*w24-2.1002435064935065e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+5.2049512987012989e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+2.7394480519480518e-04*w32*Jinv21+5.2049512987012989e-04*Jinv21*w20+1.3697240259740259e-04*w37*Jinv12-8.2183441558441553e-04*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-4.7666396103896100e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+8.2183441558441553e-05*w5*Jinv21+5.7528409090909093e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+5.2049512987012989e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11-3.1047077922077924e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-1.6436688311688311e-04*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+2.4655032467532466e-04*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8-8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+2.4655032467532466e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+5.2049512987012989e-04*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+1.4793019480519481e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-2.1002435064935065e-04*w58*Jinv12+1.4793019480519481e-03*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-8.2183441558441553e-05*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-3.8352272727272725e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51+5.7528409090909093e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-7.1225649350649346e-04*w25*Jinv11-2.6298701298701297e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[25][56]=(0.0)*D;
  A[25][57]=(0.0)*D;
  A[25][58]=(2.1915584415584417e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+2.1915584415584417e-04*Jinv02*w39+1.1870941558441559e-04*Jinv22*w38+1.4975649350649350e-03*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-2.6481331168831169e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+1.4975649350649350e-03*Jinv02*w24+3.4699675324675324e-04*Jinv22*w35-5.8441558441558442e-04*Jinv22*w20+4.3831168831168834e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-2.1915584415584417e-04*Jinv02*w3-2.7394480519480521e-05*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.6525974025974026e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+3.4699675324675324e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-4.2918019480519480e-04*Jinv12*w13+1.1870941558441559e-04*w37*Jinv12+2.1915584415584417e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12+1.4975649350649350e-03*Jinv22*w26-2.6481331168831169e-04*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-4.2918019480519480e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9+7.4878246753246752e-04*Jinv22*w23-2.7394480519480521e-05*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-4.2918019480519480e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7-2.7394480519480521e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+6.4935064935064935e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-5.8441558441558442e-04*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-5.8441558441558442e-04*Jinv02*w18+3.4699675324675324e-04*Jinv02*w33-5.4788961038961042e-05*w48*Jinv02+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+1.1870941558441559e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-1.0957792207792208e-04*Jinv02*w51-2.6481331168831169e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[25][59]=(0.0)*D;
  A[26][0]=(0.0)*D;
  A[26][1]=(0.0)*D;
  A[26][2]=(2.6481331168831169e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+2.6481331168831169e-04*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54-1.0247564935064935e-04*Jinv21*w59-1.4975649350649350e-03*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17-7.4878246753246752e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55-1.0247564935064935e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+2.7394480519480521e-05*w48*Jinv00-1.4975649350649350e-03*Jinv02*w24-3.4699675324675324e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01-1.4975649350649350e-03*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10+5.8441558441558442e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-7.4878246753246752e-04*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+2.6481331168831169e-04*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11+2.7394480519480521e-05*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.4878246753246752e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-3.4699675324675324e-04*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+5.8441558441558442e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-7.4878246753246752e-04*Jinv02*w45-1.4975649350649350e-03*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-3.4699675324675324e-04*w34*Jinv11+5.8441558441558442e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+2.1915584415584417e-04*w51*Jinv00-3.4699675324675324e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-2.1915584415584417e-04*w41*Jinv20+2.6481331168831169e-04*Jinv12*w13+5.8441558441558442e-04*w19*Jinv10-3.4699675324675324e-04*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21+5.8441558441558442e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41-1.4975649350649350e-03*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12-3.4699675324675324e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11-1.4975649350649350e-03*Jinv22*w26-7.4878246753246752e-04*w46*Jinv11+2.6481331168831169e-04*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02+2.7394480519480521e-05*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29-3.4699675324675324e-04*Jinv01*w33-2.1915584415584417e-04*w39*Jinv00+2.6481331168831169e-04*Jinv02*w12+5.8441558441558442e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10-7.4878246753246752e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-7.4878246753246752e-04*w46*Jinv12+2.1915584415584417e-04*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+2.7394480519480521e-05*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01+2.1915584415584417e-04*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56-1.4975649350649350e-03*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02+5.8441558441558442e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5-7.4878246753246752e-04*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11+2.6481331168831169e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+5.8441558441558442e-04*Jinv02*w18+2.6481331168831169e-04*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8-1.0247564935064935e-04*w57*Jinv00-1.0247564935064935e-04*Jinv20*w59-3.4699675324675324e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-3.4699675324675324e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11-1.4975649350649350e-03*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-2.1915584415584417e-04*w40*Jinv10+5.8441558441558442e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+2.6481331168831169e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-1.4975649350649350e-03*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-7.4878246753246752e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[26][3]=(0.0)*D;
  A[26][4]=(0.0)*D;
  A[26][5]=(-9.1314935064935065e-06*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-2.4655032467532466e-04*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8+2.1915584415584417e-04*w57*Jinv02+7.1225649350649346e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23-1.3971185064935066e-03*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-1.9176136363636362e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10-1.4793019480519481e-03*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01-1.8262987012987013e-05*w28*Jinv11-1.9176136363636362e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+1.2601461038961039e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+1.0683847402597404e-03*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-9.5880681818181818e-04*Jinv00*w18-1.1231737012987013e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24+2.1915584415584417e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-1.4793019480519481e-03*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+1.0683847402597404e-03*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-9.5880681818181818e-04*w19*Jinv10+1.2601461038961039e-03*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-1.4793019480519481e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00-1.3971185064935066e-03*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41+1.4245129870129869e-03*w26*Jinv20-3.5612824675324673e-04*w11*Jinv20-2.4655032467532466e-04*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+1.0683847402597404e-03*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+1.0683847402597404e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-1.9176136363636362e-04*w49*Jinv10-1.8262987012987013e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-1.4793019480519481e-03*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-1.1231737012987013e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-1.6436688311688311e-04*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10-4.6570616883116886e-04*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-4.6570616883116886e-04*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11-3.5612824675324673e-04*w9*Jinv00-1.1231737012987013e-03*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+1.4245129870129869e-03*w24*Jinv00-1.8262987012987013e-05*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.4793019480519481e-03*w19*Jinv12+1.0683847402597404e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-4.6570616883116886e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-3.5612824675324673e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.1915584415584417e-04*w58*Jinv12-1.4793019480519481e-03*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.2601461038961039e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+1.4245129870129869e-03*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00-4.6570616883116886e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+3.2873376623376621e-04*w40*Jinv10-9.5880681818181818e-04*w20*Jinv20-1.8262987012987013e-05*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-1.3971185064935066e-03*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-2.4655032467532466e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56-1.8262987012987013e-05*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-4.6570616883116886e-04*Jinv21*w50-1.8262987012987013e-05*Jinv01*w27-4.6570616883116886e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[26][6]=(0.0)*D;
  A[26][7]=(0.0)*D;
  A[26][8]=(-3.8352272727272725e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.3971185064935066e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-3.8352272727272725e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-9.5880681818181818e-04*w3*Jinv00-2.7394480519480521e-05*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23+2.3833198051948050e-03*Jinv22*w38-2.7394480519480518e-04*Jinv00*w54+4.2918019480519480e-04*Jinv21*w59-1.1505681818181819e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.0957792207792208e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+1.9176136363636362e-04*w48*Jinv00-1.1505681818181819e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+4.2918019480519480e-04*w58*Jinv11+4.2918019480519480e-04*w57*Jinv01-1.1505681818181819e-03*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-6.0267857142857139e-04*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+4.7483766233766234e-04*w28*Jinv11+1.9176136363636362e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17-1.2053571428571428e-03*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12-2.7394480519480521e-05*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2+1.3971185064935066e-03*Jinv00*w18+1.3697240259740260e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-1.1505681818181819e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11+2.6298701298701297e-03*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-9.5880681818181818e-04*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13+1.3971185064935066e-03*w19*Jinv10-1.2053571428571428e-03*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+2.3833198051948050e-03*w37*Jinv12-2.7394480519480518e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+3.2873376623376621e-04*w26*Jinv20+6.6659902597402602e-04*w11*Jinv20-1.3971185064935066e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-1.1505681818181819e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-3.8352272727272725e-04*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02+1.9176136363636362e-04*w49*Jinv10+4.7483766233766234e-04*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.3833198051948050e-03*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+1.3697240259740260e-03*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.3833198051948050e-03*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+2.3833198051948050e-03*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+2.3833198051948050e-03*w7*Jinv11+6.6659902597402602e-04*w9*Jinv00+1.3697240259740260e-03*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.2873376623376621e-04*w24*Jinv00+4.7483766233766234e-04*Jinv02*w27+2.3833198051948050e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+6.6659902597402602e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12+2.3833198051948050e-03*Jinv21*w8+1.0957792207792208e-04*w57*Jinv00+1.0957792207792208e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-1.2053571428571428e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.2873376623376621e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2-6.0267857142857139e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-6.0267857142857139e-04*Jinv20*w17-2.7394480519480521e-05*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20+4.7483766233766234e-04*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.3833198051948050e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-9.5880681818181818e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-1.3971185064935066e-03*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56+4.7483766233766234e-04*Jinv12*w28-1.1505681818181819e-03*w25*Jinv11-2.7394480519480518e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50+4.7483766233766234e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[26][9]=(0.0)*D;
  A[26][10]=(0.0)*D;
  A[26][11]=(1.1870941558441559e-04*Jinv00*w12+8.3198051948051951e-05*w28*Jinv10-6.7573051948051951e-04*Jinv20*w8+1.1870941558441559e-04*w13*Jinv10+4.9310064935064932e-04*w3*Jinv00-2.7394480519480521e-05*w32*Jinv20+3.6525974025974026e-05*Jinv20*w23+1.1870941558441559e-04*Jinv00*w54-3.2873376623376621e-04*w43*Jinv10-1.1668019480519480e-04*w58*Jinv10-2.7394480519480521e-05*w48*Jinv00-1.1668019480519480e-04*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+3.6525974025974026e-05*w45*Jinv00+8.3198051948051951e-05*Jinv00*w27-2.7394480519480521e-05*Jinv20*w50+8.3198051948051951e-05*Jinv20*w29+3.6525974025974026e-05*w46*Jinv10+2.9220779220779221e-04*Jinv00*w33-2.7394480519480521e-05*w31*Jinv10+3.6525974025974026e-05*w21*Jinv00-1.0227272727272728e-03*Jinv00*w18-6.7573051948051951e-04*w36*Jinv00+4.9310064935064932e-04*w51*Jinv00+4.9310064935064932e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-1.0227272727272728e-03*w19*Jinv10+2.9220779220779221e-04*w34*Jinv10-6.7573051948051951e-04*w6*Jinv00+1.1870941558441559e-04*Jinv20*w56-2.5568181818181820e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20+1.1870941558441559e-04*Jinv20*w14-2.7394480519480521e-05*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-6.7573051948051951e-04*w7*Jinv10-6.7573051948051951e-04*Jinv20*w38+4.9310064935064932e-04*Jinv20*w53+3.6525974025974026e-05*w22*Jinv10+4.9310064935064932e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-6.7573051948051951e-04*w37*Jinv10-2.5568181818181820e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44-1.1668019480519480e-04*w57*Jinv00-1.1668019480519480e-04*Jinv20*w59-1.1668019480519480e-04*w0*Jinv00+2.9220779220779221e-04*w35*Jinv20-2.5568181818181820e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-3.2873376623376621e-04*Jinv20*w17-2.7394480519480521e-05*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-1.0227272727272728e-03*w20*Jinv20+4.9310064935064932e-04*w5*Jinv20+1.1870941558441559e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+3.6525974025974026e-05*w47*Jinv20)*D;
  A[26][12]=(0.0)*D;
  A[26][13]=(0.0)*D;
  A[26][14]=(-5.7528409090909093e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+8.2183441558441553e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8+4.2918019480519480e-04*w57*Jinv02-5.7528409090909093e-04*w13*Jinv10+8.2183441558441553e-04*Jinv02*w39+2.5750811688311690e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20-5.4788961038961036e-04*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54+3.1960227272727273e-04*Jinv21*w59+4.7666396103896100e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.5750811688311690e-03*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+8.7662337662337668e-04*w16*Jinv11+8.7662337662337668e-04*Jinv21*w17+2.0271915584415584e-03*Jinv01*w45-1.3971185064935066e-03*Jinv12*w55+4.2918019480519480e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+3.1047077922077924e-04*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00+4.7666396103896100e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+3.1960227272727273e-04*w58*Jinv11+3.1960227272727273e-04*w57*Jinv01+4.0543831168831168e-03*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+1.6436688311688311e-03*w16*Jinv10-1.4793019480519481e-03*Jinv22*w20+2.5750811688311690e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00+4.3831168831168834e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+3.1047077922077924e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-7.3965097402597403e-04*w13*Jinv11+8.7662337662337668e-04*w15*Jinv01+5.5702110389610384e-04*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+4.3831168831168834e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2-1.4793019480519481e-03*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38+2.3833198051948050e-03*Jinv02*w45+4.0543831168831168e-03*Jinv01*w24+4.2918019480519480e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-9.5880681818181818e-04*w19*Jinv11-4.6570616883116886e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10-5.4788961038961036e-04*w4*Jinv11+8.2183441558441553e-04*w41*Jinv20-5.7528409090909093e-04*Jinv12*w13-1.4793019480519481e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21-9.5880681818181818e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+8.2183441558441553e-04*Jinv22*w41+4.7666396103896100e-03*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+4.7666396103896100e-03*Jinv22*w26+2.0271915584415584e-03*w46*Jinv11-5.7528409090909093e-04*Jinv20*w14-5.4788961038961036e-04*w5*Jinv21-1.3971185064935066e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+5.5702110389610384e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+8.2183441558441553e-04*w39*Jinv00-5.7528409090909093e-04*Jinv02*w12-9.5880681818181818e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+3.1047077922077924e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10+2.0271915584415584e-03*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23-4.6570616883116886e-04*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12+2.3833198051948050e-03*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-1.3697240259740259e-04*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-4.6570616883116886e-04*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10-1.1231737012987013e-03*Jinv21*w56+4.7666396103896100e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02-1.4793019480519481e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5+2.3833198051948050e-03*w47*Jinv22+2.7394480519480521e-05*w49*Jinv11-7.3965097402597403e-04*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+4.2918019480519480e-04*w58*Jinv12-1.4793019480519481e-03*Jinv02*w18-7.3965097402597403e-04*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+4.2918019480519480e-04*w57*Jinv00+4.2918019480519480e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-1.1231737012987013e-03*Jinv01*w54-1.1231737012987013e-03*w55*Jinv11+4.7666396103896100e-03*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+1.6436688311688311e-03*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+1.6436688311688311e-03*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00+8.2183441558441553e-04*w40*Jinv10-1.4793019480519481e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-5.7528409090909093e-04*Jinv22*w14+4.3831168831168834e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-1.3971185064935066e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+4.0543831168831168e-03*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12+2.7394480519480521e-05*Jinv21*w50+5.5702110389610384e-04*Jinv01*w27+2.7394480519480521e-05*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[26][15]=(0.0)*D;
  A[26][16]=(0.0)*D;
  A[26][17]=(4.9310064935064932e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-5.4788961038961036e-04*w57*Jinv02+4.9310064935064932e-04*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44-6.0267857142857139e-04*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20+2.1367694805194807e-03*Jinv22*w38+1.3149350649350649e-03*Jinv00*w54-9.6793831168831166e-04*Jinv21*w59-1.2820616883116883e-02*w25*Jinv12-5.4788961038961036e-04*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-6.9034090909090911e-03*w16*Jinv11-6.9034090909090911e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+2.6298701298701297e-03*Jinv12*w55-2.7394480519480518e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-6.5746753246753243e-04*w10*Jinv12-1.2820616883116883e-02*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35-9.6793831168831166e-04*w58*Jinv11-9.6793831168831166e-04*w57*Jinv01-9.5332792207792200e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+4.9310064935064934e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41-2.9586038961038961e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-6.5746753246753243e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+1.8628246753246754e-03*w13*Jinv11-6.9034090909090911e-03*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+5.4788961038961042e-05*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39-5.4788961038961036e-04*Jinv22*w2+2.7942370129870131e-03*Jinv00*w18+1.7532467532467534e-03*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-9.5332792207792200e-03*Jinv01*w24-5.4788961038961036e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.7942370129870131e-03*w19*Jinv11+1.3149350649350649e-03*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34-2.9586038961038961e-03*Jinv22*w44-6.0267857142857139e-04*w4*Jinv10+2.6298701298701297e-03*Jinv12*w13+2.7942370129870131e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+2.7942370129870131e-03*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12+1.3149350649350649e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-8.5470779220779228e-03*w26*Jinv20-1.6436688311688311e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35-1.2820616883116883e-02*Jinv22*w26-3.1777597402597403e-03*w46*Jinv11+4.9310064935064932e-04*Jinv20*w14+2.6298701298701297e-03*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+1.0957792207792208e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33+2.6298701298701297e-03*Jinv02*w12+2.7942370129870131e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-6.5746753246753243e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+1.7532467532467534e-03*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23+1.3149350649350649e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8+5.4788961038961042e-05*Jinv22*w50-2.0819805194805196e-03*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-2.0819805194805196e-03*w21*Jinv01+1.3149350649350649e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11-1.6436688311688311e-04*w9*Jinv00+1.7532467532467534e-03*w37*Jinv10+2.5750811688311690e-03*Jinv21*w56-8.5470779220779228e-03*w24*Jinv00+1.0957792207792207e-03*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.9310064935064934e-03*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22+8.2183441558441553e-04*w49*Jinv11+1.8628246753246754e-03*Jinv21*w14-1.6436688311688311e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-5.4788961038961036e-04*w58*Jinv12+4.9310064935064934e-03*Jinv02*w18+1.8628246753246754e-03*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-2.7394480519480518e-04*w57*Jinv00-2.7394480519480518e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-2.0819805194805196e-03*Jinv21*w23+2.5750811688311690e-03*Jinv01*w54+2.5750811688311690e-03*w55*Jinv11-8.5470779220779228e-03*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+2.7942370129870131e-03*w20*Jinv20+1.0957792207792207e-03*Jinv22*w29+2.6298701298701297e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+2.1367694805194807e-03*Jinv02*w36-2.9586038961038961e-03*Jinv12*w43-6.0267857142857139e-04*w5*Jinv20+1.6436688311688311e-04*Jinv02*w51+2.6298701298701297e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28-9.5332792207792200e-03*w25*Jinv11+1.3149350649350649e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12+8.2183441558441553e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+8.2183441558441553e-04*w48*Jinv01-5.4788961038961036e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[26][18]=(0.0)*D;
  A[26][19]=(0.0)*D;
  A[26][20]=(8.2183441558441553e-05*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-1.2327516233766234e-03*Jinv20*w8+8.2183441558441553e-05*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+4.3831168831168834e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+8.2183441558441553e-05*Jinv00*w54-4.2918019480519480e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-1.5523538961038962e-04*w58*Jinv10+2.4655032467532466e-04*w48*Jinv00-4.2918019480519480e-04*w58*Jinv11-4.2918019480519480e-04*w57*Jinv01+1.1505681818181819e-03*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-1.5523538961038962e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.0957792207792207e-03*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-4.7483766233766234e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+2.4655032467532466e-04*w31*Jinv10-2.4655032467532466e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-1.3149350649350649e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-2.3833198051948050e-03*Jinv21*w38+1.1505681818181819e-03*Jinv01*w24-1.3149350649350649e-03*w34*Jinv11-2.6298701298701297e-03*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53+4.3831168831168834e-04*w51*Jinv00+4.3831168831168834e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-8.2183441558441553e-04*w41*Jinv20-1.3149350649350649e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-2.6298701298701297e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00+8.2183441558441553e-05*Jinv20*w56+3.7804383116883116e-03*w26*Jinv20-1.4610389610389610e-04*w11*Jinv20-1.3149350649350649e-03*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+2.4655032467532466e-04*w49*Jinv10-4.7483766233766234e-04*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-8.2183441558441553e-04*w39*Jinv00-2.6298701298701297e-03*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+1.3971185064935066e-03*w52*Jinv11+4.3831168831168834e-04*Jinv20*w53-2.4655032467532466e-04*w22*Jinv10-1.6436688311688311e-04*w22*Jinv11-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-2.3833198051948050e-03*w36*Jinv01+4.3831168831168834e-04*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11-1.4610389610389610e-04*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10+5.2049512987012989e-04*Jinv21*w56+3.7804383116883116e-03*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-1.4610389610389610e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+5.2049512987012989e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-1.5523538961038962e-04*w57*Jinv00-1.5523538961038962e-04*Jinv20*w59-1.5523538961038962e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20-1.6436688311688311e-04*Jinv21*w23+5.2049512987012989e-04*Jinv01*w54+5.2049512987012989e-04*w55*Jinv11+3.7804383116883116e-03*w25*Jinv10-1.5523538961038962e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-8.2183441558441553e-04*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+4.3831168831168834e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+1.1505681818181819e-03*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50-4.7483766233766234e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[26][21]=(0.0)*D;
  A[26][22]=(0.0)*D;
  A[26][23]=(2.3833198051948050e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-2.1367694805194807e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8-7.4878246753246752e-04*w57*Jinv02+2.3833198051948050e-03*w13*Jinv10-2.1367694805194807e-03*Jinv02*w39-2.6846590909090908e-03*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3-3.6982548701298703e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54-3.7439123376623376e-04*Jinv21*w59-7.3965097402597406e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-3.5612824675324673e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-2.6846590909090908e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17-2.4655032467532467e-03*Jinv01*w45+2.3833198051948050e-03*Jinv12*w55-7.4878246753246752e-04*w58*Jinv10-4.2735389610389614e-03*w15*Jinv02-7.8530844155844158e-04*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00-7.3965097402597406e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-3.7439123376623376e-04*w58*Jinv11-3.7439123376623376e-04*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26-3.5612824675324673e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-4.2735389610389614e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20-2.6846590909090908e-03*w43*Jinv11-3.6982548701298703e-03*w45*Jinv00-5.4788961038961036e-04*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.8530844155844158e-04*w11*Jinv22-3.5612824675324673e-04*Jinv11*w1+1.1505681818181819e-03*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-1.1414366883116883e-03*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-3.6982548701298703e-03*w46*Jinv10-4.2735389610389614e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-3.6982548701298703e-03*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+7.1225649350649346e-04*Jinv02*w3-3.6982548701298703e-03*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22-5.4788961038961036e-04*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2+2.6298701298701297e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+8.2183441558441553e-05*Jinv21*w38-3.6982548701298703e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24-7.4878246753246752e-04*Jinv22*w59+1.3971185064935066e-03*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+2.3833198051948050e-03*Jinv12*w13+2.6298701298701297e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21+1.3971185064935066e-03*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-2.1367694805194807e-03*Jinv22*w41-7.3965097402597406e-03*w26*Jinv20-7.8530844155844158e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+2.7394480519480518e-04*w31*Jinv11-7.3965097402597406e-03*Jinv22*w26-2.4655032467532467e-03*w46*Jinv11+2.3833198051948050e-03*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21+2.3833198051948050e-03*Jinv02*w54+7.1225649350649346e-04*w4*Jinv12-3.6982548701298703e-03*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-1.1414366883116883e-03*Jinv21*w29-2.1367694805194807e-03*w39*Jinv00+2.3833198051948050e-03*Jinv02*w12+1.3971185064935066e-03*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-7.8530844155844158e-04*Jinv02*w9+1.6436688311688311e-04*w7*Jinv10-2.4655032467532467e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38-3.6982548701298703e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-3.6982548701298703e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8-3.6982548701298703e-03*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-7.8530844155844158e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56-7.3965097402597406e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12+7.1225649350649346e-04*Jinv22*w5-3.6982548701298703e-03*w47*Jinv22+1.1505681818181819e-03*Jinv21*w14-7.8530844155844158e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-7.4878246753246752e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18+1.1505681818181819e-03*Jinv01*w12-7.4878246753246752e-04*w57*Jinv00-7.4878246753246752e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11-7.3965097402597406e-03*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-4.2735389610389614e-03*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9-4.2735389610389614e-03*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+2.6298701298701297e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.3833198051948050e-03*Jinv22*w14-5.4788961038961036e-04*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+2.3833198051948050e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00-3.6982548701298703e-03*w47*Jinv20-4.2735389610389614e-03*w16*Jinv12-1.1414366883116883e-03*Jinv01*w27-7.4878246753246752e-04*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[26][24]=(0.0)*D;
  A[26][25]=(0.0)*D;
  A[26][26]=(-2.3833198051948050e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-1.6436688311688311e-04*Jinv20*w8-2.3833198051948050e-03*w13*Jinv10+4.7666396103896100e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-1.0683847402597404e-03*Jinv01*w3+3.6982548701298703e-03*Jinv20*w23-2.3833198051948050e-03*Jinv00*w54+7.4878246753246752e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w43*Jinv10+4.7666396103896100e-03*w42*Jinv01+6.5746753246753243e-04*w11*Jinv21+4.7666396103896100e-03*w16*Jinv11+4.7666396103896100e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45+7.4878246753246752e-04*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+7.4878246753246752e-04*w58*Jinv11+7.4878246753246752e-04*w57*Jinv01+7.3965097402597406e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+4.2735389610389614e-03*w16*Jinv10+4.7666396103896100e-03*w43*Jinv11+3.6982548701298703e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-2.0271915584415584e-03*w13*Jinv11+4.7666396103896100e-03*w15*Jinv01-9.1314935064935068e-05*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+3.6982548701298703e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10+3.6982548701298703e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.6298701298701297e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00+5.7528409090909093e-04*Jinv21*w38+7.3965097402597406e-03*Jinv01*w24+1.6436688311688311e-03*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11-1.0683847402597404e-03*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-1.0683847402597404e-03*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-2.6298701298701297e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-1.6436688311688311e-04*w6*Jinv00-2.3833198051948050e-03*Jinv20*w56+7.3965097402597406e-03*w26*Jinv20+7.8530844155844158e-04*w11*Jinv20+1.6436688311688311e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.4655032467532467e-03*w46*Jinv11-2.3833198051948050e-03*Jinv20*w14-1.0683847402597404e-03*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10-9.1314935064935068e-05*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.3149350649350649e-03*Jinv01*w18+5.7528409090909093e-04*w37*Jinv11+5.7528409090909093e-04*w6*Jinv01-1.6436688311688311e-04*w7*Jinv10+2.4655032467532467e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38-1.0683847402597404e-03*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53+3.6982548701298703e-03*w22*Jinv10+2.4655032467532467e-03*w22*Jinv11+2.4655032467532467e-03*w21*Jinv01-1.0683847402597404e-03*Jinv01*w51+5.7528409090909093e-04*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+5.7528409090909093e-04*w7*Jinv11+7.8530844155844158e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-2.0271915584415584e-03*Jinv21*w56+7.3965097402597406e-03*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11-2.0271915584415584e-03*Jinv21*w14+7.8530844155844158e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-2.0271915584415584e-03*Jinv01*w12+5.7528409090909093e-04*Jinv21*w8+7.4878246753246752e-04*w57*Jinv00+7.4878246753246752e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+2.4655032467532467e-03*Jinv21*w23-2.0271915584415584e-03*Jinv01*w54-2.0271915584415584e-03*w55*Jinv11+7.3965097402597406e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+4.2735389610389614e-03*w15*Jinv00+6.5746753246753243e-04*Jinv01*w9+4.2735389610389614e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.6298701298701297e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+7.3965097402597406e-03*w25*Jinv11-2.3833198051948050e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00+3.6982548701298703e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-9.1314935064935068e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+6.5746753246753243e-04*w10*Jinv11)*D;
  A[26][27]=(0.0)*D;
  A[26][28]=(0.0)*D;
  A[26][29]=(6.5746753246753243e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3+1.5726461038961039e-04*Jinv21*w59+1.5726461038961039e-04*Jinv21*w2+6.5746753246753243e-04*w42*Jinv01+2.0495129870129870e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.1870941558441558e-03*Jinv01*w45+1.5726461038961039e-04*w58*Jinv11+1.5726461038961039e-04*w57*Jinv01+2.3741883116883116e-03*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+6.5746753246753243e-04*w43*Jinv11+3.2873376623376621e-04*Jinv21*w41+1.5726461038961039e-04*Jinv11*w1-6.7573051948051951e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+3.2873376623376621e-04*Jinv01*w39-2.6481331168831169e-04*Jinv21*w38+2.3741883116883116e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-1.0227272727272728e-03*w19*Jinv11-1.0957792207792208e-04*Jinv21*w53-1.0957792207792208e-04*w4*Jinv11-5.4788961038961042e-05*w32*Jinv21-1.0227272727272728e-03*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11+1.1870941558441558e-03*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-1.0227272727272728e-03*Jinv01*w18-2.6481331168831169e-04*w37*Jinv11-2.6481331168831169e-04*w6*Jinv01+1.1870941558441558e-03*w47*Jinv21-1.0957792207792208e-04*w52*Jinv11+1.1870941558441558e-03*w22*Jinv11+1.1870941558441558e-03*w21*Jinv01-1.0957792207792208e-04*Jinv01*w51-2.6481331168831169e-04*w36*Jinv01-2.6481331168831169e-04*w7*Jinv11-6.7573051948051951e-04*Jinv21*w56-5.4788961038961042e-05*w49*Jinv11-6.7573051948051951e-04*Jinv21*w14-6.7573051948051951e-04*Jinv01*w12-2.6481331168831169e-04*Jinv21*w8+1.1870941558441558e-03*Jinv21*w23-6.7573051948051951e-04*Jinv01*w54-6.7573051948051951e-04*w55*Jinv11+2.0495129870129870e-04*Jinv01*w9+3.2873376623376621e-04*w40*Jinv11-5.4788961038961042e-05*Jinv01*w30+2.3741883116883116e-03*w25*Jinv11-5.4788961038961042e-05*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+2.0495129870129870e-04*w10*Jinv11)*D;
  A[26][30]=(0.0)*D;
  A[26][31]=(0.0)*D;
  A[26][32]=(8.2183441558441553e-05*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12-4.6570616883116886e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+2.7394480519480521e-05*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-1.9176136363636362e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-5.4788961038961042e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-2.4655032467532466e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02-2.4655032467532466e-04*w48*Jinv00+8.2183441558441553e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-6.3920454545454550e-05*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+8.7662337662337668e-04*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10-3.5612824675324673e-04*Jinv02*w3-1.9176136363636362e-04*w21*Jinv00+7.1225649350649346e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+2.7394480519480521e-05*Jinv22*w59+8.7662337662337668e-04*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+8.2183441558441553e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+7.1225649350649346e-04*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+8.7662337662337668e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20-4.6570616883116886e-04*w52*Jinv12+8.7662337662337668e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+7.1225649350649346e-04*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-2.4655032467532466e-04*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+8.7662337662337668e-04*Jinv01*w33+7.1225649350649346e-04*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-2.4655032467532466e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.5612824675324673e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8-1.9176136363636362e-04*w22*Jinv10+2.7394480519480521e-05*Jinv22*w50-1.9176136363636362e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.9176136363636362e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+2.7394480519480521e-05*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02-3.5612824675324673e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+2.7394480519480521e-05*w58*Jinv12+8.2183441558441553e-05*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+8.2183441558441553e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+8.7662337662337668e-04*w35*Jinv20-1.9176136363636362e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+2.7394480519480521e-05*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+7.1225649350649346e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30-4.6570616883116886e-04*Jinv02*w51-2.4655032467532466e-04*Jinv22*w56+7.1225649350649346e-04*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[26][33]=(0.0)*D;
  A[26][34]=(0.0)*D;
  A[26][35]=(-3.2873376623376621e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+1.3149350649350649e-03*Jinv22*w53+1.2601461038961039e-03*Jinv20*w8-3.4699675324675324e-04*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10+3.9448051948051946e-03*Jinv21*w44-1.2053571428571428e-03*w3*Jinv00-8.2183441558441553e-04*w32*Jinv20-1.1505681818181819e-03*Jinv01*w3+1.3149350649350649e-03*Jinv22*w38+3.8352272727272725e-04*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59+3.4699675324675324e-04*w0*Jinv02+4.2004870129870131e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+3.9448051948051946e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17-2.1915584415584414e-03*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01-3.2873376623376621e-03*Jinv21*w26+4.2004870129870131e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+3.9448051948051946e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+4.2004870129870131e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01+9.8620129870129864e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17+1.6436688311688311e-03*Jinv00*w33+1.0957792207792207e-03*w22*Jinv12-8.2183441558441553e-04*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-8.2183441558441553e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+3.4699675324675324e-04*Jinv22*w2+1.7532467532467534e-03*Jinv00*w18+2.5750811688311690e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45-3.2873376623376621e-03*Jinv01*w24-3.4699675324675324e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00+1.9724025974025973e-03*Jinv22*w44-1.2053571428571428e-03*w4*Jinv10-1.1505681818181819e-03*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13+1.7532467532467534e-03*w19*Jinv10+1.6436688311688311e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.2601461038961039e-03*w6*Jinv00+1.3149350649350649e-03*w37*Jinv12+3.8352272727272725e-04*Jinv20*w56-2.1915584415584414e-03*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-2.1915584415584414e-03*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14-1.1505681818181819e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+9.8620129870129864e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12+2.1367694805194807e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+1.3149350649350649e-03*w6*Jinv01+1.2601461038961039e-03*w7*Jinv10-2.1915584415584414e-03*w47*Jinv21+2.5750811688311690e-03*Jinv20*w38+1.0957792207792207e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-8.2183441558441553e-04*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8+8.2183441558441553e-04*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-1.0957792207792207e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-8.2183441558441553e-04*Jinv02*w30+8.2183441558441553e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.3149350649350649e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.5750811688311690e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56-2.1915584415584414e-03*w24*Jinv00-1.3149350649350649e-03*w6*Jinv02-1.3149350649350649e-03*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44-3.4699675324675324e-04*w58*Jinv12+5.4788961038961042e-05*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+3.8352272727272725e-04*w0*Jinv00+1.6436688311688311e-03*w35*Jinv20-1.0957792207792207e-03*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11-2.1915584415584414e-03*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00+8.2183441558441553e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17-8.2183441558441553e-04*w30*Jinv00+1.7532467532467534e-03*w20*Jinv20-7.1225649350649346e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+1.3149350649350649e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.2053571428571428e-03*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-3.2873376623376621e-03*w25*Jinv11+3.8352272727272725e-04*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+9.8620129870129864e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+3.4699675324675324e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[26][36]=(0.0)*D;
  A[26][37]=(0.0)*D;
  A[26][38]=(2.4655032467532466e-04*Jinv00*w12-4.8396915584415583e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53-1.0135957792207792e-03*Jinv20*w8-4.2918019480519480e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+8.4922889610389610e-04*w3*Jinv00+4.6570616883116886e-04*w32*Jinv20+1.9176136363636362e-04*Jinv20*w23-2.3833198051948050e-03*Jinv22*w38+1.3697240259740259e-04*Jinv00*w54+1.1505681818181819e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-2.7394480519480520e-03*w43*Jinv10+5.2049512987012989e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00+1.1505681818181819e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-3.1960227272727273e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20+3.8352272727272725e-04*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02-4.8396915584415583e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+2.4655032467532466e-04*Jinv20*w50-4.8396915584415583e-04*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-2.5202922077922079e-03*Jinv00*w33-1.6436688311688311e-04*w22*Jinv12+4.6570616883116886e-04*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+1.9176136363636362e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-2.0271915584415584e-03*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45-4.2918019480519480e-04*Jinv22*w59+4.3831168831168834e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+8.4922889610389610e-04*w4*Jinv10-7.1225649350649346e-04*w41*Jinv20+5.2049512987012989e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10-2.5202922077922079e-03*w34*Jinv10-1.0135957792207792e-03*w6*Jinv00-2.3833198051948050e-03*w37*Jinv12+1.3697240259740259e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+1.4793019480519481e-03*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+1.1505681818181819e-03*Jinv22*w26+2.4655032467532466e-04*Jinv20*w14+5.2049512987012989e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10-7.1225649350649346e-04*w39*Jinv00+5.2049512987012989e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.0135957792207792e-03*w7*Jinv10-2.0271915584415584e-03*Jinv20*w38-1.6436688311688311e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+4.3831168831168834e-04*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+1.9176136363636362e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+4.3831168831168834e-04*w52*Jinv10+1.1870941558441559e-04*w9*Jinv00-2.0271915584415584e-03*w37*Jinv10+1.4793019480519481e-03*w24*Jinv00-4.7483766233766234e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22+1.1870941558441559e-04*w10*Jinv10-2.7394480519480520e-03*Jinv20*w44-4.2918019480519480e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-3.1960227272727273e-04*w0*Jinv00-2.5202922077922079e-03*w35*Jinv20+1.4793019480519481e-03*w25*Jinv10-3.1960227272727273e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+4.6570616883116886e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-4.7483766233766234e-04*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-2.3833198051948050e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.4922889610389610e-04*w5*Jinv20+1.3971185064935066e-03*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56-4.7483766233766234e-04*Jinv12*w28+1.3697240259740259e-04*w55*Jinv10-2.7394480519480520e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[26][39]=(0.0)*D;
  A[26][40]=(0.0)*D;
  A[26][41]=(1.3149350649350649e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+7.1225649350649346e-04*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10-3.2873376623376621e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-2.1367694805194807e-03*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59+2.1915584415584417e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+8.2183441558441553e-04*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+3.8352272727272725e-04*w48*Jinv00-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10-3.2873376623376621e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-3.1047077922077924e-04*Jinv11*w1+9.3141233766233771e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00-3.2873376623376621e-04*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.7532467532467534e-03*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00-9.8620129870129864e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-9.8620129870129864e-04*w41*Jinv20-8.2183441558441553e-04*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+1.7532467532467534e-03*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-3.1777597402597403e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14+8.2183441558441553e-04*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-8.2183441558441553e-04*Jinv02*w12+1.7532467532467534e-03*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11+3.2873376623376621e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-3.2873376623376621e-04*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50-1.0409902597402598e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-1.0409902597402598e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-3.2873376623376621e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10+1.7532467532467534e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00-3.8352272727272725e-04*w6*Jinv02-7.1225649350649346e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11+9.3141233766233771e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12+9.3141233766233771e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59-2.7394480519480518e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-1.0409902597402598e-03*Jinv21*w23+1.7532467532467534e-03*Jinv01*w54+1.7532467532467534e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20-8.2183441558441553e-04*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+7.1225649350649346e-04*Jinv02*w51+8.2183441558441553e-04*Jinv22*w56-3.1777597402597403e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[26][42]=(0.0)*D;
  A[26][43]=(0.0)*D;
  A[26][44]=(-1.3149350649350649e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8+5.4788961038961036e-04*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20+1.1505681818181819e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59+1.2820616883116883e-02*w25*Jinv12+5.4788961038961036e-04*w0*Jinv02-4.2004870129870131e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17+2.1915584415584414e-03*Jinv01*w45-2.6298701298701297e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+2.9586038961038961e-03*w15*Jinv02+6.5746753246753243e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00+1.2820616883116883e-02*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+3.2873376623376621e-03*Jinv21*w26-4.2004870129870131e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-4.9310064935064934e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+2.9586038961038961e-03*w42*Jinv02-5.4788961038961036e-04*Jinv00*w27+6.5746753246753243e-04*w11*Jinv22-4.2004870129870131e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01-9.8620129870129864e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10+2.9586038961038961e-03*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+4.2735389610389614e-03*w22*Jinv12-5.4788961038961042e-05*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39+5.4788961038961036e-04*Jinv22*w2-2.1367694805194807e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+3.2873376623376621e-03*Jinv01*w24+5.4788961038961036e-04*Jinv22*w59-4.9310064935064934e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34+2.9586038961038961e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+1.1505681818181819e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.6298701298701297e-03*Jinv12*w13-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12-4.9310064935064934e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11+1.2820616883116883e-02*Jinv22*w26+2.1915584415584414e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14+1.1505681818181819e-03*w5*Jinv21-2.6298701298701297e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-9.8620129870129864e-04*Jinv21*w29-4.9310064935064934e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-2.6298701298701297e-03*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+6.5746753246753243e-04*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+2.1915584415584414e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38+4.2735389610389614e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-7.6704545454545450e-04*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.9310064935064934e-03*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-5.4788961038961042e-05*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+5.4788961038961036e-04*w58*Jinv12-4.9310064935064934e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29-2.6298701298701297e-03*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36+2.9586038961038961e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+7.6704545454545450e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-2.6298701298701297e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28+3.2873376623376621e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20+2.9586038961038961e-03*w16*Jinv12-5.4788961038961042e-05*Jinv21*w50-9.8620129870129864e-04*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01+5.4788961038961036e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[26][45]=(0.0)*D;
  A[26][46]=(0.0)*D;
  A[26][47]=(2.1367694805194807e-03*w40*Jinv12-7.1225649350649346e-04*Jinv22*w53+7.4878246753246752e-04*w57*Jinv02+2.1367694805194807e-03*Jinv02*w39+3.1777597402597403e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-1.6436688311688311e-04*Jinv22*w38+3.9265422077922079e-04*Jinv21*w59+7.3965097402597406e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+3.7439123376623376e-04*Jinv21*w2+3.1777597402597403e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45-2.3833198051948050e-03*Jinv12*w55+4.2735389610389614e-03*w15*Jinv02+7.8530844155844158e-04*w10*Jinv12+7.3965097402597406e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.7439123376623376e-04*w0*Jinv01-2.6298701298701297e-03*Jinv22*w20+3.1777597402597403e-03*w43*Jinv11+1.5888798701298701e-03*Jinv21*w41+4.2735389610389614e-03*w42*Jinv02+7.8530844155844158e-04*w11*Jinv22+3.7439123376623376e-04*Jinv11*w1-1.0135957792207792e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11+4.2735389610389614e-03*Jinv22*w17+3.6982548701298703e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-3.5612824675324673e-04*w32*Jinv22+1.5888798701298701e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+3.6982548701298703e-03*Jinv02*w45+2.4655032467532467e-03*Jinv01*w24+7.4878246753246752e-04*Jinv22*w59+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.3833198051948050e-03*Jinv12*w13-3.5612824675324673e-04*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-1.6436688311688311e-04*w37*Jinv12+2.1367694805194807e-03*Jinv22*w41-7.1225649350649346e-04*w52*Jinv12+1.0957792207792207e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+7.3965097402597406e-03*Jinv22*w26+2.4655032467532467e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-2.3833198051948050e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12+3.6982548701298703e-03*w21*Jinv02-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33-2.3833198051948050e-03*Jinv02*w12-1.2327516233766234e-03*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+7.8530844155844158e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+2.4655032467532467e-03*w47*Jinv21+3.6982548701298703e-03*Jinv22*w23-1.6436688311688311e-04*w52*Jinv11-3.5612824675324673e-04*w31*Jinv12+3.6982548701298703e-03*w46*Jinv12-1.6436688311688311e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50+1.2327516233766234e-03*w22*Jinv11-1.6436688311688311e-04*Jinv12*w7+1.2327516233766234e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-3.5612824675324673e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-8.2183441558441553e-05*w7*Jinv11-1.2327516233766234e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5+3.6982548701298703e-03*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11-1.0135957792207792e-03*Jinv21*w14+7.4878246753246752e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-1.0135957792207792e-03*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+1.2327516233766234e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02+3.9265422077922079e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.3833198051948050e-03*Jinv22*w14+1.5888798701298701e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-3.5612824675324673e-04*Jinv01*w30-7.1225649350649346e-04*Jinv02*w51-2.3833198051948050e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+2.4655032467532467e-03*w25*Jinv11+4.2735389610389614e-03*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+3.9265422077922079e-04*w10*Jinv11)*D;
  A[26][48]=(0.0)*D;
  A[26][49]=(0.0)*D;
  A[26][50]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10-3.2873376623376621e-04*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-3.2873376623376621e-04*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480521e-05*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+3.5612824675324673e-04*w48*Jinv00-8.2183441558441553e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10+7.6704545454545450e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10+4.6570616883116886e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-2.7394480519480521e-05*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39-2.7394480519480521e-05*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-8.2183441558441553e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-3.2873376623376621e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11+1.6436688311688311e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54+4.6570616883116886e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-2.7394480519480521e-05*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+3.5612824675324673e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-2.7394480519480521e-05*Jinv02*w30-5.2049512987012989e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02+4.6570616883116886e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12-2.7394480519480518e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59-8.2183441558441553e-04*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+1.3697240259740259e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20+2.4655032467532466e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56+7.1225649350649346e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01-2.7394480519480521e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[26][51]=(0.0)*D;
  A[26][52]=(0.0)*D;
  A[26][53]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-7.1225649350649346e-04*w40*Jinv12-1.0683847402597404e-03*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-7.1225649350649346e-04*Jinv02*w39+8.2183441558441553e-05*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+8.2183441558441553e-05*Jinv22*w38-1.6436688311688311e-04*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12-2.1915584415584417e-04*w0*Jinv02-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10-1.6436688311688311e-04*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10+1.4793019480519481e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02+9.1314935064935065e-06*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-3.5612824675324673e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.6436688311688311e-04*Jinv22*w17-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12+2.7394480519480518e-04*w31*Jinv10+2.4655032467532466e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+4.6570616883116886e-04*w32*Jinv22-2.1915584415584417e-04*Jinv22*w2+5.2049512987012989e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+3.8352272727272725e-04*Jinv02*w45+1.2784090909090910e-04*Jinv22*w59-7.1225649350649346e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-3.8352272727272725e-04*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+8.2183441558441553e-05*w37*Jinv12-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.0683847402597404e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10-3.8352272727272725e-04*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+2.1915584415584417e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+5.2049512987012989e-04*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23+4.6570616883116886e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12-7.1225649350649346e-04*Jinv20*w53+1.3971185064935066e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50+1.3971185064935066e-03*Jinv12*w7+4.6570616883116886e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-7.1225649350649346e-04*w52*Jinv10-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00+1.8262987012987013e-05*Jinv02*w27+1.3971185064935066e-03*w6*Jinv02+1.4793019480519481e-03*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22-1.3697240259740259e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12+1.4793019480519481e-03*Jinv02*w18+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20+1.8262987012987013e-05*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14+8.2183441558441553e-05*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-1.0683847402597404e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+1.8262987012987013e-05*Jinv12*w28-1.6436688311688311e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.6436688311688311e-04*w16*Jinv12-2.1915584415584417e-04*Jinv12*w1)*D;
  A[26][54]=(0.0)*D;
  A[26][55]=(0.0)*D;
  A[26][56]=(-8.2183441558441553e-04*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-2.1002435064935065e-04*w57*Jinv02-8.2183441558441553e-04*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+8.2183441558441553e-05*Jinv01*w3+1.3697240259740259e-04*Jinv22*w38+9.1314935064935065e-06*Jinv21*w59-4.7666396103896100e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+5.7528409090909093e-04*Jinv12*w55-2.6298701298701297e-03*w15*Jinv02-3.1047077922077924e-04*w10*Jinv12-4.7666396103896100e-03*Jinv02*w24-7.1225649350649346e-04*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-7.1225649350649346e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01+1.4793019480519481e-03*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-3.8352272727272725e-04*Jinv21*w41-1.6436688311688311e-03*w42*Jinv02-3.1047077922077924e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11-2.6298701298701297e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+5.4788961038961036e-04*Jinv02*w3+2.4655032467532466e-04*w32*Jinv22-3.8352272727272725e-04*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+5.2049512987012989e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-7.1225649350649346e-04*Jinv01*w24-2.1002435064935065e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+5.2049512987012989e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-7.1225649350649346e-04*Jinv12*w34-1.6436688311688311e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+2.7394480519480518e-04*w32*Jinv21+5.2049512987012989e-04*Jinv21*w20+1.3697240259740259e-04*w37*Jinv12-8.2183441558441553e-04*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11-4.7666396103896100e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+8.2183441558441553e-05*w5*Jinv21+5.7528409090909093e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+5.2049512987012989e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11-3.1047077922077924e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-1.6436688311688311e-04*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11+2.4655032467532466e-04*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12+5.2049512987012989e-04*Jinv22*w8-8.2183441558441553e-05*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11+5.2049512987012989e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+2.4655032467532466e-04*Jinv02*w30-8.2183441558441553e-05*Jinv12*w49+5.2049512987012989e-04*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+5.2049512987012989e-04*w6*Jinv02+1.4793019480519481e-03*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-2.1002435064935065e-04*w58*Jinv12+1.4793019480519481e-03*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-7.1225649350649346e-04*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-8.2183441558441553e-05*w48*Jinv02-1.3697240259740259e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-3.8352272727272725e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-1.6436688311688311e-03*Jinv12*w43+2.7394480519480518e-04*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51+5.7528409090909093e-04*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-7.1225649350649346e-04*w25*Jinv11-2.6298701298701297e-03*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-1.3697240259740259e-04*w10*Jinv11)*D;
  A[26][57]=(0.0)*D;
  A[26][58]=(0.0)*D;
  A[26][59]=(2.1915584415584417e-04*w40*Jinv12-1.0957792207792208e-04*Jinv22*w53+2.0292207792207792e-05*w57*Jinv02+2.1915584415584417e-04*Jinv02*w39+1.1870941558441559e-04*Jinv22*w38+1.4975649350649350e-03*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-2.6481331168831169e-04*Jinv12*w55+5.4788961038961036e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+1.4975649350649350e-03*Jinv02*w24+3.4699675324675324e-04*Jinv22*w35-5.8441558441558442e-04*Jinv22*w20+4.3831168831168834e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+5.4788961038961036e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-2.1915584415584417e-04*Jinv02*w3-2.7394480519480521e-05*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.6525974025974026e-05*Jinv02*w45+2.0292207792207792e-05*Jinv22*w59+3.4699675324675324e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-4.2918019480519480e-04*Jinv12*w13+1.1870941558441559e-04*w37*Jinv12+2.1915584415584417e-04*Jinv22*w41-1.0957792207792208e-04*w52*Jinv12+1.4975649350649350e-03*Jinv22*w26-2.6481331168831169e-04*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-4.2918019480519480e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9+7.4878246753246752e-04*Jinv22*w23-2.7394480519480521e-05*w31*Jinv12+3.6525974025974026e-05*w46*Jinv12-4.2918019480519480e-04*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7-2.7394480519480521e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49+6.4935064935064935e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-5.8441558441558442e-04*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5+3.6525974025974026e-05*w47*Jinv22+2.0292207792207792e-05*w58*Jinv12-5.8441558441558442e-04*Jinv02*w18+3.4699675324675324e-04*Jinv02*w33-5.4788961038961042e-05*w48*Jinv02+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+1.1870941558441559e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-1.0957792207792208e-04*Jinv02*w51-2.6481331168831169e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+5.4788961038961036e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[27][0]=(-2.0495129870129870e-04*Jinv00*w12-1.8939393939393939e-04*w28*Jinv10+4.2613636363636367e-05*w40*Jinv12-1.0247564935064935e-04*Jinv22*w53+1.1668019480519480e-04*Jinv20*w8-5.4112554112554113e-05*w57*Jinv02-2.0495129870129870e-04*w13*Jinv10+4.2613636363636367e-05*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-1.5726461038961039e-04*w3*Jinv00-1.5726461038961039e-04*w32*Jinv20-1.5726461038961039e-04*Jinv01*w3-8.3198051948051951e-05*Jinv20*w23-1.0247564935064935e-04*Jinv22*w38+1.4204545454545455e-05*Jinv00*w54-5.4112554112554113e-05*Jinv21*w59-6.4935064935064935e-05*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-5.4112554112554113e-05*w11*Jinv21+4.2613636363636367e-05*w16*Jinv11+4.2613636363636367e-05*Jinv21*w17-6.4935064935064935e-05*Jinv01*w45+1.4204545454545455e-05*Jinv12*w55-5.4112554112554113e-05*w58*Jinv10+4.2613636363636367e-05*w15*Jinv02-5.4112554112554113e-05*w10*Jinv12+1.1668019480519480e-04*w48*Jinv00-6.4935064935064935e-05*Jinv02*w24+1.9480519480519481e-04*Jinv22*w35-5.4112554112554113e-05*w58*Jinv11-5.4112554112554113e-05*w57*Jinv01-6.4935064935064935e-05*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+4.2613636363636367e-05*w16*Jinv10+1.4204545454545455e-05*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11-6.4935064935064935e-05*w45*Jinv00+4.2613636363636367e-05*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-1.8939393939393939e-04*Jinv00*w27-5.4112554112554113e-05*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-2.0495129870129870e-04*w13*Jinv11+4.2613636363636367e-05*w15*Jinv01-1.8939393939393939e-04*w28*Jinv11+1.1668019480519480e-04*Jinv20*w50-1.8939393939393939e-04*Jinv20*w29-6.4935064935064935e-05*w46*Jinv10+4.2613636363636367e-05*Jinv22*w17+1.9480519480519481e-04*Jinv00*w33-8.3198051948051951e-05*w22*Jinv12-1.5726461038961039e-04*w31*Jinv10-1.5726461038961039e-04*Jinv02*w3-8.3198051948051951e-05*w21*Jinv00-1.5726461038961039e-04*w32*Jinv22+4.2613636363636367e-05*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2+1.4204545454545455e-05*Jinv00*w18-1.0247564935064935e-04*w36*Jinv00-1.0247564935064935e-04*Jinv21*w38-6.4935064935064935e-05*Jinv02*w45-6.4935064935064935e-05*Jinv01*w24-5.4112554112554113e-05*Jinv22*w59+1.9480519480519481e-04*w34*Jinv11+1.4204545454545455e-05*w19*Jinv11-1.0247564935064935e-04*Jinv21*w53-1.0247564935064935e-04*w51*Jinv00+1.9480519480519481e-04*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-1.5726461038961039e-04*w4*Jinv10-1.5726461038961039e-04*w4*Jinv11+4.2613636363636367e-05*w41*Jinv20-2.0495129870129870e-04*Jinv12*w13+1.4204545454545455e-05*w19*Jinv10+1.9480519480519481e-04*w34*Jinv10-1.5726461038961039e-04*w32*Jinv21+1.4204545454545455e-05*Jinv21*w20+1.1668019480519480e-04*w6*Jinv00-1.0247564935064935e-04*w37*Jinv12+1.4204545454545455e-05*Jinv20*w56+4.2613636363636367e-05*Jinv22*w41-6.4935064935064935e-05*w26*Jinv20-5.4112554112554113e-05*w11*Jinv20-1.0247564935064935e-04*w52*Jinv12+1.9480519480519481e-04*Jinv21*w35-1.5726461038961039e-04*w31*Jinv11-6.4935064935064935e-05*Jinv22*w26-6.4935064935064935e-05*w46*Jinv11-2.0495129870129870e-04*Jinv20*w14-1.5726461038961039e-04*w5*Jinv21+1.4204545454545455e-05*Jinv02*w54-1.5726461038961039e-04*w4*Jinv12-8.3198051948051951e-05*w21*Jinv02+1.1668019480519480e-04*w49*Jinv10-1.8939393939393939e-04*Jinv21*w29+1.9480519480519481e-04*Jinv01*w33+4.2613636363636367e-05*w39*Jinv00-2.0495129870129870e-04*Jinv02*w12+1.4204545454545455e-05*Jinv01*w18-1.0247564935064935e-04*w37*Jinv11-5.4112554112554113e-05*Jinv02*w9+1.1668019480519480e-04*w6*Jinv01+1.1668019480519480e-04*w7*Jinv10-6.4935064935064935e-05*w47*Jinv21-1.0247564935064935e-04*Jinv20*w38-8.3198051948051951e-05*Jinv22*w23-1.0247564935064935e-04*w52*Jinv11-1.5726461038961039e-04*w31*Jinv12-6.4935064935064935e-05*w46*Jinv12-1.0247564935064935e-04*Jinv20*w53+1.1668019480519480e-04*Jinv22*w8-8.3198051948051951e-05*w22*Jinv10+1.1668019480519480e-04*Jinv22*w50-8.3198051948051951e-05*w22*Jinv11+1.1668019480519480e-04*Jinv12*w7-8.3198051948051951e-05*w21*Jinv01-1.0247564935064935e-04*Jinv01*w51-1.5726461038961039e-04*Jinv02*w30+1.1668019480519480e-04*Jinv12*w49-1.0247564935064935e-04*w36*Jinv01-1.0247564935064935e-04*w52*Jinv10+1.1668019480519480e-04*w7*Jinv11-5.4112554112554113e-05*w9*Jinv00-1.0247564935064935e-04*w37*Jinv10+1.4204545454545455e-05*Jinv21*w56-6.4935064935064935e-05*w24*Jinv00-1.8939393939393939e-04*Jinv02*w27+1.1668019480519480e-04*w6*Jinv02+1.4204545454545455e-05*w19*Jinv12-1.5726461038961039e-04*Jinv22*w5-6.4935064935064935e-05*w47*Jinv22+1.1668019480519480e-04*w49*Jinv11-2.0495129870129870e-04*Jinv21*w14-5.4112554112554113e-05*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-5.4112554112554113e-05*w58*Jinv12+1.4204545454545455e-05*Jinv02*w18-2.0495129870129870e-04*Jinv01*w12+1.1668019480519480e-04*Jinv21*w8-5.4112554112554113e-05*w57*Jinv00-5.4112554112554113e-05*Jinv20*w59+1.9480519480519481e-04*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+1.9480519480519481e-04*w35*Jinv20-8.3198051948051951e-05*Jinv21*w23+1.4204545454545455e-05*Jinv01*w54+1.4204545454545455e-05*w55*Jinv11-6.4935064935064935e-05*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+4.2613636363636367e-05*w15*Jinv00+1.1668019480519480e-04*w48*Jinv02-5.4112554112554113e-05*Jinv01*w9+4.2613636363636367e-05*Jinv20*w17-1.5726461038961039e-04*w30*Jinv00+4.2613636363636367e-05*w40*Jinv10+1.4204545454545455e-05*w20*Jinv20-1.8939393939393939e-04*Jinv22*w29-2.0495129870129870e-04*Jinv22*w14+4.2613636363636367e-05*w40*Jinv11-1.0247564935064935e-04*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-1.5726461038961039e-04*w5*Jinv20-1.5726461038961039e-04*Jinv01*w30-1.0247564935064935e-04*Jinv02*w51+1.4204545454545455e-05*Jinv22*w56-1.8939393939393939e-04*Jinv12*w28-6.4935064935064935e-05*w25*Jinv11+1.4204545454545455e-05*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00-6.4935064935064935e-05*w47*Jinv20+4.2613636363636367e-05*w16*Jinv12+1.1668019480519480e-04*Jinv21*w50-1.8939393939393939e-04*Jinv01*w27+1.1668019480519480e-04*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-5.4112554112554113e-05*w10*Jinv11)*D;
  A[27][1]=(0.0)*D;
  A[27][2]=(0.0)*D;
  A[27][3]=(3.9265422077922079e-04*Jinv00*w12-5.2962662337662338e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+3.9265422077922079e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44+3.7439123376623376e-04*w32*Jinv20-7.4878246753246752e-04*Jinv01*w3+9.1314935064935065e-06*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38-1.0247564935064935e-04*Jinv21*w59-1.8262987012987013e-05*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-1.0592532467532468e-03*w16*Jinv11-1.0592532467532468e-03*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55-1.0592532467532468e-03*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00-1.8262987012987013e-05*Jinv02*w24-9.6793831168831166e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01-1.8262987012987013e-05*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+1.5726461038961039e-04*w1*Jinv10-5.6615259740259744e-04*w16*Jinv10+3.1047077922077924e-04*Jinv22*w20+7.3051948051948052e-05*w43*Jinv11-5.2962662337662338e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*w11*Jinv22+9.2329545454545453e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-1.0592532467532468e-03*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-1.0592532467532468e-03*Jinv22*w17-6.9399350649350649e-04*Jinv00*w33+4.8396915584415583e-04*w22*Jinv12+3.7439123376623376e-04*w31*Jinv10-7.4878246753246752e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-3.7439123376623376e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+1.7349837662337662e-04*Jinv00*w18+3.1960227272727273e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-1.8262987012987013e-05*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-9.6793831168831166e-04*w34*Jinv11+3.1047077922077924e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-9.6793831168831166e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-7.4878246753246752e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20-3.9265422077922079e-04*Jinv12*w13+1.7349837662337662e-04*w19*Jinv10-6.9399350649350649e-04*w34*Jinv10-3.7439123376623376e-04*w32*Jinv21+3.1047077922077924e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12-5.2962662337662338e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12-9.6793831168831166e-04*Jinv21*w35-3.7439123376623376e-04*w31*Jinv11-1.8262987012987013e-05*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11+3.9265422077922079e-04*Jinv20*w14-7.4878246753246752e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54-7.4878246753246752e-04*w4*Jinv12+4.8396915584415583e-04*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-9.6793831168831166e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00-3.9265422077922079e-04*Jinv02*w12+3.1047077922077924e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9+1.5523538961038962e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-9.1314935064935065e-06*w47*Jinv21+3.1960227272727273e-04*Jinv20*w38+4.8396915584415583e-04*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11-3.7439123376623376e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.5523538961038962e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+3.1960227272727273e-04*Jinv22*w50+4.8396915584415583e-04*w22*Jinv11+1.5523538961038962e-04*Jinv12*w7+4.8396915584415583e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51-3.7439123376623376e-04*Jinv02*w30+3.1960227272727273e-04*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.5523538961038962e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+3.1960227272727273e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-4.0584415584415584e-05*Jinv02*w27+1.5523538961038962e-04*w6*Jinv02+3.1047077922077924e-04*w19*Jinv12-7.4878246753246752e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+3.1960227272727273e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+3.1047077922077924e-04*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.5523538961038962e-04*Jinv21*w8-9.6793831168831166e-04*Jinv02*w33+1.5726461038961039e-04*w0*Jinv00-6.9399350649350649e-04*w35*Jinv20+4.8396915584415583e-04*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2-5.6615259740259744e-04*w15*Jinv00+3.1960227272727273e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-5.6615259740259744e-04*Jinv20*w17+3.7439123376623376e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10+1.7349837662337662e-04*w20*Jinv20-4.0584415584415584e-05*Jinv22*w29-3.9265422077922079e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.7439123376623376e-04*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56-4.0584415584415584e-05*Jinv12*w28-1.8262987012987013e-05*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-1.0592532467532468e-03*w16*Jinv12+3.1960227272727273e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+3.1960227272727273e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[27][4]=(0.0)*D;
  A[27][5]=(0.0)*D;
  A[27][6]=(-1.7349837662337662e-04*Jinv00*w12+3.6525974025974026e-05*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-1.0247564935064935e-04*w57*Jinv02-1.7349837662337662e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00-3.1960227272727273e-04*w32*Jinv20+4.2918019480519480e-04*Jinv01*w3+9.1314935064935065e-06*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38-1.0247564935064935e-04*Jinv21*w59+4.7483766233766234e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55-4.9310064935064932e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+4.7483766233766234e-04*Jinv02*w24-2.7394480519480518e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+4.7483766233766234e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.1668019480519480e-04*w1*Jinv10+5.6615259740259744e-04*w16*Jinv10-7.8530844155844158e-04*Jinv22*w20-4.9310064935064932e-04*w43*Jinv11+3.6525974025974026e-05*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*w11*Jinv22-1.0247564935064935e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-4.9310064935064932e-04*Jinv22*w17+6.9399350649350649e-04*Jinv00*w33-9.1314935064935065e-06*w22*Jinv12-3.1960227272727273e-04*w31*Jinv10+4.2918019480519480e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2-3.9265422077922079e-04*Jinv00*w18-3.7439123376623376e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45+4.7483766233766234e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-2.7394480519480518e-04*w34*Jinv11-7.8530844155844158e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-2.7394480519480518e-04*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.2918019480519480e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20+1.3697240259740259e-04*Jinv12*w13-3.9265422077922079e-04*w19*Jinv10+6.9399350649350649e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21-7.8530844155844158e-04*Jinv21*w20-7.4878246753246752e-04*w37*Jinv12+3.6525974025974026e-05*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-2.7394480519480518e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11+4.7483766233766234e-04*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11-1.7349837662337662e-04*Jinv20*w14+4.2918019480519480e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54+4.2918019480519480e-04*w4*Jinv12-9.1314935064935065e-06*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-2.7394480519480518e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00+1.3697240259740259e-04*Jinv02*w12-7.8530844155844158e-04*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.4878246753246752e-04*w6*Jinv01-9.1314935064935065e-06*w47*Jinv21-3.7439123376623376e-04*Jinv20*w38-9.1314935064935065e-06*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53-7.4878246753246752e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+1.0957792207792208e-04*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11-7.4878246753246752e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30+1.0957792207792208e-04*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10-7.4878246753246752e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00-3.7439123376623376e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-4.0584415584415584e-05*Jinv02*w27-7.4878246753246752e-04*w6*Jinv02-7.8530844155844158e-04*w19*Jinv12+4.2918019480519480e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+1.0957792207792208e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-7.8530844155844158e-04*Jinv02*w18+1.3697240259740259e-04*Jinv01*w12-7.4878246753246752e-04*Jinv21*w8-2.7394480519480518e-04*Jinv02*w33-1.1668019480519480e-04*w0*Jinv00+6.9399350649350649e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+5.6615259740259744e-04*w15*Jinv00+1.0957792207792208e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+5.6615259740259744e-04*Jinv20*w17-3.1960227272727273e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10-3.9265422077922079e-04*w20*Jinv20-4.0584415584415584e-05*Jinv22*w29+1.3697240259740259e-04*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56-4.0584415584415584e-05*Jinv12*w28+4.7483766233766234e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-4.9310064935064932e-04*w16*Jinv12+1.0957792207792208e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+1.0957792207792208e-04*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[27][7]=(0.0)*D;
  A[27][8]=(0.0)*D;
  A[27][9]=(-1.4204545454545455e-05*Jinv00*w12+1.8939393939393939e-04*w28*Jinv10+1.5726461038961039e-04*Jinv20*w8-1.4204545454545455e-05*w13*Jinv10-1.1668019480519480e-04*w3*Jinv00+1.0247564935064935e-04*w32*Jinv20+6.4935064935064935e-05*Jinv20*w23-1.4204545454545455e-05*Jinv00*w54-4.2613636363636367e-05*w43*Jinv10+5.4112554112554113e-05*w58*Jinv10+1.0247564935064935e-04*w48*Jinv00+5.4112554112554113e-05*w1*Jinv10-4.2613636363636367e-05*w16*Jinv10+6.4935064935064935e-05*w45*Jinv00+1.8939393939393939e-04*Jinv00*w27+1.0247564935064935e-04*Jinv20*w50+1.8939393939393939e-04*Jinv20*w29+6.4935064935064935e-05*w46*Jinv10-1.9480519480519481e-04*Jinv00*w33+1.0247564935064935e-04*w31*Jinv10+6.4935064935064935e-05*w21*Jinv00+2.0495129870129870e-04*Jinv00*w18+1.5726461038961039e-04*w36*Jinv00-1.1668019480519480e-04*w51*Jinv00-1.1668019480519480e-04*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+2.0495129870129870e-04*w19*Jinv10-1.9480519480519481e-04*w34*Jinv10+1.5726461038961039e-04*w6*Jinv00-1.4204545454545455e-05*Jinv20*w56+8.3198051948051951e-05*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20-1.4204545454545455e-05*Jinv20*w14+1.0247564935064935e-04*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+1.5726461038961039e-04*w7*Jinv10+1.5726461038961039e-04*Jinv20*w38-1.1668019480519480e-04*Jinv20*w53+6.4935064935064935e-05*w22*Jinv10-1.1668019480519480e-04*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+1.5726461038961039e-04*w37*Jinv10+8.3198051948051951e-05*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-4.2613636363636367e-05*Jinv20*w44+5.4112554112554113e-05*w57*Jinv00+5.4112554112554113e-05*Jinv20*w59+5.4112554112554113e-05*w0*Jinv00-1.9480519480519481e-04*w35*Jinv20+8.3198051948051951e-05*w25*Jinv10+5.4112554112554113e-05*Jinv20*w2-4.2613636363636367e-05*w15*Jinv00-4.2613636363636367e-05*Jinv20*w17+1.0247564935064935e-04*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+2.0495129870129870e-04*w20*Jinv20-1.1668019480519480e-04*w5*Jinv20-1.4204545454545455e-05*w55*Jinv10-4.2613636363636367e-05*w42*Jinv00+6.4935064935064935e-05*w47*Jinv20)*D;
  A[27][10]=(0.0)*D;
  A[27][11]=(0.0)*D;
  A[27][12]=(-6.5746753246753243e-04*Jinv00*w12+6.0876623376623375e-04*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53+1.7349837662337662e-04*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-2.5568181818181820e-04*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-3.9265422077922079e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+5.6818181818181818e-05*w0*Jinv02+1.2175324675324675e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.5568181818181820e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+5.5702110389610384e-04*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+1.7349837662337662e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01+5.5702110389610384e-04*Jinv21*w26+1.2175324675324675e-04*w0*Jinv01+5.6818181818181818e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10-2.1915584415584417e-04*Jinv22*w20-2.5568181818181820e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+6.0876623376623375e-04*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22+1.2175324675324675e-04*Jinv11*w1+1.2784090909090910e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+1.7349837662337662e-04*Jinv20*w50+6.0876623376623375e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.0957792207792207e-03*w22*Jinv12-3.9265422077922079e-04*w31*Jinv10-3.9265422077922079e-04*Jinv02*w3+1.0957792207792207e-03*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+5.6818181818181818e-05*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+5.5702110389610384e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-2.5568181818181820e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53+1.3697240259740259e-04*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20+1.7349837662337662e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-2.5568181818181820e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+5.5702110389610384e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54-3.9265422077922079e-04*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+1.7349837662337662e-04*w49*Jinv10+6.4935064935064935e-04*Jinv21*w29-2.5568181818181820e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-2.5568181818181820e-04*w6*Jinv01+1.7349837662337662e-04*w7*Jinv10+5.5702110389610384e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38+1.0957792207792207e-03*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53+1.7349837662337662e-04*Jinv22*w8+1.0957792207792207e-03*w22*Jinv10+1.7349837662337662e-04*Jinv22*w50+6.2094155844155847e-04*w22*Jinv11+1.7349837662337662e-04*Jinv12*w7+6.2094155844155847e-04*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+1.7349837662337662e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01+1.3697240259740259e-04*w52*Jinv10-2.5568181818181820e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00+6.0876623376623375e-04*Jinv02*w27+1.7349837662337662e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12-3.9265422077922079e-04*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22-2.5568181818181820e-04*w49*Jinv11+1.2784090909090910e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+1.2784090909090910e-04*Jinv01*w12-2.5568181818181820e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+5.6818181818181818e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20+6.2094155844155847e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10+5.6818181818181818e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+1.7349837662337662e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17-3.9265422077922079e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+6.0876623376623375e-04*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56+6.0876623376623375e-04*Jinv12*w28+5.5702110389610384e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12-2.5568181818181820e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-2.5568181818181820e-04*w48*Jinv01+5.6818181818181818e-05*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[27][13]=(0.0)*D;
  A[27][14]=(0.0)*D;
  A[27][15]=(4.3831168831168834e-04*Jinv00*w12-2.1915584415584417e-04*w40*Jinv12+7.3051948051948052e-05*Jinv22*w53-5.6615259740259744e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+4.3831168831168834e-04*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39+6.9399350649350649e-04*Jinv21*w44+5.6615259740259744e-04*w3*Jinv00+5.2962662337662338e-04*w32*Jinv20+6.2094155844155847e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23-4.9310064935064932e-04*Jinv22*w38+1.9480519480519481e-04*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12-3.0438311688311688e-05*w0*Jinv02-2.1915584415584417e-04*w43*Jinv10+6.9399350649350649e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55-4.3831168831168834e-04*w15*Jinv02+1.2175324675324675e-05*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+1.0957792207792207e-03*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35+1.9480519480519481e-04*w58*Jinv11+1.9480519480519481e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-4.2613636363636367e-05*w1*Jinv10-6.5746753246753243e-04*Jinv22*w20+6.9399350649350649e-04*w43*Jinv11+9.1314935064935063e-04*Jinv21*w41-4.3831168831168834e-04*w42*Jinv02+1.2175324675324675e-05*w11*Jinv22+2.7394480519480518e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01-2.4350649350649351e-04*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50-4.3831168831168834e-04*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.2962662337662338e-04*w31*Jinv10+7.3051948051948052e-05*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22+9.1314935064935063e-04*Jinv01*w39-3.0438311688311688e-05*Jinv22*w2-4.3831168831168834e-04*Jinv00*w18-5.2962662337662338e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11-3.4699675324675324e-04*Jinv21*w53+3.6525974025974026e-05*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-4.3831168831168834e-04*Jinv22*w44+5.6615259740259744e-04*w4*Jinv10+6.2094155844155847e-04*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20-2.1915584415584417e-04*Jinv12*w13-4.3831168831168834e-04*w19*Jinv10+3.1047077922077924e-04*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-5.6615259740259744e-04*w6*Jinv00-4.9310064935064932e-04*w37*Jinv12-2.1915584415584417e-04*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20+7.3051948051948052e-05*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+3.1047077922077924e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11+4.3831168831168834e-04*Jinv20*w14+6.2094155844155847e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54+7.3051948051948052e-05*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10-2.4350649350649351e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00-2.1915584415584417e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.2175324675324675e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01-5.6615259740259744e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-5.2962662337662338e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23-3.4699675324675324e-04*w52*Jinv11+3.6525974025974026e-05*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+3.6525974025974026e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-3.4699675324675324e-04*Jinv01*w51+3.6525974025974026e-05*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01+3.6525974025974026e-05*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-5.2962662337662338e-04*w37*Jinv10-2.5568181818181820e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-7.3051948051948055e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-6.5746753246753243e-04*w19*Jinv12+7.3051948051948052e-05*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12-6.5746753246753243e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8-1.1323051948051949e-03*Jinv02*w33-4.2613636363636367e-05*w0*Jinv00-4.3831168831168834e-04*Jinv21*w23-2.5568181818181820e-04*Jinv01*w54-2.5568181818181820e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2+3.6525974025974026e-05*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+5.2962662337662338e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-4.3831168831168834e-04*w20*Jinv20-7.3051948051948055e-04*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+9.1314935064935063e-04*w40*Jinv11-4.9310064935064932e-04*Jinv02*w36-4.3831168831168834e-04*Jinv12*w43+5.6615259740259744e-04*w5*Jinv20+3.1047077922077924e-04*Jinv01*w30+7.3051948051948052e-05*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56-7.3051948051948055e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11-2.1915584415584417e-04*w42*Jinv00-4.3831168831168834e-04*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-2.4350649350649351e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-3.0438311688311688e-05*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[27][16]=(0.0)*D;
  A[27][17]=(0.0)*D;
  A[27][18]=(2.1915584415584417e-04*Jinv00*w12-6.0876623376623375e-04*w28*Jinv10+3.9265422077922079e-04*Jinv20*w8+2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv21*w44-1.7349837662337662e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-4.2918019480519480e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23+2.1915584415584417e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+4.9310064935064932e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17+9.1314935064935065e-06*Jinv01*w45-1.4204545454545455e-05*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-4.7483766233766234e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01-1.4204545454545455e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+4.9310064935064932e-04*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-6.0876623376623375e-04*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-1.3697240259740259e-04*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01+4.0584415584415584e-05*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50-6.0876623376623375e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10-5.4788961038961036e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+6.5746753246753243e-04*Jinv00*w18+3.9265422077922079e-04*w36*Jinv00+7.4878246753246752e-04*Jinv21*w38-4.7483766233766234e-04*Jinv01*w24+2.7394480519480518e-04*w34*Jinv11+7.8530844155844158e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-1.7349837662337662e-04*w51*Jinv00-1.7349837662337662e-04*w4*Jinv10-4.2918019480519480e-04*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21+7.8530844155844158e-04*Jinv21*w20+3.9265422077922079e-04*w6*Jinv00+2.1915584415584417e-04*Jinv20*w56-1.0957792207792207e-03*w26*Jinv20-5.6818181818181818e-05*w11*Jinv20+2.7394480519480518e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+9.1314935064935065e-06*w46*Jinv11+2.1915584415584417e-04*Jinv20*w14-4.2918019480519480e-04*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10+4.0584415584415584e-05*Jinv21*w29+2.7394480519480518e-04*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00+7.8530844155844158e-04*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01+3.9265422077922079e-04*w7*Jinv10+9.1314935064935065e-06*w47*Jinv21+3.9265422077922079e-04*Jinv20*w38-4.2918019480519480e-04*w52*Jinv11-1.7349837662337662e-04*Jinv20*w53-5.4788961038961036e-04*w22*Jinv10+9.1314935064935065e-06*w22*Jinv11+9.1314935064935065e-06*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01-1.7349837662337662e-04*w52*Jinv10+7.4878246753246752e-04*w7*Jinv11-5.6818181818181818e-05*w9*Jinv00+3.9265422077922079e-04*w37*Jinv10-1.3697240259740259e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00-1.0957792207792208e-04*w49*Jinv11-1.3697240259740259e-04*Jinv21*w14-5.6818181818181818e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44-1.3697240259740259e-04*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20+9.1314935064935065e-06*Jinv21*w23-1.3697240259740259e-04*Jinv01*w54-1.3697240259740259e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+4.3831168831168834e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-1.7349837662337662e-04*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30-4.7483766233766234e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-1.0957792207792208e-04*Jinv21*w50+4.0584415584415584e-05*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[27][19]=(0.0)*D;
  A[27][20]=(0.0)*D;
  A[27][21]=(5.4788961038961036e-04*Jinv00*w12-1.5827922077922079e-03*w28*Jinv10+5.4788961038961036e-04*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10+5.4788961038961036e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00-9.1314935064935065e-06*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3-1.0957792207792207e-03*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12-6.4935064935064935e-05*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.4788961038961036e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17-1.1414366883116883e-03*Jinv01*w45+5.4788961038961036e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-9.1314935064935065e-06*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01-1.1414366883116883e-03*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-6.4935064935064935e-05*w1*Jinv10+5.4788961038961036e-04*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41+5.4788961038961036e-04*w42*Jinv02-1.5827922077922079e-03*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1+4.0178571428571428e-04*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-1.2987012987012987e-03*w28*Jinv11-9.1314935064935065e-06*Jinv20*w50-1.5827922077922079e-03*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.0957792207792207e-03*w22*Jinv12-9.1314935064935065e-06*w31*Jinv10-9.1314935064935065e-06*Jinv02*w3-1.0957792207792207e-03*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-6.4935064935064935e-05*Jinv22*w2+5.4788961038961036e-04*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45-1.1414366883116883e-03*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-1.8262987012987014e-04*w34*Jinv11+6.6659902597402602e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+5.4788961038961036e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+5.4788961038961036e-04*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10-2.1915584415584417e-04*w32*Jinv21+6.6659902597402602e-04*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+5.4788961038961036e-04*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-1.8262987012987014e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-1.1414366883116883e-03*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21+5.4788961038961036e-04*Jinv02*w54-9.1314935064935065e-06*w4*Jinv12-1.0957792207792207e-03*w21*Jinv02-9.1314935064935065e-06*w49*Jinv10-1.2987012987012987e-03*Jinv21*w29-1.8262987012987014e-04*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+6.6659902597402602e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9+1.1870941558441559e-04*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10-1.1414366883116883e-03*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38-1.0957792207792207e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53-9.1314935064935065e-06*Jinv22*w8-1.0957792207792207e-03*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50-1.1870941558441558e-03*w22*Jinv11-9.1314935064935065e-06*Jinv12*w7-1.1870941558441558e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10+1.1870941558441559e-04*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+6.6659902597402602e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00-1.5827922077922079e-03*Jinv02*w27-9.1314935064935065e-06*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-9.1314935064935065e-06*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+1.1870941558441559e-04*w49*Jinv11+4.0178571428571428e-04*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10+5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18+4.0178571428571428e-04*Jinv01*w12+1.1870941558441559e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-6.4935064935064935e-05*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.1870941558441558e-03*Jinv21*w23+6.6659902597402602e-04*Jinv01*w54+6.6659902597402602e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10-6.4935064935064935e-05*Jinv20*w2+5.4788961038961036e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9+5.4788961038961036e-04*Jinv20*w17-9.1314935064935065e-06*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10+5.4788961038961036e-04*w20*Jinv20-1.5827922077922079e-03*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+5.4788961038961036e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20-2.1915584415584417e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51+5.4788961038961036e-04*Jinv22*w56-1.5827922077922079e-03*Jinv12*w28-1.1414366883116883e-03*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+5.4788961038961036e-04*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12+1.1870941558441559e-04*Jinv21*w50-1.2987012987012987e-03*Jinv01*w27+1.1870941558441559e-04*w48*Jinv01-6.4935064935064935e-05*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[27][22]=(0.0)*D;
  A[27][23]=(0.0)*D;
  A[27][24]=(-5.4788961038961036e-04*Jinv00*w12+1.5827922077922079e-03*w28*Jinv10+9.1314935064935065e-06*Jinv20*w8-5.4788961038961036e-04*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+9.1314935064935065e-06*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20+1.2784090909090910e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23-5.4788961038961036e-04*Jinv00*w54-9.2329545454545453e-05*Jinv21*w59-9.2329545454545453e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01-5.6818181818181818e-05*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17-4.5657467532467534e-05*Jinv01*w45+6.4935064935064935e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00-9.2329545454545453e-05*w58*Jinv11-9.2329545454545453e-05*w57*Jinv01-9.1314935064935068e-05*Jinv21*w26-9.2329545454545453e-05*w0*Jinv01+6.4935064935064935e-05*w1*Jinv10-5.4788961038961036e-04*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41+1.5827922077922079e-03*Jinv00*w27-9.2329545454545453e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01+2.8409090909090908e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50+1.5827922077922079e-03*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-4.9310064935064932e-04*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10+1.0957792207792207e-03*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-5.4788961038961036e-04*Jinv00*w18+9.1314935064935065e-06*w36*Jinv00-2.1002435064935065e-04*Jinv21*w38-9.1314935064935068e-05*Jinv01*w24-6.7573051948051951e-04*w34*Jinv11-1.4610389610389610e-04*w19*Jinv11+1.2784090909090910e-04*Jinv21*w53+9.1314935064935065e-06*w51*Jinv00+9.1314935064935065e-06*w4*Jinv10+1.2784090909090910e-04*w4*Jinv11-5.4788961038961036e-04*w41*Jinv20-5.4788961038961036e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+6.3920454545454550e-05*w32*Jinv21-1.4610389610389610e-04*Jinv21*w20+9.1314935064935065e-06*w6*Jinv00-5.4788961038961036e-04*Jinv20*w56+1.0957792207792207e-03*w26*Jinv20+6.4935064935064935e-05*w11*Jinv20-6.7573051948051951e-04*Jinv21*w35+6.3920454545454550e-05*w31*Jinv11-4.5657467532467534e-05*w46*Jinv11-5.4788961038961036e-04*Jinv20*w14+1.2784090909090910e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10+2.8409090909090908e-04*Jinv21*w29-6.7573051948051951e-04*Jinv01*w33-5.4788961038961036e-04*w39*Jinv00-1.4610389610389610e-04*Jinv01*w18-2.1002435064935065e-04*w37*Jinv11-2.1002435064935065e-04*w6*Jinv01+9.1314935064935065e-06*w7*Jinv10-4.5657467532467534e-05*w47*Jinv21+9.1314935064935065e-06*Jinv20*w38+1.2784090909090910e-04*w52*Jinv11+9.1314935064935065e-06*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10-4.5657467532467534e-05*w22*Jinv11-4.5657467532467534e-05*w21*Jinv01+1.2784090909090910e-04*Jinv01*w51-2.1002435064935065e-04*w36*Jinv01+9.1314935064935065e-06*w52*Jinv10-2.1002435064935065e-04*w7*Jinv11+6.4935064935064935e-05*w9*Jinv00+9.1314935064935065e-06*w37*Jinv10+1.1870941558441559e-04*Jinv21*w56+1.0957792207792207e-03*w24*Jinv00+6.3920454545454550e-05*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.1870941558441559e-04*Jinv01*w12-2.1002435064935065e-04*Jinv21*w8+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20-4.5657467532467534e-05*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+1.0957792207792207e-03*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2-5.4788961038961036e-04*w15*Jinv00-5.6818181818181818e-05*Jinv01*w9-5.4788961038961036e-04*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00-5.4788961038961036e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+9.1314935064935065e-06*w5*Jinv20+6.3920454545454550e-05*Jinv01*w30-9.1314935064935068e-05*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20+6.3920454545454550e-05*Jinv21*w50+2.8409090909090908e-04*Jinv01*w27+6.3920454545454550e-05*w48*Jinv01-5.6818181818181818e-05*w10*Jinv11)*D;
  A[27][25]=(0.0)*D;
  A[27][26]=(0.0)*D;
  A[27][27]=(1.2175324675324675e-04*Jinv21*w44+2.0292207792207792e-05*Jinv01*w3+9.4696969696969697e-05*Jinv21*w59+9.4696969696969697e-05*Jinv21*w2+1.2175324675324675e-04*w42*Jinv01+9.4696969696969697e-05*w11*Jinv21+1.2175324675324675e-04*w16*Jinv11+1.2175324675324675e-04*Jinv21*w17+6.4935064935064935e-04*Jinv01*w45+9.4696969696969697e-05*w58*Jinv11+9.4696969696969697e-05*w57*Jinv01+6.4935064935064935e-04*Jinv21*w26+9.4696969696969697e-05*w0*Jinv01+1.2175324675324675e-04*w43*Jinv11+1.2175324675324675e-04*Jinv21*w41+9.4696969696969697e-05*Jinv11*w1-3.2467532467532468e-04*w13*Jinv11+1.2175324675324675e-04*w15*Jinv01+8.3874458874458877e-04*w28*Jinv11+1.2175324675324675e-04*Jinv01*w39+2.0292207792207792e-05*Jinv21*w38+6.4935064935064935e-04*Jinv01*w24+2.4350649350649351e-04*w34*Jinv11-3.2467532467532468e-04*w19*Jinv11+2.0292207792207792e-05*Jinv21*w53+2.0292207792207792e-05*w4*Jinv11+2.0292207792207792e-05*w32*Jinv21-3.2467532467532468e-04*Jinv21*w20+2.4350649350649351e-04*Jinv21*w35+2.0292207792207792e-05*w31*Jinv11+6.4935064935064935e-04*w46*Jinv11+2.0292207792207792e-05*w5*Jinv21+8.3874458874458877e-04*Jinv21*w29+2.4350649350649351e-04*Jinv01*w33-3.2467532467532468e-04*Jinv01*w18+2.0292207792207792e-05*w37*Jinv11+2.0292207792207792e-05*w6*Jinv01+6.4935064935064935e-04*w47*Jinv21+2.0292207792207792e-05*w52*Jinv11+6.4935064935064935e-04*w22*Jinv11+6.4935064935064935e-04*w21*Jinv01+2.0292207792207792e-05*Jinv01*w51+2.0292207792207792e-05*w36*Jinv01+2.0292207792207792e-05*w7*Jinv11-3.2467532467532468e-04*Jinv21*w56+2.0292207792207792e-05*w49*Jinv11-3.2467532467532468e-04*Jinv21*w14-3.2467532467532468e-04*Jinv01*w12+2.0292207792207792e-05*Jinv21*w8+6.4935064935064935e-04*Jinv21*w23-3.2467532467532468e-04*Jinv01*w54-3.2467532467532468e-04*w55*Jinv11+9.4696969696969697e-05*Jinv01*w9+1.2175324675324675e-04*w40*Jinv11+2.0292207792207792e-05*Jinv01*w30+6.4935064935064935e-04*w25*Jinv11+2.0292207792207792e-05*Jinv21*w50+8.3874458874458877e-04*Jinv01*w27+2.0292207792207792e-05*w48*Jinv01+9.4696969696969697e-05*w10*Jinv11)*D;
  A[27][28]=(0.0)*D;
  A[27][29]=(0.0)*D;
  A[27][30]=(-3.9265422077922079e-04*Jinv00*w12-4.0584415584415584e-05*w28*Jinv10-5.6615259740259744e-04*w40*Jinv12+3.1960227272727273e-04*Jinv22*w53+3.1960227272727273e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02-3.9265422077922079e-04*w13*Jinv10-5.6615259740259744e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44-3.7439123376623376e-04*w3*Jinv00-7.4878246753246752e-04*w32*Jinv20-3.7439123376623376e-04*Jinv01*w3+4.8396915584415583e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+3.1047077922077924e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+1.5726461038961039e-04*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17-1.8262987012987013e-05*Jinv01*w45+1.7349837662337662e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02+1.5523538961038962e-04*w48*Jinv00-6.9399350649350649e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+7.3051948051948052e-05*w43*Jinv11-1.8262987012987013e-05*w45*Jinv00-1.0592532467532468e-03*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-4.0584415584415584e-05*Jinv00*w27+9.2329545454545453e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11+1.5523538961038962e-04*Jinv20*w50-4.0584415584415584e-05*Jinv20*w29-1.8262987012987013e-05*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-9.6793831168831166e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-7.4878246753246752e-04*w31*Jinv10+3.7439123376623376e-04*Jinv02*w3+4.8396915584415583e-04*w21*Jinv00-1.0592532467532468e-03*Jinv01*w39+1.5726461038961039e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-9.6793831168831166e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-6.9399350649350649e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44-3.7439123376623376e-04*w4*Jinv10-3.7439123376623376e-04*w4*Jinv11-1.0592532467532468e-03*w41*Jinv20+3.9265422077922079e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-9.6793831168831166e-04*w34*Jinv10-7.4878246753246752e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+3.1960227272727273e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+3.1047077922077924e-04*Jinv20*w56-5.6615259740259744e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+3.1960227272727273e-04*w52*Jinv12-9.6793831168831166e-04*Jinv21*w35-7.4878246753246752e-04*w31*Jinv11-1.8262987012987013e-05*w46*Jinv11-3.9265422077922079e-04*Jinv20*w14-3.7439123376623376e-04*w5*Jinv21+1.7349837662337662e-04*Jinv02*w54+3.7439123376623376e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02+1.5523538961038962e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-9.6793831168831166e-04*Jinv01*w33-1.0592532467532468e-03*w39*Jinv00+3.9265422077922079e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+3.1960227272727273e-04*w6*Jinv01+3.1960227272727273e-04*w7*Jinv10-1.8262987012987013e-05*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11-9.1314935064935065e-06*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8+4.8396915584415583e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+4.8396915584415583e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+4.8396915584415583e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51-2.7394480519480518e-04*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10+3.1960227272727273e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10+3.1047077922077924e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.0957792207792208e-04*w6*Jinv02+3.7439123376623376e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+1.5523538961038962e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.9265422077922079e-04*Jinv01*w12+3.1960227272727273e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-6.9399350649350649e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-9.6793831168831166e-04*w35*Jinv20+4.8396915584415583e-04*Jinv21*w23+3.1047077922077924e-04*Jinv01*w54+3.1047077922077924e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-7.4878246753246752e-04*w30*Jinv00-1.0592532467532468e-03*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w14-1.0592532467532468e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43-3.7439123376623376e-04*w5*Jinv20-7.4878246753246752e-04*Jinv01*w30+3.1960227272727273e-04*Jinv02*w51+1.7349837662337662e-04*Jinv22*w56-9.1314935064935065e-06*w25*Jinv11+3.1047077922077924e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00-1.8262987012987013e-05*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12+1.5523538961038962e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+1.5523538961038962e-04*w48*Jinv01+1.5726461038961039e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[27][31]=(0.0)*D;
  A[27][32]=(0.0)*D;
  A[27][33]=(5.2962662337662338e-04*Jinv00*w12-9.6793831168831166e-04*Jinv22*w53-6.9399350649350649e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02+5.2962662337662338e-04*w13*Jinv10-2.2646103896103898e-03*Jinv21*w44+6.9399350649350649e-04*w3*Jinv00+9.6793831168831166e-04*w32*Jinv20+4.2004870129870131e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-3.0438311688311688e-05*Jinv21*w59-1.9480519480519481e-04*w0*Jinv02-2.2524350649350650e-04*Jinv21*w2-1.1323051948051949e-03*w43*Jinv10-2.2646103896103898e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17+9.8620129870129864e-04*Jinv01*w45-5.2962662337662338e-04*Jinv12*w55+1.1323051948051949e-03*w15*Jinv02+2.7394480519480518e-04*w48*Jinv00-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01+9.8620129870129864e-04*Jinv21*w26-2.2524350649350650e-04*w0*Jinv01-1.9480519480519481e-04*w1*Jinv10-2.2646103896103898e-03*w43*Jinv11-1.1323051948051949e-03*Jinv21*w41-1.1323051948051949e-03*w42*Jinv02-2.2524350649350650e-04*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-4.8701298701298701e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+1.1323051948051949e-03*Jinv22*w17-4.9310064935064932e-04*w22*Jinv12+9.6793831168831166e-04*w31*Jinv10+9.6793831168831166e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+6.9399350649350649e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39-1.9480519480519481e-04*Jinv22*w2-5.2962662337662338e-04*Jinv00*w18-9.6793831168831166e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+9.8620129870129864e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59-2.7759740259740259e-03*w34*Jinv11-4.9310064935064932e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00-1.1323051948051949e-03*Jinv22*w44+6.9399350649350649e-04*w4*Jinv10+4.2004870129870131e-04*w4*Jinv11+1.1323051948051949e-03*w41*Jinv20+5.2962662337662338e-04*Jinv12*w13-5.2962662337662338e-04*w19*Jinv10+4.2004870129870131e-04*w32*Jinv21-4.9310064935064932e-04*Jinv21*w20-6.9399350649350649e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+4.9310064935064932e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20-9.6793831168831166e-04*w52*Jinv12-2.7759740259740259e-03*Jinv21*w35+4.2004870129870131e-04*w31*Jinv11+9.8620129870129864e-04*w46*Jinv11+5.2962662337662338e-04*Jinv20*w14+4.2004870129870131e-04*w5*Jinv21-5.2962662337662338e-04*Jinv02*w54+9.6793831168831166e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-4.8701298701298701e-04*Jinv21*w29-2.7759740259740259e-03*Jinv01*w33+1.1323051948051949e-03*w39*Jinv00+5.2962662337662338e-04*Jinv02*w12-4.9310064935064932e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-2.7394480519480518e-04*w6*Jinv01-6.9399350649350649e-04*w7*Jinv10+9.8620129870129864e-04*w47*Jinv21-9.6793831168831166e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+6.9399350649350649e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-6.9399350649350649e-04*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+6.9399350649350649e-04*Jinv02*w30-6.9399350649350649e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-9.6793831168831166e-04*w37*Jinv10-4.9310064935064932e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02+9.6793831168831166e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10-1.1323051948051949e-03*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12+3.6525974025974026e-05*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-1.9480519480519481e-04*w0*Jinv00+4.9310064935064932e-04*Jinv21*w23-4.9310064935064932e-04*Jinv01*w54-4.9310064935064932e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2-6.9399350649350649e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9+9.6793831168831166e-04*w30*Jinv00+1.1323051948051949e-03*w40*Jinv10-5.2962662337662338e-04*w20*Jinv20+5.2962662337662338e-04*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36-1.1323051948051949e-03*Jinv12*w43+6.9399350649350649e-04*w5*Jinv20+4.2004870129870131e-04*Jinv01*w30-9.6793831168831166e-04*Jinv02*w51-5.2962662337662338e-04*Jinv22*w56+9.8620129870129864e-04*w25*Jinv11-1.1323051948051949e-03*w42*Jinv00+1.1323051948051949e-03*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50-4.8701298701298701e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01-1.9480519480519481e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[27][34]=(0.0)*D;
  A[27][35]=(0.0)*D;
  A[27][36]=(-1.3697240259740259e-04*Jinv00*w12+4.0584415584415584e-05*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.2918019480519480e-04*Jinv22*w53+3.7439123376623376e-04*Jinv20*w8+1.0247564935064935e-04*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-3.1960227272727273e-04*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23+7.4878246753246752e-04*Jinv22*w38-3.1047077922077924e-04*Jinv00*w54-4.7483766233766234e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0592532467532468e-03*w43*Jinv10-1.3697240259740259e-04*Jinv12*w55-1.4204545454545455e-05*w58*Jinv10+4.9310064935064932e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-4.2918019480519480e-04*w48*Jinv00-4.7483766233766234e-04*Jinv02*w24+2.7394480519480518e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.8530844155844158e-04*Jinv22*w20+1.8262987012987013e-05*w45*Jinv00+4.9310064935064932e-04*w42*Jinv02+4.0584415584415584e-05*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-4.2918019480519480e-04*Jinv20*w50+4.0584415584415584e-05*Jinv20*w29+1.8262987012987013e-05*w46*Jinv10+4.9310064935064932e-04*Jinv22*w17+9.6793831168831166e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10-4.2918019480519480e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-1.0957792207792208e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.9265422077922079e-04*Jinv00*w18+7.4878246753246752e-04*w36*Jinv00+9.1314935064935065e-06*Jinv02*w45+1.0247564935064935e-04*Jinv22*w59-1.5523538961038962e-04*w51*Jinv00+2.7394480519480518e-04*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-3.1960227272727273e-04*w4*Jinv10-7.3051948051948052e-05*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13+3.9265422077922079e-04*w19*Jinv10+9.6793831168831166e-04*w34*Jinv10+3.7439123376623376e-04*w6*Jinv00+7.4878246753246752e-04*w37*Jinv12-3.1047077922077924e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-4.8396915584415583e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20-4.2918019480519480e-04*w52*Jinv12-4.7483766233766234e-04*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14-1.3697240259740259e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-4.2918019480519480e-04*w49*Jinv10-7.3051948051948052e-05*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.7439123376623376e-04*w7*Jinv10+7.4878246753246752e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-1.0957792207792208e-04*w31*Jinv12+9.1314935064935065e-06*w46*Jinv12-1.5523538961038962e-04*Jinv20*w53+7.4878246753246752e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10-1.0957792207792208e-04*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-1.0957792207792208e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49-1.5523538961038962e-04*w52*Jinv10-9.2329545454545453e-05*w9*Jinv00+7.4878246753246752e-04*w37*Jinv10-4.8396915584415583e-04*w24*Jinv00+4.0584415584415584e-05*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+7.8530844155844158e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+9.1314935064935065e-06*w47*Jinv22-9.2329545454545453e-05*w10*Jinv10+1.0592532467532468e-03*Jinv20*w44+1.0247564935064935e-04*w58*Jinv12+7.8530844155844158e-04*Jinv02*w18-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+2.7394480519480518e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+9.6793831168831166e-04*w35*Jinv20-4.8396915584415583e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00-1.0957792207792208e-04*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+3.9265422077922079e-04*w20*Jinv20+4.0584415584415584e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14+7.4878246753246752e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-3.1960227272727273e-04*w5*Jinv20-4.2918019480519480e-04*Jinv02*w51-1.3697240259740259e-04*Jinv22*w56+4.0584415584415584e-05*Jinv12*w28-3.1047077922077924e-04*w55*Jinv10+1.0592532467532468e-03*w42*Jinv00+1.8262987012987013e-05*w47*Jinv20+4.9310064935064932e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[27][37]=(0.0)*D;
  A[27][38]=(0.0)*D;
  A[27][39]=(-2.1915584415584417e-04*Jinv00*w12-7.3051948051948055e-04*w28*Jinv10-5.2962662337662338e-04*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-2.1915584415584417e-04*w13*Jinv10+6.9399350649350649e-04*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00+7.3051948051948052e-05*w32*Jinv20+3.1047077922077924e-04*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23+3.6525974025974026e-05*Jinv22*w38-6.5746753246753243e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-4.2613636363636367e-05*w0*Jinv02-4.3831168831168834e-04*w43*Jinv10+6.9399350649350649e-04*w42*Jinv01+1.9480519480519481e-04*w11*Jinv21+9.1314935064935063e-04*w16*Jinv11+9.1314935064935063e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45-4.3831168831168834e-04*Jinv12*w55+1.2175324675324675e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02-4.9310064935064932e-04*w48*Jinv00+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-3.0438311688311688e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10+6.9399350649350649e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02-7.3051948051948055e-04*Jinv00*w27+2.7394480519480518e-04*w13*Jinv11+9.1314935064935063e-04*w15*Jinv01-2.4350649350649351e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-7.3051948051948055e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12+7.3051948051948052e-05*w31*Jinv10+5.2962662337662338e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+5.6615259740259744e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-4.2613636363636367e-05*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18+7.3051948051948052e-05*w36*Jinv00-3.4699675324675324e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-2.5568181818181820e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-4.9310064935064932e-04*w51*Jinv00-2.1915584415584417e-04*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+3.1047077922077924e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+4.3831168831168834e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10+6.2094155844155847e-04*w32*Jinv21-2.5568181818181820e-04*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00+3.6525974025974026e-05*w37*Jinv12-6.5746753246753243e-04*Jinv20*w56+5.4788961038961036e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20-5.2962662337662338e-04*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+6.2094155844155847e-04*w31*Jinv11+1.0957792207792208e-04*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14+3.1047077922077924e-04*w5*Jinv21-4.3831168831168834e-04*Jinv02*w54+5.2962662337662338e-04*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-2.4350649350649351e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+4.3831168831168834e-04*Jinv02*w12-2.5568181818181820e-04*Jinv01*w18-3.4699675324675324e-04*w37*Jinv11-3.8352272727272725e-04*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+7.3051948051948052e-05*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11+5.6615259740259744e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12-4.9310064935064932e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-5.6615259740259744e-04*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51+5.6615259740259744e-04*Jinv02*w30-5.6615259740259744e-04*Jinv12*w49-3.4699675324675324e-04*w36*Jinv01-4.9310064935064932e-04*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00+7.3051948051948052e-05*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-3.6525974025974026e-05*w6*Jinv02+5.2962662337662338e-04*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-4.3831168831168834e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12+2.7394480519480518e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+1.2175324675324675e-05*w57*Jinv00+1.2175324675324675e-05*Jinv20*w59-3.0438311688311688e-05*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10-3.0438311688311688e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-5.6615259740259744e-04*w48*Jinv02+1.9480519480519481e-04*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+7.3051948051948052e-05*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+4.3831168831168834e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11+3.6525974025974026e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+6.2094155844155847e-04*Jinv01*w30-5.2962662337662338e-04*Jinv02*w51-4.3831168831168834e-04*Jinv22*w56+5.4788961038961042e-05*w25*Jinv11-6.5746753246753243e-04*w55*Jinv10-4.3831168831168834e-04*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50-2.4350649350649351e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01-4.2613636363636367e-05*Jinv12*w1+1.9480519480519481e-04*w10*Jinv11)*D;
  A[27][40]=(0.0)*D;
  A[27][41]=(0.0)*D;
  A[27][42]=(2.1915584415584417e-04*Jinv00*w12+7.3051948051948055e-04*w28*Jinv10+2.1915584415584417e-04*w40*Jinv12-7.3051948051948052e-05*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+3.0438311688311688e-05*w57*Jinv02+2.1915584415584417e-04*w13*Jinv10+2.1915584415584417e-04*Jinv02*w39+2.2646103896103898e-03*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00-7.3051948051948052e-05*w32*Jinv20-4.2004870129870131e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38+6.5746753246753243e-04*Jinv00*w54+3.0438311688311688e-05*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02+2.2524350649350650e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+2.2646103896103898e-03*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17-9.8620129870129864e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.2175324675324675e-05*w58*Jinv10+4.3831168831168834e-04*w15*Jinv02-1.2175324675324675e-05*w10*Jinv12+4.9310064935064932e-04*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-9.8620129870129864e-04*Jinv21*w26+2.2524350649350650e-04*w0*Jinv01+3.0438311688311688e-05*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+6.5746753246753243e-04*Jinv22*w20+2.2646103896103898e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+7.3051948051948055e-04*Jinv00*w27-1.2175324675324675e-05*w11*Jinv22+2.2524350649350650e-04*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+4.8701298701298701e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w50+7.3051948051948055e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+4.3831168831168834e-04*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12-7.3051948051948052e-05*w31*Jinv10-7.3051948051948052e-05*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+3.0438311688311688e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18-7.3051948051948052e-05*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-9.8620129870129864e-04*Jinv01*w24+3.0438311688311688e-05*Jinv22*w59+2.7759740259740259e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10-4.2004870129870131e-04*w4*Jinv11+4.3831168831168834e-04*w41*Jinv20+2.1915584415584417e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-4.2004870129870131e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12+6.5746753246753243e-04*Jinv20*w56+2.1915584415584417e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+3.0438311688311688e-05*w11*Jinv20-7.3051948051948052e-05*w52*Jinv12+2.7759740259740259e-03*Jinv21*w35-4.2004870129870131e-04*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-9.8620129870129864e-04*w46*Jinv11+2.1915584415584417e-04*Jinv20*w14-4.2004870129870131e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54-7.3051948051948052e-05*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02+4.9310064935064932e-04*w49*Jinv10+4.8701298701298701e-04*Jinv21*w29+2.7759740259740259e-03*Jinv01*w33+4.3831168831168834e-04*w39*Jinv00+2.1915584415584417e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.2175324675324675e-05*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10-9.8620129870129864e-04*w47*Jinv21-7.3051948051948052e-05*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53+4.9310064935064932e-04*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11+4.9310064935064932e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+3.0438311688311688e-05*w9*Jinv00-7.3051948051948052e-05*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+7.3051948051948055e-04*Jinv02*w27+4.9310064935064932e-04*w6*Jinv02+6.5746753246753243e-04*w19*Jinv12-7.3051948051948052e-05*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+3.0438311688311688e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+3.0438311688311688e-05*w58*Jinv12+6.5746753246753243e-04*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.2175324675324675e-05*w57*Jinv00-1.2175324675324675e-05*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+3.0438311688311688e-05*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+3.0438311688311688e-05*Jinv01*w9+2.1915584415584417e-04*Jinv20*w17-7.3051948051948052e-05*w30*Jinv00+4.3831168831168834e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+7.3051948051948055e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20-4.2004870129870131e-04*Jinv01*w30-7.3051948051948052e-05*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+7.3051948051948055e-04*Jinv12*w28-9.8620129870129864e-04*w25*Jinv11+6.5746753246753243e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+4.3831168831168834e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+4.8701298701298701e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+3.0438311688311688e-05*Jinv12*w1+3.0438311688311688e-05*w10*Jinv11)*D;
  A[27][43]=(0.0)*D;
  A[27][44]=(0.0)*D;
  A[27][45]=(-5.4788961038961036e-04*w40*Jinv12+9.1314935064935065e-06*Jinv22*w53+6.4935064935064935e-05*w57*Jinv02-5.4788961038961036e-04*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+6.3920454545454550e-05*Jinv01*w3+9.1314935064935065e-06*Jinv22*w38-5.6818181818181818e-05*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-9.2329545454545453e-05*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01-9.2329545454545453e-05*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17-9.1314935064935068e-05*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12+1.0957792207792207e-03*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-5.6818181818181818e-05*w58*Jinv11-5.6818181818181818e-05*w57*Jinv01-4.5657467532467534e-05*Jinv21*w26-9.2329545454545453e-05*w0*Jinv01-5.4788961038961036e-04*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22-9.2329545454545453e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01+2.8409090909090908e-04*w28*Jinv11-5.4788961038961036e-04*Jinv22*w17+1.0957792207792207e-03*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3+9.1314935064935065e-06*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39+6.4935064935064935e-05*Jinv22*w2+1.2784090909090910e-04*Jinv21*w38+1.0957792207792207e-03*Jinv02*w45-4.5657467532467534e-05*Jinv01*w24+6.4935064935064935e-05*Jinv22*w59-6.7573051948051951e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11-2.1002435064935065e-04*Jinv21*w53-4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+6.3920454545454550e-05*w4*Jinv11-5.4788961038961036e-04*Jinv12*w13+1.2784090909090910e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+9.1314935064935065e-06*w37*Jinv12-5.4788961038961036e-04*Jinv22*w41+9.1314935064935065e-06*w52*Jinv12-6.7573051948051951e-04*Jinv21*w35+1.2784090909090910e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26-9.1314935064935068e-05*w46*Jinv11+6.3920454545454550e-05*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+2.8409090909090908e-04*Jinv21*w29-6.7573051948051951e-04*Jinv01*w33-5.4788961038961036e-04*Jinv02*w12+1.1870941558441559e-04*Jinv01*w18+1.2784090909090910e-04*w37*Jinv11+6.4935064935064935e-05*Jinv02*w9+6.3920454545454550e-05*w6*Jinv01-9.1314935064935068e-05*w47*Jinv21+1.0957792207792207e-03*Jinv22*w23-2.1002435064935065e-04*w52*Jinv11+9.1314935064935065e-06*w31*Jinv12+1.0957792207792207e-03*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8+9.1314935064935065e-06*Jinv22*w50-4.5657467532467534e-05*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7-4.5657467532467534e-05*w21*Jinv01-2.1002435064935065e-04*Jinv01*w51+9.1314935064935065e-06*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49+1.2784090909090910e-04*w36*Jinv01+6.3920454545454550e-05*w7*Jinv11-1.4610389610389610e-04*Jinv21*w56+1.5827922077922079e-03*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5+1.0957792207792207e-03*w47*Jinv22-2.1002435064935065e-04*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+6.4935064935064935e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+1.1870941558441559e-04*Jinv01*w12+6.3920454545454550e-05*Jinv21*w8-4.9310064935064932e-04*Jinv02*w33-4.5657467532467534e-05*Jinv21*w23-1.4610389610389610e-04*Jinv01*w54-1.4610389610389610e-04*w55*Jinv11+9.1314935064935065e-06*w48*Jinv02-9.2329545454545453e-05*Jinv01*w9+1.5827922077922079e-03*Jinv22*w29-5.4788961038961036e-04*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+9.1314935064935065e-06*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.2784090909090910e-04*Jinv01*w30+9.1314935064935065e-06*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+1.5827922077922079e-03*Jinv12*w28-4.5657467532467534e-05*w25*Jinv11-5.4788961038961036e-04*w16*Jinv12-2.1002435064935065e-04*Jinv21*w50+2.8409090909090908e-04*Jinv01*w27-2.1002435064935065e-04*w48*Jinv01+6.4935064935064935e-05*Jinv12*w1-9.2329545454545453e-05*w10*Jinv11)*D;
  A[27][46]=(0.0)*D;
  A[27][47]=(0.0)*D;
  A[27][48]=(1.3697240259740259e-04*Jinv00*w12-4.0584415584415584e-05*w28*Jinv10+5.6615259740259744e-04*w40*Jinv12-3.7439123376623376e-04*Jinv22*w53+1.0957792207792208e-04*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02+1.3697240259740259e-04*w13*Jinv10+5.6615259740259744e-04*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+4.2918019480519480e-04*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3-9.1314935064935065e-06*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38-7.8530844155844158e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-1.1668019480519480e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.7483766233766234e-04*Jinv01*w45-3.9265422077922079e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02-7.4878246753246752e-04*w48*Jinv00+6.9399350649350649e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-4.9310064935064932e-04*w43*Jinv11+4.7483766233766234e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-4.0584415584415584e-05*Jinv00*w27-1.0247564935064935e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-7.4878246753246752e-04*Jinv20*w50-4.0584415584415584e-05*Jinv20*w29+4.7483766233766234e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-2.7394480519480518e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12+4.2918019480519480e-04*w31*Jinv10-3.1960227272727273e-04*Jinv02*w3-9.1314935064935065e-06*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39-1.1668019480519480e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-2.7394480519480518e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00+6.9399350649350649e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20-1.7349837662337662e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-2.7394480519480518e-04*w34*Jinv10+4.2918019480519480e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+1.0957792207792208e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12-7.8530844155844158e-04*Jinv20*w56+5.6615259740259744e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-3.7439123376623376e-04*w52*Jinv12-2.7394480519480518e-04*Jinv21*w35+4.2918019480519480e-04*w31*Jinv11+4.7483766233766234e-04*w46*Jinv11+1.3697240259740259e-04*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21-3.9265422077922079e-04*Jinv02*w54-3.1960227272727273e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-7.4878246753246752e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-2.7394480519480518e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00-1.7349837662337662e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.0957792207792208e-04*w6*Jinv01+1.0957792207792208e-04*w7*Jinv10+4.7483766233766234e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8-9.1314935064935065e-06*w22*Jinv10-9.1314935064935065e-06*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-7.8530844155844158e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.0957792207792208e-04*w6*Jinv02-3.1960227272727273e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22-7.4878246753246752e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.3697240259740259e-04*Jinv01*w12+1.0957792207792208e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+6.9399350649350649e-04*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00-2.7394480519480518e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-7.8530844155844158e-04*Jinv01*w54-7.8530844155844158e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-1.0247564935064935e-04*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.2918019480519480e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20-1.7349837662337662e-04*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+4.2918019480519480e-04*Jinv01*w30-3.7439123376623376e-04*Jinv02*w51-3.9265422077922079e-04*Jinv22*w56-9.1314935064935065e-06*w25*Jinv11-7.8530844155844158e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00+4.7483766233766234e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12-7.4878246753246752e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27-7.4878246753246752e-04*w48*Jinv01-1.1668019480519480e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[27][49]=(0.0)*D;
  A[27][50]=(0.0)*D;
  A[27][51]=(-1.3697240259740259e-04*Jinv00*w12+4.0584415584415584e-05*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+7.4878246753246752e-04*Jinv22*w53-1.0957792207792208e-04*Jinv20*w8-9.2329545454545453e-05*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39-1.0957792207792208e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23-1.5523538961038962e-04*Jinv22*w38+7.8530844155844158e-04*Jinv00*w54+1.8262987012987013e-05*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+4.9310064935064932e-04*w43*Jinv10+3.9265422077922079e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10-7.3051948051948052e-05*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+7.4878246753246752e-04*w48*Jinv00+1.8262987012987013e-05*Jinv02*w24+9.6793831168831166e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-3.1047077922077924e-04*Jinv22*w20-4.7483766233766234e-04*w45*Jinv00+1.0592532467532468e-03*w42*Jinv02+4.0584415584415584e-05*Jinv00*w27-1.4204545454545455e-05*w11*Jinv22+7.4878246753246752e-04*Jinv20*w50+4.0584415584415584e-05*Jinv20*w29-4.7483766233766234e-04*w46*Jinv10-7.3051948051948052e-05*Jinv22*w17+2.7394480519480518e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-4.2918019480519480e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-3.1960227272727273e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2-1.3697240259740259e-04*Jinv00*w18-4.2918019480519480e-04*w36*Jinv00-4.8396915584415583e-04*Jinv02*w45-9.2329545454545453e-05*Jinv22*w59+7.4878246753246752e-04*w51*Jinv00+9.6793831168831166e-04*Jinv12*w34+1.0592532467532468e-03*Jinv22*w44-1.0957792207792208e-04*w4*Jinv10+4.9310064935064932e-04*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13-1.3697240259740259e-04*w19*Jinv10+2.7394480519480518e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00-1.5523538961038962e-04*w37*Jinv12+7.8530844155844158e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41+9.1314935064935065e-06*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20+7.4878246753246752e-04*w52*Jinv12+1.8262987012987013e-05*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14+3.9265422077922079e-04*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02+7.4878246753246752e-04*w49*Jinv10+4.9310064935064932e-04*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-1.0957792207792208e-04*w7*Jinv10-4.2918019480519480e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-3.1960227272727273e-04*w31*Jinv12-4.8396915584415583e-04*w46*Jinv12+7.4878246753246752e-04*Jinv20*w53-4.2918019480519480e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+3.7439123376623376e-04*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7-3.1960227272727273e-04*Jinv02*w30+3.7439123376623376e-04*Jinv12*w49+7.4878246753246752e-04*w52*Jinv10+1.0247564935064935e-04*w9*Jinv00-4.2918019480519480e-04*w37*Jinv10+9.1314935064935065e-06*w24*Jinv00+4.0584415584415584e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-3.1047077922077924e-04*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5-4.8396915584415583e-04*w47*Jinv22+1.0247564935064935e-04*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-9.2329545454545453e-05*w58*Jinv12-3.1047077922077924e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+9.6793831168831166e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+2.7394480519480518e-04*w35*Jinv20+9.1314935064935065e-06*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.7439123376623376e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-1.3697240259740259e-04*w20*Jinv20+4.0584415584415584e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14-1.5523538961038962e-04*Jinv02*w36+1.0592532467532468e-03*Jinv12*w43-1.0957792207792208e-04*w5*Jinv20+7.4878246753246752e-04*Jinv02*w51+3.9265422077922079e-04*Jinv22*w56+4.0584415584415584e-05*Jinv12*w28+7.8530844155844158e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00-4.7483766233766234e-04*w47*Jinv20-7.3051948051948052e-05*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[27][52]=(0.0)*D;
  A[27][53]=(0.0)*D;
  A[27][54]=(4.3831168831168834e-04*w40*Jinv12+3.9265422077922079e-04*Jinv22*w53-5.6818181818181818e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39+4.9310064935064932e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3-1.7349837662337662e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.7483766233766234e-04*Jinv01*w45+6.5746753246753243e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12-5.4788961038961036e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01+9.1314935064935065e-06*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+2.1915584415584417e-04*Jinv22*w20+4.9310064935064932e-04*w43*Jinv11+4.9310064935064932e-04*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-1.3697240259740259e-04*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+4.0584415584415584e-05*w28*Jinv11+2.1915584415584417e-04*Jinv22*w17-5.4788961038961036e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-1.7349837662337662e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-1.4204545454545455e-05*Jinv22*w2-4.2918019480519480e-04*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45+9.1314935064935065e-06*Jinv01*w24-5.6818181818181818e-05*Jinv22*w59+2.7394480519480518e-04*w34*Jinv11-1.3697240259740259e-04*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-1.0957792207792208e-04*w4*Jinv11+2.1915584415584417e-04*Jinv12*w13-4.2918019480519480e-04*w32*Jinv21-1.3697240259740259e-04*Jinv21*w20-1.7349837662337662e-04*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41+3.9265422077922079e-04*w52*Jinv12+2.7394480519480518e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-4.7483766233766234e-04*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21+6.5746753246753243e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02+4.0584415584415584e-05*Jinv21*w29+2.7394480519480518e-04*Jinv01*w33+2.1915584415584417e-04*Jinv02*w12-1.3697240259740259e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-1.4204545454545455e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01-4.7483766233766234e-04*w47*Jinv21-5.4788961038961036e-04*Jinv22*w23+7.4878246753246752e-04*w52*Jinv11-1.7349837662337662e-04*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8+3.9265422077922079e-04*Jinv22*w50+9.1314935064935065e-06*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+9.1314935064935065e-06*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-1.7349837662337662e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49-4.2918019480519480e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11+7.8530844155844158e-04*Jinv21*w56-6.0876623376623375e-04*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+7.4878246753246752e-04*w49*Jinv11-1.3697240259740259e-04*Jinv21*w14-5.6818181818181818e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-1.3697240259740259e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33+9.1314935064935065e-06*Jinv21*w23+7.8530844155844158e-04*Jinv01*w54+7.8530844155844158e-04*w55*Jinv11+3.9265422077922079e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9-6.0876623376623375e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-1.7349837662337662e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-4.2918019480519480e-04*Jinv01*w30+3.9265422077922079e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-6.0876623376623375e-04*Jinv12*w28+9.1314935064935065e-06*w25*Jinv11+2.1915584415584417e-04*w16*Jinv12+7.4878246753246752e-04*Jinv21*w50+4.0584415584415584e-05*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01-1.4204545454545455e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[27][55]=(0.0)*D;
  A[27][56]=(0.0)*D;
  A[27][57]=(-4.2613636363636367e-05*w40*Jinv12+1.5726461038961039e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-4.2613636363636367e-05*Jinv02*w39-1.1668019480519480e-04*Jinv22*w38+6.4935064935064935e-05*w25*Jinv12+5.4112554112554113e-05*w0*Jinv02+2.0495129870129870e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+5.4112554112554113e-05*w10*Jinv12+6.4935064935064935e-05*Jinv02*w24-1.9480519480519481e-04*Jinv22*w35-1.4204545454545455e-05*Jinv22*w20-4.2613636363636367e-05*w42*Jinv02+5.4112554112554113e-05*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17+6.4935064935064935e-05*w22*Jinv12+1.0247564935064935e-04*Jinv02*w3-1.1668019480519480e-04*w32*Jinv22+5.4112554112554113e-05*Jinv22*w2+8.3198051948051951e-05*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-1.9480519480519481e-04*Jinv12*w34-4.2613636363636367e-05*Jinv22*w44-1.4204545454545455e-05*Jinv12*w13-1.1668019480519480e-04*w37*Jinv12-4.2613636363636367e-05*Jinv22*w41+1.5726461038961039e-04*w52*Jinv12+6.4935064935064935e-05*Jinv22*w26+2.0495129870129870e-04*Jinv02*w54+1.0247564935064935e-04*w4*Jinv12+6.4935064935064935e-05*w21*Jinv02-1.4204545454545455e-05*Jinv02*w12+5.4112554112554113e-05*Jinv02*w9+6.4935064935064935e-05*Jinv22*w23-1.1668019480519480e-04*w31*Jinv12+8.3198051948051951e-05*w46*Jinv12+1.0247564935064935e-04*Jinv22*w8+1.5726461038961039e-04*Jinv22*w50+1.0247564935064935e-04*Jinv12*w7-1.1668019480519480e-04*Jinv02*w30+1.5726461038961039e-04*Jinv12*w49+1.8939393939393939e-04*Jinv02*w27+1.0247564935064935e-04*w6*Jinv02-1.4204545454545455e-05*w19*Jinv12+1.0247564935064935e-04*Jinv22*w5+8.3198051948051951e-05*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12-1.4204545454545455e-05*Jinv02*w18-1.9480519480519481e-04*Jinv02*w33+1.5726461038961039e-04*w48*Jinv02+1.8939393939393939e-04*Jinv22*w29-1.4204545454545455e-05*Jinv22*w14-1.1668019480519480e-04*Jinv02*w36-4.2613636363636367e-05*Jinv12*w43+1.5726461038961039e-04*Jinv02*w51+2.0495129870129870e-04*Jinv22*w56+1.8939393939393939e-04*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+5.4112554112554113e-05*Jinv12*w1)*D;
  A[27][58]=(0.0)*D;
  A[27][59]=(0.0)*D;
  A[28][0]=(0.0)*D;
  A[28][1]=(-2.0495129870129870e-04*Jinv00*w12-1.8939393939393939e-04*w28*Jinv10+4.2613636363636367e-05*w40*Jinv12-1.0247564935064935e-04*Jinv22*w53+1.1668019480519480e-04*Jinv20*w8-5.4112554112554113e-05*w57*Jinv02-2.0495129870129870e-04*w13*Jinv10+4.2613636363636367e-05*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-1.5726461038961039e-04*w3*Jinv00-1.5726461038961039e-04*w32*Jinv20-1.5726461038961039e-04*Jinv01*w3-8.3198051948051951e-05*Jinv20*w23-1.0247564935064935e-04*Jinv22*w38+1.4204545454545455e-05*Jinv00*w54-5.4112554112554113e-05*Jinv21*w59-6.4935064935064935e-05*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-5.4112554112554113e-05*w11*Jinv21+4.2613636363636367e-05*w16*Jinv11+4.2613636363636367e-05*Jinv21*w17-6.4935064935064935e-05*Jinv01*w45+1.4204545454545455e-05*Jinv12*w55-5.4112554112554113e-05*w58*Jinv10+4.2613636363636367e-05*w15*Jinv02-5.4112554112554113e-05*w10*Jinv12+1.1668019480519480e-04*w48*Jinv00-6.4935064935064935e-05*Jinv02*w24+1.9480519480519481e-04*Jinv22*w35-5.4112554112554113e-05*w58*Jinv11-5.4112554112554113e-05*w57*Jinv01-6.4935064935064935e-05*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+4.2613636363636367e-05*w16*Jinv10+1.4204545454545455e-05*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11-6.4935064935064935e-05*w45*Jinv00+4.2613636363636367e-05*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-1.8939393939393939e-04*Jinv00*w27-5.4112554112554113e-05*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-2.0495129870129870e-04*w13*Jinv11+4.2613636363636367e-05*w15*Jinv01-1.8939393939393939e-04*w28*Jinv11+1.1668019480519480e-04*Jinv20*w50-1.8939393939393939e-04*Jinv20*w29-6.4935064935064935e-05*w46*Jinv10+4.2613636363636367e-05*Jinv22*w17+1.9480519480519481e-04*Jinv00*w33-8.3198051948051951e-05*w22*Jinv12-1.5726461038961039e-04*w31*Jinv10-1.5726461038961039e-04*Jinv02*w3-8.3198051948051951e-05*w21*Jinv00-1.5726461038961039e-04*w32*Jinv22+4.2613636363636367e-05*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2+1.4204545454545455e-05*Jinv00*w18-1.0247564935064935e-04*w36*Jinv00-1.0247564935064935e-04*Jinv21*w38-6.4935064935064935e-05*Jinv02*w45-6.4935064935064935e-05*Jinv01*w24-5.4112554112554113e-05*Jinv22*w59+1.9480519480519481e-04*w34*Jinv11+1.4204545454545455e-05*w19*Jinv11-1.0247564935064935e-04*Jinv21*w53-1.0247564935064935e-04*w51*Jinv00+1.9480519480519481e-04*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-1.5726461038961039e-04*w4*Jinv10-1.5726461038961039e-04*w4*Jinv11+4.2613636363636367e-05*w41*Jinv20-2.0495129870129870e-04*Jinv12*w13+1.4204545454545455e-05*w19*Jinv10+1.9480519480519481e-04*w34*Jinv10-1.5726461038961039e-04*w32*Jinv21+1.4204545454545455e-05*Jinv21*w20+1.1668019480519480e-04*w6*Jinv00-1.0247564935064935e-04*w37*Jinv12+1.4204545454545455e-05*Jinv20*w56+4.2613636363636367e-05*Jinv22*w41-6.4935064935064935e-05*w26*Jinv20-5.4112554112554113e-05*w11*Jinv20-1.0247564935064935e-04*w52*Jinv12+1.9480519480519481e-04*Jinv21*w35-1.5726461038961039e-04*w31*Jinv11-6.4935064935064935e-05*Jinv22*w26-6.4935064935064935e-05*w46*Jinv11-2.0495129870129870e-04*Jinv20*w14-1.5726461038961039e-04*w5*Jinv21+1.4204545454545455e-05*Jinv02*w54-1.5726461038961039e-04*w4*Jinv12-8.3198051948051951e-05*w21*Jinv02+1.1668019480519480e-04*w49*Jinv10-1.8939393939393939e-04*Jinv21*w29+1.9480519480519481e-04*Jinv01*w33+4.2613636363636367e-05*w39*Jinv00-2.0495129870129870e-04*Jinv02*w12+1.4204545454545455e-05*Jinv01*w18-1.0247564935064935e-04*w37*Jinv11-5.4112554112554113e-05*Jinv02*w9+1.1668019480519480e-04*w6*Jinv01+1.1668019480519480e-04*w7*Jinv10-6.4935064935064935e-05*w47*Jinv21-1.0247564935064935e-04*Jinv20*w38-8.3198051948051951e-05*Jinv22*w23-1.0247564935064935e-04*w52*Jinv11-1.5726461038961039e-04*w31*Jinv12-6.4935064935064935e-05*w46*Jinv12-1.0247564935064935e-04*Jinv20*w53+1.1668019480519480e-04*Jinv22*w8-8.3198051948051951e-05*w22*Jinv10+1.1668019480519480e-04*Jinv22*w50-8.3198051948051951e-05*w22*Jinv11+1.1668019480519480e-04*Jinv12*w7-8.3198051948051951e-05*w21*Jinv01-1.0247564935064935e-04*Jinv01*w51-1.5726461038961039e-04*Jinv02*w30+1.1668019480519480e-04*Jinv12*w49-1.0247564935064935e-04*w36*Jinv01-1.0247564935064935e-04*w52*Jinv10+1.1668019480519480e-04*w7*Jinv11-5.4112554112554113e-05*w9*Jinv00-1.0247564935064935e-04*w37*Jinv10+1.4204545454545455e-05*Jinv21*w56-6.4935064935064935e-05*w24*Jinv00-1.8939393939393939e-04*Jinv02*w27+1.1668019480519480e-04*w6*Jinv02+1.4204545454545455e-05*w19*Jinv12-1.5726461038961039e-04*Jinv22*w5-6.4935064935064935e-05*w47*Jinv22+1.1668019480519480e-04*w49*Jinv11-2.0495129870129870e-04*Jinv21*w14-5.4112554112554113e-05*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-5.4112554112554113e-05*w58*Jinv12+1.4204545454545455e-05*Jinv02*w18-2.0495129870129870e-04*Jinv01*w12+1.1668019480519480e-04*Jinv21*w8-5.4112554112554113e-05*w57*Jinv00-5.4112554112554113e-05*Jinv20*w59+1.9480519480519481e-04*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+1.9480519480519481e-04*w35*Jinv20-8.3198051948051951e-05*Jinv21*w23+1.4204545454545455e-05*Jinv01*w54+1.4204545454545455e-05*w55*Jinv11-6.4935064935064935e-05*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+4.2613636363636367e-05*w15*Jinv00+1.1668019480519480e-04*w48*Jinv02-5.4112554112554113e-05*Jinv01*w9+4.2613636363636367e-05*Jinv20*w17-1.5726461038961039e-04*w30*Jinv00+4.2613636363636367e-05*w40*Jinv10+1.4204545454545455e-05*w20*Jinv20-1.8939393939393939e-04*Jinv22*w29-2.0495129870129870e-04*Jinv22*w14+4.2613636363636367e-05*w40*Jinv11-1.0247564935064935e-04*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-1.5726461038961039e-04*w5*Jinv20-1.5726461038961039e-04*Jinv01*w30-1.0247564935064935e-04*Jinv02*w51+1.4204545454545455e-05*Jinv22*w56-1.8939393939393939e-04*Jinv12*w28-6.4935064935064935e-05*w25*Jinv11+1.4204545454545455e-05*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00-6.4935064935064935e-05*w47*Jinv20+4.2613636363636367e-05*w16*Jinv12+1.1668019480519480e-04*Jinv21*w50-1.8939393939393939e-04*Jinv01*w27+1.1668019480519480e-04*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-5.4112554112554113e-05*w10*Jinv11)*D;
  A[28][2]=(0.0)*D;
  A[28][3]=(0.0)*D;
  A[28][4]=(3.9265422077922079e-04*Jinv00*w12-5.2962662337662338e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+3.9265422077922079e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44+3.7439123376623376e-04*w32*Jinv20-7.4878246753246752e-04*Jinv01*w3+9.1314935064935065e-06*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38-1.0247564935064935e-04*Jinv21*w59-1.8262987012987013e-05*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-1.0592532467532468e-03*w16*Jinv11-1.0592532467532468e-03*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55-1.0592532467532468e-03*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00-1.8262987012987013e-05*Jinv02*w24-9.6793831168831166e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01-1.8262987012987013e-05*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+1.5726461038961039e-04*w1*Jinv10-5.6615259740259744e-04*w16*Jinv10+3.1047077922077924e-04*Jinv22*w20+7.3051948051948052e-05*w43*Jinv11-5.2962662337662338e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*w11*Jinv22+9.2329545454545453e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-1.0592532467532468e-03*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-1.0592532467532468e-03*Jinv22*w17-6.9399350649350649e-04*Jinv00*w33+4.8396915584415583e-04*w22*Jinv12+3.7439123376623376e-04*w31*Jinv10-7.4878246753246752e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-3.7439123376623376e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+1.7349837662337662e-04*Jinv00*w18+3.1960227272727273e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-1.8262987012987013e-05*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-9.6793831168831166e-04*w34*Jinv11+3.1047077922077924e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-9.6793831168831166e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-7.4878246753246752e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20-3.9265422077922079e-04*Jinv12*w13+1.7349837662337662e-04*w19*Jinv10-6.9399350649350649e-04*w34*Jinv10-3.7439123376623376e-04*w32*Jinv21+3.1047077922077924e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12-5.2962662337662338e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12-9.6793831168831166e-04*Jinv21*w35-3.7439123376623376e-04*w31*Jinv11-1.8262987012987013e-05*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11+3.9265422077922079e-04*Jinv20*w14-7.4878246753246752e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54-7.4878246753246752e-04*w4*Jinv12+4.8396915584415583e-04*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-9.6793831168831166e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00-3.9265422077922079e-04*Jinv02*w12+3.1047077922077924e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9+1.5523538961038962e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-9.1314935064935065e-06*w47*Jinv21+3.1960227272727273e-04*Jinv20*w38+4.8396915584415583e-04*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11-3.7439123376623376e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.5523538961038962e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+3.1960227272727273e-04*Jinv22*w50+4.8396915584415583e-04*w22*Jinv11+1.5523538961038962e-04*Jinv12*w7+4.8396915584415583e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51-3.7439123376623376e-04*Jinv02*w30+3.1960227272727273e-04*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.5523538961038962e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+3.1960227272727273e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-4.0584415584415584e-05*Jinv02*w27+1.5523538961038962e-04*w6*Jinv02+3.1047077922077924e-04*w19*Jinv12-7.4878246753246752e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+3.1960227272727273e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+3.1047077922077924e-04*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.5523538961038962e-04*Jinv21*w8-9.6793831168831166e-04*Jinv02*w33+1.5726461038961039e-04*w0*Jinv00-6.9399350649350649e-04*w35*Jinv20+4.8396915584415583e-04*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2-5.6615259740259744e-04*w15*Jinv00+3.1960227272727273e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-5.6615259740259744e-04*Jinv20*w17+3.7439123376623376e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10+1.7349837662337662e-04*w20*Jinv20-4.0584415584415584e-05*Jinv22*w29-3.9265422077922079e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.7439123376623376e-04*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56-4.0584415584415584e-05*Jinv12*w28-1.8262987012987013e-05*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-1.0592532467532468e-03*w16*Jinv12+3.1960227272727273e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+3.1960227272727273e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[28][5]=(0.0)*D;
  A[28][6]=(0.0)*D;
  A[28][7]=(-1.7349837662337662e-04*Jinv00*w12+3.6525974025974026e-05*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-1.0247564935064935e-04*w57*Jinv02-1.7349837662337662e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00-3.1960227272727273e-04*w32*Jinv20+4.2918019480519480e-04*Jinv01*w3+9.1314935064935065e-06*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38-1.0247564935064935e-04*Jinv21*w59+4.7483766233766234e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55-4.9310064935064932e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+4.7483766233766234e-04*Jinv02*w24-2.7394480519480518e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+4.7483766233766234e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.1668019480519480e-04*w1*Jinv10+5.6615259740259744e-04*w16*Jinv10-7.8530844155844158e-04*Jinv22*w20-4.9310064935064932e-04*w43*Jinv11+3.6525974025974026e-05*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*w11*Jinv22-1.0247564935064935e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-4.9310064935064932e-04*Jinv22*w17+6.9399350649350649e-04*Jinv00*w33-9.1314935064935065e-06*w22*Jinv12-3.1960227272727273e-04*w31*Jinv10+4.2918019480519480e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2-3.9265422077922079e-04*Jinv00*w18-3.7439123376623376e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45+4.7483766233766234e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-2.7394480519480518e-04*w34*Jinv11-7.8530844155844158e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-2.7394480519480518e-04*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.2918019480519480e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20+1.3697240259740259e-04*Jinv12*w13-3.9265422077922079e-04*w19*Jinv10+6.9399350649350649e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21-7.8530844155844158e-04*Jinv21*w20-7.4878246753246752e-04*w37*Jinv12+3.6525974025974026e-05*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-2.7394480519480518e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11+4.7483766233766234e-04*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11-1.7349837662337662e-04*Jinv20*w14+4.2918019480519480e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54+4.2918019480519480e-04*w4*Jinv12-9.1314935064935065e-06*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-2.7394480519480518e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00+1.3697240259740259e-04*Jinv02*w12-7.8530844155844158e-04*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.4878246753246752e-04*w6*Jinv01-9.1314935064935065e-06*w47*Jinv21-3.7439123376623376e-04*Jinv20*w38-9.1314935064935065e-06*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53-7.4878246753246752e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+1.0957792207792208e-04*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11-7.4878246753246752e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30+1.0957792207792208e-04*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10-7.4878246753246752e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00-3.7439123376623376e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-4.0584415584415584e-05*Jinv02*w27-7.4878246753246752e-04*w6*Jinv02-7.8530844155844158e-04*w19*Jinv12+4.2918019480519480e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+1.0957792207792208e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-7.8530844155844158e-04*Jinv02*w18+1.3697240259740259e-04*Jinv01*w12-7.4878246753246752e-04*Jinv21*w8-2.7394480519480518e-04*Jinv02*w33-1.1668019480519480e-04*w0*Jinv00+6.9399350649350649e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+5.6615259740259744e-04*w15*Jinv00+1.0957792207792208e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+5.6615259740259744e-04*Jinv20*w17-3.1960227272727273e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10-3.9265422077922079e-04*w20*Jinv20-4.0584415584415584e-05*Jinv22*w29+1.3697240259740259e-04*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56-4.0584415584415584e-05*Jinv12*w28+4.7483766233766234e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-4.9310064935064932e-04*w16*Jinv12+1.0957792207792208e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+1.0957792207792208e-04*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[28][8]=(0.0)*D;
  A[28][9]=(0.0)*D;
  A[28][10]=(-1.4204545454545455e-05*Jinv00*w12+1.8939393939393939e-04*w28*Jinv10+1.5726461038961039e-04*Jinv20*w8-1.4204545454545455e-05*w13*Jinv10-1.1668019480519480e-04*w3*Jinv00+1.0247564935064935e-04*w32*Jinv20+6.4935064935064935e-05*Jinv20*w23-1.4204545454545455e-05*Jinv00*w54-4.2613636363636367e-05*w43*Jinv10+5.4112554112554113e-05*w58*Jinv10+1.0247564935064935e-04*w48*Jinv00+5.4112554112554113e-05*w1*Jinv10-4.2613636363636367e-05*w16*Jinv10+6.4935064935064935e-05*w45*Jinv00+1.8939393939393939e-04*Jinv00*w27+1.0247564935064935e-04*Jinv20*w50+1.8939393939393939e-04*Jinv20*w29+6.4935064935064935e-05*w46*Jinv10-1.9480519480519481e-04*Jinv00*w33+1.0247564935064935e-04*w31*Jinv10+6.4935064935064935e-05*w21*Jinv00+2.0495129870129870e-04*Jinv00*w18+1.5726461038961039e-04*w36*Jinv00-1.1668019480519480e-04*w51*Jinv00-1.1668019480519480e-04*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+2.0495129870129870e-04*w19*Jinv10-1.9480519480519481e-04*w34*Jinv10+1.5726461038961039e-04*w6*Jinv00-1.4204545454545455e-05*Jinv20*w56+8.3198051948051951e-05*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20-1.4204545454545455e-05*Jinv20*w14+1.0247564935064935e-04*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+1.5726461038961039e-04*w7*Jinv10+1.5726461038961039e-04*Jinv20*w38-1.1668019480519480e-04*Jinv20*w53+6.4935064935064935e-05*w22*Jinv10-1.1668019480519480e-04*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+1.5726461038961039e-04*w37*Jinv10+8.3198051948051951e-05*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-4.2613636363636367e-05*Jinv20*w44+5.4112554112554113e-05*w57*Jinv00+5.4112554112554113e-05*Jinv20*w59+5.4112554112554113e-05*w0*Jinv00-1.9480519480519481e-04*w35*Jinv20+8.3198051948051951e-05*w25*Jinv10+5.4112554112554113e-05*Jinv20*w2-4.2613636363636367e-05*w15*Jinv00-4.2613636363636367e-05*Jinv20*w17+1.0247564935064935e-04*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+2.0495129870129870e-04*w20*Jinv20-1.1668019480519480e-04*w5*Jinv20-1.4204545454545455e-05*w55*Jinv10-4.2613636363636367e-05*w42*Jinv00+6.4935064935064935e-05*w47*Jinv20)*D;
  A[28][11]=(0.0)*D;
  A[28][12]=(0.0)*D;
  A[28][13]=(-6.5746753246753243e-04*Jinv00*w12+6.0876623376623375e-04*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53+1.7349837662337662e-04*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-2.5568181818181820e-04*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-3.9265422077922079e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+5.6818181818181818e-05*w0*Jinv02+1.2175324675324675e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.5568181818181820e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+5.5702110389610384e-04*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+1.7349837662337662e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01+5.5702110389610384e-04*Jinv21*w26+1.2175324675324675e-04*w0*Jinv01+5.6818181818181818e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10-2.1915584415584417e-04*Jinv22*w20-2.5568181818181820e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+6.0876623376623375e-04*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22+1.2175324675324675e-04*Jinv11*w1+1.2784090909090910e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+1.7349837662337662e-04*Jinv20*w50+6.0876623376623375e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.0957792207792207e-03*w22*Jinv12-3.9265422077922079e-04*w31*Jinv10-3.9265422077922079e-04*Jinv02*w3+1.0957792207792207e-03*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+5.6818181818181818e-05*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+5.5702110389610384e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-2.5568181818181820e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53+1.3697240259740259e-04*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20+1.7349837662337662e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-2.5568181818181820e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+5.5702110389610384e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54-3.9265422077922079e-04*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+1.7349837662337662e-04*w49*Jinv10+6.4935064935064935e-04*Jinv21*w29-2.5568181818181820e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-2.5568181818181820e-04*w6*Jinv01+1.7349837662337662e-04*w7*Jinv10+5.5702110389610384e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38+1.0957792207792207e-03*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53+1.7349837662337662e-04*Jinv22*w8+1.0957792207792207e-03*w22*Jinv10+1.7349837662337662e-04*Jinv22*w50+6.2094155844155847e-04*w22*Jinv11+1.7349837662337662e-04*Jinv12*w7+6.2094155844155847e-04*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+1.7349837662337662e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01+1.3697240259740259e-04*w52*Jinv10-2.5568181818181820e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00+6.0876623376623375e-04*Jinv02*w27+1.7349837662337662e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12-3.9265422077922079e-04*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22-2.5568181818181820e-04*w49*Jinv11+1.2784090909090910e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+1.2784090909090910e-04*Jinv01*w12-2.5568181818181820e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+5.6818181818181818e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20+6.2094155844155847e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10+5.6818181818181818e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+1.7349837662337662e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17-3.9265422077922079e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+6.0876623376623375e-04*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56+6.0876623376623375e-04*Jinv12*w28+5.5702110389610384e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12-2.5568181818181820e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-2.5568181818181820e-04*w48*Jinv01+5.6818181818181818e-05*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[28][14]=(0.0)*D;
  A[28][15]=(0.0)*D;
  A[28][16]=(4.3831168831168834e-04*Jinv00*w12-2.1915584415584417e-04*w40*Jinv12+7.3051948051948052e-05*Jinv22*w53-5.6615259740259744e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+4.3831168831168834e-04*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39+6.9399350649350649e-04*Jinv21*w44+5.6615259740259744e-04*w3*Jinv00+5.2962662337662338e-04*w32*Jinv20+6.2094155844155847e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23-4.9310064935064932e-04*Jinv22*w38+1.9480519480519481e-04*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12-3.0438311688311688e-05*w0*Jinv02-2.1915584415584417e-04*w43*Jinv10+6.9399350649350649e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55-4.3831168831168834e-04*w15*Jinv02+1.2175324675324675e-05*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+1.0957792207792207e-03*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35+1.9480519480519481e-04*w58*Jinv11+1.9480519480519481e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-4.2613636363636367e-05*w1*Jinv10-6.5746753246753243e-04*Jinv22*w20+6.9399350649350649e-04*w43*Jinv11+9.1314935064935063e-04*Jinv21*w41-4.3831168831168834e-04*w42*Jinv02+1.2175324675324675e-05*w11*Jinv22+2.7394480519480518e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01-2.4350649350649351e-04*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50-4.3831168831168834e-04*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.2962662337662338e-04*w31*Jinv10+7.3051948051948052e-05*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22+9.1314935064935063e-04*Jinv01*w39-3.0438311688311688e-05*Jinv22*w2-4.3831168831168834e-04*Jinv00*w18-5.2962662337662338e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11-3.4699675324675324e-04*Jinv21*w53+3.6525974025974026e-05*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-4.3831168831168834e-04*Jinv22*w44+5.6615259740259744e-04*w4*Jinv10+6.2094155844155847e-04*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20-2.1915584415584417e-04*Jinv12*w13-4.3831168831168834e-04*w19*Jinv10+3.1047077922077924e-04*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-5.6615259740259744e-04*w6*Jinv00-4.9310064935064932e-04*w37*Jinv12-2.1915584415584417e-04*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20+7.3051948051948052e-05*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+3.1047077922077924e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11+4.3831168831168834e-04*Jinv20*w14+6.2094155844155847e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54+7.3051948051948052e-05*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10-2.4350649350649351e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00-2.1915584415584417e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.2175324675324675e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01-5.6615259740259744e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-5.2962662337662338e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23-3.4699675324675324e-04*w52*Jinv11+3.6525974025974026e-05*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+3.6525974025974026e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-3.4699675324675324e-04*Jinv01*w51+3.6525974025974026e-05*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01+3.6525974025974026e-05*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-5.2962662337662338e-04*w37*Jinv10-2.5568181818181820e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-7.3051948051948055e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-6.5746753246753243e-04*w19*Jinv12+7.3051948051948052e-05*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12-6.5746753246753243e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8-1.1323051948051949e-03*Jinv02*w33-4.2613636363636367e-05*w0*Jinv00-4.3831168831168834e-04*Jinv21*w23-2.5568181818181820e-04*Jinv01*w54-2.5568181818181820e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2+3.6525974025974026e-05*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+5.2962662337662338e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-4.3831168831168834e-04*w20*Jinv20-7.3051948051948055e-04*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+9.1314935064935063e-04*w40*Jinv11-4.9310064935064932e-04*Jinv02*w36-4.3831168831168834e-04*Jinv12*w43+5.6615259740259744e-04*w5*Jinv20+3.1047077922077924e-04*Jinv01*w30+7.3051948051948052e-05*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56-7.3051948051948055e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11-2.1915584415584417e-04*w42*Jinv00-4.3831168831168834e-04*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-2.4350649350649351e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-3.0438311688311688e-05*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[28][17]=(0.0)*D;
  A[28][18]=(0.0)*D;
  A[28][19]=(2.1915584415584417e-04*Jinv00*w12-6.0876623376623375e-04*w28*Jinv10+3.9265422077922079e-04*Jinv20*w8+2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv21*w44-1.7349837662337662e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-4.2918019480519480e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23+2.1915584415584417e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+4.9310064935064932e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17+9.1314935064935065e-06*Jinv01*w45-1.4204545454545455e-05*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-4.7483766233766234e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01-1.4204545454545455e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+4.9310064935064932e-04*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-6.0876623376623375e-04*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-1.3697240259740259e-04*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01+4.0584415584415584e-05*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50-6.0876623376623375e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10-5.4788961038961036e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+6.5746753246753243e-04*Jinv00*w18+3.9265422077922079e-04*w36*Jinv00+7.4878246753246752e-04*Jinv21*w38-4.7483766233766234e-04*Jinv01*w24+2.7394480519480518e-04*w34*Jinv11+7.8530844155844158e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-1.7349837662337662e-04*w51*Jinv00-1.7349837662337662e-04*w4*Jinv10-4.2918019480519480e-04*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21+7.8530844155844158e-04*Jinv21*w20+3.9265422077922079e-04*w6*Jinv00+2.1915584415584417e-04*Jinv20*w56-1.0957792207792207e-03*w26*Jinv20-5.6818181818181818e-05*w11*Jinv20+2.7394480519480518e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+9.1314935064935065e-06*w46*Jinv11+2.1915584415584417e-04*Jinv20*w14-4.2918019480519480e-04*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10+4.0584415584415584e-05*Jinv21*w29+2.7394480519480518e-04*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00+7.8530844155844158e-04*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01+3.9265422077922079e-04*w7*Jinv10+9.1314935064935065e-06*w47*Jinv21+3.9265422077922079e-04*Jinv20*w38-4.2918019480519480e-04*w52*Jinv11-1.7349837662337662e-04*Jinv20*w53-5.4788961038961036e-04*w22*Jinv10+9.1314935064935065e-06*w22*Jinv11+9.1314935064935065e-06*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01-1.7349837662337662e-04*w52*Jinv10+7.4878246753246752e-04*w7*Jinv11-5.6818181818181818e-05*w9*Jinv00+3.9265422077922079e-04*w37*Jinv10-1.3697240259740259e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00-1.0957792207792208e-04*w49*Jinv11-1.3697240259740259e-04*Jinv21*w14-5.6818181818181818e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44-1.3697240259740259e-04*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20+9.1314935064935065e-06*Jinv21*w23-1.3697240259740259e-04*Jinv01*w54-1.3697240259740259e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+4.3831168831168834e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-1.7349837662337662e-04*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30-4.7483766233766234e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-1.0957792207792208e-04*Jinv21*w50+4.0584415584415584e-05*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[28][20]=(0.0)*D;
  A[28][21]=(0.0)*D;
  A[28][22]=(5.4788961038961036e-04*Jinv00*w12-1.5827922077922079e-03*w28*Jinv10+5.4788961038961036e-04*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10+5.4788961038961036e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00-9.1314935064935065e-06*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3-1.0957792207792207e-03*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12-6.4935064935064935e-05*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.4788961038961036e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17-1.1414366883116883e-03*Jinv01*w45+5.4788961038961036e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-9.1314935064935065e-06*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01-1.1414366883116883e-03*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-6.4935064935064935e-05*w1*Jinv10+5.4788961038961036e-04*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41+5.4788961038961036e-04*w42*Jinv02-1.5827922077922079e-03*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1+4.0178571428571428e-04*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-1.2987012987012987e-03*w28*Jinv11-9.1314935064935065e-06*Jinv20*w50-1.5827922077922079e-03*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.0957792207792207e-03*w22*Jinv12-9.1314935064935065e-06*w31*Jinv10-9.1314935064935065e-06*Jinv02*w3-1.0957792207792207e-03*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-6.4935064935064935e-05*Jinv22*w2+5.4788961038961036e-04*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45-1.1414366883116883e-03*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-1.8262987012987014e-04*w34*Jinv11+6.6659902597402602e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+5.4788961038961036e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+5.4788961038961036e-04*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10-2.1915584415584417e-04*w32*Jinv21+6.6659902597402602e-04*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+5.4788961038961036e-04*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-1.8262987012987014e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-1.1414366883116883e-03*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21+5.4788961038961036e-04*Jinv02*w54-9.1314935064935065e-06*w4*Jinv12-1.0957792207792207e-03*w21*Jinv02-9.1314935064935065e-06*w49*Jinv10-1.2987012987012987e-03*Jinv21*w29-1.8262987012987014e-04*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+6.6659902597402602e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9+1.1870941558441559e-04*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10-1.1414366883116883e-03*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38-1.0957792207792207e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53-9.1314935064935065e-06*Jinv22*w8-1.0957792207792207e-03*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50-1.1870941558441558e-03*w22*Jinv11-9.1314935064935065e-06*Jinv12*w7-1.1870941558441558e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10+1.1870941558441559e-04*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+6.6659902597402602e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00-1.5827922077922079e-03*Jinv02*w27-9.1314935064935065e-06*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-9.1314935064935065e-06*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+1.1870941558441559e-04*w49*Jinv11+4.0178571428571428e-04*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10+5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18+4.0178571428571428e-04*Jinv01*w12+1.1870941558441559e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-6.4935064935064935e-05*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.1870941558441558e-03*Jinv21*w23+6.6659902597402602e-04*Jinv01*w54+6.6659902597402602e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10-6.4935064935064935e-05*Jinv20*w2+5.4788961038961036e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9+5.4788961038961036e-04*Jinv20*w17-9.1314935064935065e-06*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10+5.4788961038961036e-04*w20*Jinv20-1.5827922077922079e-03*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+5.4788961038961036e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20-2.1915584415584417e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51+5.4788961038961036e-04*Jinv22*w56-1.5827922077922079e-03*Jinv12*w28-1.1414366883116883e-03*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+5.4788961038961036e-04*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12+1.1870941558441559e-04*Jinv21*w50-1.2987012987012987e-03*Jinv01*w27+1.1870941558441559e-04*w48*Jinv01-6.4935064935064935e-05*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[28][23]=(0.0)*D;
  A[28][24]=(0.0)*D;
  A[28][25]=(-5.4788961038961036e-04*Jinv00*w12+1.5827922077922079e-03*w28*Jinv10+9.1314935064935065e-06*Jinv20*w8-5.4788961038961036e-04*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+9.1314935064935065e-06*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20+1.2784090909090910e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23-5.4788961038961036e-04*Jinv00*w54-9.2329545454545453e-05*Jinv21*w59-9.2329545454545453e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01-5.6818181818181818e-05*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17-4.5657467532467534e-05*Jinv01*w45+6.4935064935064935e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00-9.2329545454545453e-05*w58*Jinv11-9.2329545454545453e-05*w57*Jinv01-9.1314935064935068e-05*Jinv21*w26-9.2329545454545453e-05*w0*Jinv01+6.4935064935064935e-05*w1*Jinv10-5.4788961038961036e-04*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41+1.5827922077922079e-03*Jinv00*w27-9.2329545454545453e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01+2.8409090909090908e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50+1.5827922077922079e-03*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-4.9310064935064932e-04*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10+1.0957792207792207e-03*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-5.4788961038961036e-04*Jinv00*w18+9.1314935064935065e-06*w36*Jinv00-2.1002435064935065e-04*Jinv21*w38-9.1314935064935068e-05*Jinv01*w24-6.7573051948051951e-04*w34*Jinv11-1.4610389610389610e-04*w19*Jinv11+1.2784090909090910e-04*Jinv21*w53+9.1314935064935065e-06*w51*Jinv00+9.1314935064935065e-06*w4*Jinv10+1.2784090909090910e-04*w4*Jinv11-5.4788961038961036e-04*w41*Jinv20-5.4788961038961036e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+6.3920454545454550e-05*w32*Jinv21-1.4610389610389610e-04*Jinv21*w20+9.1314935064935065e-06*w6*Jinv00-5.4788961038961036e-04*Jinv20*w56+1.0957792207792207e-03*w26*Jinv20+6.4935064935064935e-05*w11*Jinv20-6.7573051948051951e-04*Jinv21*w35+6.3920454545454550e-05*w31*Jinv11-4.5657467532467534e-05*w46*Jinv11-5.4788961038961036e-04*Jinv20*w14+1.2784090909090910e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10+2.8409090909090908e-04*Jinv21*w29-6.7573051948051951e-04*Jinv01*w33-5.4788961038961036e-04*w39*Jinv00-1.4610389610389610e-04*Jinv01*w18-2.1002435064935065e-04*w37*Jinv11-2.1002435064935065e-04*w6*Jinv01+9.1314935064935065e-06*w7*Jinv10-4.5657467532467534e-05*w47*Jinv21+9.1314935064935065e-06*Jinv20*w38+1.2784090909090910e-04*w52*Jinv11+9.1314935064935065e-06*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10-4.5657467532467534e-05*w22*Jinv11-4.5657467532467534e-05*w21*Jinv01+1.2784090909090910e-04*Jinv01*w51-2.1002435064935065e-04*w36*Jinv01+9.1314935064935065e-06*w52*Jinv10-2.1002435064935065e-04*w7*Jinv11+6.4935064935064935e-05*w9*Jinv00+9.1314935064935065e-06*w37*Jinv10+1.1870941558441559e-04*Jinv21*w56+1.0957792207792207e-03*w24*Jinv00+6.3920454545454550e-05*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.1870941558441559e-04*Jinv01*w12-2.1002435064935065e-04*Jinv21*w8+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20-4.5657467532467534e-05*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+1.0957792207792207e-03*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2-5.4788961038961036e-04*w15*Jinv00-5.6818181818181818e-05*Jinv01*w9-5.4788961038961036e-04*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00-5.4788961038961036e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+9.1314935064935065e-06*w5*Jinv20+6.3920454545454550e-05*Jinv01*w30-9.1314935064935068e-05*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20+6.3920454545454550e-05*Jinv21*w50+2.8409090909090908e-04*Jinv01*w27+6.3920454545454550e-05*w48*Jinv01-5.6818181818181818e-05*w10*Jinv11)*D;
  A[28][26]=(0.0)*D;
  A[28][27]=(0.0)*D;
  A[28][28]=(1.2175324675324675e-04*Jinv21*w44+2.0292207792207792e-05*Jinv01*w3+9.4696969696969697e-05*Jinv21*w59+9.4696969696969697e-05*Jinv21*w2+1.2175324675324675e-04*w42*Jinv01+9.4696969696969697e-05*w11*Jinv21+1.2175324675324675e-04*w16*Jinv11+1.2175324675324675e-04*Jinv21*w17+6.4935064935064935e-04*Jinv01*w45+9.4696969696969697e-05*w58*Jinv11+9.4696969696969697e-05*w57*Jinv01+6.4935064935064935e-04*Jinv21*w26+9.4696969696969697e-05*w0*Jinv01+1.2175324675324675e-04*w43*Jinv11+1.2175324675324675e-04*Jinv21*w41+9.4696969696969697e-05*Jinv11*w1-3.2467532467532468e-04*w13*Jinv11+1.2175324675324675e-04*w15*Jinv01+8.3874458874458877e-04*w28*Jinv11+1.2175324675324675e-04*Jinv01*w39+2.0292207792207792e-05*Jinv21*w38+6.4935064935064935e-04*Jinv01*w24+2.4350649350649351e-04*w34*Jinv11-3.2467532467532468e-04*w19*Jinv11+2.0292207792207792e-05*Jinv21*w53+2.0292207792207792e-05*w4*Jinv11+2.0292207792207792e-05*w32*Jinv21-3.2467532467532468e-04*Jinv21*w20+2.4350649350649351e-04*Jinv21*w35+2.0292207792207792e-05*w31*Jinv11+6.4935064935064935e-04*w46*Jinv11+2.0292207792207792e-05*w5*Jinv21+8.3874458874458877e-04*Jinv21*w29+2.4350649350649351e-04*Jinv01*w33-3.2467532467532468e-04*Jinv01*w18+2.0292207792207792e-05*w37*Jinv11+2.0292207792207792e-05*w6*Jinv01+6.4935064935064935e-04*w47*Jinv21+2.0292207792207792e-05*w52*Jinv11+6.4935064935064935e-04*w22*Jinv11+6.4935064935064935e-04*w21*Jinv01+2.0292207792207792e-05*Jinv01*w51+2.0292207792207792e-05*w36*Jinv01+2.0292207792207792e-05*w7*Jinv11-3.2467532467532468e-04*Jinv21*w56+2.0292207792207792e-05*w49*Jinv11-3.2467532467532468e-04*Jinv21*w14-3.2467532467532468e-04*Jinv01*w12+2.0292207792207792e-05*Jinv21*w8+6.4935064935064935e-04*Jinv21*w23-3.2467532467532468e-04*Jinv01*w54-3.2467532467532468e-04*w55*Jinv11+9.4696969696969697e-05*Jinv01*w9+1.2175324675324675e-04*w40*Jinv11+2.0292207792207792e-05*Jinv01*w30+6.4935064935064935e-04*w25*Jinv11+2.0292207792207792e-05*Jinv21*w50+8.3874458874458877e-04*Jinv01*w27+2.0292207792207792e-05*w48*Jinv01+9.4696969696969697e-05*w10*Jinv11)*D;
  A[28][29]=(0.0)*D;
  A[28][30]=(0.0)*D;
  A[28][31]=(-3.9265422077922079e-04*Jinv00*w12-4.0584415584415584e-05*w28*Jinv10-5.6615259740259744e-04*w40*Jinv12+3.1960227272727273e-04*Jinv22*w53+3.1960227272727273e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02-3.9265422077922079e-04*w13*Jinv10-5.6615259740259744e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44-3.7439123376623376e-04*w3*Jinv00-7.4878246753246752e-04*w32*Jinv20-3.7439123376623376e-04*Jinv01*w3+4.8396915584415583e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+3.1047077922077924e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+1.5726461038961039e-04*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17-1.8262987012987013e-05*Jinv01*w45+1.7349837662337662e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02+1.5523538961038962e-04*w48*Jinv00-6.9399350649350649e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+7.3051948051948052e-05*w43*Jinv11-1.8262987012987013e-05*w45*Jinv00-1.0592532467532468e-03*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-4.0584415584415584e-05*Jinv00*w27+9.2329545454545453e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11+1.5523538961038962e-04*Jinv20*w50-4.0584415584415584e-05*Jinv20*w29-1.8262987012987013e-05*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-9.6793831168831166e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-7.4878246753246752e-04*w31*Jinv10+3.7439123376623376e-04*Jinv02*w3+4.8396915584415583e-04*w21*Jinv00-1.0592532467532468e-03*Jinv01*w39+1.5726461038961039e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-9.6793831168831166e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-6.9399350649350649e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44-3.7439123376623376e-04*w4*Jinv10-3.7439123376623376e-04*w4*Jinv11-1.0592532467532468e-03*w41*Jinv20+3.9265422077922079e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-9.6793831168831166e-04*w34*Jinv10-7.4878246753246752e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+3.1960227272727273e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+3.1047077922077924e-04*Jinv20*w56-5.6615259740259744e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+3.1960227272727273e-04*w52*Jinv12-9.6793831168831166e-04*Jinv21*w35-7.4878246753246752e-04*w31*Jinv11-1.8262987012987013e-05*w46*Jinv11-3.9265422077922079e-04*Jinv20*w14-3.7439123376623376e-04*w5*Jinv21+1.7349837662337662e-04*Jinv02*w54+3.7439123376623376e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02+1.5523538961038962e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-9.6793831168831166e-04*Jinv01*w33-1.0592532467532468e-03*w39*Jinv00+3.9265422077922079e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+3.1960227272727273e-04*w6*Jinv01+3.1960227272727273e-04*w7*Jinv10-1.8262987012987013e-05*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11-9.1314935064935065e-06*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8+4.8396915584415583e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+4.8396915584415583e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+4.8396915584415583e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51-2.7394480519480518e-04*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10+3.1960227272727273e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10+3.1047077922077924e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.0957792207792208e-04*w6*Jinv02+3.7439123376623376e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+1.5523538961038962e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.9265422077922079e-04*Jinv01*w12+3.1960227272727273e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-6.9399350649350649e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-9.6793831168831166e-04*w35*Jinv20+4.8396915584415583e-04*Jinv21*w23+3.1047077922077924e-04*Jinv01*w54+3.1047077922077924e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-7.4878246753246752e-04*w30*Jinv00-1.0592532467532468e-03*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w14-1.0592532467532468e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43-3.7439123376623376e-04*w5*Jinv20-7.4878246753246752e-04*Jinv01*w30+3.1960227272727273e-04*Jinv02*w51+1.7349837662337662e-04*Jinv22*w56-9.1314935064935065e-06*w25*Jinv11+3.1047077922077924e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00-1.8262987012987013e-05*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12+1.5523538961038962e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+1.5523538961038962e-04*w48*Jinv01+1.5726461038961039e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[28][32]=(0.0)*D;
  A[28][33]=(0.0)*D;
  A[28][34]=(5.2962662337662338e-04*Jinv00*w12-9.6793831168831166e-04*Jinv22*w53-6.9399350649350649e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02+5.2962662337662338e-04*w13*Jinv10-2.2646103896103898e-03*Jinv21*w44+6.9399350649350649e-04*w3*Jinv00+9.6793831168831166e-04*w32*Jinv20+4.2004870129870131e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-3.0438311688311688e-05*Jinv21*w59-1.9480519480519481e-04*w0*Jinv02-2.2524350649350650e-04*Jinv21*w2-1.1323051948051949e-03*w43*Jinv10-2.2646103896103898e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17+9.8620129870129864e-04*Jinv01*w45-5.2962662337662338e-04*Jinv12*w55+1.1323051948051949e-03*w15*Jinv02+2.7394480519480518e-04*w48*Jinv00-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01+9.8620129870129864e-04*Jinv21*w26-2.2524350649350650e-04*w0*Jinv01-1.9480519480519481e-04*w1*Jinv10-2.2646103896103898e-03*w43*Jinv11-1.1323051948051949e-03*Jinv21*w41-1.1323051948051949e-03*w42*Jinv02-2.2524350649350650e-04*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-4.8701298701298701e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+1.1323051948051949e-03*Jinv22*w17-4.9310064935064932e-04*w22*Jinv12+9.6793831168831166e-04*w31*Jinv10+9.6793831168831166e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+6.9399350649350649e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39-1.9480519480519481e-04*Jinv22*w2-5.2962662337662338e-04*Jinv00*w18-9.6793831168831166e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+9.8620129870129864e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59-2.7759740259740259e-03*w34*Jinv11-4.9310064935064932e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00-1.1323051948051949e-03*Jinv22*w44+6.9399350649350649e-04*w4*Jinv10+4.2004870129870131e-04*w4*Jinv11+1.1323051948051949e-03*w41*Jinv20+5.2962662337662338e-04*Jinv12*w13-5.2962662337662338e-04*w19*Jinv10+4.2004870129870131e-04*w32*Jinv21-4.9310064935064932e-04*Jinv21*w20-6.9399350649350649e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+4.9310064935064932e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20-9.6793831168831166e-04*w52*Jinv12-2.7759740259740259e-03*Jinv21*w35+4.2004870129870131e-04*w31*Jinv11+9.8620129870129864e-04*w46*Jinv11+5.2962662337662338e-04*Jinv20*w14+4.2004870129870131e-04*w5*Jinv21-5.2962662337662338e-04*Jinv02*w54+9.6793831168831166e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-4.8701298701298701e-04*Jinv21*w29-2.7759740259740259e-03*Jinv01*w33+1.1323051948051949e-03*w39*Jinv00+5.2962662337662338e-04*Jinv02*w12-4.9310064935064932e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-2.7394480519480518e-04*w6*Jinv01-6.9399350649350649e-04*w7*Jinv10+9.8620129870129864e-04*w47*Jinv21-9.6793831168831166e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+6.9399350649350649e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-6.9399350649350649e-04*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+6.9399350649350649e-04*Jinv02*w30-6.9399350649350649e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-9.6793831168831166e-04*w37*Jinv10-4.9310064935064932e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02+9.6793831168831166e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10-1.1323051948051949e-03*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12+3.6525974025974026e-05*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-1.9480519480519481e-04*w0*Jinv00+4.9310064935064932e-04*Jinv21*w23-4.9310064935064932e-04*Jinv01*w54-4.9310064935064932e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2-6.9399350649350649e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9+9.6793831168831166e-04*w30*Jinv00+1.1323051948051949e-03*w40*Jinv10-5.2962662337662338e-04*w20*Jinv20+5.2962662337662338e-04*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36-1.1323051948051949e-03*Jinv12*w43+6.9399350649350649e-04*w5*Jinv20+4.2004870129870131e-04*Jinv01*w30-9.6793831168831166e-04*Jinv02*w51-5.2962662337662338e-04*Jinv22*w56+9.8620129870129864e-04*w25*Jinv11-1.1323051948051949e-03*w42*Jinv00+1.1323051948051949e-03*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50-4.8701298701298701e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01-1.9480519480519481e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[28][35]=(0.0)*D;
  A[28][36]=(0.0)*D;
  A[28][37]=(-1.3697240259740259e-04*Jinv00*w12+4.0584415584415584e-05*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.2918019480519480e-04*Jinv22*w53+3.7439123376623376e-04*Jinv20*w8+1.0247564935064935e-04*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-3.1960227272727273e-04*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23+7.4878246753246752e-04*Jinv22*w38-3.1047077922077924e-04*Jinv00*w54-4.7483766233766234e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0592532467532468e-03*w43*Jinv10-1.3697240259740259e-04*Jinv12*w55-1.4204545454545455e-05*w58*Jinv10+4.9310064935064932e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-4.2918019480519480e-04*w48*Jinv00-4.7483766233766234e-04*Jinv02*w24+2.7394480519480518e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.8530844155844158e-04*Jinv22*w20+1.8262987012987013e-05*w45*Jinv00+4.9310064935064932e-04*w42*Jinv02+4.0584415584415584e-05*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-4.2918019480519480e-04*Jinv20*w50+4.0584415584415584e-05*Jinv20*w29+1.8262987012987013e-05*w46*Jinv10+4.9310064935064932e-04*Jinv22*w17+9.6793831168831166e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10-4.2918019480519480e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-1.0957792207792208e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.9265422077922079e-04*Jinv00*w18+7.4878246753246752e-04*w36*Jinv00+9.1314935064935065e-06*Jinv02*w45+1.0247564935064935e-04*Jinv22*w59-1.5523538961038962e-04*w51*Jinv00+2.7394480519480518e-04*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-3.1960227272727273e-04*w4*Jinv10-7.3051948051948052e-05*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13+3.9265422077922079e-04*w19*Jinv10+9.6793831168831166e-04*w34*Jinv10+3.7439123376623376e-04*w6*Jinv00+7.4878246753246752e-04*w37*Jinv12-3.1047077922077924e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-4.8396915584415583e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20-4.2918019480519480e-04*w52*Jinv12-4.7483766233766234e-04*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14-1.3697240259740259e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-4.2918019480519480e-04*w49*Jinv10-7.3051948051948052e-05*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.7439123376623376e-04*w7*Jinv10+7.4878246753246752e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-1.0957792207792208e-04*w31*Jinv12+9.1314935064935065e-06*w46*Jinv12-1.5523538961038962e-04*Jinv20*w53+7.4878246753246752e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10-1.0957792207792208e-04*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-1.0957792207792208e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49-1.5523538961038962e-04*w52*Jinv10-9.2329545454545453e-05*w9*Jinv00+7.4878246753246752e-04*w37*Jinv10-4.8396915584415583e-04*w24*Jinv00+4.0584415584415584e-05*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+7.8530844155844158e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+9.1314935064935065e-06*w47*Jinv22-9.2329545454545453e-05*w10*Jinv10+1.0592532467532468e-03*Jinv20*w44+1.0247564935064935e-04*w58*Jinv12+7.8530844155844158e-04*Jinv02*w18-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+2.7394480519480518e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+9.6793831168831166e-04*w35*Jinv20-4.8396915584415583e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00-1.0957792207792208e-04*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+3.9265422077922079e-04*w20*Jinv20+4.0584415584415584e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14+7.4878246753246752e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-3.1960227272727273e-04*w5*Jinv20-4.2918019480519480e-04*Jinv02*w51-1.3697240259740259e-04*Jinv22*w56+4.0584415584415584e-05*Jinv12*w28-3.1047077922077924e-04*w55*Jinv10+1.0592532467532468e-03*w42*Jinv00+1.8262987012987013e-05*w47*Jinv20+4.9310064935064932e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[28][38]=(0.0)*D;
  A[28][39]=(0.0)*D;
  A[28][40]=(-2.1915584415584417e-04*Jinv00*w12-7.3051948051948055e-04*w28*Jinv10-5.2962662337662338e-04*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-2.1915584415584417e-04*w13*Jinv10+6.9399350649350649e-04*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00+7.3051948051948052e-05*w32*Jinv20+3.1047077922077924e-04*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23+3.6525974025974026e-05*Jinv22*w38-6.5746753246753243e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-4.2613636363636367e-05*w0*Jinv02-4.3831168831168834e-04*w43*Jinv10+6.9399350649350649e-04*w42*Jinv01+1.9480519480519481e-04*w11*Jinv21+9.1314935064935063e-04*w16*Jinv11+9.1314935064935063e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45-4.3831168831168834e-04*Jinv12*w55+1.2175324675324675e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02-4.9310064935064932e-04*w48*Jinv00+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-3.0438311688311688e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10+6.9399350649350649e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02-7.3051948051948055e-04*Jinv00*w27+2.7394480519480518e-04*w13*Jinv11+9.1314935064935063e-04*w15*Jinv01-2.4350649350649351e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-7.3051948051948055e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12+7.3051948051948052e-05*w31*Jinv10+5.2962662337662338e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+5.6615259740259744e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-4.2613636363636367e-05*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18+7.3051948051948052e-05*w36*Jinv00-3.4699675324675324e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-2.5568181818181820e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-4.9310064935064932e-04*w51*Jinv00-2.1915584415584417e-04*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+3.1047077922077924e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+4.3831168831168834e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10+6.2094155844155847e-04*w32*Jinv21-2.5568181818181820e-04*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00+3.6525974025974026e-05*w37*Jinv12-6.5746753246753243e-04*Jinv20*w56+5.4788961038961036e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20-5.2962662337662338e-04*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+6.2094155844155847e-04*w31*Jinv11+1.0957792207792208e-04*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14+3.1047077922077924e-04*w5*Jinv21-4.3831168831168834e-04*Jinv02*w54+5.2962662337662338e-04*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-2.4350649350649351e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+4.3831168831168834e-04*Jinv02*w12-2.5568181818181820e-04*Jinv01*w18-3.4699675324675324e-04*w37*Jinv11-3.8352272727272725e-04*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+7.3051948051948052e-05*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11+5.6615259740259744e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12-4.9310064935064932e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-5.6615259740259744e-04*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51+5.6615259740259744e-04*Jinv02*w30-5.6615259740259744e-04*Jinv12*w49-3.4699675324675324e-04*w36*Jinv01-4.9310064935064932e-04*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00+7.3051948051948052e-05*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-3.6525974025974026e-05*w6*Jinv02+5.2962662337662338e-04*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-4.3831168831168834e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12+2.7394480519480518e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+1.2175324675324675e-05*w57*Jinv00+1.2175324675324675e-05*Jinv20*w59-3.0438311688311688e-05*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10-3.0438311688311688e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-5.6615259740259744e-04*w48*Jinv02+1.9480519480519481e-04*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+7.3051948051948052e-05*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+4.3831168831168834e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11+3.6525974025974026e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+6.2094155844155847e-04*Jinv01*w30-5.2962662337662338e-04*Jinv02*w51-4.3831168831168834e-04*Jinv22*w56+5.4788961038961042e-05*w25*Jinv11-6.5746753246753243e-04*w55*Jinv10-4.3831168831168834e-04*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50-2.4350649350649351e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01-4.2613636363636367e-05*Jinv12*w1+1.9480519480519481e-04*w10*Jinv11)*D;
  A[28][41]=(0.0)*D;
  A[28][42]=(0.0)*D;
  A[28][43]=(2.1915584415584417e-04*Jinv00*w12+7.3051948051948055e-04*w28*Jinv10+2.1915584415584417e-04*w40*Jinv12-7.3051948051948052e-05*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+3.0438311688311688e-05*w57*Jinv02+2.1915584415584417e-04*w13*Jinv10+2.1915584415584417e-04*Jinv02*w39+2.2646103896103898e-03*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00-7.3051948051948052e-05*w32*Jinv20-4.2004870129870131e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38+6.5746753246753243e-04*Jinv00*w54+3.0438311688311688e-05*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02+2.2524350649350650e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+2.2646103896103898e-03*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17-9.8620129870129864e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.2175324675324675e-05*w58*Jinv10+4.3831168831168834e-04*w15*Jinv02-1.2175324675324675e-05*w10*Jinv12+4.9310064935064932e-04*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-9.8620129870129864e-04*Jinv21*w26+2.2524350649350650e-04*w0*Jinv01+3.0438311688311688e-05*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+6.5746753246753243e-04*Jinv22*w20+2.2646103896103898e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+7.3051948051948055e-04*Jinv00*w27-1.2175324675324675e-05*w11*Jinv22+2.2524350649350650e-04*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+4.8701298701298701e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w50+7.3051948051948055e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+4.3831168831168834e-04*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12-7.3051948051948052e-05*w31*Jinv10-7.3051948051948052e-05*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+3.0438311688311688e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18-7.3051948051948052e-05*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-9.8620129870129864e-04*Jinv01*w24+3.0438311688311688e-05*Jinv22*w59+2.7759740259740259e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10-4.2004870129870131e-04*w4*Jinv11+4.3831168831168834e-04*w41*Jinv20+2.1915584415584417e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-4.2004870129870131e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12+6.5746753246753243e-04*Jinv20*w56+2.1915584415584417e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+3.0438311688311688e-05*w11*Jinv20-7.3051948051948052e-05*w52*Jinv12+2.7759740259740259e-03*Jinv21*w35-4.2004870129870131e-04*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-9.8620129870129864e-04*w46*Jinv11+2.1915584415584417e-04*Jinv20*w14-4.2004870129870131e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54-7.3051948051948052e-05*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02+4.9310064935064932e-04*w49*Jinv10+4.8701298701298701e-04*Jinv21*w29+2.7759740259740259e-03*Jinv01*w33+4.3831168831168834e-04*w39*Jinv00+2.1915584415584417e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.2175324675324675e-05*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10-9.8620129870129864e-04*w47*Jinv21-7.3051948051948052e-05*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53+4.9310064935064932e-04*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11+4.9310064935064932e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+3.0438311688311688e-05*w9*Jinv00-7.3051948051948052e-05*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+7.3051948051948055e-04*Jinv02*w27+4.9310064935064932e-04*w6*Jinv02+6.5746753246753243e-04*w19*Jinv12-7.3051948051948052e-05*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+3.0438311688311688e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+3.0438311688311688e-05*w58*Jinv12+6.5746753246753243e-04*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.2175324675324675e-05*w57*Jinv00-1.2175324675324675e-05*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+3.0438311688311688e-05*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+3.0438311688311688e-05*Jinv01*w9+2.1915584415584417e-04*Jinv20*w17-7.3051948051948052e-05*w30*Jinv00+4.3831168831168834e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+7.3051948051948055e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20-4.2004870129870131e-04*Jinv01*w30-7.3051948051948052e-05*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+7.3051948051948055e-04*Jinv12*w28-9.8620129870129864e-04*w25*Jinv11+6.5746753246753243e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+4.3831168831168834e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+4.8701298701298701e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+3.0438311688311688e-05*Jinv12*w1+3.0438311688311688e-05*w10*Jinv11)*D;
  A[28][44]=(0.0)*D;
  A[28][45]=(0.0)*D;
  A[28][46]=(-5.4788961038961036e-04*w40*Jinv12+9.1314935064935065e-06*Jinv22*w53+6.4935064935064935e-05*w57*Jinv02-5.4788961038961036e-04*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+6.3920454545454550e-05*Jinv01*w3+9.1314935064935065e-06*Jinv22*w38-5.6818181818181818e-05*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-9.2329545454545453e-05*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01-9.2329545454545453e-05*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17-9.1314935064935068e-05*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12+1.0957792207792207e-03*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-5.6818181818181818e-05*w58*Jinv11-5.6818181818181818e-05*w57*Jinv01-4.5657467532467534e-05*Jinv21*w26-9.2329545454545453e-05*w0*Jinv01-5.4788961038961036e-04*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22-9.2329545454545453e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01+2.8409090909090908e-04*w28*Jinv11-5.4788961038961036e-04*Jinv22*w17+1.0957792207792207e-03*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3+9.1314935064935065e-06*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39+6.4935064935064935e-05*Jinv22*w2+1.2784090909090910e-04*Jinv21*w38+1.0957792207792207e-03*Jinv02*w45-4.5657467532467534e-05*Jinv01*w24+6.4935064935064935e-05*Jinv22*w59-6.7573051948051951e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11-2.1002435064935065e-04*Jinv21*w53-4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+6.3920454545454550e-05*w4*Jinv11-5.4788961038961036e-04*Jinv12*w13+1.2784090909090910e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+9.1314935064935065e-06*w37*Jinv12-5.4788961038961036e-04*Jinv22*w41+9.1314935064935065e-06*w52*Jinv12-6.7573051948051951e-04*Jinv21*w35+1.2784090909090910e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26-9.1314935064935068e-05*w46*Jinv11+6.3920454545454550e-05*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+2.8409090909090908e-04*Jinv21*w29-6.7573051948051951e-04*Jinv01*w33-5.4788961038961036e-04*Jinv02*w12+1.1870941558441559e-04*Jinv01*w18+1.2784090909090910e-04*w37*Jinv11+6.4935064935064935e-05*Jinv02*w9+6.3920454545454550e-05*w6*Jinv01-9.1314935064935068e-05*w47*Jinv21+1.0957792207792207e-03*Jinv22*w23-2.1002435064935065e-04*w52*Jinv11+9.1314935064935065e-06*w31*Jinv12+1.0957792207792207e-03*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8+9.1314935064935065e-06*Jinv22*w50-4.5657467532467534e-05*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7-4.5657467532467534e-05*w21*Jinv01-2.1002435064935065e-04*Jinv01*w51+9.1314935064935065e-06*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49+1.2784090909090910e-04*w36*Jinv01+6.3920454545454550e-05*w7*Jinv11-1.4610389610389610e-04*Jinv21*w56+1.5827922077922079e-03*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5+1.0957792207792207e-03*w47*Jinv22-2.1002435064935065e-04*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+6.4935064935064935e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+1.1870941558441559e-04*Jinv01*w12+6.3920454545454550e-05*Jinv21*w8-4.9310064935064932e-04*Jinv02*w33-4.5657467532467534e-05*Jinv21*w23-1.4610389610389610e-04*Jinv01*w54-1.4610389610389610e-04*w55*Jinv11+9.1314935064935065e-06*w48*Jinv02-9.2329545454545453e-05*Jinv01*w9+1.5827922077922079e-03*Jinv22*w29-5.4788961038961036e-04*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+9.1314935064935065e-06*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.2784090909090910e-04*Jinv01*w30+9.1314935064935065e-06*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+1.5827922077922079e-03*Jinv12*w28-4.5657467532467534e-05*w25*Jinv11-5.4788961038961036e-04*w16*Jinv12-2.1002435064935065e-04*Jinv21*w50+2.8409090909090908e-04*Jinv01*w27-2.1002435064935065e-04*w48*Jinv01+6.4935064935064935e-05*Jinv12*w1-9.2329545454545453e-05*w10*Jinv11)*D;
  A[28][47]=(0.0)*D;
  A[28][48]=(0.0)*D;
  A[28][49]=(1.3697240259740259e-04*Jinv00*w12-4.0584415584415584e-05*w28*Jinv10+5.6615259740259744e-04*w40*Jinv12-3.7439123376623376e-04*Jinv22*w53+1.0957792207792208e-04*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02+1.3697240259740259e-04*w13*Jinv10+5.6615259740259744e-04*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+4.2918019480519480e-04*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3-9.1314935064935065e-06*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38-7.8530844155844158e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-1.1668019480519480e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.7483766233766234e-04*Jinv01*w45-3.9265422077922079e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02-7.4878246753246752e-04*w48*Jinv00+6.9399350649350649e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-4.9310064935064932e-04*w43*Jinv11+4.7483766233766234e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-4.0584415584415584e-05*Jinv00*w27-1.0247564935064935e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-7.4878246753246752e-04*Jinv20*w50-4.0584415584415584e-05*Jinv20*w29+4.7483766233766234e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-2.7394480519480518e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12+4.2918019480519480e-04*w31*Jinv10-3.1960227272727273e-04*Jinv02*w3-9.1314935064935065e-06*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39-1.1668019480519480e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-2.7394480519480518e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00+6.9399350649350649e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20-1.7349837662337662e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-2.7394480519480518e-04*w34*Jinv10+4.2918019480519480e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+1.0957792207792208e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12-7.8530844155844158e-04*Jinv20*w56+5.6615259740259744e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-3.7439123376623376e-04*w52*Jinv12-2.7394480519480518e-04*Jinv21*w35+4.2918019480519480e-04*w31*Jinv11+4.7483766233766234e-04*w46*Jinv11+1.3697240259740259e-04*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21-3.9265422077922079e-04*Jinv02*w54-3.1960227272727273e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-7.4878246753246752e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-2.7394480519480518e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00-1.7349837662337662e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.0957792207792208e-04*w6*Jinv01+1.0957792207792208e-04*w7*Jinv10+4.7483766233766234e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8-9.1314935064935065e-06*w22*Jinv10-9.1314935064935065e-06*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-7.8530844155844158e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.0957792207792208e-04*w6*Jinv02-3.1960227272727273e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22-7.4878246753246752e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.3697240259740259e-04*Jinv01*w12+1.0957792207792208e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+6.9399350649350649e-04*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00-2.7394480519480518e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-7.8530844155844158e-04*Jinv01*w54-7.8530844155844158e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-1.0247564935064935e-04*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.2918019480519480e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20-1.7349837662337662e-04*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+4.2918019480519480e-04*Jinv01*w30-3.7439123376623376e-04*Jinv02*w51-3.9265422077922079e-04*Jinv22*w56-9.1314935064935065e-06*w25*Jinv11-7.8530844155844158e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00+4.7483766233766234e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12-7.4878246753246752e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27-7.4878246753246752e-04*w48*Jinv01-1.1668019480519480e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[28][50]=(0.0)*D;
  A[28][51]=(0.0)*D;
  A[28][52]=(-1.3697240259740259e-04*Jinv00*w12+4.0584415584415584e-05*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+7.4878246753246752e-04*Jinv22*w53-1.0957792207792208e-04*Jinv20*w8-9.2329545454545453e-05*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39-1.0957792207792208e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23-1.5523538961038962e-04*Jinv22*w38+7.8530844155844158e-04*Jinv00*w54+1.8262987012987013e-05*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+4.9310064935064932e-04*w43*Jinv10+3.9265422077922079e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10-7.3051948051948052e-05*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+7.4878246753246752e-04*w48*Jinv00+1.8262987012987013e-05*Jinv02*w24+9.6793831168831166e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-3.1047077922077924e-04*Jinv22*w20-4.7483766233766234e-04*w45*Jinv00+1.0592532467532468e-03*w42*Jinv02+4.0584415584415584e-05*Jinv00*w27-1.4204545454545455e-05*w11*Jinv22+7.4878246753246752e-04*Jinv20*w50+4.0584415584415584e-05*Jinv20*w29-4.7483766233766234e-04*w46*Jinv10-7.3051948051948052e-05*Jinv22*w17+2.7394480519480518e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-4.2918019480519480e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-3.1960227272727273e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2-1.3697240259740259e-04*Jinv00*w18-4.2918019480519480e-04*w36*Jinv00-4.8396915584415583e-04*Jinv02*w45-9.2329545454545453e-05*Jinv22*w59+7.4878246753246752e-04*w51*Jinv00+9.6793831168831166e-04*Jinv12*w34+1.0592532467532468e-03*Jinv22*w44-1.0957792207792208e-04*w4*Jinv10+4.9310064935064932e-04*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13-1.3697240259740259e-04*w19*Jinv10+2.7394480519480518e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00-1.5523538961038962e-04*w37*Jinv12+7.8530844155844158e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41+9.1314935064935065e-06*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20+7.4878246753246752e-04*w52*Jinv12+1.8262987012987013e-05*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14+3.9265422077922079e-04*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02+7.4878246753246752e-04*w49*Jinv10+4.9310064935064932e-04*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-1.0957792207792208e-04*w7*Jinv10-4.2918019480519480e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-3.1960227272727273e-04*w31*Jinv12-4.8396915584415583e-04*w46*Jinv12+7.4878246753246752e-04*Jinv20*w53-4.2918019480519480e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+3.7439123376623376e-04*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7-3.1960227272727273e-04*Jinv02*w30+3.7439123376623376e-04*Jinv12*w49+7.4878246753246752e-04*w52*Jinv10+1.0247564935064935e-04*w9*Jinv00-4.2918019480519480e-04*w37*Jinv10+9.1314935064935065e-06*w24*Jinv00+4.0584415584415584e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-3.1047077922077924e-04*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5-4.8396915584415583e-04*w47*Jinv22+1.0247564935064935e-04*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-9.2329545454545453e-05*w58*Jinv12-3.1047077922077924e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+9.6793831168831166e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+2.7394480519480518e-04*w35*Jinv20+9.1314935064935065e-06*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.7439123376623376e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-1.3697240259740259e-04*w20*Jinv20+4.0584415584415584e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14-1.5523538961038962e-04*Jinv02*w36+1.0592532467532468e-03*Jinv12*w43-1.0957792207792208e-04*w5*Jinv20+7.4878246753246752e-04*Jinv02*w51+3.9265422077922079e-04*Jinv22*w56+4.0584415584415584e-05*Jinv12*w28+7.8530844155844158e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00-4.7483766233766234e-04*w47*Jinv20-7.3051948051948052e-05*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[28][53]=(0.0)*D;
  A[28][54]=(0.0)*D;
  A[28][55]=(4.3831168831168834e-04*w40*Jinv12+3.9265422077922079e-04*Jinv22*w53-5.6818181818181818e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39+4.9310064935064932e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3-1.7349837662337662e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.7483766233766234e-04*Jinv01*w45+6.5746753246753243e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12-5.4788961038961036e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01+9.1314935064935065e-06*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+2.1915584415584417e-04*Jinv22*w20+4.9310064935064932e-04*w43*Jinv11+4.9310064935064932e-04*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-1.3697240259740259e-04*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+4.0584415584415584e-05*w28*Jinv11+2.1915584415584417e-04*Jinv22*w17-5.4788961038961036e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-1.7349837662337662e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-1.4204545454545455e-05*Jinv22*w2-4.2918019480519480e-04*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45+9.1314935064935065e-06*Jinv01*w24-5.6818181818181818e-05*Jinv22*w59+2.7394480519480518e-04*w34*Jinv11-1.3697240259740259e-04*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-1.0957792207792208e-04*w4*Jinv11+2.1915584415584417e-04*Jinv12*w13-4.2918019480519480e-04*w32*Jinv21-1.3697240259740259e-04*Jinv21*w20-1.7349837662337662e-04*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41+3.9265422077922079e-04*w52*Jinv12+2.7394480519480518e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-4.7483766233766234e-04*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21+6.5746753246753243e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02+4.0584415584415584e-05*Jinv21*w29+2.7394480519480518e-04*Jinv01*w33+2.1915584415584417e-04*Jinv02*w12-1.3697240259740259e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-1.4204545454545455e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01-4.7483766233766234e-04*w47*Jinv21-5.4788961038961036e-04*Jinv22*w23+7.4878246753246752e-04*w52*Jinv11-1.7349837662337662e-04*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8+3.9265422077922079e-04*Jinv22*w50+9.1314935064935065e-06*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+9.1314935064935065e-06*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-1.7349837662337662e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49-4.2918019480519480e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11+7.8530844155844158e-04*Jinv21*w56-6.0876623376623375e-04*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+7.4878246753246752e-04*w49*Jinv11-1.3697240259740259e-04*Jinv21*w14-5.6818181818181818e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-1.3697240259740259e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33+9.1314935064935065e-06*Jinv21*w23+7.8530844155844158e-04*Jinv01*w54+7.8530844155844158e-04*w55*Jinv11+3.9265422077922079e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9-6.0876623376623375e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-1.7349837662337662e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-4.2918019480519480e-04*Jinv01*w30+3.9265422077922079e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-6.0876623376623375e-04*Jinv12*w28+9.1314935064935065e-06*w25*Jinv11+2.1915584415584417e-04*w16*Jinv12+7.4878246753246752e-04*Jinv21*w50+4.0584415584415584e-05*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01-1.4204545454545455e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[28][56]=(0.0)*D;
  A[28][57]=(0.0)*D;
  A[28][58]=(-4.2613636363636367e-05*w40*Jinv12+1.5726461038961039e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-4.2613636363636367e-05*Jinv02*w39-1.1668019480519480e-04*Jinv22*w38+6.4935064935064935e-05*w25*Jinv12+5.4112554112554113e-05*w0*Jinv02+2.0495129870129870e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+5.4112554112554113e-05*w10*Jinv12+6.4935064935064935e-05*Jinv02*w24-1.9480519480519481e-04*Jinv22*w35-1.4204545454545455e-05*Jinv22*w20-4.2613636363636367e-05*w42*Jinv02+5.4112554112554113e-05*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17+6.4935064935064935e-05*w22*Jinv12+1.0247564935064935e-04*Jinv02*w3-1.1668019480519480e-04*w32*Jinv22+5.4112554112554113e-05*Jinv22*w2+8.3198051948051951e-05*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-1.9480519480519481e-04*Jinv12*w34-4.2613636363636367e-05*Jinv22*w44-1.4204545454545455e-05*Jinv12*w13-1.1668019480519480e-04*w37*Jinv12-4.2613636363636367e-05*Jinv22*w41+1.5726461038961039e-04*w52*Jinv12+6.4935064935064935e-05*Jinv22*w26+2.0495129870129870e-04*Jinv02*w54+1.0247564935064935e-04*w4*Jinv12+6.4935064935064935e-05*w21*Jinv02-1.4204545454545455e-05*Jinv02*w12+5.4112554112554113e-05*Jinv02*w9+6.4935064935064935e-05*Jinv22*w23-1.1668019480519480e-04*w31*Jinv12+8.3198051948051951e-05*w46*Jinv12+1.0247564935064935e-04*Jinv22*w8+1.5726461038961039e-04*Jinv22*w50+1.0247564935064935e-04*Jinv12*w7-1.1668019480519480e-04*Jinv02*w30+1.5726461038961039e-04*Jinv12*w49+1.8939393939393939e-04*Jinv02*w27+1.0247564935064935e-04*w6*Jinv02-1.4204545454545455e-05*w19*Jinv12+1.0247564935064935e-04*Jinv22*w5+8.3198051948051951e-05*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12-1.4204545454545455e-05*Jinv02*w18-1.9480519480519481e-04*Jinv02*w33+1.5726461038961039e-04*w48*Jinv02+1.8939393939393939e-04*Jinv22*w29-1.4204545454545455e-05*Jinv22*w14-1.1668019480519480e-04*Jinv02*w36-4.2613636363636367e-05*Jinv12*w43+1.5726461038961039e-04*Jinv02*w51+2.0495129870129870e-04*Jinv22*w56+1.8939393939393939e-04*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+5.4112554112554113e-05*Jinv12*w1)*D;
  A[28][59]=(0.0)*D;
  A[29][0]=(0.0)*D;
  A[29][1]=(0.0)*D;
  A[29][2]=(-2.0495129870129870e-04*Jinv00*w12-1.8939393939393939e-04*w28*Jinv10+4.2613636363636367e-05*w40*Jinv12-1.0247564935064935e-04*Jinv22*w53+1.1668019480519480e-04*Jinv20*w8-5.4112554112554113e-05*w57*Jinv02-2.0495129870129870e-04*w13*Jinv10+4.2613636363636367e-05*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-1.5726461038961039e-04*w3*Jinv00-1.5726461038961039e-04*w32*Jinv20-1.5726461038961039e-04*Jinv01*w3-8.3198051948051951e-05*Jinv20*w23-1.0247564935064935e-04*Jinv22*w38+1.4204545454545455e-05*Jinv00*w54-5.4112554112554113e-05*Jinv21*w59-6.4935064935064935e-05*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-5.4112554112554113e-05*w11*Jinv21+4.2613636363636367e-05*w16*Jinv11+4.2613636363636367e-05*Jinv21*w17-6.4935064935064935e-05*Jinv01*w45+1.4204545454545455e-05*Jinv12*w55-5.4112554112554113e-05*w58*Jinv10+4.2613636363636367e-05*w15*Jinv02-5.4112554112554113e-05*w10*Jinv12+1.1668019480519480e-04*w48*Jinv00-6.4935064935064935e-05*Jinv02*w24+1.9480519480519481e-04*Jinv22*w35-5.4112554112554113e-05*w58*Jinv11-5.4112554112554113e-05*w57*Jinv01-6.4935064935064935e-05*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+4.2613636363636367e-05*w16*Jinv10+1.4204545454545455e-05*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11-6.4935064935064935e-05*w45*Jinv00+4.2613636363636367e-05*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-1.8939393939393939e-04*Jinv00*w27-5.4112554112554113e-05*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-2.0495129870129870e-04*w13*Jinv11+4.2613636363636367e-05*w15*Jinv01-1.8939393939393939e-04*w28*Jinv11+1.1668019480519480e-04*Jinv20*w50-1.8939393939393939e-04*Jinv20*w29-6.4935064935064935e-05*w46*Jinv10+4.2613636363636367e-05*Jinv22*w17+1.9480519480519481e-04*Jinv00*w33-8.3198051948051951e-05*w22*Jinv12-1.5726461038961039e-04*w31*Jinv10-1.5726461038961039e-04*Jinv02*w3-8.3198051948051951e-05*w21*Jinv00-1.5726461038961039e-04*w32*Jinv22+4.2613636363636367e-05*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2+1.4204545454545455e-05*Jinv00*w18-1.0247564935064935e-04*w36*Jinv00-1.0247564935064935e-04*Jinv21*w38-6.4935064935064935e-05*Jinv02*w45-6.4935064935064935e-05*Jinv01*w24-5.4112554112554113e-05*Jinv22*w59+1.9480519480519481e-04*w34*Jinv11+1.4204545454545455e-05*w19*Jinv11-1.0247564935064935e-04*Jinv21*w53-1.0247564935064935e-04*w51*Jinv00+1.9480519480519481e-04*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-1.5726461038961039e-04*w4*Jinv10-1.5726461038961039e-04*w4*Jinv11+4.2613636363636367e-05*w41*Jinv20-2.0495129870129870e-04*Jinv12*w13+1.4204545454545455e-05*w19*Jinv10+1.9480519480519481e-04*w34*Jinv10-1.5726461038961039e-04*w32*Jinv21+1.4204545454545455e-05*Jinv21*w20+1.1668019480519480e-04*w6*Jinv00-1.0247564935064935e-04*w37*Jinv12+1.4204545454545455e-05*Jinv20*w56+4.2613636363636367e-05*Jinv22*w41-6.4935064935064935e-05*w26*Jinv20-5.4112554112554113e-05*w11*Jinv20-1.0247564935064935e-04*w52*Jinv12+1.9480519480519481e-04*Jinv21*w35-1.5726461038961039e-04*w31*Jinv11-6.4935064935064935e-05*Jinv22*w26-6.4935064935064935e-05*w46*Jinv11-2.0495129870129870e-04*Jinv20*w14-1.5726461038961039e-04*w5*Jinv21+1.4204545454545455e-05*Jinv02*w54-1.5726461038961039e-04*w4*Jinv12-8.3198051948051951e-05*w21*Jinv02+1.1668019480519480e-04*w49*Jinv10-1.8939393939393939e-04*Jinv21*w29+1.9480519480519481e-04*Jinv01*w33+4.2613636363636367e-05*w39*Jinv00-2.0495129870129870e-04*Jinv02*w12+1.4204545454545455e-05*Jinv01*w18-1.0247564935064935e-04*w37*Jinv11-5.4112554112554113e-05*Jinv02*w9+1.1668019480519480e-04*w6*Jinv01+1.1668019480519480e-04*w7*Jinv10-6.4935064935064935e-05*w47*Jinv21-1.0247564935064935e-04*Jinv20*w38-8.3198051948051951e-05*Jinv22*w23-1.0247564935064935e-04*w52*Jinv11-1.5726461038961039e-04*w31*Jinv12-6.4935064935064935e-05*w46*Jinv12-1.0247564935064935e-04*Jinv20*w53+1.1668019480519480e-04*Jinv22*w8-8.3198051948051951e-05*w22*Jinv10+1.1668019480519480e-04*Jinv22*w50-8.3198051948051951e-05*w22*Jinv11+1.1668019480519480e-04*Jinv12*w7-8.3198051948051951e-05*w21*Jinv01-1.0247564935064935e-04*Jinv01*w51-1.5726461038961039e-04*Jinv02*w30+1.1668019480519480e-04*Jinv12*w49-1.0247564935064935e-04*w36*Jinv01-1.0247564935064935e-04*w52*Jinv10+1.1668019480519480e-04*w7*Jinv11-5.4112554112554113e-05*w9*Jinv00-1.0247564935064935e-04*w37*Jinv10+1.4204545454545455e-05*Jinv21*w56-6.4935064935064935e-05*w24*Jinv00-1.8939393939393939e-04*Jinv02*w27+1.1668019480519480e-04*w6*Jinv02+1.4204545454545455e-05*w19*Jinv12-1.5726461038961039e-04*Jinv22*w5-6.4935064935064935e-05*w47*Jinv22+1.1668019480519480e-04*w49*Jinv11-2.0495129870129870e-04*Jinv21*w14-5.4112554112554113e-05*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-5.4112554112554113e-05*w58*Jinv12+1.4204545454545455e-05*Jinv02*w18-2.0495129870129870e-04*Jinv01*w12+1.1668019480519480e-04*Jinv21*w8-5.4112554112554113e-05*w57*Jinv00-5.4112554112554113e-05*Jinv20*w59+1.9480519480519481e-04*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+1.9480519480519481e-04*w35*Jinv20-8.3198051948051951e-05*Jinv21*w23+1.4204545454545455e-05*Jinv01*w54+1.4204545454545455e-05*w55*Jinv11-6.4935064935064935e-05*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+4.2613636363636367e-05*w15*Jinv00+1.1668019480519480e-04*w48*Jinv02-5.4112554112554113e-05*Jinv01*w9+4.2613636363636367e-05*Jinv20*w17-1.5726461038961039e-04*w30*Jinv00+4.2613636363636367e-05*w40*Jinv10+1.4204545454545455e-05*w20*Jinv20-1.8939393939393939e-04*Jinv22*w29-2.0495129870129870e-04*Jinv22*w14+4.2613636363636367e-05*w40*Jinv11-1.0247564935064935e-04*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-1.5726461038961039e-04*w5*Jinv20-1.5726461038961039e-04*Jinv01*w30-1.0247564935064935e-04*Jinv02*w51+1.4204545454545455e-05*Jinv22*w56-1.8939393939393939e-04*Jinv12*w28-6.4935064935064935e-05*w25*Jinv11+1.4204545454545455e-05*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00-6.4935064935064935e-05*w47*Jinv20+4.2613636363636367e-05*w16*Jinv12+1.1668019480519480e-04*Jinv21*w50-1.8939393939393939e-04*Jinv01*w27+1.1668019480519480e-04*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-5.4112554112554113e-05*w10*Jinv11)*D;
  A[29][3]=(0.0)*D;
  A[29][4]=(0.0)*D;
  A[29][5]=(3.9265422077922079e-04*Jinv00*w12-5.2962662337662338e-04*w40*Jinv12+2.1915584415584417e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-1.0247564935064935e-04*w57*Jinv02+3.9265422077922079e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44+3.7439123376623376e-04*w32*Jinv20-7.4878246753246752e-04*Jinv01*w3+9.1314935064935065e-06*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38-1.0247564935064935e-04*Jinv21*w59-1.8262987012987013e-05*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-1.0592532467532468e-03*w16*Jinv11-1.0592532467532468e-03*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55-1.0592532467532468e-03*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00-1.8262987012987013e-05*Jinv02*w24-9.6793831168831166e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01-1.8262987012987013e-05*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+1.5726461038961039e-04*w1*Jinv10-5.6615259740259744e-04*w16*Jinv10+3.1047077922077924e-04*Jinv22*w20+7.3051948051948052e-05*w43*Jinv11-5.2962662337662338e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*w11*Jinv22+9.2329545454545453e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-1.0592532467532468e-03*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-1.0592532467532468e-03*Jinv22*w17-6.9399350649350649e-04*Jinv00*w33+4.8396915584415583e-04*w22*Jinv12+3.7439123376623376e-04*w31*Jinv10-7.4878246753246752e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-3.7439123376623376e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+1.7349837662337662e-04*Jinv00*w18+3.1960227272727273e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-1.8262987012987013e-05*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-9.6793831168831166e-04*w34*Jinv11+3.1047077922077924e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-9.6793831168831166e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-7.4878246753246752e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20-3.9265422077922079e-04*Jinv12*w13+1.7349837662337662e-04*w19*Jinv10-6.9399350649350649e-04*w34*Jinv10-3.7439123376623376e-04*w32*Jinv21+3.1047077922077924e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12-5.2962662337662338e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20+2.1915584415584417e-04*w52*Jinv12-9.6793831168831166e-04*Jinv21*w35-3.7439123376623376e-04*w31*Jinv11-1.8262987012987013e-05*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11+3.9265422077922079e-04*Jinv20*w14-7.4878246753246752e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54-7.4878246753246752e-04*w4*Jinv12+4.8396915584415583e-04*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-9.6793831168831166e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00-3.9265422077922079e-04*Jinv02*w12+3.1047077922077924e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9+1.5523538961038962e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-9.1314935064935065e-06*w47*Jinv21+3.1960227272727273e-04*Jinv20*w38+4.8396915584415583e-04*Jinv22*w23+2.1915584415584417e-04*w52*Jinv11-3.7439123376623376e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+1.5523538961038962e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+3.1960227272727273e-04*Jinv22*w50+4.8396915584415583e-04*w22*Jinv11+1.5523538961038962e-04*Jinv12*w7+4.8396915584415583e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51-3.7439123376623376e-04*Jinv02*w30+3.1960227272727273e-04*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+1.5523538961038962e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+3.1960227272727273e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-4.0584415584415584e-05*Jinv02*w27+1.5523538961038962e-04*w6*Jinv02+3.1047077922077924e-04*w19*Jinv12-7.4878246753246752e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+3.1960227272727273e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12+3.1047077922077924e-04*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.5523538961038962e-04*Jinv21*w8-9.6793831168831166e-04*Jinv02*w33+1.5726461038961039e-04*w0*Jinv00-6.9399350649350649e-04*w35*Jinv20+4.8396915584415583e-04*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2-5.6615259740259744e-04*w15*Jinv00+3.1960227272727273e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-5.6615259740259744e-04*Jinv20*w17+3.7439123376623376e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10+1.7349837662337662e-04*w20*Jinv20-4.0584415584415584e-05*Jinv22*w29-3.9265422077922079e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.7439123376623376e-04*Jinv01*w30+2.1915584415584417e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56-4.0584415584415584e-05*Jinv12*w28-1.8262987012987013e-05*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-1.0592532467532468e-03*w16*Jinv12+3.1960227272727273e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+3.1960227272727273e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[29][6]=(0.0)*D;
  A[29][7]=(0.0)*D;
  A[29][8]=(-1.7349837662337662e-04*Jinv00*w12+3.6525974025974026e-05*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53-1.0247564935064935e-04*w57*Jinv02-1.7349837662337662e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00-3.1960227272727273e-04*w32*Jinv20+4.2918019480519480e-04*Jinv01*w3+9.1314935064935065e-06*Jinv20*w23-7.4878246753246752e-04*Jinv22*w38-1.0247564935064935e-04*Jinv21*w59+4.7483766233766234e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-9.1314935064935065e-06*Jinv01*w45+1.3697240259740259e-04*Jinv12*w55-4.9310064935064932e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+4.7483766233766234e-04*Jinv02*w24-2.7394480519480518e-04*Jinv22*w35-1.0247564935064935e-04*w58*Jinv11-1.0247564935064935e-04*w57*Jinv01+4.7483766233766234e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.1668019480519480e-04*w1*Jinv10+5.6615259740259744e-04*w16*Jinv10-7.8530844155844158e-04*Jinv22*w20-4.9310064935064932e-04*w43*Jinv11+3.6525974025974026e-05*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*w11*Jinv22-1.0247564935064935e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-4.9310064935064932e-04*Jinv22*w17+6.9399350649350649e-04*Jinv00*w33-9.1314935064935065e-06*w22*Jinv12-3.1960227272727273e-04*w31*Jinv10+4.2918019480519480e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2-3.9265422077922079e-04*Jinv00*w18-3.7439123376623376e-04*w36*Jinv00-7.4878246753246752e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45+4.7483766233766234e-04*Jinv01*w24-1.0247564935064935e-04*Jinv22*w59-2.7394480519480518e-04*w34*Jinv11-7.8530844155844158e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-2.7394480519480518e-04*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.2918019480519480e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20+1.3697240259740259e-04*Jinv12*w13-3.9265422077922079e-04*w19*Jinv10+6.9399350649350649e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21-7.8530844155844158e-04*Jinv21*w20-7.4878246753246752e-04*w37*Jinv12+3.6525974025974026e-05*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-2.7394480519480518e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11+4.7483766233766234e-04*Jinv22*w26-9.1314935064935065e-06*w46*Jinv11-1.7349837662337662e-04*Jinv20*w14+4.2918019480519480e-04*w5*Jinv21+1.3697240259740259e-04*Jinv02*w54+4.2918019480519480e-04*w4*Jinv12-9.1314935064935065e-06*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-2.7394480519480518e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00+1.3697240259740259e-04*Jinv02*w12-7.8530844155844158e-04*Jinv01*w18-7.4878246753246752e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.4878246753246752e-04*w6*Jinv01-9.1314935064935065e-06*w47*Jinv21-3.7439123376623376e-04*Jinv20*w38-9.1314935064935065e-06*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53-7.4878246753246752e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+1.0957792207792208e-04*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11-7.4878246753246752e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30+1.0957792207792208e-04*Jinv12*w49-7.4878246753246752e-04*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10-7.4878246753246752e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00-3.7439123376623376e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-4.0584415584415584e-05*Jinv02*w27-7.4878246753246752e-04*w6*Jinv02-7.8530844155844158e-04*w19*Jinv12+4.2918019480519480e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+1.0957792207792208e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-1.0247564935064935e-04*w58*Jinv12-7.8530844155844158e-04*Jinv02*w18+1.3697240259740259e-04*Jinv01*w12-7.4878246753246752e-04*Jinv21*w8-2.7394480519480518e-04*Jinv02*w33-1.1668019480519480e-04*w0*Jinv00+6.9399350649350649e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+5.6615259740259744e-04*w15*Jinv00+1.0957792207792208e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+5.6615259740259744e-04*Jinv20*w17-3.1960227272727273e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10-3.9265422077922079e-04*w20*Jinv20-4.0584415584415584e-05*Jinv22*w29+1.3697240259740259e-04*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.4878246753246752e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+1.3697240259740259e-04*Jinv22*w56-4.0584415584415584e-05*Jinv12*w28+4.7483766233766234e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-4.9310064935064932e-04*w16*Jinv12+1.0957792207792208e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+1.0957792207792208e-04*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[29][9]=(0.0)*D;
  A[29][10]=(0.0)*D;
  A[29][11]=(-1.4204545454545455e-05*Jinv00*w12+1.8939393939393939e-04*w28*Jinv10+1.5726461038961039e-04*Jinv20*w8-1.4204545454545455e-05*w13*Jinv10-1.1668019480519480e-04*w3*Jinv00+1.0247564935064935e-04*w32*Jinv20+6.4935064935064935e-05*Jinv20*w23-1.4204545454545455e-05*Jinv00*w54-4.2613636363636367e-05*w43*Jinv10+5.4112554112554113e-05*w58*Jinv10+1.0247564935064935e-04*w48*Jinv00+5.4112554112554113e-05*w1*Jinv10-4.2613636363636367e-05*w16*Jinv10+6.4935064935064935e-05*w45*Jinv00+1.8939393939393939e-04*Jinv00*w27+1.0247564935064935e-04*Jinv20*w50+1.8939393939393939e-04*Jinv20*w29+6.4935064935064935e-05*w46*Jinv10-1.9480519480519481e-04*Jinv00*w33+1.0247564935064935e-04*w31*Jinv10+6.4935064935064935e-05*w21*Jinv00+2.0495129870129870e-04*Jinv00*w18+1.5726461038961039e-04*w36*Jinv00-1.1668019480519480e-04*w51*Jinv00-1.1668019480519480e-04*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+2.0495129870129870e-04*w19*Jinv10-1.9480519480519481e-04*w34*Jinv10+1.5726461038961039e-04*w6*Jinv00-1.4204545454545455e-05*Jinv20*w56+8.3198051948051951e-05*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20-1.4204545454545455e-05*Jinv20*w14+1.0247564935064935e-04*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+1.5726461038961039e-04*w7*Jinv10+1.5726461038961039e-04*Jinv20*w38-1.1668019480519480e-04*Jinv20*w53+6.4935064935064935e-05*w22*Jinv10-1.1668019480519480e-04*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+1.5726461038961039e-04*w37*Jinv10+8.3198051948051951e-05*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-4.2613636363636367e-05*Jinv20*w44+5.4112554112554113e-05*w57*Jinv00+5.4112554112554113e-05*Jinv20*w59+5.4112554112554113e-05*w0*Jinv00-1.9480519480519481e-04*w35*Jinv20+8.3198051948051951e-05*w25*Jinv10+5.4112554112554113e-05*Jinv20*w2-4.2613636363636367e-05*w15*Jinv00-4.2613636363636367e-05*Jinv20*w17+1.0247564935064935e-04*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+2.0495129870129870e-04*w20*Jinv20-1.1668019480519480e-04*w5*Jinv20-1.4204545454545455e-05*w55*Jinv10-4.2613636363636367e-05*w42*Jinv00+6.4935064935064935e-05*w47*Jinv20)*D;
  A[29][12]=(0.0)*D;
  A[29][13]=(0.0)*D;
  A[29][14]=(-6.5746753246753243e-04*Jinv00*w12+6.0876623376623375e-04*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+1.3697240259740259e-04*Jinv22*w53+1.7349837662337662e-04*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-2.5568181818181820e-04*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-3.9265422077922079e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+1.1668019480519480e-04*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+5.6818181818181818e-05*w0*Jinv02+1.2175324675324675e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.5568181818181820e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17+5.5702110389610384e-04*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+1.7349837662337662e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+1.1668019480519480e-04*w58*Jinv11+1.1668019480519480e-04*w57*Jinv01+5.5702110389610384e-04*Jinv21*w26+1.2175324675324675e-04*w0*Jinv01+5.6818181818181818e-05*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10-2.1915584415584417e-04*Jinv22*w20-2.5568181818181820e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+6.0876623376623375e-04*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22+1.2175324675324675e-04*Jinv11*w1+1.2784090909090910e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+1.7349837662337662e-04*Jinv20*w50+6.0876623376623375e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.0957792207792207e-03*w22*Jinv12-3.9265422077922079e-04*w31*Jinv10-3.9265422077922079e-04*Jinv02*w3+1.0957792207792207e-03*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+5.6818181818181818e-05*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00+2.7394480519480521e-05*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+5.5702110389610384e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-2.5568181818181820e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+2.7394480519480521e-05*Jinv21*w53+1.3697240259740259e-04*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20+1.7349837662337662e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+1.3697240259740259e-04*w52*Jinv12-2.5568181818181820e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+5.5702110389610384e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54-3.9265422077922079e-04*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+1.7349837662337662e-04*w49*Jinv10+6.4935064935064935e-04*Jinv21*w29-2.5568181818181820e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18+2.7394480519480521e-05*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-2.5568181818181820e-04*w6*Jinv01+1.7349837662337662e-04*w7*Jinv10+5.5702110389610384e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38+1.0957792207792207e-03*Jinv22*w23+2.7394480519480521e-05*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+1.3697240259740259e-04*Jinv20*w53+1.7349837662337662e-04*Jinv22*w8+1.0957792207792207e-03*w22*Jinv10+1.7349837662337662e-04*Jinv22*w50+6.2094155844155847e-04*w22*Jinv11+1.7349837662337662e-04*Jinv12*w7+6.2094155844155847e-04*w21*Jinv01+2.7394480519480521e-05*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+1.7349837662337662e-04*Jinv12*w49+2.7394480519480521e-05*w36*Jinv01+1.3697240259740259e-04*w52*Jinv10-2.5568181818181820e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00+6.0876623376623375e-04*Jinv02*w27+1.7349837662337662e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12-3.9265422077922079e-04*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22-2.5568181818181820e-04*w49*Jinv11+1.2784090909090910e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+1.2784090909090910e-04*Jinv01*w12-2.5568181818181820e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+5.6818181818181818e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20+6.2094155844155847e-04*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10+5.6818181818181818e-05*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00+1.7349837662337662e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17-3.9265422077922079e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+6.0876623376623375e-04*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20+3.5612824675324673e-04*Jinv01*w30+1.3697240259740259e-04*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56+6.0876623376623375e-04*Jinv12*w28+5.5702110389610384e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12-2.5568181818181820e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-2.5568181818181820e-04*w48*Jinv01+5.6818181818181818e-05*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[29][15]=(0.0)*D;
  A[29][16]=(0.0)*D;
  A[29][17]=(4.3831168831168834e-04*Jinv00*w12-2.1915584415584417e-04*w40*Jinv12+7.3051948051948052e-05*Jinv22*w53-5.6615259740259744e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+4.3831168831168834e-04*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39+6.9399350649350649e-04*Jinv21*w44+5.6615259740259744e-04*w3*Jinv00+5.2962662337662338e-04*w32*Jinv20+6.2094155844155847e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23-4.9310064935064932e-04*Jinv22*w38+1.9480519480519481e-04*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12-3.0438311688311688e-05*w0*Jinv02-2.1915584415584417e-04*w43*Jinv10+6.9399350649350649e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55-4.3831168831168834e-04*w15*Jinv02+1.2175324675324675e-05*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+1.0957792207792207e-03*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35+1.9480519480519481e-04*w58*Jinv11+1.9480519480519481e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26-4.2613636363636367e-05*w1*Jinv10-6.5746753246753243e-04*Jinv22*w20+6.9399350649350649e-04*w43*Jinv11+9.1314935064935063e-04*Jinv21*w41-4.3831168831168834e-04*w42*Jinv02+1.2175324675324675e-05*w11*Jinv22+2.7394480519480518e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01-2.4350649350649351e-04*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50-4.3831168831168834e-04*Jinv22*w17+5.4788961038961036e-04*w22*Jinv12+5.2962662337662338e-04*w31*Jinv10+7.3051948051948052e-05*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22+9.1314935064935063e-04*Jinv01*w39-3.0438311688311688e-05*Jinv22*w2-4.3831168831168834e-04*Jinv00*w18-5.2962662337662338e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11-3.4699675324675324e-04*Jinv21*w53+3.6525974025974026e-05*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-4.3831168831168834e-04*Jinv22*w44+5.6615259740259744e-04*w4*Jinv10+6.2094155844155847e-04*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20-2.1915584415584417e-04*Jinv12*w13-4.3831168831168834e-04*w19*Jinv10+3.1047077922077924e-04*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-5.6615259740259744e-04*w6*Jinv00-4.9310064935064932e-04*w37*Jinv12-2.1915584415584417e-04*Jinv22*w41+5.4788961038961036e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20+7.3051948051948052e-05*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+3.1047077922077924e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11+4.3831168831168834e-04*Jinv20*w14+6.2094155844155847e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54+7.3051948051948052e-05*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10-2.4350649350649351e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00-2.1915584415584417e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.2175324675324675e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01-5.6615259740259744e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-5.2962662337662338e-04*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23-3.4699675324675324e-04*w52*Jinv11+3.6525974025974026e-05*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12+3.6525974025974026e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10+3.6525974025974026e-05*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-3.4699675324675324e-04*Jinv01*w51+3.6525974025974026e-05*Jinv02*w30+3.6525974025974026e-05*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01+3.6525974025974026e-05*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-5.2962662337662338e-04*w37*Jinv10-2.5568181818181820e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-7.3051948051948055e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02-6.5746753246753243e-04*w19*Jinv12+7.3051948051948052e-05*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12-6.5746753246753243e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8-1.1323051948051949e-03*Jinv02*w33-4.2613636363636367e-05*w0*Jinv00-4.3831168831168834e-04*Jinv21*w23-2.5568181818181820e-04*Jinv01*w54-2.5568181818181820e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2+3.6525974025974026e-05*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+5.2962662337662338e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-4.3831168831168834e-04*w20*Jinv20-7.3051948051948055e-04*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+9.1314935064935063e-04*w40*Jinv11-4.9310064935064932e-04*Jinv02*w36-4.3831168831168834e-04*Jinv12*w43+5.6615259740259744e-04*w5*Jinv20+3.1047077922077924e-04*Jinv01*w30+7.3051948051948052e-05*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56-7.3051948051948055e-04*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11-2.1915584415584417e-04*w42*Jinv00-4.3831168831168834e-04*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-2.4350649350649351e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-3.0438311688311688e-05*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[29][18]=(0.0)*D;
  A[29][19]=(0.0)*D;
  A[29][20]=(2.1915584415584417e-04*Jinv00*w12-6.0876623376623375e-04*w28*Jinv10+3.9265422077922079e-04*Jinv20*w8+2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv21*w44-1.7349837662337662e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-4.2918019480519480e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23+2.1915584415584417e-04*Jinv00*w54+1.0247564935064935e-04*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+4.9310064935064932e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17+9.1314935064935065e-06*Jinv01*w45-1.4204545454545455e-05*w58*Jinv10-1.3697240259740259e-04*w48*Jinv00+1.0247564935064935e-04*w58*Jinv11+1.0247564935064935e-04*w57*Jinv01-4.7483766233766234e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01-1.4204545454545455e-05*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+4.9310064935064932e-04*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-6.0876623376623375e-04*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-1.3697240259740259e-04*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01+4.0584415584415584e-05*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50-6.0876623376623375e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10-5.4788961038961036e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+6.5746753246753243e-04*Jinv00*w18+3.9265422077922079e-04*w36*Jinv00+7.4878246753246752e-04*Jinv21*w38-4.7483766233766234e-04*Jinv01*w24+2.7394480519480518e-04*w34*Jinv11+7.8530844155844158e-04*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53-1.7349837662337662e-04*w51*Jinv00-1.7349837662337662e-04*w4*Jinv10-4.2918019480519480e-04*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21+7.8530844155844158e-04*Jinv21*w20+3.9265422077922079e-04*w6*Jinv00+2.1915584415584417e-04*Jinv20*w56-1.0957792207792207e-03*w26*Jinv20-5.6818181818181818e-05*w11*Jinv20+2.7394480519480518e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+9.1314935064935065e-06*w46*Jinv11+2.1915584415584417e-04*Jinv20*w14-4.2918019480519480e-04*w5*Jinv21-1.3697240259740259e-04*w49*Jinv10+4.0584415584415584e-05*Jinv21*w29+2.7394480519480518e-04*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00+7.8530844155844158e-04*Jinv01*w18+7.4878246753246752e-04*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01+3.9265422077922079e-04*w7*Jinv10+9.1314935064935065e-06*w47*Jinv21+3.9265422077922079e-04*Jinv20*w38-4.2918019480519480e-04*w52*Jinv11-1.7349837662337662e-04*Jinv20*w53-5.4788961038961036e-04*w22*Jinv10+9.1314935064935065e-06*w22*Jinv11+9.1314935064935065e-06*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51+7.4878246753246752e-04*w36*Jinv01-1.7349837662337662e-04*w52*Jinv10+7.4878246753246752e-04*w7*Jinv11-5.6818181818181818e-05*w9*Jinv00+3.9265422077922079e-04*w37*Jinv10-1.3697240259740259e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00-1.0957792207792208e-04*w49*Jinv11-1.3697240259740259e-04*Jinv21*w14-5.6818181818181818e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44-1.3697240259740259e-04*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20+9.1314935064935065e-06*Jinv21*w23-1.3697240259740259e-04*Jinv01*w54-1.3697240259740259e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+4.3831168831168834e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-1.7349837662337662e-04*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30-4.7483766233766234e-04*w25*Jinv11+2.1915584415584417e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-1.0957792207792208e-04*Jinv21*w50+4.0584415584415584e-05*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[29][21]=(0.0)*D;
  A[29][22]=(0.0)*D;
  A[29][23]=(5.4788961038961036e-04*Jinv00*w12-1.5827922077922079e-03*w28*Jinv10+5.4788961038961036e-04*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10+5.4788961038961036e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00-9.1314935064935065e-06*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3-1.0957792207792207e-03*Jinv20*w23-9.1314935064935065e-06*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54-1.5726461038961039e-04*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12-6.4935064935064935e-05*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.4788961038961036e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17-1.1414366883116883e-03*Jinv01*w45+5.4788961038961036e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-9.1314935064935065e-06*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35-1.5726461038961039e-04*w58*Jinv11-1.5726461038961039e-04*w57*Jinv01-1.1414366883116883e-03*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-6.4935064935064935e-05*w1*Jinv10+5.4788961038961036e-04*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41+5.4788961038961036e-04*w42*Jinv02-1.5827922077922079e-03*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1+4.0178571428571428e-04*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-1.2987012987012987e-03*w28*Jinv11-9.1314935064935065e-06*Jinv20*w50-1.5827922077922079e-03*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17+4.9310064935064932e-04*Jinv00*w33-1.0957792207792207e-03*w22*Jinv12-9.1314935064935065e-06*w31*Jinv10-9.1314935064935065e-06*Jinv02*w3-1.0957792207792207e-03*w21*Jinv00-9.1314935064935065e-06*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-6.4935064935064935e-05*Jinv22*w2+5.4788961038961036e-04*Jinv00*w18-9.1314935064935065e-06*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45-1.1414366883116883e-03*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-1.8262987012987014e-04*w34*Jinv11+6.6659902597402602e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+5.4788961038961036e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+5.4788961038961036e-04*w19*Jinv10+4.9310064935064932e-04*w34*Jinv10-2.1915584415584417e-04*w32*Jinv21+6.6659902597402602e-04*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00-9.1314935064935065e-06*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+5.4788961038961036e-04*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-1.8262987012987014e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-1.1414366883116883e-03*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21+5.4788961038961036e-04*Jinv02*w54-9.1314935064935065e-06*w4*Jinv12-1.0957792207792207e-03*w21*Jinv02-9.1314935064935065e-06*w49*Jinv10-1.2987012987012987e-03*Jinv21*w29-1.8262987012987014e-04*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+6.6659902597402602e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9+1.1870941558441559e-04*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10-1.1414366883116883e-03*w47*Jinv21-9.1314935064935065e-06*Jinv20*w38-1.0957792207792207e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-9.1314935064935065e-06*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53-9.1314935064935065e-06*Jinv22*w8-1.0957792207792207e-03*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50-1.1870941558441558e-03*w22*Jinv11-9.1314935064935065e-06*Jinv12*w7-1.1870941558441558e-03*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-9.1314935064935065e-06*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10+1.1870941558441559e-04*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-9.1314935064935065e-06*w37*Jinv10+6.6659902597402602e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00-1.5827922077922079e-03*Jinv02*w27-9.1314935064935065e-06*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-9.1314935064935065e-06*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+1.1870941558441559e-04*w49*Jinv11+4.0178571428571428e-04*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10+5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18+4.0178571428571428e-04*Jinv01*w12+1.1870941558441559e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-6.4935064935064935e-05*w0*Jinv00+4.9310064935064932e-04*w35*Jinv20-1.1870941558441558e-03*Jinv21*w23+6.6659902597402602e-04*Jinv01*w54+6.6659902597402602e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10-6.4935064935064935e-05*Jinv20*w2+5.4788961038961036e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9+5.4788961038961036e-04*Jinv20*w17-9.1314935064935065e-06*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10+5.4788961038961036e-04*w20*Jinv20-1.5827922077922079e-03*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11-9.1314935064935065e-06*Jinv02*w36+5.4788961038961036e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20-2.1915584415584417e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51+5.4788961038961036e-04*Jinv22*w56-1.5827922077922079e-03*Jinv12*w28-1.1414366883116883e-03*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10+5.4788961038961036e-04*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12+1.1870941558441559e-04*Jinv21*w50-1.2987012987012987e-03*Jinv01*w27+1.1870941558441559e-04*w48*Jinv01-6.4935064935064935e-05*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[29][24]=(0.0)*D;
  A[29][25]=(0.0)*D;
  A[29][26]=(-5.4788961038961036e-04*Jinv00*w12+1.5827922077922079e-03*w28*Jinv10+9.1314935064935065e-06*Jinv20*w8-5.4788961038961036e-04*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+9.1314935064935065e-06*w3*Jinv00+9.1314935064935065e-06*w32*Jinv20+1.2784090909090910e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23-5.4788961038961036e-04*Jinv00*w54-9.2329545454545453e-05*Jinv21*w59-9.2329545454545453e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01-5.6818181818181818e-05*w11*Jinv21-7.6704545454545450e-04*w16*Jinv11-7.6704545454545450e-04*Jinv21*w17-4.5657467532467534e-05*Jinv01*w45+6.4935064935064935e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00-9.2329545454545453e-05*w58*Jinv11-9.2329545454545453e-05*w57*Jinv01-9.1314935064935068e-05*Jinv21*w26-9.2329545454545453e-05*w0*Jinv01+6.4935064935064935e-05*w1*Jinv10-5.4788961038961036e-04*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41+1.5827922077922079e-03*Jinv00*w27-9.2329545454545453e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11-7.6704545454545450e-04*w15*Jinv01+2.8409090909090908e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50+1.5827922077922079e-03*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-4.9310064935064932e-04*Jinv00*w33+9.1314935064935065e-06*w31*Jinv10+1.0957792207792207e-03*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-5.4788961038961036e-04*Jinv00*w18+9.1314935064935065e-06*w36*Jinv00-2.1002435064935065e-04*Jinv21*w38-9.1314935064935068e-05*Jinv01*w24-6.7573051948051951e-04*w34*Jinv11-1.4610389610389610e-04*w19*Jinv11+1.2784090909090910e-04*Jinv21*w53+9.1314935064935065e-06*w51*Jinv00+9.1314935064935065e-06*w4*Jinv10+1.2784090909090910e-04*w4*Jinv11-5.4788961038961036e-04*w41*Jinv20-5.4788961038961036e-04*w19*Jinv10-4.9310064935064932e-04*w34*Jinv10+6.3920454545454550e-05*w32*Jinv21-1.4610389610389610e-04*Jinv21*w20+9.1314935064935065e-06*w6*Jinv00-5.4788961038961036e-04*Jinv20*w56+1.0957792207792207e-03*w26*Jinv20+6.4935064935064935e-05*w11*Jinv20-6.7573051948051951e-04*Jinv21*w35+6.3920454545454550e-05*w31*Jinv11-4.5657467532467534e-05*w46*Jinv11-5.4788961038961036e-04*Jinv20*w14+1.2784090909090910e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10+2.8409090909090908e-04*Jinv21*w29-6.7573051948051951e-04*Jinv01*w33-5.4788961038961036e-04*w39*Jinv00-1.4610389610389610e-04*Jinv01*w18-2.1002435064935065e-04*w37*Jinv11-2.1002435064935065e-04*w6*Jinv01+9.1314935064935065e-06*w7*Jinv10-4.5657467532467534e-05*w47*Jinv21+9.1314935064935065e-06*Jinv20*w38+1.2784090909090910e-04*w52*Jinv11+9.1314935064935065e-06*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10-4.5657467532467534e-05*w22*Jinv11-4.5657467532467534e-05*w21*Jinv01+1.2784090909090910e-04*Jinv01*w51-2.1002435064935065e-04*w36*Jinv01+9.1314935064935065e-06*w52*Jinv10-2.1002435064935065e-04*w7*Jinv11+6.4935064935064935e-05*w9*Jinv00+9.1314935064935065e-06*w37*Jinv10+1.1870941558441559e-04*Jinv21*w56+1.0957792207792207e-03*w24*Jinv00+6.3920454545454550e-05*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.1870941558441559e-04*Jinv01*w12-2.1002435064935065e-04*Jinv21*w8+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00-4.9310064935064932e-04*w35*Jinv20-4.5657467532467534e-05*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+1.0957792207792207e-03*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2-5.4788961038961036e-04*w15*Jinv00-5.6818181818181818e-05*Jinv01*w9-5.4788961038961036e-04*Jinv20*w17+9.1314935064935065e-06*w30*Jinv00-5.4788961038961036e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+9.1314935064935065e-06*w5*Jinv20+6.3920454545454550e-05*Jinv01*w30-9.1314935064935068e-05*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20+6.3920454545454550e-05*Jinv21*w50+2.8409090909090908e-04*Jinv01*w27+6.3920454545454550e-05*w48*Jinv01-5.6818181818181818e-05*w10*Jinv11)*D;
  A[29][27]=(0.0)*D;
  A[29][28]=(0.0)*D;
  A[29][29]=(1.2175324675324675e-04*Jinv21*w44+2.0292207792207792e-05*Jinv01*w3+9.4696969696969697e-05*Jinv21*w59+9.4696969696969697e-05*Jinv21*w2+1.2175324675324675e-04*w42*Jinv01+9.4696969696969697e-05*w11*Jinv21+1.2175324675324675e-04*w16*Jinv11+1.2175324675324675e-04*Jinv21*w17+6.4935064935064935e-04*Jinv01*w45+9.4696969696969697e-05*w58*Jinv11+9.4696969696969697e-05*w57*Jinv01+6.4935064935064935e-04*Jinv21*w26+9.4696969696969697e-05*w0*Jinv01+1.2175324675324675e-04*w43*Jinv11+1.2175324675324675e-04*Jinv21*w41+9.4696969696969697e-05*Jinv11*w1-3.2467532467532468e-04*w13*Jinv11+1.2175324675324675e-04*w15*Jinv01+8.3874458874458877e-04*w28*Jinv11+1.2175324675324675e-04*Jinv01*w39+2.0292207792207792e-05*Jinv21*w38+6.4935064935064935e-04*Jinv01*w24+2.4350649350649351e-04*w34*Jinv11-3.2467532467532468e-04*w19*Jinv11+2.0292207792207792e-05*Jinv21*w53+2.0292207792207792e-05*w4*Jinv11+2.0292207792207792e-05*w32*Jinv21-3.2467532467532468e-04*Jinv21*w20+2.4350649350649351e-04*Jinv21*w35+2.0292207792207792e-05*w31*Jinv11+6.4935064935064935e-04*w46*Jinv11+2.0292207792207792e-05*w5*Jinv21+8.3874458874458877e-04*Jinv21*w29+2.4350649350649351e-04*Jinv01*w33-3.2467532467532468e-04*Jinv01*w18+2.0292207792207792e-05*w37*Jinv11+2.0292207792207792e-05*w6*Jinv01+6.4935064935064935e-04*w47*Jinv21+2.0292207792207792e-05*w52*Jinv11+6.4935064935064935e-04*w22*Jinv11+6.4935064935064935e-04*w21*Jinv01+2.0292207792207792e-05*Jinv01*w51+2.0292207792207792e-05*w36*Jinv01+2.0292207792207792e-05*w7*Jinv11-3.2467532467532468e-04*Jinv21*w56+2.0292207792207792e-05*w49*Jinv11-3.2467532467532468e-04*Jinv21*w14-3.2467532467532468e-04*Jinv01*w12+2.0292207792207792e-05*Jinv21*w8+6.4935064935064935e-04*Jinv21*w23-3.2467532467532468e-04*Jinv01*w54-3.2467532467532468e-04*w55*Jinv11+9.4696969696969697e-05*Jinv01*w9+1.2175324675324675e-04*w40*Jinv11+2.0292207792207792e-05*Jinv01*w30+6.4935064935064935e-04*w25*Jinv11+2.0292207792207792e-05*Jinv21*w50+8.3874458874458877e-04*Jinv01*w27+2.0292207792207792e-05*w48*Jinv01+9.4696969696969697e-05*w10*Jinv11)*D;
  A[29][30]=(0.0)*D;
  A[29][31]=(0.0)*D;
  A[29][32]=(-3.9265422077922079e-04*Jinv00*w12-4.0584415584415584e-05*w28*Jinv10-5.6615259740259744e-04*w40*Jinv12+3.1960227272727273e-04*Jinv22*w53+3.1960227272727273e-04*Jinv20*w8+1.1668019480519480e-04*w57*Jinv02-3.9265422077922079e-04*w13*Jinv10-5.6615259740259744e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44-3.7439123376623376e-04*w3*Jinv00-7.4878246753246752e-04*w32*Jinv20-3.7439123376623376e-04*Jinv01*w3+4.8396915584415583e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+3.1047077922077924e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+1.5726461038961039e-04*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17-1.8262987012987013e-05*Jinv01*w45+1.7349837662337662e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02+1.5523538961038962e-04*w48*Jinv00-6.9399350649350649e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+7.3051948051948052e-05*w43*Jinv11-1.8262987012987013e-05*w45*Jinv00-1.0592532467532468e-03*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-4.0584415584415584e-05*Jinv00*w27+9.2329545454545453e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11+1.5523538961038962e-04*Jinv20*w50-4.0584415584415584e-05*Jinv20*w29-1.8262987012987013e-05*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-9.6793831168831166e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-7.4878246753246752e-04*w31*Jinv10+3.7439123376623376e-04*Jinv02*w3+4.8396915584415583e-04*w21*Jinv00-1.0592532467532468e-03*Jinv01*w39+1.5726461038961039e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24+1.1668019480519480e-04*Jinv22*w59-9.6793831168831166e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-6.9399350649350649e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44-3.7439123376623376e-04*w4*Jinv10-3.7439123376623376e-04*w4*Jinv11-1.0592532467532468e-03*w41*Jinv20+3.9265422077922079e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-9.6793831168831166e-04*w34*Jinv10-7.4878246753246752e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+3.1960227272727273e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+3.1047077922077924e-04*Jinv20*w56-5.6615259740259744e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+3.1960227272727273e-04*w52*Jinv12-9.6793831168831166e-04*Jinv21*w35-7.4878246753246752e-04*w31*Jinv11-1.8262987012987013e-05*w46*Jinv11-3.9265422077922079e-04*Jinv20*w14-3.7439123376623376e-04*w5*Jinv21+1.7349837662337662e-04*Jinv02*w54+3.7439123376623376e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02+1.5523538961038962e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-9.6793831168831166e-04*Jinv01*w33-1.0592532467532468e-03*w39*Jinv00+3.9265422077922079e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+3.1960227272727273e-04*w6*Jinv01+3.1960227272727273e-04*w7*Jinv10-1.8262987012987013e-05*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11-9.1314935064935065e-06*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8+4.8396915584415583e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+4.8396915584415583e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+4.8396915584415583e-04*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51-2.7394480519480518e-04*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10+3.1960227272727273e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10+3.1047077922077924e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.0957792207792208e-04*w6*Jinv02+3.7439123376623376e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22+1.5523538961038962e-04*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44+1.1668019480519480e-04*w58*Jinv12-3.9265422077922079e-04*Jinv01*w12+3.1960227272727273e-04*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-6.9399350649350649e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-9.6793831168831166e-04*w35*Jinv20+4.8396915584415583e-04*Jinv21*w23+3.1047077922077924e-04*Jinv01*w54+3.1047077922077924e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-7.4878246753246752e-04*w30*Jinv00-1.0592532467532468e-03*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w14-1.0592532467532468e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43-3.7439123376623376e-04*w5*Jinv20-7.4878246753246752e-04*Jinv01*w30+3.1960227272727273e-04*Jinv02*w51+1.7349837662337662e-04*Jinv22*w56-9.1314935064935065e-06*w25*Jinv11+3.1047077922077924e-04*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00-1.8262987012987013e-05*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12+1.5523538961038962e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27+1.5523538961038962e-04*w48*Jinv01+1.5726461038961039e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[29][33]=(0.0)*D;
  A[29][34]=(0.0)*D;
  A[29][35]=(5.2962662337662338e-04*Jinv00*w12-9.6793831168831166e-04*Jinv22*w53-6.9399350649350649e-04*Jinv20*w8+1.9480519480519481e-04*w57*Jinv02+5.2962662337662338e-04*w13*Jinv10-2.2646103896103898e-03*Jinv21*w44+6.9399350649350649e-04*w3*Jinv00+9.6793831168831166e-04*w32*Jinv20+4.2004870129870131e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38-3.0438311688311688e-05*Jinv21*w59-1.9480519480519481e-04*w0*Jinv02-2.2524350649350650e-04*Jinv21*w2-1.1323051948051949e-03*w43*Jinv10-2.2646103896103898e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17+9.8620129870129864e-04*Jinv01*w45-5.2962662337662338e-04*Jinv12*w55+1.1323051948051949e-03*w15*Jinv02+2.7394480519480518e-04*w48*Jinv00-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01+9.8620129870129864e-04*Jinv21*w26-2.2524350649350650e-04*w0*Jinv01-1.9480519480519481e-04*w1*Jinv10-2.2646103896103898e-03*w43*Jinv11-1.1323051948051949e-03*Jinv21*w41-1.1323051948051949e-03*w42*Jinv02-2.2524350649350650e-04*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-4.8701298701298701e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+1.1323051948051949e-03*Jinv22*w17-4.9310064935064932e-04*w22*Jinv12+9.6793831168831166e-04*w31*Jinv10+9.6793831168831166e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+6.9399350649350649e-04*w32*Jinv22-1.1323051948051949e-03*Jinv01*w39-1.9480519480519481e-04*Jinv22*w2-5.2962662337662338e-04*Jinv00*w18-9.6793831168831166e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45+9.8620129870129864e-04*Jinv01*w24+1.9480519480519481e-04*Jinv22*w59-2.7759740259740259e-03*w34*Jinv11-4.9310064935064932e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00-1.1323051948051949e-03*Jinv22*w44+6.9399350649350649e-04*w4*Jinv10+4.2004870129870131e-04*w4*Jinv11+1.1323051948051949e-03*w41*Jinv20+5.2962662337662338e-04*Jinv12*w13-5.2962662337662338e-04*w19*Jinv10+4.2004870129870131e-04*w32*Jinv21-4.9310064935064932e-04*Jinv21*w20-6.9399350649350649e-04*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+4.9310064935064932e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20-9.6793831168831166e-04*w52*Jinv12-2.7759740259740259e-03*Jinv21*w35+4.2004870129870131e-04*w31*Jinv11+9.8620129870129864e-04*w46*Jinv11+5.2962662337662338e-04*Jinv20*w14+4.2004870129870131e-04*w5*Jinv21-5.2962662337662338e-04*Jinv02*w54+9.6793831168831166e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-4.8701298701298701e-04*Jinv21*w29-2.7759740259740259e-03*Jinv01*w33+1.1323051948051949e-03*w39*Jinv00+5.2962662337662338e-04*Jinv02*w12-4.9310064935064932e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-2.7394480519480518e-04*w6*Jinv01-6.9399350649350649e-04*w7*Jinv10+9.8620129870129864e-04*w47*Jinv21-9.6793831168831166e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+6.9399350649350649e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-6.9399350649350649e-04*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+6.9399350649350649e-04*Jinv02*w30-6.9399350649350649e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-9.6793831168831166e-04*w37*Jinv10-4.9310064935064932e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00+2.7394480519480518e-04*w6*Jinv02+9.6793831168831166e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10-1.1323051948051949e-03*Jinv20*w44+1.9480519480519481e-04*w58*Jinv12+3.6525974025974026e-05*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8-1.9480519480519481e-04*w0*Jinv00+4.9310064935064932e-04*Jinv21*w23-4.9310064935064932e-04*Jinv01*w54-4.9310064935064932e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2-6.9399350649350649e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9+9.6793831168831166e-04*w30*Jinv00+1.1323051948051949e-03*w40*Jinv10-5.2962662337662338e-04*w20*Jinv20+5.2962662337662338e-04*Jinv22*w14-1.1323051948051949e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36-1.1323051948051949e-03*Jinv12*w43+6.9399350649350649e-04*w5*Jinv20+4.2004870129870131e-04*Jinv01*w30-9.6793831168831166e-04*Jinv02*w51-5.2962662337662338e-04*Jinv22*w56+9.8620129870129864e-04*w25*Jinv11-1.1323051948051949e-03*w42*Jinv00+1.1323051948051949e-03*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50-4.8701298701298701e-04*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01-1.9480519480519481e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[29][36]=(0.0)*D;
  A[29][37]=(0.0)*D;
  A[29][38]=(-1.3697240259740259e-04*Jinv00*w12+4.0584415584415584e-05*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.2918019480519480e-04*Jinv22*w53+3.7439123376623376e-04*Jinv20*w8+1.0247564935064935e-04*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-3.1960227272727273e-04*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23+7.4878246753246752e-04*Jinv22*w38-3.1047077922077924e-04*Jinv00*w54-4.7483766233766234e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+1.0592532467532468e-03*w43*Jinv10-1.3697240259740259e-04*Jinv12*w55-1.4204545454545455e-05*w58*Jinv10+4.9310064935064932e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-4.2918019480519480e-04*w48*Jinv00-4.7483766233766234e-04*Jinv02*w24+2.7394480519480518e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.8530844155844158e-04*Jinv22*w20+1.8262987012987013e-05*w45*Jinv00+4.9310064935064932e-04*w42*Jinv02+4.0584415584415584e-05*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-4.2918019480519480e-04*Jinv20*w50+4.0584415584415584e-05*Jinv20*w29+1.8262987012987013e-05*w46*Jinv10+4.9310064935064932e-04*Jinv22*w17+9.6793831168831166e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10-4.2918019480519480e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-1.0957792207792208e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.9265422077922079e-04*Jinv00*w18+7.4878246753246752e-04*w36*Jinv00+9.1314935064935065e-06*Jinv02*w45+1.0247564935064935e-04*Jinv22*w59-1.5523538961038962e-04*w51*Jinv00+2.7394480519480518e-04*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-3.1960227272727273e-04*w4*Jinv10-7.3051948051948052e-05*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13+3.9265422077922079e-04*w19*Jinv10+9.6793831168831166e-04*w34*Jinv10+3.7439123376623376e-04*w6*Jinv00+7.4878246753246752e-04*w37*Jinv12-3.1047077922077924e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-4.8396915584415583e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20-4.2918019480519480e-04*w52*Jinv12-4.7483766233766234e-04*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14-1.3697240259740259e-04*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-4.2918019480519480e-04*w49*Jinv10-7.3051948051948052e-05*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.7439123376623376e-04*w7*Jinv10+7.4878246753246752e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-1.0957792207792208e-04*w31*Jinv12+9.1314935064935065e-06*w46*Jinv12-1.5523538961038962e-04*Jinv20*w53+7.4878246753246752e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10-1.0957792207792208e-04*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-1.0957792207792208e-04*Jinv02*w30-1.0957792207792208e-04*Jinv12*w49-1.5523538961038962e-04*w52*Jinv10-9.2329545454545453e-05*w9*Jinv00+7.4878246753246752e-04*w37*Jinv10-4.8396915584415583e-04*w24*Jinv00+4.0584415584415584e-05*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+7.8530844155844158e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5+9.1314935064935065e-06*w47*Jinv22-9.2329545454545453e-05*w10*Jinv10+1.0592532467532468e-03*Jinv20*w44+1.0247564935064935e-04*w58*Jinv12+7.8530844155844158e-04*Jinv02*w18-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+2.7394480519480518e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+9.6793831168831166e-04*w35*Jinv20-4.8396915584415583e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00-1.0957792207792208e-04*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+3.9265422077922079e-04*w20*Jinv20+4.0584415584415584e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14+7.4878246753246752e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-3.1960227272727273e-04*w5*Jinv20-4.2918019480519480e-04*Jinv02*w51-1.3697240259740259e-04*Jinv22*w56+4.0584415584415584e-05*Jinv12*w28-3.1047077922077924e-04*w55*Jinv10+1.0592532467532468e-03*w42*Jinv00+1.8262987012987013e-05*w47*Jinv20+4.9310064935064932e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[29][39]=(0.0)*D;
  A[29][40]=(0.0)*D;
  A[29][41]=(-2.1915584415584417e-04*Jinv00*w12-7.3051948051948055e-04*w28*Jinv10-5.2962662337662338e-04*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-2.1915584415584417e-04*w13*Jinv10+6.9399350649350649e-04*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00+7.3051948051948052e-05*w32*Jinv20+3.1047077922077924e-04*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23+3.6525974025974026e-05*Jinv22*w38-6.5746753246753243e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-4.2613636363636367e-05*w0*Jinv02-4.3831168831168834e-04*w43*Jinv10+6.9399350649350649e-04*w42*Jinv01+1.9480519480519481e-04*w11*Jinv21+9.1314935064935063e-04*w16*Jinv11+9.1314935064935063e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45-4.3831168831168834e-04*Jinv12*w55+1.2175324675324675e-05*w58*Jinv10+2.1915584415584417e-04*w15*Jinv02-4.9310064935064932e-04*w48*Jinv00+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-3.0438311688311688e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10+6.9399350649350649e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02-7.3051948051948055e-04*Jinv00*w27+2.7394480519480518e-04*w13*Jinv11+9.1314935064935063e-04*w15*Jinv01-2.4350649350649351e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-7.3051948051948055e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10+2.1915584415584417e-04*Jinv22*w17-1.1323051948051949e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12+7.3051948051948052e-05*w31*Jinv10+5.2962662337662338e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+5.6615259740259744e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-4.2613636363636367e-05*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18+7.3051948051948052e-05*w36*Jinv00-3.4699675324675324e-04*Jinv21*w38+5.4788961038961036e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-2.5568181818181820e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-4.9310064935064932e-04*w51*Jinv00-2.1915584415584417e-04*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+3.1047077922077924e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+4.3831168831168834e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10-1.1323051948051949e-03*w34*Jinv10+6.2094155844155847e-04*w32*Jinv21-2.5568181818181820e-04*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00+3.6525974025974026e-05*w37*Jinv12-6.5746753246753243e-04*Jinv20*w56+5.4788961038961036e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20-5.2962662337662338e-04*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+6.2094155844155847e-04*w31*Jinv11+1.0957792207792208e-04*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14+3.1047077922077924e-04*w5*Jinv21-4.3831168831168834e-04*Jinv02*w54+5.2962662337662338e-04*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10-2.4350649350649351e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+4.3831168831168834e-04*Jinv02*w12-2.5568181818181820e-04*Jinv01*w18-3.4699675324675324e-04*w37*Jinv11-3.8352272727272725e-04*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+7.3051948051948052e-05*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11+5.6615259740259744e-04*w31*Jinv12+5.4788961038961036e-04*w46*Jinv12-4.9310064935064932e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-5.6615259740259744e-04*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51+5.6615259740259744e-04*Jinv02*w30-5.6615259740259744e-04*Jinv12*w49-3.4699675324675324e-04*w36*Jinv01-4.9310064935064932e-04*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00+7.3051948051948052e-05*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56+5.4788961038961036e-04*w24*Jinv00-3.6525974025974026e-05*w6*Jinv02+5.2962662337662338e-04*Jinv22*w5+5.4788961038961036e-04*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-4.3831168831168834e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12+2.7394480519480518e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+1.2175324675324675e-05*w57*Jinv00+1.2175324675324675e-05*Jinv20*w59-3.0438311688311688e-05*w0*Jinv00-1.1323051948051949e-03*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11+5.4788961038961036e-04*w25*Jinv10-3.0438311688311688e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-5.6615259740259744e-04*w48*Jinv02+1.9480519480519481e-04*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+7.3051948051948052e-05*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+4.3831168831168834e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11+3.6525974025974026e-05*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+6.2094155844155847e-04*Jinv01*w30-5.2962662337662338e-04*Jinv02*w51-4.3831168831168834e-04*Jinv22*w56+5.4788961038961042e-05*w25*Jinv11-6.5746753246753243e-04*w55*Jinv10-4.3831168831168834e-04*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20+2.1915584415584417e-04*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50-2.4350649350649351e-04*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01-4.2613636363636367e-05*Jinv12*w1+1.9480519480519481e-04*w10*Jinv11)*D;
  A[29][42]=(0.0)*D;
  A[29][43]=(0.0)*D;
  A[29][44]=(2.1915584415584417e-04*Jinv00*w12+7.3051948051948055e-04*w28*Jinv10+2.1915584415584417e-04*w40*Jinv12-7.3051948051948052e-05*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+3.0438311688311688e-05*w57*Jinv02+2.1915584415584417e-04*w13*Jinv10+2.1915584415584417e-04*Jinv02*w39+2.2646103896103898e-03*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00-7.3051948051948052e-05*w32*Jinv20-4.2004870129870131e-04*Jinv01*w3-5.4788961038961036e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38+6.5746753246753243e-04*Jinv00*w54+3.0438311688311688e-05*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.0438311688311688e-05*w0*Jinv02+2.2524350649350650e-04*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+2.2646103896103898e-03*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17-9.8620129870129864e-04*Jinv01*w45+2.1915584415584417e-04*Jinv12*w55-1.2175324675324675e-05*w58*Jinv10+4.3831168831168834e-04*w15*Jinv02-1.2175324675324675e-05*w10*Jinv12+4.9310064935064932e-04*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-9.8620129870129864e-04*Jinv21*w26+2.2524350649350650e-04*w0*Jinv01+3.0438311688311688e-05*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+6.5746753246753243e-04*Jinv22*w20+2.2646103896103898e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+1.1323051948051949e-03*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+7.3051948051948055e-04*Jinv00*w27-1.2175324675324675e-05*w11*Jinv22+2.2524350649350650e-04*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+4.8701298701298701e-04*w28*Jinv11+4.9310064935064932e-04*Jinv20*w50+7.3051948051948055e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+4.3831168831168834e-04*Jinv22*w17+1.1323051948051949e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12-7.3051948051948052e-05*w31*Jinv10-7.3051948051948052e-05*Jinv02*w3-5.4788961038961036e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+1.1323051948051949e-03*Jinv01*w39+3.0438311688311688e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18-7.3051948051948052e-05*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-9.8620129870129864e-04*Jinv01*w24+3.0438311688311688e-05*Jinv22*w59+2.7759740259740259e-03*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10-4.2004870129870131e-04*w4*Jinv11+4.3831168831168834e-04*w41*Jinv20+2.1915584415584417e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10+1.1323051948051949e-03*w34*Jinv10-4.2004870129870131e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12+6.5746753246753243e-04*Jinv20*w56+2.1915584415584417e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+3.0438311688311688e-05*w11*Jinv20-7.3051948051948052e-05*w52*Jinv12+2.7759740259740259e-03*Jinv21*w35-4.2004870129870131e-04*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-9.8620129870129864e-04*w46*Jinv11+2.1915584415584417e-04*Jinv20*w14-4.2004870129870131e-04*w5*Jinv21+2.1915584415584417e-04*Jinv02*w54-7.3051948051948052e-05*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02+4.9310064935064932e-04*w49*Jinv10+4.8701298701298701e-04*Jinv21*w29+2.7759740259740259e-03*Jinv01*w33+4.3831168831168834e-04*w39*Jinv00+2.1915584415584417e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.2175324675324675e-05*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10-9.8620129870129864e-04*w47*Jinv21-7.3051948051948052e-05*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53+4.9310064935064932e-04*Jinv22*w8-5.4788961038961036e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11+4.9310064935064932e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+3.0438311688311688e-05*w9*Jinv00-7.3051948051948052e-05*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+7.3051948051948055e-04*Jinv02*w27+4.9310064935064932e-04*w6*Jinv02+6.5746753246753243e-04*w19*Jinv12-7.3051948051948052e-05*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+3.0438311688311688e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+3.0438311688311688e-05*w58*Jinv12+6.5746753246753243e-04*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.2175324675324675e-05*w57*Jinv00-1.2175324675324675e-05*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+3.0438311688311688e-05*w0*Jinv00+1.1323051948051949e-03*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+3.0438311688311688e-05*Jinv01*w9+2.1915584415584417e-04*Jinv20*w17-7.3051948051948052e-05*w30*Jinv00+4.3831168831168834e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+7.3051948051948055e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14+1.1323051948051949e-03*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20-4.2004870129870131e-04*Jinv01*w30-7.3051948051948052e-05*Jinv02*w51+2.1915584415584417e-04*Jinv22*w56+7.3051948051948055e-04*Jinv12*w28-9.8620129870129864e-04*w25*Jinv11+6.5746753246753243e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+4.3831168831168834e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+4.8701298701298701e-04*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+3.0438311688311688e-05*Jinv12*w1+3.0438311688311688e-05*w10*Jinv11)*D;
  A[29][45]=(0.0)*D;
  A[29][46]=(0.0)*D;
  A[29][47]=(-5.4788961038961036e-04*w40*Jinv12+9.1314935064935065e-06*Jinv22*w53+6.4935064935064935e-05*w57*Jinv02-5.4788961038961036e-04*Jinv02*w39-7.6704545454545450e-04*Jinv21*w44+6.3920454545454550e-05*Jinv01*w3+9.1314935064935065e-06*Jinv22*w38-5.6818181818181818e-05*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12+6.4935064935064935e-05*w0*Jinv02-9.2329545454545453e-05*Jinv21*w2-7.6704545454545450e-04*w42*Jinv01-9.2329545454545453e-05*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17-9.1314935064935068e-05*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-5.4788961038961036e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12+1.0957792207792207e-03*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-5.6818181818181818e-05*w58*Jinv11-5.6818181818181818e-05*w57*Jinv01-4.5657467532467534e-05*Jinv21*w26-9.2329545454545453e-05*w0*Jinv01-5.4788961038961036e-04*Jinv22*w20-7.6704545454545450e-04*w43*Jinv11-7.6704545454545450e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+6.4935064935064935e-05*w11*Jinv22-9.2329545454545453e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01+2.8409090909090908e-04*w28*Jinv11-5.4788961038961036e-04*Jinv22*w17+1.0957792207792207e-03*w22*Jinv12+9.1314935064935065e-06*Jinv02*w3+9.1314935064935065e-06*w32*Jinv22-7.6704545454545450e-04*Jinv01*w39+6.4935064935064935e-05*Jinv22*w2+1.2784090909090910e-04*Jinv21*w38+1.0957792207792207e-03*Jinv02*w45-4.5657467532467534e-05*Jinv01*w24+6.4935064935064935e-05*Jinv22*w59-6.7573051948051951e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11-2.1002435064935065e-04*Jinv21*w53-4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+6.3920454545454550e-05*w4*Jinv11-5.4788961038961036e-04*Jinv12*w13+1.2784090909090910e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+9.1314935064935065e-06*w37*Jinv12-5.4788961038961036e-04*Jinv22*w41+9.1314935064935065e-06*w52*Jinv12-6.7573051948051951e-04*Jinv21*w35+1.2784090909090910e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26-9.1314935064935068e-05*w46*Jinv11+6.3920454545454550e-05*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+9.1314935064935065e-06*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02+2.8409090909090908e-04*Jinv21*w29-6.7573051948051951e-04*Jinv01*w33-5.4788961038961036e-04*Jinv02*w12+1.1870941558441559e-04*Jinv01*w18+1.2784090909090910e-04*w37*Jinv11+6.4935064935064935e-05*Jinv02*w9+6.3920454545454550e-05*w6*Jinv01-9.1314935064935068e-05*w47*Jinv21+1.0957792207792207e-03*Jinv22*w23-2.1002435064935065e-04*w52*Jinv11+9.1314935064935065e-06*w31*Jinv12+1.0957792207792207e-03*w46*Jinv12+9.1314935064935065e-06*Jinv22*w8+9.1314935064935065e-06*Jinv22*w50-4.5657467532467534e-05*w22*Jinv11+9.1314935064935065e-06*Jinv12*w7-4.5657467532467534e-05*w21*Jinv01-2.1002435064935065e-04*Jinv01*w51+9.1314935064935065e-06*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49+1.2784090909090910e-04*w36*Jinv01+6.3920454545454550e-05*w7*Jinv11-1.4610389610389610e-04*Jinv21*w56+1.5827922077922079e-03*Jinv02*w27+9.1314935064935065e-06*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+9.1314935064935065e-06*Jinv22*w5+1.0957792207792207e-03*w47*Jinv22-2.1002435064935065e-04*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+6.4935064935064935e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+1.1870941558441559e-04*Jinv01*w12+6.3920454545454550e-05*Jinv21*w8-4.9310064935064932e-04*Jinv02*w33-4.5657467532467534e-05*Jinv21*w23-1.4610389610389610e-04*Jinv01*w54-1.4610389610389610e-04*w55*Jinv11+9.1314935064935065e-06*w48*Jinv02-9.2329545454545453e-05*Jinv01*w9+1.5827922077922079e-03*Jinv22*w29-5.4788961038961036e-04*Jinv22*w14-7.6704545454545450e-04*w40*Jinv11+9.1314935064935065e-06*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.2784090909090910e-04*Jinv01*w30+9.1314935064935065e-06*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+1.5827922077922079e-03*Jinv12*w28-4.5657467532467534e-05*w25*Jinv11-5.4788961038961036e-04*w16*Jinv12-2.1002435064935065e-04*Jinv21*w50+2.8409090909090908e-04*Jinv01*w27-2.1002435064935065e-04*w48*Jinv01+6.4935064935064935e-05*Jinv12*w1-9.2329545454545453e-05*w10*Jinv11)*D;
  A[29][48]=(0.0)*D;
  A[29][49]=(0.0)*D;
  A[29][50]=(1.3697240259740259e-04*Jinv00*w12-4.0584415584415584e-05*w28*Jinv10+5.6615259740259744e-04*w40*Jinv12-3.7439123376623376e-04*Jinv22*w53+1.0957792207792208e-04*Jinv20*w8-1.5726461038961039e-04*w57*Jinv02+1.3697240259740259e-04*w13*Jinv10+5.6615259740259744e-04*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+4.2918019480519480e-04*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3-9.1314935064935065e-06*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38-7.8530844155844158e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59-1.1668019480519480e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21+3.6525974025974026e-05*w16*Jinv11+3.6525974025974026e-05*Jinv21*w17+4.7483766233766234e-04*Jinv01*w45-3.9265422077922079e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.6525974025974026e-05*w15*Jinv02-7.4878246753246752e-04*w48*Jinv00+6.9399350649350649e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10+3.6525974025974026e-05*w16*Jinv10-4.9310064935064932e-04*w43*Jinv11+4.7483766233766234e-04*w45*Jinv00-4.9310064935064932e-04*Jinv21*w41+3.6525974025974026e-05*w42*Jinv02-4.0584415584415584e-05*Jinv00*w27-1.0247564935064935e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11+3.6525974025974026e-05*w15*Jinv01-4.0584415584415584e-05*w28*Jinv11-7.4878246753246752e-04*Jinv20*w50-4.0584415584415584e-05*Jinv20*w29+4.7483766233766234e-04*w46*Jinv10-3.6525974025974026e-05*Jinv22*w17-2.7394480519480518e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12+4.2918019480519480e-04*w31*Jinv10-3.1960227272727273e-04*Jinv02*w3-9.1314935064935065e-06*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-4.9310064935064932e-04*Jinv01*w39-1.1668019480519480e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38-9.1314935064935065e-06*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24-1.5726461038961039e-04*Jinv22*w59-2.7394480519480518e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00+6.9399350649350649e-04*Jinv12*w34+3.6525974025974026e-05*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20-1.7349837662337662e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-2.7394480519480518e-04*w34*Jinv10+4.2918019480519480e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+1.0957792207792208e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12-7.8530844155844158e-04*Jinv20*w56+5.6615259740259744e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-3.7439123376623376e-04*w52*Jinv12-2.7394480519480518e-04*Jinv21*w35+4.2918019480519480e-04*w31*Jinv11+4.7483766233766234e-04*w46*Jinv11+1.3697240259740259e-04*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21-3.9265422077922079e-04*Jinv02*w54-3.1960227272727273e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02-7.4878246753246752e-04*w49*Jinv10-4.0584415584415584e-05*Jinv21*w29-2.7394480519480518e-04*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00-1.7349837662337662e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11+1.0957792207792208e-04*w6*Jinv01+1.0957792207792208e-04*w7*Jinv10+4.7483766233766234e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-9.1314935064935065e-06*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53-1.0957792207792208e-04*Jinv22*w8-9.1314935064935065e-06*w22*Jinv10-9.1314935064935065e-06*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-7.8530844155844158e-04*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.0957792207792208e-04*w6*Jinv02-3.1960227272727273e-04*Jinv22*w5-9.1314935064935065e-06*w47*Jinv22-7.4878246753246752e-04*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-1.5726461038961039e-04*w58*Jinv12+1.3697240259740259e-04*Jinv01*w12+1.0957792207792208e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59+6.9399350649350649e-04*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00-2.7394480519480518e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-7.8530844155844158e-04*Jinv01*w54-7.8530844155844158e-04*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2+3.6525974025974026e-05*w15*Jinv00-1.0247564935064935e-04*Jinv01*w9+3.6525974025974026e-05*Jinv20*w17+4.2918019480519480e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20-1.7349837662337662e-04*Jinv22*w14-4.9310064935064932e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.6525974025974026e-05*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+4.2918019480519480e-04*Jinv01*w30-3.7439123376623376e-04*Jinv02*w51-3.9265422077922079e-04*Jinv22*w56-9.1314935064935065e-06*w25*Jinv11-7.8530844155844158e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00+4.7483766233766234e-04*w47*Jinv20-3.6525974025974026e-05*w16*Jinv12-7.4878246753246752e-04*Jinv21*w50-4.0584415584415584e-05*Jinv01*w27-7.4878246753246752e-04*w48*Jinv01-1.1668019480519480e-04*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[29][51]=(0.0)*D;
  A[29][52]=(0.0)*D;
  A[29][53]=(-1.3697240259740259e-04*Jinv00*w12+4.0584415584415584e-05*w28*Jinv10+5.2962662337662338e-04*w40*Jinv12+7.4878246753246752e-04*Jinv22*w53-1.0957792207792208e-04*Jinv20*w8-9.2329545454545453e-05*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10+5.2962662337662338e-04*Jinv02*w39-1.0957792207792208e-04*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23-1.5523538961038962e-04*Jinv22*w38+7.8530844155844158e-04*Jinv00*w54+1.8262987012987013e-05*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+4.9310064935064932e-04*w43*Jinv10+3.9265422077922079e-04*Jinv12*w55+6.4935064935064935e-05*w58*Jinv10-7.3051948051948052e-05*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12+7.4878246753246752e-04*w48*Jinv00+1.8262987012987013e-05*Jinv02*w24+9.6793831168831166e-04*Jinv22*w35+1.0247564935064935e-04*w1*Jinv10-3.6525974025974026e-05*w16*Jinv10-3.1047077922077924e-04*Jinv22*w20-4.7483766233766234e-04*w45*Jinv00+1.0592532467532468e-03*w42*Jinv02+4.0584415584415584e-05*Jinv00*w27-1.4204545454545455e-05*w11*Jinv22+7.4878246753246752e-04*Jinv20*w50+4.0584415584415584e-05*Jinv20*w29-4.7483766233766234e-04*w46*Jinv10-7.3051948051948052e-05*Jinv22*w17+2.7394480519480518e-04*Jinv00*w33+9.1314935064935065e-06*w22*Jinv12-4.2918019480519480e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00-3.1960227272727273e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2-1.3697240259740259e-04*Jinv00*w18-4.2918019480519480e-04*w36*Jinv00-4.8396915584415583e-04*Jinv02*w45-9.2329545454545453e-05*Jinv22*w59+7.4878246753246752e-04*w51*Jinv00+9.6793831168831166e-04*Jinv12*w34+1.0592532467532468e-03*Jinv22*w44-1.0957792207792208e-04*w4*Jinv10+4.9310064935064932e-04*w41*Jinv20-1.3697240259740259e-04*Jinv12*w13-1.3697240259740259e-04*w19*Jinv10+2.7394480519480518e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00-1.5523538961038962e-04*w37*Jinv12+7.8530844155844158e-04*Jinv20*w56+5.2962662337662338e-04*Jinv22*w41+9.1314935064935065e-06*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20+7.4878246753246752e-04*w52*Jinv12+1.8262987012987013e-05*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14+3.9265422077922079e-04*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+9.1314935064935065e-06*w21*Jinv02+7.4878246753246752e-04*w49*Jinv10+4.9310064935064932e-04*w39*Jinv00-1.3697240259740259e-04*Jinv02*w12-1.4204545454545455e-05*Jinv02*w9-1.0957792207792208e-04*w7*Jinv10-4.2918019480519480e-04*Jinv20*w38+9.1314935064935065e-06*Jinv22*w23-3.1960227272727273e-04*w31*Jinv12-4.8396915584415583e-04*w46*Jinv12+7.4878246753246752e-04*Jinv20*w53-4.2918019480519480e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10+3.7439123376623376e-04*Jinv22*w50-4.2918019480519480e-04*Jinv12*w7-3.1960227272727273e-04*Jinv02*w30+3.7439123376623376e-04*Jinv12*w49+7.4878246753246752e-04*w52*Jinv10+1.0247564935064935e-04*w9*Jinv00-4.2918019480519480e-04*w37*Jinv10+9.1314935064935065e-06*w24*Jinv00+4.0584415584415584e-05*Jinv02*w27-4.2918019480519480e-04*w6*Jinv02-3.1047077922077924e-04*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5-4.8396915584415583e-04*w47*Jinv22+1.0247564935064935e-04*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-9.2329545454545453e-05*w58*Jinv12-3.1047077922077924e-04*Jinv02*w18+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+9.6793831168831166e-04*Jinv02*w33+1.0247564935064935e-04*w0*Jinv00+2.7394480519480518e-04*w35*Jinv20+9.1314935064935065e-06*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2-3.6525974025974026e-05*w15*Jinv00+3.7439123376623376e-04*w48*Jinv02-3.6525974025974026e-05*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-1.3697240259740259e-04*w20*Jinv20+4.0584415584415584e-05*Jinv22*w29-1.3697240259740259e-04*Jinv22*w14-1.5523538961038962e-04*Jinv02*w36+1.0592532467532468e-03*Jinv12*w43-1.0957792207792208e-04*w5*Jinv20+7.4878246753246752e-04*Jinv02*w51+3.9265422077922079e-04*Jinv22*w56+4.0584415584415584e-05*Jinv12*w28+7.8530844155844158e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00-4.7483766233766234e-04*w47*Jinv20-7.3051948051948052e-05*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[29][54]=(0.0)*D;
  A[29][55]=(0.0)*D;
  A[29][56]=(4.3831168831168834e-04*w40*Jinv12+3.9265422077922079e-04*Jinv22*w53-5.6818181818181818e-05*w57*Jinv02+4.3831168831168834e-04*Jinv02*w39+4.9310064935064932e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3-1.7349837662337662e-04*Jinv22*w38+6.4935064935064935e-05*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-1.4204545454545455e-05*w0*Jinv02+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21-3.6525974025974026e-05*w16*Jinv11-3.6525974025974026e-05*Jinv21*w17-4.7483766233766234e-04*Jinv01*w45+6.5746753246753243e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.4204545454545455e-05*w10*Jinv12-5.4788961038961036e-04*Jinv02*w24+5.2962662337662338e-04*Jinv22*w35+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01+9.1314935064935065e-06*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+2.1915584415584417e-04*Jinv22*w20+4.9310064935064932e-04*w43*Jinv11+4.9310064935064932e-04*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02-1.4204545454545455e-05*w11*Jinv22+1.0247564935064935e-04*Jinv11*w1-1.3697240259740259e-04*w13*Jinv11-3.6525974025974026e-05*w15*Jinv01+4.0584415584415584e-05*w28*Jinv11+2.1915584415584417e-04*Jinv22*w17-5.4788961038961036e-04*w22*Jinv12-1.3697240259740259e-04*Jinv02*w3-1.7349837662337662e-04*w32*Jinv22+4.9310064935064932e-04*Jinv01*w39-1.4204545454545455e-05*Jinv22*w2-4.2918019480519480e-04*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45+9.1314935064935065e-06*Jinv01*w24-5.6818181818181818e-05*Jinv22*w59+2.7394480519480518e-04*w34*Jinv11-1.3697240259740259e-04*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53+5.2962662337662338e-04*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44-1.0957792207792208e-04*w4*Jinv11+2.1915584415584417e-04*Jinv12*w13-4.2918019480519480e-04*w32*Jinv21-1.3697240259740259e-04*Jinv21*w20-1.7349837662337662e-04*w37*Jinv12+4.3831168831168834e-04*Jinv22*w41+3.9265422077922079e-04*w52*Jinv12+2.7394480519480518e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-4.7483766233766234e-04*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21+6.5746753246753243e-04*Jinv02*w54-1.3697240259740259e-04*w4*Jinv12-5.4788961038961036e-04*w21*Jinv02+4.0584415584415584e-05*Jinv21*w29+2.7394480519480518e-04*Jinv01*w33+2.1915584415584417e-04*Jinv02*w12-1.3697240259740259e-04*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-1.4204545454545455e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01-4.7483766233766234e-04*w47*Jinv21-5.4788961038961036e-04*Jinv22*w23+7.4878246753246752e-04*w52*Jinv11-1.7349837662337662e-04*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12-1.3697240259740259e-04*Jinv22*w8+3.9265422077922079e-04*Jinv22*w50+9.1314935064935065e-06*w22*Jinv11-1.3697240259740259e-04*Jinv12*w7+9.1314935064935065e-06*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-1.7349837662337662e-04*Jinv02*w30+3.9265422077922079e-04*Jinv12*w49-4.2918019480519480e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11+7.8530844155844158e-04*Jinv21*w56-6.0876623376623375e-04*Jinv02*w27-1.3697240259740259e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-1.3697240259740259e-04*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22+7.4878246753246752e-04*w49*Jinv11-1.3697240259740259e-04*Jinv21*w14-5.6818181818181818e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-1.3697240259740259e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+5.2962662337662338e-04*Jinv02*w33+9.1314935064935065e-06*Jinv21*w23+7.8530844155844158e-04*Jinv01*w54+7.8530844155844158e-04*w55*Jinv11+3.9265422077922079e-04*w48*Jinv02+1.0247564935064935e-04*Jinv01*w9-6.0876623376623375e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14+4.9310064935064932e-04*w40*Jinv11-1.7349837662337662e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43-4.2918019480519480e-04*Jinv01*w30+3.9265422077922079e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-6.0876623376623375e-04*Jinv12*w28+9.1314935064935065e-06*w25*Jinv11+2.1915584415584417e-04*w16*Jinv12+7.4878246753246752e-04*Jinv21*w50+4.0584415584415584e-05*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01-1.4204545454545455e-05*Jinv12*w1+1.0247564935064935e-04*w10*Jinv11)*D;
  A[29][57]=(0.0)*D;
  A[29][58]=(0.0)*D;
  A[29][59]=(-4.2613636363636367e-05*w40*Jinv12+1.5726461038961039e-04*Jinv22*w53+9.4696969696969697e-05*w57*Jinv02-4.2613636363636367e-05*Jinv02*w39-1.1668019480519480e-04*Jinv22*w38+6.4935064935064935e-05*w25*Jinv12+5.4112554112554113e-05*w0*Jinv02+2.0495129870129870e-04*Jinv12*w55+3.0438311688311688e-05*w15*Jinv02+5.4112554112554113e-05*w10*Jinv12+6.4935064935064935e-05*Jinv02*w24-1.9480519480519481e-04*Jinv22*w35-1.4204545454545455e-05*Jinv22*w20-4.2613636363636367e-05*w42*Jinv02+5.4112554112554113e-05*w11*Jinv22+3.0438311688311688e-05*Jinv22*w17+6.4935064935064935e-05*w22*Jinv12+1.0247564935064935e-04*Jinv02*w3-1.1668019480519480e-04*w32*Jinv22+5.4112554112554113e-05*Jinv22*w2+8.3198051948051951e-05*Jinv02*w45+9.4696969696969697e-05*Jinv22*w59-1.9480519480519481e-04*Jinv12*w34-4.2613636363636367e-05*Jinv22*w44-1.4204545454545455e-05*Jinv12*w13-1.1668019480519480e-04*w37*Jinv12-4.2613636363636367e-05*Jinv22*w41+1.5726461038961039e-04*w52*Jinv12+6.4935064935064935e-05*Jinv22*w26+2.0495129870129870e-04*Jinv02*w54+1.0247564935064935e-04*w4*Jinv12+6.4935064935064935e-05*w21*Jinv02-1.4204545454545455e-05*Jinv02*w12+5.4112554112554113e-05*Jinv02*w9+6.4935064935064935e-05*Jinv22*w23-1.1668019480519480e-04*w31*Jinv12+8.3198051948051951e-05*w46*Jinv12+1.0247564935064935e-04*Jinv22*w8+1.5726461038961039e-04*Jinv22*w50+1.0247564935064935e-04*Jinv12*w7-1.1668019480519480e-04*Jinv02*w30+1.5726461038961039e-04*Jinv12*w49+1.8939393939393939e-04*Jinv02*w27+1.0247564935064935e-04*w6*Jinv02-1.4204545454545455e-05*w19*Jinv12+1.0247564935064935e-04*Jinv22*w5+8.3198051948051951e-05*w47*Jinv22+9.4696969696969697e-05*w58*Jinv12-1.4204545454545455e-05*Jinv02*w18-1.9480519480519481e-04*Jinv02*w33+1.5726461038961039e-04*w48*Jinv02+1.8939393939393939e-04*Jinv22*w29-1.4204545454545455e-05*Jinv22*w14-1.1668019480519480e-04*Jinv02*w36-4.2613636363636367e-05*Jinv12*w43+1.5726461038961039e-04*Jinv02*w51+2.0495129870129870e-04*Jinv22*w56+1.8939393939393939e-04*Jinv12*w28+3.0438311688311688e-05*w16*Jinv12+5.4112554112554113e-05*Jinv12*w1)*D;
  A[30][0]=(-1.1870941558441558e-03*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+2.6481331168831169e-04*Jinv22*w53+6.7573051948051951e-04*Jinv20*w8-2.0495129870129870e-04*w57*Jinv02-1.1870941558441558e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-1.1870941558441558e-03*w3*Jinv00-2.3741883116883116e-03*w32*Jinv20-1.1870941558441558e-03*Jinv01*w3+6.7573051948051951e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+2.6481331168831169e-04*Jinv00*w54-2.0495129870129870e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+2.6481331168831169e-04*Jinv12*w55-2.0495129870129870e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12+1.0227272727272728e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-2.0495129870129870e-04*w58*Jinv11-2.0495129870129870e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+1.0957792207792208e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-1.1870941558441558e-03*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+1.0227272727272728e-03*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+1.0957792207792208e-04*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+6.7573051948051951e-04*w22*Jinv12-2.3741883116883116e-03*w31*Jinv10-1.1870941558441558e-03*Jinv02*w3+6.7573051948051951e-04*w21*Jinv00-2.3741883116883116e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+1.0957792207792208e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-2.0495129870129870e-04*Jinv22*w59-6.5746753246753243e-04*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+2.6481331168831169e-04*Jinv21*w53+2.6481331168831169e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-1.1870941558441558e-03*w4*Jinv10-1.1870941558441558e-03*w4*Jinv11-6.5746753246753243e-04*w41*Jinv20-1.1870941558441558e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-2.3741883116883116e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+6.7573051948051951e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+2.6481331168831169e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+2.6481331168831169e-04*w52*Jinv12-6.5746753246753243e-04*Jinv21*w35-2.3741883116883116e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-1.1870941558441558e-03*Jinv20*w14-1.1870941558441558e-03*w5*Jinv21+2.6481331168831169e-04*Jinv02*w54-1.1870941558441558e-03*w4*Jinv12+6.7573051948051951e-04*w21*Jinv02+1.0227272727272728e-03*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-6.5746753246753243e-04*Jinv01*w33-6.5746753246753243e-04*w39*Jinv00-1.1870941558441558e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.0957792207792208e-04*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9+6.7573051948051951e-04*w6*Jinv01+6.7573051948051951e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38+6.7573051948051951e-04*Jinv22*w23+2.6481331168831169e-04*w52*Jinv11-2.3741883116883116e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.6481331168831169e-04*Jinv20*w53+6.7573051948051951e-04*Jinv22*w8+6.7573051948051951e-04*w22*Jinv10+1.0227272727272728e-03*Jinv22*w50+6.7573051948051951e-04*w22*Jinv11+6.7573051948051951e-04*Jinv12*w7+6.7573051948051951e-04*w21*Jinv01+2.6481331168831169e-04*Jinv01*w51-2.3741883116883116e-03*Jinv02*w30+1.0227272727272728e-03*Jinv12*w49+1.0957792207792208e-04*w36*Jinv01+2.6481331168831169e-04*w52*Jinv10+6.7573051948051951e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+2.6481331168831169e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+6.7573051948051951e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-1.1870941558441558e-03*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.0227272727272728e-03*w49*Jinv11-1.1870941558441558e-03*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-2.0495129870129870e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.1870941558441558e-03*Jinv01*w12+6.7573051948051951e-04*Jinv21*w8-2.0495129870129870e-04*w57*Jinv00-2.0495129870129870e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+6.7573051948051951e-04*Jinv21*w23+2.6481331168831169e-04*Jinv01*w54+2.6481331168831169e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00+1.0227272727272728e-03*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9-3.2873376623376621e-04*Jinv20*w17-2.3741883116883116e-03*w30*Jinv00-6.5746753246753243e-04*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-1.1870941558441558e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-1.1870941558441558e-03*w5*Jinv20-2.3741883116883116e-03*Jinv01*w30+2.6481331168831169e-04*Jinv02*w51+2.6481331168831169e-04*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11+2.6481331168831169e-04*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+1.0957792207792208e-04*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+1.0227272727272728e-03*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+1.0227272727272728e-03*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[30][1]=(0.0)*D;
  A[30][2]=(0.0)*D;
  A[30][3]=(2.4655032467532467e-03*Jinv00*w12+3.7439123376623376e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-1.1505681818181819e-03*Jinv20*w8-3.9265422077922079e-04*w57*Jinv02+2.4655032467532467e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10-1.5888798701298701e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12-1.3971185064935066e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-3.1777597402597403e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+1.1414366883116883e-03*w1*Jinv10+5.4788961038961036e-04*w16*Jinv10+8.2183441558441553e-05*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02+3.7439123376623376e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.7439123376623376e-04*w28*Jinv11-1.3971185064935066e-03*Jinv20*w50+3.7439123376623376e-04*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-1.5888798701298701e-03*Jinv22*w17+1.0957792207792207e-03*Jinv00*w33+1.0135957792207792e-03*w22*Jinv12+4.9310064935064934e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-3.9265422077922079e-04*Jinv22*w59-3.1777597402597403e-03*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53-3.1777597402597403e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11+2.6846590909090908e-03*w41*Jinv20-1.2327516233766234e-03*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20-1.1505681818181819e-03*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-3.1777597402597403e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+2.4655032467532467e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.0135957792207792e-03*w21*Jinv02-1.3971185064935066e-03*w49*Jinv10-3.7439123376623376e-04*Jinv21*w29-3.1777597402597403e-03*Jinv01*w33+2.6846590909090908e-03*w39*Jinv00-1.2327516233766234e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01-1.1505681818181819e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+1.0135957792207792e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+1.2327516233766234e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50+1.0135957792207792e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7+1.0135957792207792e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.2327516233766234e-03*w7*Jinv11+3.5612824675324673e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56-3.7439123376623376e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+8.2183441558441553e-05*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.2327516233766234e-03*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+3.5612824675324673e-04*w10*Jinv10-3.9265422077922079e-04*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59-3.1777597402597403e-03*Jinv02*w33+1.1414366883116883e-03*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+1.0135957792207792e-03*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+1.1414366883116883e-03*Jinv20*w2+5.4788961038961036e-04*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9+5.4788961038961036e-04*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00+2.6846590909090908e-03*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-3.7439123376623376e-04*Jinv22*w29-1.2327516233766234e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-3.7439123376623376e-04*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11-8.2183441558441553e-05*w55*Jinv10-3.5612824675324673e-04*w47*Jinv20-1.5888798701298701e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.7439123376623376e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[30][4]=(0.0)*D;
  A[30][5]=(0.0)*D;
  A[30][6]=(-2.0271915584415584e-03*Jinv00*w12-3.1960227272727273e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+7.3965097402597403e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-2.0271915584415584e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-4.0543831168831168e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+1.1231737012987013e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+9.5880681818181818e-04*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-5.5702110389610384e-04*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+4.6570616883116886e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-3.1960227272727273e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+9.5880681818181818e-04*Jinv20*w50-3.1960227272727273e-04*Jinv20*w29+4.6570616883116886e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17-8.7662337662337668e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12-4.0543831168831168e-03*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+1.1231737012987013e-03*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+5.4788961038961036e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-8.2183441558441553e-05*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-2.5750811688311690e-03*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-8.7662337662337668e-04*w34*Jinv10+7.1225649350649346e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.3965097402597403e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41-2.7394480519480521e-05*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+7.1225649350649346e-04*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-8.2183441558441553e-05*w46*Jinv11-2.0271915584415584e-03*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02+9.5880681818181818e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-2.5750811688311690e-03*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+7.3965097402597403e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21+5.4788961038961036e-04*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12-8.2183441558441553e-05*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+1.1231737012987013e-03*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+5.4788961038961036e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-2.7394480519480521e-05*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-8.2183441558441553e-05*w47*Jinv22-5.2049512987012989e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33-5.5702110389610384e-04*w0*Jinv00-8.7662337662337668e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-2.7394480519480521e-05*w25*Jinv10-5.5702110389610384e-04*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17-4.0543831168831168e-03*w30*Jinv00-2.5750811688311690e-03*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+7.1225649350649346e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+4.6570616883116886e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12-5.2049512987012989e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27-5.2049512987012989e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[30][7]=(0.0)*D;
  A[30][8]=(0.0)*D;
  A[30][9]=(7.4878246753246752e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-2.6481331168831169e-04*Jinv20*w8+7.4878246753246752e-04*w13*Jinv10+3.6525974025974026e-05*w3*Jinv00+1.4975649350649350e-03*w32*Jinv20-4.2918019480519480e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+3.4699675324675324e-04*w43*Jinv10-1.4204545454545455e-05*w58*Jinv10-5.8441558441558442e-04*w48*Jinv00+6.4935064935064935e-05*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10-2.1915584415584417e-04*w45*Jinv00+1.0247564935064935e-04*Jinv00*w27-5.8441558441558442e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+4.3831168831168834e-04*Jinv00*w33+1.4975649350649350e-03*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-5.4788961038961042e-05*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00+1.1870941558441559e-04*w51*Jinv00+3.6525974025974026e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-5.4788961038961042e-05*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10-2.6481331168831169e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56-2.7394480519480521e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20+7.4878246753246752e-04*Jinv20*w14-5.8441558441558442e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-2.6481331168831169e-04*w7*Jinv10-1.0957792207792208e-04*Jinv20*w38+1.1870941558441559e-04*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10+1.1870941558441559e-04*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10-2.7394480519480521e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+3.4699675324675324e-04*Jinv20*w44-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-2.7394480519480521e-05*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00+2.1915584415584417e-04*Jinv20*w17+1.4975649350649350e-03*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+3.6525974025974026e-05*w5*Jinv20-4.2918019480519480e-04*w55*Jinv10+3.4699675324675324e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20)*D;
  A[30][10]=(0.0)*D;
  A[30][11]=(0.0)*D;
  A[30][12]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-3.1777597402597403e-03*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53+1.0135957792207792e-03*Jinv20*w8-3.9265422077922079e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-3.1777597402597403e-03*Jinv02*w39-1.2327516233766234e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20+2.4655032467532467e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.9265422077922079e-04*Jinv21*w59+8.2183441558441553e-05*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+1.1414366883116883e-03*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10+3.7439123376623376e-04*w11*Jinv21+5.4788961038961036e-04*w16*Jinv11+5.4788961038961036e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10-1.5888798701298701e-03*w15*Jinv02-3.7439123376623376e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.1414366883116883e-03*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+1.6436688311688311e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-3.7439123376623376e-04*w11*Jinv22+1.1414366883116883e-03*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+5.4788961038961036e-04*w15*Jinv01+3.5612824675324673e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.5888798701298701e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10-1.2327516233766234e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22+1.0957792207792207e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-3.9265422077922079e-04*Jinv22*w59+2.6846590909090908e-03*w34*Jinv11-8.2183441558441553e-05*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44-1.2327516233766234e-03*w4*Jinv10+2.4655032467532467e-03*w4*Jinv11-3.1777597402597403e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21+1.0135957792207792e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-3.1777597402597403e-03*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-3.7439123376623376e-04*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+2.6846590909090908e-03*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11+8.2183441558441553e-05*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14+2.4655032467532467e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-1.2327516233766234e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10+3.5612824675324673e-04*Jinv21*w29+2.6846590909090908e-03*Jinv01*w33-3.1777597402597403e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-3.5612824675324673e-04*w37*Jinv11-3.7439123376623376e-04*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01+1.0135957792207792e-03*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53+1.0135957792207792e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50-1.1505681818181819e-03*w22*Jinv11+1.0135957792207792e-03*Jinv12*w7-1.1505681818181819e-03*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11-3.7439123376623376e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+8.2183441558441553e-05*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+1.0135957792207792e-03*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-1.2327516233766234e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-1.3971185064935066e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-3.7439123376623376e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44-3.9265422077922079e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.1505681818181819e-03*Jinv21*w23+8.2183441558441553e-05*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02+3.7439123376623376e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-3.1777597402597403e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14+1.0957792207792207e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43-1.2327516233766234e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.5888798701298701e-03*w16*Jinv12-1.3971185064935066e-03*Jinv21*w50+3.5612824675324673e-04*Jinv01*w27-1.3971185064935066e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1+3.7439123376623376e-04*w10*Jinv11)*D;
  A[30][13]=(0.0)*D;
  A[30][14]=(0.0)*D;
  A[30][15]=(3.1777597402597403e-03*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53-9.3141233766233771e-04*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+1.0409902597402598e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3-1.7532467532467534e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+5.4788961038961042e-05*Jinv12*w55+5.2962662337662338e-04*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12-1.7532467532467534e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01-5.4788961038961042e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+3.2873376623376621e-03*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.0409902597402598e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11-1.7532467532467534e-03*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10-1.0957792207792207e-03*Jinv02*w3-1.7532467532467534e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+3.6525974025974026e-05*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+3.2873376623376621e-03*Jinv22*w44+1.0409902597402598e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20-1.0957792207792207e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-9.3141233766233771e-04*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+3.2873376623376621e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+3.1777597402597403e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54-1.0957792207792207e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02-1.7532467532467534e-03*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00-1.0957792207792207e-03*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.7532467532467534e-03*w6*Jinv01-9.3141233766233771e-04*w7*Jinv10+3.8352272727272725e-04*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-1.7532467532467534e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-9.3141233766233771e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-9.3141233766233771e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10-1.7532467532467534e-03*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-3.2873376623376621e-04*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-1.0957792207792207e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22-1.7532467532467534e-03*w49*Jinv11+1.0409902597402598e-03*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18+1.0409902597402598e-03*Jinv01*w12-1.7532467532467534e-03*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20-9.3141233766233771e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10+3.6525974025974026e-05*Jinv22*w29-1.0957792207792207e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+3.2873376623376621e-03*Jinv12*w43+1.0409902597402598e-03*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-1.7532467532467534e-03*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-1.7532467532467534e-03*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[30][16]=(0.0)*D;
  A[30][17]=(0.0)*D;
  A[30][18]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-8.2183441558441553e-05*Jinv20*w8-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv21*w44+1.9176136363636362e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+5.2049512987012989e-04*Jinv20*w23+5.4788961038961036e-04*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-8.7662337662337668e-04*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-5.4788961038961042e-05*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-1.3697240259740259e-04*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+5.2049512987012989e-04*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38+2.7394480519480518e-04*Jinv01*w24-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00+1.9176136363636362e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+5.4788961038961036e-04*Jinv20*w56+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20-7.6704545454545450e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+8.2183441558441553e-05*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-3.5612824675324673e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+5.2049512987012989e-04*w52*Jinv11+2.4655032467532466e-04*Jinv20*w53+5.2049512987012989e-04*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+6.3920454545454550e-05*w10*Jinv10-8.7662337662337668e-04*Jinv20*w44-3.5612824675324673e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-7.1225649350649346e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+1.9176136363636362e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+2.7394480519480518e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10-8.7662337662337668e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[30][19]=(0.0)*D;
  A[30][20]=(0.0)*D;
  A[30][21]=(1.6436688311688311e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+7.1225649350649346e-04*w32*Jinv20-2.0271915584415584e-03*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-8.2183441558441553e-05*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54-1.7349837662337662e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-5.5702110389610384e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-3.1960227272727273e-04*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+5.4788961038961036e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12-5.2049512987012989e-04*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-5.5702110389610384e-04*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-8.7662337662337668e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22-5.5702110389610384e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11-5.2049512987012989e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+7.1225649350649346e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22-8.7662337662337668e-04*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00+4.6570616883116886e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-2.5750811688311690e-03*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10-2.0271915584415584e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-4.0543831168831168e-03*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-8.2183441558441553e-05*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-2.5750811688311690e-03*Jinv21*w35-4.0543831168831168e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+5.4788961038961036e-04*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-2.0271915584415584e-03*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-5.2049512987012989e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29-2.5750811688311690e-03*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-2.7394480519480521e-05*Jinv01*w18+4.6570616883116886e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9+1.1231737012987013e-03*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+5.4788961038961036e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50+7.3965097402597403e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7+7.3965097402597403e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+4.6570616883116886e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+1.1231737012987013e-03*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+9.5880681818181818e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+1.1231737012987013e-03*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.3965097402597403e-04*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02-3.1960227272727273e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17+7.1225649350649346e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-8.7662337662337668e-04*w40*Jinv11-8.2183441558441553e-05*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20-4.0543831168831168e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12+9.5880681818181818e-04*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+9.5880681818181818e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-3.1960227272727273e-04*w10*Jinv11)*D;
  A[30][22]=(0.0)*D;
  A[30][23]=(0.0)*D;
  A[30][24]=(-1.6436688311688311e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-8.7662337662337668e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23+5.2049512987012989e-04*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10-8.7662337662337668e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-8.7662337662337668e-04*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1+1.9176136363636362e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+2.7394480519480518e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00-3.5612824675324673e-04*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00+5.2049512987012989e-04*Jinv20*w56-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-1.0957792207792208e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+2.4655032467532466e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+5.2049512987012989e-04*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38+5.4788961038961036e-04*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-8.2183441558441553e-05*w22*Jinv11-8.2183441558441553e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+5.4788961038961036e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+5.2049512987012989e-04*w7*Jinv11-1.0957792207792208e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11+1.9176136363636362e-04*Jinv21*w14-1.0957792207792208e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+1.9176136363636362e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.3697240259740259e-04*Jinv01*w9-3.8352272727272725e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11-3.5612824675324673e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-3.5612824675324673e-04*w25*Jinv11+5.2049512987012989e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-1.3697240259740259e-04*w10*Jinv11)*D;
  A[30][25]=(0.0)*D;
  A[30][26]=(0.0)*D;
  A[30][27]=(3.4699675324675324e-04*Jinv21*w44+7.4878246753246752e-04*Jinv01*w3-1.4204545454545455e-05*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2+3.4699675324675324e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+3.4699675324675324e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41+6.4935064935064935e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+4.3831168831168834e-04*Jinv01*w39-2.1915584415584417e-04*Jinv21*w38-5.4788961038961042e-05*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53+7.4878246753246752e-04*w4*Jinv11+1.4975649350649350e-03*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35+1.4975649350649350e-03*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11+7.4878246753246752e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-2.7394480519480521e-05*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-1.0957792207792208e-04*w47*Jinv21-4.2918019480519480e-04*w52*Jinv11-2.6481331168831169e-04*w22*Jinv11-2.6481331168831169e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51-2.1915584415584417e-04*w36*Jinv01-4.2918019480519480e-04*w7*Jinv11+1.1870941558441559e-04*Jinv21*w56-5.8441558441558442e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+3.6525974025974026e-05*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8-2.6481331168831169e-04*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+1.0247564935064935e-04*Jinv01*w9+4.3831168831168834e-04*w40*Jinv11+1.4975649350649350e-03*Jinv01*w30-5.4788961038961042e-05*w25*Jinv11-5.8441558441558442e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-5.8441558441558442e-04*w48*Jinv01+1.0247564935064935e-04*w10*Jinv11)*D;
  A[30][28]=(0.0)*D;
  A[30][29]=(0.0)*D;
  A[30][30]=(-2.4655032467532467e-03*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-7.3965097402597403e-04*Jinv22*w53+2.0271915584415584e-03*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-7.3965097402597406e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3+2.0271915584415584e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-5.7528409090909093e-04*Jinv00*w54-6.5746753246753243e-04*Jinv21*w59+1.1870941558441558e-03*w0*Jinv02+9.1314935064935068e-05*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+1.0683847402597404e-03*Jinv01*w45-7.3965097402597403e-04*Jinv12*w55-6.5746753246753243e-04*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02+1.3149350649350649e-03*w48*Jinv00-4.9310064935064932e-04*Jinv22*w35-6.5746753246753243e-04*w58*Jinv11-6.5746753246753243e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+9.1314935064935068e-05*w0*Jinv01+9.1314935064935068e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-1.6436688311688311e-03*w43*Jinv11+1.0683847402597404e-03*w45*Jinv00-4.7666396103896100e-03*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27+9.1314935064935068e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+1.0683847402597404e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-4.7666396103896100e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-7.3965097402597406e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3+2.0271915584415584e-03*w21*Jinv00-4.7666396103896100e-03*Jinv01*w39+1.1870941558441558e-03*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+1.0683847402597404e-03*w36*Jinv00+1.0683847402597404e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-4.7666396103896100e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.7528409090909093e-04*Jinv21*w53-5.7528409090909093e-04*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11-4.7666396103896100e-03*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-4.7666396103896100e-03*w34*Jinv10-7.3965097402597406e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+2.0271915584415584e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-5.7528409090909093e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-7.3965097402597403e-04*w52*Jinv12-4.7666396103896100e-03*Jinv21*w35-7.3965097402597406e-03*w31*Jinv11+1.0683847402597404e-03*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21-7.3965097402597403e-04*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.3149350649350649e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.7666396103896100e-03*Jinv01*w33-4.7666396103896100e-03*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.0683847402597404e-03*w37*Jinv11+2.0271915584415584e-03*w6*Jinv01+2.0271915584415584e-03*w7*Jinv10+1.0683847402597404e-03*w47*Jinv21+1.0683847402597404e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-5.7528409090909093e-04*w52*Jinv11+3.5612824675324673e-04*w46*Jinv12-5.7528409090909093e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+2.0271915584415584e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50+2.0271915584415584e-03*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+2.0271915584415584e-03*w21*Jinv01-5.7528409090909093e-04*Jinv01*w51-1.3149350649350649e-03*Jinv12*w49+1.0683847402597404e-03*w36*Jinv01-5.7528409090909093e-04*w52*Jinv10+2.0271915584415584e-03*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+1.0683847402597404e-03*w37*Jinv10-5.7528409090909093e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+1.2327516233766234e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.3149350649350649e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12-2.4655032467532467e-03*Jinv01*w12+2.0271915584415584e-03*Jinv21*w8-6.5746753246753243e-04*w57*Jinv00-6.5746753246753243e-04*Jinv20*w59-4.9310064935064932e-04*Jinv02*w33+9.1314935064935068e-05*w0*Jinv00-4.7666396103896100e-03*w35*Jinv20+2.0271915584415584e-03*Jinv21*w23-5.7528409090909093e-04*Jinv01*w54-5.7528409090909093e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+9.1314935064935068e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02-7.4878246753246752e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-7.3965097402597406e-03*w30*Jinv00-4.7666396103896100e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+1.2327516233766234e-03*Jinv22*w14-4.7666396103896100e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20-7.3965097402597406e-03*Jinv01*w30-7.3965097402597403e-04*Jinv02*w51-7.3965097402597403e-04*Jinv22*w56+3.5612824675324673e-04*w25*Jinv11-5.7528409090909093e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+1.0683847402597404e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12+1.3149350649350649e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01+1.1870941558441558e-03*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[30][31]=(0.0)*D;
  A[30][32]=(0.0)*D;
  A[30][33]=(3.1777597402597403e-03*Jinv00*w12+9.6793831168831166e-04*w28*Jinv10+2.9586038961038961e-03*w40*Jinv12+1.8628246753246754e-03*Jinv22*w53-1.8628246753246754e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+2.9586038961038961e-03*Jinv02*w39+4.9310064935064934e-03*Jinv21*w44+2.0819805194805196e-03*w3*Jinv00+9.5332792207792200e-03*w32*Jinv20-2.1915584415584414e-03*Jinv01*w3-2.5750811688311690e-03*Jinv20*w23-6.0267857142857139e-04*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12+4.3831168831168834e-04*w0*Jinv02+9.8620129870129864e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+4.9310064935064934e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-1.1505681818181819e-03*Jinv01*w45+9.3141233766233771e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+5.6615259740259744e-04*w10*Jinv12-2.7942370129870131e-03*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+9.8620129870129864e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-3.2873376623376621e-04*Jinv22*w20+4.9310064935064934e-03*w43*Jinv11-1.3149350649350649e-03*w45*Jinv00+1.9724025974025973e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02+9.6793831168831166e-04*Jinv00*w27+5.6615259740259744e-04*w11*Jinv22+9.8620129870129864e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.2004870129870131e-04*w28*Jinv11-2.7942370129870131e-03*Jinv20*w50+9.6793831168831166e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17+6.9034090909090911e-03*Jinv00*w33-1.2601461038961039e-03*w22*Jinv12+9.5332792207792200e-03*w31*Jinv10+2.0819805194805196e-03*Jinv02*w3-2.5750811688311690e-03*w21*Jinv00+9.8620129870129864e-04*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+4.3831168831168834e-04*Jinv22*w2+1.6436688311688311e-04*Jinv21*w38-1.2053571428571428e-03*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+2.0819805194805196e-03*w4*Jinv10-2.1915584415584414e-03*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20+1.0409902597402598e-03*Jinv12*w13+6.9034090909090911e-03*w34*Jinv10-3.2873376623376621e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20-1.8628246753246754e-03*w6*Jinv00-6.0267857142857139e-04*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+2.9586038961038961e-03*Jinv22*w41-8.2183441558441553e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20+1.8628246753246754e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35-3.2873376623376621e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26-1.1505681818181819e-03*w46*Jinv11+3.1777597402597403e-03*Jinv20*w14-2.1915584415584414e-03*w5*Jinv21+9.3141233766233771e-04*Jinv02*w54+2.0819805194805196e-03*w4*Jinv12-1.2601461038961039e-03*w21*Jinv02-2.7942370129870131e-03*w49*Jinv10+4.2004870129870131e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00+1.0409902597402598e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01-1.8628246753246754e-03*w7*Jinv10-1.1505681818181819e-03*w47*Jinv21-1.2601461038961039e-03*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11+9.8620129870129864e-04*w31*Jinv12-1.2053571428571428e-03*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8-2.5750811688311690e-03*w22*Jinv10+5.4788961038961042e-05*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51+9.8620129870129864e-04*Jinv02*w30+1.6436688311688311e-04*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10+7.6704545454545450e-04*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00+1.3149350649350649e-03*Jinv21*w56-8.2183441558441553e-04*w24*Jinv00+6.9399350649350649e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.2873376623376621e-04*w19*Jinv12+2.0819805194805196e-03*Jinv22*w5-1.2053571428571428e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-3.2873376623376621e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+5.9172077922077923e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00+6.9034090909090911e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11-8.2183441558441553e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+7.3051948051948052e-05*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+9.5332792207792200e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10+6.9399350649350649e-04*Jinv22*w29+1.0409902597402598e-03*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-6.0267857142857139e-04*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+2.0819805194805196e-03*w5*Jinv20-3.2873376623376621e-03*Jinv01*w30+1.8628246753246754e-03*Jinv02*w51+9.3141233766233771e-04*Jinv22*w56+6.9399350649350649e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11-8.2183441558441553e-04*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.2004870129870131e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01+4.3831168831168834e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[30][34]=(0.0)*D;
  A[30][35]=(0.0)*D;
  A[30][36]=(-7.1225649350649346e-04*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-1.3697240259740259e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23-7.1225649350649346e-04*Jinv22*w38+1.3971185064935066e-03*Jinv00*w54+2.7394480519480518e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+1.4793019480519481e-03*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35+1.8262987012987013e-05*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+2.4655032467532466e-04*w45*Jinv00-5.4788961038961042e-05*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.4793019480519481e-03*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10-3.8352272727272725e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-1.0683847402597404e-03*w36*Jinv00+8.2183441558441553e-05*Jinv02*w45-1.3697240259740259e-04*Jinv22*w59+8.2183441558441553e-05*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-1.6436688311688311e-04*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-7.1225649350649346e-04*w37*Jinv12+1.3971185064935066e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+4.6570616883116886e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+5.2049512987012989e-04*w52*Jinv12+2.7394480519480518e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02+1.4793019480519481e-03*w49*Jinv10-1.6436688311688311e-04*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10-1.0683847402597404e-03*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10+5.2049512987012989e-04*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+8.2183441558441553e-05*w52*Jinv10+1.2784090909090910e-04*w9*Jinv00-1.0683847402597404e-03*w37*Jinv10+4.6570616883116886e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22+1.2784090909090910e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-1.3697240259740259e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33+1.8262987012987013e-05*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+4.6570616883116886e-04*w25*Jinv10+1.8262987012987013e-05*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+5.2049512987012989e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-7.1225649350649346e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20+5.2049512987012989e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28+1.3971185064935066e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20-3.8352272727272725e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[30][37]=(0.0)*D;
  A[30][38]=(0.0)*D;
  A[30][39]=(-2.1915584415584414e-03*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12+9.3141233766233771e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-2.1915584415584414e-03*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-3.2873376623376621e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23-1.2053571428571428e-03*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59-3.2873376623376621e-04*w25*Jinv12+4.3831168831168834e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2+4.9310064935064934e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+9.6793831168831166e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+1.8628246753246754e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+6.9399350649350649e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-3.2873376623376621e-04*Jinv02*w24+2.9586038961038961e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-1.0957792207792208e-04*w0*Jinv01+9.8620129870129864e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+6.9034090909090911e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+6.9399350649350649e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.0819805194805196e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17+1.9724025974025973e-03*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12-3.2873376623376621e-03*w31*Jinv10+1.0409902597402598e-03*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+9.8620129870129864e-04*w32*Jinv22+6.9034090909090911e-03*Jinv01*w39+4.3831168831168834e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-1.1505681818181819e-03*w36*Jinv00-1.3149350649350649e-03*Jinv21*w38-6.0267857142857139e-04*Jinv02*w45+2.7394480519480518e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11-8.2183441558441553e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53+1.3149350649350649e-03*w51*Jinv00+2.9586038961038961e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+2.0819805194805196e-03*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10+9.5332792207792200e-03*w32*Jinv21-8.2183441558441553e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-1.2053571428571428e-03*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+4.2004870129870131e-04*w11*Jinv20+9.3141233766233771e-04*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35+9.5332792207792200e-03*w31*Jinv11-3.2873376623376621e-04*Jinv22*w26-2.1915584415584414e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+1.8628246753246754e-03*Jinv02*w54+1.0409902597402598e-03*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+2.0819805194805196e-03*Jinv02*w12-8.2183441558441553e-04*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+6.9399350649350649e-04*Jinv02*w9-2.5750811688311690e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-1.1505681818181819e-03*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11+9.8620129870129864e-04*w31*Jinv12-6.0267857142857139e-04*w46*Jinv12+1.3149350649350649e-03*Jinv20*w53-1.2601461038961039e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10-1.8628246753246754e-03*w22*Jinv11-1.2601461038961039e-03*Jinv12*w7-1.8628246753246754e-03*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51+9.8620129870129864e-04*Jinv02*w30-1.3149350649350649e-03*w36*Jinv01+1.3149350649350649e-03*w52*Jinv10-2.5750811688311690e-03*w7*Jinv11+4.2004870129870131e-04*w9*Jinv00-1.1505681818181819e-03*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+5.6615259740259744e-04*Jinv02*w27-1.2601461038961039e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12+1.0409902597402598e-03*Jinv22*w5-6.0267857142857139e-04*w47*Jinv22-2.7942370129870131e-03*w49*Jinv11+2.0819805194805196e-03*Jinv21*w14+4.2004870129870131e-04*w10*Jinv10+4.9310064935064934e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18+2.0819805194805196e-03*Jinv01*w12-2.5750811688311690e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+2.9586038961038961e-03*Jinv02*w33+9.8620129870129864e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-1.8628246753246754e-03*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+9.8620129870129864e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+9.6793831168831166e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-3.2873376623376621e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20+5.6615259740259744e-04*Jinv22*w29+2.0819805194805196e-03*Jinv22*w14+6.9034090909090911e-03*w40*Jinv11-1.2053571428571428e-03*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+9.5332792207792200e-03*Jinv01*w30+9.3141233766233771e-04*Jinv02*w51+1.8628246753246754e-03*Jinv22*w56+5.6615259740259744e-04*Jinv12*w28+2.6298701298701297e-03*w55*Jinv10+4.9310064935064934e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-2.7942370129870131e-03*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27-2.7942370129870131e-03*w48*Jinv01+4.3831168831168834e-04*Jinv12*w1+9.6793831168831166e-04*w10*Jinv11)*D;
  A[30][40]=(0.0)*D;
  A[30][41]=(0.0)*D;
  A[30][42]=(2.1915584415584414e-03*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-3.6525974025974026e-05*w57*Jinv02+2.1915584415584414e-03*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+3.2873376623376621e-03*w32*Jinv20+2.1915584415584414e-03*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-9.8620129870129864e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-5.4788961038961042e-05*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-9.8620129870129864e-04*w0*Jinv01-9.8620129870129864e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41-3.2873376623376621e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22-9.8620129870129864e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.2004870129870131e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+3.2873376623376621e-03*w31*Jinv10+1.0957792207792207e-03*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24-3.6525974025974026e-05*Jinv22*w59-3.9448051948051946e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.3149350649350649e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+2.1915584415584414e-03*w4*Jinv11-3.9448051948051946e-03*w41*Jinv20+1.0957792207792207e-03*Jinv12*w13+7.6704545454545450e-04*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+3.2873376623376621e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-4.2004870129870131e-04*w11*Jinv20-5.4788961038961042e-05*w52*Jinv12-3.9448051948051946e-03*Jinv21*w35+3.2873376623376621e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11+2.1915584415584414e-03*Jinv20*w14+2.1915584415584414e-03*w5*Jinv21-5.4788961038961042e-05*Jinv02*w54+1.0957792207792207e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-4.2004870129870131e-04*Jinv21*w29-3.9448051948051946e-03*Jinv01*w33-3.9448051948051946e-03*w39*Jinv00+1.0957792207792207e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.1505681818181819e-03*w47*Jinv21+1.1505681818181819e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-1.3149350649350649e-03*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11-4.2004870129870131e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10-1.3149350649350649e-03*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+1.0957792207792207e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-4.2004870129870131e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-3.6525974025974026e-05*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-9.8620129870129864e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10-9.8620129870129864e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+3.2873376623376621e-03*w30*Jinv00-3.9448051948051946e-03*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29+1.0957792207792207e-03*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+3.2873376623376621e-03*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.2004870129870131e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[30][43]=(0.0)*D;
  A[30][44]=(0.0)*D;
  A[30][45]=(-7.6704545454545450e-04*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-1.3697240259740259e-04*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+8.2183441558441553e-05*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+1.8262987012987013e-05*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-1.0683847402597404e-03*Jinv01*w45+5.2049512987012989e-04*Jinv12*w55-3.8352272727272725e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-3.5612824675324673e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+1.8262987012987013e-05*w0*Jinv01+2.7394480519480518e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02-1.0957792207792208e-04*w11*Jinv22+1.8262987012987013e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.8352272727272725e-04*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+2.4655032467532466e-04*Jinv21*w38-7.1225649350649346e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.3697240259740259e-04*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11+4.6570616883116886e-04*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21+4.6570616883116886e-04*Jinv21*w20+8.2183441558441553e-05*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26-1.0683847402597404e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+5.2049512987012989e-04*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12+4.6570616883116886e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01-1.0683847402597404e-03*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+1.3971185064935066e-03*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8+5.2049512987012989e-04*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11+8.2183441558441553e-05*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.4793019480519481e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-1.3697240259740259e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+5.2049512987012989e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-3.8352272727272725e-04*w16*Jinv12+1.4793019480519481e-03*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+1.4793019480519481e-03*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[30][46]=(0.0)*D;
  A[30][47]=(0.0)*D;
  A[30][48]=(1.6436688311688311e-04*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10+1.4793019480519481e-03*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+4.0178571428571428e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+1.4793019480519481e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-5.2049512987012989e-04*Jinv20*w23-9.5880681818181818e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.7394480519480521e-05*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02+4.7483766233766234e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+2.6298701298701297e-03*w48*Jinv00-2.7394480519480521e-05*Jinv02*w24+1.4793019480519481e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+4.7483766233766234e-04*w0*Jinv01+4.7483766233766234e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480521e-05*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27+2.7394480519480518e-04*w11*Jinv22+4.7483766233766234e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.6298701298701297e-03*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12-1.1505681818181819e-03*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3-5.2049512987012989e-04*w21*Jinv00+2.6298701298701297e-03*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38-9.5880681818181818e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+4.0178571428571428e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00+1.4793019480519481e-03*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-1.1505681818181819e-03*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-9.5880681818181818e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56+1.4793019480519481e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11-2.7394480519480521e-05*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02+2.6298701298701297e-03*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9-5.2049512987012989e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+2.6298701298701297e-03*w31*Jinv12-9.5880681818181818e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-5.2049512987012989e-04*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+2.6298701298701297e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-5.2049512987012989e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5-9.5880681818181818e-04*w47*Jinv22+2.6298701298701297e-03*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+4.0178571428571428e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-5.2049512987012989e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+1.4793019480519481e-03*Jinv02*w33+4.7483766233766234e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10+4.7483766233766234e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-9.5880681818181818e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30+1.1505681818181819e-03*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.6298701298701297e-03*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.6298701298701297e-03*w48*Jinv01-6.2094155844155847e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[30][49]=(0.0)*D;
  A[30][50]=(0.0)*D;
  A[30][51]=(-1.6436688311688311e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-2.0271915584415584e-03*Jinv22*w53+5.2049512987012989e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39-1.6436688311688311e-04*w3*Jinv00+1.1505681818181819e-03*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+4.6570616883116886e-04*w25*Jinv12-4.8396915584415583e-04*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10-1.0135957792207792e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-7.1225649350649346e-04*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00+4.6570616883116886e-04*Jinv02*w24-2.7394480519480520e-03*Jinv22*w35-4.7483766233766234e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-2.5202922077922079e-03*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-2.6298701298701297e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-7.1225649350649346e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.1505681818181819e-03*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-4.8396915584415583e-04*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+8.4922889610389610e-04*Jinv02*w45+1.1870941558441559e-04*Jinv22*w59-2.3833198051948050e-03*w51*Jinv00-2.7394480519480520e-03*Jinv12*w34-2.5202922077922079e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv10-2.1367694805194807e-03*w41*Jinv20+1.9176136363636362e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10+5.2049512987012989e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-2.0271915584415584e-03*w52*Jinv12+4.6570616883116886e-04*Jinv22*w26-1.6436688311688311e-04*Jinv20*w14-1.0135957792207792e-03*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10-2.1367694805194807e-03*w39*Jinv00+1.9176136363636362e-04*Jinv02*w12-1.7349837662337662e-04*Jinv02*w9+5.2049512987012989e-04*w7*Jinv10+1.3971185064935066e-03*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23+1.4793019480519481e-03*w31*Jinv12+8.4922889610389610e-04*w46*Jinv12-2.3833198051948050e-03*Jinv20*w53+1.3697240259740259e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+1.3697240259740259e-04*Jinv12*w7+1.4793019480519481e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49-2.3833198051948050e-03*w52*Jinv10-4.2918019480519480e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-3.1960227272727273e-04*Jinv02*w27+1.3697240259740259e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5+8.4922889610389610e-04*w47*Jinv22-4.2918019480519480e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-2.7394480519480520e-03*Jinv02*w33-4.7483766233766234e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10-4.7483766233766234e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.1505681818181819e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-3.1960227272727273e-04*Jinv22*w29+1.9176136363636362e-04*Jinv22*w14+4.3831168831168834e-04*Jinv02*w36-2.5202922077922079e-03*Jinv12*w43-1.6436688311688311e-04*w5*Jinv20-2.0271915584415584e-03*Jinv02*w51-1.0135957792207792e-03*Jinv22*w56-3.1960227272727273e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-7.1225649350649346e-04*w16*Jinv12-4.8396915584415583e-04*Jinv12*w1)*D;
  A[30][52]=(0.0)*D;
  A[30][53]=(0.0)*D;
  A[30][54]=(-2.7394480519480520e-03*w40*Jinv12-1.0135957792207792e-03*Jinv22*w53+1.1870941558441559e-04*w57*Jinv02-2.7394480519480520e-03*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-1.6436688311688311e-04*Jinv01*w3+8.4922889610389610e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12-4.8396915584415583e-04*w0*Jinv02-4.7483766233766234e-04*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-2.0271915584415584e-03*Jinv12*w55-7.1225649350649346e-04*w15*Jinv02-3.1960227272727273e-04*w10*Jinv12+2.4655032467532466e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-4.7483766233766234e-04*w0*Jinv01+4.6570616883116886e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-2.5202922077922079e-03*w42*Jinv02-3.1960227272727273e-04*w11*Jinv22-4.7483766233766234e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11-7.1225649350649346e-04*Jinv22*w17+1.3697240259740259e-04*w22*Jinv12+1.9176136363636362e-04*Jinv02*w3+1.4793019480519481e-03*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-4.8396915584415583e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-2.5202922077922079e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv11+3.8352272727272725e-04*Jinv12*w13+1.1505681818181819e-03*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+8.4922889610389610e-04*w37*Jinv12-2.7394480519480520e-03*Jinv22*w41-1.0135957792207792e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+1.1505681818181819e-03*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11-1.6436688311688311e-04*w5*Jinv21-2.0271915584415584e-03*Jinv02*w54+1.9176136363636362e-04*w4*Jinv12+1.3697240259740259e-04*w21*Jinv02-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+3.8352272727272725e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-3.1960227272727273e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+1.3971185064935066e-03*w47*Jinv21+1.3697240259740259e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+2.4655032467532466e-04*Jinv22*w8-1.2327516233766234e-03*Jinv22*w50+5.2049512987012989e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7+5.2049512987012989e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-2.3833198051948050e-03*Jinv21*w56-1.7349837662337662e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+4.6570616883116886e-04*w19*Jinv12+1.9176136363636362e-04*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14+1.1870941558441559e-04*w58*Jinv12+4.6570616883116886e-04*Jinv02*w18-1.6436688311688311e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33+5.2049512987012989e-04*Jinv21*w23-2.3833198051948050e-03*Jinv01*w54-2.3833198051948050e-03*w55*Jinv11-1.2327516233766234e-03*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9-1.7349837662337662e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+8.4922889610389610e-04*Jinv02*w36-2.5202922077922079e-03*Jinv12*w43+1.1505681818181819e-03*Jinv01*w30-1.0135957792207792e-03*Jinv02*w51-2.0271915584415584e-03*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-7.1225649350649346e-04*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-4.8396915584415583e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[30][55]=(0.0)*D;
  A[30][56]=(0.0)*D;
  A[30][57]=(-3.2873376623376621e-04*w40*Jinv12-6.7573051948051951e-04*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02-3.2873376623376621e-04*Jinv02*w39+4.9310064935064932e-04*Jinv22*w38-2.7394480519480521e-05*w25*Jinv12+8.3198051948051951e-05*w0*Jinv02-6.7573051948051951e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12-2.7394480519480521e-05*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35-2.7394480519480521e-05*Jinv22*w20+2.9220779220779221e-04*w42*Jinv02-1.1668019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17+1.1870941558441559e-04*w22*Jinv12+3.6525974025974026e-05*Jinv02*w3-2.5568181818181820e-04*w32*Jinv22+8.3198051948051951e-05*Jinv22*w2+4.9310064935064932e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59-3.2873376623376621e-04*Jinv12*w34+2.9220779220779221e-04*Jinv22*w44+3.6525974025974026e-05*Jinv12*w13+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv22*w41-6.7573051948051951e-04*w52*Jinv12-2.7394480519480521e-05*Jinv22*w26-6.7573051948051951e-04*Jinv02*w54+3.6525974025974026e-05*w4*Jinv12+1.1870941558441559e-04*w21*Jinv02+3.6525974025974026e-05*Jinv02*w12-1.1668019480519480e-04*Jinv02*w9+1.1870941558441559e-04*Jinv22*w23-2.5568181818181820e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+1.1870941558441559e-04*Jinv22*w8-1.0227272727272728e-03*Jinv22*w50+1.1870941558441559e-04*Jinv12*w7-2.5568181818181820e-04*Jinv02*w30-1.0227272727272728e-03*Jinv12*w49-1.1668019480519480e-04*Jinv02*w27+1.1870941558441559e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12+3.6525974025974026e-05*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18-3.2873376623376621e-04*Jinv02*w33-1.0227272727272728e-03*w48*Jinv02-1.1668019480519480e-04*Jinv22*w29+3.6525974025974026e-05*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36+2.9220779220779221e-04*Jinv12*w43-6.7573051948051951e-04*Jinv02*w51-6.7573051948051951e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12+8.3198051948051951e-05*Jinv12*w1)*D;
  A[30][58]=(0.0)*D;
  A[30][59]=(0.0)*D;
  A[31][0]=(0.0)*D;
  A[31][1]=(-1.1870941558441558e-03*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+2.6481331168831169e-04*Jinv22*w53+6.7573051948051951e-04*Jinv20*w8-2.0495129870129870e-04*w57*Jinv02-1.1870941558441558e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-1.1870941558441558e-03*w3*Jinv00-2.3741883116883116e-03*w32*Jinv20-1.1870941558441558e-03*Jinv01*w3+6.7573051948051951e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+2.6481331168831169e-04*Jinv00*w54-2.0495129870129870e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+2.6481331168831169e-04*Jinv12*w55-2.0495129870129870e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12+1.0227272727272728e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-2.0495129870129870e-04*w58*Jinv11-2.0495129870129870e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+1.0957792207792208e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-1.1870941558441558e-03*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+1.0227272727272728e-03*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+1.0957792207792208e-04*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+6.7573051948051951e-04*w22*Jinv12-2.3741883116883116e-03*w31*Jinv10-1.1870941558441558e-03*Jinv02*w3+6.7573051948051951e-04*w21*Jinv00-2.3741883116883116e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+1.0957792207792208e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-2.0495129870129870e-04*Jinv22*w59-6.5746753246753243e-04*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+2.6481331168831169e-04*Jinv21*w53+2.6481331168831169e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-1.1870941558441558e-03*w4*Jinv10-1.1870941558441558e-03*w4*Jinv11-6.5746753246753243e-04*w41*Jinv20-1.1870941558441558e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-2.3741883116883116e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+6.7573051948051951e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+2.6481331168831169e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+2.6481331168831169e-04*w52*Jinv12-6.5746753246753243e-04*Jinv21*w35-2.3741883116883116e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-1.1870941558441558e-03*Jinv20*w14-1.1870941558441558e-03*w5*Jinv21+2.6481331168831169e-04*Jinv02*w54-1.1870941558441558e-03*w4*Jinv12+6.7573051948051951e-04*w21*Jinv02+1.0227272727272728e-03*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-6.5746753246753243e-04*Jinv01*w33-6.5746753246753243e-04*w39*Jinv00-1.1870941558441558e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.0957792207792208e-04*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9+6.7573051948051951e-04*w6*Jinv01+6.7573051948051951e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38+6.7573051948051951e-04*Jinv22*w23+2.6481331168831169e-04*w52*Jinv11-2.3741883116883116e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.6481331168831169e-04*Jinv20*w53+6.7573051948051951e-04*Jinv22*w8+6.7573051948051951e-04*w22*Jinv10+1.0227272727272728e-03*Jinv22*w50+6.7573051948051951e-04*w22*Jinv11+6.7573051948051951e-04*Jinv12*w7+6.7573051948051951e-04*w21*Jinv01+2.6481331168831169e-04*Jinv01*w51-2.3741883116883116e-03*Jinv02*w30+1.0227272727272728e-03*Jinv12*w49+1.0957792207792208e-04*w36*Jinv01+2.6481331168831169e-04*w52*Jinv10+6.7573051948051951e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+2.6481331168831169e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+6.7573051948051951e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-1.1870941558441558e-03*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.0227272727272728e-03*w49*Jinv11-1.1870941558441558e-03*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-2.0495129870129870e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.1870941558441558e-03*Jinv01*w12+6.7573051948051951e-04*Jinv21*w8-2.0495129870129870e-04*w57*Jinv00-2.0495129870129870e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+6.7573051948051951e-04*Jinv21*w23+2.6481331168831169e-04*Jinv01*w54+2.6481331168831169e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00+1.0227272727272728e-03*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9-3.2873376623376621e-04*Jinv20*w17-2.3741883116883116e-03*w30*Jinv00-6.5746753246753243e-04*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-1.1870941558441558e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-1.1870941558441558e-03*w5*Jinv20-2.3741883116883116e-03*Jinv01*w30+2.6481331168831169e-04*Jinv02*w51+2.6481331168831169e-04*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11+2.6481331168831169e-04*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+1.0957792207792208e-04*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+1.0227272727272728e-03*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+1.0227272727272728e-03*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[31][2]=(0.0)*D;
  A[31][3]=(0.0)*D;
  A[31][4]=(2.4655032467532467e-03*Jinv00*w12+3.7439123376623376e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-1.1505681818181819e-03*Jinv20*w8-3.9265422077922079e-04*w57*Jinv02+2.4655032467532467e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10-1.5888798701298701e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12-1.3971185064935066e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-3.1777597402597403e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+1.1414366883116883e-03*w1*Jinv10+5.4788961038961036e-04*w16*Jinv10+8.2183441558441553e-05*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02+3.7439123376623376e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.7439123376623376e-04*w28*Jinv11-1.3971185064935066e-03*Jinv20*w50+3.7439123376623376e-04*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-1.5888798701298701e-03*Jinv22*w17+1.0957792207792207e-03*Jinv00*w33+1.0135957792207792e-03*w22*Jinv12+4.9310064935064934e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-3.9265422077922079e-04*Jinv22*w59-3.1777597402597403e-03*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53-3.1777597402597403e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11+2.6846590909090908e-03*w41*Jinv20-1.2327516233766234e-03*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20-1.1505681818181819e-03*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-3.1777597402597403e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+2.4655032467532467e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.0135957792207792e-03*w21*Jinv02-1.3971185064935066e-03*w49*Jinv10-3.7439123376623376e-04*Jinv21*w29-3.1777597402597403e-03*Jinv01*w33+2.6846590909090908e-03*w39*Jinv00-1.2327516233766234e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01-1.1505681818181819e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+1.0135957792207792e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+1.2327516233766234e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50+1.0135957792207792e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7+1.0135957792207792e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.2327516233766234e-03*w7*Jinv11+3.5612824675324673e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56-3.7439123376623376e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+8.2183441558441553e-05*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.2327516233766234e-03*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+3.5612824675324673e-04*w10*Jinv10-3.9265422077922079e-04*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59-3.1777597402597403e-03*Jinv02*w33+1.1414366883116883e-03*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+1.0135957792207792e-03*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+1.1414366883116883e-03*Jinv20*w2+5.4788961038961036e-04*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9+5.4788961038961036e-04*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00+2.6846590909090908e-03*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-3.7439123376623376e-04*Jinv22*w29-1.2327516233766234e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-3.7439123376623376e-04*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11-8.2183441558441553e-05*w55*Jinv10-3.5612824675324673e-04*w47*Jinv20-1.5888798701298701e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.7439123376623376e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[31][5]=(0.0)*D;
  A[31][6]=(0.0)*D;
  A[31][7]=(-2.0271915584415584e-03*Jinv00*w12-3.1960227272727273e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+7.3965097402597403e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-2.0271915584415584e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-4.0543831168831168e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+1.1231737012987013e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+9.5880681818181818e-04*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-5.5702110389610384e-04*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+4.6570616883116886e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-3.1960227272727273e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+9.5880681818181818e-04*Jinv20*w50-3.1960227272727273e-04*Jinv20*w29+4.6570616883116886e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17-8.7662337662337668e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12-4.0543831168831168e-03*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+1.1231737012987013e-03*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+5.4788961038961036e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-8.2183441558441553e-05*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-2.5750811688311690e-03*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-8.7662337662337668e-04*w34*Jinv10+7.1225649350649346e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.3965097402597403e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41-2.7394480519480521e-05*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+7.1225649350649346e-04*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-8.2183441558441553e-05*w46*Jinv11-2.0271915584415584e-03*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02+9.5880681818181818e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-2.5750811688311690e-03*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+7.3965097402597403e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21+5.4788961038961036e-04*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12-8.2183441558441553e-05*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+1.1231737012987013e-03*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+5.4788961038961036e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-2.7394480519480521e-05*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-8.2183441558441553e-05*w47*Jinv22-5.2049512987012989e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33-5.5702110389610384e-04*w0*Jinv00-8.7662337662337668e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-2.7394480519480521e-05*w25*Jinv10-5.5702110389610384e-04*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17-4.0543831168831168e-03*w30*Jinv00-2.5750811688311690e-03*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+7.1225649350649346e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+4.6570616883116886e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12-5.2049512987012989e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27-5.2049512987012989e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[31][8]=(0.0)*D;
  A[31][9]=(0.0)*D;
  A[31][10]=(7.4878246753246752e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-2.6481331168831169e-04*Jinv20*w8+7.4878246753246752e-04*w13*Jinv10+3.6525974025974026e-05*w3*Jinv00+1.4975649350649350e-03*w32*Jinv20-4.2918019480519480e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+3.4699675324675324e-04*w43*Jinv10-1.4204545454545455e-05*w58*Jinv10-5.8441558441558442e-04*w48*Jinv00+6.4935064935064935e-05*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10-2.1915584415584417e-04*w45*Jinv00+1.0247564935064935e-04*Jinv00*w27-5.8441558441558442e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+4.3831168831168834e-04*Jinv00*w33+1.4975649350649350e-03*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-5.4788961038961042e-05*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00+1.1870941558441559e-04*w51*Jinv00+3.6525974025974026e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-5.4788961038961042e-05*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10-2.6481331168831169e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56-2.7394480519480521e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20+7.4878246753246752e-04*Jinv20*w14-5.8441558441558442e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-2.6481331168831169e-04*w7*Jinv10-1.0957792207792208e-04*Jinv20*w38+1.1870941558441559e-04*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10+1.1870941558441559e-04*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10-2.7394480519480521e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+3.4699675324675324e-04*Jinv20*w44-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-2.7394480519480521e-05*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00+2.1915584415584417e-04*Jinv20*w17+1.4975649350649350e-03*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+3.6525974025974026e-05*w5*Jinv20-4.2918019480519480e-04*w55*Jinv10+3.4699675324675324e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20)*D;
  A[31][11]=(0.0)*D;
  A[31][12]=(0.0)*D;
  A[31][13]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-3.1777597402597403e-03*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53+1.0135957792207792e-03*Jinv20*w8-3.9265422077922079e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-3.1777597402597403e-03*Jinv02*w39-1.2327516233766234e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20+2.4655032467532467e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.9265422077922079e-04*Jinv21*w59+8.2183441558441553e-05*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+1.1414366883116883e-03*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10+3.7439123376623376e-04*w11*Jinv21+5.4788961038961036e-04*w16*Jinv11+5.4788961038961036e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10-1.5888798701298701e-03*w15*Jinv02-3.7439123376623376e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.1414366883116883e-03*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+1.6436688311688311e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-3.7439123376623376e-04*w11*Jinv22+1.1414366883116883e-03*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+5.4788961038961036e-04*w15*Jinv01+3.5612824675324673e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.5888798701298701e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10-1.2327516233766234e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22+1.0957792207792207e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-3.9265422077922079e-04*Jinv22*w59+2.6846590909090908e-03*w34*Jinv11-8.2183441558441553e-05*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44-1.2327516233766234e-03*w4*Jinv10+2.4655032467532467e-03*w4*Jinv11-3.1777597402597403e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21+1.0135957792207792e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-3.1777597402597403e-03*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-3.7439123376623376e-04*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+2.6846590909090908e-03*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11+8.2183441558441553e-05*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14+2.4655032467532467e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-1.2327516233766234e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10+3.5612824675324673e-04*Jinv21*w29+2.6846590909090908e-03*Jinv01*w33-3.1777597402597403e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-3.5612824675324673e-04*w37*Jinv11-3.7439123376623376e-04*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01+1.0135957792207792e-03*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53+1.0135957792207792e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50-1.1505681818181819e-03*w22*Jinv11+1.0135957792207792e-03*Jinv12*w7-1.1505681818181819e-03*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11-3.7439123376623376e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+8.2183441558441553e-05*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+1.0135957792207792e-03*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-1.2327516233766234e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-1.3971185064935066e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-3.7439123376623376e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44-3.9265422077922079e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.1505681818181819e-03*Jinv21*w23+8.2183441558441553e-05*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02+3.7439123376623376e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-3.1777597402597403e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14+1.0957792207792207e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43-1.2327516233766234e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.5888798701298701e-03*w16*Jinv12-1.3971185064935066e-03*Jinv21*w50+3.5612824675324673e-04*Jinv01*w27-1.3971185064935066e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1+3.7439123376623376e-04*w10*Jinv11)*D;
  A[31][14]=(0.0)*D;
  A[31][15]=(0.0)*D;
  A[31][16]=(3.1777597402597403e-03*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53-9.3141233766233771e-04*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+1.0409902597402598e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3-1.7532467532467534e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+5.4788961038961042e-05*Jinv12*w55+5.2962662337662338e-04*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12-1.7532467532467534e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01-5.4788961038961042e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+3.2873376623376621e-03*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.0409902597402598e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11-1.7532467532467534e-03*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10-1.0957792207792207e-03*Jinv02*w3-1.7532467532467534e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+3.6525974025974026e-05*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+3.2873376623376621e-03*Jinv22*w44+1.0409902597402598e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20-1.0957792207792207e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-9.3141233766233771e-04*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+3.2873376623376621e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+3.1777597402597403e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54-1.0957792207792207e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02-1.7532467532467534e-03*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00-1.0957792207792207e-03*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.7532467532467534e-03*w6*Jinv01-9.3141233766233771e-04*w7*Jinv10+3.8352272727272725e-04*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-1.7532467532467534e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-9.3141233766233771e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-9.3141233766233771e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10-1.7532467532467534e-03*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-3.2873376623376621e-04*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-1.0957792207792207e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22-1.7532467532467534e-03*w49*Jinv11+1.0409902597402598e-03*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18+1.0409902597402598e-03*Jinv01*w12-1.7532467532467534e-03*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20-9.3141233766233771e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10+3.6525974025974026e-05*Jinv22*w29-1.0957792207792207e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+3.2873376623376621e-03*Jinv12*w43+1.0409902597402598e-03*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-1.7532467532467534e-03*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-1.7532467532467534e-03*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[31][17]=(0.0)*D;
  A[31][18]=(0.0)*D;
  A[31][19]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-8.2183441558441553e-05*Jinv20*w8-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv21*w44+1.9176136363636362e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+5.2049512987012989e-04*Jinv20*w23+5.4788961038961036e-04*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-8.7662337662337668e-04*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-5.4788961038961042e-05*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-1.3697240259740259e-04*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+5.2049512987012989e-04*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38+2.7394480519480518e-04*Jinv01*w24-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00+1.9176136363636362e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+5.4788961038961036e-04*Jinv20*w56+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20-7.6704545454545450e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+8.2183441558441553e-05*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-3.5612824675324673e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+5.2049512987012989e-04*w52*Jinv11+2.4655032467532466e-04*Jinv20*w53+5.2049512987012989e-04*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+6.3920454545454550e-05*w10*Jinv10-8.7662337662337668e-04*Jinv20*w44-3.5612824675324673e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-7.1225649350649346e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+1.9176136363636362e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+2.7394480519480518e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10-8.7662337662337668e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[31][20]=(0.0)*D;
  A[31][21]=(0.0)*D;
  A[31][22]=(1.6436688311688311e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+7.1225649350649346e-04*w32*Jinv20-2.0271915584415584e-03*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-8.2183441558441553e-05*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54-1.7349837662337662e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-5.5702110389610384e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-3.1960227272727273e-04*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+5.4788961038961036e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12-5.2049512987012989e-04*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-5.5702110389610384e-04*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-8.7662337662337668e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22-5.5702110389610384e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11-5.2049512987012989e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+7.1225649350649346e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22-8.7662337662337668e-04*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00+4.6570616883116886e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-2.5750811688311690e-03*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10-2.0271915584415584e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-4.0543831168831168e-03*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-8.2183441558441553e-05*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-2.5750811688311690e-03*Jinv21*w35-4.0543831168831168e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+5.4788961038961036e-04*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-2.0271915584415584e-03*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-5.2049512987012989e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29-2.5750811688311690e-03*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-2.7394480519480521e-05*Jinv01*w18+4.6570616883116886e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9+1.1231737012987013e-03*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+5.4788961038961036e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50+7.3965097402597403e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7+7.3965097402597403e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+4.6570616883116886e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+1.1231737012987013e-03*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+9.5880681818181818e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+1.1231737012987013e-03*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.3965097402597403e-04*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02-3.1960227272727273e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17+7.1225649350649346e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-8.7662337662337668e-04*w40*Jinv11-8.2183441558441553e-05*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20-4.0543831168831168e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12+9.5880681818181818e-04*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+9.5880681818181818e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-3.1960227272727273e-04*w10*Jinv11)*D;
  A[31][23]=(0.0)*D;
  A[31][24]=(0.0)*D;
  A[31][25]=(-1.6436688311688311e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-8.7662337662337668e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23+5.2049512987012989e-04*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10-8.7662337662337668e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-8.7662337662337668e-04*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1+1.9176136363636362e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+2.7394480519480518e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00-3.5612824675324673e-04*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00+5.2049512987012989e-04*Jinv20*w56-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-1.0957792207792208e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+2.4655032467532466e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+5.2049512987012989e-04*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38+5.4788961038961036e-04*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-8.2183441558441553e-05*w22*Jinv11-8.2183441558441553e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+5.4788961038961036e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+5.2049512987012989e-04*w7*Jinv11-1.0957792207792208e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11+1.9176136363636362e-04*Jinv21*w14-1.0957792207792208e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+1.9176136363636362e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.3697240259740259e-04*Jinv01*w9-3.8352272727272725e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11-3.5612824675324673e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-3.5612824675324673e-04*w25*Jinv11+5.2049512987012989e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-1.3697240259740259e-04*w10*Jinv11)*D;
  A[31][26]=(0.0)*D;
  A[31][27]=(0.0)*D;
  A[31][28]=(3.4699675324675324e-04*Jinv21*w44+7.4878246753246752e-04*Jinv01*w3-1.4204545454545455e-05*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2+3.4699675324675324e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+3.4699675324675324e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41+6.4935064935064935e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+4.3831168831168834e-04*Jinv01*w39-2.1915584415584417e-04*Jinv21*w38-5.4788961038961042e-05*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53+7.4878246753246752e-04*w4*Jinv11+1.4975649350649350e-03*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35+1.4975649350649350e-03*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11+7.4878246753246752e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-2.7394480519480521e-05*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-1.0957792207792208e-04*w47*Jinv21-4.2918019480519480e-04*w52*Jinv11-2.6481331168831169e-04*w22*Jinv11-2.6481331168831169e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51-2.1915584415584417e-04*w36*Jinv01-4.2918019480519480e-04*w7*Jinv11+1.1870941558441559e-04*Jinv21*w56-5.8441558441558442e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+3.6525974025974026e-05*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8-2.6481331168831169e-04*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+1.0247564935064935e-04*Jinv01*w9+4.3831168831168834e-04*w40*Jinv11+1.4975649350649350e-03*Jinv01*w30-5.4788961038961042e-05*w25*Jinv11-5.8441558441558442e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-5.8441558441558442e-04*w48*Jinv01+1.0247564935064935e-04*w10*Jinv11)*D;
  A[31][29]=(0.0)*D;
  A[31][30]=(0.0)*D;
  A[31][31]=(-2.4655032467532467e-03*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-7.3965097402597403e-04*Jinv22*w53+2.0271915584415584e-03*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-7.3965097402597406e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3+2.0271915584415584e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-5.7528409090909093e-04*Jinv00*w54-6.5746753246753243e-04*Jinv21*w59+1.1870941558441558e-03*w0*Jinv02+9.1314935064935068e-05*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+1.0683847402597404e-03*Jinv01*w45-7.3965097402597403e-04*Jinv12*w55-6.5746753246753243e-04*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02+1.3149350649350649e-03*w48*Jinv00-4.9310064935064932e-04*Jinv22*w35-6.5746753246753243e-04*w58*Jinv11-6.5746753246753243e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+9.1314935064935068e-05*w0*Jinv01+9.1314935064935068e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-1.6436688311688311e-03*w43*Jinv11+1.0683847402597404e-03*w45*Jinv00-4.7666396103896100e-03*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27+9.1314935064935068e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+1.0683847402597404e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-4.7666396103896100e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-7.3965097402597406e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3+2.0271915584415584e-03*w21*Jinv00-4.7666396103896100e-03*Jinv01*w39+1.1870941558441558e-03*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+1.0683847402597404e-03*w36*Jinv00+1.0683847402597404e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-4.7666396103896100e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.7528409090909093e-04*Jinv21*w53-5.7528409090909093e-04*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11-4.7666396103896100e-03*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-4.7666396103896100e-03*w34*Jinv10-7.3965097402597406e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+2.0271915584415584e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-5.7528409090909093e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-7.3965097402597403e-04*w52*Jinv12-4.7666396103896100e-03*Jinv21*w35-7.3965097402597406e-03*w31*Jinv11+1.0683847402597404e-03*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21-7.3965097402597403e-04*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.3149350649350649e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.7666396103896100e-03*Jinv01*w33-4.7666396103896100e-03*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.0683847402597404e-03*w37*Jinv11+2.0271915584415584e-03*w6*Jinv01+2.0271915584415584e-03*w7*Jinv10+1.0683847402597404e-03*w47*Jinv21+1.0683847402597404e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-5.7528409090909093e-04*w52*Jinv11+3.5612824675324673e-04*w46*Jinv12-5.7528409090909093e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+2.0271915584415584e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50+2.0271915584415584e-03*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+2.0271915584415584e-03*w21*Jinv01-5.7528409090909093e-04*Jinv01*w51-1.3149350649350649e-03*Jinv12*w49+1.0683847402597404e-03*w36*Jinv01-5.7528409090909093e-04*w52*Jinv10+2.0271915584415584e-03*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+1.0683847402597404e-03*w37*Jinv10-5.7528409090909093e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+1.2327516233766234e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.3149350649350649e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12-2.4655032467532467e-03*Jinv01*w12+2.0271915584415584e-03*Jinv21*w8-6.5746753246753243e-04*w57*Jinv00-6.5746753246753243e-04*Jinv20*w59-4.9310064935064932e-04*Jinv02*w33+9.1314935064935068e-05*w0*Jinv00-4.7666396103896100e-03*w35*Jinv20+2.0271915584415584e-03*Jinv21*w23-5.7528409090909093e-04*Jinv01*w54-5.7528409090909093e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+9.1314935064935068e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02-7.4878246753246752e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-7.3965097402597406e-03*w30*Jinv00-4.7666396103896100e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+1.2327516233766234e-03*Jinv22*w14-4.7666396103896100e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20-7.3965097402597406e-03*Jinv01*w30-7.3965097402597403e-04*Jinv02*w51-7.3965097402597403e-04*Jinv22*w56+3.5612824675324673e-04*w25*Jinv11-5.7528409090909093e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+1.0683847402597404e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12+1.3149350649350649e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01+1.1870941558441558e-03*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[31][32]=(0.0)*D;
  A[31][33]=(0.0)*D;
  A[31][34]=(3.1777597402597403e-03*Jinv00*w12+9.6793831168831166e-04*w28*Jinv10+2.9586038961038961e-03*w40*Jinv12+1.8628246753246754e-03*Jinv22*w53-1.8628246753246754e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+2.9586038961038961e-03*Jinv02*w39+4.9310064935064934e-03*Jinv21*w44+2.0819805194805196e-03*w3*Jinv00+9.5332792207792200e-03*w32*Jinv20-2.1915584415584414e-03*Jinv01*w3-2.5750811688311690e-03*Jinv20*w23-6.0267857142857139e-04*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12+4.3831168831168834e-04*w0*Jinv02+9.8620129870129864e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+4.9310064935064934e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-1.1505681818181819e-03*Jinv01*w45+9.3141233766233771e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+5.6615259740259744e-04*w10*Jinv12-2.7942370129870131e-03*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+9.8620129870129864e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-3.2873376623376621e-04*Jinv22*w20+4.9310064935064934e-03*w43*Jinv11-1.3149350649350649e-03*w45*Jinv00+1.9724025974025973e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02+9.6793831168831166e-04*Jinv00*w27+5.6615259740259744e-04*w11*Jinv22+9.8620129870129864e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.2004870129870131e-04*w28*Jinv11-2.7942370129870131e-03*Jinv20*w50+9.6793831168831166e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17+6.9034090909090911e-03*Jinv00*w33-1.2601461038961039e-03*w22*Jinv12+9.5332792207792200e-03*w31*Jinv10+2.0819805194805196e-03*Jinv02*w3-2.5750811688311690e-03*w21*Jinv00+9.8620129870129864e-04*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+4.3831168831168834e-04*Jinv22*w2+1.6436688311688311e-04*Jinv21*w38-1.2053571428571428e-03*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+2.0819805194805196e-03*w4*Jinv10-2.1915584415584414e-03*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20+1.0409902597402598e-03*Jinv12*w13+6.9034090909090911e-03*w34*Jinv10-3.2873376623376621e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20-1.8628246753246754e-03*w6*Jinv00-6.0267857142857139e-04*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+2.9586038961038961e-03*Jinv22*w41-8.2183441558441553e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20+1.8628246753246754e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35-3.2873376623376621e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26-1.1505681818181819e-03*w46*Jinv11+3.1777597402597403e-03*Jinv20*w14-2.1915584415584414e-03*w5*Jinv21+9.3141233766233771e-04*Jinv02*w54+2.0819805194805196e-03*w4*Jinv12-1.2601461038961039e-03*w21*Jinv02-2.7942370129870131e-03*w49*Jinv10+4.2004870129870131e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00+1.0409902597402598e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01-1.8628246753246754e-03*w7*Jinv10-1.1505681818181819e-03*w47*Jinv21-1.2601461038961039e-03*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11+9.8620129870129864e-04*w31*Jinv12-1.2053571428571428e-03*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8-2.5750811688311690e-03*w22*Jinv10+5.4788961038961042e-05*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51+9.8620129870129864e-04*Jinv02*w30+1.6436688311688311e-04*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10+7.6704545454545450e-04*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00+1.3149350649350649e-03*Jinv21*w56-8.2183441558441553e-04*w24*Jinv00+6.9399350649350649e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.2873376623376621e-04*w19*Jinv12+2.0819805194805196e-03*Jinv22*w5-1.2053571428571428e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-3.2873376623376621e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+5.9172077922077923e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00+6.9034090909090911e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11-8.2183441558441553e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+7.3051948051948052e-05*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+9.5332792207792200e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10+6.9399350649350649e-04*Jinv22*w29+1.0409902597402598e-03*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-6.0267857142857139e-04*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+2.0819805194805196e-03*w5*Jinv20-3.2873376623376621e-03*Jinv01*w30+1.8628246753246754e-03*Jinv02*w51+9.3141233766233771e-04*Jinv22*w56+6.9399350649350649e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11-8.2183441558441553e-04*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.2004870129870131e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01+4.3831168831168834e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[31][35]=(0.0)*D;
  A[31][36]=(0.0)*D;
  A[31][37]=(-7.1225649350649346e-04*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-1.3697240259740259e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23-7.1225649350649346e-04*Jinv22*w38+1.3971185064935066e-03*Jinv00*w54+2.7394480519480518e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+1.4793019480519481e-03*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35+1.8262987012987013e-05*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+2.4655032467532466e-04*w45*Jinv00-5.4788961038961042e-05*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.4793019480519481e-03*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10-3.8352272727272725e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-1.0683847402597404e-03*w36*Jinv00+8.2183441558441553e-05*Jinv02*w45-1.3697240259740259e-04*Jinv22*w59+8.2183441558441553e-05*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-1.6436688311688311e-04*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-7.1225649350649346e-04*w37*Jinv12+1.3971185064935066e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+4.6570616883116886e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+5.2049512987012989e-04*w52*Jinv12+2.7394480519480518e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02+1.4793019480519481e-03*w49*Jinv10-1.6436688311688311e-04*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10-1.0683847402597404e-03*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10+5.2049512987012989e-04*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+8.2183441558441553e-05*w52*Jinv10+1.2784090909090910e-04*w9*Jinv00-1.0683847402597404e-03*w37*Jinv10+4.6570616883116886e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22+1.2784090909090910e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-1.3697240259740259e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33+1.8262987012987013e-05*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+4.6570616883116886e-04*w25*Jinv10+1.8262987012987013e-05*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+5.2049512987012989e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-7.1225649350649346e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20+5.2049512987012989e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28+1.3971185064935066e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20-3.8352272727272725e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[31][38]=(0.0)*D;
  A[31][39]=(0.0)*D;
  A[31][40]=(-2.1915584415584414e-03*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12+9.3141233766233771e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-2.1915584415584414e-03*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-3.2873376623376621e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23-1.2053571428571428e-03*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59-3.2873376623376621e-04*w25*Jinv12+4.3831168831168834e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2+4.9310064935064934e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+9.6793831168831166e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+1.8628246753246754e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+6.9399350649350649e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-3.2873376623376621e-04*Jinv02*w24+2.9586038961038961e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-1.0957792207792208e-04*w0*Jinv01+9.8620129870129864e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+6.9034090909090911e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+6.9399350649350649e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.0819805194805196e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17+1.9724025974025973e-03*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12-3.2873376623376621e-03*w31*Jinv10+1.0409902597402598e-03*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+9.8620129870129864e-04*w32*Jinv22+6.9034090909090911e-03*Jinv01*w39+4.3831168831168834e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-1.1505681818181819e-03*w36*Jinv00-1.3149350649350649e-03*Jinv21*w38-6.0267857142857139e-04*Jinv02*w45+2.7394480519480518e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11-8.2183441558441553e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53+1.3149350649350649e-03*w51*Jinv00+2.9586038961038961e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+2.0819805194805196e-03*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10+9.5332792207792200e-03*w32*Jinv21-8.2183441558441553e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-1.2053571428571428e-03*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+4.2004870129870131e-04*w11*Jinv20+9.3141233766233771e-04*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35+9.5332792207792200e-03*w31*Jinv11-3.2873376623376621e-04*Jinv22*w26-2.1915584415584414e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+1.8628246753246754e-03*Jinv02*w54+1.0409902597402598e-03*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+2.0819805194805196e-03*Jinv02*w12-8.2183441558441553e-04*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+6.9399350649350649e-04*Jinv02*w9-2.5750811688311690e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-1.1505681818181819e-03*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11+9.8620129870129864e-04*w31*Jinv12-6.0267857142857139e-04*w46*Jinv12+1.3149350649350649e-03*Jinv20*w53-1.2601461038961039e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10-1.8628246753246754e-03*w22*Jinv11-1.2601461038961039e-03*Jinv12*w7-1.8628246753246754e-03*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51+9.8620129870129864e-04*Jinv02*w30-1.3149350649350649e-03*w36*Jinv01+1.3149350649350649e-03*w52*Jinv10-2.5750811688311690e-03*w7*Jinv11+4.2004870129870131e-04*w9*Jinv00-1.1505681818181819e-03*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+5.6615259740259744e-04*Jinv02*w27-1.2601461038961039e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12+1.0409902597402598e-03*Jinv22*w5-6.0267857142857139e-04*w47*Jinv22-2.7942370129870131e-03*w49*Jinv11+2.0819805194805196e-03*Jinv21*w14+4.2004870129870131e-04*w10*Jinv10+4.9310064935064934e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18+2.0819805194805196e-03*Jinv01*w12-2.5750811688311690e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+2.9586038961038961e-03*Jinv02*w33+9.8620129870129864e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-1.8628246753246754e-03*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+9.8620129870129864e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+9.6793831168831166e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-3.2873376623376621e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20+5.6615259740259744e-04*Jinv22*w29+2.0819805194805196e-03*Jinv22*w14+6.9034090909090911e-03*w40*Jinv11-1.2053571428571428e-03*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+9.5332792207792200e-03*Jinv01*w30+9.3141233766233771e-04*Jinv02*w51+1.8628246753246754e-03*Jinv22*w56+5.6615259740259744e-04*Jinv12*w28+2.6298701298701297e-03*w55*Jinv10+4.9310064935064934e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-2.7942370129870131e-03*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27-2.7942370129870131e-03*w48*Jinv01+4.3831168831168834e-04*Jinv12*w1+9.6793831168831166e-04*w10*Jinv11)*D;
  A[31][41]=(0.0)*D;
  A[31][42]=(0.0)*D;
  A[31][43]=(2.1915584415584414e-03*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-3.6525974025974026e-05*w57*Jinv02+2.1915584415584414e-03*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+3.2873376623376621e-03*w32*Jinv20+2.1915584415584414e-03*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-9.8620129870129864e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-5.4788961038961042e-05*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-9.8620129870129864e-04*w0*Jinv01-9.8620129870129864e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41-3.2873376623376621e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22-9.8620129870129864e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.2004870129870131e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+3.2873376623376621e-03*w31*Jinv10+1.0957792207792207e-03*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24-3.6525974025974026e-05*Jinv22*w59-3.9448051948051946e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.3149350649350649e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+2.1915584415584414e-03*w4*Jinv11-3.9448051948051946e-03*w41*Jinv20+1.0957792207792207e-03*Jinv12*w13+7.6704545454545450e-04*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+3.2873376623376621e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-4.2004870129870131e-04*w11*Jinv20-5.4788961038961042e-05*w52*Jinv12-3.9448051948051946e-03*Jinv21*w35+3.2873376623376621e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11+2.1915584415584414e-03*Jinv20*w14+2.1915584415584414e-03*w5*Jinv21-5.4788961038961042e-05*Jinv02*w54+1.0957792207792207e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-4.2004870129870131e-04*Jinv21*w29-3.9448051948051946e-03*Jinv01*w33-3.9448051948051946e-03*w39*Jinv00+1.0957792207792207e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.1505681818181819e-03*w47*Jinv21+1.1505681818181819e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-1.3149350649350649e-03*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11-4.2004870129870131e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10-1.3149350649350649e-03*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+1.0957792207792207e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-4.2004870129870131e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-3.6525974025974026e-05*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-9.8620129870129864e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10-9.8620129870129864e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+3.2873376623376621e-03*w30*Jinv00-3.9448051948051946e-03*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29+1.0957792207792207e-03*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+3.2873376623376621e-03*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.2004870129870131e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[31][44]=(0.0)*D;
  A[31][45]=(0.0)*D;
  A[31][46]=(-7.6704545454545450e-04*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-1.3697240259740259e-04*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+8.2183441558441553e-05*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+1.8262987012987013e-05*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-1.0683847402597404e-03*Jinv01*w45+5.2049512987012989e-04*Jinv12*w55-3.8352272727272725e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-3.5612824675324673e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+1.8262987012987013e-05*w0*Jinv01+2.7394480519480518e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02-1.0957792207792208e-04*w11*Jinv22+1.8262987012987013e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.8352272727272725e-04*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+2.4655032467532466e-04*Jinv21*w38-7.1225649350649346e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.3697240259740259e-04*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11+4.6570616883116886e-04*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21+4.6570616883116886e-04*Jinv21*w20+8.2183441558441553e-05*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26-1.0683847402597404e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+5.2049512987012989e-04*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12+4.6570616883116886e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01-1.0683847402597404e-03*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+1.3971185064935066e-03*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8+5.2049512987012989e-04*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11+8.2183441558441553e-05*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.4793019480519481e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-1.3697240259740259e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+5.2049512987012989e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-3.8352272727272725e-04*w16*Jinv12+1.4793019480519481e-03*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+1.4793019480519481e-03*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[31][47]=(0.0)*D;
  A[31][48]=(0.0)*D;
  A[31][49]=(1.6436688311688311e-04*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10+1.4793019480519481e-03*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+4.0178571428571428e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+1.4793019480519481e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-5.2049512987012989e-04*Jinv20*w23-9.5880681818181818e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.7394480519480521e-05*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02+4.7483766233766234e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+2.6298701298701297e-03*w48*Jinv00-2.7394480519480521e-05*Jinv02*w24+1.4793019480519481e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+4.7483766233766234e-04*w0*Jinv01+4.7483766233766234e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480521e-05*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27+2.7394480519480518e-04*w11*Jinv22+4.7483766233766234e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.6298701298701297e-03*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12-1.1505681818181819e-03*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3-5.2049512987012989e-04*w21*Jinv00+2.6298701298701297e-03*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38-9.5880681818181818e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+4.0178571428571428e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00+1.4793019480519481e-03*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-1.1505681818181819e-03*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-9.5880681818181818e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56+1.4793019480519481e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11-2.7394480519480521e-05*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02+2.6298701298701297e-03*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9-5.2049512987012989e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+2.6298701298701297e-03*w31*Jinv12-9.5880681818181818e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-5.2049512987012989e-04*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+2.6298701298701297e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-5.2049512987012989e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5-9.5880681818181818e-04*w47*Jinv22+2.6298701298701297e-03*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+4.0178571428571428e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-5.2049512987012989e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+1.4793019480519481e-03*Jinv02*w33+4.7483766233766234e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10+4.7483766233766234e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-9.5880681818181818e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30+1.1505681818181819e-03*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.6298701298701297e-03*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.6298701298701297e-03*w48*Jinv01-6.2094155844155847e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[31][50]=(0.0)*D;
  A[31][51]=(0.0)*D;
  A[31][52]=(-1.6436688311688311e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-2.0271915584415584e-03*Jinv22*w53+5.2049512987012989e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39-1.6436688311688311e-04*w3*Jinv00+1.1505681818181819e-03*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+4.6570616883116886e-04*w25*Jinv12-4.8396915584415583e-04*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10-1.0135957792207792e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-7.1225649350649346e-04*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00+4.6570616883116886e-04*Jinv02*w24-2.7394480519480520e-03*Jinv22*w35-4.7483766233766234e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-2.5202922077922079e-03*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-2.6298701298701297e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-7.1225649350649346e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.1505681818181819e-03*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-4.8396915584415583e-04*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+8.4922889610389610e-04*Jinv02*w45+1.1870941558441559e-04*Jinv22*w59-2.3833198051948050e-03*w51*Jinv00-2.7394480519480520e-03*Jinv12*w34-2.5202922077922079e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv10-2.1367694805194807e-03*w41*Jinv20+1.9176136363636362e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10+5.2049512987012989e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-2.0271915584415584e-03*w52*Jinv12+4.6570616883116886e-04*Jinv22*w26-1.6436688311688311e-04*Jinv20*w14-1.0135957792207792e-03*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10-2.1367694805194807e-03*w39*Jinv00+1.9176136363636362e-04*Jinv02*w12-1.7349837662337662e-04*Jinv02*w9+5.2049512987012989e-04*w7*Jinv10+1.3971185064935066e-03*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23+1.4793019480519481e-03*w31*Jinv12+8.4922889610389610e-04*w46*Jinv12-2.3833198051948050e-03*Jinv20*w53+1.3697240259740259e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+1.3697240259740259e-04*Jinv12*w7+1.4793019480519481e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49-2.3833198051948050e-03*w52*Jinv10-4.2918019480519480e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-3.1960227272727273e-04*Jinv02*w27+1.3697240259740259e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5+8.4922889610389610e-04*w47*Jinv22-4.2918019480519480e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-2.7394480519480520e-03*Jinv02*w33-4.7483766233766234e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10-4.7483766233766234e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.1505681818181819e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-3.1960227272727273e-04*Jinv22*w29+1.9176136363636362e-04*Jinv22*w14+4.3831168831168834e-04*Jinv02*w36-2.5202922077922079e-03*Jinv12*w43-1.6436688311688311e-04*w5*Jinv20-2.0271915584415584e-03*Jinv02*w51-1.0135957792207792e-03*Jinv22*w56-3.1960227272727273e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-7.1225649350649346e-04*w16*Jinv12-4.8396915584415583e-04*Jinv12*w1)*D;
  A[31][53]=(0.0)*D;
  A[31][54]=(0.0)*D;
  A[31][55]=(-2.7394480519480520e-03*w40*Jinv12-1.0135957792207792e-03*Jinv22*w53+1.1870941558441559e-04*w57*Jinv02-2.7394480519480520e-03*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-1.6436688311688311e-04*Jinv01*w3+8.4922889610389610e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12-4.8396915584415583e-04*w0*Jinv02-4.7483766233766234e-04*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-2.0271915584415584e-03*Jinv12*w55-7.1225649350649346e-04*w15*Jinv02-3.1960227272727273e-04*w10*Jinv12+2.4655032467532466e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-4.7483766233766234e-04*w0*Jinv01+4.6570616883116886e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-2.5202922077922079e-03*w42*Jinv02-3.1960227272727273e-04*w11*Jinv22-4.7483766233766234e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11-7.1225649350649346e-04*Jinv22*w17+1.3697240259740259e-04*w22*Jinv12+1.9176136363636362e-04*Jinv02*w3+1.4793019480519481e-03*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-4.8396915584415583e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-2.5202922077922079e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv11+3.8352272727272725e-04*Jinv12*w13+1.1505681818181819e-03*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+8.4922889610389610e-04*w37*Jinv12-2.7394480519480520e-03*Jinv22*w41-1.0135957792207792e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+1.1505681818181819e-03*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11-1.6436688311688311e-04*w5*Jinv21-2.0271915584415584e-03*Jinv02*w54+1.9176136363636362e-04*w4*Jinv12+1.3697240259740259e-04*w21*Jinv02-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+3.8352272727272725e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-3.1960227272727273e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+1.3971185064935066e-03*w47*Jinv21+1.3697240259740259e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+2.4655032467532466e-04*Jinv22*w8-1.2327516233766234e-03*Jinv22*w50+5.2049512987012989e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7+5.2049512987012989e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-2.3833198051948050e-03*Jinv21*w56-1.7349837662337662e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+4.6570616883116886e-04*w19*Jinv12+1.9176136363636362e-04*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14+1.1870941558441559e-04*w58*Jinv12+4.6570616883116886e-04*Jinv02*w18-1.6436688311688311e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33+5.2049512987012989e-04*Jinv21*w23-2.3833198051948050e-03*Jinv01*w54-2.3833198051948050e-03*w55*Jinv11-1.2327516233766234e-03*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9-1.7349837662337662e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+8.4922889610389610e-04*Jinv02*w36-2.5202922077922079e-03*Jinv12*w43+1.1505681818181819e-03*Jinv01*w30-1.0135957792207792e-03*Jinv02*w51-2.0271915584415584e-03*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-7.1225649350649346e-04*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-4.8396915584415583e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[31][56]=(0.0)*D;
  A[31][57]=(0.0)*D;
  A[31][58]=(-3.2873376623376621e-04*w40*Jinv12-6.7573051948051951e-04*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02-3.2873376623376621e-04*Jinv02*w39+4.9310064935064932e-04*Jinv22*w38-2.7394480519480521e-05*w25*Jinv12+8.3198051948051951e-05*w0*Jinv02-6.7573051948051951e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12-2.7394480519480521e-05*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35-2.7394480519480521e-05*Jinv22*w20+2.9220779220779221e-04*w42*Jinv02-1.1668019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17+1.1870941558441559e-04*w22*Jinv12+3.6525974025974026e-05*Jinv02*w3-2.5568181818181820e-04*w32*Jinv22+8.3198051948051951e-05*Jinv22*w2+4.9310064935064932e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59-3.2873376623376621e-04*Jinv12*w34+2.9220779220779221e-04*Jinv22*w44+3.6525974025974026e-05*Jinv12*w13+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv22*w41-6.7573051948051951e-04*w52*Jinv12-2.7394480519480521e-05*Jinv22*w26-6.7573051948051951e-04*Jinv02*w54+3.6525974025974026e-05*w4*Jinv12+1.1870941558441559e-04*w21*Jinv02+3.6525974025974026e-05*Jinv02*w12-1.1668019480519480e-04*Jinv02*w9+1.1870941558441559e-04*Jinv22*w23-2.5568181818181820e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+1.1870941558441559e-04*Jinv22*w8-1.0227272727272728e-03*Jinv22*w50+1.1870941558441559e-04*Jinv12*w7-2.5568181818181820e-04*Jinv02*w30-1.0227272727272728e-03*Jinv12*w49-1.1668019480519480e-04*Jinv02*w27+1.1870941558441559e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12+3.6525974025974026e-05*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18-3.2873376623376621e-04*Jinv02*w33-1.0227272727272728e-03*w48*Jinv02-1.1668019480519480e-04*Jinv22*w29+3.6525974025974026e-05*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36+2.9220779220779221e-04*Jinv12*w43-6.7573051948051951e-04*Jinv02*w51-6.7573051948051951e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12+8.3198051948051951e-05*Jinv12*w1)*D;
  A[31][59]=(0.0)*D;
  A[32][0]=(0.0)*D;
  A[32][1]=(0.0)*D;
  A[32][2]=(-1.1870941558441558e-03*Jinv00*w12-1.5726461038961039e-04*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+2.6481331168831169e-04*Jinv22*w53+6.7573051948051951e-04*Jinv20*w8-2.0495129870129870e-04*w57*Jinv02-1.1870941558441558e-03*w13*Jinv10-6.5746753246753243e-04*Jinv02*w39+3.1047077922077924e-04*Jinv21*w44-1.1870941558441558e-03*w3*Jinv00-2.3741883116883116e-03*w32*Jinv20-1.1870941558441558e-03*Jinv01*w3+6.7573051948051951e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+2.6481331168831169e-04*Jinv00*w54-2.0495129870129870e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12-6.4935064935064935e-04*w0*Jinv02-6.4935064935064935e-04*Jinv21*w2+3.1047077922077924e-04*w43*Jinv10+3.1047077922077924e-04*w42*Jinv01-1.5726461038961039e-04*w11*Jinv21-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+2.6481331168831169e-04*Jinv12*w55-2.0495129870129870e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12+1.0227272727272728e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-2.0495129870129870e-04*w58*Jinv11-2.0495129870129870e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-6.4935064935064935e-04*w0*Jinv01-6.4935064935064935e-04*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+3.1047077922077924e-04*w43*Jinv11+1.0957792207792208e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+3.1047077922077924e-04*w42*Jinv02-1.5726461038961039e-04*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-04*Jinv11*w1-1.1870941558441558e-03*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11+1.0227272727272728e-03*Jinv20*w50-1.5726461038961039e-04*Jinv20*w29+1.0957792207792208e-04*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+6.7573051948051951e-04*w22*Jinv12-2.3741883116883116e-03*w31*Jinv10-1.1870941558441558e-03*Jinv02*w3+6.7573051948051951e-04*w21*Jinv00-2.3741883116883116e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-6.4935064935064935e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+1.0957792207792208e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-2.0495129870129870e-04*Jinv22*w59-6.5746753246753243e-04*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+2.6481331168831169e-04*Jinv21*w53+2.6481331168831169e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+3.1047077922077924e-04*Jinv22*w44-1.1870941558441558e-03*w4*Jinv10-1.1870941558441558e-03*w4*Jinv11-6.5746753246753243e-04*w41*Jinv20-1.1870941558441558e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-2.3741883116883116e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+6.7573051948051951e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+2.6481331168831169e-04*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+2.6481331168831169e-04*w52*Jinv12-6.5746753246753243e-04*Jinv21*w35-2.3741883116883116e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-1.1870941558441558e-03*Jinv20*w14-1.1870941558441558e-03*w5*Jinv21+2.6481331168831169e-04*Jinv02*w54-1.1870941558441558e-03*w4*Jinv12+6.7573051948051951e-04*w21*Jinv02+1.0227272727272728e-03*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-6.5746753246753243e-04*Jinv01*w33-6.5746753246753243e-04*w39*Jinv00-1.1870941558441558e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.0957792207792208e-04*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9+6.7573051948051951e-04*w6*Jinv01+6.7573051948051951e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38+6.7573051948051951e-04*Jinv22*w23+2.6481331168831169e-04*w52*Jinv11-2.3741883116883116e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.6481331168831169e-04*Jinv20*w53+6.7573051948051951e-04*Jinv22*w8+6.7573051948051951e-04*w22*Jinv10+1.0227272727272728e-03*Jinv22*w50+6.7573051948051951e-04*w22*Jinv11+6.7573051948051951e-04*Jinv12*w7+6.7573051948051951e-04*w21*Jinv01+2.6481331168831169e-04*Jinv01*w51-2.3741883116883116e-03*Jinv02*w30+1.0227272727272728e-03*Jinv12*w49+1.0957792207792208e-04*w36*Jinv01+2.6481331168831169e-04*w52*Jinv10+6.7573051948051951e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+2.6481331168831169e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+6.7573051948051951e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-1.1870941558441558e-03*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+1.0227272727272728e-03*w49*Jinv11-1.1870941558441558e-03*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.1047077922077924e-04*Jinv20*w44-2.0495129870129870e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.1870941558441558e-03*Jinv01*w12+6.7573051948051951e-04*Jinv21*w8-2.0495129870129870e-04*w57*Jinv00-2.0495129870129870e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-6.4935064935064935e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+6.7573051948051951e-04*Jinv21*w23+2.6481331168831169e-04*Jinv01*w54+2.6481331168831169e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10-6.4935064935064935e-04*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00+1.0227272727272728e-03*w48*Jinv02-1.5726461038961039e-04*Jinv01*w9-3.2873376623376621e-04*Jinv20*w17-2.3741883116883116e-03*w30*Jinv00-6.5746753246753243e-04*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-1.1870941558441558e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+3.1047077922077924e-04*Jinv12*w43-1.1870941558441558e-03*w5*Jinv20-2.3741883116883116e-03*Jinv01*w30+2.6481331168831169e-04*Jinv02*w51+2.6481331168831169e-04*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11+2.6481331168831169e-04*w55*Jinv10+3.1047077922077924e-04*w42*Jinv00+1.0957792207792208e-04*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+1.0227272727272728e-03*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+1.0227272727272728e-03*w48*Jinv01-6.4935064935064935e-04*Jinv12*w1-1.5726461038961039e-04*w10*Jinv11)*D;
  A[32][3]=(0.0)*D;
  A[32][4]=(0.0)*D;
  A[32][5]=(2.4655032467532467e-03*Jinv00*w12+3.7439123376623376e-04*w28*Jinv10-1.5888798701298701e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-1.1505681818181819e-03*Jinv20*w8-3.9265422077922079e-04*w57*Jinv02+2.4655032467532467e-03*w13*Jinv10-1.5888798701298701e-03*Jinv02*w39-1.0957792207792207e-03*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3-1.3697240259740260e-03*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-8.2183441558441553e-05*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+4.5657467532467534e-05*Jinv21*w2-1.0957792207792207e-03*w42*Jinv01-3.9265422077922079e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55+3.9265422077922079e-04*w58*Jinv10-1.5888798701298701e-03*w15*Jinv02-3.9265422077922079e-04*w10*Jinv12-1.3971185064935066e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-3.1777597402597403e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+4.5657467532467534e-05*w0*Jinv01+1.1414366883116883e-03*w1*Jinv10+5.4788961038961036e-04*w16*Jinv10+8.2183441558441553e-05*Jinv22*w20-1.0957792207792207e-03*w43*Jinv11-3.5612824675324673e-04*w45*Jinv00-1.5888798701298701e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02+3.7439123376623376e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22+4.5657467532467534e-05*Jinv11*w1-1.2327516233766234e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-3.7439123376623376e-04*w28*Jinv11-1.3971185064935066e-03*Jinv20*w50+3.7439123376623376e-04*Jinv20*w29-3.5612824675324673e-04*w46*Jinv10-1.5888798701298701e-03*Jinv22*w17+1.0957792207792207e-03*Jinv00*w33+1.0135957792207792e-03*w22*Jinv12+4.9310064935064934e-03*w31*Jinv10-2.4655032467532467e-03*Jinv02*w3-1.3697240259740260e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22-1.5888798701298701e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-3.9265422077922079e-04*Jinv22*w59-3.1777597402597403e-03*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53-3.1777597402597403e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11+2.6846590909090908e-03*w41*Jinv20-1.2327516233766234e-03*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-2.4655032467532467e-03*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20-1.1505681818181819e-03*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-8.2183441558441553e-05*Jinv20*w56-1.5888798701298701e-03*Jinv22*w41+3.5612824675324673e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-3.1777597402597403e-03*Jinv21*w35-2.4655032467532467e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11+2.4655032467532467e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54-2.4655032467532467e-03*w4*Jinv12+1.0135957792207792e-03*w21*Jinv02-1.3971185064935066e-03*w49*Jinv10-3.7439123376623376e-04*Jinv21*w29-3.1777597402597403e-03*Jinv01*w33+2.6846590909090908e-03*w39*Jinv00-1.2327516233766234e-03*Jinv02*w12+8.2183441558441553e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.2327516233766234e-03*w6*Jinv01-1.1505681818181819e-03*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+1.0135957792207792e-03*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12+1.2327516233766234e-03*Jinv22*w8-1.3697240259740260e-03*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50+1.0135957792207792e-03*w22*Jinv11+1.2327516233766234e-03*Jinv12*w7+1.0135957792207792e-03*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.2327516233766234e-03*w7*Jinv11+3.5612824675324673e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+8.2183441558441553e-05*Jinv21*w56-3.7439123376623376e-04*Jinv02*w27+1.2327516233766234e-03*w6*Jinv02+8.2183441558441553e-05*w19*Jinv12-2.4655032467532467e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.2327516233766234e-03*w49*Jinv11-1.2327516233766234e-03*Jinv21*w14+3.5612824675324673e-04*w10*Jinv10-3.9265422077922079e-04*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18-1.2327516233766234e-03*Jinv01*w12+1.2327516233766234e-03*Jinv21*w8+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59-3.1777597402597403e-03*Jinv02*w33+1.1414366883116883e-03*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+1.0135957792207792e-03*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+1.1414366883116883e-03*Jinv20*w2+5.4788961038961036e-04*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02-3.9265422077922079e-04*Jinv01*w9+5.4788961038961036e-04*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00+2.6846590909090908e-03*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-3.7439123376623376e-04*Jinv22*w29-1.2327516233766234e-03*Jinv22*w14-1.5888798701298701e-03*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20-2.4655032467532467e-03*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-3.7439123376623376e-04*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11-8.2183441558441553e-05*w55*Jinv10-3.5612824675324673e-04*w47*Jinv20-1.5888798701298701e-03*w16*Jinv12+1.2327516233766234e-03*Jinv21*w50-3.7439123376623376e-04*Jinv01*w27+1.2327516233766234e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1-3.9265422077922079e-04*w10*Jinv11)*D;
  A[32][6]=(0.0)*D;
  A[32][7]=(0.0)*D;
  A[32][8]=(-2.0271915584415584e-03*Jinv00*w12-3.1960227272727273e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+7.3965097402597403e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-2.0271915584415584e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-4.0543831168831168e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+1.1231737012987013e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.3697240259740259e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-9.1314935064935065e-06*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45-2.7394480519480518e-04*Jinv12*w55-1.7349837662337662e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+9.5880681818181818e-04*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26-9.1314935064935065e-06*w0*Jinv01-5.5702110389610384e-04*w1*Jinv10-4.3831168831168834e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+4.6570616883116886e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-3.1960227272727273e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-9.1314935064935065e-06*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+9.5880681818181818e-04*Jinv20*w50-3.1960227272727273e-04*Jinv20*w29+4.6570616883116886e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17-8.7662337662337668e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12-4.0543831168831168e-03*w31*Jinv10+1.6436688311688311e-04*Jinv02*w3+1.1231737012987013e-03*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+5.4788961038961036e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-8.2183441558441553e-05*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-2.5750811688311690e-03*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-8.7662337662337668e-04*w34*Jinv10+7.1225649350649346e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.3965097402597403e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41-2.7394480519480521e-05*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35+7.1225649350649346e-04*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-8.2183441558441553e-05*w46*Jinv11-2.0271915584415584e-03*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-2.7394480519480518e-04*Jinv02*w54+1.6436688311688311e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02+9.5880681818181818e-04*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-2.5750811688311690e-03*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+7.3965097402597403e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21+5.4788961038961036e-04*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12-8.2183441558441553e-05*w46*Jinv12-3.8352272727272725e-04*Jinv20*w53+1.6436688311688311e-04*Jinv22*w8+1.1231737012987013e-03*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+1.6436688311688311e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+5.4788961038961036e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-2.7394480519480521e-05*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+1.6436688311688311e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12+1.6436688311688311e-04*Jinv22*w5-8.2183441558441553e-05*w47*Jinv22-5.2049512987012989e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.7349837662337662e-04*w57*Jinv00-1.7349837662337662e-04*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33-5.5702110389610384e-04*w0*Jinv00-8.7662337662337668e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-2.7394480519480521e-05*w25*Jinv10-5.5702110389610384e-04*Jinv20*w2-4.3831168831168834e-04*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-4.3831168831168834e-04*Jinv20*w17-4.0543831168831168e-03*w30*Jinv00-2.5750811688311690e-03*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+7.1225649350649346e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-2.7394480519480518e-04*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+4.6570616883116886e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12-5.2049512987012989e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27-5.2049512987012989e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[32][9]=(0.0)*D;
  A[32][10]=(0.0)*D;
  A[32][11]=(7.4878246753246752e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-2.6481331168831169e-04*Jinv20*w8+7.4878246753246752e-04*w13*Jinv10+3.6525974025974026e-05*w3*Jinv00+1.4975649350649350e-03*w32*Jinv20-4.2918019480519480e-04*Jinv20*w23-4.2918019480519480e-04*Jinv00*w54+3.4699675324675324e-04*w43*Jinv10-1.4204545454545455e-05*w58*Jinv10-5.8441558441558442e-04*w48*Jinv00+6.4935064935064935e-05*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10-2.1915584415584417e-04*w45*Jinv00+1.0247564935064935e-04*Jinv00*w27-5.8441558441558442e-04*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10+4.3831168831168834e-04*Jinv00*w33+1.4975649350649350e-03*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-5.4788961038961042e-05*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00+1.1870941558441559e-04*w51*Jinv00+3.6525974025974026e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-5.4788961038961042e-05*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10-2.6481331168831169e-04*w6*Jinv00-4.2918019480519480e-04*Jinv20*w56-2.7394480519480521e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20+7.4878246753246752e-04*Jinv20*w14-5.8441558441558442e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-2.6481331168831169e-04*w7*Jinv10-1.0957792207792208e-04*Jinv20*w38+1.1870941558441559e-04*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10+1.1870941558441559e-04*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10-2.7394480519480521e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+3.4699675324675324e-04*Jinv20*w44-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+6.4935064935064935e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-2.7394480519480521e-05*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00+2.1915584415584417e-04*Jinv20*w17+1.4975649350649350e-03*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+3.6525974025974026e-05*w5*Jinv20-4.2918019480519480e-04*w55*Jinv10+3.4699675324675324e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20)*D;
  A[32][12]=(0.0)*D;
  A[32][13]=(0.0)*D;
  A[32][14]=(-2.4655032467532467e-03*Jinv00*w12-3.9265422077922079e-04*w28*Jinv10-3.1777597402597403e-03*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53+1.0135957792207792e-03*Jinv20*w8-3.9265422077922079e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-3.1777597402597403e-03*Jinv02*w39-1.2327516233766234e-03*w3*Jinv00-2.4655032467532467e-03*w32*Jinv20+2.4655032467532467e-03*Jinv01*w3+1.2327516233766234e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.9265422077922079e-04*Jinv21*w59+8.2183441558441553e-05*w25*Jinv12+4.5657467532467534e-05*w0*Jinv02+1.1414366883116883e-03*Jinv21*w2-1.0957792207792207e-03*w43*Jinv10+3.7439123376623376e-04*w11*Jinv21+5.4788961038961036e-04*w16*Jinv11+5.4788961038961036e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10-1.5888798701298701e-03*w15*Jinv02-3.7439123376623376e-04*w10*Jinv12+1.2327516233766234e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-1.5888798701298701e-03*Jinv22*w35+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.1414366883116883e-03*w0*Jinv01+4.5657467532467534e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+1.6436688311688311e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41-1.0957792207792207e-03*w42*Jinv02-3.9265422077922079e-04*Jinv00*w27-3.7439123376623376e-04*w11*Jinv22+1.1414366883116883e-03*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+5.4788961038961036e-04*w15*Jinv01+3.5612824675324673e-04*w28*Jinv11+1.2327516233766234e-03*Jinv20*w50-3.9265422077922079e-04*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.5888798701298701e-03*Jinv22*w17-1.5888798701298701e-03*Jinv00*w33+1.2327516233766234e-03*w22*Jinv12-2.4655032467532467e-03*w31*Jinv10-1.2327516233766234e-03*Jinv02*w3+1.2327516233766234e-03*w21*Jinv00-2.4655032467532467e-03*w32*Jinv22+1.0957792207792207e-03*Jinv01*w39+4.5657467532467534e-05*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00-3.5612824675324673e-04*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-3.9265422077922079e-04*Jinv22*w59+2.6846590909090908e-03*w34*Jinv11-8.2183441558441553e-05*Jinv21*w53+8.2183441558441553e-05*w51*Jinv00-1.5888798701298701e-03*Jinv12*w34-1.0957792207792207e-03*Jinv22*w44-1.2327516233766234e-03*w4*Jinv10+2.4655032467532467e-03*w4*Jinv11-3.1777597402597403e-03*w41*Jinv20-2.4655032467532467e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-1.5888798701298701e-03*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21+1.0135957792207792e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-3.1777597402597403e-03*Jinv22*w41+8.2183441558441553e-05*w26*Jinv20-3.7439123376623376e-04*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+2.6846590909090908e-03*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11+8.2183441558441553e-05*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14+2.4655032467532467e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-1.2327516233766234e-03*w4*Jinv12+1.2327516233766234e-03*w21*Jinv02+1.2327516233766234e-03*w49*Jinv10+3.5612824675324673e-04*Jinv21*w29+2.6846590909090908e-03*Jinv01*w33-3.1777597402597403e-03*w39*Jinv00-2.4655032467532467e-03*Jinv02*w12-3.5612824675324673e-04*w37*Jinv11-3.7439123376623376e-04*Jinv02*w9-1.3697240259740260e-03*w6*Jinv01+1.0135957792207792e-03*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+1.2327516233766234e-03*Jinv22*w23-8.2183441558441553e-05*w52*Jinv11-2.4655032467532467e-03*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53+1.0135957792207792e-03*Jinv22*w8+1.2327516233766234e-03*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50-1.1505681818181819e-03*w22*Jinv11+1.0135957792207792e-03*Jinv12*w7-1.1505681818181819e-03*w21*Jinv01-8.2183441558441553e-05*Jinv01*w51-2.4655032467532467e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49-3.5612824675324673e-04*w36*Jinv01+8.2183441558441553e-05*w52*Jinv10-1.3697240259740260e-03*w7*Jinv11-3.7439123376623376e-04*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10+8.2183441558441553e-05*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+1.0135957792207792e-03*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-1.2327516233766234e-03*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-1.3971185064935066e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14-3.7439123376623376e-04*w10*Jinv10-1.0957792207792207e-03*Jinv20*w44-3.9265422077922079e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-1.3697240259740260e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59-1.5888798701298701e-03*Jinv02*w33+4.5657467532467534e-05*w0*Jinv00-1.5888798701298701e-03*w35*Jinv20-1.1505681818181819e-03*Jinv21*w23+8.2183441558441553e-05*w25*Jinv10+4.5657467532467534e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02+3.7439123376623376e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-2.4655032467532467e-03*w30*Jinv00-3.1777597402597403e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29-2.4655032467532467e-03*Jinv22*w14+1.0957792207792207e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-1.0957792207792207e-03*Jinv12*w43-1.2327516233766234e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10-1.0957792207792207e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.5888798701298701e-03*w16*Jinv12-1.3971185064935066e-03*Jinv21*w50+3.5612824675324673e-04*Jinv01*w27-1.3971185064935066e-03*w48*Jinv01+4.5657467532467534e-05*Jinv12*w1+3.7439123376623376e-04*w10*Jinv11)*D;
  A[32][15]=(0.0)*D;
  A[32][16]=(0.0)*D;
  A[32][17]=(3.1777597402597403e-03*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53-9.3141233766233771e-04*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+1.0409902597402598e-03*w3*Jinv00+3.1777597402597403e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3-1.7532467532467534e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38-7.1225649350649346e-04*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+5.4788961038961042e-05*Jinv12*w55+5.2962662337662338e-04*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12-1.7532467532467534e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01-5.4788961038961042e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+3.2873376623376621e-03*w42*Jinv02+5.2962662337662338e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.0409902597402598e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11-1.7532467532467534e-03*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12+3.1777597402597403e-03*w31*Jinv10-1.0957792207792207e-03*Jinv02*w3-1.7532467532467534e-03*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-7.1225649350649346e-04*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+3.6525974025974026e-05*Jinv22*w59+3.2873376623376621e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+3.2873376623376621e-03*Jinv22*w44+1.0409902597402598e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+3.2873376623376621e-03*w41*Jinv20-1.0957792207792207e-03*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+3.1777597402597403e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-9.3141233766233771e-04*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12-7.1225649350649346e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+3.2873376623376621e-03*Jinv21*w35+3.1777597402597403e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+3.1777597402597403e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54-1.0957792207792207e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02-1.7532467532467534e-03*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+3.2873376623376621e-03*Jinv01*w33+3.2873376623376621e-03*w39*Jinv00-1.0957792207792207e-03*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.7532467532467534e-03*w6*Jinv01-9.3141233766233771e-04*w7*Jinv10+3.8352272727272725e-04*Jinv22*w23-7.1225649350649346e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-1.7532467532467534e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-9.3141233766233771e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-9.3141233766233771e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49-7.1225649350649346e-04*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10-1.7532467532467534e-03*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-3.2873376623376621e-04*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-1.0957792207792207e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22-1.7532467532467534e-03*w49*Jinv11+1.0409902597402598e-03*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18+1.0409902597402598e-03*Jinv01*w12-1.7532467532467534e-03*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20-9.3141233766233771e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+5.2962662337662338e-04*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+3.1777597402597403e-03*w30*Jinv00+3.2873376623376621e-03*w40*Jinv10+3.6525974025974026e-05*Jinv22*w29-1.0957792207792207e-03*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+3.2873376623376621e-03*Jinv12*w43+1.0409902597402598e-03*w5*Jinv20+3.1777597402597403e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28-7.1225649350649346e-04*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-1.7532467532467534e-03*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27-1.7532467532467534e-03*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+5.2962662337662338e-04*w10*Jinv11)*D;
  A[32][18]=(0.0)*D;
  A[32][19]=(0.0)*D;
  A[32][20]=(-7.1225649350649346e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-8.2183441558441553e-05*Jinv20*w8-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv21*w44+1.9176136363636362e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+5.2049512987012989e-04*Jinv20*w23+5.4788961038961036e-04*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-8.7662337662337668e-04*w43*Jinv10-5.4788961038961042e-05*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-5.4788961038961042e-05*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-1.3697240259740259e-04*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+5.2049512987012989e-04*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-3.5612824675324673e-04*Jinv00*w18-3.5612824675324673e-04*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38+2.7394480519480518e-04*Jinv01*w24-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00+1.9176136363636362e-04*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+5.4788961038961036e-04*Jinv20*w56+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20-7.6704545454545450e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+8.2183441558441553e-05*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-3.5612824675324673e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-1.6436688311688311e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-3.5612824675324673e-04*Jinv20*w38+5.2049512987012989e-04*w52*Jinv11+2.4655032467532466e-04*Jinv20*w53+5.2049512987012989e-04*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-1.6436688311688311e-04*w7*Jinv11+6.3920454545454550e-05*w9*Jinv00-3.5612824675324673e-04*w37*Jinv10+2.7394480519480518e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+6.3920454545454550e-05*w10*Jinv10-8.7662337662337668e-04*Jinv20*w44-3.5612824675324673e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23+2.7394480519480518e-04*Jinv01*w54+2.7394480519480518e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-7.1225649350649346e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+1.9176136363636362e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+2.7394480519480518e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10-8.7662337662337668e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[32][21]=(0.0)*D;
  A[32][22]=(0.0)*D;
  A[32][23]=(1.6436688311688311e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12-2.7394480519480518e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+7.1225649350649346e-04*w32*Jinv20-2.0271915584415584e-03*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-8.2183441558441553e-05*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54-1.7349837662337662e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-9.1314935064935065e-06*w0*Jinv02-5.5702110389610384e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-3.1960227272727273e-04*w11*Jinv21-4.3831168831168834e-04*w16*Jinv11-4.3831168831168834e-04*Jinv21*w17+5.4788961038961036e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12-5.2049512987012989e-04*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35-1.7349837662337662e-04*w58*Jinv11-1.7349837662337662e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-5.5702110389610384e-04*w0*Jinv01-9.1314935064935065e-06*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-8.7662337662337668e-04*Jinv21*w41+5.4788961038961042e-05*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22-5.5702110389610384e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11-4.3831168831168834e-04*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11-5.2049512987012989e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+1.6436688311688311e-04*w22*Jinv12+7.1225649350649346e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22-8.7662337662337668e-04*Jinv01*w39-9.1314935064935065e-06*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00+4.6570616883116886e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-2.5750811688311690e-03*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+5.4788961038961042e-05*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10-2.0271915584415584e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20+1.6436688311688311e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10-4.0543831168831168e-03*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-8.2183441558441553e-05*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20-2.7394480519480518e-04*w52*Jinv12-2.5750811688311690e-03*Jinv21*w35-4.0543831168831168e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+5.4788961038961036e-04*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-2.0271915584415584e-03*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12+1.6436688311688311e-04*w21*Jinv02-5.2049512987012989e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29-2.5750811688311690e-03*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00+1.6436688311688311e-04*Jinv02*w12-2.7394480519480521e-05*Jinv01*w18+4.6570616883116886e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9+1.1231737012987013e-03*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+5.4788961038961036e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+1.6436688311688311e-04*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10-5.2049512987012989e-04*Jinv22*w50+7.3965097402597403e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7+7.3965097402597403e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30-5.2049512987012989e-04*Jinv12*w49+4.6570616883116886e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+1.1231737012987013e-03*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+9.5880681818181818e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+1.1231737012987013e-03*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33-9.1314935064935065e-06*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+7.3965097402597403e-04*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-9.1314935064935065e-06*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-5.2049512987012989e-04*w48*Jinv02-3.1960227272727273e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17+7.1225649350649346e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+1.6436688311688311e-04*Jinv22*w14-8.7662337662337668e-04*w40*Jinv11-8.2183441558441553e-05*Jinv02*w36+5.4788961038961042e-05*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20-4.0543831168831168e-03*Jinv01*w30-2.7394480519480518e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+5.4788961038961042e-05*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12+9.5880681818181818e-04*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+9.5880681818181818e-04*w48*Jinv01-9.1314935064935065e-06*Jinv12*w1-3.1960227272727273e-04*w10*Jinv11)*D;
  A[32][24]=(0.0)*D;
  A[32][25]=(0.0)*D;
  A[32][26]=(-1.6436688311688311e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-8.7662337662337668e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3-1.6436688311688311e-04*Jinv20*w23+5.2049512987012989e-04*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59+9.1314935064935065e-06*Jinv21*w2-5.4788961038961042e-05*w43*Jinv10-8.7662337662337668e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-3.5612824675324673e-04*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+9.1314935064935065e-06*w0*Jinv01+9.1314935064935065e-06*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-8.7662337662337668e-04*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27+9.1314935064935065e-06*Jinv11*w1+1.9176136363636362e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-1.6436688311688311e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+2.7394480519480518e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+2.7394480519480518e-04*w51*Jinv00-3.5612824675324673e-04*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00+5.2049512987012989e-04*Jinv20*w56-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-1.0957792207792208e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-3.5612824675324673e-04*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+2.4655032467532466e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+5.2049512987012989e-04*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-3.5612824675324673e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38+5.4788961038961036e-04*w52*Jinv11+2.7394480519480518e-04*Jinv20*w53-1.6436688311688311e-04*w22*Jinv10-8.2183441558441553e-05*w22*Jinv11-8.2183441558441553e-05*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+5.4788961038961036e-04*w36*Jinv01+2.7394480519480518e-04*w52*Jinv10+5.2049512987012989e-04*w7*Jinv11-1.0957792207792208e-04*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11+1.9176136363636362e-04*Jinv21*w14-1.0957792207792208e-04*w10*Jinv10-5.4788961038961042e-05*Jinv20*w44+1.9176136363636362e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59+9.1314935064935065e-06*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10+9.1314935064935065e-06*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.3697240259740259e-04*Jinv01*w9-3.8352272727272725e-04*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11-3.5612824675324673e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-3.5612824675324673e-04*w25*Jinv11+5.2049512987012989e-04*w55*Jinv10-5.4788961038961042e-05*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-1.3697240259740259e-04*w10*Jinv11)*D;
  A[32][27]=(0.0)*D;
  A[32][28]=(0.0)*D;
  A[32][29]=(3.4699675324675324e-04*Jinv21*w44+7.4878246753246752e-04*Jinv01*w3-1.4204545454545455e-05*Jinv21*w59+6.4935064935064935e-05*Jinv21*w2+3.4699675324675324e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+3.4699675324675324e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41+6.4935064935064935e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+4.3831168831168834e-04*Jinv01*w39-2.1915584415584417e-04*Jinv21*w38-5.4788961038961042e-05*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11-4.2918019480519480e-04*Jinv21*w53+7.4878246753246752e-04*w4*Jinv11+1.4975649350649350e-03*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35+1.4975649350649350e-03*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11+7.4878246753246752e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-2.7394480519480521e-05*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-1.0957792207792208e-04*w47*Jinv21-4.2918019480519480e-04*w52*Jinv11-2.6481331168831169e-04*w22*Jinv11-2.6481331168831169e-04*w21*Jinv01-4.2918019480519480e-04*Jinv01*w51-2.1915584415584417e-04*w36*Jinv01-4.2918019480519480e-04*w7*Jinv11+1.1870941558441559e-04*Jinv21*w56-5.8441558441558442e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+3.6525974025974026e-05*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8-2.6481331168831169e-04*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+1.0247564935064935e-04*Jinv01*w9+4.3831168831168834e-04*w40*Jinv11+1.4975649350649350e-03*Jinv01*w30-5.4788961038961042e-05*w25*Jinv11-5.8441558441558442e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-5.8441558441558442e-04*w48*Jinv01+1.0247564935064935e-04*w10*Jinv11)*D;
  A[32][30]=(0.0)*D;
  A[32][31]=(0.0)*D;
  A[32][32]=(-2.4655032467532467e-03*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-7.3965097402597403e-04*Jinv22*w53+2.0271915584415584e-03*Jinv20*w8+1.2784090909090910e-04*w57*Jinv02-2.4655032467532467e-03*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44-2.4655032467532467e-03*w3*Jinv00-7.3965097402597406e-03*w32*Jinv20-2.4655032467532467e-03*Jinv01*w3+2.0271915584415584e-03*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-5.7528409090909093e-04*Jinv00*w54-6.5746753246753243e-04*Jinv21*w59+1.1870941558441558e-03*w0*Jinv02+9.1314935064935068e-05*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.5888798701298701e-03*w16*Jinv11-1.5888798701298701e-03*Jinv21*w17+1.0683847402597404e-03*Jinv01*w45-7.3965097402597403e-04*Jinv12*w55-6.5746753246753243e-04*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02+1.3149350649350649e-03*w48*Jinv00-4.9310064935064932e-04*Jinv22*w35-6.5746753246753243e-04*w58*Jinv11-6.5746753246753243e-04*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26+9.1314935064935068e-05*w0*Jinv01+9.1314935064935068e-05*w1*Jinv10-1.5888798701298701e-03*w16*Jinv10-1.6436688311688311e-03*w43*Jinv11+1.0683847402597404e-03*w45*Jinv00-4.7666396103896100e-03*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27+9.1314935064935068e-05*Jinv11*w1-2.4655032467532467e-03*w13*Jinv11-1.5888798701298701e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+1.0683847402597404e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-4.7666396103896100e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-7.3965097402597406e-03*w31*Jinv10+1.2327516233766234e-03*Jinv02*w3+2.0271915584415584e-03*w21*Jinv00-4.7666396103896100e-03*Jinv01*w39+1.1870941558441558e-03*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+1.0683847402597404e-03*w36*Jinv00+1.0683847402597404e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24+1.2784090909090910e-04*Jinv22*w59-4.7666396103896100e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.7528409090909093e-04*Jinv21*w53-5.7528409090909093e-04*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44-2.4655032467532467e-03*w4*Jinv10-2.4655032467532467e-03*w4*Jinv11-4.7666396103896100e-03*w41*Jinv20+1.2327516233766234e-03*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-4.7666396103896100e-03*w34*Jinv10-7.3965097402597406e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+2.0271915584415584e-03*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-5.7528409090909093e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-7.3965097402597403e-04*w52*Jinv12-4.7666396103896100e-03*Jinv21*w35-7.3965097402597406e-03*w31*Jinv11+1.0683847402597404e-03*w46*Jinv11-2.4655032467532467e-03*Jinv20*w14-2.4655032467532467e-03*w5*Jinv21-7.3965097402597403e-04*Jinv02*w54+1.2327516233766234e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.3149350649350649e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.7666396103896100e-03*Jinv01*w33-4.7666396103896100e-03*w39*Jinv00+1.2327516233766234e-03*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.0683847402597404e-03*w37*Jinv11+2.0271915584415584e-03*w6*Jinv01+2.0271915584415584e-03*w7*Jinv10+1.0683847402597404e-03*w47*Jinv21+1.0683847402597404e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-5.7528409090909093e-04*w52*Jinv11+3.5612824675324673e-04*w46*Jinv12-5.7528409090909093e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+2.0271915584415584e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50+2.0271915584415584e-03*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+2.0271915584415584e-03*w21*Jinv01-5.7528409090909093e-04*Jinv01*w51-1.3149350649350649e-03*Jinv12*w49+1.0683847402597404e-03*w36*Jinv01-5.7528409090909093e-04*w52*Jinv10+2.0271915584415584e-03*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+1.0683847402597404e-03*w37*Jinv10-5.7528409090909093e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+1.2327516233766234e-03*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22+1.3149350649350649e-03*w49*Jinv11-2.4655032467532467e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+1.2784090909090910e-04*w58*Jinv12-2.4655032467532467e-03*Jinv01*w12+2.0271915584415584e-03*Jinv21*w8-6.5746753246753243e-04*w57*Jinv00-6.5746753246753243e-04*Jinv20*w59-4.9310064935064932e-04*Jinv02*w33+9.1314935064935068e-05*w0*Jinv00-4.7666396103896100e-03*w35*Jinv20+2.0271915584415584e-03*Jinv21*w23-5.7528409090909093e-04*Jinv01*w54-5.7528409090909093e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+9.1314935064935068e-05*Jinv20*w2-1.5888798701298701e-03*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02-7.4878246753246752e-04*Jinv01*w9-1.5888798701298701e-03*Jinv20*w17-7.3965097402597406e-03*w30*Jinv00-4.7666396103896100e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+1.2327516233766234e-03*Jinv22*w14-4.7666396103896100e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43-2.4655032467532467e-03*w5*Jinv20-7.3965097402597406e-03*Jinv01*w30-7.3965097402597403e-04*Jinv02*w51-7.3965097402597403e-04*Jinv22*w56+3.5612824675324673e-04*w25*Jinv11-5.7528409090909093e-04*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+1.0683847402597404e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12+1.3149350649350649e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01+1.1870941558441558e-03*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[32][33]=(0.0)*D;
  A[32][34]=(0.0)*D;
  A[32][35]=(3.1777597402597403e-03*Jinv00*w12+9.6793831168831166e-04*w28*Jinv10+2.9586038961038961e-03*w40*Jinv12+1.8628246753246754e-03*Jinv22*w53-1.8628246753246754e-03*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02+3.1777597402597403e-03*w13*Jinv10+2.9586038961038961e-03*Jinv02*w39+4.9310064935064934e-03*Jinv21*w44+2.0819805194805196e-03*w3*Jinv00+9.5332792207792200e-03*w32*Jinv20-2.1915584415584414e-03*Jinv01*w3-2.5750811688311690e-03*Jinv20*w23-6.0267857142857139e-04*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12+4.3831168831168834e-04*w0*Jinv02+9.8620129870129864e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+4.9310064935064934e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-1.1505681818181819e-03*Jinv01*w45+9.3141233766233771e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+5.6615259740259744e-04*w10*Jinv12-2.7942370129870131e-03*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+9.8620129870129864e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-3.2873376623376621e-04*Jinv22*w20+4.9310064935064934e-03*w43*Jinv11-1.3149350649350649e-03*w45*Jinv00+1.9724025974025973e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02+9.6793831168831166e-04*Jinv00*w27+5.6615259740259744e-04*w11*Jinv22+9.8620129870129864e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.2004870129870131e-04*w28*Jinv11-2.7942370129870131e-03*Jinv20*w50+9.6793831168831166e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17+6.9034090909090911e-03*Jinv00*w33-1.2601461038961039e-03*w22*Jinv12+9.5332792207792200e-03*w31*Jinv10+2.0819805194805196e-03*Jinv02*w3-2.5750811688311690e-03*w21*Jinv00+9.8620129870129864e-04*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+4.3831168831168834e-04*Jinv22*w2+1.6436688311688311e-04*Jinv21*w38-1.2053571428571428e-03*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24+2.7394480519480518e-04*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+4.9310064935064932e-04*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+2.0819805194805196e-03*w4*Jinv10-2.1915584415584414e-03*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20+1.0409902597402598e-03*Jinv12*w13+6.9034090909090911e-03*w34*Jinv10-3.2873376623376621e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20-1.8628246753246754e-03*w6*Jinv00-6.0267857142857139e-04*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56+2.9586038961038961e-03*Jinv22*w41-8.2183441558441553e-04*w26*Jinv20+6.2094155844155847e-04*w11*Jinv20+1.8628246753246754e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35-3.2873376623376621e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26-1.1505681818181819e-03*w46*Jinv11+3.1777597402597403e-03*Jinv20*w14-2.1915584415584414e-03*w5*Jinv21+9.3141233766233771e-04*Jinv02*w54+2.0819805194805196e-03*w4*Jinv12-1.2601461038961039e-03*w21*Jinv02-2.7942370129870131e-03*w49*Jinv10+4.2004870129870131e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00+1.0409902597402598e-03*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01-1.8628246753246754e-03*w7*Jinv10-1.1505681818181819e-03*w47*Jinv21-1.2601461038961039e-03*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11+9.8620129870129864e-04*w31*Jinv12-1.2053571428571428e-03*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8-2.5750811688311690e-03*w22*Jinv10+5.4788961038961042e-05*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51+9.8620129870129864e-04*Jinv02*w30+1.6436688311688311e-04*w36*Jinv01+4.9310064935064932e-04*w52*Jinv10+7.6704545454545450e-04*w7*Jinv11+6.2094155844155847e-04*w9*Jinv00+1.3149350649350649e-03*Jinv21*w56-8.2183441558441553e-04*w24*Jinv00+6.9399350649350649e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.2873376623376621e-04*w19*Jinv12+2.0819805194805196e-03*Jinv22*w5-1.2053571428571428e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+6.2094155844155847e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-3.2873376623376621e-04*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+5.9172077922077923e-03*Jinv02*w33-1.0957792207792208e-04*w0*Jinv00+6.9034090909090911e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11-8.2183441558441553e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+7.3051948051948052e-05*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17+9.5332792207792200e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10+6.9399350649350649e-04*Jinv22*w29+1.0409902597402598e-03*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-6.0267857142857139e-04*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+2.0819805194805196e-03*w5*Jinv20-3.2873376623376621e-03*Jinv01*w30+1.8628246753246754e-03*Jinv02*w51+9.3141233766233771e-04*Jinv22*w56+6.9399350649350649e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11-8.2183441558441553e-04*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.2004870129870131e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01+4.3831168831168834e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[32][36]=(0.0)*D;
  A[32][37]=(0.0)*D;
  A[32][38]=(-7.1225649350649346e-04*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-1.3697240259740259e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23-7.1225649350649346e-04*Jinv22*w38+1.3971185064935066e-03*Jinv00*w54+2.7394480519480518e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10+2.7394480519480518e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+1.4793019480519481e-03*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35+1.8262987012987013e-05*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+2.4655032467532466e-04*w45*Jinv00-5.4788961038961042e-05*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.4793019480519481e-03*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.4655032467532466e-04*w46*Jinv10-3.8352272727272725e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22+9.1314935064935065e-06*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-1.0683847402597404e-03*w36*Jinv00+8.2183441558441553e-05*Jinv02*w45-1.3697240259740259e-04*Jinv22*w59+8.2183441558441553e-05*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-1.6436688311688311e-04*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-7.1225649350649346e-04*w37*Jinv12+1.3971185064935066e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+4.6570616883116886e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+5.2049512987012989e-04*w52*Jinv12+2.7394480519480518e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+2.7394480519480518e-04*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02+1.4793019480519481e-03*w49*Jinv10-1.6436688311688311e-04*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10-1.0683847402597404e-03*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12+8.2183441558441553e-05*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10+5.2049512987012989e-04*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+8.2183441558441553e-05*w52*Jinv10+1.2784090909090910e-04*w9*Jinv00-1.0683847402597404e-03*w37*Jinv10+4.6570616883116886e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5+8.2183441558441553e-05*w47*Jinv22+1.2784090909090910e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-1.3697240259740259e-04*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33+1.8262987012987013e-05*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+4.6570616883116886e-04*w25*Jinv10+1.8262987012987013e-05*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+5.2049512987012989e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-7.1225649350649346e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20+5.2049512987012989e-04*Jinv02*w51+2.7394480519480518e-04*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28+1.3971185064935066e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+2.4655032467532466e-04*w47*Jinv20-3.8352272727272725e-04*w16*Jinv12+9.1314935064935065e-06*Jinv12*w1)*D;
  A[32][39]=(0.0)*D;
  A[32][40]=(0.0)*D;
  A[32][41]=(-2.1915584415584414e-03*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12+9.3141233766233771e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+2.7394480519480518e-04*w57*Jinv02-2.1915584415584414e-03*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-3.2873376623376621e-03*w32*Jinv20+3.1777597402597403e-03*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23-1.2053571428571428e-03*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59-3.2873376623376621e-04*w25*Jinv12+4.3831168831168834e-04*w0*Jinv02-1.0957792207792208e-04*Jinv21*w2+4.9310064935064934e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+9.6793831168831166e-04*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+1.8628246753246754e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+6.9399350649350649e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-3.2873376623376621e-04*Jinv02*w24+2.9586038961038961e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-1.0957792207792208e-04*w0*Jinv01+9.8620129870129864e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+6.9034090909090911e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+6.9399350649350649e-04*w11*Jinv22-1.0957792207792208e-04*Jinv11*w1+2.0819805194805196e-03*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01+6.2094155844155847e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17+1.9724025974025973e-03*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12-3.2873376623376621e-03*w31*Jinv10+1.0409902597402598e-03*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+9.8620129870129864e-04*w32*Jinv22+6.9034090909090911e-03*Jinv01*w39+4.3831168831168834e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-1.1505681818181819e-03*w36*Jinv00-1.3149350649350649e-03*Jinv21*w38-6.0267857142857139e-04*Jinv02*w45+2.7394480519480518e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11-8.2183441558441553e-04*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53+1.3149350649350649e-03*w51*Jinv00+2.9586038961038961e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10+3.1777597402597403e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+2.0819805194805196e-03*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10+9.5332792207792200e-03*w32*Jinv21-8.2183441558441553e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-1.2053571428571428e-03*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20+4.2004870129870131e-04*w11*Jinv20+9.3141233766233771e-04*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35+9.5332792207792200e-03*w31*Jinv11-3.2873376623376621e-04*Jinv22*w26-2.1915584415584414e-03*Jinv20*w14+3.1777597402597403e-03*w5*Jinv21+1.8628246753246754e-03*Jinv02*w54+1.0409902597402598e-03*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+6.2094155844155847e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+2.0819805194805196e-03*Jinv02*w12-8.2183441558441553e-04*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+6.9399350649350649e-04*Jinv02*w9-2.5750811688311690e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-1.1505681818181819e-03*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11+9.8620129870129864e-04*w31*Jinv12-6.0267857142857139e-04*w46*Jinv12+1.3149350649350649e-03*Jinv20*w53-1.2601461038961039e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10-1.8628246753246754e-03*w22*Jinv11-1.2601461038961039e-03*Jinv12*w7-1.8628246753246754e-03*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51+9.8620129870129864e-04*Jinv02*w30-1.3149350649350649e-03*w36*Jinv01+1.3149350649350649e-03*w52*Jinv10-2.5750811688311690e-03*w7*Jinv11+4.2004870129870131e-04*w9*Jinv00-1.1505681818181819e-03*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+5.6615259740259744e-04*Jinv02*w27-1.2601461038961039e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12+1.0409902597402598e-03*Jinv22*w5-6.0267857142857139e-04*w47*Jinv22-2.7942370129870131e-03*w49*Jinv11+2.0819805194805196e-03*Jinv21*w14+4.2004870129870131e-04*w10*Jinv10+4.9310064935064934e-03*Jinv20*w44+2.7394480519480518e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18+2.0819805194805196e-03*Jinv01*w12-2.5750811688311690e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+2.9586038961038961e-03*Jinv02*w33+9.8620129870129864e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-1.8628246753246754e-03*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10+9.8620129870129864e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+9.6793831168831166e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-3.2873376623376621e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20+5.6615259740259744e-04*Jinv22*w29+2.0819805194805196e-03*Jinv22*w14+6.9034090909090911e-03*w40*Jinv11-1.2053571428571428e-03*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+9.5332792207792200e-03*Jinv01*w30+9.3141233766233771e-04*Jinv02*w51+1.8628246753246754e-03*Jinv22*w56+5.6615259740259744e-04*Jinv12*w28+2.6298701298701297e-03*w55*Jinv10+4.9310064935064934e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-2.7942370129870131e-03*Jinv21*w50+6.2094155844155847e-04*Jinv01*w27-2.7942370129870131e-03*w48*Jinv01+4.3831168831168834e-04*Jinv12*w1+9.6793831168831166e-04*w10*Jinv11)*D;
  A[32][42]=(0.0)*D;
  A[32][43]=(0.0)*D;
  A[32][44]=(2.1915584415584414e-03*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-3.6525974025974026e-05*w57*Jinv02+2.1915584415584414e-03*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+1.0957792207792207e-03*w3*Jinv00+3.2873376623376621e-03*w32*Jinv20+2.1915584415584414e-03*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23-5.4788961038961042e-05*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-9.8620129870129864e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-5.4788961038961042e-05*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-9.8620129870129864e-04*w0*Jinv01-9.8620129870129864e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41-3.2873376623376621e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22-9.8620129870129864e-04*Jinv11*w1+1.0957792207792207e-03*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.2004870129870131e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+3.2873376623376621e-03*w31*Jinv10+1.0957792207792207e-03*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00+1.0957792207792207e-03*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38-5.4788961038961042e-05*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24-3.6525974025974026e-05*Jinv22*w59-3.9448051948051946e-03*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.3149350649350649e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44+1.0957792207792207e-03*w4*Jinv10+2.1915584415584414e-03*w4*Jinv11-3.9448051948051946e-03*w41*Jinv20+1.0957792207792207e-03*Jinv12*w13+7.6704545454545450e-04*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+3.2873376623376621e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-5.4788961038961042e-05*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20-4.2004870129870131e-04*w11*Jinv20-5.4788961038961042e-05*w52*Jinv12-3.9448051948051946e-03*Jinv21*w35+3.2873376623376621e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11+2.1915584415584414e-03*Jinv20*w14+2.1915584415584414e-03*w5*Jinv21-5.4788961038961042e-05*Jinv02*w54+1.0957792207792207e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-4.2004870129870131e-04*Jinv21*w29-3.9448051948051946e-03*Jinv01*w33-3.9448051948051946e-03*w39*Jinv00+1.0957792207792207e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.1505681818181819e-03*w47*Jinv21+1.1505681818181819e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11+1.0957792207792207e-03*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+1.0957792207792207e-03*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-1.3149350649350649e-03*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11-4.2004870129870131e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10-1.3149350649350649e-03*Jinv21*w56-5.4788961038961042e-05*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12+1.0957792207792207e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+1.0957792207792207e-03*Jinv21*w14-4.2004870129870131e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-3.6525974025974026e-05*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+1.0957792207792207e-03*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-9.8620129870129864e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11-5.4788961038961042e-05*w25*Jinv10-9.8620129870129864e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+3.2873376623376621e-03*w30*Jinv00-3.9448051948051946e-03*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29+1.0957792207792207e-03*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43+1.0957792207792207e-03*w5*Jinv20+3.2873376623376621e-03*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.2004870129870131e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[32][45]=(0.0)*D;
  A[32][46]=(0.0)*D;
  A[32][47]=(-7.6704545454545450e-04*w40*Jinv12+2.7394480519480518e-04*Jinv22*w53-1.3697240259740259e-04*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3+8.2183441558441553e-05*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12+9.1314935064935065e-06*w0*Jinv02+1.8262987012987013e-05*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-1.0683847402597404e-03*Jinv01*w45+5.2049512987012989e-04*Jinv12*w55-3.8352272727272725e-04*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-3.5612824675324673e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+1.8262987012987013e-05*w0*Jinv01+2.7394480519480518e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02-1.0957792207792208e-04*w11*Jinv22+1.8262987012987013e-05*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.8352272727272725e-04*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39+9.1314935064935065e-06*Jinv22*w2+2.4655032467532466e-04*Jinv21*w38-7.1225649350649346e-04*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-1.3697240259740259e-04*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11+4.6570616883116886e-04*w19*Jinv11+1.3971185064935066e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21+4.6570616883116886e-04*Jinv21*w20+8.2183441558441553e-05*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41+2.7394480519480518e-04*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26-1.0683847402597404e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+5.2049512987012989e-04*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12+4.6570616883116886e-04*Jinv01*w18+2.4655032467532466e-04*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01-1.0683847402597404e-03*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+1.3971185064935066e-03*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8+5.2049512987012989e-04*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01+1.3971185064935066e-03*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30+5.2049512987012989e-04*Jinv12*w49+2.4655032467532466e-04*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11+8.2183441558441553e-05*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.4793019480519481e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-1.3697240259740259e-04*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+5.2049512987012989e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+8.2183441558441553e-05*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+2.7394480519480518e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11-3.8352272727272725e-04*w16*Jinv12+1.4793019480519481e-03*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+1.4793019480519481e-03*w48*Jinv01+9.1314935064935065e-06*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[32][48]=(0.0)*D;
  A[32][49]=(0.0)*D;
  A[32][50]=(1.6436688311688311e-04*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10+1.4793019480519481e-03*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-5.2049512987012989e-04*Jinv20*w8+4.0178571428571428e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+1.4793019480519481e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-5.2049512987012989e-04*Jinv20*w23-9.5880681818181818e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.7394480519480521e-05*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02+4.7483766233766234e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-4.3831168831168834e-04*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+2.6298701298701297e-03*w48*Jinv00-2.7394480519480521e-05*Jinv02*w24+1.4793019480519481e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+4.7483766233766234e-04*w0*Jinv01+4.7483766233766234e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480521e-05*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41-5.4788961038961042e-05*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27+2.7394480519480518e-04*w11*Jinv22+4.7483766233766234e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.6298701298701297e-03*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10-4.3831168831168834e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12-1.1505681818181819e-03*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3-5.2049512987012989e-04*w21*Jinv00+2.6298701298701297e-03*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38-9.5880681818181818e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24+4.0178571428571428e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00+1.4793019480519481e-03*Jinv12*w34-5.4788961038961042e-05*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-1.1505681818181819e-03*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-5.2049512987012989e-04*w6*Jinv00-9.5880681818181818e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56+1.4793019480519481e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11-2.7394480519480521e-05*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02+2.6298701298701297e-03*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9-5.2049512987012989e-04*w6*Jinv01-5.2049512987012989e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11+2.6298701298701297e-03*w31*Jinv12-9.5880681818181818e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8-5.2049512987012989e-04*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-5.2049512987012989e-04*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-5.2049512987012989e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51+2.6298701298701297e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-5.2049512987012989e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5-9.5880681818181818e-04*w47*Jinv22+2.6298701298701297e-03*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+4.0178571428571428e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-5.2049512987012989e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+1.4793019480519481e-03*Jinv02*w33+4.7483766233766234e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-5.2049512987012989e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10+4.7483766233766234e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-9.5880681818181818e-04*Jinv02*w36-5.4788961038961042e-05*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30+1.1505681818181819e-03*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20-4.3831168831168834e-04*w16*Jinv12+2.6298701298701297e-03*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.6298701298701297e-03*w48*Jinv01-6.2094155844155847e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[32][51]=(0.0)*D;
  A[32][52]=(0.0)*D;
  A[32][53]=(-1.6436688311688311e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-2.0271915584415584e-03*Jinv22*w53+5.2049512987012989e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39-1.6436688311688311e-04*w3*Jinv00+1.1505681818181819e-03*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+4.6570616883116886e-04*w25*Jinv12-4.8396915584415583e-04*w0*Jinv02-1.3149350649350649e-03*w43*Jinv10-1.0135957792207792e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-7.1225649350649346e-04*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00+4.6570616883116886e-04*Jinv02*w24-2.7394480519480520e-03*Jinv22*w35-4.7483766233766234e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+2.4655032467532466e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-2.5202922077922079e-03*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-2.6298701298701297e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-7.1225649350649346e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.1505681818181819e-03*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-4.8396915584415583e-04*Jinv22*w2+2.7394480519480518e-04*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+8.4922889610389610e-04*Jinv02*w45+1.1870941558441559e-04*Jinv22*w59-2.3833198051948050e-03*w51*Jinv00-2.7394480519480520e-03*Jinv12*w34-2.5202922077922079e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv10-2.1367694805194807e-03*w41*Jinv20+1.9176136363636362e-04*Jinv12*w13+2.7394480519480518e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10+5.2049512987012989e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+2.7394480519480518e-04*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-2.0271915584415584e-03*w52*Jinv12+4.6570616883116886e-04*Jinv22*w26-1.6436688311688311e-04*Jinv20*w14-1.0135957792207792e-03*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10-2.1367694805194807e-03*w39*Jinv00+1.9176136363636362e-04*Jinv02*w12-1.7349837662337662e-04*Jinv02*w9+5.2049512987012989e-04*w7*Jinv10+1.3971185064935066e-03*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23+1.4793019480519481e-03*w31*Jinv12+8.4922889610389610e-04*w46*Jinv12-2.3833198051948050e-03*Jinv20*w53+1.3697240259740259e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+1.3697240259740259e-04*Jinv12*w7+1.4793019480519481e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49-2.3833198051948050e-03*w52*Jinv10-4.2918019480519480e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10+2.7394480519480518e-04*w24*Jinv00-3.1960227272727273e-04*Jinv02*w27+1.3697240259740259e-04*w6*Jinv02+2.4655032467532466e-04*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5+8.4922889610389610e-04*w47*Jinv22-4.2918019480519480e-04*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+2.4655032467532466e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-2.7394480519480520e-03*Jinv02*w33-4.7483766233766234e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+2.7394480519480518e-04*w25*Jinv10-4.7483766233766234e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.1505681818181819e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-3.1960227272727273e-04*Jinv22*w29+1.9176136363636362e-04*Jinv22*w14+4.3831168831168834e-04*Jinv02*w36-2.5202922077922079e-03*Jinv12*w43-1.6436688311688311e-04*w5*Jinv20-2.0271915584415584e-03*Jinv02*w51-1.0135957792207792e-03*Jinv22*w56-3.1960227272727273e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-7.1225649350649346e-04*w16*Jinv12-4.8396915584415583e-04*Jinv12*w1)*D;
  A[32][54]=(0.0)*D;
  A[32][55]=(0.0)*D;
  A[32][56]=(-2.7394480519480520e-03*w40*Jinv12-1.0135957792207792e-03*Jinv22*w53+1.1870941558441559e-04*w57*Jinv02-2.7394480519480520e-03*Jinv02*w39-1.3149350649350649e-03*Jinv21*w44-1.6436688311688311e-04*Jinv01*w3+8.4922889610389610e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59+2.4655032467532466e-04*w25*Jinv12-4.8396915584415583e-04*w0*Jinv02-4.7483766233766234e-04*Jinv21*w2-1.3149350649350649e-03*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-2.0271915584415584e-03*Jinv12*w55-7.1225649350649346e-04*w15*Jinv02-3.1960227272727273e-04*w10*Jinv12+2.4655032467532466e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-4.7483766233766234e-04*w0*Jinv01+4.6570616883116886e-04*Jinv22*w20-1.3149350649350649e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-2.5202922077922079e-03*w42*Jinv02-3.1960227272727273e-04*w11*Jinv22-4.7483766233766234e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11-7.1225649350649346e-04*Jinv22*w17+1.3697240259740259e-04*w22*Jinv12+1.9176136363636362e-04*Jinv02*w3+1.4793019480519481e-03*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-4.8396915584415583e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45+2.7394480519480518e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11+2.7394480519480518e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-2.5202922077922079e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv11+3.8352272727272725e-04*Jinv12*w13+1.1505681818181819e-03*w32*Jinv21+2.7394480519480518e-04*Jinv21*w20+8.4922889610389610e-04*w37*Jinv12-2.7394480519480520e-03*Jinv22*w41-1.0135957792207792e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+1.1505681818181819e-03*w31*Jinv11+2.4655032467532466e-04*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11-1.6436688311688311e-04*w5*Jinv21-2.0271915584415584e-03*Jinv02*w54+1.9176136363636362e-04*w4*Jinv12+1.3697240259740259e-04*w21*Jinv02-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+3.8352272727272725e-04*Jinv02*w12+2.7394480519480518e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-3.1960227272727273e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+1.3971185064935066e-03*w47*Jinv21+1.3697240259740259e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+2.4655032467532466e-04*Jinv22*w8-1.2327516233766234e-03*Jinv22*w50+5.2049512987012989e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7+5.2049512987012989e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-2.3833198051948050e-03*Jinv21*w56-1.7349837662337662e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+4.6570616883116886e-04*w19*Jinv12+1.9176136363636362e-04*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14+1.1870941558441559e-04*w58*Jinv12+4.6570616883116886e-04*Jinv02*w18-1.6436688311688311e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33+5.2049512987012989e-04*Jinv21*w23-2.3833198051948050e-03*Jinv01*w54-2.3833198051948050e-03*w55*Jinv11-1.2327516233766234e-03*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9-1.7349837662337662e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+8.4922889610389610e-04*Jinv02*w36-2.5202922077922079e-03*Jinv12*w43+1.1505681818181819e-03*Jinv01*w30-1.0135957792207792e-03*Jinv02*w51-2.0271915584415584e-03*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28+2.7394480519480518e-04*w25*Jinv11-7.1225649350649346e-04*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-4.8396915584415583e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[32][57]=(0.0)*D;
  A[32][58]=(0.0)*D;
  A[32][59]=(-3.2873376623376621e-04*w40*Jinv12-6.7573051948051951e-04*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02-3.2873376623376621e-04*Jinv02*w39+4.9310064935064932e-04*Jinv22*w38-2.7394480519480521e-05*w25*Jinv12+8.3198051948051951e-05*w0*Jinv02-6.7573051948051951e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12-2.7394480519480521e-05*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35-2.7394480519480521e-05*Jinv22*w20+2.9220779220779221e-04*w42*Jinv02-1.1668019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17+1.1870941558441559e-04*w22*Jinv12+3.6525974025974026e-05*Jinv02*w3-2.5568181818181820e-04*w32*Jinv22+8.3198051948051951e-05*Jinv22*w2+4.9310064935064932e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59-3.2873376623376621e-04*Jinv12*w34+2.9220779220779221e-04*Jinv22*w44+3.6525974025974026e-05*Jinv12*w13+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv22*w41-6.7573051948051951e-04*w52*Jinv12-2.7394480519480521e-05*Jinv22*w26-6.7573051948051951e-04*Jinv02*w54+3.6525974025974026e-05*w4*Jinv12+1.1870941558441559e-04*w21*Jinv02+3.6525974025974026e-05*Jinv02*w12-1.1668019480519480e-04*Jinv02*w9+1.1870941558441559e-04*Jinv22*w23-2.5568181818181820e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+1.1870941558441559e-04*Jinv22*w8-1.0227272727272728e-03*Jinv22*w50+1.1870941558441559e-04*Jinv12*w7-2.5568181818181820e-04*Jinv02*w30-1.0227272727272728e-03*Jinv12*w49-1.1668019480519480e-04*Jinv02*w27+1.1870941558441559e-04*w6*Jinv02-2.7394480519480521e-05*w19*Jinv12+3.6525974025974026e-05*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12-2.7394480519480521e-05*Jinv02*w18-3.2873376623376621e-04*Jinv02*w33-1.0227272727272728e-03*w48*Jinv02-1.1668019480519480e-04*Jinv22*w29+3.6525974025974026e-05*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36+2.9220779220779221e-04*Jinv12*w43-6.7573051948051951e-04*Jinv02*w51-6.7573051948051951e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12+8.3198051948051951e-05*Jinv12*w1)*D;
  A[33][0]=(-3.2873376623376621e-04*Jinv00*w12+1.9480519480519481e-04*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12-4.3831168831168834e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-6.5746753246753243e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.9220779220779221e-04*Jinv20*w23-4.3831168831168834e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-3.4699675324675324e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17-3.4699675324675324e-04*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+1.8262987012987013e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00-3.4699675324675324e-04*Jinv02*w24+3.6525974025974025e-03*Jinv22*w35+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01-3.4699675324675324e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+1.8262987012987013e-03*w16*Jinv10-2.1915584415584417e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-3.4699675324675324e-04*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+1.9480519480519481e-04*Jinv00*w27+4.2613636363636367e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01+1.9480519480519481e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+1.9480519480519481e-04*Jinv20*w29-3.4699675324675324e-04*w46*Jinv10+1.8262987012987013e-03*Jinv22*w17+3.6525974025974025e-03*Jinv00*w33-2.9220779220779221e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3-2.9220779220779221e-04*w21*Jinv00-6.5746753246753243e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18-4.3831168831168834e-04*w36*Jinv00-4.3831168831168834e-04*Jinv21*w38-3.4699675324675324e-04*Jinv02*w45-3.4699675324675324e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+3.6525974025974025e-03*w34*Jinv11-2.1915584415584417e-04*w19*Jinv11-4.3831168831168834e-04*Jinv21*w53-4.3831168831168834e-04*w51*Jinv00+3.6525974025974025e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-6.5746753246753243e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11+1.8262987012987013e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10+3.6525974025974025e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21-2.1915584415584417e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00-4.3831168831168834e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-3.4699675324675324e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20-4.3831168831168834e-04*w52*Jinv12+3.6525974025974025e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-3.4699675324675324e-04*Jinv22*w26-3.4699675324675324e-04*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12-2.9220779220779221e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+3.6525974025974025e-03*Jinv01*w33+1.8262987012987013e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12-2.1915584415584417e-04*Jinv01*w18-4.3831168831168834e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-3.4699675324675324e-04*w47*Jinv21-4.3831168831168834e-04*Jinv20*w38-2.9220779220779221e-04*Jinv22*w23-4.3831168831168834e-04*w52*Jinv11-6.5746753246753243e-04*w31*Jinv12-3.4699675324675324e-04*w46*Jinv12-4.3831168831168834e-04*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8-2.9220779220779221e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50-2.9220779220779221e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7-2.9220779220779221e-04*w21*Jinv01-4.3831168831168834e-04*Jinv01*w51-6.5746753246753243e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.3831168831168834e-04*w36*Jinv01-4.3831168831168834e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-4.3831168831168834e-04*w37*Jinv10-2.1915584415584417e-04*Jinv21*w56-3.4699675324675324e-04*w24*Jinv00+1.9480519480519481e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5-3.4699675324675324e-04*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+3.6525974025974025e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+3.6525974025974025e-03*w35*Jinv20-2.9220779220779221e-04*Jinv21*w23-2.1915584415584417e-04*Jinv01*w54-2.1915584415584417e-04*w55*Jinv11-3.4699675324675324e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+1.8262987012987013e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+1.8262987012987013e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+1.8262987012987013e-03*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+1.9480519480519481e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11-4.3831168831168834e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-6.5746753246753243e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30-4.3831168831168834e-04*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56+1.9480519480519481e-04*Jinv12*w28-3.4699675324675324e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-3.4699675324675324e-04*w47*Jinv20+1.8262987012987013e-03*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[33][1]=(0.0)*D;
  A[33][2]=(0.0)*D;
  A[33][3]=(5.4788961038961036e-04*Jinv00*w12-6.9399350649350649e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-1.0592532467532468e-03*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00+1.0957792207792207e-03*w32*Jinv20-4.7666396103896100e-03*Jinv01*w3+1.2053571428571428e-03*Jinv20*w23+1.6436688311688311e-03*Jinv22*w38+3.2873376623376621e-04*Jinv00*w54-1.0592532467532468e-03*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-6.9034090909090911e-03*w16*Jinv11-6.9034090909090911e-03*Jinv21*w17+8.7662337662337668e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.6615259740259744e-04*w58*Jinv10-6.9034090909090911e-03*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+6.0267857142857139e-04*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-1.3806818181818182e-02*Jinv22*w35-1.0592532467532468e-03*w58*Jinv11-1.0592532467532468e-03*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-6.9399350649350649e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-6.9034090909090911e-03*w15*Jinv01-9.6793831168831166e-04*w28*Jinv11+6.0267857142857139e-04*Jinv20*w50-6.9399350649350649e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-6.9034090909090911e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33+2.5202922077922079e-03*w22*Jinv12+1.0957792207792207e-03*w31*Jinv10-4.7666396103896100e-03*Jinv02*w3+1.2053571428571428e-03*w21*Jinv00-3.1777597402597403e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+4.3831168831168834e-04*Jinv00*w18+8.7662337662337668e-04*w36*Jinv00+1.6436688311688311e-03*Jinv21*w38+8.7662337662337668e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24-1.0592532467532468e-03*Jinv22*w59-1.3806818181818182e-02*w34*Jinv11+8.2183441558441553e-04*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00-1.3806818181818182e-02*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-4.7666396103896100e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+4.3831168831168834e-04*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10-3.1777597402597403e-03*w32*Jinv21+8.2183441558441553e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00+1.6436688311688311e-03*w37*Jinv12+3.2873376623376621e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+1.2601461038961039e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-1.3806818181818182e-02*Jinv21*w35-3.1777597402597403e-03*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+8.7662337662337668e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14-4.7666396103896100e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-4.7666396103896100e-03*w4*Jinv12+2.5202922077922079e-03*w21*Jinv02+6.0267857142857139e-04*w49*Jinv10-9.6793831168831166e-04*Jinv21*w29-1.3806818181818182e-02*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+8.2183441558441553e-04*Jinv01*w18+1.6436688311688311e-03*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+6.5746753246753243e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+8.7662337662337668e-04*w47*Jinv21+8.7662337662337668e-04*Jinv20*w38+2.5202922077922079e-03*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-3.1777597402597403e-03*w31*Jinv12+8.7662337662337668e-04*w46*Jinv12+1.3697240259740260e-03*Jinv20*w53+6.5746753246753243e-04*Jinv22*w8+1.2053571428571428e-03*w22*Jinv10+2.7394480519480520e-03*Jinv22*w50+2.5202922077922079e-03*w22*Jinv11+6.5746753246753243e-04*Jinv12*w7+2.5202922077922079e-03*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-3.1777597402597403e-03*Jinv02*w30+2.7394480519480520e-03*Jinv12*w49+1.6436688311688311e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10+6.5746753246753243e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+8.7662337662337668e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+1.2601461038961039e-03*w24*Jinv00-9.6793831168831166e-04*Jinv02*w27+6.5746753246753243e-04*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12-4.7666396103896100e-03*Jinv22*w5+8.7662337662337668e-04*w47*Jinv22+2.7394480519480520e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-1.0592532467532468e-03*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+6.5746753246753243e-04*Jinv21*w8-5.6615259740259744e-04*w57*Jinv00-5.6615259740259744e-04*Jinv20*w59-1.3806818181818182e-02*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20+2.5202922077922079e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+1.2601461038961039e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+2.7394480519480520e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17+1.0957792207792207e-03*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10+4.3831168831168834e-04*w20*Jinv20-9.6793831168831166e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+1.6436688311688311e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-3.1777597402597403e-03*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-9.6793831168831166e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11+3.2873376623376621e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-6.9034090909090911e-03*w16*Jinv12+2.7394480519480520e-03*Jinv21*w50-9.6793831168831166e-04*Jinv01*w27+2.7394480519480520e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[33][4]=(0.0)*D;
  A[33][5]=(0.0)*D;
  A[33][6]=(-4.3831168831168834e-04*Jinv00*w12+6.9399350649350649e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-4.3831168831168834e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00-8.7662337662337668e-04*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3-1.2601461038961039e-03*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38-3.2873376623376621e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+3.8352272727272725e-04*Jinv12*w55+5.6615259740259744e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+6.9399350649350649e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+6.9399350649350649e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-8.7662337662337668e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3-1.2601461038961039e-03*w21*Jinv00+7.6704545454545450e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-1.0957792207792207e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53-6.0267857142857139e-04*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10+2.1367694805194807e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+3.8352272727272725e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12-3.2873376623376621e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.2053571428571428e-03*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-4.3831168831168834e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+3.8352272727272725e-04*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-1.0957792207792207e-03*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+7.6704545454545450e-04*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-6.0267857142857139e-04*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-1.2601461038961039e-03*w22*Jinv10+7.6704545454545450e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+7.6704545454545450e-04*Jinv02*w30+7.6704545454545450e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01-6.0267857142857139e-04*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00-1.0957792207792207e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-1.2053571428571428e-03*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+7.6704545454545450e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8+5.6615259740259744e-04*w57*Jinv00+5.6615259740259744e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-1.2053571428571428e-03*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+7.6704545454545450e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+5.9172077922077923e-03*Jinv20*w17-8.7662337662337668e-04*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+7.6704545454545450e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11-3.2873376623376621e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[33][7]=(0.0)*D;
  A[33][8]=(0.0)*D;
  A[33][9]=(2.1915584415584417e-04*Jinv00*w12-1.9480519480519481e-04*w28*Jinv10+6.5746753246753243e-04*Jinv20*w8+2.1915584415584417e-04*w13*Jinv10-3.2873376623376621e-04*w3*Jinv00+4.3831168831168834e-04*w32*Jinv20+3.4699675324675324e-04*Jinv20*w23+2.1915584415584417e-04*Jinv00*w54-1.8262987012987013e-03*w43*Jinv10-4.2613636363636367e-05*w58*Jinv10+4.3831168831168834e-04*w48*Jinv00-4.2613636363636367e-05*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10+3.4699675324675324e-04*w45*Jinv00-1.9480519480519481e-04*Jinv00*w27+4.3831168831168834e-04*Jinv20*w50-1.9480519480519481e-04*Jinv20*w29+3.4699675324675324e-04*w46*Jinv10-3.6525974025974025e-03*Jinv00*w33+4.3831168831168834e-04*w31*Jinv10+3.4699675324675324e-04*w21*Jinv00+3.2873376623376621e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-3.2873376623376621e-04*w51*Jinv00-3.2873376623376621e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+3.2873376623376621e-04*w19*Jinv10-3.6525974025974025e-03*w34*Jinv10+6.5746753246753243e-04*w6*Jinv00+2.1915584415584417e-04*Jinv20*w56+2.9220779220779221e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+2.1915584415584417e-04*Jinv20*w14+4.3831168831168834e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+6.5746753246753243e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-3.2873376623376621e-04*Jinv20*w53+3.4699675324675324e-04*w22*Jinv10-3.2873376623376621e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10+2.9220779220779221e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-1.8262987012987013e-03*Jinv20*w44-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-4.2613636363636367e-05*w0*Jinv00-3.6525974025974025e-03*w35*Jinv20+2.9220779220779221e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00-1.8262987012987013e-03*Jinv20*w17+4.3831168831168834e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-3.2873376623376621e-04*w5*Jinv20+2.1915584415584417e-04*w55*Jinv10-1.8262987012987013e-03*w42*Jinv00+3.4699675324675324e-04*w47*Jinv20)*D;
  A[33][10]=(0.0)*D;
  A[33][11]=(0.0)*D;
  A[33][12]=(-1.5888798701298701e-03*Jinv00*w12-5.2962662337662338e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-5.2962662337662338e-04*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20+2.6846590909090908e-03*Jinv01*w3+1.3697240259740260e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02-1.6436688311688311e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-3.6160714285714286e-03*w16*Jinv11-3.6160714285714286e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-4.6022727272727274e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12+1.3697240259740260e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.6436688311688311e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-3.6160714285714286e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22-1.6436688311688311e-04*Jinv11*w1+5.4788961038961036e-04*w13*Jinv11-3.6160714285714286e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+1.3697240259740260e-03*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-4.6022727272727274e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+1.3697240259740260e-03*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3+1.3697240259740260e-03*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-3.6160714285714286e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+6.5746753246753243e-04*Jinv01*w24-5.2962662337662338e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+3.2873376623376621e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10+2.6846590909090908e-03*w4*Jinv11-4.6022727272727274e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+2.6846590909090908e-03*w32*Jinv21+3.2873376623376621e-04*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-5.2962662337662338e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+2.6846590909090908e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+6.5746753246753243e-04*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14+2.6846590909090908e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+1.3697240259740260e-03*w21*Jinv02+1.3697240259740260e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-4.6022727272727274e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+3.2873376623376621e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+6.5746753246753243e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+1.3697240259740260e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+1.3697240259740260e-03*Jinv22*w8+1.3697240259740260e-03*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11-5.2962662337662338e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+3.2873376623376621e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+1.3697240259740260e-03*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11+5.4788961038961036e-04*Jinv21*w14-5.2962662337662338e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-5.2962662337662338e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+5.4788961038961036e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.2873376623376621e-04*Jinv01*w54+3.2873376623376621e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-4.6022727272727274e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-3.6160714285714286e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20+2.6846590909090908e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+6.5746753246753243e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-4.6022727272727274e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[33][13]=(0.0)*D;
  A[33][14]=(0.0)*D;
  A[33][15]=(2.3011363636363637e-03*Jinv00*w12-7.8896103896103891e-03*w40*Jinv12+1.9724025974025973e-03*Jinv22*w53-2.9586038961038961e-03*Jinv20*w8-1.1323051948051949e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10-7.8896103896103891e-03*Jinv02*w39+2.9586038961038961e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3-6.5746753246753243e-04*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38+1.1323051948051949e-03*Jinv21*w59+1.9724025974025973e-03*w25*Jinv12-1.1323051948051949e-03*w0*Jinv02-6.9399350649350649e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+2.1915584415584417e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17-1.9724025974025973e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-1.1834415584415585e-02*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-6.5746753246753243e-04*w48*Jinv00+1.9724025974025973e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35+1.1323051948051949e-03*w58*Jinv11+1.1323051948051949e-03*w57*Jinv01-2.9586038961038961e-03*Jinv21*w26-6.9399350649350649e-04*w0*Jinv01-9.1314935064935063e-04*w1*Jinv10-9.8620129870129864e-04*Jinv22*w20+3.9448051948051946e-03*Jinv21*w41-1.1834415584415585e-02*w42*Jinv02-2.1915584415584417e-04*w11*Jinv22-6.9399350649350649e-04*Jinv11*w1+2.3011363636363637e-03*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01+1.6436688311688311e-03*w28*Jinv11-6.5746753246753243e-04*Jinv20*w50-1.1834415584415585e-02*Jinv22*w17+1.3149350649350649e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+1.9724025974025973e-03*Jinv02*w3-6.5746753246753243e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22+3.9448051948051946e-03*Jinv01*w39-1.1323051948051949e-03*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00+9.8620129870129864e-04*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45-2.9586038961038961e-03*Jinv01*w24-1.1323051948051949e-03*Jinv22*w59+2.3668831168831169e-02*w34*Jinv11-1.9724025974025973e-03*Jinv21*w53+6.5746753246753243e-04*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-1.1834415584415585e-02*Jinv22*w44+2.9586038961038961e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+3.2873376623376621e-03*w32*Jinv21-2.9586038961038961e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12-7.8896103896103891e-03*Jinv22*w41+6.5746753246753243e-04*w26*Jinv20+9.1314935064935063e-04*w11*Jinv20+1.9724025974025973e-03*w52*Jinv12+2.3668831168831169e-02*Jinv21*w35+3.2873376623376621e-03*w31*Jinv11+1.9724025974025973e-03*Jinv22*w26-1.9724025974025973e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+1.9724025974025973e-03*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02-6.5746753246753243e-04*w49*Jinv10+1.6436688311688311e-03*Jinv21*w29+2.3668831168831169e-02*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+9.8620129870129864e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+1.9724025974025973e-03*w6*Jinv01-2.9586038961038961e-03*w7*Jinv10-1.9724025974025973e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-1.9724025974025973e-03*w52*Jinv11+1.3149350649350649e-03*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12+6.5746753246753243e-04*Jinv20*w53-9.8620129870129864e-04*Jinv22*w8-6.5746753246753243e-04*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-3.6160714285714286e-03*w22*Jinv11-9.8620129870129864e-04*Jinv12*w7-3.6160714285714286e-03*w21*Jinv01-1.9724025974025973e-03*Jinv01*w51+1.3149350649350649e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49+9.8620129870129864e-04*w36*Jinv01+6.5746753246753243e-04*w52*Jinv10+1.9724025974025973e-03*w7*Jinv11+9.1314935064935063e-04*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+6.5746753246753243e-04*w24*Jinv00-1.1323051948051949e-03*Jinv02*w27-9.8620129870129864e-04*w6*Jinv02-9.8620129870129864e-04*w19*Jinv12+1.9724025974025973e-03*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11+2.3011363636363637e-03*Jinv21*w14+9.1314935064935063e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44-1.1323051948051949e-03*w58*Jinv12-9.8620129870129864e-04*Jinv02*w18+2.3011363636363637e-03*Jinv01*w12+1.9724025974025973e-03*Jinv21*w8-1.1834415584415585e-02*Jinv02*w33-9.1314935064935063e-04*w0*Jinv00-3.6160714285714286e-03*Jinv21*w23+6.5746753246753243e-04*w25*Jinv10-9.1314935064935063e-04*Jinv20*w2+1.3149350649350649e-03*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+2.3011363636363637e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20-1.1323051948051949e-03*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14+3.9448051948051946e-03*w40*Jinv11-9.8620129870129864e-04*Jinv02*w36-1.1834415584415585e-02*Jinv12*w43+2.9586038961038961e-03*w5*Jinv20+3.2873376623376621e-03*Jinv01*w30+1.9724025974025973e-03*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56-1.1323051948051949e-03*Jinv12*w28-2.9586038961038961e-03*w25*Jinv11-3.9448051948051946e-03*w42*Jinv00-1.1834415584415585e-02*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50+1.6436688311688311e-03*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-1.1323051948051949e-03*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[33][16]=(0.0)*D;
  A[33][17]=(0.0)*D;
  A[33][18]=(-7.1225649350649346e-04*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+1.5888798701298701e-03*Jinv20*w8-7.1225649350649346e-04*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23-7.1225649350649346e-04*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59+4.9310064935064932e-04*Jinv21*w2+4.6022727272727274e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+5.2962662337662338e-04*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+5.2962662337662338e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+5.2962662337662338e-04*Jinv00*w27+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+1.5888798701298701e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-1.3149350649350649e-03*Jinv01*w24+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-1.3697240259740260e-03*w51*Jinv00-1.3697240259740260e-03*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.5888798701298701e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.5888798701298701e-03*w6*Jinv00-7.1225649350649346e-04*Jinv20*w56-1.3697240259740260e-03*w26*Jinv20-3.8352272727272725e-04*w11*Jinv20+1.9724025974025973e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+2.1367694805194807e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11+4.2735389610389614e-03*w6*Jinv01+1.5888798701298701e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.5888798701298701e-03*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-1.3697240259740260e-03*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+4.2735389610389614e-03*w36*Jinv01-1.3697240259740260e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56-1.3697240259740260e-03*w24*Jinv00-7.6704545454545450e-04*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10+4.6022727272727274e-03*Jinv20*w44-3.8352272727272725e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+5.2962662337662338e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11-1.3697240259740260e-03*w25*Jinv10+5.2962662337662338e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+4.6022727272727274e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-1.3697240259740260e-03*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30-1.3149350649350649e-03*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10+4.6022727272727274e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20-7.6704545454545450e-04*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[33][19]=(0.0)*D;
  A[33][20]=(0.0)*D;
  A[33][21]=(3.8352272727272725e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02+3.8352272727272725e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-2.5750811688311690e-03*Jinv01*w3-1.0957792207792207e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+3.8352272727272725e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+3.8352272727272725e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+3.2873376623376621e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+2.5568181818181820e-04*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+3.8352272727272725e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01-1.8262987012987014e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+3.2873376623376621e-03*Jinv00*w33-1.0957792207792207e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.0957792207792207e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45+3.6525974025974026e-05*Jinv22*w59-6.5746753246753243e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+3.2873376623376621e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-2.5750811688311690e-03*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+3.8352272727272725e-04*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10+3.2873376623376621e-03*w34*Jinv10-2.5750811688311690e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+3.8352272727272725e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-6.5746753246753243e-03*Jinv21*w35-2.5750811688311690e-03*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26+3.8352272727272725e-04*Jinv20*w14-2.5750811688311690e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-1.0957792207792207e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-1.8262987012987014e-04*Jinv21*w29-6.5746753246753243e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+3.8352272727272725e-04*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+5.4788961038961042e-05*Jinv20*w38-1.0957792207792207e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-1.0957792207792207e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+5.4788961038961036e-04*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+5.4788961038961036e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-3.2873376623376621e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+3.8352272727272725e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+3.2873376623376621e-03*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+3.2873376623376621e-03*w35*Jinv20+5.4788961038961036e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-2.5750811688311690e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28+3.8352272727272725e-04*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-1.8262987012987014e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[33][22]=(0.0)*D;
  A[33][23]=(0.0)*D;
  A[33][24]=(-3.8352272727272725e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-3.8352272727272725e-04*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23-3.8352272727272725e-04*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59-7.3051948051948052e-05*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17+1.0957792207792207e-03*Jinv01*w45-3.6525974025974026e-05*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+1.6436688311688311e-03*Jinv21*w26-7.3051948051948052e-05*w0*Jinv01-3.6525974025974026e-05*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41-4.9310064935064932e-04*Jinv00*w27-7.3051948051948052e-05*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01-6.7573051948051951e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-3.2873376623376621e-03*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10+1.0957792207792207e-03*w21*Jinv00+6.5746753246753243e-04*Jinv01*w39-3.8352272727272725e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+1.6436688311688311e-03*Jinv01*w24-9.8620129870129868e-03*w34*Jinv11-8.2183441558441553e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-5.4788961038961042e-05*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20-3.8352272727272725e-04*w19*Jinv10-3.2873376623376621e-03*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21-8.2183441558441553e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*Jinv20*w56+1.0957792207792207e-03*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20-9.8620129870129868e-03*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+1.0957792207792207e-03*w46*Jinv11-3.8352272727272725e-04*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21-5.4788961038961042e-05*w49*Jinv10-6.7573051948051951e-04*Jinv21*w29-9.8620129870129868e-03*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00-8.2183441558441553e-04*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.0957792207792207e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11-5.4788961038961042e-05*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10+1.0957792207792207e-03*w22*Jinv11+1.0957792207792207e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-2.6298701298701297e-03*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11-3.6525974025974026e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56+1.0957792207792207e-03*w24*Jinv00-1.0957792207792208e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-3.6525974025974026e-05*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-3.6525974025974026e-05*w0*Jinv00-3.2873376623376621e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11+1.0957792207792207e-03*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+6.5746753246753243e-04*w40*Jinv11-5.4788961038961042e-05*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30+1.6436688311688311e-03*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20-1.0957792207792208e-04*Jinv21*w50-6.7573051948051951e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[33][25]=(0.0)*D;
  A[33][26]=(0.0)*D;
  A[33][27]=(-5.1136363636363640e-04*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3+3.0438311688311688e-05*Jinv21*w59+3.0438311688311688e-05*Jinv21*w2-5.1136363636363640e-04*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21-5.1136363636363640e-04*w16*Jinv11-5.1136363636363640e-04*Jinv21*w17-3.1047077922077924e-04*Jinv01*w45+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-3.1047077922077924e-04*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01-5.1136363636363640e-04*w43*Jinv11-5.1136363636363640e-04*Jinv21*w41+3.0438311688311688e-05*Jinv11*w1+2.1915584415584417e-04*w13*Jinv11-5.1136363636363640e-04*w15*Jinv01+2.4350649350649351e-04*w28*Jinv11-5.1136363636363640e-04*Jinv01*w39+5.4788961038961036e-04*Jinv21*w38-3.1047077922077924e-04*Jinv01*w24+5.5519480519480519e-03*w34*Jinv11+2.1915584415584417e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+5.4788961038961036e-04*w4*Jinv11+5.4788961038961036e-04*w32*Jinv21+2.1915584415584417e-04*Jinv21*w20+5.5519480519480519e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-3.1047077922077924e-04*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21+2.4350649350649351e-04*Jinv21*w29+5.5519480519480519e-03*Jinv01*w33+2.1915584415584417e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01-3.1047077922077924e-04*w47*Jinv21+5.4788961038961036e-04*w52*Jinv11-3.1047077922077924e-04*w22*Jinv11-3.1047077922077924e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+5.4788961038961036e-04*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11+2.1915584415584417e-04*Jinv21*w56+5.4788961038961036e-04*w49*Jinv11+2.1915584415584417e-04*Jinv21*w14+2.1915584415584417e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-3.1047077922077924e-04*Jinv21*w23+2.1915584415584417e-04*Jinv01*w54+2.1915584415584417e-04*w55*Jinv11+3.0438311688311688e-05*Jinv01*w9-5.1136363636363640e-04*w40*Jinv11+5.4788961038961036e-04*Jinv01*w30-3.1047077922077924e-04*w25*Jinv11+5.4788961038961036e-04*Jinv21*w50+2.4350649350649351e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+3.0438311688311688e-05*w10*Jinv11)*D;
  A[33][28]=(0.0)*D;
  A[33][29]=(0.0)*D;
  A[33][30]=(-1.5888798701298701e-03*Jinv00*w12-9.6793831168831166e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12+8.7662337662337668e-04*Jinv22*w53+2.7394480519480520e-03*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-3.1777597402597403e-03*w3*Jinv00-4.7666396103896100e-03*w32*Jinv20-3.1777597402597403e-03*Jinv01*w3+2.5202922077922079e-03*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-1.0592532467532468e-03*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45+4.3831168831168834e-04*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-5.6615259740259744e-04*w10*Jinv12+6.5746753246753243e-04*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+8.7662337662337668e-04*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-6.9034090909090911e-03*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-9.6793831168831166e-04*Jinv00*w27-5.6615259740259744e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-9.6793831168831166e-04*w28*Jinv11+6.5746753246753243e-04*Jinv20*w50-9.6793831168831166e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-1.3806818181818182e-02*Jinv00*w33+1.2053571428571428e-03*w22*Jinv12-4.7666396103896100e-03*w31*Jinv10+1.0957792207792207e-03*Jinv02*w3+2.5202922077922079e-03*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22-6.9034090909090911e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.2601461038961039e-03*Jinv02*w45+8.7662337662337668e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-1.3806818181818182e-02*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-03*Jinv21*w53+1.6436688311688311e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-3.1777597402597403e-03*w4*Jinv10-3.1777597402597403e-03*w4*Jinv11-6.9034090909090911e-03*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-1.3806818181818182e-02*w34*Jinv10-4.7666396103896100e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+2.7394480519480520e-03*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41+8.7662337662337668e-04*w26*Jinv20-1.0592532467532468e-03*w11*Jinv20+8.7662337662337668e-04*w52*Jinv12-1.3806818181818182e-02*Jinv21*w35-4.7666396103896100e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14-3.1777597402597403e-03*w5*Jinv21+4.3831168831168834e-04*Jinv02*w54+1.0957792207792207e-03*w4*Jinv12+1.2053571428571428e-03*w21*Jinv02+6.5746753246753243e-04*w49*Jinv10-9.6793831168831166e-04*Jinv21*w29-1.3806818181818182e-02*Jinv01*w33-6.9034090909090911e-03*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.6615259740259744e-04*Jinv02*w9+2.7394480519480520e-03*w6*Jinv01+2.7394480519480520e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38+1.2053571428571428e-03*Jinv22*w23+1.6436688311688311e-03*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+1.2601461038961039e-03*w46*Jinv12+1.6436688311688311e-03*Jinv20*w53+6.0267857142857139e-04*Jinv22*w8+2.5202922077922079e-03*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+2.5202922077922079e-03*w22*Jinv11+6.0267857142857139e-04*Jinv12*w7+2.5202922077922079e-03*w21*Jinv01+1.6436688311688311e-03*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+1.6436688311688311e-03*w52*Jinv10+2.7394480519480520e-03*w7*Jinv11-1.0592532467532468e-03*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10+8.2183441558441553e-04*Jinv21*w56+8.7662337662337668e-04*w24*Jinv00-6.9399350649350649e-04*Jinv02*w27+6.0267857142857139e-04*w6*Jinv02+3.2873376623376621e-04*w19*Jinv12+1.0957792207792207e-03*Jinv22*w5+1.2601461038961039e-03*w47*Jinv22+6.5746753246753243e-04*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-1.0592532467532468e-03*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+3.2873376623376621e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+2.7394480519480520e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-1.1834415584415585e-02*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-1.3806818181818182e-02*w35*Jinv20+2.5202922077922079e-03*Jinv21*w23+8.2183441558441553e-04*Jinv01*w54+8.2183441558441553e-04*w55*Jinv11+8.7662337662337668e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-1.0592532467532468e-03*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-4.7666396103896100e-03*w30*Jinv00-6.9034090909090911e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-6.9399350649350649e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-6.9034090909090911e-03*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-3.1777597402597403e-03*w5*Jinv20-4.7666396103896100e-03*Jinv01*w30+8.7662337662337668e-04*Jinv02*w51+4.3831168831168834e-04*Jinv22*w56-6.9399350649350649e-04*Jinv12*w28+8.7662337662337668e-04*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+6.5746753246753243e-04*Jinv21*w50-9.6793831168831166e-04*Jinv01*w27+6.5746753246753243e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-1.0592532467532468e-03*w10*Jinv11)*D;
  A[33][31]=(0.0)*D;
  A[33][32]=(0.0)*D;
  A[33][33]=(2.3011363636363637e-03*Jinv00*w12-6.9034090909090911e-03*Jinv22*w53-5.9172077922077923e-03*Jinv20*w8+1.8262987012987013e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10-1.7751623376623376e-02*Jinv21*w44+5.9172077922077923e-03*w3*Jinv00+6.9034090909090911e-03*w32*Jinv20+3.9448051948051946e-03*Jinv01*w3-1.6436688311688311e-03*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38-4.3831168831168834e-04*Jinv21*w59-1.8262987012987013e-03*w0*Jinv02-2.2646103896103898e-03*Jinv21*w2-5.9172077922077923e-03*w43*Jinv10-1.7751623376623376e-02*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+4.9310064935064934e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55+5.9172077922077923e-03*w15*Jinv02+9.8620129870129864e-04*w48*Jinv00-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+4.9310064935064934e-03*Jinv21*w26-2.2646103896103898e-03*w0*Jinv01-1.8262987012987013e-03*w1*Jinv10-1.7751623376623376e-02*w43*Jinv11-1.1834415584415585e-02*Jinv21*w41-5.9172077922077923e-03*w42*Jinv02-2.2646103896103898e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-2.7759740259740259e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+5.9172077922077923e-03*Jinv22*w17-1.6436688311688311e-03*w22*Jinv12+6.9034090909090911e-03*w31*Jinv10+6.9034090909090911e-03*Jinv02*w3-1.6436688311688311e-03*w21*Jinv00+5.9172077922077923e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-1.8262987012987013e-03*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-6.9034090909090911e-03*w36*Jinv00-2.9586038961038961e-03*Jinv21*w38+1.6436688311688311e-03*Jinv02*w45+4.9310064935064934e-03*Jinv01*w24+1.8262987012987013e-03*Jinv22*w59-3.5503246753246752e-02*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11-2.9586038961038961e-03*Jinv21*w53-9.8620129870129864e-04*w51*Jinv00-5.9172077922077923e-03*Jinv22*w44+5.9172077922077923e-03*w4*Jinv10+3.9448051948051946e-03*w4*Jinv11+5.9172077922077923e-03*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+3.9448051948051946e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-5.9172077922077923e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12+1.6436688311688311e-03*w26*Jinv20+1.8262987012987013e-03*w11*Jinv20-6.9034090909090911e-03*w52*Jinv12-3.5503246753246752e-02*Jinv21*w35+3.9448051948051946e-03*w31*Jinv11+4.9310064935064934e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+3.9448051948051946e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+6.9034090909090911e-03*w4*Jinv12-1.6436688311688311e-03*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10-2.7759740259740259e-03*Jinv21*w29-3.5503246753246752e-02*Jinv01*w33+5.9172077922077923e-03*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-2.9586038961038961e-03*w37*Jinv11-1.9724025974025973e-03*w6*Jinv01-5.9172077922077923e-03*w7*Jinv10+4.9310064935064934e-03*w47*Jinv21-6.9034090909090911e-03*Jinv20*w38-1.6436688311688311e-03*Jinv22*w23-2.9586038961038961e-03*w52*Jinv11+5.9172077922077923e-03*w31*Jinv12+1.6436688311688311e-03*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8-1.6436688311688311e-03*w22*Jinv10-5.9172077922077923e-03*Jinv22*w50+3.2873376623376621e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7+3.2873376623376621e-03*w21*Jinv01-2.9586038961038961e-03*Jinv01*w51+5.9172077922077923e-03*Jinv02*w30-5.9172077922077923e-03*Jinv12*w49-2.9586038961038961e-03*w36*Jinv01-9.8620129870129864e-04*w52*Jinv10-1.9724025974025973e-03*w7*Jinv11+1.8262987012987013e-03*w9*Jinv00-6.9034090909090911e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56+1.6436688311688311e-03*w24*Jinv00+9.8620129870129864e-04*w6*Jinv02+6.9034090909090911e-03*Jinv22*w5+1.6436688311688311e-03*w47*Jinv22-1.9724025974025973e-03*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+1.8262987012987013e-03*w10*Jinv10-5.9172077922077923e-03*Jinv20*w44+1.8262987012987013e-03*w58*Jinv12+1.3149350649350649e-03*Jinv01*w12-1.9724025974025973e-03*Jinv21*w8-1.8262987012987013e-03*w0*Jinv00+3.2873376623376621e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11+1.6436688311688311e-03*w25*Jinv10-1.8262987012987013e-03*Jinv20*w2-5.9172077922077923e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9+6.9034090909090911e-03*w30*Jinv00+5.9172077922077923e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11-9.8620129870129864e-04*Jinv02*w36-5.9172077922077923e-03*Jinv12*w43+5.9172077922077923e-03*w5*Jinv20+3.9448051948051946e-03*Jinv01*w30-6.9034090909090911e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56+4.9310064935064934e-03*w25*Jinv11-5.9172077922077923e-03*w42*Jinv00+5.9172077922077923e-03*w16*Jinv12-1.9724025974025973e-03*Jinv21*w50-2.7759740259740259e-03*Jinv01*w27-1.9724025974025973e-03*w48*Jinv01-1.8262987012987013e-03*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[33][34]=(0.0)*D;
  A[33][35]=(0.0)*D;
  A[33][36]=(-7.1225649350649346e-04*Jinv00*w12+9.6793831168831166e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+3.1777597402597403e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-2.7394480519480520e-03*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-8.7662337662337668e-04*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-1.3149350649350649e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+6.9034090909090911e-03*w43*Jinv10-3.8352272727272725e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-1.6436688311688311e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+1.0592532467532468e-03*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-1.3149350649350649e-03*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02+9.6793831168831166e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-1.6436688311688311e-03*Jinv20*w50+9.6793831168831166e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.3806818181818182e-02*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-8.7662337662337668e-04*w21*Jinv00-7.6704545454545450e-04*w32*Jinv22+4.9310064935064932e-04*Jinv22*w2+1.5888798701298701e-03*Jinv00*w18+4.7666396103896100e-03*w36*Jinv00-5.4788961038961042e-05*Jinv02*w45+4.9310064935064932e-04*Jinv22*w59-6.5746753246753243e-04*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-2.7394480519480520e-03*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-3.8352272727272725e-04*Jinv12*w13+1.5888798701298701e-03*w19*Jinv10+1.3806818181818182e-02*w34*Jinv10+3.1777597402597403e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-2.5202922077922079e-03*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12-1.3149350649350649e-03*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-3.8352272727272725e-04*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-1.6436688311688311e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-3.8352272727272725e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+3.1777597402597403e-03*w7*Jinv10+4.7666396103896100e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23-7.6704545454545450e-04*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-6.5746753246753243e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-8.7662337662337668e-04*w22*Jinv10-7.6704545454545450e-04*Jinv22*w50+4.2735389610389614e-03*Jinv12*w7-7.6704545454545450e-04*Jinv02*w30-7.6704545454545450e-04*Jinv12*w49-6.5746753246753243e-04*w52*Jinv10-7.6704545454545450e-04*w9*Jinv00+4.7666396103896100e-03*w37*Jinv10-2.5202922077922079e-03*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-7.6704545454545450e-04*w10*Jinv10+6.9034090909090911e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+1.0592532467532468e-03*w0*Jinv00+1.3806818181818182e-02*w35*Jinv20-2.5202922077922079e-03*w25*Jinv10+1.0592532467532468e-03*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-7.6704545454545450e-04*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+4.2735389610389614e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-2.7394480519480520e-03*w5*Jinv20-2.1367694805194807e-03*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28-8.2183441558441553e-04*w55*Jinv10+6.9034090909090911e-03*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+4.9310064935064932e-04*Jinv12*w1)*D;
  A[33][37]=(0.0)*D;
  A[33][38]=(0.0)*D;
  A[33][39]=(1.3149350649350649e-03*Jinv00*w12-1.1323051948051949e-03*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+9.1314935064935063e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10+1.3149350649350649e-03*w3*Jinv00+1.9724025974025973e-03*w32*Jinv20+3.2873376623376621e-03*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23+6.5746753246753243e-04*Jinv22*w38-9.8620129870129864e-04*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59-9.1314935064935063e-04*w0*Jinv02-6.9399350649350649e-04*Jinv21*w2-1.1834415584415585e-02*w43*Jinv10+1.1323051948051949e-03*w11*Jinv21+3.9448051948051946e-03*w16*Jinv11+3.9448051948051946e-03*Jinv21*w17-2.9586038961038961e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02-9.8620129870129864e-04*w48*Jinv00+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.9724025974025973e-03*Jinv21*w26-6.9399350649350649e-04*w0*Jinv01-1.1323051948051949e-03*w1*Jinv10-7.8896103896103891e-03*w16*Jinv10+1.9724025974025973e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02-1.1323051948051949e-03*Jinv00*w27-6.9399350649350649e-04*Jinv11*w1+2.3011363636363637e-03*w13*Jinv11+3.9448051948051946e-03*w15*Jinv01+1.6436688311688311e-03*w28*Jinv11-9.8620129870129864e-04*Jinv20*w50-1.1323051948051949e-03*Jinv20*w29+1.9724025974025973e-03*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33-6.5746753246753243e-04*w22*Jinv12+1.9724025974025973e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00+2.9586038961038961e-03*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-9.1314935064935063e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+1.9724025974025973e-03*w36*Jinv00-1.9724025974025973e-03*Jinv21*w38+6.5746753246753243e-04*Jinv02*w45-1.9724025974025973e-03*Jinv01*w24+9.1314935064935063e-04*Jinv22*w59+2.3668831168831169e-02*w34*Jinv11+9.8620129870129864e-04*Jinv21*w53-9.8620129870129864e-04*w51*Jinv00-3.9448051948051946e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+3.2873376623376621e-03*w4*Jinv11-1.1834415584415585e-02*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21+1.3149350649350649e-03*w6*Jinv00+6.5746753246753243e-04*w37*Jinv12-9.8620129870129864e-04*Jinv20*w56+1.3149350649350649e-03*w26*Jinv20-1.1323051948051949e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12+2.3668831168831169e-02*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11-2.9586038961038961e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14+3.2873376623376621e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12-6.5746753246753243e-04*w21*Jinv02-9.8620129870129864e-04*w49*Jinv10+1.6436688311688311e-03*Jinv21*w29+2.3668831168831169e-02*Jinv01*w33-1.1834415584415585e-02*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12-1.9724025974025973e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.9586038961038961e-03*w47*Jinv21+1.9724025974025973e-03*Jinv20*w38-6.5746753246753243e-04*Jinv22*w23+9.8620129870129864e-04*w52*Jinv11+2.9586038961038961e-03*w31*Jinv12+6.5746753246753243e-04*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53-6.5746753246753243e-04*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10-2.9586038961038961e-03*Jinv22*w50-3.6160714285714286e-03*w22*Jinv11-6.5746753246753243e-04*Jinv12*w7-3.6160714285714286e-03*w21*Jinv01+9.8620129870129864e-04*Jinv01*w51+2.9586038961038961e-03*Jinv02*w30-2.9586038961038961e-03*Jinv12*w49-1.9724025974025973e-03*w36*Jinv01-9.8620129870129864e-04*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11-1.1323051948051949e-03*w9*Jinv00+1.9724025974025973e-03*w37*Jinv10+1.3149350649350649e-03*w24*Jinv00-6.5746753246753243e-04*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5+6.5746753246753243e-04*w47*Jinv22+1.9724025974025973e-03*w49*Jinv11+2.3011363636363637e-03*Jinv21*w14-1.1323051948051949e-03*w10*Jinv10-1.1834415584415585e-02*Jinv20*w44+9.1314935064935063e-04*w58*Jinv12+2.3011363636363637e-03*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-1.1323051948051949e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-3.6160714285714286e-03*Jinv21*w23+1.3149350649350649e-03*w25*Jinv10-1.1323051948051949e-03*Jinv20*w2-7.8896103896103891e-03*w15*Jinv00-2.9586038961038961e-03*w48*Jinv02+1.1323051948051949e-03*Jinv01*w9-7.8896103896103891e-03*Jinv20*w17+1.9724025974025973e-03*w30*Jinv00-1.1834415584415585e-02*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11+6.5746753246753243e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56-1.9724025974025973e-03*w25*Jinv11-9.8620129870129864e-04*w55*Jinv10-1.1834415584415585e-02*w42*Jinv00+1.9724025974025973e-03*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12+1.9724025974025973e-03*Jinv21*w50+1.6436688311688311e-03*Jinv01*w27+1.9724025974025973e-03*w48*Jinv01-9.1314935064935063e-04*Jinv12*w1+1.1323051948051949e-03*w10*Jinv11)*D;
  A[33][40]=(0.0)*D;
  A[33][41]=(0.0)*D;
  A[33][42]=(-1.3149350649350649e-03*Jinv00*w12+1.1323051948051949e-03*w28*Jinv10+7.8896103896103891e-03*w40*Jinv12-1.9724025974025973e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+1.1323051948051949e-03*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+7.8896103896103891e-03*Jinv02*w39+1.7751623376623376e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.9724025974025973e-03*w32*Jinv20-3.9448051948051946e-03*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+9.8620129870129864e-04*Jinv22*w38+9.8620129870129864e-04*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59-1.9724025974025973e-03*w25*Jinv12+1.1323051948051949e-03*w0*Jinv02+2.2646103896103898e-03*Jinv21*w2+1.1834415584415585e-02*w43*Jinv10+1.7751623376623376e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+1.1834415584415585e-02*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+9.8620129870129864e-04*w48*Jinv00-1.9724025974025973e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26+2.2646103896103898e-03*w0*Jinv01+1.1323051948051949e-03*w1*Jinv10+7.8896103896103891e-03*w16*Jinv10+9.8620129870129864e-04*Jinv22*w20+1.7751623376623376e-02*w43*Jinv11-1.9724025974025973e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+1.1834415584415585e-02*w42*Jinv02+1.1323051948051949e-03*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+2.2646103896103898e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+2.7759740259740259e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+1.1323051948051949e-03*Jinv20*w29-1.9724025974025973e-03*w46*Jinv10+1.1834415584415585e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-1.9724025974025973e-03*w31*Jinv10-1.9724025974025973e-03*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+1.1323051948051949e-03*Jinv22*w2-1.3149350649350649e-03*Jinv00*w18-1.9724025974025973e-03*w36*Jinv00+2.9586038961038961e-03*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24+1.1323051948051949e-03*Jinv22*w59+3.5503246753246752e-02*w34*Jinv11+9.8620129870129864e-04*w19*Jinv11+2.9586038961038961e-03*Jinv21*w53+9.8620129870129864e-04*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+1.1834415584415585e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-3.9448051948051946e-03*w4*Jinv11+1.1834415584415585e-02*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-3.9448051948051946e-03*w32*Jinv21+9.8620129870129864e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+9.8620129870129864e-04*w37*Jinv12+9.8620129870129864e-04*Jinv20*w56+7.8896103896103891e-03*Jinv22*w41-1.3149350649350649e-03*w26*Jinv20+1.1323051948051949e-03*w11*Jinv20-1.9724025974025973e-03*w52*Jinv12+3.5503246753246752e-02*Jinv21*w35-3.9448051948051946e-03*w31*Jinv11-1.9724025974025973e-03*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-3.9448051948051946e-03*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-1.9724025974025973e-03*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10+2.7759740259740259e-03*Jinv21*w29+3.5503246753246752e-02*Jinv01*w33+1.1834415584415585e-02*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12+9.8620129870129864e-04*Jinv01*w18+2.9586038961038961e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9+1.9724025974025973e-03*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21-1.9724025974025973e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+2.9586038961038961e-03*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12+9.8620129870129864e-04*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-3.2873376623376621e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7-3.2873376623376621e-03*w21*Jinv01+2.9586038961038961e-03*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49+2.9586038961038961e-03*w36*Jinv01+9.8620129870129864e-04*w52*Jinv10+1.9724025974025973e-03*w7*Jinv11+1.1323051948051949e-03*w9*Jinv00-1.9724025974025973e-03*w37*Jinv10+9.8620129870129864e-04*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00+1.1323051948051949e-03*Jinv02*w27+9.8620129870129864e-04*w6*Jinv02+9.8620129870129864e-04*w19*Jinv12-1.9724025974025973e-03*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22+1.9724025974025973e-03*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.1323051948051949e-03*w10*Jinv10+1.1834415584415585e-02*Jinv20*w44+1.1323051948051949e-03*w58*Jinv12+9.8620129870129864e-04*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+1.9724025974025973e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+1.1323051948051949e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-3.2873376623376621e-03*Jinv21*w23+9.8620129870129864e-04*Jinv01*w54+9.8620129870129864e-04*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10+1.1323051948051949e-03*Jinv20*w2+7.8896103896103891e-03*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+7.8896103896103891e-03*Jinv20*w17-1.9724025974025973e-03*w30*Jinv00+1.1834415584415585e-02*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20+1.1323051948051949e-03*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+9.8620129870129864e-04*Jinv02*w36+1.1834415584415585e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-3.9448051948051946e-03*Jinv01*w30-1.9724025974025973e-03*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56+1.1323051948051949e-03*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+9.8620129870129864e-04*w55*Jinv10+1.1834415584415585e-02*w42*Jinv00-1.9724025974025973e-03*w47*Jinv20+1.1834415584415585e-02*w16*Jinv12+1.9724025974025973e-03*Jinv21*w50+2.7759740259740259e-03*Jinv01*w27+1.9724025974025973e-03*w48*Jinv01+1.1323051948051949e-03*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[33][43]=(0.0)*D;
  A[33][44]=(0.0)*D;
  A[33][45]=(-1.3149350649350649e-03*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-3.6525974025974026e-05*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3-5.4788961038961042e-05*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02-7.3051948051948052e-05*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.6436688311688311e-03*Jinv01*w45-3.8352272727272725e-04*Jinv12*w55-1.3149350649350649e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+1.0957792207792207e-03*Jinv02*w24-3.2873376623376621e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+1.0957792207792207e-03*Jinv21*w26-7.3051948051948052e-05*w0*Jinv01-3.8352272727272725e-04*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*w11*Jinv22-7.3051948051948052e-05*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-6.7573051948051951e-04*w28*Jinv11-1.3149350649350649e-03*Jinv22*w17+1.0957792207792207e-03*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-3.6525974025974026e-05*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+1.0957792207792207e-03*Jinv02*w45+1.0957792207792207e-03*Jinv01*w24-3.6525974025974026e-05*Jinv22*w59-9.8620129870129868e-03*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-3.2873376623376621e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-1.0957792207792208e-04*w4*Jinv11-3.8352272727272725e-04*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-5.4788961038961042e-05*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12-9.8620129870129868e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26+1.6436688311688311e-03*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21-3.8352272727272725e-04*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02-6.7573051948051951e-04*Jinv21*w29-9.8620129870129868e-03*Jinv01*w33-3.8352272727272725e-04*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.6436688311688311e-03*w47*Jinv21+1.0957792207792207e-03*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+1.0957792207792207e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-8.2183441558441553e-04*Jinv21*w56-4.9310064935064932e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+1.0957792207792207e-03*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-3.6525974025974026e-05*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-3.2873376623376621e-03*Jinv02*w33+1.0957792207792207e-03*Jinv21*w23-8.2183441558441553e-04*Jinv01*w54-8.2183441558441553e-04*w55*Jinv11-5.4788961038961042e-05*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9-4.9310064935064932e-04*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+1.0957792207792207e-03*w25*Jinv11-1.3149350649350649e-03*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50-6.7573051948051951e-04*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-3.6525974025974026e-05*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[33][46]=(0.0)*D;
  A[33][47]=(0.0)*D;
  A[33][48]=(3.8352272727272725e-04*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12-1.0957792207792207e-03*Jinv22*w53+7.6704545454545450e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+3.8352272727272725e-04*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+7.6704545454545450e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-6.0267857142857139e-04*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+5.6615259740259744e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27+5.6615259740259744e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-2.7394480519480518e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-1.2601461038961039e-03*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-8.7662337662337668e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-1.2053571428571428e-03*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+7.6704545454545450e-04*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20-4.3831168831168834e-04*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+7.6704545454545450e-04*w6*Jinv00-6.0267857142857139e-04*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-1.0957792207792207e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11+3.8352272727272725e-04*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54-8.7662337662337668e-04*w4*Jinv12-1.2601461038961039e-03*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-4.3831168831168834e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01+7.6704545454545450e-04*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-1.2601461038961039e-03*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12-1.2053571428571428e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10+7.6704545454545450e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+6.9399350649350649e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.2873376623376621e-04*w19*Jinv12-8.7662337662337668e-04*Jinv22*w5-1.2053571428571428e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12-3.2873376623376621e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+4.9310064935064932e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+6.9399350649350649e-04*Jinv22*w29-4.3831168831168834e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11-6.0267857142857139e-04*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+7.6704545454545450e-04*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-1.0957792207792207e-03*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+6.9399350649350649e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[33][49]=(0.0)*D;
  A[33][50]=(0.0)*D;
  A[33][51]=(-3.8352272727272725e-04*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+4.7666396103896100e-03*Jinv22*w53-7.6704545454545450e-04*Jinv20*w8-7.6704545454545450e-04*w57*Jinv02-3.8352272727272725e-04*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-7.6704545454545450e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-6.5746753246753243e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-1.3149350649350649e-03*w25*Jinv12+1.0592532467532468e-03*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10+1.5888798701298701e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+1.3806818181818182e-02*Jinv22*w35+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20-1.3149350649350649e-03*w45*Jinv00+6.9034090909090911e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+4.2735389610389614e-03*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.9724025974025973e-03*Jinv00*w33-8.7662337662337668e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-2.7394480519480520e-03*w32*Jinv22+1.0592532467532468e-03*Jinv22*w2-3.8352272727272725e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-2.5202922077922079e-03*Jinv02*w45-7.6704545454545450e-04*Jinv22*w59+4.2735389610389614e-03*w51*Jinv00+1.3806818181818182e-02*Jinv12*w34+6.9034090909090911e-03*Jinv22*w44-7.6704545454545450e-04*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.8352272727272725e-04*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10-7.6704545454545450e-04*w6*Jinv00-6.5746753246753243e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+4.7666396103896100e-03*w52*Jinv12-1.3149350649350649e-03*Jinv22*w26-3.8352272727272725e-04*Jinv20*w14+1.5888798701298701e-03*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-8.7662337662337668e-04*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+4.3831168831168834e-04*Jinv02*w9-7.6704545454545450e-04*w7*Jinv10-2.1367694805194807e-03*Jinv20*w38-8.7662337662337668e-04*Jinv22*w23-2.7394480519480520e-03*w31*Jinv12-2.5202922077922079e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-1.6436688311688311e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10+3.1777597402597403e-03*Jinv22*w50-1.6436688311688311e-03*Jinv12*w7-2.7394480519480520e-03*Jinv02*w30+3.1777597402597403e-03*Jinv12*w49+4.2735389610389614e-03*w52*Jinv10+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10-5.4788961038961042e-05*w24*Jinv00+9.6793831168831166e-04*Jinv02*w27-1.6436688311688311e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-2.5202922077922079e-03*w47*Jinv22+4.9310064935064932e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-7.6704545454545450e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+1.3806818181818182e-02*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+3.1777597402597403e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-6.5746753246753243e-04*Jinv02*w36+6.9034090909090911e-03*Jinv12*w43-7.6704545454545450e-04*w5*Jinv20+4.7666396103896100e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+9.6793831168831166e-04*Jinv12*w28+2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+1.0592532467532468e-03*Jinv12*w1)*D;
  A[33][52]=(0.0)*D;
  A[33][53]=(0.0)*D;
  A[33][54]=(4.6022727272727274e-03*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-3.8352272727272725e-04*w57*Jinv02+4.6022727272727274e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-7.6704545454545450e-04*Jinv01*w3-1.3697240259740260e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12+5.2962662337662338e-04*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.5888798701298701e-03*Jinv12*w55+6.5746753246753243e-04*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-7.1225649350649346e-04*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41+4.6022727272727274e-03*w42*Jinv02+5.2962662337662338e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+6.5746753246753243e-04*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-1.3697240259740260e-03*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39+5.2962662337662338e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-1.3697240259740260e-03*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+4.6022727272727274e-03*Jinv22*w44-7.6704545454545450e-04*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-1.3697240259740260e-03*w37*Jinv12+4.6022727272727274e-03*Jinv22*w41+1.5888798701298701e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*w5*Jinv21+1.5888798701298701e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-1.3149350649350649e-03*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12-1.3697240259740260e-03*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+1.5888798701298701e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.6704545454545450e-04*w7*Jinv11+2.1367694805194807e-03*Jinv21*w56+5.2962662337662338e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-1.3697240259740260e-03*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11+1.5888798701298701e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+5.2962662337662338e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-1.3697240259740260e-03*Jinv02*w36+4.6022727272727274e-03*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+1.5888798701298701e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+5.2962662337662338e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11+6.5746753246753243e-04*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01+5.2962662337662338e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[33][55]=(0.0)*D;
  A[33][56]=(0.0)*D;
  A[33][57]=(-1.8262987012987013e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-1.8262987012987013e-03*Jinv02*w39-3.2873376623376621e-04*Jinv22*w38+3.4699675324675324e-04*w25*Jinv12-4.2613636363636367e-05*w0*Jinv02+3.2873376623376621e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+3.4699675324675324e-04*Jinv02*w24-3.6525974025974025e-03*Jinv22*w35+2.1915584415584417e-04*Jinv22*w20-1.8262987012987013e-03*w42*Jinv02-4.2613636363636367e-05*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+3.4699675324675324e-04*w22*Jinv12+4.3831168831168834e-04*Jinv02*w3-3.2873376623376621e-04*w32*Jinv22-4.2613636363636367e-05*Jinv22*w2+2.9220779220779221e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-3.6525974025974025e-03*Jinv12*w34-1.8262987012987013e-03*Jinv22*w44+2.1915584415584417e-04*Jinv12*w13-3.2873376623376621e-04*w37*Jinv12-1.8262987012987013e-03*Jinv22*w41+6.5746753246753243e-04*w52*Jinv12+3.4699675324675324e-04*Jinv22*w26+3.2873376623376621e-04*Jinv02*w54+4.3831168831168834e-04*w4*Jinv12+3.4699675324675324e-04*w21*Jinv02+2.1915584415584417e-04*Jinv02*w12-4.2613636363636367e-05*Jinv02*w9+3.4699675324675324e-04*Jinv22*w23-3.2873376623376621e-04*w31*Jinv12+2.9220779220779221e-04*w46*Jinv12+4.3831168831168834e-04*Jinv22*w8+6.5746753246753243e-04*Jinv22*w50+4.3831168831168834e-04*Jinv12*w7-3.2873376623376621e-04*Jinv02*w30+6.5746753246753243e-04*Jinv12*w49-1.9480519480519481e-04*Jinv02*w27+4.3831168831168834e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12+4.3831168831168834e-04*Jinv22*w5+2.9220779220779221e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-3.6525974025974025e-03*Jinv02*w33+6.5746753246753243e-04*w48*Jinv02-1.9480519480519481e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14-3.2873376623376621e-04*Jinv02*w36-1.8262987012987013e-03*Jinv12*w43+6.5746753246753243e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56-1.9480519480519481e-04*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-4.2613636363636367e-05*Jinv12*w1)*D;
  A[33][58]=(0.0)*D;
  A[33][59]=(0.0)*D;
  A[34][0]=(0.0)*D;
  A[34][1]=(-3.2873376623376621e-04*Jinv00*w12+1.9480519480519481e-04*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12-4.3831168831168834e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-6.5746753246753243e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.9220779220779221e-04*Jinv20*w23-4.3831168831168834e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-3.4699675324675324e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17-3.4699675324675324e-04*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+1.8262987012987013e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00-3.4699675324675324e-04*Jinv02*w24+3.6525974025974025e-03*Jinv22*w35+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01-3.4699675324675324e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+1.8262987012987013e-03*w16*Jinv10-2.1915584415584417e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-3.4699675324675324e-04*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+1.9480519480519481e-04*Jinv00*w27+4.2613636363636367e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01+1.9480519480519481e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+1.9480519480519481e-04*Jinv20*w29-3.4699675324675324e-04*w46*Jinv10+1.8262987012987013e-03*Jinv22*w17+3.6525974025974025e-03*Jinv00*w33-2.9220779220779221e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3-2.9220779220779221e-04*w21*Jinv00-6.5746753246753243e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18-4.3831168831168834e-04*w36*Jinv00-4.3831168831168834e-04*Jinv21*w38-3.4699675324675324e-04*Jinv02*w45-3.4699675324675324e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+3.6525974025974025e-03*w34*Jinv11-2.1915584415584417e-04*w19*Jinv11-4.3831168831168834e-04*Jinv21*w53-4.3831168831168834e-04*w51*Jinv00+3.6525974025974025e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-6.5746753246753243e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11+1.8262987012987013e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10+3.6525974025974025e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21-2.1915584415584417e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00-4.3831168831168834e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-3.4699675324675324e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20-4.3831168831168834e-04*w52*Jinv12+3.6525974025974025e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-3.4699675324675324e-04*Jinv22*w26-3.4699675324675324e-04*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12-2.9220779220779221e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+3.6525974025974025e-03*Jinv01*w33+1.8262987012987013e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12-2.1915584415584417e-04*Jinv01*w18-4.3831168831168834e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-3.4699675324675324e-04*w47*Jinv21-4.3831168831168834e-04*Jinv20*w38-2.9220779220779221e-04*Jinv22*w23-4.3831168831168834e-04*w52*Jinv11-6.5746753246753243e-04*w31*Jinv12-3.4699675324675324e-04*w46*Jinv12-4.3831168831168834e-04*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8-2.9220779220779221e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50-2.9220779220779221e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7-2.9220779220779221e-04*w21*Jinv01-4.3831168831168834e-04*Jinv01*w51-6.5746753246753243e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.3831168831168834e-04*w36*Jinv01-4.3831168831168834e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-4.3831168831168834e-04*w37*Jinv10-2.1915584415584417e-04*Jinv21*w56-3.4699675324675324e-04*w24*Jinv00+1.9480519480519481e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5-3.4699675324675324e-04*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+3.6525974025974025e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+3.6525974025974025e-03*w35*Jinv20-2.9220779220779221e-04*Jinv21*w23-2.1915584415584417e-04*Jinv01*w54-2.1915584415584417e-04*w55*Jinv11-3.4699675324675324e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+1.8262987012987013e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+1.8262987012987013e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+1.8262987012987013e-03*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+1.9480519480519481e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11-4.3831168831168834e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-6.5746753246753243e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30-4.3831168831168834e-04*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56+1.9480519480519481e-04*Jinv12*w28-3.4699675324675324e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-3.4699675324675324e-04*w47*Jinv20+1.8262987012987013e-03*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[34][2]=(0.0)*D;
  A[34][3]=(0.0)*D;
  A[34][4]=(5.4788961038961036e-04*Jinv00*w12-6.9399350649350649e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-1.0592532467532468e-03*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00+1.0957792207792207e-03*w32*Jinv20-4.7666396103896100e-03*Jinv01*w3+1.2053571428571428e-03*Jinv20*w23+1.6436688311688311e-03*Jinv22*w38+3.2873376623376621e-04*Jinv00*w54-1.0592532467532468e-03*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-6.9034090909090911e-03*w16*Jinv11-6.9034090909090911e-03*Jinv21*w17+8.7662337662337668e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.6615259740259744e-04*w58*Jinv10-6.9034090909090911e-03*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+6.0267857142857139e-04*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-1.3806818181818182e-02*Jinv22*w35-1.0592532467532468e-03*w58*Jinv11-1.0592532467532468e-03*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-6.9399350649350649e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-6.9034090909090911e-03*w15*Jinv01-9.6793831168831166e-04*w28*Jinv11+6.0267857142857139e-04*Jinv20*w50-6.9399350649350649e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-6.9034090909090911e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33+2.5202922077922079e-03*w22*Jinv12+1.0957792207792207e-03*w31*Jinv10-4.7666396103896100e-03*Jinv02*w3+1.2053571428571428e-03*w21*Jinv00-3.1777597402597403e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+4.3831168831168834e-04*Jinv00*w18+8.7662337662337668e-04*w36*Jinv00+1.6436688311688311e-03*Jinv21*w38+8.7662337662337668e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24-1.0592532467532468e-03*Jinv22*w59-1.3806818181818182e-02*w34*Jinv11+8.2183441558441553e-04*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00-1.3806818181818182e-02*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-4.7666396103896100e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+4.3831168831168834e-04*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10-3.1777597402597403e-03*w32*Jinv21+8.2183441558441553e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00+1.6436688311688311e-03*w37*Jinv12+3.2873376623376621e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+1.2601461038961039e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-1.3806818181818182e-02*Jinv21*w35-3.1777597402597403e-03*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+8.7662337662337668e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14-4.7666396103896100e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-4.7666396103896100e-03*w4*Jinv12+2.5202922077922079e-03*w21*Jinv02+6.0267857142857139e-04*w49*Jinv10-9.6793831168831166e-04*Jinv21*w29-1.3806818181818182e-02*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+8.2183441558441553e-04*Jinv01*w18+1.6436688311688311e-03*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+6.5746753246753243e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+8.7662337662337668e-04*w47*Jinv21+8.7662337662337668e-04*Jinv20*w38+2.5202922077922079e-03*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-3.1777597402597403e-03*w31*Jinv12+8.7662337662337668e-04*w46*Jinv12+1.3697240259740260e-03*Jinv20*w53+6.5746753246753243e-04*Jinv22*w8+1.2053571428571428e-03*w22*Jinv10+2.7394480519480520e-03*Jinv22*w50+2.5202922077922079e-03*w22*Jinv11+6.5746753246753243e-04*Jinv12*w7+2.5202922077922079e-03*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-3.1777597402597403e-03*Jinv02*w30+2.7394480519480520e-03*Jinv12*w49+1.6436688311688311e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10+6.5746753246753243e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+8.7662337662337668e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+1.2601461038961039e-03*w24*Jinv00-9.6793831168831166e-04*Jinv02*w27+6.5746753246753243e-04*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12-4.7666396103896100e-03*Jinv22*w5+8.7662337662337668e-04*w47*Jinv22+2.7394480519480520e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-1.0592532467532468e-03*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+6.5746753246753243e-04*Jinv21*w8-5.6615259740259744e-04*w57*Jinv00-5.6615259740259744e-04*Jinv20*w59-1.3806818181818182e-02*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20+2.5202922077922079e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+1.2601461038961039e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+2.7394480519480520e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17+1.0957792207792207e-03*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10+4.3831168831168834e-04*w20*Jinv20-9.6793831168831166e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+1.6436688311688311e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-3.1777597402597403e-03*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-9.6793831168831166e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11+3.2873376623376621e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-6.9034090909090911e-03*w16*Jinv12+2.7394480519480520e-03*Jinv21*w50-9.6793831168831166e-04*Jinv01*w27+2.7394480519480520e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[34][5]=(0.0)*D;
  A[34][6]=(0.0)*D;
  A[34][7]=(-4.3831168831168834e-04*Jinv00*w12+6.9399350649350649e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-4.3831168831168834e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00-8.7662337662337668e-04*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3-1.2601461038961039e-03*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38-3.2873376623376621e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+3.8352272727272725e-04*Jinv12*w55+5.6615259740259744e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+6.9399350649350649e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+6.9399350649350649e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-8.7662337662337668e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3-1.2601461038961039e-03*w21*Jinv00+7.6704545454545450e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-1.0957792207792207e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53-6.0267857142857139e-04*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10+2.1367694805194807e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+3.8352272727272725e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12-3.2873376623376621e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.2053571428571428e-03*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-4.3831168831168834e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+3.8352272727272725e-04*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-1.0957792207792207e-03*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+7.6704545454545450e-04*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-6.0267857142857139e-04*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-1.2601461038961039e-03*w22*Jinv10+7.6704545454545450e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+7.6704545454545450e-04*Jinv02*w30+7.6704545454545450e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01-6.0267857142857139e-04*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00-1.0957792207792207e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-1.2053571428571428e-03*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+7.6704545454545450e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8+5.6615259740259744e-04*w57*Jinv00+5.6615259740259744e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-1.2053571428571428e-03*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+7.6704545454545450e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+5.9172077922077923e-03*Jinv20*w17-8.7662337662337668e-04*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+7.6704545454545450e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11-3.2873376623376621e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[34][8]=(0.0)*D;
  A[34][9]=(0.0)*D;
  A[34][10]=(2.1915584415584417e-04*Jinv00*w12-1.9480519480519481e-04*w28*Jinv10+6.5746753246753243e-04*Jinv20*w8+2.1915584415584417e-04*w13*Jinv10-3.2873376623376621e-04*w3*Jinv00+4.3831168831168834e-04*w32*Jinv20+3.4699675324675324e-04*Jinv20*w23+2.1915584415584417e-04*Jinv00*w54-1.8262987012987013e-03*w43*Jinv10-4.2613636363636367e-05*w58*Jinv10+4.3831168831168834e-04*w48*Jinv00-4.2613636363636367e-05*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10+3.4699675324675324e-04*w45*Jinv00-1.9480519480519481e-04*Jinv00*w27+4.3831168831168834e-04*Jinv20*w50-1.9480519480519481e-04*Jinv20*w29+3.4699675324675324e-04*w46*Jinv10-3.6525974025974025e-03*Jinv00*w33+4.3831168831168834e-04*w31*Jinv10+3.4699675324675324e-04*w21*Jinv00+3.2873376623376621e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-3.2873376623376621e-04*w51*Jinv00-3.2873376623376621e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+3.2873376623376621e-04*w19*Jinv10-3.6525974025974025e-03*w34*Jinv10+6.5746753246753243e-04*w6*Jinv00+2.1915584415584417e-04*Jinv20*w56+2.9220779220779221e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+2.1915584415584417e-04*Jinv20*w14+4.3831168831168834e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+6.5746753246753243e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-3.2873376623376621e-04*Jinv20*w53+3.4699675324675324e-04*w22*Jinv10-3.2873376623376621e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10+2.9220779220779221e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-1.8262987012987013e-03*Jinv20*w44-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-4.2613636363636367e-05*w0*Jinv00-3.6525974025974025e-03*w35*Jinv20+2.9220779220779221e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00-1.8262987012987013e-03*Jinv20*w17+4.3831168831168834e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-3.2873376623376621e-04*w5*Jinv20+2.1915584415584417e-04*w55*Jinv10-1.8262987012987013e-03*w42*Jinv00+3.4699675324675324e-04*w47*Jinv20)*D;
  A[34][11]=(0.0)*D;
  A[34][12]=(0.0)*D;
  A[34][13]=(-1.5888798701298701e-03*Jinv00*w12-5.2962662337662338e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-5.2962662337662338e-04*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20+2.6846590909090908e-03*Jinv01*w3+1.3697240259740260e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02-1.6436688311688311e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-3.6160714285714286e-03*w16*Jinv11-3.6160714285714286e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-4.6022727272727274e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12+1.3697240259740260e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.6436688311688311e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-3.6160714285714286e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22-1.6436688311688311e-04*Jinv11*w1+5.4788961038961036e-04*w13*Jinv11-3.6160714285714286e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+1.3697240259740260e-03*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-4.6022727272727274e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+1.3697240259740260e-03*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3+1.3697240259740260e-03*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-3.6160714285714286e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+6.5746753246753243e-04*Jinv01*w24-5.2962662337662338e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+3.2873376623376621e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10+2.6846590909090908e-03*w4*Jinv11-4.6022727272727274e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+2.6846590909090908e-03*w32*Jinv21+3.2873376623376621e-04*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-5.2962662337662338e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+2.6846590909090908e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+6.5746753246753243e-04*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14+2.6846590909090908e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+1.3697240259740260e-03*w21*Jinv02+1.3697240259740260e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-4.6022727272727274e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+3.2873376623376621e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+6.5746753246753243e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+1.3697240259740260e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+1.3697240259740260e-03*Jinv22*w8+1.3697240259740260e-03*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11-5.2962662337662338e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+3.2873376623376621e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+1.3697240259740260e-03*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11+5.4788961038961036e-04*Jinv21*w14-5.2962662337662338e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-5.2962662337662338e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+5.4788961038961036e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.2873376623376621e-04*Jinv01*w54+3.2873376623376621e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-4.6022727272727274e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-3.6160714285714286e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20+2.6846590909090908e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+6.5746753246753243e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-4.6022727272727274e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[34][14]=(0.0)*D;
  A[34][15]=(0.0)*D;
  A[34][16]=(2.3011363636363637e-03*Jinv00*w12-7.8896103896103891e-03*w40*Jinv12+1.9724025974025973e-03*Jinv22*w53-2.9586038961038961e-03*Jinv20*w8-1.1323051948051949e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10-7.8896103896103891e-03*Jinv02*w39+2.9586038961038961e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3-6.5746753246753243e-04*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38+1.1323051948051949e-03*Jinv21*w59+1.9724025974025973e-03*w25*Jinv12-1.1323051948051949e-03*w0*Jinv02-6.9399350649350649e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+2.1915584415584417e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17-1.9724025974025973e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-1.1834415584415585e-02*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-6.5746753246753243e-04*w48*Jinv00+1.9724025974025973e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35+1.1323051948051949e-03*w58*Jinv11+1.1323051948051949e-03*w57*Jinv01-2.9586038961038961e-03*Jinv21*w26-6.9399350649350649e-04*w0*Jinv01-9.1314935064935063e-04*w1*Jinv10-9.8620129870129864e-04*Jinv22*w20+3.9448051948051946e-03*Jinv21*w41-1.1834415584415585e-02*w42*Jinv02-2.1915584415584417e-04*w11*Jinv22-6.9399350649350649e-04*Jinv11*w1+2.3011363636363637e-03*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01+1.6436688311688311e-03*w28*Jinv11-6.5746753246753243e-04*Jinv20*w50-1.1834415584415585e-02*Jinv22*w17+1.3149350649350649e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+1.9724025974025973e-03*Jinv02*w3-6.5746753246753243e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22+3.9448051948051946e-03*Jinv01*w39-1.1323051948051949e-03*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00+9.8620129870129864e-04*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45-2.9586038961038961e-03*Jinv01*w24-1.1323051948051949e-03*Jinv22*w59+2.3668831168831169e-02*w34*Jinv11-1.9724025974025973e-03*Jinv21*w53+6.5746753246753243e-04*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-1.1834415584415585e-02*Jinv22*w44+2.9586038961038961e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+3.2873376623376621e-03*w32*Jinv21-2.9586038961038961e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12-7.8896103896103891e-03*Jinv22*w41+6.5746753246753243e-04*w26*Jinv20+9.1314935064935063e-04*w11*Jinv20+1.9724025974025973e-03*w52*Jinv12+2.3668831168831169e-02*Jinv21*w35+3.2873376623376621e-03*w31*Jinv11+1.9724025974025973e-03*Jinv22*w26-1.9724025974025973e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+1.9724025974025973e-03*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02-6.5746753246753243e-04*w49*Jinv10+1.6436688311688311e-03*Jinv21*w29+2.3668831168831169e-02*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+9.8620129870129864e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+1.9724025974025973e-03*w6*Jinv01-2.9586038961038961e-03*w7*Jinv10-1.9724025974025973e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-1.9724025974025973e-03*w52*Jinv11+1.3149350649350649e-03*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12+6.5746753246753243e-04*Jinv20*w53-9.8620129870129864e-04*Jinv22*w8-6.5746753246753243e-04*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-3.6160714285714286e-03*w22*Jinv11-9.8620129870129864e-04*Jinv12*w7-3.6160714285714286e-03*w21*Jinv01-1.9724025974025973e-03*Jinv01*w51+1.3149350649350649e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49+9.8620129870129864e-04*w36*Jinv01+6.5746753246753243e-04*w52*Jinv10+1.9724025974025973e-03*w7*Jinv11+9.1314935064935063e-04*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+6.5746753246753243e-04*w24*Jinv00-1.1323051948051949e-03*Jinv02*w27-9.8620129870129864e-04*w6*Jinv02-9.8620129870129864e-04*w19*Jinv12+1.9724025974025973e-03*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11+2.3011363636363637e-03*Jinv21*w14+9.1314935064935063e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44-1.1323051948051949e-03*w58*Jinv12-9.8620129870129864e-04*Jinv02*w18+2.3011363636363637e-03*Jinv01*w12+1.9724025974025973e-03*Jinv21*w8-1.1834415584415585e-02*Jinv02*w33-9.1314935064935063e-04*w0*Jinv00-3.6160714285714286e-03*Jinv21*w23+6.5746753246753243e-04*w25*Jinv10-9.1314935064935063e-04*Jinv20*w2+1.3149350649350649e-03*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+2.3011363636363637e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20-1.1323051948051949e-03*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14+3.9448051948051946e-03*w40*Jinv11-9.8620129870129864e-04*Jinv02*w36-1.1834415584415585e-02*Jinv12*w43+2.9586038961038961e-03*w5*Jinv20+3.2873376623376621e-03*Jinv01*w30+1.9724025974025973e-03*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56-1.1323051948051949e-03*Jinv12*w28-2.9586038961038961e-03*w25*Jinv11-3.9448051948051946e-03*w42*Jinv00-1.1834415584415585e-02*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50+1.6436688311688311e-03*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-1.1323051948051949e-03*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[34][17]=(0.0)*D;
  A[34][18]=(0.0)*D;
  A[34][19]=(-7.1225649350649346e-04*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+1.5888798701298701e-03*Jinv20*w8-7.1225649350649346e-04*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23-7.1225649350649346e-04*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59+4.9310064935064932e-04*Jinv21*w2+4.6022727272727274e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+5.2962662337662338e-04*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+5.2962662337662338e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+5.2962662337662338e-04*Jinv00*w27+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+1.5888798701298701e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-1.3149350649350649e-03*Jinv01*w24+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-1.3697240259740260e-03*w51*Jinv00-1.3697240259740260e-03*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.5888798701298701e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.5888798701298701e-03*w6*Jinv00-7.1225649350649346e-04*Jinv20*w56-1.3697240259740260e-03*w26*Jinv20-3.8352272727272725e-04*w11*Jinv20+1.9724025974025973e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+2.1367694805194807e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11+4.2735389610389614e-03*w6*Jinv01+1.5888798701298701e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.5888798701298701e-03*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-1.3697240259740260e-03*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+4.2735389610389614e-03*w36*Jinv01-1.3697240259740260e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56-1.3697240259740260e-03*w24*Jinv00-7.6704545454545450e-04*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10+4.6022727272727274e-03*Jinv20*w44-3.8352272727272725e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+5.2962662337662338e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11-1.3697240259740260e-03*w25*Jinv10+5.2962662337662338e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+4.6022727272727274e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-1.3697240259740260e-03*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30-1.3149350649350649e-03*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10+4.6022727272727274e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20-7.6704545454545450e-04*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[34][20]=(0.0)*D;
  A[34][21]=(0.0)*D;
  A[34][22]=(3.8352272727272725e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02+3.8352272727272725e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-2.5750811688311690e-03*Jinv01*w3-1.0957792207792207e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+3.8352272727272725e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+3.8352272727272725e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+3.2873376623376621e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+2.5568181818181820e-04*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+3.8352272727272725e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01-1.8262987012987014e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+3.2873376623376621e-03*Jinv00*w33-1.0957792207792207e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.0957792207792207e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45+3.6525974025974026e-05*Jinv22*w59-6.5746753246753243e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+3.2873376623376621e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-2.5750811688311690e-03*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+3.8352272727272725e-04*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10+3.2873376623376621e-03*w34*Jinv10-2.5750811688311690e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+3.8352272727272725e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-6.5746753246753243e-03*Jinv21*w35-2.5750811688311690e-03*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26+3.8352272727272725e-04*Jinv20*w14-2.5750811688311690e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-1.0957792207792207e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-1.8262987012987014e-04*Jinv21*w29-6.5746753246753243e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+3.8352272727272725e-04*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+5.4788961038961042e-05*Jinv20*w38-1.0957792207792207e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-1.0957792207792207e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+5.4788961038961036e-04*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+5.4788961038961036e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-3.2873376623376621e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+3.8352272727272725e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+3.2873376623376621e-03*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+3.2873376623376621e-03*w35*Jinv20+5.4788961038961036e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-2.5750811688311690e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28+3.8352272727272725e-04*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-1.8262987012987014e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[34][23]=(0.0)*D;
  A[34][24]=(0.0)*D;
  A[34][25]=(-3.8352272727272725e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-3.8352272727272725e-04*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23-3.8352272727272725e-04*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59-7.3051948051948052e-05*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17+1.0957792207792207e-03*Jinv01*w45-3.6525974025974026e-05*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+1.6436688311688311e-03*Jinv21*w26-7.3051948051948052e-05*w0*Jinv01-3.6525974025974026e-05*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41-4.9310064935064932e-04*Jinv00*w27-7.3051948051948052e-05*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01-6.7573051948051951e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-3.2873376623376621e-03*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10+1.0957792207792207e-03*w21*Jinv00+6.5746753246753243e-04*Jinv01*w39-3.8352272727272725e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+1.6436688311688311e-03*Jinv01*w24-9.8620129870129868e-03*w34*Jinv11-8.2183441558441553e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-5.4788961038961042e-05*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20-3.8352272727272725e-04*w19*Jinv10-3.2873376623376621e-03*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21-8.2183441558441553e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*Jinv20*w56+1.0957792207792207e-03*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20-9.8620129870129868e-03*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+1.0957792207792207e-03*w46*Jinv11-3.8352272727272725e-04*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21-5.4788961038961042e-05*w49*Jinv10-6.7573051948051951e-04*Jinv21*w29-9.8620129870129868e-03*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00-8.2183441558441553e-04*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.0957792207792207e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11-5.4788961038961042e-05*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10+1.0957792207792207e-03*w22*Jinv11+1.0957792207792207e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-2.6298701298701297e-03*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11-3.6525974025974026e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56+1.0957792207792207e-03*w24*Jinv00-1.0957792207792208e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-3.6525974025974026e-05*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-3.6525974025974026e-05*w0*Jinv00-3.2873376623376621e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11+1.0957792207792207e-03*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+6.5746753246753243e-04*w40*Jinv11-5.4788961038961042e-05*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30+1.6436688311688311e-03*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20-1.0957792207792208e-04*Jinv21*w50-6.7573051948051951e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[34][26]=(0.0)*D;
  A[34][27]=(0.0)*D;
  A[34][28]=(-5.1136363636363640e-04*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3+3.0438311688311688e-05*Jinv21*w59+3.0438311688311688e-05*Jinv21*w2-5.1136363636363640e-04*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21-5.1136363636363640e-04*w16*Jinv11-5.1136363636363640e-04*Jinv21*w17-3.1047077922077924e-04*Jinv01*w45+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-3.1047077922077924e-04*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01-5.1136363636363640e-04*w43*Jinv11-5.1136363636363640e-04*Jinv21*w41+3.0438311688311688e-05*Jinv11*w1+2.1915584415584417e-04*w13*Jinv11-5.1136363636363640e-04*w15*Jinv01+2.4350649350649351e-04*w28*Jinv11-5.1136363636363640e-04*Jinv01*w39+5.4788961038961036e-04*Jinv21*w38-3.1047077922077924e-04*Jinv01*w24+5.5519480519480519e-03*w34*Jinv11+2.1915584415584417e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+5.4788961038961036e-04*w4*Jinv11+5.4788961038961036e-04*w32*Jinv21+2.1915584415584417e-04*Jinv21*w20+5.5519480519480519e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-3.1047077922077924e-04*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21+2.4350649350649351e-04*Jinv21*w29+5.5519480519480519e-03*Jinv01*w33+2.1915584415584417e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01-3.1047077922077924e-04*w47*Jinv21+5.4788961038961036e-04*w52*Jinv11-3.1047077922077924e-04*w22*Jinv11-3.1047077922077924e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+5.4788961038961036e-04*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11+2.1915584415584417e-04*Jinv21*w56+5.4788961038961036e-04*w49*Jinv11+2.1915584415584417e-04*Jinv21*w14+2.1915584415584417e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-3.1047077922077924e-04*Jinv21*w23+2.1915584415584417e-04*Jinv01*w54+2.1915584415584417e-04*w55*Jinv11+3.0438311688311688e-05*Jinv01*w9-5.1136363636363640e-04*w40*Jinv11+5.4788961038961036e-04*Jinv01*w30-3.1047077922077924e-04*w25*Jinv11+5.4788961038961036e-04*Jinv21*w50+2.4350649350649351e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+3.0438311688311688e-05*w10*Jinv11)*D;
  A[34][29]=(0.0)*D;
  A[34][30]=(0.0)*D;
  A[34][31]=(-1.5888798701298701e-03*Jinv00*w12-9.6793831168831166e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12+8.7662337662337668e-04*Jinv22*w53+2.7394480519480520e-03*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-3.1777597402597403e-03*w3*Jinv00-4.7666396103896100e-03*w32*Jinv20-3.1777597402597403e-03*Jinv01*w3+2.5202922077922079e-03*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-1.0592532467532468e-03*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45+4.3831168831168834e-04*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-5.6615259740259744e-04*w10*Jinv12+6.5746753246753243e-04*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+8.7662337662337668e-04*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-6.9034090909090911e-03*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-9.6793831168831166e-04*Jinv00*w27-5.6615259740259744e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-9.6793831168831166e-04*w28*Jinv11+6.5746753246753243e-04*Jinv20*w50-9.6793831168831166e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-1.3806818181818182e-02*Jinv00*w33+1.2053571428571428e-03*w22*Jinv12-4.7666396103896100e-03*w31*Jinv10+1.0957792207792207e-03*Jinv02*w3+2.5202922077922079e-03*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22-6.9034090909090911e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.2601461038961039e-03*Jinv02*w45+8.7662337662337668e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-1.3806818181818182e-02*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-03*Jinv21*w53+1.6436688311688311e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-3.1777597402597403e-03*w4*Jinv10-3.1777597402597403e-03*w4*Jinv11-6.9034090909090911e-03*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-1.3806818181818182e-02*w34*Jinv10-4.7666396103896100e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+2.7394480519480520e-03*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41+8.7662337662337668e-04*w26*Jinv20-1.0592532467532468e-03*w11*Jinv20+8.7662337662337668e-04*w52*Jinv12-1.3806818181818182e-02*Jinv21*w35-4.7666396103896100e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14-3.1777597402597403e-03*w5*Jinv21+4.3831168831168834e-04*Jinv02*w54+1.0957792207792207e-03*w4*Jinv12+1.2053571428571428e-03*w21*Jinv02+6.5746753246753243e-04*w49*Jinv10-9.6793831168831166e-04*Jinv21*w29-1.3806818181818182e-02*Jinv01*w33-6.9034090909090911e-03*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.6615259740259744e-04*Jinv02*w9+2.7394480519480520e-03*w6*Jinv01+2.7394480519480520e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38+1.2053571428571428e-03*Jinv22*w23+1.6436688311688311e-03*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+1.2601461038961039e-03*w46*Jinv12+1.6436688311688311e-03*Jinv20*w53+6.0267857142857139e-04*Jinv22*w8+2.5202922077922079e-03*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+2.5202922077922079e-03*w22*Jinv11+6.0267857142857139e-04*Jinv12*w7+2.5202922077922079e-03*w21*Jinv01+1.6436688311688311e-03*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+1.6436688311688311e-03*w52*Jinv10+2.7394480519480520e-03*w7*Jinv11-1.0592532467532468e-03*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10+8.2183441558441553e-04*Jinv21*w56+8.7662337662337668e-04*w24*Jinv00-6.9399350649350649e-04*Jinv02*w27+6.0267857142857139e-04*w6*Jinv02+3.2873376623376621e-04*w19*Jinv12+1.0957792207792207e-03*Jinv22*w5+1.2601461038961039e-03*w47*Jinv22+6.5746753246753243e-04*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-1.0592532467532468e-03*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+3.2873376623376621e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+2.7394480519480520e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-1.1834415584415585e-02*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-1.3806818181818182e-02*w35*Jinv20+2.5202922077922079e-03*Jinv21*w23+8.2183441558441553e-04*Jinv01*w54+8.2183441558441553e-04*w55*Jinv11+8.7662337662337668e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-1.0592532467532468e-03*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-4.7666396103896100e-03*w30*Jinv00-6.9034090909090911e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-6.9399350649350649e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-6.9034090909090911e-03*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-3.1777597402597403e-03*w5*Jinv20-4.7666396103896100e-03*Jinv01*w30+8.7662337662337668e-04*Jinv02*w51+4.3831168831168834e-04*Jinv22*w56-6.9399350649350649e-04*Jinv12*w28+8.7662337662337668e-04*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+6.5746753246753243e-04*Jinv21*w50-9.6793831168831166e-04*Jinv01*w27+6.5746753246753243e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-1.0592532467532468e-03*w10*Jinv11)*D;
  A[34][32]=(0.0)*D;
  A[34][33]=(0.0)*D;
  A[34][34]=(2.3011363636363637e-03*Jinv00*w12-6.9034090909090911e-03*Jinv22*w53-5.9172077922077923e-03*Jinv20*w8+1.8262987012987013e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10-1.7751623376623376e-02*Jinv21*w44+5.9172077922077923e-03*w3*Jinv00+6.9034090909090911e-03*w32*Jinv20+3.9448051948051946e-03*Jinv01*w3-1.6436688311688311e-03*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38-4.3831168831168834e-04*Jinv21*w59-1.8262987012987013e-03*w0*Jinv02-2.2646103896103898e-03*Jinv21*w2-5.9172077922077923e-03*w43*Jinv10-1.7751623376623376e-02*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+4.9310064935064934e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55+5.9172077922077923e-03*w15*Jinv02+9.8620129870129864e-04*w48*Jinv00-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+4.9310064935064934e-03*Jinv21*w26-2.2646103896103898e-03*w0*Jinv01-1.8262987012987013e-03*w1*Jinv10-1.7751623376623376e-02*w43*Jinv11-1.1834415584415585e-02*Jinv21*w41-5.9172077922077923e-03*w42*Jinv02-2.2646103896103898e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-2.7759740259740259e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+5.9172077922077923e-03*Jinv22*w17-1.6436688311688311e-03*w22*Jinv12+6.9034090909090911e-03*w31*Jinv10+6.9034090909090911e-03*Jinv02*w3-1.6436688311688311e-03*w21*Jinv00+5.9172077922077923e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-1.8262987012987013e-03*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-6.9034090909090911e-03*w36*Jinv00-2.9586038961038961e-03*Jinv21*w38+1.6436688311688311e-03*Jinv02*w45+4.9310064935064934e-03*Jinv01*w24+1.8262987012987013e-03*Jinv22*w59-3.5503246753246752e-02*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11-2.9586038961038961e-03*Jinv21*w53-9.8620129870129864e-04*w51*Jinv00-5.9172077922077923e-03*Jinv22*w44+5.9172077922077923e-03*w4*Jinv10+3.9448051948051946e-03*w4*Jinv11+5.9172077922077923e-03*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+3.9448051948051946e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-5.9172077922077923e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12+1.6436688311688311e-03*w26*Jinv20+1.8262987012987013e-03*w11*Jinv20-6.9034090909090911e-03*w52*Jinv12-3.5503246753246752e-02*Jinv21*w35+3.9448051948051946e-03*w31*Jinv11+4.9310064935064934e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+3.9448051948051946e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+6.9034090909090911e-03*w4*Jinv12-1.6436688311688311e-03*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10-2.7759740259740259e-03*Jinv21*w29-3.5503246753246752e-02*Jinv01*w33+5.9172077922077923e-03*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-2.9586038961038961e-03*w37*Jinv11-1.9724025974025973e-03*w6*Jinv01-5.9172077922077923e-03*w7*Jinv10+4.9310064935064934e-03*w47*Jinv21-6.9034090909090911e-03*Jinv20*w38-1.6436688311688311e-03*Jinv22*w23-2.9586038961038961e-03*w52*Jinv11+5.9172077922077923e-03*w31*Jinv12+1.6436688311688311e-03*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8-1.6436688311688311e-03*w22*Jinv10-5.9172077922077923e-03*Jinv22*w50+3.2873376623376621e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7+3.2873376623376621e-03*w21*Jinv01-2.9586038961038961e-03*Jinv01*w51+5.9172077922077923e-03*Jinv02*w30-5.9172077922077923e-03*Jinv12*w49-2.9586038961038961e-03*w36*Jinv01-9.8620129870129864e-04*w52*Jinv10-1.9724025974025973e-03*w7*Jinv11+1.8262987012987013e-03*w9*Jinv00-6.9034090909090911e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56+1.6436688311688311e-03*w24*Jinv00+9.8620129870129864e-04*w6*Jinv02+6.9034090909090911e-03*Jinv22*w5+1.6436688311688311e-03*w47*Jinv22-1.9724025974025973e-03*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+1.8262987012987013e-03*w10*Jinv10-5.9172077922077923e-03*Jinv20*w44+1.8262987012987013e-03*w58*Jinv12+1.3149350649350649e-03*Jinv01*w12-1.9724025974025973e-03*Jinv21*w8-1.8262987012987013e-03*w0*Jinv00+3.2873376623376621e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11+1.6436688311688311e-03*w25*Jinv10-1.8262987012987013e-03*Jinv20*w2-5.9172077922077923e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9+6.9034090909090911e-03*w30*Jinv00+5.9172077922077923e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11-9.8620129870129864e-04*Jinv02*w36-5.9172077922077923e-03*Jinv12*w43+5.9172077922077923e-03*w5*Jinv20+3.9448051948051946e-03*Jinv01*w30-6.9034090909090911e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56+4.9310064935064934e-03*w25*Jinv11-5.9172077922077923e-03*w42*Jinv00+5.9172077922077923e-03*w16*Jinv12-1.9724025974025973e-03*Jinv21*w50-2.7759740259740259e-03*Jinv01*w27-1.9724025974025973e-03*w48*Jinv01-1.8262987012987013e-03*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[34][35]=(0.0)*D;
  A[34][36]=(0.0)*D;
  A[34][37]=(-7.1225649350649346e-04*Jinv00*w12+9.6793831168831166e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+3.1777597402597403e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-2.7394480519480520e-03*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-8.7662337662337668e-04*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-1.3149350649350649e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+6.9034090909090911e-03*w43*Jinv10-3.8352272727272725e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-1.6436688311688311e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+1.0592532467532468e-03*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-1.3149350649350649e-03*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02+9.6793831168831166e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-1.6436688311688311e-03*Jinv20*w50+9.6793831168831166e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.3806818181818182e-02*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-8.7662337662337668e-04*w21*Jinv00-7.6704545454545450e-04*w32*Jinv22+4.9310064935064932e-04*Jinv22*w2+1.5888798701298701e-03*Jinv00*w18+4.7666396103896100e-03*w36*Jinv00-5.4788961038961042e-05*Jinv02*w45+4.9310064935064932e-04*Jinv22*w59-6.5746753246753243e-04*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-2.7394480519480520e-03*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-3.8352272727272725e-04*Jinv12*w13+1.5888798701298701e-03*w19*Jinv10+1.3806818181818182e-02*w34*Jinv10+3.1777597402597403e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-2.5202922077922079e-03*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12-1.3149350649350649e-03*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-3.8352272727272725e-04*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-1.6436688311688311e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-3.8352272727272725e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+3.1777597402597403e-03*w7*Jinv10+4.7666396103896100e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23-7.6704545454545450e-04*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-6.5746753246753243e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-8.7662337662337668e-04*w22*Jinv10-7.6704545454545450e-04*Jinv22*w50+4.2735389610389614e-03*Jinv12*w7-7.6704545454545450e-04*Jinv02*w30-7.6704545454545450e-04*Jinv12*w49-6.5746753246753243e-04*w52*Jinv10-7.6704545454545450e-04*w9*Jinv00+4.7666396103896100e-03*w37*Jinv10-2.5202922077922079e-03*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-7.6704545454545450e-04*w10*Jinv10+6.9034090909090911e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+1.0592532467532468e-03*w0*Jinv00+1.3806818181818182e-02*w35*Jinv20-2.5202922077922079e-03*w25*Jinv10+1.0592532467532468e-03*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-7.6704545454545450e-04*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+4.2735389610389614e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-2.7394480519480520e-03*w5*Jinv20-2.1367694805194807e-03*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28-8.2183441558441553e-04*w55*Jinv10+6.9034090909090911e-03*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+4.9310064935064932e-04*Jinv12*w1)*D;
  A[34][38]=(0.0)*D;
  A[34][39]=(0.0)*D;
  A[34][40]=(1.3149350649350649e-03*Jinv00*w12-1.1323051948051949e-03*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+9.1314935064935063e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10+1.3149350649350649e-03*w3*Jinv00+1.9724025974025973e-03*w32*Jinv20+3.2873376623376621e-03*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23+6.5746753246753243e-04*Jinv22*w38-9.8620129870129864e-04*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59-9.1314935064935063e-04*w0*Jinv02-6.9399350649350649e-04*Jinv21*w2-1.1834415584415585e-02*w43*Jinv10+1.1323051948051949e-03*w11*Jinv21+3.9448051948051946e-03*w16*Jinv11+3.9448051948051946e-03*Jinv21*w17-2.9586038961038961e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02-9.8620129870129864e-04*w48*Jinv00+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.9724025974025973e-03*Jinv21*w26-6.9399350649350649e-04*w0*Jinv01-1.1323051948051949e-03*w1*Jinv10-7.8896103896103891e-03*w16*Jinv10+1.9724025974025973e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02-1.1323051948051949e-03*Jinv00*w27-6.9399350649350649e-04*Jinv11*w1+2.3011363636363637e-03*w13*Jinv11+3.9448051948051946e-03*w15*Jinv01+1.6436688311688311e-03*w28*Jinv11-9.8620129870129864e-04*Jinv20*w50-1.1323051948051949e-03*Jinv20*w29+1.9724025974025973e-03*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33-6.5746753246753243e-04*w22*Jinv12+1.9724025974025973e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00+2.9586038961038961e-03*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-9.1314935064935063e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+1.9724025974025973e-03*w36*Jinv00-1.9724025974025973e-03*Jinv21*w38+6.5746753246753243e-04*Jinv02*w45-1.9724025974025973e-03*Jinv01*w24+9.1314935064935063e-04*Jinv22*w59+2.3668831168831169e-02*w34*Jinv11+9.8620129870129864e-04*Jinv21*w53-9.8620129870129864e-04*w51*Jinv00-3.9448051948051946e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+3.2873376623376621e-03*w4*Jinv11-1.1834415584415585e-02*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21+1.3149350649350649e-03*w6*Jinv00+6.5746753246753243e-04*w37*Jinv12-9.8620129870129864e-04*Jinv20*w56+1.3149350649350649e-03*w26*Jinv20-1.1323051948051949e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12+2.3668831168831169e-02*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11-2.9586038961038961e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14+3.2873376623376621e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12-6.5746753246753243e-04*w21*Jinv02-9.8620129870129864e-04*w49*Jinv10+1.6436688311688311e-03*Jinv21*w29+2.3668831168831169e-02*Jinv01*w33-1.1834415584415585e-02*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12-1.9724025974025973e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.9586038961038961e-03*w47*Jinv21+1.9724025974025973e-03*Jinv20*w38-6.5746753246753243e-04*Jinv22*w23+9.8620129870129864e-04*w52*Jinv11+2.9586038961038961e-03*w31*Jinv12+6.5746753246753243e-04*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53-6.5746753246753243e-04*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10-2.9586038961038961e-03*Jinv22*w50-3.6160714285714286e-03*w22*Jinv11-6.5746753246753243e-04*Jinv12*w7-3.6160714285714286e-03*w21*Jinv01+9.8620129870129864e-04*Jinv01*w51+2.9586038961038961e-03*Jinv02*w30-2.9586038961038961e-03*Jinv12*w49-1.9724025974025973e-03*w36*Jinv01-9.8620129870129864e-04*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11-1.1323051948051949e-03*w9*Jinv00+1.9724025974025973e-03*w37*Jinv10+1.3149350649350649e-03*w24*Jinv00-6.5746753246753243e-04*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5+6.5746753246753243e-04*w47*Jinv22+1.9724025974025973e-03*w49*Jinv11+2.3011363636363637e-03*Jinv21*w14-1.1323051948051949e-03*w10*Jinv10-1.1834415584415585e-02*Jinv20*w44+9.1314935064935063e-04*w58*Jinv12+2.3011363636363637e-03*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-1.1323051948051949e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-3.6160714285714286e-03*Jinv21*w23+1.3149350649350649e-03*w25*Jinv10-1.1323051948051949e-03*Jinv20*w2-7.8896103896103891e-03*w15*Jinv00-2.9586038961038961e-03*w48*Jinv02+1.1323051948051949e-03*Jinv01*w9-7.8896103896103891e-03*Jinv20*w17+1.9724025974025973e-03*w30*Jinv00-1.1834415584415585e-02*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11+6.5746753246753243e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56-1.9724025974025973e-03*w25*Jinv11-9.8620129870129864e-04*w55*Jinv10-1.1834415584415585e-02*w42*Jinv00+1.9724025974025973e-03*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12+1.9724025974025973e-03*Jinv21*w50+1.6436688311688311e-03*Jinv01*w27+1.9724025974025973e-03*w48*Jinv01-9.1314935064935063e-04*Jinv12*w1+1.1323051948051949e-03*w10*Jinv11)*D;
  A[34][41]=(0.0)*D;
  A[34][42]=(0.0)*D;
  A[34][43]=(-1.3149350649350649e-03*Jinv00*w12+1.1323051948051949e-03*w28*Jinv10+7.8896103896103891e-03*w40*Jinv12-1.9724025974025973e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+1.1323051948051949e-03*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+7.8896103896103891e-03*Jinv02*w39+1.7751623376623376e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.9724025974025973e-03*w32*Jinv20-3.9448051948051946e-03*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+9.8620129870129864e-04*Jinv22*w38+9.8620129870129864e-04*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59-1.9724025974025973e-03*w25*Jinv12+1.1323051948051949e-03*w0*Jinv02+2.2646103896103898e-03*Jinv21*w2+1.1834415584415585e-02*w43*Jinv10+1.7751623376623376e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+1.1834415584415585e-02*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+9.8620129870129864e-04*w48*Jinv00-1.9724025974025973e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26+2.2646103896103898e-03*w0*Jinv01+1.1323051948051949e-03*w1*Jinv10+7.8896103896103891e-03*w16*Jinv10+9.8620129870129864e-04*Jinv22*w20+1.7751623376623376e-02*w43*Jinv11-1.9724025974025973e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+1.1834415584415585e-02*w42*Jinv02+1.1323051948051949e-03*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+2.2646103896103898e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+2.7759740259740259e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+1.1323051948051949e-03*Jinv20*w29-1.9724025974025973e-03*w46*Jinv10+1.1834415584415585e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-1.9724025974025973e-03*w31*Jinv10-1.9724025974025973e-03*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+1.1323051948051949e-03*Jinv22*w2-1.3149350649350649e-03*Jinv00*w18-1.9724025974025973e-03*w36*Jinv00+2.9586038961038961e-03*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24+1.1323051948051949e-03*Jinv22*w59+3.5503246753246752e-02*w34*Jinv11+9.8620129870129864e-04*w19*Jinv11+2.9586038961038961e-03*Jinv21*w53+9.8620129870129864e-04*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+1.1834415584415585e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-3.9448051948051946e-03*w4*Jinv11+1.1834415584415585e-02*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-3.9448051948051946e-03*w32*Jinv21+9.8620129870129864e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+9.8620129870129864e-04*w37*Jinv12+9.8620129870129864e-04*Jinv20*w56+7.8896103896103891e-03*Jinv22*w41-1.3149350649350649e-03*w26*Jinv20+1.1323051948051949e-03*w11*Jinv20-1.9724025974025973e-03*w52*Jinv12+3.5503246753246752e-02*Jinv21*w35-3.9448051948051946e-03*w31*Jinv11-1.9724025974025973e-03*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-3.9448051948051946e-03*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-1.9724025974025973e-03*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10+2.7759740259740259e-03*Jinv21*w29+3.5503246753246752e-02*Jinv01*w33+1.1834415584415585e-02*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12+9.8620129870129864e-04*Jinv01*w18+2.9586038961038961e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9+1.9724025974025973e-03*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21-1.9724025974025973e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+2.9586038961038961e-03*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12+9.8620129870129864e-04*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-3.2873376623376621e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7-3.2873376623376621e-03*w21*Jinv01+2.9586038961038961e-03*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49+2.9586038961038961e-03*w36*Jinv01+9.8620129870129864e-04*w52*Jinv10+1.9724025974025973e-03*w7*Jinv11+1.1323051948051949e-03*w9*Jinv00-1.9724025974025973e-03*w37*Jinv10+9.8620129870129864e-04*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00+1.1323051948051949e-03*Jinv02*w27+9.8620129870129864e-04*w6*Jinv02+9.8620129870129864e-04*w19*Jinv12-1.9724025974025973e-03*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22+1.9724025974025973e-03*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.1323051948051949e-03*w10*Jinv10+1.1834415584415585e-02*Jinv20*w44+1.1323051948051949e-03*w58*Jinv12+9.8620129870129864e-04*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+1.9724025974025973e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+1.1323051948051949e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-3.2873376623376621e-03*Jinv21*w23+9.8620129870129864e-04*Jinv01*w54+9.8620129870129864e-04*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10+1.1323051948051949e-03*Jinv20*w2+7.8896103896103891e-03*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+7.8896103896103891e-03*Jinv20*w17-1.9724025974025973e-03*w30*Jinv00+1.1834415584415585e-02*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20+1.1323051948051949e-03*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+9.8620129870129864e-04*Jinv02*w36+1.1834415584415585e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-3.9448051948051946e-03*Jinv01*w30-1.9724025974025973e-03*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56+1.1323051948051949e-03*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+9.8620129870129864e-04*w55*Jinv10+1.1834415584415585e-02*w42*Jinv00-1.9724025974025973e-03*w47*Jinv20+1.1834415584415585e-02*w16*Jinv12+1.9724025974025973e-03*Jinv21*w50+2.7759740259740259e-03*Jinv01*w27+1.9724025974025973e-03*w48*Jinv01+1.1323051948051949e-03*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[34][44]=(0.0)*D;
  A[34][45]=(0.0)*D;
  A[34][46]=(-1.3149350649350649e-03*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-3.6525974025974026e-05*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3-5.4788961038961042e-05*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02-7.3051948051948052e-05*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.6436688311688311e-03*Jinv01*w45-3.8352272727272725e-04*Jinv12*w55-1.3149350649350649e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+1.0957792207792207e-03*Jinv02*w24-3.2873376623376621e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+1.0957792207792207e-03*Jinv21*w26-7.3051948051948052e-05*w0*Jinv01-3.8352272727272725e-04*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*w11*Jinv22-7.3051948051948052e-05*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-6.7573051948051951e-04*w28*Jinv11-1.3149350649350649e-03*Jinv22*w17+1.0957792207792207e-03*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-3.6525974025974026e-05*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+1.0957792207792207e-03*Jinv02*w45+1.0957792207792207e-03*Jinv01*w24-3.6525974025974026e-05*Jinv22*w59-9.8620129870129868e-03*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-3.2873376623376621e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-1.0957792207792208e-04*w4*Jinv11-3.8352272727272725e-04*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-5.4788961038961042e-05*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12-9.8620129870129868e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26+1.6436688311688311e-03*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21-3.8352272727272725e-04*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02-6.7573051948051951e-04*Jinv21*w29-9.8620129870129868e-03*Jinv01*w33-3.8352272727272725e-04*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.6436688311688311e-03*w47*Jinv21+1.0957792207792207e-03*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+1.0957792207792207e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-8.2183441558441553e-04*Jinv21*w56-4.9310064935064932e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+1.0957792207792207e-03*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-3.6525974025974026e-05*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-3.2873376623376621e-03*Jinv02*w33+1.0957792207792207e-03*Jinv21*w23-8.2183441558441553e-04*Jinv01*w54-8.2183441558441553e-04*w55*Jinv11-5.4788961038961042e-05*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9-4.9310064935064932e-04*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+1.0957792207792207e-03*w25*Jinv11-1.3149350649350649e-03*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50-6.7573051948051951e-04*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-3.6525974025974026e-05*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[34][47]=(0.0)*D;
  A[34][48]=(0.0)*D;
  A[34][49]=(3.8352272727272725e-04*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12-1.0957792207792207e-03*Jinv22*w53+7.6704545454545450e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+3.8352272727272725e-04*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+7.6704545454545450e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-6.0267857142857139e-04*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+5.6615259740259744e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27+5.6615259740259744e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-2.7394480519480518e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-1.2601461038961039e-03*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-8.7662337662337668e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-1.2053571428571428e-03*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+7.6704545454545450e-04*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20-4.3831168831168834e-04*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+7.6704545454545450e-04*w6*Jinv00-6.0267857142857139e-04*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-1.0957792207792207e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11+3.8352272727272725e-04*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54-8.7662337662337668e-04*w4*Jinv12-1.2601461038961039e-03*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-4.3831168831168834e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01+7.6704545454545450e-04*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-1.2601461038961039e-03*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12-1.2053571428571428e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10+7.6704545454545450e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+6.9399350649350649e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.2873376623376621e-04*w19*Jinv12-8.7662337662337668e-04*Jinv22*w5-1.2053571428571428e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12-3.2873376623376621e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+4.9310064935064932e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+6.9399350649350649e-04*Jinv22*w29-4.3831168831168834e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11-6.0267857142857139e-04*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+7.6704545454545450e-04*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-1.0957792207792207e-03*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+6.9399350649350649e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[34][50]=(0.0)*D;
  A[34][51]=(0.0)*D;
  A[34][52]=(-3.8352272727272725e-04*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+4.7666396103896100e-03*Jinv22*w53-7.6704545454545450e-04*Jinv20*w8-7.6704545454545450e-04*w57*Jinv02-3.8352272727272725e-04*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-7.6704545454545450e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-6.5746753246753243e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-1.3149350649350649e-03*w25*Jinv12+1.0592532467532468e-03*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10+1.5888798701298701e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+1.3806818181818182e-02*Jinv22*w35+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20-1.3149350649350649e-03*w45*Jinv00+6.9034090909090911e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+4.2735389610389614e-03*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.9724025974025973e-03*Jinv00*w33-8.7662337662337668e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-2.7394480519480520e-03*w32*Jinv22+1.0592532467532468e-03*Jinv22*w2-3.8352272727272725e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-2.5202922077922079e-03*Jinv02*w45-7.6704545454545450e-04*Jinv22*w59+4.2735389610389614e-03*w51*Jinv00+1.3806818181818182e-02*Jinv12*w34+6.9034090909090911e-03*Jinv22*w44-7.6704545454545450e-04*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.8352272727272725e-04*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10-7.6704545454545450e-04*w6*Jinv00-6.5746753246753243e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+4.7666396103896100e-03*w52*Jinv12-1.3149350649350649e-03*Jinv22*w26-3.8352272727272725e-04*Jinv20*w14+1.5888798701298701e-03*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-8.7662337662337668e-04*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+4.3831168831168834e-04*Jinv02*w9-7.6704545454545450e-04*w7*Jinv10-2.1367694805194807e-03*Jinv20*w38-8.7662337662337668e-04*Jinv22*w23-2.7394480519480520e-03*w31*Jinv12-2.5202922077922079e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-1.6436688311688311e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10+3.1777597402597403e-03*Jinv22*w50-1.6436688311688311e-03*Jinv12*w7-2.7394480519480520e-03*Jinv02*w30+3.1777597402597403e-03*Jinv12*w49+4.2735389610389614e-03*w52*Jinv10+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10-5.4788961038961042e-05*w24*Jinv00+9.6793831168831166e-04*Jinv02*w27-1.6436688311688311e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-2.5202922077922079e-03*w47*Jinv22+4.9310064935064932e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-7.6704545454545450e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+1.3806818181818182e-02*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+3.1777597402597403e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-6.5746753246753243e-04*Jinv02*w36+6.9034090909090911e-03*Jinv12*w43-7.6704545454545450e-04*w5*Jinv20+4.7666396103896100e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+9.6793831168831166e-04*Jinv12*w28+2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+1.0592532467532468e-03*Jinv12*w1)*D;
  A[34][53]=(0.0)*D;
  A[34][54]=(0.0)*D;
  A[34][55]=(4.6022727272727274e-03*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-3.8352272727272725e-04*w57*Jinv02+4.6022727272727274e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-7.6704545454545450e-04*Jinv01*w3-1.3697240259740260e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12+5.2962662337662338e-04*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.5888798701298701e-03*Jinv12*w55+6.5746753246753243e-04*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-7.1225649350649346e-04*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41+4.6022727272727274e-03*w42*Jinv02+5.2962662337662338e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+6.5746753246753243e-04*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-1.3697240259740260e-03*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39+5.2962662337662338e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-1.3697240259740260e-03*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+4.6022727272727274e-03*Jinv22*w44-7.6704545454545450e-04*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-1.3697240259740260e-03*w37*Jinv12+4.6022727272727274e-03*Jinv22*w41+1.5888798701298701e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*w5*Jinv21+1.5888798701298701e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-1.3149350649350649e-03*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12-1.3697240259740260e-03*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+1.5888798701298701e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.6704545454545450e-04*w7*Jinv11+2.1367694805194807e-03*Jinv21*w56+5.2962662337662338e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-1.3697240259740260e-03*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11+1.5888798701298701e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+5.2962662337662338e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-1.3697240259740260e-03*Jinv02*w36+4.6022727272727274e-03*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+1.5888798701298701e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+5.2962662337662338e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11+6.5746753246753243e-04*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01+5.2962662337662338e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[34][56]=(0.0)*D;
  A[34][57]=(0.0)*D;
  A[34][58]=(-1.8262987012987013e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-1.8262987012987013e-03*Jinv02*w39-3.2873376623376621e-04*Jinv22*w38+3.4699675324675324e-04*w25*Jinv12-4.2613636363636367e-05*w0*Jinv02+3.2873376623376621e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+3.4699675324675324e-04*Jinv02*w24-3.6525974025974025e-03*Jinv22*w35+2.1915584415584417e-04*Jinv22*w20-1.8262987012987013e-03*w42*Jinv02-4.2613636363636367e-05*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+3.4699675324675324e-04*w22*Jinv12+4.3831168831168834e-04*Jinv02*w3-3.2873376623376621e-04*w32*Jinv22-4.2613636363636367e-05*Jinv22*w2+2.9220779220779221e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-3.6525974025974025e-03*Jinv12*w34-1.8262987012987013e-03*Jinv22*w44+2.1915584415584417e-04*Jinv12*w13-3.2873376623376621e-04*w37*Jinv12-1.8262987012987013e-03*Jinv22*w41+6.5746753246753243e-04*w52*Jinv12+3.4699675324675324e-04*Jinv22*w26+3.2873376623376621e-04*Jinv02*w54+4.3831168831168834e-04*w4*Jinv12+3.4699675324675324e-04*w21*Jinv02+2.1915584415584417e-04*Jinv02*w12-4.2613636363636367e-05*Jinv02*w9+3.4699675324675324e-04*Jinv22*w23-3.2873376623376621e-04*w31*Jinv12+2.9220779220779221e-04*w46*Jinv12+4.3831168831168834e-04*Jinv22*w8+6.5746753246753243e-04*Jinv22*w50+4.3831168831168834e-04*Jinv12*w7-3.2873376623376621e-04*Jinv02*w30+6.5746753246753243e-04*Jinv12*w49-1.9480519480519481e-04*Jinv02*w27+4.3831168831168834e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12+4.3831168831168834e-04*Jinv22*w5+2.9220779220779221e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-3.6525974025974025e-03*Jinv02*w33+6.5746753246753243e-04*w48*Jinv02-1.9480519480519481e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14-3.2873376623376621e-04*Jinv02*w36-1.8262987012987013e-03*Jinv12*w43+6.5746753246753243e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56-1.9480519480519481e-04*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-4.2613636363636367e-05*Jinv12*w1)*D;
  A[34][59]=(0.0)*D;
  A[35][0]=(0.0)*D;
  A[35][1]=(0.0)*D;
  A[35][2]=(-3.2873376623376621e-04*Jinv00*w12+1.9480519480519481e-04*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12-4.3831168831168834e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-6.5746753246753243e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.9220779220779221e-04*Jinv20*w23-4.3831168831168834e-04*Jinv22*w38-2.1915584415584417e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-3.4699675324675324e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+4.2613636363636367e-05*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17-3.4699675324675324e-04*Jinv01*w45-2.1915584415584417e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+1.8262987012987013e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00-3.4699675324675324e-04*Jinv02*w24+3.6525974025974025e-03*Jinv22*w35+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01-3.4699675324675324e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+1.8262987012987013e-03*w16*Jinv10-2.1915584415584417e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-3.4699675324675324e-04*w45*Jinv00+1.8262987012987013e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+1.9480519480519481e-04*Jinv00*w27+4.2613636363636367e-05*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01+1.9480519480519481e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+1.9480519480519481e-04*Jinv20*w29-3.4699675324675324e-04*w46*Jinv10+1.8262987012987013e-03*Jinv22*w17+3.6525974025974025e-03*Jinv00*w33-2.9220779220779221e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3-2.9220779220779221e-04*w21*Jinv00-6.5746753246753243e-04*w32*Jinv22+1.8262987012987013e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-2.1915584415584417e-04*Jinv00*w18-4.3831168831168834e-04*w36*Jinv00-4.3831168831168834e-04*Jinv21*w38-3.4699675324675324e-04*Jinv02*w45-3.4699675324675324e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+3.6525974025974025e-03*w34*Jinv11-2.1915584415584417e-04*w19*Jinv11-4.3831168831168834e-04*Jinv21*w53-4.3831168831168834e-04*w51*Jinv00+3.6525974025974025e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-6.5746753246753243e-04*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11+1.8262987012987013e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13-2.1915584415584417e-04*w19*Jinv10+3.6525974025974025e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21-2.1915584415584417e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00-4.3831168831168834e-04*w37*Jinv12-2.1915584415584417e-04*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-3.4699675324675324e-04*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20-4.3831168831168834e-04*w52*Jinv12+3.6525974025974025e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-3.4699675324675324e-04*Jinv22*w26-3.4699675324675324e-04*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21-2.1915584415584417e-04*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12-2.9220779220779221e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+3.6525974025974025e-03*Jinv01*w33+1.8262987012987013e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12-2.1915584415584417e-04*Jinv01*w18-4.3831168831168834e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-3.4699675324675324e-04*w47*Jinv21-4.3831168831168834e-04*Jinv20*w38-2.9220779220779221e-04*Jinv22*w23-4.3831168831168834e-04*w52*Jinv11-6.5746753246753243e-04*w31*Jinv12-3.4699675324675324e-04*w46*Jinv12-4.3831168831168834e-04*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8-2.9220779220779221e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50-2.9220779220779221e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7-2.9220779220779221e-04*w21*Jinv01-4.3831168831168834e-04*Jinv01*w51-6.5746753246753243e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.3831168831168834e-04*w36*Jinv01-4.3831168831168834e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-4.3831168831168834e-04*w37*Jinv10-2.1915584415584417e-04*Jinv21*w56-3.4699675324675324e-04*w24*Jinv00+1.9480519480519481e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5-3.4699675324675324e-04*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+3.6525974025974025e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+3.6525974025974025e-03*w35*Jinv20-2.9220779220779221e-04*Jinv21*w23-2.1915584415584417e-04*Jinv01*w54-2.1915584415584417e-04*w55*Jinv11-3.4699675324675324e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+1.8262987012987013e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+4.2613636363636367e-05*Jinv01*w9+1.8262987012987013e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+1.8262987012987013e-03*w40*Jinv10-2.1915584415584417e-04*w20*Jinv20+1.9480519480519481e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.8262987012987013e-03*w40*Jinv11-4.3831168831168834e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-6.5746753246753243e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30-4.3831168831168834e-04*Jinv02*w51-2.1915584415584417e-04*Jinv22*w56+1.9480519480519481e-04*Jinv12*w28-3.4699675324675324e-04*w25*Jinv11-2.1915584415584417e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-3.4699675324675324e-04*w47*Jinv20+1.8262987012987013e-03*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+4.2613636363636367e-05*w10*Jinv11)*D;
  A[35][3]=(0.0)*D;
  A[35][4]=(0.0)*D;
  A[35][5]=(5.4788961038961036e-04*Jinv00*w12-6.9399350649350649e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-1.4793019480519481e-03*Jinv20*w8-1.0592532467532468e-03*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-4.9310064935064932e-04*w3*Jinv00+1.0957792207792207e-03*w32*Jinv20-4.7666396103896100e-03*Jinv01*w3+1.2053571428571428e-03*Jinv20*w23+1.6436688311688311e-03*Jinv22*w38+3.2873376623376621e-04*Jinv00*w54-1.0592532467532468e-03*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-2.3011363636363637e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-6.9034090909090911e-03*w16*Jinv11-6.9034090909090911e-03*Jinv21*w17+8.7662337662337668e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.6615259740259744e-04*w58*Jinv10-6.9034090909090911e-03*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+6.0267857142857139e-04*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-1.3806818181818182e-02*Jinv22*w35-1.0592532467532468e-03*w58*Jinv11-1.0592532467532468e-03*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+8.2183441558441553e-04*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-6.9399350649350649e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-6.9034090909090911e-03*w15*Jinv01-9.6793831168831166e-04*w28*Jinv11+6.0267857142857139e-04*Jinv20*w50-6.9399350649350649e-04*Jinv20*w29+8.2183441558441553e-04*w46*Jinv10-6.9034090909090911e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33+2.5202922077922079e-03*w22*Jinv12+1.0957792207792207e-03*w31*Jinv10-4.7666396103896100e-03*Jinv02*w3+1.2053571428571428e-03*w21*Jinv00-3.1777597402597403e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+4.3831168831168834e-04*Jinv00*w18+8.7662337662337668e-04*w36*Jinv00+1.6436688311688311e-03*Jinv21*w38+8.7662337662337668e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24-1.0592532467532468e-03*Jinv22*w59-1.3806818181818182e-02*w34*Jinv11+8.2183441558441553e-04*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.3697240259740260e-03*w51*Jinv00-1.3806818181818182e-02*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-4.9310064935064932e-04*w4*Jinv10-4.7666396103896100e-03*w4*Jinv11-3.6160714285714286e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+4.3831168831168834e-04*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10-3.1777597402597403e-03*w32*Jinv21+8.2183441558441553e-04*Jinv21*w20-1.4793019480519481e-03*w6*Jinv00+1.6436688311688311e-03*w37*Jinv12+3.2873376623376621e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+1.2601461038961039e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-1.3806818181818182e-02*Jinv21*w35-3.1777597402597403e-03*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+8.7662337662337668e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14-4.7666396103896100e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-4.7666396103896100e-03*w4*Jinv12+2.5202922077922079e-03*w21*Jinv02+6.0267857142857139e-04*w49*Jinv10-9.6793831168831166e-04*Jinv21*w29-1.3806818181818182e-02*Jinv01*w33-3.6160714285714286e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+8.2183441558441553e-04*Jinv01*w18+1.6436688311688311e-03*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+6.5746753246753243e-04*w6*Jinv01-1.4793019480519481e-03*w7*Jinv10+8.7662337662337668e-04*w47*Jinv21+8.7662337662337668e-04*Jinv20*w38+2.5202922077922079e-03*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-3.1777597402597403e-03*w31*Jinv12+8.7662337662337668e-04*w46*Jinv12+1.3697240259740260e-03*Jinv20*w53+6.5746753246753243e-04*Jinv22*w8+1.2053571428571428e-03*w22*Jinv10+2.7394480519480520e-03*Jinv22*w50+2.5202922077922079e-03*w22*Jinv11+6.5746753246753243e-04*Jinv12*w7+2.5202922077922079e-03*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-3.1777597402597403e-03*Jinv02*w30+2.7394480519480520e-03*Jinv12*w49+1.6436688311688311e-03*w36*Jinv01+1.3697240259740260e-03*w52*Jinv10+6.5746753246753243e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+8.7662337662337668e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+1.2601461038961039e-03*w24*Jinv00-9.6793831168831166e-04*Jinv02*w27+6.5746753246753243e-04*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12-4.7666396103896100e-03*Jinv22*w5+8.7662337662337668e-04*w47*Jinv22+2.7394480519480520e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-2.3011363636363637e-03*Jinv20*w44-1.0592532467532468e-03*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+6.5746753246753243e-04*Jinv21*w8-5.6615259740259744e-04*w57*Jinv00-5.6615259740259744e-04*Jinv20*w59-1.3806818181818182e-02*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20+2.5202922077922079e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+1.2601461038961039e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+2.7394480519480520e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17+1.0957792207792207e-03*w30*Jinv00-3.6160714285714286e-03*w40*Jinv10+4.3831168831168834e-04*w20*Jinv20-9.6793831168831166e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+1.6436688311688311e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-4.9310064935064932e-04*w5*Jinv20-3.1777597402597403e-03*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-9.6793831168831166e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11+3.2873376623376621e-04*w55*Jinv10-2.3011363636363637e-03*w42*Jinv00+8.2183441558441553e-04*w47*Jinv20-6.9034090909090911e-03*w16*Jinv12+2.7394480519480520e-03*Jinv21*w50-9.6793831168831166e-04*Jinv01*w27+2.7394480519480520e-03*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[35][6]=(0.0)*D;
  A[35][7]=(0.0)*D;
  A[35][8]=(-4.3831168831168834e-04*Jinv00*w12+6.9399350649350649e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53+4.9310064935064932e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-4.3831168831168834e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+1.4793019480519481e-03*w3*Jinv00-8.7662337662337668e-04*w32*Jinv20+2.1367694805194807e-03*Jinv01*w3-1.2601461038961039e-03*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38-3.2873376623376621e-04*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2+3.6160714285714286e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+3.8352272727272725e-04*Jinv12*w55+5.6615259740259744e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-1.3697240259740260e-03*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-8.2183441558441553e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+6.9399350649350649e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11-1.3697240259740260e-03*Jinv20*w50+6.9399350649350649e-04*Jinv20*w29-8.2183441558441553e-04*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-8.7662337662337668e-04*w31*Jinv10+2.1367694805194807e-03*Jinv02*w3-1.2601461038961039e-03*w21*Jinv00+7.6704545454545450e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-1.0957792207792207e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53-6.0267857142857139e-04*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+1.4793019480519481e-03*w4*Jinv10+2.1367694805194807e-03*w4*Jinv11+2.3011363636363637e-03*w41*Jinv20+3.8352272727272725e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10+7.6704545454545450e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20+4.9310064935064932e-04*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12-3.2873376623376621e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.2053571428571428e-03*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+7.6704545454545450e-04*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-4.3831168831168834e-04*Jinv20*w14+2.1367694805194807e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54+2.1367694805194807e-03*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-1.3697240259740260e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+2.3011363636363637e-03*w39*Jinv00+3.8352272727272725e-04*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01+4.9310064935064932e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21-1.0957792207792207e-03*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+7.6704545454545450e-04*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-6.0267857142857139e-04*Jinv20*w53-4.2735389610389614e-03*Jinv22*w8-1.2601461038961039e-03*w22*Jinv10+7.6704545454545450e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-4.2735389610389614e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+7.6704545454545450e-04*Jinv02*w30+7.6704545454545450e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01-6.0267857142857139e-04*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00-1.0957792207792207e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-1.2053571428571428e-03*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27-4.2735389610389614e-03*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12+2.1367694805194807e-03*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+7.6704545454545450e-04*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10+3.6160714285714286e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8+5.6615259740259744e-04*w57*Jinv00+5.6615259740259744e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-1.2053571428571428e-03*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+7.6704545454545450e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+5.9172077922077923e-03*Jinv20*w17-8.7662337662337668e-04*w30*Jinv00+2.3011363636363637e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+1.4793019480519481e-03*w5*Jinv20+7.6704545454545450e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11-3.2873376623376621e-04*w55*Jinv10+3.6160714285714286e-03*w42*Jinv00-8.2183441558441553e-04*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+7.6704545454545450e-04*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27+7.6704545454545450e-04*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[35][9]=(0.0)*D;
  A[35][10]=(0.0)*D;
  A[35][11]=(2.1915584415584417e-04*Jinv00*w12-1.9480519480519481e-04*w28*Jinv10+6.5746753246753243e-04*Jinv20*w8+2.1915584415584417e-04*w13*Jinv10-3.2873376623376621e-04*w3*Jinv00+4.3831168831168834e-04*w32*Jinv20+3.4699675324675324e-04*Jinv20*w23+2.1915584415584417e-04*Jinv00*w54-1.8262987012987013e-03*w43*Jinv10-4.2613636363636367e-05*w58*Jinv10+4.3831168831168834e-04*w48*Jinv00-4.2613636363636367e-05*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10+3.4699675324675324e-04*w45*Jinv00-1.9480519480519481e-04*Jinv00*w27+4.3831168831168834e-04*Jinv20*w50-1.9480519480519481e-04*Jinv20*w29+3.4699675324675324e-04*w46*Jinv10-3.6525974025974025e-03*Jinv00*w33+4.3831168831168834e-04*w31*Jinv10+3.4699675324675324e-04*w21*Jinv00+3.2873376623376621e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-3.2873376623376621e-04*w51*Jinv00-3.2873376623376621e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+3.2873376623376621e-04*w19*Jinv10-3.6525974025974025e-03*w34*Jinv10+6.5746753246753243e-04*w6*Jinv00+2.1915584415584417e-04*Jinv20*w56+2.9220779220779221e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+2.1915584415584417e-04*Jinv20*w14+4.3831168831168834e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+6.5746753246753243e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-3.2873376623376621e-04*Jinv20*w53+3.4699675324675324e-04*w22*Jinv10-3.2873376623376621e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10+2.9220779220779221e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-1.8262987012987013e-03*Jinv20*w44-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-4.2613636363636367e-05*w0*Jinv00-3.6525974025974025e-03*w35*Jinv20+2.9220779220779221e-04*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00-1.8262987012987013e-03*Jinv20*w17+4.3831168831168834e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-3.2873376623376621e-04*w5*Jinv20+2.1915584415584417e-04*w55*Jinv10-1.8262987012987013e-03*w42*Jinv00+3.4699675324675324e-04*w47*Jinv20)*D;
  A[35][12]=(0.0)*D;
  A[35][13]=(0.0)*D;
  A[35][14]=(-1.5888798701298701e-03*Jinv00*w12-5.2962662337662338e-04*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8-5.2962662337662338e-04*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-1.5888798701298701e-03*w32*Jinv20+2.6846590909090908e-03*Jinv01*w3+1.3697240259740260e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02-1.6436688311688311e-04*Jinv21*w2-6.5746753246753243e-04*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21-3.6160714285714286e-03*w16*Jinv11-3.6160714285714286e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-4.6022727272727274e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12+1.3697240259740260e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.6436688311688311e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-3.6160714285714286e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22-1.6436688311688311e-04*Jinv11*w1+5.4788961038961036e-04*w13*Jinv11-3.6160714285714286e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+1.3697240259740260e-03*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-4.6022727272727274e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+1.3697240259740260e-03*w22*Jinv12-1.5888798701298701e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3+1.3697240259740260e-03*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-3.6160714285714286e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+6.5746753246753243e-04*Jinv01*w24-5.2962662337662338e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+3.2873376623376621e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10+2.6846590909090908e-03*w4*Jinv11-4.6022727272727274e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+2.6846590909090908e-03*w32*Jinv21+3.2873376623376621e-04*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-5.2962662337662338e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+2.6846590909090908e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+6.5746753246753243e-04*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14+2.6846590909090908e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+1.3697240259740260e-03*w21*Jinv02+1.3697240259740260e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-4.6022727272727274e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+3.2873376623376621e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+6.5746753246753243e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+1.3697240259740260e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+1.3697240259740260e-03*Jinv22*w8+1.3697240259740260e-03*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11-5.2962662337662338e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+3.2873376623376621e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+1.3697240259740260e-03*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-7.6704545454545450e-04*w49*Jinv11+5.4788961038961036e-04*Jinv21*w14-5.2962662337662338e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-5.2962662337662338e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+5.4788961038961036e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+3.2873376623376621e-04*Jinv01*w54+3.2873376623376621e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-1.5888798701298701e-03*w30*Jinv00-4.6022727272727274e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-3.6160714285714286e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20+2.6846590909090908e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+6.5746753246753243e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-4.6022727272727274e-03*w16*Jinv12-7.6704545454545450e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[35][15]=(0.0)*D;
  A[35][16]=(0.0)*D;
  A[35][17]=(2.3011363636363637e-03*Jinv00*w12-7.8896103896103891e-03*w40*Jinv12+1.9724025974025973e-03*Jinv22*w53-2.9586038961038961e-03*Jinv20*w8-1.1323051948051949e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10-7.8896103896103891e-03*Jinv02*w39+2.9586038961038961e-03*w3*Jinv00+2.3011363636363637e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3-6.5746753246753243e-04*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38+1.1323051948051949e-03*Jinv21*w59+1.9724025974025973e-03*w25*Jinv12-1.1323051948051949e-03*w0*Jinv02-6.9399350649350649e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+2.1915584415584417e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17-1.9724025974025973e-03*Jinv01*w45+1.3149350649350649e-03*Jinv12*w55-1.1834415584415585e-02*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-6.5746753246753243e-04*w48*Jinv00+1.9724025974025973e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35+1.1323051948051949e-03*w58*Jinv11+1.1323051948051949e-03*w57*Jinv01-2.9586038961038961e-03*Jinv21*w26-6.9399350649350649e-04*w0*Jinv01-9.1314935064935063e-04*w1*Jinv10-9.8620129870129864e-04*Jinv22*w20+3.9448051948051946e-03*Jinv21*w41-1.1834415584415585e-02*w42*Jinv02-2.1915584415584417e-04*w11*Jinv22-6.9399350649350649e-04*Jinv11*w1+2.3011363636363637e-03*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01+1.6436688311688311e-03*w28*Jinv11-6.5746753246753243e-04*Jinv20*w50-1.1834415584415585e-02*Jinv22*w17+1.3149350649350649e-03*w22*Jinv12+2.3011363636363637e-03*w31*Jinv10+1.9724025974025973e-03*Jinv02*w3-6.5746753246753243e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22+3.9448051948051946e-03*Jinv01*w39-1.1323051948051949e-03*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-2.3011363636363637e-03*w36*Jinv00+9.8620129870129864e-04*Jinv21*w38+1.3149350649350649e-03*Jinv02*w45-2.9586038961038961e-03*Jinv01*w24-1.1323051948051949e-03*Jinv22*w59+2.3668831168831169e-02*w34*Jinv11-1.9724025974025973e-03*Jinv21*w53+6.5746753246753243e-04*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-1.1834415584415585e-02*Jinv22*w44+2.9586038961038961e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11+3.9448051948051946e-03*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+3.2873376623376621e-03*w32*Jinv21-2.9586038961038961e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12-7.8896103896103891e-03*Jinv22*w41+6.5746753246753243e-04*w26*Jinv20+9.1314935064935063e-04*w11*Jinv20+1.9724025974025973e-03*w52*Jinv12+2.3668831168831169e-02*Jinv21*w35+3.2873376623376621e-03*w31*Jinv11+1.9724025974025973e-03*Jinv22*w26-1.9724025974025973e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21+1.3149350649350649e-03*Jinv02*w54+1.9724025974025973e-03*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02-6.5746753246753243e-04*w49*Jinv10+1.6436688311688311e-03*Jinv21*w29+2.3668831168831169e-02*Jinv01*w33+3.9448051948051946e-03*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+9.8620129870129864e-04*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+1.9724025974025973e-03*w6*Jinv01-2.9586038961038961e-03*w7*Jinv10-1.9724025974025973e-03*w47*Jinv21-2.3011363636363637e-03*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-1.9724025974025973e-03*w52*Jinv11+1.3149350649350649e-03*w31*Jinv12+1.3149350649350649e-03*w46*Jinv12+6.5746753246753243e-04*Jinv20*w53-9.8620129870129864e-04*Jinv22*w8-6.5746753246753243e-04*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-3.6160714285714286e-03*w22*Jinv11-9.8620129870129864e-04*Jinv12*w7-3.6160714285714286e-03*w21*Jinv01-1.9724025974025973e-03*Jinv01*w51+1.3149350649350649e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49+9.8620129870129864e-04*w36*Jinv01+6.5746753246753243e-04*w52*Jinv10+1.9724025974025973e-03*w7*Jinv11+9.1314935064935063e-04*w9*Jinv00-2.3011363636363637e-03*w37*Jinv10+6.5746753246753243e-04*w24*Jinv00-1.1323051948051949e-03*Jinv02*w27-9.8620129870129864e-04*w6*Jinv02-9.8620129870129864e-04*w19*Jinv12+1.9724025974025973e-03*Jinv22*w5+1.3149350649350649e-03*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11+2.3011363636363637e-03*Jinv21*w14+9.1314935064935063e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44-1.1323051948051949e-03*w58*Jinv12-9.8620129870129864e-04*Jinv02*w18+2.3011363636363637e-03*Jinv01*w12+1.9724025974025973e-03*Jinv21*w8-1.1834415584415585e-02*Jinv02*w33-9.1314935064935063e-04*w0*Jinv00-3.6160714285714286e-03*Jinv21*w23+6.5746753246753243e-04*w25*Jinv10-9.1314935064935063e-04*Jinv20*w2+1.3149350649350649e-03*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+2.3011363636363637e-03*w30*Jinv00+3.9448051948051946e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20-1.1323051948051949e-03*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14+3.9448051948051946e-03*w40*Jinv11-9.8620129870129864e-04*Jinv02*w36-1.1834415584415585e-02*Jinv12*w43+2.9586038961038961e-03*w5*Jinv20+3.2873376623376621e-03*Jinv01*w30+1.9724025974025973e-03*Jinv02*w51+1.3149350649350649e-03*Jinv22*w56-1.1323051948051949e-03*Jinv12*w28-2.9586038961038961e-03*w25*Jinv11-3.9448051948051946e-03*w42*Jinv00-1.1834415584415585e-02*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50+1.6436688311688311e-03*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-1.1323051948051949e-03*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[35][18]=(0.0)*D;
  A[35][19]=(0.0)*D;
  A[35][20]=(-7.1225649350649346e-04*Jinv00*w12+5.2962662337662338e-04*w28*Jinv10+1.5888798701298701e-03*Jinv20*w8-7.1225649350649346e-04*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-2.1367694805194807e-03*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23-7.1225649350649346e-04*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59+4.9310064935064932e-04*Jinv21*w2+4.6022727272727274e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45+5.2962662337662338e-04*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+5.2962662337662338e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+5.2962662337662338e-04*Jinv00*w27+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+5.2962662337662338e-04*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+1.5888798701298701e-03*Jinv00*w18+1.5888798701298701e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-1.3149350649350649e-03*Jinv01*w24+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-1.3697240259740260e-03*w51*Jinv00-1.3697240259740260e-03*w4*Jinv10-2.1367694805194807e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+1.5888798701298701e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-7.6704545454545450e-04*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.5888798701298701e-03*w6*Jinv00-7.1225649350649346e-04*Jinv20*w56-1.3697240259740260e-03*w26*Jinv20-3.8352272727272725e-04*w11*Jinv20+1.9724025974025973e-03*Jinv21*w35-7.6704545454545450e-04*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-2.1367694805194807e-03*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+2.1367694805194807e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11+4.2735389610389614e-03*w6*Jinv01+1.5888798701298701e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.5888798701298701e-03*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-1.3697240259740260e-03*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+4.2735389610389614e-03*w36*Jinv01-1.3697240259740260e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+1.5888798701298701e-03*w37*Jinv10-3.8352272727272725e-04*Jinv21*w56-1.3697240259740260e-03*w24*Jinv00-7.6704545454545450e-04*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10+4.6022727272727274e-03*Jinv20*w44-3.8352272727272725e-04*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+5.2962662337662338e-04*w57*Jinv00+5.2962662337662338e-04*Jinv20*w59+5.2962662337662338e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-3.8352272727272725e-04*Jinv01*w54-3.8352272727272725e-04*w55*Jinv11-1.3697240259740260e-03*w25*Jinv10+5.2962662337662338e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+4.6022727272727274e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-1.3697240259740260e-03*w5*Jinv20-7.6704545454545450e-04*Jinv01*w30-1.3149350649350649e-03*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10+4.6022727272727274e-03*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20-7.6704545454545450e-04*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27-7.6704545454545450e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[35][21]=(0.0)*D;
  A[35][22]=(0.0)*D;
  A[35][23]=(3.8352272727272725e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02+3.8352272727272725e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-2.5750811688311690e-03*Jinv01*w3-1.0957792207792207e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+3.8352272727272725e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+1.3149350649350649e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-3.6525974025974026e-05*w11*Jinv21+2.3011363636363637e-03*w16*Jinv11+2.3011363636363637e-03*Jinv21*w17+3.8352272727272725e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+5.4788961038961042e-05*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+3.2873376623376621e-03*Jinv22*w35-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01+2.5568181818181820e-04*w0*Jinv01+3.6525974025974026e-05*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+3.8352272727272725e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-1.0957792207792207e-03*w45*Jinv00+2.3011363636363637e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11+2.3011363636363637e-03*w15*Jinv01-1.8262987012987014e-04*w28*Jinv11+5.4788961038961042e-05*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-1.0957792207792207e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+3.2873376623376621e-03*Jinv00*w33-1.0957792207792207e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.0957792207792207e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+2.3011363636363637e-03*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-5.4788961038961042e-05*Jinv21*w38-1.0957792207792207e-03*Jinv02*w45+3.6525974025974026e-05*Jinv22*w59-6.5746753246753243e-03*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11-5.4788961038961042e-05*Jinv21*w53+5.4788961038961042e-05*w51*Jinv00+3.2873376623376621e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-2.5750811688311690e-03*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+3.8352272727272725e-04*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10+3.2873376623376621e-03*w34*Jinv10-2.5750811688311690e-03*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+3.8352272727272725e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20+3.6525974025974026e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12-6.5746753246753243e-03*Jinv21*w35-2.5750811688311690e-03*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26+3.8352272727272725e-04*Jinv20*w14-2.5750811688311690e-03*w5*Jinv21+3.8352272727272725e-04*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-1.0957792207792207e-03*w21*Jinv02+5.4788961038961042e-05*w49*Jinv10-1.8262987012987014e-04*Jinv21*w29-6.5746753246753243e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+3.8352272727272725e-04*Jinv02*w12-3.2873376623376621e-04*Jinv01*w18-5.4788961038961042e-05*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+5.4788961038961042e-05*Jinv20*w38-1.0957792207792207e-03*Jinv22*w23-5.4788961038961042e-05*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-1.0957792207792207e-03*w46*Jinv12+5.4788961038961042e-05*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-1.0957792207792207e-03*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+5.4788961038961036e-04*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+5.4788961038961036e-04*w21*Jinv01-5.4788961038961042e-05*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-5.4788961038961042e-05*w36*Jinv01+5.4788961038961042e-05*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11+3.6525974025974026e-05*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-3.2873376623376621e-04*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+3.8352272727272725e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-1.0957792207792207e-03*w47*Jinv22-1.0957792207792208e-04*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14+3.6525974025974026e-05*w10*Jinv10+1.3149350649350649e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59+3.2873376623376621e-03*Jinv02*w33+3.6525974025974026e-05*w0*Jinv00+3.2873376623376621e-03*w35*Jinv20+5.4788961038961036e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10+3.6525974025974026e-05*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29+3.8352272727272725e-04*Jinv22*w14+2.3011363636363637e-03*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-2.5750811688311690e-03*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+3.8352272727272725e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28+3.8352272727272725e-04*w55*Jinv10+1.3149350649350649e-03*w42*Jinv00-1.0957792207792207e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-1.0957792207792208e-04*Jinv21*w50-1.8262987012987014e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[35][24]=(0.0)*D;
  A[35][25]=(0.0)*D;
  A[35][26]=(-3.8352272727272725e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-3.8352272727272725e-04*w13*Jinv10+9.8620129870129864e-04*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23-3.8352272727272725e-04*Jinv00*w54-7.3051948051948052e-05*Jinv21*w59-7.3051948051948052e-05*Jinv21*w2-1.3149350649350649e-03*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17+1.0957792207792207e-03*Jinv01*w45-3.6525974025974026e-05*w58*Jinv10-5.4788961038961042e-05*w48*Jinv00-7.3051948051948052e-05*w58*Jinv11-7.3051948051948052e-05*w57*Jinv01+1.6436688311688311e-03*Jinv21*w26-7.3051948051948052e-05*w0*Jinv01-3.6525974025974026e-05*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+9.8620129870129864e-04*w43*Jinv11+1.0957792207792207e-03*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41-4.9310064935064932e-04*Jinv00*w27-7.3051948051948052e-05*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01-6.7573051948051951e-04*w28*Jinv11-5.4788961038961042e-05*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+1.0957792207792207e-03*w46*Jinv10-3.2873376623376621e-03*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10+1.0957792207792207e-03*w21*Jinv00+6.5746753246753243e-04*Jinv01*w39-3.8352272727272725e-04*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+1.6436688311688311e-03*Jinv01*w24-9.8620129870129868e-03*w34*Jinv11-8.2183441558441553e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-5.4788961038961042e-05*w51*Jinv00-5.4788961038961042e-05*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11-1.3149350649350649e-03*w41*Jinv20-3.8352272727272725e-04*w19*Jinv10-3.2873376623376621e-03*w34*Jinv10-1.0957792207792208e-04*w32*Jinv21-8.2183441558441553e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*Jinv20*w56+1.0957792207792207e-03*w26*Jinv20-3.6525974025974026e-05*w11*Jinv20-9.8620129870129868e-03*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+1.0957792207792207e-03*w46*Jinv11-3.8352272727272725e-04*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21-5.4788961038961042e-05*w49*Jinv10-6.7573051948051951e-04*Jinv21*w29-9.8620129870129868e-03*Jinv01*w33-1.3149350649350649e-03*w39*Jinv00-8.2183441558441553e-04*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.0957792207792207e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11-5.4788961038961042e-05*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10+1.0957792207792207e-03*w22*Jinv11+1.0957792207792207e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-2.6298701298701297e-03*w36*Jinv01-5.4788961038961042e-05*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11-3.6525974025974026e-05*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56+1.0957792207792207e-03*w24*Jinv00-1.0957792207792208e-04*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-3.6525974025974026e-05*w10*Jinv10-1.3149350649350649e-03*Jinv20*w44-7.1225649350649346e-04*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-3.6525974025974026e-05*w0*Jinv00-3.2873376623376621e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11+1.0957792207792207e-03*w25*Jinv10-3.6525974025974026e-05*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+6.5746753246753243e-04*w40*Jinv11-5.4788961038961042e-05*w5*Jinv20-1.0957792207792208e-04*Jinv01*w30+1.6436688311688311e-03*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10-1.3149350649350649e-03*w42*Jinv00+1.0957792207792207e-03*w47*Jinv20-1.0957792207792208e-04*Jinv21*w50-6.7573051948051951e-04*Jinv01*w27-1.0957792207792208e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[35][27]=(0.0)*D;
  A[35][28]=(0.0)*D;
  A[35][29]=(-5.1136363636363640e-04*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3+3.0438311688311688e-05*Jinv21*w59+3.0438311688311688e-05*Jinv21*w2-5.1136363636363640e-04*w42*Jinv01+3.0438311688311688e-05*w11*Jinv21-5.1136363636363640e-04*w16*Jinv11-5.1136363636363640e-04*Jinv21*w17-3.1047077922077924e-04*Jinv01*w45+3.0438311688311688e-05*w58*Jinv11+3.0438311688311688e-05*w57*Jinv01-3.1047077922077924e-04*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01-5.1136363636363640e-04*w43*Jinv11-5.1136363636363640e-04*Jinv21*w41+3.0438311688311688e-05*Jinv11*w1+2.1915584415584417e-04*w13*Jinv11-5.1136363636363640e-04*w15*Jinv01+2.4350649350649351e-04*w28*Jinv11-5.1136363636363640e-04*Jinv01*w39+5.4788961038961036e-04*Jinv21*w38-3.1047077922077924e-04*Jinv01*w24+5.5519480519480519e-03*w34*Jinv11+2.1915584415584417e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+5.4788961038961036e-04*w4*Jinv11+5.4788961038961036e-04*w32*Jinv21+2.1915584415584417e-04*Jinv21*w20+5.5519480519480519e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-3.1047077922077924e-04*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21+2.4350649350649351e-04*Jinv21*w29+5.5519480519480519e-03*Jinv01*w33+2.1915584415584417e-04*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01-3.1047077922077924e-04*w47*Jinv21+5.4788961038961036e-04*w52*Jinv11-3.1047077922077924e-04*w22*Jinv11-3.1047077922077924e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+5.4788961038961036e-04*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11+2.1915584415584417e-04*Jinv21*w56+5.4788961038961036e-04*w49*Jinv11+2.1915584415584417e-04*Jinv21*w14+2.1915584415584417e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-3.1047077922077924e-04*Jinv21*w23+2.1915584415584417e-04*Jinv01*w54+2.1915584415584417e-04*w55*Jinv11+3.0438311688311688e-05*Jinv01*w9-5.1136363636363640e-04*w40*Jinv11+5.4788961038961036e-04*Jinv01*w30-3.1047077922077924e-04*w25*Jinv11+5.4788961038961036e-04*Jinv21*w50+2.4350649350649351e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+3.0438311688311688e-05*w10*Jinv11)*D;
  A[35][30]=(0.0)*D;
  A[35][31]=(0.0)*D;
  A[35][32]=(-1.5888798701298701e-03*Jinv00*w12-9.6793831168831166e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12+8.7662337662337668e-04*Jinv22*w53+2.7394480519480520e-03*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-1.5888798701298701e-03*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-3.1777597402597403e-03*w3*Jinv00-4.7666396103896100e-03*w32*Jinv20-3.1777597402597403e-03*Jinv01*w3+2.5202922077922079e-03*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-1.0592532467532468e-03*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45+4.3831168831168834e-04*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-5.6615259740259744e-04*w10*Jinv12+6.5746753246753243e-04*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+8.7662337662337668e-04*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-6.9034090909090911e-03*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-9.6793831168831166e-04*Jinv00*w27-5.6615259740259744e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-9.6793831168831166e-04*w28*Jinv11+6.5746753246753243e-04*Jinv20*w50-9.6793831168831166e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-1.3806818181818182e-02*Jinv00*w33+1.2053571428571428e-03*w22*Jinv12-4.7666396103896100e-03*w31*Jinv10+1.0957792207792207e-03*Jinv02*w3+2.5202922077922079e-03*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22-6.9034090909090911e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.2601461038961039e-03*Jinv02*w45+8.7662337662337668e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-1.3806818181818182e-02*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-03*Jinv21*w53+1.6436688311688311e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-3.1777597402597403e-03*w4*Jinv10-3.1777597402597403e-03*w4*Jinv11-6.9034090909090911e-03*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-1.3806818181818182e-02*w34*Jinv10-4.7666396103896100e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+2.7394480519480520e-03*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41+8.7662337662337668e-04*w26*Jinv20-1.0592532467532468e-03*w11*Jinv20+8.7662337662337668e-04*w52*Jinv12-1.3806818181818182e-02*Jinv21*w35-4.7666396103896100e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-1.5888798701298701e-03*Jinv20*w14-3.1777597402597403e-03*w5*Jinv21+4.3831168831168834e-04*Jinv02*w54+1.0957792207792207e-03*w4*Jinv12+1.2053571428571428e-03*w21*Jinv02+6.5746753246753243e-04*w49*Jinv10-9.6793831168831166e-04*Jinv21*w29-1.3806818181818182e-02*Jinv01*w33-6.9034090909090911e-03*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.6615259740259744e-04*Jinv02*w9+2.7394480519480520e-03*w6*Jinv01+2.7394480519480520e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38+1.2053571428571428e-03*Jinv22*w23+1.6436688311688311e-03*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+1.2601461038961039e-03*w46*Jinv12+1.6436688311688311e-03*Jinv20*w53+6.0267857142857139e-04*Jinv22*w8+2.5202922077922079e-03*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+2.5202922077922079e-03*w22*Jinv11+6.0267857142857139e-04*Jinv12*w7+2.5202922077922079e-03*w21*Jinv01+1.6436688311688311e-03*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+1.6436688311688311e-03*w52*Jinv10+2.7394480519480520e-03*w7*Jinv11-1.0592532467532468e-03*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10+8.2183441558441553e-04*Jinv21*w56+8.7662337662337668e-04*w24*Jinv00-6.9399350649350649e-04*Jinv02*w27+6.0267857142857139e-04*w6*Jinv02+3.2873376623376621e-04*w19*Jinv12+1.0957792207792207e-03*Jinv22*w5+1.2601461038961039e-03*w47*Jinv22+6.5746753246753243e-04*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-1.0592532467532468e-03*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+3.2873376623376621e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+2.7394480519480520e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-1.1834415584415585e-02*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-1.3806818181818182e-02*w35*Jinv20+2.5202922077922079e-03*Jinv21*w23+8.2183441558441553e-04*Jinv01*w54+8.2183441558441553e-04*w55*Jinv11+8.7662337662337668e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-1.0592532467532468e-03*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-4.7666396103896100e-03*w30*Jinv00-6.9034090909090911e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-6.9399350649350649e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-6.9034090909090911e-03*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-3.1777597402597403e-03*w5*Jinv20-4.7666396103896100e-03*Jinv01*w30+8.7662337662337668e-04*Jinv02*w51+4.3831168831168834e-04*Jinv22*w56-6.9399350649350649e-04*Jinv12*w28+8.7662337662337668e-04*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+6.5746753246753243e-04*Jinv21*w50-9.6793831168831166e-04*Jinv01*w27+6.5746753246753243e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-1.0592532467532468e-03*w10*Jinv11)*D;
  A[35][33]=(0.0)*D;
  A[35][34]=(0.0)*D;
  A[35][35]=(2.3011363636363637e-03*Jinv00*w12-6.9034090909090911e-03*Jinv22*w53-5.9172077922077923e-03*Jinv20*w8+1.8262987012987013e-03*w57*Jinv02+2.3011363636363637e-03*w13*Jinv10-1.7751623376623376e-02*Jinv21*w44+5.9172077922077923e-03*w3*Jinv00+6.9034090909090911e-03*w32*Jinv20+3.9448051948051946e-03*Jinv01*w3-1.6436688311688311e-03*Jinv20*w23-9.8620129870129864e-04*Jinv22*w38-4.3831168831168834e-04*Jinv21*w59-1.8262987012987013e-03*w0*Jinv02-2.2646103896103898e-03*Jinv21*w2-5.9172077922077923e-03*w43*Jinv10-1.7751623376623376e-02*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+4.9310064935064934e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55+5.9172077922077923e-03*w15*Jinv02+9.8620129870129864e-04*w48*Jinv00-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+4.9310064935064934e-03*Jinv21*w26-2.2646103896103898e-03*w0*Jinv01-1.8262987012987013e-03*w1*Jinv10-1.7751623376623376e-02*w43*Jinv11-1.1834415584415585e-02*Jinv21*w41-5.9172077922077923e-03*w42*Jinv02-2.2646103896103898e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-2.7759740259740259e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+5.9172077922077923e-03*Jinv22*w17-1.6436688311688311e-03*w22*Jinv12+6.9034090909090911e-03*w31*Jinv10+6.9034090909090911e-03*Jinv02*w3-1.6436688311688311e-03*w21*Jinv00+5.9172077922077923e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-1.8262987012987013e-03*Jinv22*w2-2.3011363636363637e-03*Jinv00*w18-6.9034090909090911e-03*w36*Jinv00-2.9586038961038961e-03*Jinv21*w38+1.6436688311688311e-03*Jinv02*w45+4.9310064935064934e-03*Jinv01*w24+1.8262987012987013e-03*Jinv22*w59-3.5503246753246752e-02*w34*Jinv11-9.8620129870129864e-04*w19*Jinv11-2.9586038961038961e-03*Jinv21*w53-9.8620129870129864e-04*w51*Jinv00-5.9172077922077923e-03*Jinv22*w44+5.9172077922077923e-03*w4*Jinv10+3.9448051948051946e-03*w4*Jinv11+5.9172077922077923e-03*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13-2.3011363636363637e-03*w19*Jinv10+3.9448051948051946e-03*w32*Jinv21-9.8620129870129864e-04*Jinv21*w20-5.9172077922077923e-03*w6*Jinv00-9.8620129870129864e-04*w37*Jinv12+1.6436688311688311e-03*w26*Jinv20+1.8262987012987013e-03*w11*Jinv20-6.9034090909090911e-03*w52*Jinv12-3.5503246753246752e-02*Jinv21*w35+3.9448051948051946e-03*w31*Jinv11+4.9310064935064934e-03*w46*Jinv11+2.3011363636363637e-03*Jinv20*w14+3.9448051948051946e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+6.9034090909090911e-03*w4*Jinv12-1.6436688311688311e-03*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10-2.7759740259740259e-03*Jinv21*w29-3.5503246753246752e-02*Jinv01*w33+5.9172077922077923e-03*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12-9.8620129870129864e-04*Jinv01*w18-2.9586038961038961e-03*w37*Jinv11-1.9724025974025973e-03*w6*Jinv01-5.9172077922077923e-03*w7*Jinv10+4.9310064935064934e-03*w47*Jinv21-6.9034090909090911e-03*Jinv20*w38-1.6436688311688311e-03*Jinv22*w23-2.9586038961038961e-03*w52*Jinv11+5.9172077922077923e-03*w31*Jinv12+1.6436688311688311e-03*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8-1.6436688311688311e-03*w22*Jinv10-5.9172077922077923e-03*Jinv22*w50+3.2873376623376621e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7+3.2873376623376621e-03*w21*Jinv01-2.9586038961038961e-03*Jinv01*w51+5.9172077922077923e-03*Jinv02*w30-5.9172077922077923e-03*Jinv12*w49-2.9586038961038961e-03*w36*Jinv01-9.8620129870129864e-04*w52*Jinv10-1.9724025974025973e-03*w7*Jinv11+1.8262987012987013e-03*w9*Jinv00-6.9034090909090911e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56+1.6436688311688311e-03*w24*Jinv00+9.8620129870129864e-04*w6*Jinv02+6.9034090909090911e-03*Jinv22*w5+1.6436688311688311e-03*w47*Jinv22-1.9724025974025973e-03*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+1.8262987012987013e-03*w10*Jinv10-5.9172077922077923e-03*Jinv20*w44+1.8262987012987013e-03*w58*Jinv12+1.3149350649350649e-03*Jinv01*w12-1.9724025974025973e-03*Jinv21*w8-1.8262987012987013e-03*w0*Jinv00+3.2873376623376621e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11+1.6436688311688311e-03*w25*Jinv10-1.8262987012987013e-03*Jinv20*w2-5.9172077922077923e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9+6.9034090909090911e-03*w30*Jinv00+5.9172077922077923e-03*w40*Jinv10-2.3011363636363637e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11-9.8620129870129864e-04*Jinv02*w36-5.9172077922077923e-03*Jinv12*w43+5.9172077922077923e-03*w5*Jinv20+3.9448051948051946e-03*Jinv01*w30-6.9034090909090911e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56+4.9310064935064934e-03*w25*Jinv11-5.9172077922077923e-03*w42*Jinv00+5.9172077922077923e-03*w16*Jinv12-1.9724025974025973e-03*Jinv21*w50-2.7759740259740259e-03*Jinv01*w27-1.9724025974025973e-03*w48*Jinv01-1.8262987012987013e-03*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[35][36]=(0.0)*D;
  A[35][37]=(0.0)*D;
  A[35][38]=(-7.1225649350649346e-04*Jinv00*w12+9.6793831168831166e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+3.1777597402597403e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-2.7394480519480520e-03*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-8.7662337662337668e-04*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-8.2183441558441553e-04*Jinv00*w54-1.3149350649350649e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+6.9034090909090911e-03*w43*Jinv10-3.8352272727272725e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-1.6436688311688311e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+1.0592532467532468e-03*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20-1.3149350649350649e-03*w45*Jinv00+9.8620129870129864e-04*w42*Jinv02+9.6793831168831166e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-1.6436688311688311e-03*Jinv20*w50+9.6793831168831166e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.3806818181818182e-02*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-8.7662337662337668e-04*w21*Jinv00-7.6704545454545450e-04*w32*Jinv22+4.9310064935064932e-04*Jinv22*w2+1.5888798701298701e-03*Jinv00*w18+4.7666396103896100e-03*w36*Jinv00-5.4788961038961042e-05*Jinv02*w45+4.9310064935064932e-04*Jinv22*w59-6.5746753246753243e-04*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-2.7394480519480520e-03*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-3.8352272727272725e-04*Jinv12*w13+1.5888798701298701e-03*w19*Jinv10+1.3806818181818182e-02*w34*Jinv10+3.1777597402597403e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-8.2183441558441553e-04*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-2.5202922077922079e-03*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12-1.3149350649350649e-03*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14-3.8352272727272725e-04*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-1.6436688311688311e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-3.8352272727272725e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+3.1777597402597403e-03*w7*Jinv10+4.7666396103896100e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23-7.6704545454545450e-04*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-6.5746753246753243e-04*Jinv20*w53+4.2735389610389614e-03*Jinv22*w8-8.7662337662337668e-04*w22*Jinv10-7.6704545454545450e-04*Jinv22*w50+4.2735389610389614e-03*Jinv12*w7-7.6704545454545450e-04*Jinv02*w30-7.6704545454545450e-04*Jinv12*w49-6.5746753246753243e-04*w52*Jinv10-7.6704545454545450e-04*w9*Jinv00+4.7666396103896100e-03*w37*Jinv10-2.5202922077922079e-03*w24*Jinv00+2.7394480519480518e-04*Jinv02*w27+4.2735389610389614e-03*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-7.6704545454545450e-04*w10*Jinv10+6.9034090909090911e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+1.0592532467532468e-03*w0*Jinv00+1.3806818181818182e-02*w35*Jinv20-2.5202922077922079e-03*w25*Jinv10+1.0592532467532468e-03*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-7.6704545454545450e-04*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+1.5888798701298701e-03*w20*Jinv20+2.7394480519480518e-04*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+4.2735389610389614e-03*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-2.7394480519480520e-03*w5*Jinv20-2.1367694805194807e-03*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56+2.7394480519480518e-04*Jinv12*w28-8.2183441558441553e-04*w55*Jinv10+6.9034090909090911e-03*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+4.9310064935064932e-04*Jinv12*w1)*D;
  A[35][39]=(0.0)*D;
  A[35][40]=(0.0)*D;
  A[35][41]=(1.3149350649350649e-03*Jinv00*w12-1.1323051948051949e-03*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+9.1314935064935063e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10+1.3149350649350649e-03*w3*Jinv00+1.9724025974025973e-03*w32*Jinv20+3.2873376623376621e-03*Jinv01*w3+1.3149350649350649e-03*Jinv20*w23+6.5746753246753243e-04*Jinv22*w38-9.8620129870129864e-04*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59-9.1314935064935063e-04*w0*Jinv02-6.9399350649350649e-04*Jinv21*w2-1.1834415584415585e-02*w43*Jinv10+1.1323051948051949e-03*w11*Jinv21+3.9448051948051946e-03*w16*Jinv11+3.9448051948051946e-03*Jinv21*w17-2.9586038961038961e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02-9.8620129870129864e-04*w48*Jinv00+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.9724025974025973e-03*Jinv21*w26-6.9399350649350649e-04*w0*Jinv01-1.1323051948051949e-03*w1*Jinv10-7.8896103896103891e-03*w16*Jinv10+1.9724025974025973e-03*w45*Jinv00+5.9172077922077923e-03*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02-1.1323051948051949e-03*Jinv00*w27-6.9399350649350649e-04*Jinv11*w1+2.3011363636363637e-03*w13*Jinv11+3.9448051948051946e-03*w15*Jinv01+1.6436688311688311e-03*w28*Jinv11-9.8620129870129864e-04*Jinv20*w50-1.1323051948051949e-03*Jinv20*w29+1.9724025974025973e-03*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33-6.5746753246753243e-04*w22*Jinv12+1.9724025974025973e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3+1.3149350649350649e-03*w21*Jinv00+2.9586038961038961e-03*w32*Jinv22+5.9172077922077923e-03*Jinv01*w39-9.1314935064935063e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+1.9724025974025973e-03*w36*Jinv00-1.9724025974025973e-03*Jinv21*w38+6.5746753246753243e-04*Jinv02*w45-1.9724025974025973e-03*Jinv01*w24+9.1314935064935063e-04*Jinv22*w59+2.3668831168831169e-02*w34*Jinv11+9.8620129870129864e-04*Jinv21*w53-9.8620129870129864e-04*w51*Jinv00-3.9448051948051946e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+3.2873376623376621e-03*w4*Jinv11-1.1834415584415585e-02*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21+1.3149350649350649e-03*w6*Jinv00+6.5746753246753243e-04*w37*Jinv12-9.8620129870129864e-04*Jinv20*w56+1.3149350649350649e-03*w26*Jinv20-1.1323051948051949e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12+2.3668831168831169e-02*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11-2.9586038961038961e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14+3.2873376623376621e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12-6.5746753246753243e-04*w21*Jinv02-9.8620129870129864e-04*w49*Jinv10+1.6436688311688311e-03*Jinv21*w29+2.3668831168831169e-02*Jinv01*w33-1.1834415584415585e-02*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12-1.9724025974025973e-03*w37*Jinv11-2.6298701298701297e-03*w6*Jinv01+1.3149350649350649e-03*w7*Jinv10-2.9586038961038961e-03*w47*Jinv21+1.9724025974025973e-03*Jinv20*w38-6.5746753246753243e-04*Jinv22*w23+9.8620129870129864e-04*w52*Jinv11+2.9586038961038961e-03*w31*Jinv12+6.5746753246753243e-04*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53-6.5746753246753243e-04*Jinv22*w8+1.3149350649350649e-03*w22*Jinv10-2.9586038961038961e-03*Jinv22*w50-3.6160714285714286e-03*w22*Jinv11-6.5746753246753243e-04*Jinv12*w7-3.6160714285714286e-03*w21*Jinv01+9.8620129870129864e-04*Jinv01*w51+2.9586038961038961e-03*Jinv02*w30-2.9586038961038961e-03*Jinv12*w49-1.9724025974025973e-03*w36*Jinv01-9.8620129870129864e-04*w52*Jinv10-2.6298701298701297e-03*w7*Jinv11-1.1323051948051949e-03*w9*Jinv00+1.9724025974025973e-03*w37*Jinv10+1.3149350649350649e-03*w24*Jinv00-6.5746753246753243e-04*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5+6.5746753246753243e-04*w47*Jinv22+1.9724025974025973e-03*w49*Jinv11+2.3011363636363637e-03*Jinv21*w14-1.1323051948051949e-03*w10*Jinv10-1.1834415584415585e-02*Jinv20*w44+9.1314935064935063e-04*w58*Jinv12+2.3011363636363637e-03*Jinv01*w12-2.6298701298701297e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-1.1323051948051949e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-3.6160714285714286e-03*Jinv21*w23+1.3149350649350649e-03*w25*Jinv10-1.1323051948051949e-03*Jinv20*w2-7.8896103896103891e-03*w15*Jinv00-2.9586038961038961e-03*w48*Jinv02+1.1323051948051949e-03*Jinv01*w9-7.8896103896103891e-03*Jinv20*w17+1.9724025974025973e-03*w30*Jinv00-1.1834415584415585e-02*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14+5.9172077922077923e-03*w40*Jinv11+6.5746753246753243e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56-1.9724025974025973e-03*w25*Jinv11-9.8620129870129864e-04*w55*Jinv10-1.1834415584415585e-02*w42*Jinv00+1.9724025974025973e-03*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12+1.9724025974025973e-03*Jinv21*w50+1.6436688311688311e-03*Jinv01*w27+1.9724025974025973e-03*w48*Jinv01-9.1314935064935063e-04*Jinv12*w1+1.1323051948051949e-03*w10*Jinv11)*D;
  A[35][42]=(0.0)*D;
  A[35][43]=(0.0)*D;
  A[35][44]=(-1.3149350649350649e-03*Jinv00*w12+1.1323051948051949e-03*w28*Jinv10+7.8896103896103891e-03*w40*Jinv12-1.9724025974025973e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+1.1323051948051949e-03*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+7.8896103896103891e-03*Jinv02*w39+1.7751623376623376e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-1.9724025974025973e-03*w32*Jinv20-3.9448051948051946e-03*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+9.8620129870129864e-04*Jinv22*w38+9.8620129870129864e-04*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59-1.9724025974025973e-03*w25*Jinv12+1.1323051948051949e-03*w0*Jinv02+2.2646103896103898e-03*Jinv21*w2+1.1834415584415585e-02*w43*Jinv10+1.7751623376623376e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+1.1834415584415585e-02*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+9.8620129870129864e-04*w48*Jinv00-1.9724025974025973e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26+2.2646103896103898e-03*w0*Jinv01+1.1323051948051949e-03*w1*Jinv10+7.8896103896103891e-03*w16*Jinv10+9.8620129870129864e-04*Jinv22*w20+1.7751623376623376e-02*w43*Jinv11-1.9724025974025973e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+1.1834415584415585e-02*w42*Jinv02+1.1323051948051949e-03*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+2.2646103896103898e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+2.7759740259740259e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+1.1323051948051949e-03*Jinv20*w29-1.9724025974025973e-03*w46*Jinv10+1.1834415584415585e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-1.9724025974025973e-03*w31*Jinv10-1.9724025974025973e-03*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+1.1323051948051949e-03*Jinv22*w2-1.3149350649350649e-03*Jinv00*w18-1.9724025974025973e-03*w36*Jinv00+2.9586038961038961e-03*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24+1.1323051948051949e-03*Jinv22*w59+3.5503246753246752e-02*w34*Jinv11+9.8620129870129864e-04*w19*Jinv11+2.9586038961038961e-03*Jinv21*w53+9.8620129870129864e-04*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+1.1834415584415585e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-3.9448051948051946e-03*w4*Jinv11+1.1834415584415585e-02*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-3.9448051948051946e-03*w32*Jinv21+9.8620129870129864e-04*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+9.8620129870129864e-04*w37*Jinv12+9.8620129870129864e-04*Jinv20*w56+7.8896103896103891e-03*Jinv22*w41-1.3149350649350649e-03*w26*Jinv20+1.1323051948051949e-03*w11*Jinv20-1.9724025974025973e-03*w52*Jinv12+3.5503246753246752e-02*Jinv21*w35-3.9448051948051946e-03*w31*Jinv11-1.9724025974025973e-03*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-3.9448051948051946e-03*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54-1.9724025974025973e-03*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10+2.7759740259740259e-03*Jinv21*w29+3.5503246753246752e-02*Jinv01*w33+1.1834415584415585e-02*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12+9.8620129870129864e-04*Jinv01*w18+2.9586038961038961e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9+1.9724025974025973e-03*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21-1.9724025974025973e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+2.9586038961038961e-03*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12+9.8620129870129864e-04*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-3.2873376623376621e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7-3.2873376623376621e-03*w21*Jinv01+2.9586038961038961e-03*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49+2.9586038961038961e-03*w36*Jinv01+9.8620129870129864e-04*w52*Jinv10+1.9724025974025973e-03*w7*Jinv11+1.1323051948051949e-03*w9*Jinv00-1.9724025974025973e-03*w37*Jinv10+9.8620129870129864e-04*Jinv21*w56-1.3149350649350649e-03*w24*Jinv00+1.1323051948051949e-03*Jinv02*w27+9.8620129870129864e-04*w6*Jinv02+9.8620129870129864e-04*w19*Jinv12-1.9724025974025973e-03*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22+1.9724025974025973e-03*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+1.1323051948051949e-03*w10*Jinv10+1.1834415584415585e-02*Jinv20*w44+1.1323051948051949e-03*w58*Jinv12+9.8620129870129864e-04*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12+1.9724025974025973e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+1.1323051948051949e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-3.2873376623376621e-03*Jinv21*w23+9.8620129870129864e-04*Jinv01*w54+9.8620129870129864e-04*w55*Jinv11-1.3149350649350649e-03*w25*Jinv10+1.1323051948051949e-03*Jinv20*w2+7.8896103896103891e-03*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+7.8896103896103891e-03*Jinv20*w17-1.9724025974025973e-03*w30*Jinv00+1.1834415584415585e-02*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20+1.1323051948051949e-03*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+9.8620129870129864e-04*Jinv02*w36+1.1834415584415585e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-3.9448051948051946e-03*Jinv01*w30-1.9724025974025973e-03*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56+1.1323051948051949e-03*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+9.8620129870129864e-04*w55*Jinv10+1.1834415584415585e-02*w42*Jinv00-1.9724025974025973e-03*w47*Jinv20+1.1834415584415585e-02*w16*Jinv12+1.9724025974025973e-03*Jinv21*w50+2.7759740259740259e-03*Jinv01*w27+1.9724025974025973e-03*w48*Jinv01+1.1323051948051949e-03*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[35][45]=(0.0)*D;
  A[35][46]=(0.0)*D;
  A[35][47]=(-1.3149350649350649e-03*w40*Jinv12-5.4788961038961042e-05*Jinv22*w53-3.6525974025974026e-05*w57*Jinv02-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-1.0957792207792208e-04*Jinv01*w3-5.4788961038961042e-05*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59+1.0957792207792207e-03*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02-7.3051948051948052e-05*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+1.6436688311688311e-03*Jinv01*w45-3.8352272727272725e-04*Jinv12*w55-1.3149350649350649e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+1.0957792207792207e-03*Jinv02*w24-3.2873376623376621e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+1.0957792207792207e-03*Jinv21*w26-7.3051948051948052e-05*w0*Jinv01-3.8352272727272725e-04*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*w11*Jinv22-7.3051948051948052e-05*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-6.7573051948051951e-04*w28*Jinv11-1.3149350649350649e-03*Jinv22*w17+1.0957792207792207e-03*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-3.6525974025974026e-05*Jinv22*w2-1.6436688311688311e-04*Jinv21*w38+1.0957792207792207e-03*Jinv02*w45+1.0957792207792207e-03*Jinv01*w24-3.6525974025974026e-05*Jinv22*w59-9.8620129870129868e-03*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-3.2873376623376621e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-1.0957792207792208e-04*w4*Jinv11-3.8352272727272725e-04*Jinv12*w13-1.6436688311688311e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-5.4788961038961042e-05*w37*Jinv12-1.3149350649350649e-03*Jinv22*w41-5.4788961038961042e-05*w52*Jinv12-9.8620129870129868e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11+1.0957792207792207e-03*Jinv22*w26+1.6436688311688311e-03*w46*Jinv11-1.0957792207792208e-04*w5*Jinv21-3.8352272727272725e-04*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+1.0957792207792207e-03*w21*Jinv02-6.7573051948051951e-04*Jinv21*w29-9.8620129870129868e-03*Jinv01*w33-3.8352272727272725e-04*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.6436688311688311e-03*w47*Jinv21+1.0957792207792207e-03*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+1.0957792207792207e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-8.2183441558441553e-04*Jinv21*w56-4.9310064935064932e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-3.8352272727272725e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+1.0957792207792207e-03*w47*Jinv22-2.6298701298701297e-03*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-3.6525974025974026e-05*w58*Jinv12-3.8352272727272725e-04*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-3.2873376623376621e-03*Jinv02*w33+1.0957792207792207e-03*Jinv21*w23-8.2183441558441553e-04*Jinv01*w54-8.2183441558441553e-04*w55*Jinv11-5.4788961038961042e-05*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9-4.9310064935064932e-04*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-1.6436688311688311e-04*Jinv01*w30-5.4788961038961042e-05*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+1.0957792207792207e-03*w25*Jinv11-1.3149350649350649e-03*w16*Jinv12-2.6298701298701297e-03*Jinv21*w50-6.7573051948051951e-04*Jinv01*w27-2.6298701298701297e-03*w48*Jinv01-3.6525974025974026e-05*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[35][48]=(0.0)*D;
  A[35][49]=(0.0)*D;
  A[35][50]=(3.8352272727272725e-04*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10+5.9172077922077923e-03*w40*Jinv12-1.0957792207792207e-03*Jinv22*w53+7.6704545454545450e-04*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+3.8352272727272725e-04*w13*Jinv10+5.9172077922077923e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+7.6704545454545450e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-6.0267857142857139e-04*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+5.6615259740259744e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00-8.2183441558441553e-04*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27+5.6615259740259744e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-2.7394480519480518e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-1.2601461038961039e-03*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-8.7662337662337668e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+3.8352272727272725e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-1.2053571428571428e-03*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+7.6704545454545450e-04*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20-4.3831168831168834e-04*Jinv12*w13+3.8352272727272725e-04*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+7.6704545454545450e-04*w6*Jinv00-6.0267857142857139e-04*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56+5.9172077922077923e-03*Jinv22*w41+5.4788961038961042e-05*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-1.0957792207792207e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11+3.8352272727272725e-04*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54-8.7662337662337668e-04*w4*Jinv12-1.2601461038961039e-03*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-4.3831168831168834e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01+7.6704545454545450e-04*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-1.2601461038961039e-03*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12-1.2053571428571428e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53-1.3697240259740260e-03*Jinv22*w8+5.4788961038961042e-05*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.3697240259740260e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10+7.6704545454545450e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+5.4788961038961042e-05*w24*Jinv00+6.9399350649350649e-04*Jinv02*w27-1.3697240259740260e-03*w6*Jinv02-3.2873376623376621e-04*w19*Jinv12-8.7662337662337668e-04*Jinv22*w5-1.2053571428571428e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12-3.2873376623376621e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+5.4788961038961042e-05*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+4.9310064935064932e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+3.8352272727272725e-04*w20*Jinv20+6.9399350649350649e-04*Jinv22*w29-4.3831168831168834e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11-6.0267857142857139e-04*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+7.6704545454545450e-04*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-1.0957792207792207e-03*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+6.9399350649350649e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[35][51]=(0.0)*D;
  A[35][52]=(0.0)*D;
  A[35][53]=(-3.8352272727272725e-04*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+4.7666396103896100e-03*Jinv22*w53-7.6704545454545450e-04*Jinv20*w8-7.6704545454545450e-04*w57*Jinv02-3.8352272727272725e-04*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-7.6704545454545450e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-6.5746753246753243e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-1.3149350649350649e-03*w25*Jinv12+1.0592532467532468e-03*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10+1.5888798701298701e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-1.3149350649350649e-03*Jinv02*w24+1.3806818181818182e-02*Jinv22*w35+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20-1.3149350649350649e-03*w45*Jinv00+6.9034090909090911e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+4.2735389610389614e-03*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+1.9724025974025973e-03*Jinv00*w33-8.7662337662337668e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-2.1367694805194807e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-2.7394480519480520e-03*w32*Jinv22+1.0592532467532468e-03*Jinv22*w2-3.8352272727272725e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-2.5202922077922079e-03*Jinv02*w45-7.6704545454545450e-04*Jinv22*w59+4.2735389610389614e-03*w51*Jinv00+1.3806818181818182e-02*Jinv12*w34+6.9034090909090911e-03*Jinv22*w44-7.6704545454545450e-04*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-3.8352272727272725e-04*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10-7.6704545454545450e-04*w6*Jinv00-6.5746753246753243e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-5.4788961038961042e-05*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+4.7666396103896100e-03*w52*Jinv12-1.3149350649350649e-03*Jinv22*w26-3.8352272727272725e-04*Jinv20*w14+1.5888798701298701e-03*Jinv02*w54-2.1367694805194807e-03*w4*Jinv12-8.7662337662337668e-04*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+4.3831168831168834e-04*Jinv02*w9-7.6704545454545450e-04*w7*Jinv10-2.1367694805194807e-03*Jinv20*w38-8.7662337662337668e-04*Jinv22*w23-2.7394480519480520e-03*w31*Jinv12-2.5202922077922079e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-1.6436688311688311e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10+3.1777597402597403e-03*Jinv22*w50-1.6436688311688311e-03*Jinv12*w7-2.7394480519480520e-03*Jinv02*w30+3.1777597402597403e-03*Jinv12*w49+4.2735389610389614e-03*w52*Jinv10+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10-5.4788961038961042e-05*w24*Jinv00+9.6793831168831166e-04*Jinv02*w27-1.6436688311688311e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12-2.1367694805194807e-03*Jinv22*w5-2.5202922077922079e-03*w47*Jinv22+4.9310064935064932e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-7.6704545454545450e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+1.3806818181818182e-02*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+3.1777597402597403e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-3.8352272727272725e-04*w20*Jinv20+9.6793831168831166e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-6.5746753246753243e-04*Jinv02*w36+6.9034090909090911e-03*Jinv12*w43-7.6704545454545450e-04*w5*Jinv20+4.7666396103896100e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+9.6793831168831166e-04*Jinv12*w28+2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-1.3149350649350649e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+1.0592532467532468e-03*Jinv12*w1)*D;
  A[35][54]=(0.0)*D;
  A[35][55]=(0.0)*D;
  A[35][56]=(4.6022727272727274e-03*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-3.8352272727272725e-04*w57*Jinv02+4.6022727272727274e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-7.6704545454545450e-04*Jinv01*w3-1.3697240259740260e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12+5.2962662337662338e-04*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.5888798701298701e-03*Jinv12*w55+6.5746753246753243e-04*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-7.1225649350649346e-04*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41+4.6022727272727274e-03*w42*Jinv02+5.2962662337662338e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+6.5746753246753243e-04*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-1.3697240259740260e-03*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39+5.2962662337662338e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-1.3697240259740260e-03*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-3.8352272727272725e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+4.6022727272727274e-03*Jinv22*w44-7.6704545454545450e-04*w4*Jinv11-7.1225649350649346e-04*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-1.3697240259740260e-03*w37*Jinv12+4.6022727272727274e-03*Jinv22*w41+1.5888798701298701e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*w5*Jinv21+1.5888798701298701e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-7.1225649350649346e-04*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+5.2962662337662338e-04*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-1.3149350649350649e-03*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-1.3697240259740260e-03*w31*Jinv12-1.3697240259740260e-03*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+1.5888798701298701e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-1.3697240259740260e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.6704545454545450e-04*w7*Jinv11+2.1367694805194807e-03*Jinv21*w56+5.2962662337662338e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-1.3697240259740260e-03*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14-3.8352272727272725e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11+1.5888798701298701e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+5.2962662337662338e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-1.3697240259740260e-03*Jinv02*w36+4.6022727272727274e-03*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+1.5888798701298701e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+5.2962662337662338e-04*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11+6.5746753246753243e-04*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01+5.2962662337662338e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[35][57]=(0.0)*D;
  A[35][58]=(0.0)*D;
  A[35][59]=(-1.8262987012987013e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-1.8262987012987013e-03*Jinv02*w39-3.2873376623376621e-04*Jinv22*w38+3.4699675324675324e-04*w25*Jinv12-4.2613636363636367e-05*w0*Jinv02+3.2873376623376621e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+3.4699675324675324e-04*Jinv02*w24-3.6525974025974025e-03*Jinv22*w35+2.1915584415584417e-04*Jinv22*w20-1.8262987012987013e-03*w42*Jinv02-4.2613636363636367e-05*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+3.4699675324675324e-04*w22*Jinv12+4.3831168831168834e-04*Jinv02*w3-3.2873376623376621e-04*w32*Jinv22-4.2613636363636367e-05*Jinv22*w2+2.9220779220779221e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-3.6525974025974025e-03*Jinv12*w34-1.8262987012987013e-03*Jinv22*w44+2.1915584415584417e-04*Jinv12*w13-3.2873376623376621e-04*w37*Jinv12-1.8262987012987013e-03*Jinv22*w41+6.5746753246753243e-04*w52*Jinv12+3.4699675324675324e-04*Jinv22*w26+3.2873376623376621e-04*Jinv02*w54+4.3831168831168834e-04*w4*Jinv12+3.4699675324675324e-04*w21*Jinv02+2.1915584415584417e-04*Jinv02*w12-4.2613636363636367e-05*Jinv02*w9+3.4699675324675324e-04*Jinv22*w23-3.2873376623376621e-04*w31*Jinv12+2.9220779220779221e-04*w46*Jinv12+4.3831168831168834e-04*Jinv22*w8+6.5746753246753243e-04*Jinv22*w50+4.3831168831168834e-04*Jinv12*w7-3.2873376623376621e-04*Jinv02*w30+6.5746753246753243e-04*Jinv12*w49-1.9480519480519481e-04*Jinv02*w27+4.3831168831168834e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12+4.3831168831168834e-04*Jinv22*w5+2.9220779220779221e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-3.6525974025974025e-03*Jinv02*w33+6.5746753246753243e-04*w48*Jinv02-1.9480519480519481e-04*Jinv22*w29+2.1915584415584417e-04*Jinv22*w14-3.2873376623376621e-04*Jinv02*w36-1.8262987012987013e-03*Jinv12*w43+6.5746753246753243e-04*Jinv02*w51+3.2873376623376621e-04*Jinv22*w56-1.9480519480519481e-04*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-4.2613636363636367e-05*Jinv12*w1)*D;
  A[36][0]=(5.4788961038961042e-05*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-3.4699675324675324e-04*w40*Jinv12+5.8441558441558442e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-3.4699675324675324e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+2.6481331168831169e-04*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20+2.6481331168831169e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23-1.4975649350649350e-03*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-2.1915584415584417e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-3.4699675324675324e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-2.1915584415584417e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+2.6481331168831169e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22-3.4699675324675324e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2-7.4878246753246752e-04*Jinv00*w18-1.4975649350649350e-03*w36*Jinv00-1.4975649350649350e-03*Jinv21*w38+2.1915584415584417e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+5.8441558441558442e-04*Jinv21*w53+5.8441558441558442e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+2.6481331168831169e-04*w4*Jinv10+2.6481331168831169e-04*w4*Jinv11-3.4699675324675324e-04*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-7.4878246753246752e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00-1.4975649350649350e-03*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-3.4699675324675324e-04*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20+5.8441558441558442e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+2.6481331168831169e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+2.6481331168831169e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-3.4699675324675324e-04*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-1.4975649350649350e-03*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-1.4975649350649350e-03*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23+5.8441558441558442e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12+2.1915584415584417e-04*w46*Jinv12+5.8441558441558442e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+5.8441558441558442e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49-1.4975649350649350e-03*w36*Jinv01+5.8441558441558442e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-1.4975649350649350e-03*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-3.6525974025974026e-05*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+2.6481331168831169e-04*Jinv22*w5+2.1915584415584417e-04*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-3.4699675324675324e-04*w40*Jinv10-7.4878246753246752e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-3.4699675324675324e-04*w40*Jinv11-1.4975649350649350e-03*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+2.6481331168831169e-04*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+5.8441558441558442e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-2.1915584415584417e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[36][1]=(0.0)*D;
  A[36][2]=(0.0)*D;
  A[36][3]=(-2.7394480519480518e-04*Jinv00*w12+3.1960227272727273e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-1.4793019480519481e-03*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-7.3965097402597403e-04*w3*Jinv00-5.4788961038961036e-04*w32*Jinv20-5.7528409090909093e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23+4.7666396103896100e-03*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+3.1047077922077924e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.5750811688311690e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+8.2183441558441553e-04*w16*Jinv11+8.2183441558441553e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10+8.2183441558441553e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+1.6436688311688311e-03*Jinv22*w35+3.1047077922077924e-04*w58*Jinv11+3.1047077922077924e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-4.6570616883116886e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+3.1960227272727273e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+8.2183441558441553e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+3.1960227272727273e-04*Jinv20*w29-4.6570616883116886e-04*w46*Jinv10+8.2183441558441553e-04*Jinv22*w17+8.7662337662337668e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12-5.4788961038961036e-04*w31*Jinv10-5.7528409090909093e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2+2.0271915584415584e-03*Jinv00*w18+4.0543831168831168e-03*w36*Jinv00+4.7666396103896100e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.4793019480519481e-03*Jinv21*w53-9.5880681818181818e-04*w51*Jinv00+1.6436688311688311e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-7.3965097402597403e-04*w4*Jinv10-5.7528409090909093e-04*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13+2.0271915584415584e-03*w19*Jinv10+8.7662337662337668e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00+4.7666396103896100e-03*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-1.1231737012987013e-03*w26*Jinv20+5.5702110389610384e-04*w11*Jinv20-1.4793019480519481e-03*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.7528409090909093e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-5.7528409090909093e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+4.7666396103896100e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+4.0543831168831168e-03*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23-1.4793019480519481e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-9.5880681818181818e-04*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01-1.4793019480519481e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49+4.7666396103896100e-03*w36*Jinv01-9.5880681818181818e-04*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11+5.5702110389610384e-04*w9*Jinv00+4.0543831168831168e-03*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56-1.1231737012987013e-03*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-5.7528409090909093e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-1.3697240259740259e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+5.5702110389610384e-04*w10*Jinv10+2.5750811688311690e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+1.6436688311688311e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+8.7662337662337668e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11-1.1231737012987013e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+4.3831168831168834e-04*Jinv20*w17-5.4788961038961036e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+2.0271915584415584e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11+4.7666396103896100e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-7.3965097402597403e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30-1.4793019480519481e-03*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+2.5750811688311690e-03*w42*Jinv00-4.6570616883116886e-04*w47*Jinv20+8.2183441558441553e-04*w16*Jinv12-1.3697240259740259e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[36][4]=(0.0)*D;
  A[36][5]=(0.0)*D;
  A[36][6]=(2.7394480519480518e-04*Jinv00*w12-3.7439123376623376e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12+2.6298701298701297e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-7.8530844155844158e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+1.1505681818181819e-03*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.3833198051948050e-03*Jinv01*w3-7.3965097402597406e-03*Jinv22*w38+8.2183441558441553e-05*Jinv00*w54-7.8530844155844158e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-2.6846590909090908e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12+2.3833198051948050e-03*Jinv02*w24-4.2735389610389614e-03*Jinv22*w35-7.8530844155844158e-04*w58*Jinv11-7.8530844155844158e-04*w57*Jinv01+2.3833198051948050e-03*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-3.5612824675324673e-04*w1*Jinv10-5.4788961038961036e-04*w16*Jinv10-3.6982548701298703e-03*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-3.7439123376623376e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.7439123376623376e-04*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10+2.3833198051948050e-03*Jinv02*w3+7.1225649350649346e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2-2.4655032467532467e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-7.3965097402597406e-03*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.3833198051948050e-03*Jinv01*w24-7.8530844155844158e-04*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11-3.6982548701298703e-03*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+1.3971185064935066e-03*w51*Jinv00-4.2735389610389614e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+1.1505681818181819e-03*w4*Jinv10+2.3833198051948050e-03*w4*Jinv11+3.5612824675324673e-04*Jinv12*w13-2.4655032467532467e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+7.1225649350649346e-04*w32*Jinv21-3.6982548701298703e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00-7.3965097402597406e-03*w37*Jinv12+8.2183441558441553e-05*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20-1.1414366883116883e-03*w11*Jinv20+2.6298701298701297e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+7.1225649350649346e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+2.3833198051948050e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54+2.3833198051948050e-03*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33+3.5612824675324673e-04*Jinv02*w12-3.6982548701298703e-03*Jinv01*w18-7.3965097402597406e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-3.6982548701298703e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+1.3971185064935066e-03*Jinv20*w53-3.6982548701298703e-03*Jinv22*w8+1.6436688311688311e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11-3.6982548701298703e-03*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49-7.3965097402597406e-03*w36*Jinv01+1.3971185064935066e-03*w52*Jinv10-3.6982548701298703e-03*w7*Jinv11-1.1414366883116883e-03*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+1.3697240259740260e-03*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27-3.6982548701298703e-03*w6*Jinv02-3.6982548701298703e-03*w19*Jinv12+2.3833198051948050e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.1414366883116883e-03*w10*Jinv10-2.6846590909090908e-03*Jinv20*w44-7.8530844155844158e-04*w58*Jinv12-3.6982548701298703e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-3.6982548701298703e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59-4.2735389610389614e-03*Jinv02*w33-3.5612824675324673e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+1.3697240259740260e-03*w25*Jinv10-3.5612824675324673e-04*Jinv20*w2-5.4788961038961036e-04*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-5.4788961038961036e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-2.4655032467532467e-03*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-7.3965097402597406e-03*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+1.1505681818181819e-03*w5*Jinv20+7.1225649350649346e-04*Jinv01*w30+2.6298701298701297e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+2.3833198051948050e-03*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-2.6846590909090908e-03*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-7.4878246753246752e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[36][7]=(0.0)*D;
  A[36][8]=(0.0)*D;
  A[36][9]=(-5.4788961038961042e-05*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+1.1870941558441558e-03*Jinv20*w8-5.4788961038961042e-05*w13*Jinv10-6.7573051948051951e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-2.6481331168831169e-04*Jinv00*w54+6.5746753246753243e-04*w43*Jinv10+2.0495129870129870e-04*w58*Jinv10-2.6481331168831169e-04*w48*Jinv00+1.5726461038961039e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-1.0957792207792208e-04*w45*Jinv00+1.5726461038961039e-04*Jinv00*w27-2.6481331168831169e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-1.0957792207792208e-04*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10-5.4788961038961042e-05*w21*Jinv00+1.1870941558441558e-03*Jinv00*w18+2.3741883116883116e-03*w36*Jinv00-1.0227272727272728e-03*w51*Jinv00-6.7573051948051951e-04*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+1.1870941558441558e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+1.1870941558441558e-03*w6*Jinv00-2.6481331168831169e-04*Jinv20*w56-6.7573051948051951e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-5.4788961038961042e-05*Jinv20*w14-2.6481331168831169e-04*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+1.1870941558441558e-03*w7*Jinv10+2.3741883116883116e-03*Jinv20*w38-1.0227272727272728e-03*Jinv20*w53-5.4788961038961042e-05*w22*Jinv10-1.0227272727272728e-03*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+2.3741883116883116e-03*w37*Jinv10-6.7573051948051951e-04*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+6.5746753246753243e-04*Jinv20*w44+2.0495129870129870e-04*w57*Jinv00+2.0495129870129870e-04*Jinv20*w59+1.5726461038961039e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-6.7573051948051951e-04*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00+3.2873376623376621e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+1.1870941558441558e-03*w20*Jinv20-6.7573051948051951e-04*w5*Jinv20-2.6481331168831169e-04*w55*Jinv10+6.5746753246753243e-04*w42*Jinv00-1.0957792207792208e-04*w47*Jinv20)*D;
  A[36][10]=(0.0)*D;
  A[36][11]=(0.0)*D;
  A[36][12]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+8.7662337662337668e-04*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53-1.9176136363636362e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+8.7662337662337668e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-5.2049512987012989e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17-4.6570616883116886e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+7.1225649350649346e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-2.4655032467532466e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+8.2183441558441553e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+3.2873376623376621e-04*w15*Jinv01+2.7394480519480521e-05*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+7.1225649350649346e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+8.2183441558441553e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00-5.4788961038961036e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+8.7662337662337668e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20-1.9176136363636362e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+8.7662337662337668e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-4.6570616883116886e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+2.7394480519480521e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+8.7662337662337668e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01-1.9176136363636362e-04*w7*Jinv10-4.6570616883116886e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+3.5612824675324673e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-1.9176136363636362e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-1.9176136363636362e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+3.5612824675324673e-04*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-5.2049512987012989e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-1.9176136363636362e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-9.1314935064935065e-06*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-3.5612824675324673e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+8.7662337662337668e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+8.2183441558441553e-04*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+7.1225649350649346e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+2.7394480519480521e-05*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1)*D;
  A[36][13]=(0.0)*D;
  A[36][14]=(0.0)*D;
  A[36][15]=(-5.2962662337662338e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-1.0409902597402598e-03*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+9.3141233766233771e-04*w3*Jinv00-8.2183441558441553e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54+2.1367694805194807e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+7.1225649350649346e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+8.2183441558441553e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-1.9724025974025973e-03*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-2.1915584415584417e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12+1.3149350649350649e-03*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.7394480519480518e-04*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00+7.6704545454545450e-04*Jinv02*w45+8.2183441558441553e-04*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+1.7532467532467534e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+9.3141233766233771e-04*w4*Jinv10-8.2183441558441553e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-1.0409902597402598e-03*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.7532467532467534e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-04*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18+5.4788961038961036e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01-1.0409902597402598e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12+1.7532467532467534e-03*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+1.7532467532467534e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56+1.7532467532467534e-03*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-3.1047077922077924e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+1.7532467532467534e-03*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-4.2735389610389614e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+9.3141233766233771e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+8.2183441558441553e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1)*D;
  A[36][16]=(0.0)*D;
  A[36][17]=(0.0)*D;
  A[36][18]=(-3.5612824675324673e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+1.2327516233766234e-03*Jinv20*w8-3.5612824675324673e-04*w13*Jinv10+4.2735389610389614e-03*Jinv21*w44-1.0135957792207792e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.3833198051948050e-03*Jinv01*w3-8.2183441558441553e-05*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+7.8530844155844158e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+3.1777597402597403e-03*w43*Jinv10+4.2735389610389614e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17-7.1225649350649346e-04*Jinv01*w45+3.9265422077922079e-04*w58*Jinv10-8.2183441558441553e-05*w48*Jinv00+7.8530844155844158e-04*w58*Jinv11+7.8530844155844158e-04*w57*Jinv01-2.3833198051948050e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+3.7439123376623376e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+4.2735389610389614e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+3.9265422077922079e-04*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10-8.2183441558441553e-05*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+2.4655032467532467e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00+7.3965097402597406e-03*Jinv21*w38-2.3833198051948050e-03*Jinv01*w24+4.2735389610389614e-03*w34*Jinv11+3.6982548701298703e-03*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.2327516233766234e-03*w51*Jinv00-1.0135957792207792e-03*w4*Jinv10-2.3833198051948050e-03*w4*Jinv11+1.0957792207792207e-03*w41*Jinv20+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+3.6982548701298703e-03*Jinv21*w20+1.2327516233766234e-03*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20+4.2735389610389614e-03*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-7.1225649350649346e-04*w46*Jinv11-3.5612824675324673e-04*Jinv20*w14-2.3833198051948050e-03*w5*Jinv21-8.2183441558441553e-05*w49*Jinv10+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33+1.0957792207792207e-03*w39*Jinv00+3.6982548701298703e-03*Jinv01*w18+7.3965097402597406e-03*w37*Jinv11+3.6982548701298703e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10-7.1225649350649346e-04*w47*Jinv21+2.4655032467532467e-03*Jinv20*w38-2.6298701298701297e-03*w52*Jinv11-1.2327516233766234e-03*Jinv20*w53-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+7.3965097402597406e-03*w36*Jinv01-1.2327516233766234e-03*w52*Jinv10+3.6982548701298703e-03*w7*Jinv11-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56-1.2327516233766234e-03*w24*Jinv00-1.6436688311688311e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14-4.5657467532467534e-05*w10*Jinv10+3.1777597402597403e-03*Jinv20*w44-3.5612824675324673e-04*Jinv01*w12+3.6982548701298703e-03*Jinv21*w8+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+3.7439123376623376e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.2327516233766234e-03*w25*Jinv10+3.7439123376623376e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+1.5888798701298701e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-1.0135957792207792e-03*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-2.3833198051948050e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10+3.1777597402597403e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.6436688311688311e-04*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[36][19]=(0.0)*D;
  A[36][20]=(0.0)*D;
  A[36][21]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20+2.4655032467532466e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54+1.6436688311688311e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02-2.7394480519480521e-05*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+8.2183441558441553e-05*Jinv21*w26-2.7394480519480521e-05*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-8.2183441558441553e-04*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-2.7394480519480521e-05*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22-8.2183441558441553e-04*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10+2.4655032467532466e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+4.6570616883116886e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+4.6570616883116886e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14+2.4655032467532466e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-8.2183441558441553e-05*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*Jinv12*w7-8.2183441558441553e-05*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49-5.2049512987012989e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.7394480519480521e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02+3.8352272727272725e-04*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14-8.2183441558441553e-04*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20+4.6570616883116886e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1)*D;
  A[36][22]=(0.0)*D;
  A[36][23]=(0.0)*D;
  A[36][24]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+2.7394480519480518e-04*w13*Jinv10-1.6436688311688311e-03*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23+5.2049512987012989e-04*Jinv00*w54-3.1047077922077924e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-8.2183441558441553e-04*w16*Jinv11-8.2183441558441553e-04*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-3.1047077922077924e-04*w58*Jinv11-3.1047077922077924e-04*w57*Jinv01+5.7528409090909093e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-1.6436688311688311e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-8.2183441558441553e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+8.2183441558441553e-05*w31*Jinv10-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39-1.6436688311688311e-04*Jinv00*w18-7.1225649350649346e-04*w36*Jinv00-4.7666396103896100e-03*Jinv21*w38+5.7528409090909093e-04*Jinv01*w24-2.6298701298701297e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+1.4793019480519481e-03*Jinv21*w53+5.2049512987012989e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-1.6436688311688311e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+5.2049512987012989e-04*Jinv20*w56-1.6436688311688311e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20-2.6298701298701297e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.4655032467532466e-04*Jinv01*w18-4.7666396103896100e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21-7.1225649350649346e-04*Jinv20*w38+1.4793019480519481e-03*w52*Jinv11+5.2049512987012989e-04*Jinv20*w53-3.5612824675324673e-04*w22*Jinv10-8.2183441558441553e-05*w22*Jinv11-8.2183441558441553e-05*w21*Jinv01+1.4793019480519481e-03*Jinv01*w51-4.7666396103896100e-03*w36*Jinv01+5.2049512987012989e-04*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11+9.1314935064935065e-06*w9*Jinv00-7.1225649350649346e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14+9.1314935064935065e-06*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+2.4655032467532466e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-3.8352272727272725e-04*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+5.7528409090909093e-04*w25*Jinv11+5.2049512987012989e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[36][25]=(0.0)*D;
  A[36][26]=(0.0)*D;
  A[36][27]=(4.3831168831168834e-04*Jinv21*w44-4.2918019480519480e-04*Jinv01*w3-1.4204545454545455e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01-2.6481331168831169e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+4.3831168831168834e-04*w43*Jinv11+3.4699675324675324e-04*Jinv21*w41+1.0247564935064935e-04*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+3.4699675324675324e-04*Jinv01*w39+1.4975649350649350e-03*Jinv21*w38-2.6481331168831169e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11-5.8441558441558442e-04*Jinv21*w53-4.2918019480519480e-04*w4*Jinv11-2.1915584415584417e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11-4.2918019480519480e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+3.6525974025974026e-05*Jinv01*w18+1.4975649350649350e-03*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01-1.0957792207792208e-04*w47*Jinv21-5.8441558441558442e-04*w52*Jinv11-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-5.8441558441558442e-04*Jinv01*w51+1.4975649350649350e-03*w36*Jinv01+7.4878246753246752e-04*w7*Jinv11+1.1870941558441559e-04*Jinv21*w56-4.2918019480519480e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-2.7394480519480521e-05*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8-5.4788961038961042e-05*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+6.4935064935064935e-05*Jinv01*w9+3.4699675324675324e-04*w40*Jinv11-2.1915584415584417e-04*Jinv01*w30-2.6481331168831169e-04*w25*Jinv11-4.2918019480519480e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[36][28]=(0.0)*D;
  A[36][29]=(0.0)*D;
  A[36][30]=(3.5612824675324673e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.2601461038961039e-03*w40*Jinv12-9.5880681818181818e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-3.5612824675324673e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+1.2601461038961039e-03*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.0683847402597404e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-4.6570616883116886e-04*Jinv20*w23+1.4245129870129869e-03*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-1.8262987012987013e-05*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-1.1231737012987013e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+2.1915584415584417e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-1.9176136363636362e-04*w22*Jinv12+1.0683847402597404e-03*w31*Jinv10-4.6570616883116886e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.5612824675324673e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-1.4793019480519481e-03*Jinv21*w53-1.4793019480519481e-03*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+7.1225649350649346e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10+1.0683847402597404e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00+1.4245129870129869e-03*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+1.2601461038961039e-03*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-1.8262987012987013e-05*w11*Jinv20-9.5880681818181818e-04*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.0683847402597404e-03*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-1.1231737012987013e-03*Jinv02*w54-1.9176136363636362e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+2.1915584415584417e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+7.1225649350649346e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-1.9176136363636362e-04*Jinv22*w23-1.4793019480519481e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12-1.4793019480519481e-03*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-4.6570616883116886e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50-4.6570616883116886e-04*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7-4.6570616883116886e-04*w21*Jinv01-1.4793019480519481e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-1.4793019480519481e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-1.8262987012987013e-05*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-1.3971185064935066e-03*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-1.6436688311688311e-04*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.8262987012987013e-05*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-3.5612824675324673e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-4.6570616883116886e-04*Jinv21*w23-1.3971185064935066e-03*Jinv01*w54-1.3971185064935066e-03*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02-1.8262987012987013e-05*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+1.0683847402597404e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29+1.3149350649350649e-03*w40*Jinv11+1.4245129870129869e-03*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+1.0683847402597404e-03*Jinv01*w30-9.5880681818181818e-04*Jinv02*w51-1.1231737012987013e-03*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50+2.1915584415584417e-04*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1-1.8262987012987013e-05*w10*Jinv11)*D;
  A[36][31]=(0.0)*D;
  A[36][32]=(0.0)*D;
  A[36][33]=(-9.6793831168831166e-04*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12+2.7942370129870131e-03*Jinv22*w53-2.0819805194805196e-03*Jinv20*w8-1.6436688311688311e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-2.9586038961038961e-03*Jinv21*w44+1.8628246753246754e-03*w3*Jinv00+2.6298701298701297e-03*Jinv01*w3+8.2183441558441553e-04*Jinv20*w23-8.5470779220779228e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-6.5746753246753243e-04*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-5.4788961038961036e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-2.9586038961038961e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+1.7532467532467534e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10+5.4788961038961036e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-6.5746753246753243e-04*w58*Jinv11-6.5746753246753243e-04*w57*Jinv01+2.6298701298701297e-03*Jinv21*w26-5.4788961038961036e-04*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-2.9586038961038961e-03*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00+1.9724025974025973e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-9.6793831168831166e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-5.4788961038961036e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-5.4788961038961036e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-9.6793831168831166e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-6.9034090909090911e-03*Jinv00*w33+4.9310064935064932e-04*Jinv02*w3+8.2183441558441553e-04*w21*Jinv00-6.0267857142857139e-04*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-9.5332792207792200e-03*w36*Jinv00-1.2820616883116883e-02*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+2.6298701298701297e-03*Jinv01*w24-1.6436688311688311e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+4.9310064935064934e-03*Jinv21*w53+2.7942370129870131e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+1.8628246753246754e-03*w4*Jinv10+2.6298701298701297e-03*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-6.9034090909090911e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-2.0819805194805196e-03*w6*Jinv00-8.5470779220779228e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41+2.5750811688311690e-03*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.7942370129870131e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11+2.6298701298701297e-03*w5*Jinv21+1.7532467532467534e-03*Jinv02*w54+4.9310064935064932e-04*w4*Jinv12-4.9310064935064932e-04*w49*Jinv10-5.4788961038961036e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33-2.9586038961038961e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-1.2820616883116883e-02*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01-2.0819805194805196e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-9.5332792207792200e-03*Jinv20*w38+4.9310064935064934e-03*w52*Jinv11-6.0267857142857139e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.7942370129870131e-03*Jinv20*w53+8.2183441558441553e-04*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+5.4788961038961042e-05*w21*Jinv01+4.9310064935064934e-03*Jinv01*w51-6.0267857142857139e-04*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49-1.2820616883116883e-02*w36*Jinv01+2.7942370129870131e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-9.5332792207792200e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56+2.5750811688311690e-03*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27-2.1367694805194807e-03*w19*Jinv12+4.9310064935064932e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-1.6436688311688311e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00-6.9034090909090911e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11+2.5750811688311690e-03*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02+1.0957792207792207e-03*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-2.9586038961038961e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-8.5470779220779228e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+1.8628246753246754e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.7942370129870131e-03*Jinv02*w51+1.7532467532467534e-03*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28+2.6298701298701297e-03*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20+2.1367694805194807e-03*Jinv21*w50-5.4788961038961036e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+1.0957792207792207e-03*w10*Jinv11)*D;
  A[36][34]=(0.0)*D;
  A[36][35]=(0.0)*D;
  A[36][36]=(-3.5612824675324673e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.6298701298701297e-03*Jinv22*w53+2.4655032467532467e-03*Jinv20*w8+7.8530844155844158e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-2.0271915584415584e-03*w3*Jinv00-1.0683847402597404e-03*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+7.3965097402597406e-03*Jinv22*w38+5.7528409090909093e-04*Jinv00*w54-2.3833198051948050e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+4.7666396103896100e-03*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+6.5746753246753243e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+5.7528409090909093e-04*w48*Jinv00-2.3833198051948050e-03*Jinv02*w24+4.2735389610389614e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+3.6982548701298703e-03*Jinv22*w20-1.0683847402597404e-03*w45*Jinv00+4.2735389610389614e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22+5.7528409090909093e-04*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-1.0683847402597404e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+4.7666396103896100e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-1.0683847402597404e-03*w31*Jinv10-2.3833198051948050e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2+2.4655032467532467e-03*Jinv00*w18+7.3965097402597406e-03*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45+7.8530844155844158e-04*Jinv22*w59-1.3149350649350649e-03*w51*Jinv00+4.2735389610389614e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-2.0271915584415584e-03*w4*Jinv10+1.6436688311688311e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13+2.4655032467532467e-03*w19*Jinv10+4.7666396103896100e-03*w34*Jinv10+2.4655032467532467e-03*w6*Jinv00+7.3965097402597406e-03*w37*Jinv12+5.7528409090909093e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-2.0271915584415584e-03*w26*Jinv20-9.1314935064935068e-05*w11*Jinv20-2.6298701298701297e-03*w52*Jinv12-2.3833198051948050e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54-2.3833198051948050e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+5.7528409090909093e-04*w49*Jinv10+1.6436688311688311e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+2.4655032467532467e-03*w7*Jinv10+7.3965097402597406e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53+3.6982548701298703e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50+3.6982548701298703e-03*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.3149350649350649e-03*w52*Jinv10-9.1314935064935068e-05*w9*Jinv00+7.3965097402597406e-03*w37*Jinv10-2.0271915584415584e-03*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+3.6982548701298703e-03*w6*Jinv02+3.6982548701298703e-03*w19*Jinv12-2.3833198051948050e-03*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22-9.1314935064935068e-05*w10*Jinv10+4.7666396103896100e-03*Jinv20*w44+7.8530844155844158e-04*w58*Jinv12+3.6982548701298703e-03*Jinv02*w18+6.5746753246753243e-04*w57*Jinv00+6.5746753246753243e-04*Jinv20*w59+4.2735389610389614e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+4.7666396103896100e-03*w35*Jinv20-2.0271915584415584e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-1.0683847402597404e-03*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14+7.3965097402597406e-03*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-2.0271915584415584e-03*w5*Jinv20-2.6298701298701297e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28+5.7528409090909093e-04*w55*Jinv10+4.7666396103896100e-03*w42*Jinv00-1.0683847402597404e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[36][37]=(0.0)*D;
  A[36][38]=(0.0)*D;
  A[36][39]=(-7.6704545454545450e-04*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10+1.6436688311688311e-03*w40*Jinv12+1.7532467532467534e-03*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10+1.6436688311688311e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-2.1915584415584414e-03*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+3.8352272727272725e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+3.4699675324675324e-04*Jinv21*w2+3.9448051948051946e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+1.3149350649350649e-03*Jinv01*w45+2.5750811688311690e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+4.9310064935064932e-04*w10*Jinv12+1.3149350649350649e-03*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35+7.1225649350649346e-04*Jinv21*w26+3.4699675324675324e-04*w0*Jinv01+4.2004870129870131e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+2.6298701298701297e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22+3.4699675324675324e-04*Jinv11*w1-8.2183441558441553e-04*w13*Jinv11-3.4699675324675324e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+1.9724025974025973e-03*Jinv00*w33-1.1505681818181819e-03*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-1.2053571428571428e-03*w32*Jinv22+3.8352272727272725e-04*Jinv22*w2-2.1915584415584414e-03*Jinv00*w18-3.2873376623376621e-03*w36*Jinv00+1.0957792207792208e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+2.1367694805194807e-03*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-8.2183441558441553e-04*Jinv12*w13-2.1915584415584414e-03*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20-1.0957792207792207e-03*w6*Jinv00-2.1915584415584414e-03*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56+1.6436688311688311e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20+9.8620129870129864e-04*w11*Jinv20+1.7532467532467534e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+3.8352272727272725e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21+2.5750811688311690e-03*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+1.3149350649350649e-03*w49*Jinv10-3.4699675324675324e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00-8.2183441558441553e-04*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18+4.9310064935064932e-04*Jinv02*w9+1.0957792207792207e-03*w6*Jinv01-1.0957792207792207e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-3.2873376623376621e-03*Jinv20*w38-1.2053571428571428e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53+5.4788961038961042e-05*w22*Jinv10+1.2601461038961039e-03*Jinv22*w50+8.2183441558441553e-04*w22*Jinv11+8.2183441558441553e-04*w21*Jinv01-1.2053571428571428e-03*Jinv02*w30+1.2601461038961039e-03*Jinv12*w49+2.1367694805194807e-03*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+9.8620129870129864e-04*w9*Jinv00-3.2873376623376621e-03*w37*Jinv10+1.3149350649350649e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-1.0957792207792207e-03*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-8.2183441558441553e-04*Jinv21*w14+9.8620129870129864e-04*w10*Jinv10+3.9448051948051946e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-8.2183441558441553e-04*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33+4.2004870129870131e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20+8.2183441558441553e-04*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10+4.2004870129870131e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.2601461038961039e-03*w48*Jinv02+1.3149350649350649e-03*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-2.1915584415584414e-03*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-8.2183441558441553e-04*Jinv22*w14-2.1915584415584414e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30+1.7532467532467534e-03*Jinv02*w51+2.5750811688311690e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+3.9448051948051946e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*Jinv21*w50-3.4699675324675324e-04*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1)*D;
  A[36][40]=(0.0)*D;
  A[36][41]=(0.0)*D;
  A[36][42]=(7.6704545454545450e-04*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00+1.1505681818181819e-03*w32*Jinv20-2.6298701298701297e-03*Jinv01*w3-5.4788961038961042e-05*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+6.5746753246753243e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+5.4788961038961036e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-7.6704545454545450e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+6.5746753246753243e-04*w58*Jinv11+6.5746753246753243e-04*w57*Jinv01-2.6298701298701297e-03*Jinv21*w26+5.4788961038961036e-04*w0*Jinv01-4.2004870129870131e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+5.4788961038961036e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+5.4788961038961036e-04*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+1.1505681818181819e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2+2.1915584415584414e-03*Jinv00*w18+3.2873376623376621e-03*w36*Jinv00+1.2820616883116883e-02*Jinv21*w38-7.6704545454545450e-04*Jinv02*w45-2.6298701298701297e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+2.9586038961038961e-03*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-4.9310064935064934e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-2.6298701298701297e-03*w4*Jinv11-4.9310064935064934e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+2.1915584415584414e-03*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20-9.8620129870129864e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12+2.9586038961038961e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14-2.6298701298701297e-03*w5*Jinv21-7.6704545454545450e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-1.3149350649350649e-03*w49*Jinv10+5.4788961038961036e-04*Jinv21*w29+2.9586038961038961e-03*Jinv01*w33-4.9310064935064934e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12+4.2735389610389614e-03*Jinv01*w18+1.2820616883116883e-02*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9+4.2735389610389614e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+3.2873376623376621e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-4.9310064935064934e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-4.9310064935064934e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49+1.2820616883116883e-02*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-9.8620129870129864e-04*w9*Jinv00+3.2873376623376621e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56-7.6704545454545450e-04*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-9.8620129870129864e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-4.2004870129870131e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11-7.6704545454545450e-04*w25*Jinv10-4.2004870129870131e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+1.1505681818181819e-03*w30*Jinv00-4.9310064935064934e-03*w40*Jinv10+2.1915584415584414e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11+4.2735389610389614e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30-2.1367694805194807e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-2.6298701298701297e-03*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10-3.9448051948051946e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50+5.4788961038961036e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[36][43]=(0.0)*D;
  A[36][44]=(0.0)*D;
  A[36][45]=(-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-1.3697240259740259e-04*w57*Jinv02-5.4788961038961042e-05*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-2.1915584415584417e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01+1.8262987012987013e-05*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-1.0683847402597404e-03*Jinv01*w45+5.2049512987012989e-04*Jinv12*w55-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-2.1915584415584417e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-2.1915584415584417e-04*Jinv11*w1+4.6570616883116886e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.8352272727272725e-04*Jinv22*w17-3.5612824675324673e-04*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+8.2183441558441553e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-7.1225649350649346e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-1.3697240259740259e-04*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+1.4793019480519481e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv11+2.7394480519480518e-04*Jinv12*w13+2.4655032467532466e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12-5.4788961038961042e-05*Jinv22*w41+5.2049512987012989e-04*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-1.0683847402597404e-03*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21+5.2049512987012989e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33+2.7394480519480518e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-1.0683847402597404e-03*w47*Jinv21-3.5612824675324673e-04*Jinv22*w23+1.4793019480519481e-03*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+1.4793019480519481e-03*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+8.2183441558441553e-05*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.3971185064935066e-03*w49*Jinv11+4.6570616883116886e-04*Jinv21*w14-1.3697240259740259e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+4.6570616883116886e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02+1.8262987012987013e-05*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+2.4655032467532466e-04*Jinv01*w30+5.2049512987012989e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-3.8352272727272725e-04*w16*Jinv12+1.3971185064935066e-03*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+1.3971185064935066e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1+1.8262987012987013e-05*w10*Jinv11)*D;
  A[36][46]=(0.0)*D;
  A[36][47]=(0.0)*D;
  A[36][48]=(-2.7394480519480518e-04*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10-1.2053571428571428e-03*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8+6.6659902597402602e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-1.2053571428571428e-03*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+3.2873376623376621e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+4.7483766233766234e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+1.3697240259740260e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+2.3833198051948050e-03*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24-6.0267857142857139e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.3833198051948050e-03*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33+1.9176136363636362e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10-3.8352272727272725e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-9.5880681818181818e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18-1.1505681818181819e-03*w36*Jinv00-1.1505681818181819e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+6.6659902597402602e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-6.0267857142857139e-04*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-2.7394480519480521e-05*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+3.2873376623376621e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-1.2053571428571428e-03*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20+4.7483766233766234e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+1.3697240259740260e-03*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12+1.9176136363636362e-04*w21*Jinv02+2.3833198051948050e-03*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-2.7394480519480521e-05*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18-1.1505681818181819e-03*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.1505681818181819e-03*Jinv20*w38+1.9176136363636362e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11-9.5880681818181818e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-9.5880681818181818e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-1.1505681818181819e-03*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11+4.7483766233766234e-04*w9*Jinv00-1.1505681818181819e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.3833198051948050e-03*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+4.7483766233766234e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+6.6659902597402602e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-6.0267857142857139e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+4.7483766233766234e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29-2.7394480519480521e-05*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+3.2873376623376621e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30+1.3971185064935066e-03*Jinv02*w51+1.3697240259740260e-03*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+2.3833198051948050e-03*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.3833198051948050e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+4.7483766233766234e-04*w10*Jinv11)*D;
  A[36][49]=(0.0)*D;
  A[36][50]=(0.0)*D;
  A[36][51]=(2.7394480519480518e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-1.4610389610389610e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+5.2049512987012989e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23+3.7804383116883116e-03*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+8.2183441558441553e-05*w25*Jinv12-1.5523538961038962e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10-1.2327516233766234e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-8.2183441558441553e-04*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-6.5746753246753243e-04*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-8.2183441558441553e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.5523538961038962e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00+4.3831168831168834e-04*Jinv02*w45-1.4610389610389610e-04*Jinv22*w59-2.6298701298701297e-03*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv10-1.3149350649350649e-03*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00+3.7804383116883116e-03*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+5.2049512987012989e-04*w26*Jinv20-4.7483766233766234e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+8.2183441558441553e-05*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-1.2327516233766234e-03*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-1.3149350649350649e-03*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-1.0957792207792207e-03*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10+1.1505681818181819e-03*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23+4.3831168831168834e-04*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12-2.6298701298701297e-03*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50-2.4655032467532466e-04*Jinv12*w7+4.3831168831168834e-04*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49-2.6298701298701297e-03*w52*Jinv10-4.7483766233766234e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10+5.2049512987012989e-04*w24*Jinv00-1.5523538961038962e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-4.7483766233766234e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.4610389610389610e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+5.2049512987012989e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-1.5523538961038962e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14+3.7804383116883116e-03*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+5.2049512987012989e-04*w5*Jinv20-1.3149350649350649e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56-1.5523538961038962e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-8.2183441558441553e-04*w16*Jinv12-1.5523538961038962e-04*Jinv12*w1)*D;
  A[36][52]=(0.0)*D;
  A[36][53]=(0.0)*D;
  A[36][54]=(-2.5202922077922079e-03*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53+1.1870941558441559e-04*w57*Jinv02-2.5202922077922079e-03*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+1.4793019480519481e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59+1.3697240259740259e-04*w25*Jinv12-3.1960227272727273e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01-4.7483766233766234e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-2.0271915584415584e-03*Jinv12*w55-7.1225649350649346e-04*w15*Jinv02-4.8396915584415583e-04*w10*Jinv12+1.3697240259740259e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+3.8352272727272725e-04*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41-2.7394480519480520e-03*w42*Jinv02-4.8396915584415583e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11-7.1225649350649346e-04*Jinv22*w17+2.4655032467532466e-04*w22*Jinv12+2.4655032467532466e-04*Jinv02*w3+8.4922889610389610e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-3.1960227272727273e-04*Jinv22*w2+1.1505681818181819e-03*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45+5.2049512987012989e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-2.7394480519480520e-03*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+4.6570616883116886e-04*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20+1.4793019480519481e-03*w37*Jinv12-2.5202922077922079e-03*Jinv22*w41-1.2327516233766234e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11+1.3697240259740259e-04*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-2.0271915584415584e-03*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+4.6570616883116886e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18+1.1505681818181819e-03*w37*Jinv11-4.8396915584415583e-04*Jinv02*w9-1.6436688311688311e-04*w6*Jinv01+1.3971185064935066e-03*w47*Jinv21+2.4655032467532466e-04*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11+8.4922889610389610e-04*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+1.9176136363636362e-04*Jinv22*w8-1.0135957792207792e-03*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11+1.9176136363636362e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+8.4922889610389610e-04*Jinv02*w30-1.0135957792207792e-03*Jinv12*w49+1.1505681818181819e-03*w36*Jinv01-1.6436688311688311e-04*w7*Jinv11-2.3833198051948050e-03*Jinv21*w56-1.7349837662337662e-04*Jinv02*w27+1.9176136363636362e-04*w6*Jinv02+3.8352272727272725e-04*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-2.3833198051948050e-03*Jinv01*w54-2.3833198051948050e-03*w55*Jinv11-1.0135957792207792e-03*w48*Jinv02-4.7483766233766234e-04*Jinv01*w9-1.7349837662337662e-04*Jinv22*w29+4.6570616883116886e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+1.4793019480519481e-03*Jinv02*w36-2.7394480519480520e-03*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-2.0271915584415584e-03*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28+5.2049512987012989e-04*w25*Jinv11-7.1225649350649346e-04*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-3.1960227272727273e-04*Jinv12*w1-4.7483766233766234e-04*w10*Jinv11)*D;
  A[36][55]=(0.0)*D;
  A[36][56]=(0.0)*D;
  A[36][57]=(2.9220779220779221e-04*w40*Jinv12-1.0227272727272728e-03*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02+2.9220779220779221e-04*Jinv02*w39-2.5568181818181820e-04*Jinv22*w38+1.1870941558441559e-04*w25*Jinv12-1.1668019480519480e-04*w0*Jinv02-6.7573051948051951e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02+8.3198051948051951e-05*w10*Jinv12+1.1870941558441559e-04*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35+3.6525974025974026e-05*Jinv22*w20-3.2873376623376621e-04*w42*Jinv02+8.3198051948051951e-05*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17-2.7394480519480521e-05*w22*Jinv12+1.1870941558441559e-04*Jinv02*w3+4.9310064935064932e-04*w32*Jinv22-1.1668019480519480e-04*Jinv22*w2+4.9310064935064932e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59-3.2873376623376621e-04*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-2.7394480519480521e-05*Jinv12*w13-2.5568181818181820e-04*w37*Jinv12+2.9220779220779221e-04*Jinv22*w41-1.0227272727272728e-03*w52*Jinv12+1.1870941558441559e-04*Jinv22*w26-6.7573051948051951e-04*Jinv02*w54+1.1870941558441559e-04*w4*Jinv12-2.7394480519480521e-05*w21*Jinv02-2.7394480519480521e-05*Jinv02*w12+8.3198051948051951e-05*Jinv02*w9-2.7394480519480521e-05*Jinv22*w23+4.9310064935064932e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+3.6525974025974026e-05*Jinv22*w8-6.7573051948051951e-04*Jinv22*w50+3.6525974025974026e-05*Jinv12*w7+4.9310064935064932e-04*Jinv02*w30-6.7573051948051951e-04*Jinv12*w49-1.1668019480519480e-04*Jinv02*w27+3.6525974025974026e-05*w6*Jinv02+3.6525974025974026e-05*w19*Jinv12+1.1870941558441559e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12+3.6525974025974026e-05*Jinv02*w18-3.2873376623376621e-04*Jinv02*w33-6.7573051948051951e-04*w48*Jinv02-1.1668019480519480e-04*Jinv22*w29-2.7394480519480521e-05*Jinv22*w14-2.5568181818181820e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-1.0227272727272728e-03*Jinv02*w51-6.7573051948051951e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12-1.1668019480519480e-04*Jinv12*w1)*D;
  A[36][58]=(0.0)*D;
  A[36][59]=(0.0)*D;
  A[37][0]=(0.0)*D;
  A[37][1]=(5.4788961038961042e-05*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-3.4699675324675324e-04*w40*Jinv12+5.8441558441558442e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-3.4699675324675324e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+2.6481331168831169e-04*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20+2.6481331168831169e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23-1.4975649350649350e-03*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-2.1915584415584417e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-3.4699675324675324e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-2.1915584415584417e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+2.6481331168831169e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22-3.4699675324675324e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2-7.4878246753246752e-04*Jinv00*w18-1.4975649350649350e-03*w36*Jinv00-1.4975649350649350e-03*Jinv21*w38+2.1915584415584417e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+5.8441558441558442e-04*Jinv21*w53+5.8441558441558442e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+2.6481331168831169e-04*w4*Jinv10+2.6481331168831169e-04*w4*Jinv11-3.4699675324675324e-04*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-7.4878246753246752e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00-1.4975649350649350e-03*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-3.4699675324675324e-04*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20+5.8441558441558442e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+2.6481331168831169e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+2.6481331168831169e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-3.4699675324675324e-04*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-1.4975649350649350e-03*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-1.4975649350649350e-03*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23+5.8441558441558442e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12+2.1915584415584417e-04*w46*Jinv12+5.8441558441558442e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+5.8441558441558442e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49-1.4975649350649350e-03*w36*Jinv01+5.8441558441558442e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-1.4975649350649350e-03*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-3.6525974025974026e-05*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+2.6481331168831169e-04*Jinv22*w5+2.1915584415584417e-04*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-3.4699675324675324e-04*w40*Jinv10-7.4878246753246752e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-3.4699675324675324e-04*w40*Jinv11-1.4975649350649350e-03*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+2.6481331168831169e-04*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+5.8441558441558442e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-2.1915584415584417e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[37][2]=(0.0)*D;
  A[37][3]=(0.0)*D;
  A[37][4]=(-2.7394480519480518e-04*Jinv00*w12+3.1960227272727273e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-1.4793019480519481e-03*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-7.3965097402597403e-04*w3*Jinv00-5.4788961038961036e-04*w32*Jinv20-5.7528409090909093e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23+4.7666396103896100e-03*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+3.1047077922077924e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.5750811688311690e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+8.2183441558441553e-04*w16*Jinv11+8.2183441558441553e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10+8.2183441558441553e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+1.6436688311688311e-03*Jinv22*w35+3.1047077922077924e-04*w58*Jinv11+3.1047077922077924e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-4.6570616883116886e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+3.1960227272727273e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+8.2183441558441553e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+3.1960227272727273e-04*Jinv20*w29-4.6570616883116886e-04*w46*Jinv10+8.2183441558441553e-04*Jinv22*w17+8.7662337662337668e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12-5.4788961038961036e-04*w31*Jinv10-5.7528409090909093e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2+2.0271915584415584e-03*Jinv00*w18+4.0543831168831168e-03*w36*Jinv00+4.7666396103896100e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.4793019480519481e-03*Jinv21*w53-9.5880681818181818e-04*w51*Jinv00+1.6436688311688311e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-7.3965097402597403e-04*w4*Jinv10-5.7528409090909093e-04*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13+2.0271915584415584e-03*w19*Jinv10+8.7662337662337668e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00+4.7666396103896100e-03*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-1.1231737012987013e-03*w26*Jinv20+5.5702110389610384e-04*w11*Jinv20-1.4793019480519481e-03*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.7528409090909093e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-5.7528409090909093e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+4.7666396103896100e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+4.0543831168831168e-03*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23-1.4793019480519481e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-9.5880681818181818e-04*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01-1.4793019480519481e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49+4.7666396103896100e-03*w36*Jinv01-9.5880681818181818e-04*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11+5.5702110389610384e-04*w9*Jinv00+4.0543831168831168e-03*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56-1.1231737012987013e-03*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-5.7528409090909093e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-1.3697240259740259e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+5.5702110389610384e-04*w10*Jinv10+2.5750811688311690e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+1.6436688311688311e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+8.7662337662337668e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11-1.1231737012987013e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+4.3831168831168834e-04*Jinv20*w17-5.4788961038961036e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+2.0271915584415584e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11+4.7666396103896100e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-7.3965097402597403e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30-1.4793019480519481e-03*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+2.5750811688311690e-03*w42*Jinv00-4.6570616883116886e-04*w47*Jinv20+8.2183441558441553e-04*w16*Jinv12-1.3697240259740259e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[37][5]=(0.0)*D;
  A[37][6]=(0.0)*D;
  A[37][7]=(2.7394480519480518e-04*Jinv00*w12-3.7439123376623376e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12+2.6298701298701297e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-7.8530844155844158e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+1.1505681818181819e-03*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.3833198051948050e-03*Jinv01*w3-7.3965097402597406e-03*Jinv22*w38+8.2183441558441553e-05*Jinv00*w54-7.8530844155844158e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-2.6846590909090908e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12+2.3833198051948050e-03*Jinv02*w24-4.2735389610389614e-03*Jinv22*w35-7.8530844155844158e-04*w58*Jinv11-7.8530844155844158e-04*w57*Jinv01+2.3833198051948050e-03*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-3.5612824675324673e-04*w1*Jinv10-5.4788961038961036e-04*w16*Jinv10-3.6982548701298703e-03*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-3.7439123376623376e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.7439123376623376e-04*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10+2.3833198051948050e-03*Jinv02*w3+7.1225649350649346e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2-2.4655032467532467e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-7.3965097402597406e-03*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.3833198051948050e-03*Jinv01*w24-7.8530844155844158e-04*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11-3.6982548701298703e-03*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+1.3971185064935066e-03*w51*Jinv00-4.2735389610389614e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+1.1505681818181819e-03*w4*Jinv10+2.3833198051948050e-03*w4*Jinv11+3.5612824675324673e-04*Jinv12*w13-2.4655032467532467e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+7.1225649350649346e-04*w32*Jinv21-3.6982548701298703e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00-7.3965097402597406e-03*w37*Jinv12+8.2183441558441553e-05*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20-1.1414366883116883e-03*w11*Jinv20+2.6298701298701297e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+7.1225649350649346e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+2.3833198051948050e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54+2.3833198051948050e-03*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33+3.5612824675324673e-04*Jinv02*w12-3.6982548701298703e-03*Jinv01*w18-7.3965097402597406e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-3.6982548701298703e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+1.3971185064935066e-03*Jinv20*w53-3.6982548701298703e-03*Jinv22*w8+1.6436688311688311e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11-3.6982548701298703e-03*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49-7.3965097402597406e-03*w36*Jinv01+1.3971185064935066e-03*w52*Jinv10-3.6982548701298703e-03*w7*Jinv11-1.1414366883116883e-03*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+1.3697240259740260e-03*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27-3.6982548701298703e-03*w6*Jinv02-3.6982548701298703e-03*w19*Jinv12+2.3833198051948050e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.1414366883116883e-03*w10*Jinv10-2.6846590909090908e-03*Jinv20*w44-7.8530844155844158e-04*w58*Jinv12-3.6982548701298703e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-3.6982548701298703e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59-4.2735389610389614e-03*Jinv02*w33-3.5612824675324673e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+1.3697240259740260e-03*w25*Jinv10-3.5612824675324673e-04*Jinv20*w2-5.4788961038961036e-04*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-5.4788961038961036e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-2.4655032467532467e-03*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-7.3965097402597406e-03*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+1.1505681818181819e-03*w5*Jinv20+7.1225649350649346e-04*Jinv01*w30+2.6298701298701297e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+2.3833198051948050e-03*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-2.6846590909090908e-03*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-7.4878246753246752e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[37][8]=(0.0)*D;
  A[37][9]=(0.0)*D;
  A[37][10]=(-5.4788961038961042e-05*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+1.1870941558441558e-03*Jinv20*w8-5.4788961038961042e-05*w13*Jinv10-6.7573051948051951e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-2.6481331168831169e-04*Jinv00*w54+6.5746753246753243e-04*w43*Jinv10+2.0495129870129870e-04*w58*Jinv10-2.6481331168831169e-04*w48*Jinv00+1.5726461038961039e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-1.0957792207792208e-04*w45*Jinv00+1.5726461038961039e-04*Jinv00*w27-2.6481331168831169e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-1.0957792207792208e-04*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10-5.4788961038961042e-05*w21*Jinv00+1.1870941558441558e-03*Jinv00*w18+2.3741883116883116e-03*w36*Jinv00-1.0227272727272728e-03*w51*Jinv00-6.7573051948051951e-04*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+1.1870941558441558e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+1.1870941558441558e-03*w6*Jinv00-2.6481331168831169e-04*Jinv20*w56-6.7573051948051951e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-5.4788961038961042e-05*Jinv20*w14-2.6481331168831169e-04*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+1.1870941558441558e-03*w7*Jinv10+2.3741883116883116e-03*Jinv20*w38-1.0227272727272728e-03*Jinv20*w53-5.4788961038961042e-05*w22*Jinv10-1.0227272727272728e-03*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+2.3741883116883116e-03*w37*Jinv10-6.7573051948051951e-04*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+6.5746753246753243e-04*Jinv20*w44+2.0495129870129870e-04*w57*Jinv00+2.0495129870129870e-04*Jinv20*w59+1.5726461038961039e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-6.7573051948051951e-04*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00+3.2873376623376621e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+1.1870941558441558e-03*w20*Jinv20-6.7573051948051951e-04*w5*Jinv20-2.6481331168831169e-04*w55*Jinv10+6.5746753246753243e-04*w42*Jinv00-1.0957792207792208e-04*w47*Jinv20)*D;
  A[37][11]=(0.0)*D;
  A[37][12]=(0.0)*D;
  A[37][13]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+8.7662337662337668e-04*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53-1.9176136363636362e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+8.7662337662337668e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-5.2049512987012989e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17-4.6570616883116886e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+7.1225649350649346e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-2.4655032467532466e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+8.2183441558441553e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+3.2873376623376621e-04*w15*Jinv01+2.7394480519480521e-05*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+7.1225649350649346e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+8.2183441558441553e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00-5.4788961038961036e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+8.7662337662337668e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20-1.9176136363636362e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+8.7662337662337668e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-4.6570616883116886e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+2.7394480519480521e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+8.7662337662337668e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01-1.9176136363636362e-04*w7*Jinv10-4.6570616883116886e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+3.5612824675324673e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-1.9176136363636362e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-1.9176136363636362e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+3.5612824675324673e-04*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-5.2049512987012989e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-1.9176136363636362e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-9.1314935064935065e-06*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-3.5612824675324673e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+8.7662337662337668e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+8.2183441558441553e-04*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+7.1225649350649346e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+2.7394480519480521e-05*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1)*D;
  A[37][14]=(0.0)*D;
  A[37][15]=(0.0)*D;
  A[37][16]=(-5.2962662337662338e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-1.0409902597402598e-03*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+9.3141233766233771e-04*w3*Jinv00-8.2183441558441553e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54+2.1367694805194807e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+7.1225649350649346e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+8.2183441558441553e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-1.9724025974025973e-03*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-2.1915584415584417e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12+1.3149350649350649e-03*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.7394480519480518e-04*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00+7.6704545454545450e-04*Jinv02*w45+8.2183441558441553e-04*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+1.7532467532467534e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+9.3141233766233771e-04*w4*Jinv10-8.2183441558441553e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-1.0409902597402598e-03*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.7532467532467534e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-04*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18+5.4788961038961036e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01-1.0409902597402598e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12+1.7532467532467534e-03*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+1.7532467532467534e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56+1.7532467532467534e-03*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-3.1047077922077924e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+1.7532467532467534e-03*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-4.2735389610389614e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+9.3141233766233771e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+8.2183441558441553e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1)*D;
  A[37][17]=(0.0)*D;
  A[37][18]=(0.0)*D;
  A[37][19]=(-3.5612824675324673e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+1.2327516233766234e-03*Jinv20*w8-3.5612824675324673e-04*w13*Jinv10+4.2735389610389614e-03*Jinv21*w44-1.0135957792207792e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.3833198051948050e-03*Jinv01*w3-8.2183441558441553e-05*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+7.8530844155844158e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+3.1777597402597403e-03*w43*Jinv10+4.2735389610389614e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17-7.1225649350649346e-04*Jinv01*w45+3.9265422077922079e-04*w58*Jinv10-8.2183441558441553e-05*w48*Jinv00+7.8530844155844158e-04*w58*Jinv11+7.8530844155844158e-04*w57*Jinv01-2.3833198051948050e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+3.7439123376623376e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+4.2735389610389614e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+3.9265422077922079e-04*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10-8.2183441558441553e-05*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+2.4655032467532467e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00+7.3965097402597406e-03*Jinv21*w38-2.3833198051948050e-03*Jinv01*w24+4.2735389610389614e-03*w34*Jinv11+3.6982548701298703e-03*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.2327516233766234e-03*w51*Jinv00-1.0135957792207792e-03*w4*Jinv10-2.3833198051948050e-03*w4*Jinv11+1.0957792207792207e-03*w41*Jinv20+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+3.6982548701298703e-03*Jinv21*w20+1.2327516233766234e-03*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20+4.2735389610389614e-03*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-7.1225649350649346e-04*w46*Jinv11-3.5612824675324673e-04*Jinv20*w14-2.3833198051948050e-03*w5*Jinv21-8.2183441558441553e-05*w49*Jinv10+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33+1.0957792207792207e-03*w39*Jinv00+3.6982548701298703e-03*Jinv01*w18+7.3965097402597406e-03*w37*Jinv11+3.6982548701298703e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10-7.1225649350649346e-04*w47*Jinv21+2.4655032467532467e-03*Jinv20*w38-2.6298701298701297e-03*w52*Jinv11-1.2327516233766234e-03*Jinv20*w53-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+7.3965097402597406e-03*w36*Jinv01-1.2327516233766234e-03*w52*Jinv10+3.6982548701298703e-03*w7*Jinv11-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56-1.2327516233766234e-03*w24*Jinv00-1.6436688311688311e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14-4.5657467532467534e-05*w10*Jinv10+3.1777597402597403e-03*Jinv20*w44-3.5612824675324673e-04*Jinv01*w12+3.6982548701298703e-03*Jinv21*w8+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+3.7439123376623376e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.2327516233766234e-03*w25*Jinv10+3.7439123376623376e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+1.5888798701298701e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-1.0135957792207792e-03*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-2.3833198051948050e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10+3.1777597402597403e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.6436688311688311e-04*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[37][20]=(0.0)*D;
  A[37][21]=(0.0)*D;
  A[37][22]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20+2.4655032467532466e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54+1.6436688311688311e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02-2.7394480519480521e-05*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+8.2183441558441553e-05*Jinv21*w26-2.7394480519480521e-05*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-8.2183441558441553e-04*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-2.7394480519480521e-05*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22-8.2183441558441553e-04*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10+2.4655032467532466e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+4.6570616883116886e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+4.6570616883116886e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14+2.4655032467532466e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-8.2183441558441553e-05*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*Jinv12*w7-8.2183441558441553e-05*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49-5.2049512987012989e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.7394480519480521e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02+3.8352272727272725e-04*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14-8.2183441558441553e-04*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20+4.6570616883116886e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1)*D;
  A[37][23]=(0.0)*D;
  A[37][24]=(0.0)*D;
  A[37][25]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+2.7394480519480518e-04*w13*Jinv10-1.6436688311688311e-03*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23+5.2049512987012989e-04*Jinv00*w54-3.1047077922077924e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-8.2183441558441553e-04*w16*Jinv11-8.2183441558441553e-04*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-3.1047077922077924e-04*w58*Jinv11-3.1047077922077924e-04*w57*Jinv01+5.7528409090909093e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-1.6436688311688311e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-8.2183441558441553e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+8.2183441558441553e-05*w31*Jinv10-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39-1.6436688311688311e-04*Jinv00*w18-7.1225649350649346e-04*w36*Jinv00-4.7666396103896100e-03*Jinv21*w38+5.7528409090909093e-04*Jinv01*w24-2.6298701298701297e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+1.4793019480519481e-03*Jinv21*w53+5.2049512987012989e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-1.6436688311688311e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+5.2049512987012989e-04*Jinv20*w56-1.6436688311688311e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20-2.6298701298701297e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.4655032467532466e-04*Jinv01*w18-4.7666396103896100e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21-7.1225649350649346e-04*Jinv20*w38+1.4793019480519481e-03*w52*Jinv11+5.2049512987012989e-04*Jinv20*w53-3.5612824675324673e-04*w22*Jinv10-8.2183441558441553e-05*w22*Jinv11-8.2183441558441553e-05*w21*Jinv01+1.4793019480519481e-03*Jinv01*w51-4.7666396103896100e-03*w36*Jinv01+5.2049512987012989e-04*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11+9.1314935064935065e-06*w9*Jinv00-7.1225649350649346e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14+9.1314935064935065e-06*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+2.4655032467532466e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-3.8352272727272725e-04*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+5.7528409090909093e-04*w25*Jinv11+5.2049512987012989e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[37][26]=(0.0)*D;
  A[37][27]=(0.0)*D;
  A[37][28]=(4.3831168831168834e-04*Jinv21*w44-4.2918019480519480e-04*Jinv01*w3-1.4204545454545455e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01-2.6481331168831169e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+4.3831168831168834e-04*w43*Jinv11+3.4699675324675324e-04*Jinv21*w41+1.0247564935064935e-04*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+3.4699675324675324e-04*Jinv01*w39+1.4975649350649350e-03*Jinv21*w38-2.6481331168831169e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11-5.8441558441558442e-04*Jinv21*w53-4.2918019480519480e-04*w4*Jinv11-2.1915584415584417e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11-4.2918019480519480e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+3.6525974025974026e-05*Jinv01*w18+1.4975649350649350e-03*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01-1.0957792207792208e-04*w47*Jinv21-5.8441558441558442e-04*w52*Jinv11-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-5.8441558441558442e-04*Jinv01*w51+1.4975649350649350e-03*w36*Jinv01+7.4878246753246752e-04*w7*Jinv11+1.1870941558441559e-04*Jinv21*w56-4.2918019480519480e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-2.7394480519480521e-05*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8-5.4788961038961042e-05*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+6.4935064935064935e-05*Jinv01*w9+3.4699675324675324e-04*w40*Jinv11-2.1915584415584417e-04*Jinv01*w30-2.6481331168831169e-04*w25*Jinv11-4.2918019480519480e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[37][29]=(0.0)*D;
  A[37][30]=(0.0)*D;
  A[37][31]=(3.5612824675324673e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.2601461038961039e-03*w40*Jinv12-9.5880681818181818e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-3.5612824675324673e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+1.2601461038961039e-03*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.0683847402597404e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-4.6570616883116886e-04*Jinv20*w23+1.4245129870129869e-03*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-1.8262987012987013e-05*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-1.1231737012987013e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+2.1915584415584417e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-1.9176136363636362e-04*w22*Jinv12+1.0683847402597404e-03*w31*Jinv10-4.6570616883116886e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.5612824675324673e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-1.4793019480519481e-03*Jinv21*w53-1.4793019480519481e-03*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+7.1225649350649346e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10+1.0683847402597404e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00+1.4245129870129869e-03*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+1.2601461038961039e-03*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-1.8262987012987013e-05*w11*Jinv20-9.5880681818181818e-04*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.0683847402597404e-03*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-1.1231737012987013e-03*Jinv02*w54-1.9176136363636362e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+2.1915584415584417e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+7.1225649350649346e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-1.9176136363636362e-04*Jinv22*w23-1.4793019480519481e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12-1.4793019480519481e-03*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-4.6570616883116886e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50-4.6570616883116886e-04*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7-4.6570616883116886e-04*w21*Jinv01-1.4793019480519481e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-1.4793019480519481e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-1.8262987012987013e-05*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-1.3971185064935066e-03*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-1.6436688311688311e-04*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.8262987012987013e-05*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-3.5612824675324673e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-4.6570616883116886e-04*Jinv21*w23-1.3971185064935066e-03*Jinv01*w54-1.3971185064935066e-03*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02-1.8262987012987013e-05*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+1.0683847402597404e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29+1.3149350649350649e-03*w40*Jinv11+1.4245129870129869e-03*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+1.0683847402597404e-03*Jinv01*w30-9.5880681818181818e-04*Jinv02*w51-1.1231737012987013e-03*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50+2.1915584415584417e-04*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1-1.8262987012987013e-05*w10*Jinv11)*D;
  A[37][32]=(0.0)*D;
  A[37][33]=(0.0)*D;
  A[37][34]=(-9.6793831168831166e-04*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12+2.7942370129870131e-03*Jinv22*w53-2.0819805194805196e-03*Jinv20*w8-1.6436688311688311e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-2.9586038961038961e-03*Jinv21*w44+1.8628246753246754e-03*w3*Jinv00+2.6298701298701297e-03*Jinv01*w3+8.2183441558441553e-04*Jinv20*w23-8.5470779220779228e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-6.5746753246753243e-04*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-5.4788961038961036e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-2.9586038961038961e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+1.7532467532467534e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10+5.4788961038961036e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-6.5746753246753243e-04*w58*Jinv11-6.5746753246753243e-04*w57*Jinv01+2.6298701298701297e-03*Jinv21*w26-5.4788961038961036e-04*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-2.9586038961038961e-03*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00+1.9724025974025973e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-9.6793831168831166e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-5.4788961038961036e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-5.4788961038961036e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-9.6793831168831166e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-6.9034090909090911e-03*Jinv00*w33+4.9310064935064932e-04*Jinv02*w3+8.2183441558441553e-04*w21*Jinv00-6.0267857142857139e-04*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-9.5332792207792200e-03*w36*Jinv00-1.2820616883116883e-02*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+2.6298701298701297e-03*Jinv01*w24-1.6436688311688311e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+4.9310064935064934e-03*Jinv21*w53+2.7942370129870131e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+1.8628246753246754e-03*w4*Jinv10+2.6298701298701297e-03*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-6.9034090909090911e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-2.0819805194805196e-03*w6*Jinv00-8.5470779220779228e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41+2.5750811688311690e-03*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.7942370129870131e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11+2.6298701298701297e-03*w5*Jinv21+1.7532467532467534e-03*Jinv02*w54+4.9310064935064932e-04*w4*Jinv12-4.9310064935064932e-04*w49*Jinv10-5.4788961038961036e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33-2.9586038961038961e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-1.2820616883116883e-02*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01-2.0819805194805196e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-9.5332792207792200e-03*Jinv20*w38+4.9310064935064934e-03*w52*Jinv11-6.0267857142857139e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.7942370129870131e-03*Jinv20*w53+8.2183441558441553e-04*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+5.4788961038961042e-05*w21*Jinv01+4.9310064935064934e-03*Jinv01*w51-6.0267857142857139e-04*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49-1.2820616883116883e-02*w36*Jinv01+2.7942370129870131e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-9.5332792207792200e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56+2.5750811688311690e-03*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27-2.1367694805194807e-03*w19*Jinv12+4.9310064935064932e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-1.6436688311688311e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00-6.9034090909090911e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11+2.5750811688311690e-03*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02+1.0957792207792207e-03*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-2.9586038961038961e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-8.5470779220779228e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+1.8628246753246754e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.7942370129870131e-03*Jinv02*w51+1.7532467532467534e-03*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28+2.6298701298701297e-03*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20+2.1367694805194807e-03*Jinv21*w50-5.4788961038961036e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+1.0957792207792207e-03*w10*Jinv11)*D;
  A[37][35]=(0.0)*D;
  A[37][36]=(0.0)*D;
  A[37][37]=(-3.5612824675324673e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.6298701298701297e-03*Jinv22*w53+2.4655032467532467e-03*Jinv20*w8+7.8530844155844158e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-2.0271915584415584e-03*w3*Jinv00-1.0683847402597404e-03*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+7.3965097402597406e-03*Jinv22*w38+5.7528409090909093e-04*Jinv00*w54-2.3833198051948050e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+4.7666396103896100e-03*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+6.5746753246753243e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+5.7528409090909093e-04*w48*Jinv00-2.3833198051948050e-03*Jinv02*w24+4.2735389610389614e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+3.6982548701298703e-03*Jinv22*w20-1.0683847402597404e-03*w45*Jinv00+4.2735389610389614e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22+5.7528409090909093e-04*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-1.0683847402597404e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+4.7666396103896100e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-1.0683847402597404e-03*w31*Jinv10-2.3833198051948050e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2+2.4655032467532467e-03*Jinv00*w18+7.3965097402597406e-03*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45+7.8530844155844158e-04*Jinv22*w59-1.3149350649350649e-03*w51*Jinv00+4.2735389610389614e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-2.0271915584415584e-03*w4*Jinv10+1.6436688311688311e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13+2.4655032467532467e-03*w19*Jinv10+4.7666396103896100e-03*w34*Jinv10+2.4655032467532467e-03*w6*Jinv00+7.3965097402597406e-03*w37*Jinv12+5.7528409090909093e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-2.0271915584415584e-03*w26*Jinv20-9.1314935064935068e-05*w11*Jinv20-2.6298701298701297e-03*w52*Jinv12-2.3833198051948050e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54-2.3833198051948050e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+5.7528409090909093e-04*w49*Jinv10+1.6436688311688311e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+2.4655032467532467e-03*w7*Jinv10+7.3965097402597406e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53+3.6982548701298703e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50+3.6982548701298703e-03*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.3149350649350649e-03*w52*Jinv10-9.1314935064935068e-05*w9*Jinv00+7.3965097402597406e-03*w37*Jinv10-2.0271915584415584e-03*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+3.6982548701298703e-03*w6*Jinv02+3.6982548701298703e-03*w19*Jinv12-2.3833198051948050e-03*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22-9.1314935064935068e-05*w10*Jinv10+4.7666396103896100e-03*Jinv20*w44+7.8530844155844158e-04*w58*Jinv12+3.6982548701298703e-03*Jinv02*w18+6.5746753246753243e-04*w57*Jinv00+6.5746753246753243e-04*Jinv20*w59+4.2735389610389614e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+4.7666396103896100e-03*w35*Jinv20-2.0271915584415584e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-1.0683847402597404e-03*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14+7.3965097402597406e-03*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-2.0271915584415584e-03*w5*Jinv20-2.6298701298701297e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28+5.7528409090909093e-04*w55*Jinv10+4.7666396103896100e-03*w42*Jinv00-1.0683847402597404e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[37][38]=(0.0)*D;
  A[37][39]=(0.0)*D;
  A[37][40]=(-7.6704545454545450e-04*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10+1.6436688311688311e-03*w40*Jinv12+1.7532467532467534e-03*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10+1.6436688311688311e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-2.1915584415584414e-03*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+3.8352272727272725e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+3.4699675324675324e-04*Jinv21*w2+3.9448051948051946e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+1.3149350649350649e-03*Jinv01*w45+2.5750811688311690e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+4.9310064935064932e-04*w10*Jinv12+1.3149350649350649e-03*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35+7.1225649350649346e-04*Jinv21*w26+3.4699675324675324e-04*w0*Jinv01+4.2004870129870131e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+2.6298701298701297e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22+3.4699675324675324e-04*Jinv11*w1-8.2183441558441553e-04*w13*Jinv11-3.4699675324675324e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+1.9724025974025973e-03*Jinv00*w33-1.1505681818181819e-03*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-1.2053571428571428e-03*w32*Jinv22+3.8352272727272725e-04*Jinv22*w2-2.1915584415584414e-03*Jinv00*w18-3.2873376623376621e-03*w36*Jinv00+1.0957792207792208e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+2.1367694805194807e-03*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-8.2183441558441553e-04*Jinv12*w13-2.1915584415584414e-03*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20-1.0957792207792207e-03*w6*Jinv00-2.1915584415584414e-03*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56+1.6436688311688311e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20+9.8620129870129864e-04*w11*Jinv20+1.7532467532467534e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+3.8352272727272725e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21+2.5750811688311690e-03*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+1.3149350649350649e-03*w49*Jinv10-3.4699675324675324e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00-8.2183441558441553e-04*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18+4.9310064935064932e-04*Jinv02*w9+1.0957792207792207e-03*w6*Jinv01-1.0957792207792207e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-3.2873376623376621e-03*Jinv20*w38-1.2053571428571428e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53+5.4788961038961042e-05*w22*Jinv10+1.2601461038961039e-03*Jinv22*w50+8.2183441558441553e-04*w22*Jinv11+8.2183441558441553e-04*w21*Jinv01-1.2053571428571428e-03*Jinv02*w30+1.2601461038961039e-03*Jinv12*w49+2.1367694805194807e-03*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+9.8620129870129864e-04*w9*Jinv00-3.2873376623376621e-03*w37*Jinv10+1.3149350649350649e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-1.0957792207792207e-03*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-8.2183441558441553e-04*Jinv21*w14+9.8620129870129864e-04*w10*Jinv10+3.9448051948051946e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-8.2183441558441553e-04*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33+4.2004870129870131e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20+8.2183441558441553e-04*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10+4.2004870129870131e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.2601461038961039e-03*w48*Jinv02+1.3149350649350649e-03*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-2.1915584415584414e-03*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-8.2183441558441553e-04*Jinv22*w14-2.1915584415584414e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30+1.7532467532467534e-03*Jinv02*w51+2.5750811688311690e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+3.9448051948051946e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*Jinv21*w50-3.4699675324675324e-04*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1)*D;
  A[37][41]=(0.0)*D;
  A[37][42]=(0.0)*D;
  A[37][43]=(7.6704545454545450e-04*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00+1.1505681818181819e-03*w32*Jinv20-2.6298701298701297e-03*Jinv01*w3-5.4788961038961042e-05*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+6.5746753246753243e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+5.4788961038961036e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-7.6704545454545450e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+6.5746753246753243e-04*w58*Jinv11+6.5746753246753243e-04*w57*Jinv01-2.6298701298701297e-03*Jinv21*w26+5.4788961038961036e-04*w0*Jinv01-4.2004870129870131e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+5.4788961038961036e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+5.4788961038961036e-04*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+1.1505681818181819e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2+2.1915584415584414e-03*Jinv00*w18+3.2873376623376621e-03*w36*Jinv00+1.2820616883116883e-02*Jinv21*w38-7.6704545454545450e-04*Jinv02*w45-2.6298701298701297e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+2.9586038961038961e-03*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-4.9310064935064934e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-2.6298701298701297e-03*w4*Jinv11-4.9310064935064934e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+2.1915584415584414e-03*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20-9.8620129870129864e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12+2.9586038961038961e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14-2.6298701298701297e-03*w5*Jinv21-7.6704545454545450e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-1.3149350649350649e-03*w49*Jinv10+5.4788961038961036e-04*Jinv21*w29+2.9586038961038961e-03*Jinv01*w33-4.9310064935064934e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12+4.2735389610389614e-03*Jinv01*w18+1.2820616883116883e-02*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9+4.2735389610389614e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+3.2873376623376621e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-4.9310064935064934e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-4.9310064935064934e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49+1.2820616883116883e-02*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-9.8620129870129864e-04*w9*Jinv00+3.2873376623376621e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56-7.6704545454545450e-04*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-9.8620129870129864e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-4.2004870129870131e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11-7.6704545454545450e-04*w25*Jinv10-4.2004870129870131e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+1.1505681818181819e-03*w30*Jinv00-4.9310064935064934e-03*w40*Jinv10+2.1915584415584414e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11+4.2735389610389614e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30-2.1367694805194807e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-2.6298701298701297e-03*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10-3.9448051948051946e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50+5.4788961038961036e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[37][44]=(0.0)*D;
  A[37][45]=(0.0)*D;
  A[37][46]=(-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-1.3697240259740259e-04*w57*Jinv02-5.4788961038961042e-05*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-2.1915584415584417e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01+1.8262987012987013e-05*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-1.0683847402597404e-03*Jinv01*w45+5.2049512987012989e-04*Jinv12*w55-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-2.1915584415584417e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-2.1915584415584417e-04*Jinv11*w1+4.6570616883116886e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.8352272727272725e-04*Jinv22*w17-3.5612824675324673e-04*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+8.2183441558441553e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-7.1225649350649346e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-1.3697240259740259e-04*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+1.4793019480519481e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv11+2.7394480519480518e-04*Jinv12*w13+2.4655032467532466e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12-5.4788961038961042e-05*Jinv22*w41+5.2049512987012989e-04*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-1.0683847402597404e-03*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21+5.2049512987012989e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33+2.7394480519480518e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-1.0683847402597404e-03*w47*Jinv21-3.5612824675324673e-04*Jinv22*w23+1.4793019480519481e-03*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+1.4793019480519481e-03*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+8.2183441558441553e-05*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.3971185064935066e-03*w49*Jinv11+4.6570616883116886e-04*Jinv21*w14-1.3697240259740259e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+4.6570616883116886e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02+1.8262987012987013e-05*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+2.4655032467532466e-04*Jinv01*w30+5.2049512987012989e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-3.8352272727272725e-04*w16*Jinv12+1.3971185064935066e-03*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+1.3971185064935066e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1+1.8262987012987013e-05*w10*Jinv11)*D;
  A[37][47]=(0.0)*D;
  A[37][48]=(0.0)*D;
  A[37][49]=(-2.7394480519480518e-04*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10-1.2053571428571428e-03*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8+6.6659902597402602e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-1.2053571428571428e-03*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+3.2873376623376621e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+4.7483766233766234e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+1.3697240259740260e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+2.3833198051948050e-03*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24-6.0267857142857139e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.3833198051948050e-03*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33+1.9176136363636362e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10-3.8352272727272725e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-9.5880681818181818e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18-1.1505681818181819e-03*w36*Jinv00-1.1505681818181819e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+6.6659902597402602e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-6.0267857142857139e-04*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-2.7394480519480521e-05*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+3.2873376623376621e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-1.2053571428571428e-03*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20+4.7483766233766234e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+1.3697240259740260e-03*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12+1.9176136363636362e-04*w21*Jinv02+2.3833198051948050e-03*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-2.7394480519480521e-05*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18-1.1505681818181819e-03*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.1505681818181819e-03*Jinv20*w38+1.9176136363636362e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11-9.5880681818181818e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-9.5880681818181818e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-1.1505681818181819e-03*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11+4.7483766233766234e-04*w9*Jinv00-1.1505681818181819e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.3833198051948050e-03*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+4.7483766233766234e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+6.6659902597402602e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-6.0267857142857139e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+4.7483766233766234e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29-2.7394480519480521e-05*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+3.2873376623376621e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30+1.3971185064935066e-03*Jinv02*w51+1.3697240259740260e-03*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+2.3833198051948050e-03*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.3833198051948050e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+4.7483766233766234e-04*w10*Jinv11)*D;
  A[37][50]=(0.0)*D;
  A[37][51]=(0.0)*D;
  A[37][52]=(2.7394480519480518e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-1.4610389610389610e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+5.2049512987012989e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23+3.7804383116883116e-03*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+8.2183441558441553e-05*w25*Jinv12-1.5523538961038962e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10-1.2327516233766234e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-8.2183441558441553e-04*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-6.5746753246753243e-04*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-8.2183441558441553e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.5523538961038962e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00+4.3831168831168834e-04*Jinv02*w45-1.4610389610389610e-04*Jinv22*w59-2.6298701298701297e-03*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv10-1.3149350649350649e-03*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00+3.7804383116883116e-03*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+5.2049512987012989e-04*w26*Jinv20-4.7483766233766234e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+8.2183441558441553e-05*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-1.2327516233766234e-03*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-1.3149350649350649e-03*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-1.0957792207792207e-03*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10+1.1505681818181819e-03*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23+4.3831168831168834e-04*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12-2.6298701298701297e-03*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50-2.4655032467532466e-04*Jinv12*w7+4.3831168831168834e-04*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49-2.6298701298701297e-03*w52*Jinv10-4.7483766233766234e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10+5.2049512987012989e-04*w24*Jinv00-1.5523538961038962e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-4.7483766233766234e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.4610389610389610e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+5.2049512987012989e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-1.5523538961038962e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14+3.7804383116883116e-03*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+5.2049512987012989e-04*w5*Jinv20-1.3149350649350649e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56-1.5523538961038962e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-8.2183441558441553e-04*w16*Jinv12-1.5523538961038962e-04*Jinv12*w1)*D;
  A[37][53]=(0.0)*D;
  A[37][54]=(0.0)*D;
  A[37][55]=(-2.5202922077922079e-03*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53+1.1870941558441559e-04*w57*Jinv02-2.5202922077922079e-03*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+1.4793019480519481e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59+1.3697240259740259e-04*w25*Jinv12-3.1960227272727273e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01-4.7483766233766234e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-2.0271915584415584e-03*Jinv12*w55-7.1225649350649346e-04*w15*Jinv02-4.8396915584415583e-04*w10*Jinv12+1.3697240259740259e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+3.8352272727272725e-04*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41-2.7394480519480520e-03*w42*Jinv02-4.8396915584415583e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11-7.1225649350649346e-04*Jinv22*w17+2.4655032467532466e-04*w22*Jinv12+2.4655032467532466e-04*Jinv02*w3+8.4922889610389610e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-3.1960227272727273e-04*Jinv22*w2+1.1505681818181819e-03*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45+5.2049512987012989e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-2.7394480519480520e-03*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+4.6570616883116886e-04*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20+1.4793019480519481e-03*w37*Jinv12-2.5202922077922079e-03*Jinv22*w41-1.2327516233766234e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11+1.3697240259740259e-04*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-2.0271915584415584e-03*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+4.6570616883116886e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18+1.1505681818181819e-03*w37*Jinv11-4.8396915584415583e-04*Jinv02*w9-1.6436688311688311e-04*w6*Jinv01+1.3971185064935066e-03*w47*Jinv21+2.4655032467532466e-04*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11+8.4922889610389610e-04*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+1.9176136363636362e-04*Jinv22*w8-1.0135957792207792e-03*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11+1.9176136363636362e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+8.4922889610389610e-04*Jinv02*w30-1.0135957792207792e-03*Jinv12*w49+1.1505681818181819e-03*w36*Jinv01-1.6436688311688311e-04*w7*Jinv11-2.3833198051948050e-03*Jinv21*w56-1.7349837662337662e-04*Jinv02*w27+1.9176136363636362e-04*w6*Jinv02+3.8352272727272725e-04*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-2.3833198051948050e-03*Jinv01*w54-2.3833198051948050e-03*w55*Jinv11-1.0135957792207792e-03*w48*Jinv02-4.7483766233766234e-04*Jinv01*w9-1.7349837662337662e-04*Jinv22*w29+4.6570616883116886e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+1.4793019480519481e-03*Jinv02*w36-2.7394480519480520e-03*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-2.0271915584415584e-03*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28+5.2049512987012989e-04*w25*Jinv11-7.1225649350649346e-04*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-3.1960227272727273e-04*Jinv12*w1-4.7483766233766234e-04*w10*Jinv11)*D;
  A[37][56]=(0.0)*D;
  A[37][57]=(0.0)*D;
  A[37][58]=(2.9220779220779221e-04*w40*Jinv12-1.0227272727272728e-03*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02+2.9220779220779221e-04*Jinv02*w39-2.5568181818181820e-04*Jinv22*w38+1.1870941558441559e-04*w25*Jinv12-1.1668019480519480e-04*w0*Jinv02-6.7573051948051951e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02+8.3198051948051951e-05*w10*Jinv12+1.1870941558441559e-04*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35+3.6525974025974026e-05*Jinv22*w20-3.2873376623376621e-04*w42*Jinv02+8.3198051948051951e-05*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17-2.7394480519480521e-05*w22*Jinv12+1.1870941558441559e-04*Jinv02*w3+4.9310064935064932e-04*w32*Jinv22-1.1668019480519480e-04*Jinv22*w2+4.9310064935064932e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59-3.2873376623376621e-04*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-2.7394480519480521e-05*Jinv12*w13-2.5568181818181820e-04*w37*Jinv12+2.9220779220779221e-04*Jinv22*w41-1.0227272727272728e-03*w52*Jinv12+1.1870941558441559e-04*Jinv22*w26-6.7573051948051951e-04*Jinv02*w54+1.1870941558441559e-04*w4*Jinv12-2.7394480519480521e-05*w21*Jinv02-2.7394480519480521e-05*Jinv02*w12+8.3198051948051951e-05*Jinv02*w9-2.7394480519480521e-05*Jinv22*w23+4.9310064935064932e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+3.6525974025974026e-05*Jinv22*w8-6.7573051948051951e-04*Jinv22*w50+3.6525974025974026e-05*Jinv12*w7+4.9310064935064932e-04*Jinv02*w30-6.7573051948051951e-04*Jinv12*w49-1.1668019480519480e-04*Jinv02*w27+3.6525974025974026e-05*w6*Jinv02+3.6525974025974026e-05*w19*Jinv12+1.1870941558441559e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12+3.6525974025974026e-05*Jinv02*w18-3.2873376623376621e-04*Jinv02*w33-6.7573051948051951e-04*w48*Jinv02-1.1668019480519480e-04*Jinv22*w29-2.7394480519480521e-05*Jinv22*w14-2.5568181818181820e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-1.0227272727272728e-03*Jinv02*w51-6.7573051948051951e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12-1.1668019480519480e-04*Jinv12*w1)*D;
  A[37][59]=(0.0)*D;
  A[38][0]=(0.0)*D;
  A[38][1]=(0.0)*D;
  A[38][2]=(5.4788961038961042e-05*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-3.4699675324675324e-04*w40*Jinv12+5.8441558441558442e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-3.4699675324675324e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+2.6481331168831169e-04*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20+2.6481331168831169e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23-1.4975649350649350e-03*Jinv22*w38+4.2918019480519480e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45+4.2918019480519480e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-2.1915584415584417e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+2.1915584415584417e-04*w45*Jinv00-3.4699675324675324e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27-6.4935064935064935e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10-2.1915584415584417e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+2.6481331168831169e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22-3.4699675324675324e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2-7.4878246753246752e-04*Jinv00*w18-1.4975649350649350e-03*w36*Jinv00-1.4975649350649350e-03*Jinv21*w38+2.1915584415584417e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+5.8441558441558442e-04*Jinv21*w53+5.8441558441558442e-04*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+2.6481331168831169e-04*w4*Jinv10+2.6481331168831169e-04*w4*Jinv11-3.4699675324675324e-04*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-7.4878246753246752e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00-1.4975649350649350e-03*w37*Jinv12+4.2918019480519480e-04*Jinv20*w56-3.4699675324675324e-04*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20+5.8441558441558442e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+2.6481331168831169e-04*w5*Jinv21+4.2918019480519480e-04*Jinv02*w54+2.6481331168831169e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-3.4699675324675324e-04*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-1.4975649350649350e-03*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21-1.4975649350649350e-03*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23+5.8441558441558442e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12+2.1915584415584417e-04*w46*Jinv12+5.8441558441558442e-04*Jinv20*w53-3.6525974025974026e-05*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-3.6525974025974026e-05*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+5.8441558441558442e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49-1.4975649350649350e-03*w36*Jinv01+5.8441558441558442e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00-1.4975649350649350e-03*w37*Jinv10+4.2918019480519480e-04*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-3.6525974025974026e-05*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+2.6481331168831169e-04*Jinv22*w5+2.1915584415584417e-04*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23+4.2918019480519480e-04*Jinv01*w54+4.2918019480519480e-04*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-3.4699675324675324e-04*w40*Jinv10-7.4878246753246752e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-3.4699675324675324e-04*w40*Jinv11-1.4975649350649350e-03*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+2.6481331168831169e-04*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+5.8441558441558442e-04*Jinv02*w51+4.2918019480519480e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11+4.2918019480519480e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20-2.1915584415584417e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[38][3]=(0.0)*D;
  A[38][4]=(0.0)*D;
  A[38][5]=(-2.7394480519480518e-04*Jinv00*w12+3.1960227272727273e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12-1.4793019480519481e-03*Jinv22*w53+8.2183441558441553e-05*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-7.3965097402597403e-04*w3*Jinv00-5.4788961038961036e-04*w32*Jinv20-5.7528409090909093e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23+4.7666396103896100e-03*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+3.1047077922077924e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.5750811688311690e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+8.2183441558441553e-04*w16*Jinv11+8.2183441558441553e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.7349837662337662e-04*w58*Jinv10+8.2183441558441553e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+1.6436688311688311e-03*Jinv22*w35+3.1047077922077924e-04*w58*Jinv11+3.1047077922077924e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+4.3831168831168834e-04*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-4.6570616883116886e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+3.1960227272727273e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+8.2183441558441553e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+3.1960227272727273e-04*Jinv20*w29-4.6570616883116886e-04*w46*Jinv10+8.2183441558441553e-04*Jinv22*w17+8.7662337662337668e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12-5.4788961038961036e-04*w31*Jinv10-5.7528409090909093e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2+2.0271915584415584e-03*Jinv00*w18+4.0543831168831168e-03*w36*Jinv00+4.7666396103896100e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.4793019480519481e-03*Jinv21*w53-9.5880681818181818e-04*w51*Jinv00+1.6436688311688311e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-7.3965097402597403e-04*w4*Jinv10-5.7528409090909093e-04*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13+2.0271915584415584e-03*w19*Jinv10+8.7662337662337668e-04*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00+4.7666396103896100e-03*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-1.1231737012987013e-03*w26*Jinv20+5.5702110389610384e-04*w11*Jinv20-1.4793019480519481e-03*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.7528409090909093e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-5.7528409090909093e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+4.7666396103896100e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21+4.0543831168831168e-03*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23-1.4793019480519481e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-9.5880681818181818e-04*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01-1.4793019480519481e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49+4.7666396103896100e-03*w36*Jinv01-9.5880681818181818e-04*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11+5.5702110389610384e-04*w9*Jinv00+4.0543831168831168e-03*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56-1.1231737012987013e-03*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-5.7528409090909093e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-1.3697240259740259e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+5.5702110389610384e-04*w10*Jinv10+2.5750811688311690e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8+1.7349837662337662e-04*w57*Jinv00+1.7349837662337662e-04*Jinv20*w59+1.6436688311688311e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+8.7662337662337668e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11-1.1231737012987013e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+4.3831168831168834e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+4.3831168831168834e-04*Jinv20*w17-5.4788961038961036e-04*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10+2.0271915584415584e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11+4.7666396103896100e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-7.3965097402597403e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30-1.4793019480519481e-03*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11-2.4655032467532466e-04*w55*Jinv10+2.5750811688311690e-03*w42*Jinv00-4.6570616883116886e-04*w47*Jinv20+8.2183441558441553e-04*w16*Jinv12-1.3697240259740259e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27-1.3697240259740259e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[38][6]=(0.0)*D;
  A[38][7]=(0.0)*D;
  A[38][8]=(2.7394480519480518e-04*Jinv00*w12-3.7439123376623376e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12+2.6298701298701297e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-7.8530844155844158e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+1.1505681818181819e-03*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.3833198051948050e-03*Jinv01*w3-7.3965097402597406e-03*Jinv22*w38+8.2183441558441553e-05*Jinv00*w54-7.8530844155844158e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-2.6846590909090908e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21-2.1367694805194807e-03*w16*Jinv11-2.1367694805194807e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-3.9265422077922079e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12+2.3833198051948050e-03*Jinv02*w24-4.2735389610389614e-03*Jinv22*w35-7.8530844155844158e-04*w58*Jinv11-7.8530844155844158e-04*w57*Jinv01+2.3833198051948050e-03*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-3.5612824675324673e-04*w1*Jinv10-5.4788961038961036e-04*w16*Jinv10-3.6982548701298703e-03*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+3.5612824675324673e-04*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-3.7439123376623376e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11-2.1367694805194807e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.7439123376623376e-04*Jinv20*w29+3.5612824675324673e-04*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10+2.3833198051948050e-03*Jinv02*w3+7.1225649350649346e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2-2.4655032467532467e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-7.3965097402597406e-03*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+2.3833198051948050e-03*Jinv01*w24-7.8530844155844158e-04*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11-3.6982548701298703e-03*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+1.3971185064935066e-03*w51*Jinv00-4.2735389610389614e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+1.1505681818181819e-03*w4*Jinv10+2.3833198051948050e-03*w4*Jinv11+3.5612824675324673e-04*Jinv12*w13-2.4655032467532467e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+7.1225649350649346e-04*w32*Jinv21-3.6982548701298703e-03*Jinv21*w20-1.2327516233766234e-03*w6*Jinv00-7.3965097402597406e-03*w37*Jinv12+8.2183441558441553e-05*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.3697240259740260e-03*w26*Jinv20-1.1414366883116883e-03*w11*Jinv20+2.6298701298701297e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+7.1225649350649346e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+2.3833198051948050e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54+2.3833198051948050e-03*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33+3.5612824675324673e-04*Jinv02*w12-3.6982548701298703e-03*Jinv01*w18-7.3965097402597406e-03*w37*Jinv11-1.0957792207792207e-03*Jinv02*w9-3.6982548701298703e-03*w6*Jinv01-1.2327516233766234e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+1.3971185064935066e-03*Jinv20*w53-3.6982548701298703e-03*Jinv22*w8+1.6436688311688311e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11-3.6982548701298703e-03*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49-7.3965097402597406e-03*w36*Jinv01+1.3971185064935066e-03*w52*Jinv10-3.6982548701298703e-03*w7*Jinv11-1.1414366883116883e-03*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56+1.3697240259740260e-03*w24*Jinv00-7.4878246753246752e-04*Jinv02*w27-3.6982548701298703e-03*w6*Jinv02-3.6982548701298703e-03*w19*Jinv12+2.3833198051948050e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.1414366883116883e-03*w10*Jinv10-2.6846590909090908e-03*Jinv20*w44-7.8530844155844158e-04*w58*Jinv12-3.6982548701298703e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-3.6982548701298703e-03*Jinv21*w8-3.9265422077922079e-04*w57*Jinv00-3.9265422077922079e-04*Jinv20*w59-4.2735389610389614e-03*Jinv02*w33-3.5612824675324673e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11+1.3697240259740260e-03*w25*Jinv10-3.5612824675324673e-04*Jinv20*w2-5.4788961038961036e-04*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-5.4788961038961036e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-2.4655032467532467e-03*w20*Jinv20-7.4878246753246752e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11-7.3965097402597406e-03*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+1.1505681818181819e-03*w5*Jinv20+7.1225649350649346e-04*Jinv01*w30+2.6298701298701297e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-7.4878246753246752e-04*Jinv12*w28+2.3833198051948050e-03*w25*Jinv11+8.2183441558441553e-05*w55*Jinv10-2.6846590909090908e-03*w42*Jinv00+3.5612824675324673e-04*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01-7.4878246753246752e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[38][9]=(0.0)*D;
  A[38][10]=(0.0)*D;
  A[38][11]=(-5.4788961038961042e-05*Jinv00*w12+1.5726461038961039e-04*w28*Jinv10+1.1870941558441558e-03*Jinv20*w8-5.4788961038961042e-05*w13*Jinv10-6.7573051948051951e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20-5.4788961038961042e-05*Jinv20*w23-2.6481331168831169e-04*Jinv00*w54+6.5746753246753243e-04*w43*Jinv10+2.0495129870129870e-04*w58*Jinv10-2.6481331168831169e-04*w48*Jinv00+1.5726461038961039e-04*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-1.0957792207792208e-04*w45*Jinv00+1.5726461038961039e-04*Jinv00*w27-2.6481331168831169e-04*Jinv20*w50+1.5726461038961039e-04*Jinv20*w29-1.0957792207792208e-04*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10-5.4788961038961042e-05*w21*Jinv00+1.1870941558441558e-03*Jinv00*w18+2.3741883116883116e-03*w36*Jinv00-1.0227272727272728e-03*w51*Jinv00-6.7573051948051951e-04*w4*Jinv10-3.1047077922077924e-04*w41*Jinv20+1.1870941558441558e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+1.1870941558441558e-03*w6*Jinv00-2.6481331168831169e-04*Jinv20*w56-6.7573051948051951e-04*w26*Jinv20+6.4935064935064935e-04*w11*Jinv20-5.4788961038961042e-05*Jinv20*w14-2.6481331168831169e-04*w49*Jinv10-3.1047077922077924e-04*w39*Jinv00+1.1870941558441558e-03*w7*Jinv10+2.3741883116883116e-03*Jinv20*w38-1.0227272727272728e-03*Jinv20*w53-5.4788961038961042e-05*w22*Jinv10-1.0227272727272728e-03*w52*Jinv10+6.4935064935064935e-04*w9*Jinv00+2.3741883116883116e-03*w37*Jinv10-6.7573051948051951e-04*w24*Jinv00+6.4935064935064935e-04*w10*Jinv10+6.5746753246753243e-04*Jinv20*w44+2.0495129870129870e-04*w57*Jinv00+2.0495129870129870e-04*Jinv20*w59+1.5726461038961039e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-6.7573051948051951e-04*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00+3.2873376623376621e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-3.1047077922077924e-04*w40*Jinv10+1.1870941558441558e-03*w20*Jinv20-6.7573051948051951e-04*w5*Jinv20-2.6481331168831169e-04*w55*Jinv10+6.5746753246753243e-04*w42*Jinv00-1.0957792207792208e-04*w47*Jinv20)*D;
  A[38][12]=(0.0)*D;
  A[38][13]=(0.0)*D;
  A[38][14]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+8.7662337662337668e-04*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53-1.9176136363636362e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+8.7662337662337668e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+3.5612824675324673e-04*w32*Jinv20-8.2183441558441553e-05*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-5.2049512987012989e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17-4.6570616883116886e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+7.1225649350649346e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-2.4655032467532466e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+8.2183441558441553e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+3.2873376623376621e-04*w15*Jinv01+2.7394480519480521e-05*w28*Jinv11-2.4655032467532466e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+7.1225649350649346e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12+3.5612824675324673e-04*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+8.2183441558441553e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00-5.4788961038961036e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-8.2183441558441553e-05*w4*Jinv11+8.7662337662337668e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20-1.9176136363636362e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+8.7662337662337668e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-4.6570616883116886e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-8.2183441558441553e-05*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10+2.7394480519480521e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+8.7662337662337668e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01-1.9176136363636362e-04*w7*Jinv10-4.6570616883116886e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+3.5612824675324673e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-1.9176136363636362e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-1.9176136363636362e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+3.5612824675324673e-04*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-5.2049512987012989e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-1.9176136363636362e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-9.1314935064935065e-06*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-3.5612824675324673e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+7.1225649350649346e-04*Jinv20*w17+3.5612824675324673e-04*w30*Jinv00+8.7662337662337668e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+8.2183441558441553e-04*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+7.1225649350649346e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+2.7394480519480521e-05*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1)*D;
  A[38][15]=(0.0)*D;
  A[38][16]=(0.0)*D;
  A[38][17]=(-5.2962662337662338e-04*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-1.0409902597402598e-03*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+1.3149350649350649e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+9.3141233766233771e-04*w3*Jinv00-8.2183441558441553e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54+2.1367694805194807e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-3.2873376623376621e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+7.1225649350649346e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55-5.2962662337662338e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+8.2183441558441553e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01-3.1047077922077924e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-1.9724025974025973e-03*w42*Jinv02-5.2962662337662338e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-2.1915584415584417e-04*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50-5.2962662337662338e-04*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12+1.3149350649350649e-03*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.7394480519480518e-04*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-3.1777597402597403e-03*w36*Jinv00+7.6704545454545450e-04*Jinv02*w45+8.2183441558441553e-04*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+1.7532467532467534e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+9.3141233766233771e-04*w4*Jinv10-8.2183441558441553e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-1.0409902597402598e-03*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.7532467532467534e-03*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-04*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18+5.4788961038961036e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01-1.0409902597402598e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-3.1777597402597403e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12+1.7532467532467534e-03*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+1.7532467532467534e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-3.1777597402597403e-03*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56+1.7532467532467534e-03*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-3.2873376623376621e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8-5.2962662337662338e-04*w57*Jinv00-5.2962662337662338e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-3.1047077922077924e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11+1.7532467532467534e-03*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-4.2735389610389614e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+9.3141233766233771e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+8.2183441558441553e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10-3.2873376623376621e-03*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1)*D;
  A[38][18]=(0.0)*D;
  A[38][19]=(0.0)*D;
  A[38][20]=(-3.5612824675324673e-04*Jinv00*w12+3.9265422077922079e-04*w28*Jinv10+1.2327516233766234e-03*Jinv20*w8-3.5612824675324673e-04*w13*Jinv10+4.2735389610389614e-03*Jinv21*w44-1.0135957792207792e-03*w3*Jinv00-3.5612824675324673e-04*w32*Jinv20-2.3833198051948050e-03*Jinv01*w3-8.2183441558441553e-05*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+7.8530844155844158e-04*Jinv21*w59+7.4878246753246752e-04*Jinv21*w2+3.1777597402597403e-03*w43*Jinv10+4.2735389610389614e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21+2.1367694805194807e-03*w16*Jinv11+2.1367694805194807e-03*Jinv21*w17-7.1225649350649346e-04*Jinv01*w45+3.9265422077922079e-04*w58*Jinv10-8.2183441558441553e-05*w48*Jinv00+7.8530844155844158e-04*w58*Jinv11+7.8530844155844158e-04*w57*Jinv01-2.3833198051948050e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01+3.7439123376623376e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+4.2735389610389614e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00+1.0957792207792207e-03*Jinv21*w41+3.9265422077922079e-04*Jinv00*w27+7.4878246753246752e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+2.1367694805194807e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50+3.9265422077922079e-04*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+1.5888798701298701e-03*Jinv00*w33-3.5612824675324673e-04*w31*Jinv10-8.2183441558441553e-05*w21*Jinv00+1.0957792207792207e-03*Jinv01*w39+2.4655032467532467e-03*Jinv00*w18+2.4655032467532467e-03*w36*Jinv00+7.3965097402597406e-03*Jinv21*w38-2.3833198051948050e-03*Jinv01*w24+4.2735389610389614e-03*w34*Jinv11+3.6982548701298703e-03*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.2327516233766234e-03*w51*Jinv00-1.0135957792207792e-03*w4*Jinv10-2.3833198051948050e-03*w4*Jinv11+1.0957792207792207e-03*w41*Jinv20+2.4655032467532467e-03*w19*Jinv10+1.5888798701298701e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+3.6982548701298703e-03*Jinv21*w20+1.2327516233766234e-03*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-1.2327516233766234e-03*w26*Jinv20-4.5657467532467534e-05*w11*Jinv20+4.2735389610389614e-03*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11-7.1225649350649346e-04*w46*Jinv11-3.5612824675324673e-04*Jinv20*w14-2.3833198051948050e-03*w5*Jinv21-8.2183441558441553e-05*w49*Jinv10+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33+1.0957792207792207e-03*w39*Jinv00+3.6982548701298703e-03*Jinv01*w18+7.3965097402597406e-03*w37*Jinv11+3.6982548701298703e-03*w6*Jinv01+1.2327516233766234e-03*w7*Jinv10-7.1225649350649346e-04*w47*Jinv21+2.4655032467532467e-03*Jinv20*w38-2.6298701298701297e-03*w52*Jinv11-1.2327516233766234e-03*Jinv20*w53-8.2183441558441553e-05*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+7.3965097402597406e-03*w36*Jinv01-1.2327516233766234e-03*w52*Jinv10+3.6982548701298703e-03*w7*Jinv11-4.5657467532467534e-05*w9*Jinv00+2.4655032467532467e-03*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56-1.2327516233766234e-03*w24*Jinv00-1.6436688311688311e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14-4.5657467532467534e-05*w10*Jinv10+3.1777597402597403e-03*Jinv20*w44-3.5612824675324673e-04*Jinv01*w12+3.6982548701298703e-03*Jinv21*w8+3.9265422077922079e-04*w57*Jinv00+3.9265422077922079e-04*Jinv20*w59+3.7439123376623376e-04*w0*Jinv00+1.5888798701298701e-03*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11-1.2327516233766234e-03*w25*Jinv10+3.7439123376623376e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00+1.0957792207792207e-03*Jinv01*w9+1.5888798701298701e-03*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00+1.0957792207792207e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+1.0957792207792207e-03*w40*Jinv11-1.0135957792207792e-03*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-2.3833198051948050e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10+3.1777597402597403e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.6436688311688311e-04*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01+1.0957792207792207e-03*w10*Jinv11)*D;
  A[38][21]=(0.0)*D;
  A[38][22]=(0.0)*D;
  A[38][23]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-8.2183441558441553e-05*w32*Jinv20+2.4655032467532466e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.2049512987012989e-04*Jinv00*w54+1.6436688311688311e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02-2.7394480519480521e-05*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-3.2873376623376621e-04*w16*Jinv11-3.2873376623376621e-04*Jinv21*w17+3.5612824675324673e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+1.3697240259740259e-04*w58*Jinv10+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-2.7394480519480518e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+8.2183441558441553e-05*Jinv21*w26-2.7394480519480521e-05*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+7.1225649350649346e-04*w45*Jinv00-8.2183441558441553e-04*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-2.7394480519480521e-05*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11-3.2873376623376621e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-2.7394480519480518e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+7.1225649350649346e-04*w46*Jinv10+3.8352272727272725e-04*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22-8.2183441558441553e-04*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv02*w45+8.2183441558441553e-05*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-5.2049512987012989e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10+2.4655032467532466e-04*w4*Jinv11+5.4788961038961042e-05*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+4.6570616883116886e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.2049512987012989e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-9.1314935064935065e-06*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+4.6570616883116886e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+3.5612824675324673e-04*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14+2.4655032467532466e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02-2.7394480519480518e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+5.4788961038961042e-05*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18-9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+3.5612824675324673e-04*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-8.2183441558441553e-05*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*Jinv12*w7-8.2183441558441553e-05*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49-5.2049512987012989e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-9.1314935064935065e-06*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.7394480519480518e-04*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+2.7394480519480518e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-9.1314935064935065e-06*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.7394480519480521e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.3697240259740259e-04*w57*Jinv00+1.3697240259740259e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-2.7394480519480518e-04*Jinv01*w54-2.7394480519480518e-04*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02+3.8352272727272725e-04*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00+5.4788961038961042e-05*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14-8.2183441558441553e-04*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20+4.6570616883116886e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+8.2183441558441553e-05*w25*Jinv11-5.2049512987012989e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+7.1225649350649346e-04*w47*Jinv20+3.8352272727272725e-04*w16*Jinv12+2.7394480519480518e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1)*D;
  A[38][24]=(0.0)*D;
  A[38][25]=(0.0)*D;
  A[38][26]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+2.7394480519480518e-04*w13*Jinv10-1.6436688311688311e-03*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23+5.2049512987012989e-04*Jinv00*w54-3.1047077922077924e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-1.6436688311688311e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-8.2183441558441553e-04*w16*Jinv11-8.2183441558441553e-04*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-1.3697240259740259e-04*w58*Jinv10+2.7394480519480518e-04*w48*Jinv00-3.1047077922077924e-04*w58*Jinv11-3.1047077922077924e-04*w57*Jinv01+5.7528409090909093e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-1.6436688311688311e-03*w43*Jinv11-7.1225649350649346e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-8.2183441558441553e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-7.1225649350649346e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+8.2183441558441553e-05*w31*Jinv10-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39-1.6436688311688311e-04*Jinv00*w18-7.1225649350649346e-04*w36*Jinv00-4.7666396103896100e-03*Jinv21*w38+5.7528409090909093e-04*Jinv01*w24-2.6298701298701297e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+1.4793019480519481e-03*Jinv21*w53+5.2049512987012989e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-1.6436688311688311e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+5.2049512987012989e-04*Jinv20*w56-1.6436688311688311e-04*w26*Jinv20+9.1314935064935065e-06*w11*Jinv20-2.6298701298701297e-03*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+2.7394480519480518e-04*w49*Jinv10-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.4655032467532466e-04*Jinv01*w18-4.7666396103896100e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21-7.1225649350649346e-04*Jinv20*w38+1.4793019480519481e-03*w52*Jinv11+5.2049512987012989e-04*Jinv20*w53-3.5612824675324673e-04*w22*Jinv10-8.2183441558441553e-05*w22*Jinv11-8.2183441558441553e-05*w21*Jinv01+1.4793019480519481e-03*Jinv01*w51-4.7666396103896100e-03*w36*Jinv01+5.2049512987012989e-04*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11+9.1314935064935065e-06*w9*Jinv00-7.1225649350649346e-04*w37*Jinv10+1.3697240259740259e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+5.2049512987012989e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14+9.1314935064935065e-06*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+2.4655032467532466e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-1.3697240259740259e-04*w57*Jinv00-1.3697240259740259e-04*Jinv20*w59-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+1.3697240259740259e-04*Jinv01*w54+1.3697240259740259e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-3.8352272727272725e-04*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+5.7528409090909093e-04*w25*Jinv11+5.2049512987012989e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-7.1225649350649346e-04*w47*Jinv20+5.2049512987012989e-04*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27+5.2049512987012989e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[38][27]=(0.0)*D;
  A[38][28]=(0.0)*D;
  A[38][29]=(4.3831168831168834e-04*Jinv21*w44-4.2918019480519480e-04*Jinv01*w3-1.4204545454545455e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+2.1915584415584417e-04*w16*Jinv11+2.1915584415584417e-04*Jinv21*w17-1.0957792207792208e-04*Jinv01*w45-1.4204545454545455e-05*w58*Jinv11-1.4204545454545455e-05*w57*Jinv01-2.6481331168831169e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+4.3831168831168834e-04*w43*Jinv11+3.4699675324675324e-04*Jinv21*w41+1.0247564935064935e-04*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11+2.1915584415584417e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+3.4699675324675324e-04*Jinv01*w39+1.4975649350649350e-03*Jinv21*w38-2.6481331168831169e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11-5.8441558441558442e-04*Jinv21*w53-4.2918019480519480e-04*w4*Jinv11-2.1915584415584417e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-1.0957792207792208e-04*w46*Jinv11-4.2918019480519480e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+3.6525974025974026e-05*Jinv01*w18+1.4975649350649350e-03*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01-1.0957792207792208e-04*w47*Jinv21-5.8441558441558442e-04*w52*Jinv11-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-5.8441558441558442e-04*Jinv01*w51+1.4975649350649350e-03*w36*Jinv01+7.4878246753246752e-04*w7*Jinv11+1.1870941558441559e-04*Jinv21*w56-4.2918019480519480e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-2.7394480519480521e-05*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8-5.4788961038961042e-05*Jinv21*w23+1.1870941558441559e-04*Jinv01*w54+1.1870941558441559e-04*w55*Jinv11+6.4935064935064935e-05*Jinv01*w9+3.4699675324675324e-04*w40*Jinv11-2.1915584415584417e-04*Jinv01*w30-2.6481331168831169e-04*w25*Jinv11-4.2918019480519480e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27-4.2918019480519480e-04*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[38][30]=(0.0)*D;
  A[38][31]=(0.0)*D;
  A[38][32]=(3.5612824675324673e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.2601461038961039e-03*w40*Jinv12-9.5880681818181818e-04*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-3.5612824675324673e-04*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+1.2601461038961039e-03*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+1.0683847402597404e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-4.6570616883116886e-04*Jinv20*w23+1.4245129870129869e-03*Jinv22*w38-1.3971185064935066e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-1.8262987012987013e-05*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-1.1231737012987013e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-2.4655032467532466e-04*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+2.1915584415584417e-04*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.4655032467532466e-04*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-1.9176136363636362e-04*w22*Jinv12+1.0683847402597404e-03*w31*Jinv10-4.6570616883116886e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.5612824675324673e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-1.4793019480519481e-03*Jinv21*w53-1.4793019480519481e-03*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20+7.1225649350649346e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10+1.0683847402597404e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00+1.4245129870129869e-03*w37*Jinv12-1.3971185064935066e-03*Jinv20*w56+1.2601461038961039e-03*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-1.8262987012987013e-05*w11*Jinv20-9.5880681818181818e-04*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.0683847402597404e-03*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-1.1231737012987013e-03*Jinv02*w54-1.9176136363636362e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10+2.1915584415584417e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00+7.1225649350649346e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-1.9176136363636362e-04*Jinv22*w23-1.4793019480519481e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12-1.4793019480519481e-03*Jinv20*w53-5.4788961038961036e-04*Jinv22*w8-4.6570616883116886e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50-4.6570616883116886e-04*w22*Jinv11-5.4788961038961036e-04*Jinv12*w7-4.6570616883116886e-04*w21*Jinv01-1.4793019480519481e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-1.4793019480519481e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-1.8262987012987013e-05*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-1.3971185064935066e-03*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27-5.4788961038961036e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-1.6436688311688311e-04*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-1.8262987012987013e-05*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-3.5612824675324673e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-4.6570616883116886e-04*Jinv21*w23-1.3971185064935066e-03*Jinv01*w54-1.3971185064935066e-03*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02-1.8262987012987013e-05*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+1.0683847402597404e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29+1.3149350649350649e-03*w40*Jinv11+1.4245129870129869e-03*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+1.0683847402597404e-03*Jinv01*w30-9.5880681818181818e-04*Jinv02*w51-1.1231737012987013e-03*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11-1.3971185064935066e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-2.4655032467532466e-04*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50+2.1915584415584417e-04*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1-1.8262987012987013e-05*w10*Jinv11)*D;
  A[38][33]=(0.0)*D;
  A[38][34]=(0.0)*D;
  A[38][35]=(-9.6793831168831166e-04*w28*Jinv10+6.5746753246753243e-04*w40*Jinv12+2.7942370129870131e-03*Jinv22*w53-2.0819805194805196e-03*Jinv20*w8-1.6436688311688311e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-2.9586038961038961e-03*Jinv21*w44+1.8628246753246754e-03*w3*Jinv00+2.6298701298701297e-03*Jinv01*w3+8.2183441558441553e-04*Jinv20*w23-8.5470779220779228e-03*Jinv22*w38+8.2183441558441553e-04*Jinv00*w54-6.5746753246753243e-04*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-5.4788961038961036e-04*Jinv21*w2-4.9310064935064934e-03*w43*Jinv10-2.9586038961038961e-03*w42*Jinv01+1.0957792207792207e-03*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+1.7532467532467534e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10+5.4788961038961036e-04*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00+1.3149350649350649e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-6.5746753246753243e-04*w58*Jinv11-6.5746753246753243e-04*w57*Jinv01+2.6298701298701297e-03*Jinv21*w26-5.4788961038961036e-04*w0*Jinv01-6.2094155844155847e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-2.9586038961038961e-03*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00+1.9724025974025973e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-9.6793831168831166e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-5.4788961038961036e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-5.4788961038961036e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-9.6793831168831166e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-6.9034090909090911e-03*Jinv00*w33+4.9310064935064932e-04*Jinv02*w3+8.2183441558441553e-04*w21*Jinv00-6.0267857142857139e-04*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2-3.1777597402597403e-03*Jinv00*w18-9.5332792207792200e-03*w36*Jinv00-1.2820616883116883e-02*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+2.6298701298701297e-03*Jinv01*w24-1.6436688311688311e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+4.9310064935064934e-03*Jinv21*w53+2.7942370129870131e-03*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+1.8628246753246754e-03*w4*Jinv10+2.6298701298701297e-03*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13-3.1777597402597403e-03*w19*Jinv10-6.9034090909090911e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-2.0819805194805196e-03*w6*Jinv00-8.5470779220779228e-03*w37*Jinv12+8.2183441558441553e-04*Jinv20*w56+6.5746753246753243e-04*Jinv22*w41+2.5750811688311690e-03*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+2.7942370129870131e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11+2.6298701298701297e-03*w5*Jinv21+1.7532467532467534e-03*Jinv02*w54+4.9310064935064932e-04*w4*Jinv12-4.9310064935064932e-04*w49*Jinv10-5.4788961038961036e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33-2.9586038961038961e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-1.2820616883116883e-02*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-4.2735389610389614e-03*w6*Jinv01-2.0819805194805196e-03*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21-9.5332792207792200e-03*Jinv20*w38+4.9310064935064934e-03*w52*Jinv11-6.0267857142857139e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.7942370129870131e-03*Jinv20*w53+8.2183441558441553e-04*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+5.4788961038961042e-05*w21*Jinv01+4.9310064935064934e-03*Jinv01*w51-6.0267857142857139e-04*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49-1.2820616883116883e-02*w36*Jinv01+2.7942370129870131e-03*w52*Jinv10-4.2735389610389614e-03*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-9.5332792207792200e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56+2.5750811688311690e-03*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27-2.1367694805194807e-03*w19*Jinv12+4.9310064935064932e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10-4.9310064935064934e-03*Jinv20*w44-1.6436688311688311e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-4.2735389610389614e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33-6.2094155844155847e-04*w0*Jinv00-6.9034090909090911e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11+2.5750811688311690e-03*w25*Jinv10-6.2094155844155847e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02+1.0957792207792207e-03*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-2.9586038961038961e-03*w40*Jinv10-3.1777597402597403e-03*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-8.5470779220779228e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+1.8628246753246754e-03*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.7942370129870131e-03*Jinv02*w51+1.7532467532467534e-03*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28+2.6298701298701297e-03*w25*Jinv11+8.2183441558441553e-04*w55*Jinv10-4.9310064935064934e-03*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20+2.1367694805194807e-03*Jinv21*w50-5.4788961038961036e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1+1.0957792207792207e-03*w10*Jinv11)*D;
  A[38][36]=(0.0)*D;
  A[38][37]=(0.0)*D;
  A[38][38]=(-3.5612824675324673e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.0957792207792207e-03*w40*Jinv12-2.6298701298701297e-03*Jinv22*w53+2.4655032467532467e-03*Jinv20*w8+7.8530844155844158e-04*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.0957792207792207e-03*Jinv02*w39-2.0271915584415584e-03*w3*Jinv00-1.0683847402597404e-03*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23+7.3965097402597406e-03*Jinv22*w38+5.7528409090909093e-04*Jinv00*w54-2.3833198051948050e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+4.7666396103896100e-03*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55+6.5746753246753243e-04*w58*Jinv10+2.1367694805194807e-03*w15*Jinv02+1.0957792207792207e-03*w10*Jinv12+5.7528409090909093e-04*w48*Jinv00-2.3833198051948050e-03*Jinv02*w24+4.2735389610389614e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.5888798701298701e-03*w16*Jinv10+3.6982548701298703e-03*Jinv22*w20-1.0683847402597404e-03*w45*Jinv00+4.2735389610389614e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+1.0957792207792207e-03*w11*Jinv22+5.7528409090909093e-04*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-1.0683847402597404e-03*w46*Jinv10+2.1367694805194807e-03*Jinv22*w17+4.7666396103896100e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-1.0683847402597404e-03*w31*Jinv10-2.3833198051948050e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2+2.4655032467532467e-03*Jinv00*w18+7.3965097402597406e-03*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45+7.8530844155844158e-04*Jinv22*w59-1.3149350649350649e-03*w51*Jinv00+4.2735389610389614e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-2.0271915584415584e-03*w4*Jinv10+1.6436688311688311e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13+2.4655032467532467e-03*w19*Jinv10+4.7666396103896100e-03*w34*Jinv10+2.4655032467532467e-03*w6*Jinv00+7.3965097402597406e-03*w37*Jinv12+5.7528409090909093e-04*Jinv20*w56+1.0957792207792207e-03*Jinv22*w41-2.0271915584415584e-03*w26*Jinv20-9.1314935064935068e-05*w11*Jinv20-2.6298701298701297e-03*w52*Jinv12-2.3833198051948050e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54-2.3833198051948050e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+5.7528409090909093e-04*w49*Jinv10+1.6436688311688311e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+1.0957792207792207e-03*Jinv02*w9+2.4655032467532467e-03*w7*Jinv10+7.3965097402597406e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-7.1225649350649346e-04*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12-1.3149350649350649e-03*Jinv20*w53+3.6982548701298703e-03*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50+3.6982548701298703e-03*Jinv12*w7-7.1225649350649346e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.3149350649350649e-03*w52*Jinv10-9.1314935064935068e-05*w9*Jinv00+7.3965097402597406e-03*w37*Jinv10-2.0271915584415584e-03*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+3.6982548701298703e-03*w6*Jinv02+3.6982548701298703e-03*w19*Jinv12-2.3833198051948050e-03*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22-9.1314935064935068e-05*w10*Jinv10+4.7666396103896100e-03*Jinv20*w44+7.8530844155844158e-04*w58*Jinv12+3.6982548701298703e-03*Jinv02*w18+6.5746753246753243e-04*w57*Jinv00+6.5746753246753243e-04*Jinv20*w59+4.2735389610389614e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+4.7666396103896100e-03*w35*Jinv20-2.0271915584415584e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.5888798701298701e-03*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02+1.5888798701298701e-03*Jinv20*w17-1.0683847402597404e-03*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10+2.4655032467532467e-03*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14+7.3965097402597406e-03*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-2.0271915584415584e-03*w5*Jinv20-2.6298701298701297e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28+5.7528409090909093e-04*w55*Jinv10+4.7666396103896100e-03*w42*Jinv00-1.0683847402597404e-03*w47*Jinv20+2.1367694805194807e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[38][39]=(0.0)*D;
  A[38][40]=(0.0)*D;
  A[38][41]=(-7.6704545454545450e-04*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10+1.6436688311688311e-03*w40*Jinv12+1.7532467532467534e-03*Jinv22*w53-1.0957792207792207e-03*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10+1.6436688311688311e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00-1.1505681818181819e-03*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+5.4788961038961042e-05*Jinv20*w23-2.1915584415584414e-03*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+3.8352272727272725e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+3.4699675324675324e-04*Jinv21*w2+3.9448051948051946e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+1.3149350649350649e-03*Jinv01*w45+2.5750811688311690e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+4.9310064935064932e-04*w10*Jinv12+1.3149350649350649e-03*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35+7.1225649350649346e-04*Jinv21*w26+3.4699675324675324e-04*w0*Jinv01+4.2004870129870131e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00+2.6298701298701297e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22+3.4699675324675324e-04*Jinv11*w1-8.2183441558441553e-04*w13*Jinv11-3.4699675324675324e-04*w28*Jinv11+1.3149350649350649e-03*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+1.9724025974025973e-03*Jinv00*w33-1.1505681818181819e-03*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3+5.4788961038961042e-05*w21*Jinv00-1.2053571428571428e-03*w32*Jinv22+3.8352272727272725e-04*Jinv22*w2-2.1915584415584414e-03*Jinv00*w18-3.2873376623376621e-03*w36*Jinv00+1.0957792207792208e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+2.1367694805194807e-03*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11+4.9310064935064934e-03*w41*Jinv20-8.2183441558441553e-04*Jinv12*w13-2.1915584415584414e-03*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20-1.0957792207792207e-03*w6*Jinv00-2.1915584415584414e-03*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56+1.6436688311688311e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20+9.8620129870129864e-04*w11*Jinv20+1.7532467532467534e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+3.8352272727272725e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21+2.5750811688311690e-03*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+1.3149350649350649e-03*w49*Jinv10-3.4699675324675324e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+4.9310064935064934e-03*w39*Jinv00-8.2183441558441553e-04*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18+4.9310064935064932e-04*Jinv02*w9+1.0957792207792207e-03*w6*Jinv01-1.0957792207792207e-03*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-3.2873376623376621e-03*Jinv20*w38-1.2053571428571428e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53+5.4788961038961042e-05*w22*Jinv10+1.2601461038961039e-03*Jinv22*w50+8.2183441558441553e-04*w22*Jinv11+8.2183441558441553e-04*w21*Jinv01-1.2053571428571428e-03*Jinv02*w30+1.2601461038961039e-03*Jinv12*w49+2.1367694805194807e-03*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11+9.8620129870129864e-04*w9*Jinv00-3.2873376623376621e-03*w37*Jinv10+1.3149350649350649e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-1.0957792207792207e-03*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-8.2183441558441553e-04*Jinv21*w14+9.8620129870129864e-04*w10*Jinv10+3.9448051948051946e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18-8.2183441558441553e-04*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33+4.2004870129870131e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20+8.2183441558441553e-04*Jinv21*w23+1.3149350649350649e-03*Jinv01*w54+1.3149350649350649e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10+4.2004870129870131e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.2601461038961039e-03*w48*Jinv02+1.3149350649350649e-03*Jinv20*w17-1.1505681818181819e-03*w30*Jinv00+4.9310064935064934e-03*w40*Jinv10-2.1915584415584414e-03*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-8.2183441558441553e-04*Jinv22*w14-2.1915584415584414e-03*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30+1.7532467532467534e-03*Jinv02*w51+2.5750811688311690e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+3.9448051948051946e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*Jinv21*w50-3.4699675324675324e-04*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1)*D;
  A[38][42]=(0.0)*D;
  A[38][43]=(0.0)*D;
  A[38][44]=(7.6704545454545450e-04*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00+1.1505681818181819e-03*w32*Jinv20-2.6298701298701297e-03*Jinv01*w3-5.4788961038961042e-05*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+6.5746753246753243e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+5.4788961038961036e-04*Jinv21*w2-3.9448051948051946e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01-1.0957792207792207e-03*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45-7.6704545454545450e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+6.5746753246753243e-04*w58*Jinv11+6.5746753246753243e-04*w57*Jinv01-2.6298701298701297e-03*Jinv21*w26+5.4788961038961036e-04*w0*Jinv01-4.2004870129870131e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22+5.4788961038961036e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+5.4788961038961036e-04*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12+1.1505681818181819e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-5.4788961038961042e-05*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2+2.1915584415584414e-03*Jinv00*w18+3.2873376623376621e-03*w36*Jinv00+1.2820616883116883e-02*Jinv21*w38-7.6704545454545450e-04*Jinv02*w45-2.6298701298701297e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+2.9586038961038961e-03*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-4.9310064935064934e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-2.6298701298701297e-03*w4*Jinv11-4.9310064935064934e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+2.1915584415584414e-03*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10-1.6436688311688311e-04*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20-9.8620129870129864e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12+2.9586038961038961e-03*Jinv21*w35-1.6436688311688311e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-1.6436688311688311e-04*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14-2.6298701298701297e-03*w5*Jinv21-7.6704545454545450e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-1.3149350649350649e-03*w49*Jinv10+5.4788961038961036e-04*Jinv21*w29+2.9586038961038961e-03*Jinv01*w33-4.9310064935064934e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12+4.2735389610389614e-03*Jinv01*w18+1.2820616883116883e-02*w37*Jinv11-5.4788961038961036e-04*Jinv02*w9+4.2735389610389614e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+3.2873376623376621e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-4.9310064935064934e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-5.4788961038961042e-05*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11+2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01-4.9310064935064934e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49+1.2820616883116883e-02*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+4.2735389610389614e-03*w7*Jinv11-9.8620129870129864e-04*w9*Jinv00+3.2873376623376621e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56-7.6704545454545450e-04*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-9.8620129870129864e-04*w10*Jinv10-3.9448051948051946e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12+4.2735389610389614e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33-4.2004870129870131e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11-7.6704545454545450e-04*w25*Jinv10-4.2004870129870131e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-1.0957792207792207e-03*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+1.1505681818181819e-03*w30*Jinv00-4.9310064935064934e-03*w40*Jinv10+2.1915584415584414e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11+4.2735389610389614e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-1.6436688311688311e-04*Jinv01*w30-2.1367694805194807e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-2.6298701298701297e-03*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10-3.9448051948051946e-03*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50+5.4788961038961036e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1-1.0957792207792207e-03*w10*Jinv11)*D;
  A[38][45]=(0.0)*D;
  A[38][46]=(0.0)*D;
  A[38][47]=(-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-1.3697240259740259e-04*w57*Jinv02-5.4788961038961042e-05*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38+2.1915584415584417e-04*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-2.1915584415584417e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01+1.8262987012987013e-05*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17-1.0683847402597404e-03*Jinv01*w45+5.2049512987012989e-04*Jinv12*w55-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-2.1915584415584417e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02+9.1314935064935065e-06*w11*Jinv22-2.1915584415584417e-04*Jinv11*w1+4.6570616883116886e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.8352272727272725e-04*Jinv22*w17-3.5612824675324673e-04*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+8.2183441558441553e-05*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-7.1225649350649346e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-1.3697240259740259e-04*Jinv22*w59-1.6436688311688311e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11+1.4793019480519481e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv11+2.7394480519480518e-04*Jinv12*w13+2.4655032467532466e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12-5.4788961038961042e-05*Jinv22*w41+5.2049512987012989e-04*w52*Jinv12-1.6436688311688311e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-1.0683847402597404e-03*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21+5.2049512987012989e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33+2.7394480519480518e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+9.1314935064935065e-06*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-1.0683847402597404e-03*w47*Jinv21-3.5612824675324673e-04*Jinv22*w23+1.4793019480519481e-03*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.7394480519480518e-04*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+1.4793019480519481e-03*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+8.2183441558441553e-05*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.3971185064935066e-03*w49*Jinv11+4.6570616883116886e-04*Jinv21*w14-1.3697240259740259e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+4.6570616883116886e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+8.2183441558441553e-05*Jinv01*w54+8.2183441558441553e-05*w55*Jinv11+2.7394480519480518e-04*w48*Jinv02+1.8262987012987013e-05*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+2.4655032467532466e-04*Jinv01*w30+5.2049512987012989e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-3.8352272727272725e-04*w16*Jinv12+1.3971185064935066e-03*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27+1.3971185064935066e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1+1.8262987012987013e-05*w10*Jinv11)*D;
  A[38][48]=(0.0)*D;
  A[38][49]=(0.0)*D;
  A[38][50]=(-2.7394480519480518e-04*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10-1.2053571428571428e-03*w40*Jinv12+1.3971185064935066e-03*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8+6.6659902597402602e-04*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10-1.2053571428571428e-03*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-5.2049512987012989e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3-2.7394480519480518e-04*Jinv20*w23+3.2873376623376621e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+4.7483766233766234e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+1.3697240259740260e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+2.3833198051948050e-03*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24-6.0267857142857139e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.3833198051948050e-03*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33+1.9176136363636362e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10-3.8352272727272725e-04*Jinv02*w3-2.7394480519480518e-04*w21*Jinv00-9.5880681818181818e-04*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18-1.1505681818181819e-03*w36*Jinv00-1.1505681818181819e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+6.6659902597402602e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-6.0267857142857139e-04*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-5.2049512987012989e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11+1.3149350649350649e-03*w41*Jinv20-2.7394480519480521e-05*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+3.2873376623376621e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56-1.2053571428571428e-03*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20+4.7483766233766234e-04*w11*Jinv20+1.3971185064935066e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+1.3697240259740260e-03*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12+1.9176136363636362e-04*w21*Jinv02+2.3833198051948050e-03*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+1.3149350649350649e-03*w39*Jinv00-2.7394480519480521e-05*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18-1.1505681818181819e-03*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.1505681818181819e-03*Jinv20*w38+1.9176136363636362e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11-9.5880681818181818e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53+5.4788961038961036e-04*Jinv22*w8-2.7394480519480518e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+5.4788961038961036e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-9.5880681818181818e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-1.1505681818181819e-03*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11+4.7483766233766234e-04*w9*Jinv00-1.1505681818181819e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+5.4788961038961036e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22+2.3833198051948050e-03*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+4.7483766233766234e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+6.6659902597402602e-04*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-6.0267857142857139e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+4.7483766233766234e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00+1.3149350649350649e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20+1.0957792207792208e-04*Jinv22*w29-2.7394480519480521e-05*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11+3.2873376623376621e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-5.2049512987012989e-04*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30+1.3971185064935066e-03*Jinv02*w51+1.3697240259740260e-03*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+2.3833198051948050e-03*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.3833198051948050e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+4.7483766233766234e-04*w10*Jinv11)*D;
  A[38][51]=(0.0)*D;
  A[38][52]=(0.0)*D;
  A[38][53]=(2.7394480519480518e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-1.4610389610389610e-04*w57*Jinv02+2.7394480519480518e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+5.2049512987012989e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20+2.7394480519480518e-04*Jinv20*w23+3.7804383116883116e-03*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+8.2183441558441553e-05*w25*Jinv12-1.5523538961038962e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10-1.2327516233766234e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-8.2183441558441553e-04*w15*Jinv02-1.0957792207792207e-03*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-6.5746753246753243e-04*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27-1.0957792207792207e-03*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-8.2183441558441553e-04*Jinv22*w17-2.1367694805194807e-03*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+4.3831168831168834e-04*w32*Jinv22-1.5523538961038962e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18+1.1505681818181819e-03*w36*Jinv00+4.3831168831168834e-04*Jinv02*w45-1.4610389610389610e-04*Jinv22*w59-2.6298701298701297e-03*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv10-1.3149350649350649e-03*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00+3.7804383116883116e-03*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41+5.2049512987012989e-04*w26*Jinv20-4.7483766233766234e-04*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+8.2183441558441553e-05*Jinv22*w26+2.7394480519480518e-04*Jinv20*w14-1.2327516233766234e-03*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-1.3149350649350649e-03*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-1.0957792207792207e-03*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10+1.1505681818181819e-03*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23+4.3831168831168834e-04*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12-2.6298701298701297e-03*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50-2.4655032467532466e-04*Jinv12*w7+4.3831168831168834e-04*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49-2.6298701298701297e-03*w52*Jinv10-4.7483766233766234e-04*w9*Jinv00+1.1505681818181819e-03*w37*Jinv10+5.2049512987012989e-04*w24*Jinv00-1.5523538961038962e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-4.7483766233766234e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.4610389610389610e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+5.2049512987012989e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-1.3149350649350649e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-1.5523538961038962e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14+3.7804383116883116e-03*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+5.2049512987012989e-04*w5*Jinv20-1.3149350649350649e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56-1.5523538961038962e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-8.2183441558441553e-04*w16*Jinv12-1.5523538961038962e-04*Jinv12*w1)*D;
  A[38][54]=(0.0)*D;
  A[38][55]=(0.0)*D;
  A[38][56]=(-2.5202922077922079e-03*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53+1.1870941558441559e-04*w57*Jinv02-2.5202922077922079e-03*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+1.4793019480519481e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59+1.3697240259740259e-04*w25*Jinv12-3.1960227272727273e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01-4.7483766233766234e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-2.0271915584415584e-03*Jinv12*w55-7.1225649350649346e-04*w15*Jinv02-4.8396915584415583e-04*w10*Jinv12+1.3697240259740259e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+3.8352272727272725e-04*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-1.3149350649350649e-03*Jinv21*w41-2.7394480519480520e-03*w42*Jinv02-4.8396915584415583e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11-7.1225649350649346e-04*Jinv22*w17+2.4655032467532466e-04*w22*Jinv12+2.4655032467532466e-04*Jinv02*w3+8.4922889610389610e-04*w32*Jinv22-1.3149350649350649e-03*Jinv01*w39-3.1960227272727273e-04*Jinv22*w2+1.1505681818181819e-03*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45+5.2049512987012989e-04*Jinv01*w24+1.1870941558441559e-04*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-2.7394480519480520e-03*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+4.6570616883116886e-04*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20+1.4793019480519481e-03*w37*Jinv12-2.5202922077922079e-03*Jinv22*w41-1.2327516233766234e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11+1.3697240259740259e-04*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-2.0271915584415584e-03*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33+4.6570616883116886e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18+1.1505681818181819e-03*w37*Jinv11-4.8396915584415583e-04*Jinv02*w9-1.6436688311688311e-04*w6*Jinv01+1.3971185064935066e-03*w47*Jinv21+2.4655032467532466e-04*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11+8.4922889610389610e-04*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+1.9176136363636362e-04*Jinv22*w8-1.0135957792207792e-03*Jinv22*w50+2.7394480519480518e-04*w22*Jinv11+1.9176136363636362e-04*Jinv12*w7+2.7394480519480518e-04*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51+8.4922889610389610e-04*Jinv02*w30-1.0135957792207792e-03*Jinv12*w49+1.1505681818181819e-03*w36*Jinv01-1.6436688311688311e-04*w7*Jinv11-2.3833198051948050e-03*Jinv21*w56-1.7349837662337662e-04*Jinv02*w27+1.9176136363636362e-04*w6*Jinv02+3.8352272727272725e-04*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33+2.7394480519480518e-04*Jinv21*w23-2.3833198051948050e-03*Jinv01*w54-2.3833198051948050e-03*w55*Jinv11-1.0135957792207792e-03*w48*Jinv02-4.7483766233766234e-04*Jinv01*w9-1.7349837662337662e-04*Jinv22*w29+4.6570616883116886e-04*Jinv22*w14-1.3149350649350649e-03*w40*Jinv11+1.4793019480519481e-03*Jinv02*w36-2.7394480519480520e-03*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-2.0271915584415584e-03*Jinv22*w56-1.7349837662337662e-04*Jinv12*w28+5.2049512987012989e-04*w25*Jinv11-7.1225649350649346e-04*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-3.1960227272727273e-04*Jinv12*w1-4.7483766233766234e-04*w10*Jinv11)*D;
  A[38][57]=(0.0)*D;
  A[38][58]=(0.0)*D;
  A[38][59]=(2.9220779220779221e-04*w40*Jinv12-1.0227272727272728e-03*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02+2.9220779220779221e-04*Jinv02*w39-2.5568181818181820e-04*Jinv22*w38+1.1870941558441559e-04*w25*Jinv12-1.1668019480519480e-04*w0*Jinv02-6.7573051948051951e-04*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02+8.3198051948051951e-05*w10*Jinv12+1.1870941558441559e-04*Jinv02*w24-3.2873376623376621e-04*Jinv22*w35+3.6525974025974026e-05*Jinv22*w20-3.2873376623376621e-04*w42*Jinv02+8.3198051948051951e-05*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17-2.7394480519480521e-05*w22*Jinv12+1.1870941558441559e-04*Jinv02*w3+4.9310064935064932e-04*w32*Jinv22-1.1668019480519480e-04*Jinv22*w2+4.9310064935064932e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59-3.2873376623376621e-04*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44-2.7394480519480521e-05*Jinv12*w13-2.5568181818181820e-04*w37*Jinv12+2.9220779220779221e-04*Jinv22*w41-1.0227272727272728e-03*w52*Jinv12+1.1870941558441559e-04*Jinv22*w26-6.7573051948051951e-04*Jinv02*w54+1.1870941558441559e-04*w4*Jinv12-2.7394480519480521e-05*w21*Jinv02-2.7394480519480521e-05*Jinv02*w12+8.3198051948051951e-05*Jinv02*w9-2.7394480519480521e-05*Jinv22*w23+4.9310064935064932e-04*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12+3.6525974025974026e-05*Jinv22*w8-6.7573051948051951e-04*Jinv22*w50+3.6525974025974026e-05*Jinv12*w7+4.9310064935064932e-04*Jinv02*w30-6.7573051948051951e-04*Jinv12*w49-1.1668019480519480e-04*Jinv02*w27+3.6525974025974026e-05*w6*Jinv02+3.6525974025974026e-05*w19*Jinv12+1.1870941558441559e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12+3.6525974025974026e-05*Jinv02*w18-3.2873376623376621e-04*Jinv02*w33-6.7573051948051951e-04*w48*Jinv02-1.1668019480519480e-04*Jinv22*w29-2.7394480519480521e-05*Jinv22*w14-2.5568181818181820e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-1.0227272727272728e-03*Jinv02*w51-6.7573051948051951e-04*Jinv22*w56-1.1668019480519480e-04*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12-1.1668019480519480e-04*Jinv12*w1)*D;
  A[39][0]=(-6.5746753246753243e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10+3.6525974025974025e-03*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53-2.9220779220779221e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+3.6525974025974025e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-3.4699675324675324e-04*Jinv22*w38-4.3831168831168834e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-2.1915584415584417e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+1.9480519480519481e-04*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17-4.3831168831168834e-04*Jinv01*w45-4.3831168831168834e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+1.8262987012987013e-03*w15*Jinv02+1.9480519480519481e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+1.8262987012987013e-03*w16*Jinv10-3.4699675324675324e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-4.3831168831168834e-04*w45*Jinv00+3.6525974025974025e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27+1.9480519480519481e-04*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-6.5746753246753243e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01+4.2613636363636367e-05*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-4.3831168831168834e-04*w46*Jinv10+1.8262987012987013e-03*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-6.5746753246753243e-04*w32*Jinv22+3.6525974025974025e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-3.4699675324675324e-04*Jinv00*w18-3.4699675324675324e-04*w36*Jinv00-3.4699675324675324e-04*Jinv21*w38-4.3831168831168834e-04*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+1.8262987012987013e-03*w34*Jinv11-3.4699675324675324e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-2.1915584415584417e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11+3.6525974025974025e-03*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-3.4699675324675324e-04*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21-3.4699675324675324e-04*Jinv21*w20-2.9220779220779221e-04*w6*Jinv00-3.4699675324675324e-04*w37*Jinv12-4.3831168831168834e-04*Jinv20*w56+3.6525974025974025e-03*Jinv22*w41-2.1915584415584417e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12+1.8262987012987013e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26-4.3831168831168834e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21-4.3831168831168834e-04*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+3.2873376623376621e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+1.8262987012987013e-03*Jinv01*w33+3.6525974025974025e-03*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-3.4699675324675324e-04*Jinv01*w18-3.4699675324675324e-04*w37*Jinv11+1.9480519480519481e-04*Jinv02*w9-2.9220779220779221e-04*w6*Jinv01-2.9220779220779221e-04*w7*Jinv10-4.3831168831168834e-04*w47*Jinv21-3.4699675324675324e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11-6.5746753246753243e-04*w31*Jinv12-4.3831168831168834e-04*w46*Jinv12-2.1915584415584417e-04*Jinv20*w53-2.9220779220779221e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11-2.9220779220779221e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-6.5746753246753243e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-3.4699675324675324e-04*w36*Jinv01-2.1915584415584417e-04*w52*Jinv10-2.9220779220779221e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-3.4699675324675324e-04*w37*Jinv10-4.3831168831168834e-04*Jinv21*w56-2.1915584415584417e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27-2.9220779220779221e-04*w6*Jinv02-3.4699675324675324e-04*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5-4.3831168831168834e-04*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-6.5746753246753243e-04*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12-3.4699675324675324e-04*Jinv02*w18-6.5746753246753243e-04*Jinv01*w12-2.9220779220779221e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+1.8262987012987013e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+1.8262987012987013e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-4.3831168831168834e-04*Jinv01*w54-4.3831168831168834e-04*w55*Jinv11-2.1915584415584417e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+1.8262987012987013e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+1.9480519480519481e-04*Jinv01*w9+1.8262987012987013e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+3.6525974025974025e-03*w40*Jinv10-3.4699675324675324e-04*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+3.6525974025974025e-03*w40*Jinv11-3.4699675324675324e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51-4.3831168831168834e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-2.1915584415584417e-04*w25*Jinv11-4.3831168831168834e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-4.3831168831168834e-04*w47*Jinv20+1.8262987012987013e-03*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+1.9480519480519481e-04*w10*Jinv11)*D;
  A[39][1]=(0.0)*D;
  A[39][2]=(0.0)*D;
  A[39][3]=(2.6846590909090908e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-5.2962662337662338e-04*w57*Jinv02+2.6846590909090908e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+5.4788961038961036e-04*w3*Jinv00+2.6846590909090908e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-4.6022727272727274e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01-1.6436688311688311e-04*w1*Jinv10-3.6160714285714286e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-5.2962662337662338e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10-4.6022727272727274e-03*Jinv22*w17-3.6160714285714286e-03*Jinv00*w33+1.3697240259740260e-03*w22*Jinv12+2.6846590909090908e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+6.5746753246753243e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-5.2962662337662338e-04*Jinv22*w59-4.6022727272727274e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.2873376623376621e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+6.5746753246753243e-04*w19*Jinv10-3.6160714285714286e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+3.2873376623376621e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-4.6022727272727274e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.6846590909090908e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+1.3697240259740260e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-5.2962662337662338e-04*Jinv21*w29-4.6022727272727274e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9+1.3697240259740260e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+6.5746753246753243e-04*Jinv20*w38+1.3697240259740260e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+3.2873376623376621e-04*Jinv20*w53+1.3697240259740260e-03*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11+1.3697240259740260e-03*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+3.2873376623376621e-04*w52*Jinv10+1.3697240259740260e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+3.2873376623376621e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+1.3697240259740260e-03*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+1.3697240259740260e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-5.2962662337662338e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+1.3697240259740260e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33-1.6436688311688311e-04*w0*Jinv00-3.6160714285714286e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+3.2873376623376621e-04*w25*Jinv10-1.6436688311688311e-04*Jinv20*w2-3.6160714285714286e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-3.6160714285714286e-03*Jinv20*w17+2.6846590909090908e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+5.4788961038961036e-04*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20-4.6022727272727274e-03*w16*Jinv12+1.3697240259740260e-03*Jinv21*w50-5.2962662337662338e-04*Jinv01*w27+1.3697240259740260e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[39][4]=(0.0)*D;
  A[39][5]=(0.0)*D;
  A[39][6]=(-2.5750811688311690e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+3.2873376623376621e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53+5.4788961038961036e-04*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02-2.5750811688311690e-03*w13*Jinv10+3.2873376623376621e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00-2.5750811688311690e-03*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+3.8352272727272725e-04*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00+3.2873376623376621e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-2.5750811688311690e-03*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+3.2873376623376621e-03*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2-1.0957792207792207e-03*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24+3.6525974025974026e-05*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-6.5746753246753243e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20+5.4788961038961036e-04*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56+3.2873376623376621e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20-1.8262987012987014e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+3.8352272727272725e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-2.5750811688311690e-03*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33-6.5746753246753243e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-1.0957792207792207e-03*w6*Jinv01+5.4788961038961036e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21+5.4788961038961042e-05*Jinv22*w23+3.8352272727272725e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53-1.0957792207792207e-03*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.0957792207792207e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10-1.0957792207792207e-03*w7*Jinv11-1.8262987012987014e-04*w9*Jinv00+5.4788961038961042e-05*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-1.0957792207792207e-03*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-1.8262987012987014e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-1.0957792207792207e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17-2.5750811688311690e-03*w30*Jinv00-6.5746753246753243e-03*w40*Jinv10+3.6525974025974026e-05*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14+3.2873376623376621e-03*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[39][7]=(0.0)*D;
  A[39][8]=(0.0)*D;
  A[39][9]=(5.4788961038961036e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10-3.1047077922077924e-04*Jinv20*w8+5.4788961038961036e-04*w13*Jinv10+2.1915584415584417e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23+5.4788961038961036e-04*Jinv00*w54-5.1136363636363640e-04*w43*Jinv10+3.0438311688311688e-05*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00+3.0438311688311688e-05*w1*Jinv10-5.1136363636363640e-04*w16*Jinv10+5.4788961038961036e-04*w45*Jinv00+3.0438311688311688e-05*Jinv00*w27+5.4788961038961036e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-5.1136363636363640e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10+5.4788961038961036e-04*w21*Jinv00-3.1047077922077924e-04*Jinv00*w18-3.1047077922077924e-04*w36*Jinv00+2.1915584415584417e-04*w51*Jinv00+2.1915584415584417e-04*w4*Jinv10+5.5519480519480519e-03*w41*Jinv20-3.1047077922077924e-04*w19*Jinv10-5.1136363636363640e-04*w34*Jinv10-3.1047077922077924e-04*w6*Jinv00+5.4788961038961036e-04*Jinv20*w56+2.1915584415584417e-04*w26*Jinv20+2.4350649350649351e-04*w11*Jinv20+5.4788961038961036e-04*Jinv20*w14+5.4788961038961036e-04*w49*Jinv10+5.5519480519480519e-03*w39*Jinv00-3.1047077922077924e-04*w7*Jinv10-3.1047077922077924e-04*Jinv20*w38+2.1915584415584417e-04*Jinv20*w53+5.4788961038961036e-04*w22*Jinv10+2.1915584415584417e-04*w52*Jinv10+2.4350649350649351e-04*w9*Jinv00-3.1047077922077924e-04*w37*Jinv10+2.1915584415584417e-04*w24*Jinv00+2.4350649350649351e-04*w10*Jinv10-5.1136363636363640e-04*Jinv20*w44+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+3.0438311688311688e-05*w0*Jinv00-5.1136363636363640e-04*w35*Jinv20+2.1915584415584417e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2-5.1136363636363640e-04*w15*Jinv00-5.1136363636363640e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00+5.5519480519480519e-03*w40*Jinv10-3.1047077922077924e-04*w20*Jinv20+2.1915584415584417e-04*w5*Jinv20+5.4788961038961036e-04*w55*Jinv10-5.1136363636363640e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20)*D;
  A[39][10]=(0.0)*D;
  A[39][11]=(0.0)*D;
  A[39][12]=(-4.7666396103896100e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-1.3806818181818182e-02*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+2.5202922077922079e-03*Jinv20*w8-1.0592532467532468e-03*w57*Jinv02-4.7666396103896100e-03*w13*Jinv10-1.3806818181818182e-02*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-3.1777597402597403e-03*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3+6.5746753246753243e-04*Jinv20*w23+8.7662337662337668e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-5.6615259740259744e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-6.9399350649350649e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17+8.7662337662337668e-04*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-1.0592532467532468e-03*w58*Jinv10-6.9034090909090911e-03*w15*Jinv02-9.6793831168831166e-04*w10*Jinv12+2.7394480519480520e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-5.6615259740259744e-04*w58*Jinv11-5.6615259740259744e-04*w57*Jinv01+4.3831168831168834e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-6.9034090909090911e-03*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+1.6436688311688311e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-9.6793831168831166e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11-5.9172077922077923e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+2.7394480519480520e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+1.6436688311688311e-03*w46*Jinv10-6.9034090909090911e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+6.5746753246753243e-04*w22*Jinv12-3.1777597402597403e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3+6.5746753246753243e-04*w21*Jinv00-3.1777597402597403e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+8.7662337662337668e-04*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38+1.6436688311688311e-03*Jinv02*w45+4.3831168831168834e-04*Jinv01*w24-1.0592532467532468e-03*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.2601461038961039e-03*w19*Jinv11+3.2873376623376621e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-1.3806818181818182e-02*w41*Jinv20-4.7666396103896100e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+1.0957792207792207e-03*w32*Jinv21+1.2601461038961039e-03*Jinv21*w20+2.5202922077922079e-03*w6*Jinv00+8.7662337662337668e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-1.3806818181818182e-02*Jinv22*w41+8.2183441558441553e-04*w26*Jinv20-9.6793831168831166e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35+1.0957792207792207e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+8.7662337662337668e-04*w46*Jinv11-4.7666396103896100e-03*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+6.5746753246753243e-04*w21*Jinv02+2.7394480519480520e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-1.3806818181818182e-02*w39*Jinv00-4.7666396103896100e-03*Jinv02*w12+1.2601461038961039e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-9.6793831168831166e-04*Jinv02*w9+1.2053571428571428e-03*w6*Jinv01+2.5202922077922079e-03*w7*Jinv10+8.7662337662337668e-04*w47*Jinv21+8.7662337662337668e-04*Jinv20*w38+6.5746753246753243e-04*Jinv22*w23+3.2873376623376621e-04*w52*Jinv11-3.1777597402597403e-03*w31*Jinv12+1.6436688311688311e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+2.5202922077922079e-03*Jinv22*w8+6.5746753246753243e-04*w22*Jinv10+2.7394480519480520e-03*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+2.5202922077922079e-03*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01+3.2873376623376621e-04*Jinv01*w51-3.1777597402597403e-03*Jinv02*w30+2.7394480519480520e-03*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10+1.2053571428571428e-03*w7*Jinv11-9.6793831168831166e-04*w9*Jinv00+8.7662337662337668e-04*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56+8.2183441558441553e-04*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+2.5202922077922079e-03*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+1.6436688311688311e-03*w47*Jinv22+6.0267857142857139e-04*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14-9.6793831168831166e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-1.0592532467532468e-03*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+1.2053571428571428e-03*Jinv21*w8-1.0592532467532468e-03*w57*Jinv00-1.0592532467532468e-03*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11+8.2183441558441553e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-6.9034090909090911e-03*w15*Jinv00+2.7394480519480520e-03*w48*Jinv02-6.9399350649350649e-04*Jinv01*w9-6.9034090909090911e-03*Jinv20*w17-3.1777597402597403e-03*w30*Jinv00-1.3806818181818182e-02*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-4.7666396103896100e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11+8.7662337662337668e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20+1.0957792207792207e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+4.3831168831168834e-04*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.6436688311688311e-03*w47*Jinv20-6.9034090909090911e-03*w16*Jinv12+6.0267857142857139e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+6.0267857142857139e-04*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-6.9399350649350649e-04*w10*Jinv11)*D;
  A[39][13]=(0.0)*D;
  A[39][14]=(0.0)*D;
  A[39][15]=(4.9310064935064934e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-3.6160714285714286e-03*Jinv20*w8-1.1323051948051949e-03*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44+2.3011363636363637e-03*w3*Jinv00+3.2873376623376621e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3+1.9724025974025973e-03*Jinv20*w23+1.3149350649350649e-03*Jinv22*w38-1.9724025974025973e-03*Jinv00*w54-9.8620129870129864e-04*w25*Jinv12-1.1323051948051949e-03*w0*Jinv02-9.1314935064935063e-04*Jinv21*w2-3.9448051948051946e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45+1.9724025974025973e-03*Jinv12*w55+1.1323051948051949e-03*w58*Jinv10-1.1834415584415585e-02*w15*Jinv02-1.1323051948051949e-03*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00-9.8620129870129864e-04*Jinv02*w24-7.8896103896103891e-03*Jinv22*w35-2.3011363636363637e-03*Jinv21*w26-9.1314935064935063e-04*w0*Jinv01-6.9399350649350649e-04*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10+1.9724025974025973e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+9.8620129870129864e-04*w45*Jinv00-1.1834415584415585e-02*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-1.1323051948051949e-03*w11*Jinv22-9.1314935064935063e-04*Jinv11*w1+2.9586038961038961e-03*w13*Jinv11+9.1314935064935063e-04*w28*Jinv11-2.6298701298701297e-03*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+9.8620129870129864e-04*w46*Jinv10-1.1834415584415585e-02*Jinv22*w17+3.9448051948051946e-03*Jinv00*w33-9.8620129870129864e-04*w22*Jinv12+3.2873376623376621e-03*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3+1.9724025974025973e-03*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-1.1323051948051949e-03*Jinv22*w2-2.9586038961038961e-03*Jinv00*w18-1.9724025974025973e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-2.3011363636363637e-03*Jinv01*w24-1.1323051948051949e-03*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+6.5746753246753243e-04*w19*Jinv11-7.8896103896103891e-03*Jinv12*w34-1.1834415584415585e-02*Jinv22*w44+2.3011363636363637e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11+2.3668831168831169e-02*w41*Jinv20+1.9724025974025973e-03*Jinv12*w13-2.9586038961038961e-03*w19*Jinv10+3.9448051948051946e-03*w34*Jinv10+2.3011363636363637e-03*w32*Jinv21+6.5746753246753243e-04*Jinv21*w20-3.6160714285714286e-03*w6*Jinv00+1.3149350649350649e-03*w37*Jinv12-1.9724025974025973e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41+1.6436688311688311e-03*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11-9.8620129870129864e-04*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21+1.9724025974025973e-03*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12-9.8620129870129864e-04*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10+9.1314935064935063e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+2.3668831168831169e-02*w39*Jinv00+1.9724025974025973e-03*Jinv02*w12+6.5746753246753243e-04*Jinv01*w18-1.1323051948051949e-03*Jinv02*w9-6.5746753246753243e-04*w6*Jinv01-3.6160714285714286e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21-1.9724025974025973e-03*Jinv20*w38-9.8620129870129864e-04*Jinv22*w23+1.3149350649350649e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12+1.3149350649350649e-03*Jinv22*w8+1.9724025974025973e-03*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-2.9586038961038961e-03*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7-2.9586038961038961e-03*w21*Jinv01+1.3149350649350649e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-6.5746753246753243e-04*w7*Jinv11+1.6436688311688311e-03*w9*Jinv00-1.9724025974025973e-03*w37*Jinv10+6.5746753246753243e-04*Jinv21*w56-2.1915584415584417e-04*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02+1.9724025974025973e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22-6.5746753246753243e-04*w49*Jinv11+2.9586038961038961e-03*Jinv21*w14+1.6436688311688311e-03*w10*Jinv10-1.1323051948051949e-03*w58*Jinv12+1.9724025974025973e-03*Jinv02*w18+2.9586038961038961e-03*Jinv01*w12-6.5746753246753243e-04*Jinv21*w8+1.1323051948051949e-03*w57*Jinv00+1.1323051948051949e-03*Jinv20*w59-7.8896103896103891e-03*Jinv02*w33-6.9399350649350649e-04*w0*Jinv00+3.9448051948051946e-03*w35*Jinv20-2.9586038961038961e-03*Jinv21*w23+6.5746753246753243e-04*Jinv01*w54+6.5746753246753243e-04*w55*Jinv11-6.9399350649350649e-04*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02+5.9172077922077923e-03*Jinv20*w17+3.2873376623376621e-03*w30*Jinv00+2.3668831168831169e-02*w40*Jinv10-2.9586038961038961e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.9724025974025973e-03*Jinv22*w14+1.3149350649350649e-03*Jinv02*w36-1.1834415584415585e-02*Jinv12*w43+2.3011363636363637e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+1.9724025974025973e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-2.3011363636363637e-03*w25*Jinv11-1.9724025974025973e-03*w55*Jinv10+9.8620129870129864e-04*w47*Jinv20-1.1834415584415585e-02*w16*Jinv12-6.5746753246753243e-04*Jinv21*w50+9.1314935064935063e-04*Jinv01*w27-6.5746753246753243e-04*w48*Jinv01-1.1323051948051949e-03*Jinv12*w1)*D;
  A[39][16]=(0.0)*D;
  A[39][17]=(0.0)*D;
  A[39][18]=(-1.6436688311688311e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.0957792207792207e-03*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20-3.8352272727272725e-04*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-3.6525974025974026e-05*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45-7.3051948051948052e-05*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01-7.3051948051948052e-05*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-3.2873376623376621e-03*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27-3.6525974025974026e-05*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10-2.6298701298701297e-03*w21*Jinv00-3.2873376623376621e-03*Jinv01*w39+1.6436688311688311e-03*Jinv00*w18+1.0957792207792207e-03*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38-3.8352272727272725e-04*Jinv01*w24-1.3149350649350649e-03*w34*Jinv11+1.0957792207792207e-03*w19*Jinv11-3.8352272727272725e-04*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-3.8352272727272725e-04*w4*Jinv11-9.8620129870129868e-03*w41*Jinv20+1.6436688311688311e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+1.0957792207792207e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-8.2183441558441553e-04*w26*Jinv20-6.7573051948051951e-04*w11*Jinv20-1.3149350649350649e-03*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-3.8352272727272725e-04*w5*Jinv21-1.0957792207792208e-04*w49*Jinv10-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-9.8620129870129868e-03*w39*Jinv00+1.0957792207792207e-03*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+1.0957792207792207e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.0957792207792207e-03*Jinv20*w38-3.8352272727272725e-04*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53-2.6298701298701297e-03*w22*Jinv10-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51+1.0957792207792207e-03*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11-6.7573051948051951e-04*w9*Jinv00+1.0957792207792207e-03*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-8.2183441558441553e-04*w24*Jinv00-5.4788961038961042e-05*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-6.7573051948051951e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-5.4788961038961042e-05*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59-7.3051948051948052e-05*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-8.2183441558441553e-04*w25*Jinv10-7.3051948051948052e-05*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-4.9310064935064932e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-9.8620129870129868e-03*w40*Jinv10+1.6436688311688311e-03*w20*Jinv20-3.2873376623376621e-03*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-3.8352272727272725e-04*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20-5.4788961038961042e-05*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01-4.9310064935064932e-04*w10*Jinv11)*D;
  A[39][19]=(0.0)*D;
  A[39][20]=(0.0)*D;
  A[39][21]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-4.3831168831168834e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+5.6615259740259744e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+6.9399350649350649e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17-1.0957792207792207e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-2.7394480519480518e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+5.6615259740259744e-04*w58*Jinv11+5.6615259740259744e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00+7.6704545454545450e-04*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-1.2053571428571428e-03*w19*Jinv11-3.2873376623376621e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-4.3831168831168834e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-8.7662337662337668e-04*w32*Jinv21-1.2053571428571428e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35-8.7662337662337668e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-1.0957792207792207e-03*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-4.3831168831168834e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12-1.2053571428571428e-03*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9-1.2601461038961039e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-1.0957792207792207e-03*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23-3.2873376623376621e-04*w52*Jinv11+7.6704545454545450e-04*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+7.6704545454545450e-04*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-3.2873376623376621e-04*Jinv01*w51+7.6704545454545450e-04*Jinv02*w30+7.6704545454545450e-04*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10-1.2601461038961039e-03*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-6.0267857142857139e-04*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12-1.2601461038961039e-03*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+4.9310064935064932e-04*Jinv21*w23-6.0267857142857139e-04*Jinv01*w54-6.0267857142857139e-04*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+7.6704545454545450e-04*w48*Jinv02+6.9399350649350649e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20-8.7662337662337668e-04*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+6.9399350649350649e-04*w10*Jinv11)*D;
  A[39][22]=(0.0)*D;
  A[39][23]=(0.0)*D;
  A[39][24]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+4.6022727272727274e-03*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59+5.2962662337662338e-04*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+4.6022727272727274e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-7.6704545454545450e-04*w48*Jinv00+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+5.2962662337662338e-04*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+4.6022727272727274e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+5.2962662337662338e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w31*Jinv10+4.2735389610389614e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-1.3149350649350649e-03*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38+1.5888798701298701e-03*Jinv01*w24+6.5746753246753243e-04*w34*Jinv11-1.3697240259740260e-03*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00-3.8352272727272725e-04*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-1.3149350649350649e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-1.3697240259740260e-03*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+2.1367694805194807e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+6.5746753246753243e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.5888798701298701e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21-7.6704545454545450e-04*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-1.3697240259740260e-03*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.5888798701298701e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-7.1225649350649346e-04*w52*Jinv11-3.8352272727272725e-04*Jinv20*w53+4.2735389610389614e-03*w22*Jinv10+1.5888798701298701e-03*w22*Jinv11+1.5888798701298701e-03*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-1.3697240259740260e-03*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-1.3697240259740260e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.5888798701298701e-03*Jinv21*w23-1.3697240259740260e-03*Jinv01*w54-1.3697240259740260e-03*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+5.2962662337662338e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-3.8352272727272725e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+1.5888798701298701e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+5.2962662337662338e-04*w10*Jinv11)*D;
  A[39][25]=(0.0)*D;
  A[39][26]=(0.0)*D;
  A[39][27]=(-1.8262987012987013e-03*Jinv21*w44+2.1915584415584417e-04*Jinv01*w3-4.2613636363636367e-05*Jinv21*w59-4.2613636363636367e-05*Jinv21*w2-1.8262987012987013e-03*w42*Jinv01-1.9480519480519481e-04*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-1.8262987012987013e-03*w43*Jinv11-3.6525974025974025e-03*Jinv21*w41-4.2613636363636367e-05*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-3.6525974025974025e-03*Jinv01*w39+3.4699675324675324e-04*Jinv21*w38+3.2873376623376621e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11+2.9220779220779221e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+2.1915584415584417e-04*w4*Jinv11+4.3831168831168834e-04*w32*Jinv21+2.9220779220779221e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+4.3831168831168834e-04*w31*Jinv11+6.5746753246753243e-04*w46*Jinv11+2.1915584415584417e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33+2.9220779220779221e-04*Jinv01*w18+3.4699675324675324e-04*w37*Jinv11+3.4699675324675324e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+2.1915584415584417e-04*w52*Jinv11+6.5746753246753243e-04*w22*Jinv11+6.5746753246753243e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+3.4699675324675324e-04*w36*Jinv01+3.4699675324675324e-04*w7*Jinv11-3.2873376623376621e-04*Jinv21*w56+4.3831168831168834e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-3.2873376623376621e-04*Jinv01*w12+3.4699675324675324e-04*Jinv21*w8+6.5746753246753243e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-1.9480519480519481e-04*Jinv01*w9-3.6525974025974025e-03*w40*Jinv11+4.3831168831168834e-04*Jinv01*w30+3.2873376623376621e-04*w25*Jinv11+4.3831168831168834e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+4.3831168831168834e-04*w48*Jinv01-1.9480519480519481e-04*w10*Jinv11)*D;
  A[39][28]=(0.0)*D;
  A[39][29]=(0.0)*D;
  A[39][30]=(-3.1777597402597403e-03*Jinv00*w12-1.0592532467532468e-03*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+4.3831168831168834e-04*Jinv22*w53+2.5202922077922079e-03*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-3.1777597402597403e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-4.7666396103896100e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3+2.7394480519480520e-03*Jinv20*w23+1.2601461038961039e-03*Jinv22*w38+1.6436688311688311e-03*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+3.2873376623376621e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-9.6793831168831166e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45+8.7662337662337668e-04*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-6.9399350649350649e-04*w10*Jinv12+6.5746753246753243e-04*w48*Jinv00+3.2873376623376621e-04*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-1.3806818181818182e-02*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-1.0592532467532468e-03*Jinv00*w27-6.9399350649350649e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-3.1777597402597403e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-1.0592532467532468e-03*w28*Jinv11+6.5746753246753243e-04*Jinv20*w50-1.0592532467532468e-03*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-6.9034090909090911e-03*Jinv00*w33+6.0267857142857139e-04*w22*Jinv12-4.7666396103896100e-03*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+2.7394480519480520e-03*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22-1.3806818181818182e-02*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+8.7662337662337668e-04*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-6.9034090909090911e-03*w34*Jinv11+8.7662337662337668e-04*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+8.2183441558441553e-04*w51*Jinv00-5.9172077922077923e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-1.3806818181818182e-02*w41*Jinv20+1.0957792207792207e-03*Jinv12*w13+8.7662337662337668e-04*w19*Jinv10-6.9034090909090911e-03*w34*Jinv10-4.7666396103896100e-03*w32*Jinv21+8.7662337662337668e-04*Jinv21*w20+2.5202922077922079e-03*w6*Jinv00+1.2601461038961039e-03*w37*Jinv12+1.6436688311688311e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-9.6793831168831166e-04*w11*Jinv20+4.3831168831168834e-04*w52*Jinv12-6.9034090909090911e-03*Jinv21*w35-4.7666396103896100e-03*w31*Jinv11+3.2873376623376621e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11-3.1777597402597403e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+8.7662337662337668e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+6.0267857142857139e-04*w21*Jinv02+6.5746753246753243e-04*w49*Jinv10-1.0592532467532468e-03*Jinv21*w29-6.9034090909090911e-03*Jinv01*w33-1.3806818181818182e-02*w39*Jinv00+1.0957792207792207e-03*Jinv02*w12+8.7662337662337668e-04*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-6.9399350649350649e-04*Jinv02*w9+2.5202922077922079e-03*w6*Jinv01+2.5202922077922079e-03*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+6.0267857142857139e-04*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+1.3697240259740260e-03*w46*Jinv12+8.2183441558441553e-04*Jinv20*w53+1.2053571428571428e-03*Jinv22*w8+2.7394480519480520e-03*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+2.7394480519480520e-03*w22*Jinv11+1.2053571428571428e-03*Jinv12*w7+2.7394480519480520e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01+8.2183441558441553e-04*w52*Jinv10+2.5202922077922079e-03*w7*Jinv11-9.6793831168831166e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+1.6436688311688311e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-5.6615259740259744e-04*Jinv02*w27+1.2053571428571428e-03*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22+6.5746753246753243e-04*w49*Jinv11-3.1777597402597403e-03*Jinv21*w14-9.6793831168831166e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-3.1777597402597403e-03*Jinv01*w12+2.5202922077922079e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-6.9034090909090911e-03*w35*Jinv20+2.7394480519480520e-03*Jinv21*w23+1.6436688311688311e-03*Jinv01*w54+1.6436688311688311e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-9.6793831168831166e-04*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-4.7666396103896100e-03*w30*Jinv00-1.3806818181818182e-02*w40*Jinv10+8.7662337662337668e-04*w20*Jinv20-5.6615259740259744e-04*Jinv22*w29+1.0957792207792207e-03*Jinv22*w14-1.3806818181818182e-02*w40*Jinv11+1.2601461038961039e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20-4.7666396103896100e-03*Jinv01*w30+4.3831168831168834e-04*Jinv02*w51+8.7662337662337668e-04*Jinv22*w56-5.6615259740259744e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+1.6436688311688311e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+6.5746753246753243e-04*Jinv21*w50-1.0592532467532468e-03*Jinv01*w27+6.5746753246753243e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-9.6793831168831166e-04*w10*Jinv11)*D;
  A[39][31]=(0.0)*D;
  A[39][32]=(0.0)*D;
  A[39][33]=(3.2873376623376621e-03*Jinv00*w12+1.1323051948051949e-03*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53-3.6160714285714286e-03*Jinv20*w8+9.1314935064935063e-04*w57*Jinv02+3.2873376623376621e-03*w13*Jinv10-1.1834415584415585e-02*Jinv21*w44+2.3011363636363637e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23+6.5746753246753243e-04*Jinv22*w38+9.8620129870129864e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-9.1314935064935063e-04*w0*Jinv02-1.1323051948051949e-03*Jinv21*w2-1.1834415584415585e-02*w42*Jinv01-1.1323051948051949e-03*w11*Jinv21-7.8896103896103891e-03*w16*Jinv11-7.8896103896103891e-03*Jinv21*w17+1.9724025974025973e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02+1.9724025974025973e-03*w48*Jinv00-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-1.1323051948051949e-03*w0*Jinv01-6.9399350649350649e-04*w1*Jinv10+3.9448051948051946e-03*w16*Jinv10-1.1834415584415585e-02*w43*Jinv11-1.9724025974025973e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+1.1323051948051949e-03*Jinv00*w27-1.1323051948051949e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-7.8896103896103891e-03*w15*Jinv01-1.1323051948051949e-03*w28*Jinv11+1.9724025974025973e-03*Jinv20*w50+1.1323051948051949e-03*Jinv20*w29-1.9724025974025973e-03*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33-6.5746753246753243e-04*w22*Jinv12+4.9310064935064934e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3-2.6298701298701297e-03*w21*Jinv00+2.9586038961038961e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-9.1314935064935063e-04*Jinv22*w2-1.9724025974025973e-03*Jinv00*w18-2.9586038961038961e-03*w36*Jinv00+1.9724025974025973e-03*Jinv21*w38+6.5746753246753243e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24+9.1314935064935063e-04*Jinv22*w59-1.1834415584415585e-02*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11-9.8620129870129864e-04*Jinv21*w53-3.9448051948051946e-03*Jinv22*w44+2.3011363636363637e-03*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+2.3668831168831169e-02*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13-1.9724025974025973e-03*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10+1.9724025974025973e-03*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20-3.6160714285714286e-03*w6*Jinv00+6.5746753246753243e-04*w37*Jinv12+9.8620129870129864e-04*Jinv20*w56+1.6436688311688311e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12-1.1834415584415585e-02*Jinv21*w35+1.9724025974025973e-03*w31*Jinv11+1.9724025974025973e-03*w46*Jinv11+3.2873376623376621e-03*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12-6.5746753246753243e-04*w21*Jinv02+1.9724025974025973e-03*w49*Jinv10-1.1323051948051949e-03*Jinv21*w29-1.1834415584415585e-02*Jinv01*w33+2.3668831168831169e-02*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18+1.9724025974025973e-03*w37*Jinv11+1.3149350649350649e-03*w6*Jinv01-3.6160714285714286e-03*w7*Jinv10+1.9724025974025973e-03*w47*Jinv21-2.9586038961038961e-03*Jinv20*w38-6.5746753246753243e-04*Jinv22*w23-9.8620129870129864e-04*w52*Jinv11+2.9586038961038961e-03*w31*Jinv12+6.5746753246753243e-04*w46*Jinv12-6.5746753246753243e-04*Jinv22*w8-2.6298701298701297e-03*w22*Jinv10-2.9586038961038961e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11-6.5746753246753243e-04*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01-9.8620129870129864e-04*Jinv01*w51+2.9586038961038961e-03*Jinv02*w30-2.9586038961038961e-03*Jinv12*w49+1.9724025974025973e-03*w36*Jinv01+1.3149350649350649e-03*w7*Jinv11+1.6436688311688311e-03*w9*Jinv00-2.9586038961038961e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56-6.5746753246753243e-04*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5+6.5746753246753243e-04*w47*Jinv22-9.8620129870129864e-04*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+1.6436688311688311e-03*w10*Jinv10+9.1314935064935063e-04*w58*Jinv12+1.3149350649350649e-03*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-6.9399350649350649e-04*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11-6.9399350649350649e-04*Jinv20*w2+3.9448051948051946e-03*w15*Jinv00-2.9586038961038961e-03*w48*Jinv02-1.1323051948051949e-03*Jinv01*w9+3.9448051948051946e-03*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00+2.3668831168831169e-02*w40*Jinv10-1.9724025974025973e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11+6.5746753246753243e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43+2.3011363636363637e-03*w5*Jinv20+1.9724025974025973e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56+1.3149350649350649e-03*w25*Jinv11+9.8620129870129864e-04*w55*Jinv10-1.9724025974025973e-03*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-9.8620129870129864e-04*Jinv21*w50-1.1323051948051949e-03*Jinv01*w27-9.8620129870129864e-04*w48*Jinv01-9.1314935064935063e-04*Jinv12*w1-1.1323051948051949e-03*w10*Jinv11)*D;
  A[39][34]=(0.0)*D;
  A[39][35]=(0.0)*D;
  A[39][36]=(-1.0957792207792208e-04*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-3.2873376623376621e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8-3.6525974025974026e-05*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10-3.2873376623376621e-03*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23+1.0957792207792207e-03*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54-3.8352272727272725e-04*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10-5.4788961038961042e-05*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-7.3051948051948052e-05*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-2.6298701298701297e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-3.8352272727272725e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-3.6525974025974026e-05*Jinv22*w2+1.0957792207792207e-03*Jinv00*w18+1.6436688311688311e-03*w36*Jinv00-5.4788961038961042e-05*Jinv02*w45-3.6525974025974026e-05*Jinv22*w59-8.2183441558441553e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10-9.8620129870129868e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+1.0957792207792207e-03*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10+1.0957792207792207e-03*w6*Jinv00+1.0957792207792207e-03*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-3.2873376623376621e-03*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20-6.7573051948051951e-04*w11*Jinv20-3.8352272727272725e-04*w52*Jinv12-3.8352272727272725e-04*Jinv22*w26-1.0957792207792208e-04*Jinv20*w14-5.4788961038961042e-05*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10-9.8620129870129868e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-4.9310064935064932e-04*Jinv02*w9+1.0957792207792207e-03*w7*Jinv10+1.6436688311688311e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23-5.4788961038961042e-05*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-8.2183441558441553e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*Jinv12*w7-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-8.2183441558441553e-04*w52*Jinv10-6.7573051948051951e-04*w9*Jinv00+1.6436688311688311e-03*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-6.7573051948051951e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-3.6525974025974026e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-7.3051948051948052e-05*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10-7.3051948051948052e-05*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+6.5746753246753243e-04*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-9.8620129870129868e-03*w40*Jinv10+1.0957792207792207e-03*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+1.0957792207792207e-03*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20-3.8352272727272725e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28-2.6298701298701297e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-3.6525974025974026e-05*Jinv12*w1)*D;
  A[39][37]=(0.0)*D;
  A[39][38]=(0.0)*D;
  A[39][39]=(3.9448051948051946e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53+3.2873376623376621e-03*Jinv20*w8+1.8262987012987013e-03*w57*Jinv02+3.9448051948051946e-03*w13*Jinv10-5.9172077922077923e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+3.9448051948051946e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3-1.9724025974025973e-03*Jinv20*w23+1.6436688311688311e-03*Jinv22*w38-2.9586038961038961e-03*Jinv00*w54-1.8262987012987013e-03*w0*Jinv02-1.8262987012987013e-03*Jinv21*w2-1.7751623376623376e-02*w43*Jinv10-5.9172077922077923e-03*w42*Jinv01-6.9034090909090911e-03*Jinv01*w45-6.9034090909090911e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10+5.9172077922077923e-03*w15*Jinv02-1.9724025974025973e-03*w48*Jinv00-2.3011363636363637e-03*Jinv21*w26-1.8262987012987013e-03*w0*Jinv01-2.2646103896103898e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10-5.9172077922077923e-03*w43*Jinv11-2.9586038961038961e-03*w45*Jinv00-5.9172077922077923e-03*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.8262987012987013e-03*Jinv11*w1+5.9172077922077923e-03*w13*Jinv11+1.8262987012987013e-03*w28*Jinv11-1.9724025974025973e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29-2.9586038961038961e-03*w46*Jinv10+5.9172077922077923e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12+3.9448051948051946e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3-1.9724025974025973e-03*w21*Jinv00+5.9172077922077923e-03*w32*Jinv22-1.8262987012987013e-03*Jinv22*w2+4.9310064935064934e-03*Jinv00*w18+4.9310064935064934e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-2.3011363636363637e-03*Jinv01*w24+1.8262987012987013e-03*Jinv22*w59+5.9172077922077923e-03*w34*Jinv11+1.6436688311688311e-03*w19*Jinv11-9.8620129870129864e-04*w51*Jinv00-5.9172077922077923e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11-3.5503246753246752e-02*w41*Jinv20+6.9034090909090911e-03*Jinv12*w13+4.9310064935064934e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+6.9034090909090911e-03*w32*Jinv21+1.6436688311688311e-03*Jinv21*w20+3.2873376623376621e-03*w6*Jinv00+1.6436688311688311e-03*w37*Jinv12-2.9586038961038961e-03*Jinv20*w56-9.8620129870129864e-04*w26*Jinv20-2.7759740259740259e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12+5.9172077922077923e-03*Jinv21*w35+6.9034090909090911e-03*w31*Jinv11-6.9034090909090911e-03*w46*Jinv11+3.9448051948051946e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21-6.9034090909090911e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02-1.9724025974025973e-03*w49*Jinv10+1.8262987012987013e-03*Jinv21*w29+5.9172077922077923e-03*Jinv01*w33-3.5503246753246752e-02*w39*Jinv00+6.9034090909090911e-03*Jinv02*w12+1.6436688311688311e-03*Jinv01*w18-1.6436688311688311e-03*w6*Jinv01+3.2873376623376621e-03*w7*Jinv10-6.9034090909090911e-03*w47*Jinv21+4.9310064935064934e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+5.9172077922077923e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53-1.6436688311688311e-03*Jinv22*w8-1.9724025974025973e-03*w22*Jinv10-5.9172077922077923e-03*Jinv22*w50-5.9172077922077923e-03*w22*Jinv11-1.6436688311688311e-03*Jinv12*w7-5.9172077922077923e-03*w21*Jinv01+5.9172077922077923e-03*Jinv02*w30-5.9172077922077923e-03*Jinv12*w49-9.8620129870129864e-04*w52*Jinv10-1.6436688311688311e-03*w7*Jinv11-2.7759740259740259e-03*w9*Jinv00+4.9310064935064934e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00-1.6436688311688311e-03*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11+5.9172077922077923e-03*Jinv21*w14-2.7759740259740259e-03*w10*Jinv10-1.7751623376623376e-02*Jinv20*w44+1.8262987012987013e-03*w58*Jinv12+5.9172077922077923e-03*Jinv01*w12-1.6436688311688311e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-2.2646103896103898e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-5.9172077922077923e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-2.2646103896103898e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-5.9172077922077923e-03*w48*Jinv02-1.1834415584415585e-02*Jinv20*w17+3.9448051948051946e-03*w30*Jinv00-3.5503246753246752e-02*w40*Jinv10+4.9310064935064934e-03*w20*Jinv20+6.9034090909090911e-03*Jinv22*w14+1.6436688311688311e-03*Jinv02*w36-5.9172077922077923e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+6.9034090909090911e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-6.9034090909090911e-03*Jinv22*w56-2.3011363636363637e-03*w25*Jinv11-2.9586038961038961e-03*w55*Jinv10-1.7751623376623376e-02*w42*Jinv00-2.9586038961038961e-03*w47*Jinv20+5.9172077922077923e-03*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+1.8262987012987013e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01-1.8262987012987013e-03*Jinv12*w1)*D;
  A[39][40]=(0.0)*D;
  A[39][41]=(0.0)*D;
  A[39][42]=(-3.9448051948051946e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53-3.2873376623376621e-03*Jinv20*w8+1.1323051948051949e-03*w57*Jinv02-3.9448051948051946e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+1.1834415584415585e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-3.9448051948051946e-03*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+1.9724025974025973e-03*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38+2.9586038961038961e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+9.8620129870129864e-04*w25*Jinv12+1.1323051948051949e-03*w0*Jinv02+1.1323051948051949e-03*Jinv21*w2+1.7751623376623376e-02*w43*Jinv10+1.1834415584415585e-02*w42*Jinv01+1.1323051948051949e-03*w11*Jinv21+7.8896103896103891e-03*w16*Jinv11+7.8896103896103891e-03*Jinv21*w17-1.9724025974025973e-03*Jinv01*w45-1.9724025974025973e-03*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+1.1834415584415585e-02*w15*Jinv02+1.1323051948051949e-03*w10*Jinv12+1.9724025974025973e-03*w48*Jinv00+9.8620129870129864e-04*Jinv02*w24+7.8896103896103891e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+1.1323051948051949e-03*w0*Jinv01+2.2646103896103898e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10-1.9724025974025973e-03*Jinv22*w20+1.1834415584415585e-02*w43*Jinv11+2.9586038961038961e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+1.1834415584415585e-02*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+1.1323051948051949e-03*w11*Jinv22+1.1323051948051949e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+7.8896103896103891e-03*w15*Jinv01+1.1323051948051949e-03*w28*Jinv11+1.9724025974025973e-03*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29+2.9586038961038961e-03*w46*Jinv10+1.1834415584415585e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12-3.9448051948051946e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3+1.9724025974025973e-03*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+1.1323051948051949e-03*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-1.9724025974025973e-03*Jinv21*w38+9.8620129870129864e-04*Jinv02*w45-1.3149350649350649e-03*Jinv01*w24+1.1323051948051949e-03*Jinv22*w59+1.1834415584415585e-02*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11+9.8620129870129864e-04*Jinv21*w53+9.8620129870129864e-04*w51*Jinv00+7.8896103896103891e-03*Jinv12*w34+1.1834415584415585e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+3.5503246753246752e-02*w41*Jinv20-1.9724025974025973e-03*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-1.9724025974025973e-03*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-3.2873376623376621e-03*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12+2.9586038961038961e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+9.8620129870129864e-04*w26*Jinv20+2.7759740259740259e-03*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+1.1834415584415585e-02*Jinv21*w35-1.9724025974025973e-03*w31*Jinv11+9.8620129870129864e-04*Jinv22*w26-1.9724025974025973e-03*w46*Jinv11-3.9448051948051946e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-1.9724025974025973e-03*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02+1.9724025974025973e-03*w49*Jinv10+1.1323051948051949e-03*Jinv21*w29+1.1834415584415585e-02*Jinv01*w33+3.5503246753246752e-02*w39*Jinv00-1.9724025974025973e-03*Jinv02*w12-1.3149350649350649e-03*Jinv01*w18-1.9724025974025973e-03*w37*Jinv11+1.1323051948051949e-03*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01-3.2873376623376621e-03*w7*Jinv10-1.9724025974025973e-03*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+9.8620129870129864e-04*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+9.8620129870129864e-04*w46*Jinv12+9.8620129870129864e-04*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8+1.9724025974025973e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01+9.8620129870129864e-04*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49-1.9724025974025973e-03*w36*Jinv01+9.8620129870129864e-04*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11+2.7759740259740259e-03*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10+9.8620129870129864e-04*Jinv21*w56+9.8620129870129864e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-1.3149350649350649e-03*w6*Jinv02-1.9724025974025973e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5+9.8620129870129864e-04*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+2.7759740259740259e-03*w10*Jinv10+1.7751623376623376e-02*Jinv20*w44+1.1323051948051949e-03*w58*Jinv12-1.9724025974025973e-03*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+7.8896103896103891e-03*Jinv02*w33+2.2646103896103898e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+9.8620129870129864e-04*Jinv01*w54+9.8620129870129864e-04*w55*Jinv11+9.8620129870129864e-04*w25*Jinv10+2.2646103896103898e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+1.1323051948051949e-03*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-3.9448051948051946e-03*w30*Jinv00+3.5503246753246752e-02*w40*Jinv10-4.9310064935064934e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29-1.9724025974025973e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11-1.3149350649350649e-03*Jinv02*w36+1.1834415584415585e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-1.9724025974025973e-03*Jinv01*w30-1.3149350649350649e-03*Jinv02*w51-1.9724025974025973e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-1.3149350649350649e-03*w25*Jinv11+2.9586038961038961e-03*w55*Jinv10+1.7751623376623376e-02*w42*Jinv00+2.9586038961038961e-03*w47*Jinv20+1.1834415584415585e-02*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+1.1323051948051949e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01+1.1323051948051949e-03*Jinv12*w1+1.1323051948051949e-03*w10*Jinv11)*D;
  A[39][43]=(0.0)*D;
  A[39][44]=(0.0)*D;
  A[39][45]=(1.9724025974025973e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+4.9310064935064932e-04*w57*Jinv02+1.9724025974025973e-03*Jinv02*w39+6.9034090909090911e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-5.4788961038961042e-05*Jinv22*w38+4.3831168831168834e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+1.0592532467532468e-03*Jinv21*w2+6.9034090909090911e-03*w42*Jinv01+9.6793831168831166e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+4.7666396103896100e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+2.1367694805194807e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+1.0592532467532468e-03*w0*Jinv01-1.3149350649350649e-03*Jinv22*w20+6.9034090909090911e-03*w43*Jinv11+1.3806818181818182e-02*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02+2.7394480519480518e-04*w11*Jinv22+1.0592532467532468e-03*Jinv11*w1-2.7394480519480520e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17+4.2735389610389614e-03*w22*Jinv12-3.8352272727272725e-04*Jinv02*w3-7.6704545454545450e-04*w32*Jinv22+1.3806818181818182e-02*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-1.3149350649350649e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+1.5888798701298701e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.5202922077922079e-03*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-2.5202922077922079e-03*Jinv21*w20-5.4788961038961042e-05*w37*Jinv12+1.9724025974025973e-03*Jinv22*w41-3.8352272727272725e-04*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+4.7666396103896100e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-2.5202922077922079e-03*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9-8.7662337662337668e-04*w6*Jinv01+4.7666396103896100e-03*w47*Jinv21+4.2735389610389614e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11-7.6704545454545450e-04*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8-7.6704545454545450e-04*Jinv22*w50+3.1777597402597403e-03*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7+3.1777597402597403e-03*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51-7.6704545454545450e-04*Jinv02*w30-7.6704545454545450e-04*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-8.7662337662337668e-04*w7*Jinv11-6.5746753246753243e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-1.6436688311688311e-03*w49*Jinv11-2.7394480519480520e-03*Jinv21*w14+4.9310064935064932e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18-2.7394480519480520e-03*Jinv01*w12-8.7662337662337668e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+3.1777597402597403e-03*Jinv21*w23-6.5746753246753243e-04*Jinv01*w54-6.5746753246753243e-04*w55*Jinv11-7.6704545454545450e-04*w48*Jinv02+9.6793831168831166e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+1.3806818181818182e-02*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30-3.8352272727272725e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.5888798701298701e-03*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12-1.6436688311688311e-03*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-1.6436688311688311e-03*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+9.6793831168831166e-04*w10*Jinv11)*D;
  A[39][46]=(0.0)*D;
  A[39][47]=(0.0)*D;
  A[39][48]=(7.6704545454545450e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23-1.2053571428571428e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-3.2873376623376621e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-1.0957792207792207e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+6.9399350649350649e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00-3.2873376623376621e-04*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27+6.9399350649350649e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+7.6704545454545450e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-4.3831168831168834e-04*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38-6.0267857142857139e-04*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-8.7662337662337668e-04*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-1.2053571428571428e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11-3.2873376623376621e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21-1.0957792207792207e-03*Jinv02*w54-4.3831168831168834e-04*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-8.7662337662337668e-04*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11+6.9399350649350649e-04*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12-6.0267857142857139e-04*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53-1.2601461038961039e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50+7.6704545454545450e-04*w22*Jinv11-1.2601461038961039e-03*Jinv12*w7+7.6704545454545450e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00+5.6615259740259744e-04*Jinv02*w27-1.2601461038961039e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12-4.3831168831168834e-04*Jinv22*w5-6.0267857142857139e-04*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+7.6704545454545450e-04*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18+7.6704545454545450e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+5.9172077922077923e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+7.6704545454545450e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+4.9310064935064932e-04*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.6615259740259744e-04*Jinv22*w29-8.7662337662337668e-04*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-1.2053571428571428e-03*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-1.0957792207792207e-03*Jinv22*w56+5.6615259740259744e-04*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[39][49]=(0.0)*D;
  A[39][50]=(0.0)*D;
  A[39][51]=(-7.6704545454545450e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-7.6704545454545450e-04*Jinv20*w23-1.3697240259740260e-03*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-7.1225649350649346e-04*w25*Jinv12+5.2962662337662338e-04*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10+1.5888798701298701e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+4.6022727272727274e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22+4.2735389610389614e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22+5.2962662337662338e-04*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00-1.3697240259740260e-03*Jinv02*w45-3.8352272727272725e-04*Jinv22*w59+2.1367694805194807e-03*w51*Jinv00+4.6022727272727274e-03*Jinv12*w34+4.6022727272727274e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10+1.9724025974025973e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-1.3697240259740260e-03*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+1.5888798701298701e-03*w52*Jinv12-7.1225649350649346e-04*Jinv22*w26-7.6704545454545450e-04*Jinv20*w14+1.5888798701298701e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+1.9724025974025973e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+5.2962662337662338e-04*Jinv02*w9-5.4788961038961042e-05*w7*Jinv10-1.3149350649350649e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-1.3697240259740260e-03*w31*Jinv12-1.3697240259740260e-03*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.5888798701298701e-03*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7-1.3697240259740260e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49+2.1367694805194807e-03*w52*Jinv10+2.7394480519480518e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10-3.8352272727272725e-04*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-1.3697240259740260e-03*w47*Jinv22+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-3.8352272727272725e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+4.6022727272727274e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-3.8352272727272725e-04*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.5888798701298701e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-1.3697240259740260e-03*Jinv02*w36+4.6022727272727274e-03*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20+1.5888798701298701e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+5.2962662337662338e-04*Jinv12*w28+4.2735389610389614e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+5.2962662337662338e-04*Jinv12*w1)*D;
  A[39][52]=(0.0)*D;
  A[39][53]=(0.0)*D;
  A[39][54]=(1.3806818181818182e-02*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-7.6704545454545450e-04*w57*Jinv02+1.3806818181818182e-02*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*Jinv01*w3-2.5202922077922079e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12+1.0592532467532468e-03*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01+2.7394480519480518e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+4.7666396103896100e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+9.6793831168831166e-04*w10*Jinv12-8.2183441558441553e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-1.3149350649350649e-03*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+1.9724025974025973e-03*Jinv21*w41+6.9034090909090911e-03*w42*Jinv02+9.6793831168831166e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-7.6704545454545450e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17-1.6436688311688311e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-2.7394480519480520e-03*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+1.0592532467532468e-03*Jinv22*w2-1.3149350649350649e-03*Jinv21*w38-6.5746753246753243e-04*Jinv02*w45-3.8352272727272725e-04*Jinv01*w24-7.6704545454545450e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+6.9034090909090911e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-2.5202922077922079e-03*w37*Jinv12+1.3806818181818182e-02*Jinv22*w41+1.5888798701298701e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-3.8352272727272725e-04*w5*Jinv21+4.7666396103896100e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.6436688311688311e-03*w21*Jinv02+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+9.6793831168831166e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-2.1367694805194807e-03*w47*Jinv21-1.6436688311688311e-03*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-2.7394480519480520e-03*w31*Jinv12-6.5746753246753243e-04*w46*Jinv12-8.7662337662337668e-04*Jinv22*w8+3.1777597402597403e-03*Jinv22*w50-7.6704545454545450e-04*w22*Jinv11-8.7662337662337668e-04*Jinv12*w7-7.6704545454545450e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-2.7394480519480520e-03*Jinv02*w30+3.1777597402597403e-03*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11+4.2735389610389614e-03*Jinv21*w56+4.3831168831168834e-04*Jinv02*w27-8.7662337662337668e-04*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-6.5746753246753243e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-7.6704545454545450e-04*Jinv21*w14-7.6704545454545450e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18-7.6704545454545450e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-7.6704545454545450e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11+3.1777597402597403e-03*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+4.3831168831168834e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-2.5202922077922079e-03*Jinv02*w36+6.9034090909090911e-03*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+1.5888798701298701e-03*Jinv02*w51+4.7666396103896100e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-3.8352272727272725e-04*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01+1.0592532467532468e-03*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[39][55]=(0.0)*D;
  A[39][56]=(0.0)*D;
  A[39][57]=(-3.6525974025974025e-03*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-3.6525974025974025e-03*Jinv02*w39+2.9220779220779221e-04*Jinv22*w38+2.1915584415584417e-04*w25*Jinv12-4.2613636363636367e-05*w0*Jinv02+6.5746753246753243e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-1.9480519480519481e-04*w10*Jinv12+2.1915584415584417e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35+3.4699675324675324e-04*Jinv22*w20-1.8262987012987013e-03*w42*Jinv02-1.9480519480519481e-04*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+4.3831168831168834e-04*w22*Jinv12+2.1915584415584417e-04*Jinv02*w3-3.2873376623376621e-04*w32*Jinv22-4.2613636363636367e-05*Jinv22*w2-3.2873376623376621e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-1.8262987012987013e-03*Jinv12*w34-1.8262987012987013e-03*Jinv22*w44+4.3831168831168834e-04*Jinv12*w13+2.9220779220779221e-04*w37*Jinv12-3.6525974025974025e-03*Jinv22*w41+3.2873376623376621e-04*w52*Jinv12+2.1915584415584417e-04*Jinv22*w26+6.5746753246753243e-04*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12+4.3831168831168834e-04*w21*Jinv02+4.3831168831168834e-04*Jinv02*w12-1.9480519480519481e-04*Jinv02*w9+4.3831168831168834e-04*Jinv22*w23-3.2873376623376621e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+3.4699675324675324e-04*Jinv22*w8+6.5746753246753243e-04*Jinv22*w50+3.4699675324675324e-04*Jinv12*w7-3.2873376623376621e-04*Jinv02*w30+6.5746753246753243e-04*Jinv12*w49-4.2613636363636367e-05*Jinv02*w27+3.4699675324675324e-04*w6*Jinv02+3.4699675324675324e-04*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+3.4699675324675324e-04*Jinv02*w18-1.8262987012987013e-03*Jinv02*w33+6.5746753246753243e-04*w48*Jinv02-4.2613636363636367e-05*Jinv22*w29+4.3831168831168834e-04*Jinv22*w14+2.9220779220779221e-04*Jinv02*w36-1.8262987012987013e-03*Jinv12*w43+3.2873376623376621e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-4.2613636363636367e-05*Jinv12*w1)*D;
  A[39][58]=(0.0)*D;
  A[39][59]=(0.0)*D;
  A[40][0]=(0.0)*D;
  A[40][1]=(-6.5746753246753243e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10+3.6525974025974025e-03*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53-2.9220779220779221e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+3.6525974025974025e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-3.4699675324675324e-04*Jinv22*w38-4.3831168831168834e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-2.1915584415584417e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+1.9480519480519481e-04*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17-4.3831168831168834e-04*Jinv01*w45-4.3831168831168834e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+1.8262987012987013e-03*w15*Jinv02+1.9480519480519481e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+1.8262987012987013e-03*w16*Jinv10-3.4699675324675324e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-4.3831168831168834e-04*w45*Jinv00+3.6525974025974025e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27+1.9480519480519481e-04*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-6.5746753246753243e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01+4.2613636363636367e-05*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-4.3831168831168834e-04*w46*Jinv10+1.8262987012987013e-03*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-6.5746753246753243e-04*w32*Jinv22+3.6525974025974025e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-3.4699675324675324e-04*Jinv00*w18-3.4699675324675324e-04*w36*Jinv00-3.4699675324675324e-04*Jinv21*w38-4.3831168831168834e-04*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+1.8262987012987013e-03*w34*Jinv11-3.4699675324675324e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-2.1915584415584417e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11+3.6525974025974025e-03*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-3.4699675324675324e-04*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21-3.4699675324675324e-04*Jinv21*w20-2.9220779220779221e-04*w6*Jinv00-3.4699675324675324e-04*w37*Jinv12-4.3831168831168834e-04*Jinv20*w56+3.6525974025974025e-03*Jinv22*w41-2.1915584415584417e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12+1.8262987012987013e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26-4.3831168831168834e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21-4.3831168831168834e-04*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+3.2873376623376621e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+1.8262987012987013e-03*Jinv01*w33+3.6525974025974025e-03*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-3.4699675324675324e-04*Jinv01*w18-3.4699675324675324e-04*w37*Jinv11+1.9480519480519481e-04*Jinv02*w9-2.9220779220779221e-04*w6*Jinv01-2.9220779220779221e-04*w7*Jinv10-4.3831168831168834e-04*w47*Jinv21-3.4699675324675324e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11-6.5746753246753243e-04*w31*Jinv12-4.3831168831168834e-04*w46*Jinv12-2.1915584415584417e-04*Jinv20*w53-2.9220779220779221e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11-2.9220779220779221e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-6.5746753246753243e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-3.4699675324675324e-04*w36*Jinv01-2.1915584415584417e-04*w52*Jinv10-2.9220779220779221e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-3.4699675324675324e-04*w37*Jinv10-4.3831168831168834e-04*Jinv21*w56-2.1915584415584417e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27-2.9220779220779221e-04*w6*Jinv02-3.4699675324675324e-04*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5-4.3831168831168834e-04*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-6.5746753246753243e-04*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12-3.4699675324675324e-04*Jinv02*w18-6.5746753246753243e-04*Jinv01*w12-2.9220779220779221e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+1.8262987012987013e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+1.8262987012987013e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-4.3831168831168834e-04*Jinv01*w54-4.3831168831168834e-04*w55*Jinv11-2.1915584415584417e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+1.8262987012987013e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+1.9480519480519481e-04*Jinv01*w9+1.8262987012987013e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+3.6525974025974025e-03*w40*Jinv10-3.4699675324675324e-04*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+3.6525974025974025e-03*w40*Jinv11-3.4699675324675324e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51-4.3831168831168834e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-2.1915584415584417e-04*w25*Jinv11-4.3831168831168834e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-4.3831168831168834e-04*w47*Jinv20+1.8262987012987013e-03*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+1.9480519480519481e-04*w10*Jinv11)*D;
  A[40][2]=(0.0)*D;
  A[40][3]=(0.0)*D;
  A[40][4]=(2.6846590909090908e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-5.2962662337662338e-04*w57*Jinv02+2.6846590909090908e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+5.4788961038961036e-04*w3*Jinv00+2.6846590909090908e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-4.6022727272727274e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01-1.6436688311688311e-04*w1*Jinv10-3.6160714285714286e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-5.2962662337662338e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10-4.6022727272727274e-03*Jinv22*w17-3.6160714285714286e-03*Jinv00*w33+1.3697240259740260e-03*w22*Jinv12+2.6846590909090908e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+6.5746753246753243e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-5.2962662337662338e-04*Jinv22*w59-4.6022727272727274e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.2873376623376621e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+6.5746753246753243e-04*w19*Jinv10-3.6160714285714286e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+3.2873376623376621e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-4.6022727272727274e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.6846590909090908e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+1.3697240259740260e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-5.2962662337662338e-04*Jinv21*w29-4.6022727272727274e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9+1.3697240259740260e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+6.5746753246753243e-04*Jinv20*w38+1.3697240259740260e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+3.2873376623376621e-04*Jinv20*w53+1.3697240259740260e-03*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11+1.3697240259740260e-03*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+3.2873376623376621e-04*w52*Jinv10+1.3697240259740260e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+3.2873376623376621e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+1.3697240259740260e-03*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+1.3697240259740260e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-5.2962662337662338e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+1.3697240259740260e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33-1.6436688311688311e-04*w0*Jinv00-3.6160714285714286e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+3.2873376623376621e-04*w25*Jinv10-1.6436688311688311e-04*Jinv20*w2-3.6160714285714286e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-3.6160714285714286e-03*Jinv20*w17+2.6846590909090908e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+5.4788961038961036e-04*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20-4.6022727272727274e-03*w16*Jinv12+1.3697240259740260e-03*Jinv21*w50-5.2962662337662338e-04*Jinv01*w27+1.3697240259740260e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[40][5]=(0.0)*D;
  A[40][6]=(0.0)*D;
  A[40][7]=(-2.5750811688311690e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+3.2873376623376621e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53+5.4788961038961036e-04*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02-2.5750811688311690e-03*w13*Jinv10+3.2873376623376621e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00-2.5750811688311690e-03*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+3.8352272727272725e-04*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00+3.2873376623376621e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-2.5750811688311690e-03*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+3.2873376623376621e-03*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2-1.0957792207792207e-03*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24+3.6525974025974026e-05*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-6.5746753246753243e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20+5.4788961038961036e-04*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56+3.2873376623376621e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20-1.8262987012987014e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+3.8352272727272725e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-2.5750811688311690e-03*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33-6.5746753246753243e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-1.0957792207792207e-03*w6*Jinv01+5.4788961038961036e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21+5.4788961038961042e-05*Jinv22*w23+3.8352272727272725e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53-1.0957792207792207e-03*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.0957792207792207e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10-1.0957792207792207e-03*w7*Jinv11-1.8262987012987014e-04*w9*Jinv00+5.4788961038961042e-05*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-1.0957792207792207e-03*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-1.8262987012987014e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-1.0957792207792207e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17-2.5750811688311690e-03*w30*Jinv00-6.5746753246753243e-03*w40*Jinv10+3.6525974025974026e-05*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14+3.2873376623376621e-03*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[40][8]=(0.0)*D;
  A[40][9]=(0.0)*D;
  A[40][10]=(5.4788961038961036e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10-3.1047077922077924e-04*Jinv20*w8+5.4788961038961036e-04*w13*Jinv10+2.1915584415584417e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23+5.4788961038961036e-04*Jinv00*w54-5.1136363636363640e-04*w43*Jinv10+3.0438311688311688e-05*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00+3.0438311688311688e-05*w1*Jinv10-5.1136363636363640e-04*w16*Jinv10+5.4788961038961036e-04*w45*Jinv00+3.0438311688311688e-05*Jinv00*w27+5.4788961038961036e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-5.1136363636363640e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10+5.4788961038961036e-04*w21*Jinv00-3.1047077922077924e-04*Jinv00*w18-3.1047077922077924e-04*w36*Jinv00+2.1915584415584417e-04*w51*Jinv00+2.1915584415584417e-04*w4*Jinv10+5.5519480519480519e-03*w41*Jinv20-3.1047077922077924e-04*w19*Jinv10-5.1136363636363640e-04*w34*Jinv10-3.1047077922077924e-04*w6*Jinv00+5.4788961038961036e-04*Jinv20*w56+2.1915584415584417e-04*w26*Jinv20+2.4350649350649351e-04*w11*Jinv20+5.4788961038961036e-04*Jinv20*w14+5.4788961038961036e-04*w49*Jinv10+5.5519480519480519e-03*w39*Jinv00-3.1047077922077924e-04*w7*Jinv10-3.1047077922077924e-04*Jinv20*w38+2.1915584415584417e-04*Jinv20*w53+5.4788961038961036e-04*w22*Jinv10+2.1915584415584417e-04*w52*Jinv10+2.4350649350649351e-04*w9*Jinv00-3.1047077922077924e-04*w37*Jinv10+2.1915584415584417e-04*w24*Jinv00+2.4350649350649351e-04*w10*Jinv10-5.1136363636363640e-04*Jinv20*w44+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+3.0438311688311688e-05*w0*Jinv00-5.1136363636363640e-04*w35*Jinv20+2.1915584415584417e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2-5.1136363636363640e-04*w15*Jinv00-5.1136363636363640e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00+5.5519480519480519e-03*w40*Jinv10-3.1047077922077924e-04*w20*Jinv20+2.1915584415584417e-04*w5*Jinv20+5.4788961038961036e-04*w55*Jinv10-5.1136363636363640e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20)*D;
  A[40][11]=(0.0)*D;
  A[40][12]=(0.0)*D;
  A[40][13]=(-4.7666396103896100e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-1.3806818181818182e-02*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+2.5202922077922079e-03*Jinv20*w8-1.0592532467532468e-03*w57*Jinv02-4.7666396103896100e-03*w13*Jinv10-1.3806818181818182e-02*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-3.1777597402597403e-03*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3+6.5746753246753243e-04*Jinv20*w23+8.7662337662337668e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-5.6615259740259744e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-6.9399350649350649e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17+8.7662337662337668e-04*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-1.0592532467532468e-03*w58*Jinv10-6.9034090909090911e-03*w15*Jinv02-9.6793831168831166e-04*w10*Jinv12+2.7394480519480520e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-5.6615259740259744e-04*w58*Jinv11-5.6615259740259744e-04*w57*Jinv01+4.3831168831168834e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-6.9034090909090911e-03*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+1.6436688311688311e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-9.6793831168831166e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11-5.9172077922077923e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+2.7394480519480520e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+1.6436688311688311e-03*w46*Jinv10-6.9034090909090911e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+6.5746753246753243e-04*w22*Jinv12-3.1777597402597403e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3+6.5746753246753243e-04*w21*Jinv00-3.1777597402597403e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+8.7662337662337668e-04*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38+1.6436688311688311e-03*Jinv02*w45+4.3831168831168834e-04*Jinv01*w24-1.0592532467532468e-03*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.2601461038961039e-03*w19*Jinv11+3.2873376623376621e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-1.3806818181818182e-02*w41*Jinv20-4.7666396103896100e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+1.0957792207792207e-03*w32*Jinv21+1.2601461038961039e-03*Jinv21*w20+2.5202922077922079e-03*w6*Jinv00+8.7662337662337668e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-1.3806818181818182e-02*Jinv22*w41+8.2183441558441553e-04*w26*Jinv20-9.6793831168831166e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35+1.0957792207792207e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+8.7662337662337668e-04*w46*Jinv11-4.7666396103896100e-03*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+6.5746753246753243e-04*w21*Jinv02+2.7394480519480520e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-1.3806818181818182e-02*w39*Jinv00-4.7666396103896100e-03*Jinv02*w12+1.2601461038961039e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-9.6793831168831166e-04*Jinv02*w9+1.2053571428571428e-03*w6*Jinv01+2.5202922077922079e-03*w7*Jinv10+8.7662337662337668e-04*w47*Jinv21+8.7662337662337668e-04*Jinv20*w38+6.5746753246753243e-04*Jinv22*w23+3.2873376623376621e-04*w52*Jinv11-3.1777597402597403e-03*w31*Jinv12+1.6436688311688311e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+2.5202922077922079e-03*Jinv22*w8+6.5746753246753243e-04*w22*Jinv10+2.7394480519480520e-03*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+2.5202922077922079e-03*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01+3.2873376623376621e-04*Jinv01*w51-3.1777597402597403e-03*Jinv02*w30+2.7394480519480520e-03*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10+1.2053571428571428e-03*w7*Jinv11-9.6793831168831166e-04*w9*Jinv00+8.7662337662337668e-04*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56+8.2183441558441553e-04*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+2.5202922077922079e-03*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+1.6436688311688311e-03*w47*Jinv22+6.0267857142857139e-04*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14-9.6793831168831166e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-1.0592532467532468e-03*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+1.2053571428571428e-03*Jinv21*w8-1.0592532467532468e-03*w57*Jinv00-1.0592532467532468e-03*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11+8.2183441558441553e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-6.9034090909090911e-03*w15*Jinv00+2.7394480519480520e-03*w48*Jinv02-6.9399350649350649e-04*Jinv01*w9-6.9034090909090911e-03*Jinv20*w17-3.1777597402597403e-03*w30*Jinv00-1.3806818181818182e-02*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-4.7666396103896100e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11+8.7662337662337668e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20+1.0957792207792207e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+4.3831168831168834e-04*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.6436688311688311e-03*w47*Jinv20-6.9034090909090911e-03*w16*Jinv12+6.0267857142857139e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+6.0267857142857139e-04*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-6.9399350649350649e-04*w10*Jinv11)*D;
  A[40][14]=(0.0)*D;
  A[40][15]=(0.0)*D;
  A[40][16]=(4.9310064935064934e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-3.6160714285714286e-03*Jinv20*w8-1.1323051948051949e-03*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44+2.3011363636363637e-03*w3*Jinv00+3.2873376623376621e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3+1.9724025974025973e-03*Jinv20*w23+1.3149350649350649e-03*Jinv22*w38-1.9724025974025973e-03*Jinv00*w54-9.8620129870129864e-04*w25*Jinv12-1.1323051948051949e-03*w0*Jinv02-9.1314935064935063e-04*Jinv21*w2-3.9448051948051946e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45+1.9724025974025973e-03*Jinv12*w55+1.1323051948051949e-03*w58*Jinv10-1.1834415584415585e-02*w15*Jinv02-1.1323051948051949e-03*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00-9.8620129870129864e-04*Jinv02*w24-7.8896103896103891e-03*Jinv22*w35-2.3011363636363637e-03*Jinv21*w26-9.1314935064935063e-04*w0*Jinv01-6.9399350649350649e-04*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10+1.9724025974025973e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+9.8620129870129864e-04*w45*Jinv00-1.1834415584415585e-02*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-1.1323051948051949e-03*w11*Jinv22-9.1314935064935063e-04*Jinv11*w1+2.9586038961038961e-03*w13*Jinv11+9.1314935064935063e-04*w28*Jinv11-2.6298701298701297e-03*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+9.8620129870129864e-04*w46*Jinv10-1.1834415584415585e-02*Jinv22*w17+3.9448051948051946e-03*Jinv00*w33-9.8620129870129864e-04*w22*Jinv12+3.2873376623376621e-03*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3+1.9724025974025973e-03*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-1.1323051948051949e-03*Jinv22*w2-2.9586038961038961e-03*Jinv00*w18-1.9724025974025973e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-2.3011363636363637e-03*Jinv01*w24-1.1323051948051949e-03*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+6.5746753246753243e-04*w19*Jinv11-7.8896103896103891e-03*Jinv12*w34-1.1834415584415585e-02*Jinv22*w44+2.3011363636363637e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11+2.3668831168831169e-02*w41*Jinv20+1.9724025974025973e-03*Jinv12*w13-2.9586038961038961e-03*w19*Jinv10+3.9448051948051946e-03*w34*Jinv10+2.3011363636363637e-03*w32*Jinv21+6.5746753246753243e-04*Jinv21*w20-3.6160714285714286e-03*w6*Jinv00+1.3149350649350649e-03*w37*Jinv12-1.9724025974025973e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41+1.6436688311688311e-03*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11-9.8620129870129864e-04*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21+1.9724025974025973e-03*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12-9.8620129870129864e-04*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10+9.1314935064935063e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+2.3668831168831169e-02*w39*Jinv00+1.9724025974025973e-03*Jinv02*w12+6.5746753246753243e-04*Jinv01*w18-1.1323051948051949e-03*Jinv02*w9-6.5746753246753243e-04*w6*Jinv01-3.6160714285714286e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21-1.9724025974025973e-03*Jinv20*w38-9.8620129870129864e-04*Jinv22*w23+1.3149350649350649e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12+1.3149350649350649e-03*Jinv22*w8+1.9724025974025973e-03*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-2.9586038961038961e-03*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7-2.9586038961038961e-03*w21*Jinv01+1.3149350649350649e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-6.5746753246753243e-04*w7*Jinv11+1.6436688311688311e-03*w9*Jinv00-1.9724025974025973e-03*w37*Jinv10+6.5746753246753243e-04*Jinv21*w56-2.1915584415584417e-04*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02+1.9724025974025973e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22-6.5746753246753243e-04*w49*Jinv11+2.9586038961038961e-03*Jinv21*w14+1.6436688311688311e-03*w10*Jinv10-1.1323051948051949e-03*w58*Jinv12+1.9724025974025973e-03*Jinv02*w18+2.9586038961038961e-03*Jinv01*w12-6.5746753246753243e-04*Jinv21*w8+1.1323051948051949e-03*w57*Jinv00+1.1323051948051949e-03*Jinv20*w59-7.8896103896103891e-03*Jinv02*w33-6.9399350649350649e-04*w0*Jinv00+3.9448051948051946e-03*w35*Jinv20-2.9586038961038961e-03*Jinv21*w23+6.5746753246753243e-04*Jinv01*w54+6.5746753246753243e-04*w55*Jinv11-6.9399350649350649e-04*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02+5.9172077922077923e-03*Jinv20*w17+3.2873376623376621e-03*w30*Jinv00+2.3668831168831169e-02*w40*Jinv10-2.9586038961038961e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.9724025974025973e-03*Jinv22*w14+1.3149350649350649e-03*Jinv02*w36-1.1834415584415585e-02*Jinv12*w43+2.3011363636363637e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+1.9724025974025973e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-2.3011363636363637e-03*w25*Jinv11-1.9724025974025973e-03*w55*Jinv10+9.8620129870129864e-04*w47*Jinv20-1.1834415584415585e-02*w16*Jinv12-6.5746753246753243e-04*Jinv21*w50+9.1314935064935063e-04*Jinv01*w27-6.5746753246753243e-04*w48*Jinv01-1.1323051948051949e-03*Jinv12*w1)*D;
  A[40][17]=(0.0)*D;
  A[40][18]=(0.0)*D;
  A[40][19]=(-1.6436688311688311e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.0957792207792207e-03*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20-3.8352272727272725e-04*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-3.6525974025974026e-05*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45-7.3051948051948052e-05*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01-7.3051948051948052e-05*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-3.2873376623376621e-03*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27-3.6525974025974026e-05*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10-2.6298701298701297e-03*w21*Jinv00-3.2873376623376621e-03*Jinv01*w39+1.6436688311688311e-03*Jinv00*w18+1.0957792207792207e-03*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38-3.8352272727272725e-04*Jinv01*w24-1.3149350649350649e-03*w34*Jinv11+1.0957792207792207e-03*w19*Jinv11-3.8352272727272725e-04*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-3.8352272727272725e-04*w4*Jinv11-9.8620129870129868e-03*w41*Jinv20+1.6436688311688311e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+1.0957792207792207e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-8.2183441558441553e-04*w26*Jinv20-6.7573051948051951e-04*w11*Jinv20-1.3149350649350649e-03*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-3.8352272727272725e-04*w5*Jinv21-1.0957792207792208e-04*w49*Jinv10-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-9.8620129870129868e-03*w39*Jinv00+1.0957792207792207e-03*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+1.0957792207792207e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.0957792207792207e-03*Jinv20*w38-3.8352272727272725e-04*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53-2.6298701298701297e-03*w22*Jinv10-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51+1.0957792207792207e-03*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11-6.7573051948051951e-04*w9*Jinv00+1.0957792207792207e-03*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-8.2183441558441553e-04*w24*Jinv00-5.4788961038961042e-05*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-6.7573051948051951e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-5.4788961038961042e-05*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59-7.3051948051948052e-05*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-8.2183441558441553e-04*w25*Jinv10-7.3051948051948052e-05*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-4.9310064935064932e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-9.8620129870129868e-03*w40*Jinv10+1.6436688311688311e-03*w20*Jinv20-3.2873376623376621e-03*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-3.8352272727272725e-04*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20-5.4788961038961042e-05*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01-4.9310064935064932e-04*w10*Jinv11)*D;
  A[40][20]=(0.0)*D;
  A[40][21]=(0.0)*D;
  A[40][22]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-4.3831168831168834e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+5.6615259740259744e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+6.9399350649350649e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17-1.0957792207792207e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-2.7394480519480518e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+5.6615259740259744e-04*w58*Jinv11+5.6615259740259744e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00+7.6704545454545450e-04*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-1.2053571428571428e-03*w19*Jinv11-3.2873376623376621e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-4.3831168831168834e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-8.7662337662337668e-04*w32*Jinv21-1.2053571428571428e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35-8.7662337662337668e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-1.0957792207792207e-03*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-4.3831168831168834e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12-1.2053571428571428e-03*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9-1.2601461038961039e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-1.0957792207792207e-03*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23-3.2873376623376621e-04*w52*Jinv11+7.6704545454545450e-04*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+7.6704545454545450e-04*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-3.2873376623376621e-04*Jinv01*w51+7.6704545454545450e-04*Jinv02*w30+7.6704545454545450e-04*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10-1.2601461038961039e-03*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-6.0267857142857139e-04*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12-1.2601461038961039e-03*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+4.9310064935064932e-04*Jinv21*w23-6.0267857142857139e-04*Jinv01*w54-6.0267857142857139e-04*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+7.6704545454545450e-04*w48*Jinv02+6.9399350649350649e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20-8.7662337662337668e-04*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+6.9399350649350649e-04*w10*Jinv11)*D;
  A[40][23]=(0.0)*D;
  A[40][24]=(0.0)*D;
  A[40][25]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+4.6022727272727274e-03*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59+5.2962662337662338e-04*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+4.6022727272727274e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-7.6704545454545450e-04*w48*Jinv00+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+5.2962662337662338e-04*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+4.6022727272727274e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+5.2962662337662338e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w31*Jinv10+4.2735389610389614e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-1.3149350649350649e-03*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38+1.5888798701298701e-03*Jinv01*w24+6.5746753246753243e-04*w34*Jinv11-1.3697240259740260e-03*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00-3.8352272727272725e-04*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-1.3149350649350649e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-1.3697240259740260e-03*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+2.1367694805194807e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+6.5746753246753243e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.5888798701298701e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21-7.6704545454545450e-04*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-1.3697240259740260e-03*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.5888798701298701e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-7.1225649350649346e-04*w52*Jinv11-3.8352272727272725e-04*Jinv20*w53+4.2735389610389614e-03*w22*Jinv10+1.5888798701298701e-03*w22*Jinv11+1.5888798701298701e-03*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-1.3697240259740260e-03*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-1.3697240259740260e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.5888798701298701e-03*Jinv21*w23-1.3697240259740260e-03*Jinv01*w54-1.3697240259740260e-03*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+5.2962662337662338e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-3.8352272727272725e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+1.5888798701298701e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+5.2962662337662338e-04*w10*Jinv11)*D;
  A[40][26]=(0.0)*D;
  A[40][27]=(0.0)*D;
  A[40][28]=(-1.8262987012987013e-03*Jinv21*w44+2.1915584415584417e-04*Jinv01*w3-4.2613636363636367e-05*Jinv21*w59-4.2613636363636367e-05*Jinv21*w2-1.8262987012987013e-03*w42*Jinv01-1.9480519480519481e-04*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-1.8262987012987013e-03*w43*Jinv11-3.6525974025974025e-03*Jinv21*w41-4.2613636363636367e-05*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-3.6525974025974025e-03*Jinv01*w39+3.4699675324675324e-04*Jinv21*w38+3.2873376623376621e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11+2.9220779220779221e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+2.1915584415584417e-04*w4*Jinv11+4.3831168831168834e-04*w32*Jinv21+2.9220779220779221e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+4.3831168831168834e-04*w31*Jinv11+6.5746753246753243e-04*w46*Jinv11+2.1915584415584417e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33+2.9220779220779221e-04*Jinv01*w18+3.4699675324675324e-04*w37*Jinv11+3.4699675324675324e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+2.1915584415584417e-04*w52*Jinv11+6.5746753246753243e-04*w22*Jinv11+6.5746753246753243e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+3.4699675324675324e-04*w36*Jinv01+3.4699675324675324e-04*w7*Jinv11-3.2873376623376621e-04*Jinv21*w56+4.3831168831168834e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-3.2873376623376621e-04*Jinv01*w12+3.4699675324675324e-04*Jinv21*w8+6.5746753246753243e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-1.9480519480519481e-04*Jinv01*w9-3.6525974025974025e-03*w40*Jinv11+4.3831168831168834e-04*Jinv01*w30+3.2873376623376621e-04*w25*Jinv11+4.3831168831168834e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+4.3831168831168834e-04*w48*Jinv01-1.9480519480519481e-04*w10*Jinv11)*D;
  A[40][29]=(0.0)*D;
  A[40][30]=(0.0)*D;
  A[40][31]=(-3.1777597402597403e-03*Jinv00*w12-1.0592532467532468e-03*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+4.3831168831168834e-04*Jinv22*w53+2.5202922077922079e-03*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-3.1777597402597403e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-4.7666396103896100e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3+2.7394480519480520e-03*Jinv20*w23+1.2601461038961039e-03*Jinv22*w38+1.6436688311688311e-03*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+3.2873376623376621e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-9.6793831168831166e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45+8.7662337662337668e-04*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-6.9399350649350649e-04*w10*Jinv12+6.5746753246753243e-04*w48*Jinv00+3.2873376623376621e-04*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-1.3806818181818182e-02*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-1.0592532467532468e-03*Jinv00*w27-6.9399350649350649e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-3.1777597402597403e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-1.0592532467532468e-03*w28*Jinv11+6.5746753246753243e-04*Jinv20*w50-1.0592532467532468e-03*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-6.9034090909090911e-03*Jinv00*w33+6.0267857142857139e-04*w22*Jinv12-4.7666396103896100e-03*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+2.7394480519480520e-03*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22-1.3806818181818182e-02*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+8.7662337662337668e-04*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-6.9034090909090911e-03*w34*Jinv11+8.7662337662337668e-04*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+8.2183441558441553e-04*w51*Jinv00-5.9172077922077923e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-1.3806818181818182e-02*w41*Jinv20+1.0957792207792207e-03*Jinv12*w13+8.7662337662337668e-04*w19*Jinv10-6.9034090909090911e-03*w34*Jinv10-4.7666396103896100e-03*w32*Jinv21+8.7662337662337668e-04*Jinv21*w20+2.5202922077922079e-03*w6*Jinv00+1.2601461038961039e-03*w37*Jinv12+1.6436688311688311e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-9.6793831168831166e-04*w11*Jinv20+4.3831168831168834e-04*w52*Jinv12-6.9034090909090911e-03*Jinv21*w35-4.7666396103896100e-03*w31*Jinv11+3.2873376623376621e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11-3.1777597402597403e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+8.7662337662337668e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+6.0267857142857139e-04*w21*Jinv02+6.5746753246753243e-04*w49*Jinv10-1.0592532467532468e-03*Jinv21*w29-6.9034090909090911e-03*Jinv01*w33-1.3806818181818182e-02*w39*Jinv00+1.0957792207792207e-03*Jinv02*w12+8.7662337662337668e-04*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-6.9399350649350649e-04*Jinv02*w9+2.5202922077922079e-03*w6*Jinv01+2.5202922077922079e-03*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+6.0267857142857139e-04*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+1.3697240259740260e-03*w46*Jinv12+8.2183441558441553e-04*Jinv20*w53+1.2053571428571428e-03*Jinv22*w8+2.7394480519480520e-03*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+2.7394480519480520e-03*w22*Jinv11+1.2053571428571428e-03*Jinv12*w7+2.7394480519480520e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01+8.2183441558441553e-04*w52*Jinv10+2.5202922077922079e-03*w7*Jinv11-9.6793831168831166e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+1.6436688311688311e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-5.6615259740259744e-04*Jinv02*w27+1.2053571428571428e-03*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22+6.5746753246753243e-04*w49*Jinv11-3.1777597402597403e-03*Jinv21*w14-9.6793831168831166e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-3.1777597402597403e-03*Jinv01*w12+2.5202922077922079e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-6.9034090909090911e-03*w35*Jinv20+2.7394480519480520e-03*Jinv21*w23+1.6436688311688311e-03*Jinv01*w54+1.6436688311688311e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-9.6793831168831166e-04*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-4.7666396103896100e-03*w30*Jinv00-1.3806818181818182e-02*w40*Jinv10+8.7662337662337668e-04*w20*Jinv20-5.6615259740259744e-04*Jinv22*w29+1.0957792207792207e-03*Jinv22*w14-1.3806818181818182e-02*w40*Jinv11+1.2601461038961039e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20-4.7666396103896100e-03*Jinv01*w30+4.3831168831168834e-04*Jinv02*w51+8.7662337662337668e-04*Jinv22*w56-5.6615259740259744e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+1.6436688311688311e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+6.5746753246753243e-04*Jinv21*w50-1.0592532467532468e-03*Jinv01*w27+6.5746753246753243e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-9.6793831168831166e-04*w10*Jinv11)*D;
  A[40][32]=(0.0)*D;
  A[40][33]=(0.0)*D;
  A[40][34]=(3.2873376623376621e-03*Jinv00*w12+1.1323051948051949e-03*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53-3.6160714285714286e-03*Jinv20*w8+9.1314935064935063e-04*w57*Jinv02+3.2873376623376621e-03*w13*Jinv10-1.1834415584415585e-02*Jinv21*w44+2.3011363636363637e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23+6.5746753246753243e-04*Jinv22*w38+9.8620129870129864e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-9.1314935064935063e-04*w0*Jinv02-1.1323051948051949e-03*Jinv21*w2-1.1834415584415585e-02*w42*Jinv01-1.1323051948051949e-03*w11*Jinv21-7.8896103896103891e-03*w16*Jinv11-7.8896103896103891e-03*Jinv21*w17+1.9724025974025973e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02+1.9724025974025973e-03*w48*Jinv00-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-1.1323051948051949e-03*w0*Jinv01-6.9399350649350649e-04*w1*Jinv10+3.9448051948051946e-03*w16*Jinv10-1.1834415584415585e-02*w43*Jinv11-1.9724025974025973e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+1.1323051948051949e-03*Jinv00*w27-1.1323051948051949e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-7.8896103896103891e-03*w15*Jinv01-1.1323051948051949e-03*w28*Jinv11+1.9724025974025973e-03*Jinv20*w50+1.1323051948051949e-03*Jinv20*w29-1.9724025974025973e-03*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33-6.5746753246753243e-04*w22*Jinv12+4.9310064935064934e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3-2.6298701298701297e-03*w21*Jinv00+2.9586038961038961e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-9.1314935064935063e-04*Jinv22*w2-1.9724025974025973e-03*Jinv00*w18-2.9586038961038961e-03*w36*Jinv00+1.9724025974025973e-03*Jinv21*w38+6.5746753246753243e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24+9.1314935064935063e-04*Jinv22*w59-1.1834415584415585e-02*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11-9.8620129870129864e-04*Jinv21*w53-3.9448051948051946e-03*Jinv22*w44+2.3011363636363637e-03*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+2.3668831168831169e-02*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13-1.9724025974025973e-03*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10+1.9724025974025973e-03*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20-3.6160714285714286e-03*w6*Jinv00+6.5746753246753243e-04*w37*Jinv12+9.8620129870129864e-04*Jinv20*w56+1.6436688311688311e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12-1.1834415584415585e-02*Jinv21*w35+1.9724025974025973e-03*w31*Jinv11+1.9724025974025973e-03*w46*Jinv11+3.2873376623376621e-03*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12-6.5746753246753243e-04*w21*Jinv02+1.9724025974025973e-03*w49*Jinv10-1.1323051948051949e-03*Jinv21*w29-1.1834415584415585e-02*Jinv01*w33+2.3668831168831169e-02*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18+1.9724025974025973e-03*w37*Jinv11+1.3149350649350649e-03*w6*Jinv01-3.6160714285714286e-03*w7*Jinv10+1.9724025974025973e-03*w47*Jinv21-2.9586038961038961e-03*Jinv20*w38-6.5746753246753243e-04*Jinv22*w23-9.8620129870129864e-04*w52*Jinv11+2.9586038961038961e-03*w31*Jinv12+6.5746753246753243e-04*w46*Jinv12-6.5746753246753243e-04*Jinv22*w8-2.6298701298701297e-03*w22*Jinv10-2.9586038961038961e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11-6.5746753246753243e-04*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01-9.8620129870129864e-04*Jinv01*w51+2.9586038961038961e-03*Jinv02*w30-2.9586038961038961e-03*Jinv12*w49+1.9724025974025973e-03*w36*Jinv01+1.3149350649350649e-03*w7*Jinv11+1.6436688311688311e-03*w9*Jinv00-2.9586038961038961e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56-6.5746753246753243e-04*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5+6.5746753246753243e-04*w47*Jinv22-9.8620129870129864e-04*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+1.6436688311688311e-03*w10*Jinv10+9.1314935064935063e-04*w58*Jinv12+1.3149350649350649e-03*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-6.9399350649350649e-04*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11-6.9399350649350649e-04*Jinv20*w2+3.9448051948051946e-03*w15*Jinv00-2.9586038961038961e-03*w48*Jinv02-1.1323051948051949e-03*Jinv01*w9+3.9448051948051946e-03*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00+2.3668831168831169e-02*w40*Jinv10-1.9724025974025973e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11+6.5746753246753243e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43+2.3011363636363637e-03*w5*Jinv20+1.9724025974025973e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56+1.3149350649350649e-03*w25*Jinv11+9.8620129870129864e-04*w55*Jinv10-1.9724025974025973e-03*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-9.8620129870129864e-04*Jinv21*w50-1.1323051948051949e-03*Jinv01*w27-9.8620129870129864e-04*w48*Jinv01-9.1314935064935063e-04*Jinv12*w1-1.1323051948051949e-03*w10*Jinv11)*D;
  A[40][35]=(0.0)*D;
  A[40][36]=(0.0)*D;
  A[40][37]=(-1.0957792207792208e-04*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-3.2873376623376621e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8-3.6525974025974026e-05*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10-3.2873376623376621e-03*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23+1.0957792207792207e-03*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54-3.8352272727272725e-04*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10-5.4788961038961042e-05*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-7.3051948051948052e-05*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-2.6298701298701297e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-3.8352272727272725e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-3.6525974025974026e-05*Jinv22*w2+1.0957792207792207e-03*Jinv00*w18+1.6436688311688311e-03*w36*Jinv00-5.4788961038961042e-05*Jinv02*w45-3.6525974025974026e-05*Jinv22*w59-8.2183441558441553e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10-9.8620129870129868e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+1.0957792207792207e-03*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10+1.0957792207792207e-03*w6*Jinv00+1.0957792207792207e-03*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-3.2873376623376621e-03*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20-6.7573051948051951e-04*w11*Jinv20-3.8352272727272725e-04*w52*Jinv12-3.8352272727272725e-04*Jinv22*w26-1.0957792207792208e-04*Jinv20*w14-5.4788961038961042e-05*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10-9.8620129870129868e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-4.9310064935064932e-04*Jinv02*w9+1.0957792207792207e-03*w7*Jinv10+1.6436688311688311e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23-5.4788961038961042e-05*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-8.2183441558441553e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*Jinv12*w7-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-8.2183441558441553e-04*w52*Jinv10-6.7573051948051951e-04*w9*Jinv00+1.6436688311688311e-03*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-6.7573051948051951e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-3.6525974025974026e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-7.3051948051948052e-05*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10-7.3051948051948052e-05*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+6.5746753246753243e-04*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-9.8620129870129868e-03*w40*Jinv10+1.0957792207792207e-03*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+1.0957792207792207e-03*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20-3.8352272727272725e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28-2.6298701298701297e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-3.6525974025974026e-05*Jinv12*w1)*D;
  A[40][38]=(0.0)*D;
  A[40][39]=(0.0)*D;
  A[40][40]=(3.9448051948051946e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53+3.2873376623376621e-03*Jinv20*w8+1.8262987012987013e-03*w57*Jinv02+3.9448051948051946e-03*w13*Jinv10-5.9172077922077923e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+3.9448051948051946e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3-1.9724025974025973e-03*Jinv20*w23+1.6436688311688311e-03*Jinv22*w38-2.9586038961038961e-03*Jinv00*w54-1.8262987012987013e-03*w0*Jinv02-1.8262987012987013e-03*Jinv21*w2-1.7751623376623376e-02*w43*Jinv10-5.9172077922077923e-03*w42*Jinv01-6.9034090909090911e-03*Jinv01*w45-6.9034090909090911e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10+5.9172077922077923e-03*w15*Jinv02-1.9724025974025973e-03*w48*Jinv00-2.3011363636363637e-03*Jinv21*w26-1.8262987012987013e-03*w0*Jinv01-2.2646103896103898e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10-5.9172077922077923e-03*w43*Jinv11-2.9586038961038961e-03*w45*Jinv00-5.9172077922077923e-03*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.8262987012987013e-03*Jinv11*w1+5.9172077922077923e-03*w13*Jinv11+1.8262987012987013e-03*w28*Jinv11-1.9724025974025973e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29-2.9586038961038961e-03*w46*Jinv10+5.9172077922077923e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12+3.9448051948051946e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3-1.9724025974025973e-03*w21*Jinv00+5.9172077922077923e-03*w32*Jinv22-1.8262987012987013e-03*Jinv22*w2+4.9310064935064934e-03*Jinv00*w18+4.9310064935064934e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-2.3011363636363637e-03*Jinv01*w24+1.8262987012987013e-03*Jinv22*w59+5.9172077922077923e-03*w34*Jinv11+1.6436688311688311e-03*w19*Jinv11-9.8620129870129864e-04*w51*Jinv00-5.9172077922077923e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11-3.5503246753246752e-02*w41*Jinv20+6.9034090909090911e-03*Jinv12*w13+4.9310064935064934e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+6.9034090909090911e-03*w32*Jinv21+1.6436688311688311e-03*Jinv21*w20+3.2873376623376621e-03*w6*Jinv00+1.6436688311688311e-03*w37*Jinv12-2.9586038961038961e-03*Jinv20*w56-9.8620129870129864e-04*w26*Jinv20-2.7759740259740259e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12+5.9172077922077923e-03*Jinv21*w35+6.9034090909090911e-03*w31*Jinv11-6.9034090909090911e-03*w46*Jinv11+3.9448051948051946e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21-6.9034090909090911e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02-1.9724025974025973e-03*w49*Jinv10+1.8262987012987013e-03*Jinv21*w29+5.9172077922077923e-03*Jinv01*w33-3.5503246753246752e-02*w39*Jinv00+6.9034090909090911e-03*Jinv02*w12+1.6436688311688311e-03*Jinv01*w18-1.6436688311688311e-03*w6*Jinv01+3.2873376623376621e-03*w7*Jinv10-6.9034090909090911e-03*w47*Jinv21+4.9310064935064934e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+5.9172077922077923e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53-1.6436688311688311e-03*Jinv22*w8-1.9724025974025973e-03*w22*Jinv10-5.9172077922077923e-03*Jinv22*w50-5.9172077922077923e-03*w22*Jinv11-1.6436688311688311e-03*Jinv12*w7-5.9172077922077923e-03*w21*Jinv01+5.9172077922077923e-03*Jinv02*w30-5.9172077922077923e-03*Jinv12*w49-9.8620129870129864e-04*w52*Jinv10-1.6436688311688311e-03*w7*Jinv11-2.7759740259740259e-03*w9*Jinv00+4.9310064935064934e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00-1.6436688311688311e-03*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11+5.9172077922077923e-03*Jinv21*w14-2.7759740259740259e-03*w10*Jinv10-1.7751623376623376e-02*Jinv20*w44+1.8262987012987013e-03*w58*Jinv12+5.9172077922077923e-03*Jinv01*w12-1.6436688311688311e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-2.2646103896103898e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-5.9172077922077923e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-2.2646103896103898e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-5.9172077922077923e-03*w48*Jinv02-1.1834415584415585e-02*Jinv20*w17+3.9448051948051946e-03*w30*Jinv00-3.5503246753246752e-02*w40*Jinv10+4.9310064935064934e-03*w20*Jinv20+6.9034090909090911e-03*Jinv22*w14+1.6436688311688311e-03*Jinv02*w36-5.9172077922077923e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+6.9034090909090911e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-6.9034090909090911e-03*Jinv22*w56-2.3011363636363637e-03*w25*Jinv11-2.9586038961038961e-03*w55*Jinv10-1.7751623376623376e-02*w42*Jinv00-2.9586038961038961e-03*w47*Jinv20+5.9172077922077923e-03*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+1.8262987012987013e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01-1.8262987012987013e-03*Jinv12*w1)*D;
  A[40][41]=(0.0)*D;
  A[40][42]=(0.0)*D;
  A[40][43]=(-3.9448051948051946e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53-3.2873376623376621e-03*Jinv20*w8+1.1323051948051949e-03*w57*Jinv02-3.9448051948051946e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+1.1834415584415585e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-3.9448051948051946e-03*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+1.9724025974025973e-03*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38+2.9586038961038961e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+9.8620129870129864e-04*w25*Jinv12+1.1323051948051949e-03*w0*Jinv02+1.1323051948051949e-03*Jinv21*w2+1.7751623376623376e-02*w43*Jinv10+1.1834415584415585e-02*w42*Jinv01+1.1323051948051949e-03*w11*Jinv21+7.8896103896103891e-03*w16*Jinv11+7.8896103896103891e-03*Jinv21*w17-1.9724025974025973e-03*Jinv01*w45-1.9724025974025973e-03*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+1.1834415584415585e-02*w15*Jinv02+1.1323051948051949e-03*w10*Jinv12+1.9724025974025973e-03*w48*Jinv00+9.8620129870129864e-04*Jinv02*w24+7.8896103896103891e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+1.1323051948051949e-03*w0*Jinv01+2.2646103896103898e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10-1.9724025974025973e-03*Jinv22*w20+1.1834415584415585e-02*w43*Jinv11+2.9586038961038961e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+1.1834415584415585e-02*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+1.1323051948051949e-03*w11*Jinv22+1.1323051948051949e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+7.8896103896103891e-03*w15*Jinv01+1.1323051948051949e-03*w28*Jinv11+1.9724025974025973e-03*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29+2.9586038961038961e-03*w46*Jinv10+1.1834415584415585e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12-3.9448051948051946e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3+1.9724025974025973e-03*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+1.1323051948051949e-03*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-1.9724025974025973e-03*Jinv21*w38+9.8620129870129864e-04*Jinv02*w45-1.3149350649350649e-03*Jinv01*w24+1.1323051948051949e-03*Jinv22*w59+1.1834415584415585e-02*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11+9.8620129870129864e-04*Jinv21*w53+9.8620129870129864e-04*w51*Jinv00+7.8896103896103891e-03*Jinv12*w34+1.1834415584415585e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+3.5503246753246752e-02*w41*Jinv20-1.9724025974025973e-03*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-1.9724025974025973e-03*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-3.2873376623376621e-03*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12+2.9586038961038961e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+9.8620129870129864e-04*w26*Jinv20+2.7759740259740259e-03*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+1.1834415584415585e-02*Jinv21*w35-1.9724025974025973e-03*w31*Jinv11+9.8620129870129864e-04*Jinv22*w26-1.9724025974025973e-03*w46*Jinv11-3.9448051948051946e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-1.9724025974025973e-03*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02+1.9724025974025973e-03*w49*Jinv10+1.1323051948051949e-03*Jinv21*w29+1.1834415584415585e-02*Jinv01*w33+3.5503246753246752e-02*w39*Jinv00-1.9724025974025973e-03*Jinv02*w12-1.3149350649350649e-03*Jinv01*w18-1.9724025974025973e-03*w37*Jinv11+1.1323051948051949e-03*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01-3.2873376623376621e-03*w7*Jinv10-1.9724025974025973e-03*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+9.8620129870129864e-04*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+9.8620129870129864e-04*w46*Jinv12+9.8620129870129864e-04*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8+1.9724025974025973e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01+9.8620129870129864e-04*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49-1.9724025974025973e-03*w36*Jinv01+9.8620129870129864e-04*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11+2.7759740259740259e-03*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10+9.8620129870129864e-04*Jinv21*w56+9.8620129870129864e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-1.3149350649350649e-03*w6*Jinv02-1.9724025974025973e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5+9.8620129870129864e-04*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+2.7759740259740259e-03*w10*Jinv10+1.7751623376623376e-02*Jinv20*w44+1.1323051948051949e-03*w58*Jinv12-1.9724025974025973e-03*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+7.8896103896103891e-03*Jinv02*w33+2.2646103896103898e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+9.8620129870129864e-04*Jinv01*w54+9.8620129870129864e-04*w55*Jinv11+9.8620129870129864e-04*w25*Jinv10+2.2646103896103898e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+1.1323051948051949e-03*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-3.9448051948051946e-03*w30*Jinv00+3.5503246753246752e-02*w40*Jinv10-4.9310064935064934e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29-1.9724025974025973e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11-1.3149350649350649e-03*Jinv02*w36+1.1834415584415585e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-1.9724025974025973e-03*Jinv01*w30-1.3149350649350649e-03*Jinv02*w51-1.9724025974025973e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-1.3149350649350649e-03*w25*Jinv11+2.9586038961038961e-03*w55*Jinv10+1.7751623376623376e-02*w42*Jinv00+2.9586038961038961e-03*w47*Jinv20+1.1834415584415585e-02*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+1.1323051948051949e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01+1.1323051948051949e-03*Jinv12*w1+1.1323051948051949e-03*w10*Jinv11)*D;
  A[40][44]=(0.0)*D;
  A[40][45]=(0.0)*D;
  A[40][46]=(1.9724025974025973e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+4.9310064935064932e-04*w57*Jinv02+1.9724025974025973e-03*Jinv02*w39+6.9034090909090911e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-5.4788961038961042e-05*Jinv22*w38+4.3831168831168834e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+1.0592532467532468e-03*Jinv21*w2+6.9034090909090911e-03*w42*Jinv01+9.6793831168831166e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+4.7666396103896100e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+2.1367694805194807e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+1.0592532467532468e-03*w0*Jinv01-1.3149350649350649e-03*Jinv22*w20+6.9034090909090911e-03*w43*Jinv11+1.3806818181818182e-02*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02+2.7394480519480518e-04*w11*Jinv22+1.0592532467532468e-03*Jinv11*w1-2.7394480519480520e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17+4.2735389610389614e-03*w22*Jinv12-3.8352272727272725e-04*Jinv02*w3-7.6704545454545450e-04*w32*Jinv22+1.3806818181818182e-02*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-1.3149350649350649e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+1.5888798701298701e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.5202922077922079e-03*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-2.5202922077922079e-03*Jinv21*w20-5.4788961038961042e-05*w37*Jinv12+1.9724025974025973e-03*Jinv22*w41-3.8352272727272725e-04*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+4.7666396103896100e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-2.5202922077922079e-03*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9-8.7662337662337668e-04*w6*Jinv01+4.7666396103896100e-03*w47*Jinv21+4.2735389610389614e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11-7.6704545454545450e-04*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8-7.6704545454545450e-04*Jinv22*w50+3.1777597402597403e-03*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7+3.1777597402597403e-03*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51-7.6704545454545450e-04*Jinv02*w30-7.6704545454545450e-04*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-8.7662337662337668e-04*w7*Jinv11-6.5746753246753243e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-1.6436688311688311e-03*w49*Jinv11-2.7394480519480520e-03*Jinv21*w14+4.9310064935064932e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18-2.7394480519480520e-03*Jinv01*w12-8.7662337662337668e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+3.1777597402597403e-03*Jinv21*w23-6.5746753246753243e-04*Jinv01*w54-6.5746753246753243e-04*w55*Jinv11-7.6704545454545450e-04*w48*Jinv02+9.6793831168831166e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+1.3806818181818182e-02*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30-3.8352272727272725e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.5888798701298701e-03*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12-1.6436688311688311e-03*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-1.6436688311688311e-03*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+9.6793831168831166e-04*w10*Jinv11)*D;
  A[40][47]=(0.0)*D;
  A[40][48]=(0.0)*D;
  A[40][49]=(7.6704545454545450e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23-1.2053571428571428e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-3.2873376623376621e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-1.0957792207792207e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+6.9399350649350649e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00-3.2873376623376621e-04*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27+6.9399350649350649e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+7.6704545454545450e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-4.3831168831168834e-04*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38-6.0267857142857139e-04*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-8.7662337662337668e-04*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-1.2053571428571428e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11-3.2873376623376621e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21-1.0957792207792207e-03*Jinv02*w54-4.3831168831168834e-04*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-8.7662337662337668e-04*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11+6.9399350649350649e-04*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12-6.0267857142857139e-04*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53-1.2601461038961039e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50+7.6704545454545450e-04*w22*Jinv11-1.2601461038961039e-03*Jinv12*w7+7.6704545454545450e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00+5.6615259740259744e-04*Jinv02*w27-1.2601461038961039e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12-4.3831168831168834e-04*Jinv22*w5-6.0267857142857139e-04*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+7.6704545454545450e-04*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18+7.6704545454545450e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+5.9172077922077923e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+7.6704545454545450e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+4.9310064935064932e-04*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.6615259740259744e-04*Jinv22*w29-8.7662337662337668e-04*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-1.2053571428571428e-03*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-1.0957792207792207e-03*Jinv22*w56+5.6615259740259744e-04*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[40][50]=(0.0)*D;
  A[40][51]=(0.0)*D;
  A[40][52]=(-7.6704545454545450e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-7.6704545454545450e-04*Jinv20*w23-1.3697240259740260e-03*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-7.1225649350649346e-04*w25*Jinv12+5.2962662337662338e-04*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10+1.5888798701298701e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+4.6022727272727274e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22+4.2735389610389614e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22+5.2962662337662338e-04*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00-1.3697240259740260e-03*Jinv02*w45-3.8352272727272725e-04*Jinv22*w59+2.1367694805194807e-03*w51*Jinv00+4.6022727272727274e-03*Jinv12*w34+4.6022727272727274e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10+1.9724025974025973e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-1.3697240259740260e-03*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+1.5888798701298701e-03*w52*Jinv12-7.1225649350649346e-04*Jinv22*w26-7.6704545454545450e-04*Jinv20*w14+1.5888798701298701e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+1.9724025974025973e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+5.2962662337662338e-04*Jinv02*w9-5.4788961038961042e-05*w7*Jinv10-1.3149350649350649e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-1.3697240259740260e-03*w31*Jinv12-1.3697240259740260e-03*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.5888798701298701e-03*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7-1.3697240259740260e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49+2.1367694805194807e-03*w52*Jinv10+2.7394480519480518e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10-3.8352272727272725e-04*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-1.3697240259740260e-03*w47*Jinv22+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-3.8352272727272725e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+4.6022727272727274e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-3.8352272727272725e-04*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.5888798701298701e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-1.3697240259740260e-03*Jinv02*w36+4.6022727272727274e-03*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20+1.5888798701298701e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+5.2962662337662338e-04*Jinv12*w28+4.2735389610389614e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+5.2962662337662338e-04*Jinv12*w1)*D;
  A[40][53]=(0.0)*D;
  A[40][54]=(0.0)*D;
  A[40][55]=(1.3806818181818182e-02*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-7.6704545454545450e-04*w57*Jinv02+1.3806818181818182e-02*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*Jinv01*w3-2.5202922077922079e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12+1.0592532467532468e-03*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01+2.7394480519480518e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+4.7666396103896100e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+9.6793831168831166e-04*w10*Jinv12-8.2183441558441553e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-1.3149350649350649e-03*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+1.9724025974025973e-03*Jinv21*w41+6.9034090909090911e-03*w42*Jinv02+9.6793831168831166e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-7.6704545454545450e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17-1.6436688311688311e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-2.7394480519480520e-03*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+1.0592532467532468e-03*Jinv22*w2-1.3149350649350649e-03*Jinv21*w38-6.5746753246753243e-04*Jinv02*w45-3.8352272727272725e-04*Jinv01*w24-7.6704545454545450e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+6.9034090909090911e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-2.5202922077922079e-03*w37*Jinv12+1.3806818181818182e-02*Jinv22*w41+1.5888798701298701e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-3.8352272727272725e-04*w5*Jinv21+4.7666396103896100e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.6436688311688311e-03*w21*Jinv02+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+9.6793831168831166e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-2.1367694805194807e-03*w47*Jinv21-1.6436688311688311e-03*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-2.7394480519480520e-03*w31*Jinv12-6.5746753246753243e-04*w46*Jinv12-8.7662337662337668e-04*Jinv22*w8+3.1777597402597403e-03*Jinv22*w50-7.6704545454545450e-04*w22*Jinv11-8.7662337662337668e-04*Jinv12*w7-7.6704545454545450e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-2.7394480519480520e-03*Jinv02*w30+3.1777597402597403e-03*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11+4.2735389610389614e-03*Jinv21*w56+4.3831168831168834e-04*Jinv02*w27-8.7662337662337668e-04*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-6.5746753246753243e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-7.6704545454545450e-04*Jinv21*w14-7.6704545454545450e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18-7.6704545454545450e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-7.6704545454545450e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11+3.1777597402597403e-03*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+4.3831168831168834e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-2.5202922077922079e-03*Jinv02*w36+6.9034090909090911e-03*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+1.5888798701298701e-03*Jinv02*w51+4.7666396103896100e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-3.8352272727272725e-04*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01+1.0592532467532468e-03*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[40][56]=(0.0)*D;
  A[40][57]=(0.0)*D;
  A[40][58]=(-3.6525974025974025e-03*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-3.6525974025974025e-03*Jinv02*w39+2.9220779220779221e-04*Jinv22*w38+2.1915584415584417e-04*w25*Jinv12-4.2613636363636367e-05*w0*Jinv02+6.5746753246753243e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-1.9480519480519481e-04*w10*Jinv12+2.1915584415584417e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35+3.4699675324675324e-04*Jinv22*w20-1.8262987012987013e-03*w42*Jinv02-1.9480519480519481e-04*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+4.3831168831168834e-04*w22*Jinv12+2.1915584415584417e-04*Jinv02*w3-3.2873376623376621e-04*w32*Jinv22-4.2613636363636367e-05*Jinv22*w2-3.2873376623376621e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-1.8262987012987013e-03*Jinv12*w34-1.8262987012987013e-03*Jinv22*w44+4.3831168831168834e-04*Jinv12*w13+2.9220779220779221e-04*w37*Jinv12-3.6525974025974025e-03*Jinv22*w41+3.2873376623376621e-04*w52*Jinv12+2.1915584415584417e-04*Jinv22*w26+6.5746753246753243e-04*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12+4.3831168831168834e-04*w21*Jinv02+4.3831168831168834e-04*Jinv02*w12-1.9480519480519481e-04*Jinv02*w9+4.3831168831168834e-04*Jinv22*w23-3.2873376623376621e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+3.4699675324675324e-04*Jinv22*w8+6.5746753246753243e-04*Jinv22*w50+3.4699675324675324e-04*Jinv12*w7-3.2873376623376621e-04*Jinv02*w30+6.5746753246753243e-04*Jinv12*w49-4.2613636363636367e-05*Jinv02*w27+3.4699675324675324e-04*w6*Jinv02+3.4699675324675324e-04*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+3.4699675324675324e-04*Jinv02*w18-1.8262987012987013e-03*Jinv02*w33+6.5746753246753243e-04*w48*Jinv02-4.2613636363636367e-05*Jinv22*w29+4.3831168831168834e-04*Jinv22*w14+2.9220779220779221e-04*Jinv02*w36-1.8262987012987013e-03*Jinv12*w43+3.2873376623376621e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-4.2613636363636367e-05*Jinv12*w1)*D;
  A[40][59]=(0.0)*D;
  A[41][0]=(0.0)*D;
  A[41][1]=(0.0)*D;
  A[41][2]=(-6.5746753246753243e-04*Jinv00*w12+4.2613636363636367e-05*w28*Jinv10+3.6525974025974025e-03*w40*Jinv12-2.1915584415584417e-04*Jinv22*w53-2.9220779220779221e-04*Jinv20*w8+4.2613636363636367e-05*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+3.6525974025974025e-03*Jinv02*w39+5.1136363636363640e-04*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23-3.4699675324675324e-04*Jinv22*w38-4.3831168831168834e-04*Jinv00*w54+4.2613636363636367e-05*Jinv21*w59-2.1915584415584417e-04*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-1.2175324675324675e-04*Jinv21*w2+5.1136363636363640e-04*w43*Jinv10+5.1136363636363640e-04*w42*Jinv01+1.9480519480519481e-04*w11*Jinv21+1.8262987012987013e-03*w16*Jinv11+1.8262987012987013e-03*Jinv21*w17-4.3831168831168834e-04*Jinv01*w45-4.3831168831168834e-04*Jinv12*w55+4.2613636363636367e-05*w58*Jinv10+1.8262987012987013e-03*w15*Jinv02+1.9480519480519481e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35+4.2613636363636367e-05*w58*Jinv11+4.2613636363636367e-05*w57*Jinv01-2.1915584415584417e-04*Jinv21*w26-1.2175324675324675e-04*w0*Jinv01-1.2175324675324675e-04*w1*Jinv10+1.8262987012987013e-03*w16*Jinv10-3.4699675324675324e-04*Jinv22*w20+5.1136363636363640e-04*w43*Jinv11-4.3831168831168834e-04*w45*Jinv00+3.6525974025974025e-03*Jinv21*w41+5.1136363636363640e-04*w42*Jinv02+4.2613636363636367e-05*Jinv00*w27+1.9480519480519481e-04*w11*Jinv22-1.2175324675324675e-04*Jinv11*w1-6.5746753246753243e-04*w13*Jinv11+1.8262987012987013e-03*w15*Jinv01+4.2613636363636367e-05*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+4.2613636363636367e-05*Jinv20*w29-4.3831168831168834e-04*w46*Jinv10+1.8262987012987013e-03*Jinv22*w17+1.8262987012987013e-03*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-6.5746753246753243e-04*w32*Jinv22+3.6525974025974025e-03*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-3.4699675324675324e-04*Jinv00*w18-3.4699675324675324e-04*w36*Jinv00-3.4699675324675324e-04*Jinv21*w38-4.3831168831168834e-04*Jinv02*w45-2.1915584415584417e-04*Jinv01*w24+4.2613636363636367e-05*Jinv22*w59+1.8262987012987013e-03*w34*Jinv11-3.4699675324675324e-04*w19*Jinv11-2.1915584415584417e-04*Jinv21*w53-2.1915584415584417e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+5.1136363636363640e-04*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11+3.6525974025974025e-03*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13-3.4699675324675324e-04*w19*Jinv10+1.8262987012987013e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21-3.4699675324675324e-04*Jinv21*w20-2.9220779220779221e-04*w6*Jinv00-3.4699675324675324e-04*w37*Jinv12-4.3831168831168834e-04*Jinv20*w56+3.6525974025974025e-03*Jinv22*w41-2.1915584415584417e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20-2.1915584415584417e-04*w52*Jinv12+1.8262987012987013e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26-4.3831168831168834e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21-4.3831168831168834e-04*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+3.2873376623376621e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+1.8262987012987013e-03*Jinv01*w33+3.6525974025974025e-03*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12-3.4699675324675324e-04*Jinv01*w18-3.4699675324675324e-04*w37*Jinv11+1.9480519480519481e-04*Jinv02*w9-2.9220779220779221e-04*w6*Jinv01-2.9220779220779221e-04*w7*Jinv10-4.3831168831168834e-04*w47*Jinv21-3.4699675324675324e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-2.1915584415584417e-04*w52*Jinv11-6.5746753246753243e-04*w31*Jinv12-4.3831168831168834e-04*w46*Jinv12-2.1915584415584417e-04*Jinv20*w53-2.9220779220779221e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+3.2873376623376621e-04*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11-2.9220779220779221e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01-2.1915584415584417e-04*Jinv01*w51-6.5746753246753243e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-3.4699675324675324e-04*w36*Jinv01-2.1915584415584417e-04*w52*Jinv10-2.9220779220779221e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-3.4699675324675324e-04*w37*Jinv10-4.3831168831168834e-04*Jinv21*w56-2.1915584415584417e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27-2.9220779220779221e-04*w6*Jinv02-3.4699675324675324e-04*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5-4.3831168831168834e-04*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-6.5746753246753243e-04*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10+5.1136363636363640e-04*Jinv20*w44+4.2613636363636367e-05*w58*Jinv12-3.4699675324675324e-04*Jinv02*w18-6.5746753246753243e-04*Jinv01*w12-2.9220779220779221e-04*Jinv21*w8+4.2613636363636367e-05*w57*Jinv00+4.2613636363636367e-05*Jinv20*w59+1.8262987012987013e-03*Jinv02*w33-1.2175324675324675e-04*w0*Jinv00+1.8262987012987013e-03*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-4.3831168831168834e-04*Jinv01*w54-4.3831168831168834e-04*w55*Jinv11-2.1915584415584417e-04*w25*Jinv10-1.2175324675324675e-04*Jinv20*w2+1.8262987012987013e-03*w15*Jinv00+3.2873376623376621e-04*w48*Jinv02+1.9480519480519481e-04*Jinv01*w9+1.8262987012987013e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+3.6525974025974025e-03*w40*Jinv10-3.4699675324675324e-04*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14+3.6525974025974025e-03*w40*Jinv11-3.4699675324675324e-04*Jinv02*w36+5.1136363636363640e-04*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30-2.1915584415584417e-04*Jinv02*w51-4.3831168831168834e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-2.1915584415584417e-04*w25*Jinv11-4.3831168831168834e-04*w55*Jinv10+5.1136363636363640e-04*w42*Jinv00-4.3831168831168834e-04*w47*Jinv20+1.8262987012987013e-03*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1+1.9480519480519481e-04*w10*Jinv11)*D;
  A[41][3]=(0.0)*D;
  A[41][4]=(0.0)*D;
  A[41][5]=(2.6846590909090908e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10-4.6022727272727274e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-5.2962662337662338e-04*w57*Jinv02+2.6846590909090908e-03*w13*Jinv10-4.6022727272727274e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+5.4788961038961036e-04*w3*Jinv00+2.6846590909090908e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54-5.2962662337662338e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-6.5746753246753243e-04*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10-4.6022727272727274e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12-7.6704545454545450e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01-1.6436688311688311e-04*w1*Jinv10-3.6160714285714286e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00-4.6022727272727274e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.5888798701298701e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-5.2962662337662338e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10-4.6022727272727274e-03*Jinv22*w17-3.6160714285714286e-03*Jinv00*w33+1.3697240259740260e-03*w22*Jinv12+2.6846590909090908e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-1.5888798701298701e-03*w32*Jinv22-4.6022727272727274e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+6.5746753246753243e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-5.2962662337662338e-04*Jinv22*w59-4.6022727272727274e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.2873376623376621e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20-1.5888798701298701e-03*Jinv12*w13+6.5746753246753243e-04*w19*Jinv10-3.6160714285714286e-03*w34*Jinv10-1.5888798701298701e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56-4.6022727272727274e-03*Jinv22*w41+3.2873376623376621e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-4.6022727272727274e-03*Jinv21*w35-1.5888798701298701e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.6846590909090908e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+1.3697240259740260e-03*w21*Jinv02-7.6704545454545450e-04*w49*Jinv10-5.2962662337662338e-04*Jinv21*w29-4.6022727272727274e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.5888798701298701e-03*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9+1.3697240259740260e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+6.5746753246753243e-04*Jinv20*w38+1.3697240259740260e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11-1.5888798701298701e-03*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12+3.2873376623376621e-04*Jinv20*w53+1.3697240259740260e-03*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11+1.3697240259740260e-03*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51-1.5888798701298701e-03*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+3.2873376623376621e-04*w52*Jinv10+1.3697240259740260e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56+3.2873376623376621e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+1.3697240259740260e-03*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22+1.3697240259740260e-03*w49*Jinv11-1.5888798701298701e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-5.2962662337662338e-04*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-1.5888798701298701e-03*Jinv01*w12+1.3697240259740260e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33-1.6436688311688311e-04*w0*Jinv00-3.6160714285714286e-03*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11+3.2873376623376621e-04*w25*Jinv10-1.6436688311688311e-04*Jinv20*w2-3.6160714285714286e-03*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-3.6160714285714286e-03*Jinv20*w17+2.6846590909090908e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-1.5888798701298701e-03*Jinv22*w14-4.6022727272727274e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+5.4788961038961036e-04*w5*Jinv20-1.5888798701298701e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20-4.6022727272727274e-03*w16*Jinv12+1.3697240259740260e-03*Jinv21*w50-5.2962662337662338e-04*Jinv01*w27+1.3697240259740260e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[41][6]=(0.0)*D;
  A[41][7]=(0.0)*D;
  A[41][8]=(-2.5750811688311690e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+3.2873376623376621e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53+5.4788961038961036e-04*Jinv20*w8+3.6525974025974026e-05*w57*Jinv02-2.5750811688311690e-03*w13*Jinv10+3.2873376623376621e-03*Jinv02*w39+1.3149350649350649e-03*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00-2.5750811688311690e-03*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23-1.0957792207792207e-03*Jinv22*w38-5.4788961038961042e-05*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+3.8352272727272725e-04*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+3.6525974025974026e-05*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.3149350649350649e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+5.4788961038961042e-05*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55-3.6525974025974026e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-1.0957792207792208e-04*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26+3.6525974025974026e-05*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10+2.3011363636363637e-03*w16*Jinv10-1.0957792207792207e-03*Jinv22*w20+1.3149350649350649e-03*w43*Jinv11-5.4788961038961042e-05*w45*Jinv00+3.2873376623376621e-03*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22+3.6525974025974026e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+3.6525974025974026e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-5.4788961038961042e-05*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+2.3011363636363637e-03*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-2.5750811688311690e-03*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+3.2873376623376621e-03*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2-1.0957792207792207e-03*Jinv21*w38+5.4788961038961042e-05*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24+3.6525974025974026e-05*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-1.0957792207792207e-03*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53-3.2873376623376621e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-6.5746753246753243e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+2.3011363636363637e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-1.0957792207792207e-03*Jinv21*w20+5.4788961038961036e-04*w6*Jinv00-1.0957792207792207e-03*w37*Jinv12-5.4788961038961042e-05*Jinv20*w56+3.2873376623376621e-03*Jinv22*w41-3.2873376623376621e-04*w26*Jinv20-1.8262987012987014e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+3.8352272727272725e-04*Jinv22*w26+5.4788961038961042e-05*w46*Jinv11-2.5750811688311690e-03*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-1.0957792207792208e-04*w49*Jinv10+3.6525974025974026e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33-6.5746753246753243e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-1.0957792207792207e-03*Jinv01*w18-1.0957792207792207e-03*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-1.0957792207792207e-03*w6*Jinv01+5.4788961038961036e-04*w7*Jinv10+5.4788961038961042e-05*w47*Jinv21+5.4788961038961042e-05*Jinv22*w23+3.8352272727272725e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+5.4788961038961042e-05*w46*Jinv12-3.2873376623376621e-04*Jinv20*w53-1.0957792207792207e-03*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+5.4788961038961042e-05*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-1.0957792207792207e-03*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+5.4788961038961042e-05*Jinv12*w49-1.0957792207792207e-03*w36*Jinv01-3.2873376623376621e-04*w52*Jinv10-1.0957792207792207e-03*w7*Jinv11-1.8262987012987014e-04*w9*Jinv00+5.4788961038961042e-05*Jinv21*w56-3.2873376623376621e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-1.0957792207792207e-03*w6*Jinv02-1.0957792207792207e-03*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5+5.4788961038961042e-05*w47*Jinv22+5.4788961038961042e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-1.8262987012987014e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+3.6525974025974026e-05*w58*Jinv12-1.0957792207792207e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-1.0957792207792207e-03*Jinv21*w8-3.6525974025974026e-05*w57*Jinv00-3.6525974025974026e-05*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00+2.3011363636363637e-03*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23+5.4788961038961042e-05*Jinv01*w54+5.4788961038961042e-05*w55*Jinv11-3.2873376623376621e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2+2.3011363636363637e-03*w15*Jinv00+5.4788961038961042e-05*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+2.3011363636363637e-03*Jinv20*w17-2.5750811688311690e-03*w30*Jinv00-6.5746753246753243e-03*w40*Jinv10+3.6525974025974026e-05*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14+3.2873376623376621e-03*w40*Jinv11-1.0957792207792207e-03*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-5.4788961038961042e-05*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-5.4788961038961042e-05*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+5.4788961038961042e-05*Jinv21*w50+3.6525974025974026e-05*Jinv01*w27+5.4788961038961042e-05*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[41][9]=(0.0)*D;
  A[41][10]=(0.0)*D;
  A[41][11]=(5.4788961038961036e-04*Jinv00*w12+3.0438311688311688e-05*w28*Jinv10-3.1047077922077924e-04*Jinv20*w8+5.4788961038961036e-04*w13*Jinv10+2.1915584415584417e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23+5.4788961038961036e-04*Jinv00*w54-5.1136363636363640e-04*w43*Jinv10+3.0438311688311688e-05*w58*Jinv10+5.4788961038961036e-04*w48*Jinv00+3.0438311688311688e-05*w1*Jinv10-5.1136363636363640e-04*w16*Jinv10+5.4788961038961036e-04*w45*Jinv00+3.0438311688311688e-05*Jinv00*w27+5.4788961038961036e-04*Jinv20*w50+3.0438311688311688e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-5.1136363636363640e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10+5.4788961038961036e-04*w21*Jinv00-3.1047077922077924e-04*Jinv00*w18-3.1047077922077924e-04*w36*Jinv00+2.1915584415584417e-04*w51*Jinv00+2.1915584415584417e-04*w4*Jinv10+5.5519480519480519e-03*w41*Jinv20-3.1047077922077924e-04*w19*Jinv10-5.1136363636363640e-04*w34*Jinv10-3.1047077922077924e-04*w6*Jinv00+5.4788961038961036e-04*Jinv20*w56+2.1915584415584417e-04*w26*Jinv20+2.4350649350649351e-04*w11*Jinv20+5.4788961038961036e-04*Jinv20*w14+5.4788961038961036e-04*w49*Jinv10+5.5519480519480519e-03*w39*Jinv00-3.1047077922077924e-04*w7*Jinv10-3.1047077922077924e-04*Jinv20*w38+2.1915584415584417e-04*Jinv20*w53+5.4788961038961036e-04*w22*Jinv10+2.1915584415584417e-04*w52*Jinv10+2.4350649350649351e-04*w9*Jinv00-3.1047077922077924e-04*w37*Jinv10+2.1915584415584417e-04*w24*Jinv00+2.4350649350649351e-04*w10*Jinv10-5.1136363636363640e-04*Jinv20*w44+3.0438311688311688e-05*w57*Jinv00+3.0438311688311688e-05*Jinv20*w59+3.0438311688311688e-05*w0*Jinv00-5.1136363636363640e-04*w35*Jinv20+2.1915584415584417e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2-5.1136363636363640e-04*w15*Jinv00-5.1136363636363640e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00+5.5519480519480519e-03*w40*Jinv10-3.1047077922077924e-04*w20*Jinv20+2.1915584415584417e-04*w5*Jinv20+5.4788961038961036e-04*w55*Jinv10-5.1136363636363640e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20)*D;
  A[41][12]=(0.0)*D;
  A[41][13]=(0.0)*D;
  A[41][14]=(-4.7666396103896100e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-1.3806818181818182e-02*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+2.5202922077922079e-03*Jinv20*w8-1.0592532467532468e-03*w57*Jinv02-4.7666396103896100e-03*w13*Jinv10-1.3806818181818182e-02*Jinv02*w39-2.3011363636363637e-03*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-3.1777597402597403e-03*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3+6.5746753246753243e-04*Jinv20*w23+8.7662337662337668e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-5.6615259740259744e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12+7.6704545454545450e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-2.3011363636363637e-03*w42*Jinv01-6.9399350649350649e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17+8.7662337662337668e-04*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-1.0592532467532468e-03*w58*Jinv10-6.9034090909090911e-03*w15*Jinv02-9.6793831168831166e-04*w10*Jinv12+2.7394480519480520e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-4.6022727272727274e-03*Jinv22*w35-5.6615259740259744e-04*w58*Jinv11-5.6615259740259744e-04*w57*Jinv01+4.3831168831168834e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-6.9034090909090911e-03*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-2.3011363636363637e-03*w43*Jinv11+1.6436688311688311e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-9.6793831168831166e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-4.9310064935064932e-04*w13*Jinv11-5.9172077922077923e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+2.7394480519480520e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29+1.6436688311688311e-03*w46*Jinv10-6.9034090909090911e-03*Jinv22*w17-4.6022727272727274e-03*Jinv00*w33+6.5746753246753243e-04*w22*Jinv12-3.1777597402597403e-03*w31*Jinv10-1.5888798701298701e-03*Jinv02*w3+6.5746753246753243e-04*w21*Jinv00-3.1777597402597403e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39+7.6704545454545450e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+8.7662337662337668e-04*w36*Jinv00+8.2183441558441553e-04*Jinv21*w38+1.6436688311688311e-03*Jinv02*w45+4.3831168831168834e-04*Jinv01*w24-1.0592532467532468e-03*Jinv22*w59-3.6160714285714286e-03*w34*Jinv11+1.2601461038961039e-03*w19*Jinv11+3.2873376623376621e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00-4.6022727272727274e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-1.3806818181818182e-02*w41*Jinv20-4.7666396103896100e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-4.6022727272727274e-03*w34*Jinv10+1.0957792207792207e-03*w32*Jinv21+1.2601461038961039e-03*Jinv21*w20+2.5202922077922079e-03*w6*Jinv00+8.7662337662337668e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-1.3806818181818182e-02*Jinv22*w41+8.2183441558441553e-04*w26*Jinv20-9.6793831168831166e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-3.6160714285714286e-03*Jinv21*w35+1.0957792207792207e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+8.7662337662337668e-04*w46*Jinv11-4.7666396103896100e-03*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54-1.5888798701298701e-03*w4*Jinv12+6.5746753246753243e-04*w21*Jinv02+2.7394480519480520e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-3.6160714285714286e-03*Jinv01*w33-1.3806818181818182e-02*w39*Jinv00-4.7666396103896100e-03*Jinv02*w12+1.2601461038961039e-03*Jinv01*w18+8.2183441558441553e-04*w37*Jinv11-9.6793831168831166e-04*Jinv02*w9+1.2053571428571428e-03*w6*Jinv01+2.5202922077922079e-03*w7*Jinv10+8.7662337662337668e-04*w47*Jinv21+8.7662337662337668e-04*Jinv20*w38+6.5746753246753243e-04*Jinv22*w23+3.2873376623376621e-04*w52*Jinv11-3.1777597402597403e-03*w31*Jinv12+1.6436688311688311e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53+2.5202922077922079e-03*Jinv22*w8+6.5746753246753243e-04*w22*Jinv10+2.7394480519480520e-03*Jinv22*w50-1.4793019480519481e-03*w22*Jinv11+2.5202922077922079e-03*Jinv12*w7-1.4793019480519481e-03*w21*Jinv01+3.2873376623376621e-04*Jinv01*w51-3.1777597402597403e-03*Jinv02*w30+2.7394480519480520e-03*Jinv12*w49+8.2183441558441553e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10+1.2053571428571428e-03*w7*Jinv11-9.6793831168831166e-04*w9*Jinv00+8.7662337662337668e-04*w37*Jinv10+1.3697240259740260e-03*Jinv21*w56+8.2183441558441553e-04*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+2.5202922077922079e-03*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12-1.5888798701298701e-03*Jinv22*w5+1.6436688311688311e-03*w47*Jinv22+6.0267857142857139e-04*w49*Jinv11-4.9310064935064932e-04*Jinv21*w14-9.6793831168831166e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-1.0592532467532468e-03*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18-4.9310064935064932e-04*Jinv01*w12+1.2053571428571428e-03*Jinv21*w8-1.0592532467532468e-03*w57*Jinv00-1.0592532467532468e-03*Jinv20*w59-4.6022727272727274e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-4.6022727272727274e-03*w35*Jinv20-1.4793019480519481e-03*Jinv21*w23+1.3697240259740260e-03*Jinv01*w54+1.3697240259740260e-03*w55*Jinv11+8.2183441558441553e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-6.9034090909090911e-03*w15*Jinv00+2.7394480519480520e-03*w48*Jinv02-6.9399350649350649e-04*Jinv01*w9-6.9034090909090911e-03*Jinv20*w17-3.1777597402597403e-03*w30*Jinv00-1.3806818181818182e-02*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-4.7666396103896100e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11+8.7662337662337668e-04*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20+1.0957792207792207e-03*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+4.3831168831168834e-04*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+1.6436688311688311e-03*w47*Jinv20-6.9034090909090911e-03*w16*Jinv12+6.0267857142857139e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+6.0267857142857139e-04*w48*Jinv01+7.6704545454545450e-04*Jinv12*w1-6.9399350649350649e-04*w10*Jinv11)*D;
  A[41][15]=(0.0)*D;
  A[41][16]=(0.0)*D;
  A[41][17]=(4.9310064935064934e-03*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+1.3149350649350649e-03*Jinv22*w53-3.6160714285714286e-03*Jinv20*w8-1.1323051948051949e-03*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44+2.3011363636363637e-03*w3*Jinv00+3.2873376623376621e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3+1.9724025974025973e-03*Jinv20*w23+1.3149350649350649e-03*Jinv22*w38-1.9724025974025973e-03*Jinv00*w54-9.8620129870129864e-04*w25*Jinv12-1.1323051948051949e-03*w0*Jinv02-9.1314935064935063e-04*Jinv21*w2-3.9448051948051946e-03*w42*Jinv01-2.3011363636363637e-03*Jinv01*w45+1.9724025974025973e-03*Jinv12*w55+1.1323051948051949e-03*w58*Jinv10-1.1834415584415585e-02*w15*Jinv02-1.1323051948051949e-03*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00-9.8620129870129864e-04*Jinv02*w24-7.8896103896103891e-03*Jinv22*w35-2.3011363636363637e-03*Jinv21*w26-9.1314935064935063e-04*w0*Jinv01-6.9399350649350649e-04*w1*Jinv10+5.9172077922077923e-03*w16*Jinv10+1.9724025974025973e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+9.8620129870129864e-04*w45*Jinv00-1.1834415584415585e-02*w42*Jinv02+2.1915584415584417e-04*Jinv00*w27-1.1323051948051949e-03*w11*Jinv22-9.1314935064935063e-04*Jinv11*w1+2.9586038961038961e-03*w13*Jinv11+9.1314935064935063e-04*w28*Jinv11-2.6298701298701297e-03*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+9.8620129870129864e-04*w46*Jinv10-1.1834415584415585e-02*Jinv22*w17+3.9448051948051946e-03*Jinv00*w33-9.8620129870129864e-04*w22*Jinv12+3.2873376623376621e-03*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3+1.9724025974025973e-03*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-1.1323051948051949e-03*Jinv22*w2-2.9586038961038961e-03*Jinv00*w18-1.9724025974025973e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-2.3011363636363637e-03*Jinv01*w24-1.1323051948051949e-03*Jinv22*w59+3.9448051948051946e-03*w34*Jinv11+6.5746753246753243e-04*w19*Jinv11-7.8896103896103891e-03*Jinv12*w34-1.1834415584415585e-02*Jinv22*w44+2.3011363636363637e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11+2.3668831168831169e-02*w41*Jinv20+1.9724025974025973e-03*Jinv12*w13-2.9586038961038961e-03*w19*Jinv10+3.9448051948051946e-03*w34*Jinv10+2.3011363636363637e-03*w32*Jinv21+6.5746753246753243e-04*Jinv21*w20-3.6160714285714286e-03*w6*Jinv00+1.3149350649350649e-03*w37*Jinv12-1.9724025974025973e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41+1.6436688311688311e-03*w11*Jinv20+1.3149350649350649e-03*w52*Jinv12+3.9448051948051946e-03*Jinv21*w35+2.3011363636363637e-03*w31*Jinv11-9.8620129870129864e-04*Jinv22*w26-2.3011363636363637e-03*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21+1.9724025974025973e-03*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12-9.8620129870129864e-04*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10+9.1314935064935063e-04*Jinv21*w29+3.9448051948051946e-03*Jinv01*w33+2.3668831168831169e-02*w39*Jinv00+1.9724025974025973e-03*Jinv02*w12+6.5746753246753243e-04*Jinv01*w18-1.1323051948051949e-03*Jinv02*w9-6.5746753246753243e-04*w6*Jinv01-3.6160714285714286e-03*w7*Jinv10-2.3011363636363637e-03*w47*Jinv21-1.9724025974025973e-03*Jinv20*w38-9.8620129870129864e-04*Jinv22*w23+1.3149350649350649e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12+1.3149350649350649e-03*Jinv22*w8+1.9724025974025973e-03*w22*Jinv10+1.3149350649350649e-03*Jinv22*w50-2.9586038961038961e-03*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7-2.9586038961038961e-03*w21*Jinv01+1.3149350649350649e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-6.5746753246753243e-04*w7*Jinv11+1.6436688311688311e-03*w9*Jinv00-1.9724025974025973e-03*w37*Jinv10+6.5746753246753243e-04*Jinv21*w56-2.1915584415584417e-04*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02+1.9724025974025973e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22-6.5746753246753243e-04*w49*Jinv11+2.9586038961038961e-03*Jinv21*w14+1.6436688311688311e-03*w10*Jinv10-1.1323051948051949e-03*w58*Jinv12+1.9724025974025973e-03*Jinv02*w18+2.9586038961038961e-03*Jinv01*w12-6.5746753246753243e-04*Jinv21*w8+1.1323051948051949e-03*w57*Jinv00+1.1323051948051949e-03*Jinv20*w59-7.8896103896103891e-03*Jinv02*w33-6.9399350649350649e-04*w0*Jinv00+3.9448051948051946e-03*w35*Jinv20-2.9586038961038961e-03*Jinv21*w23+6.5746753246753243e-04*Jinv01*w54+6.5746753246753243e-04*w55*Jinv11-6.9399350649350649e-04*Jinv20*w2+5.9172077922077923e-03*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02+5.9172077922077923e-03*Jinv20*w17+3.2873376623376621e-03*w30*Jinv00+2.3668831168831169e-02*w40*Jinv10-2.9586038961038961e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.9724025974025973e-03*Jinv22*w14+1.3149350649350649e-03*Jinv02*w36-1.1834415584415585e-02*Jinv12*w43+2.3011363636363637e-03*w5*Jinv20+2.3011363636363637e-03*Jinv01*w30+1.3149350649350649e-03*Jinv02*w51+1.9724025974025973e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-2.3011363636363637e-03*w25*Jinv11-1.9724025974025973e-03*w55*Jinv10+9.8620129870129864e-04*w47*Jinv20-1.1834415584415585e-02*w16*Jinv12-6.5746753246753243e-04*Jinv21*w50+9.1314935064935063e-04*Jinv01*w27-6.5746753246753243e-04*w48*Jinv01-1.1323051948051949e-03*Jinv12*w1)*D;
  A[41][18]=(0.0)*D;
  A[41][19]=(0.0)*D;
  A[41][20]=(-1.6436688311688311e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10+1.0957792207792207e-03*Jinv20*w8-1.6436688311688311e-04*w13*Jinv10-1.3149350649350649e-03*Jinv21*w44-7.1225649350649346e-04*w3*Jinv00-1.0957792207792208e-04*w32*Jinv20-3.8352272727272725e-04*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54-3.6525974025974026e-05*Jinv21*w59-3.6525974025974026e-05*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10-1.3149350649350649e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-5.4788961038961042e-05*Jinv01*w45-7.3051948051948052e-05*w58*Jinv10-1.0957792207792208e-04*w48*Jinv00-3.6525974025974026e-05*w58*Jinv11-3.6525974025974026e-05*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26-3.6525974025974026e-05*w0*Jinv01-7.3051948051948052e-05*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-1.3149350649350649e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-3.2873376623376621e-03*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27-3.6525974025974026e-05*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-3.6525974025974026e-05*w28*Jinv11-1.0957792207792208e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+6.5746753246753243e-04*Jinv00*w33-1.0957792207792208e-04*w31*Jinv10-2.6298701298701297e-03*w21*Jinv00-3.2873376623376621e-03*Jinv01*w39+1.6436688311688311e-03*Jinv00*w18+1.0957792207792207e-03*w36*Jinv00+1.0957792207792207e-03*Jinv21*w38-3.8352272727272725e-04*Jinv01*w24-1.3149350649350649e-03*w34*Jinv11+1.0957792207792207e-03*w19*Jinv11-3.8352272727272725e-04*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00-7.1225649350649346e-04*w4*Jinv10-3.8352272727272725e-04*w4*Jinv11-9.8620129870129868e-03*w41*Jinv20+1.6436688311688311e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+1.0957792207792207e-03*Jinv21*w20+1.0957792207792207e-03*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-8.2183441558441553e-04*w26*Jinv20-6.7573051948051951e-04*w11*Jinv20-1.3149350649350649e-03*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-5.4788961038961042e-05*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-3.8352272727272725e-04*w5*Jinv21-1.0957792207792208e-04*w49*Jinv10-3.6525974025974026e-05*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33-9.8620129870129868e-03*w39*Jinv00+1.0957792207792207e-03*Jinv01*w18+1.0957792207792207e-03*w37*Jinv11+1.0957792207792207e-03*w6*Jinv01+1.0957792207792207e-03*w7*Jinv10-5.4788961038961042e-05*w47*Jinv21+1.0957792207792207e-03*Jinv20*w38-3.8352272727272725e-04*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53-2.6298701298701297e-03*w22*Jinv10-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51+1.0957792207792207e-03*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+1.0957792207792207e-03*w7*Jinv11-6.7573051948051951e-04*w9*Jinv00+1.0957792207792207e-03*w37*Jinv10-5.4788961038961042e-05*Jinv21*w56-8.2183441558441553e-04*w24*Jinv00-5.4788961038961042e-05*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-6.7573051948051951e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-5.4788961038961042e-05*Jinv01*w12+1.0957792207792207e-03*Jinv21*w8-7.3051948051948052e-05*w57*Jinv00-7.3051948051948052e-05*Jinv20*w59-7.3051948051948052e-05*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23-5.4788961038961042e-05*Jinv01*w54-5.4788961038961042e-05*w55*Jinv11-8.2183441558441553e-04*w25*Jinv10-7.3051948051948052e-05*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-4.9310064935064932e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.0957792207792208e-04*w30*Jinv00-9.8620129870129868e-03*w40*Jinv10+1.6436688311688311e-03*w20*Jinv20-3.2873376623376621e-03*w40*Jinv11-7.1225649350649346e-04*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-3.8352272727272725e-04*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20-5.4788961038961042e-05*Jinv21*w50-3.6525974025974026e-05*Jinv01*w27-5.4788961038961042e-05*w48*Jinv01-4.9310064935064932e-04*w10*Jinv11)*D;
  A[41][21]=(0.0)*D;
  A[41][22]=(0.0)*D;
  A[41][23]=(2.1367694805194807e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+3.8352272727272725e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+2.1367694805194807e-03*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+3.6160714285714286e-03*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+7.6704545454545450e-04*w32*Jinv20-4.3831168831168834e-04*Jinv01*w3-4.2735389610389614e-03*Jinv20*w23+5.4788961038961042e-05*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+5.6615259740259744e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12-4.9310064935064932e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+3.6160714285714286e-03*w42*Jinv01+6.9399350649350649e-04*w11*Jinv21+5.9172077922077923e-03*w16*Jinv11+5.9172077922077923e-03*Jinv21*w17-1.0957792207792207e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-2.7394480519480518e-04*w10*Jinv12+7.6704545454545450e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+5.6615259740259744e-04*w58*Jinv11+5.6615259740259744e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20+3.6160714285714286e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41-9.8620129870129864e-04*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+1.4793019480519481e-03*w13*Jinv11+5.9172077922077923e-03*w15*Jinv01-2.1915584415584417e-04*w28*Jinv11+7.6704545454545450e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-4.2735389610389614e-03*w22*Jinv12+7.6704545454545450e-04*w31*Jinv10+3.8352272727272725e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00+7.6704545454545450e-04*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39-4.9310064935064932e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+5.4788961038961042e-05*w36*Jinv00-8.2183441558441553e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+2.3011363636363637e-03*w34*Jinv11-1.2053571428571428e-03*w19*Jinv11-3.2873376623376621e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-9.8620129870129864e-04*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-4.3831168831168834e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20+2.1367694805194807e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-8.7662337662337668e-04*w32*Jinv21-1.2053571428571428e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+5.4788961038961042e-05*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20+3.8352272727272725e-04*w52*Jinv12+2.3011363636363637e-03*Jinv21*w35-8.7662337662337668e-04*w31*Jinv11-2.1367694805194807e-03*Jinv22*w26-1.0957792207792207e-03*w46*Jinv11+2.1367694805194807e-03*Jinv20*w14-4.3831168831168834e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54+3.8352272727272725e-04*w4*Jinv12-4.2735389610389614e-03*w21*Jinv02+7.6704545454545450e-04*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+2.3011363636363637e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00+2.1367694805194807e-03*Jinv02*w12-1.2053571428571428e-03*Jinv01*w18-8.2183441558441553e-04*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9-1.2601461038961039e-03*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-1.0957792207792207e-03*w47*Jinv21+5.4788961038961042e-05*Jinv20*w38-4.2735389610389614e-03*Jinv22*w23-3.2873376623376621e-04*w52*Jinv11+7.6704545454545450e-04*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-4.2735389610389614e-03*w22*Jinv10+7.6704545454545450e-04*Jinv22*w50+4.9310064935064932e-04*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+4.9310064935064932e-04*w21*Jinv01-3.2873376623376621e-04*Jinv01*w51+7.6704545454545450e-04*Jinv02*w30+7.6704545454545450e-04*Jinv12*w49-8.2183441558441553e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10-1.2601461038961039e-03*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+5.4788961038961042e-05*w37*Jinv10-6.0267857142857139e-04*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12+3.8352272727272725e-04*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22-1.3697240259740260e-03*w49*Jinv11+1.4793019480519481e-03*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18+1.4793019480519481e-03*Jinv01*w12-1.2601461038961039e-03*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+4.9310064935064932e-04*Jinv21*w23-6.0267857142857139e-04*Jinv01*w54-6.0267857142857139e-04*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+7.6704545454545450e-04*w48*Jinv02+6.9399350649350649e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+7.6704545454545450e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+2.1367694805194807e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+5.4788961038961042e-05*Jinv02*w36-9.8620129870129864e-04*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20-8.7662337662337668e-04*Jinv01*w30+3.8352272727272725e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-1.3697240259740260e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27-1.3697240259740260e-03*w48*Jinv01-4.9310064935064932e-04*Jinv12*w1+6.9399350649350649e-04*w10*Jinv11)*D;
  A[41][24]=(0.0)*D;
  A[41][25]=(0.0)*D;
  A[41][26]=(-2.1367694805194807e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-2.1367694805194807e-03*w13*Jinv10+4.6022727272727274e-03*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-7.6704545454545450e-04*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+5.2962662337662338e-04*Jinv21*w59+5.2962662337662338e-04*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+4.6022727272727274e-03*w42*Jinv01+5.2962662337662338e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+1.5888798701298701e-03*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-7.6704545454545450e-04*w48*Jinv00+5.2962662337662338e-04*w58*Jinv11+5.2962662337662338e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+5.2962662337662338e-04*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+4.6022727272727274e-03*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+5.2962662337662338e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11-7.6704545454545450e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-7.6704545454545450e-04*w31*Jinv10+4.2735389610389614e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-1.3149350649350649e-03*Jinv00*w18-5.4788961038961042e-05*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38+1.5888798701298701e-03*Jinv01*w24+6.5746753246753243e-04*w34*Jinv11-1.3697240259740260e-03*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.8352272727272725e-04*w51*Jinv00-3.8352272727272725e-04*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-1.3149350649350649e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21-1.3697240259740260e-03*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+2.1367694805194807e-03*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+6.5746753246753243e-04*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.5888798701298701e-03*w46*Jinv11-2.1367694805194807e-03*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21-7.6704545454545450e-04*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-1.3697240259740260e-03*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+1.5888798701298701e-03*w47*Jinv21-5.4788961038961042e-05*Jinv20*w38-7.1225649350649346e-04*w52*Jinv11-3.8352272727272725e-04*Jinv20*w53+4.2735389610389614e-03*w22*Jinv10+1.5888798701298701e-03*w22*Jinv11+1.5888798701298701e-03*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01-3.8352272727272725e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-5.4788961038961042e-05*w37*Jinv10-1.3697240259740260e-03*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-1.3697240259740260e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+4.9310064935064932e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.5888798701298701e-03*Jinv21*w23-1.3697240259740260e-03*Jinv01*w54-1.3697240259740260e-03*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+5.2962662337662338e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-7.6704545454545450e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-1.3149350649350649e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-3.8352272727272725e-04*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30+1.5888798701298701e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+5.2962662337662338e-04*w10*Jinv11)*D;
  A[41][27]=(0.0)*D;
  A[41][28]=(0.0)*D;
  A[41][29]=(-1.8262987012987013e-03*Jinv21*w44+2.1915584415584417e-04*Jinv01*w3-4.2613636363636367e-05*Jinv21*w59-4.2613636363636367e-05*Jinv21*w2-1.8262987012987013e-03*w42*Jinv01-1.9480519480519481e-04*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-1.8262987012987013e-03*w43*Jinv11-3.6525974025974025e-03*Jinv21*w41-4.2613636363636367e-05*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-3.6525974025974025e-03*Jinv01*w39+3.4699675324675324e-04*Jinv21*w38+3.2873376623376621e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11+2.9220779220779221e-04*w19*Jinv11+2.1915584415584417e-04*Jinv21*w53+2.1915584415584417e-04*w4*Jinv11+4.3831168831168834e-04*w32*Jinv21+2.9220779220779221e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+4.3831168831168834e-04*w31*Jinv11+6.5746753246753243e-04*w46*Jinv11+2.1915584415584417e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33+2.9220779220779221e-04*Jinv01*w18+3.4699675324675324e-04*w37*Jinv11+3.4699675324675324e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+2.1915584415584417e-04*w52*Jinv11+6.5746753246753243e-04*w22*Jinv11+6.5746753246753243e-04*w21*Jinv01+2.1915584415584417e-04*Jinv01*w51+3.4699675324675324e-04*w36*Jinv01+3.4699675324675324e-04*w7*Jinv11-3.2873376623376621e-04*Jinv21*w56+4.3831168831168834e-04*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-3.2873376623376621e-04*Jinv01*w12+3.4699675324675324e-04*Jinv21*w8+6.5746753246753243e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-1.9480519480519481e-04*Jinv01*w9-3.6525974025974025e-03*w40*Jinv11+4.3831168831168834e-04*Jinv01*w30+3.2873376623376621e-04*w25*Jinv11+4.3831168831168834e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+4.3831168831168834e-04*w48*Jinv01-1.9480519480519481e-04*w10*Jinv11)*D;
  A[41][30]=(0.0)*D;
  A[41][31]=(0.0)*D;
  A[41][32]=(-3.1777597402597403e-03*Jinv00*w12-1.0592532467532468e-03*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+4.3831168831168834e-04*Jinv22*w53+2.5202922077922079e-03*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-3.1777597402597403e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.5888798701298701e-03*w3*Jinv00-4.7666396103896100e-03*w32*Jinv20-1.5888798701298701e-03*Jinv01*w3+2.7394480519480520e-03*Jinv20*w23+1.2601461038961039e-03*Jinv22*w38+1.6436688311688311e-03*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+3.2873376623376621e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+7.6704545454545450e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-9.6793831168831166e-04*w11*Jinv21-4.6022727272727274e-03*w16*Jinv11-4.6022727272727274e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45+8.7662337662337668e-04*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-6.9399350649350649e-04*w10*Jinv12+6.5746753246753243e-04*w48*Jinv00+3.2873376623376621e-04*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26+7.6704545454545450e-04*w0*Jinv01+7.6704545454545450e-04*w1*Jinv10-4.6022727272727274e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-1.3806818181818182e-02*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-1.0592532467532468e-03*Jinv00*w27-6.9399350649350649e-04*w11*Jinv22+7.6704545454545450e-04*Jinv11*w1-3.1777597402597403e-03*w13*Jinv11-4.6022727272727274e-03*w15*Jinv01-1.0592532467532468e-03*w28*Jinv11+6.5746753246753243e-04*Jinv20*w50-1.0592532467532468e-03*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-6.9034090909090911e-03*Jinv00*w33+6.0267857142857139e-04*w22*Jinv12-4.7666396103896100e-03*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+2.7394480519480520e-03*w21*Jinv00-4.9310064935064932e-04*w32*Jinv22-1.3806818181818182e-02*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2+8.7662337662337668e-04*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-6.9034090909090911e-03*w34*Jinv11+8.7662337662337668e-04*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+8.2183441558441553e-04*w51*Jinv00-5.9172077922077923e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-1.5888798701298701e-03*w4*Jinv10-1.5888798701298701e-03*w4*Jinv11-1.3806818181818182e-02*w41*Jinv20+1.0957792207792207e-03*Jinv12*w13+8.7662337662337668e-04*w19*Jinv10-6.9034090909090911e-03*w34*Jinv10-4.7666396103896100e-03*w32*Jinv21+8.7662337662337668e-04*Jinv21*w20+2.5202922077922079e-03*w6*Jinv00+1.2601461038961039e-03*w37*Jinv12+1.6436688311688311e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-9.6793831168831166e-04*w11*Jinv20+4.3831168831168834e-04*w52*Jinv12-6.9034090909090911e-03*Jinv21*w35-4.7666396103896100e-03*w31*Jinv11+3.2873376623376621e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11-3.1777597402597403e-03*Jinv20*w14-1.5888798701298701e-03*w5*Jinv21+8.7662337662337668e-04*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12+6.0267857142857139e-04*w21*Jinv02+6.5746753246753243e-04*w49*Jinv10-1.0592532467532468e-03*Jinv21*w29-6.9034090909090911e-03*Jinv01*w33-1.3806818181818182e-02*w39*Jinv00+1.0957792207792207e-03*Jinv02*w12+8.7662337662337668e-04*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-6.9399350649350649e-04*Jinv02*w9+2.5202922077922079e-03*w6*Jinv01+2.5202922077922079e-03*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+6.0267857142857139e-04*Jinv22*w23+8.2183441558441553e-04*w52*Jinv11-4.9310064935064932e-04*w31*Jinv12+1.3697240259740260e-03*w46*Jinv12+8.2183441558441553e-04*Jinv20*w53+1.2053571428571428e-03*Jinv22*w8+2.7394480519480520e-03*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+2.7394480519480520e-03*w22*Jinv11+1.2053571428571428e-03*Jinv12*w7+2.7394480519480520e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-4.9310064935064932e-04*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01+8.2183441558441553e-04*w52*Jinv10+2.5202922077922079e-03*w7*Jinv11-9.6793831168831166e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+1.6436688311688311e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-5.6615259740259744e-04*Jinv02*w27+1.2053571428571428e-03*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22+6.5746753246753243e-04*w49*Jinv11-3.1777597402597403e-03*Jinv21*w14-9.6793831168831166e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-3.1777597402597403e-03*Jinv01*w12+2.5202922077922079e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33+7.6704545454545450e-04*w0*Jinv00-6.9034090909090911e-03*w35*Jinv20+2.7394480519480520e-03*Jinv21*w23+1.6436688311688311e-03*Jinv01*w54+1.6436688311688311e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+7.6704545454545450e-04*Jinv20*w2-4.6022727272727274e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-9.6793831168831166e-04*Jinv01*w9-4.6022727272727274e-03*Jinv20*w17-4.7666396103896100e-03*w30*Jinv00-1.3806818181818182e-02*w40*Jinv10+8.7662337662337668e-04*w20*Jinv20-5.6615259740259744e-04*Jinv22*w29+1.0957792207792207e-03*Jinv22*w14-1.3806818181818182e-02*w40*Jinv11+1.2601461038961039e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-1.5888798701298701e-03*w5*Jinv20-4.7666396103896100e-03*Jinv01*w30+4.3831168831168834e-04*Jinv02*w51+8.7662337662337668e-04*Jinv22*w56-5.6615259740259744e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+1.6436688311688311e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+6.5746753246753243e-04*Jinv21*w50-1.0592532467532468e-03*Jinv01*w27+6.5746753246753243e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1-9.6793831168831166e-04*w10*Jinv11)*D;
  A[41][33]=(0.0)*D;
  A[41][34]=(0.0)*D;
  A[41][35]=(3.2873376623376621e-03*Jinv00*w12+1.1323051948051949e-03*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53-3.6160714285714286e-03*Jinv20*w8+9.1314935064935063e-04*w57*Jinv02+3.2873376623376621e-03*w13*Jinv10-1.1834415584415585e-02*Jinv21*w44+2.3011363636363637e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3-2.6298701298701297e-03*Jinv20*w23+6.5746753246753243e-04*Jinv22*w38+9.8620129870129864e-04*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-9.1314935064935063e-04*w0*Jinv02-1.1323051948051949e-03*Jinv21*w2-1.1834415584415585e-02*w42*Jinv01-1.1323051948051949e-03*w11*Jinv21-7.8896103896103891e-03*w16*Jinv11-7.8896103896103891e-03*Jinv21*w17+1.9724025974025973e-03*Jinv01*w45-2.3011363636363637e-03*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10+3.9448051948051946e-03*w15*Jinv02+1.9724025974025973e-03*w48*Jinv00-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+1.3149350649350649e-03*Jinv21*w26-1.1323051948051949e-03*w0*Jinv01-6.9399350649350649e-04*w1*Jinv10+3.9448051948051946e-03*w16*Jinv10-1.1834415584415585e-02*w43*Jinv11-1.9724025974025973e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+1.1323051948051949e-03*Jinv00*w27-1.1323051948051949e-03*Jinv11*w1+1.3149350649350649e-03*w13*Jinv11-7.8896103896103891e-03*w15*Jinv01-1.1323051948051949e-03*w28*Jinv11+1.9724025974025973e-03*Jinv20*w50+1.1323051948051949e-03*Jinv20*w29-1.9724025974025973e-03*w46*Jinv10+3.9448051948051946e-03*Jinv22*w17+5.9172077922077923e-03*Jinv00*w33-6.5746753246753243e-04*w22*Jinv12+4.9310064935064934e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3-2.6298701298701297e-03*w21*Jinv00+2.9586038961038961e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-9.1314935064935063e-04*Jinv22*w2-1.9724025974025973e-03*Jinv00*w18-2.9586038961038961e-03*w36*Jinv00+1.9724025974025973e-03*Jinv21*w38+6.5746753246753243e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24+9.1314935064935063e-04*Jinv22*w59-1.1834415584415585e-02*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11-9.8620129870129864e-04*Jinv21*w53-3.9448051948051946e-03*Jinv22*w44+2.3011363636363637e-03*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+2.3668831168831169e-02*w41*Jinv20+2.3011363636363637e-03*Jinv12*w13-1.9724025974025973e-03*w19*Jinv10+5.9172077922077923e-03*w34*Jinv10+1.9724025974025973e-03*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20-3.6160714285714286e-03*w6*Jinv00+6.5746753246753243e-04*w37*Jinv12+9.8620129870129864e-04*Jinv20*w56+1.6436688311688311e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12-1.1834415584415585e-02*Jinv21*w35+1.9724025974025973e-03*w31*Jinv11+1.9724025974025973e-03*w46*Jinv11+3.2873376623376621e-03*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21-2.3011363636363637e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12-6.5746753246753243e-04*w21*Jinv02+1.9724025974025973e-03*w49*Jinv10-1.1323051948051949e-03*Jinv21*w29-1.1834415584415585e-02*Jinv01*w33+2.3668831168831169e-02*w39*Jinv00+2.3011363636363637e-03*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18+1.9724025974025973e-03*w37*Jinv11+1.3149350649350649e-03*w6*Jinv01-3.6160714285714286e-03*w7*Jinv10+1.9724025974025973e-03*w47*Jinv21-2.9586038961038961e-03*Jinv20*w38-6.5746753246753243e-04*Jinv22*w23-9.8620129870129864e-04*w52*Jinv11+2.9586038961038961e-03*w31*Jinv12+6.5746753246753243e-04*w46*Jinv12-6.5746753246753243e-04*Jinv22*w8-2.6298701298701297e-03*w22*Jinv10-2.9586038961038961e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11-6.5746753246753243e-04*Jinv12*w7+1.3149350649350649e-03*w21*Jinv01-9.8620129870129864e-04*Jinv01*w51+2.9586038961038961e-03*Jinv02*w30-2.9586038961038961e-03*Jinv12*w49+1.9724025974025973e-03*w36*Jinv01+1.3149350649350649e-03*w7*Jinv11+1.6436688311688311e-03*w9*Jinv00-2.9586038961038961e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56-6.5746753246753243e-04*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5+6.5746753246753243e-04*w47*Jinv22-9.8620129870129864e-04*w49*Jinv11+1.3149350649350649e-03*Jinv21*w14+1.6436688311688311e-03*w10*Jinv10+9.1314935064935063e-04*w58*Jinv12+1.3149350649350649e-03*Jinv01*w12+1.3149350649350649e-03*Jinv21*w8+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-6.9399350649350649e-04*w0*Jinv00+5.9172077922077923e-03*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11-6.9399350649350649e-04*Jinv20*w2+3.9448051948051946e-03*w15*Jinv00-2.9586038961038961e-03*w48*Jinv02-1.1323051948051949e-03*Jinv01*w9+3.9448051948051946e-03*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00+2.3668831168831169e-02*w40*Jinv10-1.9724025974025973e-03*w20*Jinv20+2.3011363636363637e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11+6.5746753246753243e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43+2.3011363636363637e-03*w5*Jinv20+1.9724025974025973e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-2.3011363636363637e-03*Jinv22*w56+1.3149350649350649e-03*w25*Jinv11+9.8620129870129864e-04*w55*Jinv10-1.9724025974025973e-03*w47*Jinv20+3.9448051948051946e-03*w16*Jinv12-9.8620129870129864e-04*Jinv21*w50-1.1323051948051949e-03*Jinv01*w27-9.8620129870129864e-04*w48*Jinv01-9.1314935064935063e-04*Jinv12*w1-1.1323051948051949e-03*w10*Jinv11)*D;
  A[41][36]=(0.0)*D;
  A[41][37]=(0.0)*D;
  A[41][38]=(-1.0957792207792208e-04*Jinv00*w12-7.3051948051948052e-05*w28*Jinv10-3.2873376623376621e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+1.0957792207792207e-03*Jinv20*w8-3.6525974025974026e-05*w57*Jinv02-1.0957792207792208e-04*w13*Jinv10-3.2873376623376621e-03*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20-1.0957792207792208e-04*Jinv20*w23+1.0957792207792207e-03*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54-3.8352272727272725e-04*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10-5.4788961038961042e-05*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-2.6298701298701297e-03*w48*Jinv00-3.8352272727272725e-04*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-7.3051948051948052e-05*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+1.0957792207792207e-03*Jinv22*w20-1.6436688311688311e-04*w45*Jinv00-1.3149350649350649e-03*w42*Jinv02-7.3051948051948052e-05*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22-2.6298701298701297e-03*Jinv20*w50-7.3051948051948052e-05*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-3.8352272727272725e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-3.6525974025974026e-05*Jinv22*w2+1.0957792207792207e-03*Jinv00*w18+1.6436688311688311e-03*w36*Jinv00-5.4788961038961042e-05*Jinv02*w45-3.6525974025974026e-05*Jinv22*w59-8.2183441558441553e-04*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10-9.8620129870129868e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+1.0957792207792207e-03*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10+1.0957792207792207e-03*w6*Jinv00+1.0957792207792207e-03*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-3.2873376623376621e-03*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20-6.7573051948051951e-04*w11*Jinv20-3.8352272727272725e-04*w52*Jinv12-3.8352272727272725e-04*Jinv22*w26-1.0957792207792208e-04*Jinv20*w14-5.4788961038961042e-05*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-2.6298701298701297e-03*w49*Jinv10-9.8620129870129868e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-4.9310064935064932e-04*Jinv02*w9+1.0957792207792207e-03*w7*Jinv10+1.6436688311688311e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23-5.4788961038961042e-05*w31*Jinv12-5.4788961038961042e-05*w46*Jinv12-8.2183441558441553e-04*Jinv20*w53+1.0957792207792207e-03*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-5.4788961038961042e-05*Jinv22*w50+1.0957792207792207e-03*Jinv12*w7-5.4788961038961042e-05*Jinv02*w30-5.4788961038961042e-05*Jinv12*w49-8.2183441558441553e-04*w52*Jinv10-6.7573051948051951e-04*w9*Jinv00+1.6436688311688311e-03*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27+1.0957792207792207e-03*w6*Jinv02+1.0957792207792207e-03*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5-5.4788961038961042e-05*w47*Jinv22-6.7573051948051951e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-3.6525974025974026e-05*w58*Jinv12+1.0957792207792207e-03*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33-7.3051948051948052e-05*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10-7.3051948051948052e-05*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-5.4788961038961042e-05*w48*Jinv02+6.5746753246753243e-04*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00-9.8620129870129868e-03*w40*Jinv10+1.0957792207792207e-03*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+1.0957792207792207e-03*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20-3.8352272727272725e-04*Jinv02*w51-5.4788961038961042e-05*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28-2.6298701298701297e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-3.6525974025974026e-05*Jinv12*w1)*D;
  A[41][39]=(0.0)*D;
  A[41][40]=(0.0)*D;
  A[41][41]=(3.9448051948051946e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-2.3011363636363637e-03*Jinv22*w53+3.2873376623376621e-03*Jinv20*w8+1.8262987012987013e-03*w57*Jinv02+3.9448051948051946e-03*w13*Jinv10-5.9172077922077923e-03*Jinv21*w44+1.3149350649350649e-03*w3*Jinv00+3.9448051948051946e-03*w32*Jinv20+2.3011363636363637e-03*Jinv01*w3-1.9724025974025973e-03*Jinv20*w23+1.6436688311688311e-03*Jinv22*w38-2.9586038961038961e-03*Jinv00*w54-1.8262987012987013e-03*w0*Jinv02-1.8262987012987013e-03*Jinv21*w2-1.7751623376623376e-02*w43*Jinv10-5.9172077922077923e-03*w42*Jinv01-6.9034090909090911e-03*Jinv01*w45-6.9034090909090911e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10+5.9172077922077923e-03*w15*Jinv02-1.9724025974025973e-03*w48*Jinv00-2.3011363636363637e-03*Jinv21*w26-1.8262987012987013e-03*w0*Jinv01-2.2646103896103898e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10-5.9172077922077923e-03*w43*Jinv11-2.9586038961038961e-03*w45*Jinv00-5.9172077922077923e-03*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.8262987012987013e-03*Jinv11*w1+5.9172077922077923e-03*w13*Jinv11+1.8262987012987013e-03*w28*Jinv11-1.9724025974025973e-03*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29-2.9586038961038961e-03*w46*Jinv10+5.9172077922077923e-03*Jinv22*w17-1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12+3.9448051948051946e-03*w31*Jinv10+2.3011363636363637e-03*Jinv02*w3-1.9724025974025973e-03*w21*Jinv00+5.9172077922077923e-03*w32*Jinv22-1.8262987012987013e-03*Jinv22*w2+4.9310064935064934e-03*Jinv00*w18+4.9310064935064934e-03*w36*Jinv00-9.8620129870129864e-04*Jinv02*w45-2.3011363636363637e-03*Jinv01*w24+1.8262987012987013e-03*Jinv22*w59+5.9172077922077923e-03*w34*Jinv11+1.6436688311688311e-03*w19*Jinv11-9.8620129870129864e-04*w51*Jinv00-5.9172077922077923e-03*Jinv22*w44+1.3149350649350649e-03*w4*Jinv10+2.3011363636363637e-03*w4*Jinv11-3.5503246753246752e-02*w41*Jinv20+6.9034090909090911e-03*Jinv12*w13+4.9310064935064934e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+6.9034090909090911e-03*w32*Jinv21+1.6436688311688311e-03*Jinv21*w20+3.2873376623376621e-03*w6*Jinv00+1.6436688311688311e-03*w37*Jinv12-2.9586038961038961e-03*Jinv20*w56-9.8620129870129864e-04*w26*Jinv20-2.7759740259740259e-03*w11*Jinv20-2.3011363636363637e-03*w52*Jinv12+5.9172077922077923e-03*Jinv21*w35+6.9034090909090911e-03*w31*Jinv11-6.9034090909090911e-03*w46*Jinv11+3.9448051948051946e-03*Jinv20*w14+2.3011363636363637e-03*w5*Jinv21-6.9034090909090911e-03*Jinv02*w54+2.3011363636363637e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02-1.9724025974025973e-03*w49*Jinv10+1.8262987012987013e-03*Jinv21*w29+5.9172077922077923e-03*Jinv01*w33-3.5503246753246752e-02*w39*Jinv00+6.9034090909090911e-03*Jinv02*w12+1.6436688311688311e-03*Jinv01*w18-1.6436688311688311e-03*w6*Jinv01+3.2873376623376621e-03*w7*Jinv10-6.9034090909090911e-03*w47*Jinv21+4.9310064935064934e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+5.9172077922077923e-03*w31*Jinv12-9.8620129870129864e-04*w46*Jinv12-9.8620129870129864e-04*Jinv20*w53-1.6436688311688311e-03*Jinv22*w8-1.9724025974025973e-03*w22*Jinv10-5.9172077922077923e-03*Jinv22*w50-5.9172077922077923e-03*w22*Jinv11-1.6436688311688311e-03*Jinv12*w7-5.9172077922077923e-03*w21*Jinv01+5.9172077922077923e-03*Jinv02*w30-5.9172077922077923e-03*Jinv12*w49-9.8620129870129864e-04*w52*Jinv10-1.6436688311688311e-03*w7*Jinv11-2.7759740259740259e-03*w9*Jinv00+4.9310064935064934e-03*w37*Jinv10-9.8620129870129864e-04*Jinv21*w56-9.8620129870129864e-04*w24*Jinv00-1.6436688311688311e-03*w6*Jinv02+2.3011363636363637e-03*Jinv22*w5-9.8620129870129864e-04*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11+5.9172077922077923e-03*Jinv21*w14-2.7759740259740259e-03*w10*Jinv10-1.7751623376623376e-02*Jinv20*w44+1.8262987012987013e-03*w58*Jinv12+5.9172077922077923e-03*Jinv01*w12-1.6436688311688311e-03*Jinv21*w8-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-2.2646103896103898e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-5.9172077922077923e-03*Jinv21*w23-9.8620129870129864e-04*Jinv01*w54-9.8620129870129864e-04*w55*Jinv11-9.8620129870129864e-04*w25*Jinv10-2.2646103896103898e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-5.9172077922077923e-03*w48*Jinv02-1.1834415584415585e-02*Jinv20*w17+3.9448051948051946e-03*w30*Jinv00-3.5503246753246752e-02*w40*Jinv10+4.9310064935064934e-03*w20*Jinv20+6.9034090909090911e-03*Jinv22*w14+1.6436688311688311e-03*Jinv02*w36-5.9172077922077923e-03*Jinv12*w43+1.3149350649350649e-03*w5*Jinv20+6.9034090909090911e-03*Jinv01*w30-2.3011363636363637e-03*Jinv02*w51-6.9034090909090911e-03*Jinv22*w56-2.3011363636363637e-03*w25*Jinv11-2.9586038961038961e-03*w55*Jinv10-1.7751623376623376e-02*w42*Jinv00-2.9586038961038961e-03*w47*Jinv20+5.9172077922077923e-03*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+1.8262987012987013e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01-1.8262987012987013e-03*Jinv12*w1)*D;
  A[41][42]=(0.0)*D;
  A[41][43]=(0.0)*D;
  A[41][44]=(-3.9448051948051946e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-1.3149350649350649e-03*Jinv22*w53-3.2873376623376621e-03*Jinv20*w8+1.1323051948051949e-03*w57*Jinv02-3.9448051948051946e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+1.1834415584415585e-02*Jinv21*w44-1.3149350649350649e-03*w3*Jinv00-3.9448051948051946e-03*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+1.9724025974025973e-03*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38+2.9586038961038961e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+9.8620129870129864e-04*w25*Jinv12+1.1323051948051949e-03*w0*Jinv02+1.1323051948051949e-03*Jinv21*w2+1.7751623376623376e-02*w43*Jinv10+1.1834415584415585e-02*w42*Jinv01+1.1323051948051949e-03*w11*Jinv21+7.8896103896103891e-03*w16*Jinv11+7.8896103896103891e-03*Jinv21*w17-1.9724025974025973e-03*Jinv01*w45-1.9724025974025973e-03*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+1.1834415584415585e-02*w15*Jinv02+1.1323051948051949e-03*w10*Jinv12+1.9724025974025973e-03*w48*Jinv00+9.8620129870129864e-04*Jinv02*w24+7.8896103896103891e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01-1.3149350649350649e-03*Jinv21*w26+1.1323051948051949e-03*w0*Jinv01+2.2646103896103898e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10-1.9724025974025973e-03*Jinv22*w20+1.1834415584415585e-02*w43*Jinv11+2.9586038961038961e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+1.1834415584415585e-02*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+1.1323051948051949e-03*w11*Jinv22+1.1323051948051949e-03*Jinv11*w1-1.3149350649350649e-03*w13*Jinv11+7.8896103896103891e-03*w15*Jinv01+1.1323051948051949e-03*w28*Jinv11+1.9724025974025973e-03*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29+2.9586038961038961e-03*w46*Jinv10+1.1834415584415585e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12-3.9448051948051946e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3+1.9724025974025973e-03*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+1.1323051948051949e-03*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-1.9724025974025973e-03*Jinv21*w38+9.8620129870129864e-04*Jinv02*w45-1.3149350649350649e-03*Jinv01*w24+1.1323051948051949e-03*Jinv22*w59+1.1834415584415585e-02*w34*Jinv11-1.3149350649350649e-03*w19*Jinv11+9.8620129870129864e-04*Jinv21*w53+9.8620129870129864e-04*w51*Jinv00+7.8896103896103891e-03*Jinv12*w34+1.1834415584415585e-02*Jinv22*w44-1.3149350649350649e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+3.5503246753246752e-02*w41*Jinv20-1.9724025974025973e-03*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-1.9724025974025973e-03*w32*Jinv21-1.3149350649350649e-03*Jinv21*w20-3.2873376623376621e-03*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12+2.9586038961038961e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+9.8620129870129864e-04*w26*Jinv20+2.7759740259740259e-03*w11*Jinv20-1.3149350649350649e-03*w52*Jinv12+1.1834415584415585e-02*Jinv21*w35-1.9724025974025973e-03*w31*Jinv11+9.8620129870129864e-04*Jinv22*w26-1.9724025974025973e-03*w46*Jinv11-3.9448051948051946e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-1.9724025974025973e-03*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02+1.9724025974025973e-03*w49*Jinv10+1.1323051948051949e-03*Jinv21*w29+1.1834415584415585e-02*Jinv01*w33+3.5503246753246752e-02*w39*Jinv00-1.9724025974025973e-03*Jinv02*w12-1.3149350649350649e-03*Jinv01*w18-1.9724025974025973e-03*w37*Jinv11+1.1323051948051949e-03*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01-3.2873376623376621e-03*w7*Jinv10-1.9724025974025973e-03*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+9.8620129870129864e-04*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+9.8620129870129864e-04*w46*Jinv12+9.8620129870129864e-04*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8+1.9724025974025973e-03*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01+9.8620129870129864e-04*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49-1.9724025974025973e-03*w36*Jinv01+9.8620129870129864e-04*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11+2.7759740259740259e-03*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10+9.8620129870129864e-04*Jinv21*w56+9.8620129870129864e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-1.3149350649350649e-03*w6*Jinv02-1.9724025974025973e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5+9.8620129870129864e-04*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11-1.3149350649350649e-03*Jinv21*w14+2.7759740259740259e-03*w10*Jinv10+1.7751623376623376e-02*Jinv20*w44+1.1323051948051949e-03*w58*Jinv12-1.9724025974025973e-03*Jinv02*w18-1.3149350649350649e-03*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+7.8896103896103891e-03*Jinv02*w33+2.2646103896103898e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+9.8620129870129864e-04*Jinv01*w54+9.8620129870129864e-04*w55*Jinv11+9.8620129870129864e-04*w25*Jinv10+2.2646103896103898e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+1.1323051948051949e-03*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-3.9448051948051946e-03*w30*Jinv00+3.5503246753246752e-02*w40*Jinv10-4.9310064935064934e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29-1.9724025974025973e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11-1.3149350649350649e-03*Jinv02*w36+1.1834415584415585e-02*Jinv12*w43-1.3149350649350649e-03*w5*Jinv20-1.9724025974025973e-03*Jinv01*w30-1.3149350649350649e-03*Jinv02*w51-1.9724025974025973e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-1.3149350649350649e-03*w25*Jinv11+2.9586038961038961e-03*w55*Jinv10+1.7751623376623376e-02*w42*Jinv00+2.9586038961038961e-03*w47*Jinv20+1.1834415584415585e-02*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+1.1323051948051949e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01+1.1323051948051949e-03*Jinv12*w1+1.1323051948051949e-03*w10*Jinv11)*D;
  A[41][45]=(0.0)*D;
  A[41][46]=(0.0)*D;
  A[41][47]=(1.9724025974025973e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+4.9310064935064932e-04*w57*Jinv02+1.9724025974025973e-03*Jinv02*w39+6.9034090909090911e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-5.4788961038961042e-05*Jinv22*w38+4.3831168831168834e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12+4.9310064935064932e-04*w0*Jinv02+1.0592532467532468e-03*Jinv21*w2+6.9034090909090911e-03*w42*Jinv01+9.6793831168831166e-04*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+4.7666396103896100e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+2.7394480519480518e-04*w10*Jinv12+2.1367694805194807e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01+1.5888798701298701e-03*Jinv21*w26+1.0592532467532468e-03*w0*Jinv01-1.3149350649350649e-03*Jinv22*w20+6.9034090909090911e-03*w43*Jinv11+1.3806818181818182e-02*Jinv21*w41+9.8620129870129864e-04*w42*Jinv02+2.7394480519480518e-04*w11*Jinv22+1.0592532467532468e-03*Jinv11*w1-2.7394480519480520e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17+4.2735389610389614e-03*w22*Jinv12-3.8352272727272725e-04*Jinv02*w3-7.6704545454545450e-04*w32*Jinv22+1.3806818181818182e-02*Jinv01*w39+4.9310064935064932e-04*Jinv22*w2-1.3149350649350649e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+1.5888798701298701e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.5202922077922079e-03*w19*Jinv11-8.2183441558441553e-04*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+9.8620129870129864e-04*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-2.5202922077922079e-03*Jinv21*w20-5.4788961038961042e-05*w37*Jinv12+1.9724025974025973e-03*Jinv22*w41-3.8352272727272725e-04*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+4.7666396103896100e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-3.8352272727272725e-04*w4*Jinv12+4.2735389610389614e-03*w21*Jinv02-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-2.5202922077922079e-03*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+2.7394480519480518e-04*Jinv02*w9-8.7662337662337668e-04*w6*Jinv01+4.7666396103896100e-03*w47*Jinv21+4.2735389610389614e-03*Jinv22*w23-8.2183441558441553e-04*w52*Jinv11-7.6704545454545450e-04*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8-7.6704545454545450e-04*Jinv22*w50+3.1777597402597403e-03*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7+3.1777597402597403e-03*w21*Jinv01-8.2183441558441553e-04*Jinv01*w51-7.6704545454545450e-04*Jinv02*w30-7.6704545454545450e-04*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-8.7662337662337668e-04*w7*Jinv11-6.5746753246753243e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-3.8352272727272725e-04*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-1.6436688311688311e-03*w49*Jinv11-2.7394480519480520e-03*Jinv21*w14+4.9310064935064932e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18-2.7394480519480520e-03*Jinv01*w12-8.7662337662337668e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+3.1777597402597403e-03*Jinv21*w23-6.5746753246753243e-04*Jinv01*w54-6.5746753246753243e-04*w55*Jinv11-7.6704545454545450e-04*w48*Jinv02+9.6793831168831166e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+1.3806818181818182e-02*w40*Jinv11-5.4788961038961042e-05*Jinv02*w36+9.8620129870129864e-04*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30-3.8352272727272725e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+1.5888798701298701e-03*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12-1.6436688311688311e-03*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-1.6436688311688311e-03*w48*Jinv01+4.9310064935064932e-04*Jinv12*w1+9.6793831168831166e-04*w10*Jinv11)*D;
  A[41][48]=(0.0)*D;
  A[41][49]=(0.0)*D;
  A[41][50]=(7.6704545454545450e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-2.1915584415584417e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+3.8352272727272725e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20+3.8352272727272725e-04*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23-1.2053571428571428e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-3.2873376623376621e-04*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-1.0957792207792207e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+2.3011363636363637e-03*w15*Jinv02+6.9399350649350649e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00-3.2873376623376621e-04*Jinv02*w24+5.9172077922077923e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+3.8352272727272725e-04*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-8.2183441558441553e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+3.6160714285714286e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27+6.9399350649350649e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+7.6704545454545450e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+2.3011363636363637e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-1.3697240259740260e-03*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-4.3831168831168834e-04*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+1.4793019480519481e-03*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38-6.0267857142857139e-04*Jinv02*w45+3.8352272727272725e-04*Jinv01*w24-2.1915584415584417e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+5.4788961038961042e-05*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00+5.9172077922077923e-03*Jinv12*w34+3.6160714285714286e-03*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10+3.8352272727272725e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-8.7662337662337668e-04*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+5.4788961038961042e-05*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00-1.2053571428571428e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11-3.2873376623376621e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14+3.8352272727272725e-04*w5*Jinv21-1.0957792207792207e-03*Jinv02*w54-4.3831168831168834e-04*w4*Jinv12-1.3697240259740260e-03*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-8.7662337662337668e-04*Jinv02*w12+5.4788961038961042e-05*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11+6.9399350649350649e-04*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38-1.3697240259740260e-03*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11+1.4793019480519481e-03*w31*Jinv12-6.0267857142857139e-04*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53-1.2601461038961039e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50+7.6704545454545450e-04*w22*Jinv11-1.2601461038961039e-03*Jinv12*w7+7.6704545454545450e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.4793019480519481e-03*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00+5.6615259740259744e-04*Jinv02*w27-1.2601461038961039e-03*w6*Jinv02-8.2183441558441553e-04*w19*Jinv12-4.3831168831168834e-04*Jinv22*w5-6.0267857142857139e-04*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+7.6704545454545450e-04*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-2.1915584415584417e-04*w58*Jinv12-8.2183441558441553e-04*Jinv02*w18+7.6704545454545450e-04*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+5.9172077922077923e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+7.6704545454545450e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+4.9310064935064932e-04*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20+5.6615259740259744e-04*Jinv22*w29-8.7662337662337668e-04*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-1.2053571428571428e-03*Jinv02*w36+3.6160714285714286e-03*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-1.0957792207792207e-03*Jinv22*w56+5.6615259740259744e-04*Jinv12*w28+3.8352272727272725e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+2.3011363636363637e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[41][51]=(0.0)*D;
  A[41][52]=(0.0)*D;
  A[41][53]=(-7.6704545454545450e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-3.8352272727272725e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-7.6704545454545450e-04*Jinv20*w23-1.3697240259740260e-03*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-7.1225649350649346e-04*w25*Jinv12+5.2962662337662338e-04*w0*Jinv02+9.8620129870129864e-04*w43*Jinv10+1.5888798701298701e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+5.2962662337662338e-04*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-7.1225649350649346e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+4.6022727272727274e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+5.2962662337662338e-04*w11*Jinv22+4.2735389610389614e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-7.1225649350649346e-04*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-1.3697240259740260e-03*w32*Jinv22+5.2962662337662338e-04*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00-1.3697240259740260e-03*Jinv02*w45-3.8352272727272725e-04*Jinv22*w59+2.1367694805194807e-03*w51*Jinv00+4.6022727272727274e-03*Jinv12*w34+4.6022727272727274e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10+1.9724025974025973e-03*w41*Jinv20-7.1225649350649346e-04*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-1.3697240259740260e-03*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+1.5888798701298701e-03*w52*Jinv12-7.1225649350649346e-04*Jinv22*w26-7.6704545454545450e-04*Jinv20*w14+1.5888798701298701e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+1.9724025974025973e-03*w39*Jinv00-7.1225649350649346e-04*Jinv02*w12+5.2962662337662338e-04*Jinv02*w9-5.4788961038961042e-05*w7*Jinv10-1.3149350649350649e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-1.3697240259740260e-03*w31*Jinv12-1.3697240259740260e-03*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.5888798701298701e-03*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7-1.3697240259740260e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49+2.1367694805194807e-03*w52*Jinv10+2.7394480519480518e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10-3.8352272727272725e-04*w24*Jinv00+5.2962662337662338e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-1.3697240259740260e-03*w47*Jinv22+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-3.8352272727272725e-04*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+4.6022727272727274e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-3.8352272727272725e-04*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.5888798701298701e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+5.2962662337662338e-04*Jinv22*w29-7.1225649350649346e-04*Jinv22*w14-1.3697240259740260e-03*Jinv02*w36+4.6022727272727274e-03*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20+1.5888798701298701e-03*Jinv02*w51+1.5888798701298701e-03*Jinv22*w56+5.2962662337662338e-04*Jinv12*w28+4.2735389610389614e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+5.2962662337662338e-04*Jinv12*w1)*D;
  A[41][54]=(0.0)*D;
  A[41][55]=(0.0)*D;
  A[41][56]=(1.3806818181818182e-02*w40*Jinv12+1.5888798701298701e-03*Jinv22*w53-7.6704545454545450e-04*w57*Jinv02+1.3806818181818182e-02*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*Jinv01*w3-2.5202922077922079e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-8.2183441558441553e-04*w25*Jinv12+1.0592532467532468e-03*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w42*Jinv01+2.7394480519480518e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+4.7666396103896100e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+9.6793831168831166e-04*w10*Jinv12-8.2183441558441553e-04*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-3.8352272727272725e-04*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01-1.3149350649350649e-03*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11+1.9724025974025973e-03*Jinv21*w41+6.9034090909090911e-03*w42*Jinv02+9.6793831168831166e-04*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-7.6704545454545450e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17-1.6436688311688311e-03*w22*Jinv12-7.1225649350649346e-04*Jinv02*w3-2.7394480519480520e-03*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+1.0592532467532468e-03*Jinv22*w2-1.3149350649350649e-03*Jinv21*w38-6.5746753246753243e-04*Jinv02*w45-3.8352272727272725e-04*Jinv01*w24-7.6704545454545450e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-5.4788961038961042e-05*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+6.9034090909090911e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv11-2.1367694805194807e-03*Jinv12*w13-2.1367694805194807e-03*w32*Jinv21-5.4788961038961042e-05*Jinv21*w20-2.5202922077922079e-03*w37*Jinv12+1.3806818181818182e-02*Jinv22*w41+1.5888798701298701e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-8.2183441558441553e-04*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-3.8352272727272725e-04*w5*Jinv21+4.7666396103896100e-03*Jinv02*w54-7.1225649350649346e-04*w4*Jinv12-1.6436688311688311e-03*w21*Jinv02+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-2.1367694805194807e-03*Jinv02*w12-5.4788961038961042e-05*Jinv01*w18-1.3149350649350649e-03*w37*Jinv11+9.6793831168831166e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-2.1367694805194807e-03*w47*Jinv21-1.6436688311688311e-03*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11-2.7394480519480520e-03*w31*Jinv12-6.5746753246753243e-04*w46*Jinv12-8.7662337662337668e-04*Jinv22*w8+3.1777597402597403e-03*Jinv22*w50-7.6704545454545450e-04*w22*Jinv11-8.7662337662337668e-04*Jinv12*w7-7.6704545454545450e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-2.7394480519480520e-03*Jinv02*w30+3.1777597402597403e-03*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11+4.2735389610389614e-03*Jinv21*w56+4.3831168831168834e-04*Jinv02*w27-8.7662337662337668e-04*w6*Jinv02-1.3149350649350649e-03*w19*Jinv12-7.1225649350649346e-04*Jinv22*w5-6.5746753246753243e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-7.6704545454545450e-04*Jinv21*w14-7.6704545454545450e-04*w58*Jinv12-1.3149350649350649e-03*Jinv02*w18-7.6704545454545450e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-7.6704545454545450e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11+3.1777597402597403e-03*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+4.3831168831168834e-04*Jinv22*w29-2.1367694805194807e-03*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11-2.5202922077922079e-03*Jinv02*w36+6.9034090909090911e-03*Jinv12*w43-2.1367694805194807e-03*Jinv01*w30+1.5888798701298701e-03*Jinv02*w51+4.7666396103896100e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-3.8352272727272725e-04*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01+1.0592532467532468e-03*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[41][57]=(0.0)*D;
  A[41][58]=(0.0)*D;
  A[41][59]=(-3.6525974025974025e-03*w40*Jinv12+3.2873376623376621e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-3.6525974025974025e-03*Jinv02*w39+2.9220779220779221e-04*Jinv22*w38+2.1915584415584417e-04*w25*Jinv12-4.2613636363636367e-05*w0*Jinv02+6.5746753246753243e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-1.9480519480519481e-04*w10*Jinv12+2.1915584415584417e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35+3.4699675324675324e-04*Jinv22*w20-1.8262987012987013e-03*w42*Jinv02-1.9480519480519481e-04*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+4.3831168831168834e-04*w22*Jinv12+2.1915584415584417e-04*Jinv02*w3-3.2873376623376621e-04*w32*Jinv22-4.2613636363636367e-05*Jinv22*w2-3.2873376623376621e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-1.8262987012987013e-03*Jinv12*w34-1.8262987012987013e-03*Jinv22*w44+4.3831168831168834e-04*Jinv12*w13+2.9220779220779221e-04*w37*Jinv12-3.6525974025974025e-03*Jinv22*w41+3.2873376623376621e-04*w52*Jinv12+2.1915584415584417e-04*Jinv22*w26+6.5746753246753243e-04*Jinv02*w54+2.1915584415584417e-04*w4*Jinv12+4.3831168831168834e-04*w21*Jinv02+4.3831168831168834e-04*Jinv02*w12-1.9480519480519481e-04*Jinv02*w9+4.3831168831168834e-04*Jinv22*w23-3.2873376623376621e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+3.4699675324675324e-04*Jinv22*w8+6.5746753246753243e-04*Jinv22*w50+3.4699675324675324e-04*Jinv12*w7-3.2873376623376621e-04*Jinv02*w30+6.5746753246753243e-04*Jinv12*w49-4.2613636363636367e-05*Jinv02*w27+3.4699675324675324e-04*w6*Jinv02+3.4699675324675324e-04*w19*Jinv12+2.1915584415584417e-04*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+3.4699675324675324e-04*Jinv02*w18-1.8262987012987013e-03*Jinv02*w33+6.5746753246753243e-04*w48*Jinv02-4.2613636363636367e-05*Jinv22*w29+4.3831168831168834e-04*Jinv22*w14+2.9220779220779221e-04*Jinv02*w36-1.8262987012987013e-03*Jinv12*w43+3.2873376623376621e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-4.2613636363636367e-05*Jinv12*w1)*D;
  A[42][0]=(3.1047077922077924e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10+5.1136363636363640e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-2.1915584415584417e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+3.1047077922077924e-04*w13*Jinv10+5.1136363636363640e-04*Jinv02*w39-5.5519480519480519e-03*Jinv21*w44+3.1047077922077924e-04*w3*Jinv00+3.1047077922077924e-04*w32*Jinv20+3.1047077922077924e-04*Jinv01*w3-2.1915584415584417e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-3.0438311688311688e-05*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-2.4350649350649351e-04*w0*Jinv02-2.4350649350649351e-04*Jinv21*w2-5.5519480519480519e-03*w43*Jinv10-5.5519480519480519e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21+5.1136363636363640e-04*w16*Jinv11+5.1136363636363640e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10+5.1136363636363640e-04*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12-2.1915584415584417e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+5.1136363636363640e-04*Jinv22*w35-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-2.4350649350649351e-04*w0*Jinv01-2.4350649350649351e-04*w1*Jinv10+5.1136363636363640e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-5.5519480519480519e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+5.1136363636363640e-04*Jinv21*w41-5.5519480519480519e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-3.0438311688311688e-05*w11*Jinv22-2.4350649350649351e-04*Jinv11*w1+3.1047077922077924e-04*w13*Jinv11+5.1136363636363640e-04*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11-2.1915584415584417e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+5.1136363636363640e-04*Jinv22*w17+5.1136363636363640e-04*Jinv00*w33-2.1915584415584417e-04*w22*Jinv12+3.1047077922077924e-04*w31*Jinv10+3.1047077922077924e-04*Jinv02*w3-2.1915584415584417e-04*w21*Jinv00+3.1047077922077924e-04*w32*Jinv22+5.1136363636363640e-04*Jinv01*w39-2.4350649350649351e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+5.1136363636363640e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+5.1136363636363640e-04*Jinv12*w34-5.5519480519480519e-03*Jinv22*w44+3.1047077922077924e-04*w4*Jinv10+3.1047077922077924e-04*w4*Jinv11+5.1136363636363640e-04*w41*Jinv20+3.1047077922077924e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+5.1136363636363640e-04*w34*Jinv10+3.1047077922077924e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.1915584415584417e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+5.1136363636363640e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+5.1136363636363640e-04*Jinv21*w35+3.1047077922077924e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+3.1047077922077924e-04*Jinv20*w14+3.1047077922077924e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+3.1047077922077924e-04*w4*Jinv12-2.1915584415584417e-04*w21*Jinv02-2.1915584415584417e-04*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29+5.1136363636363640e-04*Jinv01*w33+5.1136363636363640e-04*w39*Jinv00+3.1047077922077924e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-3.0438311688311688e-05*Jinv02*w9-2.1915584415584417e-04*w6*Jinv01-2.1915584415584417e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-2.1915584415584417e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.1047077922077924e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-2.1915584415584417e-04*Jinv22*w8-2.1915584415584417e-04*w22*Jinv10-2.1915584415584417e-04*Jinv22*w50-2.1915584415584417e-04*w22*Jinv11-2.1915584415584417e-04*Jinv12*w7-2.1915584415584417e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+3.1047077922077924e-04*Jinv02*w30-2.1915584415584417e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-2.1915584415584417e-04*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-2.1915584415584417e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.1047077922077924e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-2.1915584415584417e-04*w49*Jinv11+3.1047077922077924e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-5.5519480519480519e-03*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+3.1047077922077924e-04*Jinv01*w12-2.1915584415584417e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59+5.1136363636363640e-04*Jinv02*w33-2.4350649350649351e-04*w0*Jinv00+5.1136363636363640e-04*w35*Jinv20-2.1915584415584417e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-2.4350649350649351e-04*Jinv20*w2+5.1136363636363640e-04*w15*Jinv00-2.1915584415584417e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9+5.1136363636363640e-04*Jinv20*w17+3.1047077922077924e-04*w30*Jinv00+5.1136363636363640e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+3.1047077922077924e-04*Jinv22*w14+5.1136363636363640e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36-5.5519480519480519e-03*Jinv12*w43+3.1047077922077924e-04*w5*Jinv20+3.1047077922077924e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-5.5519480519480519e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+5.1136363636363640e-04*w16*Jinv12-2.1915584415584417e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27-2.1915584415584417e-04*w48*Jinv01-2.4350649350649351e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[42][1]=(0.0)*D;
  A[42][2]=(0.0)*D;
  A[42][3]=(3.6525974025974026e-05*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8+7.3051948051948052e-05*w57*Jinv02-6.5746753246753243e-04*Jinv02*w39+9.8620129870129868e-03*Jinv21*w44-5.4788961038961036e-04*w3*Jinv00-1.6436688311688311e-03*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+6.7573051948051951e-04*w0*Jinv02+6.7573051948051951e-04*Jinv21*w2+6.5746753246753243e-03*w43*Jinv10+9.8620129870129868e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+6.7573051948051951e-04*w0*Jinv01+1.8262987012987014e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+9.8620129870129868e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+9.8620129870129868e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22+6.7573051948051951e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01+7.3051948051948052e-05*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-1.6436688311688311e-03*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39+6.7573051948051951e-04*Jinv22*w2+2.5750811688311690e-03*Jinv00*w18+2.5750811688311690e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+7.3051948051948052e-05*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+2.6298701298701297e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34+9.8620129870129868e-03*Jinv22*w44-5.4788961038961036e-04*w4*Jinv10-1.6436688311688311e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.0957792207792207e-03*Jinv12*w13+2.5750811688311690e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-1.0957792207792207e-03*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35-1.0957792207792207e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-1.6436688311688311e-03*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54-1.6436688311688311e-03*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+7.3051948051948052e-05*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-1.0957792207792207e-03*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+8.2183441558441553e-04*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+2.5750811688311690e-03*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+8.2183441558441553e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+8.2183441558441553e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+8.2183441558441553e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.5750811688311690e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+8.2183441558441553e-04*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.6436688311688311e-03*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+6.5746753246753243e-03*Jinv20*w44+7.3051948051948052e-05*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+8.2183441558441553e-04*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33+1.8262987012987014e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+1.8262987012987014e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10+2.5750811688311690e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-1.0957792207792207e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36+9.8620129870129868e-03*Jinv12*w43-5.4788961038961036e-04*w5*Jinv20-1.0957792207792207e-03*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+6.5746753246753243e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+7.3051948051948052e-05*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+6.7573051948051951e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[42][4]=(0.0)*D;
  A[42][5]=(0.0)*D;
  A[42][6]=(-6.5746753246753243e-04*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-5.4788961038961036e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3-3.2873376623376621e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-3.2873376623376621e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10+3.6160714285714286e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-3.2873376623376621e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+3.6160714285714286e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3-3.2873376623376621e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2-2.6846590909090908e-03*Jinv00*w18-2.6846590909090908e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+7.6704545454545450e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-2.6846590909090908e-03*w19*Jinv10+3.6160714285714286e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-5.4788961038961036e-04*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20+1.6436688311688311e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.6704545454545450e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02-3.2873376623376621e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-5.4788961038961036e-04*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-2.6846590909090908e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8-3.2873376623376621e-04*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+1.6436688311688311e-04*w9*Jinv00-2.6846590909090908e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.6436688311688311e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00+3.6160714285714286e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2+3.6160714285714286e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+3.6160714285714286e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-2.6846590909090908e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[42][7]=(0.0)*D;
  A[42][8]=(0.0)*D;
  A[42][9]=(3.4699675324675324e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10+3.2873376623376621e-04*Jinv20*w8+3.4699675324675324e-04*w13*Jinv10+2.9220779220779221e-04*w3*Jinv00+3.4699675324675324e-04*w32*Jinv20+2.1915584415584417e-04*Jinv20*w23+4.3831168831168834e-04*Jinv00*w54-3.6525974025974025e-03*w43*Jinv10-4.2613636363636367e-05*w58*Jinv10+2.1915584415584417e-04*w48*Jinv00-1.9480519480519481e-04*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10+4.3831168831168834e-04*w45*Jinv00-4.2613636363636367e-05*Jinv00*w27+2.1915584415584417e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+4.3831168831168834e-04*w46*Jinv10-1.8262987012987013e-03*Jinv00*w33+3.4699675324675324e-04*w31*Jinv10+2.1915584415584417e-04*w21*Jinv00+6.5746753246753243e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-3.2873376623376621e-04*w51*Jinv00+2.9220779220779221e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+3.2873376623376621e-04*w6*Jinv00+4.3831168831168834e-04*Jinv20*w56-3.2873376623376621e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+3.4699675324675324e-04*Jinv20*w14+2.1915584415584417e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+3.2873376623376621e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-3.2873376623376621e-04*Jinv20*w53+2.1915584415584417e-04*w22*Jinv10-3.2873376623376621e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10-3.2873376623376621e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-3.6525974025974025e-03*Jinv20*w44-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-1.9480519480519481e-04*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20-3.2873376623376621e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00-1.8262987012987013e-03*Jinv20*w17+3.4699675324675324e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20+2.9220779220779221e-04*w5*Jinv20+4.3831168831168834e-04*w55*Jinv10-3.6525974025974025e-03*w42*Jinv00+4.3831168831168834e-04*w47*Jinv20)*D;
  A[42][10]=(0.0)*D;
  A[42][11]=(0.0)*D;
  A[42][12]=(-1.6436688311688311e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8+7.3051948051948052e-05*w57*Jinv02-1.6436688311688311e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+6.5746753246753243e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.0957792207792207e-03*w32*Jinv20+8.2183441558441553e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+6.7573051948051951e-04*w0*Jinv02+1.8262987012987014e-04*Jinv21*w2+9.8620129870129868e-03*w43*Jinv10+6.5746753246753243e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17+2.5750811688311690e-03*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55+7.3051948051948052e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+2.5750811688311690e-03*Jinv21*w26+1.8262987012987014e-04*w0*Jinv01+6.7573051948051951e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+6.5746753246753243e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41+9.8620129870129868e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+7.3051948051948052e-05*w11*Jinv22+1.8262987012987014e-04*Jinv11*w1-5.4788961038961036e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+8.2183441558441553e-04*w22*Jinv12-1.0957792207792207e-03*w31*Jinv10-1.0957792207792207e-03*Jinv02*w3+8.2183441558441553e-04*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39+6.7573051948051951e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+2.5750811688311690e-03*Jinv01*w24+7.3051948051948052e-05*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+9.8620129870129868e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10-9.8620129870129864e-04*w41*Jinv20-1.6436688311688311e-03*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10+1.0957792207792208e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+7.3051948051948052e-05*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.6298701298701297e-03*Jinv22*w26+2.5750811688311690e-03*w46*Jinv11-1.6436688311688311e-03*Jinv20*w14+1.6436688311688311e-04*Jinv02*w54-1.0957792207792207e-03*w4*Jinv12+8.2183441558441553e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-1.6436688311688311e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+2.5750811688311690e-03*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38+8.2183441558441553e-04*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+8.2183441558441553e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+7.3051948051948052e-05*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-1.0957792207792207e-03*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-5.4788961038961036e-04*Jinv21*w14+7.3051948051948052e-05*w10*Jinv10+9.8620129870129868e-03*Jinv20*w44+7.3051948051948052e-05*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-5.4788961038961036e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33+6.7573051948051951e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10+6.7573051948051951e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17-1.0957792207792207e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-1.6436688311688311e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+9.8620129870129868e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+1.0957792207792208e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+2.5750811688311690e-03*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+9.8620129870129868e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01+6.7573051948051951e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[42][13]=(0.0)*D;
  A[42][14]=(0.0)*D;
  A[42][15]=(2.9586038961038961e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+3.9448051948051946e-03*Jinv22*w53-2.3011363636363637e-03*Jinv20*w8-2.2646103896103898e-03*w57*Jinv02+2.9586038961038961e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-2.3668831168831169e-02*Jinv21*w44+3.6160714285714286e-03*w3*Jinv00+1.9724025974025973e-03*w32*Jinv20+2.9586038961038961e-03*Jinv01*w3-1.9724025974025973e-03*Jinv22*w38+1.9724025974025973e-03*Jinv00*w54-1.1323051948051949e-03*Jinv21*w59-2.9586038961038961e-03*w25*Jinv12-2.7759740259740259e-03*w0*Jinv02-1.6436688311688311e-03*Jinv21*w2-2.3668831168831169e-02*w43*Jinv10-2.3668831168831169e-02*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17-3.2873376623376621e-03*Jinv01*w45+3.9448051948051946e-03*Jinv12*w55-1.1323051948051949e-03*w58*Jinv10-1.7751623376623376e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12-2.9586038961038961e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-1.1323051948051949e-03*w58*Jinv11-1.1323051948051949e-03*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26-1.6436688311688311e-03*w0*Jinv01-1.6436688311688311e-03*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10-2.9586038961038961e-03*Jinv22*w20-2.3668831168831169e-02*w43*Jinv11-9.8620129870129864e-04*w45*Jinv00-3.9448051948051946e-03*Jinv21*w41-3.5503246753246752e-02*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22-1.6436688311688311e-03*Jinv11*w1+3.6160714285714286e-03*w13*Jinv11-5.9172077922077923e-03*w15*Jinv01+6.9399350649350649e-04*w28*Jinv11-2.1915584415584417e-04*Jinv20*w29-9.8620129870129864e-04*w46*Jinv10-1.7751623376623376e-02*Jinv22*w17-3.9448051948051946e-03*Jinv00*w33-9.8620129870129864e-04*w22*Jinv12+1.9724025974025973e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3+3.2873376623376621e-03*w32*Jinv22-3.9448051948051946e-03*Jinv01*w39-2.7759740259740259e-03*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-3.2873376623376621e-03*w36*Jinv00-9.8620129870129864e-04*Jinv21*w38-1.9724025974025973e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24-2.2646103896103898e-03*Jinv22*w59-1.9724025974025973e-03*w19*Jinv11+1.9724025974025973e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-3.5503246753246752e-02*Jinv22*w44+3.6160714285714286e-03*w4*Jinv10+2.9586038961038961e-03*w4*Jinv11+4.9310064935064934e-03*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10-3.9448051948051946e-03*w34*Jinv10+1.9724025974025973e-03*w32*Jinv21-1.9724025974025973e-03*Jinv21*w20-2.3011363636363637e-03*w6*Jinv00-1.9724025974025973e-03*w37*Jinv12+1.9724025974025973e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41-1.9724025974025973e-03*w26*Jinv20+6.9399350649350649e-04*w11*Jinv20+3.9448051948051946e-03*w52*Jinv12+1.9724025974025973e-03*w31*Jinv11-2.9586038961038961e-03*Jinv22*w26-3.2873376623376621e-03*w46*Jinv11+2.9586038961038961e-03*Jinv20*w14+2.9586038961038961e-03*w5*Jinv21+3.9448051948051946e-03*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-9.8620129870129864e-04*w21*Jinv02+6.9399350649350649e-04*Jinv21*w29+4.9310064935064934e-03*Jinv02*w12-1.9724025974025973e-03*Jinv01*w18-9.8620129870129864e-04*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9-2.3011363636363637e-03*w7*Jinv10-3.2873376623376621e-03*w47*Jinv21-3.2873376623376621e-03*Jinv20*w38-9.8620129870129864e-04*Jinv22*w23+1.9724025974025973e-03*w52*Jinv11+3.2873376623376621e-03*w31*Jinv12-1.9724025974025973e-03*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53-9.8620129870129864e-04*Jinv22*w8+1.3149350649350649e-03*Jinv22*w50-2.3011363636363637e-03*w22*Jinv11-9.8620129870129864e-04*Jinv12*w7-2.3011363636363637e-03*w21*Jinv01+1.9724025974025973e-03*Jinv01*w51+3.2873376623376621e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-9.8620129870129864e-04*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+6.9399350649350649e-04*w9*Jinv00-3.2873376623376621e-03*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56-1.9724025974025973e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27-9.8620129870129864e-04*w6*Jinv02-2.9586038961038961e-03*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5-1.9724025974025973e-03*w47*Jinv22+3.6160714285714286e-03*Jinv21*w14+6.9399350649350649e-04*w10*Jinv10-2.3668831168831169e-02*Jinv20*w44-2.2646103896103898e-03*w58*Jinv12-2.9586038961038961e-03*Jinv02*w18+3.6160714285714286e-03*Jinv01*w12-1.1323051948051949e-03*w57*Jinv00-1.1323051948051949e-03*Jinv20*w59-1.1834415584415585e-02*Jinv02*w33-1.6436688311688311e-03*w0*Jinv00-3.9448051948051946e-03*w35*Jinv20-2.3011363636363637e-03*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11-1.9724025974025973e-03*w25*Jinv10-1.6436688311688311e-03*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17+1.9724025974025973e-03*w30*Jinv00-4.9310064935064934e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14-3.9448051948051946e-03*w40*Jinv11-1.9724025974025973e-03*Jinv02*w36-3.5503246753246752e-02*Jinv12*w43+3.6160714285714286e-03*w5*Jinv20+1.9724025974025973e-03*Jinv01*w30+3.9448051948051946e-03*Jinv02*w51+3.9448051948051946e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+1.9724025974025973e-03*w55*Jinv10-2.3668831168831169e-02*w42*Jinv00-9.8620129870129864e-04*w47*Jinv20-1.7751623376623376e-02*w16*Jinv12+6.9399350649350649e-04*Jinv01*w27-2.7759740259740259e-03*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[42][16]=(0.0)*D;
  A[42][17]=(0.0)*D;
  A[42][18]=(-1.3149350649350649e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.5888798701298701e-03*Jinv20*w8-1.3149350649350649e-03*w13*Jinv10+1.9724025974025973e-03*Jinv21*w44-2.5202922077922079e-03*w3*Jinv00-8.7662337662337668e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3-8.2183441558441553e-04*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59+2.7394480519480518e-04*Jinv21*w2+1.3806818181818182e-02*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45+1.0592532467532468e-03*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01+9.6793831168831166e-04*w1*Jinv10+6.9034090909090911e-03*w16*Jinv10+1.9724025974025973e-03*w43*Jinv11-1.6436688311688311e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+4.3831168831168834e-04*Jinv00*w27+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-1.6436688311688311e-03*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-8.7662337662337668e-04*w31*Jinv10-8.2183441558441553e-04*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+4.7666396103896100e-03*Jinv00*w18+3.1777597402597403e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-2.1367694805194807e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-2.7394480519480520e-03*w51*Jinv00-2.5202922077922079e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+4.7666396103896100e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.5888798701298701e-03*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56-6.5746753246753243e-04*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-7.6704545454545450e-04*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+4.2735389610389614e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11+2.1367694805194807e-03*w6*Jinv01+1.5888798701298701e-03*w7*Jinv10-7.6704545454545450e-04*w47*Jinv21+3.1777597402597403e-03*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-2.7394480519480520e-03*Jinv20*w53-8.2183441558441553e-04*w22*Jinv10-3.8352272727272725e-04*w22*Jinv11-3.8352272727272725e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+4.2735389610389614e-03*w36*Jinv01-2.7394480519480520e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-7.6704545454545450e-04*w9*Jinv00+3.1777597402597403e-03*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56-6.5746753246753243e-04*w24*Jinv00-3.8352272727272725e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-7.6704545454545450e-04*w10*Jinv10+1.3806818181818182e-02*Jinv20*w44-5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+1.0592532467532468e-03*w57*Jinv00+1.0592532467532468e-03*Jinv20*w59+9.6793831168831166e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11-6.5746753246753243e-04*w25*Jinv10+9.6793831168831166e-04*Jinv20*w2+6.9034090909090911e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+6.9034090909090911e-03*Jinv20*w17-8.7662337662337668e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+4.7666396103896100e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-2.5202922077922079e-03*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-2.1367694805194807e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.3806818181818182e-02*w42*Jinv00-1.6436688311688311e-03*w47*Jinv20-3.8352272727272725e-04*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[42][19]=(0.0)*D;
  A[42][20]=(0.0)*D;
  A[42][21]=(1.3149350649350649e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+3.6160714285714286e-03*w16*Jinv11+3.6160714285714286e-03*Jinv21*w17-2.6846590909090908e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01-2.6846590909090908e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00+3.6160714285714286e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+3.6160714285714286e-03*w15*Jinv01+1.6436688311688311e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+3.6160714285714286e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-2.6846590909090908e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-2.6846590909090908e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+1.6436688311688311e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9-3.2873376623376621e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10-2.6846590909090908e-03*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-5.4788961038961036e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-5.4788961038961036e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10-3.2873376623376621e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22-3.2873376623376621e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.2873376623376621e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-5.4788961038961036e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14+3.6160714285714286e-03*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30+7.6704545454545450e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-2.6846590909090908e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-3.2873376623376621e-04*Jinv21*w50+1.6436688311688311e-04*Jinv01*w27-3.2873376623376621e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[42][22]=(0.0)*D;
  A[42][23]=(0.0)*D;
  A[42][24]=(-1.3149350649350649e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-3.8352272727272725e-04*Jinv20*w8-1.3149350649350649e-03*w13*Jinv10+1.3806818181818182e-02*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+1.0592532467532468e-03*Jinv21*w59+9.6793831168831166e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.3806818181818182e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+6.9034090909090911e-03*w16*Jinv11+6.9034090909090911e-03*Jinv21*w17+3.1777597402597403e-03*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-3.8352272727272725e-04*w48*Jinv00+1.0592532467532468e-03*w58*Jinv11+1.0592532467532468e-03*w57*Jinv01+4.7666396103896100e-03*Jinv21*w26+9.6793831168831166e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+1.3806818181818182e-02*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+9.6793831168831166e-04*Jinv11*w1-2.5202922077922079e-03*w13*Jinv11+6.9034090909090911e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10+2.1367694805194807e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-2.1367694805194807e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-1.6436688311688311e-03*Jinv21*w38+4.7666396103896100e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11-6.5746753246753243e-04*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-8.7662337662337668e-04*w32*Jinv21-6.5746753246753243e-04*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-8.7662337662337668e-04*w31*Jinv11+3.1777597402597403e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-3.8352272727272725e-04*w49*Jinv10-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv01*w18-1.6436688311688311e-03*w37*Jinv11-8.2183441558441553e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+3.1777597402597403e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-7.6704545454545450e-04*Jinv20*w53+2.1367694805194807e-03*w22*Jinv10+1.5888798701298701e-03*w22*Jinv11+1.5888798701298701e-03*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-1.6436688311688311e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10-8.2183441558441553e-04*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-2.7394480519480520e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-2.5202922077922079e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-2.5202922077922079e-03*Jinv01*w12-8.2183441558441553e-04*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+2.7394480519480518e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.5888798701298701e-03*Jinv21*w23-2.7394480519480520e-03*Jinv01*w54-2.7394480519480520e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+4.3831168831168834e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-5.4788961038961042e-05*w5*Jinv20-8.7662337662337668e-04*Jinv01*w30+4.7666396103896100e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+4.3831168831168834e-04*w10*Jinv11)*D;
  A[42][25]=(0.0)*D;
  A[42][26]=(0.0)*D;
  A[42][27]=(-3.6525974025974025e-03*Jinv21*w44+3.4699675324675324e-04*Jinv01*w3-4.2613636363636367e-05*Jinv21*w59-1.9480519480519481e-04*Jinv21*w2-3.6525974025974025e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-3.6525974025974025e-03*w43*Jinv11-1.8262987012987013e-03*Jinv21*w41-1.9480519480519481e-04*Jinv11*w1+2.9220779220779221e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.8262987012987013e-03*Jinv01*w39+4.3831168831168834e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11+4.3831168831168834e-04*Jinv21*w53+3.4699675324675324e-04*w4*Jinv11+3.4699675324675324e-04*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+3.4699675324675324e-04*w31*Jinv11+6.5746753246753243e-04*w46*Jinv11+3.4699675324675324e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33-3.2873376623376621e-04*Jinv01*w18+4.3831168831168834e-04*w37*Jinv11+2.1915584415584417e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+4.3831168831168834e-04*w52*Jinv11+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*w21*Jinv01+4.3831168831168834e-04*Jinv01*w51+4.3831168831168834e-04*w36*Jinv01+2.1915584415584417e-04*w7*Jinv11-3.2873376623376621e-04*Jinv21*w56+2.1915584415584417e-04*w49*Jinv11+2.9220779220779221e-04*Jinv21*w14+2.9220779220779221e-04*Jinv01*w12+2.1915584415584417e-04*Jinv21*w8+3.2873376623376621e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*w40*Jinv11+3.4699675324675324e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11+2.1915584415584417e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+2.1915584415584417e-04*w48*Jinv01-4.2613636363636367e-05*w10*Jinv11)*D;
  A[42][28]=(0.0)*D;
  A[42][29]=(0.0)*D;
  A[42][30]=(-1.0957792207792207e-03*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12+2.5750811688311690e-03*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-1.0957792207792207e-03*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39+9.8620129870129868e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.6436688311688311e-03*w32*Jinv20-1.0957792207792207e-03*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+1.8262987012987014e-04*w0*Jinv02+6.7573051948051951e-04*Jinv21*w2+9.8620129870129868e-03*w43*Jinv10+9.8620129870129868e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+2.5750811688311690e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+8.2183441558441553e-04*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26+6.7573051948051951e-04*w0*Jinv01+6.7573051948051951e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+9.8620129870129868e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+6.5746753246753243e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+6.7573051948051951e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01+7.3051948051948052e-05*w28*Jinv11+8.2183441558441553e-04*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-1.6436688311688311e-03*w31*Jinv10+7.1225649350649346e-04*w21*Jinv00-5.4788961038961036e-04*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39+1.8262987012987014e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34+6.5746753246753243e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10-1.0957792207792207e-03*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+1.0957792207792208e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-1.6436688311688311e-03*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20+7.3051948051948052e-05*w11*Jinv20+2.5750811688311690e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35-1.6436688311688311e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-1.0957792207792207e-03*Jinv20*w14-1.0957792207792207e-03*w5*Jinv21+2.5750811688311690e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02+8.2183441558441553e-04*w49*Jinv10+7.3051948051948052e-05*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+1.0957792207792208e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11-5.4788961038961036e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-5.4788961038961036e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+7.3051948051948052e-05*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12+1.0957792207792208e-04*w47*Jinv22+8.2183441558441553e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+7.3051948051948052e-05*w10*Jinv10+9.8620129870129868e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33+6.7573051948051951e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+6.7573051948051951e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02+7.3051948051948052e-05*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.6436688311688311e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+6.5746753246753243e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20-1.6436688311688311e-03*Jinv01*w30+2.5750811688311690e-03*Jinv02*w51+2.5750811688311690e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+9.8620129870129868e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+8.2183441558441553e-04*Jinv21*w50+7.3051948051948052e-05*Jinv01*w27+8.2183441558441553e-04*w48*Jinv01+1.8262987012987014e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[42][31]=(0.0)*D;
  A[42][32]=(0.0)*D;
  A[42][33]=(1.9724025974025973e-03*Jinv00*w12-1.1323051948051949e-03*w28*Jinv10-3.9448051948051946e-03*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53-2.3011363636363637e-03*Jinv20*w8+6.9399350649350649e-04*w57*Jinv02+1.9724025974025973e-03*w13*Jinv10-3.9448051948051946e-03*Jinv02*w39-3.5503246753246752e-02*Jinv21*w44+3.6160714285714286e-03*w3*Jinv00+2.9586038961038961e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3-1.9724025974025973e-03*Jinv22*w38-9.8620129870129864e-04*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+1.9724025974025973e-03*w25*Jinv12-1.6436688311688311e-03*w0*Jinv02-2.7759740259740259e-03*Jinv21*w2-2.3668831168831169e-02*w43*Jinv10-3.5503246753246752e-02*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+3.9448051948051946e-03*Jinv01*w45-3.2873376623376621e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10-2.1915584415584417e-04*w10*Jinv12+1.9724025974025973e-03*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+3.9448051948051946e-03*Jinv21*w26-2.7759740259740259e-03*w0*Jinv01-1.6436688311688311e-03*w1*Jinv10-3.9448051948051946e-03*w16*Jinv10-9.8620129870129864e-04*Jinv22*w20-3.5503246753246752e-02*w43*Jinv11+1.9724025974025973e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-2.3668831168831169e-02*w42*Jinv02-1.1323051948051949e-03*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-2.7759740259740259e-03*Jinv11*w1+3.2873376623376621e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-2.2646103896103898e-03*w28*Jinv11-1.1323051948051949e-03*Jinv20*w29+1.9724025974025973e-03*w46*Jinv10-5.9172077922077923e-03*Jinv00*w33+2.9586038961038961e-03*w31*Jinv10+2.9586038961038961e-03*Jinv02*w3+3.6160714285714286e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-1.6436688311688311e-03*Jinv22*w2-3.2873376623376621e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-2.9586038961038961e-03*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+3.9448051948051946e-03*Jinv01*w24+6.9399350649350649e-04*Jinv22*w59-1.7751623376623376e-02*w34*Jinv11-1.9724025974025973e-03*w19*Jinv11-2.9586038961038961e-03*Jinv21*w53-1.9724025974025973e-03*w51*Jinv00-5.9172077922077923e-03*Jinv12*w34-2.3668831168831169e-02*Jinv22*w44+3.6160714285714286e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11+1.9724025974025973e-03*Jinv12*w13-3.2873376623376621e-03*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21-1.9724025974025973e-03*Jinv21*w20-2.3011363636363637e-03*w6*Jinv00-1.9724025974025973e-03*w37*Jinv12-9.8620129870129864e-04*Jinv20*w56-3.9448051948051946e-03*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+6.9399350649350649e-04*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12-1.7751623376623376e-02*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11+1.9724025974025973e-03*Jinv22*w26+3.9448051948051946e-03*w46*Jinv11+1.9724025974025973e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21-3.2873376623376621e-03*Jinv02*w54+2.9586038961038961e-03*w4*Jinv12-2.2646103896103898e-03*Jinv21*w29-1.7751623376623376e-02*Jinv01*w33+1.9724025974025973e-03*Jinv02*w12-1.9724025974025973e-03*Jinv01*w18-2.9586038961038961e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-9.8620129870129864e-04*w6*Jinv01-2.3011363636363637e-03*w7*Jinv10+3.9448051948051946e-03*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38-2.9586038961038961e-03*w52*Jinv11+3.6160714285714286e-03*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12-1.9724025974025973e-03*Jinv20*w53-2.3011363636363637e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11+1.3149350649350649e-03*w21*Jinv01-2.9586038961038961e-03*Jinv01*w51+3.6160714285714286e-03*Jinv02*w30-2.3011363636363637e-03*Jinv12*w49-2.9586038961038961e-03*w36*Jinv01-1.9724025974025973e-03*w52*Jinv10-9.8620129870129864e-04*w7*Jinv11+6.9399350649350649e-04*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10-1.9724025974025973e-03*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00-1.1323051948051949e-03*Jinv02*w27-9.8620129870129864e-04*w19*Jinv12+2.9586038961038961e-03*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22-9.8620129870129864e-04*w49*Jinv11+3.2873376623376621e-03*Jinv21*w14+6.9399350649350649e-04*w10*Jinv10-2.3668831168831169e-02*Jinv20*w44+6.9399350649350649e-04*w58*Jinv12-9.8620129870129864e-04*Jinv02*w18+3.2873376623376621e-03*Jinv01*w12-9.8620129870129864e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33-1.6436688311688311e-03*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-1.9724025974025973e-03*Jinv01*w54-1.9724025974025973e-03*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10-1.6436688311688311e-03*Jinv20*w2-3.9448051948051946e-03*w15*Jinv00-2.3011363636363637e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-3.9448051948051946e-03*Jinv20*w17+2.9586038961038961e-03*w30*Jinv00-3.2873376623376621e-03*w20*Jinv20-1.1323051948051949e-03*Jinv22*w29+1.9724025974025973e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11-1.9724025974025973e-03*Jinv02*w36-2.3668831168831169e-02*Jinv12*w43+3.6160714285714286e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-3.2873376623376621e-03*Jinv22*w56-1.1323051948051949e-03*Jinv12*w28+3.9448051948051946e-03*w25*Jinv11-9.8620129870129864e-04*w55*Jinv10-2.3668831168831169e-02*w42*Jinv00+1.9724025974025973e-03*w47*Jinv20-9.8620129870129864e-04*Jinv21*w50-2.2646103896103898e-03*Jinv01*w27-9.8620129870129864e-04*w48*Jinv01-1.6436688311688311e-03*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[42][34]=(0.0)*D;
  A[42][35]=(0.0)*D;
  A[42][36]=(-8.7662337662337668e-04*Jinv00*w12+1.0592532467532468e-03*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+1.5888798701298701e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-8.7662337662337668e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-2.5202922077922079e-03*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-1.6436688311688311e-03*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+1.3806818181818182e-02*w43*Jinv10-7.6704545454545450e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-8.2183441558441553e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+9.6793831168831166e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+1.9724025974025973e-03*w42*Jinv02+1.0592532467532468e-03*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-8.2183441558441553e-04*Jinv20*w50+1.0592532467532468e-03*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+6.9034090909090911e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+2.7394480519480518e-04*Jinv22*w2+3.1777597402597403e-03*Jinv00*w18+4.7666396103896100e-03*w36*Jinv00-7.6704545454545450e-04*Jinv02*w45+4.9310064935064932e-04*Jinv22*w59-6.5746753246753243e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-2.5202922077922079e-03*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+3.1777597402597403e-03*w19*Jinv10+6.9034090909090911e-03*w34*Jinv10+1.5888798701298701e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-1.6436688311688311e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-2.7394480519480520e-03*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-8.7662337662337668e-04*Jinv20*w14-7.6704545454545450e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-8.2183441558441553e-04*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+1.5888798701298701e-03*w7*Jinv10+4.7666396103896100e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12-6.5746753246753243e-04*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-6.5746753246753243e-04*w52*Jinv10-7.6704545454545450e-04*w9*Jinv00+4.7666396103896100e-03*w37*Jinv10-2.7394480519480520e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22-7.6704545454545450e-04*w10*Jinv10+1.3806818181818182e-02*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33+9.6793831168831166e-04*w0*Jinv00+6.9034090909090911e-03*w35*Jinv20-2.7394480519480520e-03*w25*Jinv10+9.6793831168831166e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+3.1777597402597403e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+4.2735389610389614e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-2.5202922077922079e-03*w5*Jinv20-2.1367694805194807e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-1.6436688311688311e-03*w55*Jinv10+1.3806818181818182e-02*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+2.7394480519480518e-04*Jinv12*w1)*D;
  A[42][37]=(0.0)*D;
  A[42][38]=(0.0)*D;
  A[42][39]=(4.9310064935064934e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12-3.2873376623376621e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+6.9399350649350649e-04*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39-2.3668831168831169e-02*Jinv21*w44+3.2873376623376621e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20+1.9724025974025973e-03*Jinv01*w3-9.8620129870129864e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38-2.9586038961038961e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-9.8620129870129864e-04*w25*Jinv12-1.6436688311688311e-03*w0*Jinv02-1.6436688311688311e-03*Jinv21*w2-3.5503246753246752e-02*w43*Jinv10-2.3668831168831169e-02*w42*Jinv01-1.1323051948051949e-03*w11*Jinv21-3.9448051948051946e-03*w16*Jinv11-3.9448051948051946e-03*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-1.1323051948051949e-03*w10*Jinv12-9.8620129870129864e-04*w48*Jinv00-9.8620129870129864e-04*Jinv02*w24-3.9448051948051946e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-3.2873376623376621e-03*Jinv21*w26-1.6436688311688311e-03*w0*Jinv01-2.7759740259740259e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10+1.9724025974025973e-03*Jinv22*w20-2.3668831168831169e-02*w43*Jinv11-2.9586038961038961e-03*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41-2.3668831168831169e-02*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.1323051948051949e-03*w11*Jinv22-1.6436688311688311e-03*Jinv11*w1+3.6160714285714286e-03*w13*Jinv11-3.9448051948051946e-03*w15*Jinv01+6.9399350649350649e-04*w28*Jinv11-9.8620129870129864e-04*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29-2.9586038961038961e-03*w46*Jinv10-1.1834415584415585e-02*Jinv00*w33+4.9310064935064934e-03*w31*Jinv10+1.9724025974025973e-03*Jinv02*w3-9.8620129870129864e-04*w21*Jinv00+3.6160714285714286e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39-1.6436688311688311e-03*Jinv22*w2+3.9448051948051946e-03*Jinv00*w18+3.9448051948051946e-03*w36*Jinv00+1.9724025974025973e-03*Jinv21*w38-1.9724025974025973e-03*Jinv02*w45-3.2873376623376621e-03*Jinv01*w24+6.9399350649350649e-04*Jinv22*w59+2.6298701298701297e-03*w19*Jinv11-9.8620129870129864e-04*Jinv21*w53-1.9724025974025973e-03*w51*Jinv00-3.9448051948051946e-03*Jinv12*w34-2.3668831168831169e-02*Jinv22*w44+3.2873376623376621e-03*w4*Jinv10+1.9724025974025973e-03*w4*Jinv11-1.7751623376623376e-02*w41*Jinv20+2.9586038961038961e-03*Jinv12*w13+3.9448051948051946e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+2.9586038961038961e-03*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12-2.9586038961038961e-03*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41-1.9724025974025973e-03*w26*Jinv20-2.2646103896103898e-03*w11*Jinv20-3.2873376623376621e-03*w52*Jinv12+2.9586038961038961e-03*w31*Jinv11-9.8620129870129864e-04*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14+1.9724025974025973e-03*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54+1.9724025974025973e-03*w4*Jinv12-9.8620129870129864e-04*w49*Jinv10+6.9399350649350649e-04*Jinv21*w29-1.7751623376623376e-02*w39*Jinv00+2.9586038961038961e-03*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+1.9724025974025973e-03*w37*Jinv11-1.1323051948051949e-03*Jinv02*w9+1.3149350649350649e-03*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21+3.9448051948051946e-03*Jinv20*w38-9.8620129870129864e-04*w52*Jinv11+3.6160714285714286e-03*w31*Jinv12-1.9724025974025973e-03*w46*Jinv12-1.9724025974025973e-03*Jinv20*w53-9.8620129870129864e-04*w22*Jinv10-2.3011363636363637e-03*Jinv22*w50-2.3011363636363637e-03*w22*Jinv11-2.3011363636363637e-03*w21*Jinv01-9.8620129870129864e-04*Jinv01*w51+3.6160714285714286e-03*Jinv02*w30-2.3011363636363637e-03*Jinv12*w49+1.9724025974025973e-03*w36*Jinv01-1.9724025974025973e-03*w52*Jinv10-2.2646103896103898e-03*w9*Jinv00+3.9448051948051946e-03*w37*Jinv10-1.9724025974025973e-03*Jinv21*w56-1.9724025974025973e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+1.9724025974025973e-03*w19*Jinv12+1.9724025974025973e-03*Jinv22*w5-1.9724025974025973e-03*w47*Jinv22+3.6160714285714286e-03*Jinv21*w14-2.2646103896103898e-03*w10*Jinv10-3.5503246753246752e-02*Jinv20*w44+6.9399350649350649e-04*w58*Jinv12+1.9724025974025973e-03*Jinv02*w18+3.6160714285714286e-03*Jinv01*w12-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-3.9448051948051946e-03*Jinv02*w33-2.7759740259740259e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-2.3011363636363637e-03*Jinv21*w23-1.9724025974025973e-03*Jinv01*w54-1.9724025974025973e-03*w55*Jinv11-1.9724025974025973e-03*w25*Jinv10-2.7759740259740259e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-2.3011363636363637e-03*w48*Jinv02-1.1323051948051949e-03*Jinv01*w9-1.1834415584415585e-02*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00-1.7751623376623376e-02*w40*Jinv10+3.9448051948051946e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+2.9586038961038961e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36-2.3668831168831169e-02*Jinv12*w43+3.2873376623376621e-03*w5*Jinv20+2.9586038961038961e-03*Jinv01*w30-3.2873376623376621e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-3.2873376623376621e-03*w25*Jinv11-2.9586038961038961e-03*w55*Jinv10-3.5503246753246752e-02*w42*Jinv00-2.9586038961038961e-03*w47*Jinv20+6.9399350649350649e-04*Jinv01*w27-1.6436688311688311e-03*Jinv12*w1-1.1323051948051949e-03*w10*Jinv11)*D;
  A[42][40]=(0.0)*D;
  A[42][41]=(0.0)*D;
  A[42][42]=(-4.9310064935064934e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-3.9448051948051946e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+2.2646103896103898e-03*w57*Jinv02-4.9310064935064934e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+3.5503246753246752e-02*Jinv21*w44-3.2873376623376621e-03*w3*Jinv00-4.9310064935064934e-03*w32*Jinv20-4.9310064935064934e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23+1.9724025974025973e-03*Jinv22*w38+2.9586038961038961e-03*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59+2.9586038961038961e-03*w25*Jinv12+2.7759740259740259e-03*w0*Jinv02+2.7759740259740259e-03*Jinv21*w2+3.5503246753246752e-02*w43*Jinv10+3.5503246753246752e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-3.9448051948051946e-03*Jinv01*w45-3.9448051948051946e-03*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+1.7751623376623376e-02*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+9.8620129870129864e-04*w48*Jinv00+2.9586038961038961e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-3.9448051948051946e-03*Jinv21*w26+2.7759740259740259e-03*w0*Jinv01+2.7759740259740259e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10+2.9586038961038961e-03*Jinv22*w20+3.5503246753246752e-02*w43*Jinv11+2.9586038961038961e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+3.5503246753246752e-02*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+2.7759740259740259e-03*Jinv11*w1-3.2873376623376621e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+2.2646103896103898e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29+2.9586038961038961e-03*w46*Jinv10+1.7751623376623376e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12-4.9310064935064934e-03*w31*Jinv10-4.9310064935064934e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00-3.2873376623376621e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+2.7759740259740259e-03*Jinv22*w2-3.9448051948051946e-03*Jinv00*w18-3.9448051948051946e-03*w36*Jinv00+2.9586038961038961e-03*Jinv21*w38+1.9724025974025973e-03*Jinv02*w45-3.9448051948051946e-03*Jinv01*w24+2.2646103896103898e-03*Jinv22*w59+1.7751623376623376e-02*w34*Jinv11+1.9724025974025973e-03*w19*Jinv11+2.9586038961038961e-03*Jinv21*w53+1.9724025974025973e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+3.5503246753246752e-02*Jinv22*w44-3.2873376623376621e-03*w4*Jinv10-4.9310064935064934e-03*w4*Jinv11+1.7751623376623376e-02*w41*Jinv20-4.9310064935064934e-03*Jinv12*w13-3.9448051948051946e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-4.9310064935064934e-03*w32*Jinv21+1.9724025974025973e-03*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+1.9724025974025973e-03*w37*Jinv12+2.9586038961038961e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+1.9724025974025973e-03*w26*Jinv20+2.2646103896103898e-03*w11*Jinv20-3.9448051948051946e-03*w52*Jinv12+1.7751623376623376e-02*Jinv21*w35-4.9310064935064934e-03*w31*Jinv11+2.9586038961038961e-03*Jinv22*w26-3.9448051948051946e-03*w46*Jinv11-4.9310064935064934e-03*Jinv20*w14-4.9310064935064934e-03*w5*Jinv21-3.9448051948051946e-03*Jinv02*w54-4.9310064935064934e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10+2.2646103896103898e-03*Jinv21*w29+1.7751623376623376e-02*Jinv01*w33+1.7751623376623376e-02*w39*Jinv00-4.9310064935064934e-03*Jinv02*w12+1.9724025974025973e-03*Jinv01*w18+2.9586038961038961e-03*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-3.9448051948051946e-03*w47*Jinv21-3.9448051948051946e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+2.9586038961038961e-03*w52*Jinv11-3.2873376623376621e-03*w31*Jinv12+1.9724025974025973e-03*w46*Jinv12+1.9724025974025973e-03*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01+2.9586038961038961e-03*Jinv01*w51-3.2873376623376621e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49+2.9586038961038961e-03*w36*Jinv01+1.9724025974025973e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+2.2646103896103898e-03*w9*Jinv00-3.9448051948051946e-03*w37*Jinv10+1.9724025974025973e-03*Jinv21*w56+1.9724025974025973e-03*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27+9.8620129870129864e-04*w6*Jinv02+2.9586038961038961e-03*w19*Jinv12-4.9310064935064934e-03*Jinv22*w5+1.9724025974025973e-03*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11-3.2873376623376621e-03*Jinv21*w14+2.2646103896103898e-03*w10*Jinv10+3.5503246753246752e-02*Jinv20*w44+2.2646103896103898e-03*w58*Jinv12+2.9586038961038961e-03*Jinv02*w18-3.2873376623376621e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+2.7759740259740259e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+1.9724025974025973e-03*Jinv01*w54+1.9724025974025973e-03*w55*Jinv11+1.9724025974025973e-03*w25*Jinv10+2.7759740259740259e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-4.9310064935064934e-03*w30*Jinv00+1.7751623376623376e-02*w40*Jinv10-3.9448051948051946e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29-4.9310064935064934e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+1.9724025974025973e-03*Jinv02*w36+3.5503246753246752e-02*Jinv12*w43-3.2873376623376621e-03*w5*Jinv20-4.9310064935064934e-03*Jinv01*w30-3.9448051948051946e-03*Jinv02*w51-3.9448051948051946e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-3.9448051948051946e-03*w25*Jinv11+2.9586038961038961e-03*w55*Jinv10+3.5503246753246752e-02*w42*Jinv00+2.9586038961038961e-03*w47*Jinv20+1.7751623376623376e-02*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+2.2646103896103898e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01+2.7759740259740259e-03*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[42][43]=(0.0)*D;
  A[42][44]=(0.0)*D;
  A[42][45]=(9.8620129870129864e-04*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53+4.9310064935064932e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39+1.3806818181818182e-02*Jinv21*w44-8.7662337662337668e-04*Jinv01*w3-7.6704545454545450e-04*Jinv22*w38+4.3831168831168834e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+9.6793831168831166e-04*Jinv21*w2+1.3806818181818182e-02*w42*Jinv01+1.0592532467532468e-03*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+4.7666396103896100e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01+3.1777597402597403e-03*Jinv21*w26+9.6793831168831166e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+1.3806818181818182e-02*w43*Jinv11+6.9034090909090911e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+4.9310064935064932e-04*w11*Jinv22+9.6793831168831166e-04*Jinv11*w1-2.5202922077922079e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17+2.1367694805194807e-03*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-5.4788961038961042e-05*w32*Jinv22+6.9034090909090911e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+3.1777597402597403e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.7394480519480520e-03*w19*Jinv11-1.6436688311688311e-03*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-8.7662337662337668e-04*w4*Jinv11-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w32*Jinv21-2.7394480519480520e-03*Jinv21*w20-7.6704545454545450e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41-7.6704545454545450e-04*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+4.7666396103896100e-03*w46*Jinv11-8.7662337662337668e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.3149350649350649e-03*Jinv02*w12-2.7394480519480520e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+4.7666396103896100e-03*w47*Jinv21+2.1367694805194807e-03*Jinv22*w23-1.6436688311688311e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-3.8352272727272725e-04*Jinv22*w8-3.8352272727272725e-04*Jinv22*w50+1.5888798701298701e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+1.5888798701298701e-03*w21*Jinv01-1.6436688311688311e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-6.5746753246753243e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-8.2183441558441553e-04*w49*Jinv11-2.5202922077922079e-03*Jinv21*w14+4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-2.5202922077922079e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+1.5888798701298701e-03*Jinv21*w23-6.5746753246753243e-04*Jinv01*w54-6.5746753246753243e-04*w55*Jinv11-3.8352272727272725e-04*w48*Jinv02+1.0592532467532468e-03*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+6.9034090909090911e-03*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.3149350649350649e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+3.1777597402597403e-03*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12-8.2183441558441553e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-8.2183441558441553e-04*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1+1.0592532467532468e-03*w10*Jinv11)*D;
  A[42][46]=(0.0)*D;
  A[42][47]=(0.0)*D;
  A[42][48]=(5.4788961038961042e-05*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10+3.6160714285714286e-03*w40*Jinv12-2.6846590909090908e-03*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8+1.6436688311688311e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10+3.6160714285714286e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-2.6846590909090908e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+3.6160714285714286e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-3.2873376623376621e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+7.6704545454545450e-04*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24+1.6436688311688311e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00+3.6160714285714286e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13+7.6704545454545450e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+1.3149350649350649e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56+3.6160714285714286e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-2.6846590909090908e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-2.6846590909090908e-03*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12-3.2873376623376621e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-3.2873376623376621e-04*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53-3.2873376623376621e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-5.4788961038961036e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-3.2873376623376621e-04*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961036e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-3.2873376623376621e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+1.6436688311688311e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+3.6160714285714286e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-5.4788961038961036e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-2.6846590909090908e-03*Jinv02*w51-2.6846590909090908e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[42][49]=(0.0)*D;
  A[42][50]=(0.0)*D;
  A[42][51]=(-5.4788961038961042e-05*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+4.7666396103896100e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-7.6704545454545450e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-5.4788961038961042e-05*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-3.8352272727272725e-04*Jinv20*w23-6.5746753246753243e-04*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+9.6793831168831166e-04*w0*Jinv02+1.9724025974025973e-03*w43*Jinv10+3.1777597402597403e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+6.9034090909090911e-03*Jinv22*w35+2.7394480519480518e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-1.6436688311688311e-03*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+1.3806818181818182e-02*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+2.1367694805194807e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00-2.5202922077922079e-03*w32*Jinv22+9.6793831168831166e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-2.7394480519480520e-03*Jinv02*w45-7.6704545454545450e-04*Jinv22*w59+4.2735389610389614e-03*w51*Jinv00+6.9034090909090911e-03*Jinv12*w34+1.3806818181818182e-02*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-8.7662337662337668e-04*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-3.8352272727272725e-04*w6*Jinv00-6.5746753246753243e-04*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+4.7666396103896100e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-5.4788961038961042e-05*Jinv20*w14+3.1777597402597403e-03*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-8.7662337662337668e-04*Jinv02*w12+4.3831168831168834e-04*Jinv02*w9-3.8352272727272725e-04*w7*Jinv10-2.1367694805194807e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-2.5202922077922079e-03*w31*Jinv12-2.7394480519480520e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-8.2183441558441553e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+1.5888798701298701e-03*Jinv22*w50-8.2183441558441553e-04*Jinv12*w7-2.5202922077922079e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49+4.2735389610389614e-03*w52*Jinv10+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10-7.6704545454545450e-04*w24*Jinv00+1.0592532467532468e-03*Jinv02*w27-8.2183441558441553e-04*w6*Jinv02-1.6436688311688311e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-2.7394480519480520e-03*w47*Jinv22+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-7.6704545454545450e-04*w58*Jinv12-1.6436688311688311e-03*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+6.9034090909090911e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-7.6704545454545450e-04*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.5888798701298701e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20+1.0592532467532468e-03*Jinv22*w29-8.7662337662337668e-04*Jinv22*w14-6.5746753246753243e-04*Jinv02*w36+1.3806818181818182e-02*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+4.7666396103896100e-03*Jinv02*w51+3.1777597402597403e-03*Jinv22*w56+1.0592532467532468e-03*Jinv12*w28+4.2735389610389614e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+9.6793831168831166e-04*Jinv12*w1)*D;
  A[42][52]=(0.0)*D;
  A[42][53]=(0.0)*D;
  A[42][54]=(6.9034090909090911e-03*w40*Jinv12+3.1777597402597403e-03*Jinv22*w53-7.6704545454545450e-04*w57*Jinv02+6.9034090909090911e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3-2.7394480519480520e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-1.6436688311688311e-03*w25*Jinv12+9.6793831168831166e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2+1.9724025974025973e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+4.7666396103896100e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+1.0592532467532468e-03*w10*Jinv12-1.6436688311688311e-03*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41+1.3806818181818182e-02*w42*Jinv02+1.0592532467532468e-03*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17-8.2183441558441553e-04*w22*Jinv12-8.7662337662337668e-04*Jinv02*w3-2.5202922077922079e-03*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39+9.6793831168831166e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-6.5746753246753243e-04*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24-7.6704545454545450e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+1.3806818181818182e-02*Jinv22*w44-5.4788961038961042e-05*w4*Jinv11-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-2.7394480519480520e-03*w37*Jinv12+6.9034090909090911e-03*Jinv22*w41+3.1777597402597403e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11-1.6436688311688311e-03*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+4.7666396103896100e-03*Jinv02*w54-8.7662337662337668e-04*w4*Jinv12-8.2183441558441553e-04*w21*Jinv02+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.3149350649350649e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+1.0592532467532468e-03*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-2.1367694805194807e-03*w47*Jinv21-8.2183441558441553e-04*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-2.5202922077922079e-03*w31*Jinv12-6.5746753246753243e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+1.5888798701298701e-03*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-2.5202922077922079e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-3.8352272727272725e-04*w7*Jinv11+4.2735389610389614e-03*Jinv21*w56+4.3831168831168834e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-8.7662337662337668e-04*Jinv22*w5-6.5746753246753243e-04*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-7.6704545454545450e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-3.8352272727272725e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11+1.5888798701298701e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+4.3831168831168834e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-2.7394480519480520e-03*Jinv02*w36+1.3806818181818182e-02*Jinv12*w43-1.3149350649350649e-03*Jinv01*w30+3.1777597402597403e-03*Jinv02*w51+4.7666396103896100e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01+9.6793831168831166e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[42][55]=(0.0)*D;
  A[42][56]=(0.0)*D;
  A[42][57]=(-1.8262987012987013e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-1.8262987012987013e-03*Jinv02*w39-3.2873376623376621e-04*Jinv22*w38+4.3831168831168834e-04*w25*Jinv12-1.9480519480519481e-04*w0*Jinv02+6.5746753246753243e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+4.3831168831168834e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35+4.3831168831168834e-04*Jinv22*w20-3.6525974025974025e-03*w42*Jinv02-4.2613636363636367e-05*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+2.1915584415584417e-04*w22*Jinv12+3.4699675324675324e-04*Jinv02*w3+2.9220779220779221e-04*w32*Jinv22-1.9480519480519481e-04*Jinv22*w2-3.2873376623376621e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-1.8262987012987013e-03*Jinv12*w34-3.6525974025974025e-03*Jinv22*w44+3.4699675324675324e-04*Jinv12*w13-3.2873376623376621e-04*w37*Jinv12-1.8262987012987013e-03*Jinv22*w41+6.5746753246753243e-04*w52*Jinv12+4.3831168831168834e-04*Jinv22*w26+6.5746753246753243e-04*Jinv02*w54+3.4699675324675324e-04*w4*Jinv12+2.1915584415584417e-04*w21*Jinv02+3.4699675324675324e-04*Jinv02*w12-4.2613636363636367e-05*Jinv02*w9+2.1915584415584417e-04*Jinv22*w23+2.9220779220779221e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+2.1915584415584417e-04*Jinv22*w8+3.2873376623376621e-04*Jinv22*w50+2.1915584415584417e-04*Jinv12*w7+2.9220779220779221e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.2613636363636367e-05*Jinv02*w27+2.1915584415584417e-04*w6*Jinv02+4.3831168831168834e-04*w19*Jinv12+3.4699675324675324e-04*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+4.3831168831168834e-04*Jinv02*w18-1.8262987012987013e-03*Jinv02*w33+3.2873376623376621e-04*w48*Jinv02-4.2613636363636367e-05*Jinv22*w29+3.4699675324675324e-04*Jinv22*w14-3.2873376623376621e-04*Jinv02*w36-3.6525974025974025e-03*Jinv12*w43+6.5746753246753243e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-1.9480519480519481e-04*Jinv12*w1)*D;
  A[42][58]=(0.0)*D;
  A[42][59]=(0.0)*D;
  A[43][0]=(0.0)*D;
  A[43][1]=(3.1047077922077924e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10+5.1136363636363640e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-2.1915584415584417e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+3.1047077922077924e-04*w13*Jinv10+5.1136363636363640e-04*Jinv02*w39-5.5519480519480519e-03*Jinv21*w44+3.1047077922077924e-04*w3*Jinv00+3.1047077922077924e-04*w32*Jinv20+3.1047077922077924e-04*Jinv01*w3-2.1915584415584417e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-3.0438311688311688e-05*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-2.4350649350649351e-04*w0*Jinv02-2.4350649350649351e-04*Jinv21*w2-5.5519480519480519e-03*w43*Jinv10-5.5519480519480519e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21+5.1136363636363640e-04*w16*Jinv11+5.1136363636363640e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10+5.1136363636363640e-04*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12-2.1915584415584417e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+5.1136363636363640e-04*Jinv22*w35-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-2.4350649350649351e-04*w0*Jinv01-2.4350649350649351e-04*w1*Jinv10+5.1136363636363640e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-5.5519480519480519e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+5.1136363636363640e-04*Jinv21*w41-5.5519480519480519e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-3.0438311688311688e-05*w11*Jinv22-2.4350649350649351e-04*Jinv11*w1+3.1047077922077924e-04*w13*Jinv11+5.1136363636363640e-04*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11-2.1915584415584417e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+5.1136363636363640e-04*Jinv22*w17+5.1136363636363640e-04*Jinv00*w33-2.1915584415584417e-04*w22*Jinv12+3.1047077922077924e-04*w31*Jinv10+3.1047077922077924e-04*Jinv02*w3-2.1915584415584417e-04*w21*Jinv00+3.1047077922077924e-04*w32*Jinv22+5.1136363636363640e-04*Jinv01*w39-2.4350649350649351e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+5.1136363636363640e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+5.1136363636363640e-04*Jinv12*w34-5.5519480519480519e-03*Jinv22*w44+3.1047077922077924e-04*w4*Jinv10+3.1047077922077924e-04*w4*Jinv11+5.1136363636363640e-04*w41*Jinv20+3.1047077922077924e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+5.1136363636363640e-04*w34*Jinv10+3.1047077922077924e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.1915584415584417e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+5.1136363636363640e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+5.1136363636363640e-04*Jinv21*w35+3.1047077922077924e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+3.1047077922077924e-04*Jinv20*w14+3.1047077922077924e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+3.1047077922077924e-04*w4*Jinv12-2.1915584415584417e-04*w21*Jinv02-2.1915584415584417e-04*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29+5.1136363636363640e-04*Jinv01*w33+5.1136363636363640e-04*w39*Jinv00+3.1047077922077924e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-3.0438311688311688e-05*Jinv02*w9-2.1915584415584417e-04*w6*Jinv01-2.1915584415584417e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-2.1915584415584417e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.1047077922077924e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-2.1915584415584417e-04*Jinv22*w8-2.1915584415584417e-04*w22*Jinv10-2.1915584415584417e-04*Jinv22*w50-2.1915584415584417e-04*w22*Jinv11-2.1915584415584417e-04*Jinv12*w7-2.1915584415584417e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+3.1047077922077924e-04*Jinv02*w30-2.1915584415584417e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-2.1915584415584417e-04*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-2.1915584415584417e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.1047077922077924e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-2.1915584415584417e-04*w49*Jinv11+3.1047077922077924e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-5.5519480519480519e-03*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+3.1047077922077924e-04*Jinv01*w12-2.1915584415584417e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59+5.1136363636363640e-04*Jinv02*w33-2.4350649350649351e-04*w0*Jinv00+5.1136363636363640e-04*w35*Jinv20-2.1915584415584417e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-2.4350649350649351e-04*Jinv20*w2+5.1136363636363640e-04*w15*Jinv00-2.1915584415584417e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9+5.1136363636363640e-04*Jinv20*w17+3.1047077922077924e-04*w30*Jinv00+5.1136363636363640e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+3.1047077922077924e-04*Jinv22*w14+5.1136363636363640e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36-5.5519480519480519e-03*Jinv12*w43+3.1047077922077924e-04*w5*Jinv20+3.1047077922077924e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-5.5519480519480519e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+5.1136363636363640e-04*w16*Jinv12-2.1915584415584417e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27-2.1915584415584417e-04*w48*Jinv01-2.4350649350649351e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[43][2]=(0.0)*D;
  A[43][3]=(0.0)*D;
  A[43][4]=(3.6525974025974026e-05*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8+7.3051948051948052e-05*w57*Jinv02-6.5746753246753243e-04*Jinv02*w39+9.8620129870129868e-03*Jinv21*w44-5.4788961038961036e-04*w3*Jinv00-1.6436688311688311e-03*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+6.7573051948051951e-04*w0*Jinv02+6.7573051948051951e-04*Jinv21*w2+6.5746753246753243e-03*w43*Jinv10+9.8620129870129868e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+6.7573051948051951e-04*w0*Jinv01+1.8262987012987014e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+9.8620129870129868e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+9.8620129870129868e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22+6.7573051948051951e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01+7.3051948051948052e-05*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-1.6436688311688311e-03*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39+6.7573051948051951e-04*Jinv22*w2+2.5750811688311690e-03*Jinv00*w18+2.5750811688311690e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+7.3051948051948052e-05*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+2.6298701298701297e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34+9.8620129870129868e-03*Jinv22*w44-5.4788961038961036e-04*w4*Jinv10-1.6436688311688311e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.0957792207792207e-03*Jinv12*w13+2.5750811688311690e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-1.0957792207792207e-03*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35-1.0957792207792207e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-1.6436688311688311e-03*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54-1.6436688311688311e-03*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+7.3051948051948052e-05*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-1.0957792207792207e-03*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+8.2183441558441553e-04*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+2.5750811688311690e-03*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+8.2183441558441553e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+8.2183441558441553e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+8.2183441558441553e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.5750811688311690e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+8.2183441558441553e-04*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.6436688311688311e-03*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+6.5746753246753243e-03*Jinv20*w44+7.3051948051948052e-05*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+8.2183441558441553e-04*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33+1.8262987012987014e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+1.8262987012987014e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10+2.5750811688311690e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-1.0957792207792207e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36+9.8620129870129868e-03*Jinv12*w43-5.4788961038961036e-04*w5*Jinv20-1.0957792207792207e-03*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+6.5746753246753243e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+7.3051948051948052e-05*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+6.7573051948051951e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[43][5]=(0.0)*D;
  A[43][6]=(0.0)*D;
  A[43][7]=(-6.5746753246753243e-04*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-5.4788961038961036e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3-3.2873376623376621e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-3.2873376623376621e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10+3.6160714285714286e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-3.2873376623376621e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+3.6160714285714286e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3-3.2873376623376621e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2-2.6846590909090908e-03*Jinv00*w18-2.6846590909090908e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+7.6704545454545450e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-2.6846590909090908e-03*w19*Jinv10+3.6160714285714286e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-5.4788961038961036e-04*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20+1.6436688311688311e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.6704545454545450e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02-3.2873376623376621e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-5.4788961038961036e-04*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-2.6846590909090908e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8-3.2873376623376621e-04*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+1.6436688311688311e-04*w9*Jinv00-2.6846590909090908e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.6436688311688311e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00+3.6160714285714286e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2+3.6160714285714286e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+3.6160714285714286e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-2.6846590909090908e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[43][8]=(0.0)*D;
  A[43][9]=(0.0)*D;
  A[43][10]=(3.4699675324675324e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10+3.2873376623376621e-04*Jinv20*w8+3.4699675324675324e-04*w13*Jinv10+2.9220779220779221e-04*w3*Jinv00+3.4699675324675324e-04*w32*Jinv20+2.1915584415584417e-04*Jinv20*w23+4.3831168831168834e-04*Jinv00*w54-3.6525974025974025e-03*w43*Jinv10-4.2613636363636367e-05*w58*Jinv10+2.1915584415584417e-04*w48*Jinv00-1.9480519480519481e-04*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10+4.3831168831168834e-04*w45*Jinv00-4.2613636363636367e-05*Jinv00*w27+2.1915584415584417e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+4.3831168831168834e-04*w46*Jinv10-1.8262987012987013e-03*Jinv00*w33+3.4699675324675324e-04*w31*Jinv10+2.1915584415584417e-04*w21*Jinv00+6.5746753246753243e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-3.2873376623376621e-04*w51*Jinv00+2.9220779220779221e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+3.2873376623376621e-04*w6*Jinv00+4.3831168831168834e-04*Jinv20*w56-3.2873376623376621e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+3.4699675324675324e-04*Jinv20*w14+2.1915584415584417e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+3.2873376623376621e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-3.2873376623376621e-04*Jinv20*w53+2.1915584415584417e-04*w22*Jinv10-3.2873376623376621e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10-3.2873376623376621e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-3.6525974025974025e-03*Jinv20*w44-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-1.9480519480519481e-04*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20-3.2873376623376621e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00-1.8262987012987013e-03*Jinv20*w17+3.4699675324675324e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20+2.9220779220779221e-04*w5*Jinv20+4.3831168831168834e-04*w55*Jinv10-3.6525974025974025e-03*w42*Jinv00+4.3831168831168834e-04*w47*Jinv20)*D;
  A[43][11]=(0.0)*D;
  A[43][12]=(0.0)*D;
  A[43][13]=(-1.6436688311688311e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8+7.3051948051948052e-05*w57*Jinv02-1.6436688311688311e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+6.5746753246753243e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.0957792207792207e-03*w32*Jinv20+8.2183441558441553e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+6.7573051948051951e-04*w0*Jinv02+1.8262987012987014e-04*Jinv21*w2+9.8620129870129868e-03*w43*Jinv10+6.5746753246753243e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17+2.5750811688311690e-03*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55+7.3051948051948052e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+2.5750811688311690e-03*Jinv21*w26+1.8262987012987014e-04*w0*Jinv01+6.7573051948051951e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+6.5746753246753243e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41+9.8620129870129868e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+7.3051948051948052e-05*w11*Jinv22+1.8262987012987014e-04*Jinv11*w1-5.4788961038961036e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+8.2183441558441553e-04*w22*Jinv12-1.0957792207792207e-03*w31*Jinv10-1.0957792207792207e-03*Jinv02*w3+8.2183441558441553e-04*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39+6.7573051948051951e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+2.5750811688311690e-03*Jinv01*w24+7.3051948051948052e-05*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+9.8620129870129868e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10-9.8620129870129864e-04*w41*Jinv20-1.6436688311688311e-03*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10+1.0957792207792208e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+7.3051948051948052e-05*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.6298701298701297e-03*Jinv22*w26+2.5750811688311690e-03*w46*Jinv11-1.6436688311688311e-03*Jinv20*w14+1.6436688311688311e-04*Jinv02*w54-1.0957792207792207e-03*w4*Jinv12+8.2183441558441553e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-1.6436688311688311e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+2.5750811688311690e-03*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38+8.2183441558441553e-04*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+8.2183441558441553e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+7.3051948051948052e-05*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-1.0957792207792207e-03*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-5.4788961038961036e-04*Jinv21*w14+7.3051948051948052e-05*w10*Jinv10+9.8620129870129868e-03*Jinv20*w44+7.3051948051948052e-05*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-5.4788961038961036e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33+6.7573051948051951e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10+6.7573051948051951e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17-1.0957792207792207e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-1.6436688311688311e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+9.8620129870129868e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+1.0957792207792208e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+2.5750811688311690e-03*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+9.8620129870129868e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01+6.7573051948051951e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[43][14]=(0.0)*D;
  A[43][15]=(0.0)*D;
  A[43][16]=(2.9586038961038961e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+3.9448051948051946e-03*Jinv22*w53-2.3011363636363637e-03*Jinv20*w8-2.2646103896103898e-03*w57*Jinv02+2.9586038961038961e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-2.3668831168831169e-02*Jinv21*w44+3.6160714285714286e-03*w3*Jinv00+1.9724025974025973e-03*w32*Jinv20+2.9586038961038961e-03*Jinv01*w3-1.9724025974025973e-03*Jinv22*w38+1.9724025974025973e-03*Jinv00*w54-1.1323051948051949e-03*Jinv21*w59-2.9586038961038961e-03*w25*Jinv12-2.7759740259740259e-03*w0*Jinv02-1.6436688311688311e-03*Jinv21*w2-2.3668831168831169e-02*w43*Jinv10-2.3668831168831169e-02*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17-3.2873376623376621e-03*Jinv01*w45+3.9448051948051946e-03*Jinv12*w55-1.1323051948051949e-03*w58*Jinv10-1.7751623376623376e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12-2.9586038961038961e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-1.1323051948051949e-03*w58*Jinv11-1.1323051948051949e-03*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26-1.6436688311688311e-03*w0*Jinv01-1.6436688311688311e-03*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10-2.9586038961038961e-03*Jinv22*w20-2.3668831168831169e-02*w43*Jinv11-9.8620129870129864e-04*w45*Jinv00-3.9448051948051946e-03*Jinv21*w41-3.5503246753246752e-02*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22-1.6436688311688311e-03*Jinv11*w1+3.6160714285714286e-03*w13*Jinv11-5.9172077922077923e-03*w15*Jinv01+6.9399350649350649e-04*w28*Jinv11-2.1915584415584417e-04*Jinv20*w29-9.8620129870129864e-04*w46*Jinv10-1.7751623376623376e-02*Jinv22*w17-3.9448051948051946e-03*Jinv00*w33-9.8620129870129864e-04*w22*Jinv12+1.9724025974025973e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3+3.2873376623376621e-03*w32*Jinv22-3.9448051948051946e-03*Jinv01*w39-2.7759740259740259e-03*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-3.2873376623376621e-03*w36*Jinv00-9.8620129870129864e-04*Jinv21*w38-1.9724025974025973e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24-2.2646103896103898e-03*Jinv22*w59-1.9724025974025973e-03*w19*Jinv11+1.9724025974025973e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-3.5503246753246752e-02*Jinv22*w44+3.6160714285714286e-03*w4*Jinv10+2.9586038961038961e-03*w4*Jinv11+4.9310064935064934e-03*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10-3.9448051948051946e-03*w34*Jinv10+1.9724025974025973e-03*w32*Jinv21-1.9724025974025973e-03*Jinv21*w20-2.3011363636363637e-03*w6*Jinv00-1.9724025974025973e-03*w37*Jinv12+1.9724025974025973e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41-1.9724025974025973e-03*w26*Jinv20+6.9399350649350649e-04*w11*Jinv20+3.9448051948051946e-03*w52*Jinv12+1.9724025974025973e-03*w31*Jinv11-2.9586038961038961e-03*Jinv22*w26-3.2873376623376621e-03*w46*Jinv11+2.9586038961038961e-03*Jinv20*w14+2.9586038961038961e-03*w5*Jinv21+3.9448051948051946e-03*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-9.8620129870129864e-04*w21*Jinv02+6.9399350649350649e-04*Jinv21*w29+4.9310064935064934e-03*Jinv02*w12-1.9724025974025973e-03*Jinv01*w18-9.8620129870129864e-04*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9-2.3011363636363637e-03*w7*Jinv10-3.2873376623376621e-03*w47*Jinv21-3.2873376623376621e-03*Jinv20*w38-9.8620129870129864e-04*Jinv22*w23+1.9724025974025973e-03*w52*Jinv11+3.2873376623376621e-03*w31*Jinv12-1.9724025974025973e-03*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53-9.8620129870129864e-04*Jinv22*w8+1.3149350649350649e-03*Jinv22*w50-2.3011363636363637e-03*w22*Jinv11-9.8620129870129864e-04*Jinv12*w7-2.3011363636363637e-03*w21*Jinv01+1.9724025974025973e-03*Jinv01*w51+3.2873376623376621e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-9.8620129870129864e-04*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+6.9399350649350649e-04*w9*Jinv00-3.2873376623376621e-03*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56-1.9724025974025973e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27-9.8620129870129864e-04*w6*Jinv02-2.9586038961038961e-03*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5-1.9724025974025973e-03*w47*Jinv22+3.6160714285714286e-03*Jinv21*w14+6.9399350649350649e-04*w10*Jinv10-2.3668831168831169e-02*Jinv20*w44-2.2646103896103898e-03*w58*Jinv12-2.9586038961038961e-03*Jinv02*w18+3.6160714285714286e-03*Jinv01*w12-1.1323051948051949e-03*w57*Jinv00-1.1323051948051949e-03*Jinv20*w59-1.1834415584415585e-02*Jinv02*w33-1.6436688311688311e-03*w0*Jinv00-3.9448051948051946e-03*w35*Jinv20-2.3011363636363637e-03*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11-1.9724025974025973e-03*w25*Jinv10-1.6436688311688311e-03*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17+1.9724025974025973e-03*w30*Jinv00-4.9310064935064934e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14-3.9448051948051946e-03*w40*Jinv11-1.9724025974025973e-03*Jinv02*w36-3.5503246753246752e-02*Jinv12*w43+3.6160714285714286e-03*w5*Jinv20+1.9724025974025973e-03*Jinv01*w30+3.9448051948051946e-03*Jinv02*w51+3.9448051948051946e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+1.9724025974025973e-03*w55*Jinv10-2.3668831168831169e-02*w42*Jinv00-9.8620129870129864e-04*w47*Jinv20-1.7751623376623376e-02*w16*Jinv12+6.9399350649350649e-04*Jinv01*w27-2.7759740259740259e-03*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[43][17]=(0.0)*D;
  A[43][18]=(0.0)*D;
  A[43][19]=(-1.3149350649350649e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.5888798701298701e-03*Jinv20*w8-1.3149350649350649e-03*w13*Jinv10+1.9724025974025973e-03*Jinv21*w44-2.5202922077922079e-03*w3*Jinv00-8.7662337662337668e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3-8.2183441558441553e-04*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59+2.7394480519480518e-04*Jinv21*w2+1.3806818181818182e-02*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45+1.0592532467532468e-03*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01+9.6793831168831166e-04*w1*Jinv10+6.9034090909090911e-03*w16*Jinv10+1.9724025974025973e-03*w43*Jinv11-1.6436688311688311e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+4.3831168831168834e-04*Jinv00*w27+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-1.6436688311688311e-03*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-8.7662337662337668e-04*w31*Jinv10-8.2183441558441553e-04*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+4.7666396103896100e-03*Jinv00*w18+3.1777597402597403e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-2.1367694805194807e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-2.7394480519480520e-03*w51*Jinv00-2.5202922077922079e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+4.7666396103896100e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.5888798701298701e-03*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56-6.5746753246753243e-04*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-7.6704545454545450e-04*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+4.2735389610389614e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11+2.1367694805194807e-03*w6*Jinv01+1.5888798701298701e-03*w7*Jinv10-7.6704545454545450e-04*w47*Jinv21+3.1777597402597403e-03*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-2.7394480519480520e-03*Jinv20*w53-8.2183441558441553e-04*w22*Jinv10-3.8352272727272725e-04*w22*Jinv11-3.8352272727272725e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+4.2735389610389614e-03*w36*Jinv01-2.7394480519480520e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-7.6704545454545450e-04*w9*Jinv00+3.1777597402597403e-03*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56-6.5746753246753243e-04*w24*Jinv00-3.8352272727272725e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-7.6704545454545450e-04*w10*Jinv10+1.3806818181818182e-02*Jinv20*w44-5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+1.0592532467532468e-03*w57*Jinv00+1.0592532467532468e-03*Jinv20*w59+9.6793831168831166e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11-6.5746753246753243e-04*w25*Jinv10+9.6793831168831166e-04*Jinv20*w2+6.9034090909090911e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+6.9034090909090911e-03*Jinv20*w17-8.7662337662337668e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+4.7666396103896100e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-2.5202922077922079e-03*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-2.1367694805194807e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.3806818181818182e-02*w42*Jinv00-1.6436688311688311e-03*w47*Jinv20-3.8352272727272725e-04*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[43][20]=(0.0)*D;
  A[43][21]=(0.0)*D;
  A[43][22]=(1.3149350649350649e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+3.6160714285714286e-03*w16*Jinv11+3.6160714285714286e-03*Jinv21*w17-2.6846590909090908e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01-2.6846590909090908e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00+3.6160714285714286e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+3.6160714285714286e-03*w15*Jinv01+1.6436688311688311e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+3.6160714285714286e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-2.6846590909090908e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-2.6846590909090908e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+1.6436688311688311e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9-3.2873376623376621e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10-2.6846590909090908e-03*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-5.4788961038961036e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-5.4788961038961036e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10-3.2873376623376621e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22-3.2873376623376621e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.2873376623376621e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-5.4788961038961036e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14+3.6160714285714286e-03*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30+7.6704545454545450e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-2.6846590909090908e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-3.2873376623376621e-04*Jinv21*w50+1.6436688311688311e-04*Jinv01*w27-3.2873376623376621e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[43][23]=(0.0)*D;
  A[43][24]=(0.0)*D;
  A[43][25]=(-1.3149350649350649e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-3.8352272727272725e-04*Jinv20*w8-1.3149350649350649e-03*w13*Jinv10+1.3806818181818182e-02*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+1.0592532467532468e-03*Jinv21*w59+9.6793831168831166e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.3806818181818182e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+6.9034090909090911e-03*w16*Jinv11+6.9034090909090911e-03*Jinv21*w17+3.1777597402597403e-03*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-3.8352272727272725e-04*w48*Jinv00+1.0592532467532468e-03*w58*Jinv11+1.0592532467532468e-03*w57*Jinv01+4.7666396103896100e-03*Jinv21*w26+9.6793831168831166e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+1.3806818181818182e-02*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+9.6793831168831166e-04*Jinv11*w1-2.5202922077922079e-03*w13*Jinv11+6.9034090909090911e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10+2.1367694805194807e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-2.1367694805194807e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-1.6436688311688311e-03*Jinv21*w38+4.7666396103896100e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11-6.5746753246753243e-04*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-8.7662337662337668e-04*w32*Jinv21-6.5746753246753243e-04*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-8.7662337662337668e-04*w31*Jinv11+3.1777597402597403e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-3.8352272727272725e-04*w49*Jinv10-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv01*w18-1.6436688311688311e-03*w37*Jinv11-8.2183441558441553e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+3.1777597402597403e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-7.6704545454545450e-04*Jinv20*w53+2.1367694805194807e-03*w22*Jinv10+1.5888798701298701e-03*w22*Jinv11+1.5888798701298701e-03*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-1.6436688311688311e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10-8.2183441558441553e-04*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-2.7394480519480520e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-2.5202922077922079e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-2.5202922077922079e-03*Jinv01*w12-8.2183441558441553e-04*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+2.7394480519480518e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.5888798701298701e-03*Jinv21*w23-2.7394480519480520e-03*Jinv01*w54-2.7394480519480520e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+4.3831168831168834e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-5.4788961038961042e-05*w5*Jinv20-8.7662337662337668e-04*Jinv01*w30+4.7666396103896100e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+4.3831168831168834e-04*w10*Jinv11)*D;
  A[43][26]=(0.0)*D;
  A[43][27]=(0.0)*D;
  A[43][28]=(-3.6525974025974025e-03*Jinv21*w44+3.4699675324675324e-04*Jinv01*w3-4.2613636363636367e-05*Jinv21*w59-1.9480519480519481e-04*Jinv21*w2-3.6525974025974025e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-3.6525974025974025e-03*w43*Jinv11-1.8262987012987013e-03*Jinv21*w41-1.9480519480519481e-04*Jinv11*w1+2.9220779220779221e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.8262987012987013e-03*Jinv01*w39+4.3831168831168834e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11+4.3831168831168834e-04*Jinv21*w53+3.4699675324675324e-04*w4*Jinv11+3.4699675324675324e-04*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+3.4699675324675324e-04*w31*Jinv11+6.5746753246753243e-04*w46*Jinv11+3.4699675324675324e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33-3.2873376623376621e-04*Jinv01*w18+4.3831168831168834e-04*w37*Jinv11+2.1915584415584417e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+4.3831168831168834e-04*w52*Jinv11+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*w21*Jinv01+4.3831168831168834e-04*Jinv01*w51+4.3831168831168834e-04*w36*Jinv01+2.1915584415584417e-04*w7*Jinv11-3.2873376623376621e-04*Jinv21*w56+2.1915584415584417e-04*w49*Jinv11+2.9220779220779221e-04*Jinv21*w14+2.9220779220779221e-04*Jinv01*w12+2.1915584415584417e-04*Jinv21*w8+3.2873376623376621e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*w40*Jinv11+3.4699675324675324e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11+2.1915584415584417e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+2.1915584415584417e-04*w48*Jinv01-4.2613636363636367e-05*w10*Jinv11)*D;
  A[43][29]=(0.0)*D;
  A[43][30]=(0.0)*D;
  A[43][31]=(-1.0957792207792207e-03*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12+2.5750811688311690e-03*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-1.0957792207792207e-03*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39+9.8620129870129868e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.6436688311688311e-03*w32*Jinv20-1.0957792207792207e-03*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+1.8262987012987014e-04*w0*Jinv02+6.7573051948051951e-04*Jinv21*w2+9.8620129870129868e-03*w43*Jinv10+9.8620129870129868e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+2.5750811688311690e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+8.2183441558441553e-04*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26+6.7573051948051951e-04*w0*Jinv01+6.7573051948051951e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+9.8620129870129868e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+6.5746753246753243e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+6.7573051948051951e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01+7.3051948051948052e-05*w28*Jinv11+8.2183441558441553e-04*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-1.6436688311688311e-03*w31*Jinv10+7.1225649350649346e-04*w21*Jinv00-5.4788961038961036e-04*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39+1.8262987012987014e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34+6.5746753246753243e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10-1.0957792207792207e-03*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+1.0957792207792208e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-1.6436688311688311e-03*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20+7.3051948051948052e-05*w11*Jinv20+2.5750811688311690e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35-1.6436688311688311e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-1.0957792207792207e-03*Jinv20*w14-1.0957792207792207e-03*w5*Jinv21+2.5750811688311690e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02+8.2183441558441553e-04*w49*Jinv10+7.3051948051948052e-05*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+1.0957792207792208e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11-5.4788961038961036e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-5.4788961038961036e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+7.3051948051948052e-05*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12+1.0957792207792208e-04*w47*Jinv22+8.2183441558441553e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+7.3051948051948052e-05*w10*Jinv10+9.8620129870129868e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33+6.7573051948051951e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+6.7573051948051951e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02+7.3051948051948052e-05*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.6436688311688311e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+6.5746753246753243e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20-1.6436688311688311e-03*Jinv01*w30+2.5750811688311690e-03*Jinv02*w51+2.5750811688311690e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+9.8620129870129868e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+8.2183441558441553e-04*Jinv21*w50+7.3051948051948052e-05*Jinv01*w27+8.2183441558441553e-04*w48*Jinv01+1.8262987012987014e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[43][32]=(0.0)*D;
  A[43][33]=(0.0)*D;
  A[43][34]=(1.9724025974025973e-03*Jinv00*w12-1.1323051948051949e-03*w28*Jinv10-3.9448051948051946e-03*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53-2.3011363636363637e-03*Jinv20*w8+6.9399350649350649e-04*w57*Jinv02+1.9724025974025973e-03*w13*Jinv10-3.9448051948051946e-03*Jinv02*w39-3.5503246753246752e-02*Jinv21*w44+3.6160714285714286e-03*w3*Jinv00+2.9586038961038961e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3-1.9724025974025973e-03*Jinv22*w38-9.8620129870129864e-04*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+1.9724025974025973e-03*w25*Jinv12-1.6436688311688311e-03*w0*Jinv02-2.7759740259740259e-03*Jinv21*w2-2.3668831168831169e-02*w43*Jinv10-3.5503246753246752e-02*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+3.9448051948051946e-03*Jinv01*w45-3.2873376623376621e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10-2.1915584415584417e-04*w10*Jinv12+1.9724025974025973e-03*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+3.9448051948051946e-03*Jinv21*w26-2.7759740259740259e-03*w0*Jinv01-1.6436688311688311e-03*w1*Jinv10-3.9448051948051946e-03*w16*Jinv10-9.8620129870129864e-04*Jinv22*w20-3.5503246753246752e-02*w43*Jinv11+1.9724025974025973e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-2.3668831168831169e-02*w42*Jinv02-1.1323051948051949e-03*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-2.7759740259740259e-03*Jinv11*w1+3.2873376623376621e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-2.2646103896103898e-03*w28*Jinv11-1.1323051948051949e-03*Jinv20*w29+1.9724025974025973e-03*w46*Jinv10-5.9172077922077923e-03*Jinv00*w33+2.9586038961038961e-03*w31*Jinv10+2.9586038961038961e-03*Jinv02*w3+3.6160714285714286e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-1.6436688311688311e-03*Jinv22*w2-3.2873376623376621e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-2.9586038961038961e-03*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+3.9448051948051946e-03*Jinv01*w24+6.9399350649350649e-04*Jinv22*w59-1.7751623376623376e-02*w34*Jinv11-1.9724025974025973e-03*w19*Jinv11-2.9586038961038961e-03*Jinv21*w53-1.9724025974025973e-03*w51*Jinv00-5.9172077922077923e-03*Jinv12*w34-2.3668831168831169e-02*Jinv22*w44+3.6160714285714286e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11+1.9724025974025973e-03*Jinv12*w13-3.2873376623376621e-03*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21-1.9724025974025973e-03*Jinv21*w20-2.3011363636363637e-03*w6*Jinv00-1.9724025974025973e-03*w37*Jinv12-9.8620129870129864e-04*Jinv20*w56-3.9448051948051946e-03*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+6.9399350649350649e-04*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12-1.7751623376623376e-02*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11+1.9724025974025973e-03*Jinv22*w26+3.9448051948051946e-03*w46*Jinv11+1.9724025974025973e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21-3.2873376623376621e-03*Jinv02*w54+2.9586038961038961e-03*w4*Jinv12-2.2646103896103898e-03*Jinv21*w29-1.7751623376623376e-02*Jinv01*w33+1.9724025974025973e-03*Jinv02*w12-1.9724025974025973e-03*Jinv01*w18-2.9586038961038961e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-9.8620129870129864e-04*w6*Jinv01-2.3011363636363637e-03*w7*Jinv10+3.9448051948051946e-03*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38-2.9586038961038961e-03*w52*Jinv11+3.6160714285714286e-03*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12-1.9724025974025973e-03*Jinv20*w53-2.3011363636363637e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11+1.3149350649350649e-03*w21*Jinv01-2.9586038961038961e-03*Jinv01*w51+3.6160714285714286e-03*Jinv02*w30-2.3011363636363637e-03*Jinv12*w49-2.9586038961038961e-03*w36*Jinv01-1.9724025974025973e-03*w52*Jinv10-9.8620129870129864e-04*w7*Jinv11+6.9399350649350649e-04*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10-1.9724025974025973e-03*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00-1.1323051948051949e-03*Jinv02*w27-9.8620129870129864e-04*w19*Jinv12+2.9586038961038961e-03*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22-9.8620129870129864e-04*w49*Jinv11+3.2873376623376621e-03*Jinv21*w14+6.9399350649350649e-04*w10*Jinv10-2.3668831168831169e-02*Jinv20*w44+6.9399350649350649e-04*w58*Jinv12-9.8620129870129864e-04*Jinv02*w18+3.2873376623376621e-03*Jinv01*w12-9.8620129870129864e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33-1.6436688311688311e-03*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-1.9724025974025973e-03*Jinv01*w54-1.9724025974025973e-03*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10-1.6436688311688311e-03*Jinv20*w2-3.9448051948051946e-03*w15*Jinv00-2.3011363636363637e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-3.9448051948051946e-03*Jinv20*w17+2.9586038961038961e-03*w30*Jinv00-3.2873376623376621e-03*w20*Jinv20-1.1323051948051949e-03*Jinv22*w29+1.9724025974025973e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11-1.9724025974025973e-03*Jinv02*w36-2.3668831168831169e-02*Jinv12*w43+3.6160714285714286e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-3.2873376623376621e-03*Jinv22*w56-1.1323051948051949e-03*Jinv12*w28+3.9448051948051946e-03*w25*Jinv11-9.8620129870129864e-04*w55*Jinv10-2.3668831168831169e-02*w42*Jinv00+1.9724025974025973e-03*w47*Jinv20-9.8620129870129864e-04*Jinv21*w50-2.2646103896103898e-03*Jinv01*w27-9.8620129870129864e-04*w48*Jinv01-1.6436688311688311e-03*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[43][35]=(0.0)*D;
  A[43][36]=(0.0)*D;
  A[43][37]=(-8.7662337662337668e-04*Jinv00*w12+1.0592532467532468e-03*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+1.5888798701298701e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-8.7662337662337668e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-2.5202922077922079e-03*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-1.6436688311688311e-03*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+1.3806818181818182e-02*w43*Jinv10-7.6704545454545450e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-8.2183441558441553e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+9.6793831168831166e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+1.9724025974025973e-03*w42*Jinv02+1.0592532467532468e-03*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-8.2183441558441553e-04*Jinv20*w50+1.0592532467532468e-03*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+6.9034090909090911e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+2.7394480519480518e-04*Jinv22*w2+3.1777597402597403e-03*Jinv00*w18+4.7666396103896100e-03*w36*Jinv00-7.6704545454545450e-04*Jinv02*w45+4.9310064935064932e-04*Jinv22*w59-6.5746753246753243e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-2.5202922077922079e-03*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+3.1777597402597403e-03*w19*Jinv10+6.9034090909090911e-03*w34*Jinv10+1.5888798701298701e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-1.6436688311688311e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-2.7394480519480520e-03*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-8.7662337662337668e-04*Jinv20*w14-7.6704545454545450e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-8.2183441558441553e-04*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+1.5888798701298701e-03*w7*Jinv10+4.7666396103896100e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12-6.5746753246753243e-04*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-6.5746753246753243e-04*w52*Jinv10-7.6704545454545450e-04*w9*Jinv00+4.7666396103896100e-03*w37*Jinv10-2.7394480519480520e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22-7.6704545454545450e-04*w10*Jinv10+1.3806818181818182e-02*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33+9.6793831168831166e-04*w0*Jinv00+6.9034090909090911e-03*w35*Jinv20-2.7394480519480520e-03*w25*Jinv10+9.6793831168831166e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+3.1777597402597403e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+4.2735389610389614e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-2.5202922077922079e-03*w5*Jinv20-2.1367694805194807e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-1.6436688311688311e-03*w55*Jinv10+1.3806818181818182e-02*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+2.7394480519480518e-04*Jinv12*w1)*D;
  A[43][38]=(0.0)*D;
  A[43][39]=(0.0)*D;
  A[43][40]=(4.9310064935064934e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12-3.2873376623376621e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+6.9399350649350649e-04*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39-2.3668831168831169e-02*Jinv21*w44+3.2873376623376621e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20+1.9724025974025973e-03*Jinv01*w3-9.8620129870129864e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38-2.9586038961038961e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-9.8620129870129864e-04*w25*Jinv12-1.6436688311688311e-03*w0*Jinv02-1.6436688311688311e-03*Jinv21*w2-3.5503246753246752e-02*w43*Jinv10-2.3668831168831169e-02*w42*Jinv01-1.1323051948051949e-03*w11*Jinv21-3.9448051948051946e-03*w16*Jinv11-3.9448051948051946e-03*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-1.1323051948051949e-03*w10*Jinv12-9.8620129870129864e-04*w48*Jinv00-9.8620129870129864e-04*Jinv02*w24-3.9448051948051946e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-3.2873376623376621e-03*Jinv21*w26-1.6436688311688311e-03*w0*Jinv01-2.7759740259740259e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10+1.9724025974025973e-03*Jinv22*w20-2.3668831168831169e-02*w43*Jinv11-2.9586038961038961e-03*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41-2.3668831168831169e-02*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.1323051948051949e-03*w11*Jinv22-1.6436688311688311e-03*Jinv11*w1+3.6160714285714286e-03*w13*Jinv11-3.9448051948051946e-03*w15*Jinv01+6.9399350649350649e-04*w28*Jinv11-9.8620129870129864e-04*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29-2.9586038961038961e-03*w46*Jinv10-1.1834415584415585e-02*Jinv00*w33+4.9310064935064934e-03*w31*Jinv10+1.9724025974025973e-03*Jinv02*w3-9.8620129870129864e-04*w21*Jinv00+3.6160714285714286e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39-1.6436688311688311e-03*Jinv22*w2+3.9448051948051946e-03*Jinv00*w18+3.9448051948051946e-03*w36*Jinv00+1.9724025974025973e-03*Jinv21*w38-1.9724025974025973e-03*Jinv02*w45-3.2873376623376621e-03*Jinv01*w24+6.9399350649350649e-04*Jinv22*w59+2.6298701298701297e-03*w19*Jinv11-9.8620129870129864e-04*Jinv21*w53-1.9724025974025973e-03*w51*Jinv00-3.9448051948051946e-03*Jinv12*w34-2.3668831168831169e-02*Jinv22*w44+3.2873376623376621e-03*w4*Jinv10+1.9724025974025973e-03*w4*Jinv11-1.7751623376623376e-02*w41*Jinv20+2.9586038961038961e-03*Jinv12*w13+3.9448051948051946e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+2.9586038961038961e-03*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12-2.9586038961038961e-03*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41-1.9724025974025973e-03*w26*Jinv20-2.2646103896103898e-03*w11*Jinv20-3.2873376623376621e-03*w52*Jinv12+2.9586038961038961e-03*w31*Jinv11-9.8620129870129864e-04*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14+1.9724025974025973e-03*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54+1.9724025974025973e-03*w4*Jinv12-9.8620129870129864e-04*w49*Jinv10+6.9399350649350649e-04*Jinv21*w29-1.7751623376623376e-02*w39*Jinv00+2.9586038961038961e-03*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+1.9724025974025973e-03*w37*Jinv11-1.1323051948051949e-03*Jinv02*w9+1.3149350649350649e-03*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21+3.9448051948051946e-03*Jinv20*w38-9.8620129870129864e-04*w52*Jinv11+3.6160714285714286e-03*w31*Jinv12-1.9724025974025973e-03*w46*Jinv12-1.9724025974025973e-03*Jinv20*w53-9.8620129870129864e-04*w22*Jinv10-2.3011363636363637e-03*Jinv22*w50-2.3011363636363637e-03*w22*Jinv11-2.3011363636363637e-03*w21*Jinv01-9.8620129870129864e-04*Jinv01*w51+3.6160714285714286e-03*Jinv02*w30-2.3011363636363637e-03*Jinv12*w49+1.9724025974025973e-03*w36*Jinv01-1.9724025974025973e-03*w52*Jinv10-2.2646103896103898e-03*w9*Jinv00+3.9448051948051946e-03*w37*Jinv10-1.9724025974025973e-03*Jinv21*w56-1.9724025974025973e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+1.9724025974025973e-03*w19*Jinv12+1.9724025974025973e-03*Jinv22*w5-1.9724025974025973e-03*w47*Jinv22+3.6160714285714286e-03*Jinv21*w14-2.2646103896103898e-03*w10*Jinv10-3.5503246753246752e-02*Jinv20*w44+6.9399350649350649e-04*w58*Jinv12+1.9724025974025973e-03*Jinv02*w18+3.6160714285714286e-03*Jinv01*w12-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-3.9448051948051946e-03*Jinv02*w33-2.7759740259740259e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-2.3011363636363637e-03*Jinv21*w23-1.9724025974025973e-03*Jinv01*w54-1.9724025974025973e-03*w55*Jinv11-1.9724025974025973e-03*w25*Jinv10-2.7759740259740259e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-2.3011363636363637e-03*w48*Jinv02-1.1323051948051949e-03*Jinv01*w9-1.1834415584415585e-02*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00-1.7751623376623376e-02*w40*Jinv10+3.9448051948051946e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+2.9586038961038961e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36-2.3668831168831169e-02*Jinv12*w43+3.2873376623376621e-03*w5*Jinv20+2.9586038961038961e-03*Jinv01*w30-3.2873376623376621e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-3.2873376623376621e-03*w25*Jinv11-2.9586038961038961e-03*w55*Jinv10-3.5503246753246752e-02*w42*Jinv00-2.9586038961038961e-03*w47*Jinv20+6.9399350649350649e-04*Jinv01*w27-1.6436688311688311e-03*Jinv12*w1-1.1323051948051949e-03*w10*Jinv11)*D;
  A[43][41]=(0.0)*D;
  A[43][42]=(0.0)*D;
  A[43][43]=(-4.9310064935064934e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-3.9448051948051946e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+2.2646103896103898e-03*w57*Jinv02-4.9310064935064934e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+3.5503246753246752e-02*Jinv21*w44-3.2873376623376621e-03*w3*Jinv00-4.9310064935064934e-03*w32*Jinv20-4.9310064935064934e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23+1.9724025974025973e-03*Jinv22*w38+2.9586038961038961e-03*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59+2.9586038961038961e-03*w25*Jinv12+2.7759740259740259e-03*w0*Jinv02+2.7759740259740259e-03*Jinv21*w2+3.5503246753246752e-02*w43*Jinv10+3.5503246753246752e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-3.9448051948051946e-03*Jinv01*w45-3.9448051948051946e-03*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+1.7751623376623376e-02*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+9.8620129870129864e-04*w48*Jinv00+2.9586038961038961e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-3.9448051948051946e-03*Jinv21*w26+2.7759740259740259e-03*w0*Jinv01+2.7759740259740259e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10+2.9586038961038961e-03*Jinv22*w20+3.5503246753246752e-02*w43*Jinv11+2.9586038961038961e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+3.5503246753246752e-02*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+2.7759740259740259e-03*Jinv11*w1-3.2873376623376621e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+2.2646103896103898e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29+2.9586038961038961e-03*w46*Jinv10+1.7751623376623376e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12-4.9310064935064934e-03*w31*Jinv10-4.9310064935064934e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00-3.2873376623376621e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+2.7759740259740259e-03*Jinv22*w2-3.9448051948051946e-03*Jinv00*w18-3.9448051948051946e-03*w36*Jinv00+2.9586038961038961e-03*Jinv21*w38+1.9724025974025973e-03*Jinv02*w45-3.9448051948051946e-03*Jinv01*w24+2.2646103896103898e-03*Jinv22*w59+1.7751623376623376e-02*w34*Jinv11+1.9724025974025973e-03*w19*Jinv11+2.9586038961038961e-03*Jinv21*w53+1.9724025974025973e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+3.5503246753246752e-02*Jinv22*w44-3.2873376623376621e-03*w4*Jinv10-4.9310064935064934e-03*w4*Jinv11+1.7751623376623376e-02*w41*Jinv20-4.9310064935064934e-03*Jinv12*w13-3.9448051948051946e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-4.9310064935064934e-03*w32*Jinv21+1.9724025974025973e-03*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+1.9724025974025973e-03*w37*Jinv12+2.9586038961038961e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+1.9724025974025973e-03*w26*Jinv20+2.2646103896103898e-03*w11*Jinv20-3.9448051948051946e-03*w52*Jinv12+1.7751623376623376e-02*Jinv21*w35-4.9310064935064934e-03*w31*Jinv11+2.9586038961038961e-03*Jinv22*w26-3.9448051948051946e-03*w46*Jinv11-4.9310064935064934e-03*Jinv20*w14-4.9310064935064934e-03*w5*Jinv21-3.9448051948051946e-03*Jinv02*w54-4.9310064935064934e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10+2.2646103896103898e-03*Jinv21*w29+1.7751623376623376e-02*Jinv01*w33+1.7751623376623376e-02*w39*Jinv00-4.9310064935064934e-03*Jinv02*w12+1.9724025974025973e-03*Jinv01*w18+2.9586038961038961e-03*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-3.9448051948051946e-03*w47*Jinv21-3.9448051948051946e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+2.9586038961038961e-03*w52*Jinv11-3.2873376623376621e-03*w31*Jinv12+1.9724025974025973e-03*w46*Jinv12+1.9724025974025973e-03*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01+2.9586038961038961e-03*Jinv01*w51-3.2873376623376621e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49+2.9586038961038961e-03*w36*Jinv01+1.9724025974025973e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+2.2646103896103898e-03*w9*Jinv00-3.9448051948051946e-03*w37*Jinv10+1.9724025974025973e-03*Jinv21*w56+1.9724025974025973e-03*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27+9.8620129870129864e-04*w6*Jinv02+2.9586038961038961e-03*w19*Jinv12-4.9310064935064934e-03*Jinv22*w5+1.9724025974025973e-03*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11-3.2873376623376621e-03*Jinv21*w14+2.2646103896103898e-03*w10*Jinv10+3.5503246753246752e-02*Jinv20*w44+2.2646103896103898e-03*w58*Jinv12+2.9586038961038961e-03*Jinv02*w18-3.2873376623376621e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+2.7759740259740259e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+1.9724025974025973e-03*Jinv01*w54+1.9724025974025973e-03*w55*Jinv11+1.9724025974025973e-03*w25*Jinv10+2.7759740259740259e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-4.9310064935064934e-03*w30*Jinv00+1.7751623376623376e-02*w40*Jinv10-3.9448051948051946e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29-4.9310064935064934e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+1.9724025974025973e-03*Jinv02*w36+3.5503246753246752e-02*Jinv12*w43-3.2873376623376621e-03*w5*Jinv20-4.9310064935064934e-03*Jinv01*w30-3.9448051948051946e-03*Jinv02*w51-3.9448051948051946e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-3.9448051948051946e-03*w25*Jinv11+2.9586038961038961e-03*w55*Jinv10+3.5503246753246752e-02*w42*Jinv00+2.9586038961038961e-03*w47*Jinv20+1.7751623376623376e-02*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+2.2646103896103898e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01+2.7759740259740259e-03*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[43][44]=(0.0)*D;
  A[43][45]=(0.0)*D;
  A[43][46]=(9.8620129870129864e-04*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53+4.9310064935064932e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39+1.3806818181818182e-02*Jinv21*w44-8.7662337662337668e-04*Jinv01*w3-7.6704545454545450e-04*Jinv22*w38+4.3831168831168834e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+9.6793831168831166e-04*Jinv21*w2+1.3806818181818182e-02*w42*Jinv01+1.0592532467532468e-03*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+4.7666396103896100e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01+3.1777597402597403e-03*Jinv21*w26+9.6793831168831166e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+1.3806818181818182e-02*w43*Jinv11+6.9034090909090911e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+4.9310064935064932e-04*w11*Jinv22+9.6793831168831166e-04*Jinv11*w1-2.5202922077922079e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17+2.1367694805194807e-03*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-5.4788961038961042e-05*w32*Jinv22+6.9034090909090911e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+3.1777597402597403e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.7394480519480520e-03*w19*Jinv11-1.6436688311688311e-03*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-8.7662337662337668e-04*w4*Jinv11-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w32*Jinv21-2.7394480519480520e-03*Jinv21*w20-7.6704545454545450e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41-7.6704545454545450e-04*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+4.7666396103896100e-03*w46*Jinv11-8.7662337662337668e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.3149350649350649e-03*Jinv02*w12-2.7394480519480520e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+4.7666396103896100e-03*w47*Jinv21+2.1367694805194807e-03*Jinv22*w23-1.6436688311688311e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-3.8352272727272725e-04*Jinv22*w8-3.8352272727272725e-04*Jinv22*w50+1.5888798701298701e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+1.5888798701298701e-03*w21*Jinv01-1.6436688311688311e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-6.5746753246753243e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-8.2183441558441553e-04*w49*Jinv11-2.5202922077922079e-03*Jinv21*w14+4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-2.5202922077922079e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+1.5888798701298701e-03*Jinv21*w23-6.5746753246753243e-04*Jinv01*w54-6.5746753246753243e-04*w55*Jinv11-3.8352272727272725e-04*w48*Jinv02+1.0592532467532468e-03*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+6.9034090909090911e-03*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.3149350649350649e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+3.1777597402597403e-03*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12-8.2183441558441553e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-8.2183441558441553e-04*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1+1.0592532467532468e-03*w10*Jinv11)*D;
  A[43][47]=(0.0)*D;
  A[43][48]=(0.0)*D;
  A[43][49]=(5.4788961038961042e-05*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10+3.6160714285714286e-03*w40*Jinv12-2.6846590909090908e-03*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8+1.6436688311688311e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10+3.6160714285714286e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-2.6846590909090908e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+3.6160714285714286e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-3.2873376623376621e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+7.6704545454545450e-04*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24+1.6436688311688311e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00+3.6160714285714286e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13+7.6704545454545450e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+1.3149350649350649e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56+3.6160714285714286e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-2.6846590909090908e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-2.6846590909090908e-03*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12-3.2873376623376621e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-3.2873376623376621e-04*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53-3.2873376623376621e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-5.4788961038961036e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-3.2873376623376621e-04*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961036e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-3.2873376623376621e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+1.6436688311688311e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+3.6160714285714286e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-5.4788961038961036e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-2.6846590909090908e-03*Jinv02*w51-2.6846590909090908e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[43][50]=(0.0)*D;
  A[43][51]=(0.0)*D;
  A[43][52]=(-5.4788961038961042e-05*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+4.7666396103896100e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-7.6704545454545450e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-5.4788961038961042e-05*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-3.8352272727272725e-04*Jinv20*w23-6.5746753246753243e-04*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+9.6793831168831166e-04*w0*Jinv02+1.9724025974025973e-03*w43*Jinv10+3.1777597402597403e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+6.9034090909090911e-03*Jinv22*w35+2.7394480519480518e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-1.6436688311688311e-03*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+1.3806818181818182e-02*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+2.1367694805194807e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00-2.5202922077922079e-03*w32*Jinv22+9.6793831168831166e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-2.7394480519480520e-03*Jinv02*w45-7.6704545454545450e-04*Jinv22*w59+4.2735389610389614e-03*w51*Jinv00+6.9034090909090911e-03*Jinv12*w34+1.3806818181818182e-02*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-8.7662337662337668e-04*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-3.8352272727272725e-04*w6*Jinv00-6.5746753246753243e-04*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+4.7666396103896100e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-5.4788961038961042e-05*Jinv20*w14+3.1777597402597403e-03*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-8.7662337662337668e-04*Jinv02*w12+4.3831168831168834e-04*Jinv02*w9-3.8352272727272725e-04*w7*Jinv10-2.1367694805194807e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-2.5202922077922079e-03*w31*Jinv12-2.7394480519480520e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-8.2183441558441553e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+1.5888798701298701e-03*Jinv22*w50-8.2183441558441553e-04*Jinv12*w7-2.5202922077922079e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49+4.2735389610389614e-03*w52*Jinv10+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10-7.6704545454545450e-04*w24*Jinv00+1.0592532467532468e-03*Jinv02*w27-8.2183441558441553e-04*w6*Jinv02-1.6436688311688311e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-2.7394480519480520e-03*w47*Jinv22+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-7.6704545454545450e-04*w58*Jinv12-1.6436688311688311e-03*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+6.9034090909090911e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-7.6704545454545450e-04*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.5888798701298701e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20+1.0592532467532468e-03*Jinv22*w29-8.7662337662337668e-04*Jinv22*w14-6.5746753246753243e-04*Jinv02*w36+1.3806818181818182e-02*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+4.7666396103896100e-03*Jinv02*w51+3.1777597402597403e-03*Jinv22*w56+1.0592532467532468e-03*Jinv12*w28+4.2735389610389614e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+9.6793831168831166e-04*Jinv12*w1)*D;
  A[43][53]=(0.0)*D;
  A[43][54]=(0.0)*D;
  A[43][55]=(6.9034090909090911e-03*w40*Jinv12+3.1777597402597403e-03*Jinv22*w53-7.6704545454545450e-04*w57*Jinv02+6.9034090909090911e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3-2.7394480519480520e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-1.6436688311688311e-03*w25*Jinv12+9.6793831168831166e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2+1.9724025974025973e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+4.7666396103896100e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+1.0592532467532468e-03*w10*Jinv12-1.6436688311688311e-03*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41+1.3806818181818182e-02*w42*Jinv02+1.0592532467532468e-03*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17-8.2183441558441553e-04*w22*Jinv12-8.7662337662337668e-04*Jinv02*w3-2.5202922077922079e-03*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39+9.6793831168831166e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-6.5746753246753243e-04*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24-7.6704545454545450e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+1.3806818181818182e-02*Jinv22*w44-5.4788961038961042e-05*w4*Jinv11-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-2.7394480519480520e-03*w37*Jinv12+6.9034090909090911e-03*Jinv22*w41+3.1777597402597403e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11-1.6436688311688311e-03*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+4.7666396103896100e-03*Jinv02*w54-8.7662337662337668e-04*w4*Jinv12-8.2183441558441553e-04*w21*Jinv02+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.3149350649350649e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+1.0592532467532468e-03*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-2.1367694805194807e-03*w47*Jinv21-8.2183441558441553e-04*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-2.5202922077922079e-03*w31*Jinv12-6.5746753246753243e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+1.5888798701298701e-03*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-2.5202922077922079e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-3.8352272727272725e-04*w7*Jinv11+4.2735389610389614e-03*Jinv21*w56+4.3831168831168834e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-8.7662337662337668e-04*Jinv22*w5-6.5746753246753243e-04*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-7.6704545454545450e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-3.8352272727272725e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11+1.5888798701298701e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+4.3831168831168834e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-2.7394480519480520e-03*Jinv02*w36+1.3806818181818182e-02*Jinv12*w43-1.3149350649350649e-03*Jinv01*w30+3.1777597402597403e-03*Jinv02*w51+4.7666396103896100e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01+9.6793831168831166e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[43][56]=(0.0)*D;
  A[43][57]=(0.0)*D;
  A[43][58]=(-1.8262987012987013e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-1.8262987012987013e-03*Jinv02*w39-3.2873376623376621e-04*Jinv22*w38+4.3831168831168834e-04*w25*Jinv12-1.9480519480519481e-04*w0*Jinv02+6.5746753246753243e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+4.3831168831168834e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35+4.3831168831168834e-04*Jinv22*w20-3.6525974025974025e-03*w42*Jinv02-4.2613636363636367e-05*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+2.1915584415584417e-04*w22*Jinv12+3.4699675324675324e-04*Jinv02*w3+2.9220779220779221e-04*w32*Jinv22-1.9480519480519481e-04*Jinv22*w2-3.2873376623376621e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-1.8262987012987013e-03*Jinv12*w34-3.6525974025974025e-03*Jinv22*w44+3.4699675324675324e-04*Jinv12*w13-3.2873376623376621e-04*w37*Jinv12-1.8262987012987013e-03*Jinv22*w41+6.5746753246753243e-04*w52*Jinv12+4.3831168831168834e-04*Jinv22*w26+6.5746753246753243e-04*Jinv02*w54+3.4699675324675324e-04*w4*Jinv12+2.1915584415584417e-04*w21*Jinv02+3.4699675324675324e-04*Jinv02*w12-4.2613636363636367e-05*Jinv02*w9+2.1915584415584417e-04*Jinv22*w23+2.9220779220779221e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+2.1915584415584417e-04*Jinv22*w8+3.2873376623376621e-04*Jinv22*w50+2.1915584415584417e-04*Jinv12*w7+2.9220779220779221e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.2613636363636367e-05*Jinv02*w27+2.1915584415584417e-04*w6*Jinv02+4.3831168831168834e-04*w19*Jinv12+3.4699675324675324e-04*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+4.3831168831168834e-04*Jinv02*w18-1.8262987012987013e-03*Jinv02*w33+3.2873376623376621e-04*w48*Jinv02-4.2613636363636367e-05*Jinv22*w29+3.4699675324675324e-04*Jinv22*w14-3.2873376623376621e-04*Jinv02*w36-3.6525974025974025e-03*Jinv12*w43+6.5746753246753243e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-1.9480519480519481e-04*Jinv12*w1)*D;
  A[43][59]=(0.0)*D;
  A[44][0]=(0.0)*D;
  A[44][1]=(0.0)*D;
  A[44][2]=(3.1047077922077924e-04*Jinv00*w12-3.0438311688311688e-05*w28*Jinv10+5.1136363636363640e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53-2.1915584415584417e-04*Jinv20*w8-3.0438311688311688e-05*w57*Jinv02+3.1047077922077924e-04*w13*Jinv10+5.1136363636363640e-04*Jinv02*w39-5.5519480519480519e-03*Jinv21*w44+3.1047077922077924e-04*w3*Jinv00+3.1047077922077924e-04*w32*Jinv20+3.1047077922077924e-04*Jinv01*w3-2.1915584415584417e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-3.0438311688311688e-05*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-2.4350649350649351e-04*w0*Jinv02-2.4350649350649351e-04*Jinv21*w2-5.5519480519480519e-03*w43*Jinv10-5.5519480519480519e-03*w42*Jinv01-3.0438311688311688e-05*w11*Jinv21+5.1136363636363640e-04*w16*Jinv11+5.1136363636363640e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-5.4788961038961036e-04*Jinv12*w55-3.0438311688311688e-05*w58*Jinv10+5.1136363636363640e-04*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12-2.1915584415584417e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+5.1136363636363640e-04*Jinv22*w35-3.0438311688311688e-05*w58*Jinv11-3.0438311688311688e-05*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-2.4350649350649351e-04*w0*Jinv01-2.4350649350649351e-04*w1*Jinv10+5.1136363636363640e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20-5.5519480519480519e-03*w43*Jinv11-5.4788961038961036e-04*w45*Jinv00+5.1136363636363640e-04*Jinv21*w41-5.5519480519480519e-03*w42*Jinv02-3.0438311688311688e-05*Jinv00*w27-3.0438311688311688e-05*w11*Jinv22-2.4350649350649351e-04*Jinv11*w1+3.1047077922077924e-04*w13*Jinv11+5.1136363636363640e-04*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11-2.1915584415584417e-04*Jinv20*w50-3.0438311688311688e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10+5.1136363636363640e-04*Jinv22*w17+5.1136363636363640e-04*Jinv00*w33-2.1915584415584417e-04*w22*Jinv12+3.1047077922077924e-04*w31*Jinv10+3.1047077922077924e-04*Jinv02*w3-2.1915584415584417e-04*w21*Jinv00+3.1047077922077924e-04*w32*Jinv22+5.1136363636363640e-04*Jinv01*w39-2.4350649350649351e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-3.0438311688311688e-05*Jinv22*w59+5.1136363636363640e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+5.1136363636363640e-04*Jinv12*w34-5.5519480519480519e-03*Jinv22*w44+3.1047077922077924e-04*w4*Jinv10+3.1047077922077924e-04*w4*Jinv11+5.1136363636363640e-04*w41*Jinv20+3.1047077922077924e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+5.1136363636363640e-04*w34*Jinv10+3.1047077922077924e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.1915584415584417e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56+5.1136363636363640e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+5.1136363636363640e-04*Jinv21*w35+3.1047077922077924e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+3.1047077922077924e-04*Jinv20*w14+3.1047077922077924e-04*w5*Jinv21-5.4788961038961036e-04*Jinv02*w54+3.1047077922077924e-04*w4*Jinv12-2.1915584415584417e-04*w21*Jinv02-2.1915584415584417e-04*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29+5.1136363636363640e-04*Jinv01*w33+5.1136363636363640e-04*w39*Jinv00+3.1047077922077924e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-3.0438311688311688e-05*Jinv02*w9-2.1915584415584417e-04*w6*Jinv01-2.1915584415584417e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-2.1915584415584417e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.1047077922077924e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-2.1915584415584417e-04*Jinv22*w8-2.1915584415584417e-04*w22*Jinv10-2.1915584415584417e-04*Jinv22*w50-2.1915584415584417e-04*w22*Jinv11-2.1915584415584417e-04*Jinv12*w7-2.1915584415584417e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+3.1047077922077924e-04*Jinv02*w30-2.1915584415584417e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-2.1915584415584417e-04*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-2.1915584415584417e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.1047077922077924e-04*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-2.1915584415584417e-04*w49*Jinv11+3.1047077922077924e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-5.5519480519480519e-03*Jinv20*w44-3.0438311688311688e-05*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+3.1047077922077924e-04*Jinv01*w12-2.1915584415584417e-04*Jinv21*w8-3.0438311688311688e-05*w57*Jinv00-3.0438311688311688e-05*Jinv20*w59+5.1136363636363640e-04*Jinv02*w33-2.4350649350649351e-04*w0*Jinv00+5.1136363636363640e-04*w35*Jinv20-2.1915584415584417e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-2.4350649350649351e-04*Jinv20*w2+5.1136363636363640e-04*w15*Jinv00-2.1915584415584417e-04*w48*Jinv02-3.0438311688311688e-05*Jinv01*w9+5.1136363636363640e-04*Jinv20*w17+3.1047077922077924e-04*w30*Jinv00+5.1136363636363640e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+3.1047077922077924e-04*Jinv22*w14+5.1136363636363640e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36-5.5519480519480519e-03*Jinv12*w43+3.1047077922077924e-04*w5*Jinv20+3.1047077922077924e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-5.4788961038961036e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11-5.4788961038961036e-04*w55*Jinv10-5.5519480519480519e-03*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20+5.1136363636363640e-04*w16*Jinv12-2.1915584415584417e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27-2.1915584415584417e-04*w48*Jinv01-2.4350649350649351e-04*Jinv12*w1-3.0438311688311688e-05*w10*Jinv11)*D;
  A[44][3]=(0.0)*D;
  A[44][4]=(0.0)*D;
  A[44][5]=(3.6525974025974026e-05*w28*Jinv10-6.5746753246753243e-04*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8+7.3051948051948052e-05*w57*Jinv02-6.5746753246753243e-04*Jinv02*w39+9.8620129870129868e-03*Jinv21*w44-5.4788961038961036e-04*w3*Jinv00-1.6436688311688311e-03*Jinv01*w3+3.2873376623376621e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54+7.3051948051948052e-05*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+6.7573051948051951e-04*w0*Jinv02+6.7573051948051951e-04*Jinv21*w2+6.5746753246753243e-03*w43*Jinv10+9.8620129870129868e-03*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+1.0957792207792208e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+3.2873376623376621e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35+7.3051948051948052e-05*w58*Jinv11+7.3051948051948052e-05*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+6.7573051948051951e-04*w0*Jinv01+1.8262987012987014e-04*w1*Jinv10-2.3011363636363637e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+9.8620129870129868e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+9.8620129870129868e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22+6.7573051948051951e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01+7.3051948051948052e-05*w28*Jinv11+3.2873376623376621e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-2.3011363636363637e-03*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-1.6436688311688311e-03*Jinv02*w3+3.2873376623376621e-04*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39+6.7573051948051951e-04*Jinv22*w2+2.5750811688311690e-03*Jinv00*w18+2.5750811688311690e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+7.3051948051948052e-05*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+2.6298701298701297e-03*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34+9.8620129870129868e-03*Jinv22*w44-5.4788961038961036e-04*w4*Jinv10-1.6436688311688311e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.0957792207792207e-03*Jinv12*w13+2.5750811688311690e-03*w19*Jinv10-2.3011363636363637e-03*w34*Jinv10-1.0957792207792207e-03*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56-6.5746753246753243e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35-1.0957792207792207e-03*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+1.0957792207792208e-04*w46*Jinv11-1.6436688311688311e-03*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54-1.6436688311688311e-03*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+3.2873376623376621e-04*w49*Jinv10+7.3051948051948052e-05*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-1.0957792207792207e-03*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9+8.2183441558441553e-04*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10+1.0957792207792208e-04*w47*Jinv21+2.5750811688311690e-03*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+8.2183441558441553e-04*Jinv22*w8+3.2873376623376621e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+8.2183441558441553e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+8.2183441558441553e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+2.5750811688311690e-03*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+8.2183441558441553e-04*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.6436688311688311e-03*Jinv22*w5+1.0957792207792208e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+6.5746753246753243e-03*Jinv20*w44+7.3051948051948052e-05*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+8.2183441558441553e-04*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33+1.8262987012987014e-04*w0*Jinv00-2.3011363636363637e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+1.8262987012987014e-04*Jinv20*w2-2.3011363636363637e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9-2.3011363636363637e-03*Jinv20*w17-1.9724025974025973e-03*w40*Jinv10+2.5750811688311690e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-1.0957792207792207e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36+9.8620129870129868e-03*Jinv12*w43-5.4788961038961036e-04*w5*Jinv20-1.0957792207792207e-03*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+6.5746753246753243e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+7.3051948051948052e-05*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+6.7573051948051951e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[44][6]=(0.0)*D;
  A[44][7]=(0.0)*D;
  A[44][8]=(-6.5746753246753243e-04*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53-5.4788961038961036e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-6.5746753246753243e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20+1.3149350649350649e-03*Jinv01*w3-3.2873376623376621e-04*Jinv20*w23-4.2735389610389614e-03*Jinv22*w38+5.4788961038961042e-05*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+2.1367694805194807e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21-9.8620129870129864e-04*w16*Jinv11-9.8620129870129864e-04*Jinv21*w17+7.6704545454545450e-04*Jinv01*w45+7.6704545454545450e-04*Jinv12*w55+3.6525974025974026e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12-3.2873376623376621e-04*w48*Jinv00+2.1367694805194807e-03*Jinv02*w24-9.8620129870129864e-04*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+2.1367694805194807e-03*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10+3.6160714285714286e-03*w16*Jinv10-4.2735389610389614e-03*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+5.4788961038961042e-05*w45*Jinv00+1.3149350649350649e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-9.8620129870129864e-04*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-3.2873376623376621e-04*Jinv20*w50+3.6525974025974026e-05*Jinv20*w29+5.4788961038961042e-05*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+3.6160714285714286e-03*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10+1.3149350649350649e-03*Jinv02*w3-3.2873376623376621e-04*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+1.3149350649350649e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2-2.6846590909090908e-03*Jinv00*w18-2.6846590909090908e-03*w36*Jinv00-4.2735389610389614e-03*Jinv21*w38+7.6704545454545450e-04*Jinv02*w45+2.1367694805194807e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11-4.2735389610389614e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00-9.8620129870129864e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+1.3149350649350649e-03*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13-2.6846590909090908e-03*w19*Jinv10+3.6160714285714286e-03*w34*Jinv10+5.4788961038961042e-05*w32*Jinv21-4.2735389610389614e-03*Jinv21*w20-5.4788961038961036e-04*w6*Jinv00-4.2735389610389614e-03*w37*Jinv12+5.4788961038961042e-05*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20+1.6436688311688311e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+5.4788961038961042e-05*w31*Jinv11+2.1367694805194807e-03*Jinv22*w26+7.6704545454545450e-04*w46*Jinv11-6.5746753246753243e-04*Jinv20*w14+1.3149350649350649e-03*w5*Jinv21+7.6704545454545450e-04*Jinv02*w54+1.3149350649350649e-03*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02-3.2873376623376621e-04*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-4.2735389610389614e-03*Jinv01*w18-4.2735389610389614e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-5.4788961038961036e-04*w7*Jinv10+7.6704545454545450e-04*w47*Jinv21-2.6846590909090908e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53-2.1367694805194807e-03*Jinv22*w8-3.2873376623376621e-04*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-2.1367694805194807e-03*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49-4.2735389610389614e-03*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+1.6436688311688311e-04*w9*Jinv00-2.6846590909090908e-03*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00-4.9310064935064932e-04*Jinv02*w27-2.1367694805194807e-03*w6*Jinv02-4.2735389610389614e-03*w19*Jinv12+1.3149350649350649e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.6436688311688311e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-4.2735389610389614e-03*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8+3.6525974025974026e-05*w57*Jinv00+3.6525974025974026e-05*Jinv20*w59-9.8620129870129864e-04*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00+3.6160714285714286e-03*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2+3.6160714285714286e-03*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9+3.6160714285714286e-03*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-2.6846590909090908e-03*w20*Jinv20-4.9310064935064932e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14+1.3149350649350649e-03*w40*Jinv11-4.2735389610389614e-03*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+5.4788961038961042e-05*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.6704545454545450e-04*Jinv22*w56-4.9310064935064932e-04*Jinv12*w28+2.1367694805194807e-03*w25*Jinv11+5.4788961038961042e-05*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+5.4788961038961042e-05*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[44][9]=(0.0)*D;
  A[44][10]=(0.0)*D;
  A[44][11]=(3.4699675324675324e-04*Jinv00*w12-4.2613636363636367e-05*w28*Jinv10+3.2873376623376621e-04*Jinv20*w8+3.4699675324675324e-04*w13*Jinv10+2.9220779220779221e-04*w3*Jinv00+3.4699675324675324e-04*w32*Jinv20+2.1915584415584417e-04*Jinv20*w23+4.3831168831168834e-04*Jinv00*w54-3.6525974025974025e-03*w43*Jinv10-4.2613636363636367e-05*w58*Jinv10+2.1915584415584417e-04*w48*Jinv00-1.9480519480519481e-04*w1*Jinv10-1.8262987012987013e-03*w16*Jinv10+4.3831168831168834e-04*w45*Jinv00-4.2613636363636367e-05*Jinv00*w27+2.1915584415584417e-04*Jinv20*w50-4.2613636363636367e-05*Jinv20*w29+4.3831168831168834e-04*w46*Jinv10-1.8262987012987013e-03*Jinv00*w33+3.4699675324675324e-04*w31*Jinv10+2.1915584415584417e-04*w21*Jinv00+6.5746753246753243e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-3.2873376623376621e-04*w51*Jinv00+2.9220779220779221e-04*w4*Jinv10-5.1136363636363640e-04*w41*Jinv20+6.5746753246753243e-04*w19*Jinv10-1.8262987012987013e-03*w34*Jinv10+3.2873376623376621e-04*w6*Jinv00+4.3831168831168834e-04*Jinv20*w56-3.2873376623376621e-04*w26*Jinv20+1.2175324675324675e-04*w11*Jinv20+3.4699675324675324e-04*Jinv20*w14+2.1915584415584417e-04*w49*Jinv10-5.1136363636363640e-04*w39*Jinv00+3.2873376623376621e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-3.2873376623376621e-04*Jinv20*w53+2.1915584415584417e-04*w22*Jinv10-3.2873376623376621e-04*w52*Jinv10+1.2175324675324675e-04*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10-3.2873376623376621e-04*w24*Jinv00+1.2175324675324675e-04*w10*Jinv10-3.6525974025974025e-03*Jinv20*w44-4.2613636363636367e-05*w57*Jinv00-4.2613636363636367e-05*Jinv20*w59-1.9480519480519481e-04*w0*Jinv00-1.8262987012987013e-03*w35*Jinv20-3.2873376623376621e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2-1.8262987012987013e-03*w15*Jinv00-1.8262987012987013e-03*Jinv20*w17+3.4699675324675324e-04*w30*Jinv00-5.1136363636363640e-04*w40*Jinv10+6.5746753246753243e-04*w20*Jinv20+2.9220779220779221e-04*w5*Jinv20+4.3831168831168834e-04*w55*Jinv10-3.6525974025974025e-03*w42*Jinv00+4.3831168831168834e-04*w47*Jinv20)*D;
  A[44][12]=(0.0)*D;
  A[44][13]=(0.0)*D;
  A[44][14]=(-1.6436688311688311e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8+7.3051948051948052e-05*w57*Jinv02-1.6436688311688311e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+6.5746753246753243e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.0957792207792207e-03*w32*Jinv20+8.2183441558441553e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+6.7573051948051951e-04*w0*Jinv02+1.8262987012987014e-04*Jinv21*w2+9.8620129870129868e-03*w43*Jinv10+6.5746753246753243e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17+2.5750811688311690e-03*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55+7.3051948051948052e-05*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24-6.5746753246753243e-04*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01+2.5750811688311690e-03*Jinv21*w26+1.8262987012987014e-04*w0*Jinv01+6.7573051948051951e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+6.5746753246753243e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-2.3011363636363637e-03*Jinv21*w41+9.8620129870129868e-03*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+7.3051948051948052e-05*w11*Jinv22+1.8262987012987014e-04*Jinv11*w1-5.4788961038961036e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+8.2183441558441553e-04*w22*Jinv12-1.0957792207792207e-03*w31*Jinv10-1.0957792207792207e-03*Jinv02*w3+8.2183441558441553e-04*w21*Jinv00-1.0957792207792207e-03*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39+6.7573051948051951e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+1.0957792207792208e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+2.5750811688311690e-03*Jinv01*w24+7.3051948051948052e-05*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+1.0957792207792208e-04*w51*Jinv00-6.5746753246753243e-04*Jinv12*w34+9.8620129870129868e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10-9.8620129870129864e-04*w41*Jinv20-1.6436688311688311e-03*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10+1.0957792207792208e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+7.3051948051948052e-05*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.6298701298701297e-03*Jinv22*w26+2.5750811688311690e-03*w46*Jinv11-1.6436688311688311e-03*Jinv20*w14+1.6436688311688311e-04*Jinv02*w54-1.0957792207792207e-03*w4*Jinv12+8.2183441558441553e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-1.6436688311688311e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+2.5750811688311690e-03*w47*Jinv21+1.0957792207792208e-04*Jinv20*w38+8.2183441558441553e-04*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11-1.0957792207792207e-03*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12+1.0957792207792208e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+8.2183441558441553e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51-1.0957792207792207e-03*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+1.0957792207792208e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11+7.3051948051948052e-05*w9*Jinv00+1.0957792207792208e-04*w37*Jinv10+1.0957792207792208e-04*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-1.0957792207792207e-03*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11-5.4788961038961036e-04*Jinv21*w14+7.3051948051948052e-05*w10*Jinv10+9.8620129870129868e-03*Jinv20*w44+7.3051948051948052e-05*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-5.4788961038961036e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+7.3051948051948052e-05*w57*Jinv00+7.3051948051948052e-05*Jinv20*w59-6.5746753246753243e-04*Jinv02*w33+6.7573051948051951e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23+1.0957792207792208e-04*Jinv01*w54+1.0957792207792208e-04*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10+6.7573051948051951e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17-1.0957792207792207e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-1.6436688311688311e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+9.8620129870129868e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20+1.0957792207792208e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+2.5750811688311690e-03*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+9.8620129870129868e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01+6.7573051948051951e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[44][15]=(0.0)*D;
  A[44][16]=(0.0)*D;
  A[44][17]=(2.9586038961038961e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10-1.1834415584415585e-02*w40*Jinv12+3.9448051948051946e-03*Jinv22*w53-2.3011363636363637e-03*Jinv20*w8-2.2646103896103898e-03*w57*Jinv02+2.9586038961038961e-03*w13*Jinv10-1.1834415584415585e-02*Jinv02*w39-2.3668831168831169e-02*Jinv21*w44+3.6160714285714286e-03*w3*Jinv00+1.9724025974025973e-03*w32*Jinv20+2.9586038961038961e-03*Jinv01*w3-1.9724025974025973e-03*Jinv22*w38+1.9724025974025973e-03*Jinv00*w54-1.1323051948051949e-03*Jinv21*w59-2.9586038961038961e-03*w25*Jinv12-2.7759740259740259e-03*w0*Jinv02-1.6436688311688311e-03*Jinv21*w2-2.3668831168831169e-02*w43*Jinv10-2.3668831168831169e-02*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21-5.9172077922077923e-03*w16*Jinv11-5.9172077922077923e-03*Jinv21*w17-3.2873376623376621e-03*Jinv01*w45+3.9448051948051946e-03*Jinv12*w55-1.1323051948051949e-03*w58*Jinv10-1.7751623376623376e-02*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12-2.9586038961038961e-03*Jinv02*w24-1.1834415584415585e-02*Jinv22*w35-1.1323051948051949e-03*w58*Jinv11-1.1323051948051949e-03*w57*Jinv01-4.9310064935064934e-03*Jinv21*w26-1.6436688311688311e-03*w0*Jinv01-1.6436688311688311e-03*w1*Jinv10-5.9172077922077923e-03*w16*Jinv10-2.9586038961038961e-03*Jinv22*w20-2.3668831168831169e-02*w43*Jinv11-9.8620129870129864e-04*w45*Jinv00-3.9448051948051946e-03*Jinv21*w41-3.5503246753246752e-02*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22-1.6436688311688311e-03*Jinv11*w1+3.6160714285714286e-03*w13*Jinv11-5.9172077922077923e-03*w15*Jinv01+6.9399350649350649e-04*w28*Jinv11-2.1915584415584417e-04*Jinv20*w29-9.8620129870129864e-04*w46*Jinv10-1.7751623376623376e-02*Jinv22*w17-3.9448051948051946e-03*Jinv00*w33-9.8620129870129864e-04*w22*Jinv12+1.9724025974025973e-03*w31*Jinv10+4.9310064935064934e-03*Jinv02*w3+3.2873376623376621e-03*w32*Jinv22-3.9448051948051946e-03*Jinv01*w39-2.7759740259740259e-03*Jinv22*w2-4.9310064935064934e-03*Jinv00*w18-3.2873376623376621e-03*w36*Jinv00-9.8620129870129864e-04*Jinv21*w38-1.9724025974025973e-03*Jinv02*w45-4.9310064935064934e-03*Jinv01*w24-2.2646103896103898e-03*Jinv22*w59-1.9724025974025973e-03*w19*Jinv11+1.9724025974025973e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-1.1834415584415585e-02*Jinv12*w34-3.5503246753246752e-02*Jinv22*w44+3.6160714285714286e-03*w4*Jinv10+2.9586038961038961e-03*w4*Jinv11+4.9310064935064934e-03*Jinv12*w13-4.9310064935064934e-03*w19*Jinv10-3.9448051948051946e-03*w34*Jinv10+1.9724025974025973e-03*w32*Jinv21-1.9724025974025973e-03*Jinv21*w20-2.3011363636363637e-03*w6*Jinv00-1.9724025974025973e-03*w37*Jinv12+1.9724025974025973e-03*Jinv20*w56-1.1834415584415585e-02*Jinv22*w41-1.9724025974025973e-03*w26*Jinv20+6.9399350649350649e-04*w11*Jinv20+3.9448051948051946e-03*w52*Jinv12+1.9724025974025973e-03*w31*Jinv11-2.9586038961038961e-03*Jinv22*w26-3.2873376623376621e-03*w46*Jinv11+2.9586038961038961e-03*Jinv20*w14+2.9586038961038961e-03*w5*Jinv21+3.9448051948051946e-03*Jinv02*w54+4.9310064935064934e-03*w4*Jinv12-9.8620129870129864e-04*w21*Jinv02+6.9399350649350649e-04*Jinv21*w29+4.9310064935064934e-03*Jinv02*w12-1.9724025974025973e-03*Jinv01*w18-9.8620129870129864e-04*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9-2.3011363636363637e-03*w7*Jinv10-3.2873376623376621e-03*w47*Jinv21-3.2873376623376621e-03*Jinv20*w38-9.8620129870129864e-04*Jinv22*w23+1.9724025974025973e-03*w52*Jinv11+3.2873376623376621e-03*w31*Jinv12-1.9724025974025973e-03*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53-9.8620129870129864e-04*Jinv22*w8+1.3149350649350649e-03*Jinv22*w50-2.3011363636363637e-03*w22*Jinv11-9.8620129870129864e-04*Jinv12*w7-2.3011363636363637e-03*w21*Jinv01+1.9724025974025973e-03*Jinv01*w51+3.2873376623376621e-03*Jinv02*w30+1.3149350649350649e-03*Jinv12*w49-9.8620129870129864e-04*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+6.9399350649350649e-04*w9*Jinv00-3.2873376623376621e-03*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56-1.9724025974025973e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27-9.8620129870129864e-04*w6*Jinv02-2.9586038961038961e-03*w19*Jinv12+4.9310064935064934e-03*Jinv22*w5-1.9724025974025973e-03*w47*Jinv22+3.6160714285714286e-03*Jinv21*w14+6.9399350649350649e-04*w10*Jinv10-2.3668831168831169e-02*Jinv20*w44-2.2646103896103898e-03*w58*Jinv12-2.9586038961038961e-03*Jinv02*w18+3.6160714285714286e-03*Jinv01*w12-1.1323051948051949e-03*w57*Jinv00-1.1323051948051949e-03*Jinv20*w59-1.1834415584415585e-02*Jinv02*w33-1.6436688311688311e-03*w0*Jinv00-3.9448051948051946e-03*w35*Jinv20-2.3011363636363637e-03*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11-1.9724025974025973e-03*w25*Jinv10-1.6436688311688311e-03*Jinv20*w2-5.9172077922077923e-03*w15*Jinv00+1.3149350649350649e-03*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9-5.9172077922077923e-03*Jinv20*w17+1.9724025974025973e-03*w30*Jinv00-4.9310064935064934e-03*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29+4.9310064935064934e-03*Jinv22*w14-3.9448051948051946e-03*w40*Jinv11-1.9724025974025973e-03*Jinv02*w36-3.5503246753246752e-02*Jinv12*w43+3.6160714285714286e-03*w5*Jinv20+1.9724025974025973e-03*Jinv01*w30+3.9448051948051946e-03*Jinv02*w51+3.9448051948051946e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28-4.9310064935064934e-03*w25*Jinv11+1.9724025974025973e-03*w55*Jinv10-2.3668831168831169e-02*w42*Jinv00-9.8620129870129864e-04*w47*Jinv20-1.7751623376623376e-02*w16*Jinv12+6.9399350649350649e-04*Jinv01*w27-2.7759740259740259e-03*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[44][18]=(0.0)*D;
  A[44][19]=(0.0)*D;
  A[44][20]=(-1.3149350649350649e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.5888798701298701e-03*Jinv20*w8-1.3149350649350649e-03*w13*Jinv10+1.9724025974025973e-03*Jinv21*w44-2.5202922077922079e-03*w3*Jinv00-8.7662337662337668e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3-8.2183441558441553e-04*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+4.9310064935064932e-04*Jinv21*w59+2.7394480519480518e-04*Jinv21*w2+1.3806818181818182e-02*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21+9.8620129870129864e-04*w16*Jinv11+9.8620129870129864e-04*Jinv21*w17-7.6704545454545450e-04*Jinv01*w45+1.0592532467532468e-03*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01-2.1367694805194807e-03*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01+9.6793831168831166e-04*w1*Jinv10+6.9034090909090911e-03*w16*Jinv10+1.9724025974025973e-03*w43*Jinv11-1.6436688311688311e-03*w45*Jinv00-1.3149350649350649e-03*Jinv21*w41+4.3831168831168834e-04*Jinv00*w27+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+9.8620129870129864e-04*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29-1.6436688311688311e-03*w46*Jinv10+4.6022727272727274e-03*Jinv00*w33-8.7662337662337668e-04*w31*Jinv10-8.2183441558441553e-04*w21*Jinv00-1.3149350649350649e-03*Jinv01*w39+4.7666396103896100e-03*Jinv00*w18+3.1777597402597403e-03*w36*Jinv00+4.2735389610389614e-03*Jinv21*w38-2.1367694805194807e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11+4.2735389610389614e-03*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-2.7394480519480520e-03*w51*Jinv00-2.5202922077922079e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+4.7666396103896100e-03*w19*Jinv10+4.6022727272727274e-03*w34*Jinv10-5.4788961038961042e-05*w32*Jinv21+4.2735389610389614e-03*Jinv21*w20+1.5888798701298701e-03*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56-6.5746753246753243e-04*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-5.4788961038961042e-05*w31*Jinv11-7.6704545454545450e-04*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+4.2735389610389614e-03*Jinv01*w18+4.2735389610389614e-03*w37*Jinv11+2.1367694805194807e-03*w6*Jinv01+1.5888798701298701e-03*w7*Jinv10-7.6704545454545450e-04*w47*Jinv21+3.1777597402597403e-03*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-2.7394480519480520e-03*Jinv20*w53-8.2183441558441553e-04*w22*Jinv10-3.8352272727272725e-04*w22*Jinv11-3.8352272727272725e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+4.2735389610389614e-03*w36*Jinv01-2.7394480519480520e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-7.6704545454545450e-04*w9*Jinv00+3.1777597402597403e-03*w37*Jinv10-7.6704545454545450e-04*Jinv21*w56-6.5746753246753243e-04*w24*Jinv00-3.8352272727272725e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-7.6704545454545450e-04*w10*Jinv10+1.3806818181818182e-02*Jinv20*w44-5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+1.0592532467532468e-03*w57*Jinv00+1.0592532467532468e-03*Jinv20*w59+9.6793831168831166e-04*w0*Jinv00+4.6022727272727274e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-7.6704545454545450e-04*Jinv01*w54-7.6704545454545450e-04*w55*Jinv11-6.5746753246753243e-04*w25*Jinv10+9.6793831168831166e-04*Jinv20*w2+6.9034090909090911e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9+6.9034090909090911e-03*Jinv20*w17-8.7662337662337668e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+4.7666396103896100e-03*w20*Jinv20-1.3149350649350649e-03*w40*Jinv11-2.5202922077922079e-03*w5*Jinv20-5.4788961038961042e-05*Jinv01*w30-2.1367694805194807e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.3806818181818182e-02*w42*Jinv00-1.6436688311688311e-03*w47*Jinv20-3.8352272727272725e-04*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[44][21]=(0.0)*D;
  A[44][22]=(0.0)*D;
  A[44][23]=(1.3149350649350649e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02+1.3149350649350649e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+5.4788961038961042e-05*w32*Jinv20-6.5746753246753243e-04*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+3.6525974025974026e-05*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+3.6160714285714286e-03*w16*Jinv11+3.6160714285714286e-03*Jinv21*w17-2.6846590909090908e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10-9.8620129870129864e-04*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12+3.8352272727272725e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+3.6525974025974026e-05*w58*Jinv11+3.6525974025974026e-05*w57*Jinv01-2.6846590909090908e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+2.1367694805194807e-03*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-4.2735389610389614e-03*w45*Jinv00+3.6160714285714286e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27-4.9310064935064932e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11+3.6160714285714286e-03*w15*Jinv01+1.6436688311688311e-04*w28*Jinv11+3.8352272727272725e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29-4.2735389610389614e-03*w46*Jinv10-9.8620129870129864e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-2.1367694805194807e-03*w22*Jinv12+5.4788961038961042e-05*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+3.6160714285714286e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+2.1367694805194807e-03*Jinv00*w18+7.6704545454545450e-04*w36*Jinv00+5.4788961038961042e-05*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-2.6846590909090908e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11+5.4788961038961042e-05*Jinv21*w53+7.6704545454545450e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-6.5746753246753243e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13+2.1367694805194807e-03*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11-4.2735389610389614e-03*Jinv22*w26-2.6846590909090908e-03*w46*Jinv11+1.3149350649350649e-03*Jinv20*w14-6.5746753246753243e-04*w5*Jinv21+2.1367694805194807e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02+3.8352272727272725e-04*w49*Jinv10+1.6436688311688311e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+5.4788961038961042e-05*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9-3.2873376623376621e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10-2.6846590909090908e-03*w47*Jinv21+7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23+5.4788961038961042e-05*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+7.6704545454545450e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-5.4788961038961036e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-5.4788961038961036e-04*w21*Jinv01+5.4788961038961042e-05*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+5.4788961038961042e-05*w36*Jinv01+7.6704545454545450e-04*w52*Jinv10-3.2873376623376621e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+7.6704545454545450e-04*w37*Jinv10+7.6704545454545450e-04*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22-3.2873376623376621e-04*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.2873376623376621e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-5.4788961038961036e-04*Jinv21*w23+7.6704545454545450e-04*Jinv01*w54+7.6704545454545450e-04*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+3.8352272727272725e-04*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+5.4788961038961042e-05*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+2.1367694805194807e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14+3.6160714285714286e-03*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30+7.6704545454545450e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-2.6846590909090908e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00-4.2735389610389614e-03*w47*Jinv20-9.8620129870129864e-04*w16*Jinv12-3.2873376623376621e-04*Jinv21*w50+1.6436688311688311e-04*Jinv01*w27-3.2873376623376621e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[44][24]=(0.0)*D;
  A[44][25]=(0.0)*D;
  A[44][26]=(-1.3149350649350649e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-3.8352272727272725e-04*Jinv20*w8-1.3149350649350649e-03*w13*Jinv10+1.3806818181818182e-02*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-5.4788961038961042e-05*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+1.0592532467532468e-03*Jinv21*w59+9.6793831168831166e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+1.3806818181818182e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+6.9034090909090911e-03*w16*Jinv11+6.9034090909090911e-03*Jinv21*w17+3.1777597402597403e-03*Jinv01*w45+4.9310064935064932e-04*w58*Jinv10-3.8352272727272725e-04*w48*Jinv00+1.0592532467532468e-03*w58*Jinv11+1.0592532467532468e-03*w57*Jinv01+4.7666396103896100e-03*Jinv21*w26+9.6793831168831166e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10+1.3806818181818182e-02*w43*Jinv11+4.2735389610389614e-03*w45*Jinv00+4.6022727272727274e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27+9.6793831168831166e-04*Jinv11*w1-2.5202922077922079e-03*w13*Jinv11+6.9034090909090911e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29+4.2735389610389614e-03*w46*Jinv10-1.3149350649350649e-03*Jinv00*w33-5.4788961038961042e-05*w31*Jinv10+2.1367694805194807e-03*w21*Jinv00+4.6022727272727274e-03*Jinv01*w39-2.1367694805194807e-03*Jinv00*w18-7.6704545454545450e-04*w36*Jinv00-1.6436688311688311e-03*Jinv21*w38+4.7666396103896100e-03*Jinv01*w24+9.8620129870129864e-04*w34*Jinv11-6.5746753246753243e-04*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.6704545454545450e-04*w51*Jinv00-5.4788961038961042e-05*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20-2.1367694805194807e-03*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-8.7662337662337668e-04*w32*Jinv21-6.5746753246753243e-04*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+4.2735389610389614e-03*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+9.8620129870129864e-04*Jinv21*w35-8.7662337662337668e-04*w31*Jinv11+3.1777597402597403e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-3.8352272727272725e-04*w49*Jinv10-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv01*w18-1.6436688311688311e-03*w37*Jinv11-8.2183441558441553e-04*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10+3.1777597402597403e-03*w47*Jinv21-7.6704545454545450e-04*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-7.6704545454545450e-04*Jinv20*w53+2.1367694805194807e-03*w22*Jinv10+1.5888798701298701e-03*w22*Jinv11+1.5888798701298701e-03*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51-1.6436688311688311e-03*w36*Jinv01-7.6704545454545450e-04*w52*Jinv10-8.2183441558441553e-04*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-7.6704545454545450e-04*w37*Jinv10-2.7394480519480520e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11-2.5202922077922079e-03*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-2.5202922077922079e-03*Jinv01*w12-8.2183441558441553e-04*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+2.7394480519480518e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+1.5888798701298701e-03*Jinv21*w23-2.7394480519480520e-03*Jinv01*w54-2.7394480519480520e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00+4.3831168831168834e-04*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-5.4788961038961042e-05*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-2.1367694805194807e-03*w20*Jinv20+4.6022727272727274e-03*w40*Jinv11-5.4788961038961042e-05*w5*Jinv20-8.7662337662337668e-04*Jinv01*w30+4.7666396103896100e-03*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00+4.2735389610389614e-03*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+4.3831168831168834e-04*w10*Jinv11)*D;
  A[44][27]=(0.0)*D;
  A[44][28]=(0.0)*D;
  A[44][29]=(-3.6525974025974025e-03*Jinv21*w44+3.4699675324675324e-04*Jinv01*w3-4.2613636363636367e-05*Jinv21*w59-1.9480519480519481e-04*Jinv21*w2-3.6525974025974025e-03*w42*Jinv01-4.2613636363636367e-05*w11*Jinv21-1.8262987012987013e-03*w16*Jinv11-1.8262987012987013e-03*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.2613636363636367e-05*w58*Jinv11-4.2613636363636367e-05*w57*Jinv01+6.5746753246753243e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-3.6525974025974025e-03*w43*Jinv11-1.8262987012987013e-03*Jinv21*w41-1.9480519480519481e-04*Jinv11*w1+2.9220779220779221e-04*w13*Jinv11-1.8262987012987013e-03*w15*Jinv01+1.2175324675324675e-04*w28*Jinv11-1.8262987012987013e-03*Jinv01*w39+4.3831168831168834e-04*Jinv21*w38+6.5746753246753243e-04*Jinv01*w24-5.1136363636363640e-04*w34*Jinv11-3.2873376623376621e-04*w19*Jinv11+4.3831168831168834e-04*Jinv21*w53+3.4699675324675324e-04*w4*Jinv11+3.4699675324675324e-04*w32*Jinv21-3.2873376623376621e-04*Jinv21*w20-5.1136363636363640e-04*Jinv21*w35+3.4699675324675324e-04*w31*Jinv11+6.5746753246753243e-04*w46*Jinv11+3.4699675324675324e-04*w5*Jinv21+1.2175324675324675e-04*Jinv21*w29-5.1136363636363640e-04*Jinv01*w33-3.2873376623376621e-04*Jinv01*w18+4.3831168831168834e-04*w37*Jinv11+2.1915584415584417e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+4.3831168831168834e-04*w52*Jinv11+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*w21*Jinv01+4.3831168831168834e-04*Jinv01*w51+4.3831168831168834e-04*w36*Jinv01+2.1915584415584417e-04*w7*Jinv11-3.2873376623376621e-04*Jinv21*w56+2.1915584415584417e-04*w49*Jinv11+2.9220779220779221e-04*Jinv21*w14+2.9220779220779221e-04*Jinv01*w12+2.1915584415584417e-04*Jinv21*w8+3.2873376623376621e-04*Jinv21*w23-3.2873376623376621e-04*Jinv01*w54-3.2873376623376621e-04*w55*Jinv11-4.2613636363636367e-05*Jinv01*w9-1.8262987012987013e-03*w40*Jinv11+3.4699675324675324e-04*Jinv01*w30+6.5746753246753243e-04*w25*Jinv11+2.1915584415584417e-04*Jinv21*w50+1.2175324675324675e-04*Jinv01*w27+2.1915584415584417e-04*w48*Jinv01-4.2613636363636367e-05*w10*Jinv11)*D;
  A[44][30]=(0.0)*D;
  A[44][31]=(0.0)*D;
  A[44][32]=(-1.0957792207792207e-03*Jinv00*w12+7.3051948051948052e-05*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12+2.5750811688311690e-03*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-1.0957792207792207e-03*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39+9.8620129870129868e-03*Jinv21*w44-1.0957792207792207e-03*w3*Jinv00-1.6436688311688311e-03*w32*Jinv20-1.0957792207792207e-03*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+1.8262987012987014e-04*w0*Jinv02+6.7573051948051951e-04*Jinv21*w2+9.8620129870129868e-03*w43*Jinv10+9.8620129870129868e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17+1.6436688311688311e-04*Jinv01*w45+2.5750811688311690e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+8.2183441558441553e-04*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+1.0957792207792208e-04*Jinv21*w26+6.7573051948051951e-04*w0*Jinv01+6.7573051948051951e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+9.8620129870129868e-03*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+6.5746753246753243e-03*w42*Jinv02+7.3051948051948052e-05*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+6.7573051948051951e-04*Jinv11*w1-1.0957792207792207e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01+7.3051948051948052e-05*w28*Jinv11+8.2183441558441553e-04*Jinv20*w50+7.3051948051948052e-05*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12-1.6436688311688311e-03*w31*Jinv10+7.1225649350649346e-04*w21*Jinv00-5.4788961038961036e-04*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39+1.8262987012987014e-04*Jinv22*w2+1.0957792207792208e-04*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38+1.0957792207792208e-04*Jinv02*w45+1.0957792207792208e-04*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+2.6298701298701297e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34+6.5746753246753243e-03*Jinv22*w44-1.0957792207792207e-03*w4*Jinv10-1.0957792207792207e-03*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+1.0957792207792208e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-1.6436688311688311e-03*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20+7.3051948051948052e-05*w11*Jinv20+2.5750811688311690e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35-1.6436688311688311e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+1.6436688311688311e-04*w46*Jinv11-1.0957792207792207e-03*Jinv20*w14-1.0957792207792207e-03*w5*Jinv21+2.5750811688311690e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02+8.2183441558441553e-04*w49*Jinv10+7.3051948051948052e-05*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+1.0957792207792208e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+1.6436688311688311e-04*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23+2.6298701298701297e-03*w52*Jinv11-5.4788961038961036e-04*w31*Jinv12+1.0957792207792208e-04*w46*Jinv12+2.6298701298701297e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50+7.1225649350649346e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-5.4788961038961036e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+2.6298701298701297e-03*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+7.3051948051948052e-05*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12+1.0957792207792208e-04*w47*Jinv22+8.2183441558441553e-04*w49*Jinv11-1.0957792207792207e-03*Jinv21*w14+7.3051948051948052e-05*w10*Jinv10+9.8620129870129868e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.0957792207792207e-03*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33+6.7573051948051951e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+6.7573051948051951e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02+7.3051948051948052e-05*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-1.6436688311688311e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+1.0957792207792208e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+6.5746753246753243e-03*Jinv12*w43-1.0957792207792207e-03*w5*Jinv20-1.6436688311688311e-03*Jinv01*w30+2.5750811688311690e-03*Jinv02*w51+2.5750811688311690e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+1.0957792207792208e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+9.8620129870129868e-03*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+8.2183441558441553e-04*Jinv21*w50+7.3051948051948052e-05*Jinv01*w27+8.2183441558441553e-04*w48*Jinv01+1.8262987012987014e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[44][33]=(0.0)*D;
  A[44][34]=(0.0)*D;
  A[44][35]=(1.9724025974025973e-03*Jinv00*w12-1.1323051948051949e-03*w28*Jinv10-3.9448051948051946e-03*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53-2.3011363636363637e-03*Jinv20*w8+6.9399350649350649e-04*w57*Jinv02+1.9724025974025973e-03*w13*Jinv10-3.9448051948051946e-03*Jinv02*w39-3.5503246753246752e-02*Jinv21*w44+3.6160714285714286e-03*w3*Jinv00+2.9586038961038961e-03*w32*Jinv20+4.9310064935064934e-03*Jinv01*w3-1.9724025974025973e-03*Jinv22*w38-9.8620129870129864e-04*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59+1.9724025974025973e-03*w25*Jinv12-1.6436688311688311e-03*w0*Jinv02-2.7759740259740259e-03*Jinv21*w2-2.3668831168831169e-02*w43*Jinv10-3.5503246753246752e-02*w42*Jinv01-4.3831168831168834e-04*w11*Jinv21-1.1834415584415585e-02*w16*Jinv11-1.1834415584415585e-02*Jinv21*w17+3.9448051948051946e-03*Jinv01*w45-3.2873376623376621e-03*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10-2.1915584415584417e-04*w10*Jinv12+1.9724025974025973e-03*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01+3.9448051948051946e-03*Jinv21*w26-2.7759740259740259e-03*w0*Jinv01-1.6436688311688311e-03*w1*Jinv10-3.9448051948051946e-03*w16*Jinv10-9.8620129870129864e-04*Jinv22*w20-3.5503246753246752e-02*w43*Jinv11+1.9724025974025973e-03*w45*Jinv00-1.1834415584415585e-02*Jinv21*w41-2.3668831168831169e-02*w42*Jinv02-1.1323051948051949e-03*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-2.7759740259740259e-03*Jinv11*w1+3.2873376623376621e-03*w13*Jinv11-1.1834415584415585e-02*w15*Jinv01-2.2646103896103898e-03*w28*Jinv11-1.1323051948051949e-03*Jinv20*w29+1.9724025974025973e-03*w46*Jinv10-5.9172077922077923e-03*Jinv00*w33+2.9586038961038961e-03*w31*Jinv10+2.9586038961038961e-03*Jinv02*w3+3.6160714285714286e-03*w32*Jinv22-1.1834415584415585e-02*Jinv01*w39-1.6436688311688311e-03*Jinv22*w2-3.2873376623376621e-03*Jinv00*w18-4.9310064935064934e-03*w36*Jinv00-2.9586038961038961e-03*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+3.9448051948051946e-03*Jinv01*w24+6.9399350649350649e-04*Jinv22*w59-1.7751623376623376e-02*w34*Jinv11-1.9724025974025973e-03*w19*Jinv11-2.9586038961038961e-03*Jinv21*w53-1.9724025974025973e-03*w51*Jinv00-5.9172077922077923e-03*Jinv12*w34-2.3668831168831169e-02*Jinv22*w44+3.6160714285714286e-03*w4*Jinv10+4.9310064935064934e-03*w4*Jinv11+1.9724025974025973e-03*Jinv12*w13-3.2873376623376621e-03*w19*Jinv10-5.9172077922077923e-03*w34*Jinv10+4.9310064935064934e-03*w32*Jinv21-1.9724025974025973e-03*Jinv21*w20-2.3011363636363637e-03*w6*Jinv00-1.9724025974025973e-03*w37*Jinv12-9.8620129870129864e-04*Jinv20*w56-3.9448051948051946e-03*Jinv22*w41+2.6298701298701297e-03*w26*Jinv20+6.9399350649350649e-04*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12-1.7751623376623376e-02*Jinv21*w35+4.9310064935064934e-03*w31*Jinv11+1.9724025974025973e-03*Jinv22*w26+3.9448051948051946e-03*w46*Jinv11+1.9724025974025973e-03*Jinv20*w14+4.9310064935064934e-03*w5*Jinv21-3.2873376623376621e-03*Jinv02*w54+2.9586038961038961e-03*w4*Jinv12-2.2646103896103898e-03*Jinv21*w29-1.7751623376623376e-02*Jinv01*w33+1.9724025974025973e-03*Jinv02*w12-1.9724025974025973e-03*Jinv01*w18-2.9586038961038961e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-9.8620129870129864e-04*w6*Jinv01-2.3011363636363637e-03*w7*Jinv10+3.9448051948051946e-03*w47*Jinv21-4.9310064935064934e-03*Jinv20*w38-2.9586038961038961e-03*w52*Jinv11+3.6160714285714286e-03*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12-1.9724025974025973e-03*Jinv20*w53-2.3011363636363637e-03*Jinv22*w50+1.3149350649350649e-03*w22*Jinv11+1.3149350649350649e-03*w21*Jinv01-2.9586038961038961e-03*Jinv01*w51+3.6160714285714286e-03*Jinv02*w30-2.3011363636363637e-03*Jinv12*w49-2.9586038961038961e-03*w36*Jinv01-1.9724025974025973e-03*w52*Jinv10-9.8620129870129864e-04*w7*Jinv11+6.9399350649350649e-04*w9*Jinv00-4.9310064935064934e-03*w37*Jinv10-1.9724025974025973e-03*Jinv21*w56+2.6298701298701297e-03*w24*Jinv00-1.1323051948051949e-03*Jinv02*w27-9.8620129870129864e-04*w19*Jinv12+2.9586038961038961e-03*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22-9.8620129870129864e-04*w49*Jinv11+3.2873376623376621e-03*Jinv21*w14+6.9399350649350649e-04*w10*Jinv10-2.3668831168831169e-02*Jinv20*w44+6.9399350649350649e-04*w58*Jinv12-9.8620129870129864e-04*Jinv02*w18+3.2873376623376621e-03*Jinv01*w12-9.8620129870129864e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33-1.6436688311688311e-03*w0*Jinv00-5.9172077922077923e-03*w35*Jinv20+1.3149350649350649e-03*Jinv21*w23-1.9724025974025973e-03*Jinv01*w54-1.9724025974025973e-03*w55*Jinv11+2.6298701298701297e-03*w25*Jinv10-1.6436688311688311e-03*Jinv20*w2-3.9448051948051946e-03*w15*Jinv00-2.3011363636363637e-03*w48*Jinv02-4.3831168831168834e-04*Jinv01*w9-3.9448051948051946e-03*Jinv20*w17+2.9586038961038961e-03*w30*Jinv00-3.2873376623376621e-03*w20*Jinv20-1.1323051948051949e-03*Jinv22*w29+1.9724025974025973e-03*Jinv22*w14-1.1834415584415585e-02*w40*Jinv11-1.9724025974025973e-03*Jinv02*w36-2.3668831168831169e-02*Jinv12*w43+3.6160714285714286e-03*w5*Jinv20+4.9310064935064934e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-3.2873376623376621e-03*Jinv22*w56-1.1323051948051949e-03*Jinv12*w28+3.9448051948051946e-03*w25*Jinv11-9.8620129870129864e-04*w55*Jinv10-2.3668831168831169e-02*w42*Jinv00+1.9724025974025973e-03*w47*Jinv20-9.8620129870129864e-04*Jinv21*w50-2.2646103896103898e-03*Jinv01*w27-9.8620129870129864e-04*w48*Jinv01-1.6436688311688311e-03*Jinv12*w1-4.3831168831168834e-04*w10*Jinv11)*D;
  A[44][36]=(0.0)*D;
  A[44][37]=(0.0)*D;
  A[44][38]=(-8.7662337662337668e-04*Jinv00*w12+1.0592532467532468e-03*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12-2.1367694805194807e-03*Jinv22*w53+1.5888798701298701e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-8.7662337662337668e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39-2.5202922077922079e-03*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23+4.2735389610389614e-03*Jinv22*w38-1.6436688311688311e-03*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+1.3806818181818182e-02*w43*Jinv10-7.6704545454545450e-04*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-8.2183441558441553e-04*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+9.8620129870129864e-04*Jinv22*w35+9.6793831168831166e-04*w1*Jinv10+4.6022727272727274e-03*w16*Jinv10+4.2735389610389614e-03*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+1.9724025974025973e-03*w42*Jinv02+1.0592532467532468e-03*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-8.2183441558441553e-04*Jinv20*w50+1.0592532467532468e-03*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+6.9034090909090911e-03*Jinv00*w33-3.8352272727272725e-04*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+2.7394480519480518e-04*Jinv22*w2+3.1777597402597403e-03*Jinv00*w18+4.7666396103896100e-03*w36*Jinv00-7.6704545454545450e-04*Jinv02*w45+4.9310064935064932e-04*Jinv22*w59-6.5746753246753243e-04*w51*Jinv00+9.8620129870129864e-04*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-2.5202922077922079e-03*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13+3.1777597402597403e-03*w19*Jinv10+6.9034090909090911e-03*w34*Jinv10+1.5888798701298701e-03*w6*Jinv00+4.2735389610389614e-03*w37*Jinv12-1.6436688311688311e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-2.7394480519480520e-03*w26*Jinv20-7.6704545454545450e-04*w11*Jinv20-2.1367694805194807e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-8.7662337662337668e-04*Jinv20*w14-7.6704545454545450e-04*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-8.2183441558441553e-04*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9+1.5888798701298701e-03*w7*Jinv10+4.7666396103896100e-03*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12-6.5746753246753243e-04*Jinv20*w53+2.1367694805194807e-03*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+2.1367694805194807e-03*Jinv12*w7-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-6.5746753246753243e-04*w52*Jinv10-7.6704545454545450e-04*w9*Jinv00+4.7666396103896100e-03*w37*Jinv10-2.7394480519480520e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+2.1367694805194807e-03*w6*Jinv02+4.2735389610389614e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22-7.6704545454545450e-04*w10*Jinv10+1.3806818181818182e-02*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12+4.2735389610389614e-03*Jinv02*w18+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+9.8620129870129864e-04*Jinv02*w33+9.6793831168831166e-04*w0*Jinv00+6.9034090909090911e-03*w35*Jinv20-2.7394480519480520e-03*w25*Jinv10+9.6793831168831166e-04*Jinv20*w2+4.6022727272727274e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02+4.6022727272727274e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+3.1777597402597403e-03*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+4.2735389610389614e-03*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-2.5202922077922079e-03*w5*Jinv20-2.1367694805194807e-03*Jinv02*w51-7.6704545454545450e-04*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-1.6436688311688311e-03*w55*Jinv10+1.3806818181818182e-02*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+2.7394480519480518e-04*Jinv12*w1)*D;
  A[44][39]=(0.0)*D;
  A[44][40]=(0.0)*D;
  A[44][41]=(4.9310064935064934e-03*Jinv00*w12-4.3831168831168834e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12-3.2873376623376621e-03*Jinv22*w53+1.3149350649350649e-03*Jinv20*w8+6.9399350649350649e-04*w57*Jinv02+4.9310064935064934e-03*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39-2.3668831168831169e-02*Jinv21*w44+3.2873376623376621e-03*w3*Jinv00+4.9310064935064934e-03*w32*Jinv20+1.9724025974025973e-03*Jinv01*w3-9.8620129870129864e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38-2.9586038961038961e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59-9.8620129870129864e-04*w25*Jinv12-1.6436688311688311e-03*w0*Jinv02-1.6436688311688311e-03*Jinv21*w2-3.5503246753246752e-02*w43*Jinv10-2.3668831168831169e-02*w42*Jinv01-1.1323051948051949e-03*w11*Jinv21-3.9448051948051946e-03*w16*Jinv11-3.9448051948051946e-03*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55-4.3831168831168834e-04*w58*Jinv10-1.1323051948051949e-03*w10*Jinv12-9.8620129870129864e-04*w48*Jinv00-9.8620129870129864e-04*Jinv02*w24-3.9448051948051946e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-3.2873376623376621e-03*Jinv21*w26-1.6436688311688311e-03*w0*Jinv01-2.7759740259740259e-03*w1*Jinv10-1.1834415584415585e-02*w16*Jinv10+1.9724025974025973e-03*Jinv22*w20-2.3668831168831169e-02*w43*Jinv11-2.9586038961038961e-03*w45*Jinv00-5.9172077922077923e-03*Jinv21*w41-2.3668831168831169e-02*w42*Jinv02-4.3831168831168834e-04*Jinv00*w27-1.1323051948051949e-03*w11*Jinv22-1.6436688311688311e-03*Jinv11*w1+3.6160714285714286e-03*w13*Jinv11-3.9448051948051946e-03*w15*Jinv01+6.9399350649350649e-04*w28*Jinv11-9.8620129870129864e-04*Jinv20*w50-4.3831168831168834e-04*Jinv20*w29-2.9586038961038961e-03*w46*Jinv10-1.1834415584415585e-02*Jinv00*w33+4.9310064935064934e-03*w31*Jinv10+1.9724025974025973e-03*Jinv02*w3-9.8620129870129864e-04*w21*Jinv00+3.6160714285714286e-03*w32*Jinv22-5.9172077922077923e-03*Jinv01*w39-1.6436688311688311e-03*Jinv22*w2+3.9448051948051946e-03*Jinv00*w18+3.9448051948051946e-03*w36*Jinv00+1.9724025974025973e-03*Jinv21*w38-1.9724025974025973e-03*Jinv02*w45-3.2873376623376621e-03*Jinv01*w24+6.9399350649350649e-04*Jinv22*w59+2.6298701298701297e-03*w19*Jinv11-9.8620129870129864e-04*Jinv21*w53-1.9724025974025973e-03*w51*Jinv00-3.9448051948051946e-03*Jinv12*w34-2.3668831168831169e-02*Jinv22*w44+3.2873376623376621e-03*w4*Jinv10+1.9724025974025973e-03*w4*Jinv11-1.7751623376623376e-02*w41*Jinv20+2.9586038961038961e-03*Jinv12*w13+3.9448051948051946e-03*w19*Jinv10-1.1834415584415585e-02*w34*Jinv10+2.9586038961038961e-03*w32*Jinv21+2.6298701298701297e-03*Jinv21*w20+1.3149350649350649e-03*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12-2.9586038961038961e-03*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41-1.9724025974025973e-03*w26*Jinv20-2.2646103896103898e-03*w11*Jinv20-3.2873376623376621e-03*w52*Jinv12+2.9586038961038961e-03*w31*Jinv11-9.8620129870129864e-04*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11+4.9310064935064934e-03*Jinv20*w14+1.9724025974025973e-03*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54+1.9724025974025973e-03*w4*Jinv12-9.8620129870129864e-04*w49*Jinv10+6.9399350649350649e-04*Jinv21*w29-1.7751623376623376e-02*w39*Jinv00+2.9586038961038961e-03*Jinv02*w12+2.6298701298701297e-03*Jinv01*w18+1.9724025974025973e-03*w37*Jinv11-1.1323051948051949e-03*Jinv02*w9+1.3149350649350649e-03*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21+3.9448051948051946e-03*Jinv20*w38-9.8620129870129864e-04*w52*Jinv11+3.6160714285714286e-03*w31*Jinv12-1.9724025974025973e-03*w46*Jinv12-1.9724025974025973e-03*Jinv20*w53-9.8620129870129864e-04*w22*Jinv10-2.3011363636363637e-03*Jinv22*w50-2.3011363636363637e-03*w22*Jinv11-2.3011363636363637e-03*w21*Jinv01-9.8620129870129864e-04*Jinv01*w51+3.6160714285714286e-03*Jinv02*w30-2.3011363636363637e-03*Jinv12*w49+1.9724025974025973e-03*w36*Jinv01-1.9724025974025973e-03*w52*Jinv10-2.2646103896103898e-03*w9*Jinv00+3.9448051948051946e-03*w37*Jinv10-1.9724025974025973e-03*Jinv21*w56-1.9724025974025973e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+1.9724025974025973e-03*w19*Jinv12+1.9724025974025973e-03*Jinv22*w5-1.9724025974025973e-03*w47*Jinv22+3.6160714285714286e-03*Jinv21*w14-2.2646103896103898e-03*w10*Jinv10-3.5503246753246752e-02*Jinv20*w44+6.9399350649350649e-04*w58*Jinv12+1.9724025974025973e-03*Jinv02*w18+3.6160714285714286e-03*Jinv01*w12-4.3831168831168834e-04*w57*Jinv00-4.3831168831168834e-04*Jinv20*w59-3.9448051948051946e-03*Jinv02*w33-2.7759740259740259e-03*w0*Jinv00-1.1834415584415585e-02*w35*Jinv20-2.3011363636363637e-03*Jinv21*w23-1.9724025974025973e-03*Jinv01*w54-1.9724025974025973e-03*w55*Jinv11-1.9724025974025973e-03*w25*Jinv10-2.7759740259740259e-03*Jinv20*w2-1.1834415584415585e-02*w15*Jinv00-2.3011363636363637e-03*w48*Jinv02-1.1323051948051949e-03*Jinv01*w9-1.1834415584415585e-02*Jinv20*w17+4.9310064935064934e-03*w30*Jinv00-1.7751623376623376e-02*w40*Jinv10+3.9448051948051946e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+2.9586038961038961e-03*Jinv22*w14-5.9172077922077923e-03*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36-2.3668831168831169e-02*Jinv12*w43+3.2873376623376621e-03*w5*Jinv20+2.9586038961038961e-03*Jinv01*w30-3.2873376623376621e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-3.2873376623376621e-03*w25*Jinv11-2.9586038961038961e-03*w55*Jinv10-3.5503246753246752e-02*w42*Jinv00-2.9586038961038961e-03*w47*Jinv20+6.9399350649350649e-04*Jinv01*w27-1.6436688311688311e-03*Jinv12*w1-1.1323051948051949e-03*w10*Jinv11)*D;
  A[44][42]=(0.0)*D;
  A[44][43]=(0.0)*D;
  A[44][44]=(-4.9310064935064934e-03*Jinv00*w12+4.3831168831168834e-04*w28*Jinv10+1.1834415584415585e-02*w40*Jinv12-3.9448051948051946e-03*Jinv22*w53-1.3149350649350649e-03*Jinv20*w8+2.2646103896103898e-03*w57*Jinv02-4.9310064935064934e-03*w13*Jinv10+1.1834415584415585e-02*Jinv02*w39+3.5503246753246752e-02*Jinv21*w44-3.2873376623376621e-03*w3*Jinv00-4.9310064935064934e-03*w32*Jinv20-4.9310064935064934e-03*Jinv01*w3+9.8620129870129864e-04*Jinv20*w23+1.9724025974025973e-03*Jinv22*w38+2.9586038961038961e-03*Jinv00*w54+4.3831168831168834e-04*Jinv21*w59+2.9586038961038961e-03*w25*Jinv12+2.7759740259740259e-03*w0*Jinv02+2.7759740259740259e-03*Jinv21*w2+3.5503246753246752e-02*w43*Jinv10+3.5503246753246752e-02*w42*Jinv01+4.3831168831168834e-04*w11*Jinv21+1.1834415584415585e-02*w16*Jinv11+1.1834415584415585e-02*Jinv21*w17-3.9448051948051946e-03*Jinv01*w45-3.9448051948051946e-03*Jinv12*w55+4.3831168831168834e-04*w58*Jinv10+1.7751623376623376e-02*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+9.8620129870129864e-04*w48*Jinv00+2.9586038961038961e-03*Jinv02*w24+1.1834415584415585e-02*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01-3.9448051948051946e-03*Jinv21*w26+2.7759740259740259e-03*w0*Jinv01+2.7759740259740259e-03*w1*Jinv10+1.1834415584415585e-02*w16*Jinv10+2.9586038961038961e-03*Jinv22*w20+3.5503246753246752e-02*w43*Jinv11+2.9586038961038961e-03*w45*Jinv00+1.1834415584415585e-02*Jinv21*w41+3.5503246753246752e-02*w42*Jinv02+4.3831168831168834e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+2.7759740259740259e-03*Jinv11*w1-3.2873376623376621e-03*w13*Jinv11+1.1834415584415585e-02*w15*Jinv01+2.2646103896103898e-03*w28*Jinv11+9.8620129870129864e-04*Jinv20*w50+4.3831168831168834e-04*Jinv20*w29+2.9586038961038961e-03*w46*Jinv10+1.7751623376623376e-02*Jinv22*w17+1.1834415584415585e-02*Jinv00*w33+9.8620129870129864e-04*w22*Jinv12-4.9310064935064934e-03*w31*Jinv10-4.9310064935064934e-03*Jinv02*w3+9.8620129870129864e-04*w21*Jinv00-3.2873376623376621e-03*w32*Jinv22+1.1834415584415585e-02*Jinv01*w39+2.7759740259740259e-03*Jinv22*w2-3.9448051948051946e-03*Jinv00*w18-3.9448051948051946e-03*w36*Jinv00+2.9586038961038961e-03*Jinv21*w38+1.9724025974025973e-03*Jinv02*w45-3.9448051948051946e-03*Jinv01*w24+2.2646103896103898e-03*Jinv22*w59+1.7751623376623376e-02*w34*Jinv11+1.9724025974025973e-03*w19*Jinv11+2.9586038961038961e-03*Jinv21*w53+1.9724025974025973e-03*w51*Jinv00+1.1834415584415585e-02*Jinv12*w34+3.5503246753246752e-02*Jinv22*w44-3.2873376623376621e-03*w4*Jinv10-4.9310064935064934e-03*w4*Jinv11+1.7751623376623376e-02*w41*Jinv20-4.9310064935064934e-03*Jinv12*w13-3.9448051948051946e-03*w19*Jinv10+1.1834415584415585e-02*w34*Jinv10-4.9310064935064934e-03*w32*Jinv21+1.9724025974025973e-03*Jinv21*w20-1.3149350649350649e-03*w6*Jinv00+1.9724025974025973e-03*w37*Jinv12+2.9586038961038961e-03*Jinv20*w56+1.1834415584415585e-02*Jinv22*w41+1.9724025974025973e-03*w26*Jinv20+2.2646103896103898e-03*w11*Jinv20-3.9448051948051946e-03*w52*Jinv12+1.7751623376623376e-02*Jinv21*w35-4.9310064935064934e-03*w31*Jinv11+2.9586038961038961e-03*Jinv22*w26-3.9448051948051946e-03*w46*Jinv11-4.9310064935064934e-03*Jinv20*w14-4.9310064935064934e-03*w5*Jinv21-3.9448051948051946e-03*Jinv02*w54-4.9310064935064934e-03*w4*Jinv12+9.8620129870129864e-04*w21*Jinv02+9.8620129870129864e-04*w49*Jinv10+2.2646103896103898e-03*Jinv21*w29+1.7751623376623376e-02*Jinv01*w33+1.7751623376623376e-02*w39*Jinv00-4.9310064935064934e-03*Jinv02*w12+1.9724025974025973e-03*Jinv01*w18+2.9586038961038961e-03*w37*Jinv11+4.3831168831168834e-04*Jinv02*w9+9.8620129870129864e-04*w6*Jinv01-1.3149350649350649e-03*w7*Jinv10-3.9448051948051946e-03*w47*Jinv21-3.9448051948051946e-03*Jinv20*w38+9.8620129870129864e-04*Jinv22*w23+2.9586038961038961e-03*w52*Jinv11-3.2873376623376621e-03*w31*Jinv12+1.9724025974025973e-03*w46*Jinv12+1.9724025974025973e-03*Jinv20*w53+9.8620129870129864e-04*Jinv22*w8+9.8620129870129864e-04*w22*Jinv10-1.3149350649350649e-03*Jinv22*w50-1.3149350649350649e-03*w22*Jinv11+9.8620129870129864e-04*Jinv12*w7-1.3149350649350649e-03*w21*Jinv01+2.9586038961038961e-03*Jinv01*w51-3.2873376623376621e-03*Jinv02*w30-1.3149350649350649e-03*Jinv12*w49+2.9586038961038961e-03*w36*Jinv01+1.9724025974025973e-03*w52*Jinv10+9.8620129870129864e-04*w7*Jinv11+2.2646103896103898e-03*w9*Jinv00-3.9448051948051946e-03*w37*Jinv10+1.9724025974025973e-03*Jinv21*w56+1.9724025974025973e-03*w24*Jinv00+4.3831168831168834e-04*Jinv02*w27+9.8620129870129864e-04*w6*Jinv02+2.9586038961038961e-03*w19*Jinv12-4.9310064935064934e-03*Jinv22*w5+1.9724025974025973e-03*w47*Jinv22+9.8620129870129864e-04*w49*Jinv11-3.2873376623376621e-03*Jinv21*w14+2.2646103896103898e-03*w10*Jinv10+3.5503246753246752e-02*Jinv20*w44+2.2646103896103898e-03*w58*Jinv12+2.9586038961038961e-03*Jinv02*w18-3.2873376623376621e-03*Jinv01*w12+9.8620129870129864e-04*Jinv21*w8+4.3831168831168834e-04*w57*Jinv00+4.3831168831168834e-04*Jinv20*w59+1.1834415584415585e-02*Jinv02*w33+2.7759740259740259e-03*w0*Jinv00+1.1834415584415585e-02*w35*Jinv20-1.3149350649350649e-03*Jinv21*w23+1.9724025974025973e-03*Jinv01*w54+1.9724025974025973e-03*w55*Jinv11+1.9724025974025973e-03*w25*Jinv10+2.7759740259740259e-03*Jinv20*w2+1.1834415584415585e-02*w15*Jinv00-1.3149350649350649e-03*w48*Jinv02+4.3831168831168834e-04*Jinv01*w9+1.1834415584415585e-02*Jinv20*w17-4.9310064935064934e-03*w30*Jinv00+1.7751623376623376e-02*w40*Jinv10-3.9448051948051946e-03*w20*Jinv20+4.3831168831168834e-04*Jinv22*w29-4.9310064935064934e-03*Jinv22*w14+1.1834415584415585e-02*w40*Jinv11+1.9724025974025973e-03*Jinv02*w36+3.5503246753246752e-02*Jinv12*w43-3.2873376623376621e-03*w5*Jinv20-4.9310064935064934e-03*Jinv01*w30-3.9448051948051946e-03*Jinv02*w51-3.9448051948051946e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-3.9448051948051946e-03*w25*Jinv11+2.9586038961038961e-03*w55*Jinv10+3.5503246753246752e-02*w42*Jinv00+2.9586038961038961e-03*w47*Jinv20+1.7751623376623376e-02*w16*Jinv12+9.8620129870129864e-04*Jinv21*w50+2.2646103896103898e-03*Jinv01*w27+9.8620129870129864e-04*w48*Jinv01+2.7759740259740259e-03*Jinv12*w1+4.3831168831168834e-04*w10*Jinv11)*D;
  A[44][45]=(0.0)*D;
  A[44][46]=(0.0)*D;
  A[44][47]=(9.8620129870129864e-04*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53+4.9310064935064932e-04*w57*Jinv02+9.8620129870129864e-04*Jinv02*w39+1.3806818181818182e-02*Jinv21*w44-8.7662337662337668e-04*Jinv01*w3-7.6704545454545450e-04*Jinv22*w38+4.3831168831168834e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02+9.6793831168831166e-04*Jinv21*w2+1.3806818181818182e-02*w42*Jinv01+1.0592532467532468e-03*w11*Jinv21+4.6022727272727274e-03*w16*Jinv11+4.6022727272727274e-03*Jinv21*w17+4.7666396103896100e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+4.3831168831168834e-04*w58*Jinv11+4.3831168831168834e-04*w57*Jinv01+3.1777597402597403e-03*Jinv21*w26+9.6793831168831166e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+1.3806818181818182e-02*w43*Jinv11+6.9034090909090911e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02+4.9310064935064932e-04*w11*Jinv22+9.6793831168831166e-04*Jinv11*w1-2.5202922077922079e-03*w13*Jinv11+4.6022727272727274e-03*w15*Jinv01-7.6704545454545450e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17+2.1367694805194807e-03*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-5.4788961038961042e-05*w32*Jinv22+6.9034090909090911e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+3.1777597402597403e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-2.7394480519480520e-03*w19*Jinv11-1.6436688311688311e-03*Jinv21*w53-1.3149350649350649e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-8.7662337662337668e-04*w4*Jinv11-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w32*Jinv21-2.7394480519480520e-03*Jinv21*w20-7.6704545454545450e-04*w37*Jinv12+9.8620129870129864e-04*Jinv22*w41-7.6704545454545450e-04*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+4.7666396103896100e-03*w46*Jinv11-8.7662337662337668e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-7.6704545454545450e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.3149350649350649e-03*Jinv02*w12-2.7394480519480520e-03*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01+4.7666396103896100e-03*w47*Jinv21+2.1367694805194807e-03*Jinv22*w23-1.6436688311688311e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-3.8352272727272725e-04*Jinv22*w8-3.8352272727272725e-04*Jinv22*w50+1.5888798701298701e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+1.5888798701298701e-03*w21*Jinv01-1.6436688311688311e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11-6.5746753246753243e-04*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-8.2183441558441553e-04*w49*Jinv11-2.5202922077922079e-03*Jinv21*w14+4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-2.5202922077922079e-03*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8-1.3149350649350649e-03*Jinv02*w33+1.5888798701298701e-03*Jinv21*w23-6.5746753246753243e-04*Jinv01*w54-6.5746753246753243e-04*w55*Jinv11-3.8352272727272725e-04*w48*Jinv02+1.0592532467532468e-03*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+6.9034090909090911e-03*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-1.3149350649350649e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+3.1777597402597403e-03*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12-8.2183441558441553e-04*Jinv21*w50-7.6704545454545450e-04*Jinv01*w27-8.2183441558441553e-04*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1+1.0592532467532468e-03*w10*Jinv11)*D;
  A[44][48]=(0.0)*D;
  A[44][49]=(0.0)*D;
  A[44][50]=(5.4788961038961042e-05*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10+3.6160714285714286e-03*w40*Jinv12-2.6846590909090908e-03*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8+1.6436688311688311e-04*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10+3.6160714285714286e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-2.6846590909090908e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+1.9724025974025973e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+3.6160714285714286e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-3.2873376623376621e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10-6.5746753246753243e-04*Jinv02*w3+3.8352272727272725e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+7.6704545454545450e-04*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24+1.6436688311688311e-04*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00+3.6160714285714286e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20-6.5746753246753243e-04*Jinv12*w13+7.6704545454545450e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+1.3149350649350649e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56+3.6160714285714286e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-2.6846590909090908e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-2.6846590909090908e-03*Jinv02*w54-6.5746753246753243e-04*w4*Jinv12-3.2873376623376621e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00-6.5746753246753243e-04*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38-3.2873376623376621e-04*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53-3.2873376623376621e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-5.4788961038961036e-04*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11-3.2873376623376621e-04*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-5.4788961038961036e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00+3.6525974025974026e-05*Jinv02*w27-3.2873376623376621e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-6.5746753246753243e-04*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+1.6436688311688311e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+3.6160714285714286e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+3.8352272727272725e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-5.4788961038961036e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29-6.5746753246753243e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-2.6846590909090908e-03*Jinv02*w51-2.6846590909090908e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+1.9724025974025973e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[44][51]=(0.0)*D;
  A[44][52]=(0.0)*D;
  A[44][53]=(-5.4788961038961042e-05*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10+4.6022727272727274e-03*w40*Jinv12+4.7666396103896100e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-7.6704545454545450e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10+4.6022727272727274e-03*Jinv02*w39-5.4788961038961042e-05*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-3.8352272727272725e-04*Jinv20*w23-6.5746753246753243e-04*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-2.1367694805194807e-03*w25*Jinv12+9.6793831168831166e-04*w0*Jinv02+1.9724025974025973e-03*w43*Jinv10+3.1777597402597403e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.3831168831168834e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+6.9034090909090911e-03*Jinv22*w35+2.7394480519480518e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-1.6436688311688311e-03*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00+1.3806818181818182e-02*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+4.3831168831168834e-04*w11*Jinv22+2.1367694805194807e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-7.1225649350649346e-04*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10-1.3149350649350649e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00-2.5202922077922079e-03*w32*Jinv22+9.6793831168831166e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-2.7394480519480520e-03*Jinv02*w45-7.6704545454545450e-04*Jinv22*w59+4.2735389610389614e-03*w51*Jinv00+6.9034090909090911e-03*Jinv12*w34+1.3806818181818182e-02*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20-8.7662337662337668e-04*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-3.8352272727272725e-04*w6*Jinv00-6.5746753246753243e-04*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56+4.6022727272727274e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+4.7666396103896100e-03*w52*Jinv12-2.1367694805194807e-03*Jinv22*w26-5.4788961038961042e-05*Jinv20*w14+3.1777597402597403e-03*Jinv02*w54-1.3149350649350649e-03*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+9.8620129870129864e-04*w39*Jinv00-8.7662337662337668e-04*Jinv02*w12+4.3831168831168834e-04*Jinv02*w9-3.8352272727272725e-04*w7*Jinv10-2.1367694805194807e-03*Jinv20*w38-7.1225649350649346e-04*Jinv22*w23-2.5202922077922079e-03*w31*Jinv12-2.7394480519480520e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-8.2183441558441553e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+1.5888798701298701e-03*Jinv22*w50-8.2183441558441553e-04*Jinv12*w7-2.5202922077922079e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49+4.2735389610389614e-03*w52*Jinv10+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10-7.6704545454545450e-04*w24*Jinv00+1.0592532467532468e-03*Jinv02*w27-8.2183441558441553e-04*w6*Jinv02-1.6436688311688311e-03*w19*Jinv12-1.3149350649350649e-03*Jinv22*w5-2.7394480519480520e-03*w47*Jinv22+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-7.6704545454545450e-04*w58*Jinv12-1.6436688311688311e-03*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59+6.9034090909090911e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-7.6704545454545450e-04*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.5888798701298701e-03*w48*Jinv02-1.3149350649350649e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20+1.0592532467532468e-03*Jinv22*w29-8.7662337662337668e-04*Jinv22*w14-6.5746753246753243e-04*Jinv02*w36+1.3806818181818182e-02*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+4.7666396103896100e-03*Jinv02*w51+3.1777597402597403e-03*Jinv22*w56+1.0592532467532468e-03*Jinv12*w28+4.2735389610389614e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12+9.6793831168831166e-04*Jinv12*w1)*D;
  A[44][54]=(0.0)*D;
  A[44][55]=(0.0)*D;
  A[44][56]=(6.9034090909090911e-03*w40*Jinv12+3.1777597402597403e-03*Jinv22*w53-7.6704545454545450e-04*w57*Jinv02+6.9034090909090911e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3-2.7394480519480520e-03*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-1.6436688311688311e-03*w25*Jinv12+9.6793831168831166e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2+1.9724025974025973e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+4.7666396103896100e-03*Jinv12*w55+9.8620129870129864e-04*w15*Jinv02+1.0592532467532468e-03*w10*Jinv12-1.6436688311688311e-03*Jinv02*w24+4.6022727272727274e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01-2.1367694805194807e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+9.8620129870129864e-04*Jinv21*w41+1.3806818181818182e-02*w42*Jinv02+1.0592532467532468e-03*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+9.8620129870129864e-04*Jinv22*w17-8.2183441558441553e-04*w22*Jinv12-8.7662337662337668e-04*Jinv02*w3-2.5202922077922079e-03*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39+9.6793831168831166e-04*Jinv22*w2-2.1367694805194807e-03*Jinv21*w38-6.5746753246753243e-04*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24-7.6704545454545450e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.6022727272727274e-03*Jinv12*w34+1.3806818181818182e-02*Jinv22*w44-5.4788961038961042e-05*w4*Jinv11-1.3149350649350649e-03*Jinv12*w13-1.3149350649350649e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-2.7394480519480520e-03*w37*Jinv12+6.9034090909090911e-03*Jinv22*w41+3.1777597402597403e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11-1.6436688311688311e-03*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+4.7666396103896100e-03*Jinv02*w54-8.7662337662337668e-04*w4*Jinv12-8.2183441558441553e-04*w21*Jinv02+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-1.3149350649350649e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11+1.0592532467532468e-03*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-2.1367694805194807e-03*w47*Jinv21-8.2183441558441553e-04*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-2.5202922077922079e-03*w31*Jinv12-6.5746753246753243e-04*w46*Jinv12-7.1225649350649346e-04*Jinv22*w8+1.5888798701298701e-03*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-7.1225649350649346e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-2.5202922077922079e-03*Jinv02*w30+1.5888798701298701e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01-3.8352272727272725e-04*w7*Jinv11+4.2735389610389614e-03*Jinv21*w56+4.3831168831168834e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-8.7662337662337668e-04*Jinv22*w5-6.5746753246753243e-04*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14-7.6704545454545450e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+4.6022727272727274e-03*Jinv02*w33-3.8352272727272725e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11+1.5888798701298701e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9+4.3831168831168834e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-2.7394480519480520e-03*Jinv02*w36+1.3806818181818182e-02*Jinv12*w43-1.3149350649350649e-03*Jinv01*w30+3.1777597402597403e-03*Jinv02*w51+4.7666396103896100e-03*Jinv22*w56+4.3831168831168834e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11+9.8620129870129864e-04*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01+9.6793831168831166e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[44][57]=(0.0)*D;
  A[44][58]=(0.0)*D;
  A[44][59]=(-1.8262987012987013e-03*w40*Jinv12+6.5746753246753243e-04*Jinv22*w53+1.2175324675324675e-04*w57*Jinv02-1.8262987012987013e-03*Jinv02*w39-3.2873376623376621e-04*Jinv22*w38+4.3831168831168834e-04*w25*Jinv12-1.9480519480519481e-04*w0*Jinv02+6.5746753246753243e-04*Jinv12*w55-5.1136363636363640e-04*w15*Jinv02-4.2613636363636367e-05*w10*Jinv12+4.3831168831168834e-04*Jinv02*w24-1.8262987012987013e-03*Jinv22*w35+4.3831168831168834e-04*Jinv22*w20-3.6525974025974025e-03*w42*Jinv02-4.2613636363636367e-05*w11*Jinv22-5.1136363636363640e-04*Jinv22*w17+2.1915584415584417e-04*w22*Jinv12+3.4699675324675324e-04*Jinv02*w3+2.9220779220779221e-04*w32*Jinv22-1.9480519480519481e-04*Jinv22*w2-3.2873376623376621e-04*Jinv02*w45+1.2175324675324675e-04*Jinv22*w59-1.8262987012987013e-03*Jinv12*w34-3.6525974025974025e-03*Jinv22*w44+3.4699675324675324e-04*Jinv12*w13-3.2873376623376621e-04*w37*Jinv12-1.8262987012987013e-03*Jinv22*w41+6.5746753246753243e-04*w52*Jinv12+4.3831168831168834e-04*Jinv22*w26+6.5746753246753243e-04*Jinv02*w54+3.4699675324675324e-04*w4*Jinv12+2.1915584415584417e-04*w21*Jinv02+3.4699675324675324e-04*Jinv02*w12-4.2613636363636367e-05*Jinv02*w9+2.1915584415584417e-04*Jinv22*w23+2.9220779220779221e-04*w31*Jinv12-3.2873376623376621e-04*w46*Jinv12+2.1915584415584417e-04*Jinv22*w8+3.2873376623376621e-04*Jinv22*w50+2.1915584415584417e-04*Jinv12*w7+2.9220779220779221e-04*Jinv02*w30+3.2873376623376621e-04*Jinv12*w49-4.2613636363636367e-05*Jinv02*w27+2.1915584415584417e-04*w6*Jinv02+4.3831168831168834e-04*w19*Jinv12+3.4699675324675324e-04*Jinv22*w5-3.2873376623376621e-04*w47*Jinv22+1.2175324675324675e-04*w58*Jinv12+4.3831168831168834e-04*Jinv02*w18-1.8262987012987013e-03*Jinv02*w33+3.2873376623376621e-04*w48*Jinv02-4.2613636363636367e-05*Jinv22*w29+3.4699675324675324e-04*Jinv22*w14-3.2873376623376621e-04*Jinv02*w36-3.6525974025974025e-03*Jinv12*w43+6.5746753246753243e-04*Jinv02*w51+6.5746753246753243e-04*Jinv22*w56-4.2613636363636367e-05*Jinv12*w28-5.1136363636363640e-04*w16*Jinv12-1.9480519480519481e-04*Jinv12*w1)*D;
  A[45][0]=(2.6481331168831169e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+2.6481331168831169e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23+2.1915584415584417e-04*Jinv22*w38+5.8441558441558442e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59-7.4878246753246752e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17-1.4975649350649350e-03*Jinv01*w45+5.8441558441558442e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-2.1915584415584417e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24-3.4699675324675324e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-7.4878246753246752e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-1.4975649350649350e-03*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+2.6481331168831169e-04*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-1.4975649350649350e-03*w46*Jinv10-2.1915584415584417e-04*Jinv22*w17-3.4699675324675324e-04*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-1.4975649350649350e-03*Jinv02*w45-7.4878246753246752e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-3.4699675324675324e-04*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-3.4699675324675324e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+2.6481331168831169e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10-3.4699675324675324e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00+2.1915584415584417e-04*w37*Jinv12+5.8441558441558442e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-3.4699675324675324e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-1.4975649350649350e-03*w46*Jinv11+2.6481331168831169e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21+5.8441558441558442e-04*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29-3.4699675324675324e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+2.6481331168831169e-04*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10-1.4975649350649350e-03*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12-1.4975649350649350e-03*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10+5.8441558441558442e-04*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-1.4975649350649350e-03*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+2.6481331168831169e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+2.6481331168831169e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-3.4699675324675324e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-3.4699675324675324e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+5.8441558441558442e-04*Jinv01*w54+5.8441558441558442e-04*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+2.6481331168831169e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11+2.1915584415584417e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+5.8441558441558442e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-7.4878246753246752e-04*w25*Jinv11+5.8441558441558442e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-1.4975649350649350e-03*w47*Jinv20-2.1915584415584417e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[45][1]=(0.0)*D;
  A[45][2]=(0.0)*D;
  A[45][3]=(-8.2183441558441553e-05*Jinv00*w12+7.1225649350649346e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44-3.5612824675324673e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+1.3697240259740259e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+7.1225649350649346e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+8.7662337662337668e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*w11*Jinv22-6.3920454545454550e-05*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+7.1225649350649346e-04*Jinv22*w17+8.2183441558441553e-04*Jinv00*w33-1.9176136363636362e-04*w22*Jinv12-3.5612824675324673e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-4.6570616883116886e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+8.7662337662337668e-04*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+8.7662337662337668e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+8.2183441558441553e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+7.1225649350649346e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+2.7394480519480521e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+8.7662337662337668e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-1.9176136363636362e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+8.7662337662337668e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-4.6570616883116886e-04*Jinv20*w38-1.9176136363636362e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50-1.9176136363636362e-04*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7-1.9176136363636362e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+2.7394480519480521e-05*w9*Jinv00-4.6570616883116886e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+2.7394480519480521e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+8.7662337662337668e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+8.2183441558441553e-04*w35*Jinv20-1.9176136363636362e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+7.1225649350649346e-04*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[45][4]=(0.0)*D;
  A[45][5]=(0.0)*D;
  A[45][6]=(2.4655032467532466e-04*Jinv00*w12+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+1.3697240259740259e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480521e-05*w3*Jinv00+4.6570616883116886e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+1.3697240259740259e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01-2.7394480519480521e-05*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+5.4788961038961042e-05*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*w11*Jinv22+1.0957792207792208e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+3.8352272727272725e-04*Jinv22*w17-8.2183441558441553e-04*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+4.6570616883116886e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480521e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-8.2183441558441553e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+7.1225649350649346e-04*w37*Jinv12+5.4788961038961042e-05*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.4655032467532466e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+7.1225649350649346e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8+5.4788961038961042e-05*Jinv02*w33-2.7394480519480521e-05*w0*Jinv00-8.2183441558441553e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-2.7394480519480521e-05*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-3.2873376623376621e-04*Jinv20*w17+4.6570616883116886e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480521e-05*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+3.8352272727272725e-04*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[45][7]=(0.0)*D;
  A[45][8]=(0.0)*D;
  A[45][9]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10-2.7394480519480521e-05*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+7.4878246753246752e-04*Jinv20*w23-5.8441558441558442e-04*Jinv00*w54+4.3831168831168834e-04*w43*Jinv10-1.4204545454545455e-05*w58*Jinv10-4.2918019480519480e-04*w48*Jinv00+1.0247564935064935e-04*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+1.4975649350649350e-03*w45*Jinv00+6.4935064935064935e-05*Jinv00*w27-4.2918019480519480e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+1.4975649350649350e-03*w46*Jinv10+3.4699675324675324e-04*Jinv00*w33-2.1915584415584417e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00-2.6481331168831169e-04*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00+1.1870941558441559e-04*w51*Jinv00-2.7394480519480521e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-2.6481331168831169e-04*w19*Jinv10+3.4699675324675324e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-5.8441558441558442e-04*Jinv20*w56+3.6525974025974026e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-4.2918019480519480e-04*Jinv20*w14-4.2918019480519480e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-5.4788961038961042e-05*w7*Jinv10-1.0957792207792208e-04*Jinv20*w38+1.1870941558441559e-04*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+1.1870941558441559e-04*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10+3.6525974025974026e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+3.4699675324675324e-04*w35*Jinv20+3.6525974025974026e-05*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00+2.1915584415584417e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-2.6481331168831169e-04*w20*Jinv20-2.7394480519480521e-05*w5*Jinv20-5.8441558441558442e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+1.4975649350649350e-03*w47*Jinv20)*D;
  A[45][10]=(0.0)*D;
  A[45][11]=(0.0)*D;
  A[45][12]=(-5.7528409090909093e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+1.6436688311688311e-03*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-5.7528409090909093e-04*w13*Jinv10+1.6436688311688311e-03*Jinv02*w39+2.5750811688311690e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-1.4793019480519481e-03*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.5750811688311690e-03*w42*Jinv01+3.1960227272727273e-04*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17+4.0543831168831168e-03*Jinv01*w45-1.4793019480519481e-03*Jinv12*w55+3.1047077922077924e-04*w58*Jinv10+8.2183441558441553e-04*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12-1.3697240259740259e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01+2.0271915584415584e-03*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+8.2183441558441553e-04*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+2.5750811688311690e-03*w43*Jinv11+4.7666396103896100e-03*w45*Jinv00+8.7662337662337668e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-7.3965097402597403e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+5.5702110389610384e-04*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+4.7666396103896100e-03*w46*Jinv10+8.2183441558441553e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+8.7662337662337668e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-4.6570616883116886e-04*Jinv21*w38+4.7666396103896100e-03*Jinv02*w45+2.0271915584415584e-03*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-1.1231737012987013e-03*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20-5.7528409090909093e-04*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-5.4788961038961036e-04*w32*Jinv21-1.1231737012987013e-03*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-1.4793019480519481e-03*Jinv20*w56+1.6436688311688311e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-5.4788961038961036e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+4.0543831168831168e-03*w46*Jinv11-5.7528409090909093e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21-1.4793019480519481e-03*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-1.3697240259740259e-04*w49*Jinv10+5.5702110389610384e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00-5.7528409090909093e-04*Jinv02*w12-1.1231737012987013e-03*Jinv01*w18-4.6570616883116886e-04*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10+4.0543831168831168e-03*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+4.7666396103896100e-03*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-4.6570616883116886e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-9.5880681818181818e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5+4.7666396103896100e-03*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11-7.3965097402597403e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18-7.3965097402597403e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8+3.1047077922077924e-04*w57*Jinv00+3.1047077922077924e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-9.5880681818181818e-04*Jinv01*w54-9.5880681818181818e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+8.2183441558441553e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+3.1960227272727273e-04*Jinv01*w9+8.2183441558441553e-04*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-5.7528409090909093e-04*Jinv22*w14+8.7662337662337668e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-5.4788961038961036e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-1.4793019480519481e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+2.0271915584415584e-03*w25*Jinv11-1.4793019480519481e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+4.7666396103896100e-03*w47*Jinv20+8.2183441558441553e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50+5.5702110389610384e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+3.1960227272727273e-04*w10*Jinv11)*D;
  A[45][13]=(0.0)*D;
  A[45][14]=(0.0)*D;
  A[45][15]=(-8.2183441558441553e-04*Jinv00*w12-9.8620129870129864e-04*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-8.2183441558441553e-04*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-3.2873376623376621e-03*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20+2.1367694805194807e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38-5.2962662337662338e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-9.8620129870129864e-04*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+2.1367694805194807e-03*Jinv22*w20-3.2873376623376621e-03*w43*Jinv11-2.3011363636363637e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-4.9310064935064932e-04*w11*Jinv22-3.1047077922077924e-04*Jinv11*w1+9.3141233766233771e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-9.8620129870129864e-04*Jinv22*w17-2.1367694805194807e-03*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+8.2183441558441553e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.7532467532467534e-03*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13+8.2183441558441553e-04*w19*Jinv10+1.7532467532467534e-03*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12-9.8620129870129864e-04*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-4.2735389610389614e-03*Jinv22*w26-3.1777597402597403e-03*w46*Jinv11-8.2183441558441553e-04*Jinv20*w14+2.1367694805194807e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+1.7532467532467534e-03*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-1.0409902597402598e-03*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-1.0409902597402598e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.7532467532467534e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11+9.3141233766233771e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+9.3141233766233771e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+1.3149350649350649e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-1.0409902597402598e-03*Jinv21*w23+1.7532467532467534e-03*Jinv01*w54+1.7532467532467534e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+3.8352272727272725e-04*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-7.1225649350649346e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+8.2183441558441553e-04*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20+7.6704545454545450e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-3.1777597402597403e-03*w25*Jinv11-9.8620129870129864e-04*w42*Jinv00-9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[45][16]=(0.0)*D;
  A[45][17]=(0.0)*D;
  A[45][18]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-8.2183441558441553e-05*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+2.4655032467532466e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.4793019480519481e-03*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59-1.0957792207792208e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17-7.1225649350649346e-04*Jinv01*w45-3.1047077922077924e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-8.2183441558441553e-04*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11-4.7666396103896100e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.0957792207792208e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-4.7666396103896100e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39+5.7528409090909093e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38-1.6436688311688311e-04*Jinv01*w24-5.4788961038961042e-05*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53+1.3697240259740259e-04*w51*Jinv00+2.4655032467532466e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.7528409090909093e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+1.4793019480519481e-03*Jinv20*w56-2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-5.4788961038961042e-05*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-7.1225649350649346e-04*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.6436688311688311e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-3.5612824675324673e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10-7.1225649350649346e-04*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+5.2049512987012989e-04*w52*Jinv11+1.3697240259740259e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01+1.3697240259740259e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-2.1002435064935065e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+5.2049512987012989e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-2.1002435064935065e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-3.1047077922077924e-04*w57*Jinv00-3.1047077922077924e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+5.2049512987012989e-04*Jinv01*w54+5.2049512987012989e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-8.2183441558441553e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-8.2183441558441553e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+5.7528409090909093e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+2.4655032467532466e-04*w5*Jinv20+8.2183441558441553e-05*Jinv01*w30-1.6436688311688311e-04*w25*Jinv11+1.4793019480519481e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00-4.7666396103896100e-03*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[45][19]=(0.0)*D;
  A[45][20]=(0.0)*D;
  A[45][21]=(2.3833198051948050e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-4.2735389610389614e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8-7.8530844155844158e-04*w57*Jinv02+2.3833198051948050e-03*w13*Jinv10-4.2735389610389614e-03*Jinv02*w39-2.6846590909090908e-03*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+7.1225649350649346e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3-3.6982548701298703e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-3.6982548701298703e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-3.5612824675324673e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-2.6846590909090908e-03*w42*Jinv01-3.7439123376623376e-04*w11*Jinv21-5.4788961038961036e-04*w16*Jinv11-5.4788961038961036e-04*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45+2.6298701298701297e-03*Jinv12*w55-7.8530844155844158e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00-3.6982548701298703e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01-2.4655032467532467e-03*Jinv21*w26-3.5612824675324673e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20-2.6846590909090908e-03*w43*Jinv11-7.3965097402597406e-03*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22-3.5612824675324673e-04*Jinv11*w1+1.1505681818181819e-03*w13*Jinv11-5.4788961038961036e-04*w15*Jinv01-1.1414366883116883e-03*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-7.3965097402597406e-03*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-3.6982548701298703e-03*w22*Jinv12+7.1225649350649346e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-3.6982548701298703e-03*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2+2.3833198051948050e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-7.3965097402597406e-03*Jinv02*w45-2.4655032467532467e-03*Jinv01*w24-7.8530844155844158e-04*Jinv22*w59+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+2.3833198051948050e-03*Jinv12*w13+2.3833198051948050e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-4.2735389610389614e-03*Jinv22*w41-3.6982548701298703e-03*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+5.4788961038961036e-04*w31*Jinv11-3.6982548701298703e-03*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11+2.3833198051948050e-03*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21+2.6298701298701297e-03*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-3.6982548701298703e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-1.1414366883116883e-03*Jinv21*w29-4.2735389610389614e-03*w39*Jinv00+2.3833198051948050e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-7.4878246753246752e-04*Jinv02*w9+3.5612824675324673e-04*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-3.6982548701298703e-03*Jinv22*w23+8.2183441558441553e-05*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12-7.3965097402597406e-03*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8-3.6982548701298703e-03*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10-7.4878246753246752e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.3971185064935066e-03*Jinv21*w56-3.6982548701298703e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-7.3965097402597406e-03*w47*Jinv22+1.1505681818181819e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-7.8530844155844158e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.1505681818181819e-03*Jinv01*w12-7.8530844155844158e-04*w57*Jinv00-7.8530844155844158e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+1.3971185064935066e-03*Jinv01*w54+1.3971185064935066e-03*w55*Jinv11-3.6982548701298703e-03*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02-3.7439123376623376e-04*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17+7.1225649350649346e-04*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+2.3833198051948050e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.3833198051948050e-03*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+2.6298701298701297e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-2.4655032467532467e-03*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00-7.3965097402597406e-03*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-1.1414366883116883e-03*Jinv01*w27-7.4878246753246752e-04*Jinv12*w1-3.7439123376623376e-04*w10*Jinv11)*D;
  A[45][22]=(0.0)*D;
  A[45][23]=(0.0)*D;
  A[45][24]=(-2.3833198051948050e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8-2.3833198051948050e-03*w13*Jinv10+3.1777597402597403e-03*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-3.5612824675324673e-04*Jinv01*w3+3.6982548701298703e-03*Jinv20*w23-2.6298701298701297e-03*Jinv00*w54+3.9265422077922079e-04*Jinv21*w59+3.7439123376623376e-04*Jinv21*w2+4.2735389610389614e-03*w43*Jinv10+3.1777597402597403e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45+7.8530844155844158e-04*w58*Jinv10-1.6436688311688311e-04*w48*Jinv00+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.7439123376623376e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10+3.1777597402597403e-03*w43*Jinv11+7.3965097402597406e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+3.7439123376623376e-04*Jinv11*w1-1.0135957792207792e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11-1.6436688311688311e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+7.3965097402597406e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+3.6982548701298703e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.3833198051948050e-03*Jinv00*w18-7.1225649350649346e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+2.4655032467532467e-03*Jinv01*w24+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00-3.5612824675324673e-04*w4*Jinv10-3.5612824675324673e-04*w4*Jinv11+4.2735389610389614e-03*w41*Jinv20-2.3833198051948050e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-2.6298701298701297e-03*Jinv20*w56+3.6982548701298703e-03*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+1.0957792207792207e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.4655032467532467e-03*w46*Jinv11-2.3833198051948050e-03*Jinv20*w14-3.5612824675324673e-04*w5*Jinv21-1.6436688311688311e-04*w49*Jinv10-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33+4.2735389610389614e-03*w39*Jinv00-1.2327516233766234e-03*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-8.2183441558441553e-05*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+2.4655032467532467e-03*w47*Jinv21-7.1225649350649346e-04*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11-1.6436688311688311e-04*Jinv20*w53+3.6982548701298703e-03*w22*Jinv10+1.2327516233766234e-03*w22*Jinv11+1.2327516233766234e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11+7.4878246753246752e-04*w9*Jinv00-7.1225649350649346e-04*w37*Jinv10-1.2327516233766234e-03*Jinv21*w56+3.6982548701298703e-03*w24*Jinv00-8.2183441558441553e-05*w49*Jinv11-1.0135957792207792e-03*Jinv21*w14+7.4878246753246752e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-1.0135957792207792e-03*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+7.8530844155844158e-04*w57*Jinv00+7.8530844155844158e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11+3.6982548701298703e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00+3.9265422077922079e-04*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-2.3833198051948050e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-3.5612824675324673e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+2.4655032467532467e-03*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00+7.3965097402597406e-03*w47*Jinv20-8.2183441558441553e-05*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+3.9265422077922079e-04*w10*Jinv11)*D;
  A[45][25]=(0.0)*D;
  A[45][26]=(0.0)*D;
  A[45][27]=(6.5746753246753243e-04*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3+2.0495129870129870e-04*Jinv21*w59+1.5726461038961039e-04*Jinv21*w2+6.5746753246753243e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17+2.3741883116883116e-03*Jinv01*w45+2.0495129870129870e-04*w58*Jinv11+2.0495129870129870e-04*w57*Jinv01+1.1870941558441558e-03*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+6.5746753246753243e-04*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+1.5726461038961039e-04*Jinv11*w1-6.7573051948051951e-04*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+6.5746753246753243e-04*Jinv01*w39-1.0957792207792208e-04*Jinv21*w38+1.1870941558441558e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-6.7573051948051951e-04*w19*Jinv11-2.6481331168831169e-04*Jinv21*w53-5.4788961038961042e-05*w4*Jinv11-1.0957792207792208e-04*w32*Jinv21-6.7573051948051951e-04*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+2.3741883116883116e-03*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-6.7573051948051951e-04*Jinv01*w18-1.0957792207792208e-04*w37*Jinv11-5.4788961038961042e-05*w6*Jinv01+2.3741883116883116e-03*w47*Jinv21-2.6481331168831169e-04*w52*Jinv11+1.1870941558441558e-03*w22*Jinv11+1.1870941558441558e-03*w21*Jinv01-2.6481331168831169e-04*Jinv01*w51-1.0957792207792208e-04*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11-1.0227272727272728e-03*Jinv21*w56-2.6481331168831169e-04*w49*Jinv11-6.7573051948051951e-04*Jinv21*w14-6.7573051948051951e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+1.1870941558441558e-03*Jinv21*w23-1.0227272727272728e-03*Jinv01*w54-1.0227272727272728e-03*w55*Jinv11+1.5726461038961039e-04*Jinv01*w9+6.5746753246753243e-04*w40*Jinv11-1.0957792207792208e-04*Jinv01*w30+1.1870941558441558e-03*w25*Jinv11-2.6481331168831169e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-2.6481331168831169e-04*w48*Jinv01+1.5726461038961039e-04*w10*Jinv11)*D;
  A[45][28]=(0.0)*D;
  A[45][29]=(0.0)*D;
  A[45][30]=(1.6436688311688311e-04*Jinv00*w12-1.8262987012987013e-05*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-1.1231737012987013e-03*Jinv22*w53-4.6570616883116886e-04*Jinv20*w8-3.5612824675324673e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-1.4793019480519481e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-9.5880681818181818e-04*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+1.2601461038961039e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02-1.8262987012987013e-05*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-1.8262987012987013e-05*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50-1.8262987012987013e-05*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12+1.0683847402597404e-03*w31*Jinv10-3.8352272727272725e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38+1.4245129870129869e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-3.5612824675324673e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-1.3971185064935066e-03*w51*Jinv00+1.2601461038961039e-03*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-5.4788961038961036e-04*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+1.0683847402597404e-03*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-4.6570616883116886e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-1.4793019480519481e-03*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+2.1915584415584417e-04*w11*Jinv20-1.1231737012987013e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+1.0683847402597404e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-9.5880681818181818e-04*Jinv02*w54-5.4788961038961036e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10-1.8262987012987013e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-5.4788961038961036e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9-4.6570616883116886e-04*w6*Jinv01-4.6570616883116886e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+1.4245129870129869e-03*w46*Jinv12-1.3971185064935066e-03*Jinv20*w53-1.9176136363636362e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-1.9176136363636362e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-1.3971185064935066e-03*w52*Jinv10-4.6570616883116886e-04*w7*Jinv11+2.1915584415584417e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10-1.4793019480519481e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-1.9176136363636362e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12+1.4245129870129869e-03*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+2.1915584415584417e-04*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-3.5612824675324673e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-4.6570616883116886e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.2601461038961039e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-1.4793019480519481e-03*Jinv01*w54-1.4793019480519481e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+1.0683847402597404e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+2.1367694805194807e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.0683847402597404e-03*Jinv01*w30-1.1231737012987013e-03*Jinv02*w51-9.5880681818181818e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-1.4793019480519481e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-1.8262987012987013e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[45][31]=(0.0)*D;
  A[45][32]=(0.0)*D;
  A[45][33]=(-7.1225649350649346e-04*Jinv00*w12+6.5746753246753243e-04*w40*Jinv12+2.5750811688311690e-03*Jinv22*w53+8.2183441558441553e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39+3.9448051948051946e-03*Jinv21*w44-8.2183441558441553e-04*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-7.6704545454545450e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38-2.1915584415584417e-04*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+4.2004870129870131e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+3.9448051948051946e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-3.2873376623376621e-03*Jinv01*w45+1.7532467532467534e-03*Jinv12*w55+3.6525974025974026e-05*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+1.6436688311688311e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-2.1915584415584414e-03*Jinv21*w26+4.2004870129870131e-04*w0*Jinv01+3.4699675324675324e-04*w1*Jinv10+3.8352272727272725e-04*Jinv22*w20+3.9448051948051946e-03*w43*Jinv11+1.9724025974025973e-03*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+3.6525974025974026e-05*w11*Jinv22+4.2004870129870131e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+9.8620129870129864e-04*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50-1.3149350649350649e-03*w31*Jinv10-8.2183441558441553e-04*Jinv02*w3+1.0957792207792207e-03*w21*Jinv00-1.2053571428571428e-03*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-2.1915584415584414e-03*Jinv02*w45-2.1915584415584414e-03*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+1.3149350649350649e-03*w51*Jinv00+1.6436688311688311e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-8.2183441558441553e-04*w4*Jinv10-7.6704545454545450e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-1.1505681818181819e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+8.2183441558441553e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20-3.4699675324675324e-04*w11*Jinv20+2.5750811688311690e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-3.2873376623376621e-03*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-7.6704545454545450e-04*w5*Jinv21+1.7532467532467534e-03*Jinv02*w54-8.2183441558441553e-04*w4*Jinv12-1.3149350649350649e-03*w49*Jinv10+9.8620129870129864e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+8.2183441558441553e-04*w7*Jinv10-3.2873376623376621e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+2.6298701298701297e-03*w52*Jinv11-1.2053571428571428e-03*w31*Jinv12-2.1915584415584414e-03*w46*Jinv12+1.3149350649350649e-03*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10+1.2601461038961039e-03*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.0957792207792207e-03*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-1.2053571428571428e-03*Jinv02*w30+1.2601461038961039e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.3149350649350649e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11-3.4699675324675324e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+3.8352272727272725e-04*w19*Jinv12-8.2183441558441553e-04*Jinv22*w5-2.1915584415584414e-03*w47*Jinv22+1.3149350649350649e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-3.4699675324675324e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+1.6436688311688311e-03*Jinv02*w33+3.4699675324675324e-04*w0*Jinv00-1.0957792207792207e-03*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10+3.4699675324675324e-04*Jinv20*w2+1.2601461038961039e-03*w48*Jinv02+7.3051948051948052e-05*Jinv01*w9-1.3149350649350649e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-8.2183441558441553e-04*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30+2.5750811688311690e-03*Jinv02*w51+1.7532467532467534e-03*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-2.1915584415584414e-03*w25*Jinv11+1.9724025974025973e-03*w42*Jinv00+1.3149350649350649e-03*Jinv21*w50+9.8620129870129864e-04*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[45][34]=(0.0)*D;
  A[45][35]=(0.0)*D;
  A[45][36]=(5.4788961038961036e-04*Jinv00*w12+1.8262987012987013e-05*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-1.3697240259740259e-04*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+4.6570616883116886e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-7.1225649350649346e-04*Jinv22*w38+1.4793019480519481e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10+5.2049512987012989e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+1.3971185064935066e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-2.1915584415584417e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02+1.8262987012987013e-05*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.3971185064935066e-03*Jinv20*w50+1.8262987012987013e-05*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-3.8352272727272725e-04*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+2.4655032467532466e-04*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22-1.0957792207792208e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-1.0683847402597404e-03*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45-1.3697240259740259e-04*Jinv22*w59+8.2183441558441553e-05*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+4.6570616883116886e-04*w4*Jinv10-1.6436688311688311e-04*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-7.1225649350649346e-04*w37*Jinv12+1.4793019480519481e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+5.2049512987012989e-04*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26+5.4788961038961036e-04*Jinv20*w14+5.2049512987012989e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.3971185064935066e-03*w49*Jinv10-1.6436688311688311e-04*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-1.0683847402597404e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23+8.2183441558441553e-05*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-3.5612824675324673e-04*Jinv12*w7+8.2183441558441553e-05*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+8.2183441558441553e-05*w52*Jinv10+1.2784090909090910e-04*w9*Jinv00-1.0683847402597404e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.2784090909090910e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.3697240259740259e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-2.1915584415584417e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+3.8352272727272725e-04*w25*Jinv10-2.1915584415584417e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-7.1225649350649346e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+4.6570616883116886e-04*w5*Jinv20+5.2049512987012989e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28+1.4793019480519481e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-3.8352272727272725e-04*w16*Jinv12-1.0957792207792208e-04*Jinv12*w1)*D;
  A[45][37]=(0.0)*D;
  A[45][38]=(0.0)*D;
  A[45][39]=(2.6298701298701297e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.7532467532467534e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-1.6436688311688311e-04*w57*Jinv02+2.6298701298701297e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-4.2735389610389614e-03*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+4.9310064935064934e-03*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2-2.9586038961038961e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-9.6793831168831166e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-9.5332792207792200e-03*Jinv01*w45+2.7942370129870131e-03*Jinv12*w55-6.5746753246753243e-04*w58*Jinv10-2.7394480519480518e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01-5.4788961038961036e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-1.2820616883116883e-02*w45*Jinv00-6.9034090909090911e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+1.0957792207792207e-03*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+1.8628246753246754e-03*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29-1.2820616883116883e-02*w46*Jinv10+1.9724025974025973e-03*Jinv00*w33+1.6436688311688311e-04*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00-6.0267857142857139e-04*w32*Jinv22-6.9034090909090911e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+2.6298701298701297e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38-8.5470779220779228e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-1.6436688311688311e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.5750811688311690e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+2.1367694805194807e-03*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-2.9586038961038961e-03*w41*Jinv20+4.9310064935064932e-04*Jinv12*w13+2.6298701298701297e-03*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10+2.5750811688311690e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+4.9310064935064934e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-5.4788961038961036e-04*w11*Jinv20+1.7532467532467534e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35-2.1367694805194807e-03*Jinv22*w26-9.5332792207792200e-03*w46*Jinv11+2.6298701298701297e-03*Jinv20*w14+2.7942370129870131e-03*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+2.1367694805194807e-03*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33-2.9586038961038961e-03*w39*Jinv00+4.9310064935064932e-04*Jinv02*w12+2.5750811688311690e-03*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9+8.2183441558441553e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-9.5332792207792200e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+8.2183441558441553e-04*w52*Jinv11-6.0267857142857139e-04*w31*Jinv12-8.5470779220779228e-03*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-4.2735389610389614e-03*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50-2.0819805194805196e-03*w22*Jinv11-2.0819805194805196e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-6.0267857142857139e-04*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01+2.1367694805194807e-03*w52*Jinv10+8.2183441558441553e-04*w7*Jinv11-5.4788961038961036e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+2.7942370129870131e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+1.3149350649350649e-03*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5-8.5470779220779228e-03*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+1.8628246753246754e-03*Jinv21*w14-5.4788961038961036e-04*w10*Jinv10-2.9586038961038961e-03*Jinv20*w44-1.6436688311688311e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18+1.8628246753246754e-03*Jinv01*w12+8.2183441558441553e-04*Jinv21*w8-6.5746753246753243e-04*w57*Jinv00-6.5746753246753243e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33-5.4788961038961036e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-2.0819805194805196e-03*Jinv21*w23+2.7942370129870131e-03*Jinv01*w54+2.7942370129870131e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-5.4788961038961036e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-9.6793831168831166e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00-2.9586038961038961e-03*w40*Jinv10+2.6298701298701297e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+4.9310064935064932e-04*Jinv22*w14-6.9034090909090911e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.7532467532467534e-03*Jinv02*w51+2.7942370129870131e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-3.1777597402597403e-03*w25*Jinv11+4.9310064935064934e-03*w55*Jinv10-2.9586038961038961e-03*w42*Jinv00-1.2820616883116883e-02*w47*Jinv20-4.9310064935064932e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-9.6793831168831166e-04*w10*Jinv11)*D;
  A[45][40]=(0.0)*D;
  A[45][41]=(0.0)*D;
  A[45][42]=(-2.6298701298701297e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-2.6298701298701297e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-7.6704545454545450e-04*Jinv22*w38-4.9310064935064934e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02-4.2004870129870131e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+3.2873376623376621e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+6.5746753246753243e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+2.1915584415584414e-03*Jinv21*w26-4.2004870129870131e-04*w0*Jinv01+5.4788961038961036e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+1.2820616883116883e-02*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22-4.2004870129870131e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-9.8620129870129864e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29+1.2820616883116883e-02*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3+4.2735389610389614e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2-2.6298701298701297e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+2.1915584415584414e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59-4.9310064935064934e-03*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11+2.9586038961038961e-03*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-2.6298701298701297e-03*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+1.1505681818181819e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-7.6704545454545450e-04*w37*Jinv12-4.9310064935064934e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+4.2735389610389614e-03*w26*Jinv20+5.4788961038961036e-04*w11*Jinv20-7.6704545454545450e-04*w52*Jinv12-4.9310064935064934e-03*Jinv21*w35+1.1505681818181819e-03*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+3.2873376623376621e-03*w46*Jinv11-2.6298701298701297e-03*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-9.8620129870129864e-04*Jinv21*w29-4.9310064935064934e-03*Jinv01*w33+2.9586038961038961e-03*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+3.2873376623376621e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8+4.2735389610389614e-03*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10-5.4788961038961042e-05*w7*Jinv11+5.4788961038961036e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+5.4788961038961036e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+6.5746753246753243e-04*w57*Jinv00+6.5746753246753243e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+5.4788961038961036e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+5.4788961038961036e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+2.9586038961038961e-03*w40*Jinv10-2.6298701298701297e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+1.1505681818181819e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+2.1915584415584414e-03*w25*Jinv11-4.9310064935064934e-03*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00+1.2820616883116883e-02*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12-1.3149350649350649e-03*Jinv21*w50-9.8620129870129864e-04*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[45][43]=(0.0)*D;
  A[45][44]=(0.0)*D;
  A[45][45]=(4.2735389610389614e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53+7.8530844155844158e-04*w57*Jinv02+4.2735389610389614e-03*Jinv02*w39+4.7666396103896100e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38+6.5746753246753243e-04*Jinv21*w59+3.6982548701298703e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.7666396103896100e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+7.3965097402597406e-03*Jinv01*w45-2.6298701298701297e-03*Jinv12*w55+2.1367694805194807e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+3.6982548701298703e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+6.5746753246753243e-04*w58*Jinv11+6.5746753246753243e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-2.3833198051948050e-03*Jinv22*w20+4.7666396103896100e-03*w43*Jinv11+4.7666396103896100e-03*Jinv21*w41+4.2735389610389614e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-2.0271915584415584e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-9.1314935064935068e-05*w28*Jinv11+2.1367694805194807e-03*Jinv22*w17+3.6982548701298703e-03*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22+4.7666396103896100e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-1.0683847402597404e-03*Jinv21*w38+7.3965097402597406e-03*Jinv02*w45+2.4655032467532467e-03*Jinv01*w24+7.8530844155844158e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-2.0271915584415584e-03*w19*Jinv11+5.7528409090909093e-04*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.3833198051948050e-03*Jinv12*w13-1.0683847402597404e-03*w32*Jinv21-2.0271915584415584e-03*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12+4.2735389610389614e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35-1.0683847402597404e-03*w31*Jinv11+3.6982548701298703e-03*Jinv22*w26+7.3965097402597406e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-2.6298701298701297e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12+3.6982548701298703e-03*w21*Jinv02-9.1314935064935068e-05*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33-2.3833198051948050e-03*Jinv02*w12-2.0271915584415584e-03*Jinv01*w18-1.0683847402597404e-03*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+7.3965097402597406e-03*w47*Jinv21+3.6982548701298703e-03*Jinv22*w23+5.7528409090909093e-04*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12+7.3965097402597406e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50+2.4655032467532467e-03*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+2.4655032467532467e-03*w21*Jinv01+5.7528409090909093e-04*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.0683847402597404e-03*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-1.3149350649350649e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5+7.3965097402597406e-03*w47*Jinv22+5.7528409090909093e-04*w49*Jinv11-2.0271915584415584e-03*Jinv21*w14+7.8530844155844158e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-2.0271915584415584e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+2.4655032467532467e-03*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.3833198051948050e-03*Jinv22*w14+4.7666396103896100e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-1.0683847402597404e-03*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-2.6298701298701297e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+2.4655032467532467e-03*w25*Jinv11+2.1367694805194807e-03*w16*Jinv12+5.7528409090909093e-04*Jinv21*w50-9.1314935064935068e-05*Jinv01*w27+5.7528409090909093e-04*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[45][46]=(0.0)*D;
  A[45][47]=(0.0)*D;
  A[45][48]=(-5.2049512987012989e-04*Jinv00*w12+4.7483766233766234e-04*w28*Jinv10-6.0267857142857139e-04*w40*Jinv12+1.3697240259740260e-03*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+6.6659902597402602e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10-6.0267857142857139e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.1505681818181819e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12+2.3833198051948050e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-1.2053571428571428e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-1.1505681818181819e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02+4.7483766233766234e-04*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.7483766233766234e-04*w28*Jinv11+2.3833198051948050e-03*Jinv20*w50+4.7483766233766234e-04*Jinv20*w29-1.1505681818181819e-03*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10-2.7394480519480521e-05*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00-9.5880681818181818e-04*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38+3.2873376623376621e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+6.6659902597402602e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00-1.2053571428571428e-03*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-3.8352272727272725e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-6.0267857142857139e-04*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.3697240259740260e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26-1.1505681818181819e-03*w46*Jinv11-5.2049512987012989e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54-2.7394480519480521e-05*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02+2.3833198051948050e-03*w49*Jinv10+4.7483766233766234e-04*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-3.8352272727272725e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-1.1505681818181819e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11-9.5880681818181818e-04*w31*Jinv12+3.2873376623376621e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53+1.9176136363636362e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+1.9176136363636362e-04*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51-9.5880681818181818e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+1.9176136363636362e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12-2.7394480519480521e-05*Jinv22*w5+3.2873376623376621e-04*w47*Jinv22+2.3833198051948050e-03*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+6.6659902597402602e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-1.2053571428571428e-03*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30+1.3697240259740260e-03*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-1.1505681818181819e-03*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+2.3833198051948050e-03*Jinv21*w50+4.7483766233766234e-04*Jinv01*w27+2.3833198051948050e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[45][49]=(0.0)*D;
  A[45][50]=(0.0)*D;
  A[45][51]=(5.2049512987012989e-04*Jinv00*w12-4.7483766233766234e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-2.0271915584415584e-03*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20-1.6436688311688311e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+3.8352272727272725e-04*w25*Jinv12-3.1960227272727273e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10-1.2327516233766234e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-7.1225649350649346e-04*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24-2.5202922077922079e-03*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+1.3697240259740259e-04*Jinv22*w20+1.1505681818181819e-03*w45*Jinv00-2.7394480519480520e-03*w42*Jinv02-4.7483766233766234e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-4.7483766233766234e-04*Jinv20*w29+1.1505681818181819e-03*w46*Jinv10-7.1225649350649346e-04*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+1.9176136363636362e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+4.6570616883116886e-04*Jinv02*w3-1.6436688311688311e-04*w21*Jinv00+8.4922889610389610e-04*w32*Jinv22-3.1960227272727273e-04*Jinv22*w2+5.2049512987012989e-04*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+1.4793019480519481e-03*Jinv02*w45+1.1870941558441559e-04*Jinv22*w59-2.3833198051948050e-03*w51*Jinv00-2.5202922077922079e-03*Jinv12*w34-2.7394480519480520e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-2.1367694805194807e-03*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13+5.2049512987012989e-04*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-2.0271915584415584e-03*w52*Jinv12+3.8352272727272725e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-1.2327516233766234e-03*Jinv02*w54+4.6570616883116886e-04*w4*Jinv12+1.9176136363636362e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-2.1367694805194807e-03*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-1.7349837662337662e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+1.3971185064935066e-03*Jinv20*w38+1.9176136363636362e-04*Jinv22*w23+8.4922889610389610e-04*w31*Jinv12+1.4793019480519481e-03*w46*Jinv12-2.3833198051948050e-03*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8-1.6436688311688311e-04*w22*Jinv10-1.0135957792207792e-03*Jinv22*w50+2.4655032467532466e-04*Jinv12*w7+8.4922889610389610e-04*Jinv02*w30-1.0135957792207792e-03*Jinv12*w49-2.3833198051948050e-03*w52*Jinv10-4.2918019480519480e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10-1.6436688311688311e-04*w24*Jinv00-4.8396915584415583e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+1.3697240259740259e-04*w19*Jinv12+4.6570616883116886e-04*Jinv22*w5+1.4793019480519481e-03*w47*Jinv22-4.2918019480519480e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+1.3697240259740259e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-2.5202922077922079e-03*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-1.6436688311688311e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.0135957792207792e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-4.8396915584415583e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14+4.3831168831168834e-04*Jinv02*w36-2.7394480519480520e-03*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20-2.0271915584415584e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56-4.8396915584415583e-04*Jinv12*w28-2.6298701298701297e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.1505681818181819e-03*w47*Jinv20-7.1225649350649346e-04*w16*Jinv12-3.1960227272727273e-04*Jinv12*w1)*D;
  A[45][52]=(0.0)*D;
  A[45][53]=(0.0)*D;
  A[45][54]=(-6.5746753246753243e-04*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53-1.4610389610389610e-04*w57*Jinv02-6.5746753246753243e-04*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+4.3831168831168834e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.5523538961038962e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55-8.2183441558441553e-04*w15*Jinv02-1.5523538961038962e-04*w10*Jinv12-2.4655032467532466e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+8.2183441558441553e-05*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-1.5523538961038962e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.7483766233766234e-04*w28*Jinv11-8.2183441558441553e-04*Jinv22*w17-2.4655032467532466e-04*w22*Jinv12+2.4655032467532466e-04*Jinv02*w3+4.3831168831168834e-04*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-1.5523538961038962e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38+3.7804383116883116e-03*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-1.4610389610389610e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11+5.2049512987012989e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+8.2183441558441553e-05*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21+5.2049512987012989e-04*Jinv21*w20+4.3831168831168834e-04*w37*Jinv12-6.5746753246753243e-04*Jinv22*w41-1.2327516233766234e-03*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-4.7483766233766234e-04*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33+8.2183441558441553e-05*Jinv02*w12+5.2049512987012989e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-1.5523538961038962e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01+1.1505681818181819e-03*w47*Jinv21-2.4655032467532466e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12+3.7804383116883116e-03*w46*Jinv12+2.4655032467532466e-04*Jinv22*w8-1.2327516233766234e-03*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-2.6298701298701297e-03*Jinv21*w56-1.0957792207792207e-03*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+8.2183441558441553e-05*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+3.7804383116883116e-03*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-1.4610389610389610e-04*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18+5.2049512987012989e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23-2.6298701298701297e-03*Jinv01*w54-2.6298701298701297e-03*w55*Jinv11-1.2327516233766234e-03*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9-1.0957792207792207e-03*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+4.3831168831168834e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-8.2183441558441553e-04*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-4.7483766233766234e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-1.5523538961038962e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[45][55]=(0.0)*D;
  A[45][56]=(0.0)*D;
  A[45][57]=(-3.2873376623376621e-04*w40*Jinv12-6.7573051948051951e-04*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02-3.2873376623376621e-04*Jinv02*w39+4.9310064935064932e-04*Jinv22*w38+3.6525974025974026e-05*w25*Jinv12-1.1668019480519480e-04*w0*Jinv02-1.0227272727272728e-03*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12+3.6525974025974026e-05*Jinv02*w24+2.9220779220779221e-04*Jinv22*w35+1.1870941558441559e-04*Jinv22*w20-3.2873376623376621e-04*w42*Jinv02-1.1668019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17+3.6525974025974026e-05*w22*Jinv12-2.7394480519480521e-05*Jinv02*w3+4.9310064935064932e-04*w32*Jinv22-1.1668019480519480e-04*Jinv22*w2-2.5568181818181820e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59+2.9220779220779221e-04*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44+1.1870941558441559e-04*Jinv12*w13+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv22*w41-6.7573051948051951e-04*w52*Jinv12+3.6525974025974026e-05*Jinv22*w26-1.0227272727272728e-03*Jinv02*w54-2.7394480519480521e-05*w4*Jinv12+3.6525974025974026e-05*w21*Jinv02+1.1870941558441559e-04*Jinv02*w12-1.1668019480519480e-04*Jinv02*w9+3.6525974025974026e-05*Jinv22*w23+4.9310064935064932e-04*w31*Jinv12-2.5568181818181820e-04*w46*Jinv12-2.7394480519480521e-05*Jinv22*w8-6.7573051948051951e-04*Jinv22*w50-2.7394480519480521e-05*Jinv12*w7+4.9310064935064932e-04*Jinv02*w30-6.7573051948051951e-04*Jinv12*w49+8.3198051948051951e-05*Jinv02*w27-2.7394480519480521e-05*w6*Jinv02+1.1870941558441559e-04*w19*Jinv12-2.7394480519480521e-05*Jinv22*w5-2.5568181818181820e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12+1.1870941558441559e-04*Jinv02*w18+2.9220779220779221e-04*Jinv02*w33-6.7573051948051951e-04*w48*Jinv02+8.3198051948051951e-05*Jinv22*w29+1.1870941558441559e-04*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-6.7573051948051951e-04*Jinv02*w51-1.0227272727272728e-03*Jinv22*w56+8.3198051948051951e-05*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12-1.1668019480519480e-04*Jinv12*w1)*D;
  A[45][58]=(0.0)*D;
  A[45][59]=(0.0)*D;
  A[46][0]=(0.0)*D;
  A[46][1]=(2.6481331168831169e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+2.6481331168831169e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23+2.1915584415584417e-04*Jinv22*w38+5.8441558441558442e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59-7.4878246753246752e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17-1.4975649350649350e-03*Jinv01*w45+5.8441558441558442e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-2.1915584415584417e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24-3.4699675324675324e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-7.4878246753246752e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-1.4975649350649350e-03*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+2.6481331168831169e-04*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-1.4975649350649350e-03*w46*Jinv10-2.1915584415584417e-04*Jinv22*w17-3.4699675324675324e-04*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-1.4975649350649350e-03*Jinv02*w45-7.4878246753246752e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-3.4699675324675324e-04*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-3.4699675324675324e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+2.6481331168831169e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10-3.4699675324675324e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00+2.1915584415584417e-04*w37*Jinv12+5.8441558441558442e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-3.4699675324675324e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-1.4975649350649350e-03*w46*Jinv11+2.6481331168831169e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21+5.8441558441558442e-04*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29-3.4699675324675324e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+2.6481331168831169e-04*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10-1.4975649350649350e-03*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12-1.4975649350649350e-03*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10+5.8441558441558442e-04*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-1.4975649350649350e-03*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+2.6481331168831169e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+2.6481331168831169e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-3.4699675324675324e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-3.4699675324675324e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+5.8441558441558442e-04*Jinv01*w54+5.8441558441558442e-04*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+2.6481331168831169e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11+2.1915584415584417e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+5.8441558441558442e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-7.4878246753246752e-04*w25*Jinv11+5.8441558441558442e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-1.4975649350649350e-03*w47*Jinv20-2.1915584415584417e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[46][2]=(0.0)*D;
  A[46][3]=(0.0)*D;
  A[46][4]=(-8.2183441558441553e-05*Jinv00*w12+7.1225649350649346e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44-3.5612824675324673e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+1.3697240259740259e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+7.1225649350649346e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+8.7662337662337668e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*w11*Jinv22-6.3920454545454550e-05*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+7.1225649350649346e-04*Jinv22*w17+8.2183441558441553e-04*Jinv00*w33-1.9176136363636362e-04*w22*Jinv12-3.5612824675324673e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-4.6570616883116886e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+8.7662337662337668e-04*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+8.7662337662337668e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+8.2183441558441553e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+7.1225649350649346e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+2.7394480519480521e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+8.7662337662337668e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-1.9176136363636362e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+8.7662337662337668e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-4.6570616883116886e-04*Jinv20*w38-1.9176136363636362e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50-1.9176136363636362e-04*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7-1.9176136363636362e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+2.7394480519480521e-05*w9*Jinv00-4.6570616883116886e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+2.7394480519480521e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+8.7662337662337668e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+8.2183441558441553e-04*w35*Jinv20-1.9176136363636362e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+7.1225649350649346e-04*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[46][5]=(0.0)*D;
  A[46][6]=(0.0)*D;
  A[46][7]=(2.4655032467532466e-04*Jinv00*w12+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+1.3697240259740259e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480521e-05*w3*Jinv00+4.6570616883116886e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+1.3697240259740259e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01-2.7394480519480521e-05*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+5.4788961038961042e-05*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*w11*Jinv22+1.0957792207792208e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+3.8352272727272725e-04*Jinv22*w17-8.2183441558441553e-04*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+4.6570616883116886e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480521e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-8.2183441558441553e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+7.1225649350649346e-04*w37*Jinv12+5.4788961038961042e-05*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.4655032467532466e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+7.1225649350649346e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8+5.4788961038961042e-05*Jinv02*w33-2.7394480519480521e-05*w0*Jinv00-8.2183441558441553e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-2.7394480519480521e-05*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-3.2873376623376621e-04*Jinv20*w17+4.6570616883116886e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480521e-05*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+3.8352272727272725e-04*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[46][8]=(0.0)*D;
  A[46][9]=(0.0)*D;
  A[46][10]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10-2.7394480519480521e-05*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+7.4878246753246752e-04*Jinv20*w23-5.8441558441558442e-04*Jinv00*w54+4.3831168831168834e-04*w43*Jinv10-1.4204545454545455e-05*w58*Jinv10-4.2918019480519480e-04*w48*Jinv00+1.0247564935064935e-04*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+1.4975649350649350e-03*w45*Jinv00+6.4935064935064935e-05*Jinv00*w27-4.2918019480519480e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+1.4975649350649350e-03*w46*Jinv10+3.4699675324675324e-04*Jinv00*w33-2.1915584415584417e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00-2.6481331168831169e-04*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00+1.1870941558441559e-04*w51*Jinv00-2.7394480519480521e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-2.6481331168831169e-04*w19*Jinv10+3.4699675324675324e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-5.8441558441558442e-04*Jinv20*w56+3.6525974025974026e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-4.2918019480519480e-04*Jinv20*w14-4.2918019480519480e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-5.4788961038961042e-05*w7*Jinv10-1.0957792207792208e-04*Jinv20*w38+1.1870941558441559e-04*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+1.1870941558441559e-04*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10+3.6525974025974026e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+3.4699675324675324e-04*w35*Jinv20+3.6525974025974026e-05*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00+2.1915584415584417e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-2.6481331168831169e-04*w20*Jinv20-2.7394480519480521e-05*w5*Jinv20-5.8441558441558442e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+1.4975649350649350e-03*w47*Jinv20)*D;
  A[46][11]=(0.0)*D;
  A[46][12]=(0.0)*D;
  A[46][13]=(-5.7528409090909093e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+1.6436688311688311e-03*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-5.7528409090909093e-04*w13*Jinv10+1.6436688311688311e-03*Jinv02*w39+2.5750811688311690e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-1.4793019480519481e-03*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.5750811688311690e-03*w42*Jinv01+3.1960227272727273e-04*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17+4.0543831168831168e-03*Jinv01*w45-1.4793019480519481e-03*Jinv12*w55+3.1047077922077924e-04*w58*Jinv10+8.2183441558441553e-04*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12-1.3697240259740259e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01+2.0271915584415584e-03*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+8.2183441558441553e-04*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+2.5750811688311690e-03*w43*Jinv11+4.7666396103896100e-03*w45*Jinv00+8.7662337662337668e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-7.3965097402597403e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+5.5702110389610384e-04*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+4.7666396103896100e-03*w46*Jinv10+8.2183441558441553e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+8.7662337662337668e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-4.6570616883116886e-04*Jinv21*w38+4.7666396103896100e-03*Jinv02*w45+2.0271915584415584e-03*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-1.1231737012987013e-03*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20-5.7528409090909093e-04*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-5.4788961038961036e-04*w32*Jinv21-1.1231737012987013e-03*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-1.4793019480519481e-03*Jinv20*w56+1.6436688311688311e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-5.4788961038961036e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+4.0543831168831168e-03*w46*Jinv11-5.7528409090909093e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21-1.4793019480519481e-03*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-1.3697240259740259e-04*w49*Jinv10+5.5702110389610384e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00-5.7528409090909093e-04*Jinv02*w12-1.1231737012987013e-03*Jinv01*w18-4.6570616883116886e-04*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10+4.0543831168831168e-03*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+4.7666396103896100e-03*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-4.6570616883116886e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-9.5880681818181818e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5+4.7666396103896100e-03*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11-7.3965097402597403e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18-7.3965097402597403e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8+3.1047077922077924e-04*w57*Jinv00+3.1047077922077924e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-9.5880681818181818e-04*Jinv01*w54-9.5880681818181818e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+8.2183441558441553e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+3.1960227272727273e-04*Jinv01*w9+8.2183441558441553e-04*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-5.7528409090909093e-04*Jinv22*w14+8.7662337662337668e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-5.4788961038961036e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-1.4793019480519481e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+2.0271915584415584e-03*w25*Jinv11-1.4793019480519481e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+4.7666396103896100e-03*w47*Jinv20+8.2183441558441553e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50+5.5702110389610384e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+3.1960227272727273e-04*w10*Jinv11)*D;
  A[46][14]=(0.0)*D;
  A[46][15]=(0.0)*D;
  A[46][16]=(-8.2183441558441553e-04*Jinv00*w12-9.8620129870129864e-04*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-8.2183441558441553e-04*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-3.2873376623376621e-03*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20+2.1367694805194807e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38-5.2962662337662338e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-9.8620129870129864e-04*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+2.1367694805194807e-03*Jinv22*w20-3.2873376623376621e-03*w43*Jinv11-2.3011363636363637e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-4.9310064935064932e-04*w11*Jinv22-3.1047077922077924e-04*Jinv11*w1+9.3141233766233771e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-9.8620129870129864e-04*Jinv22*w17-2.1367694805194807e-03*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+8.2183441558441553e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.7532467532467534e-03*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13+8.2183441558441553e-04*w19*Jinv10+1.7532467532467534e-03*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12-9.8620129870129864e-04*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-4.2735389610389614e-03*Jinv22*w26-3.1777597402597403e-03*w46*Jinv11-8.2183441558441553e-04*Jinv20*w14+2.1367694805194807e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+1.7532467532467534e-03*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-1.0409902597402598e-03*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-1.0409902597402598e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.7532467532467534e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11+9.3141233766233771e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+9.3141233766233771e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+1.3149350649350649e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-1.0409902597402598e-03*Jinv21*w23+1.7532467532467534e-03*Jinv01*w54+1.7532467532467534e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+3.8352272727272725e-04*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-7.1225649350649346e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+8.2183441558441553e-04*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20+7.6704545454545450e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-3.1777597402597403e-03*w25*Jinv11-9.8620129870129864e-04*w42*Jinv00-9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[46][17]=(0.0)*D;
  A[46][18]=(0.0)*D;
  A[46][19]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-8.2183441558441553e-05*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+2.4655032467532466e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.4793019480519481e-03*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59-1.0957792207792208e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17-7.1225649350649346e-04*Jinv01*w45-3.1047077922077924e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-8.2183441558441553e-04*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11-4.7666396103896100e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.0957792207792208e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-4.7666396103896100e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39+5.7528409090909093e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38-1.6436688311688311e-04*Jinv01*w24-5.4788961038961042e-05*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53+1.3697240259740259e-04*w51*Jinv00+2.4655032467532466e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.7528409090909093e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+1.4793019480519481e-03*Jinv20*w56-2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-5.4788961038961042e-05*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-7.1225649350649346e-04*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.6436688311688311e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-3.5612824675324673e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10-7.1225649350649346e-04*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+5.2049512987012989e-04*w52*Jinv11+1.3697240259740259e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01+1.3697240259740259e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-2.1002435064935065e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+5.2049512987012989e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-2.1002435064935065e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-3.1047077922077924e-04*w57*Jinv00-3.1047077922077924e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+5.2049512987012989e-04*Jinv01*w54+5.2049512987012989e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-8.2183441558441553e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-8.2183441558441553e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+5.7528409090909093e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+2.4655032467532466e-04*w5*Jinv20+8.2183441558441553e-05*Jinv01*w30-1.6436688311688311e-04*w25*Jinv11+1.4793019480519481e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00-4.7666396103896100e-03*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[46][20]=(0.0)*D;
  A[46][21]=(0.0)*D;
  A[46][22]=(2.3833198051948050e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-4.2735389610389614e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8-7.8530844155844158e-04*w57*Jinv02+2.3833198051948050e-03*w13*Jinv10-4.2735389610389614e-03*Jinv02*w39-2.6846590909090908e-03*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+7.1225649350649346e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3-3.6982548701298703e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-3.6982548701298703e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-3.5612824675324673e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-2.6846590909090908e-03*w42*Jinv01-3.7439123376623376e-04*w11*Jinv21-5.4788961038961036e-04*w16*Jinv11-5.4788961038961036e-04*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45+2.6298701298701297e-03*Jinv12*w55-7.8530844155844158e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00-3.6982548701298703e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01-2.4655032467532467e-03*Jinv21*w26-3.5612824675324673e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20-2.6846590909090908e-03*w43*Jinv11-7.3965097402597406e-03*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22-3.5612824675324673e-04*Jinv11*w1+1.1505681818181819e-03*w13*Jinv11-5.4788961038961036e-04*w15*Jinv01-1.1414366883116883e-03*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-7.3965097402597406e-03*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-3.6982548701298703e-03*w22*Jinv12+7.1225649350649346e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-3.6982548701298703e-03*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2+2.3833198051948050e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-7.3965097402597406e-03*Jinv02*w45-2.4655032467532467e-03*Jinv01*w24-7.8530844155844158e-04*Jinv22*w59+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+2.3833198051948050e-03*Jinv12*w13+2.3833198051948050e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-4.2735389610389614e-03*Jinv22*w41-3.6982548701298703e-03*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+5.4788961038961036e-04*w31*Jinv11-3.6982548701298703e-03*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11+2.3833198051948050e-03*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21+2.6298701298701297e-03*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-3.6982548701298703e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-1.1414366883116883e-03*Jinv21*w29-4.2735389610389614e-03*w39*Jinv00+2.3833198051948050e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-7.4878246753246752e-04*Jinv02*w9+3.5612824675324673e-04*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-3.6982548701298703e-03*Jinv22*w23+8.2183441558441553e-05*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12-7.3965097402597406e-03*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8-3.6982548701298703e-03*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10-7.4878246753246752e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.3971185064935066e-03*Jinv21*w56-3.6982548701298703e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-7.3965097402597406e-03*w47*Jinv22+1.1505681818181819e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-7.8530844155844158e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.1505681818181819e-03*Jinv01*w12-7.8530844155844158e-04*w57*Jinv00-7.8530844155844158e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+1.3971185064935066e-03*Jinv01*w54+1.3971185064935066e-03*w55*Jinv11-3.6982548701298703e-03*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02-3.7439123376623376e-04*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17+7.1225649350649346e-04*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+2.3833198051948050e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.3833198051948050e-03*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+2.6298701298701297e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-2.4655032467532467e-03*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00-7.3965097402597406e-03*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-1.1414366883116883e-03*Jinv01*w27-7.4878246753246752e-04*Jinv12*w1-3.7439123376623376e-04*w10*Jinv11)*D;
  A[46][23]=(0.0)*D;
  A[46][24]=(0.0)*D;
  A[46][25]=(-2.3833198051948050e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8-2.3833198051948050e-03*w13*Jinv10+3.1777597402597403e-03*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-3.5612824675324673e-04*Jinv01*w3+3.6982548701298703e-03*Jinv20*w23-2.6298701298701297e-03*Jinv00*w54+3.9265422077922079e-04*Jinv21*w59+3.7439123376623376e-04*Jinv21*w2+4.2735389610389614e-03*w43*Jinv10+3.1777597402597403e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45+7.8530844155844158e-04*w58*Jinv10-1.6436688311688311e-04*w48*Jinv00+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.7439123376623376e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10+3.1777597402597403e-03*w43*Jinv11+7.3965097402597406e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+3.7439123376623376e-04*Jinv11*w1-1.0135957792207792e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11-1.6436688311688311e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+7.3965097402597406e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+3.6982548701298703e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.3833198051948050e-03*Jinv00*w18-7.1225649350649346e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+2.4655032467532467e-03*Jinv01*w24+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00-3.5612824675324673e-04*w4*Jinv10-3.5612824675324673e-04*w4*Jinv11+4.2735389610389614e-03*w41*Jinv20-2.3833198051948050e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-2.6298701298701297e-03*Jinv20*w56+3.6982548701298703e-03*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+1.0957792207792207e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.4655032467532467e-03*w46*Jinv11-2.3833198051948050e-03*Jinv20*w14-3.5612824675324673e-04*w5*Jinv21-1.6436688311688311e-04*w49*Jinv10-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33+4.2735389610389614e-03*w39*Jinv00-1.2327516233766234e-03*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-8.2183441558441553e-05*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+2.4655032467532467e-03*w47*Jinv21-7.1225649350649346e-04*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11-1.6436688311688311e-04*Jinv20*w53+3.6982548701298703e-03*w22*Jinv10+1.2327516233766234e-03*w22*Jinv11+1.2327516233766234e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11+7.4878246753246752e-04*w9*Jinv00-7.1225649350649346e-04*w37*Jinv10-1.2327516233766234e-03*Jinv21*w56+3.6982548701298703e-03*w24*Jinv00-8.2183441558441553e-05*w49*Jinv11-1.0135957792207792e-03*Jinv21*w14+7.4878246753246752e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-1.0135957792207792e-03*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+7.8530844155844158e-04*w57*Jinv00+7.8530844155844158e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11+3.6982548701298703e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00+3.9265422077922079e-04*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-2.3833198051948050e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-3.5612824675324673e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+2.4655032467532467e-03*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00+7.3965097402597406e-03*w47*Jinv20-8.2183441558441553e-05*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+3.9265422077922079e-04*w10*Jinv11)*D;
  A[46][26]=(0.0)*D;
  A[46][27]=(0.0)*D;
  A[46][28]=(6.5746753246753243e-04*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3+2.0495129870129870e-04*Jinv21*w59+1.5726461038961039e-04*Jinv21*w2+6.5746753246753243e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17+2.3741883116883116e-03*Jinv01*w45+2.0495129870129870e-04*w58*Jinv11+2.0495129870129870e-04*w57*Jinv01+1.1870941558441558e-03*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+6.5746753246753243e-04*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+1.5726461038961039e-04*Jinv11*w1-6.7573051948051951e-04*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+6.5746753246753243e-04*Jinv01*w39-1.0957792207792208e-04*Jinv21*w38+1.1870941558441558e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-6.7573051948051951e-04*w19*Jinv11-2.6481331168831169e-04*Jinv21*w53-5.4788961038961042e-05*w4*Jinv11-1.0957792207792208e-04*w32*Jinv21-6.7573051948051951e-04*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+2.3741883116883116e-03*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-6.7573051948051951e-04*Jinv01*w18-1.0957792207792208e-04*w37*Jinv11-5.4788961038961042e-05*w6*Jinv01+2.3741883116883116e-03*w47*Jinv21-2.6481331168831169e-04*w52*Jinv11+1.1870941558441558e-03*w22*Jinv11+1.1870941558441558e-03*w21*Jinv01-2.6481331168831169e-04*Jinv01*w51-1.0957792207792208e-04*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11-1.0227272727272728e-03*Jinv21*w56-2.6481331168831169e-04*w49*Jinv11-6.7573051948051951e-04*Jinv21*w14-6.7573051948051951e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+1.1870941558441558e-03*Jinv21*w23-1.0227272727272728e-03*Jinv01*w54-1.0227272727272728e-03*w55*Jinv11+1.5726461038961039e-04*Jinv01*w9+6.5746753246753243e-04*w40*Jinv11-1.0957792207792208e-04*Jinv01*w30+1.1870941558441558e-03*w25*Jinv11-2.6481331168831169e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-2.6481331168831169e-04*w48*Jinv01+1.5726461038961039e-04*w10*Jinv11)*D;
  A[46][29]=(0.0)*D;
  A[46][30]=(0.0)*D;
  A[46][31]=(1.6436688311688311e-04*Jinv00*w12-1.8262987012987013e-05*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-1.1231737012987013e-03*Jinv22*w53-4.6570616883116886e-04*Jinv20*w8-3.5612824675324673e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-1.4793019480519481e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-9.5880681818181818e-04*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+1.2601461038961039e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02-1.8262987012987013e-05*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-1.8262987012987013e-05*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50-1.8262987012987013e-05*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12+1.0683847402597404e-03*w31*Jinv10-3.8352272727272725e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38+1.4245129870129869e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-3.5612824675324673e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-1.3971185064935066e-03*w51*Jinv00+1.2601461038961039e-03*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-5.4788961038961036e-04*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+1.0683847402597404e-03*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-4.6570616883116886e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-1.4793019480519481e-03*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+2.1915584415584417e-04*w11*Jinv20-1.1231737012987013e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+1.0683847402597404e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-9.5880681818181818e-04*Jinv02*w54-5.4788961038961036e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10-1.8262987012987013e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-5.4788961038961036e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9-4.6570616883116886e-04*w6*Jinv01-4.6570616883116886e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+1.4245129870129869e-03*w46*Jinv12-1.3971185064935066e-03*Jinv20*w53-1.9176136363636362e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-1.9176136363636362e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-1.3971185064935066e-03*w52*Jinv10-4.6570616883116886e-04*w7*Jinv11+2.1915584415584417e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10-1.4793019480519481e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-1.9176136363636362e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12+1.4245129870129869e-03*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+2.1915584415584417e-04*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-3.5612824675324673e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-4.6570616883116886e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.2601461038961039e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-1.4793019480519481e-03*Jinv01*w54-1.4793019480519481e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+1.0683847402597404e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+2.1367694805194807e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.0683847402597404e-03*Jinv01*w30-1.1231737012987013e-03*Jinv02*w51-9.5880681818181818e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-1.4793019480519481e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-1.8262987012987013e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[46][32]=(0.0)*D;
  A[46][33]=(0.0)*D;
  A[46][34]=(-7.1225649350649346e-04*Jinv00*w12+6.5746753246753243e-04*w40*Jinv12+2.5750811688311690e-03*Jinv22*w53+8.2183441558441553e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39+3.9448051948051946e-03*Jinv21*w44-8.2183441558441553e-04*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-7.6704545454545450e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38-2.1915584415584417e-04*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+4.2004870129870131e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+3.9448051948051946e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-3.2873376623376621e-03*Jinv01*w45+1.7532467532467534e-03*Jinv12*w55+3.6525974025974026e-05*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+1.6436688311688311e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-2.1915584415584414e-03*Jinv21*w26+4.2004870129870131e-04*w0*Jinv01+3.4699675324675324e-04*w1*Jinv10+3.8352272727272725e-04*Jinv22*w20+3.9448051948051946e-03*w43*Jinv11+1.9724025974025973e-03*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+3.6525974025974026e-05*w11*Jinv22+4.2004870129870131e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+9.8620129870129864e-04*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50-1.3149350649350649e-03*w31*Jinv10-8.2183441558441553e-04*Jinv02*w3+1.0957792207792207e-03*w21*Jinv00-1.2053571428571428e-03*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-2.1915584415584414e-03*Jinv02*w45-2.1915584415584414e-03*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+1.3149350649350649e-03*w51*Jinv00+1.6436688311688311e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-8.2183441558441553e-04*w4*Jinv10-7.6704545454545450e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-1.1505681818181819e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+8.2183441558441553e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20-3.4699675324675324e-04*w11*Jinv20+2.5750811688311690e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-3.2873376623376621e-03*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-7.6704545454545450e-04*w5*Jinv21+1.7532467532467534e-03*Jinv02*w54-8.2183441558441553e-04*w4*Jinv12-1.3149350649350649e-03*w49*Jinv10+9.8620129870129864e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+8.2183441558441553e-04*w7*Jinv10-3.2873376623376621e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+2.6298701298701297e-03*w52*Jinv11-1.2053571428571428e-03*w31*Jinv12-2.1915584415584414e-03*w46*Jinv12+1.3149350649350649e-03*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10+1.2601461038961039e-03*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.0957792207792207e-03*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-1.2053571428571428e-03*Jinv02*w30+1.2601461038961039e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.3149350649350649e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11-3.4699675324675324e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+3.8352272727272725e-04*w19*Jinv12-8.2183441558441553e-04*Jinv22*w5-2.1915584415584414e-03*w47*Jinv22+1.3149350649350649e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-3.4699675324675324e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+1.6436688311688311e-03*Jinv02*w33+3.4699675324675324e-04*w0*Jinv00-1.0957792207792207e-03*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10+3.4699675324675324e-04*Jinv20*w2+1.2601461038961039e-03*w48*Jinv02+7.3051948051948052e-05*Jinv01*w9-1.3149350649350649e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-8.2183441558441553e-04*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30+2.5750811688311690e-03*Jinv02*w51+1.7532467532467534e-03*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-2.1915584415584414e-03*w25*Jinv11+1.9724025974025973e-03*w42*Jinv00+1.3149350649350649e-03*Jinv21*w50+9.8620129870129864e-04*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[46][35]=(0.0)*D;
  A[46][36]=(0.0)*D;
  A[46][37]=(5.4788961038961036e-04*Jinv00*w12+1.8262987012987013e-05*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-1.3697240259740259e-04*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+4.6570616883116886e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-7.1225649350649346e-04*Jinv22*w38+1.4793019480519481e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10+5.2049512987012989e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+1.3971185064935066e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-2.1915584415584417e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02+1.8262987012987013e-05*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.3971185064935066e-03*Jinv20*w50+1.8262987012987013e-05*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-3.8352272727272725e-04*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+2.4655032467532466e-04*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22-1.0957792207792208e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-1.0683847402597404e-03*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45-1.3697240259740259e-04*Jinv22*w59+8.2183441558441553e-05*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+4.6570616883116886e-04*w4*Jinv10-1.6436688311688311e-04*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-7.1225649350649346e-04*w37*Jinv12+1.4793019480519481e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+5.2049512987012989e-04*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26+5.4788961038961036e-04*Jinv20*w14+5.2049512987012989e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.3971185064935066e-03*w49*Jinv10-1.6436688311688311e-04*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-1.0683847402597404e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23+8.2183441558441553e-05*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-3.5612824675324673e-04*Jinv12*w7+8.2183441558441553e-05*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+8.2183441558441553e-05*w52*Jinv10+1.2784090909090910e-04*w9*Jinv00-1.0683847402597404e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.2784090909090910e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.3697240259740259e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-2.1915584415584417e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+3.8352272727272725e-04*w25*Jinv10-2.1915584415584417e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-7.1225649350649346e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+4.6570616883116886e-04*w5*Jinv20+5.2049512987012989e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28+1.4793019480519481e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-3.8352272727272725e-04*w16*Jinv12-1.0957792207792208e-04*Jinv12*w1)*D;
  A[46][38]=(0.0)*D;
  A[46][39]=(0.0)*D;
  A[46][40]=(2.6298701298701297e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.7532467532467534e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-1.6436688311688311e-04*w57*Jinv02+2.6298701298701297e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-4.2735389610389614e-03*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+4.9310064935064934e-03*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2-2.9586038961038961e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-9.6793831168831166e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-9.5332792207792200e-03*Jinv01*w45+2.7942370129870131e-03*Jinv12*w55-6.5746753246753243e-04*w58*Jinv10-2.7394480519480518e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01-5.4788961038961036e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-1.2820616883116883e-02*w45*Jinv00-6.9034090909090911e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+1.0957792207792207e-03*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+1.8628246753246754e-03*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29-1.2820616883116883e-02*w46*Jinv10+1.9724025974025973e-03*Jinv00*w33+1.6436688311688311e-04*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00-6.0267857142857139e-04*w32*Jinv22-6.9034090909090911e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+2.6298701298701297e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38-8.5470779220779228e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-1.6436688311688311e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.5750811688311690e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+2.1367694805194807e-03*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-2.9586038961038961e-03*w41*Jinv20+4.9310064935064932e-04*Jinv12*w13+2.6298701298701297e-03*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10+2.5750811688311690e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+4.9310064935064934e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-5.4788961038961036e-04*w11*Jinv20+1.7532467532467534e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35-2.1367694805194807e-03*Jinv22*w26-9.5332792207792200e-03*w46*Jinv11+2.6298701298701297e-03*Jinv20*w14+2.7942370129870131e-03*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+2.1367694805194807e-03*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33-2.9586038961038961e-03*w39*Jinv00+4.9310064935064932e-04*Jinv02*w12+2.5750811688311690e-03*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9+8.2183441558441553e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-9.5332792207792200e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+8.2183441558441553e-04*w52*Jinv11-6.0267857142857139e-04*w31*Jinv12-8.5470779220779228e-03*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-4.2735389610389614e-03*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50-2.0819805194805196e-03*w22*Jinv11-2.0819805194805196e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-6.0267857142857139e-04*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01+2.1367694805194807e-03*w52*Jinv10+8.2183441558441553e-04*w7*Jinv11-5.4788961038961036e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+2.7942370129870131e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+1.3149350649350649e-03*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5-8.5470779220779228e-03*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+1.8628246753246754e-03*Jinv21*w14-5.4788961038961036e-04*w10*Jinv10-2.9586038961038961e-03*Jinv20*w44-1.6436688311688311e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18+1.8628246753246754e-03*Jinv01*w12+8.2183441558441553e-04*Jinv21*w8-6.5746753246753243e-04*w57*Jinv00-6.5746753246753243e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33-5.4788961038961036e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-2.0819805194805196e-03*Jinv21*w23+2.7942370129870131e-03*Jinv01*w54+2.7942370129870131e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-5.4788961038961036e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-9.6793831168831166e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00-2.9586038961038961e-03*w40*Jinv10+2.6298701298701297e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+4.9310064935064932e-04*Jinv22*w14-6.9034090909090911e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.7532467532467534e-03*Jinv02*w51+2.7942370129870131e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-3.1777597402597403e-03*w25*Jinv11+4.9310064935064934e-03*w55*Jinv10-2.9586038961038961e-03*w42*Jinv00-1.2820616883116883e-02*w47*Jinv20-4.9310064935064932e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-9.6793831168831166e-04*w10*Jinv11)*D;
  A[46][41]=(0.0)*D;
  A[46][42]=(0.0)*D;
  A[46][43]=(-2.6298701298701297e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-2.6298701298701297e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-7.6704545454545450e-04*Jinv22*w38-4.9310064935064934e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02-4.2004870129870131e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+3.2873376623376621e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+6.5746753246753243e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+2.1915584415584414e-03*Jinv21*w26-4.2004870129870131e-04*w0*Jinv01+5.4788961038961036e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+1.2820616883116883e-02*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22-4.2004870129870131e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-9.8620129870129864e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29+1.2820616883116883e-02*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3+4.2735389610389614e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2-2.6298701298701297e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+2.1915584415584414e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59-4.9310064935064934e-03*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11+2.9586038961038961e-03*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-2.6298701298701297e-03*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+1.1505681818181819e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-7.6704545454545450e-04*w37*Jinv12-4.9310064935064934e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+4.2735389610389614e-03*w26*Jinv20+5.4788961038961036e-04*w11*Jinv20-7.6704545454545450e-04*w52*Jinv12-4.9310064935064934e-03*Jinv21*w35+1.1505681818181819e-03*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+3.2873376623376621e-03*w46*Jinv11-2.6298701298701297e-03*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-9.8620129870129864e-04*Jinv21*w29-4.9310064935064934e-03*Jinv01*w33+2.9586038961038961e-03*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+3.2873376623376621e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8+4.2735389610389614e-03*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10-5.4788961038961042e-05*w7*Jinv11+5.4788961038961036e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+5.4788961038961036e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+6.5746753246753243e-04*w57*Jinv00+6.5746753246753243e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+5.4788961038961036e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+5.4788961038961036e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+2.9586038961038961e-03*w40*Jinv10-2.6298701298701297e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+1.1505681818181819e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+2.1915584415584414e-03*w25*Jinv11-4.9310064935064934e-03*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00+1.2820616883116883e-02*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12-1.3149350649350649e-03*Jinv21*w50-9.8620129870129864e-04*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[46][44]=(0.0)*D;
  A[46][45]=(0.0)*D;
  A[46][46]=(4.2735389610389614e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53+7.8530844155844158e-04*w57*Jinv02+4.2735389610389614e-03*Jinv02*w39+4.7666396103896100e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38+6.5746753246753243e-04*Jinv21*w59+3.6982548701298703e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.7666396103896100e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+7.3965097402597406e-03*Jinv01*w45-2.6298701298701297e-03*Jinv12*w55+2.1367694805194807e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+3.6982548701298703e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+6.5746753246753243e-04*w58*Jinv11+6.5746753246753243e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-2.3833198051948050e-03*Jinv22*w20+4.7666396103896100e-03*w43*Jinv11+4.7666396103896100e-03*Jinv21*w41+4.2735389610389614e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-2.0271915584415584e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-9.1314935064935068e-05*w28*Jinv11+2.1367694805194807e-03*Jinv22*w17+3.6982548701298703e-03*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22+4.7666396103896100e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-1.0683847402597404e-03*Jinv21*w38+7.3965097402597406e-03*Jinv02*w45+2.4655032467532467e-03*Jinv01*w24+7.8530844155844158e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-2.0271915584415584e-03*w19*Jinv11+5.7528409090909093e-04*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.3833198051948050e-03*Jinv12*w13-1.0683847402597404e-03*w32*Jinv21-2.0271915584415584e-03*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12+4.2735389610389614e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35-1.0683847402597404e-03*w31*Jinv11+3.6982548701298703e-03*Jinv22*w26+7.3965097402597406e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-2.6298701298701297e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12+3.6982548701298703e-03*w21*Jinv02-9.1314935064935068e-05*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33-2.3833198051948050e-03*Jinv02*w12-2.0271915584415584e-03*Jinv01*w18-1.0683847402597404e-03*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+7.3965097402597406e-03*w47*Jinv21+3.6982548701298703e-03*Jinv22*w23+5.7528409090909093e-04*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12+7.3965097402597406e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50+2.4655032467532467e-03*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+2.4655032467532467e-03*w21*Jinv01+5.7528409090909093e-04*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.0683847402597404e-03*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-1.3149350649350649e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5+7.3965097402597406e-03*w47*Jinv22+5.7528409090909093e-04*w49*Jinv11-2.0271915584415584e-03*Jinv21*w14+7.8530844155844158e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-2.0271915584415584e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+2.4655032467532467e-03*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.3833198051948050e-03*Jinv22*w14+4.7666396103896100e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-1.0683847402597404e-03*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-2.6298701298701297e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+2.4655032467532467e-03*w25*Jinv11+2.1367694805194807e-03*w16*Jinv12+5.7528409090909093e-04*Jinv21*w50-9.1314935064935068e-05*Jinv01*w27+5.7528409090909093e-04*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[46][47]=(0.0)*D;
  A[46][48]=(0.0)*D;
  A[46][49]=(-5.2049512987012989e-04*Jinv00*w12+4.7483766233766234e-04*w28*Jinv10-6.0267857142857139e-04*w40*Jinv12+1.3697240259740260e-03*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+6.6659902597402602e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10-6.0267857142857139e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.1505681818181819e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12+2.3833198051948050e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-1.2053571428571428e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-1.1505681818181819e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02+4.7483766233766234e-04*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.7483766233766234e-04*w28*Jinv11+2.3833198051948050e-03*Jinv20*w50+4.7483766233766234e-04*Jinv20*w29-1.1505681818181819e-03*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10-2.7394480519480521e-05*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00-9.5880681818181818e-04*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38+3.2873376623376621e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+6.6659902597402602e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00-1.2053571428571428e-03*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-3.8352272727272725e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-6.0267857142857139e-04*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.3697240259740260e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26-1.1505681818181819e-03*w46*Jinv11-5.2049512987012989e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54-2.7394480519480521e-05*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02+2.3833198051948050e-03*w49*Jinv10+4.7483766233766234e-04*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-3.8352272727272725e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-1.1505681818181819e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11-9.5880681818181818e-04*w31*Jinv12+3.2873376623376621e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53+1.9176136363636362e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+1.9176136363636362e-04*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51-9.5880681818181818e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+1.9176136363636362e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12-2.7394480519480521e-05*Jinv22*w5+3.2873376623376621e-04*w47*Jinv22+2.3833198051948050e-03*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+6.6659902597402602e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-1.2053571428571428e-03*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30+1.3697240259740260e-03*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-1.1505681818181819e-03*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+2.3833198051948050e-03*Jinv21*w50+4.7483766233766234e-04*Jinv01*w27+2.3833198051948050e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[46][50]=(0.0)*D;
  A[46][51]=(0.0)*D;
  A[46][52]=(5.2049512987012989e-04*Jinv00*w12-4.7483766233766234e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-2.0271915584415584e-03*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20-1.6436688311688311e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+3.8352272727272725e-04*w25*Jinv12-3.1960227272727273e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10-1.2327516233766234e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-7.1225649350649346e-04*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24-2.5202922077922079e-03*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+1.3697240259740259e-04*Jinv22*w20+1.1505681818181819e-03*w45*Jinv00-2.7394480519480520e-03*w42*Jinv02-4.7483766233766234e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-4.7483766233766234e-04*Jinv20*w29+1.1505681818181819e-03*w46*Jinv10-7.1225649350649346e-04*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+1.9176136363636362e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+4.6570616883116886e-04*Jinv02*w3-1.6436688311688311e-04*w21*Jinv00+8.4922889610389610e-04*w32*Jinv22-3.1960227272727273e-04*Jinv22*w2+5.2049512987012989e-04*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+1.4793019480519481e-03*Jinv02*w45+1.1870941558441559e-04*Jinv22*w59-2.3833198051948050e-03*w51*Jinv00-2.5202922077922079e-03*Jinv12*w34-2.7394480519480520e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-2.1367694805194807e-03*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13+5.2049512987012989e-04*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-2.0271915584415584e-03*w52*Jinv12+3.8352272727272725e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-1.2327516233766234e-03*Jinv02*w54+4.6570616883116886e-04*w4*Jinv12+1.9176136363636362e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-2.1367694805194807e-03*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-1.7349837662337662e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+1.3971185064935066e-03*Jinv20*w38+1.9176136363636362e-04*Jinv22*w23+8.4922889610389610e-04*w31*Jinv12+1.4793019480519481e-03*w46*Jinv12-2.3833198051948050e-03*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8-1.6436688311688311e-04*w22*Jinv10-1.0135957792207792e-03*Jinv22*w50+2.4655032467532466e-04*Jinv12*w7+8.4922889610389610e-04*Jinv02*w30-1.0135957792207792e-03*Jinv12*w49-2.3833198051948050e-03*w52*Jinv10-4.2918019480519480e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10-1.6436688311688311e-04*w24*Jinv00-4.8396915584415583e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+1.3697240259740259e-04*w19*Jinv12+4.6570616883116886e-04*Jinv22*w5+1.4793019480519481e-03*w47*Jinv22-4.2918019480519480e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+1.3697240259740259e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-2.5202922077922079e-03*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-1.6436688311688311e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.0135957792207792e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-4.8396915584415583e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14+4.3831168831168834e-04*Jinv02*w36-2.7394480519480520e-03*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20-2.0271915584415584e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56-4.8396915584415583e-04*Jinv12*w28-2.6298701298701297e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.1505681818181819e-03*w47*Jinv20-7.1225649350649346e-04*w16*Jinv12-3.1960227272727273e-04*Jinv12*w1)*D;
  A[46][53]=(0.0)*D;
  A[46][54]=(0.0)*D;
  A[46][55]=(-6.5746753246753243e-04*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53-1.4610389610389610e-04*w57*Jinv02-6.5746753246753243e-04*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+4.3831168831168834e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.5523538961038962e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55-8.2183441558441553e-04*w15*Jinv02-1.5523538961038962e-04*w10*Jinv12-2.4655032467532466e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+8.2183441558441553e-05*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-1.5523538961038962e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.7483766233766234e-04*w28*Jinv11-8.2183441558441553e-04*Jinv22*w17-2.4655032467532466e-04*w22*Jinv12+2.4655032467532466e-04*Jinv02*w3+4.3831168831168834e-04*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-1.5523538961038962e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38+3.7804383116883116e-03*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-1.4610389610389610e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11+5.2049512987012989e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+8.2183441558441553e-05*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21+5.2049512987012989e-04*Jinv21*w20+4.3831168831168834e-04*w37*Jinv12-6.5746753246753243e-04*Jinv22*w41-1.2327516233766234e-03*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-4.7483766233766234e-04*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33+8.2183441558441553e-05*Jinv02*w12+5.2049512987012989e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-1.5523538961038962e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01+1.1505681818181819e-03*w47*Jinv21-2.4655032467532466e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12+3.7804383116883116e-03*w46*Jinv12+2.4655032467532466e-04*Jinv22*w8-1.2327516233766234e-03*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-2.6298701298701297e-03*Jinv21*w56-1.0957792207792207e-03*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+8.2183441558441553e-05*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+3.7804383116883116e-03*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-1.4610389610389610e-04*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18+5.2049512987012989e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23-2.6298701298701297e-03*Jinv01*w54-2.6298701298701297e-03*w55*Jinv11-1.2327516233766234e-03*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9-1.0957792207792207e-03*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+4.3831168831168834e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-8.2183441558441553e-04*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-4.7483766233766234e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-1.5523538961038962e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[46][56]=(0.0)*D;
  A[46][57]=(0.0)*D;
  A[46][58]=(-3.2873376623376621e-04*w40*Jinv12-6.7573051948051951e-04*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02-3.2873376623376621e-04*Jinv02*w39+4.9310064935064932e-04*Jinv22*w38+3.6525974025974026e-05*w25*Jinv12-1.1668019480519480e-04*w0*Jinv02-1.0227272727272728e-03*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12+3.6525974025974026e-05*Jinv02*w24+2.9220779220779221e-04*Jinv22*w35+1.1870941558441559e-04*Jinv22*w20-3.2873376623376621e-04*w42*Jinv02-1.1668019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17+3.6525974025974026e-05*w22*Jinv12-2.7394480519480521e-05*Jinv02*w3+4.9310064935064932e-04*w32*Jinv22-1.1668019480519480e-04*Jinv22*w2-2.5568181818181820e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59+2.9220779220779221e-04*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44+1.1870941558441559e-04*Jinv12*w13+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv22*w41-6.7573051948051951e-04*w52*Jinv12+3.6525974025974026e-05*Jinv22*w26-1.0227272727272728e-03*Jinv02*w54-2.7394480519480521e-05*w4*Jinv12+3.6525974025974026e-05*w21*Jinv02+1.1870941558441559e-04*Jinv02*w12-1.1668019480519480e-04*Jinv02*w9+3.6525974025974026e-05*Jinv22*w23+4.9310064935064932e-04*w31*Jinv12-2.5568181818181820e-04*w46*Jinv12-2.7394480519480521e-05*Jinv22*w8-6.7573051948051951e-04*Jinv22*w50-2.7394480519480521e-05*Jinv12*w7+4.9310064935064932e-04*Jinv02*w30-6.7573051948051951e-04*Jinv12*w49+8.3198051948051951e-05*Jinv02*w27-2.7394480519480521e-05*w6*Jinv02+1.1870941558441559e-04*w19*Jinv12-2.7394480519480521e-05*Jinv22*w5-2.5568181818181820e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12+1.1870941558441559e-04*Jinv02*w18+2.9220779220779221e-04*Jinv02*w33-6.7573051948051951e-04*w48*Jinv02+8.3198051948051951e-05*Jinv22*w29+1.1870941558441559e-04*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-6.7573051948051951e-04*Jinv02*w51-1.0227272727272728e-03*Jinv22*w56+8.3198051948051951e-05*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12-1.1668019480519480e-04*Jinv12*w1)*D;
  A[46][59]=(0.0)*D;
  A[47][0]=(0.0)*D;
  A[47][1]=(0.0)*D;
  A[47][2]=(2.6481331168831169e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12+4.2918019480519480e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8+1.4204545454545455e-05*w57*Jinv02+2.6481331168831169e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.0957792207792208e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23+2.1915584415584417e-04*Jinv22*w38+5.8441558441558442e-04*Jinv00*w54+1.4204545454545455e-05*Jinv21*w59-7.4878246753246752e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-2.1915584415584417e-04*w16*Jinv11-2.1915584415584417e-04*Jinv21*w17-1.4975649350649350e-03*Jinv01*w45+5.8441558441558442e-04*Jinv12*w55+1.4204545454545455e-05*w58*Jinv10-2.1915584415584417e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-1.1870941558441559e-04*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24-3.4699675324675324e-04*Jinv22*w35+1.4204545454545455e-05*w58*Jinv11+1.4204545454545455e-05*w57*Jinv01-7.4878246753246752e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-2.1915584415584417e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-1.4975649350649350e-03*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+2.6481331168831169e-04*w13*Jinv11-2.1915584415584417e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11-1.1870941558441559e-04*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-1.4975649350649350e-03*w46*Jinv10-2.1915584415584417e-04*Jinv22*w17-3.4699675324675324e-04*Jinv00*w33-3.6525974025974026e-05*w22*Jinv12+1.0957792207792208e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+1.0957792207792208e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+2.1915584415584417e-04*Jinv21*w38-1.4975649350649350e-03*Jinv02*w45-7.4878246753246752e-04*Jinv01*w24+1.4204545454545455e-05*Jinv22*w59-3.4699675324675324e-04*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11+4.2918019480519480e-04*Jinv21*w53+4.2918019480519480e-04*w51*Jinv00-3.4699675324675324e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+2.6481331168831169e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10-3.4699675324675324e-04*w34*Jinv10+1.0957792207792208e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00+2.1915584415584417e-04*w37*Jinv12+5.8441558441558442e-04*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20+4.2918019480519480e-04*w52*Jinv12-3.4699675324675324e-04*Jinv21*w35+1.0957792207792208e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-1.4975649350649350e-03*w46*Jinv11+2.6481331168831169e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21+5.8441558441558442e-04*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-3.6525974025974026e-05*w21*Jinv02-1.1870941558441559e-04*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29-3.4699675324675324e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+2.6481331168831169e-04*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10-1.4975649350649350e-03*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38-3.6525974025974026e-05*Jinv22*w23+4.2918019480519480e-04*w52*Jinv11+1.0957792207792208e-04*w31*Jinv12-1.4975649350649350e-03*w46*Jinv12+4.2918019480519480e-04*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10-1.1870941558441559e-04*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01+4.2918019480519480e-04*Jinv01*w51+1.0957792207792208e-04*Jinv02*w30-1.1870941558441559e-04*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01+4.2918019480519480e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10+5.8441558441558442e-04*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-1.4975649350649350e-03*w47*Jinv22-1.1870941558441559e-04*w49*Jinv11+2.6481331168831169e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+1.4204545454545455e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+2.6481331168831169e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8+1.4204545454545455e-05*w57*Jinv00+1.4204545454545455e-05*Jinv20*w59-3.4699675324675324e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-3.4699675324675324e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23+5.8441558441558442e-04*Jinv01*w54+5.8441558441558442e-04*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-2.1915584415584417e-04*w15*Jinv00-1.1870941558441559e-04*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-2.1915584415584417e-04*Jinv20*w17+1.0957792207792208e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+2.6481331168831169e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11+2.1915584415584417e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.0957792207792208e-04*Jinv01*w30+4.2918019480519480e-04*Jinv02*w51+5.8441558441558442e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-7.4878246753246752e-04*w25*Jinv11+5.8441558441558442e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-1.4975649350649350e-03*w47*Jinv20-2.1915584415584417e-04*w16*Jinv12-1.1870941558441559e-04*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-1.1870941558441559e-04*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[47][3]=(0.0)*D;
  A[47][4]=(0.0)*D;
  A[47][5]=(-8.2183441558441553e-05*Jinv00*w12+7.1225649350649346e-04*w40*Jinv12-5.4788961038961036e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8+1.3697240259740259e-04*w57*Jinv02-8.2183441558441553e-05*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44-3.5612824675324673e-04*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+1.3697240259740259e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+7.1225649350649346e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+8.7662337662337668e-04*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+3.2873376623376621e-04*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*w11*Jinv22-6.3920454545454550e-05*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+7.1225649350649346e-04*Jinv22*w17+8.2183441558441553e-04*Jinv00*w33-1.9176136363636362e-04*w22*Jinv12-3.5612824675324673e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-4.6570616883116886e-04*w36*Jinv00-5.4788961038961036e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+8.7662337662337668e-04*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+8.7662337662337668e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+8.2183441558441553e-04*w34*Jinv10+3.5612824675324673e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+7.1225649350649346e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+2.7394480519480521e-05*w11*Jinv20-5.4788961038961036e-04*w52*Jinv12+8.7662337662337668e-04*Jinv21*w35+3.5612824675324673e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-05*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-1.9176136363636362e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+8.7662337662337668e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21-4.6570616883116886e-04*Jinv20*w38-1.9176136363636362e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-2.4655032467532466e-04*Jinv22*w50-1.9176136363636362e-04*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7-1.9176136363636362e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30-2.4655032467532466e-04*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+2.7394480519480521e-05*w9*Jinv00-4.6570616883116886e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+2.7394480519480521e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+8.7662337662337668e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+8.2183441558441553e-04*w35*Jinv20-1.9176136363636362e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+3.2873376623376621e-04*w15*Jinv00-2.4655032467532466e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+3.2873376623376621e-04*Jinv20*w17-3.5612824675324673e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*Jinv01*w30-5.4788961038961036e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+7.1225649350649346e-04*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[47][6]=(0.0)*D;
  A[47][7]=(0.0)*D;
  A[47][8]=(2.4655032467532466e-04*Jinv00*w12+5.4788961038961042e-05*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53+1.3697240259740259e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480521e-05*w3*Jinv00+4.6570616883116886e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+1.3697240259740259e-04*Jinv21*w59+1.6436688311688311e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17+7.1225649350649346e-04*Jinv01*w45-5.2049512987012989e-04*Jinv12*w55+3.8352272727272725e-04*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12+2.7394480519480518e-04*w48*Jinv00+1.6436688311688311e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+1.3697240259740259e-04*w58*Jinv11+1.3697240259740259e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01-2.7394480519480521e-05*w1*Jinv10-3.2873376623376621e-04*w16*Jinv10+1.6436688311688311e-04*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+5.4788961038961042e-05*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*w11*Jinv22+1.0957792207792208e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01-9.1314935064935065e-06*w28*Jinv11+2.7394480519480518e-04*Jinv20*w50+3.8352272727272725e-04*Jinv22*w17-8.2183441558441553e-04*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12+4.6570616883116886e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-8.2183441558441553e-05*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+8.2183441558441553e-05*Jinv00*w18+3.5612824675324673e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38+7.1225649350649346e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.3697240259740259e-04*Jinv22*w59+5.4788961038961042e-05*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-5.2049512987012989e-04*Jinv21*w53-2.7394480519480518e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480521e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+8.2183441558441553e-05*w19*Jinv10-8.2183441558441553e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+7.1225649350649346e-04*w37*Jinv12+5.4788961038961042e-05*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+5.4788961038961042e-05*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11+2.4655032467532466e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21-5.2049512987012989e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02+2.7394480519480518e-04*w49*Jinv10-9.1314935064935065e-06*Jinv21*w29+5.4788961038961042e-05*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+7.1225649350649346e-04*w47*Jinv21+3.5612824675324673e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-5.2049512987012989e-04*w52*Jinv11-8.2183441558441553e-05*w31*Jinv12+7.1225649350649346e-04*w46*Jinv12-2.7394480519480518e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10-2.7394480519480518e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01-5.2049512987012989e-04*Jinv01*w51-8.2183441558441553e-05*Jinv02*w30-2.7394480519480518e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01-2.7394480519480518e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+3.5612824675324673e-04*w37*Jinv10-5.2049512987012989e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+7.1225649350649346e-04*w47*Jinv22-2.7394480519480518e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44+1.3697240259740259e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8+5.4788961038961042e-05*Jinv02*w33-2.7394480519480521e-05*w0*Jinv00-8.2183441558441553e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-5.2049512987012989e-04*Jinv01*w54-5.2049512987012989e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-2.7394480519480521e-05*Jinv20*w2-3.2873376623376621e-04*w15*Jinv00-2.7394480519480518e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-3.2873376623376621e-04*Jinv20*w17+4.6570616883116886e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10+8.2183441558441553e-05*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480521e-05*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-5.2049512987012989e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+3.8352272727272725e-04*w16*Jinv12-2.7394480519480518e-04*Jinv21*w50-9.1314935064935065e-06*Jinv01*w27-2.7394480519480518e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[47][9]=(0.0)*D;
  A[47][10]=(0.0)*D;
  A[47][11]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10-2.7394480519480521e-05*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+7.4878246753246752e-04*Jinv20*w23-5.8441558441558442e-04*Jinv00*w54+4.3831168831168834e-04*w43*Jinv10-1.4204545454545455e-05*w58*Jinv10-4.2918019480519480e-04*w48*Jinv00+1.0247564935064935e-04*w1*Jinv10+2.1915584415584417e-04*w16*Jinv10+1.4975649350649350e-03*w45*Jinv00+6.4935064935064935e-05*Jinv00*w27-4.2918019480519480e-04*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+1.4975649350649350e-03*w46*Jinv10+3.4699675324675324e-04*Jinv00*w33-2.1915584415584417e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00-2.6481331168831169e-04*Jinv00*w18-1.0957792207792208e-04*w36*Jinv00+1.1870941558441559e-04*w51*Jinv00-2.7394480519480521e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-2.6481331168831169e-04*w19*Jinv10+3.4699675324675324e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00-5.8441558441558442e-04*Jinv20*w56+3.6525974025974026e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-4.2918019480519480e-04*Jinv20*w14-4.2918019480519480e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-5.4788961038961042e-05*w7*Jinv10-1.0957792207792208e-04*Jinv20*w38+1.1870941558441559e-04*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+1.1870941558441559e-04*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-1.0957792207792208e-04*w37*Jinv10+3.6525974025974026e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44-1.4204545454545455e-05*w57*Jinv00-1.4204545454545455e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+3.4699675324675324e-04*w35*Jinv20+3.6525974025974026e-05*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+2.1915584415584417e-04*w15*Jinv00+2.1915584415584417e-04*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-2.6481331168831169e-04*w20*Jinv20-2.7394480519480521e-05*w5*Jinv20-5.8441558441558442e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+1.4975649350649350e-03*w47*Jinv20)*D;
  A[47][12]=(0.0)*D;
  A[47][13]=(0.0)*D;
  A[47][14]=(-5.7528409090909093e-04*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+1.6436688311688311e-03*w40*Jinv12-5.2049512987012989e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+3.1047077922077924e-04*w57*Jinv02-5.7528409090909093e-04*w13*Jinv10+1.6436688311688311e-03*Jinv02*w39+2.5750811688311690e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38-1.4793019480519481e-03*Jinv00*w54+1.7349837662337662e-04*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+2.1002435064935065e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.5750811688311690e-03*w42*Jinv01+3.1960227272727273e-04*w11*Jinv21+4.3831168831168834e-04*w16*Jinv11+4.3831168831168834e-04*Jinv21*w17+4.0543831168831168e-03*Jinv01*w45-1.4793019480519481e-03*Jinv12*w55+3.1047077922077924e-04*w58*Jinv10+8.2183441558441553e-04*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12-1.3697240259740259e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35+1.7349837662337662e-04*w58*Jinv11+1.7349837662337662e-04*w57*Jinv01+2.0271915584415584e-03*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+8.2183441558441553e-04*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+2.5750811688311690e-03*w43*Jinv11+4.7666396103896100e-03*w45*Jinv00+8.7662337662337668e-04*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-7.3965097402597403e-04*w13*Jinv11+4.3831168831168834e-04*w15*Jinv01+5.5702110389610384e-04*w28*Jinv11-1.3697240259740259e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+4.7666396103896100e-03*w46*Jinv10+8.2183441558441553e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33+2.4655032467532466e-04*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+8.2183441558441553e-05*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+8.7662337662337668e-04*Jinv01*w39+2.1002435064935065e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-4.6570616883116886e-04*Jinv21*w38+4.7666396103896100e-03*Jinv02*w45+2.0271915584415584e-03*Jinv01*w24+3.1047077922077924e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-1.1231737012987013e-03*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-5.2049512987012989e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20-5.7528409090909093e-04*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-5.4788961038961036e-04*w32*Jinv21-1.1231737012987013e-03*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12-1.4793019480519481e-03*Jinv20*w56+1.6436688311688311e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20-5.2049512987012989e-04*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35-5.4788961038961036e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+4.0543831168831168e-03*w46*Jinv11-5.7528409090909093e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21-1.4793019480519481e-03*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12+2.4655032467532466e-04*w21*Jinv02-1.3697240259740259e-04*w49*Jinv10+5.5702110389610384e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00-5.7528409090909093e-04*Jinv02*w12-1.1231737012987013e-03*Jinv01*w18-4.6570616883116886e-04*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10+4.0543831168831168e-03*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38+2.4655032467532466e-04*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12+4.7666396103896100e-03*w46*Jinv12-5.2049512987012989e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-1.3697240259740259e-04*Jinv22*w50+8.2183441558441553e-05*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7+8.2183441558441553e-05*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-1.3697240259740259e-04*Jinv12*w49-4.6570616883116886e-04*w36*Jinv01-5.2049512987012989e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10-9.5880681818181818e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5+4.7666396103896100e-03*w47*Jinv22+3.8352272727272725e-04*w49*Jinv11-7.3965097402597403e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+3.1047077922077924e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18-7.3965097402597403e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8+3.1047077922077924e-04*w57*Jinv00+3.1047077922077924e-04*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-9.5880681818181818e-04*Jinv01*w54-9.5880681818181818e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+8.2183441558441553e-04*w15*Jinv00-1.3697240259740259e-04*w48*Jinv02+3.1960227272727273e-04*Jinv01*w9+8.2183441558441553e-04*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-5.7528409090909093e-04*Jinv22*w14+8.7662337662337668e-04*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-5.4788961038961036e-04*Jinv01*w30-5.2049512987012989e-04*Jinv02*w51-1.4793019480519481e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+2.0271915584415584e-03*w25*Jinv11-1.4793019480519481e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+4.7666396103896100e-03*w47*Jinv20+8.2183441558441553e-04*w16*Jinv12+3.8352272727272725e-04*Jinv21*w50+5.5702110389610384e-04*Jinv01*w27+3.8352272727272725e-04*w48*Jinv01+2.1002435064935065e-04*Jinv12*w1+3.1960227272727273e-04*w10*Jinv11)*D;
  A[47][15]=(0.0)*D;
  A[47][16]=(0.0)*D;
  A[47][17]=(-8.2183441558441553e-04*Jinv00*w12-9.8620129870129864e-04*w40*Jinv12+7.6704545454545450e-04*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-8.2183441558441553e-04*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-3.2873376623376621e-03*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20+2.1367694805194807e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38-5.2962662337662338e-04*Jinv21*w59-4.2735389610389614e-03*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-3.2873376623376621e-03*w42*Jinv01-5.2962662337662338e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-3.1777597402597403e-03*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-9.8620129870129864e-04*w15*Jinv02-4.9310064935064932e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-4.2735389610389614e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-5.2962662337662338e-04*w58*Jinv11-5.2962662337662338e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+2.1367694805194807e-03*Jinv22*w20-3.2873376623376621e-03*w43*Jinv11-2.3011363636363637e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02-4.9310064935064932e-04*w11*Jinv22-3.1047077922077924e-04*Jinv11*w1+9.3141233766233771e-04*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-9.8620129870129864e-04*Jinv22*w17-2.1367694805194807e-03*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3+2.1367694805194807e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22-2.3011363636363637e-03*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+8.2183441558441553e-04*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+7.1225649350649346e-04*Jinv21*w38-4.2735389610389614e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.7532467532467534e-03*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10+9.8620129870129864e-04*w41*Jinv20+1.3149350649350649e-03*Jinv12*w13+8.2183441558441553e-04*w19*Jinv10+1.7532467532467534e-03*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12-9.8620129870129864e-04*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20+7.6704545454545450e-04*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-4.2735389610389614e-03*Jinv22*w26-3.1777597402597403e-03*w46*Jinv11-8.2183441558441553e-04*Jinv20*w14+2.1367694805194807e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-2.1367694805194807e-03*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+1.3149350649350649e-03*Jinv02*w12+1.7532467532467534e-03*Jinv01*w18+7.1225649350649346e-04*w37*Jinv11-4.9310064935064932e-04*Jinv02*w9+3.2873376623376621e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10-3.1777597402597403e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-2.1367694805194807e-03*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12-4.2735389610389614e-03*w46*Jinv12+3.8352272727272725e-04*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10+3.8352272727272725e-04*Jinv22*w50-1.0409902597402598e-03*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-1.0409902597402598e-03*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30+3.8352272727272725e-04*Jinv12*w49+7.1225649350649346e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10+3.2873376623376621e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.7532467532467534e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02+2.1367694805194807e-03*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5-4.2735389610389614e-03*w47*Jinv22+3.2873376623376621e-04*w49*Jinv11+9.3141233766233771e-04*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12+2.1367694805194807e-03*Jinv02*w18+9.3141233766233771e-04*Jinv01*w12+3.2873376623376621e-04*Jinv21*w8+1.3149350649350649e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00-1.0409902597402598e-03*Jinv21*w23+1.7532467532467534e-03*Jinv01*w54+1.7532467532467534e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2+3.8352272727272725e-04*w48*Jinv02-5.2962662337662338e-04*Jinv01*w9-7.1225649350649346e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+8.2183441558441553e-04*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+1.3149350649350649e-03*Jinv22*w14-2.3011363636363637e-03*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20+7.6704545454545450e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-3.1777597402597403e-03*w25*Jinv11-9.8620129870129864e-04*w42*Jinv00-9.8620129870129864e-04*w16*Jinv12+3.2873376623376621e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+3.2873376623376621e-04*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1-5.2962662337662338e-04*w10*Jinv11)*D;
  A[47][18]=(0.0)*D;
  A[47][19]=(0.0)*D;
  A[47][20]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10-8.2183441558441553e-05*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+2.4655032467532466e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.4793019480519481e-03*Jinv00*w54-1.3697240259740259e-04*Jinv21*w59-1.0957792207792208e-04*Jinv21*w2-1.6436688311688311e-03*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17-7.1225649350649346e-04*Jinv01*w45-3.1047077922077924e-04*w58*Jinv10+5.2049512987012989e-04*w48*Jinv00-1.3697240259740259e-04*w58*Jinv11-1.3697240259740259e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-8.2183441558441553e-04*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11-4.7666396103896100e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-1.0957792207792208e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01+9.1314935064935065e-06*w28*Jinv11+5.2049512987012989e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-4.7666396103896100e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39+5.7528409090909093e-04*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-7.1225649350649346e-04*Jinv21*w38-1.6436688311688311e-04*Jinv01*w24-5.4788961038961042e-05*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+5.2049512987012989e-04*Jinv21*w53+1.3697240259740259e-04*w51*Jinv00+2.4655032467532466e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.7528409090909093e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00+1.4793019480519481e-03*Jinv20*w56-2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-5.4788961038961042e-05*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-7.1225649350649346e-04*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21+5.2049512987012989e-04*w49*Jinv10+9.1314935064935065e-06*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00-1.6436688311688311e-04*Jinv01*w18-7.1225649350649346e-04*w37*Jinv11-3.5612824675324673e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10-7.1225649350649346e-04*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+5.2049512987012989e-04*w52*Jinv11+1.3697240259740259e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01+5.2049512987012989e-04*Jinv01*w51-7.1225649350649346e-04*w36*Jinv01+1.3697240259740259e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11-2.1002435064935065e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10+5.2049512987012989e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+2.7394480519480518e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-2.1002435064935065e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8-3.1047077922077924e-04*w57*Jinv00-3.1047077922077924e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+5.2049512987012989e-04*Jinv01*w54+5.2049512987012989e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-8.2183441558441553e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-8.2183441558441553e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+5.7528409090909093e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+2.4655032467532466e-04*w5*Jinv20+8.2183441558441553e-05*Jinv01*w30-1.6436688311688311e-04*w25*Jinv11+1.4793019480519481e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00-4.7666396103896100e-03*w47*Jinv20+2.7394480519480518e-04*Jinv21*w50+9.1314935064935065e-06*Jinv01*w27+2.7394480519480518e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[47][21]=(0.0)*D;
  A[47][22]=(0.0)*D;
  A[47][23]=(2.3833198051948050e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10-4.2735389610389614e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8-7.8530844155844158e-04*w57*Jinv02+2.3833198051948050e-03*w13*Jinv10-4.2735389610389614e-03*Jinv02*w39-2.6846590909090908e-03*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+7.1225649350649346e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3-3.6982548701298703e-03*Jinv20*w23+7.1225649350649346e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54-3.9265422077922079e-04*Jinv21*w59-3.6982548701298703e-03*w25*Jinv12-7.4878246753246752e-04*w0*Jinv02-3.5612824675324673e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-2.6846590909090908e-03*w42*Jinv01-3.7439123376623376e-04*w11*Jinv21-5.4788961038961036e-04*w16*Jinv11-5.4788961038961036e-04*Jinv21*w17-4.9310064935064934e-03*Jinv01*w45+2.6298701298701297e-03*Jinv12*w55-7.8530844155844158e-04*w58*Jinv10-2.1367694805194807e-03*w15*Jinv02-7.4878246753246752e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00-3.6982548701298703e-03*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-3.9265422077922079e-04*w58*Jinv11-3.9265422077922079e-04*w57*Jinv01-2.4655032467532467e-03*Jinv21*w26-3.5612824675324673e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-2.1367694805194807e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20-2.6846590909090908e-03*w43*Jinv11-7.3965097402597406e-03*w45*Jinv00-1.0957792207792207e-03*Jinv21*w41-4.2735389610389614e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27-7.4878246753246752e-04*w11*Jinv22-3.5612824675324673e-04*Jinv11*w1+1.1505681818181819e-03*w13*Jinv11-5.4788961038961036e-04*w15*Jinv01-1.1414366883116883e-03*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29-7.3965097402597406e-03*w46*Jinv10-2.1367694805194807e-03*Jinv22*w17-1.0957792207792207e-03*Jinv00*w33-3.6982548701298703e-03*w22*Jinv12+7.1225649350649346e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-3.6982548701298703e-03*w21*Jinv00+7.1225649350649346e-04*w32*Jinv22-1.0957792207792207e-03*Jinv01*w39-7.4878246753246752e-04*Jinv22*w2+2.3833198051948050e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.5612824675324673e-04*Jinv21*w38-7.3965097402597406e-03*Jinv02*w45-2.4655032467532467e-03*Jinv01*w24-7.8530844155844158e-04*Jinv22*w59+1.3697240259740260e-03*w19*Jinv11+8.2183441558441553e-05*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-4.2735389610389614e-03*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+2.3833198051948050e-03*Jinv12*w13+2.3833198051948050e-03*w19*Jinv10-1.0957792207792207e-03*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+1.3697240259740260e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+7.1225649350649346e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-4.2735389610389614e-03*Jinv22*w41-3.6982548701298703e-03*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+5.4788961038961036e-04*w31*Jinv11-3.6982548701298703e-03*Jinv22*w26-4.9310064935064934e-03*w46*Jinv11+2.3833198051948050e-03*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21+2.6298701298701297e-03*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-3.6982548701298703e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10-1.1414366883116883e-03*Jinv21*w29-4.2735389610389614e-03*w39*Jinv00+2.3833198051948050e-03*Jinv02*w12+1.3697240259740260e-03*Jinv01*w18+3.5612824675324673e-04*w37*Jinv11-7.4878246753246752e-04*Jinv02*w9+3.5612824675324673e-04*w7*Jinv10-4.9310064935064934e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38-3.6982548701298703e-03*Jinv22*w23+8.2183441558441553e-05*w52*Jinv11+7.1225649350649346e-04*w31*Jinv12-7.3965097402597406e-03*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8-3.6982548701298703e-03*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01+8.2183441558441553e-05*Jinv01*w51+7.1225649350649346e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49+3.5612824675324673e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10-7.4878246753246752e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10+1.3971185064935066e-03*Jinv21*w56-3.6982548701298703e-03*w24*Jinv00-1.0957792207792207e-03*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-7.3965097402597406e-03*w47*Jinv22+1.1505681818181819e-03*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-7.8530844155844158e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18+1.1505681818181819e-03*Jinv01*w12-7.8530844155844158e-04*w57*Jinv00-7.8530844155844158e-04*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-1.0957792207792207e-03*w35*Jinv20-1.2327516233766234e-03*Jinv21*w23+1.3971185064935066e-03*Jinv01*w54+1.3971185064935066e-03*w55*Jinv11-3.6982548701298703e-03*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-2.1367694805194807e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02-3.7439123376623376e-04*Jinv01*w9-2.1367694805194807e-03*Jinv20*w17+7.1225649350649346e-04*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+2.3833198051948050e-03*w20*Jinv20-1.0957792207792207e-03*Jinv22*w29+2.3833198051948050e-03*Jinv22*w14-1.0957792207792207e-03*w40*Jinv11+7.1225649350649346e-04*Jinv02*w36-4.2735389610389614e-03*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+2.6298701298701297e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-2.4655032467532467e-03*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00-7.3965097402597406e-03*w47*Jinv20-2.1367694805194807e-03*w16*Jinv12-1.1414366883116883e-03*Jinv01*w27-7.4878246753246752e-04*Jinv12*w1-3.7439123376623376e-04*w10*Jinv11)*D;
  A[47][24]=(0.0)*D;
  A[47][25]=(0.0)*D;
  A[47][26]=(-2.3833198051948050e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8-2.3833198051948050e-03*w13*Jinv10+3.1777597402597403e-03*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-3.5612824675324673e-04*Jinv01*w3+3.6982548701298703e-03*Jinv20*w23-2.6298701298701297e-03*Jinv00*w54+3.9265422077922079e-04*Jinv21*w59+3.7439123376623376e-04*Jinv21*w2+4.2735389610389614e-03*w43*Jinv10+3.1777597402597403e-03*w42*Jinv01+3.9265422077922079e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+2.4655032467532467e-03*Jinv01*w45+7.8530844155844158e-04*w58*Jinv10-1.6436688311688311e-04*w48*Jinv00+3.9265422077922079e-04*w58*Jinv11+3.9265422077922079e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+3.7439123376623376e-04*w0*Jinv01+7.4878246753246752e-04*w1*Jinv10+2.1367694805194807e-03*w16*Jinv10+3.1777597402597403e-03*w43*Jinv11+7.3965097402597406e-03*w45*Jinv00+1.5888798701298701e-03*Jinv21*w41+1.0957792207792207e-03*Jinv00*w27+3.7439123376623376e-04*Jinv11*w1-1.0135957792207792e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-4.5657467532467534e-05*w28*Jinv11-1.6436688311688311e-04*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29+7.3965097402597406e-03*w46*Jinv10+1.0957792207792207e-03*Jinv00*w33-7.1225649350649346e-04*w31*Jinv10+3.6982548701298703e-03*w21*Jinv00+1.5888798701298701e-03*Jinv01*w39-2.3833198051948050e-03*Jinv00*w18-7.1225649350649346e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+2.4655032467532467e-03*Jinv01*w24+1.0957792207792207e-03*w34*Jinv11-1.2327516233766234e-03*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00-3.5612824675324673e-04*w4*Jinv10-3.5612824675324673e-04*w4*Jinv11+4.2735389610389614e-03*w41*Jinv20-2.3833198051948050e-03*w19*Jinv10+1.0957792207792207e-03*w34*Jinv10-3.5612824675324673e-04*w32*Jinv21-1.2327516233766234e-03*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-2.6298701298701297e-03*Jinv20*w56+3.6982548701298703e-03*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+1.0957792207792207e-03*Jinv21*w35-3.5612824675324673e-04*w31*Jinv11+2.4655032467532467e-03*w46*Jinv11-2.3833198051948050e-03*Jinv20*w14-3.5612824675324673e-04*w5*Jinv21-1.6436688311688311e-04*w49*Jinv10-4.5657467532467534e-05*Jinv21*w29+1.0957792207792207e-03*Jinv01*w33+4.2735389610389614e-03*w39*Jinv00-1.2327516233766234e-03*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-8.2183441558441553e-05*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+2.4655032467532467e-03*w47*Jinv21-7.1225649350649346e-04*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11-1.6436688311688311e-04*Jinv20*w53+3.6982548701298703e-03*w22*Jinv10+1.2327516233766234e-03*w22*Jinv11+1.2327516233766234e-03*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11+7.4878246753246752e-04*w9*Jinv00-7.1225649350649346e-04*w37*Jinv10-1.2327516233766234e-03*Jinv21*w56+3.6982548701298703e-03*w24*Jinv00-8.2183441558441553e-05*w49*Jinv11-1.0135957792207792e-03*Jinv21*w14+7.4878246753246752e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-1.0135957792207792e-03*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8+7.8530844155844158e-04*w57*Jinv00+7.8530844155844158e-04*Jinv20*w59+7.4878246753246752e-04*w0*Jinv00+1.0957792207792207e-03*w35*Jinv20+1.2327516233766234e-03*Jinv21*w23-1.2327516233766234e-03*Jinv01*w54-1.2327516233766234e-03*w55*Jinv11+3.6982548701298703e-03*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+2.1367694805194807e-03*w15*Jinv00+3.9265422077922079e-04*Jinv01*w9+2.1367694805194807e-03*Jinv20*w17-7.1225649350649346e-04*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-2.3833198051948050e-03*w20*Jinv20+1.5888798701298701e-03*w40*Jinv11-3.5612824675324673e-04*w5*Jinv20-3.5612824675324673e-04*Jinv01*w30+2.4655032467532467e-03*w25*Jinv11-2.6298701298701297e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00+7.3965097402597406e-03*w47*Jinv20-8.2183441558441553e-05*Jinv21*w50-4.5657467532467534e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01+3.9265422077922079e-04*w10*Jinv11)*D;
  A[47][27]=(0.0)*D;
  A[47][28]=(0.0)*D;
  A[47][29]=(6.5746753246753243e-04*Jinv21*w44-5.4788961038961042e-05*Jinv01*w3+2.0495129870129870e-04*Jinv21*w59+1.5726461038961039e-04*Jinv21*w2+6.5746753246753243e-04*w42*Jinv01+1.5726461038961039e-04*w11*Jinv21+3.2873376623376621e-04*w16*Jinv11+3.2873376623376621e-04*Jinv21*w17+2.3741883116883116e-03*Jinv01*w45+2.0495129870129870e-04*w58*Jinv11+2.0495129870129870e-04*w57*Jinv01+1.1870941558441558e-03*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+6.5746753246753243e-04*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+1.5726461038961039e-04*Jinv11*w1-6.7573051948051951e-04*w13*Jinv11+3.2873376623376621e-04*w15*Jinv01+6.4935064935064935e-04*w28*Jinv11+6.5746753246753243e-04*Jinv01*w39-1.0957792207792208e-04*Jinv21*w38+1.1870941558441558e-03*Jinv01*w24-3.1047077922077924e-04*w34*Jinv11-6.7573051948051951e-04*w19*Jinv11-2.6481331168831169e-04*Jinv21*w53-5.4788961038961042e-05*w4*Jinv11-1.0957792207792208e-04*w32*Jinv21-6.7573051948051951e-04*Jinv21*w20-3.1047077922077924e-04*Jinv21*w35-1.0957792207792208e-04*w31*Jinv11+2.3741883116883116e-03*w46*Jinv11-5.4788961038961042e-05*w5*Jinv21+6.4935064935064935e-04*Jinv21*w29-3.1047077922077924e-04*Jinv01*w33-6.7573051948051951e-04*Jinv01*w18-1.0957792207792208e-04*w37*Jinv11-5.4788961038961042e-05*w6*Jinv01+2.3741883116883116e-03*w47*Jinv21-2.6481331168831169e-04*w52*Jinv11+1.1870941558441558e-03*w22*Jinv11+1.1870941558441558e-03*w21*Jinv01-2.6481331168831169e-04*Jinv01*w51-1.0957792207792208e-04*w36*Jinv01-5.4788961038961042e-05*w7*Jinv11-1.0227272727272728e-03*Jinv21*w56-2.6481331168831169e-04*w49*Jinv11-6.7573051948051951e-04*Jinv21*w14-6.7573051948051951e-04*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+1.1870941558441558e-03*Jinv21*w23-1.0227272727272728e-03*Jinv01*w54-1.0227272727272728e-03*w55*Jinv11+1.5726461038961039e-04*Jinv01*w9+6.5746753246753243e-04*w40*Jinv11-1.0957792207792208e-04*Jinv01*w30+1.1870941558441558e-03*w25*Jinv11-2.6481331168831169e-04*Jinv21*w50+6.4935064935064935e-04*Jinv01*w27-2.6481331168831169e-04*w48*Jinv01+1.5726461038961039e-04*w10*Jinv11)*D;
  A[47][30]=(0.0)*D;
  A[47][31]=(0.0)*D;
  A[47][32]=(1.6436688311688311e-04*Jinv00*w12-1.8262987012987013e-05*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-1.1231737012987013e-03*Jinv22*w53-4.6570616883116886e-04*Jinv20*w8-3.5612824675324673e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-1.4793019480519481e-03*Jinv00*w54-2.1915584415584417e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-9.5880681818181818e-04*Jinv12*w55-2.1915584415584417e-04*w58*Jinv10+3.2873376623376621e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12-8.2183441558441553e-05*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+1.2601461038961039e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02-1.8262987012987013e-05*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01-1.8262987012987013e-05*w28*Jinv11-8.2183441558441553e-05*Jinv20*w50-1.8262987012987013e-05*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10+3.2873376623376621e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33-5.4788961038961036e-04*w22*Jinv12+1.0683847402597404e-03*w31*Jinv10-3.8352272727272725e-04*w21*Jinv00+3.5612824675324673e-04*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00-2.4655032467532466e-04*Jinv21*w38+1.4245129870129869e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-3.5612824675324673e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11-1.3971185064935066e-03*Jinv21*w53-1.3971185064935066e-03*w51*Jinv00+1.2601461038961039e-03*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-5.4788961038961036e-04*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10+1.0683847402597404e-03*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-4.6570616883116886e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-1.4793019480519481e-03*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+2.1915584415584417e-04*w11*Jinv20-1.1231737012987013e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35+1.0683847402597404e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-9.5880681818181818e-04*Jinv02*w54-5.4788961038961036e-04*w21*Jinv02-8.2183441558441553e-05*w49*Jinv10-1.8262987012987013e-05*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-5.4788961038961036e-04*Jinv01*w18-2.4655032467532466e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9-4.6570616883116886e-04*w6*Jinv01-4.6570616883116886e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-5.4788961038961036e-04*Jinv22*w23-1.3971185064935066e-03*w52*Jinv11+3.5612824675324673e-04*w31*Jinv12+1.4245129870129869e-03*w46*Jinv12-1.3971185064935066e-03*Jinv20*w53-1.9176136363636362e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+4.3831168831168834e-04*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-1.9176136363636362e-04*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01-1.3971185064935066e-03*Jinv01*w51+3.5612824675324673e-04*Jinv02*w30+4.3831168831168834e-04*Jinv12*w49-2.4655032467532466e-04*w36*Jinv01-1.3971185064935066e-03*w52*Jinv10-4.6570616883116886e-04*w7*Jinv11+2.1915584415584417e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10-1.4793019480519481e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-1.9176136363636362e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12+1.4245129870129869e-03*w47*Jinv22-8.2183441558441553e-05*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+2.1915584415584417e-04*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-3.5612824675324673e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-4.6570616883116886e-04*Jinv21*w8-2.1915584415584417e-04*w57*Jinv00-2.1915584415584417e-04*Jinv20*w59+1.2601461038961039e-03*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23-1.4793019480519481e-03*Jinv01*w54-1.4793019480519481e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+4.3831168831168834e-04*w48*Jinv02+2.1915584415584417e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17+1.0683847402597404e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29+2.1367694805194807e-03*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.0683847402597404e-03*Jinv01*w30-1.1231737012987013e-03*Jinv02*w51-9.5880681818181818e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-1.4793019480519481e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20+3.2873376623376621e-04*w16*Jinv12-8.2183441558441553e-05*Jinv21*w50-1.8262987012987013e-05*Jinv01*w27-8.2183441558441553e-05*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+2.1915584415584417e-04*w10*Jinv11)*D;
  A[47][33]=(0.0)*D;
  A[47][34]=(0.0)*D;
  A[47][35]=(-7.1225649350649346e-04*Jinv00*w12+6.5746753246753243e-04*w40*Jinv12+2.5750811688311690e-03*Jinv22*w53+8.2183441558441553e-04*Jinv20*w8-2.5568181818181820e-04*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10+6.5746753246753243e-04*Jinv02*w39+3.9448051948051946e-03*Jinv21*w44-8.2183441558441553e-04*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-7.6704545454545450e-04*Jinv01*w3+1.0957792207792207e-03*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38-2.1915584415584417e-04*Jinv21*w59-1.0957792207792207e-03*w25*Jinv12+3.8352272727272725e-04*w0*Jinv02+4.2004870129870131e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+3.9448051948051946e-03*w42*Jinv01+7.3051948051948052e-05*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-3.2873376623376621e-03*Jinv01*w45+1.7532467532467534e-03*Jinv12*w55+3.6525974025974026e-05*w10*Jinv12-1.3149350649350649e-03*w48*Jinv00-1.0957792207792207e-03*Jinv02*w24+1.6436688311688311e-03*Jinv22*w35-2.1915584415584417e-04*w58*Jinv11-2.1915584415584417e-04*w57*Jinv01-2.1915584415584414e-03*Jinv21*w26+4.2004870129870131e-04*w0*Jinv01+3.4699675324675324e-04*w1*Jinv10+3.8352272727272725e-04*Jinv22*w20+3.9448051948051946e-03*w43*Jinv11+1.9724025974025973e-03*Jinv21*w41+2.6298701298701297e-03*w42*Jinv02+3.6525974025974026e-05*w11*Jinv22+4.2004870129870131e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+9.8620129870129864e-04*w28*Jinv11-1.3149350649350649e-03*Jinv20*w50-1.3149350649350649e-03*w31*Jinv10-8.2183441558441553e-04*Jinv02*w3+1.0957792207792207e-03*w21*Jinv00-1.2053571428571428e-03*w32*Jinv22+1.9724025974025973e-03*Jinv01*w39+3.8352272727272725e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-2.1915584415584414e-03*Jinv02*w45-2.1915584415584414e-03*Jinv01*w24-2.5568181818181820e-04*Jinv22*w59+4.9310064935064934e-03*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11+2.6298701298701297e-03*Jinv21*w53+1.3149350649350649e-03*w51*Jinv00+1.6436688311688311e-03*Jinv12*w34+2.6298701298701297e-03*Jinv22*w44-8.2183441558441553e-04*w4*Jinv10-7.6704545454545450e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-3.2873376623376621e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-1.1505681818181819e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+8.2183441558441553e-04*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41-1.0957792207792207e-03*w26*Jinv20-3.4699675324675324e-04*w11*Jinv20+2.5750811688311690e-03*w52*Jinv12+4.9310064935064934e-03*Jinv21*w35-1.1505681818181819e-03*w31*Jinv11-1.0957792207792207e-03*Jinv22*w26-3.2873376623376621e-03*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14-7.6704545454545450e-04*w5*Jinv21+1.7532467532467534e-03*Jinv02*w54-8.2183441558441553e-04*w4*Jinv12-1.3149350649350649e-03*w49*Jinv10+9.8620129870129864e-04*Jinv21*w29+4.9310064935064934e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-3.2873376623376621e-04*Jinv02*w12+7.6704545454545450e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9+5.4788961038961042e-05*w6*Jinv01+8.2183441558441553e-04*w7*Jinv10-3.2873376623376621e-03*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+2.6298701298701297e-03*w52*Jinv11-1.2053571428571428e-03*w31*Jinv12-2.1915584415584414e-03*w46*Jinv12+1.3149350649350649e-03*Jinv20*w53+1.0957792207792207e-03*w22*Jinv10+1.2601461038961039e-03*Jinv22*w50-1.0957792207792207e-03*w22*Jinv11-1.0957792207792207e-03*w21*Jinv01+2.6298701298701297e-03*Jinv01*w51-1.2053571428571428e-03*Jinv02*w30+1.2601461038961039e-03*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+1.3149350649350649e-03*w52*Jinv10+5.4788961038961042e-05*w7*Jinv11-3.4699675324675324e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56-1.0957792207792207e-03*w24*Jinv00+4.9310064935064932e-04*Jinv02*w27+3.8352272727272725e-04*w19*Jinv12-8.2183441558441553e-04*Jinv22*w5-2.1915584415584414e-03*w47*Jinv22+1.3149350649350649e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-3.4699675324675324e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-2.5568181818181820e-04*w58*Jinv12+3.8352272727272725e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+5.4788961038961042e-05*Jinv21*w8+1.6436688311688311e-03*Jinv02*w33+3.4699675324675324e-04*w0*Jinv00-1.0957792207792207e-03*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11-1.0957792207792207e-03*w25*Jinv10+3.4699675324675324e-04*Jinv20*w2+1.2601461038961039e-03*w48*Jinv02+7.3051948051948052e-05*Jinv01*w9-1.3149350649350649e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20+4.9310064935064932e-04*Jinv22*w29-3.2873376623376621e-04*Jinv22*w14+1.9724025974025973e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36+2.6298701298701297e-03*Jinv12*w43-8.2183441558441553e-04*w5*Jinv20-1.1505681818181819e-03*Jinv01*w30+2.5750811688311690e-03*Jinv02*w51+1.7532467532467534e-03*Jinv22*w56+4.9310064935064932e-04*Jinv12*w28-2.1915584415584414e-03*w25*Jinv11+1.9724025974025973e-03*w42*Jinv00+1.3149350649350649e-03*Jinv21*w50+9.8620129870129864e-04*Jinv01*w27+1.3149350649350649e-03*w48*Jinv01+3.8352272727272725e-04*Jinv12*w1+7.3051948051948052e-05*w10*Jinv11)*D;
  A[47][36]=(0.0)*D;
  A[47][37]=(0.0)*D;
  A[47][38]=(5.4788961038961036e-04*Jinv00*w12+1.8262987012987013e-05*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12+5.2049512987012989e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-1.3697240259740259e-04*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+4.6570616883116886e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-7.1225649350649346e-04*Jinv22*w38+1.4793019480519481e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10+5.2049512987012989e-04*Jinv12*w55+2.1915584415584417e-04*w58*Jinv10-3.8352272727272725e-04*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12+1.3971185064935066e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-2.1915584415584417e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-2.1367694805194807e-03*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02+1.8262987012987013e-05*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22+1.3971185064935066e-03*Jinv20*w50+1.8262987012987013e-05*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-3.8352272727272725e-04*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+2.4655032467532466e-04*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22-1.0957792207792208e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-1.0683847402597404e-03*w36*Jinv00-7.1225649350649346e-04*Jinv02*w45-1.3697240259740259e-04*Jinv22*w59+8.2183441558441553e-05*w51*Jinv00-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+4.6570616883116886e-04*w4*Jinv10-1.6436688311688311e-04*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-7.1225649350649346e-04*w37*Jinv12+1.4793019480519481e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20+5.2049512987012989e-04*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26+5.4788961038961036e-04*Jinv20*w14+5.2049512987012989e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+1.3971185064935066e-03*w49*Jinv10-1.6436688311688311e-04*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-1.0683847402597404e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23+8.2183441558441553e-05*w31*Jinv12-7.1225649350649346e-04*w46*Jinv12+8.2183441558441553e-05*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+2.7394480519480518e-04*Jinv22*w50-3.5612824675324673e-04*Jinv12*w7+8.2183441558441553e-05*Jinv02*w30+2.7394480519480518e-04*Jinv12*w49+8.2183441558441553e-05*w52*Jinv10+1.2784090909090910e-04*w9*Jinv00-1.0683847402597404e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00+9.1314935064935065e-06*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-7.1225649350649346e-04*w47*Jinv22+1.2784090909090910e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.3697240259740259e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+2.1915584415584417e-04*w57*Jinv00+2.1915584415584417e-04*Jinv20*w59-5.4788961038961042e-05*Jinv02*w33-2.1915584415584417e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20+3.8352272727272725e-04*w25*Jinv10-2.1915584415584417e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00+2.7394480519480518e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-7.1225649350649346e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+4.6570616883116886e-04*w5*Jinv20+5.2049512987012989e-04*Jinv02*w51+5.2049512987012989e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28+1.4793019480519481e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-3.8352272727272725e-04*w16*Jinv12-1.0957792207792208e-04*Jinv12*w1)*D;
  A[47][39]=(0.0)*D;
  A[47][40]=(0.0)*D;
  A[47][41]=(2.6298701298701297e-03*Jinv00*w12+1.0957792207792207e-03*w28*Jinv10-9.8620129870129864e-04*w40*Jinv12+1.7532467532467534e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-1.6436688311688311e-04*w57*Jinv02+2.6298701298701297e-03*w13*Jinv10-9.8620129870129864e-04*Jinv02*w39-4.9310064935064934e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-4.2735389610389614e-03*Jinv20*w23+1.0957792207792208e-04*Jinv22*w38+4.9310064935064934e-03*Jinv00*w54-4.3831168831168834e-04*Jinv21*w59-2.1367694805194807e-03*w25*Jinv12-5.4788961038961042e-05*w0*Jinv02-6.2094155844155847e-04*Jinv21*w2-2.9586038961038961e-03*w43*Jinv10-4.9310064935064934e-03*w42*Jinv01-9.6793831168831166e-04*w11*Jinv21-2.3011363636363637e-03*w16*Jinv11-2.3011363636363637e-03*Jinv21*w17-9.5332792207792200e-03*Jinv01*w45+2.7942370129870131e-03*Jinv12*w55-6.5746753246753243e-04*w58*Jinv10-2.7394480519480518e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-2.1367694805194807e-03*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-4.3831168831168834e-04*w58*Jinv11-4.3831168831168834e-04*w57*Jinv01-3.1777597402597403e-03*Jinv21*w26-6.2094155844155847e-04*w0*Jinv01-5.4788961038961036e-04*w1*Jinv10-9.8620129870129864e-04*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-4.9310064935064934e-03*w43*Jinv11-1.2820616883116883e-02*w45*Jinv00-6.9034090909090911e-03*Jinv21*w41-1.9724025974025973e-03*w42*Jinv02+1.0957792207792207e-03*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22-6.2094155844155847e-04*Jinv11*w1+1.8628246753246754e-03*w13*Jinv11-2.3011363636363637e-03*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+1.0957792207792207e-03*Jinv20*w29-1.2820616883116883e-02*w46*Jinv10+1.9724025974025973e-03*Jinv00*w33+1.6436688311688311e-04*w31*Jinv10-3.2873376623376621e-04*Jinv02*w3-4.2735389610389614e-03*w21*Jinv00-6.0267857142857139e-04*w32*Jinv22-6.9034090909090911e-03*Jinv01*w39-5.4788961038961042e-05*Jinv22*w2+2.6298701298701297e-03*Jinv00*w18+1.6436688311688311e-04*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38-8.5470779220779228e-03*Jinv02*w45-3.1777597402597403e-03*Jinv01*w24-1.6436688311688311e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.5750811688311690e-03*w19*Jinv11+8.2183441558441553e-04*Jinv21*w53+2.1367694805194807e-03*w51*Jinv00+6.5746753246753243e-04*Jinv12*w34-1.9724025974025973e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-2.9586038961038961e-03*w41*Jinv20+4.9310064935064932e-04*Jinv12*w13+2.6298701298701297e-03*w19*Jinv10+1.9724025974025973e-03*w34*Jinv10+2.5750811688311690e-03*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+1.0957792207792208e-04*w37*Jinv12+4.9310064935064934e-03*Jinv20*w56-9.8620129870129864e-04*Jinv22*w41-4.2735389610389614e-03*w26*Jinv20-5.4788961038961036e-04*w11*Jinv20+1.7532467532467534e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35-2.1367694805194807e-03*Jinv22*w26-9.5332792207792200e-03*w46*Jinv11+2.6298701298701297e-03*Jinv20*w14+2.7942370129870131e-03*Jinv02*w54-3.2873376623376621e-04*w4*Jinv12+2.1367694805194807e-03*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33-2.9586038961038961e-03*w39*Jinv00+4.9310064935064932e-04*Jinv02*w12+2.5750811688311690e-03*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9+8.2183441558441553e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-9.5332792207792200e-03*w47*Jinv21+1.6436688311688311e-04*Jinv20*w38+8.2183441558441553e-04*w52*Jinv11-6.0267857142857139e-04*w31*Jinv12-8.5470779220779228e-03*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-4.2735389610389614e-03*w22*Jinv10+1.3697240259740260e-03*Jinv22*w50-2.0819805194805196e-03*w22*Jinv11-2.0819805194805196e-03*w21*Jinv01+8.2183441558441553e-04*Jinv01*w51-6.0267857142857139e-04*Jinv02*w30+1.3697240259740260e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01+2.1367694805194807e-03*w52*Jinv10+8.2183441558441553e-04*w7*Jinv11-5.4788961038961036e-04*w9*Jinv00+1.6436688311688311e-04*w37*Jinv10+2.7942370129870131e-03*Jinv21*w56-4.2735389610389614e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27+1.3149350649350649e-03*w19*Jinv12-3.2873376623376621e-04*Jinv22*w5-8.5470779220779228e-03*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+1.8628246753246754e-03*Jinv21*w14-5.4788961038961036e-04*w10*Jinv10-2.9586038961038961e-03*Jinv20*w44-1.6436688311688311e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18+1.8628246753246754e-03*Jinv01*w12+8.2183441558441553e-04*Jinv21*w8-6.5746753246753243e-04*w57*Jinv00-6.5746753246753243e-04*Jinv20*w59+6.5746753246753243e-04*Jinv02*w33-5.4788961038961036e-04*w0*Jinv00+1.9724025974025973e-03*w35*Jinv20-2.0819805194805196e-03*Jinv21*w23+2.7942370129870131e-03*Jinv01*w54+2.7942370129870131e-03*w55*Jinv11-4.2735389610389614e-03*w25*Jinv10-5.4788961038961036e-04*Jinv20*w2-9.8620129870129864e-04*w15*Jinv00+1.3697240259740260e-03*w48*Jinv02-9.6793831168831166e-04*Jinv01*w9-9.8620129870129864e-04*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00-2.9586038961038961e-03*w40*Jinv10+2.6298701298701297e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29+4.9310064935064932e-04*Jinv22*w14-6.9034090909090911e-03*w40*Jinv11+1.0957792207792208e-04*Jinv02*w36-1.9724025974025973e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+1.7532467532467534e-03*Jinv02*w51+2.7942370129870131e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28-3.1777597402597403e-03*w25*Jinv11+4.9310064935064934e-03*w55*Jinv10-2.9586038961038961e-03*w42*Jinv00-1.2820616883116883e-02*w47*Jinv20-4.9310064935064932e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-5.4788961038961042e-05*Jinv12*w1-9.6793831168831166e-04*w10*Jinv11)*D;
  A[47][42]=(0.0)*D;
  A[47][43]=(0.0)*D;
  A[47][44]=(-2.6298701298701297e-03*Jinv00*w12-1.0957792207792207e-03*w28*Jinv10+9.8620129870129864e-04*w40*Jinv12-7.6704545454545450e-04*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-2.6298701298701297e-03*w13*Jinv10+9.8620129870129864e-04*Jinv02*w39-3.9448051948051946e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-1.6436688311688311e-04*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3+4.2735389610389614e-03*Jinv20*w23-7.6704545454545450e-04*Jinv22*w38-4.9310064935064934e-03*Jinv00*w54+2.1915584415584417e-04*Jinv21*w59+4.2735389610389614e-03*w25*Jinv12+2.7394480519480518e-04*w0*Jinv02-4.2004870129870131e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10-3.9448051948051946e-03*w42*Jinv01-7.3051948051948052e-05*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+3.2873376623376621e-03*Jinv01*w45-2.1367694805194807e-03*Jinv12*w55+6.5746753246753243e-04*w58*Jinv10+9.8620129870129864e-04*w15*Jinv02+4.9310064935064932e-04*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+4.2735389610389614e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35+2.1915584415584417e-04*w58*Jinv11+2.1915584415584417e-04*w57*Jinv01+2.1915584415584414e-03*Jinv21*w26-4.2004870129870131e-04*w0*Jinv01+5.4788961038961036e-04*w1*Jinv10+9.8620129870129864e-04*w16*Jinv10-2.1367694805194807e-03*Jinv22*w20-3.9448051948051946e-03*w43*Jinv11+1.2820616883116883e-02*w45*Jinv00-1.9724025974025973e-03*Jinv21*w41+1.9724025974025973e-03*w42*Jinv02-1.0957792207792207e-03*Jinv00*w27+4.9310064935064932e-04*w11*Jinv22-4.2004870129870131e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-9.8620129870129864e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-1.0957792207792207e-03*Jinv20*w29+1.2820616883116883e-02*w46*Jinv10+9.8620129870129864e-04*Jinv22*w17-1.9724025974025973e-03*Jinv00*w33+2.1367694805194807e-03*w22*Jinv12-1.6436688311688311e-04*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3+4.2735389610389614e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22-1.9724025974025973e-03*Jinv01*w39+2.7394480519480518e-04*Jinv22*w2-2.6298701298701297e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+4.2735389610389614e-03*Jinv02*w45+2.1915584415584414e-03*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59-4.9310064935064934e-03*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11-2.6298701298701297e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34+1.9724025974025973e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11+2.9586038961038961e-03*w41*Jinv20-1.3149350649350649e-03*Jinv12*w13-2.6298701298701297e-03*w19*Jinv10-1.9724025974025973e-03*w34*Jinv10+1.1505681818181819e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-7.6704545454545450e-04*w37*Jinv12-4.9310064935064934e-03*Jinv20*w56+9.8620129870129864e-04*Jinv22*w41+4.2735389610389614e-03*w26*Jinv20+5.4788961038961036e-04*w11*Jinv20-7.6704545454545450e-04*w52*Jinv12-4.9310064935064934e-03*Jinv21*w35+1.1505681818181819e-03*w31*Jinv11+4.2735389610389614e-03*Jinv22*w26+3.2873376623376621e-03*w46*Jinv11-2.6298701298701297e-03*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-2.1367694805194807e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12+2.1367694805194807e-03*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-9.8620129870129864e-04*Jinv21*w29-4.9310064935064934e-03*Jinv01*w33+2.9586038961038961e-03*w39*Jinv00-1.3149350649350649e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11+4.9310064935064932e-04*Jinv02*w9-5.4788961038961042e-05*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10+3.2873376623376621e-03*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38+2.1367694805194807e-03*Jinv22*w23-2.6298701298701297e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12+4.2735389610389614e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8+4.2735389610389614e-03*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+1.0957792207792207e-03*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7+1.0957792207792207e-03*w21*Jinv01-2.6298701298701297e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.6436688311688311e-04*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10-5.4788961038961042e-05*w7*Jinv11+5.4788961038961036e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+4.2735389610389614e-03*w24*Jinv00-5.4788961038961036e-04*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-2.1367694805194807e-03*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5+4.2735389610389614e-03*w47*Jinv22-1.3149350649350649e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+5.4788961038961036e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44+4.9310064935064932e-04*w58*Jinv12-2.1367694805194807e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-5.4788961038961042e-05*Jinv21*w8+6.5746753246753243e-04*w57*Jinv00+6.5746753246753243e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+5.4788961038961036e-04*w0*Jinv00-1.9724025974025973e-03*w35*Jinv20+1.0957792207792207e-03*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+4.2735389610389614e-03*w25*Jinv10+5.4788961038961036e-04*Jinv20*w2+9.8620129870129864e-04*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-7.3051948051948052e-05*Jinv01*w9+9.8620129870129864e-04*Jinv20*w17-1.6436688311688311e-04*w30*Jinv00+2.9586038961038961e-03*w40*Jinv10-2.6298701298701297e-03*w20*Jinv20-5.4788961038961036e-04*Jinv22*w29-1.3149350649350649e-03*Jinv22*w14-1.9724025974025973e-03*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36+1.9724025974025973e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20+1.1505681818181819e-03*Jinv01*w30-7.6704545454545450e-04*Jinv02*w51-2.1367694805194807e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28+2.1915584415584414e-03*w25*Jinv11-4.9310064935064934e-03*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00+1.2820616883116883e-02*w47*Jinv20+9.8620129870129864e-04*w16*Jinv12-1.3149350649350649e-03*Jinv21*w50-9.8620129870129864e-04*Jinv01*w27-1.3149350649350649e-03*w48*Jinv01+2.7394480519480518e-04*Jinv12*w1-7.3051948051948052e-05*w10*Jinv11)*D;
  A[47][45]=(0.0)*D;
  A[47][46]=(0.0)*D;
  A[47][47]=(4.2735389610389614e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53+7.8530844155844158e-04*w57*Jinv02+4.2735389610389614e-03*Jinv02*w39+4.7666396103896100e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-7.1225649350649346e-04*Jinv22*w38+6.5746753246753243e-04*Jinv21*w59+3.6982548701298703e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.7666396103896100e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.5888798701298701e-03*w16*Jinv11+1.5888798701298701e-03*Jinv21*w17+7.3965097402597406e-03*Jinv01*w45-2.6298701298701297e-03*Jinv12*w55+2.1367694805194807e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+3.6982548701298703e-03*Jinv02*w24+1.0957792207792207e-03*Jinv22*w35+6.5746753246753243e-04*w58*Jinv11+6.5746753246753243e-04*w57*Jinv01+2.4655032467532467e-03*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-2.3833198051948050e-03*Jinv22*w20+4.7666396103896100e-03*w43*Jinv11+4.7666396103896100e-03*Jinv21*w41+4.2735389610389614e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-2.0271915584415584e-03*w13*Jinv11+1.5888798701298701e-03*w15*Jinv01-9.1314935064935068e-05*w28*Jinv11+2.1367694805194807e-03*Jinv22*w17+3.6982548701298703e-03*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-7.1225649350649346e-04*w32*Jinv22+4.7666396103896100e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-1.0683847402597404e-03*Jinv21*w38+7.3965097402597406e-03*Jinv02*w45+2.4655032467532467e-03*Jinv01*w24+7.8530844155844158e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-2.0271915584415584e-03*w19*Jinv11+5.7528409090909093e-04*Jinv21*w53+1.0957792207792207e-03*Jinv12*w34+4.2735389610389614e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-2.3833198051948050e-03*Jinv12*w13-1.0683847402597404e-03*w32*Jinv21-2.0271915584415584e-03*Jinv21*w20-7.1225649350649346e-04*w37*Jinv12+4.2735389610389614e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35-1.0683847402597404e-03*w31*Jinv11+3.6982548701298703e-03*Jinv22*w26+7.3965097402597406e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21-2.6298701298701297e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12+3.6982548701298703e-03*w21*Jinv02-9.1314935064935068e-05*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33-2.3833198051948050e-03*Jinv02*w12-2.0271915584415584e-03*Jinv01*w18-1.0683847402597404e-03*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01+7.3965097402597406e-03*w47*Jinv21+3.6982548701298703e-03*Jinv22*w23+5.7528409090909093e-04*w52*Jinv11-7.1225649350649346e-04*w31*Jinv12+7.3965097402597406e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50+2.4655032467532467e-03*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+2.4655032467532467e-03*w21*Jinv01+5.7528409090909093e-04*Jinv01*w51-7.1225649350649346e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49-1.0683847402597404e-03*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11-1.3149350649350649e-03*Jinv21*w56+1.0957792207792207e-03*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5+7.3965097402597406e-03*w47*Jinv22+5.7528409090909093e-04*w49*Jinv11-2.0271915584415584e-03*Jinv21*w14+7.8530844155844158e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-2.0271915584415584e-03*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.0957792207792207e-03*Jinv02*w33+2.4655032467532467e-03*Jinv21*w23-1.3149350649350649e-03*Jinv01*w54-1.3149350649350649e-03*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+1.0957792207792207e-03*Jinv22*w29-2.3833198051948050e-03*Jinv22*w14+4.7666396103896100e-03*w40*Jinv11-7.1225649350649346e-04*Jinv02*w36+4.2735389610389614e-03*Jinv12*w43-1.0683847402597404e-03*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51-2.6298701298701297e-03*Jinv22*w56+1.0957792207792207e-03*Jinv12*w28+2.4655032467532467e-03*w25*Jinv11+2.1367694805194807e-03*w16*Jinv12+5.7528409090909093e-04*Jinv21*w50-9.1314935064935068e-05*Jinv01*w27+5.7528409090909093e-04*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[47][48]=(0.0)*D;
  A[47][49]=(0.0)*D;
  A[47][50]=(-5.2049512987012989e-04*Jinv00*w12+4.7483766233766234e-04*w28*Jinv10-6.0267857142857139e-04*w40*Jinv12+1.3697240259740260e-03*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+6.6659902597402602e-04*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10-6.0267857142857139e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+2.6298701298701297e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+2.5568181818181820e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+3.8352272727272725e-04*w16*Jinv11+3.8352272727272725e-04*Jinv21*w17-1.1505681818181819e-03*Jinv01*w45+1.3971185064935066e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-04*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12+2.3833198051948050e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-1.2053571428571428e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+3.8352272727272725e-04*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-1.1505681818181819e-03*w45*Jinv00+2.1367694805194807e-03*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02+4.7483766233766234e-04*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-5.2049512987012989e-04*w13*Jinv11+3.8352272727272725e-04*w15*Jinv01+4.7483766233766234e-04*w28*Jinv11+2.3833198051948050e-03*Jinv20*w50+4.7483766233766234e-04*Jinv20*w29-1.1505681818181819e-03*w46*Jinv10-3.2873376623376621e-04*Jinv22*w17+1.3149350649350649e-03*Jinv00*w33+5.4788961038961036e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10-2.7394480519480521e-05*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00-9.5880681818181818e-04*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39+2.5568181818181820e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38+3.2873376623376621e-04*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+6.6659902597402602e-04*Jinv22*w59+1.3149350649350649e-03*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00-1.2053571428571428e-03*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-3.8352272727272725e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+1.3149350649350649e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+2.6298701298701297e-03*Jinv20*w56-6.0267857142857139e-04*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.3697240259740260e-03*w52*Jinv12+1.3149350649350649e-03*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26-1.1505681818181819e-03*w46*Jinv11-5.2049512987012989e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+1.3971185064935066e-03*Jinv02*w54-2.7394480519480521e-05*w4*Jinv12+5.4788961038961036e-04*w21*Jinv02+2.3833198051948050e-03*w49*Jinv10+4.7483766233766234e-04*Jinv21*w29+1.3149350649350649e-03*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-3.8352272727272725e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9-2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10-1.1505681818181819e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38+5.4788961038961036e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11-9.5880681818181818e-04*w31*Jinv12+3.2873376623376621e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53+1.9176136363636362e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+1.9176136363636362e-04*Jinv12*w7+1.6436688311688311e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51-9.5880681818181818e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-2.7394480519480518e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+2.6298701298701297e-03*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27+1.9176136363636362e-04*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12-2.7394480519480521e-05*Jinv22*w5+3.2873376623376621e-04*w47*Jinv22+2.3833198051948050e-03*w49*Jinv11-5.2049512987012989e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+6.6659902597402602e-04*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-5.2049512987012989e-04*Jinv01*w12-2.7394480519480518e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-1.2053571428571428e-03*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+1.3149350649350649e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23+2.6298701298701297e-03*Jinv01*w54+2.6298701298701297e-03*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+3.8352272727272725e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+3.8352272727272725e-04*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-3.8352272727272725e-04*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30+1.3697240259740260e-03*Jinv02*w51+1.3971185064935066e-03*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11+2.6298701298701297e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-1.1505681818181819e-03*w47*Jinv20-3.2873376623376621e-04*w16*Jinv12+2.3833198051948050e-03*Jinv21*w50+4.7483766233766234e-04*Jinv01*w27+2.3833198051948050e-03*w48*Jinv01+2.5568181818181820e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[47][51]=(0.0)*D;
  A[47][52]=(0.0)*D;
  A[47][53]=(5.2049512987012989e-04*Jinv00*w12-4.7483766233766234e-04*w28*Jinv10-1.3697240259740260e-03*w40*Jinv12-2.0271915584415584e-03*Jinv22*w53+2.7394480519480518e-04*Jinv20*w8+1.1870941558441559e-04*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-1.3697240259740260e-03*Jinv02*w39+2.7394480519480518e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20-1.6436688311688311e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38-2.6298701298701297e-03*Jinv00*w54+3.8352272727272725e-04*w25*Jinv12-3.1960227272727273e-04*w0*Jinv02-2.1367694805194807e-03*w43*Jinv10-1.2327516233766234e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-7.1225649350649346e-04*w15*Jinv02-1.7349837662337662e-04*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+3.8352272727272725e-04*Jinv02*w24-2.5202922077922079e-03*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-3.8352272727272725e-04*w16*Jinv10+1.3697240259740259e-04*Jinv22*w20+1.1505681818181819e-03*w45*Jinv00-2.7394480519480520e-03*w42*Jinv02-4.7483766233766234e-04*Jinv00*w27-1.7349837662337662e-04*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-4.7483766233766234e-04*Jinv20*w29+1.1505681818181819e-03*w46*Jinv10-7.1225649350649346e-04*Jinv22*w17-1.3149350649350649e-03*Jinv00*w33+1.9176136363636362e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+4.6570616883116886e-04*Jinv02*w3-1.6436688311688311e-04*w21*Jinv00+8.4922889610389610e-04*w32*Jinv22-3.1960227272727273e-04*Jinv22*w2+5.2049512987012989e-04*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+1.4793019480519481e-03*Jinv02*w45+1.1870941558441559e-04*Jinv22*w59-2.3833198051948050e-03*w51*Jinv00-2.5202922077922079e-03*Jinv12*w34-2.7394480519480520e-03*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10-2.1367694805194807e-03*w41*Jinv20+2.4655032467532466e-04*Jinv12*w13+5.2049512987012989e-04*w19*Jinv10-1.3149350649350649e-03*w34*Jinv10+2.7394480519480518e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12-2.6298701298701297e-03*Jinv20*w56-1.3697240259740260e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-2.0271915584415584e-03*w52*Jinv12+3.8352272727272725e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-1.2327516233766234e-03*Jinv02*w54+4.6570616883116886e-04*w4*Jinv12+1.9176136363636362e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-2.1367694805194807e-03*w39*Jinv00+2.4655032467532466e-04*Jinv02*w12-1.7349837662337662e-04*Jinv02*w9+2.7394480519480518e-04*w7*Jinv10+1.3971185064935066e-03*Jinv20*w38+1.9176136363636362e-04*Jinv22*w23+8.4922889610389610e-04*w31*Jinv12+1.4793019480519481e-03*w46*Jinv12-2.3833198051948050e-03*Jinv20*w53+2.4655032467532466e-04*Jinv22*w8-1.6436688311688311e-04*w22*Jinv10-1.0135957792207792e-03*Jinv22*w50+2.4655032467532466e-04*Jinv12*w7+8.4922889610389610e-04*Jinv02*w30-1.0135957792207792e-03*Jinv12*w49-2.3833198051948050e-03*w52*Jinv10-4.2918019480519480e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10-1.6436688311688311e-04*w24*Jinv00-4.8396915584415583e-04*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+1.3697240259740259e-04*w19*Jinv12+4.6570616883116886e-04*Jinv22*w5+1.4793019480519481e-03*w47*Jinv22-4.2918019480519480e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+1.1870941558441559e-04*w58*Jinv12+1.3697240259740259e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-2.5202922077922079e-03*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-1.3149350649350649e-03*w35*Jinv20-1.6436688311688311e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-3.8352272727272725e-04*w15*Jinv00-1.0135957792207792e-03*w48*Jinv02-3.8352272727272725e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-4.8396915584415583e-04*Jinv22*w29+2.4655032467532466e-04*Jinv22*w14+4.3831168831168834e-04*Jinv02*w36-2.7394480519480520e-03*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20-2.0271915584415584e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56-4.8396915584415583e-04*Jinv12*w28-2.6298701298701297e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.1505681818181819e-03*w47*Jinv20-7.1225649350649346e-04*w16*Jinv12-3.1960227272727273e-04*Jinv12*w1)*D;
  A[47][54]=(0.0)*D;
  A[47][55]=(0.0)*D;
  A[47][56]=(-6.5746753246753243e-04*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53-1.4610389610389610e-04*w57*Jinv02-6.5746753246753243e-04*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+2.7394480519480518e-04*Jinv01*w3+4.3831168831168834e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.5523538961038962e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-2.1367694805194807e-03*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-3.8352272727272725e-04*w16*Jinv11-3.8352272727272725e-04*Jinv21*w17+1.1505681818181819e-03*Jinv01*w45-1.3149350649350649e-03*Jinv12*w55-8.2183441558441553e-04*w15*Jinv02-1.5523538961038962e-04*w10*Jinv12-2.4655032467532466e-04*Jinv02*w24-1.3697240259740260e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+8.2183441558441553e-05*Jinv22*w20-2.1367694805194807e-03*w43*Jinv11-2.1367694805194807e-03*Jinv21*w41-6.5746753246753243e-04*w42*Jinv02-1.5523538961038962e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+5.2049512987012989e-04*w13*Jinv11-3.8352272727272725e-04*w15*Jinv01-4.7483766233766234e-04*w28*Jinv11-8.2183441558441553e-04*Jinv22*w17-2.4655032467532466e-04*w22*Jinv12+2.4655032467532466e-04*Jinv02*w3+4.3831168831168834e-04*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-1.5523538961038962e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38+3.7804383116883116e-03*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-1.4610389610389610e-04*Jinv22*w59-1.3149350649350649e-03*w34*Jinv11+5.2049512987012989e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-1.3697240259740260e-03*Jinv12*w34-6.5746753246753243e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv11+8.2183441558441553e-05*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21+5.2049512987012989e-04*Jinv21*w20+4.3831168831168834e-04*w37*Jinv12-6.5746753246753243e-04*Jinv22*w41-1.2327516233766234e-03*w52*Jinv12-1.3149350649350649e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26+1.1505681818181819e-03*w46*Jinv11+2.7394480519480518e-04*w5*Jinv21-1.3149350649350649e-03*Jinv02*w54+2.4655032467532466e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-4.7483766233766234e-04*Jinv21*w29-1.3149350649350649e-03*Jinv01*w33+8.2183441558441553e-05*Jinv02*w12+5.2049512987012989e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-1.5523538961038962e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01+1.1505681818181819e-03*w47*Jinv21-2.4655032467532466e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+4.3831168831168834e-04*w31*Jinv12+3.7804383116883116e-03*w46*Jinv12+2.4655032467532466e-04*Jinv22*w8-1.2327516233766234e-03*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+2.4655032467532466e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+4.3831168831168834e-04*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+2.7394480519480518e-04*w7*Jinv11-2.6298701298701297e-03*Jinv21*w56-1.0957792207792207e-03*Jinv02*w27+2.4655032467532466e-04*w6*Jinv02+8.2183441558441553e-05*w19*Jinv12+2.4655032467532466e-04*Jinv22*w5+3.7804383116883116e-03*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+5.2049512987012989e-04*Jinv21*w14-1.4610389610389610e-04*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18+5.2049512987012989e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.3697240259740260e-03*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23-2.6298701298701297e-03*Jinv01*w54-2.6298701298701297e-03*w55*Jinv11-1.2327516233766234e-03*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9-1.0957792207792207e-03*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+4.3831168831168834e-04*Jinv02*w36-6.5746753246753243e-04*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-1.3149350649350649e-03*Jinv22*w56-1.0957792207792207e-03*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-8.2183441558441553e-04*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-4.7483766233766234e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-1.5523538961038962e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[47][57]=(0.0)*D;
  A[47][58]=(0.0)*D;
  A[47][59]=(-3.2873376623376621e-04*w40*Jinv12-6.7573051948051951e-04*Jinv22*w53-3.2467532467532468e-04*w57*Jinv02-3.2873376623376621e-04*Jinv02*w39+4.9310064935064932e-04*Jinv22*w38+3.6525974025974026e-05*w25*Jinv12-1.1668019480519480e-04*w0*Jinv02-1.0227272727272728e-03*Jinv12*w55+2.1915584415584417e-04*w15*Jinv02-1.1668019480519480e-04*w10*Jinv12+3.6525974025974026e-05*Jinv02*w24+2.9220779220779221e-04*Jinv22*w35+1.1870941558441559e-04*Jinv22*w20-3.2873376623376621e-04*w42*Jinv02-1.1668019480519480e-04*w11*Jinv22+2.1915584415584417e-04*Jinv22*w17+3.6525974025974026e-05*w22*Jinv12-2.7394480519480521e-05*Jinv02*w3+4.9310064935064932e-04*w32*Jinv22-1.1668019480519480e-04*Jinv22*w2-2.5568181818181820e-04*Jinv02*w45-3.2467532467532468e-04*Jinv22*w59+2.9220779220779221e-04*Jinv12*w34-3.2873376623376621e-04*Jinv22*w44+1.1870941558441559e-04*Jinv12*w13+4.9310064935064932e-04*w37*Jinv12-3.2873376623376621e-04*Jinv22*w41-6.7573051948051951e-04*w52*Jinv12+3.6525974025974026e-05*Jinv22*w26-1.0227272727272728e-03*Jinv02*w54-2.7394480519480521e-05*w4*Jinv12+3.6525974025974026e-05*w21*Jinv02+1.1870941558441559e-04*Jinv02*w12-1.1668019480519480e-04*Jinv02*w9+3.6525974025974026e-05*Jinv22*w23+4.9310064935064932e-04*w31*Jinv12-2.5568181818181820e-04*w46*Jinv12-2.7394480519480521e-05*Jinv22*w8-6.7573051948051951e-04*Jinv22*w50-2.7394480519480521e-05*Jinv12*w7+4.9310064935064932e-04*Jinv02*w30-6.7573051948051951e-04*Jinv12*w49+8.3198051948051951e-05*Jinv02*w27-2.7394480519480521e-05*w6*Jinv02+1.1870941558441559e-04*w19*Jinv12-2.7394480519480521e-05*Jinv22*w5-2.5568181818181820e-04*w47*Jinv22-3.2467532467532468e-04*w58*Jinv12+1.1870941558441559e-04*Jinv02*w18+2.9220779220779221e-04*Jinv02*w33-6.7573051948051951e-04*w48*Jinv02+8.3198051948051951e-05*Jinv22*w29+1.1870941558441559e-04*Jinv22*w14+4.9310064935064932e-04*Jinv02*w36-3.2873376623376621e-04*Jinv12*w43-6.7573051948051951e-04*Jinv02*w51-1.0227272727272728e-03*Jinv22*w56+8.3198051948051951e-05*Jinv12*w28+2.1915584415584417e-04*w16*Jinv12-1.1668019480519480e-04*Jinv12*w1)*D;
  A[48][0]=(6.7573051948051951e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12-3.6525974025974026e-05*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8-8.3198051948051951e-05*w57*Jinv02+6.7573051948051951e-04*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+6.7573051948051951e-04*w3*Jinv00+1.0227272727272728e-03*w32*Jinv20+6.7573051948051951e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-1.1870941558441559e-04*Jinv22*w38-3.6525974025974026e-05*Jinv00*w54-8.3198051948051951e-05*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21-2.9220779220779221e-04*w16*Jinv11-2.9220779220779221e-04*Jinv21*w17-1.1870941558441559e-04*Jinv01*w45-3.6525974025974026e-05*Jinv12*w55-8.3198051948051951e-05*w58*Jinv10-2.9220779220779221e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12+2.5568181818181820e-04*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35-8.3198051948051951e-05*w58*Jinv11-8.3198051948051951e-05*w57*Jinv01+2.7394480519480521e-05*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10-2.9220779220779221e-04*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-1.1870941558441559e-04*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+6.7573051948051951e-04*w13*Jinv11-2.9220779220779221e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+2.5568181818181820e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29-1.1870941558441559e-04*w46*Jinv10-2.9220779220779221e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+1.0227272727272728e-03*w31*Jinv10+6.7573051948051951e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+1.0227272727272728e-03*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2+2.7394480519480521e-05*Jinv00*w18-1.1870941558441559e-04*w36*Jinv00-1.1870941558441559e-04*Jinv21*w38-1.1870941558441559e-04*Jinv02*w45+2.7394480519480521e-05*Jinv01*w24-8.3198051948051951e-05*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11+2.7394480519480521e-05*w19*Jinv11-3.6525974025974026e-05*Jinv21*w53-3.6525974025974026e-05*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+6.7573051948051951e-04*w4*Jinv10+6.7573051948051951e-04*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+6.7573051948051951e-04*Jinv12*w13+2.7394480519480521e-05*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+1.0227272727272728e-03*w32*Jinv21+2.7394480519480521e-05*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00-1.1870941558441559e-04*w37*Jinv12-3.6525974025974026e-05*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41+2.7394480519480521e-05*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20-3.6525974025974026e-05*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35+1.0227272727272728e-03*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-1.1870941558441559e-04*w46*Jinv11+6.7573051948051951e-04*Jinv20*w14+6.7573051948051951e-04*w5*Jinv21-3.6525974025974026e-05*Jinv02*w54+6.7573051948051951e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.5568181818181820e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+6.7573051948051951e-04*Jinv02*w12+2.7394480519480521e-05*Jinv01*w18-1.1870941558441559e-04*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-1.1870941558441559e-04*w47*Jinv21-1.1870941558441559e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-3.6525974025974026e-05*w52*Jinv11+1.0227272727272728e-03*w31*Jinv12-1.1870941558441559e-04*w46*Jinv12-3.6525974025974026e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10+2.5568181818181820e-04*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01-3.6525974025974026e-05*Jinv01*w51+1.0227272727272728e-03*Jinv02*w30+2.5568181818181820e-04*Jinv12*w49-1.1870941558441559e-04*w36*Jinv01-3.6525974025974026e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00-1.1870941558441559e-04*w37*Jinv10-3.6525974025974026e-05*Jinv21*w56+2.7394480519480521e-05*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12+6.7573051948051951e-04*Jinv22*w5-1.1870941558441559e-04*w47*Jinv22+2.5568181818181820e-04*w49*Jinv11+6.7573051948051951e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-8.3198051948051951e-05*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18+6.7573051948051951e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-8.3198051948051951e-05*w57*Jinv00-8.3198051948051951e-05*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23-3.6525974025974026e-05*Jinv01*w54-3.6525974025974026e-05*w55*Jinv11+2.7394480519480521e-05*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2-2.9220779220779221e-04*w15*Jinv00+2.5568181818181820e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9-2.9220779220779221e-04*Jinv20*w17+1.0227272727272728e-03*w30*Jinv00+3.2873376623376621e-04*w40*Jinv10+2.7394480519480521e-05*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+6.7573051948051951e-04*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11-1.1870941558441559e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+6.7573051948051951e-04*w5*Jinv20+1.0227272727272728e-03*Jinv01*w30-3.6525974025974026e-05*Jinv02*w51-3.6525974025974026e-05*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28+2.7394480519480521e-05*w25*Jinv11-3.6525974025974026e-05*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-1.1870941558441559e-04*w47*Jinv20-2.9220779220779221e-04*w16*Jinv12+2.5568181818181820e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27+2.5568181818181820e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[48][1]=(0.0)*D;
  A[48][2]=(0.0)*D;
  A[48][3]=(-1.3697240259740260e-03*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+9.5880681818181818e-04*Jinv20*w8+4.8396915584415583e-04*w57*Jinv02-1.3697240259740260e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20+2.0271915584415584e-03*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23-1.3697240259740259e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+4.8396915584415583e-04*Jinv21*w59-4.6570616883116886e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+3.2873376623376621e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+2.5202922077922079e-03*w16*Jinv11+2.5202922077922079e-03*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-1.9176136363636362e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10+2.5202922077922079e-03*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12-3.2873376623376621e-04*w48*Jinv00-4.6570616883116886e-04*Jinv02*w24+2.7394480519480520e-03*Jinv22*w35+4.8396915584415583e-04*w58*Jinv11+4.8396915584415583e-04*w57*Jinv01-4.6570616883116886e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-6.6659902597402602e-04*w1*Jinv10+1.2053571428571428e-03*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11+2.7394480519480518e-04*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.0135957792207792e-03*w13*Jinv11+2.5202922077922079e-03*w15*Jinv01+3.1960227272727273e-04*w28*Jinv11-3.2873376623376621e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10+2.5202922077922079e-03*Jinv22*w17+6.0267857142857139e-04*Jinv00*w33-8.4922889610389610e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10+2.0271915584415584e-03*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+2.7394480519480521e-05*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00-1.3697240259740259e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-4.6570616883116886e-04*Jinv01*w24+4.8396915584415583e-04*Jinv22*w59+2.7394480519480520e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11-3.8352272727272725e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+2.7394480519480520e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+2.0271915584415584e-03*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+1.0135957792207792e-03*Jinv12*w13+2.7394480519480521e-05*w19*Jinv10+6.0267857142857139e-04*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20+9.5880681818181818e-04*w6*Jinv00-1.3697240259740259e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41-1.9176136363636362e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-3.8352272727272725e-04*w52*Jinv12+2.7394480519480520e-03*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-4.6570616883116886e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11-1.3697240259740260e-03*Jinv20*w14+2.0271915584415584e-03*w5*Jinv21-1.9176136363636362e-04*Jinv02*w54+2.0271915584415584e-03*w4*Jinv12-8.4922889610389610e-04*w21*Jinv02-3.2873376623376621e-04*w49*Jinv10+3.1960227272727273e-04*Jinv21*w29+2.7394480519480520e-03*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+1.0135957792207792e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-1.3697240259740259e-04*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01+9.5880681818181818e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-8.4922889610389610e-04*Jinv22*w23-3.8352272727272725e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50-8.4922889610389610e-04*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-8.4922889610389610e-04*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49-1.3697240259740259e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.9176136363636362e-04*Jinv21*w56-1.9176136363636362e-04*w24*Jinv00+3.1960227272727273e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+2.0271915584415584e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-1.4793019480519481e-03*w49*Jinv11+1.0135957792207792e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+4.8396915584415583e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+1.0135957792207792e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59+2.7394480519480520e-03*Jinv02*w33-6.6659902597402602e-04*w0*Jinv00+6.0267857142857139e-04*w35*Jinv20-8.4922889610389610e-04*Jinv21*w23-1.9176136363636362e-04*Jinv01*w54-1.9176136363636362e-04*w55*Jinv11-1.9176136363636362e-04*w25*Jinv10-6.6659902597402602e-04*Jinv20*w2+1.2053571428571428e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+1.2053571428571428e-03*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480521e-05*w20*Jinv20+3.1960227272727273e-04*Jinv22*w29+1.0135957792207792e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-1.3697240259740259e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30-3.8352272727272725e-04*Jinv02*w51-1.9176136363636362e-04*Jinv22*w56+3.1960227272727273e-04*Jinv12*w28-4.6570616883116886e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20+2.5202922077922079e-03*w16*Jinv12-1.4793019480519481e-03*Jinv21*w50+3.1960227272727273e-04*Jinv01*w27-1.4793019480519481e-03*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[48][4]=(0.0)*D;
  A[48][5]=(0.0)*D;
  A[48][6]=(1.1231737012987013e-03*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02+1.1231737012987013e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00+9.5880681818181818e-04*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2-3.2873376623376621e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-1.4245129870129869e-03*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+3.5612824675324673e-04*w1*Jinv10-1.2601461038961039e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11+2.7394480519480518e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11-1.4245129870129869e-03*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17-1.3697240259740260e-03*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+9.5880681818181818e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+1.6436688311688311e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-1.3697240259740260e-03*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.9176136363636362e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11+1.1231737012987013e-03*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-1.4245129870129869e-03*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21-8.2183441558441553e-05*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00+3.5612824675324673e-04*Jinv21*w56+1.9176136363636362e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33+3.5612824675324673e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+1.9176136363636362e-04*w25*Jinv10+3.5612824675324673e-04*Jinv20*w2-1.2601461038961039e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-1.2601461038961039e-03*Jinv20*w17+9.5880681818181818e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10+1.0957792207792208e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[48][7]=(0.0)*D;
  A[48][8]=(0.0)*D;
  A[48][9]=(-4.2918019480519480e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+1.1870941558441559e-04*w3*Jinv00-5.8441558441558442e-04*w32*Jinv20-2.1915584415584417e-04*Jinv20*w23+7.4878246753246752e-04*Jinv00*w54+2.1915584415584417e-04*w43*Jinv10+6.4935064935064935e-05*w58*Jinv10+1.4975649350649350e-03*w48*Jinv00-1.4204545454545455e-05*w1*Jinv10+3.4699675324675324e-04*w16*Jinv10-4.2918019480519480e-04*w45*Jinv00+1.0247564935064935e-04*Jinv00*w27+1.4975649350649350e-03*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+4.3831168831168834e-04*Jinv00*w33-5.8441558441558442e-04*w31*Jinv10-2.1915584415584417e-04*w21*Jinv00-5.4788961038961042e-05*Jinv00*w18-2.6481331168831169e-04*w36*Jinv00+3.6525974025974026e-05*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-5.4788961038961042e-05*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00+7.4878246753246752e-04*Jinv20*w56-2.7394480519480521e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-4.2918019480519480e-04*Jinv20*w14+1.4975649350649350e-03*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-1.0957792207792208e-04*w7*Jinv10-2.6481331168831169e-04*Jinv20*w38+3.6525974025974026e-05*Jinv20*w53-2.1915584415584417e-04*w22*Jinv10+3.6525974025974026e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-2.6481331168831169e-04*w37*Jinv10-2.7394480519480521e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+2.1915584415584417e-04*Jinv20*w44+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-2.7394480519480521e-05*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+3.4699675324675324e-04*w15*Jinv00+3.4699675324675324e-04*Jinv20*w17-5.8441558441558442e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+1.1870941558441559e-04*w5*Jinv20+7.4878246753246752e-04*w55*Jinv10+2.1915584415584417e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20)*D;
  A[48][10]=(0.0)*D;
  A[48][11]=(0.0)*D;
  A[48][12]=(2.0271915584415584e-03*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+2.7394480519480520e-03*w40*Jinv12-1.9176136363636362e-04*Jinv22*w53-8.4922889610389610e-04*Jinv20*w8+4.8396915584415583e-04*w57*Jinv02+2.0271915584415584e-03*w13*Jinv10+2.7394480519480520e-03*Jinv02*w39+3.2873376623376621e-04*Jinv21*w44+1.0135957792207792e-03*w3*Jinv00+1.2327516233766234e-03*w32*Jinv20-1.3697240259740260e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-3.8352272727272725e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-6.6659902597402602e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+3.2873376623376621e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21+1.2053571428571428e-03*w16*Jinv11+1.2053571428571428e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45-3.8352272727272725e-04*Jinv12*w55+4.8396915584415583e-04*w58*Jinv10+2.5202922077922079e-03*w15*Jinv02+3.1960227272727273e-04*w10*Jinv12-1.4793019480519481e-03*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480521e-05*Jinv21*w26-6.6659902597402602e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+2.5202922077922079e-03*w16*Jinv10-4.6570616883116886e-04*Jinv22*w20+3.2873376623376621e-04*w43*Jinv11-1.3697240259740259e-04*w45*Jinv00+6.0267857142857139e-04*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+3.1960227272727273e-04*w11*Jinv22-6.6659902597402602e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+1.2053571428571428e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11-1.4793019480519481e-03*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-1.3697240259740259e-04*w46*Jinv10+2.5202922077922079e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10+1.0135957792207792e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+6.0267857142857139e-04*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-4.6570616883116886e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00+2.7394480519480518e-04*Jinv21*w38-1.3697240259740259e-04*Jinv02*w45+2.7394480519480521e-05*Jinv01*w24+4.8396915584415583e-04*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-1.9176136363636362e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-1.9176136363636362e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44+1.0135957792207792e-03*w4*Jinv10-1.3697240259740260e-03*w4*Jinv11+2.7394480519480520e-03*w41*Jinv20+2.0271915584415584e-03*Jinv12*w13-4.6570616883116886e-04*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21-1.9176136363636362e-04*Jinv21*w20-8.4922889610389610e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-3.8352272727272725e-04*Jinv20*w56+2.7394480519480520e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+3.1960227272727273e-04*w11*Jinv20-1.9176136363636362e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11+2.0271915584415584e-03*Jinv20*w14-1.3697240259740260e-03*w5*Jinv21-3.8352272727272725e-04*Jinv02*w54+1.0135957792207792e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02-1.4793019480519481e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+2.7394480519480520e-03*w39*Jinv00+2.0271915584415584e-03*Jinv02*w12-1.9176136363636362e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11+3.1960227272727273e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01-8.4922889610389610e-04*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-1.3697240259740259e-04*w46*Jinv12-1.9176136363636362e-04*Jinv20*w53-8.4922889610389610e-04*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+9.5880681818181818e-04*w22*Jinv11-8.4922889610389610e-04*Jinv12*w7+9.5880681818181818e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01-1.9176136363636362e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11+3.1960227272727273e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27-8.4922889610389610e-04*w6*Jinv02-4.6570616883116886e-04*w19*Jinv12+1.0135957792207792e-03*Jinv22*w5-1.3697240259740259e-04*w47*Jinv22-3.2873376623376621e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+3.1960227272727273e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44+4.8396915584415583e-04*w58*Jinv12-4.6570616883116886e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.8396915584415583e-04*w57*Jinv00+4.8396915584415583e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20+9.5880681818181818e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+2.5202922077922079e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+2.5202922077922079e-03*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+2.7394480519480520e-03*w40*Jinv10-4.6570616883116886e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29+2.0271915584415584e-03*Jinv22*w14+6.0267857142857139e-04*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43+1.0135957792207792e-03*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30-1.9176136363636362e-04*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28+2.7394480519480521e-05*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-1.3697240259740259e-04*w47*Jinv20+2.5202922077922079e-03*w16*Jinv12-3.2873376623376621e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27-3.2873376623376621e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[48][13]=(0.0)*D;
  A[48][14]=(0.0)*D;
  A[48][15]=(-2.5750811688311690e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-1.0957792207792207e-03*Jinv22*w53+1.2053571428571428e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-2.5750811688311690e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-1.2601461038961039e-03*w3*Jinv00-1.7532467532467534e-03*w32*Jinv20-2.5750811688311690e-03*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2-3.6525974025974026e-05*w11*Jinv21-1.6436688311688311e-03*w16*Jinv11-1.6436688311688311e-03*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45-1.0957792207792207e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+2.1915584415584414e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+8.2183441558441553e-04*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-1.6436688311688311e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20-3.8352272727272725e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-1.2601461038961039e-03*w13*Jinv11-1.6436688311688311e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+2.1915584415584414e-03*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-1.7532467532467534e-03*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2+8.2183441558441553e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00-3.8352272727272725e-04*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+8.2183441558441553e-04*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+1.0957792207792207e-03*Jinv21*w53+1.3149350649350649e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44-1.2601461038961039e-03*w4*Jinv10-2.5750811688311690e-03*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+8.2183441558441553e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.7532467532467534e-03*w32*Jinv21+1.2053571428571428e-03*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w11*Jinv20-1.0957792207792207e-03*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35-1.7532467532467534e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+3.2873376623376621e-04*w46*Jinv11-2.5750811688311690e-03*Jinv20*w14-2.5750811688311690e-03*w5*Jinv21-1.0957792207792207e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02+2.1915584415584414e-03*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-3.8352272727272725e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.2053571428571428e-03*w7*Jinv10+3.2873376623376621e-04*w47*Jinv21+3.2873376623376621e-04*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+5.4788961038961042e-05*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-1.0957792207792207e-03*Jinv22*w50+1.2053571428571428e-03*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+1.2053571428571428e-03*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10+3.6525974025974026e-05*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+2.1915584415584414e-03*w49*Jinv11-1.2601461038961039e-03*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10+4.9310064935064932e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.2601461038961039e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+1.2053571428571428e-03*Jinv21*w23+2.5568181818181820e-04*Jinv20*w2-1.6436688311688311e-03*w15*Jinv00-1.0957792207792207e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-1.6436688311688311e-03*Jinv20*w17-1.7532467532467534e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+8.2183441558441553e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43-1.2601461038961039e-03*w5*Jinv20-1.7532467532467534e-03*Jinv01*w30-1.0957792207792207e-03*Jinv02*w51-1.0957792207792207e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+8.2183441558441553e-04*w25*Jinv11+1.0957792207792207e-03*w55*Jinv10-3.8352272727272725e-04*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12+2.1915584415584414e-03*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27+2.1915584415584414e-03*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[48][16]=(0.0)*D;
  A[48][17]=(0.0)*D;
  A[48][18]=(5.4788961038961036e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+5.4788961038961036e-04*w13*Jinv10-3.8352272727272725e-04*Jinv21*w44+2.4655032467532466e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.2049512987012989e-04*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23-7.1225649350649346e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-7.1225649350649346e-04*w43*Jinv10-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-8.7662337662337668e-04*w16*Jinv10-3.8352272727272725e-04*w43*Jinv11+5.2049512987012989e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-1.3697240259740259e-04*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10+5.4788961038961036e-04*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-3.5612824675324673e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+2.7394480519480518e-04*Jinv01*w24-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+1.9176136363636362e-04*w51*Jinv00+2.4655032467532466e-04*w4*Jinv10+5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-7.1225649350649346e-04*Jinv20*w56+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20-7.6704545454545450e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14+5.2049512987012989e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-3.5612824675324673e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11+1.9176136363636362e-04*Jinv20*w53+5.4788961038961036e-04*w22*Jinv10+8.2183441558441553e-05*w22*Jinv11+8.2183441558441553e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01+1.9176136363636362e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11+6.3920454545454550e-05*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+6.3920454545454550e-05*w10*Jinv10-7.1225649350649346e-04*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-8.7662337662337668e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-8.7662337662337668e-04*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+2.4655032467532466e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+2.7394480519480518e-04*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10-7.1225649350649346e-04*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[48][19]=(0.0)*D;
  A[48][20]=(0.0)*D;
  A[48][21]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39-3.2873376623376621e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20+1.1231737012987013e-03*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+3.5612824675324673e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10-3.2873376623376621e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-1.2601461038961039e-03*w16*Jinv11-1.2601461038961039e-03*Jinv21*w17+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+3.5612824675324673e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20-3.2873376623376621e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22+3.5612824675324673e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11-1.2601461038961039e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22-1.3697240259740260e-03*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+2.7394480519480518e-04*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-9.1314935064935065e-06*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+1.9176136363636362e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10+1.1231737012987013e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+9.5880681818181818e-04*w32*Jinv21+1.9176136363636362e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+9.5880681818181818e-04*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26-5.2049512987012989e-04*Jinv20*w14+1.1231737012987013e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+1.9176136363636362e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10-2.7394480519480518e-04*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-1.4245129870129869e-03*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14-1.3697240259740260e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20+9.5880681818181818e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-1.4245129870129869e-03*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27-1.4245129870129869e-03*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[48][22]=(0.0)*D;
  A[48][23]=(0.0)*D;
  A[48][24]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+8.2183441558441553e-05*Jinv20*w8+5.2049512987012989e-04*w13*Jinv10-7.1225649350649346e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w43*Jinv10-7.1225649350649346e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-8.7662337662337668e-04*w16*Jinv11-8.7662337662337668e-04*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-7.1225649350649346e-04*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-8.7662337662337668e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+2.7394480519480518e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+5.2049512987012989e-04*Jinv21*w38-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-1.0957792207792208e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-8.2183441558441553e-05*w46*Jinv11+5.2049512987012989e-04*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+2.4655032467532466e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38-7.1225649350649346e-04*w52*Jinv11-3.5612824675324673e-04*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+5.2049512987012989e-04*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+1.9176136363636362e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14-1.0957792207792208e-04*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+2.4655032467532466e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+1.9176136363636362e-04*Jinv01*w54+1.9176136363636362e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.3697240259740259e-04*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30-3.5612824675324673e-04*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-1.3697240259740259e-04*w10*Jinv11)*D;
  A[48][25]=(0.0)*D;
  A[48][26]=(0.0)*D;
  A[48][27]=(2.1915584415584417e-04*Jinv21*w44-4.2918019480519480e-04*Jinv01*w3+6.4935064935064935e-05*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+2.1915584415584417e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+3.4699675324675324e-04*w16*Jinv11+3.4699675324675324e-04*Jinv21*w17-2.6481331168831169e-04*Jinv01*w45+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+2.1915584415584417e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41-1.4204545454545455e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11+3.4699675324675324e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+4.3831168831168834e-04*Jinv01*w39-4.2918019480519480e-04*Jinv21*w38-5.4788961038961042e-05*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53-4.2918019480519480e-04*w4*Jinv11-5.8441558441558442e-04*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-5.8441558441558442e-04*w31*Jinv11-2.6481331168831169e-04*w46*Jinv11-4.2918019480519480e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-2.7394480519480521e-05*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-2.1915584415584417e-04*w6*Jinv01-2.6481331168831169e-04*w47*Jinv21+7.4878246753246752e-04*w52*Jinv11-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-2.1915584415584417e-04*w7*Jinv11+3.6525974025974026e-05*Jinv21*w56+1.4975649350649350e-03*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+1.1870941558441559e-04*Jinv01*w12-2.1915584415584417e-04*Jinv21*w8-1.0957792207792208e-04*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11+1.0247564935064935e-04*Jinv01*w9+4.3831168831168834e-04*w40*Jinv11-5.8441558441558442e-04*Jinv01*w30-5.4788961038961042e-05*w25*Jinv11+1.4975649350649350e-03*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27+1.4975649350649350e-03*w48*Jinv01+1.0247564935064935e-04*w10*Jinv11)*D;
  A[48][28]=(0.0)*D;
  A[48][29]=(0.0)*D;
  A[48][30]=(1.2327516233766234e-03*Jinv00*w12+1.5523538961038962e-04*w28*Jinv10-1.4793019480519481e-03*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+6.2094155844155847e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10-1.4793019480519481e-03*Jinv02*w39+8.2183441558441553e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.0957792207792207e-03*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12-4.0178571428571428e-04*w0*Jinv02+1.4610389610389610e-04*Jinv21*w2+8.2183441558441553e-04*w43*Jinv10+8.2183441558441553e-04*w42*Jinv01+1.5523538961038962e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02-2.7394480519480518e-04*w10*Jinv12-3.7804383116883116e-03*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24-1.4793019480519481e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26+1.4610389610389610e-04*w0*Jinv01+1.4610389610389610e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20+8.2183441558441553e-04*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+1.5523538961038962e-04*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22+1.4610389610389610e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+1.5523538961038962e-04*w28*Jinv11-3.7804383116883116e-03*Jinv20*w50+1.5523538961038962e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33+9.5880681818181818e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39-4.0178571428571428e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00-1.4793019480519481e-03*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+1.3149350649350649e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56-1.4793019480519481e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+1.5523538961038962e-04*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-8.2183441558441553e-05*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12+9.5880681818181818e-04*w21*Jinv02-3.7804383116883116e-03*w49*Jinv10+1.5523538961038962e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-8.2183441558441553e-05*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+9.5880681818181818e-04*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53+9.5880681818181818e-04*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-2.6298701298701297e-03*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11+9.5880681818181818e-04*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-2.6298701298701297e-03*Jinv12*w49-8.2183441558441553e-05*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11+1.5523538961038962e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27+9.5880681818181818e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-3.7804383116883116e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+1.5523538961038962e-04*w10*Jinv10+8.2183441558441553e-04*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59-1.4793019480519481e-03*Jinv02*w33+1.4610389610389610e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10+1.4610389610389610e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00-2.6298701298701297e-03*w48*Jinv02+1.5523538961038962e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11+4.3831168831168834e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10+8.2183441558441553e-04*w42*Jinv00-8.2183441558441553e-05*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-3.7804383116883116e-03*Jinv21*w50+1.5523538961038962e-04*Jinv01*w27-3.7804383116883116e-03*w48*Jinv01-4.0178571428571428e-04*Jinv12*w1+1.5523538961038962e-04*w10*Jinv11)*D;
  A[48][31]=(0.0)*D;
  A[48][32]=(0.0)*D;
  A[48][33]=(-1.7532467532467534e-03*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10-2.9586038961038961e-03*w40*Jinv12-2.0819805194805196e-03*Jinv22*w53+6.0267857142857139e-04*Jinv20*w8-4.3831168831168834e-04*w57*Jinv02-1.7532467532467534e-03*w13*Jinv10-2.9586038961038961e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-2.7942370129870131e-03*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-1.0409902597402598e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-5.6615259740259744e-04*w10*Jinv12+8.5470779220779228e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01+1.6436688311688311e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27-5.6615259740259744e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11+8.5470779220779228e-03*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33+1.2053571428571428e-03*w22*Jinv12-2.7942370129870131e-03*w31*Jinv10-1.8628246753246754e-03*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-9.8620129870129864e-04*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+3.2873376623376621e-04*Jinv00*w18-4.9310064935064932e-04*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.2601461038961039e-03*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-4.3831168831168834e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-5.9172077922077923e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-9.3141233766233771e-04*Jinv12*w13+3.2873376623376621e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+6.0267857142857139e-04*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-2.9586038961038961e-03*Jinv22*w41+5.4788961038961042e-05*w11*Jinv20-2.0819805194805196e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-1.7532467532467534e-03*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-1.0409902597402598e-03*Jinv02*w54-1.8628246753246754e-03*w4*Jinv12+1.2053571428571428e-03*w21*Jinv02+8.5470779220779228e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-9.3141233766233771e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01+6.0267857142857139e-04*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-4.9310064935064932e-04*Jinv20*w38+1.2053571428571428e-03*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+1.2601461038961039e-03*w46*Jinv12+6.0267857142857139e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-9.8620129870129864e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+6.0267857142857139e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51-9.8620129870129864e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+7.6704545454545450e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-4.9310064935064932e-04*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56-6.9399350649350649e-04*Jinv02*w27+6.0267857142857139e-04*w6*Jinv02+3.2873376623376621e-04*w19*Jinv12-1.8628246753246754e-03*Jinv22*w5+1.2601461038961039e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-4.3831168831168834e-04*w58*Jinv12+3.2873376623376621e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33+1.6436688311688311e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+1.6436688311688311e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00-9.8620129870129864e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-2.7942370129870131e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-6.9399350649350649e-04*Jinv22*w29-9.3141233766233771e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-2.0819805194805196e-03*Jinv02*w51-1.0409902597402598e-03*Jinv22*w56-6.9399350649350649e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.3149350649350649e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[48][34]=(0.0)*D;
  A[48][35]=(0.0)*D;
  A[48][36]=(5.2049512987012989e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8+9.1314935064935065e-06*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+1.3697240259740259e-04*w3*Jinv00+1.4793019480519481e-03*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+2.7394480519480518e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-8.2183441558441553e-04*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-4.7666396103896100e-03*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35-3.1047077922077924e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-3.8352272727272725e-04*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-4.7666396103896100e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17-1.6436688311688311e-03*Jinv00*w33+8.2183441558441553e-05*w22*Jinv12+1.4793019480519481e-03*w31*Jinv10+5.2049512987012989e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+5.2049512987012989e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18+5.7528409090909093e-04*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45+9.1314935064935065e-06*Jinv22*w59-2.4655032467532466e-04*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10-2.6298701298701297e-03*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10-1.6436688311688311e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12+2.7394480519480518e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12+8.2183441558441553e-05*w21*Jinv02-4.7666396103896100e-03*w49*Jinv10-2.6298701298701297e-03*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10+5.7528409090909093e-04*Jinv20*w38+8.2183441558441553e-05*Jinv22*w23+5.2049512987012989e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-7.1225649350649346e-04*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7+5.2049512987012989e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-2.4655032467532466e-04*w52*Jinv10-2.1002435064935065e-04*w9*Jinv00+5.7528409090909093e-04*w37*Jinv10+2.4655032467532466e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22-2.1002435064935065e-04*w10*Jinv10-8.2183441558441553e-04*Jinv20*w44+9.1314935064935065e-06*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33-3.1047077922077924e-04*w0*Jinv00-1.6436688311688311e-03*w35*Jinv20+2.4655032467532466e-04*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00-7.1225649350649346e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+1.4793019480519481e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-1.6436688311688311e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20-1.6436688311688311e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-8.2183441558441553e-04*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[48][37]=(0.0)*D;
  A[48][38]=(0.0)*D;
  A[48][39]=(7.6704545454545450e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12-1.0409902597402598e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-4.3831168831168834e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20-1.7532467532467534e-03*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23+1.2601461038961039e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59+3.2873376623376621e-04*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+1.6436688311688311e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+2.7394480519480518e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17-4.9310064935064932e-04*Jinv01*w45-2.0819805194805196e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-6.9399350649350649e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00+3.2873376623376621e-04*Jinv02*w24-2.9586038961038961e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26+1.6436688311688311e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20+2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27-6.9399350649350649e-04*w11*Jinv22+1.6436688311688311e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+6.0267857142857139e-04*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-9.3141233766233771e-04*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+9.8620129870129864e-04*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00-1.3149350649350649e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+3.2873376623376621e-04*Jinv01*w24-4.3831168831168834e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-2.9586038961038961e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-1.7532467532467534e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.8628246753246754e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-2.7942370129870131e-03*w32*Jinv21+5.4788961038961042e-05*w6*Jinv00+1.2601461038961039e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20-1.0409902597402598e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.7942370129870131e-03*w31*Jinv11+3.2873376623376621e-04*Jinv22*w26-4.9310064935064932e-04*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14-1.7532467532467534e-03*w5*Jinv21-2.0819805194805196e-03*Jinv02*w54-9.3141233766233771e-04*w4*Jinv12+6.0267857142857139e-04*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-1.8628246753246754e-03*Jinv02*w12-1.3149350649350649e-03*w37*Jinv11-6.9399350649350649e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-4.9310064935064932e-04*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+6.0267857142857139e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+1.3697240259740260e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+1.2053571428571428e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10-9.8620129870129864e-04*Jinv22*w50+6.0267857142857139e-04*w22*Jinv11+1.2053571428571428e-03*Jinv12*w7+6.0267857142857139e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-9.8620129870129864e-04*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00-5.6615259740259744e-04*Jinv02*w27+1.2053571428571428e-03*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12-9.3141233766233771e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22+8.5470779220779228e-03*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.3831168831168834e-04*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-2.9586038961038961e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+6.0267857142857139e-04*Jinv21*w23+3.8352272727272725e-04*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-9.8620129870129864e-04*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-5.6615259740259744e-04*Jinv22*w29-1.8628246753246754e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11+1.2601461038961039e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20-2.7942370129870131e-03*Jinv01*w30-1.0409902597402598e-03*Jinv02*w51-2.0819805194805196e-03*Jinv22*w56-5.6615259740259744e-04*Jinv12*w28+3.2873376623376621e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+8.5470779220779228e-03*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+8.5470779220779228e-03*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[48][40]=(0.0)*D;
  A[48][41]=(0.0)*D;
  A[48][42]=(-7.6704545454545450e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+1.0957792207792207e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-7.6704545454545450e-04*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.0957792207792207e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+4.2735389610389614e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-3.2873376623376621e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-3.6525974025974026e-05*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+2.1367694805194807e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-7.6704545454545450e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-2.1367694805194807e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+1.0957792207792207e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*Jinv20*w14-7.6704545454545450e-04*w5*Jinv21+1.0957792207792207e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-1.3149350649350649e-03*w47*Jinv21-1.3149350649350649e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-5.4788961038961042e-05*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+2.1367694805194807e-03*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-2.1367694805194807e-03*Jinv01*w30+1.0957792207792207e-03*Jinv02*w51+1.0957792207792207e-03*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11+4.2735389610389614e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-3.2873376623376621e-03*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01-3.6525974025974026e-05*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[48][43]=(0.0)*D;
  A[48][44]=(0.0)*D;
  A[48][45]=(-7.6704545454545450e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+9.1314935064935065e-06*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-8.2183441558441553e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-8.2183441558441553e-04*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17+5.7528409090909093e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-5.4788961038961042e-05*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-3.5612824675324673e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01+2.7394480519480518e-04*Jinv22*w20-8.2183441558441553e-04*w43*Jinv11-1.6436688311688311e-03*Jinv21*w41-3.8352272727272725e-04*w42*Jinv02-1.0957792207792208e-04*w11*Jinv22-3.1047077922077924e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11-5.4788961038961042e-05*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w32*Jinv22-1.6436688311688311e-03*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24+9.1314935064935065e-06*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.2049512987012989e-04*Jinv12*w13+1.4793019480519481e-03*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+1.4793019480519481e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+5.7528409090909093e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+5.2049512987012989e-04*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01+5.7528409090909093e-04*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+5.2049512987012989e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+8.2183441558441553e-05*Jinv22*w8-7.1225649350649346e-04*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+8.2183441558441553e-05*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+5.2049512987012989e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11-2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+8.2183441558441553e-05*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-4.7666396103896100e-03*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14+9.1314935064935065e-06*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+1.3697240259740259e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-7.1225649350649346e-04*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-1.6436688311688311e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+1.4793019480519481e-03*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11-5.4788961038961042e-05*w16*Jinv12-4.7666396103896100e-03*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27-4.7666396103896100e-03*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[48][46]=(0.0)*D;
  A[48][47]=(0.0)*D;
  A[48][48]=(1.6436688311688311e-04*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10+4.9310064935064932e-04*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-1.1870941558441558e-03*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+4.9310064935064932e-04*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+2.6298701298701297e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-3.6982548701298703e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59-1.2784090909090910e-04*w0*Jinv02-7.8530844155844158e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.3833198051948050e-03*Jinv01*w45-1.2327516233766234e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-7.3965097402597406e-03*w48*Jinv00+4.9310064935064932e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26-7.8530844155844158e-04*w0*Jinv01-7.8530844155844158e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-4.2735389610389614e-03*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27-7.8530844155844158e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-7.3965097402597406e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+2.6298701298701297e-03*w31*Jinv10+7.3965097402597403e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-4.2735389610389614e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+2.3833198051948050e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-1.1870941558441558e-03*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-3.6982548701298703e-03*Jinv21*w53-3.6982548701298703e-03*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+7.3965097402597403e-04*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-4.2735389610389614e-03*w34*Jinv10+2.6298701298701297e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-3.6982548701298703e-03*Jinv20*w56+4.9310064935064932e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-1.2327516233766234e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+2.6298701298701297e-03*w31*Jinv11+2.3833198051948050e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-1.2327516233766234e-03*Jinv02*w54+7.3965097402597403e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-7.3965097402597406e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+7.3965097402597403e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+2.3833198051948050e-03*w47*Jinv21+2.3833198051948050e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-3.6982548701298703e-03*w52*Jinv11+1.3149350649350649e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-3.6982548701298703e-03*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+7.1225649350649346e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01-3.6982548701298703e-03*Jinv01*w51+1.3149350649350649e-03*Jinv02*w30+2.3833198051948050e-03*w36*Jinv01-3.6982548701298703e-03*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+2.3833198051948050e-03*w37*Jinv10-3.6982548701298703e-03*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+7.3965097402597403e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-7.3965097402597406e-03*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.1870941558441558e-03*w58*Jinv12+1.6436688311688311e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-7.8530844155844158e-04*w0*Jinv00-4.2735389610389614e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23-3.6982548701298703e-03*Jinv01*w54-3.6982548701298703e-03*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-7.8530844155844158e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-7.4878246753246752e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.6298701298701297e-03*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+7.3965097402597403e-04*Jinv22*w14-4.2735389610389614e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+2.6298701298701297e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56+3.5612824675324673e-04*w25*Jinv11-3.6982548701298703e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12-7.3965097402597406e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27-7.3965097402597406e-03*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[48][49]=(0.0)*D;
  A[48][50]=(0.0)*D;
  A[48][51]=(-1.6436688311688311e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-7.1225649350649346e-04*Jinv20*w8-4.5657467532467534e-05*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-1.6436688311688311e-04*w3*Jinv00-2.6298701298701297e-03*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-1.2327516233766234e-03*Jinv22*w38+3.6982548701298703e-03*Jinv00*w54-3.5612824675324673e-04*w25*Jinv12+3.9265422077922079e-04*w0*Jinv02+2.1367694805194807e-03*w43*Jinv10+1.2327516233766234e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+7.3965097402597406e-03*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+3.1777597402597403e-03*Jinv22*w35+7.8530844155844158e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20-2.3833198051948050e-03*w45*Jinv00+1.5888798701298701e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+7.3965097402597406e-03*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10+1.0957792207792207e-03*Jinv22*w17+4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.6298701298701297e-03*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22+3.9265422077922079e-04*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-2.3833198051948050e-03*w36*Jinv00-1.0135957792207792e-03*Jinv02*w45-4.5657467532467534e-05*Jinv22*w59+3.6982548701298703e-03*w51*Jinv00+3.1777597402597403e-03*Jinv12*w34+1.5888798701298701e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv10+4.2735389610389614e-03*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10+4.2735389610389614e-03*w34*Jinv10-7.1225649350649346e-04*w6*Jinv00-1.2327516233766234e-03*w37*Jinv12+3.6982548701298703e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+2.4655032467532467e-03*w52*Jinv12-3.5612824675324673e-04*Jinv22*w26-1.6436688311688311e-04*Jinv20*w14+1.2327516233766234e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+7.3965097402597406e-03*w49*Jinv10+4.2735389610389614e-03*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12+3.9265422077922079e-04*Jinv02*w9-7.1225649350649346e-04*w7*Jinv10-2.3833198051948050e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-1.2327516233766234e-03*w31*Jinv12-1.0135957792207792e-03*w46*Jinv12+3.6982548701298703e-03*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+2.4655032467532467e-03*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-1.2327516233766234e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49+3.6982548701298703e-03*w52*Jinv10+7.4878246753246752e-04*w9*Jinv00-2.3833198051948050e-03*w37*Jinv10-3.5612824675324673e-04*w24*Jinv00+3.7439123376623376e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-1.0135957792207792e-03*w47*Jinv22+7.4878246753246752e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44-4.5657467532467534e-05*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+3.1777597402597403e-03*Jinv02*w33+7.8530844155844158e-04*w0*Jinv00+4.2735389610389614e-03*w35*Jinv20-3.5612824675324673e-04*w25*Jinv10+7.8530844155844158e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+2.4655032467532467e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.6298701298701297e-03*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20+3.7439123376623376e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14-1.2327516233766234e-03*Jinv02*w36+1.5888798701298701e-03*Jinv12*w43-1.6436688311688311e-04*w5*Jinv20+2.4655032467532467e-03*Jinv02*w51+1.2327516233766234e-03*Jinv22*w56+3.7439123376623376e-04*Jinv12*w28+3.6982548701298703e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+1.0957792207792207e-03*w16*Jinv12+3.9265422077922079e-04*Jinv12*w1)*D;
  A[48][52]=(0.0)*D;
  A[48][53]=(0.0)*D;
  A[48][54]=(3.1777597402597403e-03*w40*Jinv12+1.2327516233766234e-03*Jinv22*w53-4.5657467532467534e-05*w57*Jinv02+3.1777597402597403e-03*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-1.6436688311688311e-04*Jinv01*w3-1.0135957792207792e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+3.9265422077922079e-04*w0*Jinv02+7.8530844155844158e-04*Jinv21*w2+2.1367694805194807e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.3833198051948050e-03*Jinv01*w45+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w15*Jinv02+3.7439123376623376e-04*w10*Jinv12-8.2183441558441553e-05*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+7.8530844155844158e-04*w0*Jinv01-3.5612824675324673e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11+4.2735389610389614e-03*Jinv21*w41+1.5888798701298701e-03*w42*Jinv02+3.7439123376623376e-04*w11*Jinv22+7.8530844155844158e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11+1.0957792207792207e-03*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-8.2183441558441553e-05*Jinv02*w3-1.2327516233766234e-03*w32*Jinv22+4.2735389610389614e-03*Jinv01*w39+3.9265422077922079e-04*Jinv22*w2-2.3833198051948050e-03*Jinv21*w38-1.2327516233766234e-03*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-4.5657467532467534e-05*Jinv22*w59+4.2735389610389614e-03*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+3.6982548701298703e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+1.5888798701298701e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-2.6298701298701297e-03*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-1.0135957792207792e-03*w37*Jinv12+3.1777597402597403e-03*Jinv22*w41+1.2327516233766234e-03*w52*Jinv12+4.2735389610389614e-03*Jinv21*w35-2.6298701298701297e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26-2.3833198051948050e-03*w46*Jinv11-1.6436688311688311e-04*w5*Jinv21+2.4655032467532467e-03*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11+3.7439123376623376e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-2.3833198051948050e-03*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+3.6982548701298703e-03*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.4655032467532467e-03*Jinv22*w50-7.1225649350649346e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-7.1225649350649346e-04*w21*Jinv01+3.6982548701298703e-03*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49-2.3833198051948050e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+3.6982548701298703e-03*Jinv21*w56+3.9265422077922079e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+7.3965097402597406e-03*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14-4.5657467532467534e-05*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-1.6436688311688311e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-7.1225649350649346e-04*Jinv21*w23+3.6982548701298703e-03*Jinv01*w54+3.6982548701298703e-03*w55*Jinv11+2.4655032467532467e-03*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+3.9265422077922079e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14+4.2735389610389614e-03*w40*Jinv11-1.0135957792207792e-03*Jinv02*w36+1.5888798701298701e-03*Jinv12*w43-2.6298701298701297e-03*Jinv01*w30+1.2327516233766234e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11+1.0957792207792207e-03*w16*Jinv12+7.3965097402597406e-03*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27+7.3965097402597406e-03*w48*Jinv01+3.9265422077922079e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[48][55]=(0.0)*D;
  A[48][56]=(0.0)*D;
  A[48][57]=(6.5746753246753243e-04*w40*Jinv12+1.1870941558441558e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-6.7573051948051951e-04*Jinv22*w38-5.4788961038961042e-05*w25*Jinv12+2.0495129870129870e-04*w0*Jinv02+1.1870941558441558e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12-5.4788961038961042e-05*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-5.4788961038961042e-05*Jinv22*w20+3.2873376623376621e-04*w42*Jinv02+1.5726461038961039e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-1.0957792207792208e-04*w22*Jinv12-2.6481331168831169e-04*Jinv02*w3-1.0227272727272728e-03*w32*Jinv22+2.0495129870129870e-04*Jinv22*w2-6.7573051948051951e-04*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+6.5746753246753243e-04*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44-2.6481331168831169e-04*Jinv12*w13-6.7573051948051951e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41+1.1870941558441558e-03*w52*Jinv12-5.4788961038961042e-05*Jinv22*w26+1.1870941558441558e-03*Jinv02*w54-2.6481331168831169e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-2.6481331168831169e-04*Jinv02*w12+1.5726461038961039e-04*Jinv02*w9-1.0957792207792208e-04*Jinv22*w23-1.0227272727272728e-03*w31*Jinv12-6.7573051948051951e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+2.3741883116883116e-03*Jinv22*w50-1.0957792207792208e-04*Jinv12*w7-1.0227272727272728e-03*Jinv02*w30+2.3741883116883116e-03*Jinv12*w49+1.5726461038961039e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-2.6481331168831169e-04*Jinv22*w5-6.7573051948051951e-04*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+6.5746753246753243e-04*Jinv02*w33+2.3741883116883116e-03*w48*Jinv02+1.5726461038961039e-04*Jinv22*w29-2.6481331168831169e-04*Jinv22*w14-6.7573051948051951e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.1870941558441558e-03*Jinv02*w51+1.1870941558441558e-03*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+2.0495129870129870e-04*Jinv12*w1)*D;
  A[48][58]=(0.0)*D;
  A[48][59]=(0.0)*D;
  A[49][0]=(0.0)*D;
  A[49][1]=(6.7573051948051951e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12-3.6525974025974026e-05*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8-8.3198051948051951e-05*w57*Jinv02+6.7573051948051951e-04*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+6.7573051948051951e-04*w3*Jinv00+1.0227272727272728e-03*w32*Jinv20+6.7573051948051951e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-1.1870941558441559e-04*Jinv22*w38-3.6525974025974026e-05*Jinv00*w54-8.3198051948051951e-05*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21-2.9220779220779221e-04*w16*Jinv11-2.9220779220779221e-04*Jinv21*w17-1.1870941558441559e-04*Jinv01*w45-3.6525974025974026e-05*Jinv12*w55-8.3198051948051951e-05*w58*Jinv10-2.9220779220779221e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12+2.5568181818181820e-04*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35-8.3198051948051951e-05*w58*Jinv11-8.3198051948051951e-05*w57*Jinv01+2.7394480519480521e-05*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10-2.9220779220779221e-04*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-1.1870941558441559e-04*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+6.7573051948051951e-04*w13*Jinv11-2.9220779220779221e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+2.5568181818181820e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29-1.1870941558441559e-04*w46*Jinv10-2.9220779220779221e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+1.0227272727272728e-03*w31*Jinv10+6.7573051948051951e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+1.0227272727272728e-03*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2+2.7394480519480521e-05*Jinv00*w18-1.1870941558441559e-04*w36*Jinv00-1.1870941558441559e-04*Jinv21*w38-1.1870941558441559e-04*Jinv02*w45+2.7394480519480521e-05*Jinv01*w24-8.3198051948051951e-05*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11+2.7394480519480521e-05*w19*Jinv11-3.6525974025974026e-05*Jinv21*w53-3.6525974025974026e-05*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+6.7573051948051951e-04*w4*Jinv10+6.7573051948051951e-04*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+6.7573051948051951e-04*Jinv12*w13+2.7394480519480521e-05*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+1.0227272727272728e-03*w32*Jinv21+2.7394480519480521e-05*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00-1.1870941558441559e-04*w37*Jinv12-3.6525974025974026e-05*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41+2.7394480519480521e-05*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20-3.6525974025974026e-05*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35+1.0227272727272728e-03*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-1.1870941558441559e-04*w46*Jinv11+6.7573051948051951e-04*Jinv20*w14+6.7573051948051951e-04*w5*Jinv21-3.6525974025974026e-05*Jinv02*w54+6.7573051948051951e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.5568181818181820e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+6.7573051948051951e-04*Jinv02*w12+2.7394480519480521e-05*Jinv01*w18-1.1870941558441559e-04*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-1.1870941558441559e-04*w47*Jinv21-1.1870941558441559e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-3.6525974025974026e-05*w52*Jinv11+1.0227272727272728e-03*w31*Jinv12-1.1870941558441559e-04*w46*Jinv12-3.6525974025974026e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10+2.5568181818181820e-04*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01-3.6525974025974026e-05*Jinv01*w51+1.0227272727272728e-03*Jinv02*w30+2.5568181818181820e-04*Jinv12*w49-1.1870941558441559e-04*w36*Jinv01-3.6525974025974026e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00-1.1870941558441559e-04*w37*Jinv10-3.6525974025974026e-05*Jinv21*w56+2.7394480519480521e-05*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12+6.7573051948051951e-04*Jinv22*w5-1.1870941558441559e-04*w47*Jinv22+2.5568181818181820e-04*w49*Jinv11+6.7573051948051951e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-8.3198051948051951e-05*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18+6.7573051948051951e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-8.3198051948051951e-05*w57*Jinv00-8.3198051948051951e-05*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23-3.6525974025974026e-05*Jinv01*w54-3.6525974025974026e-05*w55*Jinv11+2.7394480519480521e-05*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2-2.9220779220779221e-04*w15*Jinv00+2.5568181818181820e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9-2.9220779220779221e-04*Jinv20*w17+1.0227272727272728e-03*w30*Jinv00+3.2873376623376621e-04*w40*Jinv10+2.7394480519480521e-05*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+6.7573051948051951e-04*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11-1.1870941558441559e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+6.7573051948051951e-04*w5*Jinv20+1.0227272727272728e-03*Jinv01*w30-3.6525974025974026e-05*Jinv02*w51-3.6525974025974026e-05*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28+2.7394480519480521e-05*w25*Jinv11-3.6525974025974026e-05*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-1.1870941558441559e-04*w47*Jinv20-2.9220779220779221e-04*w16*Jinv12+2.5568181818181820e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27+2.5568181818181820e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[49][2]=(0.0)*D;
  A[49][3]=(0.0)*D;
  A[49][4]=(-1.3697240259740260e-03*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+9.5880681818181818e-04*Jinv20*w8+4.8396915584415583e-04*w57*Jinv02-1.3697240259740260e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20+2.0271915584415584e-03*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23-1.3697240259740259e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+4.8396915584415583e-04*Jinv21*w59-4.6570616883116886e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+3.2873376623376621e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+2.5202922077922079e-03*w16*Jinv11+2.5202922077922079e-03*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-1.9176136363636362e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10+2.5202922077922079e-03*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12-3.2873376623376621e-04*w48*Jinv00-4.6570616883116886e-04*Jinv02*w24+2.7394480519480520e-03*Jinv22*w35+4.8396915584415583e-04*w58*Jinv11+4.8396915584415583e-04*w57*Jinv01-4.6570616883116886e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-6.6659902597402602e-04*w1*Jinv10+1.2053571428571428e-03*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11+2.7394480519480518e-04*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.0135957792207792e-03*w13*Jinv11+2.5202922077922079e-03*w15*Jinv01+3.1960227272727273e-04*w28*Jinv11-3.2873376623376621e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10+2.5202922077922079e-03*Jinv22*w17+6.0267857142857139e-04*Jinv00*w33-8.4922889610389610e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10+2.0271915584415584e-03*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+2.7394480519480521e-05*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00-1.3697240259740259e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-4.6570616883116886e-04*Jinv01*w24+4.8396915584415583e-04*Jinv22*w59+2.7394480519480520e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11-3.8352272727272725e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+2.7394480519480520e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+2.0271915584415584e-03*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+1.0135957792207792e-03*Jinv12*w13+2.7394480519480521e-05*w19*Jinv10+6.0267857142857139e-04*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20+9.5880681818181818e-04*w6*Jinv00-1.3697240259740259e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41-1.9176136363636362e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-3.8352272727272725e-04*w52*Jinv12+2.7394480519480520e-03*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-4.6570616883116886e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11-1.3697240259740260e-03*Jinv20*w14+2.0271915584415584e-03*w5*Jinv21-1.9176136363636362e-04*Jinv02*w54+2.0271915584415584e-03*w4*Jinv12-8.4922889610389610e-04*w21*Jinv02-3.2873376623376621e-04*w49*Jinv10+3.1960227272727273e-04*Jinv21*w29+2.7394480519480520e-03*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+1.0135957792207792e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-1.3697240259740259e-04*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01+9.5880681818181818e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-8.4922889610389610e-04*Jinv22*w23-3.8352272727272725e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50-8.4922889610389610e-04*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-8.4922889610389610e-04*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49-1.3697240259740259e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.9176136363636362e-04*Jinv21*w56-1.9176136363636362e-04*w24*Jinv00+3.1960227272727273e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+2.0271915584415584e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-1.4793019480519481e-03*w49*Jinv11+1.0135957792207792e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+4.8396915584415583e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+1.0135957792207792e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59+2.7394480519480520e-03*Jinv02*w33-6.6659902597402602e-04*w0*Jinv00+6.0267857142857139e-04*w35*Jinv20-8.4922889610389610e-04*Jinv21*w23-1.9176136363636362e-04*Jinv01*w54-1.9176136363636362e-04*w55*Jinv11-1.9176136363636362e-04*w25*Jinv10-6.6659902597402602e-04*Jinv20*w2+1.2053571428571428e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+1.2053571428571428e-03*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480521e-05*w20*Jinv20+3.1960227272727273e-04*Jinv22*w29+1.0135957792207792e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-1.3697240259740259e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30-3.8352272727272725e-04*Jinv02*w51-1.9176136363636362e-04*Jinv22*w56+3.1960227272727273e-04*Jinv12*w28-4.6570616883116886e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20+2.5202922077922079e-03*w16*Jinv12-1.4793019480519481e-03*Jinv21*w50+3.1960227272727273e-04*Jinv01*w27-1.4793019480519481e-03*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[49][5]=(0.0)*D;
  A[49][6]=(0.0)*D;
  A[49][7]=(1.1231737012987013e-03*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02+1.1231737012987013e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00+9.5880681818181818e-04*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2-3.2873376623376621e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-1.4245129870129869e-03*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+3.5612824675324673e-04*w1*Jinv10-1.2601461038961039e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11+2.7394480519480518e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11-1.4245129870129869e-03*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17-1.3697240259740260e-03*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+9.5880681818181818e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+1.6436688311688311e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-1.3697240259740260e-03*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.9176136363636362e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11+1.1231737012987013e-03*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-1.4245129870129869e-03*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21-8.2183441558441553e-05*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00+3.5612824675324673e-04*Jinv21*w56+1.9176136363636362e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33+3.5612824675324673e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+1.9176136363636362e-04*w25*Jinv10+3.5612824675324673e-04*Jinv20*w2-1.2601461038961039e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-1.2601461038961039e-03*Jinv20*w17+9.5880681818181818e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10+1.0957792207792208e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[49][8]=(0.0)*D;
  A[49][9]=(0.0)*D;
  A[49][10]=(-4.2918019480519480e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+1.1870941558441559e-04*w3*Jinv00-5.8441558441558442e-04*w32*Jinv20-2.1915584415584417e-04*Jinv20*w23+7.4878246753246752e-04*Jinv00*w54+2.1915584415584417e-04*w43*Jinv10+6.4935064935064935e-05*w58*Jinv10+1.4975649350649350e-03*w48*Jinv00-1.4204545454545455e-05*w1*Jinv10+3.4699675324675324e-04*w16*Jinv10-4.2918019480519480e-04*w45*Jinv00+1.0247564935064935e-04*Jinv00*w27+1.4975649350649350e-03*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+4.3831168831168834e-04*Jinv00*w33-5.8441558441558442e-04*w31*Jinv10-2.1915584415584417e-04*w21*Jinv00-5.4788961038961042e-05*Jinv00*w18-2.6481331168831169e-04*w36*Jinv00+3.6525974025974026e-05*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-5.4788961038961042e-05*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00+7.4878246753246752e-04*Jinv20*w56-2.7394480519480521e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-4.2918019480519480e-04*Jinv20*w14+1.4975649350649350e-03*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-1.0957792207792208e-04*w7*Jinv10-2.6481331168831169e-04*Jinv20*w38+3.6525974025974026e-05*Jinv20*w53-2.1915584415584417e-04*w22*Jinv10+3.6525974025974026e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-2.6481331168831169e-04*w37*Jinv10-2.7394480519480521e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+2.1915584415584417e-04*Jinv20*w44+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-2.7394480519480521e-05*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+3.4699675324675324e-04*w15*Jinv00+3.4699675324675324e-04*Jinv20*w17-5.8441558441558442e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+1.1870941558441559e-04*w5*Jinv20+7.4878246753246752e-04*w55*Jinv10+2.1915584415584417e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20)*D;
  A[49][11]=(0.0)*D;
  A[49][12]=(0.0)*D;
  A[49][13]=(2.0271915584415584e-03*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+2.7394480519480520e-03*w40*Jinv12-1.9176136363636362e-04*Jinv22*w53-8.4922889610389610e-04*Jinv20*w8+4.8396915584415583e-04*w57*Jinv02+2.0271915584415584e-03*w13*Jinv10+2.7394480519480520e-03*Jinv02*w39+3.2873376623376621e-04*Jinv21*w44+1.0135957792207792e-03*w3*Jinv00+1.2327516233766234e-03*w32*Jinv20-1.3697240259740260e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-3.8352272727272725e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-6.6659902597402602e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+3.2873376623376621e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21+1.2053571428571428e-03*w16*Jinv11+1.2053571428571428e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45-3.8352272727272725e-04*Jinv12*w55+4.8396915584415583e-04*w58*Jinv10+2.5202922077922079e-03*w15*Jinv02+3.1960227272727273e-04*w10*Jinv12-1.4793019480519481e-03*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480521e-05*Jinv21*w26-6.6659902597402602e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+2.5202922077922079e-03*w16*Jinv10-4.6570616883116886e-04*Jinv22*w20+3.2873376623376621e-04*w43*Jinv11-1.3697240259740259e-04*w45*Jinv00+6.0267857142857139e-04*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+3.1960227272727273e-04*w11*Jinv22-6.6659902597402602e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+1.2053571428571428e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11-1.4793019480519481e-03*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-1.3697240259740259e-04*w46*Jinv10+2.5202922077922079e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10+1.0135957792207792e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+6.0267857142857139e-04*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-4.6570616883116886e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00+2.7394480519480518e-04*Jinv21*w38-1.3697240259740259e-04*Jinv02*w45+2.7394480519480521e-05*Jinv01*w24+4.8396915584415583e-04*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-1.9176136363636362e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-1.9176136363636362e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44+1.0135957792207792e-03*w4*Jinv10-1.3697240259740260e-03*w4*Jinv11+2.7394480519480520e-03*w41*Jinv20+2.0271915584415584e-03*Jinv12*w13-4.6570616883116886e-04*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21-1.9176136363636362e-04*Jinv21*w20-8.4922889610389610e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-3.8352272727272725e-04*Jinv20*w56+2.7394480519480520e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+3.1960227272727273e-04*w11*Jinv20-1.9176136363636362e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11+2.0271915584415584e-03*Jinv20*w14-1.3697240259740260e-03*w5*Jinv21-3.8352272727272725e-04*Jinv02*w54+1.0135957792207792e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02-1.4793019480519481e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+2.7394480519480520e-03*w39*Jinv00+2.0271915584415584e-03*Jinv02*w12-1.9176136363636362e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11+3.1960227272727273e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01-8.4922889610389610e-04*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-1.3697240259740259e-04*w46*Jinv12-1.9176136363636362e-04*Jinv20*w53-8.4922889610389610e-04*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+9.5880681818181818e-04*w22*Jinv11-8.4922889610389610e-04*Jinv12*w7+9.5880681818181818e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01-1.9176136363636362e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11+3.1960227272727273e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27-8.4922889610389610e-04*w6*Jinv02-4.6570616883116886e-04*w19*Jinv12+1.0135957792207792e-03*Jinv22*w5-1.3697240259740259e-04*w47*Jinv22-3.2873376623376621e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+3.1960227272727273e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44+4.8396915584415583e-04*w58*Jinv12-4.6570616883116886e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.8396915584415583e-04*w57*Jinv00+4.8396915584415583e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20+9.5880681818181818e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+2.5202922077922079e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+2.5202922077922079e-03*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+2.7394480519480520e-03*w40*Jinv10-4.6570616883116886e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29+2.0271915584415584e-03*Jinv22*w14+6.0267857142857139e-04*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43+1.0135957792207792e-03*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30-1.9176136363636362e-04*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28+2.7394480519480521e-05*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-1.3697240259740259e-04*w47*Jinv20+2.5202922077922079e-03*w16*Jinv12-3.2873376623376621e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27-3.2873376623376621e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[49][14]=(0.0)*D;
  A[49][15]=(0.0)*D;
  A[49][16]=(-2.5750811688311690e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-1.0957792207792207e-03*Jinv22*w53+1.2053571428571428e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-2.5750811688311690e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-1.2601461038961039e-03*w3*Jinv00-1.7532467532467534e-03*w32*Jinv20-2.5750811688311690e-03*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2-3.6525974025974026e-05*w11*Jinv21-1.6436688311688311e-03*w16*Jinv11-1.6436688311688311e-03*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45-1.0957792207792207e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+2.1915584415584414e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+8.2183441558441553e-04*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-1.6436688311688311e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20-3.8352272727272725e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-1.2601461038961039e-03*w13*Jinv11-1.6436688311688311e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+2.1915584415584414e-03*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-1.7532467532467534e-03*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2+8.2183441558441553e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00-3.8352272727272725e-04*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+8.2183441558441553e-04*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+1.0957792207792207e-03*Jinv21*w53+1.3149350649350649e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44-1.2601461038961039e-03*w4*Jinv10-2.5750811688311690e-03*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+8.2183441558441553e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.7532467532467534e-03*w32*Jinv21+1.2053571428571428e-03*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w11*Jinv20-1.0957792207792207e-03*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35-1.7532467532467534e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+3.2873376623376621e-04*w46*Jinv11-2.5750811688311690e-03*Jinv20*w14-2.5750811688311690e-03*w5*Jinv21-1.0957792207792207e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02+2.1915584415584414e-03*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-3.8352272727272725e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.2053571428571428e-03*w7*Jinv10+3.2873376623376621e-04*w47*Jinv21+3.2873376623376621e-04*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+5.4788961038961042e-05*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-1.0957792207792207e-03*Jinv22*w50+1.2053571428571428e-03*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+1.2053571428571428e-03*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10+3.6525974025974026e-05*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+2.1915584415584414e-03*w49*Jinv11-1.2601461038961039e-03*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10+4.9310064935064932e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.2601461038961039e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+1.2053571428571428e-03*Jinv21*w23+2.5568181818181820e-04*Jinv20*w2-1.6436688311688311e-03*w15*Jinv00-1.0957792207792207e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-1.6436688311688311e-03*Jinv20*w17-1.7532467532467534e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+8.2183441558441553e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43-1.2601461038961039e-03*w5*Jinv20-1.7532467532467534e-03*Jinv01*w30-1.0957792207792207e-03*Jinv02*w51-1.0957792207792207e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+8.2183441558441553e-04*w25*Jinv11+1.0957792207792207e-03*w55*Jinv10-3.8352272727272725e-04*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12+2.1915584415584414e-03*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27+2.1915584415584414e-03*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[49][17]=(0.0)*D;
  A[49][18]=(0.0)*D;
  A[49][19]=(5.4788961038961036e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+5.4788961038961036e-04*w13*Jinv10-3.8352272727272725e-04*Jinv21*w44+2.4655032467532466e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.2049512987012989e-04*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23-7.1225649350649346e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-7.1225649350649346e-04*w43*Jinv10-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-8.7662337662337668e-04*w16*Jinv10-3.8352272727272725e-04*w43*Jinv11+5.2049512987012989e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-1.3697240259740259e-04*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10+5.4788961038961036e-04*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-3.5612824675324673e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+2.7394480519480518e-04*Jinv01*w24-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+1.9176136363636362e-04*w51*Jinv00+2.4655032467532466e-04*w4*Jinv10+5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-7.1225649350649346e-04*Jinv20*w56+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20-7.6704545454545450e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14+5.2049512987012989e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-3.5612824675324673e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11+1.9176136363636362e-04*Jinv20*w53+5.4788961038961036e-04*w22*Jinv10+8.2183441558441553e-05*w22*Jinv11+8.2183441558441553e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01+1.9176136363636362e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11+6.3920454545454550e-05*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+6.3920454545454550e-05*w10*Jinv10-7.1225649350649346e-04*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-8.7662337662337668e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-8.7662337662337668e-04*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+2.4655032467532466e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+2.7394480519480518e-04*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10-7.1225649350649346e-04*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[49][20]=(0.0)*D;
  A[49][21]=(0.0)*D;
  A[49][22]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39-3.2873376623376621e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20+1.1231737012987013e-03*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+3.5612824675324673e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10-3.2873376623376621e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-1.2601461038961039e-03*w16*Jinv11-1.2601461038961039e-03*Jinv21*w17+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+3.5612824675324673e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20-3.2873376623376621e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22+3.5612824675324673e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11-1.2601461038961039e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22-1.3697240259740260e-03*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+2.7394480519480518e-04*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-9.1314935064935065e-06*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+1.9176136363636362e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10+1.1231737012987013e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+9.5880681818181818e-04*w32*Jinv21+1.9176136363636362e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+9.5880681818181818e-04*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26-5.2049512987012989e-04*Jinv20*w14+1.1231737012987013e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+1.9176136363636362e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10-2.7394480519480518e-04*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-1.4245129870129869e-03*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14-1.3697240259740260e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20+9.5880681818181818e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-1.4245129870129869e-03*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27-1.4245129870129869e-03*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[49][23]=(0.0)*D;
  A[49][24]=(0.0)*D;
  A[49][25]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+8.2183441558441553e-05*Jinv20*w8+5.2049512987012989e-04*w13*Jinv10-7.1225649350649346e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w43*Jinv10-7.1225649350649346e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-8.7662337662337668e-04*w16*Jinv11-8.7662337662337668e-04*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-7.1225649350649346e-04*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-8.7662337662337668e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+2.7394480519480518e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+5.2049512987012989e-04*Jinv21*w38-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-1.0957792207792208e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-8.2183441558441553e-05*w46*Jinv11+5.2049512987012989e-04*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+2.4655032467532466e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38-7.1225649350649346e-04*w52*Jinv11-3.5612824675324673e-04*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+5.2049512987012989e-04*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+1.9176136363636362e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14-1.0957792207792208e-04*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+2.4655032467532466e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+1.9176136363636362e-04*Jinv01*w54+1.9176136363636362e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.3697240259740259e-04*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30-3.5612824675324673e-04*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-1.3697240259740259e-04*w10*Jinv11)*D;
  A[49][26]=(0.0)*D;
  A[49][27]=(0.0)*D;
  A[49][28]=(2.1915584415584417e-04*Jinv21*w44-4.2918019480519480e-04*Jinv01*w3+6.4935064935064935e-05*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+2.1915584415584417e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+3.4699675324675324e-04*w16*Jinv11+3.4699675324675324e-04*Jinv21*w17-2.6481331168831169e-04*Jinv01*w45+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+2.1915584415584417e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41-1.4204545454545455e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11+3.4699675324675324e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+4.3831168831168834e-04*Jinv01*w39-4.2918019480519480e-04*Jinv21*w38-5.4788961038961042e-05*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53-4.2918019480519480e-04*w4*Jinv11-5.8441558441558442e-04*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-5.8441558441558442e-04*w31*Jinv11-2.6481331168831169e-04*w46*Jinv11-4.2918019480519480e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-2.7394480519480521e-05*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-2.1915584415584417e-04*w6*Jinv01-2.6481331168831169e-04*w47*Jinv21+7.4878246753246752e-04*w52*Jinv11-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-2.1915584415584417e-04*w7*Jinv11+3.6525974025974026e-05*Jinv21*w56+1.4975649350649350e-03*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+1.1870941558441559e-04*Jinv01*w12-2.1915584415584417e-04*Jinv21*w8-1.0957792207792208e-04*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11+1.0247564935064935e-04*Jinv01*w9+4.3831168831168834e-04*w40*Jinv11-5.8441558441558442e-04*Jinv01*w30-5.4788961038961042e-05*w25*Jinv11+1.4975649350649350e-03*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27+1.4975649350649350e-03*w48*Jinv01+1.0247564935064935e-04*w10*Jinv11)*D;
  A[49][29]=(0.0)*D;
  A[49][30]=(0.0)*D;
  A[49][31]=(1.2327516233766234e-03*Jinv00*w12+1.5523538961038962e-04*w28*Jinv10-1.4793019480519481e-03*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+6.2094155844155847e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10-1.4793019480519481e-03*Jinv02*w39+8.2183441558441553e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.0957792207792207e-03*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12-4.0178571428571428e-04*w0*Jinv02+1.4610389610389610e-04*Jinv21*w2+8.2183441558441553e-04*w43*Jinv10+8.2183441558441553e-04*w42*Jinv01+1.5523538961038962e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02-2.7394480519480518e-04*w10*Jinv12-3.7804383116883116e-03*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24-1.4793019480519481e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26+1.4610389610389610e-04*w0*Jinv01+1.4610389610389610e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20+8.2183441558441553e-04*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+1.5523538961038962e-04*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22+1.4610389610389610e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+1.5523538961038962e-04*w28*Jinv11-3.7804383116883116e-03*Jinv20*w50+1.5523538961038962e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33+9.5880681818181818e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39-4.0178571428571428e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00-1.4793019480519481e-03*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+1.3149350649350649e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56-1.4793019480519481e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+1.5523538961038962e-04*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-8.2183441558441553e-05*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12+9.5880681818181818e-04*w21*Jinv02-3.7804383116883116e-03*w49*Jinv10+1.5523538961038962e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-8.2183441558441553e-05*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+9.5880681818181818e-04*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53+9.5880681818181818e-04*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-2.6298701298701297e-03*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11+9.5880681818181818e-04*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-2.6298701298701297e-03*Jinv12*w49-8.2183441558441553e-05*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11+1.5523538961038962e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27+9.5880681818181818e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-3.7804383116883116e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+1.5523538961038962e-04*w10*Jinv10+8.2183441558441553e-04*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59-1.4793019480519481e-03*Jinv02*w33+1.4610389610389610e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10+1.4610389610389610e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00-2.6298701298701297e-03*w48*Jinv02+1.5523538961038962e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11+4.3831168831168834e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10+8.2183441558441553e-04*w42*Jinv00-8.2183441558441553e-05*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-3.7804383116883116e-03*Jinv21*w50+1.5523538961038962e-04*Jinv01*w27-3.7804383116883116e-03*w48*Jinv01-4.0178571428571428e-04*Jinv12*w1+1.5523538961038962e-04*w10*Jinv11)*D;
  A[49][32]=(0.0)*D;
  A[49][33]=(0.0)*D;
  A[49][34]=(-1.7532467532467534e-03*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10-2.9586038961038961e-03*w40*Jinv12-2.0819805194805196e-03*Jinv22*w53+6.0267857142857139e-04*Jinv20*w8-4.3831168831168834e-04*w57*Jinv02-1.7532467532467534e-03*w13*Jinv10-2.9586038961038961e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-2.7942370129870131e-03*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-1.0409902597402598e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-5.6615259740259744e-04*w10*Jinv12+8.5470779220779228e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01+1.6436688311688311e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27-5.6615259740259744e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11+8.5470779220779228e-03*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33+1.2053571428571428e-03*w22*Jinv12-2.7942370129870131e-03*w31*Jinv10-1.8628246753246754e-03*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-9.8620129870129864e-04*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+3.2873376623376621e-04*Jinv00*w18-4.9310064935064932e-04*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.2601461038961039e-03*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-4.3831168831168834e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-5.9172077922077923e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-9.3141233766233771e-04*Jinv12*w13+3.2873376623376621e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+6.0267857142857139e-04*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-2.9586038961038961e-03*Jinv22*w41+5.4788961038961042e-05*w11*Jinv20-2.0819805194805196e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-1.7532467532467534e-03*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-1.0409902597402598e-03*Jinv02*w54-1.8628246753246754e-03*w4*Jinv12+1.2053571428571428e-03*w21*Jinv02+8.5470779220779228e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-9.3141233766233771e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01+6.0267857142857139e-04*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-4.9310064935064932e-04*Jinv20*w38+1.2053571428571428e-03*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+1.2601461038961039e-03*w46*Jinv12+6.0267857142857139e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-9.8620129870129864e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+6.0267857142857139e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51-9.8620129870129864e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+7.6704545454545450e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-4.9310064935064932e-04*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56-6.9399350649350649e-04*Jinv02*w27+6.0267857142857139e-04*w6*Jinv02+3.2873376623376621e-04*w19*Jinv12-1.8628246753246754e-03*Jinv22*w5+1.2601461038961039e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-4.3831168831168834e-04*w58*Jinv12+3.2873376623376621e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33+1.6436688311688311e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+1.6436688311688311e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00-9.8620129870129864e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-2.7942370129870131e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-6.9399350649350649e-04*Jinv22*w29-9.3141233766233771e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-2.0819805194805196e-03*Jinv02*w51-1.0409902597402598e-03*Jinv22*w56-6.9399350649350649e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.3149350649350649e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[49][35]=(0.0)*D;
  A[49][36]=(0.0)*D;
  A[49][37]=(5.2049512987012989e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8+9.1314935064935065e-06*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+1.3697240259740259e-04*w3*Jinv00+1.4793019480519481e-03*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+2.7394480519480518e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-8.2183441558441553e-04*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-4.7666396103896100e-03*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35-3.1047077922077924e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-3.8352272727272725e-04*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-4.7666396103896100e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17-1.6436688311688311e-03*Jinv00*w33+8.2183441558441553e-05*w22*Jinv12+1.4793019480519481e-03*w31*Jinv10+5.2049512987012989e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+5.2049512987012989e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18+5.7528409090909093e-04*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45+9.1314935064935065e-06*Jinv22*w59-2.4655032467532466e-04*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10-2.6298701298701297e-03*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10-1.6436688311688311e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12+2.7394480519480518e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12+8.2183441558441553e-05*w21*Jinv02-4.7666396103896100e-03*w49*Jinv10-2.6298701298701297e-03*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10+5.7528409090909093e-04*Jinv20*w38+8.2183441558441553e-05*Jinv22*w23+5.2049512987012989e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-7.1225649350649346e-04*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7+5.2049512987012989e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-2.4655032467532466e-04*w52*Jinv10-2.1002435064935065e-04*w9*Jinv00+5.7528409090909093e-04*w37*Jinv10+2.4655032467532466e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22-2.1002435064935065e-04*w10*Jinv10-8.2183441558441553e-04*Jinv20*w44+9.1314935064935065e-06*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33-3.1047077922077924e-04*w0*Jinv00-1.6436688311688311e-03*w35*Jinv20+2.4655032467532466e-04*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00-7.1225649350649346e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+1.4793019480519481e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-1.6436688311688311e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20-1.6436688311688311e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-8.2183441558441553e-04*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[49][38]=(0.0)*D;
  A[49][39]=(0.0)*D;
  A[49][40]=(7.6704545454545450e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12-1.0409902597402598e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-4.3831168831168834e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20-1.7532467532467534e-03*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23+1.2601461038961039e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59+3.2873376623376621e-04*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+1.6436688311688311e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+2.7394480519480518e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17-4.9310064935064932e-04*Jinv01*w45-2.0819805194805196e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-6.9399350649350649e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00+3.2873376623376621e-04*Jinv02*w24-2.9586038961038961e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26+1.6436688311688311e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20+2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27-6.9399350649350649e-04*w11*Jinv22+1.6436688311688311e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+6.0267857142857139e-04*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-9.3141233766233771e-04*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+9.8620129870129864e-04*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00-1.3149350649350649e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+3.2873376623376621e-04*Jinv01*w24-4.3831168831168834e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-2.9586038961038961e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-1.7532467532467534e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.8628246753246754e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-2.7942370129870131e-03*w32*Jinv21+5.4788961038961042e-05*w6*Jinv00+1.2601461038961039e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20-1.0409902597402598e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.7942370129870131e-03*w31*Jinv11+3.2873376623376621e-04*Jinv22*w26-4.9310064935064932e-04*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14-1.7532467532467534e-03*w5*Jinv21-2.0819805194805196e-03*Jinv02*w54-9.3141233766233771e-04*w4*Jinv12+6.0267857142857139e-04*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-1.8628246753246754e-03*Jinv02*w12-1.3149350649350649e-03*w37*Jinv11-6.9399350649350649e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-4.9310064935064932e-04*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+6.0267857142857139e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+1.3697240259740260e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+1.2053571428571428e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10-9.8620129870129864e-04*Jinv22*w50+6.0267857142857139e-04*w22*Jinv11+1.2053571428571428e-03*Jinv12*w7+6.0267857142857139e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-9.8620129870129864e-04*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00-5.6615259740259744e-04*Jinv02*w27+1.2053571428571428e-03*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12-9.3141233766233771e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22+8.5470779220779228e-03*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.3831168831168834e-04*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-2.9586038961038961e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+6.0267857142857139e-04*Jinv21*w23+3.8352272727272725e-04*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-9.8620129870129864e-04*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-5.6615259740259744e-04*Jinv22*w29-1.8628246753246754e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11+1.2601461038961039e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20-2.7942370129870131e-03*Jinv01*w30-1.0409902597402598e-03*Jinv02*w51-2.0819805194805196e-03*Jinv22*w56-5.6615259740259744e-04*Jinv12*w28+3.2873376623376621e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+8.5470779220779228e-03*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+8.5470779220779228e-03*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[49][41]=(0.0)*D;
  A[49][42]=(0.0)*D;
  A[49][43]=(-7.6704545454545450e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+1.0957792207792207e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-7.6704545454545450e-04*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.0957792207792207e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+4.2735389610389614e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-3.2873376623376621e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-3.6525974025974026e-05*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+2.1367694805194807e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-7.6704545454545450e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-2.1367694805194807e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+1.0957792207792207e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*Jinv20*w14-7.6704545454545450e-04*w5*Jinv21+1.0957792207792207e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-1.3149350649350649e-03*w47*Jinv21-1.3149350649350649e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-5.4788961038961042e-05*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+2.1367694805194807e-03*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-2.1367694805194807e-03*Jinv01*w30+1.0957792207792207e-03*Jinv02*w51+1.0957792207792207e-03*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11+4.2735389610389614e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-3.2873376623376621e-03*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01-3.6525974025974026e-05*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[49][44]=(0.0)*D;
  A[49][45]=(0.0)*D;
  A[49][46]=(-7.6704545454545450e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+9.1314935064935065e-06*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-8.2183441558441553e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-8.2183441558441553e-04*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17+5.7528409090909093e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-5.4788961038961042e-05*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-3.5612824675324673e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01+2.7394480519480518e-04*Jinv22*w20-8.2183441558441553e-04*w43*Jinv11-1.6436688311688311e-03*Jinv21*w41-3.8352272727272725e-04*w42*Jinv02-1.0957792207792208e-04*w11*Jinv22-3.1047077922077924e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11-5.4788961038961042e-05*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w32*Jinv22-1.6436688311688311e-03*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24+9.1314935064935065e-06*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.2049512987012989e-04*Jinv12*w13+1.4793019480519481e-03*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+1.4793019480519481e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+5.7528409090909093e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+5.2049512987012989e-04*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01+5.7528409090909093e-04*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+5.2049512987012989e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+8.2183441558441553e-05*Jinv22*w8-7.1225649350649346e-04*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+8.2183441558441553e-05*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+5.2049512987012989e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11-2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+8.2183441558441553e-05*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-4.7666396103896100e-03*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14+9.1314935064935065e-06*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+1.3697240259740259e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-7.1225649350649346e-04*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-1.6436688311688311e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+1.4793019480519481e-03*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11-5.4788961038961042e-05*w16*Jinv12-4.7666396103896100e-03*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27-4.7666396103896100e-03*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[49][47]=(0.0)*D;
  A[49][48]=(0.0)*D;
  A[49][49]=(1.6436688311688311e-04*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10+4.9310064935064932e-04*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-1.1870941558441558e-03*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+4.9310064935064932e-04*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+2.6298701298701297e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-3.6982548701298703e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59-1.2784090909090910e-04*w0*Jinv02-7.8530844155844158e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.3833198051948050e-03*Jinv01*w45-1.2327516233766234e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-7.3965097402597406e-03*w48*Jinv00+4.9310064935064932e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26-7.8530844155844158e-04*w0*Jinv01-7.8530844155844158e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-4.2735389610389614e-03*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27-7.8530844155844158e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-7.3965097402597406e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+2.6298701298701297e-03*w31*Jinv10+7.3965097402597403e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-4.2735389610389614e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+2.3833198051948050e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-1.1870941558441558e-03*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-3.6982548701298703e-03*Jinv21*w53-3.6982548701298703e-03*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+7.3965097402597403e-04*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-4.2735389610389614e-03*w34*Jinv10+2.6298701298701297e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-3.6982548701298703e-03*Jinv20*w56+4.9310064935064932e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-1.2327516233766234e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+2.6298701298701297e-03*w31*Jinv11+2.3833198051948050e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-1.2327516233766234e-03*Jinv02*w54+7.3965097402597403e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-7.3965097402597406e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+7.3965097402597403e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+2.3833198051948050e-03*w47*Jinv21+2.3833198051948050e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-3.6982548701298703e-03*w52*Jinv11+1.3149350649350649e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-3.6982548701298703e-03*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+7.1225649350649346e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01-3.6982548701298703e-03*Jinv01*w51+1.3149350649350649e-03*Jinv02*w30+2.3833198051948050e-03*w36*Jinv01-3.6982548701298703e-03*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+2.3833198051948050e-03*w37*Jinv10-3.6982548701298703e-03*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+7.3965097402597403e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-7.3965097402597406e-03*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.1870941558441558e-03*w58*Jinv12+1.6436688311688311e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-7.8530844155844158e-04*w0*Jinv00-4.2735389610389614e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23-3.6982548701298703e-03*Jinv01*w54-3.6982548701298703e-03*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-7.8530844155844158e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-7.4878246753246752e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.6298701298701297e-03*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+7.3965097402597403e-04*Jinv22*w14-4.2735389610389614e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+2.6298701298701297e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56+3.5612824675324673e-04*w25*Jinv11-3.6982548701298703e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12-7.3965097402597406e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27-7.3965097402597406e-03*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[49][50]=(0.0)*D;
  A[49][51]=(0.0)*D;
  A[49][52]=(-1.6436688311688311e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-7.1225649350649346e-04*Jinv20*w8-4.5657467532467534e-05*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-1.6436688311688311e-04*w3*Jinv00-2.6298701298701297e-03*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-1.2327516233766234e-03*Jinv22*w38+3.6982548701298703e-03*Jinv00*w54-3.5612824675324673e-04*w25*Jinv12+3.9265422077922079e-04*w0*Jinv02+2.1367694805194807e-03*w43*Jinv10+1.2327516233766234e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+7.3965097402597406e-03*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+3.1777597402597403e-03*Jinv22*w35+7.8530844155844158e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20-2.3833198051948050e-03*w45*Jinv00+1.5888798701298701e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+7.3965097402597406e-03*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10+1.0957792207792207e-03*Jinv22*w17+4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.6298701298701297e-03*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22+3.9265422077922079e-04*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-2.3833198051948050e-03*w36*Jinv00-1.0135957792207792e-03*Jinv02*w45-4.5657467532467534e-05*Jinv22*w59+3.6982548701298703e-03*w51*Jinv00+3.1777597402597403e-03*Jinv12*w34+1.5888798701298701e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv10+4.2735389610389614e-03*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10+4.2735389610389614e-03*w34*Jinv10-7.1225649350649346e-04*w6*Jinv00-1.2327516233766234e-03*w37*Jinv12+3.6982548701298703e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+2.4655032467532467e-03*w52*Jinv12-3.5612824675324673e-04*Jinv22*w26-1.6436688311688311e-04*Jinv20*w14+1.2327516233766234e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+7.3965097402597406e-03*w49*Jinv10+4.2735389610389614e-03*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12+3.9265422077922079e-04*Jinv02*w9-7.1225649350649346e-04*w7*Jinv10-2.3833198051948050e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-1.2327516233766234e-03*w31*Jinv12-1.0135957792207792e-03*w46*Jinv12+3.6982548701298703e-03*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+2.4655032467532467e-03*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-1.2327516233766234e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49+3.6982548701298703e-03*w52*Jinv10+7.4878246753246752e-04*w9*Jinv00-2.3833198051948050e-03*w37*Jinv10-3.5612824675324673e-04*w24*Jinv00+3.7439123376623376e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-1.0135957792207792e-03*w47*Jinv22+7.4878246753246752e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44-4.5657467532467534e-05*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+3.1777597402597403e-03*Jinv02*w33+7.8530844155844158e-04*w0*Jinv00+4.2735389610389614e-03*w35*Jinv20-3.5612824675324673e-04*w25*Jinv10+7.8530844155844158e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+2.4655032467532467e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.6298701298701297e-03*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20+3.7439123376623376e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14-1.2327516233766234e-03*Jinv02*w36+1.5888798701298701e-03*Jinv12*w43-1.6436688311688311e-04*w5*Jinv20+2.4655032467532467e-03*Jinv02*w51+1.2327516233766234e-03*Jinv22*w56+3.7439123376623376e-04*Jinv12*w28+3.6982548701298703e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+1.0957792207792207e-03*w16*Jinv12+3.9265422077922079e-04*Jinv12*w1)*D;
  A[49][53]=(0.0)*D;
  A[49][54]=(0.0)*D;
  A[49][55]=(3.1777597402597403e-03*w40*Jinv12+1.2327516233766234e-03*Jinv22*w53-4.5657467532467534e-05*w57*Jinv02+3.1777597402597403e-03*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-1.6436688311688311e-04*Jinv01*w3-1.0135957792207792e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+3.9265422077922079e-04*w0*Jinv02+7.8530844155844158e-04*Jinv21*w2+2.1367694805194807e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.3833198051948050e-03*Jinv01*w45+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w15*Jinv02+3.7439123376623376e-04*w10*Jinv12-8.2183441558441553e-05*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+7.8530844155844158e-04*w0*Jinv01-3.5612824675324673e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11+4.2735389610389614e-03*Jinv21*w41+1.5888798701298701e-03*w42*Jinv02+3.7439123376623376e-04*w11*Jinv22+7.8530844155844158e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11+1.0957792207792207e-03*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-8.2183441558441553e-05*Jinv02*w3-1.2327516233766234e-03*w32*Jinv22+4.2735389610389614e-03*Jinv01*w39+3.9265422077922079e-04*Jinv22*w2-2.3833198051948050e-03*Jinv21*w38-1.2327516233766234e-03*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-4.5657467532467534e-05*Jinv22*w59+4.2735389610389614e-03*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+3.6982548701298703e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+1.5888798701298701e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-2.6298701298701297e-03*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-1.0135957792207792e-03*w37*Jinv12+3.1777597402597403e-03*Jinv22*w41+1.2327516233766234e-03*w52*Jinv12+4.2735389610389614e-03*Jinv21*w35-2.6298701298701297e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26-2.3833198051948050e-03*w46*Jinv11-1.6436688311688311e-04*w5*Jinv21+2.4655032467532467e-03*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11+3.7439123376623376e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-2.3833198051948050e-03*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+3.6982548701298703e-03*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.4655032467532467e-03*Jinv22*w50-7.1225649350649346e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-7.1225649350649346e-04*w21*Jinv01+3.6982548701298703e-03*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49-2.3833198051948050e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+3.6982548701298703e-03*Jinv21*w56+3.9265422077922079e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+7.3965097402597406e-03*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14-4.5657467532467534e-05*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-1.6436688311688311e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-7.1225649350649346e-04*Jinv21*w23+3.6982548701298703e-03*Jinv01*w54+3.6982548701298703e-03*w55*Jinv11+2.4655032467532467e-03*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+3.9265422077922079e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14+4.2735389610389614e-03*w40*Jinv11-1.0135957792207792e-03*Jinv02*w36+1.5888798701298701e-03*Jinv12*w43-2.6298701298701297e-03*Jinv01*w30+1.2327516233766234e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11+1.0957792207792207e-03*w16*Jinv12+7.3965097402597406e-03*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27+7.3965097402597406e-03*w48*Jinv01+3.9265422077922079e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[49][56]=(0.0)*D;
  A[49][57]=(0.0)*D;
  A[49][58]=(6.5746753246753243e-04*w40*Jinv12+1.1870941558441558e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-6.7573051948051951e-04*Jinv22*w38-5.4788961038961042e-05*w25*Jinv12+2.0495129870129870e-04*w0*Jinv02+1.1870941558441558e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12-5.4788961038961042e-05*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-5.4788961038961042e-05*Jinv22*w20+3.2873376623376621e-04*w42*Jinv02+1.5726461038961039e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-1.0957792207792208e-04*w22*Jinv12-2.6481331168831169e-04*Jinv02*w3-1.0227272727272728e-03*w32*Jinv22+2.0495129870129870e-04*Jinv22*w2-6.7573051948051951e-04*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+6.5746753246753243e-04*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44-2.6481331168831169e-04*Jinv12*w13-6.7573051948051951e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41+1.1870941558441558e-03*w52*Jinv12-5.4788961038961042e-05*Jinv22*w26+1.1870941558441558e-03*Jinv02*w54-2.6481331168831169e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-2.6481331168831169e-04*Jinv02*w12+1.5726461038961039e-04*Jinv02*w9-1.0957792207792208e-04*Jinv22*w23-1.0227272727272728e-03*w31*Jinv12-6.7573051948051951e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+2.3741883116883116e-03*Jinv22*w50-1.0957792207792208e-04*Jinv12*w7-1.0227272727272728e-03*Jinv02*w30+2.3741883116883116e-03*Jinv12*w49+1.5726461038961039e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-2.6481331168831169e-04*Jinv22*w5-6.7573051948051951e-04*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+6.5746753246753243e-04*Jinv02*w33+2.3741883116883116e-03*w48*Jinv02+1.5726461038961039e-04*Jinv22*w29-2.6481331168831169e-04*Jinv22*w14-6.7573051948051951e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.1870941558441558e-03*Jinv02*w51+1.1870941558441558e-03*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+2.0495129870129870e-04*Jinv12*w1)*D;
  A[49][59]=(0.0)*D;
  A[50][0]=(0.0)*D;
  A[50][1]=(0.0)*D;
  A[50][2]=(6.7573051948051951e-04*Jinv00*w12+1.1668019480519480e-04*w28*Jinv10+3.2873376623376621e-04*w40*Jinv12-3.6525974025974026e-05*Jinv22*w53-4.9310064935064932e-04*Jinv20*w8-8.3198051948051951e-05*w57*Jinv02+6.7573051948051951e-04*w13*Jinv10+3.2873376623376621e-04*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+6.7573051948051951e-04*w3*Jinv00+1.0227272727272728e-03*w32*Jinv20+6.7573051948051951e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-1.1870941558441559e-04*Jinv22*w38-3.6525974025974026e-05*Jinv00*w54-8.3198051948051951e-05*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12+3.2467532467532468e-04*w0*Jinv02+3.2467532467532468e-04*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.1668019480519480e-04*w11*Jinv21-2.9220779220779221e-04*w16*Jinv11-2.9220779220779221e-04*Jinv21*w17-1.1870941558441559e-04*Jinv01*w45-3.6525974025974026e-05*Jinv12*w55-8.3198051948051951e-05*w58*Jinv10-2.9220779220779221e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12+2.5568181818181820e-04*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35-8.3198051948051951e-05*w58*Jinv11-8.3198051948051951e-05*w57*Jinv01+2.7394480519480521e-05*Jinv21*w26+3.2467532467532468e-04*w0*Jinv01+3.2467532467532468e-04*w1*Jinv10-2.9220779220779221e-04*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-1.1870941558441559e-04*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41-2.1915584415584417e-04*w42*Jinv02+1.1668019480519480e-04*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+3.2467532467532468e-04*Jinv11*w1+6.7573051948051951e-04*w13*Jinv11-2.9220779220779221e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+2.5568181818181820e-04*Jinv20*w50+1.1668019480519480e-04*Jinv20*w29-1.1870941558441559e-04*w46*Jinv10-2.9220779220779221e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+1.0227272727272728e-03*w31*Jinv10+6.7573051948051951e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+1.0227272727272728e-03*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39+3.2467532467532468e-04*Jinv22*w2+2.7394480519480521e-05*Jinv00*w18-1.1870941558441559e-04*w36*Jinv00-1.1870941558441559e-04*Jinv21*w38-1.1870941558441559e-04*Jinv02*w45+2.7394480519480521e-05*Jinv01*w24-8.3198051948051951e-05*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11+2.7394480519480521e-05*w19*Jinv11-3.6525974025974026e-05*Jinv21*w53-3.6525974025974026e-05*w51*Jinv00+3.2873376623376621e-04*Jinv12*w34-2.1915584415584417e-04*Jinv22*w44+6.7573051948051951e-04*w4*Jinv10+6.7573051948051951e-04*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+6.7573051948051951e-04*Jinv12*w13+2.7394480519480521e-05*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+1.0227272727272728e-03*w32*Jinv21+2.7394480519480521e-05*Jinv21*w20-4.9310064935064932e-04*w6*Jinv00-1.1870941558441559e-04*w37*Jinv12-3.6525974025974026e-05*Jinv20*w56+3.2873376623376621e-04*Jinv22*w41+2.7394480519480521e-05*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20-3.6525974025974026e-05*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35+1.0227272727272728e-03*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-1.1870941558441559e-04*w46*Jinv11+6.7573051948051951e-04*Jinv20*w14+6.7573051948051951e-04*w5*Jinv21-3.6525974025974026e-05*Jinv02*w54+6.7573051948051951e-04*w4*Jinv12-4.9310064935064932e-04*w21*Jinv02+2.5568181818181820e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+6.7573051948051951e-04*Jinv02*w12+2.7394480519480521e-05*Jinv01*w18-1.1870941558441559e-04*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-4.9310064935064932e-04*w7*Jinv10-1.1870941558441559e-04*w47*Jinv21-1.1870941558441559e-04*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-3.6525974025974026e-05*w52*Jinv11+1.0227272727272728e-03*w31*Jinv12-1.1870941558441559e-04*w46*Jinv12-3.6525974025974026e-05*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10+2.5568181818181820e-04*Jinv22*w50-4.9310064935064932e-04*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7-4.9310064935064932e-04*w21*Jinv01-3.6525974025974026e-05*Jinv01*w51+1.0227272727272728e-03*Jinv02*w30+2.5568181818181820e-04*Jinv12*w49-1.1870941558441559e-04*w36*Jinv01-3.6525974025974026e-05*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00-1.1870941558441559e-04*w37*Jinv10-3.6525974025974026e-05*Jinv21*w56+2.7394480519480521e-05*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12+6.7573051948051951e-04*Jinv22*w5-1.1870941558441559e-04*w47*Jinv22+2.5568181818181820e-04*w49*Jinv11+6.7573051948051951e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-8.3198051948051951e-05*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18+6.7573051948051951e-04*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8-8.3198051948051951e-05*w57*Jinv00-8.3198051948051951e-05*Jinv20*w59+3.2873376623376621e-04*Jinv02*w33+3.2467532467532468e-04*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-4.9310064935064932e-04*Jinv21*w23-3.6525974025974026e-05*Jinv01*w54-3.6525974025974026e-05*w55*Jinv11+2.7394480519480521e-05*w25*Jinv10+3.2467532467532468e-04*Jinv20*w2-2.9220779220779221e-04*w15*Jinv00+2.5568181818181820e-04*w48*Jinv02+1.1668019480519480e-04*Jinv01*w9-2.9220779220779221e-04*Jinv20*w17+1.0227272727272728e-03*w30*Jinv00+3.2873376623376621e-04*w40*Jinv10+2.7394480519480521e-05*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+6.7573051948051951e-04*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11-1.1870941558441559e-04*Jinv02*w36-2.1915584415584417e-04*Jinv12*w43+6.7573051948051951e-04*w5*Jinv20+1.0227272727272728e-03*Jinv01*w30-3.6525974025974026e-05*Jinv02*w51-3.6525974025974026e-05*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28+2.7394480519480521e-05*w25*Jinv11-3.6525974025974026e-05*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-1.1870941558441559e-04*w47*Jinv20-2.9220779220779221e-04*w16*Jinv12+2.5568181818181820e-04*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27+2.5568181818181820e-04*w48*Jinv01+3.2467532467532468e-04*Jinv12*w1+1.1668019480519480e-04*w10*Jinv11)*D;
  A[50][3]=(0.0)*D;
  A[50][4]=(0.0)*D;
  A[50][5]=(-1.3697240259740260e-03*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10+1.3697240259740260e-03*w40*Jinv12-3.8352272727272725e-04*Jinv22*w53+9.5880681818181818e-04*Jinv20*w8+4.8396915584415583e-04*w57*Jinv02-1.3697240259740260e-03*w13*Jinv10+1.3697240259740260e-03*Jinv02*w39+7.1225649350649346e-04*Jinv21*w44-3.5612824675324673e-04*w3*Jinv00-1.3971185064935066e-03*w32*Jinv20+2.0271915584415584e-03*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23-1.3697240259740259e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54+4.8396915584415583e-04*Jinv21*w59-4.6570616883116886e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+3.2873376623376621e-04*w43*Jinv10+7.1225649350649346e-04*w42*Jinv01+1.7349837662337662e-04*w11*Jinv21+2.5202922077922079e-03*w16*Jinv11+2.5202922077922079e-03*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45-1.9176136363636362e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10+2.5202922077922079e-03*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12-3.2873376623376621e-04*w48*Jinv00-4.6570616883116886e-04*Jinv02*w24+2.7394480519480520e-03*Jinv22*w35+4.8396915584415583e-04*w58*Jinv11+4.8396915584415583e-04*w57*Jinv01-4.6570616883116886e-04*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-6.6659902597402602e-04*w1*Jinv10+1.2053571428571428e-03*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+7.1225649350649346e-04*w43*Jinv11+2.7394480519480518e-04*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+1.0135957792207792e-03*w13*Jinv11+2.5202922077922079e-03*w15*Jinv01+3.1960227272727273e-04*w28*Jinv11-3.2873376623376621e-04*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10+2.5202922077922079e-03*Jinv22*w17+6.0267857142857139e-04*Jinv00*w33-8.4922889610389610e-04*w22*Jinv12-1.3971185064935066e-03*w31*Jinv10+2.0271915584415584e-03*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2+2.7394480519480521e-05*Jinv00*w18+3.8352272727272725e-04*w36*Jinv00-1.3697240259740259e-04*Jinv21*w38-2.4655032467532466e-04*Jinv02*w45-4.6570616883116886e-04*Jinv01*w24+4.8396915584415583e-04*Jinv22*w59+2.7394480519480520e-03*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11-3.8352272727272725e-04*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+2.7394480519480520e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+2.0271915584415584e-03*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+1.0135957792207792e-03*Jinv12*w13+2.7394480519480521e-05*w19*Jinv10+6.0267857142857139e-04*w34*Jinv10+1.2327516233766234e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20+9.5880681818181818e-04*w6*Jinv00-1.3697240259740259e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56+1.3697240259740260e-03*Jinv22*w41-1.9176136363636362e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-3.8352272727272725e-04*w52*Jinv12+2.7394480519480520e-03*Jinv21*w35+1.2327516233766234e-03*w31*Jinv11-4.6570616883116886e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11-1.3697240259740260e-03*Jinv20*w14+2.0271915584415584e-03*w5*Jinv21-1.9176136363636362e-04*Jinv02*w54+2.0271915584415584e-03*w4*Jinv12-8.4922889610389610e-04*w21*Jinv02-3.2873376623376621e-04*w49*Jinv10+3.1960227272727273e-04*Jinv21*w29+2.7394480519480520e-03*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+1.0135957792207792e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-1.3697240259740259e-04*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01+9.5880681818181818e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21+3.8352272727272725e-04*Jinv20*w38-8.4922889610389610e-04*Jinv22*w23-3.8352272727272725e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-2.4655032467532466e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53-4.3831168831168834e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50-8.4922889610389610e-04*w22*Jinv11-4.3831168831168834e-04*Jinv12*w7-8.4922889610389610e-04*w21*Jinv01-3.8352272727272725e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49-1.3697240259740259e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+3.8352272727272725e-04*w37*Jinv10-1.9176136363636362e-04*Jinv21*w56-1.9176136363636362e-04*w24*Jinv00+3.1960227272727273e-04*Jinv02*w27-4.3831168831168834e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+2.0271915584415584e-03*Jinv22*w5-2.4655032467532466e-04*w47*Jinv22-1.4793019480519481e-03*w49*Jinv11+1.0135957792207792e-03*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+3.2873376623376621e-04*Jinv20*w44+4.8396915584415583e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+1.0135957792207792e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59+2.7394480519480520e-03*Jinv02*w33-6.6659902597402602e-04*w0*Jinv00+6.0267857142857139e-04*w35*Jinv20-8.4922889610389610e-04*Jinv21*w23-1.9176136363636362e-04*Jinv01*w54-1.9176136363636362e-04*w55*Jinv11-1.9176136363636362e-04*w25*Jinv10-6.6659902597402602e-04*Jinv20*w2+1.2053571428571428e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02+1.7349837662337662e-04*Jinv01*w9+1.2053571428571428e-03*Jinv20*w17-1.3971185064935066e-03*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480521e-05*w20*Jinv20+3.1960227272727273e-04*Jinv22*w29+1.0135957792207792e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-1.3697240259740259e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+1.2327516233766234e-03*Jinv01*w30-3.8352272727272725e-04*Jinv02*w51-1.9176136363636362e-04*Jinv22*w56+3.1960227272727273e-04*Jinv12*w28-4.6570616883116886e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10+3.2873376623376621e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20+2.5202922077922079e-03*w16*Jinv12-1.4793019480519481e-03*Jinv21*w50+3.1960227272727273e-04*Jinv01*w27-1.4793019480519481e-03*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1+1.7349837662337662e-04*w10*Jinv11)*D;
  A[50][6]=(0.0)*D;
  A[50][7]=(0.0)*D;
  A[50][8]=(1.1231737012987013e-03*Jinv00*w12-1.0957792207792208e-04*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02+1.1231737012987013e-03*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+3.8352272727272725e-04*Jinv21*w44-4.3831168831168834e-04*w3*Jinv00+9.5880681818181818e-04*w32*Jinv20-5.2049512987012989e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-3.5612824675324673e-04*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-2.7394480519480518e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+1.3697240259740259e-04*Jinv21*w2-3.2873376623376621e-04*w43*Jinv10+3.8352272727272725e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-2.7394480519480518e-04*Jinv01*w45+3.5612824675324673e-04*Jinv12*w55+9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-1.4245129870129869e-03*w48*Jinv00-2.7394480519480518e-04*Jinv02*w24+7.6704545454545450e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-2.7394480519480518e-04*Jinv21*w26+1.3697240259740259e-04*w0*Jinv01+3.5612824675324673e-04*w1*Jinv10-1.2601461038961039e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20+3.8352272727272725e-04*w43*Jinv11+2.7394480519480518e-04*w45*Jinv00+5.4788961038961042e-05*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-1.0957792207792208e-04*Jinv00*w27-9.1314935064935065e-06*w11*Jinv22+1.3697240259740259e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.0957792207792208e-04*w28*Jinv11-1.4245129870129869e-03*Jinv20*w50-1.0957792207792208e-04*Jinv20*w29+2.7394480519480518e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17-1.3697240259740260e-03*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+9.5880681818181818e-04*w31*Jinv10-5.2049512987012989e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2+1.6436688311688311e-04*Jinv21*w38-2.7394480519480518e-04*Jinv02*w45-2.7394480519480518e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+7.6704545454545450e-04*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+7.6704545454545450e-04*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-4.3831168831168834e-04*w4*Jinv10-5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-1.3697240259740260e-03*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00+1.6436688311688311e-04*w37*Jinv12-3.5612824675324673e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.9176136363636362e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+7.6704545454545450e-04*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-2.7394480519480518e-04*Jinv22*w26-2.7394480519480518e-04*w46*Jinv11+1.1231737012987013e-03*Jinv20*w14-5.2049512987012989e-04*w5*Jinv21+3.5612824675324673e-04*Jinv02*w54-5.2049512987012989e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-1.4245129870129869e-03*w49*Jinv10+1.0957792207792208e-04*Jinv21*w29+7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10-2.7394480519480518e-04*w47*Jinv21-8.2183441558441553e-05*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12-2.7394480519480518e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53+7.1225649350649346e-04*Jinv22*w8+1.6436688311688311e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+7.1225649350649346e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00+3.5612824675324673e-04*Jinv21*w56+1.9176136363636362e-04*w24*Jinv00+1.0957792207792208e-04*Jinv02*w27+7.1225649350649346e-04*w6*Jinv02+3.5612824675324673e-04*w19*Jinv12-5.2049512987012989e-04*Jinv22*w5-2.7394480519480518e-04*w47*Jinv22+7.1225649350649346e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59+7.6704545454545450e-04*Jinv02*w33+3.5612824675324673e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20-8.2183441558441553e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+1.9176136363636362e-04*w25*Jinv10+3.5612824675324673e-04*Jinv20*w2-1.2601461038961039e-03*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-1.2601461038961039e-03*Jinv20*w17+9.5880681818181818e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10+1.0957792207792208e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-4.3831168831168834e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+3.5612824675324673e-04*Jinv22*w56+1.0957792207792208e-04*Jinv12*w28-2.7394480519480518e-04*w25*Jinv11-3.5612824675324673e-04*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+2.7394480519480518e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12+7.1225649350649346e-04*Jinv21*w50+1.0957792207792208e-04*Jinv01*w27+7.1225649350649346e-04*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[50][9]=(0.0)*D;
  A[50][10]=(0.0)*D;
  A[50][11]=(-4.2918019480519480e-04*Jinv00*w12+1.0247564935064935e-04*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+1.1870941558441559e-04*w3*Jinv00-5.8441558441558442e-04*w32*Jinv20-2.1915584415584417e-04*Jinv20*w23+7.4878246753246752e-04*Jinv00*w54+2.1915584415584417e-04*w43*Jinv10+6.4935064935064935e-05*w58*Jinv10+1.4975649350649350e-03*w48*Jinv00-1.4204545454545455e-05*w1*Jinv10+3.4699675324675324e-04*w16*Jinv10-4.2918019480519480e-04*w45*Jinv00+1.0247564935064935e-04*Jinv00*w27+1.4975649350649350e-03*Jinv20*w50+1.0247564935064935e-04*Jinv20*w29-4.2918019480519480e-04*w46*Jinv10+4.3831168831168834e-04*Jinv00*w33-5.8441558441558442e-04*w31*Jinv10-2.1915584415584417e-04*w21*Jinv00-5.4788961038961042e-05*Jinv00*w18-2.6481331168831169e-04*w36*Jinv00+3.6525974025974026e-05*w51*Jinv00+1.1870941558441559e-04*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-5.4788961038961042e-05*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10-1.0957792207792208e-04*w6*Jinv00+7.4878246753246752e-04*Jinv20*w56-2.7394480519480521e-05*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-4.2918019480519480e-04*Jinv20*w14+1.4975649350649350e-03*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-1.0957792207792208e-04*w7*Jinv10-2.6481331168831169e-04*Jinv20*w38+3.6525974025974026e-05*Jinv20*w53-2.1915584415584417e-04*w22*Jinv10+3.6525974025974026e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-2.6481331168831169e-04*w37*Jinv10-2.7394480519480521e-05*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+2.1915584415584417e-04*Jinv20*w44+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59-1.4204545454545455e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-2.7394480519480521e-05*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+3.4699675324675324e-04*w15*Jinv00+3.4699675324675324e-04*Jinv20*w17-5.8441558441558442e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20+1.1870941558441559e-04*w5*Jinv20+7.4878246753246752e-04*w55*Jinv10+2.1915584415584417e-04*w42*Jinv00-4.2918019480519480e-04*w47*Jinv20)*D;
  A[50][12]=(0.0)*D;
  A[50][13]=(0.0)*D;
  A[50][14]=(2.0271915584415584e-03*Jinv00*w12+1.7349837662337662e-04*w28*Jinv10+2.7394480519480520e-03*w40*Jinv12-1.9176136363636362e-04*Jinv22*w53-8.4922889610389610e-04*Jinv20*w8+4.8396915584415583e-04*w57*Jinv02+2.0271915584415584e-03*w13*Jinv10+2.7394480519480520e-03*Jinv02*w39+3.2873376623376621e-04*Jinv21*w44+1.0135957792207792e-03*w3*Jinv00+1.2327516233766234e-03*w32*Jinv20-1.3697240259740260e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23-2.4655032467532466e-04*Jinv22*w38-3.8352272727272725e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.1870941558441559e-04*w0*Jinv02-6.6659902597402602e-04*Jinv21*w2+7.1225649350649346e-04*w43*Jinv10+3.2873376623376621e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21+1.2053571428571428e-03*w16*Jinv11+1.2053571428571428e-03*Jinv21*w17+3.8352272727272725e-04*Jinv01*w45-3.8352272727272725e-04*Jinv12*w55+4.8396915584415583e-04*w58*Jinv10+2.5202922077922079e-03*w15*Jinv02+3.1960227272727273e-04*w10*Jinv12-1.4793019480519481e-03*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+1.3697240259740260e-03*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480521e-05*Jinv21*w26-6.6659902597402602e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+2.5202922077922079e-03*w16*Jinv10-4.6570616883116886e-04*Jinv22*w20+3.2873376623376621e-04*w43*Jinv11-1.3697240259740259e-04*w45*Jinv00+6.0267857142857139e-04*Jinv21*w41+7.1225649350649346e-04*w42*Jinv02+1.7349837662337662e-04*Jinv00*w27+3.1960227272727273e-04*w11*Jinv22-6.6659902597402602e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+1.2053571428571428e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11-1.4793019480519481e-03*Jinv20*w50+1.7349837662337662e-04*Jinv20*w29-1.3697240259740259e-04*w46*Jinv10+2.5202922077922079e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.3831168831168834e-04*w22*Jinv12+1.2327516233766234e-03*w31*Jinv10+1.0135957792207792e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00+1.2327516233766234e-03*w32*Jinv22+6.0267857142857139e-04*Jinv01*w39-1.1870941558441559e-04*Jinv22*w2-4.6570616883116886e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00+2.7394480519480518e-04*Jinv21*w38-1.3697240259740259e-04*Jinv02*w45+2.7394480519480521e-05*Jinv01*w24+4.8396915584415583e-04*Jinv22*w59-7.6704545454545450e-04*w34*Jinv11-1.9176136363636362e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53-1.9176136363636362e-04*w51*Jinv00+1.3697240259740260e-03*Jinv12*w34+7.1225649350649346e-04*Jinv22*w44+1.0135957792207792e-03*w4*Jinv10-1.3697240259740260e-03*w4*Jinv11+2.7394480519480520e-03*w41*Jinv20+2.0271915584415584e-03*Jinv12*w13-4.6570616883116886e-04*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10-1.3971185064935066e-03*w32*Jinv21-1.9176136363636362e-04*Jinv21*w20-8.4922889610389610e-04*w6*Jinv00-2.4655032467532466e-04*w37*Jinv12-3.8352272727272725e-04*Jinv20*w56+2.7394480519480520e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+3.1960227272727273e-04*w11*Jinv20-1.9176136363636362e-04*w52*Jinv12-7.6704545454545450e-04*Jinv21*w35-1.3971185064935066e-03*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26+3.8352272727272725e-04*w46*Jinv11+2.0271915584415584e-03*Jinv20*w14-1.3697240259740260e-03*w5*Jinv21-3.8352272727272725e-04*Jinv02*w54+1.0135957792207792e-03*w4*Jinv12-4.3831168831168834e-04*w21*Jinv02-1.4793019480519481e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33+2.7394480519480520e-03*w39*Jinv00+2.0271915584415584e-03*Jinv02*w12-1.9176136363636362e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11+3.1960227272727273e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01-8.4922889610389610e-04*w7*Jinv10+3.8352272727272725e-04*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-4.3831168831168834e-04*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11+1.2327516233766234e-03*w31*Jinv12-1.3697240259740259e-04*w46*Jinv12-1.9176136363636362e-04*Jinv20*w53-8.4922889610389610e-04*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-1.4793019480519481e-03*Jinv22*w50+9.5880681818181818e-04*w22*Jinv11-8.4922889610389610e-04*Jinv12*w7+9.5880681818181818e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51+1.2327516233766234e-03*Jinv02*w30-1.4793019480519481e-03*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01-1.9176136363636362e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11+3.1960227272727273e-04*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27-8.4922889610389610e-04*w6*Jinv02-4.6570616883116886e-04*w19*Jinv12+1.0135957792207792e-03*Jinv22*w5-1.3697240259740259e-04*w47*Jinv22-3.2873376623376621e-04*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14+3.1960227272727273e-04*w10*Jinv10+7.1225649350649346e-04*Jinv20*w44+4.8396915584415583e-04*w58*Jinv12-4.6570616883116886e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+4.8396915584415583e-04*w57*Jinv00+4.8396915584415583e-04*Jinv20*w59+1.3697240259740260e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20+9.5880681818181818e-04*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+2.5202922077922079e-03*w15*Jinv00-1.4793019480519481e-03*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+2.5202922077922079e-03*Jinv20*w17+1.2327516233766234e-03*w30*Jinv00+2.7394480519480520e-03*w40*Jinv10-4.6570616883116886e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29+2.0271915584415584e-03*Jinv22*w14+6.0267857142857139e-04*w40*Jinv11-2.4655032467532466e-04*Jinv02*w36+7.1225649350649346e-04*Jinv12*w43+1.0135957792207792e-03*w5*Jinv20-1.3971185064935066e-03*Jinv01*w30-1.9176136363636362e-04*Jinv02*w51-3.8352272727272725e-04*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28+2.7394480519480521e-05*w25*Jinv11-3.8352272727272725e-04*w55*Jinv10+7.1225649350649346e-04*w42*Jinv00-1.3697240259740259e-04*w47*Jinv20+2.5202922077922079e-03*w16*Jinv12-3.2873376623376621e-04*Jinv21*w50-2.5568181818181820e-04*Jinv01*w27-3.2873376623376621e-04*w48*Jinv01-1.1870941558441559e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[50][15]=(0.0)*D;
  A[50][16]=(0.0)*D;
  A[50][17]=(-2.5750811688311690e-03*Jinv00*w12-3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-1.0957792207792207e-03*Jinv22*w53+1.2053571428571428e-03*Jinv20*w8+4.9310064935064932e-04*w57*Jinv02-2.5750811688311690e-03*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39-1.2601461038961039e-03*w3*Jinv00-1.7532467532467534e-03*w32*Jinv20-2.5750811688311690e-03*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+3.8352272727272725e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-4.9310064935064932e-04*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12+3.6525974025974026e-05*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2-3.6525974025974026e-05*w11*Jinv21-1.6436688311688311e-03*w16*Jinv11-1.6436688311688311e-03*Jinv21*w17+3.2873376623376621e-04*Jinv01*w45-1.0957792207792207e-03*Jinv12*w55-4.9310064935064932e-04*w58*Jinv10+3.2873376623376621e-03*w15*Jinv02+3.6525974025974026e-05*w10*Jinv12+2.1915584415584414e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35-4.9310064935064932e-04*w58*Jinv11-4.9310064935064932e-04*w57*Jinv01+8.2183441558441553e-04*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-1.6436688311688311e-03*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20-3.8352272727272725e-04*w45*Jinv00-6.5746753246753243e-04*Jinv21*w41+1.3149350649350649e-03*w42*Jinv02-3.6525974025974026e-05*Jinv00*w27+3.6525974025974026e-05*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-1.2601461038961039e-03*w13*Jinv11-1.6436688311688311e-03*w15*Jinv01-3.8352272727272725e-04*w28*Jinv11+2.1915584415584414e-03*Jinv20*w50-3.6525974025974026e-05*Jinv20*w29-3.8352272727272725e-04*w46*Jinv10+3.2873376623376621e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-1.7532467532467534e-03*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+3.8352272727272725e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39+3.6525974025974026e-05*Jinv22*w2+8.2183441558441553e-04*Jinv00*w18+3.2873376623376621e-04*w36*Jinv00-3.8352272727272725e-04*Jinv21*w38+3.8352272727272725e-04*Jinv02*w45+8.2183441558441553e-04*Jinv01*w24+4.9310064935064932e-04*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+1.0957792207792207e-03*Jinv21*w53+1.3149350649350649e-03*Jinv12*w34+1.3149350649350649e-03*Jinv22*w44-1.2601461038961039e-03*w4*Jinv10-2.5750811688311690e-03*w4*Jinv11-2.6298701298701297e-03*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+8.2183441558441553e-04*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.7532467532467534e-03*w32*Jinv21+1.2053571428571428e-03*w6*Jinv00+3.8352272727272725e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w11*Jinv20-1.0957792207792207e-03*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35-1.7532467532467534e-03*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+3.2873376623376621e-04*w46*Jinv11-2.5750811688311690e-03*Jinv20*w14-2.5750811688311690e-03*w5*Jinv21-1.0957792207792207e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02+2.1915584415584414e-03*w49*Jinv10-3.8352272727272725e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33-2.6298701298701297e-03*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12-3.8352272727272725e-04*w37*Jinv11+3.6525974025974026e-05*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.2053571428571428e-03*w7*Jinv10+3.2873376623376621e-04*w47*Jinv21+3.2873376623376621e-04*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+3.8352272727272725e-04*w31*Jinv12+3.8352272727272725e-04*w46*Jinv12+5.4788961038961042e-05*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-1.0957792207792207e-03*Jinv22*w50+1.2053571428571428e-03*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+1.2053571428571428e-03*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+3.8352272727272725e-04*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49-3.8352272727272725e-04*w36*Jinv01-1.0957792207792208e-04*w7*Jinv11-3.8352272727272725e-04*w9*Jinv00+3.2873376623376621e-04*w37*Jinv10+3.6525974025974026e-05*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5+3.8352272727272725e-04*w47*Jinv22+2.1915584415584414e-03*w49*Jinv11-1.2601461038961039e-03*Jinv21*w14-3.8352272727272725e-04*w10*Jinv10+4.9310064935064932e-04*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-1.2601461038961039e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-4.9310064935064932e-04*w57*Jinv00-4.9310064935064932e-04*Jinv20*w59+1.3149350649350649e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+1.2053571428571428e-03*Jinv21*w23+2.5568181818181820e-04*Jinv20*w2-1.6436688311688311e-03*w15*Jinv00-1.0957792207792207e-03*w48*Jinv02-3.6525974025974026e-05*Jinv01*w9-1.6436688311688311e-03*Jinv20*w17-1.7532467532467534e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10+8.2183441558441553e-04*w20*Jinv20+3.6525974025974026e-05*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+3.8352272727272725e-04*Jinv02*w36+1.3149350649350649e-03*Jinv12*w43-1.2601461038961039e-03*w5*Jinv20-1.7532467532467534e-03*Jinv01*w30-1.0957792207792207e-03*Jinv02*w51-1.0957792207792207e-03*Jinv22*w56+3.6525974025974026e-05*Jinv12*w28+8.2183441558441553e-04*w25*Jinv11+1.0957792207792207e-03*w55*Jinv10-3.8352272727272725e-04*w47*Jinv20+3.2873376623376621e-03*w16*Jinv12+2.1915584415584414e-03*Jinv21*w50-3.8352272727272725e-04*Jinv01*w27+2.1915584415584414e-03*w48*Jinv01+3.6525974025974026e-05*Jinv12*w1-3.6525974025974026e-05*w10*Jinv11)*D;
  A[50][18]=(0.0)*D;
  A[50][19]=(0.0)*D;
  A[50][20]=(5.4788961038961036e-04*Jinv00*w12-1.3697240259740259e-04*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+5.4788961038961036e-04*w13*Jinv10-3.8352272727272725e-04*Jinv21*w44+2.4655032467532466e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.2049512987012989e-04*Jinv01*w3+5.4788961038961036e-04*Jinv20*w23-7.1225649350649346e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-7.1225649350649346e-04*w43*Jinv10-3.8352272727272725e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+2.7394480519480518e-04*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+2.7394480519480518e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-8.7662337662337668e-04*w16*Jinv10-3.8352272727272725e-04*w43*Jinv11+5.2049512987012989e-04*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41-1.3697240259740259e-04*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.0957792207792208e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50-1.3697240259740259e-04*Jinv20*w29+5.2049512987012989e-04*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10+5.4788961038961036e-04*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-3.5612824675324673e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+2.7394480519480518e-04*Jinv01*w24-7.6704545454545450e-04*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+1.9176136363636362e-04*w51*Jinv00+2.4655032467532466e-04*w4*Jinv10+5.2049512987012989e-04*w4*Jinv11-1.0957792207792208e-04*w41*Jinv20-3.5612824675324673e-04*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-7.1225649350649346e-04*Jinv20*w56+2.4655032467532466e-04*w26*Jinv20+6.3920454545454550e-05*w11*Jinv20-7.6704545454545450e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+2.7394480519480518e-04*w46*Jinv11+5.4788961038961036e-04*Jinv20*w14+5.2049512987012989e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-1.0957792207792208e-04*Jinv21*w29-7.6704545454545450e-04*Jinv01*w33-1.0957792207792208e-04*w39*Jinv00-3.5612824675324673e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-7.1225649350649346e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+2.7394480519480518e-04*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11+1.9176136363636362e-04*Jinv20*w53+5.4788961038961036e-04*w22*Jinv10+8.2183441558441553e-05*w22*Jinv11+8.2183441558441553e-05*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01+1.9176136363636362e-04*w52*Jinv10-7.1225649350649346e-04*w7*Jinv11+6.3920454545454550e-05*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10-3.5612824675324673e-04*Jinv21*w56+2.4655032467532466e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+6.3920454545454550e-05*w10*Jinv10-7.1225649350649346e-04*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+8.2183441558441553e-05*Jinv21*w23-3.5612824675324673e-04*Jinv01*w54-3.5612824675324673e-04*w55*Jinv11+2.4655032467532466e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-8.7662337662337668e-04*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-8.7662337662337668e-04*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-1.0957792207792208e-04*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+2.4655032467532466e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+2.7394480519480518e-04*w25*Jinv11-7.1225649350649346e-04*w55*Jinv10-7.1225649350649346e-04*w42*Jinv00+5.2049512987012989e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50-1.0957792207792208e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[50][21]=(0.0)*D;
  A[50][22]=(0.0)*D;
  A[50][23]=(-5.2049512987012989e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+7.6704545454545450e-04*w40*Jinv12+3.5612824675324673e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-5.2049512987012989e-04*w13*Jinv10+7.6704545454545450e-04*Jinv02*w39-3.2873376623376621e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20+1.1231737012987013e-03*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23-2.7394480519480518e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12+1.3697240259740259e-04*w0*Jinv02+3.5612824675324673e-04*Jinv21*w2+3.8352272727272725e-04*w43*Jinv10-3.2873376623376621e-04*w42*Jinv01-1.0957792207792208e-04*w11*Jinv21-1.2601461038961039e-03*w16*Jinv11-1.2601461038961039e-03*Jinv21*w17+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02+1.0957792207792208e-04*w10*Jinv12+7.1225649350649346e-04*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+3.5612824675324673e-04*w0*Jinv01+1.3697240259740259e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-2.7394480519480518e-04*Jinv22*w20-3.2873376623376621e-04*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41+3.8352272727272725e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.0957792207792208e-04*w11*Jinv22+3.5612824675324673e-04*Jinv11*w1-4.3831168831168834e-04*w13*Jinv11-1.2601461038961039e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11+7.1225649350649346e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+7.1225649350649346e-04*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22-1.3697240259740260e-03*Jinv01*w39+1.3697240259740259e-04*Jinv22*w2-2.7394480519480518e-04*Jinv00*w18-2.7394480519480518e-04*w36*Jinv00+2.7394480519480518e-04*Jinv21*w38+1.6436688311688311e-04*Jinv02*w45-9.1314935064935065e-06*Jinv22*w59-1.0957792207792208e-04*w34*Jinv11+1.9176136363636362e-04*w19*Jinv11-3.5612824675324673e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+3.8352272727272725e-04*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10+1.1231737012987013e-03*w4*Jinv11+7.6704545454545450e-04*w41*Jinv20-5.2049512987012989e-04*Jinv12*w13-2.7394480519480518e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+9.5880681818181818e-04*w32*Jinv21+1.9176136363636362e-04*Jinv21*w20-8.2183441558441553e-05*w6*Jinv00-2.7394480519480518e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+7.6704545454545450e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20+1.0957792207792208e-04*w11*Jinv20+3.5612824675324673e-04*w52*Jinv12-1.0957792207792208e-04*Jinv21*w35+9.5880681818181818e-04*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26-5.2049512987012989e-04*Jinv20*w14+1.1231737012987013e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+7.1225649350649346e-04*w21*Jinv02+7.1225649350649346e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33+7.6704545454545450e-04*w39*Jinv00-5.2049512987012989e-04*Jinv02*w12+1.9176136363636362e-04*Jinv01*w18+2.7394480519480518e-04*w37*Jinv11+1.0957792207792208e-04*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01-8.2183441558441553e-05*w7*Jinv10-2.7394480519480518e-04*Jinv20*w38+7.1225649350649346e-04*Jinv22*w23-3.5612824675324673e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+7.1225649350649346e-04*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01-3.5612824675324673e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+7.1225649350649346e-04*Jinv12*w49+2.7394480519480518e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11+1.0957792207792208e-04*w9*Jinv00-2.7394480519480518e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-2.7394480519480518e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-1.4245129870129869e-03*w49*Jinv11-4.3831168831168834e-04*Jinv21*w14+1.0957792207792208e-04*w10*Jinv10+3.8352272727272725e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-2.7394480519480518e-04*Jinv02*w18-4.3831168831168834e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.3697240259740259e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10+1.3697240259740259e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+7.1225649350649346e-04*w48*Jinv02-1.0957792207792208e-04*Jinv01*w9+5.4788961038961042e-05*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+7.6704545454545450e-04*w40*Jinv10-2.7394480519480518e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-5.2049512987012989e-04*Jinv22*w14-1.3697240259740260e-03*w40*Jinv11-2.7394480519480518e-04*Jinv02*w36+3.8352272727272725e-04*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20+9.5880681818181818e-04*Jinv01*w30+3.5612824675324673e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+1.6436688311688311e-04*w55*Jinv10+3.8352272727272725e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-1.4245129870129869e-03*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27-1.4245129870129869e-03*w48*Jinv01+1.3697240259740259e-04*Jinv12*w1-1.0957792207792208e-04*w10*Jinv11)*D;
  A[50][24]=(0.0)*D;
  A[50][25]=(0.0)*D;
  A[50][26]=(5.2049512987012989e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+8.2183441558441553e-05*Jinv20*w8+5.2049512987012989e-04*w13*Jinv10-7.1225649350649346e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+5.4788961038961036e-04*Jinv01*w3-7.1225649350649346e-04*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.3697240259740259e-04*Jinv21*w2-3.8352272727272725e-04*w43*Jinv10-7.1225649350649346e-04*w42*Jinv01-1.3697240259740259e-04*w11*Jinv21-8.7662337662337668e-04*w16*Jinv11-8.7662337662337668e-04*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26-1.3697240259740259e-04*w0*Jinv01-1.3697240259740259e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-7.1225649350649346e-04*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-1.3697240259740259e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-8.7662337662337668e-04*w15*Jinv01+6.3920454545454550e-05*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-7.1225649350649346e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+2.7394480519480518e-04*Jinv00*w18+2.7394480519480518e-04*w36*Jinv00+5.2049512987012989e-04*Jinv21*w38-3.5612824675324673e-04*Jinv01*w24-1.0957792207792208e-04*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-7.1225649350649346e-04*Jinv21*w53-3.5612824675324673e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+5.4788961038961036e-04*w4*Jinv11-7.6704545454545450e-04*w41*Jinv20+2.7394480519480518e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+8.2183441558441553e-05*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-3.5612824675324673e-04*w26*Jinv20-1.0957792207792208e-04*w11*Jinv20-1.0957792207792208e-04*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11-8.2183441558441553e-05*w46*Jinv11+5.2049512987012989e-04*Jinv20*w14+5.4788961038961036e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10+6.3920454545454550e-05*Jinv21*w29-1.0957792207792208e-04*Jinv01*w33-7.6704545454545450e-04*w39*Jinv00+2.4655032467532466e-04*Jinv01*w18+5.2049512987012989e-04*w37*Jinv11+5.4788961038961036e-04*w6*Jinv01+8.2183441558441553e-05*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21+2.7394480519480518e-04*Jinv20*w38-7.1225649350649346e-04*w52*Jinv11-3.5612824675324673e-04*Jinv20*w53-7.1225649350649346e-04*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-7.1225649350649346e-04*Jinv01*w51+5.2049512987012989e-04*w36*Jinv01-3.5612824675324673e-04*w52*Jinv10+5.4788961038961036e-04*w7*Jinv11-1.0957792207792208e-04*w9*Jinv00+2.7394480519480518e-04*w37*Jinv10+1.9176136363636362e-04*Jinv21*w56-3.5612824675324673e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14-1.0957792207792208e-04*w10*Jinv10-3.8352272727272725e-04*Jinv20*w44+2.4655032467532466e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.3697240259740259e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+1.9176136363636362e-04*Jinv01*w54+1.9176136363636362e-04*w55*Jinv11-3.5612824675324673e-04*w25*Jinv10-1.3697240259740259e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00-1.3697240259740259e-04*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-7.6704545454545450e-04*w40*Jinv10+2.7394480519480518e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30-3.5612824675324673e-04*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-3.8352272727272725e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+6.3920454545454550e-05*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01-1.3697240259740259e-04*w10*Jinv11)*D;
  A[50][27]=(0.0)*D;
  A[50][28]=(0.0)*D;
  A[50][29]=(2.1915584415584417e-04*Jinv21*w44-4.2918019480519480e-04*Jinv01*w3+6.4935064935064935e-05*Jinv21*w59-1.4204545454545455e-05*Jinv21*w2+2.1915584415584417e-04*w42*Jinv01+1.0247564935064935e-04*w11*Jinv21+3.4699675324675324e-04*w16*Jinv11+3.4699675324675324e-04*Jinv21*w17-2.6481331168831169e-04*Jinv01*w45+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01+2.1915584415584417e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41-1.4204545454545455e-05*Jinv11*w1+1.1870941558441559e-04*w13*Jinv11+3.4699675324675324e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+4.3831168831168834e-04*Jinv01*w39-4.2918019480519480e-04*Jinv21*w38-5.4788961038961042e-05*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11-2.7394480519480521e-05*w19*Jinv11+7.4878246753246752e-04*Jinv21*w53-4.2918019480519480e-04*w4*Jinv11-5.8441558441558442e-04*w32*Jinv21-2.7394480519480521e-05*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-5.8441558441558442e-04*w31*Jinv11-2.6481331168831169e-04*w46*Jinv11-4.2918019480519480e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33-2.7394480519480521e-05*Jinv01*w18-4.2918019480519480e-04*w37*Jinv11-2.1915584415584417e-04*w6*Jinv01-2.6481331168831169e-04*w47*Jinv21+7.4878246753246752e-04*w52*Jinv11-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01+7.4878246753246752e-04*Jinv01*w51-4.2918019480519480e-04*w36*Jinv01-2.1915584415584417e-04*w7*Jinv11+3.6525974025974026e-05*Jinv21*w56+1.4975649350649350e-03*w49*Jinv11+1.1870941558441559e-04*Jinv21*w14+1.1870941558441559e-04*Jinv01*w12-2.1915584415584417e-04*Jinv21*w8-1.0957792207792208e-04*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11+1.0247564935064935e-04*Jinv01*w9+4.3831168831168834e-04*w40*Jinv11-5.8441558441558442e-04*Jinv01*w30-5.4788961038961042e-05*w25*Jinv11+1.4975649350649350e-03*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27+1.4975649350649350e-03*w48*Jinv01+1.0247564935064935e-04*w10*Jinv11)*D;
  A[50][30]=(0.0)*D;
  A[50][31]=(0.0)*D;
  A[50][32]=(1.2327516233766234e-03*Jinv00*w12+1.5523538961038962e-04*w28*Jinv10-1.4793019480519481e-03*w40*Jinv12+8.2183441558441553e-05*Jinv22*w53-4.3831168831168834e-04*Jinv20*w8+6.2094155844155847e-04*w57*Jinv02+1.2327516233766234e-03*w13*Jinv10-1.4793019480519481e-03*Jinv02*w39+8.2183441558441553e-04*Jinv21*w44+1.2327516233766234e-03*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20+1.2327516233766234e-03*Jinv01*w3-4.3831168831168834e-04*Jinv20*w23+4.3831168831168834e-04*Jinv22*w38+2.4655032467532466e-04*Jinv00*w54+1.0957792207792207e-03*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12-4.0178571428571428e-04*w0*Jinv02+1.4610389610389610e-04*Jinv21*w2+8.2183441558441553e-04*w43*Jinv10+8.2183441558441553e-04*w42*Jinv01+1.5523538961038962e-04*w11*Jinv21+1.3697240259740260e-03*w16*Jinv11+1.3697240259740260e-03*Jinv21*w17-8.2183441558441553e-05*Jinv01*w45+8.2183441558441553e-05*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02-2.7394480519480518e-04*w10*Jinv12-3.7804383116883116e-03*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24-1.4793019480519481e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26+1.4610389610389610e-04*w0*Jinv01+1.4610389610389610e-04*w1*Jinv10+1.3697240259740260e-03*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20+8.2183441558441553e-04*w43*Jinv11-8.2183441558441553e-05*w45*Jinv00+6.5746753246753243e-04*Jinv21*w41+4.3831168831168834e-04*w42*Jinv02+1.5523538961038962e-04*Jinv00*w27-2.7394480519480518e-04*w11*Jinv22+1.4610389610389610e-04*Jinv11*w1+1.2327516233766234e-03*w13*Jinv11+1.3697240259740260e-03*w15*Jinv01+1.5523538961038962e-04*w28*Jinv11-3.7804383116883116e-03*Jinv20*w50+1.5523538961038962e-04*Jinv20*w29-8.2183441558441553e-05*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33+9.5880681818181818e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3-4.3831168831168834e-04*w21*Jinv00-1.3149350649350649e-03*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39-4.0178571428571428e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-8.2183441558441553e-05*w36*Jinv00-8.2183441558441553e-05*Jinv21*w38+4.3831168831168834e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24+6.2094155844155847e-04*Jinv22*w59+6.5746753246753243e-04*w34*Jinv11-2.4655032467532466e-04*w19*Jinv11+2.4655032467532466e-04*Jinv21*w53+2.4655032467532466e-04*w51*Jinv00-1.4793019480519481e-03*Jinv12*w34+4.3831168831168834e-04*Jinv22*w44+1.2327516233766234e-03*w4*Jinv10+1.2327516233766234e-03*w4*Jinv11+6.5746753246753243e-04*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10+1.3149350649350649e-03*w32*Jinv21-2.4655032467532466e-04*Jinv21*w20-4.3831168831168834e-04*w6*Jinv00+4.3831168831168834e-04*w37*Jinv12+2.4655032467532466e-04*Jinv20*w56-1.4793019480519481e-03*Jinv22*w41-2.4655032467532466e-04*w26*Jinv20+1.5523538961038962e-04*w11*Jinv20+8.2183441558441553e-05*w52*Jinv12+6.5746753246753243e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-8.2183441558441553e-05*w46*Jinv11+1.2327516233766234e-03*Jinv20*w14+1.2327516233766234e-03*w5*Jinv21+8.2183441558441553e-05*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12+9.5880681818181818e-04*w21*Jinv02-3.7804383116883116e-03*w49*Jinv10+1.5523538961038962e-04*Jinv21*w29+6.5746753246753243e-04*Jinv01*w33+6.5746753246753243e-04*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12-2.4655032467532466e-04*Jinv01*w18-8.2183441558441553e-05*w37*Jinv11-2.7394480519480518e-04*Jinv02*w9-4.3831168831168834e-04*w6*Jinv01-4.3831168831168834e-04*w7*Jinv10-8.2183441558441553e-05*w47*Jinv21-8.2183441558441553e-05*Jinv20*w38+9.5880681818181818e-04*Jinv22*w23+2.4655032467532466e-04*w52*Jinv11-1.3149350649350649e-03*w31*Jinv12+4.3831168831168834e-04*w46*Jinv12+2.4655032467532466e-04*Jinv20*w53+9.5880681818181818e-04*Jinv22*w8-4.3831168831168834e-04*w22*Jinv10-2.6298701298701297e-03*Jinv22*w50-4.3831168831168834e-04*w22*Jinv11+9.5880681818181818e-04*Jinv12*w7-4.3831168831168834e-04*w21*Jinv01+2.4655032467532466e-04*Jinv01*w51-1.3149350649350649e-03*Jinv02*w30-2.6298701298701297e-03*Jinv12*w49-8.2183441558441553e-05*w36*Jinv01+2.4655032467532466e-04*w52*Jinv10-4.3831168831168834e-04*w7*Jinv11+1.5523538961038962e-04*w9*Jinv00-8.2183441558441553e-05*w37*Jinv10+2.4655032467532466e-04*Jinv21*w56-2.4655032467532466e-04*w24*Jinv00-2.7394480519480518e-04*Jinv02*w27+9.5880681818181818e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5+4.3831168831168834e-04*w47*Jinv22-3.7804383116883116e-03*w49*Jinv11+1.2327516233766234e-03*Jinv21*w14+1.5523538961038962e-04*w10*Jinv10+8.2183441558441553e-04*Jinv20*w44+6.2094155844155847e-04*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18+1.2327516233766234e-03*Jinv01*w12-4.3831168831168834e-04*Jinv21*w8+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59-1.4793019480519481e-03*Jinv02*w33+1.4610389610389610e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20-4.3831168831168834e-04*Jinv21*w23+2.4655032467532466e-04*Jinv01*w54+2.4655032467532466e-04*w55*Jinv11-2.4655032467532466e-04*w25*Jinv10+1.4610389610389610e-04*Jinv20*w2+1.3697240259740260e-03*w15*Jinv00-2.6298701298701297e-03*w48*Jinv02+1.5523538961038962e-04*Jinv01*w9+1.3697240259740260e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00+6.5746753246753243e-04*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-2.7394480519480518e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11+4.3831168831168834e-04*Jinv02*w36+4.3831168831168834e-04*Jinv12*w43+1.2327516233766234e-03*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30+8.2183441558441553e-05*Jinv02*w51+8.2183441558441553e-05*Jinv22*w56-2.7394480519480518e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11+2.4655032467532466e-04*w55*Jinv10+8.2183441558441553e-04*w42*Jinv00-8.2183441558441553e-05*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-3.7804383116883116e-03*Jinv21*w50+1.5523538961038962e-04*Jinv01*w27-3.7804383116883116e-03*w48*Jinv01-4.0178571428571428e-04*Jinv12*w1+1.5523538961038962e-04*w10*Jinv11)*D;
  A[50][33]=(0.0)*D;
  A[50][34]=(0.0)*D;
  A[50][35]=(-1.7532467532467534e-03*Jinv00*w12+2.7394480519480518e-04*w28*Jinv10-2.9586038961038961e-03*w40*Jinv12-2.0819805194805196e-03*Jinv22*w53+6.0267857142857139e-04*Jinv20*w8-4.3831168831168834e-04*w57*Jinv02-1.7532467532467534e-03*w13*Jinv10-2.9586038961038961e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44-1.3697240259740260e-03*w3*Jinv00-2.7942370129870131e-03*w32*Jinv20+7.6704545454545450e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02-4.9310064935064932e-04*Jinv21*w2-9.8620129870129864e-04*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+1.3149350649350649e-03*Jinv01*w45-1.0409902597402598e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-5.6615259740259744e-04*w10*Jinv12+8.5470779220779228e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-5.9172077922077923e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+5.4788961038961042e-05*Jinv21*w26-4.9310064935064932e-04*w0*Jinv01+1.6436688311688311e-04*w1*Jinv10-6.5746753246753243e-04*w16*Jinv10+3.2873376623376621e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11-1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02+2.7394480519480518e-04*Jinv00*w27-5.6615259740259744e-04*w11*Jinv22-4.9310064935064932e-04*Jinv11*w1+3.8352272727272725e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-2.7394480519480518e-04*w28*Jinv11+8.5470779220779228e-03*Jinv20*w50+2.7394480519480518e-04*Jinv20*w29-1.3149350649350649e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33+1.2053571428571428e-03*w22*Jinv12-2.7942370129870131e-03*w31*Jinv10-1.8628246753246754e-03*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-9.8620129870129864e-04*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+3.2873376623376621e-04*Jinv00*w18-4.9310064935064932e-04*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.2601461038961039e-03*Jinv02*w45+5.4788961038961042e-05*Jinv01*w24-4.3831168831168834e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-5.9172077922077923e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44-1.3697240259740260e-03*w4*Jinv10+7.6704545454545450e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-9.3141233766233771e-04*Jinv12*w13+3.2873376623376621e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10+2.1367694805194807e-03*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+6.0267857142857139e-04*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56-2.9586038961038961e-03*Jinv22*w41+5.4788961038961042e-05*w11*Jinv20-2.0819805194805196e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35+2.1367694805194807e-03*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+1.3149350649350649e-03*w46*Jinv11-1.7532467532467534e-03*Jinv20*w14+7.6704545454545450e-04*w5*Jinv21-1.0409902597402598e-03*Jinv02*w54-1.8628246753246754e-03*w4*Jinv12+1.2053571428571428e-03*w21*Jinv02+8.5470779220779228e-03*w49*Jinv10-2.7394480519480518e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-9.3141233766233771e-04*Jinv02*w12+3.8352272727272725e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.6615259740259744e-04*Jinv02*w9+7.6704545454545450e-04*w6*Jinv01+6.0267857142857139e-04*w7*Jinv10+1.3149350649350649e-03*w47*Jinv21-4.9310064935064932e-04*Jinv20*w38+1.2053571428571428e-03*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+1.2601461038961039e-03*w46*Jinv12+6.0267857142857139e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-9.8620129870129864e-04*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11+6.0267857142857139e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51-9.8620129870129864e-04*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01+7.6704545454545450e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00-4.9310064935064932e-04*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56-6.9399350649350649e-04*Jinv02*w27+6.0267857142857139e-04*w6*Jinv02+3.2873376623376621e-04*w19*Jinv12-1.8628246753246754e-03*Jinv22*w5+1.2601461038961039e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+3.8352272727272725e-04*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10-4.3831168831168834e-04*w58*Jinv12+3.2873376623376621e-04*Jinv02*w18+3.8352272727272725e-04*Jinv01*w12+7.6704545454545450e-04*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-5.9172077922077923e-03*Jinv02*w33+1.6436688311688311e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+1.6436688311688311e-04*Jinv20*w2-6.5746753246753243e-04*w15*Jinv00-9.8620129870129864e-04*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9-6.5746753246753243e-04*Jinv20*w17-2.7942370129870131e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10+3.2873376623376621e-04*w20*Jinv20-6.9399350649350649e-04*Jinv22*w29-9.3141233766233771e-04*Jinv22*w14-9.8620129870129864e-04*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43-1.3697240259740260e-03*w5*Jinv20+2.1367694805194807e-03*Jinv01*w30-2.0819805194805196e-03*Jinv02*w51-1.0409902597402598e-03*Jinv22*w56-6.9399350649350649e-04*Jinv12*w28+5.4788961038961042e-05*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10-1.3149350649350649e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-2.7394480519480518e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[50][36]=(0.0)*D;
  A[50][37]=(0.0)*D;
  A[50][38]=(5.2049512987012989e-04*Jinv00*w12-4.2918019480519480e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8+9.1314935064935065e-06*w57*Jinv02+5.2049512987012989e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+1.3697240259740259e-04*w3*Jinv00+1.4793019480519481e-03*w32*Jinv20+5.4788961038961036e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-2.3833198051948050e-03*Jinv00*w54+2.7394480519480518e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-8.2183441558441553e-04*w43*Jinv10-3.5612824675324673e-04*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-4.7666396103896100e-03*w48*Jinv00+2.7394480519480518e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35-3.1047077922077924e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-3.5612824675324673e-04*Jinv22*w20+1.3971185064935066e-03*w45*Jinv00-3.8352272727272725e-04*w42*Jinv02-4.2918019480519480e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-4.7666396103896100e-03*Jinv20*w50-4.2918019480519480e-04*Jinv20*w29+1.3971185064935066e-03*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17-1.6436688311688311e-03*Jinv00*w33+8.2183441558441553e-05*w22*Jinv12+1.4793019480519481e-03*w31*Jinv10+5.2049512987012989e-04*Jinv02*w3+5.4788961038961036e-04*w21*Jinv00+5.2049512987012989e-04*w32*Jinv22-1.3697240259740259e-04*Jinv22*w2-8.2183441558441553e-05*Jinv00*w18+5.7528409090909093e-04*w36*Jinv00+2.7394480519480518e-04*Jinv02*w45+9.1314935064935065e-06*Jinv22*w59-2.4655032467532466e-04*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+1.3697240259740259e-04*w4*Jinv10-2.6298701298701297e-03*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-8.2183441558441553e-05*w19*Jinv10-1.6436688311688311e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-2.3833198051948050e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+2.4655032467532466e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12+2.7394480519480518e-04*Jinv22*w26+5.2049512987012989e-04*Jinv20*w14-3.5612824675324673e-04*Jinv02*w54+5.2049512987012989e-04*w4*Jinv12+8.2183441558441553e-05*w21*Jinv02-4.7666396103896100e-03*w49*Jinv10-2.6298701298701297e-03*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10+5.7528409090909093e-04*Jinv20*w38+8.2183441558441553e-05*Jinv22*w23+5.2049512987012989e-04*w31*Jinv12+2.7394480519480518e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-7.1225649350649346e-04*Jinv22*w8+5.4788961038961036e-04*w22*Jinv10-7.1225649350649346e-04*Jinv22*w50-7.1225649350649346e-04*Jinv12*w7+5.2049512987012989e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49-2.4655032467532466e-04*w52*Jinv10-2.1002435064935065e-04*w9*Jinv00+5.7528409090909093e-04*w37*Jinv10+2.4655032467532466e-04*w24*Jinv00-1.0957792207792208e-04*Jinv02*w27-7.1225649350649346e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12+5.2049512987012989e-04*Jinv22*w5+2.7394480519480518e-04*w47*Jinv22-2.1002435064935065e-04*w10*Jinv10-8.2183441558441553e-04*Jinv20*w44+9.1314935064935065e-06*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33-3.1047077922077924e-04*w0*Jinv00-1.6436688311688311e-03*w35*Jinv20+2.4655032467532466e-04*w25*Jinv10-3.1047077922077924e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00-7.1225649350649346e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+1.4793019480519481e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10-8.2183441558441553e-05*w20*Jinv20-1.0957792207792208e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-1.6436688311688311e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+1.3697240259740259e-04*w5*Jinv20-1.6436688311688311e-04*Jinv02*w51-3.5612824675324673e-04*Jinv22*w56-1.0957792207792208e-04*Jinv12*w28-2.3833198051948050e-03*w55*Jinv10-8.2183441558441553e-04*w42*Jinv00+1.3971185064935066e-03*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-1.3697240259740259e-04*Jinv12*w1)*D;
  A[50][39]=(0.0)*D;
  A[50][40]=(0.0)*D;
  A[50][41]=(7.6704545454545450e-04*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-5.9172077922077923e-03*w40*Jinv12-1.0409902597402598e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8-4.3831168831168834e-04*w57*Jinv02+7.6704545454545450e-04*w13*Jinv10-5.9172077922077923e-03*Jinv02*w39+3.8352272727272725e-04*w3*Jinv00+2.1367694805194807e-03*w32*Jinv20-1.7532467532467534e-03*Jinv01*w3+7.6704545454545450e-04*Jinv20*w23+1.2601461038961039e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59+3.2873376623376621e-04*w25*Jinv12-2.7394480519480518e-04*w0*Jinv02+1.6436688311688311e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10+2.7394480519480518e-04*w11*Jinv21-6.5746753246753243e-04*w16*Jinv11-6.5746753246753243e-04*Jinv21*w17-4.9310064935064932e-04*Jinv01*w45-2.0819805194805196e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-3.6160714285714286e-03*w15*Jinv02-6.9399350649350649e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00+3.2873376623376621e-04*Jinv02*w24-2.9586038961038961e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01+3.2873376623376621e-04*Jinv21*w26+1.6436688311688311e-04*w0*Jinv01-4.9310064935064932e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20+2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-2.3011363636363637e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27-6.9399350649350649e-04*w11*Jinv22+1.6436688311688311e-04*Jinv11*w1-1.3697240259740260e-03*w13*Jinv11-6.5746753246753243e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-3.6160714285714286e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+6.0267857142857139e-04*w22*Jinv12+2.1367694805194807e-03*w31*Jinv10-9.3141233766233771e-04*Jinv02*w3+7.6704545454545450e-04*w21*Jinv00+9.8620129870129864e-04*Jinv01*w39-2.7394480519480518e-04*Jinv22*w2+5.4788961038961042e-05*Jinv00*w18+1.3149350649350649e-03*w36*Jinv00-1.3149350649350649e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+3.2873376623376621e-04*Jinv01*w24-4.3831168831168834e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11+2.1367694805194807e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-2.9586038961038961e-03*Jinv12*w34-2.3011363636363637e-03*Jinv22*w44+3.8352272727272725e-04*w4*Jinv10-1.7532467532467534e-03*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.8628246753246754e-03*Jinv12*w13+5.4788961038961042e-05*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-2.7942370129870131e-03*w32*Jinv21+5.4788961038961042e-05*w6*Jinv00+1.2601461038961039e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56-5.9172077922077923e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.7394480519480518e-04*w11*Jinv20-1.0409902597402598e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.7942370129870131e-03*w31*Jinv11+3.2873376623376621e-04*Jinv22*w26-4.9310064935064932e-04*w46*Jinv11+7.6704545454545450e-04*Jinv20*w14-1.7532467532467534e-03*w5*Jinv21-2.0819805194805196e-03*Jinv02*w54-9.3141233766233771e-04*w4*Jinv12+6.0267857142857139e-04*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33-1.9724025974025973e-03*w39*Jinv00-1.8628246753246754e-03*Jinv02*w12-1.3149350649350649e-03*w37*Jinv11-6.9399350649350649e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+5.4788961038961042e-05*w7*Jinv10-4.9310064935064932e-04*w47*Jinv21+1.3149350649350649e-03*Jinv20*w38+6.0267857142857139e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+1.3697240259740260e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+1.2053571428571428e-03*Jinv22*w8+7.6704545454545450e-04*w22*Jinv10-9.8620129870129864e-04*Jinv22*w50+6.0267857142857139e-04*w22*Jinv11+1.2053571428571428e-03*Jinv12*w7+6.0267857142857139e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51-9.8620129870129864e-04*Jinv12*w49-1.3149350649350649e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-2.7394480519480518e-04*w9*Jinv00+1.3149350649350649e-03*w37*Jinv10+3.8352272727272725e-04*w24*Jinv00-5.6615259740259744e-04*Jinv02*w27+1.2053571428571428e-03*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12-9.3141233766233771e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22+8.5470779220779228e-03*w49*Jinv11-1.3697240259740260e-03*Jinv21*w14-2.7394480519480518e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44-4.3831168831168834e-04*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18-1.3697240259740260e-03*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-2.9586038961038961e-03*Jinv02*w33-4.9310064935064932e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+6.0267857142857139e-04*Jinv21*w23+3.8352272727272725e-04*w25*Jinv10-4.9310064935064932e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-9.8620129870129864e-04*w48*Jinv02+2.7394480519480518e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+2.1367694805194807e-03*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+5.4788961038961042e-05*w20*Jinv20-5.6615259740259744e-04*Jinv22*w29-1.8628246753246754e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11+1.2601461038961039e-03*Jinv02*w36-2.3011363636363637e-03*Jinv12*w43+3.8352272727272725e-04*w5*Jinv20-2.7942370129870131e-03*Jinv01*w30-1.0409902597402598e-03*Jinv02*w51-2.0819805194805196e-03*Jinv22*w56-5.6615259740259744e-04*Jinv12*w28+3.2873376623376621e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-9.8620129870129864e-04*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-3.6160714285714286e-03*w16*Jinv12+8.5470779220779228e-03*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27+8.5470779220779228e-03*w48*Jinv01-2.7394480519480518e-04*Jinv12*w1+2.7394480519480518e-04*w10*Jinv11)*D;
  A[50][42]=(0.0)*D;
  A[50][43]=(0.0)*D;
  A[50][44]=(-7.6704545454545450e-04*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+1.0957792207792207e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-4.9310064935064932e-04*w57*Jinv02-7.6704545454545450e-04*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+9.8620129870129864e-04*Jinv21*w44-3.8352272727272725e-04*w3*Jinv00-2.1367694805194807e-03*w32*Jinv20-7.6704545454545450e-04*Jinv01*w3-7.6704545454545450e-04*Jinv20*w23-3.8352272727272725e-04*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59-5.4788961038961042e-05*w25*Jinv12-3.6525974025974026e-05*w0*Jinv02+4.9310064935064932e-04*Jinv21*w2+9.8620129870129864e-04*w43*Jinv10+9.8620129870129864e-04*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.0957792207792207e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-3.2873376623376621e-03*w15*Jinv02-3.6525974025974026e-05*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-5.4788961038961042e-05*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-5.4788961038961042e-05*Jinv21*w26+4.9310064935064932e-04*w0*Jinv01+4.9310064935064932e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-5.4788961038961042e-05*Jinv22*w20+9.8620129870129864e-04*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-1.3149350649350649e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27-3.6525974025974026e-05*w11*Jinv22+4.9310064935064932e-04*Jinv11*w1-3.8352272727272725e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+2.7394480519480518e-04*w28*Jinv11+4.2735389610389614e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-3.2873376623376621e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-2.1367694805194807e-03*w31*Jinv10-5.4788961038961042e-05*Jinv02*w3-7.6704545454545450e-04*w21*Jinv00-3.8352272727272725e-04*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-3.6525974025974026e-05*Jinv22*w2-5.4788961038961042e-05*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38-3.8352272727272725e-04*Jinv02*w45-5.4788961038961042e-05*Jinv01*w24-4.9310064935064932e-04*Jinv22*w59+1.9724025974025973e-03*w34*Jinv11-3.8352272727272725e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+2.1367694805194807e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-1.3149350649350649e-03*Jinv22*w44-3.8352272727272725e-04*w4*Jinv10-7.6704545454545450e-04*w4*Jinv11+1.9724025974025973e-03*w41*Jinv20-5.4788961038961042e-05*Jinv12*w13-5.4788961038961042e-05*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-2.1367694805194807e-03*w32*Jinv21-3.8352272727272725e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-3.8352272727272725e-04*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-3.8352272727272725e-04*w26*Jinv20+2.7394480519480518e-04*w11*Jinv20+1.0957792207792207e-03*w52*Jinv12+1.9724025974025973e-03*Jinv21*w35-2.1367694805194807e-03*w31*Jinv11-5.4788961038961042e-05*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11-7.6704545454545450e-04*Jinv20*w14-7.6704545454545450e-04*w5*Jinv21+1.0957792207792207e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+2.7394480519480518e-04*Jinv21*w29+1.9724025974025973e-03*Jinv01*w33+1.9724025974025973e-03*w39*Jinv00-5.4788961038961042e-05*Jinv02*w12-3.8352272727272725e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11-3.6525974025974026e-05*Jinv02*w9-7.6704545454545450e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-1.3149350649350649e-03*w47*Jinv21-1.3149350649350649e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-3.8352272727272725e-04*w31*Jinv12-3.8352272727272725e-04*w46*Jinv12+2.1367694805194807e-03*Jinv20*w53-5.4788961038961042e-05*Jinv22*w8-7.6704545454545450e-04*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-3.8352272727272725e-04*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+2.1367694805194807e-03*w52*Jinv10-7.6704545454545450e-04*w7*Jinv11+2.7394480519480518e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10+2.1367694805194807e-03*Jinv21*w56-3.8352272727272725e-04*w24*Jinv00-3.6525974025974026e-05*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-3.8352272727272725e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-3.8352272727272725e-04*Jinv21*w14+2.7394480519480518e-04*w10*Jinv10+9.8620129870129864e-04*Jinv20*w44-4.9310064935064932e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18-3.8352272727272725e-04*Jinv01*w12-7.6704545454545450e-04*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+4.9310064935064932e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23+2.1367694805194807e-03*Jinv01*w54+2.1367694805194807e-03*w55*Jinv11-3.8352272727272725e-04*w25*Jinv10+4.9310064935064932e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-2.1367694805194807e-03*w30*Jinv00+1.9724025974025973e-03*w40*Jinv10-5.4788961038961042e-05*w20*Jinv20-3.6525974025974026e-05*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-3.8352272727272725e-04*Jinv02*w36-1.3149350649350649e-03*Jinv12*w43-3.8352272727272725e-04*w5*Jinv20-2.1367694805194807e-03*Jinv01*w30+1.0957792207792207e-03*Jinv02*w51+1.0957792207792207e-03*Jinv22*w56-3.6525974025974026e-05*Jinv12*w28-5.4788961038961042e-05*w25*Jinv11+4.2735389610389614e-03*w55*Jinv10+9.8620129870129864e-04*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-3.2873376623376621e-03*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+2.7394480519480518e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01-3.6525974025974026e-05*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[50][45]=(0.0)*D;
  A[50][46]=(0.0)*D;
  A[50][47]=(-7.6704545454545450e-04*w40*Jinv12-3.5612824675324673e-04*Jinv22*w53+9.1314935064935065e-06*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-8.2183441558441553e-04*Jinv21*w44+5.2049512987012989e-04*Jinv01*w3+2.7394480519480518e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-3.5612824675324673e-04*w25*Jinv12-1.3697240259740259e-04*w0*Jinv02-3.1047077922077924e-04*Jinv21*w2-8.2183441558441553e-04*w42*Jinv01-4.2918019480519480e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17+5.7528409090909093e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-5.4788961038961042e-05*w15*Jinv02-1.0957792207792208e-04*w10*Jinv12-3.5612824675324673e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-8.2183441558441553e-05*Jinv21*w26-3.1047077922077924e-04*w0*Jinv01+2.7394480519480518e-04*Jinv22*w20-8.2183441558441553e-04*w43*Jinv11-1.6436688311688311e-03*Jinv21*w41-3.8352272727272725e-04*w42*Jinv02-1.0957792207792208e-04*w11*Jinv22-3.1047077922077924e-04*Jinv11*w1+1.3697240259740259e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11-5.4788961038961042e-05*Jinv22*w17-7.1225649350649346e-04*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w32*Jinv22-1.6436688311688311e-03*Jinv01*w39-1.3697240259740259e-04*Jinv22*w2+1.3971185064935066e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-8.2183441558441553e-05*Jinv01*w24+9.1314935064935065e-06*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+2.4655032467532466e-04*w19*Jinv11-2.3833198051948050e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-3.8352272727272725e-04*Jinv22*w44+5.2049512987012989e-04*w4*Jinv11+5.2049512987012989e-04*Jinv12*w13+1.4793019480519481e-03*w32*Jinv21+2.4655032467532466e-04*Jinv21*w20+2.7394480519480518e-04*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41-3.5612824675324673e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+1.4793019480519481e-03*w31*Jinv11-3.5612824675324673e-04*Jinv22*w26+5.7528409090909093e-04*w46*Jinv11+5.2049512987012989e-04*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-7.1225649350649346e-04*w21*Jinv02-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+5.2049512987012989e-04*Jinv02*w12+2.4655032467532466e-04*Jinv01*w18+1.3971185064935066e-03*w37*Jinv11-1.0957792207792208e-04*Jinv02*w9+5.4788961038961036e-04*w6*Jinv01+5.7528409090909093e-04*w47*Jinv21-7.1225649350649346e-04*Jinv22*w23-2.3833198051948050e-03*w52*Jinv11+5.2049512987012989e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+8.2183441558441553e-05*Jinv22*w8-7.1225649350649346e-04*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11+8.2183441558441553e-05*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01-2.3833198051948050e-03*Jinv01*w51+5.2049512987012989e-04*Jinv02*w30-7.1225649350649346e-04*Jinv12*w49+1.3971185064935066e-03*w36*Jinv01+5.4788961038961036e-04*w7*Jinv11-2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+8.2183441558441553e-05*w6*Jinv02+2.7394480519480518e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-4.7666396103896100e-03*w49*Jinv11+1.3697240259740259e-04*Jinv21*w14+9.1314935064935065e-06*w58*Jinv12+2.7394480519480518e-04*Jinv02*w18+1.3697240259740259e-04*Jinv01*w12+5.4788961038961036e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-7.1225649350649346e-04*w48*Jinv02-4.2918019480519480e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+5.2049512987012989e-04*Jinv22*w14-1.6436688311688311e-03*w40*Jinv11+2.7394480519480518e-04*Jinv02*w36-3.8352272727272725e-04*Jinv12*w43+1.4793019480519481e-03*Jinv01*w30-3.5612824675324673e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-8.2183441558441553e-05*w25*Jinv11-5.4788961038961042e-05*w16*Jinv12-4.7666396103896100e-03*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27-4.7666396103896100e-03*w48*Jinv01-1.3697240259740259e-04*Jinv12*w1-4.2918019480519480e-04*w10*Jinv11)*D;
  A[50][48]=(0.0)*D;
  A[50][49]=(0.0)*D;
  A[50][50]=(1.6436688311688311e-04*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10+4.9310064935064932e-04*w40*Jinv12-1.2327516233766234e-03*Jinv22*w53+7.1225649350649346e-04*Jinv20*w8-1.1870941558441558e-03*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+4.9310064935064932e-04*Jinv02*w39-2.1367694805194807e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00+2.6298701298701297e-03*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+7.1225649350649346e-04*Jinv20*w23+3.5612824675324673e-04*Jinv22*w38-3.6982548701298703e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59-1.2784090909090910e-04*w0*Jinv02-7.8530844155844158e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.3833198051948050e-03*Jinv01*w45-1.2327516233766234e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10+5.4788961038961036e-04*w15*Jinv02-7.3965097402597406e-03*w48*Jinv00+4.9310064935064932e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+3.5612824675324673e-04*Jinv21*w26-7.8530844155844158e-04*w0*Jinv01-7.8530844155844158e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-4.2735389610389614e-03*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27-7.8530844155844158e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-7.3965097402597406e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10+5.4788961038961036e-04*Jinv22*w17-4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12+2.6298701298701297e-03*w31*Jinv10+7.3965097402597403e-04*Jinv02*w3+7.1225649350649346e-04*w21*Jinv00+1.3149350649350649e-03*w32*Jinv22-4.2735389610389614e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+2.3833198051948050e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+3.5612824675324673e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-1.1870941558441558e-03*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11+3.5612824675324673e-04*w19*Jinv11-3.6982548701298703e-03*Jinv21*w53-3.6982548701298703e-03*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+7.3965097402597403e-04*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-4.2735389610389614e-03*w34*Jinv10+2.6298701298701297e-03*w32*Jinv21+3.5612824675324673e-04*Jinv21*w20+7.1225649350649346e-04*w6*Jinv00+3.5612824675324673e-04*w37*Jinv12-3.6982548701298703e-03*Jinv20*w56+4.9310064935064932e-04*Jinv22*w41+3.5612824675324673e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-1.2327516233766234e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+2.6298701298701297e-03*w31*Jinv11+2.3833198051948050e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-1.2327516233766234e-03*Jinv02*w54+7.3965097402597403e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-7.3965097402597406e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+7.3965097402597403e-04*Jinv02*w12+3.5612824675324673e-04*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11+7.1225649350649346e-04*w6*Jinv01+7.1225649350649346e-04*w7*Jinv10+2.3833198051948050e-03*w47*Jinv21+2.3833198051948050e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-3.6982548701298703e-03*w52*Jinv11+1.3149350649350649e-03*w31*Jinv12+3.5612824675324673e-04*w46*Jinv12-3.6982548701298703e-03*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+7.1225649350649346e-04*w22*Jinv10+7.1225649350649346e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7+7.1225649350649346e-04*w21*Jinv01-3.6982548701298703e-03*Jinv01*w51+1.3149350649350649e-03*Jinv02*w30+2.3833198051948050e-03*w36*Jinv01-3.6982548701298703e-03*w52*Jinv10+7.1225649350649346e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+2.3833198051948050e-03*w37*Jinv10-3.6982548701298703e-03*Jinv21*w56+3.5612824675324673e-04*w24*Jinv00-3.5612824675324673e-04*w6*Jinv02+7.3965097402597403e-04*Jinv22*w5+3.5612824675324673e-04*w47*Jinv22-7.3965097402597406e-03*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44-1.1870941558441558e-03*w58*Jinv12+1.6436688311688311e-04*Jinv01*w12+7.1225649350649346e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-7.8530844155844158e-04*w0*Jinv00-4.2735389610389614e-03*w35*Jinv20+7.1225649350649346e-04*Jinv21*w23-3.6982548701298703e-03*Jinv01*w54-3.6982548701298703e-03*w55*Jinv11+3.5612824675324673e-04*w25*Jinv10-7.8530844155844158e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-7.4878246753246752e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.6298701298701297e-03*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+7.3965097402597403e-04*Jinv22*w14-4.2735389610389614e-03*w40*Jinv11+3.5612824675324673e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20+2.6298701298701297e-03*Jinv01*w30-1.2327516233766234e-03*Jinv02*w51-1.2327516233766234e-03*Jinv22*w56+3.5612824675324673e-04*w25*Jinv11-3.6982548701298703e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20+5.4788961038961036e-04*w16*Jinv12-7.3965097402597406e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27-7.3965097402597406e-03*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[50][51]=(0.0)*D;
  A[50][52]=(0.0)*D;
  A[50][53]=(-1.6436688311688311e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-7.1225649350649346e-04*Jinv20*w8-4.5657467532467534e-05*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-1.6436688311688311e-04*w3*Jinv00-2.6298701298701297e-03*w32*Jinv20-7.1225649350649346e-04*Jinv20*w23-1.2327516233766234e-03*Jinv22*w38+3.6982548701298703e-03*Jinv00*w54-3.5612824675324673e-04*w25*Jinv12+3.9265422077922079e-04*w0*Jinv02+2.1367694805194807e-03*w43*Jinv10+1.2327516233766234e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+7.3965097402597406e-03*w48*Jinv00-3.5612824675324673e-04*Jinv02*w24+3.1777597402597403e-03*Jinv22*w35+7.8530844155844158e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10-8.2183441558441553e-05*Jinv22*w20-2.3833198051948050e-03*w45*Jinv00+1.5888798701298701e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+7.3965097402597406e-03*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10+1.0957792207792207e-03*Jinv22*w17+4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.6298701298701297e-03*w31*Jinv10-1.6436688311688311e-04*Jinv02*w3-7.1225649350649346e-04*w21*Jinv00-1.2327516233766234e-03*w32*Jinv22+3.9265422077922079e-04*Jinv22*w2-3.5612824675324673e-04*Jinv00*w18-2.3833198051948050e-03*w36*Jinv00-1.0135957792207792e-03*Jinv02*w45-4.5657467532467534e-05*Jinv22*w59+3.6982548701298703e-03*w51*Jinv00+3.1777597402597403e-03*Jinv12*w34+1.5888798701298701e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv10+4.2735389610389614e-03*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13-3.5612824675324673e-04*w19*Jinv10+4.2735389610389614e-03*w34*Jinv10-7.1225649350649346e-04*w6*Jinv00-1.2327516233766234e-03*w37*Jinv12+3.6982548701298703e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-3.5612824675324673e-04*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+2.4655032467532467e-03*w52*Jinv12-3.5612824675324673e-04*Jinv22*w26-1.6436688311688311e-04*Jinv20*w14+1.2327516233766234e-03*Jinv02*w54-1.6436688311688311e-04*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+7.3965097402597406e-03*w49*Jinv10+4.2735389610389614e-03*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12+3.9265422077922079e-04*Jinv02*w9-7.1225649350649346e-04*w7*Jinv10-2.3833198051948050e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-1.2327516233766234e-03*w31*Jinv12-1.0135957792207792e-03*w46*Jinv12+3.6982548701298703e-03*Jinv20*w53-1.6436688311688311e-04*Jinv22*w8-7.1225649350649346e-04*w22*Jinv10+2.4655032467532467e-03*Jinv22*w50-1.6436688311688311e-04*Jinv12*w7-1.2327516233766234e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49+3.6982548701298703e-03*w52*Jinv10+7.4878246753246752e-04*w9*Jinv00-2.3833198051948050e-03*w37*Jinv10-3.5612824675324673e-04*w24*Jinv00+3.7439123376623376e-04*Jinv02*w27-1.6436688311688311e-04*w6*Jinv02-8.2183441558441553e-05*w19*Jinv12-1.6436688311688311e-04*Jinv22*w5-1.0135957792207792e-03*w47*Jinv22+7.4878246753246752e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44-4.5657467532467534e-05*w58*Jinv12-8.2183441558441553e-05*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+3.1777597402597403e-03*Jinv02*w33+7.8530844155844158e-04*w0*Jinv00+4.2735389610389614e-03*w35*Jinv20-3.5612824675324673e-04*w25*Jinv10+7.8530844155844158e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+2.4655032467532467e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.6298701298701297e-03*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-3.5612824675324673e-04*w20*Jinv20+3.7439123376623376e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14-1.2327516233766234e-03*Jinv02*w36+1.5888798701298701e-03*Jinv12*w43-1.6436688311688311e-04*w5*Jinv20+2.4655032467532467e-03*Jinv02*w51+1.2327516233766234e-03*Jinv22*w56+3.7439123376623376e-04*Jinv12*w28+3.6982548701298703e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+1.0957792207792207e-03*w16*Jinv12+3.9265422077922079e-04*Jinv12*w1)*D;
  A[50][54]=(0.0)*D;
  A[50][55]=(0.0)*D;
  A[50][56]=(3.1777597402597403e-03*w40*Jinv12+1.2327516233766234e-03*Jinv22*w53-4.5657467532467534e-05*w57*Jinv02+3.1777597402597403e-03*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-1.6436688311688311e-04*Jinv01*w3-1.0135957792207792e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59-8.2183441558441553e-05*w25*Jinv12+3.9265422077922079e-04*w0*Jinv02+7.8530844155844158e-04*Jinv21*w2+2.1367694805194807e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.3833198051948050e-03*Jinv01*w45+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w15*Jinv02+3.7439123376623376e-04*w10*Jinv12-8.2183441558441553e-05*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-3.5612824675324673e-04*Jinv21*w26+7.8530844155844158e-04*w0*Jinv01-3.5612824675324673e-04*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11+4.2735389610389614e-03*Jinv21*w41+1.5888798701298701e-03*w42*Jinv02+3.7439123376623376e-04*w11*Jinv22+7.8530844155844158e-04*Jinv11*w1-1.6436688311688311e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11+1.0957792207792207e-03*Jinv22*w17-1.6436688311688311e-04*w22*Jinv12-8.2183441558441553e-05*Jinv02*w3-1.2327516233766234e-03*w32*Jinv22+4.2735389610389614e-03*Jinv01*w39+3.9265422077922079e-04*Jinv22*w2-2.3833198051948050e-03*Jinv21*w38-1.2327516233766234e-03*Jinv02*w45-3.5612824675324673e-04*Jinv01*w24-4.5657467532467534e-05*Jinv22*w59+4.2735389610389614e-03*w34*Jinv11-3.5612824675324673e-04*w19*Jinv11+3.6982548701298703e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+1.5888798701298701e-03*Jinv22*w44-1.6436688311688311e-04*w4*Jinv11-1.6436688311688311e-04*Jinv12*w13-2.6298701298701297e-03*w32*Jinv21-3.5612824675324673e-04*Jinv21*w20-1.0135957792207792e-03*w37*Jinv12+3.1777597402597403e-03*Jinv22*w41+1.2327516233766234e-03*w52*Jinv12+4.2735389610389614e-03*Jinv21*w35-2.6298701298701297e-03*w31*Jinv11-8.2183441558441553e-05*Jinv22*w26-2.3833198051948050e-03*w46*Jinv11-1.6436688311688311e-04*w5*Jinv21+2.4655032467532467e-03*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12-1.6436688311688311e-04*w21*Jinv02+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33-1.6436688311688311e-04*Jinv02*w12-3.5612824675324673e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11+3.7439123376623376e-04*Jinv02*w9-7.1225649350649346e-04*w6*Jinv01-2.3833198051948050e-03*w47*Jinv21-1.6436688311688311e-04*Jinv22*w23+3.6982548701298703e-03*w52*Jinv11-1.2327516233766234e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.4655032467532467e-03*Jinv22*w50-7.1225649350649346e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-7.1225649350649346e-04*w21*Jinv01+3.6982548701298703e-03*Jinv01*w51-1.2327516233766234e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49-2.3833198051948050e-03*w36*Jinv01-7.1225649350649346e-04*w7*Jinv11+3.6982548701298703e-03*Jinv21*w56+3.9265422077922079e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-3.5612824675324673e-04*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+7.3965097402597406e-03*w49*Jinv11-1.6436688311688311e-04*Jinv21*w14-4.5657467532467534e-05*w58*Jinv12-3.5612824675324673e-04*Jinv02*w18-1.6436688311688311e-04*Jinv01*w12-7.1225649350649346e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-7.1225649350649346e-04*Jinv21*w23+3.6982548701298703e-03*Jinv01*w54+3.6982548701298703e-03*w55*Jinv11+2.4655032467532467e-03*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+3.9265422077922079e-04*Jinv22*w29-1.6436688311688311e-04*Jinv22*w14+4.2735389610389614e-03*w40*Jinv11-1.0135957792207792e-03*Jinv02*w36+1.5888798701298701e-03*Jinv12*w43-2.6298701298701297e-03*Jinv01*w30+1.2327516233766234e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28-3.5612824675324673e-04*w25*Jinv11+1.0957792207792207e-03*w16*Jinv12+7.3965097402597406e-03*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27+7.3965097402597406e-03*w48*Jinv01+3.9265422077922079e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[50][57]=(0.0)*D;
  A[50][58]=(0.0)*D;
  A[50][59]=(6.5746753246753243e-04*w40*Jinv12+1.1870941558441558e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-6.7573051948051951e-04*Jinv22*w38-5.4788961038961042e-05*w25*Jinv12+2.0495129870129870e-04*w0*Jinv02+1.1870941558441558e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12-5.4788961038961042e-05*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-5.4788961038961042e-05*Jinv22*w20+3.2873376623376621e-04*w42*Jinv02+1.5726461038961039e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-1.0957792207792208e-04*w22*Jinv12-2.6481331168831169e-04*Jinv02*w3-1.0227272727272728e-03*w32*Jinv22+2.0495129870129870e-04*Jinv22*w2-6.7573051948051951e-04*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+6.5746753246753243e-04*Jinv12*w34+3.2873376623376621e-04*Jinv22*w44-2.6481331168831169e-04*Jinv12*w13-6.7573051948051951e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41+1.1870941558441558e-03*w52*Jinv12-5.4788961038961042e-05*Jinv22*w26+1.1870941558441558e-03*Jinv02*w54-2.6481331168831169e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-2.6481331168831169e-04*Jinv02*w12+1.5726461038961039e-04*Jinv02*w9-1.0957792207792208e-04*Jinv22*w23-1.0227272727272728e-03*w31*Jinv12-6.7573051948051951e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+2.3741883116883116e-03*Jinv22*w50-1.0957792207792208e-04*Jinv12*w7-1.0227272727272728e-03*Jinv02*w30+2.3741883116883116e-03*Jinv12*w49+1.5726461038961039e-04*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-5.4788961038961042e-05*w19*Jinv12-2.6481331168831169e-04*Jinv22*w5-6.7573051948051951e-04*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-5.4788961038961042e-05*Jinv02*w18+6.5746753246753243e-04*Jinv02*w33+2.3741883116883116e-03*w48*Jinv02+1.5726461038961039e-04*Jinv22*w29-2.6481331168831169e-04*Jinv22*w14-6.7573051948051951e-04*Jinv02*w36+3.2873376623376621e-04*Jinv12*w43+1.1870941558441558e-03*Jinv02*w51+1.1870941558441558e-03*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+2.0495129870129870e-04*Jinv12*w1)*D;
  A[51][0]=(5.4788961038961042e-05*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12-1.4975649350649350e-03*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+2.6481331168831169e-04*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23+5.8441558441558442e-04*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-3.4699675324675324e-04*w16*Jinv11-3.4699675324675324e-04*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-7.4878246753246752e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.4699675324675324e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-3.4699675324675324e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-3.4699675324675324e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-3.4699675324675324e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12+2.6481331168831169e-04*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+2.6481331168831169e-04*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+5.8441558441558442e-04*w36*Jinv00+5.8441558441558442e-04*Jinv21*w38+4.2918019480519480e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11-1.4975649350649350e-03*Jinv21*w53-1.4975649350649350e-03*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-2.1915584415584417e-04*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+2.6481331168831169e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00+5.8441558441558442e-04*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41+2.1915584415584417e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20-1.4975649350649350e-03*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+2.6481331168831169e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21-7.4878246753246752e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-2.1915584415584417e-04*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+5.8441558441558442e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+5.8441558441558442e-04*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23-1.4975649350649350e-03*w52*Jinv11+2.6481331168831169e-04*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12-1.4975649350649350e-03*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01-1.4975649350649350e-03*Jinv01*w51+2.6481331168831169e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.8441558441558442e-04*w36*Jinv01-1.4975649350649350e-03*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+5.8441558441558442e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+2.1915584415584417e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+2.1915584415584417e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-3.4699675324675324e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-3.4699675324675324e-04*Jinv20*w17+2.6481331168831169e-04*w30*Jinv00-2.1915584415584417e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11+5.8441558441558442e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+2.6481331168831169e-04*Jinv01*w30-1.4975649350649350e-03*Jinv02*w51-7.4878246753246752e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-3.4699675324675324e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[51][1]=(0.0)*D;
  A[51][2]=(0.0)*D;
  A[51][3]=(1.0957792207792208e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8-1.8262987012987013e-05*w57*Jinv02+7.1225649350649346e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*Jinv01*w3-1.9176136363636362e-04*Jinv20*w23-1.4793019480519481e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-1.8262987012987013e-05*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35-1.8262987012987013e-05*w58*Jinv11-1.8262987012987013e-05*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.2601461038961039e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+2.1915584415584417e-04*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.6570616883116886e-04*w22*Jinv12+1.0683847402597404e-03*Jinv02*w3-1.9176136363636362e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-1.1231737012987013e-03*Jinv00*w18-9.5880681818181818e-04*w36*Jinv00-1.4793019480519481e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24-1.8262987012987013e-05*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-1.3971185064935066e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.4245129870129869e-03*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+1.0683847402597404e-03*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-1.1231737012987013e-03*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-1.3971185064935066e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00-1.4793019480519481e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-3.5612824675324673e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+1.0683847402597404e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+1.0683847402597404e-03*w4*Jinv12-4.6570616883116886e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+2.1915584415584417e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-1.3971185064935066e-03*Jinv01*w18-1.4793019480519481e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21-9.5880681818181818e-04*Jinv20*w38-4.6570616883116886e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+1.4245129870129869e-03*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-1.9176136363636362e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-4.6570616883116886e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-4.6570616883116886e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.4793019480519481e-03*w36*Jinv01+1.4245129870129869e-03*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11-3.5612824675324673e-04*w9*Jinv00-9.5880681818181818e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+1.0683847402597404e-03*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-3.5612824675324673e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44-1.8262987012987013e-05*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20-4.6570616883116886e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.2601461038961039e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+1.2601461038961039e-03*Jinv20*w17+3.2873376623376621e-04*w40*Jinv10-1.1231737012987013e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-1.4793019480519481e-03*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50+2.1915584415584417e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[51][4]=(0.0)*D;
  A[51][5]=(0.0)*D;
  A[51][6]=(-2.7394480519480521e-05*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.1505681818181819e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+4.7483766233766234e-04*w57*Jinv02-2.7394480519480521e-05*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-9.5880681818181818e-04*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3+1.9176136363636362e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54+4.7483766233766234e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+4.7483766233766234e-04*w58*Jinv11+4.7483766233766234e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-1.2053571428571428e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17-6.0267857142857139e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12-3.8352272727272725e-04*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3+1.9176136363636362e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2+1.3697240259740260e-03*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+4.7483766233766234e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.1505681818181819e-03*Jinv21*w53+3.2873376623376621e-04*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-9.5880681818181818e-04*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+1.3697240259740260e-03*w19*Jinv10-6.0267857142857139e-04*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+6.6659902597402602e-04*w11*Jinv20-1.1505681818181819e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-2.7394480519480521e-05*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.3833198051948050e-03*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+1.3971185064935066e-03*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23-1.1505681818181819e-03*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.2873376623376621e-04*Jinv20*w53+2.3833198051948050e-03*Jinv22*w8+1.9176136363636362e-04*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+2.3833198051948050e-03*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-1.1505681818181819e-03*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+3.2873376623376621e-04*w52*Jinv10+2.3833198051948050e-03*w7*Jinv11+6.6659902597402602e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.3833198051948050e-03*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+6.6659902597402602e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+4.7483766233766234e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+2.3833198051948050e-03*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-6.0267857142857139e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2-1.2053571428571428e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-1.2053571428571428e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+1.3697240259740260e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-9.5880681818181818e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30-1.1505681818181819e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[51][7]=(0.0)*D;
  A[51][8]=(0.0)*D;
  A[51][9]=(-2.7394480519480521e-05*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10-6.7573051948051951e-04*Jinv20*w8-2.7394480519480521e-05*w13*Jinv10+4.9310064935064932e-04*w3*Jinv00+1.1870941558441559e-04*w32*Jinv20-2.7394480519480521e-05*Jinv20*w23+3.6525974025974026e-05*Jinv00*w54-3.2873376623376621e-04*w43*Jinv10+8.3198051948051951e-05*w58*Jinv10+3.6525974025974026e-05*w48*Jinv00-1.1668019480519480e-04*w1*Jinv10+2.9220779220779221e-04*w16*Jinv10+1.1870941558441559e-04*w45*Jinv00-1.1668019480519480e-04*Jinv00*w27+3.6525974025974026e-05*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29+1.1870941558441559e-04*w46*Jinv10-3.2873376623376621e-04*Jinv00*w33+1.1870941558441559e-04*w31*Jinv10-2.7394480519480521e-05*w21*Jinv00-6.7573051948051951e-04*Jinv00*w18-1.0227272727272728e-03*w36*Jinv00-2.5568181818181820e-04*w51*Jinv00+4.9310064935064932e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-6.7573051948051951e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-6.7573051948051951e-04*w6*Jinv00+3.6525974025974026e-05*Jinv20*w56+4.9310064935064932e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20-2.7394480519480521e-05*Jinv20*w14+3.6525974025974026e-05*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-6.7573051948051951e-04*w7*Jinv10-1.0227272727272728e-03*Jinv20*w38-2.5568181818181820e-04*Jinv20*w53-2.7394480519480521e-05*w22*Jinv10-2.5568181818181820e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-1.0227272727272728e-03*w37*Jinv10+4.9310064935064932e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44+8.3198051948051951e-05*w57*Jinv00+8.3198051948051951e-05*Jinv20*w59-1.1668019480519480e-04*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+4.9310064935064932e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+2.9220779220779221e-04*w15*Jinv00+2.9220779220779221e-04*Jinv20*w17+1.1870941558441559e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-6.7573051948051951e-04*w20*Jinv20+4.9310064935064932e-04*w5*Jinv20+3.6525974025974026e-05*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+1.1870941558441559e-04*w47*Jinv20)*D;
  A[51][10]=(0.0)*D;
  A[51][11]=(0.0)*D;
  A[51][12]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20-3.5612824675324673e-04*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-5.4788961038961036e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+8.2183441558441553e-04*w16*Jinv11+8.2183441558441553e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+8.7662337662337668e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-1.9176136363636362e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-4.6570616883116886e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+8.7662337662337668e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+8.2183441558441553e-04*w15*Jinv01+2.7394480519480521e-05*w28*Jinv11-1.9176136363636362e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10+8.7662337662337668e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv02*w45-4.6570616883116886e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10-3.5612824675324673e-04*w4*Jinv11+7.1225649350649346e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-3.5612824675324673e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-1.9176136363636362e-04*w49*Jinv10+2.7394480519480521e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+7.1225649350649346e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18+1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10-1.9176136363636362e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+8.2183441558441553e-05*Jinv02*w30-1.9176136363636362e-04*Jinv12*w49+7.1225649350649346e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+1.3697240259740259e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+8.7662337662337668e-04*w15*Jinv00-1.9176136363636362e-04*w48*Jinv02+8.7662337662337668e-04*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00+7.1225649350649346e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-4.6570616883116886e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20+8.7662337662337668e-04*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+2.7394480519480521e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1)*D;
  A[51][13]=(0.0)*D;
  A[51][14]=(0.0)*D;
  A[51][15]=(-8.2183441558441553e-04*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-3.2873376623376621e-03*Jinv22*w53+1.2601461038961039e-03*Jinv20*w8+9.8620129870129864e-04*w57*Jinv02-8.2183441558441553e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-1.2053571428571428e-03*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+2.1367694805194807e-03*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+1.6436688311688311e-04*w25*Jinv12+4.2004870129870131e-04*w0*Jinv02+3.4699675324675324e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+7.1225649350649346e-04*Jinv01*w45-2.1915584415584414e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+1.6436688311688311e-04*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+1.3149350649350649e-03*Jinv21*w26+3.4699675324675324e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10+1.6436688311688311e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+3.8352272727272725e-04*w45*Jinv00+3.9448051948051946e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22+3.4699675324675324e-04*Jinv11*w1-8.2183441558441553e-04*w13*Jinv11-3.4699675324675324e-04*w28*Jinv11+3.6525974025974026e-05*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3+5.4788961038961042e-05*w32*Jinv22+4.2004870129870131e-04*Jinv22*w2+2.5750811688311690e-03*Jinv00*w18+1.7532467532467534e-03*w36*Jinv00+7.6704545454545450e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24+9.8620129870129864e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11-2.1915584415584414e-03*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+3.9448051948051946e-03*Jinv22*w44-1.2053571428571428e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11-7.6704545454545450e-04*Jinv12*w13+2.5750811688311690e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20+1.2601461038961039e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-3.2873376623376621e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-04*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-2.1915584415584414e-03*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-3.4699675324675324e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-7.6704545454545450e-04*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18-2.1915584415584417e-04*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01+1.2601461038961039e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+1.7532467532467534e-03*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12-2.1915584415584414e-03*Jinv20*w53+1.3149350649350649e-03*Jinv22*w8-1.0957792207792207e-03*Jinv22*w50+8.2183441558441553e-04*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7+8.2183441558441553e-04*w21*Jinv01+5.4788961038961042e-05*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49-2.1915584415584414e-03*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+1.7532467532467534e-03*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-8.2183441558441553e-04*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+9.8620129870129864e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-8.2183441558441553e-04*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20+8.2183441558441553e-04*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2+1.6436688311688311e-03*w15*Jinv00-1.0957792207792207e-03*w48*Jinv02+1.6436688311688311e-03*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+2.5750811688311690e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14+2.1367694805194807e-03*Jinv02*w36+3.9448051948051946e-03*Jinv12*w43-1.2053571428571428e-03*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-3.2873376623376621e-03*Jinv02*w51-2.1915584415584414e-03*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50-3.4699675324675324e-04*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01+4.2004870129870131e-04*Jinv12*w1)*D;
  A[51][16]=(0.0)*D;
  A[51][17]=(0.0)*D;
  A[51][18]=(4.6570616883116886e-04*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10-1.0135957792207792e-03*Jinv20*w8+4.6570616883116886e-04*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+8.4922889610389610e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23+3.8352272727272725e-04*Jinv00*w54-4.7483766233766234e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-2.7394480519480520e-03*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-4.8396915584415583e-04*w58*Jinv10+1.9176136363636362e-04*w48*Jinv00-4.7483766233766234e-04*w58*Jinv11-4.7483766233766234e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-3.1960227272727273e-04*w1*Jinv10-2.5202922077922079e-03*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11+1.3697240259740259e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.7349837662337662e-04*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11+1.9176136363636362e-04*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+1.3697240259740259e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+2.4655032467532466e-04*w31*Jinv10+2.4655032467532466e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-2.0271915584415584e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+1.3971185064935066e-03*Jinv01*w24-2.1367694805194807e-03*w34*Jinv11-2.3833198051948050e-03*w19*Jinv11+1.1505681818181819e-03*Jinv21*w53+1.4793019480519481e-03*w51*Jinv00+8.4922889610389610e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-7.1225649350649346e-04*w41*Jinv20-2.0271915584415584e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-2.3833198051948050e-03*Jinv21*w20-1.0135957792207792e-03*w6*Jinv00+3.8352272727272725e-04*Jinv20*w56+4.3831168831168834e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.1367694805194807e-03*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+5.2049512987012989e-04*w46*Jinv11+4.6570616883116886e-04*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+1.9176136363636362e-04*w49*Jinv10-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-7.1225649350649346e-04*w39*Jinv00-2.3833198051948050e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-1.0135957792207792e-03*w7*Jinv10+5.2049512987012989e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+1.1505681818181819e-03*w52*Jinv11+1.4793019480519481e-03*Jinv20*w53+2.4655032467532466e-04*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01+1.1505681818181819e-03*Jinv01*w51-2.6298701298701297e-03*w36*Jinv01+1.4793019480519481e-03*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00-1.6436688311688311e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-2.7394480519480520e-03*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-4.8396915584415583e-04*w57*Jinv00-4.8396915584415583e-04*Jinv20*w59-3.1960227272727273e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-3.1960227272727273e-04*Jinv20*w2-2.5202922077922079e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-2.5202922077922079e-03*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-2.0271915584415584e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+8.4922889610389610e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+1.3971185064935066e-03*w25*Jinv11+3.8352272727272725e-04*w55*Jinv10-2.7394480519480520e-03*w42*Jinv00+1.3697240259740259e-04*w47*Jinv20-1.6436688311688311e-04*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[51][19]=(0.0)*D;
  A[51][20]=(0.0)*D;
  A[51][21]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20+4.6570616883116886e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+7.1225649350649346e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02-2.7394480519480521e-05*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-8.2183441558441553e-04*w16*Jinv11-8.2183441558441553e-04*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+3.5612824675324673e-04*Jinv21*w26-2.7394480519480521e-05*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-3.2873376623376621e-04*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-2.7394480519480521e-05*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11-8.2183441558441553e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-3.2873376623376621e-04*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00+1.6436688311688311e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+4.6570616883116886e-04*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14+4.6570616883116886e-04*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18+1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-2.7394480519480518e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*Jinv12*w7-2.7394480519480518e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+7.1225649350649346e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18-2.7394480519480521e-05*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14-3.2873376623376621e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1)*D;
  A[51][22]=(0.0)*D;
  A[51][23]=(0.0)*D;
  A[51][24]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8+2.7394480519480518e-04*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.4655032467532466e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+1.8262987012987013e-05*Jinv21*w59-2.1915584415584417e-04*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+1.8262987012987013e-05*w58*Jinv11+1.8262987012987013e-05*w57*Jinv01-1.0683847402597404e-03*Jinv21*w26-2.1915584415584417e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-2.1915584415584417e-04*Jinv11*w1+4.6570616883116886e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+5.2049512987012989e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+1.4793019480519481e-03*Jinv21*w38-1.0683847402597404e-03*Jinv01*w24-1.6436688311688311e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00+8.2183441558441553e-05*w4*Jinv10+2.4655032467532466e-04*w4*Jinv11-3.8352272727272725e-04*w41*Jinv20+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.6436688311688311e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+2.4655032467532466e-04*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-3.8352272727272725e-04*w39*Jinv00+8.2183441558441553e-05*Jinv01*w18+1.4793019480519481e-03*w37*Jinv11+1.3971185064935066e-03*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+5.2049512987012989e-04*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53-3.5612824675324673e-04*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.4793019480519481e-03*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+1.3971185064935066e-03*w7*Jinv11-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-7.1225649350649346e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+4.6570616883116886e-04*Jinv21*w14-1.3697240259740259e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+4.6570616883116886e-04*Jinv01*w12+1.3971185064935066e-03*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-7.1225649350649346e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+8.2183441558441553e-05*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30-1.0683847402597404e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[51][25]=(0.0)*D;
  A[51][26]=(0.0)*D;
  A[51][27]=(4.3831168831168834e-04*Jinv21*w44-2.1915584415584417e-04*Jinv01*w3+6.4935064935064935e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+3.4699675324675324e-04*w16*Jinv11+3.4699675324675324e-04*Jinv21*w17-2.6481331168831169e-04*Jinv01*w45+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+4.3831168831168834e-04*w43*Jinv11+2.1915584415584417e-04*Jinv21*w41+1.0247564935064935e-04*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11+3.4699675324675324e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+2.1915584415584417e-04*Jinv01*w39-5.8441558441558442e-04*Jinv21*w38-1.0957792207792208e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11+1.4975649350649350e-03*Jinv21*w53-2.1915584415584417e-04*w4*Jinv11-4.2918019480519480e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-2.6481331168831169e-04*w46*Jinv11-2.1915584415584417e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+1.1870941558441559e-04*Jinv01*w18-5.8441558441558442e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-2.6481331168831169e-04*w47*Jinv21+1.4975649350649350e-03*w52*Jinv11-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01+1.4975649350649350e-03*Jinv01*w51-5.8441558441558442e-04*w36*Jinv01-4.2918019480519480e-04*w7*Jinv11+3.6525974025974026e-05*Jinv21*w56+7.4878246753246752e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-2.7394480519480521e-05*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8-5.4788961038961042e-05*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11-1.4204545454545455e-05*Jinv01*w9+2.1915584415584417e-04*w40*Jinv11-4.2918019480519480e-04*Jinv01*w30-1.0957792207792208e-04*w25*Jinv11+7.4878246753246752e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[51][28]=(0.0)*D;
  A[51][29]=(0.0)*D;
  A[51][30]=(8.2183441558441553e-05*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10+4.3831168831168834e-04*w40*Jinv12+4.0543831168831168e-03*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8+5.5702110389610384e-04*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10+4.3831168831168834e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00-5.7528409090909093e-04*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23-9.5880681818181818e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-4.6570616883116886e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.1047077922077924e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+2.0271915584415584e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00-4.6570616883116886e-04*Jinv02*w24+8.7662337662337668e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+8.2183441558441553e-04*Jinv21*w41+2.5750811688311690e-03*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+1.6436688311688311e-03*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12-5.7528409090909093e-04*w31*Jinv10-5.4788961038961036e-04*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00-7.3965097402597403e-04*w32*Jinv22+8.2183441558441553e-04*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-1.4793019480519481e-03*w36*Jinv00-1.4793019480519481e-03*Jinv21*w38-1.1231737012987013e-03*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24+5.5702110389610384e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+4.7666396103896100e-03*Jinv21*w53+4.7666396103896100e-03*w51*Jinv00+8.7662337662337668e-04*Jinv12*w34+2.5750811688311690e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+8.2183441558441553e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+1.6436688311688311e-03*w34*Jinv10-5.7528409090909093e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-9.5880681818181818e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56+4.3831168831168834e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+4.0543831168831168e-03*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35-5.7528409090909093e-04*w31*Jinv11-4.6570616883116886e-04*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21+2.0271915584415584e-03*Jinv02*w54-5.4788961038961036e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+8.2183441558441553e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-1.4793019480519481e-03*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.4793019480519481e-03*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23+4.7666396103896100e-03*w52*Jinv11-7.3965097402597403e-04*w31*Jinv12-1.1231737012987013e-03*w46*Jinv12+4.7666396103896100e-03*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01+4.7666396103896100e-03*Jinv01*w51-7.3965097402597403e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-1.4793019480519481e-03*w36*Jinv01+4.7666396103896100e-03*w52*Jinv10-1.3697240259740259e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-1.4793019480519481e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+3.1960227272727273e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12-5.4788961038961036e-04*Jinv22*w5-1.1231737012987013e-03*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+5.5702110389610384e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+8.7662337662337668e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+1.6436688311688311e-03*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02+3.1047077922077924e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17-5.7528409090909093e-04*w30*Jinv00+8.2183441558441553e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20+3.1960227272727273e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+8.2183441558441553e-04*w40*Jinv11-9.5880681818181818e-04*Jinv02*w36+2.5750811688311690e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-5.7528409090909093e-04*Jinv01*w30+4.0543831168831168e-03*Jinv02*w51+2.0271915584415584e-03*Jinv22*w56+3.1960227272727273e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+3.1047077922077924e-04*w10*Jinv11)*D;
  A[51][31]=(0.0)*D;
  A[51][32]=(0.0)*D;
  A[51][33]=(-3.2873376623376621e-04*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12-9.5332792207792200e-03*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-2.9586038961038961e-03*Jinv21*w44-6.0267857142857139e-04*w3*Jinv00+4.9310064935064932e-04*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+2.7942370129870131e-03*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54+1.0957792207792207e-03*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02-5.4788961038961036e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-2.9586038961038961e-03*w42*Jinv01-6.5746753246753243e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17+2.6298701298701297e-03*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+1.3149350649350649e-03*Jinv02*w24-6.9034090909090911e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26-5.4788961038961036e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-2.9586038961038961e-03*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22-5.4788961038961036e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01-5.4788961038961036e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+8.2183441558441553e-04*w22*Jinv12+4.9310064935064932e-04*w31*Jinv10+1.8628246753246754e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2+1.7532467532467534e-03*Jinv00*w18+2.7942370129870131e-03*w36*Jinv00+4.9310064935064934e-03*Jinv21*w38+2.5750811688311690e-03*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-1.2820616883116883e-02*Jinv21*w53-8.5470779220779228e-03*w51*Jinv00-6.9034090909090911e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44-6.0267857142857139e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+1.7532467532467534e-03*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+2.6298701298701297e-03*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+2.7942370129870131e-03*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.6436688311688311e-04*w11*Jinv20-9.5332792207792200e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35+2.6298701298701297e-03*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+2.6298701298701297e-03*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+8.2183441558441553e-04*w21*Jinv02-5.4788961038961036e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33+2.1367694805194807e-03*Jinv01*w18+4.9310064935064934e-03*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+2.6298701298701297e-03*w47*Jinv21+2.7942370129870131e-03*Jinv20*w38+8.2183441558441553e-04*Jinv22*w23-1.2820616883116883e-02*w52*Jinv11+1.8628246753246754e-03*w31*Jinv12+2.5750811688311690e-03*w46*Jinv12-8.5470779220779228e-03*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-2.0819805194805196e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-1.2820616883116883e-02*Jinv01*w51+1.8628246753246754e-03*Jinv02*w30-2.0819805194805196e-03*Jinv12*w49+4.9310064935064934e-03*w36*Jinv01-8.5470779220779228e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-1.6436688311688311e-04*w9*Jinv00+2.7942370129870131e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-9.6793831168831166e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12+2.5750811688311690e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-1.6436688311688311e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-6.9034090909090911e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-2.0819805194805196e-03*w48*Jinv02-6.5746753246753243e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17+4.9310064935064932e-04*w30*Jinv00+1.7532467532467534e-03*w20*Jinv20-9.6793831168831166e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+2.7942370129870131e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43-6.0267857142857139e-04*w5*Jinv20+2.6298701298701297e-03*Jinv01*w30-9.5332792207792200e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-9.6793831168831166e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-5.4788961038961036e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-6.2094155844155847e-04*Jinv12*w1-6.5746753246753243e-04*w10*Jinv11)*D;
  A[51][34]=(0.0)*D;
  A[51][35]=(0.0)*D;
  A[51][36]=(2.4655032467532466e-04*Jinv00*w12-1.5523538961038962e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-4.7483766233766234e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+4.3831168831168834e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20+2.4655032467532466e-04*Jinv20*w23-2.6298701298701297e-03*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-6.5746753246753243e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-1.5523538961038962e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+8.2183441558441553e-05*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02-1.5523538961038962e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-2.4655032467532466e-04*Jinv20*w50-1.5523538961038962e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+5.2049512987012989e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00+5.2049512987012989e-04*Jinv02*w45-4.7483766233766234e-04*Jinv22*w59+3.7804383116883116e-03*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10-8.2183441558441553e-04*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.2327516233766234e-03*w6*Jinv00-2.6298701298701297e-03*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+4.3831168831168834e-04*w26*Jinv20-1.4610389610389610e-04*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+2.4655032467532466e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10-8.2183441558441553e-04*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.2327516233766234e-03*w7*Jinv10-1.3149350649350649e-03*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+5.2049512987012989e-04*w31*Jinv12+5.2049512987012989e-04*w46*Jinv12+3.7804383116883116e-03*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+5.2049512987012989e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+3.7804383116883116e-03*w52*Jinv10-1.4610389610389610e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10+4.3831168831168834e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5+5.2049512987012989e-04*w47*Jinv22-1.4610389610389610e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-4.7483766233766234e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-1.5523538961038962e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+4.3831168831168834e-04*w25*Jinv10-1.5523538961038962e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00-8.2183441558441553e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-2.6298701298701297e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20+1.1505681818181819e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28-2.4655032467532466e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[51][37]=(0.0)*D;
  A[51][38]=(0.0)*D;
  A[51][39]=(5.4788961038961042e-05*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+1.7532467532467534e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+7.1225649350649346e-04*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+8.2183441558441553e-04*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+7.1225649350649346e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-3.2873376623376621e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-2.1915584415584417e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10+3.8352272727272725e-04*w21*Jinv00+9.3141233766233771e-04*w32*Jinv22-3.1047077922077924e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+1.7532467532467534e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+7.6704545454545450e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-8.2183441558441553e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+1.7532467532467534e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-8.2183441558441553e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+8.2183441558441553e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+3.8352272727272725e-04*Jinv01*w18-5.2962662337662338e-04*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+8.2183441558441553e-04*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23+9.3141233766233771e-04*w31*Jinv12+1.7532467532467534e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-1.0409902597402598e-03*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01+9.3141233766233771e-04*Jinv02*w30-1.0409902597402598e-03*Jinv12*w49-4.2735389610389614e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.7532467532467534e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-1.0409902597402598e-03*w48*Jinv02+1.3149350649350649e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29+1.7532467532467534e-03*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-8.2183441558441553e-04*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1)*D;
  A[51][40]=(0.0)*D;
  A[51][41]=(0.0)*D;
  A[51][42]=(-5.4788961038961042e-05*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+3.2873376623376621e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-9.8620129870129864e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-4.2004870129870131e-04*w0*Jinv02+5.4788961038961036e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+6.5746753246753243e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17-2.6298701298701297e-03*Jinv01*w45+2.1915584415584414e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-4.9310064935064934e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26+5.4788961038961036e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+5.4788961038961036e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01+5.4788961038961036e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-4.9310064935064934e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10+1.1505681818181819e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-4.2004870129870131e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-4.9310064935064934e-03*Jinv21*w38-7.6704545454545450e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-9.8620129870129864e-04*Jinv22*w59+2.9586038961038961e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+1.2820616883116883e-02*Jinv21*w53+4.2735389610389614e-03*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-3.9448051948051946e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-2.6298701298701297e-03*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+3.2873376623376621e-03*w52*Jinv12+2.9586038961038961e-03*Jinv21*w35-2.6298701298701297e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-2.6298701298701297e-03*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+2.1915584415584414e-03*Jinv02*w54+1.1505681818181819e-03*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+5.4788961038961036e-04*Jinv21*w29+2.9586038961038961e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.9310064935064934e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-2.6298701298701297e-03*w47*Jinv21-2.1367694805194807e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23+1.2820616883116883e-02*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+1.2820616883116883e-02*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-4.9310064935064934e-03*w36*Jinv01+4.2735389610389614e-03*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10+4.2735389610389614e-03*Jinv21*w56-7.6704545454545450e-04*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-1.3149350649350649e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+1.1505681818181819e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-9.8620129870129864e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11-7.6704545454545450e-04*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+6.5746753246753243e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-2.6298701298701297e-03*Jinv01*w30+3.2873376623376621e-03*Jinv02*w51+2.1915584415584414e-03*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11+4.2735389610389614e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-4.9310064935064934e-03*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+5.4788961038961036e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01-4.2004870129870131e-04*Jinv12*w1+6.5746753246753243e-04*w10*Jinv11)*D;
  A[51][43]=(0.0)*D;
  A[51][44]=(0.0)*D;
  A[51][45]=(-3.8352272727272725e-04*w40*Jinv12-7.1225649350649346e-04*Jinv22*w53+9.1314935064935065e-06*w57*Jinv02-3.8352272727272725e-04*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3+5.2049512987012989e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-1.6436688311688311e-03*w42*Jinv01-3.1047077922077924e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17+5.7528409090909093e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-5.4788961038961042e-05*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+5.2049512987012989e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-8.2183441558441553e-04*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11-5.4788961038961042e-05*Jinv22*w17-3.5612824675324673e-04*w22*Jinv12+8.2183441558441553e-05*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-8.2183441558441553e-04*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2+1.4793019480519481e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24+9.1314935064935065e-06*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11-4.7666396103896100e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv11+2.7394480519480518e-04*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+5.2049512987012989e-04*w37*Jinv12-3.8352272727272725e-04*Jinv22*w41-7.1225649350649346e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26+5.7528409090909093e-04*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+2.7394480519480518e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+1.4793019480519481e-03*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+5.7528409090909093e-04*w47*Jinv21-3.5612824675324673e-04*Jinv22*w23-4.7666396103896100e-03*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01-4.7666396103896100e-03*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49+1.4793019480519481e-03*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14+9.1314935064935065e-06*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+2.4655032467532466e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-8.2183441558441553e-05*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02-3.1047077922077924e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-8.2183441558441553e-04*w40*Jinv11+5.2049512987012989e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-7.1225649350649346e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-5.4788961038961042e-05*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1-3.1047077922077924e-04*w10*Jinv11)*D;
  A[51][46]=(0.0)*D;
  A[51][47]=(0.0)*D;
  A[51][48]=(3.5612824675324673e-04*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10-5.4788961038961036e-04*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8-1.1414366883116883e-03*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-5.4788961038961036e-04*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+2.3833198051948050e-03*w32*Jinv20+7.1225649350649346e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23+1.3971185064935066e-03*Jinv22*w38-3.6982548701298703e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-3.5612824675324673e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-7.8530844155844158e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.3833198051948050e-03*Jinv01*w45-2.4655032467532467e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-3.9265422077922079e-04*w10*Jinv12-3.6982548701298703e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10+8.2183441558441553e-05*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-2.1367694805194807e-03*Jinv21*w41-2.6846590909090908e-03*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.6982548701298703e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10-4.2735389610389614e-03*Jinv00*w33+2.3833198051948050e-03*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00+1.1505681818181819e-03*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-3.5612824675324673e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+2.6298701298701297e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-1.1414366883116883e-03*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-7.3965097402597406e-03*Jinv21*w53-7.3965097402597406e-03*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-2.6846590909090908e-03*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-4.2735389610389614e-03*w34*Jinv10+2.3833198051948050e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*w37*Jinv12-3.6982548701298703e-03*Jinv20*w56-5.4788961038961036e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-7.8530844155844158e-04*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+2.3833198051948050e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+2.3833198051948050e-03*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+7.1225649350649346e-04*w5*Jinv21-2.4655032467532467e-03*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-3.6982548701298703e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10+2.3833198051948050e-03*w47*Jinv21+2.6298701298701297e-03*Jinv20*w38-7.3965097402597406e-03*w52*Jinv11+1.1505681818181819e-03*w31*Jinv12+1.3697240259740260e-03*w46*Jinv12-7.3965097402597406e-03*Jinv20*w53+3.5612824675324673e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+3.5612824675324673e-04*w21*Jinv01-7.3965097402597406e-03*Jinv01*w51+1.1505681818181819e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01-7.3965097402597406e-03*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-7.8530844155844158e-04*w9*Jinv00+2.6298701298701297e-03*w37*Jinv10-3.6982548701298703e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-3.7439123376623376e-04*Jinv02*w27+8.2183441558441553e-05*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22-3.6982548701298703e-03*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-7.8530844155844158e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-1.1414366883116883e-03*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-4.2735389610389614e-03*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-3.6982548701298703e-03*Jinv01*w54-3.6982548701298703e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-7.8530844155844158e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.3833198051948050e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-3.7439123376623376e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+1.3971185064935066e-03*Jinv02*w36-2.6846590909090908e-03*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+2.3833198051948050e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-2.4655032467532467e-03*Jinv22*w56-3.7439123376623376e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-3.6982548701298703e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20-3.6982548701298703e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27-3.6982548701298703e-03*w48*Jinv01-3.5612824675324673e-04*Jinv12*w1-7.8530844155844158e-04*w10*Jinv11)*D;
  A[51][49]=(0.0)*D;
  A[51][50]=(0.0)*D;
  A[51][51]=(-3.5612824675324673e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+7.3965097402597406e-03*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-9.1314935064935068e-05*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-2.3833198051948050e-03*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38+3.6982548701298703e-03*Jinv00*w54-1.0683847402597404e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+4.2735389610389614e-03*w43*Jinv10+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+6.5746753246753243e-04*w10*Jinv12+3.6982548701298703e-03*w48*Jinv00-1.0683847402597404e-03*Jinv02*w24+4.7666396103896100e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10+5.7528409090909093e-04*Jinv22*w20-2.3833198051948050e-03*w45*Jinv00+4.7666396103896100e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+6.5746753246753243e-04*w11*Jinv22+3.6982548701298703e-03*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.3833198051948050e-03*w31*Jinv10-1.0683847402597404e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-2.0271915584415584e-03*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-2.6298701298701297e-03*w36*Jinv00-2.0271915584415584e-03*Jinv02*w45-9.1314935064935068e-05*Jinv22*w59+7.3965097402597406e-03*w51*Jinv00+4.7666396103896100e-03*Jinv12*w34+4.7666396103896100e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10+2.1367694805194807e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10+4.2735389610389614e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12+3.6982548701298703e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20+7.8530844155844158e-04*w11*Jinv20+7.3965097402597406e-03*w52*Jinv12-1.0683847402597404e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14+2.4655032467532467e-03*Jinv02*w54-1.0683847402597404e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+3.6982548701298703e-03*w49*Jinv10+2.1367694805194807e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+6.5746753246753243e-04*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10-2.6298701298701297e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-2.0271915584415584e-03*w31*Jinv12-2.0271915584415584e-03*w46*Jinv12+7.3965097402597406e-03*Jinv20*w53+5.7528409090909093e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+2.4655032467532467e-03*Jinv22*w50+5.7528409090909093e-04*Jinv12*w7-2.0271915584415584e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49+7.3965097402597406e-03*w52*Jinv10+7.8530844155844158e-04*w9*Jinv00-2.6298701298701297e-03*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+5.7528409090909093e-04*w6*Jinv02+5.7528409090909093e-04*w19*Jinv12-1.0683847402597404e-03*Jinv22*w5-2.0271915584415584e-03*w47*Jinv22+7.8530844155844158e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-9.1314935064935068e-05*w58*Jinv12+5.7528409090909093e-04*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+4.7666396103896100e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+4.2735389610389614e-03*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+2.4655032467532467e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.3833198051948050e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-1.3149350649350649e-03*Jinv02*w36+4.7666396103896100e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20+7.3965097402597406e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28+3.6982548701298703e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[51][52]=(0.0)*D;
  A[51][53]=(0.0)*D;
  A[51][54]=(1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-4.5657467532467534e-05*w57*Jinv02+1.5888798701298701e-03*Jinv02*w39+4.2735389610389614e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-1.2327516233766234e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12+3.7439123376623376e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w42*Jinv01+7.8530844155844158e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.3833198051948050e-03*Jinv01*w45+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-7.1225649350649346e-04*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20+4.2735389610389614e-03*w43*Jinv11+2.1367694805194807e-03*Jinv21*w41+3.1777597402597403e-03*w42*Jinv02+3.9265422077922079e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11+1.0957792207792207e-03*Jinv22*w17-8.2183441558441553e-05*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-1.0135957792207792e-03*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39+3.7439123376623376e-04*Jinv22*w2-2.6298701298701297e-03*Jinv21*w38-1.2327516233766234e-03*Jinv02*w45-7.1225649350649346e-04*Jinv01*w24-4.5657467532467534e-05*Jinv22*w59+4.2735389610389614e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+7.3965097402597406e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+3.1777597402597403e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-3.5612824675324673e-04*Jinv12*w13-2.3833198051948050e-03*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-1.2327516233766234e-03*w37*Jinv12+1.5888798701298701e-03*Jinv22*w41+2.4655032467532467e-03*w52*Jinv12+4.2735389610389614e-03*Jinv21*w35-2.3833198051948050e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-2.3833198051948050e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+2.4655032467532467e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33-3.5612824675324673e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+3.9265422077922079e-04*Jinv02*w9-1.6436688311688311e-04*w6*Jinv01-2.3833198051948050e-03*w47*Jinv21-8.2183441558441553e-05*Jinv22*w23+7.3965097402597406e-03*w52*Jinv11-1.0135957792207792e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12-8.2183441558441553e-05*Jinv22*w8+1.2327516233766234e-03*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+7.3965097402597406e-03*Jinv01*w51-1.0135957792207792e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-1.6436688311688311e-04*w7*Jinv11+3.6982548701298703e-03*Jinv21*w56+3.9265422077922079e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+3.6982548701298703e-03*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14-4.5657467532467534e-05*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+3.6982548701298703e-03*Jinv01*w54+3.6982548701298703e-03*w55*Jinv11+1.2327516233766234e-03*w48*Jinv02+7.8530844155844158e-04*Jinv01*w9+3.9265422077922079e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-1.2327516233766234e-03*Jinv02*w36+3.1777597402597403e-03*Jinv12*w43-2.3833198051948050e-03*Jinv01*w30+2.4655032467532467e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28-7.1225649350649346e-04*w25*Jinv11+1.0957792207792207e-03*w16*Jinv12+3.6982548701298703e-03*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27+3.6982548701298703e-03*w48*Jinv01+3.7439123376623376e-04*Jinv12*w1+7.8530844155844158e-04*w10*Jinv11)*D;
  A[51][55]=(0.0)*D;
  A[51][56]=(0.0)*D;
  A[51][57]=(3.2873376623376621e-04*w40*Jinv12+2.3741883116883116e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+3.2873376623376621e-04*Jinv02*w39-1.0227272727272728e-03*Jinv22*w38-1.0957792207792208e-04*w25*Jinv12+1.5726461038961039e-04*w0*Jinv02+1.1870941558441558e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+2.0495129870129870e-04*w10*Jinv12-1.0957792207792208e-04*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-2.6481331168831169e-04*Jinv22*w20+6.5746753246753243e-04*w42*Jinv02+2.0495129870129870e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-5.4788961038961042e-05*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-6.7573051948051951e-04*w32*Jinv22+1.5726461038961039e-04*Jinv22*w2-6.7573051948051951e-04*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+6.5746753246753243e-04*Jinv12*w34+6.5746753246753243e-04*Jinv22*w44-5.4788961038961042e-05*Jinv12*w13-1.0227272727272728e-03*w37*Jinv12+3.2873376623376621e-04*Jinv22*w41+2.3741883116883116e-03*w52*Jinv12-1.0957792207792208e-04*Jinv22*w26+1.1870941558441558e-03*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-5.4788961038961042e-05*Jinv02*w12+2.0495129870129870e-04*Jinv02*w9-5.4788961038961042e-05*Jinv22*w23-6.7573051948051951e-04*w31*Jinv12-6.7573051948051951e-04*w46*Jinv12-2.6481331168831169e-04*Jinv22*w8+1.1870941558441558e-03*Jinv22*w50-2.6481331168831169e-04*Jinv12*w7-6.7573051948051951e-04*Jinv02*w30+1.1870941558441558e-03*Jinv12*w49+1.5726461038961039e-04*Jinv02*w27-2.6481331168831169e-04*w6*Jinv02-2.6481331168831169e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5-6.7573051948051951e-04*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-2.6481331168831169e-04*Jinv02*w18+6.5746753246753243e-04*Jinv02*w33+1.1870941558441558e-03*w48*Jinv02+1.5726461038961039e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.0227272727272728e-03*Jinv02*w36+6.5746753246753243e-04*Jinv12*w43+2.3741883116883116e-03*Jinv02*w51+1.1870941558441558e-03*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+1.5726461038961039e-04*Jinv12*w1)*D;
  A[51][58]=(0.0)*D;
  A[51][59]=(0.0)*D;
  A[52][0]=(0.0)*D;
  A[52][1]=(5.4788961038961042e-05*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12-1.4975649350649350e-03*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+2.6481331168831169e-04*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23+5.8441558441558442e-04*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-3.4699675324675324e-04*w16*Jinv11-3.4699675324675324e-04*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-7.4878246753246752e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.4699675324675324e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-3.4699675324675324e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-3.4699675324675324e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-3.4699675324675324e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12+2.6481331168831169e-04*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+2.6481331168831169e-04*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+5.8441558441558442e-04*w36*Jinv00+5.8441558441558442e-04*Jinv21*w38+4.2918019480519480e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11-1.4975649350649350e-03*Jinv21*w53-1.4975649350649350e-03*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-2.1915584415584417e-04*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+2.6481331168831169e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00+5.8441558441558442e-04*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41+2.1915584415584417e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20-1.4975649350649350e-03*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+2.6481331168831169e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21-7.4878246753246752e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-2.1915584415584417e-04*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+5.8441558441558442e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+5.8441558441558442e-04*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23-1.4975649350649350e-03*w52*Jinv11+2.6481331168831169e-04*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12-1.4975649350649350e-03*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01-1.4975649350649350e-03*Jinv01*w51+2.6481331168831169e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.8441558441558442e-04*w36*Jinv01-1.4975649350649350e-03*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+5.8441558441558442e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+2.1915584415584417e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+2.1915584415584417e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-3.4699675324675324e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-3.4699675324675324e-04*Jinv20*w17+2.6481331168831169e-04*w30*Jinv00-2.1915584415584417e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11+5.8441558441558442e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+2.6481331168831169e-04*Jinv01*w30-1.4975649350649350e-03*Jinv02*w51-7.4878246753246752e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-3.4699675324675324e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[52][2]=(0.0)*D;
  A[52][3]=(0.0)*D;
  A[52][4]=(1.0957792207792208e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8-1.8262987012987013e-05*w57*Jinv02+7.1225649350649346e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*Jinv01*w3-1.9176136363636362e-04*Jinv20*w23-1.4793019480519481e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-1.8262987012987013e-05*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35-1.8262987012987013e-05*w58*Jinv11-1.8262987012987013e-05*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.2601461038961039e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+2.1915584415584417e-04*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.6570616883116886e-04*w22*Jinv12+1.0683847402597404e-03*Jinv02*w3-1.9176136363636362e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-1.1231737012987013e-03*Jinv00*w18-9.5880681818181818e-04*w36*Jinv00-1.4793019480519481e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24-1.8262987012987013e-05*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-1.3971185064935066e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.4245129870129869e-03*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+1.0683847402597404e-03*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-1.1231737012987013e-03*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-1.3971185064935066e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00-1.4793019480519481e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-3.5612824675324673e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+1.0683847402597404e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+1.0683847402597404e-03*w4*Jinv12-4.6570616883116886e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+2.1915584415584417e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-1.3971185064935066e-03*Jinv01*w18-1.4793019480519481e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21-9.5880681818181818e-04*Jinv20*w38-4.6570616883116886e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+1.4245129870129869e-03*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-1.9176136363636362e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-4.6570616883116886e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-4.6570616883116886e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.4793019480519481e-03*w36*Jinv01+1.4245129870129869e-03*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11-3.5612824675324673e-04*w9*Jinv00-9.5880681818181818e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+1.0683847402597404e-03*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-3.5612824675324673e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44-1.8262987012987013e-05*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20-4.6570616883116886e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.2601461038961039e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+1.2601461038961039e-03*Jinv20*w17+3.2873376623376621e-04*w40*Jinv10-1.1231737012987013e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-1.4793019480519481e-03*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50+2.1915584415584417e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[52][5]=(0.0)*D;
  A[52][6]=(0.0)*D;
  A[52][7]=(-2.7394480519480521e-05*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.1505681818181819e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+4.7483766233766234e-04*w57*Jinv02-2.7394480519480521e-05*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-9.5880681818181818e-04*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3+1.9176136363636362e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54+4.7483766233766234e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+4.7483766233766234e-04*w58*Jinv11+4.7483766233766234e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-1.2053571428571428e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17-6.0267857142857139e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12-3.8352272727272725e-04*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3+1.9176136363636362e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2+1.3697240259740260e-03*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+4.7483766233766234e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.1505681818181819e-03*Jinv21*w53+3.2873376623376621e-04*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-9.5880681818181818e-04*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+1.3697240259740260e-03*w19*Jinv10-6.0267857142857139e-04*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+6.6659902597402602e-04*w11*Jinv20-1.1505681818181819e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-2.7394480519480521e-05*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.3833198051948050e-03*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+1.3971185064935066e-03*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23-1.1505681818181819e-03*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.2873376623376621e-04*Jinv20*w53+2.3833198051948050e-03*Jinv22*w8+1.9176136363636362e-04*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+2.3833198051948050e-03*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-1.1505681818181819e-03*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+3.2873376623376621e-04*w52*Jinv10+2.3833198051948050e-03*w7*Jinv11+6.6659902597402602e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.3833198051948050e-03*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+6.6659902597402602e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+4.7483766233766234e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+2.3833198051948050e-03*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-6.0267857142857139e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2-1.2053571428571428e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-1.2053571428571428e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+1.3697240259740260e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-9.5880681818181818e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30-1.1505681818181819e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[52][8]=(0.0)*D;
  A[52][9]=(0.0)*D;
  A[52][10]=(-2.7394480519480521e-05*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10-6.7573051948051951e-04*Jinv20*w8-2.7394480519480521e-05*w13*Jinv10+4.9310064935064932e-04*w3*Jinv00+1.1870941558441559e-04*w32*Jinv20-2.7394480519480521e-05*Jinv20*w23+3.6525974025974026e-05*Jinv00*w54-3.2873376623376621e-04*w43*Jinv10+8.3198051948051951e-05*w58*Jinv10+3.6525974025974026e-05*w48*Jinv00-1.1668019480519480e-04*w1*Jinv10+2.9220779220779221e-04*w16*Jinv10+1.1870941558441559e-04*w45*Jinv00-1.1668019480519480e-04*Jinv00*w27+3.6525974025974026e-05*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29+1.1870941558441559e-04*w46*Jinv10-3.2873376623376621e-04*Jinv00*w33+1.1870941558441559e-04*w31*Jinv10-2.7394480519480521e-05*w21*Jinv00-6.7573051948051951e-04*Jinv00*w18-1.0227272727272728e-03*w36*Jinv00-2.5568181818181820e-04*w51*Jinv00+4.9310064935064932e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-6.7573051948051951e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-6.7573051948051951e-04*w6*Jinv00+3.6525974025974026e-05*Jinv20*w56+4.9310064935064932e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20-2.7394480519480521e-05*Jinv20*w14+3.6525974025974026e-05*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-6.7573051948051951e-04*w7*Jinv10-1.0227272727272728e-03*Jinv20*w38-2.5568181818181820e-04*Jinv20*w53-2.7394480519480521e-05*w22*Jinv10-2.5568181818181820e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-1.0227272727272728e-03*w37*Jinv10+4.9310064935064932e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44+8.3198051948051951e-05*w57*Jinv00+8.3198051948051951e-05*Jinv20*w59-1.1668019480519480e-04*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+4.9310064935064932e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+2.9220779220779221e-04*w15*Jinv00+2.9220779220779221e-04*Jinv20*w17+1.1870941558441559e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-6.7573051948051951e-04*w20*Jinv20+4.9310064935064932e-04*w5*Jinv20+3.6525974025974026e-05*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+1.1870941558441559e-04*w47*Jinv20)*D;
  A[52][11]=(0.0)*D;
  A[52][12]=(0.0)*D;
  A[52][13]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20-3.5612824675324673e-04*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-5.4788961038961036e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+8.2183441558441553e-04*w16*Jinv11+8.2183441558441553e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+8.7662337662337668e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-1.9176136363636362e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-4.6570616883116886e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+8.7662337662337668e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+8.2183441558441553e-04*w15*Jinv01+2.7394480519480521e-05*w28*Jinv11-1.9176136363636362e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10+8.7662337662337668e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv02*w45-4.6570616883116886e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10-3.5612824675324673e-04*w4*Jinv11+7.1225649350649346e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-3.5612824675324673e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-1.9176136363636362e-04*w49*Jinv10+2.7394480519480521e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+7.1225649350649346e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18+1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10-1.9176136363636362e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+8.2183441558441553e-05*Jinv02*w30-1.9176136363636362e-04*Jinv12*w49+7.1225649350649346e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+1.3697240259740259e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+8.7662337662337668e-04*w15*Jinv00-1.9176136363636362e-04*w48*Jinv02+8.7662337662337668e-04*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00+7.1225649350649346e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-4.6570616883116886e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20+8.7662337662337668e-04*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+2.7394480519480521e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1)*D;
  A[52][14]=(0.0)*D;
  A[52][15]=(0.0)*D;
  A[52][16]=(-8.2183441558441553e-04*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-3.2873376623376621e-03*Jinv22*w53+1.2601461038961039e-03*Jinv20*w8+9.8620129870129864e-04*w57*Jinv02-8.2183441558441553e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-1.2053571428571428e-03*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+2.1367694805194807e-03*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+1.6436688311688311e-04*w25*Jinv12+4.2004870129870131e-04*w0*Jinv02+3.4699675324675324e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+7.1225649350649346e-04*Jinv01*w45-2.1915584415584414e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+1.6436688311688311e-04*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+1.3149350649350649e-03*Jinv21*w26+3.4699675324675324e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10+1.6436688311688311e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+3.8352272727272725e-04*w45*Jinv00+3.9448051948051946e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22+3.4699675324675324e-04*Jinv11*w1-8.2183441558441553e-04*w13*Jinv11-3.4699675324675324e-04*w28*Jinv11+3.6525974025974026e-05*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3+5.4788961038961042e-05*w32*Jinv22+4.2004870129870131e-04*Jinv22*w2+2.5750811688311690e-03*Jinv00*w18+1.7532467532467534e-03*w36*Jinv00+7.6704545454545450e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24+9.8620129870129864e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11-2.1915584415584414e-03*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+3.9448051948051946e-03*Jinv22*w44-1.2053571428571428e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11-7.6704545454545450e-04*Jinv12*w13+2.5750811688311690e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20+1.2601461038961039e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-3.2873376623376621e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-04*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-2.1915584415584414e-03*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-3.4699675324675324e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-7.6704545454545450e-04*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18-2.1915584415584417e-04*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01+1.2601461038961039e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+1.7532467532467534e-03*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12-2.1915584415584414e-03*Jinv20*w53+1.3149350649350649e-03*Jinv22*w8-1.0957792207792207e-03*Jinv22*w50+8.2183441558441553e-04*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7+8.2183441558441553e-04*w21*Jinv01+5.4788961038961042e-05*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49-2.1915584415584414e-03*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+1.7532467532467534e-03*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-8.2183441558441553e-04*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+9.8620129870129864e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-8.2183441558441553e-04*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20+8.2183441558441553e-04*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2+1.6436688311688311e-03*w15*Jinv00-1.0957792207792207e-03*w48*Jinv02+1.6436688311688311e-03*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+2.5750811688311690e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14+2.1367694805194807e-03*Jinv02*w36+3.9448051948051946e-03*Jinv12*w43-1.2053571428571428e-03*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-3.2873376623376621e-03*Jinv02*w51-2.1915584415584414e-03*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50-3.4699675324675324e-04*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01+4.2004870129870131e-04*Jinv12*w1)*D;
  A[52][17]=(0.0)*D;
  A[52][18]=(0.0)*D;
  A[52][19]=(4.6570616883116886e-04*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10-1.0135957792207792e-03*Jinv20*w8+4.6570616883116886e-04*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+8.4922889610389610e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23+3.8352272727272725e-04*Jinv00*w54-4.7483766233766234e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-2.7394480519480520e-03*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-4.8396915584415583e-04*w58*Jinv10+1.9176136363636362e-04*w48*Jinv00-4.7483766233766234e-04*w58*Jinv11-4.7483766233766234e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-3.1960227272727273e-04*w1*Jinv10-2.5202922077922079e-03*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11+1.3697240259740259e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.7349837662337662e-04*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11+1.9176136363636362e-04*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+1.3697240259740259e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+2.4655032467532466e-04*w31*Jinv10+2.4655032467532466e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-2.0271915584415584e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+1.3971185064935066e-03*Jinv01*w24-2.1367694805194807e-03*w34*Jinv11-2.3833198051948050e-03*w19*Jinv11+1.1505681818181819e-03*Jinv21*w53+1.4793019480519481e-03*w51*Jinv00+8.4922889610389610e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-7.1225649350649346e-04*w41*Jinv20-2.0271915584415584e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-2.3833198051948050e-03*Jinv21*w20-1.0135957792207792e-03*w6*Jinv00+3.8352272727272725e-04*Jinv20*w56+4.3831168831168834e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.1367694805194807e-03*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+5.2049512987012989e-04*w46*Jinv11+4.6570616883116886e-04*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+1.9176136363636362e-04*w49*Jinv10-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-7.1225649350649346e-04*w39*Jinv00-2.3833198051948050e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-1.0135957792207792e-03*w7*Jinv10+5.2049512987012989e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+1.1505681818181819e-03*w52*Jinv11+1.4793019480519481e-03*Jinv20*w53+2.4655032467532466e-04*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01+1.1505681818181819e-03*Jinv01*w51-2.6298701298701297e-03*w36*Jinv01+1.4793019480519481e-03*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00-1.6436688311688311e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-2.7394480519480520e-03*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-4.8396915584415583e-04*w57*Jinv00-4.8396915584415583e-04*Jinv20*w59-3.1960227272727273e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-3.1960227272727273e-04*Jinv20*w2-2.5202922077922079e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-2.5202922077922079e-03*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-2.0271915584415584e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+8.4922889610389610e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+1.3971185064935066e-03*w25*Jinv11+3.8352272727272725e-04*w55*Jinv10-2.7394480519480520e-03*w42*Jinv00+1.3697240259740259e-04*w47*Jinv20-1.6436688311688311e-04*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[52][20]=(0.0)*D;
  A[52][21]=(0.0)*D;
  A[52][22]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20+4.6570616883116886e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+7.1225649350649346e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02-2.7394480519480521e-05*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-8.2183441558441553e-04*w16*Jinv11-8.2183441558441553e-04*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+3.5612824675324673e-04*Jinv21*w26-2.7394480519480521e-05*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-3.2873376623376621e-04*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-2.7394480519480521e-05*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11-8.2183441558441553e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-3.2873376623376621e-04*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00+1.6436688311688311e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+4.6570616883116886e-04*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14+4.6570616883116886e-04*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18+1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-2.7394480519480518e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*Jinv12*w7-2.7394480519480518e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+7.1225649350649346e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18-2.7394480519480521e-05*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14-3.2873376623376621e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1)*D;
  A[52][23]=(0.0)*D;
  A[52][24]=(0.0)*D;
  A[52][25]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8+2.7394480519480518e-04*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.4655032467532466e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+1.8262987012987013e-05*Jinv21*w59-2.1915584415584417e-04*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+1.8262987012987013e-05*w58*Jinv11+1.8262987012987013e-05*w57*Jinv01-1.0683847402597404e-03*Jinv21*w26-2.1915584415584417e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-2.1915584415584417e-04*Jinv11*w1+4.6570616883116886e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+5.2049512987012989e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+1.4793019480519481e-03*Jinv21*w38-1.0683847402597404e-03*Jinv01*w24-1.6436688311688311e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00+8.2183441558441553e-05*w4*Jinv10+2.4655032467532466e-04*w4*Jinv11-3.8352272727272725e-04*w41*Jinv20+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.6436688311688311e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+2.4655032467532466e-04*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-3.8352272727272725e-04*w39*Jinv00+8.2183441558441553e-05*Jinv01*w18+1.4793019480519481e-03*w37*Jinv11+1.3971185064935066e-03*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+5.2049512987012989e-04*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53-3.5612824675324673e-04*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.4793019480519481e-03*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+1.3971185064935066e-03*w7*Jinv11-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-7.1225649350649346e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+4.6570616883116886e-04*Jinv21*w14-1.3697240259740259e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+4.6570616883116886e-04*Jinv01*w12+1.3971185064935066e-03*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-7.1225649350649346e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+8.2183441558441553e-05*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30-1.0683847402597404e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[52][26]=(0.0)*D;
  A[52][27]=(0.0)*D;
  A[52][28]=(4.3831168831168834e-04*Jinv21*w44-2.1915584415584417e-04*Jinv01*w3+6.4935064935064935e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+3.4699675324675324e-04*w16*Jinv11+3.4699675324675324e-04*Jinv21*w17-2.6481331168831169e-04*Jinv01*w45+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+4.3831168831168834e-04*w43*Jinv11+2.1915584415584417e-04*Jinv21*w41+1.0247564935064935e-04*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11+3.4699675324675324e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+2.1915584415584417e-04*Jinv01*w39-5.8441558441558442e-04*Jinv21*w38-1.0957792207792208e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11+1.4975649350649350e-03*Jinv21*w53-2.1915584415584417e-04*w4*Jinv11-4.2918019480519480e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-2.6481331168831169e-04*w46*Jinv11-2.1915584415584417e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+1.1870941558441559e-04*Jinv01*w18-5.8441558441558442e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-2.6481331168831169e-04*w47*Jinv21+1.4975649350649350e-03*w52*Jinv11-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01+1.4975649350649350e-03*Jinv01*w51-5.8441558441558442e-04*w36*Jinv01-4.2918019480519480e-04*w7*Jinv11+3.6525974025974026e-05*Jinv21*w56+7.4878246753246752e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-2.7394480519480521e-05*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8-5.4788961038961042e-05*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11-1.4204545454545455e-05*Jinv01*w9+2.1915584415584417e-04*w40*Jinv11-4.2918019480519480e-04*Jinv01*w30-1.0957792207792208e-04*w25*Jinv11+7.4878246753246752e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[52][29]=(0.0)*D;
  A[52][30]=(0.0)*D;
  A[52][31]=(8.2183441558441553e-05*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10+4.3831168831168834e-04*w40*Jinv12+4.0543831168831168e-03*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8+5.5702110389610384e-04*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10+4.3831168831168834e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00-5.7528409090909093e-04*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23-9.5880681818181818e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-4.6570616883116886e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.1047077922077924e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+2.0271915584415584e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00-4.6570616883116886e-04*Jinv02*w24+8.7662337662337668e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+8.2183441558441553e-04*Jinv21*w41+2.5750811688311690e-03*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+1.6436688311688311e-03*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12-5.7528409090909093e-04*w31*Jinv10-5.4788961038961036e-04*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00-7.3965097402597403e-04*w32*Jinv22+8.2183441558441553e-04*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-1.4793019480519481e-03*w36*Jinv00-1.4793019480519481e-03*Jinv21*w38-1.1231737012987013e-03*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24+5.5702110389610384e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+4.7666396103896100e-03*Jinv21*w53+4.7666396103896100e-03*w51*Jinv00+8.7662337662337668e-04*Jinv12*w34+2.5750811688311690e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+8.2183441558441553e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+1.6436688311688311e-03*w34*Jinv10-5.7528409090909093e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-9.5880681818181818e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56+4.3831168831168834e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+4.0543831168831168e-03*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35-5.7528409090909093e-04*w31*Jinv11-4.6570616883116886e-04*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21+2.0271915584415584e-03*Jinv02*w54-5.4788961038961036e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+8.2183441558441553e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-1.4793019480519481e-03*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.4793019480519481e-03*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23+4.7666396103896100e-03*w52*Jinv11-7.3965097402597403e-04*w31*Jinv12-1.1231737012987013e-03*w46*Jinv12+4.7666396103896100e-03*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01+4.7666396103896100e-03*Jinv01*w51-7.3965097402597403e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-1.4793019480519481e-03*w36*Jinv01+4.7666396103896100e-03*w52*Jinv10-1.3697240259740259e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-1.4793019480519481e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+3.1960227272727273e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12-5.4788961038961036e-04*Jinv22*w5-1.1231737012987013e-03*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+5.5702110389610384e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+8.7662337662337668e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+1.6436688311688311e-03*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02+3.1047077922077924e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17-5.7528409090909093e-04*w30*Jinv00+8.2183441558441553e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20+3.1960227272727273e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+8.2183441558441553e-04*w40*Jinv11-9.5880681818181818e-04*Jinv02*w36+2.5750811688311690e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-5.7528409090909093e-04*Jinv01*w30+4.0543831168831168e-03*Jinv02*w51+2.0271915584415584e-03*Jinv22*w56+3.1960227272727273e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+3.1047077922077924e-04*w10*Jinv11)*D;
  A[52][32]=(0.0)*D;
  A[52][33]=(0.0)*D;
  A[52][34]=(-3.2873376623376621e-04*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12-9.5332792207792200e-03*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-2.9586038961038961e-03*Jinv21*w44-6.0267857142857139e-04*w3*Jinv00+4.9310064935064932e-04*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+2.7942370129870131e-03*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54+1.0957792207792207e-03*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02-5.4788961038961036e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-2.9586038961038961e-03*w42*Jinv01-6.5746753246753243e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17+2.6298701298701297e-03*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+1.3149350649350649e-03*Jinv02*w24-6.9034090909090911e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26-5.4788961038961036e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-2.9586038961038961e-03*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22-5.4788961038961036e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01-5.4788961038961036e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+8.2183441558441553e-04*w22*Jinv12+4.9310064935064932e-04*w31*Jinv10+1.8628246753246754e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2+1.7532467532467534e-03*Jinv00*w18+2.7942370129870131e-03*w36*Jinv00+4.9310064935064934e-03*Jinv21*w38+2.5750811688311690e-03*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-1.2820616883116883e-02*Jinv21*w53-8.5470779220779228e-03*w51*Jinv00-6.9034090909090911e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44-6.0267857142857139e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+1.7532467532467534e-03*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+2.6298701298701297e-03*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+2.7942370129870131e-03*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.6436688311688311e-04*w11*Jinv20-9.5332792207792200e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35+2.6298701298701297e-03*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+2.6298701298701297e-03*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+8.2183441558441553e-04*w21*Jinv02-5.4788961038961036e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33+2.1367694805194807e-03*Jinv01*w18+4.9310064935064934e-03*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+2.6298701298701297e-03*w47*Jinv21+2.7942370129870131e-03*Jinv20*w38+8.2183441558441553e-04*Jinv22*w23-1.2820616883116883e-02*w52*Jinv11+1.8628246753246754e-03*w31*Jinv12+2.5750811688311690e-03*w46*Jinv12-8.5470779220779228e-03*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-2.0819805194805196e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-1.2820616883116883e-02*Jinv01*w51+1.8628246753246754e-03*Jinv02*w30-2.0819805194805196e-03*Jinv12*w49+4.9310064935064934e-03*w36*Jinv01-8.5470779220779228e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-1.6436688311688311e-04*w9*Jinv00+2.7942370129870131e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-9.6793831168831166e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12+2.5750811688311690e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-1.6436688311688311e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-6.9034090909090911e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-2.0819805194805196e-03*w48*Jinv02-6.5746753246753243e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17+4.9310064935064932e-04*w30*Jinv00+1.7532467532467534e-03*w20*Jinv20-9.6793831168831166e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+2.7942370129870131e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43-6.0267857142857139e-04*w5*Jinv20+2.6298701298701297e-03*Jinv01*w30-9.5332792207792200e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-9.6793831168831166e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-5.4788961038961036e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-6.2094155844155847e-04*Jinv12*w1-6.5746753246753243e-04*w10*Jinv11)*D;
  A[52][35]=(0.0)*D;
  A[52][36]=(0.0)*D;
  A[52][37]=(2.4655032467532466e-04*Jinv00*w12-1.5523538961038962e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-4.7483766233766234e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+4.3831168831168834e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20+2.4655032467532466e-04*Jinv20*w23-2.6298701298701297e-03*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-6.5746753246753243e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-1.5523538961038962e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+8.2183441558441553e-05*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02-1.5523538961038962e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-2.4655032467532466e-04*Jinv20*w50-1.5523538961038962e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+5.2049512987012989e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00+5.2049512987012989e-04*Jinv02*w45-4.7483766233766234e-04*Jinv22*w59+3.7804383116883116e-03*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10-8.2183441558441553e-04*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.2327516233766234e-03*w6*Jinv00-2.6298701298701297e-03*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+4.3831168831168834e-04*w26*Jinv20-1.4610389610389610e-04*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+2.4655032467532466e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10-8.2183441558441553e-04*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.2327516233766234e-03*w7*Jinv10-1.3149350649350649e-03*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+5.2049512987012989e-04*w31*Jinv12+5.2049512987012989e-04*w46*Jinv12+3.7804383116883116e-03*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+5.2049512987012989e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+3.7804383116883116e-03*w52*Jinv10-1.4610389610389610e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10+4.3831168831168834e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5+5.2049512987012989e-04*w47*Jinv22-1.4610389610389610e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-4.7483766233766234e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-1.5523538961038962e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+4.3831168831168834e-04*w25*Jinv10-1.5523538961038962e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00-8.2183441558441553e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-2.6298701298701297e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20+1.1505681818181819e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28-2.4655032467532466e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[52][38]=(0.0)*D;
  A[52][39]=(0.0)*D;
  A[52][40]=(5.4788961038961042e-05*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+1.7532467532467534e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+7.1225649350649346e-04*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+8.2183441558441553e-04*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+7.1225649350649346e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-3.2873376623376621e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-2.1915584415584417e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10+3.8352272727272725e-04*w21*Jinv00+9.3141233766233771e-04*w32*Jinv22-3.1047077922077924e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+1.7532467532467534e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+7.6704545454545450e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-8.2183441558441553e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+1.7532467532467534e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-8.2183441558441553e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+8.2183441558441553e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+3.8352272727272725e-04*Jinv01*w18-5.2962662337662338e-04*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+8.2183441558441553e-04*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23+9.3141233766233771e-04*w31*Jinv12+1.7532467532467534e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-1.0409902597402598e-03*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01+9.3141233766233771e-04*Jinv02*w30-1.0409902597402598e-03*Jinv12*w49-4.2735389610389614e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.7532467532467534e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-1.0409902597402598e-03*w48*Jinv02+1.3149350649350649e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29+1.7532467532467534e-03*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-8.2183441558441553e-04*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1)*D;
  A[52][41]=(0.0)*D;
  A[52][42]=(0.0)*D;
  A[52][43]=(-5.4788961038961042e-05*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+3.2873376623376621e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-9.8620129870129864e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-4.2004870129870131e-04*w0*Jinv02+5.4788961038961036e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+6.5746753246753243e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17-2.6298701298701297e-03*Jinv01*w45+2.1915584415584414e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-4.9310064935064934e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26+5.4788961038961036e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+5.4788961038961036e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01+5.4788961038961036e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-4.9310064935064934e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10+1.1505681818181819e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-4.2004870129870131e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-4.9310064935064934e-03*Jinv21*w38-7.6704545454545450e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-9.8620129870129864e-04*Jinv22*w59+2.9586038961038961e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+1.2820616883116883e-02*Jinv21*w53+4.2735389610389614e-03*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-3.9448051948051946e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-2.6298701298701297e-03*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+3.2873376623376621e-03*w52*Jinv12+2.9586038961038961e-03*Jinv21*w35-2.6298701298701297e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-2.6298701298701297e-03*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+2.1915584415584414e-03*Jinv02*w54+1.1505681818181819e-03*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+5.4788961038961036e-04*Jinv21*w29+2.9586038961038961e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.9310064935064934e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-2.6298701298701297e-03*w47*Jinv21-2.1367694805194807e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23+1.2820616883116883e-02*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+1.2820616883116883e-02*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-4.9310064935064934e-03*w36*Jinv01+4.2735389610389614e-03*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10+4.2735389610389614e-03*Jinv21*w56-7.6704545454545450e-04*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-1.3149350649350649e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+1.1505681818181819e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-9.8620129870129864e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11-7.6704545454545450e-04*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+6.5746753246753243e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-2.6298701298701297e-03*Jinv01*w30+3.2873376623376621e-03*Jinv02*w51+2.1915584415584414e-03*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11+4.2735389610389614e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-4.9310064935064934e-03*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+5.4788961038961036e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01-4.2004870129870131e-04*Jinv12*w1+6.5746753246753243e-04*w10*Jinv11)*D;
  A[52][44]=(0.0)*D;
  A[52][45]=(0.0)*D;
  A[52][46]=(-3.8352272727272725e-04*w40*Jinv12-7.1225649350649346e-04*Jinv22*w53+9.1314935064935065e-06*w57*Jinv02-3.8352272727272725e-04*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3+5.2049512987012989e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-1.6436688311688311e-03*w42*Jinv01-3.1047077922077924e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17+5.7528409090909093e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-5.4788961038961042e-05*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+5.2049512987012989e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-8.2183441558441553e-04*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11-5.4788961038961042e-05*Jinv22*w17-3.5612824675324673e-04*w22*Jinv12+8.2183441558441553e-05*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-8.2183441558441553e-04*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2+1.4793019480519481e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24+9.1314935064935065e-06*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11-4.7666396103896100e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv11+2.7394480519480518e-04*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+5.2049512987012989e-04*w37*Jinv12-3.8352272727272725e-04*Jinv22*w41-7.1225649350649346e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26+5.7528409090909093e-04*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+2.7394480519480518e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+1.4793019480519481e-03*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+5.7528409090909093e-04*w47*Jinv21-3.5612824675324673e-04*Jinv22*w23-4.7666396103896100e-03*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01-4.7666396103896100e-03*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49+1.4793019480519481e-03*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14+9.1314935064935065e-06*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+2.4655032467532466e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-8.2183441558441553e-05*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02-3.1047077922077924e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-8.2183441558441553e-04*w40*Jinv11+5.2049512987012989e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-7.1225649350649346e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-5.4788961038961042e-05*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1-3.1047077922077924e-04*w10*Jinv11)*D;
  A[52][47]=(0.0)*D;
  A[52][48]=(0.0)*D;
  A[52][49]=(3.5612824675324673e-04*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10-5.4788961038961036e-04*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8-1.1414366883116883e-03*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-5.4788961038961036e-04*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+2.3833198051948050e-03*w32*Jinv20+7.1225649350649346e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23+1.3971185064935066e-03*Jinv22*w38-3.6982548701298703e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-3.5612824675324673e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-7.8530844155844158e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.3833198051948050e-03*Jinv01*w45-2.4655032467532467e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-3.9265422077922079e-04*w10*Jinv12-3.6982548701298703e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10+8.2183441558441553e-05*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-2.1367694805194807e-03*Jinv21*w41-2.6846590909090908e-03*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.6982548701298703e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10-4.2735389610389614e-03*Jinv00*w33+2.3833198051948050e-03*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00+1.1505681818181819e-03*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-3.5612824675324673e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+2.6298701298701297e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-1.1414366883116883e-03*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-7.3965097402597406e-03*Jinv21*w53-7.3965097402597406e-03*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-2.6846590909090908e-03*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-4.2735389610389614e-03*w34*Jinv10+2.3833198051948050e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*w37*Jinv12-3.6982548701298703e-03*Jinv20*w56-5.4788961038961036e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-7.8530844155844158e-04*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+2.3833198051948050e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+2.3833198051948050e-03*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+7.1225649350649346e-04*w5*Jinv21-2.4655032467532467e-03*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-3.6982548701298703e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10+2.3833198051948050e-03*w47*Jinv21+2.6298701298701297e-03*Jinv20*w38-7.3965097402597406e-03*w52*Jinv11+1.1505681818181819e-03*w31*Jinv12+1.3697240259740260e-03*w46*Jinv12-7.3965097402597406e-03*Jinv20*w53+3.5612824675324673e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+3.5612824675324673e-04*w21*Jinv01-7.3965097402597406e-03*Jinv01*w51+1.1505681818181819e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01-7.3965097402597406e-03*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-7.8530844155844158e-04*w9*Jinv00+2.6298701298701297e-03*w37*Jinv10-3.6982548701298703e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-3.7439123376623376e-04*Jinv02*w27+8.2183441558441553e-05*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22-3.6982548701298703e-03*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-7.8530844155844158e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-1.1414366883116883e-03*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-4.2735389610389614e-03*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-3.6982548701298703e-03*Jinv01*w54-3.6982548701298703e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-7.8530844155844158e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.3833198051948050e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-3.7439123376623376e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+1.3971185064935066e-03*Jinv02*w36-2.6846590909090908e-03*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+2.3833198051948050e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-2.4655032467532467e-03*Jinv22*w56-3.7439123376623376e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-3.6982548701298703e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20-3.6982548701298703e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27-3.6982548701298703e-03*w48*Jinv01-3.5612824675324673e-04*Jinv12*w1-7.8530844155844158e-04*w10*Jinv11)*D;
  A[52][50]=(0.0)*D;
  A[52][51]=(0.0)*D;
  A[52][52]=(-3.5612824675324673e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+7.3965097402597406e-03*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-9.1314935064935068e-05*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-2.3833198051948050e-03*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38+3.6982548701298703e-03*Jinv00*w54-1.0683847402597404e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+4.2735389610389614e-03*w43*Jinv10+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+6.5746753246753243e-04*w10*Jinv12+3.6982548701298703e-03*w48*Jinv00-1.0683847402597404e-03*Jinv02*w24+4.7666396103896100e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10+5.7528409090909093e-04*Jinv22*w20-2.3833198051948050e-03*w45*Jinv00+4.7666396103896100e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+6.5746753246753243e-04*w11*Jinv22+3.6982548701298703e-03*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.3833198051948050e-03*w31*Jinv10-1.0683847402597404e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-2.0271915584415584e-03*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-2.6298701298701297e-03*w36*Jinv00-2.0271915584415584e-03*Jinv02*w45-9.1314935064935068e-05*Jinv22*w59+7.3965097402597406e-03*w51*Jinv00+4.7666396103896100e-03*Jinv12*w34+4.7666396103896100e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10+2.1367694805194807e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10+4.2735389610389614e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12+3.6982548701298703e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20+7.8530844155844158e-04*w11*Jinv20+7.3965097402597406e-03*w52*Jinv12-1.0683847402597404e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14+2.4655032467532467e-03*Jinv02*w54-1.0683847402597404e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+3.6982548701298703e-03*w49*Jinv10+2.1367694805194807e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+6.5746753246753243e-04*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10-2.6298701298701297e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-2.0271915584415584e-03*w31*Jinv12-2.0271915584415584e-03*w46*Jinv12+7.3965097402597406e-03*Jinv20*w53+5.7528409090909093e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+2.4655032467532467e-03*Jinv22*w50+5.7528409090909093e-04*Jinv12*w7-2.0271915584415584e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49+7.3965097402597406e-03*w52*Jinv10+7.8530844155844158e-04*w9*Jinv00-2.6298701298701297e-03*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+5.7528409090909093e-04*w6*Jinv02+5.7528409090909093e-04*w19*Jinv12-1.0683847402597404e-03*Jinv22*w5-2.0271915584415584e-03*w47*Jinv22+7.8530844155844158e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-9.1314935064935068e-05*w58*Jinv12+5.7528409090909093e-04*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+4.7666396103896100e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+4.2735389610389614e-03*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+2.4655032467532467e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.3833198051948050e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-1.3149350649350649e-03*Jinv02*w36+4.7666396103896100e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20+7.3965097402597406e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28+3.6982548701298703e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[52][53]=(0.0)*D;
  A[52][54]=(0.0)*D;
  A[52][55]=(1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-4.5657467532467534e-05*w57*Jinv02+1.5888798701298701e-03*Jinv02*w39+4.2735389610389614e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-1.2327516233766234e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12+3.7439123376623376e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w42*Jinv01+7.8530844155844158e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.3833198051948050e-03*Jinv01*w45+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-7.1225649350649346e-04*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20+4.2735389610389614e-03*w43*Jinv11+2.1367694805194807e-03*Jinv21*w41+3.1777597402597403e-03*w42*Jinv02+3.9265422077922079e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11+1.0957792207792207e-03*Jinv22*w17-8.2183441558441553e-05*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-1.0135957792207792e-03*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39+3.7439123376623376e-04*Jinv22*w2-2.6298701298701297e-03*Jinv21*w38-1.2327516233766234e-03*Jinv02*w45-7.1225649350649346e-04*Jinv01*w24-4.5657467532467534e-05*Jinv22*w59+4.2735389610389614e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+7.3965097402597406e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+3.1777597402597403e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-3.5612824675324673e-04*Jinv12*w13-2.3833198051948050e-03*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-1.2327516233766234e-03*w37*Jinv12+1.5888798701298701e-03*Jinv22*w41+2.4655032467532467e-03*w52*Jinv12+4.2735389610389614e-03*Jinv21*w35-2.3833198051948050e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-2.3833198051948050e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+2.4655032467532467e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33-3.5612824675324673e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+3.9265422077922079e-04*Jinv02*w9-1.6436688311688311e-04*w6*Jinv01-2.3833198051948050e-03*w47*Jinv21-8.2183441558441553e-05*Jinv22*w23+7.3965097402597406e-03*w52*Jinv11-1.0135957792207792e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12-8.2183441558441553e-05*Jinv22*w8+1.2327516233766234e-03*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+7.3965097402597406e-03*Jinv01*w51-1.0135957792207792e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-1.6436688311688311e-04*w7*Jinv11+3.6982548701298703e-03*Jinv21*w56+3.9265422077922079e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+3.6982548701298703e-03*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14-4.5657467532467534e-05*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+3.6982548701298703e-03*Jinv01*w54+3.6982548701298703e-03*w55*Jinv11+1.2327516233766234e-03*w48*Jinv02+7.8530844155844158e-04*Jinv01*w9+3.9265422077922079e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-1.2327516233766234e-03*Jinv02*w36+3.1777597402597403e-03*Jinv12*w43-2.3833198051948050e-03*Jinv01*w30+2.4655032467532467e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28-7.1225649350649346e-04*w25*Jinv11+1.0957792207792207e-03*w16*Jinv12+3.6982548701298703e-03*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27+3.6982548701298703e-03*w48*Jinv01+3.7439123376623376e-04*Jinv12*w1+7.8530844155844158e-04*w10*Jinv11)*D;
  A[52][56]=(0.0)*D;
  A[52][57]=(0.0)*D;
  A[52][58]=(3.2873376623376621e-04*w40*Jinv12+2.3741883116883116e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+3.2873376623376621e-04*Jinv02*w39-1.0227272727272728e-03*Jinv22*w38-1.0957792207792208e-04*w25*Jinv12+1.5726461038961039e-04*w0*Jinv02+1.1870941558441558e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+2.0495129870129870e-04*w10*Jinv12-1.0957792207792208e-04*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-2.6481331168831169e-04*Jinv22*w20+6.5746753246753243e-04*w42*Jinv02+2.0495129870129870e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-5.4788961038961042e-05*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-6.7573051948051951e-04*w32*Jinv22+1.5726461038961039e-04*Jinv22*w2-6.7573051948051951e-04*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+6.5746753246753243e-04*Jinv12*w34+6.5746753246753243e-04*Jinv22*w44-5.4788961038961042e-05*Jinv12*w13-1.0227272727272728e-03*w37*Jinv12+3.2873376623376621e-04*Jinv22*w41+2.3741883116883116e-03*w52*Jinv12-1.0957792207792208e-04*Jinv22*w26+1.1870941558441558e-03*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-5.4788961038961042e-05*Jinv02*w12+2.0495129870129870e-04*Jinv02*w9-5.4788961038961042e-05*Jinv22*w23-6.7573051948051951e-04*w31*Jinv12-6.7573051948051951e-04*w46*Jinv12-2.6481331168831169e-04*Jinv22*w8+1.1870941558441558e-03*Jinv22*w50-2.6481331168831169e-04*Jinv12*w7-6.7573051948051951e-04*Jinv02*w30+1.1870941558441558e-03*Jinv12*w49+1.5726461038961039e-04*Jinv02*w27-2.6481331168831169e-04*w6*Jinv02-2.6481331168831169e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5-6.7573051948051951e-04*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-2.6481331168831169e-04*Jinv02*w18+6.5746753246753243e-04*Jinv02*w33+1.1870941558441558e-03*w48*Jinv02+1.5726461038961039e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.0227272727272728e-03*Jinv02*w36+6.5746753246753243e-04*Jinv12*w43+2.3741883116883116e-03*Jinv02*w51+1.1870941558441558e-03*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+1.5726461038961039e-04*Jinv12*w1)*D;
  A[52][59]=(0.0)*D;
  A[53][0]=(0.0)*D;
  A[53][1]=(0.0)*D;
  A[53][2]=(5.4788961038961042e-05*Jinv00*w12-1.0247564935064935e-04*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12-1.4975649350649350e-03*Jinv22*w53-1.1870941558441559e-04*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+2.6481331168831169e-04*w32*Jinv20+1.0957792207792208e-04*Jinv01*w3+2.7394480519480521e-05*Jinv20*w23+5.8441558441558442e-04*Jinv22*w38-7.4878246753246752e-04*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-3.4699675324675324e-04*w16*Jinv11-3.4699675324675324e-04*Jinv21*w17+4.2918019480519480e-04*Jinv01*w45-7.4878246753246752e-04*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.4699675324675324e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-4.3831168831168834e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-3.4699675324675324e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+4.2918019480519480e-04*w45*Jinv00-2.1915584415584417e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02-1.0247564935064935e-04*Jinv00*w27+1.4204545454545455e-05*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11-3.4699675324675324e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50-1.0247564935064935e-04*Jinv20*w29+4.2918019480519480e-04*w46*Jinv10-3.4699675324675324e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12+2.6481331168831169e-04*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3+2.7394480519480521e-05*w21*Jinv00+2.6481331168831169e-04*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+5.8441558441558442e-04*w36*Jinv00+5.8441558441558442e-04*Jinv21*w38+4.2918019480519480e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11-1.4975649350649350e-03*Jinv21*w53-1.4975649350649350e-03*w51*Jinv00-4.3831168831168834e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10+1.0957792207792208e-04*w4*Jinv11-2.1915584415584417e-04*w41*Jinv20+5.4788961038961042e-05*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10+2.6481331168831169e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20-1.1870941558441559e-04*w6*Jinv00+5.8441558441558442e-04*w37*Jinv12-7.4878246753246752e-04*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41+2.1915584415584417e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20-1.4975649350649350e-03*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35+2.6481331168831169e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26+4.2918019480519480e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14+1.0957792207792208e-04*w5*Jinv21-7.4878246753246752e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-2.1915584415584417e-04*w39*Jinv00+5.4788961038961042e-05*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+5.8441558441558442e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9-1.1870941558441559e-04*w6*Jinv01-1.1870941558441559e-04*w7*Jinv10+4.2918019480519480e-04*w47*Jinv21+5.8441558441558442e-04*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23-1.4975649350649350e-03*w52*Jinv11+2.6481331168831169e-04*w31*Jinv12+4.2918019480519480e-04*w46*Jinv12-1.4975649350649350e-03*Jinv20*w53-1.1870941558441559e-04*Jinv22*w8+2.7394480519480521e-05*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-1.1870941558441559e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01-1.4975649350649350e-03*Jinv01*w51+2.6481331168831169e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+5.8441558441558442e-04*w36*Jinv01-1.4975649350649350e-03*w52*Jinv10-1.1870941558441559e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00+5.8441558441558442e-04*w37*Jinv10-7.4878246753246752e-04*Jinv21*w56+2.1915584415584417e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-1.1870941558441559e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5+4.2918019480519480e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12-1.1870941558441559e-04*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-4.3831168831168834e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23-7.4878246753246752e-04*Jinv01*w54-7.4878246753246752e-04*w55*Jinv11+2.1915584415584417e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-3.4699675324675324e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-3.4699675324675324e-04*Jinv20*w17+2.6481331168831169e-04*w30*Jinv00-2.1915584415584417e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+5.4788961038961042e-05*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11+5.8441558441558442e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20+2.6481331168831169e-04*Jinv01*w30-1.4975649350649350e-03*Jinv02*w51-7.4878246753246752e-04*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11-7.4878246753246752e-04*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+4.2918019480519480e-04*w47*Jinv20-3.4699675324675324e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[53][3]=(0.0)*D;
  A[53][4]=(0.0)*D;
  A[53][5]=(1.0957792207792208e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12+2.1367694805194807e-03*Jinv22*w53+4.3831168831168834e-04*Jinv20*w8-1.8262987012987013e-05*w57*Jinv02+7.1225649350649346e-04*Jinv02*w39+1.6436688311688311e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.0683847402597404e-03*Jinv01*w3-1.9176136363636362e-04*Jinv20*w23-1.4793019480519481e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54-1.8262987012987013e-05*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.2784090909090910e-04*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+1.6436688311688311e-04*w42*Jinv01-2.1915584415584417e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35-1.8262987012987013e-05*w58*Jinv11-1.8262987012987013e-05*w57*Jinv01-2.4655032467532466e-04*Jinv21*w26-1.2784090909090910e-04*w0*Jinv01-1.1870941558441559e-04*w1*Jinv10+1.2601461038961039e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+1.6436688311688311e-04*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+7.1225649350649346e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22-1.2784090909090910e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+2.1915584415584417e-04*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+1.3697240259740260e-03*Jinv00*w33-4.6570616883116886e-04*w22*Jinv12+1.0683847402597404e-03*Jinv02*w3-1.9176136363636362e-04*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-1.1231737012987013e-03*Jinv00*w18-9.5880681818181818e-04*w36*Jinv00-1.4793019480519481e-03*Jinv21*w38-5.4788961038961036e-04*Jinv02*w45-2.4655032467532466e-04*Jinv01*w24-1.8262987012987013e-05*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-1.3971185064935066e-03*w19*Jinv11+2.1367694805194807e-03*Jinv21*w53+1.4245129870129869e-03*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+1.0683847402597404e-03*w4*Jinv11+3.2873376623376621e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-1.1231737012987013e-03*w19*Jinv10+1.3697240259740260e-03*w34*Jinv10+1.6436688311688311e-04*w32*Jinv21-1.3971185064935066e-03*Jinv21*w20+4.3831168831168834e-04*w6*Jinv00-1.4793019480519481e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20-3.5612824675324673e-04*w11*Jinv20+2.1367694805194807e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35+1.6436688311688311e-04*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+1.0683847402597404e-03*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+1.0683847402597404e-03*w4*Jinv12-4.6570616883116886e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+2.1915584415584417e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33+3.2873376623376621e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-1.3971185064935066e-03*Jinv01*w18-1.4793019480519481e-03*w37*Jinv11-2.1915584415584417e-04*Jinv02*w9-8.2183441558441553e-05*w6*Jinv01+4.3831168831168834e-04*w7*Jinv10-5.4788961038961036e-04*w47*Jinv21-9.5880681818181818e-04*Jinv20*w38-4.6570616883116886e-04*Jinv22*w23+2.1367694805194807e-03*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-5.4788961038961036e-04*w46*Jinv12+1.4245129870129869e-03*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-1.9176136363636362e-04*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50-4.6570616883116886e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-4.6570616883116886e-04*w21*Jinv01+2.1367694805194807e-03*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-1.4793019480519481e-03*w36*Jinv01+1.4245129870129869e-03*w52*Jinv10-8.2183441558441553e-05*w7*Jinv11-3.5612824675324673e-04*w9*Jinv00-9.5880681818181818e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-1.6436688311688311e-04*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12+1.0683847402597404e-03*Jinv22*w5-5.4788961038961036e-04*w47*Jinv22-3.8352272727272725e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-3.5612824675324673e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44-1.8262987012987013e-05*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-8.2183441558441553e-05*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33-1.1870941558441559e-04*w0*Jinv00+1.3697240259740260e-03*w35*Jinv20-4.6570616883116886e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-1.6436688311688311e-04*w25*Jinv10-1.1870941558441559e-04*Jinv20*w2+1.2601461038961039e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02-2.1915584415584417e-04*Jinv01*w9+1.2601461038961039e-03*Jinv20*w17+3.2873376623376621e-04*w40*Jinv10-1.1231737012987013e-03*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-1.4793019480519481e-03*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+1.6436688311688311e-04*Jinv01*w30+2.1367694805194807e-03*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-2.4655032467532466e-04*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-3.8352272727272725e-04*Jinv21*w50+2.1915584415584417e-04*Jinv01*w27-3.8352272727272725e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1-2.1915584415584417e-04*w10*Jinv11)*D;
  A[53][6]=(0.0)*D;
  A[53][7]=(0.0)*D;
  A[53][8]=(-2.7394480519480521e-05*Jinv00*w12+1.0957792207792208e-04*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12-1.1505681818181819e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8+4.7483766233766234e-04*w57*Jinv02-2.7394480519480521e-05*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+2.1367694805194807e-03*Jinv21*w44-9.5880681818181818e-04*w3*Jinv00-3.8352272727272725e-04*w32*Jinv20-1.3971185064935066e-03*Jinv01*w3+1.9176136363636362e-04*Jinv20*w23+2.6298701298701297e-03*Jinv22*w38+3.5612824675324673e-04*Jinv00*w54+4.7483766233766234e-04*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+4.2918019480519480e-04*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+2.1367694805194807e-03*w42*Jinv01+5.4788961038961036e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+5.4788961038961036e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+2.1367694805194807e-03*Jinv22*w35+4.7483766233766234e-04*w58*Jinv11+4.7483766233766234e-04*w57*Jinv01-1.3971185064935066e-03*Jinv21*w26+4.2918019480519480e-04*w0*Jinv01+2.5568181818181820e-04*w1*Jinv10-1.2053571428571428e-03*w16*Jinv10+2.3833198051948050e-03*Jinv22*w20+2.1367694805194807e-03*w43*Jinv11-2.7394480519480518e-04*w45*Jinv00+3.8352272727272725e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+1.0957792207792208e-04*Jinv00*w27+5.4788961038961036e-04*w11*Jinv22+4.2918019480519480e-04*Jinv11*w1-2.7394480519480518e-04*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+1.0957792207792208e-04*Jinv20*w29-2.7394480519480518e-04*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17-6.0267857142857139e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12-3.8352272727272725e-04*w31*Jinv10-1.3971185064935066e-03*Jinv02*w3+1.9176136363636362e-04*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22+3.8352272727272725e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2+1.3697240259740260e-03*Jinv00*w18+1.3971185064935066e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-1.3971185064935066e-03*Jinv01*w24+4.7483766233766234e-04*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11+2.3833198051948050e-03*w19*Jinv11-1.1505681818181819e-03*Jinv21*w53+3.2873376623376621e-04*w51*Jinv00+2.1367694805194807e-03*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-9.5880681818181818e-04*w4*Jinv10-1.3971185064935066e-03*w4*Jinv11-3.2873376623376621e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13+1.3697240259740260e-03*w19*Jinv10-6.0267857142857139e-04*w34*Jinv10-5.2049512987012989e-04*w32*Jinv21+2.3833198051948050e-03*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+2.6298701298701297e-03*w37*Jinv12+3.5612824675324673e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+6.6659902597402602e-04*w11*Jinv20-1.1505681818181819e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-5.2049512987012989e-04*w31*Jinv11-1.3971185064935066e-03*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-2.7394480519480521e-05*Jinv20*w14-1.3971185064935066e-03*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-1.3971185064935066e-03*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-3.2873376623376621e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12+2.3833198051948050e-03*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11+5.4788961038961036e-04*Jinv02*w9+2.3833198051948050e-03*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21+1.3971185064935066e-03*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23-1.1505681818181819e-03*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.2873376623376621e-04*Jinv20*w53+2.3833198051948050e-03*Jinv22*w8+1.9176136363636362e-04*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+2.3833198051948050e-03*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-1.1505681818181819e-03*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01+3.2873376623376621e-04*w52*Jinv10+2.3833198051948050e-03*w7*Jinv11+6.6659902597402602e-04*w9*Jinv00+1.3971185064935066e-03*w37*Jinv10+1.6436688311688311e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+4.2918019480519480e-04*Jinv02*w27+2.3833198051948050e-03*w6*Jinv02+2.3833198051948050e-03*w19*Jinv12-1.3971185064935066e-03*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22+1.6436688311688311e-04*w49*Jinv11-2.7394480519480518e-04*Jinv21*w14+6.6659902597402602e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44+4.7483766233766234e-04*w58*Jinv12+2.3833198051948050e-03*Jinv02*w18-2.7394480519480518e-04*Jinv01*w12+2.3833198051948050e-03*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+2.1367694805194807e-03*Jinv02*w33+2.5568181818181820e-04*w0*Jinv00-6.0267857142857139e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+1.6436688311688311e-04*Jinv01*w54+1.6436688311688311e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+2.5568181818181820e-04*Jinv20*w2-1.2053571428571428e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02+5.4788961038961036e-04*Jinv01*w9-1.2053571428571428e-03*Jinv20*w17-3.8352272727272725e-04*w30*Jinv00-3.2873376623376621e-04*w40*Jinv10+1.3697240259740260e-03*w20*Jinv20+4.2918019480519480e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+3.8352272727272725e-04*w40*Jinv11+2.6298701298701297e-03*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-9.5880681818181818e-04*w5*Jinv20-5.2049512987012989e-04*Jinv01*w30-1.1505681818181819e-03*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56+4.2918019480519480e-04*Jinv12*w28-1.3971185064935066e-03*w25*Jinv11+3.5612824675324673e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00-2.7394480519480518e-04*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+1.6436688311688311e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+1.6436688311688311e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+5.4788961038961036e-04*w10*Jinv11)*D;
  A[53][9]=(0.0)*D;
  A[53][10]=(0.0)*D;
  A[53][11]=(-2.7394480519480521e-05*Jinv00*w12-1.1668019480519480e-04*w28*Jinv10-6.7573051948051951e-04*Jinv20*w8-2.7394480519480521e-05*w13*Jinv10+4.9310064935064932e-04*w3*Jinv00+1.1870941558441559e-04*w32*Jinv20-2.7394480519480521e-05*Jinv20*w23+3.6525974025974026e-05*Jinv00*w54-3.2873376623376621e-04*w43*Jinv10+8.3198051948051951e-05*w58*Jinv10+3.6525974025974026e-05*w48*Jinv00-1.1668019480519480e-04*w1*Jinv10+2.9220779220779221e-04*w16*Jinv10+1.1870941558441559e-04*w45*Jinv00-1.1668019480519480e-04*Jinv00*w27+3.6525974025974026e-05*Jinv20*w50-1.1668019480519480e-04*Jinv20*w29+1.1870941558441559e-04*w46*Jinv10-3.2873376623376621e-04*Jinv00*w33+1.1870941558441559e-04*w31*Jinv10-2.7394480519480521e-05*w21*Jinv00-6.7573051948051951e-04*Jinv00*w18-1.0227272727272728e-03*w36*Jinv00-2.5568181818181820e-04*w51*Jinv00+4.9310064935064932e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-6.7573051948051951e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-6.7573051948051951e-04*w6*Jinv00+3.6525974025974026e-05*Jinv20*w56+4.9310064935064932e-04*w26*Jinv20-3.2467532467532468e-04*w11*Jinv20-2.7394480519480521e-05*Jinv20*w14+3.6525974025974026e-05*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-6.7573051948051951e-04*w7*Jinv10-1.0227272727272728e-03*Jinv20*w38-2.5568181818181820e-04*Jinv20*w53-2.7394480519480521e-05*w22*Jinv10-2.5568181818181820e-04*w52*Jinv10-3.2467532467532468e-04*w9*Jinv00-1.0227272727272728e-03*w37*Jinv10+4.9310064935064932e-04*w24*Jinv00-3.2467532467532468e-04*w10*Jinv10-3.2873376623376621e-04*Jinv20*w44+8.3198051948051951e-05*w57*Jinv00+8.3198051948051951e-05*Jinv20*w59-1.1668019480519480e-04*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20+4.9310064935064932e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+2.9220779220779221e-04*w15*Jinv00+2.9220779220779221e-04*Jinv20*w17+1.1870941558441559e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-6.7573051948051951e-04*w20*Jinv20+4.9310064935064932e-04*w5*Jinv20+3.6525974025974026e-05*w55*Jinv10-3.2873376623376621e-04*w42*Jinv00+1.1870941558441559e-04*w47*Jinv20)*D;
  A[53][12]=(0.0)*D;
  A[53][13]=(0.0)*D;
  A[53][14]=(3.5612824675324673e-04*Jinv00*w12+1.3697240259740259e-04*w28*Jinv10+7.1225649350649346e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20-3.5612824675324673e-04*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38+7.1225649350649346e-04*Jinv00*w54-5.4788961038961036e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2+1.0957792207792208e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01+8.2183441558441553e-04*w16*Jinv11+8.2183441558441553e-04*Jinv21*w17-2.4655032467532466e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+8.7662337662337668e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-1.9176136363636362e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-4.6570616883116886e-04*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-6.3920454545454550e-05*w1*Jinv10+8.7662337662337668e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11-5.2049512987012989e-04*w45*Jinv00+3.2873376623376621e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1+8.2183441558441553e-04*w15*Jinv01+2.7394480519480521e-05*w28*Jinv11-1.9176136363636362e-04*Jinv20*w50+1.3697240259740259e-04*Jinv20*w29-5.2049512987012989e-04*w46*Jinv10+8.7662337662337668e-04*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+3.2873376623376621e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-5.2049512987012989e-04*Jinv02*w45-4.6570616883116886e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10-3.5612824675324673e-04*w4*Jinv11+7.1225649350649346e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-8.2183441558441553e-05*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+7.1225649350649346e-04*Jinv20*w56+7.1225649350649346e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35-8.2183441558441553e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-2.4655032467532466e-04*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14-3.5612824675324673e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-1.9176136363636362e-04*w49*Jinv10+2.7394480519480521e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+7.1225649350649346e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18+1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10-2.4655032467532466e-04*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10-1.9176136363636362e-04*Jinv22*w50+2.7394480519480521e-05*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7+2.7394480519480521e-05*w21*Jinv01+8.2183441558441553e-05*Jinv02*w30-1.9176136363636362e-04*Jinv12*w49+7.1225649350649346e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11+1.3697240259740259e-04*w10*Jinv10+1.0957792207792208e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-6.3920454545454550e-05*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+2.7394480519480521e-05*Jinv21*w23+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10-6.3920454545454550e-05*Jinv20*w2+8.7662337662337668e-04*w15*Jinv00-1.9176136363636362e-04*w48*Jinv02+8.7662337662337668e-04*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00+7.1225649350649346e-04*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+3.2873376623376621e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20-8.2183441558441553e-05*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-4.6570616883116886e-04*w25*Jinv11+7.1225649350649346e-04*w55*Jinv10+1.0957792207792208e-04*w42*Jinv00-5.2049512987012989e-04*w47*Jinv20+8.7662337662337668e-04*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+2.7394480519480521e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1)*D;
  A[53][15]=(0.0)*D;
  A[53][16]=(0.0)*D;
  A[53][17]=(-8.2183441558441553e-04*Jinv00*w12+3.6525974025974026e-05*w28*Jinv10+1.3149350649350649e-03*w40*Jinv12-3.2873376623376621e-03*Jinv22*w53+1.2601461038961039e-03*Jinv20*w8+9.8620129870129864e-04*w57*Jinv02-8.2183441558441553e-04*w13*Jinv10+1.3149350649350649e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-1.2053571428571428e-03*w3*Jinv00-3.2873376623376621e-04*w32*Jinv20-1.3149350649350649e-03*Jinv01*w3+2.1367694805194807e-03*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+1.6436688311688311e-04*w25*Jinv12+4.2004870129870131e-04*w0*Jinv02+3.4699675324675324e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+7.1225649350649346e-04*Jinv01*w45-2.1915584415584414e-03*Jinv12*w55+4.9310064935064932e-04*w58*Jinv10+4.9310064935064934e-03*w15*Jinv02-2.1915584415584417e-04*w10*Jinv12+1.6436688311688311e-04*Jinv02*w24+1.9724025974025973e-03*Jinv22*w35+1.3149350649350649e-03*Jinv21*w26+3.4699675324675324e-04*w0*Jinv01+3.8352272727272725e-04*w1*Jinv10+1.6436688311688311e-03*w16*Jinv10+2.6298701298701297e-03*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11+3.8352272727272725e-04*w45*Jinv00+3.9448051948051946e-03*w42*Jinv02+3.6525974025974026e-05*Jinv00*w27-2.1915584415584417e-04*w11*Jinv22+3.4699675324675324e-04*Jinv11*w1-8.2183441558441553e-04*w13*Jinv11-3.4699675324675324e-04*w28*Jinv11+3.6525974025974026e-05*Jinv20*w29+3.8352272727272725e-04*w46*Jinv10+4.9310064935064934e-03*Jinv22*w17+6.5746753246753243e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-3.2873376623376621e-04*w31*Jinv10-1.1505681818181819e-03*Jinv02*w3+5.4788961038961042e-05*w32*Jinv22+4.2004870129870131e-04*Jinv22*w2+2.5750811688311690e-03*Jinv00*w18+1.7532467532467534e-03*w36*Jinv00+7.6704545454545450e-04*Jinv02*w45+1.3149350649350649e-03*Jinv01*w24+9.8620129870129864e-04*Jinv22*w59-1.9724025974025973e-03*w34*Jinv11+1.3149350649350649e-03*w19*Jinv11-2.1915584415584414e-03*w51*Jinv00+1.9724025974025973e-03*Jinv12*w34+3.9448051948051946e-03*Jinv22*w44-1.2053571428571428e-03*w4*Jinv10-1.3149350649350649e-03*w4*Jinv11-7.6704545454545450e-04*Jinv12*w13+2.5750811688311690e-03*w19*Jinv10+6.5746753246753243e-04*w34*Jinv10-7.1225649350649346e-04*w32*Jinv21+1.3149350649350649e-03*Jinv21*w20+1.2601461038961039e-03*w6*Jinv00+2.1367694805194807e-03*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+1.3149350649350649e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-2.5568181818181820e-04*w11*Jinv20-3.2873376623376621e-03*w52*Jinv12-1.9724025974025973e-03*Jinv21*w35-7.1225649350649346e-04*w31*Jinv11+1.6436688311688311e-04*Jinv22*w26+7.1225649350649346e-04*w46*Jinv11-8.2183441558441553e-04*Jinv20*w14-1.3149350649350649e-03*w5*Jinv21-2.1915584415584414e-03*Jinv02*w54-1.1505681818181819e-03*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02-3.4699675324675324e-04*Jinv21*w29-1.9724025974025973e-03*Jinv01*w33-7.6704545454545450e-04*Jinv02*w12+1.3149350649350649e-03*Jinv01*w18-2.1915584415584417e-04*Jinv02*w9-1.3149350649350649e-03*w6*Jinv01+1.2601461038961039e-03*w7*Jinv10+7.1225649350649346e-04*w47*Jinv21+1.7532467532467534e-03*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+5.4788961038961042e-05*w31*Jinv12+7.6704545454545450e-04*w46*Jinv12-2.1915584415584414e-03*Jinv20*w53+1.3149350649350649e-03*Jinv22*w8-1.0957792207792207e-03*Jinv22*w50+8.2183441558441553e-04*w22*Jinv11+1.3149350649350649e-03*Jinv12*w7+8.2183441558441553e-04*w21*Jinv01+5.4788961038961042e-05*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49-2.1915584415584414e-03*w52*Jinv10-1.3149350649350649e-03*w7*Jinv11-2.5568181818181820e-04*w9*Jinv00+1.7532467532467534e-03*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00+7.3051948051948052e-05*Jinv02*w27+1.3149350649350649e-03*w6*Jinv02+2.6298701298701297e-03*w19*Jinv12-1.1505681818181819e-03*Jinv22*w5+7.6704545454545450e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-8.2183441558441553e-04*Jinv21*w14-2.5568181818181820e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+9.8620129870129864e-04*w58*Jinv12+2.6298701298701297e-03*Jinv02*w18-8.2183441558441553e-04*Jinv01*w12-1.3149350649350649e-03*Jinv21*w8+4.9310064935064932e-04*w57*Jinv00+4.9310064935064932e-04*Jinv20*w59+1.9724025974025973e-03*Jinv02*w33+3.8352272727272725e-04*w0*Jinv00+6.5746753246753243e-04*w35*Jinv20+8.2183441558441553e-04*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+3.8352272727272725e-04*Jinv20*w2+1.6436688311688311e-03*w15*Jinv00-1.0957792207792207e-03*w48*Jinv02+1.6436688311688311e-03*Jinv20*w17-3.2873376623376621e-04*w30*Jinv00+2.5750811688311690e-03*w20*Jinv20+7.3051948051948052e-05*Jinv22*w29-7.6704545454545450e-04*Jinv22*w14+2.1367694805194807e-03*Jinv02*w36+3.9448051948051946e-03*Jinv12*w43-1.2053571428571428e-03*w5*Jinv20-7.1225649350649346e-04*Jinv01*w30-3.2873376623376621e-03*Jinv02*w51-2.1915584415584414e-03*Jinv22*w56+7.3051948051948052e-05*Jinv12*w28+1.3149350649350649e-03*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00+3.8352272727272725e-04*w47*Jinv20+4.9310064935064934e-03*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50-3.4699675324675324e-04*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01+4.2004870129870131e-04*Jinv12*w1)*D;
  A[53][18]=(0.0)*D;
  A[53][19]=(0.0)*D;
  A[53][20]=(4.6570616883116886e-04*Jinv00*w12-1.7349837662337662e-04*w28*Jinv10-1.0135957792207792e-03*Jinv20*w8+4.6570616883116886e-04*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+8.4922889610389610e-04*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20+1.3971185064935066e-03*Jinv01*w3+2.4655032467532466e-04*Jinv20*w23+3.8352272727272725e-04*Jinv00*w54-4.7483766233766234e-04*Jinv21*w59-4.2918019480519480e-04*Jinv21*w2-2.7394480519480520e-03*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01-5.4788961038961036e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45-4.8396915584415583e-04*w58*Jinv10+1.9176136363636362e-04*w48*Jinv00-4.7483766233766234e-04*w58*Jinv11-4.7483766233766234e-04*w57*Jinv01+1.3971185064935066e-03*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01-3.1960227272727273e-04*w1*Jinv10-2.5202922077922079e-03*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11+1.3697240259740259e-04*w45*Jinv00-3.8352272727272725e-04*Jinv21*w41-1.7349837662337662e-04*Jinv00*w27-4.2918019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01-4.2918019480519480e-04*w28*Jinv11+1.9176136363636362e-04*Jinv20*w50-1.7349837662337662e-04*Jinv20*w29+1.3697240259740259e-04*w46*Jinv10-1.3697240259740260e-03*Jinv00*w33+2.4655032467532466e-04*w31*Jinv10+2.4655032467532466e-04*w21*Jinv00-3.8352272727272725e-04*Jinv01*w39-2.0271915584415584e-03*Jinv00*w18-1.2327516233766234e-03*w36*Jinv00-2.6298701298701297e-03*Jinv21*w38+1.3971185064935066e-03*Jinv01*w24-2.1367694805194807e-03*w34*Jinv11-2.3833198051948050e-03*w19*Jinv11+1.1505681818181819e-03*Jinv21*w53+1.4793019480519481e-03*w51*Jinv00+8.4922889610389610e-04*w4*Jinv10+1.3971185064935066e-03*w4*Jinv11-7.1225649350649346e-04*w41*Jinv20-2.0271915584415584e-03*w19*Jinv10-1.3697240259740260e-03*w34*Jinv10+5.2049512987012989e-04*w32*Jinv21-2.3833198051948050e-03*Jinv21*w20-1.0135957792207792e-03*w6*Jinv00+3.8352272727272725e-04*Jinv20*w56+4.3831168831168834e-04*w26*Jinv20+1.1870941558441559e-04*w11*Jinv20-2.1367694805194807e-03*Jinv21*w35+5.2049512987012989e-04*w31*Jinv11+5.2049512987012989e-04*w46*Jinv11+4.6570616883116886e-04*Jinv20*w14+1.3971185064935066e-03*w5*Jinv21+1.9176136363636362e-04*w49*Jinv10-4.2918019480519480e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-7.1225649350649346e-04*w39*Jinv00-2.3833198051948050e-03*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11-2.3833198051948050e-03*w6*Jinv01-1.0135957792207792e-03*w7*Jinv10+5.2049512987012989e-04*w47*Jinv21-1.2327516233766234e-03*Jinv20*w38+1.1505681818181819e-03*w52*Jinv11+1.4793019480519481e-03*Jinv20*w53+2.4655032467532466e-04*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01+1.1505681818181819e-03*Jinv01*w51-2.6298701298701297e-03*w36*Jinv01+1.4793019480519481e-03*w52*Jinv10-2.3833198051948050e-03*w7*Jinv11+1.1870941558441559e-04*w9*Jinv00-1.2327516233766234e-03*w37*Jinv10-1.6436688311688311e-04*Jinv21*w56+4.3831168831168834e-04*w24*Jinv00-1.6436688311688311e-04*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14+1.1870941558441559e-04*w10*Jinv10-2.7394480519480520e-03*Jinv20*w44+2.7394480519480518e-04*Jinv01*w12-2.3833198051948050e-03*Jinv21*w8-4.8396915584415583e-04*w57*Jinv00-4.8396915584415583e-04*Jinv20*w59-3.1960227272727273e-04*w0*Jinv00-1.3697240259740260e-03*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23-1.6436688311688311e-04*Jinv01*w54-1.6436688311688311e-04*w55*Jinv11+4.3831168831168834e-04*w25*Jinv10-3.1960227272727273e-04*Jinv20*w2-2.5202922077922079e-03*w15*Jinv00-5.4788961038961036e-04*Jinv01*w9-2.5202922077922079e-03*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-7.1225649350649346e-04*w40*Jinv10-2.0271915584415584e-03*w20*Jinv20-3.8352272727272725e-04*w40*Jinv11+8.4922889610389610e-04*w5*Jinv20+5.2049512987012989e-04*Jinv01*w30+1.3971185064935066e-03*w25*Jinv11+3.8352272727272725e-04*w55*Jinv10-2.7394480519480520e-03*w42*Jinv00+1.3697240259740259e-04*w47*Jinv20-1.6436688311688311e-04*Jinv21*w50-4.2918019480519480e-04*Jinv01*w27-1.6436688311688311e-04*w48*Jinv01-5.4788961038961036e-04*w10*Jinv11)*D;
  A[53][21]=(0.0)*D;
  A[53][22]=(0.0)*D;
  A[53][23]=(-2.7394480519480518e-04*Jinv00*w12-9.1314935064935065e-06*w28*Jinv10+3.8352272727272725e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-2.7394480519480518e-04*w13*Jinv10+3.8352272727272725e-04*Jinv02*w39+5.4788961038961042e-05*Jinv21*w44-8.2183441558441553e-05*w3*Jinv00-2.7394480519480518e-04*w32*Jinv20+4.6570616883116886e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38+1.6436688311688311e-04*Jinv00*w54+7.1225649350649346e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02-2.7394480519480521e-05*Jinv21*w2+7.6704545454545450e-04*w43*Jinv10+5.4788961038961042e-05*w42*Jinv01-8.2183441558441553e-04*w16*Jinv11-8.2183441558441553e-04*Jinv21*w17+8.2183441558441553e-05*Jinv01*w45+1.6436688311688311e-04*Jinv12*w55-9.1314935064935065e-06*w58*Jinv10+5.4788961038961042e-05*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12+3.5612824675324673e-04*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+3.5612824675324673e-04*Jinv21*w26-2.7394480519480521e-05*w0*Jinv01+1.0957792207792208e-04*w1*Jinv10+5.4788961038961042e-05*w16*Jinv10-5.2049512987012989e-04*Jinv22*w20+5.4788961038961042e-05*w43*Jinv11+1.6436688311688311e-04*w45*Jinv00-3.2873376623376621e-04*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*Jinv00*w27+1.3697240259740259e-04*w11*Jinv22-2.7394480519480521e-05*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11-8.2183441558441553e-04*w15*Jinv01+5.4788961038961042e-05*w28*Jinv11+3.5612824675324673e-04*Jinv20*w50-9.1314935064935065e-06*Jinv20*w29+1.6436688311688311e-04*w46*Jinv10+5.4788961038961042e-05*Jinv22*w17+5.4788961038961042e-05*Jinv00*w33+3.5612824675324673e-04*w22*Jinv12-2.7394480519480518e-04*w31*Jinv10-8.2183441558441553e-05*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22-3.2873376623376621e-04*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00+1.6436688311688311e-04*Jinv02*w45+3.5612824675324673e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59-5.4788961038961042e-05*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+7.1225649350649346e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-8.2183441558441553e-05*w4*Jinv10+4.6570616883116886e-04*w4*Jinv11+3.8352272727272725e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+5.4788961038961042e-05*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+1.6436688311688311e-04*Jinv20*w56+3.8352272727272725e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20+1.3697240259740259e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12-5.4788961038961042e-05*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+8.2183441558441553e-05*w46*Jinv11-2.7394480519480518e-04*Jinv20*w14+4.6570616883116886e-04*w5*Jinv21+1.6436688311688311e-04*Jinv02*w54-8.2183441558441553e-05*w4*Jinv12+3.5612824675324673e-04*w21*Jinv02+3.5612824675324673e-04*w49*Jinv10+5.4788961038961042e-05*Jinv21*w29-5.4788961038961042e-05*Jinv01*w33+3.8352272727272725e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18+1.3697240259740259e-04*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+8.2183441558441553e-05*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+3.5612824675324673e-04*Jinv22*w23-2.7394480519480518e-04*w31*Jinv12+1.6436688311688311e-04*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+3.5612824675324673e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*Jinv12*w7-2.7394480519480518e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+7.1225649350649346e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.3697240259740259e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.5612824675324673e-04*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-9.1314935064935065e-06*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-5.2049512987012989e-04*w19*Jinv12-8.2183441558441553e-05*Jinv22*w5+1.6436688311688311e-04*w47*Jinv22-3.5612824675324673e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14+1.3697240259740259e-04*w10*Jinv10+7.6704545454545450e-04*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.2049512987012989e-04*Jinv02*w18-2.7394480519480521e-05*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-9.1314935064935065e-06*w57*Jinv00-9.1314935064935065e-06*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+1.0957792207792208e-04*w0*Jinv00+5.4788961038961042e-05*w35*Jinv20+3.5612824675324673e-04*Jinv01*w54+3.5612824675324673e-04*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10+1.0957792207792208e-04*Jinv20*w2+5.4788961038961042e-05*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02+5.4788961038961042e-05*Jinv20*w17-2.7394480519480518e-04*w30*Jinv00+3.8352272727272725e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20-9.1314935064935065e-06*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14-3.2873376623376621e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-8.2183441558441553e-05*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+1.6436688311688311e-04*Jinv22*w56-9.1314935064935065e-06*Jinv12*w28+3.5612824675324673e-04*w25*Jinv11+1.6436688311688311e-04*w55*Jinv10+7.6704545454545450e-04*w42*Jinv00+1.6436688311688311e-04*w47*Jinv20+5.4788961038961042e-05*w16*Jinv12-3.5612824675324673e-04*Jinv21*w50+5.4788961038961042e-05*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1)*D;
  A[53][24]=(0.0)*D;
  A[53][25]=(0.0)*D;
  A[53][26]=(2.7394480519480518e-04*Jinv00*w12+9.1314935064935065e-06*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8+2.7394480519480518e-04*w13*Jinv10-2.1367694805194807e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00+2.7394480519480518e-04*w32*Jinv20+2.4655032467532466e-04*Jinv01*w3-3.5612824675324673e-04*Jinv20*w23-1.6436688311688311e-04*Jinv00*w54+1.8262987012987013e-05*Jinv21*w59-2.1915584415584417e-04*Jinv21*w2-7.6704545454545450e-04*w43*Jinv10-2.1367694805194807e-03*w42*Jinv01+2.1915584415584417e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-1.6436688311688311e-04*Jinv01*w45+9.1314935064935065e-06*w58*Jinv10-3.5612824675324673e-04*w48*Jinv00+1.8262987012987013e-05*w58*Jinv11+1.8262987012987013e-05*w57*Jinv01-1.0683847402597404e-03*Jinv21*w26-2.1915584415584417e-04*w0*Jinv01-1.0957792207792208e-04*w1*Jinv10-5.4788961038961042e-05*w16*Jinv10-2.1367694805194807e-03*w43*Jinv11-1.6436688311688311e-04*w45*Jinv00-7.1225649350649346e-04*Jinv21*w41+9.1314935064935065e-06*Jinv00*w27-2.1915584415584417e-04*Jinv11*w1+4.6570616883116886e-04*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+1.2784090909090910e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+9.1314935064935065e-06*Jinv20*w29-1.6436688311688311e-04*w46*Jinv10-5.4788961038961042e-05*Jinv00*w33+2.7394480519480518e-04*w31*Jinv10-3.5612824675324673e-04*w21*Jinv00-7.1225649350649346e-04*Jinv01*w39+5.2049512987012989e-04*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+1.4793019480519481e-03*Jinv21*w38-1.0683847402597404e-03*Jinv01*w24-1.6436688311688311e-04*w34*Jinv11+8.2183441558441553e-05*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-7.1225649350649346e-04*w51*Jinv00+8.2183441558441553e-05*w4*Jinv10+2.4655032467532466e-04*w4*Jinv11-3.8352272727272725e-04*w41*Jinv20+5.2049512987012989e-04*w19*Jinv10-5.4788961038961042e-05*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21+8.2183441558441553e-05*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00-1.6436688311688311e-04*Jinv20*w56-7.1225649350649346e-04*w26*Jinv20-1.3697240259740259e-04*w11*Jinv20-1.6436688311688311e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11-1.6436688311688311e-04*w46*Jinv11+2.7394480519480518e-04*Jinv20*w14+2.4655032467532466e-04*w5*Jinv21-3.5612824675324673e-04*w49*Jinv10+1.2784090909090910e-04*Jinv21*w29-1.6436688311688311e-04*Jinv01*w33-3.8352272727272725e-04*w39*Jinv00+8.2183441558441553e-05*Jinv01*w18+1.4793019480519481e-03*w37*Jinv11+1.3971185064935066e-03*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-1.6436688311688311e-04*w47*Jinv21+5.2049512987012989e-04*Jinv20*w38-2.1367694805194807e-03*w52*Jinv11-7.1225649350649346e-04*Jinv20*w53-3.5612824675324673e-04*w22*Jinv10-3.5612824675324673e-04*w22*Jinv11-3.5612824675324673e-04*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.4793019480519481e-03*w36*Jinv01-7.1225649350649346e-04*w52*Jinv10+1.3971185064935066e-03*w7*Jinv11-1.3697240259740259e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+3.8352272727272725e-04*Jinv21*w56-7.1225649350649346e-04*w24*Jinv00-7.1225649350649346e-04*w49*Jinv11+4.6570616883116886e-04*Jinv21*w14-1.3697240259740259e-04*w10*Jinv10-7.6704545454545450e-04*Jinv20*w44+4.6570616883116886e-04*Jinv01*w12+1.3971185064935066e-03*Jinv21*w8+9.1314935064935065e-06*w57*Jinv00+9.1314935064935065e-06*Jinv20*w59-1.0957792207792208e-04*w0*Jinv00-5.4788961038961042e-05*w35*Jinv20-3.5612824675324673e-04*Jinv21*w23+3.8352272727272725e-04*Jinv01*w54+3.8352272727272725e-04*w55*Jinv11-7.1225649350649346e-04*w25*Jinv10-1.0957792207792208e-04*Jinv20*w2-5.4788961038961042e-05*w15*Jinv00+2.1915584415584417e-04*Jinv01*w9-5.4788961038961042e-05*Jinv20*w17+2.7394480519480518e-04*w30*Jinv00-3.8352272727272725e-04*w40*Jinv10+5.2049512987012989e-04*w20*Jinv20-7.1225649350649346e-04*w40*Jinv11+8.2183441558441553e-05*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30-1.0683847402597404e-03*w25*Jinv11-1.6436688311688311e-04*w55*Jinv10-7.6704545454545450e-04*w42*Jinv00-1.6436688311688311e-04*w47*Jinv20-7.1225649350649346e-04*Jinv21*w50+1.2784090909090910e-04*Jinv01*w27-7.1225649350649346e-04*w48*Jinv01+2.1915584415584417e-04*w10*Jinv11)*D;
  A[53][27]=(0.0)*D;
  A[53][28]=(0.0)*D;
  A[53][29]=(4.3831168831168834e-04*Jinv21*w44-2.1915584415584417e-04*Jinv01*w3+6.4935064935064935e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+3.4699675324675324e-04*w16*Jinv11+3.4699675324675324e-04*Jinv21*w17-2.6481331168831169e-04*Jinv01*w45+6.4935064935064935e-05*w58*Jinv11+6.4935064935064935e-05*w57*Jinv01-1.0957792207792208e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+4.3831168831168834e-04*w43*Jinv11+2.1915584415584417e-04*Jinv21*w41+1.0247564935064935e-04*Jinv11*w1-2.7394480519480521e-05*w13*Jinv11+3.4699675324675324e-04*w15*Jinv01+2.0292207792207792e-05*w28*Jinv11+2.1915584415584417e-04*Jinv01*w39-5.8441558441558442e-04*Jinv21*w38-1.0957792207792208e-04*Jinv01*w24+5.4788961038961036e-04*w34*Jinv11+1.1870941558441559e-04*w19*Jinv11+1.4975649350649350e-03*Jinv21*w53-2.1915584415584417e-04*w4*Jinv11-4.2918019480519480e-04*w32*Jinv21+1.1870941558441559e-04*Jinv21*w20+5.4788961038961036e-04*Jinv21*w35-4.2918019480519480e-04*w31*Jinv11-2.6481331168831169e-04*w46*Jinv11-2.1915584415584417e-04*w5*Jinv21+2.0292207792207792e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+1.1870941558441559e-04*Jinv01*w18-5.8441558441558442e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-2.6481331168831169e-04*w47*Jinv21+1.4975649350649350e-03*w52*Jinv11-5.4788961038961042e-05*w22*Jinv11-5.4788961038961042e-05*w21*Jinv01+1.4975649350649350e-03*Jinv01*w51-5.8441558441558442e-04*w36*Jinv01-4.2918019480519480e-04*w7*Jinv11+3.6525974025974026e-05*Jinv21*w56+7.4878246753246752e-04*w49*Jinv11-2.7394480519480521e-05*Jinv21*w14-2.7394480519480521e-05*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8-5.4788961038961042e-05*Jinv21*w23+3.6525974025974026e-05*Jinv01*w54+3.6525974025974026e-05*w55*Jinv11-1.4204545454545455e-05*Jinv01*w9+2.1915584415584417e-04*w40*Jinv11-4.2918019480519480e-04*Jinv01*w30-1.0957792207792208e-04*w25*Jinv11+7.4878246753246752e-04*Jinv21*w50+2.0292207792207792e-05*Jinv01*w27+7.4878246753246752e-04*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[53][30]=(0.0)*D;
  A[53][31]=(0.0)*D;
  A[53][32]=(8.2183441558441553e-05*Jinv00*w12+4.2918019480519480e-04*w28*Jinv10+4.3831168831168834e-04*w40*Jinv12+4.0543831168831168e-03*Jinv22*w53-1.3697240259740259e-04*Jinv20*w8+5.5702110389610384e-04*w57*Jinv02+8.2183441558441553e-05*w13*Jinv10+4.3831168831168834e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+1.6436688311688311e-04*w3*Jinv00-5.7528409090909093e-04*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3-2.4655032467532466e-04*Jinv20*w23-9.5880681818181818e-04*Jinv22*w38+2.3833198051948050e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-4.6570616883116886e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.1047077922077924e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-1.3971185064935066e-03*Jinv01*w45+2.0271915584415584e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+1.7349837662337662e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00-4.6570616883116886e-04*Jinv02*w24+8.7662337662337668e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-1.3971185064935066e-03*w45*Jinv00+8.2183441558441553e-04*Jinv21*w41+2.5750811688311690e-03*w42*Jinv02+4.2918019480519480e-04*Jinv00*w27+1.7349837662337662e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+4.2918019480519480e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50+4.2918019480519480e-04*Jinv20*w29-1.3971185064935066e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+1.6436688311688311e-03*Jinv00*w33+2.7394480519480521e-05*w22*Jinv12-5.7528409090909093e-04*w31*Jinv10-5.4788961038961036e-04*Jinv02*w3-2.4655032467532466e-04*w21*Jinv00-7.3965097402597403e-04*w32*Jinv22+8.2183441558441553e-04*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-5.2049512987012989e-04*Jinv00*w18-1.4793019480519481e-03*w36*Jinv00-1.4793019480519481e-03*Jinv21*w38-1.1231737012987013e-03*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24+5.5702110389610384e-04*Jinv22*w59+1.6436688311688311e-03*w34*Jinv11-5.2049512987012989e-04*w19*Jinv11+4.7666396103896100e-03*Jinv21*w53+4.7666396103896100e-03*w51*Jinv00+8.7662337662337668e-04*Jinv12*w34+2.5750811688311690e-03*Jinv22*w44+1.6436688311688311e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+8.2183441558441553e-04*w41*Jinv20-2.7394480519480518e-04*Jinv12*w13-5.2049512987012989e-04*w19*Jinv10+1.6436688311688311e-03*w34*Jinv10-5.7528409090909093e-04*w32*Jinv21-5.2049512987012989e-04*Jinv21*w20-1.3697240259740259e-04*w6*Jinv00-9.5880681818181818e-04*w37*Jinv12+2.3833198051948050e-03*Jinv20*w56+4.3831168831168834e-04*Jinv22*w41-5.4788961038961036e-04*w26*Jinv20+3.1047077922077924e-04*w11*Jinv20+4.0543831168831168e-03*w52*Jinv12+1.6436688311688311e-03*Jinv21*w35-5.7528409090909093e-04*w31*Jinv11-4.6570616883116886e-04*Jinv22*w26-1.3971185064935066e-03*w46*Jinv11+8.2183441558441553e-05*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21+2.0271915584415584e-03*Jinv02*w54-5.4788961038961036e-04*w4*Jinv12+2.7394480519480521e-05*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10+4.2918019480519480e-04*Jinv21*w29+1.6436688311688311e-03*Jinv01*w33+8.2183441558441553e-04*w39*Jinv00-2.7394480519480518e-04*Jinv02*w12-5.2049512987012989e-04*Jinv01*w18-1.4793019480519481e-03*w37*Jinv11+1.7349837662337662e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01-1.3697240259740259e-04*w7*Jinv10-1.3971185064935066e-03*w47*Jinv21-1.4793019480519481e-03*Jinv20*w38+2.7394480519480521e-05*Jinv22*w23+4.7666396103896100e-03*w52*Jinv11-7.3965097402597403e-04*w31*Jinv12-1.1231737012987013e-03*w46*Jinv12+4.7666396103896100e-03*Jinv20*w53+3.8352272727272725e-04*Jinv22*w8-2.4655032467532466e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11+3.8352272727272725e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01+4.7666396103896100e-03*Jinv01*w51-7.3965097402597403e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-1.4793019480519481e-03*w36*Jinv01+4.7666396103896100e-03*w52*Jinv10-1.3697240259740259e-04*w7*Jinv11+3.1047077922077924e-04*w9*Jinv00-1.4793019480519481e-03*w37*Jinv10+2.3833198051948050e-03*Jinv21*w56-5.4788961038961036e-04*w24*Jinv00+3.1960227272727273e-04*Jinv02*w27+3.8352272727272725e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12-5.4788961038961036e-04*Jinv22*w5-1.1231737012987013e-03*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+3.1047077922077924e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+5.5702110389610384e-04*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+8.2183441558441553e-05*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+8.7662337662337668e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+1.6436688311688311e-03*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23+2.3833198051948050e-03*Jinv01*w54+2.3833198051948050e-03*w55*Jinv11-5.4788961038961036e-04*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02+3.1047077922077924e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17-5.7528409090909093e-04*w30*Jinv00+8.2183441558441553e-04*w40*Jinv10-5.2049512987012989e-04*w20*Jinv20+3.1960227272727273e-04*Jinv22*w29-2.7394480519480518e-04*Jinv22*w14+8.2183441558441553e-04*w40*Jinv11-9.5880681818181818e-04*Jinv02*w36+2.5750811688311690e-03*Jinv12*w43+1.6436688311688311e-04*w5*Jinv20-5.7528409090909093e-04*Jinv01*w30+4.0543831168831168e-03*Jinv02*w51+2.0271915584415584e-03*Jinv22*w56+3.1960227272727273e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+2.3833198051948050e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.3971185064935066e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+4.2918019480519480e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+3.1047077922077924e-04*w10*Jinv11)*D;
  A[53][33]=(0.0)*D;
  A[53][34]=(0.0)*D;
  A[53][35]=(-3.2873376623376621e-04*Jinv00*w12-2.7394480519480518e-04*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12-9.5332792207792200e-03*Jinv22*w53+1.3697240259740260e-03*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02-3.2873376623376621e-04*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-2.9586038961038961e-03*Jinv21*w44-6.0267857142857139e-04*w3*Jinv00+4.9310064935064932e-04*w32*Jinv20+1.6436688311688311e-04*Jinv01*w3+2.7942370129870131e-03*Jinv22*w38-2.1367694805194807e-03*Jinv00*w54+1.0957792207792207e-03*Jinv21*w59+1.3149350649350649e-03*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02-5.4788961038961036e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-2.9586038961038961e-03*w42*Jinv01-6.5746753246753243e-04*w11*Jinv21+1.9724025974025973e-03*w16*Jinv11+1.9724025974025973e-03*Jinv21*w17+2.6298701298701297e-03*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-4.3831168831168834e-04*w10*Jinv12+1.3149350649350649e-03*Jinv02*w24-6.9034090909090911e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26-5.4788961038961036e-04*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+6.5746753246753243e-04*w16*Jinv10+8.2183441558441553e-04*Jinv22*w20-2.9586038961038961e-03*w43*Jinv11+1.3149350649350649e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-2.7394480519480518e-04*Jinv00*w27-4.3831168831168834e-04*w11*Jinv22-5.4788961038961036e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.9724025974025973e-03*w15*Jinv01-5.4788961038961036e-04*w28*Jinv11-2.7394480519480518e-04*Jinv20*w29+1.3149350649350649e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+8.2183441558441553e-04*w22*Jinv12+4.9310064935064932e-04*w31*Jinv10+1.8628246753246754e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2+1.7532467532467534e-03*Jinv00*w18+2.7942370129870131e-03*w36*Jinv00+4.9310064935064934e-03*Jinv21*w38+2.5750811688311690e-03*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59-2.9586038961038961e-03*w34*Jinv11+2.1367694805194807e-03*w19*Jinv11-1.2820616883116883e-02*Jinv21*w53-8.5470779220779228e-03*w51*Jinv00-6.9034090909090911e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44-6.0267857142857139e-04*w4*Jinv10+1.6436688311688311e-04*w4*Jinv11+1.7532467532467534e-03*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+2.6298701298701297e-03*w32*Jinv21+2.1367694805194807e-03*Jinv21*w20+1.3697240259740260e-03*w6*Jinv00+2.7942370129870131e-03*w37*Jinv12-2.1367694805194807e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.6436688311688311e-04*w11*Jinv20-9.5332792207792200e-03*w52*Jinv12-2.9586038961038961e-03*Jinv21*w35+2.6298701298701297e-03*w31*Jinv11+1.3149350649350649e-03*Jinv22*w26+2.6298701298701297e-03*w46*Jinv11-3.2873376623376621e-04*Jinv20*w14+1.6436688311688311e-04*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+8.2183441558441553e-04*w21*Jinv02-5.4788961038961036e-04*Jinv21*w29-2.9586038961038961e-03*Jinv01*w33+2.1367694805194807e-03*Jinv01*w18+4.9310064935064934e-03*w37*Jinv11-4.3831168831168834e-04*Jinv02*w9+2.1367694805194807e-03*w6*Jinv01+1.3697240259740260e-03*w7*Jinv10+2.6298701298701297e-03*w47*Jinv21+2.7942370129870131e-03*Jinv20*w38+8.2183441558441553e-04*Jinv22*w23-1.2820616883116883e-02*w52*Jinv11+1.8628246753246754e-03*w31*Jinv12+2.5750811688311690e-03*w46*Jinv12-8.5470779220779228e-03*Jinv20*w53-4.9310064935064932e-04*Jinv22*w8-2.0819805194805196e-03*Jinv22*w50+5.4788961038961042e-05*w22*Jinv11-4.9310064935064932e-04*Jinv12*w7+5.4788961038961042e-05*w21*Jinv01-1.2820616883116883e-02*Jinv01*w51+1.8628246753246754e-03*Jinv02*w30-2.0819805194805196e-03*Jinv12*w49+4.9310064935064934e-03*w36*Jinv01-8.5470779220779228e-03*w52*Jinv10+2.1367694805194807e-03*w7*Jinv11-1.6436688311688311e-04*w9*Jinv00+2.7942370129870131e-03*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-9.6793831168831166e-04*Jinv02*w27-4.9310064935064932e-04*w6*Jinv02+8.2183441558441553e-04*w19*Jinv12+2.5750811688311690e-03*w47*Jinv22-4.2735389610389614e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-1.6436688311688311e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+8.2183441558441553e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+2.1367694805194807e-03*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-6.9034090909090911e-03*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+5.4788961038961042e-05*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+6.5746753246753243e-04*w15*Jinv00-2.0819805194805196e-03*w48*Jinv02-6.5746753246753243e-04*Jinv01*w9+6.5746753246753243e-04*Jinv20*w17+4.9310064935064932e-04*w30*Jinv00+1.7532467532467534e-03*w20*Jinv20-9.6793831168831166e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+2.7942370129870131e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43-6.0267857142857139e-04*w5*Jinv20+2.6298701298701297e-03*Jinv01*w30-9.5332792207792200e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-9.6793831168831166e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+1.3149350649350649e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-4.2735389610389614e-03*Jinv21*w50-5.4788961038961036e-04*Jinv01*w27-4.2735389610389614e-03*w48*Jinv01-6.2094155844155847e-04*Jinv12*w1-6.5746753246753243e-04*w10*Jinv11)*D;
  A[53][36]=(0.0)*D;
  A[53][37]=(0.0)*D;
  A[53][38]=(2.4655032467532466e-04*Jinv00*w12-1.5523538961038962e-04*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12+1.1505681818181819e-03*Jinv22*w53-1.2327516233766234e-03*Jinv20*w8-4.7483766233766234e-04*w57*Jinv02+2.4655032467532466e-04*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+4.3831168831168834e-04*w3*Jinv00+8.2183441558441553e-05*w32*Jinv20+2.4655032467532466e-04*Jinv20*w23-2.6298701298701297e-03*Jinv22*w38-2.4655032467532466e-04*Jinv00*w54+1.3971185064935066e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-6.5746753246753243e-04*w43*Jinv10-1.6436688311688311e-04*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-1.3149350649350649e-03*w15*Jinv02-5.4788961038961036e-04*w10*Jinv12-2.4655032467532466e-04*w48*Jinv00+1.3971185064935066e-03*Jinv02*w24-2.1367694805194807e-03*Jinv22*w35-1.5523538961038962e-04*w1*Jinv10-1.3697240259740260e-03*w16*Jinv10-2.3833198051948050e-03*Jinv22*w20+8.2183441558441553e-05*w45*Jinv00-2.1367694805194807e-03*w42*Jinv02-1.5523538961038962e-04*Jinv00*w27-5.4788961038961036e-04*w11*Jinv22-2.4655032467532466e-04*Jinv20*w50-1.5523538961038962e-04*Jinv20*w29+8.2183441558441553e-05*w46*Jinv10-1.3149350649350649e-03*Jinv22*w17-6.5746753246753243e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+8.2183441558441553e-05*w31*Jinv10+1.3971185064935066e-03*Jinv02*w3+2.4655032467532466e-04*w21*Jinv00+5.2049512987012989e-04*w32*Jinv22-4.2918019480519480e-04*Jinv22*w2-1.2327516233766234e-03*Jinv00*w18-1.3149350649350649e-03*w36*Jinv00+5.2049512987012989e-04*Jinv02*w45-4.7483766233766234e-04*Jinv22*w59+3.7804383116883116e-03*w51*Jinv00-2.1367694805194807e-03*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+4.3831168831168834e-04*w4*Jinv10-8.2183441558441553e-04*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13-1.2327516233766234e-03*w19*Jinv10-6.5746753246753243e-04*w34*Jinv10-1.2327516233766234e-03*w6*Jinv00-2.6298701298701297e-03*w37*Jinv12-2.4655032467532466e-04*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+4.3831168831168834e-04*w26*Jinv20-1.4610389610389610e-04*w11*Jinv20+1.1505681818181819e-03*w52*Jinv12+1.3971185064935066e-03*Jinv22*w26+2.4655032467532466e-04*Jinv20*w14-1.6436688311688311e-04*Jinv02*w54+1.3971185064935066e-03*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.4655032467532466e-04*w49*Jinv10-8.2183441558441553e-04*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12-5.4788961038961036e-04*Jinv02*w9-1.2327516233766234e-03*w7*Jinv10-1.3149350649350649e-03*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+5.2049512987012989e-04*w31*Jinv12+5.2049512987012989e-04*w46*Jinv12+3.7804383116883116e-03*Jinv20*w53-2.3833198051948050e-03*Jinv22*w8+2.4655032467532466e-04*w22*Jinv10-1.6436688311688311e-04*Jinv22*w50-2.3833198051948050e-03*Jinv12*w7+5.2049512987012989e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+3.7804383116883116e-03*w52*Jinv10-1.4610389610389610e-04*w9*Jinv00-1.3149350649350649e-03*w37*Jinv10+4.3831168831168834e-04*w24*Jinv00-4.2918019480519480e-04*Jinv02*w27-2.3833198051948050e-03*w6*Jinv02-2.3833198051948050e-03*w19*Jinv12+1.3971185064935066e-03*Jinv22*w5+5.2049512987012989e-04*w47*Jinv22-1.4610389610389610e-04*w10*Jinv10-6.5746753246753243e-04*Jinv20*w44-4.7483766233766234e-04*w58*Jinv12-2.3833198051948050e-03*Jinv02*w18-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-2.1367694805194807e-03*Jinv02*w33-1.5523538961038962e-04*w0*Jinv00-6.5746753246753243e-04*w35*Jinv20+4.3831168831168834e-04*w25*Jinv10-1.5523538961038962e-04*Jinv20*w2-1.3697240259740260e-03*w15*Jinv00-1.6436688311688311e-04*w48*Jinv02-1.3697240259740260e-03*Jinv20*w17+8.2183441558441553e-05*w30*Jinv00-8.2183441558441553e-04*w40*Jinv10-1.2327516233766234e-03*w20*Jinv20-4.2918019480519480e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-2.6298701298701297e-03*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+4.3831168831168834e-04*w5*Jinv20+1.1505681818181819e-03*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56-4.2918019480519480e-04*Jinv12*w28-2.4655032467532466e-04*w55*Jinv10-6.5746753246753243e-04*w42*Jinv00+8.2183441558441553e-05*w47*Jinv20-1.3149350649350649e-03*w16*Jinv12-4.2918019480519480e-04*Jinv12*w1)*D;
  A[53][39]=(0.0)*D;
  A[53][40]=(0.0)*D;
  A[53][41]=(5.4788961038961042e-05*Jinv00*w12-4.9310064935064932e-04*w28*Jinv10-2.3011363636363637e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+3.8352272727272725e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02+5.4788961038961042e-05*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-9.8620129870129864e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+1.3149350649350649e-03*w32*Jinv20-7.1225649350649346e-04*Jinv01*w3+3.8352272727272725e-04*Jinv20*w23+1.7532467532467534e-03*Jinv22*w38-4.2735389610389614e-03*Jinv00*w54+7.1225649350649346e-04*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02+2.1915584415584417e-04*Jinv21*w2-1.9724025974025973e-03*w43*Jinv10-9.8620129870129864e-04*w42*Jinv01+8.2183441558441553e-04*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10-1.9724025974025973e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12-2.1367694805194807e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+7.1225649350649346e-04*Jinv21*w26+2.1915584415584417e-04*w0*Jinv01-2.7394480519480518e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20-9.8620129870129864e-04*w43*Jinv11+2.1367694805194807e-03*w45*Jinv00-3.2873376623376621e-03*w42*Jinv02-4.9310064935064932e-04*Jinv00*w27-5.2962662337662338e-04*w11*Jinv22+2.1915584415584417e-04*Jinv11*w1-3.2873376623376621e-04*w13*Jinv11-2.1915584415584417e-04*w28*Jinv11-2.1367694805194807e-03*Jinv20*w50-4.9310064935064932e-04*Jinv20*w29+2.1367694805194807e-03*w46*Jinv10-1.9724025974025973e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33+3.2873376623376621e-04*w22*Jinv12+1.3149350649350649e-03*w31*Jinv10+3.8352272727272725e-04*w21*Jinv00+9.3141233766233771e-04*w32*Jinv22-3.1047077922077924e-04*Jinv22*w2+7.6704545454545450e-04*Jinv00*w18+2.1367694805194807e-03*w36*Jinv00+1.7532467532467534e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11+3.8352272727272725e-04*w19*Jinv11-4.2735389610389614e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-7.1225649350649346e-04*w4*Jinv11-9.8620129870129864e-04*w41*Jinv20+7.6704545454545450e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10-8.2183441558441553e-04*w32*Jinv21+3.8352272727272725e-04*Jinv21*w20+3.8352272727272725e-04*w6*Jinv00+1.7532467532467534e-03*w37*Jinv12-4.2735389610389614e-03*Jinv20*w56-2.3011363636363637e-03*Jinv22*w41+7.6704545454545450e-04*w26*Jinv20-4.9310064935064932e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-8.2183441558441553e-04*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+8.2183441558441553e-04*w46*Jinv11+5.4788961038961042e-05*Jinv20*w14-7.1225649350649346e-04*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02-2.1367694805194807e-03*w49*Jinv10-2.1915584415584417e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33-9.8620129870129864e-04*w39*Jinv00+3.8352272727272725e-04*Jinv01*w18-5.2962662337662338e-04*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01+3.8352272727272725e-04*w7*Jinv10+8.2183441558441553e-04*w47*Jinv21+2.1367694805194807e-03*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23+9.3141233766233771e-04*w31*Jinv12+1.7532467532467534e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8+3.8352272727272725e-04*w22*Jinv10-1.0409902597402598e-03*Jinv22*w50+3.2873376623376621e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.2873376623376621e-04*w21*Jinv01+9.3141233766233771e-04*Jinv02*w30-1.0409902597402598e-03*Jinv12*w49-4.2735389610389614e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11-4.9310064935064932e-04*w9*Jinv00+2.1367694805194807e-03*w37*Jinv10-2.1367694805194807e-03*Jinv21*w56+7.6704545454545450e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.7532467532467534e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-3.2873376623376621e-04*Jinv21*w14-4.9310064935064932e-04*w10*Jinv10-1.9724025974025973e-03*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-3.2873376623376621e-04*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33-2.7394480519480518e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+3.2873376623376621e-04*Jinv21*w23-2.1367694805194807e-03*Jinv01*w54-2.1367694805194807e-03*w55*Jinv11+7.6704545454545450e-04*w25*Jinv10-2.7394480519480518e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-1.0409902597402598e-03*w48*Jinv02+1.3149350649350649e-03*Jinv20*w17+1.3149350649350649e-03*w30*Jinv00-9.8620129870129864e-04*w40*Jinv10+7.6704545454545450e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29+1.7532467532467534e-03*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20-8.2183441558441553e-04*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-4.2735389610389614e-03*w55*Jinv10-1.9724025974025973e-03*w42*Jinv00+2.1367694805194807e-03*w47*Jinv20-1.9724025974025973e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50-2.1915584415584417e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1)*D;
  A[53][42]=(0.0)*D;
  A[53][43]=(0.0)*D;
  A[53][44]=(-5.4788961038961042e-05*Jinv00*w12+4.9310064935064932e-04*w28*Jinv10-1.3149350649350649e-03*w40*Jinv12+3.2873376623376621e-03*Jinv22*w53-3.8352272727272725e-04*Jinv20*w8-9.8620129870129864e-04*w57*Jinv02-5.4788961038961042e-05*w13*Jinv10-1.3149350649350649e-03*Jinv02*w39+2.9586038961038961e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-1.3149350649350649e-03*w32*Jinv20-1.6436688311688311e-04*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23-2.1367694805194807e-03*Jinv22*w38+4.2735389610389614e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12-4.2004870129870131e-04*w0*Jinv02+5.4788961038961036e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+2.9586038961038961e-03*w42*Jinv01+6.5746753246753243e-04*w11*Jinv21-1.9724025974025973e-03*w16*Jinv11-1.9724025974025973e-03*Jinv21*w17-2.6298701298701297e-03*Jinv01*w45+2.1915584415584414e-03*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-4.9310064935064934e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24-1.9724025974025973e-03*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26+5.4788961038961036e-04*w0*Jinv01+2.7394480519480518e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.6298701298701297e-03*Jinv22*w20+2.9586038961038961e-03*w43*Jinv11-2.1367694805194807e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+4.9310064935064932e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22+5.4788961038961036e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.9724025974025973e-03*w15*Jinv01+5.4788961038961036e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50+4.9310064935064932e-04*Jinv20*w29-2.1367694805194807e-03*w46*Jinv10-4.9310064935064934e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-5.4788961038961042e-05*w22*Jinv12-1.3149350649350649e-03*w31*Jinv10+1.1505681818181819e-03*Jinv02*w3-3.8352272727272725e-04*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-4.2004870129870131e-04*Jinv22*w2-7.6704545454545450e-04*Jinv00*w18-2.1367694805194807e-03*w36*Jinv00-4.9310064935064934e-03*Jinv21*w38-7.6704545454545450e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-9.8620129870129864e-04*Jinv22*w59+2.9586038961038961e-03*w34*Jinv11-2.1367694805194807e-03*w19*Jinv11+1.2820616883116883e-02*Jinv21*w53+4.2735389610389614e-03*w51*Jinv00-1.9724025974025973e-03*Jinv12*w34-3.9448051948051946e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-1.6436688311688311e-04*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+7.6704545454545450e-04*Jinv12*w13-7.6704545454545450e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-2.6298701298701297e-03*w32*Jinv21-2.1367694805194807e-03*Jinv21*w20-3.8352272727272725e-04*w6*Jinv00-2.1367694805194807e-03*w37*Jinv12+4.2735389610389614e-03*Jinv20*w56-1.3149350649350649e-03*Jinv22*w41-7.6704545454545450e-04*w26*Jinv20+4.9310064935064932e-04*w11*Jinv20+3.2873376623376621e-03*w52*Jinv12+2.9586038961038961e-03*Jinv21*w35-2.6298701298701297e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-2.6298701298701297e-03*w46*Jinv11-5.4788961038961042e-05*Jinv20*w14-1.6436688311688311e-04*w5*Jinv21+2.1915584415584414e-03*Jinv02*w54+1.1505681818181819e-03*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10+5.4788961038961036e-04*Jinv21*w29+2.9586038961038961e-03*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+7.6704545454545450e-04*Jinv02*w12-2.1367694805194807e-03*Jinv01*w18-4.9310064935064934e-03*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-2.1367694805194807e-03*w6*Jinv01-3.8352272727272725e-04*w7*Jinv10-2.6298701298701297e-03*w47*Jinv21-2.1367694805194807e-03*Jinv20*w38-5.4788961038961042e-05*Jinv22*w23+1.2820616883116883e-02*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-7.6704545454545450e-04*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-1.3149350649350649e-03*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-5.4788961038961042e-05*w22*Jinv11-1.3149350649350649e-03*Jinv12*w7-5.4788961038961042e-05*w21*Jinv01+1.2820616883116883e-02*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-4.9310064935064934e-03*w36*Jinv01+4.2735389610389614e-03*w52*Jinv10-2.1367694805194807e-03*w7*Jinv11+4.9310064935064932e-04*w9*Jinv00-2.1367694805194807e-03*w37*Jinv10+4.2735389610389614e-03*Jinv21*w56-7.6704545454545450e-04*w24*Jinv00-7.3051948051948052e-05*Jinv02*w27-1.3149350649350649e-03*w6*Jinv02-2.6298701298701297e-03*w19*Jinv12+1.1505681818181819e-03*Jinv22*w5-7.6704545454545450e-04*w47*Jinv22+4.2735389610389614e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+4.9310064935064932e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44-9.8620129870129864e-04*w58*Jinv12-2.6298701298701297e-03*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-2.1367694805194807e-03*Jinv21*w8-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-1.9724025974025973e-03*Jinv02*w33+2.7394480519480518e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-5.4788961038961042e-05*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11-7.6704545454545450e-04*w25*Jinv10+2.7394480519480518e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+6.5746753246753243e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17-1.3149350649350649e-03*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10-7.6704545454545450e-04*w20*Jinv20-7.3051948051948052e-05*Jinv22*w29+7.6704545454545450e-04*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-2.1367694805194807e-03*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-2.6298701298701297e-03*Jinv01*w30+3.2873376623376621e-03*Jinv02*w51+2.1915584415584414e-03*Jinv22*w56-7.3051948051948052e-05*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11+4.2735389610389614e-03*w55*Jinv10+1.9724025974025973e-03*w42*Jinv00-2.1367694805194807e-03*w47*Jinv20-4.9310064935064934e-03*w16*Jinv12+4.2735389610389614e-03*Jinv21*w50+5.4788961038961036e-04*Jinv01*w27+4.2735389610389614e-03*w48*Jinv01-4.2004870129870131e-04*Jinv12*w1+6.5746753246753243e-04*w10*Jinv11)*D;
  A[53][45]=(0.0)*D;
  A[53][46]=(0.0)*D;
  A[53][47]=(-3.8352272727272725e-04*w40*Jinv12-7.1225649350649346e-04*Jinv22*w53+9.1314935064935065e-06*w57*Jinv02-3.8352272727272725e-04*Jinv02*w39-1.6436688311688311e-03*Jinv21*w44+5.4788961038961036e-04*Jinv01*w3+5.2049512987012989e-04*Jinv22*w38-5.4788961038961036e-04*Jinv21*w59-7.1225649350649346e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-4.2918019480519480e-04*Jinv21*w2-1.6436688311688311e-03*w42*Jinv01-3.1047077922077924e-04*w11*Jinv21-7.1225649350649346e-04*w16*Jinv11-7.1225649350649346e-04*Jinv21*w17+5.7528409090909093e-04*Jinv01*w45-1.6436688311688311e-04*Jinv12*w55-5.4788961038961042e-05*w15*Jinv02-1.3697240259740259e-04*w10*Jinv12-7.1225649350649346e-04*Jinv02*w24-5.4788961038961042e-05*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26-4.2918019480519480e-04*w0*Jinv01+5.2049512987012989e-04*Jinv22*w20-1.6436688311688311e-03*w43*Jinv11-8.2183441558441553e-04*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02-1.3697240259740259e-04*w11*Jinv22-4.2918019480519480e-04*Jinv11*w1+2.4655032467532466e-04*w13*Jinv11-7.1225649350649346e-04*w15*Jinv01-2.1002435064935065e-04*w28*Jinv11-5.4788961038961042e-05*Jinv22*w17-3.5612824675324673e-04*w22*Jinv12+8.2183441558441553e-05*Jinv02*w3+2.7394480519480518e-04*w32*Jinv22-8.2183441558441553e-04*Jinv01*w39-1.0957792207792208e-04*Jinv22*w2+1.4793019480519481e-03*Jinv21*w38-1.6436688311688311e-04*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24+9.1314935064935065e-06*Jinv22*w59-2.6298701298701297e-03*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11-4.7666396103896100e-03*Jinv21*w53-5.4788961038961042e-05*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+5.4788961038961036e-04*w4*Jinv11+2.7394480519480518e-04*Jinv12*w13+1.3971185064935066e-03*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+5.2049512987012989e-04*w37*Jinv12-3.8352272727272725e-04*Jinv22*w41-7.1225649350649346e-04*w52*Jinv12-2.6298701298701297e-03*Jinv21*w35+1.3971185064935066e-03*w31*Jinv11-7.1225649350649346e-04*Jinv22*w26+5.7528409090909093e-04*w46*Jinv11+5.4788961038961036e-04*w5*Jinv21-1.6436688311688311e-04*Jinv02*w54+8.2183441558441553e-05*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02-2.1002435064935065e-04*Jinv21*w29-2.6298701298701297e-03*Jinv01*w33+2.7394480519480518e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18+1.4793019480519481e-03*w37*Jinv11-1.3697240259740259e-04*Jinv02*w9+5.2049512987012989e-04*w6*Jinv01+5.7528409090909093e-04*w47*Jinv21-3.5612824675324673e-04*Jinv22*w23-4.7666396103896100e-03*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8-3.5612824675324673e-04*Jinv22*w50-8.2183441558441553e-05*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-8.2183441558441553e-05*w21*Jinv01-4.7666396103896100e-03*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49+1.4793019480519481e-03*w36*Jinv01+5.2049512987012989e-04*w7*Jinv11-2.4655032467532466e-04*Jinv21*w56+9.1314935064935065e-06*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+5.2049512987012989e-04*w19*Jinv12+8.2183441558441553e-05*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-2.3833198051948050e-03*w49*Jinv11+2.4655032467532466e-04*Jinv21*w14+9.1314935064935065e-06*w58*Jinv12+5.2049512987012989e-04*Jinv02*w18+2.4655032467532466e-04*Jinv01*w12+5.2049512987012989e-04*Jinv21*w8-5.4788961038961042e-05*Jinv02*w33-8.2183441558441553e-05*Jinv21*w23-2.4655032467532466e-04*Jinv01*w54-2.4655032467532466e-04*w55*Jinv11-3.5612824675324673e-04*w48*Jinv02-3.1047077922077924e-04*Jinv01*w9+9.1314935064935065e-06*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-8.2183441558441553e-04*w40*Jinv11+5.2049512987012989e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+1.3971185064935066e-03*Jinv01*w30-7.1225649350649346e-04*Jinv02*w51-1.6436688311688311e-04*Jinv22*w56+9.1314935064935065e-06*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11-5.4788961038961042e-05*w16*Jinv12-2.3833198051948050e-03*Jinv21*w50-2.1002435064935065e-04*Jinv01*w27-2.3833198051948050e-03*w48*Jinv01-1.0957792207792208e-04*Jinv12*w1-3.1047077922077924e-04*w10*Jinv11)*D;
  A[53][48]=(0.0)*D;
  A[53][49]=(0.0)*D;
  A[53][50]=(3.5612824675324673e-04*Jinv00*w12-7.4878246753246752e-04*w28*Jinv10-5.4788961038961036e-04*w40*Jinv12-4.9310064935064934e-03*Jinv22*w53+1.6436688311688311e-04*Jinv20*w8-1.1414366883116883e-03*w57*Jinv02+3.5612824675324673e-04*w13*Jinv10-5.4788961038961036e-04*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+7.1225649350649346e-04*w3*Jinv00+2.3833198051948050e-03*w32*Jinv20+7.1225649350649346e-04*Jinv01*w3+3.5612824675324673e-04*Jinv20*w23+1.3971185064935066e-03*Jinv22*w38-3.6982548701298703e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59+3.5612824675324673e-04*w25*Jinv12-3.5612824675324673e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-7.8530844155844158e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.3833198051948050e-03*Jinv01*w45-2.4655032467532467e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-3.9265422077922079e-04*w10*Jinv12-3.6982548701298703e-03*w48*Jinv00+3.5612824675324673e-04*Jinv02*w24-1.0957792207792207e-03*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+7.1225649350649346e-04*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10+8.2183441558441553e-05*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+2.3833198051948050e-03*w45*Jinv00-2.1367694805194807e-03*Jinv21*w41-2.6846590909090908e-03*w42*Jinv02-7.4878246753246752e-04*Jinv00*w27-3.9265422077922079e-04*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.4878246753246752e-04*w28*Jinv11-3.6982548701298703e-03*Jinv20*w50-7.4878246753246752e-04*Jinv20*w29+2.3833198051948050e-03*w46*Jinv10-4.2735389610389614e-03*Jinv00*w33+2.3833198051948050e-03*w31*Jinv10+5.4788961038961036e-04*Jinv02*w3+3.5612824675324673e-04*w21*Jinv00+1.1505681818181819e-03*w32*Jinv22-2.1367694805194807e-03*Jinv01*w39-3.5612824675324673e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+2.6298701298701297e-03*w36*Jinv00+2.6298701298701297e-03*Jinv21*w38+1.3697240259740260e-03*Jinv02*w45+7.1225649350649346e-04*Jinv01*w24-1.1414366883116883e-03*Jinv22*w59-4.2735389610389614e-03*w34*Jinv11+1.6436688311688311e-04*w19*Jinv11-7.3965097402597406e-03*Jinv21*w53-7.3965097402597406e-03*w51*Jinv00-1.0957792207792207e-03*Jinv12*w34-2.6846590909090908e-03*Jinv22*w44+7.1225649350649346e-04*w4*Jinv10+7.1225649350649346e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+2.7394480519480518e-04*Jinv12*w13+1.6436688311688311e-04*w19*Jinv10-4.2735389610389614e-03*w34*Jinv10+2.3833198051948050e-03*w32*Jinv21+1.6436688311688311e-04*Jinv21*w20+1.6436688311688311e-04*w6*Jinv00+1.3971185064935066e-03*w37*Jinv12-3.6982548701298703e-03*Jinv20*w56-5.4788961038961036e-04*Jinv22*w41+7.1225649350649346e-04*w26*Jinv20-7.8530844155844158e-04*w11*Jinv20-4.9310064935064934e-03*w52*Jinv12-4.2735389610389614e-03*Jinv21*w35+2.3833198051948050e-03*w31*Jinv11+3.5612824675324673e-04*Jinv22*w26+2.3833198051948050e-03*w46*Jinv11+3.5612824675324673e-04*Jinv20*w14+7.1225649350649346e-04*w5*Jinv21-2.4655032467532467e-03*Jinv02*w54+5.4788961038961036e-04*w4*Jinv12-3.6982548701298703e-03*w49*Jinv10-7.4878246753246752e-04*Jinv21*w29-4.2735389610389614e-03*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+2.7394480519480518e-04*Jinv02*w12+1.6436688311688311e-04*Jinv01*w18+2.6298701298701297e-03*w37*Jinv11-3.9265422077922079e-04*Jinv02*w9+1.6436688311688311e-04*w6*Jinv01+1.6436688311688311e-04*w7*Jinv10+2.3833198051948050e-03*w47*Jinv21+2.6298701298701297e-03*Jinv20*w38-7.3965097402597406e-03*w52*Jinv11+1.1505681818181819e-03*w31*Jinv12+1.3697240259740260e-03*w46*Jinv12-7.3965097402597406e-03*Jinv20*w53+3.5612824675324673e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11+3.5612824675324673e-04*w21*Jinv01-7.3965097402597406e-03*Jinv01*w51+1.1505681818181819e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+2.6298701298701297e-03*w36*Jinv01-7.3965097402597406e-03*w52*Jinv10+1.6436688311688311e-04*w7*Jinv11-7.8530844155844158e-04*w9*Jinv00+2.6298701298701297e-03*w37*Jinv10-3.6982548701298703e-03*Jinv21*w56+7.1225649350649346e-04*w24*Jinv00-3.7439123376623376e-04*Jinv02*w27+8.2183441558441553e-05*w19*Jinv12+5.4788961038961036e-04*Jinv22*w5+1.3697240259740260e-03*w47*Jinv22-3.6982548701298703e-03*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14-7.8530844155844158e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-1.1414366883116883e-03*w58*Jinv12+8.2183441558441553e-05*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12+1.6436688311688311e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-1.0957792207792207e-03*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-4.2735389610389614e-03*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23-3.6982548701298703e-03*Jinv01*w54-3.6982548701298703e-03*w55*Jinv11+7.1225649350649346e-04*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-7.8530844155844158e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.3833198051948050e-03*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-3.7439123376623376e-04*Jinv22*w29+2.7394480519480518e-04*Jinv22*w14-2.1367694805194807e-03*w40*Jinv11+1.3971185064935066e-03*Jinv02*w36-2.6846590909090908e-03*Jinv12*w43+7.1225649350649346e-04*w5*Jinv20+2.3833198051948050e-03*Jinv01*w30-4.9310064935064934e-03*Jinv02*w51-2.4655032467532467e-03*Jinv22*w56-3.7439123376623376e-04*Jinv12*w28+7.1225649350649346e-04*w25*Jinv11-3.6982548701298703e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00+2.3833198051948050e-03*w47*Jinv20-3.6982548701298703e-03*Jinv21*w50-7.4878246753246752e-04*Jinv01*w27-3.6982548701298703e-03*w48*Jinv01-3.5612824675324673e-04*Jinv12*w1-7.8530844155844158e-04*w10*Jinv11)*D;
  A[53][51]=(0.0)*D;
  A[53][52]=(0.0)*D;
  A[53][53]=(-3.5612824675324673e-04*Jinv00*w12+7.4878246753246752e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+7.3965097402597406e-03*Jinv22*w53-1.6436688311688311e-04*Jinv20*w8-9.1314935064935068e-05*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-7.1225649350649346e-04*w3*Jinv00-2.3833198051948050e-03*w32*Jinv20-3.5612824675324673e-04*Jinv20*w23-1.3149350649350649e-03*Jinv22*w38+3.6982548701298703e-03*Jinv00*w54-1.0683847402597404e-03*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+4.2735389610389614e-03*w43*Jinv10+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+6.5746753246753243e-04*w10*Jinv12+3.6982548701298703e-03*w48*Jinv00-1.0683847402597404e-03*Jinv02*w24+4.7666396103896100e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10+5.7528409090909093e-04*Jinv22*w20-2.3833198051948050e-03*w45*Jinv00+4.7666396103896100e-03*w42*Jinv02+7.4878246753246752e-04*Jinv00*w27+6.5746753246753243e-04*w11*Jinv22+3.6982548701298703e-03*Jinv20*w50+7.4878246753246752e-04*Jinv20*w29-2.3833198051948050e-03*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17+4.2735389610389614e-03*Jinv00*w33-3.5612824675324673e-04*w22*Jinv12-2.3833198051948050e-03*w31*Jinv10-1.0683847402597404e-03*Jinv02*w3-3.5612824675324673e-04*w21*Jinv00-2.0271915584415584e-03*w32*Jinv22+7.4878246753246752e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-2.6298701298701297e-03*w36*Jinv00-2.0271915584415584e-03*Jinv02*w45-9.1314935064935068e-05*Jinv22*w59+7.3965097402597406e-03*w51*Jinv00+4.7666396103896100e-03*Jinv12*w34+4.7666396103896100e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv10+2.1367694805194807e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10+4.2735389610389614e-03*w34*Jinv10-1.6436688311688311e-04*w6*Jinv00-1.3149350649350649e-03*w37*Jinv12+3.6982548701298703e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-7.1225649350649346e-04*w26*Jinv20+7.8530844155844158e-04*w11*Jinv20+7.3965097402597406e-03*w52*Jinv12-1.0683847402597404e-03*Jinv22*w26-3.5612824675324673e-04*Jinv20*w14+2.4655032467532467e-03*Jinv02*w54-1.0683847402597404e-03*w4*Jinv12-3.5612824675324673e-04*w21*Jinv02+3.6982548701298703e-03*w49*Jinv10+2.1367694805194807e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+6.5746753246753243e-04*Jinv02*w9-1.6436688311688311e-04*w7*Jinv10-2.6298701298701297e-03*Jinv20*w38-3.5612824675324673e-04*Jinv22*w23-2.0271915584415584e-03*w31*Jinv12-2.0271915584415584e-03*w46*Jinv12+7.3965097402597406e-03*Jinv20*w53+5.7528409090909093e-04*Jinv22*w8-3.5612824675324673e-04*w22*Jinv10+2.4655032467532467e-03*Jinv22*w50+5.7528409090909093e-04*Jinv12*w7-2.0271915584415584e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49+7.3965097402597406e-03*w52*Jinv10+7.8530844155844158e-04*w9*Jinv00-2.6298701298701297e-03*w37*Jinv10-7.1225649350649346e-04*w24*Jinv00+7.4878246753246752e-04*Jinv02*w27+5.7528409090909093e-04*w6*Jinv02+5.7528409090909093e-04*w19*Jinv12-1.0683847402597404e-03*Jinv22*w5-2.0271915584415584e-03*w47*Jinv22+7.8530844155844158e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-9.1314935064935068e-05*w58*Jinv12+5.7528409090909093e-04*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+4.7666396103896100e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+4.2735389610389614e-03*w35*Jinv20-7.1225649350649346e-04*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+2.4655032467532467e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.3833198051948050e-03*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+7.4878246753246752e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-1.3149350649350649e-03*Jinv02*w36+4.7666396103896100e-03*Jinv12*w43-7.1225649350649346e-04*w5*Jinv20+7.3965097402597406e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+7.4878246753246752e-04*Jinv12*w28+3.6982548701298703e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00-2.3833198051948050e-03*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12+7.4878246753246752e-04*Jinv12*w1)*D;
  A[53][54]=(0.0)*D;
  A[53][55]=(0.0)*D;
  A[53][56]=(1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-4.5657467532467534e-05*w57*Jinv02+1.5888798701298701e-03*Jinv02*w39+4.2735389610389614e-03*Jinv21*w44-7.1225649350649346e-04*Jinv01*w3-1.2327516233766234e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59-1.6436688311688311e-04*w25*Jinv12+3.7439123376623376e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w42*Jinv01+7.8530844155844158e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.3833198051948050e-03*Jinv01*w45+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12-1.6436688311688311e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-7.1225649350649346e-04*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-1.6436688311688311e-04*Jinv22*w20+4.2735389610389614e-03*w43*Jinv11+2.1367694805194807e-03*Jinv21*w41+3.1777597402597403e-03*w42*Jinv02+3.9265422077922079e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-3.5612824675324673e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.4878246753246752e-04*w28*Jinv11+1.0957792207792207e-03*Jinv22*w17-8.2183441558441553e-05*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-1.0135957792207792e-03*w32*Jinv22+2.1367694805194807e-03*Jinv01*w39+3.7439123376623376e-04*Jinv22*w2-2.6298701298701297e-03*Jinv21*w38-1.2327516233766234e-03*Jinv02*w45-7.1225649350649346e-04*Jinv01*w24-4.5657467532467534e-05*Jinv22*w59+4.2735389610389614e-03*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+7.3965097402597406e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+3.1777597402597403e-03*Jinv22*w44-7.1225649350649346e-04*w4*Jinv11-3.5612824675324673e-04*Jinv12*w13-2.3833198051948050e-03*w32*Jinv21-1.6436688311688311e-04*Jinv21*w20-1.2327516233766234e-03*w37*Jinv12+1.5888798701298701e-03*Jinv22*w41+2.4655032467532467e-03*w52*Jinv12+4.2735389610389614e-03*Jinv21*w35-2.3833198051948050e-03*w31*Jinv11-1.6436688311688311e-04*Jinv22*w26-2.3833198051948050e-03*w46*Jinv11-7.1225649350649346e-04*w5*Jinv21+2.4655032467532467e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02+7.4878246753246752e-04*Jinv21*w29+4.2735389610389614e-03*Jinv01*w33-3.5612824675324673e-04*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.6298701298701297e-03*w37*Jinv11+3.9265422077922079e-04*Jinv02*w9-1.6436688311688311e-04*w6*Jinv01-2.3833198051948050e-03*w47*Jinv21-8.2183441558441553e-05*Jinv22*w23+7.3965097402597406e-03*w52*Jinv11-1.0135957792207792e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12-8.2183441558441553e-05*Jinv22*w8+1.2327516233766234e-03*Jinv22*w50-3.5612824675324673e-04*w22*Jinv11-8.2183441558441553e-05*Jinv12*w7-3.5612824675324673e-04*w21*Jinv01+7.3965097402597406e-03*Jinv01*w51-1.0135957792207792e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49-2.6298701298701297e-03*w36*Jinv01-1.6436688311688311e-04*w7*Jinv11+3.6982548701298703e-03*Jinv21*w56+3.9265422077922079e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+3.6982548701298703e-03*w49*Jinv11-3.5612824675324673e-04*Jinv21*w14-4.5657467532467534e-05*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-3.5612824675324673e-04*Jinv01*w12-1.6436688311688311e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-3.5612824675324673e-04*Jinv21*w23+3.6982548701298703e-03*Jinv01*w54+3.6982548701298703e-03*w55*Jinv11+1.2327516233766234e-03*w48*Jinv02+7.8530844155844158e-04*Jinv01*w9+3.9265422077922079e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14+2.1367694805194807e-03*w40*Jinv11-1.2327516233766234e-03*Jinv02*w36+3.1777597402597403e-03*Jinv12*w43-2.3833198051948050e-03*Jinv01*w30+2.4655032467532467e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28-7.1225649350649346e-04*w25*Jinv11+1.0957792207792207e-03*w16*Jinv12+3.6982548701298703e-03*Jinv21*w50+7.4878246753246752e-04*Jinv01*w27+3.6982548701298703e-03*w48*Jinv01+3.7439123376623376e-04*Jinv12*w1+7.8530844155844158e-04*w10*Jinv11)*D;
  A[53][57]=(0.0)*D;
  A[53][58]=(0.0)*D;
  A[53][59]=(3.2873376623376621e-04*w40*Jinv12+2.3741883116883116e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+3.2873376623376621e-04*Jinv02*w39-1.0227272727272728e-03*Jinv22*w38-1.0957792207792208e-04*w25*Jinv12+1.5726461038961039e-04*w0*Jinv02+1.1870941558441558e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+2.0495129870129870e-04*w10*Jinv12-1.0957792207792208e-04*Jinv02*w24+6.5746753246753243e-04*Jinv22*w35-2.6481331168831169e-04*Jinv22*w20+6.5746753246753243e-04*w42*Jinv02+2.0495129870129870e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-5.4788961038961042e-05*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-6.7573051948051951e-04*w32*Jinv22+1.5726461038961039e-04*Jinv22*w2-6.7573051948051951e-04*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+6.5746753246753243e-04*Jinv12*w34+6.5746753246753243e-04*Jinv22*w44-5.4788961038961042e-05*Jinv12*w13-1.0227272727272728e-03*w37*Jinv12+3.2873376623376621e-04*Jinv22*w41+2.3741883116883116e-03*w52*Jinv12-1.0957792207792208e-04*Jinv22*w26+1.1870941558441558e-03*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12-5.4788961038961042e-05*w21*Jinv02-5.4788961038961042e-05*Jinv02*w12+2.0495129870129870e-04*Jinv02*w9-5.4788961038961042e-05*Jinv22*w23-6.7573051948051951e-04*w31*Jinv12-6.7573051948051951e-04*w46*Jinv12-2.6481331168831169e-04*Jinv22*w8+1.1870941558441558e-03*Jinv22*w50-2.6481331168831169e-04*Jinv12*w7-6.7573051948051951e-04*Jinv02*w30+1.1870941558441558e-03*Jinv12*w49+1.5726461038961039e-04*Jinv02*w27-2.6481331168831169e-04*w6*Jinv02-2.6481331168831169e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5-6.7573051948051951e-04*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-2.6481331168831169e-04*Jinv02*w18+6.5746753246753243e-04*Jinv02*w33+1.1870941558441558e-03*w48*Jinv02+1.5726461038961039e-04*Jinv22*w29-5.4788961038961042e-05*Jinv22*w14-1.0227272727272728e-03*Jinv02*w36+6.5746753246753243e-04*Jinv12*w43+2.3741883116883116e-03*Jinv02*w51+1.1870941558441558e-03*Jinv22*w56+1.5726461038961039e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+1.5726461038961039e-04*Jinv12*w1)*D;
  A[54][0]=(1.0957792207792208e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12-7.4878246753246752e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+2.6481331168831169e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38-1.4975649350649350e-03*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-3.4699675324675324e-04*w16*Jinv11-3.4699675324675324e-04*Jinv21*w17+5.8441558441558442e-04*Jinv01*w45-1.4975649350649350e-03*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.4699675324675324e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-3.4699675324675324e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+5.8441558441558442e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-3.4699675324675324e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+5.8441558441558442e-04*w46*Jinv10-3.4699675324675324e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+2.6481331168831169e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+2.6481331168831169e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38+5.8441558441558442e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-2.1915584415584417e-04*w34*Jinv11+2.1915584415584417e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+2.6481331168831169e-04*w32*Jinv21+2.1915584415584417e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12-1.4975649350649350e-03*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-7.4878246753246752e-04*w52*Jinv12-2.1915584415584417e-04*Jinv21*w35+2.6481331168831169e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+5.8441558441558442e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-1.4975649350649350e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-2.1915584415584417e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12+2.1915584415584417e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10+5.8441558441558442e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11+2.6481331168831169e-04*w31*Jinv12+5.8441558441558442e-04*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51+2.6481331168831169e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-1.4975649350649350e-03*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5+5.8441558441558442e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23-1.4975649350649350e-03*Jinv01*w54-1.4975649350649350e-03*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-3.4699675324675324e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-3.4699675324675324e-04*Jinv20*w17+2.6481331168831169e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+2.6481331168831169e-04*Jinv01*w30-7.4878246753246752e-04*Jinv02*w51-1.4975649350649350e-03*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11-1.4975649350649350e-03*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+5.8441558441558442e-04*w47*Jinv20-3.4699675324675324e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[54][1]=(0.0)*D;
  A[54][2]=(0.0)*D;
  A[54][3]=(-3.5612824675324673e-04*Jinv00*w12+7.1225649350649346e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44-8.2183441558441553e-05*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-9.1314935064935065e-06*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+8.7662337662337668e-04*w16*Jinv11+8.7662337662337668e-04*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+8.7662337662337668e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+8.2183441558441553e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*w11*Jinv22-6.3920454545454550e-05*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+8.7662337662337668e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+8.7662337662337668e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-4.6570616883116886e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+7.1225649350649346e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-4.6570616883116886e-04*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+7.1225649350649346e-04*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+2.7394480519480521e-05*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+7.1225649350649346e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-3.5612824675324673e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+7.1225649350649346e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.9176136363636362e-04*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-1.9176136363636362e-04*Jinv12*w49-5.2049512987012989e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+2.7394480519480521e-05*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-1.9176136363636362e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+2.7394480519480521e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+7.1225649350649346e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+8.2183441558441553e-04*w15*Jinv00-1.9176136363636362e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+8.2183441558441553e-04*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-4.6570616883116886e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+8.2183441558441553e-05*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+8.7662337662337668e-04*w16*Jinv12-1.9176136363636362e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-1.9176136363636362e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[54][4]=(0.0)*D;
  A[54][5]=(0.0)*D;
  A[54][6]=(4.6570616883116886e-04*Jinv00*w12+5.4788961038961042e-05*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-9.1314935064935065e-06*w57*Jinv02+4.6570616883116886e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480521e-05*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-9.1314935064935065e-06*Jinv21*w59-5.2049512987012989e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+5.4788961038961042e-05*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01-2.7394480519480521e-05*w1*Jinv10-8.2183441558441553e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+5.4788961038961042e-05*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*w11*Jinv22+1.0957792207792208e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+5.4788961038961042e-05*Jinv22*w17-3.2873376623376621e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12+2.4655032467532466e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480521e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+1.6436688311688311e-04*w37*Jinv12+5.4788961038961042e-05*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11+4.6570616883116886e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01-5.2049512987012989e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8+3.8352272727272725e-04*Jinv02*w33-2.7394480519480521e-05*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-2.7394480519480521e-05*Jinv20*w2-8.2183441558441553e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-8.2183441558441553e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480521e-05*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[54][7]=(0.0)*D;
  A[54][8]=(0.0)*D;
  A[54][9]=(-2.1915584415584417e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-2.1915584415584417e-04*w13*Jinv10-2.7394480519480521e-05*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20-4.2918019480519480e-04*Jinv20*w23+1.4975649350649350e-03*Jinv00*w54+4.3831168831168834e-04*w43*Jinv10+6.4935064935064935e-05*w58*Jinv10+7.4878246753246752e-04*w48*Jinv00+1.0247564935064935e-04*w1*Jinv10+3.4699675324675324e-04*w16*Jinv10-5.8441558441558442e-04*w45*Jinv00-1.4204545454545455e-05*Jinv00*w27+7.4878246753246752e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-5.8441558441558442e-04*w46*Jinv10+2.1915584415584417e-04*Jinv00*w33-4.2918019480519480e-04*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-1.0957792207792208e-04*Jinv00*w18-2.6481331168831169e-04*w36*Jinv00+3.6525974025974026e-05*w51*Jinv00-2.7394480519480521e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-1.0957792207792208e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00+1.4975649350649350e-03*Jinv20*w56+1.1870941558441559e-04*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-2.1915584415584417e-04*Jinv20*w14+7.4878246753246752e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-5.4788961038961042e-05*w7*Jinv10-2.6481331168831169e-04*Jinv20*w38+3.6525974025974026e-05*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10+3.6525974025974026e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-2.6481331168831169e-04*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+3.4699675324675324e-04*w15*Jinv00+3.4699675324675324e-04*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20-2.7394480519480521e-05*w5*Jinv20+1.4975649350649350e-03*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-5.8441558441558442e-04*w47*Jinv20)*D;
  A[54][10]=(0.0)*D;
  A[54][11]=(0.0)*D;
  A[54][12]=(1.0683847402597404e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-4.6570616883116886e-04*Jinv20*w8-1.8262987012987013e-05*w57*Jinv02+1.0683847402597404e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+1.2601461038961039e-03*w16*Jinv11+1.2601461038961039e-03*Jinv21*w17-9.5880681818181818e-04*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-1.8262987012987013e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-1.1231737012987013e-03*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11-1.4793019480519481e-03*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.2601461038961039e-03*w15*Jinv01-3.5612824675324673e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-1.4793019480519481e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38-1.4793019480519481e-03*Jinv02*w45-1.1231737012987013e-03*Jinv01*w24-1.8262987012987013e-05*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+2.1367694805194807e-03*w41*Jinv20+1.0683847402597404e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-1.6436688311688311e-04*Jinv21*w20-4.6570616883116886e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41-1.3971185064935066e-03*w26*Jinv20+2.1915584415584417e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35-1.3971185064935066e-03*Jinv22*w26-9.5880681818181818e-04*w46*Jinv11+1.0683847402597404e-03*Jinv20*w14+2.1367694805194807e-03*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-3.5612824675324673e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00+1.0683847402597404e-03*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-1.9176136363636362e-04*w6*Jinv01-4.6570616883116886e-04*w7*Jinv10-9.5880681818181818e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-1.4793019480519481e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-4.6570616883116886e-04*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11-4.6570616883116886e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-1.9176136363636362e-04*w7*Jinv11+2.1915584415584417e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+1.4245129870129869e-03*Jinv21*w56-1.3971185064935066e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27-4.6570616883116886e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-1.4793019480519481e-03*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+2.1915584415584417e-04*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-1.8262987012987013e-05*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-1.9176136363636362e-04*Jinv21*w8-1.8262987012987013e-05*w57*Jinv00-1.8262987012987013e-05*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23+1.4245129870129869e-03*Jinv01*w54+1.4245129870129869e-03*w55*Jinv11-1.3971185064935066e-03*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.0683847402597404e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+7.1225649350649346e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-1.1231737012987013e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-1.4793019480519481e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50-3.5612824675324673e-04*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[54][13]=(0.0)*D;
  A[54][14]=(0.0)*D;
  A[54][15]=(-1.3149350649350649e-03*Jinv00*w12+1.9724025974025973e-03*w40*Jinv12-2.1915584415584414e-03*Jinv22*w53+8.2183441558441553e-04*Jinv20*w8+9.8620129870129864e-04*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+1.9724025974025973e-03*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-8.2183441558441553e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-8.2183441558441553e-04*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38+4.9310064935064932e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+4.2004870129870131e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.6436688311688311e-03*w16*Jinv11+1.6436688311688311e-03*Jinv21*w17+1.7532467532467534e-03*Jinv01*w45-3.2873376623376621e-03*Jinv12*w55+4.9310064935064934e-03*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01+2.5750811688311690e-03*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+3.4699675324675324e-04*w1*Jinv10+1.6436688311688311e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+3.9448051948051946e-03*w42*Jinv02+7.3051948051948052e-05*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.2053571428571428e-03*w13*Jinv11+1.6436688311688311e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+1.0957792207792207e-03*Jinv20*w50+4.9310064935064934e-03*Jinv22*w17+1.3149350649350649e-03*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-7.6704545454545450e-04*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39+4.2004870129870131e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.8352272727272725e-04*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+2.5750811688311690e-03*Jinv01*w24+9.8620129870129864e-04*Jinv22*w59+1.0957792207792208e-04*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+3.9448051948051946e-03*Jinv22*w44-8.2183441558441553e-04*w4*Jinv10-8.2183441558441553e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-3.2873376623376621e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+8.2183441558441553e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12+1.9724025974025973e-03*Jinv22*w41+1.3149350649350649e-03*w26*Jinv20-3.4699675324675324e-04*w11*Jinv20-2.1915584415584414e-03*w52*Jinv12-3.2873376623376621e-04*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.7532467532467534e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-8.2183441558441553e-04*w5*Jinv21-3.2873376623376621e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-1.9724025974025973e-03*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9+8.2183441558441553e-04*w7*Jinv10+1.7532467532467534e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-1.0957792207792207e-03*Jinv22*w50+1.2601461038961039e-03*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+1.2601461038961039e-03*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10-3.4699675324675324e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.1915584415584414e-03*Jinv21*w56+1.3149350649350649e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-1.2053571428571428e-03*Jinv21*w14-3.4699675324675324e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+9.8620129870129864e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-1.2053571428571428e-03*Jinv01*w12+1.3149350649350649e-03*Jinv02*w33+3.4699675324675324e-04*w0*Jinv00+1.2601461038961039e-03*Jinv21*w23-2.1915584415584414e-03*Jinv01*w54-2.1915584415584414e-03*w55*Jinv11+1.3149350649350649e-03*w25*Jinv10+3.4699675324675324e-04*Jinv20*w2-1.0957792207792207e-03*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-7.1225649350649346e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36+3.9448051948051946e-03*Jinv12*w43-8.2183441558441553e-04*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30-2.1915584415584414e-03*Jinv02*w51-3.2873376623376621e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+2.5750811688311690e-03*w25*Jinv11+1.9724025974025973e-03*w42*Jinv00+4.9310064935064934e-03*w16*Jinv12-2.5568181818181820e-04*Jinv01*w27+4.2004870129870131e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[54][16]=(0.0)*D;
  A[54][17]=(0.0)*D;
  A[54][18]=(2.4655032467532466e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+2.4655032467532466e-04*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+4.6570616883116886e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3+1.3971185064935066e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.0957792207792208e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45+1.8262987012987013e-05*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-2.1915584415584417e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11+1.4793019480519481e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27-1.0957792207792208e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+1.4793019480519481e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10+1.3971185064935066e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-1.0683847402597404e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+5.2049512987012989e-04*Jinv01*w24-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+4.6570616883116886e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20-1.0683847402597404e-03*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+8.2183441558441553e-05*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20-3.8352272727272725e-04*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11+5.2049512987012989e-04*w46*Jinv11+2.4655032467532466e-04*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.5612824675324673e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+5.2049512987012989e-04*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11+3.8352272727272725e-04*Jinv20*w53+1.3971185064935066e-03*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+8.2183441558441553e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.8262987012987013e-05*w57*Jinv00+1.8262987012987013e-05*Jinv20*w59-2.1915584415584417e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10-2.1915584415584417e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.0683847402597404e-03*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+4.6570616883116886e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+5.2049512987012989e-04*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.4793019480519481e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[54][19]=(0.0)*D;
  A[54][20]=(0.0)*D;
  A[54][21]=(-1.3971185064935066e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+4.7483766233766234e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-2.7394480519480521e-05*Jinv01*w3+2.3833198051948050e-03*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-1.1505681818181819e-03*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21-1.2053571428571428e-03*w16*Jinv11-1.2053571428571428e-03*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-1.1505681818181819e-03*Jinv12*w55+4.7483766233766234e-04*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+1.3697240259740260e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-6.0267857142857139e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-9.5880681818181818e-04*w13*Jinv11-1.2053571428571428e-03*w15*Jinv01+6.6659902597402602e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.3833198051948050e-03*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+2.3833198051948050e-03*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22-6.0267857142857139e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+1.3697240259740260e-03*Jinv01*w24+4.7483766233766234e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480521e-05*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-3.8352272727272725e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12-1.1505681818181819e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14-2.7394480519480521e-05*w5*Jinv21-1.1505681818181819e-03*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+2.3833198051948050e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+6.6659902597402602e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9+1.9176136363636362e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.3971185064935066e-03*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+2.3833198051948050e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+2.3833198051948050e-03*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+1.9176136363636362e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.2873376623376621e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11-9.5880681818181818e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+4.7483766233766234e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18-9.5880681818181818e-04*Jinv01*w12+1.9176136363636362e-04*Jinv21*w8+4.7483766233766234e-04*w57*Jinv00+4.7483766233766234e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+3.2873376623376621e-04*Jinv01*w54+3.2873376623376621e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14-6.0267857142857139e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-3.8352272727272725e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51-1.1505681818181819e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.3697240259740260e-03*w25*Jinv11-1.1505681818181819e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50+6.6659902597402602e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[54][22]=(0.0)*D;
  A[54][23]=(0.0)*D;
  A[54][24]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-2.7394480519480520e-03*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+4.6570616883116886e-04*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.1505681818181819e-03*Jinv00*w54-4.8396915584415583e-04*Jinv21*w59-3.1960227272727273e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-2.7394480519480520e-03*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-2.5202922077922079e-03*w16*Jinv11-2.5202922077922079e-03*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45-4.7483766233766234e-04*w58*Jinv10-1.6436688311688311e-04*w48*Jinv00-4.8396915584415583e-04*w58*Jinv11-4.8396915584415583e-04*w57*Jinv01-2.0271915584415584e-03*Jinv21*w26-3.1960227272727273e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.7394480519480520e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-3.1960227272727273e-04*Jinv11*w1+8.4922889610389610e-04*w13*Jinv11-2.5202922077922079e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-1.6436688311688311e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.3971185064935066e-03*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+1.3697240259740259e-04*Jinv21*w38-2.0271915584415584e-03*Jinv01*w24-7.1225649350649346e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+4.6570616883116886e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+1.3971185064935066e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00+1.1505681818181819e-03*Jinv20*w56-2.3833198051948050e-03*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-7.1225649350649346e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.2327516233766234e-03*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+4.6570616883116886e-04*w5*Jinv21-1.6436688311688311e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+4.3831168831168834e-04*Jinv01*w18+1.3697240259740259e-04*w37*Jinv11+2.4655032467532466e-04*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21+5.2049512987012989e-04*Jinv20*w38+3.8352272727272725e-04*w52*Jinv11-1.6436688311688311e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-1.0135957792207792e-03*w22*Jinv11-1.0135957792207792e-03*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51+1.3697240259740259e-04*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11-4.2918019480519480e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+1.4793019480519481e-03*Jinv21*w56-2.3833198051948050e-03*w24*Jinv00+1.9176136363636362e-04*w49*Jinv11+8.4922889610389610e-04*Jinv21*w14-4.2918019480519480e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+8.4922889610389610e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-4.7483766233766234e-04*w57*Jinv00-4.7483766233766234e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.0135957792207792e-03*Jinv21*w23+1.4793019480519481e-03*Jinv01*w54+1.4793019480519481e-03*w55*Jinv11-2.3833198051948050e-03*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-1.7349837662337662e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30-2.0271915584415584e-03*w25*Jinv11+1.1505681818181819e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+1.9176136363636362e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.9176136363636362e-04*w48*Jinv01-1.7349837662337662e-04*w10*Jinv11)*D;
  A[54][25]=(0.0)*D;
  A[54][26]=(0.0)*D;
  A[54][27]=(-3.2873376623376621e-04*Jinv21*w44-2.7394480519480521e-05*Jinv01*w3+8.3198051948051951e-05*Jinv21*w59-1.1668019480519480e-04*Jinv21*w2-3.2873376623376621e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21+2.9220779220779221e-04*w16*Jinv11+2.9220779220779221e-04*Jinv21*w17-1.0227272727272728e-03*Jinv01*w45+8.3198051948051951e-05*w58*Jinv11+8.3198051948051951e-05*w57*Jinv01-6.7573051948051951e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-3.2873376623376621e-04*w43*Jinv11-3.2873376623376621e-04*Jinv21*w41-1.1668019480519480e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11+2.9220779220779221e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11-3.2873376623376621e-04*Jinv01*w39+1.1870941558441559e-04*Jinv21*w38-6.7573051948051951e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11+3.6525974025974026e-05*Jinv21*w53-2.7394480519480521e-05*w4*Jinv11+1.1870941558441559e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35+1.1870941558441559e-04*w31*Jinv11-1.0227272727272728e-03*w46*Jinv11-2.7394480519480521e-05*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+4.9310064935064932e-04*Jinv01*w18+1.1870941558441559e-04*w37*Jinv11-2.7394480519480521e-05*w6*Jinv01-1.0227272727272728e-03*w47*Jinv21+3.6525974025974026e-05*w52*Jinv11-6.7573051948051951e-04*w22*Jinv11-6.7573051948051951e-04*w21*Jinv01+3.6525974025974026e-05*Jinv01*w51+1.1870941558441559e-04*w36*Jinv01-2.7394480519480521e-05*w7*Jinv11-2.5568181818181820e-04*Jinv21*w56+3.6525974025974026e-05*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14+4.9310064935064932e-04*Jinv01*w12-2.7394480519480521e-05*Jinv21*w8-6.7573051948051951e-04*Jinv21*w23-2.5568181818181820e-04*Jinv01*w54-2.5568181818181820e-04*w55*Jinv11-1.1668019480519480e-04*Jinv01*w9-3.2873376623376621e-04*w40*Jinv11+1.1870941558441559e-04*Jinv01*w30-6.7573051948051951e-04*w25*Jinv11+3.6525974025974026e-05*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27+3.6525974025974026e-05*w48*Jinv01-1.1668019480519480e-04*w10*Jinv11)*D;
  A[54][28]=(0.0)*D;
  A[54][29]=(0.0)*D;
  A[54][30]=(1.6436688311688311e-04*Jinv00*w12+3.1047077922077924e-04*w28*Jinv10+8.7662337662337668e-04*w40*Jinv12+2.0271915584415584e-03*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+5.5702110389610384e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+8.7662337662337668e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00-5.7528409090909093e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-1.3697240259740259e-04*Jinv20*w23-1.1231737012987013e-03*Jinv22*w38+4.7666396103896100e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-1.4793019480519481e-03*Jinv01*w45+4.0543831168831168e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+3.1960227272727273e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-4.6570616883116886e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-1.4793019480519481e-03*w45*Jinv00+1.6436688311688311e-03*Jinv21*w41+2.5750811688311690e-03*w42*Jinv02+3.1047077922077924e-04*Jinv00*w27+3.1960227272727273e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50+3.1047077922077924e-04*Jinv20*w29-1.4793019480519481e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+8.2183441558441553e-04*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12-5.7528409090909093e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-7.3965097402597403e-04*w32*Jinv22+1.6436688311688311e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38-9.5880681818181818e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+5.5702110389610384e-04*Jinv22*w59+8.2183441558441553e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00+4.3831168831168834e-04*Jinv12*w34+2.5750811688311690e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20-5.4788961038961036e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+8.2183441558441553e-04*w34*Jinv10-5.7528409090909093e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-1.1231737012987013e-03*w37*Jinv12+4.7666396103896100e-03*Jinv20*w56+8.7662337662337668e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+2.0271915584415584e-03*w52*Jinv12+8.2183441558441553e-04*Jinv21*w35-5.7528409090909093e-04*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26-1.4793019480519481e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21+4.0543831168831168e-03*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+8.2183441558441553e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00-5.4788961038961036e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+3.1960227272727273e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10-1.4793019480519481e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11-7.3965097402597403e-04*w31*Jinv12-9.5880681818181818e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51-7.3965097402597403e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+4.7666396103896100e-03*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02-4.6570616883116886e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5-9.5880681818181818e-04*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+5.5702110389610384e-04*w58*Jinv12-4.6570616883116886e-04*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+4.3831168831168834e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+8.2183441558441553e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23+4.7666396103896100e-03*Jinv01*w54+4.7666396103896100e-03*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17-5.7528409090909093e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29-5.4788961038961036e-04*Jinv22*w14+1.6436688311688311e-03*w40*Jinv11-1.1231737012987013e-03*Jinv02*w36+2.5750811688311690e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-5.7528409090909093e-04*Jinv01*w30+2.0271915584415584e-03*Jinv02*w51+4.0543831168831168e-03*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+4.7666396103896100e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.4793019480519481e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[54][31]=(0.0)*D;
  A[54][32]=(0.0)*D;
  A[54][33]=(-7.1225649350649346e-04*Jinv00*w12-2.3011363636363637e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-8.2183441558441553e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23+1.7532467532467534e-03*Jinv22*w38+5.4788961038961036e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55-1.9724025974025973e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+7.1225649350649346e-04*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11-9.8620129870129864e-04*Jinv21*w41-3.2873376623376621e-03*w42*Jinv02-5.2962662337662338e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50-1.9724025974025973e-03*Jinv22*w17+3.2873376623376621e-04*w22*Jinv12-8.2183441558441553e-04*w31*Jinv10-3.8352272727272725e-04*w21*Jinv00+9.3141233766233771e-04*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-3.1047077922077924e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+8.2183441558441553e-04*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.7532467532467534e-03*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+7.1225649350649346e-04*w19*Jinv10+1.3149350649350649e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00+1.7532467532467534e-03*w37*Jinv12-2.3011363636363637e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+7.6704545454545450e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+8.2183441558441553e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+9.3141233766233771e-04*w31*Jinv12+1.7532467532467534e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-1.0409902597402598e-03*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51+9.3141233766233771e-04*Jinv02*w30-1.0409902597402598e-03*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+8.2183441558441553e-04*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.7532467532467534e-03*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8-2.3011363636363637e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+3.8352272727272725e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-1.0409902597402598e-03*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9-8.2183441558441553e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+1.7532467532467534e-03*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-9.8620129870129864e-04*w42*Jinv00-1.9724025974025973e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[54][34]=(0.0)*D;
  A[54][35]=(0.0)*D;
  A[54][36]=(5.4788961038961036e-04*Jinv00*w12-3.1047077922077924e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8+9.1314935064935065e-06*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+2.4655032467532466e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-4.7666396103896100e-03*Jinv00*w54+5.2049512987012989e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-1.6436688311688311e-03*w43*Jinv10-7.1225649350649346e-04*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+5.2049512987012989e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+1.4793019480519481e-03*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02-3.1047077922077924e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-3.1047077922077924e-04*Jinv20*w29+1.4793019480519481e-03*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17-8.2183441558441553e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-1.0957792207792208e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18+5.7528409090909093e-04*w36*Jinv00+5.2049512987012989e-04*Jinv02*w45+9.1314935064935065e-06*Jinv22*w59-2.4655032467532466e-04*w51*Jinv00-3.8352272727272725e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+2.4655032467532466e-04*w4*Jinv10-2.6298701298701297e-03*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-8.2183441558441553e-04*w34*Jinv10-8.2183441558441553e-05*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-4.7666396103896100e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12+5.2049512987012989e-04*Jinv22*w26+5.4788961038961036e-04*Jinv20*w14-7.1225649350649346e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-2.6298701298701297e-03*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-8.2183441558441553e-05*w7*Jinv10+5.7528409090909093e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12+5.2049512987012989e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50-3.5612824675324673e-04*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-2.4655032467532466e-04*w52*Jinv10-2.1002435064935065e-04*w9*Jinv00+5.7528409090909093e-04*w37*Jinv10+1.3697240259740259e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5+5.2049512987012989e-04*w47*Jinv22-2.1002435064935065e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+9.1314935064935065e-06*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-3.8352272727272725e-04*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-8.2183441558441553e-04*w35*Jinv20+1.3697240259740259e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14-1.6436688311688311e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+2.4655032467532466e-04*w5*Jinv20-1.6436688311688311e-04*Jinv02*w51-7.1225649350649346e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28-4.7666396103896100e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+1.4793019480519481e-03*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-1.0957792207792208e-04*Jinv12*w1)*D;
  A[54][37]=(0.0)*D;
  A[54][38]=(0.0)*D;
  A[54][39]=(1.6436688311688311e-04*Jinv00*w12-6.5746753246753243e-04*w28*Jinv10-6.9034090909090911e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10-6.9034090909090911e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+2.6298701298701297e-03*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23+2.5750811688311690e-03*Jinv22*w38-1.2820616883116883e-02*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-2.9586038961038961e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+2.7942370129870131e-03*Jinv01*w45-9.5332792207792200e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-9.6793831168831166e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+1.7532467532467534e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-5.4788961038961036e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+4.9310064935064934e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-6.5746753246753243e-04*Jinv00*w27-9.6793831168831166e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1-6.0267857142857139e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-1.6436688311688311e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-6.5746753246753243e-04*Jinv20*w29+4.9310064935064934e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+2.6298701298701297e-03*w31*Jinv10+2.1367694805194807e-03*w21*Jinv00+1.8628246753246754e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+2.6298701298701297e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38+2.7942370129870131e-03*Jinv02*w45+1.7532467532467534e-03*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+1.0957792207792208e-04*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20+1.6436688311688311e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+4.9310064935064932e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+2.5750811688311690e-03*w37*Jinv12-1.2820616883116883e-02*Jinv20*w56-6.9034090909090911e-03*Jinv22*w41+2.1367694805194807e-03*w26*Jinv20-5.4788961038961036e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12+4.9310064935064932e-04*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+2.7942370129870131e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21-9.5332792207792200e-03*Jinv02*w54-4.9310064935064932e-04*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-1.6436688311688311e-04*Jinv21*w29-2.9586038961038961e-03*w39*Jinv00+1.0957792207792208e-04*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-9.6793831168831166e-04*Jinv02*w9+5.4788961038961042e-05*w7*Jinv10+2.7942370129870131e-03*w47*Jinv21+2.6298701298701297e-03*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11+1.8628246753246754e-03*w31*Jinv12+2.7942370129870131e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53+8.2183441558441553e-04*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-2.0819805194805196e-03*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11+8.2183441558441553e-04*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.8628246753246754e-03*Jinv02*w30-2.0819805194805196e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10-5.4788961038961036e-04*w9*Jinv00+2.6298701298701297e-03*w37*Jinv10-8.5470779220779228e-03*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+8.2183441558441553e-04*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12+2.7942370129870131e-03*w47*Jinv22-6.0267857142857139e-04*Jinv21*w14-5.4788961038961036e-04*w10*Jinv10-2.9586038961038961e-03*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18-6.0267857142857139e-04*Jinv01*w12+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33-5.4788961038961036e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23-8.5470779220779228e-03*Jinv01*w54-8.5470779220779228e-03*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10-5.4788961038961036e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00-2.0819805194805196e-03*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17+2.6298701298701297e-03*w30*Jinv00-2.9586038961038961e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+2.5750811688311690e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+4.9310064935064932e-04*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-9.5332792207792200e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+1.7532467532467534e-03*w25*Jinv11-1.2820616883116883e-02*w55*Jinv10-2.9586038961038961e-03*w42*Jinv00+4.9310064935064934e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-1.6436688311688311e-04*Jinv01*w27-6.2094155844155847e-04*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[54][40]=(0.0)*D;
  A[54][41]=(0.0)*D;
  A[54][42]=(-1.6436688311688311e-04*Jinv00*w12+6.5746753246753243e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+2.1915584415584414e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-9.8620129870129864e-04*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-2.6298701298701297e-03*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23-7.6704545454545450e-04*Jinv22*w38+1.2820616883116883e-02*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-4.2004870129870131e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+3.2873376623376621e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-4.9310064935064934e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01+5.4788961038961036e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-4.9310064935064934e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+6.5746753246753243e-04*Jinv00*w27-7.3051948051948052e-05*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+4.2735389610389614e-03*Jinv20*w50+6.5746753246753243e-04*Jinv20*w29-4.9310064935064934e-03*w46*Jinv10-4.9310064935064934e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-2.6298701298701297e-03*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-4.2004870129870131e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-2.6298701298701297e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24-9.8620129870129864e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.2735389610389614e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-3.9448051948051946e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11+2.9586038961038961e-03*w41*Jinv20+1.1505681818181819e-03*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-7.6704545454545450e-04*w37*Jinv12+1.2820616883116883e-02*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20+5.4788961038961036e-04*w11*Jinv20+2.1915584415584414e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21+3.2873376623376621e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+2.9586038961038961e-03*w39*Jinv00+1.1505681818181819e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-2.1367694805194807e-03*w47*Jinv21-2.6298701298701297e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-5.4788961038961042e-05*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+4.2735389610389614e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11+5.4788961038961036e-04*w9*Jinv00-2.6298701298701297e-03*w37*Jinv10+4.2735389610389614e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+5.4788961038961036e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44-9.8620129870129864e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+5.4788961038961036e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10+5.4788961038961036e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17-2.6298701298701297e-03*w30*Jinv00+2.9586038961038961e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+1.1505681818181819e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30+2.1915584415584414e-03*Jinv02*w51+3.2873376623376621e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11+1.2820616883116883e-02*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-4.9310064935064934e-03*w47*Jinv20-4.9310064935064934e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-4.2004870129870131e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[54][43]=(0.0)*D;
  A[54][44]=(0.0)*D;
  A[54][45]=(-2.1367694805194807e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-4.7483766233766234e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+2.4655032467532466e-04*Jinv01*w3+5.2049512987012989e-04*Jinv22*w38-1.0957792207792207e-03*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-1.5523538961038962e-04*Jinv21*w2-6.5746753246753243e-04*w42*Jinv01-1.5523538961038962e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55-1.3149350649350649e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01-1.2327516233766234e-03*Jinv21*w26-1.5523538961038962e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11-6.5746753246753243e-04*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-1.5523538961038962e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01-1.4610389610389610e-04*w28*Jinv11-1.3149350649350649e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+8.2183441558441553e-05*Jinv21*w38-2.6298701298701297e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-4.7483766233766234e-04*Jinv22*w59-8.2183441558441553e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.4655032467532466e-04*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+8.2183441558441553e-05*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+5.2049512987012989e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-8.2183441558441553e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11+2.4655032467532466e-04*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02-1.4610389610389610e-04*Jinv21*w29-8.2183441558441553e-04*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+4.3831168831168834e-04*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01-1.3149350649350649e-03*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+5.2049512987012989e-04*w31*Jinv12-2.6298701298701297e-03*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+5.2049512987012989e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+2.4655032467532466e-04*w7*Jinv11+3.7804383116883116e-03*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-2.6298701298701297e-03*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-4.7483766233766234e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.2327516233766234e-03*Jinv21*w23+3.7804383116883116e-03*Jinv01*w54+3.7804383116883116e-03*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.5523538961038962e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+5.2049512987012989e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.2183441558441553e-05*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11-1.3149350649350649e-03*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-1.4610389610389610e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-1.5523538961038962e-04*w10*Jinv11)*D;
  A[54][46]=(0.0)*D;
  A[54][47]=(0.0)*D;
  A[54][48]=(7.1225649350649346e-04*Jinv00*w12-7.8530844155844158e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12-2.4655032467532467e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8-1.1414366883116883e-03*w57*Jinv02+7.1225649350649346e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+2.3833198051948050e-03*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38-7.3965097402597406e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59+8.2183441558441553e-05*w25*Jinv12-3.5612824675324673e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.6298701298701297e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-3.7439123376623376e-04*w10*Jinv12-3.6982548701298703e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-5.4788961038961036e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-4.2735389610389614e-03*Jinv21*w41-2.6846590909090908e-03*w42*Jinv02-7.8530844155844158e-04*Jinv00*w27-3.7439123376623376e-04*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.8530844155844158e-04*w28*Jinv11-3.6982548701298703e-03*Jinv20*w50-7.8530844155844158e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-2.1367694805194807e-03*Jinv00*w33+2.3833198051948050e-03*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+1.1505681818181819e-03*w32*Jinv22-4.2735389610389614e-03*Jinv01*w39-3.5612824675324673e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.3833198051948050e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+1.3971185064935066e-03*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24-1.1414366883116883e-03*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-3.6982548701298703e-03*Jinv21*w53-3.6982548701298703e-03*w51*Jinv00-5.4788961038961036e-04*Jinv12*w34-2.6846590909090908e-03*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10+2.3833198051948050e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12-7.3965097402597406e-03*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-2.4655032467532467e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+2.3833198051948050e-03*w31*Jinv11+8.2183441558441553e-05*Jinv22*w26+2.6298701298701297e-03*w46*Jinv11+7.1225649350649346e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.6982548701298703e-03*w49*Jinv10-7.8530844155844158e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11-3.7439123376623376e-04*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+2.6298701298701297e-03*w47*Jinv21+2.3833198051948050e-03*Jinv20*w38-3.6982548701298703e-03*w52*Jinv11+1.1505681818181819e-03*w31*Jinv12+1.3971185064935066e-03*w46*Jinv12-3.6982548701298703e-03*Jinv20*w53+1.6436688311688311e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+1.6436688311688311e-04*w21*Jinv01-3.6982548701298703e-03*Jinv01*w51+1.1505681818181819e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-3.6982548701298703e-03*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+2.3833198051948050e-03*w37*Jinv10-7.3965097402597406e-03*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+3.5612824675324673e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5+1.3971185064935066e-03*w47*Jinv22-3.6982548701298703e-03*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-1.1414366883116883e-03*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-5.4788961038961036e-04*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23-7.3965097402597406e-03*Jinv01*w54-7.3965097402597406e-03*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-7.4878246753246752e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.3833198051948050e-03*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-4.2735389610389614e-03*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.6846590909090908e-03*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+2.3833198051948050e-03*Jinv01*w30-2.4655032467532467e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-7.3965097402597406e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-3.6982548701298703e-03*Jinv21*w50-7.8530844155844158e-04*Jinv01*w27-3.6982548701298703e-03*w48*Jinv01-3.5612824675324673e-04*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[54][49]=(0.0)*D;
  A[54][50]=(0.0)*D;
  A[54][51]=(-7.1225649350649346e-04*Jinv00*w12+7.8530844155844158e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-4.5657467532467534e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-3.5612824675324673e-04*w3*Jinv00-2.3833198051948050e-03*w32*Jinv20-1.6436688311688311e-04*Jinv20*w23-1.2327516233766234e-03*Jinv22*w38+7.3965097402597406e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12+3.7439123376623376e-04*w0*Jinv02+4.2735389610389614e-03*w43*Jinv10+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+3.6982548701298703e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-2.6298701298701297e-03*w45*Jinv00+3.1777597402597403e-03*w42*Jinv02+7.8530844155844158e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+3.6982548701298703e-03*Jinv20*w50+7.8530844155844158e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+1.0957792207792207e-03*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12-2.3833198051948050e-03*w31*Jinv10-3.5612824675324673e-04*Jinv02*w3-1.6436688311688311e-04*w21*Jinv00-1.0135957792207792e-03*w32*Jinv22+3.7439123376623376e-04*Jinv22*w2-7.1225649350649346e-04*Jinv00*w18-2.3833198051948050e-03*w36*Jinv00-1.2327516233766234e-03*Jinv02*w45-4.5657467532467534e-05*Jinv22*w59+3.6982548701298703e-03*w51*Jinv00+1.5888798701298701e-03*Jinv12*w34+3.1777597402597403e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+4.2735389610389614e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-7.1225649350649346e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-1.2327516233766234e-03*w37*Jinv12+7.3965097402597406e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+2.4655032467532467e-03*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+2.4655032467532467e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02+3.6982548701298703e-03*w49*Jinv10+4.2735389610389614e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+3.9265422077922079e-04*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-2.3833198051948050e-03*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23-1.0135957792207792e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12+3.6982548701298703e-03*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-1.6436688311688311e-04*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50-8.2183441558441553e-05*Jinv12*w7-1.0135957792207792e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49+3.6982548701298703e-03*w52*Jinv10+7.4878246753246752e-04*w9*Jinv00-2.3833198051948050e-03*w37*Jinv10-1.6436688311688311e-04*w24*Jinv00+3.9265422077922079e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+7.4878246753246752e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-4.5657467532467534e-05*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+1.5888798701298701e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-1.6436688311688311e-04*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.3833198051948050e-03*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-7.1225649350649346e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-1.2327516233766234e-03*Jinv02*w36+3.1777597402597403e-03*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+2.4655032467532467e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28+7.3965097402597406e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+1.0957792207792207e-03*w16*Jinv12+3.7439123376623376e-04*Jinv12*w1)*D;
  A[54][52]=(0.0)*D;
  A[54][53]=(0.0)*D;
  A[54][54]=(4.7666396103896100e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-9.1314935064935068e-05*w57*Jinv02+4.7666396103896100e-03*Jinv02*w39+4.2735389610389614e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-2.0271915584415584e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59+5.7528409090909093e-04*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.6298701298701297e-03*Jinv01*w45+7.3965097402597406e-03*Jinv12*w55+1.6436688311688311e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+5.7528409090909093e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-1.0683847402597404e-03*Jinv22*w20+4.2735389610389614e-03*w43*Jinv11+4.2735389610389614e-03*Jinv21*w41+4.7666396103896100e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.8530844155844158e-04*w28*Jinv11+1.6436688311688311e-03*Jinv22*w17+5.7528409090909093e-04*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-2.0271915584415584e-03*w32*Jinv22+4.2735389610389614e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-2.3833198051948050e-03*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-9.1314935064935068e-05*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11+3.6982548701298703e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+4.7666396103896100e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-1.0683847402597404e-03*Jinv12*w13-2.3833198051948050e-03*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-2.0271915584415584e-03*w37*Jinv12+4.7666396103896100e-03*Jinv22*w41+2.4655032467532467e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-2.3833198051948050e-03*w31*Jinv11+5.7528409090909093e-04*Jinv22*w26-2.6298701298701297e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21+7.3965097402597406e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12+5.7528409090909093e-04*w21*Jinv02+7.8530844155844158e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-1.0683847402597404e-03*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01-2.6298701298701297e-03*w47*Jinv21+5.7528409090909093e-04*Jinv22*w23+3.6982548701298703e-03*w52*Jinv11-2.0271915584415584e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.4655032467532467e-03*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01+3.6982548701298703e-03*Jinv01*w51-2.0271915584415584e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49-2.3833198051948050e-03*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11+7.3965097402597406e-03*Jinv21*w56+6.5746753246753243e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.0683847402597404e-03*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22+3.6982548701298703e-03*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-9.1314935064935068e-05*w58*Jinv12-1.0683847402597404e-03*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23+7.3965097402597406e-03*Jinv01*w54+7.3965097402597406e-03*w55*Jinv11+2.4655032467532467e-03*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+6.5746753246753243e-04*Jinv22*w29-1.0683847402597404e-03*Jinv22*w14+4.2735389610389614e-03*w40*Jinv11-2.0271915584415584e-03*Jinv02*w36+4.7666396103896100e-03*Jinv12*w43-2.3833198051948050e-03*Jinv01*w30+2.4655032467532467e-03*Jinv02*w51+7.3965097402597406e-03*Jinv22*w56+6.5746753246753243e-04*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11+1.6436688311688311e-03*w16*Jinv12+3.6982548701298703e-03*Jinv21*w50+7.8530844155844158e-04*Jinv01*w27+3.6982548701298703e-03*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[54][55]=(0.0)*D;
  A[54][56]=(0.0)*D;
  A[54][57]=(6.5746753246753243e-04*w40*Jinv12+1.1870941558441558e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-6.7573051948051951e-04*Jinv22*w38-2.6481331168831169e-04*w25*Jinv12+1.5726461038961039e-04*w0*Jinv02+2.3741883116883116e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12-2.6481331168831169e-04*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35-1.0957792207792208e-04*Jinv22*w20+6.5746753246753243e-04*w42*Jinv02+1.5726461038961039e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-2.6481331168831169e-04*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-6.7573051948051951e-04*w32*Jinv22+1.5726461038961039e-04*Jinv22*w2-1.0227272727272728e-03*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+3.2873376623376621e-04*Jinv12*w34+6.5746753246753243e-04*Jinv22*w44-1.0957792207792208e-04*Jinv12*w13-6.7573051948051951e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41+1.1870941558441558e-03*w52*Jinv12-2.6481331168831169e-04*Jinv22*w26+2.3741883116883116e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12-2.6481331168831169e-04*w21*Jinv02-1.0957792207792208e-04*Jinv02*w12+1.5726461038961039e-04*Jinv02*w9-2.6481331168831169e-04*Jinv22*w23-6.7573051948051951e-04*w31*Jinv12-1.0227272727272728e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8+1.1870941558441558e-03*Jinv22*w50-5.4788961038961042e-05*Jinv12*w7-6.7573051948051951e-04*Jinv02*w30+1.1870941558441558e-03*Jinv12*w49+2.0495129870129870e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.0957792207792208e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-1.0227272727272728e-03*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-1.0957792207792208e-04*Jinv02*w18+3.2873376623376621e-04*Jinv02*w33+1.1870941558441558e-03*w48*Jinv02+2.0495129870129870e-04*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-6.7573051948051951e-04*Jinv02*w36+6.5746753246753243e-04*Jinv12*w43+1.1870941558441558e-03*Jinv02*w51+2.3741883116883116e-03*Jinv22*w56+2.0495129870129870e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+1.5726461038961039e-04*Jinv12*w1)*D;
  A[54][58]=(0.0)*D;
  A[54][59]=(0.0)*D;
  A[55][0]=(0.0)*D;
  A[55][1]=(1.0957792207792208e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12-7.4878246753246752e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+2.6481331168831169e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38-1.4975649350649350e-03*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-3.4699675324675324e-04*w16*Jinv11-3.4699675324675324e-04*Jinv21*w17+5.8441558441558442e-04*Jinv01*w45-1.4975649350649350e-03*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.4699675324675324e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-3.4699675324675324e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+5.8441558441558442e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-3.4699675324675324e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+5.8441558441558442e-04*w46*Jinv10-3.4699675324675324e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+2.6481331168831169e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+2.6481331168831169e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38+5.8441558441558442e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-2.1915584415584417e-04*w34*Jinv11+2.1915584415584417e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+2.6481331168831169e-04*w32*Jinv21+2.1915584415584417e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12-1.4975649350649350e-03*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-7.4878246753246752e-04*w52*Jinv12-2.1915584415584417e-04*Jinv21*w35+2.6481331168831169e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+5.8441558441558442e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-1.4975649350649350e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-2.1915584415584417e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12+2.1915584415584417e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10+5.8441558441558442e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11+2.6481331168831169e-04*w31*Jinv12+5.8441558441558442e-04*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51+2.6481331168831169e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-1.4975649350649350e-03*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5+5.8441558441558442e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23-1.4975649350649350e-03*Jinv01*w54-1.4975649350649350e-03*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-3.4699675324675324e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-3.4699675324675324e-04*Jinv20*w17+2.6481331168831169e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+2.6481331168831169e-04*Jinv01*w30-7.4878246753246752e-04*Jinv02*w51-1.4975649350649350e-03*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11-1.4975649350649350e-03*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+5.8441558441558442e-04*w47*Jinv20-3.4699675324675324e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[55][2]=(0.0)*D;
  A[55][3]=(0.0)*D;
  A[55][4]=(-3.5612824675324673e-04*Jinv00*w12+7.1225649350649346e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44-8.2183441558441553e-05*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-9.1314935064935065e-06*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+8.7662337662337668e-04*w16*Jinv11+8.7662337662337668e-04*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+8.7662337662337668e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+8.2183441558441553e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*w11*Jinv22-6.3920454545454550e-05*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+8.7662337662337668e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+8.7662337662337668e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-4.6570616883116886e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+7.1225649350649346e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-4.6570616883116886e-04*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+7.1225649350649346e-04*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+2.7394480519480521e-05*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+7.1225649350649346e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-3.5612824675324673e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+7.1225649350649346e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.9176136363636362e-04*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-1.9176136363636362e-04*Jinv12*w49-5.2049512987012989e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+2.7394480519480521e-05*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-1.9176136363636362e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+2.7394480519480521e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+7.1225649350649346e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+8.2183441558441553e-04*w15*Jinv00-1.9176136363636362e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+8.2183441558441553e-04*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-4.6570616883116886e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+8.2183441558441553e-05*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+8.7662337662337668e-04*w16*Jinv12-1.9176136363636362e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-1.9176136363636362e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[55][5]=(0.0)*D;
  A[55][6]=(0.0)*D;
  A[55][7]=(4.6570616883116886e-04*Jinv00*w12+5.4788961038961042e-05*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-9.1314935064935065e-06*w57*Jinv02+4.6570616883116886e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480521e-05*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-9.1314935064935065e-06*Jinv21*w59-5.2049512987012989e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+5.4788961038961042e-05*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01-2.7394480519480521e-05*w1*Jinv10-8.2183441558441553e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+5.4788961038961042e-05*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*w11*Jinv22+1.0957792207792208e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+5.4788961038961042e-05*Jinv22*w17-3.2873376623376621e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12+2.4655032467532466e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480521e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+1.6436688311688311e-04*w37*Jinv12+5.4788961038961042e-05*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11+4.6570616883116886e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01-5.2049512987012989e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8+3.8352272727272725e-04*Jinv02*w33-2.7394480519480521e-05*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-2.7394480519480521e-05*Jinv20*w2-8.2183441558441553e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-8.2183441558441553e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480521e-05*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[55][8]=(0.0)*D;
  A[55][9]=(0.0)*D;
  A[55][10]=(-2.1915584415584417e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-2.1915584415584417e-04*w13*Jinv10-2.7394480519480521e-05*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20-4.2918019480519480e-04*Jinv20*w23+1.4975649350649350e-03*Jinv00*w54+4.3831168831168834e-04*w43*Jinv10+6.4935064935064935e-05*w58*Jinv10+7.4878246753246752e-04*w48*Jinv00+1.0247564935064935e-04*w1*Jinv10+3.4699675324675324e-04*w16*Jinv10-5.8441558441558442e-04*w45*Jinv00-1.4204545454545455e-05*Jinv00*w27+7.4878246753246752e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-5.8441558441558442e-04*w46*Jinv10+2.1915584415584417e-04*Jinv00*w33-4.2918019480519480e-04*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-1.0957792207792208e-04*Jinv00*w18-2.6481331168831169e-04*w36*Jinv00+3.6525974025974026e-05*w51*Jinv00-2.7394480519480521e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-1.0957792207792208e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00+1.4975649350649350e-03*Jinv20*w56+1.1870941558441559e-04*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-2.1915584415584417e-04*Jinv20*w14+7.4878246753246752e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-5.4788961038961042e-05*w7*Jinv10-2.6481331168831169e-04*Jinv20*w38+3.6525974025974026e-05*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10+3.6525974025974026e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-2.6481331168831169e-04*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+3.4699675324675324e-04*w15*Jinv00+3.4699675324675324e-04*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20-2.7394480519480521e-05*w5*Jinv20+1.4975649350649350e-03*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-5.8441558441558442e-04*w47*Jinv20)*D;
  A[55][11]=(0.0)*D;
  A[55][12]=(0.0)*D;
  A[55][13]=(1.0683847402597404e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-4.6570616883116886e-04*Jinv20*w8-1.8262987012987013e-05*w57*Jinv02+1.0683847402597404e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+1.2601461038961039e-03*w16*Jinv11+1.2601461038961039e-03*Jinv21*w17-9.5880681818181818e-04*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-1.8262987012987013e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-1.1231737012987013e-03*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11-1.4793019480519481e-03*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.2601461038961039e-03*w15*Jinv01-3.5612824675324673e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-1.4793019480519481e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38-1.4793019480519481e-03*Jinv02*w45-1.1231737012987013e-03*Jinv01*w24-1.8262987012987013e-05*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+2.1367694805194807e-03*w41*Jinv20+1.0683847402597404e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-1.6436688311688311e-04*Jinv21*w20-4.6570616883116886e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41-1.3971185064935066e-03*w26*Jinv20+2.1915584415584417e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35-1.3971185064935066e-03*Jinv22*w26-9.5880681818181818e-04*w46*Jinv11+1.0683847402597404e-03*Jinv20*w14+2.1367694805194807e-03*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-3.5612824675324673e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00+1.0683847402597404e-03*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-1.9176136363636362e-04*w6*Jinv01-4.6570616883116886e-04*w7*Jinv10-9.5880681818181818e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-1.4793019480519481e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-4.6570616883116886e-04*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11-4.6570616883116886e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-1.9176136363636362e-04*w7*Jinv11+2.1915584415584417e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+1.4245129870129869e-03*Jinv21*w56-1.3971185064935066e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27-4.6570616883116886e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-1.4793019480519481e-03*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+2.1915584415584417e-04*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-1.8262987012987013e-05*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-1.9176136363636362e-04*Jinv21*w8-1.8262987012987013e-05*w57*Jinv00-1.8262987012987013e-05*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23+1.4245129870129869e-03*Jinv01*w54+1.4245129870129869e-03*w55*Jinv11-1.3971185064935066e-03*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.0683847402597404e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+7.1225649350649346e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-1.1231737012987013e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-1.4793019480519481e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50-3.5612824675324673e-04*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[55][14]=(0.0)*D;
  A[55][15]=(0.0)*D;
  A[55][16]=(-1.3149350649350649e-03*Jinv00*w12+1.9724025974025973e-03*w40*Jinv12-2.1915584415584414e-03*Jinv22*w53+8.2183441558441553e-04*Jinv20*w8+9.8620129870129864e-04*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+1.9724025974025973e-03*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-8.2183441558441553e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-8.2183441558441553e-04*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38+4.9310064935064932e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+4.2004870129870131e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.6436688311688311e-03*w16*Jinv11+1.6436688311688311e-03*Jinv21*w17+1.7532467532467534e-03*Jinv01*w45-3.2873376623376621e-03*Jinv12*w55+4.9310064935064934e-03*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01+2.5750811688311690e-03*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+3.4699675324675324e-04*w1*Jinv10+1.6436688311688311e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+3.9448051948051946e-03*w42*Jinv02+7.3051948051948052e-05*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.2053571428571428e-03*w13*Jinv11+1.6436688311688311e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+1.0957792207792207e-03*Jinv20*w50+4.9310064935064934e-03*Jinv22*w17+1.3149350649350649e-03*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-7.6704545454545450e-04*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39+4.2004870129870131e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.8352272727272725e-04*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+2.5750811688311690e-03*Jinv01*w24+9.8620129870129864e-04*Jinv22*w59+1.0957792207792208e-04*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+3.9448051948051946e-03*Jinv22*w44-8.2183441558441553e-04*w4*Jinv10-8.2183441558441553e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-3.2873376623376621e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+8.2183441558441553e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12+1.9724025974025973e-03*Jinv22*w41+1.3149350649350649e-03*w26*Jinv20-3.4699675324675324e-04*w11*Jinv20-2.1915584415584414e-03*w52*Jinv12-3.2873376623376621e-04*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.7532467532467534e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-8.2183441558441553e-04*w5*Jinv21-3.2873376623376621e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-1.9724025974025973e-03*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9+8.2183441558441553e-04*w7*Jinv10+1.7532467532467534e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-1.0957792207792207e-03*Jinv22*w50+1.2601461038961039e-03*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+1.2601461038961039e-03*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10-3.4699675324675324e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.1915584415584414e-03*Jinv21*w56+1.3149350649350649e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-1.2053571428571428e-03*Jinv21*w14-3.4699675324675324e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+9.8620129870129864e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-1.2053571428571428e-03*Jinv01*w12+1.3149350649350649e-03*Jinv02*w33+3.4699675324675324e-04*w0*Jinv00+1.2601461038961039e-03*Jinv21*w23-2.1915584415584414e-03*Jinv01*w54-2.1915584415584414e-03*w55*Jinv11+1.3149350649350649e-03*w25*Jinv10+3.4699675324675324e-04*Jinv20*w2-1.0957792207792207e-03*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-7.1225649350649346e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36+3.9448051948051946e-03*Jinv12*w43-8.2183441558441553e-04*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30-2.1915584415584414e-03*Jinv02*w51-3.2873376623376621e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+2.5750811688311690e-03*w25*Jinv11+1.9724025974025973e-03*w42*Jinv00+4.9310064935064934e-03*w16*Jinv12-2.5568181818181820e-04*Jinv01*w27+4.2004870129870131e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[55][17]=(0.0)*D;
  A[55][18]=(0.0)*D;
  A[55][19]=(2.4655032467532466e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+2.4655032467532466e-04*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+4.6570616883116886e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3+1.3971185064935066e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.0957792207792208e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45+1.8262987012987013e-05*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-2.1915584415584417e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11+1.4793019480519481e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27-1.0957792207792208e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+1.4793019480519481e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10+1.3971185064935066e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-1.0683847402597404e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+5.2049512987012989e-04*Jinv01*w24-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+4.6570616883116886e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20-1.0683847402597404e-03*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+8.2183441558441553e-05*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20-3.8352272727272725e-04*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11+5.2049512987012989e-04*w46*Jinv11+2.4655032467532466e-04*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.5612824675324673e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+5.2049512987012989e-04*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11+3.8352272727272725e-04*Jinv20*w53+1.3971185064935066e-03*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+8.2183441558441553e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.8262987012987013e-05*w57*Jinv00+1.8262987012987013e-05*Jinv20*w59-2.1915584415584417e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10-2.1915584415584417e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.0683847402597404e-03*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+4.6570616883116886e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+5.2049512987012989e-04*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.4793019480519481e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[55][20]=(0.0)*D;
  A[55][21]=(0.0)*D;
  A[55][22]=(-1.3971185064935066e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+4.7483766233766234e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-2.7394480519480521e-05*Jinv01*w3+2.3833198051948050e-03*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-1.1505681818181819e-03*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21-1.2053571428571428e-03*w16*Jinv11-1.2053571428571428e-03*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-1.1505681818181819e-03*Jinv12*w55+4.7483766233766234e-04*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+1.3697240259740260e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-6.0267857142857139e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-9.5880681818181818e-04*w13*Jinv11-1.2053571428571428e-03*w15*Jinv01+6.6659902597402602e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.3833198051948050e-03*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+2.3833198051948050e-03*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22-6.0267857142857139e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+1.3697240259740260e-03*Jinv01*w24+4.7483766233766234e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480521e-05*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-3.8352272727272725e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12-1.1505681818181819e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14-2.7394480519480521e-05*w5*Jinv21-1.1505681818181819e-03*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+2.3833198051948050e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+6.6659902597402602e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9+1.9176136363636362e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.3971185064935066e-03*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+2.3833198051948050e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+2.3833198051948050e-03*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+1.9176136363636362e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.2873376623376621e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11-9.5880681818181818e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+4.7483766233766234e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18-9.5880681818181818e-04*Jinv01*w12+1.9176136363636362e-04*Jinv21*w8+4.7483766233766234e-04*w57*Jinv00+4.7483766233766234e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+3.2873376623376621e-04*Jinv01*w54+3.2873376623376621e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14-6.0267857142857139e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-3.8352272727272725e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51-1.1505681818181819e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.3697240259740260e-03*w25*Jinv11-1.1505681818181819e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50+6.6659902597402602e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[55][23]=(0.0)*D;
  A[55][24]=(0.0)*D;
  A[55][25]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-2.7394480519480520e-03*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+4.6570616883116886e-04*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.1505681818181819e-03*Jinv00*w54-4.8396915584415583e-04*Jinv21*w59-3.1960227272727273e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-2.7394480519480520e-03*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-2.5202922077922079e-03*w16*Jinv11-2.5202922077922079e-03*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45-4.7483766233766234e-04*w58*Jinv10-1.6436688311688311e-04*w48*Jinv00-4.8396915584415583e-04*w58*Jinv11-4.8396915584415583e-04*w57*Jinv01-2.0271915584415584e-03*Jinv21*w26-3.1960227272727273e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.7394480519480520e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-3.1960227272727273e-04*Jinv11*w1+8.4922889610389610e-04*w13*Jinv11-2.5202922077922079e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-1.6436688311688311e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.3971185064935066e-03*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+1.3697240259740259e-04*Jinv21*w38-2.0271915584415584e-03*Jinv01*w24-7.1225649350649346e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+4.6570616883116886e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+1.3971185064935066e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00+1.1505681818181819e-03*Jinv20*w56-2.3833198051948050e-03*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-7.1225649350649346e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.2327516233766234e-03*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+4.6570616883116886e-04*w5*Jinv21-1.6436688311688311e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+4.3831168831168834e-04*Jinv01*w18+1.3697240259740259e-04*w37*Jinv11+2.4655032467532466e-04*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21+5.2049512987012989e-04*Jinv20*w38+3.8352272727272725e-04*w52*Jinv11-1.6436688311688311e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-1.0135957792207792e-03*w22*Jinv11-1.0135957792207792e-03*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51+1.3697240259740259e-04*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11-4.2918019480519480e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+1.4793019480519481e-03*Jinv21*w56-2.3833198051948050e-03*w24*Jinv00+1.9176136363636362e-04*w49*Jinv11+8.4922889610389610e-04*Jinv21*w14-4.2918019480519480e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+8.4922889610389610e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-4.7483766233766234e-04*w57*Jinv00-4.7483766233766234e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.0135957792207792e-03*Jinv21*w23+1.4793019480519481e-03*Jinv01*w54+1.4793019480519481e-03*w55*Jinv11-2.3833198051948050e-03*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-1.7349837662337662e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30-2.0271915584415584e-03*w25*Jinv11+1.1505681818181819e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+1.9176136363636362e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.9176136363636362e-04*w48*Jinv01-1.7349837662337662e-04*w10*Jinv11)*D;
  A[55][26]=(0.0)*D;
  A[55][27]=(0.0)*D;
  A[55][28]=(-3.2873376623376621e-04*Jinv21*w44-2.7394480519480521e-05*Jinv01*w3+8.3198051948051951e-05*Jinv21*w59-1.1668019480519480e-04*Jinv21*w2-3.2873376623376621e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21+2.9220779220779221e-04*w16*Jinv11+2.9220779220779221e-04*Jinv21*w17-1.0227272727272728e-03*Jinv01*w45+8.3198051948051951e-05*w58*Jinv11+8.3198051948051951e-05*w57*Jinv01-6.7573051948051951e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-3.2873376623376621e-04*w43*Jinv11-3.2873376623376621e-04*Jinv21*w41-1.1668019480519480e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11+2.9220779220779221e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11-3.2873376623376621e-04*Jinv01*w39+1.1870941558441559e-04*Jinv21*w38-6.7573051948051951e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11+3.6525974025974026e-05*Jinv21*w53-2.7394480519480521e-05*w4*Jinv11+1.1870941558441559e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35+1.1870941558441559e-04*w31*Jinv11-1.0227272727272728e-03*w46*Jinv11-2.7394480519480521e-05*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+4.9310064935064932e-04*Jinv01*w18+1.1870941558441559e-04*w37*Jinv11-2.7394480519480521e-05*w6*Jinv01-1.0227272727272728e-03*w47*Jinv21+3.6525974025974026e-05*w52*Jinv11-6.7573051948051951e-04*w22*Jinv11-6.7573051948051951e-04*w21*Jinv01+3.6525974025974026e-05*Jinv01*w51+1.1870941558441559e-04*w36*Jinv01-2.7394480519480521e-05*w7*Jinv11-2.5568181818181820e-04*Jinv21*w56+3.6525974025974026e-05*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14+4.9310064935064932e-04*Jinv01*w12-2.7394480519480521e-05*Jinv21*w8-6.7573051948051951e-04*Jinv21*w23-2.5568181818181820e-04*Jinv01*w54-2.5568181818181820e-04*w55*Jinv11-1.1668019480519480e-04*Jinv01*w9-3.2873376623376621e-04*w40*Jinv11+1.1870941558441559e-04*Jinv01*w30-6.7573051948051951e-04*w25*Jinv11+3.6525974025974026e-05*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27+3.6525974025974026e-05*w48*Jinv01-1.1668019480519480e-04*w10*Jinv11)*D;
  A[55][29]=(0.0)*D;
  A[55][30]=(0.0)*D;
  A[55][31]=(1.6436688311688311e-04*Jinv00*w12+3.1047077922077924e-04*w28*Jinv10+8.7662337662337668e-04*w40*Jinv12+2.0271915584415584e-03*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+5.5702110389610384e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+8.7662337662337668e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00-5.7528409090909093e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-1.3697240259740259e-04*Jinv20*w23-1.1231737012987013e-03*Jinv22*w38+4.7666396103896100e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-1.4793019480519481e-03*Jinv01*w45+4.0543831168831168e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+3.1960227272727273e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-4.6570616883116886e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-1.4793019480519481e-03*w45*Jinv00+1.6436688311688311e-03*Jinv21*w41+2.5750811688311690e-03*w42*Jinv02+3.1047077922077924e-04*Jinv00*w27+3.1960227272727273e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50+3.1047077922077924e-04*Jinv20*w29-1.4793019480519481e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+8.2183441558441553e-04*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12-5.7528409090909093e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-7.3965097402597403e-04*w32*Jinv22+1.6436688311688311e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38-9.5880681818181818e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+5.5702110389610384e-04*Jinv22*w59+8.2183441558441553e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00+4.3831168831168834e-04*Jinv12*w34+2.5750811688311690e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20-5.4788961038961036e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+8.2183441558441553e-04*w34*Jinv10-5.7528409090909093e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-1.1231737012987013e-03*w37*Jinv12+4.7666396103896100e-03*Jinv20*w56+8.7662337662337668e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+2.0271915584415584e-03*w52*Jinv12+8.2183441558441553e-04*Jinv21*w35-5.7528409090909093e-04*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26-1.4793019480519481e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21+4.0543831168831168e-03*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+8.2183441558441553e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00-5.4788961038961036e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+3.1960227272727273e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10-1.4793019480519481e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11-7.3965097402597403e-04*w31*Jinv12-9.5880681818181818e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51-7.3965097402597403e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+4.7666396103896100e-03*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02-4.6570616883116886e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5-9.5880681818181818e-04*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+5.5702110389610384e-04*w58*Jinv12-4.6570616883116886e-04*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+4.3831168831168834e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+8.2183441558441553e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23+4.7666396103896100e-03*Jinv01*w54+4.7666396103896100e-03*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17-5.7528409090909093e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29-5.4788961038961036e-04*Jinv22*w14+1.6436688311688311e-03*w40*Jinv11-1.1231737012987013e-03*Jinv02*w36+2.5750811688311690e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-5.7528409090909093e-04*Jinv01*w30+2.0271915584415584e-03*Jinv02*w51+4.0543831168831168e-03*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+4.7666396103896100e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.4793019480519481e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[55][32]=(0.0)*D;
  A[55][33]=(0.0)*D;
  A[55][34]=(-7.1225649350649346e-04*Jinv00*w12-2.3011363636363637e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-8.2183441558441553e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23+1.7532467532467534e-03*Jinv22*w38+5.4788961038961036e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55-1.9724025974025973e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+7.1225649350649346e-04*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11-9.8620129870129864e-04*Jinv21*w41-3.2873376623376621e-03*w42*Jinv02-5.2962662337662338e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50-1.9724025974025973e-03*Jinv22*w17+3.2873376623376621e-04*w22*Jinv12-8.2183441558441553e-04*w31*Jinv10-3.8352272727272725e-04*w21*Jinv00+9.3141233766233771e-04*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-3.1047077922077924e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+8.2183441558441553e-04*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.7532467532467534e-03*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+7.1225649350649346e-04*w19*Jinv10+1.3149350649350649e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00+1.7532467532467534e-03*w37*Jinv12-2.3011363636363637e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+7.6704545454545450e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+8.2183441558441553e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+9.3141233766233771e-04*w31*Jinv12+1.7532467532467534e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-1.0409902597402598e-03*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51+9.3141233766233771e-04*Jinv02*w30-1.0409902597402598e-03*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+8.2183441558441553e-04*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.7532467532467534e-03*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8-2.3011363636363637e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+3.8352272727272725e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-1.0409902597402598e-03*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9-8.2183441558441553e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+1.7532467532467534e-03*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-9.8620129870129864e-04*w42*Jinv00-1.9724025974025973e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[55][35]=(0.0)*D;
  A[55][36]=(0.0)*D;
  A[55][37]=(5.4788961038961036e-04*Jinv00*w12-3.1047077922077924e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8+9.1314935064935065e-06*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+2.4655032467532466e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-4.7666396103896100e-03*Jinv00*w54+5.2049512987012989e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-1.6436688311688311e-03*w43*Jinv10-7.1225649350649346e-04*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+5.2049512987012989e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+1.4793019480519481e-03*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02-3.1047077922077924e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-3.1047077922077924e-04*Jinv20*w29+1.4793019480519481e-03*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17-8.2183441558441553e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-1.0957792207792208e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18+5.7528409090909093e-04*w36*Jinv00+5.2049512987012989e-04*Jinv02*w45+9.1314935064935065e-06*Jinv22*w59-2.4655032467532466e-04*w51*Jinv00-3.8352272727272725e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+2.4655032467532466e-04*w4*Jinv10-2.6298701298701297e-03*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-8.2183441558441553e-04*w34*Jinv10-8.2183441558441553e-05*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-4.7666396103896100e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12+5.2049512987012989e-04*Jinv22*w26+5.4788961038961036e-04*Jinv20*w14-7.1225649350649346e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-2.6298701298701297e-03*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-8.2183441558441553e-05*w7*Jinv10+5.7528409090909093e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12+5.2049512987012989e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50-3.5612824675324673e-04*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-2.4655032467532466e-04*w52*Jinv10-2.1002435064935065e-04*w9*Jinv00+5.7528409090909093e-04*w37*Jinv10+1.3697240259740259e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5+5.2049512987012989e-04*w47*Jinv22-2.1002435064935065e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+9.1314935064935065e-06*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-3.8352272727272725e-04*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-8.2183441558441553e-04*w35*Jinv20+1.3697240259740259e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14-1.6436688311688311e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+2.4655032467532466e-04*w5*Jinv20-1.6436688311688311e-04*Jinv02*w51-7.1225649350649346e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28-4.7666396103896100e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+1.4793019480519481e-03*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-1.0957792207792208e-04*Jinv12*w1)*D;
  A[55][38]=(0.0)*D;
  A[55][39]=(0.0)*D;
  A[55][40]=(1.6436688311688311e-04*Jinv00*w12-6.5746753246753243e-04*w28*Jinv10-6.9034090909090911e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10-6.9034090909090911e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+2.6298701298701297e-03*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23+2.5750811688311690e-03*Jinv22*w38-1.2820616883116883e-02*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-2.9586038961038961e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+2.7942370129870131e-03*Jinv01*w45-9.5332792207792200e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-9.6793831168831166e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+1.7532467532467534e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-5.4788961038961036e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+4.9310064935064934e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-6.5746753246753243e-04*Jinv00*w27-9.6793831168831166e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1-6.0267857142857139e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-1.6436688311688311e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-6.5746753246753243e-04*Jinv20*w29+4.9310064935064934e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+2.6298701298701297e-03*w31*Jinv10+2.1367694805194807e-03*w21*Jinv00+1.8628246753246754e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+2.6298701298701297e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38+2.7942370129870131e-03*Jinv02*w45+1.7532467532467534e-03*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+1.0957792207792208e-04*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20+1.6436688311688311e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+4.9310064935064932e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+2.5750811688311690e-03*w37*Jinv12-1.2820616883116883e-02*Jinv20*w56-6.9034090909090911e-03*Jinv22*w41+2.1367694805194807e-03*w26*Jinv20-5.4788961038961036e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12+4.9310064935064932e-04*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+2.7942370129870131e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21-9.5332792207792200e-03*Jinv02*w54-4.9310064935064932e-04*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-1.6436688311688311e-04*Jinv21*w29-2.9586038961038961e-03*w39*Jinv00+1.0957792207792208e-04*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-9.6793831168831166e-04*Jinv02*w9+5.4788961038961042e-05*w7*Jinv10+2.7942370129870131e-03*w47*Jinv21+2.6298701298701297e-03*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11+1.8628246753246754e-03*w31*Jinv12+2.7942370129870131e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53+8.2183441558441553e-04*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-2.0819805194805196e-03*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11+8.2183441558441553e-04*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.8628246753246754e-03*Jinv02*w30-2.0819805194805196e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10-5.4788961038961036e-04*w9*Jinv00+2.6298701298701297e-03*w37*Jinv10-8.5470779220779228e-03*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+8.2183441558441553e-04*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12+2.7942370129870131e-03*w47*Jinv22-6.0267857142857139e-04*Jinv21*w14-5.4788961038961036e-04*w10*Jinv10-2.9586038961038961e-03*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18-6.0267857142857139e-04*Jinv01*w12+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33-5.4788961038961036e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23-8.5470779220779228e-03*Jinv01*w54-8.5470779220779228e-03*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10-5.4788961038961036e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00-2.0819805194805196e-03*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17+2.6298701298701297e-03*w30*Jinv00-2.9586038961038961e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+2.5750811688311690e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+4.9310064935064932e-04*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-9.5332792207792200e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+1.7532467532467534e-03*w25*Jinv11-1.2820616883116883e-02*w55*Jinv10-2.9586038961038961e-03*w42*Jinv00+4.9310064935064934e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-1.6436688311688311e-04*Jinv01*w27-6.2094155844155847e-04*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[55][41]=(0.0)*D;
  A[55][42]=(0.0)*D;
  A[55][43]=(-1.6436688311688311e-04*Jinv00*w12+6.5746753246753243e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+2.1915584415584414e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-9.8620129870129864e-04*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-2.6298701298701297e-03*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23-7.6704545454545450e-04*Jinv22*w38+1.2820616883116883e-02*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-4.2004870129870131e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+3.2873376623376621e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-4.9310064935064934e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01+5.4788961038961036e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-4.9310064935064934e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+6.5746753246753243e-04*Jinv00*w27-7.3051948051948052e-05*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+4.2735389610389614e-03*Jinv20*w50+6.5746753246753243e-04*Jinv20*w29-4.9310064935064934e-03*w46*Jinv10-4.9310064935064934e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-2.6298701298701297e-03*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-4.2004870129870131e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-2.6298701298701297e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24-9.8620129870129864e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.2735389610389614e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-3.9448051948051946e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11+2.9586038961038961e-03*w41*Jinv20+1.1505681818181819e-03*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-7.6704545454545450e-04*w37*Jinv12+1.2820616883116883e-02*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20+5.4788961038961036e-04*w11*Jinv20+2.1915584415584414e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21+3.2873376623376621e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+2.9586038961038961e-03*w39*Jinv00+1.1505681818181819e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-2.1367694805194807e-03*w47*Jinv21-2.6298701298701297e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-5.4788961038961042e-05*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+4.2735389610389614e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11+5.4788961038961036e-04*w9*Jinv00-2.6298701298701297e-03*w37*Jinv10+4.2735389610389614e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+5.4788961038961036e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44-9.8620129870129864e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+5.4788961038961036e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10+5.4788961038961036e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17-2.6298701298701297e-03*w30*Jinv00+2.9586038961038961e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+1.1505681818181819e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30+2.1915584415584414e-03*Jinv02*w51+3.2873376623376621e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11+1.2820616883116883e-02*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-4.9310064935064934e-03*w47*Jinv20-4.9310064935064934e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-4.2004870129870131e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[55][44]=(0.0)*D;
  A[55][45]=(0.0)*D;
  A[55][46]=(-2.1367694805194807e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-4.7483766233766234e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+2.4655032467532466e-04*Jinv01*w3+5.2049512987012989e-04*Jinv22*w38-1.0957792207792207e-03*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-1.5523538961038962e-04*Jinv21*w2-6.5746753246753243e-04*w42*Jinv01-1.5523538961038962e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55-1.3149350649350649e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01-1.2327516233766234e-03*Jinv21*w26-1.5523538961038962e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11-6.5746753246753243e-04*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-1.5523538961038962e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01-1.4610389610389610e-04*w28*Jinv11-1.3149350649350649e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+8.2183441558441553e-05*Jinv21*w38-2.6298701298701297e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-4.7483766233766234e-04*Jinv22*w59-8.2183441558441553e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.4655032467532466e-04*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+8.2183441558441553e-05*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+5.2049512987012989e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-8.2183441558441553e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11+2.4655032467532466e-04*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02-1.4610389610389610e-04*Jinv21*w29-8.2183441558441553e-04*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+4.3831168831168834e-04*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01-1.3149350649350649e-03*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+5.2049512987012989e-04*w31*Jinv12-2.6298701298701297e-03*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+5.2049512987012989e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+2.4655032467532466e-04*w7*Jinv11+3.7804383116883116e-03*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-2.6298701298701297e-03*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-4.7483766233766234e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.2327516233766234e-03*Jinv21*w23+3.7804383116883116e-03*Jinv01*w54+3.7804383116883116e-03*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.5523538961038962e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+5.2049512987012989e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.2183441558441553e-05*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11-1.3149350649350649e-03*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-1.4610389610389610e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-1.5523538961038962e-04*w10*Jinv11)*D;
  A[55][47]=(0.0)*D;
  A[55][48]=(0.0)*D;
  A[55][49]=(7.1225649350649346e-04*Jinv00*w12-7.8530844155844158e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12-2.4655032467532467e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8-1.1414366883116883e-03*w57*Jinv02+7.1225649350649346e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+2.3833198051948050e-03*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38-7.3965097402597406e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59+8.2183441558441553e-05*w25*Jinv12-3.5612824675324673e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.6298701298701297e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-3.7439123376623376e-04*w10*Jinv12-3.6982548701298703e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-5.4788961038961036e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-4.2735389610389614e-03*Jinv21*w41-2.6846590909090908e-03*w42*Jinv02-7.8530844155844158e-04*Jinv00*w27-3.7439123376623376e-04*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.8530844155844158e-04*w28*Jinv11-3.6982548701298703e-03*Jinv20*w50-7.8530844155844158e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-2.1367694805194807e-03*Jinv00*w33+2.3833198051948050e-03*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+1.1505681818181819e-03*w32*Jinv22-4.2735389610389614e-03*Jinv01*w39-3.5612824675324673e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.3833198051948050e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+1.3971185064935066e-03*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24-1.1414366883116883e-03*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-3.6982548701298703e-03*Jinv21*w53-3.6982548701298703e-03*w51*Jinv00-5.4788961038961036e-04*Jinv12*w34-2.6846590909090908e-03*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10+2.3833198051948050e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12-7.3965097402597406e-03*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-2.4655032467532467e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+2.3833198051948050e-03*w31*Jinv11+8.2183441558441553e-05*Jinv22*w26+2.6298701298701297e-03*w46*Jinv11+7.1225649350649346e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.6982548701298703e-03*w49*Jinv10-7.8530844155844158e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11-3.7439123376623376e-04*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+2.6298701298701297e-03*w47*Jinv21+2.3833198051948050e-03*Jinv20*w38-3.6982548701298703e-03*w52*Jinv11+1.1505681818181819e-03*w31*Jinv12+1.3971185064935066e-03*w46*Jinv12-3.6982548701298703e-03*Jinv20*w53+1.6436688311688311e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+1.6436688311688311e-04*w21*Jinv01-3.6982548701298703e-03*Jinv01*w51+1.1505681818181819e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-3.6982548701298703e-03*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+2.3833198051948050e-03*w37*Jinv10-7.3965097402597406e-03*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+3.5612824675324673e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5+1.3971185064935066e-03*w47*Jinv22-3.6982548701298703e-03*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-1.1414366883116883e-03*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-5.4788961038961036e-04*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23-7.3965097402597406e-03*Jinv01*w54-7.3965097402597406e-03*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-7.4878246753246752e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.3833198051948050e-03*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-4.2735389610389614e-03*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.6846590909090908e-03*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+2.3833198051948050e-03*Jinv01*w30-2.4655032467532467e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-7.3965097402597406e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-3.6982548701298703e-03*Jinv21*w50-7.8530844155844158e-04*Jinv01*w27-3.6982548701298703e-03*w48*Jinv01-3.5612824675324673e-04*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[55][50]=(0.0)*D;
  A[55][51]=(0.0)*D;
  A[55][52]=(-7.1225649350649346e-04*Jinv00*w12+7.8530844155844158e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-4.5657467532467534e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-3.5612824675324673e-04*w3*Jinv00-2.3833198051948050e-03*w32*Jinv20-1.6436688311688311e-04*Jinv20*w23-1.2327516233766234e-03*Jinv22*w38+7.3965097402597406e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12+3.7439123376623376e-04*w0*Jinv02+4.2735389610389614e-03*w43*Jinv10+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+3.6982548701298703e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-2.6298701298701297e-03*w45*Jinv00+3.1777597402597403e-03*w42*Jinv02+7.8530844155844158e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+3.6982548701298703e-03*Jinv20*w50+7.8530844155844158e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+1.0957792207792207e-03*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12-2.3833198051948050e-03*w31*Jinv10-3.5612824675324673e-04*Jinv02*w3-1.6436688311688311e-04*w21*Jinv00-1.0135957792207792e-03*w32*Jinv22+3.7439123376623376e-04*Jinv22*w2-7.1225649350649346e-04*Jinv00*w18-2.3833198051948050e-03*w36*Jinv00-1.2327516233766234e-03*Jinv02*w45-4.5657467532467534e-05*Jinv22*w59+3.6982548701298703e-03*w51*Jinv00+1.5888798701298701e-03*Jinv12*w34+3.1777597402597403e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+4.2735389610389614e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-7.1225649350649346e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-1.2327516233766234e-03*w37*Jinv12+7.3965097402597406e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+2.4655032467532467e-03*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+2.4655032467532467e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02+3.6982548701298703e-03*w49*Jinv10+4.2735389610389614e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+3.9265422077922079e-04*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-2.3833198051948050e-03*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23-1.0135957792207792e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12+3.6982548701298703e-03*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-1.6436688311688311e-04*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50-8.2183441558441553e-05*Jinv12*w7-1.0135957792207792e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49+3.6982548701298703e-03*w52*Jinv10+7.4878246753246752e-04*w9*Jinv00-2.3833198051948050e-03*w37*Jinv10-1.6436688311688311e-04*w24*Jinv00+3.9265422077922079e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+7.4878246753246752e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-4.5657467532467534e-05*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+1.5888798701298701e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-1.6436688311688311e-04*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.3833198051948050e-03*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-7.1225649350649346e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-1.2327516233766234e-03*Jinv02*w36+3.1777597402597403e-03*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+2.4655032467532467e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28+7.3965097402597406e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+1.0957792207792207e-03*w16*Jinv12+3.7439123376623376e-04*Jinv12*w1)*D;
  A[55][53]=(0.0)*D;
  A[55][54]=(0.0)*D;
  A[55][55]=(4.7666396103896100e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-9.1314935064935068e-05*w57*Jinv02+4.7666396103896100e-03*Jinv02*w39+4.2735389610389614e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-2.0271915584415584e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59+5.7528409090909093e-04*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.6298701298701297e-03*Jinv01*w45+7.3965097402597406e-03*Jinv12*w55+1.6436688311688311e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+5.7528409090909093e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-1.0683847402597404e-03*Jinv22*w20+4.2735389610389614e-03*w43*Jinv11+4.2735389610389614e-03*Jinv21*w41+4.7666396103896100e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.8530844155844158e-04*w28*Jinv11+1.6436688311688311e-03*Jinv22*w17+5.7528409090909093e-04*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-2.0271915584415584e-03*w32*Jinv22+4.2735389610389614e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-2.3833198051948050e-03*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-9.1314935064935068e-05*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11+3.6982548701298703e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+4.7666396103896100e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-1.0683847402597404e-03*Jinv12*w13-2.3833198051948050e-03*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-2.0271915584415584e-03*w37*Jinv12+4.7666396103896100e-03*Jinv22*w41+2.4655032467532467e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-2.3833198051948050e-03*w31*Jinv11+5.7528409090909093e-04*Jinv22*w26-2.6298701298701297e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21+7.3965097402597406e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12+5.7528409090909093e-04*w21*Jinv02+7.8530844155844158e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-1.0683847402597404e-03*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01-2.6298701298701297e-03*w47*Jinv21+5.7528409090909093e-04*Jinv22*w23+3.6982548701298703e-03*w52*Jinv11-2.0271915584415584e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.4655032467532467e-03*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01+3.6982548701298703e-03*Jinv01*w51-2.0271915584415584e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49-2.3833198051948050e-03*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11+7.3965097402597406e-03*Jinv21*w56+6.5746753246753243e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.0683847402597404e-03*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22+3.6982548701298703e-03*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-9.1314935064935068e-05*w58*Jinv12-1.0683847402597404e-03*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23+7.3965097402597406e-03*Jinv01*w54+7.3965097402597406e-03*w55*Jinv11+2.4655032467532467e-03*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+6.5746753246753243e-04*Jinv22*w29-1.0683847402597404e-03*Jinv22*w14+4.2735389610389614e-03*w40*Jinv11-2.0271915584415584e-03*Jinv02*w36+4.7666396103896100e-03*Jinv12*w43-2.3833198051948050e-03*Jinv01*w30+2.4655032467532467e-03*Jinv02*w51+7.3965097402597406e-03*Jinv22*w56+6.5746753246753243e-04*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11+1.6436688311688311e-03*w16*Jinv12+3.6982548701298703e-03*Jinv21*w50+7.8530844155844158e-04*Jinv01*w27+3.6982548701298703e-03*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[55][56]=(0.0)*D;
  A[55][57]=(0.0)*D;
  A[55][58]=(6.5746753246753243e-04*w40*Jinv12+1.1870941558441558e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-6.7573051948051951e-04*Jinv22*w38-2.6481331168831169e-04*w25*Jinv12+1.5726461038961039e-04*w0*Jinv02+2.3741883116883116e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12-2.6481331168831169e-04*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35-1.0957792207792208e-04*Jinv22*w20+6.5746753246753243e-04*w42*Jinv02+1.5726461038961039e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-2.6481331168831169e-04*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-6.7573051948051951e-04*w32*Jinv22+1.5726461038961039e-04*Jinv22*w2-1.0227272727272728e-03*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+3.2873376623376621e-04*Jinv12*w34+6.5746753246753243e-04*Jinv22*w44-1.0957792207792208e-04*Jinv12*w13-6.7573051948051951e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41+1.1870941558441558e-03*w52*Jinv12-2.6481331168831169e-04*Jinv22*w26+2.3741883116883116e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12-2.6481331168831169e-04*w21*Jinv02-1.0957792207792208e-04*Jinv02*w12+1.5726461038961039e-04*Jinv02*w9-2.6481331168831169e-04*Jinv22*w23-6.7573051948051951e-04*w31*Jinv12-1.0227272727272728e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8+1.1870941558441558e-03*Jinv22*w50-5.4788961038961042e-05*Jinv12*w7-6.7573051948051951e-04*Jinv02*w30+1.1870941558441558e-03*Jinv12*w49+2.0495129870129870e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.0957792207792208e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-1.0227272727272728e-03*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-1.0957792207792208e-04*Jinv02*w18+3.2873376623376621e-04*Jinv02*w33+1.1870941558441558e-03*w48*Jinv02+2.0495129870129870e-04*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-6.7573051948051951e-04*Jinv02*w36+6.5746753246753243e-04*Jinv12*w43+1.1870941558441558e-03*Jinv02*w51+2.3741883116883116e-03*Jinv22*w56+2.0495129870129870e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+1.5726461038961039e-04*Jinv12*w1)*D;
  A[55][59]=(0.0)*D;
  A[56][0]=(0.0)*D;
  A[56][1]=(0.0)*D;
  A[56][2]=(1.0957792207792208e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-4.3831168831168834e-04*w40*Jinv12-7.4878246753246752e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8-6.4935064935064935e-05*w57*Jinv02+1.0957792207792208e-04*w13*Jinv10-4.3831168831168834e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+2.6481331168831169e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-1.1870941558441559e-04*Jinv20*w23+4.2918019480519480e-04*Jinv22*w38-1.4975649350649350e-03*Jinv00*w54-6.4935064935064935e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-2.0292207792207792e-05*w0*Jinv02-2.0292207792207792e-05*Jinv21*w2-5.4788961038961036e-04*w43*Jinv10-5.4788961038961036e-04*w42*Jinv01-1.0247564935064935e-04*w11*Jinv21-3.4699675324675324e-04*w16*Jinv11-3.4699675324675324e-04*Jinv21*w17+5.8441558441558442e-04*Jinv01*w45-1.4975649350649350e-03*Jinv12*w55-6.4935064935064935e-05*w58*Jinv10-3.4699675324675324e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-3.6525974025974026e-05*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-6.4935064935064935e-05*w58*Jinv11-6.4935064935064935e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-2.0292207792207792e-05*w0*Jinv01-2.0292207792207792e-05*w1*Jinv10-3.4699675324675324e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11+5.8441558441558442e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-5.4788961038961036e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-2.0292207792207792e-05*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-3.4699675324675324e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11-3.6525974025974026e-05*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+5.8441558441558442e-04*w46*Jinv10-3.4699675324675324e-04*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33-1.1870941558441559e-04*w22*Jinv12+2.6481331168831169e-04*w31*Jinv10+5.4788961038961042e-05*Jinv02*w3-1.1870941558441559e-04*w21*Jinv00+2.6481331168831169e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39-2.0292207792207792e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+4.2918019480519480e-04*w36*Jinv00+4.2918019480519480e-04*Jinv21*w38+5.8441558441558442e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-6.4935064935064935e-05*Jinv22*w59-2.1915584415584417e-04*w34*Jinv11+2.1915584415584417e-04*w19*Jinv11-7.4878246753246752e-04*Jinv21*w53-7.4878246753246752e-04*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34-5.4788961038961036e-04*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+2.6481331168831169e-04*w32*Jinv21+2.1915584415584417e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00+4.2918019480519480e-04*w37*Jinv12-1.4975649350649350e-03*Jinv20*w56-4.3831168831168834e-04*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-7.4878246753246752e-04*w52*Jinv12-2.1915584415584417e-04*Jinv21*w35+2.6481331168831169e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+5.8441558441558442e-04*w46*Jinv11+1.0957792207792208e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-1.4975649350649350e-03*Jinv02*w54+5.4788961038961042e-05*w4*Jinv12-1.1870941558441559e-04*w21*Jinv02-3.6525974025974026e-05*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-2.1915584415584417e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12+2.1915584415584417e-04*Jinv01*w18+4.2918019480519480e-04*w37*Jinv11-1.0247564935064935e-04*Jinv02*w9+2.7394480519480521e-05*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10+5.8441558441558442e-04*w47*Jinv21+4.2918019480519480e-04*Jinv20*w38-1.1870941558441559e-04*Jinv22*w23-7.4878246753246752e-04*w52*Jinv11+2.6481331168831169e-04*w31*Jinv12+5.8441558441558442e-04*w46*Jinv12-7.4878246753246752e-04*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-1.1870941558441559e-04*w22*Jinv10-3.6525974025974026e-05*Jinv22*w50-1.1870941558441559e-04*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-1.1870941558441559e-04*w21*Jinv01-7.4878246753246752e-04*Jinv01*w51+2.6481331168831169e-04*Jinv02*w30-3.6525974025974026e-05*Jinv12*w49+4.2918019480519480e-04*w36*Jinv01-7.4878246753246752e-04*w52*Jinv10+2.7394480519480521e-05*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+4.2918019480519480e-04*w37*Jinv10-1.4975649350649350e-03*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12+5.4788961038961042e-05*Jinv22*w5+5.8441558441558442e-04*w47*Jinv22-3.6525974025974026e-05*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44-6.4935064935064935e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12+2.7394480519480521e-05*Jinv21*w8-6.4935064935064935e-05*w57*Jinv00-6.4935064935064935e-05*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-2.0292207792207792e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-1.1870941558441559e-04*Jinv21*w23-1.4975649350649350e-03*Jinv01*w54-1.4975649350649350e-03*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10-2.0292207792207792e-05*Jinv20*w2-3.4699675324675324e-04*w15*Jinv00-3.6525974025974026e-05*w48*Jinv02-1.0247564935064935e-04*Jinv01*w9-3.4699675324675324e-04*Jinv20*w17+2.6481331168831169e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11+4.2918019480519480e-04*Jinv02*w36-5.4788961038961036e-04*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+2.6481331168831169e-04*Jinv01*w30-7.4878246753246752e-04*Jinv02*w51-1.4975649350649350e-03*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11-1.4975649350649350e-03*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00+5.8441558441558442e-04*w47*Jinv20-3.4699675324675324e-04*w16*Jinv12-3.6525974025974026e-05*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27-3.6525974025974026e-05*w48*Jinv01-2.0292207792207792e-05*Jinv12*w1-1.0247564935064935e-04*w10*Jinv11)*D;
  A[56][3]=(0.0)*D;
  A[56][4]=(0.0)*D;
  A[56][5]=(-3.5612824675324673e-04*Jinv00*w12+7.1225649350649346e-04*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53+2.7394480519480521e-05*Jinv20*w8-9.1314935064935065e-06*w57*Jinv02-3.5612824675324673e-04*w13*Jinv10+7.1225649350649346e-04*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44-8.2183441558441553e-05*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-9.1314935064935065e-06*Jinv21*w59-5.4788961038961036e-04*w25*Jinv12-6.3920454545454550e-05*w0*Jinv02-6.3920454545454550e-05*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.3697240259740259e-04*w11*Jinv21+8.7662337662337668e-04*w16*Jinv11+8.7662337662337668e-04*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+8.7662337662337668e-04*w15*Jinv02+1.3697240259740259e-04*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.4788961038961036e-04*Jinv21*w26-6.3920454545454550e-05*w0*Jinv01-5.4788961038961042e-05*w1*Jinv10+8.2183441558441553e-04*w16*Jinv10-5.4788961038961036e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11+7.1225649350649346e-04*Jinv21*w41+1.0957792207792208e-04*w42*Jinv02+1.3697240259740259e-04*w11*Jinv22-6.3920454545454550e-05*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+8.7662337662337668e-04*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+8.7662337662337668e-04*Jinv22*w17+3.2873376623376621e-04*Jinv00*w33-2.4655032467532466e-04*w22*Jinv12-8.2183441558441553e-05*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+8.2183441558441553e-05*w32*Jinv22+7.1225649350649346e-04*Jinv01*w39-6.3920454545454550e-05*Jinv22*w2-4.6570616883116886e-04*Jinv00*w18-2.4655032467532466e-04*w36*Jinv00-5.2049512987012989e-04*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-5.4788961038961036e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+7.1225649350649346e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+7.1225649350649346e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.0957792207792208e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13-4.6570616883116886e-04*w19*Jinv10+3.2873376623376621e-04*w34*Jinv10+8.2183441558441553e-05*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20+2.7394480519480521e-05*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12+7.1225649350649346e-04*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+2.7394480519480521e-05*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+7.1225649350649346e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11-3.5612824675324673e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.4655032467532466e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+7.1225649350649346e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-5.2049512987012989e-04*w37*Jinv11+1.3697240259740259e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01+2.7394480519480521e-05*w7*Jinv10-5.2049512987012989e-04*w47*Jinv21-2.4655032467532466e-04*Jinv20*w38-2.4655032467532466e-04*Jinv22*w23+7.1225649350649346e-04*w52*Jinv11+8.2183441558441553e-05*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53-2.4655032467532466e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10-1.9176136363636362e-04*Jinv22*w50-2.4655032467532466e-04*w22*Jinv11-2.4655032467532466e-04*Jinv12*w7-2.4655032467532466e-04*w21*Jinv01+7.1225649350649346e-04*Jinv01*w51+8.2183441558441553e-05*Jinv02*w30-1.9176136363636362e-04*Jinv12*w49-5.2049512987012989e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+2.7394480519480521e-05*w9*Jinv00-2.4655032467532466e-04*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27-2.4655032467532466e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22-1.9176136363636362e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+2.7394480519480521e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+7.1225649350649346e-04*Jinv02*w33-5.4788961038961042e-05*w0*Jinv00+3.2873376623376621e-04*w35*Jinv20-2.4655032467532466e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-5.4788961038961042e-05*Jinv20*w2+8.2183441558441553e-04*w15*Jinv00-1.9176136363636362e-04*w48*Jinv02+1.3697240259740259e-04*Jinv01*w9+8.2183441558441553e-04*Jinv20*w17-8.2183441558441553e-05*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10-4.6570616883116886e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14+7.1225649350649346e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+1.0957792207792208e-04*Jinv12*w43+8.2183441558441553e-05*Jinv01*w30+7.1225649350649346e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.4788961038961036e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+8.7662337662337668e-04*w16*Jinv12-1.9176136363636362e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27-1.9176136363636362e-04*w48*Jinv01-6.3920454545454550e-05*Jinv12*w1+1.3697240259740259e-04*w10*Jinv11)*D;
  A[56][6]=(0.0)*D;
  A[56][7]=(0.0)*D;
  A[56][8]=(4.6570616883116886e-04*Jinv00*w12+5.4788961038961042e-05*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-9.1314935064935065e-06*w57*Jinv02+4.6570616883116886e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480521e-05*w3*Jinv00+2.4655032467532466e-04*w32*Jinv20-2.7394480519480518e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23+1.6436688311688311e-04*Jinv22*w38-9.1314935064935065e-06*Jinv21*w59-5.2049512987012989e-04*w25*Jinv12+1.0957792207792208e-04*w0*Jinv02+1.0957792207792208e-04*Jinv21*w2+5.4788961038961042e-05*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01-9.1314935064935065e-06*w11*Jinv21+5.4788961038961042e-05*w16*Jinv11+5.4788961038961042e-05*Jinv21*w17-5.2049512987012989e-04*Jinv01*w45+7.1225649350649346e-04*Jinv12*w55+5.4788961038961042e-05*w15*Jinv02-9.1314935064935065e-06*w10*Jinv12-3.5612824675324673e-04*w48*Jinv00-5.2049512987012989e-04*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+1.0957792207792208e-04*w0*Jinv01-2.7394480519480521e-05*w1*Jinv10-8.2183441558441553e-04*w16*Jinv10+7.1225649350649346e-04*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+5.4788961038961042e-05*Jinv21*w41+7.6704545454545450e-04*w42*Jinv02-9.1314935064935065e-06*w11*Jinv22+1.0957792207792208e-04*Jinv11*w1-8.2183441558441553e-05*w13*Jinv11+5.4788961038961042e-05*w15*Jinv01+1.3697240259740259e-04*w28*Jinv11-3.5612824675324673e-04*Jinv20*w50+5.4788961038961042e-05*Jinv22*w17-3.2873376623376621e-04*Jinv00*w33-2.7394480519480518e-04*w22*Jinv12+2.4655032467532466e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00-2.7394480519480518e-04*w32*Jinv22+5.4788961038961042e-05*Jinv01*w39+1.0957792207792208e-04*Jinv22*w2+3.5612824675324673e-04*Jinv00*w18+8.2183441558441553e-05*w36*Jinv00+1.6436688311688311e-04*Jinv21*w38-5.2049512987012989e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24-9.1314935064935065e-06*Jinv22*w59+3.8352272727272725e-04*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11+1.6436688311688311e-04*Jinv21*w53+3.5612824675324673e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+7.6704545454545450e-04*Jinv22*w44-2.7394480519480521e-05*w4*Jinv10-2.7394480519480518e-04*w4*Jinv11-5.4788961038961042e-05*w41*Jinv20-8.2183441558441553e-05*Jinv12*w13+3.5612824675324673e-04*w19*Jinv10-3.2873376623376621e-04*w34*Jinv10-2.7394480519480518e-04*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+1.6436688311688311e-04*w37*Jinv12+5.4788961038961042e-05*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+5.4788961038961042e-05*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12+3.8352272727272725e-04*Jinv21*w35-2.7394480519480518e-04*w31*Jinv11-5.2049512987012989e-04*Jinv22*w26-5.2049512987012989e-04*w46*Jinv11+4.6570616883116886e-04*Jinv20*w14-2.7394480519480518e-04*w5*Jinv21+7.1225649350649346e-04*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02-3.5612824675324673e-04*w49*Jinv10+1.3697240259740259e-04*Jinv21*w29+3.8352272727272725e-04*Jinv01*w33-5.4788961038961042e-05*w39*Jinv00-8.2183441558441553e-05*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+1.6436688311688311e-04*w37*Jinv11-9.1314935064935065e-06*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01-5.2049512987012989e-04*w47*Jinv21+8.2183441558441553e-05*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23+1.6436688311688311e-04*w52*Jinv11-2.7394480519480518e-04*w31*Jinv12-5.2049512987012989e-04*w46*Jinv12+3.5612824675324673e-04*Jinv20*w53+3.5612824675324673e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10+3.5612824675324673e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+3.5612824675324673e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01+1.6436688311688311e-04*Jinv01*w51-2.7394480519480518e-04*Jinv02*w30+3.5612824675324673e-04*Jinv12*w49+1.6436688311688311e-04*w36*Jinv01+3.5612824675324673e-04*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11+5.4788961038961042e-05*w9*Jinv00+8.2183441558441553e-05*w37*Jinv10+7.1225649350649346e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00+1.3697240259740259e-04*Jinv02*w27+3.5612824675324673e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5-5.2049512987012989e-04*w47*Jinv22+3.5612824675324673e-04*w49*Jinv11-8.2183441558441553e-05*Jinv21*w14+5.4788961038961042e-05*w10*Jinv10+5.4788961038961042e-05*Jinv20*w44-9.1314935064935065e-06*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18-8.2183441558441553e-05*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8+3.8352272727272725e-04*Jinv02*w33-2.7394480519480521e-05*w0*Jinv00-3.2873376623376621e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23+7.1225649350649346e-04*Jinv01*w54+7.1225649350649346e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-2.7394480519480521e-05*Jinv20*w2-8.2183441558441553e-04*w15*Jinv00+3.5612824675324673e-04*w48*Jinv02-9.1314935064935065e-06*Jinv01*w9-8.2183441558441553e-04*Jinv20*w17+2.4655032467532466e-04*w30*Jinv00-5.4788961038961042e-05*w40*Jinv10+3.5612824675324673e-04*w20*Jinv20+1.3697240259740259e-04*Jinv22*w29-8.2183441558441553e-05*Jinv22*w14+5.4788961038961042e-05*w40*Jinv11+1.6436688311688311e-04*Jinv02*w36+7.6704545454545450e-04*Jinv12*w43-2.7394480519480521e-05*w5*Jinv20-2.7394480519480518e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51+7.1225649350649346e-04*Jinv22*w56+1.3697240259740259e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+5.4788961038961042e-05*w42*Jinv00+5.4788961038961042e-05*w16*Jinv12+3.5612824675324673e-04*Jinv21*w50+1.3697240259740259e-04*Jinv01*w27+3.5612824675324673e-04*w48*Jinv01+1.0957792207792208e-04*Jinv12*w1-9.1314935064935065e-06*w10*Jinv11)*D;
  A[56][9]=(0.0)*D;
  A[56][10]=(0.0)*D;
  A[56][11]=(-2.1915584415584417e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-5.4788961038961042e-05*Jinv20*w8-2.1915584415584417e-04*w13*Jinv10-2.7394480519480521e-05*w3*Jinv00-4.2918019480519480e-04*w32*Jinv20-4.2918019480519480e-04*Jinv20*w23+1.4975649350649350e-03*Jinv00*w54+4.3831168831168834e-04*w43*Jinv10+6.4935064935064935e-05*w58*Jinv10+7.4878246753246752e-04*w48*Jinv00+1.0247564935064935e-04*w1*Jinv10+3.4699675324675324e-04*w16*Jinv10-5.8441558441558442e-04*w45*Jinv00-1.4204545454545455e-05*Jinv00*w27+7.4878246753246752e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-5.8441558441558442e-04*w46*Jinv10+2.1915584415584417e-04*Jinv00*w33-4.2918019480519480e-04*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-1.0957792207792208e-04*Jinv00*w18-2.6481331168831169e-04*w36*Jinv00+3.6525974025974026e-05*w51*Jinv00-2.7394480519480521e-05*w4*Jinv10+5.4788961038961036e-04*w41*Jinv20-1.0957792207792208e-04*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10-5.4788961038961042e-05*w6*Jinv00+1.4975649350649350e-03*Jinv20*w56+1.1870941558441559e-04*w26*Jinv20+2.0292207792207792e-05*w11*Jinv20-2.1915584415584417e-04*Jinv20*w14+7.4878246753246752e-04*w49*Jinv10+5.4788961038961036e-04*w39*Jinv00-5.4788961038961042e-05*w7*Jinv10-2.6481331168831169e-04*Jinv20*w38+3.6525974025974026e-05*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10+3.6525974025974026e-05*w52*Jinv10+2.0292207792207792e-05*w9*Jinv00-2.6481331168831169e-04*w37*Jinv10+1.1870941558441559e-04*w24*Jinv00+2.0292207792207792e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+6.4935064935064935e-05*w57*Jinv00+6.4935064935064935e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20+1.1870941558441559e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+3.4699675324675324e-04*w15*Jinv00+3.4699675324675324e-04*Jinv20*w17-4.2918019480519480e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-1.0957792207792208e-04*w20*Jinv20-2.7394480519480521e-05*w5*Jinv20+1.4975649350649350e-03*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00-5.8441558441558442e-04*w47*Jinv20)*D;
  A[56][12]=(0.0)*D;
  A[56][13]=(0.0)*D;
  A[56][14]=(1.0683847402597404e-03*Jinv00*w12-2.1915584415584417e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12+7.1225649350649346e-04*Jinv22*w53-4.6570616883116886e-04*Jinv20*w8-1.8262987012987013e-05*w57*Jinv02+1.0683847402597404e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39+1.0957792207792208e-04*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+1.6436688311688311e-04*w32*Jinv20-8.2183441558441553e-05*Jinv20*w23-5.4788961038961036e-04*Jinv22*w38+2.1367694805194807e-03*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59-1.3971185064935066e-03*w25*Jinv12-1.2784090909090910e-04*w0*Jinv02-1.1870941558441559e-04*Jinv21*w2+1.6436688311688311e-04*w43*Jinv10+1.0957792207792208e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21+1.2601461038961039e-03*w16*Jinv11+1.2601461038961039e-03*Jinv21*w17-9.5880681818181818e-04*Jinv01*w45+2.1367694805194807e-03*Jinv12*w55-1.8262987012987013e-05*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+2.1915584415584417e-04*w10*Jinv12-3.8352272727272725e-04*w48*Jinv00-1.3971185064935066e-03*Jinv02*w24+7.1225649350649346e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01-1.1231737012987013e-03*Jinv21*w26-1.1870941558441559e-04*w0*Jinv01-1.2784090909090910e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-2.4655032467532466e-04*Jinv22*w20+1.0957792207792208e-04*w43*Jinv11-1.4793019480519481e-03*w45*Jinv00+1.3697240259740260e-03*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-2.1915584415584417e-04*Jinv00*w27+2.1915584415584417e-04*w11*Jinv22-1.1870941558441559e-04*Jinv11*w1+3.5612824675324673e-04*w13*Jinv11+1.2601461038961039e-03*w15*Jinv01-3.5612824675324673e-04*w28*Jinv11-3.8352272727272725e-04*Jinv20*w50-2.1915584415584417e-04*Jinv20*w29-1.4793019480519481e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+7.1225649350649346e-04*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12+1.6436688311688311e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3-8.2183441558441553e-05*w21*Jinv00+1.6436688311688311e-04*w32*Jinv22+1.3697240259740260e-03*Jinv01*w39-1.2784090909090910e-04*Jinv22*w2-2.4655032467532466e-04*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38-1.4793019480519481e-03*Jinv02*w45-1.1231737012987013e-03*Jinv01*w24-1.8262987012987013e-05*Jinv22*w59+3.2873376623376621e-04*w34*Jinv11-1.6436688311688311e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+7.1225649350649346e-04*w51*Jinv00+7.1225649350649346e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+2.1367694805194807e-03*w41*Jinv20+1.0683847402597404e-03*Jinv12*w13-2.4655032467532466e-04*w19*Jinv10+7.1225649350649346e-04*w34*Jinv10-1.6436688311688311e-04*Jinv21*w20-4.6570616883116886e-04*w6*Jinv00-5.4788961038961036e-04*w37*Jinv12+2.1367694805194807e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41-1.3971185064935066e-03*w26*Jinv20+2.1915584415584417e-04*w11*Jinv20+7.1225649350649346e-04*w52*Jinv12+3.2873376623376621e-04*Jinv21*w35-1.3971185064935066e-03*Jinv22*w26-9.5880681818181818e-04*w46*Jinv11+1.0683847402597404e-03*Jinv20*w14+2.1367694805194807e-03*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02-3.8352272727272725e-04*w49*Jinv10-3.5612824675324673e-04*Jinv21*w29+3.2873376623376621e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00+1.0683847402597404e-03*Jinv02*w12-1.6436688311688311e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11+2.1915584415584417e-04*Jinv02*w9-1.9176136363636362e-04*w6*Jinv01-4.6570616883116886e-04*w7*Jinv10-9.5880681818181818e-04*w47*Jinv21-5.4788961038961036e-04*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11+1.6436688311688311e-04*w31*Jinv12-1.4793019480519481e-03*w46*Jinv12+7.1225649350649346e-04*Jinv20*w53-4.6570616883116886e-04*Jinv22*w8-8.2183441558441553e-05*w22*Jinv10-3.8352272727272725e-04*Jinv22*w50+4.3831168831168834e-04*w22*Jinv11-4.6570616883116886e-04*Jinv12*w7+4.3831168831168834e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51+1.6436688311688311e-04*Jinv02*w30-3.8352272727272725e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+7.1225649350649346e-04*w52*Jinv10-1.9176136363636362e-04*w7*Jinv11+2.1915584415584417e-04*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+1.4245129870129869e-03*Jinv21*w56-1.3971185064935066e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27-4.6570616883116886e-04*w6*Jinv02-2.4655032467532466e-04*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-1.4793019480519481e-03*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11+3.5612824675324673e-04*Jinv21*w14+2.1915584415584417e-04*w10*Jinv10+1.6436688311688311e-04*Jinv20*w44-1.8262987012987013e-05*w58*Jinv12-2.4655032467532466e-04*Jinv02*w18+3.5612824675324673e-04*Jinv01*w12-1.9176136363636362e-04*Jinv21*w8-1.8262987012987013e-05*w57*Jinv00-1.8262987012987013e-05*Jinv20*w59+7.1225649350649346e-04*Jinv02*w33-1.2784090909090910e-04*w0*Jinv00+7.1225649350649346e-04*w35*Jinv20+4.3831168831168834e-04*Jinv21*w23+1.4245129870129869e-03*Jinv01*w54+1.4245129870129869e-03*w55*Jinv11-1.3971185064935066e-03*w25*Jinv10-1.2784090909090910e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00-3.8352272727272725e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17+1.6436688311688311e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-2.4655032467532466e-04*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29+1.0683847402597404e-03*Jinv22*w14+1.3697240259740260e-03*w40*Jinv11-5.4788961038961036e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+7.1225649350649346e-04*Jinv02*w51+2.1367694805194807e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28-1.1231737012987013e-03*w25*Jinv11+2.1367694805194807e-03*w55*Jinv10+1.6436688311688311e-04*w42*Jinv00-1.4793019480519481e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50-3.5612824675324673e-04*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01-1.2784090909090910e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[56][15]=(0.0)*D;
  A[56][16]=(0.0)*D;
  A[56][17]=(-1.3149350649350649e-03*Jinv00*w12+1.9724025974025973e-03*w40*Jinv12-2.1915584415584414e-03*Jinv22*w53+8.2183441558441553e-04*Jinv20*w8+9.8620129870129864e-04*w57*Jinv02-1.3149350649350649e-03*w13*Jinv10+1.9724025974025973e-03*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44-8.2183441558441553e-04*w3*Jinv00-7.1225649350649346e-04*w32*Jinv20-8.2183441558441553e-04*Jinv01*w3-1.3149350649350649e-03*Jinv20*w23+7.6704545454545450e-04*Jinv22*w38+4.9310064935064932e-04*Jinv21*w59+2.6298701298701297e-03*w25*Jinv12+4.2004870129870131e-04*w0*Jinv02+3.8352272727272725e-04*Jinv21*w2+1.9724025974025973e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+3.6525974025974026e-05*w11*Jinv21+1.6436688311688311e-03*w16*Jinv11+1.6436688311688311e-03*Jinv21*w17+1.7532467532467534e-03*Jinv01*w45-3.2873376623376621e-03*Jinv12*w55+4.9310064935064934e-03*w15*Jinv02+7.3051948051948052e-05*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+2.6298701298701297e-03*Jinv02*w24+1.3149350649350649e-03*Jinv22*w35+4.9310064935064932e-04*w58*Jinv11+4.9310064935064932e-04*w57*Jinv01+2.5750811688311690e-03*Jinv21*w26+3.8352272727272725e-04*w0*Jinv01+3.4699675324675324e-04*w1*Jinv10+1.6436688311688311e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11+6.5746753246753243e-04*Jinv21*w41+3.9448051948051946e-03*w42*Jinv02+7.3051948051948052e-05*w11*Jinv22+3.8352272727272725e-04*Jinv11*w1-1.2053571428571428e-03*w13*Jinv11+1.6436688311688311e-03*w15*Jinv01-2.5568181818181820e-04*w28*Jinv11+1.0957792207792207e-03*Jinv20*w50+4.9310064935064934e-03*Jinv22*w17+1.3149350649350649e-03*w22*Jinv12-7.1225649350649346e-04*w31*Jinv10-7.6704545454545450e-04*Jinv02*w3-1.3149350649350649e-03*w21*Jinv00+5.4788961038961042e-05*w32*Jinv22+6.5746753246753243e-04*Jinv01*w39+4.2004870129870131e-04*Jinv22*w2+1.3149350649350649e-03*Jinv00*w18+7.1225649350649346e-04*w36*Jinv00+3.8352272727272725e-04*Jinv21*w38+2.1367694805194807e-03*Jinv02*w45+2.5750811688311690e-03*Jinv01*w24+9.8620129870129864e-04*Jinv22*w59+1.0957792207792208e-04*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00+1.3149350649350649e-03*Jinv12*w34+3.9448051948051946e-03*Jinv22*w44-8.2183441558441553e-04*w4*Jinv10-8.2183441558441553e-04*w4*Jinv11-1.9724025974025973e-03*w41*Jinv20-1.1505681818181819e-03*Jinv12*w13+1.3149350649350649e-03*w19*Jinv10-3.2873376623376621e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+8.2183441558441553e-04*w6*Jinv00+7.6704545454545450e-04*w37*Jinv12+1.9724025974025973e-03*Jinv22*w41+1.3149350649350649e-03*w26*Jinv20-3.4699675324675324e-04*w11*Jinv20-2.1915584415584414e-03*w52*Jinv12-3.2873376623376621e-04*w31*Jinv11+2.6298701298701297e-03*Jinv22*w26+1.7532467532467534e-03*w46*Jinv11-1.3149350649350649e-03*Jinv20*w14-8.2183441558441553e-04*w5*Jinv21-3.2873376623376621e-03*Jinv02*w54-7.6704545454545450e-04*w4*Jinv12+1.3149350649350649e-03*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-2.5568181818181820e-04*Jinv21*w29-1.9724025974025973e-03*w39*Jinv00-1.1505681818181819e-03*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18+3.8352272727272725e-04*w37*Jinv11+7.3051948051948052e-05*Jinv02*w9+8.2183441558441553e-04*w7*Jinv10+1.7532467532467534e-03*w47*Jinv21+7.1225649350649346e-04*Jinv20*w38+1.3149350649350649e-03*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11+5.4788961038961042e-05*w31*Jinv12+2.1367694805194807e-03*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-1.3149350649350649e-03*w22*Jinv10-1.0957792207792207e-03*Jinv22*w50+1.2601461038961039e-03*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+1.2601461038961039e-03*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51+5.4788961038961042e-05*Jinv02*w30-1.0957792207792207e-03*Jinv12*w49+3.8352272727272725e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10-3.4699675324675324e-04*w9*Jinv00+7.1225649350649346e-04*w37*Jinv10-2.1915584415584414e-03*Jinv21*w56+1.3149350649350649e-03*w24*Jinv00-2.1915584415584417e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02+1.6436688311688311e-04*w19*Jinv12-7.6704545454545450e-04*Jinv22*w5+2.1367694805194807e-03*w47*Jinv22-1.2053571428571428e-03*Jinv21*w14-3.4699675324675324e-04*w10*Jinv10+1.9724025974025973e-03*Jinv20*w44+9.8620129870129864e-04*w58*Jinv12+1.6436688311688311e-04*Jinv02*w18-1.2053571428571428e-03*Jinv01*w12+1.3149350649350649e-03*Jinv02*w33+3.4699675324675324e-04*w0*Jinv00+1.2601461038961039e-03*Jinv21*w23-2.1915584415584414e-03*Jinv01*w54-2.1915584415584414e-03*w55*Jinv11+1.3149350649350649e-03*w25*Jinv10+3.4699675324675324e-04*Jinv20*w2-1.0957792207792207e-03*w48*Jinv02+3.6525974025974026e-05*Jinv01*w9-7.1225649350649346e-04*w30*Jinv00-1.9724025974025973e-03*w40*Jinv10+1.3149350649350649e-03*w20*Jinv20-2.1915584415584417e-04*Jinv22*w29-1.1505681818181819e-03*Jinv22*w14+6.5746753246753243e-04*w40*Jinv11+7.6704545454545450e-04*Jinv02*w36+3.9448051948051946e-03*Jinv12*w43-8.2183441558441553e-04*w5*Jinv20-3.2873376623376621e-04*Jinv01*w30-2.1915584415584414e-03*Jinv02*w51-3.2873376623376621e-03*Jinv22*w56-2.1915584415584417e-04*Jinv12*w28+2.5750811688311690e-03*w25*Jinv11+1.9724025974025973e-03*w42*Jinv00+4.9310064935064934e-03*w16*Jinv12-2.5568181818181820e-04*Jinv01*w27+4.2004870129870131e-04*Jinv12*w1+3.6525974025974026e-05*w10*Jinv11)*D;
  A[56][18]=(0.0)*D;
  A[56][19]=(0.0)*D;
  A[56][20]=(2.4655032467532466e-04*Jinv00*w12+2.1915584415584417e-04*w28*Jinv10-3.5612824675324673e-04*Jinv20*w8+2.4655032467532466e-04*w13*Jinv10-7.6704545454545450e-04*Jinv21*w44+4.6570616883116886e-04*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20+2.7394480519480518e-04*Jinv01*w3+1.3971185064935066e-03*Jinv20*w23-2.1367694805194807e-03*Jinv00*w54+9.1314935064935065e-06*Jinv21*w59-1.0957792207792208e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-7.6704545454545450e-04*w42*Jinv01+9.1314935064935065e-06*w11*Jinv21-5.4788961038961042e-05*w16*Jinv11-5.4788961038961042e-05*Jinv21*w17+5.2049512987012989e-04*Jinv01*w45+1.8262987012987013e-05*w58*Jinv10-7.1225649350649346e-04*w48*Jinv00+9.1314935064935065e-06*w58*Jinv11+9.1314935064935065e-06*w57*Jinv01+5.2049512987012989e-04*Jinv21*w26-1.0957792207792208e-04*w0*Jinv01-2.1915584415584417e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-7.6704545454545450e-04*w43*Jinv11+1.4793019480519481e-03*w45*Jinv00-5.4788961038961042e-05*Jinv21*w41+2.1915584415584417e-04*Jinv00*w27-1.0957792207792208e-04*Jinv11*w1+8.2183441558441553e-05*w13*Jinv11-5.4788961038961042e-05*w15*Jinv01-1.3697240259740259e-04*w28*Jinv11-7.1225649350649346e-04*Jinv20*w50+2.1915584415584417e-04*Jinv20*w29+1.4793019480519481e-03*w46*Jinv10-7.1225649350649346e-04*Jinv00*w33+5.4788961038961036e-04*w31*Jinv10+1.3971185064935066e-03*w21*Jinv00-5.4788961038961042e-05*Jinv01*w39-1.0683847402597404e-03*Jinv00*w18-1.6436688311688311e-04*w36*Jinv00-1.6436688311688311e-04*Jinv21*w38+5.2049512987012989e-04*Jinv01*w24-3.8352272727272725e-04*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11-1.6436688311688311e-04*Jinv21*w53+3.8352272727272725e-04*w51*Jinv00+4.6570616883116886e-04*w4*Jinv10+2.7394480519480518e-04*w4*Jinv11-1.6436688311688311e-04*w41*Jinv20-1.0683847402597404e-03*w19*Jinv10-7.1225649350649346e-04*w34*Jinv10+2.7394480519480518e-04*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-3.5612824675324673e-04*w6*Jinv00-2.1367694805194807e-03*Jinv20*w56+8.2183441558441553e-05*w26*Jinv20+1.2784090909090910e-04*w11*Jinv20-3.8352272727272725e-04*Jinv21*w35+2.7394480519480518e-04*w31*Jinv11+5.2049512987012989e-04*w46*Jinv11+2.4655032467532466e-04*Jinv20*w14+2.7394480519480518e-04*w5*Jinv21-7.1225649350649346e-04*w49*Jinv10-1.3697240259740259e-04*Jinv21*w29-3.8352272727272725e-04*Jinv01*w33-1.6436688311688311e-04*w39*Jinv00-7.1225649350649346e-04*Jinv01*w18-1.6436688311688311e-04*w37*Jinv11-3.5612824675324673e-04*w6*Jinv01-3.5612824675324673e-04*w7*Jinv10+5.2049512987012989e-04*w47*Jinv21-1.6436688311688311e-04*Jinv20*w38-1.6436688311688311e-04*w52*Jinv11+3.8352272727272725e-04*Jinv20*w53+1.3971185064935066e-03*w22*Jinv10+2.7394480519480518e-04*w22*Jinv11+2.7394480519480518e-04*w21*Jinv01-1.6436688311688311e-04*Jinv01*w51-1.6436688311688311e-04*w36*Jinv01+3.8352272727272725e-04*w52*Jinv10-3.5612824675324673e-04*w7*Jinv11+1.2784090909090910e-04*w9*Jinv00-1.6436688311688311e-04*w37*Jinv10-7.1225649350649346e-04*Jinv21*w56+8.2183441558441553e-05*w24*Jinv00-3.5612824675324673e-04*w49*Jinv11+8.2183441558441553e-05*Jinv21*w14+1.2784090909090910e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+8.2183441558441553e-05*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.8262987012987013e-05*w57*Jinv00+1.8262987012987013e-05*Jinv20*w59-2.1915584415584417e-04*w0*Jinv00-7.1225649350649346e-04*w35*Jinv20+2.7394480519480518e-04*Jinv21*w23-7.1225649350649346e-04*Jinv01*w54-7.1225649350649346e-04*w55*Jinv11+8.2183441558441553e-05*w25*Jinv10-2.1915584415584417e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00+9.1314935064935065e-06*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00-1.6436688311688311e-04*w40*Jinv10-1.0683847402597404e-03*w20*Jinv20-5.4788961038961042e-05*w40*Jinv11+4.6570616883116886e-04*w5*Jinv20+2.7394480519480518e-04*Jinv01*w30+5.2049512987012989e-04*w25*Jinv11-2.1367694805194807e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00+1.4793019480519481e-03*w47*Jinv20-3.5612824675324673e-04*Jinv21*w50-1.3697240259740259e-04*Jinv01*w27-3.5612824675324673e-04*w48*Jinv01+9.1314935064935065e-06*w10*Jinv11)*D;
  A[56][21]=(0.0)*D;
  A[56][22]=(0.0)*D;
  A[56][23]=(-1.3971185064935066e-03*Jinv00*w12+5.4788961038961036e-04*w28*Jinv10+2.1367694805194807e-03*w40*Jinv12+1.6436688311688311e-04*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8+4.7483766233766234e-04*w57*Jinv02-1.3971185064935066e-03*w13*Jinv10+2.1367694805194807e-03*Jinv02*w39+7.6704545454545450e-04*Jinv21*w44-2.7394480519480518e-04*w3*Jinv00-5.2049512987012989e-04*w32*Jinv20-2.7394480519480521e-05*Jinv01*w3+2.3833198051948050e-03*Jinv20*w23-5.2049512987012989e-04*Jinv22*w38-1.1505681818181819e-03*Jinv00*w54-9.1314935064935065e-06*Jinv21*w59+2.3833198051948050e-03*w25*Jinv12+4.2918019480519480e-04*w0*Jinv02+2.5568181818181820e-04*Jinv21*w2+2.1367694805194807e-03*w43*Jinv10+7.6704545454545450e-04*w42*Jinv01+1.0957792207792208e-04*w11*Jinv21-1.2053571428571428e-03*w16*Jinv11-1.2053571428571428e-03*Jinv21*w17+1.3971185064935066e-03*Jinv01*w45-1.1505681818181819e-03*Jinv12*w55+4.7483766233766234e-04*w58*Jinv10+1.3149350649350649e-03*w15*Jinv02+4.2918019480519480e-04*w10*Jinv12+1.6436688311688311e-04*w48*Jinv00+2.3833198051948050e-03*Jinv02*w24+3.8352272727272725e-04*Jinv22*w35-9.1314935064935065e-06*w58*Jinv11-9.1314935064935065e-06*w57*Jinv01+1.3697240259740260e-03*Jinv21*w26+2.5568181818181820e-04*w0*Jinv01+4.2918019480519480e-04*w1*Jinv10+1.3149350649350649e-03*w16*Jinv10-1.3971185064935066e-03*Jinv22*w20+7.6704545454545450e-04*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-6.0267857142857139e-04*Jinv21*w41+2.1367694805194807e-03*w42*Jinv02+5.4788961038961036e-04*Jinv00*w27+4.2918019480519480e-04*w11*Jinv22+2.5568181818181820e-04*Jinv11*w1-9.5880681818181818e-04*w13*Jinv11-1.2053571428571428e-03*w15*Jinv01+6.6659902597402602e-04*w28*Jinv11+1.6436688311688311e-04*Jinv20*w50+5.4788961038961036e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10+1.3149350649350649e-03*Jinv22*w17+3.8352272727272725e-04*Jinv00*w33+2.3833198051948050e-03*w22*Jinv12-5.2049512987012989e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3+2.3833198051948050e-03*w21*Jinv00-5.2049512987012989e-04*w32*Jinv22-6.0267857142857139e-04*Jinv01*w39+4.2918019480519480e-04*Jinv22*w2-1.3971185064935066e-03*Jinv00*w18-5.2049512987012989e-04*w36*Jinv00-2.7394480519480518e-04*Jinv21*w38+2.6298701298701297e-03*Jinv02*w45+1.3697240259740260e-03*Jinv01*w24+4.7483766233766234e-04*Jinv22*w59-3.2873376623376621e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+3.5612824675324673e-04*Jinv21*w53+1.6436688311688311e-04*w51*Jinv00+3.8352272727272725e-04*Jinv12*w34+2.1367694805194807e-03*Jinv22*w44-2.7394480519480518e-04*w4*Jinv10-2.7394480519480521e-05*w4*Jinv11+2.1367694805194807e-03*w41*Jinv20-1.3971185064935066e-03*Jinv12*w13-1.3971185064935066e-03*w19*Jinv10+3.8352272727272725e-04*w34*Jinv10-3.8352272727272725e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00-5.2049512987012989e-04*w37*Jinv12-1.1505681818181819e-03*Jinv20*w56+2.1367694805194807e-03*Jinv22*w41+2.3833198051948050e-03*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+1.6436688311688311e-04*w52*Jinv12-3.2873376623376621e-04*Jinv21*w35-3.8352272727272725e-04*w31*Jinv11+2.3833198051948050e-03*Jinv22*w26+1.3971185064935066e-03*w46*Jinv11-1.3971185064935066e-03*Jinv20*w14-2.7394480519480521e-05*w5*Jinv21-1.1505681818181819e-03*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+2.3833198051948050e-03*w21*Jinv02+1.6436688311688311e-04*w49*Jinv10+6.6659902597402602e-04*Jinv21*w29-3.2873376623376621e-04*Jinv01*w33+2.1367694805194807e-03*w39*Jinv00-1.3971185064935066e-03*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-2.7394480519480518e-04*w37*Jinv11+4.2918019480519480e-04*Jinv02*w9+1.9176136363636362e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.3971185064935066e-03*w47*Jinv21-5.2049512987012989e-04*Jinv20*w38+2.3833198051948050e-03*Jinv22*w23+3.5612824675324673e-04*w52*Jinv11-5.2049512987012989e-04*w31*Jinv12+2.6298701298701297e-03*w46*Jinv12+1.6436688311688311e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+2.3833198051948050e-03*w22*Jinv10+1.6436688311688311e-04*Jinv22*w50+3.5612824675324673e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7+3.5612824675324673e-04*w21*Jinv01+3.5612824675324673e-04*Jinv01*w51-5.2049512987012989e-04*Jinv02*w30+1.6436688311688311e-04*Jinv12*w49-2.7394480519480518e-04*w36*Jinv01+1.6436688311688311e-04*w52*Jinv10+1.9176136363636362e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-5.2049512987012989e-04*w37*Jinv10+3.2873376623376621e-04*Jinv21*w56+2.3833198051948050e-03*w24*Jinv00+5.4788961038961036e-04*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-1.3971185064935066e-03*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5+2.6298701298701297e-03*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11-9.5880681818181818e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.1367694805194807e-03*Jinv20*w44+4.7483766233766234e-04*w58*Jinv12-1.3971185064935066e-03*Jinv02*w18-9.5880681818181818e-04*Jinv01*w12+1.9176136363636362e-04*Jinv21*w8+4.7483766233766234e-04*w57*Jinv00+4.7483766233766234e-04*Jinv20*w59+3.8352272727272725e-04*Jinv02*w33+4.2918019480519480e-04*w0*Jinv00+3.8352272727272725e-04*w35*Jinv20+3.5612824675324673e-04*Jinv21*w23+3.2873376623376621e-04*Jinv01*w54+3.2873376623376621e-04*w55*Jinv11+2.3833198051948050e-03*w25*Jinv10+4.2918019480519480e-04*Jinv20*w2+1.3149350649350649e-03*w15*Jinv00+1.6436688311688311e-04*w48*Jinv02+1.0957792207792208e-04*Jinv01*w9+1.3149350649350649e-03*Jinv20*w17-5.2049512987012989e-04*w30*Jinv00+2.1367694805194807e-03*w40*Jinv10-1.3971185064935066e-03*w20*Jinv20+5.4788961038961036e-04*Jinv22*w29-1.3971185064935066e-03*Jinv22*w14-6.0267857142857139e-04*w40*Jinv11-5.2049512987012989e-04*Jinv02*w36+2.1367694805194807e-03*Jinv12*w43-2.7394480519480518e-04*w5*Jinv20-3.8352272727272725e-04*Jinv01*w30+1.6436688311688311e-04*Jinv02*w51-1.1505681818181819e-03*Jinv22*w56+5.4788961038961036e-04*Jinv12*w28+1.3697240259740260e-03*w25*Jinv11-1.1505681818181819e-03*w55*Jinv10+2.1367694805194807e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20+1.3149350649350649e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50+6.6659902597402602e-04*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+4.2918019480519480e-04*Jinv12*w1+1.0957792207792208e-04*w10*Jinv11)*D;
  A[56][24]=(0.0)*D;
  A[56][25]=(0.0)*D;
  A[56][26]=(1.3971185064935066e-03*Jinv00*w12-5.4788961038961036e-04*w28*Jinv10+2.7394480519480518e-04*Jinv20*w8+1.3971185064935066e-03*w13*Jinv10-2.7394480519480520e-03*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00+5.2049512987012989e-04*w32*Jinv20+4.6570616883116886e-04*Jinv01*w3-2.3833198051948050e-03*Jinv20*w23+1.1505681818181819e-03*Jinv00*w54-4.8396915584415583e-04*Jinv21*w59-3.1960227272727273e-04*Jinv21*w2-2.1367694805194807e-03*w43*Jinv10-2.7394480519480520e-03*w42*Jinv01-1.7349837662337662e-04*w11*Jinv21-2.5202922077922079e-03*w16*Jinv11-2.5202922077922079e-03*Jinv21*w17-1.2327516233766234e-03*Jinv01*w45-4.7483766233766234e-04*w58*Jinv10-1.6436688311688311e-04*w48*Jinv00-4.8396915584415583e-04*w58*Jinv11-4.8396915584415583e-04*w57*Jinv01-2.0271915584415584e-03*Jinv21*w26-3.1960227272727273e-04*w0*Jinv01-4.2918019480519480e-04*w1*Jinv10-1.3149350649350649e-03*w16*Jinv10-2.7394480519480520e-03*w43*Jinv11-2.6298701298701297e-03*w45*Jinv00-1.3697240259740260e-03*Jinv21*w41-5.4788961038961036e-04*Jinv00*w27-3.1960227272727273e-04*Jinv11*w1+8.4922889610389610e-04*w13*Jinv11-2.5202922077922079e-03*w15*Jinv01+1.1870941558441559e-04*w28*Jinv11-1.6436688311688311e-04*Jinv20*w50-5.4788961038961036e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10-3.8352272727272725e-04*Jinv00*w33+5.2049512987012989e-04*w31*Jinv10-2.3833198051948050e-03*w21*Jinv00-1.3697240259740260e-03*Jinv01*w39+1.3971185064935066e-03*Jinv00*w18+5.2049512987012989e-04*w36*Jinv00+1.3697240259740259e-04*Jinv21*w38-2.0271915584415584e-03*Jinv01*w24-7.1225649350649346e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11+3.8352272727272725e-04*Jinv21*w53-1.6436688311688311e-04*w51*Jinv00+2.7394480519480518e-04*w4*Jinv10+4.6570616883116886e-04*w4*Jinv11-2.1367694805194807e-03*w41*Jinv20+1.3971185064935066e-03*w19*Jinv10-3.8352272727272725e-04*w34*Jinv10+2.4655032467532466e-04*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+2.7394480519480518e-04*w6*Jinv00+1.1505681818181819e-03*Jinv20*w56-2.3833198051948050e-03*w26*Jinv20-4.2918019480519480e-04*w11*Jinv20-7.1225649350649346e-04*Jinv21*w35+2.4655032467532466e-04*w31*Jinv11-1.2327516233766234e-03*w46*Jinv11+1.3971185064935066e-03*Jinv20*w14+4.6570616883116886e-04*w5*Jinv21-1.6436688311688311e-04*w49*Jinv10+1.1870941558441559e-04*Jinv21*w29-7.1225649350649346e-04*Jinv01*w33-2.1367694805194807e-03*w39*Jinv00+4.3831168831168834e-04*Jinv01*w18+1.3697240259740259e-04*w37*Jinv11+2.4655032467532466e-04*w6*Jinv01+2.7394480519480518e-04*w7*Jinv10-1.2327516233766234e-03*w47*Jinv21+5.2049512987012989e-04*Jinv20*w38+3.8352272727272725e-04*w52*Jinv11-1.6436688311688311e-04*Jinv20*w53-2.3833198051948050e-03*w22*Jinv10-1.0135957792207792e-03*w22*Jinv11-1.0135957792207792e-03*w21*Jinv01+3.8352272727272725e-04*Jinv01*w51+1.3697240259740259e-04*w36*Jinv01-1.6436688311688311e-04*w52*Jinv10+2.4655032467532466e-04*w7*Jinv11-4.2918019480519480e-04*w9*Jinv00+5.2049512987012989e-04*w37*Jinv10+1.4793019480519481e-03*Jinv21*w56-2.3833198051948050e-03*w24*Jinv00+1.9176136363636362e-04*w49*Jinv11+8.4922889610389610e-04*Jinv21*w14-4.2918019480519480e-04*w10*Jinv10-2.1367694805194807e-03*Jinv20*w44+8.4922889610389610e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-4.7483766233766234e-04*w57*Jinv00-4.7483766233766234e-04*Jinv20*w59-4.2918019480519480e-04*w0*Jinv00-3.8352272727272725e-04*w35*Jinv20-1.0135957792207792e-03*Jinv21*w23+1.4793019480519481e-03*Jinv01*w54+1.4793019480519481e-03*w55*Jinv11-2.3833198051948050e-03*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-1.3149350649350649e-03*w15*Jinv00-1.7349837662337662e-04*Jinv01*w9-1.3149350649350649e-03*Jinv20*w17+5.2049512987012989e-04*w30*Jinv00-2.1367694805194807e-03*w40*Jinv10+1.3971185064935066e-03*w20*Jinv20-1.3697240259740260e-03*w40*Jinv11+2.7394480519480518e-04*w5*Jinv20+2.4655032467532466e-04*Jinv01*w30-2.0271915584415584e-03*w25*Jinv11+1.1505681818181819e-03*w55*Jinv10-2.1367694805194807e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+1.9176136363636362e-04*Jinv21*w50+1.1870941558441559e-04*Jinv01*w27+1.9176136363636362e-04*w48*Jinv01-1.7349837662337662e-04*w10*Jinv11)*D;
  A[56][27]=(0.0)*D;
  A[56][28]=(0.0)*D;
  A[56][29]=(-3.2873376623376621e-04*Jinv21*w44-2.7394480519480521e-05*Jinv01*w3+8.3198051948051951e-05*Jinv21*w59-1.1668019480519480e-04*Jinv21*w2-3.2873376623376621e-04*w42*Jinv01-1.1668019480519480e-04*w11*Jinv21+2.9220779220779221e-04*w16*Jinv11+2.9220779220779221e-04*Jinv21*w17-1.0227272727272728e-03*Jinv01*w45+8.3198051948051951e-05*w58*Jinv11+8.3198051948051951e-05*w57*Jinv01-6.7573051948051951e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-3.2873376623376621e-04*w43*Jinv11-3.2873376623376621e-04*Jinv21*w41-1.1668019480519480e-04*Jinv11*w1+4.9310064935064932e-04*w13*Jinv11+2.9220779220779221e-04*w15*Jinv01-3.2467532467532468e-04*w28*Jinv11-3.2873376623376621e-04*Jinv01*w39+1.1870941558441559e-04*Jinv21*w38-6.7573051948051951e-04*Jinv01*w24+2.1915584415584417e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11+3.6525974025974026e-05*Jinv21*w53-2.7394480519480521e-05*w4*Jinv11+1.1870941558441559e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20+2.1915584415584417e-04*Jinv21*w35+1.1870941558441559e-04*w31*Jinv11-1.0227272727272728e-03*w46*Jinv11-2.7394480519480521e-05*w5*Jinv21-3.2467532467532468e-04*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33+4.9310064935064932e-04*Jinv01*w18+1.1870941558441559e-04*w37*Jinv11-2.7394480519480521e-05*w6*Jinv01-1.0227272727272728e-03*w47*Jinv21+3.6525974025974026e-05*w52*Jinv11-6.7573051948051951e-04*w22*Jinv11-6.7573051948051951e-04*w21*Jinv01+3.6525974025974026e-05*Jinv01*w51+1.1870941558441559e-04*w36*Jinv01-2.7394480519480521e-05*w7*Jinv11-2.5568181818181820e-04*Jinv21*w56+3.6525974025974026e-05*w49*Jinv11+4.9310064935064932e-04*Jinv21*w14+4.9310064935064932e-04*Jinv01*w12-2.7394480519480521e-05*Jinv21*w8-6.7573051948051951e-04*Jinv21*w23-2.5568181818181820e-04*Jinv01*w54-2.5568181818181820e-04*w55*Jinv11-1.1668019480519480e-04*Jinv01*w9-3.2873376623376621e-04*w40*Jinv11+1.1870941558441559e-04*Jinv01*w30-6.7573051948051951e-04*w25*Jinv11+3.6525974025974026e-05*Jinv21*w50-3.2467532467532468e-04*Jinv01*w27+3.6525974025974026e-05*w48*Jinv01-1.1668019480519480e-04*w10*Jinv11)*D;
  A[56][30]=(0.0)*D;
  A[56][31]=(0.0)*D;
  A[56][32]=(1.6436688311688311e-04*Jinv00*w12+3.1047077922077924e-04*w28*Jinv10+8.7662337662337668e-04*w40*Jinv12+2.0271915584415584e-03*Jinv22*w53-2.4655032467532466e-04*Jinv20*w8+5.5702110389610384e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10+8.7662337662337668e-04*Jinv02*w39+2.6298701298701297e-03*Jinv21*w44+8.2183441558441553e-05*w3*Jinv00-5.7528409090909093e-04*w32*Jinv20+8.2183441558441553e-05*Jinv01*w3-1.3697240259740259e-04*Jinv20*w23-1.1231737012987013e-03*Jinv22*w38+4.7666396103896100e-03*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59-2.4655032467532466e-04*w25*Jinv12+2.1915584415584417e-04*w0*Jinv02+2.1002435064935065e-04*Jinv21*w2+2.6298701298701297e-03*w43*Jinv10+2.6298701298701297e-03*w42*Jinv01+4.2918019480519480e-04*w11*Jinv21+7.1225649350649346e-04*w16*Jinv11+7.1225649350649346e-04*Jinv21*w17-1.4793019480519481e-03*Jinv01*w45+4.0543831168831168e-03*Jinv12*w55+5.4788961038961036e-04*w58*Jinv10+6.5746753246753243e-04*w15*Jinv02+3.1960227272727273e-04*w10*Jinv12+2.4655032467532466e-04*w48*Jinv00-2.4655032467532466e-04*Jinv02*w24+4.3831168831168834e-04*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01-5.2049512987012989e-04*Jinv21*w26+2.1002435064935065e-04*w0*Jinv01+2.1002435064935065e-04*w1*Jinv10+7.1225649350649346e-04*w16*Jinv10-4.6570616883116886e-04*Jinv22*w20+2.6298701298701297e-03*w43*Jinv11-1.4793019480519481e-03*w45*Jinv00+1.6436688311688311e-03*Jinv21*w41+2.5750811688311690e-03*w42*Jinv02+3.1047077922077924e-04*Jinv00*w27+3.1960227272727273e-04*w11*Jinv22+2.1002435064935065e-04*Jinv11*w1+1.6436688311688311e-04*w13*Jinv11+7.1225649350649346e-04*w15*Jinv01+3.1047077922077924e-04*w28*Jinv11+2.4655032467532466e-04*Jinv20*w50+3.1047077922077924e-04*Jinv20*w29-1.4793019480519481e-03*w46*Jinv10+6.5746753246753243e-04*Jinv22*w17+8.2183441558441553e-04*Jinv00*w33+3.8352272727272725e-04*w22*Jinv12-5.7528409090909093e-04*w31*Jinv10-2.7394480519480518e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-7.3965097402597403e-04*w32*Jinv22+1.6436688311688311e-03*Jinv01*w39+2.1915584415584417e-04*Jinv22*w2-5.4788961038961036e-04*Jinv00*w18-1.3971185064935066e-03*w36*Jinv00-1.3971185064935066e-03*Jinv21*w38-9.5880681818181818e-04*Jinv02*w45-5.2049512987012989e-04*Jinv01*w24+5.5702110389610384e-04*Jinv22*w59+8.2183441558441553e-04*w34*Jinv11-5.4788961038961036e-04*w19*Jinv11+2.3833198051948050e-03*Jinv21*w53+2.3833198051948050e-03*w51*Jinv00+4.3831168831168834e-04*Jinv12*w34+2.5750811688311690e-03*Jinv22*w44+8.2183441558441553e-05*w4*Jinv10+8.2183441558441553e-05*w4*Jinv11+1.6436688311688311e-03*w41*Jinv20-5.4788961038961036e-04*Jinv12*w13-5.4788961038961036e-04*w19*Jinv10+8.2183441558441553e-04*w34*Jinv10-5.7528409090909093e-04*w32*Jinv21-5.4788961038961036e-04*Jinv21*w20-2.4655032467532466e-04*w6*Jinv00-1.1231737012987013e-03*w37*Jinv12+4.7666396103896100e-03*Jinv20*w56+8.7662337662337668e-04*Jinv22*w41-5.2049512987012989e-04*w26*Jinv20+4.2918019480519480e-04*w11*Jinv20+2.0271915584415584e-03*w52*Jinv12+8.2183441558441553e-04*Jinv21*w35-5.7528409090909093e-04*w31*Jinv11-2.4655032467532466e-04*Jinv22*w26-1.4793019480519481e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14+8.2183441558441553e-05*w5*Jinv21+4.0543831168831168e-03*Jinv02*w54-2.7394480519480518e-04*w4*Jinv12+3.8352272727272725e-04*w21*Jinv02+2.4655032467532466e-04*w49*Jinv10+3.1047077922077924e-04*Jinv21*w29+8.2183441558441553e-04*Jinv01*w33+1.6436688311688311e-03*w39*Jinv00-5.4788961038961036e-04*Jinv02*w12-5.4788961038961036e-04*Jinv01*w18-1.3971185064935066e-03*w37*Jinv11+3.1960227272727273e-04*Jinv02*w9-2.4655032467532466e-04*w6*Jinv01-2.4655032467532466e-04*w7*Jinv10-1.4793019480519481e-03*w47*Jinv21-1.3971185064935066e-03*Jinv20*w38+3.8352272727272725e-04*Jinv22*w23+2.3833198051948050e-03*w52*Jinv11-7.3965097402597403e-04*w31*Jinv12-9.5880681818181818e-04*w46*Jinv12+2.3833198051948050e-03*Jinv20*w53+2.7394480519480521e-05*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+8.2183441558441553e-05*Jinv22*w50-1.3697240259740259e-04*w22*Jinv11+2.7394480519480521e-05*Jinv12*w7-1.3697240259740259e-04*w21*Jinv01+2.3833198051948050e-03*Jinv01*w51-7.3965097402597403e-04*Jinv02*w30+8.2183441558441553e-05*Jinv12*w49-1.3971185064935066e-03*w36*Jinv01+2.3833198051948050e-03*w52*Jinv10-2.4655032467532466e-04*w7*Jinv11+4.2918019480519480e-04*w9*Jinv00-1.3971185064935066e-03*w37*Jinv10+4.7666396103896100e-03*Jinv21*w56-5.2049512987012989e-04*w24*Jinv00+1.7349837662337662e-04*Jinv02*w27+2.7394480519480521e-05*w6*Jinv02-4.6570616883116886e-04*w19*Jinv12-2.7394480519480518e-04*Jinv22*w5-9.5880681818181818e-04*w47*Jinv22+2.4655032467532466e-04*w49*Jinv11+1.6436688311688311e-04*Jinv21*w14+4.2918019480519480e-04*w10*Jinv10+2.6298701298701297e-03*Jinv20*w44+5.5702110389610384e-04*w58*Jinv12-4.6570616883116886e-04*Jinv02*w18+1.6436688311688311e-04*Jinv01*w12-2.4655032467532466e-04*Jinv21*w8+5.4788961038961036e-04*w57*Jinv00+5.4788961038961036e-04*Jinv20*w59+4.3831168831168834e-04*Jinv02*w33+2.1002435064935065e-04*w0*Jinv00+8.2183441558441553e-04*w35*Jinv20-1.3697240259740259e-04*Jinv21*w23+4.7666396103896100e-03*Jinv01*w54+4.7666396103896100e-03*w55*Jinv11-5.2049512987012989e-04*w25*Jinv10+2.1002435064935065e-04*Jinv20*w2+7.1225649350649346e-04*w15*Jinv00+8.2183441558441553e-05*w48*Jinv02+4.2918019480519480e-04*Jinv01*w9+7.1225649350649346e-04*Jinv20*w17-5.7528409090909093e-04*w30*Jinv00+1.6436688311688311e-03*w40*Jinv10-5.4788961038961036e-04*w20*Jinv20+1.7349837662337662e-04*Jinv22*w29-5.4788961038961036e-04*Jinv22*w14+1.6436688311688311e-03*w40*Jinv11-1.1231737012987013e-03*Jinv02*w36+2.5750811688311690e-03*Jinv12*w43+8.2183441558441553e-05*w5*Jinv20-5.7528409090909093e-04*Jinv01*w30+2.0271915584415584e-03*Jinv02*w51+4.0543831168831168e-03*Jinv22*w56+1.7349837662337662e-04*Jinv12*w28-5.2049512987012989e-04*w25*Jinv11+4.7666396103896100e-03*w55*Jinv10+2.6298701298701297e-03*w42*Jinv00-1.4793019480519481e-03*w47*Jinv20+6.5746753246753243e-04*w16*Jinv12+2.4655032467532466e-04*Jinv21*w50+3.1047077922077924e-04*Jinv01*w27+2.4655032467532466e-04*w48*Jinv01+2.1915584415584417e-04*Jinv12*w1+4.2918019480519480e-04*w10*Jinv11)*D;
  A[56][33]=(0.0)*D;
  A[56][34]=(0.0)*D;
  A[56][35]=(-7.1225649350649346e-04*Jinv00*w12-2.3011363636363637e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+3.2873376623376621e-04*Jinv20*w8+5.4788961038961042e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10-2.3011363636363637e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44-3.2873376623376621e-04*w3*Jinv00-8.2183441558441553e-04*w32*Jinv20+5.4788961038961042e-05*Jinv01*w3-3.8352272727272725e-04*Jinv20*w23+1.7532467532467534e-03*Jinv22*w38+5.4788961038961036e-04*Jinv21*w59+7.1225649350649346e-04*w25*Jinv12-3.1047077922077924e-04*w0*Jinv02-2.7394480519480518e-04*Jinv21*w2-9.8620129870129864e-04*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-4.9310064935064932e-04*w11*Jinv21+1.3149350649350649e-03*w16*Jinv11+1.3149350649350649e-03*Jinv21*w17+2.1367694805194807e-03*Jinv01*w45-3.1777597402597403e-03*Jinv12*w55-1.9724025974025973e-03*w15*Jinv02-5.2962662337662338e-04*w10*Jinv12+2.1367694805194807e-03*w48*Jinv00+7.1225649350649346e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+7.6704545454545450e-04*Jinv21*w26-2.7394480519480518e-04*w0*Jinv01+2.1915584415584417e-04*w1*Jinv10+7.1225649350649346e-04*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11-9.8620129870129864e-04*Jinv21*w41-3.2873376623376621e-03*w42*Jinv02-5.2962662337662338e-04*w11*Jinv22-2.7394480519480518e-04*Jinv11*w1+5.4788961038961042e-05*w13*Jinv11+1.3149350649350649e-03*w15*Jinv01-4.9310064935064932e-04*w28*Jinv11+2.1367694805194807e-03*Jinv20*w50-1.9724025974025973e-03*Jinv22*w17+3.2873376623376621e-04*w22*Jinv12-8.2183441558441553e-04*w31*Jinv10-3.8352272727272725e-04*w21*Jinv00+9.3141233766233771e-04*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-3.1047077922077924e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+8.2183441558441553e-04*w36*Jinv00+2.1367694805194807e-03*Jinv21*w38+1.7532467532467534e-03*Jinv02*w45+7.6704545454545450e-04*Jinv01*w24+5.4788961038961042e-05*Jinv22*w59-9.8620129870129864e-04*w34*Jinv11+7.6704545454545450e-04*w19*Jinv11-4.2735389610389614e-03*Jinv21*w53-2.1367694805194807e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-3.2873376623376621e-03*Jinv22*w44-3.2873376623376621e-04*w4*Jinv10+5.4788961038961042e-05*w4*Jinv11+9.8620129870129864e-04*w41*Jinv20+7.1225649350649346e-04*w19*Jinv10+1.3149350649350649e-03*w32*Jinv21+7.6704545454545450e-04*Jinv21*w20+3.2873376623376621e-04*w6*Jinv00+1.7532467532467534e-03*w37*Jinv12-2.3011363636363637e-03*Jinv22*w41+3.8352272727272725e-04*w26*Jinv20-2.1915584415584417e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12-9.8620129870129864e-04*Jinv21*w35+1.3149350649350649e-03*w31*Jinv11+7.1225649350649346e-04*Jinv22*w26+2.1367694805194807e-03*w46*Jinv11-7.1225649350649346e-04*Jinv20*w14+5.4788961038961042e-05*w5*Jinv21-3.1777597402597403e-03*Jinv02*w54+3.2873376623376621e-04*w21*Jinv02+2.1367694805194807e-03*w49*Jinv10-4.9310064935064932e-04*Jinv21*w29-9.8620129870129864e-04*Jinv01*w33+9.8620129870129864e-04*w39*Jinv00+7.6704545454545450e-04*Jinv01*w18+2.1367694805194807e-03*w37*Jinv11-5.2962662337662338e-04*Jinv02*w9+3.8352272727272725e-04*w6*Jinv01+3.2873376623376621e-04*w7*Jinv10+2.1367694805194807e-03*w47*Jinv21+8.2183441558441553e-04*Jinv20*w38+3.2873376623376621e-04*Jinv22*w23-4.2735389610389614e-03*w52*Jinv11+9.3141233766233771e-04*w31*Jinv12+1.7532467532467534e-03*w46*Jinv12-2.1367694805194807e-03*Jinv20*w53+3.2873376623376621e-04*Jinv22*w8-3.8352272727272725e-04*w22*Jinv10-1.0409902597402598e-03*Jinv22*w50+3.8352272727272725e-04*w22*Jinv11+3.2873376623376621e-04*Jinv12*w7+3.8352272727272725e-04*w21*Jinv01-4.2735389610389614e-03*Jinv01*w51+9.3141233766233771e-04*Jinv02*w30-1.0409902597402598e-03*Jinv12*w49+2.1367694805194807e-03*w36*Jinv01-2.1367694805194807e-03*w52*Jinv10+3.8352272727272725e-04*w7*Jinv11-2.1915584415584417e-04*w9*Jinv00+8.2183441558441553e-04*w37*Jinv10-4.2735389610389614e-03*Jinv21*w56+3.8352272727272725e-04*w24*Jinv00-5.2962662337662338e-04*Jinv02*w27+3.2873376623376621e-04*w6*Jinv02+7.1225649350649346e-04*w19*Jinv12+1.7532467532467534e-03*w47*Jinv22-2.1367694805194807e-03*w49*Jinv11+5.4788961038961042e-05*Jinv21*w14-2.1915584415584417e-04*w10*Jinv10-9.8620129870129864e-04*Jinv20*w44+5.4788961038961042e-05*w58*Jinv12+7.1225649350649346e-04*Jinv02*w18+5.4788961038961042e-05*Jinv01*w12+3.8352272727272725e-04*Jinv21*w8-2.3011363636363637e-03*Jinv02*w33+2.1915584415584417e-04*w0*Jinv00+3.8352272727272725e-04*Jinv21*w23-4.2735389610389614e-03*Jinv01*w54-4.2735389610389614e-03*w55*Jinv11+3.8352272727272725e-04*w25*Jinv10+2.1915584415584417e-04*Jinv20*w2-1.0409902597402598e-03*w48*Jinv02-4.9310064935064932e-04*Jinv01*w9-8.2183441558441553e-04*w30*Jinv00+9.8620129870129864e-04*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-5.2962662337662338e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+1.7532467532467534e-03*Jinv02*w36-3.2873376623376621e-03*Jinv12*w43-3.2873376623376621e-04*w5*Jinv20+1.3149350649350649e-03*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-3.1777597402597403e-03*Jinv22*w56-5.2962662337662338e-04*Jinv12*w28+7.6704545454545450e-04*w25*Jinv11-9.8620129870129864e-04*w42*Jinv00-1.9724025974025973e-03*w16*Jinv12-2.1367694805194807e-03*Jinv21*w50-4.9310064935064932e-04*Jinv01*w27-2.1367694805194807e-03*w48*Jinv01-3.1047077922077924e-04*Jinv12*w1-4.9310064935064932e-04*w10*Jinv11)*D;
  A[56][36]=(0.0)*D;
  A[56][37]=(0.0)*D;
  A[56][38]=(5.4788961038961036e-04*Jinv00*w12-3.1047077922077924e-04*w28*Jinv10-5.4788961038961042e-05*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-8.2183441558441553e-05*Jinv20*w8+9.1314935064935065e-06*w57*Jinv02+5.4788961038961036e-04*w13*Jinv10-5.4788961038961042e-05*Jinv02*w39+2.4655032467532466e-04*w3*Jinv00+1.3971185064935066e-03*w32*Jinv20+5.2049512987012989e-04*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-4.7666396103896100e-03*Jinv00*w54+5.2049512987012989e-04*w25*Jinv12-1.0957792207792208e-04*w0*Jinv02-1.6436688311688311e-03*w43*Jinv10-7.1225649350649346e-04*Jinv12*w55-5.4788961038961036e-04*w58*Jinv10-5.4788961038961042e-05*w15*Jinv02+9.1314935064935065e-06*w10*Jinv12-2.3833198051948050e-03*w48*Jinv00+5.2049512987012989e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-4.2918019480519480e-04*w1*Jinv10-7.1225649350649346e-04*w16*Jinv10-7.1225649350649346e-04*Jinv22*w20+1.4793019480519481e-03*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02-3.1047077922077924e-04*Jinv00*w27+9.1314935064935065e-06*w11*Jinv22-2.3833198051948050e-03*Jinv20*w50-3.1047077922077924e-04*Jinv20*w29+1.4793019480519481e-03*w46*Jinv10-5.4788961038961042e-05*Jinv22*w17-8.2183441558441553e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+1.3971185064935066e-03*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-1.0957792207792208e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18+5.7528409090909093e-04*w36*Jinv00+5.2049512987012989e-04*Jinv02*w45+9.1314935064935065e-06*Jinv22*w59-2.4655032467532466e-04*w51*Jinv00-3.8352272727272725e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+2.4655032467532466e-04*w4*Jinv10-2.6298701298701297e-03*w41*Jinv20+8.2183441558441553e-05*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10-8.2183441558441553e-04*w34*Jinv10-8.2183441558441553e-05*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12-4.7666396103896100e-03*Jinv20*w56-5.4788961038961042e-05*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20-2.1002435064935065e-04*w11*Jinv20-1.6436688311688311e-04*w52*Jinv12+5.2049512987012989e-04*Jinv22*w26+5.4788961038961036e-04*Jinv20*w14-7.1225649350649346e-04*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-2.3833198051948050e-03*w49*Jinv10-2.6298701298701297e-03*w39*Jinv00+8.2183441558441553e-05*Jinv02*w12+9.1314935064935065e-06*Jinv02*w9-8.2183441558441553e-05*w7*Jinv10+5.7528409090909093e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12+5.2049512987012989e-04*w46*Jinv12-2.4655032467532466e-04*Jinv20*w53-3.5612824675324673e-04*Jinv22*w8+5.2049512987012989e-04*w22*Jinv10-3.5612824675324673e-04*Jinv22*w50-3.5612824675324673e-04*Jinv12*w7+2.7394480519480518e-04*Jinv02*w30-3.5612824675324673e-04*Jinv12*w49-2.4655032467532466e-04*w52*Jinv10-2.1002435064935065e-04*w9*Jinv00+5.7528409090909093e-04*w37*Jinv10+1.3697240259740259e-04*w24*Jinv00-1.3697240259740259e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-7.1225649350649346e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5+5.2049512987012989e-04*w47*Jinv22-2.1002435064935065e-04*w10*Jinv10-1.6436688311688311e-03*Jinv20*w44+9.1314935064935065e-06*w58*Jinv12-7.1225649350649346e-04*Jinv02*w18-5.4788961038961036e-04*w57*Jinv00-5.4788961038961036e-04*Jinv20*w59-3.8352272727272725e-04*Jinv02*w33-4.2918019480519480e-04*w0*Jinv00-8.2183441558441553e-04*w35*Jinv20+1.3697240259740259e-04*w25*Jinv10-4.2918019480519480e-04*Jinv20*w2-7.1225649350649346e-04*w15*Jinv00-3.5612824675324673e-04*w48*Jinv02-7.1225649350649346e-04*Jinv20*w17+1.3971185064935066e-03*w30*Jinv00-2.6298701298701297e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20-1.3697240259740259e-04*Jinv22*w29+8.2183441558441553e-05*Jinv22*w14-1.6436688311688311e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+2.4655032467532466e-04*w5*Jinv20-1.6436688311688311e-04*Jinv02*w51-7.1225649350649346e-04*Jinv22*w56-1.3697240259740259e-04*Jinv12*w28-4.7666396103896100e-03*w55*Jinv10-1.6436688311688311e-03*w42*Jinv00+1.4793019480519481e-03*w47*Jinv20-5.4788961038961042e-05*w16*Jinv12-1.0957792207792208e-04*Jinv12*w1)*D;
  A[56][39]=(0.0)*D;
  A[56][40]=(0.0)*D;
  A[56][41]=(1.6436688311688311e-04*Jinv00*w12-6.5746753246753243e-04*w28*Jinv10-6.9034090909090911e-03*w40*Jinv12-3.1777597402597403e-03*Jinv22*w53+5.4788961038961042e-05*Jinv20*w8+1.0957792207792208e-04*w57*Jinv02+1.6436688311688311e-04*w13*Jinv10-6.9034090909090911e-03*Jinv02*w39-1.9724025974025973e-03*Jinv21*w44+5.4788961038961042e-05*w3*Jinv00+2.6298701298701297e-03*w32*Jinv20-3.2873376623376621e-04*Jinv01*w3+2.1367694805194807e-03*Jinv20*w23+2.5750811688311690e-03*Jinv22*w38-1.2820616883116883e-02*Jinv00*w54+5.4788961038961036e-04*Jinv21*w59+8.2183441558441553e-04*w25*Jinv12-6.2094155844155847e-04*w0*Jinv02-5.4788961038961042e-05*Jinv21*w2-2.9586038961038961e-03*w43*Jinv10-1.9724025974025973e-03*w42*Jinv01-2.7394480519480518e-04*w11*Jinv21+6.5746753246753243e-04*w16*Jinv11+6.5746753246753243e-04*Jinv21*w17+2.7942370129870131e-03*Jinv01*w45-9.5332792207792200e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10-2.9586038961038961e-03*w15*Jinv02-9.6793831168831166e-04*w10*Jinv12-4.2735389610389614e-03*w48*Jinv00+8.2183441558441553e-04*Jinv02*w24-2.3011363636363637e-03*Jinv22*w35+5.4788961038961036e-04*w58*Jinv11+5.4788961038961036e-04*w57*Jinv01+1.7532467532467534e-03*Jinv21*w26-5.4788961038961042e-05*w0*Jinv01-5.4788961038961036e-04*w1*Jinv10+1.9724025974025973e-03*w16*Jinv10+1.3149350649350649e-03*Jinv22*w20-1.9724025974025973e-03*w43*Jinv11+4.9310064935064934e-03*w45*Jinv00-9.8620129870129864e-04*Jinv21*w41-4.9310064935064934e-03*w42*Jinv02-6.5746753246753243e-04*Jinv00*w27-9.6793831168831166e-04*w11*Jinv22-5.4788961038961042e-05*Jinv11*w1-6.0267857142857139e-04*w13*Jinv11+6.5746753246753243e-04*w15*Jinv01-1.6436688311688311e-04*w28*Jinv11-4.2735389610389614e-03*Jinv20*w50-6.5746753246753243e-04*Jinv20*w29+4.9310064935064934e-03*w46*Jinv10-2.9586038961038961e-03*Jinv22*w17-9.8620129870129864e-04*Jinv00*w33-4.9310064935064932e-04*w22*Jinv12+2.6298701298701297e-03*w31*Jinv10+2.1367694805194807e-03*w21*Jinv00+1.8628246753246754e-03*w32*Jinv22-9.8620129870129864e-04*Jinv01*w39-6.2094155844155847e-04*Jinv22*w2+1.6436688311688311e-04*Jinv00*w18+2.6298701298701297e-03*w36*Jinv00+1.3149350649350649e-03*Jinv21*w38+2.7942370129870131e-03*Jinv02*w45+1.7532467532467534e-03*Jinv01*w24+1.0957792207792208e-04*Jinv22*w59+1.0957792207792208e-04*w19*Jinv11-2.1367694805194807e-03*Jinv21*w53-4.2735389610389614e-03*w51*Jinv00-2.3011363636363637e-03*Jinv12*w34-4.9310064935064934e-03*Jinv22*w44+5.4788961038961042e-05*w4*Jinv10-3.2873376623376621e-04*w4*Jinv11-2.9586038961038961e-03*w41*Jinv20+1.6436688311688311e-04*w19*Jinv10-9.8620129870129864e-04*w34*Jinv10+4.9310064935064932e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+5.4788961038961042e-05*w6*Jinv00+2.5750811688311690e-03*w37*Jinv12-1.2820616883116883e-02*Jinv20*w56-6.9034090909090911e-03*Jinv22*w41+2.1367694805194807e-03*w26*Jinv20-5.4788961038961036e-04*w11*Jinv20-3.1777597402597403e-03*w52*Jinv12+4.9310064935064932e-04*w31*Jinv11+8.2183441558441553e-04*Jinv22*w26+2.7942370129870131e-03*w46*Jinv11+1.6436688311688311e-04*Jinv20*w14-3.2873376623376621e-04*w5*Jinv21-9.5332792207792200e-03*Jinv02*w54-4.9310064935064932e-04*w21*Jinv02-4.2735389610389614e-03*w49*Jinv10-1.6436688311688311e-04*Jinv21*w29-2.9586038961038961e-03*w39*Jinv00+1.0957792207792208e-04*Jinv01*w18+1.3149350649350649e-03*w37*Jinv11-9.6793831168831166e-04*Jinv02*w9+5.4788961038961042e-05*w7*Jinv10+2.7942370129870131e-03*w47*Jinv21+2.6298701298701297e-03*Jinv20*w38-4.9310064935064932e-04*Jinv22*w23-2.1367694805194807e-03*w52*Jinv11+1.8628246753246754e-03*w31*Jinv12+2.7942370129870131e-03*w46*Jinv12-4.2735389610389614e-03*Jinv20*w53+8.2183441558441553e-04*Jinv22*w8+2.1367694805194807e-03*w22*Jinv10-2.0819805194805196e-03*Jinv22*w50+1.3697240259740260e-03*w22*Jinv11+8.2183441558441553e-04*Jinv12*w7+1.3697240259740260e-03*w21*Jinv01-2.1367694805194807e-03*Jinv01*w51+1.8628246753246754e-03*Jinv02*w30-2.0819805194805196e-03*Jinv12*w49+1.3149350649350649e-03*w36*Jinv01-4.2735389610389614e-03*w52*Jinv10-5.4788961038961036e-04*w9*Jinv00+2.6298701298701297e-03*w37*Jinv10-8.5470779220779228e-03*Jinv21*w56+2.1367694805194807e-03*w24*Jinv00-4.3831168831168834e-04*Jinv02*w27+8.2183441558441553e-04*w6*Jinv02+1.3149350649350649e-03*w19*Jinv12+2.7942370129870131e-03*w47*Jinv22-6.0267857142857139e-04*Jinv21*w14-5.4788961038961036e-04*w10*Jinv10-2.9586038961038961e-03*Jinv20*w44+1.0957792207792208e-04*w58*Jinv12+1.3149350649350649e-03*Jinv02*w18-6.0267857142857139e-04*Jinv01*w12+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59-2.3011363636363637e-03*Jinv02*w33-5.4788961038961036e-04*w0*Jinv00-9.8620129870129864e-04*w35*Jinv20+1.3697240259740260e-03*Jinv21*w23-8.5470779220779228e-03*Jinv01*w54-8.5470779220779228e-03*w55*Jinv11+2.1367694805194807e-03*w25*Jinv10-5.4788961038961036e-04*Jinv20*w2+1.9724025974025973e-03*w15*Jinv00-2.0819805194805196e-03*w48*Jinv02-2.7394480519480518e-04*Jinv01*w9+1.9724025974025973e-03*Jinv20*w17+2.6298701298701297e-03*w30*Jinv00-2.9586038961038961e-03*w40*Jinv10+1.6436688311688311e-04*w20*Jinv20-4.3831168831168834e-04*Jinv22*w29-9.8620129870129864e-04*w40*Jinv11+2.5750811688311690e-03*Jinv02*w36-4.9310064935064934e-03*Jinv12*w43+5.4788961038961042e-05*w5*Jinv20+4.9310064935064932e-04*Jinv01*w30-3.1777597402597403e-03*Jinv02*w51-9.5332792207792200e-03*Jinv22*w56-4.3831168831168834e-04*Jinv12*w28+1.7532467532467534e-03*w25*Jinv11-1.2820616883116883e-02*w55*Jinv10-2.9586038961038961e-03*w42*Jinv00+4.9310064935064934e-03*w47*Jinv20-2.9586038961038961e-03*w16*Jinv12-1.6436688311688311e-04*Jinv01*w27-6.2094155844155847e-04*Jinv12*w1-2.7394480519480518e-04*w10*Jinv11)*D;
  A[56][42]=(0.0)*D;
  A[56][43]=(0.0)*D;
  A[56][44]=(-1.6436688311688311e-04*Jinv00*w12+6.5746753246753243e-04*w28*Jinv10-1.9724025974025973e-03*w40*Jinv12+2.1915584415584414e-03*Jinv22*w53-5.4788961038961042e-05*Jinv20*w8-9.8620129870129864e-04*w57*Jinv02-1.6436688311688311e-04*w13*Jinv10-1.9724025974025973e-03*Jinv02*w39+1.9724025974025973e-03*Jinv21*w44-5.4788961038961042e-05*w3*Jinv00-2.6298701298701297e-03*w32*Jinv20-5.4788961038961042e-05*Jinv01*w3-2.1367694805194807e-03*Jinv20*w23-7.6704545454545450e-04*Jinv22*w38+1.2820616883116883e-02*Jinv00*w54-5.4788961038961036e-04*Jinv21*w59-2.6298701298701297e-03*w25*Jinv12-4.2004870129870131e-04*w0*Jinv02+2.7394480519480518e-04*Jinv21*w2+2.9586038961038961e-03*w43*Jinv10+1.9724025974025973e-03*w42*Jinv01+4.9310064935064932e-04*w11*Jinv21-1.3149350649350649e-03*w16*Jinv11-1.3149350649350649e-03*Jinv21*w17-2.1367694805194807e-03*Jinv01*w45+3.2873376623376621e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-4.9310064935064934e-03*w15*Jinv02-7.3051948051948052e-05*w10*Jinv12+4.2735389610389614e-03*w48*Jinv00-2.6298701298701297e-03*Jinv02*w24-1.3149350649350649e-03*Jinv22*w35-5.4788961038961036e-04*w58*Jinv11-5.4788961038961036e-04*w57*Jinv01-7.6704545454545450e-04*Jinv21*w26+2.7394480519480518e-04*w0*Jinv01+5.4788961038961036e-04*w1*Jinv10-1.9724025974025973e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20+1.9724025974025973e-03*w43*Jinv11-4.9310064935064934e-03*w45*Jinv00+9.8620129870129864e-04*Jinv21*w41-3.9448051948051946e-03*w42*Jinv02+6.5746753246753243e-04*Jinv00*w27-7.3051948051948052e-05*w11*Jinv22+2.7394480519480518e-04*Jinv11*w1-5.4788961038961042e-05*w13*Jinv11-1.3149350649350649e-03*w15*Jinv01+4.9310064935064932e-04*w28*Jinv11+4.2735389610389614e-03*Jinv20*w50+6.5746753246753243e-04*Jinv20*w29-4.9310064935064934e-03*w46*Jinv10-4.9310064935064934e-03*Jinv22*w17+9.8620129870129864e-04*Jinv00*w33-1.3149350649350649e-03*w22*Jinv12-2.6298701298701297e-03*w31*Jinv10+7.6704545454545450e-04*Jinv02*w3-2.1367694805194807e-03*w21*Jinv00-5.4788961038961042e-05*w32*Jinv22+9.8620129870129864e-04*Jinv01*w39-4.2004870129870131e-04*Jinv22*w2-1.6436688311688311e-04*Jinv00*w18-2.6298701298701297e-03*w36*Jinv00-2.1367694805194807e-03*Jinv21*w38-2.1367694805194807e-03*Jinv02*w45-7.6704545454545450e-04*Jinv01*w24-9.8620129870129864e-04*Jinv22*w59+9.8620129870129864e-04*w34*Jinv11-7.6704545454545450e-04*w19*Jinv11+4.2735389610389614e-03*Jinv21*w53+4.2735389610389614e-03*w51*Jinv00-1.3149350649350649e-03*Jinv12*w34-3.9448051948051946e-03*Jinv22*w44-5.4788961038961042e-05*w4*Jinv10-5.4788961038961042e-05*w4*Jinv11+2.9586038961038961e-03*w41*Jinv20+1.1505681818181819e-03*Jinv12*w13-1.6436688311688311e-04*w19*Jinv10+9.8620129870129864e-04*w34*Jinv10-1.3149350649350649e-03*w32*Jinv21-7.6704545454545450e-04*Jinv21*w20-5.4788961038961042e-05*w6*Jinv00-7.6704545454545450e-04*w37*Jinv12+1.2820616883116883e-02*Jinv20*w56-1.9724025974025973e-03*Jinv22*w41-2.1367694805194807e-03*w26*Jinv20+5.4788961038961036e-04*w11*Jinv20+2.1915584415584414e-03*w52*Jinv12+9.8620129870129864e-04*Jinv21*w35-1.3149350649350649e-03*w31*Jinv11-2.6298701298701297e-03*Jinv22*w26-2.1367694805194807e-03*w46*Jinv11-1.6436688311688311e-04*Jinv20*w14-5.4788961038961042e-05*w5*Jinv21+3.2873376623376621e-03*Jinv02*w54+7.6704545454545450e-04*w4*Jinv12-1.3149350649350649e-03*w21*Jinv02+4.2735389610389614e-03*w49*Jinv10+4.9310064935064932e-04*Jinv21*w29+9.8620129870129864e-04*Jinv01*w33+2.9586038961038961e-03*w39*Jinv00+1.1505681818181819e-03*Jinv02*w12-7.6704545454545450e-04*Jinv01*w18-2.1367694805194807e-03*w37*Jinv11-7.3051948051948052e-05*Jinv02*w9-3.8352272727272725e-04*w6*Jinv01-5.4788961038961042e-05*w7*Jinv10-2.1367694805194807e-03*w47*Jinv21-2.6298701298701297e-03*Jinv20*w38-1.3149350649350649e-03*Jinv22*w23+4.2735389610389614e-03*w52*Jinv11-5.4788961038961042e-05*w31*Jinv12-2.1367694805194807e-03*w46*Jinv12+4.2735389610389614e-03*Jinv20*w53-5.4788961038961042e-05*Jinv22*w8-2.1367694805194807e-03*w22*Jinv10+1.0957792207792207e-03*Jinv22*w50-3.8352272727272725e-04*w22*Jinv11-5.4788961038961042e-05*Jinv12*w7-3.8352272727272725e-04*w21*Jinv01+4.2735389610389614e-03*Jinv01*w51-5.4788961038961042e-05*Jinv02*w30+1.0957792207792207e-03*Jinv12*w49-2.1367694805194807e-03*w36*Jinv01+4.2735389610389614e-03*w52*Jinv10-3.8352272727272725e-04*w7*Jinv11+5.4788961038961036e-04*w9*Jinv00-2.6298701298701297e-03*w37*Jinv10+4.2735389610389614e-03*Jinv21*w56-2.1367694805194807e-03*w24*Jinv00+2.1915584415584417e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12+7.6704545454545450e-04*Jinv22*w5-2.1367694805194807e-03*w47*Jinv22+2.1367694805194807e-03*w49*Jinv11-5.4788961038961042e-05*Jinv21*w14+5.4788961038961036e-04*w10*Jinv10+2.9586038961038961e-03*Jinv20*w44-9.8620129870129864e-04*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18-5.4788961038961042e-05*Jinv01*w12-3.8352272727272725e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-1.3149350649350649e-03*Jinv02*w33+5.4788961038961036e-04*w0*Jinv00+9.8620129870129864e-04*w35*Jinv20-3.8352272727272725e-04*Jinv21*w23+4.2735389610389614e-03*Jinv01*w54+4.2735389610389614e-03*w55*Jinv11-2.1367694805194807e-03*w25*Jinv10+5.4788961038961036e-04*Jinv20*w2-1.9724025974025973e-03*w15*Jinv00+1.0957792207792207e-03*w48*Jinv02+4.9310064935064932e-04*Jinv01*w9-1.9724025974025973e-03*Jinv20*w17-2.6298701298701297e-03*w30*Jinv00+2.9586038961038961e-03*w40*Jinv10-1.6436688311688311e-04*w20*Jinv20+2.1915584415584417e-04*Jinv22*w29+1.1505681818181819e-03*Jinv22*w14+9.8620129870129864e-04*w40*Jinv11-7.6704545454545450e-04*Jinv02*w36-3.9448051948051946e-03*Jinv12*w43-5.4788961038961042e-05*w5*Jinv20-1.3149350649350649e-03*Jinv01*w30+2.1915584415584414e-03*Jinv02*w51+3.2873376623376621e-03*Jinv22*w56+2.1915584415584417e-04*Jinv12*w28-7.6704545454545450e-04*w25*Jinv11+1.2820616883116883e-02*w55*Jinv10+2.9586038961038961e-03*w42*Jinv00-4.9310064935064934e-03*w47*Jinv20-4.9310064935064934e-03*w16*Jinv12+2.1367694805194807e-03*Jinv21*w50+4.9310064935064932e-04*Jinv01*w27+2.1367694805194807e-03*w48*Jinv01-4.2004870129870131e-04*Jinv12*w1+4.9310064935064932e-04*w10*Jinv11)*D;
  A[56][45]=(0.0)*D;
  A[56][46]=(0.0)*D;
  A[56][47]=(-2.1367694805194807e-03*w40*Jinv12-1.6436688311688311e-04*Jinv22*w53-4.7483766233766234e-04*w57*Jinv02-2.1367694805194807e-03*Jinv02*w39-6.5746753246753243e-04*Jinv21*w44+2.4655032467532466e-04*Jinv01*w3+5.2049512987012989e-04*Jinv22*w38-1.0957792207792207e-03*Jinv21*w59-2.3833198051948050e-03*w25*Jinv12-4.2918019480519480e-04*w0*Jinv02-1.5523538961038962e-04*Jinv21*w2-6.5746753246753243e-04*w42*Jinv01-1.5523538961038962e-04*w11*Jinv21-1.3697240259740260e-03*w16*Jinv11-1.3697240259740260e-03*Jinv21*w17-1.3149350649350649e-03*Jinv01*w45+1.1505681818181819e-03*Jinv12*w55-1.3149350649350649e-03*w15*Jinv02-4.2918019480519480e-04*w10*Jinv12-2.3833198051948050e-03*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01-1.2327516233766234e-03*Jinv21*w26-1.5523538961038962e-04*w0*Jinv01+1.3971185064935066e-03*Jinv22*w20-6.5746753246753243e-04*w43*Jinv11-6.5746753246753243e-04*Jinv21*w41-2.1367694805194807e-03*w42*Jinv02-4.2918019480519480e-04*w11*Jinv22-1.5523538961038962e-04*Jinv11*w1+4.3831168831168834e-04*w13*Jinv11-1.3697240259740260e-03*w15*Jinv01-1.4610389610389610e-04*w28*Jinv11-1.3149350649350649e-03*Jinv22*w17-2.3833198051948050e-03*w22*Jinv12+2.7394480519480518e-04*Jinv02*w3+5.2049512987012989e-04*w32*Jinv22-6.5746753246753243e-04*Jinv01*w39-4.2918019480519480e-04*Jinv22*w2+8.2183441558441553e-05*Jinv21*w38-2.6298701298701297e-03*Jinv02*w45-1.2327516233766234e-03*Jinv01*w24-4.7483766233766234e-04*Jinv22*w59-8.2183441558441553e-04*w34*Jinv11+4.3831168831168834e-04*w19*Jinv11-2.4655032467532466e-04*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-2.1367694805194807e-03*Jinv22*w44+2.4655032467532466e-04*w4*Jinv11+1.3971185064935066e-03*Jinv12*w13+8.2183441558441553e-05*w32*Jinv21+4.3831168831168834e-04*Jinv21*w20+5.2049512987012989e-04*w37*Jinv12-2.1367694805194807e-03*Jinv22*w41-1.6436688311688311e-04*w52*Jinv12-8.2183441558441553e-04*Jinv21*w35+8.2183441558441553e-05*w31*Jinv11-2.3833198051948050e-03*Jinv22*w26-1.3149350649350649e-03*w46*Jinv11+2.4655032467532466e-04*w5*Jinv21+1.1505681818181819e-03*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-2.3833198051948050e-03*w21*Jinv02-1.4610389610389610e-04*Jinv21*w29-8.2183441558441553e-04*Jinv01*w33+1.3971185064935066e-03*Jinv02*w12+4.3831168831168834e-04*Jinv01*w18+8.2183441558441553e-05*w37*Jinv11-4.2918019480519480e-04*Jinv02*w9+2.4655032467532466e-04*w6*Jinv01-1.3149350649350649e-03*w47*Jinv21-2.3833198051948050e-03*Jinv22*w23-2.4655032467532466e-04*w52*Jinv11+5.2049512987012989e-04*w31*Jinv12-2.6298701298701297e-03*w46*Jinv12+2.7394480519480518e-04*Jinv22*w8-1.6436688311688311e-04*Jinv22*w50-1.2327516233766234e-03*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-1.2327516233766234e-03*w21*Jinv01-2.4655032467532466e-04*Jinv01*w51+5.2049512987012989e-04*Jinv02*w30-1.6436688311688311e-04*Jinv12*w49+8.2183441558441553e-05*w36*Jinv01+2.4655032467532466e-04*w7*Jinv11+3.7804383116883116e-03*Jinv21*w56-5.4788961038961036e-04*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+1.3971185064935066e-03*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5-2.6298701298701297e-03*w47*Jinv22-2.4655032467532466e-04*w49*Jinv11+4.3831168831168834e-04*Jinv21*w14-4.7483766233766234e-04*w58*Jinv12+1.3971185064935066e-03*Jinv02*w18+4.3831168831168834e-04*Jinv01*w12+2.4655032467532466e-04*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33-1.2327516233766234e-03*Jinv21*w23+3.7804383116883116e-03*Jinv01*w54+3.7804383116883116e-03*w55*Jinv11-1.6436688311688311e-04*w48*Jinv02-1.5523538961038962e-04*Jinv01*w9-5.4788961038961036e-04*Jinv22*w29+1.3971185064935066e-03*Jinv22*w14-6.5746753246753243e-04*w40*Jinv11+5.2049512987012989e-04*Jinv02*w36-2.1367694805194807e-03*Jinv12*w43+8.2183441558441553e-05*Jinv01*w30-1.6436688311688311e-04*Jinv02*w51+1.1505681818181819e-03*Jinv22*w56-5.4788961038961036e-04*Jinv12*w28-1.2327516233766234e-03*w25*Jinv11-1.3149350649350649e-03*w16*Jinv12-2.4655032467532466e-04*Jinv21*w50-1.4610389610389610e-04*Jinv01*w27-2.4655032467532466e-04*w48*Jinv01-4.2918019480519480e-04*Jinv12*w1-1.5523538961038962e-04*w10*Jinv11)*D;
  A[56][48]=(0.0)*D;
  A[56][49]=(0.0)*D;
  A[56][50]=(7.1225649350649346e-04*Jinv00*w12-7.8530844155844158e-04*w28*Jinv10-1.0957792207792207e-03*w40*Jinv12-2.4655032467532467e-03*Jinv22*w53+3.5612824675324673e-04*Jinv20*w8-1.1414366883116883e-03*w57*Jinv02+7.1225649350649346e-04*w13*Jinv10-1.0957792207792207e-03*Jinv02*w39-4.2735389610389614e-03*Jinv21*w44+3.5612824675324673e-04*w3*Jinv00+2.3833198051948050e-03*w32*Jinv20+3.5612824675324673e-04*Jinv01*w3+1.6436688311688311e-04*Jinv20*w23+1.3697240259740260e-03*Jinv22*w38-7.3965097402597406e-03*Jinv00*w54-1.0957792207792207e-03*Jinv21*w59+8.2183441558441553e-05*w25*Jinv12-3.5612824675324673e-04*w0*Jinv02-7.4878246753246752e-04*Jinv21*w2-4.2735389610389614e-03*w43*Jinv10-4.2735389610389614e-03*w42*Jinv01-7.4878246753246752e-04*w11*Jinv21-1.0957792207792207e-03*w16*Jinv11-1.0957792207792207e-03*Jinv21*w17+2.6298701298701297e-03*Jinv01*w45-4.9310064935064934e-03*Jinv12*w55-1.0957792207792207e-03*w58*Jinv10-3.7439123376623376e-04*w10*Jinv12-3.6982548701298703e-03*w48*Jinv00+8.2183441558441553e-05*Jinv02*w24-5.4788961038961036e-04*Jinv22*w35-1.0957792207792207e-03*w58*Jinv11-1.0957792207792207e-03*w57*Jinv01+1.6436688311688311e-04*Jinv21*w26-7.4878246753246752e-04*w0*Jinv01-7.4878246753246752e-04*w1*Jinv10-1.0957792207792207e-03*w16*Jinv10+3.5612824675324673e-04*Jinv22*w20-4.2735389610389614e-03*w43*Jinv11+2.6298701298701297e-03*w45*Jinv00-4.2735389610389614e-03*Jinv21*w41-2.6846590909090908e-03*w42*Jinv02-7.8530844155844158e-04*Jinv00*w27-3.7439123376623376e-04*w11*Jinv22-7.4878246753246752e-04*Jinv11*w1+7.1225649350649346e-04*w13*Jinv11-1.0957792207792207e-03*w15*Jinv01-7.8530844155844158e-04*w28*Jinv11-3.6982548701298703e-03*Jinv20*w50-7.8530844155844158e-04*Jinv20*w29+2.6298701298701297e-03*w46*Jinv10-2.1367694805194807e-03*Jinv00*w33+2.3833198051948050e-03*w31*Jinv10+2.7394480519480518e-04*Jinv02*w3+1.6436688311688311e-04*w21*Jinv00+1.1505681818181819e-03*w32*Jinv22-4.2735389610389614e-03*Jinv01*w39-3.5612824675324673e-04*Jinv22*w2+7.1225649350649346e-04*Jinv00*w18+2.3833198051948050e-03*w36*Jinv00+2.3833198051948050e-03*Jinv21*w38+1.3971185064935066e-03*Jinv02*w45+1.6436688311688311e-04*Jinv01*w24-1.1414366883116883e-03*Jinv22*w59-2.1367694805194807e-03*w34*Jinv11+7.1225649350649346e-04*w19*Jinv11-3.6982548701298703e-03*Jinv21*w53-3.6982548701298703e-03*w51*Jinv00-5.4788961038961036e-04*Jinv12*w34-2.6846590909090908e-03*Jinv22*w44+3.5612824675324673e-04*w4*Jinv10+3.5612824675324673e-04*w4*Jinv11-4.2735389610389614e-03*w41*Jinv20+5.4788961038961036e-04*Jinv12*w13+7.1225649350649346e-04*w19*Jinv10-2.1367694805194807e-03*w34*Jinv10+2.3833198051948050e-03*w32*Jinv21+7.1225649350649346e-04*Jinv21*w20+3.5612824675324673e-04*w6*Jinv00+1.3697240259740260e-03*w37*Jinv12-7.3965097402597406e-03*Jinv20*w56-1.0957792207792207e-03*Jinv22*w41+1.6436688311688311e-04*w26*Jinv20-7.4878246753246752e-04*w11*Jinv20-2.4655032467532467e-03*w52*Jinv12-2.1367694805194807e-03*Jinv21*w35+2.3833198051948050e-03*w31*Jinv11+8.2183441558441553e-05*Jinv22*w26+2.6298701298701297e-03*w46*Jinv11+7.1225649350649346e-04*Jinv20*w14+3.5612824675324673e-04*w5*Jinv21-4.9310064935064934e-03*Jinv02*w54+2.7394480519480518e-04*w4*Jinv12-3.6982548701298703e-03*w49*Jinv10-7.8530844155844158e-04*Jinv21*w29-2.1367694805194807e-03*Jinv01*w33-4.2735389610389614e-03*w39*Jinv00+5.4788961038961036e-04*Jinv02*w12+7.1225649350649346e-04*Jinv01*w18+2.3833198051948050e-03*w37*Jinv11-3.7439123376623376e-04*Jinv02*w9+3.5612824675324673e-04*w6*Jinv01+3.5612824675324673e-04*w7*Jinv10+2.6298701298701297e-03*w47*Jinv21+2.3833198051948050e-03*Jinv20*w38-3.6982548701298703e-03*w52*Jinv11+1.1505681818181819e-03*w31*Jinv12+1.3971185064935066e-03*w46*Jinv12-3.6982548701298703e-03*Jinv20*w53+1.6436688311688311e-04*w22*Jinv10-1.2327516233766234e-03*Jinv22*w50+1.6436688311688311e-04*w22*Jinv11+1.6436688311688311e-04*w21*Jinv01-3.6982548701298703e-03*Jinv01*w51+1.1505681818181819e-03*Jinv02*w30-1.2327516233766234e-03*Jinv12*w49+2.3833198051948050e-03*w36*Jinv01-3.6982548701298703e-03*w52*Jinv10+3.5612824675324673e-04*w7*Jinv11-7.4878246753246752e-04*w9*Jinv00+2.3833198051948050e-03*w37*Jinv10-7.3965097402597406e-03*Jinv21*w56+1.6436688311688311e-04*w24*Jinv00-3.9265422077922079e-04*Jinv02*w27+3.5612824675324673e-04*w19*Jinv12+2.7394480519480518e-04*Jinv22*w5+1.3971185064935066e-03*w47*Jinv22-3.6982548701298703e-03*w49*Jinv11+7.1225649350649346e-04*Jinv21*w14-7.4878246753246752e-04*w10*Jinv10-4.2735389610389614e-03*Jinv20*w44-1.1414366883116883e-03*w58*Jinv12+3.5612824675324673e-04*Jinv02*w18+7.1225649350649346e-04*Jinv01*w12+3.5612824675324673e-04*Jinv21*w8-1.0957792207792207e-03*w57*Jinv00-1.0957792207792207e-03*Jinv20*w59-5.4788961038961036e-04*Jinv02*w33-7.4878246753246752e-04*w0*Jinv00-2.1367694805194807e-03*w35*Jinv20+1.6436688311688311e-04*Jinv21*w23-7.3965097402597406e-03*Jinv01*w54-7.3965097402597406e-03*w55*Jinv11+1.6436688311688311e-04*w25*Jinv10-7.4878246753246752e-04*Jinv20*w2-1.0957792207792207e-03*w15*Jinv00-1.2327516233766234e-03*w48*Jinv02-7.4878246753246752e-04*Jinv01*w9-1.0957792207792207e-03*Jinv20*w17+2.3833198051948050e-03*w30*Jinv00-4.2735389610389614e-03*w40*Jinv10+7.1225649350649346e-04*w20*Jinv20-3.9265422077922079e-04*Jinv22*w29+5.4788961038961036e-04*Jinv22*w14-4.2735389610389614e-03*w40*Jinv11+1.3697240259740260e-03*Jinv02*w36-2.6846590909090908e-03*Jinv12*w43+3.5612824675324673e-04*w5*Jinv20+2.3833198051948050e-03*Jinv01*w30-2.4655032467532467e-03*Jinv02*w51-4.9310064935064934e-03*Jinv22*w56-3.9265422077922079e-04*Jinv12*w28+1.6436688311688311e-04*w25*Jinv11-7.3965097402597406e-03*w55*Jinv10-4.2735389610389614e-03*w42*Jinv00+2.6298701298701297e-03*w47*Jinv20-3.6982548701298703e-03*Jinv21*w50-7.8530844155844158e-04*Jinv01*w27-3.6982548701298703e-03*w48*Jinv01-3.5612824675324673e-04*Jinv12*w1-7.4878246753246752e-04*w10*Jinv11)*D;
  A[56][51]=(0.0)*D;
  A[56][52]=(0.0)*D;
  A[56][53]=(-7.1225649350649346e-04*Jinv00*w12+7.8530844155844158e-04*w28*Jinv10+1.5888798701298701e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-3.5612824675324673e-04*Jinv20*w8-4.5657467532467534e-05*w57*Jinv02-7.1225649350649346e-04*w13*Jinv10+1.5888798701298701e-03*Jinv02*w39-3.5612824675324673e-04*w3*Jinv00-2.3833198051948050e-03*w32*Jinv20-1.6436688311688311e-04*Jinv20*w23-1.2327516233766234e-03*Jinv22*w38+7.3965097402597406e-03*Jinv00*w54-1.6436688311688311e-04*w25*Jinv12+3.7439123376623376e-04*w0*Jinv02+4.2735389610389614e-03*w43*Jinv10+2.4655032467532467e-03*Jinv12*w55+1.0957792207792207e-03*w58*Jinv10+1.0957792207792207e-03*w15*Jinv02+3.9265422077922079e-04*w10*Jinv12+3.6982548701298703e-03*w48*Jinv00-1.6436688311688311e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+7.4878246753246752e-04*w1*Jinv10+1.0957792207792207e-03*w16*Jinv10-1.6436688311688311e-04*Jinv22*w20-2.6298701298701297e-03*w45*Jinv00+3.1777597402597403e-03*w42*Jinv02+7.8530844155844158e-04*Jinv00*w27+3.9265422077922079e-04*w11*Jinv22+3.6982548701298703e-03*Jinv20*w50+7.8530844155844158e-04*Jinv20*w29-2.6298701298701297e-03*w46*Jinv10+1.0957792207792207e-03*Jinv22*w17+2.1367694805194807e-03*Jinv00*w33-8.2183441558441553e-05*w22*Jinv12-2.3833198051948050e-03*w31*Jinv10-3.5612824675324673e-04*Jinv02*w3-1.6436688311688311e-04*w21*Jinv00-1.0135957792207792e-03*w32*Jinv22+3.7439123376623376e-04*Jinv22*w2-7.1225649350649346e-04*Jinv00*w18-2.3833198051948050e-03*w36*Jinv00-1.2327516233766234e-03*Jinv02*w45-4.5657467532467534e-05*Jinv22*w59+3.6982548701298703e-03*w51*Jinv00+1.5888798701298701e-03*Jinv12*w34+3.1777597402597403e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv10+4.2735389610389614e-03*w41*Jinv20-3.5612824675324673e-04*Jinv12*w13-7.1225649350649346e-04*w19*Jinv10+2.1367694805194807e-03*w34*Jinv10-3.5612824675324673e-04*w6*Jinv00-1.2327516233766234e-03*w37*Jinv12+7.3965097402597406e-03*Jinv20*w56+1.5888798701298701e-03*Jinv22*w41-1.6436688311688311e-04*w26*Jinv20+7.4878246753246752e-04*w11*Jinv20+2.4655032467532467e-03*w52*Jinv12-1.6436688311688311e-04*Jinv22*w26-7.1225649350649346e-04*Jinv20*w14+2.4655032467532467e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12-8.2183441558441553e-05*w21*Jinv02+3.6982548701298703e-03*w49*Jinv10+4.2735389610389614e-03*w39*Jinv00-3.5612824675324673e-04*Jinv02*w12+3.9265422077922079e-04*Jinv02*w9-3.5612824675324673e-04*w7*Jinv10-2.3833198051948050e-03*Jinv20*w38-8.2183441558441553e-05*Jinv22*w23-1.0135957792207792e-03*w31*Jinv12-1.2327516233766234e-03*w46*Jinv12+3.6982548701298703e-03*Jinv20*w53-8.2183441558441553e-05*Jinv22*w8-1.6436688311688311e-04*w22*Jinv10+1.2327516233766234e-03*Jinv22*w50-8.2183441558441553e-05*Jinv12*w7-1.0135957792207792e-03*Jinv02*w30+1.2327516233766234e-03*Jinv12*w49+3.6982548701298703e-03*w52*Jinv10+7.4878246753246752e-04*w9*Jinv00-2.3833198051948050e-03*w37*Jinv10-1.6436688311688311e-04*w24*Jinv00+3.9265422077922079e-04*Jinv02*w27-8.2183441558441553e-05*w6*Jinv02-1.6436688311688311e-04*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.2327516233766234e-03*w47*Jinv22+7.4878246753246752e-04*w10*Jinv10+4.2735389610389614e-03*Jinv20*w44-4.5657467532467534e-05*w58*Jinv12-1.6436688311688311e-04*Jinv02*w18+1.0957792207792207e-03*w57*Jinv00+1.0957792207792207e-03*Jinv20*w59+1.5888798701298701e-03*Jinv02*w33+7.4878246753246752e-04*w0*Jinv00+2.1367694805194807e-03*w35*Jinv20-1.6436688311688311e-04*w25*Jinv10+7.4878246753246752e-04*Jinv20*w2+1.0957792207792207e-03*w15*Jinv00+1.2327516233766234e-03*w48*Jinv02+1.0957792207792207e-03*Jinv20*w17-2.3833198051948050e-03*w30*Jinv00+4.2735389610389614e-03*w40*Jinv10-7.1225649350649346e-04*w20*Jinv20+3.9265422077922079e-04*Jinv22*w29-3.5612824675324673e-04*Jinv22*w14-1.2327516233766234e-03*Jinv02*w36+3.1777597402597403e-03*Jinv12*w43-3.5612824675324673e-04*w5*Jinv20+2.4655032467532467e-03*Jinv02*w51+2.4655032467532467e-03*Jinv22*w56+3.9265422077922079e-04*Jinv12*w28+7.3965097402597406e-03*w55*Jinv10+4.2735389610389614e-03*w42*Jinv00-2.6298701298701297e-03*w47*Jinv20+1.0957792207792207e-03*w16*Jinv12+3.7439123376623376e-04*Jinv12*w1)*D;
  A[56][54]=(0.0)*D;
  A[56][55]=(0.0)*D;
  A[56][56]=(4.7666396103896100e-03*w40*Jinv12+2.4655032467532467e-03*Jinv22*w53-9.1314935064935068e-05*w57*Jinv02+4.7666396103896100e-03*Jinv02*w39+4.2735389610389614e-03*Jinv21*w44-3.5612824675324673e-04*Jinv01*w3-2.0271915584415584e-03*Jinv22*w38+1.0957792207792207e-03*Jinv21*w59+5.7528409090909093e-04*w25*Jinv12+7.4878246753246752e-04*w0*Jinv02+7.4878246753246752e-04*Jinv21*w2+4.2735389610389614e-03*w42*Jinv01+7.4878246753246752e-04*w11*Jinv21+1.0957792207792207e-03*w16*Jinv11+1.0957792207792207e-03*Jinv21*w17-2.6298701298701297e-03*Jinv01*w45+7.3965097402597406e-03*Jinv12*w55+1.6436688311688311e-03*w15*Jinv02+7.4878246753246752e-04*w10*Jinv12+5.7528409090909093e-04*Jinv02*w24+1.5888798701298701e-03*Jinv22*w35+1.0957792207792207e-03*w58*Jinv11+1.0957792207792207e-03*w57*Jinv01-1.6436688311688311e-04*Jinv21*w26+7.4878246753246752e-04*w0*Jinv01-1.0683847402597404e-03*Jinv22*w20+4.2735389610389614e-03*w43*Jinv11+4.2735389610389614e-03*Jinv21*w41+4.7666396103896100e-03*w42*Jinv02+7.4878246753246752e-04*w11*Jinv22+7.4878246753246752e-04*Jinv11*w1-7.1225649350649346e-04*w13*Jinv11+1.0957792207792207e-03*w15*Jinv01+7.8530844155844158e-04*w28*Jinv11+1.6436688311688311e-03*Jinv22*w17+5.7528409090909093e-04*w22*Jinv12-3.5612824675324673e-04*Jinv02*w3-2.0271915584415584e-03*w32*Jinv22+4.2735389610389614e-03*Jinv01*w39+7.4878246753246752e-04*Jinv22*w2-2.3833198051948050e-03*Jinv21*w38-1.3149350649350649e-03*Jinv02*w45-1.6436688311688311e-04*Jinv01*w24-9.1314935064935068e-05*Jinv22*w59+2.1367694805194807e-03*w34*Jinv11-7.1225649350649346e-04*w19*Jinv11+3.6982548701298703e-03*Jinv21*w53+1.5888798701298701e-03*Jinv12*w34+4.7666396103896100e-03*Jinv22*w44-3.5612824675324673e-04*w4*Jinv11-1.0683847402597404e-03*Jinv12*w13-2.3833198051948050e-03*w32*Jinv21-7.1225649350649346e-04*Jinv21*w20-2.0271915584415584e-03*w37*Jinv12+4.7666396103896100e-03*Jinv22*w41+2.4655032467532467e-03*w52*Jinv12+2.1367694805194807e-03*Jinv21*w35-2.3833198051948050e-03*w31*Jinv11+5.7528409090909093e-04*Jinv22*w26-2.6298701298701297e-03*w46*Jinv11-3.5612824675324673e-04*w5*Jinv21+7.3965097402597406e-03*Jinv02*w54-3.5612824675324673e-04*w4*Jinv12+5.7528409090909093e-04*w21*Jinv02+7.8530844155844158e-04*Jinv21*w29+2.1367694805194807e-03*Jinv01*w33-1.0683847402597404e-03*Jinv02*w12-7.1225649350649346e-04*Jinv01*w18-2.3833198051948050e-03*w37*Jinv11+7.4878246753246752e-04*Jinv02*w9-3.5612824675324673e-04*w6*Jinv01-2.6298701298701297e-03*w47*Jinv21+5.7528409090909093e-04*Jinv22*w23+3.6982548701298703e-03*w52*Jinv11-2.0271915584415584e-03*w31*Jinv12-1.3149350649350649e-03*w46*Jinv12-3.5612824675324673e-04*Jinv22*w8+2.4655032467532467e-03*Jinv22*w50-1.6436688311688311e-04*w22*Jinv11-3.5612824675324673e-04*Jinv12*w7-1.6436688311688311e-04*w21*Jinv01+3.6982548701298703e-03*Jinv01*w51-2.0271915584415584e-03*Jinv02*w30+2.4655032467532467e-03*Jinv12*w49-2.3833198051948050e-03*w36*Jinv01-3.5612824675324673e-04*w7*Jinv11+7.3965097402597406e-03*Jinv21*w56+6.5746753246753243e-04*Jinv02*w27-3.5612824675324673e-04*w6*Jinv02-1.0683847402597404e-03*w19*Jinv12-3.5612824675324673e-04*Jinv22*w5-1.3149350649350649e-03*w47*Jinv22+3.6982548701298703e-03*w49*Jinv11-7.1225649350649346e-04*Jinv21*w14-9.1314935064935068e-05*w58*Jinv12-1.0683847402597404e-03*Jinv02*w18-7.1225649350649346e-04*Jinv01*w12-3.5612824675324673e-04*Jinv21*w8+1.5888798701298701e-03*Jinv02*w33-1.6436688311688311e-04*Jinv21*w23+7.3965097402597406e-03*Jinv01*w54+7.3965097402597406e-03*w55*Jinv11+2.4655032467532467e-03*w48*Jinv02+7.4878246753246752e-04*Jinv01*w9+6.5746753246753243e-04*Jinv22*w29-1.0683847402597404e-03*Jinv22*w14+4.2735389610389614e-03*w40*Jinv11-2.0271915584415584e-03*Jinv02*w36+4.7666396103896100e-03*Jinv12*w43-2.3833198051948050e-03*Jinv01*w30+2.4655032467532467e-03*Jinv02*w51+7.3965097402597406e-03*Jinv22*w56+6.5746753246753243e-04*Jinv12*w28-1.6436688311688311e-04*w25*Jinv11+1.6436688311688311e-03*w16*Jinv12+3.6982548701298703e-03*Jinv21*w50+7.8530844155844158e-04*Jinv01*w27+3.6982548701298703e-03*w48*Jinv01+7.4878246753246752e-04*Jinv12*w1+7.4878246753246752e-04*w10*Jinv11)*D;
  A[56][57]=(0.0)*D;
  A[56][58]=(0.0)*D;
  A[56][59]=(6.5746753246753243e-04*w40*Jinv12+1.1870941558441558e-03*Jinv22*w53+6.4935064935064935e-04*w57*Jinv02+6.5746753246753243e-04*Jinv02*w39-6.7573051948051951e-04*Jinv22*w38-2.6481331168831169e-04*w25*Jinv12+1.5726461038961039e-04*w0*Jinv02+2.3741883116883116e-03*Jinv12*w55-3.1047077922077924e-04*w15*Jinv02+1.5726461038961039e-04*w10*Jinv12-2.6481331168831169e-04*Jinv02*w24+3.2873376623376621e-04*Jinv22*w35-1.0957792207792208e-04*Jinv22*w20+6.5746753246753243e-04*w42*Jinv02+1.5726461038961039e-04*w11*Jinv22-3.1047077922077924e-04*Jinv22*w17-2.6481331168831169e-04*w22*Jinv12-5.4788961038961042e-05*Jinv02*w3-6.7573051948051951e-04*w32*Jinv22+1.5726461038961039e-04*Jinv22*w2-1.0227272727272728e-03*Jinv02*w45+6.4935064935064935e-04*Jinv22*w59+3.2873376623376621e-04*Jinv12*w34+6.5746753246753243e-04*Jinv22*w44-1.0957792207792208e-04*Jinv12*w13-6.7573051948051951e-04*w37*Jinv12+6.5746753246753243e-04*Jinv22*w41+1.1870941558441558e-03*w52*Jinv12-2.6481331168831169e-04*Jinv22*w26+2.3741883116883116e-03*Jinv02*w54-5.4788961038961042e-05*w4*Jinv12-2.6481331168831169e-04*w21*Jinv02-1.0957792207792208e-04*Jinv02*w12+1.5726461038961039e-04*Jinv02*w9-2.6481331168831169e-04*Jinv22*w23-6.7573051948051951e-04*w31*Jinv12-1.0227272727272728e-03*w46*Jinv12-5.4788961038961042e-05*Jinv22*w8+1.1870941558441558e-03*Jinv22*w50-5.4788961038961042e-05*Jinv12*w7-6.7573051948051951e-04*Jinv02*w30+1.1870941558441558e-03*Jinv12*w49+2.0495129870129870e-04*Jinv02*w27-5.4788961038961042e-05*w6*Jinv02-1.0957792207792208e-04*w19*Jinv12-5.4788961038961042e-05*Jinv22*w5-1.0227272727272728e-03*w47*Jinv22+6.4935064935064935e-04*w58*Jinv12-1.0957792207792208e-04*Jinv02*w18+3.2873376623376621e-04*Jinv02*w33+1.1870941558441558e-03*w48*Jinv02+2.0495129870129870e-04*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14-6.7573051948051951e-04*Jinv02*w36+6.5746753246753243e-04*Jinv12*w43+1.1870941558441558e-03*Jinv02*w51+2.3741883116883116e-03*Jinv22*w56+2.0495129870129870e-04*Jinv12*w28-3.1047077922077924e-04*w16*Jinv12+1.5726461038961039e-04*Jinv12*w1)*D;
  A[57][0]=(-1.5726461038961039e-04*Jinv00*w12-5.4112554112554113e-05*w28*Jinv10+4.2613636363636367e-05*w40*Jinv12-6.4935064935064935e-05*Jinv22*w53+1.1668019480519480e-04*Jinv20*w8-1.8939393939393939e-04*w57*Jinv02-1.5726461038961039e-04*w13*Jinv10+4.2613636363636367e-05*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-1.5726461038961039e-04*w3*Jinv00-2.0495129870129870e-04*w32*Jinv20-1.5726461038961039e-04*Jinv01*w3+1.1668019480519480e-04*Jinv20*w23+1.4204545454545455e-05*Jinv22*w38-6.4935064935064935e-05*Jinv00*w54-1.8939393939393939e-04*Jinv21*w59-1.0247564935064935e-04*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-5.4112554112554113e-05*w11*Jinv21+1.9480519480519481e-04*w16*Jinv11+1.9480519480519481e-04*Jinv21*w17+1.4204545454545455e-05*Jinv01*w45-6.4935064935064935e-05*Jinv12*w55-1.8939393939393939e-04*w58*Jinv10+1.9480519480519481e-04*w15*Jinv02-5.4112554112554113e-05*w10*Jinv12-8.3198051948051951e-05*w48*Jinv00-1.0247564935064935e-04*Jinv02*w24+4.2613636363636367e-05*Jinv22*w35-1.8939393939393939e-04*w58*Jinv11-1.8939393939393939e-04*w57*Jinv01-1.0247564935064935e-04*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+1.9480519480519481e-04*w16*Jinv10-1.0247564935064935e-04*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11+1.4204545454545455e-05*w45*Jinv00+4.2613636363636367e-05*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-5.4112554112554113e-05*Jinv00*w27-5.4112554112554113e-05*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-1.5726461038961039e-04*w13*Jinv11+1.9480519480519481e-04*w15*Jinv01-5.4112554112554113e-05*w28*Jinv11-8.3198051948051951e-05*Jinv20*w50-5.4112554112554113e-05*Jinv20*w29+1.4204545454545455e-05*w46*Jinv10+1.9480519480519481e-04*Jinv22*w17+4.2613636363636367e-05*Jinv00*w33+1.1668019480519480e-04*w22*Jinv12-2.0495129870129870e-04*w31*Jinv10-1.5726461038961039e-04*Jinv02*w3+1.1668019480519480e-04*w21*Jinv00-2.0495129870129870e-04*w32*Jinv22+4.2613636363636367e-05*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2-1.0247564935064935e-04*Jinv00*w18+1.4204545454545455e-05*w36*Jinv00+1.4204545454545455e-05*Jinv21*w38+1.4204545454545455e-05*Jinv02*w45-1.0247564935064935e-04*Jinv01*w24-1.8939393939393939e-04*Jinv22*w59+4.2613636363636367e-05*w34*Jinv11-1.0247564935064935e-04*w19*Jinv11-6.4935064935064935e-05*Jinv21*w53-6.4935064935064935e-05*w51*Jinv00+4.2613636363636367e-05*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-1.5726461038961039e-04*w4*Jinv10-1.5726461038961039e-04*w4*Jinv11+4.2613636363636367e-05*w41*Jinv20-1.5726461038961039e-04*Jinv12*w13-1.0247564935064935e-04*w19*Jinv10+4.2613636363636367e-05*w34*Jinv10-2.0495129870129870e-04*w32*Jinv21-1.0247564935064935e-04*Jinv21*w20+1.1668019480519480e-04*w6*Jinv00+1.4204545454545455e-05*w37*Jinv12-6.4935064935064935e-05*Jinv20*w56+4.2613636363636367e-05*Jinv22*w41-1.0247564935064935e-04*w26*Jinv20-5.4112554112554113e-05*w11*Jinv20-6.4935064935064935e-05*w52*Jinv12+4.2613636363636367e-05*Jinv21*w35-2.0495129870129870e-04*w31*Jinv11-1.0247564935064935e-04*Jinv22*w26+1.4204545454545455e-05*w46*Jinv11-1.5726461038961039e-04*Jinv20*w14-1.5726461038961039e-04*w5*Jinv21-6.4935064935064935e-05*Jinv02*w54-1.5726461038961039e-04*w4*Jinv12+1.1668019480519480e-04*w21*Jinv02-8.3198051948051951e-05*w49*Jinv10-5.4112554112554113e-05*Jinv21*w29+4.2613636363636367e-05*Jinv01*w33+4.2613636363636367e-05*w39*Jinv00-1.5726461038961039e-04*Jinv02*w12-1.0247564935064935e-04*Jinv01*w18+1.4204545454545455e-05*w37*Jinv11-5.4112554112554113e-05*Jinv02*w9+1.1668019480519480e-04*w6*Jinv01+1.1668019480519480e-04*w7*Jinv10+1.4204545454545455e-05*w47*Jinv21+1.4204545454545455e-05*Jinv20*w38+1.1668019480519480e-04*Jinv22*w23-6.4935064935064935e-05*w52*Jinv11-2.0495129870129870e-04*w31*Jinv12+1.4204545454545455e-05*w46*Jinv12-6.4935064935064935e-05*Jinv20*w53+1.1668019480519480e-04*Jinv22*w8+1.1668019480519480e-04*w22*Jinv10-8.3198051948051951e-05*Jinv22*w50+1.1668019480519480e-04*w22*Jinv11+1.1668019480519480e-04*Jinv12*w7+1.1668019480519480e-04*w21*Jinv01-6.4935064935064935e-05*Jinv01*w51-2.0495129870129870e-04*Jinv02*w30-8.3198051948051951e-05*Jinv12*w49+1.4204545454545455e-05*w36*Jinv01-6.4935064935064935e-05*w52*Jinv10+1.1668019480519480e-04*w7*Jinv11-5.4112554112554113e-05*w9*Jinv00+1.4204545454545455e-05*w37*Jinv10-6.4935064935064935e-05*Jinv21*w56-1.0247564935064935e-04*w24*Jinv00-5.4112554112554113e-05*Jinv02*w27+1.1668019480519480e-04*w6*Jinv02-1.0247564935064935e-04*w19*Jinv12-1.5726461038961039e-04*Jinv22*w5+1.4204545454545455e-05*w47*Jinv22-8.3198051948051951e-05*w49*Jinv11-1.5726461038961039e-04*Jinv21*w14-5.4112554112554113e-05*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-1.8939393939393939e-04*w58*Jinv12-1.0247564935064935e-04*Jinv02*w18-1.5726461038961039e-04*Jinv01*w12+1.1668019480519480e-04*Jinv21*w8-1.8939393939393939e-04*w57*Jinv00-1.8939393939393939e-04*Jinv20*w59+4.2613636363636367e-05*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+4.2613636363636367e-05*w35*Jinv20+1.1668019480519480e-04*Jinv21*w23-6.4935064935064935e-05*Jinv01*w54-6.4935064935064935e-05*w55*Jinv11-1.0247564935064935e-04*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+1.9480519480519481e-04*w15*Jinv00-8.3198051948051951e-05*w48*Jinv02-5.4112554112554113e-05*Jinv01*w9+1.9480519480519481e-04*Jinv20*w17-2.0495129870129870e-04*w30*Jinv00+4.2613636363636367e-05*w40*Jinv10-1.0247564935064935e-04*w20*Jinv20-5.4112554112554113e-05*Jinv22*w29-1.5726461038961039e-04*Jinv22*w14+4.2613636363636367e-05*w40*Jinv11+1.4204545454545455e-05*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-1.5726461038961039e-04*w5*Jinv20-2.0495129870129870e-04*Jinv01*w30-6.4935064935064935e-05*Jinv02*w51-6.4935064935064935e-05*Jinv22*w56-5.4112554112554113e-05*Jinv12*w28-1.0247564935064935e-04*w25*Jinv11-6.4935064935064935e-05*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00+1.4204545454545455e-05*w47*Jinv20+1.9480519480519481e-04*w16*Jinv12-8.3198051948051951e-05*Jinv21*w50-5.4112554112554113e-05*Jinv01*w27-8.3198051948051951e-05*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-5.4112554112554113e-05*w10*Jinv11)*D;
  A[57][1]=(0.0)*D;
  A[57][2]=(0.0)*D;
  A[57][3]=(3.7439123376623376e-04*Jinv00*w12-5.2962662337662338e-04*w40*Jinv12-1.8262987012987013e-05*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02+3.7439123376623376e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44+3.9265422077922079e-04*w32*Jinv20-7.4878246753246752e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+3.1047077922077924e-04*Jinv22*w38-4.0584415584415584e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-9.6793831168831166e-04*w16*Jinv11-9.6793831168831166e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-9.6793831168831166e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+9.1314935064935065e-06*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-1.0592532467532468e-03*Jinv22*w35-4.0584415584415584e-05*w58*Jinv11-4.0584415584415584e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+1.5726461038961039e-04*w1*Jinv10-6.9399350649350649e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+7.3051948051948052e-05*w43*Jinv11-5.2962662337662338e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*w11*Jinv22+9.2329545454545453e-05*Jinv11*w1-3.7439123376623376e-04*w13*Jinv11-9.6793831168831166e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-9.6793831168831166e-04*Jinv22*w17-5.6615259740259744e-04*Jinv00*w33+3.1960227272727273e-04*w22*Jinv12+3.9265422077922079e-04*w31*Jinv10-7.4878246753246752e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+3.1960227272727273e-04*Jinv00*w18+1.7349837662337662e-04*w36*Jinv00+3.1047077922077924e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-1.0592532467532468e-03*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11-1.8262987012987013e-05*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00-1.0592532467532468e-03*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-7.4878246753246752e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20-3.7439123376623376e-04*Jinv12*w13+3.1960227272727273e-04*w19*Jinv10-5.6615259740259744e-04*w34*Jinv10-3.9265422077922079e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00+3.1047077922077924e-04*w37*Jinv12-5.2962662337662338e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20-1.8262987012987013e-05*w52*Jinv12-1.0592532467532468e-03*Jinv21*w35-3.9265422077922079e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11+3.7439123376623376e-04*Jinv20*w14-7.4878246753246752e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54-7.4878246753246752e-04*w4*Jinv12+3.1960227272727273e-04*w21*Jinv02+9.1314935064935065e-06*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-1.0592532467532468e-03*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00-3.7439123376623376e-04*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+3.1047077922077924e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9+1.5523538961038962e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.3697240259740259e-04*w47*Jinv21+1.7349837662337662e-04*Jinv20*w38+3.1960227272727273e-04*Jinv22*w23-1.8262987012987013e-05*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+1.5523538961038962e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+4.8396915584415583e-04*Jinv22*w50+3.1960227272727273e-04*w22*Jinv11+1.5523538961038962e-04*Jinv12*w7+3.1960227272727273e-04*w21*Jinv01-1.8262987012987013e-05*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+4.8396915584415583e-04*Jinv12*w49+3.1047077922077924e-04*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10+1.5523538961038962e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+1.7349837662337662e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27+1.5523538961038962e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12-7.4878246753246752e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+4.8396915584415583e-04*w49*Jinv11-3.7439123376623376e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18-3.7439123376623376e-04*Jinv01*w12+1.5523538961038962e-04*Jinv21*w8-1.0592532467532468e-03*Jinv02*w33+1.5726461038961039e-04*w0*Jinv00-5.6615259740259744e-04*w35*Jinv20+3.1960227272727273e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2-6.9399350649350649e-04*w15*Jinv00+4.8396915584415583e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-6.9399350649350649e-04*Jinv20*w17+3.9265422077922079e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10+3.1960227272727273e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29-3.7439123376623376e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+3.1047077922077924e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.9265422077922079e-04*Jinv01*w30-1.8262987012987013e-05*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-9.6793831168831166e-04*w16*Jinv12+4.8396915584415583e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+4.8396915584415583e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[57][4]=(0.0)*D;
  A[57][5]=(0.0)*D;
  A[57][6]=(-3.1960227272727273e-04*Jinv00*w12+3.6525974025974026e-05*w40*Jinv12+4.7483766233766234e-04*Jinv22*w53-4.0584415584415584e-05*w57*Jinv02-3.1960227272727273e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00-1.7349837662337662e-04*w32*Jinv20+4.2918019480519480e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23-7.8530844155844158e-04*Jinv22*w38-4.0584415584415584e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-2.7394480519480518e-04*w16*Jinv11-2.7394480519480518e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-2.7394480519480518e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+9.1314935064935065e-06*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-4.0584415584415584e-05*w58*Jinv11-4.0584415584415584e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.1668019480519480e-04*w1*Jinv10+6.9399350649350649e-04*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-4.9310064935064932e-04*w43*Jinv11+3.6525974025974026e-05*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*w11*Jinv22-1.0247564935064935e-04*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-2.7394480519480518e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-2.7394480519480518e-04*Jinv22*w17+5.6615259740259744e-04*Jinv00*w33+1.0957792207792208e-04*w22*Jinv12-1.7349837662337662e-04*w31*Jinv10+4.2918019480519480e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2-3.7439123376623376e-04*Jinv00*w18-3.9265422077922079e-04*w36*Jinv00-7.8530844155844158e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+4.7483766233766234e-04*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.2918019480519480e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13-3.7439123376623376e-04*w19*Jinv10+5.6615259740259744e-04*w34*Jinv10+1.3697240259740259e-04*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-7.8530844155844158e-04*w37*Jinv12+3.6525974025974026e-05*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+4.7483766233766234e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+1.3697240259740259e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11-3.1960227272727273e-04*Jinv20*w14+4.2918019480519480e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54+4.2918019480519480e-04*w4*Jinv12+1.0957792207792208e-04*w21*Jinv02+9.1314935064935065e-06*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-7.8530844155844158e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.4878246753246752e-04*w6*Jinv01+1.3697240259740259e-04*w47*Jinv21-3.9265422077922079e-04*Jinv20*w38+1.0957792207792208e-04*Jinv22*w23+4.7483766233766234e-04*w52*Jinv11+1.3697240259740259e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53-7.4878246753246752e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50+1.0957792207792208e-04*w22*Jinv11-7.4878246753246752e-04*Jinv12*w7+1.0957792207792208e-04*w21*Jinv01+4.7483766233766234e-04*Jinv01*w51+1.3697240259740259e-04*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-7.8530844155844158e-04*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10-7.4878246753246752e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00-3.9265422077922079e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-7.4878246753246752e-04*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+4.2918019480519480e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22-9.1314935064935065e-06*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12-7.4878246753246752e-04*Jinv21*w8-4.9310064935064932e-04*Jinv02*w33-1.1668019480519480e-04*w0*Jinv00+5.6615259740259744e-04*w35*Jinv20+1.0957792207792208e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+6.9399350649350649e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+6.9399350649350649e-04*Jinv20*w17-1.7349837662337662e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10-3.7439123376623376e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.8530844155844158e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+1.3697240259740259e-04*Jinv01*w30+4.7483766233766234e-04*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-2.7394480519480518e-04*w16*Jinv12-9.1314935064935065e-06*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-9.1314935064935065e-06*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[57][7]=(0.0)*D;
  A[57][8]=(0.0)*D;
  A[57][9]=(1.0247564935064935e-04*Jinv00*w12+5.4112554112554113e-05*w28*Jinv10+1.5726461038961039e-04*Jinv20*w8+1.0247564935064935e-04*w13*Jinv10-1.1668019480519480e-04*w3*Jinv00-1.4204545454545455e-05*w32*Jinv20+1.0247564935064935e-04*Jinv20*w23+6.4935064935064935e-05*Jinv00*w54-4.2613636363636367e-05*w43*Jinv10+1.8939393939393939e-04*w58*Jinv10+6.4935064935064935e-05*w48*Jinv00+5.4112554112554113e-05*w1*Jinv10-1.9480519480519481e-04*w16*Jinv10-1.4204545454545455e-05*w45*Jinv00+5.4112554112554113e-05*Jinv00*w27+6.4935064935064935e-05*Jinv20*w50+5.4112554112554113e-05*Jinv20*w29-1.4204545454545455e-05*w46*Jinv10-4.2613636363636367e-05*Jinv00*w33-1.4204545454545455e-05*w31*Jinv10+1.0247564935064935e-04*w21*Jinv00+1.5726461038961039e-04*Jinv00*w18+2.0495129870129870e-04*w36*Jinv00+8.3198051948051951e-05*w51*Jinv00-1.1668019480519480e-04*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+1.5726461038961039e-04*w19*Jinv10-4.2613636363636367e-05*w34*Jinv10+1.5726461038961039e-04*w6*Jinv00+6.4935064935064935e-05*Jinv20*w56-1.1668019480519480e-04*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20+1.0247564935064935e-04*Jinv20*w14+6.4935064935064935e-05*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+1.5726461038961039e-04*w7*Jinv10+2.0495129870129870e-04*Jinv20*w38+8.3198051948051951e-05*Jinv20*w53+1.0247564935064935e-04*w22*Jinv10+8.3198051948051951e-05*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+2.0495129870129870e-04*w37*Jinv10-1.1668019480519480e-04*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-4.2613636363636367e-05*Jinv20*w44+1.8939393939393939e-04*w57*Jinv00+1.8939393939393939e-04*Jinv20*w59+5.4112554112554113e-05*w0*Jinv00-4.2613636363636367e-05*w35*Jinv20-1.1668019480519480e-04*w25*Jinv10+5.4112554112554113e-05*Jinv20*w2-1.9480519480519481e-04*w15*Jinv00-1.9480519480519481e-04*Jinv20*w17-1.4204545454545455e-05*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+1.5726461038961039e-04*w20*Jinv20-1.1668019480519480e-04*w5*Jinv20+6.4935064935064935e-05*w55*Jinv10-4.2613636363636367e-05*w42*Jinv00-1.4204545454545455e-05*w47*Jinv20)*D;
  A[57][10]=(0.0)*D;
  A[57][11]=(0.0)*D;
  A[57][12]=(-7.4878246753246752e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-1.0592532467532468e-03*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53+3.1960227272727273e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02-7.4878246753246752e-04*w13*Jinv10-1.0592532467532468e-03*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44-3.7439123376623376e-04*w3*Jinv00-3.9265422077922079e-04*w32*Jinv20+3.7439123376623376e-04*Jinv01*w3+1.5523538961038962e-04*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38-1.8262987012987013e-05*Jinv00*w54+4.2918019480519480e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+1.5726461038961039e-04*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01-6.9399350649350649e-04*w16*Jinv11-6.9399350649350649e-04*Jinv21*w17+1.7349837662337662e-04*Jinv01*w45-1.8262987012987013e-05*Jinv12*w55-4.0584415584415584e-05*w58*Jinv10-9.6793831168831166e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12+4.8396915584415583e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+3.1960227272727273e-04*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-9.6793831168831166e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11+3.1047077922077924e-04*w45*Jinv00-5.6615259740259744e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22+1.5726461038961039e-04*Jinv11*w1-6.9399350649350649e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+4.8396915584415583e-04*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+3.1047077922077924e-04*w46*Jinv10-9.6793831168831166e-04*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.5523538961038962e-04*w22*Jinv12-3.9265422077922079e-04*w31*Jinv10-3.7439123376623376e-04*Jinv02*w3+1.5523538961038962e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.6615259740259744e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00+3.1047077922077924e-04*Jinv02*w45+3.1960227272727273e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-9.1314935064935065e-06*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-3.7439123376623376e-04*w4*Jinv10+3.7439123376623376e-04*w4*Jinv11-1.0592532467532468e-03*w41*Jinv20-7.4878246753246752e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.9265422077922079e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+3.1960227272727273e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12-1.8262987012987013e-05*Jinv20*w56-1.0592532467532468e-03*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35+3.9265422077922079e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.7349837662337662e-04*w46*Jinv11-7.4878246753246752e-04*Jinv20*w14+3.7439123376623376e-04*w5*Jinv21-1.8262987012987013e-05*Jinv02*w54-3.7439123376623376e-04*w4*Jinv12+1.5523538961038962e-04*w21*Jinv02+4.8396915584415583e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-1.0592532467532468e-03*w39*Jinv00-7.4878246753246752e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18-1.0247564935064935e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+3.1960227272727273e-04*w7*Jinv10+1.7349837662337662e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38+1.5523538961038962e-04*Jinv22*w23-3.9265422077922079e-04*w31*Jinv12+3.1047077922077924e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+3.1960227272727273e-04*Jinv22*w8+1.5523538961038962e-04*w22*Jinv10+4.8396915584415583e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+3.1960227272727273e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-3.9265422077922079e-04*Jinv02*w30+4.8396915584415583e-04*Jinv12*w49-9.1314935064935065e-06*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+3.1960227272727273e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-3.7439123376623376e-04*Jinv22*w5+3.1047077922077924e-04*w47*Jinv22+9.1314935064935065e-06*w49*Jinv11-1.0247564935064935e-04*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-1.0957792207792208e-04*Jinv21*w8-4.0584415584415584e-05*w57*Jinv00-4.0584415584415584e-05*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-9.6793831168831166e-04*w15*Jinv00+4.8396915584415583e-04*w48*Jinv02-9.6793831168831166e-04*Jinv20*w17-3.9265422077922079e-04*w30*Jinv00-1.0592532467532468e-03*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-7.4878246753246752e-04*Jinv22*w14-5.6615259740259744e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.7439123376623376e-04*w5*Jinv20+3.9265422077922079e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51-1.8262987012987013e-05*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+3.1960227272727273e-04*w25*Jinv11-1.8262987012987013e-05*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+3.1047077922077924e-04*w47*Jinv20-9.6793831168831166e-04*w16*Jinv12+9.1314935064935065e-06*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1)*D;
  A[57][13]=(0.0)*D;
  A[57][14]=(0.0)*D;
  A[57][15]=(9.6793831168831166e-04*Jinv00*w12-1.1323051948051949e-03*w40*Jinv12+9.8620129870129864e-04*Jinv22*w53-6.9399350649350649e-04*Jinv20*w8-4.8701298701298701e-04*w57*Jinv02+9.6793831168831166e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-1.1323051948051949e-03*Jinv21*w44+6.9399350649350649e-04*w3*Jinv00+5.2962662337662338e-04*w32*Jinv20+9.6793831168831166e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-4.9310064935064932e-04*Jinv22*w38-5.4788961038961036e-04*w25*Jinv12-2.2524350649350650e-04*w0*Jinv02-1.9480519480519481e-04*Jinv21*w2-1.1323051948051949e-03*w43*Jinv10-1.1323051948051949e-03*w42*Jinv01-5.2962662337662338e-04*Jinv01*w45+9.8620129870129864e-04*Jinv12*w55-2.7759740259740259e-03*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35-9.6793831168831166e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-1.9480519480519481e-04*w1*Jinv10-5.4788961038961036e-04*Jinv22*w20-1.1323051948051949e-03*w43*Jinv11-2.2646103896103898e-03*w42*Jinv02-3.0438311688311688e-05*w11*Jinv22-1.9480519480519481e-04*Jinv11*w1+6.9399350649350649e-04*w13*Jinv11+1.9480519480519481e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-2.7759740259740259e-03*Jinv22*w17-2.7394480519480518e-04*w22*Jinv12+5.2962662337662338e-04*w31*Jinv10+4.2004870129870131e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22-2.2524350649350650e-04*Jinv22*w2-9.6793831168831166e-04*Jinv00*w18-5.2962662337662338e-04*w36*Jinv00-4.9310064935064932e-04*Jinv02*w45-9.6793831168831166e-04*Jinv01*w24-4.8701298701298701e-04*Jinv22*w59+1.1323051948051949e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+4.9310064935064932e-04*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-2.2646103896103898e-03*Jinv22*w44+6.9399350649350649e-04*w4*Jinv10+9.6793831168831166e-04*w4*Jinv11+1.1323051948051949e-03*w41*Jinv20+4.2004870129870131e-04*Jinv12*w13-9.6793831168831166e-04*w19*Jinv10+5.2962662337662338e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-6.9399350649350649e-04*w6*Jinv00-4.9310064935064932e-04*w37*Jinv12-1.1323051948051949e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20+9.8620129870129864e-04*w52*Jinv12+1.1323051948051949e-03*Jinv21*w35+5.2962662337662338e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.2962662337662338e-04*w46*Jinv11+9.6793831168831166e-04*Jinv20*w14+9.6793831168831166e-04*w5*Jinv21+9.8620129870129864e-04*Jinv02*w54+4.2004870129870131e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+1.1323051948051949e-03*Jinv01*w33+1.1323051948051949e-03*w39*Jinv00+4.2004870129870131e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-3.0438311688311688e-05*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-6.9399350649350649e-04*w7*Jinv10-5.2962662337662338e-04*w47*Jinv21-5.2962662337662338e-04*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-4.9310064935064932e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50-6.9399350649350649e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7-6.9399350649350649e-04*w21*Jinv01+3.6525974025974026e-05*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+4.9310064935064932e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-5.2962662337662338e-04*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+4.2004870129870131e-04*Jinv22*w5-4.9310064935064932e-04*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+6.9399350649350649e-04*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10-1.1323051948051949e-03*Jinv20*w44-4.8701298701298701e-04*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+6.9399350649350649e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.1323051948051949e-03*Jinv02*w33-1.9480519480519481e-04*w0*Jinv00-6.9399350649350649e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2+4.9310064935064932e-04*w48*Jinv02+5.2962662337662338e-04*w30*Jinv00+1.1323051948051949e-03*w40*Jinv10-9.6793831168831166e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+4.2004870129870131e-04*Jinv22*w14-4.9310064935064932e-04*Jinv02*w36-2.2646103896103898e-03*Jinv12*w43+6.9399350649350649e-04*w5*Jinv20+5.2962662337662338e-04*Jinv01*w30+9.8620129870129864e-04*Jinv02*w51+9.8620129870129864e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-9.6793831168831166e-04*w25*Jinv11-1.1323051948051949e-03*w42*Jinv00-2.7759740259740259e-03*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-2.2524350649350650e-04*Jinv12*w1)*D;
  A[57][16]=(0.0)*D;
  A[57][17]=(0.0)*D;
  A[57][18]=(-2.1915584415584417e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10+3.7439123376623376e-04*Jinv20*w8-2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv21*w44-3.1960227272727273e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-4.2918019480519480e-04*Jinv01*w3-4.2918019480519480e-04*Jinv20*w23+1.8262987012987013e-05*Jinv00*w54+4.0584415584415584e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+1.0592532467532468e-03*w43*Jinv10+4.9310064935064932e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+2.7394480519480518e-04*w16*Jinv11+2.7394480519480518e-04*Jinv21*w17-1.3697240259740259e-04*Jinv01*w45+4.0584415584415584e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00+4.0584415584415584e-05*w58*Jinv11+4.0584415584415584e-05*w57*Jinv01-4.2918019480519480e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+9.6793831168831166e-04*w16*Jinv10+4.9310064935064932e-04*w43*Jinv11-3.1047077922077924e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-1.4204545454545455e-05*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-1.0957792207792208e-04*w13*Jinv11+2.7394480519480518e-04*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-3.1047077922077924e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+7.4878246753246752e-04*Jinv00*w18+3.9265422077922079e-04*w36*Jinv00+7.8530844155844158e-04*Jinv21*w38-4.2918019480519480e-04*Jinv01*w24+4.9310064935064932e-04*w34*Jinv11+7.4878246753246752e-04*w19*Jinv11-4.7483766233766234e-04*Jinv21*w53-4.8396915584415583e-04*w51*Jinv00-3.1960227272727273e-04*w4*Jinv10-4.2918019480519480e-04*w4*Jinv11-7.3051948051948052e-05*w41*Jinv20+7.4878246753246752e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21+7.4878246753246752e-04*Jinv21*w20+3.7439123376623376e-04*w6*Jinv00+1.8262987012987013e-05*Jinv20*w56-1.5523538961038962e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20+4.9310064935064932e-04*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11-1.3697240259740259e-04*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14-4.2918019480519480e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-7.3051948051948052e-05*w39*Jinv00+7.4878246753246752e-04*Jinv01*w18+7.8530844155844158e-04*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01+3.7439123376623376e-04*w7*Jinv10-1.3697240259740259e-04*w47*Jinv21+3.9265422077922079e-04*Jinv20*w38-4.7483766233766234e-04*w52*Jinv11-4.8396915584415583e-04*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01-4.7483766233766234e-04*Jinv01*w51+7.8530844155844158e-04*w36*Jinv01-4.8396915584415583e-04*w52*Jinv10+7.4878246753246752e-04*w7*Jinv11-9.2329545454545453e-05*w9*Jinv00+3.9265422077922079e-04*w37*Jinv10+9.1314935064935065e-06*Jinv21*w56-1.5523538961038962e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11-1.0957792207792208e-04*Jinv21*w14-9.2329545454545453e-05*w10*Jinv10+1.0592532467532468e-03*Jinv20*w44-1.0957792207792208e-04*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8+4.0584415584415584e-05*w57*Jinv00+4.0584415584415584e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20-1.0957792207792208e-04*Jinv21*w23+9.1314935064935065e-06*Jinv01*w54+9.1314935064935065e-06*w55*Jinv11-1.5523538961038962e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+9.6793831168831166e-04*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+9.6793831168831166e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+7.4878246753246752e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-3.1960227272727273e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30-4.2918019480519480e-04*w25*Jinv11+1.8262987012987013e-05*w55*Jinv10+1.0592532467532468e-03*w42*Jinv00-3.1047077922077924e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[57][19]=(0.0)*D;
  A[57][20]=(0.0)*D;
  A[57][21]=(4.2918019480519480e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53+1.0957792207792208e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02+4.2918019480519480e-04*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+1.3697240259740259e-04*w32*Jinv20-3.1960227272727273e-04*Jinv01*w3-7.4878246753246752e-04*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38+4.7483766233766234e-04*Jinv00*w54-7.4878246753246752e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.1668019480519480e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01+6.9399350649350649e-04*w16*Jinv11+6.9399350649350649e-04*Jinv21*w17-3.9265422077922079e-04*Jinv01*w45+4.7483766233766234e-04*Jinv12*w55-4.0584415584415584e-05*w58*Jinv10-2.7394480519480518e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-9.1314935064935065e-06*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-3.7439123376623376e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10-2.7394480519480518e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11-7.8530844155844158e-04*w45*Jinv00+5.6615259740259744e-04*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-1.1668019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11+6.9399350649350649e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11-9.1314935064935065e-06*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.8530844155844158e-04*w46*Jinv10-2.7394480519480518e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.4878246753246752e-04*w22*Jinv12+1.3697240259740259e-04*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3-7.4878246753246752e-04*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22+5.6615259740259744e-04*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00-7.8530844155844158e-04*Jinv02*w45-3.7439123376623376e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-9.1314935064935065e-06*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10-3.1960227272727273e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20+4.2918019480519480e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-1.7349837662337662e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+1.0957792207792208e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12+4.7483766233766234e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35-1.7349837662337662e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-3.9265422077922079e-04*w46*Jinv11+4.2918019480519480e-04*Jinv20*w14-3.1960227272727273e-04*w5*Jinv21+4.7483766233766234e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12-7.4878246753246752e-04*w21*Jinv02-9.1314935064935065e-06*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00+4.2918019480519480e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18-1.0247564935064935e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.0957792207792208e-04*w7*Jinv10-3.9265422077922079e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38-7.4878246753246752e-04*Jinv22*w23+1.3697240259740259e-04*w31*Jinv12-7.8530844155844158e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+1.0957792207792208e-04*Jinv22*w8-7.4878246753246752e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50+1.0957792207792208e-04*Jinv12*w7+1.3697240259740259e-04*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-9.1314935064935065e-06*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+1.0957792207792208e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5-7.8530844155844158e-04*w47*Jinv22+9.1314935064935065e-06*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-4.0584415584415584e-05*w57*Jinv00-4.0584415584415584e-05*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2-2.7394480519480518e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-2.7394480519480518e-04*Jinv20*w17+1.3697240259740259e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.2918019480519480e-04*Jinv22*w14+5.6615259740259744e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20-1.7349837662337662e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51+4.7483766233766234e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.7439123376623376e-04*w25*Jinv11+4.7483766233766234e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00-7.8530844155844158e-04*w47*Jinv20-2.7394480519480518e-04*w16*Jinv12+9.1314935064935065e-06*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1)*D;
  A[57][22]=(0.0)*D;
  A[57][23]=(0.0)*D;
  A[57][24]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+1.0592532467532468e-03*Jinv21*w44-1.0957792207792208e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3+7.4878246753246752e-04*Jinv20*w23-4.7483766233766234e-04*Jinv00*w54+4.0584415584415584e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w43*Jinv10+1.0592532467532468e-03*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+9.6793831168831166e-04*w16*Jinv11+9.6793831168831166e-04*Jinv21*w17+3.9265422077922079e-04*Jinv01*w45+4.0584415584415584e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00+4.0584415584415584e-05*w58*Jinv11+4.0584415584415584e-05*w57*Jinv01+7.4878246753246752e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+2.7394480519480518e-04*w16*Jinv10+1.0592532467532468e-03*w43*Jinv11+7.8530844155844158e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-3.1960227272727273e-04*w13*Jinv11+9.6793831168831166e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.8530844155844158e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.2918019480519480e-04*Jinv00*w18-1.3697240259740259e-04*w36*Jinv00-3.1047077922077924e-04*Jinv21*w38+7.4878246753246752e-04*Jinv01*w24-7.3051948051948052e-05*w34*Jinv11-1.5523538961038962e-04*w19*Jinv11+1.8262987012987013e-05*Jinv21*w53+9.1314935064935065e-06*w51*Jinv00-1.0957792207792208e-04*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+4.9310064935064932e-04*w41*Jinv20-4.2918019480519480e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21-1.5523538961038962e-04*Jinv21*w20-1.0957792207792208e-04*w6*Jinv00-4.7483766233766234e-04*Jinv20*w56+7.4878246753246752e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20-7.3051948051948052e-05*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11+3.9265422077922079e-04*w46*Jinv11-4.2918019480519480e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33+4.9310064935064932e-04*w39*Jinv00-1.5523538961038962e-04*Jinv01*w18-3.1047077922077924e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-1.0957792207792208e-04*w7*Jinv10+3.9265422077922079e-04*w47*Jinv21-1.3697240259740259e-04*Jinv20*w38+1.8262987012987013e-05*w52*Jinv11+9.1314935064935065e-06*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+3.7439123376623376e-04*w22*Jinv11+3.7439123376623376e-04*w21*Jinv01+1.8262987012987013e-05*Jinv01*w51-3.1047077922077924e-04*w36*Jinv01+9.1314935064935065e-06*w52*Jinv10-4.2918019480519480e-04*w7*Jinv11+1.0247564935064935e-04*w9*Jinv00-1.3697240259740259e-04*w37*Jinv10-4.8396915584415583e-04*Jinv21*w56+7.4878246753246752e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11-3.1960227272727273e-04*Jinv21*w14+1.0247564935064935e-04*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-3.1960227272727273e-04*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8+4.0584415584415584e-05*w57*Jinv00+4.0584415584415584e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.7439123376623376e-04*Jinv21*w23-4.8396915584415583e-04*Jinv01*w54-4.8396915584415583e-04*w55*Jinv11+7.4878246753246752e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+2.7394480519480518e-04*w15*Jinv00-1.4204545454545455e-05*Jinv01*w9+2.7394480519480518e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-4.2918019480519480e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11-1.0957792207792208e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30+7.4878246753246752e-04*w25*Jinv11-4.7483766233766234e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00+7.8530844155844158e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[57][25]=(0.0)*D;
  A[57][26]=(0.0)*D;
  A[57][27]=(-4.2613636363636367e-05*Jinv21*w44+1.0247564935064935e-04*Jinv01*w3+1.8939393939393939e-04*Jinv21*w59+5.4112554112554113e-05*Jinv21*w2-4.2613636363636367e-05*w42*Jinv01+5.4112554112554113e-05*w11*Jinv21-1.9480519480519481e-04*w16*Jinv11-1.9480519480519481e-04*Jinv21*w17+2.0495129870129870e-04*Jinv01*w45+1.8939393939393939e-04*w58*Jinv11+1.8939393939393939e-04*w57*Jinv01+1.5726461038961039e-04*Jinv21*w26+5.4112554112554113e-05*w0*Jinv01-4.2613636363636367e-05*w43*Jinv11-4.2613636363636367e-05*Jinv21*w41+5.4112554112554113e-05*Jinv11*w1-1.1668019480519480e-04*w13*Jinv11-1.9480519480519481e-04*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-4.2613636363636367e-05*Jinv01*w39-1.4204545454545455e-05*Jinv21*w38+1.5726461038961039e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11-1.1668019480519480e-04*w19*Jinv11+6.4935064935064935e-05*Jinv21*w53+1.0247564935064935e-04*w4*Jinv11-1.4204545454545455e-05*w32*Jinv21-1.1668019480519480e-04*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35-1.4204545454545455e-05*w31*Jinv11+2.0495129870129870e-04*w46*Jinv11+1.0247564935064935e-04*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33-1.1668019480519480e-04*Jinv01*w18-1.4204545454545455e-05*w37*Jinv11+1.0247564935064935e-04*w6*Jinv01+2.0495129870129870e-04*w47*Jinv21+6.4935064935064935e-05*w52*Jinv11+1.5726461038961039e-04*w22*Jinv11+1.5726461038961039e-04*w21*Jinv01+6.4935064935064935e-05*Jinv01*w51-1.4204545454545455e-05*w36*Jinv01+1.0247564935064935e-04*w7*Jinv11+8.3198051948051951e-05*Jinv21*w56+6.4935064935064935e-05*w49*Jinv11-1.1668019480519480e-04*Jinv21*w14-1.1668019480519480e-04*Jinv01*w12+1.0247564935064935e-04*Jinv21*w8+1.5726461038961039e-04*Jinv21*w23+8.3198051948051951e-05*Jinv01*w54+8.3198051948051951e-05*w55*Jinv11+5.4112554112554113e-05*Jinv01*w9-4.2613636363636367e-05*w40*Jinv11-1.4204545454545455e-05*Jinv01*w30+1.5726461038961039e-04*w25*Jinv11+6.4935064935064935e-05*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27+6.4935064935064935e-05*w48*Jinv01+5.4112554112554113e-05*w10*Jinv11)*D;
  A[57][28]=(0.0)*D;
  A[57][29]=(0.0)*D;
  A[57][30]=(-3.9265422077922079e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12+5.5702110389610384e-04*Jinv22*w53+1.7349837662337662e-04*Jinv20*w8+6.4935064935064935e-04*w57*Jinv02-3.9265422077922079e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-3.9265422077922079e-04*Jinv01*w3+1.7349837662337662e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+6.0876623376623375e-04*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12+1.2175324675324675e-04*w0*Jinv02+5.6818181818181818e-05*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45+5.5702110389610384e-04*Jinv12*w55+6.0876623376623375e-04*w58*Jinv10-2.5568181818181820e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+6.0876623376623375e-04*w58*Jinv11+6.0876623376623375e-04*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26+5.6818181818181818e-05*w0*Jinv01+5.6818181818181818e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-2.5568181818181820e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+5.6818181818181818e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11+1.0957792207792207e-03*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10-2.5568181818181820e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33-2.5568181818181820e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+1.7349837662337662e-04*w21*Jinv00+1.2784090909090910e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39+1.2175324675324675e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24+6.4935064935064935e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34-2.5568181818181820e-04*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10-3.9265422077922079e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+1.7349837662337662e-04*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+5.5702110389610384e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11-3.9265422077922079e-04*Jinv20*w14-3.9265422077922079e-04*w5*Jinv21+5.5702110389610384e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.5568181818181820e-04*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9+1.7349837662337662e-04*w6*Jinv01+1.7349837662337662e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38-2.5568181818181820e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11+1.2784090909090910e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53-2.5568181818181820e-04*Jinv22*w8+1.7349837662337662e-04*w22*Jinv10+6.2094155844155847e-04*Jinv22*w50+1.7349837662337662e-04*w22*Jinv11-2.5568181818181820e-04*Jinv12*w7+1.7349837662337662e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+1.2784090909090910e-04*Jinv02*w30+6.2094155844155847e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10+1.7349837662337662e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27-2.5568181818181820e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+6.4935064935064935e-04*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.7349837662337662e-04*Jinv21*w8+6.0876623376623375e-04*w57*Jinv00+6.0876623376623375e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+5.6818181818181818e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+1.7349837662337662e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10+5.6818181818181818e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00+6.2094155844155847e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36-2.5568181818181820e-04*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30+5.5702110389610384e-04*Jinv02*w51+5.5702110389610384e-04*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28+1.3697240259740259e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20-2.5568181818181820e-04*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01+1.2175324675324675e-04*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[57][31]=(0.0)*D;
  A[57][32]=(0.0)*D;
  A[57][33]=(5.2962662337662338e-04*Jinv00*w12+9.1314935064935063e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53-5.6615259740259744e-04*Jinv20*w8-2.4350649350649351e-04*w57*Jinv02+5.2962662337662338e-04*w13*Jinv10+9.1314935064935063e-04*Jinv02*w39-4.3831168831168834e-04*Jinv21*w44+5.6615259740259744e-04*w3*Jinv00+4.3831168831168834e-04*w32*Jinv20+7.3051948051948052e-05*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-7.3051948051948055e-04*Jinv21*w59-3.4699675324675324e-04*w25*Jinv12-3.0438311688311688e-05*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-4.3831168831168834e-04*w42*Jinv01+1.2175324675324675e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55+1.6436688311688311e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-3.4699675324675324e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35-7.3051948051948055e-04*w58*Jinv11-7.3051948051948055e-04*w57*Jinv01+7.3051948051948052e-05*Jinv21*w26-3.0438311688311688e-05*w0*Jinv01-4.2613636363636367e-05*w1*Jinv10-2.1915584415584417e-04*Jinv22*w20-4.3831168831168834e-04*w43*Jinv11-2.1915584415584417e-04*Jinv21*w41+6.9399350649350649e-04*w42*Jinv02+4.2613636363636367e-05*w11*Jinv22-3.0438311688311688e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50+1.6436688311688311e-03*Jinv22*w17-3.8352272727272725e-04*w22*Jinv12+4.3831168831168834e-04*w31*Jinv10+6.2094155844155847e-04*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-5.2962662337662338e-04*Jinv00*w18-4.3831168831168834e-04*w36*Jinv00-6.5746753246753243e-04*Jinv21*w38-2.5568181818181820e-04*Jinv02*w45+7.3051948051948052e-05*Jinv01*w24-2.4350649350649351e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-4.9310064935064932e-04*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+6.9399350649350649e-04*Jinv22*w44+5.6615259740259744e-04*w4*Jinv10+7.3051948051948052e-05*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20+3.1047077922077924e-04*Jinv12*w13-5.2962662337662338e-04*w19*Jinv10-2.1915584415584417e-04*w32*Jinv21-4.9310064935064932e-04*Jinv21*w20-5.6615259740259744e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12+9.1314935064935063e-04*Jinv22*w41+3.6525974025974026e-05*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-3.4699675324675324e-04*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11+5.2962662337662338e-04*Jinv20*w14+7.3051948051948052e-05*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+6.2094155844155847e-04*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00+3.1047077922077924e-04*Jinv02*w12-4.9310064935064932e-04*Jinv01*w18-6.5746753246753243e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-5.6615259740259744e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-4.3831168831168834e-04*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-2.5568181818181820e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49-6.5746753246753243e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-4.3831168831168834e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+3.6525974025974026e-05*w24*Jinv00+1.9480519480519481e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+6.2094155844155847e-04*Jinv22*w5-2.5568181818181820e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-2.4350649350649351e-04*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+3.6525974025974026e-05*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8+1.8262987012987013e-03*Jinv02*w33-4.2613636363636367e-05*w0*Jinv00+3.6525974025974026e-05*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+3.6525974025974026e-05*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-4.3831168831168834e-04*w48*Jinv02+1.2175324675324675e-05*Jinv01*w9+4.3831168831168834e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-5.2962662337662338e-04*w20*Jinv20+1.9480519480519481e-04*Jinv22*w29+3.1047077922077924e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+6.9399350649350649e-04*Jinv12*w43+5.6615259740259744e-04*w5*Jinv20-2.1915584415584417e-04*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+1.9480519480519481e-04*Jinv12*w28+7.3051948051948052e-05*w25*Jinv11-2.1915584415584417e-04*w42*Jinv00+1.6436688311688311e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+1.2175324675324675e-05*w10*Jinv11)*D;
  A[57][34]=(0.0)*D;
  A[57][35]=(0.0)*D;
  A[57][36]=(-1.3697240259740259e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.7483766233766234e-04*Jinv22*w53+3.9265422077922079e-04*Jinv20*w8+4.0584415584415584e-05*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-1.7349837662337662e-04*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23+7.8530844155844158e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-4.2918019480519480e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+4.3831168831168834e-04*w43*Jinv10+9.1314935064935065e-06*Jinv12*w55-6.0876623376623375e-04*w58*Jinv10+2.7394480519480518e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-4.2918019480519480e-04*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35-1.4204545454545455e-05*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.4878246753246752e-04*Jinv22*w20+2.1915584415584417e-04*w45*Jinv00+4.9310064935064932e-04*w42*Jinv02-1.4204545454545455e-05*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-5.4788961038961036e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10+2.7394480519480518e-04*Jinv22*w17+4.3831168831168834e-04*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+2.1915584415584417e-04*w31*Jinv10-4.2918019480519480e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-1.3697240259740259e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.9265422077922079e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-1.3697240259740259e-04*Jinv02*w45+4.0584415584415584e-05*Jinv22*w59-1.0957792207792207e-03*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.7349837662337662e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13+3.9265422077922079e-04*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10+3.9265422077922079e-04*w6*Jinv00+7.8530844155844158e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-1.7349837662337662e-04*w26*Jinv20-5.6818181818181818e-05*w11*Jinv20-4.7483766233766234e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14+9.1314935064935065e-06*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.9265422077922079e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-1.3697240259740259e-04*w31*Jinv12-1.3697240259740259e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+7.4878246753246752e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+9.1314935064935065e-06*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-1.3697240259740259e-04*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49-1.0957792207792207e-03*w52*Jinv10-5.6818181818181818e-05*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10-1.7349837662337662e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5-1.3697240259740259e-04*w47*Jinv22-5.6818181818181818e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+4.0584415584415584e-05*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18-6.0876623376623375e-04*w57*Jinv00-6.0876623376623375e-04*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-1.4204545454545455e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-1.7349837662337662e-04*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00+9.1314935064935065e-06*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10+3.9265422077922079e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14+7.8530844155844158e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-1.7349837662337662e-04*w5*Jinv20-4.7483766233766234e-04*Jinv02*w51+9.1314935064935065e-06*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28-5.4788961038961036e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20+2.7394480519480518e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[57][37]=(0.0)*D;
  A[57][38]=(0.0)*D;
  A[57][39]=(7.3051948051948052e-05*Jinv00*w12+1.2175324675324675e-05*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8-2.4350649350649351e-04*w57*Jinv02+7.3051948051948052e-05*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+5.2962662337662338e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-2.5568181818181820e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-2.1915584415584417e-04*w25*Jinv12-4.2613636363636367e-05*Jinv21*w2-4.3831168831168834e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-4.3831168831168834e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-7.3051948051948055e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+1.9480519480519481e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+9.1314935064935063e-04*Jinv22*w35-5.2962662337662338e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-3.0438311688311688e-05*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10-3.4699675324675324e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-6.5746753246753243e-04*w45*Jinv00+6.9399350649350649e-04*w42*Jinv02+1.2175324675324675e-05*Jinv00*w27+1.9480519480519481e-04*w11*Jinv22-4.2613636363636367e-05*Jinv11*w1+5.6615259740259744e-04*w13*Jinv11+4.2613636363636367e-05*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+1.2175324675324675e-05*Jinv20*w29-6.5746753246753243e-04*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10+3.1047077922077924e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22+7.3051948051948052e-05*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45-5.2962662337662338e-04*Jinv01*w24-2.4350649350649351e-04*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11+5.4788961038961036e-04*w51*Jinv00+9.1314935064935063e-04*Jinv12*w34+6.9399350649350649e-04*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+5.2962662337662338e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+6.2094155844155847e-04*Jinv12*w13+7.3051948051948052e-05*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+4.3831168831168834e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00-2.5568181818181820e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-4.9310064935064932e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+4.3831168831168834e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26-4.3831168831168834e-04*w46*Jinv11+7.3051948051948052e-05*Jinv20*w14+5.2962662337662338e-04*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54+3.1047077922077924e-04*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+6.2094155844155847e-04*Jinv02*w12+3.6525974025974026e-05*Jinv01*w18+1.9480519480519481e-04*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10-4.3831168831168834e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-5.6615259740259744e-04*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-5.6615259740259744e-04*w21*Jinv01+2.7394480519480518e-04*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+5.4788961038961036e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56-4.9310064935064932e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-3.4699675324675324e-04*w19*Jinv12+3.1047077922077924e-04*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11+5.6615259740259744e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-4.3831168831168834e-04*Jinv20*w44-2.4350649350649351e-04*w58*Jinv12-3.4699675324675324e-04*Jinv02*w18+5.6615259740259744e-04*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8-7.3051948051948055e-04*w57*Jinv00-7.3051948051948055e-04*Jinv20*w59+9.1314935064935063e-04*Jinv02*w33-3.0438311688311688e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-5.6615259740259744e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11-4.9310064935064932e-04*w25*Jinv10-3.0438311688311688e-05*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02-1.1323051948051949e-03*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+7.3051948051948052e-05*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29+6.2094155844155847e-04*Jinv22*w14-2.5568181818181820e-04*Jinv02*w36+6.9399350649350649e-04*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+4.3831168831168834e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-5.2962662337662338e-04*w25*Jinv11+1.0957792207792207e-03*w55*Jinv10-4.3831168831168834e-04*w42*Jinv00-6.5746753246753243e-04*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01)*D;
  A[57][40]=(0.0)*D;
  A[57][41]=(0.0)*D;
  A[57][42]=(-7.3051948051948052e-05*Jinv00*w12-1.2175324675324675e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12-9.8620129870129864e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+4.8701298701298701e-04*w57*Jinv02-7.3051948051948052e-05*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20-7.3051948051948052e-05*Jinv01*w3+4.9310064935064932e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+7.3051948051948055e-04*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+2.2524350649350650e-04*w0*Jinv02+3.0438311688311688e-05*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-1.2175324675324675e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45-9.8620129870129864e-04*Jinv12*w55+7.3051948051948055e-04*w58*Jinv10+2.7759740259740259e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+7.3051948051948055e-04*w58*Jinv11+7.3051948051948055e-04*w57*Jinv01-7.3051948051948052e-05*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01+3.0438311688311688e-05*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+6.5746753246753243e-04*w45*Jinv00+2.1915584415584417e-04*Jinv21*w41+2.2646103896103898e-03*w42*Jinv02-1.2175324675324675e-05*Jinv00*w27+3.0438311688311688e-05*w11*Jinv22+3.0438311688311688e-05*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+3.0438311688311688e-05*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50-1.2175324675324675e-05*Jinv20*w29+6.5746753246753243e-04*w46*Jinv10+2.7759740259740259e-03*Jinv22*w17+2.1915584415584417e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+2.1915584415584417e-04*w31*Jinv10-4.2004870129870131e-04*Jinv02*w3+4.9310064935064932e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+2.1915584415584417e-04*Jinv01*w39+2.2524350649350650e-04*Jinv22*w2-7.3051948051948052e-05*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+6.5746753246753243e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45-7.3051948051948052e-05*Jinv01*w24+4.8701298701298701e-04*Jinv22*w59+4.3831168831168834e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+2.2646103896103898e-03*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10-7.3051948051948052e-05*w4*Jinv11+4.3831168831168834e-04*w41*Jinv20-4.2004870129870131e-04*Jinv12*w13-7.3051948051948052e-05*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10+2.1915584415584417e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+3.0438311688311688e-05*w11*Jinv20-9.8620129870129864e-04*w52*Jinv12+4.3831168831168834e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11-7.3051948051948052e-05*Jinv20*w14-7.3051948051948052e-05*w5*Jinv21-9.8620129870129864e-04*Jinv02*w54-4.2004870129870131e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+3.0438311688311688e-05*Jinv21*w29+4.3831168831168834e-04*Jinv01*w33+4.3831168831168834e-04*w39*Jinv00-4.2004870129870131e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18+6.5746753246753243e-04*w37*Jinv11+3.0438311688311688e-05*Jinv02*w9+4.9310064935064932e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8+4.9310064935064932e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49+6.5746753246753243e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+4.9310064935064932e-04*w7*Jinv11+3.0438311688311688e-05*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00+3.0438311688311688e-05*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-4.2004870129870131e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+3.0438311688311688e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+4.8701298701298701e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12+4.9310064935064932e-04*Jinv21*w8+7.3051948051948055e-04*w57*Jinv00+7.3051948051948055e-04*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+3.0438311688311688e-05*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02-1.2175324675324675e-05*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00+4.3831168831168834e-04*w40*Jinv10-7.3051948051948052e-05*w20*Jinv20+3.0438311688311688e-05*Jinv22*w29-4.2004870129870131e-04*Jinv22*w14+2.1915584415584417e-04*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+2.2646103896103898e-03*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20+2.1915584415584417e-04*Jinv01*w30-9.8620129870129864e-04*Jinv02*w51-9.8620129870129864e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28-7.3051948051948052e-05*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+6.5746753246753243e-04*w47*Jinv20+2.7759740259740259e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50+3.0438311688311688e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01+2.2524350649350650e-04*Jinv12*w1-1.2175324675324675e-05*w10*Jinv11)*D;
  A[57][43]=(0.0)*D;
  A[57][44]=(0.0)*D;
  A[57][45]=(4.9310064935064932e-04*w40*Jinv12+9.1314935064935065e-06*Jinv22*w53+4.0584415584415584e-05*w57*Jinv02+4.9310064935064932e-04*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3-1.3697240259740259e-04*Jinv22*w38-6.0876623376623375e-04*Jinv21*w59+7.4878246753246752e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-1.4204545454545455e-05*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.7483766233766234e-04*Jinv12*w55+2.7394480519480518e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-6.0876623376623375e-04*w58*Jinv11-6.0876623376623375e-04*w57*Jinv01+3.9265422077922079e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01-4.2918019480519480e-04*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41+4.9310064935064932e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22-1.4204545454545455e-05*Jinv11*w1-1.7349837662337662e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-5.6818181818181818e-05*w28*Jinv11+2.7394480519480518e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-1.3697240259740259e-04*w32*Jinv22+4.3831168831168834e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2+2.1915584415584417e-04*Jinv21*w38+7.8530844155844158e-04*Jinv02*w45+3.9265422077922079e-04*Jinv01*w24+4.0584415584415584e-05*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11-1.7349837662337662e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-4.2918019480519480e-04*Jinv12*w13+2.1915584415584417e-04*w32*Jinv21-1.7349837662337662e-04*Jinv21*w20-1.3697240259740259e-04*w37*Jinv12+4.9310064935064932e-04*Jinv22*w41+9.1314935064935065e-06*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+7.4878246753246752e-04*Jinv22*w26+6.5746753246753243e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21-4.7483766233766234e-04*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-5.6818181818181818e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-4.2918019480519480e-04*Jinv02*w12-1.7349837662337662e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+7.4878246753246752e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11-1.3697240259740259e-04*w31*Jinv12+7.8530844155844158e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+9.1314935064935065e-06*Jinv22*w50+3.9265422077922079e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+3.9265422077922079e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51-1.3697240259740259e-04*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11-1.0957792207792207e-03*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5+7.8530844155844158e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11-1.7349837662337662e-04*Jinv21*w14+4.0584415584415584e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18-1.7349837662337662e-04*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.9265422077922079e-04*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+9.1314935064935065e-06*w48*Jinv02-1.4204545454545455e-05*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+4.3831168831168834e-04*w40*Jinv11-1.3697240259740259e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43+2.1915584415584417e-04*Jinv01*w30+9.1314935064935065e-06*Jinv02*w51-4.7483766233766234e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.9265422077922079e-04*w25*Jinv11+2.7394480519480518e-04*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50-5.6818181818181818e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1-1.4204545454545455e-05*w10*Jinv11)*D;
  A[57][46]=(0.0)*D;
  A[57][47]=(0.0)*D;
  A[57][48]=(-9.1314935064935065e-06*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12-1.1414366883116883e-03*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.2987012987012987e-03*w57*Jinv02-9.1314935064935065e-06*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39+5.4788961038961036e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-9.1314935064935065e-06*Jinv01*w3-9.1314935064935065e-06*Jinv20*w23+6.6659902597402602e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54-1.5827922077922079e-03*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-6.4935064935064935e-05*Jinv21*w2+5.4788961038961036e-04*w43*Jinv10+5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17+5.4788961038961036e-04*Jinv01*w45-1.1414366883116883e-03*Jinv12*w55-1.5827922077922079e-03*w58*Jinv10-1.8262987012987014e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12-1.0957792207792207e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-1.5827922077922079e-03*w58*Jinv11-1.5827922077922079e-03*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26-6.4935064935064935e-05*w0*Jinv01-6.4935064935064935e-05*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+5.4788961038961036e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+5.4788961038961036e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11-1.0957792207792207e-03*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-1.8262987012987014e-04*Jinv22*w17+5.4788961038961036e-04*Jinv00*w33+1.1870941558441559e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3-9.1314935064935065e-06*w21*Jinv00+4.0178571428571428e-04*w32*Jinv22+5.4788961038961036e-04*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18+5.4788961038961036e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+6.6659902597402602e-04*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24-1.2987012987012987e-03*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-9.1314935064935065e-06*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20-2.1915584415584417e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10+5.4788961038961036e-04*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00+6.6659902597402602e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20-1.1414366883116883e-03*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+5.4788961038961036e-04*w46*Jinv11-9.1314935064935065e-06*Jinv20*w14-9.1314935064935065e-06*w5*Jinv21-1.1414366883116883e-03*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+1.1870941558441559e-04*w21*Jinv02-1.0957792207792207e-03*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00-2.1915584415584417e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9-9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10+5.4788961038961036e-04*w47*Jinv21+5.4788961038961036e-04*Jinv20*w38+1.1870941558441559e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11+4.0178571428571428e-04*w31*Jinv12+6.6659902597402602e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+1.1870941558441559e-04*Jinv22*w8-9.1314935064935065e-06*w22*Jinv10-1.1870941558441558e-03*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11+1.1870941558441559e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51+4.0178571428571428e-04*Jinv02*w30-1.1870941558441558e-03*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10-9.1314935064935065e-06*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00+5.4788961038961036e-04*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+1.1870941558441559e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5+6.6659902597402602e-04*w47*Jinv22-1.0957792207792207e-03*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10+5.4788961038961036e-04*Jinv20*w44-1.2987012987012987e-03*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-9.1314935064935065e-06*Jinv21*w8-1.5827922077922079e-03*w57*Jinv00-1.5827922077922079e-03*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-6.4935064935064935e-05*w0*Jinv00+5.4788961038961036e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-6.4935064935064935e-05*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-1.1870941558441558e-03*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+5.4788961038961036e-04*w40*Jinv11+6.6659902597402602e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30-1.1414366883116883e-03*Jinv02*w51-1.1414366883116883e-03*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28-9.1314935064935065e-06*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+5.4788961038961036e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-1.8262987012987014e-04*w16*Jinv12-1.0957792207792207e-03*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-1.0957792207792207e-03*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[57][49]=(0.0)*D;
  A[57][50]=(0.0)*D;
  A[57][51]=(9.1314935064935065e-06*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12-9.1314935064935068e-05*Jinv22*w53+9.1314935064935065e-06*Jinv20*w8+2.8409090909090908e-04*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+9.1314935064935065e-06*w3*Jinv00-5.4788961038961036e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23-1.4610389610389610e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54+1.2784090909090910e-04*w25*Jinv12-9.2329545454545453e-05*w0*Jinv02-5.4788961038961036e-04*w43*Jinv10-4.5657467532467534e-05*Jinv12*w55+1.5827922077922079e-03*w58*Jinv10-6.7573051948051951e-04*w15*Jinv02-5.6818181818181818e-05*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+1.2784090909090910e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35+6.4935064935064935e-05*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10-2.1002435064935065e-04*Jinv22*w20-5.4788961038961036e-04*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02+6.4935064935064935e-05*Jinv00*w27-5.6818181818181818e-05*w11*Jinv22+1.0957792207792207e-03*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-6.7573051948051951e-04*Jinv22*w17-5.4788961038961036e-04*Jinv00*w33+6.3920454545454550e-05*w22*Jinv12-5.4788961038961036e-04*w31*Jinv10+1.2784090909090910e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00+1.1870941558441559e-04*w32*Jinv22-9.2329545454545453e-05*Jinv22*w2+9.1314935064935065e-06*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00+1.1870941558441559e-04*Jinv02*w45+2.8409090909090908e-04*Jinv22*w59+1.0957792207792207e-03*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv10-5.4788961038961036e-04*w41*Jinv20+6.3920454545454550e-05*Jinv12*w13+9.1314935064935065e-06*w19*Jinv10-5.4788961038961036e-04*w34*Jinv10+9.1314935064935065e-06*w6*Jinv00-1.4610389610389610e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+9.1314935064935065e-06*w26*Jinv20+6.4935064935064935e-05*w11*Jinv20-9.1314935064935068e-05*w52*Jinv12+1.2784090909090910e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14-4.5657467532467534e-05*Jinv02*w54+1.2784090909090910e-04*w4*Jinv12+6.3920454545454550e-05*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-5.4788961038961036e-04*w39*Jinv00+6.3920454545454550e-05*Jinv02*w12-5.6818181818181818e-05*Jinv02*w9+9.1314935064935065e-06*w7*Jinv10-5.4788961038961036e-04*Jinv20*w38+6.3920454545454550e-05*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12+1.0957792207792207e-03*Jinv20*w53-2.1002435064935065e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10-4.5657467532467534e-05*Jinv22*w50-2.1002435064935065e-04*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-4.5657467532467534e-05*Jinv12*w49+1.0957792207792207e-03*w52*Jinv10+6.4935064935064935e-05*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+9.1314935064935065e-06*w24*Jinv00-9.2329545454545453e-05*Jinv02*w27-2.1002435064935065e-04*w6*Jinv02-2.1002435064935065e-04*w19*Jinv12+1.2784090909090910e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+2.8409090909090908e-04*w58*Jinv12-2.1002435064935065e-04*Jinv02*w18+1.5827922077922079e-03*w57*Jinv00+1.5827922077922079e-03*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33+6.4935064935064935e-05*w0*Jinv00-5.4788961038961036e-04*w35*Jinv20+9.1314935064935065e-06*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00-4.5657467532467534e-05*w48*Jinv02-4.9310064935064932e-04*Jinv20*w17-5.4788961038961036e-04*w30*Jinv00-5.4788961038961036e-04*w40*Jinv10+9.1314935064935065e-06*w20*Jinv20-9.2329545454545453e-05*Jinv22*w29+6.3920454545454550e-05*Jinv22*w14-1.4610389610389610e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+9.1314935064935065e-06*w5*Jinv20-9.1314935064935068e-05*Jinv02*w51-4.5657467532467534e-05*Jinv22*w56-9.2329545454545453e-05*Jinv12*w28+1.0957792207792207e-03*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-6.7573051948051951e-04*w16*Jinv12-9.2329545454545453e-05*Jinv12*w1)*D;
  A[57][52]=(0.0)*D;
  A[57][53]=(0.0)*D;
  A[57][54]=(-7.6704545454545450e-04*w40*Jinv12-4.5657467532467534e-05*Jinv22*w53+2.8409090909090908e-04*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3+1.1870941558441559e-04*Jinv22*w38+1.5827922077922079e-03*Jinv21*w59-2.1002435064935065e-04*w25*Jinv12-9.2329545454545453e-05*w0*Jinv02+6.4935064935064935e-05*Jinv21*w2-5.4788961038961036e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-9.1314935064935068e-05*Jinv12*w55-6.7573051948051951e-04*w15*Jinv02-9.2329545454545453e-05*w10*Jinv12-2.1002435064935065e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35+1.5827922077922079e-03*w58*Jinv11+1.5827922077922079e-03*w57*Jinv01+9.1314935064935065e-06*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+1.2784090909090910e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-5.4788961038961036e-04*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02-9.2329545454545453e-05*w11*Jinv22+6.4935064935064935e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01+6.4935064935064935e-05*w28*Jinv11-6.7573051948051951e-04*Jinv22*w17-2.1002435064935065e-04*w22*Jinv12+6.3920454545454550e-05*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-5.4788961038961036e-04*Jinv01*w39-9.2329545454545453e-05*Jinv22*w2-5.4788961038961036e-04*Jinv21*w38-1.4610389610389610e-04*Jinv02*w45+9.1314935064935065e-06*Jinv01*w24+2.8409090909090908e-04*Jinv22*w59-5.4788961038961036e-04*w34*Jinv11+9.1314935064935065e-06*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11+1.2784090909090910e-04*Jinv12*w13-5.4788961038961036e-04*w32*Jinv21+9.1314935064935065e-06*Jinv21*w20+1.1870941558441559e-04*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41-4.5657467532467534e-05*w52*Jinv12-5.4788961038961036e-04*Jinv21*w35-5.4788961038961036e-04*w31*Jinv11-2.1002435064935065e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21-9.1314935064935068e-05*Jinv02*w54+6.3920454545454550e-05*w4*Jinv12-2.1002435064935065e-04*w21*Jinv02+6.4935064935064935e-05*Jinv21*w29-5.4788961038961036e-04*Jinv01*w33+1.2784090909090910e-04*Jinv02*w12+9.1314935064935065e-06*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-9.2329545454545453e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-5.4788961038961036e-04*w47*Jinv21-2.1002435064935065e-04*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+1.1870941558441559e-04*w31*Jinv12-1.4610389610389610e-04*w46*Jinv12+6.3920454545454550e-05*Jinv22*w8-4.5657467532467534e-05*Jinv22*w50+9.1314935064935065e-06*w22*Jinv11+6.3920454545454550e-05*Jinv12*w7+9.1314935064935065e-06*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+1.1870941558441559e-04*Jinv02*w30-4.5657467532467534e-05*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11+1.0957792207792207e-03*Jinv21*w56-5.6818181818181818e-05*Jinv02*w27+6.3920454545454550e-05*w6*Jinv02+1.2784090909090910e-04*w19*Jinv12+6.3920454545454550e-05*Jinv22*w5-1.4610389610389610e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14+2.8409090909090908e-04*w58*Jinv12+1.2784090909090910e-04*Jinv02*w18+9.1314935064935065e-06*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33+9.1314935064935065e-06*Jinv21*w23+1.0957792207792207e-03*Jinv01*w54+1.0957792207792207e-03*w55*Jinv11-4.5657467532467534e-05*w48*Jinv02+6.4935064935064935e-05*Jinv01*w9-5.6818181818181818e-05*Jinv22*w29+1.2784090909090910e-04*Jinv22*w14-5.4788961038961036e-04*w40*Jinv11+1.1870941558441559e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43-5.4788961038961036e-04*Jinv01*w30-4.5657467532467534e-05*Jinv02*w51-9.1314935064935068e-05*Jinv22*w56-5.6818181818181818e-05*Jinv12*w28+9.1314935064935065e-06*w25*Jinv11-6.7573051948051951e-04*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50+6.4935064935064935e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01-9.2329545454545453e-05*Jinv12*w1+6.4935064935064935e-05*w10*Jinv11)*D;
  A[57][55]=(0.0)*D;
  A[57][56]=(0.0)*D;
  A[57][57]=(1.2175324675324675e-04*w40*Jinv12+6.4935064935064935e-04*Jinv22*w53+8.3874458874458877e-04*w57*Jinv02+1.2175324675324675e-04*Jinv02*w39-3.2467532467532468e-04*Jinv22*w38+2.0292207792207792e-05*w25*Jinv12+9.4696969696969697e-05*w0*Jinv02+6.4935064935064935e-04*Jinv12*w55+2.4350649350649351e-04*w15*Jinv02+9.4696969696969697e-05*w10*Jinv12+2.0292207792207792e-05*Jinv02*w24+1.2175324675324675e-04*Jinv22*w35+2.0292207792207792e-05*Jinv22*w20+1.2175324675324675e-04*w42*Jinv02+9.4696969696969697e-05*w11*Jinv22+2.4350649350649351e-04*Jinv22*w17+2.0292207792207792e-05*w22*Jinv12+2.0292207792207792e-05*Jinv02*w3-3.2467532467532468e-04*w32*Jinv22+9.4696969696969697e-05*Jinv22*w2-3.2467532467532468e-04*Jinv02*w45+8.3874458874458877e-04*Jinv22*w59+1.2175324675324675e-04*Jinv12*w34+1.2175324675324675e-04*Jinv22*w44+2.0292207792207792e-05*Jinv12*w13-3.2467532467532468e-04*w37*Jinv12+1.2175324675324675e-04*Jinv22*w41+6.4935064935064935e-04*w52*Jinv12+2.0292207792207792e-05*Jinv22*w26+6.4935064935064935e-04*Jinv02*w54+2.0292207792207792e-05*w4*Jinv12+2.0292207792207792e-05*w21*Jinv02+2.0292207792207792e-05*Jinv02*w12+9.4696969696969697e-05*Jinv02*w9+2.0292207792207792e-05*Jinv22*w23-3.2467532467532468e-04*w31*Jinv12-3.2467532467532468e-04*w46*Jinv12+2.0292207792207792e-05*Jinv22*w8+6.4935064935064935e-04*Jinv22*w50+2.0292207792207792e-05*Jinv12*w7-3.2467532467532468e-04*Jinv02*w30+6.4935064935064935e-04*Jinv12*w49+9.4696969696969697e-05*Jinv02*w27+2.0292207792207792e-05*w6*Jinv02+2.0292207792207792e-05*w19*Jinv12+2.0292207792207792e-05*Jinv22*w5-3.2467532467532468e-04*w47*Jinv22+8.3874458874458877e-04*w58*Jinv12+2.0292207792207792e-05*Jinv02*w18+1.2175324675324675e-04*Jinv02*w33+6.4935064935064935e-04*w48*Jinv02+9.4696969696969697e-05*Jinv22*w29+2.0292207792207792e-05*Jinv22*w14-3.2467532467532468e-04*Jinv02*w36+1.2175324675324675e-04*Jinv12*w43+6.4935064935064935e-04*Jinv02*w51+6.4935064935064935e-04*Jinv22*w56+9.4696969696969697e-05*Jinv12*w28+2.4350649350649351e-04*w16*Jinv12+9.4696969696969697e-05*Jinv12*w1)*D;
  A[57][58]=(0.0)*D;
  A[57][59]=(0.0)*D;
  A[58][0]=(0.0)*D;
  A[58][1]=(-1.5726461038961039e-04*Jinv00*w12-5.4112554112554113e-05*w28*Jinv10+4.2613636363636367e-05*w40*Jinv12-6.4935064935064935e-05*Jinv22*w53+1.1668019480519480e-04*Jinv20*w8-1.8939393939393939e-04*w57*Jinv02-1.5726461038961039e-04*w13*Jinv10+4.2613636363636367e-05*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-1.5726461038961039e-04*w3*Jinv00-2.0495129870129870e-04*w32*Jinv20-1.5726461038961039e-04*Jinv01*w3+1.1668019480519480e-04*Jinv20*w23+1.4204545454545455e-05*Jinv22*w38-6.4935064935064935e-05*Jinv00*w54-1.8939393939393939e-04*Jinv21*w59-1.0247564935064935e-04*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-5.4112554112554113e-05*w11*Jinv21+1.9480519480519481e-04*w16*Jinv11+1.9480519480519481e-04*Jinv21*w17+1.4204545454545455e-05*Jinv01*w45-6.4935064935064935e-05*Jinv12*w55-1.8939393939393939e-04*w58*Jinv10+1.9480519480519481e-04*w15*Jinv02-5.4112554112554113e-05*w10*Jinv12-8.3198051948051951e-05*w48*Jinv00-1.0247564935064935e-04*Jinv02*w24+4.2613636363636367e-05*Jinv22*w35-1.8939393939393939e-04*w58*Jinv11-1.8939393939393939e-04*w57*Jinv01-1.0247564935064935e-04*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+1.9480519480519481e-04*w16*Jinv10-1.0247564935064935e-04*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11+1.4204545454545455e-05*w45*Jinv00+4.2613636363636367e-05*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-5.4112554112554113e-05*Jinv00*w27-5.4112554112554113e-05*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-1.5726461038961039e-04*w13*Jinv11+1.9480519480519481e-04*w15*Jinv01-5.4112554112554113e-05*w28*Jinv11-8.3198051948051951e-05*Jinv20*w50-5.4112554112554113e-05*Jinv20*w29+1.4204545454545455e-05*w46*Jinv10+1.9480519480519481e-04*Jinv22*w17+4.2613636363636367e-05*Jinv00*w33+1.1668019480519480e-04*w22*Jinv12-2.0495129870129870e-04*w31*Jinv10-1.5726461038961039e-04*Jinv02*w3+1.1668019480519480e-04*w21*Jinv00-2.0495129870129870e-04*w32*Jinv22+4.2613636363636367e-05*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2-1.0247564935064935e-04*Jinv00*w18+1.4204545454545455e-05*w36*Jinv00+1.4204545454545455e-05*Jinv21*w38+1.4204545454545455e-05*Jinv02*w45-1.0247564935064935e-04*Jinv01*w24-1.8939393939393939e-04*Jinv22*w59+4.2613636363636367e-05*w34*Jinv11-1.0247564935064935e-04*w19*Jinv11-6.4935064935064935e-05*Jinv21*w53-6.4935064935064935e-05*w51*Jinv00+4.2613636363636367e-05*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-1.5726461038961039e-04*w4*Jinv10-1.5726461038961039e-04*w4*Jinv11+4.2613636363636367e-05*w41*Jinv20-1.5726461038961039e-04*Jinv12*w13-1.0247564935064935e-04*w19*Jinv10+4.2613636363636367e-05*w34*Jinv10-2.0495129870129870e-04*w32*Jinv21-1.0247564935064935e-04*Jinv21*w20+1.1668019480519480e-04*w6*Jinv00+1.4204545454545455e-05*w37*Jinv12-6.4935064935064935e-05*Jinv20*w56+4.2613636363636367e-05*Jinv22*w41-1.0247564935064935e-04*w26*Jinv20-5.4112554112554113e-05*w11*Jinv20-6.4935064935064935e-05*w52*Jinv12+4.2613636363636367e-05*Jinv21*w35-2.0495129870129870e-04*w31*Jinv11-1.0247564935064935e-04*Jinv22*w26+1.4204545454545455e-05*w46*Jinv11-1.5726461038961039e-04*Jinv20*w14-1.5726461038961039e-04*w5*Jinv21-6.4935064935064935e-05*Jinv02*w54-1.5726461038961039e-04*w4*Jinv12+1.1668019480519480e-04*w21*Jinv02-8.3198051948051951e-05*w49*Jinv10-5.4112554112554113e-05*Jinv21*w29+4.2613636363636367e-05*Jinv01*w33+4.2613636363636367e-05*w39*Jinv00-1.5726461038961039e-04*Jinv02*w12-1.0247564935064935e-04*Jinv01*w18+1.4204545454545455e-05*w37*Jinv11-5.4112554112554113e-05*Jinv02*w9+1.1668019480519480e-04*w6*Jinv01+1.1668019480519480e-04*w7*Jinv10+1.4204545454545455e-05*w47*Jinv21+1.4204545454545455e-05*Jinv20*w38+1.1668019480519480e-04*Jinv22*w23-6.4935064935064935e-05*w52*Jinv11-2.0495129870129870e-04*w31*Jinv12+1.4204545454545455e-05*w46*Jinv12-6.4935064935064935e-05*Jinv20*w53+1.1668019480519480e-04*Jinv22*w8+1.1668019480519480e-04*w22*Jinv10-8.3198051948051951e-05*Jinv22*w50+1.1668019480519480e-04*w22*Jinv11+1.1668019480519480e-04*Jinv12*w7+1.1668019480519480e-04*w21*Jinv01-6.4935064935064935e-05*Jinv01*w51-2.0495129870129870e-04*Jinv02*w30-8.3198051948051951e-05*Jinv12*w49+1.4204545454545455e-05*w36*Jinv01-6.4935064935064935e-05*w52*Jinv10+1.1668019480519480e-04*w7*Jinv11-5.4112554112554113e-05*w9*Jinv00+1.4204545454545455e-05*w37*Jinv10-6.4935064935064935e-05*Jinv21*w56-1.0247564935064935e-04*w24*Jinv00-5.4112554112554113e-05*Jinv02*w27+1.1668019480519480e-04*w6*Jinv02-1.0247564935064935e-04*w19*Jinv12-1.5726461038961039e-04*Jinv22*w5+1.4204545454545455e-05*w47*Jinv22-8.3198051948051951e-05*w49*Jinv11-1.5726461038961039e-04*Jinv21*w14-5.4112554112554113e-05*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-1.8939393939393939e-04*w58*Jinv12-1.0247564935064935e-04*Jinv02*w18-1.5726461038961039e-04*Jinv01*w12+1.1668019480519480e-04*Jinv21*w8-1.8939393939393939e-04*w57*Jinv00-1.8939393939393939e-04*Jinv20*w59+4.2613636363636367e-05*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+4.2613636363636367e-05*w35*Jinv20+1.1668019480519480e-04*Jinv21*w23-6.4935064935064935e-05*Jinv01*w54-6.4935064935064935e-05*w55*Jinv11-1.0247564935064935e-04*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+1.9480519480519481e-04*w15*Jinv00-8.3198051948051951e-05*w48*Jinv02-5.4112554112554113e-05*Jinv01*w9+1.9480519480519481e-04*Jinv20*w17-2.0495129870129870e-04*w30*Jinv00+4.2613636363636367e-05*w40*Jinv10-1.0247564935064935e-04*w20*Jinv20-5.4112554112554113e-05*Jinv22*w29-1.5726461038961039e-04*Jinv22*w14+4.2613636363636367e-05*w40*Jinv11+1.4204545454545455e-05*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-1.5726461038961039e-04*w5*Jinv20-2.0495129870129870e-04*Jinv01*w30-6.4935064935064935e-05*Jinv02*w51-6.4935064935064935e-05*Jinv22*w56-5.4112554112554113e-05*Jinv12*w28-1.0247564935064935e-04*w25*Jinv11-6.4935064935064935e-05*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00+1.4204545454545455e-05*w47*Jinv20+1.9480519480519481e-04*w16*Jinv12-8.3198051948051951e-05*Jinv21*w50-5.4112554112554113e-05*Jinv01*w27-8.3198051948051951e-05*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-5.4112554112554113e-05*w10*Jinv11)*D;
  A[58][2]=(0.0)*D;
  A[58][3]=(0.0)*D;
  A[58][4]=(3.7439123376623376e-04*Jinv00*w12-5.2962662337662338e-04*w40*Jinv12-1.8262987012987013e-05*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02+3.7439123376623376e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44+3.9265422077922079e-04*w32*Jinv20-7.4878246753246752e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+3.1047077922077924e-04*Jinv22*w38-4.0584415584415584e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-9.6793831168831166e-04*w16*Jinv11-9.6793831168831166e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-9.6793831168831166e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+9.1314935064935065e-06*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-1.0592532467532468e-03*Jinv22*w35-4.0584415584415584e-05*w58*Jinv11-4.0584415584415584e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+1.5726461038961039e-04*w1*Jinv10-6.9399350649350649e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+7.3051948051948052e-05*w43*Jinv11-5.2962662337662338e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*w11*Jinv22+9.2329545454545453e-05*Jinv11*w1-3.7439123376623376e-04*w13*Jinv11-9.6793831168831166e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-9.6793831168831166e-04*Jinv22*w17-5.6615259740259744e-04*Jinv00*w33+3.1960227272727273e-04*w22*Jinv12+3.9265422077922079e-04*w31*Jinv10-7.4878246753246752e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+3.1960227272727273e-04*Jinv00*w18+1.7349837662337662e-04*w36*Jinv00+3.1047077922077924e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-1.0592532467532468e-03*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11-1.8262987012987013e-05*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00-1.0592532467532468e-03*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-7.4878246753246752e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20-3.7439123376623376e-04*Jinv12*w13+3.1960227272727273e-04*w19*Jinv10-5.6615259740259744e-04*w34*Jinv10-3.9265422077922079e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00+3.1047077922077924e-04*w37*Jinv12-5.2962662337662338e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20-1.8262987012987013e-05*w52*Jinv12-1.0592532467532468e-03*Jinv21*w35-3.9265422077922079e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11+3.7439123376623376e-04*Jinv20*w14-7.4878246753246752e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54-7.4878246753246752e-04*w4*Jinv12+3.1960227272727273e-04*w21*Jinv02+9.1314935064935065e-06*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-1.0592532467532468e-03*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00-3.7439123376623376e-04*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+3.1047077922077924e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9+1.5523538961038962e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.3697240259740259e-04*w47*Jinv21+1.7349837662337662e-04*Jinv20*w38+3.1960227272727273e-04*Jinv22*w23-1.8262987012987013e-05*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+1.5523538961038962e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+4.8396915584415583e-04*Jinv22*w50+3.1960227272727273e-04*w22*Jinv11+1.5523538961038962e-04*Jinv12*w7+3.1960227272727273e-04*w21*Jinv01-1.8262987012987013e-05*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+4.8396915584415583e-04*Jinv12*w49+3.1047077922077924e-04*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10+1.5523538961038962e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+1.7349837662337662e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27+1.5523538961038962e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12-7.4878246753246752e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+4.8396915584415583e-04*w49*Jinv11-3.7439123376623376e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18-3.7439123376623376e-04*Jinv01*w12+1.5523538961038962e-04*Jinv21*w8-1.0592532467532468e-03*Jinv02*w33+1.5726461038961039e-04*w0*Jinv00-5.6615259740259744e-04*w35*Jinv20+3.1960227272727273e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2-6.9399350649350649e-04*w15*Jinv00+4.8396915584415583e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-6.9399350649350649e-04*Jinv20*w17+3.9265422077922079e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10+3.1960227272727273e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29-3.7439123376623376e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+3.1047077922077924e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.9265422077922079e-04*Jinv01*w30-1.8262987012987013e-05*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-9.6793831168831166e-04*w16*Jinv12+4.8396915584415583e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+4.8396915584415583e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[58][5]=(0.0)*D;
  A[58][6]=(0.0)*D;
  A[58][7]=(-3.1960227272727273e-04*Jinv00*w12+3.6525974025974026e-05*w40*Jinv12+4.7483766233766234e-04*Jinv22*w53-4.0584415584415584e-05*w57*Jinv02-3.1960227272727273e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00-1.7349837662337662e-04*w32*Jinv20+4.2918019480519480e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23-7.8530844155844158e-04*Jinv22*w38-4.0584415584415584e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-2.7394480519480518e-04*w16*Jinv11-2.7394480519480518e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-2.7394480519480518e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+9.1314935064935065e-06*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-4.0584415584415584e-05*w58*Jinv11-4.0584415584415584e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.1668019480519480e-04*w1*Jinv10+6.9399350649350649e-04*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-4.9310064935064932e-04*w43*Jinv11+3.6525974025974026e-05*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*w11*Jinv22-1.0247564935064935e-04*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-2.7394480519480518e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-2.7394480519480518e-04*Jinv22*w17+5.6615259740259744e-04*Jinv00*w33+1.0957792207792208e-04*w22*Jinv12-1.7349837662337662e-04*w31*Jinv10+4.2918019480519480e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2-3.7439123376623376e-04*Jinv00*w18-3.9265422077922079e-04*w36*Jinv00-7.8530844155844158e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+4.7483766233766234e-04*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.2918019480519480e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13-3.7439123376623376e-04*w19*Jinv10+5.6615259740259744e-04*w34*Jinv10+1.3697240259740259e-04*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-7.8530844155844158e-04*w37*Jinv12+3.6525974025974026e-05*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+4.7483766233766234e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+1.3697240259740259e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11-3.1960227272727273e-04*Jinv20*w14+4.2918019480519480e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54+4.2918019480519480e-04*w4*Jinv12+1.0957792207792208e-04*w21*Jinv02+9.1314935064935065e-06*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-7.8530844155844158e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.4878246753246752e-04*w6*Jinv01+1.3697240259740259e-04*w47*Jinv21-3.9265422077922079e-04*Jinv20*w38+1.0957792207792208e-04*Jinv22*w23+4.7483766233766234e-04*w52*Jinv11+1.3697240259740259e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53-7.4878246753246752e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50+1.0957792207792208e-04*w22*Jinv11-7.4878246753246752e-04*Jinv12*w7+1.0957792207792208e-04*w21*Jinv01+4.7483766233766234e-04*Jinv01*w51+1.3697240259740259e-04*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-7.8530844155844158e-04*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10-7.4878246753246752e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00-3.9265422077922079e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-7.4878246753246752e-04*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+4.2918019480519480e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22-9.1314935064935065e-06*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12-7.4878246753246752e-04*Jinv21*w8-4.9310064935064932e-04*Jinv02*w33-1.1668019480519480e-04*w0*Jinv00+5.6615259740259744e-04*w35*Jinv20+1.0957792207792208e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+6.9399350649350649e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+6.9399350649350649e-04*Jinv20*w17-1.7349837662337662e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10-3.7439123376623376e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.8530844155844158e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+1.3697240259740259e-04*Jinv01*w30+4.7483766233766234e-04*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-2.7394480519480518e-04*w16*Jinv12-9.1314935064935065e-06*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-9.1314935064935065e-06*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[58][8]=(0.0)*D;
  A[58][9]=(0.0)*D;
  A[58][10]=(1.0247564935064935e-04*Jinv00*w12+5.4112554112554113e-05*w28*Jinv10+1.5726461038961039e-04*Jinv20*w8+1.0247564935064935e-04*w13*Jinv10-1.1668019480519480e-04*w3*Jinv00-1.4204545454545455e-05*w32*Jinv20+1.0247564935064935e-04*Jinv20*w23+6.4935064935064935e-05*Jinv00*w54-4.2613636363636367e-05*w43*Jinv10+1.8939393939393939e-04*w58*Jinv10+6.4935064935064935e-05*w48*Jinv00+5.4112554112554113e-05*w1*Jinv10-1.9480519480519481e-04*w16*Jinv10-1.4204545454545455e-05*w45*Jinv00+5.4112554112554113e-05*Jinv00*w27+6.4935064935064935e-05*Jinv20*w50+5.4112554112554113e-05*Jinv20*w29-1.4204545454545455e-05*w46*Jinv10-4.2613636363636367e-05*Jinv00*w33-1.4204545454545455e-05*w31*Jinv10+1.0247564935064935e-04*w21*Jinv00+1.5726461038961039e-04*Jinv00*w18+2.0495129870129870e-04*w36*Jinv00+8.3198051948051951e-05*w51*Jinv00-1.1668019480519480e-04*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+1.5726461038961039e-04*w19*Jinv10-4.2613636363636367e-05*w34*Jinv10+1.5726461038961039e-04*w6*Jinv00+6.4935064935064935e-05*Jinv20*w56-1.1668019480519480e-04*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20+1.0247564935064935e-04*Jinv20*w14+6.4935064935064935e-05*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+1.5726461038961039e-04*w7*Jinv10+2.0495129870129870e-04*Jinv20*w38+8.3198051948051951e-05*Jinv20*w53+1.0247564935064935e-04*w22*Jinv10+8.3198051948051951e-05*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+2.0495129870129870e-04*w37*Jinv10-1.1668019480519480e-04*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-4.2613636363636367e-05*Jinv20*w44+1.8939393939393939e-04*w57*Jinv00+1.8939393939393939e-04*Jinv20*w59+5.4112554112554113e-05*w0*Jinv00-4.2613636363636367e-05*w35*Jinv20-1.1668019480519480e-04*w25*Jinv10+5.4112554112554113e-05*Jinv20*w2-1.9480519480519481e-04*w15*Jinv00-1.9480519480519481e-04*Jinv20*w17-1.4204545454545455e-05*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+1.5726461038961039e-04*w20*Jinv20-1.1668019480519480e-04*w5*Jinv20+6.4935064935064935e-05*w55*Jinv10-4.2613636363636367e-05*w42*Jinv00-1.4204545454545455e-05*w47*Jinv20)*D;
  A[58][11]=(0.0)*D;
  A[58][12]=(0.0)*D;
  A[58][13]=(-7.4878246753246752e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-1.0592532467532468e-03*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53+3.1960227272727273e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02-7.4878246753246752e-04*w13*Jinv10-1.0592532467532468e-03*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44-3.7439123376623376e-04*w3*Jinv00-3.9265422077922079e-04*w32*Jinv20+3.7439123376623376e-04*Jinv01*w3+1.5523538961038962e-04*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38-1.8262987012987013e-05*Jinv00*w54+4.2918019480519480e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+1.5726461038961039e-04*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01-6.9399350649350649e-04*w16*Jinv11-6.9399350649350649e-04*Jinv21*w17+1.7349837662337662e-04*Jinv01*w45-1.8262987012987013e-05*Jinv12*w55-4.0584415584415584e-05*w58*Jinv10-9.6793831168831166e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12+4.8396915584415583e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+3.1960227272727273e-04*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-9.6793831168831166e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11+3.1047077922077924e-04*w45*Jinv00-5.6615259740259744e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22+1.5726461038961039e-04*Jinv11*w1-6.9399350649350649e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+4.8396915584415583e-04*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+3.1047077922077924e-04*w46*Jinv10-9.6793831168831166e-04*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.5523538961038962e-04*w22*Jinv12-3.9265422077922079e-04*w31*Jinv10-3.7439123376623376e-04*Jinv02*w3+1.5523538961038962e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.6615259740259744e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00+3.1047077922077924e-04*Jinv02*w45+3.1960227272727273e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-9.1314935064935065e-06*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-3.7439123376623376e-04*w4*Jinv10+3.7439123376623376e-04*w4*Jinv11-1.0592532467532468e-03*w41*Jinv20-7.4878246753246752e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.9265422077922079e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+3.1960227272727273e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12-1.8262987012987013e-05*Jinv20*w56-1.0592532467532468e-03*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35+3.9265422077922079e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.7349837662337662e-04*w46*Jinv11-7.4878246753246752e-04*Jinv20*w14+3.7439123376623376e-04*w5*Jinv21-1.8262987012987013e-05*Jinv02*w54-3.7439123376623376e-04*w4*Jinv12+1.5523538961038962e-04*w21*Jinv02+4.8396915584415583e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-1.0592532467532468e-03*w39*Jinv00-7.4878246753246752e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18-1.0247564935064935e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+3.1960227272727273e-04*w7*Jinv10+1.7349837662337662e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38+1.5523538961038962e-04*Jinv22*w23-3.9265422077922079e-04*w31*Jinv12+3.1047077922077924e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+3.1960227272727273e-04*Jinv22*w8+1.5523538961038962e-04*w22*Jinv10+4.8396915584415583e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+3.1960227272727273e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-3.9265422077922079e-04*Jinv02*w30+4.8396915584415583e-04*Jinv12*w49-9.1314935064935065e-06*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+3.1960227272727273e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-3.7439123376623376e-04*Jinv22*w5+3.1047077922077924e-04*w47*Jinv22+9.1314935064935065e-06*w49*Jinv11-1.0247564935064935e-04*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-1.0957792207792208e-04*Jinv21*w8-4.0584415584415584e-05*w57*Jinv00-4.0584415584415584e-05*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-9.6793831168831166e-04*w15*Jinv00+4.8396915584415583e-04*w48*Jinv02-9.6793831168831166e-04*Jinv20*w17-3.9265422077922079e-04*w30*Jinv00-1.0592532467532468e-03*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-7.4878246753246752e-04*Jinv22*w14-5.6615259740259744e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.7439123376623376e-04*w5*Jinv20+3.9265422077922079e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51-1.8262987012987013e-05*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+3.1960227272727273e-04*w25*Jinv11-1.8262987012987013e-05*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+3.1047077922077924e-04*w47*Jinv20-9.6793831168831166e-04*w16*Jinv12+9.1314935064935065e-06*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1)*D;
  A[58][14]=(0.0)*D;
  A[58][15]=(0.0)*D;
  A[58][16]=(9.6793831168831166e-04*Jinv00*w12-1.1323051948051949e-03*w40*Jinv12+9.8620129870129864e-04*Jinv22*w53-6.9399350649350649e-04*Jinv20*w8-4.8701298701298701e-04*w57*Jinv02+9.6793831168831166e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-1.1323051948051949e-03*Jinv21*w44+6.9399350649350649e-04*w3*Jinv00+5.2962662337662338e-04*w32*Jinv20+9.6793831168831166e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-4.9310064935064932e-04*Jinv22*w38-5.4788961038961036e-04*w25*Jinv12-2.2524350649350650e-04*w0*Jinv02-1.9480519480519481e-04*Jinv21*w2-1.1323051948051949e-03*w43*Jinv10-1.1323051948051949e-03*w42*Jinv01-5.2962662337662338e-04*Jinv01*w45+9.8620129870129864e-04*Jinv12*w55-2.7759740259740259e-03*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35-9.6793831168831166e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-1.9480519480519481e-04*w1*Jinv10-5.4788961038961036e-04*Jinv22*w20-1.1323051948051949e-03*w43*Jinv11-2.2646103896103898e-03*w42*Jinv02-3.0438311688311688e-05*w11*Jinv22-1.9480519480519481e-04*Jinv11*w1+6.9399350649350649e-04*w13*Jinv11+1.9480519480519481e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-2.7759740259740259e-03*Jinv22*w17-2.7394480519480518e-04*w22*Jinv12+5.2962662337662338e-04*w31*Jinv10+4.2004870129870131e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22-2.2524350649350650e-04*Jinv22*w2-9.6793831168831166e-04*Jinv00*w18-5.2962662337662338e-04*w36*Jinv00-4.9310064935064932e-04*Jinv02*w45-9.6793831168831166e-04*Jinv01*w24-4.8701298701298701e-04*Jinv22*w59+1.1323051948051949e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+4.9310064935064932e-04*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-2.2646103896103898e-03*Jinv22*w44+6.9399350649350649e-04*w4*Jinv10+9.6793831168831166e-04*w4*Jinv11+1.1323051948051949e-03*w41*Jinv20+4.2004870129870131e-04*Jinv12*w13-9.6793831168831166e-04*w19*Jinv10+5.2962662337662338e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-6.9399350649350649e-04*w6*Jinv00-4.9310064935064932e-04*w37*Jinv12-1.1323051948051949e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20+9.8620129870129864e-04*w52*Jinv12+1.1323051948051949e-03*Jinv21*w35+5.2962662337662338e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.2962662337662338e-04*w46*Jinv11+9.6793831168831166e-04*Jinv20*w14+9.6793831168831166e-04*w5*Jinv21+9.8620129870129864e-04*Jinv02*w54+4.2004870129870131e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+1.1323051948051949e-03*Jinv01*w33+1.1323051948051949e-03*w39*Jinv00+4.2004870129870131e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-3.0438311688311688e-05*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-6.9399350649350649e-04*w7*Jinv10-5.2962662337662338e-04*w47*Jinv21-5.2962662337662338e-04*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-4.9310064935064932e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50-6.9399350649350649e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7-6.9399350649350649e-04*w21*Jinv01+3.6525974025974026e-05*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+4.9310064935064932e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-5.2962662337662338e-04*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+4.2004870129870131e-04*Jinv22*w5-4.9310064935064932e-04*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+6.9399350649350649e-04*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10-1.1323051948051949e-03*Jinv20*w44-4.8701298701298701e-04*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+6.9399350649350649e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.1323051948051949e-03*Jinv02*w33-1.9480519480519481e-04*w0*Jinv00-6.9399350649350649e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2+4.9310064935064932e-04*w48*Jinv02+5.2962662337662338e-04*w30*Jinv00+1.1323051948051949e-03*w40*Jinv10-9.6793831168831166e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+4.2004870129870131e-04*Jinv22*w14-4.9310064935064932e-04*Jinv02*w36-2.2646103896103898e-03*Jinv12*w43+6.9399350649350649e-04*w5*Jinv20+5.2962662337662338e-04*Jinv01*w30+9.8620129870129864e-04*Jinv02*w51+9.8620129870129864e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-9.6793831168831166e-04*w25*Jinv11-1.1323051948051949e-03*w42*Jinv00-2.7759740259740259e-03*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-2.2524350649350650e-04*Jinv12*w1)*D;
  A[58][17]=(0.0)*D;
  A[58][18]=(0.0)*D;
  A[58][19]=(-2.1915584415584417e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10+3.7439123376623376e-04*Jinv20*w8-2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv21*w44-3.1960227272727273e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-4.2918019480519480e-04*Jinv01*w3-4.2918019480519480e-04*Jinv20*w23+1.8262987012987013e-05*Jinv00*w54+4.0584415584415584e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+1.0592532467532468e-03*w43*Jinv10+4.9310064935064932e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+2.7394480519480518e-04*w16*Jinv11+2.7394480519480518e-04*Jinv21*w17-1.3697240259740259e-04*Jinv01*w45+4.0584415584415584e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00+4.0584415584415584e-05*w58*Jinv11+4.0584415584415584e-05*w57*Jinv01-4.2918019480519480e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+9.6793831168831166e-04*w16*Jinv10+4.9310064935064932e-04*w43*Jinv11-3.1047077922077924e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-1.4204545454545455e-05*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-1.0957792207792208e-04*w13*Jinv11+2.7394480519480518e-04*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-3.1047077922077924e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+7.4878246753246752e-04*Jinv00*w18+3.9265422077922079e-04*w36*Jinv00+7.8530844155844158e-04*Jinv21*w38-4.2918019480519480e-04*Jinv01*w24+4.9310064935064932e-04*w34*Jinv11+7.4878246753246752e-04*w19*Jinv11-4.7483766233766234e-04*Jinv21*w53-4.8396915584415583e-04*w51*Jinv00-3.1960227272727273e-04*w4*Jinv10-4.2918019480519480e-04*w4*Jinv11-7.3051948051948052e-05*w41*Jinv20+7.4878246753246752e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21+7.4878246753246752e-04*Jinv21*w20+3.7439123376623376e-04*w6*Jinv00+1.8262987012987013e-05*Jinv20*w56-1.5523538961038962e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20+4.9310064935064932e-04*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11-1.3697240259740259e-04*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14-4.2918019480519480e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-7.3051948051948052e-05*w39*Jinv00+7.4878246753246752e-04*Jinv01*w18+7.8530844155844158e-04*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01+3.7439123376623376e-04*w7*Jinv10-1.3697240259740259e-04*w47*Jinv21+3.9265422077922079e-04*Jinv20*w38-4.7483766233766234e-04*w52*Jinv11-4.8396915584415583e-04*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01-4.7483766233766234e-04*Jinv01*w51+7.8530844155844158e-04*w36*Jinv01-4.8396915584415583e-04*w52*Jinv10+7.4878246753246752e-04*w7*Jinv11-9.2329545454545453e-05*w9*Jinv00+3.9265422077922079e-04*w37*Jinv10+9.1314935064935065e-06*Jinv21*w56-1.5523538961038962e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11-1.0957792207792208e-04*Jinv21*w14-9.2329545454545453e-05*w10*Jinv10+1.0592532467532468e-03*Jinv20*w44-1.0957792207792208e-04*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8+4.0584415584415584e-05*w57*Jinv00+4.0584415584415584e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20-1.0957792207792208e-04*Jinv21*w23+9.1314935064935065e-06*Jinv01*w54+9.1314935064935065e-06*w55*Jinv11-1.5523538961038962e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+9.6793831168831166e-04*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+9.6793831168831166e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+7.4878246753246752e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-3.1960227272727273e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30-4.2918019480519480e-04*w25*Jinv11+1.8262987012987013e-05*w55*Jinv10+1.0592532467532468e-03*w42*Jinv00-3.1047077922077924e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[58][20]=(0.0)*D;
  A[58][21]=(0.0)*D;
  A[58][22]=(4.2918019480519480e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53+1.0957792207792208e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02+4.2918019480519480e-04*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+1.3697240259740259e-04*w32*Jinv20-3.1960227272727273e-04*Jinv01*w3-7.4878246753246752e-04*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38+4.7483766233766234e-04*Jinv00*w54-7.4878246753246752e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.1668019480519480e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01+6.9399350649350649e-04*w16*Jinv11+6.9399350649350649e-04*Jinv21*w17-3.9265422077922079e-04*Jinv01*w45+4.7483766233766234e-04*Jinv12*w55-4.0584415584415584e-05*w58*Jinv10-2.7394480519480518e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-9.1314935064935065e-06*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-3.7439123376623376e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10-2.7394480519480518e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11-7.8530844155844158e-04*w45*Jinv00+5.6615259740259744e-04*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-1.1668019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11+6.9399350649350649e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11-9.1314935064935065e-06*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.8530844155844158e-04*w46*Jinv10-2.7394480519480518e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.4878246753246752e-04*w22*Jinv12+1.3697240259740259e-04*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3-7.4878246753246752e-04*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22+5.6615259740259744e-04*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00-7.8530844155844158e-04*Jinv02*w45-3.7439123376623376e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-9.1314935064935065e-06*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10-3.1960227272727273e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20+4.2918019480519480e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-1.7349837662337662e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+1.0957792207792208e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12+4.7483766233766234e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35-1.7349837662337662e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-3.9265422077922079e-04*w46*Jinv11+4.2918019480519480e-04*Jinv20*w14-3.1960227272727273e-04*w5*Jinv21+4.7483766233766234e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12-7.4878246753246752e-04*w21*Jinv02-9.1314935064935065e-06*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00+4.2918019480519480e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18-1.0247564935064935e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.0957792207792208e-04*w7*Jinv10-3.9265422077922079e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38-7.4878246753246752e-04*Jinv22*w23+1.3697240259740259e-04*w31*Jinv12-7.8530844155844158e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+1.0957792207792208e-04*Jinv22*w8-7.4878246753246752e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50+1.0957792207792208e-04*Jinv12*w7+1.3697240259740259e-04*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-9.1314935064935065e-06*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+1.0957792207792208e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5-7.8530844155844158e-04*w47*Jinv22+9.1314935064935065e-06*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-4.0584415584415584e-05*w57*Jinv00-4.0584415584415584e-05*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2-2.7394480519480518e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-2.7394480519480518e-04*Jinv20*w17+1.3697240259740259e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.2918019480519480e-04*Jinv22*w14+5.6615259740259744e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20-1.7349837662337662e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51+4.7483766233766234e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.7439123376623376e-04*w25*Jinv11+4.7483766233766234e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00-7.8530844155844158e-04*w47*Jinv20-2.7394480519480518e-04*w16*Jinv12+9.1314935064935065e-06*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1)*D;
  A[58][23]=(0.0)*D;
  A[58][24]=(0.0)*D;
  A[58][25]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+1.0592532467532468e-03*Jinv21*w44-1.0957792207792208e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3+7.4878246753246752e-04*Jinv20*w23-4.7483766233766234e-04*Jinv00*w54+4.0584415584415584e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w43*Jinv10+1.0592532467532468e-03*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+9.6793831168831166e-04*w16*Jinv11+9.6793831168831166e-04*Jinv21*w17+3.9265422077922079e-04*Jinv01*w45+4.0584415584415584e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00+4.0584415584415584e-05*w58*Jinv11+4.0584415584415584e-05*w57*Jinv01+7.4878246753246752e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+2.7394480519480518e-04*w16*Jinv10+1.0592532467532468e-03*w43*Jinv11+7.8530844155844158e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-3.1960227272727273e-04*w13*Jinv11+9.6793831168831166e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.8530844155844158e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.2918019480519480e-04*Jinv00*w18-1.3697240259740259e-04*w36*Jinv00-3.1047077922077924e-04*Jinv21*w38+7.4878246753246752e-04*Jinv01*w24-7.3051948051948052e-05*w34*Jinv11-1.5523538961038962e-04*w19*Jinv11+1.8262987012987013e-05*Jinv21*w53+9.1314935064935065e-06*w51*Jinv00-1.0957792207792208e-04*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+4.9310064935064932e-04*w41*Jinv20-4.2918019480519480e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21-1.5523538961038962e-04*Jinv21*w20-1.0957792207792208e-04*w6*Jinv00-4.7483766233766234e-04*Jinv20*w56+7.4878246753246752e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20-7.3051948051948052e-05*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11+3.9265422077922079e-04*w46*Jinv11-4.2918019480519480e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33+4.9310064935064932e-04*w39*Jinv00-1.5523538961038962e-04*Jinv01*w18-3.1047077922077924e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-1.0957792207792208e-04*w7*Jinv10+3.9265422077922079e-04*w47*Jinv21-1.3697240259740259e-04*Jinv20*w38+1.8262987012987013e-05*w52*Jinv11+9.1314935064935065e-06*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+3.7439123376623376e-04*w22*Jinv11+3.7439123376623376e-04*w21*Jinv01+1.8262987012987013e-05*Jinv01*w51-3.1047077922077924e-04*w36*Jinv01+9.1314935064935065e-06*w52*Jinv10-4.2918019480519480e-04*w7*Jinv11+1.0247564935064935e-04*w9*Jinv00-1.3697240259740259e-04*w37*Jinv10-4.8396915584415583e-04*Jinv21*w56+7.4878246753246752e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11-3.1960227272727273e-04*Jinv21*w14+1.0247564935064935e-04*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-3.1960227272727273e-04*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8+4.0584415584415584e-05*w57*Jinv00+4.0584415584415584e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.7439123376623376e-04*Jinv21*w23-4.8396915584415583e-04*Jinv01*w54-4.8396915584415583e-04*w55*Jinv11+7.4878246753246752e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+2.7394480519480518e-04*w15*Jinv00-1.4204545454545455e-05*Jinv01*w9+2.7394480519480518e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-4.2918019480519480e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11-1.0957792207792208e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30+7.4878246753246752e-04*w25*Jinv11-4.7483766233766234e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00+7.8530844155844158e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[58][26]=(0.0)*D;
  A[58][27]=(0.0)*D;
  A[58][28]=(-4.2613636363636367e-05*Jinv21*w44+1.0247564935064935e-04*Jinv01*w3+1.8939393939393939e-04*Jinv21*w59+5.4112554112554113e-05*Jinv21*w2-4.2613636363636367e-05*w42*Jinv01+5.4112554112554113e-05*w11*Jinv21-1.9480519480519481e-04*w16*Jinv11-1.9480519480519481e-04*Jinv21*w17+2.0495129870129870e-04*Jinv01*w45+1.8939393939393939e-04*w58*Jinv11+1.8939393939393939e-04*w57*Jinv01+1.5726461038961039e-04*Jinv21*w26+5.4112554112554113e-05*w0*Jinv01-4.2613636363636367e-05*w43*Jinv11-4.2613636363636367e-05*Jinv21*w41+5.4112554112554113e-05*Jinv11*w1-1.1668019480519480e-04*w13*Jinv11-1.9480519480519481e-04*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-4.2613636363636367e-05*Jinv01*w39-1.4204545454545455e-05*Jinv21*w38+1.5726461038961039e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11-1.1668019480519480e-04*w19*Jinv11+6.4935064935064935e-05*Jinv21*w53+1.0247564935064935e-04*w4*Jinv11-1.4204545454545455e-05*w32*Jinv21-1.1668019480519480e-04*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35-1.4204545454545455e-05*w31*Jinv11+2.0495129870129870e-04*w46*Jinv11+1.0247564935064935e-04*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33-1.1668019480519480e-04*Jinv01*w18-1.4204545454545455e-05*w37*Jinv11+1.0247564935064935e-04*w6*Jinv01+2.0495129870129870e-04*w47*Jinv21+6.4935064935064935e-05*w52*Jinv11+1.5726461038961039e-04*w22*Jinv11+1.5726461038961039e-04*w21*Jinv01+6.4935064935064935e-05*Jinv01*w51-1.4204545454545455e-05*w36*Jinv01+1.0247564935064935e-04*w7*Jinv11+8.3198051948051951e-05*Jinv21*w56+6.4935064935064935e-05*w49*Jinv11-1.1668019480519480e-04*Jinv21*w14-1.1668019480519480e-04*Jinv01*w12+1.0247564935064935e-04*Jinv21*w8+1.5726461038961039e-04*Jinv21*w23+8.3198051948051951e-05*Jinv01*w54+8.3198051948051951e-05*w55*Jinv11+5.4112554112554113e-05*Jinv01*w9-4.2613636363636367e-05*w40*Jinv11-1.4204545454545455e-05*Jinv01*w30+1.5726461038961039e-04*w25*Jinv11+6.4935064935064935e-05*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27+6.4935064935064935e-05*w48*Jinv01+5.4112554112554113e-05*w10*Jinv11)*D;
  A[58][29]=(0.0)*D;
  A[58][30]=(0.0)*D;
  A[58][31]=(-3.9265422077922079e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12+5.5702110389610384e-04*Jinv22*w53+1.7349837662337662e-04*Jinv20*w8+6.4935064935064935e-04*w57*Jinv02-3.9265422077922079e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-3.9265422077922079e-04*Jinv01*w3+1.7349837662337662e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+6.0876623376623375e-04*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12+1.2175324675324675e-04*w0*Jinv02+5.6818181818181818e-05*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45+5.5702110389610384e-04*Jinv12*w55+6.0876623376623375e-04*w58*Jinv10-2.5568181818181820e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+6.0876623376623375e-04*w58*Jinv11+6.0876623376623375e-04*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26+5.6818181818181818e-05*w0*Jinv01+5.6818181818181818e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-2.5568181818181820e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+5.6818181818181818e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11+1.0957792207792207e-03*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10-2.5568181818181820e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33-2.5568181818181820e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+1.7349837662337662e-04*w21*Jinv00+1.2784090909090910e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39+1.2175324675324675e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24+6.4935064935064935e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34-2.5568181818181820e-04*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10-3.9265422077922079e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+1.7349837662337662e-04*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+5.5702110389610384e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11-3.9265422077922079e-04*Jinv20*w14-3.9265422077922079e-04*w5*Jinv21+5.5702110389610384e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.5568181818181820e-04*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9+1.7349837662337662e-04*w6*Jinv01+1.7349837662337662e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38-2.5568181818181820e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11+1.2784090909090910e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53-2.5568181818181820e-04*Jinv22*w8+1.7349837662337662e-04*w22*Jinv10+6.2094155844155847e-04*Jinv22*w50+1.7349837662337662e-04*w22*Jinv11-2.5568181818181820e-04*Jinv12*w7+1.7349837662337662e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+1.2784090909090910e-04*Jinv02*w30+6.2094155844155847e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10+1.7349837662337662e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27-2.5568181818181820e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+6.4935064935064935e-04*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.7349837662337662e-04*Jinv21*w8+6.0876623376623375e-04*w57*Jinv00+6.0876623376623375e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+5.6818181818181818e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+1.7349837662337662e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10+5.6818181818181818e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00+6.2094155844155847e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36-2.5568181818181820e-04*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30+5.5702110389610384e-04*Jinv02*w51+5.5702110389610384e-04*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28+1.3697240259740259e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20-2.5568181818181820e-04*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01+1.2175324675324675e-04*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[58][32]=(0.0)*D;
  A[58][33]=(0.0)*D;
  A[58][34]=(5.2962662337662338e-04*Jinv00*w12+9.1314935064935063e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53-5.6615259740259744e-04*Jinv20*w8-2.4350649350649351e-04*w57*Jinv02+5.2962662337662338e-04*w13*Jinv10+9.1314935064935063e-04*Jinv02*w39-4.3831168831168834e-04*Jinv21*w44+5.6615259740259744e-04*w3*Jinv00+4.3831168831168834e-04*w32*Jinv20+7.3051948051948052e-05*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-7.3051948051948055e-04*Jinv21*w59-3.4699675324675324e-04*w25*Jinv12-3.0438311688311688e-05*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-4.3831168831168834e-04*w42*Jinv01+1.2175324675324675e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55+1.6436688311688311e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-3.4699675324675324e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35-7.3051948051948055e-04*w58*Jinv11-7.3051948051948055e-04*w57*Jinv01+7.3051948051948052e-05*Jinv21*w26-3.0438311688311688e-05*w0*Jinv01-4.2613636363636367e-05*w1*Jinv10-2.1915584415584417e-04*Jinv22*w20-4.3831168831168834e-04*w43*Jinv11-2.1915584415584417e-04*Jinv21*w41+6.9399350649350649e-04*w42*Jinv02+4.2613636363636367e-05*w11*Jinv22-3.0438311688311688e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50+1.6436688311688311e-03*Jinv22*w17-3.8352272727272725e-04*w22*Jinv12+4.3831168831168834e-04*w31*Jinv10+6.2094155844155847e-04*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-5.2962662337662338e-04*Jinv00*w18-4.3831168831168834e-04*w36*Jinv00-6.5746753246753243e-04*Jinv21*w38-2.5568181818181820e-04*Jinv02*w45+7.3051948051948052e-05*Jinv01*w24-2.4350649350649351e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-4.9310064935064932e-04*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+6.9399350649350649e-04*Jinv22*w44+5.6615259740259744e-04*w4*Jinv10+7.3051948051948052e-05*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20+3.1047077922077924e-04*Jinv12*w13-5.2962662337662338e-04*w19*Jinv10-2.1915584415584417e-04*w32*Jinv21-4.9310064935064932e-04*Jinv21*w20-5.6615259740259744e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12+9.1314935064935063e-04*Jinv22*w41+3.6525974025974026e-05*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-3.4699675324675324e-04*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11+5.2962662337662338e-04*Jinv20*w14+7.3051948051948052e-05*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+6.2094155844155847e-04*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00+3.1047077922077924e-04*Jinv02*w12-4.9310064935064932e-04*Jinv01*w18-6.5746753246753243e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-5.6615259740259744e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-4.3831168831168834e-04*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-2.5568181818181820e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49-6.5746753246753243e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-4.3831168831168834e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+3.6525974025974026e-05*w24*Jinv00+1.9480519480519481e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+6.2094155844155847e-04*Jinv22*w5-2.5568181818181820e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-2.4350649350649351e-04*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+3.6525974025974026e-05*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8+1.8262987012987013e-03*Jinv02*w33-4.2613636363636367e-05*w0*Jinv00+3.6525974025974026e-05*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+3.6525974025974026e-05*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-4.3831168831168834e-04*w48*Jinv02+1.2175324675324675e-05*Jinv01*w9+4.3831168831168834e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-5.2962662337662338e-04*w20*Jinv20+1.9480519480519481e-04*Jinv22*w29+3.1047077922077924e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+6.9399350649350649e-04*Jinv12*w43+5.6615259740259744e-04*w5*Jinv20-2.1915584415584417e-04*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+1.9480519480519481e-04*Jinv12*w28+7.3051948051948052e-05*w25*Jinv11-2.1915584415584417e-04*w42*Jinv00+1.6436688311688311e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+1.2175324675324675e-05*w10*Jinv11)*D;
  A[58][35]=(0.0)*D;
  A[58][36]=(0.0)*D;
  A[58][37]=(-1.3697240259740259e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.7483766233766234e-04*Jinv22*w53+3.9265422077922079e-04*Jinv20*w8+4.0584415584415584e-05*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-1.7349837662337662e-04*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23+7.8530844155844158e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-4.2918019480519480e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+4.3831168831168834e-04*w43*Jinv10+9.1314935064935065e-06*Jinv12*w55-6.0876623376623375e-04*w58*Jinv10+2.7394480519480518e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-4.2918019480519480e-04*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35-1.4204545454545455e-05*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.4878246753246752e-04*Jinv22*w20+2.1915584415584417e-04*w45*Jinv00+4.9310064935064932e-04*w42*Jinv02-1.4204545454545455e-05*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-5.4788961038961036e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10+2.7394480519480518e-04*Jinv22*w17+4.3831168831168834e-04*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+2.1915584415584417e-04*w31*Jinv10-4.2918019480519480e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-1.3697240259740259e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.9265422077922079e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-1.3697240259740259e-04*Jinv02*w45+4.0584415584415584e-05*Jinv22*w59-1.0957792207792207e-03*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.7349837662337662e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13+3.9265422077922079e-04*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10+3.9265422077922079e-04*w6*Jinv00+7.8530844155844158e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-1.7349837662337662e-04*w26*Jinv20-5.6818181818181818e-05*w11*Jinv20-4.7483766233766234e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14+9.1314935064935065e-06*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.9265422077922079e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-1.3697240259740259e-04*w31*Jinv12-1.3697240259740259e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+7.4878246753246752e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+9.1314935064935065e-06*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-1.3697240259740259e-04*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49-1.0957792207792207e-03*w52*Jinv10-5.6818181818181818e-05*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10-1.7349837662337662e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5-1.3697240259740259e-04*w47*Jinv22-5.6818181818181818e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+4.0584415584415584e-05*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18-6.0876623376623375e-04*w57*Jinv00-6.0876623376623375e-04*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-1.4204545454545455e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-1.7349837662337662e-04*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00+9.1314935064935065e-06*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10+3.9265422077922079e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14+7.8530844155844158e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-1.7349837662337662e-04*w5*Jinv20-4.7483766233766234e-04*Jinv02*w51+9.1314935064935065e-06*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28-5.4788961038961036e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20+2.7394480519480518e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[58][38]=(0.0)*D;
  A[58][39]=(0.0)*D;
  A[58][40]=(7.3051948051948052e-05*Jinv00*w12+1.2175324675324675e-05*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8-2.4350649350649351e-04*w57*Jinv02+7.3051948051948052e-05*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+5.2962662337662338e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-2.5568181818181820e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-2.1915584415584417e-04*w25*Jinv12-4.2613636363636367e-05*Jinv21*w2-4.3831168831168834e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-4.3831168831168834e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-7.3051948051948055e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+1.9480519480519481e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+9.1314935064935063e-04*Jinv22*w35-5.2962662337662338e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-3.0438311688311688e-05*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10-3.4699675324675324e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-6.5746753246753243e-04*w45*Jinv00+6.9399350649350649e-04*w42*Jinv02+1.2175324675324675e-05*Jinv00*w27+1.9480519480519481e-04*w11*Jinv22-4.2613636363636367e-05*Jinv11*w1+5.6615259740259744e-04*w13*Jinv11+4.2613636363636367e-05*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+1.2175324675324675e-05*Jinv20*w29-6.5746753246753243e-04*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10+3.1047077922077924e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22+7.3051948051948052e-05*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45-5.2962662337662338e-04*Jinv01*w24-2.4350649350649351e-04*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11+5.4788961038961036e-04*w51*Jinv00+9.1314935064935063e-04*Jinv12*w34+6.9399350649350649e-04*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+5.2962662337662338e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+6.2094155844155847e-04*Jinv12*w13+7.3051948051948052e-05*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+4.3831168831168834e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00-2.5568181818181820e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-4.9310064935064932e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+4.3831168831168834e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26-4.3831168831168834e-04*w46*Jinv11+7.3051948051948052e-05*Jinv20*w14+5.2962662337662338e-04*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54+3.1047077922077924e-04*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+6.2094155844155847e-04*Jinv02*w12+3.6525974025974026e-05*Jinv01*w18+1.9480519480519481e-04*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10-4.3831168831168834e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-5.6615259740259744e-04*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-5.6615259740259744e-04*w21*Jinv01+2.7394480519480518e-04*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+5.4788961038961036e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56-4.9310064935064932e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-3.4699675324675324e-04*w19*Jinv12+3.1047077922077924e-04*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11+5.6615259740259744e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-4.3831168831168834e-04*Jinv20*w44-2.4350649350649351e-04*w58*Jinv12-3.4699675324675324e-04*Jinv02*w18+5.6615259740259744e-04*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8-7.3051948051948055e-04*w57*Jinv00-7.3051948051948055e-04*Jinv20*w59+9.1314935064935063e-04*Jinv02*w33-3.0438311688311688e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-5.6615259740259744e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11-4.9310064935064932e-04*w25*Jinv10-3.0438311688311688e-05*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02-1.1323051948051949e-03*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+7.3051948051948052e-05*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29+6.2094155844155847e-04*Jinv22*w14-2.5568181818181820e-04*Jinv02*w36+6.9399350649350649e-04*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+4.3831168831168834e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-5.2962662337662338e-04*w25*Jinv11+1.0957792207792207e-03*w55*Jinv10-4.3831168831168834e-04*w42*Jinv00-6.5746753246753243e-04*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01)*D;
  A[58][41]=(0.0)*D;
  A[58][42]=(0.0)*D;
  A[58][43]=(-7.3051948051948052e-05*Jinv00*w12-1.2175324675324675e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12-9.8620129870129864e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+4.8701298701298701e-04*w57*Jinv02-7.3051948051948052e-05*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20-7.3051948051948052e-05*Jinv01*w3+4.9310064935064932e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+7.3051948051948055e-04*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+2.2524350649350650e-04*w0*Jinv02+3.0438311688311688e-05*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-1.2175324675324675e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45-9.8620129870129864e-04*Jinv12*w55+7.3051948051948055e-04*w58*Jinv10+2.7759740259740259e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+7.3051948051948055e-04*w58*Jinv11+7.3051948051948055e-04*w57*Jinv01-7.3051948051948052e-05*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01+3.0438311688311688e-05*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+6.5746753246753243e-04*w45*Jinv00+2.1915584415584417e-04*Jinv21*w41+2.2646103896103898e-03*w42*Jinv02-1.2175324675324675e-05*Jinv00*w27+3.0438311688311688e-05*w11*Jinv22+3.0438311688311688e-05*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+3.0438311688311688e-05*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50-1.2175324675324675e-05*Jinv20*w29+6.5746753246753243e-04*w46*Jinv10+2.7759740259740259e-03*Jinv22*w17+2.1915584415584417e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+2.1915584415584417e-04*w31*Jinv10-4.2004870129870131e-04*Jinv02*w3+4.9310064935064932e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+2.1915584415584417e-04*Jinv01*w39+2.2524350649350650e-04*Jinv22*w2-7.3051948051948052e-05*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+6.5746753246753243e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45-7.3051948051948052e-05*Jinv01*w24+4.8701298701298701e-04*Jinv22*w59+4.3831168831168834e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+2.2646103896103898e-03*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10-7.3051948051948052e-05*w4*Jinv11+4.3831168831168834e-04*w41*Jinv20-4.2004870129870131e-04*Jinv12*w13-7.3051948051948052e-05*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10+2.1915584415584417e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+3.0438311688311688e-05*w11*Jinv20-9.8620129870129864e-04*w52*Jinv12+4.3831168831168834e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11-7.3051948051948052e-05*Jinv20*w14-7.3051948051948052e-05*w5*Jinv21-9.8620129870129864e-04*Jinv02*w54-4.2004870129870131e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+3.0438311688311688e-05*Jinv21*w29+4.3831168831168834e-04*Jinv01*w33+4.3831168831168834e-04*w39*Jinv00-4.2004870129870131e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18+6.5746753246753243e-04*w37*Jinv11+3.0438311688311688e-05*Jinv02*w9+4.9310064935064932e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8+4.9310064935064932e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49+6.5746753246753243e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+4.9310064935064932e-04*w7*Jinv11+3.0438311688311688e-05*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00+3.0438311688311688e-05*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-4.2004870129870131e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+3.0438311688311688e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+4.8701298701298701e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12+4.9310064935064932e-04*Jinv21*w8+7.3051948051948055e-04*w57*Jinv00+7.3051948051948055e-04*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+3.0438311688311688e-05*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02-1.2175324675324675e-05*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00+4.3831168831168834e-04*w40*Jinv10-7.3051948051948052e-05*w20*Jinv20+3.0438311688311688e-05*Jinv22*w29-4.2004870129870131e-04*Jinv22*w14+2.1915584415584417e-04*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+2.2646103896103898e-03*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20+2.1915584415584417e-04*Jinv01*w30-9.8620129870129864e-04*Jinv02*w51-9.8620129870129864e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28-7.3051948051948052e-05*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+6.5746753246753243e-04*w47*Jinv20+2.7759740259740259e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50+3.0438311688311688e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01+2.2524350649350650e-04*Jinv12*w1-1.2175324675324675e-05*w10*Jinv11)*D;
  A[58][44]=(0.0)*D;
  A[58][45]=(0.0)*D;
  A[58][46]=(4.9310064935064932e-04*w40*Jinv12+9.1314935064935065e-06*Jinv22*w53+4.0584415584415584e-05*w57*Jinv02+4.9310064935064932e-04*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3-1.3697240259740259e-04*Jinv22*w38-6.0876623376623375e-04*Jinv21*w59+7.4878246753246752e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-1.4204545454545455e-05*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.7483766233766234e-04*Jinv12*w55+2.7394480519480518e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-6.0876623376623375e-04*w58*Jinv11-6.0876623376623375e-04*w57*Jinv01+3.9265422077922079e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01-4.2918019480519480e-04*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41+4.9310064935064932e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22-1.4204545454545455e-05*Jinv11*w1-1.7349837662337662e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-5.6818181818181818e-05*w28*Jinv11+2.7394480519480518e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-1.3697240259740259e-04*w32*Jinv22+4.3831168831168834e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2+2.1915584415584417e-04*Jinv21*w38+7.8530844155844158e-04*Jinv02*w45+3.9265422077922079e-04*Jinv01*w24+4.0584415584415584e-05*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11-1.7349837662337662e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-4.2918019480519480e-04*Jinv12*w13+2.1915584415584417e-04*w32*Jinv21-1.7349837662337662e-04*Jinv21*w20-1.3697240259740259e-04*w37*Jinv12+4.9310064935064932e-04*Jinv22*w41+9.1314935064935065e-06*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+7.4878246753246752e-04*Jinv22*w26+6.5746753246753243e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21-4.7483766233766234e-04*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-5.6818181818181818e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-4.2918019480519480e-04*Jinv02*w12-1.7349837662337662e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+7.4878246753246752e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11-1.3697240259740259e-04*w31*Jinv12+7.8530844155844158e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+9.1314935064935065e-06*Jinv22*w50+3.9265422077922079e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+3.9265422077922079e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51-1.3697240259740259e-04*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11-1.0957792207792207e-03*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5+7.8530844155844158e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11-1.7349837662337662e-04*Jinv21*w14+4.0584415584415584e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18-1.7349837662337662e-04*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.9265422077922079e-04*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+9.1314935064935065e-06*w48*Jinv02-1.4204545454545455e-05*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+4.3831168831168834e-04*w40*Jinv11-1.3697240259740259e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43+2.1915584415584417e-04*Jinv01*w30+9.1314935064935065e-06*Jinv02*w51-4.7483766233766234e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.9265422077922079e-04*w25*Jinv11+2.7394480519480518e-04*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50-5.6818181818181818e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1-1.4204545454545455e-05*w10*Jinv11)*D;
  A[58][47]=(0.0)*D;
  A[58][48]=(0.0)*D;
  A[58][49]=(-9.1314935064935065e-06*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12-1.1414366883116883e-03*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.2987012987012987e-03*w57*Jinv02-9.1314935064935065e-06*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39+5.4788961038961036e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-9.1314935064935065e-06*Jinv01*w3-9.1314935064935065e-06*Jinv20*w23+6.6659902597402602e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54-1.5827922077922079e-03*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-6.4935064935064935e-05*Jinv21*w2+5.4788961038961036e-04*w43*Jinv10+5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17+5.4788961038961036e-04*Jinv01*w45-1.1414366883116883e-03*Jinv12*w55-1.5827922077922079e-03*w58*Jinv10-1.8262987012987014e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12-1.0957792207792207e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-1.5827922077922079e-03*w58*Jinv11-1.5827922077922079e-03*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26-6.4935064935064935e-05*w0*Jinv01-6.4935064935064935e-05*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+5.4788961038961036e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+5.4788961038961036e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11-1.0957792207792207e-03*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-1.8262987012987014e-04*Jinv22*w17+5.4788961038961036e-04*Jinv00*w33+1.1870941558441559e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3-9.1314935064935065e-06*w21*Jinv00+4.0178571428571428e-04*w32*Jinv22+5.4788961038961036e-04*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18+5.4788961038961036e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+6.6659902597402602e-04*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24-1.2987012987012987e-03*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-9.1314935064935065e-06*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20-2.1915584415584417e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10+5.4788961038961036e-04*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00+6.6659902597402602e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20-1.1414366883116883e-03*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+5.4788961038961036e-04*w46*Jinv11-9.1314935064935065e-06*Jinv20*w14-9.1314935064935065e-06*w5*Jinv21-1.1414366883116883e-03*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+1.1870941558441559e-04*w21*Jinv02-1.0957792207792207e-03*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00-2.1915584415584417e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9-9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10+5.4788961038961036e-04*w47*Jinv21+5.4788961038961036e-04*Jinv20*w38+1.1870941558441559e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11+4.0178571428571428e-04*w31*Jinv12+6.6659902597402602e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+1.1870941558441559e-04*Jinv22*w8-9.1314935064935065e-06*w22*Jinv10-1.1870941558441558e-03*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11+1.1870941558441559e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51+4.0178571428571428e-04*Jinv02*w30-1.1870941558441558e-03*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10-9.1314935064935065e-06*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00+5.4788961038961036e-04*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+1.1870941558441559e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5+6.6659902597402602e-04*w47*Jinv22-1.0957792207792207e-03*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10+5.4788961038961036e-04*Jinv20*w44-1.2987012987012987e-03*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-9.1314935064935065e-06*Jinv21*w8-1.5827922077922079e-03*w57*Jinv00-1.5827922077922079e-03*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-6.4935064935064935e-05*w0*Jinv00+5.4788961038961036e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-6.4935064935064935e-05*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-1.1870941558441558e-03*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+5.4788961038961036e-04*w40*Jinv11+6.6659902597402602e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30-1.1414366883116883e-03*Jinv02*w51-1.1414366883116883e-03*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28-9.1314935064935065e-06*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+5.4788961038961036e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-1.8262987012987014e-04*w16*Jinv12-1.0957792207792207e-03*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-1.0957792207792207e-03*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[58][50]=(0.0)*D;
  A[58][51]=(0.0)*D;
  A[58][52]=(9.1314935064935065e-06*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12-9.1314935064935068e-05*Jinv22*w53+9.1314935064935065e-06*Jinv20*w8+2.8409090909090908e-04*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+9.1314935064935065e-06*w3*Jinv00-5.4788961038961036e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23-1.4610389610389610e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54+1.2784090909090910e-04*w25*Jinv12-9.2329545454545453e-05*w0*Jinv02-5.4788961038961036e-04*w43*Jinv10-4.5657467532467534e-05*Jinv12*w55+1.5827922077922079e-03*w58*Jinv10-6.7573051948051951e-04*w15*Jinv02-5.6818181818181818e-05*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+1.2784090909090910e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35+6.4935064935064935e-05*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10-2.1002435064935065e-04*Jinv22*w20-5.4788961038961036e-04*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02+6.4935064935064935e-05*Jinv00*w27-5.6818181818181818e-05*w11*Jinv22+1.0957792207792207e-03*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-6.7573051948051951e-04*Jinv22*w17-5.4788961038961036e-04*Jinv00*w33+6.3920454545454550e-05*w22*Jinv12-5.4788961038961036e-04*w31*Jinv10+1.2784090909090910e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00+1.1870941558441559e-04*w32*Jinv22-9.2329545454545453e-05*Jinv22*w2+9.1314935064935065e-06*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00+1.1870941558441559e-04*Jinv02*w45+2.8409090909090908e-04*Jinv22*w59+1.0957792207792207e-03*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv10-5.4788961038961036e-04*w41*Jinv20+6.3920454545454550e-05*Jinv12*w13+9.1314935064935065e-06*w19*Jinv10-5.4788961038961036e-04*w34*Jinv10+9.1314935064935065e-06*w6*Jinv00-1.4610389610389610e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+9.1314935064935065e-06*w26*Jinv20+6.4935064935064935e-05*w11*Jinv20-9.1314935064935068e-05*w52*Jinv12+1.2784090909090910e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14-4.5657467532467534e-05*Jinv02*w54+1.2784090909090910e-04*w4*Jinv12+6.3920454545454550e-05*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-5.4788961038961036e-04*w39*Jinv00+6.3920454545454550e-05*Jinv02*w12-5.6818181818181818e-05*Jinv02*w9+9.1314935064935065e-06*w7*Jinv10-5.4788961038961036e-04*Jinv20*w38+6.3920454545454550e-05*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12+1.0957792207792207e-03*Jinv20*w53-2.1002435064935065e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10-4.5657467532467534e-05*Jinv22*w50-2.1002435064935065e-04*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-4.5657467532467534e-05*Jinv12*w49+1.0957792207792207e-03*w52*Jinv10+6.4935064935064935e-05*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+9.1314935064935065e-06*w24*Jinv00-9.2329545454545453e-05*Jinv02*w27-2.1002435064935065e-04*w6*Jinv02-2.1002435064935065e-04*w19*Jinv12+1.2784090909090910e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+2.8409090909090908e-04*w58*Jinv12-2.1002435064935065e-04*Jinv02*w18+1.5827922077922079e-03*w57*Jinv00+1.5827922077922079e-03*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33+6.4935064935064935e-05*w0*Jinv00-5.4788961038961036e-04*w35*Jinv20+9.1314935064935065e-06*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00-4.5657467532467534e-05*w48*Jinv02-4.9310064935064932e-04*Jinv20*w17-5.4788961038961036e-04*w30*Jinv00-5.4788961038961036e-04*w40*Jinv10+9.1314935064935065e-06*w20*Jinv20-9.2329545454545453e-05*Jinv22*w29+6.3920454545454550e-05*Jinv22*w14-1.4610389610389610e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+9.1314935064935065e-06*w5*Jinv20-9.1314935064935068e-05*Jinv02*w51-4.5657467532467534e-05*Jinv22*w56-9.2329545454545453e-05*Jinv12*w28+1.0957792207792207e-03*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-6.7573051948051951e-04*w16*Jinv12-9.2329545454545453e-05*Jinv12*w1)*D;
  A[58][53]=(0.0)*D;
  A[58][54]=(0.0)*D;
  A[58][55]=(-7.6704545454545450e-04*w40*Jinv12-4.5657467532467534e-05*Jinv22*w53+2.8409090909090908e-04*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3+1.1870941558441559e-04*Jinv22*w38+1.5827922077922079e-03*Jinv21*w59-2.1002435064935065e-04*w25*Jinv12-9.2329545454545453e-05*w0*Jinv02+6.4935064935064935e-05*Jinv21*w2-5.4788961038961036e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-9.1314935064935068e-05*Jinv12*w55-6.7573051948051951e-04*w15*Jinv02-9.2329545454545453e-05*w10*Jinv12-2.1002435064935065e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35+1.5827922077922079e-03*w58*Jinv11+1.5827922077922079e-03*w57*Jinv01+9.1314935064935065e-06*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+1.2784090909090910e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-5.4788961038961036e-04*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02-9.2329545454545453e-05*w11*Jinv22+6.4935064935064935e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01+6.4935064935064935e-05*w28*Jinv11-6.7573051948051951e-04*Jinv22*w17-2.1002435064935065e-04*w22*Jinv12+6.3920454545454550e-05*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-5.4788961038961036e-04*Jinv01*w39-9.2329545454545453e-05*Jinv22*w2-5.4788961038961036e-04*Jinv21*w38-1.4610389610389610e-04*Jinv02*w45+9.1314935064935065e-06*Jinv01*w24+2.8409090909090908e-04*Jinv22*w59-5.4788961038961036e-04*w34*Jinv11+9.1314935064935065e-06*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11+1.2784090909090910e-04*Jinv12*w13-5.4788961038961036e-04*w32*Jinv21+9.1314935064935065e-06*Jinv21*w20+1.1870941558441559e-04*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41-4.5657467532467534e-05*w52*Jinv12-5.4788961038961036e-04*Jinv21*w35-5.4788961038961036e-04*w31*Jinv11-2.1002435064935065e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21-9.1314935064935068e-05*Jinv02*w54+6.3920454545454550e-05*w4*Jinv12-2.1002435064935065e-04*w21*Jinv02+6.4935064935064935e-05*Jinv21*w29-5.4788961038961036e-04*Jinv01*w33+1.2784090909090910e-04*Jinv02*w12+9.1314935064935065e-06*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-9.2329545454545453e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-5.4788961038961036e-04*w47*Jinv21-2.1002435064935065e-04*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+1.1870941558441559e-04*w31*Jinv12-1.4610389610389610e-04*w46*Jinv12+6.3920454545454550e-05*Jinv22*w8-4.5657467532467534e-05*Jinv22*w50+9.1314935064935065e-06*w22*Jinv11+6.3920454545454550e-05*Jinv12*w7+9.1314935064935065e-06*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+1.1870941558441559e-04*Jinv02*w30-4.5657467532467534e-05*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11+1.0957792207792207e-03*Jinv21*w56-5.6818181818181818e-05*Jinv02*w27+6.3920454545454550e-05*w6*Jinv02+1.2784090909090910e-04*w19*Jinv12+6.3920454545454550e-05*Jinv22*w5-1.4610389610389610e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14+2.8409090909090908e-04*w58*Jinv12+1.2784090909090910e-04*Jinv02*w18+9.1314935064935065e-06*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33+9.1314935064935065e-06*Jinv21*w23+1.0957792207792207e-03*Jinv01*w54+1.0957792207792207e-03*w55*Jinv11-4.5657467532467534e-05*w48*Jinv02+6.4935064935064935e-05*Jinv01*w9-5.6818181818181818e-05*Jinv22*w29+1.2784090909090910e-04*Jinv22*w14-5.4788961038961036e-04*w40*Jinv11+1.1870941558441559e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43-5.4788961038961036e-04*Jinv01*w30-4.5657467532467534e-05*Jinv02*w51-9.1314935064935068e-05*Jinv22*w56-5.6818181818181818e-05*Jinv12*w28+9.1314935064935065e-06*w25*Jinv11-6.7573051948051951e-04*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50+6.4935064935064935e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01-9.2329545454545453e-05*Jinv12*w1+6.4935064935064935e-05*w10*Jinv11)*D;
  A[58][56]=(0.0)*D;
  A[58][57]=(0.0)*D;
  A[58][58]=(1.2175324675324675e-04*w40*Jinv12+6.4935064935064935e-04*Jinv22*w53+8.3874458874458877e-04*w57*Jinv02+1.2175324675324675e-04*Jinv02*w39-3.2467532467532468e-04*Jinv22*w38+2.0292207792207792e-05*w25*Jinv12+9.4696969696969697e-05*w0*Jinv02+6.4935064935064935e-04*Jinv12*w55+2.4350649350649351e-04*w15*Jinv02+9.4696969696969697e-05*w10*Jinv12+2.0292207792207792e-05*Jinv02*w24+1.2175324675324675e-04*Jinv22*w35+2.0292207792207792e-05*Jinv22*w20+1.2175324675324675e-04*w42*Jinv02+9.4696969696969697e-05*w11*Jinv22+2.4350649350649351e-04*Jinv22*w17+2.0292207792207792e-05*w22*Jinv12+2.0292207792207792e-05*Jinv02*w3-3.2467532467532468e-04*w32*Jinv22+9.4696969696969697e-05*Jinv22*w2-3.2467532467532468e-04*Jinv02*w45+8.3874458874458877e-04*Jinv22*w59+1.2175324675324675e-04*Jinv12*w34+1.2175324675324675e-04*Jinv22*w44+2.0292207792207792e-05*Jinv12*w13-3.2467532467532468e-04*w37*Jinv12+1.2175324675324675e-04*Jinv22*w41+6.4935064935064935e-04*w52*Jinv12+2.0292207792207792e-05*Jinv22*w26+6.4935064935064935e-04*Jinv02*w54+2.0292207792207792e-05*w4*Jinv12+2.0292207792207792e-05*w21*Jinv02+2.0292207792207792e-05*Jinv02*w12+9.4696969696969697e-05*Jinv02*w9+2.0292207792207792e-05*Jinv22*w23-3.2467532467532468e-04*w31*Jinv12-3.2467532467532468e-04*w46*Jinv12+2.0292207792207792e-05*Jinv22*w8+6.4935064935064935e-04*Jinv22*w50+2.0292207792207792e-05*Jinv12*w7-3.2467532467532468e-04*Jinv02*w30+6.4935064935064935e-04*Jinv12*w49+9.4696969696969697e-05*Jinv02*w27+2.0292207792207792e-05*w6*Jinv02+2.0292207792207792e-05*w19*Jinv12+2.0292207792207792e-05*Jinv22*w5-3.2467532467532468e-04*w47*Jinv22+8.3874458874458877e-04*w58*Jinv12+2.0292207792207792e-05*Jinv02*w18+1.2175324675324675e-04*Jinv02*w33+6.4935064935064935e-04*w48*Jinv02+9.4696969696969697e-05*Jinv22*w29+2.0292207792207792e-05*Jinv22*w14-3.2467532467532468e-04*Jinv02*w36+1.2175324675324675e-04*Jinv12*w43+6.4935064935064935e-04*Jinv02*w51+6.4935064935064935e-04*Jinv22*w56+9.4696969696969697e-05*Jinv12*w28+2.4350649350649351e-04*w16*Jinv12+9.4696969696969697e-05*Jinv12*w1)*D;
  A[58][59]=(0.0)*D;
  A[59][0]=(0.0)*D;
  A[59][1]=(0.0)*D;
  A[59][2]=(-1.5726461038961039e-04*Jinv00*w12-5.4112554112554113e-05*w28*Jinv10+4.2613636363636367e-05*w40*Jinv12-6.4935064935064935e-05*Jinv22*w53+1.1668019480519480e-04*Jinv20*w8-1.8939393939393939e-04*w57*Jinv02-1.5726461038961039e-04*w13*Jinv10+4.2613636363636367e-05*Jinv02*w39-3.0438311688311688e-05*Jinv21*w44-1.5726461038961039e-04*w3*Jinv00-2.0495129870129870e-04*w32*Jinv20-1.5726461038961039e-04*Jinv01*w3+1.1668019480519480e-04*Jinv20*w23+1.4204545454545455e-05*Jinv22*w38-6.4935064935064935e-05*Jinv00*w54-1.8939393939393939e-04*Jinv21*w59-1.0247564935064935e-04*w25*Jinv12-9.4696969696969697e-05*w0*Jinv02-9.4696969696969697e-05*Jinv21*w2-3.0438311688311688e-05*w43*Jinv10-3.0438311688311688e-05*w42*Jinv01-5.4112554112554113e-05*w11*Jinv21+1.9480519480519481e-04*w16*Jinv11+1.9480519480519481e-04*Jinv21*w17+1.4204545454545455e-05*Jinv01*w45-6.4935064935064935e-05*Jinv12*w55-1.8939393939393939e-04*w58*Jinv10+1.9480519480519481e-04*w15*Jinv02-5.4112554112554113e-05*w10*Jinv12-8.3198051948051951e-05*w48*Jinv00-1.0247564935064935e-04*Jinv02*w24+4.2613636363636367e-05*Jinv22*w35-1.8939393939393939e-04*w58*Jinv11-1.8939393939393939e-04*w57*Jinv01-1.0247564935064935e-04*Jinv21*w26-9.4696969696969697e-05*w0*Jinv01-9.4696969696969697e-05*w1*Jinv10+1.9480519480519481e-04*w16*Jinv10-1.0247564935064935e-04*Jinv22*w20-3.0438311688311688e-05*w43*Jinv11+1.4204545454545455e-05*w45*Jinv00+4.2613636363636367e-05*Jinv21*w41-3.0438311688311688e-05*w42*Jinv02-5.4112554112554113e-05*Jinv00*w27-5.4112554112554113e-05*w11*Jinv22-9.4696969696969697e-05*Jinv11*w1-1.5726461038961039e-04*w13*Jinv11+1.9480519480519481e-04*w15*Jinv01-5.4112554112554113e-05*w28*Jinv11-8.3198051948051951e-05*Jinv20*w50-5.4112554112554113e-05*Jinv20*w29+1.4204545454545455e-05*w46*Jinv10+1.9480519480519481e-04*Jinv22*w17+4.2613636363636367e-05*Jinv00*w33+1.1668019480519480e-04*w22*Jinv12-2.0495129870129870e-04*w31*Jinv10-1.5726461038961039e-04*Jinv02*w3+1.1668019480519480e-04*w21*Jinv00-2.0495129870129870e-04*w32*Jinv22+4.2613636363636367e-05*Jinv01*w39-9.4696969696969697e-05*Jinv22*w2-1.0247564935064935e-04*Jinv00*w18+1.4204545454545455e-05*w36*Jinv00+1.4204545454545455e-05*Jinv21*w38+1.4204545454545455e-05*Jinv02*w45-1.0247564935064935e-04*Jinv01*w24-1.8939393939393939e-04*Jinv22*w59+4.2613636363636367e-05*w34*Jinv11-1.0247564935064935e-04*w19*Jinv11-6.4935064935064935e-05*Jinv21*w53-6.4935064935064935e-05*w51*Jinv00+4.2613636363636367e-05*Jinv12*w34-3.0438311688311688e-05*Jinv22*w44-1.5726461038961039e-04*w4*Jinv10-1.5726461038961039e-04*w4*Jinv11+4.2613636363636367e-05*w41*Jinv20-1.5726461038961039e-04*Jinv12*w13-1.0247564935064935e-04*w19*Jinv10+4.2613636363636367e-05*w34*Jinv10-2.0495129870129870e-04*w32*Jinv21-1.0247564935064935e-04*Jinv21*w20+1.1668019480519480e-04*w6*Jinv00+1.4204545454545455e-05*w37*Jinv12-6.4935064935064935e-05*Jinv20*w56+4.2613636363636367e-05*Jinv22*w41-1.0247564935064935e-04*w26*Jinv20-5.4112554112554113e-05*w11*Jinv20-6.4935064935064935e-05*w52*Jinv12+4.2613636363636367e-05*Jinv21*w35-2.0495129870129870e-04*w31*Jinv11-1.0247564935064935e-04*Jinv22*w26+1.4204545454545455e-05*w46*Jinv11-1.5726461038961039e-04*Jinv20*w14-1.5726461038961039e-04*w5*Jinv21-6.4935064935064935e-05*Jinv02*w54-1.5726461038961039e-04*w4*Jinv12+1.1668019480519480e-04*w21*Jinv02-8.3198051948051951e-05*w49*Jinv10-5.4112554112554113e-05*Jinv21*w29+4.2613636363636367e-05*Jinv01*w33+4.2613636363636367e-05*w39*Jinv00-1.5726461038961039e-04*Jinv02*w12-1.0247564935064935e-04*Jinv01*w18+1.4204545454545455e-05*w37*Jinv11-5.4112554112554113e-05*Jinv02*w9+1.1668019480519480e-04*w6*Jinv01+1.1668019480519480e-04*w7*Jinv10+1.4204545454545455e-05*w47*Jinv21+1.4204545454545455e-05*Jinv20*w38+1.1668019480519480e-04*Jinv22*w23-6.4935064935064935e-05*w52*Jinv11-2.0495129870129870e-04*w31*Jinv12+1.4204545454545455e-05*w46*Jinv12-6.4935064935064935e-05*Jinv20*w53+1.1668019480519480e-04*Jinv22*w8+1.1668019480519480e-04*w22*Jinv10-8.3198051948051951e-05*Jinv22*w50+1.1668019480519480e-04*w22*Jinv11+1.1668019480519480e-04*Jinv12*w7+1.1668019480519480e-04*w21*Jinv01-6.4935064935064935e-05*Jinv01*w51-2.0495129870129870e-04*Jinv02*w30-8.3198051948051951e-05*Jinv12*w49+1.4204545454545455e-05*w36*Jinv01-6.4935064935064935e-05*w52*Jinv10+1.1668019480519480e-04*w7*Jinv11-5.4112554112554113e-05*w9*Jinv00+1.4204545454545455e-05*w37*Jinv10-6.4935064935064935e-05*Jinv21*w56-1.0247564935064935e-04*w24*Jinv00-5.4112554112554113e-05*Jinv02*w27+1.1668019480519480e-04*w6*Jinv02-1.0247564935064935e-04*w19*Jinv12-1.5726461038961039e-04*Jinv22*w5+1.4204545454545455e-05*w47*Jinv22-8.3198051948051951e-05*w49*Jinv11-1.5726461038961039e-04*Jinv21*w14-5.4112554112554113e-05*w10*Jinv10-3.0438311688311688e-05*Jinv20*w44-1.8939393939393939e-04*w58*Jinv12-1.0247564935064935e-04*Jinv02*w18-1.5726461038961039e-04*Jinv01*w12+1.1668019480519480e-04*Jinv21*w8-1.8939393939393939e-04*w57*Jinv00-1.8939393939393939e-04*Jinv20*w59+4.2613636363636367e-05*Jinv02*w33-9.4696969696969697e-05*w0*Jinv00+4.2613636363636367e-05*w35*Jinv20+1.1668019480519480e-04*Jinv21*w23-6.4935064935064935e-05*Jinv01*w54-6.4935064935064935e-05*w55*Jinv11-1.0247564935064935e-04*w25*Jinv10-9.4696969696969697e-05*Jinv20*w2+1.9480519480519481e-04*w15*Jinv00-8.3198051948051951e-05*w48*Jinv02-5.4112554112554113e-05*Jinv01*w9+1.9480519480519481e-04*Jinv20*w17-2.0495129870129870e-04*w30*Jinv00+4.2613636363636367e-05*w40*Jinv10-1.0247564935064935e-04*w20*Jinv20-5.4112554112554113e-05*Jinv22*w29-1.5726461038961039e-04*Jinv22*w14+4.2613636363636367e-05*w40*Jinv11+1.4204545454545455e-05*Jinv02*w36-3.0438311688311688e-05*Jinv12*w43-1.5726461038961039e-04*w5*Jinv20-2.0495129870129870e-04*Jinv01*w30-6.4935064935064935e-05*Jinv02*w51-6.4935064935064935e-05*Jinv22*w56-5.4112554112554113e-05*Jinv12*w28-1.0247564935064935e-04*w25*Jinv11-6.4935064935064935e-05*w55*Jinv10-3.0438311688311688e-05*w42*Jinv00+1.4204545454545455e-05*w47*Jinv20+1.9480519480519481e-04*w16*Jinv12-8.3198051948051951e-05*Jinv21*w50-5.4112554112554113e-05*Jinv01*w27-8.3198051948051951e-05*w48*Jinv01-9.4696969696969697e-05*Jinv12*w1-5.4112554112554113e-05*w10*Jinv11)*D;
  A[59][3]=(0.0)*D;
  A[59][4]=(0.0)*D;
  A[59][5]=(3.7439123376623376e-04*Jinv00*w12-5.2962662337662338e-04*w40*Jinv12-1.8262987012987013e-05*Jinv22*w53-2.7394480519480518e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02+3.7439123376623376e-04*w13*Jinv10-5.2962662337662338e-04*Jinv02*w39+7.3051948051948052e-05*Jinv21*w44+3.9265422077922079e-04*w32*Jinv20-7.4878246753246752e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23+3.1047077922077924e-04*Jinv22*w38-4.0584415584415584e-05*Jinv21*w59+2.1915584415584417e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+9.2329545454545453e-05*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10+7.3051948051948052e-05*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-9.6793831168831166e-04*w16*Jinv11-9.6793831168831166e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-9.6793831168831166e-04*w15*Jinv02+1.4204545454545455e-05*w10*Jinv12+9.1314935064935065e-06*w48*Jinv00+2.1915584415584417e-04*Jinv02*w24-1.0592532467532468e-03*Jinv22*w35-4.0584415584415584e-05*w58*Jinv11-4.0584415584415584e-05*w57*Jinv01+2.1915584415584417e-04*Jinv21*w26+9.2329545454545453e-05*w0*Jinv01+1.5726461038961039e-04*w1*Jinv10-6.9399350649350649e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+7.3051948051948052e-05*w43*Jinv11-5.2962662337662338e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*w11*Jinv22+9.2329545454545453e-05*Jinv11*w1-3.7439123376623376e-04*w13*Jinv11-9.6793831168831166e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-9.6793831168831166e-04*Jinv22*w17-5.6615259740259744e-04*Jinv00*w33+3.1960227272727273e-04*w22*Jinv12+3.9265422077922079e-04*w31*Jinv10-7.4878246753246752e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.2962662337662338e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+3.1960227272727273e-04*Jinv00*w18+1.7349837662337662e-04*w36*Jinv00+3.1047077922077924e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45+2.1915584415584417e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-1.0592532467532468e-03*w34*Jinv11+4.2918019480519480e-04*w19*Jinv11-1.8262987012987013e-05*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00-1.0592532467532468e-03*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-7.4878246753246752e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20-3.7439123376623376e-04*Jinv12*w13+3.1960227272727273e-04*w19*Jinv10-5.6615259740259744e-04*w34*Jinv10-3.9265422077922079e-04*w32*Jinv21+4.2918019480519480e-04*Jinv21*w20-2.7394480519480518e-04*w6*Jinv00+3.1047077922077924e-04*w37*Jinv12-5.2962662337662338e-04*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20+1.1668019480519480e-04*w11*Jinv20-1.8262987012987013e-05*w52*Jinv12-1.0592532467532468e-03*Jinv21*w35-3.9265422077922079e-04*w31*Jinv11+2.1915584415584417e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11+3.7439123376623376e-04*Jinv20*w14-7.4878246753246752e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54-7.4878246753246752e-04*w4*Jinv12+3.1960227272727273e-04*w21*Jinv02+9.1314935064935065e-06*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-1.0592532467532468e-03*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00-3.7439123376623376e-04*Jinv02*w12+4.2918019480519480e-04*Jinv01*w18+3.1047077922077924e-04*w37*Jinv11+1.4204545454545455e-05*Jinv02*w9+1.5523538961038962e-04*w6*Jinv01-2.7394480519480518e-04*w7*Jinv10+1.3697240259740259e-04*w47*Jinv21+1.7349837662337662e-04*Jinv20*w38+3.1960227272727273e-04*Jinv22*w23-1.8262987012987013e-05*w52*Jinv11-3.9265422077922079e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+1.5523538961038962e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10+4.8396915584415583e-04*Jinv22*w50+3.1960227272727273e-04*w22*Jinv11+1.5523538961038962e-04*Jinv12*w7+3.1960227272727273e-04*w21*Jinv01-1.8262987012987013e-05*Jinv01*w51-3.9265422077922079e-04*Jinv02*w30+4.8396915584415583e-04*Jinv12*w49+3.1047077922077924e-04*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10+1.5523538961038962e-04*w7*Jinv11+1.1668019480519480e-04*w9*Jinv00+1.7349837662337662e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27+1.5523538961038962e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12-7.4878246753246752e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22+4.8396915584415583e-04*w49*Jinv11-3.7439123376623376e-04*Jinv21*w14+1.1668019480519480e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18-3.7439123376623376e-04*Jinv01*w12+1.5523538961038962e-04*Jinv21*w8-1.0592532467532468e-03*Jinv02*w33+1.5726461038961039e-04*w0*Jinv00-5.6615259740259744e-04*w35*Jinv20+3.1960227272727273e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10+1.5726461038961039e-04*Jinv20*w2-6.9399350649350649e-04*w15*Jinv00+4.8396915584415583e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-6.9399350649350649e-04*Jinv20*w17+3.9265422077922079e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10+3.1960227272727273e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29-3.7439123376623376e-04*Jinv22*w14-5.2962662337662338e-04*w40*Jinv11+3.1047077922077924e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.9265422077922079e-04*Jinv01*w30-1.8262987012987013e-05*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+2.1915584415584417e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-9.6793831168831166e-04*w16*Jinv12+4.8396915584415583e-04*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27+4.8396915584415583e-04*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[59][6]=(0.0)*D;
  A[59][7]=(0.0)*D;
  A[59][8]=(-3.1960227272727273e-04*Jinv00*w12+3.6525974025974026e-05*w40*Jinv12+4.7483766233766234e-04*Jinv22*w53-4.0584415584415584e-05*w57*Jinv02-3.1960227272727273e-04*w13*Jinv10+3.6525974025974026e-05*Jinv02*w39-4.9310064935064932e-04*Jinv21*w44+2.7394480519480518e-04*w3*Jinv00-1.7349837662337662e-04*w32*Jinv20+4.2918019480519480e-04*Jinv01*w3-1.0957792207792208e-04*Jinv20*w23-7.8530844155844158e-04*Jinv22*w38-4.0584415584415584e-05*Jinv21*w59+4.2918019480519480e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.0247564935064935e-04*Jinv21*w2+3.6525974025974026e-05*w43*Jinv10-4.9310064935064932e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21-2.7394480519480518e-04*w16*Jinv11-2.7394480519480518e-04*Jinv21*w17+1.3697240259740259e-04*Jinv01*w45-9.1314935064935065e-06*Jinv12*w55-2.7394480519480518e-04*w15*Jinv02-6.4935064935064935e-05*w10*Jinv12+9.1314935064935065e-06*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-4.9310064935064932e-04*Jinv22*w35-4.0584415584415584e-05*w58*Jinv11-4.0584415584415584e-05*w57*Jinv01+4.2918019480519480e-04*Jinv21*w26-1.0247564935064935e-04*w0*Jinv01-1.1668019480519480e-04*w1*Jinv10+6.9399350649350649e-04*w16*Jinv10-7.4878246753246752e-04*Jinv22*w20-4.9310064935064932e-04*w43*Jinv11+3.6525974025974026e-05*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*w11*Jinv22-1.0247564935064935e-04*Jinv11*w1+1.0957792207792208e-04*w13*Jinv11-2.7394480519480518e-04*w15*Jinv01-1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-2.7394480519480518e-04*Jinv22*w17+5.6615259740259744e-04*Jinv00*w33+1.0957792207792208e-04*w22*Jinv12-1.7349837662337662e-04*w31*Jinv10+4.2918019480519480e-04*Jinv02*w3-1.0957792207792208e-04*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22+3.6525974025974026e-05*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2-3.7439123376623376e-04*Jinv00*w18-3.9265422077922079e-04*w36*Jinv00-7.8530844155844158e-04*Jinv21*w38+1.3697240259740259e-04*Jinv02*w45+4.2918019480519480e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-4.9310064935064932e-04*w34*Jinv11-7.4878246753246752e-04*w19*Jinv11+4.7483766233766234e-04*Jinv21*w53-9.1314935064935065e-06*w51*Jinv00-4.9310064935064932e-04*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+2.7394480519480518e-04*w4*Jinv10+4.2918019480519480e-04*w4*Jinv11-3.6525974025974026e-05*w41*Jinv20+1.0957792207792208e-04*Jinv12*w13-3.7439123376623376e-04*w19*Jinv10+5.6615259740259744e-04*w34*Jinv10+1.3697240259740259e-04*w32*Jinv21-7.4878246753246752e-04*Jinv21*w20-7.8530844155844158e-04*w37*Jinv12+3.6525974025974026e-05*Jinv22*w41+1.0957792207792208e-04*w26*Jinv20-1.5726461038961039e-04*w11*Jinv20+4.7483766233766234e-04*w52*Jinv12-4.9310064935064932e-04*Jinv21*w35+1.3697240259740259e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.3697240259740259e-04*w46*Jinv11-3.1960227272727273e-04*Jinv20*w14+4.2918019480519480e-04*w5*Jinv21-9.1314935064935065e-06*Jinv02*w54+4.2918019480519480e-04*w4*Jinv12+1.0957792207792208e-04*w21*Jinv02+9.1314935064935065e-06*w49*Jinv10-1.0247564935064935e-04*Jinv21*w29-4.9310064935064932e-04*Jinv01*w33-3.6525974025974026e-05*w39*Jinv00+1.0957792207792208e-04*Jinv02*w12-7.4878246753246752e-04*Jinv01*w18-7.8530844155844158e-04*w37*Jinv11-6.4935064935064935e-05*Jinv02*w9-7.4878246753246752e-04*w6*Jinv01+1.3697240259740259e-04*w47*Jinv21-3.9265422077922079e-04*Jinv20*w38+1.0957792207792208e-04*Jinv22*w23+4.7483766233766234e-04*w52*Jinv11+1.3697240259740259e-04*w31*Jinv12+1.3697240259740259e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53-7.4878246753246752e-04*Jinv22*w8-1.0957792207792208e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50+1.0957792207792208e-04*w22*Jinv11-7.4878246753246752e-04*Jinv12*w7+1.0957792207792208e-04*w21*Jinv01+4.7483766233766234e-04*Jinv01*w51+1.3697240259740259e-04*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-7.8530844155844158e-04*w36*Jinv01-9.1314935064935065e-06*w52*Jinv10-7.4878246753246752e-04*w7*Jinv11-1.5726461038961039e-04*w9*Jinv00-3.9265422077922079e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+1.0957792207792208e-04*w24*Jinv00-1.0247564935064935e-04*Jinv02*w27-7.4878246753246752e-04*w6*Jinv02-7.4878246753246752e-04*w19*Jinv12+4.2918019480519480e-04*Jinv22*w5+1.3697240259740259e-04*w47*Jinv22-9.1314935064935065e-06*w49*Jinv11+1.0957792207792208e-04*Jinv21*w14-1.5726461038961039e-04*w10*Jinv10+3.6525974025974026e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12-7.4878246753246752e-04*Jinv02*w18+1.0957792207792208e-04*Jinv01*w12-7.4878246753246752e-04*Jinv21*w8-4.9310064935064932e-04*Jinv02*w33-1.1668019480519480e-04*w0*Jinv00+5.6615259740259744e-04*w35*Jinv20+1.0957792207792208e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+1.0957792207792208e-04*w25*Jinv10-1.1668019480519480e-04*Jinv20*w2+6.9399350649350649e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+6.9399350649350649e-04*Jinv20*w17-1.7349837662337662e-04*w30*Jinv00-3.6525974025974026e-05*w40*Jinv10-3.7439123376623376e-04*w20*Jinv20-1.0247564935064935e-04*Jinv22*w29+1.0957792207792208e-04*Jinv22*w14+3.6525974025974026e-05*w40*Jinv11-7.8530844155844158e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+2.7394480519480518e-04*w5*Jinv20+1.3697240259740259e-04*Jinv01*w30+4.7483766233766234e-04*Jinv02*w51-9.1314935064935065e-06*Jinv22*w56-1.0247564935064935e-04*Jinv12*w28+4.2918019480519480e-04*w25*Jinv11+3.6525974025974026e-05*w42*Jinv00-2.7394480519480518e-04*w16*Jinv12-9.1314935064935065e-06*Jinv21*w50-1.0247564935064935e-04*Jinv01*w27-9.1314935064935065e-06*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[59][9]=(0.0)*D;
  A[59][10]=(0.0)*D;
  A[59][11]=(1.0247564935064935e-04*Jinv00*w12+5.4112554112554113e-05*w28*Jinv10+1.5726461038961039e-04*Jinv20*w8+1.0247564935064935e-04*w13*Jinv10-1.1668019480519480e-04*w3*Jinv00-1.4204545454545455e-05*w32*Jinv20+1.0247564935064935e-04*Jinv20*w23+6.4935064935064935e-05*Jinv00*w54-4.2613636363636367e-05*w43*Jinv10+1.8939393939393939e-04*w58*Jinv10+6.4935064935064935e-05*w48*Jinv00+5.4112554112554113e-05*w1*Jinv10-1.9480519480519481e-04*w16*Jinv10-1.4204545454545455e-05*w45*Jinv00+5.4112554112554113e-05*Jinv00*w27+6.4935064935064935e-05*Jinv20*w50+5.4112554112554113e-05*Jinv20*w29-1.4204545454545455e-05*w46*Jinv10-4.2613636363636367e-05*Jinv00*w33-1.4204545454545455e-05*w31*Jinv10+1.0247564935064935e-04*w21*Jinv00+1.5726461038961039e-04*Jinv00*w18+2.0495129870129870e-04*w36*Jinv00+8.3198051948051951e-05*w51*Jinv00-1.1668019480519480e-04*w4*Jinv10+3.0438311688311688e-05*w41*Jinv20+1.5726461038961039e-04*w19*Jinv10-4.2613636363636367e-05*w34*Jinv10+1.5726461038961039e-04*w6*Jinv00+6.4935064935064935e-05*Jinv20*w56-1.1668019480519480e-04*w26*Jinv20+9.4696969696969697e-05*w11*Jinv20+1.0247564935064935e-04*Jinv20*w14+6.4935064935064935e-05*w49*Jinv10+3.0438311688311688e-05*w39*Jinv00+1.5726461038961039e-04*w7*Jinv10+2.0495129870129870e-04*Jinv20*w38+8.3198051948051951e-05*Jinv20*w53+1.0247564935064935e-04*w22*Jinv10+8.3198051948051951e-05*w52*Jinv10+9.4696969696969697e-05*w9*Jinv00+2.0495129870129870e-04*w37*Jinv10-1.1668019480519480e-04*w24*Jinv00+9.4696969696969697e-05*w10*Jinv10-4.2613636363636367e-05*Jinv20*w44+1.8939393939393939e-04*w57*Jinv00+1.8939393939393939e-04*Jinv20*w59+5.4112554112554113e-05*w0*Jinv00-4.2613636363636367e-05*w35*Jinv20-1.1668019480519480e-04*w25*Jinv10+5.4112554112554113e-05*Jinv20*w2-1.9480519480519481e-04*w15*Jinv00-1.9480519480519481e-04*Jinv20*w17-1.4204545454545455e-05*w30*Jinv00+3.0438311688311688e-05*w40*Jinv10+1.5726461038961039e-04*w20*Jinv20-1.1668019480519480e-04*w5*Jinv20+6.4935064935064935e-05*w55*Jinv10-4.2613636363636367e-05*w42*Jinv00-1.4204545454545455e-05*w47*Jinv20)*D;
  A[59][12]=(0.0)*D;
  A[59][13]=(0.0)*D;
  A[59][14]=(-7.4878246753246752e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10-1.0592532467532468e-03*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53+3.1960227272727273e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02-7.4878246753246752e-04*w13*Jinv10-1.0592532467532468e-03*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44-3.7439123376623376e-04*w3*Jinv00-3.9265422077922079e-04*w32*Jinv20+3.7439123376623376e-04*Jinv01*w3+1.5523538961038962e-04*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38-1.8262987012987013e-05*Jinv00*w54+4.2918019480519480e-04*w25*Jinv12+9.2329545454545453e-05*w0*Jinv02+1.5726461038961039e-04*Jinv21*w2+7.3051948051948052e-05*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01-6.9399350649350649e-04*w16*Jinv11-6.9399350649350649e-04*Jinv21*w17+1.7349837662337662e-04*Jinv01*w45-1.8262987012987013e-05*Jinv12*w55-4.0584415584415584e-05*w58*Jinv10-9.6793831168831166e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12+4.8396915584415583e-04*w48*Jinv00+4.2918019480519480e-04*Jinv02*w24-5.2962662337662338e-04*Jinv22*w35+3.1960227272727273e-04*Jinv21*w26+1.5726461038961039e-04*w0*Jinv01+9.2329545454545453e-05*w1*Jinv10-9.6793831168831166e-04*w16*Jinv10+2.1915584415584417e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11+3.1047077922077924e-04*w45*Jinv00-5.6615259740259744e-04*Jinv21*w41+7.3051948051948052e-05*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22+1.5726461038961039e-04*Jinv11*w1-6.9399350649350649e-04*w15*Jinv01+1.1668019480519480e-04*w28*Jinv11+4.8396915584415583e-04*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29+3.1047077922077924e-04*w46*Jinv10-9.6793831168831166e-04*Jinv22*w17-5.2962662337662338e-04*Jinv00*w33+1.5523538961038962e-04*w22*Jinv12-3.9265422077922079e-04*w31*Jinv10-3.7439123376623376e-04*Jinv02*w3+1.5523538961038962e-04*w21*Jinv00-3.9265422077922079e-04*w32*Jinv22-5.6615259740259744e-04*Jinv01*w39+9.2329545454545453e-05*Jinv22*w2+2.1915584415584417e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00+3.1047077922077924e-04*Jinv02*w45+3.1960227272727273e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-9.1314935064935065e-06*w51*Jinv00-5.2962662337662338e-04*Jinv12*w34+7.3051948051948052e-05*Jinv22*w44-3.7439123376623376e-04*w4*Jinv10+3.7439123376623376e-04*w4*Jinv11-1.0592532467532468e-03*w41*Jinv20-7.4878246753246752e-04*Jinv12*w13+2.1915584415584417e-04*w19*Jinv10-5.2962662337662338e-04*w34*Jinv10+3.9265422077922079e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+3.1960227272727273e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12-1.8262987012987013e-05*Jinv20*w56-1.0592532467532468e-03*Jinv22*w41+4.2918019480519480e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35+3.9265422077922079e-04*w31*Jinv11+4.2918019480519480e-04*Jinv22*w26+1.7349837662337662e-04*w46*Jinv11-7.4878246753246752e-04*Jinv20*w14+3.7439123376623376e-04*w5*Jinv21-1.8262987012987013e-05*Jinv02*w54-3.7439123376623376e-04*w4*Jinv12+1.5523538961038962e-04*w21*Jinv02+4.8396915584415583e-04*w49*Jinv10+1.1668019480519480e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-1.0592532467532468e-03*w39*Jinv00-7.4878246753246752e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18-1.0247564935064935e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+3.1960227272727273e-04*w7*Jinv10+1.7349837662337662e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38+1.5523538961038962e-04*Jinv22*w23-3.9265422077922079e-04*w31*Jinv12+3.1047077922077924e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+3.1960227272727273e-04*Jinv22*w8+1.5523538961038962e-04*w22*Jinv10+4.8396915584415583e-04*Jinv22*w50-2.7394480519480518e-04*w22*Jinv11+3.1960227272727273e-04*Jinv12*w7-2.7394480519480518e-04*w21*Jinv01-3.9265422077922079e-04*Jinv02*w30+4.8396915584415583e-04*Jinv12*w49-9.1314935064935065e-06*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56+4.2918019480519480e-04*w24*Jinv00+1.4204545454545455e-05*Jinv02*w27+3.1960227272727273e-04*w6*Jinv02+2.1915584415584417e-04*w19*Jinv12-3.7439123376623376e-04*Jinv22*w5+3.1047077922077924e-04*w47*Jinv22+9.1314935064935065e-06*w49*Jinv11-1.0247564935064935e-04*w10*Jinv10+7.3051948051948052e-05*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+2.1915584415584417e-04*Jinv02*w18-1.0957792207792208e-04*Jinv21*w8-4.0584415584415584e-05*w57*Jinv00-4.0584415584415584e-05*Jinv20*w59-5.2962662337662338e-04*Jinv02*w33+9.2329545454545453e-05*w0*Jinv00-5.2962662337662338e-04*w35*Jinv20-2.7394480519480518e-04*Jinv21*w23-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11+4.2918019480519480e-04*w25*Jinv10+9.2329545454545453e-05*Jinv20*w2-9.6793831168831166e-04*w15*Jinv00+4.8396915584415583e-04*w48*Jinv02-9.6793831168831166e-04*Jinv20*w17-3.9265422077922079e-04*w30*Jinv00-1.0592532467532468e-03*w40*Jinv10+2.1915584415584417e-04*w20*Jinv20+1.4204545454545455e-05*Jinv22*w29-7.4878246753246752e-04*Jinv22*w14-5.6615259740259744e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36+7.3051948051948052e-05*Jinv12*w43-3.7439123376623376e-04*w5*Jinv20+3.9265422077922079e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51-1.8262987012987013e-05*Jinv22*w56+1.4204545454545455e-05*Jinv12*w28+3.1960227272727273e-04*w25*Jinv11-1.8262987012987013e-05*w55*Jinv10+7.3051948051948052e-05*w42*Jinv00+3.1047077922077924e-04*w47*Jinv20-9.6793831168831166e-04*w16*Jinv12+9.1314935064935065e-06*Jinv21*w50+1.1668019480519480e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+9.2329545454545453e-05*Jinv12*w1)*D;
  A[59][15]=(0.0)*D;
  A[59][16]=(0.0)*D;
  A[59][17]=(9.6793831168831166e-04*Jinv00*w12-1.1323051948051949e-03*w40*Jinv12+9.8620129870129864e-04*Jinv22*w53-6.9399350649350649e-04*Jinv20*w8-4.8701298701298701e-04*w57*Jinv02+9.6793831168831166e-04*w13*Jinv10-1.1323051948051949e-03*Jinv02*w39-1.1323051948051949e-03*Jinv21*w44+6.9399350649350649e-04*w3*Jinv00+5.2962662337662338e-04*w32*Jinv20+9.6793831168831166e-04*Jinv01*w3+2.7394480519480518e-04*Jinv20*w23-4.9310064935064932e-04*Jinv22*w38-5.4788961038961036e-04*w25*Jinv12-2.2524350649350650e-04*w0*Jinv02-1.9480519480519481e-04*Jinv21*w2-1.1323051948051949e-03*w43*Jinv10-1.1323051948051949e-03*w42*Jinv01-5.2962662337662338e-04*Jinv01*w45+9.8620129870129864e-04*Jinv12*w55-2.7759740259740259e-03*w15*Jinv02-3.0438311688311688e-05*w10*Jinv12-4.9310064935064932e-04*w48*Jinv00-5.4788961038961036e-04*Jinv02*w24-1.1323051948051949e-03*Jinv22*w35-9.6793831168831166e-04*Jinv21*w26-1.9480519480519481e-04*w0*Jinv01-1.9480519480519481e-04*w1*Jinv10-5.4788961038961036e-04*Jinv22*w20-1.1323051948051949e-03*w43*Jinv11-2.2646103896103898e-03*w42*Jinv02-3.0438311688311688e-05*w11*Jinv22-1.9480519480519481e-04*Jinv11*w1+6.9399350649350649e-04*w13*Jinv11+1.9480519480519481e-04*w28*Jinv11-4.9310064935064932e-04*Jinv20*w50-2.7759740259740259e-03*Jinv22*w17-2.7394480519480518e-04*w22*Jinv12+5.2962662337662338e-04*w31*Jinv10+4.2004870129870131e-04*Jinv02*w3+2.7394480519480518e-04*w21*Jinv00+3.6525974025974026e-05*w32*Jinv22-2.2524350649350650e-04*Jinv22*w2-9.6793831168831166e-04*Jinv00*w18-5.2962662337662338e-04*w36*Jinv00-4.9310064935064932e-04*Jinv02*w45-9.6793831168831166e-04*Jinv01*w24-4.8701298701298701e-04*Jinv22*w59+1.1323051948051949e-03*w34*Jinv11-2.7394480519480518e-04*w19*Jinv11+4.9310064935064932e-04*w51*Jinv00-1.1323051948051949e-03*Jinv12*w34-2.2646103896103898e-03*Jinv22*w44+6.9399350649350649e-04*w4*Jinv10+9.6793831168831166e-04*w4*Jinv11+1.1323051948051949e-03*w41*Jinv20+4.2004870129870131e-04*Jinv12*w13-9.6793831168831166e-04*w19*Jinv10+5.2962662337662338e-04*w32*Jinv21-2.7394480519480518e-04*Jinv21*w20-6.9399350649350649e-04*w6*Jinv00-4.9310064935064932e-04*w37*Jinv12-1.1323051948051949e-03*Jinv22*w41-2.7394480519480518e-04*w26*Jinv20+1.9480519480519481e-04*w11*Jinv20+9.8620129870129864e-04*w52*Jinv12+1.1323051948051949e-03*Jinv21*w35+5.2962662337662338e-04*w31*Jinv11-5.4788961038961036e-04*Jinv22*w26-5.2962662337662338e-04*w46*Jinv11+9.6793831168831166e-04*Jinv20*w14+9.6793831168831166e-04*w5*Jinv21+9.8620129870129864e-04*Jinv02*w54+4.2004870129870131e-04*w4*Jinv12-2.7394480519480518e-04*w21*Jinv02-4.9310064935064932e-04*w49*Jinv10+1.9480519480519481e-04*Jinv21*w29+1.1323051948051949e-03*Jinv01*w33+1.1323051948051949e-03*w39*Jinv00+4.2004870129870131e-04*Jinv02*w12-2.7394480519480518e-04*Jinv01*w18-3.0438311688311688e-05*Jinv02*w9+2.7394480519480518e-04*w6*Jinv01-6.9399350649350649e-04*w7*Jinv10-5.2962662337662338e-04*w47*Jinv21-5.2962662337662338e-04*Jinv20*w38-2.7394480519480518e-04*Jinv22*w23+3.6525974025974026e-05*w31*Jinv12-4.9310064935064932e-04*w46*Jinv12+4.9310064935064932e-04*Jinv20*w53-2.7394480519480518e-04*Jinv22*w8+2.7394480519480518e-04*w22*Jinv10+4.9310064935064932e-04*Jinv22*w50-6.9399350649350649e-04*w22*Jinv11-2.7394480519480518e-04*Jinv12*w7-6.9399350649350649e-04*w21*Jinv01+3.6525974025974026e-05*Jinv02*w30+4.9310064935064932e-04*Jinv12*w49+4.9310064935064932e-04*w52*Jinv10+2.7394480519480518e-04*w7*Jinv11+1.9480519480519481e-04*w9*Jinv00-5.2962662337662338e-04*w37*Jinv10+4.9310064935064932e-04*Jinv21*w56-2.7394480519480518e-04*w24*Jinv00-3.0438311688311688e-05*Jinv02*w27-2.7394480519480518e-04*w6*Jinv02-5.4788961038961036e-04*w19*Jinv12+4.2004870129870131e-04*Jinv22*w5-4.9310064935064932e-04*w47*Jinv22-4.9310064935064932e-04*w49*Jinv11+6.9399350649350649e-04*Jinv21*w14+1.9480519480519481e-04*w10*Jinv10-1.1323051948051949e-03*Jinv20*w44-4.8701298701298701e-04*w58*Jinv12-5.4788961038961036e-04*Jinv02*w18+6.9399350649350649e-04*Jinv01*w12+2.7394480519480518e-04*Jinv21*w8-1.1323051948051949e-03*Jinv02*w33-1.9480519480519481e-04*w0*Jinv00-6.9399350649350649e-04*Jinv21*w23+4.9310064935064932e-04*Jinv01*w54+4.9310064935064932e-04*w55*Jinv11-2.7394480519480518e-04*w25*Jinv10-1.9480519480519481e-04*Jinv20*w2+4.9310064935064932e-04*w48*Jinv02+5.2962662337662338e-04*w30*Jinv00+1.1323051948051949e-03*w40*Jinv10-9.6793831168831166e-04*w20*Jinv20-3.0438311688311688e-05*Jinv22*w29+4.2004870129870131e-04*Jinv22*w14-4.9310064935064932e-04*Jinv02*w36-2.2646103896103898e-03*Jinv12*w43+6.9399350649350649e-04*w5*Jinv20+5.2962662337662338e-04*Jinv01*w30+9.8620129870129864e-04*Jinv02*w51+9.8620129870129864e-04*Jinv22*w56-3.0438311688311688e-05*Jinv12*w28-9.6793831168831166e-04*w25*Jinv11-1.1323051948051949e-03*w42*Jinv00-2.7759740259740259e-03*w16*Jinv12-4.9310064935064932e-04*Jinv21*w50+1.9480519480519481e-04*Jinv01*w27-4.9310064935064932e-04*w48*Jinv01-2.2524350649350650e-04*Jinv12*w1)*D;
  A[59][18]=(0.0)*D;
  A[59][19]=(0.0)*D;
  A[59][20]=(-2.1915584415584417e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10+3.7439123376623376e-04*Jinv20*w8-2.1915584415584417e-04*w13*Jinv10+4.9310064935064932e-04*Jinv21*w44-3.1960227272727273e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-4.2918019480519480e-04*Jinv01*w3-4.2918019480519480e-04*Jinv20*w23+1.8262987012987013e-05*Jinv00*w54+4.0584415584415584e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+1.0592532467532468e-03*w43*Jinv10+4.9310064935064932e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21+2.7394480519480518e-04*w16*Jinv11+2.7394480519480518e-04*Jinv21*w17-1.3697240259740259e-04*Jinv01*w45+4.0584415584415584e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00+4.0584415584415584e-05*w58*Jinv11+4.0584415584415584e-05*w57*Jinv01-4.2918019480519480e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+9.6793831168831166e-04*w16*Jinv10+4.9310064935064932e-04*w43*Jinv11-3.1047077922077924e-04*w45*Jinv00-3.6525974025974026e-05*Jinv21*w41-1.4204545454545455e-05*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-1.0957792207792208e-04*w13*Jinv11+2.7394480519480518e-04*w15*Jinv01+1.0247564935064935e-04*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29-3.1047077922077924e-04*w46*Jinv10+5.2962662337662338e-04*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10-4.2918019480519480e-04*w21*Jinv00-3.6525974025974026e-05*Jinv01*w39+7.4878246753246752e-04*Jinv00*w18+3.9265422077922079e-04*w36*Jinv00+7.8530844155844158e-04*Jinv21*w38-4.2918019480519480e-04*Jinv01*w24+4.9310064935064932e-04*w34*Jinv11+7.4878246753246752e-04*w19*Jinv11-4.7483766233766234e-04*Jinv21*w53-4.8396915584415583e-04*w51*Jinv00-3.1960227272727273e-04*w4*Jinv10-4.2918019480519480e-04*w4*Jinv11-7.3051948051948052e-05*w41*Jinv20+7.4878246753246752e-04*w19*Jinv10+5.2962662337662338e-04*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21+7.4878246753246752e-04*Jinv21*w20+3.7439123376623376e-04*w6*Jinv00+1.8262987012987013e-05*Jinv20*w56-1.5523538961038962e-04*w26*Jinv20-9.2329545454545453e-05*w11*Jinv20+4.9310064935064932e-04*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11-1.3697240259740259e-04*w46*Jinv11-2.1915584415584417e-04*Jinv20*w14-4.2918019480519480e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10+1.0247564935064935e-04*Jinv21*w29+4.9310064935064932e-04*Jinv01*w33-7.3051948051948052e-05*w39*Jinv00+7.4878246753246752e-04*Jinv01*w18+7.8530844155844158e-04*w37*Jinv11+7.4878246753246752e-04*w6*Jinv01+3.7439123376623376e-04*w7*Jinv10-1.3697240259740259e-04*w47*Jinv21+3.9265422077922079e-04*Jinv20*w38-4.7483766233766234e-04*w52*Jinv11-4.8396915584415583e-04*Jinv20*w53-4.2918019480519480e-04*w22*Jinv10-1.0957792207792208e-04*w22*Jinv11-1.0957792207792208e-04*w21*Jinv01-4.7483766233766234e-04*Jinv01*w51+7.8530844155844158e-04*w36*Jinv01-4.8396915584415583e-04*w52*Jinv10+7.4878246753246752e-04*w7*Jinv11-9.2329545454545453e-05*w9*Jinv00+3.9265422077922079e-04*w37*Jinv10+9.1314935064935065e-06*Jinv21*w56-1.5523538961038962e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11-1.0957792207792208e-04*Jinv21*w14-9.2329545454545453e-05*w10*Jinv10+1.0592532467532468e-03*Jinv20*w44-1.0957792207792208e-04*Jinv01*w12+7.4878246753246752e-04*Jinv21*w8+4.0584415584415584e-05*w57*Jinv00+4.0584415584415584e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00+5.2962662337662338e-04*w35*Jinv20-1.0957792207792208e-04*Jinv21*w23+9.1314935064935065e-06*Jinv01*w54+9.1314935064935065e-06*w55*Jinv11-1.5523538961038962e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+9.6793831168831166e-04*w15*Jinv00+6.4935064935064935e-05*Jinv01*w9+9.6793831168831166e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00-7.3051948051948052e-05*w40*Jinv10+7.4878246753246752e-04*w20*Jinv20-3.6525974025974026e-05*w40*Jinv11-3.1960227272727273e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30-4.2918019480519480e-04*w25*Jinv11+1.8262987012987013e-05*w55*Jinv10+1.0592532467532468e-03*w42*Jinv00-3.1047077922077924e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50+1.0247564935064935e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01+6.4935064935064935e-05*w10*Jinv11)*D;
  A[59][21]=(0.0)*D;
  A[59][22]=(0.0)*D;
  A[59][23]=(4.2918019480519480e-04*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-4.9310064935064932e-04*w40*Jinv12-9.1314935064935065e-06*Jinv22*w53+1.0957792207792208e-04*Jinv20*w8-4.0584415584415584e-05*w57*Jinv02+4.2918019480519480e-04*w13*Jinv10-4.9310064935064932e-04*Jinv02*w39+3.6525974025974026e-05*Jinv21*w44+1.0957792207792208e-04*w3*Jinv00+1.3697240259740259e-04*w32*Jinv20-3.1960227272727273e-04*Jinv01*w3-7.4878246753246752e-04*Jinv20*w23+1.3697240259740259e-04*Jinv22*w38+4.7483766233766234e-04*Jinv00*w54-7.4878246753246752e-04*w25*Jinv12-1.0247564935064935e-04*w0*Jinv02-1.1668019480519480e-04*Jinv21*w2-4.9310064935064932e-04*w43*Jinv10+3.6525974025974026e-05*w42*Jinv01+6.9399350649350649e-04*w16*Jinv11+6.9399350649350649e-04*Jinv21*w17-3.9265422077922079e-04*Jinv01*w45+4.7483766233766234e-04*Jinv12*w55-4.0584415584415584e-05*w58*Jinv10-2.7394480519480518e-04*w15*Jinv02-1.0247564935064935e-04*w10*Jinv12-9.1314935064935065e-06*w48*Jinv00-7.4878246753246752e-04*Jinv02*w24+3.6525974025974026e-05*Jinv22*w35-3.7439123376623376e-04*Jinv21*w26-1.1668019480519480e-04*w0*Jinv01-1.0247564935064935e-04*w1*Jinv10-2.7394480519480518e-04*w16*Jinv10+4.2918019480519480e-04*Jinv22*w20+3.6525974025974026e-05*w43*Jinv11-7.8530844155844158e-04*w45*Jinv00+5.6615259740259744e-04*Jinv21*w41-4.9310064935064932e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.0247564935064935e-04*w11*Jinv22-1.1668019480519480e-04*Jinv11*w1+2.7394480519480518e-04*w13*Jinv11+6.9399350649350649e-04*w15*Jinv01-1.5726461038961039e-04*w28*Jinv11-9.1314935064935065e-06*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29-7.8530844155844158e-04*w46*Jinv10-2.7394480519480518e-04*Jinv22*w17+3.6525974025974026e-05*Jinv00*w33-7.4878246753246752e-04*w22*Jinv12+1.3697240259740259e-04*w31*Jinv10+1.0957792207792208e-04*Jinv02*w3-7.4878246753246752e-04*w21*Jinv00+1.3697240259740259e-04*w32*Jinv22+5.6615259740259744e-04*Jinv01*w39-1.0247564935064935e-04*Jinv22*w2+4.2918019480519480e-04*Jinv00*w18+1.3697240259740259e-04*w36*Jinv00-7.8530844155844158e-04*Jinv02*w45-3.7439123376623376e-04*Jinv01*w24-4.0584415584415584e-05*Jinv22*w59-3.6525974025974026e-05*w34*Jinv11+1.0957792207792208e-04*w19*Jinv11-9.1314935064935065e-06*w51*Jinv00+3.6525974025974026e-05*Jinv12*w34-4.9310064935064932e-04*Jinv22*w44+1.0957792207792208e-04*w4*Jinv10-3.1960227272727273e-04*w4*Jinv11-4.9310064935064932e-04*w41*Jinv20+4.2918019480519480e-04*Jinv12*w13+4.2918019480519480e-04*w19*Jinv10+3.6525974025974026e-05*w34*Jinv10-1.7349837662337662e-04*w32*Jinv21+1.0957792207792208e-04*Jinv21*w20+1.0957792207792208e-04*w6*Jinv00+1.3697240259740259e-04*w37*Jinv12+4.7483766233766234e-04*Jinv20*w56-4.9310064935064932e-04*Jinv22*w41-7.4878246753246752e-04*w26*Jinv20-1.0247564935064935e-04*w11*Jinv20-9.1314935064935065e-06*w52*Jinv12-3.6525974025974026e-05*Jinv21*w35-1.7349837662337662e-04*w31*Jinv11-7.4878246753246752e-04*Jinv22*w26-3.9265422077922079e-04*w46*Jinv11+4.2918019480519480e-04*Jinv20*w14-3.1960227272727273e-04*w5*Jinv21+4.7483766233766234e-04*Jinv02*w54+1.0957792207792208e-04*w4*Jinv12-7.4878246753246752e-04*w21*Jinv02-9.1314935064935065e-06*w49*Jinv10-1.5726461038961039e-04*Jinv21*w29-3.6525974025974026e-05*Jinv01*w33-4.9310064935064932e-04*w39*Jinv00+4.2918019480519480e-04*Jinv02*w12+1.0957792207792208e-04*Jinv01*w18-1.0247564935064935e-04*Jinv02*w9-1.0957792207792208e-04*w6*Jinv01+1.0957792207792208e-04*w7*Jinv10-3.9265422077922079e-04*w47*Jinv21+1.3697240259740259e-04*Jinv20*w38-7.4878246753246752e-04*Jinv22*w23+1.3697240259740259e-04*w31*Jinv12-7.8530844155844158e-04*w46*Jinv12-9.1314935064935065e-06*Jinv20*w53+1.0957792207792208e-04*Jinv22*w8-7.4878246753246752e-04*w22*Jinv10-9.1314935064935065e-06*Jinv22*w50+1.0957792207792208e-04*Jinv12*w7+1.3697240259740259e-04*Jinv02*w30-9.1314935064935065e-06*Jinv12*w49-9.1314935064935065e-06*w52*Jinv10-1.0957792207792208e-04*w7*Jinv11-1.0247564935064935e-04*w9*Jinv00+1.3697240259740259e-04*w37*Jinv10-9.1314935064935065e-06*Jinv21*w56-7.4878246753246752e-04*w24*Jinv00-6.4935064935064935e-05*Jinv02*w27+1.0957792207792208e-04*w6*Jinv02+4.2918019480519480e-04*w19*Jinv12+1.0957792207792208e-04*Jinv22*w5-7.8530844155844158e-04*w47*Jinv22+9.1314935064935065e-06*w49*Jinv11+2.7394480519480518e-04*Jinv21*w14-1.0247564935064935e-04*w10*Jinv10-4.9310064935064932e-04*Jinv20*w44-4.0584415584415584e-05*w58*Jinv12+4.2918019480519480e-04*Jinv02*w18+2.7394480519480518e-04*Jinv01*w12-1.0957792207792208e-04*Jinv21*w8-4.0584415584415584e-05*w57*Jinv00-4.0584415584415584e-05*Jinv20*w59+3.6525974025974026e-05*Jinv02*w33-1.0247564935064935e-04*w0*Jinv00+3.6525974025974026e-05*w35*Jinv20-9.1314935064935065e-06*Jinv01*w54-9.1314935064935065e-06*w55*Jinv11-7.4878246753246752e-04*w25*Jinv10-1.0247564935064935e-04*Jinv20*w2-2.7394480519480518e-04*w15*Jinv00-9.1314935064935065e-06*w48*Jinv02-2.7394480519480518e-04*Jinv20*w17+1.3697240259740259e-04*w30*Jinv00-4.9310064935064932e-04*w40*Jinv10+4.2918019480519480e-04*w20*Jinv20-6.4935064935064935e-05*Jinv22*w29+4.2918019480519480e-04*Jinv22*w14+5.6615259740259744e-04*w40*Jinv11+1.3697240259740259e-04*Jinv02*w36-4.9310064935064932e-04*Jinv12*w43+1.0957792207792208e-04*w5*Jinv20-1.7349837662337662e-04*Jinv01*w30-9.1314935064935065e-06*Jinv02*w51+4.7483766233766234e-04*Jinv22*w56-6.4935064935064935e-05*Jinv12*w28-3.7439123376623376e-04*w25*Jinv11+4.7483766233766234e-04*w55*Jinv10-4.9310064935064932e-04*w42*Jinv00-7.8530844155844158e-04*w47*Jinv20-2.7394480519480518e-04*w16*Jinv12+9.1314935064935065e-06*Jinv21*w50-1.5726461038961039e-04*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01-1.0247564935064935e-04*Jinv12*w1)*D;
  A[59][24]=(0.0)*D;
  A[59][25]=(0.0)*D;
  A[59][26]=(-4.2918019480519480e-04*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-1.0957792207792208e-04*Jinv20*w8-4.2918019480519480e-04*w13*Jinv10+1.0592532467532468e-03*Jinv21*w44-1.0957792207792208e-04*w3*Jinv00-1.3697240259740259e-04*w32*Jinv20-2.1915584415584417e-04*Jinv01*w3+7.4878246753246752e-04*Jinv20*w23-4.7483766233766234e-04*Jinv00*w54+4.0584415584415584e-05*Jinv21*w59+1.0247564935064935e-04*Jinv21*w2+4.9310064935064932e-04*w43*Jinv10+1.0592532467532468e-03*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+9.6793831168831166e-04*w16*Jinv11+9.6793831168831166e-04*Jinv21*w17+3.9265422077922079e-04*Jinv01*w45+4.0584415584415584e-05*w58*Jinv10+9.1314935064935065e-06*w48*Jinv00+4.0584415584415584e-05*w58*Jinv11+4.0584415584415584e-05*w57*Jinv01+7.4878246753246752e-04*Jinv21*w26+1.0247564935064935e-04*w0*Jinv01+1.0247564935064935e-04*w1*Jinv10+2.7394480519480518e-04*w16*Jinv10+1.0592532467532468e-03*w43*Jinv11+7.8530844155844158e-04*w45*Jinv00+5.2962662337662338e-04*Jinv21*w41+6.4935064935064935e-05*Jinv00*w27+1.0247564935064935e-04*Jinv11*w1-3.1960227272727273e-04*w13*Jinv11+9.6793831168831166e-04*w15*Jinv01-9.2329545454545453e-05*w28*Jinv11+9.1314935064935065e-06*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29+7.8530844155844158e-04*w46*Jinv10-3.6525974025974026e-05*Jinv00*w33-1.3697240259740259e-04*w31*Jinv10+7.4878246753246752e-04*w21*Jinv00+5.2962662337662338e-04*Jinv01*w39-4.2918019480519480e-04*Jinv00*w18-1.3697240259740259e-04*w36*Jinv00-3.1047077922077924e-04*Jinv21*w38+7.4878246753246752e-04*Jinv01*w24-7.3051948051948052e-05*w34*Jinv11-1.5523538961038962e-04*w19*Jinv11+1.8262987012987013e-05*Jinv21*w53+9.1314935064935065e-06*w51*Jinv00-1.0957792207792208e-04*w4*Jinv10-2.1915584415584417e-04*w4*Jinv11+4.9310064935064932e-04*w41*Jinv20-4.2918019480519480e-04*w19*Jinv10-3.6525974025974026e-05*w34*Jinv10-1.3697240259740259e-04*w32*Jinv21-1.5523538961038962e-04*Jinv21*w20-1.0957792207792208e-04*w6*Jinv00-4.7483766233766234e-04*Jinv20*w56+7.4878246753246752e-04*w26*Jinv20+1.0247564935064935e-04*w11*Jinv20-7.3051948051948052e-05*Jinv21*w35-1.3697240259740259e-04*w31*Jinv11+3.9265422077922079e-04*w46*Jinv11-4.2918019480519480e-04*Jinv20*w14-2.1915584415584417e-04*w5*Jinv21+9.1314935064935065e-06*w49*Jinv10-9.2329545454545453e-05*Jinv21*w29-7.3051948051948052e-05*Jinv01*w33+4.9310064935064932e-04*w39*Jinv00-1.5523538961038962e-04*Jinv01*w18-3.1047077922077924e-04*w37*Jinv11-4.2918019480519480e-04*w6*Jinv01-1.0957792207792208e-04*w7*Jinv10+3.9265422077922079e-04*w47*Jinv21-1.3697240259740259e-04*Jinv20*w38+1.8262987012987013e-05*w52*Jinv11+9.1314935064935065e-06*Jinv20*w53+7.4878246753246752e-04*w22*Jinv10+3.7439123376623376e-04*w22*Jinv11+3.7439123376623376e-04*w21*Jinv01+1.8262987012987013e-05*Jinv01*w51-3.1047077922077924e-04*w36*Jinv01+9.1314935064935065e-06*w52*Jinv10-4.2918019480519480e-04*w7*Jinv11+1.0247564935064935e-04*w9*Jinv00-1.3697240259740259e-04*w37*Jinv10-4.8396915584415583e-04*Jinv21*w56+7.4878246753246752e-04*w24*Jinv00+9.1314935064935065e-06*w49*Jinv11-3.1960227272727273e-04*Jinv21*w14+1.0247564935064935e-04*w10*Jinv10+4.9310064935064932e-04*Jinv20*w44-3.1960227272727273e-04*Jinv01*w12-4.2918019480519480e-04*Jinv21*w8+4.0584415584415584e-05*w57*Jinv00+4.0584415584415584e-05*Jinv20*w59+1.0247564935064935e-04*w0*Jinv00-3.6525974025974026e-05*w35*Jinv20+3.7439123376623376e-04*Jinv21*w23-4.8396915584415583e-04*Jinv01*w54-4.8396915584415583e-04*w55*Jinv11+7.4878246753246752e-04*w25*Jinv10+1.0247564935064935e-04*Jinv20*w2+2.7394480519480518e-04*w15*Jinv00-1.4204545454545455e-05*Jinv01*w9+2.7394480519480518e-04*Jinv20*w17-1.3697240259740259e-04*w30*Jinv00+4.9310064935064932e-04*w40*Jinv10-4.2918019480519480e-04*w20*Jinv20+5.2962662337662338e-04*w40*Jinv11-1.0957792207792208e-04*w5*Jinv20-1.3697240259740259e-04*Jinv01*w30+7.4878246753246752e-04*w25*Jinv11-4.7483766233766234e-04*w55*Jinv10+4.9310064935064932e-04*w42*Jinv00+7.8530844155844158e-04*w47*Jinv20+9.1314935064935065e-06*Jinv21*w50-9.2329545454545453e-05*Jinv01*w27+9.1314935064935065e-06*w48*Jinv01-1.4204545454545455e-05*w10*Jinv11)*D;
  A[59][27]=(0.0)*D;
  A[59][28]=(0.0)*D;
  A[59][29]=(-4.2613636363636367e-05*Jinv21*w44+1.0247564935064935e-04*Jinv01*w3+1.8939393939393939e-04*Jinv21*w59+5.4112554112554113e-05*Jinv21*w2-4.2613636363636367e-05*w42*Jinv01+5.4112554112554113e-05*w11*Jinv21-1.9480519480519481e-04*w16*Jinv11-1.9480519480519481e-04*Jinv21*w17+2.0495129870129870e-04*Jinv01*w45+1.8939393939393939e-04*w58*Jinv11+1.8939393939393939e-04*w57*Jinv01+1.5726461038961039e-04*Jinv21*w26+5.4112554112554113e-05*w0*Jinv01-4.2613636363636367e-05*w43*Jinv11-4.2613636363636367e-05*Jinv21*w41+5.4112554112554113e-05*Jinv11*w1-1.1668019480519480e-04*w13*Jinv11-1.9480519480519481e-04*w15*Jinv01+9.4696969696969697e-05*w28*Jinv11-4.2613636363636367e-05*Jinv01*w39-1.4204545454545455e-05*Jinv21*w38+1.5726461038961039e-04*Jinv01*w24+3.0438311688311688e-05*w34*Jinv11-1.1668019480519480e-04*w19*Jinv11+6.4935064935064935e-05*Jinv21*w53+1.0247564935064935e-04*w4*Jinv11-1.4204545454545455e-05*w32*Jinv21-1.1668019480519480e-04*Jinv21*w20+3.0438311688311688e-05*Jinv21*w35-1.4204545454545455e-05*w31*Jinv11+2.0495129870129870e-04*w46*Jinv11+1.0247564935064935e-04*w5*Jinv21+9.4696969696969697e-05*Jinv21*w29+3.0438311688311688e-05*Jinv01*w33-1.1668019480519480e-04*Jinv01*w18-1.4204545454545455e-05*w37*Jinv11+1.0247564935064935e-04*w6*Jinv01+2.0495129870129870e-04*w47*Jinv21+6.4935064935064935e-05*w52*Jinv11+1.5726461038961039e-04*w22*Jinv11+1.5726461038961039e-04*w21*Jinv01+6.4935064935064935e-05*Jinv01*w51-1.4204545454545455e-05*w36*Jinv01+1.0247564935064935e-04*w7*Jinv11+8.3198051948051951e-05*Jinv21*w56+6.4935064935064935e-05*w49*Jinv11-1.1668019480519480e-04*Jinv21*w14-1.1668019480519480e-04*Jinv01*w12+1.0247564935064935e-04*Jinv21*w8+1.5726461038961039e-04*Jinv21*w23+8.3198051948051951e-05*Jinv01*w54+8.3198051948051951e-05*w55*Jinv11+5.4112554112554113e-05*Jinv01*w9-4.2613636363636367e-05*w40*Jinv11-1.4204545454545455e-05*Jinv01*w30+1.5726461038961039e-04*w25*Jinv11+6.4935064935064935e-05*Jinv21*w50+9.4696969696969697e-05*Jinv01*w27+6.4935064935064935e-05*w48*Jinv01+5.4112554112554113e-05*w10*Jinv11)*D;
  A[59][30]=(0.0)*D;
  A[59][31]=(0.0)*D;
  A[59][32]=(-3.9265422077922079e-04*Jinv00*w12+1.4204545454545455e-05*w28*Jinv10+5.4788961038961042e-05*w40*Jinv12+5.5702110389610384e-04*Jinv22*w53+1.7349837662337662e-04*Jinv20*w8+6.4935064935064935e-04*w57*Jinv02-3.9265422077922079e-04*w13*Jinv10+5.4788961038961042e-05*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44-3.9265422077922079e-04*w3*Jinv00-6.5746753246753243e-04*w32*Jinv20-3.9265422077922079e-04*Jinv01*w3+1.7349837662337662e-04*Jinv20*w23-3.5612824675324673e-04*Jinv22*w38+5.4788961038961036e-04*Jinv00*w54+6.0876623376623375e-04*Jinv21*w59+2.7394480519480521e-05*w25*Jinv12+1.2175324675324675e-04*w0*Jinv02+5.6818181818181818e-05*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01+1.4204545454545455e-05*w11*Jinv21-5.2962662337662338e-04*w16*Jinv11-5.2962662337662338e-04*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45+5.5702110389610384e-04*Jinv12*w55+6.0876623376623375e-04*w58*Jinv10-2.5568181818181820e-04*w15*Jinv02+1.1668019480519480e-04*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+2.7394480519480521e-05*Jinv02*w24+5.4788961038961042e-05*Jinv22*w35+6.0876623376623375e-04*w58*Jinv11+6.0876623376623375e-04*w57*Jinv01+1.3697240259740259e-04*Jinv21*w26+5.6818181818181818e-05*w0*Jinv01+5.6818181818181818e-05*w1*Jinv10-5.2962662337662338e-04*w16*Jinv10+2.7394480519480521e-05*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-2.1915584415584417e-04*w45*Jinv00-4.3831168831168834e-04*Jinv21*w41-2.5568181818181820e-04*w42*Jinv02+1.4204545454545455e-05*Jinv00*w27+1.1668019480519480e-04*w11*Jinv22+5.6818181818181818e-05*Jinv11*w1-3.9265422077922079e-04*w13*Jinv11-5.2962662337662338e-04*w15*Jinv01+1.4204545454545455e-05*w28*Jinv11+1.0957792207792207e-03*Jinv20*w50+1.4204545454545455e-05*Jinv20*w29-2.1915584415584417e-04*w46*Jinv10-2.5568181818181820e-04*Jinv22*w17-4.3831168831168834e-04*Jinv00*w33-2.5568181818181820e-04*w22*Jinv12-6.5746753246753243e-04*w31*Jinv10+3.5612824675324673e-04*Jinv02*w3+1.7349837662337662e-04*w21*Jinv00+1.2784090909090910e-04*w32*Jinv22-4.3831168831168834e-04*Jinv01*w39+1.2175324675324675e-04*Jinv22*w2+1.3697240259740259e-04*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-2.1915584415584417e-04*Jinv21*w38-3.5612824675324673e-04*Jinv02*w45+1.3697240259740259e-04*Jinv01*w24+6.4935064935064935e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11+1.3697240259740259e-04*w19*Jinv11+5.4788961038961036e-04*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+5.4788961038961042e-05*Jinv12*w34-2.5568181818181820e-04*Jinv22*w44-3.9265422077922079e-04*w4*Jinv10-3.9265422077922079e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+3.5612824675324673e-04*Jinv12*w13+1.3697240259740259e-04*w19*Jinv10-4.3831168831168834e-04*w34*Jinv10-6.5746753246753243e-04*w32*Jinv21+1.3697240259740259e-04*Jinv21*w20+1.7349837662337662e-04*w6*Jinv00-3.5612824675324673e-04*w37*Jinv12+5.4788961038961036e-04*Jinv20*w56+5.4788961038961042e-05*Jinv22*w41+1.3697240259740259e-04*w26*Jinv20+1.4204545454545455e-05*w11*Jinv20+5.5702110389610384e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-6.5746753246753243e-04*w31*Jinv11+2.7394480519480521e-05*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11-3.9265422077922079e-04*Jinv20*w14-3.9265422077922079e-04*w5*Jinv21+5.5702110389610384e-04*Jinv02*w54+3.5612824675324673e-04*w4*Jinv12-2.5568181818181820e-04*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10+1.4204545454545455e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+3.5612824675324673e-04*Jinv02*w12+1.3697240259740259e-04*Jinv01*w18-2.1915584415584417e-04*w37*Jinv11+1.1668019480519480e-04*Jinv02*w9+1.7349837662337662e-04*w6*Jinv01+1.7349837662337662e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38-2.5568181818181820e-04*Jinv22*w23+5.4788961038961036e-04*w52*Jinv11+1.2784090909090910e-04*w31*Jinv12-3.5612824675324673e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53-2.5568181818181820e-04*Jinv22*w8+1.7349837662337662e-04*w22*Jinv10+6.2094155844155847e-04*Jinv22*w50+1.7349837662337662e-04*w22*Jinv11-2.5568181818181820e-04*Jinv12*w7+1.7349837662337662e-04*w21*Jinv01+5.4788961038961036e-04*Jinv01*w51+1.2784090909090910e-04*Jinv02*w30+6.2094155844155847e-04*Jinv12*w49-2.1915584415584417e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10+1.7349837662337662e-04*w7*Jinv11+1.4204545454545455e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+1.3697240259740259e-04*w24*Jinv00+1.1668019480519480e-04*Jinv02*w27-2.5568181818181820e-04*w6*Jinv02+2.7394480519480521e-05*w19*Jinv12+3.5612824675324673e-04*Jinv22*w5-3.5612824675324673e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11-3.9265422077922079e-04*Jinv21*w14+1.4204545454545455e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44+6.4935064935064935e-04*w58*Jinv12+2.7394480519480521e-05*Jinv02*w18-3.9265422077922079e-04*Jinv01*w12+1.7349837662337662e-04*Jinv21*w8+6.0876623376623375e-04*w57*Jinv00+6.0876623376623375e-04*Jinv20*w59+5.4788961038961042e-05*Jinv02*w33+5.6818181818181818e-05*w0*Jinv00-4.3831168831168834e-04*w35*Jinv20+1.7349837662337662e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+1.3697240259740259e-04*w25*Jinv10+5.6818181818181818e-05*Jinv20*w2-5.2962662337662338e-04*w15*Jinv00+6.2094155844155847e-04*w48*Jinv02+1.4204545454545455e-05*Jinv01*w9-5.2962662337662338e-04*Jinv20*w17-6.5746753246753243e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+1.3697240259740259e-04*w20*Jinv20+1.1668019480519480e-04*Jinv22*w29+3.5612824675324673e-04*Jinv22*w14-4.3831168831168834e-04*w40*Jinv11-3.5612824675324673e-04*Jinv02*w36-2.5568181818181820e-04*Jinv12*w43-3.9265422077922079e-04*w5*Jinv20-6.5746753246753243e-04*Jinv01*w30+5.5702110389610384e-04*Jinv02*w51+5.5702110389610384e-04*Jinv22*w56+1.1668019480519480e-04*Jinv12*w28+1.3697240259740259e-04*w25*Jinv11+5.4788961038961036e-04*w55*Jinv10-2.1915584415584417e-04*w42*Jinv00-2.1915584415584417e-04*w47*Jinv20-2.5568181818181820e-04*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50+1.4204545454545455e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01+1.2175324675324675e-04*Jinv12*w1+1.4204545454545455e-05*w10*Jinv11)*D;
  A[59][33]=(0.0)*D;
  A[59][34]=(0.0)*D;
  A[59][35]=(5.2962662337662338e-04*Jinv00*w12+9.1314935064935063e-04*w40*Jinv12+1.0957792207792208e-04*Jinv22*w53-5.6615259740259744e-04*Jinv20*w8-2.4350649350649351e-04*w57*Jinv02+5.2962662337662338e-04*w13*Jinv10+9.1314935064935063e-04*Jinv02*w39-4.3831168831168834e-04*Jinv21*w44+5.6615259740259744e-04*w3*Jinv00+4.3831168831168834e-04*w32*Jinv20+7.3051948051948052e-05*Jinv01*w3-3.6525974025974026e-05*Jinv20*w23-1.6436688311688311e-04*Jinv22*w38-7.3051948051948055e-04*Jinv21*w59-3.4699675324675324e-04*w25*Jinv12-3.0438311688311688e-05*Jinv21*w2-2.1915584415584417e-04*w43*Jinv10-4.3831168831168834e-04*w42*Jinv01+1.2175324675324675e-05*w11*Jinv21-1.1323051948051949e-03*w16*Jinv11-1.1323051948051949e-03*Jinv21*w17-2.1915584415584417e-04*Jinv01*w45+5.4788961038961042e-05*Jinv12*w55+1.6436688311688311e-03*w15*Jinv02+4.2613636363636367e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-3.4699675324675324e-04*Jinv02*w24+1.8262987012987013e-03*Jinv22*w35-7.3051948051948055e-04*w58*Jinv11-7.3051948051948055e-04*w57*Jinv01+7.3051948051948052e-05*Jinv21*w26-3.0438311688311688e-05*w0*Jinv01-4.2613636363636367e-05*w1*Jinv10-2.1915584415584417e-04*Jinv22*w20-4.3831168831168834e-04*w43*Jinv11-2.1915584415584417e-04*Jinv21*w41+6.9399350649350649e-04*w42*Jinv02+4.2613636363636367e-05*w11*Jinv22-3.0438311688311688e-05*Jinv11*w1+3.6525974025974026e-05*w13*Jinv11-1.1323051948051949e-03*w15*Jinv01-3.0438311688311688e-05*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50+1.6436688311688311e-03*Jinv22*w17-3.8352272727272725e-04*w22*Jinv12+4.3831168831168834e-04*w31*Jinv10+6.2094155844155847e-04*Jinv02*w3-3.6525974025974026e-05*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22-2.1915584415584417e-04*Jinv01*w39-5.2962662337662338e-04*Jinv00*w18-4.3831168831168834e-04*w36*Jinv00-6.5746753246753243e-04*Jinv21*w38-2.5568181818181820e-04*Jinv02*w45+7.3051948051948052e-05*Jinv01*w24-2.4350649350649351e-04*Jinv22*w59-4.3831168831168834e-04*w34*Jinv11-4.9310064935064932e-04*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53+5.4788961038961036e-04*w51*Jinv00+1.8262987012987013e-03*Jinv12*w34+6.9399350649350649e-04*Jinv22*w44+5.6615259740259744e-04*w4*Jinv10+7.3051948051948052e-05*w4*Jinv11+2.1915584415584417e-04*w41*Jinv20+3.1047077922077924e-04*Jinv12*w13-5.2962662337662338e-04*w19*Jinv10-2.1915584415584417e-04*w32*Jinv21-4.9310064935064932e-04*Jinv21*w20-5.6615259740259744e-04*w6*Jinv00-1.6436688311688311e-04*w37*Jinv12+9.1314935064935063e-04*Jinv22*w41+3.6525974025974026e-05*w26*Jinv20+4.2613636363636367e-05*w11*Jinv20+1.0957792207792208e-04*w52*Jinv12-4.3831168831168834e-04*Jinv21*w35-2.1915584415584417e-04*w31*Jinv11-3.4699675324675324e-04*Jinv22*w26-2.1915584415584417e-04*w46*Jinv11+5.2962662337662338e-04*Jinv20*w14+7.3051948051948052e-05*w5*Jinv21+5.4788961038961042e-05*Jinv02*w54+6.2094155844155847e-04*w4*Jinv12-3.8352272727272725e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10-3.0438311688311688e-05*Jinv21*w29-4.3831168831168834e-04*Jinv01*w33+2.1915584415584417e-04*w39*Jinv00+3.1047077922077924e-04*Jinv02*w12-4.9310064935064932e-04*Jinv01*w18-6.5746753246753243e-04*w37*Jinv11+4.2613636363636367e-05*Jinv02*w9-4.9310064935064932e-04*w6*Jinv01-5.6615259740259744e-04*w7*Jinv10-2.1915584415584417e-04*w47*Jinv21-4.3831168831168834e-04*Jinv20*w38-3.8352272727272725e-04*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+2.7394480519480518e-04*w31*Jinv12-2.5568181818181820e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53+5.4788961038961042e-05*Jinv22*w8-3.6525974025974026e-05*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50+3.6525974025974026e-05*w22*Jinv11+5.4788961038961042e-05*Jinv12*w7+3.6525974025974026e-05*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+2.7394480519480518e-04*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49-6.5746753246753243e-04*w36*Jinv01+5.4788961038961036e-04*w52*Jinv10-4.9310064935064932e-04*w7*Jinv11+4.2613636363636367e-05*w9*Jinv00-4.3831168831168834e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56+3.6525974025974026e-05*w24*Jinv00+1.9480519480519481e-04*Jinv02*w27+5.4788961038961042e-05*w6*Jinv02-2.1915584415584417e-04*w19*Jinv12+6.2094155844155847e-04*Jinv22*w5-2.5568181818181820e-04*w47*Jinv22+5.4788961038961036e-04*w49*Jinv11+3.6525974025974026e-05*Jinv21*w14+4.2613636363636367e-05*w10*Jinv10-2.1915584415584417e-04*Jinv20*w44-2.4350649350649351e-04*w58*Jinv12-2.1915584415584417e-04*Jinv02*w18+3.6525974025974026e-05*Jinv01*w12-4.9310064935064932e-04*Jinv21*w8+1.8262987012987013e-03*Jinv02*w33-4.2613636363636367e-05*w0*Jinv00+3.6525974025974026e-05*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11+3.6525974025974026e-05*w25*Jinv10-4.2613636363636367e-05*Jinv20*w2-4.3831168831168834e-04*w48*Jinv02+1.2175324675324675e-05*Jinv01*w9+4.3831168831168834e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10-5.2962662337662338e-04*w20*Jinv20+1.9480519480519481e-04*Jinv22*w29+3.1047077922077924e-04*Jinv22*w14-2.1915584415584417e-04*w40*Jinv11-1.6436688311688311e-04*Jinv02*w36+6.9399350649350649e-04*Jinv12*w43+5.6615259740259744e-04*w5*Jinv20-2.1915584415584417e-04*Jinv01*w30+1.0957792207792208e-04*Jinv02*w51+5.4788961038961042e-05*Jinv22*w56+1.9480519480519481e-04*Jinv12*w28+7.3051948051948052e-05*w25*Jinv11-2.1915584415584417e-04*w42*Jinv00+1.6436688311688311e-03*w16*Jinv12+5.4788961038961036e-04*Jinv21*w50-3.0438311688311688e-05*Jinv01*w27+5.4788961038961036e-04*w48*Jinv01+1.2175324675324675e-05*w10*Jinv11)*D;
  A[59][36]=(0.0)*D;
  A[59][37]=(0.0)*D;
  A[59][38]=(-1.3697240259740259e-04*Jinv00*w12-1.4204545454545455e-05*w28*Jinv10-3.6525974025974026e-05*w40*Jinv12-4.7483766233766234e-04*Jinv22*w53+3.9265422077922079e-04*Jinv20*w8+4.0584415584415584e-05*w57*Jinv02-1.3697240259740259e-04*w13*Jinv10-3.6525974025974026e-05*Jinv02*w39-1.7349837662337662e-04*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20-1.3697240259740259e-04*Jinv20*w23+7.8530844155844158e-04*Jinv22*w38-5.4788961038961036e-04*Jinv00*w54-4.2918019480519480e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02+4.3831168831168834e-04*w43*Jinv10+9.1314935064935065e-06*Jinv12*w55-6.0876623376623375e-04*w58*Jinv10+2.7394480519480518e-04*w15*Jinv02+6.4935064935064935e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00-4.2918019480519480e-04*Jinv02*w24+4.9310064935064932e-04*Jinv22*w35-1.4204545454545455e-05*w1*Jinv10+5.2962662337662338e-04*w16*Jinv10+7.4878246753246752e-04*Jinv22*w20+2.1915584415584417e-04*w45*Jinv00+4.9310064935064932e-04*w42*Jinv02-1.4204545454545455e-05*Jinv00*w27+6.4935064935064935e-05*w11*Jinv22-5.4788961038961036e-04*Jinv20*w50-1.4204545454545455e-05*Jinv20*w29+2.1915584415584417e-04*w46*Jinv10+2.7394480519480518e-04*Jinv22*w17+4.3831168831168834e-04*Jinv00*w33-1.0957792207792208e-04*w22*Jinv12+2.1915584415584417e-04*w31*Jinv10-4.2918019480519480e-04*Jinv02*w3-1.3697240259740259e-04*w21*Jinv00-1.3697240259740259e-04*w32*Jinv22+1.0247564935064935e-04*Jinv22*w2+3.9265422077922079e-04*Jinv00*w18+6.5746753246753243e-04*w36*Jinv00-1.3697240259740259e-04*Jinv02*w45+4.0584415584415584e-05*Jinv22*w59-1.0957792207792207e-03*w51*Jinv00+4.9310064935064932e-04*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.7349837662337662e-04*w4*Jinv10+2.1915584415584417e-04*w41*Jinv20-1.0957792207792208e-04*Jinv12*w13+3.9265422077922079e-04*w19*Jinv10+4.3831168831168834e-04*w34*Jinv10+3.9265422077922079e-04*w6*Jinv00+7.8530844155844158e-04*w37*Jinv12-5.4788961038961036e-04*Jinv20*w56-3.6525974025974026e-05*Jinv22*w41-1.7349837662337662e-04*w26*Jinv20-5.6818181818181818e-05*w11*Jinv20-4.7483766233766234e-04*w52*Jinv12-4.2918019480519480e-04*Jinv22*w26-1.3697240259740259e-04*Jinv20*w14+9.1314935064935065e-06*Jinv02*w54-4.2918019480519480e-04*w4*Jinv12-1.0957792207792208e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+2.1915584415584417e-04*w39*Jinv00-1.0957792207792208e-04*Jinv02*w12+6.4935064935064935e-05*Jinv02*w9+3.9265422077922079e-04*w7*Jinv10+6.5746753246753243e-04*Jinv20*w38-1.0957792207792208e-04*Jinv22*w23-1.3697240259740259e-04*w31*Jinv12-1.3697240259740259e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+7.4878246753246752e-04*Jinv22*w8-1.3697240259740259e-04*w22*Jinv10+9.1314935064935065e-06*Jinv22*w50+7.4878246753246752e-04*Jinv12*w7-1.3697240259740259e-04*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49-1.0957792207792207e-03*w52*Jinv10-5.6818181818181818e-05*w9*Jinv00+6.5746753246753243e-04*w37*Jinv10-1.7349837662337662e-04*w24*Jinv00+1.0247564935064935e-04*Jinv02*w27+7.4878246753246752e-04*w6*Jinv02+7.4878246753246752e-04*w19*Jinv12-4.2918019480519480e-04*Jinv22*w5-1.3697240259740259e-04*w47*Jinv22-5.6818181818181818e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+4.0584415584415584e-05*w58*Jinv12+7.4878246753246752e-04*Jinv02*w18-6.0876623376623375e-04*w57*Jinv00-6.0876623376623375e-04*Jinv20*w59+4.9310064935064932e-04*Jinv02*w33-1.4204545454545455e-05*w0*Jinv00+4.3831168831168834e-04*w35*Jinv20-1.7349837662337662e-04*w25*Jinv10-1.4204545454545455e-05*Jinv20*w2+5.2962662337662338e-04*w15*Jinv00+9.1314935064935065e-06*w48*Jinv02+5.2962662337662338e-04*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00+2.1915584415584417e-04*w40*Jinv10+3.9265422077922079e-04*w20*Jinv20+1.0247564935064935e-04*Jinv22*w29-1.0957792207792208e-04*Jinv22*w14+7.8530844155844158e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43-1.7349837662337662e-04*w5*Jinv20-4.7483766233766234e-04*Jinv02*w51+9.1314935064935065e-06*Jinv22*w56+1.0247564935064935e-04*Jinv12*w28-5.4788961038961036e-04*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+2.1915584415584417e-04*w47*Jinv20+2.7394480519480518e-04*w16*Jinv12+1.0247564935064935e-04*Jinv12*w1)*D;
  A[59][39]=(0.0)*D;
  A[59][40]=(0.0)*D;
  A[59][41]=(7.3051948051948052e-05*Jinv00*w12+1.2175324675324675e-05*w28*Jinv10+1.8262987012987013e-03*w40*Jinv12+5.4788961038961042e-05*Jinv22*w53+3.6525974025974026e-05*Jinv20*w8-2.4350649350649351e-04*w57*Jinv02+7.3051948051948052e-05*w13*Jinv10+1.8262987012987013e-03*Jinv02*w39-2.1915584415584417e-04*Jinv21*w44+3.6525974025974026e-05*w3*Jinv00-2.1915584415584417e-04*w32*Jinv20+5.2962662337662338e-04*Jinv01*w3-4.9310064935064932e-04*Jinv20*w23-2.5568181818181820e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54-2.1915584415584417e-04*w25*Jinv12-4.2613636363636367e-05*Jinv21*w2-4.3831168831168834e-04*w43*Jinv10-2.1915584415584417e-04*w42*Jinv01-4.3831168831168834e-04*Jinv01*w45+1.0957792207792208e-04*Jinv12*w55-7.3051948051948055e-04*w58*Jinv10+1.6436688311688311e-03*w15*Jinv02+1.9480519480519481e-04*w10*Jinv12+5.4788961038961036e-04*w48*Jinv00-2.1915584415584417e-04*Jinv02*w24+9.1314935064935063e-04*Jinv22*w35-5.2962662337662338e-04*Jinv21*w26-4.2613636363636367e-05*w0*Jinv01-3.0438311688311688e-05*w1*Jinv10-1.1323051948051949e-03*w16*Jinv10-3.4699675324675324e-04*Jinv22*w20-2.1915584415584417e-04*w43*Jinv11-6.5746753246753243e-04*w45*Jinv00+6.9399350649350649e-04*w42*Jinv02+1.2175324675324675e-05*Jinv00*w27+1.9480519480519481e-04*w11*Jinv22-4.2613636363636367e-05*Jinv11*w1+5.6615259740259744e-04*w13*Jinv11+4.2613636363636367e-05*w28*Jinv11+5.4788961038961036e-04*Jinv20*w50+1.2175324675324675e-05*Jinv20*w29-6.5746753246753243e-04*w46*Jinv10+1.6436688311688311e-03*Jinv22*w17-2.1915584415584417e-04*Jinv00*w33+5.4788961038961042e-05*w22*Jinv12-2.1915584415584417e-04*w31*Jinv10+3.1047077922077924e-04*Jinv02*w3-4.9310064935064932e-04*w21*Jinv00+2.7394480519480518e-04*w32*Jinv22+7.3051948051948052e-05*Jinv00*w18-2.1915584415584417e-04*w36*Jinv00-1.6436688311688311e-04*Jinv02*w45-5.2962662337662338e-04*Jinv01*w24-2.4350649350649351e-04*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11+3.6525974025974026e-05*w19*Jinv11+5.4788961038961036e-04*w51*Jinv00+9.1314935064935063e-04*Jinv12*w34+6.9399350649350649e-04*Jinv22*w44+3.6525974025974026e-05*w4*Jinv10+5.2962662337662338e-04*w4*Jinv11-4.3831168831168834e-04*w41*Jinv20+6.2094155844155847e-04*Jinv12*w13+7.3051948051948052e-05*w19*Jinv10-2.1915584415584417e-04*w34*Jinv10+4.3831168831168834e-04*w32*Jinv21+3.6525974025974026e-05*Jinv21*w20+3.6525974025974026e-05*w6*Jinv00-2.5568181818181820e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56+1.8262987012987013e-03*Jinv22*w41-4.9310064935064932e-04*w26*Jinv20-3.0438311688311688e-05*w11*Jinv20+5.4788961038961042e-05*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+4.3831168831168834e-04*w31*Jinv11-2.1915584415584417e-04*Jinv22*w26-4.3831168831168834e-04*w46*Jinv11+7.3051948051948052e-05*Jinv20*w14+5.2962662337662338e-04*w5*Jinv21+1.0957792207792208e-04*Jinv02*w54+3.1047077922077924e-04*w4*Jinv12+5.4788961038961042e-05*w21*Jinv02+5.4788961038961036e-04*w49*Jinv10+4.2613636363636367e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-4.3831168831168834e-04*w39*Jinv00+6.2094155844155847e-04*Jinv02*w12+3.6525974025974026e-05*Jinv01*w18+1.9480519480519481e-04*Jinv02*w9-3.6525974025974026e-05*w6*Jinv01+3.6525974025974026e-05*w7*Jinv10-4.3831168831168834e-04*w47*Jinv21-2.1915584415584417e-04*Jinv20*w38+5.4788961038961042e-05*Jinv22*w23+2.7394480519480518e-04*w31*Jinv12-1.6436688311688311e-04*w46*Jinv12+5.4788961038961036e-04*Jinv20*w53-3.8352272727272725e-04*Jinv22*w8-4.9310064935064932e-04*w22*Jinv10-4.3831168831168834e-04*Jinv22*w50-5.6615259740259744e-04*w22*Jinv11-3.8352272727272725e-04*Jinv12*w7-5.6615259740259744e-04*w21*Jinv01+2.7394480519480518e-04*Jinv02*w30-4.3831168831168834e-04*Jinv12*w49+5.4788961038961036e-04*w52*Jinv10-3.6525974025974026e-05*w7*Jinv11-3.0438311688311688e-05*w9*Jinv00-2.1915584415584417e-04*w37*Jinv10+5.4788961038961036e-04*Jinv21*w56-4.9310064935064932e-04*w24*Jinv00+4.2613636363636367e-05*Jinv02*w27-3.8352272727272725e-04*w6*Jinv02-3.4699675324675324e-04*w19*Jinv12+3.1047077922077924e-04*Jinv22*w5-1.6436688311688311e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11+5.6615259740259744e-04*Jinv21*w14-3.0438311688311688e-05*w10*Jinv10-4.3831168831168834e-04*Jinv20*w44-2.4350649350649351e-04*w58*Jinv12-3.4699675324675324e-04*Jinv02*w18+5.6615259740259744e-04*Jinv01*w12-3.6525974025974026e-05*Jinv21*w8-7.3051948051948055e-04*w57*Jinv00-7.3051948051948055e-04*Jinv20*w59+9.1314935064935063e-04*Jinv02*w33-3.0438311688311688e-05*w0*Jinv00-2.1915584415584417e-04*w35*Jinv20-5.6615259740259744e-04*Jinv21*w23+5.4788961038961036e-04*Jinv01*w54+5.4788961038961036e-04*w55*Jinv11-4.9310064935064932e-04*w25*Jinv10-3.0438311688311688e-05*Jinv20*w2-1.1323051948051949e-03*w15*Jinv00-4.3831168831168834e-04*w48*Jinv02-1.1323051948051949e-03*Jinv20*w17-2.1915584415584417e-04*w30*Jinv00-4.3831168831168834e-04*w40*Jinv10+7.3051948051948052e-05*w20*Jinv20+4.2613636363636367e-05*Jinv22*w29+6.2094155844155847e-04*Jinv22*w14-2.5568181818181820e-04*Jinv02*w36+6.9399350649350649e-04*Jinv12*w43+3.6525974025974026e-05*w5*Jinv20+4.3831168831168834e-04*Jinv01*w30+5.4788961038961042e-05*Jinv02*w51+1.0957792207792208e-04*Jinv22*w56+4.2613636363636367e-05*Jinv12*w28-5.2962662337662338e-04*w25*Jinv11+1.0957792207792207e-03*w55*Jinv10-4.3831168831168834e-04*w42*Jinv00-6.5746753246753243e-04*w47*Jinv20+1.6436688311688311e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50+4.2613636363636367e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01)*D;
  A[59][42]=(0.0)*D;
  A[59][43]=(0.0)*D;
  A[59][44]=(-7.3051948051948052e-05*Jinv00*w12-1.2175324675324675e-05*w28*Jinv10+1.1323051948051949e-03*w40*Jinv12-9.8620129870129864e-04*Jinv22*w53-3.6525974025974026e-05*Jinv20*w8+4.8701298701298701e-04*w57*Jinv02-7.3051948051948052e-05*w13*Jinv10+1.1323051948051949e-03*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-3.6525974025974026e-05*w3*Jinv00+2.1915584415584417e-04*w32*Jinv20-7.3051948051948052e-05*Jinv01*w3+4.9310064935064932e-04*Jinv20*w23+4.9310064935064932e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54+7.3051948051948055e-04*Jinv21*w59+5.4788961038961036e-04*w25*Jinv12+2.2524350649350650e-04*w0*Jinv02+3.0438311688311688e-05*Jinv21*w2+4.3831168831168834e-04*w43*Jinv10+4.3831168831168834e-04*w42*Jinv01-1.2175324675324675e-05*w11*Jinv21+1.1323051948051949e-03*w16*Jinv11+1.1323051948051949e-03*Jinv21*w17+2.1915584415584417e-04*Jinv01*w45-9.8620129870129864e-04*Jinv12*w55+7.3051948051948055e-04*w58*Jinv10+2.7759740259740259e-03*w15*Jinv02+3.0438311688311688e-05*w10*Jinv12-5.4788961038961036e-04*w48*Jinv00+5.4788961038961036e-04*Jinv02*w24+1.1323051948051949e-03*Jinv22*w35+7.3051948051948055e-04*w58*Jinv11+7.3051948051948055e-04*w57*Jinv01-7.3051948051948052e-05*Jinv21*w26+3.0438311688311688e-05*w0*Jinv01+3.0438311688311688e-05*w1*Jinv10+1.1323051948051949e-03*w16*Jinv10+5.4788961038961036e-04*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+6.5746753246753243e-04*w45*Jinv00+2.1915584415584417e-04*Jinv21*w41+2.2646103896103898e-03*w42*Jinv02-1.2175324675324675e-05*Jinv00*w27+3.0438311688311688e-05*w11*Jinv22+3.0438311688311688e-05*Jinv11*w1-3.6525974025974026e-05*w13*Jinv11+1.1323051948051949e-03*w15*Jinv01+3.0438311688311688e-05*w28*Jinv11-5.4788961038961036e-04*Jinv20*w50-1.2175324675324675e-05*Jinv20*w29+6.5746753246753243e-04*w46*Jinv10+2.7759740259740259e-03*Jinv22*w17+2.1915584415584417e-04*Jinv00*w33+2.7394480519480518e-04*w22*Jinv12+2.1915584415584417e-04*w31*Jinv10-4.2004870129870131e-04*Jinv02*w3+4.9310064935064932e-04*w21*Jinv00-3.6525974025974026e-05*w32*Jinv22+2.1915584415584417e-04*Jinv01*w39+2.2524350649350650e-04*Jinv22*w2-7.3051948051948052e-05*Jinv00*w18+2.1915584415584417e-04*w36*Jinv00+6.5746753246753243e-04*Jinv21*w38+4.9310064935064932e-04*Jinv02*w45-7.3051948051948052e-05*Jinv01*w24+4.8701298701298701e-04*Jinv22*w59+4.3831168831168834e-04*w34*Jinv11+4.9310064935064932e-04*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-5.4788961038961036e-04*w51*Jinv00+1.1323051948051949e-03*Jinv12*w34+2.2646103896103898e-03*Jinv22*w44-3.6525974025974026e-05*w4*Jinv10-7.3051948051948052e-05*w4*Jinv11+4.3831168831168834e-04*w41*Jinv20-4.2004870129870131e-04*Jinv12*w13-7.3051948051948052e-05*w19*Jinv10+2.1915584415584417e-04*w34*Jinv10+2.1915584415584417e-04*w32*Jinv21+4.9310064935064932e-04*Jinv21*w20-3.6525974025974026e-05*w6*Jinv00+4.9310064935064932e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56+1.1323051948051949e-03*Jinv22*w41+4.9310064935064932e-04*w26*Jinv20+3.0438311688311688e-05*w11*Jinv20-9.8620129870129864e-04*w52*Jinv12+4.3831168831168834e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+5.4788961038961036e-04*Jinv22*w26+2.1915584415584417e-04*w46*Jinv11-7.3051948051948052e-05*Jinv20*w14-7.3051948051948052e-05*w5*Jinv21-9.8620129870129864e-04*Jinv02*w54-4.2004870129870131e-04*w4*Jinv12+2.7394480519480518e-04*w21*Jinv02-5.4788961038961036e-04*w49*Jinv10+3.0438311688311688e-05*Jinv21*w29+4.3831168831168834e-04*Jinv01*w33+4.3831168831168834e-04*w39*Jinv00-4.2004870129870131e-04*Jinv02*w12+4.9310064935064932e-04*Jinv01*w18+6.5746753246753243e-04*w37*Jinv11+3.0438311688311688e-05*Jinv02*w9+4.9310064935064932e-04*w6*Jinv01-3.6525974025974026e-05*w7*Jinv10+2.1915584415584417e-04*w47*Jinv21+2.1915584415584417e-04*Jinv20*w38+2.7394480519480518e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11-3.6525974025974026e-05*w31*Jinv12+4.9310064935064932e-04*w46*Jinv12-5.4788961038961036e-04*Jinv20*w53+2.7394480519480518e-04*Jinv22*w8+4.9310064935064932e-04*w22*Jinv10-4.9310064935064932e-04*Jinv22*w50-3.6525974025974026e-05*w22*Jinv11+2.7394480519480518e-04*Jinv12*w7-3.6525974025974026e-05*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51-3.6525974025974026e-05*Jinv02*w30-4.9310064935064932e-04*Jinv12*w49+6.5746753246753243e-04*w36*Jinv01-5.4788961038961036e-04*w52*Jinv10+4.9310064935064932e-04*w7*Jinv11+3.0438311688311688e-05*w9*Jinv00+2.1915584415584417e-04*w37*Jinv10-5.4788961038961036e-04*Jinv21*w56+4.9310064935064932e-04*w24*Jinv00+3.0438311688311688e-05*Jinv02*w27+2.7394480519480518e-04*w6*Jinv02+5.4788961038961036e-04*w19*Jinv12-4.2004870129870131e-04*Jinv22*w5+4.9310064935064932e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11-3.6525974025974026e-05*Jinv21*w14+3.0438311688311688e-05*w10*Jinv10+4.3831168831168834e-04*Jinv20*w44+4.8701298701298701e-04*w58*Jinv12+5.4788961038961036e-04*Jinv02*w18-3.6525974025974026e-05*Jinv01*w12+4.9310064935064932e-04*Jinv21*w8+7.3051948051948055e-04*w57*Jinv00+7.3051948051948055e-04*Jinv20*w59+1.1323051948051949e-03*Jinv02*w33+3.0438311688311688e-05*w0*Jinv00+2.1915584415584417e-04*w35*Jinv20-3.6525974025974026e-05*Jinv21*w23-5.4788961038961036e-04*Jinv01*w54-5.4788961038961036e-04*w55*Jinv11+4.9310064935064932e-04*w25*Jinv10+3.0438311688311688e-05*Jinv20*w2+1.1323051948051949e-03*w15*Jinv00-4.9310064935064932e-04*w48*Jinv02-1.2175324675324675e-05*Jinv01*w9+1.1323051948051949e-03*Jinv20*w17+2.1915584415584417e-04*w30*Jinv00+4.3831168831168834e-04*w40*Jinv10-7.3051948051948052e-05*w20*Jinv20+3.0438311688311688e-05*Jinv22*w29-4.2004870129870131e-04*Jinv22*w14+2.1915584415584417e-04*w40*Jinv11+4.9310064935064932e-04*Jinv02*w36+2.2646103896103898e-03*Jinv12*w43-3.6525974025974026e-05*w5*Jinv20+2.1915584415584417e-04*Jinv01*w30-9.8620129870129864e-04*Jinv02*w51-9.8620129870129864e-04*Jinv22*w56+3.0438311688311688e-05*Jinv12*w28-7.3051948051948052e-05*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+4.3831168831168834e-04*w42*Jinv00+6.5746753246753243e-04*w47*Jinv20+2.7759740259740259e-03*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50+3.0438311688311688e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01+2.2524350649350650e-04*Jinv12*w1-1.2175324675324675e-05*w10*Jinv11)*D;
  A[59][45]=(0.0)*D;
  A[59][46]=(0.0)*D;
  A[59][47]=(4.9310064935064932e-04*w40*Jinv12+9.1314935064935065e-06*Jinv22*w53+4.0584415584415584e-05*w57*Jinv02+4.9310064935064932e-04*Jinv02*w39+4.3831168831168834e-04*Jinv21*w44-1.3697240259740259e-04*Jinv01*w3-1.3697240259740259e-04*Jinv22*w38-6.0876623376623375e-04*Jinv21*w59+7.4878246753246752e-04*w25*Jinv12+1.0247564935064935e-04*w0*Jinv02-1.4204545454545455e-05*Jinv21*w2+4.3831168831168834e-04*w42*Jinv01-1.4204545454545455e-05*w11*Jinv21+5.2962662337662338e-04*w16*Jinv11+5.2962662337662338e-04*Jinv21*w17+6.5746753246753243e-04*Jinv01*w45-4.7483766233766234e-04*Jinv12*w55+2.7394480519480518e-04*w15*Jinv02+1.0247564935064935e-04*w10*Jinv12+7.4878246753246752e-04*Jinv02*w24-3.6525974025974026e-05*Jinv22*w35-6.0876623376623375e-04*w58*Jinv11-6.0876623376623375e-04*w57*Jinv01+3.9265422077922079e-04*Jinv21*w26-1.4204545454545455e-05*w0*Jinv01-4.2918019480519480e-04*Jinv22*w20+4.3831168831168834e-04*w43*Jinv11+4.3831168831168834e-04*Jinv21*w41+4.9310064935064932e-04*w42*Jinv02+1.0247564935064935e-04*w11*Jinv22-1.4204545454545455e-05*Jinv11*w1-1.7349837662337662e-04*w13*Jinv11+5.2962662337662338e-04*w15*Jinv01-5.6818181818181818e-05*w28*Jinv11+2.7394480519480518e-04*Jinv22*w17+7.4878246753246752e-04*w22*Jinv12-1.0957792207792208e-04*Jinv02*w3-1.3697240259740259e-04*w32*Jinv22+4.3831168831168834e-04*Jinv01*w39+1.0247564935064935e-04*Jinv22*w2+2.1915584415584417e-04*Jinv21*w38+7.8530844155844158e-04*Jinv02*w45+3.9265422077922079e-04*Jinv01*w24+4.0584415584415584e-05*Jinv22*w59+2.1915584415584417e-04*w34*Jinv11-1.7349837662337662e-04*w19*Jinv11-5.4788961038961036e-04*Jinv21*w53-3.6525974025974026e-05*Jinv12*w34+4.9310064935064932e-04*Jinv22*w44-1.3697240259740259e-04*w4*Jinv11-4.2918019480519480e-04*Jinv12*w13+2.1915584415584417e-04*w32*Jinv21-1.7349837662337662e-04*Jinv21*w20-1.3697240259740259e-04*w37*Jinv12+4.9310064935064932e-04*Jinv22*w41+9.1314935064935065e-06*w52*Jinv12+2.1915584415584417e-04*Jinv21*w35+2.1915584415584417e-04*w31*Jinv11+7.4878246753246752e-04*Jinv22*w26+6.5746753246753243e-04*w46*Jinv11-1.3697240259740259e-04*w5*Jinv21-4.7483766233766234e-04*Jinv02*w54-1.0957792207792208e-04*w4*Jinv12+7.4878246753246752e-04*w21*Jinv02-5.6818181818181818e-05*Jinv21*w29+2.1915584415584417e-04*Jinv01*w33-4.2918019480519480e-04*Jinv02*w12-1.7349837662337662e-04*Jinv01*w18+2.1915584415584417e-04*w37*Jinv11+1.0247564935064935e-04*Jinv02*w9-1.3697240259740259e-04*w6*Jinv01+6.5746753246753243e-04*w47*Jinv21+7.4878246753246752e-04*Jinv22*w23-5.4788961038961036e-04*w52*Jinv11-1.3697240259740259e-04*w31*Jinv12+7.8530844155844158e-04*w46*Jinv12-1.0957792207792208e-04*Jinv22*w8+9.1314935064935065e-06*Jinv22*w50+3.9265422077922079e-04*w22*Jinv11-1.0957792207792208e-04*Jinv12*w7+3.9265422077922079e-04*w21*Jinv01-5.4788961038961036e-04*Jinv01*w51-1.3697240259740259e-04*Jinv02*w30+9.1314935064935065e-06*Jinv12*w49+2.1915584415584417e-04*w36*Jinv01-1.3697240259740259e-04*w7*Jinv11-1.0957792207792207e-03*Jinv21*w56+6.4935064935064935e-05*Jinv02*w27-1.0957792207792208e-04*w6*Jinv02-4.2918019480519480e-04*w19*Jinv12-1.0957792207792208e-04*Jinv22*w5+7.8530844155844158e-04*w47*Jinv22-5.4788961038961036e-04*w49*Jinv11-1.7349837662337662e-04*Jinv21*w14+4.0584415584415584e-05*w58*Jinv12-4.2918019480519480e-04*Jinv02*w18-1.7349837662337662e-04*Jinv01*w12-1.3697240259740259e-04*Jinv21*w8-3.6525974025974026e-05*Jinv02*w33+3.9265422077922079e-04*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11+9.1314935064935065e-06*w48*Jinv02-1.4204545454545455e-05*Jinv01*w9+6.4935064935064935e-05*Jinv22*w29-4.2918019480519480e-04*Jinv22*w14+4.3831168831168834e-04*w40*Jinv11-1.3697240259740259e-04*Jinv02*w36+4.9310064935064932e-04*Jinv12*w43+2.1915584415584417e-04*Jinv01*w30+9.1314935064935065e-06*Jinv02*w51-4.7483766233766234e-04*Jinv22*w56+6.4935064935064935e-05*Jinv12*w28+3.9265422077922079e-04*w25*Jinv11+2.7394480519480518e-04*w16*Jinv12-5.4788961038961036e-04*Jinv21*w50-5.6818181818181818e-05*Jinv01*w27-5.4788961038961036e-04*w48*Jinv01+1.0247564935064935e-04*Jinv12*w1-1.4204545454545455e-05*w10*Jinv11)*D;
  A[59][48]=(0.0)*D;
  A[59][49]=(0.0)*D;
  A[59][50]=(-9.1314935064935065e-06*Jinv00*w12-6.4935064935064935e-05*w28*Jinv10-2.1915584415584417e-04*w40*Jinv12-1.1414366883116883e-03*Jinv22*w53-9.1314935064935065e-06*Jinv20*w8-1.2987012987012987e-03*w57*Jinv02-9.1314935064935065e-06*w13*Jinv10-2.1915584415584417e-04*Jinv02*w39+5.4788961038961036e-04*Jinv21*w44-9.1314935064935065e-06*w3*Jinv00+5.4788961038961036e-04*w32*Jinv20-9.1314935064935065e-06*Jinv01*w3-9.1314935064935065e-06*Jinv20*w23+6.6659902597402602e-04*Jinv22*w38-1.0957792207792207e-03*Jinv00*w54-1.5827922077922079e-03*Jinv21*w59+5.4788961038961042e-05*w25*Jinv12-1.2175324675324675e-04*w0*Jinv02-6.4935064935064935e-05*Jinv21*w2+5.4788961038961036e-04*w43*Jinv10+5.4788961038961036e-04*w42*Jinv01-6.4935064935064935e-05*w11*Jinv21+4.9310064935064932e-04*w16*Jinv11+4.9310064935064932e-04*Jinv21*w17+5.4788961038961036e-04*Jinv01*w45-1.1414366883116883e-03*Jinv12*w55-1.5827922077922079e-03*w58*Jinv10-1.8262987012987014e-04*w15*Jinv02-1.5726461038961039e-04*w10*Jinv12-1.0957792207792207e-03*w48*Jinv00+5.4788961038961042e-05*Jinv02*w24-2.1915584415584417e-04*Jinv22*w35-1.5827922077922079e-03*w58*Jinv11-1.5827922077922079e-03*w57*Jinv01-9.1314935064935065e-06*Jinv21*w26-6.4935064935064935e-05*w0*Jinv01-6.4935064935064935e-05*w1*Jinv10+4.9310064935064932e-04*w16*Jinv10+5.4788961038961042e-05*Jinv22*w20+5.4788961038961036e-04*w43*Jinv11+5.4788961038961036e-04*w45*Jinv00+5.4788961038961036e-04*Jinv21*w41+1.6436688311688311e-04*w42*Jinv02-6.4935064935064935e-05*Jinv00*w27-1.5726461038961039e-04*w11*Jinv22-6.4935064935064935e-05*Jinv11*w1-9.1314935064935065e-06*w13*Jinv11+4.9310064935064932e-04*w15*Jinv01-6.4935064935064935e-05*w28*Jinv11-1.0957792207792207e-03*Jinv20*w50-6.4935064935064935e-05*Jinv20*w29+5.4788961038961036e-04*w46*Jinv10-1.8262987012987014e-04*Jinv22*w17+5.4788961038961036e-04*Jinv00*w33+1.1870941558441559e-04*w22*Jinv12+5.4788961038961036e-04*w31*Jinv10-2.1915584415584417e-04*Jinv02*w3-9.1314935064935065e-06*w21*Jinv00+4.0178571428571428e-04*w32*Jinv22+5.4788961038961036e-04*Jinv01*w39-1.2175324675324675e-04*Jinv22*w2-9.1314935064935065e-06*Jinv00*w18+5.4788961038961036e-04*w36*Jinv00+5.4788961038961036e-04*Jinv21*w38+6.6659902597402602e-04*Jinv02*w45-9.1314935064935065e-06*Jinv01*w24-1.2987012987012987e-03*Jinv22*w59+5.4788961038961036e-04*w34*Jinv11-9.1314935064935065e-06*w19*Jinv11-1.0957792207792207e-03*Jinv21*w53-1.0957792207792207e-03*w51*Jinv00-2.1915584415584417e-04*Jinv12*w34+1.6436688311688311e-04*Jinv22*w44-9.1314935064935065e-06*w4*Jinv10-9.1314935064935065e-06*w4*Jinv11+5.4788961038961036e-04*w41*Jinv20-2.1915584415584417e-04*Jinv12*w13-9.1314935064935065e-06*w19*Jinv10+5.4788961038961036e-04*w34*Jinv10+5.4788961038961036e-04*w32*Jinv21-9.1314935064935065e-06*Jinv21*w20-9.1314935064935065e-06*w6*Jinv00+6.6659902597402602e-04*w37*Jinv12-1.0957792207792207e-03*Jinv20*w56-2.1915584415584417e-04*Jinv22*w41-9.1314935064935065e-06*w26*Jinv20-6.4935064935064935e-05*w11*Jinv20-1.1414366883116883e-03*w52*Jinv12+5.4788961038961036e-04*Jinv21*w35+5.4788961038961036e-04*w31*Jinv11+5.4788961038961042e-05*Jinv22*w26+5.4788961038961036e-04*w46*Jinv11-9.1314935064935065e-06*Jinv20*w14-9.1314935064935065e-06*w5*Jinv21-1.1414366883116883e-03*Jinv02*w54-2.1915584415584417e-04*w4*Jinv12+1.1870941558441559e-04*w21*Jinv02-1.0957792207792207e-03*w49*Jinv10-6.4935064935064935e-05*Jinv21*w29+5.4788961038961036e-04*Jinv01*w33+5.4788961038961036e-04*w39*Jinv00-2.1915584415584417e-04*Jinv02*w12-9.1314935064935065e-06*Jinv01*w18+5.4788961038961036e-04*w37*Jinv11-1.5726461038961039e-04*Jinv02*w9-9.1314935064935065e-06*w6*Jinv01-9.1314935064935065e-06*w7*Jinv10+5.4788961038961036e-04*w47*Jinv21+5.4788961038961036e-04*Jinv20*w38+1.1870941558441559e-04*Jinv22*w23-1.0957792207792207e-03*w52*Jinv11+4.0178571428571428e-04*w31*Jinv12+6.6659902597402602e-04*w46*Jinv12-1.0957792207792207e-03*Jinv20*w53+1.1870941558441559e-04*Jinv22*w8-9.1314935064935065e-06*w22*Jinv10-1.1870941558441558e-03*Jinv22*w50-9.1314935064935065e-06*w22*Jinv11+1.1870941558441559e-04*Jinv12*w7-9.1314935064935065e-06*w21*Jinv01-1.0957792207792207e-03*Jinv01*w51+4.0178571428571428e-04*Jinv02*w30-1.1870941558441558e-03*Jinv12*w49+5.4788961038961036e-04*w36*Jinv01-1.0957792207792207e-03*w52*Jinv10-9.1314935064935065e-06*w7*Jinv11-6.4935064935064935e-05*w9*Jinv00+5.4788961038961036e-04*w37*Jinv10-1.0957792207792207e-03*Jinv21*w56-9.1314935064935065e-06*w24*Jinv00-1.5726461038961039e-04*Jinv02*w27+1.1870941558441559e-04*w6*Jinv02+5.4788961038961042e-05*w19*Jinv12-2.1915584415584417e-04*Jinv22*w5+6.6659902597402602e-04*w47*Jinv22-1.0957792207792207e-03*w49*Jinv11-9.1314935064935065e-06*Jinv21*w14-6.4935064935064935e-05*w10*Jinv10+5.4788961038961036e-04*Jinv20*w44-1.2987012987012987e-03*w58*Jinv12+5.4788961038961042e-05*Jinv02*w18-9.1314935064935065e-06*Jinv01*w12-9.1314935064935065e-06*Jinv21*w8-1.5827922077922079e-03*w57*Jinv00-1.5827922077922079e-03*Jinv20*w59-2.1915584415584417e-04*Jinv02*w33-6.4935064935064935e-05*w0*Jinv00+5.4788961038961036e-04*w35*Jinv20-9.1314935064935065e-06*Jinv21*w23-1.0957792207792207e-03*Jinv01*w54-1.0957792207792207e-03*w55*Jinv11-9.1314935064935065e-06*w25*Jinv10-6.4935064935064935e-05*Jinv20*w2+4.9310064935064932e-04*w15*Jinv00-1.1870941558441558e-03*w48*Jinv02-6.4935064935064935e-05*Jinv01*w9+4.9310064935064932e-04*Jinv20*w17+5.4788961038961036e-04*w30*Jinv00+5.4788961038961036e-04*w40*Jinv10-9.1314935064935065e-06*w20*Jinv20-1.5726461038961039e-04*Jinv22*w29-2.1915584415584417e-04*Jinv22*w14+5.4788961038961036e-04*w40*Jinv11+6.6659902597402602e-04*Jinv02*w36+1.6436688311688311e-04*Jinv12*w43-9.1314935064935065e-06*w5*Jinv20+5.4788961038961036e-04*Jinv01*w30-1.1414366883116883e-03*Jinv02*w51-1.1414366883116883e-03*Jinv22*w56-1.5726461038961039e-04*Jinv12*w28-9.1314935064935065e-06*w25*Jinv11-1.0957792207792207e-03*w55*Jinv10+5.4788961038961036e-04*w42*Jinv00+5.4788961038961036e-04*w47*Jinv20-1.8262987012987014e-04*w16*Jinv12-1.0957792207792207e-03*Jinv21*w50-6.4935064935064935e-05*Jinv01*w27-1.0957792207792207e-03*w48*Jinv01-1.2175324675324675e-04*Jinv12*w1-6.4935064935064935e-05*w10*Jinv11)*D;
  A[59][51]=(0.0)*D;
  A[59][52]=(0.0)*D;
  A[59][53]=(9.1314935064935065e-06*Jinv00*w12+6.4935064935064935e-05*w28*Jinv10-3.8352272727272725e-04*w40*Jinv12-9.1314935064935068e-05*Jinv22*w53+9.1314935064935065e-06*Jinv20*w8+2.8409090909090908e-04*w57*Jinv02+9.1314935064935065e-06*w13*Jinv10-3.8352272727272725e-04*Jinv02*w39+9.1314935064935065e-06*w3*Jinv00-5.4788961038961036e-04*w32*Jinv20+9.1314935064935065e-06*Jinv20*w23-1.4610389610389610e-04*Jinv22*w38+1.0957792207792207e-03*Jinv00*w54+1.2784090909090910e-04*w25*Jinv12-9.2329545454545453e-05*w0*Jinv02-5.4788961038961036e-04*w43*Jinv10-4.5657467532467534e-05*Jinv12*w55+1.5827922077922079e-03*w58*Jinv10-6.7573051948051951e-04*w15*Jinv02-5.6818181818181818e-05*w10*Jinv12+1.0957792207792207e-03*w48*Jinv00+1.2784090909090910e-04*Jinv02*w24-7.6704545454545450e-04*Jinv22*w35+6.4935064935064935e-05*w1*Jinv10-4.9310064935064932e-04*w16*Jinv10-2.1002435064935065e-04*Jinv22*w20-5.4788961038961036e-04*w45*Jinv00-7.6704545454545450e-04*w42*Jinv02+6.4935064935064935e-05*Jinv00*w27-5.6818181818181818e-05*w11*Jinv22+1.0957792207792207e-03*Jinv20*w50+6.4935064935064935e-05*Jinv20*w29-5.4788961038961036e-04*w46*Jinv10-6.7573051948051951e-04*Jinv22*w17-5.4788961038961036e-04*Jinv00*w33+6.3920454545454550e-05*w22*Jinv12-5.4788961038961036e-04*w31*Jinv10+1.2784090909090910e-04*Jinv02*w3+9.1314935064935065e-06*w21*Jinv00+1.1870941558441559e-04*w32*Jinv22-9.2329545454545453e-05*Jinv22*w2+9.1314935064935065e-06*Jinv00*w18-5.4788961038961036e-04*w36*Jinv00+1.1870941558441559e-04*Jinv02*w45+2.8409090909090908e-04*Jinv22*w59+1.0957792207792207e-03*w51*Jinv00-7.6704545454545450e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv10-5.4788961038961036e-04*w41*Jinv20+6.3920454545454550e-05*Jinv12*w13+9.1314935064935065e-06*w19*Jinv10-5.4788961038961036e-04*w34*Jinv10+9.1314935064935065e-06*w6*Jinv00-1.4610389610389610e-04*w37*Jinv12+1.0957792207792207e-03*Jinv20*w56-3.8352272727272725e-04*Jinv22*w41+9.1314935064935065e-06*w26*Jinv20+6.4935064935064935e-05*w11*Jinv20-9.1314935064935068e-05*w52*Jinv12+1.2784090909090910e-04*Jinv22*w26+9.1314935064935065e-06*Jinv20*w14-4.5657467532467534e-05*Jinv02*w54+1.2784090909090910e-04*w4*Jinv12+6.3920454545454550e-05*w21*Jinv02+1.0957792207792207e-03*w49*Jinv10-5.4788961038961036e-04*w39*Jinv00+6.3920454545454550e-05*Jinv02*w12-5.6818181818181818e-05*Jinv02*w9+9.1314935064935065e-06*w7*Jinv10-5.4788961038961036e-04*Jinv20*w38+6.3920454545454550e-05*Jinv22*w23+1.1870941558441559e-04*w31*Jinv12+1.1870941558441559e-04*w46*Jinv12+1.0957792207792207e-03*Jinv20*w53-2.1002435064935065e-04*Jinv22*w8+9.1314935064935065e-06*w22*Jinv10-4.5657467532467534e-05*Jinv22*w50-2.1002435064935065e-04*Jinv12*w7+1.1870941558441559e-04*Jinv02*w30-4.5657467532467534e-05*Jinv12*w49+1.0957792207792207e-03*w52*Jinv10+6.4935064935064935e-05*w9*Jinv00-5.4788961038961036e-04*w37*Jinv10+9.1314935064935065e-06*w24*Jinv00-9.2329545454545453e-05*Jinv02*w27-2.1002435064935065e-04*w6*Jinv02-2.1002435064935065e-04*w19*Jinv12+1.2784090909090910e-04*Jinv22*w5+1.1870941558441559e-04*w47*Jinv22+6.4935064935064935e-05*w10*Jinv10-5.4788961038961036e-04*Jinv20*w44+2.8409090909090908e-04*w58*Jinv12-2.1002435064935065e-04*Jinv02*w18+1.5827922077922079e-03*w57*Jinv00+1.5827922077922079e-03*Jinv20*w59-7.6704545454545450e-04*Jinv02*w33+6.4935064935064935e-05*w0*Jinv00-5.4788961038961036e-04*w35*Jinv20+9.1314935064935065e-06*w25*Jinv10+6.4935064935064935e-05*Jinv20*w2-4.9310064935064932e-04*w15*Jinv00-4.5657467532467534e-05*w48*Jinv02-4.9310064935064932e-04*Jinv20*w17-5.4788961038961036e-04*w30*Jinv00-5.4788961038961036e-04*w40*Jinv10+9.1314935064935065e-06*w20*Jinv20-9.2329545454545453e-05*Jinv22*w29+6.3920454545454550e-05*Jinv22*w14-1.4610389610389610e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43+9.1314935064935065e-06*w5*Jinv20-9.1314935064935068e-05*Jinv02*w51-4.5657467532467534e-05*Jinv22*w56-9.2329545454545453e-05*Jinv12*w28+1.0957792207792207e-03*w55*Jinv10-5.4788961038961036e-04*w42*Jinv00-5.4788961038961036e-04*w47*Jinv20-6.7573051948051951e-04*w16*Jinv12-9.2329545454545453e-05*Jinv12*w1)*D;
  A[59][54]=(0.0)*D;
  A[59][55]=(0.0)*D;
  A[59][56]=(-7.6704545454545450e-04*w40*Jinv12-4.5657467532467534e-05*Jinv22*w53+2.8409090909090908e-04*w57*Jinv02-7.6704545454545450e-04*Jinv02*w39-5.4788961038961036e-04*Jinv21*w44+9.1314935064935065e-06*Jinv01*w3+1.1870941558441559e-04*Jinv22*w38+1.5827922077922079e-03*Jinv21*w59-2.1002435064935065e-04*w25*Jinv12-9.2329545454545453e-05*w0*Jinv02+6.4935064935064935e-05*Jinv21*w2-5.4788961038961036e-04*w42*Jinv01+6.4935064935064935e-05*w11*Jinv21-4.9310064935064932e-04*w16*Jinv11-4.9310064935064932e-04*Jinv21*w17-5.4788961038961036e-04*Jinv01*w45-9.1314935064935068e-05*Jinv12*w55-6.7573051948051951e-04*w15*Jinv02-9.2329545454545453e-05*w10*Jinv12-2.1002435064935065e-04*Jinv02*w24-3.8352272727272725e-04*Jinv22*w35+1.5827922077922079e-03*w58*Jinv11+1.5827922077922079e-03*w57*Jinv01+9.1314935064935065e-06*Jinv21*w26+6.4935064935064935e-05*w0*Jinv01+1.2784090909090910e-04*Jinv22*w20-5.4788961038961036e-04*w43*Jinv11-5.4788961038961036e-04*Jinv21*w41-7.6704545454545450e-04*w42*Jinv02-9.2329545454545453e-05*w11*Jinv22+6.4935064935064935e-05*Jinv11*w1+9.1314935064935065e-06*w13*Jinv11-4.9310064935064932e-04*w15*Jinv01+6.4935064935064935e-05*w28*Jinv11-6.7573051948051951e-04*Jinv22*w17-2.1002435064935065e-04*w22*Jinv12+6.3920454545454550e-05*Jinv02*w3+1.1870941558441559e-04*w32*Jinv22-5.4788961038961036e-04*Jinv01*w39-9.2329545454545453e-05*Jinv22*w2-5.4788961038961036e-04*Jinv21*w38-1.4610389610389610e-04*Jinv02*w45+9.1314935064935065e-06*Jinv01*w24+2.8409090909090908e-04*Jinv22*w59-5.4788961038961036e-04*w34*Jinv11+9.1314935064935065e-06*w19*Jinv11+1.0957792207792207e-03*Jinv21*w53-3.8352272727272725e-04*Jinv12*w34-7.6704545454545450e-04*Jinv22*w44+9.1314935064935065e-06*w4*Jinv11+1.2784090909090910e-04*Jinv12*w13-5.4788961038961036e-04*w32*Jinv21+9.1314935064935065e-06*Jinv21*w20+1.1870941558441559e-04*w37*Jinv12-7.6704545454545450e-04*Jinv22*w41-4.5657467532467534e-05*w52*Jinv12-5.4788961038961036e-04*Jinv21*w35-5.4788961038961036e-04*w31*Jinv11-2.1002435064935065e-04*Jinv22*w26-5.4788961038961036e-04*w46*Jinv11+9.1314935064935065e-06*w5*Jinv21-9.1314935064935068e-05*Jinv02*w54+6.3920454545454550e-05*w4*Jinv12-2.1002435064935065e-04*w21*Jinv02+6.4935064935064935e-05*Jinv21*w29-5.4788961038961036e-04*Jinv01*w33+1.2784090909090910e-04*Jinv02*w12+9.1314935064935065e-06*Jinv01*w18-5.4788961038961036e-04*w37*Jinv11-9.2329545454545453e-05*Jinv02*w9+9.1314935064935065e-06*w6*Jinv01-5.4788961038961036e-04*w47*Jinv21-2.1002435064935065e-04*Jinv22*w23+1.0957792207792207e-03*w52*Jinv11+1.1870941558441559e-04*w31*Jinv12-1.4610389610389610e-04*w46*Jinv12+6.3920454545454550e-05*Jinv22*w8-4.5657467532467534e-05*Jinv22*w50+9.1314935064935065e-06*w22*Jinv11+6.3920454545454550e-05*Jinv12*w7+9.1314935064935065e-06*w21*Jinv01+1.0957792207792207e-03*Jinv01*w51+1.1870941558441559e-04*Jinv02*w30-4.5657467532467534e-05*Jinv12*w49-5.4788961038961036e-04*w36*Jinv01+9.1314935064935065e-06*w7*Jinv11+1.0957792207792207e-03*Jinv21*w56-5.6818181818181818e-05*Jinv02*w27+6.3920454545454550e-05*w6*Jinv02+1.2784090909090910e-04*w19*Jinv12+6.3920454545454550e-05*Jinv22*w5-1.4610389610389610e-04*w47*Jinv22+1.0957792207792207e-03*w49*Jinv11+9.1314935064935065e-06*Jinv21*w14+2.8409090909090908e-04*w58*Jinv12+1.2784090909090910e-04*Jinv02*w18+9.1314935064935065e-06*Jinv01*w12+9.1314935064935065e-06*Jinv21*w8-3.8352272727272725e-04*Jinv02*w33+9.1314935064935065e-06*Jinv21*w23+1.0957792207792207e-03*Jinv01*w54+1.0957792207792207e-03*w55*Jinv11-4.5657467532467534e-05*w48*Jinv02+6.4935064935064935e-05*Jinv01*w9-5.6818181818181818e-05*Jinv22*w29+1.2784090909090910e-04*Jinv22*w14-5.4788961038961036e-04*w40*Jinv11+1.1870941558441559e-04*Jinv02*w36-7.6704545454545450e-04*Jinv12*w43-5.4788961038961036e-04*Jinv01*w30-4.5657467532467534e-05*Jinv02*w51-9.1314935064935068e-05*Jinv22*w56-5.6818181818181818e-05*Jinv12*w28+9.1314935064935065e-06*w25*Jinv11-6.7573051948051951e-04*w16*Jinv12+1.0957792207792207e-03*Jinv21*w50+6.4935064935064935e-05*Jinv01*w27+1.0957792207792207e-03*w48*Jinv01-9.2329545454545453e-05*Jinv12*w1+6.4935064935064935e-05*w10*Jinv11)*D;
  A[59][57]=(0.0)*D;
  A[59][58]=(0.0)*D;
  A[59][59]=(1.2175324675324675e-04*w40*Jinv12+6.4935064935064935e-04*Jinv22*w53+8.3874458874458877e-04*w57*Jinv02+1.2175324675324675e-04*Jinv02*w39-3.2467532467532468e-04*Jinv22*w38+2.0292207792207792e-05*w25*Jinv12+9.4696969696969697e-05*w0*Jinv02+6.4935064935064935e-04*Jinv12*w55+2.4350649350649351e-04*w15*Jinv02+9.4696969696969697e-05*w10*Jinv12+2.0292207792207792e-05*Jinv02*w24+1.2175324675324675e-04*Jinv22*w35+2.0292207792207792e-05*Jinv22*w20+1.2175324675324675e-04*w42*Jinv02+9.4696969696969697e-05*w11*Jinv22+2.4350649350649351e-04*Jinv22*w17+2.0292207792207792e-05*w22*Jinv12+2.0292207792207792e-05*Jinv02*w3-3.2467532467532468e-04*w32*Jinv22+9.4696969696969697e-05*Jinv22*w2-3.2467532467532468e-04*Jinv02*w45+8.3874458874458877e-04*Jinv22*w59+1.2175324675324675e-04*Jinv12*w34+1.2175324675324675e-04*Jinv22*w44+2.0292207792207792e-05*Jinv12*w13-3.2467532467532468e-04*w37*Jinv12+1.2175324675324675e-04*Jinv22*w41+6.4935064935064935e-04*w52*Jinv12+2.0292207792207792e-05*Jinv22*w26+6.4935064935064935e-04*Jinv02*w54+2.0292207792207792e-05*w4*Jinv12+2.0292207792207792e-05*w21*Jinv02+2.0292207792207792e-05*Jinv02*w12+9.4696969696969697e-05*Jinv02*w9+2.0292207792207792e-05*Jinv22*w23-3.2467532467532468e-04*w31*Jinv12-3.2467532467532468e-04*w46*Jinv12+2.0292207792207792e-05*Jinv22*w8+6.4935064935064935e-04*Jinv22*w50+2.0292207792207792e-05*Jinv12*w7-3.2467532467532468e-04*Jinv02*w30+6.4935064935064935e-04*Jinv12*w49+9.4696969696969697e-05*Jinv02*w27+2.0292207792207792e-05*w6*Jinv02+2.0292207792207792e-05*w19*Jinv12+2.0292207792207792e-05*Jinv22*w5-3.2467532467532468e-04*w47*Jinv22+8.3874458874458877e-04*w58*Jinv12+2.0292207792207792e-05*Jinv02*w18+1.2175324675324675e-04*Jinv02*w33+6.4935064935064935e-04*w48*Jinv02+9.4696969696969697e-05*Jinv22*w29+2.0292207792207792e-05*Jinv22*w14-3.2467532467532468e-04*Jinv02*w36+1.2175324675324675e-04*Jinv12*w43+6.4935064935064935e-04*Jinv02*w51+6.4935064935064935e-04*Jinv22*w56+9.4696969696969697e-05*Jinv12*w28+2.4350649350649351e-04*w16*Jinv12+9.4696969696969697e-05*Jinv12*w1)*D;
  int nbf = 60;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}
