
#include <math.h>
#include <utility>
#include <map>
#include <ptv.h>
#include <DofT.h>
#include <ScalarFunction.h>

using namespace std; 



/* ------------- order 1 ---------------*/


  
void matrix_factory_vector_mass_2D_order1 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[6];
  double idof0[3];
  idof0[0]=x0;  idof0[1]=y0;
  idof0[2] = 0.0;
  ptv ipdof0(3,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[3];
  idof1[0]=x0;  idof1[1]=y0;
  idof1[2] = 1.0;
  ptv ipdof1(3,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[3];
  idof2[0]=G00+x0;  idof2[1]=G10+y0;
  idof2[2] = 0.0;
  ptv ipdof2(3,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[3];
  idof3[0]=G00+x0;  idof3[1]=G10+y0;
  idof3[2] = 1.0;
  ptv ipdof3(3,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[3];
  idof4[0]=G01+x0;  idof4[1]=y0+G11;
  idof4[2] = 0.0;
  ptv ipdof4(3,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[3];
  idof5[0]=G01+x0;  idof5[1]=y0+G11;
  idof5[2] = 1.0;
  ptv ipdof5(3,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

 
  double A[6][6];
  A[0][0]=(8.3333333333333329e-02)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(4.1666666666666664e-02)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(4.1666666666666664e-02)*D;
  A[0][5]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(8.3333333333333329e-02)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(4.1666666666666664e-02)*D;
  A[1][4]=(0.0)*D;
  A[1][5]=(4.1666666666666664e-02)*D;
  A[2][0]=(4.1666666666666664e-02)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(8.3333333333333329e-02)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(4.1666666666666664e-02)*D;
  A[2][5]=(0.0)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(4.1666666666666664e-02)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(8.3333333333333329e-02)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(4.1666666666666664e-02)*D;
  A[4][0]=(4.1666666666666664e-02)*D;
  A[4][1]=(0.0)*D;
  A[4][2]=(4.1666666666666664e-02)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(8.3333333333333329e-02)*D;
  A[4][5]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(4.1666666666666664e-02)*D;
  A[5][2]=(0.0)*D;
  A[5][3]=(4.1666666666666664e-02)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(8.3333333333333329e-02)*D;
  int nbf = 6;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 2 ---------------*/


  
void matrix_factory_vector_mass_2D_order2 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[12];
  double idof0[3];
  idof0[0]=x0;  idof0[1]=y0;
  idof0[2] = 0.0;
  ptv ipdof0(3,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[3];
  idof1[0]=x0;  idof1[1]=y0;
  idof1[2] = 1.0;
  ptv ipdof1(3,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[3];
  idof2[0]=G00/2.0+x0;  idof2[1]=G10/2.0+y0;
  idof2[2] = 0.0;
  ptv ipdof2(3,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[3];
  idof3[0]=G00/2.0+x0;  idof3[1]=G10/2.0+y0;
  idof3[2] = 1.0;
  ptv ipdof3(3,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[3];
  idof4[0]=G00+x0;  idof4[1]=G10+y0;
  idof4[2] = 0.0;
  ptv ipdof4(3,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[3];
  idof5[0]=G00+x0;  idof5[1]=G10+y0;
  idof5[2] = 1.0;
  ptv ipdof5(3,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

  double idof6[3];
  idof6[0]=G01/2.0+x0;  idof6[1]=y0+G11/2.0;
  idof6[2] = 0.0;
  ptv ipdof6(3,idof6);
  iidof[6] = dof.insert_dof(element,6,ipdof6);

  double idof7[3];
  idof7[0]=G01/2.0+x0;  idof7[1]=y0+G11/2.0;
  idof7[2] = 1.0;
  ptv ipdof7(3,idof7);
  iidof[7] = dof.insert_dof(element,7,ipdof7);

  double idof8[3];
  idof8[0]=G01/2.0+G00/2.0+x0;  idof8[1]=G10/2.0+y0+G11/2.0;
  idof8[2] = 0.0;
  ptv ipdof8(3,idof8);
  iidof[8] = dof.insert_dof(element,8,ipdof8);

  double idof9[3];
  idof9[0]=G01/2.0+G00/2.0+x0;  idof9[1]=G10/2.0+y0+G11/2.0;
  idof9[2] = 1.0;
  ptv ipdof9(3,idof9);
  iidof[9] = dof.insert_dof(element,9,ipdof9);

  double idof10[3];
  idof10[0]=G01+x0;  idof10[1]=y0+G11;
  idof10[2] = 0.0;
  ptv ipdof10(3,idof10);
  iidof[10] = dof.insert_dof(element,10,ipdof10);

  double idof11[3];
  idof11[0]=G01+x0;  idof11[1]=y0+G11;
  idof11[2] = 1.0;
  ptv ipdof11(3,idof11);
  iidof[11] = dof.insert_dof(element,11,ipdof11);

 
  double A[12][12];
  A[0][0]=(1.6666666666666666e-02)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(0.0)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(-2.7777777777777779e-03)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(0.0)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(-1.1111111111111112e-02)*D;
  A[0][9]=(0.0)*D;
  A[0][10]=(-2.7777777777777779e-03)*D;
  A[0][11]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(1.6666666666666666e-02)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(0.0)*D;
  A[1][4]=(0.0)*D;
  A[1][5]=(-2.7777777777777779e-03)*D;
  A[1][6]=(0.0)*D;
  A[1][7]=(0.0)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(-1.1111111111111112e-02)*D;
  A[1][10]=(0.0)*D;
  A[1][11]=(-2.7777777777777779e-03)*D;
  A[2][0]=(0.0)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(8.8888888888888892e-02)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(0.0)*D;
  A[2][5]=(0.0)*D;
  A[2][6]=(4.4444444444444446e-02)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(4.4444444444444446e-02)*D;
  A[2][9]=(0.0)*D;
  A[2][10]=(-1.1111111111111112e-02)*D;
  A[2][11]=(0.0)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(0.0)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(8.8888888888888892e-02)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(0.0)*D;
  A[3][6]=(0.0)*D;
  A[3][7]=(4.4444444444444446e-02)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(4.4444444444444446e-02)*D;
  A[3][10]=(0.0)*D;
  A[3][11]=(-1.1111111111111112e-02)*D;
  A[4][0]=(-2.7777777777777779e-03)*D;
  A[4][1]=(0.0)*D;
  A[4][2]=(0.0)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(1.6666666666666666e-02)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(-1.1111111111111112e-02)*D;
  A[4][7]=(0.0)*D;
  A[4][8]=(0.0)*D;
  A[4][9]=(0.0)*D;
  A[4][10]=(-2.7777777777777779e-03)*D;
  A[4][11]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(-2.7777777777777779e-03)*D;
  A[5][2]=(0.0)*D;
  A[5][3]=(0.0)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(1.6666666666666666e-02)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(-1.1111111111111112e-02)*D;
  A[5][8]=(0.0)*D;
  A[5][9]=(0.0)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(-2.7777777777777779e-03)*D;
  A[6][0]=(0.0)*D;
  A[6][1]=(0.0)*D;
  A[6][2]=(4.4444444444444446e-02)*D;
  A[6][3]=(0.0)*D;
  A[6][4]=(-1.1111111111111112e-02)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(8.8888888888888892e-02)*D;
  A[6][7]=(0.0)*D;
  A[6][8]=(4.4444444444444446e-02)*D;
  A[6][9]=(0.0)*D;
  A[6][10]=(0.0)*D;
  A[6][11]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(0.0)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(4.4444444444444446e-02)*D;
  A[7][4]=(0.0)*D;
  A[7][5]=(-1.1111111111111112e-02)*D;
  A[7][6]=(0.0)*D;
  A[7][7]=(8.8888888888888892e-02)*D;
  A[7][8]=(0.0)*D;
  A[7][9]=(4.4444444444444446e-02)*D;
  A[7][10]=(0.0)*D;
  A[7][11]=(0.0)*D;
  A[8][0]=(-1.1111111111111112e-02)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(4.4444444444444446e-02)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(0.0)*D;
  A[8][5]=(0.0)*D;
  A[8][6]=(4.4444444444444446e-02)*D;
  A[8][7]=(0.0)*D;
  A[8][8]=(8.8888888888888892e-02)*D;
  A[8][9]=(0.0)*D;
  A[8][10]=(0.0)*D;
  A[8][11]=(0.0)*D;
  A[9][0]=(0.0)*D;
  A[9][1]=(-1.1111111111111112e-02)*D;
  A[9][2]=(0.0)*D;
  A[9][3]=(4.4444444444444446e-02)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(0.0)*D;
  A[9][6]=(0.0)*D;
  A[9][7]=(4.4444444444444446e-02)*D;
  A[9][8]=(0.0)*D;
  A[9][9]=(8.8888888888888892e-02)*D;
  A[9][10]=(0.0)*D;
  A[9][11]=(0.0)*D;
  A[10][0]=(-2.7777777777777779e-03)*D;
  A[10][1]=(0.0)*D;
  A[10][2]=(-1.1111111111111112e-02)*D;
  A[10][3]=(0.0)*D;
  A[10][4]=(-2.7777777777777779e-03)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(0.0)*D;
  A[10][7]=(0.0)*D;
  A[10][8]=(0.0)*D;
  A[10][9]=(0.0)*D;
  A[10][10]=(1.6666666666666666e-02)*D;
  A[10][11]=(0.0)*D;
  A[11][0]=(0.0)*D;
  A[11][1]=(-2.7777777777777779e-03)*D;
  A[11][2]=(0.0)*D;
  A[11][3]=(-1.1111111111111112e-02)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(-2.7777777777777779e-03)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(0.0)*D;
  A[11][8]=(0.0)*D;
  A[11][9]=(0.0)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(1.6666666666666666e-02)*D;
  int nbf = 12;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 3 ---------------*/


  
void matrix_factory_vector_mass_2D_order3 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[20];
  double idof0[3];
  idof0[0]=x0;  idof0[1]=y0;
  idof0[2] = 0.0;
  ptv ipdof0(3,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[3];
  idof1[0]=x0;  idof1[1]=y0;
  idof1[2] = 1.0;
  ptv ipdof1(3,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[3];
  idof2[0]=G00/3.0+x0;  idof2[1]=G10/3.0+y0;
  idof2[2] = 0.0;
  ptv ipdof2(3,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[3];
  idof3[0]=G00/3.0+x0;  idof3[1]=G10/3.0+y0;
  idof3[2] = 1.0;
  ptv ipdof3(3,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[3];
  idof4[0]=(2.0/3.0)*G00+x0;  idof4[1]=(2.0/3.0)*G10+y0;
  idof4[2] = 0.0;
  ptv ipdof4(3,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[3];
  idof5[0]=(2.0/3.0)*G00+x0;  idof5[1]=(2.0/3.0)*G10+y0;
  idof5[2] = 1.0;
  ptv ipdof5(3,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

  double idof6[3];
  idof6[0]=G00+x0;  idof6[1]=G10+y0;
  idof6[2] = 0.0;
  ptv ipdof6(3,idof6);
  iidof[6] = dof.insert_dof(element,6,ipdof6);

  double idof7[3];
  idof7[0]=G00+x0;  idof7[1]=G10+y0;
  idof7[2] = 1.0;
  ptv ipdof7(3,idof7);
  iidof[7] = dof.insert_dof(element,7,ipdof7);

  double idof8[3];
  idof8[0]=G01/3.0+x0;  idof8[1]=y0+G11/3.0;
  idof8[2] = 0.0;
  ptv ipdof8(3,idof8);
  iidof[8] = dof.insert_dof(element,8,ipdof8);

  double idof9[3];
  idof9[0]=G01/3.0+x0;  idof9[1]=y0+G11/3.0;
  idof9[2] = 1.0;
  ptv ipdof9(3,idof9);
  iidof[9] = dof.insert_dof(element,9,ipdof9);

  double idof10[3];
  idof10[0]=G01/3.0+G00/3.0+x0;  idof10[1]=G10/3.0+y0+G11/3.0;
  idof10[2] = 0.0;
  ptv ipdof10(3,idof10);
  iidof[10] = dof.insert_dof(element,10,ipdof10);

  double idof11[3];
  idof11[0]=G01/3.0+G00/3.0+x0;  idof11[1]=G10/3.0+y0+G11/3.0;
  idof11[2] = 1.0;
  ptv ipdof11(3,idof11);
  iidof[11] = dof.insert_dof(element,11,ipdof11);

  double idof12[3];
  idof12[0]=G01/3.0+(2.0/3.0)*G00+x0;  idof12[1]=(2.0/3.0)*G10+y0+G11/3.0;
  idof12[2] = 0.0;
  ptv ipdof12(3,idof12);
  iidof[12] = dof.insert_dof(element,12,ipdof12);

  double idof13[3];
  idof13[0]=G01/3.0+(2.0/3.0)*G00+x0;  idof13[1]=(2.0/3.0)*G10+y0+G11/3.0;
  idof13[2] = 1.0;
  ptv ipdof13(3,idof13);
  iidof[13] = dof.insert_dof(element,13,ipdof13);

  double idof14[3];
  idof14[0]=(2.0/3.0)*G01+x0;  idof14[1]=y0+(2.0/3.0)*G11;
  idof14[2] = 0.0;
  ptv ipdof14(3,idof14);
  iidof[14] = dof.insert_dof(element,14,ipdof14);

  double idof15[3];
  idof15[0]=(2.0/3.0)*G01+x0;  idof15[1]=y0+(2.0/3.0)*G11;
  idof15[2] = 1.0;
  ptv ipdof15(3,idof15);
  iidof[15] = dof.insert_dof(element,15,ipdof15);

  double idof16[3];
  idof16[0]=(2.0/3.0)*G01+G00/3.0+x0;  idof16[1]=G10/3.0+y0+(2.0/3.0)*G11;
  idof16[2] = 0.0;
  ptv ipdof16(3,idof16);
  iidof[16] = dof.insert_dof(element,16,ipdof16);

  double idof17[3];
  idof17[0]=(2.0/3.0)*G01+G00/3.0+x0;  idof17[1]=G10/3.0+y0+(2.0/3.0)*G11;
  idof17[2] = 1.0;
  ptv ipdof17(3,idof17);
  iidof[17] = dof.insert_dof(element,17,ipdof17);

  double idof18[3];
  idof18[0]=G01+x0;  idof18[1]=y0+G11;
  idof18[2] = 0.0;
  ptv ipdof18(3,idof18);
  iidof[18] = dof.insert_dof(element,18,ipdof18);

  double idof19[3];
  idof19[0]=G01+x0;  idof19[1]=y0+G11;
  idof19[2] = 1.0;
  ptv ipdof19(3,idof19);
  iidof[19] = dof.insert_dof(element,19,ipdof19);

 
  double A[20][20];
  A[0][0]=(5.6547619047619046e-03)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(1.3392857142857143e-03)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(0.0)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(8.1845238095238097e-04)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(1.3392857142857143e-03)*D;
  A[0][9]=(0.0)*D;
  A[0][10]=(2.6785714285714286e-03)*D;
  A[0][11]=(0.0)*D;
  A[0][12]=(2.0089285714285712e-03)*D;
  A[0][13]=(0.0)*D;
  A[0][14]=(0.0)*D;
  A[0][15]=(0.0)*D;
  A[0][16]=(2.0089285714285712e-03)*D;
  A[0][17]=(0.0)*D;
  A[0][18]=(8.1845238095238097e-04)*D;
  A[0][19]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(5.6547619047619046e-03)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(1.3392857142857143e-03)*D;
  A[1][4]=(0.0)*D;
  A[1][5]=(0.0)*D;
  A[1][6]=(0.0)*D;
  A[1][7]=(8.1845238095238097e-04)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(1.3392857142857143e-03)*D;
  A[1][10]=(0.0)*D;
  A[1][11]=(2.6785714285714286e-03)*D;
  A[1][12]=(0.0)*D;
  A[1][13]=(2.0089285714285712e-03)*D;
  A[1][14]=(0.0)*D;
  A[1][15]=(0.0)*D;
  A[1][16]=(0.0)*D;
  A[1][17]=(2.0089285714285712e-03)*D;
  A[1][18]=(0.0)*D;
  A[1][19]=(8.1845238095238097e-04)*D;
  A[2][0]=(1.3392857142857143e-03)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(4.0178571428571432e-02)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(-1.4062500000000000e-02)*D;
  A[2][5]=(0.0)*D;
  A[2][6]=(0.0)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(2.0089285714285716e-02)*D;
  A[2][9]=(0.0)*D;
  A[2][10]=(1.2053571428571429e-02)*D;
  A[2][11]=(0.0)*D;
  A[2][12]=(-1.0044642857142858e-02)*D;
  A[2][13]=(0.0)*D;
  A[2][14]=(-1.0044642857142858e-02)*D;
  A[2][15]=(0.0)*D;
  A[2][16]=(-4.0178571428571425e-03)*D;
  A[2][17]=(0.0)*D;
  A[2][18]=(2.0089285714285712e-03)*D;
  A[2][19]=(0.0)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(1.3392857142857143e-03)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(4.0178571428571432e-02)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(-1.4062500000000000e-02)*D;
  A[3][6]=(0.0)*D;
  A[3][7]=(0.0)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(2.0089285714285716e-02)*D;
  A[3][10]=(0.0)*D;
  A[3][11]=(1.2053571428571429e-02)*D;
  A[3][12]=(0.0)*D;
  A[3][13]=(-1.0044642857142858e-02)*D;
  A[3][14]=(0.0)*D;
  A[3][15]=(-1.0044642857142858e-02)*D;
  A[3][16]=(0.0)*D;
  A[3][17]=(-4.0178571428571425e-03)*D;
  A[3][18]=(0.0)*D;
  A[3][19]=(2.0089285714285712e-03)*D;
  A[4][0]=(0.0)*D;
  A[4][1]=(0.0)*D;
  A[4][2]=(-1.4062500000000000e-02)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(4.0178571428571432e-02)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(1.3392857142857143e-03)*D;
  A[4][7]=(0.0)*D;
  A[4][8]=(-1.0044642857142858e-02)*D;
  A[4][9]=(0.0)*D;
  A[4][10]=(1.2053571428571429e-02)*D;
  A[4][11]=(0.0)*D;
  A[4][12]=(2.0089285714285716e-02)*D;
  A[4][13]=(0.0)*D;
  A[4][14]=(-4.0178571428571425e-03)*D;
  A[4][15]=(0.0)*D;
  A[4][16]=(-1.0044642857142858e-02)*D;
  A[4][17]=(0.0)*D;
  A[4][18]=(2.0089285714285712e-03)*D;
  A[4][19]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(0.0)*D;
  A[5][2]=(0.0)*D;
  A[5][3]=(-1.4062500000000000e-02)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(4.0178571428571432e-02)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(1.3392857142857143e-03)*D;
  A[5][8]=(0.0)*D;
  A[5][9]=(-1.0044642857142858e-02)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(1.2053571428571429e-02)*D;
  A[5][12]=(0.0)*D;
  A[5][13]=(2.0089285714285716e-02)*D;
  A[5][14]=(0.0)*D;
  A[5][15]=(-4.0178571428571425e-03)*D;
  A[5][16]=(0.0)*D;
  A[5][17]=(-1.0044642857142858e-02)*D;
  A[5][18]=(0.0)*D;
  A[5][19]=(2.0089285714285712e-03)*D;
  A[6][0]=(8.1845238095238097e-04)*D;
  A[6][1]=(0.0)*D;
  A[6][2]=(0.0)*D;
  A[6][3]=(0.0)*D;
  A[6][4]=(1.3392857142857143e-03)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(5.6547619047619046e-03)*D;
  A[6][7]=(0.0)*D;
  A[6][8]=(2.0089285714285712e-03)*D;
  A[6][9]=(0.0)*D;
  A[6][10]=(2.6785714285714286e-03)*D;
  A[6][11]=(0.0)*D;
  A[6][12]=(1.3392857142857143e-03)*D;
  A[6][13]=(0.0)*D;
  A[6][14]=(2.0089285714285712e-03)*D;
  A[6][15]=(0.0)*D;
  A[6][16]=(0.0)*D;
  A[6][17]=(0.0)*D;
  A[6][18]=(8.1845238095238097e-04)*D;
  A[6][19]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(8.1845238095238097e-04)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(0.0)*D;
  A[7][4]=(0.0)*D;
  A[7][5]=(1.3392857142857143e-03)*D;
  A[7][6]=(0.0)*D;
  A[7][7]=(5.6547619047619046e-03)*D;
  A[7][8]=(0.0)*D;
  A[7][9]=(2.0089285714285712e-03)*D;
  A[7][10]=(0.0)*D;
  A[7][11]=(2.6785714285714286e-03)*D;
  A[7][12]=(0.0)*D;
  A[7][13]=(1.3392857142857143e-03)*D;
  A[7][14]=(0.0)*D;
  A[7][15]=(2.0089285714285712e-03)*D;
  A[7][16]=(0.0)*D;
  A[7][17]=(0.0)*D;
  A[7][18]=(0.0)*D;
  A[7][19]=(8.1845238095238097e-04)*D;
  A[8][0]=(1.3392857142857143e-03)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(2.0089285714285716e-02)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(-1.0044642857142858e-02)*D;
  A[8][5]=(0.0)*D;
  A[8][6]=(2.0089285714285712e-03)*D;
  A[8][7]=(0.0)*D;
  A[8][8]=(4.0178571428571432e-02)*D;
  A[8][9]=(0.0)*D;
  A[8][10]=(1.2053571428571429e-02)*D;
  A[8][11]=(0.0)*D;
  A[8][12]=(-4.0178571428571425e-03)*D;
  A[8][13]=(0.0)*D;
  A[8][14]=(-1.4062500000000000e-02)*D;
  A[8][15]=(0.0)*D;
  A[8][16]=(-1.0044642857142858e-02)*D;
  A[8][17]=(0.0)*D;
  A[8][18]=(0.0)*D;
  A[8][19]=(0.0)*D;
  A[9][0]=(0.0)*D;
  A[9][1]=(1.3392857142857143e-03)*D;
  A[9][2]=(0.0)*D;
  A[9][3]=(2.0089285714285716e-02)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(-1.0044642857142858e-02)*D;
  A[9][6]=(0.0)*D;
  A[9][7]=(2.0089285714285712e-03)*D;
  A[9][8]=(0.0)*D;
  A[9][9]=(4.0178571428571432e-02)*D;
  A[9][10]=(0.0)*D;
  A[9][11]=(1.2053571428571429e-02)*D;
  A[9][12]=(0.0)*D;
  A[9][13]=(-4.0178571428571425e-03)*D;
  A[9][14]=(0.0)*D;
  A[9][15]=(-1.4062500000000000e-02)*D;
  A[9][16]=(0.0)*D;
  A[9][17]=(-1.0044642857142858e-02)*D;
  A[9][18]=(0.0)*D;
  A[9][19]=(0.0)*D;
  A[10][0]=(2.6785714285714286e-03)*D;
  A[10][1]=(0.0)*D;
  A[10][2]=(1.2053571428571429e-02)*D;
  A[10][3]=(0.0)*D;
  A[10][4]=(1.2053571428571429e-02)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(2.6785714285714286e-03)*D;
  A[10][7]=(0.0)*D;
  A[10][8]=(1.2053571428571429e-02)*D;
  A[10][9]=(0.0)*D;
  A[10][10]=(1.4464285714285716e-01)*D;
  A[10][11]=(0.0)*D;
  A[10][12]=(1.2053571428571429e-02)*D;
  A[10][13]=(0.0)*D;
  A[10][14]=(1.2053571428571429e-02)*D;
  A[10][15]=(0.0)*D;
  A[10][16]=(1.2053571428571429e-02)*D;
  A[10][17]=(0.0)*D;
  A[10][18]=(2.6785714285714286e-03)*D;
  A[10][19]=(0.0)*D;
  A[11][0]=(0.0)*D;
  A[11][1]=(2.6785714285714286e-03)*D;
  A[11][2]=(0.0)*D;
  A[11][3]=(1.2053571428571429e-02)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(1.2053571428571429e-02)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(2.6785714285714286e-03)*D;
  A[11][8]=(0.0)*D;
  A[11][9]=(1.2053571428571429e-02)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(1.4464285714285716e-01)*D;
  A[11][12]=(0.0)*D;
  A[11][13]=(1.2053571428571429e-02)*D;
  A[11][14]=(0.0)*D;
  A[11][15]=(1.2053571428571429e-02)*D;
  A[11][16]=(0.0)*D;
  A[11][17]=(1.2053571428571429e-02)*D;
  A[11][18]=(0.0)*D;
  A[11][19]=(2.6785714285714286e-03)*D;
  A[12][0]=(2.0089285714285712e-03)*D;
  A[12][1]=(0.0)*D;
  A[12][2]=(-1.0044642857142858e-02)*D;
  A[12][3]=(0.0)*D;
  A[12][4]=(2.0089285714285716e-02)*D;
  A[12][5]=(0.0)*D;
  A[12][6]=(1.3392857142857143e-03)*D;
  A[12][7]=(0.0)*D;
  A[12][8]=(-4.0178571428571425e-03)*D;
  A[12][9]=(0.0)*D;
  A[12][10]=(1.2053571428571429e-02)*D;
  A[12][11]=(0.0)*D;
  A[12][12]=(4.0178571428571432e-02)*D;
  A[12][13]=(0.0)*D;
  A[12][14]=(-1.0044642857142858e-02)*D;
  A[12][15]=(0.0)*D;
  A[12][16]=(-1.4062500000000000e-02)*D;
  A[12][17]=(0.0)*D;
  A[12][18]=(0.0)*D;
  A[12][19]=(0.0)*D;
  A[13][0]=(0.0)*D;
  A[13][1]=(2.0089285714285712e-03)*D;
  A[13][2]=(0.0)*D;
  A[13][3]=(-1.0044642857142858e-02)*D;
  A[13][4]=(0.0)*D;
  A[13][5]=(2.0089285714285716e-02)*D;
  A[13][6]=(0.0)*D;
  A[13][7]=(1.3392857142857143e-03)*D;
  A[13][8]=(0.0)*D;
  A[13][9]=(-4.0178571428571425e-03)*D;
  A[13][10]=(0.0)*D;
  A[13][11]=(1.2053571428571429e-02)*D;
  A[13][12]=(0.0)*D;
  A[13][13]=(4.0178571428571432e-02)*D;
  A[13][14]=(0.0)*D;
  A[13][15]=(-1.0044642857142858e-02)*D;
  A[13][16]=(0.0)*D;
  A[13][17]=(-1.4062500000000000e-02)*D;
  A[13][18]=(0.0)*D;
  A[13][19]=(0.0)*D;
  A[14][0]=(0.0)*D;
  A[14][1]=(0.0)*D;
  A[14][2]=(-1.0044642857142858e-02)*D;
  A[14][3]=(0.0)*D;
  A[14][4]=(-4.0178571428571425e-03)*D;
  A[14][5]=(0.0)*D;
  A[14][6]=(2.0089285714285712e-03)*D;
  A[14][7]=(0.0)*D;
  A[14][8]=(-1.4062500000000000e-02)*D;
  A[14][9]=(0.0)*D;
  A[14][10]=(1.2053571428571429e-02)*D;
  A[14][11]=(0.0)*D;
  A[14][12]=(-1.0044642857142858e-02)*D;
  A[14][13]=(0.0)*D;
  A[14][14]=(4.0178571428571432e-02)*D;
  A[14][15]=(0.0)*D;
  A[14][16]=(2.0089285714285716e-02)*D;
  A[14][17]=(0.0)*D;
  A[14][18]=(1.3392857142857143e-03)*D;
  A[14][19]=(0.0)*D;
  A[15][0]=(0.0)*D;
  A[15][1]=(0.0)*D;
  A[15][2]=(0.0)*D;
  A[15][3]=(-1.0044642857142858e-02)*D;
  A[15][4]=(0.0)*D;
  A[15][5]=(-4.0178571428571425e-03)*D;
  A[15][6]=(0.0)*D;
  A[15][7]=(2.0089285714285712e-03)*D;
  A[15][8]=(0.0)*D;
  A[15][9]=(-1.4062500000000000e-02)*D;
  A[15][10]=(0.0)*D;
  A[15][11]=(1.2053571428571429e-02)*D;
  A[15][12]=(0.0)*D;
  A[15][13]=(-1.0044642857142858e-02)*D;
  A[15][14]=(0.0)*D;
  A[15][15]=(4.0178571428571432e-02)*D;
  A[15][16]=(0.0)*D;
  A[15][17]=(2.0089285714285716e-02)*D;
  A[15][18]=(0.0)*D;
  A[15][19]=(1.3392857142857143e-03)*D;
  A[16][0]=(2.0089285714285712e-03)*D;
  A[16][1]=(0.0)*D;
  A[16][2]=(-4.0178571428571425e-03)*D;
  A[16][3]=(0.0)*D;
  A[16][4]=(-1.0044642857142858e-02)*D;
  A[16][5]=(0.0)*D;
  A[16][6]=(0.0)*D;
  A[16][7]=(0.0)*D;
  A[16][8]=(-1.0044642857142858e-02)*D;
  A[16][9]=(0.0)*D;
  A[16][10]=(1.2053571428571429e-02)*D;
  A[16][11]=(0.0)*D;
  A[16][12]=(-1.4062500000000000e-02)*D;
  A[16][13]=(0.0)*D;
  A[16][14]=(2.0089285714285716e-02)*D;
  A[16][15]=(0.0)*D;
  A[16][16]=(4.0178571428571432e-02)*D;
  A[16][17]=(0.0)*D;
  A[16][18]=(1.3392857142857143e-03)*D;
  A[16][19]=(0.0)*D;
  A[17][0]=(0.0)*D;
  A[17][1]=(2.0089285714285712e-03)*D;
  A[17][2]=(0.0)*D;
  A[17][3]=(-4.0178571428571425e-03)*D;
  A[17][4]=(0.0)*D;
  A[17][5]=(-1.0044642857142858e-02)*D;
  A[17][6]=(0.0)*D;
  A[17][7]=(0.0)*D;
  A[17][8]=(0.0)*D;
  A[17][9]=(-1.0044642857142858e-02)*D;
  A[17][10]=(0.0)*D;
  A[17][11]=(1.2053571428571429e-02)*D;
  A[17][12]=(0.0)*D;
  A[17][13]=(-1.4062500000000000e-02)*D;
  A[17][14]=(0.0)*D;
  A[17][15]=(2.0089285714285716e-02)*D;
  A[17][16]=(0.0)*D;
  A[17][17]=(4.0178571428571432e-02)*D;
  A[17][18]=(0.0)*D;
  A[17][19]=(1.3392857142857143e-03)*D;
  A[18][0]=(8.1845238095238097e-04)*D;
  A[18][1]=(0.0)*D;
  A[18][2]=(2.0089285714285712e-03)*D;
  A[18][3]=(0.0)*D;
  A[18][4]=(2.0089285714285712e-03)*D;
  A[18][5]=(0.0)*D;
  A[18][6]=(8.1845238095238097e-04)*D;
  A[18][7]=(0.0)*D;
  A[18][8]=(0.0)*D;
  A[18][9]=(0.0)*D;
  A[18][10]=(2.6785714285714286e-03)*D;
  A[18][11]=(0.0)*D;
  A[18][12]=(0.0)*D;
  A[18][13]=(0.0)*D;
  A[18][14]=(1.3392857142857143e-03)*D;
  A[18][15]=(0.0)*D;
  A[18][16]=(1.3392857142857143e-03)*D;
  A[18][17]=(0.0)*D;
  A[18][18]=(5.6547619047619046e-03)*D;
  A[18][19]=(0.0)*D;
  A[19][0]=(0.0)*D;
  A[19][1]=(8.1845238095238097e-04)*D;
  A[19][2]=(0.0)*D;
  A[19][3]=(2.0089285714285712e-03)*D;
  A[19][4]=(0.0)*D;
  A[19][5]=(2.0089285714285712e-03)*D;
  A[19][6]=(0.0)*D;
  A[19][7]=(8.1845238095238097e-04)*D;
  A[19][8]=(0.0)*D;
  A[19][9]=(0.0)*D;
  A[19][10]=(0.0)*D;
  A[19][11]=(2.6785714285714286e-03)*D;
  A[19][12]=(0.0)*D;
  A[19][13]=(0.0)*D;
  A[19][14]=(0.0)*D;
  A[19][15]=(1.3392857142857143e-03)*D;
  A[19][16]=(0.0)*D;
  A[19][17]=(1.3392857142857143e-03)*D;
  A[19][18]=(0.0)*D;
  A[19][19]=(5.6547619047619046e-03)*D;
  int nbf = 20;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 4 ---------------*/


  
void matrix_factory_vector_mass_2D_order4 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[30];
  double idof0[3];
  idof0[0]=x0;  idof0[1]=y0;
  idof0[2] = 0.0;
  ptv ipdof0(3,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[3];
  idof1[0]=x0;  idof1[1]=y0;
  idof1[2] = 1.0;
  ptv ipdof1(3,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[3];
  idof2[0]=G00/4.0+x0;  idof2[1]=G10/4.0+y0;
  idof2[2] = 0.0;
  ptv ipdof2(3,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[3];
  idof3[0]=G00/4.0+x0;  idof3[1]=G10/4.0+y0;
  idof3[2] = 1.0;
  ptv ipdof3(3,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[3];
  idof4[0]=G00/2.0+x0;  idof4[1]=G10/2.0+y0;
  idof4[2] = 0.0;
  ptv ipdof4(3,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[3];
  idof5[0]=G00/2.0+x0;  idof5[1]=G10/2.0+y0;
  idof5[2] = 1.0;
  ptv ipdof5(3,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

  double idof6[3];
  idof6[0]=(3.0/4.0)*G00+x0;  idof6[1]=(3.0/4.0)*G10+y0;
  idof6[2] = 0.0;
  ptv ipdof6(3,idof6);
  iidof[6] = dof.insert_dof(element,6,ipdof6);

  double idof7[3];
  idof7[0]=(3.0/4.0)*G00+x0;  idof7[1]=(3.0/4.0)*G10+y0;
  idof7[2] = 1.0;
  ptv ipdof7(3,idof7);
  iidof[7] = dof.insert_dof(element,7,ipdof7);

  double idof8[3];
  idof8[0]=G00+x0;  idof8[1]=G10+y0;
  idof8[2] = 0.0;
  ptv ipdof8(3,idof8);
  iidof[8] = dof.insert_dof(element,8,ipdof8);

  double idof9[3];
  idof9[0]=G00+x0;  idof9[1]=G10+y0;
  idof9[2] = 1.0;
  ptv ipdof9(3,idof9);
  iidof[9] = dof.insert_dof(element,9,ipdof9);

  double idof10[3];
  idof10[0]=G01/4.0+x0;  idof10[1]=y0+G11/4.0;
  idof10[2] = 0.0;
  ptv ipdof10(3,idof10);
  iidof[10] = dof.insert_dof(element,10,ipdof10);

  double idof11[3];
  idof11[0]=G01/4.0+x0;  idof11[1]=y0+G11/4.0;
  idof11[2] = 1.0;
  ptv ipdof11(3,idof11);
  iidof[11] = dof.insert_dof(element,11,ipdof11);

  double idof12[3];
  idof12[0]=G01/4.0+G00/4.0+x0;  idof12[1]=G10/4.0+y0+G11/4.0;
  idof12[2] = 0.0;
  ptv ipdof12(3,idof12);
  iidof[12] = dof.insert_dof(element,12,ipdof12);

  double idof13[3];
  idof13[0]=G01/4.0+G00/4.0+x0;  idof13[1]=G10/4.0+y0+G11/4.0;
  idof13[2] = 1.0;
  ptv ipdof13(3,idof13);
  iidof[13] = dof.insert_dof(element,13,ipdof13);

  double idof14[3];
  idof14[0]=G01/4.0+G00/2.0+x0;  idof14[1]=G10/2.0+y0+G11/4.0;
  idof14[2] = 0.0;
  ptv ipdof14(3,idof14);
  iidof[14] = dof.insert_dof(element,14,ipdof14);

  double idof15[3];
  idof15[0]=G01/4.0+G00/2.0+x0;  idof15[1]=G10/2.0+y0+G11/4.0;
  idof15[2] = 1.0;
  ptv ipdof15(3,idof15);
  iidof[15] = dof.insert_dof(element,15,ipdof15);

  double idof16[3];
  idof16[0]=G01/4.0+(3.0/4.0)*G00+x0;  idof16[1]=(3.0/4.0)*G10+y0+G11/4.0;
  idof16[2] = 0.0;
  ptv ipdof16(3,idof16);
  iidof[16] = dof.insert_dof(element,16,ipdof16);

  double idof17[3];
  idof17[0]=G01/4.0+(3.0/4.0)*G00+x0;  idof17[1]=(3.0/4.0)*G10+y0+G11/4.0;
  idof17[2] = 1.0;
  ptv ipdof17(3,idof17);
  iidof[17] = dof.insert_dof(element,17,ipdof17);

  double idof18[3];
  idof18[0]=G01/2.0+x0;  idof18[1]=y0+G11/2.0;
  idof18[2] = 0.0;
  ptv ipdof18(3,idof18);
  iidof[18] = dof.insert_dof(element,18,ipdof18);

  double idof19[3];
  idof19[0]=G01/2.0+x0;  idof19[1]=y0+G11/2.0;
  idof19[2] = 1.0;
  ptv ipdof19(3,idof19);
  iidof[19] = dof.insert_dof(element,19,ipdof19);

  double idof20[3];
  idof20[0]=G01/2.0+G00/4.0+x0;  idof20[1]=G10/4.0+y0+G11/2.0;
  idof20[2] = 0.0;
  ptv ipdof20(3,idof20);
  iidof[20] = dof.insert_dof(element,20,ipdof20);

  double idof21[3];
  idof21[0]=G01/2.0+G00/4.0+x0;  idof21[1]=G10/4.0+y0+G11/2.0;
  idof21[2] = 1.0;
  ptv ipdof21(3,idof21);
  iidof[21] = dof.insert_dof(element,21,ipdof21);

  double idof22[3];
  idof22[0]=G01/2.0+G00/2.0+x0;  idof22[1]=G10/2.0+y0+G11/2.0;
  idof22[2] = 0.0;
  ptv ipdof22(3,idof22);
  iidof[22] = dof.insert_dof(element,22,ipdof22);

  double idof23[3];
  idof23[0]=G01/2.0+G00/2.0+x0;  idof23[1]=G10/2.0+y0+G11/2.0;
  idof23[2] = 1.0;
  ptv ipdof23(3,idof23);
  iidof[23] = dof.insert_dof(element,23,ipdof23);

  double idof24[3];
  idof24[0]=(3.0/4.0)*G01+x0;  idof24[1]=y0+(3.0/4.0)*G11;
  idof24[2] = 0.0;
  ptv ipdof24(3,idof24);
  iidof[24] = dof.insert_dof(element,24,ipdof24);

  double idof25[3];
  idof25[0]=(3.0/4.0)*G01+x0;  idof25[1]=y0+(3.0/4.0)*G11;
  idof25[2] = 1.0;
  ptv ipdof25(3,idof25);
  iidof[25] = dof.insert_dof(element,25,ipdof25);

  double idof26[3];
  idof26[0]=(3.0/4.0)*G01+G00/4.0+x0;  idof26[1]=G10/4.0+y0+(3.0/4.0)*G11;
  idof26[2] = 0.0;
  ptv ipdof26(3,idof26);
  iidof[26] = dof.insert_dof(element,26,ipdof26);

  double idof27[3];
  idof27[0]=(3.0/4.0)*G01+G00/4.0+x0;  idof27[1]=G10/4.0+y0+(3.0/4.0)*G11;
  idof27[2] = 1.0;
  ptv ipdof27(3,idof27);
  iidof[27] = dof.insert_dof(element,27,ipdof27);

  double idof28[3];
  idof28[0]=G01+x0;  idof28[1]=y0+G11;
  idof28[2] = 0.0;
  ptv ipdof28(3,idof28);
  iidof[28] = dof.insert_dof(element,28,ipdof28);

  double idof29[3];
  idof29[0]=G01+x0;  idof29[1]=y0+G11;
  idof29[2] = 1.0;
  ptv ipdof29(3,idof29);
  iidof[29] = dof.insert_dof(element,29,ipdof29);

 
  double A[30][30];
  A[0][0]=(2.5573192239858908e-03)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(1.4109347442680777e-03)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(-7.0546737213403885e-04)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(0.0)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(-2.3809523809523810e-04)*D;
  A[0][9]=(0.0)*D;
  A[0][10]=(1.4109347442680777e-03)*D;
  A[0][11]=(0.0)*D;
  A[0][12]=(1.4109347442680777e-03)*D;
  A[0][13]=(0.0)*D;
  A[0][14]=(-1.4109347442680777e-03)*D;
  A[0][15]=(0.0)*D;
  A[0][16]=(-9.8765432098765434e-04)*D;
  A[0][17]=(0.0)*D;
  A[0][18]=(-7.0546737213403885e-04)*D;
  A[0][19]=(0.0)*D;
  A[0][20]=(-1.4109347442680777e-03)*D;
  A[0][21]=(0.0)*D;
  A[0][22]=(-1.0582010582010582e-04)*D;
  A[0][23]=(0.0)*D;
  A[0][24]=(0.0)*D;
  A[0][25]=(0.0)*D;
  A[0][26]=(-9.8765432098765434e-04)*D;
  A[0][27]=(0.0)*D;
  A[0][28]=(-2.3809523809523810e-04)*D;
  A[0][29]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(2.5573192239858908e-03)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(1.4109347442680777e-03)*D;
  A[1][4]=(0.0)*D;
  A[1][5]=(-7.0546737213403885e-04)*D;
  A[1][6]=(0.0)*D;
  A[1][7]=(0.0)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(-2.3809523809523810e-04)*D;
  A[1][10]=(0.0)*D;
  A[1][11]=(1.4109347442680777e-03)*D;
  A[1][12]=(0.0)*D;
  A[1][13]=(1.4109347442680777e-03)*D;
  A[1][14]=(0.0)*D;
  A[1][15]=(-1.4109347442680777e-03)*D;
  A[1][16]=(0.0)*D;
  A[1][17]=(-9.8765432098765434e-04)*D;
  A[1][18]=(0.0)*D;
  A[1][19]=(-7.0546737213403885e-04)*D;
  A[1][20]=(0.0)*D;
  A[1][21]=(-1.4109347442680777e-03)*D;
  A[1][22]=(0.0)*D;
  A[1][23]=(-1.0582010582010582e-04)*D;
  A[1][24]=(0.0)*D;
  A[1][25]=(0.0)*D;
  A[1][26]=(0.0)*D;
  A[1][27]=(-9.8765432098765434e-04)*D;
  A[1][28]=(0.0)*D;
  A[1][29]=(-2.3809523809523810e-04)*D;
  A[2][0]=(1.4109347442680777e-03)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(2.2574955908289243e-02)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(-1.1287477954144622e-02)*D;
  A[2][5]=(0.0)*D;
  A[2][6]=(6.7724867724867728e-03)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(0.0)*D;
  A[2][9]=(0.0)*D;
  A[2][10]=(1.1287477954144622e-02)*D;
  A[2][11]=(0.0)*D;
  A[2][12]=(1.1287477954144622e-02)*D;
  A[2][13]=(0.0)*D;
  A[2][14]=(2.2574955908289240e-03)*D;
  A[2][15]=(0.0)*D;
  A[2][16]=(4.5149911816578479e-03)*D;
  A[2][17]=(0.0)*D;
  A[2][18]=(-8.4656084656084662e-03)*D;
  A[2][19]=(0.0)*D;
  A[2][20]=(-2.2574955908289240e-03)*D;
  A[2][21]=(0.0)*D;
  A[2][22]=(5.6437389770723099e-04)*D;
  A[2][23]=(0.0)*D;
  A[2][24]=(4.5149911816578479e-03)*D;
  A[2][25]=(0.0)*D;
  A[2][26]=(2.2574955908289240e-03)*D;
  A[2][27]=(0.0)*D;
  A[2][28]=(-9.8765432098765434e-04)*D;
  A[2][29]=(0.0)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(1.4109347442680777e-03)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(2.2574955908289243e-02)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(-1.1287477954144622e-02)*D;
  A[3][6]=(0.0)*D;
  A[3][7]=(6.7724867724867728e-03)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(0.0)*D;
  A[3][10]=(0.0)*D;
  A[3][11]=(1.1287477954144622e-02)*D;
  A[3][12]=(0.0)*D;
  A[3][13]=(1.1287477954144622e-02)*D;
  A[3][14]=(0.0)*D;
  A[3][15]=(2.2574955908289240e-03)*D;
  A[3][16]=(0.0)*D;
  A[3][17]=(4.5149911816578479e-03)*D;
  A[3][18]=(0.0)*D;
  A[3][19]=(-8.4656084656084662e-03)*D;
  A[3][20]=(0.0)*D;
  A[3][21]=(-2.2574955908289240e-03)*D;
  A[3][22]=(0.0)*D;
  A[3][23]=(5.6437389770723099e-04)*D;
  A[3][24]=(0.0)*D;
  A[3][25]=(4.5149911816578479e-03)*D;
  A[3][26]=(0.0)*D;
  A[3][27]=(2.2574955908289240e-03)*D;
  A[3][28]=(0.0)*D;
  A[3][29]=(-9.8765432098765434e-04)*D;
  A[4][0]=(-7.0546737213403885e-04)*D;
  A[4][1]=(0.0)*D;
  A[4][2]=(-1.1287477954144622e-02)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(2.7936507936507936e-02)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(-1.1287477954144622e-02)*D;
  A[4][7]=(0.0)*D;
  A[4][8]=(-7.0546737213403885e-04)*D;
  A[4][9]=(0.0)*D;
  A[4][10]=(-8.4656084656084662e-03)*D;
  A[4][11]=(0.0)*D;
  A[4][12]=(3.3862433862433864e-03)*D;
  A[4][13]=(0.0)*D;
  A[4][14]=(3.3862433862433864e-03)*D;
  A[4][15]=(0.0)*D;
  A[4][16]=(-8.4656084656084662e-03)*D;
  A[4][17]=(0.0)*D;
  A[4][18]=(4.2328042328042330e-04)*D;
  A[4][19]=(0.0)*D;
  A[4][20]=(-6.7724867724867728e-03)*D;
  A[4][21]=(0.0)*D;
  A[4][22]=(4.2328042328042330e-04)*D;
  A[4][23]=(0.0)*D;
  A[4][24]=(5.6437389770723099e-04)*D;
  A[4][25]=(0.0)*D;
  A[4][26]=(5.6437389770723099e-04)*D;
  A[4][27]=(0.0)*D;
  A[4][28]=(-1.0582010582010582e-04)*D;
  A[4][29]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(-7.0546737213403885e-04)*D;
  A[5][2]=(0.0)*D;
  A[5][3]=(-1.1287477954144622e-02)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(2.7936507936507936e-02)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(-1.1287477954144622e-02)*D;
  A[5][8]=(0.0)*D;
  A[5][9]=(-7.0546737213403885e-04)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(-8.4656084656084662e-03)*D;
  A[5][12]=(0.0)*D;
  A[5][13]=(3.3862433862433864e-03)*D;
  A[5][14]=(0.0)*D;
  A[5][15]=(3.3862433862433864e-03)*D;
  A[5][16]=(0.0)*D;
  A[5][17]=(-8.4656084656084662e-03)*D;
  A[5][18]=(0.0)*D;
  A[5][19]=(4.2328042328042330e-04)*D;
  A[5][20]=(0.0)*D;
  A[5][21]=(-6.7724867724867728e-03)*D;
  A[5][22]=(0.0)*D;
  A[5][23]=(4.2328042328042330e-04)*D;
  A[5][24]=(0.0)*D;
  A[5][25]=(5.6437389770723099e-04)*D;
  A[5][26]=(0.0)*D;
  A[5][27]=(5.6437389770723099e-04)*D;
  A[5][28]=(0.0)*D;
  A[5][29]=(-1.0582010582010582e-04)*D;
  A[6][0]=(0.0)*D;
  A[6][1]=(0.0)*D;
  A[6][2]=(6.7724867724867728e-03)*D;
  A[6][3]=(0.0)*D;
  A[6][4]=(-1.1287477954144622e-02)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(2.2574955908289243e-02)*D;
  A[6][7]=(0.0)*D;
  A[6][8]=(1.4109347442680777e-03)*D;
  A[6][9]=(0.0)*D;
  A[6][10]=(4.5149911816578479e-03)*D;
  A[6][11]=(0.0)*D;
  A[6][12]=(2.2574955908289240e-03)*D;
  A[6][13]=(0.0)*D;
  A[6][14]=(1.1287477954144622e-02)*D;
  A[6][15]=(0.0)*D;
  A[6][16]=(1.1287477954144622e-02)*D;
  A[6][17]=(0.0)*D;
  A[6][18]=(5.6437389770723099e-04)*D;
  A[6][19]=(0.0)*D;
  A[6][20]=(-2.2574955908289240e-03)*D;
  A[6][21]=(0.0)*D;
  A[6][22]=(-8.4656084656084662e-03)*D;
  A[6][23]=(0.0)*D;
  A[6][24]=(2.2574955908289240e-03)*D;
  A[6][25]=(0.0)*D;
  A[6][26]=(4.5149911816578479e-03)*D;
  A[6][27]=(0.0)*D;
  A[6][28]=(-9.8765432098765434e-04)*D;
  A[6][29]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(0.0)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(6.7724867724867728e-03)*D;
  A[7][4]=(0.0)*D;
  A[7][5]=(-1.1287477954144622e-02)*D;
  A[7][6]=(0.0)*D;
  A[7][7]=(2.2574955908289243e-02)*D;
  A[7][8]=(0.0)*D;
  A[7][9]=(1.4109347442680777e-03)*D;
  A[7][10]=(0.0)*D;
  A[7][11]=(4.5149911816578479e-03)*D;
  A[7][12]=(0.0)*D;
  A[7][13]=(2.2574955908289240e-03)*D;
  A[7][14]=(0.0)*D;
  A[7][15]=(1.1287477954144622e-02)*D;
  A[7][16]=(0.0)*D;
  A[7][17]=(1.1287477954144622e-02)*D;
  A[7][18]=(0.0)*D;
  A[7][19]=(5.6437389770723099e-04)*D;
  A[7][20]=(0.0)*D;
  A[7][21]=(-2.2574955908289240e-03)*D;
  A[7][22]=(0.0)*D;
  A[7][23]=(-8.4656084656084662e-03)*D;
  A[7][24]=(0.0)*D;
  A[7][25]=(2.2574955908289240e-03)*D;
  A[7][26]=(0.0)*D;
  A[7][27]=(4.5149911816578479e-03)*D;
  A[7][28]=(0.0)*D;
  A[7][29]=(-9.8765432098765434e-04)*D;
  A[8][0]=(-2.3809523809523810e-04)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(0.0)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(-7.0546737213403885e-04)*D;
  A[8][5]=(0.0)*D;
  A[8][6]=(1.4109347442680777e-03)*D;
  A[8][7]=(0.0)*D;
  A[8][8]=(2.5573192239858908e-03)*D;
  A[8][9]=(0.0)*D;
  A[8][10]=(-9.8765432098765434e-04)*D;
  A[8][11]=(0.0)*D;
  A[8][12]=(-1.4109347442680777e-03)*D;
  A[8][13]=(0.0)*D;
  A[8][14]=(1.4109347442680777e-03)*D;
  A[8][15]=(0.0)*D;
  A[8][16]=(1.4109347442680777e-03)*D;
  A[8][17]=(0.0)*D;
  A[8][18]=(-1.0582010582010582e-04)*D;
  A[8][19]=(0.0)*D;
  A[8][20]=(-1.4109347442680777e-03)*D;
  A[8][21]=(0.0)*D;
  A[8][22]=(-7.0546737213403885e-04)*D;
  A[8][23]=(0.0)*D;
  A[8][24]=(-9.8765432098765434e-04)*D;
  A[8][25]=(0.0)*D;
  A[8][26]=(0.0)*D;
  A[8][27]=(0.0)*D;
  A[8][28]=(-2.3809523809523810e-04)*D;
  A[8][29]=(0.0)*D;
  A[9][0]=(0.0)*D;
  A[9][1]=(-2.3809523809523810e-04)*D;
  A[9][2]=(0.0)*D;
  A[9][3]=(0.0)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(-7.0546737213403885e-04)*D;
  A[9][6]=(0.0)*D;
  A[9][7]=(1.4109347442680777e-03)*D;
  A[9][8]=(0.0)*D;
  A[9][9]=(2.5573192239858908e-03)*D;
  A[9][10]=(0.0)*D;
  A[9][11]=(-9.8765432098765434e-04)*D;
  A[9][12]=(0.0)*D;
  A[9][13]=(-1.4109347442680777e-03)*D;
  A[9][14]=(0.0)*D;
  A[9][15]=(1.4109347442680777e-03)*D;
  A[9][16]=(0.0)*D;
  A[9][17]=(1.4109347442680777e-03)*D;
  A[9][18]=(0.0)*D;
  A[9][19]=(-1.0582010582010582e-04)*D;
  A[9][20]=(0.0)*D;
  A[9][21]=(-1.4109347442680777e-03)*D;
  A[9][22]=(0.0)*D;
  A[9][23]=(-7.0546737213403885e-04)*D;
  A[9][24]=(0.0)*D;
  A[9][25]=(-9.8765432098765434e-04)*D;
  A[9][26]=(0.0)*D;
  A[9][27]=(0.0)*D;
  A[9][28]=(0.0)*D;
  A[9][29]=(-2.3809523809523810e-04)*D;
  A[10][0]=(1.4109347442680777e-03)*D;
  A[10][1]=(0.0)*D;
  A[10][2]=(1.1287477954144622e-02)*D;
  A[10][3]=(0.0)*D;
  A[10][4]=(-8.4656084656084662e-03)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(4.5149911816578479e-03)*D;
  A[10][7]=(0.0)*D;
  A[10][8]=(-9.8765432098765434e-04)*D;
  A[10][9]=(0.0)*D;
  A[10][10]=(2.2574955908289243e-02)*D;
  A[10][11]=(0.0)*D;
  A[10][12]=(1.1287477954144622e-02)*D;
  A[10][13]=(0.0)*D;
  A[10][14]=(-2.2574955908289240e-03)*D;
  A[10][15]=(0.0)*D;
  A[10][16]=(2.2574955908289240e-03)*D;
  A[10][17]=(0.0)*D;
  A[10][18]=(-1.1287477954144622e-02)*D;
  A[10][19]=(0.0)*D;
  A[10][20]=(2.2574955908289240e-03)*D;
  A[10][21]=(0.0)*D;
  A[10][22]=(5.6437389770723099e-04)*D;
  A[10][23]=(0.0)*D;
  A[10][24]=(6.7724867724867728e-03)*D;
  A[10][25]=(0.0)*D;
  A[10][26]=(4.5149911816578479e-03)*D;
  A[10][27]=(0.0)*D;
  A[10][28]=(0.0)*D;
  A[10][29]=(0.0)*D;
  A[11][0]=(0.0)*D;
  A[11][1]=(1.4109347442680777e-03)*D;
  A[11][2]=(0.0)*D;
  A[11][3]=(1.1287477954144622e-02)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(-8.4656084656084662e-03)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(4.5149911816578479e-03)*D;
  A[11][8]=(0.0)*D;
  A[11][9]=(-9.8765432098765434e-04)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(2.2574955908289243e-02)*D;
  A[11][12]=(0.0)*D;
  A[11][13]=(1.1287477954144622e-02)*D;
  A[11][14]=(0.0)*D;
  A[11][15]=(-2.2574955908289240e-03)*D;
  A[11][16]=(0.0)*D;
  A[11][17]=(2.2574955908289240e-03)*D;
  A[11][18]=(0.0)*D;
  A[11][19]=(-1.1287477954144622e-02)*D;
  A[11][20]=(0.0)*D;
  A[11][21]=(2.2574955908289240e-03)*D;
  A[11][22]=(0.0)*D;
  A[11][23]=(5.6437389770723099e-04)*D;
  A[11][24]=(0.0)*D;
  A[11][25]=(6.7724867724867728e-03)*D;
  A[11][26]=(0.0)*D;
  A[11][27]=(4.5149911816578479e-03)*D;
  A[11][28]=(0.0)*D;
  A[11][29]=(0.0)*D;
  A[12][0]=(1.4109347442680777e-03)*D;
  A[12][1]=(0.0)*D;
  A[12][2]=(1.1287477954144622e-02)*D;
  A[12][3]=(0.0)*D;
  A[12][4]=(3.3862433862433864e-03)*D;
  A[12][5]=(0.0)*D;
  A[12][6]=(2.2574955908289240e-03)*D;
  A[12][7]=(0.0)*D;
  A[12][8]=(-1.4109347442680777e-03)*D;
  A[12][9]=(0.0)*D;
  A[12][10]=(1.1287477954144622e-02)*D;
  A[12][11]=(0.0)*D;
  A[12][12]=(9.4814814814814810e-02)*D;
  A[12][13]=(0.0)*D;
  A[12][14]=(-1.3544973544973546e-02)*D;
  A[12][15]=(0.0)*D;
  A[12][16]=(-2.2574955908289240e-03)*D;
  A[12][17]=(0.0)*D;
  A[12][18]=(3.3862433862433864e-03)*D;
  A[12][19]=(0.0)*D;
  A[12][20]=(-1.3544973544973546e-02)*D;
  A[12][21]=(0.0)*D;
  A[12][22]=(-6.7724867724867728e-03)*D;
  A[12][23]=(0.0)*D;
  A[12][24]=(2.2574955908289240e-03)*D;
  A[12][25]=(0.0)*D;
  A[12][26]=(-2.2574955908289240e-03)*D;
  A[12][27]=(0.0)*D;
  A[12][28]=(-1.4109347442680777e-03)*D;
  A[12][29]=(0.0)*D;
  A[13][0]=(0.0)*D;
  A[13][1]=(1.4109347442680777e-03)*D;
  A[13][2]=(0.0)*D;
  A[13][3]=(1.1287477954144622e-02)*D;
  A[13][4]=(0.0)*D;
  A[13][5]=(3.3862433862433864e-03)*D;
  A[13][6]=(0.0)*D;
  A[13][7]=(2.2574955908289240e-03)*D;
  A[13][8]=(0.0)*D;
  A[13][9]=(-1.4109347442680777e-03)*D;
  A[13][10]=(0.0)*D;
  A[13][11]=(1.1287477954144622e-02)*D;
  A[13][12]=(0.0)*D;
  A[13][13]=(9.4814814814814810e-02)*D;
  A[13][14]=(0.0)*D;
  A[13][15]=(-1.3544973544973546e-02)*D;
  A[13][16]=(0.0)*D;
  A[13][17]=(-2.2574955908289240e-03)*D;
  A[13][18]=(0.0)*D;
  A[13][19]=(3.3862433862433864e-03)*D;
  A[13][20]=(0.0)*D;
  A[13][21]=(-1.3544973544973546e-02)*D;
  A[13][22]=(0.0)*D;
  A[13][23]=(-6.7724867724867728e-03)*D;
  A[13][24]=(0.0)*D;
  A[13][25]=(2.2574955908289240e-03)*D;
  A[13][26]=(0.0)*D;
  A[13][27]=(-2.2574955908289240e-03)*D;
  A[13][28]=(0.0)*D;
  A[13][29]=(-1.4109347442680777e-03)*D;
  A[14][0]=(-1.4109347442680777e-03)*D;
  A[14][1]=(0.0)*D;
  A[14][2]=(2.2574955908289240e-03)*D;
  A[14][3]=(0.0)*D;
  A[14][4]=(3.3862433862433864e-03)*D;
  A[14][5]=(0.0)*D;
  A[14][6]=(1.1287477954144622e-02)*D;
  A[14][7]=(0.0)*D;
  A[14][8]=(1.4109347442680777e-03)*D;
  A[14][9]=(0.0)*D;
  A[14][10]=(-2.2574955908289240e-03)*D;
  A[14][11]=(0.0)*D;
  A[14][12]=(-1.3544973544973546e-02)*D;
  A[14][13]=(0.0)*D;
  A[14][14]=(9.4814814814814810e-02)*D;
  A[14][15]=(0.0)*D;
  A[14][16]=(1.1287477954144622e-02)*D;
  A[14][17]=(0.0)*D;
  A[14][18]=(-6.7724867724867728e-03)*D;
  A[14][19]=(0.0)*D;
  A[14][20]=(-1.3544973544973546e-02)*D;
  A[14][21]=(0.0)*D;
  A[14][22]=(3.3862433862433864e-03)*D;
  A[14][23]=(0.0)*D;
  A[14][24]=(-2.2574955908289240e-03)*D;
  A[14][25]=(0.0)*D;
  A[14][26]=(2.2574955908289240e-03)*D;
  A[14][27]=(0.0)*D;
  A[14][28]=(-1.4109347442680777e-03)*D;
  A[14][29]=(0.0)*D;
  A[15][0]=(0.0)*D;
  A[15][1]=(-1.4109347442680777e-03)*D;
  A[15][2]=(0.0)*D;
  A[15][3]=(2.2574955908289240e-03)*D;
  A[15][4]=(0.0)*D;
  A[15][5]=(3.3862433862433864e-03)*D;
  A[15][6]=(0.0)*D;
  A[15][7]=(1.1287477954144622e-02)*D;
  A[15][8]=(0.0)*D;
  A[15][9]=(1.4109347442680777e-03)*D;
  A[15][10]=(0.0)*D;
  A[15][11]=(-2.2574955908289240e-03)*D;
  A[15][12]=(0.0)*D;
  A[15][13]=(-1.3544973544973546e-02)*D;
  A[15][14]=(0.0)*D;
  A[15][15]=(9.4814814814814810e-02)*D;
  A[15][16]=(0.0)*D;
  A[15][17]=(1.1287477954144622e-02)*D;
  A[15][18]=(0.0)*D;
  A[15][19]=(-6.7724867724867728e-03)*D;
  A[15][20]=(0.0)*D;
  A[15][21]=(-1.3544973544973546e-02)*D;
  A[15][22]=(0.0)*D;
  A[15][23]=(3.3862433862433864e-03)*D;
  A[15][24]=(0.0)*D;
  A[15][25]=(-2.2574955908289240e-03)*D;
  A[15][26]=(0.0)*D;
  A[15][27]=(2.2574955908289240e-03)*D;
  A[15][28]=(0.0)*D;
  A[15][29]=(-1.4109347442680777e-03)*D;
  A[16][0]=(-9.8765432098765434e-04)*D;
  A[16][1]=(0.0)*D;
  A[16][2]=(4.5149911816578479e-03)*D;
  A[16][3]=(0.0)*D;
  A[16][4]=(-8.4656084656084662e-03)*D;
  A[16][5]=(0.0)*D;
  A[16][6]=(1.1287477954144622e-02)*D;
  A[16][7]=(0.0)*D;
  A[16][8]=(1.4109347442680777e-03)*D;
  A[16][9]=(0.0)*D;
  A[16][10]=(2.2574955908289240e-03)*D;
  A[16][11]=(0.0)*D;
  A[16][12]=(-2.2574955908289240e-03)*D;
  A[16][13]=(0.0)*D;
  A[16][14]=(1.1287477954144622e-02)*D;
  A[16][15]=(0.0)*D;
  A[16][16]=(2.2574955908289243e-02)*D;
  A[16][17]=(0.0)*D;
  A[16][18]=(5.6437389770723099e-04)*D;
  A[16][19]=(0.0)*D;
  A[16][20]=(2.2574955908289240e-03)*D;
  A[16][21]=(0.0)*D;
  A[16][22]=(-1.1287477954144622e-02)*D;
  A[16][23]=(0.0)*D;
  A[16][24]=(4.5149911816578479e-03)*D;
  A[16][25]=(0.0)*D;
  A[16][26]=(6.7724867724867728e-03)*D;
  A[16][27]=(0.0)*D;
  A[16][28]=(0.0)*D;
  A[16][29]=(0.0)*D;
  A[17][0]=(0.0)*D;
  A[17][1]=(-9.8765432098765434e-04)*D;
  A[17][2]=(0.0)*D;
  A[17][3]=(4.5149911816578479e-03)*D;
  A[17][4]=(0.0)*D;
  A[17][5]=(-8.4656084656084662e-03)*D;
  A[17][6]=(0.0)*D;
  A[17][7]=(1.1287477954144622e-02)*D;
  A[17][8]=(0.0)*D;
  A[17][9]=(1.4109347442680777e-03)*D;
  A[17][10]=(0.0)*D;
  A[17][11]=(2.2574955908289240e-03)*D;
  A[17][12]=(0.0)*D;
  A[17][13]=(-2.2574955908289240e-03)*D;
  A[17][14]=(0.0)*D;
  A[17][15]=(1.1287477954144622e-02)*D;
  A[17][16]=(0.0)*D;
  A[17][17]=(2.2574955908289243e-02)*D;
  A[17][18]=(0.0)*D;
  A[17][19]=(5.6437389770723099e-04)*D;
  A[17][20]=(0.0)*D;
  A[17][21]=(2.2574955908289240e-03)*D;
  A[17][22]=(0.0)*D;
  A[17][23]=(-1.1287477954144622e-02)*D;
  A[17][24]=(0.0)*D;
  A[17][25]=(4.5149911816578479e-03)*D;
  A[17][26]=(0.0)*D;
  A[17][27]=(6.7724867724867728e-03)*D;
  A[17][28]=(0.0)*D;
  A[17][29]=(0.0)*D;
  A[18][0]=(-7.0546737213403885e-04)*D;
  A[18][1]=(0.0)*D;
  A[18][2]=(-8.4656084656084662e-03)*D;
  A[18][3]=(0.0)*D;
  A[18][4]=(4.2328042328042330e-04)*D;
  A[18][5]=(0.0)*D;
  A[18][6]=(5.6437389770723099e-04)*D;
  A[18][7]=(0.0)*D;
  A[18][8]=(-1.0582010582010582e-04)*D;
  A[18][9]=(0.0)*D;
  A[18][10]=(-1.1287477954144622e-02)*D;
  A[18][11]=(0.0)*D;
  A[18][12]=(3.3862433862433864e-03)*D;
  A[18][13]=(0.0)*D;
  A[18][14]=(-6.7724867724867728e-03)*D;
  A[18][15]=(0.0)*D;
  A[18][16]=(5.6437389770723099e-04)*D;
  A[18][17]=(0.0)*D;
  A[18][18]=(2.7936507936507936e-02)*D;
  A[18][19]=(0.0)*D;
  A[18][20]=(3.3862433862433864e-03)*D;
  A[18][21]=(0.0)*D;
  A[18][22]=(4.2328042328042330e-04)*D;
  A[18][23]=(0.0)*D;
  A[18][24]=(-1.1287477954144622e-02)*D;
  A[18][25]=(0.0)*D;
  A[18][26]=(-8.4656084656084662e-03)*D;
  A[18][27]=(0.0)*D;
  A[18][28]=(-7.0546737213403885e-04)*D;
  A[18][29]=(0.0)*D;
  A[19][0]=(0.0)*D;
  A[19][1]=(-7.0546737213403885e-04)*D;
  A[19][2]=(0.0)*D;
  A[19][3]=(-8.4656084656084662e-03)*D;
  A[19][4]=(0.0)*D;
  A[19][5]=(4.2328042328042330e-04)*D;
  A[19][6]=(0.0)*D;
  A[19][7]=(5.6437389770723099e-04)*D;
  A[19][8]=(0.0)*D;
  A[19][9]=(-1.0582010582010582e-04)*D;
  A[19][10]=(0.0)*D;
  A[19][11]=(-1.1287477954144622e-02)*D;
  A[19][12]=(0.0)*D;
  A[19][13]=(3.3862433862433864e-03)*D;
  A[19][14]=(0.0)*D;
  A[19][15]=(-6.7724867724867728e-03)*D;
  A[19][16]=(0.0)*D;
  A[19][17]=(5.6437389770723099e-04)*D;
  A[19][18]=(0.0)*D;
  A[19][19]=(2.7936507936507936e-02)*D;
  A[19][20]=(0.0)*D;
  A[19][21]=(3.3862433862433864e-03)*D;
  A[19][22]=(0.0)*D;
  A[19][23]=(4.2328042328042330e-04)*D;
  A[19][24]=(0.0)*D;
  A[19][25]=(-1.1287477954144622e-02)*D;
  A[19][26]=(0.0)*D;
  A[19][27]=(-8.4656084656084662e-03)*D;
  A[19][28]=(0.0)*D;
  A[19][29]=(-7.0546737213403885e-04)*D;
  A[20][0]=(-1.4109347442680777e-03)*D;
  A[20][1]=(0.0)*D;
  A[20][2]=(-2.2574955908289240e-03)*D;
  A[20][3]=(0.0)*D;
  A[20][4]=(-6.7724867724867728e-03)*D;
  A[20][5]=(0.0)*D;
  A[20][6]=(-2.2574955908289240e-03)*D;
  A[20][7]=(0.0)*D;
  A[20][8]=(-1.4109347442680777e-03)*D;
  A[20][9]=(0.0)*D;
  A[20][10]=(2.2574955908289240e-03)*D;
  A[20][11]=(0.0)*D;
  A[20][12]=(-1.3544973544973546e-02)*D;
  A[20][13]=(0.0)*D;
  A[20][14]=(-1.3544973544973546e-02)*D;
  A[20][15]=(0.0)*D;
  A[20][16]=(2.2574955908289240e-03)*D;
  A[20][17]=(0.0)*D;
  A[20][18]=(3.3862433862433864e-03)*D;
  A[20][19]=(0.0)*D;
  A[20][20]=(9.4814814814814810e-02)*D;
  A[20][21]=(0.0)*D;
  A[20][22]=(3.3862433862433864e-03)*D;
  A[20][23]=(0.0)*D;
  A[20][24]=(1.1287477954144622e-02)*D;
  A[20][25]=(0.0)*D;
  A[20][26]=(1.1287477954144622e-02)*D;
  A[20][27]=(0.0)*D;
  A[20][28]=(1.4109347442680777e-03)*D;
  A[20][29]=(0.0)*D;
  A[21][0]=(0.0)*D;
  A[21][1]=(-1.4109347442680777e-03)*D;
  A[21][2]=(0.0)*D;
  A[21][3]=(-2.2574955908289240e-03)*D;
  A[21][4]=(0.0)*D;
  A[21][5]=(-6.7724867724867728e-03)*D;
  A[21][6]=(0.0)*D;
  A[21][7]=(-2.2574955908289240e-03)*D;
  A[21][8]=(0.0)*D;
  A[21][9]=(-1.4109347442680777e-03)*D;
  A[21][10]=(0.0)*D;
  A[21][11]=(2.2574955908289240e-03)*D;
  A[21][12]=(0.0)*D;
  A[21][13]=(-1.3544973544973546e-02)*D;
  A[21][14]=(0.0)*D;
  A[21][15]=(-1.3544973544973546e-02)*D;
  A[21][16]=(0.0)*D;
  A[21][17]=(2.2574955908289240e-03)*D;
  A[21][18]=(0.0)*D;
  A[21][19]=(3.3862433862433864e-03)*D;
  A[21][20]=(0.0)*D;
  A[21][21]=(9.4814814814814810e-02)*D;
  A[21][22]=(0.0)*D;
  A[21][23]=(3.3862433862433864e-03)*D;
  A[21][24]=(0.0)*D;
  A[21][25]=(1.1287477954144622e-02)*D;
  A[21][26]=(0.0)*D;
  A[21][27]=(1.1287477954144622e-02)*D;
  A[21][28]=(0.0)*D;
  A[21][29]=(1.4109347442680777e-03)*D;
  A[22][0]=(-1.0582010582010582e-04)*D;
  A[22][1]=(0.0)*D;
  A[22][2]=(5.6437389770723099e-04)*D;
  A[22][3]=(0.0)*D;
  A[22][4]=(4.2328042328042330e-04)*D;
  A[22][5]=(0.0)*D;
  A[22][6]=(-8.4656084656084662e-03)*D;
  A[22][7]=(0.0)*D;
  A[22][8]=(-7.0546737213403885e-04)*D;
  A[22][9]=(0.0)*D;
  A[22][10]=(5.6437389770723099e-04)*D;
  A[22][11]=(0.0)*D;
  A[22][12]=(-6.7724867724867728e-03)*D;
  A[22][13]=(0.0)*D;
  A[22][14]=(3.3862433862433864e-03)*D;
  A[22][15]=(0.0)*D;
  A[22][16]=(-1.1287477954144622e-02)*D;
  A[22][17]=(0.0)*D;
  A[22][18]=(4.2328042328042330e-04)*D;
  A[22][19]=(0.0)*D;
  A[22][20]=(3.3862433862433864e-03)*D;
  A[22][21]=(0.0)*D;
  A[22][22]=(2.7936507936507936e-02)*D;
  A[22][23]=(0.0)*D;
  A[22][24]=(-8.4656084656084662e-03)*D;
  A[22][25]=(0.0)*D;
  A[22][26]=(-1.1287477954144622e-02)*D;
  A[22][27]=(0.0)*D;
  A[22][28]=(-7.0546737213403885e-04)*D;
  A[22][29]=(0.0)*D;
  A[23][0]=(0.0)*D;
  A[23][1]=(-1.0582010582010582e-04)*D;
  A[23][2]=(0.0)*D;
  A[23][3]=(5.6437389770723099e-04)*D;
  A[23][4]=(0.0)*D;
  A[23][5]=(4.2328042328042330e-04)*D;
  A[23][6]=(0.0)*D;
  A[23][7]=(-8.4656084656084662e-03)*D;
  A[23][8]=(0.0)*D;
  A[23][9]=(-7.0546737213403885e-04)*D;
  A[23][10]=(0.0)*D;
  A[23][11]=(5.6437389770723099e-04)*D;
  A[23][12]=(0.0)*D;
  A[23][13]=(-6.7724867724867728e-03)*D;
  A[23][14]=(0.0)*D;
  A[23][15]=(3.3862433862433864e-03)*D;
  A[23][16]=(0.0)*D;
  A[23][17]=(-1.1287477954144622e-02)*D;
  A[23][18]=(0.0)*D;
  A[23][19]=(4.2328042328042330e-04)*D;
  A[23][20]=(0.0)*D;
  A[23][21]=(3.3862433862433864e-03)*D;
  A[23][22]=(0.0)*D;
  A[23][23]=(2.7936507936507936e-02)*D;
  A[23][24]=(0.0)*D;
  A[23][25]=(-8.4656084656084662e-03)*D;
  A[23][26]=(0.0)*D;
  A[23][27]=(-1.1287477954144622e-02)*D;
  A[23][28]=(0.0)*D;
  A[23][29]=(-7.0546737213403885e-04)*D;
  A[24][0]=(0.0)*D;
  A[24][1]=(0.0)*D;
  A[24][2]=(4.5149911816578479e-03)*D;
  A[24][3]=(0.0)*D;
  A[24][4]=(5.6437389770723099e-04)*D;
  A[24][5]=(0.0)*D;
  A[24][6]=(2.2574955908289240e-03)*D;
  A[24][7]=(0.0)*D;
  A[24][8]=(-9.8765432098765434e-04)*D;
  A[24][9]=(0.0)*D;
  A[24][10]=(6.7724867724867728e-03)*D;
  A[24][11]=(0.0)*D;
  A[24][12]=(2.2574955908289240e-03)*D;
  A[24][13]=(0.0)*D;
  A[24][14]=(-2.2574955908289240e-03)*D;
  A[24][15]=(0.0)*D;
  A[24][16]=(4.5149911816578479e-03)*D;
  A[24][17]=(0.0)*D;
  A[24][18]=(-1.1287477954144622e-02)*D;
  A[24][19]=(0.0)*D;
  A[24][20]=(1.1287477954144622e-02)*D;
  A[24][21]=(0.0)*D;
  A[24][22]=(-8.4656084656084662e-03)*D;
  A[24][23]=(0.0)*D;
  A[24][24]=(2.2574955908289243e-02)*D;
  A[24][25]=(0.0)*D;
  A[24][26]=(1.1287477954144622e-02)*D;
  A[24][27]=(0.0)*D;
  A[24][28]=(1.4109347442680777e-03)*D;
  A[24][29]=(0.0)*D;
  A[25][0]=(0.0)*D;
  A[25][1]=(0.0)*D;
  A[25][2]=(0.0)*D;
  A[25][3]=(4.5149911816578479e-03)*D;
  A[25][4]=(0.0)*D;
  A[25][5]=(5.6437389770723099e-04)*D;
  A[25][6]=(0.0)*D;
  A[25][7]=(2.2574955908289240e-03)*D;
  A[25][8]=(0.0)*D;
  A[25][9]=(-9.8765432098765434e-04)*D;
  A[25][10]=(0.0)*D;
  A[25][11]=(6.7724867724867728e-03)*D;
  A[25][12]=(0.0)*D;
  A[25][13]=(2.2574955908289240e-03)*D;
  A[25][14]=(0.0)*D;
  A[25][15]=(-2.2574955908289240e-03)*D;
  A[25][16]=(0.0)*D;
  A[25][17]=(4.5149911816578479e-03)*D;
  A[25][18]=(0.0)*D;
  A[25][19]=(-1.1287477954144622e-02)*D;
  A[25][20]=(0.0)*D;
  A[25][21]=(1.1287477954144622e-02)*D;
  A[25][22]=(0.0)*D;
  A[25][23]=(-8.4656084656084662e-03)*D;
  A[25][24]=(0.0)*D;
  A[25][25]=(2.2574955908289243e-02)*D;
  A[25][26]=(0.0)*D;
  A[25][27]=(1.1287477954144622e-02)*D;
  A[25][28]=(0.0)*D;
  A[25][29]=(1.4109347442680777e-03)*D;
  A[26][0]=(-9.8765432098765434e-04)*D;
  A[26][1]=(0.0)*D;
  A[26][2]=(2.2574955908289240e-03)*D;
  A[26][3]=(0.0)*D;
  A[26][4]=(5.6437389770723099e-04)*D;
  A[26][5]=(0.0)*D;
  A[26][6]=(4.5149911816578479e-03)*D;
  A[26][7]=(0.0)*D;
  A[26][8]=(0.0)*D;
  A[26][9]=(0.0)*D;
  A[26][10]=(4.5149911816578479e-03)*D;
  A[26][11]=(0.0)*D;
  A[26][12]=(-2.2574955908289240e-03)*D;
  A[26][13]=(0.0)*D;
  A[26][14]=(2.2574955908289240e-03)*D;
  A[26][15]=(0.0)*D;
  A[26][16]=(6.7724867724867728e-03)*D;
  A[26][17]=(0.0)*D;
  A[26][18]=(-8.4656084656084662e-03)*D;
  A[26][19]=(0.0)*D;
  A[26][20]=(1.1287477954144622e-02)*D;
  A[26][21]=(0.0)*D;
  A[26][22]=(-1.1287477954144622e-02)*D;
  A[26][23]=(0.0)*D;
  A[26][24]=(1.1287477954144622e-02)*D;
  A[26][25]=(0.0)*D;
  A[26][26]=(2.2574955908289243e-02)*D;
  A[26][27]=(0.0)*D;
  A[26][28]=(1.4109347442680777e-03)*D;
  A[26][29]=(0.0)*D;
  A[27][0]=(0.0)*D;
  A[27][1]=(-9.8765432098765434e-04)*D;
  A[27][2]=(0.0)*D;
  A[27][3]=(2.2574955908289240e-03)*D;
  A[27][4]=(0.0)*D;
  A[27][5]=(5.6437389770723099e-04)*D;
  A[27][6]=(0.0)*D;
  A[27][7]=(4.5149911816578479e-03)*D;
  A[27][8]=(0.0)*D;
  A[27][9]=(0.0)*D;
  A[27][10]=(0.0)*D;
  A[27][11]=(4.5149911816578479e-03)*D;
  A[27][12]=(0.0)*D;
  A[27][13]=(-2.2574955908289240e-03)*D;
  A[27][14]=(0.0)*D;
  A[27][15]=(2.2574955908289240e-03)*D;
  A[27][16]=(0.0)*D;
  A[27][17]=(6.7724867724867728e-03)*D;
  A[27][18]=(0.0)*D;
  A[27][19]=(-8.4656084656084662e-03)*D;
  A[27][20]=(0.0)*D;
  A[27][21]=(1.1287477954144622e-02)*D;
  A[27][22]=(0.0)*D;
  A[27][23]=(-1.1287477954144622e-02)*D;
  A[27][24]=(0.0)*D;
  A[27][25]=(1.1287477954144622e-02)*D;
  A[27][26]=(0.0)*D;
  A[27][27]=(2.2574955908289243e-02)*D;
  A[27][28]=(0.0)*D;
  A[27][29]=(1.4109347442680777e-03)*D;
  A[28][0]=(-2.3809523809523810e-04)*D;
  A[28][1]=(0.0)*D;
  A[28][2]=(-9.8765432098765434e-04)*D;
  A[28][3]=(0.0)*D;
  A[28][4]=(-1.0582010582010582e-04)*D;
  A[28][5]=(0.0)*D;
  A[28][6]=(-9.8765432098765434e-04)*D;
  A[28][7]=(0.0)*D;
  A[28][8]=(-2.3809523809523810e-04)*D;
  A[28][9]=(0.0)*D;
  A[28][10]=(0.0)*D;
  A[28][11]=(0.0)*D;
  A[28][12]=(-1.4109347442680777e-03)*D;
  A[28][13]=(0.0)*D;
  A[28][14]=(-1.4109347442680777e-03)*D;
  A[28][15]=(0.0)*D;
  A[28][16]=(0.0)*D;
  A[28][17]=(0.0)*D;
  A[28][18]=(-7.0546737213403885e-04)*D;
  A[28][19]=(0.0)*D;
  A[28][20]=(1.4109347442680777e-03)*D;
  A[28][21]=(0.0)*D;
  A[28][22]=(-7.0546737213403885e-04)*D;
  A[28][23]=(0.0)*D;
  A[28][24]=(1.4109347442680777e-03)*D;
  A[28][25]=(0.0)*D;
  A[28][26]=(1.4109347442680777e-03)*D;
  A[28][27]=(0.0)*D;
  A[28][28]=(2.5573192239858908e-03)*D;
  A[28][29]=(0.0)*D;
  A[29][0]=(0.0)*D;
  A[29][1]=(-2.3809523809523810e-04)*D;
  A[29][2]=(0.0)*D;
  A[29][3]=(-9.8765432098765434e-04)*D;
  A[29][4]=(0.0)*D;
  A[29][5]=(-1.0582010582010582e-04)*D;
  A[29][6]=(0.0)*D;
  A[29][7]=(-9.8765432098765434e-04)*D;
  A[29][8]=(0.0)*D;
  A[29][9]=(-2.3809523809523810e-04)*D;
  A[29][10]=(0.0)*D;
  A[29][11]=(0.0)*D;
  A[29][12]=(0.0)*D;
  A[29][13]=(-1.4109347442680777e-03)*D;
  A[29][14]=(0.0)*D;
  A[29][15]=(-1.4109347442680777e-03)*D;
  A[29][16]=(0.0)*D;
  A[29][17]=(0.0)*D;
  A[29][18]=(0.0)*D;
  A[29][19]=(-7.0546737213403885e-04)*D;
  A[29][20]=(0.0)*D;
  A[29][21]=(1.4109347442680777e-03)*D;
  A[29][22]=(0.0)*D;
  A[29][23]=(-7.0546737213403885e-04)*D;
  A[29][24]=(0.0)*D;
  A[29][25]=(1.4109347442680777e-03)*D;
  A[29][26]=(0.0)*D;
  A[29][27]=(1.4109347442680777e-03)*D;
  A[29][28]=(0.0)*D;
  A[29][29]=(2.5573192239858908e-03)*D;
  int nbf = 30;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}


/* ------------- order 5 ---------------*/


  
void matrix_factory_vector_mass_2D_order5 (map<pair<int,int>,double>& matrix, 
         DofT<ptv,ptv_is_less>& dof, 
         int element, double pp0[2], double pp1[2], double pp2[2]){
  
  
  
  
  //variables

  double x0 = pp0[0]; double y0 = pp0[1]; 
  double x1 = pp1[0]; double y1 = pp1[1]; 
  double x2 = pp2[0]; double y2 = pp2[1]; 
  
  double G00 = x1 - x0; 
  double G01 = x2 - x0;  

  double G10 = y1 - y0; 
  double G11 = y2 - y0; 



  
  double D = fabs(G00*G11-G01*G10);  

  int    iidof[42];
  double idof0[3];
  idof0[0]=x0;  idof0[1]=y0;
  idof0[2] = 0.0;
  ptv ipdof0(3,idof0);
  iidof[0] = dof.insert_dof(element,0,ipdof0);

  double idof1[3];
  idof1[0]=x0;  idof1[1]=y0;
  idof1[2] = 1.0;
  ptv ipdof1(3,idof1);
  iidof[1] = dof.insert_dof(element,1,ipdof1);

  double idof2[3];
  idof2[0]=G00/5.0+x0;  idof2[1]=G10/5.0+y0;
  idof2[2] = 0.0;
  ptv ipdof2(3,idof2);
  iidof[2] = dof.insert_dof(element,2,ipdof2);

  double idof3[3];
  idof3[0]=G00/5.0+x0;  idof3[1]=G10/5.0+y0;
  idof3[2] = 1.0;
  ptv ipdof3(3,idof3);
  iidof[3] = dof.insert_dof(element,3,ipdof3);

  double idof4[3];
  idof4[0]=(2.0/5.0)*G00+x0;  idof4[1]=(2.0/5.0)*G10+y0;
  idof4[2] = 0.0;
  ptv ipdof4(3,idof4);
  iidof[4] = dof.insert_dof(element,4,ipdof4);

  double idof5[3];
  idof5[0]=(2.0/5.0)*G00+x0;  idof5[1]=(2.0/5.0)*G10+y0;
  idof5[2] = 1.0;
  ptv ipdof5(3,idof5);
  iidof[5] = dof.insert_dof(element,5,ipdof5);

  double idof6[3];
  idof6[0]=(3.0/5.0)*G00+x0;  idof6[1]=(3.0/5.0)*G10+y0;
  idof6[2] = 0.0;
  ptv ipdof6(3,idof6);
  iidof[6] = dof.insert_dof(element,6,ipdof6);

  double idof7[3];
  idof7[0]=(3.0/5.0)*G00+x0;  idof7[1]=(3.0/5.0)*G10+y0;
  idof7[2] = 1.0;
  ptv ipdof7(3,idof7);
  iidof[7] = dof.insert_dof(element,7,ipdof7);

  double idof8[3];
  idof8[0]=(4.0/5.0)*G00+x0;  idof8[1]=(4.0/5.0)*G10+y0;
  idof8[2] = 0.0;
  ptv ipdof8(3,idof8);
  iidof[8] = dof.insert_dof(element,8,ipdof8);

  double idof9[3];
  idof9[0]=(4.0/5.0)*G00+x0;  idof9[1]=(4.0/5.0)*G10+y0;
  idof9[2] = 1.0;
  ptv ipdof9(3,idof9);
  iidof[9] = dof.insert_dof(element,9,ipdof9);

  double idof10[3];
  idof10[0]=G00+x0;  idof10[1]=G10+y0;
  idof10[2] = 0.0;
  ptv ipdof10(3,idof10);
  iidof[10] = dof.insert_dof(element,10,ipdof10);

  double idof11[3];
  idof11[0]=G00+x0;  idof11[1]=G10+y0;
  idof11[2] = 1.0;
  ptv ipdof11(3,idof11);
  iidof[11] = dof.insert_dof(element,11,ipdof11);

  double idof12[3];
  idof12[0]=G01/5.0+x0;  idof12[1]=y0+G11/5.0;
  idof12[2] = 0.0;
  ptv ipdof12(3,idof12);
  iidof[12] = dof.insert_dof(element,12,ipdof12);

  double idof13[3];
  idof13[0]=G01/5.0+x0;  idof13[1]=y0+G11/5.0;
  idof13[2] = 1.0;
  ptv ipdof13(3,idof13);
  iidof[13] = dof.insert_dof(element,13,ipdof13);

  double idof14[3];
  idof14[0]=G01/5.0+G00/5.0+x0;  idof14[1]=G10/5.0+y0+G11/5.0;
  idof14[2] = 0.0;
  ptv ipdof14(3,idof14);
  iidof[14] = dof.insert_dof(element,14,ipdof14);

  double idof15[3];
  idof15[0]=G01/5.0+G00/5.0+x0;  idof15[1]=G10/5.0+y0+G11/5.0;
  idof15[2] = 1.0;
  ptv ipdof15(3,idof15);
  iidof[15] = dof.insert_dof(element,15,ipdof15);

  double idof16[3];
  idof16[0]=G01/5.0+(2.0/5.0)*G00+x0;  idof16[1]=(2.0/5.0)*G10+y0+G11/5.0;
  idof16[2] = 0.0;
  ptv ipdof16(3,idof16);
  iidof[16] = dof.insert_dof(element,16,ipdof16);

  double idof17[3];
  idof17[0]=G01/5.0+(2.0/5.0)*G00+x0;  idof17[1]=(2.0/5.0)*G10+y0+G11/5.0;
  idof17[2] = 1.0;
  ptv ipdof17(3,idof17);
  iidof[17] = dof.insert_dof(element,17,ipdof17);

  double idof18[3];
  idof18[0]=G01/5.0+(3.0/5.0)*G00+x0;  idof18[1]=(3.0/5.0)*G10+y0+G11/5.0;
  idof18[2] = 0.0;
  ptv ipdof18(3,idof18);
  iidof[18] = dof.insert_dof(element,18,ipdof18);

  double idof19[3];
  idof19[0]=G01/5.0+(3.0/5.0)*G00+x0;  idof19[1]=(3.0/5.0)*G10+y0+G11/5.0;
  idof19[2] = 1.0;
  ptv ipdof19(3,idof19);
  iidof[19] = dof.insert_dof(element,19,ipdof19);

  double idof20[3];
  idof20[0]=G01/5.0+(4.0/5.0)*G00+x0;  idof20[1]=(4.0/5.0)*G10+y0+G11/5.0;
  idof20[2] = 0.0;
  ptv ipdof20(3,idof20);
  iidof[20] = dof.insert_dof(element,20,ipdof20);

  double idof21[3];
  idof21[0]=G01/5.0+(4.0/5.0)*G00+x0;  idof21[1]=(4.0/5.0)*G10+y0+G11/5.0;
  idof21[2] = 1.0;
  ptv ipdof21(3,idof21);
  iidof[21] = dof.insert_dof(element,21,ipdof21);

  double idof22[3];
  idof22[0]=(2.0/5.0)*G01+x0;  idof22[1]=y0+(2.0/5.0)*G11;
  idof22[2] = 0.0;
  ptv ipdof22(3,idof22);
  iidof[22] = dof.insert_dof(element,22,ipdof22);

  double idof23[3];
  idof23[0]=(2.0/5.0)*G01+x0;  idof23[1]=y0+(2.0/5.0)*G11;
  idof23[2] = 1.0;
  ptv ipdof23(3,idof23);
  iidof[23] = dof.insert_dof(element,23,ipdof23);

  double idof24[3];
  idof24[0]=(2.0/5.0)*G01+G00/5.0+x0;  idof24[1]=G10/5.0+y0+(2.0/5.0)*G11;
  idof24[2] = 0.0;
  ptv ipdof24(3,idof24);
  iidof[24] = dof.insert_dof(element,24,ipdof24);

  double idof25[3];
  idof25[0]=(2.0/5.0)*G01+G00/5.0+x0;  idof25[1]=G10/5.0+y0+(2.0/5.0)*G11;
  idof25[2] = 1.0;
  ptv ipdof25(3,idof25);
  iidof[25] = dof.insert_dof(element,25,ipdof25);

  double idof26[3];
  idof26[0]=(2.0/5.0)*G01+(2.0/5.0)*G00+x0;  idof26[1]=(2.0/5.0)*G10+y0+(2.0/5.0)*G11;
  idof26[2] = 0.0;
  ptv ipdof26(3,idof26);
  iidof[26] = dof.insert_dof(element,26,ipdof26);

  double idof27[3];
  idof27[0]=(2.0/5.0)*G01+(2.0/5.0)*G00+x0;  idof27[1]=(2.0/5.0)*G10+y0+(2.0/5.0)*G11;
  idof27[2] = 1.0;
  ptv ipdof27(3,idof27);
  iidof[27] = dof.insert_dof(element,27,ipdof27);

  double idof28[3];
  idof28[0]=(2.0/5.0)*G01+(3.0/5.0)*G00+x0;  idof28[1]=(3.0/5.0)*G10+y0+(2.0/5.0)*G11;
  idof28[2] = 0.0;
  ptv ipdof28(3,idof28);
  iidof[28] = dof.insert_dof(element,28,ipdof28);

  double idof29[3];
  idof29[0]=(2.0/5.0)*G01+(3.0/5.0)*G00+x0;  idof29[1]=(3.0/5.0)*G10+y0+(2.0/5.0)*G11;
  idof29[2] = 1.0;
  ptv ipdof29(3,idof29);
  iidof[29] = dof.insert_dof(element,29,ipdof29);

  double idof30[3];
  idof30[0]=(3.0/5.0)*G01+x0;  idof30[1]=y0+(3.0/5.0)*G11;
  idof30[2] = 0.0;
  ptv ipdof30(3,idof30);
  iidof[30] = dof.insert_dof(element,30,ipdof30);

  double idof31[3];
  idof31[0]=(3.0/5.0)*G01+x0;  idof31[1]=y0+(3.0/5.0)*G11;
  idof31[2] = 1.0;
  ptv ipdof31(3,idof31);
  iidof[31] = dof.insert_dof(element,31,ipdof31);

  double idof32[3];
  idof32[0]=(3.0/5.0)*G01+G00/5.0+x0;  idof32[1]=G10/5.0+y0+(3.0/5.0)*G11;
  idof32[2] = 0.0;
  ptv ipdof32(3,idof32);
  iidof[32] = dof.insert_dof(element,32,ipdof32);

  double idof33[3];
  idof33[0]=(3.0/5.0)*G01+G00/5.0+x0;  idof33[1]=G10/5.0+y0+(3.0/5.0)*G11;
  idof33[2] = 1.0;
  ptv ipdof33(3,idof33);
  iidof[33] = dof.insert_dof(element,33,ipdof33);

  double idof34[3];
  idof34[0]=(3.0/5.0)*G01+(2.0/5.0)*G00+x0;  idof34[1]=(2.0/5.0)*G10+y0+(3.0/5.0)*G11;
  idof34[2] = 0.0;
  ptv ipdof34(3,idof34);
  iidof[34] = dof.insert_dof(element,34,ipdof34);

  double idof35[3];
  idof35[0]=(3.0/5.0)*G01+(2.0/5.0)*G00+x0;  idof35[1]=(2.0/5.0)*G10+y0+(3.0/5.0)*G11;
  idof35[2] = 1.0;
  ptv ipdof35(3,idof35);
  iidof[35] = dof.insert_dof(element,35,ipdof35);

  double idof36[3];
  idof36[0]=(4.0/5.0)*G01+x0;  idof36[1]=y0+(4.0/5.0)*G11;
  idof36[2] = 0.0;
  ptv ipdof36(3,idof36);
  iidof[36] = dof.insert_dof(element,36,ipdof36);

  double idof37[3];
  idof37[0]=(4.0/5.0)*G01+x0;  idof37[1]=y0+(4.0/5.0)*G11;
  idof37[2] = 1.0;
  ptv ipdof37(3,idof37);
  iidof[37] = dof.insert_dof(element,37,ipdof37);

  double idof38[3];
  idof38[0]=(4.0/5.0)*G01+G00/5.0+x0;  idof38[1]=G10/5.0+y0+(4.0/5.0)*G11;
  idof38[2] = 0.0;
  ptv ipdof38(3,idof38);
  iidof[38] = dof.insert_dof(element,38,ipdof38);

  double idof39[3];
  idof39[0]=(4.0/5.0)*G01+G00/5.0+x0;  idof39[1]=G10/5.0+y0+(4.0/5.0)*G11;
  idof39[2] = 1.0;
  ptv ipdof39(3,idof39);
  iidof[39] = dof.insert_dof(element,39,ipdof39);

  double idof40[3];
  idof40[0]=G01+x0;  idof40[1]=y0+G11;
  idof40[2] = 0.0;
  ptv ipdof40(3,idof40);
  iidof[40] = dof.insert_dof(element,40,ipdof40);

  double idof41[3];
  idof41[0]=G01+x0;  idof41[1]=y0+G11;
  idof41[2] = 1.0;
  ptv ipdof41(3,idof41);
  iidof[41] = dof.insert_dof(element,41,ipdof41);

 
  double A[42][42];
  A[0][0]=(1.3894525613275613e-03)*D;
  A[0][1]=(0.0)*D;
  A[0][2]=(1.1813634860509861e-03)*D;
  A[0][3]=(0.0)*D;
  A[0][4]=(-9.5824314574314575e-04)*D;
  A[0][5]=(0.0)*D;
  A[0][6]=(4.6268112674362677e-04)*D;
  A[0][7]=(0.0)*D;
  A[0][8]=(-2.2964432686654910e-05)*D;
  A[0][9]=(0.0)*D;
  A[0][10]=(1.2387745677676232e-04)*D;
  A[0][11]=(0.0)*D;
  A[0][12]=(1.1813634860509861e-03)*D;
  A[0][13]=(0.0)*D;
  A[0][14]=(8.9248136123136121e-04)*D;
  A[0][15]=(0.0)*D;
  A[0][16]=(-9.7468359187109191e-04)*D;
  A[0][17]=(0.0)*D;
  A[0][18]=(8.7943338811394365e-04)*D;
  A[0][19]=(0.0)*D;
  A[0][20]=(3.6364701078242745e-04)*D;
  A[0][21]=(0.0)*D;
  A[0][22]=(-9.5824314574314575e-04)*D;
  A[0][23]=(0.0)*D;
  A[0][24]=(-9.7468359187109191e-04)*D;
  A[0][25]=(0.0)*D;
  A[0][26]=(6.4978906124739453e-04)*D;
  A[0][27]=(0.0)*D;
  A[0][28]=(2.0720181310459088e-04)*D;
  A[0][29]=(0.0)*D;
  A[0][30]=(4.6268112674362677e-04)*D;
  A[0][31]=(0.0)*D;
  A[0][32]=(8.7943338811394365e-04)*D;
  A[0][33]=(0.0)*D;
  A[0][34]=(2.0720181310459088e-04)*D;
  A[0][35]=(0.0)*D;
  A[0][36]=(-2.2964432686654910e-05)*D;
  A[0][37]=(0.0)*D;
  A[0][38]=(3.6364701078242745e-04)*D;
  A[0][39]=(0.0)*D;
  A[0][40]=(1.2387745677676232e-04)*D;
  A[0][41]=(0.0)*D;
  A[1][0]=(0.0)*D;
  A[1][1]=(1.3894525613275613e-03)*D;
  A[1][2]=(0.0)*D;
  A[1][3]=(1.1813634860509861e-03)*D;
  A[1][4]=(0.0)*D;
  A[1][5]=(-9.5824314574314575e-04)*D;
  A[1][6]=(0.0)*D;
  A[1][7]=(4.6268112674362677e-04)*D;
  A[1][8]=(0.0)*D;
  A[1][9]=(-2.2964432686654910e-05)*D;
  A[1][10]=(0.0)*D;
  A[1][11]=(1.2387745677676232e-04)*D;
  A[1][12]=(0.0)*D;
  A[1][13]=(1.1813634860509861e-03)*D;
  A[1][14]=(0.0)*D;
  A[1][15]=(8.9248136123136121e-04)*D;
  A[1][16]=(0.0)*D;
  A[1][17]=(-9.7468359187109191e-04)*D;
  A[1][18]=(0.0)*D;
  A[1][19]=(8.7943338811394365e-04)*D;
  A[1][20]=(0.0)*D;
  A[1][21]=(3.6364701078242745e-04)*D;
  A[1][22]=(0.0)*D;
  A[1][23]=(-9.5824314574314575e-04)*D;
  A[1][24]=(0.0)*D;
  A[1][25]=(-9.7468359187109191e-04)*D;
  A[1][26]=(0.0)*D;
  A[1][27]=(6.4978906124739453e-04)*D;
  A[1][28]=(0.0)*D;
  A[1][29]=(2.0720181310459088e-04)*D;
  A[1][30]=(0.0)*D;
  A[1][31]=(4.6268112674362677e-04)*D;
  A[1][32]=(0.0)*D;
  A[1][33]=(8.7943338811394365e-04)*D;
  A[1][34]=(0.0)*D;
  A[1][35]=(2.0720181310459088e-04)*D;
  A[1][36]=(0.0)*D;
  A[1][37]=(-2.2964432686654910e-05)*D;
  A[1][38]=(0.0)*D;
  A[1][39]=(3.6364701078242745e-04)*D;
  A[1][40]=(0.0)*D;
  A[1][41]=(1.2387745677676232e-04)*D;
  A[2][0]=(1.1813634860509861e-03)*D;
  A[2][1]=(0.0)*D;
  A[2][2]=(1.4705065703329592e-02)*D;
  A[2][3]=(0.0)*D;
  A[2][4]=(-9.5837362547431994e-03)*D;
  A[2][5]=(0.0)*D;
  A[2][6]=(7.7048281258350702e-03)*D;
  A[2][7]=(0.0)*D;
  A[2][8]=(-3.9698458209742934e-03)*D;
  A[2][9]=(0.0)*D;
  A[2][10]=(-2.2964432686654910e-05)*D;
  A[2][11]=(0.0)*D;
  A[2][12]=(7.3525328516647961e-03)*D;
  A[2][13]=(0.0)*D;
  A[2][14]=(1.0242658897172786e-02)*D;
  A[2][15]=(0.0)*D;
  A[2][16]=(9.7859798380631720e-05)*D;
  A[2][17]=(0.0)*D;
  A[2][18]=(-2.8053142202447757e-03)*D;
  A[2][19]=(0.0)*D;
  A[2][20]=(-2.4823768855886910e-03)*D;
  A[2][21]=(0.0)*D;
  A[2][22]=(-7.3525328516647961e-03)*D;
  A[2][23]=(0.0)*D;
  A[2][24]=(-3.8165321368446368e-03)*D;
  A[2][25]=(0.0)*D;
  A[2][26]=(-1.3700371773288440e-03)*D;
  A[2][27]=(0.0)*D;
  A[2][28]=(-4.5667905910961469e-05)*D;
  A[2][29]=(0.0)*D;
  A[2][30]=(5.7606801313398537e-03)*D;
  A[2][31]=(0.0)*D;
  A[2][32]=(7.8287838704505376e-04)*D;
  A[2][33]=(0.0)*D;
  A[2][34]=(-1.4483250160333494e-03)*D;
  A[2][35]=(0.0)*D;
  A[2][36]=(-2.4823768855886910e-03)*D;
  A[2][37]=(0.0)*D;
  A[2][38]=(-4.1101115319865323e-04)*D;
  A[2][39]=(0.0)*D;
  A[2][40]=(3.6364701078242745e-04)*D;
  A[2][41]=(0.0)*D;
  A[3][0]=(0.0)*D;
  A[3][1]=(1.1813634860509861e-03)*D;
  A[3][2]=(0.0)*D;
  A[3][3]=(1.4705065703329592e-02)*D;
  A[3][4]=(0.0)*D;
  A[3][5]=(-9.5837362547431994e-03)*D;
  A[3][6]=(0.0)*D;
  A[3][7]=(7.7048281258350702e-03)*D;
  A[3][8]=(0.0)*D;
  A[3][9]=(-3.9698458209742934e-03)*D;
  A[3][10]=(0.0)*D;
  A[3][11]=(-2.2964432686654910e-05)*D;
  A[3][12]=(0.0)*D;
  A[3][13]=(7.3525328516647961e-03)*D;
  A[3][14]=(0.0)*D;
  A[3][15]=(1.0242658897172786e-02)*D;
  A[3][16]=(0.0)*D;
  A[3][17]=(9.7859798380631720e-05)*D;
  A[3][18]=(0.0)*D;
  A[3][19]=(-2.8053142202447757e-03)*D;
  A[3][20]=(0.0)*D;
  A[3][21]=(-2.4823768855886910e-03)*D;
  A[3][22]=(0.0)*D;
  A[3][23]=(-7.3525328516647961e-03)*D;
  A[3][24]=(0.0)*D;
  A[3][25]=(-3.8165321368446368e-03)*D;
  A[3][26]=(0.0)*D;
  A[3][27]=(-1.3700371773288440e-03)*D;
  A[3][28]=(0.0)*D;
  A[3][29]=(-4.5667905910961469e-05)*D;
  A[3][30]=(0.0)*D;
  A[3][31]=(5.7606801313398537e-03)*D;
  A[3][32]=(0.0)*D;
  A[3][33]=(7.8287838704505376e-04)*D;
  A[3][34]=(0.0)*D;
  A[3][35]=(-1.4483250160333494e-03)*D;
  A[3][36]=(0.0)*D;
  A[3][37]=(-2.4823768855886910e-03)*D;
  A[3][38]=(0.0)*D;
  A[3][39]=(-4.1101115319865323e-04)*D;
  A[3][40]=(0.0)*D;
  A[3][41]=(3.6364701078242745e-04)*D;
  A[4][0]=(-9.5824314574314575e-04)*D;
  A[4][1]=(0.0)*D;
  A[4][2]=(-9.5837362547431994e-03)*D;
  A[4][3]=(0.0)*D;
  A[4][4]=(2.2077170514670515e-02)*D;
  A[4][5]=(0.0)*D;
  A[4][6]=(-1.3478556230292342e-02)*D;
  A[4][7]=(0.0)*D;
  A[4][8]=(7.7048281258350702e-03)*D;
  A[4][9]=(0.0)*D;
  A[4][10]=(4.6268112674362677e-04)*D;
  A[4][11]=(0.0)*D;
  A[4][12]=(-7.3525328516647961e-03)*D;
  A[4][13]=(0.0)*D;
  A[4][14]=(-1.4352770429159319e-03)*D;
  A[4][15]=(0.0)*D;
  A[4][16]=(5.2844291125541129e-03)*D;
  A[4][17]=(0.0)*D;
  A[4][18]=(5.0887095157928488e-03)*D;
  A[4][19]=(0.0)*D;
  A[4][20]=(5.7606801313398537e-03)*D;
  A[4][21]=(0.0)*D;
  A[4][22]=(2.9488419245363691e-03)*D;
  A[4][23]=(0.0)*D;
  A[4][24]=(-5.4801487093153762e-03)*D;
  A[4][25]=(0.0)*D;
  A[4][26]=(-5.8715879028379029e-04)*D;
  A[4][27]=(0.0)*D;
  A[4][28]=(-1.3569892042114265e-03)*D;
  A[4][29]=(0.0)*D;
  A[4][30]=(-1.3569892042114265e-03)*D;
  A[4][31]=(0.0)*D;
  A[4][32]=(3.1315135481802150e-03)*D;
  A[4][33]=(0.0)*D;
  A[4][34]=(2.8183621933621935e-03)*D;
  A[4][35]=(0.0)*D;
  A[4][36]=(-4.5667905910961469e-05)*D;
  A[4][37]=(0.0)*D;
  A[4][38]=(-1.4483250160333494e-03)*D;
  A[4][39]=(0.0)*D;
  A[4][40]=(2.0720181310459088e-04)*D;
  A[4][41]=(0.0)*D;
  A[5][0]=(0.0)*D;
  A[5][1]=(-9.5824314574314575e-04)*D;
  A[5][2]=(0.0)*D;
  A[5][3]=(-9.5837362547431994e-03)*D;
  A[5][4]=(0.0)*D;
  A[5][5]=(2.2077170514670515e-02)*D;
  A[5][6]=(0.0)*D;
  A[5][7]=(-1.3478556230292342e-02)*D;
  A[5][8]=(0.0)*D;
  A[5][9]=(7.7048281258350702e-03)*D;
  A[5][10]=(0.0)*D;
  A[5][11]=(4.6268112674362677e-04)*D;
  A[5][12]=(0.0)*D;
  A[5][13]=(-7.3525328516647961e-03)*D;
  A[5][14]=(0.0)*D;
  A[5][15]=(-1.4352770429159319e-03)*D;
  A[5][16]=(0.0)*D;
  A[5][17]=(5.2844291125541129e-03)*D;
  A[5][18]=(0.0)*D;
  A[5][19]=(5.0887095157928488e-03)*D;
  A[5][20]=(0.0)*D;
  A[5][21]=(5.7606801313398537e-03)*D;
  A[5][22]=(0.0)*D;
  A[5][23]=(2.9488419245363691e-03)*D;
  A[5][24]=(0.0)*D;
  A[5][25]=(-5.4801487093153762e-03)*D;
  A[5][26]=(0.0)*D;
  A[5][27]=(-5.8715879028379029e-04)*D;
  A[5][28]=(0.0)*D;
  A[5][29]=(-1.3569892042114265e-03)*D;
  A[5][30]=(0.0)*D;
  A[5][31]=(-1.3569892042114265e-03)*D;
  A[5][32]=(0.0)*D;
  A[5][33]=(3.1315135481802150e-03)*D;
  A[5][34]=(0.0)*D;
  A[5][35]=(2.8183621933621935e-03)*D;
  A[5][36]=(0.0)*D;
  A[5][37]=(-4.5667905910961469e-05)*D;
  A[5][38]=(0.0)*D;
  A[5][39]=(-1.4483250160333494e-03)*D;
  A[5][40]=(0.0)*D;
  A[5][41]=(2.0720181310459088e-04)*D;
  A[6][0]=(4.6268112674362677e-04)*D;
  A[6][1]=(0.0)*D;
  A[6][2]=(7.7048281258350702e-03)*D;
  A[6][3]=(0.0)*D;
  A[6][4]=(-1.3478556230292342e-02)*D;
  A[6][5]=(0.0)*D;
  A[6][6]=(2.2077170514670515e-02)*D;
  A[6][7]=(0.0)*D;
  A[6][8]=(-9.5837362547431994e-03)*D;
  A[6][9]=(0.0)*D;
  A[6][10]=(-9.5824314574314575e-04)*D;
  A[6][11]=(0.0)*D;
  A[6][12]=(5.7606801313398537e-03)*D;
  A[6][13]=(0.0)*D;
  A[6][14]=(5.0887095157928488e-03)*D;
  A[6][15]=(0.0)*D;
  A[6][16]=(5.2844291125541129e-03)*D;
  A[6][17]=(0.0)*D;
  A[6][18]=(-1.4352770429159319e-03)*D;
  A[6][19]=(0.0)*D;
  A[6][20]=(-7.3525328516647961e-03)*D;
  A[6][21]=(0.0)*D;
  A[6][22]=(-1.3569892042114265e-03)*D;
  A[6][23]=(0.0)*D;
  A[6][24]=(-5.8715879028379029e-04)*D;
  A[6][25]=(0.0)*D;
  A[6][26]=(-5.4801487093153762e-03)*D;
  A[6][27]=(0.0)*D;
  A[6][28]=(2.9488419245363691e-03)*D;
  A[6][29]=(0.0)*D;
  A[6][30]=(2.8183621933621935e-03)*D;
  A[6][31]=(0.0)*D;
  A[6][32]=(3.1315135481802150e-03)*D;
  A[6][33]=(0.0)*D;
  A[6][34]=(-1.3569892042114265e-03)*D;
  A[6][35]=(0.0)*D;
  A[6][36]=(-1.4483250160333494e-03)*D;
  A[6][37]=(0.0)*D;
  A[6][38]=(-4.5667905910961469e-05)*D;
  A[6][39]=(0.0)*D;
  A[6][40]=(2.0720181310459088e-04)*D;
  A[6][41]=(0.0)*D;
  A[7][0]=(0.0)*D;
  A[7][1]=(4.6268112674362677e-04)*D;
  A[7][2]=(0.0)*D;
  A[7][3]=(7.7048281258350702e-03)*D;
  A[7][4]=(0.0)*D;
  A[7][5]=(-1.3478556230292342e-02)*D;
  A[7][6]=(0.0)*D;
  A[7][7]=(2.2077170514670515e-02)*D;
  A[7][8]=(0.0)*D;
  A[7][9]=(-9.5837362547431994e-03)*D;
  A[7][10]=(0.0)*D;
  A[7][11]=(-9.5824314574314575e-04)*D;
  A[7][12]=(0.0)*D;
  A[7][13]=(5.7606801313398537e-03)*D;
  A[7][14]=(0.0)*D;
  A[7][15]=(5.0887095157928488e-03)*D;
  A[7][16]=(0.0)*D;
  A[7][17]=(5.2844291125541129e-03)*D;
  A[7][18]=(0.0)*D;
  A[7][19]=(-1.4352770429159319e-03)*D;
  A[7][20]=(0.0)*D;
  A[7][21]=(-7.3525328516647961e-03)*D;
  A[7][22]=(0.0)*D;
  A[7][23]=(-1.3569892042114265e-03)*D;
  A[7][24]=(0.0)*D;
  A[7][25]=(-5.8715879028379029e-04)*D;
  A[7][26]=(0.0)*D;
  A[7][27]=(-5.4801487093153762e-03)*D;
  A[7][28]=(0.0)*D;
  A[7][29]=(2.9488419245363691e-03)*D;
  A[7][30]=(0.0)*D;
  A[7][31]=(2.8183621933621935e-03)*D;
  A[7][32]=(0.0)*D;
  A[7][33]=(3.1315135481802150e-03)*D;
  A[7][34]=(0.0)*D;
  A[7][35]=(-1.3569892042114265e-03)*D;
  A[7][36]=(0.0)*D;
  A[7][37]=(-1.4483250160333494e-03)*D;
  A[7][38]=(0.0)*D;
  A[7][39]=(-4.5667905910961469e-05)*D;
  A[7][40]=(0.0)*D;
  A[7][41]=(2.0720181310459088e-04)*D;
  A[8][0]=(-2.2964432686654910e-05)*D;
  A[8][1]=(0.0)*D;
  A[8][2]=(-3.9698458209742934e-03)*D;
  A[8][3]=(0.0)*D;
  A[8][4]=(7.7048281258350702e-03)*D;
  A[8][5]=(0.0)*D;
  A[8][6]=(-9.5837362547431994e-03)*D;
  A[8][7]=(0.0)*D;
  A[8][8]=(1.4705065703329592e-02)*D;
  A[8][9]=(0.0)*D;
  A[8][10]=(1.1813634860509861e-03)*D;
  A[8][11]=(0.0)*D;
  A[8][12]=(-2.4823768855886910e-03)*D;
  A[8][13]=(0.0)*D;
  A[8][14]=(-2.8053142202447757e-03)*D;
  A[8][15]=(0.0)*D;
  A[8][16]=(9.7859798380631720e-05)*D;
  A[8][17]=(0.0)*D;
  A[8][18]=(1.0242658897172786e-02)*D;
  A[8][19]=(0.0)*D;
  A[8][20]=(7.3525328516647961e-03)*D;
  A[8][21]=(0.0)*D;
  A[8][22]=(-4.5667905910961469e-05)*D;
  A[8][23]=(0.0)*D;
  A[8][24]=(-1.3700371773288440e-03)*D;
  A[8][25]=(0.0)*D;
  A[8][26]=(-3.8165321368446368e-03)*D;
  A[8][27]=(0.0)*D;
  A[8][28]=(-7.3525328516647961e-03)*D;
  A[8][29]=(0.0)*D;
  A[8][30]=(-1.4483250160333494e-03)*D;
  A[8][31]=(0.0)*D;
  A[8][32]=(7.8287838704505376e-04)*D;
  A[8][33]=(0.0)*D;
  A[8][34]=(5.7606801313398537e-03)*D;
  A[8][35]=(0.0)*D;
  A[8][36]=(-4.1101115319865323e-04)*D;
  A[8][37]=(0.0)*D;
  A[8][38]=(-2.4823768855886910e-03)*D;
  A[8][39]=(0.0)*D;
  A[8][40]=(3.6364701078242745e-04)*D;
  A[8][41]=(0.0)*D;
  A[9][0]=(0.0)*D;
  A[9][1]=(-2.2964432686654910e-05)*D;
  A[9][2]=(0.0)*D;
  A[9][3]=(-3.9698458209742934e-03)*D;
  A[9][4]=(0.0)*D;
  A[9][5]=(7.7048281258350702e-03)*D;
  A[9][6]=(0.0)*D;
  A[9][7]=(-9.5837362547431994e-03)*D;
  A[9][8]=(0.0)*D;
  A[9][9]=(1.4705065703329592e-02)*D;
  A[9][10]=(0.0)*D;
  A[9][11]=(1.1813634860509861e-03)*D;
  A[9][12]=(0.0)*D;
  A[9][13]=(-2.4823768855886910e-03)*D;
  A[9][14]=(0.0)*D;
  A[9][15]=(-2.8053142202447757e-03)*D;
  A[9][16]=(0.0)*D;
  A[9][17]=(9.7859798380631720e-05)*D;
  A[9][18]=(0.0)*D;
  A[9][19]=(1.0242658897172786e-02)*D;
  A[9][20]=(0.0)*D;
  A[9][21]=(7.3525328516647961e-03)*D;
  A[9][22]=(0.0)*D;
  A[9][23]=(-4.5667905910961469e-05)*D;
  A[9][24]=(0.0)*D;
  A[9][25]=(-1.3700371773288440e-03)*D;
  A[9][26]=(0.0)*D;
  A[9][27]=(-3.8165321368446368e-03)*D;
  A[9][28]=(0.0)*D;
  A[9][29]=(-7.3525328516647961e-03)*D;
  A[9][30]=(0.0)*D;
  A[9][31]=(-1.4483250160333494e-03)*D;
  A[9][32]=(0.0)*D;
  A[9][33]=(7.8287838704505376e-04)*D;
  A[9][34]=(0.0)*D;
  A[9][35]=(5.7606801313398537e-03)*D;
  A[9][36]=(0.0)*D;
  A[9][37]=(-4.1101115319865323e-04)*D;
  A[9][38]=(0.0)*D;
  A[9][39]=(-2.4823768855886910e-03)*D;
  A[9][40]=(0.0)*D;
  A[9][41]=(3.6364701078242745e-04)*D;
  A[10][0]=(1.2387745677676232e-04)*D;
  A[10][1]=(0.0)*D;
  A[10][2]=(-2.2964432686654910e-05)*D;
  A[10][3]=(0.0)*D;
  A[10][4]=(4.6268112674362677e-04)*D;
  A[10][5]=(0.0)*D;
  A[10][6]=(-9.5824314574314575e-04)*D;
  A[10][7]=(0.0)*D;
  A[10][8]=(1.1813634860509861e-03)*D;
  A[10][9]=(0.0)*D;
  A[10][10]=(1.3894525613275613e-03)*D;
  A[10][11]=(0.0)*D;
  A[10][12]=(3.6364701078242745e-04)*D;
  A[10][13]=(0.0)*D;
  A[10][14]=(8.7943338811394365e-04)*D;
  A[10][15]=(0.0)*D;
  A[10][16]=(-9.7468359187109191e-04)*D;
  A[10][17]=(0.0)*D;
  A[10][18]=(8.9248136123136121e-04)*D;
  A[10][19]=(0.0)*D;
  A[10][20]=(1.1813634860509861e-03)*D;
  A[10][21]=(0.0)*D;
  A[10][22]=(2.0720181310459088e-04)*D;
  A[10][23]=(0.0)*D;
  A[10][24]=(6.4978906124739453e-04)*D;
  A[10][25]=(0.0)*D;
  A[10][26]=(-9.7468359187109191e-04)*D;
  A[10][27]=(0.0)*D;
  A[10][28]=(-9.5824314574314575e-04)*D;
  A[10][29]=(0.0)*D;
  A[10][30]=(2.0720181310459088e-04)*D;
  A[10][31]=(0.0)*D;
  A[10][32]=(8.7943338811394365e-04)*D;
  A[10][33]=(0.0)*D;
  A[10][34]=(4.6268112674362677e-04)*D;
  A[10][35]=(0.0)*D;
  A[10][36]=(3.6364701078242745e-04)*D;
  A[10][37]=(0.0)*D;
  A[10][38]=(-2.2964432686654910e-05)*D;
  A[10][39]=(0.0)*D;
  A[10][40]=(1.2387745677676232e-04)*D;
  A[10][41]=(0.0)*D;
  A[11][0]=(0.0)*D;
  A[11][1]=(1.2387745677676232e-04)*D;
  A[11][2]=(0.0)*D;
  A[11][3]=(-2.2964432686654910e-05)*D;
  A[11][4]=(0.0)*D;
  A[11][5]=(4.6268112674362677e-04)*D;
  A[11][6]=(0.0)*D;
  A[11][7]=(-9.5824314574314575e-04)*D;
  A[11][8]=(0.0)*D;
  A[11][9]=(1.1813634860509861e-03)*D;
  A[11][10]=(0.0)*D;
  A[11][11]=(1.3894525613275613e-03)*D;
  A[11][12]=(0.0)*D;
  A[11][13]=(3.6364701078242745e-04)*D;
  A[11][14]=(0.0)*D;
  A[11][15]=(8.7943338811394365e-04)*D;
  A[11][16]=(0.0)*D;
  A[11][17]=(-9.7468359187109191e-04)*D;
  A[11][18]=(0.0)*D;
  A[11][19]=(8.9248136123136121e-04)*D;
  A[11][20]=(0.0)*D;
  A[11][21]=(1.1813634860509861e-03)*D;
  A[11][22]=(0.0)*D;
  A[11][23]=(2.0720181310459088e-04)*D;
  A[11][24]=(0.0)*D;
  A[11][25]=(6.4978906124739453e-04)*D;
  A[11][26]=(0.0)*D;
  A[11][27]=(-9.7468359187109191e-04)*D;
  A[11][28]=(0.0)*D;
  A[11][29]=(-9.5824314574314575e-04)*D;
  A[11][30]=(0.0)*D;
  A[11][31]=(2.0720181310459088e-04)*D;
  A[11][32]=(0.0)*D;
  A[11][33]=(8.7943338811394365e-04)*D;
  A[11][34]=(0.0)*D;
  A[11][35]=(4.6268112674362677e-04)*D;
  A[11][36]=(0.0)*D;
  A[11][37]=(3.6364701078242745e-04)*D;
  A[11][38]=(0.0)*D;
  A[11][39]=(-2.2964432686654910e-05)*D;
  A[11][40]=(0.0)*D;
  A[11][41]=(1.2387745677676232e-04)*D;
  A[12][0]=(1.1813634860509861e-03)*D;
  A[12][1]=(0.0)*D;
  A[12][2]=(7.3525328516647961e-03)*D;
  A[12][3]=(0.0)*D;
  A[12][4]=(-7.3525328516647961e-03)*D;
  A[12][5]=(0.0)*D;
  A[12][6]=(5.7606801313398537e-03)*D;
  A[12][7]=(0.0)*D;
  A[12][8]=(-2.4823768855886910e-03)*D;
  A[12][9]=(0.0)*D;
  A[12][10]=(3.6364701078242745e-04)*D;
  A[12][11]=(0.0)*D;
  A[12][12]=(1.4705065703329592e-02)*D;
  A[12][13]=(0.0)*D;
  A[12][14]=(1.0242658897172786e-02)*D;
  A[12][15]=(0.0)*D;
  A[12][16]=(-3.8165321368446368e-03)*D;
  A[12][17]=(0.0)*D;
  A[12][18]=(7.8287838704505376e-04)*D;
  A[12][19]=(0.0)*D;
  A[12][20]=(-4.1101115319865323e-04)*D;
  A[12][21]=(0.0)*D;
  A[12][22]=(-9.5837362547431994e-03)*D;
  A[12][23]=(0.0)*D;
  A[12][24]=(9.7859798380631720e-05)*D;
  A[12][25]=(0.0)*D;
  A[12][26]=(-1.3700371773288440e-03)*D;
  A[12][27]=(0.0)*D;
  A[12][28]=(-1.4483250160333494e-03)*D;
  A[12][29]=(0.0)*D;
  A[12][30]=(7.7048281258350702e-03)*D;
  A[12][31]=(0.0)*D;
  A[12][32]=(-2.8053142202447757e-03)*D;
  A[12][33]=(0.0)*D;
  A[12][34]=(-4.5667905910961469e-05)*D;
  A[12][35]=(0.0)*D;
  A[12][36]=(-3.9698458209742934e-03)*D;
  A[12][37]=(0.0)*D;
  A[12][38]=(-2.4823768855886910e-03)*D;
  A[12][39]=(0.0)*D;
  A[12][40]=(-2.2964432686654910e-05)*D;
  A[12][41]=(0.0)*D;
  A[13][0]=(0.0)*D;
  A[13][1]=(1.1813634860509861e-03)*D;
  A[13][2]=(0.0)*D;
  A[13][3]=(7.3525328516647961e-03)*D;
  A[13][4]=(0.0)*D;
  A[13][5]=(-7.3525328516647961e-03)*D;
  A[13][6]=(0.0)*D;
  A[13][7]=(5.7606801313398537e-03)*D;
  A[13][8]=(0.0)*D;
  A[13][9]=(-2.4823768855886910e-03)*D;
  A[13][10]=(0.0)*D;
  A[13][11]=(3.6364701078242745e-04)*D;
  A[13][12]=(0.0)*D;
  A[13][13]=(1.4705065703329592e-02)*D;
  A[13][14]=(0.0)*D;
  A[13][15]=(1.0242658897172786e-02)*D;
  A[13][16]=(0.0)*D;
  A[13][17]=(-3.8165321368446368e-03)*D;
  A[13][18]=(0.0)*D;
  A[13][19]=(7.8287838704505376e-04)*D;
  A[13][20]=(0.0)*D;
  A[13][21]=(-4.1101115319865323e-04)*D;
  A[13][22]=(0.0)*D;
  A[13][23]=(-9.5837362547431994e-03)*D;
  A[13][24]=(0.0)*D;
  A[13][25]=(9.7859798380631720e-05)*D;
  A[13][26]=(0.0)*D;
  A[13][27]=(-1.3700371773288440e-03)*D;
  A[13][28]=(0.0)*D;
  A[13][29]=(-1.4483250160333494e-03)*D;
  A[13][30]=(0.0)*D;
  A[13][31]=(7.7048281258350702e-03)*D;
  A[13][32]=(0.0)*D;
  A[13][33]=(-2.8053142202447757e-03)*D;
  A[13][34]=(0.0)*D;
  A[13][35]=(-4.5667905910961469e-05)*D;
  A[13][36]=(0.0)*D;
  A[13][37]=(-3.9698458209742934e-03)*D;
  A[13][38]=(0.0)*D;
  A[13][39]=(-2.4823768855886910e-03)*D;
  A[13][40]=(0.0)*D;
  A[13][41]=(-2.2964432686654910e-05)*D;
  A[14][0]=(8.9248136123136121e-04)*D;
  A[14][1]=(0.0)*D;
  A[14][2]=(1.0242658897172786e-02)*D;
  A[14][3]=(0.0)*D;
  A[14][4]=(-1.4352770429159319e-03)*D;
  A[14][5]=(0.0)*D;
  A[14][6]=(5.0887095157928488e-03)*D;
  A[14][7]=(0.0)*D;
  A[14][8]=(-2.8053142202447757e-03)*D;
  A[14][9]=(0.0)*D;
  A[14][10]=(8.7943338811394365e-04)*D;
  A[14][11]=(0.0)*D;
  A[14][12]=(1.0242658897172786e-02)*D;
  A[14][13]=(0.0)*D;
  A[14][14]=(7.0459054834054832e-02)*D;
  A[14][15]=(0.0)*D;
  A[14][16]=(-1.5657567740901074e-02)*D;
  A[14][17]=(0.0)*D;
  A[14][18]=(1.1743175805675805e-02)*D;
  A[14][19]=(0.0)*D;
  A[14][20]=(7.8287838704505376e-04)*D;
  A[14][21]=(0.0)*D;
  A[14][22]=(-1.4352770429159319e-03)*D;
  A[14][23]=(0.0)*D;
  A[14][24]=(-1.5657567740901074e-02)*D;
  A[14][25]=(0.0)*D;
  A[14][26]=(3.9143919352252685e-03)*D;
  A[14][27]=(0.0)*D;
  A[14][28]=(3.1315135481802150e-03)*D;
  A[14][29]=(0.0)*D;
  A[14][30]=(5.0887095157928488e-03)*D;
  A[14][31]=(0.0)*D;
  A[14][32]=(1.1743175805675805e-02)*D;
  A[14][33]=(0.0)*D;
  A[14][34]=(3.1315135481802150e-03)*D;
  A[14][35]=(0.0)*D;
  A[14][36]=(-2.8053142202447757e-03)*D;
  A[14][37]=(0.0)*D;
  A[14][38]=(7.8287838704505376e-04)*D;
  A[14][39]=(0.0)*D;
  A[14][40]=(8.7943338811394365e-04)*D;
  A[14][41]=(0.0)*D;
  A[15][0]=(0.0)*D;
  A[15][1]=(8.9248136123136121e-04)*D;
  A[15][2]=(0.0)*D;
  A[15][3]=(1.0242658897172786e-02)*D;
  A[15][4]=(0.0)*D;
  A[15][5]=(-1.4352770429159319e-03)*D;
  A[15][6]=(0.0)*D;
  A[15][7]=(5.0887095157928488e-03)*D;
  A[15][8]=(0.0)*D;
  A[15][9]=(-2.8053142202447757e-03)*D;
  A[15][10]=(0.0)*D;
  A[15][11]=(8.7943338811394365e-04)*D;
  A[15][12]=(0.0)*D;
  A[15][13]=(1.0242658897172786e-02)*D;
  A[15][14]=(0.0)*D;
  A[15][15]=(7.0459054834054832e-02)*D;
  A[15][16]=(0.0)*D;
  A[15][17]=(-1.5657567740901074e-02)*D;
  A[15][18]=(0.0)*D;
  A[15][19]=(1.1743175805675805e-02)*D;
  A[15][20]=(0.0)*D;
  A[15][21]=(7.8287838704505376e-04)*D;
  A[15][22]=(0.0)*D;
  A[15][23]=(-1.4352770429159319e-03)*D;
  A[15][24]=(0.0)*D;
  A[15][25]=(-1.5657567740901074e-02)*D;
  A[15][26]=(0.0)*D;
  A[15][27]=(3.9143919352252685e-03)*D;
  A[15][28]=(0.0)*D;
  A[15][29]=(3.1315135481802150e-03)*D;
  A[15][30]=(0.0)*D;
  A[15][31]=(5.0887095157928488e-03)*D;
  A[15][32]=(0.0)*D;
  A[15][33]=(1.1743175805675805e-02)*D;
  A[15][34]=(0.0)*D;
  A[15][35]=(3.1315135481802150e-03)*D;
  A[15][36]=(0.0)*D;
  A[15][37]=(-2.8053142202447757e-03)*D;
  A[15][38]=(0.0)*D;
  A[15][39]=(7.8287838704505376e-04)*D;
  A[15][40]=(0.0)*D;
  A[15][41]=(8.7943338811394365e-04)*D;
  A[16][0]=(-9.7468359187109191e-04)*D;
  A[16][1]=(0.0)*D;
  A[16][2]=(9.7859798380631720e-05)*D;
  A[16][3]=(0.0)*D;
  A[16][4]=(5.2844291125541129e-03)*D;
  A[16][5]=(0.0)*D;
  A[16][6]=(5.2844291125541129e-03)*D;
  A[16][7]=(0.0)*D;
  A[16][8]=(9.7859798380631720e-05)*D;
  A[16][9]=(0.0)*D;
  A[16][10]=(-9.7468359187109191e-04)*D;
  A[16][11]=(0.0)*D;
  A[16][12]=(-3.8165321368446368e-03)*D;
  A[16][13]=(0.0)*D;
  A[16][14]=(-1.5657567740901074e-02)*D;
  A[16][15]=(0.0)*D;
  A[16][16]=(7.6330642736892731e-02)*D;
  A[16][17]=(0.0)*D;
  A[16][18]=(-1.5657567740901074e-02)*D;
  A[16][19]=(0.0)*D;
  A[16][20]=(-3.8165321368446368e-03)*D;
  A[16][21]=(0.0)*D;
  A[16][22]=(-5.4801487093153762e-03)*D;
  A[16][23]=(0.0)*D;
  A[16][24]=(-1.1743175805675805e-02)*D;
  A[16][25]=(0.0)*D;
  A[16][26]=(-1.1743175805675805e-02)*D;
  A[16][27]=(0.0)*D;
  A[16][28]=(-5.4801487093153762e-03)*D;
  A[16][29]=(0.0)*D;
  A[16][30]=(-5.8715879028379029e-04)*D;
  A[16][31]=(0.0)*D;
  A[16][32]=(3.9143919352252685e-03)*D;
  A[16][33]=(0.0)*D;
  A[16][34]=(-5.8715879028379029e-04)*D;
  A[16][35]=(0.0)*D;
  A[16][36]=(-1.3700371773288440e-03)*D;
  A[16][37]=(0.0)*D;
  A[16][38]=(-1.3700371773288440e-03)*D;
  A[16][39]=(0.0)*D;
  A[16][40]=(6.4978906124739453e-04)*D;
  A[16][41]=(0.0)*D;
  A[17][0]=(0.0)*D;
  A[17][1]=(-9.7468359187109191e-04)*D;
  A[17][2]=(0.0)*D;
  A[17][3]=(9.7859798380631720e-05)*D;
  A[17][4]=(0.0)*D;
  A[17][5]=(5.2844291125541129e-03)*D;
  A[17][6]=(0.0)*D;
  A[17][7]=(5.2844291125541129e-03)*D;
  A[17][8]=(0.0)*D;
  A[17][9]=(9.7859798380631720e-05)*D;
  A[17][10]=(0.0)*D;
  A[17][11]=(-9.7468359187109191e-04)*D;
  A[17][12]=(0.0)*D;
  A[17][13]=(-3.8165321368446368e-03)*D;
  A[17][14]=(0.0)*D;
  A[17][15]=(-1.5657567740901074e-02)*D;
  A[17][16]=(0.0)*D;
  A[17][17]=(7.6330642736892731e-02)*D;
  A[17][18]=(0.0)*D;
  A[17][19]=(-1.5657567740901074e-02)*D;
  A[17][20]=(0.0)*D;
  A[17][21]=(-3.8165321368446368e-03)*D;
  A[17][22]=(0.0)*D;
  A[17][23]=(-5.4801487093153762e-03)*D;
  A[17][24]=(0.0)*D;
  A[17][25]=(-1.1743175805675805e-02)*D;
  A[17][26]=(0.0)*D;
  A[17][27]=(-1.1743175805675805e-02)*D;
  A[17][28]=(0.0)*D;
  A[17][29]=(-5.4801487093153762e-03)*D;
  A[17][30]=(0.0)*D;
  A[17][31]=(-5.8715879028379029e-04)*D;
  A[17][32]=(0.0)*D;
  A[17][33]=(3.9143919352252685e-03)*D;
  A[17][34]=(0.0)*D;
  A[17][35]=(-5.8715879028379029e-04)*D;
  A[17][36]=(0.0)*D;
  A[17][37]=(-1.3700371773288440e-03)*D;
  A[17][38]=(0.0)*D;
  A[17][39]=(-1.3700371773288440e-03)*D;
  A[17][40]=(0.0)*D;
  A[17][41]=(6.4978906124739453e-04)*D;
  A[18][0]=(8.7943338811394365e-04)*D;
  A[18][1]=(0.0)*D;
  A[18][2]=(-2.8053142202447757e-03)*D;
  A[18][3]=(0.0)*D;
  A[18][4]=(5.0887095157928488e-03)*D;
  A[18][5]=(0.0)*D;
  A[18][6]=(-1.4352770429159319e-03)*D;
  A[18][7]=(0.0)*D;
  A[18][8]=(1.0242658897172786e-02)*D;
  A[18][9]=(0.0)*D;
  A[18][10]=(8.9248136123136121e-04)*D;
  A[18][11]=(0.0)*D;
  A[18][12]=(7.8287838704505376e-04)*D;
  A[18][13]=(0.0)*D;
  A[18][14]=(1.1743175805675805e-02)*D;
  A[18][15]=(0.0)*D;
  A[18][16]=(-1.5657567740901074e-02)*D;
  A[18][17]=(0.0)*D;
  A[18][18]=(7.0459054834054832e-02)*D;
  A[18][19]=(0.0)*D;
  A[18][20]=(1.0242658897172786e-02)*D;
  A[18][21]=(0.0)*D;
  A[18][22]=(3.1315135481802150e-03)*D;
  A[18][23]=(0.0)*D;
  A[18][24]=(3.9143919352252685e-03)*D;
  A[18][25]=(0.0)*D;
  A[18][26]=(-1.5657567740901074e-02)*D;
  A[18][27]=(0.0)*D;
  A[18][28]=(-1.4352770429159319e-03)*D;
  A[18][29]=(0.0)*D;
  A[18][30]=(3.1315135481802150e-03)*D;
  A[18][31]=(0.0)*D;
  A[18][32]=(1.1743175805675805e-02)*D;
  A[18][33]=(0.0)*D;
  A[18][34]=(5.0887095157928488e-03)*D;
  A[18][35]=(0.0)*D;
  A[18][36]=(7.8287838704505376e-04)*D;
  A[18][37]=(0.0)*D;
  A[18][38]=(-2.8053142202447757e-03)*D;
  A[18][39]=(0.0)*D;
  A[18][40]=(8.7943338811394365e-04)*D;
  A[18][41]=(0.0)*D;
  A[19][0]=(0.0)*D;
  A[19][1]=(8.7943338811394365e-04)*D;
  A[19][2]=(0.0)*D;
  A[19][3]=(-2.8053142202447757e-03)*D;
  A[19][4]=(0.0)*D;
  A[19][5]=(5.0887095157928488e-03)*D;
  A[19][6]=(0.0)*D;
  A[19][7]=(-1.4352770429159319e-03)*D;
  A[19][8]=(0.0)*D;
  A[19][9]=(1.0242658897172786e-02)*D;
  A[19][10]=(0.0)*D;
  A[19][11]=(8.9248136123136121e-04)*D;
  A[19][12]=(0.0)*D;
  A[19][13]=(7.8287838704505376e-04)*D;
  A[19][14]=(0.0)*D;
  A[19][15]=(1.1743175805675805e-02)*D;
  A[19][16]=(0.0)*D;
  A[19][17]=(-1.5657567740901074e-02)*D;
  A[19][18]=(0.0)*D;
  A[19][19]=(7.0459054834054832e-02)*D;
  A[19][20]=(0.0)*D;
  A[19][21]=(1.0242658897172786e-02)*D;
  A[19][22]=(0.0)*D;
  A[19][23]=(3.1315135481802150e-03)*D;
  A[19][24]=(0.0)*D;
  A[19][25]=(3.9143919352252685e-03)*D;
  A[19][26]=(0.0)*D;
  A[19][27]=(-1.5657567740901074e-02)*D;
  A[19][28]=(0.0)*D;
  A[19][29]=(-1.4352770429159319e-03)*D;
  A[19][30]=(0.0)*D;
  A[19][31]=(3.1315135481802150e-03)*D;
  A[19][32]=(0.0)*D;
  A[19][33]=(1.1743175805675805e-02)*D;
  A[19][34]=(0.0)*D;
  A[19][35]=(5.0887095157928488e-03)*D;
  A[19][36]=(0.0)*D;
  A[19][37]=(7.8287838704505376e-04)*D;
  A[19][38]=(0.0)*D;
  A[19][39]=(-2.8053142202447757e-03)*D;
  A[19][40]=(0.0)*D;
  A[19][41]=(8.7943338811394365e-04)*D;
  A[20][0]=(3.6364701078242745e-04)*D;
  A[20][1]=(0.0)*D;
  A[20][2]=(-2.4823768855886910e-03)*D;
  A[20][3]=(0.0)*D;
  A[20][4]=(5.7606801313398537e-03)*D;
  A[20][5]=(0.0)*D;
  A[20][6]=(-7.3525328516647961e-03)*D;
  A[20][7]=(0.0)*D;
  A[20][8]=(7.3525328516647961e-03)*D;
  A[20][9]=(0.0)*D;
  A[20][10]=(1.1813634860509861e-03)*D;
  A[20][11]=(0.0)*D;
  A[20][12]=(-4.1101115319865323e-04)*D;
  A[20][13]=(0.0)*D;
  A[20][14]=(7.8287838704505376e-04)*D;
  A[20][15]=(0.0)*D;
  A[20][16]=(-3.8165321368446368e-03)*D;
  A[20][17]=(0.0)*D;
  A[20][18]=(1.0242658897172786e-02)*D;
  A[20][19]=(0.0)*D;
  A[20][20]=(1.4705065703329592e-02)*D;
  A[20][21]=(0.0)*D;
  A[20][22]=(-1.4483250160333494e-03)*D;
  A[20][23]=(0.0)*D;
  A[20][24]=(-1.3700371773288440e-03)*D;
  A[20][25]=(0.0)*D;
  A[20][26]=(9.7859798380631720e-05)*D;
  A[20][27]=(0.0)*D;
  A[20][28]=(-9.5837362547431994e-03)*D;
  A[20][29]=(0.0)*D;
  A[20][30]=(-4.5667905910961469e-05)*D;
  A[20][31]=(0.0)*D;
  A[20][32]=(-2.8053142202447757e-03)*D;
  A[20][33]=(0.0)*D;
  A[20][34]=(7.7048281258350702e-03)*D;
  A[20][35]=(0.0)*D;
  A[20][36]=(-2.4823768855886910e-03)*D;
  A[20][37]=(0.0)*D;
  A[20][38]=(-3.9698458209742934e-03)*D;
  A[20][39]=(0.0)*D;
  A[20][40]=(-2.2964432686654910e-05)*D;
  A[20][41]=(0.0)*D;
  A[21][0]=(0.0)*D;
  A[21][1]=(3.6364701078242745e-04)*D;
  A[21][2]=(0.0)*D;
  A[21][3]=(-2.4823768855886910e-03)*D;
  A[21][4]=(0.0)*D;
  A[21][5]=(5.7606801313398537e-03)*D;
  A[21][6]=(0.0)*D;
  A[21][7]=(-7.3525328516647961e-03)*D;
  A[21][8]=(0.0)*D;
  A[21][9]=(7.3525328516647961e-03)*D;
  A[21][10]=(0.0)*D;
  A[21][11]=(1.1813634860509861e-03)*D;
  A[21][12]=(0.0)*D;
  A[21][13]=(-4.1101115319865323e-04)*D;
  A[21][14]=(0.0)*D;
  A[21][15]=(7.8287838704505376e-04)*D;
  A[21][16]=(0.0)*D;
  A[21][17]=(-3.8165321368446368e-03)*D;
  A[21][18]=(0.0)*D;
  A[21][19]=(1.0242658897172786e-02)*D;
  A[21][20]=(0.0)*D;
  A[21][21]=(1.4705065703329592e-02)*D;
  A[21][22]=(0.0)*D;
  A[21][23]=(-1.4483250160333494e-03)*D;
  A[21][24]=(0.0)*D;
  A[21][25]=(-1.3700371773288440e-03)*D;
  A[21][26]=(0.0)*D;
  A[21][27]=(9.7859798380631720e-05)*D;
  A[21][28]=(0.0)*D;
  A[21][29]=(-9.5837362547431994e-03)*D;
  A[21][30]=(0.0)*D;
  A[21][31]=(-4.5667905910961469e-05)*D;
  A[21][32]=(0.0)*D;
  A[21][33]=(-2.8053142202447757e-03)*D;
  A[21][34]=(0.0)*D;
  A[21][35]=(7.7048281258350702e-03)*D;
  A[21][36]=(0.0)*D;
  A[21][37]=(-2.4823768855886910e-03)*D;
  A[21][38]=(0.0)*D;
  A[21][39]=(-3.9698458209742934e-03)*D;
  A[21][40]=(0.0)*D;
  A[21][41]=(-2.2964432686654910e-05)*D;
  A[22][0]=(-9.5824314574314575e-04)*D;
  A[22][1]=(0.0)*D;
  A[22][2]=(-7.3525328516647961e-03)*D;
  A[22][3]=(0.0)*D;
  A[22][4]=(2.9488419245363691e-03)*D;
  A[22][5]=(0.0)*D;
  A[22][6]=(-1.3569892042114265e-03)*D;
  A[22][7]=(0.0)*D;
  A[22][8]=(-4.5667905910961469e-05)*D;
  A[22][9]=(0.0)*D;
  A[22][10]=(2.0720181310459088e-04)*D;
  A[22][11]=(0.0)*D;
  A[22][12]=(-9.5837362547431994e-03)*D;
  A[22][13]=(0.0)*D;
  A[22][14]=(-1.4352770429159319e-03)*D;
  A[22][15]=(0.0)*D;
  A[22][16]=(-5.4801487093153762e-03)*D;
  A[22][17]=(0.0)*D;
  A[22][18]=(3.1315135481802150e-03)*D;
  A[22][19]=(0.0)*D;
  A[22][20]=(-1.4483250160333494e-03)*D;
  A[22][21]=(0.0)*D;
  A[22][22]=(2.2077170514670515e-02)*D;
  A[22][23]=(0.0)*D;
  A[22][24]=(5.2844291125541129e-03)*D;
  A[22][25]=(0.0)*D;
  A[22][26]=(-5.8715879028379029e-04)*D;
  A[22][27]=(0.0)*D;
  A[22][28]=(2.8183621933621935e-03)*D;
  A[22][29]=(0.0)*D;
  A[22][30]=(-1.3478556230292342e-02)*D;
  A[22][31]=(0.0)*D;
  A[22][32]=(5.0887095157928488e-03)*D;
  A[22][33]=(0.0)*D;
  A[22][34]=(-1.3569892042114265e-03)*D;
  A[22][35]=(0.0)*D;
  A[22][36]=(7.7048281258350702e-03)*D;
  A[22][37]=(0.0)*D;
  A[22][38]=(5.7606801313398537e-03)*D;
  A[22][39]=(0.0)*D;
  A[22][40]=(4.6268112674362677e-04)*D;
  A[22][41]=(0.0)*D;
  A[23][0]=(0.0)*D;
  A[23][1]=(-9.5824314574314575e-04)*D;
  A[23][2]=(0.0)*D;
  A[23][3]=(-7.3525328516647961e-03)*D;
  A[23][4]=(0.0)*D;
  A[23][5]=(2.9488419245363691e-03)*D;
  A[23][6]=(0.0)*D;
  A[23][7]=(-1.3569892042114265e-03)*D;
  A[23][8]=(0.0)*D;
  A[23][9]=(-4.5667905910961469e-05)*D;
  A[23][10]=(0.0)*D;
  A[23][11]=(2.0720181310459088e-04)*D;
  A[23][12]=(0.0)*D;
  A[23][13]=(-9.5837362547431994e-03)*D;
  A[23][14]=(0.0)*D;
  A[23][15]=(-1.4352770429159319e-03)*D;
  A[23][16]=(0.0)*D;
  A[23][17]=(-5.4801487093153762e-03)*D;
  A[23][18]=(0.0)*D;
  A[23][19]=(3.1315135481802150e-03)*D;
  A[23][20]=(0.0)*D;
  A[23][21]=(-1.4483250160333494e-03)*D;
  A[23][22]=(0.0)*D;
  A[23][23]=(2.2077170514670515e-02)*D;
  A[23][24]=(0.0)*D;
  A[23][25]=(5.2844291125541129e-03)*D;
  A[23][26]=(0.0)*D;
  A[23][27]=(-5.8715879028379029e-04)*D;
  A[23][28]=(0.0)*D;
  A[23][29]=(2.8183621933621935e-03)*D;
  A[23][30]=(0.0)*D;
  A[23][31]=(-1.3478556230292342e-02)*D;
  A[23][32]=(0.0)*D;
  A[23][33]=(5.0887095157928488e-03)*D;
  A[23][34]=(0.0)*D;
  A[23][35]=(-1.3569892042114265e-03)*D;
  A[23][36]=(0.0)*D;
  A[23][37]=(7.7048281258350702e-03)*D;
  A[23][38]=(0.0)*D;
  A[23][39]=(5.7606801313398537e-03)*D;
  A[23][40]=(0.0)*D;
  A[23][41]=(4.6268112674362677e-04)*D;
  A[24][0]=(-9.7468359187109191e-04)*D;
  A[24][1]=(0.0)*D;
  A[24][2]=(-3.8165321368446368e-03)*D;
  A[24][3]=(0.0)*D;
  A[24][4]=(-5.4801487093153762e-03)*D;
  A[24][5]=(0.0)*D;
  A[24][6]=(-5.8715879028379029e-04)*D;
  A[24][7]=(0.0)*D;
  A[24][8]=(-1.3700371773288440e-03)*D;
  A[24][9]=(0.0)*D;
  A[24][10]=(6.4978906124739453e-04)*D;
  A[24][11]=(0.0)*D;
  A[24][12]=(9.7859798380631720e-05)*D;
  A[24][13]=(0.0)*D;
  A[24][14]=(-1.5657567740901074e-02)*D;
  A[24][15]=(0.0)*D;
  A[24][16]=(-1.1743175805675805e-02)*D;
  A[24][17]=(0.0)*D;
  A[24][18]=(3.9143919352252685e-03)*D;
  A[24][19]=(0.0)*D;
  A[24][20]=(-1.3700371773288440e-03)*D;
  A[24][21]=(0.0)*D;
  A[24][22]=(5.2844291125541129e-03)*D;
  A[24][23]=(0.0)*D;
  A[24][24]=(7.6330642736892731e-02)*D;
  A[24][25]=(0.0)*D;
  A[24][26]=(-1.1743175805675805e-02)*D;
  A[24][27]=(0.0)*D;
  A[24][28]=(-5.8715879028379029e-04)*D;
  A[24][29]=(0.0)*D;
  A[24][30]=(5.2844291125541129e-03)*D;
  A[24][31]=(0.0)*D;
  A[24][32]=(-1.5657567740901074e-02)*D;
  A[24][33]=(0.0)*D;
  A[24][34]=(-5.4801487093153762e-03)*D;
  A[24][35]=(0.0)*D;
  A[24][36]=(9.7859798380631720e-05)*D;
  A[24][37]=(0.0)*D;
  A[24][38]=(-3.8165321368446368e-03)*D;
  A[24][39]=(0.0)*D;
  A[24][40]=(-9.7468359187109191e-04)*D;
  A[24][41]=(0.0)*D;
  A[25][0]=(0.0)*D;
  A[25][1]=(-9.7468359187109191e-04)*D;
  A[25][2]=(0.0)*D;
  A[25][3]=(-3.8165321368446368e-03)*D;
  A[25][4]=(0.0)*D;
  A[25][5]=(-5.4801487093153762e-03)*D;
  A[25][6]=(0.0)*D;
  A[25][7]=(-5.8715879028379029e-04)*D;
  A[25][8]=(0.0)*D;
  A[25][9]=(-1.3700371773288440e-03)*D;
  A[25][10]=(0.0)*D;
  A[25][11]=(6.4978906124739453e-04)*D;
  A[25][12]=(0.0)*D;
  A[25][13]=(9.7859798380631720e-05)*D;
  A[25][14]=(0.0)*D;
  A[25][15]=(-1.5657567740901074e-02)*D;
  A[25][16]=(0.0)*D;
  A[25][17]=(-1.1743175805675805e-02)*D;
  A[25][18]=(0.0)*D;
  A[25][19]=(3.9143919352252685e-03)*D;
  A[25][20]=(0.0)*D;
  A[25][21]=(-1.3700371773288440e-03)*D;
  A[25][22]=(0.0)*D;
  A[25][23]=(5.2844291125541129e-03)*D;
  A[25][24]=(0.0)*D;
  A[25][25]=(7.6330642736892731e-02)*D;
  A[25][26]=(0.0)*D;
  A[25][27]=(-1.1743175805675805e-02)*D;
  A[25][28]=(0.0)*D;
  A[25][29]=(-5.8715879028379029e-04)*D;
  A[25][30]=(0.0)*D;
  A[25][31]=(5.2844291125541129e-03)*D;
  A[25][32]=(0.0)*D;
  A[25][33]=(-1.5657567740901074e-02)*D;
  A[25][34]=(0.0)*D;
  A[25][35]=(-5.4801487093153762e-03)*D;
  A[25][36]=(0.0)*D;
  A[25][37]=(9.7859798380631720e-05)*D;
  A[25][38]=(0.0)*D;
  A[25][39]=(-3.8165321368446368e-03)*D;
  A[25][40]=(0.0)*D;
  A[25][41]=(-9.7468359187109191e-04)*D;
  A[26][0]=(6.4978906124739453e-04)*D;
  A[26][1]=(0.0)*D;
  A[26][2]=(-1.3700371773288440e-03)*D;
  A[26][3]=(0.0)*D;
  A[26][4]=(-5.8715879028379029e-04)*D;
  A[26][5]=(0.0)*D;
  A[26][6]=(-5.4801487093153762e-03)*D;
  A[26][7]=(0.0)*D;
  A[26][8]=(-3.8165321368446368e-03)*D;
  A[26][9]=(0.0)*D;
  A[26][10]=(-9.7468359187109191e-04)*D;
  A[26][11]=(0.0)*D;
  A[26][12]=(-1.3700371773288440e-03)*D;
  A[26][13]=(0.0)*D;
  A[26][14]=(3.9143919352252685e-03)*D;
  A[26][15]=(0.0)*D;
  A[26][16]=(-1.1743175805675805e-02)*D;
  A[26][17]=(0.0)*D;
  A[26][18]=(-1.5657567740901074e-02)*D;
  A[26][19]=(0.0)*D;
  A[26][20]=(9.7859798380631720e-05)*D;
  A[26][21]=(0.0)*D;
  A[26][22]=(-5.8715879028379029e-04)*D;
  A[26][23]=(0.0)*D;
  A[26][24]=(-1.1743175805675805e-02)*D;
  A[26][25]=(0.0)*D;
  A[26][26]=(7.6330642736892731e-02)*D;
  A[26][27]=(0.0)*D;
  A[26][28]=(5.2844291125541129e-03)*D;
  A[26][29]=(0.0)*D;
  A[26][30]=(-5.4801487093153762e-03)*D;
  A[26][31]=(0.0)*D;
  A[26][32]=(-1.5657567740901074e-02)*D;
  A[26][33]=(0.0)*D;
  A[26][34]=(5.2844291125541129e-03)*D;
  A[26][35]=(0.0)*D;
  A[26][36]=(-3.8165321368446368e-03)*D;
  A[26][37]=(0.0)*D;
  A[26][38]=(9.7859798380631720e-05)*D;
  A[26][39]=(0.0)*D;
  A[26][40]=(-9.7468359187109191e-04)*D;
  A[26][41]=(0.0)*D;
  A[27][0]=(0.0)*D;
  A[27][1]=(6.4978906124739453e-04)*D;
  A[27][2]=(0.0)*D;
  A[27][3]=(-1.3700371773288440e-03)*D;
  A[27][4]=(0.0)*D;
  A[27][5]=(-5.8715879028379029e-04)*D;
  A[27][6]=(0.0)*D;
  A[27][7]=(-5.4801487093153762e-03)*D;
  A[27][8]=(0.0)*D;
  A[27][9]=(-3.8165321368446368e-03)*D;
  A[27][10]=(0.0)*D;
  A[27][11]=(-9.7468359187109191e-04)*D;
  A[27][12]=(0.0)*D;
  A[27][13]=(-1.3700371773288440e-03)*D;
  A[27][14]=(0.0)*D;
  A[27][15]=(3.9143919352252685e-03)*D;
  A[27][16]=(0.0)*D;
  A[27][17]=(-1.1743175805675805e-02)*D;
  A[27][18]=(0.0)*D;
  A[27][19]=(-1.5657567740901074e-02)*D;
  A[27][20]=(0.0)*D;
  A[27][21]=(9.7859798380631720e-05)*D;
  A[27][22]=(0.0)*D;
  A[27][23]=(-5.8715879028379029e-04)*D;
  A[27][24]=(0.0)*D;
  A[27][25]=(-1.1743175805675805e-02)*D;
  A[27][26]=(0.0)*D;
  A[27][27]=(7.6330642736892731e-02)*D;
  A[27][28]=(0.0)*D;
  A[27][29]=(5.2844291125541129e-03)*D;
  A[27][30]=(0.0)*D;
  A[27][31]=(-5.4801487093153762e-03)*D;
  A[27][32]=(0.0)*D;
  A[27][33]=(-1.5657567740901074e-02)*D;
  A[27][34]=(0.0)*D;
  A[27][35]=(5.2844291125541129e-03)*D;
  A[27][36]=(0.0)*D;
  A[27][37]=(-3.8165321368446368e-03)*D;
  A[27][38]=(0.0)*D;
  A[27][39]=(9.7859798380631720e-05)*D;
  A[27][40]=(0.0)*D;
  A[27][41]=(-9.7468359187109191e-04)*D;
  A[28][0]=(2.0720181310459088e-04)*D;
  A[28][1]=(0.0)*D;
  A[28][2]=(-4.5667905910961469e-05)*D;
  A[28][3]=(0.0)*D;
  A[28][4]=(-1.3569892042114265e-03)*D;
  A[28][5]=(0.0)*D;
  A[28][6]=(2.9488419245363691e-03)*D;
  A[28][7]=(0.0)*D;
  A[28][8]=(-7.3525328516647961e-03)*D;
  A[28][9]=(0.0)*D;
  A[28][10]=(-9.5824314574314575e-04)*D;
  A[28][11]=(0.0)*D;
  A[28][12]=(-1.4483250160333494e-03)*D;
  A[28][13]=(0.0)*D;
  A[28][14]=(3.1315135481802150e-03)*D;
  A[28][15]=(0.0)*D;
  A[28][16]=(-5.4801487093153762e-03)*D;
  A[28][17]=(0.0)*D;
  A[28][18]=(-1.4352770429159319e-03)*D;
  A[28][19]=(0.0)*D;
  A[28][20]=(-9.5837362547431994e-03)*D;
  A[28][21]=(0.0)*D;
  A[28][22]=(2.8183621933621935e-03)*D;
  A[28][23]=(0.0)*D;
  A[28][24]=(-5.8715879028379029e-04)*D;
  A[28][25]=(0.0)*D;
  A[28][26]=(5.2844291125541129e-03)*D;
  A[28][27]=(0.0)*D;
  A[28][28]=(2.2077170514670515e-02)*D;
  A[28][29]=(0.0)*D;
  A[28][30]=(-1.3569892042114265e-03)*D;
  A[28][31]=(0.0)*D;
  A[28][32]=(5.0887095157928488e-03)*D;
  A[28][33]=(0.0)*D;
  A[28][34]=(-1.3478556230292342e-02)*D;
  A[28][35]=(0.0)*D;
  A[28][36]=(5.7606801313398537e-03)*D;
  A[28][37]=(0.0)*D;
  A[28][38]=(7.7048281258350702e-03)*D;
  A[28][39]=(0.0)*D;
  A[28][40]=(4.6268112674362677e-04)*D;
  A[28][41]=(0.0)*D;
  A[29][0]=(0.0)*D;
  A[29][1]=(2.0720181310459088e-04)*D;
  A[29][2]=(0.0)*D;
  A[29][3]=(-4.5667905910961469e-05)*D;
  A[29][4]=(0.0)*D;
  A[29][5]=(-1.3569892042114265e-03)*D;
  A[29][6]=(0.0)*D;
  A[29][7]=(2.9488419245363691e-03)*D;
  A[29][8]=(0.0)*D;
  A[29][9]=(-7.3525328516647961e-03)*D;
  A[29][10]=(0.0)*D;
  A[29][11]=(-9.5824314574314575e-04)*D;
  A[29][12]=(0.0)*D;
  A[29][13]=(-1.4483250160333494e-03)*D;
  A[29][14]=(0.0)*D;
  A[29][15]=(3.1315135481802150e-03)*D;
  A[29][16]=(0.0)*D;
  A[29][17]=(-5.4801487093153762e-03)*D;
  A[29][18]=(0.0)*D;
  A[29][19]=(-1.4352770429159319e-03)*D;
  A[29][20]=(0.0)*D;
  A[29][21]=(-9.5837362547431994e-03)*D;
  A[29][22]=(0.0)*D;
  A[29][23]=(2.8183621933621935e-03)*D;
  A[29][24]=(0.0)*D;
  A[29][25]=(-5.8715879028379029e-04)*D;
  A[29][26]=(0.0)*D;
  A[29][27]=(5.2844291125541129e-03)*D;
  A[29][28]=(0.0)*D;
  A[29][29]=(2.2077170514670515e-02)*D;
  A[29][30]=(0.0)*D;
  A[29][31]=(-1.3569892042114265e-03)*D;
  A[29][32]=(0.0)*D;
  A[29][33]=(5.0887095157928488e-03)*D;
  A[29][34]=(0.0)*D;
  A[29][35]=(-1.3478556230292342e-02)*D;
  A[29][36]=(0.0)*D;
  A[29][37]=(5.7606801313398537e-03)*D;
  A[29][38]=(0.0)*D;
  A[29][39]=(7.7048281258350702e-03)*D;
  A[29][40]=(0.0)*D;
  A[29][41]=(4.6268112674362677e-04)*D;
  A[30][0]=(4.6268112674362677e-04)*D;
  A[30][1]=(0.0)*D;
  A[30][2]=(5.7606801313398537e-03)*D;
  A[30][3]=(0.0)*D;
  A[30][4]=(-1.3569892042114265e-03)*D;
  A[30][5]=(0.0)*D;
  A[30][6]=(2.8183621933621935e-03)*D;
  A[30][7]=(0.0)*D;
  A[30][8]=(-1.4483250160333494e-03)*D;
  A[30][9]=(0.0)*D;
  A[30][10]=(2.0720181310459088e-04)*D;
  A[30][11]=(0.0)*D;
  A[30][12]=(7.7048281258350702e-03)*D;
  A[30][13]=(0.0)*D;
  A[30][14]=(5.0887095157928488e-03)*D;
  A[30][15]=(0.0)*D;
  A[30][16]=(-5.8715879028379029e-04)*D;
  A[30][17]=(0.0)*D;
  A[30][18]=(3.1315135481802150e-03)*D;
  A[30][19]=(0.0)*D;
  A[30][20]=(-4.5667905910961469e-05)*D;
  A[30][21]=(0.0)*D;
  A[30][22]=(-1.3478556230292342e-02)*D;
  A[30][23]=(0.0)*D;
  A[30][24]=(5.2844291125541129e-03)*D;
  A[30][25]=(0.0)*D;
  A[30][26]=(-5.4801487093153762e-03)*D;
  A[30][27]=(0.0)*D;
  A[30][28]=(-1.3569892042114265e-03)*D;
  A[30][29]=(0.0)*D;
  A[30][30]=(2.2077170514670515e-02)*D;
  A[30][31]=(0.0)*D;
  A[30][32]=(-1.4352770429159319e-03)*D;
  A[30][33]=(0.0)*D;
  A[30][34]=(2.9488419245363691e-03)*D;
  A[30][35]=(0.0)*D;
  A[30][36]=(-9.5837362547431994e-03)*D;
  A[30][37]=(0.0)*D;
  A[30][38]=(-7.3525328516647961e-03)*D;
  A[30][39]=(0.0)*D;
  A[30][40]=(-9.5824314574314575e-04)*D;
  A[30][41]=(0.0)*D;
  A[31][0]=(0.0)*D;
  A[31][1]=(4.6268112674362677e-04)*D;
  A[31][2]=(0.0)*D;
  A[31][3]=(5.7606801313398537e-03)*D;
  A[31][4]=(0.0)*D;
  A[31][5]=(-1.3569892042114265e-03)*D;
  A[31][6]=(0.0)*D;
  A[31][7]=(2.8183621933621935e-03)*D;
  A[31][8]=(0.0)*D;
  A[31][9]=(-1.4483250160333494e-03)*D;
  A[31][10]=(0.0)*D;
  A[31][11]=(2.0720181310459088e-04)*D;
  A[31][12]=(0.0)*D;
  A[31][13]=(7.7048281258350702e-03)*D;
  A[31][14]=(0.0)*D;
  A[31][15]=(5.0887095157928488e-03)*D;
  A[31][16]=(0.0)*D;
  A[31][17]=(-5.8715879028379029e-04)*D;
  A[31][18]=(0.0)*D;
  A[31][19]=(3.1315135481802150e-03)*D;
  A[31][20]=(0.0)*D;
  A[31][21]=(-4.5667905910961469e-05)*D;
  A[31][22]=(0.0)*D;
  A[31][23]=(-1.3478556230292342e-02)*D;
  A[31][24]=(0.0)*D;
  A[31][25]=(5.2844291125541129e-03)*D;
  A[31][26]=(0.0)*D;
  A[31][27]=(-5.4801487093153762e-03)*D;
  A[31][28]=(0.0)*D;
  A[31][29]=(-1.3569892042114265e-03)*D;
  A[31][30]=(0.0)*D;
  A[31][31]=(2.2077170514670515e-02)*D;
  A[31][32]=(0.0)*D;
  A[31][33]=(-1.4352770429159319e-03)*D;
  A[31][34]=(0.0)*D;
  A[31][35]=(2.9488419245363691e-03)*D;
  A[31][36]=(0.0)*D;
  A[31][37]=(-9.5837362547431994e-03)*D;
  A[31][38]=(0.0)*D;
  A[31][39]=(-7.3525328516647961e-03)*D;
  A[31][40]=(0.0)*D;
  A[31][41]=(-9.5824314574314575e-04)*D;
  A[32][0]=(8.7943338811394365e-04)*D;
  A[32][1]=(0.0)*D;
  A[32][2]=(7.8287838704505376e-04)*D;
  A[32][3]=(0.0)*D;
  A[32][4]=(3.1315135481802150e-03)*D;
  A[32][5]=(0.0)*D;
  A[32][6]=(3.1315135481802150e-03)*D;
  A[32][7]=(0.0)*D;
  A[32][8]=(7.8287838704505376e-04)*D;
  A[32][9]=(0.0)*D;
  A[32][10]=(8.7943338811394365e-04)*D;
  A[32][11]=(0.0)*D;
  A[32][12]=(-2.8053142202447757e-03)*D;
  A[32][13]=(0.0)*D;
  A[32][14]=(1.1743175805675805e-02)*D;
  A[32][15]=(0.0)*D;
  A[32][16]=(3.9143919352252685e-03)*D;
  A[32][17]=(0.0)*D;
  A[32][18]=(1.1743175805675805e-02)*D;
  A[32][19]=(0.0)*D;
  A[32][20]=(-2.8053142202447757e-03)*D;
  A[32][21]=(0.0)*D;
  A[32][22]=(5.0887095157928488e-03)*D;
  A[32][23]=(0.0)*D;
  A[32][24]=(-1.5657567740901074e-02)*D;
  A[32][25]=(0.0)*D;
  A[32][26]=(-1.5657567740901074e-02)*D;
  A[32][27]=(0.0)*D;
  A[32][28]=(5.0887095157928488e-03)*D;
  A[32][29]=(0.0)*D;
  A[32][30]=(-1.4352770429159319e-03)*D;
  A[32][31]=(0.0)*D;
  A[32][32]=(7.0459054834054832e-02)*D;
  A[32][33]=(0.0)*D;
  A[32][34]=(-1.4352770429159319e-03)*D;
  A[32][35]=(0.0)*D;
  A[32][36]=(1.0242658897172786e-02)*D;
  A[32][37]=(0.0)*D;
  A[32][38]=(1.0242658897172786e-02)*D;
  A[32][39]=(0.0)*D;
  A[32][40]=(8.9248136123136121e-04)*D;
  A[32][41]=(0.0)*D;
  A[33][0]=(0.0)*D;
  A[33][1]=(8.7943338811394365e-04)*D;
  A[33][2]=(0.0)*D;
  A[33][3]=(7.8287838704505376e-04)*D;
  A[33][4]=(0.0)*D;
  A[33][5]=(3.1315135481802150e-03)*D;
  A[33][6]=(0.0)*D;
  A[33][7]=(3.1315135481802150e-03)*D;
  A[33][8]=(0.0)*D;
  A[33][9]=(7.8287838704505376e-04)*D;
  A[33][10]=(0.0)*D;
  A[33][11]=(8.7943338811394365e-04)*D;
  A[33][12]=(0.0)*D;
  A[33][13]=(-2.8053142202447757e-03)*D;
  A[33][14]=(0.0)*D;
  A[33][15]=(1.1743175805675805e-02)*D;
  A[33][16]=(0.0)*D;
  A[33][17]=(3.9143919352252685e-03)*D;
  A[33][18]=(0.0)*D;
  A[33][19]=(1.1743175805675805e-02)*D;
  A[33][20]=(0.0)*D;
  A[33][21]=(-2.8053142202447757e-03)*D;
  A[33][22]=(0.0)*D;
  A[33][23]=(5.0887095157928488e-03)*D;
  A[33][24]=(0.0)*D;
  A[33][25]=(-1.5657567740901074e-02)*D;
  A[33][26]=(0.0)*D;
  A[33][27]=(-1.5657567740901074e-02)*D;
  A[33][28]=(0.0)*D;
  A[33][29]=(5.0887095157928488e-03)*D;
  A[33][30]=(0.0)*D;
  A[33][31]=(-1.4352770429159319e-03)*D;
  A[33][32]=(0.0)*D;
  A[33][33]=(7.0459054834054832e-02)*D;
  A[33][34]=(0.0)*D;
  A[33][35]=(-1.4352770429159319e-03)*D;
  A[33][36]=(0.0)*D;
  A[33][37]=(1.0242658897172786e-02)*D;
  A[33][38]=(0.0)*D;
  A[33][39]=(1.0242658897172786e-02)*D;
  A[33][40]=(0.0)*D;
  A[33][41]=(8.9248136123136121e-04)*D;
  A[34][0]=(2.0720181310459088e-04)*D;
  A[34][1]=(0.0)*D;
  A[34][2]=(-1.4483250160333494e-03)*D;
  A[34][3]=(0.0)*D;
  A[34][4]=(2.8183621933621935e-03)*D;
  A[34][5]=(0.0)*D;
  A[34][6]=(-1.3569892042114265e-03)*D;
  A[34][7]=(0.0)*D;
  A[34][8]=(5.7606801313398537e-03)*D;
  A[34][9]=(0.0)*D;
  A[34][10]=(4.6268112674362677e-04)*D;
  A[34][11]=(0.0)*D;
  A[34][12]=(-4.5667905910961469e-05)*D;
  A[34][13]=(0.0)*D;
  A[34][14]=(3.1315135481802150e-03)*D;
  A[34][15]=(0.0)*D;
  A[34][16]=(-5.8715879028379029e-04)*D;
  A[34][17]=(0.0)*D;
  A[34][18]=(5.0887095157928488e-03)*D;
  A[34][19]=(0.0)*D;
  A[34][20]=(7.7048281258350702e-03)*D;
  A[34][21]=(0.0)*D;
  A[34][22]=(-1.3569892042114265e-03)*D;
  A[34][23]=(0.0)*D;
  A[34][24]=(-5.4801487093153762e-03)*D;
  A[34][25]=(0.0)*D;
  A[34][26]=(5.2844291125541129e-03)*D;
  A[34][27]=(0.0)*D;
  A[34][28]=(-1.3478556230292342e-02)*D;
  A[34][29]=(0.0)*D;
  A[34][30]=(2.9488419245363691e-03)*D;
  A[34][31]=(0.0)*D;
  A[34][32]=(-1.4352770429159319e-03)*D;
  A[34][33]=(0.0)*D;
  A[34][34]=(2.2077170514670515e-02)*D;
  A[34][35]=(0.0)*D;
  A[34][36]=(-7.3525328516647961e-03)*D;
  A[34][37]=(0.0)*D;
  A[34][38]=(-9.5837362547431994e-03)*D;
  A[34][39]=(0.0)*D;
  A[34][40]=(-9.5824314574314575e-04)*D;
  A[34][41]=(0.0)*D;
  A[35][0]=(0.0)*D;
  A[35][1]=(2.0720181310459088e-04)*D;
  A[35][2]=(0.0)*D;
  A[35][3]=(-1.4483250160333494e-03)*D;
  A[35][4]=(0.0)*D;
  A[35][5]=(2.8183621933621935e-03)*D;
  A[35][6]=(0.0)*D;
  A[35][7]=(-1.3569892042114265e-03)*D;
  A[35][8]=(0.0)*D;
  A[35][9]=(5.7606801313398537e-03)*D;
  A[35][10]=(0.0)*D;
  A[35][11]=(4.6268112674362677e-04)*D;
  A[35][12]=(0.0)*D;
  A[35][13]=(-4.5667905910961469e-05)*D;
  A[35][14]=(0.0)*D;
  A[35][15]=(3.1315135481802150e-03)*D;
  A[35][16]=(0.0)*D;
  A[35][17]=(-5.8715879028379029e-04)*D;
  A[35][18]=(0.0)*D;
  A[35][19]=(5.0887095157928488e-03)*D;
  A[35][20]=(0.0)*D;
  A[35][21]=(7.7048281258350702e-03)*D;
  A[35][22]=(0.0)*D;
  A[35][23]=(-1.3569892042114265e-03)*D;
  A[35][24]=(0.0)*D;
  A[35][25]=(-5.4801487093153762e-03)*D;
  A[35][26]=(0.0)*D;
  A[35][27]=(5.2844291125541129e-03)*D;
  A[35][28]=(0.0)*D;
  A[35][29]=(-1.3478556230292342e-02)*D;
  A[35][30]=(0.0)*D;
  A[35][31]=(2.9488419245363691e-03)*D;
  A[35][32]=(0.0)*D;
  A[35][33]=(-1.4352770429159319e-03)*D;
  A[35][34]=(0.0)*D;
  A[35][35]=(2.2077170514670515e-02)*D;
  A[35][36]=(0.0)*D;
  A[35][37]=(-7.3525328516647961e-03)*D;
  A[35][38]=(0.0)*D;
  A[35][39]=(-9.5837362547431994e-03)*D;
  A[35][40]=(0.0)*D;
  A[35][41]=(-9.5824314574314575e-04)*D;
  A[36][0]=(-2.2964432686654910e-05)*D;
  A[36][1]=(0.0)*D;
  A[36][2]=(-2.4823768855886910e-03)*D;
  A[36][3]=(0.0)*D;
  A[36][4]=(-4.5667905910961469e-05)*D;
  A[36][5]=(0.0)*D;
  A[36][6]=(-1.4483250160333494e-03)*D;
  A[36][7]=(0.0)*D;
  A[36][8]=(-4.1101115319865323e-04)*D;
  A[36][9]=(0.0)*D;
  A[36][10]=(3.6364701078242745e-04)*D;
  A[36][11]=(0.0)*D;
  A[36][12]=(-3.9698458209742934e-03)*D;
  A[36][13]=(0.0)*D;
  A[36][14]=(-2.8053142202447757e-03)*D;
  A[36][15]=(0.0)*D;
  A[36][16]=(-1.3700371773288440e-03)*D;
  A[36][17]=(0.0)*D;
  A[36][18]=(7.8287838704505376e-04)*D;
  A[36][19]=(0.0)*D;
  A[36][20]=(-2.4823768855886910e-03)*D;
  A[36][21]=(0.0)*D;
  A[36][22]=(7.7048281258350702e-03)*D;
  A[36][23]=(0.0)*D;
  A[36][24]=(9.7859798380631720e-05)*D;
  A[36][25]=(0.0)*D;
  A[36][26]=(-3.8165321368446368e-03)*D;
  A[36][27]=(0.0)*D;
  A[36][28]=(5.7606801313398537e-03)*D;
  A[36][29]=(0.0)*D;
  A[36][30]=(-9.5837362547431994e-03)*D;
  A[36][31]=(0.0)*D;
  A[36][32]=(1.0242658897172786e-02)*D;
  A[36][33]=(0.0)*D;
  A[36][34]=(-7.3525328516647961e-03)*D;
  A[36][35]=(0.0)*D;
  A[36][36]=(1.4705065703329592e-02)*D;
  A[36][37]=(0.0)*D;
  A[36][38]=(7.3525328516647961e-03)*D;
  A[36][39]=(0.0)*D;
  A[36][40]=(1.1813634860509861e-03)*D;
  A[36][41]=(0.0)*D;
  A[37][0]=(0.0)*D;
  A[37][1]=(-2.2964432686654910e-05)*D;
  A[37][2]=(0.0)*D;
  A[37][3]=(-2.4823768855886910e-03)*D;
  A[37][4]=(0.0)*D;
  A[37][5]=(-4.5667905910961469e-05)*D;
  A[37][6]=(0.0)*D;
  A[37][7]=(-1.4483250160333494e-03)*D;
  A[37][8]=(0.0)*D;
  A[37][9]=(-4.1101115319865323e-04)*D;
  A[37][10]=(0.0)*D;
  A[37][11]=(3.6364701078242745e-04)*D;
  A[37][12]=(0.0)*D;
  A[37][13]=(-3.9698458209742934e-03)*D;
  A[37][14]=(0.0)*D;
  A[37][15]=(-2.8053142202447757e-03)*D;
  A[37][16]=(0.0)*D;
  A[37][17]=(-1.3700371773288440e-03)*D;
  A[37][18]=(0.0)*D;
  A[37][19]=(7.8287838704505376e-04)*D;
  A[37][20]=(0.0)*D;
  A[37][21]=(-2.4823768855886910e-03)*D;
  A[37][22]=(0.0)*D;
  A[37][23]=(7.7048281258350702e-03)*D;
  A[37][24]=(0.0)*D;
  A[37][25]=(9.7859798380631720e-05)*D;
  A[37][26]=(0.0)*D;
  A[37][27]=(-3.8165321368446368e-03)*D;
  A[37][28]=(0.0)*D;
  A[37][29]=(5.7606801313398537e-03)*D;
  A[37][30]=(0.0)*D;
  A[37][31]=(-9.5837362547431994e-03)*D;
  A[37][32]=(0.0)*D;
  A[37][33]=(1.0242658897172786e-02)*D;
  A[37][34]=(0.0)*D;
  A[37][35]=(-7.3525328516647961e-03)*D;
  A[37][36]=(0.0)*D;
  A[37][37]=(1.4705065703329592e-02)*D;
  A[37][38]=(0.0)*D;
  A[37][39]=(7.3525328516647961e-03)*D;
  A[37][40]=(0.0)*D;
  A[37][41]=(1.1813634860509861e-03)*D;
  A[38][0]=(3.6364701078242745e-04)*D;
  A[38][1]=(0.0)*D;
  A[38][2]=(-4.1101115319865323e-04)*D;
  A[38][3]=(0.0)*D;
  A[38][4]=(-1.4483250160333494e-03)*D;
  A[38][5]=(0.0)*D;
  A[38][6]=(-4.5667905910961469e-05)*D;
  A[38][7]=(0.0)*D;
  A[38][8]=(-2.4823768855886910e-03)*D;
  A[38][9]=(0.0)*D;
  A[38][10]=(-2.2964432686654910e-05)*D;
  A[38][11]=(0.0)*D;
  A[38][12]=(-2.4823768855886910e-03)*D;
  A[38][13]=(0.0)*D;
  A[38][14]=(7.8287838704505376e-04)*D;
  A[38][15]=(0.0)*D;
  A[38][16]=(-1.3700371773288440e-03)*D;
  A[38][17]=(0.0)*D;
  A[38][18]=(-2.8053142202447757e-03)*D;
  A[38][19]=(0.0)*D;
  A[38][20]=(-3.9698458209742934e-03)*D;
  A[38][21]=(0.0)*D;
  A[38][22]=(5.7606801313398537e-03)*D;
  A[38][23]=(0.0)*D;
  A[38][24]=(-3.8165321368446368e-03)*D;
  A[38][25]=(0.0)*D;
  A[38][26]=(9.7859798380631720e-05)*D;
  A[38][27]=(0.0)*D;
  A[38][28]=(7.7048281258350702e-03)*D;
  A[38][29]=(0.0)*D;
  A[38][30]=(-7.3525328516647961e-03)*D;
  A[38][31]=(0.0)*D;
  A[38][32]=(1.0242658897172786e-02)*D;
  A[38][33]=(0.0)*D;
  A[38][34]=(-9.5837362547431994e-03)*D;
  A[38][35]=(0.0)*D;
  A[38][36]=(7.3525328516647961e-03)*D;
  A[38][37]=(0.0)*D;
  A[38][38]=(1.4705065703329592e-02)*D;
  A[38][39]=(0.0)*D;
  A[38][40]=(1.1813634860509861e-03)*D;
  A[38][41]=(0.0)*D;
  A[39][0]=(0.0)*D;
  A[39][1]=(3.6364701078242745e-04)*D;
  A[39][2]=(0.0)*D;
  A[39][3]=(-4.1101115319865323e-04)*D;
  A[39][4]=(0.0)*D;
  A[39][5]=(-1.4483250160333494e-03)*D;
  A[39][6]=(0.0)*D;
  A[39][7]=(-4.5667905910961469e-05)*D;
  A[39][8]=(0.0)*D;
  A[39][9]=(-2.4823768855886910e-03)*D;
  A[39][10]=(0.0)*D;
  A[39][11]=(-2.2964432686654910e-05)*D;
  A[39][12]=(0.0)*D;
  A[39][13]=(-2.4823768855886910e-03)*D;
  A[39][14]=(0.0)*D;
  A[39][15]=(7.8287838704505376e-04)*D;
  A[39][16]=(0.0)*D;
  A[39][17]=(-1.3700371773288440e-03)*D;
  A[39][18]=(0.0)*D;
  A[39][19]=(-2.8053142202447757e-03)*D;
  A[39][20]=(0.0)*D;
  A[39][21]=(-3.9698458209742934e-03)*D;
  A[39][22]=(0.0)*D;
  A[39][23]=(5.7606801313398537e-03)*D;
  A[39][24]=(0.0)*D;
  A[39][25]=(-3.8165321368446368e-03)*D;
  A[39][26]=(0.0)*D;
  A[39][27]=(9.7859798380631720e-05)*D;
  A[39][28]=(0.0)*D;
  A[39][29]=(7.7048281258350702e-03)*D;
  A[39][30]=(0.0)*D;
  A[39][31]=(-7.3525328516647961e-03)*D;
  A[39][32]=(0.0)*D;
  A[39][33]=(1.0242658897172786e-02)*D;
  A[39][34]=(0.0)*D;
  A[39][35]=(-9.5837362547431994e-03)*D;
  A[39][36]=(0.0)*D;
  A[39][37]=(7.3525328516647961e-03)*D;
  A[39][38]=(0.0)*D;
  A[39][39]=(1.4705065703329592e-02)*D;
  A[39][40]=(0.0)*D;
  A[39][41]=(1.1813634860509861e-03)*D;
  A[40][0]=(1.2387745677676232e-04)*D;
  A[40][1]=(0.0)*D;
  A[40][2]=(3.6364701078242745e-04)*D;
  A[40][3]=(0.0)*D;
  A[40][4]=(2.0720181310459088e-04)*D;
  A[40][5]=(0.0)*D;
  A[40][6]=(2.0720181310459088e-04)*D;
  A[40][7]=(0.0)*D;
  A[40][8]=(3.6364701078242745e-04)*D;
  A[40][9]=(0.0)*D;
  A[40][10]=(1.2387745677676232e-04)*D;
  A[40][11]=(0.0)*D;
  A[40][12]=(-2.2964432686654910e-05)*D;
  A[40][13]=(0.0)*D;
  A[40][14]=(8.7943338811394365e-04)*D;
  A[40][15]=(0.0)*D;
  A[40][16]=(6.4978906124739453e-04)*D;
  A[40][17]=(0.0)*D;
  A[40][18]=(8.7943338811394365e-04)*D;
  A[40][19]=(0.0)*D;
  A[40][20]=(-2.2964432686654910e-05)*D;
  A[40][21]=(0.0)*D;
  A[40][22]=(4.6268112674362677e-04)*D;
  A[40][23]=(0.0)*D;
  A[40][24]=(-9.7468359187109191e-04)*D;
  A[40][25]=(0.0)*D;
  A[40][26]=(-9.7468359187109191e-04)*D;
  A[40][27]=(0.0)*D;
  A[40][28]=(4.6268112674362677e-04)*D;
  A[40][29]=(0.0)*D;
  A[40][30]=(-9.5824314574314575e-04)*D;
  A[40][31]=(0.0)*D;
  A[40][32]=(8.9248136123136121e-04)*D;
  A[40][33]=(0.0)*D;
  A[40][34]=(-9.5824314574314575e-04)*D;
  A[40][35]=(0.0)*D;
  A[40][36]=(1.1813634860509861e-03)*D;
  A[40][37]=(0.0)*D;
  A[40][38]=(1.1813634860509861e-03)*D;
  A[40][39]=(0.0)*D;
  A[40][40]=(1.3894525613275613e-03)*D;
  A[40][41]=(0.0)*D;
  A[41][0]=(0.0)*D;
  A[41][1]=(1.2387745677676232e-04)*D;
  A[41][2]=(0.0)*D;
  A[41][3]=(3.6364701078242745e-04)*D;
  A[41][4]=(0.0)*D;
  A[41][5]=(2.0720181310459088e-04)*D;
  A[41][6]=(0.0)*D;
  A[41][7]=(2.0720181310459088e-04)*D;
  A[41][8]=(0.0)*D;
  A[41][9]=(3.6364701078242745e-04)*D;
  A[41][10]=(0.0)*D;
  A[41][11]=(1.2387745677676232e-04)*D;
  A[41][12]=(0.0)*D;
  A[41][13]=(-2.2964432686654910e-05)*D;
  A[41][14]=(0.0)*D;
  A[41][15]=(8.7943338811394365e-04)*D;
  A[41][16]=(0.0)*D;
  A[41][17]=(6.4978906124739453e-04)*D;
  A[41][18]=(0.0)*D;
  A[41][19]=(8.7943338811394365e-04)*D;
  A[41][20]=(0.0)*D;
  A[41][21]=(-2.2964432686654910e-05)*D;
  A[41][22]=(0.0)*D;
  A[41][23]=(4.6268112674362677e-04)*D;
  A[41][24]=(0.0)*D;
  A[41][25]=(-9.7468359187109191e-04)*D;
  A[41][26]=(0.0)*D;
  A[41][27]=(-9.7468359187109191e-04)*D;
  A[41][28]=(0.0)*D;
  A[41][29]=(4.6268112674362677e-04)*D;
  A[41][30]=(0.0)*D;
  A[41][31]=(-9.5824314574314575e-04)*D;
  A[41][32]=(0.0)*D;
  A[41][33]=(8.9248136123136121e-04)*D;
  A[41][34]=(0.0)*D;
  A[41][35]=(-9.5824314574314575e-04)*D;
  A[41][36]=(0.0)*D;
  A[41][37]=(1.1813634860509861e-03)*D;
  A[41][38]=(0.0)*D;
  A[41][39]=(1.1813634860509861e-03)*D;
  A[41][40]=(0.0)*D;
  A[41][41]=(1.3894525613275613e-03)*D;
  int nbf = 42;  

  pair<int,int> index; 
  for (int i=0; i< nbf; i++) {
    index.first = iidof[i]; 
    for (int j=0; j< nbf; j++) {
      index.second = iidof[j]; 
      matrix[index] += A[i][j];   
    }
  }

}
